/******************************************************************************

 @File         PVRShellOS.cpp

 @Title        

 @Copyright    Copyright (C) 2006 - 2006 by Imagination Technologies Limited.

 @Platform     Support for windowed Symbian UIQ

 @Description  Makes programming for 3D APIs easier by wrapping surface
               initialization, Texture allocation and other functions for use by a demo.

******************************************************************************/
#include "PVRShellOS.h"
#include <PVRShellOS.rsg>

#include "PVRShellAPI.h"
#include "PVRShellImpl.h"
#include "PVRShell.h"

/****************************************************************************
 ** DEFINES                                                               **
 ****************************************************************************/

const TUid KUIDPVRShell={UID_OGL};
const TUid KUIDPVRShellView={0x00442233};


/****************************************************************************
 ** GLOBAL CODE                                                            **
 ****************************************************************************/

/*!***********************************************************************
 @Function		NewApplication
 @description	
*************************************************************************/
LOCAL_C CApaApplication* NewApplication(){
	return new CShellApplication;
}

/*!***********************************************************************
 @Function		E32Main
 @description	Program entry point
*************************************************************************/ 
GLDEF_C TInt E32Main(void){
  return EikStart::RunApplication(NewApplication);
}




/*!***************************************************************************
	Class: CPVRShellView
*****************************************************************************/

/*!***********************************************************************
 @Function		CPVRShellView
 @Input			aAppUi
 @Description	Constructor
*************************************************************************/
CPVRShellView::CPVRShellView(CQikAppUi* aAppUi):CQikViewBase(*aAppUi,KNullViewId)
{
}

/*!***********************************************************************
 @Function		NewL
*************************************************************************/
CPVRShellView* CPVRShellView::NewL(CQikAppUi* aAppUi,const TRect& aRect)
{
	CPVRShellView* cmv = new(ELeave)CPVRShellView(aAppUi);
	CleanupStack::PushL(cmv);
	cmv->ConstructL(aRect);
	CleanupStack::Pop();
	return cmv;
}

/*!***********************************************************************
 @Function		ViewId
 @Description	Returns the app's UID numbers
*************************************************************************/
TVwsViewId CPVRShellView::ViewId(void) const
{
	return TVwsViewId(KUIDPVRShell,KUIDPVRShellView);
}

/*!***********************************************************************
 @Function		~CPVRShellView
*************************************************************************/
CPVRShellView::~CPVRShellView()
{
  delete iPeriodic;

  init->OsDoReleaseAPI();
  delete init;
  delete ipDemo;

}

/*!***********************************************************************
 @Function		OfferKeyEventL
 @Description	Implements keyboard response
*************************************************************************/
TKeyResponse CPVRShellView::OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
{
	if(aType == EEventKeyDown)
	{
		RDebug::Printf("Key code: %d",aKeyEvent.iScanCode);
  		switch (aKeyEvent.iScanCode)
		{
  			case 'R':
				break;
			case 'J':case 'j':		init->KeyPressed(PVRShellKeyNameSELECT);	return EKeyWasConsumed;
			case 'K':case 'k':		init->KeyPressed(PVRShellKeyNameACTION1);	return EKeyWasConsumed;
			case 'L':case 'l':		init->KeyPressed(PVRShellKeyNameACTION2);	return EKeyWasConsumed;
			case 'T':case 't':		init->KeyPressed(PVRShellKeyNameUP);		return EKeyWasConsumed;
			case 'V':case 'v':		init->KeyPressed(PVRShellKeyNameDOWN);		return EKeyWasConsumed;
			case 'F':case 'f':		init->KeyPressed(PVRShellKeyNameLEFT);		return EKeyWasConsumed;
			case 'H':case 'h':		init->KeyPressed(PVRShellKeyNameRIGHT);		return EKeyWasConsumed;

			case 'Q': case 172: case 167: case 212:	case 213:
				init->KeyPressed(PVRShellKeyNameQUIT);		return EKeyWasConsumed;
		}
	}
	return EKeyWasNotConsumed;
}

/*!***********************************************************************
 @Function		HandleCommandL
 @Description	UIQ command interface
*************************************************************************/
void CPVRShellView::HandleCommandL(CQikCommand& aCommand)
{
	TKeyEvent aKeyEvent;

    switch (aCommand.Id())
    {
        case EQikCmdGoBack:
            RequestFocusL(NULL);

         case EEikCmdExit:
			aKeyEvent.iScanCode = 'Q';
			OfferKeyEventL(aKeyEvent,EEventKeyDown);
			      break;
        default:
            CQikViewBase::HandleCommandL(aCommand);
            break;
    }
}

/*!***********************************************************************
 @Function		ConstructL
 @Description	Second phase constructor
*************************************************************************/
void CPVRShellView::ConstructL(const TRect& /* aRect */)
{
	BaseConstructL();
}

/*!***********************************************************************
 @Function		ViewConstructL
 @Description	
*************************************************************************/
void CPVRShellView::ViewConstructL()
{
	InitComponentArrayL();
	ViewConstructFromResourceL(DSATEST_VIEW_CONFIG,DSATEST_VIEW_CONTROLS);
	
	ActivateL();
	TQikViewMode vm;
	vm.SetFullscreen();

	SetViewModeL(vm);
}

/*!***********************************************************************
 @Function		ViewActivatedL
 @Description	
*************************************************************************/
void CPVRShellView::ViewActivatedL(const TVwsViewId &aPrevViewId, TUid /* aCustomMessageId */, const TDesC8 & /* aCustomMessage */)
{
	SetParentView(aPrevViewId);
	
	ipDemo = NewDemo();

	init = new PVRShellInit();

	init->ShellView = this;
	init->Init(*ipDemo);
	init->CommandLine("");

	ipDemo->PVRShellSet(prefWidth, Size().iWidth);
	ipDemo->PVRShellSet(prefHeight, Size().iHeight);

	iPeriodic = CPeriodic::NewL(CActive::EPriorityIdle);
	iPeriodic->Start(100000,10000,TCallBack(CPVRShellView::DrawCallback,this)); 
}

/*!***********************************************************************
 @Function		ViewDeactivated
 @Description	
*************************************************************************/
void CPVRShellView::ViewDeactivated()
{
	delete iPeriodic;
}

/*!***********************************************************************
 @Function		DrawCallback
 @Description	Called every frame - runs shell code, and user demo
				code.
*************************************************************************/
TInt CPVRShellView::DrawCallback(TAny* aInstance){

	if(!((CPVRShellView*) aInstance)->init->Run())	CBaActiveScheduler::Exit();
	return 0;
}





/*!***************************************************************************
	Class: CShellAppUI
*****************************************************************************/

/*!***********************************************************************
 @Function		ConstructL
  @Description	Second phase constructor
*************************************************************************/
void CShellAppUI::ConstructL(){
	iCoeEnv->DisableExitChecks(ETrue);

	CQikAppUi::ConstructL();

	AddViewL(*CPVRShellView::NewL(this,ClientRect()));
}

/*!***********************************************************************
 @Function		HandleCommandL
  @Description	Command interface
*************************************************************************/
void CShellAppUI::HandleCommandL(TInt aCommand)
{
  switch (aCommand){
	case EEikCmdExit: 
			CEikAppUi::Exit();
			break;
		default:
			break;
	}
}



/*!***************************************************************************
	Class: CShellDocument
*****************************************************************************/

/*!***********************************************************************
 @Function		CShellDocument
 @Description	Constructor
*************************************************************************/
CShellDocument::CShellDocument(CEikApplication& aApp) : CQikDocument(aApp)
{
	
}

/*!***********************************************************************
 @Function		CShellDocument
 @Description	Destructor
*************************************************************************/
CShellDocument::~CShellDocument()
{
	CloseSTDLIB();
}

/*!***********************************************************************
 @Function		NewL
 @Description	
*************************************************************************/
CShellDocument* CShellDocument::NewL(CEikApplication& aApp)
{
	CShellDocument* self=new(ELeave) CShellDocument(aApp);
	return self;
}

/*!***********************************************************************
 @Function		CreateAppUiL
 @Description	
*************************************************************************/
CQikAppUi* CShellDocument::CreateAppUiL()
{
  return(new(ELeave) CShellAppUI);
}

/*!***********************************************************************
 @Function		AppDllUid
 @Description	
*************************************************************************/
TUid CShellApplication::AppDllUid() const
{
  return(KUIDPVRShell);
}


CApaDocument* CShellApplication::CreateDocumentL()
{
	return CShellDocument::NewL(*this);
}





/*!***************************************************************************
	Class: PVRShellInit
*****************************************************************************/

/*!***********************************************************************
 @Function		OsInit
 @description	Initialisation for OS-specific code.
*************************************************************************/
void PVRShellInit::OsInit()
{
	SetDataPath("\\System\\Data\\");
}

/*!***********************************************************************
 @Function		OsInitOS
 @description	Saves instance handle and creates main window
				In this function, we save the instance handle in a global variable and
				create and display the main program window.
*************************************************************************/
bool PVRShellInit::OsInitOS()
{
	// Initialize global strings

	m_pShell->m_pShellData->gIsRotated = 1;
	m_pShell->m_pShellData->gFullScreen  = 1;
	m_pShell->m_pShellData->gFSAAMode = 2;
	m_pShell->m_pShellData->gSwapInterval = 1;

	return true;
}

/*!***********************************************************************
 @Function		OsReleaseOS
 @description	Destroys main window
*************************************************************************/
void PVRShellInit::OsReleaseOS()
{
}

/*!***********************************************************************
 @Function		OsDoInitAPI
 @Return		true on success
 @description	Perform GL initialization and bring up window / fullscreen
*************************************************************************/
bool PVRShellInit::OsDoInitAPI()
{

	if(!ApiInitAPI())
	{
		RDebug::Printf("OsDoInitAPI ApiInitAPI() FAILED\n");
		return false;
	}


	/* No problem occured */
	return true;
}

/*!***********************************************************************
 @Function		OsDoReleaseAPI
 @description	Clean up after we're done
*************************************************************************/
void PVRShellInit::OsDoReleaseAPI()
{
	ApiReleaseAPI();
}

/*!***********************************************************************
 @Function		OsRenderComplete
 @Returns		false when the app should quit
 @description	Main message loop / render loop
*************************************************************************/
void PVRShellInit::OsRenderComplete()
{
}

/*!***********************************************************************
 @Function		OsPixmapCopy
 @Return		true if the copy succeeded
 @description	When using pixmaps, copy the render to the display
*************************************************************************/
bool PVRShellInit::OsPixmapCopy()
{
	return true;
}

/*!***********************************************************************
 @Function		OsGetNativeDisplayType
 @Return		The 'NativeDisplayType' for EGL
 @description	Called from InitAPI() to get the NativeDisplayType
*************************************************************************/
void *PVRShellInit::OsGetNativeDisplayType()
{
	return EGL_DEFAULT_DISPLAY;
}

/*!***********************************************************************
 @Function		OsGetNativePixmapType
 @Return		The 'NativePixmapType' for EGL
 @description	Called from InitAPI() to get the NativePixmapType
*************************************************************************/
void *PVRShellInit::OsGetNativePixmapType()
{
	// Pixmap support: return the pixmap
	return 0;
}

/*!***********************************************************************
 @Function		OsGetNativeWindowType
 @Return		The 'NativeWindowType' for EGL
 @description	Called from InitAPI() to get the NativeWindowType
*************************************************************************/
void *PVRShellInit::OsGetNativeWindowType()
{
	return ShellView->DrawableWindow();
}

/*!***********************************************************************
 @Function		OsGet
 @Input			prefName		value to retrieve
 @Output		pp				pointer to which to write the value
 @Description	Retrieves OS-specific data
*************************************************************************/
bool PVRShellInit::OsGet(const prefNamePtrEnum prefName, void **pp)
{
	return false;
}

/*!***********************************************************************
 @Function		OsDisplayDebugString
 @Input			str		string to output
 @Description	Prints a debug string
*************************************************************************/
void PVRShellInit::OsDisplayDebugString(char const * const str)
{
#ifndef NOSHELLDEBUG
	RDebug::Printf(str);
#endif
}


/*!***********************************************************************
 @Function		OsGetTime
 @Return		Time in milliseconds since the beginning of the application
 @Description	Gets the time in milliseconds since the beginning of the application
*************************************************************************/
unsigned long PVRShellInit::OsGetTime()
{
	TTime timeNow;
	timeNow.UniversalTime();
	return (static_cast<unsigned long>(timeNow.Int64())/1000);
}

/*!***********************************************************************
 @Function		OsGetTime
 @Return		Time in milliseconds since the beginning of the application
 @Description	Gets the time in milliseconds since the beginning of the application
*************************************************************************/
void PVRShellInit::OsExit()
{
}











