/******************************************************************************

 @File         OGLESLighting.h

 @Title        Lighting

 @Copyright    Copyright (C) 2003 - 2006 by Imagination Technologies Limited.

 @Platform     Independant

 @Description  PVRShell.

******************************************************************************/
#ifndef STRUCT_MESH_DEFINED
#define STRUCT_MESH_DEFINED 

typedef struct { unsigned int      nNumVertex;
                 unsigned int      nNumFaces;
                 unsigned int      nNumStrips;
                 unsigned int      nFlags;
                 unsigned int      nMaterial;
                 float             fCenter[3];
                 float             *pVertex;
                 float             *pUV;
                 float             *pNormals;
                 float             *pPackedVertex;
                 unsigned int      *pVertexColor;
                 unsigned int      *pVertexMaterial;
                 unsigned short    *pFaces;
                 unsigned short    *pStrips;
                 unsigned short    *pStripLength;
                 struct
                 {
                     unsigned int  nType;
                     unsigned int  nNumPatches;
                     unsigned int  nNumVertices;
                     unsigned int  nNumSubdivisions;
                     float         *pControlPoints;
                     float         *pUVs;
                 } Patch;
               }   Struct_Mesh;

#endif //STRUCT_MESH_DEFINED

/**********************************************************
 * GENERAL DEFINITIONS
 **********************************************************/

#ifndef _NULL_
#define _NULL_ 0
#endif

#define NUM_MESHES     1
#define NUM_BONES	   0
#define NUM_MATERIALS  1
#define NUM_LIGHTS     0
#define NUM_CAMERAS    0
#define NUM_PATHS      0
#define NUM_FRAMES     0

/* MESH FLAGS */

#ifndef MF_MATERIAL
#define MF_MATERIAL        1
#define MF_UV              2
#define MF_NORMALS         4
#define MF_VERTEXCOLOR     8
#define MF_VERTEXMATERIAL  16
#define MF_STRIPS          32
#define MF_NOFACELIST      64

#define MF_NOPATCH         0
#define MF_RECTPATCH       1
#define MF_TRIPATCH        2
#define MF_MIXPATCH        3

#endif


/* MESHES */
#define M_TORUS_KNOT01  0


/**********************************************************
 * MESHES DATA 
 **********************************************************/

/* MESH 0 */

#define torus_knot01_NumVertex  50272
#define torus_knot01_NumFaces   49152
#define torus_knot01_NumStrips  560
#define torus_knot01_Flags      0x00000166
#define torus_knot01_Material   0

float torus_knot01_Vertices[] = {
	51.2777f,29.1004f,1.6762f,  50.6094f,28.4221f,1.0437f,  49.5072f,27.5470f,2.4654f,  48.8412f,26.8972f,1.8973f,  47.5776f,26.2703f,3.3160f,  
	46.9127f,25.6492f,2.7941f,  45.5079f,25.2774f,4.1690f,  44.8425f,24.6823f,3.6782f,  43.3191f,24.5659f,4.9697f,  42.6519f,23.9926f,4.4974f,  
	41.0349f,24.1256f,5.6677f,  40.3654f,23.5679f,5.2010f,  38.6817f,23.9385f,6.2158f,  38.0097f,23.3894f,5.7446f,  36.2877f,23.9805f,6.5736f,  
	35.6144f,23.4319f,6.0894f,  33.8835f,24.2217f,6.7077f,  33.2087f,23.6666f,6.2055f,  31.4987f,24.6299f,6.5932f,  30.8252f,24.0593f,6.0687f,  
	29.1660f,25.1676f,6.2126f,  28.4928f,24.5761f,5.6646f,  26.9132f,25.7990f,5.5569f,  26.2430f,25.1796f,4.9856f,  24.7709f,26.4853f,4.6253f,  
	24.1035f,25.8328f,4.0314f,  22.7648f,27.1893f,3.4233f,  22.1027f,26.4997f,2.8105f,  20.9210f,27.8749f,1.9654f,  20.2640f,27.1454f,1.3366f,  
	19.2602f,28.5086f,0.2706f,  18.6096f,27.7371f,-0.3694f,  17.8016f,29.0590f,-1.6351f,  17.1569f,28.2451f,-2.2826f,  16.5589f,29.4984f,-3.7227f,  
	15.9208f,28.6435f,-4.3725f,  15.5435f,29.8036f,-5.9580f,  14.9117f,28.9089f,-6.6055f,  14.7618f,29.9542f,-8.3045f,  14.1358f,29.0221f,-8.9460f,  
	14.2161f,29.9343f,-10.7247f,  13.5954f,28.9680f,-11.3563f,  13.9045f,29.7319f,-13.1792f,  13.2887f,28.7352f,-13.7979f,  13.8212f,29.3393f,-15.6292f,  
	13.2096f,28.3162f,-16.2321f,  13.9565f,28.7524f,-18.0360f,  13.3483f,27.7071f,-18.6212f,  14.2968f,27.9711f,-20.3627f,  13.6914f,26.9074f,-20.9294f,  
	14.8252f,26.9980f,-22.5751f,  14.2218f,25.9200f,-23.1225f,  15.5217f,25.8395f,-24.6410f,  14.9199f,24.7518f,-25.1681f,  16.3636f,24.5054f,-26.5309f,  
	15.7627f,23.4106f,-27.0387f,  17.3257f,23.0066f,-28.2202f,  16.7255f,21.9085f,-28.7091f,  18.3814f,21.3577f,-29.6872f,  17.7815f,20.2583f,-30.1585f,  
	19.5023f,19.5739f,-30.9152f,  18.9025f,18.4762f,-31.3696f,  20.6591f,17.6735f,-31.8910f,  20.0592f,16.5788f,-32.3302f,  21.8221f,15.6745f,-32.6067f,  
	21.2222f,14.5843f,-33.0321f,  22.9619f,13.5969f,-33.0584f,  22.3618f,12.5121f,-33.4715f,  24.0491f,11.4610f,-33.2470f,  23.4492f,10.3820f,-33.6491f,  
	25.0559f,9.2875f,-33.1773f,  24.4553f,8.2136f,-33.5702f,  25.9546f,7.0961f,-32.8593f,  25.3546f,6.0288f,-33.2435f,  26.7216f,4.9092f,-32.3055f,  
	26.1211f,3.8465f,-32.6826f,  27.3331f,2.7451f,-31.5336f,  26.7326f,1.6873f,-31.9041f,  27.7695f,0.6242f,-30.5636f,  27.1688f,-0.4295f,-30.9283f,  
	28.0135f,-1.4351f,-29.4192f,  27.4120f,-2.4859f,-29.7786f,  28.0507f,-3.4159f,-28.1265f,  27.4486f,-4.4639f,-28.4807f,  27.8709f,-5.3017f,-26.7139f,  
	27.2673f,-6.3480f,-27.0632f,  27.4664f,-7.0784f,-25.2114f,  26.8611f,-8.1229f,-25.5557f,  26.8342f,-8.7326f,-23.6506f,  26.2267f,-9.7756f,-23.9897f,  
	25.9746f,-10.2532f,-22.0635f,  25.3641f,-11.2949f,-22.3973f,  24.8915f,-11.6313f,-20.4828f,  24.2773f,-12.6713f,-20.8112f,  23.5929f,-12.8598f,-18.9408f,  
	22.9743f,-13.8974f,-19.2635f,  22.0902f,-13.9335f,-17.4693f,  21.4667f,-14.9678f,-17.7862f,  51.2777f,29.1004f,1.6762f,  49.5072f,27.5470f,2.4654f,  
	50.1689f,28.1756f,3.0608f,  48.2388f,26.8717f,3.8573f,  48.8941f,27.4516f,4.4219f,  46.8260f,26.4080f,5.1956f,  47.4749f,26.9409f,5.7362f,  
	45.2929f,26.1637f,6.4573f,  45.9358f,26.6522f,6.9812f,  43.6657f,26.1429f,7.6209f,  44.3031f,26.5904f,8.1355f,  41.9722f,26.3462f,8.6664f,  
	42.6047f,26.7566f,9.1795f,  40.2416f,26.7699f,9.5760f,  40.8696f,27.1479f,10.0952f,  38.5033f,27.4069f,10.3341f,  39.1276f,27.7576f,10.8670f,  
	36.7871f,28.2462f,10.9278f,  37.4084f,28.5754f,11.4816f,  35.1224f,29.2734f,11.3468f,  35.7415f,29.5872f,11.9287f,  33.5377f,30.4711f,11.5841f,  
	34.1554f,30.7757f,12.2004f,  32.0600f,31.8187f,11.6354f,  32.6772f,32.1206f,12.2923f,  30.7147f,33.2931f,11.5001f,  31.3323f,33.5988f,12.2026f,  
	29.5247f,34.8691f,11.1803f,  30.1436f,35.1851f,11.9329f,  28.5105f,36.5198f,10.6816f,  29.1314f,36.8524f,11.4879f,  27.6894f,38.2169f,10.0125f,  
	28.3132f,38.5720f,10.8752f,  27.0754f,39.9314f,9.1844f,  27.7028f,40.3146f,10.1052f,  26.6791f,41.6339f,8.2115f,  27.3108f,42.0504f,9.1911f,  
	26.5072f,43.2953f,7.1105f,  27.1439f,43.7496f,8.1486f,  26.5626f,44.8872f,5.9001f,  27.2048f,45.3832f,6.9954f,  26.8445f,46.3823f,4.6012f,  
	27.4926f,46.9233f,5.7514f,  27.3480f,47.7551f,3.2359f,  28.0023f,48.3434f,4.4378f,  28.0644f,48.9821f,1.8276f,  28.7252f,49.6193f,3.0771f,  
	28.9816f,50.0422f,0.4003f,  29.6490f,50.7292f,1.6926f,  30.0838f,50.9174f,-1.0214f,  30.7578f,51.6540f,0.3079f,  31.3522f,51.5926f,-2.4133f,  
	32.0327f,52.3779f,-1.0532f,  32.7650f,52.0564f,-3.7516f,  33.4518f,52.8887f,-2.3675f,  34.2982f,52.3007f,-5.0133f,  34.9909f,53.1774f,-3.6125f,  
	35.9253f,52.3214f,-6.1769f,  36.6237f,53.2392f,-4.7668f,  37.6188f,52.1182f,-7.2224f,  38.3221f,53.0730f,-5.8107f,  39.3494f,51.6944f,-8.1320f,  
	40.0572f,52.6817f,-6.7264f,  41.0877f,51.0575f,-8.8901f,  41.7992f,52.0719f,-7.4982f,  42.8039f,50.2182f,-9.4838f,  43.5183f,51.2542f,-8.1129f,  
	44.4686f,49.1909f,-9.9028f,  45.1852f,50.2424f,-8.5599f,  46.0533f,47.9932f,-10.1400f,  46.7713f,49.0539f,-8.8317f,  47.5310f,46.6457f,-10.1914f,  
	48.2495f,47.7090f,-8.9235f,  48.8763f,45.1713f,-10.0560f,  49.5944f,46.2307f,-8.8338f,  50.0663f,43.5952f,-9.7363f,  50.7832f,44.6444f,-8.5642f,  
	51.0805f,41.9445f,-9.2376f,  51.7953f,42.9772f,-8.1192f,  51.9016f,40.2474f,-8.5684f,  52.6135f,41.2576f,-7.5064f,  52.5156f,38.5330f,-7.7403f,  
	53.2239f,39.5150f,-6.7364f,  52.9119f,36.8304f,-6.7675f,  53.6159f,37.7792f,-5.8223f,  53.0838f,35.1690f,-5.6664f,  53.7829f,36.0800f,-4.7798f,  
	53.0283f,33.5771f,-4.4561f,  53.7219f,34.4464f,-3.6267f,  52.7465f,32.0820f,-3.1572f,  53.4342f,32.9063f,-2.3827f,  52.2430f,30.7092f,-1.7919f,  
	52.9244f,31.4862f,-1.0691f,  51.5265f,29.4823f,-0.3835f,  52.2015f,30.2103f,0.2916f,  50.6094f,28.4221f,1.0437f,  51.2777f,29.1004f,1.6762f,  
	50.6094f,28.4221f,1.0437f,  49.9385f,27.7228f,0.4426f,  48.8412f,26.8972f,1.8973f,  48.1729f,26.2285f,1.3523f,  46.9127f,25.6492f,2.7941f,  
	46.2452f,25.0097f,2.2900f,  44.8425f,24.6823f,3.6782f,  44.1743f,24.0689f,3.2018f,  42.6519f,23.9926f,4.4974f,  41.9823f,23.4001f,4.0349f,  
	40.3654f,23.5679f,5.2010f,  39.6938f,22.9901f,4.7414f,  38.0097f,23.3894f,5.7446f,  37.3371f,22.8185f,5.2781f,  35.6144f,23.4319f,6.0894f,  
	34.9404f,22.8611f,5.6101f,  33.2087f,23.6666f,6.2055f,  32.5358f,23.0870f,5.7073f,  30.8252f,24.0593f,6.0687f,  30.1525f,23.4651f,5.5498f,  
	28.4928f,24.5761f,5.6646f,  27.8227f,23.9599f,5.1235f,  26.2430f,25.1796f,4.9856f,  25.5755f,24.5357f,4.4223f,  24.1035f,25.8328f,4.0314f,  
	23.4409f,25.1564f,3.4484f,  22.1027f,26.4997f,2.8105f,  21.4449f,25.7873f,2.2101f,  20.2640f,27.1454f,1.3366f,  19.6124f,26.3941f,0.7229f,  
	18.6096f,27.7371f,-0.3694f,  17.9637f,26.9457f,-0.9932f,  17.1569f,28.2451f,-2.2826f,  16.5175f,27.4141f,-2.9115f,  15.9208f,28.6435f,-4.3725f,  
	15.2878f,27.7732f,-5.0021f,  14.9117f,28.9089f,-6.6055f,  14.2846f,28.0011f,-7.2321f,  14.1358f,29.0221f,-8.9460f,  13.5142f,28.0795f,-9.5657f,  
	13.5954f,28.9680f,-11.3563f,  12.9788f,27.9937f,-11.9659f,  13.2887f,28.7352f,-13.7979f,  12.6764f,27.7329f,-14.3943f,  13.2096f,28.3162f,-16.2321f,  
	12.6010f,27.2896f,-16.8131f,  13.3483f,27.7071f,-18.6212f,  12.7426f,26.6601f,-19.1856f,  13.6914f,26.9074f,-20.9294f,  13.0879f,25.8438f,-21.4761f,  
	14.2218f,25.9200f,-23.1225f,  13.6201f,24.8443f,-23.6502f,  14.9199f,24.7518f,-25.1681f,  14.3192f,23.6667f,-25.6775f,  15.7627f,23.4106f,-27.0387f,  
	15.1627f,22.3199f,-27.5299f,  16.7255f,21.9085f,-28.7091f,  16.1258f,20.8145f,-29.1831f,  17.7815f,20.2583f,-30.1585f,  17.1818f,19.1642f,-30.6158f,  
	18.9025f,18.4762f,-31.3696f,  18.3026f,17.3835f,-31.8118f,  20.0592f,16.5788f,-32.3302f,  19.4591f,15.4890f,-32.7586f,  21.2222f,14.5843f,-33.0321f,  
	20.6217f,13.4988f,-33.4480f,  22.3618f,12.5121f,-33.4715f,  21.7612f,11.4314f,-33.8762f,  23.4492f,10.3820f,-33.6491f,  22.8476f,9.3057f,-34.0443f,  
	24.4553f,8.2136f,-33.5702f,  23.8540f,7.1433f,-33.9565f,  25.3546f,6.0288f,-33.2435f,  24.7526f,4.9626f,-33.6224f,  26.1211f,3.8465f,-32.6826f,  
	25.5188f,2.7848f,-33.0548f,  26.7326f,1.6873f,-31.9041f,  26.1298f,0.6296f,-32.2703f,  27.1688f,-0.4295f,-30.9283f,  26.5650f,-1.4845f,-31.2891f,  
	27.4120f,-2.4859f,-29.7786f,  26.8075f,-3.5382f,-30.1342f,  27.4486f,-4.4639f,-28.4807f,  26.8424f,-5.5147f,-28.8316f,  27.2673f,-6.3480f,-27.0632f,  
	26.6592f,-7.3974f,-27.4091f,  26.8611f,-8.1229f,-25.5557f,  26.2507f,-9.1710f,-25.8965f,  26.2267f,-9.7756f,-23.9897f,  25.6132f,-10.8227f,-24.3253f,  
	25.3641f,-11.2949f,-22.3973f,  24.7467f,-12.3409f,-22.7276f,  24.2773f,-12.6713f,-20.8112f,  23.6555f,-13.7156f,-21.1357f,  22.9743f,-13.8974f,-19.2635f,  
	22.3474f,-14.9390f,-19.5822f,  21.4667f,-14.9678f,-17.7862f,  20.8339f,-16.0057f,-18.0989f,  49.5072f,27.5470f,2.4654f,  47.5776f,26.2703f,3.3160f,  
	48.2388f,26.8717f,3.8573f,  46.1695f,25.8531f,4.6743f,  46.8260f,26.4080f,5.1956f,  44.6408f,25.6525f,5.9491f,  45.2929f,26.1637f,6.4573f,  
	43.0177f,25.6722f,7.1185f,  43.6657f,26.1429f,7.6209f,  41.3281f,25.9116f,8.1625f,  41.9722f,26.3462f,8.6664f,  39.6008f,26.3668f,9.0632f,  
	40.2416f,26.7699f,9.5760f,  37.8654f,27.0299f,9.8053f,  38.5033f,27.4069f,10.3341f,  36.1515f,27.8896f,10.3760f,  36.7871f,28.2462f,10.9278f,  
	34.4886f,28.9312f,10.7656f,  35.1224f,29.2734f,11.3468f,  32.9050f,30.1368f,10.9675f,  33.5377f,30.4711f,11.5841f,  31.4278f,31.4859f,10.9780f,  
	32.0600f,31.8187f,11.6354f,  30.0824f,32.9553f,10.7972f,  30.7147f,33.2931f,11.5001f,  28.8917f,34.5199f,10.4280f,  29.5247f,34.8691f,11.1803f,  
	27.8761f,36.1530f,9.8768f,  28.5105f,36.5198f,10.6816f,  27.0530f,37.8265f,9.1530f,  27.6894f,38.2169f,10.0125f,  26.4364f,39.5119f,8.2690f,  
	27.0754f,39.9314f,9.1844f,  26.0370f,41.1804f,7.2399f,  26.6791f,41.6339f,8.2115f,  25.8616f,42.8032f,6.0834f,  26.5072f,43.2953f,7.1105f,  
	25.9131f,44.3528f,4.8192f,  26.5626f,44.8872f,5.9001f,  26.1907f,45.8026f,3.4689f,  26.8445f,46.3823f,4.6012f,  26.6896f,47.1278f,2.0557f,  
	27.3480f,47.7551f,3.2359f,  27.4014f,48.3057f,0.6038f,  28.0644f,48.9821f,1.8276f,  28.3137f,49.3161f,-0.8621f,  28.9816f,50.0422f,0.4003f,  
	29.4110f,50.1418f,-2.3168f,  30.0838f,50.9174f,-1.0214f,  30.6746f,50.7686f,-3.7355f,  31.3522f,51.5926f,-2.4133f,  32.0828f,51.1859f,-5.0938f,  
	32.7650f,52.0564f,-3.7516f,  33.6114f,51.3864f,-6.3686f,  34.2982f,52.3007f,-5.0133f,  35.2345f,51.3668f,-7.5380f,  35.9253f,52.3214f,-6.1769f,  
	36.9241f,51.1273f,-8.5820f,  37.6188f,52.1182f,-7.2224f,  38.6514f,50.6721f,-9.4827f,  39.3494f,51.6944f,-8.1320f,  40.3868f,50.0090f,-10.2248f,  
	41.0877f,51.0575f,-8.8901f,  42.1007f,49.1493f,-10.7955f,  42.8039f,50.2182f,-9.4838f,  43.7636f,48.1077f,-11.1851f,  44.4686f,49.1909f,-9.9028f,  
	45.3472f,46.9021f,-11.3869f,  46.0533f,47.9932f,-10.1400f,  46.8244f,45.5530f,-11.3975f,  47.5310f,46.6457f,-10.1914f,  48.1698f,44.0836f,-11.2166f,  
	48.8763f,45.1713f,-10.0560f,  49.3605f,42.5190f,-10.8474f,  50.0663f,43.5952f,-9.7363f,  50.3761f,40.8859f,-10.2962f,  51.0805f,41.9445f,-9.2376f,  
	51.1992f,39.2124f,-9.5724f,  51.9016f,40.2474f,-8.5684f,  51.8157f,37.5270f,-8.6884f,  52.5156f,38.5330f,-7.7403f,  52.2151f,35.8586f,-7.6594f,  
	52.9119f,36.8304f,-6.7675f,  52.3906f,34.2357f,-6.5029f,  53.0838f,35.1690f,-5.6664f,  52.3391f,32.6861f,-5.2387f,  53.0283f,33.5771f,-4.4561f,  
	52.0615f,31.2363f,-3.8884f,  52.7465f,32.0820f,-3.1572f,  51.5625f,29.9111f,-2.4752f,  52.2430f,30.7092f,-1.7919f,  50.8508f,28.7333f,-1.0233f,  
	51.5265f,29.4823f,-0.3835f,  49.9385f,27.7228f,0.4426f,  50.6094f,28.4221f,1.0437f,  49.9385f,27.7228f,0.4426f,  49.2668f,27.0054f,-0.1315f,  
	48.1729f,26.2285f,1.3523f,  47.5032f,25.5425f,0.8286f,  46.2452f,25.0097f,2.2900f,  45.5760f,24.3532f,1.8030f,  44.1743f,24.0689f,3.2018f,  
	43.5046f,23.4379f,2.7374f,  41.9823f,23.4001f,4.0349f,  41.3112f,22.7892f,3.5813f,  39.6938f,22.9901f,4.7414f,  39.0221f,22.3921f,4.2876f,  
	37.3371f,22.8185f,5.2781f,  36.6641f,22.2271f,4.8172f,  34.9404f,22.8611f,5.6101f,  34.2684f,22.2673f,5.1344f,  32.5358f,23.0870f,5.7073f,  
	31.8639f,22.4850f,5.2143f,  30.1525f,23.4651f,5.5498f,  29.4830f,22.8472f,5.0365f,  27.8227f,23.9599f,5.1235f,  27.1555f,23.3200f,4.5892f,  
	25.5755f,24.5357f,4.4223f,  24.9128f,23.8682f,3.8682f,  23.4409f,25.1564f,3.4484f,  22.7828f,24.4574f,2.8762f,  21.4449f,25.7873f,2.2101f,  
	20.7927f,25.0529f,1.6229f,  19.6124f,26.3941f,0.7229f,  18.9656f,25.6223f,0.1235f,  17.9637f,26.9457f,-0.9932f,  17.3234f,26.1363f,-1.6000f,  
	16.5175f,27.4141f,-2.9115f,  15.8836f,26.5665f,-3.5218f,  15.2878f,27.7732f,-5.0021f,  14.6598f,26.8887f,-5.6122f,  14.2846f,28.0011f,-7.2321f,  
	13.6623f,27.0816f,-7.8382f,  13.5142f,28.0795f,-9.5657f,  12.8970f,27.1276f,-10.1644f,  12.9788f,27.9937f,-11.9659f,  12.3662f,27.0126f,-12.5539f,  
	12.6764f,27.7329f,-14.3943f,  12.0677f,26.7258f,-14.9692f,  12.6010f,27.2896f,-16.8131f,  11.9953f,26.2603f,-17.3735f,  12.7426f,26.6601f,-19.1856f,  
	12.1394f,25.6120f,-19.7301f,  13.0879f,25.8438f,-21.4761f,  12.4866f,24.7814f,-22.0030f,  13.6201f,24.8443f,-23.6502f,  13.0199f,23.7704f,-24.1599f,  
	14.3192f,23.6667f,-25.6775f,  13.7197f,22.5850f,-26.1699f,  15.1627f,22.3199f,-27.5299f,  14.5635f,21.2329f,-28.0057f,  16.1258f,20.8145f,-29.1831f,  
	15.5266f,19.7254f,-29.6426f,  17.1818f,19.1642f,-30.6158f,  16.5822f,18.0749f,-31.0604f,  18.3026f,17.3835f,-31.8118f,  17.7026f,16.2956f,-32.2426f,  
	19.4591f,15.4890f,-32.7586f,  18.8585f,14.4041f,-33.1768f,  20.6217f,13.4988f,-33.4480f,  20.0208f,12.4175f,-33.8549f,  21.7612f,11.4314f,-33.8762f,  
	21.1591f,10.3538f,-34.2734f,  22.8476f,9.3057f,-34.0443f,  22.2455f,8.2334f,-34.4323f,  23.8540f,7.1433f,-33.9565f,  23.2509f,6.0745f,-34.3371f,  
	24.7526f,4.9626f,-33.6224f,  24.1488f,3.8981f,-33.9961f,  25.5188f,2.7848f,-33.0548f,  24.9144f,1.7240f,-33.4224f,  26.1298f,0.6296f,-32.2703f,  
	25.5243f,-0.4288f,-32.6325f,  26.5650f,-1.4845f,-31.2891f,  25.9584f,-2.5404f,-31.6461f,  26.8075f,-3.5382f,-30.1342f,  26.1990f,-4.5929f,-30.4864f,  
	26.8424f,-5.5147f,-28.8316f,  26.2320f,-6.5683f,-29.1789f,  26.6592f,-7.3974f,-27.4091f,  26.0464f,-8.4499f,-27.7513f,  26.2507f,-9.1710f,-25.8965f,  
	25.6347f,-10.2230f,-26.2337f,  25.6132f,-10.8227f,-24.3253f,  24.9931f,-11.8742f,-24.6572f,  24.7467f,-12.3409f,-22.7276f,  24.1221f,-13.3912f,-23.0538f,  
	23.6555f,-13.7156f,-21.1357f,  23.0256f,-14.7640f,-21.4561f,  22.3474f,-14.9390f,-19.5822f,  21.7114f,-15.9846f,-19.8966f,  20.8339f,-16.0057f,-18.0989f,  
	20.1912f,-17.0473f,-18.4071f,  47.5776f,26.2703f,3.3160f,  45.5079f,25.2774f,4.1690f,  46.1695f,25.8531f,4.6743f,  43.9824f,25.1196f,5.4535f,  
	44.6408f,25.6525f,5.9491f,  42.3624f,25.1784f,6.6258f,  43.0177f,25.6722f,7.1185f,  40.6755f,25.4529f,7.6657f,  41.3281f,25.9116f,8.1625f,  
	38.9507f,25.9382f,8.5554f,  39.6008f,26.3668f,9.0632f,  37.2175f,26.6261f,9.2797f,  37.8654f,27.0299f,9.8053f,  35.5054f,27.5049f,9.8262f,  
	36.1515f,27.8896f,10.3760f,  33.8438f,28.5595f,10.1856f,  34.4886f,28.9312f,10.7656f,  32.2612f,29.7719f,10.3517f,  32.9050f,30.1368f,10.9675f,  
	30.7846f,31.1212f,10.3217f,  31.4278f,31.4859f,10.9780f,  29.4392f,32.5845f,10.0961f,  30.0824f,32.9553f,10.7972f,  28.2482f,34.1367f,9.6787f,  
	28.8917f,34.5199f,10.4280f,  27.2318f,35.7512f,9.0767f,  27.8761f,36.1530f,9.8768f,  26.4074f,37.4005f,8.3004f,  27.0530f,37.8265f,9.1530f,  
	25.7893f,39.0562f,7.3630f,  26.4364f,39.5119f,8.2690f,  25.3879f,40.6900f,6.2807f,  26.0370f,41.1804f,7.2399f,  25.2101f,42.2741f,5.0719f,  
	25.8616f,42.8032f,6.0834f,  25.2589f,43.7813f,3.7573f,  25.9131f,44.3528f,4.8192f,  25.5336f,45.1858f,2.3594f,  26.1907f,45.8026f,3.4689f,  
	26.0294f,46.4635f,0.9022f,  26.6896f,47.1278f,2.0557f,  26.7378f,47.5927f,-0.5896f,  27.4014f,48.3057f,0.6038f,  27.6468f,48.5540f,-2.0902f,  
	28.3137f,49.3161f,-0.8621f,  28.7407f,49.3309f,-3.5740f,  29.4110f,50.1418f,-2.3168f,  30.0008f,49.9102f,-5.0158f,  30.6746f,50.7686f,-3.7355f,  
	31.4057f,50.2819f,-6.3907f,  32.0828f,51.1859f,-5.0938f,  32.9311f,50.4397f,-7.6752f,  33.6114f,51.3864f,-6.3686f,  34.5512f,50.3809f,-8.8475f,  
	35.2345f,51.3668f,-7.5380f,  36.2380f,50.1065f,-9.8874f,  36.9241f,51.1273f,-8.5820f,  37.9628f,49.6212f,-10.7771f,  38.6514f,50.6721f,-9.4827f,  
	39.6961f,48.9332f,-11.5014f,  40.3868f,50.0090f,-10.2248f,  41.4082f,48.0544f,-12.0479f,  42.1007f,49.1493f,-10.7955f,  43.0697f,46.9999f,-12.4073f,  
	43.7636f,48.1077f,-11.1851f,  44.6524f,45.7875f,-12.5734f,  45.3472f,46.9021f,-11.3869f,  46.1290f,44.4381f,-12.5434f,  46.8244f,45.5530f,-11.3975f,  
	47.4743f,42.9748f,-12.3177f,  48.1698f,44.0836f,-11.2166f,  48.6654f,41.4227f,-11.9003f,  49.3605f,42.5190f,-10.8474f,  49.6818f,39.8081f,-11.2983f,  
	50.3761f,40.8859f,-10.2962f,  50.5061f,38.1589f,-10.5220f,  51.1992f,39.2124f,-9.5724f,  51.1242f,36.5032f,-9.5847f,  51.8157f,37.5270f,-8.6884f,  
	51.5257f,34.8693f,-8.5024f,  52.2151f,35.8586f,-7.6594f,  51.7035f,33.2853f,-7.2936f,  52.3906f,34.2357f,-6.5029f,  51.6546f,31.7781f,-5.9790f,  
	52.3391f,32.6861f,-5.2387f,  51.3799f,30.3736f,-4.5811f,  52.0615f,31.2363f,-3.8884f,  50.8841f,29.0958f,-3.1238f,  51.5625f,29.9111f,-2.4752f,  
	50.1757f,27.9667f,-1.6321f,  50.8508f,28.7333f,-1.0233f,  49.2668f,27.0054f,-0.1315f,  49.9385f,27.7228f,0.4426f,  49.2668f,27.0054f,-0.1315f,  
	48.5950f,26.2716f,-0.6809f,  47.5032f,25.5425f,0.8286f,  46.8331f,24.8408f,0.3251f,  45.5760f,24.3532f,1.8030f,  44.9063f,23.6808f,1.3309f,  
	43.5046f,23.4379f,2.7374f,  42.8342f,22.7902f,2.2842f,  41.3112f,22.7892f,3.5813f,  40.6405f,22.1599f,3.1349f,  39.0221f,22.3921f,4.2876f,  
	38.3505f,21.7753f,3.8403f,  36.6641f,22.2271f,4.8172f,  35.9935f,21.6142f,4.3599f,  34.2684f,22.2673f,5.1344f,  33.5978f,21.6526f,4.6639f,  
	31.8639f,22.4850f,5.2143f,  31.1954f,21.8605f,4.7262f,  29.4830f,22.8472f,5.0365f,  28.8165f,22.2066f,4.5293f,  27.1555f,23.3200f,4.5892f,  
	26.4932f,22.6572f,4.0627f,  24.9128f,23.8682f,3.8682f,  24.2547f,23.1786f,3.3234f,  22.7828f,24.4574f,2.8762f,  22.1304f,23.7363f,2.3153f,  
	20.7927f,25.0529f,1.6229f,  20.1455f,24.2977f,1.0484f,  18.9656f,25.6223f,0.1235f,  18.3248f,24.8321f,-0.4606f,  17.3234f,26.1363f,-1.6000f,  
	16.6889f,25.3096f,-2.1900f,  15.8836f,26.5665f,-3.5218f,  15.2550f,25.7040f,-4.1142f,  14.6598f,26.8887f,-5.6122f,  14.0369f,25.9914f,-6.2032f,  
	13.6623f,27.0816f,-7.8382f,  13.0447f,26.1517f,-8.4243f,  12.8970f,27.1276f,-10.1644f,  12.2843f,26.1676f,-10.7424f,  12.3662f,27.0126f,-12.5539f,  
	11.7575f,26.0254f,-13.1212f,  12.0677f,26.7258f,-14.9692f,  11.4622f,25.7148f,-15.5241f,  11.9953f,26.2603f,-17.3735f,  11.3925f,25.2287f,-17.9142f,  
	12.1394f,25.6120f,-19.7301f,  11.5386f,24.5642f,-20.2547f,  12.4866f,24.7814f,-22.0030f,  11.8871f,23.7201f,-22.5118f,  13.0199f,23.7704f,-24.1599f,  
	12.4212f,22.6992f,-24.6524f,  13.7197f,22.5850f,-26.1699f,  13.1213f,21.5065f,-26.6466f,  14.5635f,21.2329f,-28.0057f,  13.9650f,20.1505f,-28.4666f,  
	15.5266f,19.7254f,-29.6426f,  14.9277f,18.6410f,-30.0888f,  16.5822f,18.0749f,-31.0604f,  15.9828f,16.9903f,-31.4930f,  17.7026f,16.2956f,-32.2426f,  
	17.1024f,15.2127f,-32.6626f,  18.8585f,14.4041f,-33.1768f,  18.2577f,13.3237f,-33.5855f,  20.0208f,12.4175f,-33.8549f,  19.4186f,11.3399f,-34.2536f,  
	21.1591f,10.3538f,-34.2734f,  20.5566f,9.2805f,-34.6629f,  22.2455f,8.2334f,-34.4323f,  21.6417f,7.1632f,-34.8142f,  23.2509f,6.0745f,-34.3371f,  
	22.6463f,5.0081f,-34.7119f,  24.1488f,3.8981f,-33.9961f,  23.5434f,2.8350f,-34.3647f,  24.9144f,1.7240f,-33.4224f,  24.3075f,0.6630f,-33.7855f,  
	25.5243f,-0.4288f,-32.6325f,  24.9161f,-1.4875f,-32.9904f,  25.9584f,-2.5404f,-31.6461f,  25.3483f,-3.5981f,-31.9993f,  26.1990f,-4.5929f,-30.4864f,  
	25.5867f,-5.6498f,-30.8348f,  26.2320f,-6.5683f,-29.1789f,  25.6171f,-7.6244f,-29.5224f,  26.0464f,-8.4499f,-27.7513f,  25.4282f,-9.5059f,-28.0899f,  
	25.6347f,-10.2230f,-26.2337f,  25.0123f,-11.2792f,-26.5671f,  24.9931f,-11.8742f,-24.6572f,  24.3661f,-12.9299f,-24.9851f,  24.1221f,-13.3912f,-23.0538f,  
	23.4896f,-14.4458f,-23.3759f,  23.0256f,-14.7640f,-21.4561f,  22.3869f,-15.8167f,-21.7721f,  21.7114f,-15.9846f,-19.8966f,  21.0657f,-17.0342f,-20.2063f,  
	20.1912f,-17.0473f,-18.4071f,  19.5378f,-18.0921f,-18.7105f,  45.5079f,25.2774f,4.1690f,  43.3191f,24.5659f,4.9697f,  43.9824f,25.1196f,5.4535f,  
	41.7010f,24.6625f,6.1421f,  42.3624f,25.1784f,6.6258f,  40.0160f,24.9705f,7.1756f,  40.6755f,25.4529f,7.6657f,  38.2929f,25.4846f,8.0524f,  
	38.9507f,25.9382f,8.5554f,  36.5612f,26.1959f,8.7574f,  37.2175f,26.6261f,9.2797f,  34.8504f,27.0924f,9.2787f,  35.5054f,27.5049f,9.8262f,  
	33.1899f,28.1587f,9.6074f,  33.8438f,28.5595f,10.1856f,  31.6081f,29.3765f,9.7377f,  32.2612f,29.7719f,10.3517f,  30.1320f,30.7249f,9.6675f,  
	30.7846f,31.1212f,10.3217f,  28.7870f,32.1811f,9.3979f,  29.4392f,32.5845f,10.0961f,  27.5959f,33.7199f,8.9337f,  28.2482f,34.1367f,9.6787f,  
	26.5793f,35.3151f,8.2826f,  27.2318f,35.7512f,9.0767f,  25.7544f,36.9394f,7.4559f,  26.4074f,37.4005f,8.3004f,  25.1354f,38.5649f,6.4678f,  
	25.7893f,39.0562f,7.3630f,  24.7330f,40.1640f,5.3350f,  25.3879f,40.6900f,6.2807f,  24.5539f,41.7092f,4.0770f,  25.2101f,42.2741f,5.0719f,  
	24.6012f,43.1740f,2.7154f,  25.2589f,43.7813f,3.7573f,  24.8742f,44.5335f,1.2734f,  25.5336f,45.1858f,2.3594f,  25.3682f,45.7643f,-0.2243f,  
	26.0294f,46.4635f,0.9022f,  26.0746f,46.8455f,-1.7521f,  26.7378f,47.5927f,-0.5896f,  26.9815f,47.7584f,-3.2839f,  27.6468f,48.5540f,-2.0902f,  
	28.0733f,48.4875f,-4.7933f,  28.7407f,49.3309f,-3.5740f,  29.3313f,49.0203f,-6.2547f,  30.0008f,49.9102f,-5.0158f,  30.7340f,49.3477f,-7.6430f,  
	31.4057f,50.2819f,-6.3907f,  32.2574f,49.4641f,-8.9344f,  32.9311f,50.4397f,-7.6752f,  33.8755f,49.3674f,-10.1069f,  34.5512f,50.3809f,-8.8475f,  
	35.5605f,49.0594f,-11.1403f,  36.2380f,50.1065f,-9.8874f,  37.2836f,48.5454f,-12.0171f,  37.9628f,49.6212f,-10.7771f,  39.0153f,47.8340f,-12.7222f,  
	39.6961f,48.9332f,-11.5014f,  40.7261f,46.9375f,-13.2435f,  41.4082f,48.0544f,-12.0479f,  42.3866f,45.8713f,-13.5721f,  43.0697f,46.9999f,-12.4073f,  
	43.9684f,44.6535f,-13.7024f,  44.6524f,45.7875f,-12.5734f,  45.4445f,43.3050f,-13.6322f,  46.1290f,44.4381f,-12.5434f,  46.7895f,41.8489f,-13.3627f,  
	47.4743f,42.9748f,-12.3177f,  47.9806f,40.3101f,-12.8984f,  48.6654f,41.4227f,-11.9003f,  48.9972f,38.7149f,-12.2473f,  49.6818f,39.8081f,-11.2983f,  
	49.8221f,37.0906f,-11.4207f,  50.5061f,38.1589f,-10.5220f,  50.4411f,35.4650f,-10.4325f,  51.1242f,36.5032f,-9.5847f,  50.8435f,33.8659f,-9.2997f,  
	51.5257f,34.8693f,-8.5024f,  51.0226f,32.3208f,-8.0418f,  51.7035f,33.2853f,-7.2936f,  50.9753f,30.8559f,-6.6801f,  51.6546f,31.7781f,-5.9790f,  
	50.7023f,29.4964f,-5.2381f,  51.3799f,30.3736f,-4.5811f,  50.2083f,28.2656f,-3.7404f,  50.8841f,29.0958f,-3.1238f,  49.5019f,27.1845f,-2.2126f,  
	50.1757f,27.9667f,-1.6321f,  48.5950f,26.2716f,-0.6809f,  49.2668f,27.0054f,-0.1315f,  48.5950f,26.2716f,-0.6809f,  47.9239f,25.5231f,-1.2067f,  
	46.8331f,24.8408f,0.3251f,  46.1633f,24.1244f,-0.1608f,  44.9063f,23.6808f,1.3309f,  44.2367f,22.9932f,0.8721f,  42.8342f,22.7902f,2.2842f,  
	42.1648f,22.1260f,1.8396f,  40.6405f,22.1599f,3.1349f,  39.9706f,21.5136f,2.6962f,  38.3505f,21.7753f,3.8403f,  37.6814f,21.1389f,3.3970f,  
	35.9935f,21.6142f,4.3599f,  35.3244f,20.9820f,3.9081f,  33.5978f,21.6526f,4.6639f,  32.9306f,21.0167f,4.1979f,  31.1954f,21.8605f,4.7262f,  
	30.5300f,21.2145f,4.2435f,  28.8165f,22.2066f,4.5293f,  28.1550f,21.5439f,4.0285f,  26.4932f,22.6572f,4.0627f,  25.8356f,21.9727f,3.5441f,  
	24.2547f,23.1786f,3.3234f,  23.6026f,22.4671f,2.7883f,  22.1304f,23.7363f,2.3153f,  21.4832f,22.9944f,1.7655f,  20.1455f,24.2977f,1.0484f,  
	19.5045f,23.5238f,0.4874f,  18.3248f,24.8321f,-0.4606f,  17.6900f,24.0239f,-1.0295f,  16.6889f,25.3096f,-2.1900f,  16.0601f,24.4671f,-2.7636f,  
	15.2550f,25.7040f,-4.1142f,  14.6319f,24.8275f,-4.6888f,  14.0369f,25.9914f,-6.2032f,  13.4192f,25.0826f,-6.7755f,  13.0447f,26.1517f,-8.4243f,  
	12.4319f,25.2124f,-8.9909f,  12.2843f,26.1676f,-10.7424f,  11.6757f,25.2003f,-11.3006f,  11.7575f,26.0254f,-13.1212f,  11.1525f,25.0331f,-13.6690f,  
	11.4622f,25.7148f,-15.5241f,  10.8599f,24.7005f,-16.0596f,  11.3925f,25.2287f,-17.9142f,  10.7925f,24.1964f,-18.4353f,  11.5386f,24.5642f,-20.2547f,  
	10.9401f,23.5165f,-20.7614f,  11.8871f,23.7201f,-22.5118f,  11.2894f,22.6605f,-23.0032f,  12.4212f,22.6992f,-24.6524f,  11.8239f,21.6305f,-25.1288f,  
	13.1213f,21.5065f,-26.6466f,  12.5239f,20.4320f,-27.1079f,  13.9650f,20.1505f,-28.4666f,  13.3672f,19.0723f,-28.9136f,  14.9277f,18.6410f,-30.0888f,  
	14.3292f,17.5611f,-30.5225f,  15.9828f,16.9903f,-31.4930f,  15.3834f,15.9106f,-31.9141f,  17.1024f,15.2127f,-32.6626f,  16.5021f,14.1343f,-33.0725f,  
	18.2577f,13.3237f,-33.5855f,  17.6559f,12.2471f,-33.9854f,  19.4186f,11.3399f,-34.2536f,  18.8162f,10.2668f,-34.6442f,  20.5566f,9.2805f,-34.6629f,  
	19.9527f,8.2099f,-35.0457f,  21.6417f,7.1632f,-34.8142f,  21.0368f,6.0958f,-35.1899f,  22.6463f,5.0081f,-34.7119f,  22.0401f,3.9436f,-35.0812f,  
	23.5434f,2.8350f,-34.3647f,  22.9355f,1.7722f,-34.7286f,  24.3075f,0.6630f,-33.7855f,  23.6982f,-0.3977f,-34.1442f,  24.9161f,-1.4875f,-32.9904f,  
	24.3046f,-2.5476f,-33.3444f,  25.3483f,-3.5981f,-31.9993f,  24.7345f,-4.6577f,-32.3486f,  25.5867f,-5.6498f,-30.8348f,  24.9703f,-6.7088f,-31.1793f,  
	25.6171f,-7.6244f,-29.5224f,  24.9973f,-8.6838f,-29.8620f,  25.4282f,-9.5059f,-28.0899f,  24.8040f,-10.5661f,-28.4246f,  25.0123f,-11.2792f,-26.5671f,  
	24.3834f,-12.3394f,-26.8964f,  24.3661f,-12.9299f,-24.9851f,  23.7316f,-13.9898f,-25.3087f,  23.4896f,-14.4458f,-23.3759f,  22.8486f,-15.5047f,-23.6934f,  
	22.3869f,-15.8167f,-21.7721f,  21.7386f,-16.8735f,-22.0834f,  21.0657f,-17.0342f,-20.2063f,  20.4095f,-18.0875f,-20.5111f,  19.5378f,-18.0921f,-18.7105f,  
	18.8726f,-19.1407f,-19.0087f,  43.3191f,24.5659f,4.9697f,  41.0349f,24.1256f,5.6677f,  41.7010f,24.6625f,6.1421f,  39.3509f,24.4657f,6.6927f,  
	40.0160f,24.9705f,7.1756f,  37.6287f,25.0070f,7.5548f,  38.2929f,25.4846f,8.0524f,  35.8979f,25.7404f,8.2393f,  36.5612f,26.1959f,8.7574f,  
	34.1880f,26.6531f,8.7345f,  34.8504f,27.0924f,9.2787f,  32.5283f,27.7297f,9.0319f,  33.1899f,28.1587f,9.6074f,  30.9472f,28.9517f,9.1265f,  
	31.6081f,29.3765f,9.7377f,  29.4717f,30.2981f,9.0165f,  30.1320f,30.7249f,9.6675f,  28.1271f,31.7460f,8.7039f,  28.7870f,32.1811f,9.3979f,  
	26.9363f,33.2706f,8.1941f,  27.5959f,33.7199f,8.9337f,  25.9199f,34.8458f,7.4956f,  26.5793f,35.3151f,8.2826f,  25.0951f,36.4446f,6.6206f,  
	25.7544f,36.9394f,7.4559f,  24.4761f,38.0397f,5.5840f,  25.1354f,38.5649f,6.4678f,  24.0734f,39.6038f,4.4035f,  24.7330f,40.1640f,5.3350f,  
	23.8940f,41.1101f,3.0993f,  24.5539f,41.7092f,4.0770f,  23.9409f,42.5329f,1.6937f,  24.6012f,43.1740f,2.7154f,  24.2134f,43.8477f,0.2108f,  
	24.8742f,44.5335f,1.2734f,  24.7066f,45.0322f,-1.3240f,  25.3682f,45.7643f,-0.2243f,  25.4123f,46.0661f,-2.8846f,  26.0746f,46.8455f,-1.7521f,  
	26.3184f,46.9315f,-4.4441f,  26.9815f,47.7584f,-3.2839f,  27.4093f,47.6139f,-5.9759f,  28.0733f,48.4875f,-4.7933f,  28.6663f,48.1014f,-7.4538f,  
	29.3313f,49.0203f,-6.2547f,  30.0680f,48.3857f,-8.8526f,  30.7340f,49.3477f,-7.6430f,  31.5904f,48.4620f,-10.1482f,  32.2574f,49.4641f,-8.9344f,  
	33.2075f,48.3290f,-11.3185f,  33.8755f,49.3674f,-10.1069f,  34.8915f,47.9889f,-12.3435f,  35.5605f,49.0594f,-11.1403f,  36.6136f,47.4476f,-13.2056f,  
	37.2836f,48.5454f,-12.0171f,  38.3444f,46.7143f,-13.8901f,  39.0153f,47.8340f,-12.7222f,  40.0543f,45.8015f,-14.3853f,  40.7261f,46.9375f,-13.2435f,  
	41.7140f,44.7249f,-14.6827f,  42.3866f,45.8713f,-13.5721f,  43.2952f,43.5030f,-14.7773f,  43.9684f,44.6535f,-13.7024f,  44.7707f,42.1565f,-14.6673f,  
	45.4445f,43.3050f,-13.6322f,  46.1153f,40.7086f,-14.3547f,  46.7895f,41.8489f,-13.3627f,  47.3060f,39.1840f,-13.8448f,  47.9806f,40.3101f,-12.8984f,  
	48.3224f,37.6088f,-13.1464f,  48.9972f,38.7149f,-12.2473f,  49.1472f,36.0100f,-12.2714f,  49.8221f,37.0906f,-11.4207f,  49.7662f,34.4149f,-11.2348f,  
	50.4411f,35.4650f,-10.4325f,  50.1689f,32.8508f,-10.0543f,  50.8435f,33.8659f,-9.2997f,  50.3483f,31.3445f,-8.7501f,  51.0226f,32.3208f,-8.0418f,  
	50.3014f,29.9218f,-7.3445f,  50.9753f,30.8559f,-6.6801f,  50.0290f,28.6069f,-5.8616f,  50.7023f,29.4964f,-5.2381f,  49.5357f,27.4224f,-4.3268f,  
	50.2083f,28.2656f,-3.7404f,  48.8300f,26.3886f,-2.7662f,  49.5019f,27.1845f,-2.2126f,  47.9239f,25.5231f,-1.2067f,  48.5950f,26.2716f,-0.6809f,  
	47.9239f,25.5231f,-1.2067f,  47.2543f,24.7613f,-1.7119f,  46.1633f,24.1244f,-0.1608f,  45.4948f,23.3945f,-0.6306f,  44.2367f,22.9932f,0.8721f,  
	43.5687f,22.2908f,0.4240f,  42.1648f,22.1260f,1.8396f,  41.4966f,21.4465f,1.4044f,  39.9706f,21.5136f,2.6962f,  39.3033f,20.8495f,2.2622f,  
	37.6814f,21.1389f,3.3970f,  37.0143f,20.4849f,2.9598f,  35.3244f,20.9820f,3.9081f,  34.6590f,20.3303f,3.4606f,  32.9306f,21.0167f,4.1979f,  
	32.2668f,20.3608f,3.7368f,  30.5300f,21.2145f,4.2435f,  29.8698f,20.5475f,3.7662f,  28.1550f,21.5439f,4.0285f,  27.4983f,20.8603f,3.5345f,  
	25.8356f,21.9727f,3.5441f,  25.1841f,21.2668f,3.0337f,  23.6026f,22.4671f,2.7883f,  22.9558f,21.7349f,2.2627f,  21.4832f,22.9944f,1.7655f,  
	20.8424f,22.2337f,1.2277f,  19.5045f,23.5238f,0.4874f,  18.8698f,22.7315f,-0.0600f,  17.6900f,24.0239f,-1.0295f,  17.0612f,23.1993f,-1.5835f,  
	16.0601f,24.4671f,-2.7636f,  15.4370f,23.6099f,-3.3208f,  14.6319f,24.8275f,-4.6888f,  14.0143f,23.9386f,-5.2460f,  13.4192f,25.0826f,-6.7755f,  
	12.8067f,24.1633f,-7.3292f,  12.4319f,25.2124f,-8.9909f,  11.8238f,24.2646f,-9.5386f,  11.6757f,25.2003f,-11.3006f,  11.0713f,24.2268f,-11.8399f,  
	11.1525f,25.0331f,-13.6690f,  10.5509f,24.0364f,-14.1980f,  10.8599f,24.7005f,-16.0596f,  10.2610f,23.6843f,-16.5760f,  10.7925f,24.1964f,-18.4353f,  
	10.1952f,23.1632f,-18.9386f,  10.9401f,23.5165f,-20.7614f,  10.3438f,22.4697f,-21.2507f,  11.2894f,22.6605f,-23.0032f,  10.6935f,21.6028f,-23.4785f,  
	11.8239f,21.6305f,-25.1288f,  11.2279f,20.5651f,-25.5897f,  12.5239f,20.4320f,-27.1079f,  11.9275f,19.3613f,-27.5550f,  13.3672f,19.0723f,-28.9136f,  
	12.7700f,17.9983f,-29.3478f,  14.3292f,17.5611f,-30.5225f,  13.7310f,16.4860f,-30.9443f,  15.3834f,15.9106f,-31.9141f,  14.7841f,14.8354f,-32.3249f,  
	16.5021f,14.1343f,-33.0725f,  15.9012f,13.0600f,-33.4731f,  17.6559f,12.2471f,-33.9854f,  17.0541f,11.1754f,-34.3766f,  18.8162f,10.2668f,-34.6442f,  
	18.2126f,9.1968f,-35.0276f,  19.9527f,8.2099f,-35.0457f,  19.3478f,7.1426f,-35.4219f,  21.0368f,6.0958f,-35.1899f,  20.4305f,5.0308f,-35.5597f,  
	22.0401f,3.9436f,-35.0812f,  21.4319f,2.8800f,-35.4455f,  22.9355f,1.7722f,-34.7286f,  22.3257f,0.7104f,-35.0877f,  23.6982f,-0.3977f,-34.1442f,  
	23.0859f,-1.4592f,-34.4986f,  24.3046f,-2.5476f,-33.3444f,  23.6898f,-3.6089f,-33.6943f,  24.7345f,-4.6577f,-32.3486f,  24.1170f,-5.7188f,-32.6938f,  
	24.9703f,-6.7088f,-31.1793f,  24.3492f,-7.7707f,-31.5198f,  24.9973f,-8.6838f,-29.8620f,  24.3717f,-9.7471f,-30.1978f,  24.8040f,-10.5661f,-28.4246f,  
	24.1736f,-11.6300f,-28.7551f,  24.3834f,-12.3394f,-26.8964f,  23.7473f,-13.4038f,-27.2214f,  23.7316f,-13.9898f,-25.3087f,  23.0888f,-15.0541f,-25.6277f,  
	22.8486f,-15.5047f,-23.6934f,  22.1984f,-16.5680f,-24.0062f,  21.7386f,-16.8735f,-22.0834f,  21.0802f,-17.9345f,-22.3896f,  20.4095f,-18.0875f,-20.5111f,  
	19.7417f,-19.1450f,-20.8106f,  18.8726f,-19.1407f,-19.0087f,  18.1952f,-20.1921f,-19.3015f,  41.0349f,24.1256f,5.6677f,  38.6817f,23.9385f,6.2158f,  
	39.3509f,24.4657f,6.6927f,  36.9599f,24.5055f,7.0619f,  37.6287f,25.0070f,7.5548f,  35.2295f,25.2593f,7.7250f,  35.8979f,25.7404f,8.2393f,  
	33.5200f,26.1868f,8.1935f,  34.1880f,26.6531f,8.7345f,  31.8608f,27.2724f,8.4596f,  32.5283f,27.7297f,9.0319f,  30.2802f,28.4973f,8.5186f,  
	30.9472f,28.9517f,9.1265f,  28.8053f,29.8406f,8.3696f,  29.4717f,30.2981f,9.0165f,  27.4613f,31.2794f,8.0150f,  28.1271f,31.7460f,8.7039f,  
	26.2712f,32.7890f,7.4610f,  26.9363f,33.2706f,8.1941f,  25.2554f,34.3436f,6.7171f,  25.9199f,34.8458f,7.4956f,  24.4312f,35.9166f,5.7959f,  
	25.0951f,36.4446f,6.6206f,  23.8128f,37.4810f,4.7133f,  24.4761f,38.0397f,5.5840f,  23.4107f,39.0102f,3.4877f,  24.0734f,39.6038f,4.4035f,  
	23.2318f,40.4779f,2.1401f,  23.8940f,41.1101f,3.0993f,  23.2791f,41.8591f,0.6937f,  23.9409f,42.5329f,1.6937f,  23.5520f,43.1300f,-0.8270f,  
	24.2134f,43.8477f,0.2108f,  24.0456f,44.2690f,-2.3958f,  24.7066f,45.0322f,-1.3240f,  24.7516f,45.2566f,-3.9858f,  25.4123f,46.0661f,-2.8846f,  
	25.6578f,46.0758f,-5.5700f,  26.3184f,46.9315f,-4.4441f,  26.7488f,46.7127f,-7.1212f,  27.4093f,47.6139f,-5.9759f,  28.0059f,47.1564f,-8.6129f,  
	28.6663f,48.1014f,-7.4538f,  29.4076f,47.3992f,-10.0194f,  30.0680f,48.3857f,-8.8526f,  30.9299f,47.4370f,-11.3169f,  31.5904f,48.4620f,-10.1482f,  
	32.5467f,47.2692f,-12.4830f,  33.2075f,48.3290f,-11.3185f,  34.2305f,46.8986f,-13.4978f,  34.8915f,47.9889f,-12.3435f,  35.9523f,46.3316f,-14.3439f,  
	36.6136f,47.4476f,-13.2056f,  37.6827f,45.5779f,-15.0069f,  38.3444f,46.7143f,-13.8901f,  39.3922f,44.6503f,-15.4755f,  40.0543f,45.8015f,-14.3853f,  
	41.0514f,43.5648f,-15.7415f,  41.7140f,44.7249f,-14.6827f,  42.6319f,42.3399f,-15.8006f,  43.2952f,43.5030f,-14.7773f,  44.1068f,40.9965f,-15.6515f,  
	44.7707f,42.1565f,-14.6673f,  45.4508f,39.5578f,-15.2970f,  46.1153f,40.7086f,-14.3547f,  46.6409f,38.0482f,-14.7430f,  47.3060f,39.1840f,-13.8448f,  
	47.6568f,36.4936f,-13.9991f,  48.3224f,37.6088f,-13.1464f,  48.4810f,34.9206f,-13.0779f,  49.1472f,36.0100f,-12.2714f,  49.0994f,33.3561f,-11.9953f,  
	49.7662f,34.4149f,-11.2348f,  49.5015f,31.8270f,-10.7697f,  50.1689f,32.8508f,-10.0543f,  49.6804f,30.3592f,-9.4221f,  50.3483f,31.3445f,-8.7501f,  
	49.6330f,28.9781f,-7.9756f,  50.3014f,29.9218f,-7.3445f,  49.3602f,27.7071f,-6.4550f,  50.0290f,28.6069f,-5.8616f,  48.8665f,26.5681f,-4.8862f,  
	49.5357f,27.4224f,-4.3268f,  48.1606f,25.5805f,-3.2961f,  48.8300f,26.3886f,-2.7662f,  47.2543f,24.7613f,-1.7119f,  47.9239f,25.5231f,-1.2067f,  
	47.2543f,24.7613f,-1.7119f,  46.5868f,23.9875f,-2.1984f,  45.4948f,23.3945f,-0.6306f,  44.8283f,22.6516f,-1.0877f,  43.5687f,22.2908f,0.4240f,  
	42.9026f,21.5750f,-0.0129f,  41.4966f,21.4465f,1.4044f,  40.8315f,20.7510f,0.9749f,  39.3033f,20.8495f,2.2622f,  38.6385f,20.1696f,1.8351f,  
	37.0143f,20.4849f,2.9598f,  36.3510f,19.8130f,2.5269f,  34.6590f,20.3303f,3.4606f,  33.9972f,19.6599f,3.0179f,  32.2668f,20.3608f,3.7368f,  
	31.6082f,19.6849f,3.2804f,  29.8698f,20.5475f,3.7662f,  29.2145f,19.8605f,3.2948f,  27.4983f,20.8603f,3.5345f,  26.8478f,20.1559f,3.0474f,  
	25.1841f,21.2668f,3.0337f,  24.5380f,20.5406f,2.5315f,  22.9558f,21.7349f,2.2627f,  22.3154f,20.9840f,1.7477f,  20.8424f,22.2337f,1.2277f,  
	20.2080f,21.4543f,0.7018f,  18.8698f,22.7315f,-0.0600f,  18.2412f,21.9223f,-0.5941f,  17.0612f,23.1993f,-1.5835f,  16.4383f,22.3593f,-2.1227f,  
	15.4370f,23.6099f,-3.3208f,  14.8196f,22.7393f,-3.8620f,  14.0143f,23.9386f,-5.2460f,  13.4022f,23.0383f,-5.7859f,  12.8067f,24.1633f,-7.3292f,  
	12.1991f,23.2344f,-7.8653f,  11.8238f,24.2646f,-9.5386f,  11.2201f,23.3096f,-10.0683f,  11.0713f,24.2268f,-11.8399f,  10.4707f,23.2477f,-12.3613f,  
	10.5509f,24.0364f,-14.1980f,  9.9532f,23.0366f,-14.7084f,  10.2610f,23.6843f,-16.5760f,  9.6651f,22.6663f,-17.0748f,  10.1952f,23.1632f,-18.9386f,  
	9.6003f,22.1299f,-19.4248f,  10.3438f,22.4697f,-21.2507f,  9.7494f,21.4238f,-21.7239f,  10.6935f,21.6028f,-23.4785f,  10.0992f,20.5477f,-23.9381f,  
	11.2279f,20.5651f,-25.5897f,  10.6332f,19.5032f,-26.0361f,  11.9275f,19.3613f,-27.5550f,  11.3320f,18.2944f,-27.9890f,  12.7700f,17.9983f,-29.3478f,  
	12.1734f,16.9289f,-29.7696f,  13.7310f,16.4860f,-30.9443f,  13.1331f,15.4152f,-31.3553f,  14.7841f,14.8354f,-32.3249f,  14.1845f,13.7646f,-32.7257f,  
	15.9012f,13.0600f,-33.4731f,  15.3004f,11.9907f,-33.8647f,  17.0541f,11.1754f,-34.3766f,  16.4514f,10.1070f,-34.7603f,  18.2126f,9.1968f,-35.0276f,  
	17.6083f,8.1305f,-35.4040f,  19.3478f,7.1426f,-35.4219f,  18.7418f,6.0781f,-35.7918f,  20.4305f,5.0308f,-35.5597f,  19.8223f,3.9674f,-35.9241f,  
	21.4319f,2.8800f,-35.4455f,  20.8219f,1.8180f,-35.8047f,  22.3257f,0.7104f,-35.0877f,  21.7131f,-0.3517f,-35.4424f,  23.0859f,-1.4592f,-34.4986f,  
	22.4707f,-2.5214f,-34.8489f,  23.6898f,-3.6089f,-33.6943f,  23.0717f,-4.6712f,-34.0399f,  24.1170f,-5.7188f,-32.6938f,  23.4952f,-6.7822f,-33.0350f,  
	24.3492f,-7.7707f,-31.5198f,  23.7228f,-8.8361f,-31.8565f,  24.3717f,-9.7471f,-30.1978f,  23.7404f,-10.8137f,-30.5294f,  24.1736f,-11.6300f,-28.7551f,  
	23.5365f,-12.6979f,-29.0813f,  23.7473f,-13.4038f,-27.2214f,  23.1034f,-14.4726f,-27.5418f,  23.0888f,-15.0541f,-25.6277f,  22.4373f,-16.1230f,-25.9419f,  
	22.1984f,-16.5680f,-24.0062f,  21.5384f,-17.6357f,-24.3139f,  21.0802f,-17.9345f,-22.3896f,  20.4105f,-19.0000f,-22.6904f,  19.7417f,-19.1450f,-20.8106f,  
	19.0620f,-20.2058f,-21.1044f,  18.1952f,-20.1921f,-19.3015f,  17.5049f,-21.2461f,-19.5886f,  38.6817f,23.9385f,6.2158f,  36.2877f,23.9805f,6.5736f,  
	36.9599f,24.5055f,7.0619f,  34.5571f,24.7530f,7.2144f,  35.2295f,25.2593f,7.7250f,  32.8478f,25.6939f,7.6559f,  33.5200f,26.1868f,8.1935f,  
	31.1888f,26.7870f,7.8907f,  31.8608f,27.2724f,8.4596f,  29.6087f,28.0137f,7.9146f,  30.2802f,28.4973f,8.5186f,  28.1343f,29.3528f,7.7273f,  
	28.8053f,29.8406f,8.3696f,  26.7910f,30.7816f,7.3320f,  27.4613f,31.2794f,8.0150f,  25.6018f,32.2756f,6.7354f,  26.2712f,32.7890f,7.4610f,  
	24.5869f,33.8091f,5.9478f,  25.2554f,34.3436f,6.7171f,  23.7638f,35.3561f,4.9826f,  24.4312f,35.9166f,5.7959f,  23.1465f,36.8899f,3.8564f,  
	23.8128f,37.4810f,4.7133f,  22.7456f,38.3844f,2.5883f,  23.4107f,39.0102f,3.4877f,  22.5680f,39.8139f,1.2002f,  23.2318f,40.4779f,2.1401f,  
	22.6166f,41.1541f,-0.2842f,  23.2791f,41.8591f,0.6937f,  22.8907f,42.3820f,-1.8396f,  23.5520f,43.1300f,-0.8270f,  23.3856f,43.4766f,-3.4394f,  
	24.0456f,44.2690f,-2.3958f,  24.0928f,44.4191f,-5.0560f,  24.7516f,45.2566f,-3.9858f,  25.0002f,45.1935f,-6.6620f,  25.6578f,46.0758f,-5.5700f,  
	26.0922f,45.7864f,-8.2298f,  26.7488f,46.7127f,-7.1212f,  27.3503f,46.1878f,-9.7325f,  28.0059f,47.1564f,-8.6129f,  28.7528f,46.3907f,-11.1446f,  
	29.4076f,47.3992f,-10.0194f,  30.2758f,46.3918f,-12.4417f,  30.9299f,47.4370f,-11.3169f,  31.8932f,46.1909f,-13.6018f,  32.5467f,47.2692f,-12.4830f,  
	33.5773f,45.7915f,-14.6050f,  34.2305f,46.8986f,-13.4978f,  35.2994f,45.2005f,-15.4340f,  35.9523f,46.3316f,-14.3439f,  37.0299f,44.4279f,-16.0748f,  
	37.6827f,45.5779f,-15.0069f,  38.7392f,43.4870f,-16.5163f,  39.3922f,44.6503f,-15.4755f,  40.3982f,42.3939f,-16.7511f,  41.0514f,43.5648f,-15.7415f,  
	41.9783f,41.1673f,-16.7750f,  42.6319f,42.3399f,-15.8006f,  43.4527f,39.8281f,-16.5877f,  44.1068f,40.9965f,-15.6515f,  44.7960f,38.3993f,-16.1924f,  
	45.4508f,39.5578f,-15.2970f,  45.9852f,36.9054f,-15.5958f,  46.6409f,38.0482f,-14.7430f,  47.0001f,35.3718f,-14.8082f,  47.6568f,36.4936f,-13.9991f,  
	47.8232f,33.8249f,-13.8430f,  48.4810f,34.9206f,-13.0779f,  48.4405f,32.2911f,-12.7168f,  49.0994f,33.3561f,-11.9953f,  48.8414f,30.7966f,-11.4487f,  
	49.5015f,31.8270f,-10.7697f,  49.0190f,29.3670f,-10.0606f,  49.6804f,30.3592f,-9.4221f,  48.9704f,28.0268f,-8.5762f,  49.6330f,28.9781f,-7.9756f,  
	48.6963f,26.7989f,-7.0208f,  49.3602f,27.7071f,-6.4550f,  48.2014f,25.7043f,-5.4210f,  48.8665f,26.5681f,-4.8862f,  47.4942f,24.7618f,-3.8044f,  
	48.1606f,25.5805f,-3.2961f,  46.5868f,23.9875f,-2.1984f,  47.2543f,24.7613f,-1.7119f,  46.5868f,23.9875f,-2.1984f,  45.9220f,23.2022f,-2.6696f,  
	44.8283f,22.6516f,-1.0877f,  44.1644f,21.8970f,-1.5316f,  42.9026f,21.5750f,-0.0129f,  42.2398f,20.8451f,-0.4428f,  40.8315f,20.7510f,0.9749f,  
	40.1692f,20.0416f,0.5535f,  38.6385f,20.1696f,1.8351f,  37.9776f,19.4735f,1.4129f,  36.3510f,19.8130f,2.5269f,  35.6914f,19.1240f,2.0990f,  
	33.9972f,19.6599f,3.0179f,  33.3405f,18.9711f,2.5793f,  31.6082f,19.6849f,3.2804f,  30.9546f,18.9904f,2.8291f,  29.2145f,19.8605f,3.2948f,  
	28.5653f,19.1536f,2.8291f,  26.8478f,20.1559f,3.0474f,  26.2029f,19.4320f,2.5672f,  24.5380f,20.5406f,2.5315f,  23.8986f,19.7960f,2.0383f,  
	22.3154f,20.9840f,1.7477f,  21.6818f,20.2144f,1.2428f,  20.2080f,21.4543f,0.7018f,  19.5800f,20.6578f,0.1876f,  18.2412f,21.9223f,-0.5941f,  
	17.6189f,21.0972f,-1.1150f,  16.4383f,22.3593f,-2.1227f,  15.8215f,21.5053f,-2.6473f,  14.8196f,22.7393f,-3.8620f,  14.2082f,21.8565f,-4.3874f,  
	13.4022f,23.0383f,-5.7859f,  12.7954f,22.1273f,-6.3094f,  12.1991f,23.2344f,-7.8653f,  11.5964f,22.2973f,-8.3843f,  11.2201f,23.3096f,-10.0683f,  
	10.6207f,22.3480f,-10.5808f,  10.4707f,23.2477f,-12.3613f,  9.8743f,22.2645f,-12.8645f,  9.9532f,23.0366f,-14.7084f,  9.3588f,22.0341f,-15.2016f,  
	9.6651f,22.6663f,-17.0748f,  9.0719f,21.6470f,-17.5567f,  9.6003f,22.1299f,-19.4248f,  9.0078f,21.0968f,-19.8949f,  9.7494f,21.4238f,-21.7239f,  
	9.1571f,20.3799f,-22.1813f,  10.0992f,20.5477f,-23.9381f,  9.5065f,19.4954f,-24.3831f,  10.6332f,19.5032f,-26.0361f,  10.0396f,18.4444f,-26.4692f,  
	11.3320f,18.2944f,-27.9890f,  10.7373f,17.2319f,-28.4103f,  12.1734f,16.9289f,-29.7696f,  11.5773f,15.8636f,-30.1804f,  13.1331f,15.4152f,-31.3553f,  
	12.5352f,14.3489f,-31.7559f,  14.1845f,13.7646f,-32.7257f,  13.5852f,12.6987f,-33.1172f,  15.3004f,11.9907f,-33.8647f,  14.6990f,10.9250f,-34.2482f,  
	16.4514f,10.1070f,-34.7603f,  15.8481f,9.0427f,-35.1364f,  17.6083f,8.1305f,-35.4040f,  17.0032f,7.0675f,-35.7737f,  18.7418f,6.0781f,-35.7918f,  
	18.1343f,5.0156f,-36.1561f,  19.8223f,3.9674f,-35.9241f,  19.2127f,2.9060f,-36.2831f,  20.8219f,1.8180f,-35.8047f,  20.2095f,0.7562f,-36.1594f,  
	21.7131f,-0.3517f,-35.4424f,  21.0978f,-1.4138f,-35.7926f,  22.4707f,-2.5214f,-34.8489f,  21.8525f,-3.5840f,-35.1947f,  23.0717f,-4.6712f,-34.0399f,  
	22.4498f,-5.7353f,-34.3815f,  23.4952f,-6.7822f,-33.0350f,  22.8685f,-7.8488f,-33.3723f,  23.7228f,-8.8361f,-31.8565f,  23.0911f,-9.9045f,-32.1889f,  
	23.7404f,-10.8137f,-30.5294f,  23.1028f,-11.8841f,-30.8566f,  23.5365f,-12.6979f,-29.0813f,  22.8920f,-13.7701f,-29.4030f,  23.1034f,-14.4726f,-27.5418f,  
	22.4511f,-15.5459f,-27.8574f,  22.4373f,-16.1230f,-25.9419f,  21.7764f,-17.1963f,-26.2510f,  21.5384f,-17.6357f,-24.3139f,  20.8676f,-18.7082f,-24.6160f,  
	20.4105f,-19.0000f,-22.6904f,  19.7293f,-20.0692f,-22.9854f,  19.0620f,-20.2058f,-21.1044f,  18.3697f,-21.2698f,-21.3924f,  17.5049f,-21.2461f,-19.5886f,  
	16.8008f,-22.3025f,-19.8695f,  36.2877f,23.9805f,6.5736f,  33.8835f,24.2217f,6.7077f,  34.5571f,24.7530f,7.2144f,  32.1739f,25.1745f,7.1223f,  
	32.8478f,25.6939f,7.6559f,  30.5151f,26.2739f,7.3262f,  31.1888f,26.7870f,7.8907f,  28.9352f,27.5011f,7.3158f,  29.6087f,28.0137f,7.9146f,  
	27.4614f,28.8352f,7.0914f,  28.1343f,29.3528f,7.7273f,  26.1189f,30.2532f,6.6568f,  26.7910f,30.7816f,7.3320f,  24.9307f,31.7310f,6.0194f,  
	25.6018f,32.2756f,6.7354f,  23.9170f,33.2433f,5.1902f,  24.5869f,33.8091f,5.9478f,  23.0952f,34.7641f,4.1832f,  23.7638f,35.3561f,4.9826f,  
	22.4794f,36.2675f,3.0159f,  23.1465f,36.8899f,3.8564f,  22.0802f,37.7277f,1.7080f,  22.7456f,38.3844f,2.5883f,  21.9043f,39.1198f,0.2821f,  
	22.5680f,39.8139f,1.2002f,  21.9547f,40.4199f,-1.2376f,  22.6166f,41.1541f,-0.2842f,  22.2307f,41.6058f,-2.8249f,  22.8907f,42.3820f,-1.8396f,  
	22.7274f,42.6571f,-4.4528f,  23.3856f,43.4766f,-3.4394f,  23.4364f,43.5560f,-6.0933f,  24.0928f,44.4191f,-5.0560f,  24.3456f,44.2870f,-7.7185f,  
	25.0002f,45.1935f,-6.6620f,  25.4393f,44.8377f,-9.3005f,  26.0922f,45.7864f,-8.2298f,  26.6989f,45.1985f,-10.8122f,  27.3503f,46.1878f,-9.7325f,  
	28.1029f,45.3633f,-12.2278f,  28.7528f,46.3907f,-11.1446f,  29.6271f,45.3293f,-13.5230f,  30.2758f,46.3918f,-12.4417f,  31.2456f,45.0972f,-14.6757f,  
	31.8932f,46.1909f,-13.6018f,  32.9306f,44.6707f,-15.6662f,  33.5773f,45.7915f,-14.6050f,  34.6533f,44.0574f,-16.4776f,  35.2994f,45.2005f,-15.4340f,  
	36.3842f,43.2675f,-17.0958f,  37.0299f,44.4279f,-16.0748f,  38.0937f,42.3148f,-17.5104f,  38.7392f,43.4870f,-16.5163f,  39.7525f,41.2154f,-17.7143f,  
	40.3982f,42.3939f,-16.7511f,  41.3324f,39.9882f,-17.7039f,  41.9783f,41.1673f,-16.7750f,  42.8062f,38.6541f,-17.4795f,  43.4527f,39.8281f,-16.5877f,  
	44.1487f,37.2360f,-17.0449f,  44.7960f,38.3993f,-16.1924f,  45.3369f,35.7582f,-16.4075f,  45.9852f,36.9054f,-15.5958f,  46.3506f,34.2460f,-15.5783f,  
	47.0001f,35.3718f,-14.8082f,  47.1724f,32.7252f,-14.5714f,  47.8232f,33.8249f,-13.8430f,  47.7882f,31.2218f,-13.4040f,  48.4405f,32.2911f,-12.7168f,  
	48.1874f,29.7615f,-12.0962f,  48.8414f,30.7966f,-11.4487f,  48.3633f,28.3695f,-10.6702f,  49.0190f,29.3670f,-10.0606f,  48.3129f,27.0694f,-9.1506f,  
	48.9704f,28.0268f,-8.5762f,  48.0369f,25.8835f,-7.5632f,  48.6963f,26.7989f,-7.0208f,  47.5402f,24.8321f,-5.9354f,  48.2014f,25.7043f,-5.4210f,  
	46.8312f,23.9332f,-4.2948f,  47.4942f,24.7618f,-3.8044f,  45.9220f,23.2022f,-2.6696f,  46.5868f,23.9875f,-2.1984f,  45.9220f,23.2022f,-2.6696f,  
	45.2605f,22.4069f,-3.1255f,  44.1644f,21.8970f,-1.5316f,  43.5041f,21.1302f,-1.9668f,  42.2398f,20.8451f,-0.4428f,  41.5802f,20.1031f,-0.8633f,  
	40.1692f,20.0416f,0.5535f,  39.5109f,19.3179f,0.1375f,  37.9776f,19.4735f,1.4129f,  37.3206f,18.7622f,0.9959f,  35.6914f,19.1240f,2.0990f,  
	35.0371f,18.4181f,1.6749f,  33.3405f,18.9711f,2.5793f,  32.6891f,18.2649f,2.1454f,  30.9546f,18.9904f,2.8291f,  30.3072f,18.2770f,2.3826f,  
	28.5653f,19.1536f,2.8291f,  27.9219f,18.4280f,2.3693f,  26.2029f,19.4320f,2.5672f,  25.5646f,18.6901f,2.0948f,  23.8986f,19.7960f,2.0383f,  
	23.2659f,19.0329f,1.5539f,  21.6818f,20.2144f,1.2428f,  21.0546f,19.4277f,0.7483f,  19.5800f,20.6578f,0.1876f,  18.9584f,19.8451f,-0.3148f,  
	17.6189f,21.0972f,-1.1150f,  17.0028f,20.2576f,-1.6227f,  15.8215f,21.5053f,-2.6473f,  15.2109f,20.6383f,-3.1575f,  14.2082f,21.8565f,-4.3874f,  
	13.6023f,20.9622f,-4.8974f,  12.7954f,22.1273f,-6.3094f,  12.1937f,21.2072f,-6.8168f,  11.5964f,22.2973f,-8.3843f,  10.9982f,21.3526f,-8.8871f,  
	10.6207f,22.3480f,-10.5808f,  10.0258f,21.3811f,-11.0760f,  9.8743f,22.2645f,-12.8645f,  9.2816f,21.2775f,-13.3513f,  9.3588f,22.0341f,-15.2016f,  
	8.7675f,21.0293f,-15.6784f,  9.0719f,21.6470f,-17.5567f,  8.4814f,20.6271f,-18.0229f,  9.0078f,21.0968f,-19.8949f,  8.4176f,20.0647f,-20.3494f,  
	9.1571f,20.3799f,-22.1813f,  8.5666f,19.3381f,-22.6241f,  9.5065f,19.4954f,-24.3831f,  8.9152f,18.4457f,-24.8146f,  10.0396f,18.4444f,-26.4692f,  
	9.4472f,17.3896f,-26.8894f,  10.7373f,17.2319f,-28.4103f,  10.1433f,16.1731f,-28.8202f,  11.5773f,15.8636f,-30.1804f,  10.9815f,14.8029f,-30.5803f,  
	12.5352f,14.3489f,-31.7559f,  11.9378f,13.2874f,-32.1469f,  13.5852f,12.6987f,-33.1172f,  12.9855f,11.6366f,-33.5002f,  14.6990f,10.9250f,-34.2482f,  
	14.0973f,9.8636f,-34.6239f,  15.8481f,9.0427f,-35.1364f,  15.2443f,7.9821f,-35.5057f,  17.0032f,7.0675f,-35.7737f,  16.3967f,6.0069f,-36.1375f,  
	18.1343f,5.0156f,-36.1561f,  17.5256f,3.9558f,-36.5147f,  19.2127f,2.9060f,-36.2831f,  18.6010f,1.8454f,-36.6374f,  20.2095f,0.7562f,-36.1594f,  
	19.5948f,-0.3050f,-36.5093f,  21.0978f,-1.4138f,-35.7926f,  20.4801f,-2.4758f,-36.1383f,  21.8525f,-3.5840f,-35.1947f,  21.2308f,-4.6478f,-35.5364f,  
	22.4498f,-5.7353f,-34.3815f,  21.8232f,-6.8021f,-34.7191f,  22.8685f,-7.8488f,-33.3723f,  22.2369f,-8.9180f,-33.7053f,  23.0911f,-9.9045f,-32.1889f,  
	22.4536f,-10.9764f,-32.5170f,  23.1028f,-11.8841f,-30.8566f,  22.4583f,-12.9586f,-31.1793f,  22.8920f,-13.7701f,-29.4030f,  22.2397f,-14.8467f,-29.7198f,  
	22.4511f,-15.5459f,-27.8574f,  21.7899f,-16.6237f,-28.1678f,  21.7764f,-17.1963f,-26.2510f,  21.1051f,-18.2746f,-26.5545f,  20.8676f,-18.7082f,-24.6160f,  
	20.1857f,-19.7847f,-24.9123f,  19.7293f,-20.0692f,-22.9854f,  19.0357f,-21.1422f,-23.2744f,  18.3697f,-21.2698f,-21.3924f,  17.6637f,-22.3369f,-21.6740f,  
	16.8008f,-22.3025f,-19.8695f,  16.0817f,-23.3614f,-20.1438f,  33.8835f,24.2217f,6.7077f,  31.4987f,24.6299f,6.5932f,  32.1739f,25.1745f,7.1223f,  
	29.8397f,25.7343f,6.7666f,  30.5151f,26.2739f,7.3262f,  28.2601f,26.9610f,6.7227f,  28.9352f,27.5011f,7.3158f,  26.7868f,28.2891f,6.4622f,  
	27.4614f,28.8352f,7.0914f,  25.4451f,29.6957f,5.9896f,  26.1189f,30.2532f,6.6568f,  24.2580f,31.1569f,5.3130f,  24.9307f,31.7310f,6.0194f,  
	23.2457f,32.6476f,4.4440f,  23.9170f,33.2433f,5.1902f,  22.4256f,34.1423f,3.3974f,  23.0952f,34.7641f,4.1832f,  21.8117f,35.6155f,2.1912f,  
	22.4794f,36.2675f,3.0159f,  21.4145f,37.0419f,0.8459f,  22.0802f,37.7277f,1.7080f,  21.2408f,38.3971f,-0.6154f,  21.9043f,39.1198f,0.2821f,  
	21.2936f,39.6580f,-2.1676f,  21.9547f,40.4199f,-1.2376f,  21.5720f,40.8029f,-3.7843f,  22.2307f,41.6058f,-2.8249f,  22.0712f,41.8124f,-5.4378f,  
	22.7274f,42.6571f,-4.4528f,  22.7827f,42.6690f,-7.0997f,  23.4364f,43.5560f,-6.0933f,  23.6942f,43.3582f,-8.7417f,  24.3456f,44.2870f,-7.7185f,  
	24.7903f,43.8681f,-10.3357f,  25.4393f,44.8377f,-9.3005f,  26.0521f,44.1901f,-11.8543f,  26.6989f,45.1985f,-10.8122f,  27.4581f,44.3186f,-13.2716f,  
	28.1029f,45.3633f,-12.2278f,  28.9842f,44.2515f,-14.5634f,  29.6271f,45.3293f,-13.5230f,  30.6043f,43.9898f,-15.7075f,  31.2456f,45.0972f,-14.6757f,  
	32.2906f,43.5380f,-16.6844f,  32.9306f,44.6707f,-15.6662f,  34.0144f,42.9040f,-17.4773f,  34.6533f,44.0574f,-16.4776f,  35.7460f,42.0984f,-18.0727f,  
	36.3842f,43.2675f,-17.0958f,  37.4560f,41.1352f,-18.4605f,  38.0937f,42.3148f,-17.5104f,  39.1150f,40.0308f,-18.6338f,  39.7525f,41.2154f,-17.7143f,  
	40.6947f,38.8041f,-18.5899f,  41.3324f,39.9882f,-17.7039f,  42.1679f,37.4760f,-18.3294f,  42.8062f,38.6541f,-17.4795f,  43.5096f,36.0694f,-17.8568f,  
	44.1487f,37.2360f,-17.0449f,  44.6968f,34.6082f,-17.1802f,  45.3369f,35.7582f,-16.4075f,  45.7091f,33.1175f,-16.3112f,  46.3506f,34.2460f,-15.5783f,  
	46.5292f,31.6228f,-15.2646f,  47.1724f,32.7252f,-14.5714f,  47.1431f,30.1496f,-14.0584f,  47.7882f,31.2218f,-13.4040f,  47.5402f,28.7232f,-12.7131f,  
	48.1874f,29.7615f,-12.0962f,  47.7139f,27.3680f,-11.2519f,  48.3633f,28.3695f,-10.6702f,  47.6611f,26.1071f,-9.6996f,  48.3129f,27.0694f,-9.1506f,  
	47.3827f,24.9621f,-8.0829f,  48.0369f,25.8835f,-7.5632f,  46.8835f,23.9527f,-6.4294f,  47.5402f,24.8321f,-5.9354f,  46.1721f,23.0961f,-4.7675f,  
	46.8312f,23.9332f,-4.2948f,  45.2605f,22.4069f,-3.1255f,  45.9220f,23.2022f,-2.6696f,  45.2605f,22.4069f,-3.1255f,  44.6027f,21.6013f,-3.5708f,  
	43.5041f,21.1302f,-1.9668f,  42.8474f,20.3530f,-2.3910f,  41.5802f,20.1031f,-0.8633f,  40.9249f,19.3486f,-1.2774f,  39.5109f,19.3179f,0.1375f,  
	38.8569f,18.5806f,-0.2726f,  37.3206f,18.7622f,0.9959f,  36.6690f,18.0355f,0.5828f,  35.0371f,18.4181f,1.6749f,  34.3881f,17.6963f,1.2553f,  
	32.6891f,18.2649f,2.1454f,  32.0437f,17.5410f,1.7155f,  30.3072f,18.2770f,2.3826f,  29.6656f,17.5458f,1.9411f,  27.9219f,18.4280f,2.3693f,  
	27.2851f,17.6851f,1.9160f,  25.5646f,18.6901f,2.0948f,  24.9332f,17.9300f,1.6296f,  23.2659f,19.0329f,1.5539f,  22.6398f,18.2528f,1.0783f,  
	21.0546f,19.4277f,0.7483f,  20.4340f,18.6246f,0.2640f,  18.9584f,19.8451f,-0.3148f,  18.3433f,19.0176f,-0.8056f,  17.0028f,20.2576f,-1.6227f,  
	16.3932f,19.4045f,-2.1173f,  15.2109f,20.6383f,-3.1575f,  14.6060f,19.7590f,-3.6536f,  13.6023f,20.9622f,-4.8974f,  13.0018f,20.0579f,-5.3927f,  
	12.1937f,21.2072f,-6.8168f,  11.5969f,20.2786f,-7.3090f,  10.9982f,21.3526f,-8.8871f,  10.4050f,20.4016f,-9.3734f,  10.0258f,21.3811f,-11.0760f,  
	9.4349f,20.4096f,-11.5554f,  9.2816f,21.2775f,-13.3513f,  8.6923f,20.2873f,-13.8221f,  8.7675f,21.0293f,-15.6784f,  8.1792f,20.0230f,-16.1399f,  
	8.4814f,20.6271f,-18.0229f,  7.8935f,19.6074f,-18.4737f,  8.4176f,20.0647f,-20.3494f,  7.8296f,19.0340f,-20.7893f,  8.5666f,19.3381f,-22.6241f,  
	7.9778f,18.2983f,-23.0535f,  8.9152f,18.4457f,-24.8146f,  8.3253f,17.3995f,-25.2332f,  9.4472f,17.3896f,-26.8894f,  8.8557f,16.3382f,-27.2981f,  
	10.1433f,16.1731f,-28.8202f,  9.5500f,15.1187f,-29.2190f,  10.9815f,14.8029f,-30.5803f,  10.3863f,13.7466f,-30.9704f,  11.9378f,13.2874f,-32.1469f,  
	11.3402f,12.2299f,-32.5290f,  12.9855f,11.6366f,-33.5002f,  12.3857f,10.5791f,-33.8750f,  14.0973f,9.8636f,-34.6239f,  13.4951f,8.8062f,-34.9923f,  
	15.2443f,7.9821f,-35.5057f,  14.6394f,6.9242f,-35.8687f,  16.3967f,6.0069f,-36.1375f,  15.7894f,4.9494f,-36.4954f,  17.5256f,3.9558f,-36.5147f,  
	16.9151f,2.8973f,-36.8682f,  18.6010f,1.8454f,-36.6374f,  17.9874f,0.7859f,-36.9868f,  19.5948f,-0.3050f,-36.5093f,  18.9780f,-1.3655f,-36.8547f,  
	20.4801f,-2.4758f,-36.1383f,  19.8593f,-3.5384f,-36.4798f,  21.2308f,-4.6478f,-35.5364f,  20.6050f,-5.7139f,-35.8741f,  21.8232f,-6.8021f,-34.7191f,  
	21.1924f,-7.8711f,-35.0525f,  22.2369f,-8.9180f,-33.7053f,  21.6001f,-9.9903f,-34.0339f,  22.4536f,-10.9764f,-32.5170f,  21.8098f,-12.0521f,-32.8406f,  
	22.4583f,-12.9586f,-31.1793f,  21.8065f,-14.0374f,-31.4972f,  22.2397f,-14.8467f,-29.7198f,  21.5789f,-15.9279f,-30.0315f,  21.7899f,-16.6237f,-28.1678f,  
	21.1190f,-17.7067f,-28.4727f,  21.1051f,-18.2746f,-26.5545f,  20.4233f,-19.3572f,-26.8522f,  20.1857f,-19.7847f,-24.9123f,  19.4917f,-20.8655f,-25.2025f,  
	19.0357f,-21.1422f,-23.2744f,  18.3289f,-22.2189f,-23.5570f,  17.6637f,-22.3369f,-21.6740f,  16.9431f,-23.4071f,-21.9488f,  16.0817f,-23.3614f,-20.1438f,  
	15.3474f,-24.4218f,-20.4113f,  31.4987f,24.6299f,6.5932f,  29.1660f,25.1676f,6.2126f,  29.8397f,25.7343f,6.7666f,  27.5864f,26.3929f,6.1363f,  
	28.2601f,26.9610f,6.7227f,  26.1136f,27.7142f,5.8413f,  26.7868f,28.2891f,6.4622f,  24.7727f,29.1089f,5.3325f,  25.4451f,29.6957f,5.9896f,  
	23.5867f,30.5532f,4.6187f,  24.2580f,31.1569f,5.3130f,  22.5759f,32.0224f,3.7121f,  23.2457f,32.6476f,4.4440f,  21.7575f,33.4913f,2.6283f,  
	22.4256f,34.1423f,3.3974f,  21.1456f,34.9347f,1.3856f,  21.8117f,35.6155f,2.1912f,  20.7507f,36.3281f,0.0055f,  21.4145f,37.0419f,0.8459f,  
	20.5794f,37.6474f,-1.4885f,  21.2408f,38.3971f,-0.6154f,  20.6348f,38.8703f,-3.0708f,  21.2936f,39.6580f,-2.1676f,  20.9158f,39.9757f,-4.7144f,  
	21.5720f,40.8029f,-3.7843f,  21.4177f,40.9447f,-6.3910f,  22.0712f,41.8124f,-5.4378f,  22.1319f,41.7607f,-8.0721f,  22.7827f,42.6690f,-7.0997f,  
	23.0462f,42.4099f,-9.7289f,  23.6942f,43.3582f,-8.7417f,  24.1449f,42.8810f,-11.3329f,  24.7903f,43.8681f,-10.3357f,  25.4092f,43.1661f,-12.8569f,  
	26.0521f,44.1901f,-11.8543f,  26.8174f,43.2602f,-14.2746f,  27.4581f,44.3186f,-13.2716f,  28.3456f,43.1617f,-15.5619f,  28.9842f,44.2515f,-14.5634f,  
	29.9675f,42.8723f,-16.6967f,  30.6043f,43.9898f,-15.7075f,  31.6555f,42.3970f,-17.6596f,  32.2906f,43.5380f,-16.6844f,  33.3805f,41.7439f,-18.4341f,  
	34.0144f,42.9040f,-17.4773f,  35.1131f,40.9242f,-19.0070f,  35.7460f,42.0984f,-18.0727f,  36.8237f,39.9519f,-19.3684f,  37.4560f,41.1352f,-18.4605f,  
	38.4829f,38.8436f,-19.5122f,  39.1150f,40.0308f,-18.6338f,  40.0625f,37.6183f,-19.4360f,  40.6947f,38.8041f,-18.5899f,  41.5353f,36.2970f,-19.1409f,  
	42.1679f,37.4760f,-18.3294f,  42.8762f,34.9023f,-18.6322f,  43.5096f,36.0694f,-17.8568f,  44.0622f,33.4580f,-17.9184f,  44.6968f,34.6082f,-17.1802f,  
	45.0730f,31.9888f,-17.0118f,  45.7091f,33.1175f,-16.3112f,  45.8914f,30.5199f,-15.9279f,  46.5292f,31.6228f,-15.2646f,  46.5033f,29.0765f,-14.6853f,  
	47.1431f,30.1496f,-14.0584f,  46.8982f,27.6832f,-13.3052f,  47.5402f,28.7232f,-12.7131f,  47.0695f,26.3638f,-11.8112f,  47.7139f,27.3680f,-11.2519f,  
	47.0141f,25.1409f,-10.2289f,  47.6611f,26.1071f,-9.6996f,  46.7331f,24.0356f,-8.5853f,  47.3827f,24.9621f,-8.0829f,  46.2312f,23.0665f,-6.9086f,  
	46.8835f,23.9527f,-6.4294f,  45.5170f,22.2505f,-5.2276f,  46.1721f,23.0961f,-4.7675f,  44.6027f,21.6013f,-3.5708f,  45.2605f,22.4069f,-3.1255f,  
	44.6027f,21.6013f,-3.5708f,  43.9491f,20.7871f,-4.0034f,  42.8474f,20.3530f,-2.3910f,  42.1952f,19.5652f,-2.8077f,  40.9249f,19.3486f,-1.2774f,  
	40.2740f,18.5824f,-1.6847f,  38.8569f,18.5806f,-0.2726f,  38.2081f,17.8298f,-0.6785f,  36.6690f,18.0355f,0.5828f,  36.0226f,17.2945f,0.1741f,  
	34.3881f,17.6963f,1.2553f,  33.7451f,16.9583f,0.8392f,  32.0437f,17.5410f,1.7155f,  31.4042f,16.8006f,1.2899f,  29.6656f,17.5458f,1.9411f,  
	29.0307f,16.7982f,1.5051f,  27.2851f,17.6851f,1.9160f,  26.6552f,16.9246f,1.4688f,  24.9332f,17.9300f,1.6296f,  24.3084f,17.1533f,1.1721f,  
	22.6398f,18.2528f,1.0783f,  22.0204f,17.4564f,0.6116f,  20.4340f,18.6246f,0.2640f,  19.8200f,17.8065f,-0.2100f,  18.3433f,19.0176f,-0.8056f,  
	17.7348f,18.1761f,-1.2846f,  16.3932f,19.4045f,-2.1173f,  15.7895f,18.5384f,-2.5993f,  14.6060f,19.7590f,-3.6536f,  14.0069f,18.8692f,-4.1362f,  
	13.0018f,20.0579f,-5.3927f,  12.4065f,19.1444f,-5.8737f,  11.5969f,20.2786f,-7.3090f,  11.0053f,19.3427f,-7.7858f,  10.4050f,20.4016f,-9.3734f,  
	9.8160f,19.4450f,-9.8447f,  9.4349f,20.4096f,-11.5554f,  8.8477f,19.4337f,-12.0195f,  8.6923f,20.2873f,-13.8221f,  8.1063f,19.2946f,-14.2781f,  
	8.1792f,20.0230f,-16.1399f,  7.5938f,19.0159f,-16.5863f,  7.8935f,19.6074f,-18.4737f,  7.3082f,18.5883f,-18.9101f,  7.8296f,19.0340f,-20.7893f,  
	7.2435f,18.0046f,-21.2159f,  7.9778f,18.2983f,-23.0535f,  7.3907f,17.2613f,-23.4698f,  8.3253f,17.3995f,-25.2332f,  7.7365f,16.3562f,-25.6400f,  
	8.8557f,16.3382f,-27.2981f,  8.2651f,15.2910f,-27.6954f,  9.5500f,15.1187f,-29.2190f,  8.9574f,14.0686f,-29.6078f,  10.3863f,13.7466f,-30.9704f,  
	9.7912f,12.6945f,-31.3513f,  11.3402f,12.2299f,-32.5290f,  10.7428f,11.1770f,-32.9027f,  12.3857f,10.5791f,-33.8750f,  11.7858f,9.5257f,-34.2424f,  
	13.4951f,8.8062f,-34.9923f,  12.8923f,7.7519f,-35.3543f,  14.6394f,6.9242f,-35.8687f,  14.0339f,5.8699f,-36.2255f,  15.7894f,4.9494f,-36.4954f,  
	15.1806f,3.8938f,-36.8480f,  16.9151f,2.8973f,-36.8682f,  16.3029f,1.8404f,-37.2169f,  17.9874f,0.7859f,-36.9868f,  17.3719f,-0.2722f,-37.3315f,  
	18.9780f,-1.3655f,-36.8547f,  18.3585f,-2.4261f,-37.1957f,  19.8593f,-3.5384f,-36.4798f,  19.2347f,-4.6028f,-36.8173f,  20.6050f,-5.7139f,-35.8741f,  
	19.9754f,-6.7816f,-36.2075f,  21.1924f,-7.8711f,-35.0525f,  20.5568f,-8.9426f,-35.3815f,  21.6001f,-9.9903f,-34.0339f,  20.9574f,-11.0659f,-34.3582f,  
	21.8098f,-12.0521f,-32.8406f,  21.1592f,-13.1317f,-33.1594f,  21.8065f,-14.0374f,-31.4972f,  21.1469f,-15.1206f,-31.8102f,  21.5789f,-15.9279f,-30.0315f,  
	20.9090f,-17.0141f,-30.3377f,  21.1190f,-17.7067f,-28.4727f,  20.4380f,-18.7940f,-28.7717f,  20.4233f,-19.3572f,-26.8522f,  19.7299f,-20.4445f,-27.1437f,  
	19.4917f,-20.8655f,-25.2025f,  18.7850f,-21.9505f,-25.4863f,  18.3289f,-22.2189f,-23.5570f,  17.6079f,-23.2993f,-23.8326f,  16.9431f,-23.4071f,-21.9488f,  
	16.2075f,-24.4797f,-22.2165f,  15.3474f,-24.4218f,-20.4113f,  14.5964f,-25.4837f,-20.6714f,  29.1660f,25.1676f,6.2126f,  26.9132f,25.7990f,5.5569f,  
	27.5864f,26.3929f,6.1363f,  25.4408f,27.1128f,5.2285f,  26.1136f,27.7142f,5.8413f,  24.1007f,28.4952f,4.6850f,  24.7727f,29.1089f,5.3325f,  
	22.9159f,29.9223f,3.9356f,  23.5867f,30.5532f,4.6187f,  21.9066f,31.3699f,2.9932f,  22.5759f,32.0224f,3.7121f,  21.0901f,32.8131f,1.8740f,  
	21.7575f,33.4913f,2.6283f,  20.4804f,34.2272f,0.5971f,  21.1456f,34.9347f,1.3856f,  20.0880f,35.5881f,-0.8157f,  20.7507f,36.3281f,0.0055f,  
	19.9194f,36.8725f,-2.3403f,  20.5794f,37.6474f,-1.4885f,  19.9777f,38.0583f,-3.9504f,  20.6348f,38.8703f,-3.0708f,  20.2617f,39.1254f,-5.6185f,  
	20.9158f,39.9757f,-4.7144f,  20.7668f,40.0554f,-7.3162f,  21.4177f,40.9447f,-6.3910f,  21.4841f,40.8324f,-9.0144f,  22.1319f,41.7607f,-8.0721f,  
	22.4015f,41.4431f,-10.6840f,  23.0462f,42.4099f,-9.7289f,  23.5032f,41.8772f,-12.2964f,  24.1449f,42.8810f,-11.3329f,  24.7704f,42.1271f,-13.8241f,  
	25.4092f,43.1661f,-12.8569f,  26.1815f,42.1886f,-15.2409f,  26.8174f,43.2602f,-14.2746f,  27.7122f,42.0606f,-16.5226f,  28.3456f,43.1617f,-15.5619f,  
	29.3363f,41.7453f,-17.6472f,  29.9675f,42.8723f,-16.6967f,  31.0262f,41.2481f,-18.5955f,  31.6555f,42.3970f,-17.6596f,  32.7528f,40.5776f,-19.3513f,  
	33.3805f,41.7439f,-18.4341f,  34.4867f,39.7452f,-19.9017f,  35.1131f,40.9242f,-19.0070f,  36.1981f,38.7651f,-20.2372f,  36.8237f,39.9519f,-19.3684f,  
	37.8578f,37.6541f,-20.3520f,  38.4829f,38.8436f,-19.5122f,  39.4374f,36.4312f,-20.2443f,  40.0625f,37.6183f,-19.4360f,  40.9098f,35.1173f,-19.9159f,  
	41.5353f,36.2970f,-19.1409f,  42.2499f,33.7350f,-19.3723f,  42.8762f,34.9023f,-18.6322f,  43.4347f,32.3079f,-18.6230f,  44.0622f,33.4580f,-17.9184f,  
	44.4440f,30.8603f,-17.6806f,  45.0730f,31.9888f,-17.0118f,  45.2605f,29.4171f,-16.5614f,  45.8914f,30.5199f,-15.9279f,  45.8702f,28.0030f,-15.2845f,  
	46.5033f,29.0765f,-14.6853f,  46.2627f,26.6421f,-13.8717f,  46.8982f,27.6832f,-13.3052f,  46.4312f,25.3577f,-12.3471f,  47.0695f,26.3638f,-11.8112f,  
	46.3729f,24.1719f,-10.7370f,  47.0141f,25.1409f,-10.2289f,  46.0889f,23.1048f,-9.0689f,  46.7331f,24.0356f,-8.5853f,  45.5838f,22.1748f,-7.3712f,  
	46.2312f,23.0665f,-6.9086f,  44.8665f,21.3978f,-5.6730f,  45.5170f,22.2505f,-5.2276f,  43.9491f,20.7871f,-4.0034f,  44.6027f,21.6013f,-3.5708f,  
	43.9491f,20.7871f,-4.0034f,  43.3000f,19.9640f,-4.4270f,  42.1952f,19.5652f,-2.8077f,  41.5475f,18.7676f,-3.2163f,  40.2740f,18.5824f,-1.6847f,  
	39.6283f,17.8044f,-2.0873f,  38.2081f,17.8298f,-0.6785f,  37.5647f,17.0662f,-1.0797f,  36.0226f,17.2945f,0.1741f,  35.3822f,16.5388f,-0.2313f,  
	33.7451f,16.9583f,0.8392f,  33.1079f,16.2050f,0.4270f,  31.4042f,16.8006f,1.2899f,  30.7713f,16.0448f,0.8691f,  29.0307f,16.7982f,1.5051f,  
	28.4025f,16.0339f,1.0741f,  26.6552f,16.9246f,1.4688f,  26.0320f,16.1480f,1.0280f,  24.3084f,17.1533f,1.1721f,  23.6904f,16.3606f,0.7221f,  
	22.0204f,17.4564f,0.6116f,  21.4077f,16.6451f,0.1539f,  19.8200f,17.8065f,-0.2100f,  19.2127f,16.9742f,-0.6737f,  17.7348f,18.1761f,-1.2846f,  
	17.1324f,17.3212f,-1.7524f,  15.7895f,18.5384f,-2.5993f,  15.1918f,17.6612f,-3.0689f,  14.0069f,18.8692f,-4.1362f,  13.4132f,17.9692f,-4.6057f,  
	12.4065f,19.1444f,-5.8737f,  11.8167f,18.2226f,-6.3404f,  11.0053f,19.3427f,-7.7858f,  10.4183f,18.4004f,-8.2484f,  9.8160f,19.4450f,-9.8447f,  
	9.2309f,18.4831f,-10.3015f,  8.8477f,19.4337f,-12.0195f,  8.2640f,18.4545f,-12.4693f,  8.1063f,19.2946f,-14.2781f,  7.5235f,18.3002f,-14.7194f,  
	7.5938f,19.0159f,-16.5863f,  7.0112f,18.0086f,-17.0186f,  7.3082f,18.5883f,-18.9101f,  6.7249f,17.5698f,-19.3333f,  7.2435f,18.0046f,-21.2159f,  
	6.6593f,16.9775f,-21.6295f,  7.3907f,17.2613f,-23.4698f,  6.8048f,16.2268f,-23.8744f,  7.7365f,16.3562f,-25.6400f,  7.1490f,15.3169f,-26.0354f,  
	8.2651f,15.2910f,-27.6954f,  7.6754f,14.2478f,-28.0825f,  8.9574f,14.0686f,-29.6078f,  8.3651f,13.0226f,-29.9872f,  9.7912f,12.6945f,-31.3513f,  
	9.1965f,11.6470f,-31.7236f,  10.7428f,11.1770f,-32.9027f,  10.1454f,10.1284f,-33.2687f,  11.7858f,9.5257f,-34.2424f,  11.1853f,8.4758f,-34.6030f,  
	12.8923f,7.7519f,-35.3543f,  12.2890f,6.7016f,-35.7098f,  14.0339f,5.8699f,-36.2255f,  13.4271f,4.8180f,-36.5770f,  15.1806f,3.8938f,-36.8480f,  
	14.5704f,2.8404f,-37.1955f,  16.3029f,1.8404f,-37.2169f,  15.6894f,0.7855f,-37.5606f,  17.3719f,-0.2722f,-37.3315f,  16.7542f,-1.3298f,-37.6718f,  
	18.3585f,-2.4261f,-37.1957f,  17.7357f,-3.4878f,-37.5328f,  19.2347f,-4.6028f,-36.8173f,  18.6069f,-5.6682f,-37.1506f,  19.9754f,-6.7816f,-36.2075f,  
	19.3415f,-7.8513f,-36.5367f,  20.5568f,-8.9426f,-35.3815f,  19.9159f,-10.0171f,-35.7062f,  20.9574f,-11.0659f,-34.3582f,  20.3086f,-12.1453f,-34.6778f,  
	21.1592f,-13.1317f,-33.1594f,  20.5013f,-14.2156f,-33.4734f,  21.1469f,-15.1206f,-31.8102f,  20.4788f,-16.2087f,-32.1177f,  20.9090f,-17.0141f,-30.3377f,  
	20.2297f,-18.1047f,-30.6381f,  20.4380f,-18.7940f,-28.7717f,  19.7460f,-19.8863f,-29.0647f,  19.7299f,-20.4445f,-27.1437f,  19.0242f,-21.5363f,-27.4288f,  
	18.7850f,-21.9505f,-25.4863f,  18.0646f,-23.0397f,-25.7629f,  17.6079f,-23.2993f,-23.8326f,  16.8721f,-24.3828f,-24.1009f,  16.2075f,-24.4797f,-22.2165f,  
	15.4555f,-25.5547f,-22.4767f,  14.5964f,-25.4837f,-20.6714f,  13.8278f,-26.5468f,-20.9237f,  26.9132f,25.7990f,5.5569f,  24.7709f,26.4853f,4.6253f,  
	25.4408f,27.1128f,5.2285f,  23.4315f,27.8549f,4.0486f,  24.1007f,28.4952f,4.6850f,  22.2478f,29.2648f,3.2656f,  22.9159f,29.9223f,3.9356f,  
	21.2400f,30.6909f,2.2895f,  21.9066f,31.3699f,2.9932f,  20.4254f,32.1088f,1.1370f,  21.0901f,32.8131f,1.8740f,  19.8180f,33.4942f,-0.1720f,  
	20.4804f,34.2272f,0.5971f,  19.4280f,34.8235f,-1.6153f,  20.0880f,35.5881f,-0.8157f,  19.2622f,36.0739f,-3.1681f,  19.9194f,36.8725f,-2.3403f,  
	19.3234f,37.2240f,-4.8038f,  19.9777f,38.0583f,-3.9504f,  19.6106f,38.2542f,-6.4945f,  20.2617f,39.1254f,-5.6185f,  20.1189f,39.1467f,-8.2112f,  
	20.7668f,40.0554f,-7.3162f,  20.8395f,39.8863f,-9.9246f,  21.4841f,40.8324f,-9.0144f,  21.7602f,40.4605f,-11.6054f,  22.4015f,41.4431f,-10.6840f,  
	22.8651f,40.8592f,-13.2247f,  23.5032f,41.8772f,-12.2964f,  24.1354f,41.0758f,-14.7549f,  24.7704f,42.1271f,-13.8241f,  25.5493f,41.1066f,-16.1698f,  
	26.1815f,42.1886f,-15.2409f,  27.0827f,40.9509f,-17.4452f,  27.7122f,42.0606f,-16.5226f,  28.7092f,40.6115f,-18.5593f,  29.3363f,41.7453f,-17.6472f,  
	30.4012f,40.0941f,-19.4930f,  31.0262f,41.2481f,-18.5955f,  32.1296f,39.4077f,-20.2302f,  32.7528f,40.5776f,-19.3513f,  33.8648f,38.5639f,-20.7586f,  
	34.4867f,39.7452f,-19.9017f,  35.5772f,37.5772f,-21.0688f,  36.1981f,38.7651f,-20.2372f,  37.2375f,36.4645f,-21.1558f,  37.8578f,37.6541f,-20.3520f,  
	38.8172f,35.2449f,-21.0179f,  39.4374f,36.4312f,-20.2443f,  40.2893f,33.9391f,-20.6576f,  40.9098f,35.1173f,-19.9159f,  41.6287f,32.5695f,-20.0810f,  
	42.2499f,33.7350f,-19.3723f,  42.8124f,31.1596f,-19.2979f,  43.4347f,32.3079f,-18.6230f,  43.8202f,29.7335f,-18.3218f,  44.4440f,30.8603f,-17.6806f,  
	44.6348f,28.3156f,-17.1694f,  45.2605f,29.4171f,-16.5614f,  45.2422f,26.9302f,-15.8603f,  45.8702f,28.0030f,-15.2845f,  45.6322f,25.6009f,-14.4170f,  
	46.2627f,26.6421f,-13.8717f,  45.7980f,24.3505f,-12.8643f,  46.4312f,25.3577f,-12.3471f,  45.7368f,23.2004f,-11.2285f,  46.3729f,24.1719f,-10.7370f,  
	45.4496f,22.1703f,-9.5378f,  46.0889f,23.1048f,-9.0689f,  44.9413f,21.2777f,-7.8211f,  45.5838f,22.1748f,-7.3712f,  44.2207f,20.5381f,-6.1077f,  
	44.8665f,21.3978f,-5.6730f,  43.3000f,19.9640f,-4.4270f,  43.9491f,20.7871f,-4.0034f,  43.3000f,19.9640f,-4.4270f,  42.6558f,19.1329f,-4.8412f,  
	41.5475f,18.7676f,-3.2163f,  40.9051f,17.9600f,-3.6195f,  39.6283f,17.8044f,-2.0873f,  38.9879f,17.0155f,-2.4846f,  37.5647f,17.0662f,-1.0797f,  
	36.9271f,16.2897f,-1.4777f,  35.3822f,16.5388f,-0.2313f,  34.7476f,15.7693f,-0.6330f,  33.1079f,16.2050f,0.4270f,  32.4773f,15.4375f,0.0189f,  
	30.7713f,16.0448f,0.8691f,  30.1451f,15.2732f,0.4523f,  28.4025f,16.0339f,1.0741f,  27.7811f,15.2542f,0.6485f,  26.0320f,16.1480f,1.0280f,  
	25.4156f,15.3559f,0.5935f,  23.6904f,16.3606f,0.7221f,  23.0792f,15.5531f,0.2798f,  21.4077f,16.6451f,0.1539f,  20.8018f,15.8195f,-0.2950f,  
	19.2127f,16.9742f,-0.6737f,  18.6118f,16.1283f,-1.1275f,  17.1324f,17.3212f,-1.7524f,  16.5362f,16.4548f,-2.2091f,  15.1918f,17.6612f,-3.0689f,  
	14.5996f,16.7734f,-3.5266f,  13.4132f,17.9692f,-4.6057f,  12.8251f,17.0603f,-5.0620f,  11.8167f,18.2226f,-6.3404f,  11.2316f,17.2935f,-6.7939f,  
	10.4183f,18.4004f,-8.2484f,  9.8355f,17.4518f,-8.6974f,  9.2309f,18.4831f,-10.3015f,  8.6497f,17.5171f,-10.7446f,  8.2640f,18.4545f,-12.4693f,  
	7.6839f,17.4727f,-12.9051f,  7.5235f,18.3002f,-14.7194f,  6.9437f,17.3049f,-15.1471f,  7.0112f,18.0086f,-17.0186f,  6.4309f,17.0012f,-17.4380f,  
	6.7249f,17.5698f,-19.3333f,  6.1438f,16.5528f,-19.7438f,  6.6593f,16.9775f,-21.6295f,  6.0766f,15.9522f,-22.0315f,  6.8048f,16.2268f,-23.8744f,  
	6.2205f,15.1959f,-24.2675f,  7.1490f,15.3169f,-26.0354f,  6.5623f,14.2811f,-26.4205f,  7.6754f,14.2478f,-28.0825f,  7.0863f,13.2085f,-28.4601f,  
	8.3651f,13.0226f,-29.9872f,  7.7735f,11.9812f,-30.3578f,  9.1965f,11.6470f,-31.7236f,  8.6020f,10.6038f,-32.0880f,  10.1454f,10.1284f,-33.2687f,  
	9.5477f,9.0835f,-33.6278f,  11.1853f,8.4758f,-34.6030f,  10.5847f,7.4301f,-34.9571f,  12.2890f,6.7016f,-35.7098f,  11.6848f,5.6540f,-36.0598f,  
	13.4271f,4.8180f,-36.5770f,  12.8194f,3.7687f,-36.9232f,  14.5704f,2.8404f,-37.1955f,  13.9591f,1.7894f,-37.5381f,  15.6894f,0.7855f,-37.5606f,  
	15.0739f,-0.2684f,-37.9000f,  16.7542f,-1.3298f,-37.6718f,  16.1337f,-2.3880f,-38.0082f,  17.7357f,-3.4878f,-37.5328f,  17.1101f,-4.5499f,-37.8656f,  
	18.6069f,-5.6682f,-37.1506f,  17.9752f,-6.7351f,-37.4797f,  19.3415f,-7.8513f,-36.5367f,  18.7029f,-8.9236f,-36.8617f,  19.9159f,-10.0171f,-35.7062f,  
	19.2694f,-11.0949f,-36.0265f,  20.3086f,-12.1453f,-34.6778f,  19.6531f,-13.2286f,-34.9928f,  20.5013f,-14.2156f,-33.4734f,  19.8356f,-15.3043f,-33.7821f,  
	20.4788f,-16.2087f,-32.1177f,  19.8019f,-17.3012f,-32.4195f,  20.2297f,-18.1047f,-30.6381f,  19.5400f,-19.2005f,-30.9327f,  19.7460f,-19.8863f,-29.0647f,  
	19.0423f,-20.9835f,-29.3513f,  19.0242f,-21.5363f,-27.4288f,  18.3054f,-22.6328f,-27.7067f,  18.0646f,-23.0397f,-25.7629f,  17.3298f,-24.1326f,-26.0322f,  
	16.8721f,-24.3828f,-24.1009f,  16.1204f,-25.4694f,-24.3615f,  15.4555f,-25.5547f,-22.4767f,  14.6862f,-26.6318f,-22.7287f,  13.8278f,-26.5468f,-20.9237f,  
	13.0409f,-27.6104f,-21.1677f,  24.7709f,26.4853f,4.6253f,  22.7648f,27.1893f,3.4233f,  23.4315f,27.8549f,4.0486f,  21.5821f,28.5819f,2.6082f,  
	22.2478f,29.2648f,3.2656f,  20.5758f,29.9867f,1.6002f,  21.2400f,30.6909f,2.2895f,  19.7631f,31.3797f,0.4165f,  20.4254f,32.1088f,1.1370f,  
	19.1579f,32.7371f,-0.9226f,  19.8180f,33.4942f,-0.1720f,  18.7705f,34.0356f,-2.3943f,  19.4280f,34.8235f,-1.6153f,  18.6076f,35.2531f,-3.9733f,  
	19.2622f,36.0739f,-3.1681f,  18.6719f,36.3687f,-5.6326f,  19.3234f,37.2240f,-4.8038f,  18.9623f,37.3633f,-7.3439f,  19.6106f,38.2542f,-6.4945f,  
	19.4740f,38.2200f,-9.0778f,  20.1189f,39.1467f,-8.2112f,  20.1980f,38.9239f,-10.8047f,  20.8395f,39.8863f,-9.9246f,  21.1221f,39.4632f,-12.4951f,  
	21.7602f,40.4605f,-11.6054f,  22.2304f,39.8285f,-14.1200f,  22.8651f,40.8592f,-13.2247f,  23.5040f,40.0137f,-15.6516f,  24.1354f,41.0758f,-14.7549f,  
	24.9211f,40.0155f,-17.0637f,  25.5493f,41.1066f,-16.1698f,  26.4573f,39.8339f,-18.3322f,  27.0827f,40.9509f,-17.4452f,  28.0866f,39.4721f,-19.4353f,  
	28.7092f,40.6115f,-18.5593f,  29.7808f,38.9362f,-20.3542f,  30.4012f,40.0941f,-19.4930f,  31.5112f,38.2353f,-21.0731f,  32.1296f,39.4077f,-20.2302f,  
	33.2480f,37.3816f,-21.5798f,  33.8648f,38.5639f,-20.7586f,  34.9615f,36.3895f,-21.8656f,  35.5772f,37.5772f,-21.0688f,  36.6225f,35.2761f,-21.9255f,  
	37.2375f,36.4645f,-21.1558f,  38.2025f,34.0604f,-21.7586f,  38.8172f,35.2449f,-21.0179f,  39.6744f,32.7632f,-21.3677f,  40.2893f,33.9391f,-20.6576f,  
	41.0132f,31.4068f,-20.7596f,  41.6287f,32.5695f,-20.0810f,  42.1958f,30.0142f,-19.9445f,  42.8124f,31.1596f,-19.2979f,  43.2021f,28.6094f,-18.9365f,  
	43.8202f,29.7335f,-18.3218f,  44.0148f,27.2164f,-17.7528f,  44.6348f,28.3156f,-17.1694f,  44.6200f,25.8590f,-16.4137f,  45.2422f,26.9302f,-15.8603f,  
	45.0074f,24.5605f,-14.9421f,  45.6322f,25.6009f,-14.4170f,  45.1704f,23.3430f,-13.3630f,  45.7980f,24.3505f,-12.8643f,  45.1061f,22.2274f,-11.7037f,  
	45.7368f,23.2004f,-11.2285f,  44.8156f,21.2328f,-9.9924f,  45.4496f,22.1703f,-9.5378f,  44.3040f,20.3761f,-8.2585f,  44.9413f,21.2777f,-7.8211f,  
	43.5799f,19.6722f,-6.5316f,  44.2207f,20.5381f,-6.1077f,  42.6558f,19.1329f,-4.8412f,  43.3000f,19.9640f,-4.4270f,  42.6558f,19.1329f,-4.8412f,  
	42.0167f,18.2937f,-5.2489f,  40.9051f,17.9600f,-3.6195f,  40.2679f,17.1431f,-4.0167f,  38.9879f,17.0155f,-2.4846f,  38.3533f,16.2152f,-2.8786f,  
	36.9271f,16.2897f,-1.4777f,  36.2953f,15.5009f,-1.8719f,  34.7476f,15.7693f,-0.6330f,  34.1195f,14.9869f,-1.0310f,  32.4773f,15.4375f,0.0189f,  
	31.8533f,14.6554f,-0.3859f,  30.1451f,15.2732f,0.4523f,  29.5256f,14.4870f,0.0400f,  27.7811f,15.2542f,0.6485f,  27.1663f,14.4596f,0.2281f,  
	25.4156f,15.3559f,0.5935f,  24.8059f,14.5494f,0.1655f,  23.0792f,15.5531f,0.2798f,  22.4748f,14.7315f,-0.1550f,  20.8018f,15.8195f,-0.2950f,  
	20.2023f,14.9801f,-0.7352f,  18.6118f,16.1283f,-1.1275f,  18.0170f,15.2705f,-1.5714f,  16.5362f,16.4548f,-2.2091f,  15.9456f,15.5773f,-2.6550f,  
	14.5996f,16.7734f,-3.5266f,  14.0133f,15.8759f,-3.9724f,  12.8251f,17.0603f,-5.0620f,  12.2420f,16.1435f,-5.5061f,  11.2316f,17.2935f,-6.7939f,  
	10.6510f,16.3575f,-7.2347f,  9.8355f,17.4518f,-8.6974f,  9.2566f,16.4984f,-9.1334f,  8.6497f,17.5171f,-10.7446f,  8.0721f,16.5475f,-11.1744f,  
	7.6839f,17.4727f,-12.9051f,  7.1070f,16.4891f,-13.3278f,  6.9437f,17.3049f,-15.1471f,  6.3665f,16.3086f,-15.5623f,  6.4309f,17.0012f,-17.4380f,  
	5.8530f,15.9946f,-17.8448f,  6.1438f,16.5528f,-19.7438f,  5.5644f,15.5370f,-20.1428f,  6.0766f,15.9522f,-22.0315f,  5.4957f,14.9301f,-22.4219f,  
	6.2205f,15.1959f,-24.2675f,  5.6372f,14.1680f,-24.6504f,  6.5623f,14.2811f,-26.4205f,  5.9766f,13.2491f,-26.7960f,  7.0863f,13.2085f,-28.4601f,  
	6.4980f,12.1737f,-28.8287f,  7.7735f,11.9812f,-30.3578f,  7.1822f,10.9440f,-30.7203f,  8.6020f,10.6038f,-32.0880f,  8.0075f,9.5646f,-32.4452f,  
	9.5477f,9.0835f,-33.6278f,  8.9500f,8.0430f,-33.9801f,  10.5847f,7.4301f,-34.9571f,  9.9833f,6.3875f,-35.3054f,  11.6848f,5.6540f,-36.0598f,  
	11.0798f,4.6095f,-36.4045f,  12.8194f,3.7687f,-36.9232f,  12.2108f,2.7224f,-37.2644f,  13.9591f,1.7894f,-37.5381f,  13.3463f,0.7400f,-37.8763f,  
	15.0739f,-0.2684f,-37.9000f,  14.4560f,-1.3222f,-38.2354f,  16.1337f,-2.3880f,-38.0082f,  15.5107f,-3.4459f,-38.3403f,  17.1101f,-4.5499f,-37.8656f,  
	16.4811f,-5.6130f,-38.1944f,  17.9752f,-6.7351f,-37.4797f,  17.3394f,-7.8040f,-37.8047f,  18.7029f,-8.9236f,-36.8617f,  18.0594f,-9.9986f,-37.1824f,  
	19.2694f,-11.0949f,-36.0265f,  18.6169f,-12.1763f,-36.3423f,  19.6531f,-13.2286f,-34.9928f,  18.9906f,-14.3163f,-35.3026f,  19.8356f,-15.3043f,-33.7821f,  
	19.1619f,-16.3971f,-34.0854f,  19.8019f,-17.3012f,-32.4195f,  19.1152f,-18.3988f,-32.7157f,  19.5400f,-19.2005f,-30.9327f,  18.8392f,-20.3014f,-31.2209f,  
	19.0423f,-20.9835f,-29.3513f,  18.3262f,-22.0856f,-29.6308f,  18.3054f,-22.6328f,-27.7067f,  17.5728f,-23.7336f,-27.9774f,  17.3298f,-24.1326f,-26.0322f,  
	16.5797f,-25.2294f,-26.2938f,  16.1204f,-25.4694f,-24.3615f,  15.3518f,-26.5591f,-24.6138f,  14.6862f,-26.6318f,-22.7287f,  13.8988f,-27.7104f,-22.9722f,  
	13.0409f,-27.6104f,-21.1677f,  12.2350f,-28.6737f,-21.4033f,  22.7648f,27.1893f,3.4233f,  20.9210f,27.8749f,1.9654f,  21.5821f,28.5819f,2.6082f,  
	19.9160f,29.2588f,0.9275f,  20.5758f,29.9867f,1.6002f,  19.1050f,30.6274f,-0.2854f,  19.7631f,31.3797f,0.4165f,  18.5019f,31.9576f,-1.6526f,  
	19.1579f,32.7371f,-0.9226f,  18.1169f,33.2264f,-3.1506f,  18.7705f,34.0356f,-2.3943f,  17.9567f,34.4121f,-4.7539f,  18.6076f,35.2531f,-3.9733f,  
	18.0240f,35.4946f,-6.4349f,  18.6719f,36.3687f,-5.6326f,  18.3176f,36.4552f,-8.1650f,  18.9623f,37.3633f,-7.3439f,  18.8325f,37.2775f,-9.9145f,  
	19.4740f,38.2200f,-9.0778f,  19.5599f,37.9474f,-11.6535f,  20.1980f,38.9239f,-10.8047f,  20.4873f,38.4536f,-13.3522f,  21.1221f,39.4632f,-12.4951f,  
	21.5990f,38.7873f,-14.9817f,  22.2304f,39.8285f,-14.1200f,  22.8758f,38.9428f,-16.5139f,  23.5040f,40.0137f,-15.6516f,  24.2959f,38.9175f,-17.9227f,  
	24.9211f,40.0155f,-17.0637f,  25.8351f,38.7118f,-19.1839f,  26.4573f,39.8339f,-18.3322f,  27.4669f,38.3291f,-20.2761f,  28.0866f,39.4721f,-19.4353f,  
	29.1635f,37.7762f,-21.1804f,  29.7808f,38.9362f,-20.3542f,  30.8959f,37.0623f,-21.8815f,  31.5112f,38.2353f,-21.0731f,  32.6343f,36.1998f,-22.3673f,  
	33.2480f,37.3816f,-21.5798f,  34.3491f,35.2034f,-22.6296f,  34.9615f,36.3895f,-21.8656f,  36.0109f,34.0901f,-22.6638f,  36.6225f,35.2761f,-21.9255f,  
	37.5912f,32.8790f,-22.4693f,  38.2025f,34.0604f,-21.7586f,  39.0631f,31.5908f,-22.0495f,  39.6744f,32.7632f,-21.3677f,  40.4014f,30.2476f,-21.4116f,  
	41.0132f,31.4068f,-20.7596f,  41.5831f,28.8723f,-20.5665f,  42.1958f,30.0142f,-19.9445f,  42.5880f,27.4885f,-19.5286f,  43.2021f,28.6094f,-18.9365f,  
	43.3990f,26.1198f,-18.3157f,  44.0148f,27.2164f,-17.7528f,  44.0021f,24.7897f,-16.9485f,  44.6200f,25.8590f,-16.4137f,  44.3871f,23.5209f,-15.4505f,  
	45.0074f,24.5605f,-14.9421f,  44.5473f,22.3352f,-13.8472f,  45.1704f,23.3430f,-13.3630f,  44.4801f,21.2527f,-12.1662f,  45.1061f,22.2274f,-11.7037f,  
	44.1865f,20.2921f,-10.4361f,  44.8156f,21.2328f,-9.9924f,  43.6716f,19.4698f,-8.6866f,  44.3040f,20.3761f,-8.2585f,  42.9442f,18.7998f,-6.9476f,  
	43.5799f,19.6722f,-6.5316f,  42.0167f,18.2937f,-5.2489f,  42.6558f,19.1329f,-4.8412f,  42.0167f,18.2937f,-5.2489f,  41.3829f,17.4469f,-5.6497f,  
	40.2679f,17.1431f,-4.0167f,  39.6364f,16.3167f,-4.4101f,  38.3533f,16.2152f,-2.8786f,  37.7244f,15.4044f,-3.2685f,  36.2953f,15.5009f,-1.8719f,  
	35.6697f,14.7006f,-2.2627f,  34.1195f,14.9869f,-1.0310f,  33.4978f,14.1911f,-1.4263f,  31.8533f,14.6554f,-0.3859f,  31.2358f,13.8597f,-0.7869f,  
	29.5256f,14.4870f,0.0400f,  28.9127f,13.6868f,-0.3679f,  27.1663f,14.4596f,0.2281f,  26.5583f,13.6510f,-0.1868f,  24.8059f,14.5494f,0.1655f,  
	24.2030f,13.7290f,-0.2560f,  22.4748f,14.7315f,-0.1550f,  21.8768f,13.8961f,-0.5823f,  20.2023f,14.9801f,-0.7352f,  19.6091f,14.1287f,-1.1668f,  
	18.0170f,15.2705f,-1.5714f,  17.4281f,14.4013f,-2.0057f,  15.9456f,15.5773f,-2.6550f,  15.3611f,14.6895f,-3.0902f,  14.0133f,15.8759f,-3.9724f,  
	13.4322f,14.9698f,-4.4069f,  12.2420f,16.1435f,-5.5061f,  11.6635f,15.2188f,-5.9384f,  10.6510f,16.3575f,-7.2347f,  10.0745f,15.4158f,-7.6634f,  
	9.2566f,16.4984f,-9.1334f,  8.6817f,15.5405f,-9.5569f,  8.0721f,16.5475f,-11.1744f,  7.4981f,15.5753f,-11.5916f,  7.1070f,16.4891f,-13.3278f,  
	6.5327f,15.5038f,-13.7383f,  6.3665f,16.3086f,-15.5623f,  5.7918f,15.3123f,-15.9652f,  5.8530f,15.9946f,-17.8448f,  5.2768f,14.9885f,-18.2405f,  
	5.5644f,15.5370f,-20.1428f,  4.9870f,14.5240f,-20.5303f,  5.4957f,14.9301f,-22.4219f,  4.9159f,13.9104f,-22.8023f,  5.6372f,14.1680f,-24.6504f,  
	5.0551f,13.1437f,-25.0235f,  5.9766f,13.2491f,-26.7960f,  5.3918f,12.2213f,-27.1625f,  6.4980f,12.1737f,-28.8287f,  5.9101f,11.1430f,-29.1893f,  
	7.1822f,10.9440f,-30.7203f,  6.5910f,9.9108f,-31.0756f,  8.0075f,9.5646f,-32.4452f,  7.4131f,8.5298f,-32.7957f,  8.9500f,8.0430f,-33.9801f,  
	8.3519f,7.0060f,-34.3266f,  9.9833f,6.3875f,-35.3054f,  9.3815f,5.3484f,-35.6483f,  11.0798f,4.6095f,-36.4045f,  10.4743f,3.5685f,-36.7441f,  
	12.2108f,2.7224f,-37.2644f,  11.6010f,1.6782f,-37.6011f,  13.3463f,0.7400f,-37.8763f,  12.7314f,-0.3087f,-38.2105f,  14.4560f,-1.3222f,-38.2354f,  
	13.8361f,-2.3752f,-38.5666f,  15.5107f,-3.4459f,-38.3403f,  14.8849f,-4.5043f,-38.6686f,  16.4811f,-5.6130f,-38.1944f,  15.8486f,-6.6775f,-38.5193f,  
	17.3394f,-7.8040f,-37.8047f,  16.6992f,-8.8751f,-38.1257f,  18.0594f,-9.9986f,-37.1824f,  17.4104f,-11.0768f,-37.4988f,  18.6169f,-12.1763f,-36.3423f,  
	17.9581f,-13.2617f,-36.6531f,  18.9906f,-14.3163f,-35.3026f,  18.3206f,-15.4080f,-35.6073f,  19.1619f,-16.3971f,-34.0854f,  18.4791f,-17.4950f,-34.3832f,  
	19.1152f,-18.3988f,-32.7157f,  18.4182f,-19.5017f,-33.0057f,  18.8392f,-20.3014f,-31.2209f,  18.1267f,-21.4073f,-31.5023f,  18.3262f,-22.0856f,-29.6308f,  
	17.5970f,-23.1924f,-29.9032f,  17.5728f,-23.7336f,-27.9774f,  16.8255f,-24.8387f,-28.2403f,  16.5797f,-25.2294f,-26.2938f,  15.8132f,-26.3299f,-26.5469f,  
	15.3518f,-26.5591f,-24.6138f,  14.5655f,-27.6511f,-24.8574f,  13.8988f,-27.7104f,-22.9722f,  13.0927f,-28.7897f,-23.2069f,  12.2350f,-28.6737f,-21.4033f,  
	11.4089f,-29.7362f,-21.6299f,  20.9210f,27.8749f,1.9654f,  19.2602f,28.5086f,0.2706f,  19.9160f,29.2588f,0.9275f,  18.4509f,29.8535f,-0.9697f,  
	19.1050f,30.6274f,-0.2854f,  17.8497f,31.1571f,-2.3630f,  18.5019f,31.9576f,-1.6526f,  17.4670f,32.3971f,-3.8856f,  18.1169f,33.2264f,-3.1506f,  
	17.3094f,33.5522f,-5.5112f,  17.9567f,34.4121f,-4.7539f,  17.3795f,34.6028f,-7.2122f,  18.0240f,35.4946f,-6.4349f,  17.6761f,35.5309f,-8.9595f,  
	18.3176f,36.4552f,-8.1650f,  18.1942f,36.3204f,-10.7231f,  18.8325f,37.2775f,-9.9145f,  18.9249f,36.9581f,-12.4728f,  19.5599f,37.9474f,-11.6535f,  
	19.8557f,37.4329f,-14.1788f,  20.4873f,38.4536f,-13.3522f,  20.9706f,37.7367f,-15.8118f,  21.5990f,38.7873f,-14.9817f,  22.2507f,37.8643f,-17.3439f,  
	22.8758f,38.9428f,-16.5139f,  23.6739f,37.8136f,-18.7488f,  24.2959f,38.9175f,-17.9227f,  25.2160f,37.5853f,-20.0026f,  25.8351f,38.7118f,-19.1839f,  
	26.8505f,37.1835f,-21.0838f,  27.4669f,38.3291f,-20.2761f,  28.5495f,36.6149f,-21.9739f,  29.1635f,37.7762f,-21.1804f,  30.2840f,35.8894f,-22.6576f,  
	30.8959f,37.0623f,-21.8815f,  32.0241f,35.0193f,-23.1233f,  32.6343f,36.1998f,-22.3673f,  33.7402f,34.0195f,-23.3630f,  34.3491f,35.2034f,-22.6296f,  
	35.4030f,32.9071f,-23.3726f,  36.0109f,34.0901f,-22.6638f,  36.9838f,31.7012f,-23.1519f,  37.5912f,32.8790f,-22.4693f,  38.4558f,30.4224f,-22.7046f,  
	39.0631f,31.5908f,-22.0495f,  39.7937f,29.0925f,-22.0386f,  40.4014f,30.2476f,-21.4116f,  40.9746f,27.7344f,-21.1651f,  41.5831f,28.8723f,-20.5665f,  
	41.9783f,26.3712f,-20.0991f,  42.5880f,27.4885f,-19.5286f,  42.7877f,25.0263f,-18.8588f,  43.3990f,26.1198f,-18.3157f,  43.3889f,23.7227f,-17.4655f,  
	44.0021f,24.7897f,-16.9485f,  43.7716f,22.4827f,-15.9429f,  44.3871f,23.5209f,-15.4505f,  43.9292f,21.3276f,-14.3173f,  44.5473f,22.3352f,-13.8472f,  
	43.8591f,20.2770f,-12.6162f,  44.4801f,21.2527f,-12.1662f,  43.5625f,19.3490f,-10.8690f,  44.1865f,20.2921f,-10.4361f,  43.0444f,18.5594f,-9.1054f,  
	43.6716f,19.4698f,-8.6866f,  42.3137f,17.9217f,-7.3557f,  42.9442f,18.7998f,-6.9476f,  41.3829f,17.4469f,-5.6497f,  42.0167f,18.2937f,-5.2489f,  
	41.3829f,17.4469f,-5.6497f,  40.7545f,16.5924f,-6.0459f,  39.6364f,16.3167f,-4.4101f,  39.0105f,15.4815f,-4.7989f,  37.7244f,15.4044f,-3.2685f,  
	37.1015f,14.5835f,-3.6550f,  35.6697f,14.7006f,-2.2627f,  35.0504f,13.8883f,-2.6509f,  33.4978f,14.1911f,-1.4263f,  32.8824f,13.3830f,-1.8182f,  
	31.2358f,13.8597f,-0.7869f,  30.6248f,13.0509f,-1.1843f,  28.9127f,13.6868f,-0.3679f,  28.3064f,12.8733f,-0.7713f,  26.5583f,13.6510f,-0.1868f,  
	25.9570f,12.8290f,-0.5964f,  24.2030f,13.7290f,-0.2560f,  23.6065f,12.8950f,-0.6714f,  21.8768f,13.8961f,-0.5823f,  21.2850f,13.0488f,-1.0021f,  
	19.6091f,14.1287f,-1.1668f,  19.0217f,13.2657f,-1.5899f,  17.4281f,14.4013f,-2.0057f,  16.8452f,13.5214f,-2.4306f,  15.3611f,14.6895f,-3.0902f,  
	14.7817f,13.7926f,-3.5153f,  13.4322f,14.9698f,-4.4069f,  12.8557f,14.0553f,-4.8308f,  11.6635f,15.2188f,-5.9384f,  11.0893f,14.2878f,-6.3595f,  
	10.0745f,15.4158f,-7.6634f,  9.5021f,14.4688f,-8.0803f,  8.6817f,15.5405f,-9.5569f,  8.1104f,14.5791f,-9.9685f,  7.4981f,15.5753f,-11.5916f,  
	6.9268f,14.6006f,-11.9973f,  6.5327f,15.5038f,-13.7383f,  5.9612f,14.5177f,-14.1371f,  5.7918f,15.3123f,-15.9652f,  5.2189f,14.3159f,-16.3573f,  
	5.2768f,14.9885f,-18.2405f,  4.7030f,13.9846f,-18.6249f,  4.9870f,14.5240f,-20.5303f,  4.4107f,13.5127f,-20.9079f,  4.9159f,13.9104f,-22.8023f,  
	4.3374f,12.8940f,-23.1729f,  5.0551f,13.1437f,-25.0235f,  4.4739f,12.1232f,-25.3877f,  5.3918f,12.2213f,-27.1625f,  4.8075f,11.1974f,-27.5209f,  
	5.9101f,11.1430f,-29.1893f,  5.3224f,10.1163f,-29.5424f,  6.5910f,9.9108f,-31.0756f,  6.0001f,8.8822f,-31.4241f,  7.4131f,8.5298f,-32.7957f,  
	6.8184f,7.4988f,-33.1402f,  8.3519f,7.0060f,-34.3266f,  7.7534f,5.9728f,-34.6676f,  9.3815f,5.3484f,-35.6483f,  8.7793f,4.3132f,-35.9862f,  
	10.4743f,3.5685f,-36.7441f,  9.8678f,2.5300f,-37.0792f,  11.6010f,1.6782f,-37.6011f,  10.9894f,0.6353f,-37.9339f,  12.7314f,-0.3087f,-38.2105f,  
	12.1149f,-1.3559f,-38.5406f,  13.8361f,-2.3752f,-38.5666f,  13.2138f,-3.4279f,-38.8941f,  14.8849f,-4.5043f,-38.6686f,  14.2559f,-5.5634f,-38.9930f,  
	15.8486f,-6.6775f,-38.5193f,  15.2122f,-7.7436f,-38.8402f,  16.6992f,-8.8751f,-38.1257f,  16.0542f,-9.9489f,-38.4425f,  17.4104f,-11.0768f,-37.4988f,  
	16.7558f,-12.1584f,-37.8105f,  17.9581f,-13.2617f,-36.6531f,  17.2926f,-14.3507f,-36.9590f,  18.3206f,-15.4080f,-35.6073f,  17.6423f,-16.5046f,-35.9066f,  
	18.4791f,-17.4950f,-34.3832f,  17.7866f,-18.5981f,-34.6751f,  18.4182f,-19.5017f,-33.0057f,  17.7103f,-20.6097f,-33.2890f,  18.1267f,-21.4073f,-31.5023f,  
	17.4018f,-22.5183f,-31.7766f,  17.5970f,-23.1924f,-29.9032f,  16.8536f,-24.3041f,-30.1679f,  16.8255f,-24.8387f,-28.2403f,  16.0624f,-25.9483f,-28.4948f,  
	15.8132f,-26.3299f,-26.5469f,  15.0294f,-27.4337f,-26.7912f,  14.5655f,-27.6511f,-24.8574f,  13.7607f,-28.7450f,-25.0921f,  13.0927f,-28.7897f,-23.2069f,  
	12.2666f,-29.8695f,-23.4323f,  11.4089f,-29.7362f,-21.6299f,  10.5616f,-30.7975f,-21.8468f,  19.2602f,28.5086f,0.2706f,  17.8016f,29.0590f,-1.6351f,  
	18.4509f,29.8535f,-0.9697f,  17.2022f,30.3370f,-3.0527f,  17.8497f,31.1571f,-2.3630f,  16.8216f,31.5493f,-4.5978f,  17.4670f,32.3971f,-3.8856f,  
	16.6663f,32.6751f,-6.2442f,  17.3094f,33.5522f,-5.5112f,  16.7390f,33.6953f,-7.9635f,  17.3795f,34.6028f,-7.2122f,  17.0384f,34.5923f,-9.7264f,  
	17.6761f,35.5309f,-8.9595f,  17.5595f,35.3508f,-11.5027f,  18.1942f,36.3204f,-10.7231f,  18.2932f,35.9579f,-13.2620f,  18.9249f,36.9581f,-12.4728f,  
	19.2271f,36.4030f,-14.9743f,  19.8557f,37.4329f,-14.1788f,  20.3452f,36.6787f,-16.6101f,  20.9706f,37.7367f,-15.8118f,  21.6283f,36.7802f,-18.1416f,  
	22.2507f,37.8643f,-17.3439f,  23.0545f,36.7058f,-19.5425f,  23.6739f,37.8136f,-18.7488f,  24.5994f,36.4566f,-20.7888f,  25.2160f,37.5853f,-20.0026f,  
	26.2365f,36.0371f,-21.8593f,  26.8505f,37.1835f,-21.0838f,  27.9379f,35.4544f,-22.7356f,  28.5495f,36.6149f,-21.9739f,  29.6744f,34.7184f,-23.4027f,  
	30.2840f,35.8894f,-22.6576f,  31.4163f,33.8418f,-23.8492f,  32.0241f,35.0193f,-23.1233f,  33.1338f,32.8395f,-24.0675f,  33.7402f,34.0195f,-23.3630f,  
	34.7975f,31.7287f,-24.0538f,  35.4030f,32.9071f,-23.3726f,  36.3790f,30.5284f,-23.8084f,  36.9838f,31.7012f,-23.1519f,  37.8512f,29.2591f,-23.3354f,  
	38.4558f,30.4224f,-22.7046f,  39.1889f,27.9426f,-22.6430f,  39.7937f,29.0925f,-22.0386f,  40.3692f,26.6013f,-21.7430f,  40.9746f,27.7344f,-21.1651f,  
	41.3719f,25.2583f,-20.6508f,  41.9783f,26.3712f,-20.0991f,  42.1799f,23.9365f,-19.3850f,  42.7877f,25.0263f,-18.8588f,  42.7794f,22.6585f,-17.9675f,  
	43.3889f,23.7227f,-17.4655f,  43.1600f,21.4462f,-16.4223f,  43.7716f,22.4827f,-15.9429f,  43.3153f,20.3203f,-14.7760f,  43.9292f,21.3276f,-14.3173f,  
	43.2426f,19.3002f,-13.0566f,  43.8591f,20.2770f,-12.6162f,  42.9432f,18.4031f,-11.2937f,  43.5625f,19.3490f,-10.8690f,  42.4221f,17.6446f,-9.5174f,  
	43.0444f,18.5594f,-9.1054f,  41.6884f,17.0376f,-7.7581f,  42.3137f,17.9217f,-7.3557f,  40.7545f,16.5924f,-6.0459f,  41.3829f,17.4469f,-5.6497f,  
	40.7545f,16.5924f,-6.0459f,  40.1315f,15.7308f,-6.4369f,  39.0105f,15.4815f,-4.7989f,  38.3903f,14.6376f,-5.1841f,  37.1015f,14.5835f,-3.6550f,  
	36.4846f,13.7521f,-4.0389f,  35.0504f,13.8883f,-2.6509f,  34.4372f,13.0649f,-3.0362f,  32.8824f,13.3830f,-1.8182f,  32.2733f,12.5628f,-2.2071f,  
	30.6248f,13.0509f,-1.1843f,  30.0202f,12.2297f,-1.5779f,  28.3064f,12.8733f,-0.7713f,  27.7066f,12.0471f,-1.1704f,  25.9570f,12.8290f,-0.5964f,  
	25.3619f,11.9940f,-1.0010f,  23.6065f,12.8950f,-0.6714f,  23.0163f,12.0493f,-1.0803f,  21.2850f,13.0488f,-1.0021f,  20.6992f,12.1898f,-1.4147f,  
	19.0217f,13.2657f,-1.5899f,  18.4404f,12.3917f,-2.0048f,  16.8452f,13.5214f,-2.4306f,  16.2676f,12.6321f,-2.8463f,  14.7817f,13.7926f,-3.5153f,  
	14.2072f,12.8868f,-3.9307f,  12.8557f,14.0553f,-4.8308f,  12.2837f,13.1338f,-5.2443f,  11.0893f,14.2878f,-6.3595f,  10.5193f,13.3507f,-6.7698f,  
	9.5021f,14.4688f,-8.0803f,  8.9334f,13.5175f,-8.4860f,  8.1104f,14.5791f,-9.9685f,  7.5420f,13.6145f,-10.3691f,  6.9268f,14.6006f,-11.9973f,  
	6.3584f,13.6244f,-12.3918f,  5.9612f,14.5177f,-14.1371f,  5.3917f,13.5308f,-14.5254f,  5.2189f,14.3159f,-16.3573f,  4.6486f,13.3211f,-16.7384f,  
	4.7030f,13.9846f,-18.6249f,  4.1303f,12.9818f,-18.9996f,  4.4107f,13.5127f,-20.9079f,  3.8359f,12.5044f,-21.2759f,  4.3374f,12.8940f,-23.1729f,  
	3.7599f,11.8810f,-23.5347f,  4.4739f,12.1232f,-25.3877f,  3.8933f,11.1064f,-25.7438f,  4.8075f,11.1974f,-27.5209f,  4.2236f,10.1776f,-27.8718f,  
	5.3224f,10.1163f,-29.5424f,  4.7352f,9.0942f,-29.8887f,  6.0001f,8.8822f,-31.4241f,  5.4091f,7.8576f,-31.7665f,  6.8184f,7.4988f,-33.1402f,  
	6.2235f,6.4720f,-33.4792f,  7.7534f,5.9728f,-34.6676f,  7.1548f,4.9438f,-35.0035f,  8.7793f,4.3132f,-35.9862f,  8.1763f,3.2810f,-36.3195f,  
	9.8678f,2.5300f,-37.0792f,  9.2598f,1.4935f,-37.4104f,  10.9894f,0.6353f,-37.9339f,  10.3766f,-0.4055f,-38.2627f,  12.1149f,-1.3559f,-38.5406f,  
	11.4964f,-2.4022f,-38.8671f,  13.2138f,-3.4279f,-38.8941f,  12.5888f,-4.4808f,-39.2179f,  14.2559f,-5.5634f,-38.9930f,  13.6237f,-6.6235f,-39.3138f,  
	15.2122f,-7.7436f,-38.8402f,  14.5715f,-8.8118f,-39.1573f,  16.0542f,-9.9489f,-38.4425f,  15.4043f,-11.0255f,-38.7550f,  16.7558f,-12.1584f,-37.8105f,  
	16.0952f,-13.2433f,-38.1176f,  17.2926f,-14.3507f,-36.9590f,  16.6194f,-15.4443f,-37.2600f,  17.6423f,-16.5046f,-35.9066f,  16.9552f,-17.6062f,-36.2005f,  
	17.7866f,-18.5981f,-34.6751f,  17.0841f,-19.7064f,-34.9606f,  17.7103f,-20.6097f,-33.2890f,  16.9907f,-21.7230f,-33.5656f,  17.4018f,-22.5183f,-31.7766f,  
	16.6635f,-23.6346f,-32.0434f,  16.8536f,-24.3041f,-30.1679f,  16.0952f,-25.4207f,-30.4242f,  16.0624f,-25.9483f,-28.4948f,  15.2827f,-27.0618f,-28.7405f,  
	15.0294f,-27.4337f,-26.7912f,  14.2276f,-28.5403f,-27.0266f,  13.7607f,-28.7450f,-25.0921f,  12.9363f,-29.8403f,-25.3172f,  12.2666f,-29.8695f,-23.4323f,  
	11.4193f,-30.9492f,-23.6478f,  10.5616f,-30.7975f,-21.8468f,  9.6926f,-31.8562f,-22.0541f,  17.8016f,29.0590f,-1.6351f,  16.5589f,29.4984f,-3.7227f,  
	17.2022f,30.3370f,-3.0527f,  16.1802f,30.6841f,-5.2888f,  16.8216f,31.5493f,-4.5978f,  16.0270f,31.7819f,-6.9542f,  16.6663f,32.6751f,-6.2442f,  
	16.1021f,32.7730f,-8.6903f,  16.7390f,33.6953f,-7.9635f,  16.4041f,33.6405f,-10.4675f,  17.0384f,34.5923f,-9.7264f,  16.9280f,34.3696f,-12.2552f,  
	17.5595f,35.3508f,-11.5027f,  17.6646f,34.9477f,-14.0231f,  18.2932f,35.9579f,-13.2620f,  18.6015f,35.3650f,-15.7407f,  19.2271f,36.4030f,-14.9743f,  
	19.7226f,35.6143f,-17.3787f,  20.3452f,36.6787f,-16.6101f,  21.0087f,35.6913f,-18.9091f,  21.6283f,36.7802f,-18.1416f,  22.4378f,35.5948f,-20.3057f,  
	23.0545f,36.7058f,-19.5425f,  23.9854f,35.3264f,-21.5446f,  24.5994f,36.4566f,-20.7888f,  25.6251f,34.8907f,-22.6046f,  26.2365f,36.0371f,-21.8593f,  
	27.3289f,34.2952f,-23.4676f,  27.9379f,35.4544f,-22.7356f,  29.0675f,33.5499f,-24.1187f,  29.6744f,34.7184f,-23.4027f,  30.8112f,32.6678f,-24.5469f,  
	31.4163f,33.8418f,-23.8492f,  32.5302f,31.6638f,-24.7449f,  33.1338f,32.8395f,-24.0675f,  34.1950f,30.5551f,-24.7091f,  34.7975f,31.7287f,-24.0538f,  
	35.7773f,29.3608f,-24.4404f,  36.3790f,30.5284f,-23.8084f,  37.2499f,28.1013f,-23.9431f,  37.8512f,29.2591f,-23.3354f,  38.5875f,26.7981f,-23.2260f,  
	39.1889f,27.9426f,-22.6430f,  39.7674f,25.4734f,-22.3011f,  40.3692f,26.6013f,-21.7430f,  40.7694f,24.1501f,-21.1845f,  41.3719f,25.2583f,-20.6508f,  
	41.5762f,22.8507f,-19.8951f,  42.1799f,23.9365f,-19.3850f,  42.1741f,21.5974f,-18.4550f,  42.7794f,22.6585f,-17.9675f,  42.5529f,20.4117f,-16.8888f,  
	43.1600f,21.4462f,-16.4223f,  42.7060f,19.3139f,-15.2234f,  43.3153f,20.3203f,-14.7760f,  42.6309f,18.3227f,-13.4873f,  43.2426f,19.3002f,-13.0566f,  
	42.3289f,17.4552f,-11.7102f,  42.9432f,18.4031f,-11.2937f,  41.8051f,16.7262f,-9.9224f,  42.4221f,17.6446f,-9.5174f,  41.0684f,16.1481f,-8.1546f,  
	41.6884f,17.0376f,-7.7581f,  40.1315f,15.7308f,-6.4369f,  40.7545f,16.5924f,-6.0459f,  40.1315f,15.7308f,-6.4369f,  39.5141f,14.8621f,-6.8238f,  
	38.3903f,14.6376f,-5.1841f,  37.7759f,13.7850f,-5.5666f,  36.4846f,13.7521f,-4.0389f,  35.8736f,12.9111f,-4.4201f,  34.4372f,13.0649f,-3.0362f,  
	33.8301f,12.2307f,-3.4188f,  32.2733f,12.5628f,-2.2071f,  31.6705f,11.7312f,-2.5929f,  30.0202f,12.2297f,-1.5779f,  29.4220f,11.3965f,-1.9681f,  
	27.7066f,12.0471f,-1.1704f,  27.1130f,11.2083f,-1.5653f,  25.3619f,11.9940f,-1.0010f,  24.7731f,11.1474f,-1.4000f,  23.0163f,12.0493f,-1.0803f,  
	22.4318f,11.1921f,-1.4830f,  20.6992f,12.1898f,-1.4147f,  20.1193f,11.3197f,-1.8202f,  18.4404f,12.3917f,-2.0048f,  17.8644f,11.5080f,-2.4116f,  
	16.2676f,12.6321f,-2.8463f,  15.6948f,11.7333f,-3.2535f,  14.2072f,12.8868f,-3.9307f,  13.6372f,11.9734f,-4.3366f,  12.2837f,13.1338f,-5.2443f,  
	11.7159f,12.2055f,-5.6478f,  10.5193f,13.3507f,-6.7698f,  9.9531f,12.4087f,-7.1696f,  8.9334f,13.5175f,-8.4860f,  8.3678f,12.5624f,-8.8814f,  
	7.5420f,13.6145f,-10.3691f,  6.9766f,12.6476f,-10.7590f,  6.3584f,13.6244f,-12.3918f,  5.7921f,12.6466f,-12.7761f,  5.3917f,13.5308f,-14.5254f,  
	4.8248f,12.5448f,-14.9030f,  4.6486f,13.3211f,-16.7384f,  4.0795f,12.3266f,-17.1101f,  4.1303f,12.9818f,-18.9996f,  3.5592f,11.9814f,-19.3648f,  
	3.8359f,12.5044f,-21.2759f,  3.2622f,11.4990f,-21.6351f,  3.7599f,11.8810f,-23.5347f,  3.1831f,10.8714f,-23.8884f,  3.8933f,11.1064f,-25.7438f,  
	3.3132f,10.0935f,-26.0924f,  4.2236f,10.1776f,-27.8718f,  3.6402f,9.1622f,-28.2159f,  4.7352f,9.0942f,-29.8887f,  4.1480f,8.0763f,-30.2289f,  
	5.4091f,7.8576f,-31.7665f,  4.8180f,6.8373f,-32.1033f,  6.2235f,6.4720f,-33.4792f,  5.6286f,5.4495f,-33.8131f,  7.1548f,4.9438f,-35.0035f,  
	6.5556f,3.9183f,-35.3350f,  8.1763f,3.2810f,-36.3195f,  7.5722f,2.2513f,-36.6490f,  9.2598f,1.4935f,-37.4104f,  8.6509f,0.4596f,-37.7378f,  
	10.3766f,-0.4055f,-38.2627f,  9.7621f,-1.4449f,-38.5880f,  11.4964f,-2.4022f,-38.8671f,  10.8756f,-3.4481f,-39.1901f,  12.5888f,-4.4808f,-39.2179f,  
	11.9610f,-5.5340f,-39.5383f,  13.6237f,-6.6235f,-39.3138f,  12.9878f,-7.6851f,-39.6310f,  14.5715f,-8.8118f,-39.1573f,  13.9267f,-9.8822f,-39.4704f,  
	15.4043f,-11.0255f,-38.7550f,  14.7491f,-12.1049f,-39.0631f,  16.0952f,-13.2433f,-38.1176f,  15.4277f,-14.3325f,-38.4201f,  16.6194f,-15.4443f,-37.2600f,  
	15.9381f,-16.5428f,-37.5558f,  16.9552f,-17.6062f,-36.2005f,  16.2589f,-18.7128f,-36.4882f,  17.0841f,-19.7064f,-34.9606f,  16.3707f,-20.8199f,-35.2396f,  
	16.9907f,-21.7230f,-33.5656f,  16.2586f,-22.8417f,-33.8349f,  16.6635f,-23.6346f,-32.0434f,  15.9110f,-24.7561f,-32.3021f,  16.0952f,-25.4207f,-30.4242f,  
	15.3209f,-26.5419f,-30.6719f,  15.2827f,-27.0618f,-28.7405f,  14.4854f,-28.1791f,-28.9772f,  14.2276f,-28.5403f,-27.0266f,  13.4067f,-29.6492f,-27.2522f,  
	12.9363f,-29.8403f,-25.3172f,  12.0910f,-30.9368f,-25.5321f,  11.4193f,-30.9492f,-23.6478f,  10.5504f,-32.0277f,-23.8532f,  9.6926f,-31.8562f,-22.0541f,  
	8.8001f,-32.9121f,-22.2507f,  16.5589f,29.4984f,-3.7227f,  15.5435f,29.8036f,-5.9580f,  16.1802f,30.6841f,-5.2888f,  15.3922f,30.8746f,-7.6410f,  
	16.0270f,31.7819f,-6.9542f,  15.4693f,31.8380f,-9.3925f,  16.1021f,32.7730f,-8.6903f,  15.7736f,32.6774f,-11.1828f,  16.4041f,33.6405f,-10.4675f,  
	16.2999f,33.3785f,-12.9810f,  16.9280f,34.3696f,-12.2552f,  17.0392f,33.9292f,-14.7565f,  17.6646f,34.9477f,-14.0231f,  17.9788f,34.3201f,-16.4788f,  
	18.6015f,35.3650f,-15.7407f,  19.1026f,34.5446f,-18.1186f,  19.7226f,35.6143f,-17.3787f,  20.3914f,34.5987f,-19.6477f,  21.0087f,35.6913f,-18.9091f,  
	21.8232f,34.4816f,-21.0399f,  22.4378f,35.5948f,-20.3057f,  23.3734f,34.1953f,-22.2716f,  23.9854f,35.3264f,-21.5446f,  25.0156f,33.7447f,-23.3215f,  
	25.6251f,34.8907f,-22.6046f,  26.7216f,33.1374f,-24.1717f,  27.3289f,34.2952f,-23.4676f,  28.4622f,32.3840f,-24.8077f,  29.0675f,33.5499f,-24.1187f,  
	30.2077f,31.4971f,-25.2186f,  30.8112f,32.6678f,-24.5469f,  31.9282f,30.4921f,-25.3974f,  32.5302f,31.6638f,-24.7449f,  33.5943f,29.3862f,-25.3409f,  
	34.1950f,30.5551f,-24.7091f,  35.1774f,28.1982f,-25.0502f,  35.7773f,29.3608f,-24.4404f,  36.6505f,26.9485f,-24.5303f,  37.2499f,28.1013f,-23.9431f,  
	37.9883f,25.6584f,-23.7900f,  38.5875f,26.7981f,-23.2260f,  39.1681f,24.3501f,-22.8420f,  39.7674f,25.4734f,-22.3011f,  40.1694f,23.0459f,-21.7025f,  
	40.7694f,24.1501f,-21.1845f,  40.9754f,21.7681f,-20.3911f,  41.5762f,22.8507f,-19.8951f,  41.5721f,20.5387f,-18.9301f,  42.1741f,21.5974f,-18.4550f,  
	41.9493f,19.3786f,-17.3446f,  42.5529f,20.4117f,-16.8888f,  42.1007f,18.3076f,-15.6616f,  42.7060f,19.3139f,-15.2234f,  42.0235f,17.3442f,-13.9100f,  
	42.6309f,18.3227f,-13.4873f,  41.7192f,16.5048f,-12.1198f,  42.3289f,17.4552f,-11.7102f,  41.1929f,15.8037f,-10.3216f,  41.8051f,16.7262f,-9.9224f,  
	40.4537f,15.2530f,-8.5461f,  41.0684f,16.1481f,-8.1546f,  39.5141f,14.8621f,-6.8238f,  40.1315f,15.7308f,-6.4369f,  39.5141f,14.8621f,-6.8238f,  
	38.9021f,13.9862f,-7.2076f,  37.7759f,13.7850f,-5.5666f,  37.1671f,12.9240f,-5.9461f,  35.8736f,12.9111f,-4.4201f,  35.2685f,12.0605f,-4.7987f,  
	33.8301f,12.2307f,-3.4188f,  33.2290f,11.3862f,-3.7988f,  31.6705f,11.7312f,-2.5929f,  31.0738f,10.8884f,-2.9758f,  29.4220f,11.3965f,-1.9681f,  
	28.8298f,10.5514f,-2.3548f,  27.1130f,11.2083f,-1.5653f,  26.5255f,10.3584f,-1.9557f,  24.7731f,11.1474f,-1.4000f,  24.1899f,10.2896f,-1.7938f,  
	22.4318f,11.1921f,-1.4830f,  21.8532f,10.3237f,-1.8798f,  20.1193f,11.3197f,-1.8202f,  19.5448f,10.4397f,-2.2186f,  17.8644f,11.5080f,-2.4116f,  
	17.2932f,10.6146f,-2.8109f,  15.6948f,11.7333f,-3.2535f,  15.1266f,10.8266f,-3.6522f,  13.6372f,11.9734f,-4.3366f,  13.0714f,11.0527f,-4.7336f,  
	11.7159f,12.2055f,-5.6478f,  11.1520f,11.2717f,-6.0418f,  9.9531f,12.4087f,-7.1696f,  9.3900f,11.4621f,-7.5598f,  8.3678f,12.5624f,-8.8814f,  
	7.8052f,11.6041f,-9.2668f,  6.9766f,12.6476f,-10.7590f,  6.4133f,11.6784f,-11.1394f,  5.7921f,12.6466f,-12.7761f,  5.2284f,11.6691f,-13.1503f,  
	4.8248f,12.5448f,-14.9030f,  4.2590f,11.5585f,-15.2716f,  4.0795f,12.3266f,-17.1101f,  3.5119f,11.3342f,-17.4725f,  3.5592f,11.9814f,-19.3648f,  
	2.9891f,10.9835f,-19.7215f,  3.2622f,11.4990f,-21.6351f,  2.6891f,10.4965f,-21.9864f,  3.1831f,10.8714f,-23.8884f,  2.6069f,9.8655f,-24.2347f,  
	3.3132f,10.0935f,-26.0924f,  2.7336f,9.0849f,-26.4342f,  3.6402f,9.1622f,-28.2159f,  3.0568f,8.1509f,-28.5539f,  4.1480f,8.0763f,-30.2289f,  
	3.5608f,7.0627f,-30.5636f,  4.8180f,6.8373f,-32.1033f,  4.2269f,5.8216f,-32.4351f,  5.6286f,5.4495f,-33.8131f,  5.0333f,4.4309f,-34.1425f,  
	6.5556f,3.9183f,-35.3350f,  5.9555f,2.8958f,-35.6626f,  7.5722f,2.2513f,-36.6490f,  6.9673f,1.2248f,-36.9747f,  8.6509f,0.4596f,-37.7378f,  
	8.0406f,-0.5722f,-38.0617f,  9.7621f,-1.4449f,-38.5880f,  9.1457f,-2.4831f,-38.9101f,  10.8756f,-3.4481f,-39.1901f,  10.2525f,-4.4936f,-39.5099f,  
	11.9610f,-5.5340f,-39.5383f,  11.3300f,-6.5880f,-39.8554f,  12.9878f,-7.6851f,-39.6310f,  12.3484f,-8.7482f,-39.9445f,  13.9267f,-9.8822f,-39.4704f,  
	13.2773f,-10.9549f,-39.7795f,  14.7491f,-12.1049f,-39.0631f,  14.0877f,-13.1881f,-39.3671f,  15.4277f,-14.3325f,-38.4201f,  14.7529f,-15.4261f,-38.7178f,  
	15.9381f,-16.5428f,-37.5558f,  15.2486f,-17.6459f,-37.8459f,  16.2589f,-18.7128f,-36.4882f,  15.5524f,-19.8246f,-36.7699f,  16.3707f,-20.8199f,-35.2396f,  
	15.6456f,-21.9390f,-35.5117f,  16.2586f,-22.8417f,-33.8349f,  15.5131f,-23.9660f,-34.0964f,  15.9110f,-24.7561f,-32.3021f,  15.1435f,-25.8828f,-32.5524f,  
	15.3209f,-26.5419f,-30.6719f,  14.5297f,-27.6675f,-30.9108f,  14.4854f,-28.1791f,-28.9772f,  13.6696f,-29.2995f,-29.2043f,  13.4067f,-29.6492f,-27.2522f,  
	12.5652f,-30.7605f,-27.4676f,  12.0910f,-30.9368f,-25.5321f,  11.2243f,-32.0333f,-25.7368f,  10.5504f,-32.0277f,-23.8532f,  9.6581f,-33.1048f,-24.0476f,  
	8.8001f,-32.9121f,-22.2507f,  7.8837f,-33.9640f,-22.4365f,  15.5435f,29.8036f,-5.9580f,  14.7618f,29.9542f,-8.3045f,  15.3922f,30.8746f,-7.6410f,  
	14.8407f,30.8913f,-10.0703f,  15.4693f,31.8380f,-9.3925f,  15.1469f,31.7042f,-11.8724f,  15.7736f,32.6774f,-11.1828f,  15.6753f,32.3788f,-13.6801f,  
	16.2999f,33.3785f,-12.9810f,  16.4169f,32.9038f,-15.4624f,  17.0392f,33.9292f,-14.7565f,  17.3588f,33.2700f,-17.1889f,  17.9788f,34.3201f,-16.4788f,  
	18.4850f,33.4712f,-18.8300f,  19.1026f,34.5446f,-18.1186f,  19.7763f,33.5040f,-20.3576f,  20.3914f,34.5987f,-19.6477f,  21.2105f,33.3679f,-21.7456f,  
	21.8232f,34.4816f,-21.0399f,  22.7632f,33.0651f,-22.9703f,  23.3734f,34.1953f,-22.2716f,  24.4076f,32.6008f,-24.0106f,  25.0156f,33.7447f,-23.3215f,  
	26.1158f,31.9830f,-24.8488f,  26.7216f,33.1374f,-24.1717f,  27.8584f,31.2222f,-25.4706f,  28.4622f,32.3840f,-24.8077f,  29.6057f,30.3315f,-25.8653f,  
	30.2077f,31.4971f,-25.2186f,  31.3277f,29.3261f,-26.0261f,  31.9282f,30.4921f,-25.3974f,  32.9950f,28.2232f,-25.9504f,  33.5943f,29.3862f,-25.3409f,  
	34.5791f,27.0417f,-25.6393f,  35.1774f,28.1982f,-25.0502f,  36.0529f,25.8018f,-25.0983f,  36.6505f,26.9485f,-24.5303f,  37.3911f,24.5247f,-24.3365f,  
	37.9883f,25.6584f,-23.7900f,  38.5709f,23.2322f,-23.3670f,  39.1681f,24.3501f,-22.8420f,  39.5720f,21.9465f,-22.2065f,  40.1694f,23.0459f,-21.7025f,  
	40.3773f,20.6896f,-20.8747f,  40.9754f,21.7681f,-20.3911f,  40.9731f,19.4830f,-19.3944f,  41.5721f,20.5387f,-18.9301f,  41.3492f,18.3473f,-17.7910f,  
	41.9493f,19.3786f,-17.3446f,  41.4991f,17.3019f,-16.0919f,  42.1007f,18.3076f,-15.6616f,  41.4202f,16.3648f,-14.3262f,  42.0235f,17.3442f,-13.9100f,  
	41.1139f,15.5520f,-12.5241f,  41.7192f,16.5048f,-12.1198f,  40.5855f,14.8773f,-10.7164f,  41.1929f,15.8037f,-10.3216f,  39.8440f,14.3524f,-8.9341f,  
	40.4537f,15.2530f,-8.5461f,  38.9021f,13.9862f,-7.2076f,  39.5141f,14.8621f,-6.8238f,  38.9021f,13.9862f,-7.2076f,  38.2955f,13.1035f,-7.5882f,  
	37.1671f,12.9240f,-5.9461f,  36.5639f,12.0549f,-6.3232f,  35.2685f,12.0605f,-4.7987f,  34.6691f,11.2007f,-5.1750f,  33.2290f,11.3862f,-3.7988f,  
	32.6338f,10.5316f,-4.1763f,  31.0738f,10.8884f,-2.9758f,  30.4829f,10.0347f,-3.3560f,  28.8298f,10.5514f,-2.3548f,  28.2434f,9.6959f,-2.7378f,  
	26.5255f,10.3584f,-1.9557f,  25.9435f,9.4977f,-2.3416f,  24.1899f,10.2896f,-1.7938f,  23.6125f,9.4207f,-2.1827f,  21.8532f,10.3237f,-1.8798f,  
	21.2799f,9.4455f,-2.2704f,  19.5448f,10.4397f,-2.2186f,  18.9749f,9.5499f,-2.6104f,  17.2932f,10.6146f,-2.8109f,  16.7266f,9.7130f,-3.2026f,  
	15.1266f,10.8266f,-3.6522f,  14.5626f,9.9121f,-4.0428f,  13.0714f,11.0527f,-4.7336f,  12.5095f,10.1259f,-5.1218f,  11.1520f,11.2717f,-6.0418f,  
	10.5912f,10.3326f,-6.4269f,  9.3900f,11.4621f,-7.5598f,  8.8300f,10.5116f,-7.9407f,  7.8052f,11.6041f,-9.2668f,  7.2447f,10.6428f,-9.6432f,  
	6.4133f,11.6784f,-11.1394f,  5.8527f,10.7089f,-11.5100f,  5.2284f,11.6691f,-13.1503f,  4.6660f,10.6905f,-13.5158f,  4.2590f,11.5585f,-15.2716f,  
	3.6950f,10.5736f,-15.6313f,  3.5119f,11.3342f,-17.4725f,  2.9455f,10.3436f,-17.8266f,  2.9891f,10.9835f,-19.7215f,  2.4198f,9.9882f,-20.0704f,  
	2.6891f,10.4965f,-21.9864f,  2.1167f,9.4976f,-22.3304f,  2.6069f,9.8655f,-24.2347f,  2.0311f,8.8637f,-24.5742f,  2.7336f,9.0849f,-26.4342f,  
	2.1542f,8.0804f,-26.7700f,  3.0568f,8.1509f,-28.5539f,  2.4735f,7.1442f,-28.8863f,  3.5608f,7.0627f,-30.5636f,  2.9736f,6.0538f,-30.8932f,  
	4.2269f,5.8216f,-32.4351f,  3.6355f,4.8101f,-32.7625f,  5.0333f,4.4309f,-34.1425f,  4.4372f,3.4158f,-34.4681f,  5.9555f,2.8958f,-35.6626f,  
	5.3547f,1.8768f,-35.9865f,  6.9673f,1.2248f,-36.9747f,  6.3614f,0.2009f,-37.2971f,  8.0406f,-0.5722f,-38.0617f,  7.4288f,-1.6023f,-38.3826f,  
	9.1457f,-2.4831f,-38.9101f,  8.5274f,-3.5203f,-39.2291f,  10.2525f,-4.4936f,-39.5099f,  9.6267f,-5.5393f,-39.8267f,  11.3300f,-6.5880f,-39.8554f,  
	10.6962f,-7.6427f,-40.1691f,  12.3484f,-8.7482f,-39.9445f,  11.7049f,-9.8129f,-40.2544f,  13.2773f,-10.9549f,-39.7795f,  12.6223f,-12.0307f,-40.0848f,  
	14.0877f,-13.1881f,-39.3671f,  13.4197f,-14.2753f,-39.6667f,  14.7529f,-15.4261f,-38.7178f,  14.0707f,-16.5240f,-39.0103f,  15.2486f,-17.6459f,-37.8459f,  
	14.5498f,-18.7541f,-38.1304f,  15.5524f,-19.8246f,-36.7699f,  14.8352f,-20.9420f,-37.0451f,  15.6456f,-21.9390f,-35.5117f,  14.9081f,-23.0638f,-35.7764f,  
	15.5131f,-23.9660f,-34.0964f,  14.7534f,-25.0958f,-34.3498f,  15.1435f,-25.8828f,-32.5524f,  14.3597f,-27.0145f,-32.7940f,  14.5297f,-27.6675f,-30.9108f,  
	13.7206f,-28.7972f,-31.1401f,  13.6696f,-29.2995f,-29.2043f,  12.8339f,-30.4234f,-29.4211f,  12.5652f,-30.7605f,-27.4676f,  11.7028f,-31.8730f,-27.6726f,  
	11.2243f,-32.0333f,-25.7368f,  10.3345f,-33.1297f,-25.9302f,  9.6581f,-33.1048f,-24.0476f,  8.7418f,-34.1793f,-24.2308f,  7.8837f,-33.9640f,-22.4365f,  
	6.9422f,-35.0107f,-22.6111f,  14.7618f,29.9542f,-8.3045f,  14.2161f,29.9343f,-10.7247f,  14.8407f,30.8913f,-10.0703f,  14.5239f,30.7219f,-12.5377f,  
	15.1469f,31.7042f,-11.8724f,  15.0541f,31.3717f,-14.3539f,  15.6753f,32.3788f,-13.6801f,  15.7975f,31.8723f,-16.1425f,  16.4169f,32.9038f,-15.4624f,  
	16.7415f,32.2153f,-17.8726f,  17.3588f,33.2700f,-17.1889f,  17.8699f,32.3948f,-19.5147f,  18.4850f,33.4712f,-18.8300f,  19.1633f,32.4077f,-21.0408f,  
	19.7763f,33.5040f,-20.3576f,  20.5998f,32.2539f,-22.4247f,  21.2105f,33.3679f,-21.7456f,  22.1546f,31.9358f,-23.6427f,  22.7632f,33.0651f,-22.9703f,  
	23.8012f,31.4591f,-24.6739f,  24.4076f,32.6008f,-24.0106f,  25.5114f,30.8318f,-25.5009f,  26.1158f,31.9830f,-24.8488f,  27.2559f,30.0646f,-26.1093f,  
	27.8584f,31.2222f,-25.4706f,  29.0049f,29.1708f,-26.4888f,  29.6057f,30.3315f,-25.8653f,  30.7285f,28.1655f,-26.6329f,  31.3277f,29.3261f,-26.0261f,  
	32.3972f,27.0660f,-26.5391f,  32.9950f,28.2232f,-25.9504f,  33.9824f,25.8911f,-26.2091f,  34.5791f,27.0417f,-25.6393f,  35.4570f,24.6610f,-25.6484f,  
	36.0529f,25.8018f,-25.0983f,  36.7957f,23.3965f,-24.8667f,  37.3911f,24.5247f,-24.3365f,  37.9758f,22.1195f,-23.8773f,  38.5709f,23.2322f,-23.3670f,  
	38.9769f,20.8517f,-22.6973f,  39.5720f,21.9465f,-22.2065f,  39.7819f,19.6149f,-21.3466f,  40.3773f,20.6896f,-20.8747f,  40.3771f,18.4301f,-19.8486f,  
	40.9731f,19.4830f,-19.3944f,  40.7523f,17.3177f,-18.2287f,  41.3492f,18.3473f,-17.7910f,  40.9011f,16.2967f,-16.5148f,  41.4991f,17.3019f,-16.0919f,  
	40.8209f,15.3845f,-14.7362f,  41.4202f,16.3648f,-14.3262f,  40.5130f,14.5969f,-12.9232f,  41.1139f,15.5520f,-12.5241f,  39.9829f,13.9472f,-11.1069f,  
	40.5855f,14.8773f,-10.7164f,  39.2394f,13.4465f,-9.3184f,  39.8440f,14.3524f,-8.9341f,  38.2955f,13.1035f,-7.5882f,  38.9021f,13.9862f,-7.2076f,  
	38.2955f,13.1035f,-7.5882f,  37.6940f,12.2141f,-7.9662f,  36.5639f,12.0549f,-6.3232f,  35.9660f,11.1779f,-6.6980f,  34.6691f,11.2007f,-5.1750f,  
	34.0752f,10.3320f,-5.5491f,  32.6338f,10.5316f,-4.1763f,  32.0441f,9.6669f,-4.5517f,  30.4829f,10.0347f,-3.3560f,  29.8976f,9.1712f,-3.7331f,  
	28.2434f,9.6959f,-2.7378f,  27.6625f,8.8301f,-3.1172f,  25.9435f,9.4977f,-2.3416f,  25.3670f,8.6263f,-2.7236f,  23.6125f,9.4207f,-2.1827f,  
	23.0402f,8.5422f,-2.5663f,  21.2799f,9.4455f,-2.2704f,  20.7112f,8.5573f,-2.6555f,  18.9749f,9.5499f,-2.6104f,  18.4096f,8.6516f,-2.9956f,  
	16.7266f,9.7130f,-3.2026f,  16.1641f,8.8031f,-3.5872f,  14.5626f,9.9121f,-4.0428f,  14.0025f,8.9909f,-4.4256f,  12.5095f,10.1259f,-5.1218f,  
	11.9508f,9.1932f,-5.5020f,  10.5912f,10.3326f,-6.4269f,  10.0334f,9.3890f,-6.8034f,  8.8300f,10.5116f,-7.9407f,  8.2721f,9.5576f,-8.3132f,  
	7.2447f,10.6428f,-9.6432f,  6.6870f,9.6806f,-10.0104f,  5.8527f,10.7089f,-11.5100f,  5.2933f,9.7375f,-11.8725f,  4.6660f,10.6905f,-13.5158f,  
	4.1052f,9.7128f,-13.8726f,  3.6950f,10.5736f,-15.6313f,  3.1319f,9.5900f,-15.9828f,  2.9455f,10.3436f,-17.8266f,  2.3797f,9.3550f,-18.1731f,  
	2.4198f,9.9882f,-20.0704f,  1.8510f,8.9960f,-20.4121f,  2.1167f,9.4976f,-22.3304f,  1.5446f,8.5023f,-22.6677f,  2.0311f,8.8637f,-24.5742f,  
	1.4555f,7.8659f,-24.9077f,  2.1542f,8.0804f,-26.7700f,  1.5747f,7.0804f,-27.1002f,  2.4735f,7.1442f,-28.8863f,  1.8902f,6.1421f,-29.2138f,  
	2.9736f,6.0538f,-30.8932f,  2.3862f,5.0494f,-31.2185f,  3.6355f,4.8101f,-32.7625f,  3.0435f,3.8024f,-33.0860f,  4.4372f,3.4158f,-34.4681f,  
	3.8407f,2.4046f,-34.7902f,  5.3547f,1.8768f,-35.9865f,  4.7531f,0.8611f,-36.3073f,  6.3614f,0.2009f,-37.2971f,  5.7541f,-0.8206f,-37.6167f,  
	7.4288f,-1.6023f,-38.3826f,  6.8153f,-2.6306f,-38.7007f,  8.5274f,-3.5203f,-39.2291f,  7.9068f,-4.5569f,-39.5454f,  9.6267f,-5.5393f,-39.8267f,  
	8.9985f,-6.5848f,-40.1405f,  10.6962f,-7.6427f,-40.1691f,  10.0588f,-8.6985f,-40.4797f,  11.7049f,-9.8129f,-40.2544f,  11.0565f,-10.8802f,-40.5610f,  
	12.6223f,-12.0307f,-40.0848f,  11.9615f,-13.1100f,-40.3863f,  13.4197f,-14.2753f,-39.6667f,  12.7453f,-15.3662f,-39.9616f,  14.0707f,-16.5240f,-39.0103f,  
	13.3800f,-17.6266f,-39.2977f,  14.5498f,-18.7541f,-38.1304f,  13.8411f,-19.8676f,-38.4088f,  14.8352f,-20.9420f,-37.0451f,  14.1066f,-22.0650f,-37.3132f,  
	14.9081f,-23.0638f,-35.7764f,  14.1574f,-24.1943f,-36.0333f,  14.7534f,-25.0958f,-34.3498f,  13.9783f,-26.2310f,-34.5948f,  14.3597f,-27.0145f,-32.7940f,  
	13.5589f,-28.1508f,-33.0264f,  13.7206f,-28.7972f,-31.1401f,  12.8923f,-29.9311f,-31.3593f,  12.8339f,-30.4234f,-29.4211f,  11.9776f,-31.5494f,-29.6276f,  
	11.7028f,-31.8730f,-27.6726f,  10.8175f,-32.9866f,-27.8662f,  10.3345f,-33.1297f,-25.9302f,  9.4206f,-34.2251f,-26.1121f,  8.7418f,-34.1793f,-24.2308f,  
	7.8003f,-35.2502f,-24.4023f,  6.9422f,-35.0107f,-22.6111f,  5.9749f,-36.0512f,-22.7739f,  14.2161f,29.9343f,-10.7247f,  13.9045f,29.7319f,-13.1792f,  
	14.5239f,30.7219f,-12.5377f,  14.4360f,30.3581f,-15.0033f,  15.0541f,31.3717f,-14.3539f,  15.1810f,30.8360f,-16.7974f,  15.7975f,31.8723f,-16.1425f,  
	16.1267f,31.1572f,-18.5308f,  16.7415f,32.2153f,-17.8726f,  17.2569f,31.3164f,-20.1738f,  17.8699f,32.3948f,-19.5147f,  18.5523f,31.3108f,-21.6983f,  
	19.1633f,32.4077f,-21.0408f,  19.9907f,31.1406f,-23.0782f,  20.5998f,32.2539f,-22.4247f,  21.5475f,30.8085f,-24.2899f,  22.1546f,31.9358f,-23.6427f,  
	23.1960f,30.3204f,-25.3127f,  23.8012f,31.4591f,-24.6739f,  24.9081f,29.6845f,-26.1291f,  25.5114f,30.8318f,-25.5009f,  26.6545f,28.9118f,-26.7251f,  
	27.2559f,30.0646f,-26.1093f,  28.4052f,28.0154f,-27.0905f,  29.0049f,29.1708f,-26.4888f,  30.1304f,27.0107f,-27.2190f,  30.7285f,28.1655f,-26.6329f,  
	31.8004f,25.9149f,-27.1086f,  32.3972f,27.0660f,-26.5391f,  33.3868f,24.7467f,-26.7609f,  33.9824f,25.8911f,-26.2091f,  34.8624f,23.5262f,-26.1821f,  
	35.4570f,24.6610f,-25.6484f,  36.2019f,22.2741f,-25.3820f,  36.7957f,23.3965f,-24.8667f,  37.3825f,21.0120f,-24.3742f,  37.9758f,22.1195f,-23.8773f,  
	38.3838f,19.7615f,-23.1761f,  38.9769f,20.8517f,-22.6973f,  39.1889f,18.5438f,-21.8082f,  39.7819f,19.6149f,-21.3466f,  39.7838f,17.3799f,-20.2938f,  
	40.3771f,18.4301f,-19.8486f,  40.1585f,16.2897f,-18.6588f,  40.7523f,17.3177f,-18.2287f,  40.3065f,15.2918f,-16.9312f,  40.9011f,16.2967f,-16.5148f,  
	40.2253f,14.4033f,-15.1407f,  40.8209f,15.3845f,-14.7362f,  39.9163f,13.6394f,-13.3178f,  40.5130f,14.5969f,-12.9232f,  39.3847f,13.0132f,-11.4936f,  
	39.9829f,13.9472f,-11.1069f,  38.6397f,12.5354f,-9.6995f,  39.2394f,13.4465f,-9.3184f,  37.6940f,12.2141f,-7.9662f,  38.2955f,13.1035f,-7.5882f,  
	37.6940f,12.2141f,-7.9662f,  37.0977f,11.3182f,-8.3416f,  35.9660f,11.1779f,-6.6980f,  35.3734f,10.2931f,-7.0708f,  34.0752f,10.3320f,-5.5491f,  
	33.4865f,9.4543f,-5.9215f,  32.0441f,9.6669f,-4.5517f,  31.4596f,8.7933f,-4.9244f,  29.8976f,9.1712f,-3.7331f,  29.3175f,8.2981f,-4.1072f,  
	27.6625f,8.8301f,-3.1172f,  27.0868f,7.9539f,-3.4934f,  25.3670f,8.6263f,-2.7236f,  24.7955f,7.7454f,-3.1013f,  23.0402f,8.5422f,-2.5663f,  
	22.4724f,7.6538f,-2.9453f,  20.7112f,8.5573f,-2.6555f,  20.1470f,7.6605f,-3.0348f,  18.4096f,8.6516f,-2.9956f,  17.8484f,7.7447f,-3.3747f,  
	16.1641f,8.8031f,-3.5872f,  15.6055f,7.8861f,-3.9648f,  14.0025f,8.9909f,-4.4256f,  13.4455f,8.0634f,-4.8010f,  11.9508f,9.1932f,-5.5020f,  
	11.3950f,8.2554f,-5.8743f,  10.0334f,9.3890f,-6.8034f,  9.4778f,8.4413f,-7.1721f,  8.2721f,9.5576f,-8.3132f,  7.7169f,8.6019f,-8.6771f,  
	6.6870f,9.6806f,-10.0104f,  6.1303f,8.7159f,-10.3699f,  5.2933f,9.7375f,-11.8725f,  4.7355f,8.7665f,-12.2267f,  4.1052f,9.7128f,-13.8726f,  
	3.5454f,8.7358f,-14.2217f,  3.1319f,9.5900f,-15.9828f,  2.5695f,8.6079f,-16.3271f,  2.3797f,9.3550f,-18.1731f,  1.8145f,8.3693f,-18.5126f,  
	1.8510f,8.9960f,-20.4121f,  1.2826f,8.0071f,-20.7472f,  1.5446f,8.5023f,-22.6677f,  0.9727f,7.5110f,-22.9990f,  1.4555f,7.8659f,-24.9077f,  
	0.8798f,6.8725f,-25.2358f,  1.5747f,7.0804f,-27.1002f,  0.9952f,6.0851f,-27.4255f,  1.8902f,6.1421f,-29.2138f,  1.3066f,5.1447f,-29.5369f,  
	2.3862f,5.0494f,-31.2185f,  1.7982f,4.0492f,-31.5400f,  3.0435f,3.8024f,-33.0860f,  2.4511f,2.7992f,-33.4062f,  3.8407f,2.4046f,-34.7902f,  
	3.2433f,1.3972f,-35.1093f,  4.7531f,0.8611f,-36.3073f,  4.1503f,-0.1516f,-36.6255f,  5.7541f,-0.8206f,-37.6167f,  5.1455f,-1.8398f,-37.9337f,  
	6.8153f,-2.6306f,-38.7007f,  6.1999f,-3.6576f,-39.0164f,  7.9068f,-4.5569f,-39.5454f,  7.2843f,-5.5926f,-39.8592f,  8.9985f,-6.5848f,-40.1405f,  
	8.3673f,-7.6307f,-40.4516f,  10.0588f,-8.6985f,-40.4797f,  9.4172f,-9.7561f,-40.7874f,  11.0565f,-10.8802f,-40.5610f,  10.4031f,-11.9502f,-40.8641f,  
	11.9615f,-13.1100f,-40.3863f,  11.2950f,-14.1925f,-40.6835f,  12.7453f,-15.3662f,-39.9616f,  12.0631f,-16.4616f,-40.2519f,  13.3800f,-17.6266f,-39.2977f,  
	12.6804f,-18.7344f,-39.5795f,  13.8411f,-19.8676f,-38.4088f,  13.1220f,-20.9867f,-38.6807f,  14.1066f,-22.0650f,-37.3132f,  13.3656f,-23.1938f,-37.5741f,  
	14.1574f,-24.1943f,-36.0333f,  13.3921f,-25.3306f,-36.2823f,  13.9783f,-26.2310f,-34.5948f,  13.1871f,-27.3714f,-34.8309f,  13.5589f,-28.1508f,-33.0264f,  
	12.7396f,-29.2921f,-33.2489f,  12.8923f,-29.9311f,-31.3593f,  12.0440f,-31.0681f,-31.5682f,  11.9776f,-31.5494f,-29.6276f,  11.0991f,-32.6777f,-29.8226f,  
	10.8175f,-32.9866f,-27.8662f,  9.9085f,-34.1005f,-28.0482f,  9.4206f,-34.2251f,-26.1121f,  8.4817f,-35.3182f,-26.2819f,  7.8003f,-35.2502f,-24.4023f,  
	6.8328f,-36.3166f,-24.5616f,  5.9749f,-36.0512f,-22.7739f,  4.9814f,-37.0840f,-22.9250f,  13.9045f,29.7319f,-13.1792f,  13.8212f,29.3393f,-15.6292f,  
	14.4360f,30.3581f,-15.0033f,  14.5674f,29.7957f,-17.4283f,  15.1810f,30.8360f,-16.7974f,  15.5144f,30.0965f,-19.1646f,  16.1267f,31.1572f,-18.5308f,  
	16.6461f,30.2368f,-20.8083f,  17.2569f,31.3164f,-20.1738f,  17.9430f,30.2139f,-22.3313f,  18.5523f,31.3108f,-21.6983f,  19.3831f,30.0285f,-23.7075f,  
	19.9907f,31.1406f,-23.0782f,  20.9416f,29.6835f,-24.9134f,  21.5475f,30.8085f,-24.2899f,  22.5919f,29.1850f,-25.9284f,  23.1960f,30.3204f,-25.3127f,  
	24.3057f,28.5413f,-26.7350f,  24.9081f,29.6845f,-26.1291f,  26.0538f,27.7637f,-27.3194f,  26.6545f,28.9118f,-26.7251f,  27.8062f,26.8653f,-27.6718f,  
	28.4052f,28.0154f,-27.0905f,  29.5329f,25.8616f,-27.7860f,  30.1304f,27.0107f,-27.2190f,  31.2044f,24.7696f,-27.6601f,  31.8004f,25.9149f,-27.1086f,  
	32.7921f,23.6082f,-27.2963f,  33.3868f,24.7467f,-26.7609f,  34.2688f,22.3971f,-26.7007f,  34.8624f,23.5262f,-26.1821f,  35.6093f,21.1571f,-25.8836f,  
	36.2019f,22.2741f,-25.3820f,  36.7906f,19.9094f,-24.8589f,  37.3825f,21.0120f,-24.3742f,  37.7925f,18.6754f,-23.6442f,  38.3838f,19.7615f,-23.1761f,  
	38.5978f,17.4761f,-22.2603f,  39.1889f,18.5438f,-21.8082f,  39.1929f,16.3321f,-20.7308f,  39.7838f,17.3799f,-20.2938f,  39.5674f,15.2630f,-19.0819f,  
	40.1585f,16.2897f,-18.6588f,  39.7151f,14.2871f,-17.3419f,  40.3065f,15.2918f,-16.9312f,  39.6333f,13.4209f,-15.5404f,  40.2253f,14.4033f,-15.1407f,  
	39.3234f,12.6795f,-13.7084f,  39.9163f,13.6394f,-13.3178f,  38.7909f,12.0754f,-11.8771f,  39.3847f,13.0132f,-11.4936f,  38.0447f,11.6190f,-10.0779f,  
	38.6397f,12.5354f,-9.6995f,  37.0977f,11.3182f,-8.3416f,  37.6940f,12.2141f,-7.9662f,  37.0977f,11.3182f,-8.3416f,  36.5061f,10.4157f,-8.7151f,  
	35.3734f,10.2931f,-7.0708f,  34.7856f,9.4006f,-7.4419f,  33.4865f,9.4543f,-5.9215f,  32.9028f,8.5686f,-6.2915f,  31.4596f,8.7933f,-4.9244f,  
	30.8801f,7.9108f,-5.2946f,  29.3175f,8.2981f,-4.1072f,  28.7423f,7.4152f,-4.4788f,  27.0868f,7.9539f,-3.4934f,  26.5158f,7.0687f,-3.8660f,  
	24.7955f,7.7454f,-3.1013f,  24.2284f,6.8549f,-3.4750f,  22.4724f,7.6538f,-2.9453f,  21.9090f,6.7568f,-3.3193f,  20.1470f,7.6605f,-3.0348f,  
	19.5867f,6.7551f,-3.4087f,  17.8484f,7.7447f,-3.3747f,  17.2909f,6.8305f,-3.7475f,  15.6055f,7.8861f,-3.9648f,  15.0498f,6.9625f,-4.3359f,  
	13.4455f,8.0634f,-4.8010f,  12.8914f,7.1302f,-5.1694f,  11.3950f,8.2554f,-5.8743f,  10.8413f,7.3130f,-6.2395f,  9.4778f,8.4413f,-7.1721f,  
	8.9248f,7.4912f,-7.5328f,  7.7169f,8.6019f,-8.6771f,  7.1627f,7.6430f,-9.0337f,  6.1303f,8.7159f,-10.3699f,  5.5753f,7.7508f,-10.7215f,  
	4.7355f,8.7665f,-12.2267f,  4.1786f,7.7954f,-12.5734f,  3.5454f,8.7358f,-14.2217f,  2.9860f,7.7597f,-14.5638f,  2.5695f,8.6079f,-16.3271f,  
	2.0076f,7.6282f,-16.6644f,  1.8145f,8.3693f,-18.5126f,  1.2493f,7.3864f,-18.8457f,  1.2826f,8.0071f,-20.7472f,  0.7141f,7.0219f,-21.0765f,  
	0.9727f,7.5110f,-22.9990f,  0.4006f,6.5239f,-23.3250f,  0.8798f,6.8725f,-25.2358f,  0.3039f,5.8835f,-25.5591f,  0.9952f,6.0851f,-27.4255f,  
	0.4153f,5.0944f,-27.7466f,  1.3066f,5.1447f,-29.5369f,  0.7225f,4.1518f,-29.8564f,  1.7982f,4.0492f,-31.5400f,  1.2097f,3.0537f,-31.8582f,  
	2.4511f,2.7992f,-33.4062f,  1.8578f,1.8000f,-33.7235f,  3.2433f,1.3972f,-35.1093f,  2.6450f,0.3933f,-35.4259f,  4.1503f,-0.1516f,-36.6255f,  
	3.5464f,-1.1613f,-36.9413f,  5.1455f,-1.8398f,-37.9337f,  4.5352f,-2.8569f,-38.2486f,  6.1999f,-3.6576f,-39.0164f,  5.5830f,-4.6830f,-39.3298f,  
	7.2843f,-5.5926f,-39.8592f,  6.6592f,-6.6279f,-40.1705f,  8.3673f,-7.6307f,-40.4516f,  7.7324f,-8.6778f,-40.7601f,  9.4172f,-9.7561f,-40.7874f,  
	8.7711f,-10.8157f,-41.0921f,  10.4031f,-11.9502f,-40.8641f,  9.7447f,-13.0228f,-41.1636f,  11.2950f,-14.1925f,-40.6835f,  10.6216f,-15.2788f,-40.9767f,  
	12.0631f,-16.4616f,-40.2519f,  11.3730f,-17.5615f,-40.5372f,  12.6804f,-18.7344f,-39.5795f,  11.9714f,-19.8473f,-39.8554f,  13.1220f,-20.9867f,-38.6807f,  
	12.3916f,-22.1115f,-38.9460f,  13.3656f,-23.1938f,-37.5741f,  12.6110f,-24.3286f,-37.8277f,  13.3921f,-25.3306f,-36.2823f,  12.6116f,-26.4724f,-36.5229f,  
	13.1871f,-27.3714f,-34.8309f,  12.3782f,-28.5174f,-35.0575f,  12.7396f,-29.2921f,-33.2489f,  11.9010f,-30.4374f,-33.4613f,  12.0440f,-31.0681f,-31.5682f,  
	11.1743f,-32.2085f,-31.7659f,  11.0991f,-32.6777f,-29.8226f,  10.1972f,-33.8077f,-30.0060f,  9.9085f,-34.1005f,-28.0482f,  8.9747f,-35.2136f,-28.2179f,  
	8.4817f,-35.3182f,-26.2819f,  7.5166f,-36.4084f,-26.4394f,  6.8328f,-36.3166f,-24.5616f,  5.8385f,-37.3767f,-24.7088f,  4.9814f,-37.0840f,-22.9250f,  
	3.9599f,-38.1080f,-23.0635f,  13.8212f,29.3393f,-15.6292f,  13.9565f,28.7524f,-18.0360f,  14.5674f,29.7957f,-17.4283f,  14.9045f,29.0342f,-19.7749f,  
	15.5144f,30.0965f,-19.1646f,  16.0373f,29.1567f,-21.4193f,  16.6461f,30.2368f,-20.8083f,  17.3354f,29.1179f,-22.9409f,  17.9430f,30.2139f,-22.3313f,  
	18.7768f,28.9183f,-24.3137f,  19.3831f,30.0285f,-23.7075f,  20.3368f,28.5614f,-25.5143f,  20.9416f,29.6835f,-24.9134f,  21.9886f,28.0534f,-26.5220f,  
	22.5919f,29.1850f,-25.9284f,  23.7040f,27.4028f,-27.3197f,  24.3057f,28.5413f,-26.7350f,  25.4536f,26.6208f,-27.8936f,  26.0538f,27.7637f,-27.3194f,  
	27.2076f,25.7209f,-28.2340f,  27.8062f,26.8653f,-27.6718f,  28.9358f,24.7184f,-28.3351f,  29.5329f,25.8616f,-27.7860f,  30.6088f,23.6304f,-28.1951f,  
	31.2044f,24.7696f,-27.6601f,  32.1979f,22.4756f,-27.8163f,  32.7921f,23.6082f,-27.2963f,  33.6759f,21.2738f,-27.2054f,  34.2688f,22.3971f,-26.7007f,  
	35.0175f,20.0454f,-26.3727f,  35.6093f,21.1571f,-25.8836f,  36.1997f,18.8116f,-25.3326f,  36.7906f,19.9094f,-24.8589f,  37.2024f,17.5934f,-24.1027f,  
	37.7925f,18.6754f,-23.6442f,  38.0084f,16.4116f,-22.7041f,  38.5978f,17.4761f,-22.2603f,  38.6039f,15.2866f,-21.1608f,  39.1929f,16.3321f,-20.7308f,  
	38.9787f,14.2375f,-19.4992f,  39.5674f,15.2630f,-19.0819f,  39.1263f,13.2823f,-17.7476f,  39.7151f,14.2871f,-17.3419f,  39.0444f,12.4373f,-15.9362f,  
	39.6333f,13.4209f,-15.5404f,  38.7342f,11.7170f,-14.0958f,  39.3234f,12.6795f,-13.7084f,  38.2010f,11.1337f,-12.2579f,  38.7909f,12.0754f,-11.8771f,  
	37.4541f,10.6975f,-10.4541f,  38.0447f,11.6190f,-10.0779f,  36.5061f,10.4157f,-8.7151f,  37.0977f,11.3182f,-8.3416f,  36.5061f,10.4157f,-8.7151f,  
	35.9191f,9.5066f,-9.0868f,  34.7856f,9.4006f,-7.4419f,  34.2025f,8.5010f,-7.8108f,  32.9028f,8.5686f,-6.2915f,  32.3238f,7.6749f,-6.6593f,  
	30.8801f,7.9108f,-5.2946f,  30.3053f,7.0193f,-5.6630f,  28.7423f,7.4152f,-4.4788f,  28.1717f,6.5237f,-4.8475f,  26.5158f,7.0687f,-3.8660f,  
	25.9490f,6.1742f,-4.2354f,  24.2284f,6.8549f,-3.4750f,  23.6653f,5.9559f,-3.8445f,  21.9090f,6.7568f,-3.3193f,  21.3492f,5.8512f,-3.6889f,  
	19.5867f,6.7551f,-3.4087f,  19.0299f,5.8421f,-3.7774f,  17.2909f,6.8305f,-3.7475f,  16.7362f,5.9093f,-4.1146f,  15.0498f,6.9625f,-4.3359f,  
	14.4969f,6.0327f,-4.7007f,  12.8914f,7.1302f,-5.1694f,  12.3392f,6.1919f,-5.5312f,  10.8413f,7.3130f,-6.2395f,  10.2902f,6.3675f,-6.5972f,  
	8.9248f,7.4912f,-7.5328f,  8.3727f,6.5373f,-7.8869f,  7.1627f,7.6430f,-9.0337f,  6.6100f,6.6832f,-9.3830f,  5.5753f,7.7508f,-10.7215f,  
	5.0209f,6.7851f,-11.0662f,  4.1786f,7.7954f,-12.5734f,  3.6221f,6.8247f,-12.9135f,  2.9860f,7.7597f,-14.5638f,  2.4270f,6.7854f,-14.8993f,  
	2.0076f,7.6282f,-16.6644f,  1.4455f,6.6508f,-16.9957f,  1.2493f,7.3864f,-18.8457f,  0.6842f,6.4069f,-19.1731f,  0.7141f,7.0219f,-21.0765f,  
	0.1454f,6.0406f,-21.4006f,  0.4006f,6.5239f,-23.3250f,  -0.1719f,5.5410f,-23.6464f,  0.3039f,5.8835f,-25.5591f,  -0.2725f,4.8993f,-25.8782f,  
	0.4153f,5.0944f,-27.7466f,  -0.1652f,4.1085f,-28.0641f,  0.7225f,4.1518f,-29.8564f,  0.1378f,3.1638f,-30.1727f,  1.2097f,3.0537f,-31.8582f,  
	0.6204f,2.0626f,-32.1737f,  1.8578f,1.8000f,-33.7235f,  1.2637f,0.8049f,-34.0385f,  2.6450f,0.3933f,-35.4259f,  2.0456f,-0.6069f,-35.7404f,  
	3.5464f,-1.1613f,-36.9413f,  2.9410f,-2.1684f,-37.2552f,  4.5352f,-2.8569f,-38.2486f,  3.9236f,-3.8715f,-38.5616f,  5.5830f,-4.6830f,-39.3298f,  
	4.9638f,-5.7072f,-39.6413f,  6.6592f,-6.6279f,-40.1705f,  6.0309f,-7.6636f,-40.4798f,  7.7324f,-8.6778f,-40.7601f,  7.0937f,-9.7260f,-41.0663f,  
	8.7711f,-10.8157f,-41.0921f,  8.1209f,-11.8772f,-41.3937f,  9.7447f,-13.0228f,-41.1636f,  9.0802f,-14.0986f,-41.4596f,  10.6216f,-15.2788f,-40.9767f,  
	9.9411f,-16.3693f,-41.2655f,  11.3730f,-17.5615f,-40.5372f,  10.6744f,-18.6663f,-40.8173f,  11.9714f,-19.8473f,-39.8554f,  11.2521f,-20.9657f,-40.1253f,  
	12.3916f,-22.1115f,-38.9460f,  11.6486f,-23.2422f,-39.2045f,  12.6110f,-24.3286f,-37.8277f,  11.8423f,-25.4691f,-38.0733f,  12.6116f,-26.4724f,-36.5229f,  
	11.8145f,-27.6202f,-36.7544f,  12.3782f,-28.5174f,-35.0575f,  11.5510f,-29.6680f,-35.2745f,  11.9010f,-30.4374f,-33.4613f,  11.0418f,-31.5869f,-33.6627f,  
	11.1743f,-32.2085f,-31.7659f,  10.2816f,-33.3516f,-31.9520f,  10.1972f,-33.8077f,-30.0060f,  9.2709f,-34.9381f,-30.1771f,  8.9747f,-35.2136f,-28.2179f,  
	8.0147f,-36.3253f,-28.3751f,  7.5166f,-36.4084f,-26.4394f,  6.5245f,-37.4940f,-26.5843f,  5.8385f,-37.3767f,-24.7088f,  4.8158f,-38.4299f,-24.8428f,  
	3.9599f,-38.1080f,-23.0635f,  2.9101f,-39.1216f,-23.1892f,  13.9565f,28.7524f,-18.0360f,  14.2968f,27.9711f,-20.3627f,  14.9045f,29.0342f,-19.7749f,  
	15.4304f,28.0771f,-22.0077f,  16.0373f,29.1567f,-21.4193f,  16.7295f,28.0234f,-23.5281f,  17.3354f,29.1179f,-22.9409f,  18.1719f,27.8108f,-24.8978f,  
	18.7768f,28.9183f,-24.3137f,  19.7330f,27.4430f,-26.0935f,  20.3368f,28.5614f,-25.5143f,  21.3861f,26.9263f,-27.0947f,  21.9886f,28.0534f,-26.5220f,  
	23.1029f,26.2694f,-27.8842f,  23.7040f,27.4028f,-27.3197f,  24.8540f,25.4838f,-28.4485f,  25.4536f,26.6208f,-27.8936f,  26.6095f,24.5827f,-28.7781f,  
	27.2076f,25.7209f,-28.2340f,  28.3392f,23.5816f,-28.8672f,  28.9358f,24.7184f,-28.3351f,  30.0137f,22.4977f,-28.7143f,  30.6088f,23.6304f,-28.1951f,  
	31.6042f,21.3494f,-28.3220f,  32.1979f,22.4756f,-27.8163f,  33.0835f,20.1565f,-27.6971f,  33.6759f,21.2738f,-27.2054f,  34.4264f,18.9393f,-26.8502f,  
	35.0175f,20.0454f,-26.3727f,  35.6099f,17.7187f,-25.7959f,  36.1997f,18.8116f,-25.3326f,  36.6137f,16.5156f,-24.5521f,  37.2024f,17.5934f,-24.1027f,  
	37.4206f,15.3505f,-23.1402f,  38.0084f,16.4116f,-22.7041f,  38.0168f,14.2433f,-21.5842f,  38.6039f,15.2866f,-21.1608f,  38.3922f,13.2131f,-19.9110f,  
	38.9787f,14.2375f,-19.4992f,  38.5402f,12.2774f,-18.1489f,  39.1263f,13.2823f,-17.7476f,  38.4585f,11.4523f,-16.3283f,  39.0444f,12.4373f,-15.9362f,  
	38.1483f,10.7519f,-14.4803f,  38.7342f,11.7170f,-14.0958f,  37.6150f,10.1881f,-12.6364f,  38.2010f,11.1337f,-12.2579f,  36.8676f,9.7707f,-10.8283f,  
	37.4541f,10.6975f,-10.4541f,  35.9191f,9.5066f,-9.0868f,  36.5061f,10.4157f,-8.7151f,  35.9191f,9.5066f,-9.0868f,  35.3364f,8.5916f,-9.4565f,  
	34.2025f,8.5010f,-7.8108f,  33.6237f,7.5944f,-8.1778f,  32.3238f,7.6749f,-6.6593f,  31.7490f,6.7730f,-7.0258f,  30.3053f,7.0193f,-5.6630f,  
	29.7347f,6.1197f,-6.0289f,  28.1717f,6.5237f,-4.8475f,  27.6049f,5.6234f,-5.2135f,  25.9490f,6.1742f,-4.2354f,  25.3861f,5.2716f,-4.6013f,  
	23.6653f,5.9559f,-3.8445f,  23.1058f,5.0484f,-4.2103f,  21.3492f,5.8512f,-3.6889f,  20.7928f,4.9378f,-4.0539f,  19.0299f,5.8421f,-3.7774f,  
	18.4759f,4.9219f,-4.1409f,  16.7362f,5.9093f,-4.1146f,  16.1842f,4.9816f,-4.4761f,  14.4969f,6.0327f,-4.7007f,  13.9459f,5.0974f,-5.0595f,  
	12.3392f,6.1919f,-5.5312f,  11.7895f,5.2500f,-5.8863f,  10.2902f,6.3675f,-6.5972f,  9.7397f,5.4176f,-6.9489f,  8.3727f,6.5373f,-7.8869f,  
	7.8220f,5.5819f,-8.2340f,  6.6100f,6.6832f,-9.3830f,  6.0579f,5.7220f,-9.7257f,  5.0209f,6.7851f,-11.0662f,  4.4668f,5.8191f,-11.4044f,  
	3.6221f,6.8247f,-12.9135f,  3.0658f,5.8554f,-13.2472f,  2.4270f,6.7854f,-14.8993f,  1.8678f,5.8130f,-15.2288f,  1.4455f,6.6508f,-16.9957f,  
	0.8833f,5.6765f,-17.3214f,  0.6842f,6.4069f,-19.1731f,  0.1185f,5.4310f,-19.4954f,  0.1454f,6.0406f,-21.4006f,  -0.4239f,5.0634f,-21.7201f,  
	-0.1719f,5.5410f,-23.6464f,  -0.7450f,4.5628f,-23.9636f,  -0.2725f,4.8993f,-25.8782f,  -0.8495f,3.9200f,-26.1938f,  -0.1652f,4.1085f,-28.0641f,  
	-0.7464f,3.1276f,-28.3785f,  0.1378f,3.1638f,-30.1727f,  -0.4478f,2.1805f,-30.4864f,  0.6204f,2.0626f,-32.1737f,  0.0302f,1.0761f,-32.4870f,  
	1.2637f,0.8049f,-34.0385f,  0.6686f,-0.1860f,-34.3516f,  2.0456f,-0.6069f,-35.7404f,  1.4448f,-1.6038f,-36.0532f,  2.9410f,-2.1684f,-37.2552f,  
	2.3346f,-3.1722f,-37.5676f,  3.9236f,-3.8715f,-38.5616f,  3.3102f,-4.8844f,-38.8729f,  4.9638f,-5.7072f,-39.6413f,  4.3419f,-6.7310f,-39.9511f,  
	6.0309f,-7.6636f,-40.4798f,  5.3993f,-8.6997f,-40.7871f,  7.0937f,-9.7260f,-41.0663f,  6.4513f,-10.7753f,-41.3699f,  8.1209f,-11.8772f,-41.3937f,  
	7.4652f,-12.9412f,-41.6924f,  9.0802f,-14.0986f,-41.4596f,  8.4094f,-15.1779f,-41.7519f,  9.9411f,-16.3693f,-41.2655f,  9.2532f,-17.4640f,-41.5498f,  
	10.6744f,-18.6663f,-40.8173f,  9.9665f,-19.7762f,-41.0920f,  11.2521f,-20.9657f,-40.1253f,  10.5211f,-22.0900f,-40.3890f,  11.6486f,-23.2422f,-39.2045f,  
	10.8924f,-24.3787f,-39.4556f,  11.8423f,-25.4691f,-38.0733f,  11.0580f,-26.6159f,-38.3104f,  11.8145f,-27.6202f,-36.7544f,  10.9999f,-28.7732f,-36.9768f,  
	11.5510f,-29.6680f,-35.2745f,  10.7040f,-30.8236f,-35.4808f,  11.0418f,-31.5869f,-33.6627f,  10.1604f,-32.7402f,-33.8530f,  10.2816f,-33.3516f,-31.9520f,  
	9.3651f,-34.4964f,-32.1260f,  9.2709f,-34.9381f,-30.1771f,  8.3186f,-36.0686f,-30.3357f,  8.0147f,-36.3253f,-28.3751f,  7.0277f,-37.4339f,-28.5196f,  
	6.5245f,-37.4940f,-26.5843f,  5.5037f,-38.5745f,-26.7158f,  4.8158f,-38.4299f,-24.8428f,  3.7642f,-39.4745f,-24.9636f,  2.9101f,-39.1216f,-23.1892f,  
	1.8313f,-40.1233f,-23.3019f,  14.2968f,27.9711f,-20.3627f,  14.8252f,26.9980f,-22.5751f,  15.4304f,28.0771f,-22.0077f,  16.1249f,26.9304f,-24.0944f,  
	16.7295f,28.0234f,-23.5281f,  17.5681f,26.7058f,-25.4615f,  18.1719f,27.8108f,-24.8978f,  19.1301f,26.3278f,-26.6528f,  19.7330f,27.4430f,-26.0935f,  
	20.7843f,25.8031f,-27.6481f,  21.3861f,26.9263f,-27.0947f,  22.5023f,25.1406f,-28.4302f,  23.1029f,26.2694f,-27.8842f,  24.2547f,24.3516f,-28.9859f,  
	24.8540f,25.4838f,-28.4485f,  26.0115f,23.4497f,-29.3055f,  26.6095f,24.5827f,-28.7781f,  27.7427f,22.4502f,-29.3837f,  28.3392f,23.5816f,-28.8672f,  
	29.4186f,21.3703f,-29.2191f,  30.0137f,22.4977f,-28.7143f,  31.0107f,20.2285f,-28.8145f,  31.6042f,21.3494f,-28.3220f,  32.4915f,19.0442f,-28.1768f,  
	33.0835f,20.1565f,-27.6971f,  33.8359f,17.8378f,-27.3170f,  34.4264f,18.9393f,-26.8502f,  35.0208f,16.6299f,-26.2497f,  35.6099f,17.7187f,-25.7959f,  
	36.0259f,15.4411f,-24.9932f,  36.6137f,16.5156f,-24.5521f,  36.8340f,14.2918f,-23.5691f,  37.4206f,15.3505f,-23.1402f,  37.4313f,13.2017f,-22.0016f,  
	38.0168f,14.2433f,-21.5842f,  37.8076f,12.1894f,-20.3176f,  38.3922f,13.2131f,-19.9110f,  37.9564f,11.2722f,-18.5460f,  38.5402f,12.2774f,-18.1489f,  
	37.8753f,10.4659f,-16.7169f,  38.4585f,11.4523f,-16.3283f,  37.5655f,9.7842f,-14.8618f,  38.1483f,10.7519f,-14.4803f,  37.0324f,9.2388f,-13.0124f,  
	37.6150f,10.1881f,-12.6364f,  36.2851f,8.8390f,-11.2003f,  36.8676f,9.7707f,-10.8283f,  35.3364f,8.5916f,-9.4565f,  35.9191f,9.5066f,-9.0868f,  
	35.3364f,8.5916f,-9.4565f,  34.7576f,7.6707f,-9.8242f,  33.6237f,7.5944f,-8.1778f,  33.0488f,6.6804f,-8.5437f,  31.7490f,6.7730f,-7.0258f,  
	31.1782f,5.8637f,-7.3901f,  29.7347f,6.1197f,-6.0289f,  29.1677f,5.2120f,-6.3927f,  27.6049f,5.6234f,-5.2135f,  27.0418f,4.7154f,-5.5767f,  
	25.3861f,5.2716f,-4.6013f,  24.8265f,4.3605f,-4.9641f,  23.1058f,5.0484f,-4.2103f,  22.5495f,4.1331f,-4.5723f,  20.7928f,4.9378f,-4.0539f,  
	20.2391f,4.0172f,-4.4144f,  18.4759f,4.9219f,-4.1409f,  17.9243f,3.9949f,-4.4997f,  16.1842f,4.9816f,-4.4761f,  15.6340f,4.0481f,-4.8323f,  
	13.9459f,5.0974f,-5.0595f,  13.3972f,4.1579f,-5.4123f,  11.7895f,5.2500f,-5.8863f,  11.2403f,4.3032f,-6.2359f,  9.7397f,5.4176f,-6.9489f,  
	9.1906f,4.4657f,-7.2941f,  7.8220f,5.5819f,-8.2340f,  7.2717f,4.6245f,-8.5750f,  6.0579f,5.7220f,-9.7257f,  5.5058f,4.7601f,-10.0624f,  
	4.4668f,5.8191f,-11.4044f,  3.9128f,4.8540f,-11.7366f,  3.0658f,5.8554f,-13.2472f,  2.5089f,4.8873f,-13.5753f,  1.8678f,5.8130f,-15.2288f,  
	1.3082f,4.8433f,-15.5530f,  0.8833f,5.6765f,-17.3214f,  0.3205f,4.7054f,-17.6421f,  0.1185f,5.4310f,-19.4954f,  -0.4480f,4.4589f,-19.8133f,  
	-0.4239f,5.0634f,-21.7201f,  -0.9941f,4.0907f,-22.0357f,  -0.7450f,4.5628f,-23.9636f,  -1.3190f,3.5896f,-24.2774f,  -0.8495f,3.9200f,-26.1938f,  
	-1.4275f,2.9458f,-26.5063f,  -0.7464f,3.1276f,-28.3785f,  -1.3285f,2.1517f,-28.6904f,  -0.4478f,2.1805f,-30.4864f,  -1.0344f,1.2022f,-30.7980f,  
	0.0302f,1.0761f,-32.4870f,  -0.5610f,0.0942f,-32.7986f,  0.6686f,-0.1860f,-34.3516f,  0.0721f,-1.1730f,-34.6632f,  1.4448f,-1.6038f,-36.0532f,  
	0.8432f,-2.5968f,-36.3648f,  2.3346f,-3.1722f,-37.5676f,  1.7265f,-4.1734f,-37.8787f,  3.3102f,-4.8844f,-38.8729f,  2.6942f,-5.8959f,-39.1831f,  
	4.3419f,-6.7310f,-39.9511f,  3.7171f,-7.7543f,-40.2594f,  5.3993f,-8.6997f,-40.7871f,  4.7647f,-9.7359f,-41.0925f,  6.4513f,-10.7753f,-41.3699f,  
	5.8042f,-11.8263f,-41.6712f,  7.4652f,-12.9412f,-41.6924f,  6.8040f,-14.0079f,-41.9882f,  8.4094f,-15.1779f,-41.7519f,  7.7321f,-16.2608f,-42.0404f,  
	9.2532f,-17.4640f,-41.5498f,  8.5569f,-18.5632f,-41.8294f,  9.9665f,-19.7762f,-41.0920f,  9.2479f,-20.8916f,-41.3612f,  10.5211f,-22.0900f,-40.3890f,  
	9.7781f,-23.2198f,-40.6461f,  10.8924f,-24.3787f,-39.4556f,  10.1218f,-25.5215f,-39.6989f,  11.0580f,-26.6159f,-38.3104f,  10.2572f,-27.7681f,-38.5389f,  
	10.9999f,-28.7732f,-36.9768f,  10.1666f,-29.9316f,-37.1889f,  10.7040f,-30.8236f,-35.4808f,  9.8356f,-31.9838f,-35.6763f,  10.1604f,-32.7402f,-33.8530f,  
	9.2558f,-33.8962f,-34.0313f,  9.3651f,-34.4964f,-32.1260f,  8.4230f,-35.6427f,-32.2876f,  8.3186f,-36.0686f,-30.3357f,  7.3395f,-37.1975f,-30.4815f,  
	7.0277f,-37.4339f,-28.5196f,  6.0119f,-38.5391f,-28.6504f,  5.5037f,-38.5745f,-26.7158f,  4.4535f,-39.6479f,-26.8336f,  3.7642f,-39.4745f,-24.9636f,  
	2.6825f,-40.5089f,-25.0707f,  1.8313f,-40.1233f,-23.3019f,  0.7236f,-41.1110f,-23.4017f,  14.8252f,26.9980f,-22.5751f,  15.5217f,25.8395f,-24.6410f,  
	16.1249f,26.9304f,-24.0944f,  16.9654f,25.6036f,-26.0057f,  17.5681f,26.7058f,-25.4615f,  18.5281f,25.2164f,-27.1932f,  19.1301f,26.3278f,-26.6528f,  
	20.1831f,24.6844f,-28.1831f,  20.7843f,25.8031f,-27.6481f,  21.9021f,24.0167f,-28.9587f,  22.5023f,25.1406f,-28.4302f,  23.6556f,23.2249f,-29.5065f,  
	24.2547f,24.3516f,-28.9859f,  25.4138f,22.3224f,-29.8173f,  26.0115f,23.4497f,-29.3055f,  27.1464f,21.3246f,-29.8856f,  27.7427f,22.4502f,-29.3837f,  
	28.8238f,20.2488f,-29.7104f,  29.4186f,21.3703f,-29.2191f,  30.4174f,19.1131f,-29.2945f,  31.0107f,20.2285f,-28.8145f,  31.8999f,17.9372f,-28.6453f,  
	32.4915f,19.0442f,-28.1768f,  33.2459f,16.7411f,-27.7737f,  33.8359f,17.8378f,-27.3170f,  34.4324f,15.5452f,-26.6946f,  35.0208f,16.6299f,-26.2497f,  
	35.4390f,14.3701f,-25.4266f,  36.0259f,15.4411f,-24.9932f,  36.2486f,13.2359f,-23.9913f,  36.8340f,14.2918f,-23.5691f,  36.8473f,12.1618f,-22.4132f,  
	37.4313f,13.2017f,-22.0016f,  37.2248f,11.1665f,-20.7195f,  37.8076f,12.1894f,-20.3176f,  37.3748f,10.2667f,-18.9390f,  37.9564f,11.2722f,-18.5460f,  
	37.2946f,9.4781f,-17.1023f,  37.8753f,10.4659f,-16.7169f,  36.9856f,8.8139f,-15.2407f,  37.5655f,9.7842f,-14.8618f,  36.4530f,8.2857f,-13.3861f,  
	37.0324f,9.2388f,-13.0124f,  35.7061f,7.9025f,-11.5702f,  36.2851f,8.8390f,-11.2003f,  34.7576f,7.6707f,-9.8242f,  35.3364f,8.5916f,-9.4565f,  
	34.7576f,7.6707f,-9.8242f,  34.1823f,6.7433f,-10.1909f,  33.0488f,6.6804f,-8.5437f,  32.4775f,5.7599f,-8.9078f,  31.1782f,5.8637f,-7.3901f,  
	30.6107f,4.9469f,-7.7527f,  29.1677f,5.2120f,-6.3927f,  28.6042f,4.2969f,-6.7540f,  27.0418f,4.7154f,-5.5767f,  26.4818f,3.7992f,-5.9373f,  
	24.8265f,4.3605f,-4.9641f,  24.2698f,3.4418f,-5.3237f,  22.5495f,4.1331f,-4.5723f,  21.9956f,3.2106f,-4.9304f,  20.2391f,4.0172f,-4.4144f,  
	19.6876f,3.0896f,-4.7709f,  17.9243f,3.9949f,-4.4997f,  17.3744f,3.0619f,-4.8537f,  15.6340f,4.0481f,-4.8323f,  15.0857f,3.1098f,-5.1832f,  
	13.3972f,4.1579f,-5.4123f,  12.8489f,3.2131f,-5.7600f,  11.2403f,4.3032f,-6.2359f,  10.6923f,3.3538f,-6.5795f,  9.1906f,4.4657f,-7.2941f,  
	8.6416f,3.5112f,-7.6337f,  7.2717f,4.6245f,-8.5750f,  6.7211f,3.6657f,-8.9104f,  5.5058f,4.7601f,-10.0624f,  4.9535f,3.7984f,-10.3933f,  
	3.9128f,4.8540f,-11.7366f,  3.3580f,3.8895f,-12.0635f,  2.5089f,4.8873f,-13.5753f,  1.9516f,3.9214f,-13.8981f,  1.3082f,4.8433f,-15.5530f,  
	0.7477f,3.8763f,-15.8723f,  0.3205f,4.7054f,-17.6421f,  -0.2437f,3.7377f,-17.9585f,  -0.4480f,4.4589f,-19.8133f,  -1.0157f,3.4909f,-20.1273f,  
	-0.9941f,4.0907f,-22.0357f,  -1.5654f,3.1229f,-22.3477f,  -1.3190f,3.5896f,-24.2774f,  -1.8941f,2.6214f,-24.5882f,  -1.4275f,2.9458f,-26.5063f,  
	-2.0067f,1.9767f,-26.8165f,  -1.3285f,2.1517f,-28.6904f,  -1.9119f,1.1811f,-29.0003f,  -1.0344f,1.2022f,-30.7980f,  -1.6221f,0.2289f,-31.1081f,  
	-0.5610f,0.0942f,-32.7986f,  -1.1536f,-0.8832f,-33.1090f,  0.0721f,-1.1730f,-34.6632f,  -0.5252f,-2.1554f,-34.9739f,  0.8432f,-2.5968f,-36.3648f,  
	0.2398f,-3.5866f,-36.6755f,  1.7265f,-4.1734f,-37.8787f,  1.1161f,-5.1725f,-38.1889f,  2.6942f,-5.8959f,-39.1831f,  2.0758f,-6.9060f,-39.4922f,  
	3.7171f,-7.7543f,-40.2594f,  3.0897f,-8.7769f,-40.5664f,  4.7647f,-9.7359f,-41.0925f,  4.1258f,-10.7731f,-41.3963f,  5.8042f,-11.8263f,-41.6712f,  
	5.1524f,-12.8792f,-41.9702f,  6.8040f,-14.0079f,-41.9882f,  6.1371f,-15.0774f,-42.2808f,  7.7321f,-16.2608f,-42.0404f,  7.0473f,-17.3476f,-42.3250f,  
	8.5569f,-18.5632f,-41.8294f,  7.8509f,-19.6676f,-42.1043f,  9.2479f,-20.8916f,-41.3612f,  8.5184f,-22.0123f,-41.6246f,  9.7781f,-23.2198f,-40.6461f,  
	9.0217f,-24.3560f,-40.8962f,  10.1218f,-25.5215f,-39.6989f,  9.3356f,-26.6700f,-39.9343f,  10.2572f,-27.7681f,-38.5389f,  9.4388f,-28.9262f,-38.7579f,  
	10.1666f,-29.9316f,-37.1889f,  9.3128f,-31.0954f,-37.3908f,  9.8356f,-31.9838f,-35.6763f,  8.9449f,-33.1476f,-35.8602f,  9.2558f,-33.8962f,-34.0313f,  
	8.3262f,-35.0548f,-34.1974f,  8.4230f,-35.6427f,-32.2876f,  7.4544f,-36.7886f,-32.4364f,  7.3395f,-37.1975f,-30.4815f,  6.3315f,-38.3245f,-30.6135f,  
	6.0119f,-38.5391f,-28.6504f,  4.9663f,-39.6390f,-28.7672f,  4.4535f,-39.6479f,-26.8336f,  3.3725f,-40.7131f,-26.9374f,  2.6825f,-40.5089f,-25.0707f,  
	1.5708f,-41.5312f,-25.1645f,  0.7236f,-41.1110f,-23.4017f,  -0.4155f,-42.0836f,-23.4872f,  15.5217f,25.8395f,-24.6410f,  16.3636f,24.5054f,-26.5309f,  
	16.9654f,25.6036f,-26.0057f,  17.9266f,24.1095f,-27.7151f,  18.5281f,25.2164f,-27.1932f,  19.5821f,23.5708f,-28.7005f,  20.1831f,24.6844f,-28.1831f,  
	21.3019f,22.8984f,-29.4703f,  21.9021f,24.0167f,-28.9587f,  23.0564f,22.1040f,-30.0112f,  23.6556f,23.2249f,-29.5065f,  24.8157f,21.2010f,-30.3141f,  
	25.4138f,22.3224f,-29.8173f,  26.5496f,20.2050f,-30.3737f,  27.1464f,21.3246f,-29.8856f,  28.2285f,19.1330f,-30.1890f,  28.8238f,20.2488f,-29.7104f,  
	29.8237f,18.0033f,-29.7633f,  30.4174f,19.1131f,-29.2945f,  31.3078f,16.8352f,-29.1036f,  31.8999f,17.9372f,-28.6453f,  32.6556f,15.6488f,-28.2215f,  
	33.2459f,16.7411f,-27.7737f,  33.8438f,14.4644f,-27.1319f,  34.4324f,15.5452f,-26.6946f,  34.8522f,13.3021f,-25.8534f,  35.4390f,14.3701f,-25.4266f,  
	35.6635f,12.1820f,-24.4081f,  36.2486f,13.2359f,-23.9913f,  36.2639f,11.1231f,-22.8205f,  36.8473f,12.1618f,-22.4132f,  36.6431f,10.1436f,-21.1179f,  
	37.2248f,11.1665f,-20.7195f,  36.7945f,9.2602f,-19.3293f,  37.3748f,10.2667f,-18.9390f,  36.7157f,8.4881f,-17.4855f,  37.2946f,9.4781f,-17.1023f,  
	36.4079f,7.8404f,-15.6179f,  36.9856f,8.8139f,-15.2407f,  35.8764f,7.3283f,-13.7585f,  36.4530f,8.2857f,-13.3861f,  35.1303f,6.9605f,-11.9391f,  
	35.7061f,7.9025f,-11.5702f,  34.1823f,6.7433f,-10.1909f,  34.7576f,7.6707f,-9.8242f,  34.1823f,6.7433f,-10.1909f,  33.6103f,5.8103f,-10.5558f,  
	32.4775f,5.7599f,-8.9078f,  31.9092f,4.8327f,-9.2703f,  30.6107f,4.9469f,-7.7527f,  30.0464f,4.0234f,-8.1132f,  28.6042f,4.2969f,-6.7540f,  
	28.0433f,3.3742f,-7.1134f,  26.4818f,3.7992f,-5.9373f,  25.9244f,2.8756f,-6.2953f,  24.2698f,3.4418f,-5.3237f,  23.7154f,2.5160f,-5.6801f,  
	21.9956f,3.2106f,-4.9304f,  21.4437f,2.2810f,-5.2851f,  19.6876f,3.0896f,-4.7709f,  19.1375f,2.1558f,-5.1231f,  17.3744f,3.0619f,-4.8537f,  
	16.8262f,2.1237f,-5.2029f,  15.0857f,3.1098f,-5.1832f,  14.5378f,2.1660f,-5.5294f,  12.8489f,3.2131f,-5.7600f,  12.3014f,2.2652f,-6.1024f,  
	10.6923f,3.3538f,-6.5795f,  10.1441f,2.4012f,-6.9179f,  8.6416f,3.5112f,-7.6337f,  8.0922f,2.5546f,-7.9680f,  6.7211f,3.6657f,-8.9104f,  
	6.1703f,2.7066f,-9.2403f,  4.9535f,3.7984f,-10.3933f,  4.4003f,2.8366f,-10.7192f,  3.3580f,3.8895f,-12.0635f,  2.8023f,2.9267f,-12.3852f,  
	1.9516f,3.9214f,-13.8981f,  1.3930f,2.9578f,-14.2163f,  0.7477f,3.8763f,-15.8723f,  0.1856f,2.9122f,-16.1875f,  -0.2437f,3.7377f,-17.9585f,  
	-0.8092f,2.7739f,-18.2711f,  -1.0157f,3.4909f,-20.1273f,  -1.5846f,2.5279f,-20.4378f,  -1.5654f,3.1229f,-22.3477f,  -2.1381f,2.1601f,-22.6570f,  
	-1.8941f,2.6214f,-24.5882f,  -2.4708f,1.6585f,-24.8967f,  -2.0067f,1.9767f,-26.8165f,  -2.5873f,1.0131f,-27.1248f,  -1.9119f,1.1811f,-29.0003f,  
	-2.4966f,0.2158f,-29.3088f,  -1.6221f,0.2289f,-31.1081f,  -2.2114f,-0.7394f,-31.4170f,  -1.1536f,-0.8832f,-33.1090f,  -1.7472f,-1.8554f,-33.4186f,  
	-0.5252f,-2.1554f,-34.9739f,  -1.1242f,-3.1339f,-35.2839f,  0.2398f,-3.5866f,-36.6755f,  -0.3655f,-4.5733f,-36.9856f,  1.1161f,-5.1725f,-38.1889f,  
	0.5034f,-6.1694f,-38.4986f,  2.0758f,-6.9060f,-39.4922f,  1.4551f,-7.9145f,-39.8005f,  3.0897f,-8.7769f,-40.5664f,  2.4586f,-9.7995f,-40.8723f,  
	4.1258f,-10.7731f,-41.3963f,  3.4828f,-11.8112f,-41.6983f,  5.1524f,-12.8792f,-41.9702f,  4.4956f,-13.9341f,-42.2668f,  6.1371f,-15.0774f,-42.2808f,  
	5.4636f,-16.1502f,-42.5703f,  7.0473f,-17.3476f,-42.3250f,  6.3538f,-18.4390f,-42.6057f,  7.8509f,-19.6676f,-42.1043f,  7.1349f,-20.7769f,-42.3741f,  
	8.5184f,-22.0123f,-41.6246f,  7.7767f,-23.1391f,-41.8817f,  9.0217f,-24.3560f,-40.8962f,  8.2509f,-25.4978f,-41.1390f,  9.3356f,-26.6700f,-39.9343f,  
	8.5330f,-27.8246f,-40.1608f,  9.4388f,-28.9262f,-38.7579f,  8.6009f,-30.0902f,-38.9672f,  9.3128f,-31.0954f,-37.3908f,  8.4377f,-32.2635f,-37.5816f,  
	8.9449f,-33.1476f,-35.8602f,  8.0299f,-34.3152f,-36.0323f,  8.3262f,-35.0548f,-34.1974f,  7.3706f,-36.2145f,-34.3509f,  7.4544f,-36.7886f,-32.4364f,  
	6.4572f,-37.9343f,-32.5715f,  6.3315f,-38.3245f,-30.6135f,  5.2938f,-39.4478f,-30.7315f,  4.9663f,-39.6390f,-28.7672f,  3.8895f,-40.7324f,-28.8699f,  
	3.3725f,-40.7131f,-26.9374f,  2.2605f,-41.7680f,-27.0274f,  1.5708f,-41.5312f,-25.1645f,  0.4268f,-42.5402f,-25.2434f,  -0.4155f,-42.0836f,-23.4872f,  
	-1.5853f,-43.0390f,-23.5588f,  16.3636f,24.5054f,-26.5309f,  17.3257f,23.0066f,-28.2202f,  17.9266f,24.1095f,-27.7151f,  18.9816f,22.4616f,-29.2016f,  
	19.5821f,23.5708f,-28.7005f,  20.7019f,21.7850f,-29.9664f,  21.3019f,22.8984f,-29.4703f,  22.4572f,20.9882f,-30.5013f,  23.0564f,22.1040f,-30.0112f,  
	24.2175f,20.0849f,-30.7972f,  24.8157f,21.2010f,-30.3141f,  25.9526f,19.0906f,-30.8491f,  26.5496f,20.2050f,-30.3737f,  27.6330f,18.0223f,-30.6560f,  
	28.2285f,19.1330f,-30.1890f,  29.2297f,16.8983f,-30.2213f,  29.8237f,18.0033f,-29.7633f,  30.7156f,15.7377f,-29.5524f,  31.3078f,16.8352f,-29.1036f,  
	32.0652f,14.5606f,-28.6607f,  32.6556f,15.6488f,-28.2215f,  33.2553f,13.3869f,-27.5616f,  33.8438f,14.4644f,-27.1319f,  34.2657f,12.2368f,-26.2737f,  
	34.8522f,13.3021f,-25.8534f,  35.0790f,11.1300f,-24.8192f,  35.6635f,12.1820f,-24.4081f,  35.6814f,10.0854f,-23.2230f,  36.2639f,11.1231f,-22.8205f,  
	36.0625f,9.1208f,-21.5123f,  36.6431f,10.1436f,-21.1179f,  36.2157f,8.2528f,-19.7164f,  36.7945f,9.2602f,-19.3293f,  36.1386f,7.4963f,-17.8660f,  
	36.7157f,8.4881f,-17.4855f,  35.8323f,6.8641f,-15.9929f,  36.4079f,7.8404f,-15.6179f,  35.3022f,6.3671f,-14.1290f,  35.8764f,7.3283f,-13.7585f,  
	34.5572f,6.0138f,-12.3062f,  35.1303f,6.9605f,-11.9391f,  33.6103f,5.8103f,-10.5558f,  34.1823f,6.7433f,-10.1909f,  33.6103f,5.8103f,-10.5558f,  
	33.0410f,4.8714f,-10.9194f,  31.9092f,4.8327f,-9.2703f,  31.3438f,3.8994f,-9.6309f,  30.0464f,4.0234f,-8.1132f,  29.4844f,3.0927f,-8.4721f,  
	28.0433f,3.3742f,-7.1134f,  27.4848f,2.4443f,-7.4705f,  25.9244f,2.8756f,-6.2953f,  25.3690f,1.9452f,-6.6506f,  23.7154f,2.5160f,-5.6801f,  
	23.1626f,1.5831f,-6.0335f,  21.4437f,2.2810f,-5.2851f,  20.8930f,1.3452f,-5.6360f,  19.1375f,2.1558f,-5.1231f,  18.5889f,1.2165f,-5.4712f,  
	16.8262f,2.1237f,-5.2029f,  16.2782f,1.1797f,-5.5481f,  14.5378f,2.1660f,-5.5294f,  13.9904f,1.2184f,-5.8708f,  12.3014f,2.2652f,-6.1024f,  
	11.7537f,1.3135f,-6.4400f,  10.1441f,2.4012f,-6.9179f,  9.5954f,1.4460f,-7.2515f,  8.0922f,2.5546f,-7.9680f,  7.5423f,1.5973f,-8.2972f,  
	6.1703f,2.7066f,-9.2403f,  5.6182f,1.7468f,-9.5654f,  4.4003f,2.8366f,-10.7192f,  3.8460f,1.8762f,-11.0402f,  2.8023f,2.9267f,-12.3852f,  
	2.2452f,1.9657f,-12.7026f,  1.3930f,2.9578f,-14.2163f,  0.8324f,1.9967f,-14.5305f,  0.1856f,2.9122f,-16.1875f,  -0.3782f,1.9518f,-16.4990f,  
	-0.8092f,2.7739f,-18.2711f,  -1.3763f,1.8148f,-18.5803f,  -1.5846f,2.5279f,-20.4378f,  -2.1554f,1.5697f,-20.7456f,  -2.1381f,2.1601f,-22.6570f,  
	-2.7127f,1.2025f,-22.9639f,  -2.4708f,1.6585f,-24.8967f,  -3.0491f,0.7012f,-25.2034f,  -2.5873f,1.0131f,-27.1248f,  -3.1696f,0.0552f,-27.4318f,  
	-2.4966f,0.2158f,-29.3088f,  -3.0830f,-0.7440f,-29.6164f,  -2.2114f,-0.7394f,-31.4170f,  -2.8017f,-1.7019f,-31.7255f,  -1.7472f,-1.8554f,-33.4186f,  
	-2.3425f,-2.8230f,-33.7278f,  -1.1242f,-3.1339f,-35.2839f,  -1.7252f,-4.1085f,-35.5938f,  -0.3655f,-4.5733f,-36.9856f,  -0.9730f,-5.5570f,-37.2956f,  
	0.5034f,-6.1694f,-38.4986f,  -0.1112f,-7.1637f,-38.8080f,  1.4551f,-7.9145f,-39.8005f,  0.8311f,-8.9222f,-40.1083f,  2.4586f,-9.7995f,-40.8723f,  
	1.8239f,-10.8221f,-41.1771f,  3.4828f,-11.8112f,-41.6983f,  2.8355f,-12.8503f,-41.9987f,  4.4956f,-13.9341f,-42.2668f,  3.8330f,-14.9913f,-42.5611f,  
	5.4636f,-16.1502f,-42.5703f,  4.7824f,-17.2267f,-42.8567f,  6.3538f,-18.4390f,-42.6057f,  5.6514f,-19.5346f,-42.8821f,  7.1349f,-20.7769f,-42.3741f,  
	6.4080f,-21.8918f,-42.6386f,  7.7767f,-23.1391f,-41.8817f,  7.0215f,-24.2714f,-42.1324f,  8.2509f,-25.4978f,-41.1390f,  7.4649f,-26.6457f,-41.3737f,  
	8.5330f,-27.8246f,-40.1608f,  7.7121f,-28.9854f,-40.3784f,  8.6009f,-30.0902f,-38.9672f,  7.7428f,-31.2591f,-39.1660f,  8.4377f,-32.2635f,-37.5816f,  
	7.5390f,-33.4363f,-37.7611f,  8.0299f,-34.3152f,-36.0323f,  7.0895f,-35.4848f,-36.1923f,  7.3706f,-36.2145f,-34.3509f,  6.3868f,-37.3751f,-34.4911f,  
	6.4572f,-37.9343f,-32.5715f,  5.4304f,-39.0777f,-32.6927f,  5.2938f,-39.4478f,-30.7315f,  4.2245f,-40.5665f,-30.8351f,  3.8895f,-40.7324f,-28.8699f,  
	2.7808f,-41.8174f,-28.9585f,  2.2605f,-41.7680f,-27.0274f,  1.1152f,-42.8114f,-27.1021f,  0.4268f,-42.5402f,-25.2434f,  -0.7497f,-43.5338f,-25.3078f,  
	-1.5853f,-43.0390f,-23.5588f,  -2.7846f,-43.9746f,-23.6173f,  17.3257f,23.0066f,-28.2202f,  18.3814f,21.3577f,-29.6872f,  18.9816f,22.4616f,-29.2016f,  
	20.1021f,20.6770f,-30.4476f,  20.7019f,21.7850f,-29.9664f,  21.8580f,19.8781f,-30.9773f,  22.4572f,20.9882f,-30.5013f,  23.6192f,18.9746f,-31.2671f,  
	24.2175f,20.0849f,-30.7972f,  25.3555f,17.9820f,-31.3122f,  25.9526f,19.0906f,-30.8491f,  27.0373f,16.9173f,-31.1117f,  27.6330f,18.0223f,-30.6560f,  
	28.6356f,15.7986f,-30.6691f,  29.2297f,16.8983f,-30.2213f,  30.1233f,14.6451f,-29.9919f,  30.7156f,15.7377f,-29.5524f,  31.4748f,13.4766f,-29.0918f,  
	32.0652f,14.5606f,-28.6607f,  32.6670f,12.3131f,-27.9841f,  33.2553f,13.3869f,-27.5616f,  33.6795f,11.1744f,-26.6879f,  34.2657f,12.2368f,-26.2737f,  
	34.4950f,10.0801f,-25.2252f,  35.0790f,11.1300f,-24.8192f,  35.0995f,9.0488f,-23.6211f,  35.6814f,10.0854f,-23.2230f,  35.4827f,8.0982f,-21.9030f,  
	36.0625f,9.1208f,-21.5123f,  35.6381f,7.2446f,-20.1005f,  36.2157f,8.2528f,-19.7164f,  35.5629f,6.5026f,-18.2441f,  36.1386f,7.4963f,-17.8660f,  
	35.2585f,5.8849f,-16.3659f,  35.8323f,6.8641f,-15.9929f,  34.7301f,5.4020f,-14.4978f,  35.3022f,6.3671f,-14.1290f,  33.9867f,5.0622f,-12.6719f,  
	34.5572f,6.0138f,-12.3062f,  33.0410f,4.8714f,-10.9194f,  33.6103f,5.8103f,-10.5558f,  33.0410f,4.8714f,-10.9194f,  32.4741f,3.9271f,-11.2812f,  
	31.3438f,3.8994f,-9.6309f,  30.7804f,2.9595f,-9.9902f,  29.4844f,3.0927f,-8.4721f,  28.9245f,2.1553f,-8.8291f,  27.4848f,2.4443f,-7.4705f,  
	26.9282f,1.5079f,-7.8253f,  25.3690f,1.9452f,-6.6506f,  24.8151f,1.0077f,-7.0033f,  23.1626f,1.5831f,-6.0335f,  22.6110f,0.6440f,-6.3837f,  
	20.8930f,1.3452f,-5.6360f,  20.3436f,0.4036f,-5.9834f,  18.5889f,1.2165f,-5.4712f,  18.0403f,0.2711f,-5.8157f,  16.2782f,1.1797f,-5.5481f,  
	15.7305f,0.2315f,-5.8889f,  13.9904f,1.2184f,-5.8708f,  13.4426f,0.2667f,-6.2079f,  11.7537f,1.3135f,-6.4400f,  11.2051f,0.3588f,-6.7731f,  
	9.5954f,1.4460f,-7.2515f,  9.0459f,0.4895f,-7.5802f,  7.5423f,1.5973f,-8.2972f,  6.9907f,0.6386f,-8.6219f,  5.6182f,1.7468f,-9.5654f,  
	5.0647f,0.7878f,-9.8859f,  3.8460f,1.8762f,-11.0402f,  3.2898f,0.9170f,-11.3569f,  2.2452f,1.9657f,-12.7026f,  1.6858f,1.0066f,-13.0160f,  
	0.8324f,1.9967f,-14.5305f,  0.2698f,1.0387f,-14.8411f,  -0.3782f,1.9518f,-16.4990f,  -0.9440f,0.9958f,-16.8071f,  -1.3763f,1.8148f,-18.5803f,  
	-1.9456f,0.8602f,-18.8869f,  -2.1554f,1.5697f,-20.7456f,  -2.7284f,0.6166f,-21.0512f,  -2.7127f,1.2025f,-22.9639f,  -3.2893f,0.2506f,-23.2692f,  
	-3.0491f,0.7012f,-25.2034f,  -3.6294f,-0.2503f,-25.5089f,  -3.1696f,0.0552f,-27.4318f,  -3.7537f,-0.8970f,-27.7379f,  -3.0830f,-0.7440f,-29.6164f,  
	-3.6708f,-1.6976f,-29.9236f,  -2.8017f,-1.7019f,-31.7255f,  -3.3940f,-2.6594f,-32.0338f,  -2.3425f,-2.8230f,-33.7278f,  -2.9398f,-3.7861f,-34.0372f,  
	-1.7252f,-4.1085f,-35.5938f,  -2.3283f,-5.0793f,-35.9038f,  -0.9730f,-5.5570f,-37.2956f,  -1.5823f,-6.5372f,-37.6058f,  -0.1112f,-7.1637f,-38.8080f,  
	-0.7288f,-8.1562f,-39.1173f,  0.8311f,-8.9222f,-40.1083f,  0.2039f,-9.9288f,-40.4156f,  1.8239f,-10.8221f,-41.1771f,  1.1854f,-11.8447f,-41.4810f,  
	2.8355f,-12.8503f,-41.9987f,  2.1832f,-13.8909f,-42.2975f,  3.8330f,-14.9913f,-42.5611f,  3.1634f,-16.0515f,-42.8530f,  4.7824f,-17.2267f,-42.8567f,  
	4.0931f,-18.3068f,-43.1396f,  5.6514f,-19.5346f,-42.8821f,  4.9390f,-20.6353f,-43.1540f,  6.4080f,-21.8918f,-42.6386f,  5.6687f,-23.0118f,-42.8975f,  
	7.0215f,-24.2714f,-42.1324f,  6.2525f,-25.4096f,-42.3759f,  7.4649f,-26.6457f,-41.3737f,  6.6616f,-27.7999f,-41.6004f,  7.7121f,-28.9854f,-40.3784f,  
	6.8721f,-30.1514f,-40.5863f,  7.7428f,-31.2591f,-39.1660f,  6.8621f,-32.4332f,-39.3542f,  7.5390f,-33.4363f,-37.7611f,  6.6158f,-34.6120f,-37.9290f,  
	7.0895f,-35.4848f,-36.1923f,  6.1215f,-36.6566f,-36.3391f,  6.3868f,-37.3751f,-34.4911f,  5.3737f,-38.5350f,-34.6174f,  5.4304f,-39.0777f,-32.6927f,  
	4.3719f,-40.2182f,-32.7996f,  4.2245f,-40.5665f,-30.8351f,  3.1228f,-41.6785f,-30.9247f,  2.7808f,-41.8174f,-28.9585f,  1.6382f,-42.8926f,-29.0314f,  
	1.1152f,-42.8114f,-27.1021f,  -0.0638f,-43.8413f,-27.1618f,  -0.7497f,-43.5338f,-25.3078f,  -1.9576f,-44.5093f,-25.3586f,  -2.7846f,-43.9746f,-23.6173f,  
	-4.0143f,-44.8886f,-23.6620f,  18.3814f,21.3577f,-29.6872f,  19.5023f,19.5739f,-30.9152f,  20.1021f,20.6770f,-30.4476f,  21.2587f,18.7730f,-31.4404f,  
	21.8580f,19.8781f,-30.9773f,  23.0206f,17.8693f,-31.7249f,  23.6192f,18.9746f,-31.2671f,  24.7580f,16.8783f,-31.7641f,  25.3555f,17.9820f,-31.3122f,  
	26.4410f,15.8169f,-31.5571f,  27.0373f,16.9173f,-31.1117f,  28.0410f,14.7032f,-31.1075f,  28.6356f,15.7986f,-30.6691f,  29.5304f,13.5564f,-30.4231f,  
	30.1233f,14.6451f,-29.9919f,  30.8839f,12.3961f,-29.5154f,  31.4748f,13.4766f,-29.0918f,  32.0783f,11.2421f,-28.4002f,  32.6670f,12.3131f,-27.9841f,  
	33.0931f,10.1141f,-27.0963f,  33.6795f,11.1744f,-26.6879f,  33.9110f,9.0315f,-25.6263f,  34.4950f,10.0801f,-25.2252f,  34.5179f,8.0127f,-24.0151f,  
	35.0995f,9.0488f,-23.6211f,  34.9036f,7.0753f,-22.2904f,  35.4827f,8.0982f,-21.9030f,  35.0613f,6.2352f,-20.4816f,  35.6381f,7.2446f,-20.1005f,  
	34.9884f,5.5069f,-18.6198f,  35.5629f,6.5026f,-18.2441f,  34.6862f,4.9027f,-16.7368f,  35.2585f,5.8849f,-16.3659f,  34.1598f,4.4331f,-14.8648f,  
	34.7301f,5.4020f,-14.4978f,  33.4182f,4.1060f,-13.0358f,  33.9867f,5.0622f,-12.6719f,  32.4741f,3.9271f,-11.2812f,  33.0410f,4.8714f,-10.9194f,  
	32.4741f,3.9271f,-11.2812f,  31.9091f,2.9770f,-11.6418f,  30.7804f,2.9595f,-9.9902f,  30.2188f,2.0135f,-10.3480f,  28.9245f,2.1553f,-8.8291f,  
	28.3662f,1.2118f,-9.1841f,  26.9282f,1.5079f,-7.8253f,  26.3727f,0.5648f,-8.1780f,  24.8151f,1.0077f,-7.0033f,  24.2621f,0.0642f,-7.3533f,  
	22.6110f,0.6440f,-6.3837f,  22.0603f,-0.3010f,-6.7309f,  20.3436f,0.4036f,-5.9834f,  19.7939f,-0.5442f,-6.3277f,  18.0403f,0.2711f,-5.8157f,  
	17.4918f,-0.6787f,-6.1563f,  15.7305f,0.2315f,-5.8889f,  15.1821f,-0.7212f,-6.2258f,  13.4426f,0.2667f,-6.2079f,  12.8937f,-0.6885f,-6.5409f,  
	11.2051f,0.3588f,-6.7731f,  10.6555f,-0.5979f,-7.1017f,  9.0459f,0.4895f,-7.5802f,  8.4945f,-0.4689f,-7.9048f,  6.9907f,0.6386f,-8.6219f,  
	6.4376f,-0.3198f,-8.9422f,  5.0647f,0.7878f,-9.8859f,  4.5090f,-0.1705f,-10.2023f,  3.2898f,0.9170f,-11.3569f,  2.7310f,-0.0410f,-11.6699f,  
	1.6858f,1.0066f,-13.0160f,  1.1239f,0.0502f,-13.3260f,  0.2698f,1.0387f,-14.8411f,  -0.2951f,0.0848f,-15.1484f,  -0.9440f,0.9958f,-16.8071f,  
	-1.5124f,0.0439f,-17.1126f,  -1.9456f,0.8602f,-18.8869f,  -2.5176f,-0.0894f,-19.1913f,  -2.7284f,0.6166f,-21.0512f,  -3.3038f,-0.3309f,-21.3551f,  
	-3.2893f,0.2506f,-23.2692f,  -3.8683f,-0.6954f,-23.5732f,  -3.6294f,-0.2503f,-25.5089f,  -4.2120f,-1.1958f,-25.8136f,  -3.7537f,-0.8970f,-27.7379f,  
	-4.3396f,-1.8426f,-28.0438f,  -3.6708f,-1.6976f,-29.9236f,  -4.2608f,-2.6457f,-30.2309f,  -3.3940f,-2.6594f,-32.0338f,  -3.9884f,-3.6116f,-32.3425f,  
	-2.9398f,-3.7861f,-34.0372f,  -3.5392f,-4.7445f,-34.3470f,  -2.3283f,-5.0793f,-35.9038f,  -2.9331f,-6.0457f,-36.2144f,  -1.5823f,-6.5372f,-37.6058f,  
	-2.1945f,-7.5146f,-37.9163f,  -0.7288f,-8.1562f,-39.1173f,  -1.3493f,-9.1467f,-39.4268f,  0.2039f,-9.9288f,-40.4156f,  -0.4265f,-10.9344f,-40.7227f,  
	1.1854f,-11.8447f,-41.4810f,  0.5426f,-12.8678f,-41.7841f,  2.1832f,-13.8909f,-42.2975f,  1.5247f,-14.9335f,-42.5948f,  3.1634f,-16.0515f,-42.8530f,  
	2.4868f,-17.1144f,-43.1424f,  4.0931f,-18.3068f,-43.1396f,  3.3950f,-19.3912f,-43.4190f,  4.9390f,-20.6353f,-43.1540f,  4.2154f,-21.7407f,-43.4212f,  
	5.6687f,-23.0118f,-42.8975f,  4.9167f,-24.1374f,-43.1501f,  6.2525f,-25.4096f,-42.3759f,  5.4675f,-26.5540f,-42.6122f,  6.6616f,-27.7999f,-41.6004f,  
	5.8406f,-28.9595f,-41.8181f,  6.8721f,-30.1514f,-40.5863f,  6.0107f,-31.3232f,-40.7842f,  6.8621f,-32.4332f,-39.3542f,  5.9578f,-33.6112f,-39.5313f,  
	6.6158f,-34.6120f,-37.9290f,  5.6659f,-35.7910f,-38.0842f,  6.1215f,-36.6566f,-36.3391f,  5.1248f,-37.8290f,-36.4726f,  5.3737f,-38.5350f,-34.6174f,  
	4.3291f,-39.6935f,-34.7296f,  4.3719f,-40.2182f,-32.7996f,  3.2805f,-41.3537f,-32.8923f,  3.1228f,-41.6785f,-30.9247f,  1.9867f,-42.7825f,-30.9982f,  
	1.6382f,-42.8926f,-29.0314f,  0.4606f,-43.9563f,-29.0889f,  -0.0638f,-43.8413f,-27.1618f,  -1.2761f,-44.8549f,-27.2075f,  -1.9576f,-44.5093f,-25.3586f,  
	-3.1979f,-45.4648f,-25.3951f,  -4.0143f,-44.8886f,-23.6620f,  -5.2787f,-45.7799f,-23.6901f,  19.5023f,19.5739f,-30.9152f,  20.6591f,17.6735f,-31.8910f,  
	21.2587f,18.7730f,-31.4404f,  22.4217f,16.7695f,-32.1712f,  23.0206f,17.8693f,-31.7249f,  24.1599f,15.7799f,-32.2053f,  24.7580f,16.8783f,-31.7641f,  
	25.8442f,14.7215f,-31.9927f,  26.4410f,15.8169f,-31.5571f,  27.4457f,13.6126f,-31.5371f,  28.0410f,14.7032f,-31.1075f,  28.9369f,12.4719f,-30.8463f,  
	29.5304f,13.5564f,-30.4231f,  30.2925f,11.3191f,-29.9321f,  30.8839f,12.3961f,-29.5154f,  31.4891f,10.1739f,-28.8102f,  32.0783f,11.2421f,-28.4002f,  
	32.5062f,9.0559f,-27.4996f,  33.0931f,10.1141f,-27.0963f,  33.3266f,7.9842f,-26.0230f,  33.9110f,9.0315f,-25.6263f,  33.9362f,6.9771f,-24.4055f,  
	34.5179f,8.0127f,-24.0151f,  34.3245f,6.0519f,-22.6748f,  34.9036f,7.0753f,-22.2904f,  34.4848f,5.2244f,-20.8605f,  35.0613f,6.2352f,-20.4816f,  
	34.4145f,4.5088f,-18.9937f,  34.9884f,5.5069f,-18.6198f,  34.1148f,3.9173f,-17.1062f,  34.6862f,4.9027f,-16.7368f,  33.5907f,3.4600f,-15.2305f,  
	34.1598f,4.4331f,-14.8648f,  32.8513f,3.1448f,-13.3986f,  33.4182f,4.1060f,-13.0358f,  31.9091f,2.9770f,-11.6418f,  32.4741f,3.9271f,-11.2812f,  
	31.9091f,2.9770f,-11.6418f,  31.3455f,2.0213f,-12.0010f,  30.2188f,2.0135f,-10.3480f,  29.6585f,1.0618f,-10.7038f,  28.3662f,1.2118f,-9.1841f,  
	27.8088f,0.2619f,-9.5374f,  26.3727f,0.5648f,-8.1780f,  25.8179f,-0.3842f,-8.5282f,  24.2621f,0.0642f,-7.3533f,  23.7096f,-0.8853f,-7.7007f,  
	22.0603f,-0.3010f,-6.7309f,  21.5092f,-1.2522f,-7.0752f,  19.7939f,-0.5442f,-6.3277f,  19.2442f,-1.4967f,-6.6684f,  17.4918f,-0.6787f,-6.1563f,  
	16.9423f,-1.6334f,-6.4934f,  15.1821f,-0.7212f,-6.2258f,  14.6324f,-1.6779f,-6.5590f,  12.8937f,-0.6885f,-6.5409f,  12.3435f,-1.6461f,-6.8697f,  
	10.6555f,-0.5979f,-7.1017f,  10.1037f,-1.5569f,-7.4264f,  8.4945f,-0.4689f,-7.9048f,  7.9411f,-1.4277f,-8.2252f,  6.4376f,-0.3198f,-8.9422f,  
	5.8818f,-1.2782f,-9.2585f,  4.5090f,-0.1705f,-10.2023f,  3.9503f,-1.1282f,-10.5151f,  2.7310f,-0.0410f,-11.6699f,  2.1693f,-0.9967f,-11.9795f,  
	1.1239f,0.0502f,-13.3260f,  0.5593f,-0.9024f,-13.6327f,  -0.2951f,0.0848f,-15.1484f,  -0.8631f,-0.8652f,-15.4531f,  -1.5124f,0.0439f,-17.1126f,  
	-2.0839f,-0.9031f,-17.4160f,  -2.5176f,-0.0894f,-19.1913f,  -3.0925f,-1.0335f,-19.4939f,  -3.3038f,-0.3309f,-21.3551f,  -3.8821f,-1.2725f,-21.6578f,  
	-3.8683f,-0.6954f,-23.5732f,  -4.4499f,-1.6352f,-23.8765f,  -4.2120f,-1.1958f,-25.8136f,  -4.7967f,-2.1344f,-26.1182f,  -4.3396f,-1.8426f,-28.0438f,  
	-4.9281f,-2.7823f,-28.3499f,  -4.2608f,-2.6457f,-30.2309f,  -4.8532f,-3.5879f,-30.5386f,  -3.9884f,-3.6116f,-32.3425f,  -4.5851f,-4.5583f,-32.6518f,  
	-3.5392f,-4.7445f,-34.3470f,  -4.1405f,-5.6977f,-34.6578f,  -2.9331f,-6.0457f,-36.2144f,  -3.5408f,-7.0085f,-36.5258f,  -2.1945f,-7.5146f,-37.9163f,  
	-2.8093f,-8.4890f,-38.2275f,  -1.3493f,-9.1467f,-39.4268f,  -1.9728f,-10.1351f,-39.7366f,  -0.4265f,-10.9344f,-40.7227f,  -1.0609f,-11.9394f,-41.0296f,  
	0.5426f,-12.8678f,-41.7841f,  -0.1058f,-13.8918f,-42.0864f,  1.5247f,-14.9335f,-42.5948f,  0.8598f,-15.9779f,-42.8903f,  2.4868f,-17.1144f,-43.1424f,  
	1.8022f,-18.1808f,-43.4291f,  3.3950f,-19.3912f,-43.4190f,  2.6867f,-20.4797f,-43.6946f,  4.2154f,-21.7407f,-43.4212f,  3.4804f,-22.8510f,-43.6832f,  
	4.9167f,-24.1374f,-43.1501f,  4.1500f,-25.2690f,-43.3965f,  5.4675f,-26.5540f,-42.6122f,  4.6659f,-27.7038f,-42.8404f,  5.8406f,-28.9595f,-41.8181f,  
	4.9993f,-30.1252f,-42.0268f,  6.0107f,-31.3232f,-40.7842f,  5.1266f,-32.4993f,-40.9718f,  5.9578f,-33.6112f,-39.5313f,  5.0277f,-34.7932f,-39.6964f,  
	5.6659f,-35.7910f,-38.0842f,  4.6878f,-36.9717f,-38.2265f,  5.1248f,-37.8290f,-36.4726f,  4.0969f,-39.0013f,-36.5921f,  4.3291f,-39.6935f,-34.7296f,  
	3.2515f,-40.8486f,-34.8278f,  3.2805f,-41.3537f,-32.8923f,  2.1547f,-42.4829f,-32.9690f,  1.9867f,-42.7825f,-30.9982f,  0.8147f,-43.8768f,-31.0562f,  
	0.4606f,-43.9563f,-29.0889f,  -0.7518f,-45.0055f,-29.1322f,  -1.2761f,-44.8549f,-27.2075f,  -2.5226f,-45.8501f,-27.2383f,  -3.1979f,-45.4648f,-25.3951f,  
	-4.4752f,-46.3996f,-25.4142f,  -5.2787f,-45.7799f,-23.6901f,  -6.5725f,-46.6451f,-23.7048f,  20.6591f,17.6735f,-31.8910f,  21.8221f,15.6745f,-32.6067f,  
	22.4217f,16.7695f,-32.1712f,  23.5612f,14.6861f,-32.6365f,  24.1599f,15.7799f,-32.2053f,  25.2466f,13.6305f,-32.4192f,  25.8442f,14.7215f,-31.9927f,  
	26.8495f,12.5258f,-31.9585f,  27.4457f,13.6126f,-31.5371f,  28.3425f,11.3908f,-31.2622f,  28.9369f,12.4719f,-30.8463f,  29.7001f,10.2449f,-30.3423f,  
	30.2925f,11.3191f,-29.9321f,  30.8989f,9.1079f,-29.2145f,  31.4891f,10.1739f,-28.8102f,  31.9186f,7.9991f,-27.8981f,  32.5062f,9.0559f,-27.4996f,  
	32.7416f,6.9375f,-26.4156f,  33.3266f,7.9842f,-26.0230f,  33.3540f,5.9413f,-24.7924f,  33.9362f,6.9771f,-24.4055f,  33.7451f,5.0275f,-23.0563f,  
	34.3245f,6.0519f,-22.6748f,  33.9083f,4.2118f,-21.2370f,  34.4848f,5.2244f,-20.8605f,  33.8408f,3.5081f,-19.3655f,  34.4145f,4.5088f,-18.9937f,  
	33.5438f,2.9284f,-17.4740f,  34.1148f,3.9173f,-17.1062f,  33.0224f,2.4827f,-15.5947f,  33.5907f,3.4600f,-15.2305f,  32.2854f,2.1786f,-13.7599f,  
	32.8513f,3.1448f,-13.3986f,  31.3455f,2.0213f,-12.0010f,  31.9091f,2.9770f,-11.6418f,  31.3455f,2.0213f,-12.0010f,  30.7830f,1.0605f,-12.3584f,  
	29.6585f,1.0618f,-10.7038f,  29.0988f,0.1041f,-11.0582f,  27.8088f,0.2619f,-9.5374f,  27.2519f,-0.6936f,-9.8884f,  25.8179f,-0.3842f,-8.5282f,  
	25.2633f,-1.3393f,-8.8763f,  23.7096f,-0.8853f,-7.7007f,  23.1566f,-1.8409f,-8.0455f,  21.5092f,-1.2522f,-7.0752f,  20.9577f,-2.2083f,-7.4164f,  
	19.2442f,-1.4967f,-6.6684f,  18.6932f,-2.4543f,-7.0060f,  16.9423f,-1.6334f,-6.4934f,  16.3913f,-2.5924f,-6.8270f,  14.6324f,-1.6779f,-6.5590f,  
	14.0811f,-2.6374f,-6.8883f,  12.3435f,-1.6461f,-6.8697f,  11.7908f,-2.6067f,-7.1949f,  10.1037f,-1.5569f,-7.4264f,  9.5496f,-2.5169f,-7.7471f,  
	7.9411f,-1.4277f,-8.2252f,  7.3848f,-2.3869f,-8.5418f,  5.8818f,-1.2782f,-9.2585f,  5.3227f,-2.2364f,-9.5715f,  3.9503f,-1.1282f,-10.5151f,  
	3.3882f,-2.0842f,-10.8246f,  2.1693f,-0.9967f,-11.9795f,  1.6045f,-1.9491f,-12.2858f,  0.5593f,-0.9024f,-13.6327f,  -0.0089f,-1.8517f,-13.9368f,  
	-0.8631f,-0.8652f,-15.4531f,  -1.4347f,-1.8109f,-15.7556f,  -2.0839f,-0.9031f,-17.4160f,  -2.6588f,-1.8449f,-17.7176f,  -3.0925f,-1.0335f,-19.4939f,  
	-3.6706f,-1.9717f,-19.7954f,  -3.8821f,-1.2725f,-21.6578f,  -4.4635f,-2.2078f,-21.9597f,  -4.4499f,-1.6352f,-23.8765f,  -5.0342f,-2.5680f,-24.1797f,  
	-4.7967f,-2.1344f,-26.1182f,  -5.3845f,-3.0668f,-26.4230f,  -4.9281f,-2.7823f,-28.3499f,  -5.5192f,-3.7155f,-28.6566f,  -4.8532f,-3.5879f,-30.5386f,  
	-5.4480f,-4.5239f,-30.8473f,  -4.5851f,-4.5583f,-32.6518f,  -5.1839f,-5.4992f,-32.9624f,  -4.1405f,-5.6977f,-34.6578f,  -4.7447f,-6.6464f,-34.9697f,  
	-3.5408f,-7.0085f,-36.5258f,  -4.1510f,-7.9672f,-36.8384f,  -2.8093f,-8.4890f,-38.2275f,  -3.4268f,-9.4603f,-38.5397f,  -1.9728f,-10.1351f,-39.7366f,  
	-2.5997f,-11.1217f,-40.0470f,  -1.0609f,-11.9394f,-41.0296f,  -1.7002f,-12.9443f,-41.3365f,  -0.1058f,-13.8918f,-42.0864f,  -0.7598f,-14.9167f,-42.3877f,  
	0.8598f,-15.9779f,-42.8903f,  0.1880f,-17.0247f,-43.1841f,  1.8022f,-18.1808f,-43.4291f,  1.1085f,-19.2503f,-43.7130f,  2.6867f,-20.4797f,-43.6946f,  
	1.9681f,-21.5723f,-43.9660f,  3.4804f,-22.8510f,-43.6832f,  2.7318f,-23.9668f,-43.9399f,  4.1500f,-25.2690f,-43.3965f,  3.3680f,-26.4057f,-43.6358f,  
	4.6659f,-27.7038f,-42.8404f,  3.8452f,-28.8597f,-43.0606f,  4.9993f,-30.1252f,-42.0268f,  4.1365f,-31.2955f,-42.2258f,  5.1266f,-32.4993f,-40.9718f,  
	4.2179f,-33.6802f,-41.1481f,  5.0277f,-34.7932f,-39.6964f,  4.0704f,-35.9778f,-39.8491f,  4.6878f,-36.9717f,-38.2265f,  3.6792f,-38.1535f,-38.3554f,  
	4.0969f,-39.0013f,-36.5921f,  3.0360f,-40.1717f,-36.6980f,  3.2515f,-40.8486f,-34.8278f,  2.1394f,-41.9989f,-34.9101f,  2.1547f,-42.4829f,-32.9690f,  
	0.9923f,-43.6042f,-33.0301f,  0.8147f,-43.8768f,-31.0562f,  -0.3935f,-44.9585f,-31.0998f,  -0.7518f,-45.0055f,-29.1322f,  -2.0000f,-46.0380f,-29.1602f,  
	-2.5226f,-45.8501f,-27.2383f,  -3.8081f,-46.8266f,-27.2513f,  -4.4752f,-46.3996f,-25.4142f,  -5.7846f,-47.3095f,-25.4194f,  -6.5725f,-46.6451f,-23.7048f,  
	-7.8985f,-47.4823f,-23.7040f,  21.8221f,15.6745f,-32.6067f,  22.9619f,13.5969f,-33.0584f,  23.5612f,14.6861f,-32.6365f,  24.6483f,12.5437f,-32.8372f,  
	25.2466f,13.6305f,-32.4192f,  26.2526f,11.4428f,-32.3720f,  26.8495f,12.5258f,-31.9585f,  27.7473f,10.3130f,-31.6710f,  28.3425f,11.3908f,-31.2622f,  
	29.1069f,9.1736f,-30.7462f,  29.7001f,10.2449f,-30.3423f,  30.3081f,8.0441f,-29.6132f,  30.8989f,9.1079f,-29.2145f,  31.3303f,6.9439f,-28.2916f,  
	31.9186f,7.9991f,-27.8981f,  32.1561f,5.8917f,-26.8040f,  32.7416f,6.9375f,-26.4156f,  32.7713f,4.9056f,-25.1757f,  33.3540f,5.9413f,-24.7924f,  
	33.1654f,4.0024f,-23.4347f,  33.7451f,5.0275f,-23.0563f,  33.3317f,3.1977f,-21.6108f,  33.9083f,4.2118f,-21.2370f,  33.2672f,2.5051f,-19.7351f,  
	33.8408f,3.5081f,-19.3655f,  32.9732f,1.9365f,-17.8397f,  33.5438f,2.9284f,-17.4740f,  32.4546f,1.5016f,-15.9571f,  33.0224f,2.4827f,-15.5947f,  
	31.7203f,1.2080f,-14.1196f,  32.2854f,2.1786f,-13.7599f,  30.7830f,1.0605f,-12.3584f,  31.3455f,2.0213f,-12.0010f,  30.7830f,1.0605f,-12.3584f,  
	30.2208f,0.0942f,-12.7145f,  29.0988f,0.1041f,-11.0582f,  28.5395f,-0.8588f,-11.4105f,  27.2519f,-0.6936f,-9.8884f,  26.6948f,-1.6551f,-10.2375f,  
	25.2633f,-1.3393f,-8.8763f,  24.7081f,-2.3001f,-9.2220f,  23.1566f,-1.8409f,-8.0455f,  22.6030f,-2.8016f,-8.3876f,  20.9577f,-2.2083f,-7.4164f,  
	20.4047f,-3.1698f,-7.7548f,  18.6932f,-2.4543f,-7.0060f,  18.1404f,-3.4165f,-7.3404f,  16.3913f,-2.5924f,-6.8270f,  15.8384f,-3.5547f,-7.1571f,  
	14.0811f,-2.6374f,-6.8883f,  13.5270f,-3.6004f,-7.2142f,  11.7908f,-2.6067f,-7.1949f,  11.2355f,-3.5687f,-7.5163f,  9.5496f,-2.5169f,-7.7471f,  
	8.9923f,-3.4779f,-8.0643f,  7.3848f,-2.3869f,-8.5418f,  6.8248f,-3.3467f,-8.8551f,  5.3227f,-2.2364f,-9.5715f,  4.7598f,-3.1936f,-9.8811f,  
	3.3882f,-2.0842f,-10.8246f,  2.8227f,-3.0374f,-11.1308f,  1.6045f,-1.9491f,-12.2858f,  1.0356f,-2.8987f,-12.5896f,  -0.0089f,-1.8517f,-13.9368f,  
	-0.5812f,-2.7970f,-14.2387f,  -1.4347f,-1.8109f,-15.7556f,  -2.0102f,-2.7515f,-16.0563f,  -2.6588f,-1.8449f,-17.7176f,  -3.2375f,-2.7810f,-18.0179f,  
	-3.6706f,-1.9717f,-19.7954f,  -4.2524f,-2.9037f,-20.0960f,  -4.4635f,-2.2078f,-21.9597f,  -5.0481f,-3.1360f,-22.2615f,  -5.0342f,-2.5680f,-24.1797f,  
	-5.6220f,-3.4943f,-24.4832f,  -5.3845f,-3.0668f,-26.4230f,  -5.9752f,-3.9922f,-26.7285f,  -5.5192f,-3.7155f,-28.6566f,  -6.1132f,-4.6420f,-28.9643f,  
	-5.4480f,-4.5239f,-30.8473f,  -6.0453f,-5.4534f,-31.1574f,  -5.1839f,-5.4992f,-32.9624f,  -5.7856f,-6.4346f,-33.2745f,  -4.7447f,-6.6464f,-34.9697f,  
	-5.3514f,-7.5900f,-35.2831f,  -4.1510f,-7.9672f,-36.8384f,  -4.7638f,-8.9217f,-37.1524f,  -3.4268f,-9.4603f,-38.5397f,  -4.0476f,-10.4287f,-38.8530f,  
	-2.5997f,-11.1217f,-40.0470f,  -3.2312f,-12.1072f,-40.3580f,  -1.7002f,-12.9443f,-41.3365f,  -2.3445f,-13.9489f,-41.6433f,  -0.7598f,-14.9167f,-42.3877f,  
	-1.4200f,-15.9428f,-42.6883f,  0.1880f,-17.0247f,-43.1841f,  -0.4920f,-18.0738f,-43.4760f,  1.1085f,-19.2503f,-43.7130f,  0.4056f,-20.3233f,-43.9937f,  
	1.9681f,-21.5723f,-43.9660f,  1.2372f,-22.6699f,-44.2331f,  2.7318f,-23.9668f,-43.9399f,  1.9693f,-25.0874f,-44.1905f,  3.3680f,-26.4057f,-43.6358f,  
	2.5683f,-27.5483f,-43.8679f,  3.8452f,-28.8597f,-43.0606f,  3.0044f,-30.0204f,-43.2720f,  4.1365f,-31.2955f,-42.2258f,  3.2503f,-32.4710f,-42.4145f,  
	4.2179f,-33.6802f,-41.1481f,  3.2829f,-34.8643f,-41.3128f,  4.0704f,-35.9778f,-39.8491f,  3.0832f,-37.1646f,-39.9891f,  3.6792f,-38.1535f,-38.3554f,  
	2.6379f,-39.3348f,-38.4709f,  3.0360f,-40.1717f,-36.6980f,  1.9411f,-41.3389f,-36.7881f,  2.1394f,-41.9989f,-34.9101f,  0.9904f,-43.1431f,-34.9768f,  
	0.9923f,-43.6042f,-33.0301f,  -0.2075f,-44.7147f,-33.0767f,  -0.3935f,-44.9585f,-31.0998f,  -1.6390f,-46.0251f,-31.1279f,  -2.0000f,-46.0380f,-29.1602f,  
	-3.2888f,-47.0537f,-29.1698f,  -3.8081f,-46.8266f,-27.2513f,  -5.1283f,-47.7797f,-27.2498f,  -5.7846f,-47.3095f,-25.4194f,  -7.1291f,-48.1931f,-25.4084f,  
	-7.8985f,-47.4823f,-23.7040f,  -9.2550f,-48.2892f,-23.6882f,  22.9619f,13.5969f,-33.0584f,  24.0491f,11.4610f,-33.2470f,  24.6483f,12.5437f,-32.8372f,  
	25.6547f,10.3635f,-32.7782f,  26.2526f,11.4428f,-32.3720f,  27.1510f,9.2383f,-32.0733f,  27.7473f,10.3130f,-31.6710f,  28.5126f,8.1047f,-31.1442f,  
	29.1069f,9.1736f,-30.7462f,  29.7161f,6.9821f,-30.0069f,  30.3081f,8.0441f,-29.6132f,  30.7409f,5.8897f,-28.6808f,  31.3303f,6.9439f,-28.2916f,  
	31.5695f,4.8461f,-27.1886f,  32.1561f,5.8917f,-26.8040f,  32.1876f,3.8694f,-25.5559f,  32.7713f,4.9056f,-25.1757f,  32.5848f,2.9760f,-23.8105f,  
	33.1654f,4.0024f,-23.4347f,  32.7543f,2.1814f,-21.9824f,  33.3317f,3.1977f,-21.6108f,  32.6931f,1.4992f,-20.1028f,  33.2672f,2.5051f,-19.7351f,  
	32.4022f,0.9409f,-18.2039f,  32.9732f,1.9365f,-17.8397f,  31.8867f,0.5163f,-16.3181f,  32.4546f,1.5016f,-15.9571f,  31.1554f,0.2324f,-14.4778f,  
	31.7203f,1.2080f,-14.1196f,  30.2208f,0.0942f,-12.7145f,  30.7830f,1.0605f,-12.3584f,  30.2208f,0.0942f,-12.7145f,  29.6587f,-0.8768f,-13.0685f,  
	28.5395f,-0.8588f,-11.4105f,  27.9797f,-1.8274f,-11.7612f,  26.6948f,-1.6551f,-10.2375f,  26.1370f,-2.6221f,-10.5846f,  24.7081f,-2.3001f,-9.2220f,  
	24.1519f,-3.2662f,-9.5653f,  22.6030f,-2.8016f,-8.3876f,  22.0477f,-3.7677f,-8.7271f,  20.4047f,-3.1698f,-7.7548f,  19.8498f,-4.1361f,-8.0903f,  
	18.1404f,-3.4165f,-7.3404f,  17.5856f,-4.3824f,-7.6715f,  15.8384f,-3.5547f,-7.1571f,  15.2826f,-4.5209f,-7.4840f,  13.5270f,-3.6004f,-7.2142f,  
	12.9701f,-4.5653f,-7.5365f,  11.2355f,-3.5687f,-7.5163f,  10.6767f,-4.5324f,-7.8343f,  8.9923f,-3.4779f,-8.0643f,  8.4308f,-4.4400f,-8.3783f,  
	6.8248f,-3.3467f,-8.8551f,  6.2606f,-4.3059f,-9.1652f,  4.7598f,-3.1936f,-9.8811f,  4.1930f,-4.1485f,-10.1876f,  2.8227f,-3.0374f,-11.1308f,  
	2.2525f,-3.9882f,-11.4345f,  1.0356f,-2.8987f,-12.5896f,  0.4621f,-3.8447f,-12.8911f,  -0.5812f,-2.7970f,-14.2387f,  -1.1579f,-3.7376f,-14.5387f,  
	-2.0102f,-2.7515f,-16.0563f,  -2.5901f,-3.6866f,-16.3557f,  -3.2375f,-2.7810f,-18.0179f,  -3.8204f,-3.7110f,-18.3173f,  -4.2524f,-2.9037f,-20.0960f,  
	-4.8382f,-3.8286f,-20.3965f,  -5.0481f,-3.1360f,-22.2615f,  -5.6368f,-4.0575f,-22.5636f,  -5.6220f,-3.4943f,-24.4832f,  -6.2133f,-4.4132f,-24.7873f,  
	-5.9752f,-3.9922f,-26.7285f,  -6.5692f,-4.9104f,-27.0350f,  -6.1132f,-4.6420f,-28.9643f,  -6.7100f,-5.5613f,-29.2737f,  -6.0453f,-5.4534f,-31.1574f,  
	-6.6457f,-6.3766f,-31.4692f,  -5.7856f,-6.4346f,-33.2745f,  -6.3901f,-7.3640f,-33.5884f,  -5.3514f,-7.5900f,-35.2831f,  -5.9608f,-8.5284f,-35.5985f,  
	-4.7638f,-8.9217f,-37.1524f,  -5.3797f,-9.8722f,-37.4681f,  -4.0476f,-10.4287f,-38.8530f,  -4.6725f,-11.3947f,-39.1676f,  -3.2312f,-12.1072f,-40.3580f,  
	-3.8671f,-13.0911f,-40.6696f,  -2.3445f,-13.9489f,-41.6433f,  -2.9943f,-14.9535f,-41.9501f,  -1.4200f,-15.9428f,-42.6883f,  -2.0875f,-16.9701f,-42.9878f,  
	-0.4920f,-18.0738f,-43.4760f,  -1.1802f,-19.1252f,-43.7657f,  0.4056f,-20.3233f,-43.9937f,  -0.3084f,-21.4003f,-44.2710f,  1.2372f,-22.6699f,-44.2331f,  
	0.4935f,-23.7715f,-44.4952f,  1.9693f,-25.0874f,-44.1905f,  1.1903f,-26.2135f,-44.4350f,  2.5683f,-27.5483f,-43.8679f,  1.7496f,-28.6957f,-44.0923f,  
	3.0044f,-30.0204f,-43.2720f,  2.1414f,-31.1864f,-43.4740f,  3.2503f,-32.4710f,-42.4145f,  2.3390f,-33.6503f,-42.5923f,  3.2829f,-34.8643f,-41.3128f,  
	2.3191f,-36.0515f,-41.4653f,  3.0832f,-37.1646f,-39.9891f,  2.0640f,-38.3520f,-40.1161f,  2.6379f,-39.3348f,-38.4709f,  1.5631f,-40.5141f,-38.5711f,  
	1.9411f,-41.3389f,-36.7881f,  0.8090f,-42.5015f,-36.8630f,  0.9904f,-43.1431f,-34.9768f,  -0.1969f,-44.2783f,-35.0290f,  -0.2075f,-44.7147f,-33.0767f,  
	-1.4456f,-45.8118f,-33.1076f,  -1.6390f,-46.0251f,-31.1279f,  -2.9263f,-47.0770f,-31.1371f,  -3.2888f,-47.0537f,-29.1698f,  -4.6149f,-48.0476f,-29.1646f,  
	-5.1283f,-47.7797f,-27.2498f,  -6.4865f,-48.7081f,-27.2315f,  -7.1291f,-48.1931f,-25.4084f,  -8.5075f,-49.0474f,-25.3818f,  -9.2550f,-48.2892f,-23.6882f,  
	-10.6416f,-49.0632f,-23.6571f,  24.0491f,11.4610f,-33.2470f,  25.0559f,9.2875f,-33.1773f,  25.6547f,10.3635f,-32.7782f,  26.5538f,8.1665f,-32.4691f,  
	27.1510f,9.2383f,-32.0733f,  27.9173f,7.0381f,-31.5364f,  28.5126f,8.1047f,-31.1442f,  29.1231f,5.9218f,-30.3953f,  29.7161f,6.9821f,-30.0069f,  
	30.1505f,4.8366f,-29.0653f,  30.7409f,5.8897f,-28.6808f,  30.9819f,3.8010f,-27.5691f,  31.5695f,4.8461f,-27.1886f,  31.6031f,2.8329f,-25.9324f,  
	32.1876f,3.8694f,-25.5559f,  32.0035f,1.9487f,-24.1831f,  32.5848f,2.9760f,-23.8105f,  32.1762f,1.1636f,-22.3512f,  32.7543f,2.1814f,-21.9824f,  
	32.1183f,0.4910f,-20.4680f,  32.6931f,1.4992f,-20.1028f,  31.8308f,-0.0576f,-18.5658f,  32.4022f,0.9409f,-18.2039f,  31.3185f,-0.4727f,-16.6770f,  
	31.8867f,0.5163f,-16.3181f,  30.5904f,-0.7474f,-14.8341f,  31.1554f,0.2324f,-14.4778f,  29.6587f,-0.8768f,-13.0685f,  30.2208f,0.0942f,-12.7145f,  
	29.6587f,-0.8768f,-13.0685f,  29.0958f,-1.8533f,-13.4212f,  27.9797f,-1.8274f,-11.7612f,  27.4190f,-2.8013f,-12.1099f,  26.1370f,-2.6221f,-10.5846f,  
	25.5780f,-3.5943f,-10.9294f,  24.1519f,-3.2662f,-9.5653f,  23.5939f,-4.2377f,-9.9063f,  22.0477f,-3.7677f,-8.7271f,  21.4902f,-4.7387f,-9.0640f,  
	19.8498f,-4.1361f,-8.0903f,  19.2925f,-5.1063f,-8.4228f,  17.5856f,-4.3824f,-7.6715f,  17.0274f,-5.3525f,-7.9997f,  15.2826f,-4.5209f,-7.4840f,  
	14.7235f,-5.4895f,-7.8076f,  12.9701f,-4.5653f,-7.5365f,  12.4093f,-5.5323f,-7.8556f,  10.6767f,-4.5324f,-7.8343f,  10.1134f,-5.4977f,-8.1492f,  
	8.4308f,-4.4400f,-8.3783f,  7.8648f,-5.4022f,-8.6891f,  6.2606f,-4.3059f,-9.1652f,  5.6920f,-5.2635f,-9.4721f,  4.1930f,-4.1485f,-10.1876f,  
	3.6211f,-5.1016f,-10.4914f,  2.2525f,-3.9882f,-11.4345f,  1.6773f,-4.9360f,-11.7359f,  0.4621f,-3.8447f,-12.8911f,  -0.1163f,-4.7864f,-13.1906f,  
	-1.1579f,-3.7376f,-14.5387f,  -1.7395f,-4.6731f,-14.8373f,  -2.5901f,-3.6866f,-16.3557f,  -3.1748f,-4.6159f,-16.6541f,  -3.8204f,-3.7110f,-18.3173f,  
	-4.4079f,-4.6340f,-18.6165f,  -4.8382f,-3.8286f,-20.3965f,  -5.4285f,-4.7468f,-20.6972f,  -5.6368f,-4.0575f,-22.5636f,  -6.2295f,-4.9714f,-22.8663f,  
	-6.2133f,-4.4132f,-24.7873f,  -6.8084f,-5.3245f,-25.0925f,  -6.5692f,-4.9104f,-27.0350f,  -7.1667f,-5.8208f,-27.3433f,  -6.7100f,-5.5613f,-29.2737f,  
	-7.3103f,-6.4736f,-29.5848f,  -6.6457f,-6.3766f,-31.4692f,  -7.2492f,-7.2930f,-31.7831f,  -6.3901f,-7.3640f,-33.5884f,  -6.9974f,-8.2873f,-33.9046f,  
	-5.9608f,-8.5284f,-35.5985f,  -6.5733f,-9.4618f,-35.9161f,  -5.3797f,-9.8722f,-37.4681f,  -5.9995f,-10.8192f,-37.7856f,  -4.6725f,-11.3947f,-39.1676f,  
	-5.3016f,-12.3580f,-39.4834f,  -3.8671f,-13.0911f,-40.6696f,  -4.5079f,-14.0738f,-40.9821f,  -2.9943f,-14.9535f,-41.9501f,  -3.6507f,-15.9582f,-42.2566f,  
	-2.0875f,-16.9701f,-42.9878f,  -2.7623f,-17.9987f,-43.2861f,  -1.1802f,-19.1252f,-43.7657f,  -1.8784f,-20.1797f,-44.0531f,  -0.3084f,-21.4003f,-44.2710f,  
	-1.0339f,-22.4805f,-44.5445f,  0.4935f,-23.7715f,-44.4952f,  -0.2651f,-24.8781f,-44.7522f,  1.1903f,-26.2135f,-44.4350f,  0.3938f,-27.3441f,-44.6728f,  
	1.7496f,-28.6957f,-44.0923f,  0.9103f,-29.8484f,-44.3083f,  2.1414f,-31.1864f,-43.4740f,  1.2547f,-32.3566f,-43.6660f,  2.3390f,-33.6503f,-42.5923f,  
	1.3999f,-34.8333f,-42.7588f,  2.3191f,-36.0515f,-41.4653f,  1.3239f,-37.2403f,-41.6056f,  2.0640f,-38.3520f,-40.1161f,  1.0119f,-39.5386f,-40.2284f,  
	1.5631f,-40.5141f,-38.5711f,  0.4513f,-41.6903f,-38.6563f,  0.8090f,-42.5015f,-36.8630f,  -0.3619f,-43.6567f,-36.9234f,  -0.1969f,-44.2783f,-35.0290f,  
	-1.4234f,-45.4017f,-35.0655f,  -1.4456f,-45.8118f,-33.1076f,  -2.7268f,-46.8961f,-33.1194f,  -2.9263f,-47.0770f,-31.1371f,  -4.2532f,-48.1087f,-31.1313f,  
	-4.6149f,-48.0476f,-29.1646f,  -5.9815f,-49.0186f,-29.1422f,  -6.4865f,-48.7081f,-27.2315f,  -7.8817f,-49.6086f,-27.1972f,  -8.5075f,-49.0474f,-25.3818f,  
	-9.9196f,-49.8699f,-25.3394f,  -10.6416f,-49.0632f,-23.6571f,  -12.0583f,-49.8022f,-23.6101f,  25.0559f,9.2875f,-33.1773f,  25.9546f,7.0961f,-32.8593f,  
	26.5538f,8.1665f,-32.4691f,  27.3200f,5.9724f,-31.9237f,  27.9173f,7.0381f,-31.5364f,  28.5279f,4.8617f,-30.7795f,  29.1231f,5.9218f,-30.3953f,  
	29.5578f,3.7830f,-29.4461f,  30.1505f,4.8366f,-29.0653f,  30.3920f,2.7548f,-27.9465f,  30.9819f,3.8010f,-27.5691f,  31.0163f,1.7945f,-26.3063f,  
	31.6031f,2.8329f,-25.9324f,  31.4200f,0.9188f,-24.5535f,  32.0035f,1.9487f,-24.1831f,  31.5962f,0.1426f,-22.7182f,  32.1762f,1.1636f,-22.3512f,  
	31.5418f,-0.5210f,-20.8317f,  32.1183f,0.4910f,-20.4680f,  31.2578f,-1.0604f,-18.9263f,  31.8308f,-0.0576f,-18.5658f,  30.7491f,-1.4665f,-17.0346f,  
	31.3185f,-0.4727f,-16.6770f,  30.0243f,-1.7323f,-15.1891f,  30.5904f,-0.7474f,-14.8341f,  29.0958f,-1.8533f,-13.4212f,  29.6587f,-0.8768f,-13.0685f,  
	29.0958f,-1.8533f,-13.4212f,  28.5320f,-2.8346f,-13.7718f,  27.4190f,-2.8013f,-12.1099f,  26.8568f,-3.7802f,-12.4565f,  25.5780f,-3.5943f,-10.9294f,  
	25.0170f,-4.5719f,-11.2721f,  23.5939f,-4.2377f,-9.9063f,  23.0335f,-5.2142f,-10.2448f,  21.4902f,-4.7387f,-9.0640f,  20.9301f,-5.7140f,-9.3981f,  
	19.2925f,-5.1063f,-8.4228f,  18.7318f,-6.0810f,-8.7525f,  17.0274f,-5.3525f,-7.9997f,  16.4659f,-6.3255f,-8.3247f,  14.7235f,-5.4895f,-7.8076f,  
	14.1603f,-6.4607f,-8.1280f,  12.4093f,-5.5323f,-7.8556f,  11.8436f,-6.5016f,-8.1717f,  10.1134f,-5.4977f,-8.1492f,  9.5451f,-6.4636f,-8.4610f,  
	7.8648f,-5.4022f,-8.6891f,  7.2941f,-6.3633f,-8.9968f,  5.6920f,-5.2635f,-9.4721f,  5.1179f,-6.2199f,-9.7764f,  3.6211f,-5.1016f,-10.4914f,  
	3.0437f,-6.0522f,-10.7929f,  1.6773f,-4.9360f,-11.7359f,  1.0966f,-5.8800f,-12.0352f,  -0.1163f,-4.7864f,-13.1906f,  -0.7004f,-5.7234f,-13.4886f,  
	-1.7395f,-4.6731f,-14.8373f,  -2.3266f,-5.6029f,-15.1347f,  -3.1748f,-4.6159f,-16.6541f,  -3.7648f,-5.5384f,-16.9520f,  -4.4079f,-4.6340f,-18.6165f,  
	-5.0007f,-5.5503f,-18.9158f,  -5.4285f,-4.7468f,-20.6972f,  -6.0235f,-5.6572f,-20.9983f,  -6.2295f,-4.9714f,-22.8663f,  -6.8266f,-5.8774f,-23.1700f,  
	-6.8084f,-5.3245f,-25.0925f,  -7.4075f,-6.2277f,-25.3995f,  -7.1667f,-5.8208f,-27.3433f,  -7.7681f,-6.7237f,-27.6535f,  -7.3103f,-6.4736f,-29.5848f,  
	-7.9141f,-7.3783f,-29.8982f,  -7.2492f,-7.2930f,-31.7831f,  -7.8558f,-8.2023f,-32.0996f,  -6.9974f,-8.2873f,-33.9046f,  -7.6079f,-9.2044f,-34.2233f,  
	-6.5733f,-9.4618f,-35.9161f,  -7.1896f,-10.3903f,-36.2358f,  -5.9995f,-10.8192f,-37.7856f,  -6.6234f,-11.7622f,-38.1051f,  -5.3016f,-12.3580f,-39.4834f,  
	-5.9351f,-13.3187f,-39.8008f,  -4.5079f,-14.0738f,-40.9821f,  -5.1549f,-15.0554f,-41.2951f,  -3.6507f,-15.9582f,-42.2566f,  -4.3135f,-16.9629f,-42.5630f,  
	-2.7623f,-17.9987f,-43.2861f,  -3.4460f,-19.0292f,-43.5831f,  -1.8784f,-20.1797f,-44.0531f,  -2.5870f,-21.2366f,-44.3377f,  -1.0339f,-22.4805f,-44.5445f,  
	-1.7731f,-23.5650f,-44.8140f,  -0.2651f,-24.8781f,-44.7522f,  -1.0400f,-25.9888f,-45.0035f,  0.3938f,-27.3441f,-44.6728f,  -0.4220f,-28.4797f,-44.9032f,  
	0.9103f,-29.8484f,-44.3083f,  0.0486f,-31.0052f,-44.5152f,  1.2547f,-32.3566f,-43.6660f,  0.3415f,-33.5308f,-43.8475f,  1.3999f,-34.8333f,-42.7588f,  
	0.4305f,-36.0186f,-42.9138f,  1.3239f,-37.2403f,-41.6056f,  0.2969f,-38.4291f,-41.7317f,  1.0119f,-39.5386f,-40.2284f,  -0.0767f,-40.7234f,-40.3261f,  
	0.4513f,-41.6903f,-38.6563f,  -0.6997f,-42.8607f,-38.7273f,  -0.3619f,-43.6567f,-36.9234f,  -1.5726f,-44.8017f,-36.9681f,  -1.4234f,-45.4017f,-35.0655f,  
	-2.6936f,-46.5141f,-35.0829f,  -2.7268f,-46.8961f,-33.1194f,  -4.0495f,-47.9619f,-33.1160f,  -4.2532f,-48.1087f,-31.1313f,  -5.6230f,-49.1192f,-31.1078f,  
	-5.9815f,-49.0186f,-29.1422f,  -7.3882f,-49.9630f,-29.1032f,  -7.8817f,-49.6086f,-27.1972f,  -9.3139f,-50.4782f,-27.1464f,  -9.9196f,-49.8699f,-25.3394f,  
	-11.3657f,-50.6581f,-25.2804f,  -12.0583f,-49.8022f,-23.6101f,  -13.5043f,-50.5035f,-23.5466f,  25.9546f,7.0961f,-32.8593f,  26.7216f,4.9092f,-32.3055f,  
	27.3200f,5.9724f,-31.9237f,  27.9317f,3.8035f,-31.1586f,  28.5279f,4.8617f,-30.7795f,  28.9640f,2.7306f,-29.8225f,  29.5578f,3.7830f,-29.4461f,  
	29.8010f,1.7090f,-28.3199f,  30.3920f,2.7548f,-27.9465f,  30.4284f,0.7560f,-26.6766f,  31.0163f,1.7945f,-26.3063f,  30.8353f,-0.1120f,-24.9207f,  
	31.4200f,0.9188f,-24.5535f,  31.0149f,-0.8801f,-23.0823f,  31.5962f,0.1426f,-22.7182f,  30.9640f,-1.5353f,-21.1928f,  31.5418f,-0.5210f,-20.8317f,  
	30.6836f,-2.0662f,-19.2845f,  31.2578f,-1.0604f,-18.9263f,  30.1784f,-2.4639f,-17.3901f,  30.7491f,-1.4665f,-17.0346f,  29.4571f,-2.7215f,-15.5420f,  
	30.0243f,-1.7323f,-15.1891f,  28.5320f,-2.8346f,-13.7718f,  29.0958f,-1.8533f,-13.4212f,  28.5320f,-2.8346f,-13.7718f,  27.9665f,-3.8208f,-14.1205f,  
	26.8568f,-3.7802f,-12.4565f,  26.2925f,-4.7644f,-12.8012f,  25.0170f,-4.5719f,-11.2721f,  24.4534f,-5.5543f,-11.6124f,  23.0335f,-5.2142f,-10.2448f,  
	22.4704f,-6.1951f,-10.5806f,  20.9301f,-5.7140f,-9.3981f,  20.3664f,-6.6939f,-9.7295f,  18.7318f,-6.0810f,-8.7525f,  18.1673f,-7.0590f,-9.0791f,  
	16.4659f,-6.3255f,-8.3247f,  15.8999f,-7.3015f,-8.6467f,  14.1603f,-6.4607f,-8.1280f,  13.5920f,-7.4346f,-8.4454f,  11.8436f,-6.5016f,-8.1717f,  
	11.2727f,-7.4720f,-8.4847f,  9.5451f,-6.4636f,-8.4610f,  8.9718f,-7.4291f,-8.7696f,  7.2941f,-6.3633f,-8.9968f,  6.7173f,-7.3238f,-9.3017f,  
	5.1179f,-6.2199f,-9.7764f,  4.5377f,-7.1743f,-10.0782f,  3.0437f,-6.0522f,-10.7929f,  2.4601f,-6.9996f,-11.0921f,  1.0966f,-5.8800f,-12.0352f,  
	0.5097f,-6.8198f,-12.3327f,  -0.7004f,-5.7234f,-13.4886f,  -1.2904f,-6.6552f,-13.7852f,  -2.3266f,-5.6029f,-15.1347f,  -2.9197f,-6.5264f,-15.4316f,  
	-3.7648f,-5.5384f,-16.9520f,  -4.3607f,-6.4544f,-17.2499f,  -5.0007f,-5.5503f,-18.9158f,  -5.5987f,-6.4589f,-19.2153f,  -6.0235f,-5.6572f,-20.9983f,  
	-6.6234f,-6.5595f,-21.3004f,  -6.8266f,-5.8774f,-23.1700f,  -7.4284f,-6.7750f,-23.4754f,  -7.4075f,-6.2277f,-25.3995f,  -8.0111f,-7.1228f,-25.7083f,  
	-7.7681f,-6.7237f,-27.6535f,  -8.3734f,-7.6183f,-27.9660f,  -7.9141f,-7.3783f,-29.8982f,  -8.5214f,-8.2750f,-30.2145f,  -7.8558f,-8.2023f,-32.0996f,  
	-8.4659f,-9.1045f,-32.4189f,  -7.6079f,-9.2044f,-34.2233f,  -8.2224f,-10.1155f,-34.5447f,  -7.1896f,-10.3903f,-36.2358f,  -7.8098f,-11.3137f,-36.5580f,  
	-6.6234f,-11.7622f,-38.1051f,  -7.2513f,-12.7012f,-38.4266f,  -5.9351f,-13.3187f,-39.8008f,  -6.5742f,-14.2771f,-40.1195f,  -5.1549f,-15.0554f,-41.2951f,  
	-5.8074f,-16.0356f,-41.6087f,  -4.3135f,-16.9629f,-42.5630f,  -4.9844f,-17.9683f,-42.8690f,  -3.4460f,-19.0292f,-43.5831f,  -4.1390f,-20.0609f,-43.8783f,  
	-2.5870f,-21.2366f,-44.3377f,  -3.3079f,-22.2967f,-44.6194f,  -1.7731f,-23.5650f,-44.8140f,  -2.5273f,-24.6529f,-45.0788f,  -1.0400f,-25.9888f,-45.0035f,  
	-1.8326f,-27.1039f,-45.2486f,  -0.4220f,-28.4797f,-44.9032f,  -1.2586f,-29.6194f,-45.1257f,  0.0486f,-31.0052f,-44.5152f,  -0.8382f,-32.1663f,-44.7127f,  
	0.3415f,-33.5308f,-43.8475f,  -0.6010f,-34.7078f,-44.0183f,  0.4305f,-36.0186f,-42.9138f,  -0.5696f,-37.2047f,-43.0554f,  0.2969f,-38.4291f,-41.7317f,  
	-0.7659f,-39.6173f,-41.8439f,  -0.0767f,-40.7234f,-40.3261f,  -1.2045f,-41.9037f,-40.4100f,  -0.6997f,-42.8607f,-38.7273f,  -1.8907f,-44.0222f,-38.7828f,  
	-1.5726f,-44.8017f,-36.9681f,  -2.8274f,-45.9376f,-36.9938f,  -2.6936f,-46.5141f,-35.0829f,  -4.0071f,-47.6097f,-35.0848f,  -4.0495f,-47.9619f,-33.1160f,  
	-5.4170f,-49.0082f,-33.0946f,  -5.6230f,-49.1192f,-31.1078f,  -7.0355f,-50.1046f,-31.0673f,  -7.3882f,-49.9630f,-29.1032f,  -8.8351f,-50.8777f,-29.0473f,  
	-9.3139f,-50.4782f,-27.1464f,  -10.7841f,-51.3145f,-27.0784f,  -11.3657f,-50.6581f,-25.2804f,  -12.8454f,-51.4092f,-25.2043f,  -13.5043f,-50.5035f,-23.5466f,  
	-14.9789f,-51.1648f,-23.4664f,  26.7216f,4.9092f,-32.3055f,  27.3331f,2.7451f,-31.5336f,  27.9317f,3.8035f,-31.1586f,  28.3678f,1.6775f,-30.1950f,  
	28.9640f,2.7306f,-29.8225f,  29.2076f,0.6619f,-28.6899f,  29.8010f,1.7090f,-28.3199f,  29.8379f,-0.2845f,-27.0439f,  30.4284f,0.7560f,-26.6766f,  
	30.2481f,-1.1453f,-25.2853f,  30.8353f,-0.1120f,-24.9207f,  30.4312f,-1.9058f,-23.4440f,  31.0149f,-0.8801f,-23.0823f,  30.3839f,-2.5531f,-21.5517f,  
	30.9640f,-1.5353f,-21.1928f,  30.1072f,-3.0760f,-19.6407f,  30.6836f,-2.0662f,-19.2845f,  29.6057f,-3.4657f,-17.7436f,  30.1784f,-2.4639f,-17.3901f,  
	28.8880f,-3.7153f,-15.8930f,  29.4571f,-2.7215f,-15.5420f,  27.9665f,-3.8208f,-14.1205f,  28.5320f,-2.8346f,-13.7718f,  27.9665f,-3.8208f,-14.1205f,  
	27.3986f,-4.8120f,-14.4673f,  26.2925f,-4.7644f,-12.8012f,  25.7255f,-5.7533f,-13.1435f,  24.4534f,-5.5543f,-11.6124f,  23.8869f,-6.5413f,-11.9502f,  
	22.4704f,-6.1951f,-10.5806f,  21.9034f,-7.1807f,-10.9139f,  20.3664f,-6.6939f,-9.7295f,  19.7988f,-7.6775f,-10.0579f,  18.1673f,-7.0590f,-9.0791f,  
	17.5982f,-8.0404f,-9.4028f,  15.8999f,-7.3015f,-8.6467f,  15.3285f,-8.2807f,-8.9657f,  13.5920f,-7.4346f,-8.4454f,  13.0181f,-8.4102f,-8.7598f,  
	11.2727f,-7.4720f,-8.4847f,  10.6964f,-8.4426f,-8.7945f,  8.9718f,-7.4291f,-8.7696f,  8.3919f,-8.3946f,-9.0754f,  6.7173f,-7.3238f,-9.3017f,  
	6.1340f,-8.2830f,-9.6040f,  4.5377f,-7.1743f,-10.0782f,  3.9510f,-8.1261f,-10.3776f,  2.4601f,-6.9996f,-11.0921f,  1.8698f,-7.9433f,-11.3895f,  
	0.5097f,-6.8198f,-12.3327f,  -0.0839f,-7.7547f,-12.6287f,  -1.2904f,-6.6552f,-13.7852f,  -1.8872f,-7.5810f,-14.0811f,  -2.9197f,-6.5264f,-15.4316f,  
	-3.5194f,-7.4435f,-15.7281f,  -4.3607f,-6.4544f,-17.2499f,  -4.9625f,-7.3627f,-17.5478f,  -5.5987f,-6.4589f,-19.2153f,  -6.2025f,-7.3592f,-19.5156f,  
	-6.6234f,-6.5595f,-21.3004f,  -7.2289f,-7.4532f,-21.6039f,  -7.4284f,-6.7750f,-23.4754f,  -8.0352f,-7.6640f,-23.7825f,  -8.0111f,-7.1228f,-25.7083f,  
	-8.6191f,-8.0090f,-26.0194f,  -8.3734f,-7.6183f,-27.9660f,  -8.9827f,-8.5042f,-28.2814f,  -8.5214f,-8.2750f,-30.2145f,  -9.1326f,-9.1637f,-30.5337f,  
	-8.4659f,-9.1045f,-32.4189f,  -9.0801f,-9.9995f,-32.7411f,  -8.2224f,-10.1155f,-34.5447f,  -8.8407f,-11.0204f,-34.8689f,  -7.8098f,-11.3137f,-36.5580f,  
	-8.4340f,-12.2318f,-36.8830f,  -7.2513f,-12.7012f,-38.4266f,  -7.8844f,-13.6366f,-38.7502f,  -6.5742f,-14.2771f,-40.1195f,  -7.2184f,-15.2327f,-40.4397f,  
	-5.8074f,-16.0356f,-41.6087f,  -6.4672f,-17.0151f,-41.9229f,  -4.9844f,-17.9683f,-42.8690f,  -5.6635f,-18.9737f,-43.1742f,  -4.1390f,-20.0609f,-43.8783f,  
	-4.8431f,-21.0948f,-44.1716f,  -3.3079f,-22.2967f,-44.6194f,  -4.0424f,-23.3593f,-44.8974f,  -2.5273f,-24.6529f,-45.0788f,  -3.2976f,-25.7445f,-45.3385f,  
	-1.8326f,-27.1039f,-45.2486f,  -2.6447f,-28.2227f,-45.4868f,  -1.2586f,-29.6194f,-45.1257f,  -2.1189f,-30.7631f,-45.3396f,  -0.8382f,-32.1663f,-44.7127f,  
	-1.7529f,-33.3304f,-44.9003f,  -0.6010f,-34.7078f,-44.0183f,  -1.5727f,-35.8862f,-44.1767f,  -0.5696f,-37.2047f,-43.0554f,  -1.6045f,-38.3910f,-43.1837f,  
	-0.7659f,-39.6173f,-41.8439f,  -1.8676f,-40.8022f,-41.9427f,  -1.2045f,-41.9037f,-40.4100f,  -2.3723f,-43.0765f,-40.4787f,  -1.8907f,-44.0222f,-38.7828f,  
	-3.1259f,-45.1763f,-38.8196f,  -2.8274f,-45.9376f,-36.9938f,  -4.1268f,-47.0582f,-37.0041f,  -4.0071f,-47.6097f,-35.0848f,  -5.3670f,-48.6875f,-35.0686f,  
	-5.4170f,-49.0082f,-33.0946f,  -6.8297f,-50.0309f,-33.0559f,  -7.0355f,-50.1046f,-31.0673f,  -8.4912f,-51.0615f,-31.0094f,  -8.8351f,-50.8777f,-29.0473f,  
	-10.3235f,-51.7601f,-28.9735f,  -10.7841f,-51.3145f,-27.0784f,  -12.2921f,-52.1143f,-26.9928f,  -12.8454f,-51.4092f,-25.2043f,  -14.3582f,-52.1203f,-25.1107f,  
	-14.9789f,-51.1648f,-23.4664f,  -16.4813f,-51.7835f,-23.3684f,  27.3331f,2.7451f,-31.5336f,  27.7695f,0.6242f,-30.5636f,  28.3678f,1.6775f,-30.1950f,  
	28.6118f,-0.3861f,-29.0564f,  29.2076f,0.6619f,-28.6899f,  29.2451f,-1.3264f,-27.4081f,  29.8379f,-0.2845f,-27.0439f,  29.6585f,-2.1807f,-25.6470f,  
	30.2481f,-1.1453f,-25.2853f,  29.8449f,-2.9343f,-23.8032f,  30.4312f,-1.9058f,-23.4440f,  29.8013f,-3.5742f,-21.9083f,  30.3839f,-2.5531f,-21.5517f,  
	29.5283f,-4.0896f,-19.9947f,  30.1072f,-3.0760f,-19.6407f,  29.0305f,-4.4717f,-18.0951f,  29.6057f,-3.4657f,-17.7436f,  28.3166f,-4.7139f,-16.2421f,  
	28.8880f,-3.7153f,-15.8930f,  27.3986f,-4.8120f,-14.4673f,  27.9665f,-3.8208f,-14.1205f,  27.3986f,-4.8120f,-14.4673f,  26.8281f,-5.8078f,-14.8118f,  
	25.7255f,-5.7533f,-13.1435f,  25.1554f,-6.7468f,-13.4834f,  23.8869f,-6.5413f,-11.9502f,  23.3164f,-7.5332f,-12.2855f,  21.9034f,-7.1807f,-10.9139f,  
	21.3323f,-8.1703f,-11.2443f,  19.7988f,-7.6775f,-10.0579f,  19.2263f,-8.6648f,-10.3835f,  17.5982f,-8.0404f,-9.4028f,  17.0235f,-9.0254f,-9.7237f,  
	15.3285f,-8.2807f,-8.9657f,  14.7513f,-9.2621f,-9.2817f,  13.0181f,-8.4102f,-8.7598f,  12.4384f,-9.3866f,-9.0710f,  10.6964f,-8.4426f,-8.7945f,  
	10.1131f,-9.4139f,-9.1014f,  8.3919f,-8.3946f,-9.0754f,  7.8051f,-9.3594f,-9.3785f,  6.1340f,-8.2830f,-9.6040f,  5.5436f,-9.2401f,-9.9039f,  
	3.9510f,-8.1261f,-10.3776f,  3.3569f,-9.0748f,-10.6749f,  1.8698f,-7.9433f,-11.3895f,  1.2723f,-8.8826f,-11.6850f,  -0.0839f,-7.7547f,-12.6287f,  
	-0.6849f,-8.6842f,-12.9237f,  -1.8872f,-7.5810f,-14.0811f,  -2.4913f,-8.5008f,-14.3763f,  -3.5194f,-7.4435f,-15.7281f,  -4.1258f,-8.3531f,-16.0244f,  
	-4.9625f,-7.3627f,-17.5478f,  -5.5709f,-8.2627f,-17.8462f,  -6.2025f,-7.3592f,-19.5156f,  -6.8125f,-8.2508f,-19.8172f,  -7.2289f,-7.4532f,-21.6039f,  
	-7.8401f,-8.3380f,-21.9091f,  -8.0352f,-7.6640f,-23.7825f,  -8.6473f,-8.5436f,-24.0919f,  -8.6191f,-8.0090f,-26.0194f,  -9.2319f,-8.8858f,-26.3335f,  
	-8.9827f,-8.5042f,-28.2814f,  -9.5965f,-9.3811f,-28.5998f,  -9.1326f,-9.1637f,-30.5337f,  -9.7481f,-10.0443f,-30.8559f,  -9.0801f,-9.9995f,-32.7411f,  
	-9.6984f,-10.8872f,-33.0664f,  -8.8407f,-11.0204f,-34.8689f,  -9.4630f,-11.9186f,-35.1962f,  -8.4340f,-12.2318f,-36.8830f,  -9.0632f,-13.1450f,-37.2104f,  
	-7.8844f,-13.6366f,-38.7502f,  -8.5223f,-14.5679f,-39.0759f,  -7.2184f,-15.2327f,-40.4397f,  -7.8691f,-16.1861f,-40.7612f,  -6.4672f,-17.0151f,-41.9229f,  
	-7.1344f,-17.9934f,-42.2373f,  -5.6635f,-18.9737f,-43.1742f,  -6.3526f,-19.9799f,-43.4784f,  -4.8431f,-21.0948f,-44.1716f,  -5.5597f,-22.1301f,-44.4623f,  
	-4.0424f,-23.3593f,-44.8974f,  -4.7917f,-24.4247f,-45.1714f,  -3.2976f,-25.7445f,-45.3385f,  -4.0858f,-26.8392f,-45.5924f,  -2.6447f,-28.2227f,-45.4868f,  
	-3.4787f,-29.3453f,-45.7175f,  -2.1189f,-30.7631f,-45.3396f,  -3.0056f,-31.9100f,-45.5447f,  -1.7529f,-33.3304f,-44.9003f,  -2.6955f,-34.4962f,-45.0764f,  
	-1.5727f,-35.8862f,-44.1767f,  -2.5782f,-37.0654f,-44.3225f,  -1.6045f,-38.3910f,-43.1837f,  -2.6778f,-39.5751f,-43.2991f,  -1.8676f,-40.8022f,-41.9427f,  
	-3.0090f,-41.9808f,-42.0266f,  -2.3723f,-43.0765f,-40.4787f,  -3.5838f,-44.2434f,-40.5290f,  -3.1259f,-45.1763f,-38.8196f,  -4.4065f,-46.3167f,-38.8409f,  
	-4.1268f,-47.0582f,-37.0041f,  -5.4739f,-48.1627f,-36.9961f,  -5.3670f,-48.6875f,-35.0686f,  -6.7742f,-49.7433f,-35.0348f,  -6.8297f,-50.0309f,-33.0559f,  
	-8.2882f,-51.0265f,-32.9993f,  -8.4912f,-51.0615f,-31.0094f,  -9.9919f,-51.9873f,-30.9333f,  -10.3235f,-51.7601f,-28.9735f,  -11.8538f,-52.6068f,-28.8816f,  
	-12.2921f,-52.1143f,-26.9928f,  -13.8375f,-52.8745f,-26.8890f,  -14.3582f,-52.1203f,-25.1107f,  -15.9038f,-52.7887f,-24.9988f,  -16.4813f,-51.7835f,-23.3684f,  
	-18.0093f,-52.3570f,-23.2528f,  27.7695f,0.6242f,-30.5636f,  28.0135f,-1.4351f,-29.4192f,  28.6118f,-0.3861f,-29.0564f,  28.6495f,-2.3700f,-27.7689f,  
	29.2451f,-1.3264f,-27.4081f,  29.0661f,-3.2183f,-26.0057f,  29.6585f,-2.1807f,-25.6470f,  29.2559f,-3.9654f,-24.1596f,  29.8449f,-2.9343f,-23.8032f,  
	29.2158f,-4.5985f,-22.2623f,  29.8013f,-3.5742f,-21.9083f,  28.9464f,-5.1069f,-20.3463f,  29.5283f,-4.0896f,-19.9947f,  28.4524f,-5.4818f,-18.4443f,  
	29.0305f,-4.4717f,-18.0951f,  27.7423f,-5.7167f,-16.5889f,  28.3166f,-4.7139f,-16.2421f,  26.8281f,-5.8078f,-14.8118f,  27.3986f,-4.8120f,-14.4673f,  
	26.8281f,-5.8078f,-14.8118f,  26.2542f,-6.8080f,-15.1539f,  25.1554f,-6.7468f,-13.4834f,  24.5812f,-7.7451f,-13.8209f,  23.3164f,-7.5332f,-12.2855f,  
	22.7416f,-8.5291f,-12.6181f,  21.3323f,-8.1703f,-11.2443f,  20.7562f,-9.1639f,-11.5720f,  19.2263f,-8.6648f,-10.3835f,  18.6480f,-9.6560f,-10.7063f,  
	17.0235f,-9.0254f,-9.7237f,  16.4428f,-10.0131f,-10.0415f,  14.7513f,-9.2621f,-9.2817f,  14.1680f,-10.2448f,-9.5945f,  12.4384f,-9.3866f,-9.0710f,  
	11.8514f,-10.3642f,-9.3793f,  10.1131f,-9.4139f,-9.1014f,  9.5225f,-10.3850f,-9.4055f,  7.8051f,-9.3594f,-9.3785f,  7.2108f,-10.3228f,-9.6790f,  
	5.5436f,-9.2401f,-9.9039f,  4.9454f,-10.1949f,-10.2013f,  3.3569f,-9.0748f,-10.6749f,  2.7550f,-10.0197f,-10.9702f,  1.2723f,-8.8826f,-11.6850f,  
	0.6666f,-9.8172f,-11.9792f,  -0.6849f,-8.6842f,-12.9237f,  -1.2940f,-9.6082f,-13.2177f,  -2.4913f,-8.5008f,-14.3763f,  -3.1029f,-9.4134f,-14.6710f,  
	-4.1258f,-8.3531f,-16.0244f,  -4.7394f,-9.2547f,-16.3208f,  -5.5709f,-8.2627f,-17.8462f,  -6.1863f,-9.1542f,-18.1456f,  -6.8125f,-8.2508f,-19.8172f,  
	-7.4290f,-9.1334f,-20.1201f,  -7.8401f,-8.3380f,-21.9091f,  -8.4572f,-9.2131f,-22.2163f,  -8.6473f,-8.5436f,-24.0919f,  -9.2647f,-9.4134f,-24.4041f,  
	-9.2319f,-8.8858f,-26.3335f,  -9.8497f,-9.7530f,-26.6506f,  -9.5965f,-9.3811f,-28.5998f,  -10.2150f,-10.2490f,-28.9214f,  -9.7481f,-10.0443f,-30.8559f,  
	-10.3681f,-10.9163f,-31.1815f,  -9.6984f,-10.8872f,-33.0664f,  -10.3207f,-11.7669f,-33.3953f,  -9.4630f,-11.9186f,-35.1962f,  -10.0902f,-12.8106f,-35.5265f,  
	-9.0632f,-13.1450f,-37.2104f,  -9.6968f,-14.0526f,-37.5406f,  -8.5223f,-14.5679f,-39.0759f,  -9.1660f,-15.4954f,-39.4037f,  -7.8691f,-16.1861f,-40.7612f,  
	-8.5265f,-17.1370f,-41.0838f,  -7.1344f,-17.9934f,-42.2373f,  -7.8105f,-18.9711f,-42.5517f,  -6.3526f,-19.9799f,-43.4784f,  -7.0530f,-20.9864f,-43.7811f,  
	-5.5597f,-22.1301f,-44.4623f,  -6.2894f,-23.1672f,-44.7502f,  -4.7917f,-24.4247f,-45.1714f,  -5.5573f,-25.4924f,-45.4408f,  -4.0858f,-26.8392f,-45.5924f,  
	-4.8943f,-27.9371f,-45.8400f,  -3.4787f,-29.3453f,-45.7175f,  -4.3376f,-30.4708f,-45.9404f,  -3.0056f,-31.9100f,-45.5447f,  -3.9185f,-33.0585f,-45.7393f,  
	-2.6955f,-34.4962f,-45.0764f,  -3.6703f,-35.6633f,-45.2409f,  -2.5782f,-37.0654f,-44.3225f,  -3.6211f,-38.2432f,-44.4560f,  -2.6778f,-39.5751f,-43.2991f,  
	-3.7901f,-40.7539f,-43.4001f,  -3.0090f,-41.9808f,-42.0266f,  -4.1935f,-43.1547f,-42.0928f,  -3.5838f,-44.2434f,-40.5290f,  -4.8416f,-45.3980f,-40.5639f,  
	-4.4065f,-46.3167f,-38.8409f,  -5.7358f,-47.4426f,-38.8441f,  -5.4739f,-48.1627f,-36.9961f,  -6.8699f,-49.2467f,-36.9704f,  -6.7742f,-49.7433f,-35.0348f,  
	-8.2295f,-50.7733f,-34.9829f,  -8.2882f,-51.0265f,-32.9993f,  -9.7947f,-51.9922f,-32.9240f,  -9.9919f,-51.9873f,-30.9333f,  -11.5380f,-52.8783f,-30.8383f,  
	-11.8538f,-52.6068f,-28.8816f,  -13.4257f,-53.4144f,-28.7708f,  -13.8375f,-52.8745f,-26.8890f,  -15.4206f,-53.5920f,-26.7661f,  -15.9038f,-52.7887f,-24.9988f,  
	-17.4801f,-53.4114f,-24.8685f,  -18.0093f,-52.3570f,-23.2528f,  -19.5623f,-52.8829f,-23.1182f,  28.0135f,-1.4351f,-29.4192f,  28.0507f,-3.4159f,-28.1265f,  
	28.6495f,-2.3700f,-27.7689f,  28.4702f,-4.2587f,-26.3614f,  29.0661f,-3.2183f,-26.0057f,  28.6633f,-4.9998f,-24.5132f,  29.2559f,-3.9654f,-24.1596f,  
	28.6267f,-5.6265f,-22.6137f,  29.2158f,-4.5985f,-22.2623f,  28.3611f,-6.1281f,-20.6954f,  28.9464f,-5.1069f,-20.3463f,  27.8709f,-6.4960f,-18.7911f,  
	28.4524f,-5.4818f,-18.4443f,  27.1646f,-6.7240f,-16.9333f,  27.7423f,-5.7167f,-16.5889f,  26.2542f,-6.8080f,-15.1539f,  26.8281f,-5.8078f,-14.8118f,  
	26.2542f,-6.8080f,-15.1539f,  25.6762f,-7.8131f,-15.4937f,  24.5812f,-7.7451f,-13.8209f,  24.0026f,-8.7477f,-14.1556f,  22.7416f,-8.5291f,-12.6181f,  
	22.1617f,-9.5293f,-12.9479f,  20.7562f,-9.1639f,-11.5720f,  20.1742f,-10.1617f,-11.8968f,  18.6480f,-9.6560f,-10.7063f,  18.0635f,-10.6503f,-11.0260f,  
	16.4428f,-10.0131f,-10.0415f,  15.8556f,-11.0026f,-10.3560f,  14.1680f,-10.2448f,-9.5945f,  13.5771f,-11.2294f,-9.9043f,  11.8514f,-10.3642f,-9.3793f,  
	11.2568f,-11.3424f,-9.6846f,  9.5225f,-10.3850f,-9.4055f,  8.9239f,-11.3554f,-9.7068f,  7.2108f,-10.3228f,-9.6790f,  6.6081f,-11.2844f,-9.9768f,  
	4.9454f,-10.1949f,-10.2013f,  4.3387f,-11.1464f,-10.4965f,  2.7550f,-10.0197f,-10.9702f,  2.1441f,-10.9604f,-11.2638f,  0.6666f,-9.8172f,-11.9792f,  
	0.0521f,-10.7466f,-12.2722f,  -1.2940f,-9.6082f,-13.2177f,  -1.9112f,-10.5252f,-13.5109f,  -3.1029f,-9.4134f,-14.6710f,  -3.7224f,-10.3181f,-14.9655f,  
	-4.7394f,-9.2547f,-16.3208f,  -5.3610f,-10.1478f,-16.6180f,  -6.1863f,-9.1542f,-18.1456f,  -6.8090f,-10.0364f,-18.4461f,  -7.4290f,-9.1334f,-20.1201f,  
	-8.0522f,-10.0060f,-20.4248f,  -8.4572f,-9.2131f,-22.2163f,  -9.0806f,-10.0779f,-22.5261f,  -9.2647f,-9.4134f,-24.4041f,  -9.8880f,-10.2729f,-24.7192f,  
	-9.8497f,-9.7530f,-26.6506f,  -10.4728f,-10.6102f,-26.9707f,  -10.2150f,-10.2490f,-28.9214f,  -10.8385f,-11.1074f,-29.2465f,  -10.3681f,-10.9163f,-31.1815f,  
	-10.9924f,-11.7793f,-31.5109f,  -10.3207f,-11.7669f,-33.3953f,  -10.9481f,-12.6391f,-33.7274f,  -10.0902f,-12.8106f,-35.5265f,  -10.7218f,-13.6955f,-35.8602f,  
	-9.6968f,-14.0526f,-37.5406f,  -10.3359f,-14.9549f,-37.8736f,  -9.1660f,-15.4954f,-39.4037f,  -9.8160f,-16.4189f,-39.7333f,  -8.5265f,-17.1370f,-41.0838f,  
	-9.1919f,-18.0856f,-41.4072f,  -7.8105f,-18.9711f,-42.5517f,  -8.4968f,-19.9477f,-42.8654f,  -7.0530f,-20.9864f,-43.7811f,  -7.7652f,-21.9934f,-44.0820f,  
	-6.2894f,-23.1672f,-44.7502f,  -7.0341f,-24.2055f,-45.0344f,  -5.5573f,-25.4924f,-45.4408f,  -6.3416f,-26.5626f,-45.7049f,  -4.8943f,-27.9371f,-45.8400f,  
	-5.7261f,-29.0374f,-46.0806f,  -4.3376f,-30.4708f,-45.9404f,  -5.2209f,-31.5977f,-46.1538f,  -3.9185f,-33.0585f,-45.7393f,  -4.8621f,-34.2085f,-45.9231f,  
	-3.6703f,-35.6633f,-45.2409f,  -4.6814f,-36.8295f,-45.3937f,  -3.6211f,-38.2432f,-44.4560f,  -4.7021f,-39.4166f,-44.5757f,  -3.7901f,-40.7539f,-43.4001f,  
	-4.9445f,-41.9290f,-43.4841f,  -4.1935f,-43.1547f,-42.0928f,  -5.4244f,-44.3178f,-42.1437f,  -4.8416f,-45.3980f,-40.5639f,  -6.1484f,-46.5396f,-40.5809f,  
	-5.7358f,-47.4426f,-38.8441f,  -7.1154f,-48.5495f,-38.8293f,  -6.8699f,-49.2467f,-36.9704f,  -8.3161f,-50.3063f,-36.9262f,  -8.2295f,-50.7733f,-34.9829f,  
	-9.7355f,-51.7747f,-34.9118f,  -9.7947f,-51.9922f,-32.9240f,  -11.3502f,-52.9241f,-32.8294f,  -11.5380f,-52.8783f,-30.8383f,  -13.1299f,-53.7308f,-30.7239f,  
	-13.4257f,-53.4144f,-28.7708f,  -15.0400f,-54.1794f,-28.6402f,  -15.4206f,-53.5920f,-26.7661f,  -17.0395f,-54.2632f,-26.6241f,  -17.4801f,-53.4114f,-24.8685f,  
	-19.0865f,-53.9852f,-24.7184f,  -19.5623f,-52.8829f,-23.1182f,  -21.1409f,-53.3583f,-22.9626f,  28.0507f,-3.4159f,-28.1265f,  27.8709f,-5.3017f,-26.7139f,  
	28.4702f,-4.2587f,-26.3614f,  28.0671f,-6.0373f,-24.8638f,  28.6633f,-4.9998f,-24.5132f,  28.0338f,-6.6580f,-22.9623f,  28.6267f,-5.6265f,-22.6137f,  
	27.7718f,-7.1533f,-21.0419f,  28.3611f,-6.1281f,-20.6954f,  27.2853f,-7.5146f,-19.1353f,  27.8709f,-6.4960f,-18.7911f,  26.5828f,-7.7358f,-17.2753f,  
	27.1646f,-6.7240f,-16.9333f,  25.6762f,-7.8131f,-15.4937f,  26.2542f,-6.8080f,-15.1539f,  25.6762f,-7.8131f,-15.4937f,  25.0938f,-8.8224f,-15.8307f,  
	24.0026f,-8.7477f,-14.1556f,  23.4189f,-9.7546f,-14.4876f,  22.1617f,-9.5293f,-12.9479f,  21.5759f,-10.5338f,-13.2748f,  20.1742f,-10.1617f,-11.8968f,  
	19.5858f,-11.1629f,-12.2185f,  18.0635f,-10.6503f,-11.0260f,  17.4723f,-11.6470f,-11.3424f,  15.8556f,-11.0026f,-10.3560f,  15.2607f,-11.9945f,-10.6674f,  
	13.5771f,-11.2294f,-9.9043f,  12.9783f,-12.2151f,-10.2109f,  11.2568f,-11.3424f,-9.6846f,  10.6538f,-12.3205f,-9.9868f,  8.9239f,-11.3554f,-9.7068f,  
	8.3166f,-12.3247f,-10.0052f,  6.6081f,-11.2844f,-9.9768f,  5.9965f,-12.2435f,-10.2721f,  4.3387f,-11.1464f,-10.4965f,  3.7225f,-12.0945f,-10.7897f,  
	2.1441f,-10.9604f,-11.2638f,  1.5240f,-11.8967f,-11.5558f,  0.0521f,-10.7466f,-12.2722f,  -0.5712f,-11.6696f,-12.5639f,  -1.9112f,-10.5252f,-13.5109f,  
	-2.5371f,-11.4348f,-13.8034f,  -3.7224f,-10.3181f,-14.9655f,  -4.3507f,-11.2147f,-15.2603f,  -5.3610f,-10.1478f,-16.6180f,  -5.9906f,-11.0317f,-16.9158f,  
	-6.8090f,-10.0364f,-18.4461f,  -7.4393f,-10.9087f,-18.7481f,  -8.0522f,-10.0060f,-20.4248f,  -8.6827f,-10.8681f,-20.7319f,  -9.0806f,-10.0779f,-22.5261f,  
	-9.7105f,-10.9320f,-22.8387f,  -9.8880f,-10.2729f,-24.7192f,  -10.5171f,-11.1217f,-25.0372f,  -10.4728f,-10.6102f,-26.9707f,  -11.1014f,-11.4571f,-27.2944f,  
	-10.8385f,-11.1074f,-29.2465f,  -11.4667f,-11.9556f,-29.5754f,  -10.9924f,-11.7793f,-31.5109f,  -11.6220f,-12.6334f,-31.8438f,  -10.9481f,-12.6391f,-33.7274f,  
	-11.5799f,-13.5029f,-34.0633f,  -10.7218f,-13.6955f,-35.8602f,  -11.3586f,-14.5736f,-36.1970f,  -10.3359f,-14.9549f,-37.8736f,  -10.9808f,-15.8516f,-38.2090f,  
	-9.8160f,-16.4189f,-39.7333f,  -10.4731f,-17.3386f,-40.0645f,  -9.1919f,-18.0856f,-41.4072f,  -9.8665f,-19.0318f,-41.7308f,  -8.4968f,-19.9477f,-42.8654f,  
	-9.1937f,-20.9234f,-43.1783f,  -7.7652f,-21.9934f,-44.0820f,  -8.4908f,-23.0005f,-44.3803f,  -7.0341f,-24.2055f,-45.0344f,  -7.7957f,-25.2453f,-45.3145f,  
	-6.3416f,-26.5626f,-45.7049f,  -7.1474f,-27.6345f,-45.9630f,  -5.7261f,-29.0374f,-46.0806f,  -6.5805f,-30.1389f,-46.3130f,  -5.2209f,-31.5977f,-46.1538f,  
	-6.1332f,-32.7261f,-46.3575f,  -4.8621f,-34.2085f,-45.9231f,  -5.8406f,-35.3581f,-46.0960f,  -4.6814f,-36.8295f,-45.3937f,  -5.7295f,-37.9920f,-45.5333f,  
	-4.7021f,-39.4166f,-44.5757f,  -5.8239f,-40.5872f,-44.6792f,  -4.9445f,-41.9290f,-43.4841f,  -6.1451f,-43.0945f,-43.5530f,  -5.4244f,-44.3178f,-42.1437f,  
	-6.7044f,-45.4692f,-42.1768f,  -6.1484f,-46.5396f,-40.5809f,  -7.5063f,-47.6637f,-40.5799f,  -7.1154f,-48.5495f,-38.8293f,  -8.5466f,-49.6334f,-38.7960f,  
	-8.3161f,-50.3063f,-36.9262f,  -9.8153f,-51.3387f,-36.8626f,  -9.7355f,-51.7747f,-34.9118f,  -11.2935f,-52.7435f,-34.8210f,  -11.3502f,-52.9241f,-32.8294f,  
	-12.9551f,-53.8182f,-32.7148f,  -13.1299f,-53.7308f,-30.7239f,  -14.7686f,-54.5411f,-30.5889f,  -15.0400f,-54.1794f,-28.6402f,  -16.6951f,-54.8979f,-28.4898f,  
	-17.0395f,-54.2632f,-26.6241f,  -18.6937f,-54.8847f,-26.4616f,  -19.0865f,-53.9852f,-24.7184f,  -20.7242f,-54.5072f,-24.5464f,  -21.1409f,-53.3583f,-22.9626f,  
	-22.7425f,-53.7806f,-22.7856f,  27.8709f,-5.3017f,-26.7139f,  27.4664f,-7.0784f,-25.2114f,  28.0671f,-6.0373f,-24.8638f,  27.4365f,-7.6934f,-23.3080f,  
	28.0338f,-6.6580f,-22.9623f,  27.1779f,-8.1825f,-21.3855f,  27.7718f,-7.1533f,-21.0419f,  26.6952f,-8.5373f,-19.4768f,  27.2853f,-7.5146f,-19.1353f,  
	25.9965f,-8.7519f,-17.6146f,  26.5828f,-7.7358f,-17.2753f,  25.0938f,-8.8224f,-15.8307f,  25.6762f,-7.8131f,-15.4937f,  25.0938f,-8.8224f,-15.8307f,  
	24.5061f,-9.8361f,-16.1649f,  23.4189f,-9.7546f,-14.4876f,  22.8291f,-10.7660f,-14.8167f,  21.5759f,-10.5338f,-13.2748f,  20.9835f,-11.5421f,-13.5985f,  
	19.5858f,-11.1629f,-12.2185f,  18.9905f,-12.1670f,-12.5368f,  17.4723f,-11.6470f,-11.3424f,  16.8733f,-12.6466f,-11.6555f,  15.2607f,-11.9945f,-10.6674f,  
	14.6575f,-12.9881f,-10.9755f,  12.9783f,-12.2151f,-10.2109f,  12.3708f,-13.2013f,-10.5143f,  10.6538f,-12.3205f,-9.9868f,  10.0416f,-13.2981f,-10.2860f,  
	8.3166f,-12.3247f,-10.0052f,  7.6998f,-13.2923f,-10.3008f,  5.9965f,-12.2435f,-10.2721f,  5.3747f,-13.1999f,-10.5651f,  3.7225f,-12.0945f,-10.7897f,  
	3.0964f,-13.0387f,-11.0809f,  1.5240f,-11.8967f,-11.5558f,  0.8943f,-12.8270f,-11.8462f,  -0.5712f,-11.6696f,-12.5639f,  -1.2040f,-12.5856f,-12.8545f,  
	-2.5371f,-11.4348f,-13.8034f,  -3.1728f,-12.3366f,-14.0959f,  -4.3507f,-11.2147f,-15.2603f,  -4.9880f,-12.1023f,-15.5554f,  -5.9906f,-11.0317f,-16.9158f,  
	-6.6288f,-11.9057f,-17.2147f,  -7.4393f,-10.9087f,-18.7481f,  -8.0777f,-11.7702f,-19.0519f,  -8.6827f,-10.8681f,-20.7319f,  -9.3205f,-11.7190f,-21.0413f,  
	-9.7105f,-10.9320f,-22.8387f,  -10.3471f,-11.7748f,-23.1539f,  -10.5171f,-11.1217f,-25.0372f,  -11.1524f,-11.9594f,-25.3587f,  -11.1014f,-11.4571f,-27.2944f,  
	-11.7353f,-12.2927f,-27.6219f,  -11.4667f,-11.9556f,-29.5754f,  -12.1004f,-12.7938f,-29.9080f,  -11.6220f,-12.6334f,-31.8438f,  -12.2562f,-13.4779f,-32.1808f,  
	-11.5799f,-13.5029f,-34.0633f,  -12.2168f,-14.3583f,-34.4028f,  -11.3586f,-14.5736f,-36.1970f,  -12.0010f,-15.4447f,-36.5369f,  -10.9808f,-15.8516f,-38.2090f,  
	-11.6322f,-16.7429f,-38.5467f,  -10.4731f,-17.3386f,-40.0645f,  -11.1385f,-18.2544f,-40.3968f,  -9.8665f,-19.0318f,-41.7308f,  -10.5505f,-19.9756f,-42.0545f,  
	-9.1937f,-20.9234f,-43.1783f,  -9.9026f,-21.8979f,-43.4897f,  -8.4908f,-23.0005f,-44.3803f,  -9.2318f,-24.0077f,-44.6755f,  -7.7957f,-25.2453f,-45.3145f,  
	-8.5771f,-26.2859f,-45.5896f,  -7.1474f,-27.6345f,-45.9630f,  -7.9739f,-28.7068f,-46.2141f,  -6.5805f,-30.1389f,-46.3130f,  -7.4619f,-31.2414f,-46.5367f,  
	-6.1332f,-32.7261f,-46.3575f,  -7.0788f,-33.8541f,-46.5510f,  -5.8406f,-35.3581f,-46.0960f,  -6.8547f,-36.5043f,-46.2565f,  -5.7295f,-37.9920f,-45.5333f,  
	-6.8166f,-39.1522f,-45.6576f,  -5.8239f,-40.5872f,-44.6792f,  -6.9913f,-41.7492f,-44.7680f,  -6.1451f,-43.0945f,-43.5530f,  -7.3945f,-44.2496f,-43.6046f,  
	-6.7044f,-45.4692f,-42.1768f,  -8.0360f,-46.6044f,-42.1922f,  -7.5063f,-47.6637f,-40.5799f,  -8.9170f,-48.7661f,-40.5603f,  -8.5466f,-49.6334f,-38.7960f,  
	-10.0326f,-50.6914f,-38.7430f,  -9.8153f,-51.3387f,-36.8626f,  -11.3689f,-52.3396f,-36.7788f,  -11.2935f,-52.7435f,-34.8210f,  -12.9042f,-53.6753f,-34.7096f,  
	-12.9551f,-53.8182f,-32.7148f,  -14.6109f,-54.6707f,-32.5790f,  -14.7686f,-54.5411f,-30.5889f,  -16.4527f,-55.3046f,-30.4335f,  -16.6951f,-54.8979f,-28.4898f,  
	-18.3905f,-55.5658f,-28.3180f,  -18.6937f,-54.8847f,-26.4616f,  -20.3845f,-55.4530f,-26.2762f,  -20.7242f,-54.5072f,-24.5464f,  -22.3902f,-54.9740f,-24.3521f,  
	-22.7425f,-53.7806f,-22.7856f,  -24.3665f,-54.1469f,-22.5853f,  27.4664f,-7.0784f,-25.2114f,  26.8342f,-8.7326f,-23.6506f,  27.4365f,-7.6934f,-23.3080f,  
	26.5790f,-9.2158f,-21.7261f,  27.1779f,-8.1825f,-21.3855f,  26.0999f,-9.5644f,-19.8154f,  26.6952f,-8.5373f,-19.4768f,  25.4049f,-9.7723f,-17.9510f,  
	25.9965f,-8.7519f,-17.6146f,  24.5061f,-9.8361f,-16.1649f,  25.0938f,-8.8224f,-15.8307f,  24.5061f,-9.8361f,-16.1649f,  23.9125f,-10.8543f,-16.4961f,  
	22.8291f,-10.7660f,-14.8167f,  22.2329f,-11.7814f,-15.1425f,  20.9835f,-11.5421f,-13.5985f,  20.3842f,-12.5537f,-13.9189f,  18.9905f,-12.1670f,-12.5368f,  
	18.3873f,-13.1744f,-12.8519f,  16.8733f,-12.6466f,-11.6555f,  16.2658f,-13.6484f,-11.9653f,  14.6575f,-12.9881f,-10.9755f,  14.0455f,-13.9829f,-11.2802f,  
	12.3708f,-13.2013f,-10.5143f,  11.7538f,-14.1878f,-10.8144f,  10.0416f,-13.2981f,-10.2860f,  9.4197f,-14.2747f,-10.5820f,  7.6998f,-13.2923f,-10.3008f,  
	7.0724f,-14.2578f,-10.5937f,  5.3747f,-13.1999f,-10.5651f,  4.7424f,-14.1531f,-10.8556f,  3.0964f,-13.0387f,-11.0809f,  2.4600f,-13.9775f,-11.3700f,  
	0.8943f,-12.8270f,-11.8462f,  0.2543f,-13.7509f,-12.1350f,  -1.2040f,-12.5856f,-12.8545f,  -1.8474f,-13.4943f,-13.1446f,  -3.1728f,-12.3366f,-14.0959f,  
	-3.8182f,-13.2297f,-14.3880f,  -4.9880f,-12.1023f,-15.5554f,  -5.6348f,-12.9801f,-15.8510f,  -6.6288f,-11.9057f,-17.2147f,  -7.2761f,-12.7690f,-17.5150f,  
	-8.0777f,-11.7702f,-19.0519f,  -8.7244f,-12.6205f,-19.3578f,  -9.3205f,-11.7190f,-21.0413f,  -9.9657f,-12.5582f,-21.3531f,  -10.3471f,-11.7748f,-23.1539f,  
	-10.9906f,-12.6058f,-23.4724f,  -11.1524f,-11.9594f,-25.3587f,  -11.7938f,-12.7850f,-25.6838f,  -11.7353f,-12.2927f,-27.6219f,  -12.3752f,-13.1174f,-27.9530f,  
	-12.1004f,-12.7938f,-29.9080f,  -12.7393f,-13.6211f,-30.2447f,  -12.2562f,-13.4779f,-32.1808f,  -12.8956f,-14.3125f,-32.5215f,  -12.2168f,-14.3583f,-34.4028f,  
	-12.8591f,-15.2052f,-34.7458f,  -12.0010f,-15.4447f,-36.5369f,  -12.6494f,-16.3087f,-36.8797f,  -11.6322f,-16.7429f,-38.5467f,  -12.2912f,-17.6288f,-38.8861f,  
	-11.1385f,-18.2544f,-40.3968f,  -11.8124f,-19.1661f,-40.7300f,  -10.5505f,-19.9756f,-42.0545f,  -11.2453f,-20.9166f,-42.3777f,  -9.9026f,-21.8979f,-43.4897f,  
	-10.6253f,-22.8711f,-43.7989f,  -9.2318f,-24.0077f,-44.6755f,  -9.9908f,-25.0148f,-44.9667f,  -8.5771f,-26.2859f,-45.5896f,  -9.3773f,-27.3261f,-45.8588f,  
	-7.9739f,-28.7068f,-46.2141f,  -8.8255f,-29.7797f,-46.4575f,  -7.4619f,-31.2414f,-46.5367f,  -8.3749f,-32.3434f,-46.7509f,  -7.0788f,-33.8541f,-46.5510f,  
	-8.0583f,-34.9790f,-46.7329f,  -6.8547f,-36.5043f,-46.2565f,  -7.9058f,-37.6486f,-46.4026f,  -6.8166f,-39.1522f,-45.6576f,  -7.9486f,-40.3047f,-45.7676f,  
	-6.9913f,-41.7492f,-44.7680f,  -8.2067f,-42.9018f,-44.8398f,  -7.3945f,-44.2496f,-43.6046f,  -8.6954f,-45.3898f,-43.6386f,  -8.0360f,-46.6044f,-42.1922f,  
	-9.4209f,-47.7194f,-42.1889f,  -8.9170f,-48.7661f,-40.5603f,  -10.3837f,-49.8440f,-40.5209f,  -10.0326f,-50.6914f,-38.7430f,  -11.5752f,-51.7192f,-38.6695f,  
	-11.3689f,-52.3396f,-36.7788f,  -12.9783f,-53.3044f,-36.6740f,  -12.9042f,-53.6753f,-34.7096f,  -14.5696f,-54.5661f,-34.5764f,  -14.6109f,-54.6707f,-32.5790f,  
	-16.3165f,-55.4764f,-32.4219f,  -16.4527f,-55.3046f,-30.4335f,  -18.1820f,-56.0171f,-30.2558f,  -18.3905f,-55.5658f,-28.3180f,  -20.1278f,-56.1796f,-28.1223f,  
	-20.3845f,-55.4530f,-26.2762f,  -22.1092f,-55.9643f,-26.0674f,  -22.3902f,-54.9740f,-24.3521f,  -24.0843f,-55.3823f,-24.1334f,  -24.3665f,-54.1469f,-22.5853f,  
	-26.0098f,-54.4547f,-22.3613f,  26.8342f,-8.7326f,-23.6506f,  25.9746f,-10.2532f,-22.0635f,  26.5790f,-9.2158f,-21.7261f,  25.4989f,-10.5957f,-20.1507f,  
	26.0999f,-9.5644f,-19.8154f,  24.8076f,-10.7972f,-18.2843f,  25.4049f,-9.7723f,-17.9510f,  23.9125f,-10.8543f,-16.4961f,  24.5061f,-9.8361f,-16.1649f,  
	23.9125f,-10.8543f,-16.4961f,  23.3124f,-11.8767f,-16.8240f,  22.2329f,-11.7814f,-15.1425f,  21.6296f,-12.8005f,-15.4650f,  20.3842f,-12.5537f,-13.9189f,  
	19.7769f,-13.5689f,-14.2359f,  18.3873f,-13.1744f,-12.8519f,  17.7756f,-14.1845f,-13.1633f,  16.2658f,-13.6484f,-11.9653f,  15.6492f,-14.6519f,-12.2714f,  
	14.0455f,-13.9829f,-11.2802f,  13.4236f,-14.9786f,-11.5814f,  11.7538f,-14.1878f,-10.8144f,  11.1266f,-15.1739f,-11.1111f,  9.4197f,-14.2747f,-10.5820f,  
	8.7865f,-15.2501f,-10.8750f,  7.0724f,-14.2578f,-10.5937f,  6.4339f,-15.2208f,-10.8838f,  4.7424f,-14.1531f,-10.8556f,  4.0993f,-15.1016f,-11.1436f,  
	2.4600f,-13.9775f,-11.3700f,  1.8127f,-14.9107f,-11.6572f,  0.2543f,-13.7509f,-12.1350f,  -0.3971f,-14.6681f,-12.4227f,  -1.8474f,-13.4943f,-13.1446f,  
	-2.5013f,-14.3948f,-13.4338f,  -3.8182f,-13.2297f,-14.3880f,  -4.4741f,-14.1134f,-14.6803f,  -5.6348f,-12.9801f,-15.8510f,  -6.2916f,-13.8474f,-16.1475f,  
	-7.2761f,-12.7690f,-17.5150f,  -7.9327f,-13.6210f,-17.8169f,  -8.7244f,-12.6205f,-19.3578f,  -9.3795f,-13.4586f,-19.6656f,  -9.9657f,-12.5582f,-21.3531f,  
	-10.6188f,-13.3852f,-21.6678f,  -10.9906f,-12.6058f,-23.4724f,  -11.6410f,-13.4239f,-23.7942f,  -11.7938f,-12.7850f,-25.6838f,  -12.4417f,-13.5986f,-26.0124f,  
	-12.3752f,-13.1174f,-27.9530f,  -13.0208f,-13.9299f,-28.2883f,  -12.7393f,-13.6211f,-30.2447f,  -13.3835f,-14.4371f,-30.5854f,  -12.8956f,-14.3125f,-32.5215f,  
	-13.5405f,-15.1372f,-32.8661f,  -12.8591f,-15.2052f,-34.7458f,  -13.5072f,-16.0434f,-35.0920f,  -12.6494f,-16.3087f,-36.8797f,  -13.3048f,-17.1657f,-37.2247f,  
	-12.2912f,-17.6288f,-38.8861f,  -12.9577f,-18.5088f,-39.2273f,  -11.8124f,-19.1661f,-40.7300f,  -12.4959f,-20.0735f,-41.0634f,  -11.2453f,-20.9166f,-42.3777f,  
	-11.9523f,-21.8549f,-42.6997f,  -10.6253f,-22.8711f,-43.7989f,  -11.3642f,-23.8429f,-44.1051f,  -9.9908f,-25.0148f,-44.9667f,  -10.7668f,-26.0204f,-45.2532f,  
	-9.3773f,-27.3261f,-45.8588f,  -10.2006f,-28.3661f,-46.1214f,  -8.8255f,-29.7797f,-46.4575f,  -9.7067f,-30.8517f,-46.6923f,  -8.3749f,-32.3434f,-46.7509f,  
	-9.3199f,-33.4423f,-46.9544f,  -8.0583f,-34.9790f,-46.7329f,  -9.0727f,-36.1022f,-46.9015f,  -7.9058f,-37.6486f,-46.4026f,  -9.0005f,-38.7859f,-46.5350f,  
	-7.9486f,-40.3047f,-45.7676f,  -9.1273f,-41.4488f,-45.8613f,  -8.2067f,-42.9018f,-44.8398f,  -9.4732f,-44.0407f,-44.8944f,  -8.6954f,-45.3898f,-43.6386f,  
	-10.0497f,-46.5110f,-43.6540f,  -9.4209f,-47.7194f,-42.1889f,  -10.8627f,-48.8111f,-42.1658f,  -10.3837f,-49.8440f,-40.5209f,  -11.9087f,-50.8930f,-40.4607f,  
	-11.5752f,-51.7192f,-38.6695f,  -13.1759f,-52.7120f,-38.5745f,  -12.9783f,-53.3044f,-36.6740f,  -14.6455f,-54.2291f,-36.5469f,  -14.5696f,-54.5661f,-34.5764f,  
	-16.2889f,-55.4104f,-34.4212f,  -16.3165f,-55.4764f,-32.4219f,  -18.0720f,-56.2307f,-32.2418f,  -18.1820f,-56.0171f,-30.2558f,  -19.9584f,-56.6746f,-30.0532f,  
	-20.1278f,-56.1796f,-28.1223f,  -21.9044f,-56.7347f,-27.9023f,  -22.1092f,-55.9643f,-26.0674f,  -23.8678f,-56.4148f,-25.8331f,  -24.0843f,-55.3823f,-24.1334f,  
	-25.8032f,-55.7288f,-23.8897f,  -26.0098f,-54.4547f,-22.3613f,  -27.6723f,-54.7008f,-22.1110f,  25.9746f,-10.2532f,-22.0635f,  24.8915f,-11.6313f,-20.4828f,  
	25.4989f,-10.5957f,-20.1507f,  24.2038f,-11.8264f,-18.6143f,  24.8076f,-10.7972f,-18.2843f,  23.3124f,-11.8767f,-16.8240f,  23.9125f,-10.8543f,-16.4961f,  
	23.3124f,-11.8767f,-16.8240f,  22.7053f,-12.9031f,-17.1485f,  21.6296f,-12.8005f,-15.4650f,  21.0183f,-13.8234f,-15.7839f,  19.7769f,-13.5689f,-14.2359f,  
	19.1611f,-14.5873f,-14.5491f,  17.7756f,-14.1845f,-13.1633f,  17.1546f,-15.1969f,-13.4710f,  15.6492f,-14.6519f,-12.2714f,  15.0227f,-15.6570f,-12.5738f,  
	13.4236f,-14.9786f,-11.5814f,  12.7913f,-15.9747f,-11.8789f,  11.1266f,-15.1739f,-11.1111f,  10.4879f,-16.1597f,-11.4044f,  8.7865f,-15.2501f,-10.8750f,  
	8.1420f,-16.2236f,-11.1648f,  6.4339f,-15.2208f,-10.8838f,  5.7842f,-16.1799f,-11.1709f,  4.0993f,-15.1016f,-11.1436f,  3.4446f,-16.0451f,-11.4292f,  
	1.8127f,-14.9107f,-11.6572f,  1.1532f,-15.8378f,-11.9426f,  -0.3971f,-14.6681f,-12.4227f,  -1.0598f,-15.5775f,-12.7091f,  -2.5013f,-14.3948f,-13.4338f,  
	-3.1666f,-15.2862f,-13.7226f,  -4.4741f,-14.1134f,-14.6803f,  -5.1410f,-14.9868f,-14.9728f,  -6.2916f,-13.8474f,-16.1475f,  -6.9587f,-14.7034f,-16.4451f,  
	-7.9327f,-13.6210f,-17.8169f,  -8.5985f,-14.4606f,-18.1203f,  -9.3795f,-13.4586f,-19.6656f,  -10.0434f,-14.2842f,-19.9759f,  -10.6188f,-13.3852f,-21.6678f,  
	-11.2797f,-14.1987f,-21.9855f,  -11.6410f,-13.4239f,-23.7942f,  -12.2987f,-14.2293f,-24.1192f,  -12.4417f,-13.5986f,-26.0124f,  -13.0960f,-14.3991f,-26.3451f,  
	-13.0208f,-13.9299f,-28.2883f,  -13.6721f,-14.7299f,-28.6276f,  -13.3835f,-14.4371f,-30.5854f,  -14.0335f,-15.2419f,-30.9301f,  -13.5405f,-15.1372f,-32.8661f,  
	-14.1910f,-15.9515f,-33.2142f,  -13.5072f,-16.0434f,-35.0920f,  -14.1619f,-16.8729f,-35.4410f,  -13.3048f,-17.1657f,-37.2247f,  -13.9670f,-18.0150f,-37.5721f,  
	-12.9577f,-18.5088f,-39.2273f,  -13.6330f,-19.3829f,-39.5694f,  -12.4959f,-20.0735f,-41.0634f,  -13.1902f,-20.9765f,-41.3966f,  -11.9523f,-21.8549f,-42.6997f,  
	-12.6740f,-22.7903f,-43.0194f,  -11.3642f,-23.8429f,-44.1051f,  -12.1184f,-24.8121f,-44.4076f,  -10.7668f,-26.0204f,-45.2532f,  -11.5637f,-27.0248f,-45.5340f,  
	-10.2006f,-28.3661f,-46.1214f,  -11.0513f,-29.4046f,-46.3761f,  -9.7067f,-30.8517f,-46.6923f,  -10.6178f,-31.9204f,-46.9171f,  -9.3199f,-33.4423f,-46.9544f,  
	-10.2974f,-34.5393f,-47.1457f,  -9.0727f,-36.1022f,-46.9015f,  -10.1291f,-37.2188f,-47.0568f,  -9.0005f,-38.7859f,-46.5350f,  -10.1404f,-39.9154f,-46.6516f,  
	-9.1273f,-41.4488f,-45.8613f,  -10.3561f,-42.5800f,-45.9380f,  -9.4732f,-44.0407f,-44.8944f,  -10.7928f,-45.1616f,-44.9305f,  -10.0497f,-46.5110f,-43.6540f,  
	-11.4614f,-47.6101f,-43.6495f,  -10.8627f,-48.8111f,-42.1658f,  -12.3639f,-49.8750f,-42.1216f,  -11.9087f,-50.8930f,-40.4607f,  -13.4938f,-51.9080f,-40.3786f,  
	-13.1759f,-52.7120f,-38.5745f,  -14.8373f,-53.6654f,-38.4567f,  -14.6455f,-54.2291f,-36.5469f,  -16.3703f,-55.1076f,-36.3970f,  -16.2889f,-55.4104f,-34.4212f,  
	-18.0621f,-56.2030f,-34.2421f,  -18.0720f,-56.2307f,-32.2418f,  -19.8792f,-56.9294f,-32.0359f,  -19.9584f,-56.6746f,-30.0532f,  -21.7792f,-57.2718f,-29.8253f,  
	-21.9044f,-56.7347f,-27.9023f,  -23.7204f,-57.2267f,-27.6555f,  -23.8678f,-56.4148f,-25.8331f,  -25.6565f,-56.8004f,-25.5727f,  -25.8032f,-55.7288f,-23.8897f,  
	-27.5468f,-56.0100f,-23.6184f,  -27.6723f,-54.7008f,-22.1110f,  -29.3535f,-54.8818f,-21.8320f,  24.8915f,-11.6313f,-20.4828f,  23.5929f,-12.8598f,-18.9408f,  
	24.2038f,-11.8264f,-18.6143f,  22.7053f,-12.9031f,-17.1485f,  23.3124f,-11.8767f,-16.8240f,  22.7053f,-12.9031f,-17.1485f,  22.0902f,-13.9335f,-17.4693f,  
	21.0183f,-13.8234f,-15.7839f,  20.3985f,-14.8499f,-16.0989f,  19.1611f,-14.5873f,-14.5491f,  18.5359f,-15.6085f,-14.8584f,  17.1546f,-15.1969f,-13.4710f,  
	16.5235f,-16.2115f,-13.7747f,  15.0227f,-15.6570f,-12.5738f,  14.3855f,-16.6632f,-12.8723f,  12.7913f,-15.9747f,-11.8789f,  12.1472f,-16.9711f,-12.1726f,  
	10.4879f,-16.1597f,-11.4044f,  9.8374f,-17.1443f,-11.6941f,  8.1420f,-16.2236f,-11.1648f,  7.4856f,-17.1941f,-11.4512f,  5.7842f,-16.1799f,-11.1709f,  
	5.1221f,-17.1348f,-11.4550f,  3.4446f,-16.0451f,-11.4292f,  2.7770f,-16.9834f,-11.7125f,  1.1532f,-15.8378f,-11.9426f,  0.4816f,-16.7577f,-12.2261f,  
	-1.0598f,-15.5775f,-12.7091f,  -1.7348f,-16.4785f,-12.9943f,  -3.1666f,-15.2862f,-13.7226f,  -3.8439f,-16.1677f,-14.0110f,  -5.1410f,-14.9868f,-14.9728f,  
	-5.8191f,-15.8491f,-15.2658f,  -6.9587f,-14.7034f,-16.4451f,  -7.6360f,-15.5470f,-16.7435f,  -8.5985f,-14.4606f,-18.1203f,  -9.2741f,-15.2875f,-18.4255f,  
	-10.0434f,-14.2842f,-19.9759f,  -10.7160f,-15.0959f,-20.2886f,  -11.2797f,-14.1987f,-21.9855f,  -11.9486f,-14.9987f,-22.3061f,  -12.2987f,-14.2293f,-24.1192f,  
	-12.9635f,-15.0207f,-24.4482f,  -13.0960f,-14.3991f,-26.3451f,  -13.7566f,-15.1860f,-26.6815f,  -13.6721f,-14.7299f,-28.6276f,  -14.3296f,-15.5174f,-28.9709f,  
	-14.0335f,-15.2419f,-30.9301f,  -14.6891f,-16.0347f,-31.2785f,  -14.1910f,-15.9515f,-33.2142f,  -14.8480f,-16.7556f,-33.5654f,  -14.1619f,-16.8729f,-35.4410f,  
	-14.8230f,-17.6930f,-35.7928f,  -13.9670f,-18.0150f,-37.5721f,  -14.6371f,-18.8566f,-37.9211f,  -13.6330f,-19.3829f,-39.5694f,  -14.3178f,-20.2508f,-39.9120f,  
	-13.1902f,-20.9765f,-41.3966f,  -13.8976f,-21.8750f,-41.7282f,  -12.6740f,-22.7903f,-43.0194f,  -13.4092f,-23.7217f,-43.3366f,  -12.1184f,-24.8121f,-44.4076f,  
	-12.8914f,-25.7787f,-44.7055f,  -11.5637f,-27.0248f,-45.5340f,  -12.3858f,-28.0268f,-45.8078f,  -11.0513f,-29.4046f,-46.3761f,  -11.9299f,-30.4394f,-46.6217f,  
	-10.6178f,-31.9204f,-46.9171f,  -11.5590f,-32.9868f,-47.1310f,  -10.2974f,-34.5393f,-47.1457f,  -11.3150f,-35.6300f,-47.3243f,  -10.1291f,-37.2188f,-47.0568f,  
	-11.2287f,-38.3279f,-47.1971f,  -10.1404f,-39.9154f,-46.6516f,  -11.3290f,-41.0329f,-46.7516f,  -10.3561f,-42.5800f,-45.9380f,  -11.6373f,-43.6945f,-45.9964f,  
	-10.7928f,-45.1616f,-44.9305f,  -12.1694f,-46.2619f,-44.9468f,  -11.4614f,-47.6101f,-43.6495f,  -12.9329f,-48.6829f,-43.6240f,  -12.3639f,-49.8750f,-42.1216f,  
	-13.9264f,-50.9063f,-42.0554f,  -13.4938f,-51.9080f,-40.3786f,  -15.1416f,-52.8847f,-40.2733f,  -14.8373f,-53.6654f,-38.4567f,  -16.5593f,-54.5732f,-38.3155f,  
	-16.3703f,-55.1076f,-36.3970f,  -18.1527f,-55.9345f,-36.2225f,  -18.0621f,-56.2030f,-34.2421f,  -19.8915f,-56.9397f,-34.0364f,  -19.8792f,-56.9294f,-32.0359f,  
	-21.7359f,-57.5666f,-31.8035f,  -21.7792f,-57.2718f,-29.8253f,  -23.6449f,-57.8043f,-29.5694f,  -23.7204f,-57.2267f,-27.6555f,  -25.5722f,-57.6511f,-27.3813f,  
	-25.6565f,-56.8004f,-25.5727f,  -27.4755f,-57.1171f,-25.2832f,  -27.5468f,-56.0100f,-23.6184f,  -29.3139f,-56.2217f,-23.3170f,  -29.3535f,-54.8818f,-21.8320f,  
	-31.0504f,-54.9948f,-21.5231f,  23.5929f,-12.8598f,-18.9408f,  22.0902f,-13.9335f,-17.4693f,  22.7053f,-12.9031f,-17.1485f,  22.0902f,-13.9335f,-17.4693f,  
	21.4667f,-14.9678f,-17.7862f,  20.3985f,-14.8499f,-16.0989f,  19.7693f,-15.8796f,-16.4099f,  18.5359f,-15.6085f,-14.8584f,  17.9005f,-16.6324f,-15.1636f,  
	16.5235f,-16.2115f,-13.7747f,  15.8817f,-17.2277f,-14.0743f,  14.3855f,-16.6632f,-12.8723f,  13.7362f,-17.6705f,-13.1666f,  12.1472f,-16.9711f,-12.1726f,  
	11.4911f,-17.9672f,-12.4624f,  9.8374f,-17.1443f,-11.6941f,  9.1746f,-18.1267f,-11.9799f,  7.4856f,-17.1941f,-11.4512f,  6.8165f,-18.1612f,-11.7342f,  
	5.1221f,-17.1348f,-11.4550f,  4.4466f,-18.0853f,-11.7363f,  2.7770f,-16.9834f,-11.7125f,  2.0966f,-17.9152f,-11.9933f,  0.4816f,-16.7577f,-12.2261f,  
	-0.2031f,-17.6698f,-12.5078f,  -1.7348f,-16.4785f,-12.9943f,  -2.4227f,-17.3701f,-13.2785f,  -3.8439f,-16.1677f,-14.0110f,  -4.5336f,-17.0386f,-14.2991f,  
	-5.8191f,-15.8491f,-15.2658f,  -6.5085f,-16.6993f,-15.5589f,  -7.6360f,-15.5470f,-16.7435f,  -8.3242f,-16.3780f,-17.0432f,  -9.2741f,-15.2875f,-18.4255f,  
	-9.9595f,-16.1002f,-18.7326f,  -10.7160f,-15.0959f,-20.2886f,  -11.3976f,-15.8936f,-20.6038f,  -11.9486f,-14.9987f,-22.3061f,  -12.6257f,-15.7840f,-22.6301f,  
	-12.9635f,-15.0207f,-24.4482f,  -13.6353f,-15.7974f,-24.7806f,  -13.7566f,-15.1860f,-26.6815f,  -14.4239f,-15.9590f,-27.0220f,  -14.3296f,-15.5174f,-28.9709f,  
	-14.9930f,-16.2914f,-29.3179f,  -14.6891f,-16.0347f,-31.2785f,  -15.3512f,-16.8157f,-31.6301f,  -14.8480f,-16.7556f,-33.5654f,  -15.5109f,-17.5485f,-33.9199f,  
	-14.8230f,-17.6930f,-35.7928f,  -15.4912f,-18.5038f,-36.1468f,  -14.6371f,-18.8566f,-37.9211f,  -15.3158f,-19.6904f,-38.2712f,  -14.3178f,-20.2508f,-39.9120f,  
	-15.0142f,-21.1126f,-40.2537f,  -13.8976f,-21.8750f,-41.7282f,  -14.6169f,-22.7680f,-42.0583f,  -13.4092f,-23.7217f,-43.3366f,  -14.1611f,-24.6491f,-43.6500f,  
	-12.8914f,-25.7787f,-44.7055f,  -13.6874f,-26.7420f,-44.9971f,  -12.3858f,-28.0268f,-45.8078f,  -13.2336f,-29.0246f,-46.0734f,  -11.9299f,-30.4394f,-46.6217f,  
	-12.8358f,-31.4711f,-46.8577f,  -11.5590f,-32.9868f,-47.1310f,  -12.5380f,-34.0469f,-47.3327f,  -11.3150f,-35.6300f,-47.3243f,  -12.3736f,-36.7135f,-47.4886f,  
	-11.2287f,-38.3279f,-47.1971f,  -12.3753f,-39.4258f,-47.3213f,  -11.3290f,-41.0329f,-46.7516f,  -12.5688f,-42.1346f,-46.8337f,  -11.6373f,-43.6945f,-45.9964f,  
	-12.9748f,-44.7892f,-46.0353f,  -12.1694f,-46.2619f,-44.9468f,  -13.6059f,-47.3368f,-44.9420f,  -12.9329f,-48.6829f,-43.6240f,  -14.4665f,-49.7240f,-43.5762f,  
	-13.9264f,-50.9063f,-42.0554f,  -15.5533f,-51.9003f,-41.9655f,  -15.1416f,-52.8847f,-40.2733f,  -16.8527f,-53.8165f,-40.1440f,  -16.5593f,-54.5732f,-38.3155f,  
	-18.3422f,-55.4297f,-38.1490f,  -18.1527f,-55.9345f,-36.2225f,  -19.9953f,-56.7053f,-36.0204f,  -19.8915f,-56.9397f,-34.0364f,  -21.7750f,-57.6140f,-33.8031f,  
	-21.7359f,-57.5666f,-31.8035f,  -23.6424f,-58.1375f,-31.5420f,  -23.6449f,-57.8043f,-29.5694f,  -25.5516f,-58.2664f,-29.2848f,  -25.5722f,-57.6511f,-27.3813f,  
	-27.4595f,-58.0032f,-27.0766f,  -27.4755f,-57.1171f,-25.2832f,  -29.3228f,-57.3603f,-24.9619f,  -29.3139f,-56.2217f,-23.3170f,  -31.1015f,-56.3607f,-22.9839f,  
	-31.0504f,-54.9948f,-21.5231f,  -32.7623f,-55.0363f,-21.1815f,  21.4667f,-14.9678f,-17.7862f,  20.8339f,-16.0057f,-18.0989f,  19.7693f,-15.8796f,-16.4099f,  
	19.1301f,-16.9126f,-16.7165f,  17.9005f,-16.6324f,-15.1636f,  17.2544f,-17.6585f,-15.4643f,  15.8817f,-17.2277f,-14.0743f,  15.2277f,-18.2458f,-14.3694f,  
	13.7362f,-17.6705f,-13.1666f,  13.0748f,-18.6781f,-13.4567f,  11.4911f,-17.9672f,-12.4624f,  10.8223f,-18.9618f,-12.7479f,  9.1746f,-18.1267f,-11.9799f,  
	8.4987f,-19.1066f,-12.2619f,  6.8165f,-18.1612f,-11.7342f,  6.1332f,-19.1247f,-12.0137f,  4.4466f,-18.0853f,-11.7363f,  3.7575f,-19.0300f,-12.0145f,  
	2.0966f,-17.9152f,-11.9933f,  1.4023f,-18.8398f,-12.2716f,  -0.2031f,-17.6698f,-12.5078f,  -0.9016f,-18.5733f,-12.7878f,  -2.4227f,-17.3701f,-13.2785f,  
	-3.1239f,-18.2516f,-13.5617f,  -4.5336f,-17.0386f,-14.2991f,  -5.2354f,-17.8976f,-14.5867f,  -6.5085f,-16.6993f,-15.5589f,  -7.2098f,-17.5368f,-15.8526f,  
	-8.3242f,-16.3780f,-17.0432f,  -9.0232f,-17.1946f,-17.3441f,  -9.9595f,-16.1002f,-18.7326f,  -10.6549f,-16.8987f,-19.0417f,  -11.3976f,-15.8936f,-20.6038f,  
	-12.0883f,-16.6760f,-20.9220f,  -12.6257f,-15.7840f,-22.6301f,  -13.3106f,-16.5538f,-22.9573f,  -13.6353f,-15.7974f,-24.7806f,  -14.3146f,-16.5593f,-25.1168f,  
	-14.4239f,-15.9590f,-27.0220f,  -15.0975f,-16.7173f,-27.3660f,  -14.9930f,-16.2914f,-29.3179f,  -15.6629f,-17.0522f,-29.6683f,  -15.3512f,-16.8157f,-31.6301f,  
	-16.0190f,-17.5839f,-31.9853f,  -15.5109f,-17.5485f,-33.9199f,  -16.1805f,-18.3303f,-34.2768f,  -15.4912f,-18.5038f,-36.1468f,  -16.1670f,-19.3048f,-36.5024f,  
	-15.3158f,-19.6904f,-38.2712f,  -16.0046f,-20.5163f,-38.6211f,  -15.0142f,-21.1126f,-40.2537f,  -15.7210f,-21.9672f,-40.5948f,  -14.6169f,-22.7680f,-42.0583f,  
	-15.3510f,-23.6554f,-42.3855f,  -14.1611f,-24.6491f,-43.6500f,  -14.9337f,-25.5720f,-43.9579f,  -13.6874f,-26.7420f,-44.9971f,  -14.5067f,-27.7002f,-45.2814f,  
	-13.2336f,-29.0246f,-46.0734f,  -14.1060f,-30.0183f,-46.3304f,  -12.8358f,-31.4711f,-46.8577f,  -13.7772f,-32.4964f,-47.0821f,  -12.5380f,-34.0469f,-47.3327f,  
	-13.5555f,-35.0998f,-47.5210f,  -12.3736f,-36.7135f,-47.4886f,  -13.4771f,-37.7862f,-47.6374f,  -12.3753f,-39.4258f,-47.3213f,  -13.5714f,-40.5086f,-47.4280f,  
	-12.5688f,-42.1346f,-46.8337f,  -13.8637f,-43.2175f,-46.8965f,  -12.9748f,-44.7892f,-46.0353f,  -14.3715f,-45.8599f,-46.0532f,  -13.6059f,-47.3368f,-44.9420f,  
	-15.1046f,-48.3814f,-44.9149f,  -14.4665f,-49.7240f,-43.5762f,  -16.0654f,-50.7291f,-43.5044f,  -15.5533f,-51.9003f,-41.9655f,  -17.2455f,-52.8502f,-41.8511f,  
	-16.8527f,-53.8165f,-40.1440f,  -18.6273f,-54.6974f,-39.9887f,  -18.3422f,-55.4297f,-38.1490f,  -20.1886f,-56.2303f,-37.9541f,  -19.9953f,-56.7053f,-36.0204f,  
	-21.8960f,-57.4132f,-35.7898f,  -21.7750f,-57.6140f,-33.8031f,  -23.7131f,-58.2207f,-33.5395f,  -23.6424f,-58.1375f,-31.5420f,  -25.5952f,-58.6357f,-31.2504f,  
	-25.5516f,-58.2664f,-29.2848f,  -27.4993f,-58.6533f,-28.9682f,  -27.4595f,-58.0032f,-27.0766f,  -29.3802f,-58.2780f,-26.7386f,  -29.3228f,-57.3603f,-24.9619f,  
	-31.1957f,-57.5260f,-24.6073f,  -31.1015f,-56.3607f,-22.9839f,  -32.9083f,-56.4227f,-22.6164f,  -32.7623f,-55.0363f,-21.1815f,  -34.4871f,-55.0029f,-20.8052f,  
	20.8339f,-16.0057f,-18.0989f,  20.1912f,-17.0473f,-18.4071f,  19.1301f,-16.9126f,-16.7165f,  18.4801f,-17.9483f,-17.0186f,  17.2544f,-17.6585f,-15.4643f,  
	16.5961f,-18.6873f,-15.7603f,  15.2277f,-18.2458f,-14.3694f,  14.5614f,-19.2649f,-14.6599f,  13.0748f,-18.6781f,-13.4567f,  12.4005f,-19.6852f,-13.7421f,  
	10.8223f,-18.9618f,-12.7479f,  10.1400f,-19.9549f,-13.0291f,  8.4987f,-19.1066f,-12.2619f,  7.8082f,-20.0837f,-12.5398f,  6.1332f,-19.1247f,-12.0137f,  
	5.4360f,-20.0833f,-12.2895f,  3.7575f,-19.0300f,-12.0145f,  3.0540f,-19.9685f,-12.2896f,  1.4023f,-18.8398f,-12.2716f,  0.6934f,-19.7567f,-12.5474f,  
	-0.9016f,-18.5733f,-12.7878f,  -1.6144f,-19.4672f,-13.0661f,  -3.1239f,-18.2516f,-13.5617f,  -3.8383f,-19.1216f,-13.8436f,  -5.2354f,-17.8976f,-14.5867f,  
	-5.9502f,-18.7443f,-14.8742f,  -7.2098f,-17.5368f,-15.8526f,  -7.9231f,-18.3601f,-16.1469f,  -9.0232f,-17.1946f,-17.3441f,  -9.7333f,-17.9967f,-17.6463f,  
	-10.6549f,-16.8987f,-19.0417f,  -11.3605f,-17.6814f,-19.3530f,  -12.0883f,-16.6760f,-20.9220f,  -12.7877f,-17.4422f,-21.2428f,  -13.3106f,-16.5538f,-22.9573f,  
	-14.0037f,-17.3078f,-23.2878f,  -14.3146f,-16.5593f,-25.1168f,  -15.0007f,-17.3051f,-25.4563f,  -15.0975f,-16.7173f,-27.3660f,  -15.7778f,-17.4610f,-27.7133f,  
	-15.6629f,-17.0522f,-29.6683f,  -16.3387f,-17.7985f,-30.0221f,  -16.0190f,-17.5839f,-31.9853f,  -16.6933f,-18.3393f,-32.3432f,  -16.1805f,-18.3303f,-34.2768f,  
	-16.8570f,-19.1006f,-34.6357f,  -16.1670f,-19.3048f,-36.5024f,  -16.8519f,-20.0961f,-36.8582f,  -16.0046f,-20.5163f,-38.6211f,  -16.7026f,-21.3332f,-38.9711f,  
	-15.7210f,-21.9672f,-40.5948f,  -16.4408f,-22.8146f,-40.9338f,  -15.3510f,-23.6554f,-42.3855f,  -16.1035f,-24.5370f,-42.7079f,  -14.9337f,-25.5720f,-43.9579f,  
	-15.7273f,-26.4889f,-44.2592f,  -14.5067f,-27.7002f,-45.2814f,  -15.3478f,-28.6531f,-45.5583f,  -14.1060f,-30.0183f,-46.3304f,  -15.0112f,-31.0052f,-46.5766f,  
	-13.7772f,-32.4964f,-47.0821f,  -14.7542f,-33.5143f,-47.2938f,  -13.5555f,-35.0998f,-47.5210f,  -14.6155f,-36.1421f,-47.6943f,  -13.4771f,-37.7862f,-47.6374f,  
	-14.6280f,-38.8444f,-47.7691f,  -13.5714f,-40.5086f,-47.4280f,  -14.8208f,-41.5736f,-47.5157f,  -13.8637f,-43.2175f,-46.8965f,  -15.2165f,-44.2774f,-46.9385f,  
	-14.3715f,-45.8599f,-46.0532f,  -15.8300f,-46.9014f,-46.0487f,  -15.1046f,-48.3814f,-44.9149f,  -16.6690f,-49.3911f,-44.8636f,  -16.0654f,-50.7291f,-43.5044f,  
	-17.7309f,-51.6911f,-43.4077f,  -17.2455f,-52.8502f,-41.8511f,  -19.0032f,-53.7500f,-41.7101f,  -18.6273f,-54.6974f,-39.9887f,  -20.4681f,-55.5228f,-39.8043f,  
	-20.1886f,-56.2303f,-37.9541f,  -22.0968f,-56.9676f,-37.7297f,  -21.8960f,-57.4132f,-35.7898f,  -23.8557f,-58.0524f,-35.5278f,  -23.7131f,-58.2207f,-33.5395f,  
	-25.7021f,-58.7528f,-33.2447f,  -25.5952f,-58.6357f,-31.2504f,  -27.5940f,-59.0560f,-30.9255f,  -27.4993f,-58.6533f,-28.9682f,  -29.4849f,-58.9592f,-28.6166f,  
	-29.3802f,-58.2780f,-26.7386f,  -31.3314f,-58.4709f,-26.3654f,  -31.1957f,-57.5260f,-24.6073f,  -33.0919f,-57.6096f,-24.2163f,  -32.9083f,-56.4227f,-22.6164f,  
	-34.7315f,-56.4040f,-22.2121f,  -34.4871f,-55.0029f,-20.8052f,  -36.2231f,-54.8909f,-20.3915f,  20.1912f,-17.0473f,-18.4071f,  19.5378f,-18.0921f,-18.7105f,  
	18.4801f,-17.9483f,-17.0186f,  17.8180f,-18.9873f,-17.3157f,  16.5961f,-18.6873f,-15.7603f,  15.9255f,-19.7176f,-16.0514f,  14.5614f,-19.2649f,-14.6599f,  
	13.8821f,-20.2844f,-14.9454f,  12.4005f,-19.6852f,-13.7421f,  11.7123f,-20.6915f,-14.0227f,  10.1400f,-19.9549f,-13.0291f,  9.4428f,-20.9460f,-13.3057f,  
	7.8082f,-20.0837f,-12.5398f,  7.1032f,-21.0569f,-12.8134f,  5.4360f,-20.0833f,-12.2895f,  4.7235f,-21.0366f,-12.5615f,  3.0540f,-19.9685f,-12.2896f,  
	2.3349f,-20.9000f,-12.5614f,  0.6934f,-19.7567f,-12.5474f,  -0.0307f,-20.6647f,-12.8208f,  -1.6144f,-19.4672f,-13.0661f,  -2.3412f,-20.3503f,-13.3423f,  
	-3.8383f,-19.1216f,-13.8436f,  -4.5667f,-19.9797f,-14.1246f,  -5.9502f,-18.7443f,-14.8742f,  -6.6781f,-19.5770f,-15.1614f,  -7.9231f,-18.3601f,-16.1469f,  
	-8.6485f,-19.1690f,-16.4416f,  -9.7333f,-17.9967f,-17.6463f,  -10.4548f,-18.7830f,-17.9501f,  -11.3605f,-17.6814f,-19.3530f,  -12.0759f,-18.4474f,-19.6664f,  
	-12.7877f,-17.4422f,-21.2428f,  -13.4964f,-18.1919f,-21.5664f,  -14.0037f,-17.3078f,-23.2878f,  -14.7043f,-18.0448f,-23.6213f,  -15.0007f,-17.3051f,-25.4563f,  
	-15.6940f,-18.0352f,-25.7988f,  -15.7778f,-17.4610f,-27.7133f,  -16.4644f,-18.1887f,-28.0641f,  -16.3387f,-17.7985f,-30.0221f,  -17.0208f,-18.5304f,-30.3789f,  
	-16.6933f,-18.3393f,-32.3432f,  -17.3740f,-19.0812f,-32.7033f,  -16.8570f,-19.1006f,-34.6357f,  -17.5415f,-19.8592f,-34.9954f,  -16.8519f,-20.0961f,-36.8582f,  
	-17.5448f,-20.8766f,-37.2149f,  -16.7026f,-21.3332f,-38.9711f,  -17.4118f,-22.1411f,-39.3196f,  -16.4408f,-22.8146f,-40.9338f,  -17.1767f,-23.6546f,-41.2686f,  
	-16.1035f,-24.5370f,-42.7079f,  -16.8747f,-25.4113f,-43.0245f,  -15.7273f,-26.4889f,-44.2592f,  -16.5401f,-27.3990f,-44.5544f,  -15.3478f,-28.6531f,-45.5583f,  
	-16.2191f,-29.5988f,-45.8249f,  -15.0112f,-31.0052f,-46.5766f,  -15.9492f,-31.9843f,-46.8109f,  -14.7542f,-33.5143f,-47.2938f,  -15.7710f,-34.5218f,-47.4912f,  
	-14.6155f,-36.1421f,-47.6943f,  -15.7204f,-37.1705f,-47.8511f,  -14.6280f,-38.8444f,-47.7691f,  -15.8299f,-39.8855f,-47.8822f,  -14.8208f,-41.5736f,-47.5157f,  
	-16.1265f,-42.6165f,-47.5829f,  -15.2165f,-44.2774f,-46.9385f,  -16.6302f,-45.3093f,-46.9582f,  -15.8300f,-46.9014f,-46.0487f,  -17.3539f,-47.9092f,-46.0199f,  
	-16.6690f,-49.3911f,-44.8636f,  -18.3006f,-50.3588f,-44.7870f,  -17.7309f,-51.6911f,-43.4077f,  -19.4633f,-52.6038f,-43.2838f,  -19.0032f,-53.7500f,-41.7101f,  
	-20.8292f,-54.5948f,-41.5394f,  -20.4681f,-55.5228f,-39.8043f,  -22.3739f,-56.2848f,-39.5896f,  -22.0968f,-56.9676f,-37.7297f,  -24.0677f,-57.6356f,-37.4729f,  
	-23.8557f,-58.0524f,-35.5278f,  -25.8707f,-58.6155f,-35.2334f,  -25.7021f,-58.7528f,-33.2447f,  -27.7420f,-59.2046f,-32.9150f,  -27.5940f,-59.0560f,-30.9255f,  
	-29.6354f,-59.3922f,-30.5639f,  -29.4849f,-58.9592f,-28.6166f,  -31.5061f,-59.1791f,-28.2282f,  -31.3314f,-58.4709f,-26.3654f,  -33.3101f,-58.5766f,-25.9539f,  
	-33.0919f,-57.6096f,-24.2163f,  -35.0083f,-57.6067f,-23.7866f,  -34.7315f,-56.4040f,-22.2121f,  -36.5687f,-56.3004f,-21.7684f,  -36.2231f,-54.8909f,-20.3915f,  
	-37.9670f,-54.6972f,-19.9387f,  19.5378f,-18.0921f,-18.7105f,  18.8726f,-19.1407f,-19.0087f,  17.8180f,-18.9873f,-17.3157f,  17.1437f,-20.0285f,-17.6075f,  
	15.9255f,-19.7176f,-16.0514f,  15.2418f,-20.7492f,-16.3373f,  13.8821f,-20.2844f,-14.9454f,  13.1888f,-21.3039f,-15.2257f,  11.7123f,-20.6915f,-14.0227f,  
	11.0091f,-21.6969f,-14.2982f,  9.4428f,-20.9460f,-13.3057f,  8.7307f,-21.9342f,-13.5774f,  7.1032f,-21.0569f,-12.8134f,  6.3825f,-22.0257f,-13.0826f,  
	4.7235f,-21.0366f,-12.5615f,  3.9949f,-21.9838f,-12.8296f,  2.3349f,-20.9000f,-12.5614f,  1.6000f,-21.8235f,-12.8300f,  -0.0307f,-20.6647f,-12.8208f,  
	-0.7698f,-21.5626f,-13.0913f,  -2.3412f,-20.3503f,-13.3423f,  -3.0832f,-21.2220f,-13.6167f,  -4.5667f,-19.9797f,-14.1246f,  -5.3094f,-20.8243f,-14.4044f,  
	-6.6781f,-19.5770f,-15.1614f,  -7.4192f,-20.3954f,-15.4482f,  -8.6485f,-19.1690f,-16.4416f,  -9.3866f,-19.9620f,-16.7372f,  -10.4548f,-18.7830f,-17.9501f,  
	-11.1871f,-19.5522f,-18.2552f,  -12.0759f,-18.4474f,-19.6664f,  -12.8015f,-19.1964f,-19.9820f,  -13.4964f,-18.1919f,-21.5664f,  -14.2135f,-18.9237f,-21.8925f,  
	-14.7043f,-18.0448f,-23.6213f,  -15.4129f,-18.7649f,-23.9574f,  -15.6940f,-18.0352f,-25.7988f,  -16.3940f,-18.7479f,-26.1446f,  -16.4644f,-18.1887f,-28.0641f,  
	-17.1574f,-18.9005f,-28.4176f,  -17.0208f,-18.5304f,-30.3789f,  -17.7090f,-19.2471f,-30.7380f,  -17.3740f,-19.0812f,-32.7033f,  -18.0618f,-19.8098f,-33.0643f,  
	-17.5415f,-19.8592f,-34.9954f,  -18.2333f,-20.6054f,-35.3562f,  -17.5448f,-20.8766f,-37.2149f,  -18.2474f,-21.6463f,-37.5706f,  -17.4118f,-22.1411f,-39.3196f,  
	-18.1350f,-22.9399f,-39.6645f,  -17.1767f,-23.6546f,-41.2686f,  -17.9292f,-24.4860f,-41.5983f,  -16.8747f,-25.4113f,-43.0245f,  -17.6625f,-26.2774f,-43.3360f,  
	-16.5401f,-27.3990f,-44.5544f,  -17.3801f,-28.3012f,-44.8399f,  -16.2191f,-29.5988f,-45.8249f,  -17.1200f,-30.5359f,-46.0806f,  -15.9492f,-31.9843f,-46.8109f,  
	-16.9238f,-32.9528f,-47.0316f,  -15.7710f,-34.5218f,-47.4912f,  -16.8299f,-35.5154f,-47.6727f,  -15.7204f,-37.1705f,-47.8511f,  -16.8737f,-38.1823f,-47.9898f,  
	-15.8299f,-39.8855f,-47.8822f,  -17.0860f,-40.9054f,-47.9751f,  -16.1265f,-42.6165f,-47.5829f,  -17.4915f,-43.6325f,-47.6278f,  -16.6302f,-45.3093f,-46.9582f,  
	-18.1082f,-46.3087f,-46.9536f,  -17.3539f,-47.9092f,-46.0199f,  -18.9449f,-48.8762f,-45.9656f,  -18.3006f,-50.3588f,-44.7870f,  -19.9999f,-51.2783f,-44.6828f,  
	-19.4633f,-52.6038f,-43.2838f,  -21.2656f,-53.4623f,-43.1297f,  -20.8292f,-54.5948f,-41.5394f,  -22.7227f,-55.3766f,-41.3376f,  -22.3739f,-56.2848f,-39.5896f,  
	-24.3456f,-56.9774f,-39.3415f,  -24.0677f,-57.6356f,-37.4729f,  -26.0979f,-58.2264f,-37.1825f,  -25.8707f,-58.6155f,-35.2334f,  -27.9409f,-59.0966f,-34.9027f,  
	-27.7420f,-59.2046f,-32.9150f,  -29.8288f,-59.5696f,-32.5471f,  -29.6354f,-59.3922f,-30.5639f,  -31.7170f,-59.6386f,-30.1637f,  -31.5061f,-59.1791f,-28.2282f,  
	-33.5590f,-59.3072f,-27.7995f,  -33.3101f,-58.5766f,-25.9539f,  -35.3126f,-58.5904f,-25.5017f,  -35.0083f,-57.6067f,-23.7866f,  -36.9415f,-57.5128f,-23.3154f,  
	-36.5687f,-56.3004f,-21.7684f,  -38.4157f,-56.1085f,-21.2836f,  -37.9670f,-54.6972f,-19.9387f,  -39.7177f,-54.4175f,-19.4434f,  18.8726f,-19.1407f,-19.0087f,  
	18.1952f,-20.1921f,-19.3015f,  17.1437f,-20.0285f,-17.6075f,  16.4563f,-21.0716f,-17.8939f,  15.2418f,-20.7492f,-16.3373f,  14.5440f,-21.7816f,-16.6175f,  
	13.1888f,-21.3039f,-15.2257f,  12.4802f,-22.3234f,-15.5005f,  11.0091f,-21.6969f,-14.2982f,  10.2906f,-22.7002f,-14.5684f,  8.7307f,-21.9342f,-13.5774f,  
	8.0025f,-22.9189f,-13.8442f,  6.3825f,-22.0257f,-13.0826f,  5.6450f,-22.9894f,-13.3472f,  3.9949f,-21.9838f,-12.8296f,  3.2497f,-22.9239f,-13.0936f,  
	1.6000f,-21.8235f,-12.8300f,  0.8492f,-22.7378f,-13.0950f,  -0.7698f,-21.5626f,-13.0913f,  -1.5249f,-22.4498f,-13.3591f,  -3.0832f,-21.2220f,-13.6167f,  
	-3.8404f,-22.0808f,-13.8891f,  -5.3094f,-20.8243f,-14.4044f,  -6.0663f,-21.6549f,-14.6831f,  -7.4192f,-20.3954f,-15.4482f,  -8.1742f,-21.1981f,-15.7350f,  
	-9.3866f,-19.9620f,-16.7372f,  -10.1366f,-20.7377f,-17.0332f,  -11.1871f,-19.5522f,-18.2552f,  -11.9309f,-20.3040f,-18.5617f,  -12.8015f,-19.1964f,-19.9820f,  
	-13.5366f,-19.9267f,-20.2994f,  -14.2135f,-18.9237f,-21.8925f,  -14.9393f,-19.6376f,-22.2208f,  -15.4129f,-18.7649f,-23.9574f,  -16.1287f,-19.4664f,-24.2965f,  
	-16.3940f,-18.7479f,-26.1446f,  -17.1007f,-19.4432f,-26.4930f,  -17.1574f,-18.9005f,-28.4176f,  -17.8564f,-19.5954f,-28.7735f,  -17.7090f,-19.2471f,-30.7380f,  
	-18.4038f,-19.9487f,-31.0981f,  -18.0618f,-19.8098f,-33.0643f,  -18.7562f,-20.5242f,-33.4270f,  -18.2333f,-20.6054f,-35.3562f,  -18.9333f,-21.3388f,-35.7166f,  
	-18.2474f,-21.6463f,-37.5706f,  -18.9619f,-22.4052f,-37.9231f,  -18.1350f,-22.9399f,-39.6645f,  -18.8726f,-23.7288f,-40.0050f,  -17.9292f,-24.4860f,-41.5983f,  
	-18.6956f,-25.3075f,-41.9239f,  -17.6625f,-26.2774f,-43.3360f,  -18.4745f,-27.1346f,-43.6384f,  -17.3801f,-28.3012f,-44.8399f,  -18.2466f,-29.1939f,-45.1152f,  
	-17.1200f,-30.5359f,-46.0806f,  -18.0543f,-31.4620f,-46.3233f,  -16.9238f,-32.9528f,-47.0316f,  -17.9376f,-33.9076f,-47.2369f,  -16.8299f,-35.5154f,-47.6727f,  
	-17.9342f,-36.4929f,-47.8366f,  -16.8737f,-38.1823f,-47.9898f,  -18.0785f,-39.1736f,-48.1086f,  -17.0860f,-40.9054f,-47.9751f,  -18.3992f,-41.8995f,-48.0460f,  
	-17.4915f,-43.6325f,-47.6278f,  -18.9192f,-44.6172f,-47.6485f,  -18.1082f,-46.3087f,-46.9536f,  -19.6528f,-47.2687f,-46.9231f,  -18.9449f,-48.8762f,-45.9656f,  
	-20.6038f,-49.7963f,-45.8833f,  -19.9999f,-51.2783f,-44.6828f,  -21.7699f,-52.1447f,-44.5480f,  -21.2656f,-53.4623f,-43.1297f,  -23.1371f,-54.2584f,-42.9439f,  
	-22.7227f,-55.3766f,-41.3376f,  -24.6846f,-56.0890f,-41.1016f,  -24.3456f,-56.9774f,-39.3415f,  -26.3799f,-57.5920f,-39.0587f,  -26.0979f,-58.2264f,-37.1825f,  
	-28.1873f,-58.7337f,-36.8545f,  -27.9409f,-59.0966f,-34.9027f,  -30.0621f,-59.4884f,-34.5324f,  -29.8288f,-59.5696f,-32.5471f,  -31.9604f,-59.8415f,-32.1390f,  
	-31.7170f,-59.6386f,-30.1637f,  -33.8344f,-59.7890f,-29.7215f,  -33.5590f,-59.3072f,-27.7995f,  -35.6394f,-59.3384f,-27.3282f,  -35.3126f,-58.5904f,-25.5017f,  
	-37.3351f,-58.5075f,-25.0060f,  -36.9415f,-57.5128f,-23.3154f,  -38.8866f,-57.3243f,-22.8010f,  -38.4157f,-56.1085f,-21.2836f,  -40.2707f,-55.8239f,-20.7542f,  
	-39.7177f,-54.4175f,-19.4434f,  -41.4727f,-54.0479f,-18.9031f,  18.1952f,-20.1921f,-19.3015f,  17.5049f,-21.2461f,-19.5886f,  16.4563f,-21.0716f,-17.8939f,  
	15.7550f,-22.1164f,-18.1743f,  14.5440f,-21.7816f,-16.6175f,  13.8310f,-22.8148f,-16.8918f,  12.4802f,-22.3234f,-15.5005f,  11.7563f,-23.3417f,-15.7694f,  
	10.2906f,-22.7002f,-14.5684f,  9.5557f,-23.7011f,-14.8330f,  8.0025f,-22.9189f,-13.8442f,  7.2570f,-23.8997f,-14.1056f,  5.6450f,-22.9894f,-13.3472f,  
	4.8904f,-23.9471f,-13.6070f,  3.2497f,-22.9239f,-13.0936f,  2.4879f,-23.8558f,-13.3532f,  0.8492f,-22.7378f,-13.0950f,  0.0815f,-23.6422f,-13.3564f,  
	-1.5249f,-22.4498f,-13.3591f,  -2.2964f,-23.3249f,-13.6241f,  -3.8404f,-22.0808f,-13.8891f,  -4.6131f,-22.9261f,-14.1594f,  -6.0663f,-21.6549f,-14.6831f,  
	-6.8384f,-22.4702f,-14.9608f,  -8.1742f,-21.1981f,-15.7350f,  -8.9424f,-21.9836f,-16.0214f,  -10.1366f,-20.7377f,-17.0332f,  -10.8993f,-21.4959f,-17.3299f,  
	-11.9309f,-20.3040f,-18.5617f,  -12.6851f,-21.0367f,-18.8694f,  -13.5366f,-19.9267f,-20.2994f,  -14.2812f,-20.6385f,-20.6184f,  -14.9393f,-19.6376f,-22.2208f,  
	-15.6731f,-20.3319f,-22.5514f,  -16.1287f,-19.4664f,-24.2965f,  -16.8518f,-20.1493f,-24.6378f,  -17.1007f,-19.4432f,-26.4930f,  -17.8136f,-20.1202f,-26.8435f,  
	-17.8564f,-19.5954f,-28.7735f,  -18.5616f,-20.2735f,-29.1304f,  -18.4038f,-19.9487f,-31.0981f,  -19.1047f,-20.6343f,-31.4601f,  -18.7562f,-20.5242f,-33.4270f,  
	-19.4577f,-21.2238f,-33.7895f,  -18.9333f,-21.3388f,-35.7166f,  -19.6434f,-22.0596f,-36.0743f,  -18.9619f,-22.4052f,-37.9231f,  -19.6890f,-23.1526f,-38.2719f,  
	-18.8726f,-23.7288f,-40.0050f,  -19.6219f,-24.5059f,-40.3423f,  -18.6956f,-25.3075f,-41.9239f,  -19.4833f,-26.1189f,-42.2410f,  -18.4745f,-27.1346f,-43.6384f,  
	-19.3097f,-27.9812f,-43.9315f,  -18.2466f,-29.1939f,-45.1152f,  -19.1431f,-30.0750f,-45.3783f,  -18.0543f,-31.4620f,-46.3233f,  -19.0245f,-32.3744f,-46.5512f,  
	-17.9376f,-33.9076f,-47.2369f,  -18.9934f,-34.8464f,-47.4252f,  -17.9342f,-36.4929f,-47.8366f,  -19.0870f,-37.4506f,-47.9811f,  -18.0785f,-39.1736f,-48.1086f,  
	-19.3377f,-40.1400f,-48.2057f,  -18.3992f,-41.8995f,-48.0460f,  -19.7729f,-42.8634f,-48.0929f,  -18.9192f,-44.6172f,-47.6485f,  -20.4121f,-45.5638f,-47.6433f,  
	-19.6528f,-47.2687f,-46.9231f,  -21.2646f,-48.1830f,-46.8645f,  -20.6038f,-49.7963f,-45.8833f,  -22.3333f,-50.6644f,-45.7701f,  -21.7699f,-52.1447f,-44.5480f,  
	-23.6100f,-52.9495f,-44.3809f,  -23.1371f,-54.2584f,-42.9439f,  -25.0788f,-54.9856f,-42.7231f,  -24.6846f,-56.0890f,-41.1016f,  -26.7119f,-56.7231f,-40.8298f,  
	-26.3799f,-57.5920f,-39.0587f,  -28.4768f,-58.1221f,-38.7371f,  -28.1873f,-58.7337f,-36.8545f,  -30.3311f,-59.1499f,-36.4856f,  -30.0621f,-59.4884f,-34.5324f,  
	-32.2322f,-59.7843f,-34.1202f,  -31.9604f,-59.8415f,-32.1390f,  -34.1315f,-60.0137f,-31.6871f,  -33.8344f,-59.7890f,-29.7215f,  -35.9830f,-59.8378f,-29.2347f,  
	-35.6394f,-59.3384f,-27.3282f,  -37.7428f,-59.2673f,-26.8111f,  -37.3351f,-58.5075f,-25.0060f,  -39.3717f,-58.3236f,-24.4648f,  -38.8866f,-57.3243f,-22.8010f,  
	-40.8411f,-57.0363f,-22.2397f,  -40.2707f,-55.8239f,-20.7542f,  -42.1301f,-55.4427f,-20.1776f,  -41.4727f,-54.0479f,-18.9031f,  -43.2272f,-53.5858f,-18.3163f,  
	17.5049f,-21.2461f,-19.5886f,  16.8008f,-22.3025f,-19.8695f,  15.7550f,-22.1164f,-18.1743f,  15.0384f,-23.1628f,-18.4485f,  13.8310f,-22.8148f,-16.8918f,  
	13.1026f,-23.8479f,-17.1599f,  11.7563f,-23.3417f,-15.7694f,  11.0156f,-24.3587f,-16.0323f,  9.5557f,-23.7011f,-14.8330f,  8.8032f,-24.6990f,-15.0917f,  
	7.2570f,-23.8997f,-14.1056f,  6.4938f,-24.8754f,-14.3615f,  4.8904f,-23.9471f,-13.6070f,  4.1184f,-24.8974f,-13.8616f,  2.4879f,-23.8558f,-13.3532f,  
	1.7084f,-24.7787f,-13.6084f,  0.0815f,-23.6422f,-13.3564f,  -0.7036f,-24.5353f,-13.6139f,  -2.2964f,-23.3249f,-13.6241f,  -3.0843f,-24.1871f,-13.8860f,  
	-4.6131f,-22.9261f,-14.1594f,  -5.4021f,-23.7566f,-14.4277f,  -6.8384f,-22.4702f,-14.9608f,  -7.6250f,-23.2686f,-15.2371f,  -8.9424f,-21.9836f,-16.0214f,  
	-9.7246f,-22.7517f,-16.3076f,  -10.8993f,-21.4959f,-17.3299f,  -11.6737f,-22.2348f,-17.6269f,  -12.6851f,-21.0367f,-18.8694f,  -13.4500f,-21.7503f,-19.1779f,  
	-14.2812f,-20.6385f,-20.6184f,  -15.0349f,-21.3298f,-20.9392f,  -15.6731f,-20.3319f,-22.5514f,  -16.4149f,-21.0066f,-22.8839f,  -16.8518f,-20.1493f,-24.6378f,  
	-17.5814f,-20.8124f,-24.9810f,  -17.8136f,-20.1202f,-26.8435f,  -18.5325f,-20.7787f,-27.1950f,  -18.5616f,-20.2735f,-29.1304f,  -19.2727f,-20.9339f,-29.4893f,  
	-19.1047f,-20.6343f,-31.4601f,  -19.8118f,-21.3033f,-31.8222f,  -19.4577f,-21.2238f,-33.7895f,  -20.1678f,-21.9090f,-34.1496f,  -19.6434f,-22.0596f,-36.0743f,  
	-20.3643f,-22.7673f,-36.4286f,  -19.6890f,-23.1526f,-38.2719f,  -20.4254f,-23.8863f,-38.6182f,  -19.6219f,-24.5059f,-40.3423f,  -20.3895f,-25.2716f,-40.6716f,  
	-19.4833f,-26.1189f,-42.2410f,  -20.2911f,-26.9186f,-42.5495f,  -19.3097f,-27.9812f,-43.9315f,  -20.1716f,-28.8155f,-44.2130f,  -19.1431f,-30.0750f,-45.3783f,  
	-20.0720f,-30.9422f,-45.6273f,  -19.0245f,-32.3744f,-46.5512f,  -20.0332f,-33.2709f,-46.7628f,  -18.9934f,-34.8464f,-47.4252f,  -20.0942f,-35.7659f,-47.5947f,  
	-19.0870f,-37.4506f,-47.9811f,  -20.2911f,-38.3842f,-48.1042f,  -19.3377f,-40.1400f,-48.2057f,  -20.6548f,-41.0774f,-48.2791f,  -19.7729f,-42.8634f,-48.0929f,  
	-21.2099f,-43.7906f,-48.1139f,  -20.4121f,-45.5638f,-47.6433f,  -21.9711f,-46.4662f,-47.6099f,  -21.2646f,-48.1830f,-46.8645f,  -22.9463f,-49.0468f,-46.7749f,  
	-22.3333f,-50.6644f,-45.7701f,  -24.1334f,-51.4721f,-45.6241f,  -23.6100f,-52.9495f,-44.3809f,  -25.5216f,-53.6861f,-44.1783f,  -25.0788f,-54.9856f,-42.7231f,  
	-27.0882f,-55.6348f,-42.4656f,  -26.7119f,-56.7231f,-40.8298f,  -28.8046f,-57.2724f,-40.5183f,  -28.4768f,-58.1221f,-38.7371f,  -30.6312f,-58.5600f,-38.3735f,  
	-30.3311f,-59.1499f,-36.4856f,  -32.5277f,-59.4681f,-36.0733f,  -32.2322f,-59.7843f,-34.1202f,  -34.4454f,-59.9774f,-33.6625f,  -34.1315f,-60.0137f,-31.6871f,  
	-36.3375f,-60.0801f,-31.1886f,  -35.9830f,-59.8378f,-29.2347f,  -38.1575f,-59.7793f,-28.7000f,  -37.7428f,-59.2673f,-26.8111f,  -39.8626f,-59.0894f,-26.2465f,  
	-39.3717f,-58.3236f,-24.4648f,  -41.4192f,-58.0340f,-23.8745f,  -40.8411f,-57.0363f,-22.2397f,  -42.8003f,-56.6452f,-21.6292f,  -42.1301f,-55.4427f,-20.1776f,  
	-43.9882f,-54.9622f,-19.5527f,  -43.2272f,-53.5858f,-18.3163f,  -44.9798f,-53.0265f,-17.6794f,  16.8008f,-22.3025f,-19.8695f,  16.0817f,-23.3614f,-20.1438f,  
	15.0384f,-23.1628f,-18.4485f,  14.3065f,-24.2100f,-18.7161f,  13.1026f,-23.8479f,-17.1599f,  12.3573f,-24.8805f,-17.4214f,  11.0156f,-24.3587f,-16.0323f,  
	10.2573f,-25.3738f,-16.2886f,  8.8032f,-24.6990f,-15.0917f,  8.0327f,-25.6931f,-15.3441f,  6.4938f,-24.8754f,-14.3615f,  5.7128f,-25.8449f,-14.6115f,  
	4.1184f,-24.8974f,-13.8616f,  3.3281f,-25.8400f,-14.1110f,  1.7084f,-24.7787f,-13.6084f,  0.9105f,-25.6914f,-13.8589f,  -0.7036f,-24.5353f,-13.6139f,  
	-1.5061f,-25.4164f,-13.8676f,  -3.0843f,-24.1871f,-13.8860f,  -3.8898f,-25.0353f,-14.1449f,  -5.4021f,-23.7566f,-14.4277f,  -6.2068f,-24.5708f,-14.6937f,  
	-7.6250f,-23.2686f,-15.2371f,  -8.4268f,-24.0498f,-15.5122f,  -9.7246f,-22.7517f,-16.3076f,  -10.5197f,-23.5003f,-16.5931f,  -11.6737f,-22.2348f,-17.6269f,  
	-12.4599f,-22.9541f,-17.9239f,  -13.4500f,-21.7503f,-19.1779f,  -14.2250f,-22.4428f,-19.4875f,  -15.0349f,-21.3298f,-20.9392f,  -15.7974f,-22.0005f,-21.2612f,  
	-16.4149f,-21.0066f,-22.8839f,  -17.1637f,-21.6601f,-23.2178f,  -17.5814f,-20.8124f,-24.9810f,  -18.3171f,-21.4555f,-25.3248f,  -18.5325f,-20.7787f,-27.1950f,  
	-19.2574f,-21.4179f,-27.5482f,  -19.2727f,-20.9339f,-29.4893f,  -19.9894f,-21.5759f,-29.8483f,  -19.8118f,-21.3033f,-31.8222f,  -20.5260f,-21.9560f,-32.1820f,  
	-20.1678f,-21.9090f,-34.1496f,  -20.8869f,-22.5794f,-34.5067f,  -20.3643f,-22.7673f,-36.4286f,  -21.0926f,-23.4593f,-36.7812f,  -20.4254f,-23.8863f,-38.6182f,  
	-21.1775f,-24.6072f,-38.9570f,  -20.3895f,-25.2716f,-40.6716f,  -21.1740f,-26.0242f,-40.9930f,  -20.2911f,-26.9186f,-42.5495f,  -21.1221f,-27.7050f,-42.8471f,  
	-20.1716f,-28.8155f,-44.2130f,  -21.0623f,-29.6355f,-44.4810f,  -20.0720f,-30.9422f,-45.6273f,  -21.0357f,-31.7934f,-45.8605f,  -20.0332f,-33.2709f,-46.7628f,  
	-21.0830f,-34.1483f,-46.9560f,  -20.0942f,-35.7659f,-47.5947f,  -21.2429f,-36.6620f,-47.7432f,  -20.2911f,-38.3842f,-48.1042f,  -21.5497f,-39.2897f,-48.2040f,  
	-20.6548f,-41.0774f,-48.2791f,  -22.0325f,-41.9795f,-48.3267f,  -21.2099f,-43.7906f,-48.1139f,  -22.7110f,-44.6751f,-48.1066f,  -21.9711f,-46.4662f,-47.6099f,  
	-23.5985f,-47.3196f,-47.5454f,  -22.9463f,-49.0468f,-46.7749f,  -24.6983f,-49.8516f,-46.6522f,  -24.1334f,-51.4721f,-45.6241f,  -26.0054f,-52.2128f,-45.4422f,  
	-25.5216f,-53.6861f,-44.1783f,  -27.5023f,-54.3455f,-43.9382f,  -27.0882f,-55.6348f,-42.4656f,  -29.1651f,-56.1991f,-42.1676f,  -28.8046f,-57.2724f,-40.5183f,  
	-30.9574f,-57.7287f,-40.1636f,  -30.6312f,-58.5600f,-38.3735f,  -32.8418f,-58.8981f,-37.9650f,  -32.5277f,-59.4681f,-36.0733f,  -34.7706f,-59.6808f,-35.6140f,  
	-34.4454f,-59.9774f,-33.6625f,  -36.6968f,-60.0611f,-33.1564f,  -36.3375f,-60.0801f,-31.1886f,  -38.5723f,-60.0347f,-30.6403f,  -38.1575f,-59.7793f,-28.7000f,  
	-40.3510f,-59.6085f,-28.1157f,  -39.8626f,-59.0894f,-26.2465f,  -41.9949f,-58.7998f,-25.6307f,  -41.4192f,-58.0340f,-23.8745f,  -43.4718f,-57.6348f,-23.2329f,  
	-42.8003f,-56.6452f,-21.6292f,  -44.7576f,-56.1480f,-20.9682f,  -43.9882f,-54.9622f,-19.5527f,  -45.8428f,-54.3779f,-18.8757f,  -44.9798f,-53.0265f,-17.6794f,  
	-46.7262f,-52.3667f,-16.9905f,  16.0817f,-23.3614f,-20.1438f,  15.3474f,-24.4218f,-20.4113f,  14.3065f,-24.2100f,-18.7161f,  13.5578f,-25.2577f,-18.9767f,  
	12.3573f,-24.8805f,-17.4214f,  11.5942f,-25.9124f,-17.6759f,  10.2573f,-25.3738f,-16.2886f,  9.4805f,-26.3861f,-16.5381f,  8.0327f,-25.6931f,-15.3441f,  
	7.2438f,-26.6821f,-15.5900f,  5.7128f,-25.8449f,-14.6115f,  4.9128f,-26.8077f,-14.8555f,  3.3281f,-25.8400f,-14.1110f,  2.5185f,-26.7733f,-14.3548f,  
	0.9105f,-25.6914f,-13.8589f,  0.0944f,-26.5930f,-14.1046f,  -1.5061f,-25.4164f,-13.8676f,  -2.3273f,-26.2844f,-14.1173f,  -3.8898f,-25.0353f,-14.1449f,  
	-4.7121f,-25.8677f,-14.4005f,  -6.2068f,-24.5708f,-14.6937f,  -7.0280f,-25.3681f,-14.9575f,  -8.4268f,-24.0498f,-15.5122f,  -9.2428f,-24.8117f,-15.7857f,  
	-10.5197f,-23.5003f,-16.5931f,  -11.3278f,-24.2293f,-16.8778f,  -12.4599f,-22.9541f,-17.9239f,  -13.2574f,-23.6519f,-18.2211f,  -14.2250f,-22.4428f,-19.4875f,  
	-15.0099f,-23.1139f,-19.7976f,  -15.7974f,-22.0005f,-21.2612f,  -16.5677f,-22.6490f,-21.5841f,  -17.1637f,-21.6601f,-23.2178f,  -17.9189f,-22.2925f,-23.5519f,  
	-18.3171f,-21.4555f,-25.3248f,  -19.0591f,-22.0779f,-25.6702f,  -19.2574f,-21.4179f,-27.5482f,  -19.9875f,-22.0369f,-27.9016f,  -19.9894f,-21.5759f,-29.8483f,  
	-20.7120f,-22.1998f,-30.2051f,  -20.5260f,-21.9560f,-32.1820f,  -21.2480f,-22.5923f,-32.5391f,  -20.8869f,-22.5794f,-34.5067f,  -21.6118f,-23.2322f,-34.8626f,  
	-21.0926f,-23.4593f,-36.7812f,  -21.8338f,-24.1370f,-37.1266f,  -21.1775f,-24.6072f,-38.9570f,  -21.9434f,-25.3135f,-39.2886f,  -21.1740f,-26.0242f,-40.9930f,  
	-21.9784f,-26.7626f,-41.3042f,  -21.1221f,-27.7050f,-42.8471f,  -21.9783f,-28.4766f,-43.1319f,  -21.0623f,-29.6355f,-44.4810f,  -21.9837f,-30.4391f,-44.7339f,  
	-21.0357f,-31.7934f,-45.8605f,  -22.0365f,-32.6256f,-46.0760f,  -21.0830f,-34.1483f,-46.9560f,  -22.1769f,-35.0030f,-47.1289f,  -21.2429f,-36.6620f,-47.7432f,  
	-22.4423f,-37.5311f,-47.8688f,  -21.5497f,-39.2897f,-48.2040f,  -22.8658f,-40.1614f,-48.2781f,  -22.0325f,-41.9795f,-48.3267f,  -23.4719f,-42.8405f,-48.3460f,  
	-22.7110f,-44.6751f,-48.1066f,  -24.2785f,-45.5123f,-48.0683f,  -23.5985f,-47.3196f,-47.5454f,  -25.2951f,-48.1157f,-47.4476f,  -24.6983f,-49.8516f,-46.6522f,  
	-26.5218f,-50.5909f,-46.4932f,  -26.0054f,-52.2128f,-45.4422f,  -27.9472f,-52.8773f,-45.2222f,  -27.5023f,-54.3455f,-43.9382f,  -29.5520f,-54.9205f,-43.6567f,  
	-29.1651f,-56.1991f,-42.1676f,  -31.3040f,-56.6703f,-41.8255f,  -30.9574f,-57.7287f,-40.1636f,  -33.1691f,-58.0842f,-39.7629f,  -32.8418f,-58.8981f,-37.9650f,  
	-35.1016f,-59.1289f,-37.5082f,  -34.7706f,-59.6808f,-35.6140f,  -37.0549f,-59.7812f,-35.1047f,  -36.6968f,-60.0611f,-33.1564f,  -38.9800f,-60.0291f,-32.5989f,  
	-38.5723f,-60.0347f,-30.6403f,  -40.8284f,-59.8722f,-30.0405f,  -40.3510f,-59.6085f,-28.1157f,  -42.5586f,-59.3207f,-27.4782f,  -41.9949f,-58.7998f,-25.6307f,  
	-44.1331f,-58.3945f,-24.9613f,  -43.4718f,-57.6348f,-23.2329f,  -45.5222f,-57.1231f,-22.5388f,  -44.7576f,-56.1480f,-20.9682f,  -46.7099f,-55.5407f,-20.2534f,  
	-45.8428f,-54.3779f,-18.8757f,  -47.6886f,-53.6870f,-18.1450f,  -46.7262f,-52.3667f,-16.9905f,  -48.4626f,-51.6031f,-16.2472f,  15.3474f,-24.4218f,-20.4113f,  
	14.5964f,-25.4837f,-20.6714f,  13.5578f,-25.2577f,-18.9767f,  12.7912f,-26.3056f,-19.2298f,  11.5942f,-25.9124f,-17.6759f,  10.8126f,-26.9426f,-17.9230f,  
	9.4805f,-26.3861f,-16.5381f,  8.6851f,-27.3945f,-16.7804f,  7.2438f,-26.6821f,-15.5900f,  6.4354f,-27.6655f,-15.8291f,  4.9128f,-26.8077f,-14.8555f,  
	4.0929f,-27.7626f,-15.0930f,  2.5185f,-26.7733f,-14.3548f,  1.6899f,-27.6967f,-14.5928f,  0.0944f,-26.5930f,-14.1046f,  -0.7415f,-27.4824f,-14.3452f,  
	-2.3273f,-26.2844f,-14.1173f,  -3.1666f,-27.1373f,-14.3625f,  -4.7121f,-25.8677f,-14.4005f,  -5.5523f,-26.6840f,-14.6528f,  -7.0280f,-25.3681f,-14.9575f,  
	-7.8648f,-26.1465f,-15.2186f,  -9.2428f,-24.8117f,-15.7857f,  -10.0731f,-25.5540f,-16.0574f,  -11.3278f,-24.2293f,-16.8778f,  -12.1486f,-24.9365f,-17.1616f,  
	-13.2574f,-23.6519f,-18.2211f,  -14.0660f,-24.3279f,-18.5180f,  -15.0099f,-23.1139f,-19.7976f,  -15.8036f,-23.7621f,-20.1078f,  -16.5677f,-22.6490f,-21.5841f,  
	-17.3450f,-23.2752f,-21.9066f,  -17.9189f,-22.2925f,-23.5519f,  -18.6811f,-22.9027f,-23.8872f,  -19.0591f,-22.0779f,-25.6702f,  -19.8061f,-22.6785f,-26.0155f,  
	-19.9875f,-22.0369f,-27.9016f,  -20.7226f,-22.6362f,-28.2528f,  -20.7120f,-22.1998f,-30.2051f,  -21.4413f,-22.8056f,-30.5594f,  -21.2480f,-22.5923f,-32.5391f,  
	-21.9743f,-23.2089f,-32.8953f,  -21.6118f,-23.2322f,-34.8626f,  -22.3471f,-23.8691f,-35.2117f,  -21.8338f,-24.1370f,-37.1266f,  -22.5861f,-24.7985f,-37.4655f,  
	-21.9434f,-25.3135f,-39.2886f,  -22.7259f,-26.0044f,-39.6106f,  -21.9784f,-26.7626f,-41.3042f,  -22.8043f,-27.4853f,-41.6032f,  -21.9783f,-28.4766f,-43.1319f,  
	-22.8612f,-29.2312f,-43.4023f,  -21.9837f,-30.4391f,-44.7339f,  -22.9382f,-31.2237f,-44.9698f,  -22.0365f,-32.6256f,-46.0760f,  -23.0771f,-33.4355f,-46.2717f,  
	-22.1769f,-35.0030f,-47.1289f,  -23.3174f,-35.8314f,-47.2793f,  -22.4423f,-37.5311f,-47.8688f,  -23.6956f,-38.3675f,-47.9690f,  -22.8658f,-40.1614f,-48.2781f,  
	-24.2406f,-40.9937f,-48.3242f,  -23.4719f,-42.8405f,-48.3460f,  -24.9749f,-43.6559f,-48.3347f,  -24.2785f,-45.5123f,-48.0683f,  -25.9135f,-46.2941f,-47.9967f,  
	-25.2951f,-48.1157f,-47.4476f,  -27.0621f,-48.8481f,-47.3134f,  -26.5218f,-50.5909f,-46.4932f,  -28.4153f,-51.2554f,-46.2957f,  -27.9472f,-52.8773f,-45.2222f,  
	-29.9588f,-53.4585f,-44.9603f,  -29.5520f,-54.9205f,-43.6567f,  -31.6651f,-55.4028f,-43.3305f,  -31.3040f,-56.6703f,-41.8255f,  -33.5042f,-57.0403f,-41.4363f,  
	-33.1691f,-58.0842f,-39.7629f,  -35.4327f,-58.3312f,-39.3127f,  -35.1016f,-59.1289f,-37.5082f,  -37.4056f,-59.2451f,-36.9999f,  -37.0549f,-59.7812f,-35.1047f,  
	-39.3736f,-59.7628f,-34.5421f,  -38.9800f,-60.0291f,-32.5989f,  -41.2868f,-59.8759f,-31.9878f,  -40.8284f,-59.8722f,-30.0405f,  -43.1005f,-59.5877f,-29.3853f,  
	-42.5586f,-59.3207f,-27.4782f,  -44.7729f,-58.9115f,-26.7849f,  -44.1331f,-58.3945f,-24.9613f,  -46.2689f,-57.8707f,-24.2375f,  -45.5222f,-57.1231f,-22.5388f,  
	-47.5664f,-56.4951f,-21.7889f,  -46.7099f,-55.5407f,-20.2534f,  -48.6512f,-54.8207f,-19.4830f,  -47.6886f,-53.6870f,-18.1450f,  -49.5211f,-52.8865f,-17.3584f,  
	-48.4626f,-51.6031f,-16.2472f,  -50.1849f,-50.7324f,-15.4471f,  14.5964f,-25.4837f,-20.6714f,  13.8278f,-26.5468f,-20.9237f,  12.7912f,-26.3056f,-19.2298f,  
	12.0062f,-27.3530f,-19.4750f,  10.8126f,-26.9426f,-17.9230f,  10.0122f,-27.9701f,-18.1624f,  8.6851f,-27.3945f,-16.7804f,  7.8698f,-28.3986f,-17.0153f,  
	6.4354f,-27.6655f,-15.8291f,  5.6065f,-28.6423f,-16.0610f,  4.0929f,-27.7626f,-15.0930f,  3.2531f,-28.7086f,-15.3240f,  1.6899f,-27.6967f,-14.5928f,  
	0.8406f,-28.6089f,-14.8248f,  -0.7415f,-27.4824f,-14.3452f,  -1.5966f,-28.3579f,-14.5804f,  -3.1666f,-27.1373f,-14.3625f,  -4.0248f,-27.9749f,-14.6035f,  
	-5.5523f,-26.6840f,-14.6528f,  -6.4093f,-27.4819f,-14.9014f,  -7.8648f,-26.1465f,-15.2186f,  -8.7172f,-26.9056f,-15.4769f,  -10.0731f,-25.5540f,-16.0574f,  
	-10.9175f,-26.2746f,-16.3273f,  -12.1486f,-24.9365f,-17.1616f,  -12.9817f,-25.6217f,-17.4444f,  -14.0660f,-24.3279f,-18.5180f,  -14.8844f,-24.9803f,-18.8143f,  
	-15.8036f,-23.7621f,-20.1078f,  -16.6049f,-24.3870f,-20.4172f,  -17.3450f,-23.2752f,-21.9066f,  -18.1299f,-23.8782f,-22.2299f,  -18.6811f,-22.9027f,-23.8872f,  
	-19.4484f,-23.4897f,-24.2221f,  -19.8061f,-22.6785f,-26.0155f,  -20.5576f,-23.2578f,-26.3584f,  -20.7226f,-22.6362f,-28.2528f,  -21.4635f,-23.2158f,-28.6015f,  
	-21.4413f,-22.8056f,-30.5594f,  -22.1738f,-23.3898f,-30.9130f,  -21.9743f,-23.2089f,-32.8953f,  -22.7088f,-23.8080f,-33.2451f,  -22.3471f,-23.8691f,-35.2117f,  
	-23.0910f,-24.4880f,-35.5548f,  -22.5861f,-24.7985f,-37.4655f,  -23.3517f,-25.4432f,-37.7952f,  -22.7259f,-26.0044f,-39.6106f,  -23.5263f,-26.6786f,-39.9209f,  
	-22.8043f,-27.4853f,-41.6032f,  -23.6529f,-28.1904f,-41.8884f,  -22.8612f,-29.2312f,-43.4023f,  -23.7728f,-29.9667f,-43.6563f,  -22.9382f,-31.2237f,-44.9698f,  
	-23.9281f,-31.9864f,-45.1864f,  -23.0771f,-33.4355f,-46.2717f,  -24.1599f,-34.2199f,-46.4455f,  -23.3174f,-35.8314f,-47.2793f,  -24.5077f,-36.6285f,-47.4047f,  
	-23.6956f,-38.3675f,-47.9690f,  -25.0040f,-39.1663f,-48.0415f,  -24.2406f,-40.9937f,-48.3242f,  -25.6756f,-41.7821f,-48.3400f,  -24.9749f,-43.6559f,-48.3347f,  
	-26.5429f,-44.4180f,-48.2900f,  -25.9135f,-46.2941f,-47.9967f,  -27.6170f,-47.0141f,-47.8887f,  -27.0621f,-48.8481f,-47.3134f,  -28.8985f,-49.5075f,-47.1403f,  
	-28.4153f,-51.2554f,-46.2957f,  -30.3785f,-51.8381f,-46.0559f,  -29.9588f,-53.4585f,-44.9603f,  -32.0346f,-53.9480f,-44.6531f,  -31.6651f,-55.4028f,-43.3305f,  
	-33.8412f,-55.7843f,-42.9565f,  -33.5042f,-57.0403f,-41.4363f,  -35.7583f,-57.3013f,-40.9965f,  -35.4327f,-58.3312f,-39.3127f,  -37.7428f,-58.4621f,-38.8096f,  
	-37.4056f,-59.2451f,-36.9999f,  -39.7464f,-59.2401f,-36.4368f,  -39.3736f,-59.7628f,-34.5421f,  -41.7183f,-59.6195f,-33.9243f,  -41.2868f,-59.8759f,-31.9878f,  
	-43.6113f,-59.5963f,-31.3195f,  -43.1005f,-59.5877f,-29.3853f,  -45.3803f,-59.1768f,-28.6725f,  -44.7729f,-58.9115f,-26.7849f,  -46.9849f,-58.3781f,-26.0353f,  
	-46.2689f,-57.8707f,-24.2375f,  -48.3975f,-57.2247f,-23.4559f,  -47.5664f,-56.4951f,-21.7889f,  -49.5975f,-55.7485f,-20.9816f,  -48.6512f,-54.8207f,-19.4830f,  
	-50.5761f,-53.9853f,-18.6552f,  -49.5211f,-52.8865f,-17.3584f,  -51.3355f,-51.9735f,-16.5138f,  -50.1849f,-50.7324f,-15.4471f,  -51.8880f,-49.7519f,-14.5886f,  
	13.8278f,-26.5468f,-20.9237f,  13.0409f,-27.6104f,-21.1677f,  12.0062f,-27.3530f,-19.4750f,  11.2022f,-28.3990f,-19.7122f,  10.0122f,-27.9701f,-18.1624f,  
	9.1916f,-28.9945f,-18.3937f,  7.8698f,-28.3986f,-17.0153f,  7.0337f,-29.3974f,-17.2422f,  5.6065f,-28.6423f,-16.0610f,  4.7572f,-29.6114f,-16.2855f,  
	3.2531f,-28.7086f,-15.3240f,  2.3918f,-29.6447f,-15.5479f,  0.8406f,-28.6089f,-14.8248f,  -0.0289f,-29.5084f,-15.0505f,  -1.5966f,-28.3579f,-14.5804f,  
	-2.4718f,-29.2189f,-14.8103f,  -4.0248f,-27.9749f,-14.6035f,  -4.9013f,-28.7950f,-14.8396f,  -6.4093f,-27.4819f,-14.9014f,  -7.2834f,-28.2609f,-15.1462f,  
	-8.7172f,-26.9056f,-15.4769f,  -9.5852f,-27.6432f,-15.7324f,  -10.9175f,-26.2746f,-16.3273f,  -11.7756f,-26.9730f,-16.5951f,  -12.9817f,-25.6217f,-17.4444f,  
	-13.8259f,-26.2828f,-17.7256f,  -14.8844f,-24.9803f,-18.8143f,  -15.7115f,-25.6087f,-19.1090f,  -16.6049f,-24.3870f,-20.4172f,  -17.4150f,-24.9877f,-20.7266f,  
	-18.1299f,-23.8782f,-22.2299f,  -18.9204f,-24.4567f,-22.5523f,  -19.4484f,-23.4897f,-24.2221f,  -20.2198f,-24.0538f,-24.5545f,  -20.5576f,-23.2578f,-26.3584f,  
	-21.3143f,-23.8157f,-26.6989f,  -21.4635f,-23.2158f,-28.6015f,  -22.2070f,-23.7717f,-28.9496f,  -22.1738f,-23.3898f,-30.9130f,  -22.9126f,-23.9548f,-31.2604f,  
	-22.7088f,-23.8080f,-33.2451f,  -23.4495f,-24.3874f,-33.5892f,  -23.0910f,-24.4880f,-35.5548f,  -23.8452f,-25.0888f,-35.8892f,  -23.3517f,-25.4432f,-37.7952f,  
	-24.1320f,-26.0703f,-38.1139f,  -23.5263f,-26.6786f,-39.9209f,  -24.3455f,-27.3345f,-40.2181f,  -23.6529f,-28.1904f,-41.8884f,  -24.5260f,-28.8760f,-42.1578f,  
	-23.7728f,-29.9667f,-43.6563f,  -24.7154f,-30.6804f,-43.8916f,  -23.9281f,-31.9864f,-45.1864f,  -24.9555f,-32.7240f,-45.3818f,  -24.1599f,-34.2199f,-46.4455f,  
	-25.2882f,-34.9742f,-46.5947f,  -24.5077f,-36.6285f,-47.4047f,  -25.7490f,-37.3896f,-47.5028f,  -25.0040f,-39.1663f,-48.0415f,  -26.3687f,-39.9230f,-48.0842f,  
	-25.6756f,-41.7821f,-48.3400f,  -27.1726f,-42.5194f,-48.3227f,  -26.5429f,-44.4180f,-48.2900f,  -28.1771f,-45.1205f,-48.2092f,  -27.6170f,-47.0141f,-47.8887f,  
	-29.3886f,-47.6635f,-47.7417f,  -28.8985f,-49.5075f,-47.1403f,  -30.8039f,-50.0871f,-46.9248f,  -30.3785f,-51.8381f,-46.0559f,  -32.4061f,-52.3308f,-45.7705f,  
	-32.0346f,-53.9480f,-44.6531f,  -34.1744f,-54.3376f,-44.2974f,  -33.8412f,-55.7843f,-42.9565f,  -36.0727f,-56.0569f,-42.5310f,  -35.7583f,-57.3013f,-40.9965f,  
	-38.0609f,-57.4455f,-40.5030f,  -37.7428f,-58.4621f,-38.8096f,  -40.0920f,-58.4700f,-38.2506f,  -39.7464f,-59.2401f,-36.4368f,  -42.1155f,-59.1075f,-35.8170f,  
	-41.7183f,-59.6195f,-33.9243f,  -44.0824f,-59.3463f,-33.2476f,  -43.6113f,-59.5963f,-31.3195f,  -45.9447f,-59.1859f,-30.5917f,  -45.3803f,-59.1768f,-28.6725f,  
	-47.6580f,-58.6365f,-27.9014f,  -46.9849f,-58.3781f,-26.0353f,  -49.1891f,-57.7170f,-25.2260f,  -48.3975f,-57.2247f,-23.4559f,  -50.5114f,-56.4544f,-22.6152f,  
	-49.5975f,-55.7485f,-20.9816f,  -51.6095f,-54.8809f,-20.1153f,  -50.5761f,-53.9853f,-18.6552f,  -52.4792f,-53.0323f,-17.7680f,  -51.3355f,-51.9735f,-16.5138f,  
	-53.1264f,-50.9460f,-15.6097f,  -51.8880f,-49.7519f,-14.5886f,  -53.5672f,-48.6589f,-13.6697f,  13.0409f,-27.6104f,-21.1677f,  12.2350f,-28.6737f,-21.4033f,  
	11.2022f,-28.3990f,-19.7122f,  10.3781f,-29.4429f,-19.9407f,  9.1916f,-28.9945f,-18.3937f,  8.3499f,-30.0149f,-18.6164f,  7.0337f,-29.3974f,-17.2422f,  
	6.1766f,-30.3897f,-17.4611f,  4.7572f,-29.6114f,-16.2855f,  3.8857f,-30.5720f,-16.5022f,  2.3918f,-29.6447f,-15.5479f,  1.5094f,-30.5694f,-15.7646f,  
	-0.0289f,-29.5084f,-15.0505f,  -0.9197f,-30.3946f,-15.2697f,  -2.4718f,-29.2189f,-14.8103f,  -3.3666f,-30.0633f,-15.0343f,  -4.9013f,-28.7950f,-14.8396f,  
	-5.7962f,-29.5968f,-15.0709f,  -7.2834f,-28.2609f,-15.1462f,  -8.1745f,-29.0190f,-15.3870f,  -9.5852f,-27.6432f,-15.7324f,  -10.4684f,-28.3589f,-15.9847f,  
	-11.7756f,-26.9730f,-16.5951f,  -12.6463f,-27.6474f,-16.8604f,  -13.8259f,-26.2828f,-17.7256f,  -14.6800f,-26.9194f,-18.0044f,  -15.7115f,-25.6087f,-19.1090f,  
	-16.5485f,-26.2121f,-19.4029f,  -17.4150f,-24.9877f,-20.7266f,  -18.2312f,-25.5627f,-21.0345f,  -18.9204f,-24.4567f,-22.5523f,  -19.7151f,-25.0108f,-22.8717f,  
	-20.2198f,-24.0538f,-24.5545f,  -20.9962f,-24.5952f,-24.8841f,  -21.3143f,-23.8157f,-26.6989f,  -22.0732f,-24.3483f,-27.0386f,  -22.2070f,-23.7717f,-28.9496f,  
	-22.9550f,-24.3070f,-29.2916f,  -22.9126f,-23.9548f,-31.2604f,  -23.6555f,-24.4984f,-31.6024f,  -23.4495f,-24.3874f,-33.5892f,  -24.1979f,-24.9472f,-33.9251f,  
	-23.8452f,-25.0888f,-35.8892f,  -24.6108f,-25.6709f,-36.2130f,  -24.1320f,-26.0703f,-38.1139f,  -24.9271f,-26.6782f,-38.4200f,  -24.3455f,-27.3345f,-40.2181f,  
	-25.1849f,-27.9704f,-40.5002f,  -24.5260f,-28.8760f,-42.1578f,  -25.4254f,-29.5399f,-42.4093f,  -24.7154f,-30.6804f,-43.8916f,  -25.6906f,-31.3695f,-44.1063f,  
	-24.9555f,-32.7240f,-45.3818f,  -26.0237f,-33.4327f,-45.5530f,  -25.2882f,-34.9742f,-46.5947f,  -26.4630f,-35.6939f,-46.7172f,  -25.7490f,-37.3896f,-47.5028f,  
	-27.0424f,-38.1104f,-47.5716f,  -26.3687f,-39.9230f,-48.0842f,  -27.7918f,-40.6309f,-48.0941f,  -27.1726f,-42.5194f,-48.3227f,  -28.7326f,-43.1997f,-48.2696f,  
	-28.1771f,-45.1205f,-48.2092f,  -29.8772f,-45.7550f,-48.0894f,  -29.3886f,-47.6635f,-47.7417f,  -31.2278f,-48.2354f,-47.5523f,  -30.8039f,-50.0871f,-46.9248f,  
	-32.7732f,-50.5788f,-46.6637f,  -32.4061f,-52.3308f,-45.7705f,  -34.4980f,-52.7250f,-45.4363f,  -34.1744f,-54.3376f,-44.2974f,  -36.3706f,-54.6192f,-43.8899f,  
	-36.0727f,-56.0569f,-42.5310f,  -38.3544f,-56.2127f,-42.0509f,  -38.0609f,-57.4455f,-40.5030f,  -40.4045f,-57.4658f,-39.9523f,  -40.0920f,-58.4700f,-38.2506f,  
	-42.4715f,-58.3485f,-37.6334f,  -42.1155f,-59.1075f,-35.8170f,  -44.5057f,-58.8420f,-35.1367f,  -44.0824f,-59.3463f,-33.2476f,  -46.4567f,-58.9386f,-32.5098f,  
	-45.9447f,-59.1859f,-30.5917f,  -48.2767f,-58.6416f,-29.8038f,  -47.6580f,-58.6365f,-27.9014f,  -49.9276f,-57.9637f,-27.0688f,  -49.1891f,-57.7170f,-25.2260f,  
	-51.3772f,-56.9264f,-24.3558f,  -50.5114f,-56.4544f,-22.6152f,  -52.6036f,-55.5579f,-21.7139f,  -51.6095f,-54.8809f,-20.1153f,  -53.5962f,-53.8907f,-19.1884f,  
	-52.4792f,-53.0323f,-17.7680f,  -54.3545f,-51.9601f,-16.8203f,  -53.1264f,-50.9460f,-15.6097f,  -54.8884f,-49.8020f,-14.6444f,  -53.5672f,-48.6589f,-13.6697f,  
	-55.2170f,-47.4515f,-12.6891f,  12.2350f,-28.6737f,-21.4033f,  11.4089f,-29.7362f,-21.6299f,  10.3781f,-29.4429f,-19.9407f,  9.5326f,-30.4843f,-20.1601f,  
	8.3499f,-30.0149f,-18.6164f,  7.4868f,-31.0301f,-18.8305f,  6.1766f,-30.3897f,-17.4611f,  5.2967f,-31.3749f,-17.6712f,  3.8857f,-30.5720f,-16.5022f,  
	2.9922f,-31.5225f,-16.7106f,  1.5094f,-30.5694f,-15.7646f,  0.6048f,-31.4818f,-15.9738f,  -0.9197f,-30.3946f,-15.2697f,  -1.8313f,-31.2652f,-15.4820f,  
	-3.3666f,-30.0633f,-15.0343f,  -4.2810f,-30.8904f,-15.2524f,  -5.7962f,-29.5968f,-15.0709f,  -6.7096f,-30.3785f,-15.2971f,  -8.1745f,-29.0190f,-15.3870f,  
	-9.0823f,-29.7555f,-15.6236f,  -10.4684f,-28.3589f,-15.9847f,  -11.3656f,-29.0505f,-16.2335f,  -12.6463f,-27.6474f,-16.8604f,  -13.5281f,-28.2968f,-17.1224f,  
	-14.6800f,-26.9194f,-18.0044f,  -15.5453f,-27.5306f,-18.2816f,  -16.5485f,-26.2121f,-19.4029f,  -17.3926f,-26.7891f,-19.6947f,  -18.2312f,-25.5627f,-21.0345f,  
	-19.0519f,-26.1123f,-21.3391f,  -19.7151f,-25.0108f,-22.8717f,  -20.5146f,-25.5409f,-23.1883f,  -20.9962f,-24.5952f,-24.8841f,  -21.7748f,-25.1095f,-25.2128f,  
	-22.0732f,-24.3483f,-27.0386f,  -22.8353f,-24.8585f,-27.3723f,  -22.9550f,-24.3070f,-29.2916f,  -23.7055f,-24.8190f,-29.6284f,  -23.6555f,-24.4984f,-31.6024f,  
	-24.4037f,-25.0209f,-31.9365f,  -24.1979f,-24.9472f,-33.9251f,  -24.9547f,-25.4871f,-34.2509f,  -24.6108f,-25.6709f,-36.2130f,  -25.3876f,-26.2326f,-36.5249f,  
	-24.9271f,-26.6782f,-38.4200f,  -25.7383f,-27.2653f,-38.7117f,  -25.1849f,-27.9704f,-40.5002f,  -26.0461f,-28.5843f,-40.7649f,  -25.4254f,-29.5399f,-42.4093f,  
	-26.3526f,-30.1794f,-42.6408f,  -25.6906f,-31.3695f,-44.1063f,  -26.7016f,-32.0306f,-44.2975f,  -26.0237f,-33.4327f,-45.5530f,  -27.1336f,-34.1084f,-45.6981f,  
	-26.4630f,-35.6939f,-46.7172f,  -27.6850f,-36.3753f,-46.8110f,  -27.0424f,-38.1104f,-47.5716f,  -28.3899f,-38.7848f,-47.6081f,  -27.7918f,-40.6309f,-48.0941f,  
	-29.2742f,-41.2843f,-48.0687f,  -28.7326f,-43.1997f,-48.2696f,  -30.3559f,-43.8146f,-48.1777f,  -29.8772f,-45.7550f,-48.0894f,  -31.6428f,-46.3147f,-47.9274f,  
	-31.2278f,-48.2354f,-47.5523f,  -33.1297f,-48.7219f,-47.3173f,  -32.7732f,-50.5788f,-46.6637f,  -34.8065f,-50.9741f,-46.3535f,  -34.4980f,-52.7250f,-45.4363f,  
	-36.6468f,-53.0128f,-45.0498f,  -36.3706f,-54.6192f,-43.8899f,  -38.6182f,-54.7848f,-43.4270f,  -38.3544f,-56.2127f,-42.0509f,  -40.6786f,-56.2446f,-41.5130f,  
	-40.4045f,-57.4658f,-39.9523f,  -42.7804f,-57.3555f,-39.3424f,  -42.4715f,-58.3485f,-37.6334f,  -44.8739f,-58.0921f,-36.9544f,  -44.5057f,-58.8420f,-35.1367f,  
	-46.9074f,-58.4392f,-34.3938f,  -46.4567f,-58.9386f,-32.5098f,  -48.8304f,-58.3932f,-31.7101f,  -48.2767f,-58.6416f,-29.8038f,  -50.6003f,-57.9604f,-28.9528f,  
	-49.9276f,-57.9637f,-27.0688f,  -52.1800f,-57.1565f,-26.1737f,  -51.3772f,-56.9264f,-24.3558f,  -53.5415f,-56.0047f,-23.4235f,  -52.6036f,-55.5579f,-21.7139f,  
	-54.6676f,-54.5339f,-20.7506f,  -53.5962f,-53.8907f,-19.1884f,  -55.5514f,-52.7766f,-18.1997f,  -54.3545f,-51.9601f,-16.8203f,  -56.1963f,-50.7671f,-15.8106f,  
	-54.8884f,-49.8020f,-14.6444f,  -56.6158f,-48.5400f,-13.6169f,  -55.2170f,-47.4515f,-12.6891f,  -56.8315f,-46.1282f,-11.6457f,  11.4089f,-29.7362f,-21.6299f,  
	10.5616f,-30.7975f,-21.8468f,  9.5326f,-30.4843f,-20.1601f,  8.6655f,-31.5218f,-20.3702f,  7.4868f,-31.0301f,-18.8305f,  6.6006f,-32.0397f,-19.0350f,  
	5.2967f,-31.3749f,-17.6712f,  4.3941f,-32.3515f,-17.8724f,  2.9922f,-31.5225f,-16.7106f,  2.0757f,-32.4621f,-16.9107f,  0.6048f,-31.4818f,-15.9738f,  
	-0.3218f,-32.3801f,-16.1752f,  -1.8313f,-31.2652f,-15.4820f,  -2.7637f,-32.1196f,-15.6874f,  -4.2810f,-30.8904f,-15.2524f,  -5.2155f,-31.6982f,-15.4642f,  
	-6.7096f,-30.3785f,-15.2971f,  -7.6412f,-31.1391f,-15.5180f,  -9.0823f,-29.7555f,-15.6236f,  -10.0058f,-30.4684f,-15.8556f,  -11.3656f,-29.0505f,-16.2335f,  
	-12.2754f,-29.7173f,-16.4781f,  -13.5281f,-28.2968f,-17.1224f,  -14.4226f,-28.9206f,-17.3819f,  -15.5453f,-27.5306f,-18.2816f,  -16.4189f,-28.1146f,-18.5559f,  
	-17.3926f,-26.7891f,-19.6947f,  -18.2418f,-27.3395f,-19.9827f,  -19.0519f,-26.1123f,-21.3391f,  -19.8779f,-26.6366f,-21.6404f,  -20.5146f,-25.5409f,-23.1883f,  
	-21.3168f,-26.0425f,-23.5034f,  -21.7748f,-25.1095f,-25.2128f,  -22.5555f,-25.6000f,-25.5355f,  -22.8353f,-24.8585f,-27.3723f,  -23.5986f,-25.3439f,-27.7009f,  
	-23.7055f,-24.8190f,-29.6284f,  -24.4592f,-25.3085f,-29.9575f,  -24.4037f,-25.0209f,-31.9365f,  -25.1577f,-25.5220f,-32.2610f,  -24.9547f,-25.4871f,-34.2509f,  
	-25.7193f,-26.0054f,-34.5652f,  -25.3876f,-26.2326f,-36.5249f,  -26.1766f,-26.7728f,-36.8229f,  -25.7383f,-27.2653f,-38.7117f,  -26.5668f,-27.8302f,-38.9867f,  
	-26.0461f,-28.5843f,-40.7649f,  -26.9301f,-29.1741f,-41.0104f,  -26.3526f,-30.1794f,-42.6408f,  -27.3104f,-30.7917f,-42.8494f,  -26.7016f,-32.0306f,-44.2975f,  
	-27.7494f,-32.6601f,-44.4631f,  -27.1336f,-34.1084f,-45.6981f,  -28.2857f,-34.7474f,-45.8153f,  -27.6850f,-36.3753f,-46.8110f,  -28.9567f,-37.0126f,-46.8731f,  
	-28.3899f,-38.7848f,-47.6081f,  -29.7925f,-39.4074f,-47.6100f,  -29.2742f,-41.2843f,-48.0687f,  -30.8165f,-41.8755f,-48.0048f,  -30.3559f,-43.8146f,-48.1777f,  
	-32.0420f,-44.3578f,-48.0439f,  -31.6428f,-46.3147f,-47.9274f,  -33.4695f,-46.7921f,-47.7202f,  -33.1297f,-48.7219f,-47.3173f,  -35.0948f,-49.1149f,-47.0333f,  
	-34.8065f,-50.9741f,-46.3535f,  -36.8968f,-51.2653f,-45.9909f,  -36.6468f,-53.0128f,-45.0498f,  -38.8478f,-53.1863f,-44.6077f,  -38.6182f,-54.7848f,-43.4270f,  
	-40.9095f,-54.8273f,-42.9058f,  -40.6786f,-56.2446f,-41.5130f,  -43.0368f,-56.1457f,-40.9148f,  -42.7804f,-57.3555f,-39.3424f,  -45.1808f,-57.1093f,-38.6697f,  
	-44.8739f,-58.0921f,-36.9544f,  -47.2891f,-57.6963f,-36.2116f,  -46.9074f,-58.4392f,-34.3938f,  -49.3095f,-57.8956f,-33.5876f,  -48.8304f,-58.3932f,-31.7101f,  
	-51.1958f,-57.7073f,-30.8457f,  -50.6003f,-57.9604f,-28.9528f,  -52.9062f,-57.1406f,-28.0377f,  -52.1800f,-57.1565f,-26.1737f,  -54.4071f,-56.2138f,-25.2149f,  
	-53.5415f,-56.0047f,-23.4235f,  -55.6750f,-54.9510f,-22.4279f,  -54.6676f,-54.5339f,-20.7506f,  -56.6965f,-53.3817f,-19.7244f,  -55.5514f,-52.7766f,-18.1997f,  
	-57.4687f,-51.5378f,-17.1482f,  -56.1963f,-50.7671f,-15.8106f,  -57.9987f,-49.4527f,-14.7379f,  -56.6158f,-48.5400f,-13.6169f,  -58.3027f,-47.1590f,-12.5262f,  
	-56.8315f,-46.1282f,-11.6457f,  -58.4047f,-44.6881f,-10.5387f,  10.5616f,-30.7975f,-21.8468f,  9.6926f,-31.8562f,-22.0541f,  8.6655f,-31.5218f,-20.3702f,  
	7.7751f,-32.5551f,-20.5703f,  6.6006f,-32.0397f,-19.0350f,  5.6911f,-33.0422f,-19.2299f,  4.3941f,-32.3515f,-17.8724f,  3.4678f,-33.3185f,-18.0643f,  
	2.0757f,-32.4621f,-16.9107f,  1.1361f,-33.3889f,-17.1020f,  -0.3218f,-32.3801f,-16.1752f,  -1.2704f,-33.2633f,-16.3686f,  -2.7637f,-32.1196f,-15.6874f,  
	-3.7177f,-32.9558f,-15.8854f,  -5.2155f,-31.6982f,-15.4642f,  -6.1697f,-32.4858f,-15.6696f,  -7.6412f,-31.1391f,-15.5180f,  -8.5901f,-31.8766f,-15.7331f,  
	-10.0058f,-30.4684f,-15.8556f,  -10.9434f,-31.1565f,-16.0824f,  -12.2754f,-29.7173f,-16.4781f,  -13.1995f,-30.3583f,-16.7190f,  -14.4226f,-28.9206f,-17.3819f,  
	-15.3268f,-29.5168f,-17.6375f,  -16.4189f,-28.1146f,-18.5559f,  -17.2984f,-28.6713f,-18.8257f,  -18.2418f,-27.3395f,-19.9827f,  -19.0968f,-27.8636f,-20.2669f,  
	-19.8779f,-26.6366f,-21.6404f,  -20.7071f,-27.1311f,-21.9397f,  -21.3168f,-26.0425f,-23.5034f,  -22.1205f,-26.5187f,-23.8124f,  -22.5555f,-25.6000f,-25.5355f,  
	-23.3367f,-26.0640f,-25.8530f,  -23.5986f,-25.3439f,-27.7009f,  -24.3635f,-25.8051f,-28.0220f,  -24.4592f,-25.3085f,-29.9575f,  -25.2164f,-25.7752f,-30.2774f,  
	-25.1577f,-25.5220f,-32.2610f,  -25.9165f,-26.0004f,-32.5744f,  -25.7193f,-26.0054f,-34.5652f,  -26.4923f,-26.5013f,-34.8662f,  -26.1766f,-26.7728f,-36.8229f,  
	-26.9786f,-27.2903f,-37.1049f,  -26.5668f,-27.8302f,-38.9867f,  -27.4133f,-28.3709f,-39.2431f,  -26.9301f,-29.1741f,-41.0104f,  -27.8396f,-29.7374f,-41.2338f,  
	-27.3104f,-30.7917f,-42.8494f,  -28.2999f,-31.3738f,-43.0333f,  -27.7494f,-32.6601f,-44.4631f,  -28.8341f,-33.2546f,-44.6018f,  -28.2857f,-34.7474f,-45.8153f,  
	-29.4825f,-35.3447f,-45.9016f,  -28.9567f,-37.0126f,-46.8731f,  -30.2787f,-37.6009f,-46.9013f,  -29.7925f,-39.4074f,-47.6100f,  -31.2512f,-39.9709f,-47.5739f,  
	-30.8165f,-41.8755f,-48.0048f,  -32.4184f,-42.3983f,-47.8997f,  -32.0420f,-44.3578f,-48.0439f,  -33.7867f,-44.8222f,-47.8654f,  -33.4695f,-46.7921f,-47.7202f,  
	-35.3577f,-47.1791f,-47.4642f,  -35.0948f,-49.1149f,-47.0333f,  -37.1161f,-49.4065f,-46.6972f,  -36.8968f,-51.2653f,-45.9909f,  -39.0394f,-51.4444f,-45.5728f,  
	-38.8478f,-53.1863f,-44.6077f,  -41.0934f,-53.2381f,-44.1070f,  -40.9095f,-54.8273f,-42.9058f,  -43.2363f,-54.7396f,-42.3238f,  -43.0368f,-56.1457f,-40.9148f,  
	-45.4210f,-55.9105f,-40.2531f,  -45.1808f,-57.1093f,-38.6697f,  -47.5953f,-56.7224f,-37.9321f,  -47.2891f,-57.6963f,-36.2116f,  -49.7060f,-57.1574f,-35.4041f,  
	-49.3095f,-57.8956f,-33.5876f,  -51.7039f,-57.2086f,-32.7153f,  -51.1958f,-57.7073f,-30.8457f,  -53.5434f,-56.8791f,-29.9159f,  -52.9062f,-57.1406f,-28.0377f,  
	-55.1855f,-56.1814f,-27.0575f,  -54.4071f,-56.2138f,-25.2149f,  -56.6011f,-55.1349f,-24.1916f,  -55.6750f,-54.9510f,-22.4279f,  -57.7704f,-53.7649f,-21.3682f,  
	-56.6965f,-53.3817f,-19.7244f,  -58.6837f,-52.1009f,-18.6344f,  -57.4687f,-51.5378f,-17.1482f,  -59.3419f,-50.1741f,-16.0330f,  -57.9987f,-49.4527f,-14.7379f,  
	-59.7554f,-48.0164f,-13.6017f,  -58.3027f,-47.1590f,-12.5262f,  -59.9428f,-45.6589f,-11.3717f,  -58.4047f,-44.6881f,-10.5387f,  -59.9298f,-43.1315f,-9.3682f,  
	9.6926f,-31.8562f,-22.0541f,  8.8001f,-32.9121f,-22.2507f,  7.7751f,-32.5551f,-20.5703f,  6.8610f,-33.5827f,-20.7600f,  5.6911f,-33.0422f,-19.2299f,  
	4.7572f,-34.0366f,-19.4147f,  3.4678f,-33.3185f,-18.0643f,  2.5175f,-34.2744f,-18.2465f,  1.1361f,-33.3889f,-17.1020f,  0.1734f,-34.3019f,-17.2844f,  
	-1.2704f,-33.2633f,-16.3686f,  -2.2420f,-34.1296f,-16.5536f,  -3.7177f,-32.9558f,-15.8854f,  -4.6929f,-33.7727f,-16.0760f,  -6.1697f,-32.4858f,-15.6696f,  
	-7.1429f,-33.2511f,-15.8682f,  -8.5901f,-31.8766f,-15.7331f,  -9.5549f,-32.5898f,-15.9422f,  -10.9434f,-31.1565f,-16.0824f,  -11.8971f,-31.8191f,-16.3045f,  
	-13.1995f,-30.3583f,-16.7190f,  -14.1350f,-30.9715f,-16.9553f,  -15.3268f,-29.5168f,-17.6375f,  -16.2381f,-30.0850f,-17.8881f,  -17.2984f,-28.6713f,-18.8257f,  
	-18.1848f,-29.2008f,-19.0912f,  -19.0968f,-27.8636f,-20.2669f,  -19.9561f,-28.3568f,-20.5483f,  -20.7071f,-27.1311f,-21.9397f,  -21.5376f,-27.5989f,-22.2327f,  
	-22.1205f,-26.5187f,-23.8124f,  -22.9242f,-26.9670f,-24.1161f,  -23.3367f,-26.0640f,-25.8530f,  -24.1180f,-26.5023f,-26.1630f,  -24.3635f,-25.8051f,-28.0220f,  
	-25.1298f,-26.2422f,-28.3341f,  -25.2164f,-25.7752f,-30.2774f,  -25.9757f,-26.2179f,-30.5866f,  -25.9165f,-26.0004f,-32.5744f,  -26.6802f,-26.4554f,-32.8750f,  
	-26.4923f,-26.5013f,-34.8662f,  -27.2744f,-26.9739f,-35.1518f,  -26.9786f,-27.2903f,-37.1049f,  -27.7938f,-27.7834f,-37.3690f,  -27.4133f,-28.3709f,-39.2431f,  
	-28.2802f,-28.8856f,-39.4781f,  -27.8396f,-29.7374f,-41.2338f,  -28.7757f,-30.2715f,-41.4331f,  -28.2999f,-31.3738f,-43.0333f,  -29.3209f,-31.9223f,-43.1910f,  
	-28.8341f,-33.2546f,-44.6018f,  -29.9583f,-33.8097f,-44.7103f,  -29.4825f,-35.3447f,-45.9016f,  -30.7245f,-35.8957f,-45.9548f,  -30.2787f,-37.6009f,-46.9013f,  
	-31.6526f,-38.1334f,-46.8922f,  -31.2512f,-39.9709f,-47.5739f,  -32.7656f,-40.4697f,-47.4973f,  -32.4184f,-42.3983f,-47.8997f,  -34.0760f,-42.8461f,-47.7506f,  
	-33.7867f,-44.8222f,-47.8654f,  -35.5909f,-45.1999f,-47.6388f,  -35.3577f,-47.1791f,-47.4642f,  -37.3011f,-47.4682f,-47.1566f,  -37.1161f,-49.4065f,-46.6972f,  
	-39.1896f,-49.5891f,-46.3057f,  -39.0394f,-51.4444f,-45.5728f,  -41.2270f,-51.5043f,-45.0960f,  -41.0934f,-53.2381f,-44.1070f,  -43.3756f,-53.1612f,-43.5451f,  
	-43.2363f,-54.7396f,-42.3238f,  -45.5904f,-54.5162f,-41.6777f,  -45.4210f,-55.9105f,-40.2531f,  -47.8208f,-55.5344f,-39.5257f,  -47.5953f,-56.7224f,-37.9321f,  
	-50.0130f,-56.1911f,-37.1287f,  -49.7060f,-57.1574f,-35.4041f,  -52.1159f,-56.4729f,-34.5295f,  -51.7039f,-57.2086f,-32.7153f,  -54.0805f,-56.3765f,-31.7762f,  
	-53.5434f,-56.8791f,-29.9159f,  -55.8635f,-55.9082f,-28.9197f,  -55.1855f,-56.1814f,-27.0575f,  -57.4301f,-55.0823f,-26.0115f,  -56.6011f,-55.1349f,-24.1916f,  
	-58.7545f,-53.9199f,-23.1030f,  -57.7704f,-53.7649f,-21.3682f,  -59.8205f,-52.4468f,-20.2438f,  -58.6837f,-52.1009f,-18.6344f,  -60.6226f,-50.6919f,-17.4801f,  
	-59.3419f,-50.1741f,-16.0330f,  -61.1647f,-48.6856f,-14.8538f,  -59.7554f,-48.0164f,-13.6017f,  -61.4602f,-46.4585f,-12.4014f,  -59.9428f,-45.6589f,-11.3717f,  
	-61.5295f,-44.0404f,-10.1537f,  -59.9298f,-43.1315f,-9.3682f,  -61.4006f,-41.4585f,-8.1338f,  8.8001f,-32.9121f,-22.2507f,  7.8837f,-33.9640f,-22.4365f,  
	6.8610f,-33.5827f,-20.7600f,  5.9220f,-34.6039f,-20.9390f,  4.7572f,-34.0366f,-19.4147f,  3.7987f,-35.0213f,-19.5891f,  2.5175f,-34.2744f,-18.2465f,  
	1.5431f,-35.2178f,-18.4190f,  0.1734f,-34.3019f,-17.2844f,  -0.8135f,-35.1996f,-17.4573f,  -2.2420f,-34.1296f,-16.5536f,  -3.2362f,-34.9778f,-16.7301f,  
	-4.6929f,-33.7727f,-16.0760f,  -5.6888f,-34.5684f,-16.2586f,  -7.1429f,-33.2511f,-15.8682f,  -8.1336f,-33.9928f,-16.0598f,  -9.5549f,-32.5898f,-15.9422f,  
	-10.5375f,-33.2780f,-16.1453f,  -11.8971f,-31.8191f,-16.3045f,  -12.8638f,-32.4540f,-16.5209f,  -14.1350f,-30.9715f,-16.9553f,  -15.0789f,-31.5565f,-17.1858f,  
	-16.2381f,-30.0850f,-17.8881f,  -17.1574f,-30.6255f,-18.1337f,  -18.1848f,-29.2008f,-19.0912f,  -19.0768f,-29.6987f,-19.3532f,  -19.9561f,-28.3568f,-20.5483f,  
	-20.8169f,-28.8222f,-20.8233f,  -21.5376f,-27.5989f,-22.2327f,  -22.3681f,-28.0374f,-22.5202f,  -22.9242f,-26.9670f,-24.1161f,  -23.7271f,-27.3881f,-24.4124f,  
	-24.1180f,-26.5023f,-26.1630f,  -24.8993f,-26.9151f,-26.4643f,  -25.1298f,-26.2422f,-28.3341f,  -25.8959f,-26.6539f,-28.6358f,  -25.9757f,-26.2179f,-30.5866f,  
	-26.7369f,-26.6360f,-30.8835f,  -26.6802f,-26.4554f,-32.8750f,  -27.4492f,-26.8864f,-33.1609f,  -27.2744f,-26.9739f,-35.1518f,  -28.0653f,-27.4218f,-35.4203f,  
	-27.7938f,-27.7834f,-37.3690f,  -28.6246f,-28.2509f,-37.6124f,  -28.2802f,-28.8856f,-39.4781f,  -29.1685f,-29.3720f,-39.6898f,  -28.7757f,-30.2715f,-41.4331f,  
	-29.7376f,-30.7735f,-41.6072f,  -29.3209f,-31.9223f,-43.1910f,  -30.3758f,-32.4337f,-43.3195f,  -29.9583f,-33.8097f,-44.7103f,  -31.1222f,-34.3211f,-44.7866f,  
	-30.7245f,-35.8957f,-45.9548f,  -32.0137f,-36.3944f,-45.9715f,  -31.6526f,-38.1334f,-46.8922f,  -33.0777f,-38.6050f,-46.8435f,  -32.7656f,-40.4697f,-47.4973f,  
	-34.3323f,-40.8974f,-47.3776f,  -34.0760f,-42.8461f,-47.7506f,  -35.7903f,-43.2111f,-47.5541f,  -35.5909f,-45.1999f,-47.6388f,  -37.4486f,-45.4837f,-47.3611f,  
	-37.3011f,-47.4682f,-47.1566f,  -39.2958f,-47.6519f,-46.7940f,  -39.1896f,-49.5891f,-46.3057f,  -41.3081f,-49.6555f,-45.8559f,  -41.2270f,-51.5043f,-45.0960f,  
	-43.4522f,-51.4378f,-44.5579f,  -43.3756f,-53.1612f,-43.5451f,  -45.6863f,-52.9503f,-42.9189f,  -45.5904f,-54.5162f,-41.6777f,  -47.9616f,-54.1525f,-40.9654f,  
	-47.8208f,-55.5344f,-39.5257f,  -50.2252f,-55.0133f,-38.7318f,  -50.0130f,-56.1911f,-37.1287f,  -52.4246f,-55.5128f,-36.2572f,  -52.1159f,-56.4729f,-34.5295f,  
	-54.5085f,-55.6412f,-33.5869f,  -54.0805f,-56.3765f,-31.7762f,  -56.4295f,-55.3988f,-30.7697f,  -55.8635f,-55.9082f,-28.9197f,  -58.1479f,-54.7943f,-27.8565f,  
	-57.4301f,-55.0823f,-26.0115f,  -59.6321f,-53.8438f,-24.8991f,  -58.7545f,-53.9199f,-23.1030f,  -60.8598f,-52.5694f,-21.9488f,  -59.8205f,-52.4468f,-20.2438f,  
	-61.8187f,-50.9971f,-19.0543f,  -60.6226f,-50.6919f,-17.4801f,  -62.5067f,-49.1553f,-16.2612f,  -61.1647f,-48.6856f,-14.8538f,  -62.9309f,-47.0731f,-13.6103f,  
	-61.4602f,-46.4585f,-12.4014f,  -63.1064f,-44.7802f,-11.1377f,  -61.5295f,-44.0404f,-10.1537f,  -63.0566f,-42.3040f,-8.8720f,  -61.4006f,-41.4585f,-8.1338f,  
	-62.8102f,-39.6707f,-6.8365f,  7.8837f,-33.9640f,-22.4365f,  6.9422f,-35.0107f,-22.6111f,  5.9220f,-34.6039f,-20.9390f,  4.9578f,-35.6170f,-21.1070f,  
	3.7987f,-35.0213f,-19.5891f,  2.8151f,-35.9952f,-19.7530f,  1.5431f,-35.2178f,-18.4190f,  0.5433f,-36.1474f,-18.5811f,  -0.8135f,-35.1996f,-17.4573f,  
	-1.8244f,-36.0804f,-17.6207f,  -3.2362f,-34.9778f,-16.7301f,  -4.2527f,-35.8059f,-16.8976f,  -5.6888f,-34.5684f,-16.2586f,  -6.7039f,-35.3413f,-16.4332f,  
	-8.1336f,-33.9928f,-16.0598f,  -9.1441f,-34.7100f,-16.2442f,  -10.5375f,-33.2780f,-16.1453f,  -11.5351f,-33.9387f,-16.3417f,  -12.8638f,-32.4540f,-16.5209f,  
	-13.8407f,-33.0604f,-16.7307f,  -15.0789f,-31.5565f,-17.1858f,  -16.0323f,-32.1131f,-17.4105f,  -17.1574f,-30.6255f,-18.1337f,  -18.0840f,-31.1337f,-18.3748f,  
	-19.0768f,-29.6987f,-19.3532f,  -19.9707f,-30.1675f,-19.6084f,  -20.8169f,-28.8222f,-20.8233f,  -21.6781f,-29.2570f,-21.0924f,  -22.3681f,-28.0374f,-22.5202f,  
	-23.1974f,-28.4474f,-22.8001f,  -23.7271f,-27.3881f,-24.4124f,  -24.5289f,-27.7824f,-24.7000f,  -24.8993f,-26.9151f,-26.4643f,  -25.6786f,-27.3012f,-26.7554f,  
	-25.8959f,-26.6539f,-28.6358f,  -26.6614f,-27.0400f,-28.9256f,  -26.7369f,-26.6360f,-30.8835f,  -27.5001f,-27.0293f,-31.1661f,  -27.4492f,-26.8864f,-33.1609f,  
	-28.2230f,-27.2921f,-33.4302f,  -28.0653f,-27.4218f,-35.4203f,  -28.8670f,-27.8441f,-35.6688f,  -28.6246f,-28.2509f,-37.6124f,  -29.4718f,-28.6909f,-37.8335f,  
	-29.1685f,-29.3720f,-39.6898f,  -30.0773f,-29.8276f,-39.8774f,  -29.7376f,-30.7735f,-41.6072f,  -30.7281f,-31.2403f,-41.7530f,  -30.3758f,-32.4337f,-43.3195f,  
	-31.4651f,-32.9040f,-43.4168f,  -31.1222f,-34.3211f,-44.7866f,  -32.3283f,-34.7838f,-44.8275f,  -32.0137f,-36.3944f,-45.9715f,  -33.3493f,-36.8360f,-45.9497f,  
	-33.0777f,-38.6050f,-46.8435f,  -34.5514f,-39.0096f,-46.7529f,  -34.3323f,-40.8974f,-47.3776f,  -35.9525f,-41.2468f,-47.2114f,  -35.7903f,-43.2111f,-47.5541f,  
	-37.5560f,-43.4867f,-47.3074f,  -37.4486f,-45.4837f,-47.3611f,  -39.3563f,-45.6662f,-47.0293f,  -39.2958f,-47.6519f,-46.7940f,  -41.3353f,-47.7233f,-46.3736f,  
	-41.3081f,-49.6555f,-45.8559f,  -43.4648f,-49.5986f,-45.3450f,  -43.4522f,-51.4378f,-44.5579f,  -45.7069f,-51.2395f,-43.9557f,  -45.6863f,-52.9503f,-42.9189f,  
	-48.0154f,-52.6003f,-42.2264f,  -47.9616f,-54.1525f,-40.9654f,  -50.3391f,-53.6441f,-40.1859f,  -50.2252f,-55.0133f,-38.7318f,  -52.6250f,-54.3448f,-37.8689f,  
	-52.4246f,-55.5128f,-36.2572f,  -54.8200f,-54.6860f,-35.3168f,  -54.5085f,-55.6412f,-33.5869f,  -56.8737f,-54.6620f,-32.5757f,  -56.4295f,-55.3988f,-30.7697f,  
	-58.7421f,-54.2756f,-29.6950f,  -58.1479f,-54.7943f,-27.8565f,  -60.3882f,-53.5380f,-26.7259f,  -59.6321f,-53.8438f,-24.8991f,  -61.7836f,-52.4668f,-23.7202f,  
	-60.8598f,-52.5694f,-21.9488f,  -62.9099f,-51.0847f,-20.7288f,  -61.8187f,-50.9971f,-19.0543f,  -63.7582f,-49.4173f,-17.7997f,  -62.5067f,-49.1553f,-16.2612f,  
	-64.3297f,-47.4922f,-14.9777f,  -62.9309f,-47.0731f,-13.6103f,  -64.6335f,-45.3383f,-12.3032f,  -63.1064f,-44.7802f,-11.1377f,  -64.6880f,-42.9825f,-9.8102f,  
	-63.0566f,-42.3040f,-8.8720f,  -64.5173f,-40.4517f,-7.5276f,  -62.8102f,-39.6707f,-6.8365f,  -64.1514f,-37.7708f,-5.4775f,  6.9422f,-35.0107f,-22.6111f,  
	5.9749f,-36.0512f,-22.7739f,  4.9578f,-35.6170f,-21.1070f,  3.9678f,-36.6208f,-21.2637f,  2.8151f,-35.9952f,-19.7530f,  1.8052f,-36.9569f,-19.9057f,  
	0.5433f,-36.1474f,-18.5811f,  -0.4817f,-37.0617f,-18.7327f,  -1.8244f,-36.0804f,-17.6207f,  -2.8592f,-36.9425f,-17.7742f,  -4.2527f,-35.8059f,-16.8976f,  
	-5.2902f,-36.6124f,-17.0561f,  -6.7039f,-35.3413f,-16.4332f,  -7.7407f,-36.0907f,-16.5995f,  -9.1441f,-34.7100f,-16.2442f,  -10.1715f,-35.4004f,-16.4209f,  
	-11.5351f,-33.9387f,-16.3417f,  -12.5446f,-34.5712f,-16.5309f,  -13.8407f,-33.0604f,-16.7307f,  -14.8286f,-33.6384f,-16.9341f,  -16.0323f,-32.1131f,-17.4105f,  
	-16.9948f,-32.6372f,-17.6298f,  -18.0840f,-31.1337f,-18.3748f,  -19.0134f,-31.6120f,-18.6087f,  -19.9707f,-30.1675f,-19.6084f,  -20.8658f,-30.6048f,-19.8572f,  
	-21.6781f,-29.2570f,-21.0924f,  -22.5380f,-29.6620f,-21.3537f,  -23.1974f,-28.4474f,-22.8001f,  -24.0250f,-28.8292f,-23.0713f,  -24.5289f,-27.7824f,-24.7000f,  
	-25.3275f,-28.1486f,-24.9776f,  -25.6786f,-27.3012f,-26.7554f,  -26.4552f,-27.6604f,-27.0350f,  -26.6614f,-27.0400f,-28.9256f,  -27.4260f,-27.4002f,-29.2016f,  
	-27.5001f,-27.0293f,-31.1661f,  -28.2642f,-27.3967f,-31.4328f,  -28.2230f,-27.2921f,-33.4302f,  -29.0031f,-27.6723f,-33.6803f,  -28.8670f,-27.8441f,-35.6688f,  
	-29.6802f,-28.2396f,-35.8957f,  -29.4718f,-28.6909f,-37.8335f,  -30.3340f,-29.1010f,-38.0312f,  -30.0773f,-29.8276f,-39.8774f,  -31.0091f,-30.2500f,-40.0376f,  
	-30.7281f,-31.2403f,-41.7530f,  -31.7473f,-31.6687f,-41.8688f,  -31.4651f,-32.9040f,-43.4168f,  -32.5912f,-33.3289f,-43.4797f,  -32.3283f,-34.7838f,-44.8275f,  
	-33.5758f,-35.1930f,-44.8309f,  -33.3493f,-36.8360f,-45.9497f,  -34.7293f,-37.2149f,-45.8871f,  -34.5514f,-39.0096f,-46.7529f,  -36.0748f,-39.3406f,-46.6168f,  
	-35.9525f,-41.2468f,-47.2114f,  -37.6215f,-41.5116f,-46.9963f,  -37.5560f,-43.4867f,-47.3074f,  -39.3700f,-43.6659f,-47.0075f,  -39.3563f,-45.6662f,-47.0293f,  
	-41.3081f,-45.7408f,-46.6404f,  -41.3353f,-47.7233f,-46.3736f,  -43.4133f,-47.6752f,-45.8927f,  -43.4648f,-49.5986f,-45.3450f,  -45.6518f,-49.4131f,-44.7701f,  
	-45.7069f,-51.2395f,-43.9557f,  -47.9813f,-50.9045f,-43.2871f,  -48.0154f,-52.6003f,-42.2264f,  -50.3527f,-52.1069f,-41.4662f,  -50.3391f,-53.6441f,-40.1859f,  
	-52.7134f,-52.9887f,-39.3370f,  -52.6250f,-54.3448f,-37.8689f,  -55.0097f,-53.5271f,-36.9364f,  -54.8200f,-54.6860f,-35.3168f,  -57.1886f,-53.7103f,-34.3070f,  
	-56.8737f,-54.6620f,-32.5757f,  -59.2025f,-53.5353f,-31.4955f,  -58.7421f,-54.2756f,-29.6950f,  -61.0098f,-53.0077f,-28.5519f,  -60.3882f,-53.5380f,-26.7259f,  
	-62.5762f,-52.1407f,-25.5278f,  -61.7836f,-52.4668f,-23.7202f,  -63.8774f,-50.9530f,-22.4748f,  -62.9099f,-51.0847f,-20.7288f,  -64.8980f,-49.4672f,-19.4431f,  
	-63.7582f,-49.4173f,-17.7997f,  -65.6328f,-47.7087f,-16.4801f,  -64.3297f,-47.4922f,-14.9777f,  -66.0848f,-45.7051f,-13.6304f,  -64.6335f,-45.3383f,-12.3032f,  
	-66.2669f,-43.4825f,-10.9324f,  -64.6880f,-42.9825f,-9.8102f,  -66.1983f,-41.0679f,-8.4202f,  -64.5173f,-40.4517f,-7.5276f,  -65.9048f,-38.4867f,-6.1218f,  
	-64.1514f,-37.7708f,-5.4775f,  -65.4174f,-35.7620f,-4.0584f,  5.9749f,-36.0512f,-22.7739f,  4.9814f,-37.0840f,-22.9250f,  3.9678f,-36.6208f,-21.2637f,  
	2.9506f,-37.6142f,-21.4084f,  1.8052f,-36.9569f,-19.9057f,  0.7689f,-37.9048f,-20.0469f,  -0.4817f,-37.0617f,-18.7327f,  -1.5321f,-37.9588f,-18.8735f,  
	-2.8592f,-36.9425f,-17.7742f,  -3.9167f,-37.7843f,-17.9177f,  -5.2902f,-36.6124f,-17.0561f,  -6.3513f,-37.3964f,-17.2051f,  -7.7407f,-36.0907f,-16.5995f,  
	-8.7965f,-36.8140f,-16.7570f,  -10.1715f,-35.4004f,-16.4209f,  -11.2129f,-36.0628f,-16.5895f,  -12.5446f,-34.5712f,-16.5309f,  -13.5670f,-35.1751f,-16.7126f,  
	-14.8286f,-33.6384f,-16.9341f,  -15.8279f,-34.1837f,-17.1309f,  -16.9948f,-32.6372f,-17.6298f,  -17.9615f,-33.1308f,-17.8414f,  -19.0134f,-31.6120f,-18.6087f,  
	-19.9452f,-32.0578f,-18.8357f,  -20.8658f,-30.6048f,-19.8572f,  -21.7602f,-31.0111f,-20.0980f,  -22.5380f,-29.6620f,-21.3537f,  -23.3962f,-30.0376f,-21.6063f,  
	-24.0250f,-28.8292f,-23.0713f,  -24.8485f,-29.1816f,-23.3327f,  -25.3275f,-28.1486f,-24.9776f,  -26.1216f,-28.4866f,-25.2438f,  -26.4552f,-27.6604f,-27.0350f,  
	-27.2284f,-27.9927f,-27.3011f,  -27.4260f,-27.4002f,-29.2016f,  -28.1883f,-27.7337f,-29.4624f,  -28.2642f,-27.3967f,-31.4328f,  -29.0308f,-27.7384f,-31.6810f,  
	-29.0031f,-27.6723f,-33.6803f,  -29.7903f,-28.0260f,-33.9096f,  -29.6802f,-28.2396f,-35.8957f,  -30.5034f,-28.6061f,-36.1003f,  -30.3340f,-29.1010f,-38.0312f,  
	-31.2139f,-29.4796f,-38.2028f,  -31.0091f,-30.2500f,-40.0376f,  -31.9638f,-30.6362f,-40.1691f,  -31.7473f,-31.6687f,-41.8688f,  -32.7978f,-32.0548f,-41.9513f,  
	-32.5912f,-33.3289f,-43.4797f,  -33.7533f,-33.7042f,-43.5065f,  -33.5758f,-35.1930f,-44.8309f,  -34.8632f,-35.5441f,-44.7950f,  -34.7293f,-37.2149f,-45.8871f,  
	-36.1550f,-37.5251f,-45.7804f,  -36.0748f,-39.3406f,-46.6168f,  -37.6440f,-39.5922f,-46.4331f,  -37.6215f,-41.5116f,-46.9963f,  -39.3367f,-41.6851f,-46.7290f,  
	-39.3700f,-43.6659f,-47.0075f,  -41.2270f,-43.7420f,-46.6516f,  -41.3081f,-45.7408f,-46.6404f,  -43.2984f,-45.7005f,-46.1917f,  -43.4133f,-47.6752f,-45.8927f,  
	-45.5221f,-47.5024f,-45.3484f,  -45.6518f,-49.4131f,-44.7701f,  -47.8598f,-49.0940f,-44.1294f,  -47.9813f,-50.9045f,-43.2871f,  -50.2658f,-50.4282f,-42.5508f,  
	-50.3527f,-52.1069f,-41.4662f,  -52.6885f,-51.4679f,-40.6365f,  -52.7134f,-52.9887f,-39.3370f,  -55.0743f,-52.1845f,-38.4180f,  -55.0097f,-53.5271f,-36.9364f,  
	-57.3688f,-52.5601f,-35.9338f,  -57.1886f,-53.7103f,-34.3070f,  -59.5214f,-52.5859f,-33.2273f,  -59.2025f,-53.5353f,-31.4955f,  -61.4862f,-52.2620f,-30.3461f,  
	-61.0098f,-53.0077f,-28.5519f,  -63.2241f,-51.5967f,-27.3407f,  -62.5762f,-52.1407f,-25.5278f,  -64.7047f,-50.6043f,-24.2626f,  -63.8774f,-50.9530f,-22.4748f,  
	-65.9064f,-49.3041f,-21.1631f,  -64.8980f,-49.4672f,-19.4431f,  -66.8178f,-47.7189f,-18.0919f,  -65.6328f,-47.7087f,-16.4801f,  -67.4353f,-45.8743f,-15.0965f,  
	-66.0848f,-45.7051f,-13.6304f,  -67.7662f,-43.7954f,-12.2193f,  -66.2669f,-43.4825f,-10.9324f,  -67.8243f,-41.5087f,-9.4991f,  -66.1983f,-41.0679f,-8.4202f,  
	-67.6306f,-39.0397f,-6.9691f,  -65.9048f,-38.4867f,-6.1218f,  -67.2124f,-36.4121f,-4.6563f,  -65.4174f,-35.7620f,-4.0584f,  -66.6010f,-33.6492f,-2.5820f,  
	4.9814f,-37.0840f,-22.9250f,  3.9599f,-38.1080f,-23.0635f,  2.9506f,-37.6142f,-21.4084f,  1.9060f,-38.5954f,-21.5410f,  0.7689f,-37.9048f,-20.0469f,  
	-0.2942f,-38.8372f,-20.1765f,  -1.5321f,-37.9588f,-18.8735f,  -2.6070f,-38.8369f,-19.0034f,  -3.9167f,-37.7843f,-17.9177f,  -4.9996f,-38.6048f,-18.0507f,  
	-6.3513f,-37.3964f,-17.2051f,  -7.4334f,-38.1553f,-17.3444f,  -8.7965f,-36.8140f,-16.7570f,  -9.8682f,-37.5099f,-16.9057f,  -11.2129f,-36.0628f,-16.5895f,  
	-12.2691f,-36.6970f,-16.7499f,  -13.5670f,-35.1751f,-16.7126f,  -14.6032f,-35.7466f,-16.8867f,  -15.8279f,-34.1837f,-17.1309f,  -16.8329f,-34.6980f,-17.3195f,  
	-17.9615f,-33.1308f,-17.8414f,  -18.9321f,-33.5912f,-18.0454f,  -19.9452f,-32.0578f,-18.8357f,  -20.8773f,-32.4718f,-19.0544f,  -21.7602f,-31.0111f,-20.0980f,  
	-22.6531f,-31.3869f,-20.3298f,  -23.3962f,-30.0376f,-21.6063f,  -24.2498f,-30.3825f,-21.8489f,  -24.8485f,-29.1816f,-23.3327f,  -25.6665f,-29.5045f,-23.5828f,  
	-26.1216f,-28.4866f,-25.2438f,  -26.9105f,-28.7967f,-25.4970f,  -27.2284f,-27.9927f,-27.3011f,  -27.9966f,-28.2974f,-27.5525f,  -28.1883f,-27.7337f,-29.4624f,  
	-28.9494f,-28.0412f,-29.7052f,  -29.0308f,-27.7384f,-31.6810f,  -29.8001f,-28.0537f,-31.9093f,  -29.7903f,-28.0260f,-33.9096f,  -30.5826f,-28.3513f,-34.1177f,  
	-30.5034f,-28.6061f,-36.1003f,  -31.3391f,-28.9425f,-36.2798f,  -31.2139f,-29.4796f,-38.2028f,  -32.1112f,-29.8242f,-38.3467f,  -31.9638f,-30.6362f,-40.1691f,  
	-32.9445f,-30.9832f,-40.2683f,  -32.7978f,-32.0548f,-41.9513f,  -33.8788f,-32.3951f,-41.9990f,  -33.7533f,-33.7042f,-43.5065f,  -34.9507f,-34.0258f,-43.4954f,  
	-34.8632f,-35.5441f,-44.7950f,  -36.1918f,-35.8314f,-44.7164f,  -36.1550f,-37.5251f,-45.7804f,  -37.6231f,-37.7611f,-45.6274f,  -37.6440f,-39.5922f,-46.4331f,  
	-39.2570f,-39.7578f,-46.1987f,  -39.3367f,-41.6851f,-46.7290f,  -41.0934f,-41.7610f,-46.4071f,  -41.2270f,-43.7420f,-46.6516f,  -43.1222f,-43.7082f,-46.2369f,  
	-43.2984f,-45.7005f,-46.1917f,  -45.3199f,-45.5400f,-45.6806f,  -45.5221f,-47.5024f,-45.3484f,  -47.6531f,-47.1999f,-44.7389f,  -47.8598f,-49.0940f,-44.1294f,  
	-50.0797f,-48.6365f,-43.4211f,  -50.2658f,-50.4282f,-42.5508f,  -52.5505f,-49.8083f,-41.7449f,  -52.6885f,-51.4679f,-40.6365f,  -55.0128f,-50.6810f,-39.7363f,  
	-55.0743f,-52.1845f,-38.4180f,  -57.4112f,-51.2309f,-37.4284f,  -57.3688f,-52.5601f,-35.9338f,  -59.6931f,-51.4437f,-34.8607f,  -59.5214f,-52.5859f,-33.2273f,  
	-61.8094f,-51.3136f,-32.0777f,  -61.4862f,-52.2620f,-30.3461f,  -63.7162f,-50.8440f,-29.1277f,  -63.2241f,-51.5967f,-27.3407f,  -65.3776f,-50.0446f,-26.0615f,  
	-64.7047f,-50.6043f,-24.2626f,  -66.7663f,-48.9309f,-22.9304f,  -65.9064f,-49.3041f,-21.1631f,  -67.8644f,-47.5226f,-19.7855f,  -66.8178f,-47.7189f,-18.0919f,  
	-68.6621f,-45.8430f,-16.6764f,  -67.4353f,-45.8743f,-15.0965f,  -69.1604f,-43.9159f,-13.6489f,  -67.7662f,-43.7954f,-12.2193f,  -69.3675f,-41.7665f,-10.7457f,  
	-67.8243f,-41.5087f,-9.4991f,  -69.2992f,-39.4204f,-8.0049f,  -67.6306f,-39.0397f,-6.9691f,  -68.9785f,-36.9014f,-5.4586f,  -67.2124f,-36.4121f,-4.6563f,  
	-68.4331f,-34.2333f,-3.1339f,  -66.6010f,-33.6492f,-2.5820f,  -67.6958f,-31.4374f,-1.0510f,  3.9599f,-38.1080f,-23.0635f,  2.9101f,-39.1216f,-23.1892f,  
	1.9060f,-38.5954f,-21.5410f,  0.8334f,-39.5628f,-21.6612f,  -0.2942f,-38.8372f,-20.1765f,  -1.3834f,-39.7522f,-20.2946f,  -2.6070f,-38.8369f,-19.0034f,  
	-3.7089f,-39.6951f,-19.1218f,  -4.9996f,-38.6048f,-18.0507f,  -6.1056f,-39.4014f,-18.1729f,  -7.4334f,-38.1553f,-17.3444f,  -8.5337f,-38.8877f,-17.4740f,  
	-9.8682f,-37.5099f,-16.9057f,  -10.9571f,-38.1780f,-17.0452f,  -12.2691f,-36.6970f,-16.7499f,  -13.3416f,-37.2991f,-16.9014f,  -14.6032f,-35.7466f,-16.8867f,  
	-15.6470f,-36.2867f,-17.0521f,  -16.8329f,-34.6980f,-17.3195f,  -17.8438f,-35.1786f,-17.4999f,  -18.9321f,-33.5912f,-18.0454f,  -19.9044f,-34.0189f,-18.2407f,  
	-20.8773f,-32.4718f,-19.0544f,  -21.8086f,-32.8541f,-19.2638f,  -22.6531f,-31.3869f,-20.3298f,  -23.5418f,-31.7308f,-20.5517f,  -24.2498f,-30.3825f,-21.8489f,  
	-25.0975f,-30.6967f,-22.0805f,  -25.6665f,-29.5045f,-23.5828f,  -26.4779f,-29.7984f,-23.8203f,  -26.9105f,-28.7967f,-25.4970f,  -27.6922f,-29.0782f,-25.7359f,  
	-27.9966f,-28.2974f,-27.5525f,  -28.7603f,-28.5755f,-27.7867f,  -28.9494f,-28.0412f,-29.7052f,  -29.7095f,-28.3222f,-29.9290f,  -29.8001f,-28.0537f,-31.9093f,  
	-30.5703f,-28.3410f,-32.1172f,  -30.5826f,-28.3513f,-34.1177f,  -31.3825f,-28.6477f,-34.3016f,  -31.3391f,-28.9425f,-36.2798f,  -32.1867f,-29.2467f,-36.4328f,  
	-32.1112f,-29.8242f,-38.3467f,  -33.0289f,-30.1325f,-38.4597f,  -32.9445f,-30.9832f,-40.2683f,  -33.9502f,-31.2880f,-40.3343f,  -33.8788f,-32.3951f,-41.9990f,  
	-34.9904f,-32.6858f,-42.0104f,  -34.9507f,-34.0258f,-43.4954f,  -36.1846f,-34.2884f,-43.4431f,  -36.1918f,-35.8314f,-44.7164f,  -37.5592f,-36.0499f,-44.5932f,  
	-37.6231f,-37.7611f,-45.6274f,  -39.1321f,-37.9170f,-45.4254f,  -39.2570f,-39.7578f,-46.1987f,  -40.9096f,-39.8316f,-45.9111f,  -41.0934f,-41.7610f,-46.4071f,  
	-42.8877f,-41.7326f,-46.0276f,  -43.1222f,-43.7082f,-46.2369f,  -45.0487f,-43.5596f,-45.7611f,  -45.3199f,-45.5400f,-45.6806f,  -47.3646f,-45.2544f,-45.1051f,  
	-47.6531f,-47.1999f,-44.7389f,  -49.7979f,-46.7626f,-44.0622f,  -50.0797f,-48.6365f,-43.4211f,  -52.3020f,-48.0383f,-42.6435f,  -52.5505f,-49.8083f,-41.7449f,  
	-54.8261f,-49.0423f,-40.8686f,  -55.0128f,-50.6810f,-39.7363f,  -57.3151f,-49.7457f,-38.7652f,  -57.4112f,-51.2309f,-37.4284f,  -59.7149f,-50.1282f,-36.3678f,  
	-59.6931f,-51.4437f,-34.8607f,  -61.9738f,-50.1789f,-33.7170f,  -61.8094f,-51.3136f,-32.0777f,  -64.0440f,-49.8956f,-30.8584f,  -63.7162f,-50.8440f,-29.1277f,  
	-65.8847f,-49.2834f,-27.8407f,  -65.3776f,-50.0446f,-26.0615f,  -67.4630f,-48.3539f,-24.7148f,  -66.7663f,-48.9309f,-22.9304f,  -68.7547f,-47.1231f,-21.5318f,  
	-67.8644f,-47.5226f,-19.7855f,  -69.7440f,-45.6119f,-18.3432f,  -68.6621f,-45.8430f,-16.6764f,  -70.4256f,-43.8418f,-15.1968f,  -69.1604f,-43.9159f,-13.6489f,  
	-70.8015f,-41.8370f,-12.1388f,  -69.3675f,-41.7665f,-10.7457f,  -70.8821f,-39.6223f,-9.2115f,  -69.2992f,-39.4204f,-8.0049f,  -70.6855f,-37.2214f,-6.4515f,  
	-68.9785f,-36.9014f,-5.4586f,  -70.2351f,-34.6585f,-3.8915f,  -68.4331f,-34.2333f,-3.1339f,  -69.5605f,-31.9555f,-1.5572f,  -67.6958f,-31.4374f,-1.0510f,  
	-68.6949f,-29.1336f,0.5309f,  2.9101f,-39.1216f,-23.1892f,  1.8313f,-40.1233f,-23.3019f,  0.8334f,-39.5628f,-21.6612f,  -0.2668f,-40.5146f,-21.7691f,  
	-1.3834f,-39.7522f,-20.2946f,  -2.5011f,-40.6488f,-20.4000f,  -3.7089f,-39.6951f,-19.1218f,  -4.8359f,-40.5310f,-19.2285f,  -6.1056f,-39.4014f,-18.1729f,  
	-7.2322f,-40.1725f,-18.2847f,  -8.5337f,-38.8877f,-17.4740f,  -9.6533f,-39.5928f,-17.5936f,  -10.9571f,-38.1780f,-17.0452f,  -12.0648f,-38.8148f,-17.1747f,  
	-13.3416f,-37.2991f,-16.9014f,  -14.4240f,-37.8699f,-17.0436f,  -15.6470f,-36.2867f,-17.0521f,  -16.6989f,-36.7931f,-17.2084f,  -17.8438f,-35.1786f,-17.4999f,  
	-18.8581f,-35.6260f,-17.6710f,  -19.9044f,-34.0189f,-18.2407f,  -20.8771f,-34.4140f,-18.4265f,  -21.8086f,-32.8541f,-19.2638f,  -22.7364f,-33.2034f,-19.4631f,  
	-23.5418f,-31.7308f,-20.5517f,  -24.4245f,-32.0428f,-20.7624f,  -25.0975f,-30.6967f,-22.0805f,  -25.9378f,-30.9806f,-22.2995f,  -26.4779f,-29.7984f,-23.8203f,  
	-27.2805f,-30.0626f,-24.0437f,  -27.6922f,-29.0782f,-25.7359f,  -28.4670f,-29.3323f,-25.9582f,  -28.7603f,-28.5755f,-27.7867f,  -29.5200f,-28.8270f,-28.0024f,  
	-29.7095f,-28.3222f,-29.9290f,  -30.4665f,-28.5753f,-30.1332f,  -30.5703f,-28.3410f,-32.1172f,  -31.3433f,-28.6003f,-32.3020f,  -31.3825f,-28.6477f,-34.3016f,  
	-32.1891f,-28.9135f,-34.4602f,  -32.1867f,-29.2467f,-36.4328f,  -33.0494f,-29.5173f,-36.5562f,  -33.0289f,-30.1325f,-38.4597f,  -33.9661f,-30.4018f,-38.5409f,  
	-33.9502f,-31.2880f,-40.3343f,  -34.9814f,-31.5473f,-40.3655f,  -34.9904f,-32.6858f,-42.0104f,  -36.1334f,-32.9226f,-41.9823f,  -36.1846f,-34.2884f,-43.4431f,  
	-37.4533f,-34.4877f,-43.3480f,  -37.5592f,-36.0499f,-44.5932f,  -38.9643f,-36.1941f,-44.4226f,  -39.1321f,-37.9170f,-45.4254f,  -40.6787f,-37.9871f,-45.1719f,  
	-40.9096f,-39.8316f,-45.9111f,  -42.5989f,-39.8072f,-45.5673f,  -42.8877f,-41.7326f,-46.0276f,  -44.7132f,-41.5950f,-45.5883f,  -45.0487f,-43.5596f,-45.7611f,  
	-46.9993f,-43.2910f,-45.2219f,  -47.3646f,-45.2544f,-45.1051f,  -49.4249f,-44.8385f,-44.4632f,  -49.7979f,-46.7626f,-44.0622f,  -51.9471f,-46.1882f,-43.3168f,  
	-52.3020f,-48.0383f,-42.6435f,  -54.5175f,-47.2966f,-41.7956f,  -54.8261f,-49.0423f,-40.8686f,  -57.0821f,-48.1297f,-39.9212f,  -57.3151f,-49.7457f,-38.7652f,  
	-59.5863f,-48.6621f,-37.7227f,  -59.7149f,-50.1282f,-36.3678f,  -61.9766f,-48.8771f,-35.2360f,  -61.9738f,-50.1789f,-33.7170f,  -64.2019f,-48.7676f,-32.5030f,  
	-64.0440f,-49.8956f,-30.8584f,  -66.2172f,-48.3339f,-29.5698f,  -65.8847f,-49.2834f,-27.8407f,  -67.9844f,-47.5830f,-26.4856f,  -67.4630f,-48.3539f,-24.7148f,  
	-69.4736f,-46.5273f,-23.3011f,  -68.7547f,-47.1231f,-21.5318f,  -70.6627f,-45.1847f,-20.0682f,  -69.7440f,-45.6119f,-18.3432f,  -71.5401f,-43.5744f,-16.8365f,  
	-70.4256f,-43.8418f,-15.1968f,  -72.1018f,-41.7189f,-13.6546f,  -70.8015f,-41.8370f,-12.1388f,  -72.3521f,-39.6420f,-10.5682f,  -70.8821f,-39.6223f,-9.2115f,  
	-72.3041f,-37.3667f,-7.6181f,  -70.6855f,-37.2214f,-6.4515f,  -71.9764f,-34.9175f,-4.8418f,  -70.2351f,-34.6585f,-3.8915f,  -71.3944f,-32.3165f,-2.2705f,  
	-69.5605f,-31.9555f,-1.5572f,  -70.5883f,-29.5857f,0.0698f,  -68.6949f,-29.1336f,0.5309f,  -69.5922f,-26.7451f,2.1596f,  1.8313f,-40.1233f,-23.3019f,  
	0.7236f,-41.1110f,-23.4017f,  -0.2668f,-40.5146f,-21.7691f,  -1.3970f,-41.4496f,-21.8635f,  -2.5011f,-40.6488f,-20.4000f,  -3.6460f,-41.5247f,-20.4929f,  
	-4.8359f,-40.5310f,-19.2285f,  -5.9858f,-41.3425f,-19.3239f,  -7.2322f,-40.1725f,-18.2847f,  -8.3802f,-40.9172f,-18.3856f,  -9.6533f,-39.5928f,-17.5936f,  
	-10.7946f,-40.2676f,-17.7020f,  -12.0648f,-38.8148f,-17.1747f,  -13.1849f,-39.4203f,-17.2943f,  -14.4240f,-37.8699f,-17.0436f,  -15.5170f,-38.4068f,-17.1760f,  
	-16.6989f,-36.7931f,-17.2084f,  -17.7563f,-37.2656f,-17.3550f,  -18.8581f,-35.6260f,-17.6710f,  -19.8745f,-36.0399f,-17.8323f,  -20.8771f,-34.4140f,-18.4265f,  
	-21.8475f,-34.7751f,-18.6019f,  -22.7364f,-33.2034f,-19.4631f,  -23.6589f,-33.5197f,-19.6512f,  -24.4245f,-32.0428f,-20.7624f,  -25.2997f,-32.3233f,-20.9608f,  
	-25.9378f,-30.9806f,-22.2995f,  -26.7685f,-31.2338f,-22.5048f,  -27.2805f,-30.0626f,-24.0437f,  -28.0742f,-30.2985f,-24.2511f,  -28.4670f,-29.3323f,-25.9582f,  
	-29.2349f,-29.5593f,-26.1627f,  -29.5200f,-28.8270f,-28.0024f,  -30.2732f,-29.0503f,-28.1995f,  -30.4665f,-28.5753f,-30.1332f,  -31.2223f,-28.8011f,-30.3153f,  
	-31.3433f,-28.6003f,-32.3020f,  -32.1185f,-28.8303f,-32.4627f,  -32.1891f,-28.9135f,-34.4602f,  -33.0059f,-29.1479f,-34.5906f,  -33.0494f,-29.5173f,-36.5562f,  
	-33.9264f,-29.7518f,-36.6493f,  -33.9661f,-30.4018f,-38.5409f,  -34.9241f,-30.6295f,-38.5888f,  -34.9814f,-31.5473f,-40.3655f,  -36.0392f,-31.7573f,-40.3590f,  
	-36.1334f,-32.9226f,-41.9823f,  -37.3073f,-33.1014f,-41.9132f,  -37.4533f,-34.4877f,-43.3480f,  -38.7564f,-34.6186f,-43.2074f,  -38.9643f,-36.1941f,-44.4226f,  
	-40.4047f,-36.2588f,-44.2024f,  -40.6787f,-37.9871f,-45.1719f,  -42.2607f,-37.9652f,-44.8639f,  -42.5989f,-39.8072f,-45.5673f,  -44.3190f,-39.6798f,-45.1654f,  
	-44.7132f,-41.5950f,-45.5883f,  -46.5634f,-41.3432f,-45.0871f,  -46.9993f,-43.2910f,-45.2219f,  -48.9673f,-42.8971f,-44.6172f,  -49.4249f,-44.8385f,-44.4632f,  
	-51.4919f,-44.2896f,-43.7533f,  -51.9471f,-46.1882f,-43.3168f,  -54.0922f,-45.4737f,-42.5015f,  -54.5175f,-47.2966f,-41.7956f,  -56.7162f,-46.4107f,-40.8768f,  
	-57.0821f,-48.1297f,-39.9212f,  -59.3095f,-47.0702f,-38.9024f,  -59.5863f,-48.6621f,-37.7227f,  -61.8177f,-47.4306f,-36.6088f,  -61.9766f,-48.8771f,-35.2360f,  
	-64.1872f,-47.4795f,-34.0336f,  -64.2019f,-48.7676f,-32.5030f,  -66.3695f,-47.2122f,-31.2193f,  -66.2172f,-48.3339f,-29.5698f,  -68.3217f,-46.6313f,-28.2127f,  
	-67.9844f,-47.5830f,-26.4856f,  -70.0086f,-45.7454f,-25.0631f,  -69.4736f,-46.5273f,-23.3011f,  -71.4022f,-44.5689f,-21.8220f,  -70.6627f,-45.1847f,-20.0682f,  
	-72.4848f,-43.1183f,-18.5399f,  -71.5401f,-43.5744f,-16.8365f,  -73.2460f,-41.4143f,-15.2671f,  -72.1018f,-41.7189f,-13.6546f,  -73.6843f,-39.4791f,-12.0518f,  
	-72.3521f,-39.6420f,-10.5682f,  -73.8068f,-37.3349f,-8.9384f,  -72.3041f,-37.3667f,-7.6181f,  -73.6270f,-35.0058f,-5.9686f,  -71.9764f,-34.9175f,-4.8418f,  
	-73.1660f,-32.5142f,-3.1785f,  -71.3944f,-32.3165f,-2.2705f,  -72.4500f,-29.8825f,-0.5995f,  -70.5883f,-29.5857f,0.0698f,  -71.5105f,-27.1313f,1.7433f,  
	-69.5922f,-26.7451f,2.1596f,  -70.3822f,-24.2805f,3.8303f,  0.7236f,-41.1110f,-23.4017f,  -0.4155f,-42.0836f,-23.4872f,  -1.3970f,-41.4496f,-21.8635f,  
	-2.5563f,-42.3655f,-21.9447f,  -3.6460f,-41.5247f,-20.4929f,  -4.8161f,-42.3775f,-20.5739f,  -5.9858f,-41.3425f,-19.3239f,  -7.1596f,-42.1286f,-19.4078f,  
	-8.3802f,-40.9172f,-18.3856f,  -9.5527f,-41.6327f,-18.4741f,  -10.7946f,-40.2676f,-17.7020f,  -11.9508f,-40.9115f,-17.7998f,  -13.1849f,-39.4203f,-17.2943f,  
	-14.3183f,-39.9922f,-17.4032f,  -15.5170f,-38.4068f,-17.1760f,  -16.6179f,-38.9097f,-17.2980f,  -17.7563f,-37.2656f,-17.3550f,  -18.8179f,-37.7041f,-17.4912f,  
	-19.8745f,-36.0399f,-17.8323f,  -20.8904f,-36.4191f,-17.9828f,  -21.8475f,-34.7751f,-18.6019f,  -22.8138f,-35.1022f,-18.7659f,  -23.6589f,-33.5197f,-19.6512f,  
	-24.5743f,-33.8033f,-19.8270f,  -25.2997f,-32.3233f,-20.9608f,  -26.1648f,-32.5718f,-21.1456f,  -26.7685f,-31.2338f,-22.5048f,  -27.5888f,-31.4577f,-22.6945f,  
	-28.0742f,-30.2985f,-24.2511f,  -28.8591f,-30.5067f,-24.4413f,  -29.2349f,-29.5593f,-26.1627f,  -29.9936f,-29.7579f,-26.3493f,  -30.2732f,-29.0503f,-28.1995f,  
	-31.0215f,-29.2467f,-28.3753f,  -31.2223f,-28.8011f,-30.3153f,  -31.9759f,-28.9986f,-30.4743f,  -32.1185f,-28.8303f,-32.4627f,  -32.8989f,-29.0309f,-32.5963f,  
	-33.0059f,-29.1479f,-34.5906f,  -33.8317f,-29.3490f,-34.6920f,  -33.9264f,-29.7518f,-36.6493f,  -34.8193f,-29.9483f,-36.7108f,  -34.9241f,-30.6295f,-38.5888f,  
	-35.9036f,-30.8124f,-38.6009f,  -36.0392f,-31.7573f,-40.3590f,  -37.1237f,-31.9145f,-40.3132f,  -37.3073f,-33.1014f,-41.9132f,  -38.5120f,-33.2176f,-41.8004f,  
	-38.7564f,-34.6186f,-43.2074f,  -40.0922f,-34.6764f,-43.0190f,  -40.4047f,-36.2588f,-44.2024f,  -41.8789f,-36.2381f,-43.9295f,  -42.2607f,-37.9652f,-44.8639f,  
	-43.8730f,-37.8469f,-44.4996f,  -44.3190f,-39.6798f,-45.1654f,  -46.0645f,-39.4443f,-44.7032f,  -46.5634f,-41.3432f,-45.0871f,  -48.4329f,-40.9716f,-44.5215f,  
	-48.9673f,-42.8971f,-44.6172f,  -50.9442f,-42.3752f,-43.9456f,  -51.4919f,-44.2896f,-43.7533f,  -53.5576f,-43.6048f,-42.9741f,  -54.0922f,-45.4737f,-42.5015f,  
	-56.2237f,-44.6180f,-41.6155f,  -56.7162f,-46.4107f,-40.8768f,  -58.8893f,-45.3800f,-39.8865f,  -59.3095f,-47.0702f,-38.9024f,  -61.4999f,-45.8641f,-37.8118f,  
	-61.8177f,-47.4306f,-36.6088f,  -64.0002f,-46.0532f,-35.4238f,  -64.1872f,-47.4795f,-34.0336f,  -66.3389f,-45.9375f,-32.7608f,  -66.3695f,-47.2122f,-31.2193f,  
	-68.4691f,-45.5152f,-29.8663f,  -68.3217f,-46.6313f,-28.2127f,  -70.3507f,-44.7908f,-26.7875f,  -70.0086f,-45.7454f,-25.0631f,  -71.9501f,-43.7749f,-23.5746f,  
	-71.4022f,-44.5689f,-21.8220f,  -73.2436f,-42.4814f,-20.2779f,  -72.4848f,-43.1183f,-18.5399f,  -74.2147f,-40.9284f,-16.9486f,  -73.2460f,-41.4143f,-15.2671f,  
	-74.8556f,-39.1365f,-13.6369f,  -73.6843f,-39.4791f,-12.0518f,  -75.1677f,-37.1265f,-10.3898f,  -73.8068f,-37.3349f,-8.9384f,  -75.1587f,-34.9220f,-7.2527f,  
	-73.6270f,-35.0058f,-5.9686f,  -74.8449f,-32.5453f,-4.2658f,  -73.1660f,-32.5142f,-3.1785f,  -74.2482f,-30.0189f,-1.4656f,  -72.4500f,-29.8825f,-0.5995f,  
	-73.3963f,-27.3639f,1.1176f,  -71.5105f,-27.1313f,1.7433f,  -72.3215f,-24.6010f,3.4584f,  -70.3822f,-24.2805f,3.8303f,  -71.0595f,-21.7496f,5.5376f,  
	-0.4155f,-42.0836f,-23.4872f,  -1.5853f,-43.0390f,-23.5588f,  -2.5563f,-42.3655f,-21.9447f,  -3.7428f,-43.2599f,-22.0132f,  -4.8161f,-42.3775f,-20.5739f,  
	-6.0122f,-43.2061f,-20.6425f,  -7.1596f,-42.1286f,-19.4078f,  -8.3605f,-42.8868f,-19.4780f,  -9.5527f,-41.6327f,-18.4741f,  -10.7429f,-42.3179f,-18.5515f,  
	-11.9508f,-40.9115f,-17.7998f,  -13.1233f,-41.5223f,-17.8861f,  -14.3183f,-39.9922f,-17.4032f,  -15.4624f,-40.5301f,-17.5011f,  -16.6179f,-38.9097f,-17.2980f,  
	-17.7255f,-39.3781f,-17.4092f,  -18.8179f,-37.7041f,-17.4912f,  -19.8813f,-38.1071f,-17.6163f,  -20.8904f,-36.4191f,-17.9828f,  -21.9038f,-36.7631f,-18.1216f,  
	-22.8138f,-35.1022f,-18.7659f,  -23.7741f,-35.3953f,-18.9175f,  -24.5743f,-33.8033f,-19.8270f,  -25.4800f,-34.0537f,-19.9893f,  -26.1648f,-32.5718f,-21.1456f,  
	-27.0191f,-32.7900f,-21.3152f,  -27.5888f,-31.4577f,-22.6945f,  -28.3989f,-31.6529f,-22.8676f,  -28.8591f,-30.5067f,-24.4413f,  -29.6326f,-30.6858f,-24.6141f,  
	-29.9936f,-29.7579f,-26.3493f,  -30.7444f,-29.9295f,-26.5155f,  -31.0215f,-29.2467f,-28.3753f,  -31.7639f,-29.4154f,-28.5292f,  -31.9759f,-28.9986f,-30.4743f,  
	-32.7304f,-29.1681f,-30.6077f,  -32.8989f,-29.0309f,-32.5963f,  -33.6836f,-29.2004f,-32.7025f,  -33.8317f,-29.3490f,-34.6920f,  -34.6689f,-29.5153f,-34.7635f,  
	-34.8193f,-29.9483f,-36.7108f,  -35.7288f,-30.1042f,-36.7381f,  -35.9036f,-30.8124f,-38.6009f,  -36.9055f,-30.9475f,-38.5756f,  -37.1237f,-31.9145f,-40.3132f,  
	-38.2352f,-32.0149f,-40.2259f,  -38.5120f,-33.2176f,-41.8004f,  -39.7466f,-33.2669f,-41.6421f,  -40.0922f,-34.6764f,-43.0190f,  -41.4601f,-34.6553f,-42.7799f,  
	-41.8789f,-36.2381f,-43.9295f,  -43.3827f,-36.1275f,-43.6021f,  -43.8730f,-37.8469f,-44.4996f,  -45.5109f,-37.6269f,-44.0766f,  -46.0645f,-39.4443f,-44.7032f,  
	-47.8307f,-39.0951f,-44.1779f,  -48.4329f,-40.9716f,-44.5215f,  -50.3134f,-40.4775f,-43.8902f,  -50.9442f,-42.3752f,-43.9456f,  -52.9228f,-41.7218f,-43.2054f,  
	-53.5576f,-43.6048f,-42.9741f,  -55.6129f,-42.7823f,-42.1248f,  -56.2237f,-44.6180f,-41.6155f,  -58.3328f,-43.6202f,-40.6584f,  -58.8893f,-45.3800f,-39.8865f,  
	-61.0285f,-44.2045f,-38.8245f,  -61.4999f,-45.8641f,-37.8118f,  -63.6441f,-44.5131f,-36.6499f,  -64.0002f,-46.0532f,-35.4238f,  -66.1260f,-44.5316f,-34.1680f,  
	-66.3389f,-45.9375f,-32.7608f,  -68.4241f,-44.2538f,-31.4184f,  -68.4691f,-45.5152f,-29.8663f,  -70.4940f,-43.6797f,-28.4450f,  -70.3507f,-44.7908f,-26.7875f,  
	-72.2970f,-42.8164f,-25.2960f,  -71.9501f,-43.7749f,-23.5746f,  -73.8036f,-41.6745f,-22.0207f,  -73.2436f,-42.4814f,-20.2779f,  -74.9913f,-40.2694f,-18.6705f,  
	-74.2147f,-40.9284f,-16.9486f,  -75.8462f,-38.6198f,-15.2964f,  -74.8556f,-39.1365f,-13.6369f,  -76.3635f,-36.7450f,-11.9475f,  -75.1677f,-37.1265f,-10.3898f,  
	-76.5454f,-34.6675f,-8.6720f,  -75.1587f,-34.9220f,-7.2527f,  -76.4026f,-32.4092f,-5.5138f,  -74.8449f,-32.5453f,-4.2658f,  -75.9520f,-29.9926f,-2.5136f,  
	-74.2482f,-30.0189f,-1.4656f,  -75.2176f,-27.4389f,0.2933f,  -73.3963f,-27.3639f,1.1176f,  -74.2280f,-24.7694f,2.8762f,  -72.3215f,-24.6010f,3.4584f,  
	-73.0165f,-22.0044f,5.2099f,  -71.0595f,-21.7496f,5.5376f,  -71.6197f,-19.1627f,7.2758f,  -1.5853f,-43.0390f,-23.5588f,  -2.7846f,-43.9746f,-23.6173f,  
	-3.7428f,-43.2599f,-22.0132f,  -4.9577f,-44.1314f,-22.0687f,  -6.0122f,-43.2061f,-20.6425f,  -7.2381f,-44.0084f,-20.6965f,  -8.3605f,-42.8868f,-19.4780f,  
	-9.5819f,-43.6156f,-19.5365f,  -10.7429f,-42.3179f,-18.5515f,  -11.9525f,-42.9707f,-18.6165f,  -13.1233f,-41.5223f,-17.8861f,  -14.3096f,-42.0991f,-17.9608f,  
	-15.4624f,-40.5301f,-17.5011f,  -16.6160f,-41.0331f,-17.5876f,  -17.7255f,-39.3781f,-17.4092f,  -18.8375f,-39.8104f,-17.5087f,  -19.8813f,-38.1071f,-17.6163f,  
	-20.9443f,-38.4742f,-17.7294f,  -21.9038f,-36.7631f,-18.1216f,  -22.9128f,-37.0722f,-18.2479f,  -23.7741f,-35.3953f,-18.9175f,  -24.7256f,-35.6541f,-19.0557f,  
	-25.4800f,-34.0537f,-19.9893f,  -26.3748f,-34.2725f,-20.1366f,  -27.0191f,-32.7900f,-21.3152f,  -27.8623f,-32.9784f,-21.4686f,  -28.3989f,-31.6529f,-22.8676f,  
	-29.1962f,-31.8183f,-23.0238f,  -29.6326f,-30.6858f,-24.6141f,  -30.3957f,-30.8377f,-24.7674f,  -30.7444f,-29.9295f,-26.5155f,  -31.4860f,-30.0736f,-26.6606f,  
	-31.7639f,-29.4154f,-28.5292f,  -32.5031f,-29.5573f,-28.6585f,  -32.7304f,-29.1681f,-30.6077f,  -33.4847f,-29.3085f,-30.7148f,  -33.6836f,-29.2004f,-32.7025f,  
	-34.4754f,-29.3379f,-32.7803f,  -34.6689f,-29.5153f,-34.7635f,  -35.5179f,-29.6446f,-34.8025f,  -35.7288f,-30.1042f,-36.7381f,  -36.6567f,-30.2168f,-36.7300f,  
	-36.9055f,-30.9475f,-38.5756f,  -37.9309f,-31.0312f,-38.5108f,  -38.2352f,-32.0149f,-40.2259f,  -39.3738f,-32.0544f,-40.0951f,  -39.7466f,-33.2669f,-41.6421f,  
	-41.0114f,-33.2440f,-41.4350f,  -41.4601f,-34.6553f,-42.7799f,  -42.8565f,-34.5510f,-42.4885f,  -43.3827f,-36.1275f,-43.6021f,  -44.9123f,-35.9219f,-43.2181f,  
	-45.5109f,-37.6269f,-44.0766f,  -47.1712f,-37.2995f,-43.5921f,  -47.8307f,-39.0951f,-44.1779f,  -49.6103f,-38.6291f,-43.5883f,  -50.3134f,-40.4775f,-43.8902f,  
	-52.1986f,-39.8570f,-43.1915f,  -52.9228f,-41.7218f,-43.2054f,  -54.8944f,-40.9350f,-42.3959f,  -55.6129f,-42.7823f,-42.1248f,  -57.6497f,-41.8211f,-41.2047f,  
	-58.3328f,-43.6202f,-40.6584f,  -60.4117f,-42.4800f,-39.6295f,  -61.0285f,-44.2045f,-38.8245f,  -63.1250f,-42.8855f,-37.6909f,  -63.6441f,-44.5131f,-36.6499f,  
	-65.7345f,-43.0187f,-35.4169f,  -66.1260f,-44.5316f,-34.1680f,  -68.1875f,-42.8686f,-32.8423f,  -68.4241f,-44.2538f,-31.4184f,  -70.4360f,-42.4313f,-30.0070f,  
	-70.4940f,-43.6797f,-28.4450f,  -72.4368f,-41.7098f,-26.9567f,  -72.2970f,-42.8164f,-25.2960f,  -74.1550f,-40.7114f,-23.7386f,  -73.8036f,-41.6745f,-22.0207f,  
	-75.5624f,-39.4488f,-20.4032f,  -74.9913f,-40.2694f,-18.6705f,  -76.6390f,-37.9381f,-17.0019f,  -75.8462f,-38.6198f,-15.2964f,  -77.3739f,-36.1969f,-13.5848f,  
	-76.3635f,-36.7450f,-11.9475f,  -77.7634f,-34.2467f,-10.2022f,  -76.5454f,-34.6675f,-8.6720f,  -77.8122f,-32.1082f,-6.9010f,  -76.4026f,-32.4092f,-5.5138f,  
	-77.5324f,-29.8040f,-3.7257f,  -75.9520f,-29.9926f,-2.5136f,  -76.9431f,-27.3550f,-0.7156f,  -75.2176f,-27.4389f,0.2933f,  -76.0690f,-24.7829f,2.0934f,  
	-74.2280f,-24.7694f,2.8762f,  -74.9401f,-22.1088f,4.6708f,  -73.0165f,-22.0044f,5.2099f,  -73.5909f,-19.3522f,6.9919f,  -71.6197f,-19.1627f,7.2758f,  
	-72.0591f,-16.5308f,9.0387f,  -2.7846f,-43.9746f,-23.6173f,  -4.0143f,-44.8886f,-23.6620f,  -4.9577f,-44.1314f,-22.0687f,  -6.2050f,-44.9783f,-22.1086f,  
	-7.2381f,-44.0084f,-20.6965f,  -8.4876f,-44.7822f,-20.7380f,  -9.5819f,-43.6156f,-19.5365f,  -10.8259f,-44.3127f,-19.5818f,  -11.9525f,-42.9707f,-18.6165f,  
	-13.1791f,-43.5898f,-18.6691f,  -14.3096f,-42.0991f,-17.9608f,  -15.5083f,-42.6410f,-18.0234f,  -16.6160f,-41.0331f,-17.5876f,  -17.7770f,-41.4997f,-17.6619f,  
	-18.8375f,-39.8104f,-17.5087f,  -19.9519f,-40.2060f,-17.5959f,  -20.9443f,-38.4742f,-17.7294f,  -22.0051f,-38.8052f,-17.8297f,  -22.9128f,-37.0722f,-18.2479f,  
	-23.9146f,-37.3457f,-18.3607f,  -24.7256f,-35.6541f,-19.0557f,  -25.6669f,-35.8800f,-19.1791f,  -26.3748f,-34.2725f,-20.1366f,  -27.2584f,-34.4604f,-20.2682f,  
	-27.8623f,-32.9784f,-21.4686f,  -28.6919f,-33.1360f,-21.6055f,  -29.1962f,-31.8183f,-23.0238f,  -29.9813f,-31.9559f,-23.1612f,  -30.3957f,-30.8377f,-24.7674f,  
	-31.1470f,-30.9620f,-24.9005f,  -31.4860f,-30.0736f,-26.6606f,  -32.2211f,-30.1917f,-26.7823f,  -32.5031f,-29.5573f,-28.6585f,  -33.2384f,-29.6714f,-28.7630f,  
	-33.4847f,-29.3085f,-30.7148f,  -34.2421f,-29.4191f,-30.7950f,  -34.4754f,-29.3379f,-32.7803f,  -35.2745f,-29.4417f,-32.8272f,  -35.5179f,-29.6446f,-34.8025f,  
	-36.3813f,-29.7350f,-34.8080f,  -36.6567f,-30.2168f,-36.7300f,  -37.6041f,-30.2832f,-36.6844f,  -37.9309f,-31.0312f,-38.5108f,  -38.9802f,-31.0598f,-38.4046f,  
	-39.3738f,-32.0544f,-40.0951f,  -40.5404f,-32.0282f,-39.9176f,  -41.0114f,-33.2440f,-41.4350f,  -42.3035f,-33.1446f,-41.1777f,  -42.8565f,-34.5510f,-42.4885f,  
	-44.2789f,-34.3587f,-42.1424f,  -44.9123f,-35.9219f,-43.2181f,  -46.4656f,-35.6157f,-42.7742f,  -47.1712f,-37.2995f,-43.5921f,  -48.8469f,-36.8616f,-43.0449f,  
	-49.6103f,-38.6291f,-43.5883f,  -51.3974f,-38.0423f,-42.9325f,  -52.1986f,-39.8570f,-43.1915f,  -54.0805f,-39.1078f,-42.4242f,  -54.8944f,-40.9350f,-42.3959f,  
	-56.8514f,-40.0133f,-41.5161f,  -57.6497f,-41.8211f,-41.2047f,  -59.6601f,-40.7203f,-40.2130f,  -60.4117f,-42.4800f,-39.6295f,  -62.4517f,-41.1984f,-38.5291f,  
	-63.1250f,-42.8855f,-37.6909f,  -65.1709f,-41.4245f,-36.4861f,  -65.7345f,-43.0187f,-35.4169f,  -67.7633f,-41.3833f,-34.1135f,  -68.1875f,-42.8686f,-32.8423f,  
	-70.1778f,-41.0668f,-31.4470f,  -70.4360f,-42.4313f,-30.0070f,  -72.3672f,-40.4739f,-28.5283f,  -72.4368f,-41.7098f,-26.9567f,  -74.2920f,-39.6086f,-25.4021f,  
	-74.1550f,-40.7114f,-23.7386f,  -75.9183f,-38.4804f,-22.1172f,  -75.5624f,-39.4488f,-20.4032f,  -77.2205f,-37.1030f,-18.7243f,  -76.6390f,-37.9381f,-17.0019f,  
	-78.1816f,-35.4918f,-15.2738f,  -77.3739f,-36.1969f,-13.5848f,  -78.7916f,-33.6667f,-11.8173f,  -77.7634f,-34.2467f,-10.2022f,  -79.0498f,-31.6476f,-8.4038f,  
	-77.8122f,-32.1082f,-6.9010f,  -78.9622f,-29.4564f,-5.0809f,  -77.5324f,-29.8040f,-3.7257f,  -78.5433f,-27.1136f,-1.8917f,  -76.9431f,-27.3550f,-0.7156f,  
	-77.8130f,-24.6413f,1.1235f,  -76.0690f,-24.7829f,2.0934f,  -76.7976f,-22.0609f,3.9294f,  -74.9401f,-22.1088f,4.6708f,  -75.5284f,-19.3927f,6.4956f,  
	-73.5909f,-19.3522f,6.9919f,  -74.0413f,-16.6550f,8.7985f,  -72.0591f,-16.5308f,9.0387f,  -72.3749f,-13.8658f,10.8196f,  -4.0143f,-44.8886f,-23.6620f,  
	-5.2787f,-45.7799f,-23.6901f,  -6.2050f,-44.9783f,-22.1086f,  -7.4787f,-45.7978f,-22.1354f,  -8.4876f,-44.7822f,-20.7380f,  -9.7628f,-45.5254f,-20.7656f,  
	-10.8259f,-44.3127f,-19.5818f,  -12.0903f,-44.9768f,-19.6141f,  -13.1791f,-43.5898f,-18.6691f,  -14.4214f,-44.1741f,-18.7091f,  -15.5083f,-42.6410f,-18.0234f,  
	-16.7178f,-43.1462f,-18.0733f,  -17.7770f,-41.4997f,-17.6619f,  -18.9434f,-41.9290f,-17.7234f,  -19.9519f,-40.2060f,-17.5959f,  -21.0665f,-40.5647f,-17.6699f,  
	-22.0051f,-38.8052f,-17.8297f,  -23.0609f,-39.0995f,-17.9163f,  -23.9146f,-37.3457f,-18.3607f,  -24.9076f,-37.5850f,-18.4589f,  -25.6669f,-35.8800f,-19.1791f,  
	-26.5976f,-36.0737f,-19.2869f,  -27.2584f,-34.4604f,-20.2682f,  -28.1285f,-34.6163f,-20.3835f,  -28.6919f,-33.1360f,-21.6055f,  -29.5083f,-33.2648f,-21.7241f,  
	-29.9813f,-31.9559f,-23.1612f,  -30.7528f,-32.0654f,-23.2790f,  -31.1470f,-30.9620f,-24.9005f,  -31.8891f,-31.0605f,-25.0112f,  -32.2211f,-30.1917f,-26.7823f,  
	-32.9488f,-30.2829f,-26.8804f,  -33.2384f,-29.6714f,-28.7630f,  -33.9732f,-29.7576f,-28.8418f,  -34.2421f,-29.4191f,-30.7950f,  -35.0027f,-29.4989f,-30.8461f,  
	-35.2745f,-29.4417f,-32.8272f,  -36.0841f,-29.5103f,-32.8424f,  -36.3813f,-29.7350f,-34.8080f,  -37.2605f,-29.7838f,-34.7779f,  -37.6041f,-30.2832f,-36.6844f,  
	-38.5725f,-30.3000f,-36.5994f,  -38.9802f,-31.0598f,-38.4046f,  -40.0554f,-31.0289f,-38.2538f,  -40.5404f,-32.0282f,-39.9176f,  -41.7331f,-31.9322f,-39.6922f,  
	-42.3035f,-33.1446f,-41.1777f,  -43.6215f,-32.9639f,-40.8678f,  -44.2789f,-34.3587f,-42.1424f,  -45.7259f,-34.0724f,-41.7383f,  -46.4656f,-35.6157f,-42.7742f,  
	-48.0362f,-35.2053f,-42.2694f,  -48.8469f,-36.8616f,-43.0449f,  -50.5330f,-36.3087f,-42.4328f,  -51.3974f,-38.0423f,-42.9325f,  -53.1850f,-37.3319f,-42.2091f,  
	-54.0805f,-39.1078f,-42.4242f,  -55.9518f,-38.2279f,-41.5873f,  -56.8514f,-40.0133f,-41.5161f,  -58.7864f,-38.9555f,-40.5652f,  -59.6601f,-40.7203f,-40.2130f,  
	-61.6358f,-39.4806f,-39.1500f,  -62.4517f,-41.1984f,-38.5291f,  -64.4450f,-39.7763f,-37.3572f,  -65.1709f,-41.4245f,-36.4861f,  -67.1588f,-39.8234f,-35.2105f,  
	-67.7633f,-41.3833f,-34.1135f,  -69.7237f,-39.6096f,-32.7403f,  -70.1778f,-41.0668f,-31.4470f,  -72.0895f,-39.1300f,-29.9839f,  -72.3672f,-40.4739f,-28.5283f,  
	-74.2121f,-38.3850f,-26.9827f,  -74.2920f,-39.6086f,-25.4021f,  -76.0529f,-37.3809f,-23.7831f,  -75.9183f,-38.4804f,-22.1172f,  -77.5807f,-36.1287f,-20.4340f,  
	-77.2205f,-37.1030f,-18.7243f,  -78.7726f,-34.6415f,-16.9855f,  -78.1816f,-35.4918f,-15.2738f,  -79.6127f,-32.9374f,-13.4895f,  -78.7916f,-33.6667f,-11.8173f,  
	-80.0941f,-31.0351f,-9.9964f,  -79.0498f,-31.6476f,-8.4038f,  -80.2171f,-28.9556f,-6.5563f,  -78.9622f,-29.4564f,-5.0809f,  -79.9907f,-26.7189f,-3.2150f,  
	-78.5433f,-27.1136f,-1.8917f,  -79.4301f,-24.3470f,-0.0169f,  -77.8130f,-24.6413f,1.1235f,  -78.5570f,-21.8615f,2.9984f,  -76.7976f,-22.0609f,3.9294f,  
	-77.3992f,-19.2835f,5.7953f,  -75.5284f,-19.3927f,6.4956f,  -75.9896f,-16.6316f,8.3449f,  -74.0413f,-16.6550f,8.7985f,  -74.3648f,-13.9250f,10.6229f,  
	-72.3749f,-13.8658f,10.8196f,  -72.5650f,-11.1798f,12.6118f,  -5.2787f,-45.7799f,-23.6901f,  -6.5725f,-46.6451f,-23.7048f,  -7.4787f,-45.7978f,-22.1354f,  
	-8.7814f,-46.5880f,-22.1475f,  -9.7628f,-45.5254f,-20.7656f,  -11.0618f,-46.2363f,-20.7794f,  -12.0903f,-44.9768f,-19.6141f,  -13.3738f,-45.6063f,-19.6330f,  
	-14.4214f,-44.1741f,-18.7091f,  -15.6780f,-44.7217f,-18.7357f,  -16.7178f,-43.1462f,-18.0733f,  -17.9361f,-43.6137f,-18.1098f,  -18.9434f,-41.9290f,-17.7234f,  
	-20.1132f,-42.3205f,-17.7715f,  -21.0665f,-40.5647f,-17.6699f,  -22.1791f,-40.8855f,-17.7301f,  -23.0609f,-39.0995f,-17.9163f,  -24.1098f,-39.3582f,-17.9883f,  
	-24.9076f,-37.5850f,-18.4589f,  -25.8912f,-37.7905f,-18.5415f,  -26.5976f,-36.0737f,-19.2869f,  -27.5154f,-36.2339f,-19.3786f,  -28.1285f,-34.6163f,-20.3835f,  
	-28.9849f,-34.7422f,-20.4810f,  -29.5083f,-33.2648f,-21.7241f,  -30.3100f,-33.3648f,-21.8239f,  -30.7528f,-32.0654f,-23.2790f,  -31.5128f,-32.1489f,-23.3753f,  
	-31.8891f,-31.0605f,-25.0112f,  -32.6210f,-31.1326f,-25.0992f,  -32.9488f,-30.2829f,-26.8804f,  -33.6730f,-30.3475f,-26.9539f,  -33.9732f,-29.7576f,-28.8418f,  
	-34.7073f,-29.8152f,-28.8929f,  -35.0027f,-29.4989f,-30.8461f,  -35.7702f,-29.5466f,-30.8670f,  -36.0841f,-29.5103f,-32.8424f,  -36.9060f,-29.5415f,-32.8240f,  
	-37.2605f,-29.7838f,-34.7779f,  -38.1578f,-29.7880f,-34.7105f,  -38.5725f,-30.3000f,-36.5994f,  -39.5644f,-30.2631f,-36.4721f,  -40.0554f,-31.0289f,-38.2538f,  
	-41.1551f,-30.9345f,-38.0573f,  -41.7331f,-31.9322f,-39.6922f,  -42.9512f,-31.7615f,-39.4163f,  -43.6215f,-32.9639f,-40.8678f,  -44.9649f,-32.6961f,-40.5019f,  
	-45.7259f,-34.0724f,-41.7383f,  -47.1921f,-33.6884f,-41.2750f,  -48.0362f,-35.2053f,-42.2694f,  -49.6201f,-34.6860f,-41.7012f,  -50.5330f,-36.3087f,-42.4328f,  
	-52.2233f,-35.6376f,-41.7545f,  -53.1850f,-37.3319f,-42.2091f,  -54.9662f,-36.4954f,-41.4171f,  -55.9518f,-38.2279f,-41.5873f,  -57.8056f,-37.2156f,-40.6798f,  
	-58.7864f,-38.9555f,-40.5652f,  -60.6912f,-37.7615f,-39.5430f,  -61.6358f,-39.4806f,-39.1500f,  -63.5692f,-38.1025f,-38.0154f,  -64.4450f,-39.7763f,-37.3572f,  
	-66.3842f,-38.2156f,-36.1144f,  -67.1588f,-39.8234f,-35.2105f,  -69.0815f,-38.0847f,-33.8647f,  -69.7237f,-39.6096f,-32.7403f,  -71.6083f,-37.7010f,-31.2986f,  
	-72.0895f,-39.1300f,-29.9839f,  -73.9168f,-37.0615f,-28.4535f,  -74.2121f,-38.3850f,-26.9827f,  -75.9640f,-36.1691f,-25.3723f,  -76.0529f,-37.3809f,-23.7831f,  
	-77.7135f,-35.0319f,-22.1018f,  -77.5807f,-36.1287f,-20.4340f,  -79.1369f,-33.6607f,-18.6906f,  -78.7726f,-34.6415f,-16.9855f,  -80.2125f,-32.0714f,-15.1902f,  
	-79.6127f,-32.9374f,-13.4895f,  -80.9272f,-30.2813f,-11.6517f,  -80.0941f,-31.0351f,-9.9964f,  -81.2755f,-28.3104f,-8.1265f,  -80.2171f,-28.9556f,-6.5563f,  
	-81.2608f,-26.1777f,-4.6628f,  -79.9907f,-26.7189f,-3.2150f,  -80.8927f,-23.9051f,-1.3082f,  -79.4301f,-24.3470f,-0.0169f,  -80.1883f,-21.5141f,1.8937f,  
	-78.5570f,-21.8615f,2.9984f,  -79.1712f,-19.0265f,4.9030f,  -77.3992f,-19.2835f,5.7953f,  -77.8707f,-16.4609f,7.6857f,  -75.9896f,-16.6316f,8.3449f,  
	-76.3208f,-13.8378f,10.2118f,  -74.3648f,-13.9250f,10.6229f,  -74.5597f,-11.1740f,12.4585f,  -72.5650f,-11.1798f,12.6118f,  -72.6285f,-8.4856f,14.4082f,  
	-6.5725f,-46.6451f,-23.7048f,  -7.8985f,-47.4823f,-23.7040f,  -8.7814f,-46.5880f,-22.1475f,  -10.1113f,-47.3468f,-22.1453f,  -11.0618f,-46.2363f,-20.7794f,  
	-12.3836f,-46.9131f,-20.7793f,  -13.3738f,-45.6063f,-19.6330f,  -14.6755f,-46.1993f,-19.6380f,  -15.6780f,-44.7217f,-18.7357f,  -16.9473f,-45.2313f,-18.7485f,  
	-17.9361f,-43.6137f,-18.1098f,  -19.1614f,-44.0427f,-18.1325f,  -20.1132f,-42.3205f,-17.7715f,  -21.2842f,-42.6731f,-17.8052f,  -22.1791f,-40.8855f,-17.7301f,  
	-23.2872f,-41.1693f,-17.7755f,  -24.1098f,-39.3582f,-17.9883f,  -25.1512f,-39.5816f,-18.0449f,  -25.8912f,-37.7905f,-18.5415f,  -26.8633f,-37.9611f,-18.6082f,  
	-27.5154f,-36.2339f,-19.3786f,  -28.4200f,-36.3627f,-19.4529f,  -28.9849f,-34.7422f,-20.4810f,  -29.8262f,-34.8381f,-20.5600f,  -30.3100f,-33.3648f,-21.8239f,  
	-31.0987f,-33.4382f,-21.9028f,  -31.5128f,-32.1489f,-23.3753f,  -32.2606f,-32.2060f,-23.4499f,  -32.6210f,-31.1326f,-25.0992f,  -33.3468f,-31.1788f,-25.1638f,  
	-33.6730f,-30.3475f,-26.9539f,  -34.3932f,-30.3851f,-27.0012f,  -34.7073f,-29.8152f,-28.8929f,  -35.4450f,-29.8433f,-28.9155f,  -35.7702f,-29.5466f,-30.8670f,  
	-36.5467f,-29.5605f,-30.8561f,  -36.9060f,-29.5415f,-32.8240f,  -37.7430f,-29.5326f,-32.7702f,  -38.1578f,-29.7880f,-34.7105f,  -39.0761f,-29.7439f,-34.6028f,  
	-39.5644f,-30.2631f,-36.4721f,  -40.5793f,-30.1687f,-36.3012f,  -41.1551f,-30.9345f,-38.0573f,  -42.2797f,-30.7719f,-37.8126f,  -42.9512f,-31.7615f,-39.4163f,  
	-44.1955f,-31.5102f,-39.0863f,  -44.9649f,-32.6961f,-40.5019f,  -46.3292f,-32.3370f,-40.0787f,  -47.1921f,-33.6884f,-41.2750f,  -48.6744f,-33.2018f,-40.7500f,  
	-49.6201f,-34.6860f,-41.7012f,  -51.2118f,-34.0540f,-41.0679f,  -52.2233f,-35.6376f,-41.7545f,  -53.9115f,-34.8453f,-41.0083f,  -54.9662f,-36.4954f,-41.4171f,  
	-56.7346f,-35.5306f,-40.5551f,  -57.8056f,-37.2156f,-40.6798f,  -59.6341f,-36.0704f,-39.7015f,  -60.6912f,-37.7615f,-39.5430f,  -62.5585f,-36.4315f,-38.4495f,  
	-63.5692f,-38.1025f,-38.0154f,  -65.4529f,-36.5873f,-36.8097f,  -66.3842f,-38.2156f,-36.1144f,  -68.2622f,-36.5181f,-34.8010f,  -69.0815f,-38.0847f,-33.8647f,  
	-70.9318f,-36.2114f,-32.4500f,  -71.6083f,-37.7010f,-31.2986f,  -73.4110f,-35.6607f,-29.7893f,  -73.9168f,-37.0615f,-28.4535f,  -75.6529f,-34.8658f,-26.8578f,  
	-75.9640f,-36.1691f,-25.3723f,  -77.6166f,-33.8315f,-23.6992f,  -77.7135f,-35.0319f,-22.1018f,  -79.2683f,-32.5661f,-20.3599f,  -79.1369f,-33.6607f,-18.6906f,  
	-80.5807f,-31.0835f,-16.8903f,  -80.2125f,-32.0714f,-15.1902f,  -81.5349f,-29.3991f,-13.3412f,  -80.9272f,-30.2813f,-11.6517f,  -82.1194f,-27.5316f,-9.7646f,  
	-81.2755f,-28.3104f,-8.1265f,  -82.3316f,-25.4991f,-6.2104f,  -81.2608f,-26.1777f,-4.6628f,  -82.1757f,-23.3230f,-2.7284f,  -80.8927f,-23.9051f,-1.3082f,  
	-81.6636f,-21.0247f,0.6343f,  -80.1883f,-21.5141f,1.8937f,  -80.8140f,-18.6260f,3.8339f,  -79.1712f,-19.0265f,4.9030f,  -79.6523f,-16.1459f,6.8322f,  
	-77.8707f,-16.4609f,7.6857f,  -78.2092f,-13.6059f,9.5936f,  -76.3208f,-13.8378f,10.2118f,  -76.5202f,-11.0230f,12.0900f,  -74.5597f,-11.1740f,12.4585f,  
	-74.6248f,-8.4147f,14.2982f,  -72.6285f,-8.4856f,14.4082f,  -72.5654f,-5.7957f,16.2017f,  -7.8985f,-47.4823f,-23.7040f,  -9.2550f,-48.2892f,-23.6882f,  
	-10.1113f,-47.3468f,-22.1453f,  -11.4676f,-48.0720f,-22.1284f,  -12.3836f,-46.9131f,-20.7793f,  -13.7275f,-47.5536f,-20.7645f,  -14.6755f,-46.1993f,-19.6380f,  
	-15.9939f,-46.7540f,-19.6284f,  -16.9473f,-45.2313f,-18.7485f,  -18.2274f,-45.7017f,-18.7468f,  -19.1614f,-44.0427f,-18.1325f,  -20.3915f,-44.4318f,-18.1404f,  
	-21.2842f,-42.6731f,-17.8052f,  -22.4538f,-42.9874f,-17.8241f,  -23.2872f,-41.1693f,-17.7755f,  -24.3903f,-41.4162f,-17.8054f,  -25.1512f,-39.5816f,-18.0449f,  
	-26.1833f,-39.7684f,-18.0853f,  -26.8633f,-37.9611f,-18.6082f,  -27.8232f,-38.0986f,-18.6575f,  -28.4200f,-36.3627f,-19.4529f,  -29.3098f,-36.4601f,-19.5089f,  
	-29.8262f,-34.8381f,-20.5600f,  -30.6538f,-34.9064f,-20.6188f,  -31.0987f,-33.4382f,-21.9028f,  -31.8736f,-33.4846f,-21.9607f,  -32.2606f,-32.2060f,-23.4499f,  
	-33.0002f,-32.2373f,-23.5019f,  -33.3468f,-31.1788f,-25.1638f,  -34.0658f,-31.1990f,-25.2033f,  -34.3932f,-30.3851f,-27.0012f,  -35.1140f,-30.3951f,-27.0213f,  
	-35.4450f,-29.8433f,-28.9155f,  -36.1886f,-29.8404f,-28.9079f,  -36.5467f,-29.5605f,-30.8561f,  -37.3353f,-29.5381f,-30.8116f,  -37.7430f,-29.5326f,-32.7702f,  
	-38.5986f,-29.4802f,-32.6781f,  -39.0761f,-29.7439f,-34.6028f,  -40.0157f,-29.6478f,-34.4537f,  -40.5793f,-30.1687f,-36.3012f,  -41.6184f,-30.0121f,-36.0841f,  
	-42.2797f,-30.7719f,-37.8126f,  -43.4309f,-30.5351f,-37.5157f,  -44.1955f,-31.5102f,-39.0863f,  -45.4620f,-31.1742f,-38.7011f,  -46.3292f,-32.3370f,-40.0787f,  
	-47.7120f,-31.8815f,-39.5955f,  -48.6744f,-33.2018f,-40.7500f,  -50.1679f,-32.6082f,-40.1613f,  -51.2118f,-34.0540f,-41.0679f,  -52.8057f,-33.3061f,-40.3681f,  
	-53.9115f,-34.8453f,-41.0083f,  -55.5915f,-33.9292f,-40.1930f,  -56.7346f,-35.5306f,-40.5551f,  -58.4826f,-34.4365f,-39.6226f,  -59.6341f,-36.0704f,-39.7015f,  
	-61.4301f,-34.7920f,-38.6518f,  -62.5585f,-36.4315f,-38.4495f,  -64.3809f,-34.9664f,-37.2846f,  -65.4529f,-36.5873f,-36.8097f,  -67.2799f,-34.9366f,-35.5332f,  
	-68.2622f,-36.5181f,-34.8010f,  -70.0718f,-34.6867f,-33.4183f,  -70.9318f,-36.2114f,-32.4500f,  -72.7035f,-34.2067f,-30.9671f,  -73.4110f,-35.6607f,-29.7893f,  
	-75.1252f,-33.4931f,-28.2140f,  -75.6529f,-34.8658f,-26.8578f,  -77.2916f,-32.5478f,-25.1988f,  -77.6166f,-33.8315f,-23.6992f,  -79.1646f,-31.3765f,-21.9650f,  
	-79.2683f,-32.5661f,-20.3599f,  -80.7111f,-29.9905f,-18.5608f,  -80.5807f,-31.0835f,-16.8903f,  -81.9068f,-28.4035f,-15.0361f,  -81.5349f,-29.3991f,-13.3412f,  
	-82.7342f,-26.6326f,-11.4427f,  -82.1194f,-27.5316f,-9.7646f,  -83.1849f,-24.6948f,-7.8312f,  -82.3316f,-25.4991f,-6.2104f,  -83.2573f,-22.6108f,-4.2533f,  
	-82.1757f,-23.3230f,-2.7284f,  -82.9577f,-20.4015f,-0.7582f,  -81.6636f,-21.0247f,0.6343f,  -82.2996f,-18.0890f,2.6065f,  -80.8140f,-18.6260f,3.8339f,  
	-81.3041f,-15.6921f,5.7989f,  -79.6523f,-16.1459f,6.8322f,  -79.9978f,-13.2331f,8.7787f,  -78.2092f,-13.6059f,9.5936f,  -78.4130f,-10.7296f,11.5128f,  
	-76.5202f,-11.0230f,12.0900f,  -76.5869f,-8.1998f,13.9724f,  -74.6248f,-8.4147f,14.2982f,  -74.5602f,-5.6599f,16.1350f,  -72.5654f,-5.7957f,16.2017f,  
	-72.3765f,-3.1230f,17.9850f,  -9.2550f,-48.2892f,-23.6882f,  -10.6416f,-49.0632f,-23.6571f,  -11.4676f,-48.0720f,-22.1284f,  -12.8500f,-48.7614f,-22.0962f,  
	-13.7275f,-47.5536f,-20.7645f,  -15.0921f,-48.1559f,-20.7347f,  -15.9939f,-46.7540f,-19.6284f,  -17.3272f,-47.2692f,-19.6040f,  -18.2274f,-45.7017f,-18.7468f,  
	-19.5165f,-46.1315f,-18.7300f,  -20.3915f,-44.4318f,-18.1404f,  -21.6240f,-44.7812f,-18.1332f,  -22.4538f,-42.9874f,-17.8241f,  -23.6216f,-43.2630f,-17.8272f,  
	-24.3903f,-41.4162f,-17.8054f,  -25.4869f,-41.6248f,-17.8189f,  -26.1833f,-39.7684f,-18.0853f,  -27.2049f,-39.9202f,-18.1084f,  -27.8232f,-38.0986f,-18.6575f,  
	-28.7695f,-38.2029f,-18.6888f,  -29.3098f,-36.4601f,-19.5089f,  -30.1857f,-36.5283f,-19.5453f,  -30.6538f,-34.9064f,-20.6188f,  -31.4668f,-34.9466f,-20.6572f,  
	-31.8736f,-33.4846f,-21.9607f,  -32.6391f,-33.5047f,-21.9968f,  -33.0002f,-32.2373f,-23.5019f,  -33.7307f,-32.2429f,-23.5299f,  -34.0658f,-31.1990f,-25.2033f,  
	-34.7829f,-31.1929f,-25.2168f,  -35.1140f,-30.3951f,-27.0213f,  -35.8379f,-30.3764f,-27.0127f,  -36.1886f,-29.8404f,-28.9079f,  -36.9417f,-29.8045f,-28.8684f,  
	-37.3353f,-29.5381f,-30.8116f,  -38.1402f,-29.4766f,-30.7308f,  -38.5986f,-29.4802f,-32.6781f,  -39.4738f,-29.3808f,-32.5466f,  -40.0157f,-29.6478f,-34.4537f,  
	-40.9787f,-29.4951f,-34.2605f,  -41.6184f,-30.0121f,-36.0841f,  -42.6842f,-29.7875f,-35.8170f,  -43.4309f,-30.5351f,-37.5157f,  -44.6056f,-30.2198f,-37.1656f,  
	-45.4620f,-31.1742f,-38.7011f,  -46.7494f,-30.7478f,-38.2575f,  -47.7120f,-31.8815f,-39.5955f,  -49.1094f,-31.3249f,-39.0501f,  -50.1679f,-32.6082f,-40.1613f,  
	-51.6677f,-31.9039f,-39.5072f,  -52.8057f,-33.3061f,-40.3681f,  -54.3961f,-32.4388f,-39.5999f,  -55.5915f,-33.9292f,-40.1930f,  -57.2564f,-32.8875f,-39.3078f,  
	-58.4826f,-34.4365f,-39.6226f,  -60.2034f,-33.2121f,-38.6192f,  -61.4301f,-34.7920f,-38.6518f,  -63.1863f,-33.3807f,-37.5309f,  -64.3809f,-34.9664f,-37.2846f,  
	-66.1513f,-33.3674f,-36.0488f,  -67.2799f,-34.9366f,-35.5332f,  -69.0428f,-33.1529f,-34.1869f,  -70.0718f,-34.6867f,-33.4183f,  -71.8064f,-32.7243f,-31.9669f,  
	-72.7035f,-34.2067f,-30.9671f,  -74.3896f,-32.0747f,-29.4178f,  -75.1252f,-33.4931f,-28.2140f,  -76.7442f,-31.2030f,-26.5750f,  -77.2916f,-32.5478f,-25.1988f,  
	-78.8274f,-30.1122f,-23.4783f,  -79.1646f,-31.3765f,-21.9650f,  -80.6015f,-28.8112f,-20.1730f,  -80.7111f,-29.9905f,-18.5608f,  -82.0366f,-27.3116f,-16.7073f,  
	-81.9068f,-28.4035f,-15.0361f,  -83.1096f,-25.6289f,-13.1320f,  -82.7342f,-26.6326f,-11.4427f,  -83.8062f,-23.7786f,-9.4976f,  -83.1849f,-24.6948f,-7.8312f,  
	-84.1189f,-21.7807f,-5.8566f,  -83.2573f,-22.6108f,-4.2533f,  -84.0483f,-19.6553f,-2.2602f,  -82.9577f,-20.4015f,-0.7582f,  -83.6026f,-17.4245f,1.2416f,  
	-82.2996f,-18.0890f,2.6065f,  -82.7978f,-15.1070f,4.6036f,  -81.3041f,-15.6921f,5.7989f,  -81.6559f,-12.7257f,7.7811f,  -79.9978f,-13.2331f,8.7787f,  
	-80.2058f,-10.2986f,10.7367f,  -78.4130f,-10.7296f,11.5128f,  -78.4812f,-7.8447f,13.4363f,  -76.5869f,-8.1998f,13.9724f,  -76.5208f,-5.3811f,15.8518f,  
	-74.5602f,-5.6599f,16.1350f,  -74.3668f,-2.9219f,17.9619f,  -72.3765f,-3.1230f,17.9850f,  -72.0639f,-0.4800f,19.7513f,  -10.6416f,-49.0632f,-23.6571f,  
	-12.0583f,-49.8022f,-23.6101f,  -12.8500f,-48.7614f,-22.0962f,  -14.2575f,-49.4129f,-22.0483f,  -15.0921f,-48.1559f,-20.7347f,  -16.4762f,-48.7183f,-20.6893f,  
	-17.3272f,-47.2692f,-19.6040f,  -18.6739f,-47.7430f,-19.5637f,  -19.5165f,-46.1315f,-18.7300f,  -20.8119f,-46.5202f,-18.6976f,  -21.6240f,-44.7812f,-18.1332f,  
	-22.8582f,-45.0904f,-18.1099f,  -23.6216f,-43.2630f,-17.8272f,  -24.7862f,-43.4986f,-17.8135f,  -25.4869f,-41.6248f,-17.8189f,  -26.5754f,-41.7963f,-17.8150f,  
	-27.2049f,-39.9202f,-18.1084f,  -28.2145f,-40.0368f,-18.1135f,  -28.7695f,-38.2029f,-18.6888f,  -29.7025f,-38.2762f,-18.7007f,  -30.1857f,-36.5283f,-19.5453f,  
	-31.0471f,-36.5670f,-19.5616f,  -31.4668f,-34.9466f,-20.6572f,  -32.2696f,-34.9595f,-20.6743f,  -32.6391f,-33.5047f,-21.9968f,  -33.3938f,-33.4990f,-22.0097f,  
	-33.7307f,-32.2429f,-23.5299f,  -34.4574f,-32.2228f,-23.5330f,  -34.7829f,-31.1929f,-25.2168f,  -35.5007f,-31.1595f,-25.2029f,  -35.8379f,-30.3764f,-27.0127f,  
	-36.5689f,-30.3272f,-26.9737f,  -36.9417f,-29.8045f,-28.8684f,  -37.7088f,-29.7329f,-28.7942f,  -38.1402f,-29.4766f,-30.7308f,  -38.9629f,-29.3722f,-30.6124f,  
	-39.4738f,-29.3808f,-32.5466f,  -40.3713f,-29.2298f,-32.3730f,  -40.9787f,-29.4951f,-34.2605f,  -41.9681f,-29.2801f,-34.0193f,  -42.6842f,-29.7875f,-35.8170f,  
	-43.7744f,-29.4902f,-35.4985f,  -44.6056f,-30.2198f,-37.1656f,  -45.8030f,-29.8200f,-36.7589f,  -46.7494f,-30.7478f,-38.2575f,  -48.0544f,-30.2260f,-37.7533f,  
	-49.1094f,-31.3249f,-39.0501f,  -50.5171f,-30.6628f,-38.4406f,  -51.6677f,-31.9039f,-39.5072f,  -53.1687f,-31.0850f,-38.7859f,  -54.3961f,-32.4388f,-39.5999f,  
	-55.9764f,-31.4499f,-38.7625f,  -57.2564f,-32.8875f,-39.3078f,  -58.8994f,-31.7187f,-38.3520f,  -60.2034f,-33.2121f,-38.6192f,  -61.8898f,-31.8572f,-37.5445f,  
	-63.1863f,-33.3807f,-37.5309f,  -64.8957f,-31.8372f,-36.3389f,  -66.1513f,-33.3674f,-36.0488f,  -67.8624f,-31.6363f,-34.7427f,  -69.0428f,-33.1529f,-34.1869f,  
	-70.7349f,-31.2388f,-32.7715f,  -71.8064f,-32.7243f,-31.9669f,  -73.4590f,-30.6347f,-30.4486f,  -74.3896f,-32.0747f,-29.4178f,  -75.9837f,-29.8200f,-27.8042f,  
	-76.7442f,-31.2030f,-26.5750f,  -78.2625f,-28.7949f,-24.8739f,  -78.8274f,-30.1122f,-23.4783f,  -80.2536f,-27.5658f,-21.6995f,  -80.6015f,-28.8112f,-20.1730f,  
	-81.9219f,-26.1421f,-18.3263f,  -82.0366f,-27.3116f,-16.7073f,  -83.2390f,-24.5375f,-14.8036f,  -83.1096f,-25.6289f,-13.1320f,  -84.1847f,-22.7662f,-11.1810f,  
	-83.8062f,-23.7786f,-9.4976f,  -84.7457f,-20.8468f,-7.5110f,  -84.1189f,-21.7807f,-5.8566f,  -84.9168f,-18.7989f,-3.8458f,  -84.0483f,-19.6553f,-2.2602f,  
	-84.7006f,-16.6440f,-0.2373f,  -83.6026f,-17.4245f,1.2416f,  -84.1077f,-14.4009f,3.2666f,  -82.7978f,-15.1070f,4.6036f,  -83.1553f,-12.0926f,6.6179f,  
	-81.6559f,-12.7257f,7.7811f,  -81.8678f,-9.7376f,9.7750f,  -80.2058f,-10.2986f,10.7367f,  -80.2755f,-7.3554f,12.6991f,  -78.4812f,-7.8447f,13.4363f,  
	-78.4138f,-4.9643f,15.3568f,  -76.5208f,-5.3811f,15.8518f,  -76.3230f,-2.5790f,17.7215f,  -74.3668f,-2.9219f,17.9619f,  -74.0465f,-0.2137f,19.7717f,  
	-72.0639f,-0.4800f,19.7513f,  -71.6305f,2.1210f,21.4934f,  -12.0583f,-49.8022f,-23.6101f,  -13.5043f,-50.5035f,-23.5466f,  -14.2575f,-49.4129f,-22.0483f,  
	-15.6889f,-50.0243f,-21.9841f,  -16.4762f,-48.7183f,-20.6893f,  -17.8783f,-49.2388f,-20.6275f,  -18.6739f,-47.7430f,-19.5637f,  -20.0315f,-48.1746f,-19.5075f,  
	-20.8119f,-46.5202f,-18.6976f,  -22.1132f,-46.8672f,-18.6487f,  -22.8582f,-45.0904f,-18.1099f,  -24.0930f,-45.3577f,-18.0694f,  -24.7862f,-43.4986f,-17.8135f,  
	-25.9457f,-43.6949f,-17.7823f,  -26.5754f,-41.7963f,-17.8150f,  -27.6545f,-41.9304f,-17.7929f,  -28.2145f,-40.0368f,-18.1135f,  -29.2124f,-40.1202f,-18.0993f,  
	-29.7025f,-38.2762f,-18.7007f,  -30.6217f,-38.3180f,-18.6926f,  -31.0471f,-36.5670f,-19.5616f,  -31.8981f,-36.5768f,-19.5569f,  -32.2696f,-34.9595f,-20.6743f,  
	-33.0606f,-34.9456f,-20.6688f,  -33.3938f,-33.4990f,-22.0097f,  -34.1430f,-33.4672f,-21.9985f,  -34.4574f,-32.2228f,-23.5330f,  -35.1826f,-32.1761f,-23.5096f,  
	-35.5007f,-31.1595f,-25.2029f,  -36.2233f,-31.0972f,-25.1599f,  -36.5689f,-30.3272f,-26.9737f,  -37.3116f,-30.2450f,-26.9016f,  -37.7088f,-29.7329f,-28.7942f,  
	-38.4918f,-29.6222f,-28.6843f,  -38.9629f,-29.3722f,-30.6124f,  -39.8069f,-29.2207f,-30.4537f,  -40.3713f,-29.2298f,-32.3730f,  -41.2950f,-29.0217f,-32.1532f,  
	-41.9681f,-29.2801f,-34.0193f,  -42.9828f,-28.9978f,-33.7285f,  -43.7744f,-29.4902f,-35.4985f,  -44.8891f,-29.1141f,-35.1253f,  -45.8030f,-29.8200f,-36.7589f,  
	-47.0210f,-29.3304f,-36.2932f,  -48.0544f,-30.2260f,-37.7533f,  -49.3734f,-29.6040f,-37.1863f,  -50.5171f,-30.6628f,-38.4406f,  -51.9303f,-29.8908f,-37.7648f,  
	-53.1687f,-31.0850f,-38.7859f,  -54.6647f,-30.1485f,-37.9959f,  -55.9764f,-31.4499f,-38.7625f,  -57.5403f,-30.3374f,-37.8548f,  -58.8994f,-31.7187f,-38.3520f,  
	-60.5134f,-30.4221f,-37.3250f,  -61.8898f,-31.8572f,-37.5445f,  -63.5347f,-30.3722f,-36.3986f,  -64.8957f,-31.8372f,-36.3389f,  -66.5510f,-30.1629f,-35.0762f,  
	-67.8624f,-31.6363f,-34.7427f,  -69.5075f,-29.7757f,-33.3672f,  -70.7349f,-31.2388f,-32.7715f,  -72.3492f,-29.1979f,-31.2888f,  -73.4590f,-30.6347f,-30.4486f,  
	-75.0231f,-28.4224f,-28.8654f,  -75.9837f,-29.8200f,-27.8042f,  -77.4797f,-27.4471f,-26.1279f,  -78.2625f,-28.7949f,-24.8739f,  -79.6735f,-26.2755f,-23.1139f,  
	-80.2536f,-27.5658f,-21.6995f,  -81.5649f,-24.9150f,-19.8654f,  -81.9219f,-26.1421f,-18.3263f,  -83.1202f,-23.3772f,-16.4289f,  -83.2390f,-24.5375f,-14.8036f,  
	-84.3139f,-21.6748f,-12.8526f,  -84.1847f,-22.7662f,-11.1810f,  -85.1270f,-19.8251f,-9.1882f,  -84.7457f,-20.8468f,-7.5110f,  -85.5484f,-17.8468f,-5.4880f,  
	-84.9168f,-18.7989f,-3.8458f,  -85.5749f,-15.7609f,-1.8051f,  -84.7006f,-16.6440f,-0.2373f,  -85.2116f,-13.5857f,1.8109f,  -84.1077f,-14.4009f,3.2666f,  
	-84.4703f,-11.3446f,5.3089f,  -83.1553f,-12.0926f,6.6179f,  -83.3707f,-9.0560f,8.6441f,  -81.8678f,-9.7376f,9.7750f,  -81.9388f,-6.7403f,11.7733f,  
	-80.2755f,-7.3554f,12.6991f,  -80.2067f,-4.4164f,14.6586f,  -78.4138f,-4.9643f,15.3568f,  -78.2117f,-2.1001f,17.2679f,  -76.3230f,-2.5790f,17.7215f,  
	-75.9952f,0.1936f,19.5743f,  -74.0465f,-0.2137f,19.7717f,  -73.6023f,2.4527f,21.5576f,  -71.6305f,2.1210f,21.4934f,  -71.0800f,4.6684f,23.2051f,  
	-13.5043f,-50.5035f,-23.5466f,  -14.9789f,-51.1648f,-23.4664f,  -15.6889f,-50.0243f,-21.9841f,  -17.1432f,-50.5934f,-21.9030f,  -17.8783f,-49.2388f,-20.6275f,  
	-19.2960f,-49.7161f,-20.5492f,  -20.0315f,-48.1746f,-19.5075f,  -21.3992f,-48.5628f,-19.4343f,  -22.1132f,-46.8672f,-18.6487f,  -23.4194f,-47.1703f,-18.5820f,  
	-24.0930f,-45.3577f,-18.0694f,  -25.3265f,-45.5834f,-18.0109f,  -25.9457f,-43.6949f,-17.7823f,  -27.0990f,-43.8513f,-17.7326f,  -27.6545f,-41.9304f,-17.7929f,  
	-28.7238f,-42.0287f,-17.7516f,  -29.2124f,-40.1202f,-18.0993f,  -30.1979f,-40.1697f,-18.0652f,  -30.6217f,-38.3180f,-18.6926f,  -31.5310f,-38.3288f,-18.6637f,  
	-31.8981f,-36.5768f,-19.5569f,  -32.7370f,-36.5582f,-19.5300f,  -33.0606f,-34.9456f,-20.6688f,  -33.8451f,-34.9049f,-20.6397f,  -34.1430f,-33.4672f,-21.9985f,  
	-34.8891f,-33.4089f,-21.9616f,  -35.1826f,-32.1761f,-23.5096f,  -35.9108f,-32.1014f,-23.4582f,  -36.2233f,-31.0972f,-25.1599f,  -36.9555f,-31.0040f,-25.0852f,  
	-37.3116f,-30.2450f,-26.9016f,  -38.0685f,-30.1265f,-26.7952f,  -38.4918f,-29.6222f,-28.6843f,  -39.2949f,-29.4680f,-28.5357f,  -39.8069f,-29.2207f,-30.4537f,  
	-40.6763f,-29.0166f,-30.2507f,  -41.2950f,-29.0217f,-32.1532f,  -42.2444f,-28.7512f,-31.8857f,  -42.9828f,-28.9978f,-33.7285f,  -44.0234f,-28.6421f,-33.3847f,  
	-44.8891f,-29.1141f,-35.1253f,  -46.0269f,-28.6534f,-34.6946f,  -47.0210f,-29.3304f,-36.2932f,  -48.2564f,-28.7456f,-35.7659f,  -49.3734f,-29.6040f,-37.1863f,  
	-50.7022f,-28.8768f,-36.5542f,  -51.9303f,-29.8908f,-37.7648f,  -53.3435f,-29.0055f,-37.0212f,  -54.6647f,-30.1485f,-37.9959f,  -56.1494f,-29.0919f,-37.1362f,  
	-57.5403f,-30.3374f,-37.8548f,  -59.0808f,-29.0999f,-36.8762f,  -60.5134f,-30.4221f,-37.3250f,  -62.0915f,-28.9975f,-36.2268f,  -63.5347f,-30.3722f,-36.3986f,  
	-65.1309f,-28.7579f,-35.1817f,  -66.5510f,-30.1629f,-35.0762f,  -68.1451f,-28.3600f,-33.7438f,  -69.5075f,-29.7757f,-33.3672f,  -71.0791f,-27.7887f,-31.9239f,  
	-72.3492f,-29.1979f,-31.2888f,  -73.8787f,-27.0343f,-29.7404f,  -75.0231f,-28.4224f,-28.8654f,  -76.4924f,-26.0918f,-27.2190f,  -77.4797f,-27.4471f,-26.1279f,  
	-78.8711f,-24.9625f,-24.3923f,  -79.6735f,-26.2755f,-23.1139f,  -80.9715f,-23.6512f,-21.2982f,  -81.5649f,-24.9150f,-19.8654f,  -82.7553f,-22.1678f,-17.9802f,  
	-83.1202f,-23.3772f,-16.4289f,  -84.1918f,-20.5231f,-14.4838f,  -84.3139f,-21.6748f,-12.8526f,  -85.2563f,-18.7331f,-10.8594f,  -85.1270f,-19.8251f,-9.1882f,  
	-85.9323f,-16.8154f,-7.1586f,  -85.5484f,-17.8468f,-5.4880f,  -86.2105f,-14.7902f,-3.4348f,  -85.5749f,-15.7609f,-1.8051f,  -86.0904f,-12.6753f,0.2614f,  
	-85.2116f,-13.5857f,1.8109f,  -85.5782f,-10.4943f,3.8766f,  -84.4703f,-11.3446f,5.3089f,  -84.6887f,-8.2655f,7.3633f,  -83.3707f,-9.0560f,8.6441f,  
	-83.4429f,-6.0099f,10.6750f,  -81.9388f,-6.7403f,11.7733f,  -81.8689f,-3.7470f,13.7691f,  -80.2067f,-4.4164f,14.6586f,  -80.0005f,-1.4933f,16.6090f,  
	-78.2117f,-2.1001f,17.2679f,  -77.8766f,0.7349f,19.1623f,  -75.9952f,0.1936f,19.5743f,  -75.5403f,2.9246f,21.4034f,  -73.6023f,2.4527f,21.5576f,  
	-73.0377f,5.0659f,23.3133f,  -71.0800f,4.6684f,23.2051f,  -70.4172f,7.1509f,24.8797f,  -14.9789f,-51.1648f,-23.4664f,  -16.4813f,-51.7835f,-23.3684f,  
	-17.1432f,-50.5934f,-21.9030f,  -18.6180f,-51.1182f,-21.8048f,  -19.2960f,-49.7161f,-20.5492f,  -20.7286f,-50.1483f,-20.4533f,  -21.3992f,-48.5628f,-19.4343f,  
	-22.7767f,-48.9054f,-19.3426f,  -23.4194f,-47.1703f,-18.5820f,  -24.7284f,-47.4295f,-18.4969f,  -25.3265f,-45.5834f,-18.0109f,  -26.5574f,-45.7667f,-17.9335f,  
	-27.0990f,-43.8513f,-17.7326f,  -28.2451f,-43.9691f,-17.6633f,  -28.7238f,-42.0287f,-17.7516f,  -29.7828f,-42.0904f,-17.6901f,  -30.1979f,-40.1697f,-18.0652f,  
	-31.1745f,-40.1856f,-18.0101f,  -31.5310f,-38.3288f,-18.6637f,  -32.4284f,-38.3092f,-18.6128f,  -32.7370f,-36.5582f,-19.5300f,  -33.5688f,-36.5112f,-19.4797f,  
	-33.8451f,-34.9049f,-20.6397f,  -34.6253f,-34.8368f,-20.5855f,  -34.8891f,-33.4089f,-21.9616f,  -35.6367f,-33.3227f,-21.8975f,  -35.9108f,-32.1014f,-23.4582f,  
	-36.6466f,-31.9969f,-23.3762f,  -36.9555f,-31.0040f,-25.0852f,  -37.7003f,-30.8764f,-24.9774f,  -38.0685f,-30.1265f,-26.7952f,  -38.8442f,-29.9674f,-26.6517f,  
	-39.2949f,-29.4680f,-28.5357f,  -40.1227f,-29.2650f,-28.3445f,  -40.6763f,-29.0166f,-30.2507f,  -41.5716f,-28.7545f,-30.0017f,  -42.2444f,-28.7512f,-31.8857f,  
	-43.2205f,-28.4120f,-31.5669f,  -44.0234f,-28.6421f,-33.3847f,  -45.0890f,-28.2066f,-32.9850f,  -46.0269f,-28.6534f,-34.6946f,  -47.1850f,-28.1024f,-34.2037f,  
	-48.2564f,-28.7456f,-35.7659f,  -49.5053f,-28.0603f,-35.1747f,  -50.7022f,-28.8768f,-36.5542f,  -52.0355f,-28.0403f,-35.8551f,  -53.3435f,-29.0055f,-37.0212f,  
	-54.7506f,-28.0035f,-36.2084f,  -56.1494f,-29.0919f,-37.1362f,  -57.6163f,-27.9133f,-36.2059f,  -59.0808f,-29.0999f,-36.8762f,  -60.5908f,-27.7367f,-35.8265f,  
	-62.0915f,-28.9975f,-36.2268f,  -63.6264f,-27.4452f,-35.0574f,  -65.1309f,-28.7579f,-35.1817f,  -66.6710f,-27.0160f,-33.8949f,  -68.1451f,-28.3600f,-33.7438f,  
	-69.6705f,-26.4314f,-32.3431f,  -71.0791f,-27.7887f,-31.9239f,  -72.5704f,-25.6793f,-30.4143f,  -73.8787f,-27.0343f,-29.7404f,  -75.3173f,-24.7523f,-28.1284f,  
	-76.4924f,-26.0918f,-27.2190f,  -77.8602f,-23.6491f,-25.5126f,  -78.8711f,-24.9625f,-24.3923f,  -80.1520f,-22.3725f,-22.6003f,  -80.9715f,-23.6512f,-21.2982f,  
	-82.1505f,-20.9300f,-19.4308f,  -82.7553f,-22.1678f,-17.9802f,  -83.8203f,-19.3309f,-16.0467f,  -84.1918f,-20.5231f,-14.4838f,  -85.1315f,-17.5896f,-12.4961f,  
	-85.2563f,-18.7331f,-10.8594f,  -86.0619f,-15.7223f,-8.8290f,  -85.9323f,-16.8154f,-7.1586f,  -86.5965f,-13.7485f,-5.0985f,  -86.2105f,-14.7902f,-3.4348f,  
	-86.7291f,-11.6855f,-1.3555f,  -86.0904f,-12.6753f,0.2614f,  -86.4603f,-9.5564f,2.3454f,  -85.5782f,-10.4943f,3.8766f,  -85.7992f,-7.3798f,5.9547f,  
	-84.6887f,-8.2655f,7.3633f,  -84.7619f,-5.1767f,9.4228f,  -83.4429f,-6.0099f,10.6750f,  -83.3719f,-2.9675f,12.7035f,  -81.8689f,-3.7470f,13.7691f,  
	-81.6588f,-0.7692f,15.7561f,  -80.0005f,-1.4933f,16.6090f,  -79.6584f,1.4008f,18.5430f,  -77.8766f,0.7349f,19.1623f,  -77.4112f,3.5284f,21.0333f,  
	-75.5403f,2.9246f,21.4034f,  -74.9617f,5.6027f,23.2026f,  -73.0377f,5.0659f,23.3133f,  -72.3573f,7.6144f,25.0324f,  -70.4172f,7.1509f,24.8797f,  
	-69.6475f,9.5584f,26.5117f,  -16.4813f,-51.7835f,-23.3684f,  -18.0093f,-52.3570f,-23.2528f,  -18.6180f,-51.1182f,-21.8048f,  -20.1128f,-51.5967f,-21.6883f,  
	-20.7286f,-50.1483f,-20.4533f,  -22.1761f,-50.5333f,-20.3382f,  -22.7767f,-48.9054f,-19.3426f,  -24.1616f,-49.2017f,-19.2319f,  -24.7284f,-47.4295f,-18.4969f,  
	-26.0391f,-47.6436f,-18.3922f,  -26.5574f,-45.7667f,-17.9335f,  -27.7845f,-45.9082f,-17.8362f,  -28.2451f,-43.9691f,-17.6633f,  -29.3837f,-44.0472f,-17.5736f,  
	-29.7828f,-42.0904f,-17.6901f,  -30.8348f,-42.1155f,-17.6072f,  -31.1745f,-40.1856f,-18.0101f,  -32.1402f,-40.1684f,-17.9329f,  -32.4284f,-38.3092f,-18.6128f,  
	-33.3188f,-38.2588f,-18.5386f,  -33.5688f,-36.5112f,-19.4797f,  -34.3957f,-36.4353f,-19.4048f,  -34.6253f,-34.8368f,-20.5855f,  -35.4058f,-34.7400f,-20.5047f,  
	-35.6367f,-33.3227f,-21.8975f,  -36.3902f,-33.2068f,-21.8037f,  -36.6466f,-31.9969f,-23.3762f,  -37.3935f,-31.8592f,-23.2622f,  -37.7003f,-30.8764f,-24.9774f,  
	-38.4625f,-30.7103f,-24.8339f,  -38.8442f,-29.9674f,-26.6517f,  -39.6435f,-29.7626f,-26.4670f,  -40.1227f,-29.2650f,-28.3445f,  -40.9761f,-29.0076f,-28.1087f,  
	-41.5716f,-28.7545f,-30.0017f,  -42.4943f,-28.4279f,-29.7029f,  -43.2205f,-28.4120f,-31.5669f,  -44.2234f,-27.9976f,-31.1935f,  -45.0890f,-28.2066f,-32.9850f,  
	-46.1776f,-27.6853f,-32.5264f,  -47.1850f,-28.1024f,-34.2037f,  -48.3602f,-27.4552f,-33.6501f,  -49.5053f,-28.0603f,-35.1747f,  -50.7632f,-27.2696f,-34.5175f,  
	-52.0355f,-28.0403f,-35.8551f,  -53.3677f,-27.0908f,-35.0874f,  -54.7506f,-28.0035f,-36.2084f,  -56.1452f,-26.8824f,-35.3254f,  -57.6163f,-27.9133f,-36.2059f,  
	-59.0582f,-26.6112f,-35.2045f,  -60.5908f,-27.7367f,-35.8265f,  -62.0632f,-26.2474f,-34.7054f,  -63.6264f,-27.4452f,-35.0574f,  -65.1105f,-25.7666f,-33.8178f,  
	-66.6710f,-27.0160f,-33.8949f,  -68.1476f,-25.1491f,-32.5392f,  -69.6705f,-26.4314f,-32.3431f,  -71.1203f,-24.3806f,-30.8756f,  -72.5704f,-25.6793f,-30.4143f,  
	-73.9746f,-23.4515f,-28.8406f,  -75.3173f,-24.7523f,-28.1284f,  -76.6579f,-22.3579f,-26.4557f,  -77.8602f,-23.6491f,-25.5126f,  -79.1203f,-21.1006f,-23.7492f,  
	-80.1520f,-22.3725f,-22.6003f,  -81.3162f,-19.6850f,-20.7559f,  -82.1505f,-20.9300f,-19.4308f,  -83.2058f,-18.1185f,-17.5146f,  -83.8203f,-19.3309f,-16.0467f,  
	-84.7546f,-16.4139f,-14.0702f,  -85.1315f,-17.5896f,-12.4961f,  -85.9349f,-14.5862f,-10.4708f,  -86.0619f,-15.7223f,-8.8290f,  -86.7264f,-12.6537f,-6.7678f,  
	-86.5965f,-13.7485f,-5.0985f,  -87.1170f,-10.6330f,-3.0120f,  -86.7291f,-11.6855f,-1.3555f,  -87.1014f,-8.5470f,0.7417f,  -86.4603f,-9.5564f,2.3454f,  
	-86.6833f,-6.4138f,4.4423f,  -85.7992f,-7.3798f,5.9547f,  -85.8734f,-4.2550f,8.0382f,  -84.7619f,-5.1767f,9.4228f,  -84.6900f,-2.0912f,11.4801f,  
	-83.3719f,-2.9675f,12.7035f,  -83.1583f,0.0600f,14.7236f,  -81.6588f,-0.7692f,15.7561f,  -81.3103f,2.1800f,17.7268f,  -79.6584f,1.4008f,18.5430f,  
	-79.1832f,4.2539f,20.4538f,  -77.4112f,3.5284f,21.0333f,  -76.8191f,6.2696f,22.8749f,  -74.9617f,5.6027f,23.2026f,  -74.2640f,8.2164f,24.9656f,  
	-72.3573f,7.6144f,25.0324f,  -71.5665f,10.0880f,26.7091f,  -69.6475f,9.5584f,26.5117f,  -68.7769f,11.8812f,28.0955f,  -18.0093f,-52.3570f,-23.2528f,  
	-19.5623f,-52.8829f,-23.1182f,  -20.1128f,-51.5967f,-21.6883f,  -21.6277f,-52.0261f,-21.5518f,  -22.1761f,-50.5333f,-20.3382f,  -23.6358f,-50.8696f,-20.2034f,  
	-24.1616f,-49.2017f,-19.2319f,  -25.5529f,-49.4501f,-19.1009f,  -26.0391f,-47.6436f,-18.3922f,  -27.3498f,-47.8128f,-18.2671f,  -27.7845f,-45.9082f,-17.8362f,  
	-29.0073f,-46.0067f,-17.7178f,  -29.3837f,-44.0472f,-17.5736f,  -30.5175f,-44.0854f,-17.4619f,  -30.8348f,-42.1155f,-17.6072f,  -31.8772f,-42.1042f,-17.5020f,  
	-32.1402f,-40.1684f,-17.9329f,  -33.0995f,-40.1176f,-17.8323f,  -33.3188f,-38.2588f,-18.5386f,  -34.2041f,-38.1772f,-18.4397f,  -34.3957f,-36.4353f,-19.4048f,  
	-35.2219f,-36.3291f,-19.3035f,  -35.4058f,-34.7400f,-20.5047f,  -36.1907f,-34.6129f,-20.3948f,  -36.3902f,-33.2068f,-21.8037f,  -37.1533f,-33.0580f,-21.6787f,  
	-37.3935f,-31.8592f,-23.2622f,  -38.1563f,-31.6842f,-23.1134f,  -38.4625f,-30.7103f,-24.8339f,  -39.2471f,-30.5008f,-24.6504f,  -39.6435f,-29.7626f,-26.4670f,  
	-40.4680f,-29.5063f,-26.2391f,  -40.9761f,-29.0076f,-28.1087f,  -41.8572f,-28.6894f,-27.8246f,  -42.4943f,-28.4279f,-29.7029f,  -43.4450f,-28.0300f,-29.3510f,  
	-44.2234f,-27.9976f,-31.1935f,  -45.2515f,-27.5014f,-30.7628f,  -46.1776f,-27.6853f,-32.5264f,  -47.2863f,-27.0719f,-32.0063f,  -48.3602f,-27.4552f,-33.6501f,  
	-49.5482f,-26.7065f,-33.0313f,  -50.7632f,-27.2696f,-34.5175f,  -52.0245f,-26.3694f,-33.7923f,  -53.3677f,-27.0908f,-35.0874f,  -54.6925f,-26.0251f,-34.2499f,  
	-56.1452f,-26.8824f,-35.3254f,  -57.5201f,-25.6404f,-34.3715f,  -59.0582f,-26.6112f,-35.2045f,  -60.4682f,-25.1849f,-34.1317f,  -62.0632f,-26.2474f,-34.7054f,  
	-63.4903f,-24.6332f,-33.5138f,  -65.1105f,-25.7666f,-33.8178f,  -66.5363f,-23.9638f,-32.5089f,  -68.1476f,-25.1491f,-32.5392f,  -69.5533f,-23.1604f,-31.1163f,  
	-71.1203f,-24.3806f,-30.8756f,  -72.4874f,-22.2115f,-29.3434f,  -73.9746f,-23.4515f,-28.8406f,  -75.2849f,-21.1110f,-27.2055f,  -76.6579f,-22.3579f,-26.4557f,  
	-77.8942f,-19.8574f,-24.7255f,  -79.1203f,-21.1006f,-23.7492f,  -80.2667f,-18.4541f,-21.9329f,  -81.3162f,-19.6850f,-20.7559f,  -82.3589f,-16.9067f,-18.8623f,  
	-83.2058f,-18.1185f,-17.5146f,  -84.1320f,-15.2264f,-15.5549f,  -84.7546f,-16.4139f,-14.0702f,  -85.5536f,-13.4266f,-12.0557f,  -85.9349f,-14.5862f,-10.4708f,  
	-86.5979f,-11.5245f,-8.4142f,  -86.7264f,-12.6537f,-6.7678f,  -87.2472f,-9.5359f,-4.6798f,  -87.1170f,-10.6330f,-3.0120f,  -87.4906f,-7.4832f,-0.9073f,  
	-87.1014f,-8.5470f,0.7417f,  -87.3259f,-5.3842f,2.8520f,  -86.6833f,-6.4138f,4.4423f,  -86.7582f,-3.2605f,6.5448f,  -85.8734f,-4.2550f,8.0382f,  
	-85.8006f,-1.1330f,10.1197f,  -84.6900f,-2.0912f,11.4801f,  -84.4734f,0.9800f,13.5293f,  -83.1583f,0.0600f,14.7236f,  -82.8039f,3.0593f,16.7278f,  
	-81.3103f,2.1800f,17.7268f,  -80.8259f,5.0887f,19.6748f,  -79.1832f,4.2539f,20.4538f,  -78.5781f,7.0551f,22.3357f,  -76.8191f,6.2696f,22.8749f,  
	-76.1045f,8.9467f,24.6805f,  -74.2640f,8.2164f,24.9656f,  -73.4522f,10.7556f,26.6866f,  -71.5665f,10.0880f,26.7091f,  -70.6711f,12.4773f,28.3382f,  
	-68.7769f,11.8812f,28.0955f,  -67.8122f,14.1108f,29.6264f,  -19.5623f,-52.8829f,-23.1182f,  -21.1409f,-53.3583f,-22.9626f,  -21.6277f,-52.0261f,-21.5518f,  
	-23.1601f,-52.4047f,-21.3948f,  -23.6358f,-50.8696f,-20.2034f,  -25.1071f,-51.1553f,-20.0474f,  -25.5529f,-49.4501f,-19.1009f,  -26.9484f,-49.6503f,-18.9488f,  
	-27.3498f,-47.8128f,-18.2671f,  -28.6601f,-47.9354f,-18.1202f,  -29.0073f,-46.0067f,-17.7178f,  -30.2283f,-46.0615f,-17.5768f,  -30.5175f,-44.0854f,-17.4619f,  
	-31.6441f,-44.0834f,-17.3274f,  -31.8772f,-42.1042f,-17.5020f,  -32.9145f,-42.0558f,-17.3729f,  -33.0995f,-40.1176f,-17.8323f,  -34.0539f,-40.0325f,-17.7068f,  
	-34.2041f,-38.1772f,-18.4397f,  -35.0883f,-38.0630f,-18.3145f,  -35.2219f,-36.3291f,-19.3035f,  -36.0514f,-36.1912f,-19.1734f,  -36.1907f,-34.6129f,-20.3948f,  
	-36.9838f,-34.4523f,-20.2542f,  -37.1533f,-33.0580f,-21.6787f,  -37.9308f,-32.8722f,-21.5197f,  -38.1563f,-31.6842f,-23.1134f,  -38.9401f,-31.4672f,-22.9258f,  
	-39.2471f,-30.5008f,-24.6504f,  -40.0560f,-30.2420f,-24.4250f,  -40.4680f,-29.5063f,-26.2391f,  -41.3200f,-29.1920f,-25.9643f,  -41.8572f,-28.6894f,-27.8246f,  
	-42.7670f,-28.3032f,-27.4888f,  -43.4450f,-28.0300f,-29.3510f,  -44.4225f,-27.5540f,-28.9429f,  -45.2515f,-27.5014f,-30.7628f,  -46.3022f,-26.9170f,-30.2715f,  
	-47.2863f,-27.0719f,-32.0063f,  -48.4113f,-26.3607f,-31.4219f,  -49.5482f,-26.7065f,-33.0313f,  -50.7438f,-25.8516f,-32.3453f,  -52.0245f,-26.3694f,-33.7923f,  
	-53.2831f,-25.3558f,-32.9978f,  -54.6925f,-26.0251f,-34.2499f,  -56.0027f,-24.8409f,-33.3418f,  -57.5201f,-25.6404f,-34.3715f,  -58.8685f,-24.2760f,-33.3462f,  
	-60.4682f,-25.1849f,-34.1317f,  -61.8380f,-23.6352f,-32.9883f,  -63.4903f,-24.6332f,-33.5138f,  -64.8642f,-22.8958f,-32.2527f,  -66.5363f,-23.9638f,-32.5089f,  
	-67.8963f,-22.0397f,-31.1323f,  -69.5533f,-23.1604f,-31.1163f,  -70.8810f,-21.0538f,-29.6281f,  -72.4874f,-22.2115f,-29.3434f,  -73.7647f,-19.9297f,-27.7492f,  
	-75.2849f,-21.1110f,-27.2055f,  -76.4945f,-18.6641f,-25.5124f,  -77.8942f,-19.8574f,-24.7255f,  -79.0198f,-17.2583f,-22.9417f,  -80.2667f,-18.4541f,-21.9329f,  
	-81.2940f,-15.7163f,-20.0669f,  -82.3589f,-16.9067f,-18.8623f,  -83.2745f,-14.0473f,-16.9247f,  -84.1320f,-15.2264f,-15.5549f,  -84.9244f,-12.2632f,-13.5567f,  
	-85.5536f,-13.4266f,-12.0557f,  -86.2131f,-10.3803f,-10.0094f,  -86.5979f,-11.5245f,-8.4142f,  -87.1175f,-8.4131f,-6.3304f,  -87.2472f,-9.5359f,-4.6798f,  
	-87.6211f,-6.3834f,-2.5732f,  -87.4906f,-7.4832f,-0.9073f,  -87.7158f,-4.3086f,1.2109f,  -87.3259f,-5.3842f,2.8520f,  -87.4012f,-2.2102f,4.9682f,  
	-86.7582f,-3.2605f,6.5448f,  -86.6847f,-0.1095f,8.6457f,  -85.8006f,-1.1330f,10.1197f,  -85.5814f,1.9751f,12.1936f,  -84.4734f,0.9800f,13.5293f,  
	-84.1138f,4.0234f,15.5629f,  -82.8039f,3.0593f,16.7278f,  -82.3111f,6.0184f,18.7095f,  -80.8259f,5.0887f,19.6748f,  -80.2087f,7.9460f,21.5942f,  
	-78.5781f,7.0551f,22.3357f,  -77.8474f,9.7928f,24.1820f,  -76.1045f,8.9467f,24.6805f,  -75.2724f,11.5496f,26.4446f,  -73.4522f,10.7556f,26.6866f,  
	-72.5321f,13.2109f,28.3605f,  -70.6711f,12.4773f,28.3382f,  -69.6776f,14.7736f,29.9146f,  -67.8122f,14.1108f,29.6264f,  -66.7605f,16.2397f,31.1001f,  
	-21.1409f,-53.3583f,-22.9626f,  -22.7425f,-53.7806f,-22.7856f,  -23.1601f,-52.4047f,-21.3948f,  -24.7093f,-52.7299f,-21.2156f,  -25.1071f,-51.1553f,-20.0474f,  
	-26.5873f,-51.3892f,-19.8695f,  -26.9484f,-49.6503f,-18.9488f,  -28.3480f,-49.8001f,-18.7740f,  -28.6601f,-47.9354f,-18.1202f,  -29.9719f,-48.0102f,-17.9498f,  
	-30.2283f,-46.0615f,-17.5768f,  -31.4446f,-46.0722f,-17.4122f,  -31.6441f,-44.0834f,-17.3274f,  -32.7670f,-44.0404f,-17.1684f,  -32.9145f,-42.0558f,-17.3729f,  
	-33.9477f,-41.9696f,-17.2185f,  -34.0539f,-40.0325f,-17.7068f,  -35.0072f,-39.9118f,-17.5546f,  -35.0883f,-38.0630f,-18.3145f,  -35.9748f,-37.9147f,-18.1606f,  
	-36.0514f,-36.1912f,-19.1734f,  -36.8879f,-36.0182f,-19.0129f,  -36.9838f,-34.4523f,-20.2542f,  -37.7900f,-34.2541f,-20.0801f,  -37.9308f,-32.8722f,-21.5197f,  
	-38.7278f,-32.6450f,-21.3228f,  -38.9401f,-31.4672f,-22.9258f,  -39.7473f,-31.2021f,-22.6972f,  -40.0560f,-30.2420f,-24.4250f,  -40.8920f,-29.9273f,-24.1536f,  
	-41.3200f,-29.1920f,-25.9643f,  -42.2010f,-28.8126f,-25.6388f,  -42.7670f,-28.3032f,-27.4888f,  -43.7047f,-27.8420f,-27.0979f,  -44.4225f,-27.5540f,-28.9429f,  
	-45.4246f,-26.9931f,-28.4753f,  -46.3022f,-26.9170f,-30.2715f,  -47.3721f,-26.2380f,-29.7169f,  -48.4113f,-26.3607f,-31.4219f,  -49.5477f,-25.5463f,-30.7710f,  
	-50.7438f,-25.8516f,-32.3453f,  -51.9411f,-24.8862f,-31.5904f,  -53.2831f,-25.3558f,-32.9978f,  -54.5321f,-24.2263f,-32.1329f,  -56.0027f,-24.8409f,-33.3418f,  
	-57.2916f,-23.5362f,-32.3622f,  -58.8685f,-24.2760f,-33.3462f,  -60.1820f,-22.7897f,-32.2501f,  -61.8380f,-23.6352f,-32.9883f,  -63.1600f,-21.9633f,-31.7750f,  
	-64.8642f,-22.8958f,-32.2527f,  -66.1775f,-21.0375f,-30.9233f,  -67.8963f,-22.0397f,-31.1323f,  -69.1829f,-19.9979f,-29.6900f,  -70.8810f,-21.0538f,-29.6281f,  
	-72.1233f,-18.8342f,-28.0774f,  -73.7647f,-19.9297f,-27.7492f,  -74.9453f,-17.5412f,-26.0963f,  -76.4945f,-18.6641f,-25.5124f,  -77.5970f,-16.1182f,-23.7649f,  
	-79.0198f,-17.2583f,-22.9417f,  -80.0293f,-14.5675f,-21.1075f,  -81.2940f,-15.7163f,-20.0669f,  -82.1967f,-12.8967f,-18.1562f,  -83.2745f,-14.0473f,-16.9247f,  
	-84.0582f,-11.1161f,-14.9481f,  -84.9244f,-12.2632f,-13.5567f,  -85.5788f,-9.2405f,-11.5262f,  -86.2131f,-10.3803f,-10.0094f,  -86.7303f,-7.2836f,-7.9356f,  
	-87.1175f,-8.4131f,-6.3304f,  -87.4907f,-5.2662f,-4.2277f,  -87.6211f,-6.3834f,-2.5732f,  -87.8466f,-3.2054f,-0.4528f,  -87.7158f,-4.3086f,1.2109f,  
	-87.7915f,-1.1222f,3.3354f,  -87.4012f,-2.2102f,4.9682f,  -87.3273f,0.9620f,7.0833f,  -86.6847f,-0.1095f,8.6457f,  -86.4635f,3.0283f,10.7393f,  
	-85.5814f,1.9751f,12.1936f,  -85.2174f,5.0560f,14.2522f,  -84.1138f,4.0234f,15.5629f,  -83.6136f,7.0272f,17.5746f,  -82.3111f,6.0184f,18.7095f,  
	-81.6829f,8.9268f,20.6633f,  -80.2087f,7.9460f,21.5942f,  -79.4629f,10.7401f,23.4787f,  -77.8474f,9.7928f,24.1820f,  -76.9959f,12.4566f,25.9873f,  
	-75.2724f,11.5496f,26.4446f,  -74.3282f,14.0692f,28.1622f,  -72.5321f,13.2109f,28.3605f,  -71.5101f,15.5732f,29.9822f,  -69.6776f,14.7736f,29.9146f,  
	-68.5930f,16.9693f,31.4344f,  -66.7605f,16.2397f,31.1001f,  -65.6293f,18.2617f,32.5129f,  -22.7425f,-53.7806f,-22.7856f,  -24.3665f,-54.1469f,-22.5853f,  
	-24.7093f,-52.7299f,-21.2156f,  -26.2727f,-52.9999f,-21.0135f,  -26.5873f,-51.3892f,-19.8695f,  -28.0764f,-51.5691f,-19.6679f,  -28.3480f,-49.8001f,-18.7740f,  
	-29.7529f,-49.8980f,-18.5745f,  -29.9719f,-48.0102f,-17.9498f,  -31.2824f,-48.0365f,-17.7548f,  -31.4446f,-46.0722f,-17.4122f,  -32.6596f,-46.0373f,-17.2221f,  
	-32.7670f,-44.0404f,-17.1684f,  -33.8872f,-43.9554f,-16.9832f,  -33.9477f,-41.9696f,-17.2185f,  -34.9800f,-41.8440f,-17.0369f,  -35.0072f,-39.9118f,-17.5546f,  
	-35.9622f,-39.7539f,-17.3736f,  -35.9748f,-37.9147f,-18.1606f,  -36.8674f,-37.7290f,-17.9762f,  -36.8879f,-36.0182f,-19.0129f,  -37.7362f,-35.8063f,-18.8191f,  
	-37.7900f,-34.2541f,-20.0801f,  -38.6139f,-34.0142f,-19.8687f,  -38.7278f,-32.6450f,-21.3228f,  -39.5469f,-32.3701f,-21.0855f,  -39.7473f,-31.2021f,-22.6972f,  
	-40.5805f,-30.8825f,-22.4237f,  -40.8920f,-29.9273f,-24.1536f,  -41.7568f,-29.5494f,-23.8328f,  -42.2010f,-28.8126f,-25.6388f,  -43.1105f,-28.3607f,-25.2594f,  
	-43.7047f,-27.8420f,-27.0979f,  -44.6685f,-27.2989f,-26.6486f,  -45.4246f,-26.9931f,-28.4753f,  -46.4483f,-26.3406f,-27.9454f,  -47.3721f,-26.2380f,-29.7169f,  
	-48.4565f,-25.4588f,-29.0966f,  -49.5477f,-25.5463f,-30.7710f,  -50.6896f,-24.6242f,-30.0519f,  -51.9411f,-24.8862f,-31.5904f,  -53.1332f,-23.8071f,-30.7656f,  
	-54.5321f,-24.2263f,-32.1329f,  -55.7645f,-22.9781f,-31.1967f,  -57.2916f,-23.5362f,-32.3622f,  -58.5506f,-22.1111f,-31.3119f,  -60.1820f,-22.7897f,-32.2501f,  
	-61.4528f,-21.1822f,-31.0839f,  -63.1600f,-21.9633f,-31.7750f,  -64.4264f,-20.1711f,-30.4930f,  -66.1775f,-21.0375f,-30.9233f,  -67.4222f,-19.0621f,-29.5279f,  
	-69.1829f,-19.9979f,-29.6900f,  -70.3887f,-17.8432f,-28.1845f,  -72.1233f,-18.8342f,-28.0774f,  -73.2730f,-16.5078f,-26.4674f,  -74.9453f,-17.5412f,-26.0963f,  
	-76.0225f,-15.0531f,-24.3885f,  -77.5970f,-16.1182f,-23.7649f,  -78.5866f,-13.4800f,-21.9666f,  -80.0293f,-14.5675f,-21.1075f,  -80.9170f,-11.7944f,-19.2283f,  
	-82.1967f,-12.8967f,-18.1562f,  -82.9698f,-10.0048f,-16.2060f,  -84.0582f,-11.1161f,-14.9481f,  -84.7057f,-8.1247f,-12.9386f,  -85.5788f,-9.2405f,-11.5262f,  
	-86.0920f,-6.1669f,-9.4678f,  -86.7303f,-7.2836f,-7.9356f,  -87.1018f,-4.1509f,-5.8423f,  -87.4907f,-5.2662f,-4.2277f,  -87.7159f,-2.0934f,-2.1106f,  
	-87.8466f,-3.2054f,-0.4528f,  -87.9223f,-0.0150f,1.6744f,  -87.7915f,-1.1222f,3.3354f,  -87.7173f,2.0630f,5.4591f,  -87.3273f,0.9620f,7.0833f,  
	-87.1045f,4.1215f,9.1914f,  -86.4635f,3.0283f,10.7393f,  -86.0959f,6.1394f,12.8182f,  -85.2174f,5.0560f,14.2522f,  -84.7109f,8.0978f,16.2894f,  
	-83.6136f,7.0272f,17.5746f,  -82.9756f,9.9809f,19.5587f,  -81.6829f,8.9268f,20.6633f,  -80.9233f,11.7726f,22.5824f,  -79.4629f,10.7401f,23.4787f,  
	-78.5931f,13.4610f,25.3225f,  -76.9959f,12.4566f,25.9873f,  -76.0287f,15.0377f,27.7467f,  -74.3282f,14.0692f,28.1622f,  -73.2784f,16.4960f,29.8281f,  
	-71.5101f,15.5732f,29.9822f,  -70.3929f,17.8353f,31.5478f,  -68.5930f,16.9693f,31.4344f,  -67.4246f,19.0580f,32.8938f,  -65.6293f,18.2617f,32.5129f,  
	-64.4266f,20.1709f,33.8616f,  -24.3665f,-54.1469f,-22.5853f,  -26.0098f,-54.4547f,-22.3613f,  -26.2727f,-52.9999f,-21.0135f,  -27.8498f,-53.2120f,-20.7866f,  
	-28.0764f,-51.5691f,-19.6679f,  -29.5749f,-51.6927f,-19.4403f,  -29.7529f,-49.8980f,-18.5745f,  -31.1602f,-49.9427f,-18.3494f,  -31.2824f,-48.0365f,-17.7548f,  
	-32.5941f,-48.0124f,-17.5333f,  -32.6596f,-46.0373f,-17.2221f,  -33.8735f,-45.9557f,-17.0050f,  -33.8872f,-43.9554f,-16.9832f,  -35.0072f,-43.8264f,-16.7701f,  
	-34.9800f,-41.8440f,-17.0369f,  -36.0138f,-41.6773f,-16.8259f,  -35.9622f,-39.7539f,-17.3736f,  -36.9224f,-39.5554f,-17.1617f,  -36.8674f,-37.7290f,-17.9762f,  
	-37.7703f,-37.5021f,-17.7584f,  -37.7362f,-35.8063f,-18.8191f,  -38.6004f,-35.5513f,-18.5884f,  -38.6139f,-34.0142f,-19.8687f,  -39.4585f,-33.7262f,-19.6174f,  
	-39.5469f,-32.3701f,-21.0855f,  -40.3908f,-32.0414f,-20.8041f,  -40.5805f,-30.8825f,-22.4237f,  -41.4419f,-30.5011f,-22.1015f,  -41.7568f,-29.5494f,-23.8328f,  
	-42.6501f,-29.1010f,-23.4588f,  -43.1105f,-28.3607f,-25.2594f,  -44.0469f,-27.8292f,-24.8225f,  -44.6685f,-27.2989f,-26.6486f,  -45.6556f,-26.6667f,-26.1378f,  
	-46.4483f,-26.3406f,-27.9454f,  -47.4890f,-25.5905f,-27.3505f,  -48.4565f,-25.4588f,-29.0966f,  -49.5498f,-24.5743f,-28.4085f,  -50.6896f,-24.6242f,-30.0519f,  
	-51.8302f,-23.5906f,-29.2631f,  -53.1332f,-23.8071f,-30.7656f,  -54.3132f,-22.6113f,-29.8695f,  -55.7645f,-22.9781f,-31.1967f,  -56.9718f,-21.6111f,-30.1897f,  
	-58.5506f,-22.1111f,-31.3119f,  -59.7719f,-20.5661f,-30.1912f,  -61.4528f,-21.1822f,-31.0839f,  -62.6730f,-19.4553f,-29.8487f,  -64.4264f,-20.1711f,-30.4930f,  
	-65.6290f,-18.2623f,-29.1446f,  -67.4222f,-19.0621f,-29.5279f,  -68.5907f,-16.9738f,-28.0688f,  -70.3887f,-17.8432f,-28.1845f,  -71.5063f,-15.5816f,-26.6192f,  
	-73.2730f,-16.5078f,-26.4674f,  -74.3234f,-14.0814f,-24.8018f,  -76.0225f,-15.0531f,-24.3885f,  -76.9904f,-12.4726f,-22.6294f,  -78.5866f,-13.4800f,-21.9666f,  
	-79.4574f,-10.7591f,-20.1227f,  -80.9170f,-11.7944f,-19.2283f,  -81.6777f,-8.9484f,-17.3090f,  -82.9698f,-10.0048f,-16.2060f,  -83.6088f,-7.0521f,-14.2225f,  
	-84.7057f,-8.1247f,-12.9386f,  -85.2137f,-5.0819f,-10.9008f,  -86.0920f,-6.1669f,-9.4678f,  -86.4608f,-3.0565f,-7.3894f,  -87.1018f,-4.1509f,-5.8423f,  
	-87.3259f,-0.9917f,-3.7343f,  -87.7159f,-2.0934f,-2.1106f,  -87.7915f,1.0924f,0.0135f,  -87.9223f,-0.0150f,1.6744f,  -87.8480f,3.1747f,3.8011f,  
	-87.7173f,2.0630f,5.4591f,  -87.4936f,5.2361f,7.5764f,  -87.1045f,4.1215f,9.1914f,  -86.7344f,7.2549f,11.2852f,  -86.0959f,6.1394f,12.8182f,  
	-85.5843f,9.2119f,14.8758f,  -84.7109f,8.0978f,16.2894f,  -84.0645f,11.0901f,18.2994f,  -82.9756f,9.9809f,19.5587f,  -82.2037f,12.8725f,21.5087f,  
	-80.9233f,11.7726f,22.5824f,  -80.0369f,14.5456f,24.4615f,  -78.5931f,13.4610f,25.3225f,  -77.6044f,16.0996f,27.1210f,  -76.0287f,15.0377f,27.7467f,  
	-74.9522f,17.5262f,29.4548f,  -73.2784f,16.4960f,29.8281f,  -72.1292f,18.8229f,31.4383f,  -70.3929f,17.8353f,31.5478f,  -69.1874f,19.9903f,33.0534f,  
	-67.4246f,19.0580f,32.8938f,  -66.1800f,21.0337f,34.2894f,  -64.4266f,20.1709f,33.8616f,  -63.1600f,21.9634f,35.1438f,  -26.0098f,-54.4547f,-22.3613f,  
	-27.6723f,-54.7008f,-22.1110f,  -27.8498f,-53.2120f,-20.7866f,  -29.4410f,-53.3635f,-20.5324f,  -29.5749f,-51.6927f,-19.4403f,  -31.0799f,-51.7582f,-19.1859f,  
	-31.1602f,-49.9427f,-18.3494f,  -32.5716f,-49.9319f,-18.0965f,  -32.5941f,-48.0124f,-17.5333f,  -33.9068f,-47.9365f,-17.2835f,  -33.8735f,-45.9557f,-17.0050f,  
	-35.0882f,-45.8253f,-16.7589f,  -35.0072f,-43.8264f,-16.7701f,  -36.1286f,-43.6520f,-16.5269f,  -36.0138f,-41.6773f,-16.8259f,  -37.0521f,-41.4661f,-16.5834f,  
	-36.9224f,-39.5554f,-17.1617f,  -37.8918f,-39.3126f,-16.9161f,  -37.7703f,-37.5021f,-17.7584f,  -38.6873f,-37.2301f,-17.5038f,  -38.6004f,-35.5513f,-18.5884f,  
	-39.4834f,-35.2470f,-18.3180f,  -39.4585f,-33.7262f,-19.6174f,  -40.3262f,-33.3840f,-19.3225f,  -40.3908f,-32.0414f,-20.8041f,  -41.2617f,-31.6513f,-20.4747f,  
	-41.4419f,-30.5011f,-22.1015f,  -42.3312f,-30.0504f,-21.7270f,  -42.6501f,-29.1010f,-23.4588f,  -43.5704f,-28.5749f,-23.0283f,  -44.0469f,-27.8292f,-24.8225f,  
	-45.0077f,-27.2108f,-24.3249f,  -45.6556f,-26.6667f,-26.1378f,  -46.6617f,-25.9392f,-25.5628f,  -47.4890f,-25.5905f,-27.3505f,  -48.5414f,-24.7373f,-26.6885f,  
	-49.5498f,-24.5743f,-28.4085f,  -50.6452f,-23.5804f,-27.6514f,  -51.8302f,-23.5906f,-29.2631f,  -52.9626f,-22.4421f,-28.4034f,  -54.3132f,-22.6113f,-29.8695f,  
	-55.4724f,-21.2982f,-28.9028f,  -56.9718f,-21.6111f,-30.1897f,  -58.1459f,-20.1253f,-29.1124f,  -59.7719f,-20.5661f,-30.1912f,  -60.9472f,-18.9023f,-29.0014f,  
	-62.6730f,-19.4553f,-29.8487f,  -63.8340f,-17.6122f,-28.5468f,  -65.6290f,-18.2623f,-29.1446f,  -66.7601f,-16.2408f,-27.7321f,  -68.5907f,-16.9738f,-28.0688f,  
	-69.6754f,-14.7785f,-26.5493f,  -71.5063f,-15.5816f,-26.6192f,  -72.5286f,-13.2196f,-24.9977f,  -74.3234f,-14.0814f,-24.8018f,  -75.2681f,-11.5623f,-23.0844f,  
	-76.9904f,-12.4726f,-22.6294f,  -77.8427f,-9.8088f,-20.8241f,  -79.4574f,-10.7591f,-20.1227f,  -80.2042f,-7.9649f,-18.2382f,  -81.6777f,-8.9484f,-17.3090f,  
	-82.3069f,-6.0409f,-15.3558f,  -83.6088f,-7.0521f,-14.2225f,  -84.1104f,-4.0474f,-12.2102f,  -85.2137f,-5.0819f,-10.9008f,  -85.5789f,-2.0017f,-8.8426f,  
	-86.4608f,-3.0565f,-7.3894f,  -86.6834f,0.0809f,-5.2960f,  -87.3259f,-0.9917f,-3.7343f,  -87.4012f,2.1811f,-1.6188f,  -87.7915f,1.0924f,0.0135f,  
	-87.7173f,4.2782f,2.1376f,  -87.8480f,3.1747f,3.8011f,  -87.6240f,6.3530f,5.9217f,  -87.4936f,5.2361f,7.5764f,  -87.1217f,8.3837f,9.6795f,  
	-86.7344f,7.2549f,11.2852f,  -86.2188f,10.3504f,13.3582f,  -85.5843f,9.2119f,14.8758f,  -84.9311f,12.2356f,16.9069f,  -84.0645f,11.0901f,18.2994f,  
	-83.2822f,14.0210f,20.2758f,  -82.2037f,12.8725f,21.5087f,  -81.3024f,15.6918f,23.4191f,  -80.0369f,14.5456f,24.4615f,  -79.0284f,17.2370f,26.2959f,  
	-77.6044f,16.0996f,27.1210f,  -76.5028f,18.6461f,28.8687f,  -74.9522f,17.5262f,29.4548f,  -73.7723f,19.9152f,31.1079f,  -72.1292f,18.8229f,31.4383f,  
	-70.8875f,21.0427f,32.9892f,  -69.1874f,19.9903f,33.0534f,  -67.9012f,22.0323f,34.4958f,  -66.1800f,21.0337f,34.2894f,  -64.8668f,22.8924f,35.6190f,  
	-63.1600f,21.9634f,35.1438f,  -61.8377f,23.6358f,36.3574f,  -27.6723f,-54.7008f,-22.1110f,  -29.3535f,-54.8818f,-21.8320f,  -29.4410f,-53.3635f,-20.5324f,  
	-31.0431f,-53.4519f,-20.2498f,  -31.0799f,-51.7582f,-19.1859f,  -32.5925f,-51.7630f,-18.9021f,  -32.5716f,-49.9319f,-18.0965f,  -33.9865f,-49.8638f,-17.8139f,  
	-33.9068f,-47.9365f,-17.2835f,  -35.2218f,-47.8063f,-17.0034f,  -35.0882f,-45.8253f,-16.7589f,  -36.3048f,-45.6443f,-16.4816f,  -36.1286f,-43.6520f,-16.5269f,  
	-37.2543f,-43.4285f,-16.2512f,  -37.0521f,-41.4661f,-16.5834f,  -38.0985f,-41.2068f,-16.3066f,  -37.8918f,-39.3126f,-16.9161f,  -38.8733f,-39.0217f,-16.6335f,  
	-38.6873f,-37.2301f,-17.5038f,  -39.6214f,-36.9067f,-17.2095f,  -39.4834f,-35.2470f,-18.3180f,  -40.3878f,-34.8876f,-18.0042f,  -40.3262f,-33.3840f,-19.3225f,  
	-41.2193f,-32.9804f,-18.9801f,  -41.2617f,-31.6513f,-20.4747f,  -42.1595f,-31.1926f,-20.0936f,  -42.3312f,-30.0504f,-21.7270f,  -43.2472f,-29.5232f,-21.2968f,  
	-43.5704f,-28.5749f,-23.0283f,  -44.5157f,-27.9635f,-22.5379f,  -45.0077f,-27.2108f,-24.3249f,  -45.9888f,-26.4989f,-23.7639f,  -46.6617f,-25.9392f,-25.5628f,  
	-47.6815f,-25.1106f,-24.9212f,  -48.5414f,-24.7373f,-26.6885f,  -49.5986f,-23.7767f,-25.9578f,  -50.6452f,-23.5804f,-27.6514f,  -51.7359f,-22.4733f,-26.8235f,  
	-52.9626f,-22.4421f,-28.4034f,  -54.0782f,-21.1778f,-27.4732f,  -55.4724f,-21.2982f,-28.9028f,  -56.6027f,-19.8675f,-27.8657f,  -58.1459f,-20.1253f,-29.1124f,  
	-59.2786f,-18.5216f,-27.9656f,  -60.9472f,-18.9023f,-29.0014f,  -62.0679f,-17.1230f,-27.7445f,  -63.8340f,-17.6122f,-28.5468f,  -64.9280f,-15.6566f,-27.1802f,  
	-66.7601f,-16.2408f,-27.7321f,  -67.8116f,-14.1122f,-26.2586f,  -69.6754f,-14.7785f,-26.5493f,  -70.6690f,-12.4826f,-24.9730f,  -72.5286f,-13.2196f,-24.9977f,  
	-73.4491f,-10.7647f,-23.3241f,  -75.2681f,-11.5623f,-23.0844f,  -76.1008f,-8.9595f,-21.3204f,  -77.8427f,-9.8088f,-20.8241f,  -78.5743f,-7.0711f,-18.9777f,  
	-80.2042f,-7.9649f,-18.2382f,  -80.8221f,-5.1084f,-16.3192f,  -82.3069f,-6.0409f,-15.3558f,  -82.8009f,-3.0810f,-13.3735f,  -84.1104f,-4.0474f,-12.2102f,  
	-84.4710f,-1.0047f,-10.1770f,  -85.5789f,-2.0017f,-8.8426f,  -85.7993f,1.1060f,-6.7689f,  -86.6834f,0.0809f,-5.2960f,  -86.7582f,3.2325f,-3.1947f,  
	-87.4012f,2.1811f,-1.6188f,  -87.3273f,5.3544f,0.4969f,  -87.7173f,4.2782f,2.1376f,  -87.4934f,7.4530f,4.2559f,  -87.6240f,6.3530f,5.9217f,  
	-87.2514f,9.5062f,8.0287f,  -87.1217f,8.3837f,9.6795f,  -86.6037f,11.4939f,11.7624f,  -86.2188f,10.3504f,13.3582f,  -85.5606f,13.3976f,15.4050f,  
	-84.9311f,12.2356f,16.9069f,  -84.1402f,15.1984f,18.9048f,  -83.2822f,14.0210f,20.2758f,  -82.3681f,16.8801f,22.2131f,  -81.3024f,15.6918f,23.4191f,  
	-80.2762f,18.4303f,25.2854f,  -79.0284f,17.2370f,26.2959f,  -77.9038f,19.8366f,28.0800f,  -76.5028f,18.6461f,28.8687f,  -75.2941f,21.0935f,30.5621f,  
	-73.7723f,19.9152f,31.1079f,  -72.4958f,22.1972f,32.7021f,  -70.8875f,21.0427f,32.9892f,  -69.5605f,23.1496f,34.4774f,  -67.9012f,22.0323f,34.4958f,  
	-66.5414f,23.9570f,35.8727f,  -64.8668f,22.8924f,35.6190f,  -63.4928f,24.6304f,36.8805f,  -61.8377f,23.6358f,36.3574f,  -60.4675f,25.1860f,37.5013f,  
	-29.3535f,-54.8818f,-21.8320f,  -31.0504f,-54.9948f,-21.5231f,  -31.0431f,-53.4519f,-20.2498f,  -32.6564f,-53.4742f,-19.9363f,  -32.5925f,-51.7630f,-18.9021f,  
	-34.1114f,-51.7046f,-18.5871f,  -33.9865f,-49.8638f,-17.8139f,  -35.4057f,-49.7356f,-17.4994f,  -35.2218f,-47.8063f,-17.0034f,  -36.5393f,-47.6200f,-16.6910f,  
	-36.3048f,-45.6443f,-16.4816f,  -37.5255f,-45.4089f,-16.1708f,  -37.2543f,-43.4285f,-16.2512f,  -38.3870f,-43.1522f,-15.9403f,  -38.0985f,-41.2068f,-16.3066f,  
	-39.1552f,-40.8957f,-15.9924f,  -38.8733f,-39.0217f,-16.6335f,  -39.8699f,-38.6767f,-16.3109f,  -39.6214f,-36.9067f,-17.2095f,  -40.5744f,-36.5262f,-16.8719f,  
	-40.3878f,-34.8876f,-18.0042f,  -41.3154f,-34.4657f,-17.6433f,  -41.2193f,-32.9804f,-18.9801f,  -42.1377f,-32.5080f,-18.5865f,  -42.1595f,-31.1926f,-20.0936f,  
	-43.0829f,-30.6580f,-19.6574f,  -43.2472f,-29.5232f,-21.2968f,  -44.1879f,-28.9119f,-20.8073f,  -44.5157f,-27.9635f,-22.5379f,  -45.4817f,-27.2601f,-21.9846f,  
	-45.9888f,-26.4989f,-23.7639f,  -46.9849f,-25.6876f,-23.1368f,  -47.6815f,-25.1106f,-24.9212f,  -48.7082f,-24.1762f,-24.2114f,  -49.5986f,-23.7767f,-25.9578f,  
	-50.6539f,-22.7045f,-25.1567f,  -51.7359f,-22.4733f,-26.8235f,  -52.8130f,-21.2519f,-25.9253f,  -54.0782f,-21.1778f,-27.4732f,  -55.1687f,-19.7970f,-26.4726f,  
	-56.6027f,-19.8675f,-27.8657f,  -57.6957f,-18.3196f,-26.7590f,  -59.2786f,-18.5216f,-27.9656f,  -60.3609f,-16.8030f,-26.7516f,  -62.0679f,-17.1230f,-27.7445f,  
	-63.1260f,-15.2315f,-26.4226f,  -64.9280f,-15.6566f,-27.1802f,  -65.9468f,-13.5941f,-25.7524f,  -67.8116f,-14.1122f,-26.2586f,  -68.7763f,-11.8829f,-24.7279f,  
	-70.6690f,-12.4826f,-24.9730f,  -71.5646f,-10.0936f,-23.3441f,  -73.4491f,-10.7647f,-23.3241f,  -74.2612f,-8.2258f,-21.6032f,  -76.1008f,-8.9595f,-21.3204f,  
	-76.8160f,-6.2824f,-19.5148f,  -78.5743f,-7.0711f,-18.9777f,  -79.1800f,-4.2706f,-17.0963f,  -80.8221f,-5.1084f,-16.3192f,  -81.3076f,-2.1992f,-14.3708f,  
	-82.8009f,-3.0810f,-13.3735f,  -83.1562f,-0.0823f,-11.3697f,  -84.4710f,-1.0047f,-10.1770f,  -84.6888f,2.0661f,-8.1280f,  -85.7993f,1.1060f,-6.7689f,  
	-85.8734f,4.2285f,-4.6871f,  -86.7582f,3.2325f,-3.1947f,  -86.6847f,6.3851f,-1.0927f,  -87.3273f,5.3544f,0.4969f,  -87.1042f,8.5174f,2.6073f,  
	-87.4934f,7.4530f,4.2559f,  -87.1212f,10.6034f,6.3610f,  -87.2514f,9.5062f,8.0287f,  -86.7323f,12.6227f,10.1158f,  -86.6037f,11.4939f,11.7624f,  
	-85.9421f,14.5564f,13.8196f,  -85.5606f,13.3976f,15.4050f,  -84.7631f,16.3847f,17.4192f,  -84.1402f,15.1984f,18.9048f,  -83.2157f,18.0901f,20.8641f,  
	-82.3681f,16.8801f,22.2131f,  -81.3267f,19.6590f,24.1069f,  -80.2762f,18.4303f,25.2854f,  -79.1311f,21.0773f,27.1020f,  -77.9038f,19.8366f,28.0800f,  
	-76.6686f,22.3376f,29.8104f,  -75.2941f,21.0935f,30.5621f,  -73.9849f,23.4341f,32.1972f,  -72.4958f,22.1972f,32.7021f,  -71.1297f,24.3665f,34.2343f,  
	-69.5605f,23.1496f,34.4774f,  -68.1552f,25.1390f,35.9006f,  -66.5414f,23.9570f,35.8727f,  -65.1159f,25.7603f,37.1819f,  -63.4928f,24.6304f,36.8805f,  
	-62.0657f,26.2450f,38.0724f,  -60.4675f,25.1860f,37.5013f,  -59.0576f,26.6122f,38.5740f,  -31.0504f,-54.9948f,-21.5231f,  -32.7623f,-55.0363f,-21.1815f,  
	-32.6564f,-53.4742f,-19.9363f,  -34.2792f,-53.4275f,-19.5899f,  -34.1114f,-51.7046f,-18.5871f,  -35.6366f,-51.5802f,-18.2386f,  -35.4057f,-49.7356f,-17.4994f,  
	-36.8282f,-49.5453f,-17.1513f,  -36.5393f,-47.6200f,-16.6910f,  -37.8611f,-47.3735f,-16.3435f,  -37.5255f,-45.4089f,-16.1708f,  -38.7523f,-45.1157f,-15.8236f,  
	-38.3870f,-43.1522f,-15.9403f,  -39.5282f,-42.8198f,-15.5914f,  -39.1552f,-40.8957f,-15.9924f,  -40.2248f,-40.5267f,-15.6376f,  -39.8699f,-38.6767f,-16.3109f,  
	-40.8830f,-38.2719f,-15.9449f,  -40.5744f,-36.5262f,-16.8719f,  -41.5484f,-36.0816f,-16.4873f,  -41.3154f,-34.4657f,-17.6433f,  -42.2663f,-33.9742f,-17.2316f,  
	-42.1377f,-32.5080f,-18.5865f,  -43.0801f,-31.9598f,-18.1384f,  -43.0829f,-30.6580f,-19.6574f,  -44.0300f,-30.0398f,-19.1626f,  -44.1879f,-28.9119f,-20.8073f,  
	-45.1492f,-28.2096f,-20.2555f,  -45.4817f,-27.2601f,-21.9846f,  -46.4634f,-26.4585f,-21.3660f,  -46.9849f,-25.6876f,-23.1368f,  -47.9894f,-24.7720f,-22.4421f,  
	-48.7082f,-24.1762f,-24.2114f,  -49.7353f,-23.1317f,-23.4317f,  -50.6539f,-22.7045f,-25.1567f,  -51.6986f,-21.5193f,-24.2856f,  -52.8130f,-21.2519f,-25.9253f,  
	-53.8684f,-19.9151f,-24.9568f,  -55.1687f,-19.7970f,-26.4726f,  -56.2256f,-18.2999f,-25.4023f,  -57.6957f,-18.3196f,-26.7590f,  -58.7422f,-16.6576f,-25.5850f,  
	-60.3609f,-16.8030f,-26.7516f,  -61.3847f,-14.9725f,-25.4723f,  -63.1260f,-15.2315f,-26.4226f,  -64.1130f,-13.2332f,-25.0391f,  -65.9468f,-13.5941f,-25.7524f,  
	-66.8829f,-11.4307f,-24.2668f,  -68.7763f,-11.8829f,-24.7279f,  -69.6468f,-9.5604f,-23.1443f,  -71.5646f,-10.0936f,-23.3441f,  -72.3556f,-7.6203f,-21.6676f,  
	-74.2612f,-8.2258f,-21.6032f,  -74.9594f,-5.6122f,-19.8403f,  -76.8160f,-6.2824f,-19.5148f,  -77.4087f,-3.5419f,-17.6736f,  -79.1800f,-4.2706f,-17.0963f,  
	-79.6561f,-1.4171f,-15.1851f,  -81.3076f,-2.1992f,-14.3708f,  -81.6570f,0.7494f,-12.4005f,  -83.1562f,-0.0823f,-11.3697f,  -83.3708f,2.9448f,-9.3499f,  
	-84.6888f,2.0661f,-8.1280f,  -84.7619f,5.1521f,-6.0705f,  -85.8734f,4.2285f,-4.6871f,  -85.8005f,7.3525f,-2.6041f,  -86.6847f,6.3851f,-1.0927f,  
	-86.4630f,9.5279f,1.0043f,  -87.1042f,8.5174f,2.6073f,  -86.7333f,11.6564f,4.7049f,  -87.1212f,10.6034f,6.3610f,  -86.6024f,13.7176f,8.4466f,  
	-86.7323f,12.6227f,10.1158f,  -86.0691f,15.6921f,12.1776f,  -85.9421f,14.5564f,13.8196f,  -85.1403f,17.5594f,15.8445f,  -84.7631f,16.3847f,17.4192f,  
	-83.8306f,19.3011f,19.3953f,  -83.2157f,18.0901f,20.8641f,  -82.1617f,20.9022f,22.7806f,  -81.3267f,19.6590f,24.1069f,  -80.1637f,22.3470f,25.9515f,  
	-79.1311f,21.0773f,27.1020f,  -77.8722f,23.6263f,28.8655f,  -76.6686f,22.3376f,29.8104f,  -75.3293f,24.7321f,31.4830f,  -73.9849f,23.4341f,32.1972f,  
	-72.5818f,25.6621f,33.7709f,  -71.1297f,24.3665f,34.2343f,  -69.6806f,26.4180f,35.7021f,  -68.1552f,25.1390f,35.9006f,  -66.6792f,27.0064f,37.2566f,  
	-65.1159f,25.7603f,37.1819f,  -63.6321f,27.4394f,38.4218f,  -62.0657f,26.2450f,38.0724f,  -60.5935f,27.7344f,39.1935f,  -59.0576f,26.6122f,38.5740f,  
	-57.6151f,27.9148f,39.5759f,  -32.7623f,-55.0363f,-21.1815f,  -34.4871f,-55.0029f,-20.8052f,  -34.2792f,-53.4275f,-19.5899f,  -35.9106f,-53.3085f,-19.2081f,  
	-35.6366f,-51.5802f,-18.2386f,  -37.1664f,-51.3873f,-17.8547f,  -36.8282f,-49.5453f,-17.1513f,  -38.2553f,-49.2887f,-16.7664f,  -37.8611f,-47.3735f,-16.3435f,  
	-39.1882f,-47.0635f,-15.9583f,  -38.7523f,-45.1157f,-15.8236f,  -39.9858f,-44.7612f,-15.4374f,  -39.5282f,-42.8198f,-15.5914f,  -40.6800f,-42.4253f,-15.2011f,  
	-40.2248f,-40.5267f,-15.6376f,  -41.3083f,-40.0948f,-15.2390f,  -40.8830f,-38.2719f,-15.9449f,  -41.9143f,-37.8007f,-15.5317f,  -41.5484f,-36.0816f,-16.4873f,  
	-42.5431f,-35.5661f,-16.0519f,  -42.2663f,-33.9742f,-17.2316f,  -43.2390f,-33.4063f,-16.7656f,  -43.0801f,-31.9598f,-18.1384f,  -44.0447f,-31.3280f,-17.6320f,  
	-44.0300f,-30.0398f,-19.1626f,  -44.9969f,-29.3313f,-18.6061f,  -45.1492f,-28.2096f,-20.2555f,  -46.1261f,-27.4102f,-19.6389f,  -46.4634f,-26.4585f,-21.3660f,  
	-47.4543f,-25.5539f,-20.6802f,  -47.9894f,-24.7720f,-22.4421f,  -48.9957f,-23.7475f,-21.6779f,  -49.7353f,-23.1317f,-23.4317f,  -50.7538f,-21.9753f,-22.5822f,  
	-51.6986f,-21.5193f,-24.2856f,  -52.7242f,-20.2197f,-23.3444f,  -53.8684f,-19.9151f,-24.9568f,  -54.8935f,-18.4628f,-23.9187f,  -56.2256f,-18.2999f,-25.4023f,  
	-57.2396f,-16.6894f,-24.2647f,  -58.7422f,-16.6576f,-25.5850f,  -59.7341f,-14.8841f,-24.3455f,  -61.3847f,-14.9725f,-25.4723f,  -62.3414f,-13.0354f,-24.1311f,  
	-64.1130f,-13.2332f,-25.0391f,  -65.0211f,-11.1339f,-23.5975f,  -66.8829f,-11.4307f,-24.2668f,  -67.7286f,-9.1742f,-22.7280f,  -69.6468f,-9.5604f,-23.1443f,  
	-70.4166f,-7.1534f,-21.5126f,  -72.3556f,-7.6203f,-21.6676f,  -73.0362f,-5.0721f,-19.9487f,  -74.9594f,-5.6122f,-19.8403f,  -75.5383f,-2.9347f,-18.0414f,  
	-77.4087f,-3.5419f,-17.6736f,  -77.8747f,-0.7481f,-15.8024f,  -79.6561f,-1.4171f,-15.1851f,  -79.9989f,1.4765f,-13.2515f,  -81.6570f,0.7494f,-12.4005f,  
	-81.8680f,3.7269f,-10.4138f,  -83.3708f,2.9448f,-9.3499f,  -83.4429f,5.9875f,-7.3212f,  -84.7619f,5.1521f,-6.0705f,  -84.6899f,8.2402f,-4.0115f,  
	-85.8005f,7.3525f,-2.6041f,  -85.5808f,10.4672f,-0.5259f,  -86.4630f,9.5279f,1.0043f,  -86.0944f,12.6473f,3.0887f,  -86.7333f,11.6564f,4.7049f,  
	-86.2163f,14.7599f,6.7832f,  -86.6024f,13.7176f,8.4466f,  -85.9396f,16.7853f,10.5072f,  -86.0691f,15.6921f,12.1776f,  -85.2653f,18.7026f,14.2076f,  
	-85.1403f,17.5594f,15.8445f,  -84.2024f,20.4924f,17.8318f,  -83.8306f,19.3011f,19.3953f,  -82.7670f,22.1387f,21.3290f,  -82.1617f,20.9022f,22.7806f,  
	-80.9841f,23.6240f,24.6483f,  -80.1637f,22.3470f,25.9515f,  -78.8843f,24.9374f,27.7437f,  -77.8722f,23.6263f,28.8655f,  -76.5058f,26.0691f,30.5718f,  
	-75.3293f,24.7321f,31.4830f,  -73.8922f,27.0142f,33.0949f,  -72.5818f,25.6621f,33.7709f,  -71.0915f,27.7723f,35.2807f,  -69.6806f,26.4180f,35.7021f,  
	-68.1559f,28.3472f,37.1031f,  -66.6792f,27.0064f,37.2566f,  -65.1397f,28.7487f,38.5436f,  -63.6321f,27.4394f,38.4218f,  -62.0976f,28.9920f,39.5913f,  
	-60.5935f,27.7344f,39.1935f,  -59.0833f,29.0983f,40.2436f,  -57.6151f,27.9148f,39.5759f,  -56.1476f,29.0941f,40.5067f,  -34.4871f,-55.0029f,-20.8052f,  
	-36.2231f,-54.8909f,-20.3915f,  -35.9106f,-53.3085f,-19.2081f,  -37.5482f,-53.1144f,-18.7891f,  -37.1664f,-51.3873f,-17.8547f,  -38.7013f,-51.1217f,-17.4323f,  
	-38.2553f,-49.2887f,-16.7664f,  -39.6871f,-48.9623f,-16.3423f,  -39.1882f,-47.0635f,-15.9583f,  -40.5202f,-46.6866f,-15.5330f,  -39.9858f,-44.7612f,-15.4374f,  
	-41.2277f,-44.3397f,-15.0089f,  -40.6800f,-42.4253f,-15.2011f,  -41.8429f,-41.9637f,-14.7664f,  -41.3083f,-40.0948f,-15.2390f,  -42.4069f,-39.5934f,-14.7929f,  
	-41.9143f,-37.8007f,-15.5317f,  -42.9633f,-37.2563f,-15.0678f,  -42.5431f,-35.5661f,-16.0519f,  -43.5568f,-34.9727f,-15.5626f,  -43.2390f,-33.4063f,-16.7656f,  
	-44.2317f,-32.7543f,-16.2418f,  -44.0447f,-31.3280f,-17.6320f,  -45.0275f,-30.6060f,-17.0645f,  -44.9969f,-29.3313f,-18.6061f,  -45.9787f,-28.5263f,-17.9852f,  
	-46.1261f,-27.4102f,-19.6389f,  -47.1122f,-26.5086f,-18.9557f,  -47.4543f,-25.5539f,-20.6802f,  -48.4478f,-24.5415f,-19.9253f,  -48.9957f,-23.7475f,-21.6779f,  
	-49.9950f,-22.6123f,-20.8442f,  -50.7538f,-21.9753f,-22.5822f,  -51.7555f,-20.7056f,-21.6628f,  -52.7242f,-20.2197f,-23.3444f,  -53.7222f,-18.8055f,-22.3336f,  
	-54.8935f,-18.4628f,-23.9187f,  -55.8787f,-16.8979f,-22.8133f,  -57.2396f,-16.6894f,-24.2647f,  -58.2023f,-14.9679f,-23.0614f,  -59.7341f,-14.8841f,-24.3455f,  
	-60.6624f,-13.0044f,-23.0438f,  -62.3414f,-13.0354f,-24.1311f,  -63.2230f,-10.9975f,-22.7314f,  -65.0211f,-11.1339f,-23.5975f,  -65.8427f,-8.9416f,-22.1023f,  
	-67.7286f,-9.1742f,-22.7280f,  -68.4772f,-6.8330f,-21.1408f,  -70.4166f,-7.1534f,-21.5126f,  -71.0794f,-4.6712f,-19.8381f,  -73.0362f,-5.0721f,-19.9487f,  
	-73.6011f,-2.4594f,-18.1933f,  -75.5383f,-2.9347f,-18.0414f,  -75.9938f,-0.2036f,-16.2122f,  -77.8747f,-0.7481f,-15.8024f,  -78.2104f,2.0863f,-13.9083f,  
	-79.9989f,1.4765f,-13.2515f,  -80.2059f,4.3992f,-11.3013f,  -81.8680f,3.7269f,-10.4138f,  -81.9388f,6.7205f,-8.4178f,  -83.4429f,5.9875f,-7.3212f,  
	-83.3718f,9.0329f,-5.2907f,  -84.6899f,8.2402f,-4.0115f,  -84.4726f,11.3193f,-1.9570f,  -85.5808f,10.4672f,-0.5259f,  -85.2154f,13.5590f,1.5401f,  
	-86.0944f,12.6473f,3.0887f,  -85.5805f,15.7316f,5.1542f,  -86.2163f,14.7599f,6.7832f,  -85.5556f,17.8172f,8.8369f,  -85.9396f,16.7853f,10.5072f,  
	-85.1359f,19.7946f,12.5364f,  -85.2653f,18.7026f,14.2076f,  -84.3247f,21.6436f,16.2002f,  -84.2024f,20.4924f,17.8318f,  -83.1323f,23.3471f,19.7772f,  
	-82.7670f,22.1387f,21.3290f,  -81.5781f,24.8863f,23.2145f,  -80.9841f,23.6240f,24.6483f,  -79.6876f,26.2487f,26.4641f,  -78.8843f,24.9374f,27.7437f,  
	-77.4945f,27.4221f,29.4792f,  -76.5058f,26.0691f,30.5718f,  -75.0382f,28.3998f,32.2180f,  -73.8922f,27.0142f,33.0949f,  -72.3636f,29.1786f,34.6436f,  
	-71.0915f,27.7723f,35.2807f,  -69.5208f,29.7598f,36.7243f,  -68.1559f,28.3472f,37.1031f,  -66.5629f,30.1505f,38.4356f,  -65.1397f,28.7487f,38.5436f,  
	-63.5441f,30.3635f,39.7606f,  -62.0976f,28.9920f,39.5913f,  -60.5197f,30.4173f,40.6898f,  -59.0833f,29.0983f,40.2436f,  -57.5425f,30.3366f,41.2227f,  
	-56.1476f,29.0941f,40.5067f,  -54.6627f,30.1507f,41.3664f,  -36.2231f,-54.8909f,-20.3915f,  -37.9670f,-54.6972f,-19.9387f,  -37.5482f,-53.1144f,-18.7891f,  
	-39.1916f,-52.8411f,-18.3298f,  -38.7013f,-51.1217f,-17.4323f,  -40.2406f,-50.7798f,-16.9689f,  -39.6871f,-48.9623f,-16.3423f,  -41.1223f,-48.5632f,-15.8767f,  
	-40.5202f,-46.6866f,-15.5330f,  -41.8585f,-46.2374f,-15.0642f,  -41.2277f,-44.3397f,-15.0089f,  -42.4777f,-43.8467f,-14.5351f,  -41.8429f,-41.9637f,-14.7664f,  
	-43.0176f,-41.4291f,-14.2837f,  -42.4069f,-39.5934f,-14.7929f,  -43.5198f,-39.0161f,-14.2960f,  -42.9633f,-37.2563f,-15.0678f,  -44.0282f,-36.6324f,-14.5499f,  
	-43.5568f,-34.9727f,-15.5626f,  -44.5878f,-34.2944f,-15.0157f,  -44.2317f,-32.7543f,-16.2418f,  -45.2404f,-32.0119f,-15.6573f,  -45.0275f,-30.6060f,-17.0645f,  
	-46.0237f,-29.7877f,-16.4332f,  -45.9787f,-28.5263f,-17.9852f,  -46.9689f,-27.6196f,-17.2983f,  -47.1122f,-26.5086f,-18.9557f,  -48.1010f,-25.5000f,-18.2039f,  
	-48.4478f,-24.5415f,-19.9253f,  -49.4351f,-23.4192f,-19.1014f,  -49.9950f,-22.6123f,-20.8442f,  -50.9789f,-21.3645f,-19.9410f,  -51.7555f,-20.7056f,-21.6628f,  
	-52.7316f,-19.3221f,-20.6741f,  -53.7222f,-18.8055f,-22.3336f,  -54.6828f,-17.2794f,-21.2556f,  -55.8787f,-16.8979f,-22.8133f,  -56.8156f,-15.2224f,-21.6421f,  
	-58.2023f,-14.9679f,-23.0614f,  -59.1048f,-13.1405f,-21.7959f,  -60.6624f,-13.0044f,-23.0438f,  -61.5191f,-11.0238f,-21.6834f,  -63.2230f,-10.9975f,-22.7314f,  
	-64.0215f,-8.8666f,-21.2780f,  -65.8427f,-8.9416f,-22.1023f,  -66.5707f,-6.6645f,-20.5585f,  -68.4772f,-6.8330f,-21.1408f,  -69.1224f,-4.4163f,-19.5105f,  
	-71.0794f,-4.6712f,-19.8381f,  -71.6299f,-2.1241f,-18.1268f,  -73.6011f,-2.4594f,-18.1933f,  -74.0456f,0.2069f,-16.4074f,  -75.9938f,-0.2036f,-16.2122f,  
	-76.3220f,2.5685f,-14.3597f,  -78.2104f,2.0863f,-13.9083f,  -78.4132f,4.9502f,-11.9975f,  -80.2059f,4.3992f,-11.3013f,  -80.2755f,7.3384f,-9.3417f,  
	-81.9388f,6.7205f,-8.4178f,  -81.8688f,9.7171f,-6.4199f,  -83.3718f,9.0329f,-5.2907f,  -83.1575f,12.0696f,-3.2645f,  -84.4726f,11.3193f,-1.9570f,  
	-84.1113f,14.3760f,0.0855f,  -85.2154f,13.5590f,1.5401f,  -84.7059f,16.6162f,3.5875f,  -85.5805f,15.7316f,5.1542f,  -84.9237f,18.7703f,7.1954f,  
	-85.5556f,17.8172f,8.8369f,  -84.7545f,20.8168f,10.8595f,  -85.1359f,19.7946f,12.5364f,  -84.1955f,22.7351f,14.5286f,  -84.3247f,21.6436f,16.2002f,  
	-83.2513f,24.5070f,18.1515f,  -83.1323f,23.3471f,19.7772f,  -81.9356f,26.1125f,21.6748f,  -81.5781f,24.8863f,23.2145f,  -80.2685f,27.5376f,25.0487f,  
	-79.6876f,26.2487f,26.4641f,  -78.2784f,28.7681f,28.2239f,  -77.4945f,27.4221f,29.4792f,  -76.0003f,29.7951f,31.1552f,  -75.0382f,28.3998f,32.2180f,  
	-73.4754f,30.6129f,33.8016f,  -72.3636f,29.1786f,34.6436f,  -70.7506f,31.2201f,36.1265f,  -69.5208f,29.7598f,36.7243f,  -67.8771f,31.6209f,38.0998f,  
	-66.5629f,30.1505f,38.4356f,  -64.9084f,31.8254f,39.6985f,  -63.5441f,30.3635f,39.7606f,  -61.8998f,31.8493f,40.9068f,  -60.5197f,30.4173f,40.6898f,  
	-58.9055f,31.7148f,41.7173f,  -57.5425f,30.3366f,41.2227f,  -55.9788f,31.4493f,42.1305f,  -54.6627f,30.1507f,41.3664f,  -53.1659f,31.0878f,42.1570f,  
	-37.9670f,-54.6972f,-19.9387f,  -39.7177f,-54.4175f,-19.4434f,  -39.1916f,-52.8411f,-18.3298f,  -40.8392f,-52.4847f,-17.8274f,  -40.2406f,-50.7798f,-16.9689f,  
	-41.7819f,-50.3587f,-16.4625f,  -41.1223f,-48.5632f,-15.8767f,  -42.5616f,-48.0858f,-15.3661f,  -41.8585f,-46.2374f,-15.0642f,  -43.2019f,-45.7115f,-14.5491f,  
	-42.4777f,-43.8467f,-14.5351f,  -43.7361f,-43.2763f,-14.0126f,  -43.0176f,-41.4291f,-14.2837f,  -44.2029f,-40.8152f,-13.7497f,  -43.5198f,-39.0161f,-14.2960f,  
	-44.6449f,-38.3568f,-13.7449f,  -44.0282f,-36.6324f,-14.5499f,  -45.1069f,-35.9220f,-13.9747f,  -44.5878f,-34.2944f,-15.0157f,  -45.6319f,-33.5249f,-14.4084f,  
	-45.2404f,-32.0119f,-15.6573f,  -46.2603f,-31.1729f,-15.0093f,  -46.0237f,-29.7877f,-16.4332f,  -47.0269f,-28.8681f,-15.7362f,  -46.9689f,-27.6196f,-17.2983f,  
	-47.9612f,-26.6066f,-16.5432f,  -48.1010f,-25.5000f,-18.2039f,  -49.0836f,-24.3823f,-17.3835f,  -49.4351f,-23.4192f,-19.1014f,  -50.4078f,-22.1852f,-18.2082f,  
	-50.9789f,-21.3645f,-19.9410f,  -51.9388f,-20.0039f,-18.9686f,  -52.7316f,-19.3221f,-20.6741f,  -53.6724f,-17.8274f,-19.6183f,  -54.6828f,-17.2794f,-21.2556f,  
	-55.5977f,-15.6433f,-20.1119f,  -56.8156f,-15.2224f,-21.6421f,  -57.6951f,-13.4414f,-20.4086f,  -59.1048f,-13.1405f,-21.7959f,  -59.9388f,-11.2124f,-20.4714f,  
	-61.5191f,-11.0238f,-21.6834f,  -62.2960f,-8.9505f,-20.2691f,  -64.0215f,-8.8666f,-21.2780f,  -64.7297f,-6.6510f,-19.7757f,  -66.5707f,-6.6645f,-20.5585f,  
	-67.1987f,-4.3120f,-18.9713f,  -69.1224f,-4.4163f,-19.5105f,  -69.6588f,-1.9346f,-17.8429f,  -71.6299f,-2.1241f,-18.1268f,  -72.0635f,0.4766f,-16.3848f,  
	-74.0456f,0.2069f,-16.4074f,  -74.3661f,2.9148f,-14.5978f,  -76.3220f,2.5685f,-14.3597f,  -76.5203f,5.3703f,-12.4902f,  -78.4132f,4.9502f,-11.9975f,  
	-78.4812f,7.8307f,-10.0769f,  -80.2755f,7.3384f,-9.3417f,  -80.2066f,10.2810f,-7.3797f,  -81.8688f,9.7171f,-6.4199f,  -81.6579f,12.7053f,-4.4260f,  
	-83.1575f,12.0696f,-3.2645f,  -82.8010f,15.0843f,-1.2500f,  -84.1113f,14.3760f,0.0855f,  -83.6076f,17.3985f,2.1097f,  -84.7059f,16.6162f,3.5875f,  
	-84.0549f,19.6281f,5.6107f,  -84.9237f,18.7703f,7.1954f,  -84.1274f,21.7516f,9.2058f,  -84.7545f,20.8168f,10.8595f,  -83.8168f,23.7480f,12.8456f,  
	-84.1955f,22.7351f,14.5286f,  -83.1219f,25.5983f,16.4799f,  -83.2513f,24.5070f,18.1515f,  -82.0505f,27.2815f,20.0554f,  -81.9356f,26.1125f,21.6748f,  
	-80.6168f,28.7821f,23.5216f,  -80.2685f,27.5376f,25.0487f,  -78.8441f,30.0840f,26.8272f,  -78.2784f,28.7681f,28.2239f,  -76.7621f,31.1762f,29.9247f,  
	-76.0003f,29.7951f,31.1552f,  -74.4077f,32.0506f,32.7692f,  -73.4754f,30.6129f,33.8016f,  -71.8242f,32.7030f,35.3200f,  -70.7506f,31.2201f,36.1265f,  
	-69.0601f,33.1346f,37.5418f,  -67.8771f,31.6209f,38.0998f,  -66.1670f,33.3527f,39.4062f,  -64.9084f,31.8254f,39.6985f,  -63.1999f,33.3697f,40.8908f,  
	-61.8998f,31.8493f,40.9068f,  -60.2135f,33.2052f,41.9820f,  -58.9055f,31.7148f,41.7173f,  -57.2630f,32.8840f,42.6734f,  -55.9788f,31.4493f,42.1305f,  
	-54.3979f,32.4390f,42.9685f,  -53.1659f,31.0878f,42.1570f,  -51.6645f,31.9070f,42.8786f,  -39.7177f,-54.4175f,-19.4434f,  -41.4727f,-54.0479f,-18.9031f,  
	-40.8392f,-52.4847f,-17.8274f,  -42.4875f,-52.0425f,-17.2804f,  -41.7819f,-50.3587f,-16.4625f,  -43.3251f,-49.8533f,-15.9095f,  -42.5616f,-48.0858f,-15.3661f,  
	-44.0031f,-47.5262f,-14.8081f,  -43.2019f,-45.7115f,-14.5491f,  -44.5501f,-45.1036f,-13.9843f,  -43.7361f,-43.2763f,-14.0126f,  -45.0012f,-42.6230f,-13.4383f,  
	-44.2029f,-40.8152f,-13.7497f,  -45.3966f,-40.1165f,-13.1614f,  -44.6449f,-38.3568f,-13.7449f,  -45.7802f,-37.6091f,-13.1365f,  -45.1069f,-35.9220f,-13.9747f,  
	-46.1955f,-35.1190f,-13.3392f,  -45.6319f,-33.5249f,-14.4084f,  -46.6845f,-32.6582f,-13.7381f,  -46.2603f,-31.1729f,-15.0093f,  -47.2852f,-30.2326f,-14.2962f,  
	-47.0269f,-28.8681f,-15.7362f,  -48.0307f,-27.8425f,-14.9716f,  -47.9612f,-26.6066f,-16.5432f,  -48.9466f,-25.4851f,-15.7200f,  -49.0836f,-24.3823f,-17.3835f,  
	-50.0519f,-23.1536f,-16.4942f,  -50.4078f,-22.1852f,-18.2082f,  -51.3573f,-20.8390f,-17.2462f,  -51.9388f,-20.0039f,-18.9686f,  -52.8648f,-18.5326f,-17.9293f,  
	-53.6724f,-17.8274f,-19.6183f,  -54.5694f,-16.2233f,-18.4968f,  -55.5977f,-15.6433f,-20.1119f,  -56.4576f,-13.9021f,-18.9058f,  -57.6951f,-13.4414f,-20.4086f,  
	-58.5089f,-11.5600f,-19.1161f,  -59.9388f,-11.2124f,-20.4714f,  -60.6958f,-9.1919f,-19.0930f,  -62.2960f,-8.9505f,-20.2691f,  -62.9858f,-6.7926f,-18.8058f,  
	-64.7297f,-6.6510f,-19.7757f,  -65.3413f,-4.3599f,-18.2299f,  -67.1987f,-4.3120f,-18.9713f,  -67.7212f,-1.8941f,-17.3466f,  -69.6588f,-1.9346f,-17.8429f,  
	-70.0813f,0.6008f,-16.1446f,  -72.0635f,0.4766f,-16.3848f,  -72.3762f,3.1194f,-14.6187f,  -74.3661f,2.9148f,-14.5978f,  -74.5598f,5.6524f,-12.7712f,  
	-76.5203f,5.3703f,-12.4902f,  -76.5869f,8.1890f,-10.6109f,  -78.4812f,7.8307f,-10.0769f,  -78.4137f,10.7151f,-8.1537f,  -80.2066f,10.2810f,-7.3797f,  
	-79.9994f,13.2154f,-5.4218f,  -81.6579f,12.7053f,-4.4260f,  -81.3070f,15.6718f,-2.4437f,  -82.8010f,15.0843f,-1.2500f,  -82.3042f,18.0652f,0.7464f,  
	-83.6076f,17.3985f,2.1097f,  -82.9638f,20.3761f,4.1099f,  -84.0549f,19.6281f,5.6107f,  -83.2654f,22.5832f,7.6034f,  -84.1274f,21.7516f,9.2058f,  
	-83.1953f,24.6651f,11.1799f,  -83.8168f,23.7480f,12.8456f,  -82.7464f,26.6025f,14.7909f,  -83.1219f,25.5983f,16.4799f,  -81.9207f,28.3734f,18.3841f,  
	-82.0505f,27.2815f,20.0554f,  -80.7268f,29.9609f,21.9090f,  -80.6168f,28.7821f,23.5216f,  -79.1819f,31.3473f,25.3133f,  -78.8441f,30.0840f,26.8272f,  
	-77.3105f,32.5196f,28.5475f,  -76.7621f,31.1762f,29.9247f,  -75.1446f,33.4672f,31.5640f,  -74.4077f,32.0506f,32.7692f,  -72.7232f,34.1832f,34.3186f,  
	-71.8242f,32.7030f,35.3200f,  -70.0915f,34.6658f,36.7712f,  -69.0601f,33.1346f,37.5418f,  -67.2984f,34.9192f,38.8885f,  -66.1670f,33.3527f,39.4062f,  
	-64.3978f,34.9525f,40.6422f,  -63.1999f,33.3697f,40.8908f,  -61.4441f,34.7822f,42.0123f,  -60.2135f,33.2052f,41.9820f,  -58.4934f,34.4301f,42.9857f,  
	-57.2630f,32.8840f,42.6734f,  -55.5979f,33.9267f,43.5591f,  -54.3979f,32.4390f,42.9685f,  -52.8073f,33.3067f,43.7370f,  -51.6645f,31.9070f,42.8786f,  
	-50.1643f,32.6115f,43.5330f,  -41.4727f,-54.0479f,-18.9031f,  -43.2272f,-53.5858f,-18.3163f,  -42.4875f,-52.0425f,-17.2804f,  -44.1357f,-51.5096f,-16.6851f,  
	-43.3251f,-49.8533f,-15.9095f,  -44.8675f,-49.2598f,-15.3077f,  -44.0031f,-47.5262f,-14.8081f,  -45.4455f,-46.8795f,-14.1994f,  -44.5501f,-45.1036f,-13.9843f,  
	-45.9008f,-44.4084f,-13.3671f,  -45.0012f,-42.6230f,-13.4383f,  -46.2703f,-41.8814f,-12.8093f,  -45.3966f,-40.1165f,-13.1614f,  -46.5961f,-39.3269f,-12.5156f,  
	-45.7802f,-37.6091f,-13.1365f,  -46.9214f,-36.7672f,-12.4681f,  -46.1955f,-35.1190f,-13.3392f,  -47.2891f,-34.2181f,-12.6411f,  -46.6845f,-32.6582f,-13.7381f,  
	-47.7393f,-31.6897f,-13.0029f,  -47.2852f,-30.2326f,-14.2962f,  -48.3085f,-29.1864f,-13.5160f,  -48.0307f,-27.8425f,-14.9716f,  -49.0265f,-26.7087f,-14.1394f,  
	-48.9466f,-25.4851f,-15.7200f,  -49.9172f,-24.2531f,-14.8284f,  -50.0519f,-23.1536f,-16.4942f,  -50.9971f,-21.8133f,-15.5365f,  -51.3573f,-20.8390f,-17.2462f,  
	-52.2738f,-19.3829f,-16.2176f,  -52.8648f,-18.5326f,-17.9293f,  -53.7484f,-16.9525f,-16.8247f,  -54.5694f,-16.2233f,-18.4968f,  -55.4133f,-14.5145f,-17.3132f,  
	-56.4576f,-13.9021f,-18.9058f,  -57.2541f,-12.0608f,-17.6407f,  -58.5089f,-11.5600f,-19.1161f,  -59.2483f,-9.5866f,-17.7697f,  -60.6958f,-9.1919f,-19.0930f,  
	-61.3686f,-7.0869f,-17.6655f,  -62.9858f,-6.7926f,-18.8058f,  -63.5819f,-4.5593f,-17.2989f,  -65.3413f,-4.3599f,-18.2299f,  -65.8504f,-2.0033f,-16.6462f,  
	-67.7212f,-1.8941f,-17.3466f,  -68.1330f,0.5774f,-15.6910f,  -70.0813f,0.6008f,-16.1446f,  -70.3863f,3.1785f,-14.4220f,  -72.3762f,3.1194f,-14.6187f,  
	-72.5652f,5.7918f,-12.8355f,  -74.5598f,5.6524f,-12.7712f,  -74.6248f,8.4071f,-10.9345f,  -76.5869f,8.1890f,-10.6109f,  -76.5207f,11.0118f,-8.7288f,  
	-78.4137f,10.7151f,-8.1537f,  -78.2105f,13.5912f,-6.2346f,  -79.9994f,13.2154f,-5.4218f,  -79.6549f,16.1284f,-3.4752f,  -81.3070f,15.6718f,-2.4437f,  
	-80.8181f,18.6048f,-0.4794f,  -82.3042f,18.0652f,0.7464f,  -81.6693f,21.0014f,2.7188f,  -82.9638f,20.3761f,4.1099f,  -82.1833f,23.2972f,6.0798f,  
	-83.2654f,22.5832f,7.6034f,  -82.3415f,25.4708f,9.5600f,  -83.1953f,24.6651f,11.1799f,  -82.1312f,27.5024f,13.1134f,  -82.7464f,26.6025f,14.7909f,  
	-81.5487f,29.3694f,16.6895f,  -81.9207f,28.3734f,18.3841f,  -80.5964f,31.0537f,20.2384f,  -80.7268f,29.9609f,21.9090f,  -79.2860f,32.5364f,23.7078f,  
	-79.1819f,31.3473f,25.3133f,  -77.6361f,33.8022f,27.0471f,  -77.3105f,32.5196f,28.5475f,  -75.6734f,34.8384f,30.2067f,  -75.1446f,33.4672f,31.5640f,  
	-73.4323f,35.6354f,33.1393f,  -72.7232f,34.1832f,34.3186f,  -70.9536f,36.1883f,35.8013f,  -70.0915f,34.6658f,36.7712f,  -68.2832f,36.4982f,38.1544f,  
	-67.2984f,34.9192f,38.8885f,  -65.4729f,36.5707f,40.1651f,  -64.3978f,34.9525f,40.6422f,  -62.5760f,36.4189f,41.8076f,  -61.4441f,34.7822f,42.0123f,  
	-59.6489f,36.0614f,43.0621f,  -58.4934f,34.4301f,42.9857f,  -56.7455f,35.5254f,43.9187f,  -55.5979f,33.9267f,43.5591f,  -53.9180f,34.8436f,44.3748f,  
	-52.8073f,33.3067f,43.7370f,  -51.2132f,34.0552f,44.4372f,  -50.1643f,32.6115f,43.5330f,  -48.6703f,33.2053f,44.1219f,  -43.2272f,-53.5858f,-18.3163f,  
	-44.9798f,-53.0265f,-17.6794f,  -44.1357f,-51.5096f,-16.6851f,  -45.7804f,-50.8826f,-16.0394f,  -44.8675f,-49.2598f,-15.3077f,  -46.4072f,-48.5738f,-14.6540f,  
	-45.4455f,-46.8795f,-14.1994f,  -46.8864f,-46.1410f,-13.5373f,  -45.9008f,-44.4084f,-13.3671f,  -47.2511f,-43.6213f,-12.6946f,  -46.2703f,-41.8814f,-12.8093f,  
	-47.5407f,-41.0460f,-12.1226f,  -46.5961f,-39.3269f,-12.5156f,  -47.7972f,-38.4410f,-11.8099f,  -46.9214f,-36.7672f,-12.4681f,  -48.0636f,-35.8259f,-11.7372f,  
	-47.2891f,-34.2181f,-12.6411f,  -48.3814f,-33.2147f,-11.8785f,  -47.7393f,-31.6897f,-13.0029f,  -48.7897f,-30.6154f,-12.2013f,  -48.3085f,-29.1864f,-13.5160f,  
	-49.3218f,-28.0322f,-12.6686f,  -49.0265f,-26.7087f,-14.1394f,  -50.0062f,-25.4649f,-13.2391f,  -49.9172f,-24.2531f,-14.8284f,  -50.8643f,-22.9101f,-13.8687f,  
	-50.9971f,-21.8133f,-15.5365f,  -51.9094f,-20.3637f,-14.5125f,  -52.2738f,-19.3829f,-16.2176f,  -53.1486f,-17.8185f,-15.1239f,  -53.7484f,-16.9525f,-16.8247f,  
	-54.5802f,-15.2683f,-15.6580f,  -55.4133f,-14.5145f,-17.3132f,  -56.1956f,-12.7061f,-16.0706f,  -57.2541f,-12.0608f,-17.6407f,  -57.9783f,-10.1278f,-16.3219f,  
	-59.2483f,-9.5866f,-17.7697f,  -59.9060f,-7.5288f,-16.3741f,  -61.3686f,-7.0869f,-17.6655f,  -61.9505f,-4.9066f,-16.1943f,  -63.5819f,-4.5593f,-17.2989f,  
	-64.0785f,-2.2603f,-15.7540f,  -65.8504f,-2.0033f,-16.6462f,  -66.2519f,0.4068f,-15.0319f,  -68.1330f,0.5774f,-15.6910f,  -68.4304f,3.0914f,-14.0110f,  
	-70.3863f,3.1785f,-14.4220f,  -70.5706f,5.7859f,-12.6822f,  -72.5652f,5.7918f,-12.8355f,  -72.6286f,8.4814f,-11.0422f,  -74.6248f,8.4071f,-10.9345f,  
	-74.5601f,11.1661f,-9.0949f,  -76.5207f,11.0118f,-8.7288f,  -76.3218f,13.8264f,-6.8507f,  -78.2105f,13.5912f,-6.2346f,  -77.8728f,16.4463f,-4.3267f,  
	-79.6549f,16.1284f,-3.4752f,  -79.1747f,19.0082f,-1.5465f,  -80.8181f,18.6048f,-0.4794f,  -80.1934f,21.4933f,1.4610f,  -81.6693f,21.0014f,2.7188f,  
	-80.8997f,23.8814f,4.6610f,  -82.1833f,23.2972f,6.0798f,  -81.2700f,26.1512f,8.0136f,  -82.3415f,25.4708f,9.5600f,  -81.2868f,28.2825f,11.4762f,  
	-82.1312f,27.5024f,13.1134f,  -80.9405f,30.2525f,15.0006f,  -81.5487f,29.3694f,16.6895f,  -80.2279f,32.0421f,18.5385f,  -80.5964f,31.0537f,20.2384f,  
	-79.1547f,33.6308f,22.0384f,  -79.2860f,32.5364f,23.7078f,  -77.7334f,35.0021f,25.4494f,  -77.6361f,33.8022f,27.0471f,  -75.9852f,36.1407f,28.7205f,  
	-75.6734f,34.8384f,30.2067f,  -73.9392f,37.0347f,31.8025f,  -73.4323f,35.6354f,33.1393f,  -71.6318f,37.6760f,34.6485f,  -70.9536f,36.1883f,35.8013f,  
	-69.1047f,38.0627f,37.2164f,  -68.2832f,36.4982f,38.1544f,  -66.4070f,38.1965f,39.4678f,  -65.4729f,36.5707f,40.1651f,  -63.5901f,38.0873f,41.3714f,  
	-62.5760f,36.4189f,41.8076f,  -60.7098f,37.7498f,42.9014f,  -59.6489f,36.0614f,43.0621f,  -57.8210f,37.2078f,44.0410f,  -56.7455f,35.5254f,43.9187f,  
	-54.9775f,36.4911f,44.7811f,  -53.9180f,34.8436f,44.3748f,  -52.2298f,35.6366f,45.1213f,  -51.2132f,34.0552f,44.4372f,  -49.6212f,34.6876f,45.0708f,  
	-48.6703f,33.2053f,44.1219f,  -47.1873f,33.6923f,44.6474f,  -44.9798f,-53.0265f,-17.6794f,  -46.7262f,-52.3667f,-16.9905f,  -45.7804f,-50.8826f,-16.0394f,  
	-47.4187f,-50.1574f,-15.3406f,  -46.4072f,-48.5738f,-14.6540f,  -47.9411f,-47.7910f,-13.9460f,  -46.8864f,-46.1410f,-13.5373f,  -48.3221f,-45.3063f,-12.8194f,  
	-47.2511f,-43.6213f,-12.6946f,  -48.5978f,-42.7370f,-11.9642f,  -47.5407f,-41.0460f,-12.1226f,  -48.8079f,-40.1120f,-11.3759f,  -47.7972f,-38.4410f,-11.8099f,  
	-48.9949f,-37.4542f,-11.0419f,  -48.0636f,-35.8259f,-11.7372f,  -49.2007f,-34.7813f,-10.9422f,  -48.3814f,-33.2147f,-11.8785f,  -49.4661f,-32.1050f,-11.0499f,  
	-48.7897f,-30.6154f,-12.2013f,  -49.8276f,-29.4329f,-11.3328f,  -49.3218f,-28.0322f,-12.6686f,  -50.3172f,-26.7682f,-11.7536f,  -50.0062f,-25.4649f,-13.2391f,  
	-50.9612f,-24.1105f,-12.2713f,  -50.8643f,-22.9101f,-13.8687f,  -51.7780f,-21.4583f,-12.8432f,  -51.9094f,-20.3637f,-14.5125f,  -52.7804f,-18.8065f,-13.4238f,  
	-53.1486f,-17.8185f,-15.1239f,  -53.9725f,-16.1506f,-13.9684f,  -54.5802f,-15.2683f,-15.6580f,  -55.3518f,-13.4849f,-14.4324f,  -56.1956f,-12.7061f,-16.0706f,  
	-56.9075f,-10.8064f,-14.7743f,  -57.9783f,-10.1278f,-16.3219f,  -58.6230f,-8.1109f,-14.9539f,  -59.9060f,-7.5288f,-16.3741f,  -60.4752f,-5.3961f,-14.9349f,  
	-61.9505f,-4.9066f,-16.1943f,  -62.4356f,-2.6607f,-14.6849f,  -64.0785f,-2.2603f,-15.7540f,  -64.4703f,0.0918f,-14.1783f,  -66.2519f,0.4068f,-15.0319f,  
	-66.5419f,2.8595f,-13.3927f,  -68.4304f,3.0914f,-14.0110f,  -68.6101f,5.6350f,-12.3137f,  -70.5706f,5.7859f,-12.6822f,  -70.6323f,8.4105f,-10.9323f,  
	-72.6286f,8.4814f,-11.0422f,  -72.5653f,11.1754f,-9.2460f,  -74.5601f,11.1661f,-9.0949f,  -74.3656f,13.9169f,-7.2595f,  -76.3218f,13.8264f,-6.8507f,  
	-75.9912f,16.6201f,-4.9838f,  -77.8728f,16.4463f,-4.3267f,  -77.4021f,19.2683f,-2.4367f,  -79.1747f,19.0082f,-1.5465f,  -78.5614f,21.8434f,0.3583f,  
	-80.1934f,21.4933f,1.4610f,  -79.4363f,24.3258f,3.3713f,  -80.8997f,23.8814f,4.6610f,  -79.9992f,26.6946f,6.5673f,  -81.2700f,26.1512f,8.0136f,  
	-80.2277f,28.9295f,9.9072f,  -81.2868f,28.2825f,11.4762f,  -80.1068f,31.0076f,13.3463f,  -80.9405f,30.2525f,15.0006f,  -79.6277f,32.9089f,16.8384f,  
	-80.2279f,32.0421f,18.5385f,  -78.7901f,34.6120f,20.3336f,  -79.1547f,33.6308f,22.0384f,  -77.6007f,36.0987f,23.7815f,  -77.7334f,35.0021f,25.4494f,  
	-76.0746f,37.3519f,27.1308f,  -75.9852f,36.1407f,28.7205f,  -74.2354f,38.3571f,30.3309f,  -73.9392f,37.0347f,31.8025f,  -72.1144f,39.1035f,33.3326f,  
	-71.6318f,37.6760f,34.6485f,  -69.7487f,39.5858f,36.0906f,  -69.1047f,38.0627f,37.2164f,  -67.1840f,39.8022f,38.5622f,  -66.4070f,38.1965f,39.4678f,  
	-64.4690f,39.7588f,40.7112f,  -63.5901f,38.0873f,41.3714f,  -61.6580f,39.4665f,42.5063f,  -60.7098f,37.7498f,42.9014f,  -58.8059f,38.9451f,43.9240f,  
	-57.8210f,37.2078f,44.0410f,  -55.9677f,38.2212f,44.9489f,  -54.9775f,36.4911f,44.7811f,  -53.1965f,37.3286f,45.5736f,  -52.2298f,35.6366f,45.1213f,  
	-50.5394f,36.3085f,45.8002f,  -49.6212f,34.6876f,45.0708f,  -48.0368f,35.2076f,45.6395f,  -47.1873f,33.6923f,44.6474f,  -45.7207f,34.0763f,45.1108f,  
	-46.7262f,-52.3667f,-16.9905f,  -48.4626f,-51.6031f,-16.2472f,  -47.4187f,-50.1574f,-15.3406f,  -49.0469f,-49.3303f,-14.5863f,  -47.9411f,-47.7910f,-13.9460f,  
	-49.4652f,-46.9077f,-13.1814f,  -48.3221f,-45.3063f,-12.8194f,  -49.7493f,-44.3710f,-12.0430f,  -48.5978f,-42.7370f,-11.9642f,  -49.9363f,-41.7515f,-11.1735f,  
	-48.8079f,-40.1120f,-11.3759f,  -50.0670f,-39.0751f,-10.5670f,  -48.9949f,-37.4542f,-11.0419f,  -50.1831f,-36.3627f,-10.2100f,  -49.2007f,-34.7813f,-10.9422f,  
	-50.3262f,-33.6297f,-10.0816f,  -49.4661f,-32.1050f,-11.0499f,  -50.5351f,-30.8870f,-10.1550f,  -49.8276f,-29.4329f,-11.3328f,  -50.8450f,-28.1409f,-10.3974f,  
	-50.3172f,-26.7682f,-11.7536f,  -51.2861f,-25.3941f,-10.7717f,  -50.9612f,-24.1105f,-12.2713f,  -51.8818f,-22.6479f,-11.2381f,  -51.7780f,-21.4583f,-12.8432f,  
	-52.6499f,-19.8994f,-11.7533f,  -52.7804f,-18.8065f,-13.4238f,  -53.6006f,-17.1461f,-12.2736f,  -53.9725f,-16.1506f,-13.9684f,  -54.7369f,-14.3838f,-12.7543f,  
	-55.3518f,-13.4849f,-14.4324f,  -56.0541f,-11.6107f,-13.1535f,  -56.9075f,-10.8064f,-14.7743f,  -57.5414f,-8.8231f,-13.4291f,  -58.6230f,-8.1109f,-14.9539f,  
	-59.1812f,-6.0192f,-13.5423f,  -60.4752f,-5.3961f,-14.9349f,  -60.9500f,-3.1978f,-13.4575f,  -62.4356f,-2.6607f,-14.6849f,  -62.8185f,-0.3621f,-13.1450f,  
	-64.4703f,0.0918f,-14.1783f,  -64.7533f,2.4866f,-12.5778f,  -66.5419f,2.8595f,-13.3927f,  -66.7172f,5.3415f,-11.7365f,  -68.6101f,5.6350f,-12.3137f,  
	-68.6702f,8.1957f,-10.6064f,  -70.6323f,8.4105f,-10.9323f,  -70.5705f,11.0395f,-9.1793f,  -72.5653f,11.1754f,-9.2460f,  -72.3753f,13.8611f,-7.4539f,  
	-74.3656f,13.9169f,-7.2595f,  -74.0425f,16.6468f,-5.4352f,  -75.9912f,16.6201f,-4.9838f,  -75.5307f,19.3808f,-3.1348f,  -77.4021f,19.2683f,-2.4367f,  
	-76.8013f,22.0458f,-0.5707f,  -78.5614f,21.8434f,0.3583f,  -77.8184f,24.6229f,2.2328f,  -79.4363f,24.3258f,3.3713f,  -78.5508f,27.0918f,5.2457f,  
	-79.9992f,26.6946f,6.5673f,  -78.9719f,29.4324f,8.4333f,  -80.2277f,28.9295f,9.9072f,  -79.0618f,31.6218f,11.7548f,  -80.1068f,31.0076f,13.3463f,  
	-78.8060f,33.6394f,15.1671f,  -79.6277f,32.9089f,16.8384f,  -78.1986f,35.4631f,18.6225f,  -78.7901f,34.6120f,20.3336f,  -77.2402f,37.0733f,22.0720f,  
	-77.6007f,36.0987f,23.7815f,  -75.9401f,38.4512f,25.4649f,  -76.0746f,37.3519f,27.1308f,  -74.3158f,39.5801f,28.7498f,  -74.2354f,38.3571f,30.3309f,  
	-72.3932f,40.4463f,31.8761f,  -72.1144f,39.1035f,33.3326f,  -70.2043f,41.0415f,34.7963f,  -69.7487f,39.5858f,36.0906f,  -67.7906f,41.3603f,37.4638f,  
	-67.1840f,39.8022f,38.5622f,  -65.1975f,41.4048f,39.8383f,  -64.4690f,39.7588f,40.7112f,  -62.4771f,41.1821f,41.8834f,  -61.6580f,39.4665f,42.5063f,  
	-59.6835f,40.7077f,43.5697f,  -58.8059f,38.9451f,43.9240f,  -56.8717f,40.0043f,44.8754f,  -55.9677f,38.2212f,44.9489f,  -54.0968f,39.1024f,45.7863f,  
	-53.1965f,37.3286f,45.5736f,  -51.4089f,38.0401f,46.2975f,  -50.5394f,36.3085f,45.8002f,  -48.8529f,36.8623f,46.4128f,  -48.0368f,35.2076f,45.6395f,  
	-46.4659f,35.6184f,46.1447f,  -45.7207f,34.0763f,45.1108f,  -44.2737f,34.3624f,45.5149f,  -48.4626f,-51.6031f,-16.2472f,  -50.1849f,-50.7324f,-15.4471f,  
	-49.0469f,-49.3303f,-14.5863f,  -50.6606f,-48.3981f,-13.7745f,  -49.4652f,-46.9077f,-13.1814f,  -50.9755f,-45.9200f,-12.3578f,  -49.7493f,-44.3710f,-12.0430f,  
	-51.1630f,-43.3315f,-11.2062f,  -49.9363f,-41.7515f,-11.1735f,  -51.2615f,-40.6608f,-10.3207f,  -50.0670f,-39.0751f,-10.5670f,  -51.3117f,-37.9320f,-9.6944f,  
	-50.1831f,-36.3627f,-10.2100f,  -51.3552f,-35.1635f,-9.3130f,  -50.3262f,-33.6297f,-10.0816f,  -51.4323f,-32.3694f,-9.1552f,  -50.5351f,-30.8870f,-10.1550f,  
	-51.5805f,-29.5593f,-9.1936f,  -50.8450f,-28.1409f,-10.3974f,  -51.8335f,-26.7390f,-9.3955f,  -51.2861f,-25.3941f,-10.7717f,  -52.2190f,-23.9122f,-9.7248f,  
	-51.8818f,-22.6479f,-11.2381f,  -52.7596f,-21.0787f,-10.1411f,  -52.6499f,-19.8994f,-11.7533f,  -53.4708f,-18.2380f,-10.6023f,  -53.6006f,-17.1461f,-12.2736f,  
	-54.3616f,-15.3876f,-11.0650f,  -54.7369f,-14.3838f,-12.7543f,  -55.4329f,-12.5269f,-11.4871f,  -56.0541f,-11.6107f,-13.1535f,  -56.6799f,-9.6532f,-11.8257f,  
	-57.5414f,-8.8231f,-13.4291f,  -58.0906f,-6.7654f,-12.0403f,  -59.1812f,-6.0192f,-13.5423f,  -59.6471f,-3.8622f,-12.0926f,  -60.9500f,-3.1978f,-13.4575f,  
	-61.3248f,-0.9471f,-11.9497f,  -62.8185f,-0.3621f,-13.1450f,  -63.0952f,1.9793f,-11.5802f,  -64.7533f,2.4866f,-12.5778f,  -64.9245f,4.9106f,-10.9604f,  
	-66.7172f,5.3415f,-11.7365f,  -66.7759f,7.8406f,-10.0703f,  -68.6702f,8.1957f,-10.6064f,  -68.6098f,10.7608f,-8.8961f,  -70.5705f,11.0395f,-9.1793f,  
	-70.3851f,13.6601f,-7.4308f,  -72.3753f,13.8611f,-7.4539f,  -72.0598f,16.5260f,-5.6731f,  -74.0425f,16.6468f,-5.4352f,  -73.5925f,19.3437f,-3.6288f,  
	-75.5307f,19.3808f,-3.1348f,  -74.9430f,22.0969f,-1.3100f,  -76.8013f,22.0458f,-0.5707f,  -76.0735f,24.7676f,1.2650f,  -77.8184f,24.6229f,2.2328f,  
	-76.9497f,27.3361f,4.0715f,  -78.5508f,27.0918f,5.2457f,  -77.5411f,29.7825f,7.0798f,  -78.9719f,29.4324f,8.4333f,  -77.8233f,32.0845f,10.2535f,  
	-79.0618f,31.6218f,11.7548f,  -77.7769f,34.2211f,13.5532f,  -78.8060f,33.6394f,15.1671f,  -77.3902f,36.1695f,16.9344f,  -78.1986f,35.4631f,18.6225f,  
	-76.6582f,37.9093f,20.3502f,  -77.2402f,37.0733f,22.0720f,  -75.5840f,39.4199f,23.7511f,  -75.9401f,38.4512f,25.4649f,  -74.1790f,40.6827f,27.0862f,  
	-74.3158f,39.5801f,28.7498f,  -72.4633f,41.6815f,30.3040f,  -72.3932f,40.4463f,31.8761f,  -70.4636f,42.4050f,33.3555f,  -70.2043f,41.0415f,34.7963f,  
	-68.2165f,42.8441f,36.1914f,  -67.7906f,41.3603f,37.4638f,  -65.7633f,42.9972f,38.7677f,  -65.1975f,41.4048f,39.8383f,  -63.1532f,42.8672f,41.0436f,  
	-62.4771f,41.1821f,41.8834f,  -60.4387f,42.4652f,42.9842f,  -59.6835f,40.7077f,43.5697f,  -57.6741f,41.8099f,44.5618f,  -56.8717f,40.0043f,44.8754f,  
	-54.9154f,40.9275f,45.7558f,  -54.0968f,39.1024f,45.7863f,  -52.2151f,39.8529f,46.5542f,  -51.4089f,38.0401f,46.2975f,  -49.6218f,38.6281f,46.9539f,  
	-48.8529f,36.8623f,46.4128f,  -47.1771f,37.3009f,46.9604f,  -46.4659f,35.6184f,46.1447f,  -44.9128f,35.9248f,46.5886f,  -44.2737f,34.3624f,45.5149f,  
	-42.8499f,34.5554f,45.8619f,  -50.1849f,-50.7324f,-15.4471f,  -51.8880f,-49.7519f,-14.5886f,  -50.6606f,-48.3981f,-13.7745f,  -52.2555f,-47.3576f,-12.9032f,  
	-50.9755f,-45.9200f,-12.3578f,  -52.4671f,-44.8248f,-11.4734f,  -51.1630f,-43.3315f,-11.2062f,  -52.5580f,-42.1843f,-10.3071f,  -51.2615f,-40.6608f,-10.3207f,  
	-52.5672f,-39.4622f,-9.4043f,  -51.3117f,-37.9320f,-9.6944f,  -52.5355f,-36.6801f,-8.7572f,  -51.3552f,-35.1635f,-9.3130f,  -52.5037f,-33.8549f,-8.3506f,  
	-51.4323f,-32.3694f,-9.1552f,  -52.5112f,-30.9993f,-8.1628f,  -51.5805f,-29.5593f,-9.1936f,  -52.5941f,-28.1220f,-8.1663f,  -51.8335f,-26.7390f,-9.3955f,  
	-52.7838f,-25.2295f,-8.3292f,  -52.2190f,-23.9122f,-9.7248f,  -53.1075f,-22.3242f,-8.6146f,  -52.7596f,-21.0787f,-10.1411f,  -53.5854f,-19.4075f,-8.9833f,  
	-53.4708f,-18.2380f,-10.6023f,  -54.2322f,-16.4789f,-9.3934f,  -54.3616f,-15.3876f,-11.0650f,  -55.0544f,-13.5393f,-9.8037f,  -55.4329f,-12.5269f,-11.4871f,  
	-56.0531f,-10.5871f,-10.1713f,  -56.6799f,-9.6532f,-11.8257f,  -57.2221f,-7.6219f,-10.4547f,  -58.0906f,-6.7654f,-12.0403f,  -58.5490f,-4.6427f,-10.6136f,  
	-59.6471f,-3.8622f,-12.0926f,  -60.0149f,-1.6533f,-10.6127f,  -61.3248f,-0.9471f,-11.9497f,  -61.5958f,1.3462f,-10.4170f,  -63.0952f,1.9793f,-11.5802f,  
	-63.2624f,4.3495f,-9.9987f,  -64.9245f,4.9106f,-10.9604f,  -64.9817f,7.3513f,-9.3331f,  -66.7759f,7.8406f,-10.0703f,  -66.7168f,10.3439f,-8.4012f,  
	-68.6098f,10.7608f,-8.8961f,  -68.4289f,13.3171f,-7.1904f,  -70.3851f,13.6601f,-7.4308f,  -70.0772f,16.2596f,-5.6936f,  -72.0598f,16.5260f,-5.6731f,  
	-71.6207f,19.1577f,-3.9103f,  -73.5925f,19.3437f,-3.6288f,  -73.0186f,21.9959f,-1.8469f,  -74.9430f,22.0969f,-1.3100f,  -74.2316f,24.7573f,0.4844f,  
	-76.0735f,24.7676f,1.2650f,  -75.2231f,27.4231f,3.0648f,  -76.9497f,27.3361f,4.0715f,  -75.9596f,29.9739f,5.8696f,  -77.5411f,29.7825f,7.0798f,  
	-76.4125f,32.3879f,8.8679f,  -77.8233f,32.0845f,10.2535f,  -76.5579f,34.6440f,12.0244f,  -77.7769f,34.2211f,13.5532f,  -76.3788f,36.7192f,15.2982f,  
	-77.3902f,36.1695f,16.9344f,  -75.8646f,38.5922f,18.6456f,  -76.6582f,37.9093f,20.3502f,  -75.0123f,40.2413f,22.0189f,  -75.5840f,39.4199f,23.7511f,  
	-73.8273f,41.6460f,25.3685f,  -74.1790f,40.6827f,27.0862f,  -72.3237f,42.7879f,28.6432f,  -72.4633f,41.6815f,30.3040f,  -70.5222f,43.6528f,31.7929f,  
	-70.4636f,42.4050f,33.3555f,  -68.4543f,44.2282f,34.7667f,  -68.2165f,42.8441f,36.1914f,  -66.1567f,44.5087f,37.5176f,  -65.7633f,42.9972f,38.7677f,  
	-63.6747f,44.4930f,40.0011f,  -63.1532f,42.8672f,41.0436f,  -61.0585f,44.1877f,42.1774f,  -60.4387f,42.4652f,42.9842f,  -58.3610f,43.6070f,44.0136f,  
	-57.6741f,41.8099f,44.5618f,  -55.6382f,42.7728f,45.4826f,  -54.9154f,40.9275f,45.7558f,  -52.9441f,41.7159f,46.5660f,  -52.2151f,39.8529f,46.5542f,  
	-50.3302f,40.4749f,47.2535f,  -49.6218f,38.6281f,46.9539f,  -47.8422f,39.0952f,47.5439f,  -47.1771f,37.3009f,46.9604f,  -45.5170f,37.6287f,47.4451f,  
	-44.9128f,35.9248f,46.5886f,  -43.3817f,36.1313f,46.9736f,  -42.8499f,34.5554f,45.8619f,  -41.4522f,34.6600f,46.1540f,  -51.8880f,-49.7519f,-14.5886f,  
	-53.5672f,-48.6589f,-13.6697f,  -52.2555f,-47.3576f,-12.9032f,  -53.8261f,-46.2061f,-11.9705f,  -52.4671f,-44.8248f,-11.4734f,  -53.9344f,-43.6194f,-10.5266f,  
	-52.5580f,-42.1843f,-10.3071f,  -53.9283f,-40.9272f,-9.3447f,  -52.5672f,-39.4622f,-9.4043f,  -53.8470f,-38.1535f,-8.4236f,  -52.5355f,-36.6801f,-8.7572f,  
	-53.7312f,-35.3180f,-7.7548f,  -52.5037f,-33.8549f,-8.3506f,  -53.6210f,-32.4361f,-7.3227f,  -52.5112f,-30.9993f,-8.1628f,  -53.5547f,-29.5195f,-7.1051f,  
	-52.5941f,-28.1220f,-8.1663f,  -53.5666f,-26.5773f,-7.0751f,  -52.7838f,-25.2295f,-8.3292f,  -53.6875f,-23.6145f,-7.2001f,  -53.1075f,-22.3242f,-8.6146f,  
	-53.9425f,-20.6347f,-7.4442f,  -53.5854f,-19.4075f,-8.9833f,  -54.3510f,-17.6392f,-7.7681f,  -54.2322f,-16.4789f,-9.3934f,  -54.9251f,-14.6307f,-8.1321f,  
	-55.0544f,-13.5393f,-9.8037f,  -55.6717f,-11.6088f,-8.4942f,  -56.0531f,-10.5871f,-10.1713f,  -56.5904f,-8.5740f,-8.8126f,  -57.2221f,-7.6219f,-10.4547f,  
	-57.6747f,-5.5259f,-9.0459f,  -58.5490f,-4.6427f,-10.6136f,  -58.9110f,-2.4685f,-9.1570f,  -60.0149f,-1.6533f,-10.6127f,  -60.2809f,0.5981f,-9.1080f,  
	-61.5958f,1.3462f,-10.4170f,  -61.7596f,3.6679f,-8.8678f,  -63.2624f,4.3495f,-9.9987f,  -63.3183f,6.7361f,-8.4074f,  -64.9817f,7.3513f,-9.3331f,  
	-64.9239f,9.7961f,-7.7030f,  -66.7168f,10.3439f,-8.4012f,  -66.5402f,12.8382f,-6.7368f,  -68.4289f,13.3171f,-7.1904f,  -68.1286f,15.8523f,-5.4961f,  
	-70.0772f,16.2596f,-5.6936f,  -69.6489f,18.8260f,-3.9745f,  -71.6207f,19.1577f,-3.9103f,  -71.0608f,21.7445f,-2.1723f,  -73.0186f,21.9959f,-1.8469f,  
	-72.3241f,24.5923f,-0.0956f,  -74.2316f,24.7573f,0.4844f,  -73.4007f,27.3514f,2.2427f,  -75.2231f,27.4231f,3.0648f,  -74.2545f,30.0033f,4.8237f,  
	-75.9596f,29.9739f,5.8696f,  -74.8536f,32.5267f,7.6219f,  -76.4125f,32.3879f,8.8679f,  -75.1699f,34.9009f,10.6068f,  -76.5579f,34.6440f,12.0244f,  
	-75.1817f,37.1028f,13.7419f,  -76.3788f,36.7192f,15.2982f,  -74.8729f,39.1105f,16.9872f,  -75.8646f,38.5922f,18.6456f,  -74.2348f,40.9014f,20.2979f,  
	-75.0123f,40.2413f,22.0189f,  -73.2668f,42.4536f,23.6262f,  -73.8273f,41.6460f,25.3685f,  -71.9766f,43.7466f,26.9220f,  -72.3237f,42.7879f,28.6432f,  
	-70.3790f,44.7636f,30.1353f,  -70.5222f,43.6528f,31.7929f,  -68.5000f,45.4889f,33.2141f,  -68.4543f,44.2282f,34.7667f,  -66.3709f,45.9134f,36.1095f,  
	-66.1567f,44.5087f,37.5176f,  -64.0327f,46.0317f,38.7738f,  -63.6747f,44.4930f,40.0011f,  -61.5325f,45.8457f,41.1632f,  -61.0585f,44.1877f,42.1774f,  
	-58.9208f,45.3650f,43.2400f,  -58.3610f,43.6070f,44.0136f,  -56.2529f,44.6066f,44.9713f,  -55.6382f,42.7728f,45.4826f,  -53.5834f,43.5971f,46.3326f,  
	-52.9441f,41.7159f,46.5660f,  -50.9660f,42.3709f,47.3067f,  -50.3302f,40.4749f,47.2535f,  -48.4497f,40.9704f,47.8854f,  -47.8422f,39.0952f,47.5439f,  
	-46.0761f,39.4452f,48.0696f,  -45.5170f,37.6287f,47.4451f,  -43.8777f,37.8501f,47.8690f,  -43.3817f,36.1313f,46.9736f,  -41.8767f,36.2426f,47.3017f,  
	-41.4522f,34.6600f,46.1540f,  -40.0844f,34.6807f,46.3930f,  -53.5672f,-48.6589f,-13.6697f,  -55.2170f,-47.4515f,-12.6891f,  -53.8261f,-46.2061f,-11.9705f,  
	-55.3671f,-44.9416f,-10.9754f,  -53.9344f,-43.6194f,-10.5266f,  -55.3715f,-42.3018f,-9.5165f,  -53.9283f,-40.9272f,-9.3447f,  -55.2671f,-39.5586f,-8.3181f,  
	-53.8470f,-38.1535f,-8.4236f,  -55.0936f,-36.7337f,-7.3783f,  -53.7312f,-35.3180f,-7.7548f,  -54.8911f,-33.8453f,-6.6876f,  -53.6210f,-32.4361f,-7.3227f,  
	-54.6990f,-30.9077f,-6.2301f,  -53.5547f,-29.5195f,-7.1051f,  -54.5540f,-27.9326f,-5.9841f,  -53.5666f,-26.5773f,-7.0751f,  -54.4899f,-24.9275f,-5.9218f,  
	-53.6875f,-23.6145f,-7.2001f,  -54.5359f,-21.8984f,-6.0114f,  -53.9425f,-20.6347f,-7.4442f,  -54.7159f,-18.8486f,-6.2168f,  -54.3510f,-17.6392f,-7.7681f,  
	-55.0473f,-15.7824f,-6.5009f,  -54.9251f,-14.6307f,-8.1321f,  -55.5424f,-12.7008f,-6.8229f,  -55.6717f,-11.6088f,-8.4942f,  -56.2066f,-9.6054f,-7.1420f,  
	-56.5904f,-8.5740f,-8.8126f,  -57.0392f,-6.4966f,-7.4162f,  -57.6747f,-5.5259f,-9.0459f,  -58.0322f,-3.3788f,-7.6075f,  -58.9110f,-2.4685f,-9.1570f,  
	-59.1729f,-0.2521f,-7.6757f,  -60.2809f,0.5981f,-9.1080f,  -60.4416f,2.8775f,-7.5871f,  -61.7596f,3.6679f,-8.8678f,  -61.8142f,6.0057f,-7.3090f,  
	-63.3183f,6.7361f,-8.4074f,  -63.2617f,9.1267f,-6.8135f,  -64.9239f,9.7961f,-7.7030f,  -64.7514f,12.2315f,-6.0780f,  -66.5402f,12.8382f,-6.7368f,  
	-66.2472f,15.3111f,-5.0842f,  -68.1286f,15.8523f,-5.4961f,  -67.7109f,18.3541f,-3.8203f,  -69.6489f,18.8260f,-3.9745f,  -69.1031f,21.3470f,-2.2806f,  
	-71.0608f,21.7445f,-2.1723f,  -70.3838f,24.2753f,-0.4650f,  -72.3241f,24.5923f,-0.0956f,  -71.5136f,27.1223f,1.6194f,  -73.4007f,27.3514f,2.2427f,  
	-72.4551f,29.8701f,3.9598f,  -74.2545f,30.0033f,4.8237f,  -73.1732f,32.4987f,6.5366f,  -74.8536f,32.5267f,7.6219f,  -73.6367f,34.9874f,9.3246f,  
	-75.1699f,34.9009f,10.6068f,  -73.8194f,37.3136f,12.2922f,  -75.1817f,37.1028f,13.7419f,  -73.7002f,39.4553f,15.4036f,  -74.8729f,39.1105f,16.9872f,  
	-73.2649f,41.3889f,18.6175f,  -74.2348f,40.9014f,20.2979f,  -72.5070f,43.0917f,21.8890f,  -73.2668f,42.4536f,23.6262f,  -71.4280f,44.5412f,25.1699f,  
	-71.9766f,43.7466f,26.9220f,  -70.0367f,45.7185f,28.4110f,  -70.3790f,44.7636f,30.1353f,  -68.3528f,46.6048f,31.5602f,  -68.5000f,45.4889f,33.2141f,  
	-66.4023f,47.1874f,34.5673f,  -66.3709f,45.9134f,36.1095f,  -64.2211f,47.4570f,37.3827f,  -64.0327f,46.0317f,38.7738f,  -61.8525f,47.4107f,39.9590f,  
	-61.5325f,45.8457f,41.1632f,  -59.3438f,47.0536f,42.2543f,  -58.9208f,45.3650f,43.2400f,  -56.7489f,46.3976f,44.2309f,  -56.2529f,44.6066f,44.9713f,  
	-54.1221f,45.4644f,45.8581f,  -53.5834f,43.5971f,46.3326f,  -51.5184f,44.2838f,47.1124f,  -50.9660f,42.3709f,47.3067f,  -48.9893f,42.8945f,47.9790f,  
	-48.4497f,40.9704f,47.8854f,  -46.5805f,41.3430f,48.4515f,  -46.0761f,39.4452f,48.0696f,  -44.3294f,39.6824f,48.5328f,  -43.8777f,37.8501f,47.8690f,  
	-42.2642f,37.9696f,48.2342f,  -41.8767f,36.2426f,47.3017f,  -40.4026f,36.2633f,47.5746f,  -40.0844f,34.6807f,46.3930f,  -38.7477f,34.6232f,46.5819f,  
	-55.2170f,-47.4515f,-12.6891f,  -56.8315f,-46.1282f,-11.6457f,  -55.3671f,-44.9416f,-10.9754f,  -56.8722f,-43.5627f,-9.9168f,  -55.3715f,-42.3018f,-9.5165f,  
	-56.7716f,-40.8712f,-8.4425f,  -55.2671f,-39.5586f,-8.3181f,  -56.5676f,-38.0777f,-7.2272f,  -55.0936f,-36.7337f,-7.3783f,  -56.2997f,-35.2026f,-6.2684f,  
	-54.8911f,-33.8453f,-6.6876f,  -56.0074f,-32.2627f,-5.5562f,  -54.6990f,-30.9077f,-6.2301f,  -55.7291f,-29.2721f,-5.0747f,  -54.5540f,-27.9326f,-5.9841f,  
	-55.5009f,-26.2409f,-4.8015f,  -54.4899f,-24.9275f,-5.9218f,  -55.3554f,-23.1772f,-4.7094f,  -54.5359f,-21.8984f,-6.0114f,  -55.3206f,-20.0864f,-4.7662f,  
	-54.7159f,-18.8486f,-6.2168f,  -55.4188f,-16.9746f,-4.9380f,  -55.0473f,-15.7824f,-6.5009f,  -55.6672f,-13.8444f,-5.1862f,  -55.5424f,-12.7008f,-6.8229f,  
	-56.0770f,-10.6985f,-5.4715f,  -56.2066f,-9.6054f,-7.1420f,  -56.6531f,-7.5382f,-5.7525f,  -57.0392f,-6.4966f,-7.4162f,  -57.3934f,-4.3687f,-5.9906f,  
	-58.0322f,-3.3788f,-7.6075f,  -58.2908f,-1.1900f,-6.1446f,  -59.1729f,-0.2521f,-7.6757f,  -59.3310f,1.9917f,-6.1785f,  -60.4416f,2.8775f,-7.5871f,  
	-60.4952f,5.1725f,-6.0568f,  -61.8142f,6.0057f,-7.3090f,  -61.7588f,8.3472f,-5.7479f,  -63.2617f,9.1267f,-6.8135f,  -63.0931f,11.5073f,-5.2250f,  
	-64.7514f,12.2315f,-6.0780f,  -64.4654f,14.6451f,-4.4649f,  -66.2472f,15.3111f,-5.0842f,  -65.8400f,17.7503f,-3.4503f,  -67.7109f,18.3541f,-3.8203f,  
	-67.1791f,20.8102f,-2.1699f,  -69.1031f,21.3470f,-2.2806f,  -68.4436f,23.8119f,-0.6177f,  -70.3838f,24.2753f,-0.4650f,  -69.5942f,26.7397f,1.2055f,  
	-71.5136f,27.1223f,1.6194f,  -70.5920f,29.5767f,3.2928f,  -72.4551f,29.8701f,3.9598f,  -71.4002f,32.3042f,5.6308f,  -73.1732f,32.4987f,6.5366f,  
	-71.9846f,34.9020f,8.1998f,  -73.6367f,34.9874f,9.3246f,  -72.3152f,37.3482f,10.9738f,  -73.8194f,37.3136f,12.2922f,  -72.3665f,39.6206f,13.9217f,  
	-73.7002f,39.4553f,15.4036f,  -72.1192f,41.6955f,17.0065f,  -73.2649f,41.3889f,18.6175f,  -71.5610f,43.5493f,20.1868f,  -72.5070f,43.0917f,21.8890f,  
	-70.6874f,45.1581f,23.4169f,  -71.4280f,44.5412f,25.1699f,  -69.5009f,46.5011f,26.6496f,  -70.0367f,45.7185f,28.4110f,  -68.0152f,47.5566f,29.8333f,  
	-68.3528f,46.6048f,31.5602f,  -66.2503f,48.3088f,32.9176f,  -66.4023f,47.1874f,34.5673f,  -64.2366f,48.7445f,35.8516f,  -64.2211f,47.4570f,37.3827f,  
	-62.0129f,48.8562f,38.5852f,  -61.8525f,47.4107f,39.9590f,  -59.6228f,48.6441f,41.0734f,  -59.3438f,47.0536f,42.2543f,  -57.1177f,48.1152f,43.2738f,  
	-56.7489f,46.3976f,44.2309f,  -54.5509f,47.2858f,45.1505f,  -54.1221f,45.4644f,45.8581f,  -51.9779f,46.1809f,46.6740f,  -51.5184f,44.2838f,47.1124f,  
	-49.4517f,44.8346f,47.8230f,  -48.9893f,42.8945f,47.9790f,  -47.0215f,43.2898f,48.5843f,  -46.5805f,41.3430f,48.4515f,  -44.7292f,41.5968f,48.9537f,  
	-44.3294f,39.6824f,48.5328f,  -42.6081f,39.8113f,48.9356f,  -42.2642f,37.9696f,48.2342f,  -40.6823f,37.9917f,48.5424f,  -40.4026f,36.2633f,47.5746f,  
	-38.9611f,36.1991f,47.7955f,  -38.7477f,34.6232f,46.5819f,  -37.4434f,34.4924f,46.7232f,  -56.8315f,-46.1282f,-11.6457f,  -58.4047f,-44.6881f,-10.5387f,  
	-56.8722f,-43.5627f,-9.9168f,  -58.3348f,-42.0690f,-8.7945f,  -56.7716f,-40.8712f,-8.4425f,  -58.1281f,-39.3271f,-7.3043f,  -56.5676f,-38.0777f,-7.2272f,  
	-57.8226f,-36.4849f,-6.0722f,  -56.2997f,-35.2026f,-6.2684f,  -57.4575f,-33.5614f,-5.0949f,  -56.0074f,-32.2627f,-5.5562f,  -57.0717f,-30.5729f,-4.3625f,  
	-55.7291f,-29.2721f,-5.0747f,  -56.7032f,-27.5321f,-3.8584f,  -55.5009f,-26.2409f,-4.8015f,  -56.3871f,-24.4490f,-3.5604f,  -55.3554f,-23.1772f,-4.7094f,  
	-56.1549f,-21.3314f,-3.4411f,  -55.3206f,-20.0864f,-4.7662f,  -56.0332f,-18.1870f,-3.4701f,  -55.4188f,-16.9746f,-4.9380f,  -56.0442f,-15.0200f,-3.6121f,  
	-55.6672f,-13.8444f,-5.1862f,  -56.2040f,-11.8346f,-3.8298f,  -56.0770f,-10.6985f,-5.4715f,  -56.5232f,-8.6330f,-4.0831f,  -56.6531f,-7.5382f,-5.7525f,  
	-57.0056f,-5.4212f,-4.3341f,  -57.3934f,-4.3687f,-5.9906f,  -57.6497f,-2.1994f,-4.5408f,  -58.2908f,-1.1900f,-6.1446f,  -58.4469f,1.0258f,-4.6660f,  
	-59.3310f,1.9917f,-6.1785f,  -59.3837f,4.2508f,-4.6722f,  -60.4952f,5.1725f,-6.0568f,  -60.4407f,7.4709f,-4.5244f,  -61.7588f,8.3472f,-5.7479f,  
	-61.5935f,10.6781f,-4.1925f,  -63.0931f,11.5073f,-5.2250f,  -62.8135f,13.8659f,-3.6487f,  -64.4654f,14.6451f,-4.4649f,  -64.0680f,17.0248f,-2.8708f,  
	-65.8400f,17.7503f,-3.4503f,  -65.3217f,20.1433f,-1.8422f,  -67.1791f,20.8102f,-2.1699f,  -66.5370f,23.2098f,-0.5509f,  -68.4436f,23.8119f,-0.6177f,  
	-67.6752f,26.2100f,1.0081f,  -69.5942f,26.7397f,1.2055f,  -68.6971f,29.1281f,2.8341f,  -70.5920f,29.5767f,3.2928f,  -69.5647f,31.9465f,4.9199f,  
	-71.4002f,32.3042f,5.6308f,  -70.2417f,34.6462f,7.2517f,  -71.9846f,34.9020f,8.1998f,  -70.6948f,37.2058f,9.8093f,  -72.3152f,37.3482f,10.9738f,  
	-70.8946f,39.6036f,12.5669f,  -72.3665f,39.6206f,13.9217f,  -70.8172f,41.8159f,15.4924f,  -72.1192f,41.6955f,17.0065f,  -70.4448f,43.8186f,18.5485f,  
	-71.5610f,43.5493f,20.1868f,  -69.7673f,45.5868f,21.6930f,  -70.6874f,45.1581f,23.4169f,  -68.7809f,47.0979f,24.8811f,  -69.5009f,46.5011f,26.6496f,  
	-67.4930f,48.3281f,28.0629f,  -68.0152f,47.5566f,29.8333f,  -65.9176f,49.2584f,31.1886f,  -66.2503f,48.3088f,32.9176f,  -64.0790f,49.8721f,34.2067f,  
	-64.2366f,48.7445f,35.8516f,  -62.0111f,50.1572f,37.0656f,  -62.0129f,48.8562f,38.5852f,  -59.7531f,50.1093f,39.7175f,  -59.6228f,48.6441f,41.0734f,  
	-57.3531f,49.7299f,42.1165f,  -57.1177f,48.1152f,43.2738f,  -54.8626f,49.0302f,44.2220f,  -54.5509f,47.2858f,45.1505f,  -52.3365f,48.0296f,45.9990f,  
	-51.9779f,46.1809f,46.6740f,  -49.8290f,46.7575f,47.4202f,  -49.4517f,44.8346f,47.8230f,  -47.3917f,45.2522f,48.4657f,  -47.0215f,43.2898f,48.5843f,  
	-45.0700f,43.5607f,49.1245f,  -44.7292f,41.5968f,48.9537f,  -42.9026f,41.7362f,49.3939f,  -42.6081f,39.8113f,48.9356f,  -40.9189f,39.8363f,49.2797f,  
	-40.6823f,37.9917f,48.5424f,  -39.1345f,37.9225f,48.7966f,  -38.9611f,36.1991f,47.7955f,  -37.5546f,36.0555f,47.9668f,  -37.4434f,34.4924f,46.7232f,  
	-36.1739f,34.2931f,46.8188f,  -58.4047f,-44.6881f,-10.5387f,  -59.9298f,-43.1315f,-9.3682f,  -58.3348f,-42.0690f,-8.7945f,  -59.7484f,-40.4605f,-7.6083f,  
	-58.1281f,-39.3271f,-7.3043f,  -59.4339f,-37.6702f,-6.1025f,  -57.8226f,-36.4849f,-6.0722f,  -59.0245f,-34.7816f,-4.8542f,  -57.4575f,-33.5614f,-5.0949f,  
	-58.5590f,-31.8129f,-3.8597f,  -57.0717f,-30.5729f,-4.3625f,  -58.0762f,-28.7790f,-3.1085f,  -56.7032f,-27.5321f,-3.8584f,  -57.6132f,-25.6922f,-2.5841f,  
	-56.3871f,-24.4490f,-3.5604f,  -57.2045f,-22.5623f,-2.2641f,  -56.1549f,-21.3314f,-3.4411f,  -56.8802f,-19.3988f,-2.1224f,  -56.0332f,-18.1870f,-3.4701f,  
	-56.6667f,-16.2075f,-2.1274f,  -56.0442f,-15.0200f,-3.6121f,  -56.5853f,-12.9942f,-2.2448f,  -56.2040f,-11.8346f,-3.8298f,  -56.6518f,-9.7622f,-2.4368f,  
	-56.5232f,-8.6330f,-4.0831f,  -56.8754f,-6.5182f,-2.6663f,  -57.0056f,-5.4212f,-4.3341f,  -57.2605f,-3.2632f,-2.8918f,  -57.6497f,-2.1994f,-4.5408f,  
	-57.8044f,-0.0038f,-3.0757f,  -58.4469f,1.0258f,-4.6660f,  -58.4989f,3.2563f,-3.1788f,  -59.3837f,4.2508f,-4.6722f,  -59.3301f,6.5127f,-3.1640f,  
	-60.4407f,7.4709f,-4.5244f,  -60.2785f,9.7581f,-2.9982f,  -61.5935f,10.6781f,-4.1925f,  -61.3198f,12.9867f,-2.6496f,  -62.8135f,13.8659f,-3.6487f,  
	-62.4253f,16.1900f,-2.0917f,  -64.0680f,17.0248f,-2.8708f,  -63.5627f,19.3577f,-1.3029f,  -65.3217f,20.1433f,-1.8422f,  -64.6965f,22.4795f,-0.2659f,  
	-66.5370f,23.2098f,-0.5509f,  -65.7894f,25.5425f,1.0306f,  -67.6752f,26.2100f,1.0081f,  -66.8030f,28.5319f,2.5915f,  -68.6971f,29.1281f,2.8341f,  
	-67.6984f,31.4319f,4.4160f,  -69.5647f,31.9465f,4.9199f,  -68.4379f,34.2243f,6.4964f,  -70.2417f,34.6462f,7.2517f,  -68.9859f,36.8890f,8.8186f,  
	-70.6948f,37.2058f,9.8093f,  -69.3097f,39.4047f,11.3625f,  -70.8946f,39.6036f,12.5669f,  -69.3812f,41.7481f,14.1012f,  -70.8172f,41.8159f,15.4924f,  
	-69.1778f,43.8950f,17.0023f,  -70.4448f,43.8186f,18.5485f,  -68.6836f,45.8199f,20.0277f,  -69.7673f,45.5868f,21.6930f,  -67.8890f,47.4988f,23.1358f,  
	-68.7809f,47.0979f,24.8811f,  -66.7952f,48.9061f,26.2794f,  -67.4930f,48.3281f,28.0629f,  -65.4097f,50.0200f,29.4098f,  -65.9176f,49.2584f,31.1886f,  
	-63.7510f,50.8206f,32.4761f,  -64.0790f,49.8721f,34.2067f,  -61.8471f,51.2916f,35.4259f,  -62.0111f,50.1572f,37.0656f,  -59.7324f,51.4240f,38.2098f,  
	-59.7531f,50.1093f,39.7175f,  -57.4510f,51.2142f,40.7787f,  -57.3531f,49.7299f,42.1165f,  -55.0517f,50.6676f,43.0884f,  -54.8626f,49.0302f,44.2220f,  
	-52.5883f,49.7984f,45.0989f,  -52.3365f,48.0296f,45.9990f,  -50.1147f,48.6302f,46.7774f,  -49.8290f,46.7575f,47.4202f,  -47.6846f,47.1967f,48.0977f,  
	-47.3917f,45.2522f,48.4657f,  -45.3462f,45.5404f,49.0421f,  -45.0700f,43.5607f,49.1245f,  -43.1425f,43.7115f,49.6012f,  -42.9026f,41.7362f,49.3939f,  
	-41.1083f,41.7656f,49.7739f,  -40.9189f,39.8363f,49.2797f,  -39.2649f,39.7637f,49.5680f,  -39.1345f,37.9225f,48.7966f,  -37.6241f,37.7675f,48.9994f,  
	-37.5546f,36.0555f,47.9668f,  -36.1862f,35.8373f,48.0906f,  -36.1739f,34.2931f,46.8188f,  -34.9405f,34.0300f,46.8708f,  -59.9298f,-43.1315f,-9.3682f,  
	-61.4006f,-41.4585f,-8.1338f,  -59.7484f,-40.4605f,-7.6083f,  -61.1060f,-38.7382f,-6.3588f,  -59.4339f,-37.6702f,-6.1025f,  -60.6815f,-35.9023f,-4.8382f,  
	-59.0245f,-34.7816f,-4.8542f,  -60.1654f,-32.9706f,-3.5750f,  -58.5590f,-31.8129f,-3.8597f,  -59.5964f,-29.9603f,-2.5649f,  -58.0762f,-28.7790f,-3.1085f,  
	-59.0129f,-26.8855f,-1.7973f,  -57.6132f,-25.6922f,-2.5841f,  -58.4514f,-23.7581f,-1.2553f,  -57.2045f,-22.5623f,-2.2641f,  -57.9451f,-20.5892f,-0.9178f,  
	-56.8802f,-19.3988f,-2.1224f,  -57.5242f,-17.3867f,-0.7576f,  -56.6667f,-16.2075f,-2.1274f,  -57.2144f,-14.1575f,-0.7438f,  -56.5853f,-12.9942f,-2.2448f,  
	-57.0365f,-10.9065f,-0.8416f,  -56.6518f,-9.7622f,-2.4368f,  -57.0051f,-7.6411f,-1.0157f,  -56.8754f,-6.5182f,-2.6663f,  -57.1300f,-4.3631f,-1.2259f,  
	-57.2605f,-3.2632f,-2.8918f,  -57.4144f,-1.0794f,-1.4346f,  -57.8044f,-0.0038f,-3.0757f,  -57.8559f,2.2060f,-1.6023f,  -58.4989f,3.2563f,-3.1788f,  
	-58.4459f,5.4892f,-1.6900f,  -59.3301f,6.5127f,-3.1640f,  -59.1705f,8.7630f,-1.6625f,  -60.2785f,9.7581f,-2.9982f,  -60.0100f,12.0225f,-1.4848f,  
	-61.3198f,12.9867f,-2.6496f,  -60.9400f,15.2603f,-1.1265f,  -62.4253f,16.1900f,-2.0917f,  -61.9321f,18.4669f,-0.5615f,  -63.5627f,19.3577f,-1.3029f,  
	-62.9536f,21.6335f,0.2326f,  -64.6965f,22.4795f,-0.2659f,  -63.9693f,24.7484f,1.2725f,  -65.7894f,25.5425f,1.0306f,  -64.9419f,27.7984f,2.5691f,  
	-66.8030f,28.5319f,2.5915f,  -65.8329f,30.7691f,4.1278f,  -67.6984f,31.4319f,4.4160f,  -66.6040f,33.6436f,5.9469f,  -68.4379f,34.2243f,6.4964f,  
	-67.2178f,36.4031f,8.0187f,  -68.9859f,36.8890f,8.8186f,  -67.6389f,39.0271f,10.3290f,  -69.3097f,39.4047f,11.3625f,  -67.8358f,41.4932f,12.8566f,  
	-69.3812f,41.7481f,14.1012f,  -67.7814f,43.7771f,15.5746f,  -69.1778f,43.8950f,17.0023f,  -67.4547f,45.8534f,18.4495f,  -68.6836f,45.8199f,20.0277f,  
	-66.8405f,47.6969f,21.4437f,  -67.8890f,47.4988f,23.1358f,  -65.9336f,49.2807f,24.5132f,  -66.7952f,48.9061f,26.2794f,  -64.7355f,50.5806f,27.6116f,  
	-65.4097f,50.0200f,29.4098f,  -63.2581f,51.5737f,30.6895f,  -63.7510f,50.8206f,32.4761f,  -61.5237f,52.2399f,33.6944f,  -61.8471f,51.2916f,35.4259f,  
	-59.5610f,52.5657f,36.5760f,  -59.7324f,51.4240f,38.2098f,  -57.4097f,52.5426f,39.2835f,  -57.4510f,51.2142f,40.7787f,  -55.1151f,52.1702f,41.7692f,  
	-55.0517f,50.6676f,43.0884f,  -52.7289f,51.4569f,43.9891f,  -52.5883f,49.7984f,45.0989f,  -50.3041f,50.4208f,45.9056f,  -50.1147f,48.6302f,46.7774f,  
	-47.8952f,49.0899f,47.4866f,  -47.6846f,47.1967f,48.0977f,  -45.5530f,47.5019f,48.7082f,  -45.3462f,45.5404f,49.0421f,  -43.3238f,45.7032f,49.5541f,  
	-43.1425f,43.7115f,49.6012f,  -41.2472f,43.7465f,50.0166f,  -41.1083f,41.7656f,49.7739f,  -39.3501f,41.6912f,50.0966f,  -39.2649f,39.7637f,49.5680f,  
	-37.6504f,39.5993f,49.8033f,  -37.6241f,37.7675f,48.9994f,  -36.1547f,37.5321f,49.1529f,  -36.1862f,35.8373f,48.0906f,  -34.8580f,35.5498f,48.1691f,  
	-34.9405f,34.0300f,46.8708f,  -33.7422f,33.7084f,46.8824f,  -61.4006f,-41.4585f,-8.1338f,  -62.8102f,-39.6707f,-6.8365f,  -61.1060f,-38.7382f,-6.3588f,  
	-62.4003f,-36.9045f,-5.0472f,  -60.6815f,-35.9023f,-4.8382f,  -61.8635f,-34.0265f,-3.5131f,  -60.1654f,-32.9706f,-3.5750f,  -61.2378f,-31.0558f,-2.2367f,  
	-59.5964f,-29.9603f,-2.5649f,  -60.5621f,-28.0085f,-1.2133f,  -59.0129f,-26.8855f,-1.7973f,  -59.8742f,-24.8983f,-0.4321f,  -58.4514f,-23.7581f,-1.2553f,  
	-59.2097f,-21.7380f,0.1229f,  -57.9451f,-20.5892f,-0.9178f,  -58.6021f,-18.5373f,0.4739f,  -57.5242f,-17.3867f,-0.7576f,  -58.0806f,-15.3047f,0.6475f,  
	-57.2144f,-14.1575f,-0.7438f,  -57.6708f,-12.0463f,0.6752f,  -57.0365f,-10.9065f,-0.8416f,  -57.3923f,-8.7705f,0.5895f,  -57.0051f,-7.6411f,-1.0157f,  
	-57.2604f,-5.4802f,0.4286f,  -57.1300f,-4.3631f,-1.2259f,  -57.2837f,-2.1827f,0.2291f,  -57.4144f,-1.0794f,-1.4346f,  -57.4656f,1.1180f,0.0305f,  
	-57.8559f,2.2060f,-1.6023f,  -57.8033f,4.4177f,-0.1276f,  -58.4459f,5.4892f,-1.6900f,  -58.2884f,7.7098f,-0.2082f,  -59.1705f,8.7630f,-1.6625f,  
	-58.9064f,10.9899f,-0.1741f,  -60.0100f,12.0225f,-1.4848f,  -59.6376f,14.2515f,0.0085f,  -60.9400f,15.2603f,-1.1265f,  -60.4579f,17.4861f,0.3695f,  
	-61.9321f,18.4669f,-0.5615f,  -61.3381f,20.6861f,0.9360f,  -62.9536f,21.6335f,0.2326f,  -62.2458f,23.8414f,1.7299f,  -63.9693f,24.7484f,1.2725f,  
	-63.1458f,26.9400f,2.7674f,  -64.9419f,27.7984f,2.5691f,  -64.0004f,29.9695f,4.0602f,  -65.8329f,30.7691f,4.1278f,  -64.7715f,32.9140f,5.6126f,  
	-66.6040f,33.6436f,5.9469f,  -65.4207f,35.7564f,7.4233f,  -67.2178f,36.4031f,8.0187f,  -65.9109f,38.4775f,9.4841f,  -67.6389f,39.0271f,10.3290f,  
	-66.2075f,41.0555f,11.7800f,  -67.8358f,41.4932f,12.8566f,  -66.2796f,43.4671f,14.2898f,  -67.7814f,43.7771f,15.5746f,  -66.1017f,45.6867f,16.9853f,  
	-67.4547f,45.8534f,18.4495f,  -65.6532f,47.6889f,19.8336f,  -66.8405f,47.6969f,21.4437f,  -64.9230f,49.4455f,22.7945f,  -65.9336f,49.2807f,24.5132f,  
	-63.9065f,50.9305f,25.8249f,  -64.7355f,50.5806f,27.6116f,  -62.6089f,52.1185f,28.8773f,  -63.2581f,51.5737f,30.6895f,  -61.0464f,52.9859f,31.9006f,  
	-61.5237f,52.2399f,33.6944f,  -59.2420f,53.5150f,34.8442f,  -59.5610f,52.5657f,36.5760f,  -57.2300f,53.6924f,37.6563f,  -57.4097f,52.5426f,39.2835f,  
	-55.0517f,53.5122f,40.2869f,  -55.1151f,52.1702f,41.7692f,  -52.7559f,52.9769f,42.6887f,  -52.7289f,51.4569f,43.9891f,  -50.3938f,52.0986f,44.8198f,  
	-50.3041f,50.4208f,45.9056f,  -48.0201f,50.8995f,46.6429f,  -47.8952f,49.0899f,47.4866f,  -45.6868f,49.4119f,48.1282f,  -45.5530f,47.5019f,48.7082f,  
	-43.4434f,47.6773f,49.2533f,  -43.3238f,45.7032f,49.5541f,  -41.3333f,45.7451f,50.0036f,  -41.2472f,43.7465f,50.0166f,  -39.3887f,43.6721f,50.3733f,  
	-39.3501f,41.6912f,50.0966f,  -37.6334f,41.5193f,50.3647f,  -37.6504f,39.5993f,49.8033f,  -36.0798f,39.3486f,49.9877f,  -36.1547f,37.5321f,49.1529f,  
	-34.7292f,37.2219f,49.2597f,  -34.8580f,35.5498f,48.1691f,  -33.5693f,35.1988f,48.2056f,  -33.7422f,33.7084f,46.8824f,  -32.5801f,33.3327f,46.8557f,  
	-62.8102f,-39.6707f,-6.8365f,  -64.1514f,-37.7708f,-5.4775f,  -62.4003f,-36.9045f,-5.0472f,  -63.6242f,-34.9623f,-3.6752f,  -61.8635f,-34.0265f,-3.5131f,  
	-62.9724f,-32.0467f,-2.1296f,  -61.2378f,-31.0558f,-2.2367f,  -62.2344f,-29.0419f,-0.8423f,  -60.5621f,-28.0085f,-1.2133f,  -61.4487f,-25.9633f,0.1915f,  
	-59.8742f,-24.8983f,-0.4321f,  -60.6525f,-22.8255f,0.9819f,  -59.2097f,-21.7380f,0.1229f,  -59.8817f,-19.6396f,1.5460f,  -58.6021f,-18.5373f,0.4739f,  
	-59.1691f,-16.4159f,1.9055f,  -58.0806f,-15.3047f,0.6475f,  -58.5439f,-13.1620f,2.0877f,  -57.6708f,-12.0463f,0.6752f,  -58.0306f,-9.8873f,2.1217f,  
	-57.3923f,-8.7705f,0.5895f,  -57.6493f,-6.5955f,2.0432f,  -57.2604f,-5.4802f,0.4286f,  -57.4144f,-3.2947f,1.8869f,  -57.2837f,-2.1827f,0.2291f,  
	-57.3347f,0.0108f,1.6916f,  -57.4656f,1.1180f,0.0305f,  -57.4134f,3.3167f,1.4965f,  -57.8033f,4.4177f,-0.1276f,  -57.6473f,6.6166f,1.3397f,  
	-58.2884f,7.7098f,-0.2082f,  -58.0278f,9.9066f,1.2600f,  -58.9064f,10.9899f,-0.1741f,  -58.5403f,13.1809f,1.2937f,  -59.6376f,14.2515f,0.0085f,  
	-59.1652f,16.4320f,1.4741f,  -60.4579f,17.4861f,0.3695f,  -59.8776f,19.6536f,1.8322f,  -61.3381f,20.6861f,0.9360f,  -60.6485f,22.8370f,2.3947f,  
	-62.2458f,23.8414f,1.7299f,  -61.4454f,25.9716f,3.1829f,  -63.1458f,26.9400f,2.7674f,  -62.2322f,29.0467f,4.2143f,  -64.0004f,29.9695f,4.0602f,  
	-62.9717f,32.0480f,5.4992f,  -64.7715f,32.9140f,5.6126f,  -63.6255f,34.9601f,7.0425f,  -65.4207f,35.7564f,7.4233f,  -64.1551f,37.7652f,8.8422f,  
	-65.9109f,38.4775f,9.4841f,  -64.5241f,40.4426f,10.8898f,  -66.2075f,41.0555f,11.7800f,  -64.6982f,42.9702f,13.1699f,  -66.2796f,43.4671f,14.2898f,  
	-64.6477f,45.3228f,15.6603f,  -66.1017f,45.6867f,16.9853f,  -64.3476f,47.4748f,18.3331f,  -65.6532f,47.6889f,19.8336f,  -63.7808f,49.3976f,21.1528f,  
	-64.9230f,49.4455f,22.7945f,  -62.9368f,51.0638f,24.0802f,  -63.9065f,50.9305f,25.8249f,  -61.8143f,52.4458f,27.0708f,  -62.6089f,52.1185f,28.8773f,  
	-60.4235f,53.5168f,30.0752f,  -61.0464f,52.9859f,31.9006f,  -58.7807f,54.2556f,33.0440f,  -59.2420f,53.5150f,34.8442f,  -56.9149f,54.6439f,35.9250f,  
	-57.2300f,53.6924f,37.6563f,  -54.8626f,54.6706f,38.6669f,  -55.0517f,53.5122f,40.2869f,  -52.6687f,54.3322f,41.2198f,  -52.7559f,52.9769f,42.6887f,  
	-50.3823f,53.6349f,43.5385f,  -50.3938f,52.0986f,44.8198f,  -48.0569f,52.5945f,45.5810f,  -48.0201f,50.8995f,46.6429f,  -45.7454f,51.2375f,47.3123f,  
	-45.6868f,49.4119f,48.1282f,  -43.4992f,49.6002f,48.7039f,  -43.4434f,47.6773f,49.2533f,  -41.3651f,47.7273f,49.7351f,  -41.3333f,45.7451f,50.0036f,  
	-39.3800f,45.6725f,50.3934f,  -39.3887f,43.6721f,50.3733f,  -37.5731f,43.4949f,50.6740f,  -37.6334f,41.5193f,50.3647f,  -35.9628f,41.2557f,50.5806f,  
	-36.0798f,39.3486f,49.9877f,  -34.5564f,39.0178f,50.1240f,  -34.7292f,37.2219f,49.2597f,  -33.3475f,36.8434f,49.3229f,  -33.5693f,35.1988f,48.2056f,  
	-32.3218f,34.7894f,48.2022f,  -32.5801f,33.3327f,46.8557f,  -31.4547f,32.9073f,46.7925f,  -64.1514f,-37.7708f,-5.4775f,  -65.4174f,-35.7620f,-4.0584f,  
	-63.6242f,-34.9623f,-3.6752f,  -64.7704f,-32.9162f,-2.2453f,  -62.9724f,-32.0467f,-2.1296f,  -64.0011f,-29.9681f,-0.6904f,  -62.2344f,-29.0419f,-0.8423f,  
	-63.1478f,-26.9351f,0.6048f,  -61.4487f,-25.9633f,0.1915f,  -62.2487f,-23.8329f,1.6447f,  -60.6525f,-22.8255f,0.9819f,  -61.3414f,-20.6748f,2.4404f,  
	-59.8817f,-19.6396f,1.5460f,  -60.4612f,-17.4724f,3.0085f,  -59.1691f,-16.4159f,1.9055f,  -59.6408f,-14.2347f,3.3715f,  -58.5439f,-13.1620f,2.0877f,  
	-58.9089f,-10.9722f,3.5547f,  -58.0306f,-9.8873f,2.1217f,  -58.2903f,-7.6899f,3.5903f,  -57.6493f,-6.5955f,2.0432f,  -57.8043f,-4.3964f,3.5106f,  
	-57.4144f,-3.2947f,1.8869f,  -57.4656f,-1.0966f,3.3525f,  -57.3347f,0.0108f,1.6916f,  -57.2826f,2.2050f,3.1546f,  -57.4134f,3.3167f,1.4965f,  
	-57.2583f,5.5020f,2.9547f,  -57.6473f,6.6166f,1.3397f,  -57.3894f,8.7910f,2.7930f,  -58.0278f,9.9066f,1.2600f,  -57.6669f,12.0668f,2.7073f,  
	-58.5403f,13.1809f,1.2937f,  -58.0763f,15.3228f,2.7334f,  -59.1652f,16.4320f,1.4741f,  -58.5972f,18.5537f,2.9060f,  -59.8776f,19.6536f,1.8322f,  
	-59.2048f,21.7525f,3.2557f,  -60.6485f,22.8370f,2.3947f,  -59.8697f,24.9096f,3.8086f,  -61.4454f,25.9716f,3.1829f,  -60.5584f,28.0165f,4.5876f,  
	-62.2322f,29.0467f,4.2143f,  -61.2354f,31.0605f,5.6087f,  -62.9717f,32.0480f,5.4992f,  -61.8627f,34.0278f,6.8828f,  -63.6255f,34.9601f,7.0425f,  
	-62.4017f,36.9023f,8.4144f,  -64.1551f,37.7652f,8.8422f,  -62.8143f,39.6650f,10.2012f,  -64.5241f,40.4426f,10.8898f,  -63.0641f,42.2948f,12.2341f,  
	-64.6982f,42.9702f,13.1699f,  -63.1177f,44.7678f,14.4971f,  -64.6477f,45.3228f,15.6603f,  -62.9459f,47.0584f,16.9678f,  -64.3476f,47.4748f,18.3331f,  
	-62.5264f,49.1380f,19.6162f,  -63.7808f,49.3976f,21.1528f,  -61.8430f,50.9782f,22.4074f,  -62.9368f,51.0638f,24.0802f,  -60.8881f,52.5499f,25.3006f,  
	-61.8143f,52.4458f,27.0708f,  -59.6654f,53.8236f,28.2494f,  -60.4235f,53.5168f,30.0752f,  -58.1851f,54.7748f,31.2061f,  -58.7807f,54.2556f,33.0440f,  
	-56.4698f,55.3808f,34.1192f,  -56.9149f,54.6439f,35.9250f,  -54.5509f,55.6256f,36.9369f,  -54.8626f,54.6706f,38.6669f,  -52.4690f,55.4998f,39.6077f,  
	-52.6687f,54.3322f,41.2198f,  -50.2697f,55.0034f,42.0836f,  -50.3823f,53.6349f,43.5385f,  -48.0052f,54.1459f,44.3191f,  -48.0569f,52.5945f,45.5810f,  
	-45.7278f,52.9476f,46.2743f,  -45.7454f,51.2375f,47.3123f,  -43.4902f,51.4387f,47.9154f,  -43.4992f,49.6002f,48.7039f,  -41.3421f,49.6591f,49.2158f,  
	-41.3651f,47.7273f,49.7351f,  -39.3241f,47.6582f,50.1564f,  -39.3800f,45.6725f,50.3934f,  -37.4707f,45.4921f,50.7260f,  -37.5731f,43.4949f,50.6740f,  
	-35.8057f,43.2207f,50.9215f,  -35.9628f,41.2557f,50.5806f,  -34.3425f,40.9068f,50.7471f,  -34.5564f,39.0178f,50.1240f,  -33.0807f,38.6138f,50.2153f,  
	-33.3475f,36.8434f,49.3229f,  -32.0117f,36.4019f,49.3450f,  -32.3218f,34.7894f,48.2022f,  -31.1159f,34.3264f,48.1612f,  -31.4547f,32.9073f,46.7925f,  
	-30.3631f,32.4370f,46.6965f,  -65.4174f,-35.7620f,-4.0584f,  -66.6010f,-33.6492f,-2.5820f,  -64.7704f,-32.9162f,-2.2453f,  -65.8320f,-30.7712f,-0.7604f,  
	-64.0011f,-29.9681f,-0.6904f,  -64.9425f,-27.7968f,0.8007f,  -63.1478f,-26.9351f,0.6048f,  -63.9710f,-24.7432f,2.0997f,  -62.2487f,-23.8329f,1.6447f,  
	-62.9560f,-21.6252f,3.1417f,  -61.3414f,-20.6748f,2.4404f,  -61.9347f,-18.4559f,3.9377f,  -60.4612f,-17.4724f,3.0085f,  -60.9428f,-15.2459f,4.5049f,  
	-59.6408f,-14.2347f,3.3715f,  -60.0122f,-12.0068f,4.8641f,  -58.9089f,-10.9722f,3.5547f,  -59.1722f,-8.7447f,5.0435f,  -58.2903f,-7.6899f,3.5903f,  
	-58.4469f,-5.4690f,5.0723f,  -57.8043f,-4.3964f,3.5106f,  -57.8559f,-2.1853f,4.9848f,  -57.4656f,-1.0966f,3.3525f,  -57.4134f,1.1015f,4.8181f,  
	-57.2826f,2.2050f,3.1546f,  -57.1279f,4.3851f,4.6094f,  -57.2583f,5.5020f,2.9547f,  -57.0021f,7.6623f,4.3986f,  -57.3894f,8.7910f,2.7930f,  
	-57.0323f,10.9282f,4.2249f,  -57.6669f,12.0668f,2.7073f,  -57.2097f,14.1773f,4.1259f,  -58.0763f,15.3228f,2.7334f,  -57.5188f,17.4053f,4.1389f,  
	-58.5972f,18.5537f,2.9060f,  -57.9392f,20.6062f,4.2981f,  -59.2048f,21.7525f,3.2557f,  -58.4457f,23.7722f,4.6337f,  -59.8697f,24.9096f,3.8086f,  
	-59.0078f,26.8966f,5.1737f,  -60.5584f,28.0165f,4.5876f,  -59.5923f,29.9682f,5.9391f,  -61.2354f,31.0605f,5.6087f,  -60.1626f,32.9754f,6.9471f,  
	-61.8627f,34.0278f,6.8828f,  -60.6806f,35.9037f,8.2080f,  -62.4017f,36.9023f,8.4144f,  -61.1075f,38.7360f,9.7259f,  -62.8143f,39.6650f,10.2012f,  
	-61.4053f,41.4527f,11.4984f,  -63.0641f,42.2948f,12.2341f,  -61.5378f,44.0312f,13.5156f,  -63.1177f,44.7678f,14.4971f,  -61.4721f,46.4467f,15.7612f,  
	-62.9459f,47.0584f,16.9678f,  -61.1813f,48.6710f,18.2110f,  -62.5264f,49.1380f,19.6162f,  -60.6439f,50.6751f,20.8350f,  -61.8430f,50.9782f,22.4074f,  
	-59.8461f,52.4290f,23.5972f,  -60.8881f,52.5499f,25.3006f,  -58.7853f,53.9010f,26.4545f,  -59.6654f,53.8236f,28.2494f,  -57.4653f,55.0637f,29.3620f,  
	-58.1851f,54.7748f,31.2061f,  -55.9024f,55.8906f,32.2698f,  -56.4698f,55.3808f,34.1192f,  -54.1221f,56.3608f,35.1264f,  -54.5509f,55.6256f,36.9369f,  
	-52.1601f,56.4596f,37.8798f,  -52.4690f,55.4998f,39.6077f,  -50.0581f,56.1807f,40.4801f,  -50.2697f,55.0034f,42.0836f,  -47.8657f,55.5273f,42.8787f,  
	-48.0052f,54.1459f,44.3191f,  -45.6342f,54.5128f,45.0320f,  -45.7278f,52.9476f,46.2743f,  -43.4167f,53.1616f,46.9012f,  -43.4902f,51.4387f,47.9154f,  
	-41.2646f,51.5075f,48.4545f,  -41.3421f,49.6591f,49.2158f,  -39.2221f,49.5951f,49.6666f,  -39.3241f,47.6582f,50.1564f,  -37.3280f,47.4768f,50.5198f,  
	-37.4707f,45.4921f,50.7260f,  -35.6112f,45.2100f,51.0045f,  -35.8057f,43.2207f,50.9215f,  -34.0912f,42.8564f,51.1185f,  -34.3425f,40.9068f,50.7471f,  
	-32.7734f,40.4799f,50.8675f,  -33.0807f,38.6138f,50.2153f,  -31.6553f,38.1426f,50.2643f,  -32.0117f,36.4019f,49.3450f,  -30.7225f,35.9029f,49.3282f,  
	-31.1159f,34.3264f,48.1612f,  -29.9493f,33.8153f,48.0862f,  -30.3631f,32.4370f,46.6965f,  -29.3083f,31.9253f,46.5679f,  -66.6010f,-33.6492f,-2.5820f,  
	-67.6958f,-31.4374f,-1.0510f,  -65.8320f,-30.7712f,-0.7604f,  -66.8022f,-28.5338f,0.7760f,  -64.9425f,-27.7968f,0.8007f,  -65.7900f,-25.5408f,2.3393f,  
	-63.9710f,-24.7432f,2.0997f,  -64.6980f,-22.4745f,3.6381f,  -62.9560f,-21.6252f,3.1417f,  -63.5646f,-19.3497f,4.6771f,  -61.9347f,-18.4559f,3.9377f,  
	-62.4275f,-16.1784f,5.4683f,  -60.9428f,-15.2459f,4.5049f,  -61.3218f,-12.9732f,6.0274f,  -60.0122f,-12.0068f,4.8641f,  -60.2801f,-9.7418f,6.3779f,  
	-59.1722f,-8.7447f,5.0435f,  -59.3310f,-6.4941f,6.5452f,  -58.4469f,-5.4690f,5.0723f,  -58.4989f,-3.2367f,6.5607f,  -57.8559f,-2.1853f,4.9848f,  
	-57.8034f,0.0253f,6.4587f,  -57.4134f,1.1015f,4.8181f,  -57.2585f,3.2851f,6.2752f,  -57.1279f,4.3851f,4.6094f,  -56.8724f,6.5397f,6.0494f,  
	-57.0021f,7.6623f,4.3986f,  -56.6475f,9.7848f,5.8207f,  -57.0323f,10.9282f,4.2249f,  -56.5802f,13.0152f,5.6277f,  -57.2097f,14.1773f,4.1259f,  
	-56.6608f,16.2278f,5.5098f,  -57.5188f,17.4053f,4.1389f,  -56.8735f,19.4179f,5.5041f,  -57.9392f,20.6062f,4.2981f,  -57.1978f,22.5790f,5.6442f,  
	-58.4457f,23.7722f,4.6337f,  -57.6068f,25.7061f,5.9624f,  -59.0078f,26.8966f,5.1737f,  -58.0705f,28.7899f,6.4849f,  -59.5923f,29.9682f,5.9391f,  
	-58.5542f,31.8209f,7.2341f,  -60.1626f,32.9754f,6.9471f,  -59.0212f,34.7864f,8.2264f,  -60.6806f,35.9037f,8.2080f,  -59.4329f,37.6714f,9.4722f,  
	-61.1075f,38.7360f,9.7259f,  -59.7502f,40.4581f,10.9753f,  -61.4053f,41.4527f,11.4984f,  -59.9350f,43.1256f,12.7326f,  -61.5378f,44.0312f,13.5156f,  
	-59.9515f,45.6502f,14.7338f,  -61.4721f,46.4467f,15.7612f,  -59.7685f,48.0046f,16.9611f,  -61.1813f,48.6710f,18.2110f,  -59.3599f,50.1598f,19.3900f,  
	-60.6439f,50.6751f,20.8350f,  -58.7061f,52.0852f,21.9896f,  -59.8461f,52.4290f,23.5972f,  -57.7982f,53.7477f,24.7213f,  -58.7853f,53.9010f,26.4545f,  
	-56.6337f,55.1174f,27.5434f,  -57.4653f,55.0637f,29.3620f,  -55.2222f,56.1646f,30.4085f,  -55.9024f,55.8906f,32.2698f,  -53.5834f,56.8638f,33.2666f,  
	-54.1221f,56.3608f,35.1264f,  -51.7473f,57.1952f,36.0658f,  -52.1601f,56.4596f,37.8798f,  -49.7511f,57.1467f,38.7553f,  -50.0581f,56.1807f,40.4801f,  
	-47.6409f,56.7148f,41.2846f,  -47.8657f,55.5273f,42.8787f,  -45.4662f,55.9065f,43.6066f,  -45.6342f,54.5128f,45.0320f,  -43.2799f,54.7392f,45.6788f,  
	-43.4167f,53.1616f,46.9012f,  -41.1341f,53.2408f,47.4642f,  -41.2646f,51.5075f,48.4545f,  -39.0755f,51.4502f,48.9322f,  -39.2221f,49.5951f,49.6666f,  
	-37.1472f,49.4151f,50.0588f,  -37.3280f,47.4768f,50.5198f,  -35.3827f,47.1896f,50.8283f,  -35.6112f,45.2100f,51.0045f,  -33.8067f,44.8333f,51.2318f,  
	-34.0912f,42.8564f,51.1185f,  -32.4309f,42.4097f,51.2684f,  -32.7734f,40.4799f,50.8675f,  -31.2587f,39.9816f,50.9445f,  -31.6553f,38.1426f,50.2643f,  
	-30.2811f,37.6099f,50.2735f,  -30.7225f,35.9029f,49.3282f,  -29.4775f,35.3526f,49.2763f,  -29.9493f,33.8153f,48.0862f,  -28.8251f,33.2599f,47.9777f,  
	-29.3083f,31.9253f,46.5679f,  -28.2871f,31.3763f,46.4104f,  -67.6958f,-31.4374f,-1.0510f,  -68.6949f,-29.1336f,0.5309f,  -66.8022f,-28.5338f,0.7760f,  
	-67.6745f,-26.2119f,2.3594f,  -65.7900f,-25.5408f,2.3393f,  -66.5375f,-23.2081f,3.9209f,  -64.6980f,-22.4745f,3.6381f,  -65.3229f,-20.1384f,5.2142f,  
	-63.5646f,-19.3497f,4.6771f,  -64.0696f,-17.0161f,6.2453f,  -62.4275f,-16.1784f,5.4683f,  -62.8150f,-13.8550f,7.0247f,  -61.3218f,-12.9732f,6.0274f,  
	-61.5949f,-10.6641f,7.5706f,  -60.2801f,-9.7418f,6.3779f,  -60.4415f,-7.4542f,7.9043f,  -59.3310f,-6.4941f,6.5452f,  -59.3837f,-4.2327f,8.0530f,  
	-58.4989f,-3.2367f,6.5607f,  -58.4460f,-1.0054f,8.0484f,  -57.8034f,0.0253f,6.4587f,  -57.6476f,2.2207f,7.9238f,  -57.2585f,3.2851f,6.2752f,  
	-57.0026f,5.4425f,7.7171f,  -56.8724f,6.5397f,6.0494f,  -56.5189f,8.6559f,7.4673f,  -56.6475f,9.7848f,5.8207f,  -56.1987f,11.8564f,7.2132f,  
	-56.5802f,13.0152f,5.6277f,  -56.0378f,15.0416f,6.9954f,  -56.6608f,16.2278f,5.5098f,  -56.0260f,18.2079f,6.8530f,  -56.8735f,19.4179f,5.5041f,  
	-56.1474f,21.3502f,6.8227f,  -57.1978f,22.5790f,5.6442f,  -56.3795f,24.4654f,6.9404f,  -57.6068f,25.7061f,5.9624f,  -56.6960f,27.5458f,7.2366f,  
	-58.0705f,28.7899f,6.4849f,  -57.0651f,30.5840f,7.7390f,  -58.5542f,31.8209f,7.2341f,  -57.4520f,33.5696f,8.4695f,  -59.0212f,34.7864f,8.2264f,  
	-57.8191f,36.4894f,9.4443f,  -59.4329f,37.6714f,9.4722f,  -58.1271f,39.3281f,10.6739f,  -59.7502f,40.4581f,10.9753f,  -58.3367f,42.0666f,12.1615f,  
	-59.9350f,43.1256f,12.7326f,  -58.4100f,44.6826f,13.9033f,  -59.9515f,45.6502f,14.7338f,  -58.3123f,47.1503f,15.8880f,  -59.7685f,48.0046f,16.9611f,  
	-58.0131f,49.4410f,18.0971f,  -59.3599f,50.1598f,19.3900f,  -57.4875f,51.5244f,20.5054f,  -58.7061f,52.0852f,21.9896f,  -56.7210f,53.3663f,23.0793f,  
	-57.7982f,53.7477f,24.7213f,  -55.7044f,54.9350f,25.7812f,  -56.6337f,55.1174f,27.5434f,  -54.4411f,56.1979f,28.5671f,  -55.2222f,56.1646f,30.4085f,  
	-52.9441f,57.1258f,31.3892f,  -53.5834f,56.8638f,33.2666f,  -51.2377f,57.6940f,34.1967f,  -51.7473f,57.1952f,36.0658f,  -49.3538f,57.8848f,36.9389f,  
	-49.7511f,57.1467f,38.7553f,  -47.3346f,57.6883f,39.5639f,  -47.6409f,56.7148f,41.2846f,  -45.2269f,57.1047f,42.0226f,  -45.4662f,55.9065f,43.6066f,  
	-43.0821f,56.1447f,44.2689f,  -43.2799f,54.7392f,45.6788f,  -40.9526f,54.8294f,46.2620f,  -41.1341f,53.2408f,47.4642f,  -38.8871f,53.1917f,47.9659f,  
	-39.0755f,51.4502f,48.9322f,  -36.9317f,51.2738f,49.3511f,  -37.1472f,49.4151f,50.0588f,  -35.1239f,49.1256f,50.3958f,  -35.3827f,47.1896f,50.8283f,  
	-33.4940f,46.8038f,51.0849f,  -33.8067f,44.8333f,51.2318f,  -32.0590f,44.3702f,51.4110f,  -32.4309f,42.4097f,51.2684f,  -30.8286f,41.8876f,51.3740f,  
	-31.2587f,39.9816f,50.9445f,  -29.7992f,39.4181f,50.9809f,  -30.2811f,37.6099f,50.2735f,  -28.9559f,37.0226f,50.2464f,  -29.4775f,35.3526f,49.2763f,  
	-28.2804f,34.7550f,49.1901f,  -28.8251f,33.2599f,47.9777f,  -27.7399f,32.6651f,47.8393f,  -28.2871f,31.3763f,46.4104f,  -27.2982f,30.7938f,46.2263f,  
	-68.6949f,-29.1336f,0.5309f,  -69.5922f,-26.7451f,2.1596f,  -67.6745f,-26.2119f,2.3594f,  -68.4431f,-23.8136f,3.9853f,  -66.5375f,-23.2081f,3.9209f,  
	-67.1795f,-20.8085f,5.5398f,  -65.3229f,-20.1384f,5.2142f,  -65.8410f,-17.7449f,6.8227f,  -64.0696f,-17.0161f,6.2453f,  -64.4665f,-14.6371f,7.8390f,  
	-62.8150f,-13.8550f,7.0247f,  -63.0942f,-11.4959f,8.6014f,  -61.5949f,-10.6641f,7.5706f,  -61.7595f,-8.3328f,9.1262f,  -60.4415f,-7.4542f,7.9043f,  
	-60.4952f,-5.1563f,9.4364f,  -59.3837f,-4.2327f,8.0530f,  -59.3302f,-1.9729f,9.5598f,  -58.4460f,-1.0054f,8.0484f,  -58.2888f,1.2102f,9.5268f,  
	-57.6476f,2.2207f,7.9238f,  -57.3905f,4.3895f,9.3733f,  -57.0026f,5.4425f,7.7171f,  -56.6488f,7.5610f,9.1365f,  -56.5189f,8.6559f,7.4673f,  
	-56.0717f,10.7207f,8.8552f,  -56.1987f,11.8564f,7.2132f,  -55.6607f,13.8668f,8.5701f,  -56.0378f,15.0416f,6.9954f,  -55.4110f,16.9969f,8.3219f,  
	-56.0260f,18.2079f,6.8530f,  -55.3123f,20.1070f,8.1490f,  -56.1474f,21.3502f,6.8227f,  -55.3468f,23.1957f,8.0909f,  -56.3795f,24.4654f,6.9404f,  
	-55.4924f,26.2571f,8.1815f,  -56.6960f,27.5458f,7.2366f,  -55.7208f,29.2860f,8.4532f,  -57.0651f,30.5840f,7.7390f,  -55.9999f,32.2740f,8.9329f,  
	-57.4520f,33.5696f,8.4695f,  -56.2937f,35.2104f,9.6428f,  -57.8191f,36.4894f,9.4443f,  -56.5638f,38.0821f,10.5993f,  -58.1271f,39.3281f,10.6739f,  
	-56.7704f,40.8722f,11.8120f,  -58.3367f,42.0666f,12.1615f,  -56.8741f,43.5604f,13.2839f,  -58.4100f,44.6826f,13.9033f,  -56.8374f,46.1226f,15.0101f,  
	-58.3123f,47.1503f,15.8880f,  -56.6264f,48.5311f,16.9785f,  -58.0131f,49.4410f,18.0971f,  -56.2114f,50.7562f,19.1700f,  -57.4875f,51.5244f,20.5054f,  
	-55.5720f,52.7634f,21.5566f,  -56.7210f,53.3663f,23.0793f,  -54.6935f,54.5196f,24.1056f,  -55.7044f,54.9350f,25.7812f,  -53.5723f,55.9901f,26.7771f,  
	-54.4411f,56.1979f,28.5671f,  -52.2151f,57.1424f,29.5263f,  -52.9441f,57.1258f,31.3892f,  -50.6400f,57.9474f,32.3045f,  -51.2377f,57.6940f,34.1967f,  
	-48.8731f,58.3823f,35.0618f,  -49.3538f,57.8848f,36.9389f,  -46.9520f,58.4310f,37.7462f,  -47.3346f,57.6883f,39.5639f,  -44.9201f,58.0869f,40.3070f,  
	-45.2269f,57.1047f,42.0226f,  -42.8266f,57.3539f,42.6959f,  -43.0821f,56.1447f,44.2689f,  -40.7233f,56.2462f,44.8683f,  -40.9526f,54.8294f,46.2620f,  
	-38.6599f,54.7898f,46.7841f,  -38.8871f,53.1917f,47.9659f,  -36.6849f,53.0210f,48.4086f,  -36.9317f,51.2738f,49.3511f,  -34.8395f,50.9849f,49.7144f,  
	-35.1239f,49.1256f,50.3958f,  -33.1583f,48.7341f,50.6806f,  -33.4940f,46.8038f,51.0849f,  -31.6641f,46.3278f,51.2930f,  -32.0590f,44.3702f,51.4110f,  
	-30.3723f,43.8278f,51.5453f,  -30.8286f,41.8876f,51.3740f,  -29.2852f,41.2966f,51.4383f,  -29.7992f,39.4181f,50.9809f,  -28.3933f,38.7967f,50.9800f,  
	-28.9559f,37.0226f,50.2464f,  -27.6836f,36.3851f,50.1844f,  -28.2804f,34.7550f,49.1901f,  -27.1275f,34.1159f,49.0732f,  -27.7399f,32.6651f,47.8393f,  
	-26.6922f,32.0351f,47.6734f,  -27.2982f,30.7938f,46.2263f,  -26.3402f,30.1811f,46.0178f,  -69.5922f,-26.7451f,2.1596f,  -70.3822f,-24.2805f,3.8303f,  
	-68.4431f,-23.8136f,3.9853f,  -69.1027f,-21.3487f,5.6482f,  -67.1795f,-20.8085f,5.5398f,  -67.7113f,-18.3521f,7.1904f,  -65.8410f,-17.7449f,6.8227f,  
	-66.2479f,-15.3061f,8.4563f,  -64.4665f,-14.6371f,7.8390f,  -64.7522f,-12.2229f,9.4524f,  -63.0942f,-11.4959f,8.6014f,  -63.2623f,-9.1150f,10.1901f,  
	-61.7595f,-8.3328f,9.1262f,  -61.8142f,-5.9917f,10.6872f,  -60.4952f,-5.1563f,9.4364f,  -60.4408f,-2.8605f,10.9671f,  -59.3302f,-1.9729f,9.5598f,  
	-59.1711f,0.2709f,11.0570f,  -58.2888f,1.2102f,9.5268f,  -58.0294f,3.3987f,10.9895f,  -57.3905f,4.3895f,9.3733f,  -57.0349f,6.5188f,10.7999f,  
	-56.6488f,7.5610f,9.1365f,  -56.2012f,9.6276f,10.5256f,  -56.0717f,10.7207f,8.8552f,  -55.5357f,12.7236f,10.2071f,  -55.6607f,13.8668f,8.5701f,  
	-55.0392f,15.8056f,9.8854f,  -55.4110f,16.9969f,8.3219f,  -54.7070f,18.8706f,9.6006f,  -55.3123f,20.1070f,8.1490f,  -54.5265f,21.9187f,9.3941f,  
	-55.3468f,23.1957f,8.0909f,  -54.4803f,24.9459f,9.3033f,  -55.4924f,26.2571f,8.1815f,  -54.5442f,27.9490f,9.3644f,  -55.7208f,29.2860f,8.4532f,  
	-54.6896f,30.9218f,9.6089f,  -55.9999f,32.2740f,8.9329f,  -54.8829f,33.8561f,10.0642f,  -56.2937f,35.2104f,9.6428f,  -55.0870f,36.7413f,10.7527f,  
	-56.5638f,38.0821f,10.5993f,  -55.2628f,39.5629f,11.6902f,  -56.7704f,40.8722f,11.8120f,  -55.3700f,42.3028f,12.8861f,  -56.8741f,43.5604f,13.2839f,  
	-55.3693f,44.9391f,14.3423f,  -56.8374f,46.1226f,15.0101f,  -55.2236f,47.4457f,16.0532f,  -56.6264f,48.5311f,16.9785f,  -54.8994f,49.7937f,18.0062f,  
	-56.2114f,50.7562f,19.1700f,  -54.3711f,51.9492f,20.1794f,  -55.5720f,52.7634f,21.5566f,  -53.6181f,53.8782f,22.5453f,  -54.6935f,54.5196f,24.1056f,  
	-52.6307f,55.5447f,25.0691f,  -53.5723f,55.9901f,26.7771f,  -51.4090f,56.9134f,27.7098f,  -52.2151f,57.1424f,29.5263f,  -49.9644f,57.9513f,30.4215f,  
	-50.6400f,57.9474f,32.3045f,  -48.3172f,58.6309f,33.1562f,  -48.8731f,58.3823f,35.0618f,  -46.4998f,58.9302f,35.8625f,  -46.9520f,58.4310f,37.7462f,  
	-44.5512f,58.8365f,38.4893f,  -44.9201f,58.0869f,40.3070f,  -42.5180f,58.3462f,40.9864f,  -42.8266f,57.3539f,42.6959f,  -40.4502f,57.4668f,43.3070f,  
	-40.7233f,56.2462f,44.8683f,  -38.3979f,56.2171f,45.4071f,  -38.6599f,54.7898f,46.7841f,  -36.4112f,54.6270f,47.2475f,  -36.6849f,53.0210f,48.4086f,  
	-34.5341f,52.7356f,48.7959f,  -34.8395f,50.9849f,49.7144f,  -32.8054f,50.5911f,50.0256f,  -33.1583f,48.7341f,50.6806f,  -31.2530f,48.2491f,50.9164f,  
	-31.6641f,46.3278f,51.2930f,  -29.8978f,45.7692f,51.4557f,  -30.3723f,43.8278f,51.5453f,  -28.7477f,43.2132f,51.6378f,  -29.2852f,41.2966f,51.4383f,  
	-27.7993f,40.6445f,51.4645f,  -28.3933f,38.7967f,50.9800f,  -27.0448f,38.1222f,50.9437f,  -27.6836f,36.3851f,50.1844f,  -26.4605f,35.7037f,50.0910f,  
	-27.1275f,34.1159f,49.0732f,  -26.0173f,33.4396f,48.9280f,  -26.6922f,32.0351f,47.6734f,  -25.6809f,31.3737f,47.4824f,  -26.3402f,30.1811f,46.0178f,  
	-25.4124f,29.5412f,45.7867f,  -70.3822f,-24.2805f,3.8303f,  -71.0595f,-21.7496f,5.5376f,  -69.1027f,-21.3487f,5.6482f,  -69.6486f,-18.8274f,7.3423f,  
	-67.7113f,-18.3521f,7.1904f,  -68.1288f,-15.8505f,8.8661f,  -66.2479f,-15.3061f,8.4563f,  -66.5408f,-12.8327f,10.1092f,  -64.7522f,-12.2229f,9.4524f,  
	-64.9243f,-9.7873f,11.0776f,  -63.2623f,-9.1150f,10.1901f,  -63.3183f,-6.7247f,11.7838f,  -61.8142f,-5.9917f,10.6872f,  -61.7589f,-3.6532f,12.2464f,  
	-60.4408f,-2.8605f,10.9671f,  -60.2793f,-0.5812f,12.4881f,  -59.1711f,0.2709f,11.0570f,  -58.9084f,2.4869f,12.5381f,  -58.0294f,3.3987f,10.9895f,  
	-57.6707f,5.5471f,12.4289f,  -57.0349f,6.5188f,10.7999f,  -56.5852f,8.5956f,12.1959f,  -56.2012f,9.6276f,10.5256f,  -55.6650f,11.6316f,11.8783f,  
	-55.5357f,12.7236f,10.2071f,  -54.9169f,14.6544f,11.5169f,  -55.0392f,15.8056f,9.8854f,  -54.3417f,17.6621f,11.1524f,  -54.7070f,18.8706f,9.6006f,  
	-53.9325f,20.6564f,10.8279f,  -54.5265f,21.9187f,9.3941f,  -53.6769f,23.6346f,10.5829f,  -54.4803f,24.9459f,9.3033f,  -53.5555f,26.5960f,10.4570f,  
	-54.5442f,27.9490f,9.3644f,  -53.5435f,29.5363f,10.4858f,  -54.6896f,30.9218f,9.6089f,  -53.6107f,32.4498f,10.7014f,  -54.8829f,33.8561f,10.0642f,  
	-53.7221f,35.3286f,11.1315f,  -55.0870f,36.7413f,10.7527f,  -53.8396f,38.1610f,11.7982f,  -55.2628f,39.5629f,11.6902f,  -53.9234f,40.9313f,12.7168f,  
	-55.3700f,42.3028f,12.8861f,  -53.9328f,43.6201f,13.8961f,  -55.3693f,44.9391f,14.3423f,  -53.8288f,46.2032f,15.3372f,  -55.2236f,47.4457f,16.0532f,  
	-53.5740f,48.6534f,17.0340f,  -54.8994f,49.7937f,18.0062f,  -53.1386f,50.9376f,18.9711f,  -54.3711f,51.9492f,20.1794f,  -52.4967f,53.0219f,21.1271f,  
	-53.6181f,53.8782f,22.5453f,  -51.6324f,54.8694f,23.4724f,  -52.6307f,55.5447f,25.0691f,  -50.5394f,56.4425f,25.9707f,  -51.4090f,56.9134f,27.7098f,  
	-49.2225f,57.7054f,28.5799f,  -49.9644f,57.9513f,30.4215f,  -47.6957f,58.6261f,31.2547f,  -48.3172f,58.6309f,33.1562f,  -45.9856f,59.1775f,33.9451f,  
	-46.4998f,58.9302f,35.8625f,  -44.1265f,59.3404f,36.6004f,  -44.5512f,58.8365f,38.4893f,  -42.1614f,59.1046f,39.1699f,  -42.5180f,58.3462f,40.9864f,  
	-40.1379f,58.4704f,41.6048f,  -40.4502f,57.4668f,43.3070f,  -38.1055f,57.4493f,43.8584f,  -38.3979f,56.2171f,45.4071f,  -36.1153f,56.0642f,45.8877f,  
	-36.4112f,54.6270f,47.2475f,  -34.2132f,54.3479f,47.6559f,  -34.5341f,52.7356f,48.7959f,  -32.4414f,52.3431f,49.1311f,  -32.8054f,50.5911f,50.0256f,  
	-30.8327f,50.1011f,50.2876f,  -31.2530f,48.2491f,50.9164f,  -29.4131f,47.6785f,51.1065f,  -29.8978f,45.7692f,51.4557f,  -28.1961f,45.1352f,51.5761f,  
	-28.7477f,43.2132f,51.6378f,  -27.1842f,42.5345f,51.6916f,  -27.7993f,40.6445f,51.4645f,  -26.3751f,39.9367f,51.4549f,  -27.0448f,38.1222f,50.9437f,  
	-25.7502f,37.4015f,50.8753f,  -26.4605f,35.7037f,50.0910f,  -25.2850f,34.9833f,49.9686f,  -26.0173f,33.4396f,48.9280f,  -24.9487f,32.7308f,48.7570f,  
	-25.6809f,31.3737f,47.4824f,  -24.7048f,30.6843f,47.2681f,  -25.4124f,29.5412f,45.7867f,  -24.5119f,28.8769f,45.5360f,  -71.0595f,-21.7496f,5.5376f,  
	-71.6197f,-19.1627f,7.2758f,  -69.6486f,-18.8274f,7.3423f,  -70.0770f,-16.2611f,9.0613f,  -68.1288f,-15.8505f,8.8661f,  -68.4291f,-13.3149f,10.5606f,  
	-66.5408f,-12.8327f,10.1092f,  -66.7171f,-10.3382f,11.7738f,  -64.9243f,-9.7873f,11.0776f,  -64.9817f,-7.3426f,12.7077f,  -63.3183f,-6.7247f,11.7838f,  
	-63.2619f,-4.3374f,13.3755f,  -61.7589f,-3.6532f,12.2464f,  -61.5944f,-1.3315f,13.7956f,  -60.2793f,-0.5812f,12.4881f,  -60.0125f,1.6699f,13.9926f,  
	-58.9084f,2.4869f,12.5381f,  -58.5453f,4.6625f,13.9956f,  -57.6707f,5.5471f,12.4289f,  -57.2171f,7.6426f,13.8374f,  -56.5852f,8.5956f,12.1959f,  
	-56.0465f,10.6094f,13.5551f,  -55.6650f,11.6316f,11.8783f,  -55.0462f,13.5630f,13.1885f,  -54.9169f,14.6544f,11.5169f,  -54.2228f,16.5023f,12.7781f,  
	-54.3417f,17.6621f,11.1524f,  -53.5750f,19.4302f,12.3676f,  -53.9325f,20.6564f,10.8279f,  -53.0961f,22.3457f,11.9984f,  -53.6769f,23.6346f,10.5829f,  
	-52.7717f,25.2500f,11.7124f,  -53.5555f,26.5960f,10.4570f,  -52.5814f,28.1410f,11.5486f,  -53.5435f,29.5363f,10.4858f,  -52.4989f,31.0155f,11.5433f,  
	-53.6107f,32.4498f,10.7014f,  -52.4923f,33.8684f,11.7293f,  -53.7221f,35.3286f,11.1315f,  -52.5253f,36.6907f,12.1340f,  -53.8396f,38.1610f,11.7982f,  
	-52.5592f,39.4695f,12.7788f,  -53.9234f,40.9313f,12.7168f,  -52.5528f,42.1881f,13.6792f,  -53.9328f,43.6201f,13.8961f,  -52.4657f,44.8250f,14.8425f,  
	-53.8288f,46.2032f,15.3372f,  -52.2580f,47.3548f,16.2699f,  -53.5740f,48.6534f,17.0340f,  -51.8957f,49.7461f,17.9524f,  -53.1386f,50.9376f,18.9711f,  
	-51.3485f,51.9654f,19.8751f,  -52.4967f,53.0219f,21.1271f,  -50.5945f,53.9756f,22.0143f,  -51.6324f,54.8694f,23.4724f,  -49.6213f,55.7380f,24.3389f,  
	-50.5394f,56.4425f,25.9707f,  -48.4270f,57.2140f,26.8114f,  -49.2225f,57.7054f,28.5799f,  -47.0191f,58.3682f,29.3898f,  -47.6957f,58.6261f,31.2547f,  
	-45.4182f,59.1686f,32.0267f,  -45.9856f,59.1775f,33.9451f,  -43.6533f,59.5901f,34.6728f,  -44.1265f,59.3404f,36.6004f,  -41.7630f,59.6162f,37.2774f,  
	-42.1614f,59.1046f,39.1699f,  -39.7918f,59.2398f,39.7909f,  -40.1379f,58.4704f,41.6048f,  -37.7878f,58.4652f,42.1646f,  -38.1055f,57.4493f,43.8584f,  
	-35.8021f,57.3079f,44.3524f,  -36.1153f,56.0642f,45.8877f,  -33.8820f,55.7941f,46.3139f,  -34.2132f,54.3479f,47.6559f,  -32.0725f,53.9602f,48.0126f,  
	-32.4414f,52.3431f,49.1311f,  -30.4104f,51.8522f,49.4173f,  -30.8327f,50.1011f,50.2876f,  -28.9265f,49.5229f,50.5037f,  -29.4131f,47.6785f,51.1065f,  
	-27.6397f,47.0296f,51.2542f,  -28.1961f,45.1352f,51.5761f,  -26.5584f,44.4344f,51.6574f,  -27.1842f,42.5345f,51.6916f,  -25.6858f,41.7974f,51.7093f,  
	-26.3751f,39.9367f,51.4549f,  -25.0089f,39.1802f,51.4127f,  -25.7502f,37.4015f,50.8753f,  -24.5078f,36.6398f,50.7773f,  -25.2850f,34.9833f,49.9686f,  
	-24.1562f,34.2290f,49.8196f,  -24.9487f,32.7308f,48.7570f,  -23.9203f,31.9929f,48.5620f,  -24.7048f,30.6843f,47.2681f,  -23.7608f,29.9703f,47.0335f,  
	-24.5119f,28.8769f,45.5360f,  -23.6389f,28.1908f,45.2666f,  -71.6197f,-19.1627f,7.2758f,  -72.0591f,-16.5308f,9.0387f,  -70.0770f,-16.2611f,9.0613f,  
	-70.3850f,-13.6613f,10.7987f,  -68.4291f,-13.3149f,10.5606f,  -68.6099f,-10.7584f,12.2665f,  -66.7171f,-10.3382f,11.7738f,  -66.7759f,-7.8349f,13.4429f,  
	-64.9817f,-7.3426f,12.7077f,  -64.9241f,-4.9013f,14.3354f,  -63.2619f,-4.3374f,13.3755f,  -63.0940f,-1.9672f,14.9571f,  -61.5944f,-1.3315f,13.7956f,  
	-61.3228f,0.9616f,15.3282f,  -60.0125f,1.6699f,13.9926f,  -59.6436f,3.8801f,15.4734f,  -58.5453f,4.6625f,13.9956f,  -58.0859f,6.7847f,15.4221f,  
	-57.2171f,7.6426f,13.8374f,  -56.6736f,9.6746f,15.2089f,  -56.0465f,10.6094f,13.5551f,  -55.4248f,12.5500f,14.8716f,  -55.0462f,13.5630f,13.1885f,  
	-54.3521f,15.4109f,14.4497f,  -54.2228f,16.5023f,12.7781f,  -53.4601f,18.2612f,13.9870f,  -53.5750f,19.4302f,12.3676f,  -52.7477f,21.1013f,13.5255f,  
	-53.0961f,22.3457f,11.9984f,  -52.2060f,23.9341f,13.1090f,  -52.7717f,25.2500f,11.7124f,  -51.8196f,26.7598f,12.7791f,  -52.5814f,28.1410f,11.5486f,  
	-51.5667f,29.5778f,12.5757f,  -52.4989f,31.0155f,11.5433f,  -51.4187f,32.3854f,12.5358f,  -52.4923f,33.8684f,11.7293f,  -51.3424f,35.1770f,12.6920f,  
	-52.5253f,36.6907f,12.1340f,  -51.3006f,37.9422f,13.0712f,  -52.5592f,39.4695f,12.7788f,  -51.2527f,40.6677f,13.6951f,  -52.5528f,42.1881f,13.6792f,  
	-51.1577f,43.3346f,14.5779f,  -52.4657f,44.8250f,14.8425f,  -50.9738f,45.9202f,15.7270f,  -52.2580f,47.3548f,16.2699f,  -50.6637f,48.3949f,17.1409f,  
	-51.8957f,49.7461f,17.9524f,  -50.1931f,50.7267f,18.8109f,  -51.3485f,51.9654f,19.8751f,  -49.5347f,52.8788f,20.7198f,  -50.5945f,53.9756f,22.0143f,  
	-48.6703f,54.8118f,22.8423f,  -49.6213f,55.7380f,24.3389f,  -47.5915f,56.4855f,25.1461f,  -48.4270f,57.2140f,26.8114f,  -46.2991f,57.8615f,27.5934f,  
	-47.0191f,58.3682f,29.3898f,  -44.8073f,58.9036f,30.1402f,  -45.4182f,59.1686f,32.0267f,  -43.1396f,59.5814f,32.7393f,  -43.6533f,59.5901f,34.6728f,  
	-41.3295f,59.8721f,35.3412f,  -41.7630f,59.6162f,37.2774f,  -39.4177f,59.7619f,37.8963f,  -39.7918f,59.2398f,39.7909f,  -37.4502f,59.2475f,40.3545f,  
	-37.7878f,58.4652f,42.1646f,  -35.4771f,58.3371f,42.6679f,  -35.8021f,57.3079f,44.3524f,  -33.5464f,57.0495f,44.7928f,  -33.8820f,55.7941f,46.3139f,  
	-31.7050f,55.4145f,46.6890f,  -32.0725f,53.9602f,48.0126f,  -29.9933f,53.4724f,48.3205f,  -30.4104f,51.8522f,49.4173f,  -28.4463f,51.2710f,49.6578f,  
	-28.9265f,49.5229f,50.5037f,  -27.0881f,48.8641f,50.6777f,  -27.6397f,47.0296f,51.2542f,  -25.9327f,46.3115f,51.3625f,  -26.5584f,44.4344f,51.6574f,  
	-24.9887f,43.6725f,51.7026f,  -25.6858f,41.7974f,51.7093f,  -24.2491f,41.0093f,51.6940f,  -25.0089f,39.1802f,51.4127f,  -23.6990f,38.3807f,51.3404f,  
	-24.5078f,36.6398f,50.7773f,  -23.3168f,35.8428f,50.6522f,  -24.1562f,34.2290f,49.8196f,  -23.0721f,33.4445f,49.6462f,  -23.9203f,31.9929f,48.5620f,  
	-22.9286f,31.2299f,48.3460f,  -23.7608f,29.9703f,47.0335f,  -22.8490f,29.2343f,46.7796f,  -23.6389f,28.1908f,45.2666f,  -22.7905f,27.4853f,44.9813f,  
	-72.0591f,-16.5308f,9.0387f,  -72.3749f,-13.8658f,10.8196f,  -70.3850f,-13.6613f,10.7987f,  -70.5704f,-11.0405f,12.5475f,  -68.6099f,-10.7584f,12.2665f,  
	-68.6703f,-8.1932f,13.9768f,  -66.7759f,-7.8349f,13.4429f,  -66.7170f,-5.3354f,15.1094f,  -64.9241f,-4.9013f,14.3354f,  -64.7525f,-2.4773f,15.9529f,  
	-63.0940f,-1.9672f,14.9571f,  -62.8167f,0.3741f,16.5219f,  -61.3228f,0.9616f,15.3282f,  -60.9470f,3.2135f,16.8367f,  -59.6436f,3.8801f,15.4734f,  
	-59.1770f,6.0368f,16.9229f,  -58.0859f,6.7847f,15.4221f,  -57.5356f,8.8430f,16.8113f,  -56.6736f,9.6746f,15.2089f,  -56.0464f,11.6329f,16.5373f,  
	-55.4248f,12.5500f,14.8716f,  -54.7277f,14.4067f,16.1387f,  -54.3521f,15.4109f,14.4497f,  -53.5898f,17.1693f,15.6583f,  -53.4601f,18.2612f,13.9870f,  
	-52.6378f,19.9225f,15.1381f,  -52.7477f,21.1013f,13.5255f,  -51.8682f,22.6708f,14.6230f,  -52.2060f,23.9341f,13.1090f,  -51.2712f,25.4165f,14.1563f,  
	-51.8196f,26.7598f,12.7791f,  -50.8298f,28.1612f,13.7809f,  -51.5667f,29.5778f,12.5757f,  -50.5197f,30.9052f,13.5372f,  -51.4187f,32.3854f,12.5358f,  
	-50.3110f,33.6457f,13.4626f,  -51.3424f,35.1770f,12.6920f,  -50.1692f,36.3756f,13.5889f,  -51.3006f,37.9422f,13.0712f,  -50.0548f,39.0849f,13.9438f,  
	-51.2527f,40.6677f,13.6951f,  -49.9272f,41.7576f,14.5476f,  -51.1577f,43.3346f,14.5779f,  -49.7434f,44.3740f,15.4147f,  -50.9738f,45.9202f,15.7270f,  
	-49.4637f,46.9072f,16.5503f,  -50.6637f,48.3949f,17.1409f,  -49.0503f,49.3269f,17.9525f,  -50.1931f,50.7267f,18.8109f,  -48.4713f,51.5975f,19.6109f,  
	-49.5347f,52.8788f,20.7198f,  -47.7028f,53.6797f,21.5064f,  -48.6703f,54.8118f,22.8423f,  -46.7302f,55.5324f,23.6125f,  -47.5915f,56.4855f,25.1461f,  
	-45.5479f,57.1147f,25.8963f,  -46.2991f,57.8615f,27.5934f,  -44.1635f,58.3870f,28.3179f,  -44.8073f,58.9036f,30.1402f,  -42.5943f,59.3143f,30.8332f,  
	-43.1396f,59.5814f,32.7393f,  -40.8685f,59.8680f,33.3945f,  -41.3295f,59.8721f,35.3412f,  -39.0221f,60.0276f,35.9534f,  -39.4177f,59.7619f,37.8963f,  
	-37.0984f,59.7828f,38.4593f,  -37.4502f,59.2475f,40.3545f,  -35.1459f,59.1339f,40.8631f,  -35.4771f,58.3371f,42.6679f,  -33.2120f,58.0926f,43.1188f,  
	-33.5464f,57.0495f,44.7928f,  -31.3452f,56.6814f,45.1831f,  -31.7050f,55.4145f,46.6890f,  -29.5885f,54.9340f,47.0159f,  -29.9933f,53.4724f,48.3205f,  
	-27.9808f,52.8929f,48.5831f,  -28.4463f,51.2710f,49.6578f,  -26.5506f,50.6073f,49.8563f,  -27.0881f,48.8641f,50.6777f,  -25.3177f,48.1338f,50.8120f,  
	-25.9327f,46.3115f,51.3625f,  -24.2958f,45.5300f,51.4347f,  -24.9887f,43.6725f,51.7026f,  -23.4840f,42.8576f,51.7143f,  -24.2491f,41.0093f,51.6940f,  
	-22.8725f,40.1764f,51.6482f,  -23.6990f,38.3807f,51.3404f,  -22.4449f,37.5445f,51.2405f,  -23.3168f,35.8428f,50.6522f,  -22.1749f,35.0144f,50.5022f,  
	-23.0721f,33.4445f,49.6462f,  -22.0294f,32.6344f,49.4511f,  -22.9286f,31.2299f,48.3460f,  -21.9737f,30.4448f,48.1103f,  -22.8490f,29.2343f,46.7796f,  
	-21.9661f,28.4792f,46.5092f,  -22.7905f,27.4853f,44.9813f,  -21.9652f,26.7622f,44.6820f,  -72.3749f,-13.8658f,10.8196f,  -72.5650f,-11.1798f,12.6118f,  
	-70.5704f,-11.0405f,12.5475f,  -70.6323f,-8.4113f,14.3005f,  -68.6703f,-8.1932f,13.9768f,  -68.6100f,-5.6321f,15.6844f,  -66.7170f,-5.3354f,15.1094f,  
	-66.5414f,-2.8531f,16.7657f,  -64.7525f,-2.4773f,15.9529f,  -64.4689f,-0.0825f,17.5533f,  -62.8167f,0.3741f,16.5219f,  -62.4331f,2.6738f,18.0625f,  
	-60.9470f,3.2135f,16.8367f,  -60.4715f,5.4115f,18.3140f,  -59.1770f,6.0368f,16.9229f,  -58.6176f,8.1291f,18.3349f,  -57.5356f,8.8430f,16.8113f,  
	-56.9002f,10.8272f,18.1572f,  -56.0464f,11.6329f,16.5373f,  -55.3429f,13.5069f,17.8162f,  -54.7277f,14.4067f,16.1387f,  -53.9619f,16.1733f,17.3529f,  
	-53.5898f,17.1693f,15.6583f,  -52.7682f,18.8296f,16.8086f,  -52.6378f,19.9225f,15.1381f,  -51.7641f,21.4817f,16.2284f,  -51.8682f,22.6708f,14.6230f,  
	-50.9456f,24.1339f,15.6567f,  -51.2712f,25.4165f,14.1563f,  -50.3009f,26.7900f,15.1382f,  -50.8298f,28.1612f,13.7809f,  -49.8107f,29.4531f,14.7165f,  
	-50.5197f,30.9052f,13.5372f,  -49.4488f,32.1233f,14.4325f,  -50.3110f,33.6457f,13.4626f,  -49.1843f,34.7966f,14.3229f,  -50.1692f,36.3756f,13.5889f,  
	-48.9796f,37.4667f,14.4208f,  -50.0548f,39.0849f,13.9438f,  -48.7952f,40.1209f,14.7523f,  -49.9272f,41.7576f,14.5476f,  -48.5879f,42.7427f,15.3382f,  
	-49.7434f,44.3740f,15.4147f,  -48.3161f,45.3085f,16.1907f,  -49.4637f,46.9072f,16.5503f,  -47.9396f,47.7901f,17.3147f,  -49.0503f,49.3269f,17.9525f,  
	-47.4223f,50.1539f,18.7066f,  -48.4713f,51.5975f,19.6109f,  -46.7353f,52.3613f,20.3542f,  -47.7028f,53.6797f,21.5064f,  -45.8580f,54.3709f,22.2369f,  
	-46.7302f,55.5324f,23.6125f,  -44.7784f,56.1406f,24.3275f,  -45.5479f,57.1147f,25.8963f,  -43.4976f,57.6278f,26.5910f,  -44.1635f,58.3870f,28.3179f,  
	-42.0267f,58.7935f,28.9869f,  -42.5943f,59.3143f,30.8332f,  -40.3877f,59.6039f,31.4707f,  -40.8685f,59.8680f,33.3945f,  -38.6118f,60.0326f,33.9954f,  
	-39.0221f,60.0276f,35.9534f,  -36.7386f,60.0619f,36.5113f,  -37.0984f,59.7828f,38.4593f,  -34.8140f,59.6848f,38.9688f,  -35.1459f,59.1339f,40.8631f,  
	-32.8846f,58.9055f,41.3205f,  -33.2120f,58.0926f,43.1188f,  -30.9993f,57.7390f,43.5207f,  -31.3452f,56.6814f,45.1831f,  -29.2030f,56.2121f,45.5259f,  
	-29.5885f,54.9340f,47.0159f,  -27.5379f,54.3608f,47.2981f,  -27.9808f,52.8929f,48.5831f,  -26.0366f,52.2293f,48.8042f,  -26.5506f,50.6073f,49.8563f,  
	-24.7239f,49.8701f,50.0152f,  -25.3177f,48.1338f,50.8120f,  -23.6190f,47.3381f,50.9104f,  -24.2958f,45.5300f,51.4347f,  -22.7265f,44.6935f,51.4734f,  
	-23.4840f,42.8576f,51.7143f,  -22.0423f,41.9959f,51.6954f,  -22.8725f,40.1764f,51.6482f,  -21.5555f,39.3051f,51.5744f,  -22.4449f,37.5445f,51.2405f,  
	-21.2439f,36.6756f,51.1153f,  -22.1749f,35.0144f,50.5022f,  -21.0788f,34.1596f,50.3298f,  -22.0294f,32.6344f,49.4511f,  -21.0280f,31.8017f,49.2357f,  
	-21.9737f,30.4448f,48.1103f,  -21.0520f,29.6407f,47.8573f,  -21.9661f,28.4792f,46.5092f,  -21.1103f,27.7072f,46.2242f,  -21.9652f,26.7622f,44.6820f,  
	-21.1612f,26.0234f,44.3707f,  -72.5650f,-11.1798f,12.6118f,  -72.6285f,-8.4856f,14.4082f,  -70.6323f,-8.4113f,14.3005f,  -70.5706f,-5.7864f,16.0506f,  
	-68.6100f,-5.6321f,15.6844f,  -68.4301f,-3.0883f,17.3818f,  -66.5414f,-2.8531f,16.7657f,  -66.2510f,-0.4004f,18.4049f,  -64.4689f,-0.0825f,17.5533f,  
	-64.0765f,2.2705f,19.1296f,  -62.4331f,2.6738f,18.0625f,  -61.9474f,4.9195f,19.5717f,  -60.4715f,5.4115f,18.3140f,  -59.9013f,7.5448f,19.7537f,  
	-58.6176f,8.1291f,18.3349f,  -57.9717f,10.1469f,19.7035f,  -56.9002f,10.8272f,18.1572f,  -56.1873f,12.7267f,19.4534f,  -55.3429f,13.5069f,17.8162f,  
	-54.5700f,15.2902f,19.0418f,  -53.9619f,16.1733f,17.3529f,  -53.1366f,17.8413f,18.5085f,  -52.7682f,18.8296f,16.8086f,  -51.8954f,20.3873f,17.8978f,  
	-51.7641f,21.4817f,16.2284f,  -50.8483f,22.9340f,17.2545f,  -50.9456f,24.1339f,15.6567f,  -49.9891f,25.4877f,16.6244f,  -50.3009f,26.7900f,15.1382f,  
	-49.3037f,28.0538f,16.0533f,  -49.8107f,29.4531f,14.7165f,  -48.7707f,30.6356f,15.5853f,  -49.4488f,32.1233f,14.4325f,  -48.3629f,33.2322f,15.2609f,  
	-49.1843f,34.7966f,14.3229f,  -48.0456f,35.8409f,15.1181f,  -48.9796f,37.4667f,14.4208f,  -47.7811f,38.4525f,15.1885f,  -48.7952f,40.1209f,14.7523f,  
	-47.5269f,41.0543f,15.4990f,  -48.5879f,42.7427f,15.3382f,  -47.2407f,43.6261f,16.0684f,  -48.3161f,45.3085f,16.1907f,  -46.8801f,46.1426f,16.9084f,  
	-47.9396f,47.7901f,17.3147f,  -46.4058f,48.5725f,18.0225f,  -47.4223f,50.1539f,18.7066f,  -45.7843f,50.8789f,19.4052f,  -46.7353f,52.3613f,20.3542f,  
	-44.9897f,53.0210f,21.0428f,  -45.8580f,54.3709f,22.2369f,  -44.0039f,54.9558f,22.9139f,  -44.7784f,56.1406f,24.3275f,  -42.8212f,56.6389f,24.9890f,  
	-43.4976f,57.6278f,26.5910f,  -41.4464f,58.0279f,27.2322f,  -42.0267f,58.7935f,28.9869f,  -39.8954f,59.0845f,29.6025f,  -40.3877f,59.6039f,31.4707f,  
	-38.1937f,59.7766f,32.0558f,  -38.6118f,60.0326f,33.9954f,  -36.3767f,60.0800f,34.5439f,  -36.7386f,60.0619f,36.5113f,  -34.4872f,59.9804f,37.0174f,  
	-34.8140f,59.6848f,38.9688f,  -32.5698f,59.4744f,39.4286f,  -32.8846f,58.9055f,41.3205f,  -30.6731f,58.5694f,41.7301f,  -30.9993f,57.7390f,43.5207f,  
	-28.8434f,57.2846f,43.8759f,  -29.2030f,56.2121f,45.5259f,  -27.1252f,55.6496f,45.8247f,  -27.5379f,54.3608f,47.2981f,  -25.5547f,53.7024f,47.5393f,  
	-26.0366f,52.2293f,48.8042f,  -24.1617f,51.4907f,48.9859f,  -24.7239f,49.8701f,50.0152f,  -22.9698f,49.0658f,50.1387f,  -23.6190f,47.3381f,50.9104f,  
	-21.9897f,46.4854f,50.9754f,  -22.7265f,44.6935f,51.4734f,  -21.2228f,43.8082f,51.4813f,  -22.0423f,41.9959f,51.6954f,  -20.6637f,41.0943f,51.6482f,  
	-21.5555f,39.3051f,51.5744f,  -20.2951f,38.3997f,51.4750f,  -21.2439f,36.6756f,51.1153f,  -20.0928f,35.7794f,50.9671f,  -21.0788f,34.1596f,50.3298f,  
	-20.0280f,33.2817f,50.1367f,  -21.0280f,31.8017f,49.2357f,  -20.0639f,30.9500f,49.0025f,  -21.0520f,29.6407f,47.8573f,  -20.1615f,28.8201f,47.5891f,  
	-21.1103f,27.7072f,46.2242f,  -20.2795f,26.9203f,45.9264f,  -21.1612f,26.0234f,44.3707f,  -20.3770f,25.2705f,44.0490f,  -72.6285f,-8.4856f,14.4082f,  
	-72.5654f,-5.7957f,16.2017f,  -70.5706f,-5.7864f,16.0506f,  -70.3863f,-3.1788f,17.7906f,  -68.4301f,-3.0883f,17.3818f,  -68.1326f,-0.5742f,19.0620f,  
	-66.2510f,-0.4004f,18.4049f,  -65.8491f,2.0104f,20.0198f,  -64.0765f,2.2705f,19.1296f,  -63.5794f,4.5694f,20.6745f,  -61.9474f,4.9195f,19.5717f,  
	-61.3647f,7.1004f,21.0434f,  -59.9013f,7.5448f,19.7537f,  -59.2425f,9.6034f,21.1499f,  -57.9717f,10.1469f,19.7035f,  -57.2464f,12.0797f,21.0224f,  
	-56.1873f,12.7267f,19.4534f,  -55.4038f,14.5351f,20.6961f,  -54.5700f,15.2902f,19.0418f,  -53.7368f,16.9745f,20.2086f,  -53.1366f,17.8413f,18.5085f,  
	-52.2600f,19.4061f,19.6027f,  -51.8954f,20.3873f,17.8978f,  -50.9810f,21.8373f,18.9224f,  -50.8483f,22.9340f,17.2545f,  -49.8997f,24.2765f,18.2141f,  
	-49.9891f,25.4877f,16.6244f,  -49.0075f,26.7313f,17.5249f,  -49.3037f,28.0538f,16.0533f,  -48.2881f,29.2081f,16.9012f,  -48.7707f,30.6356f,15.5853f,  
	-47.7188f,31.7091f,16.3867f,  -48.3629f,33.2322f,15.2609f,  -47.2686f,34.2352f,16.0237f,  -48.0456f,35.8409f,15.1181f,  -46.9023f,36.7810f,15.8487f,  
	-47.7811f,38.4525f,15.1885f,  -46.5788f,39.3376f,15.8941f,  -47.5269f,41.0543f,15.4990f,  -46.2557f,41.8888f,16.1854f,  -47.2407f,43.6261f,16.0684f,  
	-45.8899f,44.4124f,16.7406f,  -46.8801f,46.1426f,16.9084f,  -45.4390f,46.8805f,17.5703f,  -46.4058f,48.5725f,18.0225f,  -44.8662f,49.2581f,18.6759f,  
	-45.7843f,50.8789f,19.4052f,  -44.1402f,51.5057f,20.0506f,  -44.9897f,53.0210f,21.0428f,  -43.2374f,53.5806f,21.6796f,  -44.0039f,54.9558f,22.9139f,  
	-42.1457f,55.4371f,23.5392f,  -42.8212f,56.6389f,24.9890f,  -40.8635f,57.0305f,25.5989f,  -41.4464f,58.0279f,27.2322f,  -39.4001f,58.3185f,27.8221f,  
	-39.8954f,59.0845f,29.6025f,  -37.7751f,59.2641f,30.1679f,  -38.1937f,59.7766f,32.0558f,  -36.0191f,59.8369f,32.5906f,  -36.3767f,60.0800f,34.5439f,  
	-34.1710f,60.0157f,35.0423f,  -34.4872f,59.9804f,37.0174f,  -32.2730f,59.7895f,37.4754f,  -32.5698f,59.4744f,39.4286f,  -30.3722f,59.1582f,39.8420f,  
	-30.6731f,58.5694f,41.7301f,  -28.5158f,58.1333f,42.0942f,  -28.8434f,57.2846f,43.8759f,  -26.7498f,56.7371f,44.1882f,  -27.1252f,55.6496f,45.8247f,  
	-25.1133f,55.0013f,46.0833f,  -25.5547f,53.7024f,47.5393f,  -23.6404f,52.9679f,47.7415f,  -24.1617f,51.4907f,48.9859f,  -22.3593f,50.6836f,49.1326f,  
	-22.9698f,49.0658f,50.1387f,  -21.2861f,48.2028f,50.2287f,  -21.9897f,46.4854f,50.9754f,  -20.4279f,45.5824f,51.0094f,  -21.2228f,43.8082f,51.4813f,  
	-19.7848f,42.8816f,51.4607f,  -20.6637f,41.0943f,51.6482f,  -19.3448f,40.1572f,51.5751f,  -20.2951f,38.3997f,51.4750f,  -19.0884f,37.4661f,51.3521f,  
	-20.0928f,35.7794f,50.9671f,  -18.9909f,34.8595f,50.7979f,  -20.0280f,33.2817f,50.1367f,  -19.0186f,32.3847f,49.9252f,  -20.0639f,30.9500f,49.0025f,  
	-19.1349f,30.0821f,48.7534f,  -20.1615f,28.8201f,47.5891f,  -19.2997f,27.9854f,47.3076f,  -20.2795f,26.9203f,45.9264f,  -19.4721f,26.1202f,45.6177f,  
	-20.3770f,25.2705f,44.0490f,  -19.6101f,24.5045f,43.7193f,  -72.5654f,-5.7957f,16.2017f,  -72.3765f,-3.1230f,17.9850f,  -70.3863f,-3.1788f,17.7906f,  
	-70.0813f,-0.6009f,19.5133f,  -68.1326f,-0.5742f,19.0620f,  -67.7205f,1.8979f,20.7179f,  -65.8491f,2.0104f,20.0198f,  -65.3396f,4.3671f,21.6035f,  
	-63.5794f,4.5694f,20.6745f,  -62.9826f,6.8033f,22.1818f,  -61.3647f,7.1004f,21.0434f,  -60.6908f,9.2062f,22.4715f,  -59.2425f,9.6034f,21.1499f,  
	-58.5021f,11.5768f,22.4963f,  -57.2464f,12.0797f,21.0224f,  -56.4488f,13.9210f,22.2876f,  -55.4038f,14.5351f,20.6961f,  -54.5585f,16.2440f,21.8799f,  
	-53.7368f,16.9745f,20.2086f,  -52.8515f,18.5550f,21.3138f,  -52.2600f,19.4061f,19.6027f,  -51.3415f,20.8627f,20.6318f,  -50.9810f,21.8373f,18.9224f,  
	-50.0342f,23.1772f,19.8801f,  -49.8997f,24.2765f,18.2141f,  -48.9271f,25.5084f,19.1060f,  -49.0075f,26.7313f,17.5249f,  -48.0093f,27.8653f,18.3577f,  
	-48.2881f,29.2081f,16.9012f,  -47.2633f,30.2533f,17.6811f,  -47.7188f,31.7091f,16.3867f,  -46.6619f,32.6771f,17.1221f,  -47.2686f,34.2352f,16.0237f,  
	-46.1737f,35.1350f,16.7215f,  -46.9023f,36.7810f,15.8487f,  -45.7597f,37.6221f,16.5170f,  -46.5788f,39.3376f,15.8941f,  -45.3782f,40.1262f,16.5397f,  
	-46.2557f,41.8888f,16.1854f,  -44.9858f,42.6293f,16.8141f,  -45.8899f,44.4124f,16.7406f,  -44.5387f,45.1067f,17.3576f,  -45.4390f,46.8805f,17.5703f,  
	-43.9966f,47.5264f,18.1786f,  -44.8662f,49.2581f,18.6759f,  -43.3240f,49.8510f,19.2774f,  -44.1402f,51.5057f,20.0506f,  -42.4922f,52.0385f,20.6458f,  
	-43.2374f,53.5806f,21.6796f,  -41.4828f,54.0432f,22.2666f,  -42.1457f,55.4371f,23.5392f,  -40.2878f,55.8184f,24.1151f,  -40.8635f,57.0305f,25.5989f,  
	-38.9101f,57.3190f,26.1598f,  -39.4001f,58.3185f,27.8221f,  -37.3631f,58.5039f,28.3640f,  -37.7751f,59.2641f,30.1679f,  -35.6719f,59.3367f,30.6850f,  
	-36.0191f,59.8369f,32.5906f,  -33.8709f,59.7899f,33.0774f,  -34.1710f,60.0157f,35.0423f,  -31.9992f,59.8454f,35.4945f,  -32.2730f,59.7895f,37.4754f,  
	-30.1018f,59.4954f,37.8887f,  -30.3722f,59.1582f,39.8420f,  -28.2258f,58.7437f,40.2112f,  -28.5158f,58.1333f,42.0942f,  -26.4180f,57.6049f,42.4165f,  
	-26.7498f,56.7371f,44.1882f,  -24.7199f,56.1040f,44.4611f,  -25.1133f,55.0013f,46.0833f,  -23.1692f,54.2763f,46.3035f,  -23.6404f,52.9679f,47.7415f,  
	-21.7980f,52.1637f,47.9094f,  -22.3593f,50.6836f,49.1326f,  -20.6278f,49.8163f,49.2462f,  -21.2861f,48.2028f,50.2287f,  -19.6712f,47.2880f,50.2879f,  
	-20.4279f,45.5824f,51.0094f,  -18.9340f,44.6365f,51.0150f,  -19.7848f,42.8816f,51.4607f,  -18.4092f,41.9181f,51.4140f,  -19.3448f,40.1572f,51.5751f,  
	-18.0828f,39.1909f,51.4782f,  -19.0884f,37.4661f,51.3521f,  -17.9344f,36.5081f,51.2078f,  -18.9909f,34.8595f,50.7979f,  -17.9341f,33.9201f,50.6097f,  
	-19.0186f,32.3847f,49.9252f,  -18.0481f,31.4714f,49.6972f,  -19.1349f,30.0821f,48.7534f,  -18.2383f,29.2005f,48.4904f,  -19.2997f,27.9854f,47.3076f,  
	-18.4646f,27.1384f,47.0144f,  -19.4721f,26.1202f,45.6177f,  -18.6849f,25.3085f,45.3003f,  -19.6101f,24.5045f,43.7193f,  -18.8601f,23.7268f,43.3825f,  
	-72.3765f,-3.1230f,17.9850f,  -72.0639f,-0.4800f,19.7513f,  -70.0813f,-0.6009f,19.5133f,  -69.6587f,1.9350f,21.2119f,  -67.7205f,1.8979f,20.7179f,  
	-67.1978f,4.3160f,22.3428f,  -65.3396f,4.3671f,21.6035f,  -64.7275f,6.6586f,23.1496f,  -62.9826f,6.8033f,22.1818f,  -62.2920f,8.9620f,23.6457f,  
	-60.6908f,9.2062f,22.4715f,  -59.9330f,11.2267f,23.8499f,  -58.5021f,11.5768f,22.4963f,  -57.6873f,13.4582f,23.7890f,  -56.4488f,13.9210f,22.2876f,  
	-55.5877f,15.6623f,23.4938f,  -54.5585f,16.2440f,21.8799f,  -53.6599f,17.8486f,23.0019f,  -52.8515f,18.5550f,21.3138f,  -51.9235f,20.0268f,22.3536f,  
	-51.3415f,20.8627f,20.6318f,  -50.3904f,22.2085f,21.5938f,  -50.0342f,23.1772f,19.8801f,  -49.0639f,24.4058f,20.7696f,  -48.9271f,25.5084f,19.1060f,  
	-47.9392f,26.6301f,19.9298f,  -48.0093f,27.8653f,18.3577f,  -47.0040f,28.8899f,19.1219f,  -47.2633f,30.2533f,17.6811f,  -46.2361f,31.1933f,18.3946f,  
	-46.6619f,32.6771f,17.1221f,  -45.6079f,33.5426f,17.7922f,  -46.1737f,35.1350f,16.7215f,  -45.0836f,35.9369f,17.3568f,  -45.7597f,37.6221f,16.5170f,  
	-44.6230f,38.3687f,17.1252f,  -45.3782f,40.1262f,16.5397f,  -44.1835f,40.8238f,17.1277f,  -44.9858f,42.6293f,16.8141f,  -43.7201f,43.2816f,17.3881f,  
	-44.5387f,45.1067f,17.3576f,  -43.1904f,45.7136f,17.9219f,  -43.9966f,47.5264f,18.1786f,  -42.5551f,48.0851f,18.7363f,  -43.3240f,49.8510f,19.2774f,  
	-41.7810f,50.3560f,19.8301f,  -42.4922f,52.0385f,20.6458f,  -40.8438f,52.4809f,21.1929f,  -41.4828f,54.0432f,22.2666f,  -39.7293f,54.4124f,22.8062f,  
	-40.2878f,55.8184f,24.1151f,  -38.4340f,56.1033f,24.6441f,  -38.9101f,57.3190f,26.1598f,  -36.9646f,57.5088f,26.6748f,  -37.3631f,58.5039f,28.3640f,  
	-35.3409f,58.5880f,28.8597f,  -35.6719f,59.3367f,30.6850f,  -33.5920f,59.3070f,31.1561f,  -33.8709f,59.7899f,33.0774f,  -31.7531f,59.6412f,33.5197f,  
	-31.9992f,59.8454f,35.4945f,  -29.8666f,59.5753f,35.9036f,  -30.1018f,59.4954f,37.8887f,  -27.9785f,59.1052f,38.2593f,  -28.2258f,58.7437f,40.2112f,  
	-26.1356f,58.2381f,40.5399f,  -26.4180f,57.6049f,42.4165f,  -24.3812f,56.9914f,42.7005f,  -24.7199f,56.1040f,44.4611f,  -22.7560f,55.3937f,44.6964f,  
	-23.1692f,54.2763f,46.3035f,  -21.2954f,53.4810f,46.4901f,  -21.7980f,52.1637f,47.9094f,  -20.0261f,51.2983f,48.0446f,  -20.6278f,49.8163f,49.2462f,  
	-18.9656f,48.8958f,49.3292f,  -19.6712f,47.2880f,50.2879f,  -18.1256f,46.3288f,50.3188f,  -18.9340f,44.6365f,51.0150f,  -17.5043f,43.6523f,50.9944f,  
	-18.4092f,41.9181f,51.4140f,  -17.0932f,40.9243f,51.3433f,  -18.0828f,39.1909f,51.4782f,  -16.8766f,38.1992f,51.3596f,  -17.9344f,36.5081f,51.2078f,  
	-16.8289f,35.5300f,51.0441f,  -17.9341f,33.9201f,50.6097f,  -16.9198f,32.9644f,50.4044f,  -18.0481f,31.4714f,49.6972f,  -17.1136f,30.5450f,49.4546f,  
	-18.2383f,29.2005f,48.4904f,  -17.3720f,28.3075f,48.2150f,  -18.4646f,27.1384f,47.0144f,  -17.6530f,26.2807f,46.7117f,  -18.6849f,25.3085f,45.3003f,  
	-17.9175f,24.4864f,44.9752f,  -18.8601f,23.7268f,43.3825f,  -18.1250f,22.9382f,43.0405f,  -72.0639f,-0.4800f,19.7513f,  -71.6305f,2.1210f,21.4934f,  
	-69.6587f,1.9350f,21.2119f,  -69.1223f,4.4170f,22.8797f,  -67.1978f,4.3160f,22.3428f,  -66.5694f,6.6689f,23.9302f,  -64.7275f,6.6586f,23.1496f,  
	-64.0186f,8.8749f,24.6524f,  -62.2920f,8.9620f,23.6457f,  -61.5145f,11.0353f,25.0600f,  -59.9330f,11.2267f,23.8499f,  -59.0981f,13.1548f,25.1745f,  
	-57.6873f,13.4582f,23.7890f,  -56.8067f,15.2394f,25.0227f,  -55.5877f,15.6623f,23.4938f,  -54.6712f,17.2989f,24.6380f,  -53.6599f,17.8486f,23.0019f,  
	-52.7172f,19.3438f,24.0583f,  -51.9235f,20.0268f,22.3536f,  -50.9621f,21.3871f,23.3260f,  -50.3904f,22.2085f,21.5938f,  -49.4156f,23.4424f,22.4873f,  
	-49.0639f,24.4058f,20.7696f,  -48.0788f,25.5237f,21.5906f,  -47.9392f,26.6301f,19.9298f,  -46.9454f,27.6421f,20.6845f,  -47.0040f,28.8899f,19.1219f,  
	-45.9983f,29.8092f,19.8193f,  -46.2361f,31.1933f,18.3946f,  -45.2145f,32.0311f,19.0423f,  -45.6079f,33.5426f,17.7922f,  -44.5620f,34.3111f,18.3993f,  
	-45.0836f,35.9369f,17.3568f,  -44.0031f,36.6462f,17.9320f,  -44.6230f,38.3687f,17.1252f,  -43.4966f,39.0266f,17.6759f,  -44.1835f,40.8238f,17.1277f,  
	-42.9973f,41.4363f,17.6613f,  -43.7201f,43.2816f,17.3881f,  -42.4615f,43.8507f,17.9101f,  -43.1904f,45.7136f,17.9219f,  -41.8467f,46.2384f,18.4366f,  
	-42.5551f,48.0851f,18.7363f,  -41.1159f,48.5618f,19.2465f,  -41.7810f,50.3560f,19.8301f,  -40.2395f,50.7769f,20.3365f,  -40.8438f,52.4809f,21.1929f,  
	-39.1975f,52.8365f,21.6945f,  -39.7293f,54.4124f,22.8062f,  -37.9798f,54.6920f,23.3009f,  -38.4340f,56.1033f,24.6441f,  -36.5867f,56.2961f,25.1293f,  
	-36.9646f,57.5088f,26.6748f,  -35.0319f,57.6035f,27.1458f,  -35.3409f,58.5880f,28.8597f,  -33.3391f,58.5752f,29.3115f,  -33.5920f,59.3070f,31.1561f,  
	-31.5390f,59.1803f,31.5848f,  -31.7531f,59.6412f,33.5197f,  -29.6705f,59.3964f,33.9208f,  -29.8666f,59.5753f,35.9036f,  -27.7779f,59.2118f,36.2717f,  
	-27.9785f,59.1052f,38.2593f,  -25.9074f,58.6259f,38.5906f,  -26.1356f,58.2381f,40.5399f,  -24.1030f,57.6484f,40.8315f,  -24.3812f,56.9914f,42.7005f,  
	-22.4079f,56.3009f,42.9477f,  -22.7560f,55.3937f,44.6964f,  -20.8602f,54.6127f,44.8989f,  -21.2954f,53.4810f,46.4901f,  -19.4912f,52.6234f,46.6445f,  
	-20.0261f,51.2983f,48.0446f,  -18.3233f,50.3784f,48.1495f,  -18.9656f,48.8958f,49.3292f,  -17.3736f,47.9296f,49.3839f,  -18.1256f,46.3288f,50.3188f,  
	-16.6456f,45.3300f,50.3235f,  -17.5043f,43.6523f,50.9944f,  -16.1365f,42.6366f,50.9496f,  -17.0932f,40.9243f,51.3433f,  -15.8355f,39.9041f,51.2506f,  
	-16.8766f,38.1992f,51.3596f,  -15.7219f,37.1869f,51.2211f,  -16.8289f,35.5300f,51.0441f,  -15.7692f,34.5354f,50.8628f,  -16.9198f,32.9644f,50.4044f,  
	-15.9449f,31.9956f,50.1838f,  -17.1136f,30.5450f,49.4546f,  -16.2127f,29.6075f,49.1990f,  -17.3720f,28.3075f,48.2150f,  -16.5321f,27.4047f,47.9293f,  
	-17.6530f,26.2807f,46.7117f,  -16.8641f,25.4141f,46.4006f,  -17.9175f,24.4864f,44.9752f,  -17.1674f,23.6549f,44.6441f,  -18.1250f,22.9382f,43.0405f,  
	-17.4029f,22.1393f,42.6949f,  -71.6305f,2.1210f,21.4934f,  -71.0800f,4.6684f,23.2051f,  -69.1223f,4.4170f,22.8797f,  -68.4769f,6.8340f,24.5102f,  
	-66.5694f,6.6689f,23.9302f,  -65.8410f,8.9466f,25.4745f,  -64.0186f,8.8749f,24.6524f,  -63.2195f,11.0059f,26.1059f,  -61.5145f,11.0353f,25.0600f,  
	-60.6570f,13.0160f,26.4206f,  -59.0981f,13.1548f,25.1745f,  -58.1947f,14.9825f,26.4403f,  -56.8067f,15.2394f,25.0227f,  -55.8683f,16.9153f,26.1943f,  
	-54.6712f,17.2989f,24.6380f,  -53.7088f,18.8255f,25.7166f,  -52.7172f,19.3438f,24.0583f,  -51.7395f,20.7270f,25.0471f,  -50.9621f,21.3871f,23.3260f,  
	-49.9762f,22.6348f,24.2296f,  -49.4156f,23.4424f,22.4873f,  -48.4259f,24.5650f,23.3118f,  -48.0788f,25.5237f,21.5906f,  -47.0885f,26.5312f,22.3421f,  
	-46.9454f,27.6421f,20.6845f,  -45.9526f,28.5484f,21.3719f,  -45.9983f,29.8092f,19.8193f,  -45.0003f,30.6263f,20.4504f,  -45.2145f,32.0311f,19.0423f,  
	-44.2040f,32.7724f,19.6266f,  -44.5620f,34.3111f,18.3993f,  -43.5291f,34.9882f,18.9462f,  -44.0031f,36.6462f,17.9320f,  -42.9368f,37.2687f,18.4495f,  
	-43.4966f,39.0266f,17.6759f,  -42.3827f,39.6025f,18.1725f,  -42.9973f,41.4363f,17.6613f,  -41.8222f,41.9695f,18.1435f,  -42.4615f,43.8507f,17.9101f,  
	-41.2109f,44.3423f,18.3834f,  -41.8467f,46.2384f,18.4366f,  -40.5084f,46.6865f,18.9050f,  -41.1159f,48.5618f,19.2465f,  -39.6804f,48.9604f,19.7119f,  
	-40.2395f,50.7769f,20.3365f,  -38.7015f,51.1177f,20.7991f,  -39.1975f,52.8365f,21.6945f,  -37.5553f,53.1094f,22.1533f,  -37.9798f,54.6920f,23.3009f,  
	-36.2356f,54.8863f,23.7540f,  -36.5867f,56.2961f,25.1293f,  -34.7502f,56.4001f,25.5727f,  -35.0319f,57.6035f,27.1458f,  -33.1165f,57.6071f,27.5750f,  
	-33.3391f,58.5752f,29.3115f,  -31.3604f,58.4707f,29.7229f,  -31.5390f,59.1803f,31.5848f,  -29.5169f,58.9619f,31.9741f,  -29.6705f,59.3964f,33.9208f,  
	-27.6276f,59.0616f,34.2825f,  -27.7779f,59.2118f,36.2717f,  -25.7373f,58.7615f,36.6020f,  -25.9074f,58.6259f,38.5906f,  -23.8901f,58.0638f,38.8862f,  
	-24.1030f,57.6484f,40.8315f,  -22.1308f,56.9823f,41.0873f,  -22.4079f,56.3009f,42.9477f,  -20.4999f,55.5396f,43.1631f,  -20.8602f,54.6127f,44.8989f,  
	-19.0323f,53.7689f,45.0700f,  -19.4912f,52.6234f,46.6445f,  -17.7553f,51.7103f,46.7692f,  -18.3233f,50.3784f,48.1495f,  -16.6907f,49.4116f,48.2265f,  
	-17.3736f,47.9296f,49.3839f,  -15.8478f,46.9225f,49.4128f,  -16.6456f,45.3300f,50.3235f,  -15.2291f,44.2984f,50.3038f,  -16.1365f,42.6366f,50.9496f,  
	-14.8291f,41.5934f,50.8827f,  -15.8355f,39.9041f,51.2506f,  -14.6321f,38.8625f,51.1377f,  -15.7219f,37.1869f,51.2211f,  -14.6160f,36.1576f,51.0645f,  
	-15.7692f,34.5354f,50.8628f,  -14.7521f,33.5277f,50.6655f,  -15.9449f,31.9956f,50.1838f,  -15.0068f,31.0162f,49.9496f,  -16.2127f,29.6075f,49.1990f,  
	-15.3414f,28.6610f,48.9322f,  -16.5321f,27.4047f,47.9293f,  -15.7180f,26.4940f,47.6344f,  -16.8641f,25.4141f,46.4006f,  -16.0951f,24.5393f,46.0828f,  
	-17.1674f,23.6549f,44.6441f,  -16.4327f,22.8148f,44.3087f,  -17.4029f,22.1393f,42.6949f,  -16.6943f,21.3313f,42.3460f,  -71.0800f,4.6684f,23.2051f,  
	-70.4172f,7.1509f,24.8797f,  -68.4769f,6.8340f,24.5102f,  -67.7281f,9.1757f,26.0978f,  -65.8410f,8.9466f,25.4745f,  -65.0190f,11.1391f,26.9698f,  
	-63.2195f,11.0059f,26.1059f,  -62.3374f,13.0441f,27.5058f,  -60.6570f,13.0160f,26.4206f,  -59.7279f,14.8960f,27.7225f,  -58.1947f,14.9825f,26.4403f,  
	-57.2307f,16.7045f,27.6440f,  -55.8683f,16.9153f,26.1943f,  -54.8815f,18.4808f,27.3003f,  -53.7088f,18.8255f,25.7166f,  -52.7094f,20.2395f,26.7275f,  
	-51.7395f,20.7270f,25.0471f,  -50.7360f,21.9968f,25.9668f,  -49.9762f,22.6348f,24.2296f,  -48.9744f,23.7703f,25.0639f,  -48.4259f,24.5650f,23.3118f,  
	-47.4309f,25.5764f,24.0664f,  -47.0885f,26.5312f,22.3421f,  -46.0998f,27.4326f,23.0258f,  -45.9526f,28.5484f,21.3719f,  -44.9689f,29.3524f,21.9925f,  
	-45.0003f,30.6263f,20.4504f,  -44.0157f,31.3471f,21.0177f,  -44.2040f,32.7724f,19.6266f,  -43.2092f,33.4231f,20.1504f,  -43.5291f,34.9882f,18.9462f,  
	-42.5138f,35.5800f,19.4352f,  -42.9368f,37.2687f,18.4495f,  -41.8867f,37.8114f,18.9130f,  -42.3827f,39.6025f,18.1725f,  -41.2835f,40.1022f,18.6180f,  
	-41.8222f,41.9695f,18.1435f,  -40.6585f,42.4294f,18.5777f,  -41.2109f,44.3423f,18.3834f,  -39.9688f,44.7624f,18.8115f,  -40.5084f,46.6865f,18.9050f,  
	-39.1761f,47.0626f,19.3300f,  -39.6804f,48.9604f,19.7119f,  -38.2497f,49.2854f,20.1353f,  -38.7015f,51.1177f,20.7991f,  -37.1677f,51.3824f,21.2208f,  
	-37.5553f,53.1094f,22.1533f,  -35.9175f,53.3037f,22.5724f,  -36.2356f,54.8863f,23.7540f,  -34.5005f,54.9984f,24.1673f,  -34.7502f,56.4001f,25.5727f,  
	-32.9280f,56.4192f,25.9765f,  -33.1165f,57.6071f,27.5750f,  -31.2206f,57.5244f,27.9659f,  -31.3604f,58.4707f,29.7229f,  -29.4084f,58.2791f,30.0968f,  
	-29.5169f,58.9619f,31.9741f,  -27.5302f,58.6571f,32.3257f,  -27.6276f,59.0616f,34.2825f,  -25.6282f,58.6427f,34.6080f,  -25.7373f,58.7615f,36.6020f,  
	-23.7461f,58.2304f,36.8978f,  -23.8901f,58.0638f,38.8862f,  -21.9296f,57.4263f,39.1470f,  -22.1308f,56.9823f,41.0873f,  -20.2205f,56.2459f,41.3123f,  
	-20.4999f,55.5396f,43.1631f,  -18.6573f,54.7153f,43.3478f,  -19.0323f,53.7689f,45.0700f,  -17.2712f,52.8688f,45.2118f,  -17.7553f,51.7103f,46.7692f,  
	-16.0888f,50.7493f,46.8662f,  -16.6907f,49.4116f,48.2265f,  -15.1245f,48.4026f,48.2778f,  -15.8478f,46.9225f,49.4128f,  -14.3865f,45.8815f,49.4172f,  
	-15.2291f,44.2984f,50.3038f,  -13.8744f,43.2383f,50.2621f,  -14.8291f,41.5934f,50.8827f,  -13.5781f,40.5280f,50.7952f,  -14.6321f,38.8625f,51.1377f,  
	-13.4800f,37.8033f,51.0062f,  -14.6160f,36.1576f,51.0645f,  -13.5556f,35.1150f,50.8914f,  -14.7521f,33.5277f,50.6655f,  -13.7748f,32.5094f,50.4539f,  
	-15.0068f,31.0162f,49.9496f,  -14.1012f,30.0283f,49.7033f,  -15.3414f,28.6610f,48.9322f,  -14.4988f,27.7077f,48.6556f,  -15.7180f,26.4940f,47.6344f,  
	-14.9264f,25.5764f,47.3321f,  -16.0951f,24.5393f,46.0828f,  -15.3438f,23.6577f,45.7599f,  -16.4327f,22.8148f,44.3087f,  -15.7134f,21.9672f,43.9694f,  
	-16.6943f,21.3313f,42.3460f,  -15.9969f,20.5143f,41.9956f,  -70.4172f,7.1509f,24.8797f,  -69.6475f,9.5584f,26.5117f,  -67.7281f,9.1757f,26.0978f,  
	-66.8821f,11.4325f,27.6368f,  -65.0190f,11.1391f,26.9698f,  -64.1105f,13.2386f,28.4116f,  -62.3374f,13.0441f,27.5058f,  -61.3800f,14.9814f,28.8472f,  
	-59.7279f,14.8960f,27.7225f,  -58.7349f,16.6699f,28.9624f,  -57.2307f,16.7045f,27.6440f,  -56.2153f,18.3155f,28.7822f,  -54.8815f,18.4808f,27.3003f,  
	-53.8552f,19.9329f,28.3385f,  -52.7094f,20.2395f,26.7275f,  -51.6820f,21.5392f,27.6691f,  -50.7360f,21.9968f,25.9668f,  -49.7151f,23.1534f,26.8169f,  
	-48.9744f,23.7703f,25.0639f,  -47.9667f,24.7938f,25.8278f,  -47.4309f,25.5764f,24.0664f,  -46.4374f,26.4808f,24.7527f,  -46.0998f,27.4326f,23.0258f,  
	-45.1209f,28.2310f,23.6422f,  -44.9689f,29.3524f,21.9925f,  -44.0002f,30.0596f,22.5488f,  -44.0157f,31.3471f,21.0177f,  -43.0487f,31.9777f,21.5242f,  
	-43.2092f,33.4231f,20.1504f,  -42.2348f,33.9895f,20.6161f,  -42.5138f,35.5800f,19.4352f,  -41.5178f,36.0939f,19.8701f,  -41.8867f,37.8114f,18.9130f,  
	-40.8547f,38.2808f,19.3256f,  -41.2835f,40.1022f,18.6180f,  -40.1991f,40.5324f,19.0161f,  -40.6585f,42.4294f,18.5777f,  -39.5064f,42.8223f,18.9674f,  
	-39.9688f,44.7624f,18.8115f,  -38.7350f,45.1158f,19.1972f,  -39.1761f,47.0626f,19.3300f,  -37.8500f,47.3709f,19.7144f,  -38.2497f,49.2854f,20.1353f,  
	-36.8238f,49.5407f,20.5194f,  -37.1677f,51.3824f,21.2208f,  -35.6378f,51.5753f,21.6046f,  -35.9175f,53.3037f,22.5724f,  -34.2871f,53.4225f,22.9537f,  
	-34.5005f,54.9984f,24.1673f,  -32.7769f,55.0319f,24.5431f,  -32.9280f,56.4192f,25.9765f,  -31.1217f,56.3578f,26.3438f,  -31.2206f,57.5244f,27.9659f,  
	-29.3469f,57.3598f,28.3211f,  -29.4084f,58.2791f,30.0968f,  -27.4872f,58.0053f,30.4348f,  -27.5302f,58.6571f,32.3257f,  -25.5820f,58.2715f,32.6427f,  
	-25.6282f,58.6427f,34.6080f,  -23.6734f,58.1455f,34.9004f,  -23.7461f,58.2304f,36.8978f,  -21.8076f,57.6253f,37.1602f,  -21.9296f,57.4263f,39.1470f,  
	-20.0269f,56.7193f,39.3783f,  -20.2205f,56.2459f,41.3123f,  -18.3725f,55.4463f,41.5075f,  -18.6573f,54.7153f,43.3478f,  -16.8793f,53.8341f,43.5040f,  
	-17.2712f,52.8688f,45.2118f,  -15.5781f,51.9199f,45.3265f,  -16.0888f,50.7493f,46.8662f,  -14.4881f,49.7450f,46.9380f,  -15.1245f,48.4026f,48.2778f,  
	-13.6231f,47.3586f,48.3047f,  -14.3865f,45.8815f,49.4172f,  -12.9880f,44.8106f,49.3996f,  -13.8744f,43.2383f,50.2621f,  -12.5779f,42.1549f,50.1996f,  
	-13.5781f,40.5280f,50.7952f,  -12.3805f,39.4442f,50.6888f,  -13.4800f,37.8033f,51.0062f,  -12.3759f,36.7303f,50.8577f,  -13.5556f,35.1150f,50.8914f,  
	-12.5378f,34.0617f,50.7034f,  -13.7748f,32.5094f,50.4539f,  -12.8329f,31.4831f,50.2295f,  -14.1012f,30.0283f,49.7033f,  -13.2273f,29.0343f,49.4465f,  
	-14.4988f,27.7077f,48.6556f,  -13.6812f,26.7483f,48.3705f,  -14.9264f,25.5764f,47.3321f,  -14.1550f,24.6533f,47.0238f,  -15.3438f,23.6577f,45.7599f,  
	-14.6100f,22.7698f,45.4325f,  -15.7134f,21.9672f,43.9694f,  -15.0069f,21.1124f,43.6280f,  -15.9969f,20.5143f,41.9956f,  -15.3090f,19.6885f,41.6451f,  
	-69.6475f,9.5584f,26.5117f,  -68.7769f,11.8812f,28.0955f,  -66.8821f,11.4325f,27.6368f,  -65.9459f,13.5962f,29.1226f,  -64.1105f,13.2386f,28.4116f,  
	-63.1229f,15.2372f,29.7954f,  -61.3800f,14.9814f,28.8472f,  -60.3554f,16.8123f,30.1269f,  -58.7349f,16.6699f,28.9624f,  -57.6872f,18.3324f,30.1369f,  
	-56.2153f,18.3155f,28.7822f,  -55.1572f,19.8125f,29.8526f,  -53.8552f,19.9329f,28.3385f,  -52.7981f,21.2699f,29.3073f,  -51.6820f,21.5392f,27.6691f,  
	-50.6352f,22.7248f,28.5408f,  -49.7151f,23.1534f,26.8169f,  -48.6867f,24.1970f,27.5963f,  -47.9667f,24.7938f,25.8278f,  -46.9596f,25.7093f,26.5231f,  
	-46.4374f,26.4808f,24.7527f,  -45.4537f,27.2813f,25.3713f,  -45.1209f,28.2310f,23.6422f,  -44.1578f,28.9320f,24.1937f,  -44.0002f,30.0596f,22.5488f,  
	-43.0505f,30.6767f,23.0438f,  -43.0487f,31.9777f,21.5242f,  -42.1045f,32.5244f,21.9720f,  -42.2348f,33.9895f,20.6161f,  -41.2824f,34.4791f,21.0274f,  
	-41.5178f,36.0939f,19.8701f,  -40.5430f,36.5364f,20.2541f,  -40.8547f,38.2808f,19.3256f,  -39.8404f,38.6836f,19.6912f,  -40.1991f,40.5324f,19.0161f,  
	-39.1291f,40.8995f,19.3703f,  -39.5064f,42.8223f,18.9674f,  -38.3649f,43.1534f,19.3158f,  -38.7350f,45.1158f,19.1972f,  -37.5091f,45.4071f,19.5436f,  
	-37.8500f,47.3709f,19.7144f,  -36.5293f,47.6158f,20.0611f,  -36.8238f,49.5407f,20.5194f,  -35.4012f,49.7307f,20.8674f,  -35.6378f,51.5753f,21.6046f,  
	-34.1138f,51.6991f,21.9526f,  -34.2871f,53.4225f,22.9537f,  -32.6656f,53.4688f,23.2995f,  -32.7769f,55.0319f,24.5431f,  -31.0657f,54.9907f,24.8843f,  
	-31.1217f,56.3578f,26.3438f,  -29.3335f,56.2197f,26.6773f,  -29.3469f,57.3598f,28.3211f,  -27.4994f,57.1174f,28.6422f,  -27.4872f,58.0053f,30.4348f,  
	-25.5994f,57.6543f,30.7398f,  -25.5820f,58.2715f,32.6427f,  -23.6734f,57.8104f,32.9282f,  -23.6734f,58.1455f,34.9004f,  -21.7670f,57.5760f,35.1607f,  
	-21.8076f,57.6253f,37.1602f,  -19.9223f,56.9519f,37.3941f,  -20.0269f,56.7193f,39.3783f,  -18.1829f,55.9495f,39.5806f,  -18.3725f,55.4463f,41.5075f,  
	-16.5864f,54.5897f,41.6749f,  -16.8793f,53.8341f,43.5040f,  -15.1673f,52.9034f,43.6335f,  -15.5781f,51.9199f,45.3265f,  -13.9496f,50.9267f,45.4163f,  
	-14.4881f,49.7450f,46.9380f,  -12.9521f,48.7045f,46.9855f,  -13.6231f,47.3586f,48.3047f,  -12.1848f,46.2835f,48.3098f,  -12.9880f,44.8106f,49.3996f,  
	-11.6487f,43.7155f,49.3610f,  -12.5779f,42.1549f,50.1996f,  -11.3365f,41.0523f,50.1178f,  -12.3805f,39.4442f,50.6888f,  -11.2333f,38.3461f,50.5650f,  
	-12.3759f,36.7303f,50.8577f,  -11.3169f,35.6465f,50.6937f,  -12.5378f,34.0617f,50.7034f,  -11.5580f,33.0006f,50.5017f,  -12.8329f,31.4831f,50.2295f,  
	-11.9254f,30.4512f,49.9937f,  -13.2273f,29.0343f,49.4465f,  -12.3810f,28.0350f,49.1804f,  -13.6812f,26.7483f,48.3705f,  -12.8865f,25.7848f,48.0787f,  
	-14.1550f,24.6533f,47.0238f,  -13.4032f,23.7254f,46.7101f,  -14.6100f,22.7698f,45.4325f,  -13.8910f,21.8765f,45.1021f,  -15.0069f,21.1124f,43.6280f,  
	-14.3115f,20.2506f,43.2857f,  -15.3090f,19.6885f,41.6451f,  -14.6309f,18.8548f,41.2947f,  -68.7769f,11.8812f,28.0955f,  -67.8122f,14.1108f,29.6264f,  
	-65.9459f,13.5962f,29.1226f,  -64.9267f,15.6591f,30.5506f,  -63.1229f,15.2372f,29.7954f,  -62.0642f,17.1292f,31.1176f,  -60.3554f,16.8123f,30.1269f,  
	-59.2721f,18.5314f,31.3414f,  -57.6872f,18.3324f,30.1369f,  -56.5931f,19.8803f,31.2438f,  -55.1572f,19.8125f,29.8526f,  -54.0652f,21.1935f,30.8536f,  
	-52.7981f,21.2699f,29.3073f,  -51.7189f,22.4917f,30.2061f,  -50.6352f,22.7248f,28.5408f,  -49.5786f,23.7961f,29.3416f,  -48.6867f,24.1970f,27.5963f,  
	-47.6574f,25.1313f,28.3066f,  -46.9596f,25.7093f,26.5231f,  -45.9615f,26.5197f,27.1501f,  -45.4537f,27.2813f,25.3713f,  -44.4858f,27.9835f,25.9243f,  
	-44.1578f,28.9320f,24.1937f,  -43.2145f,29.5423f,24.6835f,  -43.0505f,30.6767f,23.0438f,  -42.1252f,31.2097f,23.4797f,  -42.1045f,32.5244f,21.9720f,  
	-41.1845f,32.9949f,22.3651f,  -41.2824f,34.4791f,21.0274f,  -40.3538f,34.8989f,21.3877f,  -40.5430f,36.5364f,20.2541f,  -39.5886f,36.9149f,20.5913f,  
	-39.8404f,38.6836f,19.6912f,  -38.8433f,39.0267f,20.0131f,  -39.1291f,40.8995f,19.3703f,  -38.0720f,41.2090f,19.6838f,  -38.3649f,43.1534f,19.3158f,  
	-37.2330f,43.4274f,19.6259f,  -37.5091f,45.4071f,19.5436f,  -36.2894f,45.6406f,19.8537f,  -36.5293f,47.6158f,20.0611f,  -35.2118f,47.8015f,20.3731f,  
	-35.4012f,49.7307f,20.8674f,  -33.9834f,49.8579f,21.1812f,  -34.1138f,51.6991f,21.9526f,  -32.5962f,51.7568f,22.2669f,  -32.6656f,53.4688f,23.2995f,  
	-31.0533f,53.4465f,23.6126f,  -31.0657f,54.9907f,24.8843f,  -29.3683f,54.8783f,25.1936f,  -29.3335f,56.2197f,26.6773f,  -27.5667f,56.0085f,26.9785f,  
	-27.4994f,57.1174f,28.6422f,  -25.6802f,56.8017f,28.9319f,  -25.5994f,57.6543f,30.7398f,  -23.7461f,57.2309f,31.0148f,  -23.6734f,57.8104f,32.9282f,  
	-21.8084f,57.2791f,33.1828f,  -21.7670f,57.5760f,35.1607f,  -19.9087f,56.9396f,35.3937f,  -19.9223f,56.9519f,37.3941f,  -18.0917f,56.2162f,37.6000f,  
	-18.1829f,55.9495f,39.5806f,  -16.3974f,55.1226f,39.7559f,  -16.5864f,54.5897f,41.6749f,  -14.8635f,53.6829f,41.8163f,  -15.1673f,52.9034f,43.6335f,  
	-13.5180f,51.9275f,43.7388f,  -13.9496f,50.9267f,45.4163f,  -12.3848f,49.8961f,45.4821f,  -12.9521f,48.7045f,46.9855f,  -11.4787f,47.6317f,47.0114f,  
	-12.1848f,46.2835f,48.3098f,  -10.8063f,45.1830f,48.2938f,  -11.6487f,43.7155f,49.3610f,  -10.3657f,42.6001f,49.3030f,  -11.3365f,41.0523f,50.1178f,  
	-10.1471f,39.9346f,50.0182f,  -11.2333f,38.3461f,50.5650f,  -10.1331f,37.2367f,50.4250f,  -11.3169f,35.6465f,50.6937f,  -10.2984f,34.5547f,50.5152f,  
	-11.5580f,33.0006f,50.5017f,  -10.6152f,31.9341f,50.2879f,  -11.9254f,30.4512f,49.9937f,  -11.0479f,29.4145f,49.7477f,  -12.3810f,28.0350f,49.1804f,  
	-11.5600f,27.0328f,48.9065f,  -12.8865f,25.7848f,48.0787f,  -12.1135f,24.8176f,47.7806f,  -13.4032f,23.7254f,46.7101f,  -12.6681f,22.7935f,46.3927f,  
	-13.8910f,21.8765f,45.1021f,  -13.1845f,20.9778f,44.7699f,  -14.3115f,20.2506f,43.2857f,  -13.6271f,19.3825f,42.9428f,  -14.6309f,18.8548f,41.2947f,  
	-13.9615f,18.0132f,40.9452f,  -67.8122f,14.1108f,29.6264f,  -66.7605f,16.2397f,31.1001f,  -64.9267f,15.6591f,30.5506f,  -63.8323f,17.6151f,31.9175f,  
	-62.0642f,17.1292f,31.1176f,  -60.9428f,18.9089f,32.3749f,  -59.2721f,18.5314f,31.3414f,  -58.1385f,20.1352f,32.4883f,  -56.5931f,19.8803f,31.2438f,  
	-55.4616f,21.3114f,32.2813f,  -54.0652f,21.1935f,30.8536f,  -52.9478f,22.4582f,31.7844f,  -51.7189f,22.4917f,30.2061f,  -50.6271f,23.5979f,31.0337f,  
	-49.5786f,23.7961f,29.3416f,  -48.5190f,24.7567f,30.0728f,  -47.6574f,25.1313f,28.3066f,  -46.6357f,25.9592f,28.9483f,  -45.9615f,26.5197f,27.1501f,  
	-44.9787f,27.2304f,27.7109f,  -44.4858f,27.9835f,25.9243f,  -43.5379f,28.5940f,26.4150f,  -43.2145f,29.5423f,24.6835f,  -42.2966f,30.0679f,25.1135f,  
	-42.1252f,31.2097f,23.4797f,  -41.2258f,31.6665f,23.8604f,  -41.1845f,32.9949f,22.3651f,  -40.2904f,33.3963f,22.7069f,  -40.3538f,34.8989f,21.3877f,  
	-39.4480f,35.2563f,21.7010f,  -39.5886f,36.9149f,20.5913f,  -38.6540f,37.2361f,20.8848f,  -38.8433f,39.0267f,20.0131f,  -37.8613f,39.3157f,20.2949f,  
	-38.0720f,41.2090f,19.6838f,  -37.0262f,41.4658f,19.9599f,  -37.2330f,43.4274f,19.6259f,  -36.1081f,43.6488f,19.9008f,  -36.2894f,45.6406f,19.8537f,  
	-35.0729f,45.8206f,20.1305f,  -35.2118f,47.8015f,20.3731f,  -33.8981f,47.9304f,20.6526f,  -33.9834f,49.8579f,21.1812f,  -32.5698f,49.9251f,21.4630f,  
	-32.5962f,51.7568f,22.2669f,  -31.0848f,51.7516f,22.5502f,  -31.0533f,53.4465f,23.6126f,  -29.4508f,53.3585f,23.8953f,  -29.3683f,54.8783f,25.1936f,  
	-27.6879f,54.6976f,25.4722f,  -27.5667f,56.0085f,26.9785f,  -25.8229f,55.7281f,27.2499f,  -25.6802f,56.8017f,28.9319f,  -23.8901f,56.4169f,29.1930f,  
	-23.7461f,57.2309f,31.0148f,  -21.9311f,56.7397f,31.2602f,  -21.8084f,57.2791f,33.1828f,  -19.9861f,56.6826f,33.4112f,  -19.9087f,56.9396f,35.3937f,  
	-18.1004f,56.2418f,35.5997f,  -18.0917f,56.2162f,37.6000f,  -16.3156f,55.4236f,37.7799f,  -16.3974f,55.1226f,39.7559f,  -14.6719f,54.2451f,39.9060f,  
	-14.8635f,53.6829f,41.8163f,  -13.2008f,52.7303f,41.9340f,  -13.5180f,51.9275f,43.7388f,  -11.9309f,50.9132f,43.8203f,  -12.3848f,49.8961f,45.4821f,  
	-10.8817f,48.8322f,45.5266f,  -11.4787f,47.6317f,47.0114f,  -10.0652f,46.5323f,47.0162f,  -10.8063f,45.1830f,48.2938f,  -9.4848f,44.0611f,48.2582f,  
	-10.3657f,42.6001f,49.3030f,  -9.1361f,41.4687f,49.2267f,  -10.1471f,39.9346f,50.0182f,  -9.0066f,38.8049f,49.9019f,  -10.1331f,37.2367f,50.4250f,  
	-9.0757f,36.1190f,50.2698f,  -10.2984f,34.5547f,50.5152f,  -9.3192f,33.4576f,50.3239f,  -10.6152f,31.9341f,50.2879f,  -9.7048f,30.8633f,50.0630f,  
	-11.0479f,29.4145f,49.7477f,  -10.1983f,28.3759f,49.4930f,  -11.5600f,27.0328f,48.9065f,  -10.7630f,26.0277f,48.6255f,  -12.1135f,24.8176f,47.7806f,  
	-11.3593f,23.8478f,47.4778f,  -12.6681f,22.7935f,46.3927f,  -11.9474f,21.8579f,46.0726f,  -13.1845f,20.9778f,44.7699f,  -12.4906f,20.0747f,44.4366f,  
	-13.6271f,19.3825f,42.9428f,  -12.9526f,18.5084f,42.6002f,  -13.9615f,18.0132f,40.9452f,  -13.2996f,17.1639f,40.5976f,  -66.7605f,16.2397f,31.1001f,  
	-65.6293f,18.2617f,32.5129f,  -63.8323f,17.6151f,31.9175f,  -62.6709f,19.4585f,33.2198f,  -60.9428f,18.9089f,32.3749f,  -59.7669f,20.5729f,33.5649f,  
	-58.1385f,20.1352f,32.4883f,  -56.9633f,21.6214f,33.5661f,  -55.4616f,21.3114f,32.2813f,  -54.3008f,22.6248f,33.2485f,  -52.9478f,22.4582f,31.7844f,  
	-51.8144f,23.6060f,32.6438f,  -50.6271f,23.5979f,31.0337f,  -49.5295f,24.5919f,31.7915f,  -48.5190f,24.7567f,30.0728f,  -47.4648f,25.6093f,30.7350f,  
	-46.6357f,25.9592f,28.9483f,  -45.6279f,26.6856f,29.5231f,  -44.9787f,27.2304f,27.7109f,  -44.0152f,27.8480f,28.2089f,  -43.5379f,28.5940f,26.4150f,  
	-42.6156f,29.1186f,26.8453f,  -42.2966f,30.0679f,25.1135f,  -41.4055f,30.5167f,25.4876f,  -41.2258f,31.6665f,23.8604f,  -40.3538f,32.0544f,24.1892f,  
	-40.2904f,33.3963f,22.7069f,  -39.4211f,33.7364f,23.0014f,  -39.4480f,35.2563f,21.7010f,  -38.5643f,35.5583f,21.9707f,  -38.6540f,37.2361f,20.8848f,  
	-37.7365f,37.5061f,21.1386f,  -37.8613f,39.3157f,20.2949f,  -36.8923f,39.5559f,20.5398f,  -37.0262f,41.4658f,19.9599f,  -35.9885f,41.6747f,20.2017f,  
	-36.1081f,43.6488f,19.9008f,  -34.9868f,43.8220f,20.1435f,  -35.0729f,45.8206f,20.1305f,  -33.8595f,45.9495f,20.3759f,  -33.8981f,47.9304f,20.6526f,  
	-32.5869f,48.0050f,20.9016f,  -32.5698f,49.9251f,21.4630f,  -31.1597f,49.9351f,21.7154f,  -31.0848f,51.7516f,22.5502f,  -29.5795f,51.6863f,22.8047f,  
	-29.4508f,53.3585f,23.8953f,  -27.8608f,53.2071f,24.1490f,  -27.6879f,54.6976f,25.4722f,  -26.0252f,54.4520f,25.7226f,  -25.8229f,55.7281f,27.2499f,  
	-24.1030f,55.3823f,27.4941f,  -23.8901f,56.4169f,29.1930f,  -22.1330f,55.9670f,29.4260f,  -21.9311f,56.7397f,31.2602f,  -20.1533f,56.1854f,31.4807f,  
	-19.9861f,56.6826f,33.4112f,  -18.2089f,56.0260f,33.6138f,  -18.1004f,56.2418f,35.5997f,  -16.3424f,55.4878f,35.7806f,  -16.3156f,55.4236f,37.7799f,  
	-14.5957f,54.5804f,37.9352f,  -14.6719f,54.2451f,39.9060f,  -13.0035f,53.3213f,40.0330f,  -13.2008f,52.7303f,41.9340f,  -11.5984f,51.7382f,42.0284f,  
	-11.9309f,50.9132f,43.8203f,  -10.4041f,49.8644f,43.8808f,  -10.8817f,48.8322f,45.5266f,  -9.4380f,47.7403f,45.5501f,  -10.0652f,46.5323f,47.0162f,  
	-8.7089f,45.4103f,47.0013f,  -9.4848f,44.0611f,48.2582f,  -8.2175f,42.9222f,48.2040f,  -9.1361f,41.4687f,49.2267f,  -7.9567f,40.3246f,49.1334f,  
	-9.0066f,38.8049f,49.9019f,  -7.9107f,37.6665f,49.7698f,  -9.0757f,36.1190f,50.2698f,  -8.0596f,34.9958f,50.1012f,  -9.3192f,33.4576f,50.3239f,  
	-8.3747f,32.3564f,50.1206f,  -9.7048f,30.8633f,50.0630f,  -8.8247f,29.7910f,49.8284f,  -10.1983f,28.3759f,49.4930f,  -9.3748f,27.3351f,49.2304f,  
	-10.7630f,26.0277f,48.6255f,  -9.9868f,25.0213f,48.3388f,  -11.3593f,23.8478f,47.4778f,  -10.6211f,22.8757f,47.1714f,  -11.9474f,21.8579f,46.0726f,  
	-11.2407f,20.9194f,45.7504f,  -12.4906f,20.0747f,44.4366f,  -11.8077f,19.1671f,44.1027f,  -12.9526f,18.5084f,42.6002f,  -12.2863f,17.6283f,42.2588f,  
	-13.2996f,17.1639f,40.5976f,  -12.6447f,16.3070f,40.2522f,  -65.6293f,18.2617f,32.5129f,  -64.4266f,20.1709f,33.8616f,  -62.6709f,19.4585f,33.2198f,  
	-61.4503f,21.1858f,34.4552f,  -59.7669f,20.5729f,33.5649f,  -58.5448f,22.1183f,34.6859f,  -56.9633f,21.6214f,33.5661f,  -55.7548f,22.9888f,34.5735f,  
	-54.3008f,22.6248f,33.2485f,  -53.1200f,23.8200f,34.1443f,  -51.8144f,23.6060f,32.6438f,  -50.6718f,24.6398f,33.4331f,  -49.5295f,24.5919f,31.7915f,  
	-48.4345f,25.4760f,32.4797f,  -47.4648f,25.6093f,30.7350f,  -46.4224f,26.3583f,31.3297f,  -45.6279f,26.6856f,29.5231f,  -44.6382f,27.3171f,30.0343f,  
	-44.0152f,27.8480f,28.2089f,  -43.0769f,28.3781f,28.6456f,  -42.6156f,29.1186f,26.8453f,  -41.7206f,29.5652f,27.2189f,  -41.4055f,30.5167f,25.4876f,  
	-40.5430f,30.8960f,25.8092f,  -40.3538f,32.0544f,24.1892f,  -39.5082f,32.3810f,24.4703f,  -39.4211f,33.7364f,23.0014f,  -38.5758f,34.0220f,23.2520f,  
	-38.5643f,35.5583f,21.9707f,  -37.6996f,35.8111f,22.2005f,  -37.7365f,37.5061f,21.1386f,  -36.8338f,37.7303f,21.3558f,  -36.8923f,39.5559f,20.5398f,  
	-35.9327f,39.7519f,20.7510f,  -35.9885f,41.6747f,20.2017f,  -34.9550f,41.8398f,20.4119f,  -34.9868f,43.8220f,20.1435f,  -33.8682f,43.9491f,20.3558f,  
	-33.8595f,45.9495f,20.3759f,  -32.6472f,46.0295f,20.5923f,  -32.5869f,48.0050f,20.9016f,  -31.2767f,48.0281f,21.1225f,  -31.1597f,49.9351f,21.7154f,  
	-29.7523f,49.8904f,21.9404f,  -29.5795f,51.6863f,22.8047f,  -28.0826f,51.5626f,23.0317f,  -27.8608f,53.2071f,24.1490f,  -26.2836f,52.9953f,24.3760f,  
	-26.0252f,54.4520f,25.7226f,  -24.3812f,54.1449f,25.9470f,  -24.1030f,55.3823f,27.4941f,  -22.4107f,54.9744f,27.7115f,  -22.1330f,55.9670f,29.4260f,  
	-20.4074f,55.4564f,29.6351f,  -20.1533f,56.1854f,31.4807f,  -18.4154f,55.5724f,31.6764f,  -18.2089f,56.0260f,33.6138f,  -16.4773f,55.3138f,33.7922f,  
	-16.3424f,55.4878f,35.7806f,  -14.6362f,54.6830f,35.9378f,  -14.5957f,54.5804f,37.9352f,  -12.9294f,53.6904f,38.0682f,  -13.0035f,53.3213f,40.0330f,  
	-11.3927f,52.3572f,40.1372f,  -11.5984f,51.7382f,42.0284f,  -10.0540f,50.7106f,42.1022f,  -10.4041f,49.8644f,43.8808f,  -8.9355f,48.7864f,43.9206f,  
	-9.4380f,47.7403f,45.5501f,  -8.0511f,46.6246f,45.5539f,  -8.7089f,45.4103f,47.0013f,  -7.4071f,44.2701f,46.9677f,  -8.2175f,42.9222f,48.2040f,  
	-7.0014f,41.7695f,48.1325f,  -7.9567f,40.3246f,49.1334f,  -6.8234f,39.1709f,49.0237f,  -7.9107f,37.6665f,49.7698f,  -6.8580f,36.5222f,49.6236f,  
	-8.0596f,34.9958f,50.1012f,  -7.0802f,33.8684f,49.9197f,  -8.3747f,32.3564f,50.1206f,  -7.4626f,31.2542f,49.9067f,  -8.8247f,29.7910f,49.8284f,  
	-7.9727f,28.7173f,49.5849f,  -9.3748f,27.3351f,49.2304f,  -8.5742f,26.2939f,48.9610f,  -9.9868f,25.0213f,48.3388f,  -9.2286f,24.0139f,48.0475f,  
	-10.6211f,22.8757f,47.1714f,  -9.8988f,21.9022f,46.8620f,  -11.2407f,20.9194f,45.7504f,  -10.5463f,19.9780f,45.4269f,  -11.8077f,19.1671f,44.1027f,  
	-11.1340f,18.2552f,43.7692f,  -12.2863f,17.6283f,42.2588f,  -11.6278f,16.7424f,41.9191f,  -12.6447f,16.3070f,40.2522f,  -11.9963f,15.4428f,39.9094f,  
	-64.4266f,20.1709f,33.8616f,  -63.1600f,21.9634f,35.1438f,  -61.4503f,21.1858f,34.4552f,  -60.1789f,22.7936f,35.6217f,  -58.5448f,22.1183f,34.6859f,  
	-57.2848f,23.5438f,35.7368f,  -55.7548f,22.9888f,34.5735f,  -54.5218f,24.2365f,35.5096f,  -53.1200f,23.8200f,34.1443f,  -51.9262f,24.8994f,34.9697f,  
	-50.6718f,24.6398f,33.4331f,  -49.5283f,25.5616f,34.1525f,  -48.4345f,25.4760f,32.4797f,  -47.3486f,26.2543f,33.0998f,  -46.4224f,26.3583f,31.3297f,  
	-45.3962f,27.0103f,31.8600f,  -44.6382f,27.3171f,30.0343f,  -43.6725f,27.8589f,30.4834f,  -43.0769f,28.3781f,28.6456f,  -42.1657f,28.8282f,29.0247f,  
	-41.7206f,29.5652f,27.2189f,  -40.8547f,29.9410f,27.5392f,  -40.5430f,30.8960f,25.8092f,  -39.7079f,31.2135f,26.0825f,  -39.5082f,32.3810f,24.4703f,  
	-38.6883f,32.6535f,24.7068f,  -38.5758f,34.0220f,23.2520f,  -37.7514f,34.2597f,23.4625f,  -37.6996f,35.8111f,22.2005f,  -36.8514f,36.0202f,22.3938f,  
	-36.8338f,37.7303f,21.3558f,  -35.9417f,37.9133f,21.5395f,  -35.9327f,39.7519f,20.7510f,  -34.9780f,39.9080f,20.9312f,  -34.9550f,41.8398f,20.4119f,  
	-33.9241f,41.9634f,20.5927f,  -33.8682f,43.9491f,20.3558f,  -32.7495f,44.0322f,20.5402f,  -32.6472f,46.0295f,20.5923f,  -31.4338f,46.0631f,20.7816f,  
	-31.2767f,48.0281f,21.1225f,  -29.9662f,48.0015f,21.3173f,  -29.7523f,49.8904f,21.9404f,  -28.3494f,49.7921f,22.1393f,  -28.0826f,51.5626f,23.0317f,  
	-26.5937f,51.3828f,23.2333f,  -26.2836f,52.9953f,24.3760f,  -24.7199f,52.7258f,24.5782f,  -24.3812f,54.1449f,25.9470f,  -22.7594f,53.7787f,26.1460f,  
	-22.4107f,54.9744f,27.7115f,  -20.7441f,54.5082f,27.9061f,  -20.4074f,55.4564f,29.6351f,  -18.7163f,54.8888f,29.8205f,  -18.4154f,55.5724f,31.6764f,  
	-16.7181f,54.9048f,31.8488f,  -16.4773f,55.3138f,33.7922f,  -14.7928f,54.5512f,33.9477f,  -14.6362f,54.6830f,35.9378f,  -12.9797f,53.8313f,36.0733f,  
	-12.9294f,53.6904f,38.0682f,  -11.3175f,52.7593f,38.1789f,  -11.3927f,52.3572f,40.1372f,  -9.8374f,51.3565f,40.2212f,  -10.0540f,50.7106f,42.1022f,  
	-8.5663f,49.6526f,42.1555f,  -8.9355f,48.7864f,43.9206f,  -7.5229f,47.6833f,43.9408f,  -8.0511f,46.6246f,45.5539f,  -6.7187f,45.4894f,45.5390f,  
	-7.4071f,44.2701f,46.9677f,  -6.1569f,43.1150f,46.9166f,  -7.0014f,41.7695f,48.1325f,  -5.8325f,40.6064f,48.0441f,  -6.8234f,39.1709f,49.0237f,  
	-5.7348f,38.0108f,48.8992f,  -6.8580f,36.5222f,49.6236f,  -5.8436f,35.3734f,49.4637f,  -7.0802f,33.8684f,49.9197f,  -6.1354f,32.7401f,49.7267f,  
	-7.4626f,31.2542f,49.9067f,  -6.5806f,30.1507f,49.6830f,  -7.9727f,28.7173f,49.5849f,  -7.1457f,27.6439f,49.3337f,  -8.5742f,26.2939f,48.9610f,  
	-7.7935f,25.2528f,48.6858f,  -9.2286f,24.0139f,48.0475f,  -8.4879f,23.0063f,47.7521f,  -9.8988f,21.9022f,46.8620f,  -9.1902f,20.9272f,46.5503f,  
	-10.5463f,19.9780f,45.4269f,  -9.8625f,19.0340f,45.1030f,  -11.1340f,18.2552f,43.7692f,  -10.4691f,17.3394f,43.4367f,  -11.6278f,16.7424f,41.9191f,  
	-10.9763f,15.8509f,41.5813f,  -11.9963f,15.4428f,39.9094f,  -11.3543f,14.5718f,39.5692f,  -63.1600f,21.9634f,35.1438f,  -61.8377f,23.6358f,36.3574f,  
	-60.1789f,22.7936f,35.6217f,  -58.8647f,24.2804f,36.7183f,  -57.2848f,23.5438f,35.7368f,  -55.9955f,24.8482f,36.7162f,  -54.5218f,24.2365f,35.5096f,  
	-53.2713f,25.3664f,36.3750f,  -51.9262f,24.8994f,34.9697f,  -50.7274f,25.8647f,35.7249f,  -49.5283f,25.5616f,34.1525f,  -48.3907f,26.3752f,34.8032f,  
	-47.3486f,26.2543f,33.0998f,  -46.2763f,26.9328f,33.6549f,  -45.3962f,27.0103f,31.8600f,  -44.3923f,27.5700f,32.3275f,  -43.6725f,27.8589f,30.4834f,  
	-42.7332f,28.3184f,30.8741f,  -42.1657f,28.8282f,29.0247f,  -41.2835f,29.2057f,29.3496f,  -40.8547f,29.9410f,27.5392f,  -40.0168f,30.2537f,27.8103f,  
	-39.7079f,31.2135f,26.0825f,  -38.8999f,31.4762f,26.3104f,  -38.6883f,32.6535f,24.7068f,  -37.8909f,32.8783f,24.9028f,  -37.7514f,34.2597f,23.4625f,  
	-36.9450f,34.4549f,23.6361f,  -36.8514f,36.0202f,22.3938f,  -36.0152f,36.1905f,22.5537f,  -35.9417f,37.9133f,21.5395f,  -35.0554f,38.0596f,21.6925f,  
	-34.9780f,39.9080f,20.9312f,  -34.0262f,40.0265f,21.0826f,  -33.9241f,41.9634f,20.5927f,  -32.8923f,42.0475f,20.7465f,  -32.7495f,44.0322f,20.5402f,  
	-31.6283f,44.0737f,20.6986f,  -31.4338f,46.0631f,20.7816f,  -30.2175f,46.0519f,20.9459f,  -29.9662f,48.0015f,21.3173f,  -28.6567f,47.9260f,21.4871f,  
	-28.3494f,49.7921f,22.1393f,  -26.9500f,49.6421f,22.3139f,  -26.5937f,51.3828f,23.2333f,  -25.1134f,51.1493f,23.4112f,  -24.7199f,52.7258f,24.5782f,  
	-23.1732f,52.4004f,24.7563f,  -22.7594f,53.7787f,26.1460f,  -21.1576f,53.3569f,26.3231f,  -20.7441f,54.5082f,27.9061f,  -19.1064f,53.9869f,28.0780f,  
	-18.7163f,54.8888f,29.8205f,  -17.0605f,54.2678f,29.9836f,  -16.7181f,54.9048f,31.8488f,  -15.0628f,54.1872f,31.9992f,  -14.7928f,54.5512f,33.9477f,  
	-13.1537f,53.7416f,34.0824f,  -12.9797f,53.8313f,36.0733f,  -11.3741f,52.9379f,36.1872f,  -11.3175f,52.7593f,38.1789f,  -9.7581f,51.7908f,38.2701f,  
	-9.8374f,51.3565f,40.2212f,  -8.3367f,50.3242f,40.2851f,  -8.5663f,49.6526f,42.1555f,  -7.1332f,48.5682f,42.1894f,  -7.5229f,47.6833f,43.9408f,  
	-6.1640f,46.5593f,43.9421f,  -6.7187f,45.4894f,45.5390f,  -5.4378f,44.3381f,45.5062f,  -6.1569f,43.1150f,46.9166f,  -4.9547f,41.9484f,46.8480f,  
	-5.8325f,40.6064f,48.0441f,  -4.7093f,39.4360f,47.9405f,  -5.7348f,38.0108f,48.8992f,  -4.6860f,36.8456f,48.7604f,  -5.8436f,35.3734f,49.4637f,  
	-4.8657f,34.2236f,49.2914f,  -6.1354f,32.7401f,49.7267f,  -5.2224f,31.6107f,49.5230f,  -6.5806f,30.1507f,49.6830f,  -5.7256f,29.0482f,49.4506f,  
	-7.1457f,27.6439f,49.3337f,  -6.3404f,26.5715f,49.0756f,  -7.7935f,25.2528f,48.6858f,  -7.0321f,24.2128f,48.4057f,  -8.4879f,23.0063f,47.7521f,  
	-7.7624f,21.9985f,47.4536f,  -9.1902f,20.9272f,46.5503f,  -8.4934f,19.9512f,46.2372f,  -9.8625f,19.0340f,45.1030f,  -9.1884f,18.0877f,44.7791f,  
	-10.4691f,17.3394f,43.4367f,  -9.8119f,16.4194f,43.1054f,  -10.9763f,15.8509f,41.5813f,  -10.3318f,14.9541f,41.2456f,  -11.3543f,14.5718f,39.5692f,  
	-10.7178f,13.6936f,39.2322f,  -61.8377f,23.6358f,36.3574f,  -60.4675f,25.1860f,37.5013f,  -58.8647f,24.2804f,36.7183f,  -57.5162f,25.6446f,37.7436f,  
	-55.9955f,24.8482f,36.7162f,  -54.6841f,26.0328f,37.6249f,  -53.2713f,25.3664f,36.3750f,  -52.0114f,26.3800f,37.1699f,  -50.7274f,25.8647f,35.7249f,  
	-49.5307f,26.7190f,36.4108f,  -48.3907f,26.3752f,34.8032f,  -47.2635f,27.0862f,35.3881f,  -46.2763f,26.9328f,33.6549f,  -45.2239f,27.5162f,34.1462f,  
	-44.3923f,27.5700f,32.3275f,  -43.4133f,28.0445f,32.7354f,  -42.7332f,28.3184f,30.8741f,  -41.8222f,28.7027f,31.2095f,  -41.2835f,29.2057f,29.3496f,  
	-40.4296f,29.5181f,29.6244f,  -40.0168f,30.2537f,27.8103f,  -39.2070f,30.5102f,28.0351f,  -38.8999f,31.4762f,26.3104f,  -38.1157f,31.6908f,26.4970f,  
	-37.8909f,32.8783f,24.9028f,  -37.1129f,33.0613f,25.0615f,  -36.9450f,34.4549f,23.6361f,  -36.1520f,34.6128f,23.7762f,  -36.0152f,36.1905f,22.5537f,  
	-35.1860f,36.3262f,22.6828f,  -35.0554f,38.0596f,21.6925f,  -34.1727f,38.1714f,21.8170f,  -34.0262f,40.0265f,21.0826f,  -33.0731f,40.1092f,21.2074f,  
	-32.8923f,42.0475f,20.7465f,  -31.8569f,42.0941f,20.8749f,  -31.6283f,44.0737f,20.6986f,  -30.5020f,44.0749f,20.8326f,  -30.2175f,46.0519f,20.9459f,  
	-28.9992f,45.9962f,21.0862f,  -28.6567f,47.9260f,21.4871f,  -27.3467f,47.8031f,21.6337f,  -26.9500f,49.6421f,22.3139f,  -25.5548f,49.4423f,22.4660f,  
	-25.1134f,51.1493f,23.4112f,  -23.6450f,50.8631f,23.5662f,  -23.1732f,52.4004f,24.7563f,  -21.6408f,52.0223f,24.9134f,  -21.1576f,53.3569f,26.3231f,  
	-19.5792f,52.8820f,26.4786f,  -19.1064f,53.9869f,28.0780f,  -17.4988f,53.4135f,28.2285f,  -17.0605f,54.2678f,29.9836f,  -15.4415f,53.5973f,30.1255f,  
	-15.0628f,54.1872f,31.9992f,  -13.4482f,53.4228f,32.1295f,  -13.1537f,53.7416f,34.0824f,  -11.5614f,52.8898f,34.1961f,  -11.3741f,52.9379f,36.1872f,  
	-9.8173f,52.0063f,36.2820f,  -9.7581f,51.7908f,38.2701f,  -8.2506f,50.7896f,38.3414f,  -8.3367f,50.3242f,40.2851f,  -6.8886f,49.2641f,40.3299f,  
	-7.1332f,48.5682f,42.1894f,  -5.7527f,47.4614f,42.2046f,  -6.1640f,46.5593f,43.9421f,  -4.8564f,45.4178f,43.9257f,  -5.4378f,44.3381f,45.5062f,  
	-4.2053f,43.1741f,45.4558f,  -4.9547f,41.9484f,46.8480f,  -3.7990f,40.7736f,46.7638f,  -4.7093f,39.4360f,47.9405f,  -3.6272f,38.2599f,47.8218f,  
	-4.6860f,36.8456f,48.7604f,  -3.6754f,35.6791f,48.6083f,  -4.8657f,34.2236f,49.2914f,  -3.9213f,33.0725f,49.1076f,  -5.2224f,31.6107f,49.5230f,  
	-4.3384f,30.4826f,49.3096f,  -5.7256f,29.0482f,49.4506f,  -4.8943f,27.9472f,49.2101f,  -6.3404f,26.5715f,49.0756f,  -5.5563f,25.5010f,48.8115f,  
	-7.0321f,24.2128f,48.4057f,  -6.2875f,23.1736f,48.1213f,  -7.7624f,21.9985f,47.4536f,  -7.0502f,20.9911f,47.1526f,  -8.4934f,19.9512f,46.2372f,  
	-7.8076f,18.9744f,45.9233f,  -9.1884f,18.0877f,44.7791f,  -8.5228f,17.1387f,44.4557f,  -9.8119f,16.4194f,43.1054f,  -9.1622f,15.4958f,42.7756f,  
	-10.3318f,14.9541f,41.2456f,  -9.6929f,14.0518f,40.9125f,  -10.7178f,13.6936f,39.2322f,  -10.0865f,12.8088f,38.8982f,  -60.4675f,25.1860f,37.5013f,  
	-59.0576f,26.6122f,38.5740f,  -57.5162f,25.6446f,37.7436f,  -56.1403f,26.8872f,38.6979f,  -54.6841f,26.0328f,37.6249f,  -53.3582f,27.0988f,38.4628f,  
	-52.0114f,26.3800f,37.1699f,  -50.7492f,27.2798f,37.8950f,  -49.5307f,26.7190f,36.4108f,  -48.3407f,27.4676f,37.0302f,  -47.2635f,27.0862f,35.3881f,  
	-46.1532f,27.6987f,35.9084f,  -45.2239f,27.5162f,34.1462f,  -44.1945f,28.0112f,34.5768f,  -43.4133f,28.0445f,32.7354f,  -42.4615f,28.4407f,33.0869f,  
	-41.8222f,28.7027f,31.2095f,  -40.9392f,29.0193f,31.4935f,  -40.4296f,29.5181f,29.6244f,  -39.6043f,29.7721f,29.8516f,  -39.2070f,30.5102f,28.0351f,  
	-38.4220f,30.7173f,28.2176f,  -38.1157f,31.6908f,26.4970f,  -37.3523f,31.8631f,26.6455f,  -37.1129f,33.0613f,25.0615f,  -36.3499f,33.2073f,25.1860f,  
	-36.1520f,34.6128f,23.7762f,  -35.3675f,34.7376f,23.8850f,  -35.1860f,36.3262f,22.6828f,  -34.3612f,36.4299f,22.7833f,  -34.1727f,38.1714f,21.8170f,  
	-33.2886f,38.2505f,21.9152f,  -33.0731f,40.1092f,21.2074f,  -32.1156f,40.1581f,21.3073f,  -31.8569f,42.0941f,20.8749f,  -30.8148f,42.1044f,20.9794f,  
	-30.5020f,44.0749f,20.8326f,  -29.3711f,44.0357f,20.9436f,  -28.9992f,45.9962f,21.0862f,  -27.7768f,45.8971f,21.2042f,  -27.3467f,47.8031f,21.6337f,  
	-26.0366f,47.6341f,21.7586f,  -25.5548f,49.4423f,22.4660f,  -24.1666f,49.1931f,22.5960f,  -23.6450f,50.8631f,23.5662f,  -22.1855f,50.5272f,23.7010f,  
	-21.6408f,52.0223f,24.9134f,  -20.1264f,51.5932f,25.0497f,  -19.5792f,52.8820f,26.4786f,  -18.0254f,52.3567f,26.6136f,  -17.4988f,53.4135f,28.2285f,  
	-15.9225f,52.7915f,28.3588f,  -15.4415f,53.5973f,30.1255f,  -13.8584f,52.8804f,30.2481f,  -13.4482f,53.4228f,32.1295f,  -11.8763f,52.6159f,32.2395f,  
	-11.5614f,52.8898f,34.1961f,  -10.0142f,51.9992f,34.2912f,  -9.8173f,52.0063f,36.2820f,  -8.3095f,51.0409f,36.3576f,  -8.2506f,50.7896f,38.3414f,  
	-6.7934f,49.7593f,38.3939f,  -6.8886f,49.2641f,40.3299f,  -5.4916f,48.1803f,40.3560f,  -5.7527f,47.4614f,42.2046f,  -4.4225f,46.3357f,42.2019f,  
	-4.8564f,45.4178f,43.9257f,  -3.5970f,44.2623f,43.8912f,  -4.2053f,43.1741f,45.4558f,  -3.0196f,42.0005f,45.3893f,  -3.7990f,40.7736f,46.7638f,  
	-2.6854f,39.5920f,46.6640f,  -3.6272f,38.2599f,47.8218f,  -2.5847f,37.0818f,47.6891f,  -3.6754f,35.6791f,48.6083f,  -2.6996f,34.5110f,48.4440f,  
	-3.9213f,33.0725f,49.1076f,  -3.0076f,31.9228f,48.9131f,  -4.3384f,30.4826f,49.3096f,  -3.4798f,29.3565f,49.0870f,  -4.8943f,27.9472f,49.2101f,  
	-4.0858f,26.8490f,48.9626f,  -5.5563f,25.5010f,48.8115f,  -4.7906f,24.4324f,48.5420f,  -6.2875f,23.1736f,48.1213f,  -5.5577f,22.1360f,47.8334f,  
	-7.0502f,20.9911f,47.1526f,  -6.3504f,19.9845f,46.8498f,  -7.8076f,18.9744f,45.9233f,  -7.1313f,17.9964f,45.6089f,  -8.5228f,17.1387f,44.4557f,  
	-7.8656f,16.1877f,44.1329f,  -9.1622f,15.4958f,42.7756f,  -8.5187f,14.5682f,42.4476f,  -9.6929f,14.0518f,40.9125f,  -9.0596f,13.1442f,40.5820f,  
	-10.0865f,12.8088f,38.8982f,  -9.4606f,11.9176f,38.5672f,  -59.0576f,26.6122f,38.5740f,  -57.6151f,27.9148f,39.5759f,  -56.1403f,26.8872f,38.6979f,  
	-54.7449f,28.0087f,39.5814f,  -53.3582f,27.0988f,38.4628f,  -52.0254f,28.0480f,39.2304f,  -50.7492f,27.2798f,37.8950f,  -49.4896f,28.0705f,38.5528f,  
	-48.3407f,27.4676f,37.0302f,  -47.1642f,28.1141f,37.5839f,  -46.1532f,27.6987f,35.9084f,  -45.0633f,28.2190f,36.3668f,  -44.1945f,28.0112f,34.5768f,  
	-43.1904f,28.4241f,34.9498f,  -42.4615f,28.4407f,33.0869f,  -41.5369f,28.7658f,33.3857f,  -40.9392f,29.0193f,31.4935f,  -40.0850f,29.2746f,31.7287f,  
	-39.6043f,29.7721f,29.8516f,  -38.8045f,29.9746f,30.0354f,  -38.4220f,30.7173f,28.2176f,  -37.6591f,30.8808f,28.3611f,  -37.3523f,31.8631f,26.6455f,  
	-36.6053f,31.9981f,26.7591f,  -36.3499f,33.2073f,25.1860f,  -35.5968f,33.3209f,25.2787f,  -35.3675f,34.7376f,23.8850f,  -34.5883f,34.8318f,23.9651f,  
	-34.3612f,36.4299f,22.7833f,  -33.5354f,36.5033f,22.8577f,  -33.2886f,38.2505f,21.9152f,  -32.4001f,38.2988f,21.9887f,  -32.1156f,40.1581f,21.3073f,  
	-31.1504f,40.1742f,21.3838f,  -30.8148f,42.1044f,20.9794f,  -29.7660f,42.0781f,21.0616f,  -29.3711f,44.0357f,20.9436f,  -28.2330f,43.9569f,21.0329f,  
	-27.7768f,45.8971f,21.2042f,  -26.5507f,45.7556f,21.3012f,  -26.0366f,47.6341f,21.7586f,  -24.7292f,47.4190f,21.8624f,  -24.1666f,49.1931f,22.5960f,  
	-22.7823f,48.8971f,22.7066f,  -22.1855f,50.5272f,23.7010f,  -20.7387f,50.1425f,23.8158f,  -20.1264f,51.5932f,25.0497f,  -18.6313f,51.1155f,25.1664f,  
	-18.0254f,52.3567f,26.6136f,  -16.4975f,51.7837f,26.7291f,  -15.9225f,52.7915f,28.3588f,  -14.3772f,52.1237f,28.4704f,  -13.8584f,52.8804f,30.2481f,  
	-12.3133f,52.1209f,30.3512f,  -11.8763f,52.6159f,32.2395f,  -10.3452f,51.7696f,32.3317f,  -10.0142f,51.9992f,34.2912f,  -8.5124f,51.0737f,34.3676f,  
	-8.3095f,51.0409f,36.3576f,  -6.8492f,50.0451f,36.4147f,  -6.7934f,49.7593f,38.3939f,  -5.3854f,48.7037f,38.4281f,  -5.4916f,48.1803f,40.3560f,  
	-4.1437f,47.0760f,40.3644f,  -4.4225f,46.3357f,42.2019f,  -3.1402f,45.1945f,42.1812f,  -3.5970f,44.2623f,43.8912f,  -2.3844f,43.0959f,43.8405f,  
	-3.0196f,42.0005f,45.3893f,  -1.8766f,40.8191f,45.3068f,  -2.6854f,39.5920f,46.6640f,  -1.6123f,38.4077f,46.5495f,  -2.5847f,37.0818f,47.6891f,  
	-1.5782f,35.9015f,47.5435f,  -2.6996f,34.5110f,48.4440f,  -1.7562f,33.3440f,48.2680f,  -3.0076f,31.9228f,48.9131f,  -2.1211f,30.7753f,48.7084f,  
	-3.4798f,29.3565f,49.0870f,  -2.6458f,28.2336f,48.8563f,  -4.0858f,26.8490f,48.9626f,  -3.2974f,25.7532f,48.7086f,  -4.7906f,24.4324f,48.5420f,  
	-4.0412f,23.3664f,48.2679f,  -5.5577f,22.1360f,47.8334f,  -4.8416f,21.1005f,47.5426f,  -6.3504f,19.9845f,46.8498f,  -5.6610f,18.9779f,46.5454f,  
	-7.1313f,17.9964f,45.6089f,  -6.4642f,17.0178f,45.2943f,  -7.8656f,16.1877f,44.1329f,  -7.2152f,15.2341f,43.8112f,  -8.5187f,14.5682f,42.4476f,  
	-7.8811f,13.6368f,42.1216f,  -9.0596f,13.1442f,40.5820f,  -8.4318f,12.2317f,40.2539f,  -9.4606f,11.9176f,38.5672f,  -8.8381f,11.0191f,38.2399f,  
	-57.6151f,27.9148f,39.5759f,  -56.1476f,29.0941f,40.5067f,  -54.7449f,28.0087f,39.5814f,  -53.3374f,29.0105f,40.3942f,  -52.0254f,28.0480f,39.2304f,  
	-50.6906f,28.8847f,39.9301f,  -49.4896f,28.0705f,38.5528f,  -48.2395f,28.7555f,39.1442f,  -47.1642f,28.1141f,37.5839f,  -46.0049f,28.6643f,38.0747f,  
	-45.0633f,28.2190f,36.3668f,  -43.9966f,28.6532f,36.7663f,  -43.1904f,28.4241f,34.9498f,  -42.2125f,28.7620f,35.2687f,  -41.5369f,28.7658f,33.3857f,  
	-40.6408f,29.0260f,33.6343f,  -40.0850f,29.2746f,31.7287f,  -39.2568f,29.4753f,31.9191f,  -38.8045f,29.9746f,30.0354f,  -38.0279f,30.1313f,30.1788f,  
	-37.6591f,30.8808f,28.3611f,  -36.9139f,31.0058f,28.4685f,  -36.6053f,31.9981f,26.7591f,  -35.8699f,32.1003f,26.8400f,  -35.5968f,33.3209f,25.2787f,  
	-34.8504f,33.4045f,25.3421f,  -34.5883f,34.8318f,23.9651f,  -33.8091f,34.8973f,24.0188f,  -33.5354f,36.5033f,22.8577f,  -32.7054f,36.5481f,22.9072f,  
	-32.4001f,38.2988f,21.9887f,  -31.5033f,38.3171f,22.0388f,  -31.1504f,40.1742f,21.3838f,  -30.1771f,40.1569f,21.4382f,  -29.7660f,42.0781f,21.0616f,  
	-28.7076f,42.0155f,21.1226f,  -28.2330f,43.9569f,21.0329f,  -27.0881f,43.8390f,21.1017f,  -26.5507f,45.7556f,21.3012f,  -25.3232f,45.5712f,21.3780f,  
	-24.7292f,47.4190f,21.8624f,  -23.4211f,47.1600f,21.9473f,  -22.7823f,48.8971f,22.7066f,  -21.4058f,48.5546f,22.7980f,  -20.7387f,50.1425f,23.8158f,  
	-19.3063f,49.7109f,23.9119f,  -18.6313f,51.1155f,25.1664f,  -17.1568f,50.5910f,25.2645f,  -16.4975f,51.7837f,26.7291f,  -14.9956f,51.1655f,26.8267f,  
	-14.3772f,52.1237f,28.4704f,  -12.8651f,51.4131f,28.5632f,  -12.3133f,52.1209f,30.3512f,  -10.8047f,51.3215f,30.4369f,  -10.3452f,51.7696f,32.3317f,  
	-8.8558f,50.8876f,32.4056f,  -8.5124f,51.0737f,34.3676f,  -7.0551f,50.1168f,34.4261f,  -6.8492f,50.0451f,36.4147f,  -5.4357f,49.0227f,36.4538f,  
	-5.3854f,48.7037f,38.4281f,  -4.0246f,47.6262f,38.4447f,  -4.1437f,47.0760f,40.3644f,  -2.8427f,45.9547f,40.3548f,  -3.1402f,45.1945f,42.1812f,  
	-1.9041f,44.0409f,42.1439f,  -2.3844f,43.0959f,43.8405f,  -1.2146f,41.9204f,43.7734f,  -1.8766f,40.8191f,45.3068f,  -0.7749f,39.6340f,45.2090f,  
	-1.6123f,38.4077f,46.5495f,  -0.5763f,37.2203f,46.4214f,  -1.5782f,35.9015f,47.5435f,  -0.6055f,34.7220f,47.3853f,  -1.7562f,33.3440f,48.2680f,  
	-0.8416f,32.1792f,48.0808f,  -2.1211f,30.7753f,48.7084f,  -1.2608f,29.6311f,48.4946f,  -2.6458f,28.2336f,48.8563f,  -1.8334f,27.1136f,48.6182f,  
	-3.2974f,25.7532f,48.7086f,  -2.5269f,24.6610f,48.4489f,  -4.0412f,23.3664f,48.2679f,  -3.3071f,22.3036f,47.9899f,  -4.8416f,21.1005f,47.5426f,  
	-4.1371f,20.0663f,47.2492f,  -5.6610f,18.9779f,46.5454f,  -4.9819f,17.9720f,46.2401f,  -6.4642f,17.0178f,45.2943f,  -5.8046f,16.0381f,44.9800f,  
	-7.2152f,15.2341f,43.8112f,  -6.5712f,14.2783f,43.4908f,  -7.8811f,13.6368f,42.1216f,  -7.2494f,12.7020f,41.7976f,  -8.4318f,12.2317f,40.2539f,  
	-7.8074f,11.3133f,39.9290f,  -8.8381f,11.0191f,38.2399f,  -8.2205f,10.1147f,37.9152f,  -56.1476f,29.0941f,40.5067f,  -54.6627f,30.1507f,41.3664f,  
	-53.3374f,29.0105f,40.3942f,  -51.9231f,29.8963f,41.1383f,  -50.6906f,28.8847f,39.9301f,  -49.3609f,29.6117f,40.5624f,  -48.2395f,28.7555f,39.1442f,  
	-47.0032f,29.3397f,39.6714f,  -46.0049f,28.6643f,38.0747f,  -44.8662f,29.1240f,38.5054f,  -43.9966f,28.6532f,36.7663f,  -42.9544f,29.0079f,37.1103f,  
	-42.2125f,28.7620f,35.2687f,  -41.2624f,29.0308f,35.5358f,  -40.6408f,29.0260f,33.6343f,  -39.7710f,29.2281f,33.8365f,  -39.2568f,29.4753f,31.9191f,  
	-38.4526f,29.6274f,32.0677f,  -38.0279f,30.1313f,30.1788f,  -37.2705f,30.2474f,30.2851f,  -36.9139f,31.0058f,28.4685f,  -36.1822f,31.0967f,28.5422f,  
	-35.8699f,32.1003f,26.8400f,  -35.1427f,32.1723f,26.8908f,  -34.8504f,33.4045f,25.3421f,  -34.1050f,33.4602f,25.3786f,  -33.8091f,34.8973f,24.0188f,  
	-33.0264f,34.9358f,24.0473f,  -32.7054f,36.5481f,22.9072f,  -31.8673f,36.5651f,22.9333f,  -31.5033f,38.3171f,22.0388f,  -30.5974f,38.3048f,22.0671f,  
	-30.1771f,40.1569f,21.4382f,  -29.1923f,40.1063f,21.4718f,  -28.7076f,42.0155f,21.1226f,  -27.6397f,41.9170f,21.1633f,  -27.0881f,43.8390f,21.1017f,  
	-25.9383f,43.6812f,21.1509f,  -25.3232f,45.5712f,21.3780f,  -24.0908f,45.3455f,21.4361f,  -23.4211f,47.1600f,21.9473f,  -22.1160f,46.8569f,22.0137f,  
	-21.4058f,48.5546f,22.7980f,  -20.0386f,48.1672f,22.8713f,  -19.3063f,49.7109f,23.9119f,  -17.8889f,49.2340f,23.9899f,  -17.1568f,50.5910f,25.2645f,  
	-15.7032f,50.0224f,25.3453f,  -14.9956f,51.1655f,26.8267f,  -13.5221f,50.5048f,26.9063f,  -12.8651f,51.4131f,28.5632f,  -11.3849f,50.6625f,28.6394f,  
	-10.8047f,51.3215f,30.4369f,  -9.3338f,50.4857f,30.5051f,  -8.8558f,50.8876f,32.4056f,  -7.4075f,49.9730f,32.4621f,  -7.0551f,50.1168f,34.4261f,  
	-5.6418f,49.1317f,34.4669f,  -5.4357f,49.0227f,36.4538f,  -4.0674f,47.9767f,36.4756f,  -4.0246f,47.6262f,38.4447f,  -2.7096f,46.5300f,38.4434f,  
	-2.8427f,45.9547f,40.3548f,  -1.5872f,44.8194f,40.3286f,  -1.9041f,44.0409f,42.1439f,  -0.7109f,42.8768f,42.0901f,  -1.2146f,41.9204f,43.7734f,  
	-0.0867f,40.7398f,43.6905f,  -0.7749f,39.6340f,45.2090f,  0.2891f,38.4448f,45.0971f,  -0.5763f,37.2203f,46.4214f,  0.4249f,36.0330f,46.2801f,  
	-0.6055f,34.7220f,47.3853f,  0.3370f,33.5442f,47.2149f,  -0.8416f,32.1792f,48.0808f,  0.0454f,31.0177f,47.8836f,  -1.2608f,29.6311f,48.4946f,  
	-0.4237f,28.4902f,48.2722f,  -1.8334f,27.1136f,48.6182f,  -1.0405f,25.9978f,48.3731f,  -2.5269f,24.6610f,48.4489f,  -1.7731f,23.5729f,48.1842f,  
	-3.3071f,22.3036f,47.9899f,  -2.5858f,21.2430f,47.7082f,  -4.1371f,20.0663f,47.2492f,  -3.4442f,19.0340f,46.9539f,  -4.9819f,17.9720f,46.2401f,  
	-4.3112f,16.9665f,45.9340f,  -5.8046f,16.0381f,44.9800f,  -5.1522f,15.0576f,44.6661f,  -6.5712f,14.2783f,43.4908f,  -5.9335f,13.3204f,43.1717f,  
	-7.2494f,12.7020f,41.7976f,  -6.6211f,11.7626f,41.4760f,  -7.8074f,11.3133f,39.9290f,  -7.1880f,10.3903f,39.6064f,  -8.2205f,10.1147f,37.9152f,  
	-7.6067f,9.2040f,37.5935f,  -54.6627f,30.1507f,41.3664f,  -53.1659f,31.0878f,42.1570f,  -51.9231f,29.8963f,41.1383f,  -50.5091f,30.6683f,41.8144f,  
	-49.3609f,29.6117f,40.5624f,  -48.0411f,30.2334f,41.1295f,  -47.0032f,29.3397f,39.6714f,  -45.7843f,29.8286f,40.1372f,  -44.8662f,29.1240f,38.5054f,  
	-43.7499f,29.4993f,38.8788f,  -42.9544f,29.0079f,37.1103f,  -41.9389f,29.2887f,37.4007f,  -41.2624f,29.0308f,35.5358f,  -40.3384f,29.2370f,35.7549f,  
	-39.7710f,29.2281f,33.8365f,  -38.9259f,29.3776f,33.9953f,  -38.4526f,29.6274f,32.0677f,  -37.6689f,29.7358f,32.1776f,  -37.2705f,30.2474f,30.2851f,  
	-36.5283f,30.3272f,30.3561f,  -36.1822f,31.0967f,28.5422f,  -35.4604f,31.1564f,28.5846f,  -35.1427f,32.1723f,26.8908f,  -34.4182f,32.2164f,26.9139f,  
	-34.1050f,33.4602f,25.3786f,  -33.3575f,33.4896f,25.3893f,  -33.0264f,34.9358f,24.0473f,  -32.2362f,34.9481f,24.0518f,  -31.8673f,36.5651f,22.9333f,  
	-31.0197f,36.5537f,22.9373f,  -30.5974f,38.3048f,22.0671f,  -29.6789f,38.2618f,22.0745f,  -29.1923f,40.1063f,21.4718f,  -28.1961f,40.0226f,21.4852f,  
	-27.6397f,41.9170f,21.1633f,  -26.5641f,41.7813f,21.1850f,  -25.9383f,43.6812f,21.1509f,  -24.7801f,43.4848f,21.1817f,  -24.0908f,45.3455f,21.4361f,  
	-22.8574f,45.0782f,21.4763f,  -22.1160f,46.8569f,22.0137f,  -20.8158f,46.5107f,22.0626f,  -20.0386f,48.1672f,22.8713f,  -18.6815f,47.7358f,22.9273f,  
	-17.8889f,49.2340f,23.9899f,  -16.4878f,48.7138f,24.0514f,  -15.7032f,50.0224f,25.3453f,  -14.2732f,49.4114f,25.4088f,  -13.5221f,50.5048f,26.9063f,  
	-12.0759f,49.8040f,26.9698f,  -11.3849f,50.6625f,28.6394f,  -9.9383f,49.8749f,28.6985f,  -9.3338f,50.4857f,30.5051f,  -7.9004f,49.6163f,30.5563f,  
	-7.4075f,49.9730f,32.4621f,  -6.0001f,49.0290f,32.5014f,  -5.6418f,49.1317f,34.4669f,  -4.2713f,48.1216f,34.4908f,  -4.0674f,47.9767f,36.4756f,  
	-2.7432f,46.9105f,36.4797f,  -2.7096f,46.5300f,38.4434f,  -1.4389f,45.4181f,38.4255f,  -1.5872f,44.8194f,40.3286f,  -0.3741f,43.6721f,40.2856f,  
	-0.7109f,42.8768f,42.0901f,  0.4403f,41.7062f,42.0201f,  -0.0867f,40.7398f,43.6905f,  1.0030f,39.5541f,43.5931f,  0.2891f,38.4448f,45.0971f,  
	1.3172f,37.2548f,44.9712f,  0.4249f,36.0330f,46.2801f,  1.3945f,34.8471f,46.1256f,  0.3370f,33.5442f,47.2149f,  1.2504f,32.3696f,47.0337f,  
	0.0454f,31.0177f,47.8836f,  0.9077f,29.8595f,47.6767f,  -0.4237f,28.4902f,48.2722f,  0.3924f,27.3540f,48.0419f,  -1.0405f,25.9978f,48.3731f,  
	-0.2659f,24.8869f,48.1219f,  -1.7731f,23.5729f,48.1842f,  -1.0335f,22.4880f,47.9146f,  -2.5858f,21.2430f,47.7082f,  -1.8772f,20.1856f,47.4235f,  
	-3.4442f,19.0340f,46.9539f,  -2.7606f,18.0034f,46.6568f,  -4.3112f,16.9665f,45.9340f,  -3.6486f,15.9614f,45.6275f,  -5.1522f,15.0576f,44.6661f,  
	-4.5068f,14.0763f,44.3528f,  -5.9335f,13.3204f,43.1717f,  -5.2996f,12.3593f,42.8542f,  -6.6211f,11.7626f,41.4760f,  -5.9981f,10.8199f,41.1562f,  
	-7.1880f,10.3903f,39.6064f,  -6.5723f,9.4621f,39.2863f,  -7.6067f,9.2040f,37.5935f,  -6.9962f,8.2869f,37.2747f,  -53.1659f,31.0878f,42.1570f,  
	-51.6645f,31.9070f,42.8786f,  -50.5091f,30.6683f,41.8144f,  -49.1008f,31.3303f,42.4241f,  -48.0411f,30.2334f,41.1295f,  -46.7353f,30.7548f,41.6338f,  
	-45.7843f,29.8286f,40.1372f,  -44.5855f,30.2278f,40.5441f,  -43.7499f,29.4993f,38.8788f,  -42.6590f,29.7954f,39.1971f,  -41.9389f,29.2887f,37.4007f,  
	-40.9492f,29.5020f,37.6413f,  -40.3384f,29.2370f,35.7549f,  -39.4395f,29.3864f,35.9288f,  -38.9259f,29.3776f,33.9953f,  -38.1024f,29.4799f,34.1138f,  
	-37.6689f,29.7358f,32.1776f,  -36.9024f,29.8052f,32.2507f,  -36.5283f,30.3272f,30.3561f,  -35.7981f,30.3741f,30.3946f,  -35.4604f,31.1564f,28.5846f,  
	-34.7432f,31.1873f,28.5984f,  -34.4182f,32.2164f,26.9139f,  -33.6931f,32.2342f,26.9103f,  -33.3575f,33.4896f,25.3893f,  -32.6037f,33.4937f,25.3754f,  
	-32.2362f,34.9481f,24.0518f,  -31.4368f,34.9335f,24.0341f,  -31.0197f,36.5537f,22.9373f,  -30.1590f,36.5137f,22.9204f,  -29.6789f,38.2618f,22.0745f,  
	-28.7478f,38.1881f,22.0618f,  -28.1961f,40.0226f,21.4852f,  -27.1898f,39.9042f,21.4799f,  -26.5641f,41.7813f,21.1850f,  -25.4772f,41.6097f,21.1885f,  
	-24.7801f,43.4848f,21.1817f,  -23.6171f,43.2490f,21.1950f,  -22.8574f,45.0782f,21.4763f,  -21.6246f,44.7698f,21.4995f,  -20.8158f,46.5107f,22.0626f,  
	-19.5209f,46.1220f,22.0947f,  -18.6815f,47.7358f,22.9273f,  -17.3359f,47.2622f,22.9672f,  -16.4878f,48.7138f,24.0514f,  -15.1054f,48.1518f,24.0962f,  
	-14.2732f,49.4114f,25.4088f,  -12.8658f,48.7605f,25.4567f,  -12.0759f,49.8040f,26.9698f,  -10.6589f,49.0656f,27.0169f,  -9.9383f,49.8749f,28.6985f,  
	-8.5253f,49.0527f,28.7414f,  -7.9004f,49.6163f,30.5563f,  -6.5045f,48.7162f,30.5910f,  -6.0001f,49.0290f,32.5014f,  -4.6328f,48.0584f,32.5242f,  
	-4.2713f,48.1216f,34.4908f,  -2.9430f,47.0896f,34.4972f,  -2.7432f,46.9105f,36.4797f,  -1.4618f,45.8268f,36.4673f,  -1.4389f,45.4181f,38.4255f,  
	-0.2098f,44.2926f,38.3909f,  -0.3741f,43.6721f,40.2856f,  0.7973f,42.5167f,40.2263f,  0.4403f,41.7062f,42.0201f,  1.5533f,40.5292f,41.9353f,  
	1.0030f,39.5541f,43.5931f,  2.0562f,38.3665f,43.4812f,  1.3172f,37.2548f,44.9712f,  2.3126f,36.0654f,44.8315f,  1.3945f,34.8471f,46.1256f,  
	2.3337f,33.6637f,45.9595f,  1.2504f,32.3696f,47.0337f,  2.1378f,31.1981f,46.8419f,  0.9077f,29.8595f,47.6767f,  1.7474f,28.7062f,47.4609f,  
	0.3924f,27.3540f,48.0419f,  1.1887f,26.2231f,47.8043f,  -0.2659f,24.8869f,48.1219f,  0.4932f,23.7798f,47.8648f,  -1.0335f,22.4880f,47.9146f,  
	-0.3079f,21.4071f,47.6411f,  -1.8772f,20.1856f,47.4235f,  -1.1791f,19.1309f,47.1361f,  -2.7606f,18.0034f,46.6568f,  -2.0859f,16.9743f,46.3584f,  
	-3.6486f,15.9614f,45.6275f,  -2.9935f,14.9569f,45.3207f,  -4.5068f,14.0763f,44.3528f,  -3.8655f,13.0933f,44.0403f,  -5.2996f,12.3593f,42.8542f,  
	-4.6714f,11.3962f,42.5381f,  -5.9981f,10.8199f,41.1562f,  -5.3790f,9.8733f,40.8384f,  -6.5723f,9.4621f,39.2863f,  -5.9598f,8.5287f,38.9687f,  
	-6.9962f,8.2869f,37.2747f,  -6.3889f,7.3636f,36.9584f,  -51.6645f,31.9070f,42.8786f,  -50.1643f,32.6115f,43.5330f,  -49.1008f,31.3303f,42.4241f,  
	-47.7028f,31.8868f,42.9697f,  -46.7353f,30.7548f,41.6338f,  -45.4468f,31.1809f,42.0777f,  -44.5855f,30.2278f,40.5441f,  -43.4102f,30.5422f,40.8942f,  
	-42.6590f,29.7954f,39.1971f,  -41.5931f,30.0186f,39.4636f,  -40.9492f,29.5020f,37.6413f,  -39.9848f,29.6534f,37.8349f,  -39.4395f,29.3864f,35.9288f,  
	-38.5634f,29.4840f,36.0604f,  -38.1024f,29.4799f,34.1138f,  -37.2980f,29.5394f,34.1937f,  -36.9024f,29.8052f,32.2507f,  -36.1498f,29.8388f,32.2899f,  
	-35.7981f,30.3741f,30.3946f,  -35.0744f,30.3904f,30.4032f,  -34.7432f,31.1873f,28.5984f,  -34.0275f,31.1912f,28.5843f,  -33.6931f,32.2342f,26.9103f,  
	-32.9635f,32.2268f,26.8813f,  -32.6037f,33.4937f,25.3754f,  -31.8416f,33.4719f,25.3387f,  -31.4368f,34.9335f,24.0341f,  -30.6245f,34.8918f,23.9951f,  
	-30.1590f,36.5137f,22.9204f,  -29.2852f,36.4449f,22.8831f,  -28.7478f,38.1881f,22.0618f,  -27.8048f,38.0819f,22.0305f,  -27.1898f,39.9042f,21.4799f,  
	-26.1700f,39.7522f,21.4563f,  -25.4772f,41.6097f,21.1885f,  -24.3823f,41.4007f,21.1746f,  -23.6171f,43.2490f,21.1950f,  -22.4511f,42.9742f,21.1917f,  
	-21.6246f,44.7698f,21.4995f,  -20.3928f,44.4202f,21.5064f,  -19.5209f,46.1220f,22.0947f,  -18.2332f,45.6923f,22.1112f,  -17.3359f,47.2622f,22.9672f,  
	-16.0046f,46.7473f,22.9912f,  -15.1054f,48.1518f,24.0962f,  -13.7410f,47.5500f,24.1261f,  -12.8658f,48.7605f,25.4567f,  -11.4833f,48.0716f,25.4890f,  
	-10.6589f,49.0656f,27.0169f,  -9.2716f,48.2920f,27.0484f,  -8.5253f,49.0527f,28.7414f,  -7.1463f,48.1988f,28.7682f,  -6.5045f,48.7162f,30.5910f,  
	-5.1457f,47.7882f,30.6095f,  -4.6328f,48.0584f,32.5242f,  -3.3054f,47.0643f,32.5299f,  -2.9430f,47.0896f,34.4972f,  -1.6555f,46.0384f,34.4874f,  
	-1.4618f,45.8268f,36.4673f,  -0.2209f,44.7278f,36.4383f,  -0.2098f,44.2926f,38.3909f,  0.9781f,43.1573f,38.3397f,  0.7973f,42.5167f,40.2263f,  
	1.9305f,41.3533f,40.1518f,  1.5533f,40.5292f,41.9353f,  2.6293f,39.3489f,41.8354f,  2.0562f,38.3665f,43.4812f,  3.0759f,37.1784f,43.3548f,  
	2.3126f,36.0654f,44.8315f,  3.2768f,34.8778f,44.6794f,  2.3337f,33.6637f,45.9595f,  3.2458f,32.4830f,45.7819f,  2.1378f,31.1981f,46.8419f,  
	3.0012f,30.0314f,46.6400f,  1.7474f,28.7062f,47.4609f,  2.5659f,27.5585f,47.2367f,  1.1887f,26.2231f,47.8043f,  1.9681f,25.0965f,47.5597f,  
	0.4932f,23.7798f,47.8648f,  1.2370f,22.6776f,47.6027f,  -0.3079f,21.4071f,47.6411f,  0.4061f,20.3299f,47.3637f,  -1.1791f,19.1309f,47.1361f,  
	-0.4909f,18.0789f,46.8463f,  -2.0859f,16.9743f,46.3584f,  -1.4197f,15.9471f,46.0587f,  -2.9935f,14.9569f,45.3207f,  -2.3433f,13.9519f,45.0138f,  
	-3.8655f,13.0933f,44.0403f,  -3.2304f,12.1095f,43.7284f,  -4.6714f,11.3962f,42.5381f,  -4.0472f,10.4306f,42.2233f,  -5.3790f,9.8733f,40.8384f,  
	-4.7630f,8.9227f,40.5226f,  -5.9598f,8.5287f,38.9687f,  -5.3504f,7.5902f,38.6532f,  -6.3889f,7.3636f,36.9584f,  -5.7848f,6.4344f,36.6443f,  
	-50.1643f,32.6115f,43.5330f,  -48.6703f,33.2053f,44.1219f,  -47.7028f,31.8868f,42.9697f,  -46.3190f,32.3424f,43.4533f,  -45.4468f,31.1809f,42.0777f,  
	-44.1796f,31.5164f,42.4629f,  -43.4102f,30.5422f,40.8942f,  -42.2589f,30.7778f,41.1906f,  -41.5931f,30.0186f,39.4636f,  -40.5525f,30.1743f,39.6812f,  
	-39.9848f,29.6534f,37.8349f,  -39.0442f,29.7481f,37.9843f,  -38.5634f,29.4840f,36.0604f,  -37.7083f,29.5345f,36.1517f,  -37.2980f,29.5394f,34.1937f,  
	-36.5097f,29.5597f,34.2379f,  -36.1498f,29.8388f,32.2899f,  -35.4063f,29.8395f,32.2975f,  -35.0744f,30.3904f,30.4032f,  -34.3549f,30.3782f,30.3827f,  
	-34.0275f,31.1912f,28.5843f,  -33.3095f,31.1692f,28.5438f,  -32.9635f,32.2268f,26.8813f,  -32.2271f,32.1938f,26.8287f,  -31.8416f,33.4719f,25.3387f,  
	-31.0674f,33.4239f,25.2801f,  -30.6245f,34.8918f,23.9951f,  -29.7993f,34.8229f,23.9353f,  -29.2852f,36.4449f,22.8831f,  -28.3986f,36.3456f,22.8272f,  
	-27.8048f,38.0819f,22.0305f,  -26.8469f,37.9441f,21.9806f,  -26.1700f,39.7522f,21.4563f,  -25.1398f,39.5649f,21.4156f,  -24.3823f,41.4007f,21.1746f,  
	-23.2813f,41.1546f,21.1444f,  -22.4511f,42.9742f,21.1917f,  -21.2822f,42.6597f,21.1725f,  -20.3928f,44.4202f,21.5064f,  -19.1641f,44.0311f,21.4981f,  
	-18.2332f,45.6923f,22.1112f,  -16.9552f,45.2221f,22.1125f,  -16.0046f,46.7473f,22.9912f,  -14.6868f,46.1931f,23.0006f,  -13.7410f,47.5500f,24.1261f,  
	-12.3974f,46.9101f,24.1409f,  -11.4833f,48.0716f,25.4890f,  -10.1265f,47.3470f,25.5062f,  -9.2716f,48.2920f,27.0484f,  -7.9147f,47.4856f,27.0644f,  
	-7.1463f,48.1988f,28.7682f,  -5.8012f,47.3156f,28.7795f,  -5.1457f,47.7882f,30.6095f,  -3.8243f,46.8351f,30.6115f,  -3.3054f,47.0643f,32.5299f,  
	-2.0167f,46.0493f,32.5198f,  -1.6555f,46.0384f,34.4874f,  -0.4073f,44.9702f,34.4613f,  -0.2209f,44.7278f,36.4383f,  0.9796f,43.6172f,36.3928f,  
	0.9781f,43.1573f,38.3397f,  2.1282f,42.0125f,38.2734f,  1.9305f,41.3533f,40.1518f,  3.0267f,40.1853f,40.0621f,  2.6293f,39.3489f,41.8354f,  
	3.6711f,38.1670f,41.7206f,  3.0759f,37.1784f,43.3548f,  4.0634f,35.9912f,43.2152f,  3.2768f,34.8778f,44.6794f,  4.2126f,33.6923f,44.5151f,  
	3.2458f,32.4830f,45.7819f,  4.1325f,31.3069f,45.5934f,  3.0012f,30.0314f,46.6400f,  3.8420f,28.8704f,46.4289f,  2.5659f,27.5585f,47.2367f,  
	3.3661f,26.4154f,47.0045f,  1.9681f,25.0965f,47.5597f,  2.7309f,23.9753f,47.3091f,  1.2370f,22.6776f,47.6027f,  1.9678f,21.5798f,47.3357f,  
	0.4061f,20.3299f,47.3637f,  1.1090f,19.2564f,47.0829f,  -0.4909f,18.0789f,46.8463f,  0.1878f,17.0298f,46.5544f,  -1.4197f,15.9471f,46.0587f,  
	-0.7591f,14.9206f,45.7581f,  -2.3433f,13.9519f,45.0138f,  -1.6998f,12.9474f,44.7068f,  -3.2304f,12.1095f,43.7284f,  -2.5997f,11.1243f,43.4173f,  
	-4.0472f,10.4306f,42.2233f,  -3.4264f,9.4620f,41.9099f,  -4.7630f,8.9227f,40.5226f,  -4.1502f,7.9680f,40.2085f,  -5.3504f,7.5902f,38.6532f,  
	-4.7441f,6.6468f,38.3396f,  -5.7848f,6.4344f,36.6443f,  -5.1837f,5.4996f,36.3320f,  -48.6703f,33.2053f,44.1219f,  -47.1873f,33.6923f,44.6474f,  
	-46.3190f,32.3424f,43.4533f,  -44.9542f,32.7012f,43.8766f,  -44.1796f,31.5164f,42.4629f,  -42.9354f,31.7668f,42.7926f,  -42.2589f,30.7778f,41.1906f,  
	-41.1328f,30.9399f,41.4359f,  -40.5525f,30.1743f,39.6812f,  -39.5365f,30.2676f,39.8524f,  -39.0442f,29.7481f,37.9843f,  -38.1264f,29.7904f,38.0912f,  
	-37.7083f,29.5345f,36.1517f,  -36.8715f,29.5416f,36.2053f,  -36.5097f,29.5597f,34.2379f,  -35.7331f,29.5439f,34.2489f,  -35.4063f,29.8395f,32.2975f,  
	-34.6697f,29.8093f,32.2746f,  -34.3549f,30.3782f,30.3827f,  -33.6357f,30.3388f,30.3344f,  -33.3095f,31.1692f,28.5438f,  -32.5867f,31.1213f,28.4787f,  
	-32.2271f,32.1938f,26.8287f,  -31.4801f,32.1351f,26.7535f,  -31.0674f,33.4239f,25.2801f,  -30.2811f,33.3498f,25.2002f,  -29.7993f,34.8229f,23.9353f,  
	-28.9608f,34.7250f,23.8566f,  -28.3986f,36.3456f,22.8272f,  -27.4961f,36.2164f,22.7524f,  -26.8469f,37.9441f,21.9806f,  -25.8766f,37.7729f,21.9136f,  
	-25.1398f,39.5649f,21.4156f,  -24.1008f,39.3423f,21.3586f,  -23.2813f,41.1546f,21.1444f,  -22.1740f,40.8704f,21.0987f,  -21.2822f,42.6597f,21.1725f,  
	-20.1129f,42.3070f,21.1385f,  -19.1641f,44.0311f,21.4981f,  -17.9412f,43.6023f,21.4752f,  -16.9552f,45.2221f,22.1125f,  -15.6867f,44.7131f,22.0996f,  
	-14.6868f,46.1931f,23.0006f,  -13.3855f,45.6008f,22.9957f,  -12.3974f,46.9101f,24.1409f,  -11.0753f,46.2340f,24.1412f,  -10.1265f,47.3470f,25.5062f,  
	-8.7963f,46.5887f,25.5086f,  -7.9147f,47.4856f,27.0644f,  -6.5882f,46.6487f,27.0654f,  -5.8012f,47.3156f,28.7795f,  -4.4909f,46.4059f,28.7748f,  
	-3.8243f,46.8351f,30.6115f,  -2.5392f,45.8594f,30.5981f,  -2.0167f,46.0493f,32.5198f,  -0.7656f,45.0155f,32.4936f,  -0.4073f,44.9702f,34.4613f,  
	0.8018f,43.8884f,34.4187f,  0.9796f,43.6172f,36.3928f,  2.1431f,42.4953f,36.3321f,  2.1282f,42.0125f,38.2734f,  3.2415f,40.8613f,38.1916f,  
	3.0267f,40.1853f,40.0621f,  4.0881f,39.0142f,39.9570f,  3.6711f,38.1670f,41.7206f,  4.6802f,36.9848f,41.5921f,  4.0634f,35.9912f,43.2152f,  
	5.0217f,34.8053f,43.0629f,  4.2126f,33.6923f,44.5151f,  5.1218f,32.5109f,44.3389f,  4.1325f,31.3069f,45.5934f,  4.9953f,30.1363f,45.3947f,  
	3.8420f,28.8704f,46.4289f,  4.6631f,27.7141f,46.2088f,  3.3661f,26.4154f,47.0045f,  4.1483f,25.2781f,46.7653f,  2.7309f,23.9753f,47.3091f,  
	3.4795f,22.8592f,47.0525f,  1.9678f,21.5798f,47.3357f,  2.6865f,20.4865f,47.0642f,  1.1090f,19.2564f,47.0829f,  1.8015f,18.1866f,46.7992f,  
	0.1878f,17.0298f,46.5544f,  0.8601f,15.9826f,46.2605f,  -0.7591f,14.9206f,45.7581f,  -0.1058f,13.8956f,45.4566f,  -1.6998f,12.9474f,44.7068f,  
	-1.0612f,11.9427f,44.3999f,  -2.5997f,11.1243f,43.4173f,  -1.9726f,10.1374f,43.1068f,  -3.4264f,9.4620f,41.9099f,  -2.8088f,8.4904f,41.5976f,  
	-4.1502f,7.9680f,40.2085f,  -3.5404f,7.0094f,39.8957f,  -4.7441f,6.6468f,38.3396f,  -4.1406f,5.6987f,38.0274f,  -5.1837f,5.4996f,36.3320f,  
	-4.5849f,4.5587f,36.0214f,  -47.1873f,33.6923f,44.6474f,  -45.7207f,34.0763f,45.1108f,  -44.9542f,32.7012f,43.8766f,  -43.6108f,32.9685f,44.2423f,  
	-42.9354f,31.7668f,42.7926f,  -41.7157f,31.9373f,43.0692f,  -41.1328f,30.9399f,41.4359f,  -40.0318f,31.0336f,41.6329f,  -39.5365f,30.2676f,39.8524f,  
	-38.5450f,30.3029f,39.9791f,  -38.1264f,29.7904f,38.0912f,  -37.2291f,29.7847f,38.1586f,  -36.8715f,29.5416f,36.2053f,  -36.0493f,29.5086f,36.2240f,  
	-35.7331f,29.5439f,34.2489f,  -34.9666f,29.4943f,34.2277f,  -34.6697f,29.8093f,32.2746f,  -33.9366f,29.7500f,32.2225f,  -33.6357f,30.3388f,30.3344f,  
	-32.9143f,30.2726f,30.2604f,  -32.5867f,31.1213f,28.4787f,  -31.8556f,31.0476f,28.3901f,  -31.4801f,32.1351f,26.7535f,  -30.7225f,32.0508f,26.6562f,  
	-30.2811f,33.3498f,25.2002f,  -29.4821f,33.2478f,25.1009f,  -28.9608f,34.7250f,23.8566f,  -28.1066f,34.5986f,23.7585f,  -27.4961f,36.2164f,22.7524f,  
	-26.5802f,36.0555f,22.6603f,  -25.8766f,37.7729f,21.9136f,  -24.8957f,37.5682f,21.8304f,  -24.1008f,39.3423f,21.3586f,  -23.0528f,39.0831f,21.2863f,  
	-22.1740f,40.8704f,21.0987f,  -21.0632f,40.5494f,21.0383f,  -20.1129f,42.3070f,21.1385f,  -18.9456f,41.9156f,21.0902f,  -17.9412f,43.6023f,21.4752f,  
	-16.7239f,43.1354f,21.4385f,  -15.6867f,44.7131f,22.0996f,  -14.4308f,44.1662f,22.0730f,  -13.3855f,45.6008f,22.9957f,  -12.1019f,44.9721f,22.9769f,  
	-11.0753f,46.2340f,24.1412f,  -9.7762f,45.5236f,24.1275f,  -8.7963f,46.5887f,25.5086f,  -7.4932f,45.7990f,25.4967f,  -6.5882f,46.6487f,27.0654f,  
	-5.2937f,45.7839f,27.0512f,  -4.4909f,46.4059f,28.7748f,  -3.2142f,45.4718f,28.7552f,  -2.5392f,45.8594f,30.5981f,  -1.2899f,44.8632f,30.5690f,  
	-0.7656f,45.0155f,32.4936f,  0.4477f,43.9663f,32.4513f,  0.8018f,43.8884f,34.4187f,  1.9748f,42.7937f,34.3611f,  2.1431f,42.4953f,36.3321f,  
	3.2700f,41.3655f,36.2558f,  3.2415f,40.8613f,38.1916f,  4.3198f,39.7057f,38.0941f,  4.0881f,39.0142f,39.9570f,  5.1165f,37.8416f,39.8378f,  
	4.6802f,36.9848f,41.5921f,  5.6592f,35.8029f,41.4503f,  5.0217f,34.8053f,43.0629f,  5.9523f,33.6228f,42.8980f,  5.1218f,32.5109f,44.3389f,  
	6.0060f,31.3345f,44.1517f,  4.9953f,30.1363f,45.3947f,  5.8371f,28.9702f,45.1860f,  4.6631f,27.7141f,46.2088f,  5.4651f,26.5636f,45.9806f,  
	4.1483f,25.2781f,46.7653f,  4.9151f,24.1462f,46.5190f,  3.4795f,22.8592f,47.0525f,  4.2147f,21.7482f,46.7905f,  2.6865f,20.4865f,47.0642f,  
	3.3937f,19.3977f,46.7888f,  1.8015f,18.1866f,46.7992f,  2.4865f,17.1199f,46.5123f,  0.8601f,15.9826f,46.2605f,  1.5243f,14.9380f,45.9649f,  
	-0.1058f,13.8956f,45.4566f,  0.5418f,12.8717f,45.1543f,  -1.0612f,11.9427f,44.3999f,  -0.4267f,10.9374f,44.0928f,  -1.9726f,10.1374f,43.1068f,  
	-1.3491f,9.1487f,42.7968f,  -2.8088f,8.4904f,41.5976f,  -2.1943f,7.5162f,41.2862f,  -3.5404f,7.0094f,39.8957f,  -2.9335f,6.0472f,39.5842f,  
	-4.1406f,5.6987f,38.0274f,  -3.5393f,4.7453f,37.7166f,  -4.5849f,4.5587f,36.0214f,  -3.9888f,3.6125f,35.7118f,  -45.7207f,34.0763f,45.1108f,  
	-44.2737f,34.3624f,45.5149f,  -43.6108f,32.9685f,44.2423f,  -42.2914f,33.1494f,44.5530f,  -41.7157f,31.9373f,43.0692f,  -40.5218f,32.0330f,43.2952f,  
	-40.0318f,31.0336f,41.6329f,  -38.9570f,31.0632f,41.7831f,  -38.5450f,30.3029f,39.9791f,  -37.5764f,30.2849f,40.0642f,  -37.2291f,29.7847f,38.1586f,  
	-36.3494f,29.7344f,38.1889f,  -36.0493f,29.5086f,36.2240f,  -35.2404f,29.4384f,36.2086f,  -34.9666f,29.4943f,34.2277f,  -34.2069f,29.4128f,34.1757f,  
	-33.9366f,29.7500f,32.2225f,  -33.2043f,29.6623f,32.1434f,  -32.9143f,30.2726f,30.2604f,  -32.1873f,30.1798f,30.1618f,  -31.8556f,31.0476f,28.3901f,  
	-31.1160f,30.9482f,28.2784f,  -30.7225f,32.0508f,26.6562f,  -29.9533f,31.9393f,26.5390f,  -29.4821f,33.2478f,25.1009f,  -28.6679f,33.1184f,24.9816f,  
	-28.1066f,34.5986f,23.7585f,  -27.2385f,34.4420f,23.6428f,  -26.5802f,36.0555f,22.6603f,  -25.6524f,35.8626f,22.5519f,  -24.8957f,37.5682f,21.8304f,  
	-23.9037f,37.3283f,21.7320f,  -23.0528f,39.0831f,21.2863f,  -21.9988f,38.7885f,21.1994f,  -21.0632f,40.5494f,21.0383f,  -19.9512f,40.1907f,20.9641f,  
	-18.9456f,41.9156f,21.0902f,  -17.7804f,41.4868f,21.0285f,  -16.7239f,43.1354f,21.4385f,  -15.5153f,42.6308f,21.3885f,  -14.4308f,44.1662f,22.0730f,  
	-13.1887f,43.5827f,22.0331f,  -12.1019f,44.9721f,22.9769f,  -10.8375f,44.3085f,22.9447f,  -9.7762f,45.5236f,24.1275f,  -8.5006f,44.7808f,24.1000f,  
	-7.4932f,45.7990f,25.4967f,  -6.2189f,44.9800f,25.4702f,  -5.2937f,45.7839f,27.0512f,  -4.0299f,44.8933f,27.0226f,  -3.2142f,45.4718f,28.7552f,  
	-1.9711f,44.5157f,28.7204f,  -1.2899f,44.8632f,30.5690f,  -0.0765f,43.8495f,30.5242f,  0.4477f,43.9663f,32.4513f,  1.6262f,42.9024f,32.3942f,  
	1.9748f,42.7937f,34.3611f,  3.1119f,41.6892f,34.2880f,  3.2700f,41.3655f,36.2558f,  4.3622f,40.2295f,36.1638f,  4.3198f,39.7057f,38.0941f,  
	5.3649f,38.5470f,37.9823f,  5.1165f,37.8416f,39.8378f,  6.1143f,36.6681f,39.7049f,  5.6592f,35.8029f,41.4503f,  6.6097f,34.6234f,41.2953f,  
	5.9523f,33.6228f,42.8980f,  6.8567f,32.4445f,42.7213f,  6.0060f,31.3345f,44.1517f,  6.8679f,30.1623f,43.9538f,  5.8371f,28.9702f,45.1860f,  
	6.6585f,27.8099f,44.9684f,  5.4651f,26.5636f,45.9806f,  6.2502f,25.4189f,45.7444f,  4.9151f,24.1462f,46.5190f,  5.6673f,23.0200f,46.2664f,  
	4.2147f,21.7482f,46.7905f,  4.9373f,20.6424f,46.5236f,  3.3937f,19.3977f,46.7888f,  4.0924f,18.3129f,46.5093f,  2.4865f,17.1199f,46.5123f,  
	3.1626f,16.0567f,46.2230f,  1.5243f,14.9380f,45.9649f,  2.1821f,13.8955f,45.6676f,  0.5418f,12.8717f,45.1543f,  1.1849f,11.8483f,44.8512f,  
	-0.4267f,10.9374f,44.0928f,  0.2039f,9.9314f,43.7857f,  -1.3491f,9.1487f,42.7968f,  -0.7289f,8.1583f,42.4872f,  -2.1943f,7.5162f,41.2862f,  
	-1.5829f,6.5392f,40.9756f,  -2.9335f,6.0472f,39.5842f,  -2.3285f,5.0806f,39.2735f,  -3.5393f,4.7453f,37.7166f,  -2.9404f,3.7875f,37.4066f,  
	-3.9888f,3.6125f,35.7118f,  -3.3947f,2.6606f,35.4031f,  -44.2737f,34.3624f,45.5149f,  -42.8499f,34.5554f,45.8619f,  -42.2914f,33.1494f,44.5530f,  
	-40.9979f,33.2488f,44.8109f,  -40.5218f,32.0330f,43.2952f,  -39.3555f,32.0582f,43.4722f,  -38.9570f,31.0632f,41.7831f,  -37.9073f,31.0337f,41.8895f,  
	-37.5764f,30.2849f,40.0642f,  -36.6283f,30.2173f,40.1102f,  -36.3494f,29.7344f,38.1889f,  -35.4864f,29.6426f,38.1834f,  -35.2404f,29.4384f,36.2086f,  
	-34.4422f,29.3329f,36.1608f,  -34.2069f,29.4128f,34.1757f,  -33.4516f,29.3007f,34.0954f,  -33.2043f,29.6623f,32.1434f,  -32.4697f,29.5466f,32.0383f,  
	-32.1873f,30.1798f,30.1618f,  -31.4547f,30.0608f,30.0390f,  -31.1160f,30.9482f,28.2784f,  -30.3665f,30.8220f,28.1461f,  -29.9533f,31.9393f,26.5390f,  
	-29.1705f,31.8012f,26.4010f,  -28.6679f,33.1184f,24.9816f,  -27.8402f,32.9600f,24.8444f,  -27.2385f,34.4420f,23.6428f,  -26.3580f,34.2547f,23.5107f,  
	-25.6524f,35.8626f,22.5519f,  -24.7121f,35.6360f,22.4282f,  -23.9037f,37.3283f,21.7320f,  -22.9037f,37.0544f,21.6190f,  -21.9988f,38.7885f,21.1994f,  
	-20.9408f,38.4574f,21.0990f,  -19.9512f,40.1907f,20.9641f,  -18.8382f,39.7956f,20.8767f,  -17.7804f,41.4868f,21.0285f,  -16.6205f,41.0208f,20.9541f,  
	-15.5153f,42.6308f,21.3885f,  -14.3170f,42.0898f,21.3259f,  -13.1887f,43.5827f,22.0331f,  -11.9622f,42.9640f,21.9804f,  -10.8375f,44.3085f,22.9447f,  
	-9.5933f,43.6117f,22.8995f,  -8.5006f,44.7808f,24.1000f,  -7.2508f,44.0077f,24.0588f,  -6.2189f,44.9800f,25.4702f,  -4.9725f,44.1336f,25.4299f,  
	-4.0299f,44.8933f,27.0226f,  -2.7976f,43.9790f,26.9795f,  -1.9711f,44.5157f,28.7204f,  -0.7621f,43.5402f,28.6704f,  -0.0765f,43.8495f,30.5242f,  
	1.1034f,42.8195f,30.4650f,  1.6262f,42.9024f,32.3942f,  2.7698f,41.8268f,32.3217f,  3.1119f,41.6892f,34.2880f,  4.2145f,40.5768f,34.1992f,  
	4.3622f,40.2295f,36.1638f,  5.4212f,39.0888f,36.0574f,  5.3649f,38.5470f,37.9823f,  6.3791f,37.3861f,37.8565f,  6.1143f,36.6681f,39.7049f,  
	7.0827f,35.4958f,39.5583f,  6.6097f,34.6234f,41.2953f,  7.5331f,33.4474f,41.1279f,  6.8567f,32.4445f,42.7213f,  7.7379f,31.2700f,42.5331f,  
	6.8679f,30.1623f,43.9538f,  7.7083f,28.9956f,43.7461f,  6.6585f,27.8099f,44.9684f,  7.4618f,26.6554f,44.7419f,  6.2502f,25.4189f,45.7444f,  
	7.0195f,24.2800f,45.5010f,  5.6673f,23.0200f,46.2664f,  6.4057f,21.8994f,46.0080f,  4.9373f,20.6424f,46.5236f,  5.6501f,19.5414f,46.2516f,  
	4.0924f,18.3129f,46.5093f,  4.7811f,17.2324f,46.2265f,  3.1626f,16.0567f,46.2230f,  3.8315f,14.9965f,45.9311f,  2.1821f,13.8955f,45.6676f,  
	2.8346f,12.8546f,45.3688f,  1.1849f,11.8483f,44.8512f,  1.8235f,10.8253f,44.5471f,  0.2039f,9.9314f,43.7857f,  0.8307f,8.9248f,43.4782f,  
	-0.7289f,8.1583f,42.4872f,  -0.1121f,7.1662f,42.1779f,  -1.5829f,6.5392f,40.9756f,  -0.9735f,5.5588f,40.6654f,  -2.3285f,5.0806f,39.2735f,  
	-1.7261f,4.1103f,38.9633f,  -2.9404f,3.7875f,37.4066f,  -2.3435f,2.8247f,37.0972f,  -3.3947f,2.6606f,35.4031f,  -2.8025f,1.7033f,35.0947f,  
	-42.8499f,34.5554f,45.8619f,  -41.4522f,34.6600f,46.1540f,  -40.9979f,33.2488f,44.8109f,  -39.7334f,33.2710f,45.0177f,  -39.3555f,32.0582f,43.4722f,  
	-38.2163f,32.0181f,43.6034f,  -37.9073f,31.0337f,41.8895f,  -36.8812f,30.9491f,41.9548f,  -36.6283f,30.2173f,40.1102f,  -35.7006f,30.1035f,40.1184f,  
	-35.4864f,29.6426f,38.1834f,  -34.6383f,29.5118f,38.1436f,  -34.4422f,29.3329f,36.1608f,  -33.6522f,29.1941f,36.0830f,  -33.4516f,29.3007f,34.0954f,  
	-32.6979f,29.1588f,33.9876f,  -32.4697f,29.5466f,32.0383f,  -31.7329f,29.4038f,31.9080f,  -31.4547f,30.0608f,30.0390f,  -30.7146f,29.9148f,29.8948f,  
	-30.3665f,30.8220f,28.1461f,  -29.6057f,30.6694f,27.9921f,  -29.1705f,31.8012f,26.4010f,  -28.3751f,31.6349f,26.2445f,  -27.8402f,32.9600f,24.8444f,  
	-27.0003f,32.7722f,24.6903f,  -26.3580f,34.2547f,23.5107f,  -25.4641f,34.0351f,23.3630f,  -24.7121f,35.6360f,22.4282f,  -23.7624f,35.3767f,22.2899f,  
	-22.9037f,37.0544f,21.6190f,  -21.8975f,36.7451f,21.4927f,  -20.9408f,38.4574f,21.0990f,  -19.8792f,38.0908f,20.9857f,  -18.8382f,39.7956f,20.8767f,  
	-17.7275f,39.3639f,20.7771f,  -16.6205f,41.0208f,20.9541f,  -15.4676f,40.5187f,20.8676f,  -14.3170f,42.0898f,21.3259f,  -13.1309f,41.5134f,21.2512f,  
	-11.9622f,42.9640f,21.9804f,  -10.7525f,42.3117f,21.9154f,  -9.5933f,43.6117f,22.8995f,  -8.3717f,42.8838f,22.8412f,  -7.2508f,44.0077f,24.0588f,  
	-6.0259f,43.2059f,24.0045f,  -4.9725f,44.1336f,25.4299f,  -3.7552f,43.2623f,25.3759f,  -2.7976f,43.9790f,26.9795f,  -1.5971f,43.0435f,26.9217f,  
	-0.7621f,43.5402f,28.6704f,  0.4152f,42.5466f,28.6065f,  1.1034f,42.8195f,30.4650f,  2.2495f,41.7759f,30.3907f,  2.7698f,41.8268f,32.3217f,  
	3.8794f,40.7415f,32.2339f,  4.2145f,40.5768f,34.1992f,  5.2844f,39.4579f,34.0960f,  5.4212f,39.0888f,36.0574f,  6.4491f,37.9445f,35.9368f,  
	6.3791f,37.3861f,37.8565f,  7.3634f,36.2250f,37.7166f,  7.0827f,35.4958f,39.5583f,  8.0234f,34.3260f,39.3988f,  7.5331f,33.4474f,41.1279f,  
	8.4322f,32.2744f,40.9484f,  7.7379f,31.2700f,42.5331f,  8.5964f,30.1005f,42.3345f,  7.7083f,28.9956f,43.7461f,  8.5294f,27.8345f,43.5286f,  
	7.4618f,26.6554f,44.7419f,  8.2482f,25.5068f,44.5073f,  7.0195f,24.2800f,45.5010f,  7.7739f,23.1471f,45.2508f,  6.4057f,21.8994f,46.0080f,  
	7.1332f,20.7841f,45.7434f,  5.6501f,19.5414f,46.2516f,  6.3522f,18.4453f,45.9753f,  4.7811f,17.2324f,46.2265f,  5.4618f,16.1559f,45.9403f,  
	3.8315f,14.9965f,45.9311f,  4.4943f,13.9389f,45.6368f,  2.8346f,12.8546f,45.3688f,  3.4821f,11.8150f,45.0683f,  1.8235f,10.8253f,44.5471f,  
	2.4579f,9.8027f,44.2422f,  0.8307f,8.9248f,43.4782f,  1.4539f,7.9176f,43.1705f,  -0.1121f,7.1662f,42.1779f,  0.5027f,6.1716f,41.8684f,  
	-0.9735f,5.5588f,40.6654f,  -0.3666f,4.5756f,40.3553f,  -1.7261f,4.1103f,38.9633f,  -1.1255f,3.1361f,38.6534f,  -2.3435f,2.8247f,37.0972f,  
	-1.7482f,1.8571f,36.7879f,  -2.8025f,1.7033f,35.0947f,  -2.2121f,0.7407f,34.7863f,  -41.4522f,34.6600f,46.1540f,  -40.0844f,34.6807f,46.3930f,  
	-39.7334f,33.2710f,45.0177f,  -38.4981f,33.2215f,45.1765f,  -38.2163f,32.0181f,43.6034f,  -37.1038f,31.9172f,43.6913f,  -36.8812f,30.9491f,41.9548f,  
	-35.8791f,30.8131f,41.9803f,  -35.7006f,30.1035f,40.1184f,  -34.7919f,29.9463f,40.0903f,  -34.6383f,29.5118f,38.1436f,  -33.8025f,29.3444f,38.0722f,  
	-33.6522f,29.1941f,36.0830f,  -32.8681f,29.0232f,35.9763f,  -32.6979f,29.1588f,33.9876f,  -31.9458f,28.9884f,33.8533f,  -31.7329f,29.4038f,31.9080f,  
	-30.9917f,29.2334f,31.7552f,  -30.7146f,29.9148f,29.8948f,  -29.9660f,29.7425f,29.7280f,  -29.6057f,30.6694f,27.9921f,  -28.8339f,30.4894f,27.8190f,  
	-28.3751f,31.6349f,26.2445f,  -27.5683f,31.4402f,26.0706f,  -27.0003f,32.7722f,24.6903f,  -26.1469f,32.5532f,24.5204f,  -25.4641f,34.0351f,23.3630f,  
	-24.5602f,33.7842f,23.2006f,  -23.7624f,35.3767f,22.2899f,  -22.8049f,35.0832f,22.1384f,  -21.8975f,36.7451f,21.4927f,  -20.8857f,36.4014f,21.3536f,  
	-19.8792f,38.0908f,20.9857f,  -18.8173f,37.6884f,20.8605f,  -17.7275f,39.3639f,20.7771f,  -16.6208f,38.8965f,20.6658f,  -15.4676f,40.5187f,20.8676f,  
	-14.3238f,39.9813f,20.7695f,  -13.1309f,41.5134f,21.2512f,  -11.9585f,40.9030f,21.1648f,  -10.7525f,42.3117f,21.9154f,  -9.5624f,41.6274f,21.8383f,  
	-8.3717f,42.8838f,22.8412f,  -7.1720f,42.1260f,22.7706f,  -6.0259f,43.2059f,24.0045f,  -4.8275f,42.3779f,23.9371f,  -3.7552f,43.2623f,25.3759f,  
	-2.5673f,42.3679f,25.3079f,  -1.5971f,43.0435f,26.9217f,  -0.4266f,42.0883f,26.8505f,  0.4152f,42.5466f,28.6065f,  1.5600f,41.5376f,28.5280f,  
	2.2495f,41.7759f,30.3907f,  3.3625f,40.7208f,30.3014f,  3.8794f,40.7415f,32.2339f,  4.9568f,39.6479f,32.1317f,  5.2844f,39.4579f,34.0960f,  
	6.3231f,38.3338f,33.9785f,  6.4491f,37.9445f,35.9368f,  7.4468f,36.7986f,35.8019f,  7.3634f,36.2250f,37.7166f,  8.3193f,35.0651f,37.5635f,  
	8.0234f,34.3260f,39.3988f,  8.9389f,33.1582f,39.2267f,  8.4322f,32.2744f,40.9484f,  9.3078f,31.1056f,40.7578f,  8.5964f,30.1005f,42.3345f,  
	9.4345f,28.9362f,42.1254f,  8.5294f,27.8345f,43.5286f,  9.3323f,26.6793f,43.3023f,  8.2482f,25.5068f,44.5073f,  9.0185f,24.3642f,44.2651f,  
	7.7739f,23.1471f,45.2508f,  8.5161f,22.0200f,44.9936f,  7.1332f,20.7841f,45.7434f,  7.8488f,19.6743f,45.4738f,  6.3522f,18.4453f,45.9753f,  
	7.0451f,17.3538f,45.6949f,  5.4618f,16.1559f,45.9403f,  6.1355f,15.0827f,45.6507f,  4.4943f,13.9389f,45.6368f,  5.1514f,12.8836f,45.3401f,  
	3.4821f,11.8150f,45.0683f,  4.1248f,10.7768f,44.7662f,  2.4579f,9.8027f,44.2422f,  3.0883f,8.7804f,43.9364f,  1.4539f,7.9176f,43.1705f,  
	2.0748f,6.9087f,42.8620f,  0.5027f,6.1716f,41.8684f,  1.1147f,5.1752f,41.5587f,  -0.3666f,4.5756f,40.3553f,  0.2383f,3.5892f,40.0451f,  
	-1.1255f,3.1361f,38.6534f,  -0.5265f,2.1575f,38.3433f,  -1.7482f,1.8571f,36.7879f,  -1.1546f,0.8849f,36.4783f,  -2.2121f,0.7407f,34.7863f,  
	-1.6231f,-0.2273f,34.4773f,  -40.0844f,34.6807f,46.3930f,  -38.7477f,34.6232f,46.5819f,  -38.4981f,33.2215f,45.1765f,  -37.2923f,33.1051f,45.2899f,  
	-37.1038f,31.9172f,43.6913f,  -36.0190f,31.7593f,43.7373f,  -35.8791f,30.8131f,41.9803f,  -34.9003f,30.6291f,41.9676f,  -34.7919f,29.9463f,40.0903f,  
	-33.8999f,29.7488f,40.0290f,  -33.8025f,29.3444f,38.0722f,  -32.9772f,29.1420f,37.9703f,  -32.8681f,29.0232f,35.9763f,  -32.0898f,28.8218f,35.8417f,  
	-31.9458f,28.9884f,33.8533f,  -31.1929f,28.7894f,33.6954f,  -30.9917f,29.2334f,31.7552f,  -30.2454f,29.0363f,31.5787f,  -29.9660f,29.7425f,29.7280f,  
	-29.2090f,29.5430f,29.5412f,  -28.8339f,30.4894f,27.8190f,  -28.0524f,30.2817f,27.6281f,  -27.5683f,31.4402f,26.0706f,  -26.7489f,31.2154f,25.8807f,  
	-26.1469f,32.5532f,24.5204f,  -25.2835f,32.3040f,24.3356f,  -24.5602f,33.7842f,23.2006f,  -23.6475f,33.5002f,23.0251f,  -22.8049f,35.0832f,22.1384f,  
	-21.8403f,34.7565f,21.9742f,  -20.8857f,36.4014f,21.3536f,  -19.8713f,36.0229f,21.2030f,  -18.8173f,37.6884f,20.8605f,  -17.7568f,37.2509f,20.7241f,  
	-16.6208f,38.8965f,20.6658f,  -15.5202f,38.3940f,20.5436f,  -14.3238f,39.9813f,20.7695f,  -13.1905f,39.4098f,20.6605f,  -11.9585f,40.9030f,21.1648f,  
	-10.8025f,40.2601f,21.0671f,  -9.5624f,41.6274f,21.8383f,  -8.3912f,40.9124f,21.7495f,  -7.1720f,42.1260f,22.7706f,  -5.9962f,41.3408f,22.6879f,  
	-4.8275f,42.3779f,23.9371f,  -3.6562f,41.5251f,23.8566f,  -2.5673f,42.3679f,25.3079f,  -1.4075f,41.4524f,25.2271f,  -0.4266f,42.0883f,26.8505f,  
	0.7130f,41.1157f,26.7653f,  1.5600f,41.5376f,28.5280f,  2.6727f,40.5151f,28.4348f,  3.3625f,40.7208f,30.3014f,  4.4440f,39.6555f,30.1980f,  
	4.9568f,39.6479f,32.1317f,  6.0034f,38.5474f,32.0153f,  6.3231f,38.3338f,33.9785f,  7.3316f,37.2066f,33.8468f,  7.4468f,36.7986f,35.8019f,  
	8.4158f,35.6523f,35.6535f,  8.3193f,35.0651f,37.5635f,  9.2494f,33.9063f,37.3975f,  8.9389f,33.1582f,39.2267f,  9.8301f,31.9938f,39.0430f,  
	9.3078f,31.1056f,40.7578f,  10.1618f,29.9415f,40.5561f,  9.4345f,28.9362f,42.1254f,  10.2533f,27.7774f,41.9066f,  9.3323f,26.6793f,43.3023f,  
	10.1182f,25.5300f,43.0676f,  9.0185f,24.3642f,44.2651f,  9.7753f,23.2278f,44.0149f,  8.5161f,22.0200f,44.9936f,  9.2455f,20.8987f,44.7305f,  
	7.8488f,19.6743f,45.4738f,  8.5543f,18.5698f,45.1992f,  7.0451f,17.3538f,45.6949f,  7.7301f,16.2665f,45.4102f,  6.1355f,15.0827f,45.6507f,  
	6.8027f,14.0127f,45.3580f,  5.1514f,12.8836f,45.3401f,  5.8030f,11.8305f,45.0411f,  4.1248f,10.7768f,44.7662f,  4.7630f,9.7399f,44.4625f,  
	3.0883f,8.7804f,43.9364f,  3.7159f,7.7575f,43.6293f,  2.0748f,6.9087f,42.8620f,  2.6926f,5.8990f,42.5529f,  1.1147f,5.1752f,41.5587f,  
	1.7247f,4.1765f,41.2484f,  0.2383f,3.5892f,40.0451f,  0.8417f,2.5994f,39.7344f,  -0.5265f,2.1575f,38.3433f,  0.0709f,1.1751f,38.0327f,  
	-1.1546f,0.8849f,36.4783f,  -0.5622f,-0.0922f,36.1680f,  -1.6231f,-0.2273f,34.4773f,  -1.0357f,-1.2002f,34.1672f,  -38.7477f,34.6232f,46.5819f,  
	-37.4434f,34.4924f,46.7232f,  -37.2923f,33.1051f,45.2899f,  -36.1178f,32.9260f,45.3594f,  -36.0190f,31.7593f,43.7373f,  -34.9618f,31.5484f,43.7433f,  
	-34.9003f,30.6291f,41.9676f,  -33.9429f,30.4006f,41.9198f,  -33.8999f,29.7488f,40.0290f,  -33.0234f,29.5132f,39.9356f,  -32.9772f,29.1420f,37.9703f,  
	-32.1624f,28.9067f,37.8391f,  -32.0898f,28.8218f,35.8417f,  -31.3149f,28.5903f,35.6823f,  -31.1929f,28.7894f,33.6954f,  -30.4390f,28.5629f,33.5128f,  
	-30.2454f,29.0363f,31.5787f,  -29.4937f,28.8120f,31.3815f,  -29.2090f,29.5430f,29.5412f,  -28.4443f,29.3164f,29.3359f,  -28.0524f,30.2817f,27.6281f,  
	-27.2596f,30.0448f,27.4205f,  -26.7489f,31.2154f,25.8807f,  -25.9199f,30.9616f,25.6754f,  -25.2835f,32.3040f,24.3356f,  -24.4109f,32.0231f,24.1376f,  
	-23.6475f,33.5002f,23.0251f,  -22.7268f,33.1842f,22.8368f,  -21.8403f,34.7565f,21.9742f,  -20.8715f,34.3959f,21.7986f,  -19.8713f,36.0229f,21.2030f,  
	-18.8563f,35.6100f,21.0415f,  -17.7568f,37.2509f,20.7241f,  -16.6999f,36.7787f,20.5773f,  -15.5202f,38.3940f,20.5436f,  -14.4275f,37.8575f,20.4110f,  
	-13.1905f,39.4098f,20.6605f,  -12.0709f,38.8053f,20.5410f,  -10.8025f,40.2601f,21.0671f,  -9.6627f,39.5858f,20.9586f,  -8.3912f,40.9124f,21.7495f,  
	-7.2413f,40.1689f,21.6494f,  -5.9962f,41.3408f,22.6879f,  -4.8450f,40.5295f,22.5928f,  -3.6562f,41.5251f,23.8566f,  -2.5109f,40.6498f,23.7640f,  
	-1.4075f,41.4524f,25.2271f,  -0.2769f,40.5177f,25.1330f,  0.7130f,41.1157f,26.7653f,  1.8218f,40.1280f,26.6662f,  2.6727f,40.5151f,28.4348f,  
	3.7549f,39.4807f,28.3280f,  4.4440f,39.6555f,30.1980f,  5.4952f,38.5816f,30.0807f,  6.0034f,38.5474f,32.0153f,  7.0197f,37.4421f,31.8848f,  
	7.3316f,37.2066f,33.8468f,  8.3111f,36.0775f,33.7015f,  8.4158f,35.6523f,35.6535f,  9.3583f,34.5060f,35.4921f,  9.2494f,33.9063f,37.3975f,  
	10.1544f,32.7498f,37.2195f,  9.8301f,31.9938f,39.0430f,  10.6987f,30.8333f,38.8478f,  10.1618f,29.9415f,40.5561f,  10.9955f,28.7825f,40.3442f,  
	10.2533f,27.7774f,41.9066f,  11.0541f,26.6243f,41.6789f,  10.1182f,25.5300f,43.0676f,  10.8892f,24.3868f,42.8241f,  9.7753f,23.2278f,44.0149f,  
	10.5182f,22.0973f,43.7582f,  9.2455f,20.8987f,44.7305f,  9.9636f,19.7831f,44.4616f,  8.5543f,18.5698f,45.1992f,  9.2509f,17.4701f,44.9195f,  
	7.7301f,16.2665f,45.4102f,  8.4078f,15.1832f,45.1216f,  6.8027f,14.0127f,45.3580f,  7.4636f,12.9458f,45.0623f,  5.8030f,11.8305f,45.0411f,  
	6.4494f,10.7797f,44.7399f,  4.7630f,9.7399f,44.4625f,  5.3979f,8.7033f,44.1570f,  3.7159f,7.7575f,43.6293f,  4.3401f,6.7345f,43.3210f,  
	2.6926f,5.8990f,42.5529f,  3.3081f,4.8879f,42.2428f,  1.7247f,4.1765f,41.2484f,  2.3329f,3.1751f,40.9372f,  0.8417f,2.5994f,39.7344f,  
	1.4434f,1.6063f,39.4228f,  0.0709f,1.1751f,38.0327f,  0.6671f,0.1883f,37.7210f,  -0.5622f,-0.0922f,36.1680f,  0.0287f,-1.0737f,35.8564f,  
	-1.0357f,-1.2002f,34.1672f,  -0.4493f,-2.1782f,33.8557f,  -37.4434f,34.4924f,46.7232f,  -36.1739f,34.2931f,46.8188f,  -36.1178f,32.9260f,45.3594f,  
	-34.9753f,32.6885f,45.3871f,  -34.9618f,31.5484f,43.7433f,  -33.9307f,31.2886f,43.7125f,  -33.9429f,30.4006f,41.9198f,  -33.0060f,30.1304f,41.8384f,  
	-33.0234f,29.5132f,39.9356f,  -32.1624f,29.2418f,39.8113f,  -32.1624f,28.9067f,37.8391f,  -31.3555f,28.6397f,37.6818f,  -31.3149f,28.5903f,35.6823f,  
	-30.5436f,28.3303f,35.4970f,  -30.4390f,28.5629f,33.5128f,  -29.6833f,28.3089f,33.3084f,  -29.4937f,28.8120f,31.3815f,  -28.7372f,28.5608f,31.1650f,  
	-28.4443f,29.3164f,29.3359f,  -27.6703f,29.0613f,29.1134f,  -27.2596f,30.0448f,27.4205f,  -26.4585f,29.7799f,27.1971f,  -25.9199f,30.9616f,25.6754f,  
	-25.0819f,30.6772f,25.4568f,  -24.4109f,32.0231f,24.1376f,  -23.5300f,31.7113f,23.9266f,  -22.7268f,33.1842f,22.8368f,  -21.8007f,32.8353f,22.6372f,  
	-20.8715f,34.3959f,21.7986f,  -19.9003f,34.0019f,21.6125f,  -18.8563f,35.6100f,21.0415f,  -17.8426f,35.1629f,20.8701f,  -16.6999f,36.7787f,20.5773f,  
	-15.6484f,36.2727f,20.4207f,  -14.4275f,37.8575f,20.4110f,  -13.3458f,37.2878f,20.2688f,  -12.0709f,38.8053f,20.5410f,  -10.9646f,38.1689f,20.4114f,  
	-9.6627f,39.5858f,20.9586f,  -8.5416f,38.8822f,20.8396f,  -7.2413f,40.1689f,21.6494f,  -6.1135f,39.3980f,21.5379f,  -4.8450f,40.5295f,22.5928f,  
	-3.7177f,39.6942f,22.4863f,  -2.5109f,40.6498f,23.7640f,  -1.3928f,39.7536f,23.6589f,  -0.2769f,40.5177f,25.1330f,  0.8243f,39.5659f,25.0257f,  
	1.8218f,40.1280f,26.6662f,  2.9011f,39.1264f,26.5539f,  3.7549f,39.4807f,28.3280f,  4.8074f,38.4359f,28.2078f,  5.4952f,38.5816f,30.0807f,  
	6.5165f,37.5012f,29.9495f,  7.0197f,37.4421f,31.8848f,  8.0071f,36.3333f,31.7408f,  8.3111f,36.0775f,33.7015f,  9.2638f,34.9469f,33.5430f,  
	9.3583f,34.5060f,35.4921f,  10.2753f,33.3607f,35.3183f,  10.1544f,32.7498f,37.2195f,  11.0361f,31.5963f,37.0295f,  10.6987f,30.8333f,38.8478f,  
	11.5461f,29.6772f,38.6417f,  10.9955f,28.7825f,40.3442f,  11.8102f,27.6286f,40.1227f,  11.0541f,26.6243f,41.6789f,  11.8387f,25.4773f,41.4416f,  
	10.8892f,24.3868f,42.8241f,  11.6454f,23.2497f,42.5734f,  10.5182f,22.0973f,43.7582f,  11.2489f,20.9728f,43.4948f,  9.9636f,19.7831f,44.4616f,  
	10.6718f,18.6727f,44.1868f,  9.2509f,17.4701f,44.9195f,  9.9392f,16.3749f,44.6351f,  8.4078f,15.1832f,45.1216f,  9.0784f,14.1036f,44.8293f,  
	7.4636f,12.9458f,45.0623f,  8.1187f,11.8820f,44.7637f,  6.4494f,10.7797f,44.7399f,  7.0920f,9.7300f,44.4361f,  5.3979f,8.7033f,44.1570f,  
	6.0289f,7.6675f,43.8497f,  4.3401f,6.7345f,43.3210f,  4.9616f,5.7111f,43.0112f,  3.3081f,4.8879f,42.2428f,  3.9217f,3.8748f,41.9313f,  
	2.3329f,3.1751f,40.9372f,  2.9394f,2.1712f,40.6249f,  1.4434f,1.6063f,39.4228f,  2.0439f,0.6096f,39.1100f,  0.6671f,0.1883f,37.7210f,  
	1.2619f,-0.8022f,37.4080f,  0.0287f,-1.0737f,35.8564f,  0.6186f,-2.0599f,35.5431f,  -0.4493f,-2.1782f,33.8557f,  0.1362f,-3.1613f,33.5420f,  
	-36.1739f,34.2931f,46.8188f,  -34.9405f,34.0300f,46.8708f,  -34.9753f,32.6885f,45.3871f,  -33.8634f,32.3975f,45.3762f,  -33.9307f,31.2886f,43.7125f,  
	-32.9252f,30.9831f,43.6463f,  -33.0060f,30.1304f,41.8384f,  -32.0898f,29.8213f,41.7247f,  -32.1624f,29.2418f,39.8113f,  -31.3141f,28.9366f,39.6597f,  
	-31.3555f,28.6397f,37.6818f,  -30.5572f,28.3426f,37.4974f,  -30.5436f,28.3303f,35.4970f,  -29.7745f,28.0422f,35.2890f,  -29.6833f,28.3089f,33.3084f,  
	-28.9262f,28.0280f,33.0839f,  -28.7372f,28.5608f,31.1650f,  -27.9742f,28.2818f,30.9306f,  -27.6703f,29.0613f,29.1134f,  -26.8900f,28.7791f,28.8747f,  
	-26.4585f,29.7799f,27.1971f,  -25.6492f,29.4855f,26.9602f,  -25.0819f,30.6772f,25.4568f,  -24.2361f,30.3631f,25.2250f,  -23.5300f,31.7113f,23.9266f,  
	-22.6431f,31.3678f,23.7044f,  -21.8007f,32.8353f,22.6372f,  -20.8711f,32.4541f,22.4274f,  -19.9003f,34.0019f,21.6125f,  -18.9287f,33.5744f,21.4169f,  
	-17.8426f,35.1629f,20.8701f,  -16.8322f,34.6825f,20.6894f,  -15.6484f,36.2727f,20.4207f,  -14.6054f,35.7337f,20.2553f,  -13.3458f,37.2878f,20.2688f,  
	-12.2748f,36.6860f,20.1172f,  -10.9646f,38.1689f,20.4114f,  -9.8746f,37.5026f,20.2722f,  -8.5416f,38.8822f,20.8396f,  -7.4400f,38.1500f,20.7101f,  
	-6.1135f,39.3980f,21.5379f,  -5.0075f,38.6020f,21.4159f,  -3.7177f,39.6942f,22.4863f,  -2.6157f,38.8365f,22.3683f,  -1.3928f,39.7536f,23.6589f,  
	-0.3027f,38.8387f,23.5414f,  0.8243f,39.5659f,25.0257f,  1.8974f,38.5986f,24.9058f,  2.9011f,39.1264f,26.5539f,  3.9517f,38.1126f,26.4286f,  
	4.8074f,38.4359f,28.2078f,  5.8305f,37.3827f,28.0740f,  6.5165f,37.5012f,29.9495f,  7.5090f,36.4154f,29.8051f,  8.0071f,36.3333f,31.7408f,  
	8.9674f,35.2216f,31.5837f,  9.2638f,34.9469f,33.5430f,  10.1906f,33.8161f,33.3722f,  10.2753f,33.3607f,35.3183f,  11.1682f,32.2174f,35.1324f,  
	11.0361f,31.5963f,37.0295f,  11.8958f,30.4463f,36.8283f,  11.5461f,29.6772f,38.6417f,  12.3737f,28.5256f,38.4256f,  11.8102f,27.6286f,40.1227f,  
	12.6076f,26.4806f,39.8910f,  11.8387f,25.4773f,41.4416f,  12.6075f,24.3361f,41.1965f,  11.6454f,23.2497f,42.5734f,  12.3882f,22.1187f,42.3153f,  
	11.2489f,20.9728f,43.4948f,  11.9685f,19.8539f,43.2249f,  10.6718f,18.6727f,44.1868f,  11.3707f,17.5675f,43.9067f,  9.9392f,16.3749f,44.6351f,  
	10.6195f,15.2842f,44.3463f,  9.0784f,14.1036f,44.8293f,  9.7423f,13.0279f,44.5335f,  8.1187f,11.8820f,44.7637f,  8.7693f,10.8200f,44.4619f,  
	7.0920f,9.7300f,44.4361f,  7.7302f,8.6820f,44.1301f,  6.0289f,7.6675f,43.8497f,  6.6567f,6.6321f,43.5405f,  4.9616f,5.7111f,43.0112f,  
	5.5809f,4.6865f,42.6997f,  3.9217f,3.8748f,41.9313f,  4.5333f,2.8601f,41.6184f,  2.9394f,2.1712f,40.6249f,  3.5446f,1.1643f,40.3110f,  
	2.0439f,0.6096f,39.1100f,  2.6429f,-0.3902f,38.7955f,  1.2619f,-0.8022f,37.4080f,  1.8558f,-1.7970f,37.0931f,  0.6186f,-2.0599f,35.5431f,  
	1.2079f,-3.0509f,35.2277f,  0.1362f,-3.1613f,33.5420f,  0.7207f,-4.1490f,33.2258f,  -34.9405f,34.0300f,46.8708f,  -33.7422f,33.7084f,46.8824f,  
	-33.8634f,32.3975f,45.3762f,  -32.7824f,32.0566f,45.3283f,  -32.9252f,30.9831f,43.6463f,  -31.9458f,30.6353f,43.5465f,  -32.0898f,29.8213f,41.7247f,  
	-31.1914f,29.4760f,41.5822f,  -31.3141f,28.9366f,39.6597f,  -30.4798f,28.5996f,39.4799f,  -30.5572f,28.3426f,37.4974f,  -29.7658f,28.0165f,37.2893f,  
	-29.7745f,28.0422f,35.2890f,  -29.0080f,27.7269f,35.0600f,  -28.9262f,28.0280f,33.0839f,  -28.1658f,27.7196f,32.8410f,  -27.9742f,28.2818f,30.9306f,  
	-27.2072f,27.9764f,30.6794f,  -26.8900f,28.7791f,28.8747f,  -26.1031f,28.4685f,28.6220f,  -25.6492f,29.4855f,26.9602f,  -24.8331f,29.1626f,26.7098f,  
	-24.2361f,30.3631f,25.2250f,  -23.3842f,30.0185f,24.9820f,  -22.6431f,31.3678f,23.7044f,  -21.7520f,30.9931f,23.4722f,  -20.8711f,32.4541f,22.4274f,  
	-19.9397f,32.0403f,22.2084f,  -18.9287f,33.5744f,21.4169f,  -17.9587f,33.1142f,21.2125f,  -16.8322f,34.6825f,20.6894f,  -15.8281f,34.1694f,20.5006f,  
	-14.6054f,35.7337f,20.2553f,  -13.5708f,35.1624f,20.0812f,  -12.2748f,36.6860f,20.1172f,  -11.2178f,36.0539f,19.9570f,  -9.8746f,37.5026f,20.2722f,  
	-8.8017f,36.8069f,20.1235f,  -7.4400f,38.1500f,20.7101f,  -6.3580f,37.3919f,20.5710f,  -5.0075f,38.6020f,21.4159f,  -3.9245f,37.7822f,21.2832f,  
	-2.6157f,38.8365f,22.3683f,  -1.5399f,37.9586f,22.2387f,  -0.3027f,38.8387f,23.5414f,  0.7609f,37.9064f,23.4121f,  1.8974f,38.5986f,24.9058f,  
	2.9428f,37.6174f,24.7737f,  3.9517f,38.1126f,26.4286f,  4.9736f,37.0887f,26.2904f,  5.8305f,37.3827f,28.0740f,  6.8253f,36.3224f,27.9273f,  
	7.5090f,36.4154f,29.8051f,  8.4744f,35.3253f,29.6477f,  8.9674f,35.2216f,31.5837f,  9.9017f,34.1082f,31.4143f,  10.1906f,33.8161f,33.3722f,  
	11.0927f,32.6860f,33.1890f,  11.1682f,32.2174f,35.1324f,  12.0385f,31.0766f,34.9351f,  11.8958f,30.4463f,36.8283f,  12.7348f,29.3000f,36.6168f,  
	12.3737f,28.5256f,38.4256f,  13.1827f,27.3795f,38.1989f,  12.6076f,26.4806f,39.8910f,  13.3883f,25.3381f,39.6510f,  12.6075f,24.3361f,41.1965f,  
	13.3619f,23.2010f,40.9433f,  12.3882f,22.1187f,42.3153f,  13.1189f,20.9935f,42.0500f,  11.9685f,19.8539f,43.2249f,  12.6779f,18.7405f,42.9489f,  
	11.3707f,17.5675f,43.9067f,  12.0608f,16.4672f,43.6215f,  10.6195f,15.2842f,44.3463f,  11.2924f,14.1978f,44.0534f,  9.7423f,13.0279f,44.5335f,  
	10.4010f,11.9548f,44.2339f,  8.7693f,10.8200f,44.4619f,  9.4148f,9.7606f,44.1574f,  7.7302f,8.6820f,44.1301f,  8.3646f,7.6352f,43.8217f,  
	6.6567f,6.6321f,43.5405f,  7.2820f,5.5964f,43.2291f,  5.5809f,4.6865f,42.6997f,  6.1978f,3.6612f,42.3863f,  4.5333f,2.8601f,41.6184f,  
	5.1434f,1.8431f,41.3035f,  3.5446f,1.1643f,40.3110f,  4.1481f,0.1550f,39.9952f,  2.6429f,-0.3902f,38.7955f,  3.2411f,-1.3938f,38.4790f,  
	1.8558f,-1.7970f,37.0931f,  2.4491f,-2.7961f,36.7758f,  1.2079f,-3.0509f,35.2277f,  1.7961f,-4.0460f,34.9095f,  0.7207f,-4.1490f,33.2258f,  
	1.3046f,-5.1414f,32.9065f,  -33.7422f,33.7084f,46.8824f,  -32.5801f,33.3327f,46.8557f,  -32.7824f,32.0566f,45.3283f,  -31.7328f,31.6699f,45.2454f,  
	-31.9458f,30.6353f,43.5465f,  -30.9895f,30.2487f,43.4165f,  -31.1914f,29.4760f,41.5822f,  -30.3126f,29.0968f,41.4104f,  -30.4798f,28.5996f,39.4799f,  
	-29.6573f,28.2324f,39.2752f,  -29.7658f,28.0165f,37.2893f,  -28.9812f,27.6628f,37.0593f,  -29.0080f,27.7269f,35.0600f,  -28.2420f,27.3844f,34.8117f,  
	-28.1658f,27.7196f,32.8410f,  -27.4045f,27.3854f,32.5804f,  -27.2072f,27.9764f,30.6794f,  -26.4356f,27.6434f,30.4139f,  -26.1031f,28.4685f,28.6220f,  
	-25.3107f,28.1305f,28.3555f,  -24.8331f,29.1626f,26.7098f,  -24.0114f,28.8105f,26.4481f,  -23.3842f,30.0185f,24.9820f,  -22.5279f,29.6439f,24.7290f,  
	-21.7520f,30.9931f,23.4722f,  -20.8583f,30.5868f,23.2310f,  -19.9397f,32.0403f,22.2084f,  -19.0086f,31.5946f,21.9810f,  -17.9587f,33.1142f,21.2125f,  
	-16.9931f,32.6219f,21.0007f,  -15.8281f,34.1694f,20.5006f,  -14.8304f,33.6242f,20.3039f,  -13.5708f,35.1624f,20.0812f,  -12.5479f,34.5608f,19.8993f,  
	-11.2178f,36.0539f,19.9570f,  -10.1753f,35.3918f,19.7882f,  -8.8017f,36.8069f,20.1235f,  -7.7461f,36.0845f,19.9661f,  -6.3580f,37.3919f,20.5710f,  
	-5.2970f,36.6087f,20.4222f,  -3.9245f,37.7822f,21.2832f,  -2.8661f,36.9408f,21.1399f,  -1.5399f,37.9586f,22.2387f,  -0.4890f,37.0617f,22.0982f,  
	0.7609f,37.9064f,23.4121f,  1.7979f,36.9588f,23.2712f,  2.9428f,37.6174f,24.7737f,  3.9603f,36.6243f,24.6292f,  4.9736f,37.0887f,26.2904f,  
	5.9677f,36.0559f,26.1397f,  6.8253f,36.3224f,27.9273f,  7.7930f,35.2562f,27.7680f,  8.4744f,35.3253f,29.6477f,  9.4138f,34.2319f,29.4781f,  
	9.9017f,34.1082f,31.4143f,  10.8110f,32.9942f,31.2325f,  11.0927f,32.6860f,33.1890f,  11.9717f,31.5573f,32.9943f,  12.0385f,31.0766f,34.9351f,  
	12.8874f,29.9386f,34.7270f,  12.7348f,29.3000f,36.6168f,  13.5543f,28.1586f,36.3942f,  13.1827f,27.3795f,38.1989f,  13.9743f,26.2384f,37.9633f,  
	13.3883f,25.3381f,39.6510f,  14.1536f,24.2014f,39.4023f,  13.3619f,23.2010f,40.9433f,  14.1032f,22.0718f,40.6824f,  13.1189f,20.9935f,42.0500f,  
	13.8383f,19.8739f,41.7780f,  12.6779f,18.7405f,42.9489f,  13.3774f,17.6324f,42.6672f,  12.0608f,16.4672f,43.6215f,  12.7425f,15.3717f,43.3315f,  
	11.2924f,14.1978f,44.0534f,  11.9592f,13.1148f,43.7561f,  10.4010f,11.9548f,44.2339f,  11.0539f,10.8849f,43.9310f,  9.4148f,9.7606f,44.1574f,  
	10.0560f,8.7032f,43.8498f,  8.3646f,7.6352f,43.8217f,  8.9961f,6.5889f,43.5105f,  7.2820f,5.5964f,43.2291f,  7.9046f,4.5607f,42.9154f,  
	6.1978f,3.6612f,42.3863f,  6.8131f,2.6342f,42.0706f,  5.1434f,1.8431f,41.3035f,  5.7517f,0.8244f,40.9865f,  4.1481f,0.1550f,39.9952f,  
	4.7507f,-0.8575f,39.6771f,  3.2411f,-1.3938f,38.4790f,  3.8385f,-2.4014f,38.1599f,  2.4491f,-2.7961f,36.7758f,  3.0413f,-3.7990f,36.4557f,  
	1.7961f,-4.0460f,34.9095f,  2.3839f,-5.0457f,34.5882f,  1.3046f,-5.1414f,32.9065f,  1.8882f,-6.1389f,32.5834f,  -32.5801f,33.3327f,46.8557f,  
	-31.4547f,32.9073f,46.7925f,  -31.7328f,31.6699f,45.2454f,  -30.7118f,31.2414f,45.1310f,  -30.9895f,30.2487f,43.4165f,  -30.0585f,29.8258f,43.2560f,  
	-30.3126f,29.0968f,41.4104f,  -29.4508f,28.6860f,41.2127f,  -29.6573f,28.2324f,39.2752f,  -28.8463f,27.8367f,39.0477f,  -28.9812f,27.6628f,37.0593f,  
	-28.2015f,27.2818f,36.8092f,  -28.2420f,27.3844f,34.8117f,  -27.4787f,27.0163f,34.5452f,  -27.4045f,27.3854f,32.5804f,  -26.6413f,27.0245f,32.3051f,  
	-26.4356f,27.6434f,30.4139f,  -25.6608f,27.2842f,30.1341f,  -25.3107f,28.1305f,28.3555f,  -24.5138f,27.7644f,28.0777f,  -24.0114f,28.8105f,26.4481f,  
	-23.1857f,28.4295f,26.1764f,  -22.5279f,29.6439f,24.7290f,  -21.6688f,29.2389f,24.4673f,  -20.8583f,30.5868f,23.2310f,  -19.9639f,30.1497f,22.9819f,  
	-19.0086f,31.5946f,21.9810f,  -18.0804f,31.1175f,21.7468f,  -16.9931f,32.6219f,21.0007f,  -16.0321f,32.0978f,20.7815f,  -14.8304f,33.6242f,20.3039f,  
	-13.8424f,33.0488f,20.1002f,  -12.5479f,34.5608f,19.8993f,  -11.5374f,33.9286f,19.7099f,  -10.1753f,35.3918f,19.7882f,  -9.1483f,34.7023f,19.6116f,  
	-7.7461f,36.0845f,19.9661f,  -6.7096f,35.3361f,19.8000f,  -5.2970f,36.6087f,20.4222f,  -4.2587f,35.8027f,20.2639f,  -2.8661f,36.9408f,21.1399f,  
	-1.8309f,36.0789f,20.9866f,  -0.4890f,37.0617f,22.0982f,  0.5365f,36.1479f,21.9470f,  1.7979f,36.9588f,23.2712f,  2.8080f,35.9973f,23.1188f,  
	3.9603f,36.6243f,24.6292f,  4.9508f,35.6205f,24.4728f,  5.9677f,36.0559f,26.1397f,  6.9351f,35.0156f,25.9769f,  7.7930f,35.2562f,27.7680f,  
	8.7349f,34.1852f,27.5969f,  9.4138f,34.2319f,29.4781f,  10.3279f,33.1365f,29.2965f,  10.8110f,32.9942f,31.2325f,  11.6967f,31.8802f,31.0393f,  
	11.9717f,31.5573f,32.9943f,  12.8288f,30.4303f,32.7888f,  12.8874f,29.9386f,34.7270f,  13.7158f,28.8047f,34.5077f,  13.5543f,28.1586f,36.3942f,  
	14.3555f,27.0215f,36.1623f,  13.9743f,26.2384f,37.9633f,  14.7494f,25.1027f,37.7187f,  14.1536f,24.2014f,39.4023f,  14.9046f,23.0706f,39.1454f,  
	14.1032f,22.0718f,40.6824f,  14.8321f,20.9483f,40.4142f,  13.8383f,19.8739f,41.7780f,  14.5470f,18.7600f,41.4998f,  13.3774f,17.6324f,42.6672f,  
	14.0678f,16.5296f,42.3801f,  12.7425f,15.3717f,43.3315f,  13.4173f,14.2803f,43.0365f,  11.9592f,13.1148f,43.7561f,  12.6196f,12.0357f,43.4549f,  
	11.0539f,10.8849f,43.9310f,  11.7019f,9.8178f,43.6246f,  10.0560f,8.7032f,43.8498f,  10.6936f,7.6470f,43.5392f,  8.9961f,6.5889f,43.5105f,  
	9.6243f,5.5434f,43.1967f,  7.9046f,4.5607f,42.9154f,  8.5251f,3.5241f,42.5990f,  6.8131f,2.6342f,42.0706f,  7.4262f,1.6062f,41.7525f,  
	5.7517f,0.8244f,40.9865f,  6.3588f,-0.1970f,40.6670f,  4.7507f,-0.8575f,39.6771f,  5.3524f,-1.8733f,39.3563f,  3.8385f,-2.4014f,38.1599f,  
	4.4348f,-3.4121f,37.8379f,  3.0413f,-3.7990f,36.4557f,  3.6330f,-4.8061f,36.1323f,  2.3839f,-5.0457f,34.5882f,  2.9714f,-6.0502f,34.2630f,  
	1.8882f,-6.1389f,32.5834f,  2.4715f,-7.1408f,32.2561f,  -31.4547f,32.9073f,46.7925f,  -30.3631f,32.4370f,46.6965f,  -30.7118f,31.2414f,45.1310f,  
	-29.7218f,30.7740f,44.9850f,  -30.0585f,29.8258f,43.2560f,  -29.1499f,29.3696f,43.0686f,  -29.4508f,28.6860f,41.2127f,  -28.6055f,28.2457f,40.9911f,  
	-28.8463f,27.8367f,39.0477f,  -28.0449f,27.4136f,38.7992f,  -28.2015f,27.2818f,36.8092f,  -27.4283f,26.8754f,36.5402f,  -27.4787f,27.0163f,34.5452f,  
	-26.7165f,26.6223f,34.2634f,  -26.6413f,27.0245f,32.3051f,  -25.8773f,26.6384f,32.0151f,  -25.6608f,27.2842f,30.1341f,  -24.8831f,26.8982f,29.8427f,  
	-24.5138f,27.7644f,28.0777f,  -23.7139f,27.3709f,27.7896f,  -23.1857f,28.4295f,26.1764f,  -22.3572f,28.0195f,25.8961f,  -21.6688f,29.2389f,24.4673f,  
	-20.8084f,28.8042f,24.1978f,  -19.9639f,30.1497f,22.9819f,  -19.0713f,29.6820f,22.7263f,  -18.0804f,31.1175f,21.7468f,  -17.1553f,30.6093f,21.5059f,  
	-16.0321f,32.0978f,20.7815f,  -15.0790f,31.5438f,20.5562f,  -13.8424f,33.0488f,20.1002f,  -12.8646f,32.4426f,19.8899f,  -11.5374f,33.9286f,19.7099f,  
	-10.5403f,33.2689f,19.5135f,  -9.1483f,34.7023f,19.6116f,  -8.1381f,33.9861f,19.4273f,  -6.7096f,35.3361f,19.8000f,  -5.6938f,34.5637f,19.6255f,  
	-4.2587f,35.8027f,20.2639f,  -3.2418f,34.9748f,20.0965f,  -1.8309f,36.0789f,20.9866f,  -0.8196f,35.1986f,20.8236f,  0.5365f,36.1479f,21.9470f,  
	1.5364f,35.2186f,21.7851f,  2.8080f,35.9973f,23.1188f,  3.7921f,35.0235f,22.9552f,  4.9508f,35.6205f,24.4728f,  5.9151f,34.6075f,24.3050f,  
	6.9351f,35.0156f,25.9769f,  7.8769f,33.9688f,25.8027f,  8.7349f,34.1852f,27.5969f,  9.6516f,33.1106f,27.4139f,  10.3279f,33.1365f,29.2965f,  
	11.2182f,32.0398f,29.1034f,  11.6967f,31.8802f,31.0393f,  12.5602f,30.7669f,30.8351f,  12.8288f,30.4303f,32.7888f,  13.6646f,29.3065f,32.5719f,  
	13.7158f,28.8047f,34.5077f,  14.5254f,27.6743f,34.2789f,  14.3555f,27.0215f,36.1623f,  15.1394f,25.8895f,35.9212f,  14.7494f,25.1027f,37.7187f,  
	15.5094f,23.9726f,37.4654f,  14.9046f,23.0706f,39.1454f,  15.6424f,21.9454f,38.8807f,  14.8321f,20.9483f,40.4142f,  15.5495f,19.8305f,40.1392f,  
	14.5470f,18.7600f,41.4998f,  15.2456f,17.6515f,41.2156f,  14.0678f,16.5296f,42.3801f,  14.7503f,15.4312f,42.0876f,  13.4173f,14.2803f,43.0365f,  
	14.0849f,13.1932f,42.7371f,  12.6196f,12.0357f,43.4549f,  13.2742f,10.9599f,43.1497f,  11.7019f,9.8178f,43.6246f,  12.3457f,8.7526f,43.3145f,  
	10.6936f,7.6470f,43.5392f,  11.3275f,6.5923f,43.2254f,  9.6243f,5.5434f,43.1967f,  10.2500f,4.4977f,42.8799f,  8.5251f,3.5241f,42.5990f,  
	9.1430f,2.4873f,42.2801f,  7.4262f,1.6062f,41.7525f,  8.0379f,0.5763f,41.4317f,  6.3588f,-0.1970f,40.6670f,  6.9649f,-1.2211f,40.3445f,  
	5.3524f,-1.8733f,39.3563f,  5.9529f,-2.8918f,39.0324f,  4.4348f,-3.4121f,37.8379f,  5.0305f,-4.4265f,37.5125f,  3.6330f,-4.8061f,36.1323f,  
	4.2245f,-5.8177f,35.8050f,  2.9714f,-6.0502f,34.2630f,  3.5586f,-7.0590f,33.9334f,  2.4715f,-7.1408f,32.2561f,  3.0546f,-8.1472f,31.9238f,  
	-30.3631f,32.4370f,46.6965f,  -29.3083f,31.9253f,46.5679f,  -29.7218f,30.7740f,44.9850f,  -28.7598f,30.2715f,44.8111f,  -29.1499f,29.3696f,43.0686f,  
	-28.2631f,28.8828f,42.8563f,  -28.6055f,28.2457f,40.9911f,  -27.7749f,27.7776f,40.7478f,  -28.0449f,27.4136f,38.7992f,  -27.2544f,26.9651f,38.5311f,  
	-27.4283f,26.8754f,36.5402f,  -26.6599f,26.4438f,36.2551f,  -26.7165f,26.6223f,34.2634f,  -25.9567f,26.2040f,33.9663f,  -25.8773f,26.6384f,32.0151f,  
	-25.1127f,26.2266f,31.7131f,  -24.8831f,26.8982f,29.8427f,  -24.1036f,26.4860f,29.5409f,  -23.7139f,27.3709f,27.7896f,  -22.9118f,26.9497f,27.4929f,  
	-22.3572f,28.0195f,25.8961f,  -21.5275f,27.5811f,25.6082f,  -20.8084f,28.8042f,24.1978f,  -19.9491f,28.3400f,23.9224f,  -19.0713f,29.6820f,22.7263f,  
	-18.1808f,29.1840f,22.4646f,  -17.1553f,30.6093f,21.5059f,  -16.2366f,30.0716f,21.2595f,  -15.0790f,31.5438f,20.5562f,  -14.1343f,30.9592f,20.3252f,  
	-12.8646f,32.4426f,19.8899f,  -11.8985f,31.8088f,19.6735f,  -10.5403f,33.2689f,19.5135f,  -9.5582f,32.5819f,19.3104f,  -8.1381f,33.9861f,19.4273f,  
	-7.1468f,33.2450f,19.2357f,  -5.6938f,34.5637f,19.6255f,  -4.6976f,33.7683f,19.4429f,  -3.2418f,34.9748f,20.0965f,  -2.2473f,34.1273f,19.9203f,  
	-0.8196f,35.1986f,20.8236f,  0.1674f,34.3014f,20.6508f,  1.5364f,35.2186f,21.7851f,  2.5114f,34.2752f,21.6128f,  3.7921f,35.0235f,22.9552f,  
	4.7506f,34.0390f,22.7809f,  5.9151f,34.6075f,24.3050f,  6.8544f,33.5865f,24.1263f,  7.8769f,33.9688f,25.8027f,  8.7936f,32.9170f,25.6171f,  
	9.6516f,33.1106f,27.4139f,  10.5443f,32.0334f,27.2198f,  11.2182f,32.0398f,29.1034f,  12.0860f,30.9425f,28.8996f,  12.5602f,30.7669f,30.8351f,  
	13.4016f,29.6557f,30.6197f,  13.6646f,29.3065f,32.5719f,  14.4810f,28.1854f,32.3453f,  14.5254f,27.6743f,34.2789f,  15.3169f,26.5482f,34.0406f,  
	15.1394f,25.8895f,35.9212f,  15.9073f,24.7625f,35.6709f,  15.5094f,23.9726f,37.4654f,  16.2552f,22.8479f,37.2039f,  15.6424f,21.9454f,38.8807f,  
	16.3676f,20.8258f,38.6088f,  15.5495f,19.8305f,40.1392f,  16.2558f,18.7184f,39.8579f,  15.2456f,17.6515f,41.2156f,  15.9354f,16.5479f,40.9255f,  
	14.7503f,15.4312f,42.0876f,  15.4248f,14.3376f,41.7901f,  14.0849f,13.1932f,42.7371f,  14.7460f,12.1100f,42.4333f,  13.2742f,10.9599f,43.1497f,  
	13.9240f,9.8868f,42.8404f,  12.3457f,8.7526f,43.3145f,  12.9851f,7.6896f,43.0010f,  11.3275f,6.5923f,43.2254f,  11.9584f,5.5382f,42.9083f,  
	10.2500f,4.4977f,42.8799f,  10.8728f,3.4525f,42.5602f,  9.1430f,2.4873f,42.2801f,  9.7593f,1.4491f,41.9581f,  8.0379f,0.5763f,41.4317f,  
	8.6483f,-0.4558f,41.1077f,  6.9649f,-1.2211f,40.3445f,  7.5695f,-2.2473f,40.0189f,  5.9529f,-2.8918f,39.0324f,  6.5526f,-3.9137f,38.7051f,  
	5.0305f,-4.4265f,37.5125f,  5.6260f,-5.4453f,37.1831f,  4.2245f,-5.8177f,35.8050f,  4.8156f,-6.8334f,35.4733f,  3.5586f,-7.0590f,33.9334f,  
	4.1457f,-8.0722f,33.5990f,  3.0546f,-8.1472f,31.9238f,  3.6379f,-9.1581f,31.5861f,  -29.3083f,31.9253f,46.5679f,  -28.2871f,31.3763f,46.4104f,  
	-28.7598f,30.2715f,44.8111f,  -27.8248f,29.7370f,44.6114f,  -28.2631f,28.8828f,42.8563f,  -27.3962f,28.3675f,42.6214f,  -27.7749f,27.7776f,40.7478f,  
	-26.9598f,27.2839f,40.4840f,  -27.2544f,26.9651f,38.5311f,  -26.4726f,26.4918f,38.2463f,  -26.6599f,26.4438f,36.2551f,  -25.8974f,25.9885f,35.9543f,  
	-25.9567f,26.2040f,33.9663f,  -25.1988f,25.7612f,33.6569f,  -25.1127f,26.2266f,31.7131f,  -24.3482f,25.7900f,31.4005f,  -24.1036f,26.4860f,29.5409f,  
	-23.3232f,26.0476f,29.2305f,  -22.9118f,26.9497f,27.4929f,  -22.1089f,26.5014f,27.1888f,  -21.5275f,27.5811f,25.6082f,  -20.6985f,27.1144f,25.3147f,  
	-19.9491f,28.3400f,23.9224f,  -19.0910f,27.8465f,23.6413f,  -18.1808f,29.1840f,22.4646f,  -17.2954f,28.6573f,22.1981f,  -16.2366f,30.0716f,21.2595f,  
	-15.3247f,29.5037f,21.0083f,  -14.1343f,30.9592f,20.3252f,  -13.1996f,30.3471f,20.0889f,  -11.8985f,31.8088f,19.6735f,  -10.9454f,31.1474f,19.4514f,  
	-9.5582f,32.5819f,19.3104f,  -8.5930f,31.8693f,19.1013f,  -7.1468f,33.2450f,19.2357f,  -6.1734f,32.4801f,19.0371f,  -4.6976f,33.7683f,19.4429f,  
	-3.7223f,32.9522f,19.2526f,  -2.2473f,34.1273f,19.9203f,  -1.2758f,33.2615f,19.7355f,  0.1674f,34.3014f,20.6508f,  1.1306f,33.3885f,20.4686f,  
	2.5114f,34.2752f,21.6128f,  3.4616f,33.3197f,21.4307f,  4.7506f,34.0390f,22.7809f,  5.6848f,33.0448f,22.5963f,  6.8544f,33.5865f,24.1263f,  
	7.7688f,32.5589f,23.9367f,  8.7936f,32.9170f,25.6171f,  9.6865f,31.8610f,25.4207f,  10.5443f,32.0334f,27.2198f,  11.4144f,30.9541f,27.0152f,  
	12.0860f,30.9425f,28.8996f,  12.9312f,29.8461f,28.6846f,  13.4016f,29.6557f,30.6197f,  14.2232f,28.5461f,30.3945f,  14.4810f,28.1854f,32.3453f,  
	15.2787f,27.0677f,32.1091f,  15.3169f,26.5482f,34.0406f,  16.0914f,25.4267f,33.7929f,  15.9073f,24.7625f,35.6709f,  16.6600f,23.6406f,35.4123f,  
	16.2552f,22.8479f,37.2039f,  16.9875f,21.7288f,36.9348f,  16.3676f,20.8258f,38.6088f,  17.0810f,19.7119f,38.3302f,  16.2558f,18.7184f,39.8579f,  
	16.9524f,17.6113f,39.5701f,  15.9354f,16.5479f,40.9255f,  16.6165f,15.4494f,40.6299f,  15.4248f,14.3376f,41.7901f,  16.0921f,13.2485f,41.4878f,  
	14.7460f,12.1100f,42.4333f,  15.4015f,11.0301f,42.1250f,  13.9240f,9.8868f,42.8404f,  14.5688f,8.8164f,42.5274f,  12.9851f,7.6896f,43.0010f,  
	13.6211f,6.6278f,42.6839f,  11.9584f,5.5382f,42.9083f,  12.5859f,4.4853f,42.5881f,  10.8728f,3.4525f,42.5602f,  11.4935f,2.4066f,42.2372f,  
	9.7593f,1.4491f,41.9581f,  10.3740f,0.4095f,41.6327f,  8.6483f,-0.4558f,41.1077f,  9.2570f,-1.4893f,40.7805f,  7.5695f,-2.2473f,40.0189f,  
	8.1732f,-3.2762f,39.6897f,  6.5526f,-3.9137f,38.7051f,  7.1520f,-4.9394f,38.3736f,  5.6260f,-5.4453f,37.1831f,  6.2210f,-6.4678f,36.8492f,  
	4.8156f,-6.8334f,35.4733f,  5.4065f,-7.8533f,35.1366f,  4.1457f,-8.0722f,33.5990f,  4.7327f,-9.0898f,33.2590f,  3.6379f,-9.1581f,31.5861f,  
	4.2213f,-10.1733f,31.2422f,  -28.2871f,31.3763f,46.4104f,  -27.2982f,30.7938f,46.2263f,  -27.8248f,29.7370f,44.6114f,  -26.9152f,29.1733f,44.3882f,  
	-27.3962f,28.3675f,42.6214f,  -26.5496f,27.8263f,42.3654f,  -26.9598f,27.2839f,40.4840f,  -26.1577f,26.7658f,40.2028f,  -26.4726f,26.4918f,38.2463f,  
	-25.7006f,25.9956f,37.9451f,  -25.8974f,25.9885f,35.9543f,  -25.1399f,25.5100f,35.6407f,  -25.1988f,25.7612f,33.6569f,  -24.4434f,25.2949f,33.3364f,  
	-24.3482f,25.7900f,31.4005f,  -23.5842f,25.3287f,31.0790f,  -23.3232f,26.0476f,29.2305f,  -22.5428f,25.5835f,28.9125f,  -22.1089f,26.5014f,27.1888f,  
	-21.3068f,26.0261f,26.8792f,  -20.6985f,27.1144f,25.3147f,  -19.8704f,26.6195f,25.0158f,  -19.0910f,27.8465f,23.6413f,  -18.2372f,27.3252f,23.3559f,  
	-17.2954f,28.6573f,22.1981f,  -16.4154f,28.1010f,21.9275f,  -15.3247f,29.5037f,21.0083f,  -14.4213f,28.9086f,20.7525f,  -13.1996f,30.3471f,20.0889f,  
	-12.2762f,29.7072f,19.8479f,  -10.9454f,31.1474f,19.4514f,  -10.0075f,30.4600f,19.2244f,  -8.5930f,31.8693f,19.1013f,  -7.6439f,31.1321f,18.8860f,  
	-6.1734f,32.4801f,19.0371f,  -5.2192f,31.6935f,18.8318f,  -3.7223f,32.9522f,19.2526f,  -2.7684f,32.1165f,19.0547f,  -1.2758f,33.2615f,19.7355f,  
	-0.3266f,32.3785f,19.5421f,  1.1306f,33.3885f,20.4686f,  2.0700f,32.4621f,20.2774f,  3.4616f,33.3197f,21.4307f,  4.3882f,32.3529f,21.2390f,  
	5.6848f,33.0448f,22.5963f,  6.5946f,32.0424f,22.4016f,  7.7688f,32.5589f,23.9367f,  8.6596f,31.5256f,23.7370f,  9.6865f,31.8610f,25.4207f,  
	10.5568f,30.8016f,25.2142f,  11.4144f,30.9541f,27.0152f,  12.2616f,29.8746f,26.7997f,  12.9312f,29.8461f,28.6846f,  13.7563f,28.7502f,28.4600f,  
	14.2232f,28.5461f,30.3945f,  15.0255f,27.4390f,30.1597f,  15.2787f,27.0677f,32.1091f,  16.0586f,25.9539f,31.8634f,  16.0914f,25.4267f,33.7929f,  
	16.8500f,24.3099f,33.5366f,  16.6600f,23.6406f,35.4123f,  17.3986f,22.5239f,35.1457f,  16.9875f,21.7288f,36.9348f,  17.7072f,20.6151f,36.6586f,  
	17.0810f,19.7119f,38.3302f,  17.7838f,18.6033f,38.0446f,  16.9524f,17.6113f,39.5701f,  17.6394f,16.5096f,39.2765f,  16.6165f,15.4494f,40.6299f,  
	17.2895f,14.3558f,40.3292f,  16.0921f,13.2485f,41.4878f,  16.7530f,12.1630f,41.1806f,  15.4015f,11.0301f,42.1250f,  16.0514f,9.9535f,41.8126f,  
	14.5688f,8.8164f,42.5274f,  15.2095f,7.7480f,42.2103f,  13.6211f,6.6278f,42.6839f,  14.2530f,5.5680f,42.3632f,  12.5859f,4.4853f,42.5881f,  
	13.2108f,3.4323f,42.2643f,  11.4935f,2.4066f,42.2372f,  12.1122f,1.3599f,41.9106f,  10.3740f,0.4095f,41.6327f,  10.9866f,-0.6310f,41.3040f,  
	9.2570f,-1.4893f,40.7805f,  9.8645f,-2.5251f,40.4495f,  8.1732f,-3.2762f,39.6897f,  8.7764f,-4.3086f,39.3563f,  7.1520f,-4.9394f,38.3736f,  
	7.7508f,-5.9685f,38.0377f,  6.2210f,-6.4678f,36.8492f,  6.8157f,-7.4943f,36.5104f,  5.4065f,-7.8533f,35.1366f,  5.9974f,-8.8775f,34.7945f,  
	4.7327f,-9.0898f,33.2590f,  5.3200f,-10.1117f,32.9129f,  4.2213f,-10.1733f,31.2422f,  4.8051f,-11.1929f,30.8915f,  -27.2982f,30.7938f,46.2263f,  
	-26.3402f,30.1811f,46.0178f,  -26.9152f,29.1733f,44.3882f,  -26.0308f,28.5830f,44.1431f,  -26.5496f,27.8263f,42.3654f,  -25.7206f,27.2608f,42.0911f,  
	-26.1577f,26.7658f,40.2028f,  -25.3696f,26.2252f,39.9047f,  -25.7006f,25.9956f,37.9451f,  -24.9370f,25.4771f,37.6306f,  -25.1399f,25.5100f,35.6407f,  
	-24.3876f,25.0091f,35.3157f,  -24.4434f,25.2949f,33.3364f,  -23.6905f,24.8053f,33.0069f,  -23.5842f,25.3287f,31.0790f,  -22.8216f,24.8432f,30.7500f,  
	-22.5428f,25.5835f,28.9125f,  -21.7636f,25.0939f,28.5893f,  -21.3068f,26.0261f,26.8792f,  -20.5057f,25.5242f,26.5646f,  -19.8704f,26.6195f,25.0158f,  
	-19.0460f,26.0980f,24.7131f,  -18.2372f,27.3252f,23.3559f,  -17.3877f,26.7751f,23.0671f,  -16.4154f,28.1010f,21.9275f,  -15.5427f,27.5181f,21.6530f,  
	-14.4213f,28.9086f,20.7525f,  -13.5276f,28.2860f,20.4930f,  -12.2762f,29.7072f,19.8479f,  -11.3661f,29.0412f,19.6030f,  -10.0075f,30.4600f,19.2244f,  
	-9.0839f,29.7476f,18.9923f,  -7.6439f,31.1321f,18.8860f,  -6.7124f,30.3726f,18.6653f,  -5.2192f,31.6935f,18.8318f,  -4.2848f,30.8862f,18.6201f,  
	-2.7684f,32.1165f,19.0547f,  -1.8354f,31.2624f,18.8493f,  -0.3266f,32.3785f,19.5421f,  0.5997f,31.4806f,19.3409f,  2.0700f,32.4621f,20.2774f,  
	2.9868f,31.5228f,20.0775f,  4.3882f,32.3529f,21.2390f,  5.2910f,31.3765f,21.0380f,  6.5946f,32.0424f,22.4016f,  7.4811f,31.0330f,22.1973f,  
	8.6596f,31.5256f,23.7370f,  9.5280f,30.4875f,23.5275f,  10.5568f,30.8016f,25.2142f,  11.4040f,29.7406f,24.9973f,  12.2616f,29.8746f,26.7997f,  
	13.0884f,28.7943f,26.5748f,  13.7563f,28.7502f,28.4600f,  14.5617f,27.6559f,28.2258f,  15.0255f,27.4390f,30.1597f,  15.8094f,26.3351f,29.9154f,  
	16.0586f,25.9539f,31.8634f,  16.8219f,24.8442f,31.6089f,  16.8500f,24.3099f,33.5366f,  17.5937f,23.1977f,33.2722f,  17.3986f,22.5239f,35.1457f,  
	18.1237f,21.4125f,34.8717f,  17.7072f,20.6151f,36.6586f,  18.4154f,19.5067f,36.3752f,  17.7838f,18.6033f,38.0446f,  18.4762f,17.4999f,37.7530f,  
	17.6394f,16.5096f,39.2765f,  18.3176f,15.4129f,38.9774f,  17.2895f,14.3558f,40.3292f,  17.9553f,13.2662f,40.0232f,  16.7530f,12.1630f,41.1806f,  
	17.4076f,11.0815f,40.8688f,  16.0514f,9.9535f,41.8126f,  16.6965f,8.8795f,41.4958f,  15.2095f,7.7480f,42.2103f,  15.8456f,6.6821f,41.8895f,  
	14.2530f,5.5680f,42.3632f,  14.8819f,4.5087f,42.0388f,  13.2108f,3.4323f,42.2643f,  13.8334f,2.3792f,41.9367f,  12.1122f,1.3599f,41.9106f,  
	12.7285f,0.3130f,41.5806f,  10.9866f,-0.6310f,41.3040f,  11.5977f,-1.6731f,40.9715f,  9.8645f,-2.5251f,40.4495f,  10.4713f,-3.5637f,40.1143f,  
	8.7764f,-4.3086f,39.3563f,  9.3788f,-5.3441f,39.0185f,  7.7508f,-5.9685f,38.0377f,  8.3491f,-7.0013f,37.6969f,  6.8157f,-7.4943f,36.5104f,  
	7.4103f,-8.5248f,36.1662f,  5.9974f,-8.8775f,34.7945f,  6.5883f,-9.9059f,34.4462f,  5.3200f,-10.1117f,32.9129f,  5.9075f,-11.1381f,32.5600f,  
	4.8051f,-11.1929f,30.8915f,  5.3894f,-12.2166f,30.5334f,  -26.3402f,30.1811f,46.0178f,  -25.4124f,29.5412f,45.7867f,  -26.0308f,28.5830f,44.1431f,  
	-25.1689f,27.9686f,43.8791f,  -25.7206f,27.2608f,42.0911f,  -24.9101f,26.6733f,41.7994f,  -25.3696f,26.2252f,39.9047f,  -24.5936f,25.6632f,39.5926f,  
	-24.9370f,25.4771f,37.6306f,  -24.1817f,24.9374f,37.3043f,  -24.3876f,25.0091f,35.3157f,  -23.6401f,24.4863f,34.9812f,  -23.6905f,24.8053f,33.0069f,  
	-22.9407f,24.2930f,32.6697f,  -22.8216f,24.8432f,30.7500f,  -22.0611f,24.3337f,30.4157f,  -21.7636f,25.0939f,28.5893f,  -20.9859f,24.5791f,28.2612f,  
	-20.5057f,25.5242f,26.5646f,  -19.7080f,24.9968f,26.2465f,  -19.0460f,26.0980f,24.7131f,  -18.2252f,25.5488f,24.4076f,  -17.3877f,26.7751f,23.0671f,  
	-16.5447f,26.1993f,22.7751f,  -15.5427f,27.5181f,21.6530f,  -14.6784f,26.9081f,21.3757f,  -13.5276f,28.2860f,20.4930f,  -12.6457f,27.6373f,20.2305f,  
	-11.3661f,29.0412f,19.6030f,  -10.4689f,28.3500f,19.3540f,  -9.0839f,29.7476f,18.9923f,  -8.1764f,29.0123f,18.7557f,  -6.7124f,30.3726f,18.6653f,  
	-5.7991f,29.5916f,18.4391f,  -4.2848f,30.8862f,18.6201f,  -3.3699f,30.0593f,18.4020f,  -1.8354f,31.2624f,18.8493f,  -0.9241f,30.3922f,18.6371f,  
	0.5997f,31.4806f,19.3409f,  1.5045f,30.5686f,19.1318f,  2.9868f,31.5228f,20.0775f,  3.8805f,30.5725f,19.8692f,  5.2910f,31.3765f,21.0380f,  
	6.1712f,30.3915f,20.8281f,  7.4811f,31.0330f,22.1973f,  8.3456f,30.0172f,21.9839f,  9.5280f,30.4875f,23.5275f,  10.3733f,29.4464f,23.3081f,  
	11.4040f,29.7406f,24.9973f,  12.2308f,28.6776f,24.7711f,  13.0884f,28.7943f,26.5748f,  13.8952f,27.7145f,26.3404f,  14.5617f,27.6559f,28.2258f,  
	15.3482f,26.5637f,27.9822f,  15.8094f,26.3351f,29.9154f,  16.5761f,25.2345f,29.6623f,  16.8219f,24.8442f,31.6089f,  17.5696f,23.7386f,31.3463f,  
	17.5937f,23.1977f,33.2722f,  18.3232f,22.0904f,33.0002f,  18.1237f,21.4125f,34.8717f,  18.8364f,20.3062f,34.5903f,  18.4154f,19.5067f,36.3752f,  
	19.1124f,18.4036f,36.0854f,  18.4762f,17.4999f,37.7530f,  19.1589f,16.4019f,37.4555f,  18.3176f,15.4129f,38.9774f,  18.9879f,14.3207f,38.6726f,  
	17.9553f,13.2662f,40.0232f,  18.6141f,12.1809f,39.7123f,  17.4076f,11.0815f,40.8688f,  18.0567f,10.0030f,40.5525f,  16.6965f,8.8795f,41.4958f,  
	17.3365f,7.8085f,41.1750f,  15.8456f,6.6821f,41.8895f,  16.4782f,5.6174f,41.5646f,  14.8819f,4.5087f,42.0388f,  15.5080f,3.4500f,41.7104f,  
	13.8334f,2.3792f,41.9367f,  14.4531f,1.3266f,41.6055f,  12.7285f,0.3130f,41.5806f,  13.3430f,-0.7349f,41.2467f,  11.5977f,-1.6731f,40.9715f,  
	12.2078f,-2.7176f,40.6347f,  10.4713f,-3.5637f,40.1143f,  11.0771f,-4.6051f,39.7747f,  9.3788f,-5.3441f,39.0185f,  9.9805f,-6.3827f,38.6758f,  
	8.3491f,-7.0013f,37.6969f,  8.9471f,-8.0378f,37.3506f,  7.4103f,-8.5248f,36.1662f,  8.0047f,-9.5595f,35.8158f,  6.5883f,-9.9059f,34.4462f,  
	7.1795f,-10.9387f,34.0912f,  5.9075f,-11.1381f,32.5600f,  6.4954f,-12.1686f,32.1997f,  5.3894f,-12.2166f,30.5334f,  5.9742f,-13.2443f,30.1671f,  
	-25.4124f,29.5412f,45.7867f,  -24.5119f,28.8769f,45.5360f,  -25.1689f,27.9686f,43.8791f,  -24.3298f,27.3323f,43.5970f,  -24.9101f,26.6733f,41.7994f,  
	-24.1156f,26.0651f,41.4932f,  -24.5936f,25.6632f,39.5926f,  -23.8293f,25.0812f,39.2683f,  -24.1817f,24.9374f,37.3043f,  -23.4340f,24.3773f,36.9681f,  
	-23.6401f,24.4863f,34.9812f,  -22.8979f,23.9425f,34.6388f,  -22.9407f,24.2930f,32.6697f,  -22.1942f,23.7584f,32.3271f,  -22.0611f,24.3337f,30.4157f,  
	-21.3029f,23.8006f,30.0765f,  -20.9859f,24.5791f,28.2612f,  -20.2117f,24.0404f,27.9298f,  -19.7080f,24.9968f,26.2465f,  -18.9133f,24.4430f,25.9261f,  
	-18.2252f,25.5488f,24.4076f,  -17.4100f,24.9748f,24.0994f,  -16.5447f,26.1993f,22.7751f,  -15.7087f,25.5970f,22.4809f,  -14.6784f,26.9081f,21.3757f,  
	-13.8243f,26.2723f,21.0964f,  -12.6457f,27.6373f,20.2305f,  -11.7751f,26.9634f,19.9651f,  -10.4689f,28.3500f,19.3540f,  -9.5861f,27.6356f,19.1016f,  
	-8.1764f,29.0123f,18.7557f,  -7.2855f,28.2547f,18.5149f,  -5.7991f,29.5916f,18.4391f,  -4.9039f,28.7900f,18.2078f,  -3.3699f,30.0593f,18.4020f,  
	-2.4755f,29.2154f,18.1781f,  -0.9241f,30.3922f,18.6371f,  -0.0333f,29.5066f,18.4180f,  1.5045f,30.5686f,19.1318f,  2.3871f,29.6441f,18.9152f,  
	3.8805f,30.5725f,19.8692f,  4.7522f,29.6122f,19.6528f,  6.1712f,30.3915f,20.8281f,  7.0298f,29.3988f,20.6097f,  8.3456f,30.0172f,21.9839f,  
	9.1871f,28.9971f,21.7612f,  10.3733f,29.4464f,23.3081f,  11.1983f,28.4021f,23.0800f,  12.2308f,28.6776f,24.7711f,  13.0374f,27.6139f,24.5359f,  
	13.8952f,27.7145f,26.3404f,  14.6827f,26.6358f,26.0970f,  15.3482f,26.5637f,27.9822f,  16.1169f,25.4740f,27.7300f,  16.5761f,25.2345f,29.6623f,  
	17.3266f,24.1373f,29.4010f,  17.5696f,23.7386f,31.3463f,  18.3025f,22.6373f,31.0760f,  18.3232f,22.0904f,33.0002f,  19.0395f,20.9880f,32.7206f,  
	18.8364f,20.3062f,34.5903f,  19.5373f,19.2050f,34.3024f,  19.1124f,18.4036f,36.0854f,  19.7991f,17.3058f,35.7895f,  19.1589f,16.4019f,37.4555f,  
	19.8330f,15.3085f,37.1521f,  18.9879f,14.3207f,38.6726f,  19.6504f,13.2331f,38.3628f,  18.6141f,12.1809f,39.7123f,  19.2668f,11.0992f,39.3966f,  
	18.0567f,10.0030f,40.5525f,  18.7001f,8.9281f,40.2319f,  17.3365f,7.8085f,41.1750f,  17.9723f,6.7394f,40.8500f,  16.4782f,5.6174f,41.5646f,  
	17.1074f,4.5539f,41.2357f,  15.5080f,3.4500f,41.7104f,  16.1308f,2.3923f,41.3784f,  14.4531f,1.3266f,41.6055f,  15.0706f,0.2735f,41.2704f,  
	13.3430f,-0.7349f,41.2467f,  13.9561f,-1.7846f,40.9085f,  12.2078f,-2.7176f,40.6347f,  12.8166f,-3.7642f,40.2934f,  11.0771f,-4.6051f,39.7747f,  
	11.6819f,-5.6491f,39.4302f,  9.9805f,-6.3827f,38.6758f,  10.5817f,-7.4248f,38.3276f,  8.9471f,-8.0378f,37.3506f,  9.5449f,-9.0782f,36.9984f,  
	8.0047f,-9.5595f,35.8158f,  8.5992f,-10.5983f,35.4588f,  7.1795f,-10.9387f,34.0912f,  7.7708f,-11.9758f,33.7288f,  6.4954f,-12.1686f,32.1997f,  
	7.0838f,-13.2033f,31.8313f,  5.9742f,-13.2443f,30.1671f,  6.5600f,-14.2761f,29.7919f,  -24.5119f,28.8769f,45.5360f,  -23.6389f,28.1908f,45.2666f,  
	-24.3298f,27.3323f,43.5970f,  -23.5111f,26.6760f,43.2997f,  -24.1156f,26.0651f,41.4932f,  -23.3364f,25.4379f,41.1740f,  -23.8293f,25.0812f,39.2683f,  
	-23.0757f,24.4800f,38.9336f,  -23.4340f,24.3773f,36.9681f,  -22.6940f,23.7976f,36.6237f,  -22.8979f,23.9425f,34.6388f,  -22.1606f,23.3780f,34.2909f,  
	-22.1942f,23.7584f,32.3271f,  -21.4512f,23.2019f,31.9796f,  -21.3029f,23.8006f,30.0765f,  -20.5485f,23.2451f,29.7343f,  -20.2117f,24.0404f,27.9298f,  
	-19.4404f,23.4765f,27.5965f,  -18.9133f,24.4430f,25.9261f,  -18.1239f,23.8655f,25.6034f,  -17.4100f,24.9748f,24.0994f,  -16.6010f,24.3752f,23.7898f,  
	-15.7087f,25.5970f,22.4809f,  -14.8817f,24.9694f,22.1857f,  -13.8243f,26.2723f,21.0964f,  -12.9802f,25.6115f,20.8149f,  -11.7751f,26.9634f,19.9651f,  
	-10.9175f,26.2663f,19.6971f,  -9.5861f,27.6356f,19.1016f,  -8.7184f,26.8986f,18.8462f,  -7.2855f,28.2547f,18.5149f,  -6.4111f,27.4760f,18.2700f,  
	-4.9039f,28.7900f,18.2078f,  -4.0277f,27.9705f,17.9717f,  -2.4755f,29.2154f,18.1781f,  -1.6002f,28.3550f,17.9483f,  -0.0333f,29.5066f,18.4180f,  
	0.8365f,28.6073f,18.1924f,  2.3871f,29.6441f,18.9152f,  3.2486f,28.7083f,18.6915f,  4.7522f,29.6122f,19.6528f,  5.6030f,28.6428f,19.4286f,  
	7.0298f,29.3988f,20.6097f,  7.8656f,28.4004f,20.3829f,  9.1871f,28.9971f,21.7612f,  10.0085f,27.9724f,21.5302f,  11.1983f,28.4021f,23.0800f,  
	12.0030f,27.3558f,22.8431f,  13.0374f,27.6139f,24.5359f,  13.8245f,26.5503f,24.2920f,  14.6827f,26.6358f,26.0970f,  15.4521f,25.5587f,25.8451f,  
	16.1169f,25.4740f,27.7300f,  16.8689f,24.3870f,27.4696f,  17.3266f,24.1373f,29.4010f,  18.0618f,23.0438f,29.1320f,  18.3025f,22.6373f,31.0760f,  
	19.0214f,21.5406f,30.7980f,  19.0395f,20.9880f,32.7206f,  19.7434f,19.8905f,32.4343f,  19.5373f,19.2050f,34.3024f,  20.2270f,18.1090f,34.0080f,  
	19.7991f,17.3058f,35.7895f,  20.4763f,16.2128f,35.4876f,  19.8330f,15.3085f,37.1521f,  20.4986f,14.2200f,36.8434f,  19.6504f,13.2331f,38.3628f,  
	20.3060f,12.1495f,38.0479f,  19.2668f,11.0992f,39.3966f,  19.9131f,10.0215f,39.0765f,  18.7001f,8.9281f,40.2319f,  19.3388f,7.8555f,39.9069f,  
	17.9723f,6.7394f,40.8500f,  18.6043f,5.6721f,40.5207f,  17.1074f,4.5539f,41.2357f,  17.7329f,3.4920f,40.9030f,  16.1308f,2.3923f,41.3784f,  
	16.7510f,1.3349f,41.0423f,  15.0706f,0.2735f,41.2704f,  15.6864f,-0.7807f,40.9310f,  13.9561f,-1.7846f,40.9085f,  14.5677f,-2.8359f,40.5657f,  
	12.8166f,-3.7642f,40.2934f,  13.4242f,-4.8131f,39.9473f,  11.6819f,-5.6491f,39.4302f,  12.2860f,-6.6962f,39.0804f,  10.5817f,-7.4248f,38.3276f,  
	11.1824f,-8.4704f,37.9737f,  9.5449f,-9.0782f,36.9984f,  10.1425f,-10.1227f,36.6396f,  8.5992f,-10.5983f,35.4588f,  9.1938f,-11.6414f,35.0946f,  
	7.7708f,-11.9758f,33.7288f,  8.3625f,-13.0171f,33.3584f,  7.0838f,-13.2033f,31.8313f,  7.6729f,-14.2424f,31.4539f,  6.5600f,-14.2761f,29.7919f,  
	7.1466f,-15.3117f,29.4071f,  -23.6389f,28.1908f,45.2666f,  -22.7905f,27.4853f,44.9813f,  -23.5111f,26.6760f,43.2997f,  -22.7115f,26.0015f,42.9890f,  
	-23.3364f,25.4379f,41.1740f,  -22.5713f,24.7928f,40.8441f,  -23.0757f,24.4800f,38.9336f,  -22.3325f,23.8608f,38.5902f,  -22.6940f,23.7976f,36.6237f,  
	-21.9608f,23.1988f,36.2733f,  -22.1606f,23.3780f,34.2909f,  -21.4283f,22.7932f,33.9378f,  -21.4512f,23.2019f,31.9796f,  -20.7128f,22.6245f,31.6291f,  
	-20.5485f,23.2451f,29.7343f,  -19.7973f,22.6661f,29.3903f,  -19.4404f,23.4765f,27.5965f,  -18.6742f,22.8904f,27.2612f,  -18.1239f,23.8655f,25.6034f,  
	-17.3401f,23.2636f,25.2798f,  -16.6010f,24.3752f,23.7898f,  -15.7999f,23.7511f,23.4798f,  -14.8817f,24.9694f,22.1857f,  -14.0635f,24.3174f,21.8891f,  
	-12.9802f,25.6115f,20.8149f,  -12.1476f,24.9277f,20.5320f,  -10.9175f,26.2663f,19.6971f,  -10.0734f,25.5463f,19.4272f,  -8.7184f,26.8986f,18.8462f,  
	-7.8657f,26.1398f,18.5876f,  -6.4111f,27.4760f,18.2700f,  -5.5544f,26.6786f,18.0215f,  -4.0277f,27.9705f,17.9717f,  -3.1695f,27.1335f,17.7308f,  
	-1.6002f,28.3550f,17.9483f,  -0.7450f,27.4797f,17.7131f,  0.8365f,28.6073f,18.1924f,  1.6859f,27.6955f,17.9606f,  3.2486f,28.7083f,18.6915f,  
	4.0898f,27.7622f,18.4608f,  5.6030f,28.6428f,19.4286f,  6.4315f,27.6664f,19.1968f,  7.8656f,28.4004f,20.3829f,  8.6817f,27.3961f,20.1483f,  
	10.0085f,27.9724f,21.5302f,  10.8097f,26.9446f,21.2910f,  12.0030f,27.3558f,22.8431f,  12.7881f,26.3084f,22.5980f,  13.8245f,26.5503f,24.2920f,  
	14.5931f,25.4872f,24.0398f,  15.4521f,25.5587f,25.8451f,  16.2045f,24.4835f,25.5851f,  16.8689f,24.3870f,27.4696f,  17.6052f,23.3030f,27.2016f,  
	18.0618f,23.0438f,29.1320f,  18.7824f,21.9544f,28.8554f,  19.0214f,21.5406f,30.7980f,  19.7274f,20.4483f,30.5132f,  19.7434f,19.8905f,32.4343f,  
	20.4355f,18.7980f,32.1415f,  20.2270f,18.1090f,34.0080f,  20.9066f,17.0179f,33.7075f,  20.4763f,16.2128f,35.4876f,  21.1444f,15.1247f,35.1801f,  
	20.4986f,14.2200f,36.8434f,  21.1567f,13.1358f,36.5294f,  20.3060f,12.1495f,38.0479f,  20.9547f,11.0701f,37.7284f,  19.9131f,10.0215f,39.0765f,  
	20.5541f,8.9466f,38.7517f,  19.3388f,7.8555f,39.9069f,  19.9730f,6.7853f,39.5776f,  18.6043f,5.6721f,40.5207f,  19.2320f,4.6069f,40.1875f,  
	17.7329f,3.4920f,40.9030f,  18.3553f,2.4310f,40.5662f,  16.7510f,1.3349f,41.0423f,  17.3690f,0.2768f,40.7019f,  15.6864f,-0.7807f,40.9310f,  
	16.3003f,-1.8361f,40.5871f,  14.5677f,-2.8359f,40.5657f,  15.1777f,-3.8890f,40.2184f,  13.4242f,-4.8131f,39.9473f,  14.0309f,-5.8646f,39.5961f,  
	12.2860f,-6.6962f,39.0804f,  12.8894f,-7.7465f,38.7249f,  11.1824f,-8.4704f,37.9737f,  11.7828f,-9.5199f,37.6133f,  10.1425f,-10.1227f,36.6396f,  
	10.7400f,-11.1712f,36.2737f,  9.1938f,-11.6414f,35.0946f,  9.7886f,-12.6887f,34.7225f,  8.3625f,-13.0171f,33.3584f,  8.9548f,-14.0628f,32.9793f,  
	7.6729f,-14.2424f,31.4539f,  8.2627f,-15.2854f,31.0671f,  7.1466f,-15.3117f,29.4071f,  7.7340f,-16.3507f,29.0122f,  -22.7905f,27.4853f,44.9813f,  
	-21.9652f,26.7622f,44.6820f,  -22.7115f,26.0015f,42.9890f,  -21.9295f,25.3102f,42.6668f,  -22.5713f,24.7928f,40.8441f,  -21.8196f,24.1310f,40.5049f,  
	-22.3325f,23.8608f,38.5902f,  -21.5984f,23.2241f,38.2406f,  -21.9608f,23.1988f,36.2733f,  -21.2345f,22.5817f,35.9177f,  -21.4283f,22.7932f,33.9378f,  
	-20.7017f,22.1894f,33.5817f,  -20.7128f,22.6245f,31.6291f,  -19.9781f,22.0255f,31.2769f,  -19.7973f,22.6661f,29.3903f,  -19.0514f,22.0662f,29.0447f,  
	-18.6742f,22.8904f,27.2612f,  -17.9131f,22.2811f,26.9256f,  -17.3401f,23.2636f,25.2798f,  -16.5631f,22.6381f,24.9566f,  -15.7999f,23.7511f,23.4798f,  
	-15.0065f,23.1033f,23.1693f,  -14.0635f,24.3174f,21.8891f,  -13.2556f,23.6428f,21.5919f,  -12.1476f,24.9277f,20.5320f,  -11.3273f,24.2211f,20.2480f,  
	-10.0734f,25.5463f,19.4272f,  -9.2429f,24.8044f,19.1552f,  -7.8657f,26.1398f,18.5876f,  -7.0293f,25.3619f,18.3266f,  -5.5544f,26.6786f,18.0215f,  
	-4.7144f,25.8630f,17.7692f,  -3.1695f,27.1335f,17.7308f,  -2.3302f,26.2808f,17.4856f,  -0.7450f,27.4797f,17.7131f,  0.0909f,26.5909f,17.4726f,  
	1.6859f,27.6955f,17.9606f,  2.5159f,26.7721f,17.7227f,  4.0898f,27.7622f,18.4608f,  4.9093f,26.8077f,18.2233f,  6.4315f,27.6664f,19.1968f,  
	7.2407f,26.6829f,18.9579f,  8.6817f,27.3961f,20.1483f,  9.4779f,26.3873f,19.9062f,  10.8097f,26.9446f,21.2910f,  11.5913f,25.9145f,21.0441f,  
	12.7881f,26.3084f,22.5980f,  13.5546f,25.2606f,22.3450f,  14.5931f,25.4872f,24.0398f,  15.3445f,24.4250f,23.7798f,  16.2045f,24.4835f,25.5851f,  
	16.9407f,23.4103f,25.3177f,  17.6052f,23.3030f,27.2016f,  18.3264f,22.2225f,26.9260f,  18.7824f,21.9544f,28.8554f,  19.4894f,20.8690f,28.5719f,  
	19.7274f,20.4483f,30.5132f,  20.4210f,19.3608f,30.2218f,  20.4355f,18.7980f,32.1415f,  21.1168f,17.7103f,31.8424f,  20.9066f,17.0179f,33.7075f,  
	21.5765f,15.9318f,33.4013f,  21.1444f,15.1247f,35.1801f,  21.8042f,14.0412f,34.8672f,  21.1567f,13.1358f,36.5294f,  21.8072f,12.0560f,36.2107f,  
	20.9547f,11.0701f,37.7284f,  21.5976f,9.9941f,37.4041f,  20.5541f,8.9466f,38.7517f,  21.1900f,7.8747f,38.4225f,  19.9730f,6.7853f,39.5776f,  
	20.6025f,5.7178f,39.2442f,  19.2320f,4.6069f,40.1875f,  19.8562f,3.5432f,39.8503f,  18.3553f,2.4310f,40.5662f,  18.9751f,1.3700f,40.2250f,  
	17.3690f,0.2768f,40.7019f,  17.9848f,-0.7817f,40.3570f,  16.3003f,-1.8361f,40.5871f,  16.9123f,-2.8925f,40.2386f,  15.1777f,-3.8890f,40.2184f,  
	15.7865f,-4.9442f,39.8659f,  14.0309f,-5.8646f,39.5961f,  14.6365f,-6.9188f,39.2393f,  12.8894f,-7.7465f,38.7249f,  13.4922f,-8.8003f,38.3632f,  
	11.7828f,-9.5199f,37.6133f,  12.3829f,-10.5732f,37.2460f,  10.7400f,-11.1712f,36.2737f,  11.3375f,-12.2239f,35.9002f,  9.7886f,-12.6887f,34.7225f,  
	10.3836f,-13.7406f,34.3419f,  8.9548f,-14.0628f,32.9793f,  9.5475f,-15.1127f,32.5908f,  8.2627f,-15.2854f,31.0671f,  8.8531f,-16.3321f,30.6703f,  
	7.7340f,-16.3507f,29.0122f,  8.3229f,-17.3938f,28.6056f,  -21.9652f,26.7622f,44.6820f,  -21.1612f,26.0234f,44.3707f,  -21.9295f,25.3102f,42.6668f,  
	-21.1640f,24.6036f,42.3350f,  -21.8196f,24.1310f,40.5049f,  -21.0795f,23.4533f,40.1590f,  -21.5984f,23.2241f,38.2406f,  -20.8733f,22.5707f,37.8853f,  
	-21.2345f,22.5817f,35.9177f,  -20.5153f,21.9470f,35.5587f,  -20.7017f,22.1894f,33.5817f,  -19.9796f,21.5656f,33.2238f,  -19.9781f,22.0255f,31.2769f,  
	-19.2492f,21.4071f,30.9231f,  -19.0514f,22.0662f,29.0447f,  -18.3105f,21.4447f,28.6989f,  -17.9131f,22.2811f,26.9256f,  -17.1584f,21.6495f,26.5908f,  
	-16.5631f,22.6381f,24.9566f,  -15.7931f,21.9899f,24.6334f,  -15.0065f,23.1033f,23.1693f,  -14.2224f,22.4335f,22.8588f,  -13.2556f,23.6428f,21.5919f,  
	-12.4585f,22.9456f,21.2946f,  -11.3273f,24.2211f,20.2480f,  -10.5190f,23.4925f,19.9630f,  -9.2429f,24.8044f,19.1552f,  -8.4273f,24.0430f,18.8818f,  
	-7.0293f,25.3619f,18.3266f,  -6.2083f,24.5653f,18.0628f,  -4.7144f,25.8630f,17.7692f,  -3.8919f,25.0309f,17.5136f,  -2.3302f,26.2808f,17.4856f,  
	-1.5090f,25.4134f,17.2360f,  0.0909f,26.5909f,17.4726f,  0.9084f,25.6893f,17.2270f,  2.5159f,26.7721f,17.7227f,  3.3250f,25.8391f,17.4790f,  
	4.9093f,26.8077f,18.2233f,  5.7101f,25.8449f,17.9795f,  7.2407f,26.6829f,18.9579f,  8.0304f,25.6936f,18.7122f,  9.4779f,26.3873f,19.9062f,  
	10.2547f,25.3752f,19.6568f,  11.5913f,25.9145f,21.0441f,  12.3544f,24.8828f,20.7897f,  13.5546f,25.2606f,22.3450f,  14.3037f,24.2127f,22.0846f,  
	15.3445f,24.4250f,23.7798f,  16.0795f,23.3641f,23.5126f,  16.9407f,23.4103f,25.3177f,  17.6614f,22.3401f,25.0430f,  18.3264f,22.2225f,26.9260f,  
	19.0336f,21.1453f,26.6437f,  19.4894f,20.8690f,28.5719f,  20.1836f,19.7879f,28.2819f,  20.4210f,19.3608f,30.2218f,  21.1031f,18.2779f,29.9241f,  
	21.1168f,17.7103f,31.8424f,  21.7877f,16.6274f,31.5375f,  21.5765f,15.9318f,33.4013f,  22.2375f,14.8503f,33.0896f,  21.8042f,14.0412f,34.8672f,  
	22.4560f,12.9623f,34.5494f,  21.8072f,12.0560f,36.2107f,  22.4513f,10.9800f,35.8871f,  21.5976f,9.9941f,37.4041f,  22.2347f,8.9214f,37.0753f,  
	21.1900f,7.8747f,38.4225f,  21.8208f,6.8058f,38.0893f,  20.6025f,5.7178f,39.2442f,  21.2279f,4.6523f,38.9067f,  19.8562f,3.5432f,39.8503f,  
	20.4774f,2.4801f,39.5086f,  18.9751f,1.3700f,40.2250f,  19.5923f,0.3091f,39.8795f,  17.9848f,-0.7817f,40.3570f,  18.5983f,-1.8408f,40.0077f,  
	16.9123f,-2.8925f,40.2386f,  17.5227f,-3.9506f,39.8851f,  15.7865f,-4.9442f,39.8659f,  16.3939f,-6.0016f,39.5081f,  14.6365f,-6.9188f,39.2393f,  
	15.2413f,-7.9762f,38.8765f,  13.4922f,-8.8003f,38.3632f,  14.0944f,-9.8577f,37.9948f,  12.3829f,-10.5732f,37.2460f,  12.9828f,-11.6306f,36.8713f,  
	11.3375f,-12.2239f,35.9002f,  11.9351f,-13.2811f,35.5183f,  10.3836f,-13.7406f,34.3419f,  10.9790f,-14.7966f,33.9521f,  9.5475f,-15.1127f,32.5908f,  
	10.1407f,-16.1665f,32.1924f,  8.8531f,-16.3321f,30.6703f,  9.4448f,-17.3835f,30.2619f,  8.3229f,-17.3938f,28.6056f,  8.9128f,-18.4398f,28.1875f,  
	-21.1612f,26.0234f,44.3707f,  -20.3770f,25.2705f,44.0490f,  -21.1640f,24.6036f,42.3350f,  -20.4129f,23.8826f,41.9957f,  -21.0795f,23.4533f,40.1590f,  
	-20.3508f,22.7607f,39.8070f,  -20.8733f,22.5707f,37.8853f,  -20.1569f,21.9016f,37.5263f,  -20.5153f,21.9470f,35.5587f,  -19.8018f,21.2944f,35.1979f,  
	-19.9796f,21.5656f,33.2238f,  -19.2640f,20.9242f,32.8644f,  -19.2492f,21.4071f,30.9231f,  -18.5253f,20.7687f,30.5695f,  -18.3105f,21.4447f,28.6989f,  
	-17.5754f,20.8023f,28.3544f,  -17.1584f,21.6495f,26.5908f,  -16.4099f,20.9963f,26.2565f,  -15.7931f,21.9899f,24.6334f,  -15.0315f,21.3205f,24.3110f,  
	-14.2224f,22.4335f,22.8588f,  -13.4479f,21.7416f,22.5491f,  -12.4585f,22.9456f,21.2946f,  -11.6723f,22.2266f,20.9973f,  -10.5190f,23.4925f,19.9630f,  
	-9.7244f,22.7444f,19.6776f,  -8.4273f,24.0430f,18.8818f,  -7.6258f,23.2625f,18.6066f,  -6.2083f,24.5653f,18.0628f,  -5.4036f,23.7515f,17.7968f,  
	-3.8919f,25.0309f,17.5136f,  -3.0865f,24.1833f,17.2547f,  -1.5090f,25.4134f,17.2360f,  -0.7052f,24.5325f,16.9822f,  0.9084f,25.6893f,17.2270f,  
	1.7058f,24.7771f,16.9766f,  3.3250f,25.8391f,17.4790f,  4.1162f,24.8967f,17.2298f,  5.7101f,25.8449f,17.9795f,  6.4920f,24.8752f,17.7296f,  
	8.0304f,25.6936f,18.7122f,  8.8009f,24.6998f,18.4599f,  10.2547f,25.3752f,19.6568f,  11.0130f,24.3604f,19.4006f,  12.3544f,24.8828f,20.7897f,  
	13.1001f,23.8500f,20.5284f,  14.3037f,24.2127f,22.0846f,  15.0364f,23.1650f,21.8172f,  16.0795f,23.3641f,23.5126f,  16.7987f,22.3053f,23.2384f,  
	17.6614f,22.3401f,25.0430f,  18.3678f,21.2725f,24.7615f,  19.0336f,21.1453f,26.6437f,  19.7275f,20.0720f,26.3549f,  20.1836f,19.7879f,28.2819f,  
	20.8657f,18.7111f,27.9856f,  21.1031f,18.2779f,29.9241f,  21.7743f,17.1996f,29.6207f,  21.7877f,16.6274f,31.5375f,  22.4491f,15.5492f,31.2271f,  
	22.2375f,14.8503f,33.0896f,  22.8899f,13.7735f,32.7729f,  22.4560f,12.9623f,34.5494f,  23.1007f,11.8874f,34.2267f,  22.4513f,10.9800f,35.8871f,  
	23.0891f,9.9077f,35.5589f,  22.2347f,8.9214f,37.0753f,  22.8662f,7.8522f,36.7423f,  21.8208f,6.8058f,38.0893f,  22.4471f,5.7395f,37.7518f,  
	21.2279f,4.6523f,38.9067f,  21.8499f,3.5880f,38.5649f,  20.4774f,2.4801f,39.5086f,  21.0955f,1.4176f,39.1627f,  19.5923f,0.3091f,39.8795f,  
	20.2070f,-0.7519f,39.5296f,  18.5983f,-1.8408f,40.0077f,  19.2100f,-2.9010f,39.6535f,  17.5227f,-3.9506f,39.8851f,  18.1316f,-5.0105f,39.5265f,  
	16.3939f,-6.0016f,39.5081f,  17.0003f,-7.0617f,39.1444f,  15.2413f,-7.9762f,38.8765f,  15.8453f,-9.0368f,38.5073f,  14.0944f,-9.8577f,37.9948f,  
	14.6963f,-10.9189f,37.6193f,  12.9828f,-11.6306f,36.8713f,  13.5825f,-12.6923f,36.4886f,  11.9351f,-13.2811f,35.5183f,  12.5327f,-14.3424f,35.1276f,  
	10.9790f,-14.7966f,33.9521f,  11.5746f,-15.8567f,33.5526f,  10.1407f,-16.1665f,32.1924f,  10.7349f,-17.2253f,31.7827f,  9.4448f,-17.3835f,30.2619f,  
	10.0372f,-18.4380f,29.8421f,  8.9128f,-18.4398f,28.1875f,  9.5042f,-19.4892f,27.7564f,  -20.3770f,25.2705f,44.0490f,  -19.6101f,24.5045f,43.7193f,  
	-20.4129f,23.8826f,41.9957f,  -19.6758f,23.1482f,41.6499f,  -20.3508f,22.7607f,39.8070f,  -19.6327f,22.0540f,39.4509f,  -20.1569f,21.9016f,37.5263f,  
	-19.4478f,21.2165f,37.1653f,  -19.8018f,21.2944f,35.1979f,  -19.0957f,20.6259f,34.8354f,  -19.2640f,20.9242f,32.8644f,  -18.5539f,20.2645f,32.5052f,  
	-18.5253f,20.7687f,30.5695f,  -17.8070f,20.1108f,30.2173f,  -17.5754f,20.8023f,28.3544f,  -16.8462f,20.1395f,28.0108f,  -16.4099f,20.9963f,26.2565f,  
	-15.6690f,20.3229f,25.9236f,  -15.0315f,21.3205f,24.3110f,  -14.2783f,20.6298f,23.9901f,  -13.4479f,21.7416f,22.5491f,  -12.6829f,21.0284f,22.2402f,  
	-11.6723f,22.2266f,20.9973f,  -10.8983f,21.4882f,20.7003f,  -9.7244f,22.7444f,19.6776f,  -8.9425f,21.9770f,19.3913f,  -7.6258f,23.2625f,18.6066f,  
	-6.8392f,22.4644f,18.3302f,  -5.4036f,23.7515f,17.7968f,  -4.6147f,22.9216f,17.5284f,  -3.0865f,24.1833f,17.2547f,  -2.2975f,23.3214f,16.9926f,  
	-0.7052f,24.5325f,16.9822f,  0.0794f,23.6398f,16.7248f,  1.7058f,24.7771f,16.9766f,  2.4861f,23.8543f,16.7215f,  4.1162f,24.8967f,17.2298f,  
	4.8889f,23.9462f,16.9752f,  6.4920f,24.8752f,17.7296f,  7.2550f,23.8998f,17.4738f,  8.8009f,24.6998f,18.4599f,  9.5533f,23.7021f,18.2013f,  
	11.0130f,24.3604f,19.4006f,  11.7540f,23.3433f,19.1379f,  13.1001f,23.8500f,20.5284f,  13.8293f,22.8165f,20.2605f,  15.0364f,23.1650f,21.8172f,  
	15.7530f,22.1186f,21.5431f,  16.7987f,22.3053f,23.2384f,  17.5033f,21.2484f,22.9576f,  18.3678f,21.2725f,24.7615f,  19.0605f,20.2081f,24.4737f,  
	19.7275f,20.0720f,26.3549f,  20.4089f,19.0025f,26.0598f,  20.8657f,18.7111f,27.9856f,  21.5366f,17.6387f,27.6834f,  21.7743f,17.1996f,29.6207f,  
	22.4355f,16.1259f,29.3116f,  22.4491f,15.5492f,31.2271f,  23.1015f,14.4757f,30.9117f,  22.8899f,13.7735f,32.7729f,  23.5346f,12.7009f,32.4513f,  
	23.1007f,11.8874f,34.2267f,  23.7385f,10.8166f,33.8993f,  23.0891f,9.9077f,35.5589f,  23.7208f,8.8392f,35.2265f,  22.8662f,7.8522f,36.7423f,  
	23.4927f,6.7861f,36.4052f,  22.4471f,5.7395f,37.7518f,  23.0694f,4.6750f,37.4101f,  21.8499f,3.5880f,38.5649f,  22.4685f,2.5250f,38.2189f,  
	21.0955f,1.4176f,39.1627f,  21.7107f,0.3558f,38.8125f,  20.2070f,-0.7519f,39.5296f,  20.8193f,-1.8133f,39.1751f,  19.2100f,-2.9010f,39.6535f,  
	19.8197f,-3.9624f,39.2945f,  18.1316f,-5.0105f,39.5265f,  18.7391f,-6.0725f,39.1624f,  17.0003f,-7.0617f,39.1444f,  17.6056f,-8.1247f,38.7747f,  
	15.8453f,-9.0368f,38.5073f,  16.4487f,-10.1010f,38.1312f,  14.6963f,-10.9189f,37.6193f,  15.2977f,-11.9842f,37.2360f,  13.5825f,-12.6923f,36.4886f,  
	14.1820f,-13.7581f,36.0973f,  12.5327f,-14.3424f,35.1276f,  13.1304f,-15.4080f,34.7274f,  11.5746f,-15.8567f,33.5526f,  12.1710f,-16.9221f,33.1420f,  
	10.7349f,-17.2253f,31.7827f,  11.3296f,-18.2875f,31.3618f,  10.0372f,-18.4380f,29.8421f,  10.6309f,-19.4964f,29.4095f,  9.5042f,-19.4892f,27.7564f,  
	10.0969f,-20.5414f,27.3118f,  -19.6101f,24.5045f,43.7193f,  -18.8601f,23.7268f,43.3825f,  -19.6758f,23.1482f,41.6499f,  -18.9515f,22.4015f,41.2995f,  
	-19.6327f,22.0540f,39.4509f,  -18.9235f,21.3333f,39.0924f,  -19.4478f,21.2165f,37.1653f,  -18.7472f,20.5172f,36.8023f,  -19.0957f,20.6259f,34.8354f,  
	-18.3957f,19.9409f,34.4730f,  -18.5539f,20.2645f,32.5052f,  -17.8494f,19.5870f,32.1475f,  -17.8070f,20.1108f,30.2173f,  -17.0946f,19.4341f,29.8663f,  
	-16.8462f,20.1395f,28.0108f,  -16.1241f,19.4577f,27.6690f,  -15.6690f,20.3229f,25.9236f,  -14.9357f,19.6291f,25.5928f,  -14.2783f,20.6298f,23.9901f,  
	-13.5337f,19.9184f,23.6707f,  -12.6829f,21.0284f,22.2402f,  -11.9291f,20.2961f,21.9326f,  -10.8983f,21.4882f,20.7003f,  -10.1359f,20.7307f,20.4035f,  
	-8.9425f,21.9770f,19.3913f,  -8.1743f,21.1918f,19.1048f,  -6.8392f,22.4644f,18.3302f,  -6.0673f,21.6498f,18.0525f,  -4.6147f,22.9216f,17.5284f,  
	-3.8410f,22.0767f,17.2578f,  -2.2975f,23.3214f,16.9926f,  -1.5264f,22.4467f,16.7278f,  0.0794f,23.6398f,16.7248f,  0.8477f,22.7357f,16.4634f,  
	2.4861f,23.8543f,16.7215f,  3.2486f,22.9224f,16.4618f,  4.8889f,23.9462f,16.9752f,  5.6434f,22.9888f,16.7155f,  7.2550f,23.8998f,17.4738f,  
	8.0004f,22.9194f,17.2125f,  9.5533f,23.7021f,18.2013f,  10.2886f,22.7012f,17.9369f,  11.7540f,23.3433f,19.1379f,  12.4787f,22.3245f,18.8691f,  
	13.8293f,22.8165f,20.2605f,  14.5423f,21.7834f,19.9862f,  15.7530f,22.1186f,21.5431f,  16.4549f,21.0734f,21.2628f,  17.5033f,21.2484f,22.9576f,  
	18.1939f,20.1940f,22.6707f,  19.0605f,20.2081f,24.4737f,  19.7403f,19.1472f,24.1799f,  20.4089f,19.0025f,26.0598f,  21.0785f,17.9371f,25.7591f,  
	21.5366f,17.6387f,27.6834f,  22.1968f,16.5707f,27.3758f,  22.4355f,16.1259f,29.3116f,  23.0872f,15.0568f,28.9975f,  23.1015f,14.4757f,30.9117f,  
	23.7456f,13.4064f,30.5912f,  23.5346f,12.7009f,32.4513f,  24.1720f,11.6326f,32.1250f,  23.7385f,10.8166f,33.8993f,  24.3699f,9.7498f,33.5677f,  
	23.7208f,8.8392f,35.2265f,  24.3470f,7.7741f,34.8900f,  23.4927f,6.7861f,36.4052f,  24.1148f,5.7222f,36.0638f,  23.0694f,4.6750f,37.4101f,  
	23.6878f,3.6122f,37.0642f,  22.4685f,2.5250f,38.2189f,  23.0837f,1.4631f,37.8687f,  21.7107f,0.3558f,38.8125f,  22.3232f,-0.7060f,38.4579f,  
	20.8193f,-1.8133f,39.1751f,  21.4294f,-2.8753f,38.8158f,  19.8197f,-3.9624f,39.2945f,  20.4278f,-5.0255f,38.9301f,  18.7391f,-6.0725f,39.1624f,  
	19.3452f,-7.1369f,38.7925f,  17.6056f,-8.1247f,38.7747f,  18.2101f,-9.1909f,38.3984f,  16.4487f,-10.1010f,38.1312f,  17.0515f,-11.1689f,37.7478f,  
	15.2977f,-11.9842f,37.2360f,  15.8988f,-13.0534f,36.8445f,  14.1820f,-13.7581f,36.0973f,  14.7815f,-14.8281f,35.6968f,  13.1304f,-15.4080f,34.7274f,  
	13.7286f,-16.4789f,34.3165f,  12.1710f,-16.9221f,33.1420f,  12.7676f,-17.9910f,32.7205f,  11.3296f,-18.2875f,31.3618f,  11.9252f,-19.3541f,30.9282f,  
	10.6309f,-19.4964f,29.4095f,  11.2256f,-20.5582f,28.9634f,  10.0969f,-20.5414f,27.3118f,  10.6913f,-21.5966f,26.8524f,  -18.8601f,23.7268f,43.3825f,  
	-18.1250f,22.9382f,43.0405f,  -18.9515f,22.4015f,41.2995f,  -18.2380f,21.6426f,40.9462f,  -18.9235f,21.3333f,39.0924f,  -18.2243f,20.6000f,38.7316f,  
	-18.7472f,20.5172f,36.8023f,  -18.0536f,19.8033f,36.4393f,  -18.3957f,19.9409f,34.4730f,  -17.7018f,19.2397f,34.1121f,  -17.8494f,19.5870f,32.1475f,  
	-17.1509f,18.8923f,31.7912f,  -17.0946f,19.4341f,29.8663f,  -16.3889f,18.7398f,29.5174f,  -16.1241f,19.4577f,27.6690f,  -15.4088f,18.7567f,27.3298f,  
	-14.9357f,19.6291f,25.5928f,  -14.2101f,18.9155f,25.2641f,  -13.5337f,19.9184f,23.6707f,  -12.7991f,19.1884f,23.3532f,  -11.9291f,20.2961f,21.9326f,  
	-11.1857f,19.5450f,21.6258f,  -10.1359f,20.7307f,20.4035f,  -9.3860f,19.9553f,20.1073f,  -8.1743f,21.1918f,19.1048f,  -7.4195f,20.3898f,18.8179f,  
	-6.0673f,21.6498f,18.0525f,  -5.3095f,20.8197f,17.7734f,  -3.8410f,22.0767f,17.2578f,  -3.0842f,21.2183f,16.9855f,  -1.5264f,22.4467f,16.7278f,  
	-0.7708f,21.5599f,16.4599f,  0.8477f,22.7357f,16.4634f,  1.5992f,21.8214f,16.1983f,  3.2486f,22.9224f,16.4618f,  3.9937f,21.9826f,16.1979f,  
	5.6434f,22.9888f,16.7155f,  6.3807f,22.0255f,16.4510f,  8.0004f,22.9194f,17.2125f,  8.7290f,21.9346f,16.9459f,  10.2886f,22.7012f,17.9369f,  
	11.0079f,21.6974f,17.6668f,  12.4787f,22.3245f,18.8691f,  13.1872f,21.3052f,18.5944f,  14.5423f,21.7834f,19.9862f,  15.2406f,20.7505f,19.7061f,  
	16.4549f,21.0734f,21.2628f,  17.1427f,20.0299f,20.9766f,  18.1939f,20.1940f,22.6707f,  18.8714f,19.1425f,22.3779f,  19.7403f,19.1472f,24.1799f,  
	20.4081f,18.0897f,23.8804f,  21.0785f,17.9371f,25.7591f,  21.7372f,16.8758f,25.4528f,  22.1968f,16.5707f,27.3758f,  22.8472f,15.5071f,27.0631f,  
	23.0872f,15.0568f,28.9975f,  23.7302f,13.9920f,28.6784f,  23.7456f,13.4064f,30.5912f,  24.3820f,12.3417f,30.2661f,  24.1720f,11.6326f,32.1250f,  
	24.8025f,10.5685f,31.7944f,  24.3699f,9.7498f,33.5677f,  24.9953f,8.6868f,33.2320f,  24.3470f,7.7741f,34.8900f,  24.9684f,6.7119f,34.5492f,  
	24.1148f,5.7222f,36.0638f,  24.7327f,4.6606f,35.7184f,  23.6878f,3.6122f,37.0642f,  24.3026f,2.5511f,36.7143f,  23.0837f,1.4631f,37.8687f,  
	23.6960f,0.4018f,37.5142f,  22.3232f,-0.7060f,38.4579f,  22.9332f,-1.7679f,38.0987f,  21.4294f,-2.8753f,38.8158f,  22.0377f,-3.9385f,38.4515f,  
	20.4278f,-5.0255f,38.9301f,  21.0343f,-6.0904f,38.5603f,  19.3452f,-7.1369f,38.7925f,  19.9503f,-8.2042f,38.4163f,  18.2101f,-9.1909f,38.3984f,  
	18.8137f,-10.2605f,38.0152f,  17.0515f,-11.1689f,37.7478f,  17.6535f,-12.2406f,37.3566f,  15.8988f,-13.0534f,36.8445f,  16.4995f,-14.1268f,36.4443f,  
	14.7815f,-14.8281f,35.6968f,  15.3811f,-15.9034f,35.2862f,  13.7286f,-16.4789f,34.3165f,  14.3268f,-17.5535f,33.8951f,  12.7676f,-17.9910f,32.7205f,  
	13.3649f,-19.0647f,32.2867f,  11.9252f,-19.3541f,30.9282f,  12.5217f,-20.4245f,30.4815f,  11.2256f,-20.5582f,28.9634f,  11.8218f,-21.6237f,28.5027f,  
	10.6913f,-21.5966f,26.8524f,  11.2873f,-22.6538f,26.3779f,  -18.1250f,22.9382f,43.0405f,  -17.4029f,22.1393f,42.6949f,  -18.2380f,21.6426f,40.9462f,  
	-17.5361f,20.8730f,40.5901f,  -18.2243f,20.6000f,38.7316f,  -17.5333f,19.8541f,38.3703f,  -18.0536f,19.8033f,36.4393f,  -17.3667f,19.0750f,36.0775f,  
	-17.7018f,19.2397f,34.1121f,  -17.0141f,18.5232f,33.7526f,  -17.1509f,18.8923f,31.7912f,  -16.4589f,18.1814f,31.4371f,  -16.3889f,18.7398f,29.5174f,  
	-15.6894f,18.0275f,29.1714f,  -15.4088f,18.7567f,27.3298f,  -14.7004f,18.0369f,26.9932f,  -14.2101f,18.9155f,25.2641f,  -13.4934f,18.1841f,24.9379f,  
	-12.7991f,19.1884f,23.3532f,  -12.0739f,18.4401f,23.0374f,  -11.1857f,19.5450f,21.6258f,  -10.4535f,18.7761f,21.3206f,  -9.3860f,19.9553f,20.1073f,  
	-8.6482f,19.1630f,19.8117f,  -7.4195f,20.3898f,18.8179f,  -6.6777f,19.5720f,18.5306f,  -5.3095f,20.8197f,17.7734f,  -4.5672f,19.9755f,17.4937f,  
	-3.0842f,21.2183f,16.9855f,  -2.3418f,20.3471f,16.7110f,  -0.7708f,21.5599f,16.4599f,  -0.0310f,20.6620f,16.1892f,  1.5992f,21.8214f,16.1983f,  
	2.3340f,20.8983f,15.9299f,  3.9937f,21.9826f,16.1979f,  4.7221f,21.0358f,15.9299f,  6.3807f,22.0255f,16.4510f,  7.1018f,21.0568f,16.1819f,  
	8.7290f,21.9346f,16.9459f,  9.4419f,20.9461f,16.6742f,  11.0079f,21.6974f,17.6668f,  11.7110f,20.6923f,17.3913f,  13.1872f,21.3052f,18.5944f,  
	13.8811f,20.2852f,18.3142f,  15.2406f,20.7505f,19.7061f,  15.9247f,19.7186f,19.4203f,  17.1427f,20.0299f,20.9766f,  17.8171f,18.9887f,20.6847f,  
	18.8714f,19.1425f,22.3779f,  19.5366f,18.0940f,22.0797f,  20.4081f,18.0897f,23.8804f,  21.0645f,17.0361f,23.5756f,  21.7372f,16.8758f,25.4528f,  
	22.3857f,15.8186f,25.1416f,  22.8472f,15.5071f,27.0631f,  23.4885f,14.4477f,26.7455f,  23.7302f,13.9920f,28.6784f,  24.3649f,12.9318f,28.3547f,  
	24.3820f,12.3417f,30.2661f,  25.0110f,11.2813f,29.9368f,  24.8025f,10.5685f,31.7944f,  25.4266f,9.5085f,31.4597f,  24.9953f,8.6868f,33.2320f,  
	25.6154f,7.6271f,32.8922f,  24.9684f,6.7119f,34.5492f,  25.5851f,5.6524f,34.2045f,  24.7327f,4.6606f,35.7184f,  25.3465f,3.6012f,35.3691f,  
	24.3026f,2.5511f,36.7143f,  24.9140f,1.4912f,36.3603f,  23.6960f,0.4018f,37.5142f,  24.3054f,-0.6590f,37.1555f,  22.9332f,-1.7679f,38.0987f,  
	23.5411f,-2.8303f,37.7348f,  22.0377f,-3.9385f,38.4515f,  22.6440f,-5.0031f,38.0821f,  21.0343f,-6.0904f,38.5603f,  21.6395f,-7.1577f,38.1846f,  
	19.9503f,-8.2042f,38.4163f,  20.5542f,-9.2744f,38.0337f,  18.8137f,-10.2605f,38.0152f,  19.4163f,-11.3335f,37.6246f,  17.6535f,-12.2406f,37.3566f,  
	18.2551f,-13.3163f,36.9570f,  16.4995f,-14.1268f,36.4443f,  17.1002f,-15.2055f,36.0344f,  15.3811f,-15.9034f,35.2862f,  15.9805f,-16.9826f,34.8654f,  
	14.3268f,-17.5535f,33.8951f,  14.9254f,-18.6331f,33.4617f,  13.3649f,-19.0647f,32.2867f,  13.9628f,-20.1426f,31.8401f,  12.5217f,-20.4245f,30.4815f,  
	13.1192f,-21.4992f,30.0203f,  11.8218f,-21.6237f,28.5027f,  12.4191f,-22.6918f,28.0270f,  11.2873f,-22.6538f,26.3779f,  11.8849f,-23.7131f,25.8871f,  
	-17.4029f,22.1393f,42.6949f,  -16.6943f,21.3313f,42.3460f,  -17.5361f,20.8730f,40.5901f,  -16.8438f,20.0925f,40.2331f,  -17.5333f,19.8541f,38.3703f,  
	-16.8497f,19.0957f,38.0100f,  -17.3667f,19.0750f,36.0775f,  -16.6865f,18.3332f,35.7170f,  -17.0141f,18.5232f,33.7526f,  -16.3329f,17.7920f,33.3954f,  
	-16.4589f,18.1814f,31.4371f,  -15.7729f,17.4540f,31.0862f,  -15.6894f,18.0275f,29.1714f,  -14.9963f,17.2978f,28.8284f,  -14.7004f,18.0369f,26.9932f,  
	-14.0001f,17.3003f,26.6596f,  -13.4934f,18.1841f,24.9379f,  -12.7852f,17.4350f,24.6141f,  -12.0739f,18.4401f,23.0374f,  -11.3586f,17.6744f,22.7239f,  
	-10.4535f,18.7761f,21.3206f,  -9.7323f,17.9905f,21.0167f,  -8.6482f,19.1630f,19.8117f,  -7.9223f,18.3549f,19.5163f,  -6.6777f,19.5720f,18.5306f,  
	-5.9502f,18.7397f,18.2435f,  -4.5672f,19.9755f,17.4937f,  -3.8385f,19.1180f,17.2125f,  -2.3418f,20.3471f,16.7110f,  -1.6143f,19.4640f,16.4346f,  
	-0.0310f,20.6620f,16.1892f,  0.6928f,19.7544f,15.9160f,  2.3340f,20.8983f,15.9299f,  3.0528f,19.9672f,15.6581f,  4.7221f,21.0358f,15.9299f,  
	5.4348f,20.0827f,15.6580f,  7.1018f,21.0568f,16.1819f,  7.8076f,20.0833f,15.9082f,  9.4419f,20.9461f,16.6742f,  10.1389f,19.9552f,16.3977f,  
	11.7110f,20.6923f,17.3913f,  12.3998f,19.6856f,17.1108f,  13.8811f,20.2852f,18.3142f,  14.5609f,19.2654f,18.0287f,  15.9247f,19.7186f,19.4203f,  
	16.5954f,18.6882f,19.1293f,  17.8171f,18.9887f,20.6847f,  18.4791f,17.9498f,20.3876f,  19.5366f,18.0940f,22.0797f,  20.1902f,17.0488f,21.7763f,  
	21.0645f,17.0361f,23.5756f,  21.7105f,15.9862f,23.2659f,  22.3857f,15.8186f,25.1416f,  23.0247f,14.7655f,24.8256f,  23.4885f,14.4477f,26.7455f,  
	24.1211f,13.3929f,26.4234f,  24.3649f,12.9318f,28.3547f,  24.9921f,11.8759f,28.0268f,  25.0110f,11.2813f,29.9368f,  25.6334f,10.2251f,29.6034f,  
	25.4266f,9.5085f,31.4597f,  26.0450f,8.4522f,31.1210f,  25.6154f,7.6271f,32.8922f,  26.2306f,6.5705f,32.5485f,  25.5851f,5.6524f,34.2045f,  
	26.1975f,4.5955f,33.8561f,  25.3465f,3.6012f,35.3691f,  25.9566f,2.5436f,35.0159f,  24.9140f,1.4912f,36.3603f,  25.5224f,0.4324f,36.0023f,  
	24.3054f,-0.6590f,37.1555f,  24.9123f,-1.7197f,36.7923f,  23.5411f,-2.8303f,37.7348f,  24.1467f,-3.8934f,37.3662f,  22.6440f,-5.0031f,38.0821f,  
	23.2488f,-6.0695f,37.7073f,  21.6395f,-7.1577f,38.1846f,  22.2432f,-8.2275f,37.8029f,  20.5542f,-9.2744f,38.0337f,  21.1569f,-10.3476f,37.6441f,  
	19.4163f,-11.3335f,37.6246f,  20.0183f,-12.4103f,37.2262f,  18.2551f,-13.3163f,36.9570f,  18.8564f,-14.3970f,36.5483f,  17.1002f,-15.2055f,36.0344f,  
	17.7004f,-16.2879f,35.6147f,  15.9805f,-16.9826f,34.8654f,  16.5801f,-18.0668f,34.4331f,  14.9254f,-18.6331f,33.4617f,  15.5244f,-19.7172f,33.0159f,  
	13.9628f,-20.1426f,31.8401f,  14.5615f,-21.2252f,31.3793f,  13.1192f,-21.4992f,30.0203f,  13.7177f,-22.5770f,29.5445f,  12.4191f,-22.6918f,28.0270f,  
	13.0179f,-23.7627f,27.5352f,  11.8849f,-23.7131f,25.8871f,  12.4845f,-24.7743f,25.3788f,  -16.6943f,21.3313f,42.3460f,  -15.9969f,20.5143f,41.9956f,  
	-16.8438f,20.0925f,40.2331f,  -16.1599f,19.3014f,39.8765f,  -16.8497f,19.0957f,38.0100f,  -16.1737f,18.3256f,37.6507f,  -16.6865f,18.3332f,35.7170f,  
	-16.0131f,17.5784f,35.3586f,  -16.3329f,17.7920f,33.3954f,  -15.6576f,17.0460f,33.0412f,  -15.7729f,17.4540f,31.0862f,  -15.0928f,16.7106f,30.7384f,  
	-14.9963f,17.2978f,28.8284f,  -14.3106f,16.5521f,28.4889f,  -14.0001f,17.3003f,26.6596f,  -13.3075f,16.5468f,26.3289f,  -12.7852f,17.4350f,24.6141f,  
	-12.0859f,16.6691f,24.2931f,  -11.3586f,17.6744f,22.7239f,  -10.6534f,16.8923f,22.4123f,  -9.7323f,17.9905f,21.0167f,  -9.0219f,17.1892f,20.7138f,  
	-7.9223f,18.3549f,19.5163f,  -7.2094f,17.5319f,19.2222f,  -5.9502f,18.7397f,18.2435f,  -5.2351f,17.8936f,17.9558f,  -3.8385f,19.1180f,17.2125f,  
	-3.1236f,18.2481f,16.9303f,  -1.6143f,19.4640f,16.4346f,  -0.9018f,18.5706f,16.1564f,  0.6928f,19.7544f,15.9160f,  1.4015f,18.8381f,15.6402f,  
	3.0528f,19.9672f,15.6581f,  3.7567f,19.0289f,15.3831f,  5.4348f,20.0827f,15.6580f,  6.1329f,19.1238f,15.3821f,  7.8076f,20.0833f,15.9082f,  
	8.4978f,19.1065f,15.6304f,  10.1389f,19.9552f,16.3977f,  10.8219f,18.9618f,16.1165f,  12.3998f,19.6856f,17.1108f,  13.0746f,18.6781f,16.8254f,  
	14.5609f,19.2654f,18.0287f,  15.2272f,18.2464f,17.7383f,  16.5954f,18.6882f,19.1293f,  17.2537f,17.6595f,18.8333f,  18.4791f,17.9498f,20.3876f,  
	19.1293f,16.9138f,20.0856f,  20.1902f,17.0488f,21.7763f,  20.8333f,16.0069f,21.4681f,  21.7105f,15.9862f,23.2659f,  22.3468f,14.9401f,22.9516f,  
	23.0247f,14.7655f,24.8256f,  23.6548f,13.7169f,24.5051f,  24.1211f,13.3929f,26.4234f,  24.7459f,12.3423f,26.0970f,  24.9921f,11.8759f,28.0268f,  
	25.6122f,10.8244f,27.6949f,  25.6334f,10.2251f,29.6034f,  26.2496f,9.1729f,29.2661f,  26.0450f,8.4522f,31.1210f,  26.6581f,7.3992f,30.7786f,  
	26.2306f,6.5705f,32.5485f,  26.8410f,5.5170f,32.2011f,  26.1975f,4.5955f,33.8561f,  26.8059f,3.5410f,33.5039f,  25.9566f,2.5436f,35.0159f,  
	26.5634f,1.4876f,34.6588f,  25.5224f,0.4324f,36.0023f,  26.1279f,-0.6258f,35.6401f,  24.9123f,-1.7197f,36.7923f,  25.5169f,-2.7806f,36.4247f,  
	24.1467f,-3.8934f,37.3662f,  24.7506f,-4.9579f,36.9924f,  23.2488f,-6.0695f,37.7073f,  23.8519f,-7.1377f,37.3268f,  22.2432f,-8.2275f,37.8029f,  
	22.8456f,-9.2998f,37.4148f,  21.1569f,-10.3476f,37.6441f,  21.7589f,-11.4243f,37.2472f,  20.0183f,-12.4103f,37.2262f,  20.6197f,-13.4918f,36.8192f,  
	18.8564f,-14.3970f,36.5483f,  19.4570f,-15.4813f,36.1304f,  17.7004f,-16.2879f,35.6147f,  18.3005f,-17.3754f,35.1842f,  16.5801f,-18.0668f,34.4331f,  
	17.1798f,-19.1557f,33.9888f,  15.5244f,-19.7172f,33.0159f,  16.1239f,-20.8065f,32.5564f,  14.5615f,-21.2252f,31.3793f,  15.1608f,-22.3114f,30.9043f,  
	13.7177f,-22.5770f,29.5445f,  14.3172f,-23.6582f,29.0527f,  13.0179f,-23.7627f,27.5352f,  13.6182f,-24.8363f,27.0260f,  12.4845f,-24.7743f,25.3788f,  
	13.0860f,-25.8369f,24.8521f,  -15.9969f,20.5143f,41.9956f,  -15.3090f,19.6885f,41.6451f,  -16.1599f,19.3014f,39.8765f,  -15.4846f,18.5004f,39.5206f,  
	-16.1737f,18.3256f,37.6507f,  -15.5050f,17.5442f,37.2932f,  -16.0131f,17.5784f,35.3586f,  -15.3457f,16.8104f,35.0032f,  -15.6576f,17.0460f,33.0412f,  
	-14.9881f,16.2854f,32.6904f,  -15.0928f,16.7106f,30.7384f,  -14.4195f,15.9525f,30.3943f,  -14.3106f,16.5521f,28.4889f,  -13.6318f,15.7908f,28.1524f,  
	-13.3075f,16.5468f,26.3289f,  -12.6228f,15.7773f,26.0015f,  -12.0859f,16.6691f,24.2931f,  -11.3955f,15.8873f,23.9748f,  -10.6534f,16.8923f,22.4123f,  
	-9.9578f,16.0948f,22.1025f,  -9.0219f,17.1892f,20.7138f,  -8.3233f,16.3728f,20.4130f,  -7.2094f,17.5319f,19.2222f,  -6.5079f,16.6950f,18.9282f,  
	-5.2351f,17.8936f,17.9558f,  -4.5329f,17.0348f,17.6679f,  -3.1236f,18.2481f,16.9303f,  -2.4226f,17.3671f,16.6472f,  -0.9018f,18.5706f,16.1564f,  
	-0.2035f,17.6676f,15.8765f,  1.4015f,18.8381f,15.6402f,  2.0960f,17.9136f,15.3619f,  3.7567f,19.0289f,15.3831f,  4.4464f,18.0840f,15.1048f,  
	6.1329f,19.1238f,15.3821f,  6.8159f,18.1607f,15.1027f,  8.4978f,19.1065f,15.6304f,  9.1744f,18.1263f,15.3485f,  10.8219f,18.9618f,16.1165f,  
	11.4911f,17.9668f,15.8310f,  13.0746f,18.6781f,16.8254f,  13.7360f,17.6707f,16.5354f,  15.2272f,18.2464f,17.7383f,  15.8812f,17.2283f,17.4431f,  
	17.2537f,17.6595f,18.8333f,  17.9000f,16.6332f,18.5325f,  19.1293f,16.9138f,20.0856f,  19.7689f,15.8804f,19.7790f,  20.8333f,16.0069f,21.4681f,  
	21.4663f,14.9686f,21.1554f,  22.3468f,14.9401f,22.9516f,  22.9738f,13.8984f,22.6328f,  23.6548f,13.7169f,24.5051f,  24.2768f,12.6723f,24.1805f,  
	24.7459f,12.3423f,26.0970f,  25.3634f,11.2961f,25.7668f,  25.6122f,10.8244f,27.6949f,  26.2258f,9.7771f,27.3592f,  26.2496f,9.1729f,29.2661f,  
	26.8602f,8.1244f,28.9251f,  26.6581f,7.3992f,30.7786f,  27.2662f,6.3498f,30.4326f,  26.8410f,5.5170f,32.2011f,  27.4473f,4.4662f,31.8502f,  
	26.8059f,3.5410f,33.5039f,  27.4106f,2.4886f,33.1480f,  26.5634f,1.4876f,34.6588f,  27.1671f,0.4328f,34.2978f,  26.1279f,-0.6258f,35.6401f,  
	26.7309f,-1.6835f,35.2737f,  25.5169f,-2.7806f,36.4247f,  26.1194f,-3.8422f,36.0523f,  24.7506f,-4.9579f,36.9924f,  25.3527f,-6.0237f,36.6135f,  
	23.8519f,-7.1377f,37.3268f,  24.4535f,-8.2081f,36.9404f,  22.8456f,-9.2998f,37.4148f,  23.4470f,-10.3753f,37.0198f,  21.7589f,-11.4243f,37.2472f,  
	22.3600f,-12.5054f,36.8424f,  20.6197f,-13.4918f,36.8192f,  21.2203f,-14.5767f,36.4036f,  19.4570f,-15.4813f,36.1304f,  20.0573f,-16.5707f,35.7023f,  
	18.3005f,-17.3754f,35.1842f,  18.9005f,-18.4677f,34.7423f,  17.1798f,-19.1557f,33.9888f,  17.7797f,-20.2501f,33.5315f,  16.1239f,-20.8065f,32.5564f,  
	16.7237f,-21.8996f,32.0831f,  15.1608f,-22.3114f,30.9043f,  15.7609f,-23.4017f,30.4136f,  14.3172f,-23.6582f,29.0527f,  14.9180f,-24.7431f,28.5438f,  
	13.6182f,-24.8363f,27.0260f,  14.2200f,-25.9123f,26.4984f,  13.0860f,-25.8369f,24.8521f,  13.6895f,-26.8999f,24.3065f,  -15.3090f,19.6885f,41.6451f,  
	-14.6309f,18.8548f,41.2947f,  -15.4846f,18.5004f,39.5206f,  -14.8172f,17.6900f,39.1661f,  -15.5050f,17.5442f,37.2932f,  -14.8425f,16.7514f,36.9385f,  
	-15.3457f,16.8104f,35.0032f,  -14.6841f,16.0296f,34.6511f,  -14.9881f,16.2854f,32.6904f,  -14.3250f,15.5115f,32.3433f,  -14.4195f,15.9525f,30.3943f,  
	-13.7527f,15.1799f,30.0536f,  -13.6318f,15.7908f,28.1524f,  -12.9602f,15.0143f,27.8198f,  -12.6228f,15.7773f,26.0015f,  -11.9461f,14.9926f,25.6772f,  
	-11.3955f,15.8873f,23.9748f,  -10.7140f,15.0905f,23.6588f,  -9.9578f,16.0948f,22.1025f,  -9.2728f,15.2823f,21.7955f,  -8.3233f,16.3728f,20.4130f,  
	-7.6350f,15.5426f,20.1130f,  -6.5079f,16.6950f,18.9282f,  -5.8182f,15.8451f,18.6347f,  -4.5329f,17.0348f,17.6679f,  -3.8435f,16.1644f,17.3798f,  
	-2.4226f,17.3671f,16.6472f,  -1.7350f,16.4759f,16.3631f,  -0.2035f,17.6676f,15.8765f,  0.4813f,16.7558f,15.5948f,  2.0960f,17.9136f,15.3619f,  
	2.7771f,16.9817f,15.0810f,  4.4464f,18.0840f,15.1048f,  5.1217f,17.1340f,14.8236f,  6.8159f,18.1607f,15.1027f,  7.4856f,17.1933f,14.8197f,  
	9.1744f,18.1263f,15.3485f,  9.8377f,17.1435f,15.0626f,  11.4911f,17.9668f,15.8310f,  12.1472f,16.9710f,15.5413f,  13.7360f,17.6707f,16.5354f,  
	14.3852f,16.6634f,16.2411f,  15.8812f,17.2283f,17.4431f,  16.5232f,16.2119f,17.1436f,  17.9000f,16.6332f,18.5325f,  18.5357f,15.6089f,18.2274f,  
	19.7689f,15.8804f,19.7790f,  20.3983f,14.8503f,19.4680f,  21.4663f,14.9686f,21.1554f,  22.0899f,13.9342f,20.8385f,  22.9738f,13.8984f,22.6328f,  
	23.5926f,12.8604f,22.3100f,  24.2768f,12.6723f,24.1805f,  24.8912f,11.6321f,23.8520f,  25.3634f,11.2961f,25.7668f,  25.9740f,10.2543f,25.4328f,  
	26.2258f,9.7771f,27.3592f,  26.8336f,8.7337f,27.0198f,  26.8602f,8.1244f,28.9251f,  27.4656f,7.0798f,28.5807f,  27.2662f,6.3498f,30.4326f,  
	27.8698f,5.3035f,30.0831f,  27.4473f,4.4662f,31.8502f,  28.0496f,3.4181f,31.4958f,  27.4106f,2.4886f,33.1480f,  28.0121f,1.4379f,32.7886f,  
	27.1671f,0.4328f,34.2978f,  27.7680f,-0.6210f,33.9331f,  26.7309f,-1.6835f,35.2737f,  27.3316f,-2.7414f,34.9030f,  26.1194f,-3.8422f,36.0523f,  
	26.7199f,-4.9046f,35.6752f,  25.3527f,-6.0237f,36.6135f,  25.9530f,-7.0911f,36.2292f,  24.4535f,-8.2081f,36.9404f,  25.0539f,-9.2811f,36.5477f,  
	23.4470f,-10.3753f,37.0198f,  24.0474f,-11.4546f,36.6175f,  22.3600f,-12.5054f,36.8424f,  22.9601f,-13.5896f,36.4295f,  21.2203f,-14.5767f,36.4036f,  
	21.8204f,-15.6666f,35.9783f,  20.0573f,-16.5707f,35.7023f,  20.6573f,-17.6649f,35.2633f,  18.9005f,-18.4677f,34.7423f,  19.5006f,-19.5656f,34.2879f,  
	17.7797f,-20.2501f,33.5315f,  18.3797f,-21.3485f,33.0609f,  16.7237f,-21.8996f,32.0831f,  17.3241f,-22.9973f,31.5948f,  15.7609f,-23.4017f,30.4136f,  
	16.3619f,-24.4961f,29.9063f,  14.9180f,-24.7431f,28.5438f,  15.5201f,-25.8311f,28.0168f,  14.2200f,-25.9123f,26.4984f,  14.8235f,-26.9896f,25.9521f,  
	13.6895f,-26.8999f,24.3065f,  14.2950f,-27.9634f,23.7405f,  -14.6309f,18.8548f,41.2947f,  -13.9615f,18.0132f,40.9452f,  -14.8172f,17.6900f,39.1661f,  
	-14.1565f,16.8699f,38.8140f,  -14.8425f,16.7514f,36.9385f,  -14.1860f,15.9475f,36.5869f,  -14.6841f,16.0296f,34.6511f,  -14.0287f,15.2369f,34.3026f,  
	-14.3250f,15.5115f,32.3433f,  -13.6680f,14.7245f,31.9997f,  -13.7527f,15.1799f,30.0536f,  -13.0923f,14.3932f,29.7169f,  -12.9602f,15.0143f,27.8198f,  
	-12.2958f,14.2235f,27.4906f,  -11.9461f,14.9926f,25.6772f,  -11.2774f,14.1935f,25.3558f,  -10.7140f,15.0905f,23.6588f,  -10.0417f,14.2790f,23.3461f,  
	-9.2728f,15.2823f,21.7955f,  -8.5972f,14.4562f,21.4899f,  -7.6350f,15.5426f,20.1130f,  -6.9574f,14.6992f,19.8142f,  -5.8182f,15.8451f,18.6347f,  
	-5.1403f,14.9832f,18.3418f,  -3.8435f,16.1644f,17.3798f,  -3.1665f,15.2833f,17.0915f,  -1.7350f,16.4759f,16.3631f,  -1.0599f,15.5753f,16.0778f,  
	0.4813f,16.7558f,15.5948f,  1.1535f,15.8358f,15.3111f,  2.7771f,16.9817f,15.0810f,  3.4445f,16.0439f,14.7978f,  5.1217f,17.1340f,14.8236f,  
	5.7844f,16.1788f,14.5393f,  7.4856f,17.1933f,14.8197f,  8.1425f,16.2225f,14.5333f,  9.8377f,17.1435f,15.0626f,  10.4880f,16.1592f,14.7730f,  
	12.1472f,16.9710f,15.5413f,  12.7913f,15.9745f,15.2476f,  14.3852f,16.6634f,16.2411f,  15.0225f,15.6572f,15.9426f,  16.5232f,16.2119f,17.1436f,  
	17.1547f,15.1969f,16.8398f,  18.5357f,15.6089f,18.2274f,  19.1612f,14.5874f,17.9180f,  20.3983f,14.8503f,19.4680f,  21.0182f,13.8238f,19.1529f,  
	22.0899f,13.9342f,20.8385f,  22.7052f,12.9034f,20.5176f,  23.5926f,12.8604f,22.3100f,  24.2037f,11.8267f,21.9835f,  24.8912f,11.6321f,23.8520f,  
	25.4985f,10.5964f,23.5199f,  25.9740f,10.2543f,25.4328f,  26.5786f,9.2166f,25.0952f,  26.8336f,8.7337f,27.0198f,  27.4359f,7.6943f,26.6771f,  
	27.4656f,7.0798f,28.5807f,  28.0663f,6.0386f,28.2330f,  27.8698f,5.3035f,30.0831f,  28.4694f,4.2604f,29.7305f,  28.0496f,3.4181f,31.4958f,  
	28.6485f,2.3723f,31.1381f,  28.0121f,1.4379f,32.7886f,  28.6106f,0.3888f,32.4256f,  27.7680f,-0.6210f,33.9331f,  28.3666f,-1.6743f,33.5643f,  
	27.3316f,-2.7414f,34.9030f,  27.9301f,-3.7994f,34.5281f,  26.7199f,-4.9046f,35.6752f,  27.3185f,-5.9679f,35.2933f,  25.9530f,-7.0911f,36.2292f,  
	26.5521f,-8.1606f,35.8391f,  25.0539f,-9.2811f,36.5477f,  25.6532f,-10.3575f,36.1484f,  24.0474f,-11.4546f,36.6175f,  24.6467f,-12.5368f,36.2078f,  
	22.9601f,-13.5896f,36.4295f,  23.5596f,-14.6785f,36.0077f,  21.8204f,-15.6666f,35.9783f,  22.4199f,-16.7611f,35.5430f,  20.6573f,-17.6649f,35.2633f,  
	21.2571f,-18.7647f,34.8126f,  19.5006f,-19.5656f,34.2879f,  20.1005f,-20.6678f,33.8209f,  18.3797f,-21.3485f,33.0609f,  18.9801f,-22.4520f,32.5758f,  
	17.3241f,-22.9973f,31.5948f,  17.9251f,-24.0998f,31.0901f,  16.3619f,-24.4961f,29.9063f,  16.9639f,-25.5945f,29.3812f,  15.5201f,-25.8311f,28.0168f,  
	16.1233f,-26.9212f,27.4713f,  14.8235f,-26.9896f,25.9521f,  15.4287f,-28.0685f,25.3854f,  14.2950f,-27.9634f,23.7405f,  14.9028f,-29.0268f,23.1530f,  
	-13.9615f,18.0132f,40.9452f,  -13.2996f,17.1639f,40.5976f,  -14.1565f,16.8699f,38.8140f,  -13.5023f,16.0405f,38.4647f,  -14.1860f,15.9475f,36.5869f,  
	-13.5356f,15.1332f,36.2387f,  -14.0287f,15.2369f,34.3026f,  -13.3792f,14.4324f,33.9576f,  -13.6680f,14.7245f,31.9997f,  -13.0169f,13.9246f,31.6603f,  
	-13.0923f,14.3932f,29.7169f,  -12.4385f,13.5932f,29.3839f,  -12.2958f,14.2235f,27.4906f,  -11.6385f,13.4191f,27.1647f,  -11.2774f,14.1935f,25.3558f,  
	-10.6168f,13.3802f,25.0382f,  -10.0417f,14.2790f,23.3461f,  -9.3779f,13.4542f,23.0354f,  -8.5972f,14.4562f,21.4899f,  -7.9312f,13.6168f,21.1861f,  
	-6.9574f,14.6992f,19.8142f,  -6.2906f,13.8437f,19.5166f,  -5.1403f,14.9832f,18.3418f,  -4.4737f,14.1103f,18.0493f,  -3.1665f,15.2833f,17.0915f,  
	-2.5011f,14.3923f,16.8027f,  -1.0599f,15.5753f,16.0778f,  -0.3966f,14.6659f,15.7912f,  1.1535f,15.8358f,15.3111f,  1.8128f,14.9092f,15.0258f,  
	3.4445f,16.0439f,14.7978f,  4.0997f,15.1002f,14.5121f,  5.7844f,16.1788f,14.5393f,  6.4346f,15.2194f,14.2521f,  8.1425f,16.2225f,14.5333f,  
	8.7869f,15.2493f,14.2436f,  10.4880f,16.1592f,14.7730f,  11.1267f,15.1735f,14.4797f,  12.7913f,15.9745f,15.2476f,  13.4237f,14.9785f,14.9501f,  
	15.0225f,15.6572f,15.9426f,  15.6495f,14.6516f,15.6402f,  17.1547f,15.1969f,16.8398f,  17.7759f,14.1842f,16.5321f,  19.1612f,14.5874f,17.9180f,  
	19.7770f,13.5690f,17.6048f,  21.0182f,13.8238f,19.1529f,  21.6298f,12.8005f,18.8339f,  22.7052f,12.9034f,20.5176f,  23.3126f,11.8766f,20.1930f,  
	24.2037f,11.8267f,21.9835f,  24.8075f,10.7975f,21.6533f,  25.4985f,10.5964f,23.5199f,  26.0997f,9.5648f,23.1844f,  26.5786f,9.2166f,25.0952f,  
	27.1776f,8.1830f,24.7545f,  27.4359f,7.6943f,26.6771f,  28.0334f,6.6588f,26.3313f,  28.0663f,6.0386f,28.2330f,  28.6627f,5.0010f,27.8821f,  
	28.4694f,4.2604f,29.7305f,  29.0652f,3.2200f,29.3746f,  28.6485f,2.3723f,31.1381f,  29.2441f,1.3286f,30.7771f,  28.6106f,0.3888f,32.4256f,  
	29.2065f,-0.6592f,32.0589f,  28.3666f,-1.6743f,33.5643f,  28.9627f,-2.7271f,33.1917f,  27.9301f,-3.7994f,34.5281f,  28.5267f,-4.8578f,34.1487f,  
	27.3185f,-5.9679f,35.2933f,  27.9158f,-7.0328f,34.9061f,  26.5521f,-8.1606f,35.8391f,  27.1497f,-9.2328f,35.4430f,  25.6532f,-10.3575f,36.1484f,  
	26.2512f,-11.4363f,35.7423f,  24.6467f,-12.5368f,36.2078f,  25.2451f,-13.6232f,35.7900f,  23.5596f,-14.6785f,36.0077f,  24.1584f,-15.7717f,35.5766f,  
	22.4199f,-16.7611f,35.5430f,  23.0192f,-17.8612f,35.0967f,  21.2571f,-18.7647f,34.8126f,  21.8566f,-19.8689f,34.3500f,  20.1005f,-20.6678f,33.8209f,  
	20.7005f,-21.7752f,33.3400f,  18.9801f,-22.4520f,32.5758f,  19.5808f,-23.5607f,32.0751f,  17.9251f,-24.0998f,31.0901f,  18.5267f,-25.2068f,30.5683f,  
	16.9639f,-25.5945f,29.3812f,  17.5667f,-26.6958f,28.8380f,  16.1233f,-26.9212f,27.4713f,  16.7281f,-28.0139f,26.9056f,  15.4287f,-28.0685f,25.3854f,  
	16.0358f,-29.1483f,24.7973f,  14.9028f,-29.0268f,23.1530f,  15.5128f,-30.0890f,22.5433f,  -13.2996f,17.1639f,40.5976f,  -12.6447f,16.3070f,40.2522f,  
	-13.5023f,16.0405f,38.4647f,  -12.8543f,15.2022f,38.1183f,  -13.5356f,15.1332f,36.2387f,  -12.8912f,14.3087f,35.8938f,  -13.3792f,14.4324f,33.9576f,  
	-12.7351f,13.6165f,33.6167f,  -13.0169f,13.9246f,31.6603f,  -12.3717f,13.1125f,31.3247f,  -12.4385f,13.5932f,29.3839f,  -11.7911f,12.7805f,29.0544f,  
	-11.6385f,13.4191f,27.1647f,  -10.9883f,12.6010f,26.8430f,  -10.6168f,13.3802f,25.0382f,  -9.9639f,12.5540f,24.7231f,  -9.3779f,13.4542f,23.0354f,  
	-8.7228f,12.6165f,22.7271f,  -7.9312f,13.6168f,21.1861f,  -7.2749f,12.7653f,20.8843f,  -6.2906f,13.8437f,19.5166f,  -5.6341f,12.9769f,19.2201f,  
	-4.4737f,14.1103f,18.0493f,  -3.8178f,13.2271f,17.7569f,  -2.5011f,14.3923f,16.8027f,  -1.8467f,13.4919f,16.5131f,  -0.3966f,14.6659f,15.7912f,  
	0.2545f,13.7491f,15.5036f,  1.8128f,14.9092f,15.0258f,  2.4606f,13.9759f,14.7385f,  4.0997f,15.1002f,14.5121f,  4.7432f,14.1514f,14.2239f,  
	6.4346f,15.2194f,14.2521f,  7.0728f,14.2569f,13.9621f,  8.7869f,15.2493f,14.2436f,  9.4200f,14.2739f,13.9506f,  11.1267f,15.1735f,14.4797f,  
	11.7540f,14.1873f,14.1830f,  13.4237f,14.9785f,14.9501f,  14.0460f,13.9823f,14.6488f,  15.6495f,14.6516f,15.6402f,  16.2663f,13.6479f,15.3340f,  
	17.7759f,14.1842f,16.5321f,  18.3876f,13.1743f,16.2207f,  19.7770f,13.5690f,17.6048f,  20.3846f,12.5534f,17.2877f,  21.6298f,12.8005f,18.8339f,  
	22.2333f,11.7809f,18.5113f,  23.3126f,11.8766f,20.1930f,  23.9127f,10.8542f,19.8650f,  24.8075f,10.7975f,21.6533f,  25.4050f,9.7724f,21.3199f,  
	26.0997f,9.5648f,23.1844f,  26.6951f,8.5375f,22.8457f,  27.1776f,8.1830f,24.7545f,  27.7716f,7.1536f,24.4107f,  28.0334f,6.6588f,26.3313f,  
	28.6264f,5.6272f,25.9825f,  28.6627f,5.0010f,27.8821f,  29.2553f,3.9665f,27.5284f,  29.0652f,3.2200f,29.3746f,  29.6578f,2.1823f,29.0157f,  
	29.2441f,1.3286f,30.7771f,  29.8371f,0.2865f,30.4127f,  29.2065f,-0.6592f,32.0589f,  29.8000f,-1.7060f,31.6888f,  28.9627f,-2.7271f,33.1917f,  
	29.5568f,-3.7796f,32.8151f,  28.5267f,-4.8578f,34.1487f,  29.1217f,-5.9171f,33.7646f,  27.9158f,-7.0328f,34.9061f,  28.5115f,-8.0998f,34.5135f,  
	27.1497f,-9.2328f,35.4430f,  27.7461f,-10.3070f,35.0408f,  26.2512f,-11.4363f,35.7423f,  26.8482f,-12.5189f,35.3287f,  25.2451f,-13.6232f,35.7900f,  
	25.8428f,-14.7138f,35.3635f,  24.1584f,-15.7717f,35.5766f,  24.7568f,-16.8704f,35.1354f,  23.0192f,-17.8612f,35.0967f,  23.6180f,-18.9655f,34.6393f,  
	21.8566f,-19.8689f,34.3500f,  22.4560f,-20.9784f,33.8744f,  20.7005f,-21.7752f,33.3400f,  21.3007f,-22.8882f,32.8443f,  19.5808f,-23.5607f,32.0751f,  
	20.1819f,-24.6744f,31.5578f,  18.5267f,-25.2068f,30.5683f,  19.1290f,-26.3173f,30.0289f,  17.5667f,-26.6958f,28.8380f,  18.1707f,-27.8005f,28.2750f,  
	16.7281f,-28.0139f,26.9056f,  17.3342f,-29.1085f,26.3186f,  16.0358f,-29.1483f,24.7973f,  16.6447f,-30.2281f,24.1870f,  15.5128f,-30.0890f,22.5433f,  
	16.1253f,-31.1500f,21.9096f,  -12.6447f,16.3070f,40.2522f,  -11.9963f,15.4428f,39.9094f,  -12.8543f,15.2022f,38.1183f,  -12.2123f,14.3554f,37.7751f,  
	-12.8912f,14.3087f,35.8938f,  -12.2520f,13.4741f,35.5528f,  -12.7351f,13.6165f,33.6167f,  -12.0967f,12.7896f,33.2797f,  -12.3717f,13.1125f,31.3247f,  
	-11.7326f,12.2888f,30.9927f,  -11.7911f,12.7805f,29.0544f,  -11.1499f,11.9550f,28.7294f,  -10.9883f,12.6010f,26.8430f,  -10.3451f,11.7708f,26.5240f,  
	-9.9639f,12.5540f,24.7231f,  -9.3187f,11.7152f,24.4108f,  -8.7228f,12.6165f,22.7271f,  -8.0764f,11.7666f,22.4213f,  -7.2749f,12.7653f,20.8843f,  
	-6.6279f,11.9025f,20.5840f,  -5.6341f,12.9769f,19.2201f,  -4.9874f,12.0995f,18.9244f,  -3.8178f,13.2271f,17.7569f,  -3.1720f,12.3341f,17.4645f,  
	-1.8467f,13.4919f,16.5131f,  -1.2036f,12.5837f,16.2232f,  0.2545f,13.7491f,15.5036f,  0.8949f,12.8252f,15.2146f,  2.4606f,13.9759f,14.7385f,  
	3.0973f,13.0368f,14.4492f,  4.7432f,14.1514f,14.2239f,  5.3753f,13.1988f,13.9335f,  7.0728f,14.2569f,13.9621f,  7.7003f,13.2913f,13.6692f,  
	9.4200f,14.2739f,13.9506f,  10.0420f,13.2974f,13.6545f,  11.7540f,14.1873f,14.1830f,  12.3714f,13.2004f,13.8828f,  14.0460f,13.9823f,14.6488f,  
	14.6582f,12.9874f,14.3441f,  16.2663f,13.6479f,15.3340f,  16.8737f,12.6462f,15.0243f,  18.3876f,13.1743f,16.2207f,  18.9911f,12.1665f,15.9056f,  
	20.3846f,12.5534f,17.2877f,  20.9842f,11.5412f,16.9672f,  22.2333f,11.7809f,18.5113f,  22.8295f,10.7655f,18.1854f,  23.9127f,10.8542f,19.8650f,  
	24.5064f,9.8359f,19.5336f,  25.4050f,9.7724f,21.3199f,  25.9966f,8.7516f,20.9833f,  26.6951f,8.5375f,22.8457f,  27.2853f,7.5145f,22.5040f,  
	27.7716f,7.1536f,24.4107f,  28.3610f,6.1283f,24.0640f,  28.6264f,5.6272f,25.9825f,  29.2155f,4.5991f,25.6309f,  29.2553f,3.9665f,27.5284f,  
	29.8445f,2.9353f,27.1718f,  29.6578f,2.1823f,29.0157f,  30.2476f,1.1467f,28.6538f,  29.8371f,0.2865f,30.4127f,  30.4275f,-0.7537f,30.0452f,  
	29.8000f,-1.7060f,31.6888f,  30.3912f,-2.7520f,31.3151f,  29.5568f,-3.7796f,32.8151f,  30.1493f,-4.8326f,32.4342f,  29.1217f,-5.9171f,33.7646f,  
	29.7152f,-6.9778f,33.3758f,  28.5115f,-8.0998f,34.5135f,  29.1059f,-9.1683f,34.1155f,  27.7461f,-10.3070f,35.0408f,  28.3414f,-11.3845f,34.6320f,  
	26.8482f,-12.5189f,35.3287f,  27.4445f,-13.6052f,34.9074f,  25.8428f,-14.7138f,35.3635f,  26.4400f,-15.8093f,34.9278f,  24.7568f,-16.8704f,35.1354f,  
	25.3545f,-17.9732f,34.6838f,  23.6180f,-18.9655f,34.6393f,  24.2165f,-20.0753f,34.1696f,  22.4560f,-20.9784f,33.8744f,  23.0554f,-22.0937f,33.3847f,  
	21.3007f,-22.8882f,32.8443f,  21.9010f,-24.0066f,32.3328f,  20.1819f,-24.6744f,31.5578f,  20.7833f,-25.7922f,31.0236f,  19.1290f,-26.3173f,30.0289f,  
	19.7321f,-27.4321f,29.4702f,  18.1707f,-27.8005f,28.2750f,  18.7758f,-28.9081f,27.6911f,  17.3342f,-29.1085f,26.3186f,  17.9419f,-30.2043f,25.7097f,  
	16.6447f,-30.2281f,24.1870f,  17.2557f,-31.3079f,23.5526f,  16.1253f,-31.1500f,21.9096f,  16.7401f,-32.2078f,21.2524f,  -11.9963f,15.4428f,39.9094f,  
	-11.3543f,14.5718f,39.5692f,  -12.2123f,14.3554f,37.7751f,  -11.5757f,13.5000f,37.4354f,  -12.2520f,13.4741f,35.5528f,  -11.6182f,12.6300f,35.2156f,  
	-12.0967f,12.7896f,33.2797f,  -11.4639f,11.9522f,32.9463f,  -11.7326f,12.2888f,30.9927f,  -11.0987f,11.4531f,30.6653f,  -11.1499f,11.9550f,28.7294f,  
	-10.5150f,11.1180f,28.4074f,  -10.3451f,11.7708f,26.5240f,  -9.7087f,10.9284f,26.2083f,  -9.3187f,11.7152f,24.4108f,  -8.6812f,10.8646f,24.1013f,  
	-8.0764f,11.7666f,22.4213f,  -7.4382f,10.9054f,22.1174f,  -6.6279f,11.9025f,20.5840f,  -5.9898f,11.0289f,20.2849f,  -4.9874f,12.0995f,18.9244f,  
	-4.3498f,11.2121f,18.6290f,  -3.1720f,12.3341f,17.4645f,  -2.5366f,11.4328f,17.1722f,  -1.2036f,12.5837f,16.2232f,  -0.5704f,11.6677f,15.9323f,  
	0.8949f,12.8252f,15.2146f,  1.5250f,11.8947f,14.9241f,  3.0973f,13.0368f,14.4492f,  3.7232f,12.0932f,14.1581f,  5.3753f,13.1988f,13.9335f,  
	5.9970f,12.2423f,13.6405f,  7.7003f,13.2913f,13.6692f,  8.3171f,12.3238f,13.3737f,  10.0420f,13.2974f,13.6545f,  10.6546f,12.3194f,13.3553f,  
	12.3714f,13.2004f,13.8828f,  12.9791f,12.2141f,13.5794f,  14.6582f,12.9874f,14.3441f,  15.2612f,11.9940f,14.0360f,  16.8737f,12.6462f,15.0243f,  
	17.4730f,11.6463f,14.7110f,  18.9911f,12.1665f,15.9056f,  19.5867f,11.1617f,15.5870f,  20.9842f,11.5412f,16.9672f,  21.5764f,10.5330f,16.6434f,  
	22.8295f,10.7655f,18.1854f,  23.4192f,9.7541f,17.8563f,  24.5064f,9.8359f,19.5336f,  25.0942f,8.8218f,19.1993f,  25.9966f,8.7516f,20.9833f,  
	26.5831f,7.7352f,20.6438f,  27.2853f,7.5145f,22.5040f,  27.8710f,6.4957f,22.1596f,  28.3610f,6.1283f,24.0640f,  28.9464f,5.1069f,23.7147f,  
	29.2155f,4.5991f,25.6309f,  29.8011f,3.5747f,25.2767f,  29.8445f,2.9353f,27.1718f,  30.4309f,1.9067f,26.8123f,  30.2476f,1.1467f,28.6538f,  
	30.8347f,0.1136f,28.2891f,  30.4275f,-0.7537f,30.0452f,  31.0158f,-1.7925f,29.6746f,  30.3912f,-2.7520f,31.3151f,  30.9810f,-3.7978f,30.9377f,  
	30.1493f,-4.8326f,32.4342f,  30.7402f,-5.8861f,32.0493f,  29.7152f,-6.9778f,33.3758f,  30.3072f,-8.0394f,32.9821f,  29.1059f,-9.1683f,34.1155f,  
	29.6992f,-10.2393f,33.7115f,  28.3414f,-11.3845f,34.6320f,  28.9359f,-12.4652f,34.2161f,  27.4445f,-13.6052f,34.9074f,  28.0401f,-14.6962f,34.4777f,  
	26.4400f,-15.8093f,34.9278f,  27.0364f,-16.9088f,34.4826f,  25.3545f,-17.9732f,34.6838f,  25.9518f,-19.0812f,34.2208f,  24.2165f,-20.0753f,34.1696f,  
	24.8148f,-21.1908f,33.6868f,  23.0554f,-22.0937f,33.3847f,  23.6548f,-23.2147f,32.8800f,  21.9010f,-24.0066f,32.3328f,  22.5015f,-25.1294f,31.8051f,  
	20.7833f,-25.7922f,31.0236f,  21.3854f,-26.9149f,30.4707f,  19.7321f,-27.4321f,29.4702f,  20.3360f,-28.5506f,28.8912f,  18.7758f,-28.9081f,27.6911f,  
	19.3822f,-30.0179f,27.0856f,  17.9419f,-30.2043f,25.7097f,  18.5512f,-31.3013f,25.0769f,  17.2557f,-31.3079f,23.5526f,  17.8687f,-32.3860f,22.8945f,  
	16.7401f,-32.2078f,21.2524f,  17.3575f,-33.2624f,20.5694f,  -11.3543f,14.5718f,39.5692f,  -10.7178f,13.6936f,39.2322f,  -11.5757f,13.5000f,37.4354f,  
	-10.9443f,12.6364f,37.0992f,  -11.6182f,12.6300f,35.2156f,  -10.9897f,11.7767f,34.8819f,  -11.4639f,11.9522f,32.9463f,  -10.8357f,11.1040f,32.6175f,  
	-11.0987f,11.4531f,30.6653f,  -10.4706f,10.6069f,30.3411f,  -10.5150f,11.1180f,28.4074f,  -9.8861f,10.2697f,28.0889f,  -9.7087f,10.9284f,26.2083f,  
	-9.0790f,10.0747f,25.8958f,  -8.6812f,10.8646f,24.1013f,  -8.0510f,10.0029f,23.7943f,  -7.4382f,10.9054f,22.1174f,  -6.8080f,10.0336f,21.8153f,  
	-5.9898f,11.0289f,20.2849f,  -5.3600f,10.1452f,19.9867f,  -4.3498f,11.2121f,18.6290f,  -3.7217f,10.3160f,18.3343f,  -2.5366f,11.4328f,17.1722f,  
	-1.9103f,10.5232f,16.8794f,  -0.5704f,11.6677f,15.9323f,  0.0532f,10.7446f,15.6404f,  1.5250f,11.8947f,14.9241f,  2.1449f,10.9590f,14.6322f,  
	3.7232f,12.0932f,14.1581f,  4.3394f,11.1451f,13.8649f,  5.9970f,12.2423f,13.6405f,  6.6087f,11.2833f,13.3452f,  8.3171f,12.3238f,13.3737f,  
	8.9249f,11.3542f,13.0751f,  10.6546f,12.3194f,13.3553f,  11.2577f,11.3413f,13.0530f,  12.9791f,12.2141f,13.5794f,  13.5778f,11.2287f,13.2729f,  
	15.2612f,11.9940f,14.0360f,  15.8564f,11.0017f,13.7245f,  17.4730f,11.6463f,14.7110f,  18.0646f,10.6488f,14.3944f,  19.5867f,11.1617f,15.5870f,  
	20.1749f,10.1606f,15.2653f,  21.5764f,10.5330f,16.6434f,  22.1622f,9.5285f,16.3164f,  23.4192f,9.7541f,17.8563f,  24.0032f,8.7468f,17.5241f,  
	25.0942f,8.8218f,19.1993f,  25.6767f,7.8122f,18.8620f,  26.5831f,7.7352f,20.6438f,  27.1649f,6.7233f,20.3017f,  27.8710f,6.4957f,22.1596f,  
	28.4526f,5.4813f,21.8126f,  28.9464f,5.1069f,23.7147f,  29.5283f,4.0895f,23.3629f,  29.8011f,3.5747f,25.2767f,  30.3838f,2.5534f,24.9198f,  
	30.4309f,1.9067f,26.8123f,  31.0145f,0.8811f,26.4504f,  30.8347f,0.1136f,28.2891f,  31.4197f,-0.9174f,27.9216f,  31.0158f,-1.7925f,29.6746f,  
	31.6024f,-2.8303f,29.3006f,  30.9810f,-3.7978f,30.9377f,  31.5690f,-4.8433f,30.5567f,  30.7402f,-5.8861f,32.0493f,  31.3297f,-6.9399f,31.6601f,  
	30.3072f,-8.0394f,32.9821f,  30.8982f,-9.1030f,32.5832f,  29.6992f,-10.2393f,33.7115f,  30.2916f,-11.3130f,33.3013f,  28.9359f,-12.4652f,34.2161f,  
	29.5297f,-13.5499f,33.7927f,  28.0401f,-14.6962f,34.4777f,  28.6349f,-15.7906f,34.0393f,  27.0364f,-16.9088f,34.4826f,  27.6324f,-18.0134f,34.0270f,  
	25.9518f,-19.0812f,34.2208f,  26.5489f,-20.1951f,33.7457f,  24.8148f,-21.1908f,33.6868f,  25.4131f,-22.3123f,33.1901f,  23.6548f,-23.2147f,32.8800f,  
	24.2541f,-24.3403f,32.3600f,  22.5015f,-25.1294f,31.8051f,  23.1024f,-26.2578f,31.2595f,  21.3854f,-26.9149f,30.4707f,  21.9881f,-28.0421f,29.8983f,  
	20.3360f,-28.5506f,28.8912f,  20.9410f,-29.6723f,28.2909f,  19.3822f,-30.0179f,27.0856f,  19.9898f,-31.1300f,26.4565f,  18.5512f,-31.3013f,25.0769f,  
	19.1625f,-32.3978f,24.4204f,  17.8687f,-32.3860f,22.8945f,  18.4840f,-33.4623f,22.2105f,  17.3575f,-33.2624f,20.5694f,  17.9776f,-34.3130f,19.8593f,  
	-10.7178f,13.6936f,39.2322f,  -10.0865f,12.8088f,38.8982f,  -10.9443f,12.6364f,37.0992f,  -10.3181f,11.7650f,36.7663f,  -10.9897f,11.7767f,34.8819f,  
	-10.3653f,10.9136f,34.5526f,  -10.8357f,11.1040f,32.6175f,  -10.2128f,10.2462f,32.2919f,  -10.4706f,10.6069f,30.3411f,  -9.8479f,9.7502f,30.0204f,  
	-9.8861f,10.2697f,28.0889f,  -9.2631f,9.4104f,27.7738f,  -9.0790f,10.0747f,25.8958f,  -8.4558f,9.2102f,25.5859f,  -8.0510f,10.0029f,23.7943f,  
	-7.4279f,9.1307f,23.4894f,  -6.8080f,10.0336f,21.8153f,  -6.1852f,9.1517f,21.5145f,  -5.3600f,10.1452f,19.9867f,  -4.7387f,9.2526f,19.6897f,  
	-3.7217f,10.3160f,18.3343f,  -3.1020f,9.4114f,18.0395f,  -1.9103f,10.5232f,16.8794f,  -1.2929f,9.6061f,16.5860f,  0.0532f,10.7446f,15.6404f,  
	0.6674f,9.8157f,15.3476f,  2.1449f,10.9590f,14.6322f,  2.7557f,10.0182f,14.3386f,  4.3394f,11.1451f,13.8649f,  4.9461f,10.1937f,13.5697f,  
	6.6087f,11.2833f,13.3452f,  7.2118f,10.3214f,13.0473f,  8.9249f,11.3542f,13.0751f,  9.5235f,10.3838f,12.7739f,  11.2577f,11.3413f,13.0530f,  
	11.8522f,10.3634f,12.7478f,  13.5778f,11.2287f,13.2729f,  14.1689f,10.2438f,12.9630f,  15.8564f,11.0017f,13.7245f,  16.4440f,10.0114f,13.4098f,  
	18.0646f,10.6488f,14.3944f,  18.6489f,9.6547f,14.0747f,  20.1749f,10.1606f,15.2653f,  20.7568f,9.1629f,14.9405f,  22.1622f,9.5285f,16.3164f,  
	22.7423f,8.5279f,15.9865f,  24.0032f,8.7468f,17.5241f,  24.5819f,7.7439f,17.1891f,  25.6767f,7.8122f,18.8620f,  26.2547f,6.8070f,18.5222f,  
	27.1649f,6.7233f,20.3017f,  27.7427f,5.7158f,19.9570f,  28.4526f,5.4813f,21.8126f,  29.0307f,4.4711f,21.4631f,  29.5283f,4.0895f,23.3629f,  
	30.1073f,3.0758f,23.0086f,  30.3838f,2.5534f,24.9198f,  30.9639f,1.5357f,24.5607f,  31.0145f,0.8811f,26.4504f,  31.5960f,-0.1418f,26.0860f,  
	31.4197f,-0.9174f,27.9216f,  32.0030f,-1.9469f,27.5510f,  31.6024f,-2.8303f,29.3006f,  32.1873f,-3.8672f,28.9236f,  31.5690f,-4.8433f,30.5567f,  
	32.1556f,-5.8885f,30.1720f,  31.3297f,-6.9399f,31.6601f,  31.9181f,-7.9950f,31.2663f,  30.8982f,-9.1030f,32.5832f,  31.4884f,-10.1686f,32.1788f,  
	30.2916f,-11.3130f,33.3013f,  30.8833f,-12.3902f,32.8845f,  29.5297f,-13.5499f,33.7927f,  30.1227f,-14.6378f,33.3615f,  28.6349f,-15.7906f,34.0393f,  
	29.2293f,-16.8899f,33.5915f,  27.6324f,-18.0134f,34.0270f,  28.2280f,-19.1235f,33.5603f,  26.5489f,-20.1951f,33.7457f,  27.1458f,-21.3148f,33.2576f,  
	25.4131f,-22.3123f,33.1901f,  26.0111f,-23.4385f,32.6789f,  24.2541f,-24.3403f,32.3600f,  24.8537f,-25.4720f,31.8231f,  23.1024f,-26.2578f,31.2595f,  
	23.7037f,-27.3912f,30.6952f,  21.9881f,-28.0421f,29.8983f,  22.5915f,-29.1732f,29.3052f,  20.9410f,-29.6723f,28.2909f,  21.5468f,-30.7972f,27.6677f,  
	19.9898f,-31.1300f,26.4565f,  20.5991f,-32.2429f,25.8039f,  19.1625f,-32.3978f,24.4204f,  19.7756f,-33.4939f,23.7379f,  18.4840f,-33.4623f,22.2105f,  
	19.1016f,-34.5360f,21.4991f,  17.9776f,-34.3130f,19.8593f,  18.6004f,-35.3575f,19.1227f,  -10.0865f,12.8088f,38.8982f,  -9.4606f,11.9176f,38.5672f,  
	-10.3181f,11.7650f,36.7663f,  -9.6956f,10.8851f,36.4375f,  -10.3653f,10.9136f,34.5526f,  -9.7460f,10.0421f,34.2264f,  -10.2128f,10.2462f,32.2919f,  
	-9.5947f,9.3789f,31.9698f,  -9.8479f,9.7502f,30.0204f,  -9.2303f,8.8832f,29.7033f,  -9.2631f,9.4104f,27.7738f,  -8.6458f,8.5409f,27.4616f,  
	-8.4558f,9.2102f,25.5859f,  -7.8389f,8.3355f,25.2785f,  -7.4279f,9.1307f,23.4894f,  -6.8114f,8.2485f,23.1861f,  -6.1852f,9.1517f,21.5145f,  
	-5.5701f,8.2607f,21.2151f,  -4.7387f,9.2526f,19.6897f,  -4.1249f,8.3512f,19.3929f,  -3.1020f,9.4114f,18.0395f,  -2.4903f,8.4989f,17.7446f,  
	-1.2929f,9.6061f,16.5860f,  -0.6841f,8.6827f,16.2921f,  0.6674f,9.8157f,15.3476f,  1.2731f,8.8811f,15.0534f,  2.7557f,10.0182f,14.3386f,  
	3.3576f,9.0735f,14.0433f,  4.9461f,10.1937f,13.5697f,  5.5446f,9.2387f,13.2721f,  7.2118f,10.3214f,13.0473f,  7.8062f,9.3581f,12.7468f,  
	9.5235f,10.3838f,12.7739f,  10.1139f,9.4130f,12.4699f,  11.8522f,10.3634f,12.7478f,  12.4393f,9.3854f,12.4394f,  14.1689f,10.2438f,12.9630f,  
	14.7527f,9.2602f,12.6500f,  16.4440f,10.0114f,13.4098f,  17.0245f,9.0239f,13.0920f,  18.6489f,9.6547f,14.0747f,  19.2270f,8.6636f,13.7520f,  
	20.7568f,9.1629f,14.9405f,  21.3331f,8.1689f,14.6126f,  22.7423f,8.5279f,15.9865f,  23.3172f,7.5316f,15.6537f,  24.5819f,7.7439f,17.1891f,  
	25.1560f,6.7454f,16.8515f,  26.2547f,6.8070f,18.5222f,  26.8287f,5.8064f,18.1798f,  27.7427f,5.7158f,19.9570f,  28.3170f,4.7128f,19.6100f,  
	29.0307f,4.4711f,21.4631f,  29.6060f,3.4649f,21.1114f,  30.1073f,3.0758f,23.0086f,  30.6837f,2.0660f,22.6522f,  30.9639f,1.5357f,24.5607f,  
	31.5418f,0.5211f,24.1992f,  31.5960f,-0.1418f,26.0860f,  32.1760f,-1.1626f,25.7188f,  32.0030f,-1.9469f,27.5510f,  32.5847f,-2.9746f,27.1779f,  
	32.1873f,-3.8672f,28.9236f,  32.7710f,-4.9032f,28.5433f,  32.1556f,-5.8885f,30.1720f,  32.7413f,-6.9342f,29.7833f,  31.9181f,-7.9950f,31.2663f,  
	32.5057f,-9.0513f,30.8677f,  31.4884f,-10.1686f,32.1788f,  32.0779f,-11.2369f,31.7686f,  30.8833f,-12.3902f,32.8845f,  31.4744f,-13.4700f,32.4607f,  
	30.1227f,-14.6378f,33.3615f,  30.7153f,-15.7300f,32.9219f,  29.2293f,-16.8899f,33.5915f,  29.8234f,-17.9943f,33.1337f,  28.2280f,-19.1235f,33.5603f,  
	28.8235f,-20.2393f,33.0816f,  27.1458f,-21.3148f,33.2576f,  27.7425f,-22.4393f,32.7562f,  26.0111f,-23.4385f,32.6789f,  26.6094f,-24.5710f,32.1517f,  
	24.8537f,-25.4720f,31.8231f,  25.4535f,-26.6091f,31.2683f,  23.7037f,-27.3912f,30.6952f,  24.3056f,-28.5292f,30.1110f,  22.5915f,-29.1732f,29.3052f,  
	23.1955f,-30.3084f,28.6899f,  21.5468f,-30.7972f,27.6677f,  22.1542f,-31.9240f,27.0213f,  20.5991f,-32.2429f,25.8039f,  21.2101f,-33.3567f,25.1255f,  
	19.7756f,-33.4939f,23.7379f,  20.3907f,-34.5889f,23.0280f,  19.1016f,-34.5360f,21.4991f,  19.7218f,-35.6052f,20.7608f,  18.6004f,-35.3575f,19.1227f,  
	19.2261f,-36.3955f,18.3572f,  -9.4606f,11.9176f,38.5672f,  -8.8381f,11.0191f,38.2399f,  -9.6956f,10.8851f,36.4375f,  -9.0780f,9.9980f,36.1116f,  
	-9.7460f,10.0421f,34.2264f,  -9.1309f,9.1620f,33.9037f,  -9.5947f,9.3789f,31.9698f,  -8.9811f,8.5021f,31.6513f,  -9.2303f,8.8832f,29.7033f,  
	-8.6177f,8.0066f,29.3893f,  -8.6458f,8.5409f,27.4616f,  -8.0340f,7.6617f,27.1520f,  -7.8389f,8.3355f,25.2785f,  -7.2279f,7.4512f,24.9729f,  
	-6.8114f,8.2485f,23.1861f,  -6.2017f,7.3573f,22.8846f,  -5.5701f,8.2607f,21.2151f,  -4.9617f,7.3609f,20.9165f,  -4.1249f,8.3512f,19.3929f,  
	-3.5184f,7.4417f,19.0964f,  -2.4903f,8.4989f,17.7446f,  -1.8864f,7.5796f,17.4495f,  -0.6841f,8.6827f,16.2921f,  -0.0831f,7.7532f,15.9971f,  
	1.2731f,8.8811f,15.0534f,  1.8706f,7.9420f,14.7578f,  3.3576f,9.0735f,14.0433f,  3.9520f,8.1246f,13.7459f,  5.5446f,9.2387f,13.2721f,  
	6.1351f,8.2817f,12.9723f,  7.8062f,9.3581f,12.7468f,  8.3927f,8.3937f,12.4438f,  10.1139f,9.4130f,12.4699f,  10.6974f,8.4414f,12.1628f,  
	12.4393f,9.3854f,12.4394f,  13.0195f,8.4083f,12.1280f,  14.7527f,9.2602f,12.6500f,  15.3296f,8.2791f,12.3340f,  17.0245f,9.0239f,13.0920f,  
	17.5991f,8.0391f,12.7712f,  19.2270f,8.6636f,13.7520f,  19.7997f,7.6759f,13.4262f,  21.3331f,8.1689f,14.6126f,  21.9043f,7.1789f,14.2820f,  
	23.3172f,7.5316f,15.6537f,  23.8877f,6.5396f,15.3182f,  25.1560f,6.7454f,16.8515f,  25.7263f,5.7516f,16.5113f,  26.8287f,5.8064f,18.1798f,  
	27.3993f,4.8104f,17.8351f,  28.3170f,4.7128f,19.6100f,  28.8885f,3.7141f,19.2607f,  29.6060f,3.4649f,21.1114f,  30.1787f,2.4631f,20.7576f,  
	30.6837f,2.0660f,22.6522f,  31.2580f,1.0599f,22.2936f,  31.5418f,0.5211f,24.1992f,  32.1183f,-0.4907f,23.8353f,  32.1760f,-1.1626f,25.7188f,  
	32.7543f,-2.1808f,25.3495f,  32.5847f,-2.9746f,27.1779f,  33.1653f,-4.0008f,26.8019f,  32.7710f,-4.9032f,28.5433f,  33.3538f,-5.9388f,28.1597f,  
	32.7413f,-6.9342f,29.7833f,  33.3263f,-7.9805f,29.3906f,  32.5057f,-9.0513f,30.8677f,  33.0928f,-10.1097f,30.4642f,  32.0779f,-11.2369f,31.7686f,  
	32.6667f,-12.3073f,31.3524f,  31.4744f,-13.4700f,32.4607f,  32.0651f,-14.5536f,32.0294f,  30.7153f,-15.7300f,32.9219f,  31.3077f,-16.8269f,32.4732f,  
	29.8234f,-17.9943f,33.1337f,  30.4172f,-19.1041f,32.6650f,  28.8235f,-20.2393f,33.0816f,  29.4186f,-21.3598f,32.5906f,  27.7425f,-22.4393f,32.7562f,  
	28.3393f,-23.5701f,32.2398f,  26.6094f,-24.5710f,32.1517f,  27.2077f,-25.7092f,31.6078f,  25.4535f,-26.6091f,31.2683f,  26.0539f,-27.7514f,30.6946f,  
	24.3056f,-28.5292f,30.1110f,  24.9078f,-29.6721f,29.5055f,  23.1955f,-30.3084f,28.6899f,  23.8010f,-31.4465f,28.0519f,  22.1542f,-31.9240f,27.0213f,  
	22.7630f,-33.0529f,26.3496f,  21.2101f,-33.3567f,25.1255f,  21.8227f,-34.4706f,24.4199f,  20.3907f,-34.5889f,23.0280f,  21.0081f,-35.6808f,22.2910f,  
	19.7218f,-35.6052f,20.7608f,  20.3445f,-36.6696f,19.9931f,  19.2261f,-36.3955f,18.3572f,  19.8547f,-37.4257f,17.5620f,  -8.8381f,11.0191f,38.2399f,  
	-8.2205f,10.1147f,37.9152f,  -9.0780f,9.9980f,36.1116f,  -8.4645f,9.1034f,35.7891f,  -9.1309f,9.1620f,33.9037f,  -8.5199f,8.2735f,33.5845f,  
	-8.9811f,8.5021f,31.6513f,  -8.3719f,7.6163f,31.3359f,  -8.6177f,8.0066f,29.3893f,  -8.0099f,7.1209f,29.0779f,  -8.0340f,7.6617f,27.1520f,  
	-7.4274f,6.7732f,26.8445f,  -7.2279f,7.4512f,24.9729f,  -6.6226f,6.5578f,24.6694f,  -6.2017f,7.3573f,22.8846f,  -5.5979f,6.4572f,22.5840f,  
	-4.9617f,7.3609f,20.9165f,  -4.3597f,6.4527f,20.6183f,  -3.5184f,7.4417f,19.0964f,  -2.9190f,6.5251f,18.8000f,  -1.8864f,7.5796f,17.4495f,  
	-1.2896f,6.6537f,17.1536f,  -0.0831f,7.7532f,15.9971f,  0.5104f,6.8185f,15.7011f,  1.8706f,7.9420f,14.7578f,  2.4612f,6.9981f,14.4604f,  
	3.9520f,8.1246f,13.7459f,  4.5388f,7.1730f,13.4465f,  6.1351f,8.2817f,12.9723f,  6.7181f,7.3229f,12.6701f,  8.3927f,8.3937f,12.4438f,  
	8.9728f,7.4279f,12.1379f,  10.6974f,8.4414f,12.1628f,  11.2742f,7.4701f,11.8528f,  13.0195f,8.4083f,12.1280f,  13.5931f,7.4329f,11.8137f,  
	15.3296f,8.2791f,12.3340f,  15.9008f,7.3002f,12.0150f,  17.5991f,8.0391f,12.7712f,  18.1683f,7.0573f,12.4473f,  19.7997f,7.6759f,13.4262f,  
	20.3675f,6.6919f,13.0975f,  21.9043f,7.1789f,14.2820f,  22.4713f,6.1931f,13.9485f,  23.8877f,6.5396f,15.3182f,  24.4543f,5.5522f,14.9801f,  
	25.7263f,5.7516f,16.5113f,  26.2932f,4.7624f,16.1688f,  27.3993f,4.8104f,17.8351f,  27.9671f,3.8191f,17.4881f,  28.8885f,3.7141f,19.2607f,  
	29.4575f,2.7202f,18.9094f,  30.1787f,2.4631f,20.7576f,  30.7494f,1.4654f,20.4018f,  31.2580f,1.0599f,22.2936f,  31.8310f,0.0571f,21.9328f,  
	32.1183f,-0.4907f,23.8353f,  32.6932f,-1.4992f,23.4696f,  32.7543f,-2.1808f,25.3495f,  33.3317f,-3.1969f,24.9776f,  33.1653f,-4.0008f,26.8019f,  
	33.7451f,-5.0259f,26.4231f,  33.3538f,-5.9388f,28.1597f,  33.9360f,-6.9743f,27.7726f,  33.3263f,-7.9805f,29.3906f,  33.9108f,-9.0279f,28.9936f,  
	33.0928f,-10.1097f,30.4642f,  33.6794f,-11.1695f,30.0554f,  32.6667f,-12.3073f,31.3524f,  33.2554f,-13.3808f,30.9295f,  32.0651f,-14.5536f,32.0294f,  
	32.6555f,-15.6413f,31.5902f,  31.3077f,-16.8269f,32.4732f,  31.8998f,-17.9288f,32.0148f,  30.4172f,-19.1041f,32.6650f,  31.0108f,-20.2185f,32.1850f,  
	29.4186f,-21.3598f,32.5906f,  30.0139f,-22.4866f,32.0859f,  28.3393f,-23.5701f,32.2398f,  28.9361f,-24.7069f,31.7078f,  27.2077f,-25.7092f,31.6078f,  
	27.8065f,-26.8530f,31.0460f,  26.0539f,-27.7514f,30.6946f,  26.6544f,-28.8990f,30.1006f,  24.9078f,-29.6721f,29.5055f,  25.5114f,-30.8187f,28.8780f,  
	23.8010f,-31.4465f,28.0519f,  24.4077f,-32.5879f,27.3892f,  22.7630f,-33.0529f,26.3496f,  23.3732f,-34.1834f,25.6510f,  21.8227f,-34.4706f,24.4199f,  
	22.4375f,-35.5830f,23.6872f,  21.0081f,-35.6808f,22.2910f,  21.6280f,-36.7695f,21.5244f,  20.3445f,-36.6696f,19.9931f,  20.9700f,-37.7277f,19.1951f,  
	19.8547f,-37.4257f,17.5620f,  20.4864f,-38.4466f,16.7361f,  -8.2205f,10.1147f,37.9152f,  -7.6067f,9.2040f,37.5935f,  -8.4645f,9.1034f,35.7891f,  
	-7.8544f,8.2013f,35.4697f,  -8.5199f,8.2735f,33.5845f,  -7.9127f,7.3768f,33.2682f,  -8.3719f,7.6163f,31.3359f,  -7.7669f,6.7221f,31.0231f,  
	-8.0099f,7.1209f,29.0779f,  -7.4067f,6.2263f,28.7687f,  -7.4274f,6.7732f,26.8445f,  -6.8258f,5.8759f,26.5392f,  -6.6226f,6.5578f,24.6694f,  
	-6.0227f,5.6558f,24.3671f,  -5.5979f,6.4572f,22.5840f,  -4.9999f,5.5488f,22.2843f,  -4.3597f,6.4527f,20.6183f,  -3.7641f,5.5372f,20.3206f,  
	-2.9190f,6.5251f,18.8000f,  -2.3259f,5.6015f,18.5032f,  -1.2896f,6.6537f,17.1536f,  -0.6996f,5.7223f,16.8570f,  0.5104f,6.8185f,15.7011f,  
	1.0975f,5.8786f,15.4034f,  2.4612f,6.9981f,14.4604f,  3.0446f,6.0510f,14.1612f,  4.5388f,7.1730f,13.4465f,  5.1187f,6.2190f,13.1448f,  
	6.7181f,7.3229f,12.6701f,  7.2951f,6.3621f,12.3651f,  8.9728f,7.4279f,12.1379f,  9.5466f,6.4616f,11.8291f,  11.2742f,7.4701f,11.8528f,  
	11.8448f,6.4999f,11.5399f,  13.5931f,7.4329f,11.8137f,  14.1612f,6.4593f,11.4963f,  15.9008f,7.3002f,12.0150f,  16.4669f,6.3237f,11.6929f,  
	18.1683f,7.0573f,12.4473f,  18.7329f,6.0789f,12.1205f,  20.3675f,6.6919f,13.0975f,  20.9311f,5.7118f,12.7660f,  22.4713f,6.1931f,13.9485f,  
	23.0345f,5.2118f,13.6125f,  24.4543f,5.5522f,14.9801f,  25.0178f,4.5696f,14.6397f,  26.2932f,4.7624f,16.1688f,  26.8575f,3.7781f,15.8239f,  
	27.9671f,3.8191f,17.4881f,  28.5326f,2.8328f,17.1391f,  29.4575f,2.7202f,18.9094f,  30.0248f,1.7307f,18.5561f,  30.7494f,1.4654f,20.4018f,  
	31.3189f,0.4715f,20.0439f,  31.8310f,0.0571f,21.9328f,  32.4025f,-0.9417f,21.5705f,  32.6932f,-1.4992f,23.4696f,  33.2674f,-2.5051f,23.1016f,  
	33.3317f,-3.1969f,24.9776f,  33.9084f,-4.2110f,24.6034f,  33.7451f,-5.0259f,26.4231f,  34.3244f,-6.0500f,26.0414f,  33.9360f,-6.9743f,27.7726f,  
	34.5179f,-8.0100f,27.3820f,  33.9108f,-9.0279f,28.9936f,  34.4950f,-10.0761f,28.5921f,  33.6794f,-11.1695f,30.0554f,  34.2659f,-12.2316f,29.6410f,  
	33.2554f,-13.3808f,30.9295f,  33.8439f,-14.4578f,30.4997f,  32.6555f,-15.6413f,31.5902f,  33.2459f,-16.7335f,31.1424f,  31.8998f,-17.9288f,32.0148f,  
	32.4918f,-19.0349f,31.5464f,  31.0108f,-20.2185f,32.1850f,  31.6046f,-21.3389f,31.6926f,  30.0139f,-22.4866f,32.0859f,  30.6092f,-23.6193f,31.5668f,  
	28.9361f,-24.7069f,31.7078f,  29.5334f,-25.8494f,31.1591f,  27.8065f,-26.8530f,31.0460f,  28.4053f,-28.0026f,30.4651f,  26.6544f,-28.8990f,30.1006f,  
	27.2562f,-30.0512f,29.4855f,  25.5114f,-30.8187f,28.8780f,  26.1161f,-31.9696f,28.2265f,  24.4077f,-32.5879f,27.3892f,  25.0156f,-33.7319f,26.7003f,  
	23.3732f,-34.1834f,25.6510f,  23.9854f,-35.3134f,24.9255f,  22.4375f,-35.5830f,23.6872f,  23.0546f,-36.6938f,22.9248f,  21.6280f,-36.7695f,21.5244f,  
	22.2504f,-37.8537f,20.7271f,  20.9700f,-37.7277f,19.1951f,  21.5984f,-38.7784f,18.3657f,  20.4864f,-38.4466f,16.7361f,  21.1213f,-39.4564f,15.8796f,  
	-7.6067f,9.2040f,37.5935f,  -6.9962f,8.2869f,37.2747f,  -7.8544f,8.2013f,35.4697f,  -7.2479f,7.2921f,35.1532f,  -7.9127f,7.3768f,33.2682f,  
	-7.3093f,6.4724f,32.9546f,  -7.7669f,6.7221f,31.0231f,  -7.1660f,5.8198f,30.7127f,  -7.4067f,6.2263f,28.7687f,  -6.8077f,5.3234f,28.4618f,  
	-6.8258f,5.8759f,26.5392f,  -6.2289f,4.9703f,26.2352f,  -6.0227f,5.6558f,24.3671f,  -5.4278f,4.7456f,24.0658f,  -4.9999f,5.5488f,22.2843f,  
	-4.4074f,4.6330f,21.9851f,  -3.7641f,5.5372f,20.3206f,  -3.1741f,4.6146f,20.0226f,  -2.3259f,5.6015f,18.5032f,  -1.7389f,4.6720f,18.2057f,  
	-0.6996f,5.7223f,16.8570f,  -0.1155f,4.7852f,16.5589f,  1.0975f,5.8786f,15.4034f,  1.6782f,4.9349f,15.1042f,  3.0446f,6.0510f,14.1612f,  
	3.6218f,5.1008f,13.8598f,  5.1187f,6.2190f,13.1448f,  5.6930f,5.2624f,12.8404f,  7.2951f,6.3621f,12.3651f,  7.8662f,5.4003f,12.0572f,  
	9.5466f,6.4616f,11.8291f,  10.1145f,5.4961f,11.5174f,  11.8448f,6.4999f,11.5399f,  12.4102f,5.5310f,11.2239f,  14.1612f,6.4593f,11.4963f,  
	14.7246f,5.4877f,11.1757f,  16.4669f,6.3237f,11.6929f,  17.0286f,5.3503f,11.3677f,  18.7329f,6.0789f,12.1205f,  19.2936f,5.1041f,11.7907f,  
	20.9311f,5.7118f,12.7660f,  21.4913f,4.7362f,12.4317f,  23.0345f,5.2118f,13.6125f,  23.5948f,4.2352f,13.2738f,  25.0178f,4.5696f,14.6397f,  
	25.5788f,3.5919f,14.2968f,  26.8575f,3.7781f,15.8239f,  27.4196f,2.7990f,15.4771f,  28.5326f,2.8328f,17.1391f,  29.0964f,1.8512f,16.7881f,  
	30.0248f,1.7307f,18.5561f,  30.5909f,0.7455f,18.2008f,  31.3189f,0.4715f,20.0439f,  31.8871f,-0.5177f,19.6846f,  32.4025f,-0.9417f,21.5705f,  
	32.9735f,-1.9372f,21.2060f,  33.2674f,-2.5051f,23.1016f,  33.8411f,-3.5081f,22.7317f,  33.9084f,-4.2110f,24.6034f,  34.4849f,-5.2234f,24.2267f,  
	34.3244f,-6.0500f,26.0414f,  34.9036f,-7.0735f,25.6568f,  34.5179f,-8.0100f,27.3820f,  35.0997f,-9.0458f,26.9874f,  34.4950f,-10.0761f,28.5921f,  
	35.0793f,-11.1258f,28.1858f,  34.2659f,-12.2316f,29.6410f,  34.8524f,-13.2965f,29.2205f,  33.8439f,-14.4578f,30.4997f,  34.4325f,-15.5385f,30.0625f,  
	33.2459f,-16.7335f,31.1424f,  33.8363f,-17.8293f,30.6855f,  32.4918f,-19.0349f,31.5464f,  33.0841f,-20.1467f,31.0666f,  31.6046f,-21.3389f,31.6926f,  
	32.1985f,-22.4651f,31.1870f,  30.6092f,-23.6193f,31.5668f,  31.2050f,-24.7579f,31.0321f,  29.5334f,-25.8494f,31.1591f,  30.1306f,-26.9980f,30.5926f,  
	28.4053f,-28.0026f,30.4651f,  29.0054f,-29.1573f,29.8639f,  27.2562f,-30.0512f,29.4855f,  27.8590f,-31.2085f,28.8472f,  26.1161f,-31.9696f,28.2265f,  
	26.7218f,-33.1240f,27.5496f,  25.0156f,-33.7319f,26.7003f,  25.6254f,-34.8768f,25.9848f,  23.9854f,-35.3134f,24.9255f,  24.5998f,-36.4435f,24.1705f,  
	23.0546f,-36.6938f,22.9248f,  23.6739f,-37.8016f,22.1317f,  22.2504f,-37.8537f,20.7271f,  22.8755f,-38.9322f,19.8978f,  21.5984f,-38.7784f,18.3657f,  
	22.2299f,-39.8198f,17.5047f,  21.1213f,-39.4564f,15.8796f,  21.7594f,-40.4538f,14.9907f,  -6.9962f,8.2869f,37.2747f,  -6.3889f,7.3636f,36.9584f,  
	-7.2479f,7.2921f,35.1532f,  -6.6448f,6.3760f,34.8390f,  -7.3093f,6.4724f,32.9546f,  -6.7094f,5.5607f,32.6431f,  -7.1660f,5.8198f,30.7127f,  
	-6.5686f,4.9095f,30.4044f,  -6.8077f,5.3234f,28.4618f,  -6.2129f,4.4125f,28.1563f,  -6.2289f,4.9703f,26.2352f,  -5.6363f,4.0566f,25.9323f,  
	-5.4278f,4.7456f,24.0658f,  -4.8378f,3.8278f,23.7652f,  -4.4074f,4.6330f,21.9851f,  -3.8198f,3.7099f,21.6859f,  -3.1741f,4.6146f,20.0226f,  
	-2.5896f,3.6857f,19.7242f,  -1.7389f,4.6720f,18.2057f,  -1.1571f,3.7365f,17.9070f,  -0.1155f,4.7852f,16.5589f,  0.4629f,3.8438f,16.2594f,  
	1.6782f,4.9349f,15.1042f,  2.2531f,3.9876f,14.8030f,  3.6218f,5.1008f,13.8598f,  4.1938f,4.1476f,13.5559f,  5.6930f,5.2624f,12.8404f,  
	6.2619f,4.3042f,12.5333f,  7.8662f,5.4003f,12.0572f,  8.4319f,4.4385f,11.7465f,  10.1145f,5.4961f,11.5174f,  10.6776f,4.5311f,11.2026f,  
	12.4102f,5.5310f,11.2239f,  12.9711f,4.5636f,10.9047f,  14.7246f,5.4877f,11.1757f,  15.2837f,4.5187f,10.8520f,  17.0286f,5.3503f,11.3677f,  
	17.5866f,4.3801f,11.0395f,  19.2936f,5.1041f,11.7907f,  19.8509f,4.1334f,11.4580f,  21.4913f,4.7362f,12.4317f,  22.0487f,3.7650f,12.0947f,  
	23.5948f,4.2352f,13.2738f,  24.1528f,3.2636f,12.9327f,  25.5788f,3.5919f,14.2968f,  26.1378f,2.6195f,13.9517f,  27.4196f,2.7990f,15.4771f,  
	27.9803f,1.8249f,15.1281f,  29.0964f,1.8512f,16.7881f,  29.6593f,0.8744f,16.4351f,  30.5909f,0.7455f,18.2008f,  31.1559f,-0.2344f,17.8442f,  
	31.8871f,-0.5177f,19.6846f,  32.4550f,-1.5031f,19.3232f,  32.9735f,-1.9372f,21.2060f,  33.5442f,-2.9293f,20.8399f,  33.8411f,-3.5081f,22.7317f,  
	34.4147f,-4.5087f,22.3595f,  34.4849f,-5.2234f,24.2267f,  35.0615f,-6.2343f,23.8477f,  34.9036f,-7.0735f,25.6568f,  35.4830f,-8.0962f,25.2689f,  
	35.0997f,-9.0458f,26.9874f,  35.6817f,-10.0822f,26.5889f,  35.0793f,-11.1258f,28.1858f,  35.6638f,-12.1774f,27.7744f,  34.8524f,-13.2965f,29.2205f,  
	35.4392f,-14.3643f,28.7937f,  34.4325f,-15.5385f,30.0625f,  35.0212f,-16.6224f,29.6173f,  33.8363f,-17.8293f,30.6855f,  34.4271f,-18.9304f,30.2186f,  
	33.0841f,-20.1467f,31.0666f,  33.6765f,-21.2639f,30.5749f,  32.1985f,-22.4651f,31.1870f,  32.7929f,-23.5969f,30.6671f,  31.2050f,-24.7579f,31.0321f,  
	31.8008f,-25.9025f,30.4810f,  30.1306f,-26.9980f,30.5926f,  30.7292f,-28.1521f,30.0068f,  29.0054f,-29.1573f,29.8639f,  29.6065f,-30.3177f,29.2408f,  
	27.8590f,-31.2085f,28.8472f,  28.4626f,-32.3701f,28.1847f,  26.7218f,-33.1240f,27.5496f,  27.3293f,-34.2805f,26.8468f,  25.6254f,-34.8768f,25.9848f,  
	26.2372f,-36.0230f,25.2402f,  24.5998f,-36.4435f,24.1705f,  25.2163f,-37.5720f,23.3849f,  23.6739f,-37.8016f,22.1317f,  24.2960f,-38.9054f,21.3063f,  
	22.8755f,-38.9322f,19.8978f,  23.5038f,-40.0031f,19.0363f,  22.2299f,-39.8198f,17.5047f,  22.8646f,-40.8506f,16.6103f,  21.7594f,-40.4538f,14.9907f,  
	22.4007f,-41.4369f,14.0694f,  -6.3889f,7.3636f,36.9584f,  -5.7848f,6.4344f,36.6443f,  -6.6448f,6.3760f,34.8390f,  -6.0449f,5.4533f,34.5270f,  
	-6.7094f,5.5607f,32.6431f,  -6.1127f,4.6415f,32.3337f,  -6.5686f,4.9095f,30.4044f,  -5.9749f,3.9918f,30.0976f,  -6.2129f,4.4125f,28.1563f,  
	-5.6217f,3.4938f,27.8520f,  -5.6363f,4.0566f,25.9323f,  -5.0479f,3.1355f,25.6303f,  -4.8378f,3.8278f,23.7652f,  -4.2520f,2.9028f,23.4647f,  
	-3.8198f,3.7099f,21.6859f,  -3.2371f,2.7803f,21.3864f,  -2.5896f,3.6857f,19.7242f,  -2.0096f,2.7506f,19.4247f,  -1.1571f,3.7365f,17.9070f,  
	-0.5806f,2.7963f,17.6070f,  0.4629f,3.8438f,16.2594f,  1.0360f,2.8982f,15.9580f,  2.2531f,3.9876f,14.8030f,  2.8234f,3.0366f,14.4992f,  
	4.1938f,4.1476f,13.5559f,  4.7610f,3.1920f,13.2492f,  6.2619f,4.3042f,12.5333f,  6.8258f,3.3453f,12.2233f,  8.4319f,4.4385f,11.7465f,  
	8.9931f,3.4768f,11.4327f,  10.6776f,4.5311f,11.2026f,  11.2365f,3.5671f,10.8845f,  12.9711f,4.5636f,10.9047f,  13.5281f,3.5982f,10.5823f,  
	15.2837f,4.5187f,10.8520f,  15.8395f,3.5525f,10.5251f,  17.5866f,4.3801f,11.0395f,  18.1416f,3.4138f,10.7081f,  19.8509f,4.1334f,11.4580f,  
	20.4057f,3.1670f,11.1224f,  22.0487f,3.7650f,12.0947f,  22.6038f,2.7987f,11.7550f,  24.1528f,3.2636f,12.9327f,  24.7089f,2.2972f,12.5892f,  
	26.1378f,2.6195f,13.9517f,  26.6955f,1.6522f,13.6044f,  27.9803f,1.8249f,15.1281f,  28.5402f,0.8558f,14.7770f,  29.6593f,0.8744f,16.4351f,  
	30.2214f,-0.0968f,16.0807f,  31.1559f,-0.2344f,17.8442f,  31.7209f,-1.2101f,17.4855f,  32.4550f,-1.5031f,19.3232f,  33.0228f,-2.4844f,18.9604f,  
	33.5442f,-2.9293f,20.8399f,  34.1151f,-3.9181f,20.4718f,  34.4147f,-4.5087f,22.3595f,  34.9887f,-5.5068f,21.9855f,  35.0615f,-6.2343f,23.8477f,  
	35.6384f,-7.2437f,23.4658f,  35.4830f,-8.0962f,25.2689f,  36.0629f,-9.1187f,24.8776f,  35.6817f,-10.0822f,26.5889f,  36.2643f,-11.1196f,26.1861f,  
	35.6638f,-12.1774f,27.7744f,  36.2489f,-13.2311f,27.3576f,  35.4392f,-14.3643f,28.7937f,  36.0264f,-15.4346f,28.3598f,  35.0212f,-16.6224f,29.6173f,  
	35.6106f,-17.7108f,29.1632f,  34.4271f,-18.9304f,30.2186f,  35.0182f,-20.0364f,29.7411f,  33.6765f,-21.2639f,30.5749f,  34.2697f,-22.3866f,30.0703f,  
	32.7929f,-23.5969f,30.6671f,  33.3872f,-24.7348f,30.1322f,  31.8008f,-25.9025f,30.4810f,  32.3980f,-27.0530f,29.9118f,  30.7292f,-28.1521f,30.0068f,  
	31.3287f,-29.3124f,29.4003f,  29.6065f,-30.3177f,29.2408f,  30.2083f,-31.4831f,28.5944f,  28.4626f,-32.3701f,28.1847f,  29.0681f,-33.5347f,27.4968f,  
	27.3293f,-34.2805f,26.8468f,  27.9389f,-35.4395f,26.1154f,  26.2372f,-36.0230f,25.2402f,  26.8511f,-37.1692f,24.4653f,  25.2163f,-37.5720f,23.3849f,  
	25.8354f,-38.6983f,22.5670f,  24.2960f,-38.9054f,21.3063f,  24.9211f,-40.0032f,20.4481f,  23.5038f,-40.0031f,19.0363f,  24.1353f,-41.0653f,18.1405f,  
	22.8646f,-40.8506f,16.6103f,  23.5028f,-41.8689f,15.6822f,  22.4007f,-41.4369f,14.0694f,  23.0455f,-42.4037f,13.1161f,  -5.7848f,6.4344f,36.6443f,  
	-5.1837f,5.4996f,36.3320f,  -6.0449f,5.4533f,34.5270f,  -5.4476f,4.5239f,34.2168f,  -6.1127f,4.6415f,32.3337f,  -5.5191f,3.7155f,32.0258f,  
	-5.9749f,3.9918f,30.0976f,  -5.3844f,3.0667f,29.7919f,  -5.6217f,3.4938f,27.8520f,  -5.0342f,2.5679f,27.5485f,  -5.0479f,3.1355f,25.6303f,  
	-4.4632f,2.2072f,25.3284f,  -4.2520f,2.9028f,23.4647f,  -3.6704f,1.9712f,23.1640f,  -3.2371f,2.7803f,21.3864f,  -2.6584f,1.8443f,21.0860f,  
	-2.0096f,2.7506f,19.4247f,  -1.4343f,1.8104f,19.1240f,  -0.5806f,2.7963f,17.6070f,  -0.0086f,1.8515f,17.3053f,  1.0360f,2.8982f,15.9580f,  
	1.6050f,1.9486f,15.6542f,  2.8234f,3.0366f,14.4992f,  3.3893f,2.0829f,14.1927f,  4.7610f,3.1920f,13.2492f,  5.3236f,2.2353f,12.9397f,  
	6.8258f,3.3453f,12.2233f,  7.3855f,2.3860f,11.9102f,  8.9931f,3.4768f,11.4327f,  9.5505f,2.5154f,11.1154f,  11.2365f,3.5671f,10.8845f,  
	11.7919f,2.6047f,10.5630f,  13.5281f,3.5982f,10.5823f,  14.0821f,2.6352f,10.2563f,  15.8395f,3.5525f,10.5251f,  16.3924f,2.5897f,10.1948f,  
	18.1416f,3.4138f,10.7081f,  18.6942f,2.4515f,10.3736f,  20.4057f,3.1670f,11.1224f,  20.9585f,2.2054f,10.7839f,  22.6038f,2.7987f,11.7550f,  
	23.1575f,1.8377f,11.4127f,  24.7089f,2.2972f,12.5892f,  25.2640f,1.3361f,12.2432f,  26.6955f,1.6522f,13.6044f,  27.2527f,0.6902f,13.2548f,  
	28.5402f,0.8558f,14.7770f,  29.0995f,-0.1071f,14.4244f,  30.2214f,-0.0968f,16.0807f,  30.7835f,-1.0633f,15.7243f,  31.7209f,-1.2101f,17.4855f,  
	32.2859f,-2.1810f,17.1255f,  33.0228f,-2.4844f,18.9604f,  33.5911f,-3.4616f,18.5959f,  34.1151f,-3.9181f,20.4718f,  34.6865f,-4.9035f,20.1022f,  
	34.9887f,-5.5068f,21.9855f,  35.5633f,-6.5027f,21.6091f,  35.6384f,-7.2437f,23.4658f,  36.2162f,-8.2518f,23.0811f,  36.0629f,-9.1187f,24.8776f,  
	36.6435f,-10.1412f,24.4829f,  36.2643f,-11.1196f,26.1861f,  36.8476f,-12.1582f,25.7788f,  36.2489f,-13.2311f,27.3576f,  36.8346f,-14.2865f,26.9348f,  
	36.0264f,-15.4346f,28.3598f,  36.6145f,-16.5088f,27.9184f,  35.6106f,-17.7108f,29.1632f,  36.2006f,-18.8035f,28.6999f,  35.0182f,-20.0364f,29.7411f,  
	35.6102f,-21.1475f,29.2520f,  34.2697f,-22.3866f,30.0703f,  34.8629f,-23.5150f,29.5521f,  33.3872f,-24.7348f,30.1322f,  33.9833f,-25.8786f,29.5804f,  
	32.3980f,-27.0530f,29.9118f,  32.9961f,-28.2098f,29.3232f,  31.3287f,-29.3124f,29.4003f,  31.9289f,-30.4781f,28.7720f,  30.2083f,-31.4831f,28.5944f,  
	30.8120f,-32.6523f,27.9238f,  29.0681f,-33.5347f,27.4968f,  29.6756f,-34.7030f,26.7813f,  27.9389f,-35.4395f,26.1154f,  28.5503f,-36.5998f,25.3543f,  
	26.8511f,-37.1692f,24.4653f,  27.4675f,-38.3145f,23.6583f,  25.8354f,-38.6983f,22.5670f,  26.4576f,-39.8202f,21.7161f,  24.9211f,-40.0032f,20.4481f,  
	25.5495f,-41.0942f,19.5552f,  24.1353f,-41.0653f,18.1405f,  24.7702f,-42.1168f,17.2100f,  23.5028f,-41.8689f,15.6822f,  24.1445f,-42.8726f,14.7206f,  
	23.0455f,-42.4037f,13.1161f,  23.6935f,-43.3530f,12.1272f,  -5.1837f,5.4996f,36.3320f,  -4.5849f,4.5587f,36.0214f,  -5.4476f,4.5239f,34.2168f,  
	-4.8533f,3.5883f,33.9080f,  -5.5191f,3.7155f,32.0258f,  -4.9283f,2.7826f,31.7190f,  -5.3844f,3.0667f,29.7919f,  -4.7968f,2.1346f,29.4871f,  
	-5.0342f,2.5679f,27.5485f,  -4.4498f,1.6350f,27.2453f,  -4.4632f,2.2072f,25.3284f,  -3.8820f,1.2723f,25.0265f,  -3.6704f,1.9712f,23.1640f,  
	-3.0922f,1.0332f,22.8625f,  -2.6584f,1.8443f,21.0860f,  -2.0837f,0.9030f,20.7845f,  -1.4343f,1.8104f,19.1240f,  -0.8630f,0.8652f,18.8217f,  
	-0.0086f,1.8515f,17.3053f,  0.5596f,0.9022f,17.0011f,  1.6050f,1.9486f,15.6542f,  2.1701f,0.9957f,15.3477f,  3.3893f,2.0829f,14.1927f,  
	3.9510f,1.1273f,13.8834f,  5.3236f,2.2353f,12.9397f,  5.8823f,1.2775f,12.6270f,  7.3855f,2.3860f,11.9102f,  7.9418f,1.4264f,11.5935f,  
	9.5505f,2.5154f,11.1154f,  10.1046f,1.5551f,10.7946f,  11.7919f,2.6047f,10.5630f,  12.3444f,1.6441f,10.2378f,  14.0821f,2.6352f,10.2563f,  
	14.6334f,1.6753f,9.9269f,  16.3924f,2.5897f,10.1948f,  16.9433f,1.6306f,9.8611f,  18.6942f,2.4515f,10.3736f,  19.2450f,1.4937f,10.0359f,  
	20.9585f,2.2054f,10.7839f,  21.5101f,1.2489f,10.4424f,  23.1575f,1.8377f,11.4127f,  23.7103f,0.8820f,11.0677f,  25.2640f,1.3361f,12.2432f,  
	25.8187f,0.3804f,11.8947f,  27.2527f,0.6902f,13.2548f,  27.8094f,-0.2653f,12.9036f,  29.0995f,-0.1071f,14.4244f,  29.6591f,-1.0651f,14.0697f,  
	30.7835f,-1.0633f,15.7243f,  31.3461f,-2.0243f,15.3664f,  32.2859f,-2.1810f,17.1255f,  32.8517f,-3.1472f,16.7638f,  33.5911f,-3.4616f,18.5959f,  
	34.1601f,-4.4347f,18.2300f,  34.6865f,-4.9035f,20.1022f,  35.2590f,-5.8859f,19.7305f,  35.5633f,-6.5027f,21.6091f,  36.1390f,-7.4964f,21.2304f,  
	36.2162f,-8.2518f,23.0811f,  36.7950f,-9.2590f,22.6938f,  36.6435f,-10.1412f,24.4829f,  37.2252f,-11.1639f,24.0844f,  36.8476f,-12.1582f,25.7788f,  
	37.4319f,-13.1976f,25.3665f,  36.8346f,-14.2865f,26.9348f,  37.4214f,-15.3449f,26.5055f,  36.6145f,-16.5088f,27.9184f,  37.2033f,-17.5864f,27.4689f,  
	36.2006f,-18.8035f,28.6999f,  36.7915f,-19.9008f,28.2262f,  35.6102f,-21.1475f,29.2520f,  36.2025f,-22.2638f,28.7508f,  34.8629f,-23.5150f,29.5521f,  
	35.4579f,-24.6491f,29.0184f,  33.9833f,-25.8786f,29.5804f,  34.5803f,-27.0287f,29.0108f,  32.9961f,-28.2098f,29.3232f,  33.5951f,-29.3724f,28.7142f,  
	31.9289f,-30.4781f,28.7720f,  32.5312f,-31.6483f,28.1203f,  30.8120f,-32.6523f,27.9238f,  31.4177f,-33.8261f,27.2264f,  29.6756f,-34.7030f,26.7813f,  
	30.2850f,-35.8736f,26.0369f,  28.5503f,-36.5998f,25.3543f,  29.1644f,-37.7606f,24.5616f,  27.4675f,-38.3145f,23.6583f,  28.0871f,-39.4571f,22.8184f,  
	26.4576f,-39.8202f,21.7161f,  27.0832f,-40.9370f,20.8300f,  25.5495f,-41.0942f,19.5552f,  26.1815f,-42.1764f,18.6266f,  24.7702f,-42.1168f,17.2100f,  
	25.4091f,-43.1555f,16.2446f,  24.1445f,-42.8726f,14.7206f,  24.7897f,-43.8607f,13.7217f,  23.6935f,-43.3530f,12.1272f,  24.3448f,-44.2821f,11.1046f,  
	-4.5849f,4.5587f,36.0214f,  -3.9888f,3.6125f,35.7118f,  -4.8533f,3.5883f,33.9080f,  -4.2612f,2.6465f,33.6001f,  -4.9283f,2.7826f,31.7190f,  
	-4.3400f,1.8432f,31.4129f,  -4.7968f,2.1346f,29.4871f,  -4.2120f,1.1959f,29.1826f,  -4.4498f,1.6350f,27.2453f,  -3.8684f,0.6956f,26.9420f,  
	-3.8820f,1.2723f,25.0265f,  -3.3038f,0.3310f,24.7237f,  -3.0922f,1.0332f,22.8625f,  -2.5176f,0.0895f,22.5599f,  -2.0837f,0.9030f,20.7845f,  
	-1.5124f,-0.0437f,20.4812f,  -0.8630f,0.8652f,18.8217f,  -0.2949f,-0.0848f,18.5169f,  0.5596f,0.9022f,17.0011f,  1.1245f,-0.0509f,16.6943f,  
	2.1701f,0.9957f,15.3477f,  2.7315f,0.0403f,15.0383f,  3.9510f,1.1273f,13.8834f,  4.5094f,0.1701f,13.5708f,  5.8823f,1.2775f,12.6270f,  
	6.4381f,0.3188f,12.3106f,  7.9418f,1.4264f,11.5935f,  8.4953f,0.4673f,11.2731f,  10.1046f,1.5551f,10.7946f,  10.6563f,0.5960f,10.4699f,  
	12.3444f,1.6441f,10.2378f,  12.8947f,0.6860f,9.9089f,  14.6334f,1.6753f,9.9269f,  15.1830f,0.7185f,9.5937f,  16.9433f,1.6306f,9.8611f,  
	17.4926f,0.6758f,9.5240f,  19.2450f,1.4937f,10.0359f,  19.7948f,0.5408f,9.6950f,  21.5101f,1.2489f,10.4424f,  22.0610f,0.2976f,10.0980f,  
	23.7103f,0.8820f,11.0677f,  24.2629f,-0.0683f,10.7198f,  25.8187f,0.3804f,11.8947f,  26.3733f,-0.5685f,11.5443f,  27.8094f,-0.2653f,12.9036f,  
	28.3668f,-1.2156f,12.5500f,  29.6591f,-1.0651f,14.0697f,  30.2194f,-2.0171f,13.7134f,  31.3461f,-2.0243f,15.3664f,  31.9096f,-2.9801f,15.0069f,  
	32.8517f,-3.1472f,16.7638f,  33.4186f,-4.1084f,16.4009f,  34.1601f,-4.4347f,18.2300f,  34.7305f,-5.4039f,17.8622f,  35.2590f,-5.8859f,19.7305f,  
	35.8328f,-6.8652f,19.3569f,  36.1390f,-7.4964f,21.2304f,  36.7161f,-8.4880f,20.8495f,  36.7950f,-9.2590f,22.6938f,  37.3752f,-10.2653f,22.3034f,  
	37.2252f,-11.1639f,24.0844f,  37.8082f,-12.1866f,23.6818f,  37.4319f,-13.1976f,25.3665f,  38.0176f,-14.2390f,24.9487f,  37.4214f,-15.3449f,26.5055f,  
	38.0093f,-16.4058f,26.0694f,  37.2033f,-17.5864f,27.4689f,  37.7934f,-18.6679f,27.0103f,  36.7915f,-19.9008f,28.2262f,  37.3831f,-21.0026f,27.7419f,  
	36.2025f,-22.2638f,28.7508f,  36.7967f,-23.3856f,28.2354f,  35.4579f,-24.6491f,29.0184f,  36.0541f,-25.7895f,28.4683f,  34.5803f,-27.0287f,29.0108f,  
	35.1783f,-28.1848f,28.4222f,  33.5951f,-29.3724f,28.7142f,  34.1961f,-30.5399f,28.0831f,  32.5312f,-31.6483f,28.1203f,  33.1354f,-32.8238f,27.4432f,  
	31.4177f,-33.8261f,27.2264f,  32.0253f,-35.0031f,26.5012f,  30.2850f,-35.8736f,26.0369f,  30.8969f,-37.0460f,25.2615f,  29.1644f,-37.7606f,24.5616f,  
	29.7816f,-38.9201f,23.7363f,  28.0871f,-39.4571f,22.8184f,  28.7100f,-40.5962f,21.9433f,  27.0832f,-40.9370f,20.8300f,  27.7125f,-42.0468f,19.9079f,  
	26.1815f,-42.1764f,18.6266f,  26.8178f,-43.2476f,17.6621f,  25.4091f,-43.1555f,16.2446f,  26.0517f,-44.1805f,15.2407f,  24.7897f,-43.8607f,13.7217f,  
	25.4387f,-44.8304f,12.6873f,  24.3448f,-44.2821f,11.1046f,  24.9994f,-45.1889f,10.0487f,  -3.9888f,3.6125f,35.7118f,  -3.3947f,2.6606f,35.4031f,  
	-4.2612f,2.6465f,33.6001f,  -3.6714f,1.6986f,33.2928f,  -4.3400f,1.8432f,31.4129f,  -3.7540f,0.8975f,31.1069f,  -4.2120f,1.1959f,29.1826f,  
	-3.6297f,0.2508f,28.8778f,  -3.8684f,0.6956f,26.9420f,  -3.2895f,-0.2502f,26.6379f,  -3.3038f,0.3310f,24.7237f,  -2.7287f,-0.6161f,24.4199f,  
	-2.5176f,0.0895f,22.5599f,  -1.9459f,-0.8597f,22.2556f,  -1.5124f,-0.0437f,20.4812f,  -0.9440f,-0.9954f,20.1757f,  -0.2949f,-0.0848f,18.5169f,  
	0.2702f,-1.0390f,18.2095f,  1.1245f,-0.0509f,16.6943f,  1.6861f,-1.0069f,16.3845f,  2.7315f,0.0403f,15.0383f,  3.2900f,-0.9171f,14.7255f,  
	4.5094f,0.1701f,13.5708f,  5.0651f,-0.7885f,13.2544f,  6.4381f,0.3188f,12.3106f,  6.9914f,-0.6399f,11.9903f,  8.4953f,0.4673f,11.2731f,  
	9.0465f,-0.4910f,10.9486f,  10.6563f,0.5960f,10.4699f,  11.2059f,-0.3610f,10.1412f,  12.8947f,0.6860f,9.9089f,  13.4434f,-0.2692f,9.5760f,  
	15.1830f,0.7185f,9.5937f,  15.7312f,-0.2343f,9.2568f,  17.4926f,0.6758f,9.5240f,  18.0411f,-0.2744f,9.1832f,  19.7948f,0.5408f,9.6950f,  
	20.3443f,-0.4070f,9.3507f,  22.0610f,0.2976f,10.0980f,  22.6119f,-0.6482f,9.7504f,  24.2629f,-0.0683f,10.7198f,  24.8156f,-1.0117f,10.3698f,  
	26.3733f,-0.5685f,11.5443f,  26.9287f,-1.5121f,11.1913f,  28.3668f,-1.2156f,12.5500f,  28.9251f,-2.1594f,12.1946f,  30.2194f,-2.0171f,13.7134f,  
	30.7809f,-2.9633f,13.3553f,  31.9096f,-2.9801f,15.0069f,  32.4745f,-3.9301f,14.6461f,  33.4186f,-4.1084f,16.4009f,  33.9871f,-5.0650f,16.0361f,  
	34.7305f,-5.4039f,17.8622f,  35.3026f,-6.3692f,17.4927f,  35.8328f,-6.8652f,19.3569f,  36.4083f,-7.8415f,18.9815f,  36.7161f,-8.4880f,20.8495f,  
	37.2949f,-9.4777f,20.4662f,  37.3752f,-10.2653f,22.3034f,  37.9570f,-11.2707f,21.9095f,  37.8082f,-12.1866f,23.6818f,  38.3929f,-13.2101f,23.2746f,  
	38.0176f,-14.2390f,24.9487f,  38.6047f,-15.2820f,24.5251f,  38.0093f,-16.4058f,26.0694f,  38.5988f,-17.4698f,25.6253f,  37.7934f,-18.6679f,27.0103f,  
	38.3844f,-19.7532f,26.5426f,  37.3831f,-21.0026f,27.7419f,  37.9768f,-22.1096f,27.2447f,  36.7967f,-23.3856f,28.2354f,  37.3924f,-24.5133f,27.7051f,  
	36.0541f,-25.7895f,28.4683f,  36.6514f,-26.9358f,27.9008f,  35.1783f,-28.1848f,28.4222f,  35.7785f,-29.3460f,27.8128f,  34.1961f,-30.5399f,28.0831f,  
	34.7992f,-31.7132f,27.4279f,  33.1354f,-32.8238f,27.4432f,  33.7416f,-34.0032f,26.7394f,  32.0253f,-35.0031f,26.5012f,  32.6355f,-36.1831f,25.7460f,  
	30.8969f,-37.0460f,25.2615f,  31.5121f,-38.2185f,24.4540f,  29.7816f,-38.9201f,23.7363f,  30.4022f,-40.0778f,22.8759f,  28.7100f,-40.5962f,21.9433f,  
	29.3368f,-41.7301f,21.0318f,  27.7125f,-42.0468f,19.9079f,  28.3462f,-43.1474f,18.9489f,  26.8178f,-43.2476f,17.6621f,  27.4578f,-44.3071f,16.6581f,  
	26.0517f,-44.1805f,15.2407f,  26.6984f,-45.1890f,14.1995f,  25.4387f,-44.8304f,12.6873f,  26.0916f,-45.7795f,11.6173f,  24.9994f,-45.1889f,10.0487f,  
	25.6571f,-46.0717f,8.9573f,  -3.3947f,2.6606f,35.4031f,  -2.8025f,1.7033f,35.0947f,  -3.6714f,1.6986f,33.2928f,  -3.0835f,0.7449f,32.9856f,  
	-3.7540f,0.8975f,31.1069f,  -3.1701f,-0.0543f,30.8008f,  -3.6297f,0.2508f,28.8778f,  -3.0496f,-0.7004f,28.5723f,  -3.2895f,-0.2502f,26.6379f,  
	-2.7132f,-1.2017f,26.3327f,  -2.7287f,-0.6161f,24.4199f,  -2.1559f,-1.5688f,24.1144f,  -1.9459f,-0.8597f,22.2556f,  -1.3766f,-1.8141f,21.9491f,  
	-0.9440f,-0.9954f,20.1757f,  -0.3782f,-1.9517f,19.8675f,  0.2702f,-1.0390f,18.2095f,  0.8325f,-1.9966f,17.8991f,  1.6861f,-1.0069f,16.3845f,  
	2.2452f,-1.9656f,16.0713f,  3.2900f,-0.9171f,14.7255f,  3.8462f,-1.8765f,14.4088f,  5.0651f,-0.7885f,13.2544f,  5.6186f,-1.7477f,12.9339f,  
	6.9914f,-0.6399f,11.9903f,  7.5427f,-1.5985f,11.6657f,  9.0465f,-0.4910f,10.9486f,  9.5961f,-1.4480f,10.6198f,  11.2059f,-0.3610f,10.1412f,  
	11.7543f,-1.3157f,9.8081f,  13.4434f,-0.2692f,9.5760f,  13.9910f,-1.2210f,9.2389f,  15.7312f,-0.2343f,9.2568f,  16.2789f,-1.1829f,8.9158f,  
	18.0411f,-0.2744f,9.1832f,  18.5895f,-1.2199f,8.8387f,  20.3443f,-0.4070f,9.3507f,  20.8939f,-1.3495f,9.0029f,  22.6119f,-0.6482f,9.7504f,  
	23.1632f,-1.5872f,9.4001f,  24.8156f,-1.0117f,10.3698f,  25.3695f,-1.9496f,10.0167f,  26.9287f,-1.5121f,11.1913f,  27.4853f,-2.4488f,10.8361f,  
	28.9251f,-2.1594f,12.1946f,  29.4849f,-3.0970f,11.8372f,  30.7809f,-2.9633f,13.3553f,  31.3440f,-3.9030f,12.9959f,  32.4745f,-3.9301f,14.6461f,  
	33.0414f,-4.8750f,14.2835f,  33.9871f,-5.0650f,16.0361f,  34.5576f,-6.0169f,15.6698f,  35.3026f,-6.3692f,17.4927f,  35.8767f,-7.3304f,17.1218f,  
	36.4083f,-7.8415f,18.9815f,  36.9859f,-8.8147f,18.6042f,  37.2949f,-9.4777f,20.4662f,  37.8758f,-10.4656f,20.0799f,  37.9570f,-11.2707f,21.9095f,  
	38.5409f,-12.2758f,21.5118f,  38.3929f,-13.2101f,23.2746f,  38.9794f,-14.2342f,22.8627f,  38.6047f,-15.2820f,24.5251f,  39.1937f,-16.3272f,24.0948f,  
	38.5988f,-17.4698f,25.6253f,  39.1894f,-18.5368f,25.1735f,  38.3844f,-19.7532f,26.5426f,  38.9778f,-20.8430f,26.0633f,  37.9768f,-22.1096f,27.2447f,  
	38.5721f,-23.2219f,26.7342f,  37.3924f,-24.5133f,27.7051f,  37.9893f,-25.6465f,27.1591f,  36.6514f,-26.9358f,27.9008f,  37.2511f,-28.0872f,27.3140f,  
	35.7785f,-29.3460f,27.8128f,  36.3808f,-30.5133f,27.1808f,  34.7992f,-31.7132f,27.4279f,  35.4044f,-32.8910f,26.7474f,  33.7416f,-34.0032f,26.7394f,  
	34.3504f,-35.1865f,26.0067f,  32.6355f,-36.1831f,25.7460f,  33.2490f,-37.3642f,24.9593f,  31.5121f,-38.2185f,24.4540f,  32.1308f,-39.3905f,23.6120f,  
	30.4022f,-40.0778f,22.8759f,  31.0269f,-41.2318f,21.9791f,  29.3368f,-41.7301f,21.0318f,  29.9684f,-42.8565f,20.0829f,  28.3462f,-43.1474f,18.9489f,  
	28.9839f,-44.2381f,17.9496f,  27.4578f,-44.3071f,16.6581f,  28.1025f,-45.3518f,15.6152f,  26.6984f,-45.1890f,14.1995f,  27.3497f,-46.1786f,13.1206f,  
	26.0916f,-45.7795f,11.6173f,  26.7482f,-46.7062f,10.5094f,  25.6571f,-46.0717f,8.9573f,  26.3176f,-46.9279f,7.8318f,  -2.8025f,1.7033f,35.0947f,  
	-2.2121f,0.7407f,34.7863f,  -3.0835f,0.7449f,32.9856f,  -2.4974f,-0.2146f,32.6780f,  -3.1701f,-0.0543f,30.8008f,  -2.5881f,-1.0119f,30.4938f,  
	-3.0496f,-0.7004f,28.5723f,  -2.4715f,-1.6573f,28.2656f,  -2.7132f,-1.2017f,26.3327f,  -2.1388f,-2.1589f,26.0259f,  -2.1559f,-1.5688f,24.1144f,  
	-1.5852f,-2.5269f,23.8066f,  -1.3766f,-1.8141f,21.9491f,  -0.8094f,-2.7734f,21.6398f,  -0.3782f,-1.9517f,19.8675f,  0.1855f,-2.9118f,19.5562f,  
	0.8325f,-1.9966f,17.8991f,  1.3927f,-2.9573f,17.5852f,  2.2452f,-1.9656f,16.0713f,  2.8023f,-2.9267f,15.7540f,  3.8462f,-1.8765f,14.4088f,  
	4.4005f,-2.8372f,14.0878f,  5.6186f,-1.7477f,12.9339f,  6.1706f,-2.7074f,12.6089f,  7.5427f,-1.5985f,11.6657f,  8.0927f,-2.5562f,11.3365f,  
	9.5961f,-1.4480f,10.6198f,  10.1446f,-2.4031f,10.2863f,  11.7543f,-1.3157f,9.8081f,  12.3018f,-2.2674f,9.4706f,  13.9910f,-1.2210f,9.2389f,  
	14.5384f,-2.1690f,8.8973f,  16.2789f,-1.1829f,8.9158f,  16.8267f,-2.1270f,8.5706f,  18.5895f,-1.2199f,8.8387f,  19.1382f,-2.1600f,8.4902f,  
	20.8939f,-1.3495f,9.0029f,  21.4441f,-2.2851f,8.6519f,  23.1632f,-1.5872f,9.4001f,  23.7158f,-2.5205f,9.0464f,  25.3695f,-1.9496f,10.0167f,  
	25.9248f,-2.8804f,9.6611f,  27.4853f,-2.4488f,10.8361f,  28.0437f,-3.3789f,10.4786f,  29.4849f,-3.0970f,11.8372f,  30.0466f,-4.0275f,11.4782f,  
	31.3440f,-3.9030f,12.9959f,  31.9095f,-4.8370f,12.6344f,  33.0414f,-4.8750f,14.2835f,  33.6106f,-5.8142f,13.9194f,  34.5576f,-6.0169f,15.6698f,  
	35.1306f,-6.9636f,15.3022f,  35.8767f,-7.3304f,17.1218f,  36.4533f,-8.2876f,16.7493f,  36.9859f,-8.8147f,18.6042f,  37.5659f,-9.7851f,18.2243f,  
	37.8758f,-10.4656f,20.0799f,  38.4590f,-11.4520f,19.6906f,  38.5409f,-12.2758f,21.5118f,  39.1270f,-13.2803f,21.1104f,  38.9794f,-14.2342f,22.8627f,  
	39.5682f,-15.2594f,22.4450f,  39.1937f,-16.3272f,24.0948f,  39.7843f,-17.3742f,23.6581f,  39.1894f,-18.5368f,25.1735f,  39.7828f,-19.6074f,24.7115f,  
	38.9778f,-20.8430f,26.0633f,  39.5732f,-21.9374f,25.5723f,  38.5721f,-23.2219f,26.7342f,  39.1690f,-24.3392f,26.2097f,  37.9893f,-25.6465f,27.1591f,  
	38.5887f,-26.7848f,26.5952f,  37.2511f,-28.0872f,27.3140f,  37.8530f,-29.2447f,26.7061f,  36.3808f,-30.5133f,27.1808f,  36.9853f,-31.6855f,26.5249f,  
	35.4044f,-32.8910f,26.7474f,  36.0123f,-34.0733f,26.0392f,  34.3504f,-35.1865f,26.0067f,  34.9627f,-36.3719f,25.2435f,  33.2490f,-37.3642f,24.9593f,  
	33.8662f,-38.5461f,24.1388f,  32.1308f,-39.3905f,23.6120f,  32.7537f,-40.5603f,22.7337f,  31.0269f,-41.2318f,21.9791f,  31.6566f,-42.3799f,21.0447f,  
	29.9684f,-42.8565f,20.0829f,  30.6041f,-43.9748f,19.0932f,  28.9839f,-44.2381f,17.9496f,  29.6268f,-45.3159f,16.9102f,  28.1025f,-45.3518f,15.6152f,  
	28.7524f,-46.3793f,14.5329f,  27.3497f,-46.1786f,13.1206f,  28.0054f,-47.1475f,12.0017f,  26.7482f,-46.7062f,10.5094f,  27.4085f,-47.6077f,9.3647f,  
	26.3176f,-46.9279f,7.8318f,  26.9806f,-47.7553f,6.6711f,  -2.2121f,0.7407f,34.7863f,  -1.6231f,-0.2273f,34.4773f,  -2.4974f,-0.2146f,32.6780f,  
	-1.9129f,-1.1795f,32.3695f,  -2.5881f,-1.0119f,30.4938f,  -2.0077f,-1.9751f,30.1855f,  -2.4715f,-1.6573f,28.2656f,  -1.8950f,-2.6200f,27.9572f,  
	-2.1388f,-2.1589f,26.0259f,  -1.5662f,-3.1215f,25.7167f,  -1.5852f,-2.5269f,23.8066f,  -1.0162f,-3.4900f,23.4961f,  -0.8094f,-2.7734f,21.6398f,  
	-0.2441f,-3.7369f,21.3273f,  0.1855f,-2.9118f,19.5562f,  0.7472f,-3.8754f,19.2412f,  1.3927f,-2.9573f,17.5852f,  1.9513f,-3.9210f,17.2670f,  
	2.8023f,-2.9267f,15.7540f,  3.3579f,-3.8896f,15.4323f,  4.4005f,-2.8372f,14.0878f,  4.9536f,-3.7988f,13.7621f,  6.1706f,-2.7074f,12.6089f,  
	6.7214f,-3.6668f,12.2790f,  8.0927f,-2.5562f,11.3365f,  8.6419f,-3.5127f,11.0023f,  10.1446f,-2.4031f,10.2863f,  10.6925f,-3.3557f,9.9479f,  
	12.3018f,-2.2674f,9.4706f,  12.8493f,-3.2159f,9.1281f,  14.5384f,-2.1690f,8.8973f,  15.0861f,-3.1129f,8.5511f,  16.8267f,-2.1270f,8.5706f,  
	17.3750f,-3.0660f,8.2210f,  19.1382f,-2.1600f,8.4902f,  19.6879f,-3.0936f,8.1380f,  21.4441f,-2.2851f,8.6519f,  21.9960f,-3.2152f,8.2970f,  
	23.7158f,-2.5205f,9.0464f,  24.2702f,-3.4467f,8.6898f,  25.9248f,-2.8804f,9.6611f,  26.4821f,-3.8043f,9.3028f,  28.0437f,-3.3789f,10.4786f,  
	28.6043f,-4.3015f,10.1192f,  30.0466f,-4.0275f,11.4782f,  30.6110f,-4.9518f,11.1169f,  31.9095f,-4.8370f,12.6344f,  32.4777f,-5.7646f,12.2713f,  
	33.6106f,-5.8142f,13.9194f,  34.1826f,-6.7474f,13.5539f,  35.1306f,-6.9636f,15.3022f,  35.7063f,-7.9053f,14.9332f,  36.4533f,-8.2876f,16.7493f,  
	37.0327f,-9.2409f,16.3746f,  37.5659f,-9.7851f,18.2243f,  38.1487f,-10.7529f,17.8421f,  38.4590f,-11.4520f,19.6906f,  39.0449f,-12.4367f,19.2983f,  
	39.1270f,-13.2803f,21.1104f,  39.7157f,-14.2848f,20.7042f,  39.5682f,-15.2594f,22.4450f,  40.1590f,-16.2854f,22.0221f,  39.7843f,-17.3742f,23.6581f,  
	40.3779f,-18.4240f,23.2123f,  39.7828f,-19.6074f,24.7115f,  40.3784f,-20.6818f,24.2392f,  39.5732f,-21.9374f,25.5723f,  40.1703f,-23.0362f,25.0688f,  
	39.1690f,-24.3392f,26.2097f,  39.7686f,-25.4613f,25.6687f,  38.5887f,-26.7848f,26.5952f,  39.1907f,-27.9291f,26.0120f,  37.8530f,-29.2447f,26.7061f,  
	38.4573f,-30.4073f,26.0760f,  36.9853f,-31.6855f,26.5249f,  37.5927f,-32.8626f,25.8430f,  36.0123f,-34.0733f,26.0392f,  36.6237f,-35.2585f,25.3017f,  
	34.9627f,-36.3719f,25.2435f,  35.5787f,-37.5592f,24.4474f,  33.8662f,-38.5461f,24.1388f,  34.4877f,-39.7272f,23.2827f,  32.7537f,-40.5603f,22.7337f,  
	33.3818f,-41.7257f,21.8179f,  31.6566f,-42.3799f,21.0447f,  32.2905f,-43.5217f,20.0693f,  30.6041f,-43.9748f,19.0932f,  31.2454f,-45.0820f,18.0625f,  
	29.6268f,-45.3159f,16.9102f,  30.2754f,-46.3784f,15.8300f,  28.7524f,-46.3793f,14.5329f,  29.4072f,-47.3881f,13.4085f,  28.0054f,-47.1475f,12.0017f,  
	28.6656f,-48.0927f,10.8434f,  27.4085f,-47.6077f,9.3647f,  28.0723f,-48.4818f,8.1819f,  26.9806f,-47.7553f,6.6711f,  27.6459f,-48.5513f,5.4785f,  
	-1.6231f,-0.2273f,34.4773f,  -1.0357f,-1.2002f,34.1672f,  -1.9129f,-1.1795f,32.3695f,  -1.3298f,-2.1498f,32.0596f,  -2.0077f,-1.9751f,30.1855f,  
	-1.4287f,-2.9440f,29.8755f,  -1.8950f,-2.6200f,27.9572f,  -1.3200f,-3.5878f,27.6464f,  -1.5662f,-3.1215f,25.7167f,  -0.9950f,-4.0892f,25.4046f,  
	-1.0162f,-3.4900f,23.4961f,  -0.4487f,-4.4576f,23.1823f,  -0.2441f,-3.7369f,21.3273f,  0.3198f,-4.7042f,21.0111f,  0.7472f,-3.8754f,19.2412f,  
	1.3077f,-4.8424f,18.9220f,  1.9513f,-3.9210f,17.2670f,  2.5087f,-4.8869f,16.9443f,  3.3579f,-3.8896f,15.4323f,  3.9126f,-4.8540f,15.1056f,  
	4.9536f,-3.7988f,13.7621f,  5.5058f,-4.7607f,13.4312f,  6.7214f,-3.6668f,12.2790f,  7.2718f,-4.6256f,11.9438f,  8.6419f,-3.5127f,11.0023f,  
	9.1907f,-4.4672f,10.6628f,  10.6925f,-3.3557f,9.9479f,  11.2407f,-4.3056f,9.6042f,  12.8493f,-3.2159f,9.1281f,  13.3974f,-4.1606f,8.7805f,  
	15.0861f,-3.1129f,8.5511f,  15.6345f,-4.0520f,8.2000f,  17.3750f,-3.0660f,8.2210f,  17.9245f,-3.9988f,7.8671f,  19.6879f,-3.0936f,8.1380f,  
	20.2394f,-4.0218f,7.7814f,  21.9960f,-3.2152f,8.2970f,  22.5497f,-4.1381f,7.9386f,  24.2702f,-3.4467f,8.6898f,  24.8268f,-4.3658f,8.3299f,  
	26.4821f,-3.8043f,9.3028f,  27.0419f,-4.7203f,8.9421f,  28.6043f,-4.3015f,10.1192f,  29.1679f,-5.2174f,9.7572f,  30.6110f,-4.9518f,11.1169f,  
	31.1783f,-5.8690f,10.7538f,  32.4777f,-5.7646f,12.2713f,  33.0489f,-6.6853f,11.9068f,  34.1826f,-6.7474f,13.5539f,  34.7577f,-7.6745f,13.1871f,  
	35.7063f,-7.9053f,14.9332f,  36.2853f,-8.8423f,14.5622f,  37.0327f,-9.2409f,16.3746f,  37.6153f,-10.1904f,15.9979f,  38.1487f,-10.7529f,17.8421f,  
	38.7345f,-11.7177f,17.4574f,  39.0449f,-12.4367f,19.2983f,  39.6338f,-13.4201f,18.9021f,  39.7157f,-14.2848f,20.7042f,  40.3069f,-15.2889f,20.2937f,  
	40.1590f,-16.2854f,22.0221f,  40.7530f,-17.3131f,21.5914f,  40.3779f,-18.4240f,23.2123f,  40.9741f,-19.4766f,22.7577f,  40.3784f,-20.6818f,24.2392f,  
	40.9762f,-21.7598f,23.7561f,  40.1703f,-23.0362f,25.0688f,  40.7705f,-24.1392f,24.5505f,  39.7686f,-25.4613f,25.6687f,  40.3710f,-26.5889f,25.1102f,  
	39.1907f,-27.9291f,26.0120f,  39.7952f,-29.0783f,25.4081f,  38.4573f,-30.4073f,26.0760f,  39.0646f,-31.5750f,25.4214f,  37.5927f,-32.8626f,25.8430f,  
	38.2038f,-34.0431f,25.1330f,  36.6237f,-35.2585f,25.3017f,  37.2390f,-36.4464f,24.5326f,  35.5787f,-37.5592f,24.4474f,  36.1992f,-38.7467f,23.6166f,  
	34.4877f,-39.7272f,23.2827f,  35.1146f,-40.9053f,22.3892f,  33.3818f,-41.7257f,21.8179f,  34.0144f,-42.8865f,20.8612f,  32.2905f,-43.5217f,20.0693f,  
	32.9304f,-44.6542f,19.0523f,  31.2454f,-45.0820f,18.0625f,  31.8929f,-46.1757f,16.9896f,  30.2754f,-46.3784f,15.8300f,  30.9296f,-47.4239f,14.7059f,  
	29.4072f,-47.3881f,13.4085f,  30.0673f,-48.3748f,12.2426f,  28.6656f,-48.0927f,10.8434f,  29.3303f,-49.0122f,9.6442f,  28.0723f,-48.4818f,8.1819f,  
	28.7397f,-49.3256f,6.9638f,  27.6459f,-48.5513f,5.4785f,  28.3128f,-49.3139f,4.2505f,  -1.0357f,-1.2002f,34.1672f,  -0.4493f,-2.1782f,33.8557f,  
	-1.3298f,-2.1498f,32.0596f,  -0.7477f,-3.1255f,31.7478f,  -1.4287f,-2.9440f,29.8755f,  -0.8509f,-3.9179f,29.5630f,  -1.3200f,-3.5878f,27.6464f,  
	-0.7461f,-4.5609f,27.3328f,  -0.9950f,-4.0892f,25.4046f,  -0.4249f,-5.0618f,25.0892f,  -0.4487f,-4.4576f,23.1823f,  0.1176f,-5.4294f,22.8646f,  
	0.3198f,-4.7042f,21.0111f,  0.8826f,-5.6752f,20.6905f,  1.3077f,-4.8424f,18.9220f,  1.8673f,-5.8122f,18.5980f,  2.5087f,-4.8869f,16.9443f,  
	3.0653f,-5.8549f,16.6164f,  3.9126f,-4.8540f,15.1056f,  4.4666f,-5.8193f,14.7735f,  5.5058f,-4.7607f,13.4312f,  6.0577f,-5.7227f,13.0947f,  
	7.2718f,-4.6256f,11.9438f,  7.8219f,-5.5830f,11.6029f,  9.1907f,-4.4672f,10.6628f,  9.7399f,-5.4196f,10.3175f,  11.2407f,-4.3056f,9.6042f,  
	11.7895f,-5.2524f,9.2548f,  13.3974f,-4.1606f,8.7805f,  13.9463f,-5.1010f,8.4275f,  15.6345f,-4.0520f,8.2000f,  16.1843f,-4.9852f,7.8439f,  
	17.9245f,-3.9988f,7.8671f,  18.4761f,-4.9264f,7.5082f,  20.2394f,-4.0218f,7.7814f,  20.7930f,-4.9428f,7.4206f,  22.5497f,-4.1381f,7.9386f,  
	23.1060f,-5.0538f,7.5764f,  24.8268f,-4.3658f,8.3299f,  25.3861f,-5.2767f,7.9670f,  27.0419f,-4.7203f,8.9421f,  27.6049f,-5.6292f,8.5783f,  
	29.1679f,-5.2174f,9.7572f,  29.7347f,-6.1256f,9.3928f,  31.1783f,-5.8690f,10.7538f,  31.7490f,-6.7785f,10.3890f,  33.0489f,-6.6853f,11.9068f,  
	33.6236f,-7.5991f,11.5408f,  34.7577f,-7.6745f,13.1871f,  35.3364f,-8.5960f,12.8183f,  36.2853f,-8.8423f,14.5622f,  36.8678f,-9.7741f,14.1896f,  
	37.6153f,-10.1904f,15.9979f,  38.2012f,-11.1358f,15.6192f,  38.7345f,-11.7177f,17.4574f,  39.3238f,-12.6801f,17.0695f,  39.6338f,-13.4201f,18.9021f,  
	40.2255f,-14.4019f,18.5025f,  40.3069f,-15.2889f,20.2937f,  40.9016f,-16.2936f,19.8765f,  40.7530f,-17.3131f,21.5914f,  41.3500f,-18.3424f,21.1532f,  
	40.9741f,-19.4766f,22.7577f,  41.5727f,-20.5317f,22.2938f,  40.9762f,-21.7598f,23.7561f,  41.5772f,-22.8412f,23.2595f,  40.7705f,-24.1392f,24.5505f,  
	41.3736f,-25.2472f,24.0163f,  40.3710f,-26.5889f,25.1102f,  40.9760f,-27.7212f,24.5329f,  39.7952f,-29.0783f,25.4081f,  40.4029f,-30.2326f,24.7816f,  
	39.0646f,-31.5750f,25.4214f,  39.6757f,-32.7465f,24.7403f,  38.2038f,-34.0431f,25.1330f,  38.8188f,-35.2270f,24.3928f,  37.2390f,-36.4464f,24.5326f,  
	37.8589f,-37.6356f,23.7297f,  36.1992f,-38.7467f,23.6166f,  36.8253f,-39.9325f,22.7488f,  35.1146f,-40.9053f,22.3892f,  35.7461f,-42.0801f,21.4554f,  
	34.0144f,-42.8865f,20.8612f,  34.6532f,-44.0396f,19.8625f,  32.9304f,-44.6542f,19.0523f,  33.5771f,-45.7748f,17.9921f,  31.8929f,-46.1757f,16.9896f,  
	32.5465f,-47.2543f,15.8716f,  30.9296f,-47.4239f,14.7059f,  31.5898f,-48.4489f,13.5383f,  30.0673f,-48.3748f,12.2426f,  30.7329f,-49.3372f,11.0331f,  
	29.3303f,-49.0122f,9.6442f,  29.9997f,-49.9023f,8.4067f,  28.7397f,-49.3256f,6.9638f,  29.4099f,-50.1370f,5.7069f,  28.3128f,-49.3139f,4.2505f,  
	28.9806f,-50.0406f,2.9881f,  -0.4493f,-2.1782f,33.8557f,  0.1362f,-3.1613f,33.5420f,  -0.7477f,-3.1255f,31.7478f,  -0.1667f,-4.1060f,31.4334f,  
	-0.8509f,-3.9179f,29.5630f,  -0.2739f,-4.8969f,29.2475f,  -0.7461f,-4.5609f,27.3328f,  -0.1731f,-5.5390f,27.0156f,  -0.4249f,-5.0618f,25.0892f,  
	0.1443f,-6.0386f,24.7699f,  0.1176f,-5.4294f,22.8646f,  0.6832f,-6.4052f,22.5424f,  0.8826f,-5.6752f,20.6905f,  1.4447f,-6.6495f,20.3650f,  
	1.8673f,-5.8122f,18.5980f,  2.4264f,-6.7845f,18.2686f,  3.0653f,-5.8549f,16.6164f,  3.6216f,-6.8244f,16.2828f,  4.4666f,-5.8193f,14.7735f,  
	5.0206f,-6.7852f,14.4354f,  6.0577f,-5.7227f,13.0947f,  6.6097f,-6.6838f,12.7522f,  7.8219f,-5.5830f,11.6029f,  8.3727f,-6.5388f,11.2558f,  
	9.7399f,-5.4196f,10.3175f,  10.2900f,-6.3694f,9.9660f,  11.7895f,-5.2524f,9.2548f,  12.3394f,-6.1952f,8.8995f,  13.9463f,-5.1010f,8.4275f,  
	14.4968f,-6.0360f,8.0688f,  16.1843f,-4.9852f,7.8439f,  16.7362f,-5.9135f,7.4823f,  18.4761f,-4.9264f,7.5082f,  19.0299f,-5.8469f,7.1445f,  
	20.7930f,-4.9428f,7.4206f,  21.3493f,-5.8566f,7.0554f,  23.1060f,-5.0538f,7.5764f,  23.6652f,-5.9611f,7.2106f,  25.3861f,-5.2767f,7.9670f,  
	25.9489f,-6.1803f,7.6005f,  27.6049f,-5.6292f,8.5783f,  28.1715f,-6.5300f,8.2117f,  29.7347f,-6.1256f,9.3928f,  30.3052f,-7.0255f,9.0265f,  
	31.7490f,-6.7785f,10.3890f,  32.3236f,-7.6803f,10.0225f,  33.6236f,-7.5991f,11.5408f,  34.2024f,-8.5063f,11.1727f,  35.3364f,-8.5960f,12.8183f,  
	35.9191f,-9.5112f,12.4480f,  36.8678f,-9.7741f,14.1896f,  37.4541f,-10.7007f,13.8152f,  38.2012f,-11.1358f,15.6192f,  38.7910f,-12.0774f,15.2378f,  
	39.3238f,-12.6801f,17.0695f,  39.9164f,-13.6395f,16.6789f,  40.2255f,-14.4019f,18.5025f,  40.8212f,-15.3831f,18.0971f,  40.9016f,-16.2936f,19.8765f,  
	41.4997f,-17.2987f,19.4531f,  41.3500f,-18.3424f,21.1532f,  41.9499f,-19.3731f,20.7071f,  41.5727f,-20.5317f,22.2938f,  42.1749f,-21.5895f,21.8180f,  
	41.5772f,-22.8412f,23.2595f,  42.1814f,-23.9269f,22.7490f,  41.3736f,-25.2472f,24.0163f,  41.9797f,-26.3594f,23.4651f,  40.9760f,-27.7212f,24.5329f,  
	41.5845f,-28.8584f,23.9346f,  40.4029f,-30.2326f,24.7816f,  41.0144f,-31.3909f,24.1302f,  39.6757f,-32.7465f,24.7403f,  40.2909f,-33.9218f,24.0304f,  
	38.8188f,-35.2270f,24.3928f,  39.4385f,-36.4129f,23.6200f,  37.8589f,-37.6356f,23.7297f,  38.4847f,-38.8241f,22.8907f,  36.8253f,-39.9325f,22.7488f,  
	37.4561f,-41.1164f,21.8416f,  35.7461f,-42.0801f,21.4554f,  36.3841f,-43.2488f,20.4795f,  34.6532f,-44.0396f,19.8625f,  35.2992f,-45.1825f,18.8201f,  
	33.5771f,-45.7748f,17.9921f,  34.2303f,-46.8821f,16.8857f,  32.5465f,-47.2543f,15.8716f,  33.2068f,-48.3139f,14.7084f,  31.5898f,-48.4489f,13.5383f,  
	32.2562f,-49.4514f,12.3249f,  30.7329f,-49.3372f,11.0331f,  31.4045f,-50.2716f,9.7823f,  29.9997f,-49.9023f,8.4067f,  30.6733f,-50.7612f,7.1268f,  
	29.4099f,-50.1370f,5.7069f,  30.0826f,-50.9132f,4.4116f,  28.9806f,-50.0406f,2.9881f,  29.6487f,-50.7289f,1.6930f,  0.1362f,-3.1613f,33.5420f,  
	0.7207f,-4.1490f,33.2258f,  -0.1667f,-4.1060f,31.4334f,  0.4136f,-5.0916f,31.1161f,  -0.2739f,-4.8969f,29.2475f,  0.3024f,-5.8811f,28.9285f,  
	-0.1731f,-5.5390f,27.0156f,  0.3992f,-6.5215f,26.6945f,  0.1443f,-6.0386f,24.7699f,  0.7129f,-7.0198f,24.4460f,  0.6832f,-6.4052f,22.5424f,  
	1.2483f,-7.3846f,22.2152f,  1.4447f,-6.6495f,20.3650f,  2.0066f,-7.6268f,20.0339f,  2.4264f,-6.7845f,18.2686f,  2.9853f,-7.7588f,17.9333f,  
	3.6216f,-6.8244f,16.2828f,  4.1780f,-7.7950f,15.9429f,  5.0206f,-6.7852f,14.4354f,  5.5747f,-7.7509f,14.0909f,  6.6097f,-6.6838f,12.7522f,  
	7.1624f,-7.6440f,12.4029f,  8.3727f,-6.5388f,11.2558f,  8.9245f,-7.4926f,10.9019f,  10.2900f,-6.3694f,9.9660f,  10.8413f,-7.3157f,9.6082f,  
	12.3394f,-6.1952f,8.8995f,  12.8911f,-7.1331f,8.5380f,  14.4968f,-6.0360f,8.0688f,  15.0497f,-6.9663f,7.7040f,  16.7362f,-5.9135f,7.4823f,  
	17.2907f,-6.8351f,7.1151f,  19.0299f,-5.8469f,7.1445f,  19.5866f,-6.7604f,6.7757f,  21.3493f,-5.8566f,7.0554f,  21.9087f,-6.7620f,6.6859f,  
	23.6652f,-5.9611f,7.2106f,  24.2281f,-6.8610f,6.8405f,  25.9489f,-6.1803f,7.6005f,  26.5155f,-7.0753f,7.2307f,  28.1715f,-6.5300f,8.2117f,  
	28.7420f,-7.4219f,7.8426f,  30.3052f,-7.0255f,9.0265f,  30.8798f,-7.9168f,8.6580f,  32.3236f,-7.6803f,10.0225f,  32.9025f,-8.5748f,9.6536f,  
	34.2024f,-8.5063f,11.1727f,  34.7854f,-9.4062f,10.8031f,  35.9191f,-9.5112f,12.4480f,  36.5059f,-10.4200f,12.0761f,  37.4541f,-10.7007f,13.8152f,  
	38.0446f,-11.6223f,13.4384f,  38.7910f,-12.0774f,15.2378f,  39.3847f,-13.0147f,14.8544f,  39.9164f,-13.6395f,16.6789f,  40.5132f,-14.5970f,16.2834f,  
	40.8212f,-15.3831f,18.0971f,  41.4206f,-16.3632f,17.6866f,  41.4997f,-17.2987f,19.4531f,  42.1010f,-18.3038f,19.0232f,  41.9499f,-19.3731f,20.7071f,  
	42.5536f,-20.4055f,20.2506f,  42.1749f,-21.5895f,21.8180f,  42.7807f,-22.6505f,21.3299f,  42.1814f,-23.9269f,22.7490f,  42.7889f,-25.0159f,22.2231f,  
	41.9797f,-26.3594f,23.4651f,  42.5893f,-27.4759f,22.8948f,  41.5845f,-28.8584f,23.9346f,  42.1970f,-29.9994f,23.3132f,  41.0144f,-31.3909f,24.1302f,  
	41.6302f,-32.5531f,23.4518f,  40.2909f,-33.9218f,24.0304f,  40.9109f,-35.0996f,23.2896f,  39.4385f,-36.4129f,23.6200f,  40.0642f,-37.5990f,22.8123f,  
	38.4847f,-38.8241f,22.8907f,  39.1151f,-40.0117f,22.0133f,  37.4561f,-41.1164f,21.8416f,  38.0937f,-42.2955f,20.8926f,  36.3841f,-43.2488f,20.4795f,  
	37.0297f,-44.4089f,19.4596f,  35.2992f,-45.1825f,18.8201f,  35.9522f,-46.3138f,17.7308f,  34.2303f,-46.8821f,16.8857f,  34.8908f,-47.9721f,15.7328f,  
	33.2068f,-48.3139f,14.7084f,  33.8742f,-49.3528f,13.4973f,  32.2562f,-49.4514f,12.3249f,  32.9299f,-50.4271f,11.0673f,  31.4045f,-50.2716f,9.7823f,  
	32.0814f,-51.1760f,8.4861f,  30.6733f,-50.7612f,7.1268f,  31.3508f,-51.5858f,5.8050f,  30.0826f,-50.9132f,4.4116f,  30.7571f,-51.6512f,3.0797f,  
	29.6487f,-50.7289f,1.6930f,  30.3161f,-51.3762f,0.3661f,  0.7207f,-4.1490f,33.2258f,  1.3046f,-5.1414f,32.9065f,  0.4136f,-5.0916f,31.1161f,  
	0.9935f,-6.0822f,30.7951f,  0.3024f,-5.8811f,28.9285f,  0.8782f,-6.8697f,28.6054f,  0.3992f,-6.5215f,26.6945f,  0.9712f,-7.5084f,26.3686f,  
	0.7129f,-7.0198f,24.4460f,  1.2812f,-8.0048f,24.1169f,  1.2483f,-7.3846f,22.2152f,  1.8133f,-8.3673f,21.8823f,  2.0066f,-7.6268f,20.0339f,  
	2.5685f,-8.6065f,19.6967f,  2.9853f,-7.7588f,17.9333f,  3.5445f,-8.7348f,17.5914f,  4.1780f,-7.7950f,15.9429f,  4.7348f,-8.7660f,15.5963f,  
	5.5747f,-7.7509f,14.0909f,  6.1298f,-8.7163f,13.7394f,  7.1624f,-7.6440f,12.4029f,  7.7163f,-8.6027f,12.0465f,  8.9245f,-7.4926f,10.9019f,  
	9.4776f,-8.4435f,10.5412f,  10.8413f,-7.3157f,9.6082f,  11.3945f,-8.2578f,9.2433f,  12.8911f,-7.1331f,8.5380f,  13.4452f,-8.0668f,8.1695f,  
	15.0497f,-6.9663f,7.7040f,  15.6051f,-7.8903f,7.3329f,  17.2907f,-6.8351f,7.1151f,  17.8481f,-7.7498f,6.7421f,  19.5866f,-6.7604f,6.7757f,  
	20.1466f,-7.6656f,6.4018f,  21.9087f,-6.7620f,6.6859f,  22.4720f,-7.6599f,6.3114f,  24.2281f,-6.8610f,6.8405f,  24.7950f,-7.7521f,6.4664f,  
	26.5155f,-7.0753f,7.2307f,  27.0863f,-7.9609f,6.8577f,  28.7420f,-7.4219f,7.8426f,  29.3170f,-8.3046f,7.4709f,  30.8798f,-7.9168f,8.6580f,  
	31.4592f,-8.8002f,8.2868f,  32.9025f,-8.5748f,9.6536f,  33.4860f,-9.4608f,9.2829f,  34.7854f,-9.4062f,10.8031f,  35.3730f,-10.2986f,10.4318f,  
	36.5059f,-10.4200f,12.0761f,  37.0974f,-11.3227f,11.7020f,  38.0446f,-11.6223f,13.4384f,  38.6395f,-12.5383f,13.0600f,  39.3847f,-13.0147f,14.8544f,  
	39.9828f,-13.9488f,14.4667f,  40.5132f,-14.5970f,16.2834f,  41.1141f,-15.5519f,15.8838f,  41.4206f,-16.3632f,17.6866f,  42.0237f,-17.3420f,17.2707f,  
	42.1010f,-18.3038f,19.0232f,  42.7065f,-19.3095f,18.5840f,  42.5536f,-20.4055f,20.2506f,  43.1611f,-21.4399f,19.7833f,  42.7807f,-22.6505f,21.3299f,  
	43.3899f,-23.7140f,20.8282f,  42.7889f,-25.0159f,22.2231f,  43.4001f,-26.1087f,21.6801f,  42.5893f,-27.4759f,22.8948f,  43.2032f,-28.5959f,22.3032f,  
	42.1970f,-29.9994f,23.3132f,  42.8139f,-31.1443f,22.6666f,  41.6302f,-32.5531f,23.4518f,  42.2510f,-33.7180f,22.7440f,  40.9109f,-35.0996f,23.2896f,  
	41.5370f,-36.2782f,22.5151f,  40.0642f,-37.5990f,22.8123f,  40.6947f,-38.7850f,21.9675f,  39.1151f,-40.0117f,22.0133f,  39.7525f,-41.1958f,21.0947f,  
	38.0937f,-42.2955f,20.8926f,  38.7391f,-43.4673f,19.8996f,  37.0297f,-44.4089f,19.4596f,  37.6826f,-45.5590f,18.3926f,  35.9522f,-46.3138f,17.7308f,  
	36.6130f,-47.4294f,16.5940f,  34.8908f,-47.9721f,15.7328f,  35.5591f,-49.0430f,14.5303f,  33.8742f,-49.3528f,13.4973f,  34.5498f,-50.3661f,12.2396f,  
	32.9299f,-50.4271f,11.0673f,  33.6099f,-51.3742f,9.7614f,  32.0814f,-51.1760f,8.4861f,  32.7634f,-52.0471f,7.1443f,  31.3508f,-51.5858f,5.8050f,  
	32.0316f,-52.3727f,4.4425f,  30.7571f,-51.6512f,3.0797f,  31.4329f,-52.3484f,1.7115f,  30.3161f,-51.3762f,0.3661f,  30.9818f,-51.9801f,-0.9942f,  
	1.3046f,-5.1414f,32.9065f,  1.8882f,-6.1389f,32.5834f,  0.9935f,-6.0822f,30.7951f,  1.5729f,-7.0773f,30.4699f,  0.8782f,-6.8697f,28.6054f,  
	1.4537f,-7.8629f,28.2775f,  0.9712f,-7.5084f,26.3686f,  1.5430f,-8.4996f,26.0375f,  1.2812f,-8.0048f,24.1169f,  1.8496f,-8.9935f,23.7819f,  
	1.8133f,-8.3673f,21.8823f,  2.3784f,-9.3530f,21.5430f,  2.5685f,-8.6065f,19.6967f,  3.1308f,-9.5885f,19.3527f,  3.5445f,-8.7348f,17.5914f,  
	4.1042f,-9.7117f,17.2425f,  4.7348f,-8.7660f,15.5963f,  5.2926f,-9.7373f,15.2422f,  6.1298f,-8.7163f,13.7394f,  6.6862f,-9.6808f,13.3802f,  
	7.7163f,-8.6027f,12.0465f,  8.2716f,-9.5592f,11.6827f,  9.4776f,-8.4435f,10.5412f,  10.0327f,-9.3908f,10.1728f,  11.3945f,-8.2578f,9.2433f,  
	11.9503f,-9.1961f,8.8710f,  13.4452f,-8.0668f,8.1695f,  14.0020f,-8.9946f,7.7941f,  15.6051f,-7.8903f,7.3329f,  16.1637f,-8.8078f,6.9552f,  
	17.8481f,-7.7498f,6.7421f,  18.4090f,-8.6564f,6.3632f,  20.1466f,-7.6656f,6.4018f,  20.7107f,-8.5633f,6.0221f,  22.4720f,-7.6599f,6.3114f,  
	23.0395f,-8.5489f,5.9321f,  24.7950f,-7.7521f,6.4664f,  25.3664f,-8.6335f,6.0884f,  27.0863f,-7.9609f,6.8577f,  27.6618f,-8.8369f,6.4814f,  
	29.3170f,-8.3046f,7.4709f,  29.8969f,-9.1787f,7.0959f,  31.4592f,-8.8002f,8.2868f,  32.0434f,-9.6741f,7.9135f,  33.4860f,-9.4608f,9.2829f,  
	34.0745f,-10.3383f,8.9104f,  35.3730f,-10.2986f,10.4318f,  35.9655f,-11.1836f,10.0584f,  37.0974f,-11.3227f,11.7020f,  37.6936f,-12.2183f,11.3265f,  
	38.6395f,-12.5383f,13.0600f,  39.2392f,-13.4496f,12.6778f,  39.9828f,-13.9488f,14.4667f,  40.5855f,-14.8789f,14.0756f,  41.1141f,-15.5519f,15.8838f,  
	41.7192f,-16.5042f,15.4798f,  42.0237f,-17.3420f,17.2707f,  42.6312f,-18.3201f,16.8469f,  42.7065f,-19.3095f,18.5840f,  43.3161f,-20.3158f,18.1357f,  
	43.1611f,-21.4399f,19.7833f,  43.7723f,-22.4757f,19.3042f,  43.3899f,-23.7140f,20.8282f,  44.0031f,-24.7803f,20.3113f,  43.4001f,-26.1087f,21.6801f,  
	44.0157f,-27.2044f,21.1176f,  43.2032f,-28.5959f,22.3032f,  43.8215f,-29.7195f,21.6885f,  42.8139f,-31.1443f,22.6666f,  43.4357f,-32.2919f,21.9925f,  
	42.2510f,-33.7180f,22.7440f,  42.8779f,-34.8842f,22.0040f,  41.5370f,-36.2782f,22.5151f,  42.1680f,-37.4573f,21.7050f,  40.6947f,-38.7850f,21.9675f,  
	41.3324f,-39.9685f,21.0825f,  39.7525f,-41.1958f,21.0947f,  40.3980f,-42.3738f,20.1326f,  38.7391f,-43.4673f,19.8996f,  39.3921f,-44.6307f,18.8595f,  
	37.6826f,-45.5590f,18.3926f,  38.3438f,-46.6949f,17.2773f,  36.6130f,-47.4294f,16.5940f,  37.2821f,-48.5274f,15.4064f,  35.5591f,-49.0430f,14.5303f,  
	36.2365f,-50.0898f,13.2791f,  34.5498f,-50.3661f,12.2396f,  35.2328f,-51.3524f,10.9309f,  33.6099f,-51.3742f,9.7614f,  34.2963f,-52.2891f,8.4067f,  
	32.7634f,-52.0471f,7.1443f,  33.4504f,-52.8810f,5.7581f,  32.0316f,-52.3727f,4.4425f,  32.7155f,-53.1189f,3.0393f,  31.4329f,-52.3484f,1.7115f,  
	32.1086f,-53.0018f,0.3052f,  30.9818f,-51.9801f,-0.9942f,  31.6445f,-52.5381f,-2.3855f,  1.8882f,-6.1389f,32.5834f,  2.4715f,-7.1408f,32.2561f,  
	1.5729f,-7.0773f,30.4699f,  2.1522f,-8.0770f,30.1398f,  1.4537f,-7.8629f,28.2775f,  2.0293f,-8.8604f,27.9442f,  1.5430f,-8.4996f,26.0375f,  
	2.1150f,-9.4947f,25.7004f,  1.8496f,-8.9935f,23.7819f,  2.4183f,-9.9856f,23.4404f,  2.3784f,-9.3530f,21.5430f,  2.9441f,-10.3414f,21.1967f,  
	3.1308f,-9.5885f,19.3527f,  3.6937f,-10.5719f,19.0014f,  4.1042f,-9.7117f,17.2425f,  4.6650f,-10.6897f,16.8858f,  5.2926f,-9.7373f,15.2422f,  
	5.8517f,-10.7085f,14.8801f,  6.6862f,-9.6808f,13.3802f,  7.2440f,-10.6437f,13.0131f,  8.2716f,-9.5592f,11.6827f,  8.8291f,-10.5128f,11.3105f,  
	10.0327f,-9.3908f,10.1728f,  10.5905f,-10.3349f,9.7963f,  11.9503f,-9.1961f,8.8710f,  12.5088f,-10.1290f,8.4909f,  14.0020f,-8.9946f,7.7941f,  
	14.5621f,-9.9163f,7.4113f,  16.1637f,-8.8078f,6.9552f,  16.7258f,-9.7174f,6.5707f,  18.4090f,-8.6564f,6.3632f,  18.9742f,-9.5556f,5.9777f,  
	20.7107f,-8.5633f,6.0221f,  21.2791f,-9.4520f,5.6368f,  23.0395f,-8.5489f,5.9321f,  23.6117f,-9.4280f,5.5481f,  25.3664f,-8.6335f,6.0884f,  
	25.9426f,-9.5048f,5.7064f,  27.6618f,-8.8369f,6.4814f,  28.2426f,-9.7038f,6.1012f,  29.8969f,-9.1787f,7.0959f,  30.4820f,-10.0426f,6.7183f,  
	32.0434f,-9.6741f,7.9135f,  32.6329f,-10.5387f,7.5380f,  34.0745f,-10.3383f,8.9104f,  34.6683f,-11.2074f,8.5356f,  35.9655f,-11.1836f,10.0584f,  
	36.5632f,-12.0603f,9.6835f,  37.6936f,-12.2183f,11.3265f,  38.2949f,-13.1080f,10.9476f,  39.2392f,-13.4496f,12.6778f,  39.8437f,-14.3555f,12.2929f,  
	40.5855f,-14.8789f,14.0756f,  41.1926f,-15.8048f,13.6810f,  41.7192f,-16.5042f,15.4798f,  42.3289f,-17.4544f,15.0689f,  42.6312f,-18.3201f,16.8469f,  
	43.2431f,-19.2975f,16.4153f,  43.3161f,-20.3158f,18.1357f,  43.9297f,-21.3224f,17.6772f,  43.7723f,-22.4757f,19.3042f,  44.3878f,-23.5133f,18.8117f,  
	44.0031f,-24.7803f,20.3113f,  44.6208f,-25.8487f,19.7767f,  44.0157f,-27.2044f,21.1176f,  44.6359f,-28.3031f,20.5340f,  43.8215f,-29.7195f,21.6885f,  
	44.4449f,-30.8456f,21.0481f,  43.4357f,-32.2919f,21.9925f,  44.0637f,-33.4410f,21.2879f,  42.8779f,-34.8842f,22.0040f,  43.5096f,-36.0513f,21.2304f,  
	42.1680f,-37.4573f,21.7050f,  42.8061f,-38.6348f,20.8560f,  41.3324f,-39.9685f,21.0825f,  41.9782f,-41.1472f,20.1545f,  40.3980f,-42.3738f,20.1326f,  
	41.0513f,-43.5447f,19.1238f,  39.3921f,-44.6307f,18.8595f,  40.0537f,-45.7813f,17.7710f,  38.3438f,-46.6949f,17.2773f,  39.0138f,-47.8148f,16.1104f,  
	37.2821f,-48.5274f,15.4064f,  37.9613f,-49.6029f,14.1681f,  36.2365f,-50.0898f,13.2791f,  36.9223f,-51.1111f,11.9747f,  35.2328f,-51.3524f,10.9309f,  
	35.9233f,-52.3076f,9.5706f,  34.2963f,-52.2891f,8.4067f,  34.9891f,-53.1675f,7.0041f,  33.4504f,-52.8810f,5.7581f,  34.1421f,-53.6746f,4.3270f,  
	32.7155f,-53.1189f,3.0393f,  33.4011f,-53.8213f,1.5936f,  32.1086f,-53.0018f,0.3052f,  32.7834f,-53.6085f,-1.1368f,  31.6445f,-52.5381f,-2.3855f,  
	32.3028f,-53.0473f,-3.8101f,  2.4715f,-7.1408f,32.2561f,  3.0546f,-8.1472f,31.9238f,  2.1522f,-8.0770f,30.1398f,  2.7315f,-9.0812f,29.8043f,  
	2.0293f,-8.8604f,27.9442f,  2.6050f,-9.8621f,27.6049f,  2.1150f,-9.4947f,25.7004f,  2.6874f,-10.4934f,25.3566f,  2.4183f,-9.9856f,23.4404f,  
	2.9875f,-10.9808f,23.0918f,  2.9441f,-10.3414f,21.1967f,  3.5105f,-11.3319f,20.8429f,  3.6937f,-10.5719f,19.0014f,  4.2578f,-11.5570f,18.6419f,  
	4.6650f,-10.6897f,16.8858f,  5.2272f,-11.6680f,16.5206f,  5.8517f,-10.7085f,14.8801f,  6.4124f,-11.6786f,14.5096f,  7.2440f,-10.6437f,13.0131f,  
	7.8041f,-11.6045f,12.6370f,  8.8291f,-10.5128f,11.3105f,  9.3891f,-11.4637f,10.9297f,  10.5905f,-10.3349f,9.7963f,  11.1511f,-11.2742f,9.4113f,  
	12.5088f,-10.1290f,8.4909f,  13.0706f,-11.0564f,8.1026f,  14.5621f,-9.9163f,7.4113f,  15.1257f,-10.8306f,7.0208f,  16.7258f,-9.7174f,6.5707f,  
	17.2923f,-10.6200f,6.1788f,  18.9742f,-9.5556f,5.9777f,  19.5437f,-10.4460f,5.5857f,  21.2791f,-9.4520f,5.6368f,  21.8522f,-10.3308f,5.2459f,  
	23.6117f,-9.4280f,5.5481f,  24.1889f,-10.2967f,5.1592f,  25.9426f,-9.5048f,5.7064f,  26.5244f,-10.3665f,5.3197f,  28.2426f,-9.7038f,6.1012f,  
	28.8287f,-10.5598f,5.7178f,  30.4820f,-10.0426f,6.7183f,  31.0727f,-10.8962f,6.3379f,  32.6329f,-10.5387f,7.5380f,  33.2280f,-11.3937f,7.1598f,  
	34.6683f,-11.2074f,8.5356f,  35.2677f,-12.0670f,8.1592f,  36.5632f,-12.0603f,9.6835f,  37.1663f,-12.9298f,9.3055f,  38.2949f,-13.1080f,10.9476f,  
	38.9015f,-13.9907f,10.5663f,  39.8437f,-14.3555f,12.2929f,  40.4532f,-15.2557f,11.9051f,  41.1926f,-15.8048f,13.6810f,  41.8048f,-16.7272f,13.2804f,  
	42.3289f,-17.4544f,15.0689f,  42.9433f,-18.4023f,14.6515f,  43.2431f,-19.2975f,16.4153f,  43.8593f,-20.2737f,15.9750f,  43.9297f,-21.3224f,17.6772f,  
	44.5478f,-22.3294f,17.2070f,  44.3878f,-23.5133f,18.8117f,  45.0079f,-24.5520f,18.3034f,  44.6208f,-25.8487f,19.7767f,  45.2432f,-26.9194f,19.2231f,  
	44.6359f,-28.3031f,20.5340f,  45.2612f,-29.4039f,19.9268f,  44.4449f,-30.8456f,21.0481f,  45.0744f,-31.9731f,20.3790f,  44.0637f,-33.4410f,21.2879f,  
	44.6967f,-34.5911f,20.5517f,  43.5096f,-36.0513f,21.2304f,  44.1486f,-37.2173f,20.4192f,  42.8061f,-38.6348f,20.8560f,  43.4525f,-39.8083f,19.9651f,  
	41.9782f,-41.1472f,20.1545f,  42.6319f,-42.3197f,19.1808f,  41.0513f,-43.5447f,19.1238f,  41.7134f,-44.7042f,18.0666f,  40.0537f,-45.7813f,17.7710f,  
	40.7245f,-46.9175f,16.6303f,  39.0138f,-47.8148f,16.1104f,  39.6945f,-48.9136f,14.8913f,  37.9613f,-49.6029f,14.1681f,  38.6495f,-50.6542f,12.8748f,  
	36.9223f,-51.1111f,11.9747f,  37.6166f,-52.1024f,10.6160f,  35.9233f,-52.3076f,9.5706f,  36.6215f,-53.2272f,8.1590f,  34.9891f,-53.1675f,7.0041f,  
	35.6883f,-54.0061f,5.5524f,  34.1421f,-53.6746f,4.3270f,  34.8373f,-54.4245f,2.8489f,  33.4011f,-53.8213f,1.5936f,  34.0877f,-54.4763f,0.1073f,  
	32.7834f,-53.6085f,-1.1368f,  33.4556f,-54.1654f,-2.6172f,  32.3028f,-53.0473f,-3.8101f,  32.9552f,-53.5053f,-5.2650f,  3.0546f,-8.1472f,31.9238f,  
	3.6379f,-9.1581f,31.5861f,  2.7315f,-9.0812f,29.8043f,  3.3111f,-10.0896f,29.4627f,  2.6050f,-9.8621f,27.6049f,  3.1811f,-10.8678f,27.2588f,  
	2.6874f,-10.4934f,25.3566f,  3.2603f,-11.4957f,25.0056f,  2.9875f,-10.9808f,23.0918f,  3.5575f,-11.9786f,22.7354f,  3.5105f,-11.3319f,20.8429f,  
	4.0780f,-12.3245f,20.4806f,  4.2578f,-11.5570f,18.6419f,  4.8233f,-12.5431f,18.2737f,  5.2272f,-11.6680f,16.5206f,  5.7909f,-12.6460f,16.1467f,  
	6.4124f,-11.6786f,14.5096f,  6.9753f,-12.6473f,14.1296f,  7.8041f,-11.6045f,12.6370f,  8.3667f,-12.5632f,12.2517f,  9.3891f,-11.4637f,10.9297f,  
	9.9520f,-12.4105f,10.5397f,  11.1511f,-11.2742f,9.4113f,  11.7149f,-12.2085f,9.0174f,  13.0706f,-11.0564f,8.1026f,  13.6361f,-11.9768f,7.7059f,  
	15.1257f,-10.8306f,7.0208f,  15.6937f,-11.7382f,6.6221f,  17.2923f,-10.6200f,6.1788f,  17.8632f,-11.5139f,5.7795f,  19.5437f,-10.4460f,5.5857f,  
	20.1181f,-11.3265f,5.1871f,  21.8522f,-10.3308f,5.2459f,  22.4306f,-11.1991f,4.8492f,  24.1889f,-10.2967f,5.1592f,  24.7718f,-11.1556f,4.7648f,  
	26.5244f,-10.3665f,5.3197f,  27.1117f,-11.2169f,4.9290f,  28.8287f,-10.5598f,5.7178f,  29.4207f,-11.4048f,5.3308f,  31.0727f,-10.8962f,6.3379f,  
	31.6692f,-11.7394f,5.9544f,  33.2280f,-11.3937f,7.1598f,  33.8291f,-12.2382f,6.7796f,  35.2677f,-12.0670f,8.1592f,  35.8726f,-12.9180f,7.7796f,  
	37.1663f,-12.9298f,9.3055f,  37.7749f,-13.7908f,8.9252f,  38.9015f,-13.9907f,10.5663f,  39.5133f,-14.8663f,10.1825f,  40.4532f,-15.2557f,11.9051f,  
	41.0679f,-16.1509f,11.5121f,  41.8048f,-16.7272f,13.2804f,  42.4219f,-17.6457f,12.8745f,  42.9433f,-18.4023f,14.6515f,  43.5624f,-19.3476f,14.2268f,  
	43.8593f,-20.2737f,15.9750f,  44.4802f,-21.2489f,15.5247f,  44.5478f,-22.3294f,17.2070f,  45.1706f,-23.3365f,16.7228f,  45.0079f,-24.5520f,18.3034f,  
	45.6329f,-25.5920f,17.7780f,  45.2432f,-26.9194f,19.2231f,  45.8707f,-27.9915f,18.6479f,  45.2612f,-29.4039f,19.9268f,  45.8926f,-30.5058f,19.2928f,  
	45.0744f,-31.9731f,20.3790f,  45.7089f,-33.1016f,19.6805f,  44.6967f,-34.5911f,20.5517f,  45.3368f,-35.7405f,19.7796f,  44.1486f,-37.2173f,20.4192f,  
	44.7957f,-38.3801f,19.5675f,  43.4525f,-39.8083f,19.9651f,  44.1068f,-40.9766f,19.0296f,  42.6319f,-42.3197f,19.1808f,  43.2945f,-43.4820f,18.1591f,  
	41.7134f,-44.7042f,18.0666f,  42.3849f,-45.8506f,16.9573f,  40.7245f,-46.9175f,16.6303f,  41.4065f,-48.0338f,15.4364f,  39.6945f,-48.9136f,14.8913f,  
	40.3848f,-49.9897f,13.6159f,  38.6495f,-50.6542f,12.8748f,  39.3471f,-51.6770f,11.5251f,  37.6166f,-52.1024f,10.6160f,  38.3196f,-53.0591f,9.2031f,  
	36.6215f,-53.2272f,8.1590f,  37.3276f,-54.1076f,6.6945f,  35.6883f,-54.0061f,5.5524f,  36.3924f,-54.8011f,4.0496f,  34.8373f,-54.4245f,2.8489f,  
	35.5352f,-55.1265f,1.3255f,  34.0877f,-54.4763f,0.1073f,  34.7737f,-55.0805f,-1.4224f,  33.4556f,-54.1654f,-2.6172f,  34.1240f,-54.6693f,-4.1331f,  
	32.9552f,-53.5053f,-5.2650f,  33.5999f,-53.9095f,-6.7497f,  3.6379f,-9.1581f,31.5861f,  4.2213f,-10.1733f,31.2422f,  3.3111f,-10.0896f,29.4627f,  
	3.8911f,-11.1023f,29.1144f,  3.1811f,-10.8678f,27.2588f,  3.7579f,-11.8772f,26.9053f,  3.2603f,-11.4957f,25.0056f,  3.8340f,-12.5010f,24.6466f,  
	3.5575f,-11.9786f,22.7354f,  4.1287f,-12.9790f,22.3704f,  4.0780f,-12.3245f,20.4806f,  4.6470f,-13.3187f,20.1093f,  4.8233f,-12.5431f,18.2737f,  
	5.3903f,-13.5294f,17.8963f,  5.7909f,-12.6460f,16.1467f,  6.3569f,-13.6234f,15.7627f,  6.9753f,-12.6473f,14.1296f,  7.5407f,-13.6146f,13.7398f,  
	8.3667f,-12.5632f,12.2517f,  8.9321f,-13.5186f,11.8566f,  9.9520f,-12.4105f,10.5397f,  10.5182f,-13.3530f,10.1399f,  11.7149f,-12.2085f,9.0174f,  
	12.2824f,-13.1365f,8.6141f,  13.6361f,-11.9768f,7.7059f,  14.2059f,-12.8910f,7.2999f,  15.6937f,-11.7382f,6.6221f,  16.2662f,-12.6374f,6.2149f,  
	17.8632f,-11.5139f,5.7795f,  18.4390f,-12.3982f,5.3725f,  20.1181f,-11.3265f,5.1871f,  20.6978f,-12.1966f,4.7816f,  22.4306f,-11.1991f,4.8492f,  
	23.0148f,-12.0575f,4.4460f,  24.7718f,-11.1556f,4.7648f,  25.3604f,-12.0027f,4.3654f,  27.1117f,-11.2169f,4.9290f,  27.7051f,-12.0556f,4.5339f,  
	29.4207f,-11.4048f,5.3308f,  30.0187f,-12.2385f,4.9401f,  31.6692f,-11.7394f,5.9544f,  32.2721f,-12.5711f,5.5684f,  33.8291f,-12.2382f,6.7796f,  
	34.4359f,-13.0729f,6.3961f,  35.8726f,-12.9180f,7.7796f,  36.4834f,-13.7593f,7.3978f,  37.7749f,-13.7908f,8.9252f,  38.3893f,-14.6432f,8.5428f,  
	39.5133f,-14.8663f,10.1825f,  40.1307f,-15.7353f,9.7942f,  41.0679f,-16.1509f,11.5121f,  41.6878f,-17.0404f,11.1147f,  42.4219f,-17.6457f,12.8745f,  
	43.0440f,-18.5599f,12.4625f,  43.5624f,-19.3476f,14.2268f,  44.1864f,-20.2903f,13.7936f,  44.4802f,-21.2489f,15.5247f,  45.1061f,-22.2230f,15.0621f,  
	45.1706f,-23.3365f,16.7228f,  45.7984f,-24.3436f,16.2236f,  45.6329f,-25.5920f,17.7780f,  46.2630f,-26.6325f,17.2332f,  45.8707f,-27.9915f,18.6479f,  
	46.5043f,-29.0642f,18.0480f,  45.8926f,-30.5058f,19.2928f,  46.5289f,-31.6084f,18.6317f,  45.7089f,-33.1016f,19.6805f,  46.3504f,-34.2294f,18.9481f,  
	45.3368f,-35.7405f,19.7796f,  45.9849f,-36.8870f,18.9686f,  44.7957f,-38.3801f,19.5675f,  45.4507f,-39.5383f,18.6727f,  44.1068f,-40.9766f,19.0296f,  
	44.7699f,-42.1357f,18.0469f,  43.2945f,-43.4820f,18.1591f,  43.9667f,-44.6326f,17.0857f,  42.3849f,-45.8506f,16.9573f,  43.0680f,-46.9786f,15.7941f,  
	41.4065f,-48.0338f,15.4364f,  42.0986f,-49.1290f,14.1853f,  40.3848f,-49.9897f,13.6159f,  41.0853f,-51.0386f,12.2823f,  39.3471f,-51.6770f,11.5251f,  
	40.0544f,-52.6662f,10.1186f,  38.3196f,-53.0591f,9.2031f,  39.0319f,-53.9774f,7.7339f,  37.3276f,-54.1076f,6.6945f,  38.0400f,-54.9448f,5.1752f,  
	36.3924f,-54.8011f,4.0496f,  37.1012f,-55.5481f,2.4970f,  35.5352f,-55.1265f,1.3255f,  36.2343f,-55.7770f,-0.2461f,  34.7737f,-55.0805f,-1.4224f,  
	35.4577f,-55.6303f,-2.9929f,  34.1240f,-54.6693f,-4.1331f,  34.7866f,-55.1171f,-5.6841f,  33.5999f,-53.9095f,-6.7497f,  34.2351f,-54.2565f,-8.2661f,  
	4.2213f,-10.1733f,31.2422f,  4.8051f,-11.1929f,30.8915f,  3.8911f,-11.1023f,29.1144f,  4.4716f,-12.1189f,28.7585f,  3.7579f,-11.8772f,26.9053f,  
	4.3353f,-12.8901f,26.5439f,  3.8340f,-12.5010f,24.6466f,  4.4089f,-13.5094f,24.2788f,  4.1287f,-12.9790f,22.3704f,  4.7012f,-13.9816f,21.9960f,  
	4.6470f,-13.3187f,20.1093f,  5.2174f,-14.3137f,19.7285f,  5.3903f,-13.5294f,17.8963f,  5.9595f,-14.5160f,17.5084f,  6.3569f,-13.6234f,15.7627f,  
	6.9253f,-14.5999f,15.3685f,  7.5407f,-13.6146f,13.7398f,  8.1089f,-14.5793f,13.3395f,  8.9321f,-13.5186f,11.8566f,  9.5008f,-14.4702f,11.4510f,  
	10.5182f,-13.3530f,10.1399f,  11.0879f,-14.2898f,9.7299f,  12.2824f,-13.1365f,8.6141f,  12.8542f,-14.0588f,8.2007f,  14.2059f,-12.8910f,7.2999f,  
	14.7802f,-13.7973f,6.8846f,  16.2662f,-12.6374f,6.2149f,  16.8437f,-13.5273f,5.7991f,  18.4390f,-12.3982f,5.3725f,  19.0202f,-13.2721f,4.9576f,  
	20.6978f,-12.1966f,4.7816f,  21.2834f,-13.0567f,4.3687f,  23.0148f,-12.0575f,4.4460f,  23.6047f,-12.9037f,4.0368f,  25.3604f,-12.0027f,4.3654f,  
	25.9552f,-12.8377f,3.9608f,  27.7051f,-12.0556f,4.5339f,  28.3046f,-12.8824f,4.1343f,  30.0187f,-12.2385f,4.9401f,  30.6233f,-13.0599f,4.5462f,  
	32.2721f,-12.5711f,5.5684f,  32.8808f,-13.3918f,5.1786f,  34.4359f,-13.0729f,6.3961f,  35.0489f,-13.8965f,6.0102f,  36.4834f,-13.7593f,7.3978f,  
	37.1002f,-14.5903f,7.0138f,  38.3893f,-14.6432f,8.5428f,  39.0093f,-15.4876f,8.1562f,  40.1307f,-15.7353f,9.7942f,  40.7535f,-16.5970f,9.4023f,  
	41.6878f,-17.0404f,11.1147f,  42.3130f,-17.9242f,10.7122f,  43.0440f,-18.5599f,12.4625f,  43.6711f,-19.4700f,12.0432f,  44.1864f,-20.2903f,13.7936f,  
	44.8153f,-21.2304f,13.3497f,  45.1061f,-22.2230f,15.0621f,  45.7369f,-23.1956f,14.5864f,  45.7984f,-24.3436f,16.2236f,  46.4313f,-25.3502f,15.7069f,  
	46.2630f,-26.6325f,17.2332f,  46.8990f,-27.6728f,16.6658f,  46.5043f,-29.0642f,18.0480f,  47.1427f,-30.1369f,17.4234f,  46.5289f,-31.6084f,18.6317f,  
	47.1721f,-32.7100f,17.9389f,  46.3504f,-34.2294f,18.9481f,  46.9997f,-35.3546f,18.1786f,  45.9849f,-36.8870f,18.9686f,  46.6408f,-38.0296f,18.1163f,  
	45.4507f,-39.5383f,18.6727f,  46.1145f,-40.6883f,17.7319f,  44.7699f,-42.1357f,18.0469f,  45.4427f,-43.2842f,17.0133f,  43.9667f,-44.6326f,17.0857f,  
	44.6505f,-45.7659f,15.9582f,  43.0680f,-46.9786f,15.7941f,  43.7615f,-48.0867f,14.5732f,  42.0986f,-49.1290f,14.1853f,  42.8013f,-50.1982f,12.8747f,  
	41.0853f,-51.0386f,12.2823f,  41.7962f,-52.0550f,10.8898f,  40.0544f,-52.6662f,10.1186f,  40.7721f,-53.6178f,8.6528f,  39.0319f,-53.9774f,7.7339f,  
	39.7519f,-54.8530f,6.2064f,  38.0400f,-54.9448f,5.1752f,  38.7587f,-55.7338f,3.6016f,  37.1012f,-55.5481f,2.4970f,  37.8126f,-56.2430f,0.8914f,  
	36.2343f,-55.7770f,-0.2461f,  36.9333f,-56.3717f,-1.8638f,  35.4577f,-55.6303f,-2.9929f,  36.1378f,-56.1217f,-4.6038f,  34.7866f,-55.1171f,-5.6841f,  
	35.4415f,-55.5052f,-7.2722f,  34.2351f,-54.2565f,-8.2661f,  34.8586f,-54.5440f,-9.8116f,  4.8051f,-11.1929f,30.8915f,  5.3894f,-12.2166f,30.5334f,  
	4.4716f,-12.1189f,28.7585f,  5.0528f,-13.1393f,28.3946f,  4.3353f,-12.8901f,26.5439f,  4.9139f,-13.9065f,26.1734f,  4.4089f,-13.5094f,24.2788f,  
	4.9850f,-14.5204f,23.9016f,  4.7012f,-13.9816f,21.9960f,  5.2751f,-14.9856f,21.6120f,  5.2174f,-14.3137f,19.7285f,  5.7899f,-15.3098f,19.3368f,  
	5.9595f,-14.5160f,17.5084f,  6.5310f,-15.5023f,17.1099f,  6.9253f,-14.5999f,15.3685f,  7.4964f,-15.5747f,14.9632f,  8.1089f,-14.5793f,13.3395f,  
	8.6802f,-15.5411f,12.9281f,  9.5008f,-14.4702f,11.4510f,  10.0730f,-15.4170f,11.0343f,  11.0879f,-14.2898f,9.7299f,  11.6619f,-15.2215f,9.3091f,  
	12.8542f,-14.0588f,8.2007f,  13.4305f,-14.9737f,7.7771f,  14.7802f,-13.7973f,6.8846f,  15.3594f,-14.6947f,6.4596f,  16.8437f,-13.5273f,5.7991f,  
	17.4265f,-14.4072f,5.3743f,  19.0202f,-13.2721f,4.9576f,  19.6073f,-14.1362f,4.5342f,  21.2834f,-13.0567f,4.3687f,  21.8748f,-13.9045f,3.9487f,  
	23.6047f,-12.9037f,4.0368f,  24.2010f,-13.7376f,3.6213f,  25.9552f,-12.8377f,3.9608f,  26.5563f,-13.6603f,3.5507f,  28.3046f,-12.8824f,4.1343f,  
	28.9111f,-13.6962f,3.7305f,  30.6233f,-13.0599f,4.5462f,  31.2340f,-13.8693f,4.1481f,  32.8808f,-13.3918f,5.1786f,  33.4960f,-14.2002f,4.7861f,  
	35.0489f,-13.8965f,6.0102f,  35.6682f,-14.7086f,5.6219f,  37.1002f,-14.5903f,7.0138f,  37.7230f,-15.4120f,6.6259f,  39.0093f,-15.4876f,8.1562f,  
	39.6351f,-16.3229f,7.7664f,  40.7535f,-16.5970f,9.4023f,  41.3819f,-17.4512f,9.0059f,  42.3130f,-17.9242f,10.7122f,  42.9434f,-18.8020f,10.3036f,  
	43.6711f,-19.4700f,12.0432f,  44.3034f,-20.3759f,11.6148f,  44.8153f,-21.2304f,13.3497f,  45.4494f,-22.1676f,12.8944f,  45.7369f,-23.1956f,14.5864f,  
	46.3728f,-24.1665f,14.0953f,  46.4313f,-25.3502f,15.7069f,  47.0699f,-26.3556f,15.1699f,  46.8990f,-27.6728f,16.6658f,  47.5398f,-28.7124f,16.0761f,  
	47.1427f,-30.1369f,17.4234f,  47.7877f,-31.2084f,16.7693f,  47.1721f,-32.7100f,17.9389f,  47.8227f,-33.8092f,17.2110f,  46.9997f,-35.3546f,18.1786f,  
	47.6565f,-36.4762f,17.3699f,  46.6408f,-38.0296f,18.1163f,  47.3052f,-39.1646f,17.2195f,  46.1145f,-40.6883f,17.7319f,  46.7878f,-41.8285f,16.7414f,  
	45.4427f,-43.2842f,17.0133f,  46.1271f,-44.4166f,15.9259f,  44.6505f,-45.7659f,15.9582f,  45.3450f,-46.8806f,14.7730f,  43.7615f,-48.0867f,14.5732f,  
	44.4660f,-49.1702f,13.2921f,  42.8013f,-50.1982f,12.8747f,  43.5152f,-51.2361f,11.5035f,  41.7962f,-52.0550f,10.8898f,  42.5185f,-53.0348f,9.4354f,  
	40.7721f,-53.6178f,8.6528f,  41.4986f,-54.5273f,7.1255f,  39.7519f,-54.8530f,6.2064f,  40.4794f,-55.6805f,4.6206f,  38.7587f,-55.7338f,3.6016f,  
	39.4815f,-56.4704f,1.9708f,  37.8126f,-56.2430f,0.8914f,  38.5258f,-56.8809f,-0.7652f,  36.9333f,-56.3717f,-1.8638f,  37.6303f,-56.9061f,-3.5271f,  
	36.1378f,-56.1217f,-4.6038f,  36.8119f,-56.5510f,-6.2572f,  35.4415f,-55.5052f,-7.2722f,  36.0863f,-55.8302f,-8.8948f,  34.8586f,-54.5440f,-9.8116f,  
	35.4682f,-54.7687f,-11.3869f,  5.3894f,-12.2166f,30.5334f,  5.9742f,-13.2443f,30.1671f,  5.0528f,-13.1393f,28.3946f,  5.6350f,-14.1635f,28.0217f,  
	4.9139f,-13.9065f,26.1734f,  5.4936f,-14.9259f,25.7935f,  4.9850f,-14.5204f,23.9016f,  5.5624f,-15.5334f,23.5145f,  5.2751f,-14.9856f,21.6120f,  
	5.8510f,-15.9914f,21.2167f,  5.7899f,-15.3098f,19.3368f,  6.3646f,-16.3063f,18.9342f,  6.5310f,-15.5023f,17.1099f,  7.1051f,-16.4876f,16.6997f,  
	7.4964f,-15.5747f,14.9632f,  8.0705f,-16.5472f,14.5462f,  8.6802f,-15.5411f,12.9281f,  9.2550f,-16.4988f,12.5049f,  10.0730f,-15.4170f,11.0343f,  
	10.6492f,-16.3593f,10.6060f,  11.6619f,-15.2215f,9.3091f,  12.2402f,-16.1465f,8.8770f,  13.4305f,-14.9737f,7.7771f,  14.0115f,-15.8803f,7.3426f,  
	15.3594f,-14.6947f,6.4596f,  15.9438f,-15.5825f,6.0244f,  17.4265f,-14.4072f,5.3743f,  18.0151f,-15.2775f,4.9398f,  19.6073f,-14.1362f,4.5342f,  
	20.2002f,-14.9880f,4.1026f,  21.8748f,-13.9045f,3.9487f,  22.4726f,-14.7399f,3.5212f,  24.2010f,-13.7376f,3.6213f,  24.8037f,-14.5587f,3.1993f,  
	26.5563f,-13.6603f,3.5507f,  27.1645f,-14.4692f,3.1354f,  28.9111f,-13.6962f,3.7305f,  29.5236f,-14.4971f,3.3220f,  31.2340f,-13.8693f,4.1481f,  
	31.8512f,-14.6652f,3.7465f,  33.4960f,-14.2002f,4.7861f,  34.1177f,-14.9959f,4.3908f,  35.6682f,-14.7086f,5.6219f,  36.2936f,-15.5098f,5.2297f,  
	37.7230f,-15.4120f,6.6259f,  38.3516f,-16.2229f,6.2351f,  39.6351f,-16.3229f,7.7664f,  40.2666f,-17.1492f,7.3728f,  41.3819f,-17.4512f,9.0059f,  
	42.0156f,-18.2978f,8.6044f,  42.9434f,-18.8020f,10.3036f,  43.5791f,-19.6741f,9.8871f,  44.3034f,-20.3759f,11.6148f,  44.9408f,-21.2773f,11.1766f,  
	45.4494f,-22.1676f,12.8944f,  46.0884f,-23.1016f,12.4257f,  46.3728f,-24.1665f,14.0953f,  47.0142f,-25.1350f,13.5858f,  47.0699f,-26.3556f,15.1699f,  
	47.7133f,-27.3592f,14.6129f,  47.5398f,-28.7124f,16.0761f,  48.1869f,-29.7500f,15.4593f,  47.7877f,-31.2084f,16.7693f,  48.4399f,-32.2770f,16.0824f,  
	47.8227f,-33.8092f,17.2110f,  48.4806f,-34.9046f,16.4462f,  47.6565f,-36.4762f,17.3699f,  48.3216f,-37.5905f,16.5185f,  47.3052f,-39.1646f,17.2195f,  
	47.9788f,-40.2904f,16.2747f,  46.7878f,-41.8285f,16.7414f,  47.4725f,-42.9536f,15.6978f,  46.1271f,-44.4166f,15.9259f,  46.8222f,-45.5315f,14.7813f,  
	45.3450f,-46.8806f,14.7730f,  46.0507f,-47.9721f,13.5274f,  44.4660f,-49.1702f,13.2921f,  45.1820f,-50.2235f,11.9492f,  43.5152f,-51.2361f,11.5035f,  
	44.2411f,-52.2385f,10.0683f,  42.5185f,-53.0348f,9.4354f,  43.2505f,-53.9733f,7.9169f,  41.4986f,-54.5273f,7.1255f,  42.2339f,-55.3891f,5.5365f,  
	40.4794f,-55.6805f,4.6206f,  41.2125f,-56.4555f,2.9736f,  39.4815f,-56.4704f,1.9708f,  40.2077f,-57.1492f,0.2842f,  38.5258f,-56.8809f,-0.7652f,  
	39.2386f,-57.4570f,-2.4725f,  37.6303f,-56.9061f,-3.5271f,  38.3229f,-57.3761f,-5.2383f,  36.8119f,-56.5510f,-6.2572f,  37.4776f,-56.9141f,-7.9506f,  
	36.0863f,-55.8302f,-8.8948f,  36.7186f,-56.0886f,-10.5524f,  35.4682f,-54.7687f,-11.3869f,  36.0614f,-54.9275f,-12.9913f,  5.9742f,-13.2443f,30.1671f,  
	6.5600f,-14.2761f,29.7919f,  5.6350f,-14.1635f,28.0217f,  6.2183f,-15.1912f,27.6392f,  5.4936f,-14.9259f,25.7935f,  6.0745f,-15.9479f,25.4034f,  
	5.5624f,-15.5334f,23.5145f,  6.1417f,-16.5489f,23.1159f,  5.8510f,-15.9914f,21.2167f,  6.4289f,-16.9981f,20.8102f,  6.3646f,-16.3063f,18.9342f,  
	6.9418f,-17.3025f,18.5194f,  7.1051f,-16.4876f,16.6997f,  7.6821f,-17.4715f,16.2773f,  8.0705f,-16.5472f,14.5462f,  8.6479f,-17.5167f,14.1166f,  
	9.2550f,-16.4988f,12.5049f,  9.8336f,-17.4527f,12.0693f,  10.6492f,-16.3593f,10.6060f,  11.2297f,-17.2956f,10.1656f,  12.2402f,-16.1465f,8.8770f,  
	12.8232f,-17.0639f,8.4330f,  14.0115f,-15.8803f,7.3426f,  14.5977f,-16.7779f,6.8969f,  15.9438f,-15.5825f,6.0244f,  16.5341f,-16.4610f,5.5785f,  
	18.0151f,-15.2775f,4.9398f,  18.6095f,-16.1357f,4.4959f,  20.2002f,-14.9880f,4.1026f,  20.7995f,-15.8275f,3.6623f,  22.4726f,-14.7399f,3.5212f,  
	23.0769f,-15.5622f,3.0862f,  24.8037f,-14.5587f,3.1993f,  25.4136f,-15.3657f,2.7710f,  27.1645f,-14.4692f,3.1354f,  27.7789f,-15.2645f,2.7145f,  
	29.5236f,-14.4971f,3.3220f,  30.1428f,-15.2836f,2.9092f,  31.8512f,-14.6652f,3.7465f,  32.4754f,-15.4473f,3.3416f,  34.1177f,-14.9959f,4.3908f,  
	34.7457f,-15.7794f,3.9914f,  36.2936f,-15.5098f,5.2297f,  36.9251f,-16.2987f,4.8346f,  38.3516f,-16.2229f,6.2351f,  38.9862f,-17.0231f,5.8408f,  
	40.2666f,-17.1492f,7.3728f,  40.9036f,-17.9659f,6.9749f,  42.0156f,-18.2978f,8.6044f,  42.6547f,-19.1369f,8.1962f,  43.5791f,-19.6741f,9.8871f,  
	44.2198f,-20.5398f,9.4624f,  44.9408f,-21.2773f,11.1766f,  45.5831f,-22.1738f,10.7269f,  46.0884f,-23.1016f,12.4257f,  46.7329f,-24.0320f,11.9408f,  
	47.0142f,-25.1350f,13.5858f,  47.6604f,-26.1004f,13.0589f,  47.7133f,-27.3592f,14.6129f,  48.3627f,-28.3601f,14.0313f,  48.1869f,-29.7500f,15.4593f,  
	48.8407f,-30.7845f,14.8121f,  48.4399f,-32.2770f,16.0824f,  49.0990f,-33.3418f,15.3611f,  48.4806f,-34.9046f,16.4462f,  49.1463f,-35.9931f,15.6409f,  
	48.3216f,-37.5905f,16.5185f,  48.9955f,-38.6963f,15.6211f,  47.9788f,-40.2904f,16.2747f,  48.6635f,-41.4022f,15.2778f,  47.4725f,-42.9536f,15.6978f,  
	48.1676f,-44.0624f,14.5979f,  46.8222f,-45.5315f,14.7813f,  47.5283f,-46.6244f,13.5765f,  46.0507f,-47.9721f,13.5274f,  46.7680f,-49.0345f,12.2192f,  
	45.1820f,-50.2235f,11.9492f,  45.9105f,-51.2424f,10.5407f,  44.2411f,-52.2385f,10.0683f,  44.9774f,-53.2005f,8.5669f,  43.2505f,-53.9733f,7.9169f,  
	43.9922f,-54.8645f,6.3335f,  42.2339f,-55.3891f,5.5365f,  42.9759f,-56.1985f,3.8826f,  41.2125f,-56.4555f,2.9736f,  41.9504f,-57.1720f,1.2663f,  
	40.2077f,-57.1492f,0.2842f,  40.9352f,-57.7647f,-1.4581f,  39.2386f,-57.4570f,-2.4725f,  39.9486f,-57.9665f,-4.2330f,  38.3229f,-57.3761f,-5.2383f,  
	39.0087f,-57.7770f,-6.9949f,  37.4776f,-56.9141f,-7.9506f,  38.1322f,-57.2067f,-9.6841f,  36.7186f,-56.0886f,-10.5524f,  37.3358f,-56.2766f,-12.2443f,  
	36.0614f,-54.9275f,-12.9913f,  36.6357f,-55.0168f,-14.6262f,  6.5600f,-14.2761f,29.7919f,  7.1466f,-15.3117f,29.4071f,  6.2183f,-15.1912f,27.6392f,  
	6.8025f,-16.2218f,27.2465f,  6.0745f,-15.9479f,25.4034f,  6.6571f,-16.9729f,25.0018f,  6.1417f,-16.5489f,23.1159f,  6.7228f,-17.5659f,22.7058f,  
	6.4289f,-16.9981f,20.8102f,  7.0091f,-18.0054f,20.3912f,  6.9418f,-17.3025f,18.5194f,  7.5216f,-18.2981f,18.0920f,  7.6821f,-17.4715f,16.2773f,  
	8.2622f,-18.4533f,15.8418f,  8.6479f,-17.5167f,14.1166f,  9.2289f,-18.4830f,13.6740f,  9.8336f,-17.4527f,12.0693f,  10.4162f,-18.4015f,11.6207f,  
	11.2297f,-17.2956f,10.1656f,  11.8146f,-18.2252f,9.7123f,  12.8232f,-17.0639f,8.4330f,  13.4112f,-17.9729f,7.9767f,  14.5977f,-16.7779f,6.8969f,  
	15.1896f,-17.6666f,6.4393f,  16.5341f,-16.4610f,5.5785f,  17.1300f,-17.3279f,5.1219f,  18.6095f,-16.1357f,4.4959f,  19.2103f,-16.9818f,4.0421f,  
	20.7995f,-15.8275f,3.6623f,  21.4052f,-16.6538f,3.2132f,  23.0769f,-15.5622f,3.0862f,  23.6883f,-16.3703f,2.6435f,  25.4136f,-15.3657f,2.7710f,  
	26.0296f,-16.1585f,2.3361f,  27.7789f,-15.2645f,2.7145f,  28.4000f,-16.0447f,2.2885f,  30.1428f,-15.2836f,2.9092f,  30.7692f,-16.0552f,2.4922f,  
	32.4754f,-15.4473f,3.3416f,  33.1057f,-16.2160f,2.9322f,  34.7457f,-15.7794f,3.9914f,  35.3799f,-16.5490f,3.5889f,  36.9251f,-16.2987f,4.8346f,  
	37.5627f,-17.0753f,4.4362f,  38.9862f,-17.0231f,5.8408f,  39.6265f,-17.8121f,5.4428f,  40.9036f,-17.9659f,6.9749f,  41.5461f,-18.7735f,6.5711f,  
	42.6547f,-19.1369f,8.1962f,  43.2988f,-19.9678f,7.7812f,  44.2198f,-20.5398f,9.4624f,  44.8655f,-21.3990f,9.0278f,  45.5831f,-22.1738f,10.7269f,  
	46.2306f,-23.0653f,10.2629f,  46.7329f,-24.0320f,11.9408f,  47.3819f,-24.9578f,11.4407f,  47.6604f,-26.1004f,13.0589f,  48.3121f,-27.0622f,12.5098f,  
	48.3627f,-28.3601f,14.0313f,  49.0183f,-29.3570f,13.4218f,  48.8407f,-30.7845f,14.8121f,  49.5010f,-31.8145f,14.1331f,  49.0990f,-33.3418f,15.3611f,  
	49.7653f,-34.3997f,14.6017f,  49.1463f,-35.9931f,15.6409f,  49.8204f,-37.0733f,14.7917f,  48.9955f,-38.6963f,15.6211f,  49.6799f,-39.7887f,14.6730f,  
	48.6635f,-41.4022f,15.2778f,  49.3583f,-42.4984f,14.2260f,  48.1676f,-44.0624f,14.5979f,  48.8736f,-45.1502f,13.4386f,  47.5283f,-46.6244f,13.5765f,  
	48.2461f,-47.6895f,12.3091f,  46.7680f,-49.0345f,12.2192f,  47.4981f,-50.0637f,10.8446f,  45.9105f,-51.2424f,10.5407f,  46.6498f,-52.2221f,9.0646f,  
	44.9774f,-53.2005f,8.5669f,  45.7242f,-54.1158f,6.9982f,  43.9922f,-54.8645f,6.3335f,  44.7416f,-55.7037f,4.6823f,  42.9759f,-56.1985f,3.8826f,  
	43.7240f,-56.9490f,2.1644f,  41.9504f,-57.1720f,1.2663f,  42.6910f,-57.8242f,-0.5012f,  40.9352f,-57.7647f,-1.4581f,  41.6612f,-58.3120f,-3.2585f,  
	39.9486f,-57.9665f,-4.2330f,  40.6534f,-58.4042f,-6.0442f,  39.0087f,-57.7770f,-6.9949f,  39.6847f,-58.1038f,-8.7970f,  38.1322f,-57.2067f,-9.6841f,  
	38.7727f,-57.4246f,-11.4573f,  37.3358f,-56.2766f,-12.2443f,  37.9347f,-56.3901f,-13.9713f,  36.6357f,-55.0168f,-14.6262f,  37.1883f,-55.0336f,-16.2892f,  
	7.1466f,-15.3117f,29.4071f,  7.7340f,-16.3507f,29.0122f,  6.8025f,-16.2218f,27.2465f,  7.3884f,-17.2562f,26.8422f,  6.6571f,-16.9729f,25.0018f,  
	7.2413f,-18.0000f,24.5886f,  6.7228f,-17.5659f,22.7058f,  7.3060f,-18.5842f,22.2830f,  7.0091f,-18.0054f,20.3912f,  7.5918f,-19.0128f,19.9592f,  
	7.5216f,-18.2981f,18.0920f,  8.1043f,-19.2924f,17.6509f,  8.2622f,-18.4533f,15.8418f,  8.8456f,-19.4326f,15.3926f,  9.2289f,-18.4830f,13.6740f,  
	9.8138f,-19.4450f,13.2177f,  10.4162f,-18.4015f,11.6207f,  11.0032f,-19.3443f,11.1584f,  11.8146f,-18.2252f,9.7123f,  12.4044f,-19.1471f,9.2456f,  
	13.4112f,-17.9729f,7.9767f,  14.0046f,-18.8736f,7.5076f,  15.1896f,-17.6666f,6.4393f,  15.7870f,-18.5442f,5.9699f,  17.1300f,-17.3279f,5.1219f,  
	17.7322f,-18.1829f,4.6541f,  19.2103f,-16.9818f,4.0421f,  19.8174f,-17.8147f,3.5782f,  21.4052f,-16.6538f,3.2132f,  22.0181f,-17.4658f,2.7551f,  
	23.6883f,-16.3703f,2.6435f,  24.3059f,-17.1637f,2.1932f,  26.0296f,-16.1585f,2.3361f,  26.6525f,-16.9355f,1.8949f,  28.4000f,-16.0447f,2.2885f,  
	29.0283f,-16.8091f,1.8572f,  30.7692f,-16.0552f,2.4922f,  31.4018f,-16.8123f,2.0702f,  33.1057f,-16.2160f,2.9322f,  33.7424f,-16.9695f,2.5193f,  
	35.3799f,-16.5490f,3.5889f,  36.0203f,-17.3048f,3.1830f,  37.5627f,-17.0753f,4.4362f,  38.2061f,-17.8390f,4.0343f,  39.6265f,-17.8121f,5.4428f,  
	40.2722f,-18.5901f,5.0395f,  41.5461f,-18.7735f,6.5711f,  42.1936f,-19.5710f,6.1616f,  43.2988f,-19.9678f,7.7812f,  43.9479f,-20.7905f,7.3576f,  
	44.8655f,-21.3990f,9.0278f,  45.5160f,-22.2516f,8.5808f,  46.2306f,-23.0653f,10.2629f,  46.8825f,-23.9506f,9.7858f,  47.3819f,-24.9578f,11.4407f,  
	48.0360f,-25.8786f,10.9207f,  48.3121f,-27.0622f,12.5098f,  48.9695f,-28.0191f,11.9353f,  49.0183f,-29.3570f,13.4218f,  49.6797f,-30.3489f,12.7833f,  
	49.5010f,-31.8145f,14.1331f,  50.1679f,-32.8375f,13.4186f,  49.7653f,-34.3997f,14.6017f,  50.4394f,-35.4494f,13.8009f,  49.8204f,-37.0733f,14.7917f,  
	50.5043f,-38.1408f,13.8938f,  49.6799f,-39.7887f,14.6730f,  50.3739f,-40.8664f,13.6720f,  49.3583f,-42.4984f,14.2260f,  50.0636f,-43.5747f,13.1161f,  
	48.8736f,-45.1502f,13.4386f,  49.5911f,-46.2114f,12.2171f,  48.2461f,-47.6895f,12.3091f,  48.9767f,-48.7225f,10.9747f,  47.4981f,-50.0637f,10.8446f,  
	48.2392f,-51.0547f,9.4013f,  46.6498f,-52.2221f,9.0646f,  47.4003f,-53.1558f,7.5190f,  45.7242f,-54.1158f,6.9982f,  46.4794f,-54.9796f,5.3590f,  
	44.7416f,-55.7037f,4.6823f,  45.4983f,-56.4839f,2.9631f,  43.7240f,-56.9490f,2.1644f,  44.4760f,-57.6343f,0.3818f,  42.6910f,-57.8242f,-0.5012f,  
	43.4315f,-58.4067f,-2.3315f,  41.6612f,-58.3120f,-3.2585f,  42.3835f,-58.7850f,-5.1148f,  40.6534f,-58.4042f,-6.0442f,  41.3495f,-58.7646f,-7.9061f,  
	39.6847f,-58.1038f,-8.7970f,  40.3477f,-58.3519f,-10.6438f,  38.7727f,-57.4246f,-11.4573f,  39.3957f,-57.5632f,-13.2704f,  37.9347f,-56.3901f,-13.9713f,  
	38.5123f,-56.4256f,-15.7309f,  37.1883f,-55.0336f,-16.2892f,  37.7162f,-54.9745f,-17.9802f,  7.7340f,-16.3507f,29.0122f,  8.3229f,-17.3938f,28.6056f,  
	7.3884f,-17.2562f,26.8422f,  7.9755f,-18.2930f,26.4263f,  7.2413f,-18.0000f,24.5886f,  7.8274f,-19.0292f,24.1625f,  7.3060f,-18.5842f,22.2830f,  
	7.8914f,-19.6035f,21.8469f,  7.5918f,-19.0128f,19.9592f,  8.1771f,-20.0199f,19.5130f,  8.1043f,-19.2924f,17.6509f,  8.6902f,-20.2852f,17.1957f,  
	8.8456f,-19.4326f,15.3926f,  9.4327f,-20.4085f,14.9290f,  9.8138f,-19.4450f,13.2177f,  10.4028f,-20.4021f,12.7467f,  11.0032f,-19.3443f,11.1584f,  
	11.5948f,-20.2803f,10.6817f,  12.4044f,-19.1471f,9.2456f,  12.9995f,-20.0613f,8.7650f,  14.0046f,-18.8736f,7.5076f,  14.6034f,-19.7638f,7.0253f,  
	15.7870f,-18.5442f,5.9699f,  16.3905f,-19.4104f,5.4879f,  17.7322f,-18.1829f,4.6541f,  18.3405f,-19.0251f,4.1750f,  19.8174f,-17.8147f,3.5782f,  
	20.4316f,-18.6335f,3.1039f,  22.0181f,-17.4658f,2.7551f,  22.6371f,-18.2628f,2.2882f,  24.3059f,-17.1637f,2.1932f,  24.9303f,-17.9408f,1.7354f,  
	26.6525f,-16.9355f,1.8949f,  27.2826f,-17.6961f,1.4474f,  29.0283f,-16.8091f,1.8572f,  29.6630f,-17.5580f,1.4201f,  31.4018f,-16.8123f,2.0702f,  
	32.0408f,-17.5530f,1.6441f,  33.7424f,-16.9695f,2.5193f,  34.3855f,-17.7077f,2.1026f,  36.0203f,-17.3048f,3.1830f,  36.6666f,-18.0461f,2.7736f,  
	38.2061f,-17.8390f,4.0343f,  38.8549f,-18.5900f,3.6277f,  40.2722f,-18.5901f,5.0395f,  40.9230f,-19.3563f,4.6314f,  42.1936f,-19.5710f,6.1616f,  
	42.8458f,-20.3585f,5.7448f,  43.9479f,-20.7905f,7.3576f,  44.6014f,-21.6048f,6.9233f,  45.5160f,-22.2516f,8.5808f,  46.1709f,-23.0963f,8.1227f,  
	46.8825f,-23.9506f,9.7858f,  47.5391f,-24.8296f,9.2914f,  48.0360f,-25.8786f,10.9207f,  48.6953f,-26.7935f,10.3781f,  48.9695f,-28.0191f,11.9353f,  
	49.6322f,-28.9700f,11.3347f,  49.6797f,-30.3489f,12.7833f,  50.3473f,-31.3334f,12.1119f,  50.1679f,-32.8375f,13.4186f,  50.8419f,-33.8522f,12.6655f,  
	50.4394f,-35.4494f,13.8009f,  51.1225f,-36.4868f,12.9536f,  50.5043f,-38.1408f,13.8938f,  51.1971f,-39.1941f,12.9453f,  50.3739f,-40.8664f,13.6720f,  
	51.0779f,-41.9250f,12.6145f,  50.0636f,-43.5747f,13.1161f,  50.7799f,-44.6256f,11.9449f,  49.5911f,-46.2114f,12.2171f,  50.3211f,-47.2417f,10.9288f,  
	48.9767f,-48.7225f,10.9747f,  49.7183f,-49.7185f,9.5713f,  48.2392f,-51.0547f,9.4013f,  48.9917f,-52.0008f,7.8871f,  47.4003f,-53.1558f,7.5190f,  
	48.1596f,-54.0387f,5.9010f,  46.4794f,-54.9796f,5.3590f,  47.2427f,-55.7846f,3.6487f,  45.4983f,-56.4839f,2.9631f,  46.2597f,-57.1983f,1.1758f,  
	44.4760f,-57.6343f,0.3818f,  45.2291f,-58.2489f,-1.4679f,  43.4315f,-58.4067f,-2.3315f,  44.1694f,-58.9128f,-4.2225f,  42.3835f,-58.7850f,-5.1148f,  
	43.0982f,-59.1778f,-7.0266f,  41.3495f,-58.7646f,-7.9061f,  42.0338f,-59.0425f,-9.8178f,  40.3477f,-58.3519f,-10.6438f,  40.9939f,-58.5160f,-12.5355f,  
	39.3957f,-57.5632f,-13.2704f,  39.9979f,-57.6184f,-15.1207f,  38.5123f,-56.4256f,-15.7309f,  39.0651f,-56.3791f,-17.5222f,  37.7162f,-54.9745f,-17.9802f,  
	38.2163f,-54.8358f,-19.6990f,  8.3229f,-17.3938f,28.6056f,  8.9128f,-18.4398f,28.1875f,  7.9755f,-18.2930f,26.4263f,  8.5643f,-19.3325f,25.9974f,  
	7.8274f,-19.0292f,24.1625f,  8.4154f,-20.0599f,23.7229f,  7.8914f,-19.6035f,21.8469f,  8.4793f,-20.6232f,21.3963f,  8.1771f,-20.0199f,19.5130f,  
	8.7653f,-21.0262f,19.0524f,  8.6902f,-20.2852f,17.1957f,  9.2794f,-21.2754f,16.7254f,  9.4327f,-20.4085f,14.9290f,  10.0235f,-21.3804f,14.4499f,  
	10.4028f,-20.4021f,12.7467f,  10.9960f,-21.3532f,12.2604f,  11.5948f,-20.2803f,10.6817f,  12.1913f,-21.2094f,10.1901f,  12.9995f,-20.0613f,8.7650f,  
	13.5996f,-20.9659f,8.2701f,  14.6034f,-19.7638f,7.0253f,  15.2082f,-20.6433f,6.5292f,  16.3905f,-19.4104f,5.4879f,  17.0000f,-20.2642f,4.9933f,  
	18.3405f,-19.0251f,4.1750f,  18.9560f,-19.8533f,3.6839f,  20.4316f,-18.6335f,3.1039f,  21.0518f,-19.4372f,2.6196f,  22.6371f,-18.2628f,2.2882f,  
	23.2628f,-19.0434f,1.8124f,  24.9303f,-17.9408f,1.7354f,  25.5620f,-18.7011f,1.2699f,  27.2826f,-17.6961f,1.4474f,  27.9191f,-18.4405f,0.9932f,  
	29.6630f,-17.5580f,1.4201f,  30.3040f,-18.2895f,0.9782f,  32.0408f,-17.5530f,1.6441f,  32.6863f,-18.2771f,1.2135f,  34.3855f,-17.7077f,2.1026f,  
	35.0345f,-18.4299f,1.6823f,  36.6666f,-18.0461f,2.7736f,  37.3183f,-18.7731f,2.3598f,  38.8549f,-18.5900f,3.6277f,  39.5087f,-19.3273f,3.2167f,  
	40.9230f,-19.3563f,4.6314f,  41.5784f,-20.1106f,4.2170f,  42.8458f,-20.3585f,5.7448f,  43.5024f,-21.1359f,5.3188f,  44.6014f,-21.6048f,6.9233f,  
	45.2592f,-22.4094f,6.4798f,  46.1709f,-23.0963f,8.1227f,  46.8300f,-23.9331f,7.6495f,  47.5391f,-24.8296f,9.2914f,  48.2003f,-25.7014f,8.7767f,  
	48.6953f,-26.7935f,10.3781f,  49.3593f,-27.7014f,9.8121f,  49.6322f,-28.9700f,11.3347f,  50.3003f,-29.9129f,10.7040f,  50.3473f,-31.3334f,12.1119f,  
	51.0210f,-32.3091f,11.4050f,  50.8419f,-33.8522f,12.6655f,  51.5239f,-34.8548f,11.8684f,  51.1225f,-36.4868f,12.9536f,  51.8137f,-37.5103f,12.0583f,  
	51.1971f,-39.1941f,12.9453f,  51.8991f,-40.2291f,11.9424f,  51.0779f,-41.9250f,12.6145f,  51.7921f,-42.9592f,11.4972f,  50.7799f,-44.6256f,11.9449f,  
	51.5082f,-45.6468f,10.7077f,  50.3211f,-47.2417f,10.9288f,  51.0618f,-48.2363f,9.5717f,  49.7183f,-49.7185f,9.5713f,  50.4713f,-50.6707f,8.0962f,  
	48.9917f,-52.0008f,7.8871f,  49.7533f,-52.8970f,6.2993f,  48.1596f,-54.0387f,5.9010f,  48.9275f,-54.8632f,4.2096f,  47.2427f,-55.7846f,3.6487f,  
	48.0116f,-56.5238f,1.8672f,  46.2597f,-57.1983f,1.1758f,  47.0232f,-57.8414f,-0.6823f,  45.2291f,-58.2489f,-1.4679f,  45.9806f,-58.7855f,-3.3826f,  
	44.1694f,-58.9128f,-4.2225f,  44.9009f,-59.3363f,-6.1738f,  43.0982f,-59.1778f,-7.0266f,  43.8021f,-59.4848f,-8.9934f,  42.0338f,-59.0425f,-9.8178f,  
	42.7018f,-59.2323f,-11.7792f,  40.9939f,-58.5160f,-12.5355f,  41.6196f,-58.5916f,-14.4690f,  39.9979f,-57.6184f,-15.1207f,  40.5752f,-57.5857f,-17.0066f,  
	39.0651f,-56.3791f,-17.5222f,  39.5897f,-56.2466f,-19.3446f,  38.2163f,-54.8358f,-19.6990f,  38.6854f,-54.6138f,-21.4447f,  8.9128f,-18.4398f,28.1875f,  
	9.5042f,-19.4892f,27.7564f,  8.5643f,-19.3325f,25.9974f,  9.1548f,-20.3743f,25.5550f,  8.4154f,-20.0599f,23.7229f,  9.0057f,-21.0921f,23.2686f,  
	8.4793f,-20.6232f,21.3963f,  9.0697f,-21.6429f,20.9310f,  8.7653f,-21.0262f,19.0524f,  9.3565f,-22.0309f,18.5762f,  9.2794f,-21.2754f,16.7254f,  
	9.8721f,-22.2627f,16.2390f,  10.0235f,-21.3804f,14.4499f,  10.6185f,-22.3475f,13.9548f,  10.9960f,-21.3532f,12.2604f,  11.5940f,-22.2983f,11.7584f,  
	12.1913f,-21.2094f,10.1901f,  12.7927f,-22.1298f,9.6830f,  13.5996f,-20.9659f,8.2701f,  14.2054f,-21.8604f,7.7602f,  15.2082f,-20.6433f,6.5292f,  
	15.8187f,-21.5109f,6.0192f,  17.0000f,-20.2642f,4.9933f,  17.6164f,-21.1045f,4.4853f,  18.9560f,-19.8533f,3.6839f,  19.5771f,-20.6667f,3.1816f,  
	21.0518f,-19.4372f,2.6196f,  21.6786f,-20.2244f,2.1249f,  23.2628f,-19.0434f,1.8124f,  23.8958f,-19.8068f,1.3276f,  25.5620f,-18.7011f,1.2699f,  
	26.1999f,-19.4445f,0.7966f,  27.9191f,-18.4405f,0.9932f,  28.5619f,-19.1664f,0.5330f,  30.3040f,-18.2895f,0.9782f,  30.9516f,-19.0033f,0.5310f,  
	32.6863f,-18.2771f,1.2135f,  33.3377f,-18.9838f,0.7789f,  35.0345f,-18.4299f,1.6823f,  35.6889f,-19.1362f,1.2574f,  37.3183f,-18.7731f,2.3598f,  
	37.9751f,-19.4846f,1.9419f,  39.5087f,-19.3273f,3.2167f,  40.1671f,-20.0509f,2.8004f,  41.5784f,-20.1106f,4.2170f,  42.2377f,-20.8530f,3.7948f,  
	43.5024f,-21.1359f,5.3188f,  44.1630f,-21.9017f,4.8853f,  45.2592f,-22.4094f,6.4798f,  45.9207f,-23.2045f,6.0234f,  46.8300f,-23.9331f,7.6495f,  
	47.4930f,-24.7613f,7.1587f,  48.2003f,-25.7014f,8.7767f,  48.8655f,-26.5648f,8.2416f,  49.3593f,-27.7014f,9.8121f,  50.0278f,-28.6004f,9.2190f,  
	50.3003f,-29.9129f,10.7040f,  50.9737f,-30.8465f,10.0409f,  51.0210f,-32.3091f,11.4050f,  51.7018f,-33.2729f,10.6568f,  51.5239f,-34.8548f,11.8684f,  
	52.2132f,-35.8438f,11.0263f,  51.8137f,-37.5103f,12.0583f,  52.5132f,-38.5161f,11.1113f,  51.8991f,-40.2291f,11.9424f,  52.6106f,-41.2406f,10.8816f,  
	51.7921f,-42.9592f,11.4972f,  52.5177f,-43.9648f,10.3151f,  51.5082f,-45.6468f,10.7077f,  52.2467f,-46.6333f,9.4025f,  51.0618f,-48.2363f,9.5717f,  
	51.8137f,-49.1882f,8.1427f,  50.4713f,-50.6707f,8.0962f,  51.2334f,-51.5740f,6.5468f,  49.7533f,-52.8970f,6.2993f,  50.5239f,-53.7354f,4.6361f,  
	48.9275f,-54.8632f,4.2096f,  49.7016f,-55.6221f,2.4442f,  48.0116f,-56.5238f,1.8672f,  48.7832f,-57.1912f,0.0117f,  47.0232f,-57.8414f,-0.6823f,  
	47.7861f,-58.4053f,-2.6095f,  45.9806f,-58.7855f,-3.3826f,  46.7267f,-59.2375f,-5.3620f,  44.9009f,-59.3363f,-6.1738f,  45.6224f,-59.6710f,-8.1847f,  
	43.8021f,-59.4848f,-8.9934f,  44.4903f,-59.6997f,-11.0144f,  42.7018f,-59.2323f,-11.7792f,  43.3499f,-59.3287f,-13.7869f,  41.6196f,-58.5916f,-14.4690f,  
	42.2205f,-58.5737f,-16.4420f,  40.5752f,-57.5857f,-17.0066f,  41.1237f,-57.4608f,-18.9269f,  39.5897f,-56.2466f,-19.3446f,  40.0822f,-56.0243f,-21.1964f,  
	38.6854f,-54.6138f,-21.4447f,  39.1199f,-54.3046f,-23.2173f,  9.5042f,-19.4892f,27.7564f,  10.0969f,-20.5414f,27.3118f,  9.1548f,-20.3743f,25.5550f,  
	9.7473f,-21.4184f,25.0977f,  9.0057f,-21.0921f,23.2686f,  9.5981f,-22.1249f,22.7993f,  9.0697f,-21.6429f,20.9310f,  9.6628f,-22.6620f,20.4497f,  
	9.3565f,-22.0309f,18.5762f,  9.9509f,-23.0336f,18.0833f,  9.8721f,-22.2627f,16.2390f,  10.4685f,-23.2461f,15.7359f,  10.6185f,-22.3475f,13.9548f,  
	11.2177f,-23.3094f,13.4431f,  11.5940f,-22.2983f,11.7584f,  12.1965f,-23.2357f,11.2398f,  12.7927f,-22.1298f,9.6830f,  13.3995f,-23.0410f,9.1597f,  
	14.2054f,-21.8604f,7.7602f,  14.8168f,-22.7438f,7.2350f,  15.8187f,-21.5109f,6.0192f,  16.4358f,-22.3657f,5.4942f,  17.6164f,-21.1045f,4.4853f,  
	18.2383f,-21.9302f,3.9646f,  19.5771f,-20.6667f,3.1816f,  20.2048f,-21.4636f,2.6674f,  21.6786f,-20.2244f,2.1249f,  22.3126f,-20.9944f,1.6197f,  
	23.8958f,-19.8068f,1.3276f,  24.5350f,-20.5529f,0.8338f,  26.1999f,-19.4445f,0.7966f,  26.8442f,-20.1687f,0.3162f,  28.5619f,-19.1664f,0.5330f,  
	29.2113f,-19.8737f,0.0667f,  30.9516f,-19.0033f,0.5310f,  31.6051f,-19.6983f,0.0790f,  33.3377f,-18.9838f,0.7789f,  33.9944f,-19.6731f,0.3395f,  
	35.6889f,-19.1362f,1.2574f,  36.3482f,-19.8253f,0.8286f,  37.9751f,-19.4846f,1.9419f,  38.6361f,-20.1806f,1.5192f,  40.1671f,-20.0509f,2.8004f,  
	40.8291f,-20.7608f,2.3772f,  42.2377f,-20.8530f,3.7948f,  42.9010f,-21.5819f,3.3663f,  44.1630f,-21.9017f,4.8853f,  44.8269f,-22.6562f,4.4407f,  
	45.9207f,-23.2045f,6.0234f,  46.5855f,-23.9894f,5.5516f,  47.4930f,-24.7613f,7.1587f,  48.1594f,-25.5797f,6.6500f,  48.8655f,-26.5648f,8.2416f,  
	49.5345f,-27.4185f,7.6822f,  50.0278f,-28.6004f,9.2190f,  50.7008f,-29.4895f,8.5966f,  50.9737f,-30.8465f,10.0409f,  51.6530f,-31.7680f,9.3396f,  
	51.7018f,-33.2729f,10.6568f,  52.3887f,-34.2230f,9.8668f,  52.2132f,-35.8438f,11.0263f,  52.9097f,-36.8154f,10.1353f,  52.5132f,-38.5161f,11.1113f,  
	53.2211f,-39.4994f,10.1086f,  52.6106f,-41.2406f,10.8816f,  53.3324f,-42.2248f,9.7579f,  52.5177f,-43.9648f,10.3151f,  53.2528f,-44.9372f,9.0665f,  
	52.2467f,-46.6333f,9.4025f,  52.9959f,-47.5786f,8.0258f,  51.8137f,-49.1882f,8.1427f,  52.5744f,-50.0924f,6.6394f,  51.2334f,-51.5740f,6.5468f,  
	52.0044f,-52.4206f,4.9209f,  50.5239f,-53.7354f,4.6361f,  51.3009f,-54.5088f,2.8969f,  49.7016f,-55.6221f,2.4442f,  50.4790f,-56.3093f,0.6022f,  
	48.7832f,-57.1912f,0.0117f,  49.5550f,-57.7786f,-1.9164f,  47.7861f,-58.4053f,-2.6095f,  48.5443f,-58.8829f,-4.6053f,  46.7267f,-59.2375f,-5.3620f,  
	47.4635f,-59.5981f,-7.4055f,  45.6224f,-59.6710f,-8.1847f,  46.3288f,-59.9103f,-10.2542f,  44.4903f,-59.6997f,-11.0144f,  45.1590f,-59.8169f,-13.0863f,  
	43.3499f,-59.3287f,-13.7869f,  43.9730f,-59.3263f,-15.8383f,  42.2205f,-58.5737f,-16.4420f,  42.7922f,-58.4577f,-18.4530f,  41.1237f,-57.4608f,-18.9269f,  
	41.6393f,-57.2396f,-20.8794f,  40.0822f,-56.0243f,-21.1964f,  40.5387f,-55.7079f,-23.0768f,  39.1199f,-54.3046f,-23.2173f,  39.5164f,-53.9047f,-25.0148f,  
	10.0969f,-20.5414f,27.3118f,  10.6913f,-21.5966f,26.8524f,  9.7473f,-21.4184f,25.0977f,  10.3415f,-22.4638f,24.6254f,  9.5981f,-22.1249f,22.7993f,  
	10.1929f,-23.1580f,22.3138f,  9.6628f,-22.6620f,20.4497f,  10.2588f,-23.6802f,19.9513f,  9.9509f,-23.0336f,18.0833f,  10.5487f,-24.0336f,17.5731f,  
	10.4685f,-23.2461f,15.7359f,  11.0689f,-24.2253f,15.2154f,  11.2177f,-23.3094f,13.4431f,  11.8212f,-24.2646f,12.9139f,  12.1965f,-23.2357f,11.2398f,  
	12.8040f,-24.1648f,10.7040f,  13.3995f,-23.0410f,9.1597f,  14.0115f,-23.9418f,8.6200f,  14.8168f,-22.7438f,7.2350f,  15.4345f,-23.6151f,6.6935f,  
	16.4358f,-22.3657f,5.4942f,  17.0583f,-23.2062f,4.9553f,  18.2383f,-21.9302f,3.9646f,  18.8666f,-22.7399f,3.4306f,  20.2048f,-21.4636f,2.6674f,  
	20.8395f,-22.2435f,2.1410f,  22.3126f,-20.9944f,1.6197f,  22.9526f,-21.7467f,1.1041f,  24.5350f,-20.5529f,0.8338f,  25.1804f,-21.2793f,0.3315f,  
	26.8442f,-20.1687f,0.3162f,  27.4950f,-20.8736f,-0.1715f,  29.2113f,-19.8737f,0.0667f,  29.8665f,-20.5613f,-0.4054f,  31.6051f,-19.6983f,0.0790f,  
	32.2639f,-20.3748f,-0.3782f,  33.9944f,-19.6731f,0.3395f,  34.6560f,-20.3438f,-0.1040f,  36.3482f,-19.8253f,0.8286f,  37.0118f,-20.4973f,0.3953f,  
	38.6361f,-20.1806f,1.5192f,  39.3006f,-20.8611f,1.0905f,  40.8291f,-20.7608f,2.3772f,  41.4949f,-21.4553f,1.9489f,  42.9010f,-21.5819f,3.3663f,  
	43.5671f,-22.2976f,2.9285f,  44.8269f,-22.6562f,4.4407f,  45.4933f,-23.3989f,3.9830f,  46.5855f,-23.9894f,5.5516f,  47.2530f,-24.7629f,5.0647f,  
	48.1594f,-25.5797f,6.6500f,  48.8287f,-26.3872f,6.1200f,  49.5345f,-27.4185f,7.6822f,  50.2069f,-28.2611f,7.0968f,  50.7008f,-29.4895f,8.5966f,  
	51.3784f,-30.3660f,7.9392f,  51.6530f,-31.7680f,9.3396f,  52.3373f,-32.6757f,8.5998f,  52.3887f,-34.2230f,9.8668f,  53.0817f,-35.1561f,9.0313f,  
	52.9097f,-36.8154f,10.1353f,  53.6134f,-37.7653f,9.1915f,  53.2211f,-39.4994f,10.1086f,  53.9383f,-40.4563f,9.0454f,  53.3324f,-42.2248f,9.7579f,  
	54.0629f,-43.1767f,8.5696f,  53.2528f,-44.9372f,9.0665f,  53.9978f,-45.8695f,7.7476f,  52.9959f,-47.5786f,8.0258f,  53.7536f,-48.4776f,6.5755f,  
	52.5744f,-50.0924f,6.6394f,  53.3439f,-50.9411f,5.0591f,  52.0044f,-52.4206f,4.9209f,  52.7821f,-53.2029f,3.2176f,  51.3009f,-54.5088f,2.8969f,  
	52.0815f,-55.2109f,1.0793f,  50.4790f,-56.3093f,0.6022f,  51.2571f,-56.9162f,-1.3152f,  49.5550f,-57.7786f,-1.9164f,  50.3227f,-58.2787f,-3.9166f,  
	48.5443f,-58.8829f,-4.6053f,  49.2939f,-59.2673f,-6.6692f,  47.4635f,-59.5981f,-7.4055f,  48.1857f,-59.8605f,-9.5116f,  46.3288f,-59.9103f,-10.2542f,  
	47.0159f,-60.0479f,-12.3789f,  45.1590f,-59.8169f,-13.0863f,  45.8027f,-59.8305f,-15.2056f,  43.9730f,-59.3263f,-15.8383f,  44.5665f,-59.2201f,-17.9310f,  
	42.7922f,-58.4577f,-18.4530f,  43.3300f,-58.2391f,-20.4991f,  41.6393f,-57.2396f,-20.8794f,  42.1174f,-56.9177f,-22.8623f,  40.5387f,-55.7079f,-23.0768f,  
	40.9554f,-55.2941f,-24.9831f,  39.5164f,-53.9047f,-25.0148f,  39.8711f,-53.4103f,-26.8362f,  10.6913f,-21.5966f,26.8524f,  11.2873f,-22.6538f,26.3779f,  
	10.3415f,-22.4638f,24.6254f,  10.9379f,-23.5104f,24.1367f,  10.1929f,-23.1580f,22.3138f,  10.7903f,-24.1912f,21.8110f,  10.2588f,-23.6802f,19.9513f,  
	10.8578f,-24.6967f,19.4351f,  10.5487f,-24.0336f,17.5731f,  11.1501f,-25.0303f,17.0450f,  11.0689f,-24.2253f,15.2154f,  11.6732f,-25.1989f,14.6767f,  
	11.8212f,-24.2646f,12.9139f,  12.4293f,-25.2126f,12.3664f,  12.8040f,-24.1648f,10.7040f,  13.4165f,-25.0845f,10.1505f,  14.0115f,-23.9418f,8.6200f,  
	14.6294f,-24.8315f,8.0626f,  15.4345f,-23.6151f,6.6935f,  16.0572f,-24.4728f,6.1367f,  17.0583f,-23.2062f,4.9553f,  17.6868f,-24.0313f,4.4015f,  
	18.8666f,-22.7399f,3.4306f,  19.5016f,-23.5327f,2.8827f,  20.8395f,-22.2435f,2.1410f,  21.4800f,-23.0056f,1.6030f,  22.9526f,-21.7467f,1.1041f,  
	23.5988f,-22.4792f,0.5786f,  25.1804f,-21.2793f,0.3315f,  25.8322f,-21.9859f,-0.1795f,  27.4950f,-20.8736f,-0.1715f,  28.1516f,-21.5579f,-0.6661f,  
	29.8665f,-20.5613f,-0.4054f,  30.5269f,-21.2290f,-0.8835f,  32.2639f,-20.3748f,-0.3782f,  32.9274f,-21.0311f,-0.8401f,  34.6560f,-20.3438f,-0.1040f,  
	35.3217f,-20.9956f,-0.5521f,  37.0118f,-20.4973f,0.3953f,  37.6784f,-21.1520f,-0.0435f,  39.3006f,-20.8611f,1.0905f,  39.9687f,-21.5243f,0.6574f,  
	41.4949f,-21.4553f,1.9489f,  42.1631f,-22.1348f,1.5128f,  43.5671f,-22.2976f,2.9285f,  44.2352f,-22.9998f,2.4797f,  45.4933f,-23.3989f,3.9830f,  
	46.1619f,-24.1284f,3.5126f,  47.2530f,-24.7629f,5.0647f,  47.9227f,-25.5243f,4.5591f,  48.8287f,-26.3872f,6.1200f,  49.5006f,-27.1826f,5.5672f,  
	50.2069f,-28.2611f,7.0968f,  50.8827f,-29.0908f,6.4796f,  51.3784f,-30.3660f,7.9392f,  52.0599f,-31.2283f,7.2469f,  52.3373f,-32.6757f,8.5998f,  
	53.0264f,-33.5664f,7.8180f,  53.0817f,-35.1561f,9.0313f,  53.7807f,-36.0680f,8.1459f,  53.6134f,-37.7653f,9.1915f,  54.3250f,-38.6897f,8.1900f,  
	53.9383f,-40.4563f,9.0454f,  54.6630f,-41.3822f,7.9202f,  54.0629f,-43.1767f,8.5696f,  54.8022f,-44.0902f,7.3127f,  53.9978f,-45.8695f,7.7476f,  
	54.7505f,-46.7571f,6.3562f,  53.7536f,-48.4776f,6.5755f,  54.5194f,-49.3224f,5.0484f,  53.3439f,-50.9411f,5.0591f,  54.1198f,-51.7267f,3.4008f,  
	52.7821f,-53.2029f,3.2176f,  53.5633f,-53.9147f,1.4346f,  52.0815f,-55.2109f,1.0793f,  52.8631f,-55.8328f,-0.8162f,  51.2571f,-56.9162f,-1.3152f,  
	52.0314f,-57.4352f,-3.3076f,  50.3227f,-58.2787f,-3.9166f,  51.0823f,-58.6841f,-5.9883f,  49.2939f,-59.2673f,-6.6692f,  50.0293f,-59.5511f,-8.7993f,  
	48.1857f,-59.8605f,-9.5116f,  48.8890f,-60.0178f,-11.6770f,  47.0159f,-60.0479f,-12.3789f,  47.6782f,-60.0776f,-14.5550f,  45.8027f,-59.8305f,-15.2056f,  
	46.4164f,-59.7351f,-17.3699f,  44.5665f,-59.2201f,-17.9310f,  45.1254f,-59.0055f,-20.0619f,  43.3300f,-58.2391f,-20.4991f,  43.8290f,-57.9134f,-22.5774f,  
	42.1174f,-56.9177f,-22.8623f,  42.5540f,-56.4917f,-24.8723f,  40.9554f,-55.2941f,-24.9831f,  41.3280f,-54.7791f,-26.9132f,  39.8711f,-53.4103f,-26.8362f,  
	40.1802f,-52.8181f,-28.6792f,  11.2873f,-22.6538f,26.3779f,  11.8849f,-23.7131f,25.8871f,  10.9379f,-23.5104f,24.1367f,  11.5365f,-24.5580f,23.6306f,  
	10.7903f,-24.1912f,21.8110f,  11.3904f,-25.2238f,21.2900f,  10.8578f,-24.6967f,19.4351f,  11.4600f,-25.7108f,18.9006f,  11.1501f,-25.0303f,17.0450f,  
	11.7550f,-26.0227f,16.4979f,  11.6732f,-25.1989f,14.6767f,  12.2818f,-26.1665f,14.1187f,  12.4293f,-25.2126f,12.3664f,  13.0420f,-26.1523f,11.8003f,  
	13.4165f,-25.0845f,10.1505f,  14.0345f,-25.9941f,9.5780f,  14.6294f,-24.8315f,8.0626f,  15.2522f,-25.7084f,7.4885f,  16.0572f,-24.4728f,6.1367f,  
	16.6858f,-25.3158f,5.5634f,  17.6868f,-24.0313f,4.4015f,  18.3220f,-24.8400f,3.8321f,  19.5016f,-23.5327f,2.8827f,  20.1424f,-24.3080f,2.3218f,  
	21.4800f,-23.0056f,1.6030f,  22.1266f,-23.7478f,1.0533f,  23.5988f,-22.4792f,0.5786f,  24.2513f,-23.1914f,0.0429f,  25.8322f,-21.9859f,-0.1795f,  
	26.4897f,-22.6711f,-0.6986f,  28.1516f,-21.5579f,-0.6661f,  28.8133f,-22.2214f,-1.1677f,  30.5269f,-21.2290f,-0.8835f,  31.1920f,-21.8755f,-1.3669f,  
	32.9274f,-21.0311f,-0.8401f,  33.5949f,-21.6671f,-1.3067f,  35.3217f,-20.9956f,-0.5521f,  35.9903f,-21.6286f,-1.0054f,  37.6784f,-21.1520f,-0.0435f,  
	38.3486f,-21.7876f,-0.4862f,  39.9687f,-21.5243f,0.6574f,  40.6387f,-22.1707f,0.2177f,  42.1631f,-22.1348f,1.5128f,  42.8326f,-22.7990f,1.0674f,  
	44.2352f,-22.9998f,2.4797f,  44.9047f,-23.6871f,2.0203f,  46.1619f,-24.1284f,3.5126f,  46.8317f,-24.8445f,3.0261f,  47.9227f,-25.5243f,4.5591f,  
	48.5938f,-26.2722f,4.0338f,  49.5006f,-27.1826f,5.5672f,  50.1744f,-27.9643f,4.9859f,  50.8827f,-29.0908f,6.4796f,  51.5611f,-29.9057f,5.8313f,  
	52.0599f,-31.2283f,7.2469f,  52.7447f,-32.0737f,6.5163f,  53.0264f,-33.5664f,7.8180f,  53.7201f,-34.4365f,6.9898f,  53.7807f,-36.0680f,8.1459f,  
	54.4860f,-36.9553f,7.2062f,  54.3250f,-38.6897f,8.1900f,  55.0429f,-39.5843f,7.1295f,  54.6630f,-41.3822f,7.9202f,  55.3953f,-42.2711f,6.7286f,  
	54.8022f,-44.0902f,7.3127f,  55.5484f,-44.9604f,5.9853f,  54.7505f,-46.7571f,6.3562f,  55.5108f,-47.5921f,4.8890f,  54.5194f,-49.3224f,5.0484f,  
	55.2912f,-50.1056f,3.4433f,  54.1198f,-51.7267f,3.4008f,  54.8991f,-52.4430f,1.6622f,  53.5633f,-53.9147f,1.4346f,  54.3457f,-54.5470f,-0.4278f,  
	52.8631f,-55.8328f,-0.8162f,  53.6413f,-56.3669f,-2.7888f,  52.0314f,-57.4352f,-3.3076f,  52.7982f,-57.8587f,-5.3745f,  51.0823f,-58.6841f,-5.9883f,  
	51.8281f,-58.9874f,-8.1295f,  50.0293f,-59.5511f,-8.7993f,  50.7462f,-59.7271f,-10.9925f,  48.8890f,-60.0178f,-11.6770f,  49.5675f,-60.0636f,-13.8974f,  
	47.6782f,-60.0776f,-14.5550f,  48.3102f,-59.9938f,-16.7793f,  46.4164f,-59.7351f,-17.3699f,  46.9948f,-59.5258f,-19.5754f,  45.1254f,-59.0055f,-20.0619f,  
	45.6442f,-58.6779f,-22.2270f,  43.8290f,-57.9134f,-22.5774f,  44.2848f,-57.4772f,-24.6842f,  42.5540f,-56.4917f,-24.8723f,  42.9443f,-55.9579f,-26.9064f,  
	41.3280f,-54.7791f,-26.9132f,  41.6525f,-54.1598f,-28.8641f,  40.1802f,-52.8181f,-28.6792f,  40.4398f,-52.1241f,-30.5425f,  11.8849f,-23.7131f,25.8871f,  
	12.4845f,-24.7743f,25.3788f,  11.5365f,-24.5580f,23.6306f,  12.1374f,-25.6060f,23.1060f,  11.3904f,-25.2238f,21.2900f,  11.9932f,-26.2550f,20.7503f,  
	11.4600f,-25.7108f,18.9006f,  12.0654f,-26.7218f,18.3464f,  11.7550f,-26.0227f,16.4979f,  12.3639f,-27.0101f,15.9308f,  12.2818f,-26.1665f,14.1187f,  
	12.8945f,-27.1268f,13.5412f,  13.0420f,-26.1523f,11.8003f,  13.6599f,-27.0829f,11.2140f,  14.0345f,-25.9941f,9.5780f,  14.6571f,-26.8918f,8.9877f,  
	15.2522f,-25.7084f,7.4885f,  15.8805f,-26.5713f,6.8965f,  16.6858f,-25.3158f,5.5634f,  17.3206f,-26.1430f,4.9730f,  18.3220f,-24.8400f,3.8321f,  
	18.9625f,-25.6315f,3.2482f,  20.1424f,-24.3080f,2.3218f,  20.7889f,-25.0634f,1.7475f,  22.1266f,-23.7478f,1.0533f,  22.7793f,-24.4696f,0.4919f,  
	24.2513f,-23.1914f,0.0429f,  24.9093f,-23.8818f,-0.5024f,  26.4897f,-22.6711f,-0.6986f,  27.1522f,-23.3347f,-1.2259f,  28.8133f,-22.2214f,-1.1677f,  
	29.4795f,-22.8624f,-1.6755f,  31.1920f,-21.8755f,-1.3669f,  31.8609f,-22.5002f,-1.8556f,  33.5949f,-21.6671f,-1.3067f,  34.2650f,-22.2827f,-1.7785f,  
	35.9903f,-21.6286f,-1.0054f,  36.6622f,-22.2407f,-1.4623f,  38.3486f,-21.7876f,-0.4862f,  39.0202f,-22.4046f,-0.9345f,  40.6387f,-22.1707f,0.2177f,  
	41.3095f,-22.8000f,-0.2296f,  42.8326f,-22.7990f,1.0674f,  43.5029f,-23.4465f,0.6135f,  44.9047f,-23.6871f,2.0203f,  45.5747f,-24.3594f,1.5474f,  
	46.8317f,-24.8445f,3.0261f,  47.5021f,-25.5457f,2.5231f,  48.5938f,-26.2722f,4.0338f,  49.2656f,-27.0057f,3.4835f,  50.1744f,-27.9643f,4.9859f,  
	50.8495f,-28.7305f,4.3771f,  51.5611f,-29.9057f,5.8313f,  52.2415f,-30.7034f,5.1483f,  52.7447f,-32.0737f,6.5163f,  53.4327f,-32.8987f,5.7429f,  
	53.7201f,-34.4365f,6.9898f,  54.4185f,-35.2826f,6.1109f,  54.4860f,-36.9553f,7.2062f,  55.1961f,-37.8138f,6.2109f,  55.0429f,-39.5843f,7.1295f,  
	55.7670f,-40.4433f,6.0053f,  55.3953f,-42.2711f,6.7286f,  56.1333f,-43.1183f,5.4690f,  55.5484f,-44.9604f,5.9853f,  56.3010f,-45.7799f,4.5835f,  
	55.5108f,-47.5921f,4.8890f,  56.2762f,-48.3672f,3.3444f,  55.2912f,-50.1056f,3.4433f,  56.0660f,-50.8209f,1.7581f,  54.8991f,-52.4430f,1.6622f,  
	55.6794f,-53.0808f,-0.1568f,  54.3457f,-54.5470f,-0.4278f,  55.1246f,-55.0919f,-2.3691f,  53.6413f,-56.3669f,-2.7888f,  54.4121f,-56.8052f,-4.8384f,  
	52.7982f,-57.8587f,-5.3745f,  53.5514f,-58.1791f,-7.5137f,  51.8281f,-58.9874f,-8.1295f,  52.5557f,-59.1808f,-10.3372f,  50.7462f,-59.7271f,-10.9925f,  
	51.4383f,-59.7885f,-13.2441f,  49.5675f,-60.0636f,-13.8974f,  50.2154f,-59.9918f,-16.1693f,  48.3102f,-59.9938f,-16.7793f,  48.9062f,-59.7912f,-19.0478f,  
	46.9948f,-59.5258f,-19.5754f,  47.5320f,-59.1981f,-21.8171f,  45.6442f,-58.6779f,-22.2270f,  46.1182f,-58.2335f,-24.4222f,  44.2848f,-57.4772f,-24.6842f,  
	44.6922f,-56.9268f,-26.8157f,  42.9443f,-55.9579f,-26.9064f,  43.2840f,-55.3136f,-28.9605f,  41.6525f,-54.1598f,-28.8641f,  41.9247f,-53.4326f,-30.8339f,  
	40.4398f,-52.1241f,-30.5425f,  40.6460f,-51.3250f,-32.4238f,  12.4845f,-24.7743f,25.3788f,  13.0860f,-25.8369f,24.8521f,  12.1374f,-25.6060f,23.1060f,  
	12.7406f,-26.6537f,22.5627f,  11.9932f,-26.2550f,20.7503f,  12.5988f,-27.2843f,20.1906f,  12.0654f,-26.7218f,18.3464f,  12.6742f,-27.7291f,17.7717f,  
	12.3639f,-27.0101f,15.9308f,  12.9764f,-27.9915f,15.3433f,  12.8945f,-27.1268f,13.5412f,  13.5120f,-28.0794f,12.9423f,  13.6599f,-27.0829f,11.2140f,  
	14.2820f,-28.0027f,10.6086f,  14.6571f,-26.8918f,8.9877f,  15.2849f,-27.7765f,8.3780f,  15.8805f,-26.5713f,6.8965f,  16.5148f,-27.4195f,6.2858f,  
	17.3206f,-26.1430f,4.9730f,  17.9606f,-26.9536f,4.3665f,  18.9625f,-25.6315f,3.2482f,  19.6087f,-26.4036f,2.6493f,  20.7889f,-25.0634f,1.7475f,  
	21.4414f,-25.7986f,1.1597f,  22.7793f,-24.4696f,0.4919f,  23.4374f,-25.1694f,-0.0808f,  24.9093f,-23.8818f,-0.5024f,  25.5722f,-24.5501f,-1.0572f,  
	27.1522f,-23.3347f,-1.2259f,  27.8192f,-23.9751f,-1.7607f,  29.4795f,-22.8624f,-1.6755f,  30.1494f,-23.4807f,-2.1895f,  31.8609f,-22.5002f,-1.8556f,  
	32.5323f,-23.1032f,-2.3498f,  34.2650f,-22.2827f,-1.7785f,  34.9383f,-22.8759f,-2.2542f,  36.6622f,-22.2407f,-1.4623f,  37.3351f,-22.8325f,-1.9243f,  
	39.0202f,-22.4046f,-0.9345f,  39.6920f,-23.0026f,-1.3893f,  41.3095f,-22.8000f,-0.2296f,  41.9805f,-23.4107f,-0.6839f,  43.5029f,-23.4465f,0.6135f,  
	44.1729f,-24.0774f,0.1483f,  45.5747f,-24.3594f,1.5474f,  46.2441f,-25.0154f,1.0607f,  47.5021f,-25.5457f,2.5231f,  48.1718f,-26.2314f,1.9981f,  
	49.2656f,-27.0057f,3.4835f,  49.9374f,-27.7227f,2.9093f,  50.8495f,-28.7305f,4.3771f,  51.5252f,-29.4790f,3.7376f,  52.2415f,-30.7034f,5.1483f,  
	52.9234f,-31.4810f,4.4266f,  53.4327f,-32.8987f,5.7429f,  54.1235f,-33.7003f,4.9228f,  54.4185f,-35.2826f,6.1109f,  55.1201f,-36.1009f,5.1802f,  
	55.1961f,-37.8138f,6.2109f,  55.9109f,-38.6381f,5.1552f,  55.7670f,-40.4433f,6.0053f,  56.4955f,-41.2623f,4.8163f,  56.1333f,-43.1183f,5.4690f,  
	56.8766f,-43.9166f,4.1373f,  56.3010f,-45.7799f,4.5835f,  57.0580f,-46.5412f,3.1058f,  56.2762f,-48.3672f,3.3444f,  57.0441f,-49.0761f,1.7207f,  
	56.0660f,-50.8209f,1.7581f,  56.8415f,-51.4592f,-0.0078f,  55.6794f,-53.0808f,-0.1568f,  56.4562f,-53.6322f,-2.0556f,  55.1246f,-55.0919f,-2.3691f,  
	55.8964f,-55.5415f,-4.3890f,  54.4121f,-56.8052f,-4.8384f,  55.1695f,-57.1400f,-6.9624f,  53.5514f,-58.1791f,-7.5137f,  54.2865f,-58.3882f,-9.7223f,  
	52.5557f,-59.1808f,-10.3372f,  53.2585f,-59.2572f,-12.6063f,  51.4383f,-59.7885f,-13.2441f,  52.0996f,-59.7291f,-15.5503f,  50.2154f,-59.9918f,-16.1693f,  
	50.8269f,-59.7969f,-18.4883f,  48.9062f,-59.7912f,-19.0478f,  49.4599f,-59.4650f,-21.3548f,  47.5320f,-59.1981f,-21.8171f,  48.0228f,-58.7479f,-24.0906f,  
	46.1182f,-58.2335f,-24.4222f,  46.5419f,-57.6690f,-26.6426f,  44.6922f,-56.9268f,-26.8157f,  45.0466f,-56.2597f,-28.9666f,  43.2840f,-55.3136f,-28.9605f,  
	43.5686f,-54.5553f,-31.0323f,  41.9247f,-53.4326f,-30.8339f,  42.1404f,-52.5948f,-32.8192f,  40.6460f,-51.3250f,-32.4238f,  40.7946f,-50.4177f,-34.3203f,  
	13.0860f,-25.8369f,24.8521f,  13.6895f,-26.8999f,24.3065f,  12.7406f,-26.6537f,22.5627f,  13.3463f,-27.7006f,21.9990f,  12.5988f,-27.2843f,20.1906f,  
	13.2075f,-28.3111f,19.6099f,  12.6742f,-27.7291f,17.7717f,  13.2864f,-28.7315f,17.1759f,  12.9764f,-27.9915f,15.3433f,  13.5933f,-28.9664f,14.7336f,  
	13.5120f,-28.0794f,12.9423f,  14.1334f,-29.0222f,12.3235f,  14.2820f,-28.0027f,10.6086f,  14.9090f,-28.9107f,9.9825f,  15.2849f,-27.7765f,8.3780f,  
	15.9182f,-28.6475f,7.7480f,  16.5148f,-27.4195f,6.2858f,  17.1539f,-28.2516f,5.6574f,  17.9606f,-26.9536f,4.3665f,  18.6060f,-27.7453f,3.7432f,  
	19.6087f,-26.4036f,2.6493f,  20.2606f,-27.1556f,2.0350f,  21.4414f,-25.7986f,1.1597f,  22.0992f,-26.5118f,0.5590f,  23.4374f,-25.1694f,-0.0808f,  
	24.1002f,-25.8467f,-0.6644f,  25.5722f,-24.5501f,-1.0572f,  26.2394f,-25.1945f,-1.6209f,  27.8192f,-23.9751f,-1.7607f,  28.4897f,-24.5918f,-2.3026f,  
	30.1494f,-23.4807f,-2.1895f,  30.8216f,-24.0759f,-2.7095f,  32.5323f,-23.1032f,-2.3498f,  33.2066f,-23.6823f,-2.8484f,  34.9383f,-22.8759f,-2.2542f,  
	35.6124f,-23.4470f,-2.7346f,  37.3351f,-22.8325f,-1.9243f,  38.0079f,-23.4036f,-2.3917f,  39.6920f,-23.0026f,-1.3893f,  40.3636f,-23.5803f,-1.8496f,  
	41.9805f,-23.4107f,-0.6839f,  42.6505f,-24.0032f,-1.1474f,  44.1729f,-24.0774f,0.1483f,  44.8415f,-24.6903f,-0.3282f,  46.2441f,-25.0154f,1.0607f,  
	46.9117f,-25.6547f,0.5553f,  48.1718f,-26.2314f,1.9981f,  48.8402f,-26.8997f,1.4530f,  49.9374f,-27.7227f,2.9093f,  50.6083f,-28.4215f,2.3083f,  
	51.5252f,-29.4790f,3.7376f,  52.2008f,-30.2075f,3.0634f,  52.9234f,-31.4810f,4.4266f,  53.6063f,-32.2355f,3.6623f,  54.1235f,-33.7003f,4.9228f,  
	54.8160f,-34.4750f,4.0551f,  55.1201f,-36.1009f,5.1802f,  55.8246f,-36.8864f,4.1929f,  55.9109f,-38.6381f,5.1552f,  56.6286f,-39.4241f,4.0383f,  
	56.4955f,-41.2623f,4.8163f,  57.2278f,-42.0343f,3.5579f,  56.8766f,-43.9166f,4.1373f,  57.6231f,-44.6590f,2.7317f,  57.0580f,-46.5412f,3.1058f,  
	57.8165f,-47.2386f,1.5506f,  57.0441f,-49.0761f,1.7207f,  57.8120f,-49.7100f,0.0168f,  56.8415f,-51.4592f,-0.0078f,  57.6132f,-52.0127f,-1.8537f,  
	56.4562f,-53.6322f,-2.0556f,  57.2257f,-54.0892f,-4.0341f,  55.8964f,-55.5415f,-4.3890f,  56.6549f,-55.8878f,-6.4850f,  55.1695f,-57.1400f,-6.9624f,  
	55.9090f,-57.3628f,-9.1582f,  54.2865f,-58.3882f,-9.7223f,  54.9969f,-58.4787f,-11.9950f,  53.2585f,-59.2572f,-12.6063f,  53.9303f,-59.2102f,-14.9330f,  
	52.0996f,-59.7291f,-15.5503f,  52.7240f,-59.5430f,-17.9063f,  50.8269f,-59.7969f,-18.4883f,  51.3952f,-59.4743f,-20.8479f,  49.4599f,-59.4650f,-21.3548f,  
	49.9661f,-59.0114f,-23.6953f,  48.0228f,-58.7479f,-24.0906f,  48.4616f,-58.1720f,-26.3903f,  46.5419f,-57.6690f,-26.6426f,  46.9103f,-56.9820f,-28.8824f,  
	45.0466f,-56.2597f,-28.9666f,  45.3433f,-55.4729f,-31.1343f,  43.5686f,-54.5553f,-31.0323f,  43.7938f,-53.6808f,-33.1175f,  42.1404f,-52.5948f,-32.8192f,  
	42.2955f,-51.6437f,-34.8167f,  40.7946f,-50.4177f,-34.3203f,  40.8817f,-49.4001f,-36.2279f,  13.6895f,-26.8999f,24.3065f,  14.2950f,-27.9634f,23.7405f,  
	13.3463f,-27.7006f,21.9990f,  13.9546f,-28.7461f,21.4139f,  13.2075f,-28.3111f,19.6099f,  13.8192f,-29.3343f,19.0076f,  13.2864f,-28.7315f,17.1759f,  
	13.9025f,-29.7288f,16.5572f,  13.5933f,-28.9664f,14.7336f,  14.2138f,-29.9327f,14.1029f,  14.1334f,-29.0222f,12.3235f,  14.7593f,-29.9545f,11.6826f,  
	14.9090f,-28.9107f,9.9825f,  15.5411f,-29.8061f,9.3347f,  15.9182f,-28.6475f,7.7480f,  16.5561f,-29.5033f,7.0989f,  17.1539f,-28.2516f,5.6574f,  
	17.7982f,-29.0656f,5.0105f,  18.6060f,-27.7453f,3.7432f,  19.2569f,-28.5175f,3.1028f,  20.2606f,-27.1556f,2.0350f,  20.9176f,-27.8860f,1.4060f,  
	22.0992f,-26.5118f,0.5590f,  22.7615f,-27.2024f,-0.0543f,  24.1002f,-25.8467f,-0.6644f,  24.7673f,-26.4998f,-1.2585f,  26.2394f,-25.1945f,-1.6209f,  
	26.9101f,-25.8145f,-2.1930f,  28.4897f,-24.5918f,-2.3026f,  29.1622f,-25.1844f,-2.8513f,  30.8216f,-24.0759f,-2.7095f,  31.4966f,-24.6460f,-3.2346f,  
	33.2066f,-23.6823f,-2.8484f,  33.8814f,-24.2378f,-3.3515f,  35.6124f,-23.4470f,-2.7346f,  36.2858f,-23.9959f,-3.2198f,  38.0079f,-23.4036f,-2.3917f,  
	38.6798f,-23.9525f,-2.8637f,  40.3636f,-23.5803f,-1.8496f,  41.0334f,-24.1382f,-2.3175f,  42.6505f,-24.0032f,-1.1474f,  43.3182f,-24.5761f,-1.6199f,  
	44.8415f,-24.6903f,-0.3282f,  45.5070f,-25.2855f,-0.8204f,  46.9117f,-25.6547f,0.5553f,  47.5768f,-26.2754f,0.0330f,  48.8402f,-26.8997f,1.4530f,  
	49.5063f,-27.5490f,0.8848f,  50.6083f,-28.4215f,2.3083f,  51.2774f,-29.1002f,1.6766f,  52.2008f,-30.2075f,3.0634f,  52.8756f,-30.9133f,2.3509f,  
	53.6063f,-32.2355f,3.6623f,  54.2891f,-32.9638f,2.8547f,  54.8160f,-34.4750f,4.0551f,  55.5095f,-35.2181f,3.1347f,  55.8246f,-36.8864f,4.1929f,  
	56.5304f,-37.6351f,3.1483f,  56.6286f,-39.4241f,4.0383f,  57.3485f,-40.1651f,2.8553f,  57.2278f,-42.0343f,3.5579f,  57.9620f,-42.7528f,2.2283f,  
	57.6231f,-44.6590f,2.7317f,  58.3702f,-45.3397f,1.2507f,  57.8165f,-47.2386f,1.5506f,  58.5743f,-47.8632f,-0.0837f,  57.8120f,-49.7100f,0.0168f,  
	58.5757f,-50.2611f,-1.7668f,  57.6132f,-52.0127f,-1.8537f,  58.3774f,-52.4733f,-3.7798f,  57.2257f,-54.0892f,-4.0341f,  57.9820f,-54.4440f,-6.0898f,  
	56.6549f,-55.8878f,-6.4850f,  57.3955f,-56.1223f,-8.6547f,  55.9090f,-57.3628f,-9.1582f,  56.6240f,-57.4663f,-11.4205f,  54.9969f,-58.4787f,-11.9950f,  
	55.6764f,-58.4441f,-14.3278f,  53.9303f,-59.2102f,-14.9330f,  54.5651f,-59.0339f,-17.3119f,  52.7240f,-59.5430f,-17.9063f,  53.3045f,-59.2256f,-20.3051f,  
	51.3952f,-59.4743f,-20.8479f,  51.9148f,-59.0196f,-23.2429f,  49.9661f,-59.0114f,-23.6953f,  50.4186f,-58.4270f,-26.0629f,  48.4616f,-58.1720f,-26.3903f,  
	48.8431f,-57.4681f,-28.7093f,  46.9103f,-56.9820f,-28.8824f,  47.2186f,-56.1698f,-31.1381f,  45.3433f,-55.4729f,-31.1343f,  45.5779f,-54.5645f,-33.3134f,  
	43.7938f,-53.6808f,-33.1175f,  43.9553f,-52.6881f,-35.2120f,  42.2955f,-51.6437f,-34.8167f,  42.3859f,-50.5777f,-36.8218f,  40.8817f,-49.4001f,-36.2279f,  
	40.9032f,-48.2691f,-38.1438f,  14.2950f,-27.9634f,23.7405f,  14.9028f,-29.0268f,23.1530f,  13.9546f,-28.7461f,21.4139f,  14.5656f,-29.7893f,20.8070f,  
	13.8192f,-29.3343f,19.0076f,  14.4342f,-30.3536f,18.3817f,  13.9025f,-29.7288f,16.5572f,  14.5218f,-30.7188f,15.9166f,  14.2138f,-29.9327f,14.1029f,  
	14.8383f,-30.8900f,13.4491f,  14.7593f,-29.9545f,11.6826f,  15.3899f,-30.8754f,11.0187f,  15.5411f,-29.8061f,9.3347f,  16.1775f,-30.6873f,8.6664f,  
	16.5561f,-29.5033f,7.0989f,  17.1989f,-30.3420f,6.4296f,  17.7982f,-29.0656f,5.0105f,  18.4477f,-29.8610f,4.3448f,  19.2569f,-28.5175f,3.1028f,  
	19.9128f,-29.2685f,2.4458f,  20.9176f,-27.8860f,1.4060f,  21.5790f,-28.5938f,0.7627f,  22.7615f,-27.2024f,-0.0543f,  23.4279f,-27.8685f,-0.6798f,  
	24.7673f,-26.4998f,-1.2585f,  25.4376f,-27.1278f,-1.8626f,  26.9101f,-25.8145f,-2.1930f,  27.5826f,-26.4095f,-2.7730f,  29.1622f,-25.1844f,-2.8513f,  
	29.8376f,-25.7507f,-3.4063f,  31.4966f,-24.6460f,-3.2346f,  32.1719f,-25.1911f,-3.7646f,  33.8814f,-24.2378f,-3.3515f,  34.5552f,-24.7694f,-3.8593f,  
	36.2858f,-23.9959f,-3.2198f,  36.9579f,-24.5208f,-3.7088f,  38.6798f,-23.9525f,-2.8637f,  39.3494f,-24.4800f,-3.3419f,  41.0334f,-24.1382f,-2.3175f,  
	41.7002f,-24.6748f,-2.7924f,  43.3182f,-24.5761f,-1.6199f,  43.9816f,-25.1300f,-2.1054f,  45.5070f,-25.2855f,-0.8204f,  46.1688f,-25.8607f,-1.3262f,  
	47.5768f,-26.2754f,0.0330f,  48.2381f,-26.8763f,-0.5086f,  49.5063f,-27.5490f,0.8848f,  50.1690f,-28.1778f,0.2899f,  51.2774f,-29.1002f,1.6766f,  
	51.9439f,-29.7562f,1.0111f,  52.8756f,-30.9133f,2.3509f,  53.5484f,-31.5933f,1.5996f,  54.2891f,-32.9638f,2.8547f,  54.9710f,-33.6619f,1.9989f,  
	55.5095f,-35.2181f,3.1347f,  56.2025f,-35.9261f,2.1615f,  56.5304f,-37.6351f,3.1483f,  57.2367f,-38.3410f,2.0415f,  57.3485f,-40.1651f,2.8553f,  
	58.0688f,-40.8550f,1.6043f,  57.9620f,-42.7528f,2.2283f,  58.6955f,-43.4120f,0.8263f,  58.3702f,-45.3397f,1.2507f,  59.1155f,-45.9503f,-0.3074f,  
	58.5743f,-47.8632f,-0.0837f,  59.3274f,-48.4074f,-1.7965f,  58.5757f,-50.2611f,-1.7668f,  59.3316f,-50.7214f,-3.6303f,  58.3774f,-52.4733f,-3.7798f,  
	59.1283f,-52.8333f,-5.7833f,  57.9820f,-54.4440f,-6.0898f,  58.7205f,-54.6878f,-8.2204f,  57.3955f,-56.1223f,-8.6547f,  58.1117f,-56.2374f,-10.8928f,  
	56.6240f,-57.4663f,-11.4205f,  57.3080f,-57.4437f,-13.7451f,  55.6764f,-58.4441f,-14.3278f,  56.3186f,-58.2781f,-16.7152f,  54.5651f,-59.0339f,-17.3119f,  
	55.1553f,-58.7232f,-19.7357f,  53.3045f,-59.2256f,-20.3051f,  53.8355f,-58.7723f,-22.7411f,  51.9148f,-59.0196f,-23.2429f,  52.3794f,-58.4297f,-25.6661f,  
	50.4186f,-58.4270f,-26.0629f,  50.8120f,-57.7097f,-28.4502f,  48.8431f,-57.4681f,-28.7093f,  49.1622f,-56.6339f,-31.0437f,  47.2186f,-56.1698f,-31.1381f,  
	47.4620f,-55.2307f,-33.4038f,  45.5779f,-54.5645f,-33.3134f,  45.7458f,-53.5330f,-35.4993f,  43.9553f,-52.6881f,-35.2120f,  44.0492f,-51.5759f,-37.3108f,  
	42.3859f,-50.5777f,-36.8218f,  42.4076f,-49.3943f,-38.8314f,  40.9032f,-48.2691f,-38.1438f,  40.8553f,-47.0235f,-40.0633f,  14.9028f,-29.0268f,23.1530f,  
	15.5128f,-30.0890f,22.5433f,  14.5656f,-29.7893f,20.8070f,  15.1794f,-30.8300f,20.1761f,  14.4342f,-30.3536f,18.3817f,  15.0522f,-31.3670f,17.7333f,  
	14.5218f,-30.7188f,15.9166f,  15.1448f,-31.7012f,15.2519f,  14.8383f,-30.8900f,13.4491f,  15.4672f,-31.8372f,12.7711f,  15.3899f,-30.8754f,11.0187f,  
	16.0246f,-31.7833f,10.3329f,  16.1775f,-30.6873f,8.6664f,  16.8186f,-31.5526f,7.9762f,  17.1989f,-30.3420f,6.4296f,  17.8467f,-31.1629f,5.7397f,  
	18.4477f,-29.8610f,4.3448f,  19.1019f,-30.6357f,3.6605f,  19.9128f,-29.2685f,2.4458f,  20.5728f,-29.9973f,1.7726f,  21.5790f,-28.5938f,0.7627f,  
	22.2443f,-29.2773f,0.1053f,  23.4279f,-27.8685f,-0.6798f,  24.0976f,-28.5094f,-1.3169f,  25.4376f,-27.1278f,-1.8626f,  26.1098f,-27.7303f,-2.4757f,  
	27.5826f,-26.4095f,-2.7730f,  28.2579f,-26.9774f,-3.3605f,  29.8376f,-25.7507f,-3.4063f,  30.5130f,-26.2909f,-3.9668f,  32.1719f,-25.1911f,-3.7646f,  
	32.8459f,-25.7110f,-4.2993f,  34.5552f,-24.7694f,-3.8593f,  35.2275f,-25.2756f,-4.3706f,  36.9579f,-24.5208f,-3.7088f,  37.6273f,-25.0228f,-4.2031f,  
	39.3494f,-24.4800f,-3.3419f,  40.0153f,-24.9845f,-3.8254f,  41.7002f,-24.6748f,-2.7924f,  42.3617f,-25.1909f,-3.2776f,  43.9816f,-25.1300f,-2.1054f,  
	44.6402f,-25.6625f,-2.6015f,  46.1688f,-25.8607f,-1.3262f,  46.8255f,-26.4151f,-1.8479f,  48.2381f,-26.8763f,-0.5086f,  48.8946f,-27.4563f,-1.0729f,  
	50.1690f,-28.1778f,0.2899f,  50.8272f,-28.7840f,-0.3343f,  51.9439f,-29.7562f,1.0111f,  52.6066f,-30.3869f,0.3113f,  53.5484f,-31.5933f,1.5996f,  
	54.2184f,-32.2442f,0.8047f,  54.9710f,-33.6619f,1.9989f,  55.6506f,-34.3261f,1.0948f,  56.2025f,-35.9261f,2.1615f,  56.8942f,-36.5933f,1.1304f,  
	57.2367f,-38.3410f,2.0415f,  57.9417f,-38.9983f,0.8705f,  58.0688f,-40.8550f,1.6043f,  58.7871f,-41.4885f,0.2844f,  58.6955f,-43.4120f,0.8263f,  
	59.4262f,-44.0040f,-0.6504f,  59.1155f,-45.9503f,-0.3074f,  59.8553f,-46.4833f,-1.9422f,  59.3274f,-48.4074f,-1.7965f,  60.0721f,-48.8635f,-3.5883f,  
	59.3316f,-50.7214f,-3.6303f,  60.0740f,-51.0833f,-5.5710f,  59.1283f,-52.8333f,-5.7833f,  59.8614f,-53.0839f,-7.8627f,  58.7205f,-54.6878f,-8.2204f,  
	59.4347f,-54.8130f,-10.4208f,  58.1117f,-56.2374f,-10.8928f,  58.7970f,-56.2263f,-13.1949f,  57.3080f,-57.4437f,-13.7451f,  57.9546f,-57.2886f,-16.1264f,  
	56.3186f,-58.2781f,-16.7152f,  56.9160f,-57.9758f,-19.1494f,  55.1553f,-58.7232f,-19.7357f,  55.6954f,-58.2738f,-22.1986f,  53.8355f,-58.7723f,-22.7411f,  
	54.3104f,-58.1801f,-25.2067f,  52.3794f,-58.4297f,-25.6661f,  52.7833f,-57.7027f,-28.1096f,  50.8120f,-57.7097f,-28.4502f,  51.1410f,-56.8574f,-30.8527f,  
	49.1622f,-56.6339f,-31.0437f,  49.4141f,-55.6681f,-33.3869f,  47.4620f,-55.2307f,-33.4038f,  47.6361f,-54.1640f,-35.6739f,  45.7458f,-53.5330f,-35.4993f,  
	45.8431f,-52.3776f,-37.6867f,  44.0492f,-51.5759f,-37.3108f,  44.0713f,-50.3424f,-39.4106f,  42.4076f,-49.3943f,-38.8314f,  42.3569f,-48.0930f,-40.8401f,  
	40.8553f,-47.0235f,-40.0633f,  40.7341f,-45.6613f,-41.9825f,  15.5128f,-30.0890f,22.5433f,  16.1253f,-31.1500f,21.9096f,  15.1794f,-30.8300f,20.1761f,  
	15.7959f,-31.8662f,19.5222f,  15.0522f,-31.3670f,17.7333f,  15.6735f,-32.3743f,17.0601f,  15.1448f,-31.7012f,15.2519f,  15.7718f,-32.6750f,14.5621f,  
	15.4672f,-31.8372f,12.7711f,  16.0999f,-32.7727f,12.0701f,  16.0246f,-31.7833f,10.3329f,  16.6635f,-32.6767f,9.6238f,  16.8186f,-31.5526f,7.9762f,  
	17.4643f,-32.4012f,7.2637f,  17.8467f,-31.1629f,5.7397f,  18.4990f,-31.9641f,5.0293f,  19.1019f,-30.6357f,3.6605f,  19.7602f,-31.3888f,2.9582f,  
	20.5728f,-29.9973f,1.7726f,  21.2367f,-30.7021f,1.0835f,  22.2443f,-29.2773f,0.1053f,  22.9129f,-29.9355f,-0.5655f,  24.0976f,-28.5094f,-1.3169f,  
	24.7690f,-29.1243f,-1.9646f,  26.1098f,-27.7303f,-2.4757f,  26.7847f,-28.3051f,-3.0980f,  28.2579f,-26.9774f,-3.3605f,  28.9332f,-27.5181f,-3.9545f,  
	30.5130f,-26.2909f,-3.9668f,  31.1869f,-26.8045f,-4.5322f,  32.8459f,-25.7110f,-4.2993f,  33.5180f,-26.2039f,-4.8376f,  35.2275f,-25.2756f,-4.3706f,  
	35.8965f,-25.7573f,-4.8864f,  37.6273f,-25.0228f,-4.2031f,  38.2923f,-25.5001f,-4.7015f,  40.0153f,-24.9845f,-3.8254f,  40.6749f,-25.4672f,-4.3172f,  
	42.3617f,-25.1909f,-3.2776f,  43.0173f,-25.6842f,-3.7711f,  44.6402f,-25.6625f,-2.6015f,  45.2926f,-26.1731f,-3.1103f,  46.8255f,-26.4151f,-1.8479f,  
	47.4758f,-26.9480f,-2.3885f,  48.8946f,-27.4563f,-1.0729f,  49.5446f,-28.0135f,-1.6622f,  50.8272f,-28.7840f,-0.3343f,  51.4798f,-29.3652f,-0.9882f,  
	52.6066f,-30.3869f,0.3113f,  53.2645f,-30.9895f,-0.4272f,  54.2184f,-32.2442f,0.8047f,  54.8842f,-32.8627f,-0.0335f,  55.6506f,-34.3261f,1.0948f,  
	56.3270f,-34.9517f,0.1375f,  56.8942f,-36.5933f,1.1304f,  57.5828f,-37.2144f,0.0392f,  57.9417f,-38.9983f,0.8705f,  58.6432f,-39.6019f,-0.3655f,  
	58.7871f,-41.4885f,0.2844f,  59.5012f,-42.0578f,-1.1070f,  59.4262f,-44.0040f,-0.6504f,  60.1505f,-44.5218f,-2.2015f,  59.8553f,-46.4833f,-1.9422f,  
	60.5862f,-46.9313f,-3.6545f,  60.0721f,-48.8635f,-3.5883f,  60.8031f,-49.2239f,-5.4565f,  60.0740f,-51.0833f,-5.5710f,  60.7986f,-51.3380f,-7.5876f,  
	59.8614f,-53.0839f,-7.8627f,  60.5702f,-53.2174f,-10.0126f,  59.4347f,-54.8130f,-10.4208f,  60.1181f,-54.8126f,-12.6867f,  58.7970f,-56.2263f,-13.1949f,  
	59.4450f,-56.0825f,-15.5556f,  57.9546f,-57.2886f,-16.1264f,  58.5562f,-56.9961f,-18.5563f,  56.9160f,-57.9758f,-19.1494f,  57.4627f,-57.5326f,-21.6247f,  
	55.6954f,-58.2738f,-22.1986f,  56.1786f,-57.6825f,-24.6926f,  54.3104f,-58.1801f,-25.2067f,  54.7233f,-57.4471f,-27.6933f,  52.7833f,-57.7027f,-28.1096f,  
	53.1211f,-56.8364f,-30.5683f,  51.1410f,-56.8574f,-30.8527f,  51.4006f,-55.8691f,-33.2631f,  49.4141f,-55.6681f,-33.3869f,  49.5941f,-54.5703f,-35.7326f,  
	47.6361f,-54.1640f,-35.6739f,  47.7369f,-52.9691f,-37.9430f,  45.8431f,-52.3776f,-37.6867f,  45.8658f,-51.0970f,-39.8717f,  44.0713f,-50.3424f,-39.4106f,  
	44.0180f,-48.9876f,-41.5054f,  42.3569f,-48.0930f,-40.8401f,  42.2299f,-46.6724f,-42.8440f,  40.7341f,-45.6613f,-41.9825f,  40.5360f,-44.1823f,-43.8957f,  
	16.1253f,-31.1500f,21.9096f,  16.7401f,-32.2078f,21.2524f,  15.7959f,-31.8662f,19.5222f,  16.4153f,-32.8977f,18.8428f,  15.6735f,-32.3743f,17.0601f,  
	16.2983f,-33.3744f,16.3609f,  15.7718f,-32.6750f,14.5621f,  16.4022f,-33.6384f,13.8481f,  16.0999f,-32.7727f,12.0701f,  16.7365f,-33.6950f,11.3442f,  
	16.6635f,-32.6767f,9.6238f,  17.3069f,-33.5545f,8.8907f,  17.4643f,-32.4012f,7.2637f,  18.1143f,-33.2311f,6.5289f,  18.4990f,-31.9641f,5.0293f,  
	19.1552f,-32.7445f,4.2991f,  19.7602f,-31.3888f,2.9582f,  20.4223f,-32.1185f,2.2379f,  21.2367f,-30.7021f,1.0835f,  21.9037f,-31.3818f,0.3790f,  
	22.9129f,-29.9355f,-0.5655f,  23.5831f,-30.5675f,-1.2485f,  24.7690f,-29.1243f,-1.9646f,  25.4430f,-29.7111f,-2.6234f,  26.7847f,-28.3051f,-3.0980f,  
	27.4594f,-28.8518f,-3.7280f,  28.9332f,-27.5181f,-3.9545f,  29.6068f,-28.0312f,-4.5542f,  31.1869f,-26.8045f,-4.5322f,  31.8588f,-27.2899f,-5.1018f,  
	33.5180f,-26.2039f,-4.8376f,  34.1866f,-26.6709f,-5.3801f,  35.8965f,-25.7573f,-4.8864f,  36.5606f,-26.2127f,-5.4055f,  38.2923f,-25.5001f,-4.7015f,  
	38.9502f,-25.9541f,-5.2061f,  40.6749f,-25.4672f,-4.3172f,  41.3278f,-25.9256f,-4.8148f,  43.0173f,-25.6842f,-3.7711f,  43.6656f,-26.1545f,-4.2742f,  
	45.2926f,-26.1731f,-3.1103f,  45.9371f,-26.6615f,-3.6345f,  47.4758f,-26.9480f,-2.3885f,  48.1180f,-27.4577f,-2.9498f,  49.5446f,-28.0135f,-1.6622f,  
	50.1873f,-28.5458f,-2.2766f,  51.4798f,-29.3652f,-0.9882f,  52.1256f,-29.9191f,-1.6759f,  53.2645f,-30.9895f,-0.4272f,  53.9163f,-31.5608f,-1.2041f,  
	54.8842f,-32.8627f,-0.0335f,  55.5447f,-33.4445f,-0.9200f,  56.3270f,-34.9517f,0.1375f,  56.9984f,-35.5338f,-0.8752f,  57.5828f,-37.2144f,0.0392f,  
	58.2663f,-37.7847f,-1.1124f,  58.6432f,-39.6019f,-0.3655f,  59.3391f,-40.1447f,-1.6693f,  59.5012f,-42.0578f,-1.1070f,  60.2079f,-42.5563f,-2.5699f,  
	60.1505f,-44.5218f,-2.2015f,  60.8651f,-44.9579f,-3.8277f,  60.5862f,-46.9313f,-3.6545f,  61.3030f,-47.2869f,-5.4416f,  60.8031f,-49.2239f,-5.4565f,  
	61.5161f,-49.4800f,-7.4000f,  60.7986f,-51.3380f,-7.5876f,  61.4990f,-51.4779f,-9.6752f,  60.5702f,-53.2174f,-10.0126f,  61.2486f,-53.2269f,-12.2290f,  
	60.1181f,-54.8126f,-12.6867f,  60.7644f,-54.6802f,-15.0126f,  59.4450f,-56.0825f,-15.5556f,  60.0481f,-55.8009f,-17.9666f,  58.5562f,-56.9961f,-18.5563f,  
	59.1071f,-56.5614f,-21.0291f,  57.4627f,-57.5326f,-21.6247f,  57.9520f,-56.9453f,-24.1324f,  56.1786f,-57.6825f,-24.6926f,  56.5987f,-56.9474f,-27.2084f,  
	54.7233f,-57.4471f,-27.6933f,  55.0686f,-56.5713f,-30.1955f,  53.1211f,-56.8364f,-30.5683f,  53.3877f,-55.8304f,-33.0345f,  51.4006f,-55.8691f,-33.2631f,  
	51.5861f,-54.7449f,-35.6745f,  49.5941f,-54.5703f,-35.7326f,  49.6983f,-53.3403f,-38.0752f,  47.7369f,-52.9691f,-37.9430f,  47.7603f,-51.6453f,-40.2067f,  
	45.8658f,-51.0970f,-39.8717f,  45.8102f,-49.6919f,-42.0478f,  44.0180f,-48.9876f,-41.5054f,  43.8856f,-47.5107f,-43.5911f,  42.2299f,-46.6724f,-42.8440f,  
	42.0233f,-45.1327f,-44.8374f,  40.5360f,-44.1823f,-43.8957f,  40.2577f,-42.5866f,-45.7976f,  16.7401f,-32.2078f,21.2524f,  17.3575f,-33.2624f,20.5694f,  
	16.4153f,-32.8977f,18.8428f,  17.0378f,-33.9236f,18.1368f,  16.2983f,-33.3744f,16.3609f,  16.9263f,-34.3656f,15.6365f,  16.4022f,-33.6384f,13.8481f,  
	17.0363f,-34.5902f,13.1080f,  16.7365f,-33.6950f,11.3442f,  17.3773f,-34.6032f,10.5929f,  17.3069f,-33.5545f,8.8907f,  17.9543f,-34.4150f,8.1336f,  
	18.1143f,-33.2311f,6.5289f,  18.7680f,-34.0412f,5.7725f,  19.1552f,-32.7445f,4.2991f,  19.8150f,-33.5022f,3.5489f,  20.4223f,-32.1185f,2.2379f,  
	21.0874f,-32.8235f,1.5003f,  21.9037f,-31.3818f,0.3790f,  22.5724f,-32.0353f,-0.3396f,  23.5831f,-30.5675f,-1.2485f,  24.2560f,-31.1713f,-1.9446f,  
	25.4430f,-29.7111f,-2.6234f,  26.1170f,-30.2693f,-3.2913f,  27.4594f,-28.8518f,-3.7280f,  28.1325f,-29.3701f,-4.3648f,  29.6068f,-28.0312f,-4.5542f,  
	30.2782f,-28.5149f,-5.1588f,  31.8588f,-27.2899f,-5.1018f,  32.5269f,-27.7479f,-5.6757f,  34.1866f,-26.6709f,-5.3801f,  34.8499f,-27.1101f,-5.9254f,  
	36.5606f,-26.2127f,-5.4055f,  37.2170f,-26.6434f,-5.9294f,  38.9502f,-25.9541f,-5.2061f,  39.6006f,-26.3824f,-5.7150f,  41.3278f,-25.9256f,-4.8148f,  
	41.9723f,-26.3597f,-5.3196f,  43.6656f,-26.1545f,-4.2742f,  44.3047f,-26.6018f,-4.7894f,  45.9371f,-26.6615f,-3.6345f,  46.5718f,-27.1263f,-4.1752f,  
	48.1180f,-27.4577f,-2.9498f,  48.7512f,-27.9426f,-3.5319f,  50.1873f,-28.5458f,-2.2766f,  50.8212f,-29.0513f,-2.9200f,  52.1256f,-29.9191f,-1.6759f,  
	52.7635f,-30.4428f,-2.3970f,  53.9163f,-31.5608f,-1.2041f,  54.5607f,-32.0974f,-2.0241f,  55.5447f,-33.4445f,-0.9200f,  56.1983f,-33.9852f,-1.8570f,  
	56.9984f,-35.5338f,-0.8752f,  57.6629f,-36.0680f,-1.9435f,  58.2663f,-37.7847f,-1.1124f,  58.9427f,-38.2977f,-2.3278f,  59.3391f,-40.1447f,-1.6693f,  
	60.0264f,-40.6205f,-3.0411f,  60.2079f,-42.5563f,-2.5699f,  60.9040f,-42.9770f,-4.1050f,  60.8651f,-44.9579f,-3.8277f,  61.5652f,-45.3054f,-5.5267f,  
	61.3030f,-47.2869f,-5.4416f,  62.0016f,-47.5417f,-7.3030f,  61.5161f,-49.4800f,-7.4000f,  62.2051f,-49.6244f,-9.4144f,  61.4990f,-51.4779f,-9.6752f,  
	62.1693f,-51.4962f,-11.8298f,  61.2486f,-53.2269f,-12.2290f,  61.8902f,-53.1058f,-14.5066f,  60.7644f,-54.6802f,-15.0126f,  61.3661f,-54.4107f,-17.3902f,  
	60.0481f,-55.8009f,-17.9666f,  60.6005f,-55.3767f,-20.4221f,  59.1071f,-56.5614f,-21.0291f,  59.6002f,-55.9812f,-23.5359f,  57.9520f,-56.9453f,-24.1324f,  
	58.3775f,-56.2120f,-26.6633f,  56.5987f,-56.9474f,-27.2084f,  56.9503f,-56.0667f,-29.7406f,  55.0686f,-56.5713f,-30.1955f,  55.3412f,-55.5525f,-32.7051f,  
	53.3877f,-55.8304f,-33.0345f,  53.5782f,-54.6848f,-35.5007f,  51.5861f,-54.7449f,-35.6745f,  51.6936f,-53.4848f,-38.0812f,  49.6983f,-53.3403f,-38.0752f,  
	49.7225f,-51.9779f,-40.4098f,  47.7603f,-51.6453f,-40.2067f,  47.7027f,-50.1939f,-42.4581f,  45.8102f,-49.6919f,-42.0478f,  45.6727f,-48.1620f,-44.2109f,  
	43.8856f,-47.5107f,-43.5911f,  43.6708f,-45.9126f,-45.6619f,  42.0233f,-45.1327f,-44.8374f,  41.7338f,-43.4745f,-46.8148f,  40.2577f,-42.5866f,-45.7976f,  
	39.8960f,-40.8752f,-47.6819f,  17.3575f,-33.2624f,20.5694f,  17.9776f,-34.3130f,19.8593f,  17.0378f,-33.9236f,18.1368f,  17.6633f,-34.9419f,17.4048f,  
	16.9263f,-34.3656f,15.6365f,  17.5577f,-35.3468f,14.8850f,  17.0363f,-34.5902f,13.1080f,  17.6742f,-35.5293f,12.3411f,  17.3773f,-34.6032f,10.5929f,  
	18.0218f,-35.4955f,9.8159f,  17.9543f,-34.4150f,8.1336f,  18.6053f,-35.2568f,7.3530f,  18.7680f,-34.0412f,5.7725f,  19.4253f,-34.8296f,4.9939f,  
	19.8150f,-33.5022f,3.5489f,  20.4779f,-34.2359f,2.7792f,  21.0874f,-32.8235f,1.5003f,  21.7543f,-33.5026f,0.7466f,  22.5724f,-32.0353f,-0.3396f,  
	23.2438f,-32.6608f,-1.0735f,  24.2560f,-31.1713f,-1.9446f,  24.9288f,-31.7462f,-2.6516f,  26.1170f,-30.2693f,-3.2913f,  26.7892f,-30.7983f,-3.9672f,  
	28.1325f,-29.3701f,-4.3648f,  28.8033f,-29.8580f,-5.0076f,  30.2782f,-28.5149f,-5.1588f,  30.9458f,-28.9701f,-5.7682f,  32.5269f,-27.7479f,-5.6757f,  
	33.1895f,-28.1769f,-6.2523f,  34.8499f,-27.1101f,-5.9254f,  35.5050f,-27.5232f,-6.4745f,  37.2170f,-26.6434f,-5.9294f,  37.8653f,-27.0469f,-6.4560f,  
	39.6006f,-26.3824f,-5.7150f,  40.2418f,-26.7852f,-6.2287f,  41.9723f,-26.3597f,-5.3196f,  42.6066f,-26.7699f,-5.8335f,  44.3047f,-26.6018f,-4.7894f,  
	44.9325f,-27.0248f,-5.3174f,  46.5718f,-27.1263f,-4.1752f,  47.1960f,-27.5659f,-4.7326f,  48.7512f,-27.9426f,-3.5319f,  49.3737f,-28.4011f,-4.1382f,  
	50.8212f,-29.0513f,-2.9200f,  51.4454f,-29.5277f,-3.5918f,  52.7635f,-30.4428f,-2.3970f,  53.3919f,-30.9334f,-3.1560f,  54.5607f,-32.0974f,-2.0241f,  
	55.1964f,-32.5953f,-2.8896f,  56.1983f,-33.9852f,-1.8570f,  56.8434f,-34.4810f,-2.8446f,  57.6629f,-36.0680f,-1.9435f,  58.3188f,-36.5483f,-3.0710f,  
	58.9427f,-38.2977f,-2.3278f,  59.6093f,-38.7476f,-3.6070f,  60.0264f,-40.6205f,-3.0411f,  60.7023f,-41.0227f,-4.4817f,  60.9040f,-42.9770f,-4.1050f,  
	61.5851f,-43.3134f,-5.7102f,  61.5652f,-45.3054f,-5.5267f,  62.2469f,-45.5563f,-7.2983f,  62.0016f,-47.5417f,-7.3030f,  62.6764f,-47.6885f,-9.2346f,  
	62.2051f,-49.6244f,-9.4144f,  62.8642f,-49.6502f,-11.4959f,  62.1693f,-51.4962f,-11.8298f,  62.8032f,-51.3863f,-14.0464f,  61.8902f,-53.1058f,-14.5066f,  
	62.4876f,-52.8491f,-16.8371f,  61.3661f,-54.4107f,-17.3902f,  61.9174f,-53.9989f,-19.8140f,  60.6005f,-55.3767f,-20.4221f,  61.0952f,-54.8066f,-22.9132f,  
	59.6002f,-55.9812f,-23.5359f,  60.0293f,-55.2537f,-26.0672f,  58.3775f,-56.2120f,-26.6633f,  58.7338f,-55.3311f,-29.2114f,  56.9503f,-56.0667f,-29.7406f,  
	57.2278f,-55.0402f,-32.2804f,  55.3412f,-55.5525f,-32.7051f,  55.5362f,-54.3912f,-35.2140f,  53.5782f,-54.6848f,-35.5007f,  53.6887f,-53.4002f,-37.9608f,  
	51.6936f,-53.4848f,-38.0812f,  51.7188f,-52.0891f,-40.4777f,  49.7225f,-51.9779f,-40.4098f,  49.6633f,-50.4850f,-42.7292f,  47.7027f,-50.1939f,-42.4581f,  
	47.5607f,-48.6150f,-44.6929f,  45.6727f,-48.1620f,-44.2109f,  45.4502f,-46.5085f,-46.3551f,  43.6708f,-45.9126f,-45.6619f,  43.3705f,-44.1941f,-47.7123f,  
	41.7338f,-43.4745f,-46.8148f,  41.3585f,-41.6993f,-48.7701f,  39.8960f,-40.8752f,-47.6819f,  39.4482f,-39.0496f,-49.5428f,  17.9776f,-34.3130f,19.8593f,  
	18.6004f,-35.3575f,19.1227f,  17.6633f,-34.9419f,17.4048f,  18.2918f,-35.9521f,16.6448f,  17.5577f,-35.3468f,14.8850f,  18.1926f,-36.3170f,14.1055f,  
	17.6742f,-35.5293f,12.3411f,  18.3157f,-36.4541f,11.5470f,  18.0218f,-35.4955f,9.8159f,  18.6699f,-36.3703f,9.0137f,  18.6053f,-35.2568f,7.3530f,  
	19.2598f,-36.0780f,6.5484f,  19.4253f,-34.8296f,4.9939f,  20.0856f,-35.5949f,4.1939f,  20.4779f,-34.2359f,2.7792f,  21.1426f,-34.9443f,1.9914f,  
	21.7543f,-33.5026f,0.7466f,  22.4237f,-34.1541f,-0.0247f,  23.2438f,-32.6608f,-1.0735f,  23.9152f,-33.2572f,-1.8201f,  24.9288f,-31.7462f,-2.6516f,  
	25.6000f,-32.2914f,-3.3683f,  26.7892f,-30.7983f,-3.9672f,  27.4594f,-31.2963f,-4.6507f,  28.8033f,-29.8580f,-5.0076f,  29.4703f,-30.3164f,-5.6560f,  
	30.9458f,-28.9701f,-5.7682f,  31.6077f,-29.3950f,-6.3808f,  33.1895f,-28.1769f,-6.2523f,  33.8435f,-28.5785f,-6.8322f,  35.5050f,-27.5232f,-6.4745f,  
	36.1515f,-27.9077f,-7.0254f,  37.8653f,-27.0469f,-6.4560f,  38.5036f,-27.4235f,-6.9859f,  40.2418f,-26.7852f,-6.2287f,  40.8718f,-27.1629f,-6.7490f,  
	42.6066f,-26.7699f,-5.8335f,  43.2281f,-27.1549f,-6.3568f,  44.9325f,-27.0248f,-5.3174f,  45.5484f,-27.4222f,-5.8581f,  47.1960f,-27.5659f,-4.7326f,  
	47.8078f,-27.9795f,-5.3097f,  49.3737f,-28.4011f,-4.1382f,  49.9848f,-28.8312f,-4.7680f,  51.4454f,-29.5277f,-3.5918f,  52.0583f,-29.9724f,-4.2962f,  
	53.3919f,-30.9334f,-3.1560f,  54.0097f,-31.3876f,-3.9552f,  55.1964f,-32.5953f,-2.8896f,  55.8217f,-33.0509f,-3.8003f,  56.8434f,-34.4810f,-2.8446f,  
	57.4781f,-34.9264f,-3.8864f,  58.3188f,-36.5483f,-3.0710f,  58.9636f,-36.9696f,-4.2580f,  59.6093f,-38.7476f,-3.6070f,  60.2633f,-39.1282f,-4.9512f,  
	60.7023f,-41.0227f,-4.4817f,  61.3625f,-41.3450f,-5.9892f,  61.5851f,-43.3134f,-5.7102f,  62.2476f,-43.5577f,-7.3860f,  62.2469f,-45.5563f,-7.2983f,  
	62.9048f,-45.7034f,-9.1390f,  62.6764f,-47.6885f,-9.2346f,  63.3217f,-47.7205f,-11.2329f,  62.8642f,-49.6502f,-11.4959f,  63.4876f,-49.5510f,-13.6397f,  
	62.8032f,-51.3863f,-14.0464f,  63.3935f,-51.1430f,-16.3168f,  62.4876f,-52.8491f,-16.8371f,  63.0351f,-52.4516f,-19.2152f,  61.9174f,-53.9989f,-19.8140f,  
	62.4112f,-53.4418f,-22.2750f,  61.0952f,-54.8066f,-22.9132f,  61.5257f,-54.0887f,-25.4304f,  60.0293f,-55.2537f,-26.0672f,  60.3887f,-54.3771f,-28.6170f,  
	58.7338f,-55.3311f,-29.2114f,  59.0153f,-54.3024f,-31.7677f,  57.2278f,-55.0402f,-32.2804f,  57.4265f,-53.8689f,-34.8193f,  55.5362f,-54.3912f,-35.2140f,  
	55.6494f,-53.0878f,-37.7161f,  53.6887f,-53.4002f,-37.9608f,  53.7150f,-51.9771f,-40.4091f,  51.7188f,-52.0891f,-40.4777f,  51.6584f,-50.5602f,-42.8565f,  
	49.6633f,-50.4850f,-42.7292f,  49.5173f,-48.8620f,-45.0288f,  47.5607f,-48.6150f,-44.6929f,  47.3312f,-46.9104f,-46.9051f,  45.4502f,-46.5085f,-46.3551f,  
	45.1398f,-44.7329f,-48.4747f,  43.3705f,-44.1941f,-47.7123f,  42.9820f,-42.3572f,-49.7361f,  41.3585f,-41.6993f,-48.7701f,  40.8948f,-39.8089f,-50.6973f,  
	39.4482f,-39.0496f,-49.5428f,  38.9122f,-37.1127f,-51.3731f,  18.6004f,-35.3575f,19.1227f,  19.2261f,-36.3955f,18.3572f,  18.2918f,-35.9521f,16.6448f,  
	18.9236f,-36.9527f,15.8558f,  18.1926f,-36.3170f,14.1055f,  18.8309f,-37.2744f,13.2974f,  18.3157f,-36.4541f,11.5470f,  18.9606f,-37.3628f,10.7262f,  
	18.6699f,-36.3703f,9.0137f,  19.3213f,-37.2260f,8.1856f,  19.2598f,-36.0780f,6.5484f,  19.9173f,-36.8773f,5.7201f,  20.0856f,-35.5949f,4.1939f,  
	20.7479f,-36.3356f,3.3736f,  21.1426f,-34.9443f,1.9914f,  21.8100f,-35.6255f,1.1836f,  22.4237f,-34.1541f,-0.0247f,  23.0935f,-34.7766f,-0.8109f,  
	23.9152f,-33.2572f,-1.8201f,  24.5852f,-33.8237f,-2.5783f,  25.6000f,-32.2914f,-3.3683f,  26.2693f,-32.8050f,-4.0943f,  27.4594f,-31.2963f,-4.6507f,  
	28.1257f,-31.7638f,-5.3410f,  29.4703f,-30.3164f,-5.6560f,  30.1317f,-30.7434f,-6.3084f,  31.6077f,-29.3950f,-6.3808f,  32.2609f,-29.7911f,-6.9962f,  
	33.8435f,-28.5785f,-6.8322f,  34.4886f,-28.9500f,-7.4133f,  36.1515f,-27.9077f,-7.0254f,  36.7875f,-28.2639f,-7.5783f,  38.5036f,-27.4235f,-6.9859f,  
	39.1300f,-27.7740f,-7.5202f,  40.8718f,-27.1629f,-6.7490f,  41.4879f,-27.5146f,-7.2756f,  43.2281f,-27.1549f,-6.3568f,  43.8366f,-27.5140f,-6.8893f,  
	45.5484f,-27.4222f,-5.8581f,  46.1503f,-27.7938f,-6.4144f,  47.8078f,-27.9795f,-5.3097f,  48.4065f,-28.3652f,-5.9056f,  49.9848f,-28.8312f,-4.7680f,  
	50.5826f,-29.2310f,-5.4253f,  52.0583f,-29.9724f,-4.2962f,  52.6585f,-30.3830f,-5.0355f,  54.0097f,-31.3876f,-3.9552f,  54.6153f,-31.8023f,-4.7941f,  
	55.8217f,-33.0509f,-3.8003f,  56.4350f,-33.4598f,-4.7599f,  57.4781f,-34.9264f,-3.8864f,  58.1004f,-35.3169f,-4.9828f,  58.9636f,-36.9696f,-4.2580f,  
	59.5947f,-37.3260f,-5.5056f,  60.2633f,-39.1282f,-4.9512f,  60.9012f,-39.4338f,-6.3587f,  61.3625f,-41.3450f,-5.9892f,  62.0039f,-41.5801f,-7.5645f,  
	62.2476f,-43.5577f,-7.3860f,  62.8865f,-43.7029f,-9.1291f,  62.9048f,-45.7034f,-9.1390f,  63.5337f,-45.7401f,-11.0454f,  63.3217f,-47.7205f,-11.2329f,  
	63.9318f,-47.6314f,-13.2935f,  63.4876f,-49.5510f,-13.6397f,  64.0682f,-49.3216f,-15.8381f,  63.3935f,-51.1430f,-16.3168f,  63.9346f,-50.7612f,-18.6361f,  
	63.0351f,-52.4516f,-19.2152f,  63.5258f,-51.9099f,-21.6321f,  62.4112f,-53.4418f,-22.2750f,  62.8413f,-52.7370f,-24.7637f,  61.5257f,-54.0887f,-25.4304f,  
	61.8867f,-53.2211f,-27.9674f,  60.3887f,-54.3771f,-28.6170f,  60.6729f,-53.3515f,-31.1759f,  59.0153f,-54.3024f,-31.7677f,  59.2170f,-53.1269f,-34.3235f,  
	57.4265f,-53.8689f,-34.8193f,  57.5424f,-52.5532f,-37.3512f,  55.6494f,-53.0878f,-37.7161f,  55.6770f,-51.6436f,-40.2053f,  53.7150f,-51.9771f,-40.4091f,  
	53.6538f,-50.4182f,-42.8378f,  51.6584f,-50.5602f,-42.8565f,  51.5090f,-48.8989f,-45.2129f,  49.5173f,-48.8620f,-45.0288f,  49.2815f,-47.1113f,-47.3025f,  
	47.3312f,-46.9104f,-46.9051f,  47.0114f,-45.0819f,-49.0890f,  45.1398f,-44.7329f,-48.4747f,  44.7389f,-42.8376f,-50.5637f,  42.9820f,-42.3572f,-49.7361f,  
	42.5029f,-40.4041f,-51.7277f,  40.8948f,-39.8089f,-50.6973f,  40.3406f,-37.8067f,-52.5896f,  38.9122f,-37.1127f,-51.3731f,  38.2861f,-35.0689f,-53.1654f,  
	19.2261f,-36.3955f,18.3572f,  19.8547f,-37.4257f,17.5620f,  18.9236f,-36.9527f,15.8558f,  19.5587f,-37.9423f,15.0370f,  18.8309f,-37.2744f,13.2974f,  
	19.4726f,-38.2173f,12.4611f,  18.9606f,-37.3628f,10.7262f,  19.6089f,-38.2540f,9.8775f,  19.3213f,-37.2260f,8.1856f,  19.9758f,-38.0610f,7.3318f,  
	19.9173f,-36.8773f,5.7201f,  20.5770f,-37.6529f,4.8695f,  20.7479f,-36.3356f,3.3736f,  21.4129f,-37.0501f,2.5309f,  21.8100f,-35.6255f,1.1836f,  
	22.4778f,-36.2783f,0.3587f,  23.0935f,-34.7766f,-0.8109f,  23.7622f,-35.3692f,-1.6107f,  24.5852f,-33.8237f,-2.5783f,  25.2536f,-34.3585f,-3.3479f,  
	26.2693f,-32.8050f,-4.0943f,  26.9351f,-33.2876f,-4.8286f,  28.1257f,-31.7638f,-5.3410f,  28.7866f,-32.1991f,-6.0365f,  30.1317f,-30.7434f,-6.3084f,  
	30.7843f,-31.1405f,-6.9640f,  32.2609f,-29.7911f,-6.9962f,  32.9051f,-30.1560f,-7.6131f,  34.4886f,-28.9500f,-7.4133f,  35.1229f,-29.2920f,-7.9957f,  
	36.7875f,-28.2639f,-7.5783f,  37.4110f,-28.5929f,-8.1339f,  39.1300f,-27.7740f,-7.5202f,  39.7415f,-28.0976f,-8.0582f,  41.4879f,-27.5146f,-7.2756f,  
	42.0898f,-27.8397f,-7.8084f,  43.8366f,-27.5140f,-6.8893f,  44.4296f,-27.8471f,-7.4333f,  46.1503f,-27.7938f,-6.4144f,  46.7376f,-28.1377f,-6.9850f,  
	48.4065f,-28.3652f,-5.9056f,  48.9902f,-28.7218f,-6.5239f,  50.5826f,-29.2310f,-5.4253f,  51.1660f,-29.5986f,-6.1122f,  52.6585f,-30.3830f,-5.0355f,  
	53.2449f,-30.7565f,-5.8091f,  54.6153f,-31.8023f,-4.7941f,  55.2073f,-32.1736f,-5.6767f,  56.4350f,-33.4598f,-4.7599f,  57.0344f,-33.8177f,-5.7692f,  
	58.1004f,-35.3169f,-4.9828f,  58.7079f,-35.6471f,-6.1353f,  59.5947f,-37.3260f,-5.5056f,  60.2090f,-37.6125f,-6.8125f,  60.9012f,-39.4338f,-6.3587f,  
	61.5198f,-39.6574f,-7.8309f,  62.0039f,-41.5801f,-7.5645f,  62.6217f,-41.7215f,-9.2051f,  62.8865f,-43.7029f,-9.1291f,  63.4968f,-43.7429f,-10.9367f,  
	63.5337f,-45.7401f,-11.0454f,  64.1282f,-45.6602f,-13.0139f,  63.9318f,-47.6314f,-13.2935f,  64.5002f,-47.4159f,-15.4092f,  64.0682f,-49.3216f,-15.8381f,  
	64.6005f,-48.9566f,-18.0864f,  63.9346f,-50.7612f,-18.6361f,  64.4199f,-50.2373f,-20.9957f,  63.5258f,-51.9099f,-21.6321f,  63.9534f,-51.2218f,-24.0786f,  
	62.8413f,-52.7370f,-24.7637f,  63.2021f,-51.8829f,-27.2739f,  61.8867f,-53.2211f,-27.9674f,  62.1724f,-52.2040f,-30.5149f,  60.6729f,-53.3515f,-31.1759f,  
	60.8769f,-52.1780f,-33.7350f,  59.2170f,-53.1269f,-34.3235f,  59.3350f,-51.8053f,-36.8725f,  57.5424f,-52.5532f,-37.3512f,  57.5712f,-51.0945f,-39.8697f,  
	55.6770f,-51.6436f,-40.2053f,  55.6154f,-50.0614f,-42.6734f,  53.6538f,-50.4182f,-42.8378f,  53.5017f,-48.7248f,-45.2420f,  51.5090f,-48.8989f,-45.2129f,  
	51.2679f,-47.1078f,-47.5405f,  49.2815f,-47.1113f,-47.3025f,  48.9534f,-45.2349f,-49.5447f,  47.0114f,-45.0819f,-49.0890f,  46.5990f,-43.1322f,-51.2385f,  
	44.7389f,-42.8376f,-50.5637f,  44.2451f,-40.8249f,-52.6164f,  42.5029f,-40.4041f,-51.7277f,  41.9312f,-38.3385f,-53.6801f,  40.3406f,-37.8067f,-52.5896f,  
	39.6945f,-35.6970f,-54.4398f,  38.2861f,-35.0689f,-53.1654f,  37.5687f,-32.9221f,-54.9131f,  19.8547f,-37.4257f,17.5620f,  20.4864f,-38.4466f,16.7361f,  
	19.5587f,-37.9423f,15.0370f,  20.1969f,-38.9192f,14.1888f,  19.4726f,-38.2173f,12.4611f,  20.1174f,-39.1443f,11.5953f,  19.6089f,-38.2540f,9.8775f,  
	20.2602f,-39.1258f,9.0013f,  19.9758f,-38.0610f,7.3318f,  20.6326f,-38.8735f,6.4536f,  20.5770f,-37.6529f,4.8695f,  21.2394f,-38.4033f,3.9941f,  
	21.4129f,-37.0501f,2.5309f,  22.0787f,-37.7366f,1.6687f,  22.4778f,-36.2783f,0.3587f,  23.1450f,-36.9013f,-0.4821f,  23.7622f,-35.3692f,-1.6107f,  
	24.4295f,-35.9300f,-2.4242f,  25.2536f,-34.3585f,-3.3479f,  25.9187f,-34.8617f,-4.1276f,  26.9351f,-33.2876f,-4.8286f,  27.5956f,-33.7371f,-5.5698f,  
	28.7866f,-32.1991f,-6.0365f,  29.4390f,-32.6034f,-6.7358f,  30.7843f,-31.1405f,-6.9640f,  31.4279f,-31.5052f,-7.6214f,  32.9051f,-30.1560f,-7.6131f,  
	33.5382f,-30.4901f,-8.2309f,  35.1229f,-29.2920f,-7.9957f,  35.7442f,-29.6055f,-8.5796f,  37.4110f,-28.5929f,-8.1339f,  38.0189f,-28.8939f,-8.6912f,  
	39.7415f,-28.0976f,-8.0582f,  40.3380f,-28.3937f,-8.5998f,  42.0898f,-27.8397f,-7.8084f,  42.6750f,-28.1385f,-8.3492f,  44.4296f,-27.8471f,-7.4333f,  
	45.0066f,-28.1526f,-7.9878f,  46.7376f,-28.1377f,-6.9850f,  47.3082f,-28.4535f,-7.5733f,  48.9902f,-28.7218f,-6.5239f,  49.5578f,-29.0478f,-7.1668f,  
	51.1660f,-29.5986f,-6.1122f,  51.7339f,-29.9313f,-6.8280f,  53.2449f,-30.7565f,-5.8091f,  53.8160f,-31.0898f,-6.6210f,  55.2073f,-32.1736f,-5.6767f,  
	55.7840f,-32.4978f,-6.6037f,  57.0344f,-33.8177f,-5.7692f,  57.6179f,-34.1200f,-6.8296f,  58.7079f,-35.6471f,-6.1353f,  59.2979f,-35.9123f,-7.3427f,  
	60.2090f,-37.6125f,-6.8125f,  60.8037f,-37.8224f,-8.1805f,  61.5198f,-39.6574f,-7.8309f,  62.1150f,-39.7929f,-9.3657f,  62.6217f,-41.7215f,-9.2051f,  
	63.2115f,-41.7630f,-10.9084f,  63.4968f,-43.7429f,-10.9367f,  64.0735f,-43.6713f,-12.8054f,  64.1282f,-45.6602f,-13.0139f,  64.6821f,-45.4586f,-15.0375f,  
	64.5002f,-47.4159f,-15.4092f,  65.0214f,-47.0687f,-17.5757f,  64.6005f,-48.9566f,-18.0864f,  65.0781f,-48.4525f,-20.3766f,  64.4199f,-50.2373f,-20.9957f,  
	64.8430f,-49.5689f,-23.3868f,  63.9534f,-51.2218f,-24.0786f,  64.3124f,-50.3853f,-26.5483f,  63.2021f,-51.8829f,-27.2739f,  63.4880f,-50.8794f,-29.7962f,  
	62.1724f,-52.2040f,-30.5149f,  62.3778f,-51.0383f,-33.0638f,  60.8769f,-52.1780f,-33.7350f,  60.9967f,-50.8570f,-36.2881f,  59.3350f,-51.8053f,-36.8725f,  
	59.3651f,-50.3391f,-39.4080f,  57.5712f,-51.0945f,-39.8697f,  57.5096f,-49.4961f,-42.3662f,  55.6154f,-50.0614f,-42.6734f,  55.4614f,-48.3430f,-45.1155f,  
	53.5017f,-48.7248f,-45.2420f,  53.2562f,-46.9000f,-47.6151f,  51.2679f,-47.1078f,-47.5405f,  50.9324f,-45.1895f,-49.8339f,  48.9534f,-45.2349f,-49.5447f,  
	48.5305f,-43.2361f,-51.7491f,  46.5990f,-43.1322f,-51.2385f,  46.0916f,-41.0642f,-53.3481f,  44.2451f,-40.8249f,-52.6164f,  43.6568f,-38.6990f,-54.6259f,  
	41.9312f,-38.3385f,-53.6801f,  41.2656f,-36.1650f,-55.5864f,  39.6945f,-35.6970f,-54.4398f,  38.9552f,-33.4843f,-56.2412f,  37.5687f,-32.9221f,-54.9131f,  
	36.7595f,-30.6790f,-56.6083f,  20.4864f,-38.4466f,16.7361f,  21.1213f,-39.4564f,15.8796f,  20.1969f,-38.9192f,14.1888f,  20.8384f,-39.8818f,13.3095f,  
	20.1174f,-39.1443f,11.5953f,  20.7655f,-40.0536f,10.7002f,  20.2602f,-39.1258f,9.0013f,  20.9140f,-39.9765f,8.0986f,  20.6326f,-38.8735f,6.4536f,  
	21.2923f,-39.6620f,5.5481f,  21.2394f,-38.4033f,3.9941f,  21.9029f,-39.1266f,3.0967f,  22.0787f,-37.7366f,1.6687f,  22.7442f,-38.3939f,0.7882f,  
	23.1450f,-36.9013f,-0.4821f,  23.8111f,-37.4928f,-1.3391f,  24.4295f,-35.9300f,-2.4242f,  25.0939f,-36.4591f,-3.2500f,  25.9187f,-34.8617f,-4.1276f,  
	26.5789f,-35.3313f,-4.9161f,  27.5956f,-33.7371f,-5.5698f,  28.2479f,-34.1549f,-6.3158f,  29.4390f,-32.6034f,-6.7358f,  30.0825f,-32.9743f,-7.4381f,  
	31.4279f,-31.5052f,-7.6214f,  32.0606f,-31.8378f,-8.2801f,  33.5382f,-30.4901f,-8.2309f,  34.1582f,-30.7945f,-8.8497f,  35.7442f,-29.6055f,-8.5796f,  
	36.3495f,-29.8899f,-9.1636f,  38.0189f,-28.8939f,-8.6912f,  38.6110f,-29.1665f,-9.2499f,  40.3380f,-28.3937f,-8.5998f,  40.9167f,-28.6630f,-9.1463f,  
	42.6750f,-28.1385f,-8.3492f,  43.2432f,-28.4097f,-8.8968f,  45.0066f,-28.1526f,-7.9878f,  45.5655f,-28.4306f,-8.5554f,  47.3082f,-28.4535f,-7.5733f,  
	47.8612f,-28.7400f,-8.1812f,  49.5578f,-29.0478f,-7.1668f,  50.1082f,-29.3410f,-7.8333f,  51.7339f,-29.9313f,-6.8280f,  52.2849f,-30.2269f,-7.5766f,  
	53.8160f,-31.0898f,-6.6210f,  54.3704f,-31.3797f,-7.4720f,  55.7840f,-32.4978f,-6.6037f,  56.3436f,-32.7709f,-7.5767f,  57.6179f,-34.1200f,-6.8296f,  
	58.1833f,-34.3623f,-7.9404f,  59.2979f,-35.9123f,-7.3427f,  59.8678f,-36.1066f,-8.6073f,  60.8037f,-37.8224f,-8.1805f,  61.3751f,-37.9501f,-9.6082f,  
	62.1150f,-39.7929f,-9.3657f,  62.6827f,-39.8344f,-10.9612f,  63.2115f,-41.7630f,-10.9084f,  63.7685f,-41.6987f,-12.6718f,  64.0735f,-43.6713f,-12.8054f,  
	64.6108f,-43.4833f,-14.7293f,  64.6821f,-45.4586f,-15.0375f,  65.1901f,-45.1298f,-17.1126f,  65.0214f,-47.0687f,-17.5757f,  65.4894f,-46.5861f,-19.7853f,  
	65.0781f,-48.4525f,-20.3766f,  65.4950f,-47.8067f,-22.7001f,  64.8430f,-49.5689f,-23.3868f,  65.1987f,-48.7540f,-25.8031f,  64.3124f,-50.3853f,-26.5483f,  
	64.5974f,-49.4003f,-29.0321f,  63.4880f,-50.8794f,-29.7962f,  63.6941f,-49.7274f,-32.3215f,  62.3778f,-51.0383f,-33.0638f,  62.4991f,-49.7245f,-35.6080f,  
	60.9967f,-50.8570f,-36.2881f,  61.0282f,-49.3904f,-38.8282f,  59.3651f,-50.3391f,-39.4080f,  59.3040f,-48.7317f,-41.9214f,  57.5096f,-49.4961f,-42.3662f,  
	57.3545f,-47.7598f,-44.8358f,  55.4614f,-48.3430f,-45.1155f,  55.2124f,-46.4914f,-47.5250f,  53.2562f,-46.9000f,-47.6151f,  52.9145f,-44.9463f,-49.9517f,  
	50.9324f,-45.1895f,-49.8339f,  50.5003f,-43.1474f,-52.0866f,  48.5305f,-43.2361f,-51.7491f,  48.0108f,-41.1178f,-53.9102f,  46.0916f,-41.0642f,-53.3481f,  
	45.4878f,-38.8822f,-55.4109f,  43.6568f,-38.6990f,-54.6259f,  42.9726f,-36.4647f,-56.5856f,  41.2656f,-36.1650f,-55.5864f,  40.5050f,-33.8881f,-57.4400f,  
	38.9552f,-33.4843f,-56.2412f,  38.1223f,-31.1751f,-57.9863f,  36.7595f,-30.6790f,-56.6083f,  35.8585f,-28.3460f,-58.2438f,  21.1213f,-39.4564f,15.8796f,  
	21.7594f,-40.4538f,14.9907f,  20.8384f,-39.8818f,13.3095f,  21.4831f,-40.8284f,12.3992f,  20.7655f,-40.0536f,10.7002f,  21.4163f,-40.9432f,9.7765f,  
	20.9140f,-39.9765f,8.0986f,  21.5708f,-40.8047f,7.1664f,  21.2923f,-39.6620f,5.5481f,  21.9535f,-40.4245f,4.6182f,  21.9029f,-39.1266f,3.0967f,  
	22.5667f,-39.8214f,2.1785f,  22.7442f,-38.3939f,0.7882f,  23.4091f,-39.0201f,-0.1111f,  23.8111f,-37.4928f,-1.3391f,  24.4749f,-38.0525f,-2.2108f,  
	25.0939f,-36.4591f,-3.2500f,  25.7540f,-36.9542f,-4.0868f,  26.5789f,-35.3313f,-4.9161f,  27.2315f,-35.7684f,-5.7110f,  28.2479f,-34.1549f,-6.3158f,  
	28.8918f,-34.5382f,-7.0662f,  30.0825f,-32.9743f,-7.4381f,  30.7152f,-33.3120f,-8.1422f,  32.0606f,-31.8378f,-8.2801f,  32.6800f,-32.1396f,-8.9395f,  
	34.1582f,-30.7945f,-8.8497f,  34.7620f,-31.0686f,-9.4675f,  36.3495f,-29.8899f,-9.1636f,  36.9386f,-30.1450f,-9.7475f,  38.6110f,-29.1665f,-9.2499f,  
	39.1847f,-29.4117f,-9.8109f,  40.9167f,-28.6630f,-9.1463f,  41.4775f,-28.9045f,-9.6965f,  43.2432f,-28.4097f,-8.8968f,  43.7919f,-28.6537f,-9.4535f,  
	45.5655f,-28.4306f,-8.5554f,  46.1053f,-28.6806f,-9.1381f,  47.8612f,-28.7400f,-8.1812f,  48.3956f,-28.9955f,-8.8078f,  50.1082f,-29.3410f,-7.8333f,  
	50.6402f,-29.5997f,-8.5273f,  52.2849f,-30.2269f,-7.5766f,  52.8179f,-30.4826f,-8.3591f,  54.3704f,-31.3797f,-7.4720f,  54.9066f,-31.6229f,-8.3637f,  
	56.3436f,-32.7709f,-7.5767f,  56.8844f,-32.9890f,-8.5953f,  58.1833f,-34.3623f,-7.9404f,  58.7281f,-34.5393f,-9.1042f,  59.8678f,-36.1066f,-8.6073f,  
	60.4145f,-36.2247f,-9.9284f,  61.3751f,-37.9501f,-9.6082f,  61.9194f,-37.9900f,-11.0942f,  62.6827f,-39.8344f,-10.9612f,  63.2185f,-39.7761f,-12.6153f,  
	63.7685f,-41.6987f,-12.6718f,  64.2875f,-41.5236f,-14.4899f,  64.6108f,-43.4833f,-14.7293f,  65.1038f,-43.1732f,-16.7050f,  65.1901f,-45.1298f,-17.1126f,  
	65.6465f,-44.6700f,-19.2320f,  65.4894f,-46.5861f,-19.7853f,  65.8981f,-45.9652f,-22.0302f,  65.4950f,-47.8067f,-22.7001f,  65.8457f,-47.0169f,-25.0508f,  
	65.1987f,-48.7540f,-25.8031f,  65.4814f,-47.7922f,-28.2355f,  64.5974f,-49.4003f,-29.0321f,  64.8033f,-48.2677f,-31.5208f,  63.6941f,-49.7274f,-32.3215f,  
	63.8164f,-48.4272f,-34.8438f,  62.4991f,-49.7245f,-35.6080f,  62.5319f,-48.2645f,-38.1401f,  61.0282f,-49.3904f,-38.8282f,  60.9680f,-47.7815f,-41.3467f,  
	59.3040f,-48.7317f,-41.9214f,  59.1486f,-46.9852f,-44.4075f,  57.3545f,-47.7598f,-44.8358f,  57.1032f,-45.8889f,-47.2716f,  55.2124f,-46.4914f,-47.5250f,  
	54.8660f,-44.5096f,-49.8961f,  52.9145f,-44.9463f,-49.9517f,  52.4748f,-42.8677f,-52.2454f,  50.5003f,-43.1474f,-52.0866f,  49.9698f,-40.9848f,-54.2933f,  
	48.0108f,-41.1178f,-53.9102f,  47.3928f,-38.8847f,-56.0215f,  45.4878f,-38.8822f,-55.4109f,  44.7863f,-36.5911f,-57.4203f,  42.9726f,-36.4647f,-56.5856f,  
	42.1917f,-34.1267f,-58.4890f,  40.5050f,-33.8881f,-57.4400f,  39.6491f,-31.5147f,-59.2336f,  38.1223f,-31.1751f,-57.9863f,  37.1959f,-28.7761f,-59.6680f,  
	35.8585f,-28.3460f,-58.2438f,  34.8666f,-25.9312f,-59.8120f,  21.7594f,-40.4538f,14.9907f,  22.4007f,-41.4369f,14.0694f,  21.4831f,-40.8284f,12.3992f,  
	22.1309f,-41.7569f,11.4586f,  21.4163f,-40.9432f,9.7765f,  22.0702f,-41.8118f,8.8212f,  21.5708f,-40.8047f,7.1664f,  22.2296f,-41.6080f,6.2073f,  
	21.9535f,-40.4245f,4.6182f,  22.6154f,-41.1593f,3.6650f,  22.5667f,-39.8214f,2.1785f,  23.2304f,-40.4857f,1.2387f,  23.4091f,-39.0201f,-0.1111f,  
	24.0723f,-39.6146f,-1.0277f,  24.4749f,-38.0525f,-2.2108f,  25.1350f,-38.5782f,-3.0960f,  25.7540f,-36.9542f,-4.0868f,  26.4072f,-37.4162f,-4.9319f,  
	27.2315f,-35.7684f,-5.7110f,  27.8762f,-36.1703f,-6.5121f,  28.8918f,-34.5382f,-7.0662f,  29.5253f,-34.8874f,-7.8197f,  30.7152f,-33.3120f,-8.1422f,  
	31.3351f,-33.6176f,-8.8475f,  32.6800f,-32.1396f,-8.9395f,  33.2833f,-32.4099f,-9.5976f,  34.7620f,-31.0686f,-9.4675f,  35.3492f,-31.3123f,-10.0843f,  
	36.9386f,-30.1450f,-9.7475f,  37.5087f,-30.3718f,-10.3317f,  39.1847f,-29.4117f,-9.8109f,  39.7397f,-29.6285f,-10.3731f,  41.4775f,-28.9045f,-9.6965f,  
	42.0177f,-29.1188f,-10.2522f,  43.7919f,-28.6537f,-9.4535f,  44.3203f,-28.8704f,-10.0211f,  46.1053f,-28.6806f,-9.1381f,  46.6253f,-28.9008f,-9.7348f,  
	48.3956f,-28.9955f,-8.8078f,  48.9101f,-29.2189f,-9.4567f,  50.6402f,-29.5997f,-8.5273f,  51.1531f,-29.8218f,-9.2500f,  52.8179f,-30.4826f,-8.3591f,  
	53.3316f,-30.6956f,-9.1771f,  54.9066f,-31.6229f,-8.3637f,  55.4233f,-31.8159f,-9.2962f,  56.8844f,-32.9890f,-8.5953f,  57.4042f,-33.1474f,-9.6625f,  
	58.7281f,-34.5393f,-9.1042f,  59.2499f,-34.6462f,-10.3209f,  60.4145f,-36.2247f,-9.9284f,  60.9348f,-36.2613f,-11.3050f,  61.9194f,-37.9900f,-11.0942f,  
	62.4329f,-37.9364f,-12.6369f,  63.2185f,-39.7761f,-12.6153f,  63.7177f,-39.6132f,-14.3234f,  64.2875f,-41.5236f,-14.4899f,  64.7638f,-41.2322f,-16.3600f,  
	65.1038f,-43.1732f,-16.7050f,  65.5468f,-42.7371f,-18.7262f,  65.6465f,-44.6700f,-19.2320f,  66.0455f,-44.0761f,-21.3887f,  65.8981f,-45.9652f,-22.0302f,  
	66.2425f,-45.2037f,-24.3046f,  65.8457f,-47.0169f,-25.0508f,  66.1250f,-46.0825f,-27.4202f,  65.4814f,-47.7922f,-28.2355f,  65.6863f,-46.6842f,-30.6753f,  
	64.8033f,-48.2677f,-31.5208f,  64.9260f,-46.9874f,-34.0087f,  63.8164f,-48.4272f,-34.8438f,  63.8506f,-46.9808f,-37.3556f,  62.5319f,-48.2645f,-38.1401f,  
	62.4730f,-46.6616f,-40.6517f,  60.9680f,-47.7815f,-41.3467f,  60.8130f,-46.0326f,-43.8380f,  59.1486f,-46.9852f,-44.4075f,  58.8961f,-45.1030f,-46.8595f,  
	57.1032f,-45.8889f,-47.2716f,  56.7534f,-43.8868f,-49.6681f,  54.8660f,-44.5096f,-49.8961f,  54.4202f,-42.4017f,-52.2227f,  52.4748f,-42.8677f,-52.2454f,  
	51.9351f,-40.6675f,-54.4907f,  49.9698f,-40.9848f,-54.2933f,  49.3394f,-38.7067f,-56.4473f,  47.3928f,-38.8847f,-56.0215f,  46.6756f,-36.5420f,-58.0763f,  
	44.7863f,-36.5911f,-57.4203f,  43.9863f,-34.1961f,-59.3701f,  42.1917f,-34.1267f,-58.4890f,  41.3138f,-31.6920f,-60.3288f,  39.6491f,-31.5147f,-59.2336f,  
	38.6980f,-29.0514f,-60.9603f,  37.1959f,-28.7761f,-59.6680f,  36.1771f,-26.2954f,-61.2790f,  34.8666f,-25.9312f,-59.8120f,  33.7850f,-23.4433f,-61.3059f,  
	22.4007f,-41.4369f,14.0694f,  23.0455f,-42.4037f,13.1161f,  22.1309f,-41.7569f,11.4586f,  22.7818f,-42.6661f,10.4843f,  22.0702f,-41.8118f,8.8212f,  
	22.7264f,-42.6570f,7.8367f,  22.2296f,-41.6080f,6.2073f,  22.8896f,-42.3848f,5.2223f,  22.6154f,-41.1593f,3.6650f,  23.2779f,-41.8647f,2.6873f,  
	23.2304f,-40.4857f,1.2387f,  23.8930f,-41.1188f,0.2790f,  24.0723f,-39.6146f,-1.0277f,  24.7325f,-40.1754f,-1.9606f,  25.1350f,-38.5782f,-3.0960f,  
	25.7890f,-39.0703f,-3.9916f,  26.4072f,-37.4162f,-4.9319f,  27.0530f,-37.8426f,-5.7854f,  27.8762f,-36.1703f,-6.5121f,  28.5110f,-36.5372f,-7.3181f,  
	29.5253f,-34.8874f,-7.8197f,  30.1463f,-35.2035f,-8.5753f,  31.3351f,-33.6176f,-8.8475f,  31.9390f,-33.8906f,-9.5516f,  33.2833f,-32.4099f,-9.5976f,  
	33.8701f,-32.6485f,-10.2543f,  35.3492f,-31.3123f,-10.0843f,  35.9173f,-31.5268f,-10.6998f,  37.5087f,-30.3718f,-10.3317f,  38.0595f,-30.5695f,-10.9152f,  
	39.7397f,-29.6285f,-10.3731f,  40.2732f,-29.8181f,-10.9379f,  42.0177f,-29.1188f,-10.2522f,  42.5366f,-29.3064f,-10.8151f,  44.3203f,-28.8704f,-10.0211f,  
	44.8277f,-29.0586f,-10.5984f,  46.6253f,-28.9008f,-9.7348f,  47.1242f,-29.0910f,-10.3490f,  48.9101f,-29.2189f,-9.4567f,  49.4044f,-29.4086f,-10.1295f,  
	51.1531f,-29.8218f,-9.2500f,  51.6456f,-30.0049f,-10.0031f,  53.3316f,-30.6956f,-9.1771f,  53.8251f,-30.8630f,-10.0311f,  55.4233f,-31.8159f,-9.2962f,  
	55.9186f,-31.9546f,-10.2727f,  57.4042f,-33.1474f,-9.6625f,  57.9010f,-33.2417f,-10.7788f,  59.2499f,-34.6462f,-10.3209f,  59.7457f,-34.6780f,-11.5900f,  
	60.9348f,-36.2613f,-11.3050f,  61.4252f,-36.2111f,-12.7361f,  62.4329f,-37.9364f,-12.6369f,  62.9113f,-37.7847f,-14.2327f,  63.7177f,-39.6132f,-14.3234f,  
	64.1760f,-39.3401f,-16.0835f,  64.7638f,-41.2322f,-16.3600f,  65.1921f,-40.8203f,-18.2765f,  65.5468f,-42.7371f,-18.7262f,  65.9347f,-42.1716f,-20.7865f,  
	66.0455f,-44.0761f,-21.3887f,  66.3821f,-43.3454f,-23.5770f,  66.2425f,-45.2037f,-24.3046f,  66.5172f,-44.3005f,-26.6001f,  66.1250f,-46.0825f,-27.4202f,  
	66.3280f,-45.0039f,-29.7996f,  65.6863f,-46.6842f,-30.6753f,  65.8091f,-45.4295f,-33.1168f,  64.9260f,-46.9874f,-34.0087f,  64.9615f,-45.5612f,-36.4881f,  
	63.8506f,-46.9808f,-37.3556f,  63.7933f,-45.3913f,-39.8485f,  62.4730f,-46.6616f,-40.6517f,  62.3192f,-44.9181f,-43.1370f,  60.8130f,-46.0326f,-43.8380f,  
	60.5605f,-44.1471f,-46.2955f,  58.8961f,-45.1030f,-46.8595f,  58.5444f,-43.0885f,-49.2716f,  56.7534f,-43.8868f,-49.6681f,  56.3031f,-41.7575f,-52.0188f,  
	54.4202f,-42.4017f,-52.2227f,  53.8731f,-40.1714f,-54.4991f,  51.9351f,-40.6675f,-54.4907f,  51.2941f,-38.3511f,-56.6811f,  49.3394f,-38.7067f,-56.4473f,  
	48.6082f,-36.3182f,-58.5422f,  46.6756f,-36.5420f,-58.0763f,  45.8583f,-34.0949f,-60.0685f,  43.9863f,-34.1961f,-59.3701f,  43.0878f,-31.7040f,-61.2533f,  
	41.3138f,-31.6920f,-60.3288f,  40.3391f,-29.1673f,-62.0986f,  38.6980f,-29.0514f,-60.9603f,  37.6529f,-26.5066f,-62.6129f,  36.1771f,-26.2954f,-61.2790f,  
	35.0671f,-23.7419f,-62.8123f,  33.7850f,-23.4433f,-61.3059f,  32.6162f,-20.8924f,-62.7182f,  23.0455f,-42.4037f,13.1161f,  23.6935f,-43.3530f,12.1272f,  
	22.7818f,-42.6661f,10.4843f,  23.4356f,-43.5535f,9.4785f,  22.7264f,-42.6570f,7.8367f,  23.3846f,-43.4770f,6.8236f,  22.8896f,-42.3848f,5.2223f,  
	23.5508f,-43.1333f,4.2099f,  23.2779f,-41.8647f,2.6873f,  23.9399f,-42.5393f,1.6869f,  23.8930f,-41.1188f,0.2790f,  24.5534f,-41.7184f,-0.7001f,  
	24.7325f,-40.1754f,-1.9606f,  25.3875f,-40.7023f,-2.9064f,  25.7890f,-39.0703f,-3.9916f,  26.4364f,-39.5264f,-4.8985f,  27.0530f,-37.8426f,-5.7854f,  
	27.6898f,-38.2331f,-6.6460f,  28.5110f,-36.5372f,-7.3181f,  29.1340f,-36.8699f,-8.1276f,  30.1463f,-35.2035f,-8.5753f,  30.7519f,-35.4856f,-9.3305f,  
	31.9390f,-33.8906f,-9.5516f,  32.5266f,-34.1308f,-10.2547f,  33.8701f,-32.6485f,-10.2543f,  34.4377f,-32.8569f,-10.9089f,  35.9173f,-31.5268f,-10.6998f,  
	36.4658f,-31.7112f,-11.3134f,  38.0595f,-30.5695f,-10.9152f,  38.5884f,-30.7395f,-11.4987f,  40.2732f,-29.8181f,-10.9379f,  40.7847f,-29.9810f,-11.5065f,  
	42.5366f,-29.3064f,-10.8151f,  43.0336f,-29.4661f,-11.3840f,  44.8277f,-29.0586f,-10.5984f,  45.3130f,-29.2183f,-11.1889f,  47.1242f,-29.0910f,-10.3490f,  
	47.6017f,-29.2500f,-10.9823f,  49.4044f,-29.4086f,-10.1295f,  49.8773f,-29.5627f,-10.8275f,  51.6456f,-30.0049f,-10.0031f,  52.1172f,-30.1467f,-10.7874f,  
	53.8251f,-30.8630f,-10.0311f,  54.2968f,-30.9813f,-10.9244f,  55.9186f,-31.9546f,-10.2727f,  56.3909f,-32.0350f,-11.2945f,  57.9010f,-33.2417f,-10.7788f,  
	58.3723f,-33.2672f,-11.9444f,  59.7457f,-34.6780f,-11.5900f,  60.2125f,-34.6297f,-12.9114f,  61.4252f,-36.2111f,-12.7361f,  61.8819f,-36.0693f,-14.2192f,  
	62.9113f,-37.7847f,-14.2327f,  63.3504f,-37.5292f,-15.8801f,  64.1760f,-39.3401f,-16.0835f,  64.5883f,-38.9525f,-17.8907f,  65.1921f,-40.8203f,-18.2765f,  
	65.5674f,-40.2843f,-20.2339f,  65.9347f,-42.1716f,-20.7865f,  66.2623f,-41.4738f,-22.8806f,  66.3821f,-43.3454f,-23.5770f,  66.6512f,-42.4766f,-25.7891f,  
	66.5172f,-44.3005f,-26.6001f,  66.7175f,-43.2556f,-28.9085f,  66.3280f,-45.0039f,-29.7996f,  66.4505f,-43.7803f,-32.1836f,  65.8091f,-45.4295f,-33.1168f,  
	65.8459f,-44.0301f,-35.5524f,  64.9615f,-45.5612f,-36.4881f,  64.9063f,-43.9922f,-38.9508f,  63.7933f,-45.3913f,-39.8485f,  63.6415f,-43.6609f,-42.3166f,  
	62.3192f,-44.9181f,-43.1370f,  62.0679f,-43.0375f,-45.5893f,  60.5605f,-44.1471f,-46.2955f,  60.2083f,-42.1285f,-48.7133f,  58.5444f,-43.0885f,-49.2716f,  
	58.0915f,-40.9460f,-51.6374f,  56.3031f,-41.7575f,-52.0188f,  55.7506f,-39.5049f,-54.3183f,  53.8731f,-40.1714f,-54.4991f,  53.2236f,-37.8240f,-56.7188f,  
	51.2941f,-38.3511f,-56.6811f,  50.5510f,-35.9236f,-58.8102f,  48.6082f,-36.3182f,-58.5422f,  47.7755f,-33.8249f,-60.5720f,  45.8583f,-34.0949f,-60.0685f,  
	44.9410f,-31.5504f,-61.9911f,  43.0878f,-31.7040f,-61.2533f,  42.0910f,-29.1217f,-63.0633f,  40.3391f,-29.1673f,-62.0986f,  39.2688f,-26.5610f,-63.7910f,  
	37.6529f,-26.5066f,-62.6129f,  36.5152f,-23.8889f,-64.1847f,  35.0671f,-23.7419f,-62.8123f,  33.8684f,-21.1254f,-64.2609f,  32.6162f,-20.8924f,-62.7182f,  
	31.3635f,-18.2895f,-64.0422f,  23.6935f,-43.3530f,12.1272f,  24.3448f,-44.2821f,11.1046f,  23.4356f,-43.5535f,9.4785f,  24.0919f,-44.4170f,8.4417f,  
	23.3846f,-43.4770f,6.8236f,  24.0446f,-44.2698f,5.7804f,  23.5508f,-43.1333f,4.2099f,  24.2124f,-43.8517f,3.1719f,  23.9399f,-42.5393f,1.6869f,  
	24.6007f,-43.1810f,0.6640f,  24.5534f,-41.7184f,-0.7001f,  25.2096f,-42.2842f,-1.6949f,  25.3875f,-40.7023f,-2.9064f,  26.0369f,-41.1929f,-3.8664f,  
	26.4364f,-39.5264f,-4.8985f,  27.0757f,-39.9461f,-5.8148f,  27.6898f,-38.2331f,-6.6460f,  28.3156f,-38.5884f,-7.5120f,  29.1340f,-36.8699f,-8.1276f,  
	29.7423f,-37.1675f,-8.9380f,  30.7519f,-35.4856f,-9.3305f,  31.3417f,-35.7337f,-10.0854f,  32.5266f,-34.1308f,-10.2547f,  33.0953f,-34.3394f,-10.9554f,  
	34.4377f,-32.8569f,-10.9089f,  34.9857f,-33.0342f,-11.5610f,  36.4658f,-31.7112f,-11.3134f,  36.9921f,-31.8672f,-11.9252f,  38.5884f,-30.7395f,-11.4987f,  
	39.0947f,-30.8826f,-12.0835f,  40.7847f,-29.9810f,-11.5065f,  41.2736f,-30.1163f,-12.0780f,  43.0336f,-29.4661f,-11.3840f,  43.5075f,-29.5986f,-11.9620f,  
	45.3130f,-29.2183f,-11.1889f,  45.7760f,-29.3489f,-11.7941f,  47.6017f,-29.2500f,-10.9823f,  48.0570f,-29.3764f,-11.6362f,  49.8773f,-29.5627f,-10.8275f,  
	50.3287f,-29.6793f,-11.5522f,  52.1172f,-30.1467f,-10.7874f,  52.5666f,-30.2443f,-11.6065f,  54.2968f,-30.9813f,-10.9244f,  54.7456f,-31.0470f,-11.8590f,  
	56.3909f,-32.0350f,-11.2945f,  56.8383f,-32.0530f,-12.3622f,  58.3723f,-33.2672f,-11.9444f,  58.8157f,-33.2193f,-13.1597f,  60.2125f,-34.6297f,-12.9114f,  
	60.6473f,-34.4966f,-14.2833f,  61.8819f,-36.0693f,-14.2192f,  62.3013f,-35.8306f,-15.7533f,  63.3504f,-37.5292f,-15.8801f,  63.7459f,-37.1656f,-17.5752f,  
	64.5883f,-38.9525f,-17.8907f,  64.9501f,-38.4465f,-19.7404f,  65.5674f,-40.2843f,-20.2339f,  65.8850f,-39.6210f,-22.2272f,  66.2623f,-41.4738f,-22.8806f,  
	66.5248f,-40.6421f,-25.0013f,  66.6512f,-42.4766f,-25.7891f,  66.8481f,-41.4693f,-28.0174f,  66.7175f,-43.2556f,-28.9085f,  66.8392f,-42.0681f,-31.2248f,  
	66.4505f,-43.7803f,-32.1836f,  66.4884f,-42.4135f,-34.5645f,  65.8459f,-44.0301f,-35.5524f,  65.7930f,-42.4883f,-37.9739f,  64.9063f,-43.9922f,-38.9508f,  
	64.7572f,-42.2825f,-41.3907f,  63.6415f,-43.6609f,-42.3166f,  63.3925f,-41.7932f,-44.7531f,  62.0679f,-43.0375f,-45.5893f,  61.7168f,-41.0231f,-48.0027f,  
	60.2083f,-42.1285f,-48.7133f,  59.7545f,-39.9811f,-51.0849f,  58.0915f,-40.9460f,-51.6374f,  57.5356f,-38.6794f,-53.9514f,  55.7506f,-39.5049f,-54.3183f,  
	55.0948f,-37.1345f,-56.5599f,  53.2236f,-37.8240f,-56.7188f,  52.4708f,-35.3649f,-58.8757f,  50.5510f,-35.9236f,-58.8102f,  49.7052f,-33.3908f,-60.8721f,  
	47.7755f,-33.8249f,-60.5720f,  46.8414f,-31.2339f,-62.5297f,  44.9410f,-31.5504f,-61.9911f,  43.9239f,-28.9154f,-63.8380f,  42.0910f,-29.1217f,-63.0633f,  
	40.9972f,-26.4578f,-64.7931f,  39.2688f,-26.5610f,-63.7910f,  38.1045f,-23.8820f,-65.3996f,  36.5152f,-23.8889f,-64.1847f,  35.2873f,-21.2085f,-65.6687f,  
	33.8684f,-21.1254f,-64.2609f,  32.5843f,-18.4572f,-65.6182f,  31.3635f,-18.2895f,-64.0422f,  30.0301f,-15.6450f,-65.2722f,  24.3448f,-44.2821f,11.1046f,  
	24.9994f,-45.1889f,10.0487f,  24.0919f,-44.4170f,8.4417f,  24.7507f,-45.2549f,7.3720f,  24.0446f,-44.2698f,5.7804f,  24.7058f,-45.0337f,4.7087f,  
	24.2124f,-43.8517f,3.1719f,  24.8736f,-44.5381f,2.1083f,  24.6007f,-43.1810f,0.6640f,  25.2584f,-43.7890f,-0.3776f,  25.2096f,-42.2842f,-1.6949f,  
	25.8614f,-42.8137f,-2.7070f,  26.0369f,-41.1929f,-3.8664f,  26.6792f,-41.6468f,-4.8389f,  27.0757f,-39.9461f,-5.8148f,  27.7050f,-40.3296f,-6.7390f,  
	28.3156f,-38.5884f,-7.5120f,  28.9276f,-38.9076f,-8.3807f,  29.7423f,-37.1675f,-8.9380f,  30.3356f,-37.4299f,-9.7495f,  31.3417f,-35.7337f,-10.0854f,  
	31.9130f,-35.9490f,-10.8386f,  33.0953f,-34.3394f,-10.9554f,  33.6446f,-34.5157f,-11.6536f,  34.9857f,-33.0342f,-11.5610f,  35.5115f,-33.1821f,-12.2100f,  
	36.9921f,-31.8672f,-11.9252f,  37.4954f,-31.9959f,-12.5362f,  39.0947f,-30.8826f,-12.0835f,  39.5778f,-30.9982f,-12.6686f,  41.2736f,-30.1163f,-12.0780f,  
	41.7386f,-30.2252f,-12.6551f,  43.5075f,-29.5986f,-11.9620f,  43.9583f,-29.7035f,-12.5508f,  45.7760f,-29.3489f,-11.7941f,  46.2159f,-29.4494f,-12.4154f,  
	48.0570f,-29.3764f,-11.6362f,  48.4903f,-29.4687f,-12.3124f,  50.3287f,-29.6793f,-11.5522f,  50.7575f,-29.7561f,-12.3071f,  52.5666f,-30.2443f,-11.6065f,  
	52.9931f,-30.2944f,-12.4628f,  54.7456f,-31.0470f,-11.8590f,  55.1698f,-31.0561f,-12.8361f,  56.8383f,-32.0530f,-12.3622f,  57.2586f,-32.0039f,-13.4767f,  
	58.8157f,-33.2193f,-13.1597f,  59.2284f,-33.0932f,-14.4238f,  60.6473f,-34.4966f,-14.2833f,  61.0465f,-34.2733f,-15.7054f,  62.3013f,-35.8306f,-15.7533f,  
	62.6792f,-35.4903f,-17.3356f,  63.7459f,-37.1656f,-17.5752f,  64.0933f,-36.6896f,-19.3144f,  64.9501f,-38.4465f,-19.7404f,  65.2568f,-37.8186f,-21.6281f,  
	65.8850f,-39.6210f,-22.2272f,  66.1401f,-38.8283f,-24.2501f,  66.5248f,-40.6421f,-25.0013f,  66.7176f,-39.6754f,-27.1416f,  66.8481f,-41.4693f,-28.0174f,  
	66.9686f,-40.3220f,-30.2570f,  66.8392f,-42.0681f,-31.2248f,  66.8782f,-40.7392f,-33.5414f,  66.4884f,-42.4135f,-34.5645f,  66.4382f,-40.9054f,-36.9346f,  
	65.7930f,-42.4883f,-37.9739f,  65.6473f,-40.8065f,-40.3752f,  64.7572f,-42.2825f,-41.3907f,  64.5116f,-40.4354f,-43.8011f,  63.3925f,-41.7932f,-44.7531f,  
	63.0439f,-39.7913f,-47.1522f,  61.7168f,-41.0231f,-48.0027f,  61.2639f,-38.8794f,-50.3707f,  59.7545f,-39.9811f,-51.0849f,  59.1974f,-37.7090f,-53.4047f,  
	57.5356f,-38.6794f,-53.9514f,  56.8757f,-36.2943f,-56.2069f,  55.0948f,-37.1345f,-56.5599f,  54.3348f,-34.6517f,-58.7375f,  52.4708f,-35.3649f,-58.8757f,  
	51.6143f,-32.8000f,-60.9636f,  49.7052f,-33.3908f,-60.8721f,  48.7568f,-30.7599f,-62.8599f,  46.8414f,-31.2339f,-62.5297f,  45.8064f,-28.5520f,-64.4093f,  
	43.9239f,-28.9154f,-63.8380f,  42.8085f,-26.1988f,-65.6020f,  40.9972f,-26.4578f,-64.7931f,  39.8079f,-23.7212f,-66.4362f,  38.1045f,-23.8820f,-65.3996f,  
	36.8486f,-21.1402f,-66.9176f,  35.2873f,-21.2085f,-65.6687f,  33.9725f,-18.4764f,-67.0585f,  32.5843f,-18.4572f,-65.6182f,  31.2181f,-15.7476f,-66.8785f,  
	30.0301f,-15.6450f,-65.2722f,  28.6208f,-12.9716f,-66.4023f,  24.9994f,-45.1889f,10.0487f,  25.6571f,-46.0717f,8.9573f,  24.7507f,-45.2549f,7.3720f,  
	25.4115f,-46.0650f,6.2709f,  24.7058f,-45.0337f,4.7087f,  25.3675f,-45.7664f,3.6081f,  24.8736f,-44.5381f,2.1083f,  25.5330f,-45.1910f,1.0228f,  
	25.2584f,-43.7890f,-0.3776f,  25.9128f,-44.3610f,-1.4400f,  25.8614f,-42.8137f,-2.7070f,  26.5072f,-43.3062f,-3.7348f,  26.6792f,-41.6468f,-4.8389f,  
	27.3127f,-42.0637f,-5.8219f,  27.7050f,-40.3296f,-6.7390f,  28.3217f,-40.6761f,-7.6683f,  28.9276f,-38.9076f,-8.3807f,  29.5255f,-39.1903f,-9.2526f,  
	30.3356f,-37.4299f,-9.7495f,  30.9111f,-37.6580f,-10.5603f,  31.9130f,-35.9490f,-10.8386f,  32.4655f,-36.1306f,-11.5897f,  33.6446f,-34.5157f,-11.6536f,  
	34.1720f,-34.6616f,-12.3482f,  35.5115f,-33.1821f,-12.2100f,  36.0142f,-33.3018f,-12.8569f,  37.4954f,-31.9959f,-12.5362f,  37.9753f,-32.0966f,-13.1456f,  
	39.5778f,-30.9982f,-12.6686f,  40.0365f,-31.0876f,-13.2563f,  41.7386f,-30.2252f,-12.6551f,  42.1799f,-30.3077f,-13.2395f,  43.9583f,-29.7035f,-12.5508f,  
	44.3855f,-29.7802f,-13.1518f,  46.2159f,-29.4494f,-12.4154f,  46.6334f,-29.5185f,-13.0550f,  48.4903f,-29.4687f,-12.3124f,  48.9005f,-29.5250f,-13.0145f,  
	50.7575f,-29.7561f,-12.3071f,  51.1634f,-29.7903f,-13.0954f,  52.9931f,-30.2944f,-12.4628f,  53.3955f,-30.2937f,-13.3580f,  55.1698f,-31.0561f,-12.8361f,  
	55.5679f,-31.0045f,-13.8571f,  57.2586f,-32.0039f,-13.4767f,  57.6497f,-31.8834f,-14.6384f,  59.2284f,-33.0932f,-14.4238f,  59.6074f,-32.8839f,-15.7372f,  
	61.0465f,-34.2733f,-15.7054f,  61.4065f,-33.9551f,-17.1758f,  62.6792f,-35.4903f,-17.3356f,  63.0117f,-35.0438f,-18.9632f,  64.0933f,-36.6896f,-19.3144f,  
	64.3884f,-36.0976f,-21.0935f,  65.2568f,-37.8186f,-21.6281f,  65.5037f,-37.0663f,-23.5483f,  66.1401f,-38.8283f,-24.2501f,  66.3281f,-37.9047f,-26.2959f,  
	66.7176f,-39.6754f,-27.1416f,  66.8364f,-38.5720f,-29.2968f,  66.9686f,-40.3220f,-30.2570f,  67.0085f,-39.0357f,-32.5005f,  66.8782f,-40.7392f,-33.5414f,  
	66.8308f,-39.2705f,-35.8506f,  66.4382f,-40.9054f,-36.9346f,  66.2965f,-39.2581f,-39.2875f,  65.6473f,-40.8065f,-40.3752f,  65.4061f,-38.9875f,-42.7497f,  
	64.5116f,-40.4354f,-43.8011f,  64.1670f,-38.4541f,-46.1761f,  63.0439f,-39.7913f,-47.1522f,  62.5938f,-37.6596f,-49.5071f,  61.2639f,-38.8794f,-50.3707f,  
	60.7076f,-36.6104f,-52.6875f,  59.1974f,-37.7090f,-53.4047f,  58.5359f,-35.3179f,-55.6660f,  56.8757f,-36.2943f,-56.2069f,  56.1110f,-33.7962f,-58.3980f,  
	54.3348f,-34.6517f,-58.7375f,  53.4701f,-32.0626f,-60.8450f,  51.6143f,-32.8000f,-60.9636f,  50.6541f,-30.1365f,-62.9760f,  48.7568f,-30.7599f,-62.8599f,  
	47.7061f,-28.0377f,-64.7677f,  45.8064f,-28.5520f,-64.4093f,  44.6716f,-25.7883f,-66.2038f,  42.8085f,-26.1988f,-65.6020f,  41.5963f,-23.4092f,-67.2768f,  
	39.8079f,-23.7212f,-66.4362f,  38.5256f,-20.9216f,-67.9862f,  36.8486f,-21.1402f,-66.9176f,  35.5043f,-18.3467f,-68.3386f,  33.9725f,-18.4764f,-67.0585f,  
	32.5741f,-15.7029f,-68.3486f,  31.2181f,-15.7476f,-66.8785f,  29.7745f,-13.0091f,-68.0362f,  28.6208f,-12.9716f,-66.4023f,  27.1408f,-10.2815f,-67.4277f,  
	25.6571f,-46.0717f,8.9573f,  26.3176f,-46.9279f,7.8318f,  25.4115f,-46.0650f,6.2709f,  26.0739f,-46.8450f,5.1378f,  25.3675f,-45.7664f,3.6081f,  
	26.0287f,-46.4662f,2.4824f,  25.5330f,-45.1910f,1.0228f,  26.1902f,-45.8084f,-0.0871f,  25.9128f,-44.3610f,-1.4400f,  26.5625f,-44.8958f,-2.5215f,  
	26.5072f,-43.3062f,-3.7348f,  27.1455f,-43.7611f,-4.7763f,  27.3127f,-42.0637f,-5.8219f,  27.9350f,-42.4427f,-6.8128f,  28.3217f,-40.6761f,-7.6683f,  
	28.9254f,-40.9848f,-8.6032f,  29.5255f,-39.1903f,-9.2526f,  30.1067f,-39.4374f,-10.1254f,  30.9111f,-37.6580f,-10.5603f,  31.4685f,-37.8511f,-11.3704f,  
	32.4655f,-36.1306f,-11.5897f,  32.9965f,-36.2803f,-12.3375f,  34.1720f,-34.6616f,-12.3482f,  34.6765f,-34.7780f,-13.0401f,  36.0142f,-33.3018f,-12.8569f,  
	36.4935f,-33.3928f,-13.5009f,  37.9753f,-32.0966f,-13.1456f,  38.4305f,-32.1710f,-13.7554f,  40.0365f,-31.0876f,-13.2563f,  40.4712f,-31.1511f,-13.8485f,  
	42.1799f,-30.3077f,-13.2395f,  42.5970f,-30.3633f,-13.8326f,  44.3855f,-29.7802f,-13.1518f,  44.7897f,-29.8279f,-13.7673f,  46.6334f,-29.5185f,-13.0550f,  
	47.0274f,-29.5551f,-13.7164f,  48.9005f,-29.5250f,-13.0145f,  49.2879f,-29.5431f,-13.7461f,  51.1634f,-29.7903f,-13.0954f,  51.5456f,-29.7787f,-13.9191f,  
	53.3955f,-30.2937f,-13.3580f,  53.7725f,-30.2380f,-14.2942f,  55.5679f,-31.0045f,-13.8571f,  55.9381f,-30.8877f,-14.9233f,  57.6497f,-31.8834f,-14.6384f,  
	58.0089f,-31.6862f,-15.8480f,  59.6074f,-32.8839f,-15.7372f,  59.9496f,-32.5863f,-17.0987f,  61.4065f,-33.9551f,-17.1758f,  61.7237f,-33.5371f,-18.6928f,  
	63.0117f,-35.0438f,-18.9632f,  63.2945f,-34.4872f,-20.6326f,  64.3884f,-36.0976f,-21.0935f,  64.6265f,-35.3863f,-22.9079f,  65.5037f,-37.0663f,-23.5483f,  
	65.6865f,-36.1876f,-25.4948f,  66.3281f,-37.9047f,-26.2959f,  66.4449f,-36.8480f,-28.3604f,  66.8364f,-38.5720f,-29.2968f,  66.8771f,-37.3324f,-31.4596f,  
	67.0085f,-39.0357f,-32.5005f,  66.9641f,-37.6115f,-34.7406f,  66.8308f,-39.2705f,-35.8506f,  66.6938f,-37.6638f,-38.1462f,  66.2965f,-39.2581f,-39.2875f,  
	66.0607f,-37.4742f,-41.6168f,  65.4061f,-38.9875f,-42.7497f,  65.0668f,-37.0344f,-45.0912f,  64.1670f,-38.4541f,-46.1761f,  63.7214f,-36.3427f,-48.5089f,  
	62.5938f,-37.6596f,-49.5071f,  62.0405f,-35.4023f,-51.8122f,  60.7076f,-36.6104f,-52.6875f,  60.0468f,-34.2218f,-54.9465f,  58.5359f,-35.3179f,-55.6660f,  
	57.7691f,-32.8130f,-57.8630f,  56.1110f,-33.7962f,-58.3980f,  55.2410f,-31.1912f,-60.5184f,  53.4701f,-32.0626f,-60.8450f,  52.5011f,-29.3743f,-62.8760f,  
	50.6541f,-30.1365f,-62.9760f,  49.5908f,-27.3813f,-64.9068f,  47.7061f,-28.0377f,-64.7677f,  46.5548f,-25.2333f,-66.5884f,  44.6716f,-25.7883f,-66.2038f,  
	43.4389f,-22.9514f,-67.9070f,  41.5963f,-23.4092f,-67.2768f,  40.2897f,-20.5566f,-68.8561f,  38.5256f,-20.9216f,-67.9862f,  37.1536f,-18.0703f,-69.4366f,  
	35.5043f,-18.3467f,-68.3386f,  34.0751f,-15.5118f,-69.6573f,  32.5741f,-15.7029f,-68.3486f,  31.0970f,-12.9006f,-69.5333f,  29.7745f,-13.0091f,-68.0362f,  
	28.2588f,-10.2539f,-69.0864f,  27.1408f,-10.2815f,-67.4277f,  25.5958f,-7.5872f,-68.3442f,  26.3176f,-46.9279f,7.8318f,  26.9806f,-47.7553f,6.6711f,  
	26.0739f,-46.8450f,5.1378f,  26.7371f,-47.5927f,3.9761f,  26.0287f,-46.4662f,2.4824f,  26.6890f,-47.1310f,1.3286f,  26.1902f,-45.8084f,-0.0871f,  
	26.8442f,-46.3885f,-1.2198f,  26.5625f,-44.8958f,-2.5215f,  27.2061f,-45.3926f,-3.6202f,  27.1455f,-43.7611f,-4.7763f,  27.7740f,-44.1771f,-5.8291f,  
	27.9350f,-42.4427f,-6.8128f,  28.5455f,-42.7827f,-7.8124f,  28.9254f,-40.9848f,-8.6032f,  29.5136f,-41.2565f,-9.5413f,  30.1067f,-39.4374f,-10.1254f,  
	30.6707f,-39.6477f,-10.9995f,  31.4685f,-37.8511f,-11.3704f,  32.0052f,-38.0106f,-12.1781f,  32.9965f,-36.2803f,-12.3375f,  33.5051f,-36.3992f,-13.0826f,  
	34.6765f,-34.7780f,-13.0401f,  35.1577f,-34.8645f,-13.7285f,  36.4935f,-33.3928f,-13.5009f,  36.9479f,-33.4568f,-14.1437f,  38.4305f,-32.1710f,-13.7554f,  
	38.8613f,-32.2195f,-14.3673f,  40.4712f,-31.1511f,-13.8485f,  40.8812f,-31.1887f,-14.4464f,  42.5970f,-30.3633f,-13.8326f,  42.9909f,-30.3916f,-14.4371f,  
	44.7897f,-29.8279f,-13.7673f,  45.1702f,-29.8458f,-14.4009f,  47.0274f,-29.5551f,-13.7164f,  47.3985f,-29.5572f,-14.4037f,  49.2879f,-29.5431f,-13.7461f,  
	49.6518f,-29.5200f,-14.5097f,  51.5456f,-29.7787f,-13.9191f,  51.9031f,-29.7177f,-14.7808f,  53.7725f,-30.2380f,-14.2942f,  54.1229f,-30.1233f,-15.2737f,  
	55.9381f,-30.8877f,-14.9233f,  56.2781f,-30.7008f,-16.0361f,  58.0089f,-31.6862f,-15.8480f,  58.3333f,-31.4075f,-17.1056f,  59.9496f,-32.5863f,-17.0987f,  
	60.2514f,-32.1954f,-18.5078f,  61.7237f,-33.5371f,-18.6928f,  61.9940f,-33.0152f,-20.2532f,  63.2945f,-34.4872f,-20.6326f,  63.5235f,-33.8169f,-22.3399f,  
	64.6265f,-35.3863f,-22.9079f,  64.8036f,-34.5532f,-24.7522f,  65.6865f,-36.1876f,-25.4948f,  65.8008f,-35.1796f,-27.4640f,  66.4449f,-36.8480f,-28.3604f,  
	66.4863f,-35.6582f,-30.4366f,  66.8771f,-37.3324f,-31.4596f,  66.8359f,-35.9570f,-33.6233f,  66.9641f,-37.6115f,-34.7406f,  66.8322f,-36.0509f,-36.9709f,  
	66.6938f,-37.6638f,-38.1462f,  66.4642f,-35.9214f,-40.4218f,  66.0607f,-37.4742f,-41.6168f,  65.7281f,-35.5566f,-43.9161f,  65.0668f,-37.0344f,-45.0912f,  
	64.6274f,-34.9513f,-47.3931f,  63.7214f,-36.3427f,-48.5089f,  63.1731f,-34.1055f,-50.7936f,  62.0405f,-35.4023f,-51.8122f,  61.3828f,-33.0250f,-54.0606f,  
	60.0468f,-34.2218f,-54.9465f,  59.2806f,-31.7189f,-57.1417f,  57.7691f,-32.8130f,-57.8630f,  56.8966f,-30.2008f,-59.9892f,  55.2410f,-31.1912f,-60.5184f,  
	54.2660f,-28.4865f,-62.5616f,  52.5011f,-29.3743f,-62.8760f,  51.4280f,-26.5939f,-64.8243f,  49.5908f,-27.3813f,-64.9068f,  48.4257f,-24.5435f,-66.7491f,  
	46.5548f,-25.2333f,-66.5884f,  45.3044f,-22.3556f,-68.3161f,  43.4389f,-22.9514f,-67.9070f,  42.1107f,-20.0512f,-69.5126f,  40.2897f,-20.5566f,-68.8561f,  
	38.8921f,-17.6520f,-70.3337f,  37.1536f,-18.0703f,-69.4366f,  35.6953f,-15.1776f,-70.7822f,  34.0751f,-15.5118f,-69.6573f,  32.5657f,-12.6481f,-70.8680f,  
	31.0970f,-12.9006f,-69.5333f,  29.5462f,-10.0816f,-70.6079f,  28.2588f,-10.2539f,-69.0864f,  26.6766f,-7.4947f,-70.0251f,  25.5958f,-7.5872f,-68.3442f,  
	23.9924f,-4.9019f,-69.1484f,  26.9806f,-47.7553f,6.6711f,  27.6459f,-48.5513f,5.4785f,  26.7371f,-47.5927f,3.9761f,  27.4006f,-48.3062f,2.7828f,  
	26.6890f,-47.1310f,1.3286f,  27.3474f,-47.7588f,0.1481f,  26.8442f,-46.3885f,-1.2198f,  27.4935f,-46.9304f,-2.3733f,  27.2061f,-45.3926f,-3.6202f,  
	27.8416f,-45.8498f,-4.7338f,  27.7740f,-44.1771f,-5.8291f,  28.3923f,-44.5533f,-6.8938f,  28.5455f,-42.7827f,-7.8124f,  29.1419f,-43.0843f,-8.8180f,  
	29.5136f,-41.2565f,-9.5413f,  30.0858f,-41.4899f,-10.4832f,  30.6707f,-39.6477f,-10.9995f,  31.2149f,-39.8229f,-11.8726f,  32.0052f,-38.0106f,-12.1781f,  
	32.5201f,-38.1376f,-12.9837f,  33.5051f,-36.3992f,-13.0826f,  33.9908f,-36.4866f,-13.8244f,  35.1577f,-34.8645f,-13.7285f,  35.6142f,-34.9231f,-14.4147f,  
	36.9479f,-33.4568f,-14.1437f,  37.3780f,-33.4944f,-14.7870f,  38.8613f,-32.2195f,-14.3673f,  39.2673f,-32.2423f,-14.9826f,  40.8812f,-31.1887f,-14.4464f,  
	41.2677f,-31.1999f,-15.0529f,  42.9909f,-30.3916f,-14.4371f,  43.3608f,-30.3921f,-15.0562f,  45.1702f,-29.8458f,-14.4009f,  45.5278f,-29.8322f,-15.0570f,  
	47.3985f,-29.5572f,-14.4037f,  47.7465f,-29.5220f,-15.1197f,  49.6518f,-29.5200f,-14.5097f,  49.9917f,-29.4524f,-15.3083f,  51.9031f,-29.7177f,-14.7808f,  
	52.2352f,-29.6031f,-15.6835f,  54.1229f,-30.1233f,-15.2737f,  54.4447f,-29.9444f,-16.2981f,  56.2781f,-30.7008f,-16.0361f,  56.5854f,-30.4386f,-17.1963f,  
	58.3333f,-31.4075f,-17.1056f,  58.6199f,-31.0416f,-18.4115f,  60.2514f,-32.1954f,-18.5078f,  60.5092f,-31.7066f,-19.9618f,  61.9940f,-33.0152f,-20.2532f,  
	62.2136f,-32.3851f,-21.8540f,  63.5235f,-33.8169f,-22.3399f,  63.6945f,-33.0297f,-24.0806f,  64.8036f,-34.5532f,-24.7522f,  64.9152f,-33.5952f,-26.6229f,  
	65.8008f,-35.1796f,-27.4640f,  65.8427f,-34.0419f,-29.4489f,  66.4863f,-35.6582f,-30.4366f,  66.4484f,-34.3351f,-32.5181f,  66.8359f,-35.9570f,-33.6233f,  
	66.7095f,-34.4470f,-35.7815f,  66.8322f,-36.0509f,-36.9709f,  66.6096f,-34.3556f,-39.1851f,  66.4642f,-35.9214f,-40.4218f,  66.1393f,-34.0458f,-42.6709f,  
	65.7281f,-35.5566f,-43.9161f,  65.2964f,-33.5090f,-46.1789f,  64.6274f,-34.9513f,-47.3931f,  64.0861f,-32.7421f,-49.6493f,  63.1731f,-34.1055f,-50.7936f,  
	62.5209f,-31.7478f,-53.0234f,  61.3828f,-33.0250f,-54.0606f,  60.6198f,-30.5327f,-56.2464f,  59.2806f,-31.7189f,-57.1417f,  58.4086f,-29.1081f,-59.2666f,  
	56.8966f,-30.2008f,-59.9892f,  55.9188f,-27.4882f,-62.0382f,  54.2660f,-28.4865f,-62.5616f,  53.1864f,-25.6892f,-64.5217f,  51.4280f,-26.5939f,-64.8243f,  
	50.2524f,-23.7306f,-66.6832f,  48.4257f,-24.5435f,-66.7491f,  47.1607f,-21.6321f,-68.4969f,  45.3044f,-22.3556f,-68.3161f,  43.9573f,-19.4143f,-69.9444f,  
	42.1107f,-20.0512f,-69.5126f,  40.6901f,-17.0989f,-71.0144f,  38.8921f,-17.6520f,-70.3337f,  37.4069f,-14.7059f,-71.7041f,  35.6953f,-15.1776f,-70.7822f,  
	34.1553f,-12.2558f,-72.0175f,  32.5657f,-12.6481f,-70.8680f,  30.9810f,-9.7676f,-71.9661f,  29.5462f,-10.0816f,-70.6079f,  27.9274f,-7.2585f,-71.5683f,  
	26.6766f,-7.4947f,-70.0251f,  25.0344f,-4.7443f,-70.8488f,  23.9924f,-4.9019f,-69.1484f,  22.3369f,-2.2375f,-69.8381f,  27.6459f,-48.5513f,5.4785f,  
	28.3128f,-49.3139f,4.2505f,  27.4006f,-48.3062f,2.7828f,  28.0637f,-48.9832f,1.5588f,  27.3474f,-47.7588f,0.1481f,  28.0028f,-48.3481f,-1.0570f,  
	27.4935f,-46.9304f,-2.3733f,  28.1365f,-47.4321f,-3.5457f,  27.8416f,-45.8498f,-4.7338f,  28.4684f,-46.2662f,-5.8631f,  28.3923f,-44.5533f,-6.8938f,  
	28.9980f,-44.8895f,-7.9679f,  29.1419f,-43.0843f,-8.8180f,  29.7236f,-43.3459f,-9.8305f,  30.0858f,-41.4899f,-10.4832f,  30.6393f,-41.6861f,-11.4264f,  
	31.2149f,-39.8229f,-11.8726f,  31.7384f,-39.9635f,-12.7451f,  32.5201f,-38.1376f,-12.9837f,  33.0127f,-38.2314f,-13.7870f,  33.9908f,-36.4866f,-13.8244f,  
	34.4521f,-36.5446f,-14.5638f,  35.6142f,-34.9231f,-14.4147f,  36.0464f,-34.9542f,-15.1005f,  37.3780f,-33.4944f,-14.7870f,  37.7830f,-33.5060f,-15.4319f,  
	39.2673f,-32.2423f,-14.9826f,  39.6495f,-32.2391f,-15.6042f,  41.2677f,-31.1999f,-15.0529f,  41.6299f,-31.1848f,-15.6711f,  43.3608f,-30.3921f,-15.0562f,  
	43.7076f,-30.3635f,-15.6947f,  45.5278f,-29.8322f,-15.0570f,  45.8624f,-29.7847f,-15.7387f,  47.7465f,-29.5220f,-15.1197f,  48.0710f,-29.4466f,-15.8678f,  
	49.9917f,-29.4524f,-15.3083f,  50.3072f,-29.3361f,-16.1459f,  52.2352f,-29.6031f,-15.6835f,  52.5400f,-29.4301f,-16.6296f,  54.4447f,-29.9444f,-16.2981f,  
	54.7357f,-29.6964f,-17.3693f,  56.5854f,-30.4386f,-17.1963f,  56.8573f,-30.0955f,-18.4053f,  58.6199f,-31.0416f,-18.4115f,  58.8653f,-30.5839f,-19.7634f,  
	60.5092f,-31.7066f,-19.9618f,  60.7192f,-31.1153f,-21.4586f,  62.2136f,-32.3851f,-21.8540f,  62.3783f,-31.6431f,-23.4915f,  63.6945f,-33.0297f,-24.0806f,  
	63.8032f,-32.1220f,-25.8514f,  64.9152f,-33.5952f,-26.6229f,  64.9574f,-32.5111f,-28.5135f,  65.8427f,-34.0419f,-29.4489f,  65.8081f,-32.7737f,-31.4438f,  
	66.4484f,-34.3351f,-32.5181f,  66.3277f,-32.8796f,-34.5986f,  66.7095f,-34.4470f,-35.7815f,  66.4946f,-32.8037f,-37.9279f,  66.6096f,-34.3556f,-39.1851f,  
	66.2936f,-32.5280f,-41.3768f,  66.1393f,-34.0458f,-42.6709f,  65.7169f,-32.0406f,-44.8869f,  65.2964f,-33.5090f,-46.1789f,  64.7639f,-31.3354f,-48.3987f,  
	64.0861f,-32.7421f,-49.6493f,  63.4416f,-30.4121f,-51.8528f,  62.5209f,-31.7478f,-53.0234f,  61.7637f,-29.2746f,-55.1924f,  60.6198f,-30.5327f,-56.2464f,  
	59.7512f,-27.9321f,-58.3629f,  58.4086f,-29.1081f,-59.2666f,  57.4310f,-26.3964f,-61.3148f,  55.9188f,-27.4882f,-62.0382f,  54.8359f,-24.6825f,-64.0041f,  
	53.1864f,-25.6892f,-64.5217f,  52.0037f,-22.8084f,-66.3917f,  50.2524f,-23.7306f,-66.6832f,  48.9761f,-20.7931f,-68.4465f,  47.1607f,-21.6321f,-68.4969f,  
	45.7980f,-18.6567f,-70.1441f,  43.9573f,-19.4143f,-69.9444f,  42.5168f,-16.4205f,-71.4673f,  40.6901f,-17.0989f,-71.0144f,  39.1807f,-14.1048f,-72.4072f,  
	37.4069f,-14.7059f,-71.7041f,  35.8387f,-11.7305f,-72.9621f,  34.1553f,-12.2558f,-72.0175f,  32.5387f,-9.3171f,-73.1378f,  30.9810f,-9.7676f,-71.9661f,  
	29.3269f,-6.8828f,-72.9476f,  27.9274f,-7.2585f,-71.5683f,  26.2470f,-4.4442f,-72.4114f,  25.0344f,-4.7443f,-70.8488f,  23.3384f,-2.0147f,-71.5555f,  
	22.3369f,-2.2375f,-69.8381f,  20.6369f,0.3929f,-70.4117f,  28.3128f,-49.3139f,4.2505f,  28.9806f,-50.0406f,2.9881f,  28.0637f,-48.9832f,1.5588f,  
	28.7254f,-49.6215f,0.3062f,  28.0028f,-48.3481f,-1.0570f,  28.6538f,-48.8969f,-2.2851f,  28.1365f,-47.4321f,-3.5457f,  28.7724f,-47.8921f,-4.7380f,  
	28.4684f,-46.2662f,-5.8631f,  29.0844f,-46.6412f,-7.0056f,  28.9980f,-44.8895f,-7.9679f,  29.5905f,-45.1841f,-9.0525f,  29.7236f,-43.3459f,-9.8305f,  
	30.2881f,-43.5684f,-10.8470f,  30.6393f,-41.6861f,-11.4264f,  31.1732f,-41.8457f,-12.3710f,  31.7384f,-39.9635f,-12.7451f,  32.2403f,-40.0689f,-13.6169f,  
	33.0127f,-38.2314f,-13.7870f,  33.4816f,-38.2938f,-14.5883f,  34.4521f,-36.5446f,-14.5638f,  34.8894f,-36.5737f,-15.3024f,  36.0464f,-34.9542f,-15.1005f,  
	36.4536f,-34.9582f,-15.7868f,  37.7830f,-33.5060f,-15.4319f,  38.1642f,-33.4913f,-16.0816f,  39.6495f,-32.2391f,-15.6042f,  40.0074f,-32.2101f,-16.2352f,  
	41.6299f,-31.1848f,-15.6711f,  41.9691f,-31.1420f,-16.3061f,  43.7076f,-30.3635f,-15.6947f,  44.0316f,-30.3036f,-16.3561f,  45.8624f,-29.7847f,-15.7387f,  
	46.1739f,-29.7005f,-16.4498f,  48.0710f,-29.4466f,-15.8678f,  48.3720f,-29.3269f,-16.6528f,  50.3072f,-29.3361f,-16.1459f,  50.5967f,-29.1666f,-17.0250f,  
	52.5400f,-29.4301f,-16.6296f,  52.8160f,-29.1934f,-17.6217f,  54.7357f,-29.6964f,-17.3693f,  54.9936f,-29.3731f,-18.4896f,  56.8573f,-30.0955f,-18.4053f,  
	57.0906f,-29.6663f,-19.6614f,  58.8653f,-30.5839f,-19.7634f,  59.0658f,-30.0293f,-21.1603f,  60.7192f,-31.1153f,-21.4586f,  60.8774f,-30.4172f,-22.9952f,  
	62.3783f,-31.6431f,-23.4915f,  62.4838f,-30.7851f,-25.1629f,  63.8032f,-32.1220f,-25.8514f,  63.8455f,-31.0920f,-27.6463f,  64.9574f,-32.5111f,-28.5135f,  
	64.9261f,-31.2995f,-30.4188f,  65.8081f,-32.7737f,-31.4438f,  65.6934f,-31.3753f,-33.4422f,  66.3277f,-32.8796f,-34.5986f,  66.1210f,-31.2922f,-36.6718f,  
	66.4946f,-32.8037f,-37.9279f,  66.1882f,-31.0291f,-40.0561f,  66.2936f,-32.5280f,-41.3768f,  65.8816f,-30.5712f,-43.5394f,  65.7169f,-32.0406f,-44.8869f,  
	65.1949f,-29.9095f,-47.0634f,  64.7639f,-31.3354f,-48.3987f,  64.1292f,-29.0408f,-50.5688f,  63.4416f,-30.4121f,-51.8528f,  62.6927f,-27.9663f,-53.9977f,  
	61.7637f,-29.2746f,-55.1924f,  60.9012f,-26.6927f,-57.2936f,  59.7512f,-27.9321f,-58.3629f,  58.7769f,-25.2299f,-60.4038f,  57.4310f,-26.3964f,-61.3148f,  
	56.3482f,-23.5911f,-63.2803f,  54.8359f,-24.6825f,-64.0041f,  53.6494f,-21.7928f,-65.8798f,  52.0037f,-22.8084f,-66.3917f,  50.7196f,-19.8531f,-68.1657f,  
	48.9761f,-20.7931f,-68.4465f,  47.6013f,-17.7913f,-70.1082f,  45.7980f,-18.6567f,-70.1441f,  44.3410f,-15.6284f,-71.6845f,  42.5168f,-16.4205f,-71.4673f,  
	40.9864f,-13.3846f,-72.8795f,  39.1807f,-14.1048f,-72.4072f,  37.5871f,-11.0811f,-73.6856f,  35.8387f,-11.7305f,-72.9621f,  34.1926f,-8.7380f,-74.1030f,  
	32.5387f,-9.3171f,-73.1378f,  30.8511f,-6.3739f,-74.1392f,  29.3269f,-6.8828f,-72.9476f,  27.6096f,-4.0066f,-73.8092f,  26.2470f,-4.4442f,-72.4114f,  
	24.5112f,-1.6502f,-73.1347f,  23.3384f,-2.0147f,-71.5555f,  21.5962f,0.6811f,-72.1434f,  20.6369f,0.3929f,-70.4117f,  18.9001f,2.9770f,-70.8684f,  
	28.9806f,-50.0406f,2.9881f,  29.6487f,-50.7289f,1.6930f,  28.7254f,-49.6215f,0.3062f,  29.3845f,-50.2191f,-0.9737f,  28.6538f,-48.8969f,-2.2851f,  
	29.2993f,-49.4032f,-3.5376f,  28.7724f,-47.8921f,-4.7380f,  29.3995f,-48.3095f,-5.9474f,  29.0844f,-46.6412f,-7.0056f,  29.6887f,-46.9731f,-8.1624f,  
	29.5905f,-45.1841f,-9.0525f,  30.1674f,-45.4376f,-10.1444f,  30.2881f,-43.5684f,-10.8470f,  30.8343f,-43.7520f,-11.8676f,  31.1732f,-41.8457f,-12.3710f,  
	31.6866f,-41.9678f,-13.3170f,  32.2403f,-40.0689f,-13.6169f,  32.7193f,-40.1407f,-14.4878f,  33.4816f,-38.2938f,-14.5883f,  33.9269f,-38.3253f,-15.3893f,  
	34.8894f,-36.5737f,-15.3024f,  35.3019f,-36.5741f,-16.0412f,  36.4536f,-34.9582f,-15.7868f,  36.8370f,-34.9350f,-16.4769f,  38.1642f,-33.4913f,-16.0816f,  
	38.5209f,-33.4507f,-16.7387f,  40.0074f,-32.2101f,-16.2352f,  40.3421f,-32.1544f,-16.8805f,  41.9691f,-31.1420f,-16.3061f,  42.2855f,-31.0698f,-16.9613f,  
	44.0316f,-30.3036f,-16.3561f,  44.3328f,-30.2097f,-17.0442f,  46.1739f,-29.7005f,-16.4498f,  46.4627f,-29.5756f,-17.1956f,  48.3720f,-29.3269f,-16.6528f,  
	48.6481f,-29.1584f,-17.4774f,  50.5967f,-29.1666f,-17.0250f,  50.8589f,-28.9384f,-17.9490f,  52.8160f,-29.1934f,-17.6217f,  53.0608f,-28.8870f,-18.6627f,  
	54.9936f,-29.3731f,-18.4896f,  55.2153f,-28.9695f,-19.6576f,  57.0906f,-29.6663f,-19.6614f,  57.2817f,-29.1456f,-20.9643f,  59.0658f,-30.0293f,-21.1603f,  
	59.2175f,-29.3728f,-22.5999f,  60.8774f,-30.4172f,-22.9952f,  60.9796f,-29.6075f,-24.5690f,  62.4838f,-30.7851f,-25.1629f,  62.5261f,-29.8089f,-26.8623f,  
	63.8455f,-31.0920f,-27.6463f,  63.8174f,-29.9376f,-29.4606f,  64.9261f,-31.2995f,-30.4188f,  64.8175f,-29.9601f,-32.3323f,  65.6934f,-31.3753f,-33.4422f,  
	65.4952f,-29.8470f,-35.4381f,  66.1210f,-31.2922f,-36.6718f,  65.8250f,-29.5748f,-38.7313f,  66.1882f,-31.0291f,-40.0561f,  65.7879f,-29.1259f,-42.1593f,  
	65.8816f,-30.5712f,-43.5394f,  65.3717f,-28.4888f,-45.6660f,  65.1949f,-29.9095f,-47.0634f,  64.5719f,-27.6573f,-49.1932f,  64.1292f,-29.0408f,-50.5688f,  
	63.3910f,-26.6300f,-52.6829f,  62.6927f,-27.9663f,-53.9977f,  61.8391f,-25.4113f,-56.0769f,  60.9012f,-26.6927f,-57.2936f,  59.9334f,-24.0087f,-59.3207f,  
	58.7769f,-25.2299f,-60.4038f,  57.6975f,-22.4336f,-62.3628f,  56.3482f,-23.5911f,-63.2803f,  55.1616f,-20.7012f,-65.1560f,  53.6494f,-21.7928f,-65.8798f,  
	52.3612f,-18.8282f,-67.6592f,  50.7196f,-19.8531f,-68.1657f,  49.3364f,-16.8330f,-69.8374f,  47.6013f,-17.7913f,-70.1082f,  46.1313f,-14.7361f,-71.6623f,  
	44.3410f,-15.6284f,-71.6845f,  42.7930f,-12.5577f,-73.1129f,  40.9864f,-13.3846f,-72.8795f,  39.3706f,-10.3189f,-74.1757f,  37.5871f,-11.0811f,-73.6856f,  
	35.9143f,-8.0401f,-74.8450f,  34.1926f,-8.7380f,-74.1030f,  32.4740f,-5.7407f,-75.1228f,  30.8511f,-6.3739f,-74.1392f,  29.0989f,-3.4391f,-75.0184f,  
	27.6096f,-4.0066f,-73.8092f,  25.8352f,-1.1505f,-74.5487f,  24.5112f,-1.6502f,-73.1347f,  22.7274f,1.1099f,-73.7367f,  21.5962f,0.6811f,-72.1434f,  
	19.8155f,3.3307f,-72.6117f,  18.9001f,2.9770f,-70.8684f,  17.1336f,5.5040f,-71.2088f,  29.6487f,-50.7289f,1.6930f,  30.3161f,-51.3762f,0.3661f,  
	29.3845f,-50.2191f,-0.9737f,  30.0400f,-50.7737f,-2.2825f,  29.2993f,-49.4032f,-3.5376f,  29.9379f,-49.8657f,-4.8116f,  29.3995f,-48.3095f,-5.9474f,  
	30.0166f,-48.6821f,-7.1756f,  29.6887f,-46.9731f,-8.1624f,  30.2793f,-47.2617f,-9.3306f,  30.1674f,-45.4376f,-10.1444f,  30.7275f,-45.6497f,-11.2436f,  
	30.8343f,-43.7520f,-11.8676f,  31.3613f,-43.8955f,-12.8926f,  31.6866f,-41.9678f,-13.3170f,  32.1781f,-42.0536f,-14.2642f,  32.7193f,-40.1407f,-14.4878f,  
	33.1752f,-40.1790f,-15.3596f,  33.9269f,-38.3253f,-15.3893f,  34.3477f,-38.3260f,-16.1906f,  35.3019f,-36.5741f,-16.0412f,  35.6907f,-36.5457f,-16.7833f,  
	36.8370f,-34.9350f,-16.4769f,  37.1959f,-34.8852f,-17.1730f,  38.5209f,-33.4507f,-16.7387f,  38.8544f,-33.3833f,-17.4083f,  40.3421f,-32.1544f,-16.8805f,  
	40.6540f,-32.0701f,-17.5439f,  42.2855f,-31.0698f,-16.9613f,  42.5793f,-30.9655f,-17.6409f,  44.3328f,-30.2097f,-17.0442f,  44.6118f,-30.0779f,-17.7650f,  
	46.4627f,-29.5756f,-17.1956f,  46.7273f,-29.4056f,-17.9792f,  48.6481f,-29.1584f,-17.4774f,  48.8982f,-28.9357f,-18.3458f,  50.8589f,-28.9384f,-17.9490f,  
	51.0919f,-28.6454f,-18.9218f,  53.0608f,-28.8870f,-18.6627f,  53.2717f,-28.5054f,-19.7520f,  55.2153f,-28.9695f,-19.6576f,  55.3976f,-28.4794f,-20.8740f,  
	57.2817f,-29.1456f,-20.9643f,  57.4271f,-28.5278f,-22.3125f,  59.2175f,-29.3728f,-22.5999f,  59.3163f,-28.6094f,-24.0799f,  60.9796f,-29.6075f,-24.5690f,  
	61.0217f,-28.6837f,-26.1748f,  62.5261f,-29.8089f,-26.8623f,  62.5009f,-28.7114f,-28.5857f,  63.8174f,-29.9376f,-29.4606f,  63.7149f,-28.6581f,-31.2877f,  
	64.8175f,-29.9601f,-32.3323f,  64.6280f,-28.4928f,-34.2481f,  65.4952f,-29.8470f,-35.4381f,  65.2102f,-28.1899f,-37.4251f,  65.8250f,-29.5748f,-38.7313f,  
	65.4372f,-27.7296f,-40.7703f,  65.7879f,-29.1259f,-42.1593f,  65.2913f,-27.0976f,-44.2307f,  65.3717f,-28.4888f,-45.6660f,  64.7622f,-26.2853f,-47.7498f,  
	64.5719f,-27.6573f,-49.1932f,  63.8466f,-25.2886f,-51.2703f,  63.3910f,-26.6300f,-52.6829f,  62.5489f,-24.1096f,-54.7338f,  61.8391f,-25.4113f,-56.0769f,  
	60.8807f,-22.7536f,-58.0840f,  59.9334f,-24.0087f,-59.3207f,  58.8607f,-21.2299f,-61.2673f,  57.6975f,-22.4336f,-62.3628f,  56.5143f,-19.5522f,-64.2329f,  
	55.1616f,-20.7012f,-65.1560f,  53.8730f,-17.7358f,-66.9358f,  52.3612f,-18.8282f,-67.6592f,  50.9735f,-15.7982f,-69.3364f,  49.3364f,-16.8330f,-69.8374f,  
	47.8573f,-13.7589f,-71.4011f,  46.1313f,-14.7361f,-71.6623f,  44.5697f,-11.6381f,-73.1033f,  42.7930f,-12.5577f,-73.1129f,  41.1587f,-9.4567f,-74.4240f,  
	39.3706f,-10.3189f,-74.1757f,  37.6744f,-7.2354f,-75.3514f,  35.9143f,-8.0401f,-74.8450f,  34.1677f,-4.9939f,-75.8816f,  32.4740f,-5.7407f,-75.1228f,  
	30.6892f,-2.7515f,-76.0184f,  29.0989f,-3.4391f,-75.0184f,  27.2878f,-0.5240f,-75.7733f,  25.8352f,-1.1505f,-74.5487f,  24.0112f,1.6721f,-75.1643f,  
	22.7274f,1.1099f,-73.7367f,  20.9033f,3.8242f,-74.2165f,  19.8155f,3.3307f,-72.6117f,  18.0031f,5.9236f,-72.9609f,  17.1336f,5.5040f,-71.2088f,  
	15.3453f,7.9628f,-71.4339f,  30.3161f,-51.3762f,0.3661f,  30.9818f,-51.9801f,-0.9942f,  30.0400f,-50.7737f,-2.2825f,  30.6906f,-51.2834f,-3.6174f,  
	29.9379f,-49.8657f,-4.8116f,  30.5685f,-50.2821f,-6.1090f,  30.0166f,-48.6821f,-7.1756f,  30.6217f,-49.0095f,-8.4195f,  30.2793f,-47.2617f,-9.3306f,  
	30.8547f,-47.5064f,-10.5098f,  30.7275f,-45.6497f,-11.2436f,  31.2698f,-45.8190f,-12.3507f,  31.3613f,-43.8955f,-12.8926f,  31.8674f,-43.9998f,-13.9211f,  
	32.1781f,-42.0536f,-14.2642f,  32.6473f,-42.1031f,-15.2139f,  33.1752f,-40.1790f,-15.3596f,  33.6072f,-40.1840f,-16.2326f,  34.3477f,-38.3260f,-16.1906f,  
	34.7450f,-38.2958f,-16.9956f,  35.6907f,-36.5457f,-16.7833f,  36.0550f,-36.4891f,-17.5307f,  37.1959f,-34.8852f,-17.1730f,  37.5314f,-34.8078f,-17.8803f,  
	38.8544f,-33.3833f,-17.4083f,  39.1650f,-33.2876f,-18.0941f,  40.6540f,-32.0701f,-17.5439f,  40.9433f,-31.9549f,-18.2297f,  42.5793f,-30.9655f,-17.6409f,  
	42.8512f,-30.8253f,-18.3512f,  44.6118f,-30.0779f,-17.7650f,  44.8674f,-29.9042f,-18.5217f,  46.7273f,-29.4056f,-17.9792f,  46.9672f,-29.1853f,-18.8052f,  
	48.8982f,-28.9357f,-18.3458f,  49.1206f,-28.6525f,-19.2624f,  51.0919f,-28.6454f,-18.9218f,  51.2929f,-28.2819f,-19.9430f,  53.2717f,-28.5054f,-19.7520f,  
	53.4455f,-28.0420f,-20.8909f,  55.3976f,-28.4794f,-20.8740f,  55.5367f,-27.8968f,-22.1380f,  57.4271f,-28.5278f,-22.3125f,  57.5224f,-27.8076f,-23.7040f,  
	59.3163f,-28.6094f,-24.0799f,  59.3580f,-27.7356f,-25.5955f,  61.0217f,-28.6837f,-26.1748f,  60.9993f,-27.6418f,-27.8089f,  62.5009f,-28.7114f,-28.5857f,  
	62.4044f,-27.4916f,-30.3265f,  63.7149f,-28.6581f,-31.2877f,  63.5342f,-27.2527f,-33.1221f,  64.6280f,-28.4928f,-34.2481f,  64.3543f,-26.8982f,-36.1598f,  
	65.2102f,-28.1899f,-37.4251f,  64.8356f,-26.4060f,-39.3962f,  65.4372f,-27.7296f,-40.7703f,  64.9551f,-25.7597f,-42.7818f,  65.2913f,-27.0976f,-44.2307f,  
	64.6968f,-24.9483f,-46.2631f,  64.7622f,-26.2853f,-47.7498f,  64.0516f,-23.9651f,-49.7842f,  63.8466f,-25.2886f,-51.2703f,  63.0183f,-22.8100f,-53.2871f,  
	62.5489f,-24.1096f,-54.7338f,  61.6026f,-21.4860f,-56.7150f,  60.8807f,-22.7536f,-58.0840f,  59.8178f,-20.0006f,-60.0124f,  58.8607f,-21.2299f,-61.2673f,  
	57.6844f,-18.3655f,-63.1263f,  56.5143f,-19.5522f,-64.2329f,  55.2291f,-16.5947f,-66.0079f,  53.8730f,-17.7358f,-66.9358f,  52.4847f,-14.7045f,-68.6137f,  
	50.9735f,-15.7982f,-69.3364f,  49.4894f,-12.7136f,-70.9053f,  47.8573f,-13.7589f,-71.4011f,  46.2859f,-10.6417f,-72.8511f,  44.5697f,-11.6381f,-73.1033f,  
	42.9207f,-8.5094f,-74.4262f,  41.1587f,-9.4567f,-74.4240f,  39.4429f,-6.3376f,-75.6133f,  37.6744f,-7.2354f,-75.3514f,  35.9033f,-4.1463f,-76.4026f,  
	34.1677f,-4.9939f,-75.8816f,  32.3535f,-1.9553f,-76.7920f,  30.6892f,-2.7515f,-76.0184f,  28.8441f,0.2186f,-76.7875f,  27.2878f,-0.5240f,-75.7733f,  
	25.4254f,2.3580f,-76.4018f,  24.0112f,1.6721f,-75.1643f,  22.1449f,4.4492f,-75.6552f,  20.9033f,3.8242f,-74.2165f,  19.0455f,6.4821f,-74.5744f,  
	18.0031f,5.9236f,-72.9609f,  16.1669f,8.4485f,-73.1921f,  15.3453f,7.9628f,-71.4339f,  13.5436f,10.3428f,-71.5457f,  30.9818f,-51.9801f,-0.9942f,  
	31.6445f,-52.5381f,-2.3855f,  30.6906f,-51.2834f,-3.6174f,  31.3349f,-51.7455f,-4.9807f,  30.5685f,-50.2821f,-6.1090f,  31.1890f,-50.6510f,-7.4266f,  
	30.6217f,-49.0095f,-8.4195f,  31.2135f,-49.2904f,-9.6785f,  30.8547f,-47.5064f,-10.5098f,  31.4137f,-47.7056f,-11.7008f,  31.2698f,-45.8190f,-12.3507f,  
	31.7924f,-45.9461f,-13.4645f,  31.8674f,-43.9998f,-13.9211f,  32.3521f,-44.0646f,-14.9546f,  32.6473f,-42.1031f,-15.2139f,  33.0931f,-42.1162f,-16.1664f,  
	33.6072f,-40.1840f,-16.2326f,  34.0159f,-40.1552f,-17.1102f,  34.7450f,-38.2958f,-16.9956f,  35.1178f,-38.2350f,-17.8058f,  36.0550f,-36.4891f,-17.5307f,  
	36.3958f,-36.4034f,-18.2885f,  37.5314f,-34.8078f,-17.8803f,  37.8439f,-34.7013f,-18.6024f,  39.1650f,-33.2876f,-18.0941f,  39.4529f,-33.1611f,-18.8005f,  
	40.9433f,-31.9549f,-18.2297f,  41.2109f,-31.8050f,-18.9443f,  42.8512f,-30.8253f,-18.3512f,  43.1001f,-30.6454f,-19.0956f,  44.8674f,-29.9042f,-18.5217f,  
	45.0990f,-29.6829f,-19.3194f,  46.9672f,-29.1853f,-18.8052f,  47.1806f,-28.9081f,-19.6787f,  49.1206f,-28.6525f,-19.2624f,  49.3128f,-28.3028f,-20.2274f,  
	51.2929f,-28.2819f,-19.9430f,  51.4590f,-27.8410f,-21.0147f,  53.4455f,-28.0420f,-20.8909f,  53.5788f,-27.4906f,-22.0793f,  55.5367f,-27.8968f,-22.1380f,  
	55.6286f,-27.2161f,-23.4478f,  57.5224f,-27.8076f,-23.7040f,  57.5635f,-26.9810f,-25.1345f,  59.3580f,-27.7356f,-25.5955f,  59.3382f,-26.7473f,-27.1436f,  
	60.9993f,-27.6418f,-27.8089f,  60.9086f,-26.4805f,-29.4649f,  62.4044f,-27.4916f,-30.3265f,  62.2325f,-26.1480f,-32.0793f,  63.5342f,-27.2527f,-33.1221f,  
	63.2720f,-25.7217f,-34.9571f,  64.3543f,-26.8982f,-36.1598f,  63.9933f,-25.1778f,-38.0604f,  64.8356f,-26.4060f,-39.3962f,  64.3688f,-24.4981f,-41.3443f,  
	64.9551f,-25.7597f,-42.7818f,  64.3768f,-23.6691f,-44.7586f,  64.6968f,-24.9483f,-46.2631f,  64.0027f,-22.6822f,-48.2500f,  64.0516f,-23.9651f,-49.7842f,  
	63.2394f,-21.5348f,-51.7616f,  63.0183f,-22.8100f,-53.2871f,  62.0868f,-20.2278f,-55.2370f,  61.6026f,-21.4860f,-56.7150f,  60.5526f,-18.7667f,-58.6197f,  
	59.8178f,-20.0006f,-60.0124f,  58.6518f,-17.1614f,-61.8549f,  57.6844f,-18.3655f,-63.1263f,  56.4062f,-15.4244f,-64.8913f,  55.2291f,-16.5947f,-66.0079f,  
	53.8440f,-13.5707f,-67.6816f,  52.4847f,-14.7045f,-68.6137f,  50.9996f,-11.6180f,-70.1835f,  49.4894f,-12.7136f,-70.9053f,  47.9125f,-9.5855f,-72.3604f,  
	46.2859f,-10.6417f,-72.8511f,  44.6265f,-7.4932f,-74.1823f,  42.9207f,-8.5094f,-74.4262f,  41.1894f,-5.3621f,-75.6262f,  39.4429f,-6.3376f,-75.6133f,  
	37.6510f,-3.2124f,-76.6768f,  35.9033f,-4.1463f,-76.4026f,  34.0633f,-1.0644f,-77.3259f,  32.3535f,-1.9553f,-76.7920f,  30.4775f,1.0645f,-77.5740f,  
	28.8441f,0.2186f,-76.7875f,  26.9461f,3.1558f,-77.4281f,  25.4254f,2.3580f,-76.4018f,  23.5191f,5.1947f,-76.9033f,  22.1449f,4.4492f,-75.6552f,  
	20.2430f,7.1702f,-76.0217f,  19.0455f,6.4821f,-74.5744f,  17.1619f,9.0724f,-74.8116f,  16.1669f,8.4485f,-73.1921f,  14.3151f,10.8947f,-73.3071f,  
	13.5436f,10.3428f,-71.5457f,  11.7355f,12.6353f,-71.5473f,  31.6445f,-52.5381f,-2.3855f,  32.3028f,-53.0473f,-3.8101f,  31.3349f,-51.7455f,-4.9807f,  
	31.9713f,-52.1582f,-6.3691f,  31.1890f,-50.6510f,-7.4266f,  31.7980f,-50.9710f,-8.7641f,  31.2135f,-49.2904f,-9.6785f,  31.7905f,-49.5228f,-10.9539f,  
	31.4137f,-47.7056f,-11.7008f,  31.9545f,-47.8591f,-12.9022f,  31.7924f,-45.9461f,-13.4645f,  32.2947f,-46.0301f,-14.5862f,  32.3521f,-44.0646f,-14.9546f,  
	32.8142f,-44.0896f,-15.9932f,  33.0931f,-42.1162f,-16.1664f,  33.5160f,-42.0922f,-17.1251f,  34.0159f,-40.1552f,-17.1102f,  34.4003f,-40.0930f,-17.9934f,  
	35.1178f,-38.2350f,-17.8058f,  35.4670f,-38.1428f,-18.6259f,  36.3958f,-36.4034f,-18.2885f,  36.7133f,-36.2870f,-19.0600f,  37.8439f,-34.7013f,-18.6024f,  
	38.1335f,-34.5634f,-19.3435f,  39.4529f,-33.1611f,-18.8005f,  39.7190f,-33.0002f,-19.5341f,  41.2109f,-31.8050f,-18.9443f,  41.4557f,-31.6166f,-19.6912f,  
	43.1001f,-30.6454f,-19.0956f,  43.3256f,-30.4201f,-19.8796f,  45.0990f,-29.6829f,-19.3194f,  45.3052f,-29.4078f,-20.1636f,  47.1806f,-28.9081f,-19.6787f,  
	47.3652f,-28.5679f,-20.6002f,  49.3128f,-28.3028f,-20.2274f,  49.4719f,-27.8798f,-21.2432f,  51.4590f,-27.8410f,-21.0147f,  51.5867f,-27.3160f,-22.1373f,  
	53.5788f,-27.4906f,-22.0793f,  53.6672f,-26.8450f,-23.3155f,  55.6286f,-27.2161f,-23.4478f,  55.6690f,-26.4327f,-24.7995f,  57.5635f,-26.9810f,-25.1345f,  
	57.5461f,-26.0430f,-26.6012f,  59.3382f,-26.7473f,-27.1436f,  59.2529f,-25.6421f,-28.7178f,  60.9086f,-26.4805f,-29.4649f,  60.7452f,-25.1976f,-31.1376f,  
	62.2325f,-26.1480f,-32.0793f,  61.9817f,-24.6806f,-33.8375f,  63.2720f,-25.7217f,-34.9571f,  62.9249f,-24.0660f,-36.7859f,  63.9933f,-25.1778f,-38.0604f,  
	63.5424f,-23.3342f,-39.9426f,  64.3688f,-24.4981f,-41.3443f,  63.8077f,-22.4697f,-43.2621f,  64.3768f,-23.6691f,-44.7586f,  63.7006f,-21.4617f,-46.6940f,  
	64.0027f,-22.6822f,-48.2500f,  63.2083f,-20.3057f,-50.1835f,  63.2394f,-21.5348f,-51.7616f,  62.3250f,-19.0003f,-53.6753f,  62.0868f,-20.2278f,-55.2370f,  
	61.0524f,-17.5492f,-57.1130f,  60.5526f,-18.7667f,-58.6197f,  59.4000f,-15.9604f,-60.4407f,  58.6518f,-17.1614f,-61.8549f,  57.3842f,-14.2449f,-63.6051f,  
	56.4062f,-15.4244f,-64.8913f,  55.0283f,-12.4162f,-66.5562f,  53.8440f,-13.5707f,-67.6816f,  52.3621f,-10.4909f,-69.2480f,  50.9996f,-11.6180f,-70.1835f,  
	49.4214f,-8.4874f,-71.6396f,  47.9125f,-9.5855f,-72.3604f,  46.2470f,-6.4254f,-73.6964f,  44.6265f,-7.4932f,-74.1823f,  42.8841f,-4.3255f,-75.3900f,  
	41.1894f,-5.3621f,-75.6262f,  39.3811f,-2.2081f,-76.6995f,  37.6510f,-3.2124f,-76.6768f,  35.7892f,-0.0939f,-77.6111f,  34.0633f,-1.0644f,-77.3259f,  
	32.1600f,1.9993f,-78.1193f,  30.4775f,1.0645f,-77.5740f,  28.5471f,4.0519f,-78.2256f,  26.9461f,3.1558f,-77.4281f,  25.0024f,6.0483f,-77.9394f,  
	23.5191f,5.1947f,-76.9033f,  21.5752f,7.9761f,-77.2779f,  20.2430f,7.1702f,-76.0217f,  18.3133f,9.8241f,-76.2646f,  17.1619f,9.0724f,-74.8116f,  
	15.2605f,11.5843f,-74.9296f,  14.3151f,10.8947f,-73.3071f,  12.4547f,13.2537f,-73.3087f,  11.7355f,12.6353f,-71.5473f,  9.9283f,14.8324f,-71.4422f,  
	32.3028f,-53.0473f,-3.8101f,  32.9552f,-53.5053f,-5.2650f,  31.9713f,-52.1582f,-6.3691f,  32.5980f,-52.5195f,-7.7823f,  31.7980f,-50.9710f,-8.7641f,  
	32.3939f,-51.2396f,-10.1228f,  31.7905f,-49.5228f,-10.9539f,  32.3509f,-49.7061f,-12.2437f,  31.9545f,-47.8591f,-12.9022f,  32.4762f,-47.9659f,-14.1150f,  
	32.2947f,-46.0301f,-14.5862f,  32.7753f,-46.0705f,-15.7159f,  32.8142f,-44.0896f,-15.9932f,  33.2539f,-44.0736f,-17.0400f,  33.5160f,-42.0922f,-17.1251f,  
	33.9148f,-42.0313f,-18.0904f,  34.4003f,-40.0930f,-17.9934f,  34.7610f,-39.9964f,-18.8867f,  35.4670f,-38.1428f,-18.6259f,  35.7927f,-38.0177f,-19.4592f,  
	36.7133f,-36.2870f,-19.0600f,  37.0077f,-36.1378f,-19.8495f,  38.1335f,-34.5634f,-19.3435f,  38.4010f,-34.3905f,-20.1105f,  39.7190f,-33.0002f,-19.5341f,  
	39.9623f,-32.8012f,-20.2982f,  41.4557f,-31.6166f,-19.6912f,  41.6773f,-31.3842f,-20.4761f,  43.3256f,-30.4201f,-19.8796f,  43.5263f,-30.1431f,-20.7087f,  
	45.3052f,-29.4078f,-20.1636f,  45.4836f,-29.0726f,-21.0551f,  47.3652f,-28.5679f,-20.6002f,  47.5184f,-28.1577f,-21.5727f,  49.4719f,-27.8798f,-21.2432f,  
	49.5947f,-27.3760f,-22.3112f,  51.5867f,-27.3160f,-22.1373f,  51.6719f,-26.7004f,-23.3095f,  53.6672f,-26.8450f,-23.3155f,  53.7068f,-26.1001f,-24.5963f,  
	55.6690f,-26.4327f,-24.7995f,  55.6535f,-25.5410f,-26.1909f,  57.5461f,-26.0430f,-26.6012f,  57.4658f,-24.9909f,-28.0980f,  59.2529f,-25.6421f,-28.7178f,  
	59.0978f,-24.4178f,-30.3131f,  60.7452f,-25.1976f,-31.1376f,  60.5056f,-23.7926f,-32.8203f,  61.9817f,-24.6806f,-33.8375f,  61.6485f,-23.0897f,-35.5943f,  
	62.9249f,-24.0660f,-36.7859f,  62.4901f,-22.2880f,-38.6009f,  63.5424f,-23.3342f,-39.9426f,  62.9992f,-21.3705f,-41.7991f,  63.8077f,-22.4697f,-43.2621f,  
	63.1505f,-20.3246f,-45.1427f,  63.7006f,-21.4617f,-46.6940f,  62.9256f,-19.1438f,-48.5797f,  63.2083f,-20.3057f,-50.1835f,  62.3131f,-17.8247f,-52.0567f,  
	62.3250f,-19.0003f,-53.6753f,  61.3087f,-16.3692f,-55.5181f,  61.0524f,-17.5492f,-57.1130f,  59.9162f,-14.7832f,-58.9078f,  59.4000f,-15.9604f,-60.4407f,  
	58.1464f,-13.0764f,-62.1713f,  57.3842f,-14.2449f,-63.6051f,  56.0173f,-11.2607f,-65.2566f,  55.0283f,-12.4162f,-66.5562f,  53.5536f,-9.3515f,-68.1148f,  
	52.3621f,-10.4909f,-69.2480f,  50.7870f,-7.3664f,-70.7012f,  49.4214f,-8.4874f,-71.6396f,  47.7544f,-5.3244f,-72.9770f,  46.2470f,-6.4254f,-73.6964f,  
	44.4980f,-3.2457f,-74.9088f,  42.8841f,-4.3255f,-75.3900f,  41.0639f,-1.1508f,-76.4705f,  39.3811f,-2.2081f,-76.6995f,  37.5019f,0.9396f,-77.6427f,  
	35.7892f,-0.0939f,-77.6111f,  33.8629f,3.0069f,-78.4141f,  32.1600f,1.9993f,-78.1193f,  30.2010f,5.0310f,-78.7806f,  28.5471f,4.0519f,-78.2256f,  
	26.5693f,6.9951f,-78.7458f,  25.0024f,6.0483f,-77.9394f,  23.0193f,8.8858f,-78.3216f,  21.5752f,7.9761f,-77.2779f,  19.6015f,10.6906f,-77.5263f,  
	18.3133f,9.8241f,-76.2646f,  16.3637f,12.3999f,-76.3856f,  15.2605f,11.5843f,-74.9296f,  13.3483f,14.0093f,-74.9312f,  12.4547f,13.2537f,-73.3087f,  
	10.5928f,15.5176f,-73.2003f,  9.9283f,14.8324f,-71.4422f,  8.1301f,16.9260f,-71.2346f,  32.9552f,-53.5053f,-5.2650f,  33.5999f,-53.9095f,-6.7497f,  
	32.5980f,-52.5195f,-7.7823f,  33.2134f,-52.8265f,-9.2217f,  32.3939f,-51.2396f,-10.1228f,  32.9748f,-51.4555f,-11.5005f,  32.3509f,-49.7061f,-12.2437f,  
	32.8933f,-49.8388f,-13.5491f,  32.4762f,-47.9659f,-14.1150f,  32.9771f,-48.0248f,-15.3393f,  32.7753f,-46.0705f,-15.7159f,  33.2339f,-46.0656f,-16.8564f,  
	33.2539f,-44.0736f,-17.0400f,  33.6696f,-44.0167f,-18.0951f,  33.9148f,-42.0313f,-18.0904f,  34.2897f,-41.9323f,-19.0664f,  34.7610f,-39.9964f,-18.8867f,  
	35.0977f,-39.8636f,-19.7932f,  35.7927f,-38.0177f,-19.4592f,  36.0947f,-37.8573f,-20.3098f,  37.0077f,-36.1378f,-19.8495f,  37.2795f,-35.9520f,-20.6635f,  
	38.4010f,-34.3905f,-20.1105f,  38.6455f,-34.1790f,-20.9063f,  39.9623f,-32.8012f,-20.2982f,  40.1824f,-32.5588f,-21.0988f,  41.6773f,-31.3842f,-20.4761f,  
	41.8746f,-31.1015f,-21.3048f,  43.5263f,-30.1431f,-20.7087f,  43.7001f,-29.8082f,-21.5843f,  45.4836f,-29.0726f,-21.0551f,  45.6317f,-28.6699f,-21.9974f,  
	47.5184f,-28.1577f,-21.5727f,  47.6367f,-27.6697f,-22.5979f,  49.5947f,-27.3760f,-22.3112f,  49.6768f,-26.7851f,-23.4299f,  51.6719f,-26.7004f,-23.3095f,  
	51.7105f,-25.9889f,-24.5284f,  53.7068f,-26.1001f,-24.5963f,  53.6929f,-25.2499f,-25.9198f,  55.6535f,-25.5410f,-26.1909f,  55.5778f,-24.5379f,-27.6160f,  
	57.4658f,-24.9909f,-28.0980f,  57.3183f,-23.8218f,-29.6200f,  59.0978f,-24.4178f,-30.3131f,  58.8689f,-23.0731f,-31.9228f,  60.5056f,-23.7926f,-32.8203f,  
	60.1860f,-22.2656f,-34.5062f,  61.6485f,-23.0897f,-35.5943f,  61.2299f,-21.3775f,-37.3419f,  62.4901f,-22.2880f,-38.6009f,  61.9652f,-20.3904f,-40.3947f,  
	62.9992f,-21.3705f,-41.7991f,  62.3618f,-19.2904f,-43.6227f,  63.1505f,-20.3246f,-45.1427f,  62.3962f,-18.0689f,-46.9777f,  62.9256f,-19.1438f,-48.5797f,  
	62.0513f,-16.7210f,-50.4089f,  62.3131f,-17.8247f,-52.0567f,  61.3172f,-15.2466f,-53.8622f,  61.3087f,-16.3692f,-55.5181f,  60.1916f,-13.6499f,-57.2824f,  
	59.9162f,-14.7832f,-58.9078f,  58.6798f,-11.9388f,-60.6145f,  58.1464f,-13.0764f,-62.1713f,  56.7940f,-10.1240f,-63.8051f,  56.0173f,-11.2607f,-65.2566f,  
	54.5538f,-8.2193f,-66.8032f,  53.5536f,-9.3515f,-68.1148f,  51.9859f,-6.2417f,-69.5612f,  50.7870f,-7.3664f,-70.7012f,  49.1229f,-4.2089f,-72.0362f,  
	47.7544f,-5.3244f,-72.9770f,  46.0034f,-2.1412f,-74.1907f,  44.4980f,-3.2457f,-74.9088f,  42.6705f,-0.0584f,-75.9935f,  41.0639f,-1.1508f,-76.4705f,  
	39.1718f,2.0183f,-77.4200f,  37.5019f,0.9396f,-77.6427f,  35.5570f,4.0702f,-78.4534f,  33.8629f,3.0069f,-78.4141f,  31.8796f,6.0763f,-79.0836f,  
	30.2010f,5.0310f,-78.7806f,  28.1931f,8.0191f,-79.3088f,  26.5693f,6.9951f,-78.7458f,  24.5504f,9.8839f,-79.1349f,  23.0193f,8.8858f,-78.3216f,  
	21.0043f,11.6571f,-78.5751f,  19.6015f,10.6906f,-77.5263f,  17.6058f,13.3275f,-77.6502f,  16.3637f,12.3999f,-76.3856f,  14.4010f,14.8891f,-76.3872f,  
	13.3483f,14.0093f,-74.9312f,  11.4321f,16.3394f,-74.8197f,  10.5928f,15.5176f,-73.2003f,  8.7375f,17.6779f,-72.9861f,  8.1301f,16.9260f,-71.2346f,  
	6.3468f,18.9110f,-70.9294f,  33.5999f,-53.9095f,-6.7497f,  34.2351f,-54.2565f,-8.2661f,  33.2134f,-52.8265f,-9.2217f,  33.8155f,-53.0774f,-10.6851f,  
	32.9748f,-51.4555f,-11.5005f,  33.5390f,-51.6168f,-12.8981f,  32.8933f,-49.8388f,-13.5491f,  33.4160f,-49.9193f,-14.8697f,  32.9771f,-48.0248f,-15.3393f,  
	33.4565f,-48.0341f,-16.5775f,  33.2339f,-46.0656f,-16.8564f,  33.6688f,-46.0153f,-18.0074f,  33.6696f,-44.0167f,-18.0951f,  34.0613f,-43.9174f,-19.1621f,  
	34.2897f,-41.9323f,-19.0664f,  34.6403f,-41.7933f,-20.0561f,  35.0977f,-39.8636f,-19.7932f,  35.4103f,-39.6926f,-20.7165f,  36.0947f,-37.8573f,-20.3098f,  
	36.3737f,-37.6581f,-21.1839f,  37.2795f,-35.9520f,-20.6635f,  37.5277f,-35.7264f,-21.5051f,  38.6455f,-34.1790f,-20.9063f,  38.8664f,-33.9237f,-21.7370f,  
	40.1824f,-32.5588f,-21.0988f,  40.3781f,-32.2665f,-21.9416f,  41.8746f,-31.1015f,-21.3048f,  42.0452f,-30.7621f,-22.1787f,  43.7001f,-29.8082f,-21.5843f,  
	43.8443f,-29.4078f,-22.5100f,  45.6317f,-28.6699f,-21.9974f,  45.7463f,-28.1920f,-22.9926f,  47.6367f,-27.6697f,-22.5979f,  47.7160f,-27.0974f,-23.6747f,  
	49.6768f,-26.7851f,-23.4299f,  49.7143f,-26.1009f,-24.5970f,  51.7105f,-25.9889f,-24.5284f,  51.6979f,-25.1747f,-25.7925f,  53.6929f,-25.2499f,-25.9198f,  
	53.6212f,-24.2908f,-27.2801f,  55.5778f,-24.5379f,-27.6160f,  55.4373f,-23.4199f,-29.0700f,  57.3183f,-23.8218f,-29.6200f,  57.0995f,-22.5342f,-31.1604f,  
	58.8689f,-23.0731f,-31.9228f,  58.5624f,-21.6077f,-33.5401f,  60.1860f,-22.2656f,-34.5062f,  59.7833f,-20.6181f,-36.1874f,  61.2299f,-21.3775f,-37.3419f,  
	60.7233f,-19.5462f,-39.0729f,  61.9652f,-20.3904f,-40.3947f,  61.3480f,-18.3766f,-42.1600f,  62.3618f,-19.2904f,-43.6227f,  61.6291f,-17.0995f,-45.4049f,  
	62.3962f,-18.0689f,-46.9777f,  61.5440f,-15.7081f,-48.7600f,  62.0513f,-16.7210f,-50.4089f,  61.0775f,-14.2008f,-52.1737f,  61.3172f,-15.2466f,-53.8622f,  
	60.2215f,-12.5799f,-55.5923f,  60.1916f,-13.6499f,-57.2824f,  58.9751f,-10.8518f,-58.9612f,  58.6798f,-11.9388f,-60.6145f,  57.3451f,-9.0256f,-62.2266f,  
	56.7940f,-10.1240f,-63.8051f,  55.3454f,-7.1138f,-65.3358f,  54.5538f,-8.2193f,-66.8032f,  52.9975f,-5.1323f,-68.2390f,  51.9859f,-6.2417f,-69.5612f,  
	50.3291f,-3.0981f,-70.8902f,  49.1229f,-4.2089f,-72.0362f,  47.3746f,-1.0307f,-73.2480f,  46.0034f,-2.1412f,-74.1907f,  44.1735f,1.0504f,-75.2768f,  
	42.6705f,-0.0584f,-75.9935f,  40.7706f,3.1239f,-76.9470f,  39.1718f,2.0183f,-77.4200f,  37.2133f,5.1709f,-78.2365f,  35.5570f,4.0702f,-78.4534f,  
	33.5541f,7.1699f,-79.1295f,  31.8796f,6.0763f,-79.0836f,  29.8460f,9.1026f,-79.6185f,  28.1931f,8.0191f,-79.3088f,  26.1425f,10.9533f,-79.7039f,  
	24.5504f,9.8839f,-79.1349f,  22.4979f,12.7070f,-79.3931f,  21.0043f,11.6571f,-78.5751f,  18.9654f,14.3512f,-78.7016f,  17.6058f,13.3275f,-77.6502f,  
	15.5947f,15.8781f,-77.6518f,  14.4010f,14.8891f,-76.3872f,  12.4317f,17.2839f,-76.2725f,  11.4321f,16.3394f,-74.8197f,  9.5199f,18.5660f,-74.5989f,  
	8.7375f,17.6779f,-72.9861f,  6.8943f,19.7296f,-72.6707f,  6.3468f,18.9110f,-70.9294f,  4.5853f,20.7817f,-70.5319f,  34.2351f,-54.2565f,-8.2661f,  
	34.8586f,-54.5440f,-9.8116f,  33.8155f,-53.0774f,-10.6851f,  34.4023f,-53.2696f,-12.1732f,  33.5390f,-51.6168f,-12.8981f,  34.0847f,-51.7214f,-14.3154f,  
	33.4160f,-49.9193f,-14.8697f,  33.9178f,-49.9453f,-16.2079f,  33.4565f,-48.0341f,-16.5775f,  33.9126f,-47.9929f,-17.8289f,  33.6688f,-46.0153f,-18.0074f,  
	34.0796f,-45.9178f,-19.1720f,  34.0613f,-43.9174f,-19.1621f,  34.4282f,-43.7737f,-20.2436f,  34.6403f,-41.7933f,-20.0561f,  34.9661f,-41.6122f,-21.0626f,  
	35.4103f,-39.6926f,-20.7165f,  35.6990f,-39.4796f,-21.6626f,  36.3737f,-37.6581f,-21.1839f,  36.6282f,-37.4168f,-22.0842f,  37.5277f,-35.7264f,-21.5051f,  
	37.7518f,-35.4555f,-22.3798f,  38.8664f,-33.9237f,-21.7370f,  39.0626f,-33.6182f,-22.6083f,  40.3781f,-32.2665f,-21.9416f,  40.5472f,-31.9181f,-22.8283f,  
	42.0452f,-30.7621f,-22.1787f,  42.1867f,-30.3586f,-23.1019f,  43.8443f,-29.4078f,-22.5100f,  43.9558f,-28.9339f,-23.4884f,  45.7463f,-28.1920f,-22.9926f,  
	45.8231f,-27.6321f,-24.0395f,  47.7160f,-27.0974f,-23.6747f,  47.7523f,-26.4344f,-24.8008f,  49.7143f,-26.1009f,-24.5970f,  49.7025f,-25.3167f,-25.8111f,  
	51.6979f,-25.1747f,-25.7925f,  51.6295f,-24.2540f,-27.0961f,  53.6212f,-24.2908f,-27.2801f,  53.4870f,-23.2190f,-28.6725f,  55.4373f,-23.4199f,-29.0700f,  
	55.2279f,-22.1853f,-30.5461f,  57.0995f,-22.5342f,-31.1604f,  56.8056f,-21.1273f,-32.7126f,  58.5624f,-21.6077f,-33.5401f,  58.1752f,-20.0229f,-35.1570f,  
	59.7833f,-20.6181f,-36.1874f,  59.2948f,-18.8522f,-37.8564f,  60.7233f,-19.5462f,-39.0729f,  60.1265f,-17.5990f,-40.7797f,  61.3480f,-18.3766f,-42.1600f,  
	60.6373f,-16.2521f,-43.8881f,  61.6291f,-17.0995f,-45.4049f,  60.8000f,-14.8034f,-47.1382f,  61.5440f,-15.7081f,-48.7600f,  60.5939f,-13.2496f,-50.4815f,  
	61.0775f,-14.2008f,-52.1737f,  60.0052f,-11.5915f,-53.8665f,  60.2215f,-12.5799f,-55.5923f,  59.0275f,-9.8338f,-57.2399f,  58.9751f,-10.8518f,-58.9612f,  
	57.6613f,-7.9842f,-60.5480f,  57.3451f,-9.0256f,-62.2266f,  55.9150f,-6.0538f,-63.7377f,  55.3454f,-7.1138f,-65.3358f,  53.8045f,-4.0574f,-66.7573f,  
	52.9975f,-5.1323f,-68.2390f,  51.3523f,-2.0110f,-69.5586f,  50.3291f,-3.0981f,-70.8902f,  48.5882f,0.0667f,-72.0968f,  47.3746f,-1.0307f,-73.2480f,  
	45.5472f,2.1565f,-74.3326f,  44.1735f,1.0504f,-75.2768f,  42.2708f,4.2374f,-76.2317f,  40.7706f,3.1239f,-76.9470f,  38.8036f,6.2902f,-77.7670f,  
	37.2133f,5.1709f,-78.2365f,  35.1958f,8.2932f,-78.9174f,  33.5541f,7.1699f,-79.1295f,  31.4997f,10.2272f,-79.6699f,  29.8460f,9.1026f,-79.6185f,  
	27.7683f,12.0757f,-80.0188f,  26.1425f,10.9533f,-79.7039f,  24.0565f,13.8224f,-79.9663f,  22.4979f,12.7070f,-79.3931f,  20.4194f,15.4534f,-79.5220f,  
	18.9654f,14.3512f,-78.7016f,  16.9090f,16.9595f,-78.7032f,  15.5947f,15.8781f,-77.6518f,  13.5745f,18.3349f,-77.5341f,  12.4317f,17.2839f,-76.2725f,  
	10.4640f,19.5753f,-76.0454f,  9.5199f,18.5660f,-74.5989f,  7.6172f,20.6841f,-74.2733f,  6.8943f,19.7296f,-72.6707f,  5.0703f,21.6668f,-72.2592f,  
	4.5853f,20.7817f,-70.5319f,  2.8513f,22.5347f,-70.0479f,  34.8586f,-54.5440f,-9.8116f,  35.4682f,-54.7687f,-11.3869f,  34.4023f,-53.2696f,-12.1732f,  
	34.9715f,-53.4004f,-13.6856f,  34.0847f,-51.7214f,-14.3154f,  34.6100f,-51.7666f,-15.7542f,  33.9178f,-49.9453f,-16.2079f,  34.3966f,-49.9156f,-17.5625f,  
	33.9126f,-47.9929f,-17.8289f,  34.3444f,-47.8993f,-19.0960f,  34.0796f,-45.9178f,-19.1720f,  34.4652f,-45.7710f,-20.3524f,  34.4282f,-43.7737f,-20.2436f,  
	34.7696f,-43.5833f,-21.3423f,  34.9661f,-41.6122f,-21.0626f,  35.2670f,-41.3853f,-22.0915f,  35.6990f,-39.4796f,-21.6626f,  35.9623f,-39.2214f,-22.6339f,  
	36.6282f,-37.4168f,-22.0842f,  36.8577f,-37.1281f,-23.0163f,  37.7518f,-35.4555f,-22.3798f,  37.9505f,-35.1334f,-23.2934f,  39.0626f,-33.6182f,-22.6083f,  
	39.2318f,-33.2563f,-23.5218f,  40.5472f,-31.9181f,-22.8283f,  40.6872f,-31.5061f,-23.7628f,  42.1867f,-30.3586f,-23.1019f,  42.2959f,-29.8829f,-24.0769f,  
	43.9558f,-28.9339f,-23.4884f,  44.0305f,-28.3800f,-24.5182f,  45.8231f,-27.6321f,-24.0395f,  45.8581f,-26.9835f,-25.1365f,  47.7523f,-26.4344f,-24.8008f,  
	47.7409f,-25.6734f,-25.9755f,  49.7025f,-25.3167f,-25.8111f,  49.6368f,-24.4280f,-27.0670f,  51.6295f,-24.2540f,-27.0961f,  51.5007f,-23.2225f,-28.4345f,  
	53.4870f,-23.2190f,-28.6725f,  53.2860f,-22.0322f,-30.0905f,  55.2279f,-22.1853f,-30.5461f,  54.9455f,-20.8327f,-32.0378f,  56.8056f,-21.1273f,-32.7126f,  
	56.4330f,-19.6021f,-34.2683f,  58.1752f,-20.0229f,-35.1570f,  57.7042f,-18.3204f,-36.7659f,  59.2948f,-18.8522f,-37.8564f,  58.7181f,-16.9708f,-39.5054f,  
	60.1265f,-17.5990f,-40.7797f,  59.4380f,-15.5412f,-42.4534f,  60.6373f,-16.2521f,-43.8881f,  59.8320f,-14.0222f,-45.5714f,  60.8000f,-14.8034f,-47.1382f,  
	59.8747f,-12.4093f,-48.8146f,  60.5939f,-13.2496f,-50.4815f,  59.5467f,-10.7015f,-52.1345f,  60.0052f,-11.5915f,-53.8665f,  58.8358f,-8.9022f,-55.4799f,  
	59.0275f,-9.8338f,-57.2399f,  57.7371f,-7.0177f,-58.7980f,  57.6613f,-7.9842f,-60.5480f,  56.2527f,-5.0575f,-62.0361f,  55.9150f,-6.0538f,-63.7377f,  
	54.3931f,-3.0353f,-65.1414f,  53.8045f,-4.0574f,-66.7573f,  52.1752f,-0.9661f,-68.0641f,  51.3523f,-2.0110f,-69.5586f,  49.6233f,1.1323f,-70.7570f,  
	48.5882f,0.0667f,-72.0968f,  46.7681f,3.2411f,-73.1771f,  45.5472f,2.1565f,-74.3326f,  43.6467f,5.3398f,-75.2864f,  42.2708f,4.2374f,-76.2317f,  
	40.3005f,7.4090f,-77.0531f,  38.8036f,6.2902f,-77.7670f,  36.7768f,9.4269f,-78.4511f,  35.1958f,8.2932f,-78.9174f,  33.1259f,11.3737f,-79.4619f,  
	31.4997f,10.2272f,-79.6699f,  29.3999f,13.2319f,-80.0743f,  27.7683f,12.0757f,-80.0188f,  25.6537f,14.9843f,-80.2848f,  24.0565f,13.8224f,-79.9663f,  
	21.9429f,16.6154f,-80.0972f,  20.4194f,15.4534f,-79.5220f,  18.3214f,18.1146f,-79.5235f,  16.9090f,16.9595f,-78.7032f,  14.8411f,19.4744f,-78.5826f,  
	13.5745f,18.3349f,-77.5341f,  11.5537f,20.6883f,-77.3007f,  10.4640f,19.5753f,-76.0454f,  8.5031f,21.7583f,-75.7098f,  7.6172f,20.6841f,-74.2733f,  
	5.7309f,22.6875f,-73.8479f,  5.0703f,21.6668f,-72.2592f,  3.2709f,23.4857f,-71.7571f,  2.8513f,22.5347f,-70.0479f,  1.1510f,24.1663f,-69.4834f,  
	35.4682f,-54.7687f,-11.3869f,  36.0614f,-54.9275f,-12.9913f,  34.9715f,-53.4004f,-13.6856f,  35.5212f,-53.4668f,-15.2239f,  34.6100f,-51.7666f,-15.7542f,  
	35.1128f,-51.7506f,-17.2129f,  34.3966f,-49.9156f,-17.5625f,  34.8511f,-49.8279f,-18.9354f,  34.3444f,-47.8993f,-19.0960f,  34.7505f,-47.7508f,-20.3806f,  
	34.4652f,-45.7710f,-20.3524f,  34.8244f,-45.5722f,-21.5507f,  34.7696f,-43.5833f,-21.3423f,  35.0851f,-43.3425f,-22.4632f,  35.2670f,-41.3853f,-22.0915f,  
	35.5414f,-41.1093f,-23.1446f,  35.9623f,-39.2214f,-22.6339f,  36.1994f,-38.9129f,-23.6354f,  36.8577f,-37.1281f,-23.0163f,  37.0608f,-36.7863f,-23.9852f,  
	37.9505f,-35.1334f,-23.2934f,  38.1214f,-34.7539f,-24.2474f,  39.2318f,-33.2563f,-23.5218f,  39.3715f,-32.8308f,-24.4815f,  40.6872f,-31.5061f,-23.7628f,  
	40.7950f,-31.0225f,-24.7481f,  42.2959f,-29.8829f,-24.0769f,  42.3687f,-29.3283f,-25.1026f,  44.0305f,-28.3800f,-24.5182f,  44.0642f,-27.7389f,-25.5981f,  
	45.8581f,-26.9835f,-25.1365f,  45.8467f,-26.2388f,-26.2827f,  47.7409f,-25.6734f,-25.9755f,  47.6771f,-24.8099f,-27.1934f,  49.6368f,-24.4280f,-27.0670f,  
	49.5124f,-23.4303f,-28.3599f,  51.5007f,-23.2225f,-28.4345f,  51.3070f,-22.0777f,-29.8012f,  53.2860f,-22.0322f,-30.0905f,  53.0140f,-20.7288f,-31.5272f,  
	54.9455f,-20.8327f,-32.0378f,  54.5865f,-19.3629f,-33.5367f,  56.4330f,-19.6021f,-34.2683f,  55.9787f,-17.9599f,-35.8201f,  57.7042f,-18.3204f,-36.7659f,  
	57.1469f,-16.5029f,-38.3588f,  58.7181f,-16.9708f,-39.5054f,  58.0515f,-14.9791f,-41.1254f,  59.4380f,-15.5412f,-42.4534f,  58.6566f,-13.3781f,-44.0863f,  
	59.8320f,-14.0222f,-45.5714f,  58.9321f,-11.6943f,-47.2014f,  59.8747f,-12.4093f,-48.8146f,  58.8536f,-9.9253f,-50.4260f,  59.5467f,-10.7015f,-52.1345f,  
	58.4036f,-8.0731f,-53.7113f,  58.8358f,-8.9022f,-55.4799f,  57.5712f,-6.1425f,-57.0068f,  57.7371f,-7.0177f,-58.7980f,  56.3530f,-4.1419f,-60.2602f,  
	56.2527f,-5.0575f,-62.0361f,  54.7533f,-2.0836f,-63.4191f,  54.3931f,-3.0353f,-65.1414f,  52.7834f,0.0186f,-66.4325f,  52.1752f,-0.9661f,-68.0641f,  
	50.4623f,2.1477f,-69.2513f,  49.6233f,1.1323f,-70.7570f,  47.8153f,4.2855f,-71.8300f,  46.7681f,3.2411f,-73.1771f,  44.8749f,6.4121f,-74.1272f,  
	43.6467f,5.3398f,-75.2864f,  41.6784f,8.5081f,-76.1069f,  40.3005f,7.4090f,-77.0531f,  38.2699f,10.5515f,-77.7384f,  36.7768f,9.4269f,-78.4511f,  
	34.6968f,12.5223f,-78.9981f,  33.1259f,11.3737f,-79.4619f,  31.0095f,14.4021f,-79.8695f,  29.3999f,13.2319f,-80.0743f,  27.2619f,16.1726f,-80.3432f,  
	25.6537f,14.9843f,-80.2848f,  23.5097f,17.8173f,-80.4175f,  21.9429f,16.6154f,-80.0972f,  19.8077f,19.3237f,-80.0987f,  18.3214f,18.1146f,-79.5235f,  
	16.2097f,20.6828f,-79.4003f,  14.8411f,19.4744f,-78.5826f,  12.7702f,21.8862f,-78.3434f,  11.5537f,20.6883f,-77.3007f,  9.5368f,22.9337f,-76.9556f,  
	8.5031f,21.7583f,-75.7098f,  6.5558f,23.8265f,-75.2707f,  5.7309f,22.6875f,-73.8479f,  3.8664f,24.5723f,-73.3277f,  3.2709f,23.4857f,-71.7571f,  
	1.5024f,25.1825f,-71.1704f,  1.1510f,24.1663f,-69.4834f,  -0.5116f,25.6754f,-68.8446f,  36.0614f,-54.9275f,-12.9913f,  36.6357f,-55.0168f,-14.6262f,  
	35.5212f,-53.4668f,-15.2239f,  36.0487f,-53.4664f,-16.7861f,  35.1128f,-51.7506f,-17.2129f,  35.5910f,-51.6707f,-18.6929f,  34.8511f,-49.8279f,-18.9354f,  
	35.2793f,-49.6795f,-20.3278f,  34.7505f,-47.7508f,-20.3806f,  35.1294f,-47.5449f,-21.6843f,  34.8244f,-45.5722f,-21.5507f,  35.1564f,-45.3179f,-22.7714f,  
	35.0851f,-43.3425f,-22.4632f,  35.3727f,-43.0481f,-23.6077f,  35.5414f,-41.1093f,-23.1446f,  35.7882f,-40.7793f,-24.2267f,  36.1994f,-38.9129f,-23.6354f,  
	36.4089f,-38.5485f,-24.6719f,  37.0608f,-36.7863f,-23.9852f,  37.2352f,-36.3852f,-24.9927f,  38.1214f,-34.7539f,-24.2474f,  38.2622f,-34.3098f,-25.2457f,  
	39.3715f,-32.8308f,-24.4815f,  39.4787f,-32.3334f,-25.4904f,  40.7950f,-31.0225f,-24.7481f,  40.8664f,-30.4607f,-25.7827f,  42.3687f,-29.3283f,-25.1026f,  
	42.4011f,-28.6876f,-26.1780f,  44.0642f,-27.7389f,-25.5981f,  44.0522f,-27.0032f,-26.7275f,  45.8467f,-26.2388f,-26.2827f,  45.7840f,-25.3931f,-27.4732f,  
	47.6771f,-24.8099f,-27.1934f,  47.5561f,-23.8390f,-28.4501f,  49.5124f,-23.4303f,-28.3599f,  49.3248f,-22.3208f,-29.6834f,  51.3070f,-22.0777f,-29.8012f,  
	51.0441f,-20.8175f,-31.1897f,  53.0140f,-20.7288f,-31.5272f,  52.6674f,-19.3093f,-32.9745f,  54.5865f,-19.3629f,-33.5367f,  54.1477f,-17.7767f,-35.0352f,  
	55.9787f,-17.9599f,-35.8201f,  55.4399f,-16.2032f,-37.3596f,  57.1469f,-16.5029f,-38.3588f,  56.5017f,-14.5752f,-39.9266f,  58.0515f,-14.9791f,-41.1254f,  
	57.2939f,-12.8820f,-42.7083f,  58.6566f,-13.3781f,-44.0863f,  57.7822f,-11.1168f,-45.6696f,  58.9321f,-11.6943f,-47.2014f,  57.9379f,-9.2761f,-48.7700f,  
	58.8536f,-9.9253f,-50.4260f,  57.7380f,-7.3606f,-51.9645f,  58.4036f,-8.0731f,-53.7113f,  57.1665f,-5.3738f,-55.2047f,  57.5712f,-6.1425f,-57.0068f,  
	56.2139f,-3.3226f,-58.4405f,  56.3530f,-4.1419f,-60.2602f,  54.8789f,-1.2184f,-61.6197f,  54.7533f,-2.0836f,-63.4191f,  53.1668f,0.9264f,-64.6914f,  
	52.7834f,0.0186f,-66.4325f,  51.0908f,3.0956f,-67.6055f,  50.4623f,2.1477f,-69.2513f,  48.6709f,5.2720f,-70.3144f,  47.8153f,4.2855f,-71.8300f,  
	45.9343f,7.4361f,-72.7740f,  44.8749f,6.4121f,-74.1272f,  42.9139f,9.5687f,-74.9446f,  41.6784f,8.5081f,-76.1069f,  39.6496f,11.6480f,-76.7916f,  
	38.2699f,10.5515f,-77.7384f,  36.1855f,13.6535f,-78.2866f,  34.6968f,12.5223f,-78.9981f,  32.5695f,15.5663f,-79.4078f,  31.0095f,14.4021f,-79.8695f,  
	28.8538f,17.3672f,-80.1405f,  27.2619f,16.1726f,-80.3432f,  25.0932f,19.0384f,-80.4774f,  23.5097f,17.8173f,-80.4175f,  21.3426f,20.5661f,-80.4190f,  
	19.8077f,19.3237f,-80.0987f,  17.6570f,21.9396f,-79.9732f,  16.2097f,20.6828f,-79.4003f,  14.0928f,23.1484f,-79.1558f,  12.7702f,21.8862f,-78.3434f,  
	10.7007f,24.1902f,-77.9893f,  9.5368f,22.9337f,-76.9556f,  7.5310f,25.0642f,-76.5033f,  6.5558f,23.8265f,-75.2707f,  4.6274f,25.7758f,-74.7329f,  
	3.8664f,24.5723f,-73.3277f,  2.0298f,26.3342f,-72.7188f,  1.5024f,25.1825f,-71.1704f,  -0.2312f,26.7558f,-70.5048f,  -0.5116f,25.6754f,-68.8446f,  
	-2.1310f,27.0605f,-68.1380f,  36.6357f,-55.0168f,-14.6262f,  37.1883f,-55.0336f,-16.2892f,  36.0487f,-53.4664f,-16.7861f,  36.5516f,-53.3961f,-18.3727f,  
	35.5910f,-51.6707f,-18.6929f,  36.0426f,-51.5239f,-20.1948f,  35.2793f,-49.6795f,-20.3278f,  35.6794f,-49.4675f,-21.7408f,  35.1294f,-47.5449f,-21.6843f,  
	35.4797f,-47.2776f,-23.0108f,  35.1564f,-45.3179f,-22.7714f,  35.4591f,-45.0048f,-24.0153f,  35.3727f,-43.0481f,-23.6077f,  35.6311f,-42.6954f,-24.7800f,  
	35.7882f,-40.7793f,-24.2267f,  36.0058f,-40.3900f,-25.3418f,  36.4089f,-38.5485f,-24.6719f,  36.5881f,-38.1224f,-25.7449f,  37.2352f,-36.3852f,-24.9927f,  
	37.3782f,-35.9179f,-26.0423f,  38.2622f,-34.3098f,-25.2457f,  38.3696f,-33.7932f,-26.2911f,  39.4787f,-32.3334f,-25.4904f,  39.5491f,-31.7580f,-26.5469f,  
	40.8664f,-30.4607f,-25.7827f,  40.8974f,-29.8135f,-26.8660f,  42.4011f,-28.6876f,-26.1780f,  42.3883f,-27.9534f,-27.3023f,  44.0522f,-27.0032f,-26.7275f,  
	43.9899f,-26.1677f,-27.9015f,  45.7840f,-25.3931f,-27.4732f,  45.6653f,-24.4414f,-28.7034f,  47.5561f,-23.8390f,-28.4501f,  47.3733f,-22.7576f,-29.7390f,  
	49.3248f,-22.3208f,-29.6834f,  49.0696f,-21.0972f,-31.0309f,  51.0441f,-20.8175f,-31.1897f,  50.7084f,-19.4422f,-32.5914f,  52.6674f,-19.3093f,-32.9745f,  
	52.2426f,-17.7742f,-34.4246f,  54.1477f,-17.7767f,-35.0352f,  53.6263f,-16.0768f,-36.5248f,  55.4399f,-16.2032f,-37.3596f,  54.8150f,-14.3366f,-38.8776f,  
	56.5017f,-14.5752f,-39.9266f,  55.7671f,-12.5424f,-41.4610f,  57.2939f,-12.8820f,-42.7083f,  56.4448f,-10.6867f,-44.2453f,  57.7822f,-11.1168f,-45.6696f,  
	56.8151f,-8.7650f,-47.1951f,  57.9379f,-9.2761f,-48.7700f,  56.8506f,-6.7769f,-50.2692f,  57.7380f,-7.3606f,-51.9645f,  56.5298f,-4.7246f,-53.4228f,  
	57.1665f,-5.3738f,-55.2047f,  55.8379f,-2.6138f,-56.6079f,  56.2139f,-3.3226f,-58.4405f,  54.7677f,-0.4546f,-59.7741f,  54.8789f,-1.2184f,-61.6197f,  
	53.3186f,1.7416f,-62.8709f,  53.1668f,0.9264f,-64.6914f,  51.4980f,3.9599f,-65.8479f,  51.0908f,3.0956f,-67.6055f,  49.3202f,6.1837f,-68.6563f,  
	48.6709f,5.2720f,-70.3144f,  46.8068f,8.3943f,-71.2499f,  45.9343f,7.4361f,-72.7740f,  43.9857f,10.5727f,-73.5862f,  42.9139f,9.5687f,-74.9446f,  
	40.8922f,12.6974f,-75.6269f,  39.6496f,11.6480f,-76.7916f,  37.5665f,14.7479f,-77.3393f,  36.1855f,13.6535f,-78.2866f,  34.0532f,16.7045f,-78.6971f,  
	32.5695f,15.5663f,-79.4078f,  30.4020f,18.5475f,-79.6802f,  28.8538f,17.3672f,-80.1405f,  26.6661f,20.2580f,-80.2758f,  25.0932f,19.0384f,-80.4774f,  
	22.8998f,21.8207f,-80.4787f,  21.3426f,20.5661f,-80.4190f,  19.1580f,23.2232f,-80.2913f,  17.6570f,21.9396f,-79.9732f,  15.4988f,24.4532f,-79.7238f,  
	14.0928f,23.1484f,-79.1558f,  11.9748f,25.5064f,-78.7933f,  10.7007f,24.1902f,-77.9893f,  8.6396f,26.3794f,-77.5246f,  7.5310f,25.0642f,-76.5033f,  
	5.5411f,27.0756f,-75.9486f,  4.6274f,25.7758f,-74.7329f,  2.7241f,27.6016f,-74.1022f,  2.0298f,26.3342f,-72.7188f,  0.2252f,27.9716f,-72.0266f,  
	-0.2312f,26.7558f,-70.5048f,  -1.9245f,28.2034f,-69.7669f,  -2.1310f,27.0605f,-68.1380f,  -3.7038f,28.3218f,-67.3699f,  37.1883f,-55.0336f,-16.2892f,  
	37.7162f,-54.9745f,-17.9802f,  36.5516f,-53.3961f,-18.3727f,  37.0272f,-53.2525f,-19.9840f,  36.0426f,-51.5239f,-20.1948f,  36.4650f,-51.3072f,-21.7192f,  
	35.6794f,-49.4675f,-21.7408f,  36.0494f,-49.1880f,-23.1773f,  35.4797f,-47.2776f,-23.0108f,  35.7991f,-46.9458f,-24.3603f,  35.4591f,-45.0048f,-24.0153f,  
	35.7308f,-44.6283f,-25.2858f,  35.6311f,-42.6954f,-24.7800f,  35.8584f,-42.2791f,-25.9833f,  36.0058f,-40.3900f,-25.3418f,  36.1914f,-39.9356f,-26.4912f,  
	36.5881f,-38.1224f,-25.7449f,  36.7345f,-37.6276f,-26.8576f,  37.3782f,-35.9179f,-26.0423f,  37.4866f,-35.3767f,-27.1366f,  38.3696f,-33.7932f,-26.2911f,  
	38.4394f,-33.1979f,-27.3820f,  39.5491f,-31.7580f,-26.5469f,  39.5787f,-31.0972f,-27.6505f,  40.8974f,-29.8135f,-26.8660f,  40.8832f,-29.0733f,-27.9972f,  
	42.3883f,-27.9534f,-27.3023f,  42.3255f,-27.1203f,-28.4710f,  43.9899f,-26.1677f,-27.9015f,  43.8724f,-25.2273f,-29.1156f,  45.6653f,-24.4414f,-28.7034f,  
	45.4859f,-23.3804f,-29.9670f,  47.3733f,-22.7576f,-29.7390f,  47.1243f,-21.5632f,-31.0536f,  49.0696f,-21.0972f,-31.0309f,  48.7430f,-19.7595f,-32.3940f,  
	50.7084f,-19.4422f,-32.5914f,  50.2961f,-17.9522f,-33.9987f,  52.2426f,-17.7742f,-34.4246f,  51.7370f,-16.1259f,-35.8689f,  53.6263f,-16.0768f,-36.5248f,  
	53.0203f,-14.2673f,-37.9965f,  54.8150f,-14.3366f,-38.8776f,  54.1024f,-12.3651f,-40.3658f,  55.7671f,-12.5424f,-41.4610f,  54.9427f,-10.4114f,-42.9530f,  
	56.4448f,-10.6867f,-44.2453f,  55.5046f,-8.4008f,-45.7281f,  56.8151f,-8.7650f,-47.1951f,  55.7565f,-6.3319f,-48.6546f,  56.8506f,-6.7769f,-50.2692f,  
	55.6721f,-4.2060f,-51.6914f,  56.5298f,-4.7246f,-53.4228f,  55.2314f,-2.0275f,-54.7939f,  55.8379f,-2.6138f,-56.6079f,  54.4217f,0.1947f,-57.9137f,  
	54.7677f,-0.4546f,-59.7741f,  53.2364f,2.4503f,-61.0019f,  53.3186f,1.7416f,-62.8709f,  51.6771f,4.7256f,-64.0084f,  51.4980f,3.9599f,-65.8479f,  
	49.7521f,7.0049f,-66.8839f,  49.3202f,6.1837f,-68.6563f,  47.4774f,9.2701f,-69.5810f,  46.8068f,8.3943f,-71.2499f,  44.8755f,11.5028f,-72.0548f,  
	43.9857f,10.5727f,-73.5862f,  41.9766f,13.6819f,-74.2641f,  40.8922f,12.6974f,-75.6269f,  38.8162f,15.7867f,-76.1727f,  37.5665f,14.7479f,-77.3393f,  
	35.4353f,17.7974f,-77.7496f,  34.0532f,16.7045f,-78.6971f,  31.8802f,19.6934f,-78.9701f,  30.4020f,18.5475f,-79.6802f,  28.2016f,21.4551f,-79.8161f,  
	26.6661f,20.2580f,-80.2758f,  24.4525f,23.0658f,-80.2770f,  22.8998f,21.8207f,-80.4787f,  20.6873f,24.5117f,-80.3494f,  19.1580f,23.2232f,-80.2913f,  
	16.9642f,25.7784f,-80.0377f,  15.4988f,24.4532f,-79.7238f,  13.3374f,26.8597f,-79.3538f,  11.9748f,25.5064f,-78.7933f,  9.8628f,27.7498f,-78.3172f,  
	8.6396f,26.3794f,-77.5246f,  6.5917f,28.4495f,-76.9539f,  5.5411f,27.0756f,-75.9486f,  3.5735f,28.9629f,-75.2969f,  2.7241f,27.6016f,-74.1022f,  
	0.8498f,29.3019f,-73.3838f,  0.2252f,27.9716f,-72.0266f,  -1.5420f,29.4818f,-71.2573f,  -1.9245f,28.2034f,-69.7669f,  -3.5741f,29.5254f,-68.9626f,  
	-3.7038f,28.3218f,-67.3699f,  -5.2260f,29.4598f,-66.5470f,  37.7162f,-54.9745f,-17.9802f,  38.2163f,-54.8358f,-19.6990f,  37.0272f,-53.2525f,-19.9840f,  
	37.4727f,-53.0325f,-21.6200f,  36.4650f,-51.3072f,-21.7192f,  36.8559f,-51.0165f,-23.2680f,  36.0494f,-49.1880f,-23.1773f,  36.3869f,-48.8379f,-24.6368f,  
	35.7991f,-46.9458f,-24.3603f,  36.0855f,-46.5451f,-25.7354f,  35.7308f,-44.6283f,-25.2858f,  35.9692f,-44.1836f,-26.5855f,  35.8584f,-42.2791f,-25.9833f,  
	36.0517f,-41.7939f,-27.2187f,  36.1914f,-39.9356f,-26.4912f,  36.3424f,-39.4097f,-27.6777f,  36.7345f,-37.6276f,-26.8576f,  36.8448f,-37.0570f,-28.0123f,  
	37.4866f,-35.3767f,-27.1366f,  37.5564f,-34.7557f,-28.2739f,  38.4394f,-33.1979f,-27.3820f,  38.4677f,-32.5168f,-28.5180f,  39.5787f,-31.0972f,-27.6505f,  
	39.5629f,-30.3436f,-28.8004f,  40.8832f,-29.0733f,-27.9972f,  40.8193f,-28.2348f,-29.1719f,  42.3255f,-27.1203f,-28.4710f,  42.2080f,-26.1832f,-29.6796f,  
	43.8724f,-25.2273f,-29.1156f,  43.6950f,-24.1786f,-30.3635f,  45.4859f,-23.3804f,-29.9670f,  45.2413f,-22.2075f,-31.2574f,  47.1243f,-21.5632f,-31.0536f,  
	46.8050f,-20.2556f,-32.3856f,  48.7430f,-19.7595f,-32.3940f,  48.3413f,-18.3078f,-33.7649f,  50.2961f,-17.9522f,-33.9987f,  49.8044f,-16.3496f,-35.4029f,  
	51.7370f,-16.1259f,-35.8689f,  51.1484f,-14.3685f,-37.2981f,  53.0203f,-14.2673f,-37.9965f,  52.3283f,-12.3531f,-39.4413f,  54.1024f,-12.3651f,-40.3658f,  
	53.3017f,-10.2955f,-41.8148f,  54.9427f,-10.4114f,-42.9530f,  54.0288f,-8.1897f,-44.3942f,  55.5046f,-8.4008f,-45.7281f,  54.4744f,-6.0333f,-47.1482f,  
	55.7565f,-6.3319f,-48.6546f,  54.6081f,-3.8269f,-50.0403f,  55.6721f,-4.2060f,-51.6914f,  54.4049f,-1.5738f,-53.0295f,  55.2314f,-2.0275f,-54.7939f,  
	53.8467f,0.7184f,-56.0706f,  54.4217f,0.1947f,-57.9137f,  52.9215f,3.0405f,-59.1165f,  53.2364f,2.4503f,-61.0019f,  51.6248f,5.3798f,-62.1185f,  
	51.6771f,4.7256f,-64.0084f,  49.9592f,7.7215f,-65.0277f,  49.7521f,7.0049f,-66.8839f,  47.9347f,10.0487f,-67.7958f,  47.4774f,9.2701f,-69.5810f,  
	45.5681f,12.3432f,-70.3767f,  44.8755f,11.5028f,-72.0548f,  42.8841f,14.5845f,-72.7267f,  41.9766f,13.6819f,-74.2641f,  39.9133f,16.7522f,-74.8065f,  
	38.8162f,15.7867f,-76.1727f,  36.6920f,18.8261f,-76.5815f,  35.4353f,17.7974f,-77.7496f,  33.2629f,20.7852f,-78.0225f,  31.8802f,19.6934f,-78.9701f,  
	29.6735f,22.6092f,-79.1064f,  28.2016f,21.4551f,-79.8161f,  25.9744f,24.2802f,-79.8173f,  24.4525f,23.0658f,-80.2770f,  22.2186f,25.7829f,-80.1463f,  
	20.6873f,24.5117f,-80.3494f,  18.4638f,27.1013f,-80.0923f,  16.9642f,25.7784f,-80.0377f,  14.7651f,28.2269f,-79.6613f,  13.3374f,26.8597f,-79.3538f,  
	11.1796f,29.1517f,-78.8675f,  9.8628f,27.7498f,-78.3172f,  7.7613f,29.8740f,-77.7318f,  6.5917f,28.4495f,-76.9539f,  4.5634f,30.3950f,-76.2825f,  
	3.5735f,28.9629f,-75.2969f,  1.6320f,30.7240f,-74.5534f,  0.8498f,29.3019f,-73.3838f,  -0.9901f,30.8738f,-72.5839f,  -1.5420f,29.4818f,-71.2573f,  
	-3.2683f,30.8645f,-70.4171f,  -3.5741f,29.5254f,-68.9626f,  -5.1756f,30.7213f,-68.0988f,  -5.2260f,29.4598f,-66.5470f,  -6.6957f,30.4765f,-65.6751f,  
	38.2163f,-54.8358f,-19.6990f,  38.6854f,-54.6138f,-21.4447f,  37.4727f,-53.0325f,-21.6200f,  37.8853f,-52.7319f,-23.2814f,  36.8559f,-51.0165f,-23.2680f,  
	37.2124f,-50.6488f,-24.8402f,  36.3869f,-48.8379f,-24.6368f,  36.6893f,-48.4129f,-26.1213f,  36.0855f,-46.5451f,-25.7354f,  36.3365f,-46.0710f,-27.1380f,  
	35.9692f,-44.1836f,-26.5855f,  36.1716f,-43.6655f,-27.9152f,  36.0517f,-41.7939f,-27.2187f,  36.2083f,-41.2336f,-28.4887f,  36.3424f,-39.4097f,-27.6777f,  
	36.4554f,-38.8053f,-28.9034f,  36.8448f,-37.0570f,-28.0123f,  36.9150f,-36.4049f,-29.2072f,  37.5564f,-34.7557f,-28.2739f,  37.5834f,-34.0479f,-29.4537f,  
	38.4677f,-32.5168f,-28.5180f,  38.4499f,-31.7427f,-29.6981f,  39.5629f,-30.3436f,-28.8004f,  39.4970f,-29.4919f,-29.9924f,  40.8193f,-28.2348f,-29.1719f,  
	40.7006f,-27.2929f,-30.3857f,  42.2080f,-26.1832f,-29.6796f,  42.0310f,-25.1387f,-30.9219f,  43.6950f,-24.1786f,-30.3635f,  43.4530f,-23.0190f,-31.6389f,  
	45.2413f,-22.2075f,-31.2574f,  44.9275f,-20.9222f,-32.5664f,  46.8050f,-20.2556f,-32.3856f,  46.4119f,-18.8349f,-33.7272f,  48.3413f,-18.3078f,-33.7649f,  
	47.8616f,-16.7442f,-35.1349f,  49.8044f,-16.3496f,-35.4029f,  49.2312f,-14.6384f,-36.7946f,  51.1484f,-14.3685f,-37.2981f,  50.4752f,-12.5067f,-38.7035f,  
	52.3283f,-12.3531f,-39.4413f,  51.5497f,-10.3411f,-40.8501f,  53.3017f,-10.2955f,-41.8148f,  52.4129f,-8.1352f,-43.2161f,  54.0288f,-8.1897f,-44.3942f,  
	53.0263f,-5.8863f,-45.7758f,  54.4744f,-6.0333f,-47.1482f,  53.3560f,-3.5939f,-48.4976f,  54.6081f,-3.8269f,-50.0403f,  53.3724f,-1.2604f,-51.3449f,  
	54.4049f,-1.5738f,-53.0295f,  53.0526f,1.1076f,-54.2761f,  53.8467f,0.7184f,-56.0706f,  52.3793f,3.5019f,-57.2470f,  52.9215f,3.0405f,-59.1165f,  
	51.3422f,5.9111f,-60.2107f,  51.6248f,5.3798f,-62.1185f,  49.9380f,8.3215f,-63.1194f,  49.9592f,7.7215f,-65.0277f,  48.1708f,10.7168f,-65.9250f,  
	47.9347f,10.0487f,-67.7958f,  46.0517f,13.0795f,-68.5805f,  45.5681f,12.3432f,-70.3767f,  43.5993f,15.3900f,-71.0409f,  42.8841f,14.5845f,-72.7267f,  
	40.8389f,17.6279f,-73.2643f,  39.9133f,16.7522f,-74.8065f,  37.8020f,19.7731f,-75.2128f,  36.6920f,18.8261f,-76.5815f,  34.5266f,21.8043f,-76.8534f,  
	33.2629f,20.7852f,-78.0225f,  31.0564f,23.7006f,-78.1586f,  29.6735f,22.6092f,-79.1064f,  27.4392f,25.4431f,-79.1074f,  25.9744f,24.2802f,-79.8173f,  
	23.7256f,27.0152f,-79.6856f,  22.2186f,25.7829f,-80.1463f,  19.9722f,28.3993f,-79.8865f,  18.4638f,27.1013f,-80.0923f,  16.2334f,29.5847f,-79.7105f,  
	14.7651f,28.2269f,-79.6613f,  12.5674f,30.5612f,-79.1660f,  11.1796f,29.1517f,-78.8675f,  9.0299f,31.3246f,-78.2688f,  7.7613f,29.8740f,-77.7318f,  
	5.6769f,31.8733f,-77.0421f,  4.5634f,30.3950f,-76.2825f,  2.5583f,32.2136f,-75.5153f,  1.6320f,30.7240f,-74.5534f,  -0.2780f,32.3555f,-73.7240f,  
	-0.9901f,30.8738f,-72.5839f,  -2.7919f,32.3163f,-71.7083f,  -3.2683f,30.8645f,-70.4171f,  -4.9492f,32.1187f,-69.5123f,  -5.1756f,30.7213f,-68.0988f,  
	-6.7269f,31.7931f,-67.1810f,  -6.6957f,30.4765f,-65.6751f,  -8.1092f,31.3735f,-64.7609f,  38.6854f,-54.6138f,-21.4447f,  39.1199f,-54.3046f,-23.2173f,  
	37.8853f,-52.7319f,-23.2814f,  38.2616f,-52.3474f,-24.9669f,  37.2124f,-50.6488f,-24.8402f,  37.5317f,-50.1998f,-26.4368f,  36.6893f,-48.4129f,-26.1213f,  
	36.9538f,-47.9087f,-27.6316f,  36.3365f,-46.0710f,-27.1380f,  36.5488f,-45.5183f,-28.5685f,  36.1716f,-43.6655f,-27.9152f,  36.3348f,-43.0681f,-29.2765f,  
	36.2083f,-41.2336f,-28.4887f,  36.3248f,-40.5916f,-29.7944f,  36.4554f,-38.8053f,-28.9034f,  36.5263f,-38.1172f,-30.1659f,  36.9150f,-36.4049f,-29.2072f,  
	36.9409f,-35.6645f,-30.4416f,  37.5834f,-34.0479f,-29.4537f,  37.5632f,-33.2466f,-30.6750f,  38.4499f,-31.7427f,-29.6981f,  38.3813f,-30.8703f,-30.9183f,  
	39.4970f,-29.4919f,-29.9924f,  39.3761f,-28.5372f,-31.2219f,  40.7006f,-27.2929f,-30.3857f,  40.5226f,-26.2441f,-31.6324f,  42.0310f,-25.1387f,-30.9219f,  
	41.7899f,-23.9838f,-32.1914f,  43.4530f,-23.0190f,-31.6389f,  43.1425f,-21.7477f,-32.9333f,  44.9275f,-20.9222f,-32.5664f,  44.5407f,-19.5244f,-33.8862f,  
	46.4119f,-18.8349f,-33.7272f,  45.9418f,-17.3029f,-35.0694f,  47.8616f,-16.7442f,-35.1349f,  47.3015f,-15.0725f,-36.4945f,  49.2312f,-14.6384f,-36.7946f,  
	48.5747f,-12.8232f,-38.1649f,  50.4752f,-12.5067f,-38.7035f,  49.7169f,-10.5474f,-40.0754f,  51.5497f,-10.3411f,-40.8501f,  50.6845f,-8.2384f,-42.2140f,  
	52.4129f,-8.1352f,-43.2161f,  51.4370f,-5.8932f,-44.5609f,  53.0263f,-5.8863f,-45.7758f,  51.9371f,-3.5108f,-47.0898f,  53.3560f,-3.5939f,-48.4976f,  
	52.1515f,-1.0926f,-49.7690f,  53.3724f,-1.2604f,-51.3449f,  52.0530f,1.3555f,-52.5611f,  53.0526f,1.1076f,-54.2761f,  51.6190f,3.8268f,-55.4253f,  
	52.3793f,3.5019f,-57.2470f,  50.8340f,6.3106f,-58.3175f,  51.3422f,5.9111f,-60.2107f,  49.6889f,8.7943f,-61.1916f,  49.9380f,8.3215f,-63.1194f,  
	48.1818f,11.2628f,-64.0004f,  48.1708f,10.7168f,-65.9250f,  46.3179f,13.6991f,-66.6971f,  46.0517f,13.0795f,-68.5805f,  44.1099f,16.0844f,-69.2356f,  
	43.5993f,15.3900f,-71.0409f,  41.5772f,18.3988f,-71.5723f,  40.8389f,17.6279f,-73.2643f,  38.7461f,20.6222f,-73.6669f,  37.8020f,19.7731f,-75.2128f,  
	35.6496f,22.7332f,-75.4830f,  34.5266f,21.8043f,-76.8534f,  32.3269f,24.7107f,-76.9890f,  31.0564f,23.7006f,-78.1586f,  28.8220f,26.5347f,-78.1595f,  
	27.4392f,25.4431f,-79.1074f,  25.1827f,28.1875f,-78.9752f,  23.7256f,27.0152f,-79.6856f,  21.4633f,29.6501f,-79.4238f,  19.9722f,28.3993f,-79.8865f,  
	17.7173f,30.9098f,-79.5005f,  16.2334f,29.5847f,-79.7105f,  14.0026f,31.9543f,-79.2077f,  12.5674f,30.5612f,-79.1660f,  10.3758f,32.7766f,-78.5558f,  
	9.0299f,31.3246f,-78.2688f,  6.8949f,33.3724f,-77.5628f,  5.6769f,31.8733f,-77.0421f,  3.6129f,33.7451f,-76.2530f,  2.5583f,32.2136f,-75.5153f,  
	0.5819f,33.9016f,-74.6580f,  -0.2780f,32.3555f,-73.7240f,  -2.1528f,33.8560f,-72.8143f,  -2.7919f,32.3163f,-71.7083f,  -4.5510f,33.6281f,-70.7635f,  
	-4.9492f,32.1187f,-69.5123f,  -6.5826f,33.2458f,-68.5488f,  -6.7269f,31.7931f,-67.1810f,  -8.2243f,32.7414f,-66.2161f,  -8.1092f,31.3735f,-64.7609f,  
	-9.4656f,32.1542f,-63.8098f,  39.1199f,-54.3046f,-23.2173f,  39.5164f,-53.9047f,-25.0148f,  38.2616f,-52.3474f,-24.9669f,  38.5984f,-51.8752f,-26.6764f,  
	37.5317f,-50.1998f,-26.4368f,  37.8106f,-49.6656f,-28.0576f,  36.9538f,-47.9087f,-27.6316f,  37.1771f,-47.3206f,-29.1676f,  36.5488f,-45.5183f,-28.5685f,  
	36.7195f,-44.8819f,-30.0277f,  36.3348f,-43.0681f,-29.2765f,  36.4553f,-42.3855f,-30.6703f,  36.3248f,-40.5916f,-29.7944f,  36.3969f,-39.8633f,-31.1337f,  
	36.5263f,-38.1172f,-30.1659f,  36.5511f,-37.3389f,-31.4648f,  36.9409f,-35.6645f,-30.4416f,  36.9181f,-34.8295f,-31.7143f,  37.5632f,-33.2466f,-30.6750f,  
	37.4912f,-32.3464f,-31.9337f,  38.3813f,-30.8703f,-30.9183f,  38.2569f,-29.8949f,-32.1739f,  39.3761f,-28.5372f,-31.2219f,  39.1954f,-27.4759f,-32.4830f,  
	40.5226f,-26.2441f,-31.6324f,  40.2805f,-25.0856f,-32.9056f,  41.7899f,-23.9838f,-32.1914f,  41.4807f,-22.7180f,-33.4800f,  43.1425f,-21.7477f,-32.9333f,  
	42.7597f,-20.3646f,-34.2391f,  44.5407f,-19.5244f,-33.8862f,  44.0778f,-18.0161f,-35.2076f,  45.9418f,-17.3029f,-35.0694f,  45.3924f,-15.6633f,-36.4030f,  
	47.3015f,-15.0725f,-36.4945f,  46.6594f,-13.2972f,-37.8347f,  48.5747f,-12.8232f,-38.1649f,  47.8344f,-10.9108f,-39.5040f,  49.7169f,-10.5474f,-40.0754f,  
	48.8732f,-8.4975f,-41.4051f,  50.6845f,-8.2384f,-42.2140f,  49.7336f,-6.0540f,-43.5242f,  51.4370f,-5.8932f,-44.5609f,  50.3758f,-3.5791f,-45.8409f,  
	51.9371f,-3.5108f,-47.0898f,  50.7633f,-1.0735f,-48.3286f,  52.1515f,-1.0926f,-49.7690f,  50.8650f,1.4581f,-50.9548f,  52.0530f,1.3555f,-52.5611f,  
	50.6538f,4.0095f,-53.6826f,  51.6190f,3.8268f,-55.4253f,  50.1090f,6.5714f,-56.4713f,  50.8340f,6.3106f,-58.3175f,  49.2161f,9.1321f,-59.2773f,  
	49.6889f,8.7943f,-61.1916f,  47.9674f,11.6775f,-62.0551f,  48.1818f,11.2628f,-64.0004f,  46.3622f,14.1914f,-64.7586f,  46.3179f,13.6991f,-66.6971f,  
	44.4071f,16.6559f,-67.3416f,  44.1099f,16.0844f,-69.2356f,  42.1156f,19.0517f,-69.7596f,  41.5772f,18.3988f,-71.5723f,  39.5082f,21.3588f,-71.9703f,  
	38.7461f,20.6222f,-73.6669f,  36.6127f,23.5561f,-73.9346f,  35.6496f,22.7332f,-75.4830f,  33.4630f,25.6221f,-75.6177f,  32.3269f,24.7107f,-76.9890f,  
	30.0990f,27.5362f,-76.9898f,  28.8220f,26.5347f,-78.1595f,  26.5649f,29.2797f,-78.0271f,  25.1827f,28.1875f,-78.9752f,  22.9118f,30.8324f,-78.7123f,  
	21.4633f,29.6501f,-79.4238f,  19.1914f,32.1796f,-79.0349f,  17.7173f,30.9098f,-79.5005f,  15.4606f,33.3070f,-78.9919f,  14.0026f,31.9543f,-79.2077f,  
	11.7759f,34.2052f,-78.5879f,  10.3758f,32.7766f,-78.5558f,  8.1965f,34.8669f,-77.8355f,  6.8949f,33.3724f,-77.5628f,  4.7778f,35.2925f,-76.7539f,  
	3.6129f,33.7451f,-76.2530f,  1.5750f,35.4856f,-75.3699f,  0.5819f,33.9016f,-74.6580f,  -1.3621f,35.4571f,-73.7161f,  -2.1528f,33.8560f,-72.8143f,  
	-3.9876f,35.2235f,-71.8308f,  -4.5510f,33.6281f,-70.7635f,  -6.2651f,34.8100f,-69.7549f,  -6.5826f,33.2458f,-68.5488f,  -8.1643f,34.2460f,-67.5332f,  
	-8.2243f,32.7414f,-66.2161f,  -9.6665f,33.5692f,-65.2097f,  -9.4656f,32.1542f,-63.8098f,  -10.7627f,32.8215f,-62.8280f,  39.5164f,-53.9047f,-25.0148f,  
	39.8711f,-53.4103f,-26.8362f,  38.5984f,-51.8752f,-26.6764f,  38.8922f,-51.3114f,-28.4088f,  37.8106f,-49.6656f,-28.0576f,  38.0455f,-49.0417f,-29.7022f,  
	37.1771f,-47.3206f,-29.1676f,  37.3559f,-46.6438f,-30.7295f,  36.7195f,-44.8819f,-30.0277f,  36.8447f,-44.1561f,-31.5160f,  36.4553f,-42.3855f,-30.6703f,  
	36.5291f,-41.6133f,-32.0940f,  36.3969f,-39.8633f,-31.1337f,  36.4208f,-39.0423f,-32.5057f,  36.5511f,-37.3389f,-31.4648f,  36.5255f,-36.4644f,-32.7983f,  
	36.9181f,-34.8295f,-31.7143f,  36.8419f,-33.8948f,-33.0214f,  37.4912f,-32.3464f,-31.9337f,  37.3624f,-31.3428f,-33.2253f,  38.2569f,-29.8949f,-32.1739f,  
	38.0723f,-28.8131f,-33.4590f,  39.1954f,-27.4759f,-32.4830f,  38.9506f,-26.3053f,-33.7691f,  40.2805f,-25.0856f,-32.9056f,  39.9705f,-23.8167f,-34.1972f,  
	41.4807f,-22.7180f,-33.4800f,  41.0996f,-21.3410f,-34.7800f,  42.7597f,-20.3646f,-34.2391f,  42.3016f,-18.8717f,-35.5471f,  44.0778f,-18.0161f,-35.2076f,  
	43.5365f,-16.4007f,-36.5216f,  45.3924f,-15.6633f,-36.4030f,  44.7620f,-13.9207f,-37.7186f,  46.6594f,-13.2972f,-37.8347f,  45.9346f,-11.4251f,-39.1457f,  
	47.8344f,-10.9108f,-39.5040f,  47.0101f,-8.9080f,-40.8032f,  48.8732f,-8.4975f,-41.4051f,  47.9452f,-6.3660f,-42.6836f,  49.7336f,-6.0540f,-43.5242f,  
	48.6987f,-3.7977f,-44.7723f,  50.3758f,-3.5791f,-45.8409f,  49.2315f,-1.2032f,-47.0485f,  50.7633f,-1.0735f,-48.3286f,  49.5089f,1.4134f,-49.4847f,  
	50.8650f,1.4581f,-50.9548f,  49.5000f,4.0470f,-52.0487f,  50.6538f,4.0095f,-53.6826f,  49.1795f,6.6890f,-54.7037f,  50.1090f,6.5714f,-56.4713f,  
	48.5277f,9.3290f,-57.4093f,  49.2161f,9.1321f,-59.2773f,  47.5313f,11.9537f,-60.1224f,  47.9674f,11.6775f,-62.0551f,  46.1839f,14.5479f,-62.7982f,  
	46.3622f,14.1914f,-64.7586f,  44.4859f,17.0947f,-65.3914f,  44.4071f,16.6559f,-67.3416f,  42.4448f,19.5755f,-67.8572f,  42.1156f,19.0517f,-69.7596f,  
	40.0754f,21.9704f,-70.1520f,  39.5082f,21.3588f,-71.9703f,  37.3994f,24.2588f,-72.2348f,  36.6127f,23.5561f,-73.9346f,  34.4455f,26.4192f,-74.0680f,  
	33.4630f,25.6221f,-75.6177f,  31.2485f,28.4306f,-75.6183f,  30.0990f,27.5362f,-76.9898f,  27.8485f,30.2732f,-76.8577f,  26.5649f,29.2797f,-78.0271f,  
	24.2928f,31.9258f,-77.7640f,  22.9118f,30.8324f,-78.7123f,  20.6305f,33.3723f,-78.3217f,  19.1914f,32.1796f,-79.0349f,  16.9164f,34.5962f,-78.5222f,  
	15.4606f,33.3070f,-78.9919f,  13.2063f,35.5858f,-78.3646f,  11.7759f,34.2052f,-78.5879f,  9.5596f,36.3310f,-77.8557f,  8.1965f,34.8669f,-77.8355f,  
	6.0330f,36.8291f,-77.0094f,  4.7778f,35.2925f,-76.7539f,  2.6842f,37.0804f,-75.8476f,  1.5750f,35.4856f,-75.3699f,  -0.4332f,37.0923f,-74.3983f,  
	-1.3621f,35.4571f,-73.7161f,  -3.2686f,36.8778f,-72.6960f,  -3.9876f,35.2235f,-71.8308f,  -5.7800f,36.4587f,-70.7789f,  -6.2651f,34.8100f,-69.7549f,  
	-7.9300f,35.8615f,-68.6895f,  -8.1643f,34.2460f,-67.5332f,  -9.6925f,35.1215f,-66.4714f,  -9.6665f,33.5692f,-65.2097f,  -11.0506f,34.2787f,-64.1680f,  
	-10.7627f,32.8215f,-62.8280f,  -11.9993f,33.3790f,-61.8208f,  39.8711f,-53.4103f,-26.8362f,  40.1802f,-52.8181f,-28.6792f,  38.8922f,-51.3114f,-28.4088f,  
	39.1394f,-50.6520f,-30.1631f,  38.0455f,-49.0417f,-29.7022f,  38.2331f,-48.3239f,-31.3699f,  37.3559f,-46.6438f,-30.7295f,  37.4864f,-45.8733f,-32.3170f,  
	36.8447f,-44.1561f,-31.5160f,  36.9206f,-43.3372f,-33.0303f,  36.5291f,-41.6133f,-32.0940f,  36.5522f,-40.7456f,-33.5466f,  36.4208f,-39.0423f,-32.5057f,  
	36.3921f,-38.1234f,-33.9084f,  36.5255f,-36.4644f,-32.7983f,  36.4447f,-35.4891f,-34.1627f,  36.8419f,-33.8948f,-33.0214f,  36.7078f,-32.8561f,-34.3583f,  
	37.3624f,-31.3428f,-33.2253f,  37.1725f,-30.2327f,-34.5439f,  38.0723f,-28.8131f,-33.4590f,  37.8230f,-27.6222f,-34.7674f,  38.9506f,-26.3053f,-33.7691f,  
	38.6376f,-25.0247f,-35.0725f,  39.9705f,-23.8167f,-34.1972f,  39.5886f,-22.4371f,-35.4995f,  41.0996f,-21.3410f,-34.7800f,  40.6435f,-19.8549f,-36.0821f,  
	42.3016f,-18.8717f,-35.5471f,  41.7656f,-17.2721f,-36.8482f,  43.5365f,-16.4007f,-36.5216f,  42.9151f,-14.6828f,-37.8186f,  44.7620f,-13.9207f,-37.7186f,  
	44.0500f,-12.0815f,-39.0066f,  45.9346f,-11.4251f,-39.1457f,  45.1269f,-9.4629f,-40.4187f,  47.0101f,-8.9080f,-40.8032f,  46.1026f,-6.8238f,-42.0534f,  
	47.9452f,-6.3660f,-42.6836f,  46.9346f,-4.1628f,-43.9024f,  48.6987f,-3.7977f,-44.7723f,  47.5822f,-1.4796f,-45.9506f,  49.2315f,-1.2032f,-47.0485f,  
	48.0079f,1.2223f,-48.1760f,  49.5089f,1.4134f,-49.4847f,  48.1775f,3.9385f,-50.5516f,  49.5000f,4.0470f,-52.0487f,  48.0615f,6.6615f,-53.0450f,  
	49.1795f,6.6890f,-54.7037f,  47.6355f,9.3815f,-55.6195f,  48.5277f,9.3290f,-57.4093f,  46.8809f,12.0868f,-58.2352f,  47.5313f,11.9537f,-60.1224f,  
	45.7860f,14.7626f,-60.8494f,  46.1839f,14.5479f,-62.7982f,  44.3449f,17.3933f,-63.4183f,  44.4859f,17.0947f,-65.3914f,  42.5593f,19.9611f,-65.8975f,  
	42.4448f,19.5755f,-67.8572f,  40.4379f,22.4465f,-68.2430f,  40.0754f,21.9704f,-70.1520f,  37.9963f,24.8294f,-70.4126f,  37.3994f,24.2588f,-72.2348f,  
	35.2574f,27.0885f,-72.3665f,  34.4455f,26.4192f,-74.0680f,  32.2508f,29.2024f,-74.0686f,  31.2485f,28.4306f,-75.6183f,  29.0116f,31.1508f,-75.4869f,  
	27.8485f,30.2732f,-76.8577f,  25.5827f,32.9118f,-76.5953f,  24.2928f,31.9258f,-77.7640f,  22.0099f,34.4676f,-77.3731f,  20.6305f,33.3723f,-78.3217f,  
	18.3451f,35.7999f,-77.8067f,  16.9164f,34.5962f,-78.5222f,  14.6426f,36.8949f,-77.8895f,  13.2063f,35.5858f,-78.3646f,  10.9608f,37.7398f,-77.6229f,  
	9.5596f,36.3310f,-77.8557f,  7.3571f,38.3288f,-77.0152f,  6.0330f,36.8291f,-77.0094f,  3.8906f,38.6588f,-76.0829f,  2.6842f,37.0804f,-75.8476f,  
	0.6179f,38.7337f,-74.8491f,  -0.4332f,37.0923f,-74.3983f,  -2.4065f,38.5626f,-73.3444f,  -3.2686f,36.8778f,-72.6960f,  -5.1355f,38.1638f,-71.6031f,  
	-5.7800f,36.4587f,-70.7789f,  -7.5255f,37.5604f,-69.6655f,  -7.9300f,35.8615f,-68.6895f,  -9.5434f,36.7845f,-67.5733f,  -9.6925f,35.1215f,-66.4714f,  
	-11.1643f,35.8739f,-65.3699f,  -11.0506f,34.2787f,-64.1680f,  -12.3751f,34.8730f,-63.0970f,  -11.9993f,33.3790f,-61.8208f,  -13.1754f,33.8312f,-60.7930f,  
	40.1802f,-52.8181f,-28.6792f,  40.4398f,-52.1241f,-30.5425f,  39.1394f,-50.6520f,-30.1631f,  39.3361f,-49.8932f,-31.9379f,  38.2331f,-48.3239f,-31.3699f,  
	38.3693f,-47.5076f,-33.0597f,  37.4864f,-45.8733f,-32.3170f,  37.5646f,-45.0057f,-33.9268f,  36.9206f,-43.3372f,-33.0303f,  36.9430f,-42.4198f,-34.5696f,  
	36.5522f,-40.7456f,-33.5466f,  36.5203f,-39.7779f,-35.0256f,  36.3921f,-38.1234f,-33.9084f,  36.3063f,-37.1020f,-35.3381f,  36.4447f,-35.4891f,-34.1627f,  
	36.3044f,-34.4090f,-35.5533f,  36.7078f,-32.8561f,-34.3583f,  36.5113f,-31.7106f,-35.7190f,  37.1725f,-30.2327f,-34.5439f,  36.9170f,-29.0137f,-35.8832f,  
	37.8230f,-27.6222f,-34.7674f,  37.5050f,-26.3216f,-36.0911f,  38.6376f,-25.0247f,-35.0725f,  38.2527f,-23.6339f,-36.3855f,  39.5886f,-22.4371f,-35.4995f,  
	39.1320f,-20.9490f,-36.8035f,  40.6435f,-19.8549f,-36.0821f,  40.1100f,-18.2626f,-37.3774f,  41.7656f,-17.2721f,-36.8482f,  41.1501f,-15.5706f,-38.1330f,  
	42.9151f,-14.6828f,-37.8186f,  42.2128f,-12.8689f,-39.0890f,  44.0500f,-12.0815f,-39.0066f,  43.2560f,-10.1527f,-40.2580f,  45.1269f,-9.4629f,-40.4187f,  
	44.2371f,-7.4196f,-41.6444f,  46.1026f,-6.8238f,-42.0534f,  45.1137f,-4.6681f,-43.2459f,  46.9346f,-4.1628f,-43.9024f,  45.8437f,-1.8979f,-45.0535f,  
	47.5822f,-1.4796f,-45.9506f,  46.3878f,0.8881f,-47.0511f,  48.0079f,1.2223f,-48.1760f,  46.7089f,3.6860f,-49.2169f,  48.1775f,3.9385f,-50.5516f,  
	46.7740f,6.4892f,-51.5235f,  48.0615f,6.6615f,-53.0450f,  46.5547f,9.2890f,-53.9386f,  47.6355f,9.3815f,-55.6195f,  46.0274f,12.0744f,-56.4259f,  
	46.8809f,12.0868f,-58.2352f,  45.1752f,14.8318f,-58.9456f,  45.7860f,14.7626f,-60.8494f,  43.9866f,17.5466f,-61.4561f,  44.3449f,17.3933f,-63.4183f,  
	42.4571f,20.2019f,-63.9141f,  42.5593f,19.9611f,-65.8975f,  40.5894f,22.7789f,-66.2761f,  40.4379f,22.4465f,-68.2430f,  38.3931f,25.2580f,-68.4993f,  
	37.9963f,24.8294f,-70.4126f,  35.8849f,27.6184f,-70.5424f,  35.2574f,27.0885f,-72.3665f,  33.0887f,29.8385f,-72.3670f,  32.2508f,29.2024f,-74.0686f,  
	30.0343f,31.8977f,-73.9382f,  29.0116f,31.1508f,-75.4869f,  26.7595f,33.7734f,-75.2260f,  25.5827f,32.9118f,-76.5953f,  23.3060f,35.4466f,-76.2053f,  
	22.0099f,34.4676f,-77.3731f,  19.7223f,36.8975f,-76.8576f,  18.3451f,35.7999f,-77.8067f,  16.0601f,38.1100f,-77.1710f,  14.6426f,36.8949f,-77.8895f,  
	12.3761f,39.0691f,-77.1413f,  10.9608f,37.7398f,-77.6229f,  8.7274f,39.7657f,-76.7712f,  7.3571f,38.3288f,-77.0152f,  5.1736f,40.1937f,-76.0717f,  
	3.8906f,38.6588f,-76.0829f,  1.7733f,40.3530f,-75.0610f,  0.6179f,38.7337f,-74.8491f,  -1.4160f,40.2491f,-73.7647f,  -2.4065f,38.5626f,-73.3444f,  
	-4.3426f,39.8961f,-72.2135f,  -5.1355f,38.1638f,-71.6031f,  -6.9578f,39.3135f,-70.4442f,  -7.5255f,37.5604f,-69.6655f,  -9.2215f,38.5297f,-68.4967f,  
	-9.5434f,36.7845f,-67.5733f,  -11.1019f,37.5798f,-66.4131f,  -11.1643f,35.8739f,-65.3699f,  -12.5775f,36.5058f,-64.2350f,  -12.3751f,34.8730f,-63.0970f,  
	-13.6395f,35.3562f,-62.0018f,  -13.1754f,33.8312f,-60.7930f,  -14.2900f,34.1823f,-59.7497f,  40.4398f,-52.1241f,-30.5425f,  40.6460f,-51.3250f,-32.4238f,  
	39.3361f,-49.8932f,-31.9379f,  39.4784f,-49.0311f,-33.7313f,  38.3693f,-47.5076f,-33.0597f,  38.4503f,-46.5901f,-34.7679f,  37.5646f,-45.0057f,-33.9268f,  
	37.5866f,-44.0361f,-35.5573f,  36.9430f,-42.4198f,-34.5696f,  36.9079f,-41.3998f,-36.1308f,  36.5203f,-39.7779f,-35.0256f,  36.4290f,-38.7059f,-36.5274f,  
	36.3063f,-37.1020f,-35.3381f,  36.1589f,-35.9747f,-36.7900f,  36.3044f,-34.4090f,-35.5533f,  36.1000f,-33.2213f,-36.9642f,  36.5113f,-31.7106f,-35.7190f,  
	36.2480f,-30.4560f,-37.0974f,  36.9170f,-29.0137f,-35.8832f,  36.5920f,-27.6850f,-37.2355f,  37.5050f,-26.3216f,-36.0911f,  37.1146f,-24.9111f,-37.4226f,  
	38.2527f,-23.6339f,-36.3855f,  37.7928f,-22.1352f,-37.6988f,  39.1320f,-20.9490f,-36.8035f,  38.5980f,-19.3552f,-38.1000f,  40.1100f,-18.2626f,-37.3774f,  
	39.4974f,-16.5690f,-38.6564f,  41.1501f,-15.5706f,-38.1330f,  40.4544f,-13.7736f,-39.3917f,  42.2128f,-12.8689f,-39.0890f,  41.4292f,-10.9657f,-40.3239f,  
	43.2560f,-10.1527f,-40.2580f,  42.3808f,-8.1431f,-41.4635f,  44.2371f,-7.4196f,-41.6444f,  43.2671f,-5.3050f,-42.8141f,  45.1137f,-4.6681f,-43.2459f,  
	44.0457f,-2.4510f,-44.3727f,  45.8437f,-1.8979f,-45.0535f,  44.6761f,0.4164f,-46.1292f,  46.3878f,0.8881f,-47.0511f,  45.1193f,3.2937f,-48.0674f,  
	46.7089f,3.6860f,-49.2169f,  45.3392f,6.1752f,-50.1653f,  46.7740f,6.4892f,-51.5235f,  45.3039f,9.0528f,-52.3953f,  46.5547f,9.2890f,-53.9386f,  
	44.9855f,11.9168f,-54.7255f,  46.0274f,12.0744f,-56.4259f,  44.3622f,14.7543f,-57.1194f,  45.1752f,14.8318f,-58.9456f,  43.4171f,17.5520f,-59.5384f,  
	43.9866f,17.5466f,-61.4561f,  42.1399f,20.2939f,-61.9411f,  42.4571f,20.2019f,-63.9141f,  40.5274f,22.9620f,-64.2850f,  40.5894f,22.7789f,-66.2761f,  
	38.5831f,25.5374f,-66.5275f,  38.3931f,25.2580f,-68.4993f,  36.3172f,27.9999f,-68.6267f,  35.8849f,27.6184f,-70.5424f,  33.7478f,30.3281f,-70.5427f,  
	33.0887f,29.8385f,-72.3670f,  30.8990f,32.5010f,-72.2381f,  30.0343f,31.8977f,-73.9382f,  27.8030f,34.4960f,-73.6796f,  26.7595f,33.7734f,-75.2260f,  
	24.4967f,36.2926f,-74.8383f,  23.3060f,35.4466f,-76.2053f,  21.0245f,37.8701f,-75.6912f,  19.7223f,36.8975f,-76.8576f,  17.4345f,39.2104f,-76.2213f,  
	16.0601f,38.1100f,-77.1710f,  13.7813f,40.2962f,-76.4190f,  12.3761f,39.0691f,-77.1413f,  10.1204f,41.1155f,-76.2815f,  8.7274f,39.7657f,-76.7712f,  
	6.5111f,41.6589f,-75.8143f,  5.1736f,40.1937f,-76.0717f,  3.0131f,41.9227f,-75.0302f,  1.7733f,40.3530f,-75.0610f,  -0.3139f,41.9084f,-73.9499f,  
	-1.4160f,40.2491f,-73.7647f,  -3.4149f,41.6259f,-72.5996f,  -4.3426f,39.8961f,-72.2135f,  -6.2364f,41.0908f,-71.0125f,  -6.9578f,39.3135f,-70.4442f,  
	-8.7325f,40.3273f,-69.2257f,  -9.2215f,38.5297f,-68.4967f,  -10.8644f,39.3672f,-67.2796f,  -11.1019f,37.5798f,-66.4131f,  -12.6029f,38.2495f,-65.2153f,  
	-12.5775f,36.5058f,-64.2350f,  -13.9311f,37.0209f,-63.0722f,  -13.6395f,35.3562f,-62.0018f,  -14.8421f,35.7319f,-60.8882f,  -14.2900f,34.1823f,-59.7497f,  
	-15.3427f,34.4366f,-58.6955f,  40.6460f,-51.3250f,-32.4238f,  40.7946f,-50.4177f,-34.3203f,  39.4784f,-49.0311f,-33.7313f,  39.5623f,-48.0633f,-35.5393f,  
	38.4503f,-46.5901f,-34.7679f,  38.4719f,-45.5669f,-36.4927f,  37.5866f,-44.0361f,-35.5573f,  37.5481f,-42.9612f,-37.2051f,  36.9079f,-41.3998f,-36.1308f,  
	36.8108f,-40.2733f,-37.7104f,  36.4290f,-38.7059f,-36.5274f,  36.2740f,-37.5266f,-38.0471f,  36.1589f,-35.9747f,-36.7900f,  35.9458f,-34.7392f,-38.2583f,  
	36.1000f,-33.2213f,-36.9642f,  35.8276f,-31.9244f,-38.3894f,  36.2480f,-30.4560f,-37.0974f,  35.9142f,-29.0917f,-38.4860f,  36.5920f,-27.6850f,-37.2355f,  
	36.1939f,-26.2468f,-38.5932f,  37.1146f,-24.9111f,-37.4226f,  36.6489f,-23.3932f,-38.7528f,  37.7928f,-22.1352f,-37.6988f,  37.2555f,-20.5313f,-39.0037f,  
	38.5980f,-19.3552f,-38.1000f,  37.9851f,-17.6607f,-39.3798f,  39.4974f,-16.5690f,-38.6564f,  38.8049f,-14.7803f,-39.9093f,  40.4544f,-13.7736f,-39.3917f,  
	39.6780f,-11.8878f,-40.6154f,  41.4292f,-10.9657f,-40.3239f,  40.5654f,-8.9821f,-41.5139f,  42.3808f,-8.1431f,-41.4635f,  41.4263f,-6.0626f,-42.6144f,  
	43.2671f,-5.3050f,-42.8141f,  42.2190f,-3.1294f,-43.9199f,  44.0457f,-2.4510f,-44.3727f,  42.9023f,-0.1847f,-45.4261f,  44.6761f,0.4164f,-46.1292f,  
	43.4359f,2.7684f,-47.1228f,  45.1193f,3.2937f,-48.0674f,  43.7816f,5.7247f,-48.9936f,  45.3392f,6.1752f,-50.1653f,  43.9044f,8.6771f,-51.0161f,  
	45.3039f,9.0528f,-52.3953f,  43.7728f,11.6167f,-53.1629f,  44.9855f,11.9168f,-54.7255f,  43.3607f,14.5315f,-55.4020f,  44.3622f,14.7543f,-57.1194f,  
	42.6460f,17.4094f,-57.6979f,  43.4171f,17.5520f,-59.5384f,  41.6131f,20.2355f,-60.0120f,  42.1399f,20.2939f,-61.9411f,  40.2530f,22.9926f,-62.3036f,  
	40.5274f,22.9620f,-64.2850f,  38.5630f,25.6627f,-64.5310f,  38.5831f,25.5374f,-66.5275f,  36.5470f,28.2264f,-66.6524f,  36.3172f,27.9999f,-68.6267f,  
	34.2170f,30.6627f,-68.6270f,  33.7478f,30.3281f,-70.5427f,  31.5910f,32.9504f,-70.4157f,  30.8990f,32.5010f,-72.2381f,  28.6953f,35.0670f,-71.9826f,  
	27.8030f,34.4960f,-73.6796f,  25.5616f,36.9912f,-73.2955f,  24.4967f,36.2926f,-74.8383f,  22.2292f,38.7012f,-74.3274f,  21.0245f,37.8701f,-75.6912f,  
	18.7425f,40.1773f,-75.0565f,  17.4345f,39.2104f,-76.2213f,  15.1523f,41.4001f,-75.4683f,  13.7813f,40.2962f,-76.4190f,  11.5123f,42.3549f,-75.5545f,  
	10.1204f,41.1155f,-76.2815f,  7.8804f,43.0293f,-75.3151f,  6.5111f,41.6589f,-75.8143f,  4.3163f,43.4159f,-74.7574f,  3.0131f,41.9227f,-75.0302f,  
	0.8808f,43.5121f,-73.8968f,  -0.3139f,41.9084f,-73.9499f,  -2.3683f,43.3238f,-72.7548f,  -3.4149f,41.6259f,-72.5996f,  -5.3739f,42.8619f,-71.3605f,  
	-6.2364f,41.0908f,-71.0125f,  -8.0847f,42.1466f,-69.7479f,  -8.7325f,40.3273f,-69.2257f,  -10.4558f,41.2055f,-67.9548f,  -10.8644f,39.3672f,-67.2796f,  
	-12.4509f,40.0743f,-66.0211f,  -12.6029f,38.2495f,-65.2153f,  -14.0450f,38.7970f,-63.9859f,  -13.9311f,37.0209f,-63.0722f,  -15.2228f,37.4223f,-61.8880f,  
	-14.8421f,35.7319f,-60.8882f,  -15.9822f,36.0042f,-59.7612f,  -15.3427f,34.4366f,-58.6955f,  -16.3346f,34.5994f,-57.6341f,  40.7946f,-50.4177f,-34.3203f,  
	40.8817f,-49.4001f,-36.2279f,  39.5623f,-48.0633f,-35.5393f,  39.5838f,-46.9860f,-37.3598f,  38.4719f,-45.5669f,-36.4927f,  38.4302f,-44.4355f,-38.2302f,  
	37.5481f,-42.9612f,-37.2051f,  37.4451f,-41.7776f,-38.8668f,  36.8108f,-40.2733f,-37.7104f,  36.6476f,-39.0381f,-39.3033f,  36.2740f,-37.5266f,-38.0471f,  
	36.0511f,-36.2380f,-39.5790f,  35.9458f,-34.7392f,-38.2583f,  35.6628f,-33.3938f,-39.7369f,  35.8276f,-31.9244f,-38.3894f,  35.4832f,-30.5176f,-39.8214f,  
	35.9142f,-29.0917f,-38.4860f,  35.5063f,-27.6181f,-39.8773f,  36.1939f,-26.2468f,-38.5932f,  35.7199f,-24.7016f,-39.9474f,  36.6489f,-23.3932f,-38.7528f,  
	36.1054f,-21.7707f,-40.0730f,  37.2555f,-20.5313f,-39.0037f,  36.6392f,-18.8272f,-40.2908f,  37.9851f,-17.6607f,-39.3798f,  37.2926f,-15.8717f,-40.6330f,  
	38.8049f,-14.7803f,-39.9093f,  38.0322f,-12.9034f,-41.1273f,  39.6780f,-11.8878f,-40.6154f,  38.8220f,-9.9221f,-41.7948f,  40.5654f,-8.9821f,-41.5139f,  
	39.6230f,-6.9280f,-42.6503f,  41.4263f,-6.0626f,-42.6144f,  40.3949f,-3.9215f,-43.7027f,  42.2190f,-3.1294f,-43.9199f,  41.0966f,-0.9049f,-44.9538f,  
	42.9023f,-0.1847f,-45.4261f,  41.6875f,2.1190f,-46.3993f,  43.4359f,2.7684f,-47.1228f,  42.1277f,5.1456f,-48.0284f,  43.7816f,5.7247f,-48.9936f,  
	42.3802f,8.1682f,-49.8244f,  43.9044f,8.6771f,-51.0161f,  42.4102f,11.1791f,-51.7651f,  43.7728f,11.6167f,-53.1629f,  42.1879f,14.1670f,-53.8228f,  
	43.3607f,14.5315f,-55.4020f,  41.6867f,17.1212f,-55.9662f,  42.6460f,17.4094f,-57.6979f,  40.8858f,20.0277f,-58.1600f,  41.6131f,20.2355f,-60.0120f,  
	39.7709f,22.8702f,-60.3659f,  40.2530f,22.9926f,-62.3036f,  38.3331f,25.6319f,-62.5439f,  38.5630f,25.6627f,-64.5310f,  36.5704f,28.2942f,-64.6532f,  
	36.5470f,28.2264f,-66.6524f,  34.4882f,30.8366f,-66.6525f,  34.2170f,30.6627f,-68.6270f,  32.0985f,33.2382f,-68.5021f,  31.5910f,32.9504f,-70.4157f,  
	29.4211f,35.4768f,-70.1640f,  28.6953f,35.0670f,-71.9826f,  26.4824f,37.5304f,-71.6034f,  25.5616f,36.9912f,-73.2955f,  23.3160f,39.3765f,-72.7895f,  
	22.2292f,38.7012f,-74.3274f,  19.9615f,40.9940f,-73.6968f,  18.7425f,40.1773f,-75.0565f,  16.4658f,42.3619f,-74.3056f,  15.1523f,41.4001f,-75.4683f,  
	12.8793f,43.4628f,-74.6027f,  11.5123f,42.3549f,-75.5545f,  9.2579f,44.2815f,-74.5826f,  7.8804f,43.0293f,-75.3151f,  5.6604f,44.8069f,-74.2470f,  
	4.3163f,43.4159f,-74.7574f,  2.1477f,45.0328f,-73.6062f,  0.8808f,43.5121f,-73.8968f,  -1.2207f,44.9605f,-72.6765f,  -2.3683f,43.3238f,-72.7548f,  
	-4.3849f,44.5966f,-71.4824f,  -5.3739f,42.8619f,-71.3605f,  -7.2892f,43.9564f,-70.0543f,  -8.0847f,42.1466f,-69.7479f,  -9.8833f,43.0632f,-68.4270f,  
	-10.4558f,41.2055f,-67.9548f,  -12.1242f,41.9489f,-66.6386f,  -12.4509f,40.0743f,-66.0211f,  -13.9795f,40.6539f,-64.7273f,  -14.0450f,38.7970f,-63.9859f,  
	-15.4253f,39.2247f,-62.7321f,  -15.2228f,37.4223f,-61.8880f,  -16.4514f,37.7139f,-60.6881f,  -15.9822f,36.0042f,-59.7612f,  -17.0601f,36.1783f,-58.6255f,  
	-16.3346f,34.5994f,-57.6341f,  -17.2658f,34.6754f,-56.5693f,  40.8817f,-49.4001f,-36.2279f,  40.9032f,-48.2691f,-38.1438f,  39.5838f,-46.9860f,-37.3598f,  
	39.5389f,-45.7973f,-39.1883f,  38.4302f,-44.4355f,-38.2302f,  38.3210f,-43.1928f,-39.9767f,  37.4451f,-41.7776f,-38.8668f,  37.2733f,-40.4833f,-40.5369f,  
	36.6476f,-39.0381f,-39.3033f,  36.4143f,-37.6924f,-40.9038f,  36.0511f,-36.2380f,-39.5790f,  35.7566f,-34.8390f,-41.1170f,  35.6628f,-33.3938f,-39.7369f,  
	35.3064f,-31.9383f,-41.2187f,  35.4832f,-30.5176f,-39.8214f,  35.0635f,-29.0016f,-41.2528f,  35.5063f,-27.6181f,-39.8773f,  35.0216f,-26.0379f,-41.2623f,  
	35.7199f,-24.7016f,-39.9474f,  35.1675f,-23.0521f,-41.2897f,  36.1054f,-21.7707f,-40.0730f,  35.4827f,-20.0484f,-41.3739f,  36.6392f,-18.8272f,-40.2908f,  
	35.9433f,-17.0292f,-41.5505f,  37.2926f,-15.8717f,-40.6330f,  36.5201f,-13.9950f,-41.8511f,  38.0322f,-12.9034f,-41.1273f,  37.1803f,-10.9470f,-42.3012f,  
	38.8220f,-9.9221f,-41.7948f,  37.8880f,-7.8863f,-42.9211f,  39.6230f,-6.9280f,-42.6503f,  38.6045f,-4.8138f,-43.7249f,  40.3949f,-3.9215f,-43.7027f,  
	39.2900f,-1.7318f,-44.7204f,  41.0966f,-0.9049f,-44.9538f,  39.9040f,1.3568f,-45.9092f,  41.6875f,2.1190f,-46.3993f,  40.4060f,4.4477f,-47.2863f,  
	42.1277f,5.1456f,-48.0284f,  40.7573f,7.5350f,-48.8408f,  42.3802f,8.1682f,-49.8244f,  40.9209f,10.6116f,-50.5559f,  42.4102f,11.1791f,-51.7651f,  
	40.8639f,13.6673f,-52.4088f,  42.1879f,14.1670f,-53.8228f,  40.5555f,16.6923f,-54.3730f,  41.6867f,17.1212f,-55.9662f,  39.9704f,19.6740f,-56.4167f,  
	40.8858f,20.0277f,-58.1600f,  39.0892f,22.5969f,-58.5050f,  39.7709f,22.8702f,-60.3659f,  37.8974f,25.4454f,-60.6004f,  38.3331f,25.6319f,-62.5439f,  
	36.3868f,28.2020f,-62.6632f,  36.5704f,28.2942f,-64.6532f,  34.5567f,30.8468f,-64.6532f,  34.4882f,30.8366f,-66.6525f,  32.4127f,33.3595f,-66.5302f,  
	32.0985f,33.2382f,-68.5021f,  29.9682f,35.7184f,-68.2550f,  29.4211f,35.4768f,-70.1640f,  27.2434f,37.9010f,-69.7907f,  26.4824f,37.5304f,-71.6034f,  
	24.2662f,39.8845f,-71.1040f,  23.3160f,39.3765f,-72.7895f,  21.0707f,41.6467f,-72.1652f,  19.9615f,40.9940f,-73.6968f,  17.6991f,43.1651f,-72.9507f,  
	16.4658f,42.3619f,-74.3056f,  14.1981f,44.4202f,-73.4423f,  12.8793f,43.4628f,-74.6027f,  10.6202f,45.3939f,-73.6293f,  9.2579f,44.2815f,-74.5826f,  
	7.0224f,46.0721f,-73.5080f,  5.6604f,44.8069f,-74.2470f,  3.4651f,46.4445f,-73.0831f,  2.1477f,45.0328f,-73.6062f,  0.0083f,46.5082f,-72.3659f,  
	-1.2207f,44.9605f,-72.6765f,  -3.2863f,46.2651f,-71.3761f,  -4.3849f,44.5966f,-71.4824f,  -6.3595f,45.7258f,-70.1396f,  -7.2892f,43.9564f,-70.0543f,  
	-9.1565f,44.9087f,-68.6881f,  -9.8833f,43.0632f,-68.4270f,  -11.6283f,43.8412f,-67.0572f,  -12.1242f,41.9489f,-66.6386f,  -13.7355f,42.5600f,-65.2836f,  
	-13.9795f,40.6539f,-64.7273f,  -15.4464f,41.1081f,-63.4061f,  -15.4253f,39.2247f,-62.7321f,  -16.7423f,39.5364f,-61.4601f,  -16.4514f,37.7139f,-60.6881f,  
	-17.6167f,37.9006f,-59.4776f,  -17.0601f,36.1783f,-58.6255f,  -18.0754f,36.2587f,-57.4856f,  -17.2658f,34.6754f,-56.5693f,  -18.1374f,34.6698f,-55.5042f,  
	40.9032f,-48.2691f,-38.1438f,  40.8553f,-47.0235f,-40.0633f,  39.5389f,-45.7973f,-39.1883f,  39.4237f,-44.4948f,-41.0212f,  38.3210f,-43.1928f,-39.9767f,  
	38.1405f,-41.8376f,-41.7269f,  37.2733f,-40.4833f,-40.5369f,  37.0292f,-39.0773f,-42.2101f,  36.4143f,-37.6924f,-40.9038f,  36.1073f,-36.2354f,-42.5060f,  
	35.7566f,-34.8390f,-41.1170f,  35.3868f,-33.3295f,-42.6541f,  35.3064f,-31.9383f,-41.2187f,  34.8733f,-30.3736f,-42.6963f,  35.0635f,-29.0016f,-41.2528f,  
	34.5660f,-27.3792f,-42.6749f,  35.0216f,-26.0379f,-41.2623f,  34.4578f,-24.3538f,-42.6328f,  35.1675f,-23.0521f,-41.2897f,  34.5354f,-21.3036f,-42.6106f,  
	35.4827f,-20.0484f,-41.3739f,  34.7801f,-18.2329f,-42.6461f,  35.9433f,-17.0292f,-41.5505f,  35.1674f,-15.1440f,-42.7742f,  36.5201f,-13.9950f,-41.8511f,  
	35.6685f,-12.0394f,-43.0246f,  37.1803f,-10.9470f,-42.3012f,  36.2508f,-8.9211f,-43.4221f,  37.8880f,-7.8863f,-42.9211f,  36.8785f,-5.7909f,-43.9861f,  
	38.6045f,-4.8138f,-43.7249f,  37.5134f,-2.6513f,-44.7299f,  39.2900f,-1.7318f,-44.7204f,  38.1159f,0.4946f,-45.6609f,  39.9040f,1.3568f,-45.9092f,  
	38.6458f,3.6430f,-46.7800f,  40.4060f,4.4477f,-47.2863f,  39.0636f,6.7882f,-48.0820f,  40.7573f,7.5350f,-48.8408f,  39.3306f,9.9240f,-49.5559f,  
	40.9209f,10.6116f,-50.5559f,  39.4113f,13.0408f,-51.1843f,  40.8639f,13.6673f,-52.4088f,  39.2718f,16.1301f,-52.9453f,  40.5555f,16.6923f,-54.3730f,  
	38.8826f,19.1804f,-54.8120f,  39.9704f,19.6740f,-56.4167f,  38.2198f,22.1774f,-56.7529f,  39.0892f,22.5969f,-58.5050f,  37.2634f,25.1064f,-58.7335f,  
	37.8974f,25.4454f,-60.6004f,  35.9995f,27.9514f,-60.7166f,  36.3868f,28.2020f,-62.6632f,  34.4213f,30.6932f,-62.6632f,  34.5567f,30.8468f,-64.6532f,  
	32.5283f,33.3123f,-64.5335f,  32.4127f,33.3595f,-66.5302f,  30.3270f,35.7876f,-66.2881f,  29.9682f,35.7184f,-68.2550f,  27.8316f,38.0966f,-67.8887f,  
	27.2434f,37.9010f,-69.7907f,  25.0635f,40.2165f,-69.2995f,  24.2662f,39.8845f,-71.1040f,  22.0513f,42.1242f,-70.4881f,  21.0707f,41.6467f,-72.1652f,  
	18.8313f,43.7961f,-71.4268f,  17.6991f,43.1651f,-72.9507f,  15.4459f,45.2107f,-72.0933f,  14.1981f,44.4202f,-73.4423f,  11.9438f,46.3477f,-72.4715f,  
	10.6202f,45.3939f,-73.6293f,  8.3792f,47.1898f,-72.5529f,  7.0224f,46.0721f,-73.5080f,  4.8105f,47.7231f,-72.3366f,  3.4651f,46.4445f,-73.0831f,  
	1.2977f,47.9403f,-71.8285f,  0.0083f,46.5082f,-72.3659f,  -2.0969f,47.8389f,-71.0433f,  -3.2863f,46.2651f,-71.3761f,  -5.3116f,47.4244f,-70.0024f,  
	-6.3595f,45.7258f,-70.1396f,  -8.2879f,46.7102f,-68.7337f,  -9.1565f,44.9087f,-68.6881f,  -10.9717f,45.7189f,-67.2698f,  -11.6283f,43.8412f,-67.0572f,  
	-13.3173f,44.4825f,-65.6454f,  -13.7355f,42.5600f,-65.2836f,  -15.2855f,43.0404f,-63.8984f,  -15.4464f,41.1081f,-63.4061f,  -16.8498f,41.4405f,-62.0643f,  
	-16.7423f,39.5364f,-61.4601f,  -17.9949f,39.7367f,-60.1758f,  -17.6167f,37.9006f,-59.4776f,  -18.7177f,37.9869f,-58.2622f,  -18.0754f,36.2587f,-57.4856f,  
	-19.0289f,36.2509f,-56.3454f,  -18.1374f,34.6698f,-55.5042f,  -18.9500f,34.5872f,-54.4422f,  40.8553f,-47.0235f,-40.0633f,  40.7341f,-45.6613f,-41.9825f,  
	39.4237f,-44.4948f,-41.0212f,  39.2344f,-43.0777f,-42.8530f,  38.1405f,-41.8376f,-41.7269f,  37.8851f,-40.3690f,-43.4754f,  37.0292f,-39.0773f,-42.2101f,  
	36.7089f,-37.5590f,-43.8801f,  36.1073f,-36.2354f,-42.5060f,  35.7230f,-34.6673f,-44.1029f,  35.3868f,-33.3295f,-42.6541f,  34.9387f,-31.7106f,-44.1829f,  
	34.8733f,-30.3736f,-42.6963f,  34.3609f,-28.7027f,-44.1609f,  34.5660f,-27.3792f,-42.6749f,  33.9883f,-25.6534f,-44.0795f,  34.4578f,-24.3538f,-42.6328f,  
	33.8135f,-22.5711f,-43.9796f,  34.5354f,-21.3036f,-42.6106f,  33.8229f,-19.4622f,-43.9010f,  34.7801f,-18.2329f,-42.6461f,  33.9973f,-16.3308f,-43.8809f,  
	35.1674f,-15.1440f,-42.7742f,  34.3124f,-13.1805f,-43.9525f,  35.6685f,-12.0394f,-43.0246f,  34.7397f,-10.0148f,-44.1448f,  36.2508f,-8.9211f,-43.4221f,  
	35.2464f,-6.8362f,-44.4819f,  36.8785f,-5.7909f,-43.9861f,  35.7971f,-3.6478f,-44.9822f,  37.5134f,-2.6513f,-44.7299f,  36.3539f,-0.4527f,-45.6587f,  
	38.1159f,0.4946f,-45.6609f,  36.8774f,2.7452f,-46.5181f,  38.6458f,3.6430f,-46.7800f,  37.3280f,5.9406f,-47.5611f,  39.0636f,6.7882f,-48.0820f,  
	37.6663f,9.1278f,-48.7823f,  39.3306f,9.9240f,-49.5559f,  37.8550f,12.2982f,-50.1701f,  39.4113f,13.0408f,-51.1843f,  37.8575f,15.4443f,-51.7078f,  
	39.2718f,16.1301f,-52.9453f,  37.6410f,18.5555f,-53.3733f,  38.8826f,19.1804f,-54.8120f,  37.1773f,21.6188f,-55.1394f,  38.2198f,22.1774f,-56.7529f,  
	36.4418f,24.6208f,-56.9753f,  37.2634f,25.1064f,-58.7335f,  35.4151f,27.5466f,-58.8466f,  35.9995f,27.9514f,-60.7166f,  34.0844f,30.3784f,-60.7165f,  
	34.4213f,30.6932f,-62.6632f,  32.4433f,33.0973f,-62.5464f,  32.5283f,33.3123f,-64.5335f,  30.4915f,35.6832f,-64.2971f,  30.3270f,35.7876f,-66.2881f,  
	28.2369f,38.1139f,-65.9297f,  27.8316f,38.0966f,-67.8887f,  25.6944f,40.3668f,-67.4070f,  25.0635f,40.2165f,-69.2995f,  22.8862f,42.4182f,-68.6941f,  
	22.0513f,42.1242f,-70.4881f,  19.8430f,44.2439f,-69.7601f,  18.8313f,43.7961f,-71.4268f,  16.6016f,45.8208f,-70.5786f,  15.4459f,45.2107f,-72.0933f,  
	13.2063f,47.1265f,-71.1292f,  11.9438f,46.3477f,-72.4715f,  9.7074f,48.1408f,-71.3981f,  8.3792f,47.1898f,-72.5529f,  6.1609f,48.8465f,-71.3793f,  
	4.8105f,47.7231f,-72.3366f,  2.6253f,49.2323f,-71.0734f,  1.2977f,47.9403f,-71.8285f,  -0.8372f,49.2911f,-70.4897f,  -2.0969f,47.8389f,-71.0433f,  
	-4.1635f,49.0232f,-69.6451f,  -5.3116f,47.4244f,-70.0024f,  -7.2924f,48.4371f,-68.5631f,  -8.2879f,46.7102f,-68.7337f,  -10.1656f,47.5498f,-67.2728f,  
	-10.9717f,45.7189f,-67.2698f,  -12.7320f,46.3887f,-65.8065f,  -13.3173f,44.4825f,-65.6454f,  -14.9455f,44.9886f,-64.2006f,  -15.2855f,43.0404f,-63.8984f,  
	-16.7722f,43.3936f,-62.4901f,  -16.8498f,41.4405f,-62.0643f,  -18.1883f,41.6554f,-60.7083f,  -17.9949f,39.7367f,-60.1758f,  -19.1818f,39.8302f,-58.8856f,  
	-18.7177f,37.9869f,-58.2622f,  -19.7549f,37.9783f,-57.0461f,  -19.0289f,36.2509f,-56.3454f,  -19.9206f,36.1595f,-55.2089f,  -18.9500f,34.5872f,-54.4422f,  
	-19.7047f,34.4328f,-53.3859f,  40.7341f,-45.6613f,-41.9825f,  40.5360f,-44.1823f,-43.8957f,  39.2344f,-43.0777f,-42.8530f,  38.9674f,-41.5455f,-44.6781f,  
	37.8851f,-40.3690f,-43.4754f,  37.5512f,-38.7872f,-45.2159f,  36.7089f,-37.5590f,-43.8801f,  36.3094f,-35.9290f,-45.5405f,  35.7230f,-34.6673f,-44.1029f,  
	35.2587f,-32.9898f,-45.6874f,  34.9387f,-31.7106f,-44.1829f,  34.4099f,-29.9857f,-45.6952f,  34.3609f,-28.7027f,-44.1609f,  33.7673f,-26.9286f,-45.6050f,  
	33.9883f,-25.6534f,-44.0795f,  33.3293f,-23.8294f,-45.4576f,  33.8135f,-22.5711f,-43.9796f,  33.0881f,-20.6961f,-45.2937f,  33.8229f,-19.4622f,-43.9010f,  
	33.0299f,-17.5349f,-45.1522f,  33.9973f,-16.3308f,-43.8809f,  33.1353f,-14.3508f,-45.0692f,  34.3124f,-13.1805f,-43.9525f,  33.3803f,-11.1486f,-45.0769f,  
	34.7397f,-10.0148f,-44.1448f,  33.7362f,-7.9318f,-45.2037f,  35.2464f,-6.8362f,-44.4819f,  34.1706f,-4.7040f,-45.4729f,  35.7971f,-3.6478f,-44.9822f,  
	34.6480f,-1.4689f,-45.9026f,  36.3539f,-0.4527f,-45.6587f,  35.1308f,1.7697f,-46.5052f,  36.8774f,2.7452f,-46.5181f,  35.5802f,5.0067f,-47.2869f,  
	37.3280f,5.9406f,-47.5611f,  35.9565f,8.2369f,-48.2484f,  37.6663f,9.1278f,-48.7823f,  36.2216f,11.4523f,-49.3836f,  37.8550f,12.2982f,-50.1701f,  
	36.3368f,14.6465f,-50.6815f,  37.8575f,15.4443f,-51.7078f,  36.2668f,17.8099f,-52.1251f,  37.6410f,18.5555f,-53.3733f,  35.9798f,20.9307f,-53.6921f,  
	37.1773f,21.6188f,-55.1394f,  35.4468f,23.9968f,-55.3558f,  36.4418f,24.6208f,-56.9753f,  34.6436f,26.9947f,-57.0853f,  35.4151f,27.5466f,-58.8466f,  
	33.5518f,29.9078f,-58.8465f,  34.0844f,30.3784f,-60.7165f,  32.1592f,32.7181f,-60.6028f,  32.4433f,33.0973f,-62.5464f,  30.4589f,35.4070f,-62.3160f,  
	30.4915f,35.6832f,-64.2971f,  28.4525f,37.9526f,-63.9474f,  28.2369f,38.1139f,-65.9297f,  26.1480f,40.3328f,-65.4589f,  25.6944f,40.3668f,-67.4070f,  
	23.5614f,42.5238f,-66.8139f,  22.8862f,42.4182f,-68.6941f,  20.7167f,44.5010f,-67.9789f,  19.8430f,44.2439f,-69.7601f,  17.6453f,46.2400f,-68.9242f,  
	16.6016f,45.8208f,-70.5786f,  14.3858f,47.7169f,-69.6251f,  13.2063f,47.1265f,-71.1292f,  10.9843f,48.9089f,-70.0634f,  9.7074f,48.1408f,-71.3981f,  
	7.4931f,49.7957f,-70.2277f,  6.1609f,48.8465f,-71.3793f,  3.9685f,50.3621f,-70.1135f,  2.6253f,49.2323f,-71.0734f,  0.4714f,50.5968f,-69.7249f,  
	-0.8372f,49.2911f,-70.4897f,  -2.9347f,50.4949f,-69.0738f,  -4.1635f,49.0232f,-69.6451f,  -6.1869f,50.0596f,-68.1791f,  -7.2924f,48.4371f,-68.5631f,  
	-9.2239f,49.3027f,-67.0661f,  -10.1656f,47.5498f,-67.2728f,  -11.9896f,48.2459f,-65.7640f,  -12.7320f,46.3887f,-65.8065f,  -14.4322f,46.9192f,-64.3076f,  
	-14.9455f,44.9886f,-64.2006f,  -16.5107f,45.3624f,-62.7303f,  -16.7722f,43.3936f,-62.4901f,  -18.1934f,43.6237f,-61.0658f,  -18.1883f,41.6554f,-60.7083f,  
	-19.4598f,41.7573f,-59.3454f,  -19.1818f,39.8302f,-58.8856f,  -20.3031f,39.8224f,-57.5945f,  -19.7549f,37.9783f,-57.0461f,  -20.7278f,37.8798f,-55.8345f,  
	-19.9206f,36.1595f,-55.2089f,  -20.7512f,35.9901f,-54.0798f,  -19.7047f,34.4328f,-53.3859f,  -20.4039f,34.2118f,-52.3369f,  40.5360f,-44.1823f,-43.8957f,  
	40.2577f,-42.5866f,-45.7976f,  38.9674f,-41.5455f,-44.6781f,  38.6196f,-39.8989f,-46.4904f,  37.5512f,-38.7872f,-45.2159f,  37.1358f,-37.0929f,-46.9421f,  
	36.3094f,-35.9290f,-45.5405f,  35.8278f,-34.1892f,-47.1839f,  35.2587f,-32.9898f,-45.6874f,  34.7120f,-31.2061f,-47.2512f,  34.4099f,-29.9857f,-45.6952f,  
	33.7984f,-28.1577f,-47.1831f,  33.7673f,-26.9286f,-45.6050f,  33.0911f,-25.0568f,-47.0193f,  33.3293f,-23.8294f,-45.4576f,  32.5883f,-21.9136f,-46.8004f,  
	33.0881f,-20.6961f,-45.2937f,  32.2817f,-18.7358f,-46.5664f,  33.0299f,-17.5349f,-45.1522f,  32.1573f,-15.5303f,-46.3554f,  33.1353f,-14.3508f,-45.0692f,  
	32.1960f,-12.3032f,-46.2024f,  33.3803f,-11.1486f,-45.0769f,  32.3737f,-9.0590f,-46.1392f,  33.7362f,-7.9318f,-45.2037f,  32.6616f,-5.8021f,-46.1936f,  
	34.1706f,-4.7040f,-45.4729f,  33.0275f,-2.5367f,-46.3885f,  34.6480f,-1.4689f,-45.9026f,  33.4361f,0.7331f,-46.7413f,  35.1308f,1.7697f,-46.5052f,  
	33.8502f,4.0024f,-47.2641f,  35.5802f,5.0067f,-47.2869f,  34.2306f,7.2664f,-47.9632f,  35.9565f,8.2369f,-48.2484f,  34.5391f,10.5176f,-48.8382f,  
	36.2216f,11.4523f,-49.3836f,  34.7359f,13.7503f,-49.8841f,  36.3368f,14.6465f,-50.6815f,  34.7835f,16.9564f,-51.0890f,  36.2668f,17.8099f,-52.1251f,  
	34.6476f,20.1249f,-52.4359f,  35.9798f,20.9307f,-53.6921f,  34.2954f,23.2451f,-53.9028f,  35.4468f,23.9968f,-55.3558f,  33.6981f,26.3051f,-55.4628f,  
	34.6436f,26.9947f,-57.0853f,  32.8326f,29.2894f,-57.0852f,  33.5518f,29.9078f,-58.8465f,  31.6808f,32.1814f,-58.7360f,  32.1592f,32.7181f,-60.6028f,  
	30.2296f,34.9638f,-60.3787f,  30.4589f,35.4070f,-62.3160f,  28.4745f,37.6155f,-61.9756f,  28.4525f,37.9526f,-63.9474f,  26.4166f,40.1151f,-63.4884f,  
	26.1480f,40.3328f,-65.4589f,  24.0653f,42.4391f,-64.8797f,  23.5614f,42.5238f,-66.8139f,  21.4377f,44.5630f,-66.1138f,  20.7167f,44.5010f,-67.9789f,  
	18.5592f,46.4613f,-67.1584f,  17.6453f,46.2400f,-68.9242f,  15.4623f,48.1088f,-67.9851f,  14.3858f,47.7169f,-69.6251f,  12.1881f,49.4809f,-68.5715f,  
	10.9843f,48.9089f,-70.0634f,  8.7844f,50.5544f,-68.9014f,  7.4931f,49.7957f,-70.2277f,  5.3043f,51.3105f,-68.9652f,  3.9685f,50.3621f,-70.1135f,  
	1.8064f,51.7336f,-68.7619f,  0.4714f,50.5968f,-69.7249f,  -1.6464f,51.8142f,-68.2982f,  -2.9347f,50.4949f,-69.0738f,  -4.9905f,51.5501f,-67.5882f,  
	-6.1869f,50.0596f,-68.1791f,  -8.1626f,50.9475f,-66.6532f,  -9.2239f,49.3027f,-67.0661f,  -11.1029f,50.0224f,-65.5189f,  -11.9896f,48.2459f,-65.7640f,  
	-13.7544f,48.7992f,-64.2174f,  -14.4322f,46.9192f,-64.3076f,  -16.0699f,47.3131f,-62.7808f,  -16.5107f,45.3624f,-62.7303f,  -18.0103f,45.6080f,-61.2423f,  
	-18.1934f,43.6237f,-61.0658f,  -19.5468f,43.7350f,-59.6335f,  -19.4598f,41.7573f,-59.3454f,  -20.6641f,41.7517f,-57.9811f,  -20.3031f,39.8224f,-57.5945f,  
	-21.3579f,39.7184f,-56.3083f,  -20.7278f,37.8798f,-55.8345f,  -21.6367f,37.6970f,-54.6317f,  -20.7512f,35.9901f,-54.0798f,  -21.5231f,35.7484f,-52.9602f,  
	-20.4039f,34.2118f,-52.3369f,  -21.0487f,33.9289f,-51.2975f,  40.2577f,-42.5866f,-45.7976f,  39.8960f,-40.8752f,-47.6819f,  38.6196f,-39.8989f,-46.4904f,  
	38.1881f,-38.1390f,-48.2839f,  37.1358f,-37.0929f,-46.9421f,  36.6363f,-35.2884f,-48.6469f,  35.8278f,-34.1892f,-47.1839f,  35.2621f,-32.3432f,-48.8025f,  
	34.7120f,-31.2061f,-47.2512f,  34.0810f,-29.3196f,-48.7869f,  33.7984f,-28.1577f,-47.1831f,  33.1031f,-26.2324f,-48.6379f,  33.0911f,-25.0568f,-47.0193f,  
	32.3320f,-23.0937f,-48.3953f,  32.5883f,-21.9136f,-46.8004f,  31.7654f,-19.9130f,-48.0994f,  32.2817f,-18.7358f,-46.5664f,  31.3951f,-16.6989f,-47.7892f,  
	32.1573f,-15.5303f,-46.3554f,  31.2071f,-13.4586f,-47.5019f,  32.1960f,-12.3032f,-46.2024f,  31.1822f,-10.1984f,-47.2724f,  32.3737f,-9.0590f,-46.1392f,  
	31.2960f,-6.9231f,-47.1321f,  32.6616f,-5.8021f,-46.1936f,  31.5202f,-3.6377f,-47.1079f,  33.0275f,-2.5367f,-46.3885f,  31.8223f,-0.3467f,-47.2226f,  
	33.4361f,0.7331f,-46.7413f,  32.1674f,2.9451f,-47.4932f,  33.8502f,4.0024f,-47.2641f,  32.5180f,6.2329f,-47.9316f,  34.2306f,7.2664f,-47.9632f,  
	32.8362f,9.5099f,-48.5434f,  34.5391f,10.5176f,-48.8382f,  33.0819f,12.7713f,-49.3291f,  34.7359f,13.7503f,-49.8841f,  33.2166f,16.0096f,-50.2826f,  
	34.7835f,16.9564f,-51.0890f,  33.2036f,19.2151f,-51.3922f,  34.6476f,20.1249f,-52.4359f,  33.0073f,22.3786f,-52.6410f,  34.2954f,23.2451f,-53.9028f,  
	32.5949f,25.4895f,-54.0068f,  33.6981f,26.3051f,-55.4628f,  31.9390f,28.5339f,-55.4626f,  32.8326f,29.2894f,-57.0852f,  31.0163f,31.4962f,-56.9778f,  
	31.6808f,32.1814f,-58.7360f,  29.8077f,34.3611f,-58.5183f,  30.2296f,34.9638f,-60.3787f,  28.3026f,37.1082f,-60.0481f,  28.4745f,37.6155f,-61.9756f,  
	26.4955f,39.7174f,-61.5294f,  26.4166f,40.1151f,-63.4884f,  24.3891f,42.1657f,-62.9246f,  24.0653f,42.4391f,-64.8797f,  21.9935f,44.4287f,-64.1968f,  
	21.4377f,44.5630f,-66.1138f,  19.3276f,46.4807f,-65.3114f,  18.5592f,46.4613f,-67.1584f,  16.4174f,48.2956f,-66.2373f,  15.4623f,48.1088f,-67.9851f,  
	13.2982f,49.8470f,-66.9480f,  12.1881f,49.4809f,-68.5715f,  10.0127f,51.1096f,-67.4231f,  8.7844f,50.5544f,-68.9014f,  6.6098f,52.0611f,-67.6483f,  
	5.3043f,51.3105f,-68.9652f,  3.1450f,52.6822f,-67.6172f,  1.8064f,51.7336f,-68.7619f,  -0.3204f,52.9586f,-67.3317f,  -1.6464f,51.8142f,-68.2982f,  
	-3.7236f,52.8831f,-66.8007f,  -4.9905f,51.5501f,-67.5882f,  -6.9999f,52.4560f,-66.0412f,  -8.1626f,50.9475f,-66.6532f,  -10.0870f,51.6876f,-65.0751f,  
	-11.1029f,50.0224f,-65.5189f,  -12.9236f,50.5965f,-63.9317f,  -13.7544f,48.7992f,-64.2174f,  -15.4573f,49.2123f,-62.6408f,  -16.0699f,47.3131f,-62.7808f,  
	-17.6420f,47.5743f,-61.2347f,  -18.0103f,45.6080f,-61.2423f,  -19.4413f,45.7297f,-59.7451f,  -19.5468f,43.7350f,-59.6335f,  -20.8317f,43.7332f,-58.1993f,  
	-20.6641f,41.7517f,-57.9811f,  -21.8000f,41.6441f,-56.6220f,  -21.3579f,39.7184f,-56.3083f,  -22.3460f,39.5242f,-55.0321f,  -21.6367f,37.6970f,-54.6317f,  
	-22.4837f,37.4362f,-53.4403f,  -21.5231f,35.7484f,-52.9602f,  -22.2370f,35.4394f,-51.8528f,  -21.0487f,33.9289f,-51.2975f,  -21.6394f,33.5884f,-50.2702f,  
	39.8960f,-40.8752f,-47.6819f,  39.4482f,-39.0496f,-49.5428f,  38.1881f,-38.1390f,-48.2839f,  37.6703f,-36.2685f,-50.0513f,  36.6363f,-35.2884f,-48.6469f,  
	36.0508f,-33.3775f,-50.3225f,  35.2621f,-32.3432f,-48.8025f,  34.6105f,-30.3945f,-50.3889f,  34.0810f,-29.3196f,-48.7869f,  33.3649f,-27.3361f,-50.2858f,  
	33.1031f,-26.2324f,-48.6379f,  32.3236f,-24.2162f,-50.0512f,  32.3320f,-23.0937f,-48.3953f,  31.4901f,-21.0463f,-49.7248f,  31.7654f,-19.9130f,-48.0994f,  
	30.8617f,-17.8364f,-49.3460f,  31.3951f,-16.6989f,-47.7892f,  30.4304f,-14.5953f,-48.9534f,  31.2071f,-13.4586f,-47.5019f,  30.1820f,-11.3305f,-48.5839f,  
	31.1822f,-10.1984f,-47.2724f,  30.0971f,-8.0478f,-48.2721f,  31.2960f,-6.9231f,-47.1321f,  30.1517f,-4.7532f,-48.0487f,  31.5202f,-3.6377f,-47.1079f,  
	30.3169f,-1.4513f,-47.9407f,  31.8223f,-0.3467f,-47.2226f,  30.5608f,1.8527f,-47.9702f,  32.1674f,2.9451f,-47.4932f,  30.8480f,5.1542f,-48.1543f,  
	32.5180f,6.2329f,-47.9316f,  31.1421f,8.4466f,-48.5042f,  32.8362f,9.5099f,-48.5434f,  31.4034f,11.7260f,-49.0260f,  33.0819f,12.7713f,-49.3291f,  
	31.5928f,14.9856f,-49.7197f,  33.2166f,16.0096f,-50.2826f,  31.6724f,18.2170f,-50.5790f,  33.2036f,19.2151f,-51.3922f,  31.6044f,21.4122f,-51.5922f,  
	33.0073f,22.3786f,-52.6410f,  31.3529f,24.5620f,-52.7422f,  32.5949f,25.4895f,-54.0068f,  30.8863f,27.6540f,-54.0066f,  31.9390f,28.5339f,-55.4626f,  
	30.1770f,30.6744f,-55.3584f,  31.0163f,31.4962f,-56.9778f,  29.2003f,33.6092f,-56.7668f,  29.8077f,34.3611f,-58.5183f,  27.9398f,36.4396f,-58.1979f,  
	28.3026f,37.1082f,-60.0481f,  26.3835f,39.1466f,-59.6153f,  26.4955f,39.7174f,-61.5294f,  24.5274f,41.7081f,-60.9820f,  24.3891f,42.1657f,-62.9246f,  
	22.3747f,44.1006f,-62.2605f,  21.9935f,44.4287f,-64.1968f,  19.9373f,46.2980f,-63.4149f,  19.3276f,46.4807f,-65.3114f,  17.2347f,48.2741f,-64.4114f,  
	16.4174f,48.2956f,-66.2373f,  14.2956f,50.0010f,-65.2207f,  13.2982f,49.8470f,-66.9480f,  11.1569f,51.4518f,-65.8182f,  10.0127f,51.1096f,-67.4231f,  
	7.8627f,52.6011f,-66.1852f,  6.6098f,52.0611f,-67.6483f,  4.4644f,53.4263f,-66.3104f,  3.1450f,52.6822f,-67.6172f,  1.0204f,53.9086f,-66.1906f,  
	-0.3204f,52.9586f,-67.3317f,  -2.4079f,54.0358f,-65.8300f,  -3.7236f,52.8831f,-66.8007f,  -5.7557f,53.8026f,-65.2406f,  -6.9999f,52.4560f,-66.0412f,  
	-8.9592f,53.2132f,-64.4404f,  -10.0870f,51.6876f,-65.0751f,  -11.9541f,52.2803f,-63.4553f,  -12.9236f,50.5965f,-63.9317f,  -14.6834f,51.0277f,-62.3126f,  
	-15.4573f,49.2123f,-62.6408f,  -17.0949f,49.4890f,-61.0432f,  -17.6420f,47.5743f,-61.2347f,  -19.1453f,47.7070f,-59.6783f,  -19.4413f,45.7297f,-59.7451f,  
	-20.8031f,45.7330f,-58.2453f,  -20.8317f,43.7332f,-58.1993f,  -22.0466f,43.6238f,-56.7705f,  -21.8000f,41.6441f,-56.6220f,  -22.8670f,41.4405f,-55.2741f,  
	-22.3460f,39.5242f,-55.0321f,  -23.2694f,39.2464f,-53.7692f,  -22.4837f,37.4362f,-53.4403f,  -23.2696f,37.1029f,-52.2638f,  -22.2370f,35.4394f,-51.8528f,  
	-22.8933f,35.0680f,-50.7608f,  -21.6394f,33.5884f,-50.2702f,  -22.1806f,33.1957f,-49.2547f,  39.4482f,-39.0496f,-49.5428f,  38.9122f,-37.1127f,-51.3731f,  
	37.6703f,-36.2685f,-50.0513f,  37.0646f,-34.2913f,-51.7851f,  36.0508f,-33.3775f,-50.3225f,  35.3776f,-31.3639f,-51.9617f,  34.6105f,-30.3945f,-50.3889f,  
	33.8722f,-28.3490f,-51.9347f,  33.3649f,-27.3361f,-50.2858f,  32.5633f,-25.2620f,-51.7397f,  32.3236f,-24.2162f,-50.0512f,  31.4602f,-22.1163f,-51.4148f,  
	31.4901f,-21.0463f,-49.7248f,  30.5664f,-18.9234f,-50.9992f,  30.8617f,-17.8364f,-49.3460f,  29.8792f,-15.6938f,-50.5320f,  30.4304f,-14.5953f,-48.9534f,  
	29.3904f,-12.4361f,-50.0513f,  30.1820f,-11.3305f,-48.5839f,  29.0856f,-9.1572f,-49.5943f,  30.0971f,-8.0478f,-48.2721f,  28.9455f,-5.8640f,-49.1947f,  
	30.1517f,-4.7532f,-48.0487f,  28.9457f,-2.5618f,-48.8834f,  30.3169f,-1.4513f,-47.9407f,  29.0577f,0.7439f,-48.6869f,  30.5608f,1.8527f,-47.9702f,  
	29.2492f,4.0486f,-48.6273f,  30.8480f,5.1542f,-48.1543f,  29.4857f,7.3459f,-48.7211f,  31.1421f,8.4466f,-48.5042f,  29.7288f,10.6323f,-48.9801f,  
	31.4034f,11.7260f,-49.0260f,  29.9399f,13.9021f,-49.4099f,  31.5928f,14.9856f,-49.7197f,  30.0803f,17.1476f,-50.0100f,  31.6724f,18.2170f,-50.5790f,  
	30.1109f,20.3622f,-50.7742f,  31.6044f,21.4122f,-51.5922f,  29.9933f,23.5383f,-51.6908f,  31.3529f,24.5620f,-52.7422f,  29.6926f,26.6650f,-52.7421f,  
	30.8863f,27.6540f,-54.0066f,  29.1774f,29.7299f,-53.9056f,  30.1770f,30.6744f,-55.3584f,  28.4179f,32.7211f,-55.1540f,  29.2003f,33.6092f,-56.7668f,  
	27.3923f,35.6210f,-56.4565f,  27.9398f,36.4396f,-58.1979f,  26.0824f,38.4123f,-57.7790f,  26.3835f,39.1466f,-59.6153f,  24.4778f,41.0742f,-59.0852f,  
	24.5274f,41.7081f,-60.9820f,  22.5747f,43.5841f,-60.3382f,  22.3747f,44.1006f,-62.2605f,  20.3780f,45.9163f,-61.5012f,  19.9373f,46.2980f,-63.4149f,  
	17.9002f,48.0446f,-62.5389f,  17.2347f,48.2741f,-64.4114f,  15.1633f,49.9403f,-63.4191f,  14.2956f,50.0010f,-65.2207f,  12.1975f,51.5751f,-64.1140f,  
	11.1569f,51.4518f,-65.8182f,  9.0416f,52.9212f,-64.6009f,  7.8627f,52.6011f,-66.1852f,  5.7419f,53.9531f,-64.8639f,  4.4644f,53.4263f,-66.3104f,  
	2.3531f,54.6478f,-64.8947f,  1.0204f,53.9086f,-66.1906f,  -1.0658f,54.9883f,-64.6926f,  -2.4079f,54.0358f,-65.8300f,  -4.4514f,54.9640f,-64.2651f,  
	-5.7557f,53.8026f,-65.2406f,  -7.7390f,54.5731f,-63.6256f,  -8.9592f,53.2132f,-64.4404f,  -10.8625f,53.8218f,-62.7963f,  -11.9541f,52.2803f,-63.4553f,  
	-13.7614f,52.7280f,-61.8019f,  -14.6834f,51.0277f,-62.3126f,  -16.3783f,51.3194f,-60.6711f,  -17.0949f,49.4890f,-61.0432f,  -18.6637f,49.6333f,-59.4341f,  
	-19.1453f,47.7070f,-59.6783f,  -20.5787f,47.7169f,-58.1185f,  -20.8031f,45.7330f,-58.2453f,  -22.0936f,45.6237f,-56.7510f,  -22.0466f,43.6238f,-56.7705f,  
	-23.1908f,43.4130f,-55.3536f,  -22.8670f,41.4405f,-55.2741f,  -23.8667f,41.1479f,-53.9411f,  -23.2694f,39.2464f,-53.7692f,  -24.1287f,38.8908f,-52.5234f,  
	-23.2696f,37.1029f,-52.2638f,  -23.9944f,36.7022f,-51.1057f,  -22.8933f,35.0680f,-50.7608f,  -23.4963f,34.6402f,-49.6842f,  -22.1806f,33.1957f,-49.2547f,  
	-22.6738f,32.7551f,-48.2523f,  38.9122f,-37.1127f,-51.3731f,  38.2861f,-35.0689f,-53.1654f,  37.0646f,-34.2913f,-51.7851f,  36.3694f,-32.2113f,-53.4785f,  
	35.3776f,-31.3639f,-51.9617f,  34.6161f,-29.2538f,-53.5564f,  33.8722f,-28.3490f,-51.9347f,  33.0468f,-26.2132f,-53.4319f,  32.5633f,-25.2620f,-51.7397f,  
	31.6765f,-23.1048f,-53.1406f,  31.4602f,-22.1163f,-51.4148f,  30.5141f,-19.9414f,-52.7206f,  30.5664f,-18.9234f,-50.9992f,  29.5631f,-16.7352f,-52.2106f,  
	29.8792f,-15.6938f,-50.5320f,  28.8208f,-13.4961f,-51.6495f,  29.3904f,-12.4361f,-50.0513f,  28.2786f,-10.2321f,-51.0760f,  29.0856f,-9.1572f,-49.5943f,  
	27.9222f,-6.9511f,-50.5263f,  28.9455f,-5.8640f,-49.1947f,  27.7321f,-3.6592f,-50.0345f,  28.9457f,-2.5618f,-48.8834f,  27.6840f,-0.3622f,-49.6311f,  
	29.0577f,0.7439f,-48.6869f,  27.7490f,2.9351f,-49.3426f,  29.2492f,4.0486f,-48.6273f,  27.8955f,6.2266f,-49.1906f,  29.4857f,7.3459f,-48.7211f,  
	28.0871f,9.5090f,-49.1922f,  29.7288f,10.6323f,-48.9801f,  28.2862f,12.7774f,-49.3586f,  29.9399f,13.9021f,-49.4099f,  28.4546f,16.0252f,-49.6950f,  
	30.0803f,17.1476f,-50.0100f,  28.5522f,19.2468f,-50.2011f,  30.1109f,20.3622f,-50.7742f,  28.5393f,22.4360f,-50.8704f,  29.9933f,23.5383f,-51.6908f,  
	28.3784f,25.5837f,-51.6906f,  29.6926f,26.6650f,-52.7421f,  28.0345f,28.6789f,-52.6440f,  29.1774f,29.7299f,-53.9056f,  27.4738f,31.7118f,-53.7076f,  
	28.4179f,32.7211f,-55.1540f,  26.6693f,34.6665f,-54.8539f,  27.3923f,35.6210f,-56.4565f,  25.5974f,37.5273f,-56.0517f,  26.0824f,38.4123f,-57.7790f,  
	24.2412f,40.2749f,-57.2667f,  24.4778f,41.0742f,-59.0852f,  22.5900f,42.8881f,-58.4627f,  22.5747f,43.5841f,-60.3382f,  20.6422f,45.3421f,-59.6031f,  
	20.3780f,45.9163f,-61.5012f,  18.4025f,47.6110f,-60.6516f,  17.9002f,48.0446f,-62.5389f,  15.8863f,49.6659f,-61.5741f,  15.1633f,49.9403f,-63.4191f,  
	13.1166f,51.4775f,-62.3398f,  12.1975f,51.5751f,-64.1140f,  10.1262f,53.0161f,-62.9227f,  9.0416f,52.9212f,-64.6009f,  6.9556f,54.2537f,-63.3023f,  
	5.7419f,53.9531f,-64.8639f,  3.6550f,55.1636f,-63.4660f,  2.3531f,54.6478f,-64.8947f,  0.2796f,55.7246f,-63.4081f,  -1.0658f,54.9883f,-64.6926f,  
	-3.1091f,55.9206f,-63.1313f,  -4.4514f,54.9640f,-64.2651f,  -6.4471f,55.7439f,-62.6446f,  -7.7390f,54.5731f,-63.6256f,  -9.6675f,55.1947f,-61.9662f,  
	-10.8625f,53.8218f,-62.7963f,  -12.7070f,54.2843f,-61.1174f,  -13.7614f,52.7280f,-61.8019f,  -15.5045f,53.0340f,-60.1246f,  -16.3783f,51.3194f,-60.6711f,  
	-18.0047f,51.4755f,-59.0168f,  -18.6637f,49.6333f,-59.4341f,  -20.1623f,49.6508f,-57.8209f,  -20.5787f,47.7169f,-58.1185f,  -21.9400f,47.6095f,-56.5640f,  
	-22.0936f,45.6237f,-56.7510f,  -23.3118f,45.4081f,-55.2693f,  -23.1908f,43.4130f,-55.3536f,  -24.2653f,43.1083f,-53.9531f,  -23.8667f,41.1479f,-53.9411f,  
	-24.7996f,40.7726f,-52.6272f,  -24.1287f,38.8908f,-52.5234f,  -24.9239f,38.4631f,-51.2990f,  -23.9944f,36.7022f,-51.1057f,  -24.6624f,36.2411f,-49.9665f,  
	-23.4963f,34.6402f,-49.6842f,  -24.0475f,34.1609f,-48.6247f,  -22.6738f,32.7551f,-48.2523f,  -23.1208f,32.2705f,-47.2640f,  38.2861f,-35.0689f,-53.1654f,  
	37.5687f,-32.9221f,-54.9131f,  36.3694f,-32.2113f,-53.4785f,  35.5841f,-30.0349f,-55.1232f,  34.6161f,-29.2538f,-53.5564f,  33.7661f,-27.0535f,-55.0988f,  
	33.0468f,-26.2132f,-53.4319f,  32.1350f,-23.9947f,-54.8726f,  31.6765f,-23.1048f,-53.1406f,  30.7057f,-20.8730f,-54.4806f,  30.5141f,-19.9414f,-52.7206f,  
	29.4873f,-17.7017f,-53.9605f,  29.5631f,-16.7352f,-52.2106f,  28.4831f,-14.4924f,-53.3511f,  28.8208f,-13.4961f,-51.6495f,  27.6899f,-11.2542f,-52.6919f,  
	28.2786f,-10.2321f,-51.0760f,  27.0994f,-7.9960f,-52.0208f,  27.9222f,-6.9511f,-50.5263f,  26.6970f,-4.7247f,-51.3744f,  27.7321f,-3.6592f,-50.0345f,  
	26.4631f,-1.4468f,-50.7866f,  27.6840f,-0.3622f,-49.6311f,  26.3731f,1.8327f,-50.2879f,  27.7490f,2.9351f,-49.3426f,  26.3986f,5.1078f,-49.9045f,  
	27.8955f,6.2266f,-49.1906f,  26.5061f,8.3754f,-49.6585f,  28.0871f,9.5090f,-49.1922f,  26.6600f,11.6310f,-49.5666f,  28.2862f,12.7774f,-49.3586f,  
	26.8229f,14.8690f,-49.6395f,  28.4546f,16.0252f,-49.6950f,  26.9551f,18.0850f,-49.8826f,  28.5522f,19.2468f,-50.2011f,  27.0158f,21.2740f,-50.2951f,  
	28.5393f,22.4360f,-50.8704f,  26.9659f,24.4286f,-50.8703f,  28.3784f,25.5837f,-51.6906f,  26.7680f,27.5395f,-51.5955f,  28.0345f,28.6789f,-52.6440f,  
	26.3841f,30.5987f,-52.4523f,  27.4738f,31.7118f,-53.7076f,  25.7835f,33.5923f,-53.4174f,  26.6693f,34.6665f,-54.8539f,  24.9368f,36.5066f,-54.4630f,  
	25.5974f,37.5273f,-56.0517f,  23.8216f,39.3238f,-55.5574f,  24.2412f,40.2749f,-57.2667f,  22.4206f,42.0247f,-56.6661f,  22.5900f,42.8881f,-58.4627f,  
	20.7251f,44.5853f,-57.7531f,  20.6422f,45.3421f,-59.6031f,  18.7331f,46.9807f,-58.7819f,  18.4025f,47.6110f,-60.6516f,  16.4523f,49.1824f,-59.7172f,  
	15.8863f,49.6659f,-61.5741f,  13.8986f,51.1605f,-60.5260f,  13.1166f,51.4775f,-62.3398f,  11.0981f,52.8841f,-61.1791f,  10.1262f,53.0161f,-62.9227f,  
	8.0849f,54.3229f,-61.6524f,  6.9556f,54.2537f,-63.3023f,  4.9037f,55.4473f,-61.9290f,  3.6550f,55.1636f,-63.4660f,  1.6054f,56.2318f,-61.9985f,  
	0.2796f,55.7246f,-63.4081f,  -1.7519f,56.6558f,-61.8586f,  -3.1091f,55.9206f,-63.1313f,  -5.1057f,56.7056f,-61.5142f,  -6.4471f,55.7439f,-62.6446f,  
	-8.3894f,56.3754f,-60.9790f,  -9.6675f,55.1947f,-61.9662f,  -11.5384f,55.6698f,-60.2708f,  -12.7070f,54.2843f,-61.1174f,  -14.4885f,54.6036f,-59.4133f,  
	-15.5045f,53.0340f,-60.1246f,  -17.1798f,53.2022f,-58.4336f,  -18.0047f,51.4755f,-59.0168f,  -19.5612f,51.5018f,-57.3577f,  -20.1623f,49.6508f,-57.8209f,  
	-21.5885f,49.5473f,-56.2126f,  -21.9400f,47.6095f,-56.5640f,  -23.2279f,47.3916f,-55.0225f,  -23.3118f,45.4081f,-55.2693f,  -24.4586f,45.0939f,-53.8050f,  
	-24.2653f,43.1083f,-53.9531f,  -25.2710f,42.7161f,-52.5735f,  -24.7996f,40.7726f,-52.6272f,  -25.6659f,40.3206f,-51.3374f,  -24.9239f,38.4631f,-51.2990f,  
	-25.6590f,37.9708f,-50.0968f,  -24.6624f,36.2411f,-49.9665f,  -25.2751f,35.7246f,-48.8482f,  -24.0475f,34.1609f,-48.6247f,  -24.5490f,33.6345f,-47.5833f,  
	-23.1208f,32.2705f,-47.2640f,  -23.5217f,31.7458f,-46.2916f,  37.5687f,-32.9221f,-54.9131f,  36.7595f,-30.6790f,-56.6083f,  35.5841f,-30.0349f,-55.1232f,  
	34.7087f,-27.7685f,-56.7120f,  33.7661f,-27.0535f,-55.0988f,  32.8281f,-24.7709f,-56.5812f,  32.1350f,-23.9947f,-54.8726f,  31.1380f,-21.7021f,-56.2492f,  
	30.7057f,-20.8730f,-54.4806f,  29.6532f,-18.5768f,-55.7518f,  29.4873f,-17.7017f,-53.9605f,  28.3829f,-15.4080f,-55.1270f,  28.4831f,-14.4924f,-53.3511f,  
	27.3297f,-12.2059f,-54.4142f,  27.6899f,-11.2542f,-52.6919f,  26.4911f,-8.9807f,-53.6525f,  27.0994f,-7.9960f,-52.0208f,  25.8580f,-5.7401f,-52.8801f,  
	26.6970f,-4.7247f,-51.3744f,  25.4159f,-2.4913f,-52.1337f,  26.4631f,-1.4468f,-50.7866f,  25.1449f,0.7603f,-51.4471f,  26.3731f,1.8327f,-50.2879f,  
	25.0206f,4.0087f,-50.8507f,  26.3986f,5.1078f,-49.9045f,  25.0130f,7.2507f,-50.3712f,  26.5061f,8.3754f,-49.6585f,  25.0891f,10.4824f,-50.0303f,  
	26.6600f,11.6310f,-49.5666f,  25.2133f,13.6988f,-49.8443f,  26.8229f,14.8690f,-49.6395f,  25.3468f,16.8966f,-49.8242f,  26.9551f,18.0850f,-49.8826f,  
	25.4489f,20.0722f,-49.9748f,  27.0158f,21.2740f,-50.2951f,  25.4796f,23.2195f,-50.2951f,  26.9659f,24.4286f,-50.8703f,  25.3994f,26.3310f,-50.7778f,  
	26.7680f,27.5395f,-51.5955f,  25.1676f,29.4009f,-51.4095f,  26.3841f,30.5987f,-52.4523f,  24.7497f,32.4169f,-52.1717f,  25.7835f,33.5923f,-53.4174f,  
	24.1118f,35.3676f,-53.0402f,  24.9368f,36.5066f,-54.4630f,  23.2262f,38.2372f,-53.9868f,  23.8216f,39.3238f,-55.5574f,  22.0691f,41.0084f,-54.9791f,  
	22.4206f,42.0247f,-56.6661f,  20.6256f,43.6588f,-55.9829f,  20.7251f,44.5853f,-57.7531f,  18.8863f,46.1646f,-56.9619f,  18.7331f,46.9807f,-58.7819f,  
	16.8516f,48.4982f,-57.8803f,  16.4523f,49.1824f,-59.7172f,  14.5301f,50.6297f,-58.7034f,  13.8986f,51.1605f,-60.5260f,  11.9406f,52.5274f,-59.4000f,  
	11.0981f,52.8841f,-61.1791f,  9.1104f,54.1595f,-59.9425f,  8.0849f,54.3229f,-61.6524f,  6.0779f,55.4938f,-60.3100f,  4.9037f,55.4473f,-61.9290f,  
	2.8888f,56.5014f,-60.4878f,  1.6054f,56.2318f,-61.9985f,  -0.4030f,57.1571f,-60.4690f,  -1.7519f,56.6558f,-61.8586f,  -3.7377f,57.4418f,-60.2538f,  
	-5.1057f,56.7056f,-61.5142f,  -7.0502f,57.3437f,-59.8516f,  -8.3894f,56.3754f,-60.9790f,  -10.2755f,56.8609f,-59.2765f,  -11.5384f,55.6698f,-60.2708f,  
	-13.3475f,56.0013f,-58.5491f,  -14.4885f,54.6036f,-59.4133f,  -16.2029f,54.7837f,-57.6943f,  -17.1798f,53.2022f,-58.4336f,  -18.7856f,53.2382f,-56.7367f,  
	-19.5612f,51.5018f,-57.3577f,  -21.0451f,51.4040f,-55.7030f,  -21.5885f,49.5473f,-56.2126f,  -22.9406f,49.3295f,-54.6175f,  -23.2279f,47.3916f,-55.0225f,  
	-24.4431f,47.0709f,-53.4993f,  -24.4586f,45.0939f,-53.8050f,  -25.5346f,44.6879f,-52.3631f,  -25.2710f,42.7161f,-52.5735f,  -26.2076f,42.2428f,-51.2203f,  
	-25.6659f,40.3206f,-51.3374f,  -26.4689f,39.7999f,-50.0728f,  -25.6590f,37.9708f,-50.0968f,  -26.3354f,37.4195f,-48.9191f,  -25.2751f,35.7246f,-48.8482f,  
	-25.8345f,35.1580f,-47.7521f,  -24.5490f,33.6345f,-47.5833f,  -25.0009f,33.0653f,-46.5622f,  -23.5217f,31.7458f,-46.2916f,  -23.8845f,31.1855f,-45.3323f,  
	36.7595f,-30.6790f,-56.6083f,  35.8585f,-28.3460f,-58.2438f,  34.7087f,-27.7685f,-56.7120f,  33.7439f,-25.4201f,-58.2371f,  32.8281f,-24.7709f,-56.5812f,  
	31.8035f,-22.4146f,-57.9960f,  31.1380f,-21.7021f,-56.2492f,  30.0579f,-19.3455f,-57.5538f,  29.6532f,-18.5768f,-55.7518f,  28.5219f,-16.2272f,-56.9467f,  
	28.3829f,-15.4080f,-55.1270f,  27.2042f,-13.0711f,-56.2136f,  27.3297f,-12.2059f,-54.4142f,  26.1078f,-9.8885f,-55.3935f,  26.4911f,-8.9807f,-53.6525f,  
	25.2295f,-6.6880f,-54.5259f,  25.8580f,-5.7401f,-52.8801f,  24.5604f,-3.4777f,-53.6493f,  25.4159f,-2.4913f,-52.1337f,  24.0855f,-0.2637f,-52.8003f,  
	25.1449f,0.7603f,-51.4471f,  23.7851f,2.9481f,-52.0130f,  25.0206f,4.0087f,-50.8507f,  23.6333f,6.1543f,-51.3180f,  25.0130f,7.2507f,-50.3712f,  
	23.6004f,9.3512f,-50.7419f,  25.0891f,10.4824f,-50.0303f,  23.6533f,12.5346f,-50.3060f,  25.2133f,13.6988f,-49.8443f,  23.7550f,15.7020f,-50.0268f,  
	25.3468f,16.8966f,-49.8242f,  23.8655f,18.8510f,-49.9150f,  25.4489f,20.0722f,-49.9748f,  23.9447f,21.9770f,-49.9749f,  25.4796f,23.2195f,-50.2951f,  
	23.9521f,25.0742f,-50.2049f,  25.3994f,26.3310f,-50.7778f,  23.8450f,28.1387f,-50.5972f,  25.1676f,29.4009f,-51.4095f,  23.5858f,31.1603f,-51.1380f,  
	24.7497f,32.4169f,-52.1717f,  23.1367f,34.1299f,-51.8076f,  24.1118f,35.3676f,-53.0402f,  22.4651f,37.0337f,-52.5816f,  23.2262f,38.2372f,-53.9868f,  
	21.5417f,39.8567f,-53.4307f,  22.0691f,41.0084f,-54.9791f,  20.3452f,42.5784f,-54.3227f,  20.6256f,43.6588f,-55.9829f,  18.8594f,45.1767f,-55.2225f,  
	18.8863f,46.1646f,-56.9619f,  17.0774f,47.6249f,-56.0946f,  16.8516f,48.4982f,-57.8803f,  15.0002f,49.8941f,-56.9034f,  14.5301f,50.6297f,-58.7034f,  
	12.6393f,51.9521f,-57.6159f,  11.9406f,52.5274f,-59.4000f,  10.0145f,53.7664f,-58.2018f,  9.1104f,54.1595f,-59.9425f,  7.1575f,55.3025f,-58.6367f,  
	6.0779f,55.4938f,-60.3100f,  4.1079f,56.5288f,-58.9019f,  2.8888f,56.5014f,-60.4878f,  0.9146f,57.4159f,-58.9860f,  -0.4030f,57.1571f,-60.4690f,  
	-2.3665f,57.9399f,-58.8849f,  -3.7377f,57.4418f,-60.2538f,  -5.6727f,58.0831f,-58.6034f,  -7.0502f,57.3437f,-59.8516f,  -8.9400f,57.8372f,-58.1517f,  
	-10.2755f,56.8609f,-59.2765f,  -12.1012f,57.2032f,-57.5470f,  -13.3475f,56.0013f,-58.5491f,  -15.0908f,56.1931f,-56.8117f,  -16.2029f,54.7837f,-57.6943f,  
	-17.8487f,54.8302f,-55.9686f,  -18.7856f,53.2382f,-56.7367f,  -20.3192f,53.1476f,-55.0437f,  -21.0451f,51.4040f,-55.7030f,  -22.4548f,51.1887f,-54.0613f,  
	-22.9406f,49.3295f,-54.6175f,  -24.2191f,49.0054f,-53.0413f,  -24.4431f,47.0709f,-53.4993f,  -25.5861f,46.6545f,-51.9997f,  -25.5346f,44.6879f,-52.3631f,  
	-26.5399f,44.1968f,-50.9496f,  -26.2076f,42.2428f,-51.2203f,  -27.0784f,41.6970f,-49.8950f,  -26.4689f,39.7999f,-50.0728f,  -27.2104f,39.2167f,-48.8361f,  
	-26.3354f,37.4195f,-48.9191f,  -26.9554f,36.8150f,-47.7673f,  -25.8345f,35.1580f,-47.7521f,  -26.3412f,34.5457f,-46.6803f,  -25.0009f,33.0653f,-46.5622f,  
	-25.4109f,32.4587f,-45.5583f,  -23.8845f,31.1855f,-45.3323f,  -24.2068f,30.5925f,-44.3887f,  35.8585f,-28.3460f,-58.2438f,  34.8666f,-25.9312f,-59.8120f,  
	33.7439f,-25.4201f,-58.2371f,  32.6909f,-22.9984f,-59.6913f,  31.8035f,-22.4146f,-57.9960f,  30.6945f,-19.9947f,-59.3358f,  30.0579f,-19.3455f,-57.5538f,  
	28.8979f,-16.9360f,-58.7793f,  28.5219f,-16.2272f,-56.9467f,  27.3154f,-13.8349f,-58.0593f,  27.2042f,-13.0711f,-56.2136f,  25.9559f,-10.7037f,-57.2140f,  
	26.1078f,-9.8885f,-55.3935f,  24.8222f,-7.5523f,-56.2836f,  25.2295f,-6.6880f,-54.5259f,  23.9111f,-4.3894f,-55.3074f,  24.5604f,-3.4777f,-53.6493f,  
	23.2130f,-1.2218f,-54.3245f,  24.0855f,-0.2637f,-52.8003f,  22.7133f,1.9441f,-53.3714f,  23.7851f,2.9481f,-52.0130f,  22.3907f,5.1048f,-52.4828f,  
	23.6333f,6.1543f,-51.3180f,  22.2194f,8.2568f,-51.6891f,  23.6004f,9.3512f,-50.7419f,  22.1696f,11.3964f,-51.0167f,  23.6533f,12.5346f,-50.3060f,  
	22.2067f,14.5217f,-50.4872f,  23.7550f,15.7020f,-50.0268f,  22.2925f,17.6315f,-50.1166f,  23.8655f,18.8510f,-49.9150f,  22.3875f,20.7225f,-49.9151f,  
	23.9447f,21.9770f,-49.9749f,  22.4511f,23.7906f,-49.8868f,  23.9521f,25.0742f,-50.2049f,  22.4390f,26.8339f,-50.0292f,  23.8450f,28.1387f,-50.5972f,  
	22.3117f,29.8442f,-50.3339f,  23.5858f,31.1603f,-51.1380f,  22.0281f,32.8146f,-50.7863f,  23.1367f,34.1299f,-51.8076f,  21.5515f,35.7339f,-51.3660f,  
	22.4651f,37.0337f,-52.5816f,  20.8474f,38.5894f,-52.0473f,  21.5417f,39.8567f,-53.4307f,  19.8888f,41.3626f,-52.8009f,  20.3452f,42.5784f,-54.3227f,  
	18.6530f,44.0337f,-53.5937f,  18.8594f,45.1767f,-55.2225f,  17.1259f,46.5775f,-54.3909f,  17.0774f,47.6249f,-56.0946f,  15.3009f,48.9664f,-55.1568f,  
	15.0002f,49.8941f,-56.9034f,  13.1822f,51.1682f,-55.8573f,  12.6393f,51.9521f,-57.6159f,  10.7818f,53.1501f,-56.4600f,  10.0145f,53.7664f,-58.2018f,  
	8.1241f,54.8766f,-56.9377f,  7.1575f,55.3025f,-58.6367f,  5.2419f,56.3134f,-57.2679f,  4.1079f,56.5288f,-58.9019f,  2.1782f,57.4279f,-57.4351f,  
	0.9146f,57.4159f,-58.9860f,  -1.0156f,58.1914f,-57.4309f,  -2.3665f,57.9399f,-58.8849f,  -4.2806f,58.5810f,-57.2557f,  -5.6727f,58.0831f,-58.6034f,  
	-7.5546f,58.5820f,-56.9155f,  -8.9400f,57.8372f,-58.1517f,  -10.7708f,58.1888f,-56.4241f,  -12.1012f,57.2032f,-57.5470f,  -13.8625f,57.4062f,-55.8008f,  
	-15.0908f,56.1931f,-56.8117f,  -16.7666f,56.2505f,-55.0666f,  -17.8487f,54.8302f,-55.9686f,  -19.4231f,54.7485f,-54.2460f,  -20.3192f,53.1476f,-55.0437f,  
	-21.7789f,52.9374f,-53.3633f,  -22.4548f,51.1887f,-54.0613f,  -23.7905f,50.8642f,-52.4387f,  -24.2191f,49.0054f,-53.0413f,  -25.4245f,48.5821f,-51.4895f,  
	-25.5861f,46.6545f,-51.9997f,  -26.6570f,46.1494f,-50.5300f,  -26.5399f,44.1968f,-50.9496f,  -27.4771f,43.6296f,-49.5664f,  -27.0784f,41.6970f,-49.8950f,  
	-27.8850f,41.0852f,-48.6006f,  -27.2104f,39.2167f,-48.8361f,  -27.8925f,38.5770f,-47.6287f,  -26.9554f,36.8150f,-47.7673f,  -27.5197f,36.1618f,-46.6437f,  
	-26.3412f,34.5457f,-46.6803f,  -26.8024f,33.8942f,-45.6298f,  -25.4109f,32.4587f,-45.5583f,  -25.7773f,31.8177f,-44.5745f,  -24.2068f,30.5925f,-44.3887f,  
	-24.4928f,29.9704f,-43.4602f,  34.8666f,-25.9312f,-59.8120f,  33.7850f,-23.4433f,-61.3059f,  32.6909f,-22.9984f,-59.6913f,  31.5522f,-20.5133f,-61.0671f,  
	30.6945f,-19.9947f,-59.3358f,  29.5044f,-17.5223f,-60.5933f,  28.8979f,-16.9360f,-58.7793f,  27.6614f,-14.4842f,-59.9196f,  27.3154f,-13.8349f,-58.0593f,  
	26.0382f,-11.4124f,-59.0830f,  25.9559f,-10.7037f,-57.2140f,  24.6432f,-8.3180f,-58.1230f,  24.8222f,-7.5523f,-56.2836f,  23.4792f,-5.2106f,-57.0798f,  
	23.9111f,-4.3894f,-55.3074f,  22.5424f,-2.0976f,-55.9934f,  23.2130f,-1.2218f,-54.3245f,  21.8235f,1.0140f,-54.9028f,  22.7133f,1.9441f,-53.3714f,  
	21.3063f,4.1204f,-53.8455f,  22.3907f,5.1048f,-52.4828f,  20.9696f,7.2180f,-52.8558f,  22.2194f,8.2568f,-51.6891f,  20.7875f,10.3035f,-51.9642f,  
	22.1696f,11.3964f,-51.0167f,  20.7286f,13.3759f,-51.1972f,  22.2067f,14.5217f,-50.4872f,  20.7570f,16.4344f,-50.5762f,  22.2925f,17.6315f,-50.1166f,  
	20.8348f,19.4774f,-50.1168f,  22.3875f,20.7225f,-49.9151f,  20.9218f,22.5022f,-49.8287f,  22.4511f,23.7906f,-49.8868f,  20.9737f,25.5087f,-49.7152f,  
	22.4390f,26.8339f,-50.0292f,  20.9492f,28.4909f,-49.7734f,  22.3117f,29.8442f,-50.3339f,  20.8049f,31.4443f,-49.9937f,  22.0281f,32.8146f,-50.7863f,  
	20.5008f,34.3600f,-50.3606f,  21.5515f,35.7339f,-51.3660f,  19.9980f,37.2280f,-50.8526f,  20.8474f,38.5894f,-52.0473f,  19.2642f,40.0323f,-51.4437f,  
	19.8888f,41.3626f,-52.8009f,  18.2705f,42.7553f,-52.1032f,  18.6530f,44.0337f,-53.5937f,  16.9961f,45.3740f,-52.7981f,  17.1259f,46.5775f,-54.3909f,  
	15.4271f,47.8622f,-53.4933f,  15.3009f,48.9664f,-55.1568f,  13.5600f,50.1889f,-54.1542f,  13.1822f,51.1682f,-55.8573f,  11.3991f,52.3214f,-54.7471f,  
	10.7818f,53.1501f,-56.4600f,  8.9610f,54.2234f,-55.2422f,  8.1241f,54.8766f,-56.9377f,  6.2713f,55.8590f,-55.6138f,  5.2419f,56.3134f,-57.2679f,  
	3.3663f,57.1927f,-55.8428f,  2.1782f,57.4279f,-57.4351f,  0.2918f,58.1919f,-55.9168f,  -1.0156f,58.1914f,-57.4309f,  -2.8977f,58.8287f,-55.8315f,  
	-4.2806f,58.5810f,-57.2557f,  -6.1432f,59.0825f,-55.5891f,  -7.5546f,58.5820f,-56.9155f,  -9.3792f,58.9411f,-55.1996f,  -10.7708f,58.1888f,-56.4241f,  
	-12.5391f,58.4022f,-54.6790f,  -13.8625f,57.4062f,-55.8008f,  -15.5578f,57.4750f,-54.0460f,  -16.7666f,56.2505f,-55.0666f,  -18.3722f,56.1793f,-53.3237f,  
	-19.4231f,54.7485f,-54.2460f,  -20.9243f,54.5457f,-52.5355f,  -21.7789f,52.9374f,-53.3633f,  -23.1645f,52.6157f,-51.7020f,  -23.7905f,50.8642f,-52.4387f,  
	-25.0526f,50.4378f,-50.8411f,  -25.4245f,48.5821f,-51.4895f,  -26.5569f,48.0669f,-49.9687f,  -26.6570f,46.1494f,-50.5300f,  -27.6580f,45.5648f,-49.0927f,  
	-27.4771f,43.6296f,-49.5664f,  -28.3478f,42.9933f,-48.2168f,  -27.8850f,41.0852f,-48.6006f,  -28.6297f,40.4140f,-47.3388f,  -27.8925f,38.5770f,-47.6287f,  
	-28.5163f,37.8860f,-46.4532f,  -27.5197f,36.1618f,-46.6437f,  -28.0351f,35.4676f,-45.5455f,  -26.8024f,33.8942f,-45.6298f,  -27.2169f,33.2066f,-44.6036f,  
	-25.7773f,31.8177f,-44.5745f,  -26.1041f,31.1465f,-43.6100f,  -24.4928f,29.9704f,-43.4602f,  -24.7456f,29.3220f,-42.5460f,  33.7850f,-23.4433f,-61.3059f,  
	32.6162f,-20.8924f,-62.7182f,  31.5522f,-20.5133f,-61.0671f,  30.3310f,-17.9761f,-62.3577f,  29.5044f,-17.5223f,-60.5933f,  28.2364f,-15.0079f,-61.7627f,  
	27.6614f,-14.4842f,-59.9196f,  26.3531f,-12.0026f,-60.9684f,  26.0382f,-11.4124f,-59.0830f,  24.6955f,-8.9722f,-60.0129f,  24.6432f,-8.3180f,-58.1230f,  
	23.2721f,-5.9273f,-58.9360f,  23.4792f,-5.2106f,-57.0798f,  22.0851f,-2.8763f,-57.7785f,  22.5424f,-2.0976f,-55.9934f,  21.1309f,0.1735f,-56.5809f,  
	21.8235f,1.0140f,-54.9028f,  20.3988f,3.2177f,-55.3829f,  21.3063f,4.1204f,-53.8455f,  19.8726f,6.2524f,-54.2219f,  20.9696f,7.2180f,-52.8558f,  
	19.5308f,9.2748f,-53.1323f,  20.7875f,10.3035f,-51.9642f,  19.3458f,12.2840f,-52.1449f,  20.7286f,13.3759f,-51.1972f,  19.2852f,15.2803f,-51.2860f,  
	20.7570f,16.4344f,-50.5762f,  19.3129f,18.2629f,-50.5765f,  20.8348f,19.4774f,-50.1168f,  19.3905f,21.2309f,-50.0318f,  20.9218f,22.5022f,-49.8287f,  
	19.4740f,24.1857f,-49.6607f,  20.9737f,25.5087f,-49.7152f,  19.5215f,27.1237f,-49.4659f,  20.9492f,28.4909f,-49.7734f,  19.4881f,30.0424f,-49.4434f,  
	20.8049f,31.4443f,-49.9937f,  19.3312f,32.9356f,-49.5828f,  20.5008f,34.3600f,-50.3606f,  19.0081f,35.7959f,-49.8670f,  19.9980f,37.2280f,-50.8526f,  
	18.4821f,38.6102f,-50.2741f,  19.2642f,40.0323f,-51.4437f,  17.7185f,41.3634f,-50.7766f,  18.2705f,42.7553f,-52.1032f,  16.6904f,44.0349f,-51.3437f,  
	16.9961f,45.3740f,-52.7981f,  15.3766f,46.6007f,-51.9415f,  15.4271f,47.8622f,-53.4933f,  13.7663f,49.0311f,-52.5359f,  13.5600f,50.1889f,-54.1542f,  
	11.8559f,51.2943f,-53.0922f,  11.3991f,52.3214f,-54.7471f,  9.6540f,53.3541f,-53.5790f,  8.9610f,54.2234f,-55.2422f,  7.1785f,55.1734f,-53.9679f,  
	6.2713f,55.8590f,-55.6138f,  4.4585f,56.7145f,-54.2365f,  3.3663f,57.1927f,-55.8428f,  1.5335f,57.9415f,-54.3683f,  0.2918f,58.1919f,-55.9168f,  
	-1.5476f,58.8221f,-54.3553f,  -2.8977f,58.8287f,-55.8315f,  -4.7297f,59.3302f,-54.1952f,  -6.1432f,59.0825f,-55.5891f,  -7.9501f,59.4473f,-53.8944f,  
	-9.3792f,58.9411f,-55.1996f,  -11.1431f,59.1642f,-53.4654f,  -12.5391f,58.4022f,-54.6790f,  -14.2429f,58.4827f,-52.9244f,  -15.5578f,57.4750f,-54.0460f,  
	-17.1844f,57.4153f,-52.2925f,  -18.3722f,56.1793f,-53.3237f,  -19.9057f,55.9860f,-51.5921f,  -20.9243f,54.5457f,-52.5355f,  -22.3520f,54.2297f,-50.8437f,  
	-23.1645f,52.6157f,-51.7020f,  -24.4767f,52.1897f,-50.0658f,  -25.0526f,50.4378f,-50.8411f,  -26.2414f,49.9167f,-49.2753f,  -26.5569f,48.0669f,-49.9687f,  
	-27.6181f,47.4694f,-48.4819f,  -27.6580f,45.5648f,-49.0927f,  -28.5907f,44.9081f,-47.6911f,  -28.3478f,42.9933f,-48.2168f,  -29.1545f,42.2945f,-46.9024f,  
	-28.6297f,40.4140f,-47.3388f,  -29.3138f,39.6887f,-46.1120f,  -28.5163f,37.8860f,-46.4532f,  -29.0879f,37.1518f,-45.3069f,  -28.0351f,35.4676f,-45.5455f,  
	-28.5009f,34.7353f,-44.4755f,  -27.2169f,33.2066f,-44.6036f,  -27.5886f,32.4875f,-43.6008f,  -26.1041f,31.1465f,-43.6100f,  -26.3946f,30.4485f,-42.6637f,  
	-24.7456f,29.3220f,-42.5460f,  -24.9682f,28.6502f,-41.6458f,  32.6162f,-20.8924f,-62.7182f,  31.3635f,-18.2895f,-64.0422f,  30.3310f,-17.9761f,-62.3577f,  
	29.0305f,-15.3971f,-63.5572f,  28.2364f,-15.0079f,-61.7627f,  26.8953f,-12.4641f,-62.8379f,  26.3531f,-12.0026f,-60.9684f,  24.9781f,-9.5036f,-61.9207f,  
	24.6955f,-8.9722f,-60.0129f,  23.2932f,-6.5272f,-60.8443f,  23.2721f,-5.9273f,-58.9360f,  21.8490f,-3.5443f,-59.6494f,  22.0851f,-2.8763f,-57.7785f,  
	20.6474f,-0.5627f,-58.3771f,  21.1309f,0.1735f,-56.5809f,  19.6836f,2.4123f,-57.0687f,  20.3988f,3.2177f,-55.3829f,  18.9470f,5.3767f,-55.7642f,  
	19.8726f,6.2524f,-54.2219f,  18.4208f,8.3278f,-54.5010f,  19.5308f,9.2748f,-53.1323f,  18.0821f,11.2650f,-53.3140f,  19.3458f,12.2840f,-52.1449f,  
	17.9022f,14.1888f,-52.2338f,  19.2852f,15.2803f,-51.2860f,  17.8481f,17.1000f,-51.2864f,  19.3129f,18.2629f,-50.5765f,  17.8835f,19.9986f,-50.4925f,  
	19.3905f,21.2309f,-50.0318f,  17.9656f,22.8878f,-49.8665f,  19.4740f,24.1857f,-49.6607f,  18.0532f,25.7659f,-49.4168f,  19.5215f,27.1237f,-49.4659f,  
	18.1003f,28.6329f,-49.1448f,  19.4881f,30.0424f,-49.4434f,  18.0626f,31.4849f,-49.0457f,  19.3312f,32.9356f,-49.5828f,  17.8946f,34.3177f,-49.1073f,  
	19.0081f,35.7959f,-49.8670f,  17.5557f,37.1206f,-49.3122f,  18.4821f,38.6102f,-50.2741f,  17.0065f,39.8817f,-49.6366f,  17.7185f,41.3634f,-50.7766f,  
	16.2139f,42.5830f,-50.0524f,  16.6904f,44.0349f,-51.3437f,  15.1503f,45.2033f,-50.5280f,  15.3766f,46.6007f,-51.9415f,  13.7976f,47.7145f,-51.0300f,  
	13.7663f,49.0311f,-52.5359f,  12.1443f,50.0865f,-51.5237f,  11.8559f,51.2943f,-53.0922f,  10.1912f,52.2835f,-51.9767f,  9.6540f,53.3541f,-53.5790f,  
	7.9480f,54.2682f,-52.3584f,  7.1785f,55.1734f,-53.9679f,  5.4361f,56.0014f,-52.6434f,  4.4585f,56.7145f,-54.2365f,  2.6882f,57.4444f,-52.8122f,  
	1.5335f,57.9415f,-54.3683f,  -0.2534f,58.5612f,-52.8522f,  -1.5476f,58.8221f,-54.3553f,  -3.3385f,59.3209f,-52.7577f,  -4.7297f,59.3302f,-54.1952f,  
	-6.5080f,59.6987f,-52.5308f,  -7.9501f,59.4473f,-53.8944f,  -9.6985f,59.6790f,-52.1808f,  -11.1431f,59.1642f,-53.4654f,  -12.8446f,59.2562f,-51.7210f,  
	-14.2429f,58.4827f,-52.9244f,  -15.8801f,58.4356f,-51.1700f,  -17.1844f,57.4153f,-52.2925f,  -18.7406f,57.2338f,-50.5492f,  -19.9057f,55.9860f,-51.5921f,  
	-21.3667f,55.6788f,-49.8785f,  -22.3520f,54.2297f,-50.8437f,  -23.7067f,53.8080f,-49.1767f,  -24.4767f,52.1897f,-50.0658f,  -25.7157f,51.6671f,-48.4616f,  
	-26.2414f,49.9167f,-49.2753f,  -27.3581f,49.3108f,-47.7444f,  -27.6181f,47.4694f,-48.4819f,  -28.6098f,46.7971f,-47.0325f,  -28.5907f,44.9081f,-47.6911f,  
	-29.4578f,44.1863f,-46.3271f,  -29.1545f,42.2945f,-46.9024f,  -29.8987f,41.5390f,-45.6260f,  -29.3138f,39.6887f,-46.1120f,  -29.9429f,38.9182f,-44.9179f,  
	-29.0879f,37.1518f,-45.3069f,  -29.6074f,36.3777f,-44.1924f,  -28.5009f,34.7353f,-44.4755f,  -28.9210f,33.9703f,-43.4328f,  -27.5886f,32.4875f,-43.6008f,  
	-27.9211f,31.7408f,-42.6201f,  -26.3946f,30.4485f,-42.6637f,  -26.6520f,29.7269f,-41.7351f,  -24.9682f,28.6502f,-41.6458f,  -25.1623f,27.9574f,-40.7597f,  
	31.3635f,-18.2895f,-64.0422f,  30.0301f,-15.6450f,-65.2722f,  29.0305f,-15.3971f,-63.5572f,  27.6556f,-12.7889f,-64.6596f,  26.8953f,-12.4641f,-62.8379f,  
	25.4863f,-9.9031f,-63.8139f,  24.9781f,-9.5036f,-61.9207f,  23.5424f,-7.0000f,-62.7721f,  23.2932f,-6.5272f,-60.8443f,  21.8380f,-4.0903f,-61.5739f,  
	21.8490f,-3.5443f,-59.6494f,  20.3811f,-1.1823f,-60.2605f,  20.6474f,-0.5627f,-58.3771f,  19.1730f,1.7179f,-58.8741f,  19.6836f,2.4123f,-57.0687f,  
	18.2087f,4.6058f,-57.4561f,  18.9470f,5.3767f,-55.7642f,  17.4767f,7.4787f,-56.0469f,  18.4208f,8.3278f,-54.5010f,  16.9591f,10.3360f,-54.6844f,  
	18.0821f,11.2650f,-53.3140f,  16.6318f,13.1788f,-53.4034f,  17.9022f,14.1888f,-52.2338f,  16.4653f,16.0084f,-52.2343f,  17.8481f,17.1000f,-51.2864f,  
	16.4264f,18.8263f,-51.2029f,  17.8835f,19.9986f,-50.4925f,  16.4745f,21.6370f,-50.3291f,  17.9656f,22.8878f,-49.8665f,  16.5692f,24.4407f,-49.6268f,  
	18.0532f,25.7659f,-49.4168f,  16.6651f,27.2398f,-49.1031f,  18.1003f,28.6329f,-49.1448f,  16.7167f,30.0329f,-48.7587f,  18.0626f,31.4849f,-49.0457f,  
	16.6767f,32.8185f,-48.5867f,  17.8946f,34.3177f,-49.1073f,  16.5011f,35.5891f,-48.5745f,  17.5557f,37.1206f,-49.3122f,  16.1466f,38.3356f,-48.7026f,  
	17.0065f,39.8817f,-49.6366f,  15.5749f,41.0434f,-48.9465f,  16.2139f,42.5830f,-50.0524f,  14.7520f,43.6939f,-49.2768f,  15.1503f,45.2033f,-50.5280f,  
	13.6533f,46.2617f,-49.6622f,  13.7976f,47.7145f,-51.0300f,  12.2595f,48.7186f,-50.0685f,  12.1443f,50.0865f,-51.5237f,  10.5634f,51.0300f,-50.4626f,  
	10.1912f,52.2835f,-51.9767f,  8.5666f,53.1589f,-50.8128f,  7.9480f,54.2682f,-52.3584f,  6.2825f,55.0656f,-51.0910f,  5.4361f,56.0014f,-52.6434f,  
	3.7361f,56.7091f,-51.2748f,  2.6882f,57.4444f,-52.8122f,  0.9627f,58.0506f,-51.3481f,  -0.2534f,58.5612f,-52.8522f,  -1.9933f,59.0547f,-51.3011f,  
	-3.3385f,59.3209f,-52.7577f,  -5.0776f,59.6911f,-51.1323f,  -6.5080f,59.6987f,-52.5308f,  -8.2299f,59.9379f,-50.8472f,  -9.6985f,59.6790f,-52.1808f,  
	-11.3867f,59.7824f,-50.4562f,  -12.8446f,59.2562f,-51.7210f,  -14.4816f,59.2226f,-49.9754f,  -15.8801f,58.4356f,-51.1700f,  -17.4488f,58.2676f,-49.4246f,  
	-18.7406f,57.2338f,-50.5492f,  -20.2256f,56.9380f,-48.8229f,  -21.3667f,55.6788f,-49.8785f,  -22.7559f,55.2649f,-48.1891f,  -23.7067f,53.8080f,-49.1767f,  
	-24.9890f,53.2880f,-47.5415f,  -25.7157f,51.6671f,-48.4616f,  -26.8824f,51.0575f,-46.8930f,  -27.3581f,49.3108f,-47.7444f,  -28.4045f,48.6277f,-46.2523f,  
	-28.6098f,46.7971f,-47.0325f,  -29.5345f,46.0572f,-45.6226f,  -29.4578f,44.1863f,-46.3271f,  -30.2610f,43.4054f,-45.0034f,  -29.8987f,41.5390f,-45.6260f,  
	-30.5853f,40.7363f,-44.3854f,  -29.9429f,38.9182f,-44.9179f,  -30.5175f,38.1058f,-43.7591f,  -29.6074f,36.3777f,-44.1924f,  -30.0785f,35.5695f,-43.1090f,  
	-28.9210f,33.9703f,-43.4328f,  -29.2990f,33.1767f,-42.4160f,  -27.9211f,31.7408f,-42.6201f,  -28.2175f,30.9701f,-41.6609f,  -26.6520f,29.7269f,-41.7351f,  
	-26.8779f,28.9843f,-40.8243f,  -25.1623f,27.9574f,-40.7597f,  -25.3318f,27.2458f,-39.8864f,  30.0301f,-15.6450f,-65.2722f,  28.6208f,-12.9716f,-66.4023f,  
	27.6556f,-12.7889f,-64.6596f,  26.2113f,-10.1638f,-65.6601f,  25.4863f,-9.9031f,-63.8139f,  24.0152f,-7.3378f,-64.6864f,  23.5424f,-7.0000f,-62.7721f,  
	22.0524f,-4.5050f,-63.5192f,  21.8380f,-4.0903f,-61.5739f,  20.3368f,-1.6746f,-62.1990f,  20.3811f,-1.1823f,-60.2605f,  18.8758f,1.1464f,-60.7681f,  
	19.1730f,1.7179f,-58.8741f,  17.6703f,3.9529f,-59.2688f,  18.2087f,4.6058f,-57.4561f,  16.7146f,6.7421f,-57.7435f,  17.4767f,7.4787f,-56.0469f,  
	15.9961f,9.5132f,-56.2328f,  16.9591f,10.3360f,-54.6844f,  15.4956f,12.2674f,-54.7747f,  16.6318f,13.1788f,-53.4034f,  15.1883f,15.0069f,-53.4040f,  
	16.4653f,16.0084f,-52.2343f,  15.0442f,17.7341f,-52.1510f,  16.4264f,18.8263f,-51.2029f,  15.0260f,20.4547f,-51.0406f,  16.4745f,21.6370f,-50.3291f,  
	15.0951f,23.1709f,-50.0924f,  16.5692f,24.4407f,-49.6268f,  15.2071f,25.8871f,-49.3190f,  16.6651f,27.2398f,-49.1031f,  15.3166f,28.6044f,-48.7266f,  
	16.7167f,30.0329f,-48.7587f,  15.3750f,31.3240f,-48.3140f,  16.6767f,32.8185f,-48.5867f,  15.3362f,34.0417f,-48.0736f,  16.5011f,35.5891f,-48.5745f,  
	15.1535f,36.7516f,-47.9907f,  16.1466f,38.3356f,-48.7026f,  14.7841f,39.4423f,-48.0447f,  15.5749f,41.0434f,-48.9465f,  14.1886f,42.0985f,-48.2095f,  
	14.7520f,43.6939f,-49.2768f,  13.3358f,44.6976f,-48.4561f,  13.6533f,46.2617f,-49.6622f,  12.1994f,47.2140f,-48.7515f,  12.2595f,48.7186f,-50.0685f,  
	10.7643f,49.6150f,-49.0628f,  10.5634f,51.0300f,-50.4626f,  9.0238f,51.8646f,-49.3575f,  8.5666f,53.1589f,-50.8128f,  6.9831f,53.9231f,-49.6058f,  
	6.2825f,55.0656f,-51.0910f,  4.6593f,55.7483f,-49.7826f,  3.7361f,56.7091f,-51.2748f,  2.0799f,57.2990f,-49.8686f,  0.9627f,58.0506f,-51.3481f,  
	-0.7172f,58.5361f,-49.8503f,  -1.9933f,59.0547f,-51.3011f,  -3.6833f,59.4246f,-49.7227f,  -5.0776f,59.6911f,-51.1323f,  -6.7626f,59.9364f,-49.4874f,  
	-8.2299f,59.9379f,-50.8472f,  -9.8941f,60.0523f,-49.1518f,  -11.3867f,59.7824f,-50.4562f,  -13.0128f,59.7629f,-48.7293f,  -14.4816f,59.2226f,-49.9754f,  
	-16.0525f,59.0697f,-48.2376f,  -17.4488f,58.2676f,-49.4246f,  -18.9482f,57.9860f,-47.6950f,  -20.2256f,56.9380f,-48.8229f,  -21.6403f,56.5355f,-47.1199f,  
	-22.7559f,55.2649f,-48.1891f,  -24.0737f,54.7519f,-46.5309f,  -24.9890f,53.2880f,-47.5415f,  -26.1992f,52.6796f,-45.9422f,  -26.8824f,51.0575f,-46.8930f,  
	-27.9785f,50.3688f,-45.3639f,  -28.4045f,48.6277f,-46.2523f,  -29.3832f,47.8749f,-44.8011f,  -29.5345f,46.0572f,-45.6226f,  -30.3943f,45.2558f,-44.2550f,  
	-30.2610f,43.4054f,-45.0034f,  -31.0042f,42.5753f,-43.7183f,  -30.5853f,40.7363f,-44.3854f,  -31.2155f,39.8899f,-43.1830f,  -30.5175f,38.1058f,-43.7591f,  
	-31.0411f,37.2579f,-42.6347f,  -30.0785f,35.5695f,-43.1090f,  -30.5047f,34.7317f,-42.0550f,  -29.2990f,33.1767f,-42.4160f,  -29.6381f,32.3587f,-41.4243f,  
	-28.2175f,30.9701f,-41.6609f,  -28.4798f,30.1784f,-40.7230f,  -26.8779f,28.9843f,-40.8243f,  -27.0767f,28.2233f,-39.9296f,  -25.3318f,27.2458f,-39.8864f,  
	-25.4785f,26.5172f,-39.0259f,  28.6208f,-12.9716f,-66.4023f,  27.1408f,-10.2815f,-67.4277f,  26.2113f,-10.1638f,-65.6601f,  24.7036f,-7.5347f,-66.5544f,  
	24.0152f,-7.3378f,-64.6864f,  22.4885f,-4.7812f,-65.4520f,  22.0524f,-4.5050f,-63.5192f,  20.5152f,-2.0311f,-64.1594f,  20.3368f,-1.6746f,-62.1990f,  
	18.7970f,0.7076f,-62.7183f,  18.8758f,1.1464f,-60.7681f,  17.3410f,3.4292f,-61.1713f,  17.6703f,3.9529f,-59.2688f,  16.1474f,6.1305f,-59.5619f,  
	16.7146f,6.7421f,-57.7435f,  15.2093f,8.8104f,-57.9326f,  15.9961f,9.5132f,-56.2328f,  14.5132f,11.4702f,-56.3244f,  15.4956f,12.2674f,-54.7747f,  
	14.0388f,14.1126f,-54.7755f,  15.1883f,15.0069f,-53.4040f,  13.7606f,16.7407f,-53.3204f,  15.0442f,17.7341f,-52.1510f,  13.6450f,19.3612f,-51.9889f,  
	15.0260f,20.4547f,-51.0406f,  13.6560f,21.9782f,-50.8056f,  15.0951f,23.1709f,-50.0924f,  13.7513f,24.5979f,-49.7887f,  15.2071f,25.8871f,-49.3190f,  
	13.8862f,27.2238f,-48.9500f,  15.3166f,28.6044f,-48.7266f,  14.0119f,29.8599f,-48.2938f,  15.3750f,31.3240f,-48.3140f,  14.0810f,32.5051f,-47.8181f,  
	15.3362f,34.0417f,-48.0736f,  14.0443f,35.1567f,-47.5130f,  15.1535f,36.7516f,-47.9907f,  13.8553f,37.8071f,-47.3625f,  14.7841f,39.4423f,-48.0447f,  
	13.4697f,40.4442f,-47.3443f,  14.1886f,42.0985f,-48.2095f,  12.8507f,43.0489f,-47.4321f,  13.3358f,44.6976f,-48.4561f,  11.9652f,45.5984f,-47.5951f,  
	12.1994f,47.2140f,-48.7515f,  10.7903f,48.0627f,-47.8011f,  10.7643f,49.6150f,-49.0628f,  9.3117f,50.4074f,-48.0174f,  9.0238f,51.8646f,-49.3575f,  
	7.5260f,52.5934f,-48.2131f,  6.9831f,53.9231f,-49.6058f,  5.4420f,54.5783f,-48.3611f,  4.6593f,55.7483f,-49.7826f,  3.0792f,56.3192f,-48.4390f,  
	2.0799f,57.2990f,-49.8686f,  0.4681f,57.7741f,-48.4303f,  -0.7172f,58.5361f,-49.8503f,  -2.3490f,58.9036f,-48.3261f,  -3.6833f,59.4246f,-49.7227f,  
	-5.3216f,59.6746f,-48.1247f,  -6.7626f,59.9364f,-49.4874f,  -8.3924f,60.0613f,-47.8300f,  -9.8941f,60.0523f,-49.1518f,  -11.4989f,60.0472f,-47.4528f,  
	-13.0128f,59.7629f,-48.7293f,  -14.5755f,59.6265f,-47.0085f,  -16.0525f,59.0697f,-48.2376f,  -17.5564f,58.8046f,-46.5141f,  -18.9482f,57.9860f,-47.6950f,  
	-20.3792f,57.5981f,-45.9874f,  -21.6403f,56.5355f,-47.1199f,  -22.9854f,56.0336f,-45.4470f,  -24.0737f,54.7519f,-46.5309f,  -25.3201f,54.1493f,-44.9081f,  
	-26.1992f,52.6796f,-45.9422f,  -27.3390f,51.9905f,-44.3825f,  -27.9785f,50.3688f,-45.3639f,  -29.0066f,49.6086f,-43.8765f,  -29.3832f,47.8749f,-44.8011f,  
	-30.2966f,47.0587f,-43.3934f,  -30.3943f,45.2558f,-44.2550f,  -31.1925f,44.4035f,-42.9282f,  -31.0042f,42.5753f,-43.7183f,  -31.6896f,41.6995f,-42.4740f,  
	-31.2155f,39.8899f,-43.1830f,  -31.7926f,39.0064f,-42.0182f,  -31.0411f,37.2579f,-42.6347f,  -31.5176f,36.3792f,-41.5430f,  -30.5047f,34.7317f,-42.0550f,  
	-30.8894f,33.8688f,-41.0295f,  -29.6381f,32.3587f,-41.4243f,  -29.9404f,31.5193f,-40.4575f,  -28.4798f,30.1784f,-40.7230f,  -28.7122f,29.3685f,-39.8044f,  
	-27.0767f,28.2233f,-39.9296f,  -27.2501f,27.4460f,-39.0508f,  -25.4785f,26.5172f,-39.0259f,  -25.6046f,25.7735f,-38.1778f,  27.1408f,-10.2815f,-67.4277f,  
	25.5958f,-7.5872f,-68.3442f,  24.7036f,-7.5347f,-66.5544f,  23.1389f,-4.9143f,-67.3391f,  22.4885f,-4.7812f,-65.4520f,  20.9131f,-2.2458f,-66.1082f,  
	20.5152f,-2.0311f,-64.1594f,  18.9380f,0.4089f,-64.6914f,  18.7970f,0.7076f,-62.7183f,  17.2266f,3.0436f,-63.1310f,  17.3410f,3.4292f,-61.1713f,  
	15.7850f,5.6544f,-61.4708f,  16.1474f,6.1305f,-59.5619f,  14.6125f,8.2399f,-59.7548f,  15.2093f,8.8104f,-57.9326f,  13.7012f,10.8010f,-58.0259f,  
	14.5132f,11.4702f,-56.3244f,  13.0365f,13.3407f,-56.3253f,  14.0388f,14.1126f,-54.7755f,  12.5975f,15.8630f,-54.6912f,  13.7606f,16.7407f,-53.3204f,  
	12.3551f,18.3753f,-53.1577f,  13.6450f,19.3612f,-51.9889f,  12.2767f,20.8830f,-51.7542f,  13.6560f,21.9782f,-50.8056f,  12.3225f,23.3942f,-50.5042f,  
	13.7513f,24.5979f,-49.7887f,  12.4499f,25.9147f,-49.4250f,  13.8862f,27.2238f,-48.9500f,  12.6107f,28.4511f,-48.5265f,  14.0119f,29.8599f,-48.2938f,  
	12.7570f,31.0054f,-47.8123f,  14.0810f,32.5051f,-47.8181f,  12.8379f,33.5784f,-47.2777f,  14.0443f,35.1567f,-47.5130f,  12.8042f,36.1657f,-46.9116f,  
	13.8553f,37.8071f,-47.3625f,  12.6076f,38.7594f,-46.6959f,  13.4697f,40.4442f,-47.3443f,  12.2061f,41.3438f,-46.6079f,  12.8507f,43.0489f,-47.4321f,  
	11.5607f,43.8996f,-46.6192f,  11.9652f,45.5984f,-47.5951f,  10.6412f,46.3997f,-46.6991f,  10.7903f,48.0627f,-47.8011f,  9.4254f,48.8122f,-46.8155f,  
	9.3117f,50.4074f,-48.0174f,  7.9018f,51.0994f,-46.9369f,  7.5260f,52.5934f,-48.2131f,  6.0708f,53.2192f,-47.0347f,  5.4420f,54.5783f,-48.3611f,  
	3.9433f,55.1280f,-47.0839f,  3.0792f,56.3192f,-48.4390f,  1.5422f,56.7816f,-47.0652f,  0.4681f,57.7741f,-48.4303f,  -1.0975f,58.1372f,-46.9665f,  
	-2.3490f,58.9036f,-48.3261f,  -3.9316f,59.1569f,-46.7823f,  -5.3216f,59.6746f,-48.1247f,  -6.9073f,59.8091f,-46.5136f,  -8.3924f,60.0613f,-47.8300f,  
	-9.9657f,60.0707f,-46.1679f,  -11.4989f,60.0472f,-47.4528f,  -13.0432f,59.9284f,-45.7583f,  -14.5755f,59.6265f,-47.0085f,  -16.0739f,59.3800f,-45.3003f,  
	-17.5564f,58.8046f,-46.5141f,  -18.9942f,58.4345f,-44.8109f,  -20.3792f,57.5981f,-45.9874f,  -21.7427f,57.1112f,-44.3083f,  -22.9854f,56.0336f,-45.4470f,  
	-24.2603f,55.4416f,-43.8087f,  -25.3201f,54.1493f,-44.9081f,  -26.4970f,53.4651f,-43.3248f,  -27.3390f,51.9905f,-44.3825f,  -28.4111f,51.2284f,-42.8648f,  
	-29.0066f,49.6086f,-43.8765f,  -29.9693f,48.7831f,-42.4336f,  -30.2966f,47.0587f,-43.3934f,  -31.1468f,46.1897f,-42.0285f,  -31.1925f,44.4035f,-42.9282f,  
	-31.9315f,43.5037f,-41.6443f,  -31.6896f,41.6995f,-42.4740f,  -32.3200f,40.7852f,-41.2700f,  -31.7926f,39.0064f,-42.0182f,  -32.3204f,38.0909f,-40.8891f,  
	-31.5176f,36.3792f,-41.5430f,  -31.9501f,35.4746f,-40.4831f,  -30.8894f,33.8688f,-41.0295f,  -31.2349f,32.9841f,-40.0322f,  -29.9404f,31.5193f,-40.4575f,  
	-30.2103f,30.6620f,-39.5131f,  -28.7122f,29.3685f,-39.8044f,  -28.9167f,28.5430f,-38.9049f,  -27.2501f,27.4460f,-39.0508f,  -27.4006f,26.6545f,-38.1873f,  
	-25.6046f,25.7735f,-38.1778f,  -25.7122f,25.0161f,-37.3418f,  25.5958f,-7.5872f,-68.3442f,  23.9924f,-4.9019f,-69.1484f,  23.1389f,-4.9143f,-67.3391f,  
	21.5238f,-2.3150f,-68.0120f,  20.9131f,-2.2458f,-66.1082f,  19.2963f,0.2556f,-66.6536f,  18.9380f,0.4089f,-64.6914f,  17.3288f,2.8027f,-65.1143f,  
	17.2266f,3.0436f,-63.1310f,  15.6334f,5.3220f,-63.4377f,  15.7850f,5.6544f,-61.4708f,  14.2156f,7.8112f,-61.6681f,  14.6125f,8.2399f,-59.7548f,  
	13.0737f,10.2711f,-59.8500f,  13.7012f,10.8010f,-58.0259f,  12.1986f,12.7046f,-58.0269f,  13.0365f,13.3407f,-56.3253f,  11.5748f,15.1161f,-56.2399f,  
	12.5975f,15.8630f,-54.6912f,  11.1783f,17.5136f,-54.5270f,  12.3551f,18.3753f,-53.1577f,  10.9805f,19.9040f,-52.9220f,  12.2767f,20.8830f,-51.7542f,  
	10.9453f,22.2966f,-51.4533f,  12.3225f,23.3942f,-50.5042f,  11.0325f,24.6995f,-50.1435f,  12.4499f,25.9147f,-49.4250f,  11.1954f,27.1218f,-49.0082f,  
	12.6107f,28.4511f,-48.5265f,  11.3867f,29.5684f,-48.0563f,  12.7570f,31.0054f,-47.8123f,  11.5549f,32.0434f,-47.2888f,  12.8379f,33.5784f,-47.2777f,  
	11.6488f,34.5464f,-46.6998f,  12.8042f,36.1657f,-46.9116f,  11.6170f,37.0729f,-46.2755f,  12.6076f,38.7594f,-46.6959f,  11.4133f,39.6115f,-45.9975f,  
	12.2061f,41.3438f,-46.6079f,  10.9929f,42.1465f,-45.8404f,  11.5607f,43.8996f,-46.6192f,  10.3194f,44.6545f,-45.7757f,  10.6412f,46.3997f,-46.6991f,  
	9.3630f,47.1063f,-45.7724f,  9.4254f,48.8122f,-46.8155f,  8.1041f,49.4666f,-45.7989f,  7.9018f,51.0994f,-46.9369f,  6.5349f,51.6943f,-45.8259f,  
	6.0708f,53.2192f,-47.0347f,  4.6576f,53.7458f,-45.8263f,  3.9433f,55.1280f,-47.0839f,  2.4868f,55.5757f,-45.7785f,  1.5422f,56.7816f,-47.0652f,  
	0.0497f,57.1385f,-45.6671f,  -1.0975f,58.1372f,-46.9665f,  -2.6163f,58.3923f,-45.4833f,  -3.9316f,59.1569f,-46.7823f,  -5.4642f,59.3001f,-45.2250f,  
	-6.9073f,59.8091f,-46.5136f,  -8.4396f,59.8329f,-44.8966f,  -9.9657f,60.0707f,-46.1679f,  -11.4817f,59.9702f,-44.5084f,  -13.0432f,59.9284f,-45.7583f,  
	-14.5261f,59.7023f,-44.0745f,  -16.0739f,59.3800f,-45.3003f,  -17.5088f,59.0303f,-43.6105f,  -18.9942f,58.4345f,-44.8109f,  -20.3669f,57.9663f,-43.1343f,  
	-21.7427f,57.1112f,-44.3083f,  -23.0377f,56.5343f,-42.6625f,  -24.2603f,55.4416f,-43.8087f,  -25.4668f,54.7673f,-42.2089f,  -26.4970f,53.4651f,-43.3248f,  
	-27.6068f,52.7067f,-41.7832f,  -28.4111f,51.2284f,-42.8648f,  -29.4182f,50.3996f,-41.3918f,  -29.9693f,48.7831f,-42.4336f,  -30.8681f,47.9033f,-41.0346f,  
	-31.1468f,46.1897f,-42.0285f,  -31.9373f,45.2716f,-40.7080f,  -31.9315f,43.5037f,-41.6443f,  -32.6142f,42.5639f,-40.4029f,  -32.3200f,40.7852f,-41.2700f,  
	-32.8992f,39.8376f,-40.1042f,  -32.3204f,38.0909f,-40.8891f,  -32.8019f,37.1486f,-39.7946f,  -31.9501f,35.4746f,-40.4831f,  -32.3409f,34.5478f,-39.4545f,  
	-31.2349f,32.9841f,-40.0322f,  -31.5453f,32.0814f,-39.0604f,  -30.2103f,30.6620f,-39.5131f,  -30.4497f,29.7893f,-38.5908f,  -28.9167f,28.5430f,-38.9049f,  
	-29.0960f,27.7039f,-38.0235f,  -27.4006f,26.6545f,-38.1873f,  -27.5305f,25.8505f,-37.3384f,  -25.7122f,25.0161f,-37.3418f,  -25.8055f,24.2460f,-36.5164f,  
	23.9924f,-4.9019f,-69.1484f,  22.3369f,-2.2375f,-69.8381f,  21.5238f,-2.3150f,-68.0120f,  19.8659f,0.2502f,-68.5713f,  19.2963f,0.2556f,-66.6536f,  
	17.6460f,2.7107f,-67.0874f,  17.3288f,2.8027f,-65.1143f,  15.6954f,5.1389f,-65.4289f,  15.6334f,5.3220f,-63.4377f,  14.0257f,7.5319f,-63.6399f,  
	14.2156f,7.8112f,-61.6681f,  12.6414f,9.8896f,-61.7657f,  13.0737f,10.2711f,-59.8500f,  11.5395f,12.2151f,-59.8512f,  12.1986f,12.7046f,-58.0269f,  
	10.7101f,14.5128f,-57.9400f,  11.5748f,15.1161f,-56.2399f,  10.1348f,16.7911f,-56.0734f,  11.1783f,17.5136f,-54.5270f,  9.7899f,19.0579f,-54.2889f,  
	10.9805f,19.9040f,-52.9220f,  9.6432f,21.3240f,-52.6197f,  10.9453f,22.2966f,-51.4533f,  9.6580f,23.5991f,-51.0933f,  11.0325f,24.6995f,-50.1435f,  
	9.7902f,25.8947f,-49.7305f,  11.1954f,27.1218f,-49.0082f,  9.9937f,28.2187f,-48.5460f,  11.3867f,29.5684f,-48.0563f,  10.2174f,30.5782f,-47.5462f,  
	11.5549f,32.0434f,-47.2888f,  10.4089f,32.9767f,-46.7305f,  11.6488f,34.5464f,-46.6998f,  10.5150f,35.4136f,-46.0903f,  11.6170f,37.0729f,-46.2755f,  
	10.4856f,37.8816f,-45.6114f,  11.4133f,39.6115f,-45.9975f,  10.2716f,40.3692f,-45.2721f,  10.9929f,42.1465f,-45.8404f,  9.8303f,42.8569f,-45.0467f,  
	10.3194f,44.6545f,-45.7757f,  9.1255f,45.3189f,-44.9058f,  9.3630f,47.1063f,-45.7724f,  8.1295f,47.7229f,-44.8186f,  8.1041f,49.4666f,-45.7989f,  
	6.8265f,50.0295f,-44.7555f,  6.5349f,51.6943f,-45.8259f,  5.2097f,52.1962f,-44.6879f,  4.6576f,53.7458f,-45.8263f,  3.2856f,54.1770f,-44.5921f,  
	2.4868f,55.5757f,-45.7785f,  1.0731f,55.9245f,-44.4501f,  0.0497f,57.1385f,-45.6671f,  -1.3983f,57.3937f,-44.2499f,  -2.6163f,58.3923f,-45.4833f,  
	-4.0878f,58.5431f,-43.9863f,  -5.4642f,59.3001f,-45.2250f,  -6.9465f,59.3380f,-43.6605f,  -8.4396f,59.8329f,-44.8966f,  -9.9178f,59.7513f,-43.2802f,  
	-11.4817f,59.9702f,-44.5084f,  -12.9395f,59.7659f,-42.8576f,  -14.5261f,59.7023f,-44.0745f,  -15.9486f,59.3755f,-42.4069f,  -17.5088f,59.0303f,-43.6105f,  
	-18.8816f,58.5842f,-41.9452f,  -20.3669f,57.9663f,-43.1343f,  -21.6735f,57.4086f,-41.4894f,  -23.0377f,56.5343f,-42.6625f,  -24.2662f,55.8749f,-41.0540f,  
	-25.4668f,54.7673f,-42.2089f,  -26.6076f,54.0182f,-40.6502f,  -27.6068f,52.7067f,-41.7832f,  -28.6526f,51.8805f,-40.2860f,  -29.4182f,50.3996f,-41.3918f,  
	-30.3610f,49.5151f,-39.9635f,  -30.8681f,47.9033f,-41.0346f,  -31.7067f,46.9729f,-39.6811f,  -31.9373f,45.2716f,-40.7080f,  -32.6704f,44.3119f,-39.4317f,  
	-32.6142f,42.5639f,-40.4029f,  -33.2443f,41.5894f,-39.2019f,  -32.8992f,39.8376f,-40.1042f,  -33.4303f,38.8623f,-38.9757f,  -32.8019f,37.1486f,-39.7946f,  
	-33.2397f,36.1835f,-38.7343f,  -32.3409f,34.5478f,-39.4545f,  -32.6944f,33.6027f,-38.4542f,  -31.5453f,32.0814f,-39.0604f,  -31.8229f,31.1636f,-38.1137f,  
	-30.4497f,29.7893f,-38.5908f,  -30.6615f,28.9037f,-37.6894f,  -29.0960f,27.7039f,-38.0235f,  -29.2522f,26.8532f,-37.1594f,  -27.5305f,25.8505f,-37.3384f,  
	-27.6437f,25.0350f,-36.5022f,  -25.8055f,24.2460f,-36.5164f,  -25.8847f,23.4645f,-35.7019f,  22.3369f,-2.2375f,-69.8381f,  20.6369f,0.3929f,-70.4117f,  
	19.8659f,0.2502f,-68.5713f,  18.1728f,2.7691f,-69.0165f,  17.6460f,2.7107f,-67.0874f,  15.9698f,5.1083f,-67.4103f,  15.6954f,5.1389f,-65.4289f,  
	14.0458f,7.4065f,-65.6364f,  14.0257f,7.5319f,-63.6399f,  12.4116f,9.6630f,-63.7400f,  12.6414f,9.8896f,-61.7657f,  11.0703f,11.8805f,-61.7669f,  
	11.5395f,12.2151f,-59.8512f,  10.0181f,14.0634f,-59.7624f,  10.7101f,14.5128f,-57.9400f,  9.2425f,16.2201f,-57.7704f,  10.1348f,16.7911f,-56.0734f,  
	8.7250f,18.3593f,-55.8317f,  9.7899f,19.0579f,-54.2889f,  8.4385f,20.4929f,-53.9835f,  9.6432f,21.3240f,-52.6197f,  8.3500f,22.6322f,-52.2580f,  
	9.6580f,23.5991f,-51.0933f,  8.4192f,24.7908f,-50.6812f,  9.7902f,25.8947f,-49.7305f,  8.6017f,26.9793f,-49.2730f,  9.9937f,28.2187f,-48.5460f,  
	8.8481f,29.2078f,-48.0454f,  10.2174f,30.5782f,-47.5462f,  9.1058f,31.4835f,-47.0034f,  10.4089f,32.9767f,-46.7305f,  9.3203f,33.8099f,-46.1435f,  
	10.5150f,35.4136f,-46.0903f,  9.4390f,36.1838f,-45.4562f,  10.4856f,37.8816f,-45.6114f,  9.4090f,38.5981f,-44.9241f,  10.2716f,40.3692f,-45.2721f,  
	9.1825f,41.0376f,-44.5246f,  9.8303f,42.8569f,-45.0467f,  8.7169f,43.4806f,-44.2307f,  9.1255f,45.3189f,-44.9058f,  7.9776f,45.8981f,-44.0128f,  
	8.1295f,47.7229f,-44.8186f,  6.9405f,48.2535f,-43.8418f,  6.8265f,50.0295f,-44.7555f,  5.5904f,50.5058f,-43.6880f,  5.2097f,52.1962f,-44.6879f,  
	3.9251f,52.6094f,-43.5264f,  3.2856f,54.1770f,-44.5921f,  1.9550f,54.5160f,-43.3364f,  1.0731f,55.9245f,-44.4501f,  -0.2984f,56.1782f,-43.1032f,  
	-1.3983f,57.3937f,-44.2499f,  -2.8016f,57.5510f,-42.8186f,  -4.0878f,58.5431f,-43.9863f,  -5.5121f,58.5943f,-42.4808f,  -6.9465f,59.3380f,-43.6605f,  
	-8.3783f,59.2753f,-42.0948f,  -9.9178f,59.7513f,-43.2802f,  -11.3413f,59.5698f,-41.6705f,  -12.9395f,59.7659f,-42.8576f,  -14.3402f,59.4640f,-41.2206f,  
	-15.9486f,59.3755f,-42.4069f,  -17.3122f,58.9544f,-40.7612f,  -18.8816f,58.5842f,-41.9452f,  -20.1908f,58.0497f,-40.3092f,  -21.6735f,57.4086f,-41.4894f,  
	-22.9156f,56.7689f,-39.8798f,  -24.2662f,55.8749f,-41.0540f,  -25.4305f,55.1404f,-39.4852f,  -26.6076f,54.0182f,-40.6502f,  -27.6857f,53.2005f,-39.1349f,  
	-28.6526f,51.8805f,-40.2860f,  -29.6342f,50.9974f,-38.8336f,  -30.3610f,49.5151f,-39.9635f,  -31.2438f,48.5785f,-38.5812f,  -31.7067f,46.9729f,-39.6811f,  
	-32.4874f,45.9995f,-38.3731f,  -32.6704f,44.3119f,-39.4317f,  -33.3497f,43.3164f,-38.1976f,  -33.2443f,41.5894f,-39.2019f,  -33.8245f,40.5862f,-38.0404f,  
	-33.4303f,38.8623f,-38.9757f,  -33.9159f,37.8633f,-37.8838f,  -33.2397f,36.1835f,-38.7343f,  -33.6380f,35.1996f,-37.7049f,  -32.6944f,33.6027f,-38.4542f,  
	-33.0128f,32.6425f,-37.4818f,  -31.8229f,31.1636f,-38.1137f,  -32.0706f,30.2333f,-37.1905f,  -30.6615f,28.9037f,-37.6894f,  -30.8482f,28.0071f,-36.8078f,  
	-29.2522f,26.8532f,-37.1594f,  -29.3898f,25.9923f,-36.3101f,  -27.6437f,25.0350f,-36.5022f,  -27.7411f,24.2097f,-35.6791f,  -25.8847f,23.4645f,-35.7019f,  
	-25.9521f,22.6725f,-34.8978f,  20.6369f,0.3929f,-70.4117f,  18.9001f,2.9770f,-70.8684f,  18.1728f,2.7691f,-69.0165f,  16.4519f,5.2307f,-69.3480f,  
	15.9698f,5.1083f,-67.4103f,  14.2756f,7.4374f,-67.6235f,  14.0458f,7.4065f,-65.6364f,  12.3883f,9.5953f,-65.7393f,  12.4116f,9.6630f,-63.7400f,  
	10.7991f,11.7066f,-63.7413f,  11.0703f,11.8805f,-61.7669f,  9.5106f,13.7756f,-61.6760f,  10.0181f,14.0634f,-59.7624f,  8.5167f,15.8103f,-59.5890f,  
	9.2425f,16.2201f,-57.7704f,  7.8042f,17.8201f,-57.5239f,  8.7250f,18.3593f,-55.8317f,  7.3517f,19.8175f,-55.5214f,  8.4385f,20.4929f,-53.9835f,  
	7.1310f,21.8155f,-53.6178f,  8.3500f,22.6322f,-52.2580f,  7.1057f,23.8290f,-51.8439f,  8.4192f,24.7908f,-50.6812f,  7.2347f,25.8714f,-50.2248f,  
	8.6017f,26.9793f,-49.2730f,  7.4705f,27.9557f,-48.7780f,  8.8481f,29.2078f,-48.0454f,  7.7617f,30.0924f,-47.5137f,  9.1058f,31.4835f,-47.0034f,  
	8.0534f,32.2891f,-46.4341f,  9.3203f,33.8099f,-46.1435f,  8.2914f,34.5470f,-45.5345f,  9.4390f,36.1838f,-45.4562f,  8.4200f,36.8634f,-44.8022f,  
	9.4090f,38.5981f,-44.9241f,  8.3870f,39.2278f,-44.2182f,  9.1825f,41.0376f,-44.5246f,  8.1444f,41.6229f,-43.7585f,  8.7169f,43.4806f,-44.2307f,  
	7.6508f,44.0236f,-43.3953f,  7.9776f,45.8981f,-44.0128f,  6.8749f,46.3966f,-43.1004f,  6.9405f,48.2535f,-43.8418f,  5.7929f,48.7034f,-42.8439f,  
	5.5904f,50.5058f,-43.6880f,  4.3943f,50.8997f,-42.5996f,  3.9251f,52.6094f,-43.5264f,  2.6805f,52.9371f,-42.3450f,  1.9550f,54.5160f,-43.3364f,  
	0.6646f,54.7668f,-42.0627f,  -0.2984f,56.1782f,-43.1032f,  -1.6277f,56.3407f,-41.7419f,  -2.8016f,57.5510f,-42.8186f,  -4.1609f,57.6147f,-41.3778f,  
	-5.5121f,58.5943f,-42.4808f,  -6.8894f,58.5505f,-40.9723f,  -8.3783f,59.2753f,-42.0948f,  -9.7587f,59.1174f,-40.5334f,  -11.3413f,59.5698f,-41.6705f,  
	-12.7111f,59.2943f,-40.0719f,  -14.3402f,59.4640f,-41.2206f,  -15.6854f,59.0705f,-39.6026f,  -17.3122f,58.9544f,-40.7612f,  -18.6150f,58.4466f,-39.1423f,  
	-20.1908f,58.0497f,-40.3092f,  -21.4382f,57.4341f,-38.7064f,  -22.9156f,56.7689f,-39.8798f,  -24.0957f,56.0542f,-38.3081f,  -25.4305f,55.1404f,-39.4852f,  
	-26.5340f,54.3370f,-37.9584f,  -27.6857f,53.2005f,-39.1349f,  -28.7002f,52.3249f,-37.6642f,  -29.6342f,50.9974f,-38.8336f,  -30.5564f,50.0610f,-37.4272f,  
	-31.2438f,48.5785f,-38.5812f,  -32.0685f,47.5977f,-37.2451f,  -32.4874f,45.9995f,-38.3731f,  -33.2136f,44.9889f,-37.1084f,  -33.3497f,43.3164f,-38.1976f,  
	-33.9779f,42.2909f,-37.0047f,  -33.8245f,40.5862f,-38.0404f,  -34.3578f,39.5584f,-36.9177f,  -33.9159f,37.8633f,-37.8838f,  -34.3599f,36.8450f,-36.8252f,  
	-33.6380f,35.1996f,-37.7049f,  -33.9990f,34.2006f,-36.7059f,  -33.0128f,32.6425f,-37.4818f,  -33.2990f,31.6699f,-36.5355f,  -32.0706f,30.2333f,-37.1905f,  
	-32.2909f,29.2925f,-36.2897f,  -30.8482f,28.0071f,-36.8078f,  -31.0138f,27.1015f,-35.9434f,  -29.3898f,25.9923f,-36.3101f,  -29.5094f,25.1228f,-35.4760f,  
	-27.7411f,24.2097f,-35.6791f,  -27.8248f,23.3753f,-34.8681f,  -25.9521f,22.6725f,-34.8978f,  -26.0105f,21.8705f,-34.1030f,  18.9001f,2.9770f,-70.8684f,  
	17.1336f,5.5040f,-71.2088f,  16.4519f,5.2307f,-69.3480f,  14.7113f,7.6239f,-69.5670f,  14.2756f,7.4374f,-67.6235f,  12.5719f,9.6875f,-67.7292f,  
	12.3883f,9.5953f,-65.7393f,  10.7306f,11.6963f,-65.7407f,  10.7991f,11.7066f,-63.7413f,  9.1964f,13.6543f,-63.6480f,  9.5106f,13.7756f,-61.6760f,  
	7.9696f,15.5687f,-61.4980f,  8.5167f,15.8103f,-59.5890f,  7.0431f,17.4496f,-59.3365f,  7.8042f,17.8201f,-57.5239f,  6.4015f,19.3095f,-57.2069f,  
	7.3517f,19.8175f,-55.5214f,  6.0218f,21.1628f,-55.1493f,  7.1310f,21.8155f,-53.6178f,  5.8724f,23.0258f,-53.1988f,  7.1057f,23.8290f,-51.8439f,  
	5.9160f,24.9140f,-51.3852f,  7.2347f,25.8714f,-50.2248f,  6.1083f,26.8432f,-49.7313f,  7.4705f,27.9557f,-48.7780f,  6.3996f,28.8272f,-48.2527f,  
	7.7617f,30.0924f,-47.5137f,  6.7360f,30.8774f,-46.9571f,  8.0534f,32.2891f,-46.4341f,  7.0624f,32.9994f,-45.8451f,  8.2914f,34.5470f,-45.5345f,  
	7.3214f,35.1949f,-44.9086f,  8.4200f,36.8634f,-44.8022f,  7.4573f,37.4584f,-44.1329f,  8.3870f,39.2278f,-44.2182f,  7.4176f,39.7774f,-43.4973f,  
	8.1444f,41.6229f,-43.7585f,  7.1549f,42.1312f,-42.9767f,  7.6508f,44.0236f,-43.3953f,  6.6309f,44.4905f,-42.5441f,  6.8749f,46.3966f,-43.1004f,  
	5.8140f,46.8200f,-42.1699f,  5.7929f,48.7034f,-42.8439f,  4.6852f,49.0772f,-41.8275f,  4.3943f,50.8997f,-42.5996f,  3.2372f,51.2148f,-41.4929f,  
	2.6805f,52.9371f,-42.3450f,  1.4742f,53.1834f,-41.1464f,  0.6646f,54.7668f,-42.0627f,  -0.5861f,54.9329f,-40.7748f,  -1.6277f,56.3407f,-41.7419f,  
	-2.9160f,56.4159f,-40.3703f,  -4.1609f,57.6147f,-41.3778f,  -5.4766f,57.5892f,-39.9321f,  -6.8894f,58.5505f,-40.9723f,  -8.2190f,58.4163f,-39.4658f,  
	-9.7587f,59.1174f,-40.5334f,  -11.0893f,58.8694f,-38.9805f,  -12.7111f,59.2943f,-40.0719f,  -14.0292f,58.9306f,-38.4893f,  -15.6854f,59.0705f,-39.6026f,  
	-16.9733f,58.5925f,-38.0086f,  -18.6150f,58.4466f,-39.1423f,  -19.8592f,57.8589f,-37.5539f,  -21.4382f,57.4341f,-38.7064f,  -22.6260f,56.7439f,-37.1392f,  
	-24.0957f,56.0542f,-38.3081f,  -25.2173f,55.2705f,-36.7766f,  -26.5340f,54.3370f,-37.9584f,  -27.5749f,53.4748f,-36.4753f,  -28.7002f,52.3249f,-37.6642f,  
	-29.6563f,51.3950f,-36.2388f,  -30.5564f,50.0610f,-37.4272f,  -31.4207f,49.0792f,-36.0671f,  -32.0685f,47.5977f,-37.2451f,  -32.8383f,46.5784f,-35.9531f,  
	-33.2136f,44.9889f,-37.1084f,  -33.8877f,43.9472f,-35.8864f,  -33.9779f,42.2909f,-37.0047f,  -34.5579f,41.2398f,-35.8524f,  -34.3578f,39.5584f,-36.9177f,  
	-34.8477f,38.5106f,-35.8303f,  -34.3599f,36.8450f,-36.8252f,  -34.7647f,35.8112f,-35.7994f,  -33.9990f,34.2006f,-36.7059f,  -34.3258f,33.1891f,-35.7356f,  
	-33.2990f,31.6699f,-36.5355f,  -33.5557f,30.6873f,-35.6139f,  -32.2909f,29.2925f,-36.2897f,  -32.4879f,28.3436f,-35.4082f,  -31.0138f,27.1015f,-35.9434f,  
	-31.1594f,26.1883f,-35.0962f,  -29.5094f,25.1228f,-35.4760f,  -29.6136f,24.2457f,-34.6559f,  -27.8248f,23.3753f,-34.8681f,  -27.8979f,22.5326f,-34.0680f,  
	-26.0105f,21.8705f,-34.1030f,  -26.0602f,21.0594f,-33.3179f,  17.1336f,5.5040f,-71.2088f,  15.3453f,7.9628f,-71.4339f,  14.7113f,7.6239f,-69.5670f,  
	12.9592f,9.9381f,-69.6758f,  12.5719f,9.6875f,-67.7292f,  10.8660f,11.8499f,-67.7307f,  10.7306f,11.6963f,-65.7407f,  9.0808f,13.7016f,-65.6446f,  
	9.1964f,13.6543f,-63.6480f,  7.6108f,15.4995f,-63.4649f,  7.9696f,15.5687f,-61.4980f,  6.4549f,17.2539f,-61.2386f,  7.0431f,17.4496f,-59.3365f,  
	5.6041f,18.9776f,-59.0114f,  6.4015f,19.3095f,-57.2069f,  5.0412f,20.6854f,-56.8265f,  6.0218f,21.1628f,-55.1493f,  4.7402f,22.3948f,-54.7227f,  
	5.8724f,23.0258f,-53.1988f,  4.6681f,24.1235f,-52.7342f,  5.9160f,24.9140f,-51.3852f,  4.7846f,25.8894f,-50.8890f,  6.1083f,26.8432f,-49.7313f,  
	5.0429f,27.7096f,-49.2078f,  6.3996f,28.8272f,-48.2527f,  5.3905f,29.5989f,-47.7037f,  6.7360f,30.8774f,-46.9571f,  5.7730f,31.5673f,-46.3825f,  
	7.0624f,32.9994f,-45.8451f,  6.1321f,33.6211f,-45.2414f,  7.3214f,35.1949f,-44.9086f,  6.4095f,35.7598f,-44.2700f,  7.4573f,37.4584f,-44.1329f,  
	6.5490f,37.9759f,-43.4517f,  7.4176f,39.7774f,-43.4973f,  6.4983f,40.2535f,-42.7641f,  7.1549f,42.1312f,-42.9767f,  6.2127f,42.5679f,-42.1823f,  
	6.6309f,44.4905f,-42.5441f,  5.6531f,44.8877f,-41.6776f,  5.8140f,46.8200f,-42.1699f,  4.7927f,47.1731f,-41.2234f,  4.6852f,49.0772f,-41.8275f,  
	3.6154f,49.3787f,-40.7946f,  3.2372f,51.2148f,-41.4929f,  2.1166f,51.4553f,-40.3698f,  1.4742f,53.1834f,-41.1464f,  0.3054f,53.3518f,-39.9336f,  
	-0.5861f,54.9329f,-40.7748f,  -1.7987f,55.0184f,-39.4754f,  -2.9160f,56.4159f,-40.3703f,  -4.1642f,56.4079f,-38.9919f,  -5.4766f,57.5892f,-39.9321f,  
	-6.7483f,57.4786f,-38.4860f,  -8.2190f,58.4163f,-39.4658f,  -9.5024f,58.1966f,-37.9650f,  -11.0893f,58.8694f,-38.9805f,  -12.3719f,58.5370f,-37.4402f,  
	-14.0292f,58.9306f,-38.4893f,  -15.2935f,58.4849f,-36.9274f,  -16.9733f,58.5925f,-38.0086f,  -18.2056f,58.0363f,-36.4420f,  -19.8592f,57.8589f,-37.5539f,  
	-21.0466f,57.1977f,-35.9983f,  -22.6260f,56.7439f,-37.1392f,  -23.7579f,55.9851f,-35.6096f,  -25.2173f,55.2705f,-36.7766f,  -26.2777f,54.4275f,-35.2872f,  
	-27.5749f,53.4748f,-36.4753f,  -28.5589f,52.5577f,-35.0362f,  -29.6563f,51.3950f,-36.2388f,  -30.5553f,50.4186f,-34.8592f,  -31.4207f,49.0792f,-36.0671f,  
	-32.2302f,48.0577f,-34.7514f,  -32.8383f,46.5784f,-35.9531f,  -33.5556f,45.5268f,-34.7046f,  -33.8877f,43.9472f,-35.8864f,  -34.5128f,42.8788f,-34.7062f,  
	-34.5579f,41.2398f,-35.8524f,  -35.0931f,40.1680f,-34.7371f,  -34.8477f,38.5106f,-35.8303f,  -35.2968f,37.4468f,-34.7777f,  -34.7647f,35.8112f,-35.7994f,  
	-35.1334f,34.7648f,-34.8043f,  -34.3258f,33.1891f,-35.7356f,  -34.6211f,32.1677f,-34.7920f,  -33.5557f,30.6873f,-35.6139f,  -33.7869f,29.6973f,-34.7138f,  
	-32.4879f,28.3436f,-35.4082f,  -32.6628f,27.3880f,-34.5460f,  -31.1594f,26.1883f,-35.0962f,  -31.2876f,25.2688f,-34.2648f,  -29.6136f,24.2457f,-34.6559f,  
	-29.7052f,23.3619f,-33.8482f,  -27.8979f,22.5326f,-34.0680f,  -27.9608f,21.6825f,-33.2788f,  -26.0602f,21.0594f,-33.3179f,  -26.1035f,20.2395f,-32.5416f,  
	15.3453f,7.9628f,-71.4339f,  13.5436f,10.3428f,-71.5457f,  12.9592f,9.9381f,-69.6758f,  11.2029f,12.1647f,-69.6774f,  10.8660f,11.8499f,-67.7307f,  
	9.1658f,13.9166f,-67.6317f,  9.0808f,13.7016f,-65.6446f,  7.4463f,15.6039f,-65.4559f,  7.6108f,15.4995f,-63.4649f,  6.0496f,17.2366f,-63.1976f,  
	6.4549f,17.2539f,-61.2386f,  4.9733f,18.8273f,-60.9039f,  5.6041f,18.9776f,-59.0114f,  4.2063f,20.3913f,-58.6205f,  5.0412f,20.6854f,-56.8265f,  
	3.7285f,21.9470f,-56.3895f,  4.7402f,22.3948f,-54.7227f,  3.5124f,23.5134f,-54.2489f,  4.6681f,24.1235f,-52.7342f,  3.5222f,25.1107f,-52.2314f,  
	4.7846f,25.8894f,-50.8890f,  3.7147f,26.7586f,-50.3626f,  5.0429f,27.7096f,-49.2078f,  4.0402f,28.4754f,-48.6610f,  5.3905f,29.5989f,-47.7037f,  
	4.4454f,30.2753f,-47.1376f,  5.7730f,31.5673f,-46.3825f,  4.8724f,32.1689f,-45.7949f,  6.1321f,33.6211f,-45.2414f,  5.2613f,34.1610f,-44.6273f,  
	6.4095f,35.7598f,-44.2700f,  5.5534f,36.2490f,-43.6224f,  6.5490f,37.9759f,-43.4517f,  5.6923f,38.4226f,-42.7611f,  6.4983f,40.2535f,-42.7641f,  
	5.6274f,40.6617f,-42.0212f,  6.2127f,42.5679f,-42.1823f,  5.3131f,42.9395f,-41.3754f,  5.6531f,44.8877f,-41.6776f,  4.7151f,45.2200f,-40.7976f,  
	4.7927f,47.1731f,-41.2234f,  3.8087f,47.4600f,-40.2622f,  3.6154f,49.3787f,-40.7946f,  2.5807f,49.6119f,-39.7463f,  2.1166f,51.4553f,-40.3698f,  
	1.0314f,51.6244f,-39.2328f,  0.3054f,53.3518f,-39.9336f,  -0.8281f,53.4460f,-38.7087f,  -1.7987f,55.0184f,-39.4754f,  -2.9745f,55.0267f,-38.1677f,  
	-4.1642f,56.4079f,-38.9919f,  -5.3720f,56.3203f,-37.6108f,  -6.7483f,57.4786f,-38.4860f,  -7.9776f,57.2873f,-37.0428f,  -9.5024f,58.1966f,-37.9650f,  
	-10.7418f,57.8965f,-36.4734f,  -12.3719f,58.5370f,-37.4402f,  -13.6045f,58.1256f,-35.9174f,  -15.2935f,58.4849f,-36.9274f,  -16.5058f,57.9630f,-35.3896f,  
	-18.2056f,58.0363f,-36.4420f,  -19.3845f,57.4078f,-34.9051f,  -21.0466f,57.1977f,-35.9983f,  -22.1810f,56.4685f,-34.4775f,  -23.7579f,55.9851f,-35.6096f,  
	-24.8306f,55.1667f,-34.1203f,  -26.2777f,54.4275f,-35.2872f,  -27.2830f,53.5291f,-33.8402f,  -28.5589f,52.5577f,-35.0362f,  -29.4869f,51.5931f,-33.6421f,  
	-30.5553f,50.4186f,-34.8592f,  -31.3999f,49.4014f,-33.5237f,  -32.2302f,48.0577f,-34.7514f,  -32.9871f,47.0026f,-33.4795f,  -33.5556f,45.5268f,-34.7046f,  
	-34.2233f,44.4473f,-33.4987f,  -34.5128f,42.8788f,-34.7062f,  -35.0920f,41.7887f,-33.5644f,  -35.0931f,40.1680f,-34.7371f,  -35.5861f,39.0794f,-33.6584f,  
	-35.2968f,37.4468f,-34.7777f,  -35.7080f,36.3700f,-33.7578f,  -35.1334f,34.7648f,-34.8043f,  -35.4687f,33.7084f,-33.8381f,  -34.6211f,32.1677f,-34.7920f,  
	-34.8886f,31.1394f,-33.8720f,  -33.7869f,29.6973f,-34.7138f,  -33.9940f,28.7013f,-33.8350f,  -32.6628f,27.3880f,-34.5460f,  -32.8183f,26.4271f,-33.7015f,  
	-31.2876f,25.2688f,-34.2648f,  -31.4013f,24.3440f,-33.4475f,  -29.7052f,23.3619f,-33.8482f,  -29.7850f,22.4720f,-33.0528f,  -27.9608f,21.6825f,-33.2788f,  
	-28.0159f,20.8254f,-32.4996f,  -26.1035f,20.2395f,-32.5416f,  -26.1423f,19.4110f,-31.7735f,  13.5436f,10.3428f,-71.5457f,  11.7355f,12.6353f,-71.5473f,  
	11.2029f,12.1647f,-69.6774f,  9.4499f,14.2957f,-69.5753f,  9.1658f,13.9166f,-67.6317f,  7.4789f,15.8801f,-67.4370f,  7.4463f,15.6039f,-65.4559f,  
	5.8341f,17.3980f,-65.1799f,  6.0496f,17.2366f,-63.1976f,  4.5197f,18.8613f,-62.8520f,  4.9733f,18.8273f,-60.9039f,  3.5311f,20.2857f,-60.5007f,  
	4.2063f,20.3913f,-58.6205f,  2.8548f,21.6899f,-58.1706f,  3.7285f,21.9470f,-56.3895f,  2.4687f,23.0942f,-55.9033f,  3.5124f,23.5134f,-54.2489f,  
	2.3427f,24.5203f,-53.7354f,  3.5222f,25.1107f,-52.2314f,  2.4377f,25.9905f,-51.6974f,  3.7147f,26.7586f,-50.3626f,  2.7079f,27.5262f,-49.8126f,  
	4.0402f,28.4754f,-48.6610f,  3.1022f,29.1454f,-48.0975f,  4.4454f,30.2753f,-47.1376f,  3.5638f,30.8632f,-46.5597f,  4.8724f,32.1689f,-45.7949f,  
	4.0325f,32.6893f,-45.1987f,  5.2613f,34.1610f,-44.6273f,  4.4480f,34.6265f,-44.0064f,  5.5534f,36.2490f,-43.6224f,  4.7505f,36.6697f,-42.9678f,  
	5.6923f,38.4226f,-42.7611f,  4.8850f,38.8045f,-42.0636f,  5.6274f,40.6617f,-42.0212f,  4.7998f,41.0089f,-41.2685f,  5.3131f,42.9395f,-41.3754f,  
	4.4536f,43.2512f,-40.5573f,  4.7151f,45.2200f,-40.7976f,  3.8139f,45.4917f,-39.9047f,  3.8087f,47.4600f,-40.2622f,  2.8586f,47.6849f,-39.2866f,  
	2.5807f,49.6119f,-39.7463f,  1.5796f,49.7803f,-38.6845f,  1.0314f,51.6244f,-39.2328f,  -0.0209f,51.7258f,-38.0831f,  -0.8281f,53.4460f,-38.7087f,  
	-1.9279f,53.4694f,-37.4738f,  -2.9745f,55.0267f,-38.1677f,  -4.1135f,54.9612f,-36.8550f,  -5.3720f,56.3203f,-37.6108f,  -6.5411f,56.1571f,-36.2298f,  
	-7.9776f,57.2873f,-37.0428f,  -9.1669f,57.0200f,-35.6054f,  -10.7418f,57.8965f,-36.4734f,  -11.9351f,57.5207f,-34.9957f,  -13.6045f,58.1256f,-35.9174f,  
	-14.7888f,57.6405f,-34.4149f,  -16.5058f,57.9630f,-35.3896f,  -17.6680f,57.3707f,-33.8781f,  -19.3845f,57.4078f,-34.9051f,  -20.5134f,56.7125f,-33.3996f,  
	-22.1810f,56.4685f,-34.4775f,  -23.2585f,55.6798f,-32.9945f,  -24.8306f,55.1667f,-34.1203f,  -25.8504f,54.2927f,-32.6711f,  -27.2830f,53.5291f,-33.8402f,  
	-28.2339f,52.5824f,-32.4366f,  -29.4869f,51.5931f,-33.6421f,  -30.3614f,50.5865f,-32.2913f,  -31.3999f,49.4014f,-33.5237f,  -32.1921f,48.3494f,-32.2320f,  
	-32.9871f,47.0026f,-33.4795f,  -33.6943f,45.9185f,-32.2506f,  -34.2233f,44.4473f,-33.4987f,  -34.8442f,43.3451f,-32.3322f,  -35.0920f,41.7887f,-33.5644f,  
	-35.6277f,40.6810f,-32.4604f,  -35.5861f,39.0794f,-33.6584f,  -36.0396f,37.9773f,-32.6138f,  -35.7080f,36.3700f,-33.7578f,  -36.0841f,35.2830f,-32.7684f,  
	-35.4687f,33.7084f,-33.8381f,  -35.7741f,32.6454f,-32.8973f,  -34.8886f,31.1394f,-33.8720f,  -35.1300f,30.1057f,-32.9753f,  -33.9940f,28.7013f,-33.8350f,  
	-34.1795f,27.7008f,-32.9757f,  -32.8183f,26.4271f,-33.7015f,  -32.9573f,25.4623f,-32.8727f,  -31.4013f,24.3440f,-33.4475f,  -31.5015f,23.4146f,-32.6438f,  
	-29.7850f,22.4720f,-33.0528f,  -29.8554f,21.5770f,-32.2686f,  -28.0159f,20.8254f,-32.4996f,  -28.0652f,19.9614f,-31.7292f,  -26.1423f,19.4110f,-31.7735f,  
	-26.1766f,18.5746f,-31.0137f,  11.7355f,12.6353f,-71.5473f,  9.9283f,14.8324f,-71.4422f,  9.4499f,14.2957f,-69.5753f,  7.7082f,16.3233f,-69.3743f,  
	7.4789f,15.8801f,-67.4370f,  5.8121f,17.7351f,-67.1517f,  5.8341f,17.3980f,-65.1799f,  4.2511f,19.0790f,-64.8225f,  4.5197f,18.8613f,-62.8520f,  
	3.0273f,20.3704f,-62.4349f,  3.5311f,20.2857f,-60.5007f,  2.1338f,21.6279f,-60.0357f,  2.8548f,21.6899f,-58.1706f,  1.5549f,22.8729f,-57.6691f,  
	2.4687f,23.0942f,-55.9033f,  1.2661f,24.1283f,-55.3754f,  2.3427f,24.5203f,-53.7354f,  1.2339f,25.4185f,-53.1893f,  2.4377f,25.9905f,-51.6974f,  
	1.4167f,26.7676f,-51.1389f,  2.7079f,27.5262f,-49.8126f,  1.7664f,28.1971f,-49.2458f,  3.1022f,29.1454f,-48.0975f,  2.2288f,29.7264f,-47.5227f,  
	3.5638f,30.8632f,-46.5597f,  2.7442f,31.3700f,-45.9743f,  4.0325f,32.6893f,-45.1987f,  3.2516f,33.1360f,-44.5972f,  4.4480f,34.6265f,-44.0064f,  
	3.6892f,35.0249f,-43.3807f,  4.7505f,36.6697f,-42.9678f,  3.9983f,37.0281f,-42.3088f,  4.8850f,38.8045f,-42.0636f,  4.1220f,39.1289f,-41.3586f,  
	4.7998f,41.0089f,-41.2685f,  4.0128f,41.3005f,-40.5068f,  4.4536f,43.2512f,-40.5573f,  3.6307f,43.5074f,-39.7282f,  3.8139f,45.4917f,-39.9047f,  
	2.9456f,45.7071f,-38.9985f,  2.8586f,47.6849f,-39.2866f,  1.9406f,47.8509f,-38.2979f,  1.5796f,49.7803f,-38.6845f,  0.6091f,49.8872f,-37.6093f,  
	-0.0209f,51.7258f,-38.0831f,  -1.0424f,51.7625f,-36.9219f,  -1.9279f,53.4694f,-37.4738f,  -2.9943f,53.4246f,-36.2318f,  -4.1135f,54.9612f,-36.8550f,  
	-5.2174f,54.8254f,-35.5397f,  -6.5411f,56.1571f,-36.2298f,  -7.6740f,55.9226f,-34.8510f,  -9.1669f,57.0200f,-35.6054f,  -10.3139f,56.6807f,-34.1782f,  
	-11.9351f,57.5207f,-34.9957f,  -13.0840f,57.0742f,-33.5345f,  -14.7888f,57.6405f,-34.4149f,  -15.9267f,57.0870f,-32.9350f,  -17.6680f,57.3707f,-33.8781f,  
	-18.7838f,56.7128f,-32.3944f,  -20.5134f,56.7125f,-33.3996f,  -21.5883f,55.9579f,-31.9291f,  -23.2585f,55.6798f,-32.9945f,  -24.2857f,54.8354f,-31.5490f,  
	-25.8504f,54.2927f,-32.6711f,  -26.8177f,53.3698f,-31.2633f,  -28.2339f,52.5824f,-32.4366f,  -29.1325f,51.5929f,-31.0750f,  -30.3614f,50.5865f,-32.2913f,  
	-31.1841f,49.5441f,-30.9836f,  -32.1921f,48.3494f,-32.2320f,  -32.9348f,47.2672f,-30.9832f,  -33.6943f,45.9185f,-32.2506f,  -34.3541f,44.8106f,-31.0616f,  
	-34.8442f,43.3451f,-32.3322f,  -35.4209f,42.2244f,-31.2045f,  -35.6277f,40.6810f,-32.4604f,  -36.1228f,39.5592f,-31.3920f,  -36.0396f,37.9773f,-32.6138f,  
	-36.4566f,36.8645f,-31.6014f,  -36.0841f,35.2830f,-32.7684f,  -36.4283f,34.1893f,-31.8063f,  -35.7741f,32.6454f,-32.8973f,  -36.0515f,31.5772f,-31.9815f,  
	-35.1300f,30.1057f,-32.9753f,  -35.3479f,29.0681f,-32.0996f,  -34.1795f,27.7008f,-32.9757f,  -34.3466f,26.6975f,-32.1336f,  -32.9573f,25.4623f,-32.8727f,  
	-33.0810f,24.4940f,-32.0589f,  -31.5015f,23.4146f,-32.6438f,  -31.5906f,22.4816f,-31.8526f,  -29.8554f,21.5770f,-32.2686f,  -29.9186f,20.6767f,-31.4940f,  
	-28.0652f,19.9614f,-31.7292f,  -28.1087f,19.0913f,-30.9680f,  -26.1766f,18.5746f,-31.0137f,  -26.2081f,17.7303f,-30.2616f,  9.9283f,14.8324f,-71.4422f,  
	8.1301f,16.9260f,-71.2346f,  7.7082f,16.3233f,-69.3743f,  5.9840f,18.2423f,-69.0792f,  5.8121f,17.7351f,-67.1517f,  4.1722f,19.4767f,-66.7815f,  
	4.2511f,19.0790f,-64.8225f,  2.7034f,20.6439f,-64.3900f,  3.0273f,20.3704f,-62.4349f,  1.5780f,21.7622f,-61.9528f,  2.1338f,21.6279f,-60.0357f,  
	0.7865f,22.8535f,-59.5161f,  1.5549f,22.8729f,-57.6691f,  0.3111f,23.9415f,-57.1233f,  1.2661f,24.1283f,-55.3754f,  0.1238f,25.0524f,-54.8128f,  
	1.2339f,25.4185f,-53.1893f,  0.1884f,26.2124f,-52.6172f,  1.4167f,26.7676f,-51.1389f,  0.4609f,27.4465f,-50.5627f,  1.7664f,28.1971f,-49.2458f,  
	0.8901f,28.7778f,-48.6674f,  2.2288f,29.7264f,-47.5227f,  1.4182f,30.2255f,-46.9408f,  2.7442f,31.3700f,-45.9743f,  1.9847f,31.8030f,-45.3847f,  
	3.2516f,33.1360f,-44.5972f,  2.5266f,33.5164f,-43.9927f,  3.6892f,35.0249f,-43.3807f,  2.9824f,35.3628f,-42.7526f,  3.9983f,37.0281f,-42.3088f,  
	3.2912f,37.3316f,-41.6444f,  4.1220f,39.1289f,-41.3586f,  3.4002f,39.4012f,-40.6468f,  4.0128f,41.3005f,-40.5068f,  3.2625f,41.5411f,-39.7358f,  
	3.6307f,43.5074f,-39.7282f,  2.8402f,43.7125f,-38.8869f,  2.9456f,45.7071f,-38.9985f,  2.1082f,45.8694f,-38.0797f,  1.9406f,47.8509f,-38.2979f,  
	1.0513f,47.9615f,-37.2956f,  0.6091f,49.8872f,-37.6093f,  -0.3331f,49.9353f,-36.5215f,  -1.0424f,51.7625f,-36.9219f,  -2.0337f,51.7368f,-35.7516f,  
	-2.9943f,53.4246f,-36.2318f,  -4.0290f,53.3148f,-34.9844f,  -5.2174f,54.8254f,-35.5397f,  -6.2888f,54.6230f,-34.2231f,  -7.6740f,55.9226f,-34.8510f,  
	-8.7688f,55.6199f,-33.4788f,  -10.3139f,56.6807f,-34.1782f,  -11.4206f,56.2737f,-32.7635f,  -13.0840f,57.0742f,-33.5345f,  -14.1903f,56.5615f,-32.0920f,  
	-15.9267f,57.0870f,-32.9350f,  -17.0217f,56.4696f,-31.4791f,  -18.7838f,56.7128f,-32.3944f,  -19.8485f,55.9963f,-30.9423f,  -21.5883f,55.9579f,-31.9291f,  
	-22.6155f,55.1479f,-30.4930f,  -24.2857f,54.8354f,-31.5490f,  -25.2625f,53.9417f,-30.1423f,  -26.8177f,53.3698f,-31.2633f,  -27.7343f,52.4032f,-29.8958f,  
	-29.1325f,51.5929f,-31.0750f,  -29.9804f,50.5664f,-29.7555f,  -31.1841f,49.5441f,-30.9836f,  -31.9579f,48.4704f,-29.7181f,  -32.9348f,47.2672f,-30.9832f,  
	-33.6301f,46.1600f,-29.7744f,  -34.3541f,44.8106f,-31.0616f,  -34.9692f,43.6830f,-29.9120f,  -35.4209f,42.2244f,-31.2045f,  -35.9560f,41.0885f,-30.1132f,  
	-36.1228f,39.5592f,-31.3920f,  -36.5801f,38.4260f,-30.3569f,  -36.4566f,36.8645f,-31.6014f,  -36.8400f,35.7449f,-30.6177f,  -36.4283f,34.1893f,-31.8063f,  
	-36.7428f,33.0906f,-30.8707f,  -36.0515f,31.5772f,-31.9815f,  -36.3036f,30.5056f,-31.0884f,  -35.3479f,29.0681f,-32.0996f,  -35.5453f,28.0286f,-31.2428f,  
	-34.3466f,26.6975f,-32.1336f,  -34.4965f,25.6917f,-31.3079f,  -33.0810f,24.4940f,-32.0589f,  -33.1918f,23.5235f,-31.2588f,  -31.5906f,22.4816f,-31.8526f,  
	-31.6708f,21.5448f,-31.0720f,  -29.9186f,20.6767f,-31.4940f,  -29.9747f,19.7721f,-30.7296f,  -28.1087f,19.0913f,-30.9680f,  -28.1484f,18.2150f,-30.2150f,  
	-26.2081f,17.7303f,-30.2616f,  -26.2374f,16.8784f,-29.5170f,  8.1301f,16.9260f,-71.2346f,  6.3468f,18.9110f,-70.9294f,  5.9840f,18.2423f,-69.0792f,  
	4.2842f,20.0475f,-68.6956f,  4.1722f,19.4767f,-66.7815f,  2.5651f,21.1015f,-66.3326f,  2.7034f,20.6439f,-64.3900f,  1.1968f,22.0904f,-63.8890f,  
	1.5780f,21.7622f,-61.9528f,  0.1767f,23.0362f,-61.4127f,  0.7865f,22.8535f,-59.5161f,  -0.5062f,23.9631f,-58.9491f,  0.3111f,23.9415f,-57.1233f,  
	-0.8736f,24.8983f,-56.5401f,  0.1238f,25.0524f,-54.8128f,  -0.9558f,25.8702f,-54.2221f,  0.1884f,26.2124f,-52.6172f,  -0.7920f,26.9065f,-52.0258f,  
	0.4609f,27.4465f,-50.5627f,  -0.4292f,28.0337f,-49.9742f,  0.8901f,28.7778f,-48.6674f,  0.0774f,29.2756f,-48.0818f,  1.4182f,30.2255f,-46.9408f,  
	0.6690f,30.6503f,-46.3555f,  1.9847f,31.8030f,-45.3847f,  1.2824f,32.1698f,-44.7933f,  2.5266f,33.5164f,-43.9927f,  1.8546f,33.8372f,-43.3873f,  
	2.9824f,35.3628f,-42.7526f,  2.3218f,35.6478f,-42.1208f,  3.2912f,37.3316f,-41.6444f,  2.6263f,37.5859f,-40.9750f,  3.4002f,39.4012f,-40.6468f,  
	2.7154f,39.6264f,-39.9273f,  3.2625f,41.5411f,-39.7358f,  2.5442f,41.7351f,-38.9537f,  2.8402f,43.7125f,-38.8869f,  2.0796f,43.8696f,-38.0336f,  
	2.1082f,45.8694f,-38.0797f,  1.2979f,45.9818f,-37.1472f,  1.0513f,47.9615f,-37.2956f,  0.1879f,48.0191f,-36.2795f,  -0.3331f,49.9353f,-36.5215f,  
	-1.2480f,49.9266f,-35.4227f,  -2.0337f,51.7368f,-35.7516f,  -2.9965f,51.6514f,-34.5734f,  -4.0290f,53.3148f,-34.9844f,  -5.0349f,53.1434f,-33.7326f,  
	-6.2888f,54.6230f,-34.2231f,  -7.3260f,54.3564f,-32.9094f,  -8.7688f,55.6199f,-33.4788f,  -9.8271f,55.2529f,-32.1150f,  -11.4206f,56.2737f,-32.7635f,  
	-12.4884f,55.8032f,-31.3634f,  -14.1903f,56.5615f,-32.0920f,  -15.2572f,55.9869f,-30.6692f,  -17.0217f,56.4696f,-31.4791f,  -18.0689f,55.7943f,-30.0511f,  
	-19.8485f,55.9963f,-30.9423f,  -20.8686f,55.2248f,-29.5212f,  -22.6155f,55.1479f,-30.4930f,  -23.5950f,54.2883f,-29.0928f,  -25.2625f,53.9417f,-30.1423f,  
	-26.1907f,53.0036f,-28.7739f,  -27.7343f,52.4032f,-29.8958f,  -28.6016f,51.3987f,-28.5688f,  -29.9804f,50.5664f,-29.7555f,  -30.7803f,49.5073f,-28.4772f,  
	-31.9579f,48.4704f,-29.7181f,  -32.6845f,47.3702f,-28.4925f,  -33.6301f,46.1600f,-29.7744f,  -34.2804f,45.0319f,-28.6050f,  -34.9692f,43.6830f,-29.9120f,  
	-35.5421f,42.5393f,-28.7994f,  -35.9560f,41.0885f,-30.1132f,  -36.4523f,39.9406f,-29.0562f,  -36.5801f,38.4260f,-30.3569f,  -37.0021f,37.2855f,-29.3518f,  
	-36.8400f,35.7449f,-30.6177f,  -37.1919f,34.6201f,-29.6620f,  -36.7428f,33.0906f,-30.8707f,  -37.0301f,31.9887f,-29.9594f,  -36.3036f,30.5056f,-31.0884f,  
	-36.5330f,29.4327f,-30.2157f,  -35.5453f,28.0286f,-31.2428f,  -35.7237f,26.9874f,-30.4038f,  -34.4965f,25.6917f,-31.3079f,  -34.6315f,24.6851f,-30.4973f,  
	-33.1918f,23.5235f,-31.2588f,  -33.2920f,22.5508f,-30.4703f,  -31.6708f,21.5448f,-31.0720f,  -31.7425f,20.6053f,-30.3024f,  -29.9747f,19.7721f,-30.7296f,  
	-30.0257f,18.8630f,-29.9738f,  -28.1484f,18.2150f,-30.2150f,  -28.1850f,17.3330f,-29.4699f,  -26.2374f,16.8784f,-29.5170f,  -26.2650f,16.0192f,-28.7796f,  
	6.3468f,18.9110f,-70.9294f,  4.5853f,20.7817f,-70.5319f,  4.2842f,20.0475f,-68.6956f,  2.6147f,21.7353f,-68.2294f,  2.5651f,21.1015f,-66.3326f,  
	0.9968f,22.6068f,-65.8114f,  1.1968f,22.0904f,-63.8890f,  -0.2640f,23.4179f,-63.3263f,  0.1767f,23.0362f,-61.4127f,  -1.1717f,24.1926f,-60.8217f,  
	-0.5062f,23.9631f,-58.9491f,  -1.7412f,24.9591f,-58.3417f,  -0.8736f,24.8983f,-56.5401f,  -1.9964f,25.7469f,-55.9263f,  -0.9558f,25.8702f,-54.2221f,  
	-1.9709f,26.5863f,-53.6101f,  -0.7920f,26.9065f,-52.0258f,  -1.7067f,27.5069f,-51.4208f,  -0.4292f,28.0337f,-49.9742f,  -1.2553f,28.5364f,-49.3778f,  
	0.0774f,29.2756f,-48.0818f,  -0.6731f,29.6977f,-47.4928f,  0.6690f,30.6503f,-46.3555f,  -0.0220f,31.0084f,-45.7689f,  1.2824f,32.1698f,-44.7933f,  
	0.6344f,32.4774f,-44.2021f,  1.8546f,33.8372f,-43.3873f,  1.2301f,34.1063f,-42.7797f,  2.3218f,35.6478f,-42.1208f,  1.7043f,35.8856f,-41.4857f,  
	2.6263f,37.5859f,-40.9750f,  1.9988f,37.7960f,-40.2994f,  2.7154f,39.6264f,-39.9273f,  2.0626f,39.8089f,-39.1979f,  2.5442f,41.7351f,-38.9537f,  
	1.8552f,41.8858f,-38.1605f,  2.0796f,43.8696f,-38.0336f,  1.3448f,43.9819f,-37.1666f,  1.2979f,45.9818f,-37.1472f,  0.5116f,46.0465f,-36.2000f,  
	0.1879f,48.0191f,-36.2795f,  -0.6507f,48.0251f,-35.2508f,  -1.2480f,49.9266f,-35.4227f,  -2.1374f,49.8635f,-34.3138f,  -2.9965f,51.6514f,-34.5734f,  
	-3.9337f,51.5092f,-33.3877f,  -5.0349f,53.1434f,-33.7326f,  -6.0103f,52.9119f,-32.4799f,  -7.3260f,54.3564f,-32.9094f,  -8.3306f,54.0291f,-31.6002f,  
	-9.8271f,55.2529f,-32.1150f,  -10.8503f,54.8251f,-30.7617f,  -12.4884f,55.8032f,-31.3634f,  -13.5206f,55.2730f,-29.9788f,  -15.2572f,55.9869f,-30.6692f,  
	-16.2799f,55.3555f,-29.2706f,  -18.0689f,55.7943f,-30.0511f,  -19.0747f,55.0648f,-28.6503f,  -20.8686f,55.2248f,-29.5212f,  -21.8437f,54.4037f,-28.1328f,  
	-23.5950f,54.2883f,-29.0928f,  -24.5281f,53.3839f,-27.7283f,  -26.1907f,53.0036f,-28.7739f,  -27.0712f,52.0267f,-27.4438f,  -28.6016f,51.3987f,-28.5688f,  
	-29.4221f,50.3604f,-27.2815f,  -30.7803f,49.5073f,-28.4772f,  -31.5336f,48.4207f,-27.2379f,  -32.6845f,47.3702f,-28.4925f,  -33.3662f,46.2480f,-27.3060f,  
	-34.2804f,45.0319f,-28.6050f,  -34.8882f,43.8865f,-27.4729f,  -35.5421f,42.5393f,-28.7994f,  -36.0754f,41.3825f,-27.7216f,  -36.4523f,39.9406f,-29.0562f,  
	-36.9119f,38.7847f,-28.0303f,  -37.0021f,37.2855f,-29.3518f,  -37.3913f,36.1394f,-28.3759f,  -37.1919f,34.6201f,-29.6620f,  -37.5150f,33.4920f,-28.7318f,  
	-37.0301f,31.9887f,-29.9594f,  -37.2928f,30.8858f,-29.0697f,  -36.5330f,29.4327f,-30.2157f,  -36.7416f,28.3587f,-29.3620f,  -35.7237f,26.9874f,-30.4038f,  
	-35.8853f,25.9464f,-29.5811f,  -34.6315f,24.6851f,-30.4973f,  -34.7542f,23.6774f,-29.6993f,  -33.2920f,22.5508f,-30.4703f,  -33.3820f,21.5768f,-29.6938f,  
	-31.7425f,20.6053f,-30.3024f,  -31.8078f,19.6630f,-29.5421f,  -30.0257f,18.8630f,-29.9738f,  -30.0724f,17.9498f,-29.2265f,  -28.1850f,17.3330f,-29.4699f,  
	-28.2191f,16.4454f,-28.7324f,  -26.2650f,16.0192f,-28.7796f,  -26.2912f,15.1532f,-28.0495f,  4.5853f,20.7817f,-70.5319f,  2.8513f,22.5347f,-70.0479f,  
	2.6147f,21.7353f,-68.2294f,  0.9815f,23.3028f,-67.6869f,  0.9968f,22.6068f,-65.8114f,  -0.5281f,23.9921f,-65.2244f,  -0.2640f,23.4179f,-63.3263f,  
	-1.6741f,24.6262f,-62.7090f,  -1.1717f,24.1926f,-60.8217f,  -2.4642f,25.2335f,-60.1867f,  -1.7412f,24.9591f,-58.3417f,  -2.9156f,25.8445f,-57.7005f,  
	-1.9964f,25.7469f,-55.9263f,  -3.0556f,26.4914f,-55.2883f,  -1.9709f,26.5863f,-53.6101f,  -2.9205f,27.2063f,-52.9824f,  -1.7067f,27.5069f,-51.4208f,  
	-2.5572f,28.0205f,-50.8065f,  -1.2553f,28.5364f,-49.3778f,  -2.0185f,28.9615f,-48.7773f,  -0.6731f,29.6977f,-47.4928f,  -1.3643f,30.0518f,-46.9026f,  
	-0.0220f,31.0084f,-45.7689f,  -0.6575f,31.3066f,-45.1831f,  0.6344f,32.4774f,-44.2021f,  0.0351f,32.7334f,-43.6099f,  1.2301f,34.1063f,-42.7797f,  
	0.6499f,34.3293f,-42.1702f,  1.7043f,35.8856f,-41.4857f,  1.1250f,36.0814f,-40.8458f,  1.9988f,37.7960f,-40.2994f,  1.4036f,37.9666f,-39.6151f,  
	2.0626f,39.8089f,-39.1979f,  1.4389f,39.9518f,-38.4581f,  1.8552f,41.8858f,-38.1605f,  1.1912f,41.9961f,-37.3536f,  1.3448f,43.9819f,-37.1666f,  
	0.6326f,44.0514f,-36.2843f,  0.5116f,46.0465f,-36.2000f,  -0.2520f,46.0647f,-35.2389f,  -0.6507f,48.0251f,-35.2508f,  -1.4664f,47.9817f,-34.2100f,  
	-2.1374f,49.8635f,-34.3138f,  -3.0042f,49.7483f,-33.1944f,  -3.9337f,51.5092f,-33.3877f,  -4.8441f,51.3111f,-32.1976f,  -6.0103f,52.9119f,-32.4799f,  
	-6.9569f,52.6233f,-31.2278f,  -8.3306f,54.0291f,-31.6002f,  -9.3039f,53.6440f,-30.2972f,  -10.8503f,54.8251f,-30.7617f,  -11.8416f,54.3402f,-29.4195f,  
	-13.5206f,55.2730f,-29.9788f,  -14.5122f,54.6873f,-28.6142f,  -16.2799f,55.3555f,-29.2706f,  -17.2645f,54.6708f,-27.8952f,  -19.0747f,55.0648f,-28.6503f,  
	-20.0385f,54.2859f,-27.2787f,  -21.8437f,54.4037f,-28.1328f,  -22.7749f,53.5375f,-26.7770f,  -24.5281f,53.3839f,-27.7283f,  -25.4155f,52.4398f,-26.3997f,  
	-27.0712f,52.0267f,-27.4438f,  -27.9066f,51.0150f,-26.1515f,  -29.4221f,50.3604f,-27.2815f,  -30.1969f,49.2933f,-26.0321f,  -31.5336f,48.4207f,-27.2379f,  
	-32.2424f,47.3106f,-26.0371f,  -33.3662f,46.2480f,-27.3060f,  -34.0054f,45.1072f,-26.1566f,  -34.8882f,43.8865f,-27.4729f,  -35.4558f,42.7269f,-26.3760f,  
	-36.0754f,41.3825f,-27.7216f,  -36.5709f,40.2169f,-26.6758f,  -36.9119f,38.7847f,-28.0303f,  -37.3374f,37.6226f,-27.0345f,  -37.3913f,36.1394f,-28.3759f,  
	-37.7500f,34.9898f,-27.4266f,  -37.5150f,33.4920f,-28.7318f,  -37.8116f,32.3632f,-27.8246f,  -37.2928f,30.8858f,-29.0697f,  -37.5329f,29.7824f,-28.2004f,  
	-36.7416f,28.3587f,-29.3620f,  -36.9315f,27.2858f,-28.5260f,  -35.8853f,25.9464f,-29.5811f,  -36.0326f,24.9055f,-28.7720f,  -34.7542f,23.6774f,-29.6993f,  
	-34.8652f,22.6698f,-28.9142f,  -33.3820f,21.5768f,-29.6938f,  -33.4641f,20.6014f,-28.9273f,  -31.8078f,19.6630f,-29.5421f,  -31.8674f,18.7183f,-28.7909f,  
	-30.0724f,17.9498f,-29.2265f,  -30.1156f,17.0327f,-28.4871f,  -28.2191f,16.4454f,-28.7324f,  -28.2509f,15.5524f,-28.0022f,  -26.2912f,15.1532f,-28.0495f,  
	-26.3173f,14.2799f,-27.3257f,  2.8513f,22.5347f,-70.0479f,  1.1510f,24.1663f,-69.4834f,  0.9815f,23.3028f,-67.6869f,  -0.6111f,24.7489f,-67.0744f,  
	-0.5281f,23.9921f,-65.2244f,  -2.0047f,25.2564f,-64.5786f,  -1.6741f,24.6262f,-62.7090f,  -3.0302f,25.7170f,-62.0436f,  -2.4642f,25.2335f,-60.1867f,  
	-3.6976f,26.1615f,-59.5143f,  -2.9156f,25.8445f,-57.7005f,  -4.0275f,26.6235f,-57.0319f,  -3.0556f,26.4914f,-55.2883f,  -4.0498f,27.1371f,-54.6322f,  
	-2.9205f,27.2063f,-52.9824f,  -3.8059f,27.7369f,-52.3435f,  -2.5572f,28.0205f,-50.8065f,  -3.3443f,28.4543f,-50.1870f,  -2.0185f,28.9615f,-48.7773f,  
	-2.7215f,29.3166f,-48.1753f,  -1.3643f,30.0518f,-46.9026f,  -1.9989f,30.3448f,-46.3135f,  -0.6575f,31.3066f,-45.1831f,  -1.2430f,31.5527f,-44.5971f,  
	0.0351f,32.7334f,-43.6099f,  -0.5187f,32.9438f,-43.0169f,  0.6499f,34.3293f,-42.1702f,  0.1089f,34.5118f,-41.5572f,  1.1250f,36.0814f,-40.8458f,  
	0.5787f,36.2400f,-40.1984f,  1.4036f,37.9666f,-39.6151f,  0.8377f,38.1008f,-38.9213f,  1.4389f,39.9518f,-38.4581f,  0.8398f,40.0583f,-37.7050f,  
	1.1912f,41.9961f,-37.3536f,  0.5487f,42.0680f,-36.5311f,  0.6326f,44.0514f,-36.2843f,  -0.0588f,44.0791f,-35.3870f,  -0.2520f,46.0647f,-35.2389f,  
	-0.9951f,46.0382f,-34.2637f,  -1.4664f,47.9817f,-34.2100f,  -2.2622f,47.8908f,-33.1560f,  -3.0042f,49.7483f,-33.1944f,  -3.8476f,49.5814f,-32.0674f,  
	-4.8441f,51.3111f,-32.1976f,  -5.7293f,51.0596f,-31.0043f,  -6.9569f,52.6233f,-31.2278f,  -7.8758f,52.2800f,-29.9779f,  -9.3039f,53.6440f,-30.2972f,  
	-10.2489f,53.2043f,-29.0010f,  -11.8416f,54.3402f,-29.4195f,  -12.7960f,53.8011f,-28.0932f,  -14.5122f,54.6873f,-28.6142f,  -15.4691f,54.0493f,-27.2687f,  
	-17.2645f,54.6708f,-27.8952f,  -18.2104f,53.9370f,-26.5450f,  -20.0385f,54.2859f,-27.2787f,  -20.9611f,53.4617f,-25.9364f,  -22.7749f,53.5375f,-26.7770f,  
	-23.6627f,52.6308f,-25.4544f,  -25.4155f,52.4398f,-26.3997f,  -26.2596f,51.4599f,-25.1067f,  -27.9066f,51.0150f,-26.1515f,  -28.6974f,49.9731f,-24.8957f,  
	-30.1969f,49.2933f,-26.0321f,  -30.9280f,48.2013f,-24.8201f,  -32.2424f,47.3106f,-26.0371f,  -32.9089f,46.1805f,-24.8730f,  -34.0054f,45.1072f,-26.1566f,  
	-34.6043f,43.9510f,-25.0424f,  -35.4558f,42.7269f,-26.3760f,  -35.9849f,41.5575f,-25.3115f,  -36.5709f,40.2169f,-26.6758f,  -37.0313f,39.0443f,-25.6607f,  
	-37.3374f,37.6226f,-27.0345f,  -37.7312f,36.4565f,-26.0663f,  -37.7500f,34.9898f,-27.4266f,  -38.0806f,33.8394f,-26.5015f,  -37.8116f,32.3632f,-27.8246f,  
	-38.0839f,31.2339f,-26.9388f,  -37.5329f,29.7824f,-28.2004f,  -37.7524f,28.6805f,-27.3499f,  -36.9315f,27.2858f,-28.5260f,  -37.1053f,26.2139f,-27.7045f,  
	-36.0326f,24.9055f,-28.7720f,  -36.1665f,23.8657f,-27.9769f,  -34.8652f,22.6698f,-28.9142f,  -34.9664f,21.6622f,-28.1400f,  -33.4641f,20.6014f,-28.9273f,  
	-33.5391f,19.6252f,-28.1706f,  -31.8674f,18.7183f,-28.7909f,  -31.9223f,17.7711f,-28.0481f,  -30.1156f,17.0327f,-28.4871f,  -30.1557f,16.1117f,-27.7553f,  
	-28.2509f,15.5524f,-28.0022f,  -28.2820f,14.6538f,-27.2784f,  -26.3173f,14.2799f,-27.3257f,  -26.3437f,13.3995f,-26.6080f,  1.1510f,24.1663f,-69.4834f,  
	-0.5116f,25.6754f,-68.8446f,  -0.6111f,24.7489f,-67.0744f,  -2.1578f,26.0725f,-66.3987f,  -2.0047f,25.2564f,-64.5786f,  -3.4295f,26.4012f,-63.8805f,  
	-3.0302f,25.7170f,-62.0436f,  -4.3290f,26.6923f,-61.3369f,  -3.6976f,26.1615f,-59.5143f,  -4.8699f,26.9802f,-58.8110f,  -4.0275f,26.6235f,-57.0319f,  
	-5.0752f,27.3005f,-56.3422f,  -4.0498f,27.1371f,-54.6322f,  -4.9801f,27.6904f,-53.9625f,  -3.8059f,27.7369f,-52.3435f,  -4.6277f,28.1847f,-51.6977f,  
	-3.3443f,28.4543f,-50.1870f,  -4.0704f,28.8153f,-49.5650f,  -2.7215f,29.3166f,-48.1753f,  -3.3669f,29.6086f,-47.5739f,  -1.9989f,30.3448f,-46.3135f,  
	-2.5822f,30.5844f,-45.7244f,  -1.2430f,31.5527f,-44.5971f,  -1.7815f,31.7527f,-44.0111f,  -0.5187f,32.9438f,-43.0169f,  -1.0321f,33.1141f,-42.4213f,  
	0.1089f,34.5118f,-41.5572f,  -0.3982f,34.6584f,-40.9376f,  0.5787f,36.2400f,-40.1984f,  0.0621f,36.3645f,-39.5423f,  0.8377f,38.1008f,-38.9213f,  
	0.2964f,38.2016f,-38.2147f,  0.8398f,40.0583f,-37.7050f,  0.2615f,40.1300f,-36.9360f,  0.5487f,42.0680f,-36.5311f,  -0.0743f,42.1021f,-35.6926f,  
	-0.0588f,44.0791f,-35.3870f,  -0.7314f,44.0663f,-34.4741f,  -0.9951f,46.0382f,-34.2637f,  -1.7205f,45.9686f,-33.2731f,  -2.2622f,47.8908f,-33.1560f,  
	-3.0376f,47.7523f,-32.0913f,  -3.8476f,49.5814f,-32.0674f,  -4.6690f,49.3646f,-30.9334f,  -5.7293f,51.0596f,-31.0043f,  -6.5902f,50.7565f,-29.8091f,  
	-7.8758f,52.2800f,-29.9779f,  -8.7697f,51.8849f,-28.7304f,  -10.2489f,53.2043f,-29.0010f,  -11.1607f,52.7121f,-27.7164f,  -12.7960f,53.8011f,-28.0932f,  
	-13.7192f,53.2111f,-26.7817f,  -15.4691f,54.0493f,-27.2687f,  -16.3906f,53.3629f,-25.9444f,  -18.2104f,53.9370f,-26.5450f,  -19.1179f,53.1580f,-25.2207f,  
	-20.9611f,53.4617f,-25.9364f,  -21.8430f,52.5965f,-24.6242f,  -23.6627f,52.6308f,-25.4544f,  -24.5095f,51.6875f,-24.1647f,  -26.2596f,51.4599f,-25.1067f,  
	-27.0607f,50.4486f,-23.8482f,  -28.6974f,49.9731f,-24.8957f,  -29.4456f,48.9051f,-23.6758f,  -30.9280f,48.2013f,-24.8201f,  -31.6174f,47.0881f,-23.6439f,  
	-32.9089f,46.1805f,-24.8730f,  -33.5353f,45.0337f,-23.7435f,  -34.6043f,43.9510f,-25.0424f,  -35.1640f,42.7837f,-23.9607f,  -35.9849f,41.5575f,-25.3115f,  
	-36.4781f,40.3801f,-24.2781f,  -37.0313f,39.0443f,-25.6607f,  -37.4587f,37.8670f,-24.6740f,  -37.7312f,36.4565f,-26.0663f,  -38.0952f,35.2893f,-25.1232f,  
	-38.0806f,33.8394f,-26.5015f,  -38.3853f,32.6884f,-25.5988f,  -38.0839f,31.2339f,-26.9388f,  -38.3338f,30.1066f,-26.0730f,  -37.7524f,28.6805f,-27.3499f,  
	-37.9539f,27.5803f,-26.5150f,  -37.1053f,26.2139f,-27.7045f,  -37.2638f,25.1440f,-26.8981f,  -36.1665f,23.8657f,-27.9769f,  -36.2890f,22.8271f,-27.1935f,  
	-34.9664f,21.6622f,-28.1400f,  -35.0591f,20.6551f,-27.3762f,  -33.5391f,19.6252f,-28.1706f,  -33.6081f,18.6480f,-27.4227f,  -31.9223f,17.7711f,-28.0481f,  
	-31.9730f,16.8216f,-27.3131f,  -30.1557f,16.1117f,-27.7553f,  -30.1942f,15.1869f,-27.0300f,  -28.2820f,14.6538f,-27.2784f,  -28.3127f,13.7498f,-26.5607f,  
	-26.3437f,13.3995f,-26.6080f,  -26.3703f,12.5123f,-25.8962f,  -0.5116f,25.6754f,-68.8446f,  -2.1310f,27.0605f,-68.1380f,  -2.1578f,26.0725f,-66.3987f,  
	-3.6554f,27.2745f,-65.6662f,  -3.4295f,26.4012f,-63.8805f,  -4.7991f,27.4279f,-63.1369f,  -4.3290f,26.6923f,-61.3369f,  -5.5686f,27.5554f,-60.5951f,  
	-4.8699f,26.9802f,-58.8110f,  -5.9793f,27.6936f,-58.0829f,  -5.0752f,27.3005f,-56.3422f,  -6.0597f,27.8817f,-55.6358f,  -4.9801f,27.6904f,-53.9625f,  
	-5.8469f,28.1573f,-53.2836f,  -4.6277f,28.1847f,-51.6977f,  -5.3879f,28.5565f,-51.0479f,  -4.0704f,28.8153f,-49.5650f,  -4.7381f,29.1105f,-48.9428f,  
	-3.3669f,29.6086f,-47.5739f,  -3.9597f,29.8450f,-46.9726f,  -2.5822f,30.5844f,-45.7244f,  -3.1171f,30.7764f,-45.1359f,  -1.7815f,31.7527f,-44.0111f,  
	-2.2784f,31.9121f,-43.4234f,  -1.0321f,33.1141f,-42.4213f,  -1.5103f,33.2491f,-41.8203f,  -0.3982f,34.6584f,-40.9376f,  -0.8748f,34.7725f,-40.3103f,  
	0.0621f,36.3645f,-39.5423f,  -0.4295f,36.4579f,-38.8740f,  0.2964f,38.2016f,-38.2147f,  -0.2243f,38.2708f,-37.4923f,  0.2615f,40.1300f,-36.9360f,  
	-0.2983f,40.1676f,-36.1507f,  -0.0743f,42.1021f,-35.6926f,  -0.6800f,42.0998f,-34.8375f,  -0.7314f,44.0663f,-34.4741f,  -1.3882f,44.0145f,-33.5438f,  
	-1.7205f,45.9686f,-33.2731f,  -2.4281f,45.8552f,-32.2691f,  -3.0376f,47.7523f,-32.0913f,  -3.7940f,47.5675f,-31.0164f,  -4.6690f,49.3646f,-30.9334f,  
	-5.4693f,49.0996f,-29.7939f,  -6.5902f,50.7565f,-29.8091f,  -7.4294f,50.4044f,-28.6124f,  -8.7697f,51.8849f,-28.7304f,  -9.6343f,51.4389f,-27.4903f,  
	-11.1607f,52.7121f,-27.7164f,  -12.0447f,52.1705f,-26.4423f,  -13.7192f,53.2111f,-26.7817f,  -14.6102f,52.5735f,-25.4871f,  -16.3906f,53.3629f,-25.9444f,  
	-17.2766f,52.6314f,-24.6422f,  -19.1179f,53.1580f,-25.2207f,  -19.9873f,52.3375f,-23.9230f,  -21.8430f,52.5965f,-24.6242f,  -22.6860f,51.6938f,-23.3419f,  
	-24.5095f,51.6875f,-24.1647f,  -25.3149f,50.7116f,-22.9074f,  -27.0607f,50.4486f,-23.8482f,  -27.8207f,49.4098f,-22.6238f,  -29.4456f,48.9051f,-23.6758f,  
	-30.1531f,47.8143f,-22.4904f,  -31.6174f,47.0881f,-23.6439f,  -32.2671f,45.9565f,-22.5016f,  -33.5353f,45.0337f,-23.7435f,  -34.1223f,43.8745f,-22.6465f,  
	-35.1640f,42.7837f,-23.9607f,  -35.6873f,41.6073f,-22.9103f,  -36.4781f,40.3801f,-24.2781f,  -36.9374f,39.1971f,-23.2736f,  -37.4587f,37.8670f,-24.6740f,  
	-37.8551f,36.6880f,-23.7132f,  -38.0952f,35.2893f,-25.1232f,  -38.4318f,34.1212f,-24.2034f,  -38.3853f,32.6884f,-25.5988f,  -38.6658f,31.5396f,-24.7171f,  
	-38.3338f,30.1066f,-26.0730f,  -38.5638f,28.9813f,-25.2238f,  -37.9539f,27.5803f,-26.5150f,  -38.1383f,26.4828f,-25.6960f,  -37.2638f,25.1440f,-26.8981f,  
	-37.4092f,24.0763f,-26.1041f,  -36.2890f,22.8271f,-27.1935f,  -36.4013f,21.7903f,-26.4212f,  -35.0591f,20.6551f,-27.3762f,  -35.1443f,19.6484f,-26.6218f,  
	-33.6081f,18.6480f,-27.4227f,  -33.6717f,17.6699f,-26.6830f,  -31.9730f,16.8216f,-27.3131f,  -32.0211f,15.8698f,-26.5849f,  -30.1942f,15.1869f,-27.0300f,  
	-30.2314f,14.2581f,-26.3109f,  -28.3127f,13.7498f,-26.5607f,  -28.3432f,12.8404f,-25.8488f,  -26.3703f,12.5123f,-25.8962f,  -26.3972f,11.6187f,-25.1903f,  
	-2.1310f,27.0605f,-68.1380f,  -3.7038f,28.3218f,-67.3699f,  -3.6554f,27.2745f,-65.6662f,  -5.0999f,28.3557f,-64.8836f,  -4.7991f,27.4279f,-63.1369f,  
	-6.1115f,28.3394f,-62.3538f,  -5.5686f,27.5554f,-60.5951f,  -6.7466f,28.3099f,-59.8246f,  -5.9793f,27.6936f,-58.0829f,  -7.0263f,28.3076f,-57.3348f,  
	-6.0597f,27.8817f,-55.6358f,  -6.9808f,28.3727f,-54.9176f,  -5.8469f,28.1573f,-53.2836f,  -6.6516f,28.5445f,-52.5988f,  -5.3879f,28.5565f,-51.0479f,  
	-6.0890f,28.8591f,-50.3968f,  -4.7381f,29.1105f,-48.9428f,  -5.3518f,29.3471f,-48.3204f,  -3.9597f,29.8450f,-46.9726f,  -4.5025f,30.0316f,-46.3721f,  
	-3.1171f,30.7764f,-45.1359f,  -3.6087f,30.9266f,-44.5463f,  -2.2784f,31.9121f,-43.4234f,  -2.7386f,32.0360f,-42.8311f,  -1.5103f,33.2491f,-41.8203f,  
	-1.9569f,33.3521f,-41.2124f,  -0.8748f,34.7725f,-40.3103f,  -1.3256f,34.8570f,-39.6716f,  -0.4295f,36.4579f,-38.8740f,  -0.9003f,36.5221f,-38.1903f,  
	-0.2243f,38.2708f,-37.4923f,  -0.7269f,38.3088f,-36.7532f,  -0.2983f,40.1676f,-36.1507f,  -0.8416f,40.1721f,-35.3477f,  -0.6800f,42.0998f,-34.8375f,  
	-1.2711f,42.0620f,-33.9634f,  -1.3882f,44.0145f,-33.5438f,  -2.0295f,43.9226f,-32.5977f,  -2.4281f,45.8552f,-32.2691f,  -3.1194f,45.6989f,-31.2518f,  
	-3.7940f,47.5675f,-31.0164f,  -4.5323f,47.3375f,-29.9325f,  -5.4693f,49.0996f,-29.7939f,  -6.2509f,48.7883f,-28.6489f,  -7.4294f,50.4044f,-28.6124f,  
	-8.2428f,50.0034f,-27.4188f,  -9.6343f,51.4389f,-27.4903f,  -10.4742f,50.9453f,-26.2565f,  -12.0447f,52.1705f,-26.4423f,  -12.8997f,51.5822f,-25.1809f,  
	-14.6102f,52.5735f,-25.4871f,  -15.4689f,51.8911f,-24.2108f,  -17.2766f,52.6314f,-24.6422f,  -18.1275f,51.8582f,-23.3631f,  -19.9873f,52.3375f,-23.9230f,  
	-20.8205f,51.4789f,-22.6522f,  -22.6860f,51.6938f,-23.3419f,  -23.4898f,50.7576f,-22.0894f,  -25.3149f,50.7116f,-22.9074f,  -26.0809f,49.7069f,-21.6820f,  
	-27.8207f,49.4098f,-22.6238f,  -28.5409f,48.3468f,-21.4323f,  -30.1531f,47.8143f,-22.4904f,  -30.8215f,46.7037f,-21.3380f,  -32.2671f,45.9565f,-22.5016f,  
	-32.8774f,44.8111f,-21.3915f,  -34.1223f,43.8745f,-22.6465f,  -34.6725f,42.7047f,-21.5807f,  -35.6873f,41.6073f,-22.9103f,  -36.1761f,40.4241f,-21.8891f,  
	-36.9374f,39.1971f,-23.2736f,  -37.3645f,38.0116f,-22.2956f,  -37.8551f,36.6880f,-23.7132f,  -38.2228f,35.5075f,-22.7764f,  -38.4318f,34.1212f,-24.2034f,  
	-38.7428f,32.9550f,-23.3055f,  -38.6658f,31.5396f,-24.7171f,  -38.9247f,30.3929f,-23.8530f,  -38.5638f,28.9813f,-25.2238f,  -38.7751f,27.8593f,-24.3914f,  
	-38.1383f,26.4828f,-25.6960f,  -38.3079f,25.3884f,-24.8905f,  -37.4092f,24.0763f,-26.1041f,  -37.5427f,23.0115f,-25.3221f,  -36.4013f,21.7903f,-26.4212f,  
	-36.5045f,20.7552f,-25.6590f,  -35.1443f,19.6484f,-26.6218f,  -35.2228f,18.6420f,-25.8760f,  -33.6717f,17.6699f,-26.6830f,  -33.7314f,16.6910f,-25.9505f,  
	-32.0211f,15.8698f,-26.5849f,  -32.0669f,14.9156f,-25.8629f,  -30.2314f,14.2581f,-26.3109f,  -30.2677f,13.3254f,-25.5976f,  -28.3432f,12.8404f,-25.8488f,  
	-28.3734f,11.9259f,-25.1426f,  -26.3972f,11.6187f,-25.1903f,  -26.4247f,10.7184f,-24.4895f,  -3.7038f,28.3218f,-67.3699f,  -5.2260f,29.4598f,-66.5470f,  
	-5.0999f,28.3557f,-64.8836f,  -6.4893f,29.3187f,-64.0568f,  -6.1115f,28.3394f,-62.3538f,  -7.3639f,29.1386f,-61.5376f,  -6.7466f,28.3099f,-59.8246f,  
	-7.8632f,28.9608f,-59.0304f,  -7.0263f,28.3076f,-57.3348f,  -8.0102f,28.8271f,-56.5717f,  -6.9808f,28.3727f,-54.9176f,  -7.8396f,28.7797f,-54.1911f,  
	-6.6516f,28.5445f,-52.5988f,  -7.3964f,28.8585f,-51.9109f,  -6.0890f,28.8591f,-50.3968f,  -6.7347f,29.0994f,-49.7445f,  -5.3518f,29.3471f,-48.3204f,  
	-5.9139f,29.5311f,-47.6985f,  -4.5025f,30.0316f,-46.3721f,  -5.0004f,30.1743f,-45.7709f,  -3.6087f,30.9266f,-44.5463f,  -4.0621f,31.0399f,-43.9530f,  
	-2.7386f,32.0360f,-42.8311f,  -3.1657f,32.1276f,-42.2329f,  -1.9569f,33.3521f,-41.2124f,  -2.3766f,33.4263f,-40.5939f,  -1.3256f,34.8570f,-39.6716f,  
	-1.7549f,34.9138f,-39.0180f,  -0.9003f,36.5221f,-38.1903f,  -1.3528f,36.5573f,-37.4899f,  -0.7269f,38.3088f,-36.7532f,  -1.2135f,38.3165f,-35.9960f,  
	-0.8416f,40.1721f,-35.3477f,  -1.3712f,40.1445f,-34.5247f,  -1.2711f,42.0620f,-33.9634f,  -1.8486f,41.9874f,-33.0714f,  -2.0295f,43.9226f,-32.5977f,  
	-2.6566f,43.7909f,-31.6357f,  -3.1194f,45.6989f,-31.2518f,  -3.7950f,45.5005f,-30.2224f,  -4.5323f,47.3375f,-29.9325f,  -5.2543f,47.0641f,-28.8394f,  
	-6.2509f,48.7883f,-28.6489f,  -7.0101f,48.4300f,-27.5032f,  -8.2428f,50.0034f,-27.4188f,  -9.0346f,49.5565f,-26.2274f,  -10.4742f,50.9453f,-26.2565f,  
	-11.2884f,50.4060f,-25.0311f,  -12.8997f,51.5822f,-25.1809f,  -13.7256f,50.9496f,-23.9339f,  -15.4689f,51.8911f,-24.2108f,  -16.2954f,51.1668f,-22.9540f,  
	-18.1275f,51.8582f,-23.3631f,  -18.9448f,51.0463f,-22.1074f,  -20.8205f,51.4789f,-22.6522f,  -21.6167f,50.5857f,-21.4084f,  -23.4898f,50.7576f,-22.0894f,  
	-24.2560f,49.7911f,-20.8666f,  -26.0809f,49.7069f,-21.6820f,  -26.8086f,48.6764f,-20.4877f,  -28.5409f,48.3468f,-21.4323f,  -29.2231f,47.2624f,-20.2725f,  
	-30.8215f,46.7037f,-21.3380f,  -31.4508f,45.5776f,-20.2170f,  -32.8774f,44.8111f,-21.3915f,  -33.4510f,43.6537f,-20.3121f,  -34.6725f,42.7047f,-21.5807f,  
	-35.1878f,41.5270f,-20.5441f,  -36.1761f,40.4241f,-21.8891f,  -36.6317f,39.2374f,-20.8948f,  -37.3645f,38.0116f,-22.2956f,  -37.7618f,36.8239f,-21.3422f,  
	-38.2228f,35.5075f,-22.7764f,  -38.5634f,34.3287f,-21.8623f,  -38.7428f,32.9550f,-23.3055f,  -39.0303f,31.7911f,-22.4260f,  -38.9247f,30.3929f,-23.8530f,  
	-39.1631f,29.2499f,-23.0065f,  -38.7751f,27.8593f,-24.3914f,  -38.9696f,26.7409f,-23.5733f,  -38.3079f,25.3884f,-24.8905f,  -38.4638f,24.2977f,-24.0976f,  
	-37.5427f,23.0115f,-25.3221f,  -37.6655f,21.9494f,-24.5507f,  -36.5045f,20.7552f,-25.6590f,  -36.5996f,19.7214f,-24.9058f,  -35.2228f,18.6420f,-25.8760f,  
	-35.2960f,17.6364f,-25.1378f,  -33.7314f,16.6910f,-25.9505f,  -33.7878f,15.7112f,-25.2245f,  -32.0669f,14.9156f,-25.8629f,  -32.1110f,13.9590f,-25.1469f,  
	-30.2677f,13.3254f,-25.5976f,  -30.3031f,12.3890f,-24.8899f,  -28.3734f,11.9259f,-25.1426f,  -28.4038f,11.0061f,-24.4414f,  -26.4247f,10.7184f,-24.4895f,  
	-26.4528f,9.8117f,-23.7938f,  -5.2260f,29.4598f,-66.5470f,  -6.6957f,30.4765f,-65.6751f,  -6.4893f,29.3187f,-64.0568f,  -7.8207f,30.1657f,-63.1925f,  
	-7.3639f,29.1386f,-61.5376f,  -8.5562f,29.8301f,-60.6935f,  -7.8632f,28.9608f,-59.0304f,  -8.9174f,29.5128f,-58.2175f,  -8.0102f,28.8271f,-56.5717f,  
	-8.9319f,29.2579f,-55.7975f,  -7.8396f,28.7797f,-54.1911f,  -8.6381f,29.1090f,-53.4594f,  -7.3964f,28.8585f,-51.9109f,  -8.0848f,29.1060f,-51.2208f,  
	-6.7347f,29.0994f,-49.7445f,  -7.3274f,29.2833f,-49.0924f,  -5.9139f,29.5311f,-47.6985f,  -6.4295f,29.6681f,-47.0761f,  -5.0004f,30.1743f,-45.7709f,  
	-5.4580f,30.2780f,-45.1666f,  -4.0621f,31.0399f,-43.9530f,  -4.4806f,31.1200f,-43.3545f,  -3.1657f,32.1276f,-42.2329f,  -3.5644f,32.1902f,-41.6251f,  
	-2.3766f,33.4263f,-40.5939f,  -2.7734f,33.4735f,-39.9615f,  -1.7549f,34.9138f,-39.0180f,  -2.1654f,34.9433f,-38.3482f,  -1.3528f,36.5573f,-37.4899f,  
	-1.7894f,36.5645f,-36.7714f,  -1.2135f,38.3165f,-35.9960f,  -1.6868f,38.2947f,-35.2181f,  -1.3712f,40.1445f,-34.5247f,  -1.8885f,40.0828f,-33.6823f,  
	-1.8486f,41.9874f,-33.0714f,  -2.4137f,41.8760f,-32.1614f,  -2.6566f,43.7909f,-31.6357f,  -3.2703f,43.6200f,-30.6588f,  -3.7950f,45.5005f,-30.2224f,  
	-4.4568f,45.2615f,-29.1806f,  -5.2543f,47.0641f,-28.8394f,  -5.9573f,46.7458f,-27.7418f,  -7.0101f,48.4300f,-27.5032f,  -7.7506f,48.0278f,-26.3555f,  
	-9.0346f,49.5565f,-26.2274f,  -9.8039f,49.0650f,-25.0403f,  -11.2884f,50.4060f,-25.0311f,  -12.0766f,49.8230f,-23.8162f,  -13.7256f,50.9496f,-23.9339f,  
	-14.5223f,50.2752f,-22.7025f,  -16.2954f,51.1668f,-22.9540f,  -17.0910f,50.4035f,-21.7169f,  -18.9448f,51.0463f,-22.1074f,  -19.7275f,50.1990f,-20.8758f,  
	-21.6167f,50.5857f,-21.4084f,  -22.3773f,49.6612f,-20.1915f,  -24.2560f,49.7911f,-20.8666f,  -24.9856f,48.7975f,-19.6727f,  -26.8086f,48.6764f,-20.4877f,  
	-27.4994f,47.6231f,-19.3235f,  -29.2231f,47.2624f,-20.2725f,  -29.8669f,46.1609f,-19.1432f,  -31.4508f,45.5776f,-20.2170f,  -32.0438f,44.4380f,-19.1261f,  
	-33.4510f,43.6537f,-20.3121f,  -33.9895f,42.4870f,-19.2616f,  -35.1878f,41.5270f,-20.5441f,  -35.6692f,40.3445f,-19.5346f,  -36.6317f,39.2374f,-20.8948f,  
	-37.0566f,38.0476f,-19.9254f,  -37.7618f,36.8239f,-21.3422f,  -38.1306f,35.6371f,-20.4121f,  -38.5634f,34.3287f,-21.8623f,  -38.8789f,33.1518f,-20.9674f,  
	-39.0303f,31.7911f,-22.4260f,  -39.2959f,30.6307f,-21.5651f,  -39.1631f,29.2499f,-23.0065f,  -39.3831f,28.1107f,-22.1752f,  -38.9696f,26.7409f,-23.5733f,  
	-39.1488f,25.6269f,-22.7687f,  -38.4638f,24.2977f,-24.0976f,  -38.6074f,23.2105f,-23.3161f,  -37.6655f,21.9494f,-24.5507f,  -37.7787f,20.8896f,-23.7890f,  
	-36.5996f,19.7214f,-24.9058f,  -36.6880f,18.6898f,-24.1607f,  -35.2960f,17.6364f,-25.1378f,  -35.3647f,16.6311f,-24.4064f,  -33.7878f,15.7112f,-25.2245f,  
	-33.8414f,14.7303f,-24.5044f,  -32.1110f,13.9590f,-25.1469f,  -32.1533f,13.0000f,-24.4364f,  -30.3031f,12.3890f,-24.8899f,  -30.3381f,11.4487f,-24.1870f,  
	-28.4038f,11.0061f,-24.4414f,  -28.4345f,10.0813f,-23.7450f,  -26.4528f,9.8117f,-23.7938f,  -26.4816f,8.8987f,-23.1028f,  -6.6957f,30.4765f,-65.6751f,  
	-8.1092f,31.3735f,-64.7609f,  -7.8207f,30.1657f,-63.1925f,  -9.0934f,30.9007f,-62.2958f,  -8.5562f,29.8301f,-60.6935f,  -9.6869f,30.4180f,-59.8271f,  
	-8.9174f,29.5128f,-58.2175f,  -9.9095f,29.9710f,-57.3905f,  -8.9319f,29.2579f,-55.7975f,  -9.7928f,29.6061f,-55.0155f,  -8.6381f,29.1090f,-53.4594f,  
	-9.3790f,29.3669f,-52.7238f,  -8.0848f,29.1060f,-51.2208f,  -8.7186f,29.2927f,-50.5300f,  -7.3274f,29.2833f,-49.0924f,  -7.8716f,29.4166f,-48.4396f,  
	-6.4295f,29.6681f,-47.0761f,  -6.9027f,29.7632f,-46.4511f,  -5.4580f,30.2780f,-45.1666f,  -5.8789f,30.3464f,-44.5580f,  -4.4806f,31.1200f,-43.3545f,  
	-4.8686f,31.1700f,-42.7476f,  -3.5644f,32.1902f,-41.6251f,  -3.9386f,32.2257f,-41.0044f,  -2.7734f,33.4735f,-39.9615f,  -3.1506f,33.4942f,-39.3134f,  
	-2.1654f,34.9433f,-38.3482f,  -2.5593f,34.9463f,-37.6605f,  -1.7894f,36.5645f,-36.7714f,  -2.2124f,36.5443f,-36.0318f,  -1.6868f,38.2947f,-35.2181f,  
	-2.1485f,38.2414f,-34.4197f,  -1.8885f,40.0828f,-33.6823f,  -2.3946f,39.9871f,-32.8200f,  -2.4137f,41.8760f,-32.1614f,  -2.9669f,41.7282f,-31.2341f,  
	-3.2703f,43.6200f,-30.6588f,  -3.8719f,43.4111f,-29.6666f,  -4.4568f,45.2615f,-29.1806f,  -5.1023f,44.9794f,-28.1307f,  -5.9573f,46.7458f,-27.7418f,  
	-6.6441f,46.3853f,-26.6387f,  -7.7506f,48.0278f,-26.3555f,  -8.4715f,47.5823f,-25.2083f,  -9.8039f,49.0650f,-25.0403f,  -10.5501f,48.5308f,-23.8598f,  
	-12.0766f,49.8230f,-23.8162f,  -12.8386f,49.1985f,-22.6132f,  -14.5223f,50.2752f,-22.7025f,  -15.2909f,49.5615f,-21.4873f,  -17.0910f,50.4035f,-21.7169f,  
	-17.8547f,49.6041f,-20.5009f,  -19.7275f,50.1990f,-20.8758f,  -20.4769f,49.3193f,-19.6684f,  -22.3773f,49.6612f,-20.1915f,  -23.1032f,48.7081f,-19.0013f,  
	-24.9856f,48.7975f,-19.6727f,  -25.6798f,47.7796f,-18.5070f,  -27.4994f,47.6231f,-19.3235f,  -28.1527f,46.5509f,-18.1885f,  -29.8669f,46.1609f,-19.1432f,  
	-30.4748f,45.0442f,-18.0431f,  -32.0438f,44.4380f,-19.1261f,  -32.6016f,43.2875f,-18.0637f,  -33.9895f,42.4870f,-19.2616f,  -34.4936f,41.3140f,-18.2383f,  
	-35.6692f,40.3445f,-19.5346f,  -36.1193f,39.1578f,-18.5503f,  -37.0566f,38.0476f,-19.9254f,  -37.4520f,36.8578f,-18.9799f,  -38.1306f,35.6371f,-20.4121f,  
	-38.4731f,34.4517f,-19.5019f,  -38.8789f,33.1518f,-20.9674f,  -39.1711f,31.9782f,-20.0917f,  -39.2959f,30.6307f,-21.5651f,  -39.5413f,29.4743f,-20.7200f,  
	-39.3831f,28.1107f,-22.1752f,  -39.5861f,26.9763f,-21.3581f,  -39.1488f,25.6269f,-22.7687f,  -39.3141f,24.5171f,-21.9761f,  -38.6074f,23.2105f,-23.3161f,  
	-38.7399f,22.1265f,-22.5448f,  -37.7787f,20.8896f,-23.7890f,  -37.8837f,19.8331f,-23.0359f,  -36.6880f,18.6898f,-24.1607f,  -36.7706f,17.6597f,-23.4227f,  
	-35.3647f,16.6311f,-24.4064f,  -35.4294f,15.6262f,-23.6812f,  -33.8414f,14.7303f,-24.5044f,  -33.8923f,13.7484f,-23.7901f,  -32.1533f,13.0000f,-24.4364f,  
	-32.1945f,12.0384f,-23.7309f,  -30.3381f,11.4487f,-24.1870f,  -30.3729f,10.5046f,-23.4889f,  -28.4345f,10.0813f,-23.7450f,  -28.4655f,9.1512f,-23.0530f,  
	-26.4816f,8.8987f,-23.1028f,  -26.5109f,7.9795f,-22.4161f,  -8.1092f,31.3735f,-64.7609f,  -9.4656f,32.1542f,-63.8098f,  -9.0934f,30.9007f,-62.2958f,  
	-10.3055f,31.5272f,-61.3727f,  -9.6869f,30.4180f,-59.8271f,  -10.7559f,30.9069f,-58.9429f,  -9.9095f,29.9710f,-57.3905f,  -10.8407f,30.3413f,-56.5529f,  
	-9.7928f,29.6061f,-55.0155f,  -10.5951f,29.8775f,-54.2280f,  -9.3790f,29.3669f,-52.7238f,  -10.0638f,29.5589f,-51.9866f,  -8.7186f,29.2927f,-50.5300f,  
	-9.3020f,29.4243f,-49.8382f,  -7.8716f,29.4166f,-48.4396f,  -8.3712f,29.5043f,-47.7848f,  -6.9027f,29.7632f,-46.4511f,  -7.3368f,29.8202f,-45.8228f,  
	-5.8789f,30.3464f,-44.5580f,  -6.2671f,30.3830f,-43.9422f,  -4.8686f,31.1700f,-42.7476f,  -5.2304f,31.1920f,-42.1290f,  -3.9386f,32.2257f,-41.0044f,  
	-4.2917f,32.2349f,-40.3690f,  -3.1506f,33.4942f,-39.3134f,  -3.5102f,33.4893f,-38.6481f,  -2.5593f,34.9463f,-37.6605f,  -2.9391f,34.9233f,-36.9519f,  
	-2.2124f,36.5443f,-36.0318f,  -2.6242f,36.4947f,-35.2711f,  -2.1485f,38.2414f,-34.4197f,  -2.5999f,38.1565f,-33.6002f,  -2.3946f,39.9871f,-32.8200f,  
	-2.8903f,39.8574f,-31.9386f,  -2.9669f,41.7282f,-31.2341f,  -3.5097f,41.5448f,-30.2892f,  -3.8719f,43.4111f,-29.6666f,  -4.4599f,43.1613f,-28.6633f,  
	-5.1023f,44.9794f,-28.1307f,  -5.7340f,44.6573f,-27.0720f,  -6.6441f,46.3853f,-26.6387f,  -7.3141f,45.9830f,-25.5321f,  -8.4715f,47.5823f,-25.2083f,  
	-9.1722f,47.0948f,-24.0639f,  -10.5501f,48.5308f,-23.8598f,  -11.2730f,47.9553f,-22.6874f,  -12.8386f,49.1985f,-22.6132f,  -13.5753f,48.5346f,-21.4227f,  
	-15.2909f,49.5615f,-21.4873f,  -16.0302f,48.8111f,-20.2900f,  -17.8547f,49.6041f,-20.5009f,  -18.5874f,48.7712f,-19.3060f,  -20.4769f,49.3193f,-19.6684f,  
	-21.1937f,48.4097f,-18.4850f,  -23.1032f,48.7081f,-19.0013f,  -23.7954f,47.7292f,-17.8372f,  -25.6798f,47.7796f,-18.5070f,  -26.3375f,46.7411f,-17.3691f,  
	-28.1527f,46.5509f,-18.1885f,  -28.7709f,45.4619f,-17.0815f,  -30.4748f,45.0442f,-18.0431f,  -31.0480f,43.9149f,-16.9707f,  -32.6016f,43.2875f,-18.0637f,  
	-33.1250f,42.1292f,-17.0282f,  -34.4936f,41.3140f,-18.2383f,  -34.9660f,40.1355f,-17.2402f,  -36.1193f,39.1578f,-18.5503f,  -36.5390f,37.9700f,-17.5900f,  
	-37.4520f,36.8578f,-18.9799f,  -37.8197f,35.6688f,-18.0548f,  -38.4731f,34.4517f,-19.5019f,  -38.7908f,33.2692f,-18.6116f,  -39.1711f,31.9782f,-20.0917f,  
	-39.4415f,30.8085f,-19.2326f,  -39.5413f,29.4743f,-20.7200f,  -39.7681f,28.3230f,-19.8899f,  -39.5861f,26.9763f,-21.3581f,  -39.7735f,25.8467f,-20.5537f,  
	-39.3141f,24.5171f,-21.9761f,  -39.4667f,23.4111f,-21.1943f,  -38.7399f,22.1265f,-22.5448f,  -38.8625f,21.0469f,-21.7826f,  -37.8837f,19.8331f,-23.0359f,  
	-37.9813f,18.7794f,-22.2903f,  -36.7706f,17.6597f,-23.4227f,  -36.8478f,16.6313f,-22.6913f,  -35.4294f,15.6262f,-23.6812f,  -35.4904f,14.6215f,-22.9618f,  
	-33.8923f,13.7484f,-23.7901f,  -33.9413f,12.7653f,-23.0806f,  -32.1945f,12.0384f,-23.7309f,  -32.2348f,11.0744f,-23.0300f,  -30.3729f,10.5046f,-23.4889f,  
	-30.4075f,9.5565f,-22.7950f,  -28.4655f,9.1512f,-23.0530f,  -28.4967f,8.2161f,-22.3651f,  -26.5109f,7.9795f,-22.4161f,  -26.5404f,7.0546f,-21.7336f,  
	-9.4656f,32.1542f,-63.8098f,  -10.7627f,32.8215f,-62.8280f,  -10.3055f,31.5272f,-61.3727f,  -11.4565f,32.0494f,-60.4282f,  -10.7559f,30.9069f,-58.9429f,  
	-11.7639f,31.3022f,-58.0451f,  -10.8407f,30.3413f,-56.5529f,  -11.7123f,30.6291f,-55.7075f,  -10.5951f,29.8775f,-54.2280f,  -11.3399f,30.0775f,-53.4373f,  
	-10.0638f,29.5589f,-51.9866f,  -10.6963f,29.6908f,-51.2478f,  -9.3020f,29.4243f,-49.8382f,  -9.8385f,29.5058f,-49.1446f,  -8.3712f,29.5043f,-47.7848f,  
	-8.8294f,29.5503f,-47.1272f,  -7.3368f,29.8202f,-45.8228f,  -7.7359f,29.8427f,-45.1883f,  -6.2671f,30.3830f,-43.9422f,  -6.6267f,30.3898f,-43.3160f,  
	-5.2304f,31.1920f,-42.1290f,  -5.5691f,31.1870f,-41.4969f,  -4.2917f,32.2349f,-40.3690f,  -4.6257f,32.2187f,-39.7173f,  -3.5102f,33.4893f,-38.6481f,  
	-3.8544f,33.4595f,-37.9625f,  -2.9391f,34.9233f,-36.9519f,  -3.3074f,34.8726f,-36.2220f,  -2.6242f,36.4947f,-35.2711f,  -3.0259f,36.4154f,-34.4886f,  
	-2.5999f,38.1565f,-33.6002f,  -3.0415f,38.0396f,-32.7602f,  -2.8903f,39.8574f,-31.9386f,  -3.3763f,39.6943f,-31.0376f,  -3.5097f,41.5448f,-30.2892f,  
	-4.0410f,41.3223f,-29.3303f,  -4.4599f,43.1613f,-28.6633f,  -5.0360f,42.8732f,-27.6480f,  -5.7340f,44.6573f,-27.0720f,  -6.3514f,44.2947f,-26.0064f,  
	-7.3141f,45.9830f,-25.5321f,  -7.9665f,45.5398f,-24.4250f,  -9.1722f,47.0948f,-24.0639f,  -9.8524f,46.5667f,-22.9240f,  -11.2730f,47.9553f,-22.6874f,  
	-11.9734f,47.3405f,-21.5240f,  -13.5753f,48.5346f,-21.4227f,  -14.2853f,47.8336f,-20.2467f,  -16.0302f,48.8111f,-20.2900f,  -16.7410f,48.0264f,-19.1107f,  
	-18.5874f,48.7712f,-19.3060f,  -19.2896f,47.9073f,-18.1325f,  -21.1937f,48.4097f,-18.4850f,  -21.8785f,47.4729f,-17.3254f,  -23.7954f,47.7292f,-17.8372f,  
	-24.4526f,46.7280f,-16.6992f,  -26.3375f,46.7411f,-17.3691f,  -26.9614f,45.6839f,-16.2579f,  -28.7709f,45.4619f,-17.0815f,  -29.3551f,44.3585f,-16.0015f,  
	-31.0480f,43.9149f,-16.9707f,  -31.5868f,42.7762f,-15.9249f,  -33.1250f,42.1292f,-17.0282f,  -33.6164f,40.9639f,-16.0178f,  -34.9660f,40.1355f,-17.2402f,  
	-35.4074f,38.9546f,-16.2664f,  -36.5390f,37.9700f,-17.5900f,  -36.9300f,36.7820f,-16.6508f,  -37.8197f,35.6688f,-18.0548f,  -38.1616f,34.4818f,-17.1502f,  
	-38.7908f,33.2692f,-18.6116f,  -39.0854f,32.0903f,-17.7386f,  -39.4415f,30.8085f,-19.2326f,  -39.6918f,29.6438f,-18.3893f,  -39.7681f,28.3230f,-19.8899f,  
	-39.9777f,27.1766f,-19.0732f,  -39.7735f,25.8467f,-20.5537f,  -39.9466f,24.7213f,-19.7607f,  -39.4667f,23.4111f,-21.1943f,  -39.6078f,22.3103f,-20.4223f,  
	-38.8625f,21.0469f,-21.7826f,  -38.9763f,19.9710f,-21.0284f,  -37.9813f,18.7794f,-22.2903f,  -38.0724f,17.7285f,-21.5516f,  -36.8478f,16.6313f,-22.6913f,  
	-36.9202f,15.6042f,-21.9659f,  -35.4904f,14.6215f,-22.9618f,  -35.5484f,13.6168f,-22.2475f,  -33.9413f,12.7653f,-23.0806f,  -33.9884f,11.7810f,-22.3759f,  
	-32.2348f,11.0744f,-23.0300f,  -32.2743f,10.1076f,-22.3330f,  -30.4075f,9.5565f,-22.7950f,  -30.4419f,8.6045f,-22.1049f,  -28.4967f,8.2161f,-22.3651f,  
	-28.5279f,7.2763f,-21.6811f,  -26.5404f,7.0546f,-21.7336f,  -26.5706f,6.1234f,-21.0545f,  -10.7627f,32.8215f,-62.8280f,  -11.9993f,33.3790f,-61.8208f,  
	-11.4565f,32.0494f,-60.4282f,  -12.5466f,32.4722f,-59.4666f,  -11.7639f,31.3022f,-58.0451f,  -12.7116f,31.6089f,-57.1371f,  -11.7123f,30.6291f,-55.7075f,  
	-12.5252f,30.8395f,-54.8574f,  -11.3399f,30.0775f,-53.4373f,  -12.0306f,30.2118f,-52.6444f,  -10.6963f,29.6908f,-51.2478f,  -11.2795f,29.7676f,-50.5073f,  
	-9.8385f,29.5058f,-49.1446f,  -10.3311f,29.5414f,-48.4489f,  -8.8294f,29.5503f,-47.1272f,  -9.2498f,29.5584f,-46.4648f,  -7.7359f,29.8427f,-45.1883f,  
	-8.1037f,29.8330f,-44.5451f,  -6.6267f,30.3898f,-43.3160f,  -6.9610f,30.3683f,-42.6778f,  -5.5691f,31.1870f,-41.4969f,  -5.8868f,31.1561f,-40.8498f,  
	-4.6257f,32.2187f,-39.7173f,  -4.9428f,32.1778f,-39.0464f,  -3.8544f,33.4595f,-37.9625f,  -4.1864f,33.4029f,-37.2560f,  -3.3074f,34.8726f,-36.2220f,  
	-3.6654f,34.7937f,-35.4700f,  -3.0259f,36.4154f,-34.4886f,  -3.4181f,36.3060f,-33.6847f,  -3.0415f,38.0396f,-32.7602f,  -3.4741f,37.8915f,-31.8992f,  
	-3.3763f,39.6943f,-31.0376f,  -3.8527f,39.4941f,-30.1205f,  -4.0410f,41.3223f,-29.3303f,  -4.5619f,41.0636f,-28.3570f,  -5.0360f,42.8732f,-27.6480f,  
	-5.5999f,42.5462f,-26.6229f,  -6.3514f,44.2947f,-26.0064f,  -6.9536f,43.8924f,-24.9369f,  -7.9665f,45.5398f,-24.4250f,  -8.6011f,45.0566f,-23.3189f,  
	-9.8524f,46.5667f,-22.9240f,  -10.5128f,45.9996f,-21.7895f,  -11.9734f,47.3405f,-21.5240f,  -12.6498f,46.6884f,-20.3719f,  -14.2853f,47.8336f,-20.2467f,  
	-14.9693f,47.0975f,-19.0858f,  -16.7410f,48.0264f,-19.1107f,  -17.4236f,47.2096f,-17.9500f,  -19.2896f,47.9073f,-18.1325f,  -19.9619f,47.0149f,-16.9804f,  
	-21.8785f,47.4729f,-17.3254f,  -22.5300f,46.5121f,-16.1902f,  -24.4526f,46.7280f,-16.6992f,  -25.0771f,45.7065f,-15.5864f,  -26.9614f,45.6839f,-16.2579f,  
	-27.5519f,44.6107f,-15.1725f,  -29.3551f,44.3585f,-16.0015f,  -29.9052f,43.2439f,-14.9473f,  -31.5868f,42.7762f,-15.9249f,  -32.0936f,41.6289f,-14.9038f,  
	-33.6164f,40.9639f,-16.0178f,  -34.0765f,39.7948f,-15.0315f,  -35.4074f,38.9546f,-16.2664f,  -35.8193f,37.7724f,-15.3139f,  -36.9300f,36.7820f,-16.6508f,  
	-37.2942f,35.5952f,-15.7324f,  -38.1616f,34.4818f,-17.1502f,  -38.4790f,33.2979f,-16.2634f,  -39.0854f,32.0903f,-17.7386f,  -39.3583f,30.9161f,-16.8820f,  
	-39.6918f,29.6438f,-18.3893f,  -39.9233f,28.4841f,-17.5601f,  -39.9777f,27.1766f,-19.0732f,  -40.1716f,26.0348f,-18.2686f,  -39.9466f,24.7213f,-19.7607f,  
	-40.1067f,23.6018f,-18.9782f,  -39.6078f,22.3103f,-20.4223f,  -39.7385f,21.2141f,-19.6588f,  -38.9763f,19.9710f,-21.0284f,  -39.0821f,18.8989f,-20.2817f,  
	-38.0724f,17.7285f,-21.5516f,  -38.1573f,16.6799f,-20.8193f,  -36.9202f,15.6042f,-21.9659f,  -36.9885f,14.5784f,-21.2458f,  -35.5484f,13.6168f,-22.2475f,  
	-35.6037f,12.6123f,-21.5381f,  -33.9884f,11.7810f,-22.3759f,  -34.0340f,10.7951f,-21.6751f,  -32.2743f,10.1076f,-22.3330f,  -32.3130f,9.1380f,-21.6399f,  
	-30.4419f,8.6045f,-22.1049f,  -30.4758f,7.6488f,-21.4185f,  -28.5279f,7.2763f,-21.6811f,  -28.5596f,6.3312f,-21.0002f,  -26.5706f,6.1234f,-21.0545f,  
	-26.6010f,5.1864f,-20.3786f,  -11.9993f,33.3790f,-61.8208f,  -13.1754f,33.8312f,-60.7930f,  -12.5466f,32.4722f,-59.4666f,  -13.5757f,32.8001f,-58.4922f,  
	-12.7116f,31.6089f,-57.1371f,  -13.5993f,31.8320f,-56.2225f,  -12.5252f,30.8395f,-54.8574f,  -13.2821f,30.9782f,-54.0040f,  -12.0306f,30.2118f,-52.6444f,  
	-12.6696f,30.2853f,-51.8497f,  -11.2795f,29.7676f,-50.5073f,  -11.8162f,29.7936f,-49.7654f,  -10.3311f,29.5414f,-48.4489f,  -10.7830f,29.5349f,-47.7497f,  
	-9.2498f,29.5584f,-46.4648f,  -9.6360f,29.5311f,-45.7953f,  -8.1037f,29.8330f,-44.5451f,  -8.4435f,29.7929f,-43.8916f,  -6.9610f,30.3683f,-42.6778f,  
	-7.2719f,30.3198f,-42.0263f,  -5.8868f,31.1561f,-40.8498f,  -6.1854f,31.1002f,-40.1851f,  -4.9428f,32.1778f,-39.0464f,  -5.2463f,32.1106f,-38.3555f,  
	-4.1864f,33.4029f,-37.2560f,  -4.5074f,33.3191f,-36.5277f,  -3.6654f,34.7937f,-35.4700f,  -4.0136f,34.6861f,-34.6964f,  -3.4181f,36.3060f,-33.6847f,  
	-3.8013f,36.1670f,-32.8590f,  -3.4741f,37.8915f,-31.8992f,  -3.8984f,37.7079f,-31.0202f,  -3.8527f,39.4941f,-30.1205f,  -4.3200f,39.2594f,-29.1867f,  
	-4.5619f,41.0636f,-28.3570f,  -5.0726f,40.7674f,-27.3711f,  -5.5999f,42.5462f,-26.6229f,  -6.1509f,42.1807f,-25.5908f,  -6.9536f,43.8924f,-24.9369f,  
	-7.5405f,43.4508f,-23.8652f,  -8.6011f,45.0566f,-23.3189f,  -9.2184f,44.5348f,-22.2148f,  -10.5128f,45.9996f,-21.7895f,  -11.1517f,45.3949f,-20.6633f,  
	-12.6498f,46.6884f,-20.3719f,  -13.3027f,46.0006f,-19.2316f,  -14.9693f,47.0975f,-19.0858f,  -15.6275f,46.3285f,-17.9405f,  -17.4236f,47.2096f,-17.9500f,  
	-18.0784f,46.3631f,-16.8082f,  -19.9619f,47.0149f,-16.9804f,  -20.6027f,46.0969f,-15.8507f,  -22.5300f,46.5121f,-16.1902f,  -23.1503f,45.5293f,-15.0783f,  
	-25.0771f,45.7065f,-15.5864f,  -25.6694f,44.6671f,-14.4980f,  -27.5519f,44.6107f,-15.1725f,  -28.1091f,43.5244f,-14.1122f,  -29.9052f,43.2439f,-14.9473f,  
	-30.4237f,42.1191f,-13.9173f,  -32.0936f,41.6289f,-14.9038f,  -32.5690f,40.4762f,-13.9067f,  -34.0765f,39.7948f,-15.0315f,  -34.5064f,38.6230f,-14.0669f,  
	-35.8193f,37.7724f,-15.3139f,  -36.2035f,36.5903f,-14.3825f,  -37.2942f,35.5952f,-15.7324f,  -37.6328f,34.4106f,-14.8323f,  -38.4790f,33.2979f,-16.2634f,  
	-38.7735f,32.1182f,-15.3937f,  -39.3583f,30.9161f,-16.8820f,  -39.6112f,29.7467f,-16.0401f,  -39.9233f,28.4841f,-17.5601f,  -40.1377f,27.3290f,-16.7435f,  
	-40.1716f,26.0348f,-18.2686f,  -40.3508f,24.8992f,-17.4752f,  -40.1067f,23.6018f,-18.9782f,  -40.2549f,22.4874f,-18.2048f,  -39.7385f,21.2141f,-19.6588f,  
	-39.8598f,20.1225f,-18.9032f,  -39.0821f,18.8989f,-20.2817f,  -39.1804f,17.8300f,-19.5416f,  -38.1573f,16.6799f,-20.8193f,  -38.2369f,15.6337f,-20.0925f,  
	-36.9885f,14.5784f,-21.2458f,  -37.0530f,13.5540f,-20.5307f,  -35.6037f,12.6123f,-21.5381f,  -35.6565f,11.6072f,-20.8325f,  -34.0340f,10.7951f,-21.6751f,  
	-34.0782f,9.8076f,-20.9780f,  -32.3130f,9.1380f,-21.6399f,  -32.3508f,8.1659f,-20.9503f,  -30.4758f,7.6488f,-21.4185f,  -30.5099f,6.6890f,-20.7350f,  
	-28.5596f,6.3312f,-21.0002f,  -28.5912f,5.3812f,-20.3223f,  -26.6010f,5.1864f,-20.3786f,  -26.6312f,4.2438f,-19.7058f,  -13.1754f,33.8312f,-60.7930f,  
	-14.2900f,34.1823f,-59.7497f,  -13.5757f,32.8001f,-58.4922f,  -14.5439f,33.0379f,-57.5092f,  -13.5993f,31.8320f,-56.2225f,  -14.4293f,31.9769f,-55.3034f,  
	-13.2821f,30.9782f,-54.0040f,  -13.9848f,31.0499f,-53.1487f,  -12.6696f,30.2853f,-51.8497f,  -13.2593f,30.3025f,-51.0540f,  -11.8162f,29.7936f,-49.7654f,  
	-12.3092f,29.7727f,-49.0211f,  -10.7830f,29.5349f,-47.7497f,  -11.1974f,29.4893f,-47.0452f,  -9.6360f,29.5311f,-45.7953f,  -9.9913f,29.4707f,-45.1174f,  
	-8.4435f,29.7929f,-43.8916f,  -8.7573f,29.7239f,-43.2267f,  -7.2719f,30.3198f,-42.0263f,  -7.5612f,30.2451f,-41.3591f,  -6.1854f,31.1002f,-40.1851f,  
	-6.4688f,31.0179f,-39.5016f,  -5.2463f,32.1106f,-38.3555f,  -5.5376f,32.0168f,-37.6436f,  -4.5074f,33.3191f,-36.5277f,  -4.8179f,33.2078f,-35.7781f,  
	-4.0136f,34.6861f,-34.6964f,  -4.3524f,34.5505f,-33.9008f,  -3.8013f,36.1670f,-32.8590f,  -4.1771f,35.9943f,-32.0141f,  -3.8984f,37.7079f,-31.0202f,  
	-4.3142f,37.4915f,-30.1231f,  -4.3200f,39.2594f,-29.1867f,  -4.7783f,38.9887f,-28.2382f,  -5.0726f,40.7674f,-27.3711f,  -5.5720f,40.4339f,-26.3757f,  
	-6.1509f,42.1807f,-25.5908f,  -6.6887f,41.7767f,-24.5537f,  -7.5405f,43.4508f,-23.8652f,  -8.1123f,42.9711f,-22.7925f,  -9.2184f,44.5348f,-22.2148f,  
	-9.8167f,43.9754f,-21.1159f,  -11.1517f,45.3949f,-20.6633f,  -11.7697f,44.7543f,-19.5458f,  -13.3027f,46.0006f,-19.2316f,  -13.9322f,45.2791f,-18.1043f,  
	-15.6275f,46.3285f,-17.9405f,  -16.2602f,45.5288f,-16.8116f,  -18.0784f,46.3631f,-16.8082f,  -18.7036f,45.4895f,-15.6866f,  -20.6027f,46.0969f,-15.8507f,  
	-21.2139f,45.1552f,-14.7425f,  -23.1503f,45.5293f,-15.0783f,  -23.7397f,44.5268f,-13.9895f,  -25.6694f,44.6671f,-14.4980f,  -26.2292f,43.6127f,-13.4337f,  
	-28.1091f,43.5244f,-14.1122f,  -28.6352f,42.4261f,-13.0753f,  -30.4237f,42.1191f,-13.9173f,  -30.9108f,40.9871f,-12.9110f,  -32.5690f,40.4762f,-13.9067f,  
	-33.0139f,39.3193f,-12.9313f,  -34.5064f,38.6230f,-14.0669f,  -34.9081f,37.4500f,-13.1235f,  -36.2035f,36.5903f,-14.3825f,  -36.5613f,35.4094f,-13.4699f,  
	-37.6328f,34.4106f,-14.8323f,  -37.9474f,33.2296f,-13.9499f,  -38.7735f,32.1182f,-15.3937f,  -39.0466f,30.9429f,-14.5392f,  -39.6112f,29.7467f,-16.0401f,  
	-39.8455f,28.5818f,-15.2115f,  -40.1377f,27.3290f,-16.7435f,  -40.3358f,26.1804f,-15.9388f,  -40.3508f,24.8992f,-17.4752f,  -40.5165f,23.7693f,-16.6914f,  
	-40.2549f,22.4874f,-18.2048f,  -40.3920f,21.3784f,-17.4398f,  -39.8598f,20.1225f,-18.9032f,  -39.9722f,19.0350f,-18.1548f,  -39.1804f,17.8300f,-19.5416f,  
	-39.2722f,16.7646f,-18.8075f,  -38.2369f,15.6337f,-20.0925f,  -38.3115f,14.5900f,-19.3710f,  -37.0530f,13.5540f,-20.5307f,  -37.1141f,12.5301f,-19.8196f,  
	-35.6565f,11.6072f,-20.8325f,  -35.7071f,10.6017f,-20.1307f,  -34.0782f,9.8076f,-20.9780f,  -34.1208f,8.8185f,-20.2844f,  -32.3508f,8.1659f,-20.9503f,  
	-32.3882f,7.1907f,-20.2634f,  -30.5099f,6.6890f,-20.7350f,  -30.5436f,5.7252f,-20.0543f,  -28.5912f,5.3812f,-20.3223f,  -28.6225f,4.4264f,-19.6470f,  
	-26.6312f,4.2438f,-19.7058f,  -26.6612f,3.2955f,-19.0354f,  -14.2900f,34.1823f,-59.7497f,  -15.3427f,34.4366f,-58.6955f,  -14.5439f,33.0379f,-57.5092f,  
	-15.4526f,33.1909f,-56.5204f,  -14.4293f,31.9769f,-55.3034f,  -15.2029f,32.0485f,-54.3821f,  -13.9848f,31.0499f,-53.1487f,  -14.6357f,31.0596f,-52.2927f,  
	-13.2593f,30.3025f,-51.0540f,  -13.8022f,30.2676f,-50.2572f,  -12.3092f,29.7727f,-49.0211f,  -12.7614f,29.7082f,-48.2734f,  -11.1974f,29.4893f,-47.0452f,  
	-11.5779f,29.4071f,-46.3343f,  -9.9913f,29.4707f,-45.1174f,  -10.3175f,29.3788f,-44.4303f,  -8.7573f,29.7239f,-43.2267f,  -9.0465f,29.6272f,-42.5482f,  
	-7.5612f,30.2451f,-41.3591f,  -7.8331f,30.1435f,-40.6748f,  -6.4688f,31.0179f,-39.5016f,  -6.7382f,30.9092f,-38.7985f,  -5.5376f,32.0168f,-37.6436f,  
	-5.8171f,31.8963f,-36.9111f,  -4.8179f,33.2078f,-35.7781f,  -5.1180f,33.0696f,-35.0067f,  -4.3524f,34.5505f,-33.9008f,  -4.6842f,34.3825f,-33.0851f,  
	-4.1771f,35.9943f,-32.0141f,  -4.5448f,35.7902f,-31.1499f,  -4.3142f,37.4915f,-30.1231f,  -4.7222f,37.2406f,-29.2094f,  -4.7783f,38.9887f,-28.2382f,  
	-5.2270f,38.6821f,-27.2780f,  -5.5720f,40.4339f,-26.3757f,  -6.0603f,40.0631f,-25.3726f,  -6.6887f,41.7767f,-24.5537f,  -7.2135f,41.3354f,-23.5127f,  
	-8.1123f,42.9711f,-22.7925f,  -8.6675f,42.4542f,-21.7221f,  -9.8167f,43.9754f,-21.1159f,  -10.3965f,43.3799f,-20.0229f,  -11.7697f,44.7543f,-19.5458f,  
	-12.3667f,44.0794f,-18.4384f,  -13.9322f,45.2791f,-18.1043f,  -14.5384f,44.5260f,-16.9907f,  -16.2602f,45.5288f,-16.8116f,  -16.8654f,44.7005f,-15.7008f,  
	-18.7036f,45.4895f,-15.6866f,  -19.3012f,44.5908f,-14.5846f,  -21.2139f,45.1552f,-14.7425f,  -21.7958f,44.1922f,-13.6557f,  -23.7397f,44.5268f,-13.9895f,  
	-24.2977f,43.5075f,-12.9236f,  -26.2292f,43.6127f,-13.4337f,  -26.7587f,42.5446f,-12.3921f,  -28.6352f,42.4261f,-13.0753f,  -29.1302f,41.3188f,-12.0616f,  
	-30.9108f,40.9871f,-12.9110f,  -31.3673f,39.8493f,-11.9264f,  -33.0139f,39.3193f,-12.9313f,  -33.4303f,38.1597f,-11.9771f,  -34.9081f,37.4500f,-13.1235f,  
	-35.2827f,36.2771f,-12.1993f,  -36.5613f,35.4094f,-13.4699f,  -36.8940f,34.2311f,-12.5753f,  -37.9474f,33.2296f,-13.9499f,  -38.2394f,32.0523f,-13.0833f,  
	-39.0466f,30.9429f,-14.5392f,  -39.2999f,29.7718f,-13.6987f,  -39.8455f,28.5818f,-15.2115f,  -40.0620f,27.4233f,-14.3953f,  -40.3358f,26.1804f,-15.9388f,  
	-40.5190f,25.0376f,-15.1443f,  -40.5165f,23.7693f,-16.6914f,  -40.6697f,22.6451f,-15.9166f,  -40.3920f,21.3784f,-17.4398f,  -40.5190f,20.2741f,-16.6825f,  
	-39.9722f,19.0350f,-18.1548f,  -40.0767f,17.9517f,-17.4126f,  -39.2722f,16.7646f,-18.8075f,  -39.3578f,15.7027f,-18.0789f,  -38.3115f,14.5900f,-19.3710f,  
	-38.3817f,13.5479f,-18.6537f,  -37.1141f,12.5301f,-19.8196f,  -37.1720f,11.5067f,-19.1123f,  -35.7071f,10.6017f,-20.1307f,  -35.7555f,9.5957f,-19.4324f,  
	-34.1208f,8.8185f,-20.2844f,  -34.1624f,7.8276f,-19.5935f,  -32.3882f,7.1907f,-20.2634f,  -32.4248f,6.2125f,-19.5792f,  -30.5436f,5.7252f,-20.0543f,  
	-30.5767f,4.7574f,-19.3759f,  -28.6225f,4.4264f,-19.6470f,  -28.6535f,3.4668f,-18.9740f,  -26.6612f,3.2955f,-19.0354f,  -26.6906f,2.3420f,-18.3674f,  
	-15.3427f,34.4366f,-58.6955f,  -16.3346f,34.5994f,-57.6341f,  -15.4526f,33.1909f,-56.5204f,  -16.3028f,33.2640f,-55.5288f,  -15.2029f,32.0485f,-54.3821f,  
	-15.9219f,32.0517f,-53.4604f,  -14.6357f,31.0596f,-52.2927f,  -15.2367f,31.0113f,-51.4368f,  -13.8022f,30.2676f,-50.2572f,  -14.3009f,30.1842f,-49.4588f,  
	-12.7614f,29.7082f,-48.2734f,  -13.1761f,29.6031f,-47.5215f,  -11.5779f,29.4071f,-46.3343f,  -11.9259f,29.2903f,-45.6166f,  -10.3175f,29.3788f,-44.4303f,  
	-10.6160f,29.2570f,-43.7322f,  -9.0465f,29.6272f,-42.5482f,  -9.3158f,29.5024f,-41.8549f,  -7.8331f,30.1435f,-40.6748f,  -8.0888f,30.0152f,-39.9727f,  
	-6.7382f,30.9092f,-38.7985f,  -6.9942f,30.7741f,-38.0760f,  -5.8171f,31.8963f,-36.9111f,  -6.0849f,31.7496f,-36.1577f,  -5.1180f,33.0696f,-35.0067f,  
	-5.4110f,32.9002f,-34.2151f,  -4.6842f,34.3825f,-33.0851f,  -5.0077f,34.1845f,-32.2498f,  -4.5448f,35.7902f,-31.1499f,  -4.9051f,35.5531f,-30.2680f,  
	-4.7222f,37.2406f,-29.2094f,  -5.1216f,36.9552f,-28.2823f,  -5.2270f,38.6821f,-27.2780f,  -5.6660f,38.3392f,-26.3079f,  -6.0603f,40.0631f,-25.3726f,  
	-6.5373f,39.6557f,-24.3632f,  -7.2135f,41.3354f,-23.5127f,  -7.7240f,40.8572f,-22.4714f,  -8.6675f,42.4542f,-21.7221f,  -9.2066f,41.9011f,-20.6548f,  
	-10.3965f,43.3799f,-20.0229f,  -10.9576f,42.7498f,-18.9373f,  -12.3667f,44.0794f,-18.4384f,  -12.9425f,43.3721f,-17.3423f,  -14.5384f,44.5260f,-16.9907f,  
	-15.1193f,43.7432f,-15.8929f,  -16.8654f,44.7005f,-15.7008f,  -17.4448f,43.8456f,-14.6074f,  -19.3012f,44.5908f,-14.5846f,  -19.8710f,43.6691f,-13.5023f,  
	-21.7958f,44.1922f,-13.6557f,  -22.3476f,43.2105f,-12.5905f,  -24.2977f,43.5075f,-12.9236f,  -24.8263f,42.4727f,-11.8794f,  -26.7587f,42.5446f,-12.3921f,  
	-27.2576f,41.4656f,-11.3729f,  -29.1302f,41.3188f,-12.0616f,  -29.5947f,40.2041f,-11.0694f,  -31.3673f,39.8493f,-11.9264f,  -31.7952f,38.7073f,-10.9628f,  
	-33.4303f,38.1597f,-11.9771f,  -33.8190f,36.9989f,-11.0423f,  -35.2827f,36.2771f,-12.1993f,  -35.6315f,35.1056f,-11.2934f,  -36.8940f,34.2311f,-12.5753f,  
	-37.2032f,33.0557f,-11.6971f,  -38.2394f,32.0523f,-13.0833f,  -38.5105f,30.8785f,-12.2311f,  -39.2999f,29.7718f,-13.6987f,  -39.5341f,28.6068f,-12.8713f,  
	-40.0620f,27.4233f,-14.3953f,  -40.2622f,26.2708f,-13.5901f,  -40.5190f,25.0376f,-15.1443f,  -40.6882f,23.9010f,-14.3595f,  -40.6697f,22.6451f,-15.9166f,  
	-40.8115f,21.5262f,-15.1499f,  -40.5190f,20.2741f,-16.6825f,  -40.6367f,19.1748f,-15.9319f,  -40.0767f,17.9517f,-17.4126f,  -40.1738f,16.8729f,-16.6765f,  
	-39.3578f,15.7027f,-18.0789f,  -39.4378f,14.6432f,-17.3548f,  -38.3817f,13.5479f,-18.6537f,  -38.4478f,12.5073f,-17.9404f,  -37.1720f,11.5067f,-19.1123f,  
	-37.2269f,10.4840f,-18.4087f,  -35.7555f,9.5957f,-19.4324f,  -35.8021f,8.5889f,-18.7368f,  -34.1624f,7.8276f,-19.5935f,  -34.2026f,6.8347f,-18.9051f,  
	-32.4248f,6.2125f,-19.5792f,  -32.4605f,5.2311f,-18.8971f,  -30.5767f,4.7574f,-19.3759f,  -30.6093f,3.7858f,-18.6997f,  -28.6535f,3.4668f,-18.9740f,  
	-28.6837f,2.5026f,-18.3030f,  -26.6906f,2.3420f,-18.3674f,  -26.7194f,1.3829f,-17.7008f,  -16.3346f,34.5994f,-57.6341f,  -17.2658f,34.6754f,-56.5693f,  
	-16.3028f,33.2640f,-55.5288f,  -17.0959f,33.2620f,-54.5370f,  -15.9219f,32.0517f,-53.4604f,  -16.5879f,31.9909f,-52.5398f,  -15.2367f,31.0113f,-51.4368f,  
	-15.7898f,30.9090f,-50.5813f,  -14.3009f,30.1842f,-49.4588f,  -14.7587f,30.0556f,-48.6585f,  -13.1761f,29.6031f,-47.5215f,  -13.5550f,29.4599f,-46.7653f,  
	-11.9259f,29.2903f,-45.6166f,  -12.2428f,29.1409f,-44.8908f,  -10.6160f,29.2570f,-43.7322f,  -10.8915f,29.1055f,-43.0218f,  -9.3158f,29.5024f,-41.8549f,  
	-9.5662f,29.3501f,-41.1461f,  -8.0888f,30.0152f,-39.9727f,  -8.3290f,29.8603f,-39.2531f,  -6.9942f,30.7741f,-38.0760f,  -7.2366f,30.6131f,-37.3342f,  
	-6.0849f,31.7496f,-36.1577f,  -6.3450f,31.5727f,-35.3845f,  -5.4110f,32.9002f,-34.2151f,  -5.6951f,32.7020f,-33.4038f,  -5.0077f,34.1845f,-32.2498f,  
	-5.3241f,33.9548f,-31.3960f,  -4.9051f,35.5531f,-30.2680f,  -5.2577f,35.2826f,-29.3715f,  -5.1216f,36.9552f,-28.2823f,  -5.5126f,36.6345f,-27.3436f,  
	-5.6660f,38.3392f,-26.3079f,  -6.0954f,37.9606f,-25.3293f,  -6.5373f,39.6557f,-24.3632f,  -7.0021f,39.2119f,-23.3511f,  -7.7240f,40.8572f,-22.4714f,  
	-8.2204f,40.3430f,-21.4306f,  -9.2066f,41.9011f,-20.6548f,  -9.7292f,41.3131f,-19.5923f,  -10.9576f,42.7498f,-18.9373f,  -11.4997f,42.0867f,-17.8604f,  
	-12.9425f,43.3721f,-17.3423f,  -13.4953f,42.6340f,-16.2597f,  -15.1193f,43.7432f,-15.8929f,  -15.6764f,42.9325f,-14.8104f,  -17.4448f,43.8456f,-14.6074f,  
	-17.9982f,42.9663f,-13.5319f,  -19.8710f,43.6691f,-13.5023f,  -20.4122f,42.7268f,-12.4402f,  -22.3476f,43.2105f,-12.5905f,  -22.8711f,42.2116f,-11.5460f,  
	-24.8263f,42.4727f,-11.8794f,  -25.3253f,41.4250f,-10.8568f,  -27.2576f,41.4656f,-11.3729f,  -27.7266f,40.3774f,-10.3750f,  -29.5947f,40.2041f,-11.0694f,  
	-30.0308f,39.0833f,-10.0980f,  -31.7952f,38.7073f,-10.9628f,  -32.1951f,37.5625f,-10.0188f,  -33.8190f,36.9989f,-11.0423f,  -34.1813f,35.8383f,-10.1262f,  
	-35.6315f,35.1056f,-11.2934f,  -35.9559f,33.9361f,-10.4043f,  -37.2032f,33.0557f,-11.6971f,  -37.4905f,31.8832f,-10.8338f,  -38.5105f,30.8785f,-12.2311f,  
	-38.7612f,29.7106f,-11.3927f,  -39.5341f,28.6068f,-12.8713f,  -39.7507f,27.4477f,-12.0554f,  -40.2622f,26.2708f,-13.5901f,  -40.4471f,25.1245f,-12.7952f,  
	-40.6882f,23.9010f,-14.3595f,  -40.8446f,22.7699f,-13.5833f,  -40.8115f,21.5262f,-15.1499f,  -40.9425f,20.4129f,-14.3905f,  -40.6367f,19.1748f,-15.9319f,  
	-40.7457f,18.0806f,-15.1877f,  -40.1738f,16.8729f,-16.6765f,  -40.2642f,15.7973f,-15.9451f,  -39.4378f,14.6432f,-17.3548f,  -39.5126f,13.5862f,-16.6349f,  
	-38.4478f,12.5073f,-17.9404f,  -38.5098f,11.4683f,-17.2309f,  -37.2269f,10.4840f,-18.4087f,  -37.2791f,9.4616f,-17.7078f,  -35.8021f,8.5889f,-18.7368f,  
	-35.8466f,7.5812f,-18.0437f,  -34.2026f,6.8347f,-18.9051f,  -34.2415f,5.8395f,-18.2188f,  -32.4605f,5.2311f,-18.8971f,  -32.4951f,4.2469f,-18.2170f,  
	-30.6093f,3.7858f,-18.6997f,  -30.6408f,2.8101f,-18.0251f,  -28.6837f,2.5026f,-18.3030f,  -28.7131f,1.5334f,-17.6333f,  -26.7194f,1.3829f,-17.7008f,  
	-26.7472f,0.4185f,-17.0357f,  -17.2658f,34.6754f,-56.5693f,  -18.1374f,34.6698f,-55.5042f,  -17.0959f,33.2620f,-54.5370f,  -17.8328f,33.1897f,-53.5474f,  
	-16.5879f,31.9909f,-52.5398f,  -17.2026f,31.8703f,-51.6215f,  -15.7898f,30.9090f,-50.5813f,  -16.2984f,30.7567f,-49.7261f,  -14.7587f,30.0556f,-48.6585f,  
	-15.1768f,29.8848f,-47.8567f,  -13.5550f,29.4599f,-46.7653f,  -13.8990f,29.2808f,-46.0042f,  -12.2428f,29.1409f,-44.8908f,  -12.5333f,28.9597f,-44.1556f,  
	-10.8915f,29.1055f,-43.0218f,  -11.1453f,28.9252f,-42.2986f,  -9.5662f,29.3501f,-41.1461f,  -9.7987f,29.1706f,-40.4220f,  -8.3290f,29.8603f,-39.2531f,  
	-8.5534f,29.6796f,-38.5160f,  -7.2366f,30.6131f,-37.3342f,  -7.4703f,30.4228f,-36.5734f,  -6.3450f,31.5727f,-35.3845f,  -6.5952f,31.3680f,-34.5923f,  
	-5.6951f,32.7020f,-33.4038f,  -5.9718f,32.4734f,-32.5740f,  -5.3241f,33.9548f,-31.3960f,  -5.6330f,33.6931f,-30.5268f,  -5.2577f,35.2826f,-29.3715f,  
	-5.6028f,34.9782f,-28.4619f,  -5.5126f,36.6345f,-27.3436f,  -5.8953f,36.2791f,-26.3946f,  -6.0954f,37.9606f,-25.3293f,  -6.5143f,37.5463f,-24.3460f,  
	-7.0021f,39.2119f,-23.3511f,  -7.4547f,38.7324f,-22.3372f,  -8.2204f,40.3430f,-21.4306f,  -8.7024f,39.7940f,-20.3923f,  -9.7292f,41.3131f,-19.5923f,  
	-10.2349f,40.6919f,-18.5362f,  -11.4997f,42.0867f,-17.8604f,  -12.0212f,41.3919f,-16.7948f,  -13.4953f,42.6340f,-16.2597f,  -14.0264f,41.8669f,-15.1903f,  
	-15.6764f,42.9325f,-14.8104f,  -16.2095f,42.0959f,-13.7441f,  -17.9982f,42.9663f,-13.5319f,  -18.5248f,42.0647f,-12.4753f,  -20.4122f,42.7268f,-12.4402f,  
	-20.9266f,41.7656f,-11.3976f,  -22.8711f,42.2116f,-11.5460f,  -23.3661f,41.1978f,-10.5221f,  -25.3253f,41.4250f,-10.8568f,  -25.7948f,40.3664f,-9.8551f,  
	-27.7266f,40.3774f,-10.3750f,  -28.1674f,39.2814f,-9.3976f,  -30.0308f,39.0833f,-10.0980f,  -30.4389f,37.9583f,-9.1463f,  -32.1951f,37.5625f,-10.0188f,  
	-32.5684f,36.4165f,-9.0936f,  -34.1813f,35.8383f,-10.1262f,  -34.5187f,34.6784f,-9.2272f,  -35.9559f,33.9361f,-10.4043f,  -36.2576f,32.7686f,-9.5306f,  
	-37.4905f,31.8832f,-10.8338f,  -37.7564f,30.7159f,-9.9848f,  -38.7612f,29.7106f,-11.3927f,  -38.9930f,28.5482f,-10.5664f,  -39.7507f,27.4477f,-12.0554f,  
	-39.9506f,26.2947f,-11.2504f,  -40.4471f,25.1245f,-12.7952f,  -40.6179f,23.9838f,-12.0094f,  -40.8446f,22.7699f,-13.5833f,  -40.9890f,21.6448f,-12.8150f,  
	-40.9425f,20.4129f,-14.3905f,  -41.0636f,19.3052f,-13.6381f,  -40.7457f,18.0806f,-15.1877f,  -40.8467f,16.9904f,-14.4487f,  -40.2642f,15.7973f,-15.9451f,  
	-40.3482f,14.7250f,-15.2182f,  -39.5126f,13.5862f,-16.6349f,  -39.5824f,12.5317f,-15.9191f,  -38.5098f,11.4683f,-17.2309f,  -38.5684f,10.4307f,-16.5243f,  
	-37.2791f,9.4616f,-17.7078f,  -37.3287f,8.4392f,-17.0095f,  -35.8466f,7.5812f,-18.0437f,  -35.8892f,6.5720f,-17.3525f,  -34.2415f,5.8395f,-18.2188f,  
	-34.2788f,4.8423f,-17.5344f,  -32.4951f,4.2469f,-18.2170f,  -32.5285f,3.2594f,-17.5385f,  -30.6408f,2.8101f,-18.0251f,  -30.6714f,1.8303f,-17.3517f,  
	-28.7131f,1.5334f,-17.6333f,  -28.7414f,0.5596f,-16.9647f,  -26.7472f,0.4185f,-17.0357f,  -26.7735f,-0.5511f,-16.3714f,  -18.1374f,34.6698f,-55.5042f,  
	-18.9500f,34.5872f,-54.4422f,  -17.8328f,33.1897f,-53.5474f,  -18.5150f,33.0517f,-52.5617f,  -17.2026f,31.8703f,-51.6215f,  -17.7691f,31.6944f,-50.7059f,  
	-16.2984f,30.7567f,-49.7261f,  -16.7637f,30.5577f,-48.8722f,  -15.1768f,29.8848f,-47.8567f,  -15.5563f,29.6744f,-47.0532f,  -13.8990f,29.2808f,-46.0042f,  
	-14.2131f,29.0673f,-45.2367f,  -12.5333f,28.9597f,-44.1556f,  -12.7988f,28.7479f,-43.4105f,  -11.1453f,28.9252f,-42.2986f,  -11.3781f,28.7168f,-41.5629f,  
	-9.7987f,29.1706f,-40.4220f,  -10.0127f,28.9651f,-39.6830f,  -8.5534f,29.6796f,-38.5160f,  -8.7675f,29.4700f,-37.7615f,  -7.4703f,30.4228f,-36.5734f,  
	-7.6926f,30.2054f,-35.7948f,  -6.5952f,31.3680f,-34.5923f,  -6.8373f,31.1339f,-33.7819f,  -5.9718f,32.4734f,-32.5740f,  -6.2410f,32.2139f,-31.7286f,  
	-5.6330f,33.6931f,-30.5268f,  -5.9350f,33.3986f,-29.6437f,  -5.6028f,34.9782f,-28.4619f,  -5.9404f,34.6401f,-27.5408f,  -5.8953f,36.2791f,-26.3946f,  
	-6.2688f,35.8889f,-25.4392f,  -6.5143f,37.5463f,-24.3460f,  -6.9227f,37.0968f,-23.3588f,  -7.4547f,38.7324f,-22.3372f,  -7.8948f,38.2183f,-21.3235f,  
	-8.7024f,39.7940f,-20.3923f,  -9.1695f,39.2115f,-19.3580f,  -10.2349f,40.6919f,-18.5362f,  -10.7222f,40.0382f,-17.4892f,  -12.0212f,41.3919f,-16.7948f,  
	-12.5230f,40.6673f,-15.7404f,  -14.0264f,41.8669f,-15.1903f,  -14.5354f,41.0728f,-14.1352f,  -16.2095f,42.0959f,-13.7441f,  -16.7175f,41.2354f,-12.6950f,  
	-18.5248f,42.0647f,-12.4753f,  -19.0259f,41.1425f,-11.4367f,  -20.9266f,41.7656f,-11.3976f,  -21.4136f,40.7877f,-10.3746f,  -23.3661f,41.1978f,-10.5221f,  
	-23.8325f,40.1712f,-9.5186f,  -25.7948f,40.3664f,-9.8551f,  -26.2369f,39.2981f,-8.8734f,  -28.1674f,39.2814f,-9.3976f,  -28.5805f,38.1794f,-8.4396f,  
	-30.4389f,37.9583f,-9.1463f,  -30.8202f,36.8304f,-8.2134f,  -32.5684f,36.4165f,-9.0936f,  -32.9163f,35.2699f,-8.1858f,  -34.5187f,34.6784f,-9.2272f,  
	-34.8328f,33.5194f,-8.3440f,  -36.2576f,32.7686f,-9.5306f,  -36.5370f,31.6053f,-8.6718f,  -37.7564f,30.7159f,-9.9848f,  -38.0023f,29.5534f,-9.1487f,  
	-38.9930f,28.5482f,-10.5664f,  -39.2071f,27.3916f,-9.7517f,  -39.9506f,26.2947f,-11.2504f,  -40.1352f,25.1473f,-10.4553f,  -40.6179f,23.9838f,-12.0094f,  
	-40.7754f,22.8494f,-11.2322f,  -40.9890f,21.6448f,-12.8150f,  -41.1222f,20.5257f,-12.0541f,  -41.0636f,19.3052f,-13.6381f,  -41.1755f,18.2021f,-12.8913f,  
	-40.8467f,16.9904f,-14.4487f,  -40.9403f,15.9043f,-13.7146f,  -40.3482f,14.7250f,-15.2182f,  -40.4262f,13.6561f,-14.4957f,  -39.5824f,12.5317f,-15.9191f,  
	-39.6478f,11.4797f,-15.2064f,  -38.5684f,10.4307f,-16.5243f,  -38.6235f,9.3940f,-15.8204f,  -37.3287f,8.4392f,-17.0095f,  -37.3756f,7.4161f,-16.3132f,  
	-35.8892f,6.5720f,-17.3525f,  -35.9298f,5.5618f,-16.6634f,  -34.2788f,4.8423f,-17.5344f,  -34.3145f,3.8426f,-16.8514f,  -32.5285f,3.2594f,-17.5385f,  
	-32.5606f,2.2684f,-16.8609f,  -30.6714f,1.8303f,-17.3517f,  -30.7006f,0.8465f,-16.6793f,  -28.7414f,0.5596f,-16.9647f,  -28.7683f,-0.4190f,-16.2968f,  
	-26.7735f,-0.5511f,-16.3714f,  -26.7982f,-1.5258f,-15.7077f,  -18.9500f,34.5872f,-54.4422f,  -19.7047f,34.4328f,-53.3859f,  -18.5150f,33.0517f,-52.5617f,  
	-19.1454f,32.8528f,-51.5812f,  -17.7691f,31.6944f,-50.7059f,  -18.2884f,31.4670f,-49.7944f,  -16.7637f,30.5577f,-48.8722f,  -17.1864f,30.3149f,-48.0200f,  
	-15.5563f,29.6744f,-47.0532f,  -15.9021f,29.4266f,-46.2467f,  -14.2131f,29.0673f,-45.2367f,  -14.4986f,28.8211f,-44.4629f,  -12.7988f,28.7479f,-43.4105f,  
	-13.0403f,28.5067f,-42.6562f,  -11.3781f,28.7168f,-41.5629f,  -11.5897f,28.4816f,-40.8151f,  -10.0127f,28.9651f,-39.6830f,  -10.2146f,28.7308f,-38.9284f,  
	-8.7675f,29.4700f,-37.7615f,  -8.9685f,29.2340f,-36.9909f,  -7.6926f,30.2054f,-35.7948f,  -7.9056f,29.9595f,-34.9990f,  -6.8373f,31.1339f,-33.7819f,  
	-7.0714f,30.8702f,-32.9562f,  -6.2410f,32.2139f,-31.7286f,  -6.5034f,31.9228f,-30.8689f,  -5.9350f,33.3986f,-29.6437f,  -6.2299f,33.0716f,-28.7483f,  
	-5.9404f,34.6401f,-27.5408f,  -6.2700f,34.2681f,-26.6119f,  -6.2688f,35.8889f,-25.4392f,  -6.6334f,35.4642f,-24.4782f,  -6.9227f,37.0968f,-23.3588f,  
	-7.3202f,36.6131f,-22.3701f,  -7.8948f,38.2183f,-21.3235f,  -8.3219f,37.6708f,-20.3120f,  -9.1695f,39.2115f,-19.3580f,  -9.6205f,38.5961f,-18.3310f,  
	-10.7222f,40.0382f,-17.4892f,  -11.1919f,39.3540f,-16.4515f,  -12.5230f,40.6673f,-15.7404f,  -13.0047f,39.9145f,-14.6985f,  -14.5354f,41.0728f,-14.1352f,  
	-15.0214f,40.2533f,-13.0957f,  -16.7175f,41.2354f,-12.6950f,  -17.2018f,40.3530f,-11.6627f,  -19.0259f,41.1425f,-11.4367f,  -19.5012f,40.2018f,-10.4166f,  
	-21.4136f,40.7877f,-10.3746f,  -21.8732f,39.7952f,-9.3712f,  -23.8325f,40.1712f,-9.5186f,  -24.2722f,39.1331f,-8.5344f,  -26.2369f,39.2981f,-8.8734f,  
	-26.6516f,38.2220f,-7.9109f,  -28.5805f,38.1794f,-8.4396f,  -28.9668f,37.0728f,-7.5004f,  -30.8202f,36.8304f,-8.2134f,  -31.1760f,35.7004f,-7.2980f,  
	-32.9163f,35.2699f,-8.1858f,  -33.2404f,34.1230f,-7.2941f,  -34.8328f,33.5194f,-8.3440f,  -35.1238f,32.3638f,-7.4761f,  -36.5370f,31.6053f,-8.6718f,  
	-36.7955f,30.4462f,-7.8264f,  -38.0023f,29.5534f,-9.1487f,  -38.2294f,28.3964f,-8.3246f,  -39.2071f,27.3916f,-9.7517f,  -39.4047f,26.2404f,-8.9474f,  
	-40.1352f,25.1473f,-10.4553f,  -40.3052f,24.0062f,-9.6692f,  -40.7754f,22.8494f,-11.2322f,  -40.9204f,21.7213f,-10.4629f,  -41.1222f,20.5257f,-12.0541f,  
	-41.2450f,19.4118f,-11.2994f,  -41.1755f,18.2021f,-12.8913f,  -41.2788f,17.1038f,-12.1498f,  -40.9403f,15.9043f,-13.7146f,  -41.0268f,14.8222f,-12.9852f,  
	-40.4262f,13.6561f,-14.4957f,  -40.4988f,12.5905f,-13.7767f,  -39.6478f,11.4797f,-15.2064f,  -39.7088f,10.4293f,-14.4967f,  -38.6235f,9.3940f,-15.8204f,  
	-38.6752f,8.3575f,-15.1186f,  -37.3756f,7.4161f,-16.3132f,  -37.4199f,6.3931f,-15.6191f,  -35.9298f,5.5618f,-16.6634f,  -35.9683f,4.5499f,-15.9757f,  
	-34.3145f,3.8426f,-16.8514f,  -34.3486f,2.8404f,-16.1694f,  -32.5606f,2.2684f,-16.8609f,  -32.5911f,1.2743f,-16.1843f,  -30.7006f,0.8465f,-16.6793f,  
	-30.7282f,-0.1415f,-16.0073f,  -28.7683f,-0.4190f,-16.2968f,  -28.7934f,-1.4021f,-15.6293f,  -26.7982f,-1.5258f,-15.7077f,  -26.8208f,-2.5054f,-15.0442f,  
	-19.7047f,34.4328f,-53.3859f,  -20.4039f,34.2118f,-52.3369f,  -19.1454f,32.8528f,-51.5812f,  -19.7250f,32.5972f,-50.6074f,  -18.2884f,31.4670f,-49.7944f,  
	-18.7613f,31.1914f,-48.8880f,  -17.1864f,30.3149f,-48.0200f,  -17.5715f,30.0315f,-47.1684f,  -15.9021f,29.4266f,-46.2467f,  -16.2156f,29.1437f,-45.4376f,  
	-14.4986f,28.8211f,-44.4629f,  -14.7567f,28.5438f,-43.6831f,  -13.0403f,28.5067f,-42.6562f,  -13.2572f,28.2377f,-41.8930f,  -11.5897f,28.4816f,-40.8151f,  
	-11.7867f,28.2178f,-40.0542f,  -10.2146f,28.7308f,-38.9284f,  -10.4011f,28.4704f,-38.1600f,  -8.9685f,29.2340f,-36.9909f,  -9.1587f,28.9701f,-36.2045f,  
	-7.9056f,29.9595f,-34.9990f,  -8.1099f,29.6850f,-34.1887f,  -7.0714f,30.8702f,-32.9562f,  -7.2984f,30.5760f,-32.1163f,  -6.5034f,31.9228f,-30.8689f,  
	-6.7589f,31.6004f,-29.9964f,  -6.2299f,33.0716f,-28.7483f,  -6.5177f,32.7117f,-27.8441f,  -6.2700f,34.2681f,-26.6119f,  -6.5918f,33.8625f,-25.6761f,  
	-6.6334f,35.4642f,-24.4782f,  -6.9885f,35.0058f,-23.5140f,  -7.3202f,36.6131f,-22.3701f,  -7.7066f,36.0962f,-21.3816f,  -8.3219f,37.6708f,-20.3120f,  
	-8.7350f,37.0901f,-19.3057f,  -9.6205f,38.5961f,-18.3310f,  -10.0558f,37.9496f,-17.3112f,  -11.1919f,39.3540f,-16.4515f,  -11.6435f,38.6408f,-15.4244f,  
	-13.0047f,39.9145f,-14.6985f,  -13.4654f,39.1350f,-13.6706f,  -15.0214f,40.2533f,-13.0957f,  -15.4852f,39.4104f,-12.0717f,  -17.2018f,40.3530f,-11.6627f,  
	-17.6617f,39.4502f,-10.6476f,  -19.5012f,40.2018f,-10.4166f,  -19.9503f,39.2448f,-9.4155f,  -21.8732f,39.7952f,-9.3712f,  -22.3070f,38.7894f,-8.3865f,  
	-24.2722f,39.1331f,-8.5344f,  -24.6852f,38.0855f,-7.5692f,  -26.6516f,38.2220f,-7.9109f,  -27.0399f,37.1397f,-6.9671f,  -28.9668f,37.0728f,-7.5004f,  
	-29.3277f,35.9627f,-6.5789f,  -31.1760f,35.7004f,-7.2980f,  -31.5078f,34.5689f,-6.3990f,  -33.2404f,34.1230f,-7.2941f,  -33.5409f,32.9782f,-6.4182f,  
	-35.1238f,32.3638f,-7.4761f,  -35.3931f,31.2113f,-6.6222f,  -36.7955f,30.4462f,-7.8264f,  -37.0342f,29.2919f,-6.9936f,  -38.2294f,28.3964f,-8.3246f,  
	-38.4390f,27.2443f,-7.5116f,  -39.4047f,26.2404f,-8.9474f,  -39.5866f,25.0953f,-8.1528f,  -40.3052f,24.0062f,-9.6692f,  -40.4616f,22.8715f,-8.8917f,  
	-40.9204f,21.7213f,-10.4629f,  -41.0538f,20.5986f,-9.7003f,  -41.2450f,19.4118f,-11.2994f,  -41.3580f,18.3030f,-10.5505f,  -41.2788f,17.1038f,-12.1498f,  
	-41.3740f,16.0100f,-11.4134f,  -41.0268f,14.8222f,-12.9852f,  -41.1069f,13.7442f,-12.2597f,  -40.4988f,12.5905f,-13.7767f,  -40.5661f,11.5274f,-13.0609f,  
	-39.7088f,10.4293f,-14.4967f,  -39.7656f,9.3800f,-13.7891f,  -38.6752f,8.3575f,-15.1186f,  -38.7235f,7.3220f,-14.4191f,  -37.4199f,6.3931f,-15.6191f,  
	-37.4615f,5.3692f,-14.9263f,  -35.9683f,4.5499f,-15.9757f,  -36.0046f,3.5363f,-15.2889f,  -34.3486f,2.8404f,-16.1694f,  -34.3807f,1.8356f,-15.4882f,  
	-32.5911f,1.2743f,-16.1843f,  -32.6198f,0.2765f,-15.5080f,  -30.7282f,-0.1415f,-16.0073f,  -30.7539f,-1.1335f,-15.3357f,  -28.7934f,-1.4021f,-15.6293f,  
	-28.8163f,-2.3897f,-14.9617f,  -26.8208f,-2.5054f,-15.0442f,  -26.8411f,-3.4900f,-14.3803f,  -20.4039f,34.2118f,-52.3369f,  -21.0487f,33.9289f,-51.2975f,  
	-19.7250f,32.5972f,-50.6074f,  -20.2542f,32.2888f,-49.6424f,  -18.7613f,31.1914f,-48.8880f,  -19.1927f,30.8715f,-47.9860f,  -17.5715f,30.0315f,-47.1684f,  
	-17.9204f,29.7100f,-46.3180f,  -16.2156f,29.1437f,-45.4376f,  -16.4980f,28.8276f,-44.6262f,  -14.7567f,28.5438f,-43.6831f,  -14.9869f,28.2373f,-42.8982f,  
	-13.2572f,28.2377f,-41.8930f,  -13.4570f,27.9397f,-41.1196f,  -11.7867f,28.2178f,-40.0542f,  -11.9658f,27.9277f,-39.2821f,  -10.4011f,28.4704f,-38.1600f,  
	-10.5749f,28.1828f,-37.3778f,  -9.1587f,28.9701f,-36.2045f,  -9.3388f,28.6787f,-35.4049f,  -8.1099f,29.6850f,-34.1887f,  -8.3064f,29.3812f,-33.3648f,  
	-7.2984f,30.5760f,-32.1163f,  -7.5184f,30.2517f,-31.2639f,  -6.7589f,31.6004f,-29.9964f,  -7.0078f,31.2463f,-29.1147f,  -6.5177f,32.7117f,-27.8441f,  
	-6.7986f,32.3192f,-26.9321f,  -6.5918f,33.8625f,-25.6761f,  -6.9054f,33.4239f,-24.7359f,  -6.9885f,35.0058f,-23.5140f,  -7.3340f,34.5147f,-22.5487f,  
	-7.7066f,36.0962f,-21.3816f,  -8.0808f,35.5462f,-20.3967f,  -8.7350f,37.0901f,-19.3057f,  -9.1343f,36.4781f,-18.3050f,  -10.0558f,37.9496f,-17.3112f,  
	-10.4751f,37.2735f,-16.3004f,  -11.6435f,38.6408f,-15.4244f,  -12.0761f,37.8998f,-14.4097f,  -13.4654f,39.1350f,-13.6706f,  -13.9057f,38.3311f,-12.6566f,  
	-15.4852f,39.4104f,-12.0717f,  -15.9265f,38.5457f,-11.0636f,  -17.6617f,39.4502f,-10.6476f,  -18.0970f,38.5294f,-9.6506f,  -19.9503f,39.2448f,-9.4155f,  
	-20.3749f,38.2727f,-8.4322f,  -22.3070f,38.7894f,-8.3865f,  -22.7149f,37.7722f,-7.4204f,  -24.6852f,38.0855f,-7.5692f,  -25.0723f,37.0298f,-6.6224f,  
	-27.0399f,37.1397f,-6.9671f,  -27.4030f,36.0521f,-6.0409f,  -29.3277f,35.9627f,-6.5789f,  -29.6645f,34.8496f,-5.6740f,  -31.5078f,34.5689f,-6.3990f,  
	-31.8156f,33.4382f,-5.5161f,  -33.5409f,32.9782f,-6.4182f,  -33.8192f,31.8356f,-5.5567f,  -35.3931f,31.2113f,-6.6222f,  -35.6419f,30.0628f,-5.7815f,  
	-37.0342f,29.2919f,-6.9936f,  -37.2545f,28.1420f,-6.1725f,  -38.4390f,27.2443f,-7.5116f,  -38.6318f,26.0981f,-6.7089f,  -39.5866f,25.0953f,-8.1528f,  
	-39.7537f,23.9566f,-7.3673f,  -40.4616f,22.8715f,-8.8917f,  -40.6053f,21.7423f,-8.1214f,  -41.0538f,20.5986f,-9.7003f,  -41.1763f,19.4814f,-8.9441f,  
	-41.3580f,18.3030f,-10.5505f,  -41.4618f,17.1992f,-9.8071f,  -41.3740f,16.0100f,-11.4134f,  -41.4616f,14.9210f,-10.6814f,  -41.1069f,13.7442f,-12.2597f,  
	-41.1807f,12.6695f,-11.5377f,  -40.5661f,11.5274f,-13.0609f,  -40.6284f,10.4661f,-12.3475f,  -39.7656f,9.3800f,-13.7891f,  -39.8184f,8.3326f,-13.0842f,  
	-38.7235f,7.3220f,-14.4191f,  -38.7686f,6.2864f,-13.7211f,  -37.4615f,5.3692f,-14.9263f,  -37.5004f,4.3443f,-14.2345f,  -36.0046f,3.5363f,-15.2889f,  
	-36.0386f,2.5210f,-14.6030f,  -34.3807f,1.8356f,-15.4882f,  -34.4106f,0.8280f,-14.8074f,  -32.6198f,0.2765f,-15.5080f,  -32.6463f,-0.7245f,-14.8320f,  
	-30.7539f,-1.1335f,-15.3357f,  -30.7773f,-2.1296f,-14.6638f,  -28.8163f,-2.3897f,-14.9617f,  -28.8368f,-3.3821f,-14.2936f,  -26.8411f,-3.4900f,-14.3803f,  
	-26.8586f,-4.4794f,-13.7156f,  -21.0487f,33.9289f,-51.2975f,  -21.6394f,33.5884f,-50.2702f,  -20.2542f,32.2888f,-49.6424f,  -20.7378f,31.9323f,-48.6854f,  
	-19.1927f,30.8715f,-47.9860f,  -19.5838f,30.5105f,-47.0890f,  -17.9204f,29.7100f,-46.3180f,  -18.2345f,29.3531f,-45.4692f,  -16.4980f,28.8276f,-44.6262f,  
	-16.7490f,28.4805f,-43.8136f,  -14.9869f,28.2373f,-42.8982f,  -15.1968f,27.9013f,-42.1063f,  -13.4570f,27.9397f,-41.1196f,  -13.6360f,27.6152f,-40.3381f,  
	-11.9658f,27.9277f,-39.2821f,  -12.1300f,27.6109f,-38.4988f,  -10.5749f,28.1828f,-37.3778f,  -10.7370f,27.8683f,-36.5841f,  -9.3388f,28.6787f,-35.4049f,  
	-9.5101f,28.3589f,-34.5929f,  -8.3064f,29.3812f,-33.3648f,  -8.4953f,29.0485f,-32.5289f,  -7.5184f,30.2517f,-31.2639f,  -7.7318f,29.8969f,-30.4019f,  
	-7.0078f,31.2463f,-29.1147f,  -7.2502f,30.8606f,-28.2246f,  -6.7986f,32.3192f,-26.9321f,  -7.0722f,31.8945f,-26.0146f,  -6.9054f,33.4239f,-24.7359f,  
	-7.2105f,32.9532f,-23.7932f,  -7.3340f,34.5147f,-22.5487f,  -7.6691f,33.9906f,-21.5853f,  -8.0808f,35.5462f,-20.3967f,  -8.4431f,34.9647f,-19.4158f,  
	-9.1343f,36.4781f,-18.3050f,  -9.5195f,35.8360f,-17.3116f,  -10.4751f,37.2735f,-16.3004f,  -10.8774f,36.5687f,-15.3004f,  -12.0761f,37.8998f,-14.4097f,  
	-12.4903f,37.1333f,-13.4076f,  -13.9057f,38.3311f,-12.6566f,  -14.3254f,37.5037f,-11.6574f,  -15.9265f,38.5457f,-11.0636f,  -16.3447f,37.6613f,-10.0727f,  
	-18.0970f,38.5294f,-9.6506f,  -18.5089f,37.5919f,-8.6706f,  -20.3749f,38.2727f,-8.4322f,  -20.7746f,37.2874f,-7.4670f,  -22.7149f,37.7722f,-7.4204f,  
	-23.0979f,36.7452f,-6.4723f,  -25.0723f,37.0298f,-6.6224f,  -25.4347f,35.9673f,-5.6932f,  -27.4030f,36.0521f,-6.0409f,  -27.7421f,34.9602f,-5.1314f,  
	-29.6645f,34.8496f,-5.6740f,  -29.9772f,33.7358f,-4.7854f,  -31.8156f,33.4382f,-5.5161f,  -32.1008f,32.3084f,-4.6481f,  -33.8192f,31.8356f,-5.5567f,  
	-34.0764f,30.6960f,-4.7089f,  -35.6419f,30.0628f,-5.7815f,  -35.8716f,28.9181f,-4.9529f,  -37.2545f,28.1420f,-6.1725f,  -37.4572f,26.9975f,-5.3622f,  
	-38.6318f,26.0981f,-6.7089f,  -38.8089f,24.9580f,-5.9159f,  -39.7537f,23.9566f,-7.3673f,  -39.9072f,22.8233f,-6.5898f,  -40.6053f,21.7423f,-8.1214f,  
	-40.7370f,20.6188f,-7.3581f,  -41.1763f,19.4814f,-8.9441f,  -41.2887f,18.3695f,-8.1939f,  -41.4618f,17.1992f,-9.8071f,  -41.5570f,16.1009f,-9.0686f,  
	-41.4616f,14.9210f,-10.6814f,  -41.5421f,13.8360f,-9.9532f,  -41.1807f,12.6695f,-11.5377f,  -41.2486f,11.5972f,-10.8185f,  -40.6284f,10.4661f,-12.3475f,  
	-40.6858f,9.4075f,-11.6370f,  -39.8184f,8.3326f,-13.0842f,  -39.8672f,7.2858f,-12.3808f,  -38.7686f,6.2864f,-13.7211f,  -38.8104f,5.2506f,-13.0243f,  
	-37.5004f,4.3443f,-14.2345f,  -37.5366f,3.3186f,-13.5437f,  -36.0386f,2.5210f,-14.6030f,  -36.0701f,1.5035f,-13.9174f,  -34.4106f,0.8280f,-14.8074f,  
	-34.4382f,-0.1823f,-14.1268f,  -32.6463f,-0.7245f,-14.8320f,  -32.6704f,-1.7293f,-14.1556f,  -30.7773f,-2.1296f,-14.6638f,  -30.7982f,-3.1300f,-13.9913f,  
	-28.8368f,-3.3821f,-14.2936f,  -28.8545f,-4.3789f,-13.6246f,  -26.8586f,-4.4794f,-13.7156f,  -26.8729f,-5.4736f,-13.0497f,  -21.6394f,33.5884f,-50.2702f,  
	-22.1806f,33.1957f,-49.2547f,  -20.7378f,31.9323f,-48.6854f,  -21.1774f,31.5313f,-47.7375f,  -19.5838f,30.5105f,-47.0890f,  -19.9363f,30.1113f,-46.1978f,  
	-18.2345f,29.3531f,-45.4692f,  -18.5134f,28.9632f,-44.6234f,  -16.7490f,28.4805f,-43.8136f,  -16.9764f,28.1032f,-42.9976f,  -15.1968f,27.9013f,-42.1063f,  
	-15.3830f,27.5383f,-41.3098f,  -13.6360f,27.6152f,-40.3381f,  -13.7975f,27.2643f,-39.5483f,  -12.1300f,27.6109f,-38.4988f,  -12.2806f,27.2679f,-37.7061f,  
	-10.7370f,27.8683f,-36.5841f,  -10.8890f,27.5267f,-35.7796f,  -9.5101f,28.3589f,-34.5929f,  -9.6729f,28.0116f,-33.7698f,  -8.4953f,29.0485f,-32.5289f,  
	-8.6774f,28.6866f,-31.6838f,  -7.7318f,29.8969f,-30.4019f,  -7.9391f,29.5117f,-29.5316f,  -7.2502f,30.8606f,-28.2246f,  -7.4861f,30.4438f,-27.3285f,  
	-7.0722f,31.8945f,-26.0146f,  -7.3384f,31.4386f,-25.0938f,  -7.2105f,32.9532f,-23.7932f,  -7.5070f,32.4500f,-22.8512f,  -7.6691f,33.9906f,-21.5853f,  
	-7.9939f,33.4352f,-20.6245f,  -8.4431f,34.9647f,-19.4158f,  -8.7930f,34.3529f,-18.4407f,  -9.5195f,35.8360f,-17.3116f,  -9.8897f,35.1646f,-16.3276f,  
	-10.8774f,36.5687f,-15.3004f,  -11.2631f,35.8376f,-14.3117f,  -12.4903f,37.1333f,-13.4076f,  -12.8856f,36.3421f,-12.4189f,  -14.3254f,37.5037f,-11.6574f,  
	-14.7236f,36.6554f,-10.6744f,  -16.3447f,37.6613f,-10.0727f,  -16.7411f,36.7586f,-9.0978f,  -18.5089f,37.5919f,-8.6706f,  -18.8973f,36.6395f,-7.7082f,  
	-20.7746f,37.2874f,-7.4670f,  -21.1502f,36.2905f,-6.5194f,  -23.0979f,36.7452f,-6.4723f,  -23.4567f,35.7096f,-5.5416f,  -25.4347f,35.9673f,-5.6932f,  
	-25.7736f,34.8988f,-4.7806f,  -27.7421f,34.9602f,-5.1314f,  -28.0573f,33.8659f,-4.2384f,  -29.9772f,33.7358f,-4.7854f,  -30.2672f,32.6216f,-3.9120f,  
	-32.1008f,32.3084f,-4.6481f,  -32.3645f,31.1805f,-3.7942f,  -34.0764f,30.6960f,-4.7089f,  -34.3139f,29.5592f,-3.8737f,  -35.8716f,28.9181f,-4.9529f,  
	-36.0829f,27.7780f,-4.1358f,  -37.4572f,26.9975f,-5.3622f,  -37.6433f,25.8586f,-4.5623f,  -38.8089f,24.9580f,-5.9159f,  -38.9713f,23.8232f,-5.1314f,  
	-39.9072f,22.8233f,-6.5898f,  -40.0476f,21.6958f,-5.8196f,  -40.7370f,20.6188f,-7.3581f,  -40.8576f,19.5009f,-6.6013f,  -41.2887f,18.3695f,-8.1939f,  
	-41.3913f,17.2636f,-7.4491f,  -41.5570f,16.1009f,-9.0686f,  -41.6440f,15.0070f,-8.3344f,  -41.5421f,13.8360f,-9.9532f,  -41.6157f,12.7540f,-9.2282f,  
	-41.2486f,11.5972f,-10.8185f,  -41.3109f,10.5285f,-10.1025f,  -40.6858f,9.4075f,-11.6370f,  -40.7385f,8.3503f,-10.9284f,  -39.8672f,7.2858f,-12.3808f,  
	-39.9122f,6.2397f,-11.6789f,  -38.8104f,5.2506f,-13.0243f,  -38.8489f,4.2148f,-12.3286f,  -37.5366f,3.3186f,-13.5437f,  -37.5698f,2.2914f,-12.8534f,  
	-36.0701f,1.5035f,-13.9174f,  -36.0988f,0.4840f,-13.2322f,  -34.4382f,-0.1823f,-14.1268f,  -34.4630f,-1.1958f,-13.4458f,  -32.6704f,-1.7293f,-14.1556f,  
	-32.6918f,-2.7379f,-13.4785f,  -30.7982f,-3.1300f,-13.9913f,  -30.8162f,-4.1343f,-13.3179f,  -28.8545f,-4.3789f,-13.6246f,  -28.8690f,-5.3803f,-12.9544f,  
	-26.8729f,-5.4736f,-13.0497f,  -26.8837f,-6.4724f,-12.3822f,  -22.1806f,33.1957f,-49.2547f,  -22.6738f,32.7551f,-48.2523f,  -21.1774f,31.5313f,-47.7375f,  
	-21.5744f,31.0894f,-46.7995f,  -19.9363f,30.1113f,-46.1978f,  -20.2500f,29.6771f,-45.3139f,  -18.5134f,28.9632f,-44.6234f,  -18.7653f,28.5421f,-43.7781f,  
	-16.9764f,28.1032f,-42.9976f,  -17.1769f,27.6982f,-42.1807f,  -15.3830f,27.5383f,-41.3098f,  -15.5488f,27.1489f,-40.5083f,  -13.7975f,27.2643f,-39.5483f,  
	-13.9432f,26.8877f,-38.7516f,  -12.2806f,27.2679f,-37.7061f,  -12.4194f,26.8986f,-36.9046f,  -10.8890f,27.5267f,-35.7796f,  -11.0311f,27.1585f,-34.9655f,  
	-9.6729f,28.0116f,-33.7698f,  -9.8284f,27.6362f,-32.9384f,  -8.6774f,28.6866f,-31.6838f,  -8.8532f,28.2955f,-30.8306f,  -7.9391f,29.5117f,-29.5316f,  
	-8.1400f,29.0965f,-28.6549f,  -7.4861f,30.4438f,-27.3285f,  -7.7153f,29.9967f,-26.4284f,  -7.3384f,31.4386f,-25.0938f,  -7.5972f,30.9508f,-24.1727f,  
	-7.5070f,32.4500f,-22.8512f,  -7.7946f,31.9159f,-21.9106f,  -7.9939f,33.4352f,-20.6245f,  -8.3081f,32.8496f,-19.6683f,  -8.7930f,34.3529f,-18.4407f,  
	-9.1299f,33.7115f,-17.4735f,  -9.8897f,35.1646f,-16.3276f,  -10.2452f,34.4663f,-15.3535f,  -11.2631f,35.8376f,-14.3117f,  -11.6318f,35.0808f,-13.3351f,  
	-12.8856f,36.3421f,-12.4189f,  -13.2613f,35.5287f,-11.4454f,  -14.7236f,36.6554f,-10.6744f,  -15.1016f,35.7871f,-9.7064f,  -16.7411f,36.7586f,-9.0978f,  
	-17.1153f,35.8395f,-8.1399f,  -18.8973f,36.6395f,-7.7082f,  -19.2628f,35.6737f,-6.7628f,  -21.1502f,36.2905f,-6.5194f,  -21.5026f,35.2834f,-5.5889f,  
	-23.4567f,35.7096f,-5.5416f,  -23.7925f,34.6665f,-4.6276f,  -25.7736f,34.8988f,-4.7806f,  -26.0887f,33.8263f,-3.8846f,  -28.0573f,33.8659f,-4.2384f,  
	-28.3497f,32.7699f,-3.3610f,  -30.2672f,32.6216f,-3.9120f,  -30.5354f,31.5081f,-3.0532f,  -32.3645f,31.1805f,-3.7942f,  -32.6080f,30.0545f,-2.9535f,  
	-34.3139f,29.5592f,-3.8737f,  -34.5323f,28.4264f,-3.0506f,  -36.0829f,27.7780f,-4.1358f,  -36.2768f,26.6430f,-3.3296f,  -37.6433f,25.8586f,-4.5623f,  
	-37.8138f,24.7247f,-3.7715f,  -38.9713f,23.8232f,-5.1314f,  -39.1199f,22.6940f,-4.3551f,  -40.0476f,21.6958f,-5.8196f,  -40.1759f,20.5740f,-5.0567f,  
	-40.8576f,19.5009f,-6.6013f,  -40.9674f,18.3892f,-5.8505f,  -41.3913f,17.2636f,-7.4491f,  -41.4849f,16.1625f,-6.7091f,  -41.6440f,15.0070f,-8.3344f,  
	-41.7235f,13.9167f,-7.6038f,  -41.6157f,12.7540f,-9.2282f,  -41.6829f,11.6763f,-8.5068f,  -41.3109f,10.5285f,-10.1025f,  -41.3677f,9.4617f,-9.3886f,  
	-40.7385f,8.3503f,-10.9284f,  -40.7868f,7.2946f,-10.2214f,  -39.9122f,6.2397f,-11.6789f,  -39.9532f,5.1943f,-10.9783f,  -38.8489f,4.2148f,-12.3286f,  
	-38.8841f,3.1782f,-11.6334f,  -37.5698f,2.2914f,-12.8534f,  -37.5999f,1.2629f,-12.1634f,  -36.0988f,0.4840f,-13.2322f,  -36.1246f,-0.5382f,-12.5465f,  
	-34.4630f,-1.1958f,-13.4458f,  -34.4850f,-2.2125f,-12.7641f,  -32.6918f,-2.7379f,-13.4785f,  -32.7101f,-3.7499f,-12.8007f,  -30.8162f,-4.1343f,-13.3179f,  
	-30.8309f,-5.1430f,-12.6432f,  -28.8690f,-5.3803f,-12.9544f,  -28.8799f,-6.3861f,-12.2824f,  -26.8837f,-6.4724f,-12.3822f,  -26.8905f,-7.4758f,-11.7127f,  
	-22.6738f,32.7551f,-48.2523f,  -23.1208f,32.2705f,-47.2640f,  -21.5744f,31.0894f,-46.7995f,  -21.9291f,30.6100f,-45.8731f,  -20.2500f,29.6771f,-45.3139f,  
	-20.5330f,29.2103f,-44.4345f,  -18.7653f,28.5421f,-43.7781f,  -18.9870f,28.0923f,-42.9359f,  -17.1769f,27.6982f,-42.1807f,  -17.3540f,27.2666f,-41.3624f,  
	-15.5488f,27.1489f,-40.5083f,  -15.6964f,26.7341f,-39.7029f,  -13.9432f,26.8877f,-38.7516f,  -14.0751f,26.4856f,-37.9486f,  -12.4194f,26.8986f,-36.9046f,  
	-12.5466f,26.5039f,-36.0955f,  -11.0311f,27.1585f,-34.9655f,  -11.1651f,26.7636f,-34.1442f,  -9.8284f,27.6362f,-32.9384f,  -9.9771f,27.2330f,-32.0994f,  
	-8.8532f,28.2955f,-30.8306f,  -9.0228f,27.8758f,-29.9712f,  -8.1400f,29.0965f,-28.6549f,  -8.3348f,28.6523f,-27.7740f,  -7.7153f,29.9967f,-26.4284f,  
	-7.9382f,29.5186f,-25.5272f,  -7.5972f,30.9508f,-24.1727f,  -7.8484f,30.4327f,-23.2520f,  -7.7946f,31.9159f,-21.9106f,  -8.0730f,31.3518f,-20.9734f,  
	-8.3081f,32.8496f,-19.6683f,  -8.6111f,32.2341f,-18.7187f,  -9.1299f,33.7115f,-17.4735f,  -9.4539f,33.0427f,-16.5151f,  -10.2452f,34.4663f,-15.3535f,  
	-10.5856f,33.7414f,-14.3902f,  -11.6318f,35.0808f,-13.3351f,  -11.9829f,34.3007f,-12.3726f,  -13.2613f,35.5287f,-11.4454f,  -13.6185f,34.6941f,-10.4860f,  
	-15.1016f,35.7871f,-9.7064f,  -15.4589f,34.9010f,-8.7547f,  -17.1153f,35.8395f,-8.1399f,  -17.4679f,34.9053f,-7.1984f,  -19.2628f,35.6737f,-6.7628f,  
	-19.6060f,34.6961f,-5.8342f,  -21.5026f,35.2834f,-5.5889f,  -21.8328f,34.2673f,-4.6749f,  -23.7925f,34.6665f,-4.6276f,  -24.1051f,33.6178f,-3.7301f,  
	-26.0887f,33.8263f,-3.8846f,  -26.3813f,32.7507f,-3.0044f,  -28.3497f,32.7699f,-3.3610f,  -28.6204f,31.6732f,-2.4984f,  -30.5354f,31.5081f,-3.0532f,  
	-30.7832f,30.3954f,-2.2078f,  -32.6080f,30.0545f,-2.9535f,  -32.8321f,28.9314f,-2.1253f,  -34.5323f,28.4264f,-3.0506f,  -34.7328f,27.2978f,-2.2390f,  
	-36.2768f,26.6430f,-3.3296f,  -36.4544f,25.5126f,-2.5332f,  -37.8138f,24.7247f,-3.7715f,  -37.9696f,23.5962f,-2.9894f,  -39.1199f,22.6940f,-4.3551f,  
	-39.2554f,21.5704f,-3.5863f,  -40.1759f,20.5740f,-5.0567f,  -40.2926f,19.4585f,-4.3003f,  -40.9674f,18.3892f,-5.8505f,  -41.0673f,17.2826f,-5.1050f,  
	-41.4849f,16.1625f,-6.7091f,  -41.5700f,15.0654f,-5.9731f,  -41.7235f,13.9167f,-7.6038f,  -41.7956f,12.8312f,-6.8772f,  -41.6829f,11.6763f,-8.5068f,  
	-41.7440f,10.6011f,-7.7878f,  -41.3677f,9.4617f,-9.3886f,  -41.4194f,8.3972f,-8.6767f,  -40.7868f,7.2946f,-10.2214f,  -40.8306f,6.2403f,-9.5160f,  
	-39.9532f,5.1943f,-10.9783f,  -39.9904f,4.1489f,-10.2784f,  -38.8841f,3.1782f,-11.6334f,  -38.9157f,2.1411f,-10.9389f,  -37.5999f,1.2629f,-12.1634f,  
	-37.6268f,0.2324f,-11.4732f,  -36.1246f,-0.5382f,-12.5465f,  -36.1473f,-1.5630f,-11.8603f,  -34.4850f,-2.2125f,-12.7641f,  -34.5038f,-3.2322f,-12.0818f,  
	-32.7101f,-3.7499f,-12.8007f,  -32.7250f,-4.7660f,-12.1214f,  -30.8309f,-5.1430f,-12.6432f,  -30.8419f,-6.1559f,-11.9668f,  -28.8799f,-6.3861f,-12.2824f,  
	-28.8867f,-7.3964f,-11.6084f,  -26.8905f,-7.4758f,-11.7127f,  -26.8929f,-8.4838f,-11.0406f,  -23.1208f,32.2705f,-47.2640f,  -23.5217f,31.7458f,-46.2916f,  
	-21.9291f,30.6100f,-45.8731f,  -22.2492f,30.0966f,-44.9555f,  -20.5330f,29.2103f,-44.4345f,  -20.7824f,28.7138f,-43.5623f,  -18.9870f,28.0923f,-42.9359f,  
	-19.1822f,27.6155f,-42.0961f,  -17.3540f,27.2666f,-41.3624f,  -17.5101f,26.8098f,-40.5436f,  -15.6964f,26.7341f,-39.7029f,  -15.8278f,26.2946f,-38.8939f,  
	-14.0751f,26.4856f,-37.9486f,  -14.1936f,26.0590f,-37.1404f,  -12.5466f,26.5039f,-36.0955f,  -12.6646f,26.0837f,-35.2806f,  -11.1651f,26.7636f,-34.1442f,  
	-11.2915f,26.3422f,-33.3164f,  -9.9771f,27.2330f,-32.0994f,  -10.1193f,26.8025f,-31.2549f,  -9.0228f,27.8758f,-29.9712f,  -9.1863f,27.4282f,-29.1077f,  
	-8.3348f,28.6523f,-27.7740f,  -8.5242f,28.1780f,-26.8915f,  -7.9382f,29.5186f,-25.5272f,  -8.1546f,29.0110f,-24.6258f,  -7.8484f,30.4327f,-23.2520f,  
	-8.0919f,29.8851f,-22.3338f,  -8.0730f,31.3518f,-20.9734f,  -8.3421f,30.7579f,-20.0417f,  -8.6111f,32.2341f,-18.7187f,  -8.9029f,31.5912f,-17.7767f,  
	-9.4539f,33.0427f,-16.5151f,  -9.7647f,32.3467f,-15.5663f,  -10.5856f,33.7414f,-14.3902f,  -10.9102f,32.9923f,-13.4401f,  -11.9829f,34.3007f,-12.3726f,  
	-12.3171f,33.4982f,-11.4233f,  -13.6185f,34.6941f,-10.4860f,  -13.9566f,33.8403f,-9.5420f,  -15.4589f,34.9010f,-8.7547f,  -15.7961f,33.9983f,-7.8187f,  
	-17.4679f,34.9053f,-7.1984f,  -17.7994f,33.9576f,-6.2732f,  -19.6060f,34.6961f,-5.8342f,  -19.9280f,33.7080f,-4.9218f,  -21.8328f,34.2673f,-4.6749f,  
	-22.1404f,33.2438f,-3.7774f,  -24.1051f,33.6178f,-3.7301f,  -24.3956f,32.5644f,-2.8485f,  -26.3813f,32.7507f,-3.0044f,  -26.6524f,31.6729f,-2.1393f,  
	-28.6204f,31.6732f,-2.4984f,  -28.8707f,30.5761f,-1.6496f,  -30.7832f,30.3954f,-2.2078f,  -31.0113f,29.2845f,-1.3756f,  -32.8321f,28.9314f,-2.1253f,  
	-33.0378f,27.8117f,-1.3090f,  -34.7328f,27.2978f,-2.2390f,  -34.9164f,26.1732f,-1.4376f,  -36.4544f,25.5126f,-2.5332f,  -36.6166f,24.3871f,-1.7460f,  
	-37.9696f,23.5962f,-2.9894f,  -38.1117f,22.4731f,-2.2155f,  -39.2554f,21.5704f,-3.5863f,  -39.3784f,20.4531f,-2.8248f,  -40.2926f,19.4585f,-4.3003f,  
	-40.3985f,18.3483f,-3.5497f,  -41.0673f,17.2826f,-5.1050f,  -41.1579f,16.1804f,-4.3641f,  -41.5700f,15.0654f,-5.9731f,  -41.6470f,13.9736f,-5.2416f,  
	-41.7956f,12.8312f,-6.8772f,  -41.8608f,11.7488f,-6.1535f,  -41.7440f,10.6011f,-7.7878f,  -41.7991f,9.5288f,-7.0713f,  -41.4194f,8.3972f,-8.6767f,  
	-41.4660f,7.3349f,-7.9668f,  -40.8306f,6.2403f,-9.5160f,  -40.8699f,5.1867f,-8.8116f,  -39.9904f,4.1489f,-10.2784f,  -40.0237f,3.1036f,-9.5794f,  
	-38.9157f,2.1411f,-10.9389f,  -38.9437f,1.1026f,-10.2441f,  -37.6268f,0.2324f,-11.4732f,  -37.6502f,-0.8002f,-10.7825f,  -36.1473f,-1.5630f,-11.8603f,  
	-36.1665f,-2.5901f,-11.1735f,  -34.5038f,-3.2322f,-12.0818f,  -34.5190f,-4.2555f,-11.3980f,  -32.7250f,-4.7660f,-12.1214f,  -32.7362f,-5.7858f,-11.4405f,  
	-30.8419f,-6.1559f,-11.9668f,  -30.8488f,-7.1730f,-11.2882f,  -28.8867f,-7.3964f,-11.6084f,  -28.8891f,-8.4111f,-10.9319f,  -26.8929f,-8.4838f,-11.0406f,  
	-26.8903f,-9.4962f,-10.3656f,  -23.5217f,31.7458f,-46.2916f,  -23.8845f,31.1855f,-45.3323f,  -22.2492f,30.0966f,-44.9555f,  -22.5323f,29.5520f,-44.0494f,  
	-20.7824f,28.7138f,-43.5623f,  -21.0020f,28.1896f,-42.6967f,  -19.1822f,27.6155f,-42.0961f,  -19.3534f,27.1136f,-41.2593f,  -17.5101f,26.8098f,-40.5436f,  
	-17.6476f,26.3288f,-39.7242f,  -15.8278f,26.2946f,-38.8939f,  -15.9438f,25.8314f,-38.0823f,  -14.1936f,26.0590f,-37.1404f,  -14.3012f,25.6082f,-36.3282f,  
	-12.6646f,26.0837f,-35.2806f,  -12.7738f,25.6384f,-34.4608f,  -11.2915f,26.3422f,-33.3164f,  -11.4108f,25.8950f,-32.4840f,  -10.1193f,26.8025f,-31.2549f,  
	-10.2553f,26.3455f,-30.4066f,  -9.1863f,27.4282f,-29.1077f,  -9.3451f,26.9518f,-28.2423f,  -8.5242f,28.1780f,-26.8915f,  -8.7077f,27.6752f,-26.0084f,  
	-8.1546f,29.0110f,-24.6258f,  -8.3643f,28.4746f,-23.7261f,  -8.0919f,29.8851f,-22.3338f,  -8.3275f,29.3080f,-21.4200f,  -8.3421f,30.7579f,-20.0417f,  
	-8.6015f,30.1366f,-19.1167f,  -8.9029f,31.5912f,-17.7767f,  -9.1833f,30.9207f,-16.8432f,  -9.7647f,32.3467f,-15.5663f,  -10.0617f,31.6259f,-14.6297f,  
	-10.9102f,32.9923f,-13.4401f,  -11.2198f,32.2199f,-12.5021f,  -12.3171f,33.4982f,-11.4233f,  -12.6340f,32.6754f,-10.4885f,  -13.9566f,33.8403f,-9.5420f,  
	-14.2761f,32.9684f,-8.6131f,  -15.7961f,33.9983f,-7.8187f,  -16.1135f,33.0807f,-6.8986f,  -17.7994f,33.9576f,-6.2732f,  -18.1107f,32.9981f,-5.3640f,  
	-19.9280f,33.7080f,-4.9218f,  -20.2283f,32.7108f,-4.0257f,  -22.1404f,33.2438f,-3.7774f,  -22.4266f,32.2141f,-2.8958f,  -24.3956f,32.5644f,-2.8485f,  
	-24.6649f,31.5074f,-1.9822f,  -26.6524f,31.6729f,-2.1393f,  -26.9031f,30.5935f,-1.2884f,  -28.8707f,30.5761f,-1.6496f,  -29.1010f,29.4797f,-0.8143f,  
	-31.0113f,29.2845f,-1.3756f,  -31.2207f,28.1760f,-0.5557f,  -33.0378f,27.8117f,-1.3090f,  -33.2261f,26.6953f,-0.5037f,  -34.9164f,26.1732f,-1.4376f,  
	-35.0840f,25.0531f,-0.6462f,  -36.6166f,24.3871f,-1.7460f,  -36.7644f,23.2666f,-0.9676f,  -38.1117f,22.4731f,-2.2155f,  -38.2405f,21.3561f,-1.4494f,  
	-39.3784f,20.4531f,-2.8248f,  -39.4899f,19.3412f,-2.0696f,  -40.3985f,18.3483f,-3.5497f,  -40.4943f,17.2427f,-2.8042f,  -41.1579f,16.1804f,-4.3641f,  
	-41.2396f,15.0838f,-3.6281f,  -41.6470f,13.9736f,-5.2416f,  -41.7163f,12.8853f,-4.5135f,  -41.8608f,11.7488f,-6.1535f,  -41.9194f,10.6700f,-5.4325f,  
	-41.7991f,9.5288f,-7.0713f,  -41.8485f,8.4594f,-6.3570f,  -41.4660f,7.3349f,-7.9668f,  -41.5076f,6.2739f,-7.2581f,  -40.8699f,5.1867f,-8.8116f,  
	-40.9049f,4.1340f,-8.1084f,  -40.0237f,3.1036f,-9.5794f,  -40.0529f,2.0576f,-8.8804f,  -38.9437f,1.1026f,-10.2441f,  -38.9680f,0.0626f,-9.5491f,  
	-37.6502f,-0.8002f,-10.7825f,  -37.6700f,-1.8345f,-10.0914f,  -36.1665f,-2.5901f,-11.1735f,  -36.1821f,-3.6203f,-10.4854f,  -34.5190f,-4.2555f,-11.3980f,  
	-34.5303f,-5.2821f,-10.7128f,  -32.7362f,-5.7858f,-11.4405f,  -32.7431f,-6.8097f,-10.7576f,  -30.8488f,-7.1730f,-11.2882f,  -30.8512f,-8.1944f,-10.6072f,  
	-28.8891f,-8.4111f,-10.9319f,  -28.8865f,-9.4301f,-10.2524f,  -26.8903f,-9.4962f,-10.3656f,  -26.8824f,-10.5128f,-9.6873f,  -23.8845f,31.1855f,-45.3323f,  
	-24.2068f,30.5925f,-44.3887f,  -22.5323f,29.5520f,-44.0494f,  -22.7824f,28.9791f,-43.1540f,  -21.0020f,28.1896f,-42.6967f,  -21.1947f,27.6402f,-41.8378f,  
	-19.3534f,27.1136f,-41.2593f,  -19.5033f,26.5879f,-40.4253f,  -17.6476f,26.3288f,-39.7242f,  -17.7672f,25.8251f,-38.9051f,  -15.9438f,25.8314f,-38.0823f,  
	-16.0471f,25.3452f,-37.2690f,  -14.3012f,25.6082f,-36.3282f,  -14.3988f,25.1337f,-35.5128f,  -12.7738f,25.6384f,-34.4608f,  -12.8751f,25.1688f,-33.6374f,  
	-11.4108f,25.8950f,-32.4840f,  -11.5235f,25.4227f,-31.6484f,  -10.2553f,26.3455f,-30.4066f,  -10.3869f,25.8610f,-29.5565f,  -9.3451f,26.9518f,-28.2423f,  
	-9.4985f,26.4480f,-27.3762f,  -8.7077f,27.6752f,-26.0084f,  -8.8856f,27.1445f,-25.1265f,  -8.3643f,28.4746f,-23.7261f,  -8.5676f,27.9093f,-22.8300f,  
	-8.3275f,29.3080f,-21.4200f,  -8.5549f,28.7039f,-20.5121f,  -8.6015f,30.1366f,-19.1167f,  -8.8513f,29.4876f,-18.1991f,  -9.1833f,30.9207f,-16.8432f,  
	-9.4517f,30.2249f,-15.9209f,  -10.0617f,31.6259f,-14.6297f,  -10.3453f,30.8811f,-13.7041f,  -11.2198f,32.2199f,-12.5021f,  -11.5138f,31.4262f,-11.5779f,  
	-12.6340f,32.6754f,-10.4885f,  -12.9340f,31.8332f,-9.5681f,  -14.2761f,32.9684f,-8.6131f,  -14.5774f,32.0803f,-7.6995f,  -16.1135f,33.0807f,-6.8986f,  
	-16.4119f,32.1498f,-5.9940f,  -18.1107f,32.9981f,-5.3640f,  -18.4014f,32.0278f,-4.4709f,  -20.2283f,32.7108f,-4.0257f,  -20.5079f,31.7058f,-3.1456f,  
	-22.4266f,32.2141f,-2.8958f,  -22.6920f,31.1793f,-2.0296f,  -24.6649f,31.5074f,-1.9822f,  -24.9142f,30.4473f,-1.1302f,  -26.9031f,30.5935f,-1.2884f,  
	-27.1340f,29.5135f,-0.4512f,  -29.1010f,29.4797f,-0.8143f,  -29.3126f,28.3845f,0.0082f,  -31.2207f,28.1760f,-0.5557f,  -31.4124f,27.0701f,0.2528f,  
	-33.2261f,26.6953f,-0.5037f,  -33.3979f,25.5828f,0.2913f,  -35.0840f,25.0531f,-0.6462f,  -35.2366f,23.9374f,0.1360f,  -36.7644f,23.2666f,-0.9676f,  
	-36.8982f,22.1520f,-0.1976f,  -38.2405f,21.3561f,-1.4494f,  -38.3569f,20.2444f,-0.6902f,  -39.4899f,19.3412f,-2.0696f,  -39.5905f,18.2340f,-1.3201f,  
	-40.4943f,17.2427f,-2.8042f,  -40.5804f,16.1429f,-2.0642f,  -41.2396f,15.0838f,-3.6281f,  -41.3129f,13.9912f,-2.8959f,  -41.7163f,12.8853f,-4.5135f,  
	-41.7783f,11.8012f,-3.7886f,  -41.9194f,10.6700f,-5.4325f,  -41.9717f,9.5945f,-4.7143f,  -41.8485f,8.4594f,-6.3570f,  -41.8924f,7.3919f,-5.6444f,  
	-41.5076f,6.2739f,-7.2581f,  -41.5443f,5.2145f,-6.5509f,  -40.9049f,4.1340f,-8.1084f,  -40.9354f,3.0810f,-7.4053f,  -40.0529f,2.0576f,-8.8804f,  
	-40.0781f,1.0108f,-8.1814f,  -38.9680f,0.0626f,-9.5491f,  -38.9883f,-0.9784f,-8.8540f,  -37.6700f,-1.8345f,-10.0914f,  -37.6858f,-2.8714f,-9.3992f,  
	-36.1821f,-3.6203f,-10.4854f,  -36.1935f,-4.6535f,-9.7960f,  -34.5303f,-5.2821f,-10.7128f,  -34.5373f,-6.3125f,-10.0256f,  -32.7431f,-6.8097f,-10.7576f,  
	-32.7455f,-7.8375f,-10.0723f,  -30.8512f,-8.1944f,-10.6072f,  -30.8486f,-9.2199f,-9.9235f,  -28.8865f,-9.4301f,-10.2524f,  -28.8786f,-10.4535f,-9.5696f,  
	-26.8824f,-10.5128f,-9.6873f,  -26.8686f,-11.5338f,-9.0050f,  -24.2068f,30.5925f,-44.3887f,  -24.4928f,29.9704f,-43.4602f,  -22.7824f,28.9791f,-43.1540f,  
	-23.0024f,28.3805f,-42.2691f,  -21.1947f,27.6402f,-41.8378f,  -21.3631f,27.0672f,-40.9852f,  -19.5033f,26.5879f,-40.4253f,  -19.6327f,26.0400f,-39.5949f,  
	-17.7672f,25.8251f,-38.9051f,  -17.8722f,25.2993f,-38.0869f,  -16.0471f,25.3452f,-37.2690f,  -16.1386f,24.8367f,-36.4546f,  -14.3988f,25.1337f,-35.5128f,  
	-14.4873f,24.6363f,-34.6955f,  -12.8751f,25.1688f,-33.6374f,  -12.9691f,24.6755f,-32.8121f,  -11.5235f,25.4227f,-31.6484f,  -11.6317f,24.9244f,-30.8116f,  
	-10.3869f,25.8610f,-29.5565f,  -10.5133f,25.3505f,-28.7058f,  -9.4985f,26.4480f,-27.3762f,  -9.6469f,25.9175f,-26.5110f,  -8.8856f,27.1445f,-25.1265f,  
	-9.0582f,26.5857f,-24.2476f,  -8.5676f,27.9093f,-22.8300f,  -8.7639f,27.3175f,-21.9391f,  -8.5549f,28.7039f,-20.5121f,  -8.7743f,28.0722f,-19.6107f,  
	-8.8513f,29.4876f,-18.1991f,  -9.0908f,28.8132f,-17.2917f,  -9.4517f,30.2249f,-15.9209f,  -9.7086f,29.5046f,-15.0088f,  -10.3453f,30.8811f,-13.7041f,  
	-10.6152f,30.1141f,-12.7916f,  -11.5138f,31.4262f,-11.5779f,  -11.7926f,30.6120f,-10.6672f,  -12.9340f,31.8332f,-9.5681f,  -13.2172f,30.9736f,-8.6623f,  
	-14.5774f,32.0803f,-7.6995f,  -14.8609f,31.1777f,-6.8010f,  -16.4119f,32.1498f,-5.9940f,  -16.6910f,31.2066f,-5.1052f,  -18.4014f,32.0278f,-4.4709f,  
	-18.6724f,31.0483f,-3.5936f,  -20.5079f,31.7058f,-3.1456f,  -20.7675f,30.6943f,-2.2808f,  -22.6920f,31.1793f,-2.0296f,  -22.9380f,30.1401f,-1.1779f,  
	-24.9142f,30.4473f,-1.1302f,  -25.1439f,29.3854f,-0.2923f,  -27.1340f,29.5135f,-0.4512f,  -27.3462f,28.4335f,0.3729f,  -29.3126f,28.3845f,0.0082f,  
	-29.5064f,27.2909f,0.8188f,  -31.4124f,27.0701f,0.2528f,  -31.5873f,25.9671f,1.0503f,  -33.3979f,25.5828f,0.2913f,  -33.5544f,24.4740f,1.0764f,  
	-35.2366f,23.9374f,0.1360f,  -35.3747f,22.8272f,0.9092f,  -36.8982f,22.1520f,-0.1976f,  -37.0191f,21.0425f,0.5649f,  -38.3569f,20.2444f,-0.6902f,  
	-38.4619f,19.1375f,0.0627f,  -39.5905f,18.2340f,-1.3201f,  -39.6807f,17.1328f,-0.5766f,  -40.5804f,16.1429f,-2.0642f,  -40.6574f,15.0474f,-1.3285f,  
	-41.3129f,13.9912f,-2.8959f,  -41.3782f,12.9030f,-2.1675f,  -41.7783f,11.8012f,-3.7886f,  -41.8333f,10.7209f,-3.0668f,  -41.9717f,9.5945f,-4.7143f,  
	-42.0178f,8.5216f,-3.9981f,  -41.8924f,7.3919f,-5.6444f,  -41.9308f,6.3267f,-4.9336f,  -41.5443f,5.2145f,-6.5509f,  -41.5761f,4.1554f,-5.8442f,  
	-40.9354f,3.0810f,-7.4053f,  -40.9615f,2.0280f,-6.7026f,  -40.0781f,1.0108f,-8.1814f,  -40.0990f,-0.0364f,-7.4825f,  -38.9883f,-0.9784f,-8.8540f,  
	-39.0045f,-2.0215f,-8.1580f,  -37.6858f,-2.8714f,-9.3992f,  -37.6975f,-3.9107f,-8.7059f,  -36.1935f,-4.6535f,-9.7960f,  -36.2007f,-5.6901f,-9.1048f,  
	-34.5373f,-6.3125f,-10.0256f,  -34.5398f,-7.3466f,-9.3362f,  -32.7455f,-7.8375f,-10.0723f,  -32.7430f,-8.8693f,-9.3844f,  -30.8486f,-9.2199f,-9.9235f,  
	-30.8406f,-10.2499f,-9.2364f,  -28.8786f,-10.4535f,-9.5696f,  -28.8647f,-11.4812f,-8.8831f,  -26.8686f,-11.5338f,-9.0050f,  -26.8484f,-12.5588f,-8.3184f,  
	-24.4928f,29.9704f,-43.4602f,  -24.7456f,29.3220f,-42.5460f,  -23.0024f,28.3805f,-42.2691f,  -23.1952f,27.7586f,-41.3944f,  -21.3631f,27.0672f,-40.9852f,  
	-21.5084f,26.4726f,-40.1396f,  -19.6327f,26.0400f,-39.5949f,  -19.7453f,25.4711f,-38.7680f,  -17.8722f,25.2993f,-38.0869f,  -17.9635f,24.7524f,-37.2700f,  
	-16.1386f,24.8367f,-36.4546f,  -16.2197f,24.3067f,-35.6401f,  -14.4873f,24.6363f,-34.6955f,  -14.5675f,24.1168f,-33.8775f,  -12.9691f,24.6755f,-32.8121f,  
	-13.0583f,24.1579f,-31.9861f,  -11.6317f,24.9244f,-30.8116f,  -11.7348f,24.4015f,-29.9744f,  -10.5133f,25.3505f,-28.7058f,  -10.6352f,24.8145f,-27.8560f,  
	-9.6469f,25.9175f,-26.5110f,  -9.7910f,25.3599f,-25.6485f,  -9.0582f,26.5857f,-24.2476f,  -9.2250f,26.0011f,-23.3733f,  -8.7639f,27.3175f,-21.9391f,  
	-8.9537f,26.6985f,-21.0539f,  -8.7743f,28.0722f,-19.6107f,  -8.9852f,27.4151f,-18.7187f,  -9.0908f,28.8132f,-17.2917f,  -9.3205f,28.1141f,-16.3937f,  
	-9.7086f,29.5046f,-15.0088f,  -9.9534f,28.7616f,-14.1087f,  -10.6152f,30.1141f,-12.7916f,  -10.8715f,29.3259f,-11.8917f,  -11.7926f,30.6120f,-10.6672f,  
	-12.0562f,29.7794f,-9.7706f,  -13.2172f,30.9736f,-8.6623f,  -13.4840f,30.0983f,-7.7712f,  -14.8609f,31.1777f,-6.8010f,  -15.1265f,30.2613f,-5.9179f,  
	-16.6910f,31.2066f,-5.1052f,  -16.9515f,30.2528f,-4.2321f,  -18.6724f,31.0483f,-3.5936f,  -18.9243f,30.0607f,-2.7315f,  -20.7675f,30.6943f,-2.2808f,  
	-21.0083f,29.6770f,-1.4306f,  -22.9380f,30.1401f,-1.1779f,  -23.1648f,29.0977f,-0.3405f,  -25.1439f,29.3854f,-0.2923f,  -25.3552f,28.3223f,0.5322f,  
	-27.3462f,28.4335f,0.3729f,  -27.5406f,27.3542f,1.1846f,  -29.5064f,27.2909f,0.8188f,  -29.6832f,26.1994f,1.6179f,  -31.5873f,25.9671f,1.0503f,  
	-31.7466f,24.8672f,1.8376f,  -33.5544f,24.4740f,1.0764f,  -33.6959f,23.3701f,1.8521f,  -35.3747f,22.8272f,0.9092f,  -35.4993f,21.7218f,1.6744f,  
	-37.0191f,21.0425f,0.5649f,  -37.1279f,19.9376f,1.3206f,  -38.4619f,19.1375f,0.0627f,  -38.5558f,18.0364f,0.8091f,  -39.6807f,17.1328f,-0.5766f,  
	-39.7612f,16.0359f,0.1621f,  -40.6574f,15.0474f,-1.3285f,  -40.7258f,13.9567f,-0.5970f,  -41.3782f,12.9030f,-2.1675f,  -41.4359f,11.8192f,-1.4427f,  
	-41.8333f,10.7209f,-3.0668f,  -41.8817f,9.6437f,-2.3475f,  -42.0178f,8.5216f,-3.9981f,  -42.0581f,7.4515f,-3.2841f,  -41.9308f,6.3267f,-4.9336f,  
	-41.9641f,5.2624f,-4.2236f,  -41.5761f,4.1554f,-5.8442f,  -41.6032f,3.0970f,-5.1381f,  -40.9615f,2.0280f,-6.7026f,  -40.9831f,0.9753f,-6.0003f,  
	-40.0990f,-0.0364f,-7.4825f,  -40.1156f,-1.0851f,-6.7829f,  -39.0045f,-2.0215f,-8.1580f,  -39.0164f,-3.0664f,-7.4610f,  -37.6975f,-3.9107f,-8.7059f,  
	-37.7047f,-4.9530f,-8.0109f,  -36.2007f,-5.6901f,-9.1048f,  -36.2031f,-6.7299f,-8.4115f,  -34.5398f,-7.3466f,-9.3362f,  -34.5372f,-8.3843f,-8.6444f,  
	-32.7430f,-8.8693f,-9.3844f,  -32.7349f,-9.9055f,-8.6933f,  -30.8406f,-10.2499f,-9.2364f,  -30.8267f,-11.2841f,-8.5457f,  -28.8647f,-11.4812f,-8.8831f,  
	-28.8444f,-12.5132f,-8.1923f,  -26.8484f,-12.5588f,-8.3184f,  -26.8213f,-13.5879f,-7.6271f,  -24.7456f,29.3220f,-42.5460f,  -24.9682f,28.6502f,-41.6458f,  
	-23.1952f,27.7586f,-41.3944f,  -23.3622f,27.1154f,-40.5301f,  -21.5084f,26.4726f,-40.1396f,  -21.6345f,25.8578f,-39.3005f,  -19.7453f,25.4711f,-38.7680f,  
	-19.8422f,24.8824f,-37.9450f,  -17.9635f,24.7524f,-37.2700f,  -18.0426f,24.1855f,-36.4551f,  -16.2197f,24.3067f,-35.6401f,  -16.2912f,23.7561f,-34.8266f,  
	-14.5675f,24.1168f,-33.8775f,  -14.6423f,23.5746f,-33.0598f,  -13.0583f,24.1579f,-31.9861f,  -13.1421f,23.6172f,-31.1604f,  -11.7348f,24.4015f,-29.9744f,  
	-11.8335f,23.8546f,-29.1384f,  -10.6352f,24.8145f,-27.8560f,  -10.7535f,24.2527f,-27.0087f,  -9.7910f,25.3599f,-25.6485f,  -9.9302f,24.7774f,-24.7901f,  
	-9.2250f,26.0011f,-23.3733f,  -9.3865f,25.3901f,-22.5041f,  -8.9537f,26.6985f,-21.0539f,  -9.1366f,26.0544f,-20.1773f,  -8.9852f,27.4151f,-18.7187f,  
	-9.1877f,26.7333f,-17.8351f,  -9.3205f,28.1141f,-16.3937f,  -9.5399f,27.3918f,-15.5069f,  -9.9534f,28.7616f,-14.1087f,  -10.1864f,27.9967f,-13.2206f,  
	-10.8715f,29.3259f,-11.8917f,  -11.1143f,28.5183f,-11.0052f,  -12.0562f,29.7794f,-9.7706f,  -12.3049f,28.9301f,-8.8880f,  -13.4840f,30.0983f,-7.7712f,  
	-13.7344f,29.2079f,-6.8950f,  -15.1265f,30.2613f,-5.9179f,  -15.3746f,29.3328f,-5.0501f,  -16.9515f,30.2528f,-4.2321f,  -17.1938f,29.2894f,-3.3740f,  
	-18.9243f,30.0607f,-2.7315f,  -19.1581f,29.0660f,-1.8840f,  -21.0083f,29.6770f,-1.4306f,  -21.2305f,28.6551f,-0.5948f,  -23.1648f,29.0977f,-0.3405f,  
	-23.3735f,28.0527f,0.4834f,  -25.3552f,28.3223f,0.5322f,  -25.5487f,27.2587f,1.3440f,  -27.5406f,27.3542f,1.1846f,  -27.7180f,26.2759f,1.9845f,  
	-29.6832f,26.1994f,1.6179f,  -29.8441f,25.1102f,2.4065f,  -31.7466f,24.8672f,1.8376f,  -31.8905f,23.7715f,2.6148f,  -33.6959f,23.3701f,1.8521f,  
	-33.8235f,22.2706f,2.6191f,  -35.4993f,21.7218f,1.6744f,  -35.6113f,20.6207f,2.4321f,  -37.1279f,19.9376f,1.3206f,  -37.2251f,18.8385f,2.0693f,  
	-38.5558f,18.0364f,0.8091f,  -38.6395f,16.9399f,1.5501f,  -39.7612f,16.0359f,0.1621f,  -39.8325f,14.9442f,0.8959f,  -40.7258f,13.9567f,-0.5970f,  
	-40.7861f,12.8706f,0.1303f,  -41.4359f,11.8192f,-1.4427f,  -41.4864f,10.7389f,-0.7207f,  -41.8817f,9.6437f,-2.3475f,  -41.9237f,8.5698f,-1.6308f,  
	-42.0581f,7.4515f,-3.2841f,  -42.0926f,6.3830f,-2.5714f,  -41.9641f,5.2624f,-4.2236f,  -41.9921f,4.1994f,-3.5146f,  -41.6032f,3.0970f,-5.1381f,  
	-41.6254f,2.0395f,-4.4328f,  -40.9831f,0.9753f,-6.0003f,  -41.0001f,-0.0783f,-5.2976f,  -40.1156f,-1.0851f,-6.7829f,  -40.1278f,-2.1351f,-6.0827f,  
	-39.0164f,-3.0664f,-7.4610f,  -39.0238f,-4.1139f,-6.7626f,  -37.7047f,-4.9530f,-8.0109f,  -37.7072f,-5.9981f,-7.3141f,  -36.2031f,-6.7299f,-8.4115f,  
	-36.2005f,-7.7730f,-7.7161f,  -34.5372f,-8.3843f,-8.6444f,  -34.5291f,-9.4261f,-7.9495f,  -32.7349f,-9.9055f,-8.6933f,  -32.7210f,-10.9457f,-7.9987f,  
	-30.8267f,-11.2841f,-8.5457f,  -30.8064f,-12.3226f,-7.8510f,  -28.8444f,-12.5132f,-8.1923f,  -28.8172f,-13.5493f,-7.4969f,  -26.8213f,-13.5879f,-7.6271f,  
	-26.7866f,-14.6210f,-6.9304f,  -24.9682f,28.6502f,-41.6458f,  -25.1623f,27.9574f,-40.7597f,  -23.3622f,27.1154f,-40.5301f,  -23.5073f,26.4528f,-39.6755f,  
	-21.6345f,25.8578f,-39.3005f,  -21.7426f,25.2243f,-38.4682f,  -19.8422f,24.8824f,-37.9450f,  -19.9250f,24.2750f,-37.1265f,  -18.0426f,24.1855f,-36.4551f,  
	-18.1109f,23.5996f,-35.6430f,  -16.2912f,23.7561f,-34.8266f,  -16.3565f,23.1846f,-34.0146f,  -14.6423f,23.5746f,-33.0598f,  -14.7111f,23.0111f,-32.2434f,  
	-13.1421f,23.6172f,-31.1604f,  -13.2214f,23.0541f,-30.3364f,  -11.8335f,23.8546f,-29.1384f,  -11.9291f,23.2833f,-28.3049f,  -10.7535f,24.2527f,-27.0087f,  
	-10.8674f,23.6672f,-26.1652f,  -9.9302f,24.7774f,-24.7901f,  -10.0651f,24.1694f,-23.9363f,  -9.3865f,25.3901f,-22.5041f,  -9.5424f,24.7544f,-21.6427f,  
	-9.1366f,26.0544f,-20.1773f,  -9.3126f,25.3858f,-19.3085f,  -9.1877f,26.7333f,-17.8351f,  -9.3817f,26.0282f,-16.9621f,  -9.5399f,27.3918f,-15.5069f,  
	-9.7491f,26.6472f,-14.6313f,  -10.1864f,27.9967f,-13.2206f,  -10.4076f,27.2117f,-12.3452f,  -11.1143f,28.5183f,-11.0052f,  -11.3438f,27.6931f,-10.1323f,  
	-12.3049f,28.9301f,-8.8880f,  -12.5388f,28.0645f,-8.0199f,  -13.7344f,29.2079f,-6.8950f,  -13.9687f,28.3042f,-6.0338f,  -15.3746f,29.3328f,-5.0501f,  
	-15.6058f,28.3935f,-4.1972f,  -17.1938f,29.2894f,-3.3740f,  -17.4190f,28.3176f,-2.5304f,  -19.1581f,29.0660f,-1.8840f,  -19.3741f,28.0654f,-1.0510f,  
	-21.2305f,28.6551f,-0.5948f,  -21.4351f,27.6294f,0.2273f,  -23.3735f,28.0527f,0.4834f,  -23.5648f,27.0062f,1.2942f,  -25.5487f,27.2587f,1.3440f,  
	-25.7254f,26.1952f,2.1436f,  -27.7180f,26.2759f,1.9845f,  -27.8795f,25.1990f,2.7733f,  -29.8441f,25.1102f,2.4065f,  -29.9896f,24.0244f,3.1845f,  
	-31.8905f,23.7715f,2.6148f,  -32.0203f,22.6796f,3.3829f,  -33.8235f,22.2706f,2.6191f,  -33.9381f,21.1751f,3.3782f,  -35.6113f,20.6207f,2.4321f,  
	-35.7113f,19.5251f,3.1824f,  -37.2251f,18.8385f,2.0693f,  -37.3116f,17.7438f,2.8121f,  -38.6395f,16.9399f,1.5501f,  -38.7134f,15.8485f,2.2859f,  
	-39.8325f,14.9442f,0.8959f,  -39.8951f,13.8573f,1.6253f,  -40.7861f,12.8706f,0.1303f,  -40.8387f,11.7885f,0.8543f,  -41.4864f,10.7389f,-0.7207f,  
	-41.5301f,9.6624f,-0.0018f,  -41.9237f,8.5698f,-1.6308f,  -41.9595f,7.4980f,-0.9158f,  -42.0926f,6.3830f,-2.5714f,  -42.1216f,5.3163f,-1.8600f,  
	-41.9921f,4.1994f,-3.5146f,  -42.0150f,3.1379f,-2.8068f,  -41.6254f,2.0395f,-4.4328f,  -41.6429f,0.9816f,-3.7274f,  -41.0001f,-0.0783f,-5.2976f,  
	-41.0124f,-1.1327f,-4.5946f,  -40.1278f,-2.1351f,-6.0827f,  -40.1352f,-3.1871f,-5.3813f,  -39.0238f,-4.1139f,-6.7626f,  -39.0262f,-5.1636f,-6.0628f,  
	-37.7072f,-5.9981f,-7.3141f,  -37.7046f,-7.0460f,-6.6155f,  -36.2005f,-7.7730f,-7.7161f,  -36.1925f,-8.8201f,-7.0178f,  -34.5291f,-9.4261f,-7.9495f,  
	-34.5152f,-10.4719f,-7.2514f,  -32.7210f,-10.9457f,-7.9987f,  -32.7007f,-11.9902f,-7.3003f,  -30.8064f,-12.3226f,-7.8510f,  -30.7791f,-13.3653f,-7.1518f,  
	-28.8172f,-13.5493f,-7.4969f,  -28.7824f,-14.5897f,-6.7964f,  -26.7866f,-14.6210f,-6.9304f,  -26.7440f,-15.6577f,-6.2282f,  -25.1623f,27.9574f,-40.7597f,  
	-25.3318f,27.2458f,-39.8864f,  -23.5073f,26.4528f,-39.6755f,  -23.6321f,25.7724f,-38.8306f,  -21.7426f,25.2243f,-38.4682f,  -21.8346f,24.5734f,-37.6429f,  
	-19.9250f,24.2750f,-37.1265f,  -19.9953f,23.6500f,-36.3129f,  -18.1109f,23.5996f,-35.6430f,  -18.1716f,22.9944f,-34.8339f,  -16.3565f,23.1846f,-34.0146f,  
	-16.4149f,22.5934f,-33.2050f,  -14.7111f,23.0111f,-32.2434f,  -14.7751f,22.4267f,-31.4293f,  -13.2214f,23.0541f,-30.3364f,  -13.2977f,22.4681f,-29.5150f,  
	-11.9291f,23.2833f,-28.3049f,  -12.0207f,22.6895f,-27.4753f,  -10.8674f,23.6672f,-26.1652f,  -10.9781f,23.0572f,-25.3262f,  -10.0651f,24.1694f,-23.9363f,  
	-10.1958f,23.5374f,-23.0898f,  -9.5424f,24.7544f,-21.6427f,  -9.6928f,24.0947f,-20.7886f,  -9.3126f,25.3858f,-19.3085f,  -9.4815f,24.6940f,-18.4494f,  
	-9.3817f,26.0282f,-16.9621f,  -9.5671f,25.3005f,-16.0994f,  -9.7491f,26.6472f,-14.6313f,  -9.9482f,25.8821f,-13.7676f,  -10.4076f,27.2117f,-12.3452f,  
	-10.6170f,26.4086f,-11.4828f,  -11.3438f,27.6931f,-10.1323f,  -11.5599f,26.8507f,-9.2732f,  -12.5388f,28.0645f,-8.0199f,  -12.7579f,27.1845f,-7.1663f,  
	-13.9687f,28.3042f,-6.0338f,  -14.1874f,27.3884f,-5.1871f,  -15.6058f,28.3935f,-4.1972f,  -15.8210f,27.4445f,-3.3586f,  -17.4190f,28.3176f,-2.5304f,  
	-17.6274f,27.3385f,-1.7012f,  -19.3741f,28.0654f,-1.0510f,  -19.5732f,27.0596f,-0.2317f,  -21.4351f,27.6294f,0.2273f,  -21.6228f,26.6009f,1.0362f,  
	-23.5648f,27.0062f,1.2942f,  -23.7396f,25.9586f,2.0926f,  -25.7254f,26.1952f,2.1436f,  -25.8864f,25.1320f,2.9318f,  -27.8795f,25.1990f,2.7733f,  
	-28.0255f,24.1246f,3.5512f,  -29.9896f,24.0244f,3.1845f,  -30.1207f,22.9417f,3.9529f,  -32.0203f,22.6796f,3.3829f,  -32.1368f,21.5913f,4.1426f,  
	-33.9381f,21.1751f,3.3782f,  -34.0403f,20.0847f,4.1293f,  -35.7113f,19.5251f,3.1824f,  -35.8001f,18.4339f,3.9262f,  -37.3116f,17.7438f,2.8121f,  
	-37.3878f,16.6543f,3.5490f,  -38.7134f,15.8485f,2.2859f,  -38.7782f,14.7622f,3.0166f,  -39.8951f,13.8573f,1.6253f,  -39.9496f,12.7746f,2.3507f,  
	-40.8387f,11.7885f,0.8543f,  -40.8839f,10.7105f,1.5748f,  -41.5301f,9.6624f,-0.0018f,  -41.5672f,8.5884f,0.7149f,  -41.9595f,7.4980f,-0.9158f,  
	-41.9895f,6.4286f,-0.2025f,  -42.1216f,5.3163f,-1.8600f,  -42.1451f,4.2517f,-1.1501f,  -42.0150f,3.1379f,-2.8068f,  -42.0328f,2.0767f,-2.0992f,  
	-41.6429f,0.9816f,-3.7274f,  -41.6554f,-0.0763f,-3.0221f,  -41.0124f,-1.1327f,-4.5946f,  -41.0200f,-2.1885f,-3.8907f,  -40.1352f,-3.1871f,-5.3813f,  
	-40.1377f,-4.2408f,-4.6788f,  -39.0262f,-5.1636f,-6.0628f,  -39.0236f,-6.2158f,-5.3613f,  -37.7046f,-7.0460f,-6.6155f,  -37.6966f,-8.0977f,-5.9142f,  
	-36.1925f,-8.8201f,-7.0178f,  -36.1786f,-9.8708f,-6.3167f,  -34.5152f,-10.4719f,-7.2514f,  -34.4950f,-11.5219f,-6.5498f,  -32.7007f,-11.9902f,-7.3003f,  
	-32.6735f,-13.0390f,-6.5978f,  -30.7791f,-13.3653f,-7.1518f,  -30.7444f,-14.4124f,-6.4478f,  -28.7824f,-14.5897f,-6.7964f,  -28.7396f,-15.6340f,-6.0903f,  
	-26.7440f,-15.6577f,-6.2282f,  -26.6926f,-16.6983f,-5.5195f,  -25.3318f,27.2458f,-39.8864f,  -25.4785f,26.5172f,-39.0259f,  -23.6321f,25.7724f,-38.8306f,  
	-23.7386f,25.0758f,-37.9953f,  -21.8346f,24.5734f,-37.6429f,  -21.9122f,23.9063f,-36.8246f,  -19.9953f,23.6500f,-36.3129f,  -20.0566f,23.0075f,-35.5038f,  
	-18.1716f,22.9944f,-34.8339f,  -18.2245f,22.3714f,-34.0286f,  -16.4149f,22.5934f,-33.2050f,  -16.4680f,21.9831f,-32.3986f,  -14.7751f,22.4267f,-31.4293f,  
	-14.8359f,21.8211f,-30.6183f,  -13.2977f,22.4681f,-29.5150f,  -13.3703f,21.8611f,-28.6978f,  -12.0207f,22.6895f,-27.4753f,  -12.1097f,22.0726f,-26.6498f,  
	-10.9781f,23.0572f,-25.3262f,  -11.0855f,22.4242f,-24.4939f,  -10.1958f,23.5374f,-23.0898f,  -10.3220f,22.8821f,-22.2500f,  -9.6928f,24.0947f,-20.7886f,  
	-9.8376f,23.4122f,-19.9434f,  -9.4815f,24.6940f,-18.4494f,  -9.6435f,23.9797f,-17.6000f,  -9.5671f,25.3005f,-16.0994f,  -9.7439f,24.5522f,-15.2481f,  
	-9.9482f,25.8821f,-13.7676f,  -10.1370f,25.0983f,-12.9163f,  -10.6170f,26.4086f,-11.4828f,  -10.8147f,25.5873f,-10.6335f,  -11.5599f,26.8507f,-9.2732f,  
	-11.7629f,25.9929f,-8.4281f,  -12.7579f,27.1845f,-7.1663f,  -12.9629f,26.2912f,-6.3268f,  -14.1874f,27.3884f,-5.1871f,  -14.3911f,26.4618f,-4.3546f,  
	-15.8210f,27.4445f,-3.3586f,  -16.0202f,26.4870f,-2.5343f,  -17.6274f,27.3385f,-1.7012f,  -17.8196f,26.3530f,-0.8857f,  -19.5732f,27.0596f,-0.2317f,  
	-19.7560f,26.0498f,0.5742f,  -21.6228f,26.6009f,1.0362f,  -21.7945f,25.5702f,1.8324f,  -23.7396f,25.9586f,2.0926f,  -23.8989f,24.9103f,2.8793f,  
	-25.8864f,25.1320f,2.9318f,  -26.0319f,24.0703f,3.7088f,  -28.0255f,24.1246f,3.5512f,  -28.1571f,23.0526f,4.3191f,  -30.1207f,22.9417f,3.9529f,  
	-30.2383f,21.8621f,4.7123f,  -32.1368f,21.5913f,4.1426f,  -32.2406f,20.5075f,4.8939f,  -34.0403f,20.0847f,4.1293f,  -34.1310f,18.9984f,4.8735f,  
	-35.8001f,18.4339f,3.9262f,  -35.8783f,17.3478f,4.6637f,  -37.3878f,16.6543f,3.5490f,  -37.4544f,15.5699f,4.2804f,  -38.7782f,14.7622f,3.0166f,  
	-38.8343f,13.6802f,3.7429f,  -39.9496f,12.7746f,2.3507f,  -39.9962f,11.6963f,3.0723f,  -40.8839f,10.7105f,1.5748f,  -40.9222f,9.6355f,2.2928f,  
	-41.5672f,8.5884f,0.7149f,  -41.5980f,7.5174f,1.4295f,  -41.9895f,6.4286f,-0.2025f,  -42.0136f,5.3619f,0.5088f,  -42.1451f,4.2517f,-1.1501f,  
	-42.1634f,3.1880f,-0.4409f,  -42.0328f,2.0767f,-2.0992f,  -42.0456f,1.0160f,-1.3921f,  -41.6554f,-0.0763f,-3.0221f,  -41.6630f,-1.1351f,-2.3162f,  
	-41.0200f,-2.1885f,-3.8907f,  -41.0225f,-3.2455f,-3.1860f,  -40.1377f,-4.2408f,-4.6788f,  -40.1351f,-5.2965f,-3.9751f,  -39.0236f,-6.2158f,-5.3613f,  
	-39.0156f,-7.2713f,-4.6575f,  -37.6966f,-8.0977f,-5.9142f,  -37.6828f,-9.1526f,-5.2104f,  -36.1786f,-9.8708f,-6.3167f,  -36.1585f,-10.9255f,-5.6122f,  
	-34.4950f,-11.5219f,-6.5498f,  -34.4679f,-12.5759f,-5.8443f,  -32.6735f,-13.0390f,-6.5978f,  -32.6388f,-14.0921f,-5.8906f,  -30.7444f,-14.4124f,-6.4478f,  
	-30.7016f,-15.4637f,-5.7384f,  -28.7396f,-15.6340f,-6.0903f,  -28.6881f,-16.6825f,-5.3782f,  -26.6926f,-16.6983f,-5.5195f,  -26.6320f,-17.7422f,-4.8042f,  
	-25.4785f,26.5172f,-39.0259f,  -25.6046f,25.7735f,-38.1778f,  -23.7386f,25.0758f,-37.9953f,  -23.8288f,24.3643f,-37.1696f,  -21.9122f,23.9063f,-36.8246f,  
	-21.9792f,23.2234f,-36.0129f,  -20.0566f,23.0075f,-35.5038f,  -20.1088f,22.3488f,-34.7001f,  -18.2245f,22.3714f,-34.0286f,  -18.2711f,21.7309f,-33.2276f,  
	-16.4680f,21.9831f,-32.3986f,  -16.5175f,21.3534f,-31.5959f,  -14.8359f,21.8211f,-30.6183f,  -14.8931f,21.1961f,-29.8117f,  -13.3703f,21.8611f,-28.6978f,  
	-13.4407f,21.2324f,-27.8847f,  -12.1097f,22.0726f,-26.6498f,  -12.1963f,21.4338f,-25.8308f,  -11.0855f,22.4242f,-24.4939f,  -11.1894f,21.7687f,-23.6678f,  
	-10.3220f,22.8821f,-22.2500f,  -10.4440f,22.2045f,-21.4186f,  -9.8376f,23.4122f,-19.9434f,  -9.9769f,22.7074f,-19.1074f,  -9.6435f,23.9797f,-17.6000f,  
	-9.7984f,23.2447f,-16.7612f,  -9.7439f,24.5522f,-15.2481f,  -9.9121f,23.7849f,-14.4084f,  -10.1370f,25.0983f,-12.9163f,  -10.3158f,24.2958f,-12.0775f,  
	-10.8147f,25.5873f,-10.6335f,  -11.0007f,24.7498f,-9.7977f,  -11.7629f,25.9929f,-8.4281f,  -11.9531f,25.1208f,-7.5967f,  -12.9629f,26.2912f,-6.3268f,  
	-13.1541f,25.3861f,-5.5012f,  -14.3911f,26.4618f,-4.3546f,  -14.5801f,25.5254f,-3.5361f,  -16.0202f,26.4870f,-2.5343f,  -16.2043f,25.5217f,-1.7236f,  
	-17.8196f,26.3530f,-0.8857f,  -17.9963f,25.3623f,-0.0837f,  -19.7560f,26.0498f,0.5742f,  -19.9233f,25.0367f,1.3673f,  -21.7945f,25.5702f,1.8324f,  
	-21.9510f,24.5378f,2.6167f,  -23.8989f,24.9103f,2.8793f,  -24.0430f,23.8625f,3.6545f,  -26.0319f,24.0703f,3.7088f,  -26.1631f,23.0102f,4.4754f,  
	-28.1571f,23.0526f,4.3191f,  -28.2752f,21.9830f,5.0776f,  -30.2383f,21.8621f,4.7123f,  -30.3431f,20.7864f,5.4630f,  -32.2406f,20.5075f,4.8939f,  
	-32.3327f,19.4276f,5.6378f,  -34.1310f,18.9984f,4.8735f,  -34.2106f,17.9170f,5.6109f,  -35.8783f,17.3478f,4.6637f,  -35.9464f,16.2666f,5.3952f,  
	-37.4544f,15.5699f,4.2804f,  -37.5120f,14.4900f,5.0070f,  -38.8343f,13.6802f,3.7429f,  -38.8823f,12.6029f,4.4649f,  -39.9962f,11.6963f,3.0723f,  
	-40.0356f,10.6214f,3.7908f,  -40.9222f,9.6355f,2.2928f,  -40.9538f,8.5639f,3.0081f,  -41.5980f,7.5174f,1.4295f,  -41.6227f,6.4496f,2.1418f,  
	-42.0136f,5.3619f,0.5088f,  -42.0322f,4.2967f,1.2191f,  -42.1634f,3.1880f,-0.4409f,  -42.1763f,2.1254f,0.2675f,  -42.0456f,1.0160f,-1.3921f,  
	-42.0533f,-0.0450f,-0.6848f,  -41.6630f,-1.1351f,-2.3162f,  -41.6655f,-2.1946f,-1.6099f,  -41.0225f,-3.2455f,-3.1860f,  -41.0199f,-4.3038f,-2.4805f,  
	-40.1351f,-5.2965f,-3.9751f,  -40.1271f,-6.3551f,-3.2692f,  -39.0156f,-7.2713f,-4.6575f,  -39.0019f,-8.3297f,-3.9515f,  -37.6828f,-9.1526f,-5.2104f,  
	-37.6628f,-10.2113f,-4.5035f,  -36.1585f,-10.9255f,-5.6122f,  -36.1316f,-11.9841f,-4.9042f,  -34.4679f,-12.5759f,-5.8443f,  -34.4335f,-13.6344f,-5.1344f,  
	-32.6388f,-14.0921f,-5.8906f,  -32.5963f,-15.1497f,-5.1783f,  -30.7016f,-15.4637f,-5.7384f,  -30.6501f,-16.5194f,-5.0232f,  -28.6881f,-16.6825f,-5.3782f,  
	-28.6272f,-17.7346f,-4.6597f,  -26.6320f,-17.7422f,-4.8042f,  -26.5613f,-18.7896f,-4.0816f,  -25.6046f,25.7735f,-38.1778f,  -25.7122f,25.0161f,-37.3418f,  
	-23.8288f,24.3643f,-37.1696f,  -23.9065f,23.6386f,-36.3523f,  -21.9792f,23.2234f,-36.0129f,  -22.0356f,22.5260f,-35.2082f,  -20.1088f,22.3488f,-34.7001f,  
	-20.1537f,21.6745f,-33.9021f,  -18.2711f,21.7309f,-33.2276f,  -18.3136f,21.0729f,-32.4311f,  -16.5175f,21.3534f,-31.5959f,  -16.5631f,20.7059f,-30.7982f,  
	-14.8931f,21.1961f,-29.8117f,  -14.9481f,20.5510f,-29.0096f,  -13.4407f,21.2324f,-27.8847f,  -13.5091f,20.5831f,-27.0778f,  -12.1963f,21.4338f,-25.8308f,  
	-12.2801f,20.7737f,-25.0178f,  -11.1894f,21.7687f,-23.6678f,  -11.2902f,21.0918f,-22.8497f,  -10.4440f,22.2045f,-21.4186f,  -10.5617f,21.5053f,-20.5957f,  
	-9.9769f,22.7074f,-19.1074f,  -10.1105f,21.9820f,-18.2812f,  -9.7984f,23.2447f,-16.7612f,  -9.9460f,22.4907f,-15.9336f,  -9.9121f,23.7849f,-14.4084f,  
	-10.0717f,22.9984f,-13.5806f,  -10.3158f,24.2958f,-12.0775f,  -10.4844f,23.4765f,-11.2517f,  -11.0007f,24.7498f,-9.7977f,  -11.1755f,23.8972f,-8.9752f,  
	-11.9531f,25.1208f,-7.5967f,  -12.1309f,24.2360f,-6.7789f,  -13.1541f,25.3861f,-5.5012f,  -13.3318f,24.4701f,-4.6894f,  -14.5801f,25.5254f,-3.5361f,  
	-14.7550f,24.5800f,-2.7309f,  -16.2043f,25.5217f,-1.7236f,  -16.3738f,24.5502f,-0.9263f,  -17.9963f,25.3623f,-0.0837f,  -18.1582f,24.3671f,0.7055f,  
	-19.9233f,25.0367f,1.3673f,  -20.0760f,24.0208f,2.1485f,  -21.9510f,24.5378f,2.6167f,  -22.0926f,23.5047f,3.3893f,  -24.0430f,23.8625f,3.6545f,  
	-24.1729f,22.8153f,4.4190f,  -26.1631f,23.0102f,4.4754f,  -26.2809f,21.9519f,5.2322f,  -28.2752f,21.9830f,5.0776f,  -28.3803f,20.9166f,5.8270f,  
	-30.3431f,20.7864f,5.4630f,  -30.4360f,19.7141f,6.2059f,  -32.3327f,19.4276f,5.6378f,  -32.4135f,18.3522f,6.3743f,  -34.2106f,17.9170f,5.6109f,  
	-34.2800f,16.8403f,6.3418f,  -35.9464f,16.2666f,5.3952f,  -36.0053f,15.1899f,6.1214f,  -37.5120f,14.4900f,5.0070f,  -37.5611f,13.4148f,5.7288f,  
	-38.8823f,12.6029f,4.4649f,  -38.9226f,11.5293f,5.1834f,  -40.0356f,10.6214f,3.7908f,  -40.0680f,9.5503f,4.5063f,  -40.9538f,8.5639f,3.0081f,  
	-40.9791f,7.4959f,3.7207f,  -41.6227f,6.4496f,2.1418f,  -41.6417f,5.3837f,2.8525f,  -42.0322f,4.2967f,1.2191f,  -42.0454f,3.2331f,1.9282f,  
	-42.1763f,2.1254f,0.2675f,  -42.1841f,1.0631f,0.9757f,  -42.0533f,-0.0450f,-0.6848f,  -42.0558f,-1.1061f,0.0226f,  -41.6655f,-2.1946f,-1.6099f,  
	-41.6629f,-3.2548f,-0.9031f,  -41.0199f,-4.3038f,-2.4805f,  -41.0120f,-5.3647f,-1.7732f,  -40.1271f,-6.3551f,-3.2692f,  -40.1135f,-7.4161f,-2.5616f,  
	-39.0019f,-8.3297f,-3.9515f,  -38.9822f,-9.3915f,-3.2428f,  -37.6628f,-10.2113f,-4.5035f,  -37.6362f,-11.2736f,-3.7935f,  -36.1316f,-11.9841f,-4.9042f,  
	-36.0976f,-13.0471f,-4.1921f,  -34.4335f,-13.6344f,-5.1344f,  -34.3913f,-14.6974f,-4.4198f,  -32.5963f,-15.1497f,-5.1783f,  -32.5450f,-16.2118f,-4.4606f,  
	-30.6501f,-16.5194f,-5.0232f,  -30.5894f,-17.5790f,-4.3020f,  -28.6272f,-17.7346f,-4.6597f,  -28.5563f,-18.7908f,-3.9341f,  -26.5613f,-18.7896f,-4.0816f,  
	-26.4801f,-19.8398f,-3.3514f,  -25.7122f,25.0161f,-37.3418f,  -25.8055f,24.2460f,-36.5164f,  -23.9065f,23.6386f,-36.3523f,  -23.9720f,22.9001f,-35.5440f,  
	-22.0356f,22.5260f,-35.2082f,  -22.0834f,21.8148f,-34.4106f,  -20.1537f,21.6745f,-33.9021f,  -20.1935f,20.9846f,-33.1096f,  -18.3136f,21.0729f,-32.4311f,  
	-18.3516f,20.3989f,-31.6402f,  -16.5631f,20.7059f,-30.7982f,  -16.6064f,20.0401f,-30.0052f,  -14.9481f,20.5510f,-29.0096f,  -15.0017f,19.8869f,-28.2134f,  
	-13.5091f,20.5831f,-27.0778f,  -13.5755f,19.9139f,-26.2767f,  -12.2801f,20.7737f,-25.0178f,  -12.3618f,20.0931f,-24.2122f,  -11.2902f,21.0918f,-22.8497f,  
	-11.3879f,20.3940f,-22.0395f,  -10.5617f,21.5053f,-20.5957f,  -10.6750f,20.7858f,-19.7821f,  -10.1105f,21.9820f,-18.2812f,  -10.2382f,21.2379f,-17.4656f,  
	-9.9460f,22.4907f,-15.9336f,  -10.0867f,21.7173f,-15.1170f,  -10.0717f,22.9984f,-13.5806f,  -10.2228f,22.1947f,-12.7652f,  -10.4844f,23.4765f,-11.2517f,  
	-10.6432f,22.6413f,-10.4385f,  -11.1755f,23.8972f,-8.9752f,  -11.3391f,23.0310f,-8.1657f,  -12.1309f,24.2360f,-6.7789f,  -12.2964f,23.3392f,-5.9744f,  
	-13.3318f,24.4701f,-4.6894f,  -13.4965f,23.5440f,-3.8906f,  -14.7550f,24.5800f,-2.7309f,  -14.9162f,23.6273f,-1.9392f,  -16.3738f,24.5502f,-0.9263f,  
	-16.5292f,23.5730f,-0.1419f,  -18.1582f,24.3671f,0.7055f,  -18.3061f,23.3681f,1.4827f,  -20.0760f,24.0208f,2.1485f,  -20.2143f,23.0030f,2.9178f,  
	-22.0926f,23.5047f,3.3893f,  -22.2205f,22.4713f,4.1510f,  -24.1729f,22.8153f,4.4190f,  -24.2896f,21.7692f,5.1734f,  -26.2809f,21.9519f,5.2322f,  
	-26.3857f,20.8959f,5.9796f,  -28.3803f,20.9166f,5.8270f,  -28.4735f,19.8532f,6.5682f,  -30.4360f,19.7141f,6.2059f,  -30.5175f,18.6460f,6.9410f,  
	-32.4135f,18.3522f,6.3743f,  -32.4838f,17.2813f,7.1041f,  -34.2800f,16.8403f,6.3418f,  -34.3399f,15.7682f,7.0670f,  -36.0053f,15.1899f,6.1214f,  
	-36.0553f,14.1181f,6.8424f,  -37.5611f,13.4148f,5.7288f,  -37.6023f,12.3435f,6.4466f,  -38.9226f,11.5293f,5.1834f,  -38.9557f,10.4597f,5.8983f,  
	-40.0680f,9.5503f,4.5063f,  -40.0938f,8.4832f,5.2186f,  -40.9791f,7.4959f,3.7207f,  -40.9984f,6.4304f,4.4314f,  -41.6417f,5.3837f,2.8525f,  
	-41.6550f,4.3200f,3.5617f,  -42.0454f,3.2331f,1.9282f,  -42.0532f,2.1703f,2.6367f,  -42.1841f,1.0631f,0.9757f,  -42.1867f,0.0012f,1.6836f,  
	-42.0558f,-1.1061f,0.0226f,  -42.0532f,-2.1674f,0.7301f,  -41.6629f,-3.2548f,-0.9031f,  -41.6551f,-4.3171f,-0.1949f,  -41.0120f,-5.3647f,-1.7732f,  
	-40.9985f,-6.4274f,-1.0645f,  -40.1135f,-7.4161f,-2.5616f,  -40.0940f,-8.4802f,-1.8516f,  -38.9822f,-9.3915f,-3.2428f,  -38.9560f,-10.4567f,-2.5314f,  
	-37.6362f,-11.2736f,-3.7935f,  -37.6027f,-12.3402f,-3.0798f,  -36.0976f,-13.0471f,-4.1921f,  -36.0559f,-14.1145f,-3.4756f,  -34.3913f,-14.6974f,-4.4198f,  
	-34.3406f,-15.7649f,-3.7000f,  -32.5450f,-16.2118f,-4.4606f,  -32.4848f,-17.2779f,-3.7372f,  -30.5894f,-17.5790f,-4.3020f,  -30.5186f,-18.6430f,-3.5740f,  
	-28.5563f,-18.7908f,-3.9341f,  -28.4750f,-19.8502f,-3.2013f,  -26.4801f,-19.8398f,-3.3514f,  -26.3875f,-20.8931f,-2.6128f,  -25.8055f,24.2460f,-36.5164f,  
	-25.8847f,23.4645f,-35.7019f,  -23.9720f,22.9001f,-35.5440f,  -24.0273f,22.1494f,-34.7444f,  -22.0834f,21.8148f,-34.4106f,  -22.1250f,21.0898f,-33.6197f,  
	-20.1935f,20.9846f,-33.1096f,  -20.2281f,20.2804f,-32.3234f,  -18.3516f,20.3989f,-31.6402f,  -18.3870f,19.7084f,-30.8546f,  -16.6064f,20.0401f,-30.0052f,  
	-16.6483f,19.3568f,-29.2183f,  -15.0017f,19.8869f,-28.2134f,  -15.0534f,19.2042f,-27.4231f,  -13.5755f,19.9139f,-26.2767f,  -13.6403f,19.2254f,-25.4828f,  
	-12.3618f,20.0931f,-24.2122f,  -12.4412f,19.3924f,-23.4141f,  -11.3879f,20.3940f,-22.0395f,  -11.4823f,19.6765f,-21.2380f,  -10.6750f,20.7858f,-19.7821f,  
	-10.7837f,20.0479f,-18.9784f,  -10.2382f,21.2379f,-17.4656f,  -10.3604f,20.4743f,-16.6605f,  -10.0867f,21.7173f,-15.1170f,  -10.2203f,20.9263f,-14.3122f,  
	-10.2228f,22.1947f,-12.7652f,  -10.3655f,21.3745f,-11.9618f,  -10.6432f,22.6413f,-10.4385f,  -10.7923f,21.7919f,-9.6380f,  -11.3391f,23.0310f,-8.1657f,  
	-11.4919f,22.1521f,-7.3692f,  -12.2964f,23.3392f,-5.9744f,  -12.4502f,22.4313f,-5.1827f,  -13.4965f,23.5440f,-3.8906f,  -13.6486f,22.6095f,-3.1051f,  
	-14.9162f,23.6273f,-1.9392f,  -15.0643f,22.6680f,-1.1602f,  -16.5292f,23.5730f,-0.1419f,  -16.6714f,22.5909f,0.6306f,  -18.3061f,23.3681f,1.4827f,  
	-18.4402f,22.3661f,2.2479f,  -20.2143f,23.0030f,2.9178f,  -20.3393f,21.9841f,3.6759f,  -22.2205f,22.4713f,4.1510f,  -22.3353f,21.4382f,4.9023f,  
	-24.2896f,21.7692f,5.1734f,  -24.3935f,20.7246f,5.9182f,  -26.3857f,20.8959f,5.9796f,  -26.4787f,19.8423f,6.7184f,  -28.4735f,19.8532f,6.5682f,  
	-28.5552f,18.7934f,7.3012f,  -30.5175f,18.6460f,6.9410f,  -30.5884f,17.5820f,7.6690f,  -32.4838f,17.2813f,7.1041f,  -32.5443f,16.2147f,7.8277f,  
	-34.3399f,15.7682f,7.0670f,  -34.3907f,14.7007f,7.7866f,  -36.0553f,14.1181f,6.8424f,  -36.0972f,13.0502f,7.5590f,  -37.6023f,12.3435f,6.4466f,  
	-37.6360f,11.2766f,7.1605f,  -38.9557f,10.4597f,5.8983f,  -38.9820f,9.3944f,6.6098f,  -40.0938f,8.4832f,5.2186f,  -40.1134f,7.4189f,5.9286f,  
	-40.9984f,6.4304f,4.4314f,  -41.0119f,5.3675f,5.1401f,  -41.6550f,4.3200f,3.5617f,  -41.6629f,3.2577f,4.2699f,  -42.0532f,2.1703f,2.6367f,  
	-42.0558f,1.1086f,3.3445f,  -42.1867f,0.0012f,1.6836f,  -42.1841f,-1.0602f,2.3912f,  -42.0532f,-2.1674f,0.7301f,  -42.0454f,-3.2302f,1.4386f,  
	-41.6551f,-4.3171f,-0.1949f,  -41.6418f,-5.3807f,0.5143f,  -40.9985f,-6.4274f,-1.0645f,  -40.9793f,-7.4929f,-0.3538f,  -40.0940f,-8.4802f,-1.8516f,  
	-40.0682f,-9.5472f,-1.1393f,  -38.9560f,-10.4567f,-2.5314f,  -38.9230f,-11.5258f,-1.8166f,  -37.6027f,-12.3402f,-3.0798f,  -37.5617f,-13.4111f,-2.3621f,  
	-36.0559f,-14.1145f,-3.4756f,  -36.0059f,-15.1865f,-2.7544f,  -34.3406f,-15.7649f,-3.7000f,  -34.2810f,-16.8367f,-2.9750f,  -32.4848f,-17.2779f,-3.7372f,  
	-32.4146f,-18.3488f,-3.0074f,  -30.5186f,-18.6430f,-3.5740f,  -30.4375f,-19.7108f,-2.8391f,  -28.4750f,-19.8502f,-3.2013f,  -28.3821f,-20.9133f,-2.4604f,  
	-26.3875f,-20.8931f,-2.6128f,  -26.2829f,-21.9489f,-1.8656f,  -25.8847f,23.4645f,-35.7019f,  -25.9521f,22.6725f,-34.8978f,  -24.0273f,22.1494f,-34.7444f,  
	-24.0752f,21.3868f,-33.9527f,  -22.1250f,21.0898f,-33.6197f,  -22.1605f,20.3523f,-32.8361f,  -20.2281f,20.2804f,-32.3234f,  -20.2595f,19.5616f,-31.5433f,  
	-18.3870f,19.7084f,-30.8546f,  -18.4208f,19.0021f,-30.0753f,  -16.6483f,19.3568f,-29.2183f,  -16.6885f,18.6566f,-28.4374f,  -15.0534f,19.2042f,-27.4231f,  
	-15.1041f,18.5035f,-26.6397f,  -13.6403f,19.2254f,-25.4828f,  -13.7038f,18.5179f,-24.6959f,  -12.4412f,19.3924f,-23.4141f,  -12.5184f,18.6731f,-22.6243f,  
	-11.4823f,19.6765f,-21.2380f,  -11.5732f,18.9411f,-20.4460f,  -10.7837f,20.0479f,-18.9784f,  -10.8883f,19.2908f,-18.1845f,  -10.3604f,20.4743f,-16.6605f,  
	-10.4771f,19.6931f,-15.8664f,  -10.2203f,20.9263f,-14.3122f,  -10.3470f,20.1187f,-13.5189f,  -10.3655f,21.3745f,-11.9618f,  -10.4999f,20.5398f,-11.1706f,  
	-10.7923f,21.7919f,-9.6380f,  -10.9319f,20.9290f,-8.8500f,  -11.4919f,22.1521f,-7.3692f,  -11.6342f,21.2611f,-6.5851f,  -12.4502f,22.4313f,-5.1827f,  
	-12.5925f,21.5142f,-4.4040f,  -13.6486f,22.6095f,-3.1051f,  -13.7885f,21.6674f,-2.3321f,  -15.0643f,22.6680f,-1.1602f,  -15.2000f,21.7026f,-0.3931f,  
	-16.6714f,22.5909f,0.6306f,  -16.8005f,21.6048f,1.3911f,  -18.4402f,22.3661f,2.2479f,  -18.5615f,21.3619f,3.0019f,  -20.3393f,21.9841f,3.6759f,  
	-20.4516f,20.9645f,4.4235f,  -22.3353f,21.4382f,4.9023f,  -22.4377f,20.4057f,5.6439f,  -24.3935f,20.7246f,5.9182f,  -24.4856f,19.6818f,6.6541f,  
	-26.4787f,19.8423f,6.7184f,  -26.5601f,18.7918f,7.4488f,  -28.5552f,18.7934f,7.3012f,  -28.6262f,17.7372f,8.0268f,  -30.5884f,17.5820f,7.6690f,  
	-30.6493f,16.5220f,8.3904f,  -32.5443f,16.2147f,7.8277f,  -32.5956f,15.1527f,8.5453f,  -34.3907f,14.7007f,7.7866f,  -34.4331f,13.6373f,8.5014f,  
	-36.0972f,13.0502f,7.5590f,  -36.1314f,11.9868f,8.2713f,  -37.6360f,11.2766f,7.1605f,  -37.6626f,10.2141f,7.8706f,  -38.9820f,9.3944f,6.6098f,  
	-39.0018f,8.3324f,7.3185f,  -40.1134f,7.4189f,5.9286f,  -40.1271f,6.3578f,6.6362f,  -41.0119f,5.3675f,5.1401f,  -41.0199f,4.3066f,5.8474f,  
	-41.6629f,3.2577f,4.2699f,  -41.6655f,2.1970f,4.9770f,  -42.0558f,1.1086f,3.3445f,  -42.0533f,0.0479f,4.0516f,  -42.1841f,-1.0602f,2.3912f,  
	-42.1763f,-2.1225f,3.0994f,  -42.0454f,-3.2302f,1.4386f,  -42.0323f,-4.2937f,2.1478f,  -41.6418f,-5.3807f,0.5143f,  -41.6229f,-6.4465f,1.2251f,  
	-40.9793f,-7.4929f,-0.3538f,  -40.9540f,-8.5607f,0.3588f,  -40.0682f,-9.5472f,-1.1393f,  -40.0359f,-10.6178f,-0.4241f,  -38.9230f,-11.5258f,-1.8166f,  
	-38.8828f,-12.5991f,-1.0982f,  -37.5617f,-13.4111f,-2.3621f,  -37.5126f,-14.4864f,-1.6401f,  -36.0059f,-15.1865f,-2.7544f,  -35.9473f,-16.2627f,-2.0285f,  
	-34.2810f,-16.8367f,-2.9750f,  -34.2117f,-17.9132f,-2.2440f,  -32.4146f,-18.3488f,-3.0074f,  -32.3341f,-19.4238f,-2.2710f,  -30.4375f,-19.7108f,-2.8391f,  
	-30.3449f,-20.7826f,-2.0965f,  -28.3821f,-20.9133f,-2.4604f,  -28.2773f,-21.9795f,-1.7111f,  -26.2829f,-21.9489f,-1.8656f,  -26.1657f,-23.0069f,-1.1091f,  
	-25.9521f,22.6725f,-34.8978f,  -26.0105f,21.8705f,-34.1030f,  -24.0752f,21.3868f,-33.9527f,  -24.1157f,20.6135f,-33.1695f,  -22.1605f,20.3523f,-32.8361f,  
	-22.1919f,19.6022f,-32.0594f,  -20.2595f,19.5616f,-31.5433f,  -20.2889f,18.8288f,-30.7697f,  -18.4208f,19.0021f,-30.0753f,  -18.4528f,18.2806f,-29.3022f,  
	-16.6885f,18.6566f,-28.4374f,  -16.7279f,17.9399f,-27.6632f,  -15.1041f,18.5035f,-26.6397f,  -15.1540f,17.7852f,-25.8631f,  -13.7038f,18.5179f,-24.6959f,  
	-13.7658f,17.7927f,-23.9170f,  -12.5184f,18.6731f,-22.6243f,  -12.5931f,17.9365f,-21.8433f,  -11.5732f,18.9411f,-20.4460f,  -11.6613f,18.1870f,-19.6631f,  
	-10.8883f,19.2908f,-18.1845f,  -10.9886f,18.5162f,-17.4011f,  -10.4771f,19.6931f,-15.8664f,  -10.5881f,18.8952f,-15.0832f,  -10.3470f,20.1187f,-13.5189f,  
	-10.4667f,19.2961f,-12.7372f,  -10.4999f,20.5398f,-11.1706f,  -10.6261f,19.6909f,-10.3913f,  -10.9319f,20.9290f,-8.8500f,  -11.0623f,20.0534f,-8.0739f,  
	-11.6342f,21.2611f,-6.5851f,  -11.7661f,20.3601f,-5.8137f,  -12.5925f,21.5142f,-4.4040f,  -12.7237f,20.5885f,-3.6376f,  -13.7885f,21.6674f,-2.3321f,  
	-13.9170f,20.7183f,-1.5709f,  -15.2000f,21.7026f,-0.3931f,  -15.3234f,20.7321f,0.3622f,  -16.8005f,21.6048f,1.3911f,  -16.9175f,20.6154f,2.1404f,  
	-18.5615f,21.3619f,3.0019f,  -18.6706f,20.3562f,3.7452f,  -20.4516f,20.9645f,4.4235f,  -20.5518f,19.9446f,5.1612f,  -22.4377f,20.4057f,5.6439f,  
	-22.5285f,19.3742f,6.3762f,  -24.4856f,19.6818f,6.6541f,  -24.5664f,18.6413f,7.3813f,  -26.5601f,18.7918f,7.4488f,  -26.6310f,17.7444f,8.1715f,  
	-28.6262f,17.7372f,8.0268f,  -28.6873f,16.6848f,8.7455f,  -30.6493f,16.5220f,8.3904f,  -30.7009f,15.4663f,9.1055f,  -32.5956f,15.1527f,8.5453f,  
	-32.6384f,14.0947f,9.2578f,  -34.4331f,13.6373f,8.5014f,  -34.4676f,12.5784f,9.2115f,  -36.1314f,11.9868f,8.2713f,  -36.1583f,10.9280f,8.9793f,  
	-37.6626f,10.2141f,7.8706f,  -37.6827f,9.1552f,8.5775f,  -39.0018f,8.3324f,7.3185f,  -39.0156f,7.2739f,8.0246f,  -40.1271f,6.3578f,6.6362f,  
	-40.1351f,5.2991f,7.3421f,  -41.0199f,4.3066f,5.8474f,  -41.0225f,3.2478f,6.5532f,  -41.6655f,2.1970f,4.9770f,  -41.6630f,1.1378f,5.6831f,  
	-42.0533f,0.0479f,4.0516f,  -42.0456f,-1.0132f,4.7590f,  -42.1763f,-2.1225f,3.0994f,  -42.1634f,-3.1851f,3.8079f,  -42.0323f,-4.2937f,2.1478f,  
	-42.0137f,-5.3589f,2.8581f,  -41.6229f,-6.4465f,1.2251f,  -41.5981f,-7.5142f,1.9374f,  -40.9540f,-8.5607f,0.3588f,  -40.9224f,-9.6319f,1.0739f,  
	-40.0359f,-10.6178f,-0.4241f,  -39.9967f,-11.6924f,0.2944f,  -38.8828f,-12.5991f,-1.0982f,  -38.8349f,-13.6766f,-0.3760f,  -37.5126f,-14.4864f,-1.6401f,  
	-37.4552f,-15.5658f,-0.9137f,  -35.9473f,-16.2627f,-2.0285f,  -35.8792f,-17.3438f,-1.2969f,  -34.2117f,-17.9132f,-2.2440f,  -34.1323f,-18.9943f,-1.5068f,  
	-32.3341f,-19.4238f,-2.2710f,  -32.2423f,-20.5033f,-1.5274f,  -30.3449f,-20.7826f,-2.0965f,  -30.2404f,-21.8581f,-1.3459f,  -28.2773f,-21.9795f,-1.7111f,  
	-28.1596f,-23.0486f,-0.9529f,  -26.1657f,-23.0069f,-1.1091f,  -26.0346f,-24.0671f,-0.3425f,  -26.0105f,21.8705f,-34.1030f,  -26.0602f,21.0594f,-33.3179f,  
	-24.1157f,20.6135f,-33.1695f,  -24.1511f,19.8294f,-32.3941f,  -22.1919f,19.6022f,-32.0594f,  -22.2207f,18.8398f,-31.2896f,  -20.2889f,18.8288f,-30.7697f,  
	-20.3162f,18.0825f,-30.0027f,  -18.4528f,18.2806f,-29.3022f,  -18.4841f,17.5442f,-28.5357f,  -16.7279f,17.9399f,-27.6632f,  -16.7669f,17.2070f,-26.8957f,  
	-15.1540f,17.7852f,-25.8631f,  -15.2030f,17.0504f,-25.0941f,  -13.7658f,17.7927f,-23.9170f,  -13.8261f,17.0511f,-23.1464f,  -12.5931f,17.9365f,-21.8433f,  
	-12.6661f,17.1818f,-21.0710f,  -11.6613f,18.1870f,-19.6631f,  -11.7462f,17.4157f,-18.8901f,  -10.9886f,18.5162f,-17.4011f,  -11.0846f,17.7250f,-16.6280f,  
	-10.5881f,18.8952f,-15.0832f,  -10.6934f,18.0821f,-14.3111f,  -10.4667f,19.2961f,-12.7372f,  -10.5796f,18.4590f,-11.9669f,  -10.6261f,19.6909f,-10.3913f,  
	-10.7443f,18.8288f,-9.6235f,  -11.0623f,20.0534f,-8.0739f,  -11.1835f,19.1670f,-7.3101f,  -11.7661f,20.3601f,-5.8137f,  -11.8881f,19.4497f,-5.0543f,  
	-12.7237f,20.5885f,-3.6376f,  -12.8445f,19.6549f,-2.8827f,  -13.9170f,20.7183f,-1.5709f,  -14.0342f,19.7630f,-0.8214f,  -15.3234f,20.7321f,0.3622f,  
	-15.4354f,19.7574f,1.1063f,  -16.9175f,20.6154f,2.1404f,  -17.0228f,19.6237f,2.8789f,  -18.6706f,20.3562f,3.7452f,  -18.7681f,19.3492f,4.4786f,  
	-20.5518f,19.9446f,5.1612f,  -20.6408f,18.9250f,5.8896f,  -22.5285f,19.3742f,6.3762f,  -22.6082f,18.3444f,7.0997f,  -24.5664f,18.6413f,7.3813f,  
	-24.6367f,17.6033f,8.1005f,  -26.6310f,17.7444f,8.1715f,  -26.6918f,16.7003f,8.8869f,  -28.6873f,16.6848f,8.7455f,  -28.7390f,15.6363f,9.4577f,  
	-30.7009f,15.4663f,9.1055f,  -30.7439f,14.4146f,9.8151f,  -32.6384f,14.0947f,9.2578f,  -32.6732f,13.0412f,9.9652f,  -34.4676f,12.5784f,9.2115f,  
	-34.4948f,11.5242f,9.9171f,  -36.1583f,10.9280f,8.9793f,  -36.1785f,9.8731f,9.6839f,  -37.6827f,9.1552f,8.5775f,  -37.6965f,8.1001f,9.2814f,  
	-39.0156f,7.2739f,8.0246f,  -39.0236f,6.2182f,8.7285f,  -40.1351f,5.2991f,7.3421f,  -40.1377f,4.2430f,8.0461f,  -41.0225f,3.2478f,6.5532f,  
	-41.0200f,2.1910f,7.2577f,  -41.6630f,1.1378f,5.6831f,  -41.6554f,0.0789f,6.3891f,  -42.0456f,-1.0132f,4.7590f,  -42.0329f,-2.0739f,5.4662f,  
	-42.1634f,-3.1851f,3.8079f,  -42.1452f,-4.2488f,4.5171f,  -42.0137f,-5.3589f,2.8581f,  -41.9896f,-6.4255f,3.5695f,  -41.5981f,-7.5142f,1.9374f,  
	-41.5674f,-8.5848f,2.6518f,  -40.9224f,-9.6319f,1.0739f,  -40.8842f,-10.7066f,1.7919f,  -39.9967f,-11.6924f,0.2944f,  -39.9500f,-12.7709f,1.0162f,  
	-38.8349f,-13.6766f,-0.3760f,  -38.7789f,-14.7580f,0.3502f,  -37.4552f,-15.5658f,-0.9137f,  -37.3887f,-16.6502f,-0.1822f,  -35.8792f,-17.3438f,-1.2969f,  
	-35.8014f,-18.4296f,-0.5595f,  -34.1323f,-18.9943f,-1.5068f,  -34.0419f,-20.0801f,-0.7628f,  -32.2423f,-20.5033f,-1.5274f,  -32.1388f,-21.5868f,-0.7762f,  
	-30.2404f,-21.8581f,-1.3459f,  -30.1232f,-22.9372f,-0.5867f,  -28.1596f,-23.0486f,-0.9529f,  -28.0282f,-24.1207f,-0.1850f,  -26.0346f,-24.0671f,-0.3425f,  
	-25.8895f,-25.1285f,0.4343f,  -26.0602f,21.0594f,-33.3179f,  -26.1035f,20.2395f,-32.5416f,  -24.1511f,19.8294f,-32.3941f,  -24.1831f,19.0350f,-31.6261f,  
	-22.2207f,18.8398f,-31.2896f,  -22.2468f,18.0658f,-30.5268f,  -20.3162f,18.0825f,-30.0027f,  -20.3426f,17.3231f,-29.2424f,  -18.4841f,17.5442f,-28.5357f,  
	-18.5150f,16.7932f,-27.7760f,  -16.7669f,17.2070f,-26.8957f,  -16.8054f,16.4589f,-26.1355f,  -15.2030f,17.0504f,-25.0941f,  -15.2509f,16.3002f,-24.3331f,  
	-13.8261f,17.0511f,-23.1464f,  -13.8855f,16.2923f,-22.3840f,  -12.6661f,17.1818f,-21.0710f,  -12.7370f,16.4105f,-20.3079f,  -11.7462f,17.4157f,-18.8901f,  
	-11.8281f,16.6281f,-18.1267f,  -11.0846f,17.7250f,-16.6280f,  -11.1762f,16.9187f,-15.8652f,  -10.6934f,18.0821f,-14.3111f,  -10.7933f,17.2544f,-13.5497f,  
	-10.5796f,18.4590f,-11.9669f,  -10.6858f,17.6082f,-11.2075f,  -10.7443f,18.8288f,-9.6235f,  -10.8547f,17.9552f,-8.8675f,  -11.1835f,19.1670f,-7.3101f,  
	-11.2960f,18.2704f,-6.5579f,  -11.8881f,19.4497f,-5.0543f,  -12.0006f,18.5305f,-4.3061f,  -12.8445f,19.6549f,-2.8827f,  -12.9548f,18.7141f,-2.1392f,  
	-14.0342f,19.7630f,-0.8214f,  -14.1406f,18.8026f,-0.0828f,  -15.4354f,19.7574f,1.1063f,  -15.5364f,18.7796f,1.8396f,  -17.0228f,19.6237f,2.8789f,  
	-17.1171f,18.6296f,3.6076f,  -18.7681f,19.3492f,4.4786f,  -18.8548f,18.3417f,5.2025f,  -20.6408f,18.9250f,5.8896f,  -20.7189f,17.9063f,6.6089f,  
	-22.6082f,18.3444f,7.0997f,  -22.6775f,17.3164f,7.8151f,  -24.6367f,17.6033f,8.1005f,  -24.6971f,16.5681f,8.8123f,  -26.6918f,16.7003f,8.8869f,  
	-26.7434f,15.6596f,9.5957f,  -28.7390f,15.6363f,9.4577f,  -28.7820f,14.5915f,10.1640f,  -30.7439f,14.4146f,9.8151f,  -30.7788f,13.3672f,10.5194f,  
	-32.6732f,13.0412f,9.9652f,  -32.7005f,11.9923f,10.6678f,  -34.4948f,11.5242f,9.9171f,  -34.5151f,10.4740f,10.6189f,  -36.1785f,9.8731f,9.6839f,  
	-36.1924f,8.8223f,10.3852f,  -37.6965f,8.1001f,9.2814f,  -37.7046f,7.0483f,9.9828f,  -39.0236f,6.2182f,8.7285f,  -39.0262f,5.1656f,9.4302f,  
	-40.1377f,4.2430f,8.0461f,  -40.1352f,3.1895f,8.7485f,  -41.0200f,2.1910f,7.2577f,  -41.0125f,1.1352f,7.9617f,  -41.6554f,0.0789f,6.3891f,  
	-41.6429f,-0.9790f,7.0945f,  -42.0329f,-2.0739f,5.4662f,  -42.0150f,-3.1351f,6.1739f,  -42.1452f,-4.2488f,4.5171f,  -42.1217f,-5.3133f,5.2271f,  
	-41.9896f,-6.4255f,3.5695f,  -41.9597f,-7.4944f,4.2825f,  -41.5674f,-8.5848f,2.6518f,  -41.5303f,-9.6585f,3.3687f,  -40.8842f,-10.7066f,1.7919f,  
	-40.8391f,-11.7848f,2.5127f,  -39.9500f,-12.7709f,1.0162f,  -39.8958f,-13.8530f,1.7415f,  -38.7789f,-14.7580f,0.3502f,  -38.7143f,-15.8443f,1.0809f,  
	-37.3887f,-16.6502f,-0.1822f,  -37.3127f,-17.7393f,0.5547f,  -35.8014f,-18.4296f,-0.5595f,  -35.7128f,-19.5203f,0.1842f,  -34.0419f,-20.0801f,-0.7628f,  
	-33.9400f,-21.1703f,-0.0117f,  -32.1388f,-21.5868f,-0.7762f,  -32.0228f,-22.6745f,-0.0168f,  -30.1232f,-22.9372f,-0.5867f,  -29.9922f,-24.0200f,0.1818f,  
	-28.0282f,-24.1207f,-0.1850f,  -27.8827f,-25.1949f,0.5928f,  -25.8895f,-25.1285f,0.4343f,  -25.7291f,-26.1913f,1.2222f,  -26.1035f,20.2395f,-32.5416f,  
	-26.1423f,19.4110f,-31.7735f,  -24.1831f,19.0350f,-31.6261f,  -24.2115f,18.2308f,-30.8658f,  -22.2468f,18.0658f,-30.5268f,  -22.2715f,17.2804f,-29.7711f,  
	-20.3426f,17.3231f,-29.2424f,  -20.3684f,16.5507f,-28.4889f,  -18.5150f,16.7932f,-27.7760f,  -18.5457f,16.0283f,-27.0233f,  -16.8054f,16.4589f,-26.1355f,  
	-16.8433f,15.6967f,-25.3830f,  -15.2509f,16.3002f,-24.3331f,  -15.2987f,15.5339f,-23.5797f,  -13.8855f,16.2923f,-22.3840f,  -13.9438f,15.5177f,-21.6302f,  
	-12.7370f,16.4105f,-20.3079f,  -12.8058f,15.6233f,-19.5538f,  -11.8281f,16.6281f,-18.1267f,  -11.9066f,15.8256f,-17.3731f,  -11.1762f,16.9187f,-15.8652f,  
	-11.2634f,16.0976f,-15.1127f,  -10.7933f,17.2544f,-13.5497f,  -10.8876f,16.4127f,-12.7987f,  -10.6858f,17.6082f,-11.2075f,  -10.7853f,16.7456f,-10.4594f,  
	-10.8547f,17.9552f,-8.8675f,  -10.9575f,17.0709f,-8.1228f,  -11.2960f,18.2704f,-6.5579f,  -11.4000f,17.3645f,-5.8166f,  -12.0006f,18.5305f,-4.3061f,  
	-12.1038f,17.6032f,-3.5689f,  -12.9548f,18.7141f,-2.1392f,  -13.0553f,17.7673f,-1.4066f,  -14.1406f,18.8026f,-0.0828f,  -14.2369f,17.8382f,0.6450f,  
	-15.5364f,18.7796f,1.8396f,  -15.6271f,17.7983f,2.5632f,  -17.1171f,18.6296f,3.6076f,  -17.2010f,17.6344f,4.3268f,  -18.8548f,18.3417f,5.2025f,  
	-18.9310f,17.3343f,5.9174f,  -20.7189f,17.9063f,6.6089f,  -20.7870f,16.8886f,7.3201f,  -22.6775f,17.3164f,7.8151f,  -22.7371f,16.2906f,8.5228f,  
	-24.6971f,16.5681f,8.8123f,  -24.7482f,15.5359f,9.5172f,  -26.7434f,15.6596f,9.5957f,  -26.7862f,14.6224f,10.2983f,  -28.7820f,14.5915f,10.1640f,  
	-28.8169f,13.5509f,10.8646f,  -30.7788f,13.3672f,10.5194f,  -30.8062f,12.3243f,11.2186f,  -32.7005f,11.9923f,10.6678f,  -32.7209f,10.9475f,11.3663f,  
	-34.5151f,10.4740f,10.6189f,  -34.5291f,9.4280f,11.3170f,  -36.1924f,8.8223f,10.3852f,  -36.2005f,7.7750f,11.0836f,  -37.7046f,7.0483f,9.9828f,  
	-37.7072f,5.9999f,10.6817f,  -39.0262f,5.1656f,9.4302f,  -39.0238f,4.1161f,10.1299f,  -40.1352f,3.1895f,8.7485f,  -40.1278f,2.1374f,9.4500f,  
	-41.0125f,1.1352f,7.9617f,  -41.0001f,0.0808f,8.6648f,  -41.6429f,-0.9790f,7.0945f,  -41.6254f,-2.0369f,7.8000f,  -42.0150f,-3.1351f,6.1739f,  
	-41.9921f,-4.1965f,6.8818f,  -42.1217f,-5.3133f,5.2271f,  -42.0927f,-6.3795f,5.9383f,  -41.9597f,-7.4944f,4.2825f,  -41.9239f,-8.5661f,4.9977f,  
	-41.5303f,-9.6585f,3.3687f,  -41.4867f,-10.7353f,4.0879f,  -40.8391f,-11.7848f,2.5127f,  -40.7866f,-12.8664f,3.2365f,  -39.8958f,-13.8530f,1.7415f,  
	-39.8332f,-14.9399f,2.4709f,  -38.7143f,-15.8443f,1.0809f,  -38.6406f,-16.9354f,1.8167f,  -37.3127f,-17.7393f,0.5547f,  -37.2265f,-18.8334f,1.2973f,  
	-35.7128f,-19.5203f,0.1842f,  -35.6131f,-20.6156f,0.9344f,  -33.9400f,-21.1703f,-0.0117f,  -33.8258f,-22.2651f,0.7471f,  -32.0228f,-22.6745f,-0.0168f,  
	-31.8931f,-23.7665f,0.7514f,  -29.9922f,-24.0200f,0.1818f,  -29.8472f,-25.1056f,0.9597f,  -27.8827f,-25.1949f,0.5928f,  -27.7217f,-26.2713f,1.3813f,  
	-25.7291f,-26.1913f,1.2222f,  -25.5528f,-27.2547f,2.0216f,  -26.1423f,19.4110f,-31.7735f,  -26.1766f,18.5746f,-31.0137f,  -24.2115f,18.2308f,-30.8658f,  
	-24.2379f,17.4169f,-30.1128f,  -22.2715f,17.2804f,-29.7711f,  -22.2954f,16.4839f,-29.0222f,  -20.3684f,16.5507f,-28.4889f,  -20.3940f,15.7661f,-27.7424f,  
	-18.5457f,16.0283f,-27.0233f,  -18.5759f,15.2507f,-26.2781f,  -16.8433f,15.6967f,-25.3830f,  -16.8815f,14.9194f,-24.6376f,  -15.2987f,15.5339f,-23.5797f,  
	-15.3462f,14.7526f,-22.8344f,  -13.9438f,15.5177f,-21.6302f,  -14.0011f,14.7278f,-20.8848f,  -12.8058f,15.6233f,-19.5538f,  -12.8724f,14.8216f,-18.8089f,  
	-11.9066f,15.8256f,-17.3731f,  -11.9820f,15.0085f,-16.6291f,  -11.2634f,16.0976f,-15.1127f,  -11.3464f,15.2625f,-14.3699f,  -10.8876f,16.4127f,-12.7987f,  
	-10.9765f,15.5588f,-12.0585f,  -10.7853f,16.7456f,-10.4594f,  -10.8784f,15.8717f,-9.7221f,  -10.9575f,17.0709f,-8.1228f,  -11.0528f,16.1766f,-7.3884f,  
	-11.4000f,17.3645f,-5.8166f,  -11.4957f,16.4495f,-5.0859f,  -12.1038f,17.6032f,-3.5689f,  -12.1980f,16.6692f,-2.8423f,  -13.0553f,17.7673f,-1.4066f,  
	-13.1464f,16.8157f,-0.6845f,  -14.2369f,17.8382f,0.6450f,  -14.3234f,16.8694f,1.3633f,  -15.6271f,17.7983f,2.5632f,  -15.7078f,16.8152f,3.2773f,  
	-17.2010f,17.6344f,4.3268f,  -17.2749f,16.6384f,5.0369f,  -18.9310f,17.3343f,5.9174f,  -18.9974f,16.3272f,6.6241f,  -20.7870f,16.8886f,7.3201f,  
	-20.8456f,15.8726f,8.0235f,  -22.7371f,16.2906f,8.5228f,  -22.7877f,15.2672f,9.2236f,  -24.7482f,15.5359f,9.5172f,  -24.7908f,14.5068f,10.2158f,  
	-26.7862f,14.6224f,10.2983f,  -26.8210f,13.5892f,10.9950f,  -28.8169f,13.5509f,10.8646f,  -28.8442f,12.5146f,11.5602f,  -30.8062f,12.3243f,11.2186f,  
	-30.8266f,11.2856f,11.9135f,  -32.7209f,10.9475f,11.3663f,  -32.7349f,9.9071f,12.0610f,  -34.5291f,9.4280f,11.3170f,  -34.5371f,8.3859f,12.0120f,  
	-36.2005f,7.7750f,11.0836f,  -36.2031f,6.7315f,11.7793f,  -37.7072f,5.9999f,10.6817f,  -37.7047f,4.9549f,11.3784f,  -39.0238f,4.1161f,10.1299f,  
	-39.0164f,3.0685f,10.8285f,  -40.1278f,2.1374f,9.4500f,  -40.1156f,1.0873f,10.1503f,  -41.0001f,0.0808f,8.6648f,  -40.9831f,-0.9729f,9.3676f,  
	-41.6254f,-2.0369f,7.8000f,  -41.6032f,-3.0944f,8.5054f,  -41.9921f,-4.1965f,6.8818f,  -41.9641f,-5.2591f,7.5906f,  -42.0927f,-6.3795f,5.9383f,  
	-42.0582f,-7.4480f,6.6512f,  -41.9239f,-8.5661f,4.9977f,  -41.8819f,-9.6403f,5.7148f,  -41.4867f,-10.7353f,4.0879f,  -41.4363f,-11.8151f,4.8096f,  
	-40.7866f,-12.8664f,3.2365f,  -40.7264f,-13.9524f,3.9640f,  -39.8332f,-14.9399f,2.4709f,  -39.7621f,-16.0314f,3.2049f,  -38.6406f,-16.9354f,1.8167f,  
	-38.5571f,-18.0313f,2.5576f,  -37.2265f,-18.8334f,1.2973f,  -37.1295f,-19.9323f,2.0460f,  -35.6131f,-20.6156f,0.9344f,  -35.5015f,-21.7160f,1.6920f,  
	-33.8258f,-22.2651f,0.7471f,  -33.6984f,-23.3647f,1.5142f,  -31.8931f,-23.7665f,0.7514f,  -31.7496f,-24.8621f,1.5286f,  -29.8472f,-25.1056f,0.9597f,  
	-29.6869f,-26.1942f,1.7479f,  -27.7217f,-26.2713f,1.3813f,  -27.5447f,-27.3493f,2.1810f,  -25.5528f,-27.2547f,2.0216f,  -25.3595f,-28.3183f,2.8333f,  
	-26.1766f,18.5746f,-31.0137f,  -26.2081f,17.7303f,-30.2616f,  -24.2379f,17.4169f,-30.1128f,  -24.2630f,16.5937f,-29.3669f,  -22.2954f,16.4839f,-29.0222f,  
	-22.3187f,15.6766f,-28.2804f,  -20.3940f,15.7661f,-27.7424f,  -20.4192f,14.9701f,-27.0031f,  -18.5759f,15.2507f,-26.2781f,  -18.6069f,14.4595f,-25.5397f,  
	-16.8815f,14.9194f,-24.6376f,  -16.9201f,14.1283f,-23.8999f,  -15.3462f,14.7526f,-22.8344f,  -15.3933f,13.9568f,-22.0969f,  -14.0011f,14.7278f,-20.8848f,  
	-14.0568f,13.9239f,-20.1480f,  -12.8724f,14.8216f,-18.8089f,  -12.9368f,14.0057f,-18.0730f,  -11.9820f,15.0085f,-16.6291f,  -12.0542f,14.1774f,-15.8943f,  
	-11.3464f,15.2625f,-14.3699f,  -11.4250f,14.4151f,-13.6374f,  -10.9765f,15.5588f,-12.0585f,  -11.0600f,14.6933f,-11.3284f,  -10.8784f,15.8717f,-9.7221f,  
	-10.9651f,14.9874f,-8.9947f,  -11.0528f,16.1766f,-7.3884f,  -11.1410f,15.2727f,-6.6643f,  -11.4957f,16.4495f,-5.0859f,  -11.5834f,15.5271f,-4.3655f,  
	-12.1980f,16.6692f,-2.8423f,  -12.2837f,15.7296f,-2.1261f,  -13.1464f,16.8157f,-0.6845f,  -13.2286f,15.8589f,0.0284f,  -14.3234f,16.8694f,1.3633f,  
	-14.4007f,15.8980f,2.0722f,  -15.7078f,16.8152f,3.2773f,  -15.7791f,15.8304f,3.9825f,  -17.2749f,16.6384f,5.0369f,  -17.3395f,15.6419f,5.7388f,  
	-18.9974f,16.3272f,6.6241f,  -19.0547f,15.3211f,7.3229f,  -20.8456f,15.8726f,8.0235f,  -20.8953f,14.8583f,8.7199f,  -22.7877f,15.2672f,9.2236f,  
	-22.8297f,14.2467f,9.9178f,  -24.7908f,14.5068f,10.2158f,  -24.8252f,13.4813f,10.9083f,  -26.8210f,13.5892f,10.9950f,  -26.8482f,12.5599f,11.6865f,  
	-28.8442f,12.5146f,11.5602f,  -28.8646f,11.4824f,12.2511f,  -30.8266f,11.2856f,11.9135f,  -30.8406f,10.2512f,12.6043f,  -32.7349f,9.9071f,12.0610f,  
	-32.7429f,8.8707f,12.7522f,  -34.5371f,8.3859f,12.0120f,  -34.5398f,7.3479f,12.7041f,  -36.2031f,6.7315f,11.7793f,  -36.2007f,5.6917f,12.4725f,  
	-37.7047f,4.9549f,11.3784f,  -37.6975f,3.9125f,12.0735f,  -39.0164f,3.0685f,10.8285f,  -39.0045f,2.0234f,11.5255f,  -40.1156f,1.0873f,10.1503f,  
	-40.0990f,0.0385f,10.8500f,  -40.9831f,-0.9729f,9.3676f,  -40.9615f,-2.0257f,10.0701f,  -41.6032f,-3.0944f,8.5054f,  -41.5761f,-4.1524f,9.2114f,  
	-41.9641f,-5.2591f,7.5906f,  -41.9309f,-6.3234f,8.3008f,  -42.0582f,-7.4480f,6.6512f,  -42.0179f,-8.5184f,7.3656f,  -41.8819f,-9.6403f,5.7148f,  
	-41.8336f,-10.7170f,6.4339f,  -41.4363f,-11.8151f,4.8096f,  -41.3786f,-12.8989f,5.5346f,  -40.7264f,-13.9524f,3.9640f,  -40.6581f,-15.0429f,4.6955f,  
	-39.7621f,-16.0314f,3.2049f,  -39.6817f,-17.1276f,3.9434f,  -38.5571f,-18.0313f,2.5576f,  -38.4634f,-19.1321f,3.3040f,  -37.1295f,-19.9323f,2.0460f,  
	-37.0211f,-21.0365f,2.8015f,  -35.5015f,-21.7160f,1.6920f,  -35.3770f,-22.8215f,2.4571f,  -33.6984f,-23.3647f,1.5142f,  -33.5573f,-24.4685f,2.2899f,  
	-31.7496f,-24.8621f,1.5286f,  -31.5909f,-25.9613f,2.3156f,  -29.6869f,-26.1942f,1.7479f,  -29.5105f,-27.2853f,2.5468f,  -27.5447f,-27.3493f,2.1810f,  
	-27.3506f,-28.4287f,2.9926f,  -25.3595f,-28.3183f,2.8333f,  -25.1489f,-29.3812f,3.6576f,  -26.2081f,17.7303f,-30.2616f,  -26.2374f,16.8784f,-29.5170f,  
	-24.2630f,16.5937f,-29.3669f,  -24.2870f,15.7615f,-28.6281f,  -22.3187f,15.6766f,-28.2804f,  -22.3416f,14.8595f,-27.5457f,  -20.4192f,14.9701f,-27.0031f,  
	-20.4453f,14.1619f,-26.2704f,  -18.6069f,14.4595f,-25.5397f,  -18.6385f,13.6555f,-24.8084f,  -16.9201f,14.1283f,-23.8999f,  -16.9589f,13.3237f,-23.1695f,  
	-15.3933f,13.9568f,-22.0969f,  -15.4398f,13.1479f,-21.3676f,  -14.0568f,13.9239f,-20.1480f,  -14.1114f,13.1063f,-19.4197f,  -12.9368f,14.0057f,-18.0730f,  
	-12.9991f,13.1760f,-17.3457f,  -12.0542f,14.1774f,-15.8943f,  -12.1231f,13.3340f,-15.1691f,  -11.4250f,14.4151f,-13.6374f,  -11.4993f,13.5559f,-12.9144f,  
	-11.0600f,14.6933f,-11.3284f,  -11.1382f,13.8171f,-10.6079f,  -10.9651f,14.9874f,-8.9947f,  -11.0456f,14.0928f,-8.2770f,  -11.1410f,15.2727f,-6.6643f,  
	-11.2221f,14.3606f,-5.9500f,  -11.5834f,15.5271f,-4.3655f,  -11.6635f,14.5984f,-3.6551f,  -12.2837f,15.7296f,-2.1261f,  -12.3612f,14.7839f,-1.4188f,  
	-13.2286f,15.8589f,0.0284f,  -13.3021f,14.8986f,0.7319f,  -14.4007f,15.8980f,2.0722f,  -14.4691f,14.9241f,2.7723f,  -15.7791f,15.8304f,3.9825f,  
	-15.8416f,14.8443f,4.6796f,  -17.3395f,15.6419f,5.7388f,  -17.3953f,14.6456f,6.4329f,  -19.0547f,15.3211f,7.3229f,  -19.1034f,14.3161f,8.0147f,  
	-20.8953f,14.8583f,8.7199f,  -20.9367f,13.8464f,9.4096f,  -22.8297f,14.2467f,9.9178f,  -22.8638f,13.2292f,10.6060f,  -24.8252f,13.4813f,10.9083f,  
	-24.8523f,12.4594f,11.5956f,  -26.8482f,12.5599f,11.6865f,  -26.8685f,11.5346f,12.3732f,  -28.8646f,11.4824f,12.2511f,  -28.8785f,10.4545f,12.9378f,  
	-30.8406f,10.2512f,12.6043f,  -30.8486f,9.2210f,13.2916f,  -32.7429f,8.8707f,12.7522f,  -32.7455f,7.8386f,13.4404f,  -34.5398f,7.3479f,12.7041f,  
	-34.5373f,6.3139f,13.3935f,  -36.2007f,5.6917f,12.4725f,  -36.1935f,4.6550f,13.1639f,  -37.6975f,3.9125f,12.0735f,  -37.6858f,2.8730f,12.7670f,  
	-39.0045f,2.0234f,11.5255f,  -38.9882f,0.9802f,12.2217f,  -40.0990f,0.0385f,10.8500f,  -40.0780f,-1.0087f,11.5490f,  -40.9615f,-2.0257f,10.0701f,  
	-40.9353f,-3.0783f,10.7727f,  -41.5761f,-4.1524f,9.2114f,  -41.5442f,-5.2115f,9.9183f,  -41.9309f,-6.3234f,8.3008f,  -41.8924f,-7.3890f,9.0120f,  
	-42.0179f,-8.5184f,7.3656f,  -41.9718f,-9.5908f,8.0816f,  -41.8336f,-10.7170f,6.4339f,  -41.7786f,-11.7972f,7.1558f,  -41.3786f,-12.8989f,5.5346f,  
	-41.3134f,-13.9868f,6.2632f,  -40.6581f,-15.0429f,4.6955f,  -40.5813f,-16.1378f,5.4311f,  -39.6817f,-17.1276f,3.9434f,  -39.5917f,-18.2286f,4.6870f,  
	-38.4634f,-19.1321f,3.3040f,  -38.3587f,-20.2384f,4.0568f,  -37.0211f,-21.0365f,2.8015f,  -36.9004f,-22.1461f,3.5641f,  -35.3770f,-22.8215f,2.4571f,  
	-35.2393f,-23.9316f,3.2305f,  -33.5573f,-24.4685f,2.2899f,  -33.4014f,-25.5766f,3.0748f,  -31.5909f,-25.9613f,2.3156f,  -31.4164f,-27.0638f,3.1129f,  
	-29.5105f,-27.2853f,2.5468f,  -29.3170f,-28.3789f,3.3574f,  -27.3506f,-28.4287f,2.9926f,  -27.1391f,-29.5083f,3.8165f,  -25.1489f,-29.3812f,3.6576f,  
	-24.9193f,-30.4432f,4.4955f,  -26.2374f,16.8784f,-29.5170f,  -26.2650f,16.0192f,-28.7796f,  -24.2870f,15.7615f,-28.6281f,  -24.3101f,14.9209f,-27.8965f,  
	-22.3416f,14.8595f,-27.5457f,  -22.3652f,14.0318f,-26.8174f,  -20.4453f,14.1619f,-26.2704f,  -20.4722f,13.3423f,-25.5445f,  -18.6385f,13.6555f,-24.8084f,  
	-18.6708f,12.8392f,-24.0842f,  -16.9589f,13.3237f,-23.1695f,  -16.9975f,12.5068f,-22.4468f,  -15.4398f,13.1479f,-21.3676f,  -15.4858f,12.3258f,-20.6461f,  
	-14.1114f,13.1063f,-19.4197f,  -14.1647f,12.2754f,-18.6994f,  -12.9991f,13.1760f,-17.3457f,  -13.0590f,12.3342f,-16.6274f,  -12.1231f,13.3340f,-15.1691f,  
	-12.1887f,12.4789f,-14.4529f,  -11.4993f,13.5559f,-12.9144f,  -11.5693f,12.6857f,-12.2004f,  -11.1382f,13.8171f,-10.6079f,  -11.2113f,12.9301f,-9.8966f,  
	-11.0456f,14.0928f,-8.2770f,  -11.1201f,13.1896f,-7.5688f,  -11.2221f,14.3606f,-5.9500f,  -11.2963f,13.4416f,-5.2454f,  -11.6635f,14.5984f,-3.6551f,  
	-11.7361f,13.6630f,-2.9533f,  -12.3612f,14.7839f,-1.4188f,  -12.4308f,13.8341f,-0.7205f,  -13.3021f,14.8986f,0.7319f,  -13.3674f,13.9350f,1.4269f,  
	-14.4691f,14.9241f,2.7723f,  -14.5292f,13.9480f,3.4644f,  -15.8416f,14.8443f,4.6796f,  -15.8956f,13.8577f,5.3689f,  -17.3953f,14.6456f,6.4329f,  
	-17.4427f,13.6498f,7.1201f,  -19.1034f,14.3161f,8.0147f,  -19.1440f,13.3129f,8.6998f,  -20.9367f,13.8464f,9.4096f,  -20.9703f,12.8371f,10.0933f,  
	-22.8638f,13.2292f,10.6060f,  -22.8906f,12.2148f,11.2888f,  -24.8523f,12.4594f,11.5956f,  -24.8724f,11.4412f,12.2779f,  -26.8685f,11.5346f,12.3732f,  
	-26.8823f,10.5134f,13.0556f,  -28.8785f,10.4545f,12.9378f,  -28.8865f,9.4308f,13.6208f,  -30.8486f,9.2210f,13.2916f,  -30.8512f,8.1951f,13.9755f,  
	-32.7455f,7.8386f,13.4404f,  -32.7431f,6.8107f,14.1257f,  -34.5373f,6.3139f,13.3935f,  -34.5302f,5.2832f,14.0809f,  -36.1935f,4.6550f,13.1639f,  
	-36.1820f,3.6216f,13.8534f,  -37.6858f,2.8730f,12.7670f,  -37.6699f,1.8360f,13.4593f,  -38.9882f,0.9802f,12.2217f,  -38.9679f,-0.0609f,12.9170f,  
	-40.0780f,-1.0087f,11.5490f,  -40.0528f,-2.0552f,12.2480f,  -40.9353f,-3.0783f,10.7727f,  -40.9047f,-4.1313f,11.4760f,  -41.5442f,-5.2115f,9.9183f,  
	-41.5075f,-6.2712f,10.6259f,  -41.8924f,-7.3890f,9.0120f,  -41.8485f,-8.4560f,9.7245f,  -41.9718f,-9.5908f,8.0816f,  -41.9196f,-10.6662f,8.7999f,  
	-41.7786f,-11.7972f,7.1558f,  -41.7167f,-12.8812f,7.8809f,  -41.3134f,-13.9868f,6.2632f,  -41.2402f,-15.0789f,6.9952f,  -40.5813f,-16.1378f,5.4311f,  
	-40.4953f,-17.2373f,6.1713f,  -39.5917f,-18.2286f,4.6870f,  -39.4914f,-19.3351f,5.4364f,  -38.3587f,-20.2384f,4.0568f,  -38.2424f,-21.3500f,4.8160f,  
	-36.9004f,-22.1461f,3.5641f,  -36.7669f,-23.2606f,4.3343f,  -35.2393f,-23.9316f,3.2305f,  -35.0872f,-25.0465f,4.0125f,  -33.4014f,-25.5766f,3.0748f,  
	-33.2300f,-26.6885f,3.8695f,  -31.4164f,-27.0638f,3.1129f,  -31.2250f,-28.1698f,3.9213f,  -29.3170f,-28.3789f,3.3574f,  -29.1060f,-29.4737f,4.1797f,  
	-27.1391f,-29.5083f,3.8165f,  -26.9083f,-30.5884f,4.6536f,  -24.9193f,-30.4432f,4.4955f,  -24.6708f,-31.5030f,5.3470f,  -26.2650f,16.0192f,-28.7796f,  
	-26.2912f,15.1532f,-28.0495f,  -24.3101f,14.9209f,-27.8965f,  -24.3338f,14.0713f,-27.1711f,  -22.3652f,14.0318f,-26.8174f,  -22.3896f,13.1940f,-26.0956f,  
	-20.4722f,13.3423f,-25.5445f,  -20.4999f,12.5116f,-24.8252f,  -18.6708f,12.8392f,-24.0842f,  -18.7034f,12.0115f,-23.3670f,  -16.9975f,12.5068f,-22.4468f,  
	-17.0363f,11.6774f,-21.7313f,  -15.4858f,12.3258f,-20.6461f,  -15.5312f,11.4910f,-19.9320f,  -14.1647f,12.2754f,-18.6994f,  -14.2165f,11.4326f,-17.9873f,  
	-13.0590f,12.3342f,-16.6274f,  -13.1165f,11.4807f,-15.9175f,  -12.1887f,12.4789f,-14.4529f,  -12.2509f,11.6126f,-13.7451f,  -11.5693f,12.6857f,-12.2004f,  
	-11.6351f,11.8045f,-11.4951f,  -11.2113f,12.9301f,-9.8966f,  -11.2792f,12.0341f,-9.1942f,  -11.1201f,13.1896f,-7.5688f,  -11.1886f,12.2789f,-6.8698f,  
	-11.2963f,13.4416f,-5.2454f,  -11.3640f,12.5152f,-4.5490f,  -11.7361f,13.6630f,-2.9533f,  -11.8016f,12.7226f,-2.2603f,  -12.4308f,13.8341f,-0.7205f,  
	-12.4928f,12.8801f,-0.0306f,  -13.3674f,13.9350f,1.4269f,  -13.4249f,12.9686f,2.1141f,  -14.5292f,13.9480f,3.4644f,  -14.5813f,12.9709f,4.1489f,  
	-15.8956f,13.8577f,5.3689f,  -15.9417f,12.8709f,6.0514f,  -17.4427f,13.6498f,7.1201f,  -17.4824f,12.6552f,7.8006f,  -19.1440f,13.3129f,8.6998f,  
	-19.1770f,12.3118f,9.3789f,  -20.9703f,12.8371f,10.0933f,  -20.9967f,11.8304f,10.7716f,  -22.8906f,12.2148f,11.2888f,  -22.9105f,11.2040f,11.9667f,  
	-24.8724f,11.4412f,12.2779f,  -24.8862f,10.4271f,12.9559f,  -26.8823f,10.5134f,13.0556f,  -26.8903f,9.4965f,13.7342f,  -28.8865f,9.4308f,13.6208f,  
	-28.8891f,8.4115f,14.3003f,  -30.8512f,8.1951f,13.9755f,  -30.8488f,7.1737f,14.6566f,  -32.7431f,6.8107f,14.1257f,  -32.7361f,5.7865f,14.8088f,  
	-34.5302f,5.2832f,14.0809f,  -34.5189f,4.2564f,14.7662f,  -36.1820f,3.6216f,13.8534f,  -36.1664f,2.5912f,14.5416f,  -37.6699f,1.8360f,13.4593f,  
	-37.6501f,0.8016f,14.1506f,  -38.9679f,-0.0609f,12.9170f,  -38.9435f,-1.1006f,13.6119f,  -40.0528f,-2.0552f,12.2480f,  -40.0234f,-3.1014f,12.9473f,  
	-40.9047f,-4.1313f,11.4760f,  -40.8697f,-5.1844f,12.1796f,  -41.5075f,-6.2712f,10.6259f,  -41.4658f,-7.3319f,11.3345f,  -41.8485f,-8.4560f,9.7245f,  
	-41.7991f,-9.5254f,10.4389f,  -41.9196f,-10.6662f,8.7999f,  -41.8610f,-11.7449f,9.5211f,  -41.7167f,-12.8812f,7.8809f,  -41.6474f,-13.9689f,8.6090f,  
	-41.2402f,-15.0789f,6.9952f,  -41.1587f,-16.1752f,7.7314f,  -40.4953f,-17.2373f,6.1713f,  -40.3998f,-18.3422f,6.9167f,  -39.4914f,-19.3351f,5.4364f,  
	-39.3801f,-20.4470f,6.1916f,  -38.2424f,-21.3500f,4.8160f,  -38.1139f,-22.4670f,5.5824f,  -36.7669f,-23.2606f,4.3343f,  -36.6196f,-24.3803f,5.1126f,  
	-35.0872f,-25.0465f,4.0125f,  -34.9200f,-26.1660f,4.8036f,  -33.2300f,-26.6885f,3.8695f,  -33.0419f,-27.8049f,4.6748f,  -31.2250f,-28.1698f,3.9213f,  
	-31.0162f,-29.2779f,4.7411f,  -29.1060f,-29.4737f,4.1797f,  -28.8758f,-30.5702f,5.0150f,  -26.9083f,-30.5884f,4.6536f,  -26.6583f,-31.6676f,5.5042f,  
	-24.6708f,-31.5030f,5.3470f,  -24.4019f,-32.5600f,6.2132f,  -26.2912f,15.1532f,-28.0495f,  -26.3173f,14.2799f,-27.3257f,  -24.3338f,14.0713f,-27.1711f,  
	-24.3580f,13.2132f,-26.4521f,  -22.3896f,13.1940f,-26.0956f,  -22.4149f,12.3465f,-25.3801f,  -20.4999f,12.5116f,-24.8252f,  -20.5282f,11.6706f,-24.1127f,  
	-18.7034f,12.0115f,-23.3670f,  -18.7366f,11.1724f,-22.6566f,  -17.0363f,11.6774f,-21.7313f,  -17.0752f,10.8362f,-21.0227f,  -15.5312f,11.4910f,-19.9320f,  
	-15.5759f,10.6448f,-19.2257f,  -14.2165f,11.4326f,-17.9873f,  -14.2667f,10.5784f,-17.2831f,  -13.1165f,11.4807f,-15.9175f,  -13.1715f,10.6162f,-15.2155f,  
	-12.2509f,11.6126f,-13.7451f,  -12.3100f,10.7352f,-13.0454f,  -11.6351f,11.8045f,-11.4951f,  -11.6968f,10.9140f,-10.7982f,  -11.2792f,12.0341f,-9.1942f,  
	-11.3420f,11.1303f,-8.5005f,  -11.1886f,12.2789f,-6.8698f,  -11.2514f,11.3602f,-6.1786f,  -11.3640f,12.5152f,-4.5490f,  -11.4253f,11.5833f,-3.8611f,  
	-11.8016f,12.7226f,-2.2603f,  -11.8602f,11.7775f,-1.5753f,  -12.4928f,12.8801f,-0.0306f,  -12.5475f,11.9226f,0.6518f,  -13.4249f,12.9686f,2.1141f,  
	-13.4749f,12.0003f,2.7939f,  -14.5813f,12.9709f,4.1489f,  -14.6259f,11.9926f,4.8268f,  -15.9417f,12.8709f,6.0514f,  -15.9803f,11.8847f,6.7272f,  
	-17.4824f,12.6552f,7.8006f,  -17.5147f,11.6622f,8.4750f,  -19.1770f,12.3118f,9.3789f,  -19.2030f,11.3127f,10.0527f,  -20.9967f,11.8304f,10.7716f,  
	-21.0164f,10.8270f,11.4449f,  -22.9105f,11.2040f,11.9667f,  -22.9242f,10.1969f,12.6402f,  -24.8862f,10.4271f,12.9559f,  -24.8941f,9.4170f,13.6299f,  
	-26.8903f,9.4965f,13.7342f,  -26.8929f,8.4839f,14.4093f,  -28.8891f,8.4115f,14.3003f,  -28.8867f,7.3966f,14.9770f,  -30.8488f,7.1737f,14.6566f,  
	-30.8419f,6.1563f,15.3353f,  -32.7361f,5.7865f,14.8088f,  -32.7249f,4.7665f,15.4898f,  -34.5189f,4.2564f,14.7662f,  -34.5036f,3.2330f,15.4501f,  
	-36.1664f,2.5912f,14.5416f,  -36.1471f,1.5639f,15.2286f,  -37.6501f,0.8016f,14.1506f,  -37.6265f,-0.2308f,14.8412f,  -38.9435f,-1.1006f,13.6119f,  
	-38.9154f,-2.1393f,14.3070f,  -40.0234f,-3.1014f,12.9473f,  -39.9901f,-4.1470f,13.6466f,  -40.8697f,-5.1844f,12.1796f,  -40.8303f,-6.2377f,12.8840f,  
	-41.4658f,-7.3319f,11.3345f,  -41.4193f,-8.3942f,12.0445f,  -41.7991f,-9.5254f,10.4389f,  -41.7440f,-10.5975f,11.1557f,  -41.8610f,-11.7449f,9.5211f,  
	-41.7958f,-12.8268f,10.2448f,  -41.6474f,-13.9689f,8.6090f,  -41.5705f,-15.0605f,9.3407f,  -41.1587f,-16.1752f,7.7314f,  -41.0683f,-17.2768f,8.4723f,  
	-40.3998f,-18.3422f,6.9167f,  -40.2940f,-19.4524f,7.6673f,  -39.3801f,-20.4470f,6.1916f,  -39.2573f,-21.5643f,6.9536f,  -38.1139f,-22.4670f,5.5824f,  
	-37.9722f,-23.5894f,6.3562f,  -36.6196f,-24.3803f,5.1126f,  -36.4578f,-25.5050f,5.8994f,  -34.9200f,-26.1660f,4.8036f,  -34.7367f,-27.2905f,5.6050f,  
	-33.0419f,-27.8049f,4.6748f,  -32.8368f,-28.9242f,5.4910f,  -31.0162f,-29.2779f,4.7411f,  -30.7883f,-30.3887f,5.5733f,  -28.8758f,-30.5702f,5.0150f,  
	-28.6263f,-31.6669f,5.8634f,  -26.6583f,-31.6676f,5.5042f,  -26.3877f,-32.7453f,6.3692f,  -24.4019f,-32.5600f,6.2132f,  -24.1118f,-33.6134f,7.0946f,  
	-26.3173f,14.2799f,-27.3257f,  -26.3437f,13.3995f,-26.6080f,  -24.3580f,13.2132f,-26.4521f,  -24.3829f,12.3467f,-25.7391f,  -22.4149f,12.3465f,-25.3801f,  
	-22.4407f,11.4899f,-24.6709f,  -20.5282f,11.6706f,-24.1127f,  -20.5574f,10.8193f,-23.4063f,  -18.7366f,11.1724f,-22.6566f,  -18.7702f,10.3223f,-21.9526f,  
	-17.0752f,10.8362f,-21.0227f,  -17.1139f,9.9842f,-20.3212f,  -15.5759f,10.6448f,-19.2257f,  -15.6197f,9.7876f,-18.5265f,  -14.2667f,10.5784f,-17.2831f,  
	-14.3152f,9.7135f,-16.5863f,  -13.1715f,10.6162f,-15.2155f,  -13.2242f,9.7406f,-14.5209f,  -12.3100f,10.7352f,-13.0454f,  -12.3656f,9.8483f,-12.3536f,  
	-11.6968f,10.9140f,-10.7982f,  -11.7541f,10.0152f,-10.1095f,  -11.3420f,11.1303f,-8.5005f,  -11.4000f,10.2179f,-7.8142f,  -11.2514f,11.3602f,-6.1786f,  
	-11.3085f,10.4355f,-5.4955f,  -11.4253f,11.5833f,-3.8611f,  -11.4805f,10.6459f,-3.1808f,  -11.8602f,11.7775f,-1.5753f,  -11.9122f,10.8280f,-0.8974f,  
	-12.5475f,11.9226f,0.6518f,  -12.5954f,10.9625f,1.3271f,  -13.4749f,12.0003f,2.7939f,  -13.5179f,11.0301f,3.4674f,  -14.6259f,11.9926f,4.8268f,  
	-14.6634f,11.0145f,5.4982f,  -15.9803f,11.8847f,6.7272f,  -16.0119f,10.8994f,7.3972f,  -17.5147f,11.6622f,8.4750f,  -17.5403f,10.6706f,9.1445f,  
	-19.2030f,11.3127f,10.0527f,  -19.2225f,10.3165f,10.7216f,  -21.0164f,10.8270f,11.4449f,  -21.0299f,9.8268f,12.1140f,  -22.9242f,10.1969f,12.6402f,  
	-22.9320f,9.1937f,13.3097f,  -24.8941f,9.4170f,13.6299f,  -24.8966f,8.4112f,14.3005f,  -26.8929f,8.4839f,14.4093f,  -26.8905f,7.4758f,15.0815f,  
	-28.8867f,7.3966f,14.9770f,  -28.8798f,6.3861f,15.6512f,  -30.8419f,6.1563f,15.3353f,  -30.8308f,5.1433f,16.0118f,  -32.7249f,4.7665f,15.4898f,  
	-32.7099f,3.7503f,16.1692f,  -34.5036f,3.2330f,15.4501f,  -34.4848f,2.2131f,16.1326f,  -36.1471f,1.5639f,15.2286f,  -36.1244f,0.5393f,15.9148f,  
	-37.6265f,-0.2308f,14.8412f,  -37.5995f,-1.2616f,15.5318f,  -38.9154f,-2.1393f,14.3070f,  -38.8837f,-3.1768f,15.0018f,  -39.9901f,-4.1470f,13.6466f,  
	-39.9529f,-5.1921f,14.3465f,  -40.8303f,-6.2377f,12.8840f,  -40.7865f,-7.2919f,13.5894f,  -41.4193f,-8.3942f,12.0445f,  -41.3675f,-9.4586f,12.7567f,  
	-41.7440f,-10.5975f,11.1557f,  -41.6829f,-11.6722f,11.8746f,  -41.7958f,-12.8268f,10.2448f,  -41.7238f,-13.9121f,10.9716f,  -41.5705f,-15.0605f,9.3407f,  
	-41.4856f,-16.1569f,10.0766f,  -41.0683f,-17.2768f,8.4723f,  -40.9685f,-18.3833f,9.2179f,  -40.2940f,-19.4524f,7.6673f,  -40.1775f,-20.5680f,8.4242f,  
	-39.2573f,-21.5643f,6.9536f,  -39.1221f,-22.6871f,7.7222f,  -37.9722f,-23.5894f,6.3562f,  -37.8168f,-24.7170f,7.1380f,  -36.4578f,-25.5050f,5.8994f,  
	-36.2803f,-26.6354f,6.6959f,  -34.7367f,-27.2905f,5.6050f,  -34.5367f,-28.4187f,6.4166f,  -32.8368f,-28.9242f,5.4910f,  -32.6129f,-30.0472f,6.3191f,  
	-30.7883f,-30.3887f,5.5733f,  -30.5412f,-31.5009f,6.4183f,  -28.6263f,-31.6669f,5.8634f,  -28.3561f,-32.7635f,6.7259f,  -26.3877f,-32.7453f,6.3692f,  
	-26.0956f,-33.8208f,7.2492f,  -24.1118f,-33.6134f,7.0946f,  -23.7999f,-34.6620f,7.9918f,  -26.3437f,13.3995f,-26.6080f,  -26.3703f,12.5123f,-25.8962f,  
	-24.3829f,12.3467f,-25.7391f,  -24.4083f,11.4725f,-25.0321f,  -22.4407f,11.4899f,-24.6709f,  -22.4676f,10.6241f,-23.9675f,  -20.5574f,10.8193f,-23.4063f,  
	-20.5873f,9.9580f,-22.7060f,  -18.7702f,10.3223f,-21.9526f,  -18.8042f,9.4620f,-21.2552f,  -17.1139f,9.9842f,-20.3212f,  -17.1524f,9.1216f,-19.6264f,  
	-15.6197f,9.7876f,-18.5265f,  -15.6624f,8.9200f,-17.8342f,  -14.3152f,9.7135f,-16.5863f,  -14.3621f,8.8376f,-15.8963f,  -13.2242f,9.7406f,-14.5209f,  
	-13.2743f,8.8554f,-13.8336f,  -12.3656f,9.8483f,-12.3536f,  -12.4178f,8.9530f,-11.6694f,  -11.7541f,10.0152f,-10.1095f,  -11.8074f,9.1076f,-9.4277f,  
	-11.4000f,10.2179f,-7.8142f,  -11.4530f,9.2990f,-7.1355f,  -11.3085f,10.4355f,-5.4955f,  -11.3601f,9.5047f,-4.8195f,  -11.4805f,10.6459f,-3.1808f,  
	-11.5296f,9.7035f,-2.5072f,  -11.9122f,10.8280f,-0.8974f,  -11.9578f,9.8752f,-0.2264f,  -12.5954f,10.9625f,1.3271f,  -12.6367f,9.9998f,1.9963f,  
	-13.5179f,11.0301f,3.4674f,  -13.5541f,10.0595f,4.1345f,  -14.6634f,11.0145f,5.4982f,  -14.6941f,10.0366f,6.1639f,  -16.0119f,10.8994f,7.3972f,  
	-16.0368f,9.9150f,8.0624f,  -17.5403f,10.6706f,9.1445f,  -17.5594f,9.6813f,9.8090f,  -19.2225f,10.3165f,10.7216f,  -19.2358f,9.3231f,11.3862f,  
	-21.0299f,9.8268f,12.1140f,  -21.0377f,8.8303f,12.7791f,  -22.9320f,9.1937f,13.3097f,  -22.9345f,8.1945f,13.9759f,  -24.8966f,8.4112f,14.3005f,  
	-24.8943f,7.4097f,14.9683f,  -26.8905f,7.4758f,15.0815f,  -26.8837f,6.4721f,15.7512f,  -28.8798f,6.3861f,15.6512f,  -28.8689f,5.3801f,16.3232f,  
	-30.8308f,5.1433f,16.0118f,  -30.8160f,4.1344f,16.6867f,  -32.7099f,3.7503f,16.1692f,  -32.6915f,2.7381f,16.8473f,  -34.4848f,2.2131f,16.1326f,  
	-34.4627f,1.1965f,16.8143f,  -36.1244f,0.5393f,15.9148f,  -36.0984f,-0.4831f,16.6008f,  -37.5995f,-1.2616f,15.5318f,  -37.5693f,-2.2903f,16.2220f,  
	-38.8837f,-3.1768f,15.0018f,  -38.8484f,-4.2131f,15.6970f,  -39.9529f,-5.1921f,14.3465f,  -39.9117f,-6.2375f,15.0472f,  -40.7865f,-7.2919f,13.5894f,  
	-40.7382f,-8.3476f,14.2967f,  -41.3675f,-9.4586f,12.7567f,  -41.3106f,-10.5249f,13.4706f,  -41.6829f,-11.6722f,11.8746f,  -41.6157f,-12.7499f,12.5963f,  
	-41.7238f,-13.9121f,10.9716f,  -41.6444f,-15.0018f,11.7023f,  -41.4856f,-16.1569f,10.0766f,  -41.3920f,-17.2580f,10.8167f,  -40.9685f,-18.3833f,9.2179f,  
	-40.8587f,-19.4952f,9.9691f,  -40.1775f,-20.5680f,8.4242f,  -40.0494f,-21.6891f,9.1871f,  -39.1221f,-22.6871f,7.7222f,  -38.9739f,-23.8154f,8.4983f,  
	-37.8168f,-24.7170f,7.1380f,  -37.6465f,-25.8508f,7.9288f,  -36.2803f,-26.6354f,6.6959f,  -36.0869f,-27.7700f,7.5021f,  -34.5367f,-28.4187f,6.4166f,  
	-34.3185f,-29.5514f,7.2396f,  -32.6129f,-30.0472f,6.3191f,  -32.3700f,-31.1726f,7.1596f,  -30.5412f,-31.5009f,6.4183f,  -30.2734f,-32.6144f,7.2771f,  
	-28.3561f,-32.7635f,6.7259f,  -28.0642f,-33.8593f,7.6031f,  -26.0956f,-33.8208f,7.2492f,  -25.7811f,-34.8931f,8.1449f,  -23.7999f,-34.6620f,7.9918f,  
	-23.4654f,-35.7046f,8.9052f,  -26.3703f,12.5123f,-25.8962f,  -26.3972f,11.6187f,-25.1903f,  -24.4083f,11.4725f,-25.0321f,  -24.4346f,10.5903f,-24.3307f,  
	-22.4676f,10.6241f,-23.9675f,  -22.4954f,9.7495f,-23.2698f,  -20.5873f,9.9580f,-22.7060f,  -20.6179f,9.0873f,-22.0116f,  -18.8042f,9.4620f,-21.2552f,  
	-18.8384f,8.5919f,-20.5638f,  -17.1524f,9.1216f,-19.6264f,  -17.1903f,8.2492f,-18.9378f,  -15.6624f,8.9200f,-17.8342f,  -15.7044f,8.0417f,-17.1481f,  
	-14.3621f,8.8376f,-15.8963f,  -14.4072f,7.9521f,-15.2130f,  -13.2743f,8.8554f,-13.8336f,  -13.3216f,7.9616f,-13.1535f,  -12.4178f,8.9530f,-11.6694f,  
	-12.4666f,8.0485f,-10.9916f,  -11.8074f,9.1076f,-9.4277f,  -11.8564f,8.1932f,-8.7530f,  -11.4530f,9.2990f,-7.1355f,  -11.5013f,8.3735f,-6.4635f,  
	-11.3601f,9.5047f,-4.8195f,  -11.4065f,8.5683f,-4.1499f,  -11.5296f,9.7035f,-2.5072f,  -11.5730f,8.7572f,-1.8401f,  -11.9578f,9.8752f,-0.2264f,  
	-11.9973f,8.9193f,0.4388f,  -12.6367f,9.9998f,1.9963f,  -12.6716f,9.0361f,2.6594f,  -13.5541f,10.0595f,4.1345f,  -13.5840f,9.0885f,4.7961f,  
	-14.6941f,10.0366f,6.1639f,  -14.7185f,9.0589f,6.8250f,  -16.0368f,9.9150f,8.0624f,  -16.0556f,8.9324f,8.7228f,  -17.5594f,9.6813f,9.8090f,  
	-17.5725f,8.6945f,10.4695f,  -19.2358f,9.3231f,11.3862f,  -19.2435f,8.3332f,12.0471f,  -21.0377f,8.8303f,12.7791f,  -21.0402f,7.8377f,13.4410f,  
	-22.9345f,8.1945f,13.9759f,  -22.9322f,7.1996f,14.6393f,  -24.8943f,7.4097f,14.9683f,  -24.8875f,6.4128f,15.6336f,  -26.8837f,6.4721f,15.7512f,  
	-26.8728f,5.4730f,16.4188f,  -28.8689f,5.3801f,16.3232f,  -28.8543f,4.3785f,16.9937f,  -30.8160f,4.1344f,16.6867f,  -30.7980f,3.1298f,17.3602f,  
	-32.6915f,2.7381f,16.8473f,  -32.6700f,1.7295f,17.5244f,  -34.4627f,1.1965f,16.8143f,  -34.4377f,0.1826f,17.4956f,  -36.0984f,-0.4831f,16.6008f,  
	-36.0695f,-1.5029f,17.2863f,  -37.5693f,-2.2903f,16.2220f,  -37.5360f,-3.3174f,16.9124f,  -38.8484f,-4.2131f,15.6970f,  -38.8099f,-5.2489f,16.3928f,  
	-39.9117f,-6.2375f,15.0472f,  -39.8666f,-7.2836f,15.7494f,  -40.7382f,-8.3476f,14.2967f,  -40.6853f,-9.4045f,15.0054f,  -41.3106f,-10.5249f,13.4706f,  
	-41.2483f,-11.5935f,14.1869f,  -41.6157f,-12.7499f,12.5963f,  -41.5421f,-13.8313f,13.3214f,  -41.6444f,-15.0018f,11.7023f,  -41.5573f,-16.0957f,12.4365f,  
	-41.3920f,-17.2580f,10.8167f,  -41.2894f,-18.3641f,11.5620f,  -40.8587f,-19.4952f,9.9691f,  -40.7384f,-20.6123f,10.7259f,  -40.0494f,-21.6891f,9.1871f,  
	-39.9093f,-22.8156f,9.9569f,  -38.9739f,-23.8154f,8.4983f,  -38.8117f,-24.9501f,9.2828f,  -37.6465f,-25.8508f,7.9288f,  -37.4608f,-26.9893f,8.7289f,  
	-36.0869f,-27.7700f,7.5021f,  -35.8758f,-28.9099f,8.3190f,  -34.3185f,-29.5514f,7.2396f,  -34.0817f,-30.6875f,8.0746f,  -32.3700f,-31.1726f,7.1596f,  
	-32.1068f,-32.3004f,8.0134f,  -30.2734f,-32.6144f,7.2771f,  -29.9839f,-33.7283f,8.1503f,  -28.0642f,-33.8593f,7.6031f,  -27.7497f,-34.9533f,8.4958f,  
	-25.7811f,-34.8931f,8.1449f,  -25.4436f,-35.9609f,9.0568f,  -23.4654f,-35.7046f,8.9052f,  -23.1070f,-36.7403f,9.8357f,  -26.3972f,11.6187f,-25.1903f,  
	-26.4247f,10.7184f,-24.4895f,  -24.4346f,10.5903f,-24.3307f,  -24.4618f,9.7005f,-23.6345f,  -22.4954f,9.7495f,-23.2698f,  -22.5240f,8.8665f,-22.5776f,  
	-20.6179f,9.0873f,-22.0116f,  -20.6490f,8.2076f,-21.3228f,  -18.8384f,8.5919f,-20.5638f,  -18.8725f,7.7126f,-19.8782f,  -17.1903f,8.2492f,-18.9378f,  
	-17.2279f,7.3665f,-18.2549f,  -15.7044f,8.0417f,-17.1481f,  -15.7450f,7.1541f,-16.4682f,  -14.4072f,7.9521f,-15.2130f,  -14.4502f,7.0582f,-14.5363f,  
	-13.3216f,7.9616f,-13.1535f,  -13.3663f,7.0587f,-12.4792f,  -12.4666f,8.0485f,-10.9916f,  -12.5119f,7.1370f,-10.3205f,  -11.8564f,8.1932f,-8.7530f,  
	-11.9014f,7.2717f,-8.0846f,  -11.5013f,8.3735f,-6.4635f,  -11.5448f,7.4420f,-5.7974f,  -11.4065f,8.5683f,-4.1499f,  -11.4475f,7.6275f,-3.4864f,  
	-11.5730f,8.7572f,-1.8401f,  -11.6107f,7.8071f,-1.1785f,  -11.9973f,8.9193f,0.4388f,  -12.0309f,7.9617f,1.0982f,  -12.6716f,9.0361f,2.6594f,  
	-12.7005f,8.0712f,3.3174f,  -13.5840f,9.0885f,4.7961f,  -13.6078f,8.1169f,5.4534f,  -14.7185f,9.0589f,6.8250f,  -14.7369f,8.0825f,7.4815f,  
	-16.0556f,8.9324f,8.7228f,  -16.0685f,7.9517f,9.3793f,  -17.5725f,8.6945f,10.4695f,  -17.5801f,7.7107f,11.1263f,  -19.2435f,8.3332f,12.0471f,  
	-19.2460f,7.3467f,12.7049f,  -21.0402f,7.8377f,13.4410f,  -21.0378f,6.8490f,14.1003f,  -22.9322f,7.1996f,14.6393f,  -22.9254f,6.2092f,15.3003f,  
	-24.8875f,6.4128f,15.6336f,  -24.8767f,5.4205f,16.2969f,  -26.8728f,5.4730f,16.4188f,  -26.8584f,4.4786f,17.0849f,  -28.8543f,4.3785f,16.9937f,  
	-28.8365f,3.3815f,17.6628f,  -30.7980f,3.1298f,17.3602f,  -30.7770f,2.1293f,18.0328f,  -32.6700f,1.7295f,17.5244f,  -32.6458f,0.7244f,18.2011f,  
	-34.4377f,0.1826f,17.4956f,  -34.4100f,-0.8279f,18.1765f,  -36.0695f,-1.5029f,17.2863f,  -36.0379f,-2.5203f,17.9720f,  -37.5360f,-3.3174f,16.9124f,  
	-37.4998f,-4.3431f,17.6033f,  -38.8099f,-5.2489f,16.3928f,  -38.7679f,-6.2847f,17.0900f,  -39.8666f,-7.2836f,15.7494f,  -39.8177f,-8.3301f,16.4528f,  
	-40.6853f,-9.4045f,15.0054f,  -40.6278f,-10.4630f,15.7162f,  -41.2483f,-11.5935f,14.1869f,  -41.1803f,-12.6653f,14.9062f,  -41.5421f,-13.8313f,13.3214f,  
	-41.4616f,-14.9163f,14.0496f,  -41.5573f,-16.0957f,12.4365f,  -41.4622f,-17.1942f,13.1756f,  -41.2894f,-18.3641f,11.5620f,  -41.1773f,-19.4752f,12.3123f,  
	-40.7384f,-20.6123f,10.7259f,  -40.6069f,-21.7348f,11.4890f,  -39.9093f,-22.8156f,9.9569f,  -39.7560f,-23.9487f,10.7346f,  -38.8117f,-24.9501f,9.2828f,  
	-38.6349f,-26.0899f,10.0759f,  -37.4608f,-26.9893f,8.7289f,  -37.2583f,-28.1336f,9.5390f,  -35.8758f,-28.9099f,8.3190f,  -35.6468f,-30.0539f,9.1475f,  
	-34.0817f,-30.6875f,8.0746f,  -33.8249f,-31.8270f,8.9223f,  -32.1068f,-32.3004f,8.0134f,  -31.8221f,-33.4299f,8.8813f,  -29.9839f,-33.7283f,8.1503f,  
	-29.6719f,-34.8417f,9.0386f,  -27.7497f,-34.9533f,8.4958f,  -27.4119f,-36.0444f,9.4047f,  -25.4436f,-35.9609f,9.0568f,  -25.0817f,-37.0235f,9.9859f,  
	-23.1070f,-36.7403f,9.8357f,  -22.7241f,-37.7677f,10.7838f,  -26.4247f,10.7184f,-24.4895f,  -26.4528f,9.8117f,-23.7938f,  -24.4618f,9.7005f,-23.6345f,  
	-24.4898f,8.8032f,-22.9434f,  -22.5240f,8.8665f,-22.5776f,  -22.5532f,7.9753f,-21.8905f,  -20.6490f,8.2076f,-21.3228f,  -20.6803f,7.3194f,-20.6394f,  
	-18.8725f,7.7126f,-19.8782f,  -18.9067f,6.8236f,-19.1978f,  -17.2279f,7.3665f,-18.2549f,  -17.2649f,6.4748f,-17.5776f,  -15.7450f,7.1541f,-16.4682f,  
	-15.7842f,6.2580f,-15.7942f,  -14.4502f,7.0582f,-14.5363f,  -14.4912f,6.1550f,-13.8649f,  -13.3663f,7.0587f,-12.4792f,  -13.4081f,6.1484f,-11.8110f,  
	-12.5119f,7.1370f,-10.3205f,  -12.5537f,6.2180f,-9.6550f,  -11.9014f,7.2717f,-8.0846f,  -11.9422f,6.3437f,-7.4216f,  -11.5448f,7.4420f,-5.7974f,  
	-11.5836f,6.5055f,-5.1370f,  -11.4475f,7.6275f,-3.4864f,  -11.4835f,6.6823f,-2.8279f,  -11.6107f,7.8071f,-1.1785f,  -11.6430f,6.8547f,-0.5223f,  
	-12.0309f,7.9617f,1.0982f,  -12.0589f,7.0022f,1.7528f,  -12.7005f,8.0712f,3.3174f,  -12.7237f,7.1052f,3.9712f,  -13.6078f,8.1169f,5.4534f,  
	-13.6258f,7.1461f,6.1064f,  -14.7369f,8.0825f,7.4815f,  -14.7496f,7.1074f,8.1344f,  -16.0685f,7.9517f,9.3793f,  -16.0761f,6.9737f,10.0324f,  
	-17.5801f,7.7107f,11.1263f,  -17.5826f,6.7300f,11.7802f,  -19.2460f,7.3467f,12.7049f,  -19.2436f,6.3639f,13.3602f,  -21.0378f,6.8490f,14.1003f,  
	-21.0311f,5.8647f,14.7572f,  -22.9254f,6.2092f,15.3003f,  -22.9147f,5.2234f,15.9595f,  -24.8767f,5.4205f,16.2969f,  -24.8624f,4.4330f,16.9587f,  
	-26.8584f,4.4786f,17.0849f,  -26.8408f,3.4890f,17.7497f,  -28.8365f,3.3815f,17.6628f,  -28.8159f,2.3890f,18.3310f,  -30.7770f,2.1293f,18.0328f,  
	-30.7533f,1.1328f,18.7050f,  -32.6458f,0.7244f,18.2011f,  -32.6191f,-0.2769f,18.8773f,  -34.4100f,-0.8279f,18.1765f,  -34.3799f,-1.8355f,18.8574f,  
	-36.0379f,-2.5203f,17.9720f,  -36.0038f,-3.5356f,18.6579f,  -37.4998f,-4.3431f,17.6033f,  -37.4606f,-5.3681f,18.2954f,  -38.7679f,-6.2847f,17.0900f,  
	-38.7226f,-7.3201f,17.7881f,  -39.8177f,-8.3301f,16.4528f,  -39.7648f,-9.3774f,17.1581f,  -40.6278f,-10.4630f,15.7162f,  -40.5654f,-11.5238f,16.4297f,  
	-41.1803f,-12.6653f,14.9062f,  -41.1065f,-13.7399f,15.6283f,  -41.4616f,-14.9163f,14.0496f,  -41.3740f,-16.0054f,14.7822f,  -41.4622f,-17.1942f,13.1756f,  
	-41.3584f,-18.2972f,13.9191f,  -41.1773f,-19.4752f,12.3123f,  -41.0549f,-20.5914f,13.0683f,  -40.6069f,-21.7348f,11.4890f,  -40.4634f,-22.8639f,12.2593f,  
	-39.7560f,-23.9487f,10.7346f,  -39.5892f,-25.0871f,11.5202f,  -38.6349f,-26.0899f,10.0759f,  -38.4423f,-27.2358f,10.8785f,  -37.2583f,-28.1336f,9.5390f,  
	-37.0386f,-29.2827f,10.3600f,  -35.6468f,-30.0539f,9.1475f,  -35.3983f,-31.2022f,9.9882f,  -33.8249f,-31.8270f,8.9223f,  -33.5471f,-32.9692f,9.7836f,  
	-31.8221f,-33.4299f,8.8813f,  -31.5150f,-34.5601f,9.7639f,  -29.6719f,-34.8417f,9.0386f,  -29.3365f,-35.9538f,9.9429f,  -27.4119f,-36.0444f,9.4047f,  
	-27.0493f,-37.1320f,10.3307f,  -25.0817f,-37.0235f,9.9859f,  -24.6946f,-38.0795f,10.9327f,  -22.7241f,-37.7677f,10.7838f,  -22.3168f,-38.7849f,11.7496f,  
	-26.4528f,9.8117f,-23.7938f,  -26.4816f,8.8987f,-23.1028f,  -24.4898f,8.8032f,-22.9434f,  -24.5184f,7.8988f,-22.2570f,  -22.5532f,7.9753f,-21.8905f,  
	-22.5828f,7.0764f,-21.2082f,  -20.6803f,7.3194f,-20.6394f,  -20.7121f,6.4222f,-19.9605f,  -18.9067f,6.8236f,-19.1978f,  -18.9407f,5.9260f,-18.5224f,  
	-17.2649f,6.4748f,-17.5776f,  -17.3008f,5.5749f,-16.9057f,  -15.7842f,6.2580f,-15.7942f,  -15.8219f,5.3530f,-15.1251f,  -14.4912f,6.1550f,-13.8649f,  
	-14.5298f,5.2444f,-13.1991f,  -13.4081f,6.1484f,-11.8110f,  -13.4471f,5.2305f,-11.1480f,  -12.5537f,6.2180f,-9.6550f,  -12.5920f,5.2922f,-8.9946f,  
	-11.9422f,6.3437f,-7.4216f,  -11.9789f,5.4103f,-6.7638f,  -11.5836f,6.5055f,-5.1370f,  -11.6179f,5.5640f,-4.4813f,  -11.4835f,6.6823f,-2.8279f,  
	-11.5144f,5.7341f,-2.1746f,  -11.6430f,6.8547f,-0.5223f,  -11.6700f,5.8999f,0.1294f,  -12.0589f,7.0022f,1.7528f,  -12.0814f,6.0410f,2.4037f,  
	-12.7237f,7.1052f,3.9712f,  -12.7413f,6.1393f,4.6211f,  -13.6258f,7.1461f,6.1064f,  -13.6383f,6.1761f,6.7561f,  -14.7496f,7.1074f,8.1344f,  
	-14.7570f,6.1345f,8.7841f,  -16.0761f,6.9737f,10.0324f,  -16.0785f,5.9982f,10.6828f,  -17.5826f,6.7300f,11.7802f,  -17.5803f,5.7526f,12.4320f,  
	-19.2436f,6.3639f,13.3602f,  -19.2369f,5.3854f,14.0134f,  -21.0311f,5.8647f,14.7572f,  -21.0204f,4.8850f,15.4126f,  -22.9147f,5.2234f,15.9595f,  
	-22.9004f,4.2424f,16.6174f,  -24.8624f,4.4330f,16.9587f,  -24.8449f,3.4504f,17.6195f,  -26.8408f,3.4890f,17.7497f,  -26.8204f,2.5042f,18.4137f,  
	-28.8159f,2.3890f,18.3310f,  -28.7927f,1.4010f,18.9989f,  -30.7533f,1.1328f,18.7050f,  -30.7275f,0.1407f,19.3769f,  -32.6191f,-0.2769f,18.8773f,  
	-32.5903f,-1.2746f,19.5537f,  -34.3799f,-1.8355f,18.8574f,  -34.3477f,-2.8402f,19.5387f,  -36.0038f,-3.5356f,18.6579f,  -35.9673f,-4.5494f,19.3450f,  
	-37.4606f,-5.3681f,18.2954f,  -37.4188f,-6.3918f,18.9883f,  -38.7226f,-7.3201f,17.7881f,  -38.6741f,-8.3556f,18.4878f,  -39.7648f,-9.3774f,17.1581f,  
	-39.7078f,-10.4264f,17.8658f,  -40.5654f,-11.5238f,16.4297f,  -40.4981f,-12.5868f,17.1456f,  -41.1065f,-13.7399f,15.6283f,  -41.0264f,-14.8181f,16.3544f,  
	-41.3740f,-16.0054f,14.7822f,  -41.2788f,-17.0985f,15.5187f,  -41.3584f,-18.2972f,13.9191f,  -41.2456f,-19.4051f,14.6677f,  -41.0549f,-20.5914f,13.0683f,  
	-40.9216f,-21.7140f,13.8309f,  -40.4634f,-22.8639f,12.2593f,  -40.3072f,-23.9982f,13.0371f,  -39.5892f,-25.0871f,11.5202f,  -39.4075f,-26.2319f,12.3147f,  
	-38.4423f,-27.2358f,10.8785f,  -38.2332f,-28.3870f,11.6914f,  -37.0386f,-29.2827f,10.3600f,  -36.8002f,-30.4368f,11.1928f,  -35.3983f,-31.2022f,9.9882f,  
	-35.1295f,-32.3542f,10.8419f,  -33.5471f,-32.9692f,9.7836f,  -33.2472f,-34.1135f,10.6593f,  -31.5150f,-34.5601f,9.7639f,  -31.1846f,-35.6905f,10.6623f,  
	-29.3365f,-35.9538f,9.9429f,  -28.9762f,-37.0639f,10.8643f,  -27.0493f,-37.1320f,10.3307f,  -26.6611f,-38.2147f,11.2745f,  -24.6946f,-38.0795f,10.9327f,  
	-24.2823f,-39.1271f,11.8975f,  -22.3168f,-38.7849f,11.7496f,  -21.8840f,-39.7907f,12.7338f,  -26.4816f,8.8987f,-23.1028f,  -26.5109f,7.9795f,-22.4161f,  
	-24.5184f,7.8988f,-22.2570f,  -24.5474f,6.9876f,-21.5751f,  -22.5828f,7.0764f,-21.2082f,  -22.6130f,6.1693f,-20.5301f,  -20.7121f,6.4222f,-19.9605f,  
	-20.7438f,5.5170f,-19.2862f,  -18.9407f,5.9260f,-18.5224f,  -18.9740f,5.0206f,-17.8518f,  -17.3008f,5.5749f,-16.9057f,  -17.3357f,4.6663f,-16.2382f,  
	-15.8219f,5.3530f,-15.1251f,  -15.8577f,4.4405f,-14.4610f,  -14.5298f,5.2444f,-13.1991f,  -14.5661f,4.3262f,-12.5379f,  -13.4471f,5.2305f,-11.1480f,  
	-13.4830f,4.3056f,-10.4895f,  -12.5920f,5.2922f,-8.9946f,  -12.6267f,4.3606f,-8.3389f,  -11.9789f,5.4103f,-6.7638f,  -12.0115f,4.4714f,-6.1102f,  
	-11.6179f,5.5640f,-4.4813f,  -11.6475f,4.6191f,-3.8302f,  -11.5144f,5.7341f,-2.1746f,  -11.5405f,4.7829f,-1.5253f,  -11.6700f,5.8999f,0.1294f,  
	-11.6918f,4.9426f,0.7777f,  -12.0814f,6.0410f,2.4037f,  -12.0986f,5.0794f,3.0509f,  -12.7413f,6.1393f,4.6211f,  -12.7536f,5.1737f,5.2680f,  
	-13.6383f,6.1761f,6.7561f,  -13.6456f,5.2077f,7.4028f,  -14.7570f,6.1345f,8.7841f,  -14.7595f,5.1638f,9.4314f,  -16.0785f,5.9982f,10.6828f,  
	-16.0762f,5.0257f,11.3313f,  -17.5803f,5.7526f,12.4320f,  -17.5736f,4.7794f,13.0818f,  -19.2369f,5.3854f,14.0134f,  -19.2262f,4.4112f,14.6653f,  
	-21.0204f,4.8850f,15.4126f,  -21.0061f,3.9100f,16.0667f,  -22.9004f,4.2424f,16.6174f,  -22.8830f,3.2663f,17.2744f,  -24.8449f,3.4504f,17.6195f,  
	-24.8246f,2.4729f,18.2797f,  -26.8204f,2.5042f,18.4137f,  -26.7976f,1.5242f,19.0776f,  -28.7927f,1.4010f,18.9989f,  -28.7675f,0.4177f,19.6665f,  
	-30.7275f,0.1407f,19.3769f,  -30.6997f,-0.8474f,20.0489f,  -32.5903f,-1.2746f,19.5537f,  -32.5597f,-2.2688f,20.2305f,  -34.3477f,-2.8402f,19.5387f,  
	-34.3134f,-3.8427f,20.2210f,  -35.9673f,-4.5494f,19.3450f,  -35.9286f,-5.5612f,20.0329f,  -37.4188f,-6.3918f,18.9883f,  -37.3744f,-7.4149f,19.6827f,  
	-38.6741f,-8.3556f,18.4878f,  -38.6222f,-9.3918f,19.1899f,  -39.7078f,-10.4264f,17.8658f,  -39.6468f,-11.4767f,18.5756f,  -40.4981f,-12.5868f,17.1456f,  
	-40.4254f,-13.6527f,17.8652f,  -41.0264f,-14.8181f,16.3544f,  -40.9398f,-15.8997f,17.0840f,  -41.2788f,-17.0985f,15.5187f,  -41.1756f,-18.1960f,16.2600f,  
	-41.2456f,-19.4051f,14.6677f,  -41.1229f,-20.5189f,15.4225f,  -40.9216f,-21.7140f,13.8309f,  -40.7768f,-22.8418f,14.6005f,  -40.3072f,-23.9982f,13.0371f,  
	-40.1373f,-25.1390f,13.8230f,  -39.4075f,-26.2319f,12.3147f,  -39.2103f,-27.3823f,13.1189f,  -38.2332f,-28.3870f,11.6914f,  -38.0065f,-29.5439f,12.5155f,  
	-36.8002f,-30.4368f,11.1928f,  -36.5421f,-31.5955f,12.0381f,  -35.1295f,-32.3542f,10.8419f,  -34.8391f,-33.5093f,11.7096f,  -33.2472f,-34.1135f,10.6593f,  
	-32.9244f,-35.2590f,11.5506f,  -31.1846f,-35.6905f,10.6623f,  -30.8293f,-36.8204f,11.5776f,  -28.9762f,-37.0639f,10.8643f,  -28.5899f,-38.1708f,11.8035f,  
	-26.6611f,-38.2147f,11.2745f,  -26.2471f,-39.2905f,12.2366f,  -24.2823f,-39.1271f,11.8975f,  -23.8437f,-40.1649f,12.8812f,  -21.8840f,-39.7907f,12.7338f,  
	-21.4243f,-40.7836f,13.7373f,  -26.5109f,7.9795f,-22.4161f,  -26.5404f,7.0546f,-21.7336f,  -24.5474f,6.9876f,-21.5751f,  -24.5771f,6.0691f,-20.8969f,  
	-22.6130f,6.1693f,-20.5301f,  -22.6435f,5.2549f,-19.8561f,  -20.7438f,5.5170f,-19.2862f,  -20.7753f,4.6044f,-18.6162f,  -18.9740f,5.0206f,-17.8518f,  
	-19.0067f,4.1068f,-17.1851f,  -17.3357f,4.6663f,-16.2382f,  -17.3692f,3.7504f,-15.5751f,  -15.8577f,4.4405f,-14.4610f,  -15.8918f,3.5204f,-13.8010f,  
	-14.5661f,4.3262f,-12.5379f,  -14.5999f,3.4007f,-11.8808f,  -13.4830f,4.3056f,-10.4895f,  -13.5158f,3.3746f,-9.8353f,  -12.6267f,4.3606f,-8.3389f,  
	-12.6577f,3.4233f,-7.6869f,  -12.0115f,4.4714f,-6.1102f,  -12.0399f,3.5287f,-5.4609f,  -11.6475f,4.6191f,-3.8302f,  -11.6726f,3.6706f,-3.1828f,  
	-11.5405f,4.7829f,-1.5253f,  -11.5616f,3.8287f,-0.8789f,  -11.6918f,4.9426f,0.7777f,  -11.7086f,3.9843f,1.4228f,  -12.0986f,5.0794f,3.0509f,  
	-12.1107f,4.1173f,3.6955f,  -12.7536f,5.1737f,5.2680f,  -12.7608f,4.2091f,5.9122f,  -13.6456f,5.2077f,7.4028f,  -13.6480f,4.2410f,8.0475f,  
	-14.7595f,5.1638f,9.4314f,  -14.7572f,4.1954f,10.0772f,  -16.0762f,5.0257f,11.3313f,  -16.0695f,4.0570f,11.9781f,  -17.5736f,4.7794f,13.0818f,  
	-17.5628f,3.8100f,13.7305f,  -19.2262f,4.4112f,14.6653f,  -19.2118f,3.4417f,15.3162f,  -21.0061f,3.9100f,16.0667f,  -20.9886f,2.9400f,16.7204f,  
	-22.8830f,3.2663f,17.2744f,  -22.8627f,2.2956f,17.9311f,  -24.8246f,2.4729f,18.2797f,  -24.8019f,1.5005f,18.9398f,  -26.7976f,1.5242f,19.0776f,  
	-26.7728f,0.5495f,19.7413f,  -28.7675f,0.4177f,19.6665f,  -28.7405f,-0.5610f,20.3346f,  -30.6997f,-0.8474f,20.0489f,  -30.6703f,-1.8312f,20.7214f,  
	-32.5597f,-2.2688f,20.2305f,  -32.5273f,-3.2600f,20.9083f,  -34.3134f,-3.8427f,20.2210f,  -34.2774f,-4.8423f,20.9042f,  -35.9286f,-5.5612f,20.0329f,  
	-35.8878f,-6.5714f,20.7223f,  -37.3744f,-7.4149f,19.6827f,  -37.3272f,-8.4378f,20.3793f,  -38.6222f,-9.3918f,19.1899f,  -38.5671f,-10.4285f,19.8938f,  
	-39.6468f,-11.4767f,18.5756f,  -39.5812f,-12.5290f,19.2888f,  -40.4254f,-13.6527f,17.8652f,  -40.3472f,-14.7211f,18.5879f,  -40.9398f,-15.8997f,17.0840f,  
	-40.8462f,-16.9849f,17.8179f,  -41.1756f,-18.1960f,16.2600f,  -41.0637f,-19.2989f,17.0069f,  -41.1229f,-20.5189f,15.4225f,  -40.9898f,-21.6376f,16.1838f,  
	-40.7768f,-22.8418f,14.6005f,  -40.6194f,-23.9758f,15.3776f,  -40.1373f,-25.1390f,13.8230f,  -39.9531f,-26.2856f,14.6181f,  -39.2103f,-27.3823f,13.1189f,  
	-38.9965f,-28.5386f,13.9338f,  -38.0065f,-29.5439f,12.5155f,  -37.7609f,-30.7058f,13.3515f,  -36.5421f,-31.5955f,12.0381f,  -36.2634f,-32.7580f,12.8967f,  
	-34.8391f,-33.5093f,11.7096f,  -34.5263f,-34.6669f,12.5924f,  -32.9244f,-35.2590f,11.5506f,  -32.5770f,-36.4056f,12.4583f,  -30.8293f,-36.8204f,11.5776f,  
	-30.4481f,-37.9486f,12.5105f,  -28.5899f,-38.1708f,11.8035f,  -28.1776f,-39.2725f,12.7610f,  -26.2471f,-39.2905f,12.2366f,  -25.8062f,-40.3585f,13.2179f,  
	-23.8437f,-40.1649f,12.8812f,  -23.3771f,-41.1919f,13.8848f,  -21.4243f,-40.7836f,13.7373f,  -20.9382f,-41.7616f,14.7598f,  -26.5404f,7.0546f,-21.7336f,  
	-26.5706f,6.1234f,-21.0545f,  -24.5771f,6.0691f,-20.8969f,  -24.6071f,5.1440f,-20.2224f,  -22.6435f,5.2549f,-19.8561f,  -22.6738f,4.3335f,-19.1860f,  
	-20.7753f,4.6044f,-18.6162f,  -20.8064f,3.6840f,-17.9497f,  -19.0067f,4.1068f,-17.1851f,  -19.0384f,3.1860f,-16.5224f,  -17.3692f,3.7504f,-15.5751f,  
	-17.4013f,2.8269f,-14.9157f,  -15.8918f,3.5204f,-13.8010f,  -15.9237f,2.5930f,-13.1446f,  -14.5999f,3.4007f,-11.8808f,  -14.6310f,2.4689f,-11.2274f,  
	-13.5158f,3.3746f,-9.8353f,  -13.5453f,2.4374f,-9.1844f,  -12.6577f,3.4233f,-7.6869f,  -12.6849f,2.4815f,-7.0388f,  -12.0399f,3.5287f,-5.4609f,  
	-12.0641f,2.5818f,-4.8148f,  -11.6726f,3.6706f,-3.1828f,  -11.6931f,2.7185f,-2.5379f,  -11.5616f,3.8287f,-0.8789f,  -11.5780f,2.8730f,-0.2355f,  
	-11.7086f,3.9843f,1.4228f,  -11.7205f,3.0250f,2.0655f,  -12.1107f,4.1173f,3.6955f,  -12.1178f,3.1558f,4.3377f,  -12.7608f,4.2091f,5.9122f,  
	-12.7632f,3.2456f,6.5547f,  -13.6480f,4.2410f,8.0475f,  -13.6457f,3.2761f,8.6909f,  -14.7572f,4.1954f,10.0772f,  -14.7504f,3.2306f,10.7214f,  
	-16.0695f,4.0570f,11.9781f,  -16.0586f,3.0919f,12.6242f,  -17.5628f,3.8100f,13.7305f,  -17.5483f,2.8453f,14.3786f,  -19.2118f,3.4417f,15.3162f,  
	-19.1942f,2.4770f,15.9669f,  -20.9886f,2.9400f,16.7204f,  -20.9682f,1.9753f,17.3739f,  -22.8627f,2.2956f,17.9311f,  -22.8400f,1.3302f,18.5878f,  
	-24.8019f,1.5005f,18.9398f,  -24.7773f,0.5336f,19.6000f,  -26.7728f,0.5495f,19.7413f,  -26.7462f,-0.4204f,20.4058f,  -28.7405f,-0.5610f,20.3346f,  
	-28.7121f,-1.5348f,21.0032f,  -30.6703f,-1.8312f,20.7214f,  -30.6395f,-2.8113f,21.3951f,  -32.5273f,-3.2600f,20.9083f,  -32.4936f,-4.2476f,21.5870f,  
	-34.2774f,-4.8423f,20.9042f,  -34.2398f,-5.8396f,21.5888f,  -35.8878f,-6.5714f,20.7223f,  -35.8449f,-7.5805f,21.4137f,  -37.3272f,-8.4378f,20.3793f,  
	-37.2775f,-9.4601f,21.0776f,  -38.5671f,-10.4285f,19.8938f,  -38.5083f,-11.4663f,20.6009f,  -39.5812f,-12.5290f,19.2888f,  -39.5111f,-13.5831f,20.0049f,  
	-40.3472f,-14.7211f,18.5879f,  -40.2631f,-15.7926f,19.3146f,  -40.8462f,-16.9849f,17.8179f,  -40.7452f,-18.0750f,18.5570f,  -41.0637f,-19.2989f,17.0069f,  
	-40.9426f,-20.4063f,17.7598f,  -40.9898f,-21.6376f,16.1838f,  -40.8455f,-22.7623f,16.9519f,  -40.6194f,-23.9758f,15.3776f,  -40.4489f,-25.1157f,16.1633f,  
	-39.9531f,-26.2856f,14.6181f,  -39.7534f,-27.4383f,15.4232f,  -38.9965f,-28.5386f,13.9338f,  -38.7650f,-29.7006f,14.7599f,  -37.7609f,-30.7058f,13.3515f,  
	-37.4956f,-31.8724f,14.2003f,  -36.2634f,-32.7580f,12.8967f,  -35.9628f,-33.9241f,13.7700f,  -34.5263f,-34.6669f,12.5924f,  -34.1893f,-35.8267f,13.4914f,  
	-32.5770f,-36.4056f,12.4583f,  -32.2038f,-37.5519f,13.3834f,  -30.4481f,-37.9486f,12.5105f,  -30.0407f,-39.0732f,13.4618f,  -28.1776f,-39.2725f,12.7610f,  
	-27.7379f,-40.3680f,13.7380f,  -25.8062f,-40.3585f,13.2179f,  -25.3365f,-41.4175f,14.2195f,  -23.3771f,-41.1919f,13.8848f,  -22.8831f,-42.2056f,14.9081f,  
	-20.9382f,-41.7616f,14.7598f,  -20.4242f,-42.7230f,15.8022f,  -26.5706f,6.1234f,-21.0545f,  -26.6010f,5.1864f,-20.3786f,  -24.6071f,5.1440f,-20.2224f,  
	-24.6369f,4.2127f,-19.5512f,  -22.6738f,4.3335f,-19.1860f,  -22.7039f,3.4050f,-18.5189f,  -20.8064f,3.6840f,-17.9497f,  -20.8367f,2.7568f,-17.2867f,  
	-19.0384f,3.1860f,-16.5224f,  -19.0690f,2.2577f,-15.8629f,  -17.4013f,2.8269f,-14.9157f,  -17.4317f,1.8963f,-14.2594f,  -15.9237f,2.5930f,-13.1446f,  
	-15.9534f,1.6592f,-12.4915f,  -14.6310f,2.4689f,-11.2274f,  -14.6593f,1.5308f,-10.5770f,  -13.5453f,2.4374f,-9.1844f,  -13.5714f,1.4956f,-8.5368f,  
	-12.6849f,2.4815f,-7.0388f,  -12.7082f,1.5353f,-6.3934f,  -12.0641f,2.5818f,-4.8148f,  -12.0840f,1.6309f,-4.1708f,  -11.6931f,2.7185f,-2.5379f,  
	-11.7092f,1.7643f,-1.8956f,  -11.5780f,2.8730f,-0.2355f,  -11.5897f,1.9156f,0.4059f,  -11.7205f,3.0250f,2.0655f,  -11.7275f,2.0657f,2.7062f,  
	-12.1178f,3.1558f,4.3377f,  -12.1202f,2.1947f,4.9786f,  -12.7632f,3.2456f,6.5547f,  -12.7609f,2.2835f,7.1963f,  -13.6457f,3.2761f,8.6909f,  
	-13.6389f,2.3144f,9.3332f,  -14.7504f,3.2306f,10.7214f,  -14.7395f,2.2689f,11.3653f,  -16.0586f,3.0919f,12.6242f,  -16.0440f,2.1311f,13.2699f,  
	-17.5483f,2.8453f,14.3786f,  -17.5304f,1.8851f,15.0268f,  -19.1942f,2.4770f,15.9669f,  -19.1736f,1.5176f,16.6177f,  -20.9682f,1.9753f,17.3739f,  
	-20.9453f,1.0162f,18.0278f,  -22.8400f,1.3302f,18.5878f,  -22.8153f,0.3705f,19.2451f,  -24.7773f,0.5336f,19.6000f,  -24.7509f,-0.4280f,20.2613f,  
	-26.7462f,-0.4204f,20.4058f,  -26.7183f,-1.3848f,21.0709f,  -28.7121f,-1.5348f,21.0032f,  -28.6823f,-2.5043f,21.6732f,  -30.6395f,-2.8113f,21.3951f,  
	-30.6076f,-3.7871f,22.0699f,  -32.4936f,-4.2476f,21.5870f,  -32.4587f,-5.2320f,22.2673f,  -34.2398f,-5.8396f,21.5888f,  -34.2006f,-6.8348f,22.2754f,  
	-35.8449f,-7.5805f,21.4137f,  -35.8002f,-8.5882f,22.1068f,  -37.2775f,-9.4601f,21.0776f,  -37.2250f,-10.4828f,21.7790f,  -38.5083f,-11.4663f,20.6009f,  
	-38.4459f,-12.5050f,21.3107f,  -39.5111f,-13.5831f,20.0049f,  -39.4362f,-14.6393f,20.7247f,  -40.2631f,-15.7926f,19.3146f,  -40.1727f,-16.8680f,20.0461f,  
	-40.7452f,-18.0750f,18.5570f,  -40.6361f,-19.1690f,19.3016f,  -40.9426f,-20.4063f,17.7598f,  -40.8117f,-21.5192f,18.5189f,  -40.8455f,-22.7623f,16.9519f,  
	-40.6893f,-23.8927f,17.7281f,  -40.4489f,-25.1157f,16.1633f,  -40.2642f,-26.2617f,16.9584f,  -39.7534f,-27.4383f,15.4232f,  -39.5372f,-28.5969f,16.2390f,  
	-38.7650f,-29.7006f,14.7599f,  -38.5147f,-30.8677f,15.5982f,  -37.4956f,-31.8724f,14.2003f,  -37.2093f,-33.0434f,15.0633f,  -35.9628f,-33.9241f,13.7700f,  
	-35.6388f,-35.0935f,14.6591f,  -34.1893f,-35.8267f,13.4914f,  -33.8271f,-36.9875f,14.4074f,  -32.2038f,-37.5519f,13.3834f,  -31.8047f,-38.6961f,14.3269f,  
	-30.0407f,-39.0732f,13.4618f,  -29.6057f,-40.1934f,14.4327f,  -27.7379f,-40.3680f,13.7380f,  -27.2689f,-41.4566f,14.7358f,  -25.3365f,-41.4175f,14.2195f,  
	-24.8385f,-42.4650f,15.2416f,  -22.8831f,-42.2056f,14.9081f,  -22.3601f,-43.2047f,15.9524f,  -20.4242f,-42.7230f,15.8022f,  -19.8834f,-43.6655f,16.8640f,  
	-26.6010f,5.1864f,-20.3786f,  -26.6312f,4.2438f,-19.7058f,  -24.6369f,4.2127f,-19.5512f,  -24.6667f,3.2748f,-18.8829f,  -22.7039f,3.4050f,-18.5189f,  
	-22.7333f,2.4702f,-17.8548f,  -20.8367f,2.7568f,-17.2867f,  -20.8661f,1.8225f,-16.6264f,  -19.0690f,2.2577f,-15.8629f,  -19.0981f,1.3225f,-15.2060f,  
	-17.4317f,1.8963f,-14.2594f,  -17.4601f,0.9592f,-13.6059f,  -15.9534f,1.6592f,-12.4915f,  -15.9805f,0.7190f,-11.8409f,  -14.6593f,1.5308f,-10.5770f,  
	-14.6844f,0.5878f,-9.9293f,  -13.5714f,1.4956f,-8.5368f,  -13.5940f,0.5489f,-7.8915f,  -12.7082f,1.5353f,-6.3934f,  -12.7277f,0.5845f,-5.7497f,  
	-12.0840f,1.6309f,-4.1708f,  -12.0997f,0.6773f,-3.5290f,  -11.7092f,1.7643f,-1.8956f,  -11.7207f,0.8079f,-1.2548f,  -11.5897f,1.9156f,0.4059f,  
	-11.5967f,0.9576f,1.0458f,  -11.7275f,2.0657f,2.7062f,  -11.7299f,1.1063f,3.3460f,  -12.1202f,2.1947f,4.9786f,  -12.1178f,1.2344f,5.6189f,  
	-12.7609f,2.2835f,7.1963f,  -12.7541f,1.3240f,7.8371f,  -13.6389f,2.3144f,9.3332f,  -13.6279f,1.3553f,9.9754f,  -14.7395f,2.2689f,11.3653f,  
	-14.7246f,1.3113f,12.0092f,  -16.0440f,2.1311f,13.2699f,  -16.0258f,1.1747f,13.9161f,  -17.5304f,1.8851f,15.0268f,  -17.5095f,0.9303f,15.6754f,  
	-19.1736f,1.5176f,16.6177f,  -19.1503f,0.5639f,17.2692f,  -20.9453f,1.0162f,18.0278f,  -20.9203f,0.0629f,18.6825f,  -22.8153f,0.3705f,19.2451f,  
	-22.7888f,-0.5836f,19.9036f,  -24.7509f,-0.4280f,20.2613f,  -24.7232f,-1.3837f,20.9235f,  -26.7183f,-1.3848f,21.0709f,  -26.6892f,-2.3442f,21.7377f,  
	-28.6823f,-2.5043f,21.6732f,  -28.6518f,-3.4688f,22.3444f,  -30.6076f,-3.7871f,22.0699f,  -30.5748f,-4.7589f,22.7464f,  -32.4587f,-5.2320f,22.2673f,  
	-32.4226f,-6.2134f,22.9497f,  -34.2006f,-6.8348f,22.2754f,  -34.1603f,-7.8277f,22.9639f,  -35.8002f,-8.5882f,22.1068f,  -35.7533f,-9.5952f,22.8030f,  
	-37.2250f,-10.4828f,21.7790f,  -37.1698f,-11.5053f,22.4830f,  -38.4459f,-12.5050f,21.3107f,  -38.3797f,-13.5449f,22.0240f,  -39.4362f,-14.6393f,20.7247f,  
	-39.3562f,-15.6986f,21.4489f,  -40.1727f,-16.8680f,20.0461f,  -40.0755f,-17.9467f,20.7827f,  -40.6361f,-19.1690f,19.3016f,  -40.5185f,-20.2678f,20.0519f,  
	-40.8117f,-21.5192f,18.5189f,  -40.6700f,-22.6374f,19.2857f,  -40.6893f,-23.8927f,17.7281f,  -40.5201f,-25.0290f,18.5131f,  -40.2642f,-26.2617f,16.9584f,  
	-40.0642f,-27.4137f,17.7636f,  -39.5372f,-28.5969f,16.2390f,  -39.3034f,-29.7610f,17.0663f,  -38.5147f,-30.8677f,15.5982f,  -38.2446f,-32.0399f,16.4502f,  
	-37.2093f,-33.0434f,15.0633f,  -36.9005f,-34.2186f,15.9416f,  -35.6388f,-35.0935f,14.6591f,  -35.2902f,-36.2651f,15.5649f,  -33.8271f,-36.9875f,14.4074f,  
	-33.4392f,-38.1477f,15.3417f,  -31.8047f,-38.6961f,14.3269f,  -31.3778f,-39.8375f,15.2901f,  -29.6057f,-40.1934f,14.4327f,  -29.1412f,-41.3083f,15.4246f,  
	-27.2689f,-41.4566f,14.7358f,  -26.7709f,-42.5353f,15.7544f,  -24.8385f,-42.4650f,15.2416f,  -24.3105f,-43.4998f,16.2854f,  -22.3601f,-43.2047f,15.9524f,  
	-21.8088f,-44.1867f,17.0172f,  -19.8834f,-43.6655f,16.8640f,  -19.3153f,-44.5873f,17.9455f,  -26.6312f,4.2438f,-19.7058f,  -26.6612f,3.2955f,-19.0354f,  
	-24.6667f,3.2748f,-18.8829f,  -24.6958f,2.3312f,-18.2171f,  -22.7333f,2.4702f,-17.8548f,  -22.7621f,1.5287f,-17.1930f,  -20.8661f,1.8225f,-16.6264f,  
	-20.8944f,0.8815f,-15.9683f,  -19.0981f,1.3225f,-15.2060f,  -19.1255f,0.3810f,-14.5515f,  -17.4601f,0.9592f,-13.6059f,  -17.4863f,0.0157f,-12.9545f,  
	-15.9805f,0.7190f,-11.8409f,  -16.0048f,-0.2265f,-11.1926f,  -14.6844f,0.5878f,-9.9293f,  -14.7063f,-0.3605f,-9.2836f,  -13.5940f,0.5489f,-7.8915f,  
	-13.6130f,-0.4027f,-7.2476f,  -12.7277f,0.5845f,-5.7497f,  -12.7430f,-0.3694f,-5.1078f,  -12.0997f,0.6773f,-3.5290f,  -12.1110f,-0.2790f,-2.8883f,  
	-11.7207f,0.8079f,-1.2548f,  -11.7276f,-0.1496f,-0.6152f,  -11.5967f,0.9576f,1.0458f,  -11.5990f,-0.0011f,1.6851f,  -11.7299f,1.1063f,3.3460f,  
	-11.7276f,0.1470f,3.9857f,  -12.1178f,1.2344f,5.6189f,  -12.1110f,0.2763f,6.2588f,  -12.7541f,1.3240f,7.8371f,  -12.7429f,0.3667f,8.4783f,  
	-13.6279f,1.3553f,9.9754f,  -13.6128f,0.4000f,10.6180f,  -14.7246f,1.3113f,12.0092f,  -14.7061f,0.3577f,12.6540f,  -16.0258f,1.1747f,13.9161f,  
	-16.0044f,0.2234f,14.5631f,  -17.5095f,0.9303f,15.6754f,  -17.4857f,-0.0190f,16.3250f,  -19.1503f,0.5639f,17.2692f,  -19.1248f,-0.3839f,17.9219f,  
	-20.9203f,0.0629f,18.6825f,  -20.8934f,-0.8847f,19.3388f,  -22.7888f,-0.5836f,19.9036f,  -22.7609f,-1.5315f,20.5634f,  -24.7232f,-1.3837f,20.9235f,  
	-24.6944f,-2.3339f,21.5876f,  -26.6892f,-2.3442f,21.7377f,  -26.6595f,-3.2981f,22.4060f,  -28.6518f,-3.4688f,22.3444f,  -28.6205f,-4.4286f,23.0176f,  
	-30.5748f,-4.7589f,22.7464f,  -30.5413f,-5.7269f,23.4250f,  -32.4226f,-6.2134f,22.9497f,  -32.3859f,-7.1915f,23.6340f,  -34.1603f,-7.8277f,22.9639f,  
	-34.1183f,-8.8189f,23.6552f,  -35.7533f,-9.5952f,22.8030f,  -35.7045f,-10.6011f,23.5016f,  -37.1698f,-11.5053f,22.4830f,  -37.1116f,-12.5281f,23.1902f,  
	-38.3797f,-13.5449f,22.0240f,  -38.3094f,-14.5869f,22.7414f,  -39.3562f,-15.6986f,21.4489f,  -39.2703f,-16.7605f,22.1780f,  -40.0755f,-17.9467f,20.7827f,  
	-39.9710f,-19.0295f,21.5247f,  -40.5185f,-20.2678f,20.0519f,  -40.3915f,-21.3715f,20.8095f,  -40.6700f,-22.6374f,19.2857f,  -40.5168f,-23.7613f,20.0607f,  
	-40.5201f,-25.0290f,18.5131f,  -40.3371f,-26.1712f,19.3075f,  -40.0642f,-27.4137f,17.7636f,  -39.8480f,-28.5714f,18.5796f,  -39.3034f,-29.7610f,17.0663f,  
	-39.0508f,-30.9307f,17.9068f,  -38.2446f,-32.0399f,16.4502f,  -37.9529f,-33.2170f,17.3169f,  -36.9005f,-34.2186f,15.9416f,  -36.5679f,-35.3970f,16.8361f,  
	-35.2902f,-36.2651f,15.5649f,  -34.9163f,-37.4373f,16.4886f,  -33.4392f,-38.1477f,15.3417f,  -33.0238f,-39.3065f,16.2956f,  -31.3778f,-39.8375f,15.2901f,  
	-30.9214f,-40.9754f,16.2744f,  -29.1412f,-41.3083f,15.4246f,  -28.6473f,-42.4152f,16.4379f,  -26.7709f,-42.5353f,15.7544f,  -26.2421f,-43.6033f,16.7956f,  
	-24.3105f,-43.4998f,16.2854f,  -23.7531f,-44.5193f,17.3510f,  -21.8088f,-44.1867f,17.0172f,  -21.2287f,-45.1494f,18.1031f,  -19.3153f,-44.5873f,17.9455f,  
	-18.7182f,-45.4864f,19.0472f,  -26.6612f,3.2955f,-19.0354f,  -26.6906f,2.3420f,-18.3674f,  -24.6958f,2.3312f,-18.2171f,  -24.7244f,1.3813f,-17.5533f,  
	-22.7621f,1.5287f,-17.1930f,  -22.7898f,0.5809f,-16.5332f,  -20.8944f,0.8815f,-15.9683f,  -20.9211f,-0.0656f,-15.3122f,  -19.1255f,0.3810f,-14.5515f,  
	-19.1509f,-0.5669f,-13.8987f,  -17.4863f,0.0157f,-12.9545f,  -17.5099f,-0.9332f,-12.3050f,  -16.0048f,-0.2265f,-11.1926f,  -16.0261f,-1.1774f,-10.5458f,  
	-14.7063f,-0.3605f,-9.2836f,  -14.7248f,-1.3140f,-8.6389f,  -13.6130f,-0.4027f,-7.2476f,  -13.6280f,-1.3580f,-6.6051f,  -12.7430f,-0.3694f,-5.1078f,  
	-12.7541f,-1.3266f,-4.4667f,  -12.1110f,-0.2790f,-2.8883f,  -12.1179f,-1.2370f,-2.2485f,  -11.7276f,-0.1496f,-0.6152f,  -11.7299f,-1.1085f,0.0242f,  
	-11.5990f,-0.0011f,1.6851f,  -11.5967f,-0.9602f,2.3247f,  -11.7276f,0.1470f,3.9857f,  -11.7206f,-0.8106f,4.6253f,  -12.1110f,0.2763f,6.2588f,  
	-12.0996f,-0.6801f,6.8995f,  -12.7429f,0.3667f,8.4783f,  -12.7275f,-0.5873f,9.1202f,  -13.6128f,0.4000f,10.6180f,  -13.5938f,-0.5518f,11.2619f,  
	-14.7061f,0.3577f,12.6540f,  -14.6841f,-0.5910f,13.2999f,  -16.0044f,0.2234f,14.5631f,  -15.9799f,-0.7224f,15.2115f,  -17.4857f,-0.0190f,16.3250f,  
	-17.4594f,-0.9623f,16.9763f,  -19.1248f,-0.3839f,17.9219f,  -19.0972f,-1.3259f,18.5766f,  -20.8934f,-0.8847f,19.3388f,  -20.8650f,-1.8256f,19.9968f,  
	-22.7609f,-1.5315f,20.5634f,  -22.7319f,-2.4733f,21.2254f,  -24.6944f,-2.3339f,21.5876f,  -24.6649f,-3.2779f,22.2535f,  -26.6595f,-3.2981f,22.4060f,  
	-26.6291f,-4.2465f,23.0765f,  -28.6205f,-4.4286f,23.0176f,  -28.5888f,-5.3836f,23.6932f,  -30.5413f,-5.7269f,23.4250f,  -30.5075f,-6.6906f,24.1057f,  
	-32.3859f,-7.1915f,23.6340f,  -32.3481f,-8.1670f,24.3212f,  -34.1183f,-8.8189f,23.6552f,  -34.0752f,-9.8079f,24.3492f,  -35.7045f,-10.6011f,23.5016f,  
	-35.6536f,-11.6062f,24.2034f,  -37.1116f,-12.5281f,23.1902f,  -37.0504f,-13.5518f,23.9014f,  -38.3094f,-14.5869f,22.7414f,  -38.2344f,-15.6305f,23.4634f,  
	-39.2703f,-16.7605f,22.1780f,  -39.1786f,-17.8254f,22.9119f,  -39.9710f,-19.0295f,21.5247f,  -39.8585f,-20.1164f,22.2734f,  -40.3915f,-21.3715f,20.8095f,  
	-40.2544f,-22.4803f,21.5747f,  -40.5168f,-23.7613f,20.0607f,  -40.3512f,-24.8907f,20.8445f,  -40.3371f,-26.1712f,19.3075f,  -40.1393f,-27.3190f,20.1123f,  
	-39.8480f,-28.5714f,18.5796f,  -39.6143f,-29.7348f,19.4083f,  -39.0508f,-30.9307f,17.9068f,  -38.7780f,-32.1058f,18.7613f,  -37.9529f,-33.2170f,17.3169f,  
	-37.6386f,-34.3981f,18.1992f,  -36.5679f,-35.3970f,16.8361f,  -36.2108f,-36.5771f,17.7481f,  -34.9163f,-37.4373f,16.4886f,  -34.5155f,-38.6095f,17.4318f,  
	-33.0238f,-39.3065f,16.2956f,  -32.5791f,-40.4635f,17.2705f,  -30.9214f,-40.9754f,16.2744f,  -30.4354f,-42.1069f,17.2803f,  -28.6473f,-42.4152f,16.4379f,  
	-28.1219f,-43.5133f,17.4742f,  -26.2421f,-43.6033f,16.7956f,  -25.6829f,-44.6577f,17.8595f,  -23.7531f,-44.5193f,17.3510f,  -23.1655f,-45.5213f,18.4388f,  
	-21.2287f,-45.1494f,18.1031f,  -20.6180f,-46.0914f,19.2110f,  -18.7182f,-45.4864f,19.0472f,  -18.0924f,-46.3605f,20.1691f,  -26.6906f,2.3420f,-18.3674f,  
	-26.7194f,1.3829f,-17.7008f,  -24.7244f,1.3813f,-17.5533f,  -24.7519f,0.4256f,-16.8910f,  -22.7898f,0.5809f,-16.5332f,  -22.8161f,-0.3729f,-15.8749f,  
	-20.9211f,-0.0656f,-15.3122f,  -20.9460f,-1.0189f,-14.6574f,  -19.1509f,-0.5669f,-13.8987f,  -19.1740f,-1.5202f,-13.2474f,  -17.5099f,-0.9332f,-12.3050f,  
	-17.5307f,-1.8876f,-11.6565f,  -16.0261f,-1.1774f,-10.5458f,  -16.0442f,-2.1336f,-9.8997f,  -14.7248f,-1.3140f,-8.6389f,  -14.7396f,-2.2714f,-7.9950f,  
	-13.6280f,-1.3580f,-6.6051f,  -13.6390f,-2.3168f,-5.9628f,  -12.7541f,-1.3266f,-4.4667f,  -12.7609f,-2.2859f,-3.8259f,  -12.1179f,-1.2370f,-2.2485f,  
	-12.1202f,-2.1969f,-1.6084f,  -11.7299f,-1.1085f,0.0242f,  -11.7275f,-2.0683f,0.6642f,  -11.5967f,-0.9602f,2.3247f,  -11.5897f,-1.9182f,2.9645f,  
	-11.7206f,-0.8106f,4.6253f,  -11.7091f,-1.7670f,5.2661f,  -12.0996f,-0.6801f,6.8995f,  -12.0839f,-1.6337f,7.5413f,  -12.7275f,-0.5873f,9.1202f,  
	-12.7081f,-1.5382f,9.7638f,  -13.5938f,-0.5518f,11.2619f,  -13.5712f,-1.4990f,11.9074f,  -14.6841f,-0.5910f,13.2999f,  -14.6588f,-1.5345f,13.9476f,  
	-15.9799f,-0.7224f,15.2115f,  -15.9527f,-1.6625f,15.8619f,  -17.4594f,-0.9623f,16.9763f,  -17.4308f,-1.8999f,17.6300f,  -19.0972f,-1.3259f,18.5766f,  
	-19.0679f,-2.2612f,19.2334f,  -20.8650f,-1.8256f,19.9968f,  -20.8353f,-2.7603f,20.6573f,  -22.7319f,-2.4733f,21.2254f,  -22.7021f,-3.4086f,21.8896f,  
	-24.6649f,-3.2779f,22.2535f,  -24.6348f,-4.2159f,22.9221f,  -26.6291f,-4.2465f,23.0765f,  -26.5985f,-5.1895f,23.7497f,  -28.5888f,-5.3836f,23.6932f,  
	-28.5570f,-6.3335f,24.3711f,  -30.5075f,-6.6906f,24.1057f,  -30.4730f,-7.6507f,24.7896f,  -32.3481f,-8.1670f,24.3212f,  -32.3098f,-9.1392f,25.0112f,  
	-34.0752f,-9.8079f,24.3492f,  -34.0307f,-10.7951f,25.0463f,  -35.6536f,-11.6062f,24.2034f,  -35.6006f,-12.6111f,24.9091f,  -37.0504f,-13.5518f,23.9014f,  
	-36.9855f,-14.5763f,24.6170f,  -38.2344f,-15.6305f,23.4634f,  -38.1549f,-16.6763f,24.1900f,  -39.1786f,-17.8254f,22.9119f,  -39.0801f,-18.8938f,23.6522f,  
	-39.8585f,-20.1164f,22.2734f,  -39.7372f,-21.2078f,23.0291f,  -40.2544f,-22.4803f,21.5747f,  -40.1063f,-23.5941f,22.3481f,  -40.3512f,-24.8907f,20.8445f,  
	-40.1722f,-26.0255f,21.6380f,  -40.1393f,-27.3190f,20.1123f,  -39.9253f,-28.4727f,20.9290f,  -39.6143f,-29.7348f,19.4083f,  -39.3617f,-30.9040f,20.2503f,  
	-38.7780f,-32.1058f,18.7613f,  -38.4837f,-33.2855f,19.6309f,  -37.6386f,-34.3981f,18.1992f,  -37.3006f,-35.5818f,19.0987f,  -36.2108f,-36.5771f,17.7481f,  
	-35.8275f,-37.7584f,18.6794f,  -34.5155f,-38.6095f,17.4318f,  -34.0858f,-39.7813f,18.3959f,  -32.5791f,-40.4635f,17.2705f,  -32.1048f,-41.6156f,18.2672f,  
	-30.4354f,-42.1069f,17.2803f,  -29.9176f,-43.2314f,18.3097f,  -28.1219f,-43.5133f,17.4742f,  -27.5653f,-44.5996f,18.5342f,  -25.6829f,-44.6577f,17.8595f,  
	-25.0925f,-45.6965f,18.9469f,  -23.1655f,-45.5213f,18.4388f,  -22.5458f,-46.5044f,19.5503f,  -20.6180f,-46.0914f,19.2110f,  -19.9768f,-47.0100f,20.3408f,  
	-18.0924f,-46.3605f,20.1691f,  -17.4384f,-47.2073f,21.3104f,  -26.7194f,1.3829f,-17.7008f,  -26.7472f,0.4185f,-17.0357f,  -24.7519f,0.4256f,-16.8910f,  
	-24.7781f,-0.5357f,-16.2300f,  -22.8161f,-0.3729f,-15.8749f,  -22.8407f,-1.3326f,-15.2176f,  -20.9460f,-1.0189f,-14.6574f,  -20.9687f,-1.9776f,-14.0038f,  
	-19.1740f,-1.5202f,-13.2474f,  -19.1945f,-2.4792f,-12.5968f,  -17.5307f,-1.8876f,-11.6565f,  -17.5485f,-2.8476f,-11.0084f,  -16.0442f,-2.1336f,-9.8997f,  
	-16.0588f,-3.0942f,-9.2540f,  -14.7396f,-2.2714f,-7.9950f,  -14.7505f,-3.2329f,-7.3512f,  -13.6390f,-2.3168f,-5.9628f,  -13.6457f,-3.2785f,-5.3206f,  
	-12.7609f,-2.2859f,-3.8259f,  -12.7632f,-3.2477f,-3.1846f,  -12.1202f,-2.1969f,-1.6084f,  -12.1178f,-3.1582f,-0.9673f,  -11.7275f,-2.0683f,0.6642f,  
	-11.7205f,-3.0276f,1.3049f,  -11.5897f,-1.9182f,2.9645f,  -11.5780f,-2.8756f,3.6060f,  -11.7091f,-1.7670f,5.2661f,  -11.6931f,-2.7213f,5.9083f,  
	-12.0839f,-1.6337f,7.5413f,  -12.0639f,-2.5847f,8.1852f,  -12.7081f,-1.5382f,9.7638f,  -12.6846f,-2.4849f,10.4094f,  -13.5712f,-1.4990f,11.9074f,  
	-13.5449f,-2.4411f,12.5550f,  -14.6588f,-1.5345f,13.9476f,  -14.6305f,-2.4723f,14.5978f,  -15.9527f,-1.6625f,15.8619f,  -15.9229f,-2.5968f,16.5152f,  
	-17.4308f,-1.8999f,17.6300f,  -17.4003f,-2.8306f,18.2863f,  -19.0679f,-2.2612f,19.2334f,  -19.0370f,-3.1898f,19.8931f,  -20.8353f,-2.7603f,20.6573f,  
	-20.8047f,-3.6879f,21.3205f,  -22.7021f,-3.4086f,21.8896f,  -22.6717f,-4.3373f,22.5568f,  -24.6348f,-4.2159f,22.9221f,  -24.6045f,-5.1477f,23.5935f,  
	-26.5985f,-5.1895f,23.7497f,  -26.5680f,-6.1264f,24.4255f,  -28.5570f,-6.3335f,24.3711f,  -28.5249f,-7.2789f,25.0523f,  -30.4730f,-7.6507f,24.7896f,  
	-30.4384f,-8.6066f,25.4763f,  -32.3098f,-9.1392f,25.0112f,  -32.2707f,-10.1086f,25.7044f,  -34.0307f,-10.7951f,25.0463f,  -33.9850f,-11.7809f,25.7472f,  
	-35.6006f,-12.6111f,24.9091f,  -35.5449f,-13.6157f,25.6190f,  -36.9855f,-14.5763f,24.6170f,  -36.9172f,-15.6017f,25.3369f,  -38.1549f,-16.6763f,24.1900f,  
	-38.0696f,-17.7244f,24.9226f,  -39.0801f,-18.8938f,23.6522f,  -38.9742f,-19.9657f,24.3992f,  -39.7372f,-21.2078f,23.0291f,  -39.6065f,-22.3035f,23.7926f,  
	-40.1063f,-23.5941f,22.3481f,  -39.9463f,-24.7129f,23.1306f,  -40.1722f,-26.0255f,21.6380f,  -39.9786f,-27.1659f,22.4428f,  -39.9253f,-28.4727f,20.9290f,  
	-39.6939f,-29.6322f,21.7584f,  -39.3617f,-30.9040f,20.2503f,  -39.0890f,-32.0781f,21.1067f,  -38.4837f,-33.2855f,19.6309f,  -38.1668f,-34.4684f,20.5172f,  
	-37.3006f,-35.5818f,19.0987f,  -36.9372f,-36.7677f,20.0170f,  -35.8275f,-37.7584f,18.6794f,  -35.4159f,-38.9406f,19.6313f,  -34.0858f,-39.7813f,18.3959f,  
	-33.6268f,-40.9498f,19.3817f,  -32.1048f,-41.6156f,18.2672f,  -31.5986f,-42.7625f,19.2877f,  -29.9176f,-43.2314f,18.3097f,  -29.3681f,-44.3460f,19.3635f,  
	-27.5653f,-44.5996f,18.5342f,  -26.9767f,-45.6720f,19.6187f,  -25.0925f,-45.6965f,18.9469f,  -24.4687f,-46.7182f,20.0594f,  -22.5458f,-46.5044f,19.5503f,  
	-21.8940f,-47.4657f,20.6855f,  -19.9768f,-47.0100f,20.3408f,  -19.3054f,-47.9026f,21.4923f,  -17.4384f,-47.2073f,21.3104f,  -16.7560f,-48.0245f,22.4709f,  
	-26.7472f,0.4185f,-17.0357f,  -26.7735f,-0.5511f,-16.3714f,  -24.7781f,-0.5357f,-16.2300f,  -24.8026f,-1.5025f,-15.5697f,  -22.8407f,-1.3326f,-15.2176f,  
	-22.8632f,-2.2975f,-14.5611f,  -20.9687f,-1.9776f,-14.0038f,  -20.9889f,-2.9420f,-13.3504f,  -19.1945f,-2.4792f,-12.5968f,  -19.2120f,-3.4437f,-11.9462f,  
	-17.5485f,-2.8476f,-11.0084f,  -17.5629f,-3.8121f,-10.3604f,  -16.0588f,-3.0942f,-9.2540f,  -16.0695f,-4.0591f,-8.6080f,  -14.7505f,-3.2329f,-7.3512f,  
	-14.7572f,-4.1976f,-6.7070f,  -13.6457f,-3.2785f,-5.3206f,  -13.6480f,-4.2429f,-4.6775f,  -12.7632f,-3.2477f,-3.1846f,  -12.7608f,-4.2114f,-2.5419f,  
	-12.1178f,-3.1582f,-0.9673f,  -12.1106f,-4.1197f,-0.3252f,  -11.7205f,-3.0276f,1.3049f,  -11.7086f,-3.9869f,1.9476f,  -11.5780f,-2.8756f,3.6060f,  
	-11.5616f,-3.8314f,4.2493f,  -11.6931f,-2.7213f,5.9083f,  -11.6725f,-3.6734f,6.5531f,  -12.0639f,-2.5847f,8.1852f,  -12.0397f,-3.5320f,8.8315f,  
	-12.6846f,-2.4849f,10.4094f,  -12.6573f,-3.4269f,11.0575f,  -13.5449f,-2.4411f,12.5550f,  -13.5153f,-3.3780f,13.2056f,  -14.6305f,-2.4723f,14.5978f,  
	-14.5992f,-3.4046f,15.2514f,  -15.9229f,-2.5968f,16.5152f,  -15.8909f,-3.5242f,17.1716f,  -17.4003f,-2.8306f,18.2863f,  -17.3680f,-3.7545f,18.9457f,  
	-19.0370f,-3.1898f,19.8931f,  -19.0051f,-4.1111f,20.5560f,  -20.8047f,-3.6879f,21.3205f,  -20.7733f,-4.6086f,21.9871f,  -22.6717f,-4.3373f,22.5568f,  
	-22.6409f,-5.2591f,23.2273f,  -24.6045f,-5.1477f,23.5935f,  -24.5744f,-6.0727f,24.2680f,  -26.5680f,-6.1264f,24.4255f,  -26.5374f,-7.0579f,25.1048f,  
	-28.5249f,-7.2789f,25.0523f,  -28.4931f,-8.2190f,25.7366f,  -30.4384f,-8.6066f,25.4763f,  -30.4036f,-9.5585f,26.1665f,  -32.2707f,-10.1086f,25.7044f,  
	-32.2310f,-11.0753f,26.4014f,  -33.9850f,-11.7809f,25.7472f,  -33.9373f,-12.7652f,26.4524f,  -35.5449f,-13.6157f,25.6190f,  -35.4869f,-14.6200f,26.3332f,  
	-36.9172f,-15.6017f,25.3369f,  -36.8444f,-16.6284f,26.0626f,  -38.0696f,-17.7244f,24.9226f,  -37.9785f,-18.7752f,25.6615f,  -38.9742f,-19.9657f,24.3992f,  
	-38.8604f,-21.0411f,25.1534f,  -39.6065f,-22.3035f,23.7926f,  -39.4653f,-23.4036f,24.5647f,  -39.9463f,-24.7129f,23.1306f,  -39.7732f,-25.8369f,23.9239f,  
	-39.9786f,-27.1659f,22.4428f,  -39.7691f,-28.3121f,23.2596f,  -39.6939f,-29.6322f,21.7584f,  -39.4439f,-30.7968f,22.6015f,  -39.0890f,-32.0781f,21.1067f,  
	-38.7949f,-33.2561f,21.9794f,  -38.1668f,-34.4684f,20.5172f,  -37.8256f,-35.6545f,21.4218f,  -36.9372f,-36.7677f,20.0170f,  -36.5464f,-37.9557f,20.9556f,  
	-35.4159f,-38.9406f,19.6313f,  -34.9754f,-40.1207f,20.6048f,  -33.6268f,-40.9498f,19.3817f,  -33.1361f,-42.1145f,20.3915f,  -31.5986f,-42.7625f,19.2877f,  
	-31.0605f,-43.9011f,20.3331f,  -29.3681f,-44.3460f,19.3635f,  -28.7859f,-45.4483f,20.4426f,  -26.9767f,-45.6720f,19.6187f,  -26.3536f,-46.7293f,20.7294f,  
	-24.4687f,-46.7182f,20.0594f,  -23.8112f,-47.7199f,21.1971f,  -21.8940f,-47.4657f,20.6855f,  -21.2102f,-48.4025f,21.8443f,  -19.3054f,-47.9026f,21.4923f,  
	-18.6035f,-48.7669f,22.6655f,  -16.7560f,-48.0245f,22.4709f,  -16.0451f,-48.8098f,23.6502f,  -26.7735f,-0.5511f,-16.3714f,  -26.7982f,-1.5258f,-15.7077f,  
	-24.8026f,-1.5025f,-15.5697f,  -24.8251f,-2.4744f,-14.9099f,  -22.8632f,-2.2975f,-14.5611f,  -22.8833f,-3.2680f,-13.9046f,  -20.9889f,-2.9420f,-13.3504f,  
	-21.0063f,-3.9118f,-12.6969f,  -19.2120f,-3.4437f,-11.9462f,  -19.2263f,-4.4130f,-11.2954f,  -17.5629f,-3.8121f,-10.3604f,  -17.5736f,-4.7813f,-9.7118f,  
	-16.0695f,-4.0591f,-8.6080f,  -16.0762f,-5.0276f,-7.9613f,  -14.7572f,-4.1976f,-6.7070f,  -14.7595f,-5.1655f,-6.0615f,  -13.6480f,-4.2429f,-4.6775f,  
	-13.6456f,-5.2099f,-4.0327f,  -12.7608f,-4.2114f,-2.5419f,  -12.7536f,-5.1759f,-1.8978f,  -12.1106f,-4.1197f,-0.3252f,  -12.0985f,-5.0818f,0.3193f,  
	-11.7086f,-3.9869f,1.9476f,  -11.6918f,-4.9451f,2.5925f,  -11.5616f,-3.8314f,4.2493f,  -11.5404f,-4.7857f,4.8955f,  -11.6725f,-3.6734f,6.5531f,  
	-11.6474f,-4.6224f,7.2007f,  -12.0397f,-3.5320f,8.8315f,  -12.0112f,-4.4750f,9.4807f,  -12.6573f,-3.4269f,11.0575f,  -12.6263f,-4.3640f,11.7091f,  
	-13.5153f,-3.3780f,13.2056f,  -13.4824f,-4.3096f,13.8600f,  -14.5992f,-3.4046f,15.2514f,  -14.5653f,-4.3301f,15.9085f,  -15.8909f,-3.5242f,17.1716f,  
	-15.8566f,-4.4447f,17.8316f,  -17.3680f,-3.7545f,18.9457f,  -17.3342f,-4.6709f,19.6090f,  -19.0051f,-4.1111f,20.5560f,  -18.9721f,-5.0251f,21.2227f,  
	-20.7733f,-4.6086f,21.9871f,  -20.7414f,-5.5218f,22.6574f,  -22.6409f,-5.2591f,23.2273f,  -22.6103f,-6.1735f,23.9012f,  -24.5744f,-6.0727f,24.2680f,  
	-24.5442f,-6.9914f,24.9463f,  -26.5374f,-7.0579f,25.1048f,  -26.5072f,-7.9832f,25.7877f,  -28.4931f,-8.2190f,25.7366f,  -28.4615f,-9.1541f,26.4247f,  
	-30.4036f,-9.5585f,26.1665f,  -30.3689f,-10.5065f,26.8606f,  -32.2310f,-11.0753f,26.4014f,  -32.1902f,-12.0394f,27.1028f,  -33.9373f,-12.7652f,26.4524f,  
	-33.8883f,-13.7480f,27.1617f,  -35.4869f,-14.6200f,26.3332f,  -35.4254f,-15.6245f,27.0529f,  -36.8444f,-16.6284f,26.0626f,  -36.7670f,-17.6566f,26.7943f,  
	-37.9785f,-18.7752f,25.6615f,  -37.8807f,-19.8284f,26.4072f,  -38.8604f,-21.0411f,25.1534f,  -38.7376f,-22.1201f,25.9157f,  -39.4653f,-23.4036f,24.5647f,  
	-39.3126f,-24.5084f,25.3469f,  -39.7732f,-25.8369f,23.9239f,  -39.5858f,-26.9663f,24.7285f,  -39.7691f,-28.3121f,23.2596f,  -39.5425f,-29.4633f,24.0895f,  
	-39.4439f,-30.7968f,22.6015f,  -39.1739f,-31.9655f,23.4602f,  -38.7949f,-33.2561f,21.9794f,  -38.4777f,-34.4377f,22.8697f,  -37.8256f,-35.6545f,21.4218f,  
	-37.4582f,-36.8434f,22.3462f,  -36.5464f,-37.9557f,20.9556f,  -36.1276f,-39.1426f,21.9156f,  -34.9754f,-40.1207f,20.6048f,  -34.5037f,-41.2986f,21.6022f,  
	-33.1361f,-42.1145f,20.3915f,  -32.6134f,-43.2727f,21.4266f,  -31.0605f,-43.9011f,20.3331f,  -30.4891f,-45.0292f,21.4045f,  -28.7859f,-45.4483f,20.4426f,  
	-28.1684f,-46.5374f,21.5491f,  -26.3536f,-46.7293f,20.7294f,  -25.6956f,-47.7683f,21.8669f,  -23.8112f,-47.7199f,21.1971f,  -23.1202f,-48.6987f,22.3604f,  
	-21.2102f,-48.4025f,21.8443f,  -20.4938f,-49.3125f,23.0273f,  -18.6035f,-48.7669f,22.6655f,  -17.8708f,-49.6002f,23.8602f,  -16.0451f,-48.8098f,23.6502f,  
	-15.3058f,-49.5606f,24.8475f,  -26.7982f,-1.5258f,-15.7077f,  -26.8208f,-2.5054f,-15.0442f,  -24.8251f,-2.4744f,-14.9099f,  -24.8452f,-3.4517f,-14.2499f,  
	-22.8833f,-3.2680f,-13.9046f,  -22.9007f,-4.2438f,-13.2477f,  -21.0063f,-3.9118f,-12.6969f,  -21.0206f,-4.8865f,-12.0428f,  -19.2263f,-4.4130f,-11.2954f,  
	-19.2370f,-5.3871f,-10.6436f,  -17.5736f,-4.7813f,-9.7118f,  -17.5803f,-5.7543f,-9.0621f,  -16.0762f,-5.0276f,-7.9613f,  -16.0785f,-5.9998f,-7.3130f,  
	-14.7595f,-5.1655f,-6.0615f,  -14.7571f,-6.1365f,-5.4141f,  -13.6456f,-5.2099f,-4.0327f,  -13.6383f,-6.1781f,-3.3861f,  -12.7536f,-5.1759f,-1.8978f,  
	-12.7413f,-6.1415f,-1.2510f,  -12.0985f,-5.0818f,0.3193f,  -12.0814f,-6.0434f,0.9664f,  -11.6918f,-4.9451f,2.5925f,  -11.6699f,-5.9024f,3.2408f,  
	-11.5404f,-4.7857f,4.8955f,  -11.5144f,-5.7373f,5.5450f,  -11.6474f,-4.6224f,7.2007f,  -11.6177f,-5.5675f,7.8517f,  -12.0112f,-4.4750f,9.4807f,  
	-11.9786f,-5.4136f,10.1340f,  -12.6263f,-4.3640f,11.7091f,  -12.5915f,-5.2962f,12.3650f,  -13.4824f,-4.3096f,13.8600f,  -13.4464f,-5.2346f,14.5185f,  
	-14.5653f,-4.3301f,15.9085f,  -14.5288f,-5.2487f,16.5696f,  -15.8566f,-4.4447f,17.8316f,  -15.8205f,-5.3578f,18.4959f,  -17.3342f,-4.6709f,19.6090f,  
	-17.2990f,-5.5798f,20.2766f,  -18.9721f,-5.0251f,21.2227f,  -18.9383f,-5.9312f,21.8935f,  -20.7414f,-5.5218f,22.6574f,  -20.7095f,-6.4269f,23.3316f,  
	-22.6103f,-6.1735f,23.9012f,  -22.5797f,-7.0808f,24.5794f,  -24.5442f,-6.9914f,24.9463f,  -24.5146f,-7.9031f,25.6285f,  -26.5072f,-7.9832f,25.7877f,  
	-26.4775f,-8.9025f,26.4745f,  -28.4615f,-9.1541f,26.4247f,  -28.4303f,-10.0841f,27.1168f,  -30.3689f,-10.5065f,26.8606f,  -30.3335f,-11.4507f,27.5591f,  
	-32.1902f,-12.0394f,27.1028f,  -32.1489f,-13.0007f,27.8082f,  -33.8883f,-13.7480f,27.1617f,  -33.8369f,-14.7298f,27.8764f,  -35.4254f,-15.6245f,27.0529f,  
	-35.3605f,-16.6292f,27.7783f,  -36.7670f,-17.6566f,26.7943f,  -36.6843f,-18.6863f,27.5324f,  -37.8807f,-19.8284f,26.4072f,  -37.7755f,-20.8843f,27.1604f,  
	-38.7376f,-22.1201f,25.9157f,  -38.6048f,-23.2030f,26.6875f,  -39.3126f,-24.5084f,25.3469f,  -39.1473f,-25.6179f,26.1397f,  -39.5858f,-26.9663f,24.7285f,  
	-39.3831f,-28.1005f,25.5453f,  -39.5425f,-29.4633f,24.0895f,  -39.2974f,-30.6187f,24.9342f,  -39.1739f,-31.9655f,23.4602f,  -38.8822f,-33.1382f,24.3359f,  
	-38.4777f,-34.4377f,22.8697f,  -38.1356f,-35.6228f,23.7791f,  -37.4582f,-36.8434f,22.3462f,  -37.0636f,-38.0323f,23.2915f,  -36.1276f,-39.1426f,21.9156f,  
	-35.6782f,-40.3287f,22.8991f,  -34.5037f,-41.2986f,21.6022f,  -34.0003f,-42.4714f,22.6251f,  -32.6134f,-43.2727f,21.4266f,  -32.0573f,-44.4219f,22.4881f,  
	-30.4891f,-45.0292f,21.4045f,  -29.8820f,-46.1459f,22.5042f,  -28.1684f,-46.5374f,21.5491f,  -27.5150f,-47.6101f,22.6835f,  -25.6956f,-47.7683f,21.8669f,  
	-25.0027f,-48.7860f,23.0317f,  -23.1202f,-48.6987f,22.3604f,  -22.3947f,-49.6521f,23.5502f,  -20.4938f,-49.3125f,23.0273f,  -19.7446f,-50.1927f,24.2344f,  
	-17.8708f,-49.6002f,23.8602f,  -17.1073f,-50.4001f,25.0760f,  -15.3058f,-49.5606f,24.8475f,  -14.5368f,-50.2748f,26.0629f,  -26.8208f,-2.5054f,-15.0442f,  
	-26.8411f,-3.4900f,-14.3803f,  -24.8452f,-3.4517f,-14.2499f,  -24.8626f,-4.4341f,-13.5892f,  -22.9007f,-4.2438f,-13.2477f,  -22.9149f,-5.2246f,-12.5900f,  
	-21.0206f,-4.8865f,-12.0428f,  -21.0312f,-5.8661f,-11.3876f,  -19.2370f,-5.3871f,-10.6436f,  -19.2437f,-6.3653f,-9.9906f,  -17.5803f,-5.7543f,-9.0621f,  
	-17.5826f,-6.7314f,-8.4106f,  -16.0785f,-5.9998f,-7.3130f,  -16.0761f,-6.9753f,-6.6625f,  -14.7571f,-6.1365f,-5.4141f,  -14.7496f,-7.1092f,-4.7645f,  
	-13.6383f,-6.1781f,-3.3861f,  -13.6258f,-7.1481f,-2.7365f,  -12.7413f,-6.1415f,-1.2510f,  -12.7237f,-7.1073f,-0.6013f,  -12.0814f,-6.0434f,0.9664f,  
	-12.0589f,-7.0046f,1.6172f,  -11.6699f,-5.9024f,3.2408f,  -11.6430f,-6.8577f,3.8927f,  -11.5144f,-5.7373f,5.5450f,  -11.4834f,-6.6855f,6.1982f,  
	-11.6177f,-5.5675f,7.8517f,  -11.5834f,-6.5086f,8.5071f,  -11.9786f,-5.4136f,10.1340f,  -11.9418f,-6.3475f,10.7919f,  -12.5915f,-5.2962f,12.3650f,  
	-12.5531f,-6.2220f,13.0254f,  -13.4464f,-5.2346f,14.5185f,  -13.4072f,-6.1527f,15.1814f,  -14.5288f,-5.2487f,16.5696f,  -14.4899f,-6.1599f,17.2356f,  
	-15.8205f,-5.3578f,18.4959f,  -15.7825f,-6.2631f,19.1650f,  -17.2990f,-5.5798f,20.2766f,  -17.2626f,-6.4803f,20.9486f,  -18.9383f,-5.9312f,21.8935f,  
	-18.9042f,-6.8287f,22.5688f,  -20.7095f,-6.4269f,23.3316f,  -20.6773f,-7.3243f,24.0105f,  -22.5797f,-7.0808f,24.5794f,  -22.5495f,-7.9802f,25.2620f,  
	-24.5146f,-7.9031f,25.6285f,  -24.4856f,-8.8078f,26.3152f,  -26.4775f,-8.9025f,26.4745f,  -26.4485f,-9.8154f,27.1656f,  -28.4303f,-10.0841f,27.1168f,  
	-28.3990f,-11.0092f,27.8135f,  -30.3335f,-11.4507f,27.5591f,  -30.2984f,-12.3908f,28.2618f,  -32.1489f,-13.0007f,27.8082f,  -32.1060f,-13.9596f,28.5190f,  
	-33.8369f,-14.7298f,27.8764f,  -33.7830f,-15.7105f,28.5966f,  -35.3605f,-16.6292f,27.7783f,  -35.2915f,-17.6342f,28.5098f,  -36.6843f,-18.6863f,27.5324f,  
	-36.5956f,-19.7174f,28.2777f,  -37.7755f,-20.8843f,27.1604f,  -37.6618f,-21.9432f,27.9227f,  -38.6048f,-23.2030f,26.6875f,  -38.4611f,-24.2899f,27.4693f,  
	-39.1473f,-25.6179f,26.1397f,  -38.9683f,-26.7318f,26.9441f,  -39.3831f,-28.1005f,25.5453f,  -39.1633f,-29.2387f,26.3764f,  -39.2974f,-30.6187f,24.9342f,  
	-39.0321f,-31.7782f,25.7954f,  -38.8822f,-33.1382f,24.3359f,  -38.5670f,-34.3148f,25.2301f,  -38.1356f,-35.6228f,23.7791f,  -37.7674f,-36.8087f,24.7091f,  
	-37.0636f,-38.0323f,23.2915f,  -36.6393f,-39.2214f,24.2601f,  -35.6782f,-40.3287f,22.8991f,  -35.1974f,-41.5109f,23.9083f,  -34.0003f,-42.4714f,22.6251f,  
	-33.4635f,-43.6366f,23.6748f,  -32.0573f,-44.4219f,22.4881f,  -31.4651f,-45.5614f,23.5786f,  -29.8820f,-46.1459f,22.5042f,  -29.2383f,-47.2478f,23.6329f,  
	-27.5150f,-47.6101f,22.6835f,  -26.8255f,-48.6629f,23.8468f,  -25.0027f,-48.7860f,23.0317f,  -24.2737f,-49.7799f,24.2250f,  -22.3947f,-49.6521f,23.5502f,  
	-21.6344f,-50.5770f,24.7665f,  -19.7446f,-50.1927f,24.2344f,  -18.9621f,-51.0404f,25.4656f,  -17.1073f,-50.4001f,25.0760f,  -16.3114f,-51.1639f,26.3130f,  
	-14.5368f,-50.2748f,26.0629f,  -13.7399f,-50.9496f,27.2943f,  -26.8411f,-3.4900f,-14.3803f,  -26.8586f,-4.4794f,-13.7156f,  -24.8626f,-4.4341f,-13.5892f,  
	-24.8769f,-5.4214f,-12.9276f,  -22.9149f,-5.2246f,-12.5900f,  -22.9255f,-6.2102f,-11.9309f,  -21.0312f,-5.8661f,-11.3876f,  -21.0379f,-6.8501f,-10.7308f,  
	-19.2437f,-6.3653f,-9.9906f,  -19.2460f,-7.3478f,-9.3354f,  -17.5826f,-6.7314f,-8.4106f,  -17.5801f,-7.7121f,-7.7566f,  -16.0761f,-6.9753f,-6.6625f,  
	-16.0686f,-7.9532f,-6.0096f,  -14.7496f,-7.1092f,-4.7645f,  -14.7369f,-8.0842f,-4.1117f,  -13.6258f,-7.1481f,-2.7365f,  -13.6078f,-8.1188f,-2.0836f,  
	-12.7237f,-7.1073f,-0.6013f,  -12.7006f,-8.0733f,0.0525f,  -12.0589f,-7.0046f,1.6172f,  -12.0310f,-7.9644f,2.2719f,  -11.6430f,-6.8577f,3.8927f,  
	-11.6107f,-7.8102f,4.5486f,  -11.4834f,-6.6855f,6.1982f,  -11.4474f,-7.6305f,6.8563f,  -11.5834f,-6.5086f,8.5071f,  -11.5445f,-7.4456f,9.1676f,  
	-11.9418f,-6.3475f,10.7919f,  -11.9010f,-7.2756f,11.4548f,  -12.5531f,-6.2220f,13.0254f,  -12.5112f,-7.1412f,13.6907f,  -13.4072f,-6.1527f,15.1814f,  
	-13.3652f,-7.0636f,15.8497f,  -14.4899f,-6.1599f,17.2356f,  -14.4487f,-7.0633f,17.9069f,  -15.7825f,-6.2631f,19.1650f,  -15.7430f,-7.1598f,19.8391f,  
	-17.2626f,-6.4803f,20.9486f,  -17.2256f,-7.3720f,21.6259f,  -18.9042f,-6.8287f,22.5688f,  -18.8696f,-7.7179f,23.2492f,  -20.6773f,-7.3243f,24.0105f,  
	-20.6454f,-8.2131f,24.6942f,  -22.5495f,-7.9802f,25.2620f,  -22.5198f,-8.8718f,25.9493f,  -24.4856f,-8.8078f,26.3152f,  -24.4574f,-9.7050f,27.0063f,  
	-26.4485f,-9.8154f,27.1656f,  -26.4197f,-10.7224f,27.8616f,  -28.3990f,-11.0092f,27.8135f,  -28.3684f,-11.9288f,28.5146f,  -30.2984f,-12.3908f,28.2618f,  
	-30.2624f,-13.3273f,28.9699f,  -32.1060f,-13.9596f,28.5190f,  -32.0616f,-14.9162f,29.2353f,  -33.7830f,-15.7105f,28.5966f,  -33.7263f,-16.6901f,29.3228f,  
	-35.2915f,-17.6342f,28.5098f,  -35.2179f,-18.6394f,29.2482f,  -36.5956f,-19.7174f,28.2777f,  -36.4998f,-20.7505f,29.0314f,  -37.6618f,-21.9432f,27.9227f,  
	-37.5388f,-23.0051f,28.6943f,  -38.4611f,-24.2899f,27.4693f,  -38.3054f,-25.3804f,28.2619f,  -38.9683f,-26.7318f,26.9441f,  -38.7739f,-27.8492f,27.7620f,  
	-39.1633f,-29.2387f,26.3764f,  -38.9250f,-30.3810f,27.2231f,  -39.0321f,-31.7782f,25.7954f,  -38.7450f,-32.9418f,26.6741f,  -38.5670f,-34.3148f,25.2301f,  
	-38.2269f,-35.4928f,26.1441f,  -37.7674f,-36.8087f,24.7091f,  -37.3707f,-37.9956f,25.6617f,  -36.6393f,-39.2214f,24.2601f,  -36.1844f,-40.4078f,25.2540f,  
	-35.1974f,-41.5109f,23.9083f,  -34.6837f,-42.6870f,24.9442f,  -33.4635f,-43.6366f,23.6748f,  -32.8906f,-44.7939f,24.7537f,  -31.4651f,-45.5614f,23.5786f,  
	-30.8359f,-46.6878f,24.6987f,  -29.2383f,-47.2478f,23.6329f,  -28.5575f,-48.3316f,24.7916f,  -26.8255f,-48.6629f,23.8468f,  -26.0984f,-49.6937f,25.0404f,  
	-24.2737f,-49.7799f,24.2250f,  -23.5079f,-50.7467f,25.4472f,  -21.6344f,-50.5770f,24.7665f,  -20.8385f,-51.4705f,26.0096f,  -18.9621f,-51.0404f,25.4656f,  
	-18.1447f,-51.8528f,26.7212f,  -16.3114f,-51.1639f,26.3130f,  -15.4848f,-51.8886f,27.5697f,  -13.7399f,-50.9496f,27.2943f,  -12.9149f,-51.5828f,28.5408f,  
	-26.8586f,-4.4794f,-13.7156f,  -26.8729f,-5.4736f,-13.0497f,  -24.8769f,-5.4214f,-12.9276f,  -24.8876f,-6.4135f,-12.2644f,  -22.9255f,-6.2102f,-11.9309f,  
	-22.9322f,-7.2003f,-11.2701f,  -21.0379f,-6.8501f,-10.7308f,  -21.0402f,-7.8385f,-10.0717f,  -19.2460f,-7.3478f,-9.3354f,  -19.2435f,-8.3343f,-8.6776f,  
	-17.5801f,-7.7121f,-7.7566f,  -17.5726f,-8.6957f,-7.1000f,  -16.0686f,-7.9532f,-6.0096f,  -16.0557f,-8.9338f,-5.3532f,  -14.7369f,-8.0842f,-4.1117f,  
	-14.7186f,-9.0604f,-3.4553f,  -13.6078f,-8.1188f,-2.0836f,  -13.5841f,-9.0903f,-1.4265f,  -12.7006f,-8.0733f,0.0525f,  -12.6717f,-9.0385f,0.7106f,  
	-12.0310f,-7.9644f,2.2719f,  -11.9974f,-8.9220f,2.9311f,  -11.6107f,-7.8102f,4.5486f,  -11.5730f,-8.7599f,5.2099f,  -11.4474f,-7.6305f,6.8563f,  
	-11.4063f,-8.5718f,7.5199f,  -11.5445f,-7.4456f,9.1676f,  -11.5010f,-8.3773f,9.8336f,  -11.9010f,-7.2756f,11.4548f,  -11.8559f,-8.1973f,12.1231f,  
	-12.5112f,-7.1412f,13.6907f,  -12.4657f,-8.0534f,14.3620f,  -13.3652f,-7.0636f,15.8497f,  -13.3204f,-7.9668f,16.5239f,  -14.4487f,-7.0633f,17.9069f,  
	-14.4054f,-7.9580f,18.5838f,  -15.7430f,-7.1598f,19.8391f,  -15.7022f,-8.0474f,20.5190f,  -17.2256f,-7.3720f,21.6259f,  -17.1876f,-8.2547f,22.3086f,  
	-18.8696f,-7.7179f,23.2492f,  -18.8350f,-8.5979f,23.9350f,  -20.6454f,-8.2131f,24.6942f,  -20.6139f,-9.0933f,25.3833f,  -22.5198f,-8.8718f,25.9493f,  
	-22.4910f,-9.7548f,26.6416f,  -24.4574f,-9.7050f,27.0063f,  -24.4296f,-10.5952f,27.7027f,  -26.4197f,-10.7224f,27.8616f,  -26.3920f,-11.6225f,28.5624f,  
	-28.3684f,-11.9288f,28.5146f,  -28.3376f,-12.8434f,29.2212f,  -30.2624f,-13.3273f,28.9699f,  -30.2257f,-14.2599f,29.6834f,  -32.0616f,-14.9162f,29.2353f,  
	-32.0154f,-15.8702f,29.9574f,  -33.7263f,-16.6901f,29.3228f,  -33.6661f,-17.6686f,30.0556f,  -35.2179f,-18.6394f,29.2482f,  -35.1386f,-19.6453f,29.9946f,  
	-36.4998f,-20.7505f,29.0314f,  -36.3963f,-21.7854f,29.7939f,  -37.5388f,-23.0051f,28.6943f,  -37.4055f,-24.0697f,29.4761f,  -38.3054f,-25.3804f,28.2619f,  
	-38.1359f,-26.4740f,29.0673f,  -38.7739f,-27.8492f,27.7620f,  -38.5626f,-28.9704f,28.5947f,  -38.9250f,-30.3810f,27.2231f,  -38.6665f,-31.5272f,28.0865f,  
	-38.7450f,-32.9418f,26.6741f,  -38.4344f,-34.1070f,27.5719f,  -38.2269f,-35.4928f,26.1441f,  -37.8598f,-36.6723f,27.0801f,  -37.3707f,-37.9956f,25.6617f,  
	-36.9442f,-39.1808f,26.6394f,  -36.1844f,-40.4078f,25.2540f,  -35.6970f,-41.5892f,26.2747f,  -34.6837f,-42.6870f,24.9442f,  -34.1341f,-43.8566f,26.0096f,  
	-32.8906f,-44.7939f,24.7537f,  -32.2805f,-45.9396f,25.8629f,  -30.8359f,-46.6878f,24.6987f,  -30.1689f,-47.7977f,25.8501f,  -28.5575f,-48.3316f,24.7916f,  
	-27.8379f,-49.3949f,25.9823f,  -26.0984f,-49.6937f,25.0404f,  -25.3330f,-50.6987f,26.2649f,  -23.5079f,-50.7467f,25.4472f,  -22.7045f,-51.6832f,26.6988f,  
	-20.8385f,-51.4705f,26.0096f,  -20.0053f,-52.3297f,27.2802f,  -18.1447f,-51.8528f,26.7212f,  -17.2939f,-52.6265f,27.9998f,  -15.4848f,-51.8886f,27.5697f,  
	-14.6271f,-52.5715f,28.8453f,  -12.9149f,-51.5828f,28.5408f,  -12.0603f,-52.1716f,29.8018f,  -26.8729f,-5.4736f,-13.0497f,  -26.8837f,-6.4724f,-12.3822f,  
	-24.8876f,-6.4135f,-12.2644f,  -24.8943f,-7.4101f,-11.5993f,  -22.9322f,-7.2003f,-11.2701f,  -22.9346f,-8.1950f,-10.6068f,  -21.0402f,-7.8385f,-10.0717f,  
	-21.0377f,-8.8311f,-9.4099f,  -19.2435f,-8.3343f,-8.6776f,  -19.2358f,-9.3240f,-8.0169f,  -17.5726f,-8.6957f,-7.1000f,  -17.5594f,-9.6824f,-6.4396f,  
	-16.0557f,-8.9338f,-5.3532f,  -16.0369f,-9.9162f,-4.6929f,  -14.7186f,-9.0604f,-3.4553f,  -14.6942f,-10.0381f,-2.7944f,  -13.5841f,-9.0903f,-1.4265f,  
	-13.5543f,-10.0616f,-0.7647f,  -12.6717f,-9.0385f,0.7106f,  -12.6368f,-10.0022f,1.3734f,  -11.9974f,-8.9220f,2.9311f,  -11.9579f,-9.8776f,3.5960f,  
	-11.5730f,-8.7599f,5.2099f,  -11.5296f,-9.7066f,5.8770f,  -11.4063f,-8.5718f,7.5199f,  -11.3600f,-9.5082f,8.1894f,  -11.5010f,-8.3773f,9.8336f,  
	-11.4527f,-9.3029f,10.5054f,  -11.8559f,-8.1973f,12.1231f,  -11.8067f,-9.1123f,12.7979f,  -12.4657f,-8.0534f,14.3620f,  -12.4167f,-8.9581f,15.0397f,  
	-13.3204f,-7.9668f,16.5239f,  -13.2727f,-8.8612f,17.2042f,  -14.4054f,-7.9580f,18.5838f,  -14.3602f,-8.8434f,19.2670f,  -15.7022f,-8.0474f,20.5190f,  
	-15.6600f,-8.9257f,21.2048f,  -17.1876f,-8.2547f,22.3086f,  -17.1491f,-9.1279f,22.9974f,  -18.8350f,-8.5979f,23.9350f,  -18.8003f,-9.4686f,24.6267f,  
	-20.6139f,-9.0933f,25.3833f,  -20.5830f,-9.9640f,26.0776f,  -22.4910f,-9.7548f,26.6416f,  -22.4626f,-10.6298f,27.3395f,  -24.4296f,-10.5952f,27.7027f,  
	-24.4031f,-11.4773f,28.4042f,  -26.3920f,-11.6225f,28.5624f,  -26.3645f,-12.5164f,29.2687f,  -28.3376f,-12.8434f,29.2212f,  -28.3067f,-13.7528f,29.9333f,  
	-30.2257f,-14.2599f,29.6834f,  -30.1881f,-15.1885f,30.4027f,  -32.0154f,-15.8702f,29.9574f,  -31.9668f,-16.8218f,30.6858f,  -33.6661f,-17.6686f,30.0556f,  
	-33.6016f,-18.6464f,30.7959f,  -35.1386f,-19.6453f,29.9946f,  -35.0532f,-20.6517f,30.7493f,  -36.3963f,-21.7854f,29.7939f,  -36.2842f,-22.8219f,30.5660f,  
	-37.4055f,-24.0697f,29.4761f,  -37.2601f,-25.1366f,30.2699f,  -38.1359f,-26.4740f,29.0673f,  -37.9513f,-27.5707f,29.8866f,  -38.5626f,-28.9704f,28.5947f,  
	-38.3330f,-30.0953f,29.4431f,  -38.6665f,-31.5272f,28.0865f,  -38.3862f,-32.6752f,28.9682f,  -38.4344f,-34.1070f,27.5719f,  -38.0983f,-35.2743f,28.4910f,  
	-37.8598f,-36.6723f,27.0801f,  -37.4640f,-37.8510f,28.0407f,  -36.9442f,-39.1808f,26.6394f,  -36.4861f,-40.3621f,27.6434f,  -35.6970f,-41.5892f,26.2747f,  
	-35.1744f,-42.7655f,27.3247f,  -34.1341f,-43.8566f,26.0096f,  -33.5475f,-45.0161f,27.1055f,  -32.2805f,-45.9396f,25.8629f,  -31.6322f,-47.0704f,27.0041f,  
	-30.1689f,-47.7977f,25.8501f,  -29.4622f,-48.8887f,27.0347f,  -27.8379f,-49.3949f,25.9823f,  -27.0785f,-50.4339f,27.2058f,  -25.3330f,-50.6987f,26.2649f,  
	-24.5281f,-51.6747f,27.5213f,  -22.7045f,-51.6832f,26.6988f,  -21.8616f,-52.5865f,27.9806f,  -20.0053f,-52.3297f,27.2802f,  -19.1361f,-53.1506f,28.5772f,  
	-17.2939f,-52.6265f,27.9998f,  -16.4090f,-53.3584f,29.3012f,  -14.6271f,-52.5715f,28.8453f,  -13.7368f,-53.2097f,30.1394f,  -12.0603f,-52.1716f,29.8018f,  
	-11.1754f,-52.7136f,31.0764f,  -26.8837f,-6.4724f,-12.3822f,  -26.8905f,-7.4758f,-11.7127f,  -24.8943f,-7.4101f,-11.5993f,  -24.8967f,-8.4114f,-10.9317f,  
	-22.9346f,-8.1950f,-10.6068f,  -22.9320f,-9.1941f,-9.9407f,  -21.0377f,-8.8311f,-9.4099f,  -21.0300f,-9.8273f,-8.7449f,  -19.2358f,-9.3240f,-8.0169f,  
	-19.2225f,-10.3172f,-7.3524f,  -17.5594f,-9.6824f,-6.4396f,  -17.5404f,-10.6714f,-5.7752f,  -16.0369f,-9.9162f,-4.6929f,  -16.0120f,-10.9006f,-4.0280f,  
	-14.6942f,-10.0381f,-2.7944f,  -14.6635f,-11.0162f,-2.1286f,  -13.5543f,-10.0616f,-0.7647f,  -13.5181f,-11.0321f,-0.0979f,  -12.6368f,-10.0022f,1.3734f,  
	-12.5956f,-10.9645f,2.0422f,  -11.9579f,-9.8776f,3.5960f,  -11.9123f,-10.8307f,4.2671f,  -11.5296f,-9.7066f,5.8770f,  -11.4805f,-10.6490f,6.5505f,  
	-11.3600f,-9.5082f,8.1894f,  -11.3083f,-10.4391f,8.8652f,  -11.4527f,-9.3029f,10.5054f,  -11.3995f,-10.2223f,11.1842f,  -11.8067f,-9.1123f,12.7979f,  
	-11.7534f,-10.0202f,13.4796f,  -12.4167f,-8.9581f,15.0397f,  -12.3643f,-9.8541f,15.7239f,  -13.2727f,-8.8612f,17.2042f,  -13.2225f,-9.7464f,17.8914f,  
	-14.3602f,-8.8434f,19.2670f,  -14.3130f,-9.7193f,19.9567f,  -15.6600f,-8.9257f,21.2048f,  -15.6168f,-9.7941f,21.8973f,  -17.1491f,-9.1279f,22.9974f,  
	-17.1103f,-9.9911f,23.6925f,  -18.8003f,-9.4686f,24.6267f,  -18.7661f,-10.3289f,25.3241f,  -20.5830f,-9.9640f,26.0776f,  -20.5525f,-10.8257f,26.7781f,  
	-22.4626f,-10.6298f,27.3395f,  -22.4355f,-11.4956f,28.0429f,  -24.4031f,-11.4773f,28.4042f,  -24.3770f,-12.3519f,29.1115f,  -26.3645f,-12.5164f,29.2687f,  
	-26.3374f,-13.4036f,29.9806f,  -28.3067f,-13.7528f,29.9333f,  -28.2756f,-14.6568f,30.6511f,  -30.1881f,-15.1885f,30.4027f,  -30.1490f,-16.1133f,31.1282f,  
	-31.9668f,-16.8218f,30.6858f,  -31.9150f,-17.7711f,31.4215f,  -33.6016f,-18.6464f,30.7959f,  -33.5323f,-19.6234f,31.5441f,  -35.0532f,-20.6517f,30.7493f,  
	-34.9606f,-21.6585f,31.5129f,  -36.2842f,-22.8219f,30.5660f,  -36.1616f,-23.8598f,31.3494f,  -37.2601f,-25.1366f,30.2699f,  -37.1013f,-26.2058f,31.0768f,  
	-37.9513f,-27.5707f,29.8866f,  -37.7502f,-28.6705f,30.7208f,  -38.3330f,-30.0953f,29.4431f,  -38.0833f,-31.2217f,30.3090f,  -38.3862f,-32.6752f,28.9682f,  
	-38.0821f,-33.8253f,29.8702f,  -38.0983f,-35.2743f,28.4910f,  -37.7347f,-36.4411f,29.4339f,  -37.4640f,-37.8510f,28.0407f,  -37.0375f,-39.0265f,29.0270f,  
	-36.4861f,-40.3621f,27.6434f,  -35.9936f,-41.5392f,28.6764f,  -35.1744f,-42.7655f,27.3247f,  -34.6151f,-43.9329f,28.4052f,  -33.5475f,-45.0161f,27.1055f,  
	-32.9224f,-46.1620f,28.2338f,  -31.6322f,-47.0704f,27.0041f,  -30.9436f,-48.1837f,28.1795f,  -29.4622f,-48.8887f,27.0347f,  -28.7146f,-49.9568f,28.2535f,  
	-27.0785f,-50.4339f,27.2058f,  -26.2781f,-51.4452f,28.4631f,  -24.5281f,-51.6747f,27.5213f,  -23.6816f,-52.6187f,28.8104f,  -21.8616f,-52.5865f,27.9806f,  
	-20.9801f,-53.4521f,29.2920f,  -19.1361f,-53.1506f,28.5772f,  -18.2300f,-53.9300f,29.9006f,  -16.4090f,-53.3584f,29.3012f,  -15.4884f,-54.0453f,30.6250f,  
	-13.7368f,-53.2097f,30.1394f,  -12.8128f,-53.8001f,31.4512f,  -11.1754f,-52.7136f,31.0764f,  -10.2619f,-53.2061f,32.3619f,  -26.8905f,-7.4758f,-11.7127f,  
	-26.8929f,-8.4838f,-11.0406f,  -24.8967f,-8.4114f,-10.9317f,  -24.8941f,-9.4170f,-10.2611f,  -22.9320f,-9.1941f,-9.9407f,  -22.9242f,-10.1970f,-9.2713f,  
	-21.0300f,-9.8273f,-8.7449f,  -21.0165f,-10.8273f,-8.0760f,  -19.2225f,-10.3172f,-7.3524f,  -19.2032f,-11.3132f,-6.6837f,  -17.5404f,-10.6714f,-5.7752f,  
	-17.5150f,-11.6629f,-5.1060f,  -16.0120f,-10.9006f,-4.0280f,  -15.9805f,-11.8860f,-3.3579f,  -14.6635f,-11.0162f,-2.1286f,  -14.6262f,-11.9942f,-1.4576f,  
	-13.5181f,-11.0321f,-0.0979f,  -13.4752f,-12.0019f,0.5752f,  -12.5956f,-10.9645f,2.0422f,  -12.5478f,-11.9250f,2.7176f,  -11.9123f,-10.8307f,4.2671f,  
	-11.8603f,-11.7803f,4.9448f,  -11.4805f,-10.6490f,6.5505f,  -11.4254f,-11.5865f,7.2306f,  -11.3083f,-10.4391f,8.8652f,  -11.2511f,-11.3644f,9.5484f,  
	-11.3995f,-10.2223f,11.1842f,  -11.3415f,-11.1349f,11.8703f,  -11.7534f,-10.0202f,13.4796f,  -11.6958f,-10.9195f,14.1683f,  -12.3643f,-9.8541f,15.7239f,  
	-12.3086f,-10.7410f,16.4157f,  -13.2225f,-9.7464f,17.8914f,  -13.1696f,-10.6220f,18.5856f,  -14.3130f,-9.7193f,19.9567f,  -14.2641f,-10.5851f,20.6536f,  
	-15.6168f,-9.7941f,21.8973f,  -15.5726f,-10.6521f,22.5968f,  -17.1103f,-9.9911f,23.6925f,  -17.0713f,-10.8433f,24.3940f,  -18.7661f,-10.3289f,25.3241f,  
	-18.7319f,-11.1794f,26.0282f,  -20.5525f,-10.8257f,26.7781f,  -20.5231f,-11.6770f,27.4845f,  -22.4355f,-11.4956f,28.0429f,  -22.4089f,-12.3526f,28.7523f,  
	-24.3770f,-12.3519f,29.1115f,  -24.3516f,-13.2184f,29.8246f,  -26.3374f,-13.4036f,29.9806f,  -26.3107f,-14.2840f,30.6985f,  -28.2756f,-14.6568f,30.6511f,  
	-28.2438f,-15.5553f,31.3752f,  -30.1490f,-16.1133f,31.1282f,  -30.1077f,-17.0343f,31.8608f,  -31.9150f,-17.7711f,31.4215f,  -31.8598f,-18.7181f,32.1646f,  
	-33.5323f,-19.6234f,31.5441f,  -33.4574f,-20.5993f,32.3006f,  -34.9606f,-21.6585f,31.5129f,  -34.8591f,-22.6655f,32.2871f,  -36.1616f,-23.8598f,31.3494f,  
	-36.0273f,-24.8990f,32.1449f,  -37.1013f,-26.2058f,31.0768f,  -36.9279f,-27.2772f,31.8976f,  -37.7502f,-28.6705f,30.7208f,  -37.5306f,-29.7715f,31.5714f,  
	-38.0833f,-31.2217f,30.3090f,  -37.8114f,-32.3501f,31.1942f,  -38.0821f,-33.8253f,29.8702f,  -37.7518f,-34.9752f,30.7952f,  -37.7347f,-36.4411f,29.4339f,  
	-37.3418f,-37.6054f,30.4018f,  -37.0375f,-39.0265f,29.0270f,  -36.5777f,-40.1988f,30.0418f,  -35.9936f,-41.5392f,28.6764f,  -35.4650f,-42.7087f,29.7397f,  
	-34.6151f,-43.9329f,28.4052f,  -34.0174f,-45.0882f,29.5183f,  -32.9224f,-46.1620f,28.2338f,  -32.2567f,-47.2920f,29.3972f,  -30.9436f,-48.1837f,28.1795f,  
	-30.2132f,-49.2757f,29.3904f,  -28.7146f,-49.9568f,28.2535f,  -27.9246f,-50.9987f,29.5080f,  -26.2781f,-51.4452f,28.4631f,  -25.4343f,-52.4257f,29.7554f,  
	-23.6816f,-52.6187f,28.8104f,  -22.7943f,-53.5257f,30.1320f,  -20.9801f,-53.4521f,29.2920f,  -20.0590f,-54.2765f,30.6333f,  -18.2300f,-53.9300f,29.9006f,  
	-17.2851f,-54.6643f,31.2501f,  -15.4884f,-54.0453f,30.6250f,  -14.5308f,-54.6838f,31.9704f,  -12.8128f,-53.8001f,31.4512f,  -11.8568f,-54.3397f,32.7782f,  
	-10.2619f,-53.2061f,32.3619f,  -9.3178f,-53.6463f,33.6576f,  -26.8929f,-8.4838f,-11.0406f,  -26.8903f,-9.4962f,-10.3656f,  -24.8941f,-9.4170f,-10.2611f,  
	-24.8863f,-10.4268f,-9.5873f,  -22.9242f,-10.1970f,-9.2713f,  -22.9107f,-11.2040f,-8.5980f,  -21.0165f,-10.8273f,-8.0760f,  -20.9969f,-11.8305f,-7.4029f,  
	-19.2032f,-11.3132f,-6.6837f,  -19.1773f,-12.3121f,-6.0101f,  -17.5150f,-11.6629f,-5.1060f,  -17.4827f,-12.6561f,-4.4315f,  -15.9805f,-11.8860f,-3.3579f,  
	-15.9420f,-12.8720f,-2.6824f,  -14.6262f,-11.9942f,-1.4576f,  -14.5817f,-12.9721f,-0.7800f,  -13.4752f,-12.0019f,0.5752f,  -13.4253f,-12.9705f,1.2550f,  
	-12.5478f,-11.9250f,2.7176f,  -12.4930f,-12.8825f,3.3999f,  -11.8603f,-11.7803f,4.9448f,  -11.8018f,-12.7255f,5.6296f,  -11.4254f,-11.5865f,7.2306f,  
	-11.3640f,-12.5190f,7.9185f,  -11.2511f,-11.3644f,9.5484f,  -11.1883f,-12.2833f,10.2394f,  -11.3415f,-11.1349f,11.8703f,  -11.2786f,-12.0394f,12.5639f,  
	-11.6958f,-10.9195f,14.1683f,  -11.6341f,-11.8101f,14.8651f,  -12.3086f,-10.7410f,16.4157f,  -12.2494f,-11.6183f,17.1149f,  -13.1696f,-10.6220f,18.5856f,  
	-13.1143f,-11.4873f,19.2876f,  -14.2641f,-10.5851f,20.6536f,  -14.2135f,-11.4401f,21.3582f,  -15.5726f,-10.6521f,22.5968f,  -15.5277f,-11.4984f,23.3031f,  
	-17.0713f,-10.8433f,24.3940f,  -17.0319f,-11.6849f,25.1026f,  -18.7319f,-11.1794f,26.0282f,  -18.6985f,-12.0185f,26.7387f,  -20.5231f,-11.6770f,27.4845f,  
	-20.4941f,-12.5185f,28.1974f,  -22.4089f,-12.3526f,28.7523f,  -22.3832f,-13.2002f,29.4680f,  -24.3516f,-13.2184f,29.8246f,  -24.3269f,-14.0766f,30.5439f,  
	-26.3107f,-14.2840f,30.6985f,  -26.2838f,-15.1574f,31.4226f,  -28.2438f,-15.5553f,31.3752f,  -28.2107f,-16.4485f,32.1062f,  -30.1077f,-17.0343f,31.8608f,  
	-30.0642f,-17.9513f,32.6004f,  -31.8598f,-18.7181f,32.1646f,  -31.8002f,-19.6625f,32.9157f,  -33.4574f,-20.5993f,32.3006f,  -33.3750f,-21.5741f,33.0671f,  
	-34.8591f,-22.6655f,32.2871f,  -34.7476f,-23.6726f,33.0727f,  -36.0273f,-24.8990f,32.1449f,  -35.8803f,-25.9394f,32.9534f,  -36.9279f,-27.2772f,31.8976f,  
	-36.7378f,-28.3494f,32.7338f,  -37.5306f,-29.7715f,31.5714f,  -37.2909f,-30.8741f,32.4402f,  -37.8114f,-32.3501f,31.1942f,  -37.5150f,-33.4785f,32.1012f,  
	-37.7518f,-34.9752f,30.7952f,  -37.3937f,-36.1231f,31.7443f,  -37.3418f,-37.6054f,30.4018f,  -36.9168f,-38.7672f,31.3974f,  -36.5777f,-40.1988f,30.0418f,  
	-36.0828f,-41.3644f,31.0863f,  -35.4650f,-42.7087f,29.7397f,  -34.8985f,-43.8672f,30.8356f,  -34.0174f,-45.0882f,29.5183f,  -33.3791f,-46.2289f,30.6668f,  
	-32.2567f,-47.2920f,29.3972f,  -31.5487f,-48.4021f,30.5967f,  -30.2132f,-49.2757f,29.3904f,  -29.4393f,-50.3428f,30.6383f,  -27.9246f,-50.9987f,29.5080f,  
	-27.0896f,-52.0110f,30.7995f,  -25.4343f,-52.4257f,29.7554f,  -24.5475f,-53.3702f,31.0828f,  -22.7943f,-53.5257f,30.1320f,  -21.8647f,-54.3920f,31.4867f,  
	-20.0590f,-54.2765f,30.6333f,  -19.0962f,-55.0560f,32.0042f,  -17.2851f,-54.6643f,31.2501f,  -16.3000f,-55.3497f,32.6252f,  -14.5308f,-54.6838f,31.9704f,  
	-13.5378f,-55.2703f,33.3353f,  -11.8568f,-54.3397f,32.7782f,  -10.8667f,-54.8252f,34.1196f,  -9.3178f,-53.6463f,33.6576f,  -8.3434f,-54.0316f,34.9612f,  
	-26.8903f,-9.4962f,-10.3656f,  -26.8824f,-10.5128f,-9.6873f,  -24.8863f,-10.4268f,-9.5873f,  -24.8725f,-11.4408f,-8.9094f,  -22.9107f,-11.2040f,-8.5980f,  
	-22.8908f,-12.2146f,-7.9203f,  -20.9969f,-11.8305f,-7.4029f,  -20.9706f,-12.8371f,-6.7247f,  -19.1773f,-12.3121f,-6.0101f,  -19.1444f,-13.3133f,-5.3310f,  
	-17.4827f,-12.6561f,-4.4315f,  -17.4432f,-13.6504f,-3.7514f,  -15.9420f,-12.8720f,-2.6824f,  -15.8960f,-13.8585f,-2.0002f,  -14.5817f,-12.9721f,-0.7800f,  
	-14.5297f,-13.9495f,-0.0955f,  -13.4253f,-12.9705f,1.2550f,  -13.3678f,-13.9370f,1.9422f,  -12.4930f,-12.8825f,3.3999f,  -12.4311f,-13.8365f,4.0895f,  
	-11.8018f,-12.7255f,5.6296f,  -11.7363f,-13.6663f,6.3225f,  -11.3640f,-12.5190f,7.9185f,  -11.2963f,-13.4455f,8.6147f,  -11.1883f,-12.2833f,10.2394f,  
	-11.1198f,-13.1945f,10.9383f,  -11.2786f,-12.0394f,12.5639f,  -11.2106f,-12.9355f,13.2663f,  -11.6341f,-11.8101f,14.8651f,  -11.5681f,-12.6912f,15.5700f,  
	-12.2494f,-11.6183f,17.1149f,  -12.1869f,-12.4854f,17.8227f,  -13.1143f,-11.4873f,19.2876f,  -13.0564f,-12.3417f,19.9979f,  -14.2135f,-11.4401f,21.3582f,  
	-14.1615f,-12.2830f,22.0701f,  -15.5277f,-11.4984f,23.3031f,  -15.4817f,-12.3335f,24.0171f,  -17.0319f,-11.6849f,25.1026f,  -16.9929f,-12.5143f,25.8182f,  
	-18.6985f,-12.0185f,26.7387f,  -18.6652f,-12.8468f,27.4561f,  -20.4941f,-12.5185f,28.1974f,  -20.4659f,-13.3493f,28.9167f,  -22.3832f,-13.2002f,29.4680f,  
	-22.3583f,-14.0381f,30.1900f,  -24.3269f,-14.0766f,30.5439f,  -24.3026f,-14.9264f,31.2695f,  -26.2838f,-15.1574f,31.4226f,  -26.2563f,-16.0239f,32.1534f,  
	-28.2107f,-16.4485f,32.1062f,  -28.1762f,-17.3361f,32.8439f,  -30.0642f,-17.9513f,32.6004f,  -30.0175f,-18.8641f,33.3476f,  -31.8002f,-19.6625f,32.9157f,  
	-31.7346f,-20.6044f,33.6761f,  -33.3750f,-21.5741f,33.0671f,  -33.2842f,-22.5478f,33.8442f,  -34.7476f,-23.6726f,33.0727f,  -34.6253f,-24.6798f,33.8701f,  
	-35.8803f,-25.9394f,32.9534f,  -35.7184f,-26.9797f,33.7763f,  -36.7378f,-28.3494f,32.7338f,  -36.5295f,-29.4225f,33.5870f,  -37.2909f,-30.8741f,32.4402f,  
	-37.0284f,-31.9765f,33.3298f,  -37.5150f,-33.4785f,32.1012f,  -37.1923f,-34.6050f,33.0314f,  -37.3937f,-36.1231f,31.7443f,  -37.0050f,-37.2688f,32.7200f,  
	-36.9168f,-38.7672f,31.3974f,  -36.4578f,-39.9230f,32.4220f,  -36.0828f,-41.3644f,31.0863f,  -35.5506f,-42.5201f,32.1630f,  -34.8985f,-43.8672f,30.8356f,  
	-34.2916f,-45.0125f,31.9668f,  -33.3791f,-46.2289f,30.6668f,  -32.6982f,-47.3510f,31.8521f,  -31.5487f,-48.4021f,30.5967f,  -30.7964f,-49.4886f,31.8344f,  
	-29.4393f,-50.3428f,30.6383f,  -28.6193f,-51.3816f,31.9247f,  -27.0896f,-52.0110f,30.7995f,  -26.2099f,-52.9882f,32.1282f,  -24.5475f,-53.3702f,31.0828f,  
	-23.6162f,-54.2746f,32.4462f,  -21.8647f,-54.3920f,31.4867f,  -20.8907f,-55.2137f,32.8743f,  -19.0962f,-55.0560f,32.0042f,  -18.0901f,-55.7862f,33.4043f,  
	-16.3000f,-55.3497f,32.6252f,  -15.2761f,-55.9819f,34.0239f,  -13.5378f,-55.2703f,33.3353f,  -12.5070f,-55.8010f,34.7190f,  -10.8667f,-54.8252f,34.1196f,  
	-9.8425f,-55.2533f,35.4733f,  -8.3434f,-54.0316f,34.9612f,  -7.3377f,-54.3591f,36.2711f,  -26.8824f,-10.5128f,-9.6873f,  -26.8686f,-11.5338f,-9.0050f,  
	-24.8725f,-11.4408f,-8.9094f,  -24.8525f,-12.4587f,-8.2273f,  -22.8908f,-12.2146f,-7.9203f,  -22.8641f,-13.2287f,-7.2377f,  -20.9706f,-12.8371f,-6.7247f,  
	-20.9371f,-13.8463f,-6.0411f,  -19.1444f,-13.3133f,-5.3310f,  -19.1039f,-14.3162f,-4.6462f,  -17.4432f,-13.6504f,-3.7514f,  -17.3958f,-14.6460f,-3.0645f,  
	-15.8960f,-13.8585f,-2.0002f,  -15.8422f,-14.8452f,-1.3109f,  -14.5297f,-13.9495f,-0.0955f,  -14.4697f,-14.9256f,0.5965f,  -13.3678f,-13.9370f,1.9422f,  
	-13.3027f,-14.9005f,2.6368f,  -12.4311f,-13.8365f,4.0895f,  -12.3616f,-14.7867f,4.7877f,  -11.7363f,-13.6663f,6.3225f,  -11.6638f,-14.6019f,7.0241f,  
	-11.2963f,-13.4455f,8.6147f,  -11.2221f,-14.3650f,9.3192f,  -11.1198f,-13.1945f,10.9383f,  -11.0453f,-14.0978f,11.6464f,  -11.2106f,-12.9355f,13.2663f,  
	-11.1374f,-13.8223f,13.9771f,  -11.5681f,-12.6912f,15.5700f,  -11.4979f,-13.5622f,16.2839f,  -12.1869f,-12.4854f,17.8227f,  -12.1210f,-13.3415f,18.5392f,  
	-13.0564f,-12.3417f,19.9979f,  -12.9963f,-13.1836f,20.7162f,  -14.1615f,-12.2830f,22.0701f,  -14.1078f,-13.1143f,22.7903f,  -15.4817f,-12.3335f,24.0171f,  
	-15.4355f,-13.1558f,24.7388f,  -16.9929f,-12.5143f,25.8182f,  -16.9536f,-13.3319f,26.5412f,  -18.6652f,-12.8468f,27.4561f,  -18.6326f,-13.6633f,28.1804f,  
	-20.4659f,-13.3493f,28.9167f,  -20.4386f,-14.1691f,29.6428f,  -22.3583f,-14.0381f,30.1900f,  -22.3340f,-14.8661f,30.9186f,  -24.3026f,-14.9264f,31.2695f,  
	-24.2781f,-15.7676f,32.0018f,  -26.2563f,-16.0239f,32.1534f,  -26.2282f,-16.8830f,32.8910f,  -28.1762f,-17.3361f,32.8439f,  -28.1397f,-18.2178f,33.5889f,  
	-30.0175f,-18.8641f,33.3476f,  -29.9660f,-19.7729f,34.1035f,  -31.7346f,-20.6044f,33.6761f,  -31.6621f,-21.5436f,34.4462f,  -33.2842f,-22.5478f,33.8442f,  
	-33.1844f,-23.5200f,34.6322f,  -34.6253f,-24.6798f,33.8701f,  -34.4898f,-25.6858f,34.6811f,  -35.7184f,-26.9797f,33.7763f,  -35.5402f,-28.0201f,34.6149f,  
	-36.5295f,-29.4225f,33.5870f,  -36.3002f,-30.4949f,34.4597f,  -37.0284f,-31.9765f,33.3298f,  -36.7412f,-33.0769f,34.2412f,  -37.1923f,-34.6050f,33.0314f,  
	-36.8408f,-35.7294f,33.9870f,  -37.0050f,-37.2688f,32.7200f,  -36.5836f,-38.4092f,33.7237f,  -36.4578f,-39.9230f,32.4220f,  -35.9625f,-41.0698f,33.4780f,  
	-35.5506f,-42.5201f,32.1630f,  -34.9785f,-43.6636f,33.2748f,  -34.2916f,-45.0125f,31.9668f,  -33.6422f,-46.1404f,33.1348f,  -32.6982f,-47.3510f,31.8521f,  
	-31.9727f,-48.4509f,33.0761f,  -30.7964f,-49.4886f,31.8344f,  -29.9971f,-50.5483f,33.1118f,  -28.6193f,-51.3816f,31.9247f,  -27.7529f,-52.3864f,33.2502f,  
	-26.2099f,-52.9882f,32.1282f,  -25.2834f,-53.9262f,33.4954f,  -23.6162f,-54.2746f,32.4462f,  -22.6379f,-55.1347f,33.8455f,  -20.8907f,-55.2137f,32.8743f,  
	-19.8705f,-55.9860f,34.2945f,  -18.0901f,-55.7862f,33.4043f,  -17.0418f,-56.4625f,34.8321f,  -15.2761f,-55.9819f,34.0239f,  -14.2107f,-56.5570f,35.4456f,  
	-12.5070f,-55.8010f,34.7190f,  -11.4384f,-56.2721f,36.1192f,  -9.8425f,-55.2533f,35.4733f,  -8.7832f,-55.6208f,36.8375f,  -7.3377f,-54.3591f,36.2711f,  
	-6.3003f,-54.6260f,37.5849f,  -26.8686f,-11.5338f,-9.0050f,  -26.8484f,-12.5588f,-8.3184f,  -24.8525f,-12.4587f,-8.2273f,  -24.8256f,-13.4805f,-7.5402f,  
	-22.8641f,-13.2287f,-7.2377f,  -22.8301f,-14.2461f,-6.5496f,  -20.9371f,-13.8463f,-6.0411f,  -20.8958f,-14.8579f,-5.3517f,  -19.1039f,-14.3162f,-4.6462f,  
	-19.0554f,-15.3210f,-3.9547f,  -17.3958f,-14.6460f,-3.0645f,  -17.3402f,-15.6423f,-2.3704f,  -15.8422f,-14.8452f,-1.3109f,  -15.7798f,-15.8313f,-0.6139f,  
	-14.4697f,-14.9256f,0.5965f,  -14.4015f,-15.8994f,1.2963f,  -13.3027f,-14.9005f,2.6368f,  -13.2293f,-15.8611f,3.3403f,  -12.3616f,-14.7867f,4.7877f,  
	-12.2843f,-15.7325f,5.4947f,  -11.6638f,-14.6019f,7.0241f,  -11.5838f,-15.5310f,7.7344f,  -11.2221f,-14.3650f,9.3192f,  -11.1410f,-15.2772f,10.0334f,  
	-11.0453f,-14.0978f,11.6464f,  -10.9647f,-14.9922f,12.3635f,  -11.1374f,-13.8223f,13.9771f,  -11.0591f,-14.6993f,14.6975f,  -11.4979f,-13.5622f,16.2839f,  
	-11.4234f,-14.4223f,17.0072f,  -12.1210f,-13.3415f,18.5392f,  -12.0520f,-14.1850f,19.2644f,  -12.9963f,-13.1836f,20.7162f,  -12.9337f,-14.0137f,21.4433f,  
	-14.1078f,-13.1143f,22.7903f,  -14.0531f,-13.9321f,23.5189f,  -15.4355f,-13.1558f,24.7388f,  -15.3885f,-13.9655f,25.4683f,  -16.9536f,-13.3319f,26.5412f,  
	-16.9144f,-14.1367f,27.2715f,  -18.6326f,-13.6633f,28.1804f,  -18.6004f,-14.4675f,28.9118f,  -20.4386f,-14.1691f,29.6428f,  -20.4118f,-14.9777f,30.3758f,  
	-22.3340f,-14.8661f,30.9186f,  -22.3098f,-15.6841f,31.6539f,  -24.2781f,-15.7676f,32.0018f,  -24.2536f,-16.5998f,32.7408f,  -26.2282f,-16.8830f,32.8910f,  
	-26.1989f,-17.7345f,33.6355f,  -28.1397f,-18.2178f,33.5889f,  -28.0994f,-19.0938f,34.3422f,  -29.9660f,-19.7729f,34.1035f,  -29.9090f,-20.6774f,34.8685f,  
	-31.6621f,-21.5436f,34.4462f,  -31.5821f,-22.4799f,35.2264f,  -33.1844f,-23.5200f,34.6322f,  -33.0731f,-24.4899f,35.4326f,  -34.4898f,-25.6858f,34.6811f,  
	-34.3400f,-26.6909f,35.5064f,  -35.5402f,-28.0201f,34.6149f,  -35.3429f,-29.0592f,35.4718f,  -36.3002f,-30.4949f,34.4597f,  -36.0479f,-31.5651f,35.3530f,  
	-36.7412f,-33.0769f,34.2412f,  -36.4270f,-34.1752f,35.1767f,  -36.8408f,-35.7294f,33.9870f,  -36.4581f,-36.8488f,34.9693f,  -36.5836f,-38.4092f,33.7237f,  
	-36.1272f,-39.5412f,34.7580f,  -35.9625f,-41.0698f,33.4780f,  -35.4281f,-42.2054f,34.5684f,  -34.9785f,-43.6636f,33.2748f,  -34.3644f,-44.7910f,34.4231f,  
	-33.6422f,-46.1404f,33.1348f,  -32.9480f,-47.2474f,34.3419f,  -31.9727f,-48.4509f,33.0761f,  -31.1995f,-49.5252f,34.3405f,  -29.9971f,-50.5483f,33.1118f,  
	-29.1502f,-51.5749f,34.4297f,  -27.7529f,-52.3864f,33.2502f,  -26.8380f,-53.3528f,34.6164f,  -25.2834f,-53.9262f,33.4954f,  -24.3079f,-54.8204f,34.9012f,  
	-22.6379f,-55.1347f,33.8455f,  -21.6107f,-55.9456f,35.2804f,  -19.8705f,-55.9860f,34.2945f,  -18.8049f,-56.7037f,35.7461f,  -17.0418f,-56.4625f,34.8321f,  
	-15.9486f,-57.0804f,36.2868f,  -14.2107f,-56.5570f,35.4456f,  -13.1039f,-57.0705f,36.8880f,  -11.4384f,-56.2721f,36.1192f,  -10.3309f,-56.6797f,37.5342f,  
	-8.7832f,-55.6208f,36.8375f,  -7.6885f,-55.9240f,38.2096f,  -6.3003f,-54.6260f,37.5849f,  -5.2299f,-54.8291f,38.9008f,  -26.8484f,-12.5588f,-8.3184f,  
	-26.8213f,-13.5879f,-7.6271f,  -24.8256f,-13.4805f,-7.5402f,  -24.7912f,-14.5059f,-6.8477f,  -22.8301f,-14.2461f,-6.5496f,  -22.7883f,-15.2664f,-5.8556f,  
	-20.8958f,-14.8579f,-5.3517f,  -20.8463f,-15.8720f,-4.6555f,  -19.0554f,-15.3210f,-3.9547f,  -18.9983f,-16.3271f,-3.2559f,  -17.3402f,-15.6423f,-2.3704f,  
	-17.2758f,-16.6388f,-1.6686f,  -15.7798f,-15.8313f,-0.6139f,  -15.7087f,-16.8160f,0.0909f,  -14.4015f,-15.8994f,1.2963f,  -14.3243f,-16.8711f,2.0051f,  
	-13.2293f,-15.8611f,3.3403f,  -13.1473f,-16.8180f,4.0529f,  -12.2843f,-15.7325f,5.4947f,  -12.1987f,-16.6725f,6.2109f,  -11.5838f,-15.5310f,7.7344f,  
	-11.4961f,-16.4535f,8.4547f,  -11.1410f,-15.2772f,10.0334f,  -11.0529f,-16.1810f,10.7569f,  -10.9647f,-14.9922f,12.3635f,  -10.8779f,-15.8772f,13.0908f,  
	-11.0591f,-14.6993f,14.6975f,  -10.9754f,-15.5657f,15.4279f,  -11.4234f,-14.4223f,17.0072f,  -11.3446f,-15.2699f,17.7397f,  -12.0520f,-14.1850f,19.2644f,  
	-11.9795f,-15.0164f,19.9990f,  -12.9337f,-14.0137f,21.4433f,  -12.8691f,-14.8299f,22.1793f,  -14.0531f,-13.9321f,23.5189f,  -13.9967f,-14.7367f,24.2557f,  
	-15.3885f,-13.9655f,25.4683f,  -15.3410f,-14.7614f,26.2057f,  -16.9144f,-14.1367f,27.2715f,  -16.8754f,-14.9282f,28.0094f,  -18.6004f,-14.4675f,28.9118f,  
	-18.5687f,-15.2593f,29.6505f,  -20.4118f,-14.9777f,30.3758f,  -20.3852f,-15.7747f,31.1157f,  -22.3098f,-15.6841f,31.6539f,  -22.2860f,-16.4913f,32.3959f,  
	-24.2536f,-16.5998f,32.7408f,  -24.2285f,-17.4227f,33.4866f,  -26.1989f,-17.7345f,33.6355f,  -26.1669f,-18.5788f,34.3879f,  -28.0994f,-19.0938f,34.3422f,  
	-28.0549f,-19.9639f,35.1038f,  -29.9090f,-20.6774f,34.8685f,  -29.8461f,-21.5772f,35.6427f,  -31.5821f,-22.4799f,35.2264f,  -31.4924f,-23.4125f,36.0180f,  
	-33.0731f,-24.4899f,35.4326f,  -32.9493f,-25.4576f,36.2462f,  -34.3400f,-26.6909f,35.5064f,  -34.1729f,-27.6937f,36.3486f,  -35.3429f,-29.0592f,35.4718f,  
	-35.1245f,-30.0955f,36.3477f,  -36.0479f,-31.5651f,35.3530f,  -35.7707f,-32.6329f,36.2688f,  -36.4270f,-34.1752f,35.1767f,  -36.0835f,-35.2686f,36.1375f,  
	-36.4581f,-36.8488f,34.9693f,  -36.0418f,-37.9604f,35.9810f,  -36.1272f,-39.5412f,34.7580f,  -35.6328f,-40.6628f,35.8256f,  -35.4281f,-42.2054f,34.5684f,  
	-34.8524f,-43.3259f,35.6949f,  -34.3644f,-44.7910f,34.4231f,  -33.7056f,-45.8986f,35.6104f,  -32.9480f,-47.2474f,34.3419f,  -32.2060f,-48.3300f,35.5897f,  
	-31.1995f,-49.5252f,34.3405f,  -30.3779f,-50.5677f,35.6465f,  -29.1502f,-51.5749f,34.4297f,  -28.2533f,-52.5643f,35.7900f,  -26.8380f,-53.3528f,34.6164f,  
	-25.8721f,-54.2762f,36.0233f,  -24.3079f,-54.8204f,34.9012f,  -23.2810f,-55.6657f,36.3453f,  -21.6107f,-55.9456f,35.2804f,  -20.5352f,-56.7015f,36.7502f,  
	-18.8049f,-56.7037f,35.7461f,  -17.6911f,-57.3621f,37.2284f,  -15.9486f,-57.0804f,36.2868f,  -14.8104f,-57.6348f,37.7663f,  -13.1039f,-57.0705f,36.8880f,  
	-11.9542f,-57.5179f,38.3493f,  -10.3309f,-56.6797f,37.5342f,  -9.1841f,-57.0197f,38.9611f,  -7.6885f,-55.9240f,38.2096f,  -6.5569f,-56.1593f,39.5877f,  
	-5.2299f,-54.8291f,38.9008f,  -4.1244f,-54.9648f,40.2171f,  -26.8213f,-13.5879f,-7.6271f,  -26.7866f,-14.6210f,-6.9304f,  -24.7912f,-14.5059f,-6.8477f,  
	-24.7489f,-15.5345f,-6.1495f,  -22.7883f,-15.2664f,-5.8556f,  -22.7379f,-16.2896f,-5.1550f,  -20.8463f,-15.8720f,-4.6555f,  -20.7879f,-16.8880f,-3.9522f,  
	-18.9983f,-16.3271f,-3.2559f,  -18.9319f,-17.3342f,-2.5493f,  -17.2758f,-16.6388f,-1.6686f,  -17.2021f,-17.6347f,-0.9588f,  -15.7087f,-16.8160f,0.0909f,  
	-15.6282f,-17.7994f,0.8048f,  -14.3243f,-16.8711f,2.0051f,  -14.2380f,-17.8398f,2.7231f,  -13.1473f,-16.8180f,4.0529f,  -13.0563f,-17.7699f,4.7748f,  
	-12.1987f,-16.6725f,6.2109f,  -12.1045f,-17.6066f,6.9374f,  -11.4961f,-16.4535f,8.4547f,  -11.4005f,-17.3682f,9.1847f,  -11.0529f,-16.1810f,10.7569f,  
	-10.9575f,-17.0759f,11.4911f,  -10.8779f,-15.8772f,13.0908f,  -10.7848f,-16.7521f,13.8284f,  -10.9754f,-15.5657f,15.4279f,  -10.8864f,-16.4197f,16.1680f,  
	-11.3446f,-15.2699f,17.7397f,  -11.2614f,-16.1053f,18.4821f,  -11.9795f,-15.0164f,19.9990f,  -11.9039f,-15.8339f,20.7431f,  -12.8691f,-14.8299f,22.1793f,  
	-12.8020f,-15.6324f,22.9243f,  -13.9967f,-14.7367f,24.2557f,  -13.9391f,-15.5268f,25.0010f,  -15.3410f,-14.7614f,26.2057f,  -15.2931f,-15.5431f,26.9512f,  
	-16.8754f,-14.9282f,28.0094f,  -16.8365f,-15.7060f,28.7550f,  -18.5687f,-15.2593f,29.6505f,  -18.5372f,-16.0380f,30.3963f,  -20.3852f,-15.7747f,31.1157f,  
	-20.3591f,-16.5594f,31.8623f,  -22.2860f,-16.4913f,32.3959f,  -22.2620f,-17.2876f,33.1448f,  -24.2285f,-17.4227f,33.4866f,  -24.2014f,-18.2366f,34.2400f,  
	-26.1669f,-18.5788f,34.3879f,  -26.1316f,-19.4153f,35.1482f,  -28.0549f,-19.9639f,35.1038f,  -28.0058f,-20.8274f,35.8740f,  -29.8461f,-21.5772f,35.6427f,  
	-29.7749f,-22.4719f,36.4274f,  -31.4924f,-23.4125f,36.0180f,  -31.3921f,-24.3414f,36.8216f,  -32.9493f,-25.4576f,36.2462f,  -32.8102f,-26.4220f,37.0752f,  
	-34.1729f,-27.6937f,36.3486f,  -33.9866f,-28.6931f,37.2084f,  -35.1245f,-30.0955f,36.3477f,  -34.8832f,-31.1288f,37.2445f,  -35.7707f,-32.6329f,36.2688f,  
	-35.4661f,-33.6956f,37.2083f,  -36.0835f,-35.2686f,36.1375f,  -35.7079f,-36.3545f,37.1263f,  -36.0418f,-37.9604f,35.9810f,  -35.5889f,-39.0621f,37.0248f,  
	-35.6328f,-40.6628f,35.8256f,  -35.0980f,-41.7701f,36.9284f,  -34.8524f,-43.3259f,35.6949f,  -34.2326f,-44.4277f,36.8597f,  -33.7056f,-45.8986f,35.6104f,  
	-32.9992f,-46.9831f,36.8382f,  -32.2060f,-48.3300f,35.5897f,  -31.4149f,-49.3820f,36.8797f,  -30.3779f,-50.5677f,35.6465f,  -29.5050f,-51.5740f,36.9960f,  
	-28.2533f,-52.5643f,35.7900f,  -27.3037f,-53.5113f,37.1927f,  -25.8721f,-54.2762f,36.0233f,  -24.8528f,-55.1511f,37.4709f,  -23.2810f,-55.6657f,36.3453f,  
	-22.2030f,-56.4558f,37.8274f,  -20.5352f,-56.7015f,36.7502f,  -19.4083f,-57.3973f,38.2542f,  -17.6911f,-57.3621f,37.2284f,  -16.5287f,-57.9555f,38.7392f,  
	-14.8104f,-57.6348f,37.7663f,  -13.6255f,-58.1209f,39.2687f,  -11.9542f,-57.5179f,38.3493f,  -10.7615f,-57.8945f,39.8264f,  -9.1841f,-57.0197f,38.9611f,  
	-7.9964f,-57.2879f,40.3977f,  -6.5569f,-56.1593f,39.5877f,  -5.3863f,-56.3226f,40.9694f,  -4.1244f,-54.9648f,40.2171f,  -2.9854f,-55.0307f,41.5299f,  
	-26.7866f,-14.6210f,-6.9304f,  -26.7440f,-15.6577f,-6.2282f,  -24.7489f,-15.5345f,-6.1495f,  -24.6978f,-16.5666f,-5.4447f,  -22.7379f,-16.2896f,-5.1550f,  
	-22.6785f,-17.3152f,-4.4474f,  -20.7879f,-16.8880f,-3.9522f,  -20.7199f,-17.9056f,-3.2411f,  -18.9319f,-17.3342f,-2.5493f,  -18.8560f,-18.3414f,-1.8348f,  
	-17.2021f,-17.6347f,-0.9588f,  -17.1184f,-18.6300f,-0.2398f,  -15.6282f,-17.7994f,0.8048f,  -15.5377f,-18.7805f,1.5282f,  -14.2380f,-17.8398f,2.7231f,  
	-14.1419f,-18.8045f,3.4507f,  -13.0563f,-17.7699f,4.7748f,  -12.9558f,-18.7168f,5.5074f,  -12.1045f,-17.6066f,6.9374f,  -12.0015f,-18.5337f,7.6739f,  
	-11.4005f,-17.3682f,9.1847f,  -11.2965f,-18.2748f,9.9259f,  -10.9575f,-17.0759f,11.4911f,  -10.8547f,-17.9611f,12.2361f,  -10.7848f,-16.7521f,13.8284f,  
	-10.6852f,-17.6148f,14.5764f,  -10.8864f,-16.4197f,16.1680f,  -10.7919f,-17.2618f,16.9187f,  -11.2614f,-16.1053f,18.4821f,  -11.1740f,-16.9268f,19.2348f,  
	-11.9039f,-15.8339f,20.7431f,  -11.8250f,-16.6372f,21.4968f,  -12.8020f,-15.6324f,22.9243f,  -12.7329f,-16.4198f,23.6783f,  -13.9391f,-15.5268f,25.0010f,  
	-13.8804f,-16.3019f,25.7550f,  -15.2931f,-15.5431f,26.9512f,  -15.2446f,-16.3101f,27.7048f,  -16.8365f,-15.7060f,28.7550f,  -16.7973f,-16.4695f,29.5081f,  
	-18.5372f,-16.0380f,30.3963f,  -18.5060f,-16.8029f,31.1490f,  -20.3591f,-16.5594f,31.8623f,  -20.3332f,-17.3315f,32.6158f,  -22.2620f,-17.2876f,33.1448f,  
	-22.2366f,-18.0732f,33.9009f,  -24.2014f,-18.2366f,34.2400f,  -24.1719f,-19.0410f,35.0007f,  -26.1316f,-19.4153f,35.1482f,  -26.0929f,-20.2434f,35.9162f,  
	-28.0058f,-20.8274f,35.8740f,  -27.9499f,-21.6844f,36.6537f,  -29.7749f,-22.4719f,36.4274f,  -29.6949f,-23.3613f,37.2228f,  -31.3921f,-24.3414f,36.8216f,  
	-31.2781f,-25.2658f,37.6391f,  -32.8102f,-26.4220f,37.0752f,  -32.6538f,-27.3820f,37.9202f,  -33.9866f,-28.6931f,37.2084f,  -33.7795f,-29.6887f,38.0872f,  
	-34.8832f,-31.1288f,37.2445f,  -34.6165f,-32.1567f,38.1634f,  -35.4661f,-33.6956f,37.2083f,  -35.1311f,-34.7509f,38.1741f,  -35.7079f,-36.3545f,37.1263f,  
	-35.2973f,-37.4309f,38.1455f,  -35.5889f,-39.0621f,37.0248f,  -35.0969f,-40.1504f,38.1024f,  -35.0980f,-41.7701f,36.9284f,  -34.5199f,-42.8598f,38.0685f,  
	-34.2326f,-44.4277f,36.8597f,  -33.5657f,-45.5075f,38.0645f,  -32.9992f,-46.9831f,36.8382f,  -32.2435f,-48.0382f,38.1083f,  -31.4149f,-49.3820f,36.8797f,  
	-30.5719f,-50.3989f,38.2140f,  -29.5050f,-51.5740f,36.9960f,  -28.5784f,-52.5389f,38.3893f,  -27.3037f,-53.5113f,37.1927f,  -26.2991f,-54.4107f,38.6379f,  
	-24.8528f,-55.1511f,37.4709f,  -23.7798f,-55.9709f,38.9590f,  -22.2030f,-56.4558f,37.8274f,  -21.0708f,-57.1854f,39.3466f,  -19.4083f,-57.3973f,38.2542f,  
	-18.2295f,-58.0269f,39.7901f,  -16.5287f,-57.9555f,38.7392f,  -15.3160f,-58.4785f,40.2768f,  -13.6255f,-58.1209f,39.2687f,  -12.3936f,-58.5333f,40.7908f,  
	-10.7615f,-57.8945f,39.8264f,  -9.5238f,-58.1956f,41.3171f,  -7.9964f,-57.2879f,40.3977f,  -6.7659f,-57.4797f,41.8414f,  -5.3863f,-56.3226f,40.9694f,  
	-4.1786f,-56.4109f,42.3505f,  -2.9854f,-55.0307f,41.5299f,  -1.8092f,-55.0225f,42.8379f,  -26.7440f,-15.6577f,-6.2282f,  -26.6926f,-16.6983f,-5.5195f,  
	-24.6978f,-16.5666f,-5.4447f,  -24.6377f,-17.6016f,-4.7330f,  -22.6785f,-17.3152f,-4.4474f,  -22.6093f,-18.3433f,-3.7321f,  -20.7199f,-17.9056f,-3.2411f,  
	-20.6421f,-18.9241f,-2.5220f,  -18.8560f,-18.3414f,-1.8348f,  -18.7696f,-19.3489f,-1.1110f,  -17.1184f,-18.6300f,-0.2398f,  -17.0243f,-19.6240f,0.4886f,  
	-15.5377f,-18.7805f,1.5282f,  -15.4370f,-19.7586f,2.2613f,  -14.1419f,-18.8045f,3.4507f,  -14.0355f,-19.7650f,4.1892f,  -12.9558f,-18.7168f,5.5074f,  
	-12.8457f,-19.6574f,6.2503f,  -12.0015f,-18.5337f,7.6739f,  -11.8891f,-19.4533f,8.4219f,  -11.2965f,-18.2748f,9.9259f,  -11.1841f,-19.1722f,10.6783f,  
	-10.8547f,-17.9611f,12.2361f,  -10.7443f,-18.8348f,12.9920f,  -10.6852f,-17.6148f,14.5764f,  -10.5789f,-18.4659f,15.3354f,  -10.7919f,-17.2618f,16.9187f,  
	-10.6919f,-18.0899f,17.6802f,  -11.1740f,-16.9268f,19.2348f,  -11.0821f,-17.7339f,19.9976f,  -11.8250f,-16.6372f,21.4968f,  -11.7428f,-17.4251f,22.2601f,  
	-12.7329f,-16.4198f,23.6783f,  -12.6616f,-17.1916f,24.4416f,  -13.8804f,-16.3019f,25.7550f,  -13.8204f,-17.0614f,26.5177f,  -15.2446f,-16.3101f,27.7048f,  
	-15.1954f,-17.0616f,28.4662f,  -16.7973f,-16.4695f,29.5081f,  -16.7583f,-17.2177f,30.2683f,  -18.5060f,-16.8029f,31.1490f,  -18.4750f,-17.5537f,31.9088f,  
	-20.3332f,-17.3315f,32.6158f,  -20.3061f,-18.0912f,33.3765f,  -22.2366f,-18.0732f,33.9009f,  -22.2094f,-18.8474f,34.6641f,  -24.1719f,-19.0410f,35.0007f,  
	-24.1401f,-19.8351f,35.7687f,  -26.0929f,-20.2434f,35.9162f,  -26.0487f,-21.0632f,36.6930f,  -27.9499f,-21.6844f,36.6537f,  -27.8867f,-22.5341f,37.4430f,  
	-29.6949f,-23.3613f,37.2228f,  -29.6029f,-24.2448f,38.0307f,  -31.2781f,-25.2658f,37.6391f,  -31.1489f,-26.1846f,38.4711f,  -32.6538f,-27.3820f,37.9202f,  
	-32.4787f,-28.3372f,38.7825f,  -33.7795f,-29.6887f,38.0872f,  -33.5492f,-30.6783f,38.9863f,  -34.6165f,-32.1567f,38.1634f,  -34.3214f,-33.1771f,39.1066f,  
	-35.1311f,-34.7509f,38.1741f,  -34.7630f,-35.7968f,39.1686f,  -35.2973f,-37.4309f,38.1455f,  -34.8491f,-38.4943f,39.1970f,  -35.0969f,-40.1504f,38.1024f,  
	-34.5627f,-41.2217f,39.2161f,  -34.5199f,-42.8598f,38.0685f,  -33.8956f,-43.9284f,39.2476f,  -33.5657f,-45.5075f,38.0645f,  -32.8495f,-46.5590f,39.3112f,  
	-32.2435f,-48.0382f,38.1083f,  -31.4355f,-49.0593f,39.4229f,  -30.5719f,-50.3989f,38.2140f,  -29.6741f,-51.3756f,39.5927f,  -28.5784f,-52.5389f,38.3893f,  
	-27.5953f,-53.4570f,39.8263f,  -26.2991f,-54.4107f,38.6379f,  -25.2387f,-55.2552f,40.1258f,  -23.7798f,-55.9709f,38.9590f,  -22.6501f,-56.7301f,40.4869f,  
	-21.0708f,-57.1854f,39.3466f,  -19.8836f,-57.8480f,40.9010f,  -18.2295f,-58.0269f,39.7901f,  -16.9969f,-58.5846f,41.3564f,  -15.3160f,-58.4785f,40.2768f,  
	-14.0527f,-58.9253f,41.8377f,  -12.3936f,-58.5333f,40.7908f,  -11.1129f,-58.8669f,42.3300f,  -9.5238f,-58.1956f,41.3171f,  -8.2393f,-58.4161f,42.8181f,  
	-6.7659f,-57.4797f,41.8414f,  -5.4944f,-57.5912f,43.2873f,  -4.1786f,-56.4109f,42.3505f,  -2.9302f,-56.4194f,43.7291f,  -1.8092f,-55.0225f,42.8379f,  
	-0.5959f,-54.9371f,44.1376f,  -26.6926f,-16.6983f,-5.5195f,  -26.6320f,-17.7422f,-4.8042f,  -24.6377f,-17.6016f,-4.7330f,  -24.5676f,-18.6396f,-4.0138f,  
	-22.6093f,-18.3433f,-3.7321f,  -22.5299f,-19.3728f,-3.0089f,  -20.6421f,-18.9241f,-2.5220f,  -20.5534f,-19.9436f,-1.7939f,  -18.7696f,-19.3489f,-1.1110f,  
	-18.6723f,-20.3558f,-0.3779f,  -17.0243f,-19.6240f,0.4886f,  -16.9193f,-20.6158f,1.2269f,  -15.4370f,-19.7586f,2.2613f,  -15.3251f,-20.7334f,3.0054f,  
	-14.0355f,-19.7650f,4.1892f,  -13.9186f,-20.7201f,4.9382f,  -12.8457f,-19.6574f,6.2503f,  -12.7252f,-20.5914f,7.0049f,  -11.8891f,-19.4533f,8.4219f,  
	-11.7672f,-20.3646f,9.1815f,  -11.1841f,-19.1722f,10.6783f,  -11.0628f,-20.0588f,11.4420f,  -10.7443f,-18.8348f,12.9920f,  -10.6260f,-19.6972f,13.7594f,  
	-10.5789f,-18.4659f,15.3354f,  -10.4659f,-19.3034f,16.1059f,  -10.6919f,-18.0899f,17.6802f,  -10.5863f,-18.9038f,18.4524f,  -11.0821f,-17.7339f,19.9976f,  
	-10.9859f,-18.5254f,20.7706f,  -11.7428f,-17.4251f,22.2601f,  -11.6575f,-18.1968f,23.0332f,  -12.6616f,-17.1916f,24.4416f,  -12.5881f,-17.9470f,25.2141f,  
	-13.8204f,-17.0614f,26.5177f,  -13.7590f,-17.8044f,27.2886f,  -15.1954f,-17.0616f,28.4662f,  -15.1460f,-17.7966f,29.2353f,  -16.7583f,-17.2177f,30.2683f,  
	-16.7192f,-17.9503f,31.0359f,  -18.4750f,-17.5537f,31.9088f,  -18.4429f,-18.2905f,32.6757f,  -20.3061f,-18.0912f,33.3765f,  -20.2777f,-18.8379f,34.1439f,  
	-22.2094f,-18.8474f,34.6641f,  -22.1807f,-19.6095f,35.4340f,  -24.1401f,-19.8351f,35.7687f,  -24.1037f,-20.6192f,36.5446f,  -26.0487f,-21.0632f,36.6930f,  
	-25.9985f,-21.8741f,37.4783f,  -27.8867f,-22.5341f,37.4430f,  -27.8132f,-23.3765f,38.2433f,  -29.6029f,-24.2448f,38.0307f,  -29.4975f,-25.1214f,38.8516f,  
	-31.1489f,-26.1846f,38.4711f,  -31.0030f,-27.0974f,39.3185f,  -32.4787f,-28.3372f,38.7825f,  -32.2825f,-29.2856f,39.6631f,  -33.5492f,-30.6783f,38.9863f,  
	-33.2925f,-31.6600f,39.9077f,  -34.3214f,-33.1771f,39.1066f,  -33.9950f,-34.1881f,40.0764f,  -34.7630f,-35.7968f,39.1686f,  -34.3589f,-36.8302f,40.1933f,  
	-34.8491f,-38.4943f,39.1970f,  -34.3602f,-39.5415f,40.2829f,  -34.5627f,-41.2217f,39.2161f,  -33.9835f,-42.2730f,40.3674f,  -33.8956f,-43.9284f,39.2476f,  
	-33.2227f,-44.9700f,40.4679f,  -32.8495f,-46.5590f,39.3112f,  -32.0811f,-47.5779f,40.6022f,  -31.4355f,-49.0593f,39.4229f,  -30.5723f,-50.0413f,40.7822f,  
	-29.6741f,-51.3756f,39.5927f,  -28.7191f,-52.3063f,41.0160f,  -27.5953f,-53.4570f,39.8263f,  -26.5546f,-54.3207f,41.3079f,  -25.2387f,-55.2552f,40.1258f,  
	-24.1194f,-56.0392f,41.6557f,  -22.6501f,-56.7301f,40.4869f,  -21.4628f,-57.4217f,42.0528f,  -19.8836f,-57.8480f,40.9010f,  -18.6395f,-58.4372f,42.4888f,  
	-16.9969f,-58.5846f,41.3564f,  -15.7102f,-59.0637f,42.9492f,  -14.0527f,-58.9253f,41.8377f,  -12.7367f,-59.2902f,43.4193f,  -11.1129f,-58.8669f,42.3300f,  
	-9.7816f,-59.1159f,43.8829f,  -8.2393f,-58.4161f,42.8181f,  -6.9102f,-58.5514f,44.3244f,  -5.4944f,-57.5912f,43.2873f,  -4.1786f,-57.6175f,44.7332f,  
	-2.9302f,-56.4194f,43.7291f,  -1.6414f,-56.3445f,45.1011f,  -0.5959f,-54.9371f,44.1376f,  0.6563f,-54.7706f,45.4265f,  -26.6320f,-17.7422f,-4.8042f,  
	-26.5613f,-18.7896f,-4.0816f,  -24.5676f,-18.6396f,-4.0138f,  -24.4871f,-19.6798f,-3.2869f,  -22.5299f,-19.3728f,-3.0089f,  -22.4394f,-20.4041f,-2.2767f,  
	-20.5534f,-19.9436f,-1.7939f,  -20.4534f,-20.9635f,-1.0564f,  -18.6723f,-20.3558f,-0.3779f,  -18.5635f,-21.3615f,0.3652f,  -16.9193f,-20.6158f,1.2269f,  
	-16.8025f,-21.6053f,1.9761f,  -15.3251f,-20.7334f,3.0054f,  -15.2019f,-21.7036f,3.7601f,  -13.9186f,-20.7201f,4.9382f,  -13.7905f,-21.6695f,5.6991f,  
	-12.7252f,-20.5914f,7.0049f,  -12.5941f,-21.5178f,7.7714f,  -11.7672f,-20.3646f,9.1815f,  -11.6353f,-21.2658f,9.9528f,  -11.0628f,-20.0588f,11.4420f,  
	-10.9325f,-20.9346f,12.2176f,  -10.6260f,-19.6972f,13.7594f,  -10.4997f,-20.5465f,14.5389f,  -10.4659f,-19.3034f,16.1059f,  -10.3460f,-20.1268f,16.8876f,  
	-10.5863f,-18.9038f,18.4524f,  -10.4751f,-19.7020f,19.2354f,  -10.9859f,-18.5254f,20.7706f,  -10.8853f,-19.3005f,21.5541f,  -11.6575f,-18.1968f,23.0332f,  
	-11.5690f,-18.9517f,23.8162f,  -12.5881f,-17.9470f,25.2141f,  -12.5124f,-18.6851f,25.9953f,  -13.7590f,-17.8044f,27.2886f,  -13.6965f,-18.5298f,28.0675f,  
	-15.1460f,-17.7966f,29.2353f,  -15.0959f,-18.5147f,30.0119f,  -16.7192f,-17.9503f,31.0359f,  -16.6790f,-18.6676f,31.8106f,  -18.4429f,-18.2905f,32.6757f,  
	-18.4098f,-19.0125f,33.4492f,  -20.2777f,-18.8379f,34.1439f,  -20.2483f,-19.5704f,34.9177f,  -22.1807f,-19.6095f,35.4340f,  -22.1483f,-20.3598f,36.2112f,  
	-24.1037f,-20.6192f,36.5446f,  -24.0627f,-21.3924f,37.3282f,  -25.9985f,-21.8741f,37.4783f,  -25.9397f,-22.6758f,38.2733f,  -27.8132f,-23.3765f,38.2433f,  
	-27.7280f,-24.2106f,39.0552f,  -29.4975f,-25.1214f,38.8516f,  -29.3776f,-25.9906f,39.6859f,  -31.0030f,-27.0974f,39.3185f,  -30.8381f,-28.0025f,40.1821f,  
	-32.2825f,-29.2856f,39.6631f,  -32.0620f,-30.2255f,40.5639f,  -33.2925f,-31.6600f,39.9077f,  -33.0066f,-32.6321f,40.8536f,  -33.9950f,-34.1881f,40.0764f,  
	-33.6347f,-35.1867f,41.0744f,  -34.3589f,-36.8302f,40.1933f,  -33.9158f,-37.8479f,41.2506f,  -34.3602f,-39.5415f,40.2829f,  -33.8278f,-40.5695f,41.4046f,  
	-33.9835f,-42.2730f,40.3674f,  -33.3565f,-43.2983f,41.5586f,  -33.2227f,-44.9700f,40.4679f,  -32.4977f,-45.9801f,41.7316f,  -32.0811f,-47.5779f,40.6022f,  
	-31.2575f,-48.5588f,41.9375f,  -30.5723f,-50.0413f,40.7822f,  -29.6513f,-50.9785f,42.1864f,  -28.7191f,-52.3063f,41.0160f,  -27.7051f,-53.1836f,42.4849f,  
	-26.5546f,-54.3207f,41.3079f,  -25.4533f,-55.1244f,42.8330f,  -24.1194f,-56.0392f,41.6557f,  -22.9399f,-56.7553f,43.2258f,  -21.4628f,-57.4217f,42.0528f,  
	-20.2155f,-58.0390f,43.6549f,  -18.6395f,-58.4372f,42.4888f,  -17.3379f,-58.9462f,44.1064f,  -15.7102f,-59.0637f,42.9492f,  -14.3672f,-59.4584f,44.5661f,  
	-12.7367f,-59.2902f,43.4193f,  -11.3663f,-59.5671f,45.0175f,  -9.7816f,-59.1159f,43.8829f,  -8.4018f,-59.2752f,45.4439f,  -6.9102f,-58.5514f,44.3244f,  
	-5.5329f,-58.5963f,45.8331f,  -4.1786f,-57.6175f,44.7332f,  -2.8189f,-57.5544f,46.1743f,  -1.6414f,-56.3445f,45.1011f,  -0.3107f,-56.1821f,46.4632f,  
	0.6563f,-54.7706f,45.4265f,  1.9467f,-54.5201f,46.7003f,  -26.5613f,-18.7896f,-4.0816f,  -26.4801f,-19.8398f,-3.3514f,  -24.4871f,-19.6798f,-3.2869f,  
	-24.3953f,-20.7224f,-2.5513f,  -22.4394f,-20.4041f,-2.2767f,  -22.3373f,-21.4365f,-1.5354f,  -20.4534f,-20.9635f,-1.0564f,  -20.3415f,-21.9829f,-0.3091f,  
	-18.5635f,-21.3615f,0.3652f,  -18.4423f,-22.3658f,1.1191f,  -16.8025f,-21.6053f,1.9761f,  -16.6736f,-22.5912f,2.7361f,  -15.2019f,-21.7036f,3.7601f,  
	-15.0666f,-22.6692f,4.5269f,  -13.7905f,-21.6695f,5.6991f,  -13.6506f,-22.6122f,6.4721f,  -12.5941f,-21.5178f,7.7714f,  -12.4519f,-22.4351f,8.5500f,  
	-11.6353f,-21.2658f,9.9528f,  -11.4932f,-22.1568f,10.7364f,  -10.9325f,-20.9346f,12.2176f,  -10.7929f,-21.7979f,13.0059f,  -10.4997f,-20.5465f,14.5389f,  
	-10.3652f,-21.3820f,15.3300f,  -10.3460f,-20.1268f,16.8876f,  -10.2192f,-20.9346f,17.6807f,  -10.4751f,-19.7020f,19.2354f,  -10.3583f,-20.4837f,20.0295f,  
	-10.8853f,-19.3005f,21.5541f,  -10.7803f,-20.0584f,22.3481f,  -11.5690f,-18.9517f,23.8162f,  -11.4772f,-19.6887f,24.6084f,  -12.5124f,-18.6851f,25.9953f,  
	-12.4348f,-19.4047f,26.7850f,  -13.6965f,-18.5298f,28.0675f,  -13.6329f,-19.2371f,28.8544f,  -15.0959f,-18.5147f,30.0119f,  -15.0445f,-19.2160f,30.7958f,  
	-16.6790f,-18.6676f,31.8106f,  -16.6377f,-19.3684f,32.5918f,  -18.4098f,-19.0125f,33.4492f,  -18.3760f,-19.7186f,34.2289f,  -20.2483f,-19.5704f,34.9177f,  
	-20.2159f,-20.2895f,35.6984f,  -22.1483f,-20.3598f,36.2112f,  -22.1123f,-21.0972f,36.9952f,  -24.0627f,-21.3924f,37.3282f,  -24.0143f,-22.1547f,38.1201f,  
	-25.9397f,-22.6758f,38.2733f,  -25.8707f,-23.4678f,39.0783f,  -27.7280f,-24.2106f,39.0552f,  -27.6302f,-25.0356f,39.8785f,  -29.3776f,-25.9906f,39.6859f,  
	-29.2407f,-26.8509f,40.5345f,  -30.8381f,-28.0025f,40.1821f,  -30.6511f,-28.8983f,41.0638f,  -32.0620f,-30.2255f,40.5639f,  -31.8146f,-31.1554f,41.4867f,  
	-33.0066f,-32.6321f,40.8536f,  -32.6888f,-33.5917f,41.8252f,  -33.6347f,-35.1867f,41.0744f,  -33.2373f,-36.1699f,42.1026f,  -33.9158f,-37.8479f,41.2506f,  
	-33.4311f,-38.8470f,42.3415f,  -33.8278f,-40.5695f,41.4046f,  -33.2487f,-41.5725f,42.5645f,  -33.3565f,-43.2983f,41.5586f,  -32.6782f,-44.2933f,42.7919f,  
	-32.4977f,-45.9801f,41.7316f,  -31.7181f,-46.9535f,43.0389f,  -31.2575f,-48.5588f,41.9375f,  -30.3760f,-49.4962f,43.3176f,  -29.6513f,-50.9785f,42.1864f,  
	-28.6705f,-51.8632f,43.6368f,  -27.7051f,-53.1836f,42.4849f,  -26.6291f,-54.0014f,43.9985f,  -25.4533f,-55.1244f,42.8330f,  -24.2898f,-55.8603f,44.4001f,  
	-22.9399f,-56.7553f,43.2258f,  -21.6981f,-57.3968f,44.8345f,  -20.2155f,-58.0390f,43.6549f,  -18.9078f,-58.5748f,45.2894f,  -17.3379f,-58.9462f,44.1064f,  
	-15.9766f,-59.3685f,45.7510f,  -14.3672f,-59.4584f,44.5661f,  -12.9662f,-59.7620f,46.2026f,  -11.3663f,-59.5671f,45.0175f,  -9.9437f,-59.7500f,46.6268f,  
	-8.4018f,-59.2752f,45.4439f,  -6.9701f,-59.3391f,47.0098f,  -5.5329f,-58.5963f,45.8331f,  -4.1085f,-58.5461f,47.3386f,  -2.8189f,-57.5544f,46.1743f,  
	-1.4144f,-57.3975f,47.6063f,  -0.3107f,-56.1821f,46.4632f,  1.0606f,-55.9289f,47.8103f,  1.9467f,-54.5201f,46.7003f,  3.2778f,-54.1811f,47.9563f,  
	-26.4801f,-19.8398f,-3.3514f,  -26.3875f,-20.8931f,-2.6128f,  -24.3953f,-20.7224f,-2.5513f,  -24.2916f,-21.7669f,-1.8067f,  -22.3373f,-21.4365f,-1.5354f,  
	-22.2229f,-22.4694f,-0.7844f,  -20.3415f,-21.9829f,-0.3091f,  -20.2167f,-23.0020f,0.4490f,  -18.4423f,-22.3658f,1.1191f,  -18.3086f,-23.3675f,1.8839f,  
	-16.6736f,-22.5912f,2.7361f,  -16.5319f,-23.5734f,3.5082f,  -15.0666f,-22.6692f,4.5269f,  -14.9187f,-23.6291f,5.3059f,  -13.6506f,-22.6122f,6.4721f,  
	-13.4986f,-23.5468f,7.2575f,  -12.4519f,-22.4351f,8.5500f,  -12.2983f,-23.3431f,9.3412f,  -11.4932f,-22.1568f,10.7364f,  -11.3403f,-23.0363f,11.5331f,  
	-10.7929f,-21.7979f,13.0059f,  -10.6438f,-22.6480f,13.8062f,  -10.3652f,-21.3820f,15.3300f,  -10.2225f,-22.2024f,16.1331f,  -10.2192f,-20.9346f,17.6807f,  
	-10.0854f,-21.7262f,18.4855f,  -10.3583f,-20.4837f,20.0295f,  -10.2357f,-21.2481f,20.8347f,  -10.7803f,-20.0584f,22.3481f,  -10.6709f,-20.7979f,23.1518f,  
	-11.4772f,-19.6887f,24.6084f,  -11.3824f,-20.4063f,25.4098f,  -12.4348f,-19.4047f,26.7850f,  -12.3551f,-20.1052f,27.5833f,  -13.6329f,-19.2371f,28.8544f,  
	-13.5673f,-19.9264f,29.6489f,  -15.0445f,-19.2160f,30.7958f,  -14.9918f,-19.8994f,31.5864f,  -16.6377f,-19.3684f,32.5918f,  -16.5958f,-20.0516f,33.3791f,  
	-18.3760f,-19.7186f,34.2289f,  -18.3396f,-20.4096f,35.0149f,  -20.2159f,-20.2895f,35.6984f,  -20.1807f,-20.9938f,36.4850f,  -22.1123f,-21.0972f,36.9952f,  
	-22.0700f,-21.8221f,37.7864f,  -24.0143f,-22.1547f,38.1201f,  -23.9572f,-22.9056f,38.9207f,  -25.8707f,-23.4678f,39.0783f,  -25.7909f,-24.2489f,39.8931f,  
	-27.6302f,-25.0356f,39.8785f,  -27.5176f,-25.8505f,40.7143f,  -29.2407f,-26.8509f,40.5345f,  -29.0839f,-27.7010f,41.3989f,  -30.6511f,-28.8983f,41.0638f,  
	-30.4395f,-29.7835f,41.9650f,  -31.8146f,-31.1554f,41.4867f,  -31.5375f,-32.0727f,42.4328f,  -32.6888f,-33.5917f,41.8252f,  -32.3361f,-34.5363f,42.8245f,  
	-33.2373f,-36.1699f,42.1026f,  -32.8002f,-37.1350f,43.1621f,  -33.4311f,-38.8470f,42.3415f,  -32.9011f,-39.8220f,43.4686f,  -33.2487f,-41.5725f,42.5645f,  
	-32.6195f,-42.5463f,43.7648f,  -32.6782f,-44.2933f,42.7919f,  -31.9460f,-45.2529f,44.0675f,  -31.7181f,-46.9535f,43.0389f,  -30.8808f,-47.8847f,44.3901f,  
	-30.3760f,-49.4962f,43.3176f,  -29.4342f,-50.3822f,44.7438f,  -28.6705f,-51.8632f,43.6368f,  -27.6267f,-52.6894f,45.1324f,  -26.6291f,-54.0014f,43.9985f,  
	-25.4894f,-54.7520f,45.5554f,  -24.2898f,-55.8603f,44.4001f,  -23.0618f,-56.5215f,46.0076f,  -21.6981f,-57.3968f,44.8345f,  -20.3931f,-57.9557f,46.4780f,  
	-18.9078f,-58.5748f,45.2894f,  -17.5373f,-59.0221f,46.9535f,  -15.9766f,-59.3685f,45.7510f,  -14.5540f,-59.6972f,47.4178f,  -12.9662f,-59.7620f,46.2026f,  
	-11.5094f,-59.9679f,47.8528f,  -9.9437f,-59.7500f,46.6268f,  -8.4656f,-59.8332f,48.2433f,  -6.9701f,-59.3391f,47.0098f,  -5.4879f,-59.3026f,48.5744f,  
	-4.1085f,-58.5461f,47.3386f,  -2.6360f,-58.3960f,48.8363f,  -1.4144f,-57.3975f,47.6063f,  0.0334f,-57.1432f,49.0237f,  1.0606f,-55.9289f,47.8103f,  
	2.4747f,-55.5805f,49.1390f,  3.2778f,-54.1811f,47.9563f,  4.6500f,-53.7501f,49.1907f,  -26.3875f,-20.8931f,-2.6128f,  -26.2829f,-21.9489f,-1.8656f,  
	-24.2916f,-21.7669f,-1.8067f,  -24.1754f,-22.8127f,-1.0526f,  -22.2229f,-22.4694f,-0.7844f,  -22.0952f,-23.5029f,-0.0227f,  -20.2167f,-23.0020f,0.4490f,  
	-20.0788f,-24.0194f,1.2179f,  -18.3086f,-23.3675f,1.8839f,  -18.1612f,-24.3666f,2.6607f,  -16.5319f,-23.5734f,3.5082f,  -16.3767f,-24.5509f,4.2927f,  
	-14.9187f,-23.6291f,5.3059f,  -14.7577f,-24.5819f,6.0976f,  -13.4986f,-23.5468f,7.2575f,  -13.3342f,-24.4730f,8.0558f,  -12.2983f,-23.3431f,9.3412f,  
	-12.1327f,-24.2403f,10.1459f,  -11.3403f,-23.0363f,11.5331f,  -11.1768f,-23.9030f,12.3423f,  -10.6438f,-22.6480f,13.8062f,  -10.4850f,-23.4834f,14.6191f,  
	-10.2225f,-22.2024f,16.1331f,  -10.0713f,-23.0067f,16.9486f,  -10.0854f,-21.7262f,18.4855f,  -9.9444f,-22.5004f,19.3021f,  -10.2357f,-21.2481f,20.8347f,  
	-10.1074f,-21.9938f,21.6503f,  -10.6709f,-20.7979f,23.1518f,  -10.5573f,-21.5175f,23.9653f,  -11.3824f,-20.4063f,25.4098f,  -11.2845f,-21.1041f,26.2202f,  
	-12.3551f,-20.1052f,27.5833f,  -12.2728f,-20.7866f,28.3894f,  -13.5673f,-19.9264f,29.6489f,  -13.5000f,-20.5964f,30.4502f,  -14.9918f,-19.8994f,31.5864f,  
	-14.9381f,-20.5635f,32.3830f,  -16.5958f,-20.0516f,33.3791f,  -16.5514f,-20.7179f,34.1725f,  -18.3396f,-20.4096f,35.0149f,  -18.3009f,-21.0838f,35.8064f,  
	-20.1807f,-20.9938f,36.4850f,  -20.1402f,-21.6836f,37.2778f,  -22.0700f,-21.8221f,37.7864f,  -22.0204f,-22.5338f,38.5850f,  -23.9572f,-22.9056f,38.9207f,  
	-23.8911f,-23.6439f,39.7293f,  -25.7909f,-24.2489f,39.8931f,  -25.6983f,-25.0185f,40.7183f,  -27.5176f,-25.8505f,40.7143f,  -27.3871f,-26.6541f,41.5635f,  
	-29.0839f,-27.7010f,41.3989f,  -28.9047f,-28.5396f,42.2802f,  -30.4395f,-29.7835f,41.9650f,  -30.2004f,-30.6556f,42.8868f,  -31.5375f,-32.0727f,42.4328f,  
	-31.2277f,-32.9748f,43.4037f,  -32.3361f,-34.5363f,42.8245f,  -31.9460f,-35.4629f,43.8523f,  -32.8002f,-37.1350f,43.1621f,  -32.3197f,-38.0770f,44.2552f,  
	-32.9011f,-39.8220f,43.4686f,  -32.3225f,-40.7690f,44.6338f,  -32.6195f,-42.5463f,43.7648f,  -31.9375f,-43.4860f,45.0057f,  -31.9460f,-45.2529f,44.0675f,  
	-31.1571f,-46.1716f,45.3858f,  -30.8808f,-47.8847f,44.3901f,  -29.9832f,-48.7659f,45.7870f,  -29.4342f,-50.3822f,44.7438f,  -28.4290f,-51.2110f,46.2153f,  
	-27.6267f,-52.6894f,45.1324f,  -26.5180f,-53.4493f,46.6721f,  -25.4894f,-54.7520f,45.5554f,  -24.2834f,-55.4281f,47.1541f,  -23.0618f,-56.5215f,46.0076f,  
	-21.7685f,-57.0998f,47.6518f,  -20.3931f,-57.9557f,46.4780f,  -19.0227f,-58.4251f,48.1533f,  -17.5373f,-59.0221f,46.9535f,  -16.1026f,-59.3738f,48.6423f,  
	-14.5540f,-59.6972f,47.4178f,  -13.0723f,-59.9250f,49.1009f,  -11.5094f,-59.9679f,47.8528f,  -9.9937f,-60.0701f,49.5125f,  -8.4656f,-59.8332f,48.2433f,  
	-6.9336f,-59.8110f,49.8603f,  -5.4879f,-59.3026f,48.5744f,  -3.9545f,-59.1605f,50.1322f,  -2.6360f,-58.3960f,48.8363f,  -1.1174f,-58.1421f,50.3198f,  
	0.0334f,-57.1432f,49.0237f,  1.5261f,-56.7870f,50.4222f,  2.4747f,-55.5805f,49.1390f,  3.9314f,-55.1333f,50.4447f,  4.6500f,-53.7501f,49.1907f,  
	6.0640f,-53.2233f,50.3996f,  -26.2829f,-21.9489f,-1.8656f,  -26.1657f,-23.0069f,-1.1091f,  -24.1754f,-22.8127f,-1.0526f,  -24.0456f,-23.8599f,-0.2881f,  
	-22.0952f,-23.5029f,-0.0227f,  -21.9539f,-24.5357f,0.7495f,  -20.0788f,-24.0194f,1.2179f,  -19.9266f,-25.0352f,1.9987f,  -18.1612f,-24.3666f,2.6607f,  
	-17.9996f,-25.3621f,3.4498f,  -16.3767f,-24.5509f,4.2927f,  -16.2074f,-25.5226f,5.0899f,  -14.7577f,-24.5819f,6.0976f,  -14.5832f,-25.5273f,6.9022f,  
	-13.3342f,-24.4730f,8.0558f,  -13.1565f,-25.3895f,8.8678f,  -12.1327f,-24.2403f,10.1459f,  -11.9552f,-25.1257f,10.9635f,  -11.1768f,-23.9030f,12.3423f,  
	-11.0021f,-24.7559f,13.1647f,  -10.4850f,-23.4834f,14.6191f,  -10.3162f,-24.3033f,15.4450f,  -10.0713f,-23.0067f,16.9486f,  -9.9115f,-23.7939f,17.7764f,  
	-9.9444f,-22.5004f,19.3021f,  -9.7964f,-23.2559f,20.1296f,  -10.1074f,-21.9938f,21.6503f,  -9.9736f,-22.7193f,22.4763f,  -10.5573f,-21.5175f,23.9653f,  
	-10.4394f,-22.2168f,24.7884f,  -11.2845f,-21.1041f,26.2202f,  -11.1831f,-21.7819f,27.0388f,  -12.2728f,-20.7866f,28.3894f,  -12.1881f,-21.4475f,29.2026f,  
	-13.5000f,-20.5964f,30.4502f,  -13.4314f,-21.2456f,31.2576f,  -14.9381f,-20.5635f,32.3830f,  -14.8821f,-21.2092f,33.1856f,  -16.5514f,-20.7179f,34.1725f,  
	-16.5052f,-21.3656f,34.9709f,  -18.3009f,-21.0838f,35.8064f,  -18.2578f,-21.7417f,36.6032f,  -20.1402f,-21.6836f,37.2778f,  -20.0935f,-22.3586f,38.0769f,  
	-22.0204f,-22.5338f,38.5850f,  -21.9633f,-23.2310f,39.3900f,  -23.8911f,-23.6439f,39.7293f,  -23.8139f,-24.3691f,40.5465f,  -25.6983f,-25.0185f,40.7183f,  
	-25.5898f,-25.7756f,41.5548f,  -27.3871f,-26.6541f,41.5635f,  -27.2365f,-27.4452f,42.4271f,  -28.9047f,-28.5396f,42.2802f,  -28.7005f,-29.3646f,43.1794f,  
	-30.2004f,-30.6556f,42.8868f,  -29.9311f,-31.5124f,43.8305f,  -31.2277f,-32.9748f,43.4037f,  -30.8830f,-33.8592f,44.4004f,  -31.9460f,-35.4629f,43.8523f,  
	-31.5144f,-36.3671f,44.9111f,  -32.3197f,-38.0770f,44.2552f,  -31.7924f,-38.9918f,45.3840f,  -32.3225f,-40.7690f,44.6338f,  -31.6927f,-41.6832f,45.8374f,  
	-31.9375f,-43.4860f,45.0057f,  -31.2000f,-44.3863f,46.2874f,  -31.1571f,-46.1716f,45.3858f,  -30.3081f,-47.0420f,46.7486f,  -29.9832f,-48.7659f,45.7870f,  
	-29.0222f,-49.5913f,47.2286f,  -28.4290f,-51.2110f,46.2153f,  -27.3582f,-51.9745f,47.7309f,  -26.5180f,-53.4493f,46.6721f,  -25.3419f,-54.1354f,48.2543f,  
	-24.2834f,-55.4281f,47.1541f,  -23.0103f,-56.0214f,48.7907f,  -21.7685f,-57.0998f,47.6518f,  -20.4073f,-57.5878f,49.3297f,  -19.0227f,-58.4251f,48.1533f,  
	-17.5854f,-58.7975f,49.8553f,  -16.1026f,-59.3738f,48.6423f,  -14.6055f,-59.6222f,50.3498f,  -13.0723f,-59.9250f,49.1009f,  -11.5284f,-60.0458f,50.7956f,  
	-9.9937f,-60.0701f,49.5125f,  -8.4208f,-60.0625f,51.1744f,  -6.9336f,-59.8110f,49.8603f,  -5.3473f,-59.6779f,51.4717f,  -3.9545f,-59.1605f,50.1322f,  
	-2.3722f,-58.9086f,51.6763f,  -1.1174f,-58.1421f,50.3198f,  0.4483f,-57.7800f,51.7840f,  1.5261f,-56.7870f,50.4222f,  3.0632f,-56.3254f,51.7965f,  
	3.9314f,-55.1333f,50.4447f,  5.4310f,-54.5837f,51.7228f,  6.0640f,-53.2233f,50.3996f,  7.5191f,-52.5978f,51.5781f,  -26.1657f,-23.0069f,-1.1091f,  
	-26.0346f,-24.0671f,-0.3425f,  -24.0456f,-23.8599f,-0.2881f,  -23.9020f,-24.9075f,0.4868f,  -21.9539f,-24.5357f,0.7495f,  -21.7980f,-25.5679f,1.5335f,  
	-19.9266f,-25.0352f,1.9987f,  -19.7596f,-26.0485f,2.7917f,  -17.9996f,-25.3621f,3.4498f,  -17.8231f,-26.3529f,4.2517f,  -16.2074f,-25.5226f,5.0899f,  
	-16.0238f,-26.4879f,5.9002f,  -14.5832f,-25.5273f,6.9022f,  -14.3942f,-26.4641f,7.7209f,  -13.1565f,-25.3895f,8.8678f,  -12.9656f,-26.2950f,9.6931f,  
	-11.9552f,-25.1257f,10.9635f,  -11.7651f,-25.9980f,11.7946f,  -11.0021f,-24.7559f,13.1647f,  -10.8160f,-25.5939f,14.0004f,  -10.3162f,-24.3033f,15.4450f,  
	-10.1374f,-25.1066f,16.2837f,  -9.9115f,-23.7939f,17.7764f,  -9.7432f,-24.5626f,18.6158f,  -9.7964f,-23.2559f,20.1296f,  -9.6414f,-23.9911f,20.9683f,  
	-9.9736f,-22.7193f,22.4763f,  -9.8341f,-23.4241f,23.3126f,  -10.4394f,-22.2168f,24.7884f,  -10.3168f,-22.8953f,25.6203f,  -11.1831f,-21.7819f,27.0388f,  
	-11.0784f,-22.4382f,27.8649f,  -12.1881f,-21.4475f,29.2026f,  -12.1013f,-22.0864f,30.0222f,  -13.4314f,-21.2456f,31.2576f,  -13.3601f,-21.8750f,32.0711f,  
	-14.8821f,-21.2092f,33.1856f,  -14.8242f,-21.8346f,33.9929f,  -16.5052f,-21.3656f,34.9709f,  -16.4550f,-21.9954f,35.7739f,  -18.2578f,-21.7417f,36.6032f,  
	-18.2092f,-22.3831f,37.4051f,  -20.0935f,-22.3586f,38.0769f,  -20.0405f,-23.0171f,38.8809f,  -21.9633f,-23.2310f,39.3900f,  -21.8967f,-23.9133f,40.2018f,  
	-23.8139f,-24.3691f,40.5465f,  -23.7228f,-25.0803f,41.3728f,  -25.5898f,-25.7756f,41.5548f,  -25.4636f,-26.5189f,42.4030f,  -27.2365f,-27.4452f,42.4271f,  
	-27.0633f,-28.2219f,43.3057f,  -28.7005f,-29.3646f,43.1794f,  -28.4684f,-30.1739f,44.0975f,  -29.9311f,-31.5124f,43.8305f,  -29.6294f,-32.3514f,44.7968f,  
	-30.8830f,-33.8592f,44.4004f,  -30.4991f,-34.7218f,45.4249f,  -31.5144f,-36.3671f,44.9111f,  -31.0382f,-37.2452f,46.0025f,  -31.7924f,-38.9918f,45.3840f,  
	-31.2158f,-39.8751f,46.5485f,  -31.6927f,-41.6832f,45.8374f,  -31.0089f,-42.5593f,47.0796f,  -31.2000f,-44.3863f,46.2874f,  -30.4033f,-45.2399f,47.6120f,  
	-30.3081f,-47.0420f,46.7486f,  -29.3963f,-47.8581f,48.1549f,  -29.0222f,-49.5913f,47.2286f,  -27.9956f,-50.3528f,48.7139f,  -27.3582f,-51.9745f,47.7309f,  
	-26.2193f,-52.6654f,49.2894f,  -25.3419f,-54.1354f,48.2543f,  -24.0973f,-54.7392f,49.8753f,  -23.0103f,-56.0214f,48.7907f,  -21.6674f,-56.5245f,50.4625f,  
	-20.4073f,-57.5878f,49.3297f,  -18.9770f,-57.9780f,51.0359f,  -17.5854f,-58.7975f,49.8553f,  -16.0828f,-59.0645f,51.5781f,  -14.6055f,-59.6222f,50.3498f,  
	-13.0433f,-59.7607f,52.0707f,  -11.5284f,-60.0458f,50.7956f,  -9.9242f,-60.0530f,52.4942f,  -8.4208f,-60.0625f,51.1744f,  -6.7907f,-59.9395f,52.8320f,  
	-5.3473f,-59.6779f,51.4717f,  -3.7094f,-59.4296f,53.0701f,  -2.3722f,-58.9086f,51.6763f,  -0.7404f,-58.5425f,53.2010f,  0.4483f,-57.7800f,51.7840f,  
	2.0602f,-57.3060f,53.2228f,  3.0632f,-56.3254f,51.7965f,  4.6443f,-55.7549f,53.1410f,  5.4310f,-54.5837f,51.7228f,  6.9719f,-53.9291f,52.9675f,  
	7.5191f,-52.5978f,51.5781f,  9.0171f,-51.8691f,52.7227f,  -26.0346f,-24.0671f,-0.3425f,  -25.8895f,-25.1285f,0.4343f,  -23.9020f,-24.9075f,0.4868f,  
	-23.7433f,-25.9555f,1.2732f,  -21.7980f,-25.5679f,1.5335f,  -21.6267f,-26.5987f,2.3296f,  -19.7596f,-26.0485f,2.7917f,  -19.5770f,-27.0584f,3.5975f,  
	-17.8231f,-26.3529f,4.2517f,  -17.6314f,-27.3383f,5.0668f,  -16.0238f,-26.4879f,5.9002f,  -15.8245f,-27.4458f,6.7246f,  -14.3942f,-26.4641f,7.7209f,  
	-14.1908f,-27.3910f,8.5531f,  -12.9656f,-26.2950f,9.6931f,  -12.7609f,-27.1885f,10.5324f,  -11.7651f,-25.9980f,11.7946f,  -11.5622f,-26.8563f,12.6396f,  
	-10.8160f,-25.5939f,14.0004f,  -10.6184f,-26.4158f,14.8496f,  -10.1374f,-25.1066f,16.2837f,  -9.9486f,-25.8916f,17.1347f,  -9.7432f,-24.5626f,18.6158f,  
	-9.5662f,-25.3112f,19.4670f,  -9.6414f,-23.9911f,20.9683f,  -9.4792f,-24.7055f,21.8179f,  -9.8341f,-23.4241f,23.3126f,  -9.6888f,-24.1076f,24.1581f,  
	-10.3168f,-22.8953f,25.6203f,  -10.1899f,-23.5514f,26.4601f,  -11.0784f,-22.4382f,27.8649f,  -10.9707f,-23.0713f,28.6979f,  -12.1013f,-22.0864f,30.0222f,  
	-12.0114f,-22.7041f,30.8480f,  -13.3601f,-21.8750f,32.0711f,  -13.2867f,-22.4825f,32.8891f,  -14.8242f,-21.8346f,33.9929f,  -14.7626f,-22.4402f,34.8042f,  
	-16.4550f,-21.9954f,35.7739f,  -16.4000f,-22.6068f,36.5812f,  -18.2092f,-22.3831f,37.4051f,  -18.1556f,-23.0060f,38.2109f,  -20.0405f,-23.0171f,38.8809f,  
	-19.9795f,-23.6591f,39.6902f,  -21.8967f,-23.9133f,40.2018f,  -21.8181f,-24.5804f,41.0208f,  -23.7228f,-25.0803f,41.3728f,  -23.6161f,-25.7765f,42.2083f,  
	-25.4636f,-26.5189f,42.4030f,  -25.3169f,-27.2469f,43.2635f,  -27.0633f,-28.2219f,43.3057f,  -26.8648f,-28.9824f,44.2002f,  -28.4684f,-30.1739f,44.0975f,  
	-28.2067f,-30.9653f,45.0350f,  -29.6294f,-32.3514f,44.7968f,  -29.2910f,-33.1691f,45.7878f,  -30.4991f,-34.7218f,45.4249f,  -30.0728f,-35.5590f,46.4788f,  
	-31.0382f,-37.2452f,46.0025f,  -30.5148f,-38.0928f,47.1267f,  -31.2158f,-39.8751f,46.5485f,  -30.5871f,-40.7219f,47.7489f,  -31.0089f,-42.5593f,47.0796f,  
	-30.2673f,-43.3905f,48.3626f,  -30.4033f,-45.2399f,47.6120f,  -29.5448f,-46.0410f,48.9786f,  -29.3963f,-47.8581f,48.1549f,  -28.4191f,-48.6121f,49.6041f,  
	-27.9956f,-50.3528f,48.7139f,  -26.9005f,-51.0433f,50.2417f,  -26.2193f,-52.6654f,49.2894f,  -25.0109f,-53.2751f,50.8871f,  -24.0973f,-54.7392f,49.8753f,  
	-22.7816f,-55.2532f,51.5325f,  -21.6674f,-56.5245f,50.4625f,  -20.2537f,-56.9294f,52.1640f,  -18.9770f,-57.9780f,51.0359f,  -17.4790f,-58.2616f,52.7647f,  
	-16.0828f,-59.0645f,51.5781f,  -14.5125f,-59.2196f,53.3160f,  -13.0433f,-59.7607f,52.0707f,  -11.4179f,-59.7824f,53.7972f,  -9.9242f,-60.0530f,52.4942f,  
	-8.2599f,-59.9406f,54.1898f,  -6.7907f,-59.9395f,52.8320f,  -5.1061f,-59.6961f,54.4773f,  -3.7094f,-59.4296f,53.0701f,  -2.0195f,-59.0613f,54.6489f,  
	-0.7404f,-58.5425f,53.2010f,  0.9395f,-58.0583f,54.6993f,  2.0602f,-57.3060f,53.2228f,  3.7173f,-56.7168f,54.6302f,  4.6443f,-55.7549f,53.1410f,  
	6.2671f,-55.0731f,54.4496f,  6.9719f,-53.9291f,52.9675f,  8.5556f,-53.1653f,54.1749f,  9.0171f,-51.8691f,52.7227f,  10.5581f,-51.0337f,53.8287f,  
	-25.8895f,-25.1285f,0.4343f,  -25.7291f,-26.1913f,1.2222f,  -23.7433f,-25.9555f,1.2732f,  -23.5689f,-27.0030f,2.0714f,  -21.6267f,-26.5987f,2.3296f,  
	-21.4392f,-27.6273f,3.1384f,  -19.5770f,-27.0584f,3.5975f,  -19.3785f,-28.0641f,4.4164f,  -17.6314f,-27.3383f,5.0668f,  -17.4231f,-28.3178f,5.8961f,  
	-15.8245f,-27.4458f,6.7246f,  -15.6099f,-28.3950f,7.5628f,  -14.1908f,-27.3910f,8.5531f,  -13.9724f,-28.3071f,9.3996f,  -12.7609f,-27.1885f,10.5324f,  
	-12.5418f,-28.0690f,11.3859f,  -11.5622f,-26.8563f,12.6396f,  -11.3461f,-27.6993f,13.4987f,  -10.6184f,-26.4158f,14.8496f,  -10.4091f,-27.2202f,15.7117f,  
	-9.9486f,-25.8916f,17.1347f,  -9.7495f,-26.6570f,17.9982f,  -9.5662f,-25.3112f,19.4670f,  -9.3806f,-26.0389f,20.3298f,  -9.4792f,-24.7055f,21.8179f,  
	-9.3098f,-25.3982f,22.6773f,  -9.6888f,-24.1076f,24.1581f,  -9.5379f,-24.7682f,25.0123f,  -10.1899f,-23.5514f,26.4601f,  -10.0590f,-24.1836f,27.3073f,  
	-10.9707f,-23.0713f,28.6979f,  -10.8593f,-23.6822f,29.5372f,  -12.0114f,-22.7041f,30.8480f,  -11.9190f,-23.2985f,31.6784f,  -13.2867f,-22.4825f,32.8891f,  
	-13.2097f,-23.0687f,33.7108f,  -14.7626f,-22.4402f,34.8042f,  -14.6968f,-23.0259f,35.6192f,  -16.4000f,-22.6068f,36.5812f,  -16.3408f,-23.1979f,37.3912f,  
	-18.1556f,-23.0060f,38.2109f,  -18.0951f,-23.6107f,39.0204f,  -19.9795f,-23.6591f,39.6902f,  -19.9081f,-24.2842f,40.5047f,  -21.8181f,-24.5804f,41.0208f,  
	-21.7257f,-25.2309f,41.8465f,  -23.6161f,-25.7765f,42.2083f,  -23.4913f,-26.4565f,43.0534f,  -25.3169f,-27.2469f,43.2635f,  -25.1475f,-27.9581f,44.1368f,  
	-26.8648f,-28.9824f,44.2002f,  -26.6392f,-29.7246f,45.1107f,  -28.2067f,-30.9653f,45.0350f,  -27.9109f,-31.7356f,45.9937f,  -29.2910f,-33.1691f,45.7878f,  
	-28.9128f,-33.9620f,46.8047f,  -30.0728f,-35.5590f,46.4788f,  -29.6019f,-36.3668f,47.5621f,  -30.5148f,-38.0928f,47.1267f,  -29.9418f,-38.9054f,48.2837f,  
	-30.5871f,-40.7219f,47.7489f,  -29.9022f,-41.5255f,48.9875f,  -30.2673f,-43.3905f,48.3626f,  -29.4651f,-44.1712f,49.6853f,  -29.5448f,-46.0410f,48.9786f,  
	-28.6217f,-46.7821f,50.3864f,  -28.4191f,-48.6121f,49.6041f,  -27.3738f,-49.2968f,51.0949f,  -26.9005f,-51.0433f,50.2417f,  -25.7356f,-51.6540f,51.8086f,  
	-25.0109f,-53.2751f,50.8871f,  -23.7306f,-53.7959f,52.5213f,  -22.7816f,-55.2532f,51.5325f,  -21.3936f,-55.6695f,53.2202f,  -20.2537f,-56.9294f,52.1640f,  
	-18.7701f,-57.2271f,53.8894f,  -17.4790f,-58.2616f,52.7647f,  -15.9109f,-58.4319f,54.5102f,  -14.5125f,-59.2196f,53.3160f,  -12.8765f,-59.2556f,55.0610f,  
	-11.4179f,-59.7824f,53.7972f,  -9.7298f,-59.6813f,55.5218f,  -8.2599f,-59.9406f,54.1898f,  -6.5385f,-59.7035f,55.8738f,  -5.1061f,-59.6961f,54.4773f,  
	-3.3673f,-59.3277f,56.1031f,  -2.0195f,-59.0613f,54.6489f,  -0.2797f,-58.5695f,56.2008f,  0.9395f,-58.0583f,54.6993f,  2.6660f,-57.4530f,56.1647f,  
	3.7173f,-56.7168f,54.6302f,  5.4168f,-56.0104f,55.9990f,  6.2671f,-55.0731f,54.4496f,  7.9328f,-54.2764f,55.7175f,  8.5556f,-53.1653f,54.1749f,  
	10.1817f,-52.2894f,55.3399f,  10.5581f,-51.0337f,53.8287f,  12.1394f,-50.0901f,54.8901f,  -25.7291f,-26.1913f,1.2222f,  -25.5528f,-27.2547f,2.0216f,  
	-23.5689f,-27.0030f,2.0714f,  -23.3778f,-28.0497f,2.8822f,  -21.4392f,-27.6273f,3.1384f,  -21.2352f,-28.6528f,3.9601f,  -19.3785f,-28.0641f,4.4164f,  
	-19.1625f,-29.0651f,5.2496f,  -17.4231f,-28.3178f,5.8961f,  -17.1984f,-29.2897f,6.7393f,  -15.6099f,-28.3950f,7.5628f,  -15.3789f,-29.3345f,8.4156f,  
	-13.9724f,-28.3071f,9.3996f,  -13.7383f,-29.2111f,10.2607f,  -12.5418f,-28.0690f,11.3859f,  -12.3082f,-28.9351f,12.2539f,  -11.3461f,-27.6993f,13.4987f,  
	-11.1169f,-28.5255f,14.3711f,  -10.4091f,-27.2202f,15.7117f,  -10.1880f,-28.0054f,16.5869f,  -9.7495f,-26.6570f,17.9982f,  -9.5400f,-27.4017f,18.8740f,  
	-9.3806f,-26.0389f,20.3298f,  -9.1863f,-26.7450f,21.2032f,  -9.3098f,-25.3982f,22.6773f,  -9.1334f,-26.0677f,23.5460f,  -9.5379f,-24.7682f,25.0123f,  
	-9.3816f,-25.4042f,25.8743f,  -10.0590f,-24.1836f,27.3073f,  -9.9233f,-24.7925f,28.1613f,  -10.8593f,-23.6822f,29.5372f,  -10.7446f,-24.2684f,30.3814f,  
	-11.9190f,-23.2985f,31.6784f,  -11.8228f,-23.8700f,32.5122f,  -13.2097f,-23.0687f,33.7108f,  -13.1287f,-23.6332f,34.5357f,  -14.6968f,-23.0259f,35.6192f,  
	-14.6272f,-23.5894f,36.4361f,  -16.3408f,-23.1979f,37.3912f,  -16.2757f,-23.7689f,38.2038f,  -18.0951f,-23.6107f,39.0204f,  -18.0256f,-24.1969f,39.8334f,  
	-19.9081f,-24.2842f,40.5047f,  -19.8248f,-24.8913f,41.3237f,  -21.7257f,-25.2309f,41.8465f,  -21.6175f,-25.8640f,42.6792f,  -23.4913f,-26.4565f,43.0534f,  
	-23.3461f,-27.1187f,43.9083f,  -25.1475f,-27.9581f,44.1368f,  -24.9538f,-28.6504f,45.0227f,  -26.6392f,-29.7246f,45.1107f,  -26.3823f,-30.4458f,46.0390f,  
	-27.9109f,-31.7356f,45.9937f,  -27.5779f,-32.4817f,46.9746f,  -28.9128f,-33.9620f,46.8047f,  -28.4927f,-34.7266f,47.8474f,  -29.6019f,-36.3668f,47.5621f,  
	-29.0839f,-37.1410f,48.6750f,  -29.9418f,-38.9054f,48.2837f,  -29.3145f,-39.6767f,49.4759f,  -29.9022f,-41.5255f,48.9875f,  -29.1588f,-42.2806f,50.2632f,  
	-29.4651f,-44.1712f,49.6853f,  -28.5997f,-44.8940f,51.0474f,  -28.6217f,-46.7821f,50.3864f,  -27.6312f,-47.4559f,51.8345f,  -27.3738f,-49.2968f,51.0949f,  
	-26.2588f,-49.9038f,52.6242f,  -25.7356f,-51.6540f,51.8086f,  -24.4983f,-52.1775f,53.4119f,  -23.7306f,-53.7959f,52.5213f,  -22.3772f,-54.2201f,54.1865f,  
	-21.3936f,-55.6695f,53.2202f,  -19.9341f,-55.9788f,54.9329f,  -18.7701f,-57.2271f,53.8894f,  -17.2146f,-57.4110f,55.6327f,  -15.9109f,-58.4319f,54.5102f,  
	-14.2749f,-58.4814f,56.2640f,  -12.8765f,-59.2556f,55.0610f,  -11.1752f,-59.1661f,56.8053f,  -9.7298f,-59.6813f,55.5218f,  -7.9820f,-59.4519f,57.2358f,  
	-6.5385f,-59.7035f,55.8738f,  -4.7606f,-59.3372f,57.5387f,  -3.3673f,-59.3277f,56.1031f,  -1.5765f,-58.8308f,57.7015f,  -0.2797f,-58.5695f,56.2008f,  
	1.5082f,-57.9510f,57.7184f,  2.6660f,-57.4530f,56.1647f,  4.4356f,-56.7247f,57.5893f,  5.4168f,-56.0104f,55.9990f,  7.1594f,-55.1833f,57.3242f,  
	7.9328f,-54.2764f,55.7175f,  9.6405f,-53.3620f,56.9395f,  10.1817f,-52.2894f,55.3399f,  11.8468f,-51.3003f,56.4559f,  12.1394f,-50.0901f,54.8901f,  
	13.7617f,-49.0346f,55.9025f,  -25.5528f,-27.2547f,2.0216f,  -25.3595f,-28.3183f,2.8333f,  -23.3778f,-28.0497f,2.8822f,  -23.1697f,-29.0944f,3.7057f,  
	-21.2352f,-28.6528f,3.9601f,  -21.0130f,-29.6750f,4.7960f,  -19.1625f,-29.0651f,5.2496f,  -18.9293f,-30.0598f,6.0967f,  -17.1984f,-29.2897f,6.7393f,  
	-16.9563f,-30.2532f,7.5972f,  -15.3789f,-29.3345f,8.4156f,  -15.1310f,-30.2633f,9.2832f,  -13.7383f,-29.2111f,10.2607f,  -13.4882f,-30.1020f,11.1367f,  
	-12.3082f,-28.9351f,12.2539f,  -12.0599f,-29.7853f,13.1359f,  -11.1169f,-28.5255f,14.3711f,  -10.8742f,-29.3334f,15.2574f,  -10.1880f,-28.0054f,16.5869f,  
	-9.9548f,-28.7705f,17.4752f,  -9.5400f,-27.4017f,18.8740f,  -9.3204f,-28.1250f,19.7611f,  -9.1863f,-26.7450f,21.2032f,  -8.9835f,-27.4278f,22.0866f,  
	-9.1334f,-26.0677f,23.5460f,  -8.9502f,-26.7122f,24.4234f,  -9.3816f,-25.4042f,25.8743f,  -9.2195f,-26.0161f,26.7437f,  -9.9233f,-24.7925f,28.1613f,  
	-9.7834f,-25.3757f,29.0205f,  -10.7446f,-24.2684f,30.3814f,  -10.6257f,-24.8305f,31.2291f,  -11.8228f,-23.8700f,32.5122f,  -11.7224f,-24.4184f,33.3490f,  
	-13.1287f,-23.6332f,34.5357f,  -13.0441f,-24.1739f,35.3617f,  -14.6272f,-23.5894f,36.4361f,  -14.5524f,-24.1312f,37.2545f,  -16.2757f,-23.7689f,38.2038f,  
	-16.2028f,-24.3199f,39.0184f,  -18.0256f,-24.1969f,39.8334f,  -17.9459f,-24.7635f,40.6488f,  -19.8248f,-24.8913f,41.3237f,  -19.7277f,-25.4797f,42.1472f,  
	-21.6175f,-25.8640f,42.6792f,  -21.4912f,-26.4784f,43.5187f,  -23.3461f,-27.1187f,43.9083f,  -23.1793f,-27.7613f,44.7726f,  -24.9538f,-28.6504f,45.0227f,  
	-24.7315f,-29.3218f,45.9229f,  -26.3823f,-30.4458f,46.0390f,  -26.0911f,-31.1433f,46.9856f,  -27.5779f,-32.4817f,46.9746f,  -27.2060f,-33.2003f,47.9776f,  
	-28.4927f,-34.7266f,47.8474f,  -28.0282f,-35.4589f,48.9160f,  -29.0839f,-37.1410f,48.6750f,  -28.5140f,-37.8757f,49.8196f,  -29.3145f,-39.6767f,49.4759f,  
	-28.6309f,-40.4016f,50.7022f,  -29.1588f,-42.2806f,50.2632f,  -28.3537f,-42.9802f,51.5757f,  -28.5997f,-44.8940f,51.0474f,  -27.6683f,-45.5521f,52.4477f,  
	-27.6312f,-47.4559f,51.8345f,  -26.5717f,-48.0544f,53.3198f,  -26.2588f,-49.9038f,52.6242f,  -25.0716f,-50.4255f,54.1893f,  -24.4983f,-52.1775f,53.4119f,  
	-23.1877f,-52.6058f,55.0463f,  -22.3772f,-54.2201f,54.1865f,  -20.9510f,-54.5380f,55.8774f,  -19.9341f,-55.9788f,54.9329f,  -18.4013f,-56.1743f,56.6645f,  
	-17.2146f,-57.4110f,55.6327f,  -15.5892f,-57.4732f,57.3856f,  -14.2749f,-58.4814f,56.2640f,  -12.5715f,-58.4037f,58.0182f,  -11.1752f,-59.1661f,56.8053f,  
	-9.4120f,-58.9453f,58.5399f,  -7.9820f,-59.4519f,57.2358f,  -6.1756f,-59.0894f,58.9310f,  -4.7606f,-59.3372f,57.5387f,  -2.9288f,-58.8377f,59.1758f,  
	-1.5765f,-58.8308f,57.7015f,  0.2639f,-58.2020f,59.2645f,  1.5082f,-57.9510f,57.7184f,  3.3402f,-57.2041f,59.1931f,  4.4356f,-56.7247f,57.5893f,  
	6.2486f,-55.8705f,58.9675f,  7.1594f,-55.1833f,57.3242f,  8.9437f,-54.2333f,58.6000f,  9.6405f,-53.3620f,56.9395f,  11.3860f,-52.3297f,58.1082f,  
	11.8468f,-51.3003f,56.4559f,  13.5513f,-50.1951f,57.5184f,  13.7617f,-49.0346f,55.9025f,  15.4229f,-47.8656f,56.8602f,  -25.3595f,-28.3183f,2.8333f,  
	-25.1489f,-29.3812f,3.6576f,  -23.1697f,-29.0944f,3.7057f,  -22.9430f,-30.1370f,4.5432f,  -21.0130f,-29.6750f,4.7960f,  -20.7729f,-30.6922f,5.6458f,  
	-18.9293f,-30.0598f,6.0967f,  -18.6778f,-31.0475f,6.9585f,  -16.9563f,-30.2532f,7.5972f,  -16.6962f,-31.2073f,8.4703f,  -15.1310f,-30.2633f,9.2832f,  
	-14.8658f,-31.1801f,10.1661f,  -13.4882f,-30.1020f,11.1367f,  -13.2220f,-30.9781f,12.0272f,  -12.0599f,-29.7853f,13.1359f,  -11.7965f,-30.6182f,14.0323f,  
	-10.8742f,-29.3334f,15.2574f,  -10.6177f,-30.1218f,16.1574f,  -9.9548f,-28.7705f,17.4752f,  -9.7098f,-29.5145f,18.3755f,  -9.3204f,-28.1250f,19.7611f,  
	-9.0906f,-28.8250f,20.6589f,  -8.9835f,-27.4278f,22.0866f,  -8.7722f,-28.0852f,22.9794f,  -8.9502f,-26.7122f,24.4234f,  -8.7599f,-27.3320f,25.3087f,  
	-9.2195f,-26.0161f,26.7437f,  -9.0520f,-26.6015f,27.6188f,  -9.7834f,-25.3757f,29.0205f,  -9.6387f,-25.9336f,29.8833f,  -10.6257f,-24.8305f,31.2291f,  
	-10.5022f,-25.3681f,32.0796f,  -11.7224f,-24.4184f,33.3490f,  -11.6186f,-24.9414f,34.1866f,  -13.0441f,-24.1739f,35.3617f,  -12.9548f,-24.6912f,36.1886f,  
	-14.5524f,-24.1312f,37.2545f,  -14.4708f,-24.6513f,38.0735f,  -16.2028f,-24.3199f,39.0184f,  -16.1211f,-24.8497f,39.8335f,  -17.9459f,-24.7635f,40.6488f,  
	-17.8542f,-25.3100f,41.4664f,  -19.7277f,-25.4797f,42.1472f,  -19.6148f,-26.0482f,42.9747f,  -21.4912f,-26.4784f,43.5187f,  -21.3460f,-27.0724f,44.3645f,  
	-23.1793f,-27.7613f,44.7726f,  -22.9866f,-28.3828f,45.6475f,  -24.7315f,-29.3218f,45.9229f,  -24.4778f,-29.9696f,46.8376f,  -26.0911f,-31.1433f,46.9856f,  
	-25.7640f,-31.8140f,47.9505f,  -27.2060f,-33.2003f,47.9776f,  -26.7928f,-33.8878f,49.0027f,  -28.0282f,-35.4589f,48.9160f,  -27.5146f,-36.1535f,50.0126f,  
	-28.5140f,-37.8757f,49.8196f,  -27.8905f,-38.5663f,50.9948f,  -28.6309f,-40.4016f,50.7022f,  -27.8878f,-41.0735f,51.9623f,  -28.3537f,-42.9802f,51.5757f,  
	-27.4843f,-43.6178f,52.9241f,  -27.6683f,-45.5521f,52.4477f,  -26.6688f,-46.1374f,53.8836f,  -26.5717f,-48.0544f,53.3198f,  -25.4406f,-48.5701f,54.8399f,  
	-25.0716f,-50.4255f,54.1893f,  -23.8112f,-50.8543f,55.7849f,  -23.1877f,-52.6058f,55.0463f,  -21.8035f,-52.9294f,56.7068f,  -20.9510f,-54.5380f,55.8774f,  
	-19.4506f,-54.7431f,57.5879f,  -18.4013f,-56.1743f,56.6645f,  -16.7970f,-56.2481f,58.4066f,  -15.5892f,-57.4732f,57.3856f,  -13.8947f,-57.4072f,59.1399f,  
	-12.5715f,-58.4037f,58.0182f,  -10.8040f,-58.1926f,59.7638f,  -9.4120f,-58.9453f,58.5399f,  -7.5881f,-58.5887f,60.2562f,  -6.1756f,-59.0894f,58.9310f,  
	-4.3133f,-58.5900f,60.5984f,  -2.9288f,-58.8377f,59.1758f,  -1.0457f,-58.2020f,60.7768f,  0.2639f,-58.2020f,59.2645f,  2.1493f,-57.4401f,60.7832f,  
	3.3402f,-57.2041f,59.1931f,  5.2160f,-56.3262f,60.6192f,  6.2486f,-55.8705f,58.9675f,  8.1033f,-54.8882f,60.2932f,  8.9437f,-54.2333f,58.6000f,  
	10.7649f,-53.1606f,59.8188f,  11.3860f,-52.3297f,58.1082f,  13.1695f,-51.1769f,59.2191f,  13.5513f,-50.1951f,57.5184f,  15.2927f,-48.9725f,58.5215f,  
	15.4229f,-47.8656f,56.8602f,  17.1216f,-46.5811f,57.7578f,  -25.1489f,-29.3812f,3.6576f,  -24.9193f,-30.4432f,4.4955f,  -22.9430f,-30.1370f,4.5432f,  
	-22.6977f,-31.1760f,5.3945f,  -20.7729f,-30.6922f,5.6458f,  -20.5137f,-31.7038f,6.5104f,  -18.6778f,-31.0475f,6.9585f,  -18.4071f,-32.0272f,7.8357f,  
	-16.6962f,-31.2073f,8.4703f,  -16.4176f,-32.1508f,9.3588f,  -14.8658f,-31.1801f,10.1661f,  -14.5831f,-32.0833f,11.0640f,  -13.2220f,-30.9781f,12.0272f,  
	-12.9390f,-31.8379f,12.9327f,  -11.7965f,-30.6182f,14.0323f,  -11.5176f,-31.4326f,14.9432f,  -10.6177f,-30.1218f,16.1574f,  -10.3478f,-30.8897f,17.0702f,  
	-9.7098f,-29.5145f,18.3755f,  -9.4529f,-30.2355f,19.2873f,  -9.0906f,-28.8250f,20.6589f,  -8.8506f,-29.4998f,21.5670f,  -8.7722f,-28.0852f,22.9794f,  
	-8.5524f,-28.7178f,23.8808f,  -8.7599f,-27.3320f,25.3087f,  -8.5629f,-27.9247f,26.2004f,  -9.0520f,-26.6015f,27.6188f,  -8.8789f,-27.1606f,28.4979f,  
	-9.6387f,-25.9336f,29.8833f,  -9.4889f,-26.4658f,30.7491f,  -10.5022f,-25.3681f,32.0796f,  -10.3750f,-25.8787f,32.9307f,  -11.6186f,-24.9414f,34.1866f,  
	-11.5102f,-25.4394f,35.0244f,  -12.9548f,-24.6912f,36.1886f,  -12.8594f,-25.1852f,37.0150f,  -14.4708f,-24.6513f,38.0735f,  -14.3817f,-25.1485f,38.8915f,  
	-16.1211f,-24.8497f,39.8335f,  -16.0291f,-25.3580f,40.6487f,  -17.8542f,-25.3100f,41.4664f,  -17.7488f,-25.8356f,42.2855f,  -19.6148f,-26.0482f,42.9747f,  
	-19.4853f,-26.5955f,43.8054f,  -21.3460f,-27.0724f,44.3645f,  -21.1775f,-27.6450f,45.2174f,  -22.9866f,-28.3828f,45.6475f,  -22.7655f,-28.9809f,46.5330f,  
	-24.4778f,-29.9696f,46.8376f,  -24.1913f,-30.5913f,47.7665f,  -25.7640f,-31.8140f,47.9505f,  -25.3987f,-32.4546f,48.9334f,  -26.7928f,-33.8878f,49.0027f,  
	-26.3334f,-34.5395f,50.0517f,  -27.5146f,-36.1535f,50.0126f,  -26.9503f,-36.8061f,51.1361f,  -27.8905f,-38.5663f,50.9948f,  -27.2099f,-39.2065f,52.2006f,  
	-27.8878f,-41.0735f,51.9623f,  -27.0826f,-41.6863f,53.2555f,  -27.4843f,-43.6178f,52.9241f,  -26.5486f,-44.1857f,54.3060f,  -26.6688f,-46.1374f,53.8836f,  
	-25.5990f,-46.6430f,55.3528f,  -25.4406f,-48.5701f,54.8399f,  -24.2370f,-48.9955f,56.3898f,  -23.8112f,-50.8543f,55.7849f,  -22.4770f,-51.1805f,57.4067f,  
	-21.8035f,-52.9294f,56.7068f,  -20.3447f,-53.1418f,58.3870f,  -19.4506f,-54.7431f,57.5879f,  -17.8776f,-54.8272f,59.3097f,  -16.7970f,-56.2481f,58.4066f,  
	-15.1222f,-56.1936f,60.1512f,  -13.8947f,-57.4072f,59.1399f,  -12.1341f,-57.2067f,60.8866f,  -10.8040f,-58.1926f,59.7638f,  -8.9739f,-57.8436f,61.4918f,  
	-7.5881f,-58.5887f,60.2562f,  -5.7065f,-58.0922f,61.9450f,  -4.3133f,-58.5900f,60.5984f,  -2.3982f,-57.9509f,62.2293f,  -1.0457f,-58.2020f,60.7768f,  
	0.8835f,-57.4289f,62.3322f,  2.1493f,-57.4401f,60.7832f,  4.0793f,-56.5427f,62.2511f,  5.2160f,-56.3262f,60.6192f,  7.1336f,-55.3157f,61.9900f,  
	8.1033f,-54.8882f,60.2932f,  9.9942f,-53.7788f,61.5583f,  10.7649f,-53.1606f,59.8188f,  12.6229f,-51.9632f,60.9755f,  13.1695f,-51.1769f,59.2191f,  
	14.9881f,-49.9030f,60.2660f,  15.2927f,-48.9725f,58.5215f,  17.0694f,-47.6314f,59.4597f,  17.1216f,-46.5811f,57.7578f,  18.8560f,-45.1797f,58.5900f,  
	-24.9193f,-30.4432f,4.4955f,  -24.6708f,-31.5030f,5.3470f,  -22.6977f,-31.1760f,5.3945f,  -22.4327f,-32.2109f,6.2606f,  -20.5137f,-31.7038f,6.5104f,  
	-20.2345f,-32.7089f,7.3904f,  -18.4071f,-32.0272f,7.8357f,  -18.1169f,-32.9977f,8.7285f,  -16.4176f,-32.1508f,9.3588f,  -16.1201f,-33.0821f,10.2627f,  
	-14.5831f,-32.0833f,11.0640f,  -14.2821f,-32.9716f,11.9773f,  -12.9390f,-31.8379f,12.9327f,  -12.6389f,-32.6803f,13.8533f,  -11.5176f,-31.4326f,14.9432f,  
	-11.2236f,-32.2271f,15.8676f,  -10.3478f,-30.8897f,17.0702f,  -10.0643f,-31.6352f,17.9954f,  -9.4529f,-30.2355f,19.2873f,  -9.1841f,-30.9317f,20.2104f,  
	-8.8506f,-29.4998f,21.5670f,  -8.6006f,-30.1496f,22.4846f,  -8.5524f,-28.7178f,23.8808f,  -8.3245f,-29.3228f,24.7895f,  -8.5629f,-27.9247f,26.2004f,  
	-8.3591f,-28.4904f,27.0967f,  -8.8789f,-27.1606f,28.4979f,  -8.6998f,-27.6930f,29.3804f,  -9.4889f,-26.4658f,30.7491f,  -9.3348f,-26.9698f,31.6156f,  
	-10.3750f,-25.8787f,32.9307f,  -10.2432f,-26.3629f,33.7818f,  -11.5102f,-25.4394f,35.0244f,  -11.3961f,-25.9125f,35.8610f,  -12.8594f,-25.1852f,37.0150f,  
	-12.7574f,-25.6547f,37.8392f,  -14.3817f,-25.1485f,38.8915f,  -14.2836f,-25.6228f,39.7079f,  -16.0291f,-25.3580f,40.6487f,  -15.9252f,-25.8441f,41.4630f,  
	-17.7488f,-25.8356f,42.2855f,  -17.6291f,-26.3387f,43.1050f,  -19.4853f,-26.5955f,43.8054f,  -19.3352f,-27.1208f,44.6400f,  -21.1775f,-27.6450f,45.2174f,  
	-20.9837f,-28.1940f,46.0772f,  -22.7655f,-28.9809f,46.5330f,  -22.5148f,-29.5533f,47.4289f,  -24.1913f,-30.5913f,47.7665f,  -23.8699f,-31.1839f,48.7095f,  
	-25.3987f,-32.4546f,48.9334f,  -24.9905f,-33.0612f,49.9362f,  -26.3334f,-34.5395f,50.0517f,  -25.8264f,-35.1512f,51.1236f,  -26.9503f,-36.8061f,51.1361f,  
	-26.3318f,-37.4111f,52.2866f,  -27.2099f,-39.2065f,52.2006f,  -26.4699f,-39.7905f,53.4362f,  -27.0826f,-41.6863f,53.2555f,  -26.2130f,-42.2326f,54.5796f,  
	-26.5486f,-44.1857f,54.3060f,  -25.5441f,-44.6770f,55.7191f,  -25.5990f,-46.6430f,55.3528f,  -24.4578f,-47.0613f,56.8505f,  -24.2370f,-48.9955f,56.3898f,  
	-22.9599f,-49.3212f,57.9653f,  -22.4770f,-51.1805f,57.4067f,  -21.0681f,-51.3978f,59.0483f,  -20.3447f,-53.1418f,58.3870f,  -18.8125f,-53.2348f,60.0792f,  
	-17.8776f,-54.8272f,59.3097f,  -16.2329f,-54.7837f,61.0347f,  -15.1222f,-56.1936f,60.1512f,  -13.3797f,-56.0042f,61.8891f,  -12.1341f,-57.2067f,60.8866f,  
	-10.3093f,-56.8669f,62.6164f,  -8.9739f,-57.8436f,61.4918f,  -7.0844f,-57.3526f,63.1926f,  -5.7065f,-58.0922f,61.9450f,  -3.7705f,-57.4529f,63.5971f,  
	-2.3982f,-57.9509f,62.2293f,  -0.4358f,-57.1705f,63.8138f,  0.8835f,-57.4289f,62.3322f,  2.8579f,-56.5162f,63.8353f,  4.0793f,-56.5427f,62.2511f,  
	6.0512f,-55.5084f,63.6613f,  7.1336f,-55.3157f,61.9900f,  9.0869f,-54.1738f,63.2970f,  9.9942f,-53.7788f,61.5583f,  11.9208f,-52.5407f,62.7576f,  
	12.6229f,-51.9632f,60.9755f,  14.5144f,-50.6412f,62.0640f,  14.9881f,-49.9030f,60.2660f,  16.8399f,-48.5074f,61.2435f,  17.0694f,-47.6314f,59.4597f,  
	18.8792f,-46.1707f,60.3276f,  18.8560f,-45.1797f,58.5900f,  20.6215f,-43.6625f,59.3502f,  -24.6708f,-31.5030f,5.3470f,  -24.4019f,-32.5600f,6.2132f,  
	-22.4327f,-32.2109f,6.2606f,  -22.1470f,-33.2406f,7.1419f,  -20.2345f,-32.7089f,7.3904f,  -19.9347f,-33.7062f,8.2861f,  -18.1169f,-32.9977f,8.7285f,  
	-17.8067f,-33.9574f,9.6371f,  -16.1201f,-33.0821f,10.2627f,  -15.8030f,-33.9999f,11.1826f,  -14.2821f,-32.9716f,11.9773f,  -13.9625f,-33.8437f,12.9064f,  
	-12.6389f,-32.6803f,13.8533f,  -12.3221f,-33.5039f,14.7881f,  -11.2236f,-32.2271f,15.8676f,  -10.9143f,-33.0002f,16.8051f,  -10.0643f,-31.6352f,17.9954f,  
	-9.7670f,-32.3565f,18.9327f,  -9.1841f,-30.9317f,20.2104f,  -8.9036f,-31.6031f,21.1438f,  -8.6006f,-30.1496f,22.4846f,  -8.3407f,-30.7718f,23.4103f,  
	-8.3245f,-29.3228f,24.7895f,  -8.0884f,-29.9003f,25.7035f,  -8.3591f,-28.4904f,27.0967f,  -8.1484f,-29.0286f,27.9968f,  -8.6998f,-27.6930f,29.3804f,  
	-8.5156f,-28.1961f,30.2639f,  -9.3348f,-26.9698f,31.6156f,  -9.1756f,-27.4461f,32.4822f,  -10.2432f,-26.3629f,33.7818f,  -10.1058f,-26.8209f,34.6313f,  
	-11.3961f,-25.9125f,35.8610f,  -11.2760f,-26.3597f,36.6944f,  -12.7574f,-25.6547f,37.8392f,  -12.6475f,-26.0999f,38.6601f,  -14.2836f,-25.6228f,39.7079f,  
	-14.1752f,-26.0735f,40.5212f,  -15.9252f,-25.8441f,41.4630f,  -15.8091f,-26.3065f,42.2752f,  -17.6291f,-26.3387f,43.1050f,  -17.4913f,-26.8194f,43.9252f,  
	-19.3352f,-27.1208f,44.6400f,  -19.1626f,-27.6224f,45.4778f,  -20.9837f,-28.1940f,46.0772f,  -20.7632f,-28.7176f,46.9433f,  -22.5148f,-29.5533f,47.4289f,  
	-22.2325f,-30.0974f,48.3347f,  -23.8699f,-31.1839f,48.7095f,  -23.5089f,-31.7440f,49.6680f,  -24.9905f,-33.0612f,49.9362f,  -24.5379f,-33.6299f,50.9576f,  
	-25.8264f,-35.1512f,51.1236f,  -25.2683f,-35.7180f,52.2187f,  -26.3318f,-37.4111f,52.2866f,  -25.6567f,-37.9629f,53.4632f,  -26.4699f,-39.7905f,53.4362f,  
	-25.6678f,-40.3115f,54.6999f,  -26.2130f,-42.2326f,54.5796f,  -25.2768f,-42.7060f,55.9326f,  -25.5441f,-44.6770f,55.7191f,  -24.4700f,-45.0848f,57.1591f,  
	-24.4578f,-47.0613f,56.8505f,  -23.2439f,-47.3834f,58.3730f,  -22.9599f,-49.3212f,57.9653f,  -21.6086f,-49.5410f,59.5603f,  -21.0681f,-51.3978f,59.0483f,  
	-19.5857f,-51.4980f,60.7021f,  -18.8125f,-53.2348f,60.0792f,  -17.2080f,-53.2017f,61.7753f,  -16.2329f,-54.7837f,61.0347f,  -14.5193f,-54.6060f,62.7541f,  
	-13.3797f,-56.0042f,61.8891f,  -11.5714f,-55.6754f,63.6110f,  -10.3093f,-56.8669f,62.6164f,  -8.4235f,-56.3839f,64.3198f,  -7.0844f,-57.3526f,63.1926f,  
	-5.1391f,-56.7166f,64.8569f,  -3.7705f,-57.4529f,63.5971f,  -1.7859f,-56.6694f,65.2024f,  -0.4358f,-57.1705f,63.8138f,  1.5727f,-56.2472f,65.3446f,  
	2.8579f,-56.5162f,63.8353f,  4.8747f,-55.4629f,65.2787f,  6.0512f,-55.5084f,63.6613f,  8.0587f,-54.3387f,65.0051f,  9.0869f,-54.1738f,63.2970f,  
	11.0752f,-52.8993f,64.5348f,  11.9208f,-52.5407f,62.7576f,  13.8796f,-51.1743f,63.8847f,  14.5144f,-50.6412f,62.0640f,  16.4371f,-49.1942f,63.0788f,  
	16.8399f,-48.5074f,61.2435f,  18.7224f,-46.9897f,62.1461f,  18.8792f,-46.1707f,60.3276f,  20.7174f,-44.5922f,61.1190f,  20.6215f,-43.6625f,59.3502f,  
	22.4171f,-42.0280f,60.0337f,  -24.4019f,-32.5600f,6.2132f,  -24.1118f,-33.6134f,7.0946f,  -22.1470f,-33.2406f,7.1419f,  -21.8399f,-34.2641f,8.0391f,  
	-19.9347f,-33.7062f,8.2861f,  -19.6140f,-34.6942f,9.1978f,  -17.8067f,-33.9574f,9.6371f,  -17.4756f,-34.9052f,10.5620f,  -15.8030f,-33.9999f,11.1826f,  
	-15.4658f,-34.9029f,12.1187f,  -13.9625f,-33.8437f,12.9064f,  -13.6246f,-34.6982f,13.8504f,  -12.3221f,-33.5039f,14.7881f,  -11.9883f,-34.3070f,15.7370f,  
	-10.9143f,-33.0002f,16.8051f,  -10.5893f,-33.7498f,17.7559f,  -9.7670f,-32.3565f,18.9327f,  -9.4562f,-33.0533f,19.8814f,  -8.9036f,-31.6031f,21.1438f,  
	-8.6114f,-32.2469f,22.0863f,  -8.3407f,-30.7718f,23.4103f,  -8.0713f,-31.3662f,24.3422f,  -8.0884f,-29.9003f,25.7035f,  -7.8441f,-30.4497f,26.6220f,  
	-8.1484f,-29.0286f,27.9968f,  -7.9314f,-29.5365f,28.8985f,  -8.5156f,-28.1961f,30.2639f,  -8.3257f,-28.6703f,31.1476f,  -9.1756f,-27.4461f,32.4822f,  
	-9.0108f,-27.8947f,33.3468f,  -10.1058f,-26.8209f,34.6313f,  -9.9628f,-27.2514f,35.4767f,  -11.2760f,-26.3597f,36.6944f,  -11.1489f,-26.7810f,37.5233f,  
	-12.6475f,-26.0999f,38.6601f,  -12.5287f,-26.5200f,39.4763f,  -14.1752f,-26.0735f,40.5212f,  -14.0564f,-26.4995f,41.3302f,  -15.8091f,-26.3065f,42.2752f,  
	-15.6771f,-26.7457f,43.0852f,  -17.4913f,-26.8194f,43.9252f,  -17.3336f,-27.2759f,44.7451f,  -19.1626f,-27.6224f,45.4778f,  -18.9665f,-28.0987f,46.3182f,  
	-20.7632f,-28.7176f,46.9433f,  -20.5145f,-29.2136f,47.8155f,  -22.2325f,-30.0974f,48.3347f,  -21.9139f,-30.6104f,49.2517f,  -23.5089f,-31.7440f,49.6680f,  
	-23.1071f,-32.2681f,50.6409f,  -24.5379f,-33.6299f,50.9576f,  -24.0378f,-34.1562f,51.9980f,  -25.2683f,-35.7180f,52.2187f,  -24.6569f,-36.2348f,53.3360f,  
	-25.6567f,-37.9629f,53.4632f,  -24.9224f,-38.4554f,54.6647f,  -25.6678f,-40.3115f,54.6999f,  -24.8018f,-40.7635f,55.9897f,  -25.2768f,-42.7060f,55.9326f,  
	-24.2732f,-43.0997f,57.3104f,  -24.4700f,-45.0848f,57.1591f,  -23.3243f,-45.4002f,58.6228f,  -23.2439f,-47.3834f,58.3730f,  -21.9568f,-47.6031f,59.9143f,  
	-21.6086f,-49.5410f,59.5603f,  -20.1839f,-49.6467f,61.1676f,  -19.5857f,-51.4980f,60.7021f,  -18.0306f,-51.4745f,62.3604f,  -17.2080f,-53.2017f,61.7753f,  
	-15.5335f,-53.0358f,63.4668f,  -14.5193f,-54.6060f,62.7541f,  -12.7388f,-54.2892f,64.4584f,  -11.5714f,-55.6754f,63.6110f,  -9.7009f,-55.2028f,65.3073f,  
	-8.4235f,-56.3839f,64.3198f,  -6.4805f,-55.7546f,65.9871f,  -5.1391f,-56.7166f,64.8569f,  -3.1437f,-55.9342f,66.4744f,  -1.7859f,-56.6694f,65.2024f,  
	0.2457f,-55.7403f,66.7533f,  1.5727f,-56.2472f,65.3446f,  3.6242f,-55.1802f,66.8144f,  4.8747f,-55.4629f,65.2787f,  6.9272f,-54.2708f,66.6534f,  
	8.0587f,-54.3387f,65.0051f,  10.1006f,-53.0331f,66.2767f,  11.0752f,-52.8993f,64.5348f,  13.0945f,-51.4934f,65.6969f,  13.8796f,-51.1743f,63.8847f,  
	15.8679f,-49.6801f,64.9340f,  16.4371f,-49.1942f,63.0788f,  18.3884f,-47.6227f,64.0142f,  18.7224f,-46.9897f,62.1461f,  20.6309f,-45.3521f,62.9675f,  
	20.7174f,-44.5922f,61.1190f,  22.5830f,-42.8948f,61.8290f,  22.4171f,-42.0280f,60.0337f,  24.2386f,-40.2775f,60.6347f,  -24.1118f,-33.6134f,7.0946f,  
	-23.7999f,-34.6620f,7.9918f,  -21.8399f,-34.2641f,8.0391f,  -21.5111f,-35.2800f,8.9525f,  -19.6140f,-34.6942f,9.1978f,  -19.2712f,-35.6720f,10.1262f,  
	-17.4756f,-34.9052f,10.5620f,  -17.1229f,-35.8397f,11.5036f,  -15.4658f,-34.9029f,12.1187f,  -15.1087f,-35.7896f,13.0704f,  -13.6246f,-34.6982f,13.8504f,  
	-13.2680f,-35.5333f,14.8093f,  -11.9883f,-34.3070f,15.7370f,  -11.6369f,-35.0876f,16.7001f,  -10.5893f,-33.7498f,17.7559f,  -10.2490f,-34.4754f,18.7190f,  
	-9.4562f,-33.0533f,19.8814f,  -9.1319f,-33.7230f,20.8403f,  -8.6114f,-32.2469f,22.0863f,  -8.3081f,-32.8629f,23.0361f,  -8.0713f,-31.3662f,24.3422f,  
	-7.7922f,-31.9320f,25.2795f,  -7.8441f,-30.4497f,26.6220f,  -7.5924f,-30.9681f,27.5429f,  -7.9314f,-29.5365f,28.8985f,  -7.7079f,-30.0146f,29.8010f,  
	-8.3257f,-28.6703f,31.1476f,  -8.1297f,-29.1157f,32.0295f,  -9.0108f,-27.8947f,33.3468f,  -8.8404f,-28.3145f,34.2070f,  -9.9628f,-27.2514f,35.4767f,  
	-9.8134f,-27.6546f,36.3169f,  -11.1489f,-26.7810f,37.5233f,  -11.0140f,-27.1760f,38.3460f,  -12.5287f,-26.5200f,39.4763f,  -12.4011f,-26.9142f,40.2861f,  
	-14.0564f,-26.4995f,41.3302f,  -13.9239f,-26.9012f,42.1344f,  -15.6771f,-26.7457f,43.0852f,  -15.5280f,-27.1604f,43.8917f,  -17.3336f,-27.2759f,44.7451f,  
	-17.1554f,-27.7070f,45.5641f,  -18.9665f,-28.0987f,46.3182f,  -18.7453f,-28.5478f,47.1607f,  -20.5145f,-29.2136f,47.8155f,  -20.2330f,-29.6798f,48.6945f,  
	-21.9139f,-30.6104f,49.2517f,  -21.5582f,-31.0892f,50.1788f,  -23.1071f,-32.2681f,50.6409f,  -22.6612f,-32.7524f,51.6285f,  -24.0378f,-34.1562f,51.9980f,  
	-23.4877f,-34.6357f,53.0568f,  -24.6569f,-36.2348f,53.3360f,  -23.9896f,-36.6959f,54.4747f,  -24.9224f,-38.4554f,54.6647f,  -24.1271f,-38.8828f,55.8892f,  
	-24.8018f,-40.7635f,55.9897f,  -23.8708f,-41.1401f,57.3018f,  -24.2732f,-43.0997f,57.3104f,  -23.1996f,-43.4055f,58.7104f,  -23.3243f,-45.4002f,58.6228f,  
	-22.1069f,-45.6172f,60.1043f,  -21.9568f,-47.6031f,59.9143f,  -20.5970f,-47.7124f,61.4679f,  -20.1839f,-49.6467f,61.1676f,  -18.6868f,-49.6318f,62.7799f,  
	-18.0306f,-51.4745f,62.3604f,  -16.4050f,-51.3205f,64.0150f,  -15.5335f,-53.0358f,63.4668f,  -13.7913f,-52.7323f,65.1442f,  -12.7388f,-54.2892f,64.4584f,  
	-10.8947f,-53.8293f,66.1382f,  -9.7009f,-55.2028f,65.3073f,  -7.7718f,-54.5834f,66.9684f,  -6.4805f,-55.7546f,65.9871f,  -4.4860f,-54.9775f,67.6080f,  
	-3.1437f,-55.9342f,66.4744f,  -1.1003f,-55.0042f,68.0372f,  0.2457f,-55.7403f,66.7533f,  2.3210f,-54.6649f,68.2421f,  3.6242f,-55.1802f,66.8144f,  
	5.7116f,-53.9712f,68.2138f,  6.9272f,-54.2708f,66.6534f,  9.0136f,-52.9396f,67.9536f,  10.1006f,-53.0331f,66.2767f,  12.1727f,-51.5929f,67.4696f,  
	13.0945f,-51.4934f,65.6969f,  15.1420f,-49.9568f,66.7776f,  15.8679f,-49.6801f,64.9340f,  17.8829f,-48.0589f,65.9000f,  18.3884f,-47.6227f,64.0142f,  
	20.3635f,-45.9292f,64.8643f,  20.6309f,-45.3521f,62.9675f,  22.5642f,-43.5940f,63.7033f,  22.5830f,-42.8948f,61.8290f,  24.4719f,-41.0800f,62.4522f,  
	24.2386f,-40.2775f,60.6347f,  26.0800f,-38.4148f,61.1471f,  -23.7999f,-34.6620f,7.9918f,  -23.4654f,-35.7046f,8.9052f,  -21.5111f,-35.2800f,8.9525f,  
	-21.1591f,-36.2872f,9.8827f,  -19.2712f,-35.6720f,10.1262f,  -18.9057f,-36.6381f,11.0717f,  -17.1229f,-35.8397f,11.5036f,  -16.7492f,-36.7593f,12.4614f,  
	-15.1087f,-35.7896f,13.0704f,  -14.7314f,-36.6582f,14.0378f,  -13.2680f,-35.5333f,14.8093f,  -12.8919f,-36.3472f,15.7833f,  -11.6369f,-35.0876f,16.7001f,  
	-11.2684f,-35.8451f,17.6765f,  -10.2490f,-34.4754f,18.7190f,  -9.8933f,-35.1746f,19.6935f,  -9.1319f,-33.7230f,20.8403f,  -8.7947f,-34.3649f,21.8076f,  
	-8.3081f,-32.8629f,23.0361f,  -7.9936f,-33.4501f,23.9924f,  -7.7922f,-31.9320f,25.2795f,  -7.5042f,-32.4665f,26.2203f,  -7.5924f,-30.9681f,27.5429f,  
	-7.3329f,-31.4560f,28.4654f,  -7.7079f,-30.0146f,29.8010f,  -7.4777f,-30.4628f,30.7021f,  -8.1297f,-29.1157f,32.0295f,  -7.9280f,-29.5309f,32.9071f,  
	-8.8404f,-28.3145f,34.2070f,  -8.6638f,-28.7057f,35.0616f,  -9.8134f,-27.6546f,36.3169f,  -9.6569f,-28.0301f,37.1499f,  -11.0140f,-27.1760f,38.3460f,  
	-10.8714f,-27.5436f,39.1609f,  -12.4011f,-26.9142f,40.2861f,  -12.2615f,-27.2832f,41.0890f,  -13.9239f,-26.9012f,42.1344f,  -13.7765f,-27.2778f,42.9322f,  
	-15.5280f,-27.1604f,43.8917f,  -15.3609f,-27.5493f,44.6940f,  -17.1554f,-27.7070f,45.5641f,  -16.9552f,-28.1112f,46.3816f,  -18.7453f,-28.5478f,47.1607f,  
	-18.4947f,-28.9681f,48.0060f,  -20.2330f,-29.6798f,48.6945f,  -19.9179f,-30.1134f,49.5794f,  -21.5582f,-31.0892f,50.1788f,  -21.1621f,-31.5307f,51.1164f,  
	-22.6612f,-32.7524f,51.6285f,  -22.1691f,-33.1930f,52.6303f,  -23.4877f,-34.6357f,53.0568f,  -22.8851f,-35.0633f,54.1331f,  -23.9896f,-36.6959f,54.4747f,  
	-23.2643f,-37.0962f,55.6330f,  -24.1271f,-38.8828f,55.8892f,  -23.2697f,-39.2395f,57.1335f,  -23.8708f,-41.1401f,57.3018f,  -22.8720f,-41.4334f,58.6344f,  
	-23.1996f,-43.4055f,58.7104f,  -22.0562f,-43.6175f,60.1270f,  -22.1069f,-45.6172f,60.1043f,  -20.8179f,-45.7284f,61.5978f,  -20.5970f,-47.7124f,61.4679f,  
	-19.1652f,-47.7051f,63.0267f,  -18.6868f,-49.6318f,62.7799f,  -17.1187f,-49.4895f,64.3893f,  -16.4050f,-51.3205f,64.0150f,  -14.7110f,-51.0313f,65.6567f,  
	-13.7913f,-52.7323f,65.1442f,  -11.9846f,-52.2870f,66.7984f,  -10.8947f,-53.8293f,66.1382f,  -8.9909f,-53.2229f,67.7839f,  -7.7718f,-54.5834f,66.9684f,  
	-5.7898f,-53.8156f,68.5838f,  -4.4860f,-54.9775f,67.6080f,  -2.4424f,-54.0514f,69.1744f,  -1.1003f,-55.0042f,68.0372f,  0.9875f,-53.9259f,69.5374f,  
	2.3210f,-54.6649f,68.2421f,  4.4328f,-53.4450f,69.6594f,  5.7116f,-53.9712f,68.2138f,  7.8329f,-52.6205f,69.5367f,  9.0136f,-52.9396f,67.9536f,  
	11.1298f,-51.4711f,69.1724f,  12.1727f,-51.5929f,67.4696f,  14.2718f,-50.0194f,68.5778f,  15.1420f,-49.9568f,66.7776f,  17.2145f,-48.2908f,67.7712f,  
	17.8829f,-48.0589f,65.9000f,  19.9197f,-46.3136f,66.7767f,  20.3635f,-45.9292f,64.8643f,  22.3609f,-44.1135f,65.6245f,  22.5642f,-43.5940f,63.7033f,  
	24.5183f,-41.7171f,64.3481f,  24.4719f,-41.0800f,62.4522f,  26.3780f,-39.1523f,62.9827f,  26.0800f,-38.4148f,61.1471f,  27.9375f,-36.4421f,61.5661f,  
	-23.4654f,-35.7046f,8.9052f,  -23.1070f,-36.7403f,9.8357f,  -21.1591f,-36.2872f,9.8827f,  -20.7835f,-37.2845f,10.8304f,  -18.9057f,-36.6381f,11.0717f,  
	-18.5178f,-37.5908f,12.0340f,  -16.7492f,-36.7593f,12.4614f,  -16.3535f,-37.6623f,13.4357f,  -14.7314f,-36.6582f,14.0378f,  -14.3328f,-37.5070f,15.0213f,  
	-12.8919f,-36.3472f,15.7833f,  -12.4970f,-37.1390f,16.7717f,  -11.2684f,-35.8451f,17.6765f,  -10.8826f,-36.5770f,18.6656f,  -9.8933f,-35.1746f,19.6935f,  
	-9.5230f,-35.8465f,20.6776f,  -8.7947f,-34.3649f,21.8076f,  -8.4448f,-34.9782f,22.7826f,  -7.9936f,-33.4501f,23.9924f,  -7.6684f,-34.0059f,24.9533f,  
	-7.5042f,-32.4665f,26.2203f,  -7.2071f,-32.9698f,27.1636f,  -7.3329f,-31.4560f,28.4654f,  -7.0658f,-31.9131f,29.3871f,  -7.4777f,-30.4628f,30.7021f,  
	-7.2410f,-30.8798f,31.5991f,  -7.9280f,-29.5309f,32.9071f,  -7.7199f,-29.9163f,33.7788f,  -8.6638f,-28.7057f,35.0616f,  -8.4806f,-29.0678f,35.9082f,  
	-9.6569f,-28.0301f,37.1499f,  -9.4936f,-28.3769f,37.9739f,  -10.8714f,-27.5436f,39.1609f,  -10.7185f,-27.8850f,39.9670f,  -12.2615f,-27.2832f,41.0890f,  
	-12.1092f,-27.6262f,41.8829f,  -13.7765f,-27.2778f,42.9322f,  -13.6137f,-27.6282f,43.7228f,  -15.3609f,-27.5493f,44.6940f,  -15.1748f,-27.9115f,45.4915f,  
	-16.9552f,-28.1112f,46.3816f,  -16.7289f,-28.4876f,47.1978f,  -18.4947f,-28.9681f,48.0060f,  -18.2142f,-29.3574f,48.8529f,  -19.9179f,-30.1134f,49.5794f,  
	-19.5662f,-30.5120f,50.4705f,  -21.1621f,-31.5307f,51.1164f,  -20.7236f,-31.9313f,52.0638f,  -22.1691f,-33.1930f,52.6303f,  -21.6280f,-33.5853f,53.6457f,  
	-22.8851f,-35.0633f,54.1331f,  -22.2282f,-35.4342f,55.2255f,  -23.2643f,-37.0962f,55.6330f,  -22.4803f,-37.4303f,56.8083f,  -23.2697f,-39.2395f,57.1335f,  
	-22.3471f,-39.5177f,58.3960f,  -22.8720f,-41.4334f,58.6344f,  -21.8057f,-41.6380f,59.9821f,  -22.0562f,-43.6175f,60.1270f,  -20.8427f,-43.7284f,61.5550f,  
	-20.8179f,-45.7284f,61.5978f,  -19.4578f,-45.7273f,63.0965f,  -19.1652f,-47.7051f,63.0267f,  -17.6626f,-47.5742f,64.5834f,  -17.1187f,-49.4895f,64.3893f,  
	-15.4821f,-49.2151f,65.9870f,  -14.7110f,-51.0313f,65.6567f,  -12.9517f,-50.6023f,67.2765f,  -11.9846f,-52.2870f,66.7984f,  -10.1169f,-51.6966f,68.4197f,  
	-8.9909f,-53.2229f,67.7839f,  -7.0328f,-52.4684f,69.3851f,  -5.7898f,-53.8156f,68.5838f,  -3.7576f,-52.8984f,70.1453f,  -2.4424f,-54.0514f,69.1744f,  
	-0.3535f,-52.9760f,70.6782f,  0.9875f,-53.9259f,69.5374f,  3.1126f,-52.7013f,70.9657f,  4.4328f,-53.4450f,69.6594f,  6.5787f,-52.0813f,70.9990f,  
	7.8329f,-52.6205f,69.5367f,  9.9838f,-51.1301f,70.7763f,  11.1298f,-51.4711f,69.1724f,  13.2723f,-49.8670f,70.3041f,  14.2718f,-50.0194f,68.5778f,  
	16.3947f,-48.3144f,69.5959f,  17.2145f,-48.2908f,67.7712f,  19.3071f,-46.4987f,68.6721f,  19.9197f,-46.3136f,66.7767f,  21.9767f,-44.4445f,67.5597f,  
	22.3609f,-44.1135f,65.6245f,  24.3770f,-42.1776f,66.2897f,  24.5183f,-41.7171f,64.3481f,  26.4869f,-39.7263f,64.8960f,  26.3780f,-39.1523f,62.9827f,  
	28.2972f,-37.1141f,63.4158f,  27.9375f,-36.4421f,61.5661f,  29.8060f,-34.3630f,61.8868f,  -23.1070f,-36.7403f,9.8357f,  -22.7241f,-37.7677f,10.7838f,  
	-20.7835f,-37.2845f,10.8304f,  -20.3843f,-38.2699f,11.7954f,  -18.5178f,-37.5908f,12.0340f,  -18.1066f,-38.5286f,13.0134f,  -16.3535f,-37.6623f,13.4357f,  
	-15.9351f,-38.5472f,14.4271f,  -14.3328f,-37.5070f,15.0213f,  -13.9138f,-38.3349f,16.0202f,  -12.4970f,-37.1390f,16.7717f,  -12.0828f,-37.9062f,17.7741f,  
	-10.8826f,-36.5770f,18.6656f,  -10.4803f,-37.2822f,19.6656f,  -9.5230f,-35.8465f,20.6776f,  -9.1380f,-36.4900f,21.6708f,  -8.4448f,-34.9782f,22.7826f,  
	-8.0822f,-35.5601f,23.7636f,  -7.6684f,-34.0059f,24.9533f,  -7.3326f,-34.5302f,25.9180f,  -7.2071f,-32.9698f,27.1636f,  -6.9011f,-33.4417f,28.1071f,  
	-7.0658f,-31.9131f,29.3871f,  -6.7914f,-32.3380f,30.3054f,  -7.2410f,-30.8798f,31.5991f,  -6.9977f,-31.2657f,32.4905f,  -7.7199f,-29.9163f,33.7788f,  
	-7.5053f,-30.2714f,34.6423f,  -8.4806f,-29.0678f,35.9082f,  -8.2911f,-29.4000f,36.7452f,  -9.4936f,-28.3769f,37.9739f,  -9.3212f,-28.6965f,38.7875f,  
	-10.7185f,-27.8850f,39.9670f,  -10.5546f,-28.1996f,40.7620f,  -12.1092f,-27.6262f,41.8829f,  -11.9438f,-27.9425f,42.6671f,  -13.6137f,-27.6282f,43.7228f,  
	-13.4346f,-27.9518f,44.5055f,  -15.1748f,-27.9115f,45.4915f,  -14.9658f,-28.2463f,46.2838f,  -16.7289f,-28.4876f,47.1978f,  -16.4762f,-28.8340f,48.0116f,  
	-18.2142f,-29.3574f,48.8529f,  -17.9007f,-29.7136f,49.7018f,  -19.5662f,-30.5120f,50.4705f,  -19.1759f,-30.8724f,51.3671f,  -20.7236f,-31.9313f,52.0638f,  
	-20.2398f,-32.2873f,53.0210f,  -21.6280f,-33.5853f,53.6457f,  -21.0364f,-33.9251f,54.6736f,  -22.2282f,-35.4342f,55.2255f,  -21.5159f,-35.7436f,56.3317f,  
	-22.4803f,-37.4303f,56.8083f,  -21.6338f,-37.6913f,57.9993f,  -22.3471f,-39.5177f,58.3960f,  -21.3595f,-39.7127f,59.6719f,  -21.8057f,-41.6380f,59.9821f,  
	-20.6711f,-41.7469f,61.3403f,  -20.8427f,-43.7284f,61.5550f,  -19.5594f,-43.7323f,62.9882f,  -19.4578f,-45.7273f,63.0965f,  -18.0274f,-45.6072f,64.5939f,  
	-17.6626f,-47.5742f,64.5834f,  -16.0915f,-47.3150f,66.1296f,  -15.4821f,-49.2151f,65.9870f,  -13.7797f,-48.8041f,67.5643f,  -12.9517f,-50.6023f,67.2765f,  
	-11.1307f,-50.0304f,68.8650f,  -10.1169f,-51.6966f,68.4197f,  -8.1938f,-50.9590f,69.9982f,  -7.0328f,-52.4684f,69.3851f,  -5.0234f,-51.5647f,70.9335f,  
	-3.7576f,-52.8984f,70.1453f,  -1.6790f,-51.8312f,71.6449f,  -0.3535f,-52.9760f,70.6782f,  1.7738f,-51.7528f,72.1101f,  3.1126f,-52.7013f,70.9657f,  
	5.2724f,-51.3312f,72.3154f,  6.5787f,-52.0813f,70.9990f,  8.7542f,-50.5758f,72.2538f,  9.9838f,-51.1301f,70.7763f,  12.1605f,-49.5021f,71.9267f,  
	13.2723f,-49.8670f,70.3041f,  15.4374f,-48.1294f,71.3428f,  16.3947f,-48.3144f,69.5959f,  18.5363f,-46.4813f,70.5181f,  19.3071f,-46.4987f,68.6721f,  
	21.4181f,-44.5813f,69.4757f,  21.9767f,-44.4445f,67.5597f,  24.0503f,-42.4539f,68.2440f,  24.3770f,-42.1776f,66.2897f,  26.4051f,-40.1270f,66.8543f,  
	26.4869f,-39.7263f,64.8960f,  28.4661f,-37.6245f,65.3427f,  28.2972f,-37.1141f,63.4158f,  30.2249f,-34.9692f,63.7467f,  29.8060f,-34.3630f,61.8868f,  
	31.6792f,-32.1833f,62.1045f,  -22.7241f,-37.7677f,10.7838f,  -22.3168f,-38.7849f,11.7496f,  -20.3843f,-38.2699f,11.7954f,  -19.9607f,-39.2421f,12.7781f,  
	-18.1066f,-38.5286f,13.0134f,  -17.6712f,-39.4498f,14.0107f,  -15.9351f,-38.5472f,14.4271f,  -15.4944f,-39.4123f,15.4347f,  -13.9138f,-38.3349f,16.0202f,  
	-13.4734f,-39.1395f,17.0343f,  -12.0828f,-37.9062f,17.7741f,  -11.6503f,-38.6477f,18.7888f,  -10.4803f,-37.2822f,19.6656f,  -10.0614f,-37.9596f,20.6760f,  
	-9.1380f,-36.4900f,21.6708f,  -8.7385f,-37.1025f,22.6715f,  -8.0822f,-35.5601f,23.7636f,  -7.7072f,-36.1104f,24.7498f,  -7.3326f,-34.5302f,25.9180f,  
	-6.9864f,-35.0225f,26.8841f,  -6.9011f,-33.4417f,28.1071f,  -6.5868f,-33.8806f,29.0482f,  -6.7914f,-32.3380f,30.3054f,  -6.5097f,-32.7308f,31.2187f,  
	-6.9977f,-31.2657f,32.4905f,  -6.7477f,-31.6201f,33.3739f,  -7.5053f,-30.2714f,34.6423f,  -7.2847f,-30.5952f,35.4959f,  -8.2911f,-29.4000f,36.7452f,  
	-8.0935f,-29.7037f,37.5707f,  -9.3212f,-28.6965f,38.7875f,  -9.1394f,-28.9882f,39.5884f,  -10.5546f,-28.1996f,40.7620f,  -10.3796f,-28.4867f,41.5450f,  
	-11.9438f,-27.9425f,42.6671f,  -11.7643f,-28.2317f,43.4407f,  -13.4346f,-27.9518f,44.5055f,  -13.2356f,-28.2486f,45.2796f,  -14.9658f,-28.2463f,46.2838f,  
	-14.7338f,-28.5524f,47.0701f,  -16.4762f,-28.8340f,48.0116f,  -16.1942f,-29.1492f,48.8234f,  -17.9007f,-29.7136f,49.7018f,  -17.5525f,-30.0342f,50.5521f,  
	-19.1759f,-30.8724f,51.3671f,  -18.7443f,-31.1916f,52.2695f,  -20.2398f,-32.2873f,53.0210f,  -19.7094f,-32.5949f,53.9867f,  -21.0364f,-33.9251f,54.6736f,  
	-20.3932f,-34.2081f,55.7120f,  -21.5159f,-35.7436f,56.3317f,  -20.7444f,-35.9852f,57.4512f,  -21.6338f,-37.6913f,57.9993f,  -20.7255f,-37.8744f,59.2019f,  
	-21.3595f,-39.7127f,59.6719f,  -20.3060f,-39.8176f,60.9574f,  -20.6711f,-41.7469f,61.3403f,  -19.4685f,-41.7542f,62.7035f,  -19.5594f,-43.7323f,62.9882f,  
	-18.2066f,-43.6223f,64.4207f,  -18.0274f,-45.6072f,64.5939f,  -16.5288f,-45.3634f,66.0819f,  -16.0915f,-47.3150f,66.1296f,  -14.4543f,-46.9230f,67.6569f,  
	-13.7797f,-48.8041f,67.5643f,  -12.0147f,-48.2529f,69.1121f,  -11.1307f,-50.0304f,68.8650f,  -9.2528f,-49.3131f,70.4126f,  -8.1938f,-50.9590f,69.9982f,  
	-6.2181f,-50.0732f,71.5254f,  -5.0234f,-51.5647f,70.9335f,  -2.9665f,-50.5112f,72.4210f,  -1.6790f,-51.8312f,71.6449f,  0.4391f,-50.6156f,73.0732f,  
	1.7738f,-51.7528f,72.1101f,  3.9364f,-50.3829f,73.4634f,  5.2724f,-51.3312f,72.3154f,  7.4621f,-49.8177f,73.5797f,  8.7542f,-50.5758f,72.2538f,  
	10.9554f,-48.9310f,73.4179f,  12.1605f,-49.5021f,71.9267f,  14.3591f,-47.7388f,72.9819f,  15.4374f,-48.1294f,71.3428f,  17.6204f,-46.2619f,72.2830f,  
	18.5363f,-46.4813f,70.5181f,  20.6947f,-44.5216f,71.3399f,  21.4181f,-44.5813f,69.4757f,  23.5439f,-42.5411f,70.1774f,  24.0503f,-42.4539f,68.2440f,  
	26.1337f,-40.3475f,68.8240f,  26.4051f,-40.1270f,66.8543f,  28.4413f,-37.9648f,67.3139f,  28.4661f,-37.6245f,65.3427f,  30.4512f,-35.4158f,65.6836f,  
	30.2249f,-34.9692f,63.7467f,  32.1548f,-32.7233f,63.9711f,  31.6792f,-32.1833f,62.1045f,  33.5509f,-29.9090f,62.2152f,  -22.3168f,-38.7849f,11.7496f,  
	-21.8840f,-39.7907f,12.7338f,  -19.9607f,-39.2421f,12.7781f,  -19.5114f,-40.1996f,13.7795f,  -17.6712f,-39.4498f,14.0107f,  -17.2119f,-40.3529f,15.0254f,  
	-15.4944f,-39.4123f,15.4347f,  -15.0307f,-40.2557f,16.4589f,  -13.4734f,-39.1395f,17.0343f,  -13.0129f,-39.9194f,18.0622f,  -11.6503f,-38.6477f,18.7888f,  
	-11.1993f,-39.3620f,19.8154f,  -10.0614f,-37.9596f,20.6760f,  -9.6260f,-38.6065f,21.6958f,  -8.7385f,-37.1025f,22.6715f,  -8.3246f,-37.6834f,23.6790f,  
	-7.7072f,-36.1104f,24.7498f,  -7.3204f,-36.6284f,25.7388f,  -6.9864f,-35.0225f,26.8841f,  -6.6306f,-35.4813f,27.8490f,  -6.5868f,-33.8806f,29.0482f,  
	-6.2641f,-34.2866f,29.9853f,  -6.5097f,-32.7308f,31.2187f,  -6.2207f,-33.0911f,32.1246f,  -6.7477f,-31.6201f,33.3739f,  -6.4915f,-31.9421f,34.2474f,  
	-7.2847f,-30.5952f,35.4959f,  -7.0565f,-30.8894f,36.3375f,  -8.0935f,-29.7037f,37.5707f,  -7.8876f,-29.9784f,38.3823f,  -9.1394f,-28.9882f,39.5884f,  
	-8.9479f,-29.2516f,40.3756f,  -10.3796f,-28.4867f,41.5450f,  -10.1926f,-28.7463f,42.3151f,  -11.7643f,-28.2317f,43.4407f,  -11.5678f,-28.4942f,44.2025f,  
	-13.2356f,-28.2486f,45.2796f,  -13.0166f,-28.5171f,46.0443f,  -14.7338f,-28.5524f,47.0701f,  -14.4758f,-28.8286f,47.8505f,  -16.1942f,-29.1492f,48.8234f,  
	-15.8813f,-29.4312f,49.6326f,  -17.5525f,-30.0342f,50.5521f,  -17.1669f,-30.3168f,51.4042f,  -18.7443f,-31.1916f,52.2695f,  -18.2699f,-31.4663f,53.1767f,  
	-19.7094f,-32.5949f,53.9867f,  -19.1313f,-32.8503f,54.9597f,  -20.3932f,-34.2081f,55.7120f,  -19.6941f,-34.4288f,56.7609f,  -20.7444f,-35.9852f,57.4512f,  
	-19.9143f,-36.1548f,58.5800f,  -20.7255f,-37.8744f,59.2019f,  -19.7536f,-37.9736f,60.4128f,  -20.3060f,-39.8176f,60.9574f,  -19.1864f,-39.8269f,62.2475f,  
	-19.4685f,-41.7542f,62.7035f,  -18.1974f,-41.6533f,64.0666f,  -18.2066f,-43.6223f,64.4207f,  -16.7863f,-43.3938f,65.8448f,  -16.5288f,-45.3634f,66.0819f,  
	-14.9640f,-44.9913f,67.5527f,  -14.4543f,-46.9230f,67.6569f,  -12.7540f,-46.3945f,69.1568f,  -12.0147f,-48.2529f,69.1121f,  -10.1918f,-47.5590f,70.6212f,  
	-9.2528f,-49.3131f,70.4126f,  -7.3213f,-48.4495f,71.9108f,  -6.2181f,-50.0732f,71.5254f,  -4.1938f,-49.0387f,72.9932f,  -2.9665f,-50.5112f,72.4210f,  
	-0.8686f,-49.3093f,73.8385f,  0.4391f,-50.6156f,73.0732f,  2.5934f,-49.2528f,74.4234f,  3.9364f,-50.3829f,73.4634f,  6.1296f,-48.8686f,74.7311f,  
	7.4621f,-49.8177f,73.5797f,  9.6777f,-48.1634f,74.7523f,  10.9554f,-48.9310f,73.4179f,  13.1783f,-47.1495f,74.4854f,  14.3591f,-47.7388f,72.9819f,  
	16.5751f,-45.8440f,73.9367f,  17.6204f,-46.2619f,72.2830f,  19.8189f,-44.2664f,73.1204f,  20.6947f,-44.5216f,71.3399f,  22.8664f,-42.4377f,72.0568f,  
	23.5439f,-42.5411f,70.1774f,  25.6776f,-40.3841f,70.7715f,  26.1337f,-40.3475f,68.8240f,  28.2231f,-38.1290f,69.2957f,  28.4413f,-37.9648f,67.3139f,  
	30.4810f,-35.6952f,67.6643f,  30.4512f,-35.4158f,65.6836f,  32.4362f,-33.1057f,65.9145f,  32.1548f,-32.7233f,63.9711f,  34.0807f,-30.3830f,64.0851f,  
	33.5509f,-29.9090f,62.2152f,  35.4138f,-27.5483f,62.2154f,  -21.8840f,-39.7907f,12.7338f,  -21.4243f,-40.7836f,13.7373f,  -19.5114f,-40.1996f,13.7795f,  
	-19.0370f,-41.1404f,14.7991f,  -17.2119f,-40.3529f,15.0254f,  -16.7279f,-41.2358f,16.0577f,  -15.0307f,-40.2557f,16.4589f,  -14.5450f,-41.0756f,17.4982f,  
	-13.0129f,-39.9194f,18.0622f,  -12.5321f,-40.6731f,19.1035f,  -11.1993f,-39.3620f,19.8154f,  -10.7296f,-40.0467f,20.8531f,  -9.6260f,-38.6065f,21.6958f,  
	-9.1743f,-39.2223f,22.7240f,  -8.3246f,-37.6834f,23.6790f,  -7.8971f,-38.2320f,24.6910f,  -7.3204f,-36.6284f,25.7388f,  -6.9222f,-37.1125f,26.7283f,  
	-6.6306f,-35.4813f,27.8490f,  -6.2652f,-35.9063f,28.8112f,  -6.2641f,-34.2866f,29.9853f,  -5.9334f,-34.6590f,30.9158f,  -6.2207f,-33.0911f,32.1246f,  
	-5.9250f,-33.4177f,33.0211f,  -6.4915f,-31.9421f,34.2474f,  -6.2279f,-32.2333f,35.1089f,  -7.0565f,-30.8894f,36.3375f,  -6.8208f,-31.1535f,37.1644f,  
	-7.8876f,-29.9784f,38.3823f,  -7.6733f,-30.2240f,39.1790f,  -8.9479f,-29.2516f,40.3756f,  -8.7462f,-29.4868f,41.1481f,  -10.1926f,-28.7463f,42.3151f,  
	-9.9909f,-28.9791f,43.0709f,  -11.5678f,-28.4942f,44.2025f,  -11.3541f,-28.7290f,44.9519f,  -13.0166f,-28.5171f,46.0443f,  -12.7751f,-28.7571f,46.7996f,  
	-14.4758f,-28.8286f,47.8505f,  -14.1908f,-29.0736f,48.6245f,  -15.8813f,-29.4312f,49.6326f,  -15.5347f,-29.6780f,50.4398f,  -17.1669f,-30.3168f,51.4042f,  
	-16.7425f,-30.5586f,52.2573f,  -18.2699f,-31.4663f,53.1767f,  -17.7518f,-31.6933f,54.0877f,  -19.1313f,-32.8503f,54.9597f,  -18.5008f,-33.0486f,55.9403f,  
	-19.6941f,-34.4288f,56.7609f,  -18.9398f,-34.5830f,57.8169f,  -19.9143f,-36.1548f,58.5800f,  -19.0234f,-36.2464f,59.7159f,  -19.7536f,-37.9736f,60.4128f,  
	-18.7181f,-37.9833f,61.6278f,  -19.1864f,-39.8269f,62.2475f,  -17.9999f,-39.7341f,63.5378f,  -18.1974f,-41.6533f,64.0666f,  -16.8598f,-41.4397f,65.4224f,  
	-16.7863f,-43.3938f,65.8448f,  -15.3001f,-43.0420f,67.2536f,  -14.9640f,-44.9913f,67.5527f,  -13.3359f,-44.4870f,68.9983f,  -12.7540f,-46.3945f,69.1568f,  
	-10.9947f,-45.7267f,70.6204f,  -10.1918f,-47.5590f,70.6212f,  -8.3141f,-46.7213f,72.0832f,  -7.3213f,-48.4495f,71.9108f,  -5.3399f,-47.4387f,73.3518f,  
	-4.1938f,-49.0387f,72.9932f,  -2.1270f,-47.8562f,74.3928f,  -0.8686f,-49.3093f,73.8385f,  1.2667f,-47.9602f,75.1789f,  2.5934f,-49.2528f,74.4234f,  
	4.7795f,-47.7449f,75.6885f,  6.1296f,-48.8686f,74.7311f,  8.3493f,-47.2126f,75.9069f,  9.6777f,-48.1634f,74.7523f,  11.9151f,-46.3714f,75.8274f,  
	13.1783f,-47.1495f,74.4854f,  15.4182f,-45.2350f,75.4509f,  16.5751f,-45.8440f,73.9367f,  18.8056f,-43.8201f,74.7865f,  19.8189f,-44.2664f,73.1204f,  
	22.0295f,-42.1456f,73.8503f,  22.8664f,-42.4377f,72.0568f,  25.0445f,-40.2362f,72.6635f,  25.6776f,-40.3841f,70.7715f,  27.8153f,-38.1143f,71.2542f,  
	28.2231f,-38.1290f,69.2957f,  30.3138f,-35.8026f,69.6549f,  30.4810f,-35.6952f,67.6643f,  32.5186f,-33.3238f,67.9014f,  32.4362f,-33.1057f,65.9145f,  
	34.4149f,-30.7012f,66.0317f,  34.0807f,-30.3830f,64.0851f,  35.9956f,-27.9564f,64.0854f,  35.4138f,-27.5483f,62.2154f,  37.2619f,-25.1084f,62.1023f,  
	-21.4243f,-40.7836f,13.7373f,  -20.9382f,-41.7616f,14.7598f,  -19.0370f,-41.1404f,14.7991f,  -18.5361f,-42.0629f,15.8375f,  -16.7279f,-41.2358f,16.0577f,  
	-16.2202f,-42.0966f,17.1066f,  -14.5450f,-41.0756f,17.4982f,  -14.0371f,-41.8705f,18.5527f,  -12.5321f,-40.6731f,19.1035f,  -12.0305f,-41.3982f,20.1578f,  
	-10.7296f,-40.0467f,20.8531f,  -10.2415f,-40.7007f,21.9010f,  -9.1743f,-39.2223f,22.7240f,  -8.7069f,-39.8058f,23.7585f,  -7.8971f,-38.2320f,24.6910f,  
	-7.4566f,-38.7466f,25.7053f,  -6.9222f,-37.1125f,26.7283f,  -6.5130f,-37.5624f,27.7166f,  -6.2652f,-35.9063f,28.8112f,  -5.8906f,-36.2971f,29.7682f,  
	-5.9334f,-34.6590f,30.9158f,  -5.5950f,-34.9968f,31.8380f,  -5.9250f,-33.4177f,33.0211f,  -5.6218f,-33.7124f,33.9060f,  -6.2279f,-32.2333f,35.1089f,  
	-5.9572f,-32.4931f,35.9555f,  -6.8208f,-31.1535f,37.1644f,  -6.5775f,-31.3873f,37.9756f,  -7.6733f,-30.2240f,39.1790f,  -7.4501f,-30.4405f,39.9596f,  
	-8.7462f,-29.4868f,41.1481f,  -8.5321f,-29.6949f,41.9040f,  -9.9909f,-28.9791f,43.0709f,  -9.7747f,-29.1843f,43.8115f,  -11.3541f,-28.7290f,44.9519f,  
	-11.1210f,-28.9361f,45.6888f,  -12.7751f,-28.7571f,46.7996f,  -12.5098f,-28.9675f,47.5450f,  -14.1908f,-29.0736f,48.6245f,  -13.8757f,-29.2860f,49.3929f,  
	-15.5347f,-29.6780f,50.4398f,  -15.1534f,-29.8873f,51.2444f,  -16.7425f,-30.5586f,52.2573f,  -16.2783f,-30.7568f,53.1110f,  -17.7518f,-31.6933f,54.0877f,  
	-17.1850f,-31.8683f,55.0035f,  -18.5008f,-33.0486f,55.9403f,  -17.8188f,-33.1862f,56.9257f,  -18.9398f,-34.5830f,57.8169f,  -18.1279f,-34.6656f,58.8785f,  
	-19.0234f,-36.2464f,59.7159f,  -18.0715f,-36.2550f,60.8551f,  -18.7181f,-37.9833f,61.6278f,  -17.6174f,-37.8974f,62.8433f,  -17.9999f,-39.7341f,63.5378f,  
	-16.7480f,-39.5348f,64.8219f,  -16.8598f,-41.4397f,65.4224f,  -15.4568f,-41.1085f,66.7645f,  -15.3001f,-43.0420f,67.2536f,  -13.7503f,-42.5631f,68.6393f,  
	-13.3359f,-44.4870f,68.9983f,  -11.6477f,-43.8474f,70.4103f,  -10.9947f,-45.7267f,70.6204f,  -9.1796f,-44.9182f,72.0397f,  -8.3141f,-46.7213f,72.0832f,  
	-6.3853f,-45.7387f,73.4905f,  -5.3399f,-47.4387f,73.3518f,  -3.3144f,-46.2812f,74.7268f,  -2.1270f,-47.8562f,74.3928f,  -0.0213f,-46.5272f,75.7171f,  
	1.2667f,-47.9602f,75.1789f,  3.4349f,-46.4658f,76.4354f,  4.7795f,-47.7449f,75.6885f,  6.9928f,-46.0947f,76.8621f,  8.3493f,-47.2126f,75.9069f,  
	10.5911f,-45.4178f,76.9849f,  11.9151f,-46.3714f,75.8274f,  14.1696f,-44.4451f,76.7997f,  15.4182f,-45.2350f,75.4509f,  17.6721f,-43.1903f,76.3099f,  
	18.8056f,-43.8201f,74.7865f,  21.0474f,-41.6697f,75.5269f,  22.0295f,-42.1456f,73.8503f,  24.2453f,-39.9062f,74.4675f,  25.0445f,-40.2362f,72.6635f,  
	27.2250f,-37.9211f,73.1558f,  27.8153f,-38.1143f,71.2542f,  29.9526f,-35.7362f,71.6214f,  30.3138f,-35.8026f,69.6549f,  32.4005f,-33.3740f,69.8978f,  
	32.5186f,-33.3238f,67.9014f,  34.5476f,-30.8581f,68.0217f,  34.4149f,-30.7012f,66.0317f,  36.3803f,-28.2103f,66.0321f,  35.9956f,-27.9564f,64.0854f,  
	37.8935f,-25.4506f,63.9694f,  37.2619f,-25.1084f,62.1023f,  39.0884f,-22.5980f,61.8739f,  -20.9382f,-41.7616f,14.7598f,  -20.4242f,-42.7230f,15.8022f,  
	-18.5361f,-42.0629f,15.8375f,  -18.0099f,-42.9649f,16.8940f,  -16.2202f,-42.0966f,17.1066f,  -15.6885f,-42.9337f,18.1722f,  -14.0371f,-41.8705f,18.5527f,  
	-13.5062f,-42.6379f,19.6218f,  -12.0305f,-41.3982f,20.1578f,  -11.5082f,-42.0934f,21.2242f,  -10.2415f,-40.7007f,21.9010f,  -9.7358f,-41.3229f,22.9574f,  
	-8.7069f,-39.8058f,23.7585f,  -8.2245f,-40.3553f,24.7974f,  -7.4566f,-38.7466f,25.7053f,  -7.0032f,-39.2265f,26.7203f,  -6.5130f,-37.5624f,27.7166f,  
	-6.0931f,-37.9773f,28.7014f,  -5.8906f,-36.2971f,29.7682f,  -5.5071f,-36.6522f,30.7182f,  -5.5950f,-34.9968f,31.8380f,  -5.2487f,-35.3014f,32.7493f,  
	-5.6218f,-33.7124f,33.9060f,  -5.3115f,-33.9745f,34.7762f,  -5.9572f,-32.4931f,35.9555f,  -5.6793f,-32.7216f,36.7861f,  -6.5775f,-31.3873f,37.9756f,  
	-6.3263f,-31.5911f,38.7700f,  -7.4501f,-30.4405f,39.9596f,  -7.2161f,-30.6293f,40.7220f,  -8.5321f,-29.6949f,41.9040f,  -8.3055f,-29.8752f,42.6427f,  
	-9.7747f,-29.1843f,43.8115f,  -9.5418f,-29.3624f,44.5370f,  -11.1210f,-28.9361f,45.6888f,  -10.8673f,-29.1149f,46.4125f,  -12.5098f,-28.9675f,47.5450f,  
	-12.2181f,-29.1476f,48.2814f,  -13.8757f,-29.2860f,49.3929f,  -13.5296f,-29.4640f,50.1551f,  -15.1534f,-29.8873f,51.2444f,  -14.7360f,-30.0570f,52.0462f,  
	-16.2783f,-30.7568f,53.1110f,  -15.7692f,-30.9081f,53.9664f,  -17.1850f,-31.8683f,55.0035f,  -16.5705f,-31.9881f,55.9216f,  -17.8188f,-33.1862f,56.9257f,  
	-17.0825f,-33.2582f,57.9150f,  -18.1279f,-34.6656f,58.8785f,  -17.2577f,-34.6716f,59.9426f,  -18.0715f,-36.2550f,60.8551f,  -17.0564f,-36.1747f,61.9949f,  
	-17.6174f,-37.8974f,62.8433f,  -16.4527f,-37.7114f,64.0535f,  -16.7480f,-39.5348f,64.8219f,  -15.4314f,-39.2239f,66.0940f,  -15.4568f,-41.1085f,66.7645f,  
	-13.9903f,-40.6556f,68.0861f,  -13.7503f,-42.5631f,68.6393f,  -12.1397f,-41.9535f,69.9945f,  -11.6477f,-43.8474f,70.4103f,  -9.9028f,-43.0710f,71.7810f,  
	-9.1796f,-44.9182f,72.0397f,  -7.3121f,-43.9678f,73.4070f,  -6.3853f,-45.7387f,73.4905f,  -4.4106f,-44.6112f,74.8346f,  -3.3144f,-46.2812f,74.7268f,  
	-1.2485f,-44.9782f,76.0286f,  -0.0213f,-46.5272f,75.7171f,  2.1187f,-45.0531f,76.9591f,  3.4349f,-46.4658f,76.4354f,  5.6315f,-44.8288f,77.6014f,  
	6.9928f,-46.0947f,76.8621f,  9.2290f,-44.3052f,77.9382f,  10.5911f,-45.4178f,76.9849f,  12.8506f,-43.4879f,77.9600f,  14.1696f,-44.4451f,76.7997f,  
	16.4380f,-42.3877f,77.6645f,  17.6721f,-43.1903f,76.3099f,  19.9369f,-41.0182f,77.0581f,  21.0474f,-41.6697f,75.5269f,  23.2935f,-39.3998f,76.1526f,  
	24.2453f,-39.9062f,74.4675f,  26.4621f,-37.5525f,74.9681f,  27.2250f,-37.9211f,73.1558f,  29.4033f,-35.4971f,73.5302f,  29.9526f,-35.7362f,71.6214f,  
	32.0841f,-33.2554f,71.8696f,  32.4005f,-33.3740f,69.8978f,  34.4766f,-30.8509f,70.0209f,  34.5476f,-30.8581f,68.0217f,  36.5615f,-28.3057f,68.0222f,  
	36.3803f,-28.2103f,66.0321f,  38.3268f,-25.6403f,65.9132f,  37.8935f,-25.4506f,63.9694f,  39.7677f,-22.8746f,63.7351f,  39.0884f,-22.5980f,61.8739f,  
	40.8853f,-20.0284f,61.5289f,  -20.4242f,-42.7230f,15.8022f,  -19.8834f,-43.6655f,16.8640f,  -18.0099f,-42.9649f,16.8940f,  -17.4580f,-43.8445f,17.9686f,  
	-15.6885f,-42.9337f,18.1722f,  -15.1317f,-43.7448f,19.2544f,  -13.5062f,-42.6379f,19.6218f,  -12.9526f,-43.3766f,20.7052f,  -11.5082f,-42.0934f,21.2242f,  
	-10.9663f,-42.7574f,22.3012f,  -9.7358f,-41.3229f,22.9574f,  -9.2129f,-41.9114f,24.0203f,  -8.2245f,-40.3553f,24.7974f,  -7.7274f,-40.8701f,25.8392f,  
	-7.0032f,-39.2265f,26.7203f,  -6.5375f,-39.6708f,27.7337f,  -6.0931f,-37.9773f,28.7014f,  -5.6629f,-38.3558f,29.6810f,  -5.5071f,-36.6522f,30.7182f,  
	-5.1149f,-36.9731f,31.6586f,  -5.2487f,-35.3014f,32.7493f,  -4.8949f,-35.5723f,33.6467f,  -5.3115f,-33.9745f,34.7762f,  -4.9941f,-34.2041f,35.6308f,  
	-5.6793f,-32.7216f,36.7861f,  -5.3940f,-32.9190f,37.5996f,  -6.3263f,-31.5911f,38.7700f,  -6.0656f,-31.7665f,39.5450f,  -7.2161f,-30.6293f,40.7220f,  
	-6.9716f,-30.7900f,41.4655f,  -8.3055f,-29.8752f,42.6427f,  -8.0646f,-30.0287f,43.3639f,  -9.5418f,-29.3624f,44.5370f,  -9.2913f,-29.5131f,45.2465f,  
	-10.8673f,-29.1149f,46.4125f,  -10.5905f,-29.2651f,47.1242f,  -12.2181f,-29.1476f,48.2814f,  -11.8991f,-29.2958f,49.0083f,  -13.5296f,-29.4640f,50.1551f,  
	-13.1513f,-29.6058f,50.9116f,  -14.7360f,-30.0570f,52.0462f,  -14.2776f,-30.1843f,52.8468f,  -15.7692f,-30.9081f,53.9664f,  -15.2161f,-31.0093f,54.8217f,  
	-16.5705f,-31.9881f,55.9216f,  -15.9049f,-32.0483f,56.8418f,  -17.0825f,-33.2582f,57.9150f,  -16.2907f,-33.2601f,58.9059f,  -17.2577f,-34.6716f,59.9426f,  
	-16.3267f,-34.5954f,61.0073f,  -17.0564f,-36.1747f,61.9949f,  -15.9792f,-36.0009f,63.1304f,  -16.4527f,-37.7114f,64.0535f,  -15.2244f,-37.4204f,65.2534f,  
	-15.4314f,-39.2239f,66.0940f,  -14.0516f,-38.7973f,67.3479f,  -13.9903f,-40.6556f,68.0861f,  -12.4623f,-40.0772f,69.3800f,  -12.1397f,-41.9535f,69.9945f,  
	-10.4715f,-41.2114f,71.3114f,  -9.9028f,-43.0710f,71.7810f,  -8.1043f,-42.1562f,73.1028f,  -7.3121f,-43.9678f,73.4070f,  -5.3968f,-42.8749f,74.7144f,  
	-4.4106f,-44.6112f,74.8346f,  -2.3937f,-43.3399f,76.1083f,  -1.2485f,-44.9782f,76.0286f,  0.8536f,-43.5311f,77.2506f,  2.1187f,-45.0531f,76.9591f,  
	4.2887f,-43.4368f,78.1123f,  5.6315f,-44.8288f,77.6014f,  7.8521f,-43.0525f,78.6710f,  9.2290f,-44.3052f,77.9382f,  11.4838f,-42.3798f,78.9118f,  
	12.8506f,-43.4879f,77.9600f,  15.1242f,-41.4262f,78.8271f,  16.4380f,-42.3877f,77.6645f,  18.7171f,-40.2022f,78.4176f,  19.9369f,-41.0182f,77.0581f,  
	22.2056f,-38.7257f,77.6902f,  23.2935f,-39.3998f,76.1526f,  25.5398f,-37.0149f,76.6600f,  26.4621f,-37.5525f,74.9681f,  28.6756f,-35.0894f,75.3485f,  
	29.4033f,-35.4971f,73.5302f,  31.5746f,-32.9700f,73.7830f,  32.0841f,-33.2554f,71.8696f,  34.2032f,-30.6798f,71.9953f,  34.4766f,-30.8509f,70.0209f,  
	36.5358f,-28.2410f,70.0215f,  36.5615f,-28.3057f,68.0222f,  38.5544f,-25.6742f,67.9005f,  38.3268f,-25.6403f,65.9132f,  40.2476f,-23.0002f,65.6732f,  
	39.7677f,-22.8746f,63.7351f,  41.6103f,-20.2396f,63.3814f,  40.8853f,-20.0284f,61.5289f,  42.6453f,-17.4104f,61.0669f,  -19.8834f,-43.6655f,16.8640f,  
	-19.3153f,-44.5873f,17.9455f,  -17.4580f,-43.8445f,17.9686f,  -16.8790f,-44.6997f,19.0618f,  -15.1317f,-43.7448f,19.2544f,  -14.5500f,-44.5282f,20.3528f,  
	-12.9526f,-43.3766f,20.7052f,  -12.3772f,-44.0846f,21.8013f,  -10.9663f,-42.7574f,22.3012f,  -10.4049f,-43.3880f,23.3872f,  -9.2129f,-41.9114f,24.0203f,  
	-8.6733f,-42.4650f,25.0884f,  -7.7274f,-40.8701f,25.8392f,  -7.2161f,-41.3488f,26.8816f,  -6.5375f,-39.6708f,27.7337f,  -6.0596f,-40.0783f,28.7441f,  
	-5.6629f,-38.3558f,29.6810f,  -5.2227f,-38.6990f,30.6527f,  -5.1149f,-36.9731f,31.6586f,  -4.7144f,-37.2592f,32.5865f,  -4.8949f,-35.5723f,33.6467f,  
	-4.5335f,-35.8094f,34.5294f,  -4.9941f,-34.2041f,35.6308f,  -4.6693f,-34.4013f,36.4684f,  -5.3940f,-32.9190f,37.5996f,  -5.1003f,-33.0868f,38.3931f,  
	-6.0656f,-31.7665f,39.5450f,  -5.7958f,-31.9130f,40.2999f,  -6.9716f,-30.7900f,41.4655f,  -6.7147f,-30.9237f,42.1896f,  -8.0646f,-30.0287f,43.3639f,  
	-7.8087f,-30.1553f,44.0668f,  -9.2913f,-29.5131f,45.2465f,  -9.0206f,-29.6365f,45.9412f,  -10.5905f,-29.2651f,47.1242f,  -10.2897f,-29.3857f,47.8234f,  
	-11.8991f,-29.2958f,49.0083f,  -11.5514f,-29.4110f,49.7265f,  -13.1513f,-29.6058f,50.9116f,  -12.7358f,-29.7095f,51.6639f,  -14.2776f,-30.1843f,52.8468f,  
	-13.7789f,-30.2665f,53.6449f,  -15.2161f,-31.0093f,54.8217f,  -14.6154f,-31.0566f,55.6775f,  -15.9049f,-32.0483f,56.8418f,  -15.1870f,-32.0447f,57.7628f,  
	-16.2907f,-33.2601f,58.9059f,  -15.4406f,-33.1865f,59.8973f,  -16.3267f,-34.5954f,61.0073f,  -15.3353f,-34.4326f,62.0684f,  -15.9792f,-36.0009f,63.1304f,  
	-14.8394f,-35.7288f,64.2572f,  -15.2244f,-37.4204f,65.2534f,  -13.9334f,-37.0198f,66.4376f,  -14.0516f,-38.7973f,67.3479f,  -12.6099f,-38.2506f,68.5774f,  
	-12.4623f,-40.0772f,69.3800f,  -10.8759f,-39.3711f,70.6391f,  -10.4715f,-41.2114f,71.3114f,  -8.7485f,-40.3349f,72.5830f,  -8.1043f,-42.1562f,73.1028f,  
	-6.2561f,-41.1018f,74.3683f,  -5.3968f,-42.8749f,74.7144f,  -3.4376f,-41.6402f,75.9546f,  -2.3937f,-43.3399f,76.1083f,  -0.3389f,-41.9257f,77.3049f,  
	0.8536f,-43.5311f,77.2506f,  2.9873f,-41.9423f,78.3860f,  4.2887f,-43.4368f,78.1123f,  6.4839f,-41.6812f,79.1707f,  7.8521f,-43.0525f,78.6710f,  
	10.0926f,-41.1397f,79.6391f,  11.4838f,-42.3798f,78.9118f,  13.7534f,-40.3221f,79.7778f,  15.1242f,-41.4262f,78.8271f,  17.4088f,-39.2357f,79.5822f,  
	18.7171f,-40.2022f,78.4176f,  21.0000f,-37.8954f,79.0538f,  22.2056f,-38.7257f,77.6902f,  24.4738f,-36.3176f,78.2025f,  25.5398f,-37.0149f,76.6600f,  
	27.7817f,-34.5201f,77.0453f,  28.6756f,-35.0894f,75.3485f,  30.8808f,-32.5226f,75.6053f,  31.5746f,-32.9700f,73.7830f,  33.7320f,-30.3478f,73.9110f,  
	34.2032f,-30.6798f,71.9953f,  36.3039f,-28.0172f,71.9959f,  36.5358f,-28.2410f,70.0215f,  38.5724f,-25.5517f,69.8972f,  38.5544f,-25.6742f,67.9005f,  
	40.5198f,-22.9727f,67.6549f,  40.2476f,-23.0002f,65.6732f,  42.1348f,-20.3013f,65.3110f,  41.6103f,-20.2396f,63.3814f,  43.4142f,-17.5564f,62.9080f,  
	42.6453f,-17.4104f,61.0669f,  44.3616f,-14.7552f,60.4884f,  -19.3153f,-44.5873f,17.9455f,  -18.7182f,-45.4864f,19.0472f,  -16.8790f,-44.6997f,19.0618f,  
	-16.2732f,-45.5285f,20.1731f,  -14.5500f,-44.5282f,20.3528f,  -13.9444f,-45.2819f,21.4662f,  -12.3772f,-44.0846f,21.8013f,  -11.7801f,-44.7600f,22.9089f,  
	-10.4049f,-43.3880f,23.3872f,  -9.8246f,-43.9840f,24.4808f,  -8.6733f,-42.4650f,25.0884f,  -8.1172f,-42.9825f,26.1598f,  -7.2161f,-41.3488f,26.8816f,  
	-6.6905f,-41.7902f,27.9235f,  -6.0596f,-40.0783f,28.7441f,  -5.5703f,-40.4494f,29.7486f,  -5.2227f,-38.6990f,30.6527f,  -4.7732f,-39.0062f,31.6135f,  
	-4.7144f,-37.2592f,32.5865f,  -4.3056f,-37.5101f,33.5008f,  -4.5335f,-35.8094f,34.5294f,  -4.1645f,-36.0128f,35.3958f,  -4.6693f,-34.4013f,36.4684f,  
	-4.3366f,-34.5678f,37.2861f,  -5.1003f,-33.0868f,38.3931f,  -4.7982f,-33.2250f,39.1660f,  -5.7958f,-31.9130f,40.2999f,  -5.5152f,-32.0321f,41.0343f,  
	-6.7147f,-30.9237f,42.1896f,  -6.4449f,-31.0306f,42.8937f,  -7.8087f,-30.1553f,44.0668f,  -7.5352f,-30.2556f,44.7527f,  -9.0206f,-29.6365f,45.9412f,  
	-8.7290f,-29.7321f,46.6208f,  -10.2897f,-29.3857f,47.8234f,  -9.9636f,-29.4757f,48.5113f,  -11.5514f,-29.4110f,49.7265f,  -11.1700f,-29.4916f,50.4379f,  
	-12.7358f,-29.7095f,51.6639f,  -12.2834f,-29.7724f,52.4114f,  -13.7789f,-30.2665f,53.6449f,  -13.2360f,-30.3001f,54.4417f,  -14.6154f,-31.0566f,55.6775f,  
	-13.9655f,-31.0462f,56.5328f,  -15.1870f,-32.0447f,57.7628f,  -14.4133f,-31.9723f,58.6839f,  -15.4406f,-33.1865f,59.8973f,  -14.5322f,-33.0332f,60.8857f,  
	-15.3353f,-34.4326f,62.0684f,  -14.2828f,-34.1781f,63.1224f,  -14.8394f,-35.7288f,64.2572f,  -13.6375f,-35.3537f,65.3706f,  -13.9334f,-37.0198f,66.4376f,  
	-12.5800f,-36.5052f,67.6005f,  -12.6099f,-38.2506f,68.5774f,  -11.1091f,-37.5817f,69.7755f,  -10.8759f,-39.3711f,70.6391f,  -9.2337f,-38.5352f,71.8565f,  
	-8.7485f,-40.3349f,72.5830f,  -6.9739f,-39.3223f,73.8022f,  -6.2561f,-41.1018f,74.3683f,  -4.3621f,-39.9083f,75.5703f,  -3.4376f,-41.6402f,75.9546f,  
	-1.4383f,-40.2645f,77.1211f,  -0.3389f,-41.9257f,77.3049f,  1.7496f,-40.3709f,78.4178f,  2.9873f,-41.9423f,78.3860f,  5.1479f,-40.2147f,79.4288f,  
	6.4839f,-41.6812f,79.1707f,  8.7008f,-39.7889f,80.1292f,  10.0926f,-41.1397f,79.6391f,  12.3488f,-39.0944f,80.5004f,  13.7534f,-40.3221f,79.7778f,  
	16.0344f,-38.1352f,80.5320f,  17.4088f,-39.2357f,79.5822f,  19.6975f,-36.9231f,80.2201f,  21.0000f,-37.8954f,79.0538f,  23.2823f,-35.4724f,79.5693f,  
	24.4738f,-36.3176f,78.2025f,  26.7371f,-33.7990f,78.5915f,  27.7817f,-34.5201f,77.0453f,  30.0147f,-31.9210f,77.3052f,  30.8808f,-32.5226f,75.6053f,  
	33.0710f,-29.8605f,75.7352f,  33.7320f,-30.3478f,73.9110f,  35.8696f,-27.6381f,73.9116f,  36.3039f,-28.0172f,71.9959f,  38.3805f,-25.2750f,71.8692f,  
	38.5724f,-25.5517f,69.8972f,  40.5797f,-22.7925f,69.6464f,  40.5198f,-22.9727f,67.6549f,  42.4499f,-20.2125f,67.2845f,  42.1348f,-20.3013f,65.3110f,  
	43.9816f,-17.5543f,64.8264f,  43.4142f,-17.5564f,62.9080f,  45.1726f,-14.8360f,62.3154f,  44.3616f,-14.7552f,60.4884f,  46.0274f,-12.0746f,59.7947f,  
	-18.7182f,-45.4864f,19.0472f,  -18.0924f,-46.3605f,20.1691f,  -16.2732f,-45.5285f,20.1731f,  -15.6412f,-46.3288f,21.3016f,  -13.9444f,-45.2819f,21.4662f,  
	-13.3149f,-46.0038f,22.5936f,  -11.7801f,-44.7600f,22.9089f,  -11.1617f,-45.4011f,24.0268f,  -9.8246f,-43.9840f,24.4808f,  -9.2256f,-44.5439f,25.5805f,  
	-8.1172f,-42.9825f,26.1598f,  -7.5447f,-43.4623f,27.2333f,  -6.6905f,-41.7902f,27.9235f,  -6.1517f,-42.1945f,28.9619f,  -5.5703f,-40.4494f,29.7486f,  
	-5.0702f,-40.7834f,30.7445f,  -4.7732f,-39.0062f,31.6135f,  -4.3141f,-39.2769f,32.5626f,  -4.3056f,-37.5101f,33.5008f,  -3.8883f,-37.7259f,34.4002f,  
	-4.1645f,-36.0128f,35.3958f,  -3.7876f,-36.1841f,36.2429f,  -4.3366f,-34.5678f,37.2861f,  -3.9959f,-34.7034f,38.0832f,  -4.7982f,-33.2250f,39.1660f,  
	-4.4865f,-33.3348f,39.9177f,  -5.5152f,-32.0321f,41.0343f,  -5.2233f,-32.1240f,41.7473f,  -6.4449f,-31.0306f,42.8937f,  -6.1599f,-31.1116f,43.5788f,  
	-7.5352f,-30.2556f,44.7527f,  -7.2436f,-30.3290f,45.4211f,  -8.7290f,-29.7321f,46.6208f,  -8.4150f,-29.7991f,47.2868f,  -9.9636f,-29.4757f,48.5113f,  
	-9.6072f,-29.5344f,49.1898f,  -11.1700f,-29.4916f,50.4379f,  -10.7553f,-29.5354f,51.1423f,  -12.2834f,-29.7724f,52.4114f,  -11.7902f,-29.7917f,53.1558f,  
	-13.2360f,-30.3001f,54.4417f,  -12.6470f,-30.2817f,55.2369f,  -13.9655f,-31.0462f,56.5328f,  -13.2625f,-30.9733f,57.3878f,  -14.4133f,-31.9723f,58.6839f,  
	-13.5836f,-31.8267f,59.6025f,  -14.5322f,-33.0332f,60.8857f,  -13.5643f,-32.7949f,61.8684f,  -14.2828f,-34.1781f,63.1224f,  -13.1690f,-33.8273f,64.1652f,  
	-13.6375f,-35.3537f,65.3706f,  -12.3732f,-34.8706f,66.4658f,  -12.5800f,-36.5052f,67.6005f,  -11.1671f,-35.8738f,68.7355f,  -11.1091f,-37.5817f,69.7755f,  
	-9.5514f,-36.7878f,70.9359f,  -9.2337f,-38.5352f,71.8565f,  -7.5379f,-37.5670f,73.0258f,  -6.9739f,-39.3223f,73.8022f,  -5.1515f,-38.1737f,74.9619f,  
	-4.3621f,-39.9083f,75.5703f,  -2.4258f,-38.5759f,76.7023f,  -1.4383f,-40.2645f,77.1211f,  0.5967f,-38.7495f,78.2072f,  1.7496f,-40.3709f,78.4178f,  
	3.8670f,-38.6781f,79.4408f,  5.1479f,-40.2147f,79.4288f,  7.3322f,-38.3504f,80.3739f,  8.7008f,-39.7889f,80.1292f,  10.9346f,-37.7641f,80.9824f,  
	12.3488f,-39.0944f,80.5004f,  14.6174f,-36.9196f,81.2507f,  16.0344f,-38.1352f,80.5320f,  18.3204f,-35.8255f,81.1692f,  19.6975f,-36.9231f,80.2201f,  
	21.9858f,-34.4937f,80.7370f,  23.2823f,-35.4724f,79.5693f,  25.5595f,-32.9383f,79.9607f,  26.7371f,-33.7990f,78.5915f,  28.9909f,-31.1755f,78.8538f,  
	30.0147f,-31.9210f,77.3052f,  32.2316f,-29.2262f,77.4367f,  33.0710f,-29.8605f,75.7352f,  35.2405f,-27.1104f,75.7358f,  35.8696f,-27.6381f,73.9116f,  
	37.9819f,-24.8487f,73.7827f,  38.3805f,-25.2750f,71.8692f,  40.4262f,-22.4628f,71.6136f,  40.5797f,-22.7925f,69.6464f,  42.5500f,-19.9746f,69.2683f,  
	42.4499f,-20.2125f,67.2845f,  44.3378f,-17.4041f,66.7892f,  43.9816f,-17.5543f,64.8264f,  45.7813f,-14.7701f,64.2199f,  45.1726f,-14.8360f,62.3154f,  
	46.8788f,-12.0903f,61.6049f,  46.0274f,-12.0746f,59.7947f,  47.6356f,-9.3814f,58.9882f,  -18.0924f,-46.3605f,20.1691f,  -17.4384f,-47.2073f,21.3104f,  
	-15.6412f,-46.3288f,21.3016f,  -14.9830f,-47.0982f,22.4468f,  -13.3149f,-46.0038f,22.5936f,  -12.6616f,-46.6921f,23.7342f,  -11.1617f,-45.4011f,24.0268f,  
	-10.5222f,-46.0064f,25.1537f,  -9.2256f,-44.5439f,25.5805f,  -8.6077f,-45.0660f,26.6852f,  -7.5447f,-43.4623f,27.2333f,  -6.9570f,-43.9043f,28.3061f,  
	-6.1517f,-42.1945f,28.9619f,  -5.6003f,-42.5606f,29.9943f,  -5.0702f,-40.7834f,30.7445f,  -4.5589f,-41.0798f,31.7309f,  -4.3141f,-39.2769f,32.5626f,  
	-3.8453f,-39.5112f,33.4985f,  -3.8883f,-37.7259f,34.4002f,  -3.4627f,-37.9081f,35.2813f,  -3.7876f,-36.1841f,36.2429f,  -3.4027f,-36.3231f,37.0699f,  
	-3.9959f,-34.7034f,38.0832f,  -3.6463f,-34.8096f,38.8588f,  -4.4865f,-33.3348f,39.9177f,  -4.1648f,-33.4167f,40.6471f,  -5.2233f,-32.1240f,41.7473f,  
	-4.9181f,-32.1899f,42.4399f,  -6.1599f,-31.1116f,43.5788f,  -5.8590f,-31.1664f,44.2447f,  -7.2436f,-30.3290f,45.4211f,  -6.9322f,-30.3754f,46.0738f,  
	-8.4150f,-29.7991f,47.2868f,  -8.0740f,-29.8373f,47.9409f,  -9.6072f,-29.5344f,49.1898f,  -9.2207f,-29.5597f,49.8591f,  -10.7553f,-29.5354f,51.1423f,  
	-10.3030f,-29.5402f,51.8417f,  -11.7902f,-29.7917f,53.1558f,  -11.2542f,-29.7643f,53.8974f,  -12.6470f,-30.2817f,55.2369f,  -12.0077f,-30.2068f,56.0313f,  
	-13.2625f,-30.9733f,57.3878f,  -12.5058f,-30.8337f,58.2408f,  -13.5836f,-31.8267f,59.6025f,  -12.6960f,-31.6028f,60.5167f,  -13.5643f,-32.7949f,61.8684f,  
	-12.5360f,-32.4669f,62.8420f,  -13.1690f,-33.8273f,64.1652f,  -11.9929f,-33.3748f,65.1929f,  -12.3732f,-34.8706f,66.4658f,  -11.0489f,-34.2765f,67.5368f,  
	-11.1671f,-35.8738f,68.7355f,  -9.6964f,-35.1226f,69.8367f,  -9.5514f,-36.7878f,70.9359f,  -7.9383f,-35.8657f,72.0524f,  -7.5379f,-37.5670f,73.0258f,  
	-5.7923f,-36.4662f,74.1399f,  -5.1515f,-38.1737f,74.9619f,  -3.2846f,-36.8887f,76.0558f,  -2.4258f,-38.5759f,76.7023f,  -0.4515f,-37.1060f,77.7577f,  
	0.5967f,-38.7495f,78.2072f,  2.6630f,-37.0977f,79.2066f,  3.8670f,-38.6781f,79.4408f,  6.0102f,-36.8490f,80.3689f,  7.3322f,-38.3504f,80.3739f,  
	9.5350f,-36.3539f,81.2157f,  10.9346f,-37.7641f,80.9824f,  13.1821f,-35.6096f,81.7259f,  14.6174f,-36.9196f,81.2507f,  16.8921f,-34.6214f,81.8849f,  
	18.3204f,-35.8255f,81.1692f,  20.6065f,-33.3985f,81.6856f,  21.9858f,-34.4937f,80.7370f,  24.2691f,-31.9528f,81.1294f,  25.5595f,-32.9383f,79.9607f,  
	27.8270f,-30.2988f,80.2246f,  28.9909f,-31.1755f,78.8538f,  31.2282f,-28.4558f,78.9865f,  32.2316f,-29.2262f,77.4367f,  34.4272f,-26.4429f,77.4373f,  
	35.2405f,-27.1104f,75.7358f,  37.3836f,-24.2802f,75.6050f,  37.9819f,-24.8487f,73.7827f,  40.0620f,-21.9892f,73.5228f,  40.4262f,-22.4628f,71.6136f,  
	42.4336f,-19.5918f,71.2284f,  42.5500f,-19.9746f,69.2683f,  44.4768f,-17.1085f,68.7629f,  44.3378f,-17.4041f,66.7892f,  46.1772f,-14.5585f,66.1694f,  
	45.7813f,-14.7701f,64.2199f,  47.5271f,-11.9605f,63.4930f,  46.8788f,-12.0903f,61.6049f,  48.5258f,-9.3321f,60.7791f,  47.6356f,-9.3814f,58.9882f,  
	49.1794f,-6.6892f,58.0725f,  -17.4384f,-47.2073f,21.3104f,  -16.7560f,-48.0245f,22.4709f,  -14.9830f,-47.0982f,22.4468f,  -14.2988f,-47.8348f,23.6079f,  
	-12.6616f,-46.6921f,23.7342f,  -11.9849f,-47.3448f,24.8867f,  -10.5222f,-46.0064f,25.1537f,  -9.8613f,-46.5738f,26.2888f,  -8.6077f,-45.0660f,26.6852f,  
	-7.9723f,-45.5496f,27.7923f,  -6.9570f,-43.9043f,28.3061f,  -6.3545f,-44.3072f,29.3758f,  -5.6003f,-42.5606f,29.9943f,  -5.0359f,-42.8879f,31.0198f,  
	-4.5589f,-41.0798f,31.7309f,  -4.0364f,-41.3382f,32.7063f,  -3.8453f,-39.5112f,33.4985f,  -3.3675f,-39.7101f,34.4178f,  -3.4627f,-37.9081f,35.2813f,  
	-3.0286f,-38.0563f,36.1435f,  -3.4027f,-36.3231f,37.0699f,  -3.0090f,-36.4312f,37.8758f,  -3.6463f,-34.8096f,38.8588f,  -3.2875f,-34.8867f,39.6119f,  
	-4.1648f,-33.4167f,40.6471f,  -3.8310f,-33.4721f,41.3553f,  -4.9181f,-32.1899f,42.4399f,  -4.5988f,-32.2297f,43.1119f,  -5.8590f,-31.1664f,44.2447f,  
	-5.5406f,-31.1949f,44.8932f,  -6.9322f,-30.3754f,46.0738f,  -6.5966f,-30.3950f,46.7127f,  -8.0740f,-29.8373f,47.9409f,  -7.7058f,-29.8449f,48.5840f,  
	-9.2207f,-29.5597f,49.8591f,  -8.7996f,-29.5497f,50.5219f,  -10.3030f,-29.5402f,51.8417f,  -9.8108f,-29.5027f,52.5371f,  -11.2542f,-29.7643f,53.8974f,  
	-10.6706f,-29.6859f,54.6375f,  -12.0077f,-30.2068f,56.0313f,  -11.3171f,-30.0712f,56.8238f,  -12.5058f,-30.8337f,58.2408f,  -11.6930f,-30.6222f,59.0904f,  
	-12.6960f,-31.6028f,60.5167f,  -11.7493f,-31.2958f,61.4237f,  -12.5360f,-32.4669f,62.8420f,  -11.4457f,-32.0436f,63.8035f,  -11.9929f,-33.3748f,65.1929f,  
	-10.7565f,-32.8173f,66.1999f,  -11.0489f,-34.2765f,67.5368f,  -9.6660f,-33.5680f,68.5779f,  -9.6964f,-35.1226f,69.8367f,  -8.1684f,-34.2477f,70.8986f,  
	-7.9383f,-35.8657f,72.0524f,  -6.2735f,-34.8149f,73.1182f,  -5.7923f,-36.4662f,74.1399f,  -3.9999f,-35.2318f,75.1925f,  -3.2846f,-36.8887f,76.0558f,  
	-1.3771f,-35.4683f,77.0771f,  -0.4515f,-37.1060f,77.7577f,  1.5565f,-35.5006f,78.7302f,  2.6630f,-37.0977f,79.2066f,  4.7574f,-35.3102f,80.1143f,  
	6.0102f,-36.8490f,80.3689f,  8.1738f,-34.8879f,81.1962f,  9.5350f,-36.3539f,81.2157f,  11.7530f,-34.2276f,81.9497f,  13.1821f,-35.6096f,81.7259f,  
	15.4372f,-33.3312f,82.3548f,  16.8921f,-34.6214f,81.8849f,  19.1679f,-32.2053f,82.3989f,  20.6065f,-33.3985f,81.6856f,  22.8881f,-30.8594f,82.0776f,  
	24.2691f,-31.9528f,81.1294f,  26.5430f,-29.3058f,81.3939f,  27.8270f,-30.2988f,80.2246f,  30.0779f,-27.5625f,80.3579f,  31.2282f,-28.4558f,78.9865f,  
	33.4437f,-25.6472f,78.9870f,  34.4272f,-26.4429f,77.4373f,  36.5956f,-23.5791f,77.3049f,  37.3836f,-24.2802f,75.6050f,  39.4934f,-21.3797f,75.3414f,  
	40.0620f,-21.9892f,73.5228f,  42.1027f,-19.0706f,73.1313f,  42.4336f,-19.5918f,71.2284f,  44.3961f,-16.6725f,70.7136f,  44.4768f,-17.1085f,68.7629f,  
	46.3537f,-14.2049f,68.1305f,  46.1772f,-14.5585f,66.1694f,  47.9613f,-11.6875f,65.4267f,  47.5271f,-11.9605f,63.4930f,  49.2123f,-9.1386f,62.6481f,  
	48.5258f,-9.3321f,60.7791f,  50.1070f,-6.5750f,59.8413f,  49.1794f,-6.6892f,58.0725f,  50.6538f,-4.0094f,57.0513f,  -16.7560f,-48.0245f,22.4709f,  
	-16.0451f,-48.8098f,23.6502f,  -14.2988f,-47.8348f,23.6079f,  -13.5886f,-48.5364f,24.7841f,  -11.9849f,-47.3448f,24.8867f,  -11.2840f,-47.9599f,26.0506f,  
	-9.8613f,-46.5738f,26.2888f,  -9.1804f,-47.1024f,27.4294f,  -7.9723f,-45.5496f,27.7923f,  -7.3199f,-45.9935f,28.8995f,  -6.3545f,-44.3072f,29.3758f,  
	-5.7368f,-44.6701f,30.4416f,  -5.0359f,-42.8879f,31.0198f,  -4.4582f,-43.1756f,32.0370f,  -4.0364f,-41.3382f,32.7063f,  -3.5035f,-41.5595f,33.6673f,  
	-3.3675f,-39.7101f,34.4178f,  -2.8801f,-39.8734f,35.3199f,  -3.0286f,-38.0563f,36.1435f,  -2.5855f,-38.1720f,36.9856f,  -3.0090f,-36.4312f,37.8758f,  
	-2.6063f,-36.5088f,38.6596f,  -3.2875f,-34.8867f,39.6119f,  -2.9174f,-34.9363f,40.3435f,  -3.8310f,-33.4721f,41.3553f,  -3.4847f,-33.5009f,42.0419f,  
	-4.5988f,-32.2297f,43.1119f,  -4.2639f,-32.2436f,43.7651f,  -5.5406f,-31.1949f,44.8932f,  -5.2005f,-31.1979f,45.5261f,  -6.5966f,-30.3950f,46.7127f,  
	-6.2365f,-30.3859f,47.3388f,  -7.7058f,-29.8449f,48.5840f,  -7.3059f,-29.8202f,49.2189f,  -8.7996f,-29.5497f,50.5219f,  -8.3416f,-29.5016f,51.1793f,  
	-9.8108f,-29.5027f,52.5371f,  -9.2738f,-29.4194f,53.2303f,  -10.6706f,-29.6859f,54.6375f,  -10.0380f,-29.5524f,55.3758f,  -11.3171f,-30.0712f,56.8238f,  
	-10.5723f,-29.8699f,57.6138f,  -11.6930f,-30.6222f,59.0904f,  -10.8224f,-30.3338f,59.9346f,  -11.7493f,-31.2958f,61.4237f,  -10.7409f,-30.8995f,62.3214f,  
	-11.4457f,-32.0436f,63.8035f,  -10.2950f,-31.5211f,64.7477f,  -10.7565f,-32.8173f,66.1999f,  -9.4609f,-32.1508f,67.1809f,  -9.6660f,-33.5680f,68.5779f,  
	-8.2243f,-32.7406f,69.5842f,  -8.1684f,-34.2477f,70.8986f,  -6.5869f,-33.2481f,71.9144f,  -6.2735f,-34.8149f,73.1182f,  -4.5595f,-33.6337f,74.1272f,  
	-3.9999f,-35.2318f,75.1925f,  -2.1644f,-33.8645f,76.1770f,  -1.3771f,-35.4683f,77.0771f,  0.5665f,-33.9140f,78.0196f,  1.5565f,-35.5006f,78.7302f,  
	3.5953f,-33.7604f,79.6145f,  4.7574f,-35.3102f,80.1143f,  6.8745f,-33.3913f,80.9243f,  8.1738f,-34.8879f,81.1962f,  10.3546f,-32.7974f,81.9181f,  
	11.7530f,-34.2276f,81.9497f,  13.9805f,-31.9772f,82.5709f,  15.4372f,-33.3312f,82.3548f,  17.6946f,-30.9346f,82.8647f,  19.1679f,-32.2053f,82.3989f,  
	21.4400f,-29.6767f,82.7892f,  22.8881f,-30.8594f,82.0776f,  25.1608f,-28.2137f,82.3420f,  26.5430f,-29.3058f,81.3939f,  28.8003f,-26.5616f,81.5276f,  
	30.0779f,-27.5625f,80.3579f,  32.3068f,-24.7367f,80.3584f,  33.4437f,-25.6472f,78.9870f,  35.6315f,-22.7575f,78.8534f,  36.5956f,-23.5791f,77.3049f,  
	38.7299f,-20.6448f,77.0382f,  39.4934f,-21.3797f,75.3414f,  41.5628f,-18.4199f,74.9444f,  42.1027f,-19.0706f,73.1313f,  44.0973f,-16.1035f,72.6081f,  
	44.3961f,-16.6725f,70.7136f,  46.3076f,-13.7154f,70.0697f,  46.3537f,-14.2049f,68.1305f,  48.1738f,-11.2759f,67.3728f,  47.9613f,-11.6875f,65.4267f,  
	49.6832f,-8.8041f,64.5634f,  49.2123f,-9.1386f,62.6481f,  50.8301f,-6.3175f,61.6887f,  50.1070f,-6.5750f,59.8413f,  51.6173f,-3.8301f,58.7953f,  
	50.6538f,-4.0094f,57.0513f,  52.0534f,-1.3546f,55.9294f,  -16.0451f,-48.8098f,23.6502f,  -15.3058f,-49.5606f,24.8475f,  -13.5886f,-48.5364f,24.7841f,  
	-12.8514f,-49.2007f,25.9749f,  -11.2840f,-47.9599f,26.0506f,  -10.5605f,-48.5359f,27.2235f,  -9.1804f,-47.1024f,27.4294f,  -8.4799f,-47.5904f,28.5737f,  
	-7.3199f,-45.9935f,28.8995f,  -6.6497f,-46.3961f,30.0062f,  -5.7368f,-44.6701f,30.4416f,  -5.1036f,-44.9921f,31.5022f,  -4.4582f,-43.1756f,32.0370f,  
	-3.8686f,-43.4246f,33.0424f,  -3.5035f,-41.5595f,33.6673f,  -2.9597f,-41.7432f,34.6131f,  -2.8801f,-39.8734f,35.3199f,  -2.3829f,-40.0020f,36.2033f,  
	-2.5855f,-38.1720f,36.9856f,  -2.1329f,-38.2553f,37.8064f,  -2.6063f,-36.5088f,38.6596f,  -2.1928f,-36.5573f,39.4220f,  -2.9174f,-34.9363f,40.3435f,  
	-2.5357f,-34.9583f,41.0531f,  -3.4847f,-33.5009f,42.0419f,  -3.1240f,-33.5034f,42.7089f,  -4.2639f,-32.2436f,43.7651f,  -3.9093f,-32.2324f,44.4013f,  
	-5.2005f,-31.1979f,45.5261f,  -4.8381f,-31.1737f,46.1446f,  -6.2365f,-30.3859f,47.3388f,  -5.8474f,-30.3471f,47.9551f,  -7.3059f,-29.8202f,49.2189f,  
	-6.8717f,-29.7609f,49.8473f,  -8.3416f,-29.5016f,51.1793f,  -7.8414f,-29.4119f,51.8337f,  -9.2738f,-29.4194f,53.2303f,  -8.6902f,-29.2859f,53.9216f,  
	-10.0380f,-29.5524f,55.3758f,  -9.3531f,-29.3587f,56.1124f,  -10.5723f,-29.8699f,57.6138f,  -9.7710f,-29.5976f,58.4001f,  -10.8224f,-30.3338f,59.9346f,  
	-9.8907f,-29.9623f,60.7720f,  -10.7409f,-30.8995f,62.3214f,  -9.6722f,-30.4100f,63.2050f,  -10.2950f,-31.5211f,64.7477f,  -9.0845f,-30.8951f,65.6697f,  
	-9.4609f,-32.1508f,67.1809f,  -8.1049f,-31.3703f,68.1317f,  -8.2243f,-32.7406f,69.5842f,  -6.7271f,-31.7926f,70.5491f,  -6.5869f,-33.2481f,71.9144f,  
	-4.9538f,-32.1215f,72.8782f,  -4.5595f,-33.6337f,74.1272f,  -2.7998f,-32.3220f,75.0728f,  -2.1644f,-33.8645f,76.1770f,  -0.2901f,-32.3652f,77.0871f,  
	0.5665f,-33.9140f,78.0196f,  2.5438f,-32.2261f,78.8780f,  3.5953f,-33.7604f,79.6145f,  5.6592f,-31.8896f,80.4046f,  6.8745f,-33.3913f,80.9243f,  
	9.0108f,-31.3433f,81.6317f,  10.3546f,-32.7974f,81.9181f,  12.5469f,-30.5825f,82.5296f,  13.9805f,-31.9772f,82.5709f,  16.2119f,-29.6081f,83.0749f,  
	17.6946f,-30.9346f,82.8647f,  19.9496f,-28.4250f,83.2520f,  21.4400f,-29.6767f,82.7892f,  23.7040f,-27.0410f,83.0525f,  25.1608f,-28.2137f,82.3420f,  
	27.4174f,-25.4702f,82.4755f,  28.8003f,-26.5616f,81.5276f,  31.0359f,-23.7272f,81.5280f,  32.3068f,-24.7367f,80.3584f,  34.5079f,-21.8295f,80.2239f,  
	35.6315f,-22.7575f,78.8534f,  37.7848f,-19.7971f,78.5843f,  38.7299f,-20.6448f,77.0382f,  40.8232f,-17.6508f,76.6366f,  41.5628f,-18.4199f,74.9444f,  
	43.5853f,-15.4113f,74.4139f,  44.0973f,-16.1035f,72.6081f,  46.0398f,-13.0983f,71.9537f,  46.3076f,-13.7154f,70.0697f,  48.1611f,-10.7327f,69.2982f,  
	48.1738f,-11.2759f,67.3728f,  49.9305f,-8.3343f,66.4922f,  49.6832f,-8.8041f,64.5634f,  51.3365f,-5.9213f,63.5831f,  50.8301f,-6.3175f,61.6887f,  
	52.3757f,-3.5085f,60.6184f,  51.6173f,-3.8301f,58.7953f,  53.0513f,-1.1101f,57.6460f,  52.0534f,-1.3546f,55.9294f,  53.3730f,1.2616f,54.7131f,  
	-15.3058f,-49.5606f,24.8475f,  -14.5368f,-50.2748f,26.0629f,  -12.8514f,-49.2007f,25.9749f,  -12.0891f,-49.8257f,27.1782f,  -10.5605f,-48.5359f,27.2235f,  
	-9.8148f,-49.0708f,28.4038f,  -8.4799f,-47.5904f,28.5737f,  -7.7590f,-48.0363f,29.7209f,  -6.6497f,-46.3961f,30.0062f,  -5.9615f,-46.7565f,31.1109f,  
	-5.1036f,-44.9921f,31.5022f,  -4.4563f,-45.2734f,32.5540f,  -3.8686f,-43.4246f,33.0424f,  -3.2661f,-43.6338f,34.0353f,  -2.9597f,-41.7432f,34.6131f,  
	-2.4048f,-41.8901f,35.5423f,  -2.3829f,-40.0020f,36.2033f,  -1.8756f,-40.0962f,37.0668f,  -2.1329f,-38.2553f,37.8064f,  -1.6695f,-38.3075f,38.6064f,  
	-2.1928f,-36.5573f,39.4220f,  -1.7680f,-36.5767f,40.1624f,  -2.5357f,-34.9583f,41.0531f,  -2.1404f,-34.9529f,41.7426f,  -3.1240f,-33.5034f,42.7089f,  
	-2.7453f,-33.4807f,43.3578f,  -3.9093f,-32.2324f,44.4013f,  -3.5344f,-32.1946f,45.0220f,  -4.8381f,-31.1737f,46.1446f,  -4.4489f,-31.1213f,46.7521f,  
	-5.8474f,-30.3471f,47.9551f,  -5.4262f,-30.2761f,48.5638f,  -6.8717f,-29.7609f,49.8473f,  -6.3979f,-29.6636f,50.4718f,  -7.8414f,-29.4119f,51.8337f,  
	-7.2969f,-29.2765f,52.4861f,  -8.6902f,-29.2859f,53.9216f,  -8.0563f,-29.0975f,54.6116f,  -9.3531f,-29.3587f,56.1124f,  -8.6133f,-29.0997f,56.8464f,  
	-9.7710f,-29.5976f,58.4001f,  -8.9094f,-29.2479f,59.1818f,  -9.8907f,-29.9623f,60.7720f,  -8.8988f,-29.5031f,61.5983f,  -9.6722f,-30.4100f,63.2050f,  
	-8.5432f,-29.8225f,64.0701f,  -9.0845f,-30.8951f,65.6697f,  -7.8123f,-30.1600f,66.5658f,  -8.1049f,-31.3703f,68.1317f,  -6.6917f,-30.4734f,69.0456f,  
	-6.7271f,-31.7926f,70.5491f,  -5.1760f,-30.7212f,71.4668f,  -4.9538f,-32.1215f,72.8782f,  -3.2725f,-30.8673f,73.7834f,  -2.7998f,-32.3220f,75.0728f,  
	-0.9986f,-30.8806f,75.9487f,  -0.2901f,-32.3652f,77.0871f,  1.6208f,-30.7336f,77.9175f,  2.5438f,-32.2261f,78.8780f,  4.5487f,-30.4086f,79.6460f,  
	5.6592f,-31.8896f,80.4046f,  7.7446f,-29.8904f,81.0954f,  9.0108f,-31.3433f,81.6317f,  11.1610f,-29.1709f,82.2316f,  12.5469f,-30.5825f,82.5296f,  
	14.7451f,-28.2486f,83.0261f,  16.2119f,-29.6081f,83.0749f,  18.4424f,-27.1258f,83.4579f,  19.9496f,-28.4250f,83.2520f,  22.1977f,-25.8078f,83.5132f,  
	23.7040f,-27.0410f,83.0525f,  25.9528f,-24.3070f,83.1854f,  27.4174f,-25.4702f,82.4755f,  29.6529f,-22.6359f,82.4757f,  31.0359f,-23.7272f,81.5280f,  
	33.2439f,-20.8109f,81.3930f,  34.5079f,-21.8295f,80.2239f,  36.6742f,-18.8511f,79.9531f,  37.7848f,-19.7971f,78.5843f,  39.8966f,-16.7766f,78.1791f,  
	40.8232f,-17.6508f,76.6366f,  42.8689f,-14.6077f,76.1000f,  43.5853f,-15.4113f,74.4139f,  45.5549f,-12.3642f,73.7504f,  46.0398f,-13.0983f,71.9537f,  
	47.9234f,-10.0672f,71.1697f,  48.1611f,-10.7327f,69.2982f,  49.9500f,-7.7372f,68.4015f,  49.9305f,-8.3343f,66.4922f,  51.6174f,-5.3929f,65.4921f,  
	51.3365f,-5.9213f,63.5831f,  52.9162f,-3.0502f,62.4892f,  52.3757f,-3.5085f,60.6184f,  53.8436f,-0.7242f,59.4419f,  53.0513f,-1.1101f,57.6460f,  
	54.4038f,1.5716f,56.3993f,  53.3730f,1.2616f,54.7131f,  54.6085f,3.8277f,53.4086f,  -14.5368f,-50.2748f,26.0629f,  -13.7399f,-50.9496f,27.2943f,  
	-12.0891f,-49.8257f,27.1782f,  -11.3016f,-50.4092f,28.3927f,  -9.8148f,-49.0708f,28.4038f,  -9.0457f,-49.5627f,29.5907f,  -7.7590f,-48.0363f,29.7209f,  
	-7.0171f,-48.4386f,30.8699f,  -5.9615f,-46.7565f,31.1109f,  -5.2568f,-47.0743f,32.2103f,  -4.4563f,-45.2734f,32.5540f,  -3.7940f,-45.5129f,33.5963f,  
	-3.2661f,-43.6338f,34.0353f,  -2.6509f,-43.8038f,35.0140f,  -2.4048f,-41.8901f,35.5423f,  -1.8385f,-42.0000f,36.4536f,  -1.8756f,-40.0962f,37.0668f,  
	-1.3567f,-40.1569f,37.9108f,  -1.6695f,-38.3075f,38.6064f,  -1.1947f,-38.3287f,39.3851f,  -1.7680f,-36.5767f,40.1624f,  -1.3299f,-36.5672f,40.8828f,  
	-2.1404f,-34.9529f,41.7426f,  -1.7284f,-34.9215f,42.4133f,  -2.7453f,-33.4807f,43.3578f,  -2.3477f,-33.4313f,43.9902f,  -3.5344f,-32.1946f,45.0220f,  
	-3.1346f,-32.1295f,45.6305f,  -4.4489f,-31.1213f,46.7521f,  -4.0300f,-31.0385f,47.3509f,  -5.4262f,-30.2761f,48.5638f,  -4.9679f,-30.1701f,49.1677f,  
	-6.3979f,-29.6636f,50.4718f,  -5.8819f,-29.5243f,51.0938f,  -7.2969f,-29.2765f,52.4861f,  -6.7041f,-29.0906f,53.1374f,  -8.0563f,-29.0975f,54.6116f,  
	-7.3690f,-28.8486f,55.3002f,  -8.6133f,-29.0997f,56.8464f,  -7.8140f,-28.7686f,57.5779f,  -8.9094f,-29.2479f,59.1818f,  -7.9880f,-28.8159f,59.9551f,  
	-8.8988f,-29.5031f,61.5983f,  -7.8464f,-28.9512f,62.4096f,  -8.5432f,-29.8225f,64.0701f,  -7.3515f,-29.1306f,64.9135f,  -7.8123f,-30.1600f,66.5658f,  
	-6.4813f,-29.3129f,67.4297f,  -6.6917f,-30.4734f,69.0456f,  -5.2225f,-29.4568f,69.9172f,  -5.1760f,-30.7212f,71.4668f,  -3.5743f,-29.5253f,72.3309f,  
	-3.2725f,-30.8673f,73.7834f,  -1.5469f,-29.4856f,74.6237f,  -0.9986f,-30.8806f,75.9487f,  0.8421f,-29.3085f,76.7493f,  1.6208f,-30.7336f,77.9175f,  
	3.5620f,-28.9735f,78.6615f,  4.5487f,-30.4086f,79.6460f,  6.5777f,-28.4633f,80.3184f,  7.7446f,-29.8904f,81.0954f,  9.8465f,-27.7666f,81.6819f,  
	11.1610f,-29.1709f,82.2316f,  13.3194f,-26.8793f,82.7190f,  14.7451f,-28.2486f,83.0261f,  16.9442f,-25.8012f,83.4036f,  18.4424f,-27.1258f,83.4579f,  
	20.6674f,-24.5353f,83.7164f,  22.1977f,-25.8078f,83.5132f,  24.4316f,-23.0918f,83.6452f,  25.9528f,-24.3070f,83.1854f,  28.1813f,-21.4814f,83.1855f,  
	29.6529f,-22.6359f,82.4757f,  31.8611f,-19.7191f,82.3406f,  33.2439f,-20.8109f,81.3930f,  35.4170f,-17.8230f,81.1213f,  36.6742f,-18.8511f,79.9531f,  
	38.7988f,-15.8121f,79.5454f,  39.8966f,-16.7766f,78.1791f,  41.9604f,-13.7065f,77.6377f,  42.8689f,-14.6077f,76.1000f,  44.8611f,-11.5256f,75.4290f,  
	45.5549f,-12.3642f,73.7504f,  47.4647f,-9.2909f,72.9555f,  47.9234f,-10.0672f,71.1697f,  49.7414f,-7.0231f,70.2585f,  49.9500f,-7.7372f,68.4015f,  
	51.6681f,-4.7416f,67.3829f,  51.6174f,-5.3929f,65.4921f,  53.2295f,-2.4631f,64.3758f,  52.9162f,-3.0502f,62.4892f,  54.4169f,-0.2038f,61.2865f,  
	53.8436f,-0.7242f,59.4419f,  55.2287f,2.0219f,58.1653f,  54.4038f,1.5716f,56.3993f,  55.6709f,4.2034f,55.0615f,  54.6085f,3.8277f,53.4086f,  
	55.7570f,6.3331f,52.0226f,  -13.7399f,-50.9496f,27.2943f,  -12.9149f,-51.5828f,28.5408f,  -11.3016f,-50.4092f,28.3927f,  -10.4877f,-50.9490f,29.6178f,  
	-9.0457f,-49.5627f,29.5907f,  -8.2526f,-50.0097f,30.7833f,  -7.0171f,-48.4386f,30.8699f,  -6.2561f,-48.7965f,32.0173f,  -5.2568f,-47.0743f,32.2103f,  
	-4.5344f,-47.3481f,33.3036f,  -3.7940f,-45.5129f,33.5963f,  -3.1168f,-45.7106f,34.6274f,  -2.6509f,-43.8038f,35.0140f,  -2.0224f,-43.9343f,35.9773f,  
	-1.8385f,-42.0000f,36.4536f,  -1.2596f,-42.0739f,37.3470f,  -1.3567f,-40.1569f,37.9108f,  -0.8257f,-40.1841f,38.7344f,  -1.1947f,-38.3287f,39.3851f,  
	-0.7064f,-38.3187f,40.1442f,  -1.3299f,-36.5672f,40.8828f,  -0.8759f,-36.5301f,41.5840f,  -1.7284f,-34.9215f,42.4133f,  -1.2984f,-34.8625f,43.0669f,  
	-2.3477f,-33.4313f,43.9902f,  -1.9268f,-33.3546f,44.6094f,  -3.1346f,-32.1295f,45.6305f,  -2.7068f,-32.0350f,46.2292f,  -4.0300f,-31.0385f,47.3509f,  
	-3.5759f,-30.9228f,47.9437f,  -4.9679f,-30.1701f,49.1677f,  -4.4694f,-30.0250f,49.7685f,  -5.8819f,-29.5243f,51.0938f,  -5.3195f,-29.3382f,51.7148f,  
	-6.7041f,-29.0906f,53.1374f,  -6.0593f,-28.8486f,53.7880f,  -7.3690f,-28.8486f,55.3002f,  -6.6232f,-28.5325f,55.9878f,  -7.8140f,-28.7686f,57.5779f,  
	-6.9554f,-28.3602f,58.3033f,  -7.9880f,-28.8159f,59.9551f,  -7.0060f,-28.2963f,60.7164f,  -7.8464f,-28.9512f,62.4096f,  -6.7304f,-28.2997f,63.2030f,  
	-7.3515f,-29.1306f,64.9135f,  -6.0995f,-28.3311f,65.7290f,  -6.4813f,-29.3129f,67.4297f,  -5.0923f,-28.3499f,68.2559f,  -5.2225f,-29.4568f,69.9172f,  
	-3.7002f,-28.3188f,70.7401f,  -3.5743f,-29.5253f,72.3309f,  -1.9256f,-28.2041f,73.1349f,  -1.5469f,-29.4856f,74.6237f,  0.2211f,-27.9750f,75.3935f,  
	0.8421f,-29.3085f,76.7493f,  2.7161f,-27.6090f,77.4680f,  3.5620f,-28.9735f,78.6615f,  5.5299f,-27.0865f,79.3139f,  6.5777f,-28.4633f,80.3184f,  
	8.6259f,-26.3936f,80.8899f,  9.8465f,-27.7666f,81.6819f,  11.9590f,-25.5236f,82.1589f,  13.3194f,-26.8793f,82.7190f,  15.4807f,-24.4739f,83.0899f,  
	16.9442f,-25.8012f,83.4036f,  19.1396f,-23.2452f,83.6585f,  20.6674f,-24.5353f,83.7164f,  22.8798f,-21.8455f,83.8469f,  24.4316f,-23.0918f,83.6452f,  
	26.6464f,-20.2835f,83.6451f,  28.1813f,-21.4814f,83.1855f,  30.3833f,-18.5729f,83.0507f,  31.8611f,-19.7191f,82.3406f,  34.0348f,-16.7303f,82.0688f,  
	35.4170f,-17.8230f,81.1213f,  37.5487f,-14.7739f,80.7121f,  38.7988f,-15.8121f,79.5454f,  40.8753f,-12.7230f,79.0007f,  41.9604f,-13.7065f,77.6377f,  
	43.9704f,-10.5969f,76.9607f,  44.8611f,-11.5256f,75.4290f,  46.7929f,-8.4170f,74.6250f,  47.4647f,-9.2909f,72.9555f,  49.3081f,-6.2043f,72.0316f,  
	49.7414f,-7.0231f,70.2585f,  51.4876f,-3.9785f,69.2233f,  51.6681f,-4.7416f,67.3829f,  53.3102f,-1.7572f,66.2459f,  53.2295f,-2.4631f,64.3758f,  
	54.7614f,0.4424f,63.1482f,  54.4169f,-0.2038f,61.2865f,  55.8335f,2.6048f,59.9810f,  55.2287f,2.0219f,58.1653f,  56.5270f,4.7185f,56.7947f,  
	55.6709f,4.2034f,55.0615f,  56.8496f,6.7746f,53.6392f,  55.7570f,6.3331f,52.0226f,  56.8158f,8.7666f,50.5628f,  -12.9149f,-51.5828f,28.5408f,  
	-12.0603f,-52.1716f,29.8018f,  -10.4877f,-50.9490f,29.6178f,  -9.6466f,-51.4429f,30.8525f,  -8.2526f,-50.0097f,30.7833f,  -7.4374f,-50.4106f,31.9782f,  
	-6.2561f,-48.7965f,32.0173f,  -5.4746f,-49.1083f,33.1624f,  -4.5344f,-47.3481f,33.3036f,  -3.7946f,-47.5776f,34.3891f,  -3.1168f,-45.7106f,34.6274f,  
	-2.4242f,-45.8659f,35.6459f,  -2.0224f,-43.9343f,35.9773f,  -1.3798f,-44.0256f,36.9246f,  -1.2596f,-42.0739f,37.3470f,  -0.6673f,-42.1112f,38.2215f,  
	-0.8257f,-40.1841f,38.7344f,  -0.2806f,-40.1774f,39.5393f,  -0.7064f,-38.3187f,40.1442f,  -0.2025f,-38.2790f,40.8840f,  -0.8759f,-36.5301f,41.5840f,  
	-0.4044f,-36.4638f,42.2677f,  -1.2984f,-34.8625f,43.0669f,  -0.8462f,-34.7755f,43.7064f,  -1.9268f,-33.3546f,44.6094f,  -1.4793f,-33.2486f,45.2179f,  
	-2.7068f,-32.0350f,46.2292f,  -2.2458f,-31.9088f,46.8209f,  -3.5759f,-30.9228f,47.9437f,  -3.0836f,-30.7701f,48.5330f,  -4.4694f,-30.0250f,49.7685f,  
	-3.9264f,-29.8360f,50.3681f,  -5.3195f,-29.3382f,51.7148f,  -4.7068f,-29.0998f,52.3355f,  -6.0593f,-28.8486f,53.7880f,  -5.3573f,-28.5438f,54.4388f,  
	-6.6232f,-28.5325f,55.9878f,  -5.8187f,-28.1437f,56.6714f,  -6.9554f,-28.3602f,58.3033f,  -6.0363f,-27.8688f,59.0196f,  -7.0060f,-28.2963f,60.7164f,  
	-5.9596f,-27.6815f,61.4635f,  -6.7304f,-28.2997f,63.2030f,  -5.5529f,-27.5448f,63.9726f,  -6.0995f,-28.3311f,65.7290f,  -4.7877f,-27.4194f,66.5114f,  
	-5.0923f,-28.3499f,68.2559f,  -3.6480f,-27.2685f,69.0382f,  -3.7002f,-28.3188f,70.7401f,  -2.1283f,-27.0581f,71.5078f,  -1.9256f,-28.2041f,73.1349f,  
	-0.2316f,-26.7560f,73.8732f,  0.2211f,-27.9750f,75.3935f,  2.0253f,-26.3383f,76.0859f,  2.7161f,-27.6090f,77.4680f,  4.6194f,-25.7837f,78.0992f,  
	5.5299f,-27.0865f,79.3139f,  7.5201f,-25.0755f,79.8693f,  8.6259f,-26.3936f,80.8899f,  10.6874f,-24.2047f,81.3554f,  11.9590f,-25.5236f,82.1589f,  
	14.0768f,-23.1666f,82.5222f,  15.4807f,-24.4739f,83.0899f,  17.6402f,-21.9596f,83.3405f,  19.1396f,-23.2452f,83.6585f,  21.3239f,-20.5894f,83.7872f,  
	22.8798f,-21.8455f,83.8469f,  25.0744f,-19.0628f,83.8467f,  26.6464f,-20.2835f,83.6451f,  28.8356f,-17.3917f,83.5109f,  30.3833f,-18.5729f,83.0507f,  
	32.5513f,-15.5918f,82.7794f,  34.0348f,-16.7303f,82.0688f,  36.1675f,-13.6797f,81.6594f,  37.5487f,-14.7739f,80.7121f,  39.6323f,-11.6742f,80.1655f,  
	40.8753f,-12.7230f,79.0007f,  42.8979f,-9.5940f,78.3194f,  43.9704f,-10.5969f,76.9607f,  45.9195f,-7.4604f,76.1495f,  46.7929f,-8.4170f,74.6250f,  
	48.6577f,-5.2946f,73.6904f,  49.3081f,-6.2043f,72.0316f,  51.0790f,-3.1165f,70.9818f,  51.4876f,-3.9785f,69.2233f,  53.1570f,-0.9446f,68.0675f,  
	53.3102f,-1.7572f,66.2459f,  54.8711f,1.2032f,64.9952f,  54.7614f,0.4424f,63.1482f,  56.2079f,3.3105f,61.8151f,  55.8335f,2.6048f,59.9810f,  
	57.1621f,5.3645f,58.5784f,  56.5270f,4.7185f,56.7947f,  57.7355f,7.3550f,55.3364f,  56.8496f,6.7746f,53.6392f,  57.9370f,9.2742f,52.1399f,  
	56.8158f,8.7666f,50.5628f,  57.7828f,11.1184f,49.0372f,  -12.0603f,-52.1716f,29.8018f,  -11.1754f,-52.7136f,31.0764f,  -9.6466f,-51.4429f,30.8525f,  
	-8.7803f,-51.8889f,32.0937f,  -7.4374f,-50.4106f,31.9782f,  -6.5986f,-50.7632f,33.1748f,  -5.4746f,-49.1083f,33.1624f,  -4.6728f,-49.3730f,34.3032f,  
	-3.7946f,-47.5776f,34.3891f,  -3.0369f,-47.7617f,35.4651f,  -2.4242f,-45.8659f,35.6459f,  -1.7154f,-45.9787f,36.6510f,  -1.3798f,-44.0256f,36.9246f,  
	-0.7222f,-44.0772f,37.8552f,  -0.6673f,-42.1112f,38.2215f,  -0.0598f,-42.1116f,39.0786f,  -0.2806f,-40.1774f,39.5393f,  0.2804f,-40.1383f,40.3254f,  
	-0.2025f,-38.2790f,40.8840f,  0.3191f,-38.2078f,41.6065f,  -0.4044f,-36.4638f,42.2677f,  0.0887f,-36.3679f,42.9369f,  -0.8462f,-34.7755f,43.7064f,  
	-0.3685f,-34.6585f,44.3344f,  -1.4793f,-33.2486f,45.2179f,  -1.0003f,-33.1112f,45.8185f,  -2.2458f,-31.9088f,46.8209f,  -1.7482f,-31.7467f,47.4083f,  
	-3.0836f,-30.7701f,48.5330f,  -2.5484f,-30.5756f,49.1206f,  -3.9264f,-29.8360f,50.3681f,  -3.3345f,-29.5977f,50.9676f,  -4.7068f,-29.0998f,52.3355f,  
	-4.0380f,-28.8021f,52.9572f,  -5.3573f,-28.5438f,54.4388f,  -4.5973f,-28.1702f,55.0872f,  -5.8187f,-28.1437f,56.6714f,  -4.9540f,-27.6761f,57.3483f,  
	-6.0363f,-27.8688f,59.0196f,  -5.0524f,-27.2866f,59.7248f,  -5.9596f,-27.6815f,61.4635f,  -4.8508f,-26.9673f,62.1906f,  -5.5529f,-27.5448f,63.9726f,  
	-4.3140f,-26.6813f,64.7134f,  -4.7877f,-27.4194f,66.5114f,  -3.4185f,-26.3925f,67.2544f,  -3.6480f,-27.2685f,69.0382f,  -2.1514f,-26.0671f,69.7701f,  
	-2.1283f,-27.0581f,71.5078f,  -0.5082f,-25.6724f,72.2145f,  -0.2316f,-26.7560f,73.8732f,  1.5015f,-25.1833f,74.5387f,  2.0253f,-26.3383f,76.0859f,  
	3.8615f,-24.5770f,76.6950f,  4.6194f,-25.7837f,78.0992f,  6.5479f,-23.8347f,78.6374f,  7.5201f,-25.0755f,79.8693f,  9.5262f,-22.9453f,80.3222f,  
	10.6874f,-24.2047f,81.3554f,  12.7566f,-21.9017f,81.7103f,  14.0768f,-23.1666f,82.5222f,  16.1949f,-20.7005f,82.7678f,  17.6402f,-21.9596f,83.3405f,  
	19.7906f,-19.3449f,83.4670f,  21.3239f,-20.5894f,83.7872f,  23.4922f,-17.8400f,83.7868f,  25.0744f,-19.0628f,83.8467f,  27.2446f,-16.1960f,83.7136f,  
	28.8356f,-17.3917f,83.5109f,  30.9918f,-14.4269f,83.2411f,  32.5513f,-15.5918f,82.7794f,  34.6790f,-12.5483f,82.3709f,  36.1675f,-13.6797f,81.6594f,  
	38.2525f,-10.5780f,81.1124f,  39.6323f,-11.6742f,80.1655f,  41.6620f,-8.5340f,79.4818f,  42.8979f,-9.5940f,78.3194f,  44.8594f,-6.4376f,77.5030f,  
	45.9195f,-7.4604f,76.1495f,  47.8012f,-4.3097f,75.2065f,  48.6577f,-5.2946f,73.6904f,  50.4495f,-2.1705f,72.6283f,  51.0790f,-3.1165f,70.9818f,  
	52.7724f,-0.0392f,69.8095f,  53.1570f,-0.9446f,68.0675f,  54.7441f,2.0657f,66.7957f,  54.8711f,1.2032f,64.9952f,  56.3455f,4.1267f,63.6363f,  
	56.2079f,3.3105f,61.8151f,  57.5653f,6.1300f,60.3821f,  57.1621f,5.3645f,58.5784f,  58.3996f,8.0642f,57.0852f,  57.7355f,7.3550f,55.3364f,  
	58.8513f,9.9200f,53.7980f,  57.9370f,9.2742f,52.1399f,  58.9313f,11.6924f,50.5714f,  57.7828f,11.1184f,49.0372f,  58.6573f,13.3800f,47.4536f,  
	-11.1754f,-52.7136f,31.0764f,  -10.2619f,-53.2061f,32.3619f,  -8.7803f,-51.8889f,32.0937f,  -7.8870f,-52.2846f,33.3409f,  -6.5986f,-50.7632f,33.1748f,  
	-5.7363f,-51.0661f,34.3711f,  -4.6728f,-49.3730f,34.3032f,  -3.8501f,-49.5892f,35.4381f,  -3.0369f,-47.7617f,35.4651f,  -2.2605f,-47.8998f,36.5307f,  
	-1.7154f,-45.9787f,36.6510f,  -0.9895f,-46.0483f,37.6417f,  -0.7222f,-44.0772f,37.8552f,  -0.0476f,-44.0882f,38.7700f,  -0.0598f,-42.1116f,39.0786f,  
	0.5645f,-42.0761f,39.9178f,  0.2804f,-40.1383f,40.3254f,  0.8596f,-40.0646f,41.0945f,  0.3191f,-38.2078f,41.6065f,  0.8619f,-38.1047f,42.3140f,  
	0.0887f,-36.3679f,42.9369f,  0.6065f,-36.2406f,43.5938f,  -0.3685f,-34.6585f,44.3344f,  0.1393f,-34.5095f,44.9535f,  -1.0003f,-33.1112f,45.8185f,  
	-0.4861f,-32.9383f,46.4138f,  -1.7482f,-31.7467f,47.4083f,  -1.2093f,-31.5443f,47.9934f,  -2.5484f,-30.5756f,49.1206f,  -1.9660f,-30.3340f,49.7079f,  
	-3.3345f,-29.5977f,50.9676f,  -2.6880f,-29.3032f,51.5686f,  -4.0380f,-28.8021f,52.9572f,  -3.3121f,-28.4392f,53.5779f,  -4.5973f,-28.1702f,55.0872f,  
	-3.7775f,-27.7216f,55.7309f,  -4.9540f,-27.6761f,57.3483f,  -4.0242f,-27.1217f,58.0167f,  -5.0524f,-27.2866f,59.7248f,  -4.0052f,-26.6087f,60.4134f,  
	-4.8508f,-26.9673f,62.1906f,  -3.6792f,-26.1481f,62.8927f,  -4.3140f,-26.6813f,64.7134f,  -3.0157f,-25.7057f,65.4192f,  -3.4185f,-26.3925f,67.2544f,  
	-1.9947f,-25.2481f,67.9517f,  -2.1514f,-26.0671f,69.7701f,  -0.6041f,-24.7427f,70.4457f,  -0.5082f,-25.6724f,72.2145f,  1.1536f,-24.1637f,72.8530f,  
	1.5015f,-25.1833f,74.5387f,  3.2694f,-23.4872f,75.1254f,  3.8615f,-24.5770f,76.6950f,  5.7261f,-22.6925f,77.2154f,  6.5479f,-23.8347f,78.6374f,  
	8.4954f,-21.7667f,79.0770f,  9.5262f,-22.9453f,80.3222f,  11.5427f,-20.7009f,80.6679f,  12.7566f,-21.9017f,81.7103f,  14.8285f,-19.4894f,81.9503f,  
	16.1949f,-20.7005f,82.7678f,  18.3062f,-18.1335f,82.8918f,  19.7906f,-19.3449f,83.4670f,  21.9269f,-16.6361f,83.4665f,  23.4922f,-17.8400f,83.7868f,  
	25.6376f,-15.0060f,83.6550f,  27.2446f,-16.1960f,83.7136f,  29.3830f,-13.2556f,83.4458f,  30.9918f,-14.4269f,83.2411f,  33.1085f,-11.3990f,82.8346f,  
	34.6790f,-12.5483f,82.3709f,  36.7595f,-9.4531f,81.8250f,  38.2525f,-10.5780f,81.1124f,  40.2839f,-7.4351f,80.4280f,  41.6620f,-8.5340f,79.4818f,  
	43.6308f,-5.3660f,78.6624f,  44.8594f,-6.4376f,77.5030f,  46.7533f,-3.2664f,76.5539f,  47.8012f,-4.3097f,75.2065f,  49.6096f,-1.1567f,74.1345f,  
	50.4495f,-2.1705f,72.6283f,  52.1630f,0.9436f,71.4419f,  52.7724f,-0.0392f,69.8095f,  54.3827f,3.0151f,68.5192f,  54.7441f,2.0657f,66.7957f,  
	56.2439f,5.0395f,65.4136f,  56.3455f,4.1267f,63.6363f,  57.7298f,7.0022f,62.1750f,  57.5653f,6.1300f,60.3821f,  58.8304f,8.8901f,58.8556f,  
	58.3996f,8.0642f,57.0852f,  59.5430f,10.6929f,55.5086f,  58.8513f,9.9200f,53.7980f,  59.8726f,12.4040f,52.1868f,  58.9313f,11.6924f,50.5714f,  
	59.8314f,14.0206f,48.9413f,  58.6573f,13.3800f,47.4536f,  59.4386f,15.5433f,45.8206f,  -10.2619f,-53.2061f,32.3619f,  -9.3178f,-53.6463f,33.6576f,  
	-7.8870f,-52.2846f,33.3409f,  -6.9668f,-52.6279f,34.5917f,  -5.7363f,-51.0661f,34.3711f,  -4.8499f,-51.3174f,35.5653f,  -3.8501f,-49.5892f,35.4381f,  
	-3.0059f,-49.7560f,36.5659f,  -2.2605f,-47.8998f,36.5307f,  -1.4645f,-47.9907f,37.5847f,  -0.9895f,-46.0483f,37.6417f,  -0.2444f,-46.0732f,38.6187f,  
	-0.0476f,-44.0882f,38.7700f,  0.6449f,-44.0593f,39.6680f,  0.5645f,-42.0761f,39.9178f,  1.2078f,-42.0025f,40.7404f,  0.8596f,-40.0646f,41.0945f,  
	1.4601f,-39.9561f,41.8485f,  0.8619f,-38.1047f,42.3140f,  1.4292f,-37.9677f,43.0087f,  0.6065f,-36.2406f,43.5938f,  1.1536f,-36.0797f,44.2408f,  
	0.1393f,-34.5095f,44.9535f,  0.6813f,-34.3244f,45.5663f,  -0.4861f,-32.9383f,46.4138f,  0.0681f,-32.7255f,47.0059f,  -1.2093f,-31.5443f,47.9934f,  
	-0.6246f,-31.2960f,48.5777f,  -1.9660f,-30.3340f,49.7079f,  -1.3302f,-30.0384f,50.2966f,  -2.6880f,-29.3032f,51.5686f,  -1.9852f,-28.9461f,52.1693f,  
	-3.3121f,-28.4392f,53.5779f,  -2.5269f,-28.0044f,54.1952f,  -3.7775f,-27.7216f,55.7309f,  -2.8926f,-27.1896f,56.3683f,  -4.0242f,-27.1217f,58.0167f,  
	-3.0305f,-26.4749f,58.6714f,  -4.0052f,-26.6087f,60.4134f,  -2.8941f,-25.8291f,61.0805f,  -3.6792f,-26.1481f,62.8927f,  -2.4465f,-25.2198f,63.5639f,  
	-3.0157f,-25.7057f,65.4192f,  -1.6607f,-24.6150f,66.0836f,  -1.9947f,-25.2481f,67.9517f,  -0.5176f,-23.9828f,68.5972f,  -0.6041f,-24.7427f,70.4457f,  
	0.9877f,-23.2969f,71.0577f,  1.1536f,-24.1637f,72.8530f,  2.8531f,-22.5328f,73.4171f,  3.2694f,-23.4872f,75.1254f,  5.0686f,-21.6685f,75.6275f,  
	5.7261f,-22.6925f,77.2154f,  7.6125f,-20.6893f,77.6411f,  8.4954f,-21.7667f,79.0770f,  10.4559f,-19.5846f,79.4130f,  11.5427f,-20.7009f,80.6679f,  
	13.5643f,-18.3471f,80.9019f,  14.8285f,-19.4894f,81.9503f,  16.8959f,-16.9757f,82.0716f,  18.3062f,-18.1335f,82.8918f,  20.4053f,-15.4718f,82.8912f,  
	21.9269f,-16.6361f,83.4665f,  24.0419f,-13.8422f,83.3364f,  25.6376f,-15.0060f,83.6550f,  27.7524f,-12.0979f,83.3901f,  29.3830f,-13.2556f,83.4458f,  
	31.4831f,-10.2515f,83.0424f,  33.1085f,-11.3990f,82.8346f,  35.1790f,-8.3187f,82.2912f,  36.7595f,-9.4531f,81.8250f,  38.7872f,-6.3161f,81.1419f,  
	40.2839f,-7.4351f,80.4280f,  42.2546f,-4.2639f,79.6079f,  43.6308f,-5.3660f,78.6624f,  45.5319f,-2.1826f,77.7097f,  46.7533f,-3.2664f,76.5539f,  
	48.5738f,-0.0923f,75.4748f,  49.6096f,-1.1567f,74.1345f,  51.3394f,1.9869f,72.9370f,  52.1630f,0.9436f,71.4419f,  53.7930f,4.0351f,70.1359f,  
	54.3827f,3.0151f,68.5192f,  55.9049f,6.0334f,67.1163f,  56.2439f,5.0395f,65.4136f,  57.6528f,7.9660f,63.9263f,  57.7298f,7.0022f,62.1750f,  
	59.0207f,9.8186f,60.6173f,  58.8304f,8.8901f,58.8556f,  60.0002f,11.5796f,57.2426f,  59.5430f,10.6929f,55.5086f,  60.5905f,13.2409f,53.8560f,  
	59.8726f,12.4040f,52.1868f,  60.7981f,14.7982f,50.5107f,  59.8314f,14.0206f,48.9413f,  60.6368f,16.2507f,47.2580f,  59.4386f,15.5433f,45.8206f,  
	60.1273f,17.6018f,44.1461f,  -9.3178f,-53.6463f,33.6576f,  -8.3434f,-54.0316f,34.9612f,  -6.9668f,-52.6279f,34.5917f,  -6.0191f,-52.9165f,35.8446f,  
	-4.8499f,-51.3174f,35.5653f,  -3.9388f,-51.5154f,36.7559f,  -3.0059f,-49.7560f,36.5659f,  -2.1392f,-49.8714f,37.6852f,  -1.4645f,-47.9907f,37.5847f,  
	-0.6468f,-48.0329f,38.6273f,  -0.2444f,-46.0732f,38.6187f,  0.5202f,-46.0541f,39.5803f,  0.6449f,-44.0593f,39.6680f,  1.3579f,-43.9884f,40.5504f,  
	1.2078f,-42.0025f,40.7404f,  1.8732f,-41.8903f,41.5482f,  1.4601f,-39.9561f,41.8485f,  2.0853f,-39.8104f,42.5892f,  1.4292f,-37.9677f,43.0087f,  
	2.0250f,-37.7950f,43.6926f,  1.1536f,-36.0797f,44.2408f,  1.7338f,-35.8814f,44.8805f,  0.6813f,-34.3244f,45.5663f,  1.2619f,-34.0990f,46.1749f,  
	0.0681f,-32.7255f,47.0059f,  0.6667f,-32.4673f,47.5964f,  -0.6246f,-31.2960f,48.5777f,  0.0121f,-30.9951f,49.1630f,  -1.3302f,-30.0384f,50.2966f,  
	-0.6391f,-29.6822f,50.8853f,  -1.9852f,-28.9461f,52.1693f,  -1.2238f,-28.5196f,52.7675f,  -2.5269f,-28.0044f,54.1952f,  -1.6770f,-27.4892f,54.8080f,  
	-2.8926f,-27.1896f,56.3683f,  -1.9436f,-26.5684f,56.9945f,  -3.0305f,-26.4749f,58.6714f,  -1.9723f,-25.7296f,59.3078f,  -2.8941f,-25.8291f,61.0805f,  
	-1.7206f,-24.9431f,61.7204f,  -2.4465f,-25.2198f,63.5639f,  -1.1552f,-24.1788f,64.1977f,  -1.6607f,-24.6150f,66.0836f,  -0.2502f,-23.4057f,66.7004f,  
	-0.5176f,-23.9828f,68.5972f,  1.0066f,-22.5977f,69.1834f,  0.9877f,-23.2969f,71.0577f,  2.6198f,-21.7303f,71.5996f,  2.8531f,-22.5328f,73.4171f,  
	4.5868f,-20.7802f,73.9011f,  5.0686f,-21.6685f,75.6275f,  6.8926f,-19.7314f,76.0391f,  7.6125f,-20.6893f,77.6411f,  9.5147f,-18.5720f,77.9670f,  
	10.4559f,-19.5846f,79.4130f,  12.4241f,-17.2930f,79.6407f,  13.5643f,-18.3471f,80.9019f,  15.5839f,-15.8915f,81.0202f,  16.8959f,-16.9757f,82.0716f,  
	18.9533f,-14.3669f,82.0708f,  20.4053f,-15.4718f,82.8912f,  22.4849f,-12.7245f,82.7631f,  24.0419f,-13.8422f,83.3364f,  26.1280f,-10.9737f,83.0749f,  
	27.7524f,-12.0979f,83.3901f,  29.8304f,-9.1254f,82.9908f,  31.4831f,-10.2515f,83.0424f,  33.5380f,-7.1943f,82.5031f,  35.1790f,-8.3187f,82.2912f,  
	37.1974f,-5.1961f,81.6112f,  38.7872f,-6.3161f,81.1419f,  40.7545f,-3.1503f,80.3231f,  42.2546f,-4.2639f,79.6079f,  44.1580f,-1.0769f,78.6540f,  
	45.5319f,-2.1826f,77.7097f,  47.3598f,1.0043f,76.6262f,  48.5738f,-0.0923f,75.4748f,  50.3155f,3.0729f,74.2691f,  51.3394f,1.9869f,72.9370f,  
	52.9851f,5.1083f,71.6184f,  53.7930f,4.0351f,70.1359f,  55.3343f,7.0912f,68.7155f,  55.9049f,6.0334f,67.1163f,  57.3355f,9.0050f,65.6062f,  
	57.6528f,7.9660f,63.9263f,  58.9672f,10.8339f,62.3402f,  59.0207f,9.8186f,60.6173f,  60.2151f,12.5650f,58.9703f,  60.0002f,11.5796f,57.2426f,  
	61.0728f,14.1889f,55.5504f,  60.5905f,13.2409f,53.8560f,  61.5408f,15.6995f,52.1349f,  60.7981f,14.7982f,50.5107f,  61.6273f,17.0946f,48.7774f,  
	60.6368f,16.2507f,47.2580f,  61.3477f,18.3759f,45.5294f,  60.1273f,17.6018f,44.1461f,  60.7241f,19.5489f,42.4392f,  -8.3434f,-54.0316f,34.9612f,  
	-7.3377f,-54.3591f,36.2711f,  -6.0191f,-52.9165f,35.8446f,  -5.0433f,-53.1481f,37.0975f,  -3.9388f,-51.5154f,36.7559f,  -3.0020f,-51.6580f,37.9413f,  
	-2.1392f,-49.8714f,37.6852f,  -1.2479f,-49.9335f,38.7956f,  -0.6468f,-48.0329f,38.6273f,  0.1926f,-48.0261f,39.6564f,  0.5202f,-46.0541f,39.5803f,  
	1.3072f,-45.9881f,40.5277f,  1.3579f,-43.9884f,40.5504f,  2.0940f,-43.8744f,41.4183f,  1.8732f,-41.8903f,41.5482f,  2.5638f,-41.7372f,42.3424f,  
	2.0853f,-39.8104f,42.5892f,  2.7388f,-39.6260f,43.3182f,  2.0250f,-37.7950f,43.6926f,  2.6535f,-37.5824f,44.3680f,  1.7338f,-35.8814f,44.8805f,  
	2.3517f,-35.6412f,45.5148f,  1.2619f,-34.0990f,46.1749f,  1.8858f,-33.8277f,46.7809f,  0.6667f,-32.4673f,47.5964f,  1.3159f,-32.1571f,48.1872f,  
	0.0121f,-30.9951f,49.1630f,  0.7030f,-30.6349f,49.7482f,  -0.6391f,-29.6822f,50.8853f,  0.1097f,-29.2586f,51.4722f,  -1.2238f,-28.5196f,52.7675f,  
	-0.3982f,-28.0153f,53.3623f,  -1.6770f,-27.4892f,54.8080f,  -0.7629f,-26.8875f,55.4114f,  -1.9436f,-26.5684f,56.9945f,  -0.9294f,-25.8514f,57.6050f,  
	-1.9723f,-25.7296f,59.3078f,  -0.8504f,-24.8805f,59.9201f,  -1.7206f,-24.9431f,61.7204f,  -0.4868f,-23.9470f,62.3265f,  -1.1552f,-24.1788f,64.1977f,  
	0.1936f,-23.0213f,64.7880f,  -0.2502f,-23.4057f,66.7004f,  1.2098f,-22.0782f,67.2622f,  1.0066f,-22.5977f,69.1834f,  2.5734f,-21.0933f,69.7037f,  
	2.6198f,-21.7303f,71.5996f,  4.2888f,-20.0427f,72.0655f,  4.5868f,-20.7802f,73.9011f,  6.3482f,-18.9094f,74.2984f,  6.8926f,-19.7314f,76.0391f,  
	8.7353f,-17.6804f,76.3546f,  9.5147f,-18.5720f,77.9670f,  11.4272f,-16.3453f,78.1880f,  12.4241f,-17.2930f,79.6407f,  14.3928f,-14.8993f,79.7558f,  
	15.5839f,-15.8915f,81.0202f,  17.5959f,-13.3403f,81.0192f,  18.9533f,-14.3669f,82.0708f,  20.9933f,-11.6720f,81.9450f,  22.4849f,-12.7245f,82.7631f,  
	24.5375f,-9.9021f,82.5057f,  26.1280f,-10.9737f,83.0749f,  28.1788f,-8.0400f,82.6808f,  29.8304f,-9.1254f,82.9908f,  31.8645f,-6.0992f,82.4569f,  
	33.5380f,-7.1943f,82.5031f,  35.5417f,-4.0943f,81.8279f,  37.1974f,-5.1961f,81.6112f,  39.1561f,-2.0441f,80.7960f,  40.7545f,-3.1503f,80.3231f,  
	42.6550f,0.0320f,79.3707f,  44.1580f,-1.0769f,78.6540f,  45.9884f,2.1145f,77.5690f,  47.3598f,1.0043f,76.6262f,  49.1089f,4.1829f,75.4154f,  
	50.3155f,3.0729f,74.2691f,  51.9728f,6.2163f,72.9411f,  52.9851f,5.1083f,71.6184f,  54.5418f,8.1950f,70.1838f,  55.3343f,7.0912f,68.7155f,  
	56.7833f,10.1013f,67.1858f,  57.3355f,9.0050f,65.6062f,  58.6707f,11.9185f,63.9949f,  58.9672f,10.8339f,62.3402f,  60.1841f,13.6322f,60.6621f,  
	60.2151f,12.5650f,58.9703f,  61.3112f,15.2317f,57.2409f,  61.0728f,14.1889f,55.5504f,  62.0468f,16.7092f,53.7862f,  61.5408f,15.6995f,52.1349f,  
	62.3933f,18.0606f,50.3529f,  61.6273f,17.0946f,48.7774f,  62.3604f,19.2861f,46.9950f,  61.3477f,18.3759f,45.5294f,  61.9649f,20.3896f,43.7642f,  
	60.7241f,19.5489f,42.4392f,  61.2306f,21.3803f,40.7080f,  -7.3377f,-54.3591f,36.2711f,  -6.3003f,-54.6260f,37.5849f,  -5.0433f,-53.1481f,37.0975f,  
	-4.0381f,-53.3201f,38.3489f,  -3.0020f,-51.6580f,37.9413f,  -2.0373f,-51.7427f,39.1209f,  -1.2479f,-49.9335f,38.7956f,  -0.3324f,-49.9418f,39.8948f,  
	0.1926f,-48.0261f,39.6564f,  1.0566f,-47.9676f,40.6727f,  1.3072f,-45.9881f,40.5277f,  2.1189f,-45.8743f,41.4611f,  2.0940f,-43.8744f,41.4183f,  
	2.8563f,-43.7150f,42.2726f,  2.5638f,-41.7372f,42.3424f,  3.2827f,-41.5413f,43.1242f,  2.7388f,-39.6260f,43.3182f,  3.4246f,-39.3986f,44.0377f,  
	2.6535f,-37.5824f,44.3680f,  3.3189f,-37.3259f,45.0367f,  2.3517f,-35.6412f,45.5148f,  3.0119f,-35.3541f,46.1451f,  1.8858f,-33.8277f,46.7809f,  
	2.5590f,-33.5042f,47.3859f,  1.3159f,-32.1571f,48.1872f,  2.0182f,-31.7880f,48.7772f,  0.7030f,-30.6349f,49.7482f,  1.4507f,-30.2086f,50.3314f,  
	0.1097f,-29.2586f,51.4722f,  0.9219f,-28.7591f,52.0561f,  -0.3982f,-28.0153f,53.3623f,  0.4913f,-27.4267f,53.9492f,  -0.7629f,-26.8875f,55.4114f,  
	0.2166f,-26.1923f,56.0011f,  -0.9294f,-25.8514f,57.6050f,  0.1491f,-25.0329f,58.1939f,  -0.8504f,-24.8805f,59.9201f,  0.3331f,-23.9233f,60.5018f,  
	-0.4868f,-23.9470f,62.3265f,  0.8061f,-22.8362f,62.8926f,  0.1936f,-23.0213f,64.7880f,  1.5940f,-21.7472f,65.3271f,  1.2098f,-22.0782f,67.2622f,  
	2.7147f,-20.6327f,67.7621f,  2.5734f,-21.0933f,69.7037f,  4.1799f,-19.4687f,70.1521f,  4.2888f,-20.0427f,72.0655f,  5.9884f,-18.2375f,72.4488f,  
	6.3482f,-18.9094f,74.2984f,  8.1309f,-16.9251f,74.6035f,  8.7353f,-17.6804f,76.3546f,  10.5907f,-15.5202f,76.5689f,  11.4272f,-16.3453f,78.1880f,  
	13.3428f,-14.0162f,78.2998f,  14.3928f,-14.8993f,79.7558f,  16.3563f,-12.4096f,79.7546f,  17.5959f,-13.3403f,81.0192f,  19.5926f,-10.7027f,80.8959f,  
	20.9933f,-11.6720f,81.9450f,  23.0081f,-8.9014f,81.6921f,  24.5375f,-9.9021f,82.5057f,  26.5565f,-7.0138f,82.1175f,  28.1788f,-8.0400f,82.6808f,  
	30.1872f,-5.0520f,82.1535f,  31.8645f,-6.0992f,82.4569f,  33.8485f,-3.0295f,81.7883f,  35.5417f,-4.0943f,81.8279f,  37.4868f,-0.9644f,81.0183f,  
	39.1561f,-2.0441f,80.7960f,  41.0487f,1.1249f,79.8475f,  42.6550f,0.0320f,79.3707f,  44.4831f,3.2193f,78.2871f,  45.9884f,2.1145f,77.5690f,  
	47.7402f,5.2980f,76.3563f,  49.1089f,4.1829f,75.4154f,  50.7736f,7.3402f,74.0816f,  51.9728f,6.2163f,72.9411f,  53.5410f,9.3258f,71.4960f,  
	54.5418f,8.1950f,70.1838f,  56.0058f,11.2363f,68.6382f,  56.7833f,10.1013f,67.1858f,  58.1364f,13.0539f,65.5529f,  58.6707f,11.9185f,63.9949f,  
	59.9077f,14.7630f,62.2891f,  60.1841f,13.6322f,60.6621f,  61.3017f,16.3515f,58.8987f,  61.3112f,15.2317f,57.2409f,  62.3075f,17.8098f,55.4363f,  
	62.0468f,16.7092f,53.7862f,  62.9216f,19.1322f,51.9575f,  62.3933f,18.0606f,50.3529f,  63.1480f,20.3170f,48.5179f,  62.3604f,19.2861f,46.9950f,  
	62.9979f,21.3662f,45.1718f,  61.9649f,20.3896f,43.7642f,  62.4899f,22.2874f,41.9703f,  61.2306f,21.3803f,40.7080f,  61.6493f,23.0931f,38.9597f,  
	-6.3003f,-54.6260f,37.5849f,  -5.2299f,-54.8291f,38.9008f,  -4.0381f,-53.3201f,38.3489f,  -3.0017f,-53.4294f,39.5974f,  -2.0373f,-51.7427f,39.1209f,  
	-1.0456f,-51.7683f,40.2914f,  -0.3324f,-49.9418f,39.8948f,  0.6105f,-49.8929f,40.9828f,  1.0566f,-47.9676f,40.6727f,  1.9473f,-47.8558f,41.6758f,  
	2.1189f,-45.8743f,41.4611f,  2.9580f,-45.7100f,42.3809f,  2.8563f,-43.7150f,42.2726f,  3.6474f,-43.5083f,43.1137f,  3.2827f,-41.5413f,43.1242f,  
	4.0340f,-41.2987f,43.8952f,  3.4246f,-39.3986f,44.0377f,  4.1469f,-39.1242f,44.7489f,  3.3189f,-37.3259f,45.0367f,  4.0256f,-37.0203f,45.6998f,  
	3.0119f,-35.3541f,46.1451f,  3.7199f,-35.0137f,46.7728f,  2.5590f,-33.5042f,47.3859f,  3.2840f,-33.1217f,47.9891f,  2.0182f,-31.7880f,48.7772f,  
	2.7763f,-31.3533f,49.3647f,  1.4507f,-30.2086f,50.3314f,  2.2608f,-29.7076f,50.9116f,  0.9219f,-28.7591f,52.0561f,  1.7977f,-28.1768f,52.6327f,  
	0.4913f,-27.4267f,53.9492f,  1.4462f,-26.7466f,54.5236f,  0.2166f,-26.1923f,56.0011f,  1.2609f,-25.3978f,56.5713f,  0.1491f,-25.0329f,58.1939f,  
	1.2903f,-24.1084f,58.7550f,  0.3331f,-23.9233f,60.5018f,  1.5770f,-22.8535f,61.0466f,  0.8061f,-22.8362f,62.8926f,  2.1526f,-21.6104f,63.4110f,  
	1.5940f,-21.7472f,65.3271f,  3.0414f,-20.3565f,65.8078f,  2.7147f,-20.6327f,67.7621f,  4.2617f,-19.0680f,68.1939f,  4.1799f,-19.4687f,70.1521f,  
	5.8195f,-17.7270f,70.5219f,  5.9884f,-18.2375f,72.4488f,  7.7120f,-16.3189f,72.7436f,  8.1309f,-16.9251f,74.6035f,  9.9290f,-14.8317f,74.8110f,  
	10.5907f,-15.5202f,76.5689f,  12.4520f,-13.2572f,76.6774f,  13.3428f,-14.0162f,78.2998f,  15.2556f,-11.5908f,78.2985f,  16.3563f,-12.4096f,79.7546f,  
	18.3066f,-9.8332f,79.6340f,  19.5926f,-10.7027f,80.8959f,  21.5661f,-7.9889f,80.6481f,  23.0081f,-8.9014f,81.6921f,  24.9912f,-6.0645f,81.3107f,  
	26.5565f,-7.0138f,82.1175f,  28.5347f,-4.0706f,81.5980f,  30.1872f,-5.0520f,82.1535f,  32.1468f,-2.0201f,81.4930f,  33.8485f,-3.0295f,81.7883f,  
	35.7750f,0.0705f,80.9864f,  37.4868f,-0.9644f,81.0183f,  39.3665f,2.1832f,80.0762f,  41.0487f,1.1249f,79.8475f,  42.8696f,4.2996f,78.7681f,  
	44.4831f,3.2193f,78.2871f,  46.2329f,6.3991f,77.0758f,  47.7402f,5.2980f,76.3563f,  49.4077f,8.4608f,75.0202f,  50.7736f,7.3402f,74.0816f,  
	52.3490f,10.4642f,72.6297f,  53.5410f,9.3258f,71.4960f,  55.0163f,12.3905f,69.9385f,  56.0058f,11.2363f,68.6382f,  57.3735f,14.2207f,66.9877f,  
	58.1364f,13.0539f,65.5529f,  59.3906f,15.9380f,63.8234f,  59.9077f,14.7630f,62.2891f,  61.0444f,17.5290f,60.4953f,  61.3017f,16.3515f,58.8987f,  
	62.3184f,18.9825f,57.0569f,  62.3075f,17.8098f,55.4363f,  63.2032f,20.2911f,53.5637f,  62.9216f,19.1322f,51.9575f,  63.6971f,21.4507f,50.0719f,  
	63.1480f,20.3170f,48.5179f,  63.8054f,22.4619f,46.6380f,  62.9979f,21.3662f,45.1718f,  63.5413f,23.3302f,43.3154f,  62.4899f,22.2874f,41.9703f,  
	62.9249f,24.0660f,40.1549f,  61.6493f,23.0931f,38.9597f,  61.9825f,24.6838f,37.2028f,  -5.2299f,-54.8291f,38.9008f,  -4.1244f,-54.9648f,40.2171f,  
	-3.0017f,-53.4294f,39.5974f,  -1.9351f,-53.4743f,40.8396f,  -1.0456f,-51.7683f,40.2914f,  -0.0236f,-51.7312f,41.4529f,  0.6105f,-49.8929f,40.9828f,  
	1.5821f,-49.7851f,42.0587f,  1.9473f,-47.8558f,41.6758f,  2.8670f,-47.6880f,42.6656f,  2.9580f,-45.7100f,42.3809f,  3.8267f,-45.4932f,43.2869f,  
	3.6474f,-43.5083f,43.1137f,  4.4713f,-43.2503f,43.9429f,  4.0340f,-41.2987f,43.8952f,  4.8215f,-41.0053f,44.6563f,  4.1469f,-39.1242f,44.7489f,  
	4.9097f,-38.7978f,45.4527f,  4.0256f,-37.0203f,45.6998f,  4.7790f,-36.6595f,46.3584f,  3.7199f,-35.0137f,46.7728f,  4.4787f,-34.6132f,47.3972f,  
	3.2840f,-33.1217f,47.9891f,  4.0637f,-32.6733f,48.5886f,  2.7763f,-31.3533f,49.3647f,  3.5955f,-30.8448f,49.9485f,  2.2608f,-29.7076f,50.9116f,  
	3.1337f,-29.1251f,51.4847f,  1.7977f,-28.1768f,52.6327f,  2.7383f,-27.5048f,53.1978f,  1.4462f,-26.7466f,54.5236f,  2.4659f,-25.9689f,55.0800f,  
	1.2609f,-25.3978f,56.5713f,  2.3686f,-24.4989f,57.1158f,  1.2903f,-24.1084f,58.7550f,  2.4929f,-23.0730f,59.2817f,  1.5770f,-22.8535f,61.0466f,  
	2.8760f,-21.6701f,61.5468f,  2.1526f,-21.6104f,63.4110f,  3.5478f,-20.2693f,63.8744f,  3.0414f,-20.3565f,65.8078f,  4.5331f,-18.8474f,66.2241f,  
	4.2617f,-19.0680f,68.1939f,  5.8444f,-17.3868f,68.5507f,  5.8195f,-17.7270f,70.5219f,  7.4857f,-15.8723f,70.8067f,  7.7120f,-16.3189f,72.7436f,  
	9.4534f,-14.2916f,72.9444f,  9.9290f,-14.8317f,74.8110f,  11.7355f,-12.6353f,74.9161f,  12.4520f,-13.2572f,76.6774f,  14.3128f,-10.8977f,76.6759f,  
	15.2556f,-11.5908f,78.2985f,  17.1576f,-9.0783f,78.1808f,  18.3066f,-9.8332f,79.6340f,  20.2360f,-7.1801f,79.3916f,  21.5661f,-7.9889f,80.6481f,  
	23.5099f,-5.2082f,80.2742f,  24.9912f,-6.0645f,81.3107f,  26.9354f,-3.1720f,80.8001f,  28.5347f,-4.0706f,81.5980f,  30.4657f,-1.0831f,80.9472f,  
	32.1468f,-2.0201f,81.4930f,  34.0502f,1.0428f,80.7007f,  35.7750f,0.0705f,80.9864f,  37.6373f,3.1888f,80.0530f,  39.3665f,2.1832f,80.0762f,  
	41.1755f,5.3372f,79.0039f,  42.8696f,4.2996f,78.7681f,  44.6127f,7.4674f,77.5614f,  46.2329f,6.3991f,77.0758f,  47.8988f,9.5588f,75.7409f,  
	49.4077f,8.4608f,75.0202f,  50.9863f,11.5909f,73.5654f,  52.3490f,10.4642f,72.6297f,  53.8316f,13.5442f,71.0644f,  55.0163f,12.3905f,69.9385f,  
	56.3948f,15.3988f,68.2747f,  57.3735f,14.2207f,66.9877f,  58.6416f,17.1372f,65.2387f,  59.3906f,15.9380f,63.8234f,  60.5437f,18.7443f,62.0035f,  
	61.0444f,17.5290f,60.4953f,  62.0793f,20.2074f,58.6205f,  62.3184f,18.9825f,57.0569f,  63.2333f,21.5173f,55.1440f,  63.2032f,20.2911f,53.5637f,  
	63.9982f,22.6681f,51.6307f,  63.6971f,21.4507f,50.0719f,  64.3736f,23.6580f,48.1375f,  63.8054f,22.4619f,46.6380f,  64.3668f,24.4907f,44.7204f,  
	63.5413f,23.3302f,43.3154f,  63.9926f,25.1743f,41.4331f,  62.9249f,24.0660f,40.1549f,  63.2721f,25.7216f,38.3262f,  61.9825f,24.6838f,37.2028f,  
	62.2333f,26.1518f,35.4438f,  -4.1244f,-54.9648f,40.2171f,  -2.9854f,-55.0307f,41.5299f,  -1.9351f,-53.4743f,40.8396f,  -0.8348f,-53.4508f,42.0748f,  
	-0.0236f,-51.7312f,41.4529f,  1.0298f,-51.6292f,42.6033f,  1.5821f,-49.7851f,42.0587f,  2.5849f,-49.6154f,43.1216f,  2.8670f,-47.6880f,42.6656f,  
	3.8175f,-47.4622f,43.6410f,  3.8267f,-45.4932f,43.2869f,  4.7288f,-45.2199f,44.1799f,  4.4713f,-43.2503f,43.9429f,  5.3312f,-42.9370f,44.7604f,  
	4.8215f,-41.0053f,44.6563f,  5.6490f,-40.6561f,45.4080f,  4.9097f,-38.7978f,45.4527f,  5.7179f,-38.4137f,46.1498f,  4.7790f,-36.6595f,46.3584f,  
	5.5819f,-36.2369f,47.0117f,  4.4787f,-34.6132f,47.3972f,  5.2910f,-34.1458f,48.0164f,  4.0637f,-32.6733f,48.5886f,  4.9032f,-32.1511f,49.1832f,  
	3.5955f,-30.8448f,49.9485f,  4.4766f,-30.2552f,50.5247f,  3.1337f,-29.1251f,51.4847f,  4.0708f,-28.4538f,52.0463f,  2.7383f,-27.5048f,53.1978f,  
	3.7436f,-26.7365f,53.7457f,  2.4659f,-25.9689f,55.0800f,  3.5495f,-25.0883f,55.6123f,  2.3686f,-24.4989f,57.1158f,  3.5382f,-23.4908f,57.6279f,  
	2.4929f,-23.0730f,59.2817f,  3.7518f,-21.9253f,59.7664f,  2.8760f,-21.6701f,61.5468f,  4.2253f,-20.3726f,61.9949f,  3.5478f,-20.2693f,63.8744f,  
	4.9892f,-18.8108f,64.2766f,  4.5331f,-18.8474f,66.2241f,  6.0626f,-17.2226f,66.5689f,  5.8444f,-17.3868f,68.5507f,  7.4559f,-15.5930f,68.8260f,  
	7.4857f,-15.8723f,70.8067f,  9.1719f,-13.9092f,71.0010f,  9.4534f,-14.2916f,72.9444f,  11.2056f,-12.1613f,73.0462f,  11.7355f,-12.6353f,74.9161f,  
	13.5439f,-10.3424f,74.9145f,  14.3128f,-10.8977f,76.6759f,  16.1650f,-8.4511f,76.5611f,  17.1576f,-9.0783f,78.1808f,  19.0407f,-6.4889f,77.9440f,  
	20.2360f,-7.1801f,79.3916f,  22.1377f,-4.4596f,79.0256f,  23.5099f,-5.2082f,80.2742f,  25.4166f,-2.3713f,79.7732f,  26.9354f,-3.1720f,80.8001f,  
	28.8339f,-0.2347f,80.1601f,  30.4657f,-1.0831f,80.9472f,  32.3417f,1.9359f,80.1662f,  34.0502f,1.0428f,80.7007f,  35.8905f,4.1245f,79.7783f,  
	37.6373f,3.1888f,80.0530f,  39.4297f,6.3141f,78.9905f,  41.1755f,5.3372f,79.0039f,  42.9074f,8.4846f,77.8049f,  44.6127f,7.4674f,77.5614f,  
	46.2724f,10.6155f,76.2314f,  47.8988f,9.5588f,75.7409f,  49.4760f,12.6864f,74.2873f,  50.9863f,11.5909f,73.5654f,  52.4720f,14.6775f,71.9967f,  
	53.8316f,13.5442f,71.0644f,  55.2173f,16.5682f,69.3919f,  56.3948f,15.3988f,68.2747f,  57.6736f,18.3399f,66.5109f,  58.6416f,17.1372f,65.2387f,  
	59.8082f,19.9764f,63.3974f,  60.5437f,18.7443f,62.0035f,  61.5942f,21.4634f,60.1001f,  62.0793f,20.2074f,58.6205f,  63.0114f,22.7900f,56.6715f,  
	63.2333f,21.5173f,55.1440f,  64.0462f,23.9481f,53.1673f,  63.9982f,22.6681f,51.6307f,  64.6926f,24.9340f,49.6449f,  64.3736f,23.6580f,48.1375f,  
	64.9523f,25.7491f,46.1611f,  64.3668f,24.4907f,44.7204f,  64.8340f,26.3991f,42.7724f,  63.9926f,25.1743f,41.4331f,  64.3537f,26.8948f,39.5327f,  
	63.2721f,25.7216f,38.3262f,  63.5345f,27.2532f,36.4907f,  62.2333f,26.1518f,35.4438f,  62.4051f,27.4953f,33.6908f,  -2.9854f,-55.0307f,41.5299f,  
	-1.8092f,-55.0225f,42.8379f,  -0.8348f,-53.4508f,42.0748f,  0.2996f,-53.3564f,43.3002f,  1.0298f,-51.6292f,42.6033f,  2.1166f,-51.4589f,43.7413f,  
	2.5849f,-49.6154f,43.1216f,  3.6199f,-49.3814f,44.1698f,  3.8175f,-47.4622f,43.6410f,  4.8023f,-47.1740f,44.6024f,  4.7288f,-45.2199f,44.1799f,  
	5.6673f,-44.8862f,45.0596f,  5.3312f,-42.9370f,44.7604f,  6.2308f,-42.5636f,45.5665f,  5.6490f,-40.6561f,45.4080f,  6.5208f,-40.2460f,46.1506f,  
	5.7179f,-38.4137f,46.1498f,  6.5747f,-37.9651f,46.8393f,  5.5819f,-36.2369f,47.0117f,  6.4372f,-35.7458f,47.6578f,  5.2910f,-34.1458f,48.0164f,  
	6.1615f,-33.6041f,48.6289f,  4.9032f,-32.1511f,49.1832f,  5.8034f,-31.5478f,49.7692f,  4.4766f,-30.2552f,50.5247f,  5.4209f,-29.5775f,51.0889f,  
	4.0708f,-28.4538f,52.0463f,  5.0721f,-27.6874f,52.5910f,  3.7436f,-26.7365f,53.7457f,  4.8127f,-25.8664f,54.2704f,  3.5495f,-25.0883f,55.6123f,  
	4.6951f,-24.0999f,56.1137f,  3.5382f,-23.4908f,57.6279f,  4.7652f,-22.3716f,58.1002f,  3.7518f,-21.9253f,59.7664f,  5.0622f,-20.6647f,60.2015f,  
	4.2253f,-20.3726f,61.9949f,  5.6223f,-18.9588f,62.3847f,  4.9892f,-18.8108f,64.2766f,  6.4703f,-17.2372f,64.6104f,  6.0626f,-17.2226f,66.5689f,  
	7.6231f,-15.4855f,66.8354f,  7.4559f,-15.5930f,68.8260f,  9.0896f,-13.6911f,69.0141f,  9.1719f,-13.9092f,71.0010f,  10.8715f,-11.8431f,71.0996f,  
	11.2056f,-12.1613f,73.0462f,  12.9621f,-9.9343f,73.0445f,  13.5439f,-10.3424f,74.9145f,  15.3459f,-7.9622f,74.8026f,  16.1650f,-8.4511f,76.5611f,  
	18.0005f,-5.9271f,76.3301f,  19.0407f,-6.4889f,77.9440f,  20.8983f,-3.8315f,77.5864f,  22.1377f,-4.4596f,79.0256f,  24.0044f,-1.6824f,78.5351f,  
	25.4166f,-2.3713f,79.7732f,  27.2793f,0.5106f,79.1452f,  28.8339f,-0.2347f,80.1601f,  30.6789f,2.7345f,79.3919f,  32.3417f,1.9359f,80.1662f,  
	34.1562f,4.9742f,79.2567f,  35.8905f,4.1245f,79.7783f,  37.6623f,7.2137f,78.7279f,  39.4297f,6.3141f,78.9905f,  41.1461f,9.4333f,77.8022f,  
	42.9074f,8.4846f,77.8049f,  44.5566f,11.6128f,76.4833f,  46.2724f,10.6155f,76.2314f,  47.8442f,13.7322f,74.7828f,  49.4760f,12.6864f,74.2873f,  
	50.9609f,15.7712f,72.7194f,  52.4720f,14.6775f,71.9967f,  53.8610f,17.7089f,70.3201f,  55.2173f,16.5682f,69.3919f,  56.5031f,19.5256f,67.6182f,  
	57.6736f,18.3399f,66.5109f,  58.8505f,21.2044f,64.6532f,  59.8082f,19.9764f,63.3974f,  60.8716f,22.7291f,61.4705f,  61.5942f,21.4634f,60.1001f,  
	62.5412f,24.0874f,58.1199f,  63.0114f,22.7900f,56.6715f,  63.8403f,25.2689f,54.6556f,  64.0462f,23.9481f,53.1673f,  64.7572f,26.2681f,51.1342f,  
	64.6926f,24.9340f,49.6449f,  65.2878f,27.0839f,47.6129f,  64.9523f,25.7491f,46.1611f,  65.4349f,27.7194f,44.1500f,  64.8340f,26.3991f,42.7724f,  
	65.2090f,28.1832f,40.8018f,  64.3537f,26.8948f,39.5327f,  64.6277f,28.4899f,37.6209f,  63.5345f,27.2532f,36.4907f,  63.7152f,28.6587f,34.6564f,  
	62.4051f,27.4953f,33.6908f,  62.5016f,28.7152f,31.9495f,  -1.8092f,-55.0225f,42.8379f,  -0.5959f,-54.9371f,44.1376f,  0.2996f,-53.3564f,43.3002f,  
	1.4700f,-53.1872f,44.5140f,  2.1166f,-51.4589f,43.7413f,  3.2374f,-51.2181f,44.8643f,  3.6199f,-49.3814f,44.1698f,  4.6905f,-49.0790f,45.2029f,  
	4.8023f,-47.1740f,44.6024f,  5.8240f,-46.8197f,45.5486f,  5.6673f,-44.8862f,45.0596f,  6.6453f,-44.4876f,45.9255f,  6.2308f,-42.5636f,45.5665f,  
	7.1739f,-42.1252f,46.3607f,  6.5208f,-40.2460f,46.1506f,  7.4401f,-39.7682f,46.8829f,  6.5747f,-37.9651f,46.8393f,  7.4826f,-37.4458f,47.5191f,  
	6.4372f,-35.7458f,47.6578f,  7.3490f,-35.1791f,48.2950f,  6.1615f,-33.6041f,48.6289f,  7.0914f,-32.9809f,49.2310f,  5.8034f,-31.5478f,49.7692f,  
	6.7656f,-30.8567f,50.3420f,  5.4209f,-29.5775f,51.0889f,  6.4286f,-28.8053f,51.6359f,  5.0721f,-27.6874f,52.5910f,  6.1367f,-26.8199f,53.1128f,  
	4.8127f,-25.8664f,54.2704f,  5.9437f,-24.8898f,54.7650f,  4.6951f,-24.0999f,56.1137f,  5.8986f,-23.0014f,56.5768f,  4.7652f,-22.3716f,58.1002f,  
	6.0443f,-21.1406f,58.5249f,  5.0622f,-20.6647f,60.2015f,  6.4215f,-19.2888f,60.5807f,  5.6223f,-18.9588f,62.3847f,  7.0607f,-17.4305f,62.7088f,  
	6.4703f,-17.2372f,64.6104f,  7.9843f,-15.5519f,64.8689f,  7.6231f,-15.4855f,66.8354f,  9.2076f,-13.6409f,67.0177f,  9.0896f,-13.6911f,69.0141f,  
	10.7388f,-11.6863f,69.1096f,  10.8715f,-11.8431f,71.0996f,  12.5773f,-9.6804f,71.0978f,  12.9621f,-9.9343f,73.0445f,  14.7142f,-7.6199f,72.9355f,  
	15.3459f,-7.9622f,74.8026f,  17.1334f,-5.5043f,74.5776f,  18.0005f,-5.9271f,76.3301f,  19.8127f,-3.3347f,75.9811f,  20.8983f,-3.8315f,77.5864f,  
	22.7227f,-1.1171f,77.1068f,  24.0044f,-1.6824f,78.5351f,  25.8286f,1.1401f,77.9200f,  27.2793f,0.5106f,79.1452f,  29.0902f,3.4249f,78.3912f,  
	30.6789f,2.7345f,79.3919f,  32.4639f,5.7235f,78.4971f,  34.1562f,4.9742f,79.2567f,  35.9033f,8.0206f,78.2208f,  37.6623f,7.2137f,78.7279f,  
	39.3590f,10.2972f,77.5532f,  41.1461f,9.4333f,77.8022f,  42.7806f,12.5336f,76.4923f,  44.5566f,11.6128f,76.4833f,  46.1186f,14.7102f,75.0436f,  
	47.8442f,13.7322f,74.7828f,  49.3240f,16.8066f,73.2202f,  50.9609f,15.7712f,72.7194f,  52.3492f,18.8012f,71.0435f,  53.8610f,17.7089f,70.3201f,  
	55.1502f,20.6741f,68.5417f,  56.5031f,19.5256f,67.6182f,  57.6870f,22.4071f,65.7494f,  58.8505f,21.2044f,64.6532f,  59.9238f,23.9828f,62.7082f,  
	60.8716f,22.7291f,61.4705f,  61.8308f,25.3872f,59.4645f,  62.5412f,24.0874f,58.1199f,  63.3840f,26.6080f,56.0701f,  63.8403f,25.2689f,54.6556f,  
	64.5661f,27.6374f,52.5801f,  64.7572f,26.2681f,51.1342f,  65.3674f,28.4722f,49.0511f,  65.2878f,27.0839f,47.6129f,  65.7848f,29.1127f,45.5423f,  
	65.4349f,27.7194f,44.1500f,  65.8231f,29.5649f,42.1117f,  65.2090f,28.1832f,40.8018f,  65.4944f,29.8409f,38.8151f,  64.6277f,28.4899f,37.6209f,  
	64.8174f,29.9575f,35.7053f,  63.7152f,28.6587f,34.6564f,  63.8178f,29.9384f,32.8291f,  62.5016f,28.7152f,31.9495f,  62.5267f,29.8131f,30.2254f,  
	-0.5959f,-54.9371f,44.1376f,  0.6563f,-54.7706f,45.4265f,  1.4700f,-53.1872f,44.5140f,  2.6765f,-52.9408f,45.7127f,  3.2374f,-51.2181f,44.8643f,  
	4.3952f,-50.9023f,45.9713f,  4.6905f,-49.0790f,45.2029f,  5.7986f,-48.7044f,46.2192f,  5.8240f,-46.8197f,45.5486f,  6.8852f,-46.3950f,46.4787f,  
	6.6453f,-44.4876f,45.9255f,  7.6659f,-44.0192f,46.7765f,  7.1739f,-42.1252f,46.3607f,  8.1634f,-41.6154f,47.1416f,  7.4401f,-39.7682f,46.8829f,  
	8.4093f,-39.2168f,47.6026f,  7.4826f,-37.4458f,47.5191f,  8.4451f,-36.8491f,48.1872f,  7.3490f,-35.1791f,48.2950f,  8.3185f,-34.5298f,48.9195f,  
	7.0914f,-32.9809f,49.2310f,  8.0817f,-32.2693f,49.8183f,  6.7656f,-30.8567f,50.3420f,  7.7899f,-30.0711f,50.8965f,  6.4286f,-28.8053f,51.6359f,  
	7.4988f,-27.9326f,52.1595f,  6.1367f,-26.8199f,53.1128f,  7.2627f,-25.8470f,53.6047f,  5.9437f,-24.8898f,54.7650f,  7.1327f,-23.8040f,55.2222f,  
	5.8986f,-23.0014f,56.5768f,  7.1548f,-21.7921f,56.9937f,  6.0443f,-21.1406f,58.5249f,  7.3733f,-19.7952f,58.8955f,  6.4215f,-19.2888f,60.5807f,  
	7.8236f,-17.7990f,60.8965f,  7.0607f,-17.4305f,62.7088f,  8.5336f,-15.7910f,62.9602f,  7.9843f,-15.5519f,64.8689f,  9.5241f,-13.7595f,65.0460f,  
	9.2076f,-13.6409f,67.0177f,  10.8097f,-11.6934f,67.1104f,  10.7388f,-11.6863f,69.1096f,  12.3962f,-9.5850f,69.1078f,  12.5773f,-9.6804f,71.0978f,  
	14.2809f,-7.4303f,70.9917f,  14.7142f,-7.6199f,72.9355f,  16.4541f,-5.2277f,72.7164f,  17.1334f,-5.5043f,74.5776f,  18.8996f,-2.9777f,74.2373f,  
	19.8127f,-3.3347f,75.9811f,  21.5937f,-0.6849f,75.5129f,  22.7227f,-1.1171f,77.1068f,  24.5066f,1.6430f,76.5052f,  25.8286f,1.1401f,77.9200f,  
	27.6028f,3.9954f,77.1811f,  29.0902f,3.4249f,78.3912f,  30.8426f,6.3594f,77.5126f,  32.4639f,5.7235f,78.4971f,  34.1830f,8.7210f,77.4779f,  
	35.9033f,8.0206f,78.2208f,  37.5766f,11.0616f,77.0623f,  39.3590f,10.2972f,77.5532f,  40.9749f,13.3623f,76.2582f,  42.7806f,12.5336f,76.4923f,  
	44.3288f,15.6037f,75.0652f,  46.1186f,14.7102f,75.0436f,  47.5894f,17.7658f,73.4905f,  49.3240f,16.8066f,73.2202f,  50.7077f,19.8266f,71.5497f,  
	52.3492f,18.8012f,71.0435f,  53.6380f,21.7657f,69.2654f,  55.1502f,20.6741f,68.5417f,  56.3375f,23.5640f,66.6673f,  57.6870f,22.4071f,65.7494f,  
	58.7669f,25.2031f,63.7920f,  59.9238f,23.9828f,62.7082f,  60.8924f,26.6672f,60.6823f,  61.8308f,25.3872f,59.4645f,  62.6851f,27.9423f,57.3866f,  
	63.3840f,26.6080f,56.0701f,  64.1227f,29.0186f,53.9577f,  64.5661f,27.6374f,52.5801f,  65.1899f,29.8904f,50.4509f,  65.3674f,28.4722f,49.0511f,  
	65.8779f,30.5551f,46.9254f,  65.7848f,29.1127f,45.5423f,  66.1857f,31.0162f,43.4400f,  65.8231f,29.5649f,42.1117f,  66.1196f,31.2829f,40.0527f,  
	65.4944f,29.8409f,38.8151f,  65.6930f,31.3696f,36.8195f,  64.8174f,29.9575f,35.7053f,  64.9262f,31.2972f,33.7920f,  63.8178f,29.9384f,32.8291f,  
	63.8461f,31.0933f,31.0144f,  62.5267f,29.8131f,30.2254f,  62.4844f,30.7895f,28.5255f,  0.6563f,-54.7706f,45.4265f,  1.9467f,-54.5201f,46.7003f,  
	2.6765f,-52.9408f,45.7127f,  3.9216f,-52.6128f,46.8944f,  4.3952f,-50.9023f,45.9713f,  5.5916f,-50.5079f,47.0597f,  5.7986f,-48.7044f,46.2192f,  
	6.9466f,-48.2534f,47.2169f,  6.8852f,-46.3950f,46.4787f,  7.9884f,-45.8955f,47.3910f,  7.6659f,-44.0192f,46.7765f,  8.7321f,-43.4750f,47.6112f,  
	8.1634f,-41.6154f,47.1416f,  9.2016f,-41.0284f,47.9068f,  8.4093f,-39.2168f,47.6026f,  9.4313f,-38.5855f,48.3074f,  8.4451f,-36.8491f,48.1872f,  
	9.4638f,-36.1681f,48.8399f,  8.3185f,-34.5298f,48.9195f,  9.3468f,-33.7914f,49.5268f,  8.0817f,-32.2693f,49.8183f,  9.1327f,-31.4632f,50.3856f,  
	7.7899f,-30.0711f,50.8965f,  8.8757f,-29.1853f,51.4267f,  7.4988f,-27.9326f,52.1595f,  8.6295f,-26.9551f,52.6528f,  7.2627f,-25.8470f,53.6047f,  
	8.4465f,-24.7655f,54.0596f,  7.1327f,-23.8040f,55.2222f,  8.3746f,-22.6081f,55.6342f,  7.1548f,-21.7921f,56.9937f,  8.4612f,-20.4693f,57.3580f,  
	7.3733f,-19.7952f,58.8955f,  8.7459f,-18.3366f,59.2046f,  7.8236f,-17.7990f,60.8965f,  9.2613f,-16.1987f,61.1418f,  8.5336f,-15.7910f,62.9602f,  
	10.0336f,-14.0450f,63.1326f,  9.5241f,-13.7595f,65.0460f,  11.0831f,-11.8645f,65.1360f,  10.8097f,-11.6934f,67.1104f,  12.4219f,-9.6497f,67.1085f,  
	12.3962f,-9.5850f,69.1078f,  14.0533f,-7.3964f,69.0045f,  14.2809f,-7.4303f,70.9917f,  15.9742f,-5.1021f,70.7783f,  16.4541f,-5.2277f,72.7164f,  
	18.1746f,-2.7665f,72.3848f,  18.8996f,-2.9777f,74.2373f,  20.6366f,-0.3934f,73.7806f,  21.5937f,-0.6849f,75.5129f,  23.3359f,2.0107f,74.9253f,  
	24.5066f,1.6430f,76.5052f,  26.2421f,4.4361f,75.7824f,  27.6028f,3.9954f,77.1811f,  29.3202f,6.8712f,76.3201f,  30.8426f,6.3594f,77.5126f,  
	32.5307f,9.3029f,76.5117f,  34.1830f,8.7210f,77.4779f,  35.8295f,11.7134f,76.3377f,  37.5766f,11.0616f,77.0623f,  39.1703f,14.0845f,75.7850f,  
	40.9749f,13.3623f,76.2582f,  42.5055f,16.3975f,74.8472f,  44.3288f,15.6037f,75.0652f,  45.7867f,18.6325f,73.5257f,  47.5894f,17.7658f,73.4905f,  
	48.9647f,20.7674f,71.8301f,  50.7077f,19.8266f,71.5497f,  51.9924f,22.7816f,69.7772f,  53.6380f,21.7657f,69.2654f,  54.8251f,24.6553f,67.3911f,  
	56.3375f,23.5640f,66.6673f,  57.4208f,26.3690f,64.7035f,  58.7669f,25.2031f,63.7920f,  59.7420f,27.9055f,61.7524f,  60.8924f,26.6672f,60.6823f,  
	61.7555f,29.2492f,58.5826f,  62.6851f,27.9423f,57.3866f,  63.4345f,30.3880f,55.2435f,  64.1227f,29.0186f,53.9577f,  64.7583f,31.3140f,51.7885f,  
	65.1899f,29.8904f,50.4509f,  65.7126f,32.0218f,48.2757f,  65.8779f,30.5551f,46.9254f,  66.2905f,32.5123f,44.7640f,  66.1857f,31.0162f,43.4400f,  
	66.4927f,32.7914f,41.3127f,  66.1196f,31.2829f,40.0527f,  66.3269f,32.8708f,37.9800f,  65.6930f,31.3696f,36.8195f,  65.8080f,32.7684f,34.8216f,  
	64.9262f,31.2972f,33.7920f,  64.9579f,32.5095f,31.8865f,  63.8461f,31.0933f,31.0144f,  63.8039f,32.1236f,29.2193f,  62.4844f,30.7895f,28.5255f,  
	62.3788f,31.6473f,26.8542f,  1.9467f,-54.5201f,46.7003f,  3.2778f,-54.1811f,47.9563f,  3.9216f,-52.6128f,46.8944f,  5.2065f,-52.1995f,48.0559f,  
	5.5916f,-50.5079f,47.0597f,  6.8283f,-50.0308f,48.1272f,  6.9466f,-48.2534f,47.2169f,  8.1361f,-47.7220f,48.1936f,  7.9884f,-45.8955f,47.3910f,  
	9.1365f,-45.3153f,48.2835f,  8.7321f,-43.4750f,47.6112f,  9.8458f,-42.8496f,48.4266f,  9.2016f,-41.0284f,47.9068f,  10.2907f,-40.3586f,48.6535f,  
	9.4313f,-38.5855f,48.3074f,  10.5076f,-37.8678f,48.9935f,  9.4638f,-36.1681f,48.8399f,  10.5393f,-35.3967f,49.4725f,  9.3468f,-33.7914f,49.5268f,  
	10.4341f,-32.9577f,50.1119f,  9.1327f,-31.4632f,50.3856f,  10.2438f,-30.5566f,50.9270f,  8.8757f,-29.1853f,51.4267f,  10.0207f,-28.1952f,51.9256f,  
	8.6295f,-26.9551f,52.6528f,  9.8173f,-25.8696f,53.1089f,  8.4465f,-24.7655f,54.0596f,  9.6829f,-23.5746f,54.4696f,  8.3746f,-22.6081f,55.6342f,  
	9.6668f,-21.2996f,55.9944f,  8.4612f,-20.4693f,57.3580f,  9.8119f,-19.0340f,57.6621f,  8.7459f,-18.3366f,59.2046f,  10.1552f,-16.7680f,59.4451f,  
	9.2613f,-16.1987f,61.1418f,  10.7273f,-14.4923f,61.3103f,  10.0336f,-14.0450f,63.1326f,  11.5544f,-12.1965f,63.2204f,  11.0831f,-11.8645f,65.1360f,  
	12.6538f,-9.8735f,65.1341f,  12.4219f,-9.6497f,67.1085f,  14.0353f,-7.5188f,67.0078f,  14.0533f,-7.3964f,69.0045f,  15.7020f,-5.1296f,68.7966f,  
	15.9742f,-5.1021f,70.7783f,  17.6501f,-2.7048f,70.4552f,  18.1746f,-2.7665f,72.3848f,  19.8678f,-0.2474f,71.9395f,  20.6366f,-0.3934f,73.7806f,  
	22.3365f,2.2369f,73.2071f,  23.3359f,2.0107f,74.9253f,  25.0314f,4.7395f,74.2189f,  26.2421f,4.4361f,75.7824f,  27.9225f,7.2501f,74.9398f,  
	29.3202f,6.8712f,76.3201f,  30.9747f,9.7563f,75.3389f,  32.5307f,9.3029f,76.5117f,  34.1475f,12.2414f,75.3921f,  35.8295f,11.7134f,76.3377f,  
	37.3977f,14.6879f,75.0808f,  39.1703f,14.0845f,75.7850f,  40.6799f,17.0780f,74.3934f,  42.5055f,16.3975f,74.8472f,  43.9467f,19.3918f,73.3251f,  
	45.7867f,18.6325f,73.5257f,  47.1498f,21.6076f,71.8798f,  48.9647f,20.7674f,71.8301f,  50.2415f,23.7046f,70.0681f,  51.9924f,22.7816f,69.7772f,  
	53.1758f,25.6624f,67.9085f,  54.8251f,24.6553f,67.3911f,  55.9085f,27.4606f,65.4270f,  57.4208f,26.3690f,64.7035f,  58.3993f,29.0810f,62.6566f,  
	59.7420f,27.9055f,61.7524f,  60.6113f,30.5061f,59.6375f,  61.7555f,29.2492f,58.5826f,  62.5134f,31.7221f,56.4155f,  63.4345f,30.3880f,55.2435f,  
	64.0800f,32.7188f,53.0409f,  64.7583f,31.3140f,51.7885f,  65.2916f,33.4879f,49.5701f,  65.7126f,32.0218f,48.2757f,  66.1357f,34.0275f,46.0610f,  
	66.2905f,32.5123f,44.7640f,  66.6073f,34.3404f,42.5734f,  66.4927f,32.7914f,41.3127f,  66.7083f,34.4353f,39.1670f,  66.3269f,32.8708f,37.9800f,  
	66.4480f,34.3268f,35.9004f,  65.8080f,32.7684f,34.8216f,  65.8431f,34.0374f,32.8268f,  64.9579f,32.5095f,31.8865f,  64.9160f,33.5940f,29.9960f,  
	63.8039f,32.1236f,29.2193f,  63.6953f,33.0314f,27.4485f,  62.3788f,31.6473f,26.8542f,  62.2141f,32.3898f,25.2155f,  3.2778f,-54.1811f,47.9563f,  
	4.6500f,-53.7501f,49.1907f,  5.2065f,-52.1995f,48.0559f,  6.5324f,-51.6970f,49.1942f,  6.8283f,-50.0308f,48.1272f,  8.1062f,-49.4675f,49.1706f,  
	8.1361f,-47.7220f,48.1936f,  9.3697f,-47.1047f,49.1470f,  9.1365f,-45.3153f,48.2835f,  10.3309f,-44.6493f,49.1531f,  9.8458f,-42.8496f,48.4266f,  
	11.0085f,-42.1381f,49.2195f,  10.2907f,-40.3586f,48.6535f,  11.4322f,-39.5997f,49.3778f,  10.5076f,-37.8678f,48.9935f,  11.6387f,-37.0579f,49.6563f,  
	10.5393f,-35.3967f,49.4725f,  11.6718f,-34.5290f,50.0801f,  10.4341f,-32.9577f,50.1119f,  11.5798f,-32.0231f,50.6689f,  10.2438f,-30.5566f,50.9270f,  
	11.4125f,-29.5460f,51.4354f,  10.0207f,-28.1952f,51.9256f,  11.2219f,-27.0973f,52.3863f,  9.8173f,-25.8696f,53.1089f,  11.0573f,-24.6751f,53.5198f,  
	9.6829f,-23.5746f,54.4696f,  10.9693f,-22.2718f,54.8280f,  9.6668f,-21.2996f,55.9944f,  11.0034f,-19.8792f,56.2953f,  9.8119f,-19.0340f,57.6621f,  
	11.1998f,-17.4891f,57.8988f,  10.1552f,-16.7680f,59.4451f,  11.5935f,-15.0939f,59.6103f,  10.7273f,-14.4923f,61.3103f,  12.2153f,-12.6839f,61.3961f,  
	11.5544f,-12.1965f,63.2204f,  13.0880f,-10.2526f,63.2184f,  12.6538f,-9.8735f,65.1341f,  14.2273f,-7.7956f,65.0358f,  14.0353f,-7.5188f,67.0078f,  
	15.6421f,-5.3097f,66.8051f,  15.7020f,-5.1296f,68.7966f,  17.3350f,-2.7936f,68.4817f,  17.6501f,-2.7048f,70.4552f,  19.3004f,-0.2495f,70.0211f,  
	19.8678f,-0.2474f,71.9395f,  21.5255f,2.3177f,71.3801f,  22.3365f,2.2369f,73.2071f,  23.9915f,4.9005f,72.5176f,  25.0314f,4.7395f,74.2189f,  
	26.6736f,7.4895f,73.3955f,  27.9225f,7.2501f,74.9398f,  29.5417f,10.0735f,73.9796f,  30.9747f,9.7563f,75.3389f,  32.5595f,12.6366f,74.2414f,  
	34.1475f,12.2414f,75.3921f,  35.6874f,15.1623f,74.1578f,  37.3977f,14.6879f,75.0808f,  38.8831f,17.6335f,73.7114f,  40.6799f,17.0780f,74.3934f,  
	42.1011f,20.0308f,72.8922f,  43.9467f,19.3918f,73.3251f,  45.2943f,22.3328f,71.6979f,  47.1498f,21.6076f,71.8798f,  48.4153f,24.5188f,70.1333f,  
	50.2415f,23.7046f,70.0681f,  51.4177f,26.5680f,68.2106f,  53.1758f,25.6624f,67.9085f,  54.2559f,28.4593f,65.9501f,  55.9085f,27.4606f,65.4270f,  
	56.8872f,30.1735f,63.3793f,  58.3993f,29.0810f,62.6566f,  59.2719f,31.6916f,60.5334f,  60.6113f,30.5061f,59.6375f,  61.3750f,32.9982f,57.4537f,  
	62.5134f,31.7221f,56.4155f,  63.1666f,34.0807f,54.1867f,  64.0800f,32.7188f,53.0409f,  64.6221f,34.9282f,50.7865f,  65.2916f,33.4879f,49.5701f,  
	65.7240f,35.5360f,47.3088f,  66.1357f,34.0275f,46.0610f,  66.4614f,35.9036f,43.8133f,  66.6073f,34.3404f,42.5734f,  66.8306f,36.0365f,40.3600f,  
	66.7083f,34.4353f,39.1670f,  66.8353f,35.9459f,37.0098f,  66.4480f,34.3268f,35.9004f,  66.4865f,35.6508f,33.8192f,  65.8431f,34.0374f,32.8268f,  
	65.8016f,35.1756f,30.8423f,  64.9160f,33.5940f,29.9960f,  64.8046f,34.5523f,28.1255f,  63.6953f,33.0314f,27.4485f,  63.5244f,33.8193f,25.7069f,  
	62.2141f,32.3898f,25.2155f,  61.9945f,33.0201f,23.6137f,  4.6500f,-53.7501f,49.1907f,  6.0640f,-53.2233f,50.3996f,  6.5324f,-51.6970f,49.1942f,  
	7.8993f,-51.1021f,50.3052f,  8.1062f,-49.4675f,49.1706f,  9.4277f,-48.8127f,50.1871f,  9.3697f,-47.1047f,49.1470f,  10.6487f,-46.3968f,50.0738f,  
	10.3309f,-44.6493f,49.1531f,  11.5724f,-43.8934f,49.9959f,  11.0085f,-42.1381f,49.2195f,  12.2216f,-41.3343f,49.9862f,  11.4322f,-39.5997f,49.3778f,  
	12.6262f,-38.7466f,50.0751f,  11.6387f,-37.0579f,49.6563f,  12.8246f,-36.1504f,50.2908f,  11.6718f,-34.5290f,50.0801f,  12.8608f,-33.5597f,50.6569f,  
	11.5798f,-32.0231f,50.6689f,  12.7811f,-30.9845f,51.1908f,  11.4125f,-29.5460f,51.4354f,  12.6361f,-28.4276f,51.9043f,  11.2219f,-27.0973f,52.3863f,  
	12.4743f,-25.8908f,52.8011f,  11.0573f,-24.6751f,53.5198f,  12.3464f,-23.3694f,53.8789f,  10.9693f,-22.2718f,54.8280f,  12.2998f,-20.8578f,55.1275f,  
	11.0034f,-19.8792f,56.2953f,  12.3774f,-18.3498f,56.5297f,  11.1998f,-17.4891f,57.8988f,  12.6173f,-15.8394f,58.0617f,  11.5935f,-15.0939f,59.6103f,  
	13.0547f,-13.3181f,59.6946f,  12.2153f,-12.6839f,61.3961f,  13.7172f,-10.7803f,61.3942f,  13.0880f,-10.2526f,63.2184f,  14.6258f,-8.2219f,63.1223f,  
	14.2273f,-7.7956f,65.0358f,  15.7956f,-5.6395f,64.8379f,  15.6421f,-5.3097f,66.8051f,  17.2349f,-3.0315f,66.4978f,  17.3350f,-2.7936f,68.4817f,  
	18.9441f,-0.3996f,68.0583f,  19.3004f,-0.2495f,70.0211f,  20.9168f,2.2517f,69.4755f,  21.5255f,2.3177f,71.3801f,  23.1401f,4.9162f,70.7073f,  
	23.9915f,4.9005f,72.5176f,  25.5948f,7.5855f,71.7135f,  26.6736f,7.4895f,73.3955f,  28.2561f,10.2491f,72.4569f,  29.5417f,10.0735f,73.9796f,  
	31.0926f,12.8923f,72.9054f,  32.5595f,12.6366f,74.2414f,  34.0688f,15.4995f,73.0315f,  35.6874f,15.1623f,74.1578f,  37.1459f,18.0545f,72.8131f,  
	38.8831f,17.6335f,73.7114f,  40.2813f,20.5386f,72.2345f,  42.1011f,20.0308f,72.8922f,  43.4297f,22.9307f,71.2877f,  45.2943f,22.3328f,71.6979f,  
	46.5451f,25.2103f,69.9716f,  48.4153f,24.5188f,70.1333f,  49.5810f,27.3565f,68.2922f,  51.4177f,26.5680f,68.2106f,  52.4913f,29.3479f,66.2639f,  
	54.2559f,28.4593f,65.9501f,  55.2317f,31.1643f,63.9082f,  56.8872f,30.1735f,63.3793f,  57.7603f,32.7855f,61.2550f,  59.2719f,31.6916f,60.5334f,  
	60.0388f,34.1944f,58.3402f,  61.3750f,32.9982f,57.4537f,  62.0337f,35.3765f,55.2064f,  63.1666f,34.0807f,54.1867f,  63.7157f,36.3181f,51.9039f,  
	64.6221f,34.9282f,50.7865f,  65.0624f,37.0120f,48.4860f,  65.7240f,35.5360f,47.3088f,  66.0575f,37.4541f,45.0111f,  66.4614f,35.9036f,43.8133f,  
	66.6918f,37.6469f,41.5387f,  66.8306f,36.0365f,40.3600f,  66.9633f,37.5977f,38.1310f,  66.8353f,35.9459f,37.0098f,  66.8772f,37.3222f,34.8467f,  
	66.4865f,35.6508f,33.8192f,  66.4457f,36.8413f,31.7437f,  65.8016f,35.1756f,30.8423f,  65.6877f,36.1841f,28.8735f,  64.8046f,34.5523f,28.1255f,  
	64.6278f,35.3864f,26.2804f,  63.5244f,33.8193f,25.7069f,  63.2955f,34.4902f,23.9988f,  61.9945f,33.0201f,23.6137f,  61.7240f,33.5419f,22.0532f,  
	6.0640f,-53.2233f,50.3996f,  7.5191f,-52.5978f,51.5781f,  7.8993f,-51.1021f,50.3052f,  9.3095f,-50.4099f,51.3858f,  9.4277f,-48.8127f,50.1871f,  
	10.7936f,-48.0620f,51.1729f,  10.6487f,-46.3968f,50.0738f,  11.9729f,-45.5947f,50.9694f,  11.5724f,-43.8934f,49.9959f,  12.8624f,-43.0419f,50.8083f,  
	12.2216f,-41.3343f,49.9862f,  13.4850f,-40.4338f,50.7217f,  12.6262f,-38.7466f,50.0751f,  13.8728f,-37.7940f,50.7402f,  12.8246f,-36.1504f,50.2908f,  
	14.0648f,-35.1401f,50.8911f,  12.8608f,-33.5597f,50.6569f,  14.1031f,-32.4859f,51.1958f,  12.7811f,-30.9845f,51.1908f,  14.0357f,-29.8378f,51.6710f,  
	12.6361f,-28.4276f,51.9043f,  13.9096f,-27.2008f,52.3258f,  12.4743f,-25.8908f,52.8011f,  13.7747f,-24.5736f,53.1633f,  12.3464f,-23.3694f,53.8789f,  
	13.6792f,-21.9531f,54.1789f,  12.2998f,-20.8578f,55.1275f,  13.6677f,-19.3353f,55.3610f,  12.3774f,-18.3498f,56.5297f,  13.7811f,-16.7162f,56.6910f,  
	12.6173f,-15.8394f,58.0617f,  14.0582f,-14.0885f,58.1448f,  13.0547f,-13.3181f,59.6946f,  14.5305f,-11.4478f,59.6927f,  13.7172f,-10.7803f,61.3942f,  
	15.2242f,-8.7904f,61.3000f,  14.6258f,-8.2219f,63.1223f,  16.1598f,-6.1131f,62.9287f,  15.7956f,-5.6395f,64.8379f,  17.3512f,-3.4143f,64.5377f,  
	17.2349f,-3.0315f,66.4978f,  18.8051f,-0.6952f,66.0846f,  18.9441f,-0.3996f,68.0583f,  20.5209f,2.0402f,67.5260f,  20.9168f,2.2517f,69.4755f,  
	22.4917f,4.7865f,68.8192f,  23.1401f,4.9162f,70.7073f,  24.7045f,7.5362f,69.9226f,  25.5948f,7.5855f,71.7135f,  27.1400f,10.2800f,70.7970f,  
	28.2561f,10.2491f,72.4569f,  29.7718f,13.0040f,71.4070f,  31.0926f,12.8923f,72.9054f,  32.5694f,15.6937f,71.7215f,  34.0688f,15.4995f,73.0315f,  
	35.4981f,18.3339f,71.7137f,  37.1459f,18.0545f,72.8131f,  38.5185f,20.9064f,71.3631f,  40.2813f,20.5386f,72.2345f,  41.5882f,23.3909f,70.6562f,  
	43.4297f,22.9307f,71.2877f,  44.6628f,25.7672f,69.5857f,  46.5451f,25.2103f,69.9716f,  47.6970f,28.0146f,68.1520f,  49.5810f,27.3565f,68.2922f,  
	50.6448f,30.1113f,66.3630f,  52.4913f,29.3479f,66.2639f,  53.4611f,32.0364f,64.2343f,  55.2317f,31.1643f,63.9082f,  56.1023f,33.7689f,61.7898f,  
	57.7603f,32.7855f,61.2550f,  58.5278f,35.2902f,59.0600f,  60.0388f,34.1944f,58.3402f,  60.7007f,36.5839f,56.0824f,  62.0337f,35.3765f,55.2064f,  
	62.5878f,37.6339f,52.9033f,  63.7157f,36.3181f,51.9039f,  64.1622f,38.4301f,49.5727f,  65.0624f,37.0120f,48.4860f,  65.4026f,38.9655f,46.1465f,  
	66.0575f,37.4541f,45.0111f,  66.2943f,39.2390f,42.6830f,  66.6918f,37.6469f,41.5387f,  66.8297f,39.2542f,39.2446f,  66.9633f,37.5977f,38.1310f,  
	67.0084f,39.0230f,35.8918f,  66.8772f,37.3222f,34.8467f,  66.8372f,38.5627f,32.6849f,  66.4457f,36.8413f,31.7437f,  66.3295f,37.8988f,29.6796f,  
	65.6877f,36.1841f,28.8735f,  65.5054f,37.0642f,26.9263f,  64.6278f,35.3864f,26.2804f,  64.3899f,36.0985f,24.4655f,  63.2955f,34.4902f,23.9988f,  
	63.0128f,35.0469f,22.3295f,  61.7240f,33.5419f,22.0532f,  61.4068f,33.9598f,20.5358f,  7.5191f,-52.5978f,51.5781f,  9.0171f,-51.8691f,52.7227f,  
	9.3095f,-50.4099f,51.3858f,  10.7632f,-49.6165f,52.4318f,  10.7936f,-48.0620f,51.1729f,  12.2030f,-47.2127f,52.1231f,  11.9729f,-45.5947f,50.9694f,  
	13.3436f,-44.6932f,51.8299f,  12.8624f,-43.0419f,50.8083f,  14.2003f,-42.0907f,51.5850f,  13.4850f,-40.4338f,50.7217f,  14.7985f,-39.4317f,51.4208f,  
	13.8728f,-37.7940f,50.7402f,  15.1713f,-36.7372f,51.3676f,  14.0648f,-35.1401f,50.8911f,  15.3559f,-34.0247f,51.4504f,  14.1031f,-32.4859f,51.1958f,  
	15.3969f,-31.3038f,51.6905f,  14.0357f,-29.8378f,51.6710f,  15.3387f,-28.5827f,52.1021f,  13.9096f,-27.2008f,52.3258f,  15.2296f,-25.8638f,52.6934f,  
	13.7747f,-24.5736f,53.1633f,  15.1178f,-23.1463f,53.4657f,  13.6792f,-21.9531f,54.1789f,  15.0488f,-20.4288f,54.4127f,  13.6677f,-19.3353f,55.3610f,  
	15.0652f,-17.7091f,55.5216f,  13.7811f,-16.7162f,56.6910f,  15.2085f,-14.9818f,56.7734f,  14.0582f,-14.0885f,58.1448f,  15.5140f,-12.2436f,58.1429f,  
	14.5305f,-11.4478f,59.6927f,  16.0122f,-9.4914f,59.6000f,  15.2242f,-8.7904f,61.3000f,  16.7285f,-6.7226f,61.1101f,  16.1598f,-6.1131f,62.9287f,  
	17.6822f,-3.9355f,62.6349f,  17.3512f,-3.4143f,64.5377f,  18.8858f,-1.1313f,64.1338f,  18.8051f,-0.6952f,66.0846f,  20.3444f,1.6866f,65.5649f,  
	20.5209f,2.0402f,67.5260f,  22.0576f,4.5134f,66.8856f,  22.4917f,4.7865f,68.8192f,  24.0180f,7.3427f,68.0536f,  24.7045f,7.5362f,69.9226f,  
	26.2124f,10.1658f,69.0282f,  27.1400f,10.2800f,70.7970f,  28.6199f,12.9699f,69.7718f,  29.7718f,13.0040f,71.4070f,  31.2150f,15.7417f,70.2499f,  
	32.5694f,15.6937f,71.7215f,  33.9678f,18.4667f,70.4320f,  35.4981f,18.3339f,71.7137f,  36.8428f,21.1279f,70.2929f,  38.5185f,20.9064f,71.3631f,  
	39.8010f,23.7055f,69.8140f,  41.5882f,23.3909f,70.6562f,  42.8007f,26.1801f,68.9824f,  44.6628f,25.7672f,69.5857f,  45.7980f,28.5310f,67.7923f,  
	47.6970f,28.0146f,68.1520f,  48.7480f,30.7363f,66.2458f,  50.6448f,30.1113f,66.3630f,  51.6056f,32.7750f,64.3519f,  53.4611f,32.0364f,64.2343f,  
	54.3263f,34.6251f,62.1287f,  56.1023f,33.7689f,61.7898f,  56.8677f,36.2669f,59.6006f,  58.5278f,35.2902f,59.0600f,  59.1904f,37.6823f,56.7998f,  
	60.7007f,36.5839f,56.0824f,  61.2578f,38.8529f,53.7677f,  62.5878f,37.6339f,52.9033f,  63.0389f,39.7661f,50.5500f,  64.1622f,38.4301f,49.5727f,  
	64.5078f,40.4118f,47.1999f,  65.4026f,38.9655f,46.1465f,  65.6448f,40.7856f,43.7732f,  66.2943f,39.2390f,42.6830f,  66.4369f,40.8870f,40.3318f,  
	66.8297f,39.2542f,39.2446f,  66.8780f,40.7241f,36.9365f,  67.0084f,39.0230f,35.8918f,  66.9694f,40.3102f,33.6496f,  66.8372f,38.5627f,32.6849f,  
	66.7191f,39.6671f,30.5302f,  66.3295f,37.8988f,29.6796f,  66.1421f,38.8239f,27.6335f,  65.5054f,37.0642f,26.9263f,  65.2589f,37.8176f,25.0059f,  
	64.3899f,36.0985f,24.4655f,  64.0951f,36.6910f,22.6866f,  63.0128f,35.0469f,22.3295f,  62.6804f,35.4937f,20.7014f,  61.4068f,33.9598f,20.5358f,  
	61.0467f,34.2779f,19.0650f,  9.0171f,-51.8691f,52.7227f,  10.5581f,-51.0337f,53.8287f,  10.7632f,-49.6165f,52.4318f,  12.2588f,-48.7198f,53.4376f,  
	12.2030f,-47.2127f,52.1231f,  13.6569f,-46.2600f,53.0337f,  13.3436f,-44.6932f,51.8299f,  14.7599f,-43.6888f,52.6502f,  14.2003f,-42.0907f,51.5850f,  
	15.5858f,-41.0355f,52.3209f,  14.7985f,-39.4317f,51.4208f,  16.1612f,-38.3238f,52.0781f,  15.1713f,-36.7372f,51.3676f,  16.5180f,-35.5745f,51.9502f,  
	15.3559f,-34.0247f,51.4504f,  16.6962f,-32.8004f,51.9624f,  15.3969f,-31.3038f,51.6905f,  16.7371f,-30.0130f,52.1337f,  15.3387f,-28.5827f,52.1021f,  
	16.6864f,-27.2178f,52.4773f,  15.2296f,-25.8638f,52.6934f,  16.5911f,-24.4170f,52.9999f,  15.1178f,-23.1463f,53.4657f,  16.4969f,-21.6114f,53.7011f,  
	15.0488f,-20.4288f,54.4127f,  16.4474f,-18.8013f,54.5735f,  15.0652f,-17.7091f,55.5216f,  16.4861f,-15.9827f,55.6037f,  15.2085f,-14.9818f,56.7734f,  
	16.6509f,-13.1540f,56.7716f,  15.5140f,-12.2436f,58.1429f,  16.9762f,-10.3130f,58.0516f,  16.0122f,-9.4914f,59.6000f,  17.4919f,-7.4575f,59.4132f,  
	16.7285f,-6.7226f,61.1101f,  18.2221f,-4.5862f,60.8218f,  17.6822f,-3.9355f,62.6349f,  19.1847f,-1.7002f,62.2393f,  18.8858f,-1.1313f,64.1338f,  
	20.3905f,1.1971f,63.6258f,  20.3444f,1.6866f,65.5649f,  21.8450f,4.1018f,64.9394f,  22.0576f,4.5134f,66.8856f,  23.5471f,7.0082f,66.1383f,  
	24.0180f,7.3427f,68.0536f,  25.4892f,9.9083f,67.1808f,  26.2124f,10.1658f,69.0282f,  27.6564f,12.7905f,68.0277f,  28.6199f,12.9699f,69.7718f,  
	30.0288f,15.6425f,68.6421f,  31.2150f,15.7417f,70.2499f,  32.5812f,18.4509f,68.9900f,  33.9678f,18.4667f,70.4320f,  35.2830f,21.1994f,69.0424f,  
	36.8428f,21.1279f,70.2929f,  38.0989f,23.8693f,68.7757f,  39.8010f,23.7055f,69.8140f,  40.9905f,26.4419f,68.1718f,  42.8007f,26.1801f,68.9824f,  
	43.9166f,28.8968f,67.2193f,  45.7980f,28.5310f,67.7923f,  46.8335f,31.2124f,65.9141f,  48.7480f,30.7363f,66.2458f,  49.6971f,33.3675f,64.2591f,  
	51.6056f,32.7750f,64.3519f,  52.4627f,35.3395f,62.2659f,  54.3263f,34.6251f,62.1287f,  55.0870f,37.1078f,59.9529f,  56.8677f,36.2669f,59.6006f,  
	57.5286f,38.6528f,57.3462f,  59.1904f,37.6823f,56.7998f,  59.7483f,39.9543f,54.4821f,  61.2578f,38.8529f,53.7677f,  61.7116f,40.9973f,51.4014f,  
	63.0389f,39.7661f,50.5500f,  63.3884f,41.7683f,48.1533f,  64.5078f,40.4118f,47.1999f,  64.7545f,42.2601f,44.7907f,  65.6448f,40.7856f,43.7732f,  
	65.7915f,42.4681f,41.3738f,  66.4369f,40.8870f,40.3318f,  66.4881f,42.3963f,37.9631f,  66.8780f,40.7241f,36.9365f,  66.8400f,42.0540f,34.6215f,  
	66.9694f,40.3102f,33.6496f,  66.8498f,41.4587f,31.4107f,  66.7191f,39.6671f,30.5302f,  66.5271f,40.6354f,28.3898f,  66.1421f,38.8239f,27.6335f,  
	65.8876f,39.6179f,25.6105f,  65.2589f,37.8176f,25.0059f,  64.9526f,38.4460f,23.1184f,  64.0951f,36.6910f,22.6866f,  63.7479f,37.1676f,20.9469f,  
	62.6804f,35.4937f,20.7014f,  62.3025f,35.8342f,19.1188f,  61.0467f,34.2779f,19.0650f,  60.6475f,34.5015f,17.6416f,  10.5581f,-51.0337f,53.8287f,  
	12.1394f,-50.0901f,54.8901f,  12.2588f,-48.7198f,53.4376f,  13.7971f,-47.7154f,54.3990f,  13.6569f,-46.2600f,53.0337f,  15.1542f,-45.2010f,53.8992f,  
	14.7599f,-43.6888f,52.6502f,  16.2212f,-42.5779f,53.4251f,  15.5858f,-41.0355f,52.3209f,  17.0178f,-39.8726f,53.0106f,  16.1612f,-38.3238f,52.0781f,  
	17.5695f,-37.1088f,52.6866f,  16.5180f,-35.5745f,51.9502f,  17.9115f,-34.3021f,52.4821f,  16.6962f,-32.8004f,51.9624f,  18.0809f,-31.4670f,52.4201f,  
	16.7371f,-30.0130f,52.1337f,  18.1199f,-28.6125f,52.5186f,  16.6864f,-27.2178f,52.4773f,  18.0738f,-25.7434f,52.7896f,  16.5911f,-24.4170f,52.9999f,  
	17.9873f,-22.8631f,53.2384f,  16.4969f,-21.6114f,53.7011f,  17.9041f,-19.9740f,53.8631f,  16.4474f,-18.8013f,54.5735f,  17.8690f,-17.0741f,54.6558f,  
	16.4861f,-15.9827f,55.6037f,  17.9218f,-14.1635f,55.6020f,  16.6509f,-13.1540f,56.7716f,  18.0999f,-11.2410f,56.6811f,  16.9762f,-10.3130f,58.0516f,  
	18.4370f,-8.3052f,57.8672f,  17.4919f,-7.4575f,59.4132f,  18.9617f,-5.3553f,59.1296f,  18.2221f,-4.5862f,60.8218f,  19.6967f,-2.3925f,60.4336f,  
	19.1847f,-1.7002f,62.2393f,  20.6583f,0.5799f,61.7417f,  20.3905f,1.1971f,63.6258f,  21.8577f,3.5586f,63.0141f,  21.8450f,4.1018f,64.9394f,  
	23.2998f,6.5384f,64.2095f,  23.5471f,7.0082f,66.1383f,  24.9828f,9.5120f,65.2864f,  25.4892f,9.9083f,67.1808f,  26.8979f,12.4689f,66.2047f,  
	27.6564f,12.7905f,68.0277f,  29.0309f,15.3979f,66.9256f,  30.0288f,15.6425f,68.6421f,  31.3621f,18.2866f,67.4124f,  32.5812f,18.4509f,68.9900f,  
	33.8657f,21.1196f,67.6328f,  35.2830f,21.1994f,69.0424f,  36.5110f,23.8794f,67.5589f,  38.0989f,23.8693f,68.7757f,  39.2634f,26.5480f,67.1679f,  
	40.9905f,26.4419f,68.1718f,  42.0848f,29.1057f,66.4428f,  43.9166f,28.8968f,67.2193f,  44.9339f,31.5313f,65.3738f,  46.8335f,31.2124f,65.9141f,  
	47.7681f,33.8036f,63.9575f,  49.6971f,33.3675f,64.2591f,  50.5434f,35.8997f,62.1992f,  52.4627f,35.3395f,62.2659f,  53.2162f,37.7985f,60.1108f,  
	55.0870f,37.1078f,59.9529f,  55.7438f,39.4790f,57.7125f,  57.5286f,38.6528f,57.3462f,  58.0853f,40.9193f,55.0345f,  59.7483f,39.9543f,54.4821f,  
	60.2031f,42.1024f,52.1122f,  61.7116f,40.9973f,51.4014f,  62.0637f,43.0119f,48.9904f,  63.3884f,41.7683f,48.1533f,  63.6386f,43.6373f,45.7180f,  
	64.7545f,42.2601f,44.7907f,  64.9047f,43.9705f,42.3528f,  65.7915f,42.4681f,41.3738f,  65.8455f,44.0111f,38.9538f,  66.4881f,42.3963f,37.9631f,  
	66.4513f,43.7642f,35.5839f,  66.8400f,42.0540f,34.6215f,  66.7193f,43.2430f,32.3059f,  66.8498f,41.4587f,31.4107f,  66.6537f,42.4679f,29.1824f,  
	66.5271f,40.6354f,28.3898f,  66.2653f,41.4687f,26.2692f,  65.8876f,39.6179f,25.6105f,  65.5705f,40.2821f,23.6177f,  64.9526f,38.4460f,23.1184f,  
	64.5911f,38.9531f,21.2683f,  63.7479f,37.1676f,20.9469f,  63.3526f,37.5318f,19.2515f,  62.3025f,35.8342f,19.1188f,  61.8832f,36.0737f,17.5835f,  
	60.6475f,34.5015f,17.6416f,  60.2127f,34.6346f,16.2690f,  12.1394f,-50.0901f,54.8901f,  13.7617f,-49.0346f,55.9025f,  13.7971f,-47.7154f,54.3990f,  
	15.3764f,-46.6013f,55.3106f,  15.1542f,-45.2010f,53.8992f,  16.6939f,-44.0326f,54.7145f,  16.2212f,-42.5779f,53.4251f,  17.7263f,-41.3572f,54.1491f,  
	17.0178f,-39.8726f,53.0106f,  18.4925f,-38.6013f,53.6472f,  17.5695f,-37.1088f,52.6866f,  19.0220f,-35.7831f,53.2407f,  17.9115f,-34.3021f,52.4821f,  
	19.3471f,-32.9200f,52.9564f,  18.0809f,-31.4670f,52.4201f,  19.5058f,-30.0241f,52.8166f,  18.1199f,-28.6125f,52.5186f,  19.5406f,-27.1029f,52.8385f,  
	18.0738f,-25.7434f,52.7896f,  19.4945f,-24.1624f,53.0324f,  17.9873f,-22.8631f,53.2384f,  19.4105f,-21.2071f,53.4023f,  17.9041f,-19.9740f,53.8631f,  
	19.3335f,-18.2374f,53.9459f,  17.8690f,-17.0741f,54.6558f,  19.3048f,-15.2548f,54.6542f,  17.9218f,-14.1635f,55.6020f,  19.3639f,-12.2597f,55.5120f,  
	18.0999f,-11.2410f,56.6811f,  19.5476f,-9.2512f,56.4984f,  18.4370f,-8.3052f,57.8672f,  19.8883f,-6.2295f,57.5871f,  18.9617f,-5.3553f,59.1296f,  
	20.4131f,-3.1961f,58.7474f,  19.6967f,-2.3925f,60.4336f,  21.1432f,-0.1542f,59.9451f,  20.6583f,0.5799f,61.7417f,  22.0954f,2.8932f,61.1425f,  
	21.8577f,3.5586f,63.0141f,  23.2803f,5.9413f,62.3002f,  23.2998f,6.5384f,64.2095f,  24.7019f,8.9837f,63.3774f,  24.9828f,9.5120f,65.2864f,  
	26.3575f,12.0107f,64.3339f,  26.8979f,12.4689f,66.2047f,  28.2385f,15.0121f,65.1296f,  29.0309f,15.3979f,66.9256f,  30.3312f,17.9766f,65.7262f,  
	31.3621f,18.2866f,67.4124f,  32.6151f,20.8900f,66.0883f,  33.8657f,21.1196f,67.6328f,  35.0643f,23.7357f,66.1846f,  36.5110f,23.8794f,67.5589f,  
	37.6488f,26.4967f,65.9879f,  39.2634f,26.5480f,67.1679f,  40.3340f,29.1543f,65.4761f,  42.0848f,29.1057f,66.4428f,  43.0818f,31.6876f,64.6339f,  
	44.9339f,31.5313f,65.3738f,  45.8517f,34.0759f,63.4521f,  47.7681f,33.8036f,63.9575f,  48.6012f,36.2963f,61.9295f,  50.5434f,35.8997f,62.1992f,  
	51.2872f,38.3271f,60.0717f,  53.2162f,37.7985f,60.1108f,  53.8666f,40.1466f,57.8922f,  55.7438f,39.4790f,57.7125f,  56.2971f,41.7314f,55.4153f,  
	58.0853f,40.9193f,55.0345f,  58.5392f,43.0625f,52.6703f,  60.2031f,42.1024f,52.1122f,  60.5562f,44.1211f,49.6969f,  62.0637f,43.0119f,48.9904f,  
	62.3162f,44.8937f,46.5394f,  63.6386f,43.6373f,45.7180f,  63.7915f,45.3684f,43.2521f,  64.9047f,43.9705f,42.3528f,  64.9611f,45.5407f,39.8919f,  
	65.8455f,44.0111f,38.9538f,  65.8099f,45.4116f,36.5202f,  66.4513f,43.7642f,35.5839f,  66.3299f,44.9894f,33.2006f,  66.7193f,43.2430f,32.3059f,  
	66.5199f,44.2898f,29.9977f,  66.6537f,42.4679f,29.1824f,  66.3855f,43.3384f,26.9706f,  66.2653f,41.4687f,26.2692f,  65.9383f,42.1677f,24.1757f,  
	65.5705f,40.2821f,23.6177f,  65.1957f,40.8194f,21.6599f,  64.5911f,38.9531f,21.2683f,  64.1791f,39.3415f,19.4609f,  63.3526f,37.5318f,19.2515f,  
	62.9137f,37.7883f,17.6030f,  61.8832f,36.0737f,17.5835f,  61.4265f,36.2158f,16.0996f,  60.2127f,34.6346f,16.2690f,  59.7458f,34.6826f,14.9476f,  
	13.7617f,-49.0346f,55.9025f,  15.4229f,-47.8656f,56.8602f,  15.3764f,-46.6013f,55.3106f,  16.9957f,-45.3747f,56.1670f,  16.6939f,-44.0326f,54.7145f,  
	18.2746f,-42.7522f,55.4740f,  17.7263f,-41.3572f,54.1491f,  19.2712f,-40.0265f,54.8154f,  18.4925f,-38.6013f,53.6472f,  20.0087f,-37.2182f,54.2252f,  
	19.0220f,-35.7831f,53.2407f,  20.5140f,-34.3471f,53.7334f,  19.3471f,-32.9200f,52.9564f,  20.8203f,-31.4284f,53.3663f,  19.5058f,-30.0241f,52.8166f,  
	20.9663f,-28.4723f,53.1456f,  19.5406f,-27.1029f,52.8385f,  20.9927f,-25.4870f,53.0867f,  19.4945f,-24.1624f,53.0324f,  20.9407f,-22.4796f,53.1992f,  
	19.4105f,-21.2071f,53.4023f,  20.8548f,-19.4526f,53.4862f,  19.3335f,-18.2374f,53.9459f,  20.7764f,-16.4093f,53.9444f,  19.3048f,-15.2548f,54.6542f,  
	20.7467f,-13.3514f,54.5643f,  19.3639f,-12.2597f,55.5120f,  20.8048f,-10.2793f,55.3302f,  19.5476f,-9.2512f,56.4984f,  20.9861f,-7.1940f,56.2208f,  
	19.8883f,-6.2295f,57.5871f,  21.3215f,-4.0973f,57.2097f,  20.4131f,-3.1961f,58.7474f,  21.8370f,-0.9928f,58.2665f,  21.1432f,-0.1542f,59.9451f,  
	22.5541f,2.1169f,59.3567f,  22.0954f,2.8932f,61.1425f,  23.4890f,5.2272f,60.4432f,  23.2803f,5.9413f,62.3002f,  24.6513f,8.3324f,61.4866f,  
	24.7019f,8.9837f,63.3774f,  26.0442f,11.4236f,62.4473f,  26.3575f,12.0107f,64.3339f,  27.6652f,14.4916f,63.2850f,  28.2385f,15.0121f,65.1296f,  
	29.5064f,17.5263f,63.9601f,  30.3312f,17.9766f,65.7262f,  31.5527f,20.5143f,64.4353f,  32.6151f,20.8900f,66.0883f,  33.7838f,23.4405f,64.6763f,  
	35.0643f,23.7357f,66.1846f,  36.1743f,26.2889f,64.6519f,  37.6488f,26.4967f,65.9879f,  38.6942f,29.0416f,64.3358f,  40.3340f,29.1543f,65.4761f,  
	41.3088f,31.6787f,63.7073f,  43.0818f,31.6876f,64.6339f,  43.9807f,34.1797f,62.7517f,  45.8517f,34.0759f,63.4521f,  46.6693f,36.5224f,61.4616f,  
	48.6012f,36.2963f,61.9295f,  49.3330f,38.6847f,59.8361f,  51.2872f,38.3271f,60.0717f,  51.9290f,40.6443f,57.8823f,  53.8666f,40.1466f,57.8922f,  
	54.4144f,42.3767f,55.6180f,  56.2971f,41.7314f,55.4153f,  56.7483f,43.8614f,53.0661f,  58.5392f,43.0625f,52.6703f,  58.8918f,45.0772f,50.2607f,  
	60.5562f,44.1211f,49.6969f,  60.8099f,46.0078f,47.2408f,  62.3162f,44.8937f,46.5394f,  62.4711f,46.6380f,44.0564f,  63.7915f,45.3684f,43.2521f,  
	63.8500f,46.9592f,40.7612f,  64.9611f,45.5407f,39.8919f,  64.9268f,46.9680f,37.4145f,  65.8099f,45.4116f,36.5202f,  65.6882f,46.6680f,34.0795f,  
	66.3299f,44.9894f,33.2006f,  66.1279f,46.0701f,30.8218f,  66.5199f,44.2898f,29.9977f,  66.2461f,45.1949f,27.7028f,  66.3855f,43.3384f,26.9706f,  
	66.0497f,44.0705f,24.7830f,  65.9383f,42.1677f,24.1757f,  65.5511f,42.7348f,22.1151f,  65.1957f,40.8194f,21.6599f,  64.7678f,41.2325f,19.7433f,  
	64.1791f,39.3415f,19.4609f,  63.7212f,39.6160f,17.6998f,  62.9137f,37.7883f,17.6030f,  62.4354f,37.9407f,16.0063f,  61.4265f,36.2158f,16.0996f,  
	60.9362f,36.2660f,14.6684f,  59.7458f,34.6826f,14.9476f,  59.2499f,34.6510f,13.6772f,  15.4229f,-47.8656f,56.8602f,  17.1216f,-46.5811f,57.7578f,  
	16.9957f,-45.3747f,56.1670f,  18.6534f,-44.0336f,56.9628f,  18.2746f,-42.7522f,55.4740f,  19.8922f,-41.3599f,56.1712f,  19.2712f,-40.0265f,54.8154f,  
	20.8547f,-38.5827f,55.4187f,  20.0087f,-37.2182f,54.2252f,  21.5618f,-35.7238f,54.7379f,  20.5140f,-34.3471f,53.7334f,  22.0409f,-32.8014f,54.1583f,  
	20.8203f,-31.4284f,53.3663f,  22.3267f,-29.8280f,53.7057f,  20.9663f,-28.4723f,53.1456f,  22.4562f,-26.8143f,53.4004f,  20.9927f,-25.4870f,53.0867f,  
	22.4686f,-23.7697f,53.2571f,  20.9407f,-22.4796f,53.1992f,  22.4066f,-20.6989f,53.2844f,  20.8548f,-19.4526f,53.4862f,  22.3113f,-17.6072f,53.4848f,  
	20.7764f,-16.4093f,53.9444f,  22.2245f,-14.4977f,53.8543f,  20.7467f,-13.3514f,54.5643f,  22.1870f,-11.3720f,54.3827f,  20.8048f,-10.2793f,55.3302f,  
	22.2362f,-8.2322f,55.0540f,  20.9861f,-7.1940f,56.2208f,  22.4066f,-5.0807f,55.8468f,  21.3215f,-4.0973f,57.2097f,  22.7277f,-1.9215f,56.7348f,  
	21.8370f,-0.9928f,58.2665f,  23.2259f,1.2429f,57.6873f,  22.5541f,2.1169f,59.3567f,  23.9222f,4.4084f,58.6701f,  23.4890f,5.2272f,60.4432f,  
	24.8318f,7.5692f,59.6462f,  24.6513f,8.3324f,61.4866f,  25.9635f,10.7177f,60.5772f,  26.0442f,11.4236f,62.4473f,  27.3208f,13.8455f,61.4233f,  
	27.6652f,14.4916f,63.2850f,  28.9015f,16.9434f,62.1445f,  29.5064f,17.5263f,63.9601f,  30.6966f,19.9991f,62.7022f,  31.5527f,20.5143f,64.4353f,  
	32.6912f,22.9990f,63.0597f,  33.7838f,23.4405f,64.6763f,  34.8653f,25.9281f,63.1828f,  36.1743f,26.2889f,64.6519f,  37.1934f,28.7695f,63.0412f,  
	38.6942f,29.0416f,64.3358f,  39.6453f,31.5046f,62.6098f,  41.3088f,31.6787f,63.7073f,  42.1871f,34.1134f,61.8682f,  43.9807f,34.1797f,62.7517f,  
	44.7809f,36.5741f,60.8034f,  46.6693f,36.5224f,61.4616f,  47.3871f,38.8651f,59.4081f,  49.3330f,38.6847f,59.8361f,  49.9642f,40.9635f,57.6830f,  
	51.9290f,40.6443f,57.8823f,  52.4693f,42.8442f,55.6391f,  54.4144f,42.3767f,55.6180f,  54.8611f,44.4852f,53.2929f,  56.7483f,43.8614f,53.0661f,  
	57.0990f,45.8636f,50.6722f,  58.8918f,45.0772f,50.2607f,  59.1454f,46.9606f,47.8101f,  60.8099f,46.0078f,47.2408f,  60.9660f,47.7575f,44.7517f,  
	62.4711f,46.6380f,44.0564f,  62.5312f,48.2421f,41.5468f,  63.8500f,46.9592f,40.7612f,  63.8170f,48.4068f,38.2516f,  64.9268f,46.9680f,37.4145f,  
	64.8053f,48.2502f,34.9275f,  65.6882f,46.6680f,34.0795f,  65.4844f,47.7783f,31.6404f,  66.1279f,46.0701f,30.8218f,  65.8497f,47.0065f,28.4532f,  
	66.2461f,45.1949f,27.7028f,  65.9028f,45.9581f,25.4292f,  66.0497f,44.0705f,24.7830f,  65.6514f,44.6664f,22.6261f,  65.5511f,42.7348f,22.1151f,  
	65.1087f,43.1724f,20.0939f,  64.7678f,41.2325f,19.7433f,  64.2919f,41.5256f,17.8722f,  63.7212f,39.6160f,17.6998f,  63.2221f,39.7800f,15.9907f,  
	62.4354f,37.9407f,16.0063f,  61.9221f,37.9946f,14.4636f,  60.9362f,36.2660f,14.6684f,  60.4158f,36.2299f,13.2903f,  59.2499f,34.6510f,13.6772f,  
	58.7281f,34.5438f,12.4603f,  17.1216f,-46.5811f,57.7578f,  18.8560f,-45.1797f,58.5900f,  18.6534f,-44.0336f,56.9628f,  20.3449f,-42.5789f,57.6915f,  
	19.8922f,-41.3599f,56.1712f,  21.5455f,-39.8534f,56.8009f,  20.8547f,-38.5827f,55.4187f,  22.4723f,-37.0267f,55.9526f,  21.5618f,-35.7238f,54.7379f,  
	23.1468f,-34.1195f,55.1789f,  22.0409f,-32.8014f,54.1583f,  23.5983f,-31.1468f,54.5092f,  22.3267f,-29.8280f,53.7057f,  23.8601f,-28.1216f,53.9681f,  
	22.4562f,-26.8143f,53.4004f,  23.9680f,-25.0553f,53.5751f,  22.4686f,-23.7697f,53.2571f,  23.9624f,-21.9550f,53.3442f,  22.4066f,-20.6989f,53.2844f,  
	23.8833f,-18.8280f,53.2832f,  22.3113f,-17.6072f,53.4848f,  23.7722f,-15.6788f,53.3940f,  22.2245f,-14.4977f,53.8543f,  23.6705f,-12.5105f,53.6720f,  
	22.1870f,-11.3720f,54.3827f,  23.6174f,-9.3264f,54.1068f,  22.2362f,-8.2322f,55.0540f,  23.6497f,-6.1295f,54.6819f,  22.4066f,-5.0807f,55.8468f,  
	23.8002f,-2.9244f,55.3761f,  22.7277f,-1.9215f,56.7348f,  24.0993f,0.2864f,56.1627f,  23.2259f,1.2429f,57.6873f,  24.5727f,3.4987f,57.0113f,  
	23.9222f,4.4084f,58.6701f,  25.2403f,6.7073f,57.8877f,  24.8318f,7.5692f,59.6462f,  26.1167f,9.9051f,58.7556f,  25.9635f,10.7177f,60.5772f,  
	27.2111f,13.0846f,59.5764f,  27.3208f,13.8455f,61.4233f,  28.5271f,16.2377f,60.3104f,  28.9015f,16.9434f,62.1445f,  30.0615f,19.3532f,60.9185f,  
	30.6966f,19.9991f,62.7022f,  31.8053f,22.4186f,61.3625f,  32.6912f,22.9990f,63.0597f,  33.7440f,25.4204f,61.6057f,  34.8653f,25.9281f,63.1828f,  
	35.8573f,28.3429f,61.6147f,  37.1934f,28.7695f,63.0412f,  38.1198f,31.1683f,61.3601f,  39.6453f,31.5046f,62.6098f,  40.5015f,33.8779f,60.8168f,  
	42.1871f,34.1134f,61.8682f,  42.9682f,36.4507f,59.9662f,  44.7809f,36.5741f,60.8034f,  45.4828f,38.8652f,58.7951f,  47.3871f,38.8651f,59.4081f,  
	48.0058f,41.0987f,57.2977f,  49.9642f,40.9635f,57.6830f,  50.4952f,43.1257f,55.4784f,  52.4693f,42.8442f,55.6391f,  52.9099f,44.9234f,53.3468f,  
	54.8611f,44.4852f,53.2929f,  55.2084f,46.4670f,50.9243f,  57.0990f,45.8636f,50.6722f,  57.3514f,47.7356f,48.2376f,  59.1454f,46.9606f,47.8101f,  
	59.3020f,48.7078f,45.3263f,  60.9660f,47.7575f,44.7517f,  61.0274f,49.3675f,42.2353f,  62.5312f,48.2421f,41.5468f,  62.4997f,49.7032f,39.0170f,  
	63.8170f,48.4068f,38.2516f,  63.6961f,49.7089f,35.7302f,  64.8053f,48.2502f,34.9275f,  64.6005f,49.3851f,32.4397f,  65.4844f,47.7783f,31.6404f,  
	65.2028f,48.7422f,29.2089f,  65.8497f,47.0065f,28.4532f,  65.5000f,47.7981f,26.1033f,  65.9028f,45.9581f,25.4292f,  65.4948f,46.5811f,23.1841f,  
	65.6514f,44.6664f,22.6261f,  65.1958f,45.1279f,20.5067f,  65.1087f,43.1724f,20.0939f,  64.6161f,43.4844f,18.1173f,  64.2919f,41.5256f,17.8722f,  
	63.7731f,41.7021f,16.0531f,  63.2221f,39.7800f,15.9907f,  62.6866f,39.8390f,14.3366f,  61.9221f,37.9946f,14.4636f,  61.3776f,37.9556f,12.9758f,  
	60.4158f,36.2299f,13.2903f,  59.8691f,36.1118f,11.9688f,  58.7281f,34.5438f,12.4603f,  58.1833f,34.3664f,11.2965f,  18.8560f,-45.1797f,58.5900f,  
	20.6215f,-43.6625f,59.3502f,  20.3449f,-42.5789f,57.6915f,  22.0692f,-41.0084f,58.3481f,  21.5455f,-39.8534f,56.8009f,  23.2302f,-38.2333f,57.3568f,  
	22.4723f,-37.0267f,55.9526f,  24.1189f,-35.3603f,56.4108f,  23.1468f,-34.1195f,55.1789f,  24.7595f,-32.4063f,55.5424f,  23.5983f,-31.1468f,54.5092f,  
	25.1803f,-29.3864f,54.7801f,  23.8601f,-28.1216f,53.9681f,  25.4133f,-26.3144f,54.1477f,  23.9680f,-25.0553f,53.5751f,  25.4957f,-23.1994f,53.6643f,  
	23.9624f,-21.9550f,53.3442f,  25.4655f,-20.0507f,53.3431f,  23.8833f,-18.8280f,53.2832f,  25.3631f,-16.8746f,53.1914f,  23.7722f,-15.6788f,53.3940f,  
	25.2300f,-13.6753f,53.2104f,  23.6705f,-12.5105f,53.6720f,  25.1059f,-10.4578f,53.3952f,  23.6174f,-9.3264f,54.1068f,  25.0295f,-7.2258f,53.7351f,  
	23.6497f,-6.1295f,54.6819f,  25.0360f,-3.9843f,54.2136f,  23.8002f,-2.9244f,55.3761f,  25.1594f,-0.7365f,54.8092f,  24.0993f,0.2864f,56.1627f,  
	25.4292f,2.5138f,55.4952f,  24.5727f,3.4987f,57.0113f,  25.8699f,5.7613f,56.2412f,  25.2403f,6.7073f,57.8877f,  26.5013f,8.9996f,57.0136f,  
	26.1167f,9.9051f,58.7556f,  27.3381f,12.2221f,57.7758f,  27.2111f,13.0846f,59.5764f,  28.3895f,15.4215f,58.4892f,  28.5271f,16.2377f,60.3104f,  
	29.6583f,18.5877f,59.1148f,  30.0615f,19.3532f,60.9185f,  31.1412f,21.7094f,59.6137f,  31.8053f,22.4186f,61.3625f,  32.8297f,24.7747f,59.9476f,  
	33.7440f,25.4204f,61.6057f,  34.7089f,27.7688f,60.0806f,  35.8573f,28.3429f,61.6147f,  36.7583f,30.6757f,59.9796f,  38.1198f,31.1683f,61.3601f,  
	38.9528f,33.4774f,59.6154f,  40.5015f,33.8779f,60.8168f,  41.2622f,36.1541f,58.9644f,  42.9682f,36.4507f,59.9662f,  43.6527f,38.6850f,58.0075f,  
	45.4828f,38.8652f,58.7951f,  46.0873f,41.0477f,56.7330f,  48.0058f,41.0987f,57.2977f,  48.5259f,43.2166f,55.1384f,  50.4952f,43.1257f,55.4784f,  
	50.9281f,45.1684f,53.2269f,  52.9099f,44.9234f,53.3468f,  53.2523f,46.8769f,51.0126f,  55.2084f,46.4670f,50.9243f,  55.4584f,48.3197f,48.5161f,  
	57.3514f,47.7356f,48.2376f,  57.5076f,49.4726f,45.7703f,  59.3020f,48.7078f,45.3263f,  59.3643f,50.3162f,42.8150f,  61.0274f,49.3675f,42.2353f,  
	60.9972f,50.8353f,39.6976f,  62.4997f,49.7032f,39.0170f,  62.3798f,51.0191f,36.4737f,  63.6961f,49.7089f,35.7302f,  63.4911f,50.8630f,33.2060f,  
	64.6005f,49.3851f,32.4397f,  64.3167f,50.3722f,29.9571f,  65.2028f,48.7422f,29.2089f,  64.8483f,49.5590f,26.7936f,  65.5000f,47.7981f,26.1033f,  
	65.0840f,48.4462f,23.7796f,  65.4948f,46.5811f,23.1841f,  65.0277f,47.0657f,20.9746f,  65.1958f,45.1279f,20.5067f,  64.6882f,45.4589f,18.4309f,  
	64.6161f,43.4844f,18.1173f,  64.0791f,43.6742f,16.1923f,  63.7731f,41.7021f,16.0531f,  63.2165f,41.7674f,14.2896f,  62.6866f,39.8390f,14.3366f,  
	62.1187f,39.7987f,12.7390f,  61.3776f,37.9556f,12.9758f,  60.8062f,37.8283f,11.5474f,  59.8691f,36.1118f,11.9688f,  59.2992f,35.9175f,10.7039f,  
	58.1833f,34.3664f,11.2965f,  57.6179f,34.1243f,10.1837f,  20.6215f,-43.6625f,59.3502f,  22.4171f,-42.0280f,60.0337f,  22.0692f,-41.0084f,58.3481f,  
	23.8223f,-39.3232f,58.9265f,  23.2302f,-38.2333f,57.3568f,  24.9408f,-36.5024f,57.8328f,  24.1189f,-35.3603f,56.4108f,  25.7903f,-33.5848f,56.7876f,  
	24.7595f,-32.4063f,55.5424f,  26.3942f,-30.5873f,55.8225f,  25.1803f,-29.3864f,54.7801f,  26.7797f,-27.5255f,54.9653f,  25.4133f,-26.3144f,54.1477f,  
	26.9802f,-24.4109f,54.2395f,  25.4957f,-23.1994f,53.6643f,  27.0308f,-21.2546f,53.6634f,  25.4655f,-20.0507f,53.3431f,  26.9702f,-18.0646f,53.2499f,  
	25.3631f,-16.8746f,53.1914f,  26.8388f,-14.8467f,53.0057f,  25.2300f,-13.6753f,53.2104f,  26.6764f,-11.6071f,52.9316f,  25.1059f,-10.4578f,53.3952f,  
	26.5224f,-8.3507f,53.0224f,  25.0295f,-7.2258f,53.7351f,  26.4142f,-5.0832f,53.2674f,  25.0360f,-3.9843f,54.2136f,  26.3881f,-1.8081f,53.6498f,  
	25.1594f,-0.7365f,54.8092f,  26.4770f,1.4705f,54.1478f,  25.4292f,2.5138f,55.4952f,  26.7098f,4.7475f,54.7350f,  25.8699f,5.7613f,56.2412f,  
	27.1106f,8.0168f,55.3811f,  26.5013f,8.9996f,57.0136f,  27.6995f,11.2728f,56.0524f,  27.3381f,12.2221f,57.7758f,  28.4911f,14.5087f,56.7119f,  
	28.3895f,15.4215f,58.4892f,  29.4937f,17.7155f,57.3219f,  29.6583f,18.5877f,59.1148f,  30.7104f,20.8835f,57.8433f,  31.1412f,21.7094f,59.6137f,  
	32.1380f,24.0018f,58.2370f,  32.8297f,24.7747f,59.9476f,  33.7676f,27.0573f,58.4651f,  34.7089f,27.7688f,60.0806f,  35.5842f,30.0350f,58.4919f,  
	36.7583f,30.6757f,59.9796f,  37.5675f,32.9187f,58.2847f,  38.9528f,33.4774f,59.6154f,  39.6921f,35.6894f,57.8150f,  41.2622f,36.1541f,58.9644f,  
	41.9281f,38.3277f,57.0588f,  43.6527f,38.6850f,58.0075f,  44.2415f,40.8113f,55.9985f,  46.0873f,41.0477f,56.7330f,  46.5950f,43.1153f,54.6252f,  
	48.5259f,43.2166f,55.1384f,  48.9495f,45.2160f,52.9351f,  50.9281f,45.1684f,53.2269f,  51.2643f,47.0865f,50.9358f,  53.2523f,46.8769f,51.0126f,  
	53.4988f,48.7028f,48.6407f,  55.4584f,48.3197f,48.5161f,  55.6134f,50.0388f,46.0761f,  57.5076f,49.4726f,45.7703f,  57.5703f,51.0720f,43.2759f,  
	59.3643f,50.3162f,42.8150f,  59.3354f,51.7835f,40.2819f,  60.9972f,50.8353f,39.6976f,  60.8787f,52.1583f,37.1454f,  62.3798f,51.0191f,36.4737f,  
	62.1755f,52.1869f,33.9261f,  63.4911f,50.8630f,33.2060f,  63.2064f,51.8688f,30.6849f,  64.3167f,50.3722f,29.9571f,  63.9589f,51.2108f,27.4884f,  
	64.8483f,49.5590f,26.7936f,  64.4261f,50.2299f,24.4024f,  65.0840f,48.4462f,23.7796f,  64.6074f,48.9525f,21.4897f,  65.0277f,47.0657f,20.9746f,  
	64.5071f,47.4153f,18.8076f,  64.6882f,45.4589f,18.4309f,  64.1347f,45.6625f,16.4061f,  64.0791f,43.6742f,16.1923f,  63.5029f,43.7471f,14.3235f,  
	63.2165f,41.7674f,14.2896f,  62.6266f,41.7275f,12.5840f,  62.1187f,39.7987f,12.7390f,  61.5236f,39.6638f,11.2033f,  60.8062f,37.8283f,11.5474f,  
	60.2116f,37.6186f,10.1789f,  59.2992f,35.9175f,10.7039f,  58.7090f,35.6527f,9.4944f,  57.6179f,34.1243f,10.1837f,  57.0344f,33.8215f,9.1238f,  
	22.4171f,-42.0280f,60.0337f,  24.2386f,-40.2775f,60.6347f,  23.8223f,-39.3232f,58.9265f,  25.5982f,-37.5265f,59.4207f,  24.9408f,-36.5024f,57.8328f,  
	26.6732f,-34.6623f,58.2235f,  25.7903f,-33.5848f,56.7876f,  27.4810f,-31.7036f,57.0774f,  26.3942f,-30.5873f,55.8225f,  28.0438f,-28.6679f,56.0136f,  
	26.7797f,-27.5255f,54.9653f,  28.3905f,-25.5686f,55.0598f,  26.9802f,-24.4109f,54.2395f,  28.5525f,-22.4190f,54.2387f,  27.0308f,-21.2546f,53.6634f,  
	28.5659f,-19.2284f,53.5685f,  26.9702f,-18.0646f,53.2499f,  28.4694f,-16.0043f,53.0614f,  26.8388f,-14.8467f,53.0057f,  28.3019f,-12.7546f,52.7238f,  
	26.6764f,-11.6071f,52.9316f,  28.1029f,-9.4850f,52.5562f,  26.5224f,-8.3507f,53.0224f,  27.9109f,-6.2023f,52.5535f,  26.4142f,-5.0832f,53.2674f,  
	27.7642f,-2.9102f,52.7044f,  26.3881f,-1.8081f,53.6498f,  27.6984f,0.3867f,52.9920f,  26.4770f,1.4705f,54.1478f,  27.7456f,3.6831f,53.3947f,  
	26.7098f,4.7475f,54.7350f,  27.9343f,6.9735f,53.8861f,  27.1106f,8.0168f,55.3811f,  28.2892f,10.2528f,54.4356f,  27.6995f,11.2728f,56.0524f,  
	28.8301f,13.5148f,55.0092f,  28.4911f,14.5087f,56.7119f,  29.5708f,16.7517f,55.5706f,  29.4937f,17.7155f,57.3219f,  30.5201f,19.9550f,56.0816f,  
	30.7104f,20.8835f,57.8433f,  31.6808f,23.1151f,56.5030f,  32.1380f,24.0018f,58.2370f,  33.0497f,26.2203f,56.7959f,  33.7676f,27.0573f,58.4651f,  
	34.6175f,29.2574f,56.9225f,  35.5842f,30.0350f,58.4919f,  36.3694f,32.2113f,56.8473f,  37.5675f,32.9187f,58.2847f,  38.2848f,35.0647f,56.5377f,  
	39.6921f,35.6894f,57.8150f,  40.3384f,37.7992f,55.9652f,  41.9281f,38.3277f,57.0588f,  42.5001f,40.3935f,55.1069f,  44.2415f,40.8113f,55.9985f,  
	44.7356f,42.8235f,53.9474f,  46.5950f,43.1153f,54.6252f,  47.0081f,45.0654f,52.4766f,  48.9495f,45.2160f,52.9351f,  49.2783f,47.0921f,50.6951f,  
	51.2643f,47.0865f,50.9358f,  51.5063f,48.8785f,48.6093f,  53.4988f,48.7028f,48.6407f,  53.6518f,50.3967f,46.2385f,  55.6134f,50.0388f,46.0761f,  
	55.6760f,51.6218f,43.6101f,  57.5703f,51.0720f,43.2759f,  57.5427f,52.5317f,40.7598f,  59.3354f,51.7835f,40.2819f,  59.2187f,53.1072f,37.7337f,  
	60.8787f,52.1583f,37.1454f,  60.6759f,53.3339f,34.5877f,  62.1755f,52.1869f,33.9261f,  61.8910f,53.2062f,31.3800f,  63.2064f,51.8688f,30.6849f,  
	62.8469f,52.7251f,28.1758f,  63.9589f,51.2108f,27.4884f,  63.5323f,51.9015f,25.0418f,  64.4261f,50.2299f,24.4024f,  63.9419f,50.7561f,22.0431f,  
	64.6074f,48.9525f,21.4897f,  64.0757f,49.3201f,19.2410f,  64.5071f,47.4153f,18.8076f,  63.9391f,47.6330f,16.6907f,  64.1347f,45.6625f,16.4061f,  
	63.5408f,45.7440f,14.4376f,  63.5029f,43.7471f,14.3235f,  62.8924f,43.7090f,12.5135f,  62.6266f,41.7275f,12.5840f,  62.0088f,41.5870f,10.9424f,  
	61.5236f,39.6638f,11.2033f,  60.9051f,39.4408f,9.7305f,  60.2116f,37.6186f,10.1789f,  59.5971f,37.3330f,8.8698f,  58.7090f,35.6527f,9.4944f,  
	58.1015f,35.3222f,8.3421f,  57.0344f,33.8215f,9.1238f,  56.4350f,33.4632f,8.1145f,  24.2386f,-40.2775f,60.6347f,  26.0800f,-38.4148f,61.1471f,  
	25.5982f,-37.5265f,59.4207f,  27.3931f,-35.6202f,59.8255f,  26.6732f,-34.6623f,58.2235f,  28.4222f,-32.7162f,58.5234f,  27.4810f,-31.7036f,57.0774f,  
	29.1841f,-29.7219f,57.2749f,  28.0438f,-28.6679f,56.0136f,  29.7024f,-26.6529f,56.1111f,  28.3905f,-25.5686f,55.0598f,  30.0044f,-23.5239f,55.0592f,  
	28.5525f,-22.4190f,54.2387f,  30.1228f,-20.3461f,54.1419f,  28.5659f,-19.2284f,53.5685f,  30.0939f,-17.1286f,53.3766f,  28.4694f,-16.0043f,53.0614f,  
	29.9545f,-13.8807f,52.7754f,  28.3019f,-12.7546f,52.7238f,  29.7439f,-10.6094f,52.3444f,  28.1029f,-9.4850f,52.5562f,  29.5007f,-7.3222f,52.0842f,  
	27.9109f,-6.2023f,52.5535f,  29.2643f,-4.0238f,51.9891f,  27.7642f,-2.9102f,52.7044f,  29.0723f,-0.7190f,52.0477f,  27.6984f,0.3867f,52.9920f,  
	28.9595f,2.5864f,52.2433f,  27.7456f,3.6831f,53.3947f,  28.9582f,5.8876f,52.5540f,  27.9343f,6.9735f,53.8861f,  29.0971f,9.1796f,52.9532f,  
	28.2892f,10.2528f,54.4356f,  29.4007f,12.4570f,53.4100f,  28.8301f,13.5148f,55.0092f,  29.8881f,15.7127f,53.8907f,  29.5708f,16.7517f,55.5706f,  
	30.5736f,18.9397f,54.3587f,  30.5201f,19.9550f,56.0816f,  31.4659f,22.1297f,54.7753f,  31.6808f,23.1151f,56.5030f,  32.5674f,25.2724f,55.1015f,  
	33.0497f,26.2203f,56.7959f,  33.8748f,28.3561f,55.2983f,  34.6175f,29.2574f,56.9225f,  35.3788f,31.3674f,55.3278f,  36.3694f,32.2113f,56.8473f,  
	37.0644f,34.2906f,55.1544f,  38.2848f,35.0647f,56.5377f,  38.9110f,37.1087f,54.7455f,  40.3384f,37.7992f,55.9652f,  40.8928f,39.8016f,54.0733f,  
	42.5001f,40.3935f,55.1069f,  42.9794f,42.3461f,53.1166f,  44.7356f,42.8235f,53.9474f,  45.1370f,44.7192f,51.8591f,  47.0081f,45.0654f,52.4766f,  
	47.3283f,46.8936f,50.2946f,  49.2783f,47.0921f,50.6951f,  49.5148f,48.8437f,48.4224f,  51.5063f,48.8785f,48.6093f,  51.6565f,50.5403f,46.2547f,  
	53.6518f,50.3967f,46.2385f,  53.7140f,51.9564f,43.8119f,  55.6760f,51.6218f,43.6101f,  55.6497f,53.0670f,41.1231f,  57.5427f,52.5317f,40.7598f,  
	57.4281f,53.8494f,38.2287f,  59.2187f,53.1072f,37.7337f,  59.0181f,54.2845f,35.1795f,  60.6759f,53.3339f,34.5877f,  60.3929f,54.3616f,32.0303f,  
	61.8910f,53.2062f,31.3800f,  61.5314f,54.0760f,28.8440f,  62.8469f,52.7251f,28.1758f,  62.4178f,53.4324f,25.6870f,  63.5323f,51.9015f,25.0418f,  
	63.0427f,52.4456f,22.6253f,  63.9419f,50.7561f,22.0431f,  63.4015f,51.1407f,19.7236f,  64.0757f,49.3201f,19.2410f,  63.4955f,49.5520f,17.0414f,  
	63.9391f,47.6330f,16.6907f,  63.3297f,47.7240f,14.6300f,  63.5408f,45.7440f,14.4376f,  62.9117f,45.7094f,12.5286f,  62.8924f,43.7090f,12.5135f,  
	62.2536f,43.5649f,10.7691f,  62.0088f,41.5870f,10.9424f,  61.3676f,41.3527f,9.3663f,  60.9051f,39.4408f,9.7305f,  60.2668f,39.1363f,8.3206f,  
	59.5971f,37.3330f,8.8698f,  58.9659f,36.9764f,7.6220f,  58.1015f,35.3222f,8.3421f,  57.4793f,34.9316f,7.2454f,  56.4350f,33.4632f,8.1145f,  
	55.8217f,33.0539f,7.1549f,  26.0800f,-38.4148f,61.1471f,  27.9375f,-36.4421f,61.5661f,  27.3931f,-35.6202f,59.8255f,  29.2016f,-33.6077f,60.1358f,  
	28.4222f,-32.7162f,58.5234f,  30.1810f,-30.6696f,58.7276f,  29.1841f,-29.7219f,57.2749f,  30.8936f,-27.6450f,57.3756f,  29.7024f,-26.6529f,56.1111f,  
	31.3618f,-24.5505f,56.1107f,  30.0044f,-23.5239f,55.0592f,  31.6145f,-21.3986f,54.9600f,  30.1228f,-20.3461f,54.1419f,  31.6844f,-18.2002f,53.9458f,  
	30.0939f,-17.1286f,53.3766f,  31.6062f,-14.9661f,53.0854f,  29.9545f,-13.8807f,52.7754f,  31.4174f,-11.7046f,52.3906f,  29.7439f,-10.6094f,52.3444f,  
	31.1564f,-8.4240f,51.8675f,  29.5007f,-7.3222f,52.0842f,  30.8627f,-5.1300f,51.5163f,  29.2643f,-4.0238f,51.9891f,  30.5753f,-1.8279f,51.3310f,  
	29.0723f,-0.7190f,52.0477f,  30.3309f,1.4762f,51.3005f,  28.9595f,2.5864f,52.2433f,  30.1648f,4.7776f,51.4077f,  28.9582f,5.8876f,52.5540f,  
	30.1093f,8.0715f,51.6304f,  29.0971f,9.1796f,52.9532f,  30.1932f,11.3532f,51.9417f,  29.4007f,12.4570f,53.4100f,  30.4402f,14.6163f,52.3111f,  
	29.8881f,15.7127f,53.8907f,  30.8701f,17.8551f,52.7040f,  30.5736f,18.9397f,54.3587f,  31.4969f,21.0624f,53.0835f,  31.4659f,22.1297f,54.7753f,  
	32.3289f,24.2296f,53.4110f,  32.5674f,25.2724f,55.1015f,  33.3688f,27.3465f,53.6470f,  33.8748f,28.3561f,55.2983f,  34.6129f,30.4013f,53.7523f,  
	35.3788f,31.3674f,55.3278f,  36.0517f,33.3804f,53.6888f,  37.0644f,34.2906f,55.1544f,  37.6702f,36.2680f,53.4205f,  38.9110f,37.1087f,54.7455f,  
	39.4472f,39.0455f,52.9154f,  40.8928f,39.8016f,54.0733f,  41.3566f,41.6915f,52.1470f,  42.9794f,42.3461f,53.1166f,  43.3683f,44.1833f,51.0933f,  
	45.1370f,44.7192f,51.8591f,  45.4478f,46.4944f,49.7411f,  47.3283f,46.8936f,50.2946f,  47.5585f,48.5989f,48.0833f,  49.5148f,48.8437f,48.4224f,  
	49.6616f,50.4670f,46.1244f,  51.6565f,50.5403f,46.2547f,  51.7178f,52.0699f,43.8779f,  53.7140f,51.9564f,43.8119f,  53.6888f,53.3803f,41.3657f,  
	55.6497f,53.0670f,41.1231f,  55.5376f,54.3722f,38.6218f,  57.4281f,53.8494f,38.2287f,  57.2305f,55.0225f,35.6914f,  59.0181f,54.2845f,35.1795f,  
	58.7379f,55.3153f,32.6248f,  60.3929f,54.3616f,32.0303f,  60.0348f,55.2404f,29.4816f,  61.5314f,54.0760f,28.8440f,  61.1019f,54.7966f,26.3268f,  
	62.4178f,53.4324f,25.6870f,  61.9251f,53.9921f,23.2265f,  63.0427f,52.4456f,22.6253f,  62.4960f,52.8460f,20.2473f,  63.4015f,51.1407f,19.7236f,  
	62.8116f,51.3866f,17.4520f,  63.4955f,49.5520f,17.0414f,  62.8729f,49.6532f,14.8974f,  63.3297f,47.7240f,14.6300f,  62.6841f,47.6944f,12.6292f,  
	62.9117f,45.7094f,12.5286f,  62.2538f,45.5637f,10.6865f,  62.2536f,43.5649f,10.7691f,  61.5912f,43.3218f,9.0925f,  61.3676f,41.3527f,9.3663f,  
	60.7069f,41.0318f,7.8563f,  60.2668f,39.1363f,8.3206f,  59.6128f,38.7558f,6.9759f,  58.9659f,36.9764f,7.6220f,  58.3212f,36.5552f,6.4346f,  
	57.4793f,34.9316f,7.2454f,  56.8445f,34.4859f,6.2034f,  55.8217f,33.0539f,7.1549f,  55.1964f,32.5982f,6.2422f,  27.9375f,-36.4421f,61.5661f,  
	29.8060f,-34.3630f,61.8868f,  29.2016f,-33.6077f,60.1358f,  31.0175f,-31.4948f,60.3467f,  30.1810f,-30.6696f,58.7276f,  31.9436f,-28.5282f,58.8315f,  
	30.8936f,-27.6450f,57.3756f,  32.6014f,-25.4811f,57.3753f,  31.3618f,-24.5505f,56.1107f,  33.0152f,-22.3679f,56.0090f,  31.6145f,-21.3986f,54.9600f,  
	33.2137f,-19.2009f,54.7594f,  31.6844f,-18.2002f,53.9458f,  33.2285f,-15.9923f,53.6487f,  31.6062f,-14.9661f,53.0854f,  33.0948f,-12.7518f,52.6940f,  
	31.4174f,-11.7046f,52.3906f,  32.8496f,-9.4888f,51.9072f,  31.1564f,-8.4240f,51.8675f,  32.5321f,-6.2097f,51.2939f,  30.8627f,-5.1300f,51.5163f,  
	32.1816f,-2.9208f,50.8542f,  30.5753f,-1.8279f,51.3310f,  31.8363f,0.3715f,50.5824f,  30.3309f,1.4762f,51.3005f,  31.5335f,3.6624f,50.4667f,  
	30.1648f,4.7776f,51.4077f,  31.3086f,6.9476f,50.4899f,  30.1093f,8.0715f,51.6304f,  31.1940f,10.2224f,50.6295f,  30.1932f,11.3532f,51.9417f,  
	31.2177f,13.4814f,50.8587f,  30.4402f,14.6163f,52.3111f,  31.4043f,16.7197f,51.1460f,  30.8701f,17.8551f,52.7040f,  31.7733f,19.9316f,51.4565f,  
	31.4969f,21.0624f,53.0835f,  32.3385f,23.1098f,51.7532f,  32.3289f,24.2296f,53.4110f,  33.1081f,26.2459f,51.9969f,  33.3688f,27.3465f,53.6470f,  
	34.0845f,29.3297f,52.1478f,  34.6129f,30.4013f,53.7523f,  35.2641f,32.3495f,52.1660f,  36.0517f,33.3804f,53.6888f,  36.6372f,35.2914f,52.0129f,  
	37.6702f,36.2680f,53.4205f,  38.1879f,38.1384f,51.6531f,  39.4472f,39.0455f,52.9154f,  39.8949f,40.8707f,51.0553f,  41.3566f,41.6915f,52.1470f,  
	41.7322f,43.4669f,50.1921f,  43.3683f,44.1833f,51.0933f,  43.6688f,45.9014f,49.0443f,  45.4478f,46.4944f,49.7411f,  45.6708f,48.1485f,47.5977f,  
	47.5585f,48.5989f,48.0833f,  47.7012f,50.1782f,45.8499f,  49.6616f,50.4670f,46.1244f,  49.7216f,51.9604f,43.8069f,  51.7178f,52.0699f,43.8779f,  
	51.6936f,53.4663f,41.4833f,  53.6888f,53.3803f,41.3657f,  53.5794f,54.6668f,38.9063f,  55.5376f,54.3722f,38.6218f,  55.3437f,55.5351f,36.1146f,  
	57.2305f,55.0225f,35.6914f,  56.9541f,56.0508f,33.1533f,  58.7379f,55.3153f,32.6248f,  58.3829f,56.1985f,30.0777f,  60.0348f,55.2404f,29.4816f,  
	59.6068f,55.9706f,26.9504f,  61.1019f,54.7966f,26.3268f,  60.6083f,55.3692f,23.8363f,  61.9251f,53.9921f,23.2265f,  61.3747f,54.4067f,20.8029f,  
	62.4960f,52.8460f,20.2473f,  61.8990f,53.1054f,17.9155f,  62.8116f,51.3866f,17.4520f,  62.1786f,51.4987f,15.2353f,  62.8729f,49.6532f,14.8974f,  
	62.2135f,49.6300f,12.8134f,  62.6841f,47.6944f,12.6292f,  62.0094f,47.5493f,10.6960f,  62.2538f,45.5637f,10.6865f,  61.5723f,45.3142f,8.9139f,  
	61.5912f,43.3218f,9.0925f,  60.9096f,42.9871f,7.4848f,  60.7069f,41.0318f,7.8563f,  60.0310f,40.6300f,6.4149f,  59.6128f,38.7558f,6.9759f,  
	58.9461f,38.3062f,5.6960f,  58.3212f,36.5552f,6.4346f,  57.6652f,36.0747f,5.3066f,  56.8445f,34.4859f,6.2034f,  56.1993f,33.9904f,5.2136f,  
	55.1964f,32.5982f,6.2422f,  54.5608f,32.0998f,5.3769f,  29.8060f,-34.3630f,61.8868f,  31.6792f,-32.1833f,62.1045f,  31.0175f,-31.4948f,60.3467f,  
	32.8344f,-29.2872f,60.4540f,  31.9436f,-28.5282f,58.8315f,  33.7021f,-26.3000f,58.8314f,  32.6014f,-25.4811f,57.3753f,  34.3011f,-23.2374f,57.2710f,  
	33.0152f,-22.3679f,56.0090f,  34.6558f,-20.1134f,55.8034f,  33.2137f,-19.2009f,54.7594f,  34.7937f,-16.9416f,54.4555f,  33.2285f,-15.9923f,53.6487f,  
	34.7472f,-13.7331f,53.2494f,  33.0948f,-12.7518f,52.6940f,  34.5513f,-10.4983f,52.2024f,  32.8496f,-9.4888f,51.9072f,  34.2439f,-7.2446f,51.3259f,  
	32.5321f,-6.2097f,51.2939f,  33.8638f,-3.9791f,50.6255f,  32.1816f,-2.9208f,50.8542f,  33.4498f,-0.7089f,50.1013f,  31.8363f,0.3715f,50.5824f,  
	33.0408f,2.5613f,49.7473f,  31.5335f,3.6624f,50.4667f,  32.6745f,5.8270f,49.5513f,  31.3086f,6.9476f,50.4899f,  32.3860f,9.0840f,49.4958f,  
	31.1940f,10.2224f,50.6295f,  32.2073f,12.3272f,49.5584f,  31.2177f,13.4814f,50.8587f,  32.1673f,15.5529f,49.7112f,  31.4043f,16.7197f,51.1460f,  
	32.2904f,18.7566f,49.9223f,  31.7733f,19.9316f,51.4565f,  32.5957f,21.9322f,50.1566f,  32.3385f,23.1098f,51.7532f,  33.0972f,25.0731f,50.3762f,  
	33.1081f,26.2459f,51.9969f,  33.8029f,28.1709f,50.5415f,  34.0845f,29.3297f,52.1478f,  34.7151f,31.2157f,50.6119f,  35.2641f,32.3495f,52.1660f,  
	35.8297f,34.1957f,50.5468f,  36.6372f,35.2914f,52.0129f,  37.1365f,37.0957f,50.3080f,  38.1879f,38.1384f,51.6531f,  38.6193f,39.8978f,49.8602f,  
	39.8949f,40.8707f,51.0553f,  40.2567f,42.5822f,49.1711f,  41.7322f,43.4669f,50.1921f,  42.0218f,45.1247f,48.2158f,  43.6688f,45.9014f,49.0443f,  
	43.8840f,47.5000f,46.9740f,  45.6708f,48.1485f,47.5977f,  45.8088f,49.6786f,45.4358f,  47.7012f,50.1782f,45.8499f,  47.7594f,51.6299f,43.6001f,  
	49.7216f,51.9604f,43.8069f,  49.6982f,53.3235f,41.4741f,  51.6936f,53.4663f,41.4833f,  51.5872f,54.7280f,39.0774f,  53.5794f,54.6668f,38.9063f,  
	53.3899f,55.8137f,36.4419f,  55.3437f,55.5351f,36.1146f,  55.0722f,56.5557f,33.6067f,  56.9541f,56.0508f,33.1533f,  56.6038f,56.9337f,30.6221f,  
	58.3829f,56.1985f,30.0777f,  57.9584f,56.9343f,27.5470f,  59.6068f,55.9706f,26.9504f,  59.1148f,56.5533f,24.4442f,  60.6083f,55.3692f,23.8363f,  
	60.0567f,55.7963f,21.3811f,  61.3747f,54.4067f,20.8029f,  60.7735f,54.6791f,18.4242f,  61.8990f,53.1054f,17.9155f,  61.2583f,53.2288f,15.6378f,  
	62.1786f,51.4987f,15.2353f,  61.5080f,51.4832f,13.0781f,  62.2135f,49.6300f,12.8134f,  61.5246f,49.4875f,10.7973f,  62.0094f,47.5493f,10.6960f,  
	61.3110f,47.2960f,8.8335f,  61.5723f,45.3142f,8.9139f,  60.8717f,44.9686f,7.2125f,  60.9096f,42.9871f,7.4848f,  60.2135f,42.5668f,5.9486f,  
	60.0310f,40.6300f,6.4149f,  59.3436f,40.1546f,5.0423f,  58.9461f,38.3062f,5.6960f,  58.2696f,37.7933f,4.4799f,  57.6652f,36.0747f,5.3066f,  
	57.0003f,35.5411f,4.2359f,  56.1993f,33.9904f,5.2136f,  55.5456f,33.4492f,4.2763f,  54.5608f,32.0998f,5.3769f,  53.9163f,31.5629f,4.5564f,  
	31.6792f,-32.1833f,62.1045f,  33.5509f,-29.9090f,62.2152f,  32.8344f,-29.2872f,60.4540f,  34.6449f,-26.9930f,60.4540f,  33.7021f,-26.3000f,58.8314f,  
	35.4502f,-23.9922f,58.7242f,  34.3011f,-23.2374f,57.2710f,  35.9859f,-20.9222f,57.0599f,  34.6558f,-20.1134f,55.8034f,  36.2750f,-17.7979f,55.4921f,  
	34.7937f,-16.9416f,54.4555f,  36.3465f,-14.6318f,54.0474f,  34.7472f,-13.7331f,53.2494f,  36.2324f,-11.4353f,52.7483f,  34.5513f,-10.4983f,52.2024f,  
	35.9687f,-8.2169f,51.6115f,  34.2439f,-7.2446f,51.3259f,  35.5931f,-4.9847f,50.6487f,  33.8638f,-3.9791f,50.6255f,  35.1439f,-1.7464f,49.8655f,  
	33.4498f,-0.7089f,50.1013f,  34.6610f,1.4930f,49.2617f,  33.0408f,2.5613f,49.7473f,  34.1834f,4.7290f,48.8306f,  32.6745f,5.8270f,49.5513f,  
	33.7487f,7.9573f,48.5601f,  32.3860f,9.0840f,49.4958f,  33.3920f,11.1735f,48.4325f,  32.2073f,12.3272f,49.5584f,  33.1460f,14.3748f,48.4242f,  
	32.1673f,15.5529f,49.7112f,  33.0394f,17.5575f,48.5069f,  32.2904f,18.7566f,49.9223f,  33.0964f,20.7170f,48.6484f,  32.5957f,21.9322f,50.1566f,  
	33.3363f,23.8482f,48.8127f,  33.0972f,25.0731f,50.3762f,  33.7728f,26.9446f,48.9612f,  33.8029f,28.1709f,50.5415f,  34.4140f,29.9984f,49.0532f,  
	34.7151f,31.2157f,50.6119f,  35.2615f,32.9996f,49.0472f,  35.8297f,34.1957f,50.5468f,  36.3110f,35.9351f,48.9031f,  37.1365f,37.0957f,50.3080f,  
	37.5516f,38.7895f,48.5820f,  38.6193f,39.8978f,49.8602f,  38.9671f,41.5444f,48.0477f,  40.2567f,42.5822f,49.1711f,  40.5351f,44.1775f,47.2700f,  
	42.0218f,45.1247f,48.2158f,  42.2287f,46.6646f,46.2228f,  43.8840f,47.5000f,46.9740f,  44.0168f,48.9770f,44.8892f,  45.8088f,49.6786f,45.4358f,  
	45.8649f,51.0839f,43.2611f,  47.7594f,51.6299f,43.6001f,  47.7367f,52.9542f,41.3380f,  49.6982f,53.3235f,41.4741f,  49.5949f,54.5549f,39.1320f,  
	51.5872f,54.7280f,39.0774f,  51.4025f,55.8535f,36.6676f,  53.3899f,55.8137f,36.4419f,  53.1243f,56.8213f,33.9773f,  55.0722f,56.5557f,33.6067f,  
	54.7280f,57.4336f,31.1054f,  56.6038f,56.9337f,30.6221f,  56.1847f,57.6713f,28.1065f,  57.9584f,56.9343f,27.5470f,  57.4702f,57.5241f,25.0399f,  
	59.1148f,56.5533f,24.4442f,  58.5648f,56.9909f,21.9719f,  60.0567f,55.7963f,21.3811f,  59.4541f,56.0807f,18.9691f,  60.7735f,54.6791f,18.4242f,  
	60.1281f,54.8140f,16.0981f,  61.2583f,53.2288f,15.6378f,  60.5797f,53.2222f,13.4189f,  61.5080f,51.4832f,13.0781f,  60.8077f,51.3453f,10.9887f,  
	61.5246f,49.4875f,10.7973f,  60.8119f,49.2331f,8.8526f,  61.3110f,47.2960f,8.8335f,  60.5936f,46.9425f,7.0439f,  60.8717f,44.9686f,7.2125f,  
	60.1570f,44.5332f,5.5849f,  60.2135f,42.5668f,5.9486f,  59.5067f,42.0689f,4.4846f,  59.3436f,40.1546f,5.0423f,  58.6475f,39.6121f,3.7374f,  
	58.2696f,37.7933f,4.4799f,  57.5857f,37.2237f,3.3258f,  57.0003f,35.5411f,4.2359f,  56.3287f,34.9587f,3.2225f,  55.5456f,33.4492f,4.2763f,  
	54.8850f,32.8672f,3.3890f,  53.9163f,31.5629f,4.5564f,  53.2646f,30.9911f,3.7791f,  33.5509f,-29.9090f,62.2152f,  35.4138f,-27.5483f,62.2154f,  
	34.6449f,-26.9930f,60.4540f,  36.4428f,-24.6195f,60.3439f,  35.4502f,-23.9922f,58.7242f,  37.1812f,-21.6134f,58.5075f,  35.9859f,-20.9222f,57.0599f,  
	37.6472f,-18.5465f,56.7406f,  36.2750f,-17.7979f,55.4921f,  37.8653f,-15.4324f,55.0742f,  36.3465f,-14.6318f,54.0474f,  37.8642f,-12.2837f,53.5354f,  
	36.2324f,-11.4353f,52.7483f,  37.6772f,-9.1100f,52.1461f,  35.9687f,-8.2169f,51.6115f,  37.3398f,-5.9204f,50.9234f,  35.5931f,-4.9847f,50.6487f,  
	36.8896f,-2.7233f,49.8790f,  35.1439f,-1.7464f,49.8655f,  36.3663f,0.4758f,49.0182f,  34.6610f,1.4930f,49.2617f,  35.8098f,3.6723f,48.3401f,  
	34.1834f,4.7290f,48.8306f,  35.2590f,6.8618f,47.8382f,  33.7487f,7.9573f,48.5601f,  34.7515f,10.0401f,47.5002f,  33.3920f,11.1735f,48.4325f,  
	34.3235f,13.2054f,47.3070f,  33.1460f,14.3748f,48.4242f,  34.0074f,16.3548f,47.2347f,  33.0394f,17.5575f,48.5069f,  33.8319f,19.4848f,47.2545f,  
	33.0964f,20.7170f,48.6484f,  33.8214f,22.5922f,47.3331f,  33.3363f,23.8482f,48.8127f,  33.9948f,25.6719f,47.4338f,  33.7728f,26.9446f,48.9612f,  
	34.3660f,28.7184f,47.5165f,  34.4140f,29.9984f,49.0532f,  34.9425f,31.7236f,47.5399f,  35.2615f,32.9996f,49.0472f,  35.7255f,34.6767f,47.4624f,  
	36.3110f,35.9351f,48.9031f,  36.7102f,37.5647f,47.2426f,  37.5516f,38.7895f,48.5820f,  37.8854f,40.3713f,46.8412f,  38.9671f,41.5444f,48.0477f,  
	39.2339f,43.0761f,46.2231f,  40.5351f,44.1775f,47.2700f,  40.7333f,45.6566f,45.3570f,  42.2287f,46.6646f,46.2228f,  42.3560f,48.0853f,44.2195f,  
	44.0168f,48.9770f,44.8892f,  44.0705f,50.3318f,42.7957f,  45.8649f,51.0839f,43.2611f,  45.8428f,52.3649f,41.0775f,  47.7367f,52.9542f,41.3380f,  
	47.6367f,54.1504f,39.0694f,  49.5949f,54.5549f,39.1320f,  49.4156f,55.6537f,36.7880f,  51.4025f,55.8535f,36.6676f,  51.1438f,56.8431f,34.2588f,  
	53.1243f,56.8213f,33.9773f,  52.7876f,57.6896f,31.5195f,  54.7280f,57.4336f,31.1054f,  54.3161f,58.1690f,28.6192f,  56.1847f,57.6713f,28.1065f,  
	55.7026f,58.2650f,25.6132f,  57.4702f,57.5241f,25.0399f,  56.9243f,57.9700f,22.5653f,  58.5648f,56.9909f,21.9719f,  57.9636f,57.2863f,19.5410f,  
	59.4541f,56.0807f,18.9691f,  58.8071f,56.2271f,16.6082f,  60.1281f,54.8140f,16.0981f,  59.4443f,54.8173f,13.8298f,  60.5797f,53.2222f,13.4189f,  
	59.8709f,53.0909f,11.2671f,  60.8077f,51.3453f,10.9887f,  60.0834f,51.0925f,8.9710f,  60.8119f,49.2331f,8.8526f,  60.0802f,48.8751f,6.9821f,  
	60.5936f,46.9425f,7.0439f,  59.8627f,46.4955f,5.3300f,  60.1570f,44.5332f,5.5849f,  59.4326f,44.0163f,4.0325f,  59.5067f,42.0689f,4.4846f,  
	58.7923f,41.5000f,3.0919f,  58.6475f,39.6121f,3.7374f,  57.9454f,39.0095f,2.4989f,  57.5857f,37.2237f,3.3258f,  56.8967f,36.6023f,2.2336f,  
	56.3287f,34.9587f,3.2225f,  55.6522f,34.3329f,2.2643f,  54.8850f,32.8672f,3.3890f,  54.2192f,32.2484f,2.5502f,  53.2646f,30.9911f,3.7791f,  
	52.6068f,30.3881f,3.0410f,  35.4138f,-27.5483f,62.2154f,  37.2619f,-25.1084f,62.1023f,  36.4428f,-24.6195f,60.3439f,  38.2213f,-22.1752f,60.1214f,  
	37.1812f,-21.6134f,58.5075f,  38.8866f,-19.1746f,58.1799f,  37.6472f,-18.5465f,56.7406f,  39.2776f,-16.1213f,56.3124f,  37.8653f,-15.4324f,55.0742f,  
	39.4188f,-13.0290f,54.5502f,  37.8642f,-12.2837f,53.5354f,  39.3400f,-9.9086f,52.9204f,  37.6772f,-9.1100f,52.1461f,  39.0742f,-6.7701f,51.4450f,  
	37.3398f,-5.9204f,50.9234f,  38.6571f,-3.6229f,50.1416f,  36.8896f,-2.7233f,49.8790f,  38.1276f,-0.4728f,49.0209f,  36.3663f,0.4758f,49.0182f,  
	37.5256f,2.6750f,48.0882f,  35.8098f,3.6723f,48.3401f,  36.8909f,5.8160f,47.3427f,  35.2590f,6.8618f,47.8382f,  36.2627f,8.9464f,46.7774f,  
	34.7515f,10.0401f,47.5002f,  35.6797f,12.0647f,46.3788f,  34.3235f,13.2054f,47.3070f,  35.1779f,15.1690f,46.1274f,  34.0074f,16.3548f,47.2347f,  
	34.7896f,18.2569f,45.9987f,  33.8319f,19.4848f,47.2545f,  34.5440f,21.3265f,45.9627f,  33.8214f,22.5922f,47.3331f,  34.4650f,24.3746f,45.9853f,  
	33.9948f,25.6719f,47.4338f,  34.5719f,27.3976f,46.0282f,  34.3660f,28.7184f,47.5165f,  34.8779f,30.3895f,46.0505f,  34.9425f,31.7236f,47.5399f,  
	35.3901f,33.3419f,46.0105f,  35.7255f,34.6767f,47.4624f,  36.1093f,36.2443f,45.8650f,  36.7102f,37.5647f,47.2426f,  37.0302f,39.0828f,45.5721f,  
	37.8854f,40.3713f,46.8412f,  38.1407f,41.8394f,45.0929f,  39.2339f,43.0761f,46.2231f,  39.4232f,44.4932f,44.3914f,  40.7333f,45.6566f,45.3570f,  
	40.8546f,47.0188f,43.4382f,  42.3560f,48.0853f,44.2195f,  42.4069f,49.3865f,42.2118f,  44.0705f,50.3318f,42.7957f,  44.0488f,51.5656f,40.6970f,  
	45.8428f,52.3649f,41.0775f,  45.7461f,53.5213f,38.8905f,  47.6367f,54.1504f,39.0694f,  47.4632f,55.2178f,36.8009f,  49.4156f,55.6537f,36.7880f,  
	49.1646f,56.6206f,34.4463f,  51.1438f,56.8431f,34.2588f,  50.8158f,57.6972f,31.8572f,  52.7876f,57.6896f,31.5195f,  52.3846f,58.4187f,29.0764f,  
	54.3161f,58.1690f,28.6192f,  53.8422f,58.7634f,26.1542f,  55.7026f,58.2650f,25.6132f,  55.1633f,58.7170f,23.1510f,  56.9243f,57.9700f,22.5653f,  
	56.3274f,58.2751f,20.1302f,  57.9636f,57.2863f,19.5410f,  57.3180f,57.4439f,17.1596f,  58.8071f,56.2271f,16.6082f,  58.1215f,56.2411f,14.3039f,  
	59.4443f,54.8173f,13.8298f,  58.7303f,54.6944f,11.6276f,  59.8709f,53.0909f,11.2671f,  59.1381f,52.8424f,9.1866f,  60.0834f,51.0925f,8.9710f,  
	59.3403f,50.7332f,7.0280f,  60.0802f,48.8751f,6.9821f,  59.3354f,48.4201f,5.1884f,  59.8627f,46.4955f,5.3300f,  59.1227f,45.9634f,3.6937f,  
	59.4326f,44.0163f,4.0325f,  58.7017f,43.4249f,2.5543f,  58.7923f,41.5000f,3.0919f,  58.0734f,40.8678f,1.7694f,  57.9454f,39.0095f,2.4989f,  
	57.2400f,38.3520f,1.3265f,  56.8967f,36.6023f,2.2336f,  56.2048f,35.9351f,1.2013f,  55.6522f,34.3329f,2.2643f,  54.9724f,33.6685f,1.3593f,  
	54.2192f,32.2484f,2.5502f,  53.5491f,31.5970f,1.7554f,  52.6068f,30.3881f,3.0410f,  51.9441f,29.7569f,2.3410f,  37.2619f,-25.1084f,62.1023f,  
	39.0884f,-22.5980f,61.8739f,  38.2213f,-22.1752f,60.1214f,  39.9722f,-19.6714f,59.7851f,  38.8866f,-19.1746f,58.1799f,  40.5592f,-16.6867f,57.7406f,  
	39.2776f,-16.1213f,56.3124f,  40.8695f,-13.6585f,55.7755f,  39.4188f,-13.0290f,54.5502f,  40.9287f,-10.5990f,53.9211f,  39.3400f,-9.9086f,52.9204f,  
	40.7665f,-7.5194f,52.2046f,  39.0742f,-6.7701f,51.4450f,  40.4160f,-4.4298f,50.6487f,  38.6571f,-3.6229f,50.1416f,  39.9147f,-1.3367f,49.2699f,  
	38.1276f,-0.4728f,49.0209f,  39.3016f,1.7542f,48.0792f,  37.5256f,2.6750f,48.0882f,  38.6164f,4.8380f,47.0819f,  36.8909f,5.8160f,47.3427f,  
	37.8996f,7.9111f,46.2766f,  36.2627f,8.9464f,46.7774f,  37.1916f,10.9724f,45.6554f,  35.6797f,12.0647f,46.3788f,  36.5308f,14.0205f,45.2039f,  
	35.1779f,15.1690f,46.1274f,  35.9532f,17.0542f,44.9025f,  34.7896f,18.2569f,45.9987f,  35.4918f,20.0728f,44.7250f,  34.5440f,21.3265f,45.9627f,  
	35.1754f,23.0748f,44.6407f,  34.4650f,24.3746f,45.9853f,  35.0282f,26.0585f,44.6137f,  34.5719f,27.3976f,46.0282f,  35.0690f,29.0202f,44.6047f,  
	34.8779f,30.3895f,46.0505f,  35.3104f,31.9536f,44.5724f,  35.3901f,33.3419f,46.0105f,  35.7593f,34.8511f,44.4727f,  36.1093f,36.2443f,45.8650f,  
	36.4160f,37.7012f,44.2622f,  37.0302f,39.0828f,45.5721f,  37.2740f,40.4885f,43.8987f,  38.1407f,41.8394f,45.0929f,  38.3210f,43.1944f,43.3425f,  
	39.4232f,44.4932f,44.3914f,  39.5384f,45.7956f,42.5586f,  40.8546f,47.0188f,43.4382f,  40.9026f,48.2641f,41.5195f,  42.4069f,49.3865f,42.2118f,  
	42.3854f,50.5700f,40.2030f,  44.0488f,51.5656f,40.6970f,  43.9554f,52.6786f,38.5984f,  45.7461f,53.5213f,38.8905f,  45.5787f,54.5532f,36.7061f,  
	47.4632f,55.2178f,36.8009f,  47.2204f,56.1577f,34.5367f,  49.1646f,56.6206f,34.4463f,  48.8463f,57.4564f,32.1127f,  50.8158f,57.6972f,31.8572f,  
	50.4233f,58.4164f,29.4703f,  52.3846f,58.4187f,29.0764f,  51.9209f,59.0107f,26.6538f,  53.8422f,58.7634f,26.1542f,  53.3120f,59.2190f,23.7191f,  
	55.1633f,58.7170f,23.1510f,  54.5735f,59.0304f,20.7265f,  56.3274f,58.2751f,20.1302f,  55.6862f,58.4436f,17.7426f,  57.3180f,57.4439f,17.1596f,  
	56.6337f,57.4694f,14.8329f,  58.1215f,56.2411f,14.3039f,  57.4054f,56.1284f,12.0640f,  58.7303f,54.6944f,11.6276f,  57.9921f,54.4528f,9.4957f,  
	59.1381f,52.8424f,9.1866f,  58.3865f,52.4850f,7.1809f,  59.3403f,50.7332f,7.0280f,  58.5843f,50.2741f,5.1625f,  59.3354f,48.4201f,5.1884f,  
	58.5822f,47.8769f,3.4740f,  59.1227f,45.9634f,3.6937f,  58.3771f,45.3536f,2.1339f,  58.7017f,43.4249f,2.5543f,  57.9673f,42.7670f,1.1497f,  
	58.0734f,40.8678f,1.7694f,  57.3525f,40.1778f,0.5167f,  57.2400f,38.3520f,1.3265f,  56.5333f,37.6462f,0.2183f,  56.2048f,35.9351f,1.2013f,  
	55.5115f,35.2270f,0.2269f,  54.9724f,33.6685f,1.3593f,  54.2903f,32.9699f,0.5033f,  53.5491f,31.5970f,1.7554f,  52.8763f,30.9166f,1.0037f,  
	51.9441f,29.7569f,2.3410f,  51.2777f,29.1004f,1.6762f,  39.0884f,-22.5980f,61.8739f,  40.8853f,-20.0284f,61.5289f,  39.9722f,-19.6714f,59.7851f,  
	41.6882f,-17.1188f,59.3346f,  40.5592f,-16.6867f,57.7406f,  42.1915f,-14.1614f,57.1903f,  40.8695f,-13.6585f,55.7755f,  42.4161f,-11.1695f,55.1312f,  
	40.9287f,-10.5990f,53.9211f,  42.3877f,-8.1554f,53.1891f,  40.7665f,-7.5194f,52.2046f,  42.1364f,-5.1302f,51.3916f,  40.4160f,-4.4298f,50.6487f,  
	41.6971f,-2.1011f,49.7608f,  39.9147f,-1.3367f,49.2699f,  41.1073f,0.9256f,48.3134f,  39.3016f,1.7542f,48.0792f,  40.4062f,3.9445f,47.0603f,  
	38.6164f,4.8380f,47.0819f,  39.6342f,6.9519f,46.0064f,  37.8996f,7.9111f,46.2766f,  38.8331f,9.9470f,45.1492f,  37.1916f,10.9724f,45.6554f,  
	38.0430f,12.9290f,44.4801f,  36.5308f,14.0205f,45.2039f,  37.3027f,15.8973f,43.9846f,  35.9532f,17.0542f,44.9025f,  36.6487f,18.8526f,43.6411f,  
	35.4918f,20.0728f,44.7250f,  36.1138f,21.7948f,43.4229f,  35.1754f,23.0748f,44.6407f,  35.7271f,24.7241f,43.2972f,  35.0282f,26.0585f,44.6137f,  
	35.5124f,27.6390f,43.2270f,  35.0690f,29.0202f,44.6047f,  35.4879f,30.5355f,43.1725f,  35.3104f,31.9536f,44.5724f,  35.6663f,33.4087f,43.0896f,  
	35.7593f,34.8511f,44.4727f,  36.0534f,36.2499f,42.9339f,  36.4160f,37.7012f,44.2622f,  36.6488f,39.0464f,42.6611f,  37.2740f,40.4885f,43.8987f,  
	37.4454f,41.7823f,42.2282f,  38.3210f,43.1944f,43.3425f,  38.4300f,44.4368f,41.5957f,  39.5384f,45.7956f,42.5586f,  39.5832f,46.9839f,40.7305f,  
	40.9026f,48.2641f,41.5195f,  40.8811f,49.3951f,39.6040f,  42.3854f,50.5700f,40.2030f,  42.2953f,51.6365f,38.1981f,  43.9554f,52.6786f,38.5984f,  
	43.7942f,53.6714f,36.5053f,  45.5787f,54.5532f,36.7061f,  45.3447f,55.4623f,34.5283f,  47.2204f,56.1577f,34.5367f,  46.9129f,56.9713f,32.2817f,  
	48.8463f,57.4564f,32.1127f,  48.4656f,58.1619f,29.7941f,  50.4233f,58.4164f,29.4703f,  49.9716f,59.0027f,27.1033f,  51.9209f,59.0107f,26.6538f,  
	51.4020f,59.4675f,24.2598f,  53.3120f,59.2190f,23.7191f,  52.7319f,59.5391f,21.3197f,  54.5735f,59.0304f,20.7265f,  53.9397f,59.2092f,18.3474f,  
	55.6862f,58.4436f,17.7426f,  55.0064f,58.4811f,15.4080f,  56.6337f,57.4694f,14.8329f,  55.9189f,57.3683f,12.5688f,  57.4054f,56.1284f,12.0640f,  
	56.6651f,55.8961f,9.8930f,  57.9921f,54.4528f,9.4957f,  57.2351f,54.1006f,7.4381f,  58.3865f,52.4850f,7.1809f,  57.6222f,52.0257f,5.2526f,  
	58.5843f,50.2741f,5.1625f,  57.8204f,49.7242f,3.3772f,  58.5822f,47.8769f,3.4740f,  57.8241f,47.2532f,1.8379f,  58.3771f,45.3536f,2.1339f,  
	57.6291f,44.6745f,0.6505f,  57.9673f,42.7670f,1.1497f,  57.2325f,42.0485f,-0.1819f,  57.3525f,40.1778f,0.5167f,  56.6322f,39.4370f,-0.6678f,  
	56.5333f,37.6462f,0.2183f,  55.8272f,36.8975f,-0.8276f,  55.5115f,35.2270f,0.2269f,  54.8178f,34.4834f,-0.6939f,  54.2903f,32.9699f,0.5033f,  
	53.6075f,32.2414f,-0.3049f,  52.8763f,30.9166f,1.0037f,  52.2015f,30.2103f,0.2916f,  51.2777f,29.1004f,1.6762f,  40.8853f,-20.0284f,61.5289f,  
	42.6453f,-17.4104f,61.0669f,  41.6882f,-17.1188f,59.3346f,  43.3622f,-14.5290f,58.7702f,  42.1915f,-14.1614f,57.1903f,  43.7768f,-11.6102f,56.5299f,  
	42.4161f,-11.1695f,55.1312f,  43.9102f,-8.6671f,54.3817f,  42.3877f,-8.1554f,53.1891f,  43.7887f,-5.7121f,52.3578f,  42.1364f,-5.1302f,51.3916f,  
	43.4442f,-2.7529f,50.4853f,  41.6971f,-2.1011f,49.7608f,  42.9119f,0.2034f,48.7865f,  41.1073f,0.9256f,48.3134f,  42.2295f,3.1507f,47.2783f,  
	40.4062f,3.9445f,47.0603f,  41.4369f,6.0852f,45.9712f,  39.6342f,6.9519f,46.0064f,  40.5761f,9.0062f,44.8688f,  38.8331f,9.9470f,45.1492f,  
	39.6886f,11.9130f,43.9684f,  38.0430f,12.9290f,44.4801f,  38.8151f,14.8060f,43.2608f,  37.3027f,15.8973f,43.9846f,  37.9948f,17.6867f,42.7297f,  
	36.6487f,18.8526f,43.6411f,  37.2642f,20.5564f,42.3528f,  36.1138f,21.7948f,43.4229f,  36.6566f,23.4173f,42.1012f,  35.7271f,24.7241f,43.2972f,  
	36.2005f,26.2696f,41.9412f,  35.5124f,27.6390f,43.2270f,  35.9195f,29.1118f,41.8349f,  35.4879f,30.5355f,43.1725f,  35.8316f,31.9419f,41.7393f,  
	35.6663f,33.4087f,43.0896f,  35.9486f,34.7538f,41.6099f,  36.0534f,36.2499f,42.9339f,  36.2757f,37.5380f,41.4011f,  36.6488f,39.0464f,42.6611f,  
	36.8115f,40.2812f,41.0677f,  37.4454f,41.7823f,42.2282f,  37.5482f,42.9656f,40.5661f,  38.4300f,44.4368f,41.5957f,  38.4714f,45.5677f,39.8585f,  
	39.5832f,46.9839f,40.7305f,  39.5616f,48.0610f,38.9102f,  40.8811f,49.3951f,39.6040f,  40.7942f,50.4129f,37.6965f,  42.2953f,51.6365f,38.1981f,  
	42.1405f,52.5875f,36.2019f,  43.7942f,53.6714f,36.5053f,  43.5694f,54.5463f,34.4214f,  45.3447f,55.4623f,34.5283f,  45.0486f,56.2502f,32.3612f,  
	46.9129f,56.9713f,32.2817f,  46.5451f,57.6597f,30.0423f,  48.4656f,58.1619f,29.7941f,  48.0275f,58.7394f,27.4951f,  49.9716f,59.0027f,27.1033f,  
	49.4660f,59.4582f,24.7639f,  51.4020f,59.4675f,24.2598f,  50.8342f,59.7925f,21.8998f,  52.7319f,59.5391f,21.3197f,  52.1084f,59.7275f,18.9636f,  
	53.9397f,59.2092f,18.3474f,  53.2676f,59.2589f,16.0192f,  55.0064f,58.4811f,15.4080f,  54.2961f,58.3929f,13.1335f,  55.9189f,57.3683f,12.5688f,  
	55.1796f,57.1477f,10.3718f,  56.6651f,55.8961f,9.8930f,  55.9059f,55.5524f,7.7953f,  57.2351f,54.1006f,7.4381f,  56.4655f,53.6451f,5.4573f,  
	57.6222f,52.0257f,5.2526f,  56.8503f,51.4735f,3.4049f,  57.8204f,49.7242f,3.3772f,  57.0522f,49.0913f,1.6713f,  57.8241f,47.2532f,1.8379f,  
	57.0645f,46.5576f,0.2803f,  57.6291f,44.6745f,0.6505f,  56.8819f,43.9322f,-0.7574f,  57.2325f,42.0485f,-0.1819f,  56.4996f,41.2769f,-1.4420f,  
	56.6322f,39.4370f,-0.6678f,  55.9141f,38.6512f,-1.7863f,  55.8272f,36.8975f,-0.8276f,  55.1223f,36.1115f,-1.8157f,  54.8178f,34.4834f,-0.6939f,  
	54.1252f,33.7086f,-1.5624f,  53.6075f,32.2414f,-0.3049f,  52.9244f,31.4862f,-1.0691f,  52.2015f,30.2103f,0.2916f,  42.6453f,-17.4104f,61.0669f,  
	44.3616f,-14.7552f,60.4884f,  43.3622f,-14.5290f,58.7702f,  44.9874f,-11.9136f,58.0933f,  43.7768f,-11.6102f,56.5299f,  45.3079f,-9.0460f,55.7619f,  
	43.9102f,-8.6671f,54.3817f,  45.3447f,-6.1655f,53.5306f,  43.7887f,-5.7121f,52.3578f,  45.1262f,-3.2809f,51.4310f,  43.4442f,-2.7529f,50.4853f,  
	44.6845f,-0.4001f,49.4907f,  42.9119f,0.2034f,48.7865f,  44.0552f,2.4702f,47.7321f,  42.2295f,3.1507f,47.2783f,  43.2769f,5.3259f,46.1718f,  
	41.4369f,6.0852f,45.9712f,  42.3910f,8.1660f,44.8191f,  40.5761f,9.0062f,44.8688f,  41.4395f,10.9901f,43.6775f,  39.6886f,11.9130f,43.9684f,  
	40.4644f,13.7989f,42.7434f,  38.8151f,14.8060f,43.2608f,  39.5071f,16.5950f,42.0062f,  37.9948f,17.6867f,42.7297f,  38.6069f,19.3810f,41.4486f,  
	37.2642f,20.5564f,42.3528f,  37.8009f,22.1604f,41.0463f,  36.6566f,23.4173f,42.1012f,  37.1217f,24.9355f,40.7692f,  36.2005f,26.2696f,41.9412f,  
	36.5978f,27.7070f,40.5825f,  35.9195f,29.1118f,41.8349f,  36.2526f,30.4758f,40.4449f,  35.8316f,31.9419f,41.7393f,  36.1034f,33.2385f,40.3129f,  
	35.9486f,34.7538f,41.6099f,  36.1611f,35.9889f,40.1404f,  36.2757f,37.5380f,41.4011f,  36.4301f,38.7166f,39.8808f,  36.8115f,40.2812f,41.0677f,  
	36.9082f,41.4072f,39.4874f,  37.5482f,42.9656f,40.5661f,  37.5862f,44.0399f,38.9182f,  38.4714f,45.5677f,39.8585f,  38.4496f,46.5906f,38.1336f,  
	39.5616f,48.0610f,38.9102f,  39.4777f,49.0289f,37.1021f,  40.7942f,50.4129f,37.6965f,  40.6456f,51.3198f,35.8011f,  42.1405f,52.5875f,36.2019f,  
	41.9250f,53.4253f,34.2177f,  43.5694f,54.5463f,34.4214f,  43.2852f,55.3053f,32.3500f,  45.0486f,56.2502f,32.3612f,  44.6947f,56.9183f,30.2107f,  
	46.5451f,57.6597f,30.0423f,  46.1220f,58.2256f,27.8225f,  48.0275f,58.7394f,27.4951f,  47.5372f,59.1912f,25.2228f,  49.4660f,59.4582f,24.7639f,  
	48.9127f,59.7864f,22.4566f,  50.8342f,59.7925f,21.8998f,  50.2235f,59.9896f,19.5807f,  52.1084f,59.7275f,18.9636f,  51.4468f,59.7894f,16.6561f,  
	53.2676f,59.2589f,16.0192f,  52.5649f,59.1848f,13.7484f,  54.2961f,58.3929f,13.1335f,  53.5613f,58.1861f,10.9238f,  55.1796f,57.1477f,10.3718f,  
	54.4215f,56.8155f,8.2464f,  55.9059f,55.5524f,7.7953f,  55.1340f,55.1044f,5.7730f,  56.4655f,53.6451f,5.4573f,  55.6885f,53.0951f,3.5567f,  
	56.8503f,51.4735f,3.4049f,  56.0745f,50.8363f,1.6370f,  57.0522f,49.0913f,1.6713f,  56.2832f,48.3842f,0.0453f,  57.0645f,46.5576f,0.2803f,  
	56.3068f,45.7964f,-1.2000f,  56.8819f,43.9322f,-0.7574f,  56.1380f,43.1344f,-2.0910f,  56.4996f,41.2769f,-1.4420f,  55.7707f,40.4582f,-2.6327f,  
	55.9141f,38.6512f,-1.7863f,  55.1989f,37.8264f,-2.8430f,  55.1223f,36.1115f,-1.8157f,  54.4206f,35.2932f,-2.7473f,  54.1252f,33.7086f,-1.5624f,  
	53.4342f,32.9063f,-2.3827f,  52.9244f,31.4862f,-1.0691f,  44.3616f,-14.7552f,60.4884f,  46.0274f,-12.0746f,59.7947f,  44.9874f,-11.9136f,58.0933f,  
	46.5568f,-9.2854f,57.3063f,  45.3079f,-9.0460f,55.7619f,  46.7777f,-6.4827f,54.8900f,  45.3447f,-6.1655f,53.5306f,  46.7142f,-3.6762f,52.5817f,  
	45.1262f,-3.2809f,51.4310f,  46.3948f,-0.8745f,50.4138f,  44.6845f,-0.4001f,49.4907f,  45.8520f,1.9147f,48.4141f,  44.0552f,2.4702f,47.7321f,  
	45.1225f,4.6869f,46.6047f,  43.2769f,5.3259f,46.1718f,  44.2465f,7.4407f,45.0009f,  42.3910f,8.1660f,44.8191f,  43.2657f,10.1759f,43.6123f,  
	41.4395f,10.9901f,43.6775f,  42.2225f,12.8933f,42.4413f,  40.4644f,13.7989f,42.7434f,  41.1597f,15.5964f,41.4831f,  39.5071f,16.5950f,42.0062f,  
	40.1190f,18.2885f,40.7259f,  38.6069f,19.3810f,41.4486f,  39.1402f,20.9749f,40.1504f,  37.8009f,22.1604f,41.0463f,  38.2601f,23.6594f,39.7310f,  
	37.1217f,24.9355f,40.7692f,  37.5112f,26.3451f,39.4367f,  36.5978f,27.7070f,40.5825f,  36.9221f,29.0355f,39.2286f,  36.2526f,30.4758f,40.4449f,  
	36.5151f,31.7300f,39.0651f,  36.1034f,33.2385f,40.3129f,  36.3070f,34.4257f,38.9006f,  36.1611f,35.9889f,40.1404f,  36.3078f,37.1154f,38.6878f,  
	36.4301f,38.7166f,39.8808f,  36.5209f,39.7882f,38.3779f,  36.9082f,41.4072f,39.4874f,  36.9428f,42.4264f,37.9258f,  37.5862f,44.0399f,38.9182f,  
	37.5639f,45.0090f,37.2872f,  38.4496f,46.5906f,38.1336f,  38.3684f,47.5079f,36.4252f,  39.4777f,49.0289f,37.1021f,  39.3353f,49.8902f,35.3097f,  
	40.6456f,51.3198f,35.8011f,  40.4396f,52.1186f,33.9208f,  41.9250f,53.4253f,34.2177f,  41.6531f,54.1530f,32.2482f,  43.2852f,55.3053f,32.3500f,  
	42.9459f,55.9505f,30.2963f,  44.6947f,56.9183f,30.2107f,  44.2877f,57.4698f,28.0798f,  46.1220f,58.2256f,27.8225f,  45.6486f,58.6711f,25.6286f,  
	47.5372f,59.1912f,25.2228f,  47.0005f,59.5208f,22.9809f,  48.9127f,59.7864f,22.4566f,  48.3174f,59.9911f,20.1881f,  50.2235f,59.9896f,19.5807f,  
	49.5754f,60.0637f,17.3077f,  51.4468f,59.7894f,16.6561f,  50.7549f,59.7302f,14.4030f,  52.5649f,59.1848f,13.7484f,  51.8377f,58.9935f,11.5397f,  
	53.5613f,58.1861f,10.9238f,  52.8074f,57.8682f,8.7835f,  54.4215f,56.8155f,8.2464f,  53.6506f,56.3787f,6.1944f,  55.1340f,55.1044f,5.7730f,  
	54.3549f,54.5610f,3.8299f,  55.6885f,53.0951f,3.5567f,  54.9079f,52.4585f,1.7356f,  56.0745f,50.8363f,1.6370f,  55.2986f,50.1230f,-0.0503f,  
	56.2832f,48.3842f,0.0453f,  55.5169f,47.6094f,-1.5020f,  56.3068f,45.7964f,-1.2000f,  55.5535f,44.9776f,-2.6036f,  56.1380f,43.1344f,-2.0910f,  
	55.3994f,42.2875f,-3.3523f,  55.7707f,40.4582f,-2.6327f,  55.0461f,39.5988f,-3.7582f,  55.1989f,37.8264f,-2.8430f,  54.4885f,36.9680f,-3.8393f,  
	54.4206f,35.2932f,-2.7473f,  53.7219f,34.4464f,-3.6267f,  53.4342f,32.9063f,-2.3827f,  46.0274f,-12.0746f,59.7947f,  47.6356f,-9.3814f,58.9882f,  
	46.5568f,-9.2854f,57.3063f,  48.0633f,-6.6583f,56.4126f,  46.7777f,-6.4827f,54.8900f,  48.1812f,-3.9318f,53.9177f,  46.7142f,-3.6762f,52.5817f,  
	48.0134f,-1.2117f,51.5400f,  46.3948f,-0.8745f,50.4138f,  47.5891f,1.4937f,49.3124f,  45.8520f,1.9147f,48.4141f,  46.9423f,4.1791f,47.2624f,  
	45.1225f,4.6869f,46.6047f,  46.1111f,6.8429f,45.4112f,  44.2465f,7.4407f,45.0009f,  45.1359f,9.4844f,43.7740f,  43.2657f,10.1759f,43.6123f,  
	44.0592f,12.1047f,42.3597f,  42.2225f,12.8933f,42.4413f,  42.9244f,14.7077f,41.1693f,  41.1597f,15.5964f,41.4831f,  41.7745f,17.2977f,40.1970f,  
	40.1190f,18.2885f,40.7259f,  40.6519f,19.8810f,39.4288f,  39.1402f,20.9749f,40.1504f,  39.5963f,22.4632f,38.8445f,  38.2601f,23.6594f,39.7310f,  
	38.6441f,25.0493f,38.4171f,  37.5112f,26.3451f,39.4367f,  37.8285f,27.6456f,38.1111f,  36.9221f,29.0355f,39.2286f,  37.1767f,30.2541f,37.8878f,  
	36.5151f,31.7300f,39.0651f,  36.7109f,32.8752f,37.7027f,  36.3070f,34.4257f,38.9006f,  36.4466f,35.5050f,37.5090f,  36.3078f,37.1154f,38.6878f,  
	36.3929f,38.1363f,37.2568f,  36.5209f,39.7882f,38.3779f,  36.5521f,40.7550f,36.8983f,  36.9428f,42.4264f,37.9258f,  36.9198f,43.3432f,36.3858f,  
	37.5639f,45.0090f,37.2872f,  37.4853f,45.8761f,35.6772f,  38.3684f,47.5079f,36.4252f,  38.2319f,48.3232f,34.7362f,  39.3353f,49.8902f,35.3097f,  
	39.1385f,50.6486f,33.5356f,  40.4396f,52.1186f,33.9208f,  40.1801f,52.8129f,32.0575f,  41.6531f,54.1530f,32.2482f,  41.3288f,54.7728f,30.2977f,  
	42.9459f,55.9505f,30.2963f,  42.5560f,56.4851f,28.2627f,  44.2877f,57.4698f,28.0798f,  43.8324f,57.9069f,25.9743f,  45.6486f,58.6711f,25.6286f,  
	45.1301f,59.0003f,23.4634f,  47.0005f,59.5208f,22.9809f,  46.4227f,59.7319f,20.7754f,  48.3174f,59.9911f,20.1881f,  47.6853f,60.0769f,17.9630f,  
	49.5754f,60.0637f,17.3077f,  48.8971f,60.0200f,15.0860f,  50.7549f,59.7302f,14.4030f,  50.0383f,59.5563f,12.2088f,  51.8377f,58.9935f,11.5397f,  
	51.0912f,58.6926f,9.3976f,  52.8074f,57.8682f,8.7835f,  52.0405f,57.4463f,6.7142f,  53.6506f,56.3787f,6.1944f,  52.8723f,55.8462f,4.2199f,  
	54.3549f,54.5610f,3.8299f,  53.5722f,53.9300f,1.9653f,  54.9079f,52.4585f,1.7356f,  54.1274f,51.7443f,-0.0049f,  55.2986f,50.1230f,-0.0503f,  
	54.5259f,49.3402f,-1.6583f,  55.5169f,47.6094f,-1.5020f,  54.7560f,46.7752f,-2.9712f,  55.5535f,44.9776f,-2.6036f,  54.8067f,44.1078f,-3.9329f,  
	55.3994f,42.2875f,-3.3523f,  54.6665f,41.3982f,-4.5456f,  55.0461f,39.5988f,-3.7582f,  54.3279f,38.7043f,-4.8198f,  54.4885f,36.9680f,-3.8393f,  
	53.7829f,36.0800f,-4.7798f,  53.7219f,34.4464f,-3.6267f,  47.6356f,-9.3814f,58.9882f,  49.1794f,-6.6892f,58.0725f,  48.0633f,-6.6583f,56.4126f,  
	49.5019f,-4.0435f,55.4161f,  48.1812f,-3.9318f,53.9177f,  49.5128f,-1.4059f,52.8501f,  48.0134f,-1.2117f,51.5400f,  49.2370f,1.2143f,50.4118f,  
	47.5891f,1.4937f,49.3124f,  48.7051f,3.8113f,48.1338f,  46.9423f,4.1791f,47.2624f,  47.9527f,6.3827f,46.0428f,  46.1111f,6.8429f,45.4112f,  
	47.0182f,8.9274f,44.1599f,  45.1359f,9.4844f,43.7740f,  45.9432f,11.4466f,42.4999f,  44.0592f,12.1047f,42.3597f,  44.7709f,13.9443f,41.0701f,  
	42.9244f,14.7077f,41.1693f,  43.5451f,16.4255f,39.8709f,  41.7745f,17.2977f,40.1970f,  42.3099f,18.8976f,38.8940f,  40.6519f,19.8810f,39.4288f,  
	41.1073f,21.3674f,38.1247f,  39.5963f,22.4632f,38.8445f,  39.9771f,23.8419f,37.5411f,  38.6441f,25.0493f,38.4171f,  38.9564f,26.3299f,37.1117f,  
	37.8285f,27.6456f,38.1111f,  38.0769f,28.8360f,36.8012f,  37.1767f,30.2541f,37.8878f,  37.3658f,31.3639f,36.5673f,  36.7109f,32.8752f,37.7027f,  
	36.8441f,33.9129f,36.3648f,  36.4466f,35.5050f,37.5090f,  36.5265f,36.4798f,36.1431f,  36.3929f,38.1363f,37.2568f,  36.4209f,39.0542f,35.8532f,  
	36.5521f,40.7550f,36.8983f,  36.5283f,41.6219f,35.4446f,  36.9198f,43.3432f,36.3858f,  36.8434f,44.1614f,34.8710f,  37.4853f,45.8761f,35.6772f,  
	37.3544f,46.6454f,34.0903f,  38.2319f,48.3232f,34.7362f,  38.0442f,49.0403f,33.0689f,  39.1385f,50.6486f,33.5356f,  38.8913f,51.3079f,31.7813f,  
	40.1801f,52.8129f,32.0575f,  39.8710f,53.4053f,30.2149f,  41.3288f,54.7728f,30.2977f,  40.9564f,55.2882f,28.3681f,  42.5560f,56.4851f,28.2627f,  
	42.1197f,56.9116f,26.2540f,  43.8324f,57.9069f,25.9743f,  43.3336f,58.2337f,23.8961f,  45.1301f,59.0003f,23.4634f,  44.5717f,59.2165f,21.3325f,  
	46.4227f,59.7319f,20.7754f,  45.8088f,59.8290f,18.6107f,  47.6853f,60.0769f,17.9630f,  47.0231f,60.0491f,15.7857f,  48.8971f,60.0200f,15.0860f,  
	48.1940f,59.8646f,12.9197f,  50.0383f,59.5563f,12.2088f,  49.3023f,59.2746f,10.0781f,  51.0912f,58.6926f,9.3976f,  50.3315f,58.2888f,7.3235f,  
	52.0405f,57.4463f,6.7142f,  51.2660f,56.9288f,4.7200f,  52.8723f,55.8462f,4.2199f,  52.0903f,55.2256f,2.3223f,  53.5722f,53.9300f,1.9653f,  
	52.7898f,53.2202f,0.1807f,  54.1274f,51.7443f,-0.0049f,  53.3506f,50.9591f,-1.6662f,  54.5259f,49.3402f,-1.6583f,  53.7593f,48.4962f,-3.1874f,  
	54.7560f,46.7752f,-2.9712f,  54.0026f,45.8881f,-4.3645f,  54.8067f,44.1078f,-3.9329f,  54.0667f,43.1940f,-5.1917f,  54.6665f,41.3982f,-4.5456f,  
	53.9415f,40.4726f,-5.6719f,  54.3279f,38.7043f,-4.8198f,  53.6159f,37.7792f,-5.8223f,  53.7829f,36.0800f,-4.7798f,  49.1794f,-6.6892f,58.0725f,  
	50.6538f,-4.0094f,57.0513f,  49.5019f,-4.0435f,55.4161f,  50.8672f,-1.4538f,54.3216f,  49.5128f,-1.4059f,52.8501f,  50.7673f,1.0815f,51.6935f,  
	49.2370f,1.2143f,50.4118f,  50.3808f,3.5898f,49.2040f,  48.7051f,3.8113f,48.1338f,  49.7398f,6.0680f,46.8849f,  47.9527f,6.3827f,46.0428f,  
	48.8804f,8.5145f,44.7632f,  47.0182f,8.9274f,44.1599f,  47.8421f,10.9303f,42.8596f,  45.9432f,11.4466f,42.4999f,  46.6676f,13.3193f,41.1874f,  
	44.7709f,13.9443f,41.0701f,  45.4006f,15.6869f,39.7533f,  43.5451f,16.4255f,39.8709f,  44.0859f,18.0413f,38.5551f,  42.3099f,18.8976f,38.8940f,  
	42.7674f,20.3907f,37.5841f,  41.1073f,21.3674f,38.1247f,  41.4874f,22.7436f,36.8237f,  39.9771f,23.8419f,37.5411f,  40.2864f,25.1109f,36.2473f,  
	38.9564f,26.3299f,37.1117f,  39.2002f,27.4999f,35.8239f,  38.0769f,28.8360f,36.8012f,  38.2606f,29.9175f,35.5140f,  37.3658f,31.3639f,36.5673f,  
	37.4936f,32.3664f,35.2746f,  36.8441f,33.9129f,36.3648f,  36.9193f,34.8470f,35.0560f,  36.5265f,36.4798f,36.1431f,  36.5513f,37.3531f,34.8084f,  
	36.4209f,39.0542f,35.8532f,  36.3961f,39.8744f,34.4799f,  36.5283f,41.6219f,35.4446f,  36.4538f,42.3931f,34.0204f,  36.8434f,44.1614f,34.8710f,  
	36.7176f,44.8857f,33.3831f,  37.3544f,46.6454f,34.0903f,  37.1752f,47.3212f,32.5286f,  38.0442f,49.0403f,33.0689f,  37.8090f,49.6638f,31.4242f,  
	38.8913f,51.3079f,31.7813f,  38.5975f,51.8714f,30.0493f,  39.8710f,53.4053f,30.2149f,  39.5164f,53.8998f,28.3940f,  40.9564f,55.2882f,28.3681f,  
	40.5399f,55.7022f,26.4631f,  42.1197f,56.9116f,26.2540f,  41.6418f,57.2343f,24.2715f,  43.3336f,58.2337f,23.8961f,  42.7962f,58.4537f,21.8500f,  
	44.5717f,59.2165f,21.3325f,  43.9781f,59.3240f,19.2397f,  45.8088f,59.8290f,18.6107f,  45.1652f,59.8171f,16.4903f,  47.0231f,60.0491f,15.7857f,  
	46.3362f,59.9133f,13.6602f,  48.1940f,59.8646f,12.9197f,  47.4712f,59.6042f,10.8134f,  49.3023f,59.2746f,10.0781f,  48.5526f,58.8919f,8.0120f,  
	50.3315f,58.2888f,7.3235f,  49.5637f,57.7903f,5.3216f,  51.2660f,56.9288f,4.7200f,  50.4876f,56.3233f,2.8004f,  52.0903f,55.2256f,2.3223f,  
	51.3086f,54.5256f,0.5033f,  52.7898f,53.2202f,0.1807f,  52.0112f,52.4384f,-1.5256f,  53.3506f,50.9591f,-1.6662f,  52.5803f,50.1112f,-3.2484f,  
	53.7593f,48.4962f,-3.1874f,  53.0009f,47.5978f,-4.6397f,  54.0026f,45.8881f,-4.3645f,  53.2569f,44.9554f,-5.6855f,  54.0667f,43.1940f,-5.1917f,  
	53.3359f,42.2424f,-6.3811f,  53.9415f,40.4726f,-5.6719f,  53.2239f,39.5150f,-6.7364f,  53.6159f,37.7792f,-5.8223f,  50.6538f,-4.0094f,57.0513f,  
	52.0534f,-1.3546f,55.9294f,  50.8672f,-1.4538f,54.3216f,  52.1539f,1.0973f,53.1355f,  50.7673f,1.0815f,51.6935f,  51.9406f,3.5183f,50.4545f,  
	50.3808f,3.5898f,49.2040f,  51.4419f,5.9043f,47.9232f,  49.7398f,6.0680f,46.8849f,  50.6905f,8.2528f,45.5738f,  48.8804f,8.5145f,44.7632f,  
	49.7236f,10.5645f,43.4326f,  47.8421f,10.9303f,42.8596f,  48.5821f,12.8432f,41.5191f,  46.6676f,13.3193f,41.1874f,  47.3091f,15.0945f,39.8461f,  
	45.4006f,15.6869f,39.7533f,  45.9495f,17.3270f,38.4179f,  44.0859f,18.0413f,38.5551f,  44.5481f,19.5498f,37.2318f,  42.7674f,20.3907f,37.5841f,  
	43.1492f,21.7730f,36.2772f,  41.4874f,22.7436f,36.8237f,  41.7959f,24.0094f,35.5329f,  40.2864f,25.1109f,36.2473f,  40.5275f,26.2688f,34.9725f,  
	39.2002f,27.4999f,35.8239f,  39.3800f,28.5610f,34.5605f,  38.2606f,29.9175f,35.5140f,  38.3839f,30.8919f,34.2571f,  37.4936f,32.3664f,35.2746f,  
	37.5647f,33.2659f,34.0139f,  36.9193f,34.8470f,35.0560f,  36.9412f,35.6809f,33.7819f,  36.5513f,37.3531f,34.8084f,  36.5255f,38.1306f,33.5080f,  
	36.3961f,39.8744f,34.4799f,  36.3231f,40.6015f,33.1400f,  36.4538f,42.3931f,34.0204f,  36.3326f,43.0741f,32.6267f,  36.7176f,44.8857f,33.3831f,  
	36.5465f,45.5209f,31.9240f,  37.1752f,47.3212f,32.5286f,  36.9515f,47.9087f,30.9923f,  37.8090f,49.6638f,31.4242f,  37.5299f,50.1974f,29.8038f,  
	38.5975f,51.8714f,30.0493f,  38.2606f,52.3435f,28.3402f,  39.5164f,53.8998f,28.3940f,  39.1200f,54.2994f,26.5978f,  40.5399f,55.7022f,26.4631f,  
	40.0836f,56.0191f,24.5832f,  41.6418f,57.2343f,24.2715f,  41.1265f,57.4566f,22.3189f,  42.7962f,58.4537f,21.8500f,  42.2245f,58.5708f,19.8393f,  
	43.9781f,59.3240f,19.2397f,  43.3551f,59.3279f,17.1876f,  45.1652f,59.8171f,16.4903f,  44.4968f,59.7016f,14.4178f,  46.3362f,59.9133f,13.6602f,  
	45.6293f,59.6758f,11.5909f,  47.4712f,59.6042f,10.8134f,  46.7343f,59.2451f,8.7677f,  48.5526f,58.8919f,8.0120f,  47.7943f,58.4158f,6.0145f,  
	49.5637f,57.7903f,5.3216f,  48.7915f,57.2042f,3.3915f,  50.4876f,56.3233f,2.8004f,  49.7091f,55.6381f,0.9574f,  51.3086f,54.5256f,0.5033f,  
	50.5306f,53.7528f,-1.2389f,  52.0112f,52.4384f,-1.5256f,  51.2394f,51.5927f,-3.1534f,  52.5803f,50.1112f,-3.2484f,  51.8188f,49.2076f,-4.7538f,  
	53.0009f,47.5978f,-4.6397f,  52.2510f,46.6523f,-6.0185f,  53.2569f,44.9554f,-5.6855f,  52.5214f,43.9836f,-6.9353f,  53.3359f,42.2424f,-6.3811f,  
	52.6135f,41.2576f,-7.5064f,  53.2239f,39.5150f,-6.7364f,  52.0534f,-1.3546f,55.9294f,  53.3730f,1.2616f,54.7131f,  52.1539f,1.0973f,53.1355f,  
	53.3579f,3.5981f,51.8641f,  51.9406f,3.5183f,50.4545f,  53.0299f,5.8942f,49.1400f,  51.4419f,5.9043f,47.9232f,  52.4177f,8.1466f,46.5777f,  
	50.6905f,8.2528f,45.5738f,  51.5554f,10.3555f,44.2091f,  49.7236f,10.5645f,43.4326f,  50.4817f,12.5243f,42.0594f,  48.5821f,12.8432f,41.5191f,  
	49.2381f,14.6584f,40.1477f,  47.3091f,15.0945f,39.8461f,  47.8688f,16.7668f,38.4846f,  45.9495f,17.3270f,38.4179f,  46.4189f,18.8591f,37.0739f,  
	44.5481f,19.5498f,37.2318f,  44.9340f,20.9468f,35.9110f,  43.1492f,21.7730f,36.2772f,  43.4589f,23.0444f,34.9805f,  41.7959f,24.0094f,35.5329f,  
	42.0360f,25.1637f,34.2617f,  40.5275f,26.2688f,34.9725f,  40.7047f,27.3174f,33.7234f,  39.3800f,28.5610f,34.5605f,  39.4998f,29.5146f,33.3297f,  
	38.3839f,30.8919f,34.2571f,  38.4515f,31.7635f,33.0349f,  37.5647f,33.2659f,34.0139f,  37.5838f,34.0661f,32.7911f,  36.9412f,35.6809f,33.7819f,  
	36.9142f,36.4203f,32.5457f,  36.5255f,38.1306f,33.5080f,  36.4536f,38.8173f,32.2447f,  36.3231f,40.6015f,33.1400f,  36.2060f,41.2417f,31.8341f,  
	36.3326f,43.0741f,32.6267f,  36.1688f,43.6701f,31.2653f,  36.5465f,45.5209f,31.9240f,  36.3336f,46.0727f,30.4931f,  36.9515f,47.9087f,30.9923f,  
	36.6867f,48.4120f,29.4824f,  37.5299f,50.1974f,29.8038f,  37.2105f,50.6459f,28.2075f,  38.2606f,52.3435f,28.3402f,  37.8843f,52.7273f,26.6559f,  
	39.1200f,54.2994f,26.5978f,  38.6856f,54.6088f,24.8258f,  40.0836f,56.0191f,24.5832f,  39.5912f,56.2421f,22.7313f,  41.1265f,57.4566f,22.3189f,  
	40.5780f,57.5821f,20.3992f,  42.2245f,58.5708f,19.8393f,  41.6237f,58.5899f,17.8657f,  43.3551f,59.3279f,17.1876f,  42.7072f,59.2329f,15.1793f,  
	44.4968f,59.7016f,14.4178f,  43.8082f,59.4881f,12.3973f,  45.6293f,59.6758f,11.5909f,  44.9078f,59.3426f,9.5779f,  46.7343f,59.2451f,8.7677f,  
	45.9882f,58.7946f,6.7868f,  47.7943f,58.4158f,6.0145f,  47.0311f,57.8532f,4.0854f,  48.7915f,57.2042f,3.3915f,  48.0188f,56.5387f,1.5353f,  
	49.7091f,55.6381f,0.9574f,  48.9341f,54.8798f,-0.8110f,  50.5306f,53.7528f,-1.2389f,  49.7593f,52.9153f,-2.9039f,  51.2394f,51.5927f,-3.1534f,  
	50.4764f,50.6902f,-4.7048f,  51.8188f,49.2076f,-4.7538f,  51.0662f,48.2556f,-6.1850f,  52.2510f,46.6523f,-6.0185f,  51.5121f,45.6663f,-7.3250f,  
	52.5214f,43.9836f,-6.9353f,  51.7953f,42.9772f,-8.1192f,  52.6135f,41.2576f,-7.5064f,  53.3730f,1.2616f,54.7131f,  54.6085f,3.8277f,53.4086f,  
	53.3579f,3.5981f,51.8641f,  54.4765f,6.0379f,50.5143f,  53.0299f,5.8942f,49.1400f,  54.0323f,8.1979f,47.7577f,  52.4177f,8.1466f,46.5777f,  
	53.3062f,10.3070f,45.1758f,  51.5554f,10.3555f,44.2091f,  52.3339f,12.3680f,42.7993f,  50.4817f,12.5243f,42.0594f,  51.1545f,14.3861f,40.6531f,  
	49.2381f,14.6584f,40.1477f,  49.8110f,16.3702f,38.7543f,  47.8688f,16.7668f,38.4846f,  48.3480f,18.3305f,37.1131f,  46.4189f,18.8591f,37.0739f,  
	46.8112f,20.2792f,35.7312f,  44.9340f,20.9468f,35.9110f,  45.2471f,22.2323f,34.5998f,  43.4589f,23.0444f,34.9805f,  43.6999f,24.2035f,33.7036f,  
	42.0360f,25.1637f,34.2617f,  42.2122f,26.2082f,33.0169f,  40.7047f,27.3174f,33.7234f,  40.8222f,28.2582f,32.5083f,  39.4998f,29.5146f,33.3297f,  
	39.5647f,30.3656f,32.1356f,  38.4515f,31.7635f,33.0349f,  38.4682f,32.5365f,31.8531f,  37.5838f,34.0661f,32.7911f,  37.5556f,34.7729f,31.6093f,  
	36.9142f,36.4203f,32.5457f,  36.8430f,37.0708f,31.3500f,  36.4536f,38.8173f,32.2447f,  36.3398f,39.4198f,31.0188f,  36.2060f,41.2417f,31.8341f,  
	36.0486f,41.8004f,30.5639f,  36.1688f,43.6701f,31.2653f,  35.9658f,44.1871f,29.9350f,  36.3336f,46.0727f,30.4931f,  36.0822f,46.5456f,29.0908f,  
	36.6867f,48.4120f,29.4824f,  36.3840f,48.8361f,27.9981f,  37.2105f,50.6459f,28.2075f,  36.8538f,51.0127f,26.6364f,  37.8843f,52.7273f,26.6559f,  
	37.4717f,53.0276f,24.9952f,  38.6856f,54.6088f,24.8258f,  38.2166f,54.8311f,23.0801f,  39.5912f,56.2421f,22.7313f,  39.0667f,56.3749f,20.9099f,  
	40.5780f,57.5821f,20.3992f,  40.0007f,57.6157f,18.5130f,  41.6237f,58.5899f,17.8657f,  40.9981f,58.5155f,15.9319f,  42.7072f,59.2329f,15.1793f,  
	42.0389f,59.0444f,13.2188f,  43.8082f,59.4881f,12.3973f,  43.1042f,59.1825f,10.4287f,  44.9078f,59.3426f,9.5779f,  44.1762f,58.9204f,7.6253f,  
	45.9882f,58.7946f,6.7868f,  45.2363f,58.2593f,4.8704f,  47.0311f,57.8532f,4.0854f,  46.2666f,57.2120f,2.2270f,  48.0188f,56.5387f,1.5353f,  
	47.2490f,55.8002f,-0.2492f,  48.9341f,54.8798f,-0.8110f,  48.1654f,54.0563f,-2.5041f,  49.7593f,52.9153f,-2.9039f,  48.9968f,52.0199f,-4.4936f,  
	50.4764f,50.6902f,-4.7048f,  49.7228f,49.7379f,-6.1822f,  51.0662f,48.2556f,-6.1850f,  50.3251f,47.2617f,-7.5434f,  51.5121f,45.6663f,-7.3250f,  
	50.7832f,44.6444f,-8.5642f,  51.7953f,42.9772f,-8.1192f,  54.6085f,3.8277f,53.4086f,  55.7570f,6.3331f,52.0226f,  54.4765f,6.0379f,50.5143f,  
	55.5067f,8.4058f,49.0938f,  54.0323f,8.1979f,47.7577f,  54.9460f,10.4197f,46.3162f,  53.3062f,10.3070f,45.1758f,  54.1068f,12.3770f,43.7259f,  
	52.3339f,12.3680f,42.7993f,  53.0255f,14.2822f,41.3535f,  51.1545f,14.3861f,40.6531f,  51.7428f,16.1441f,39.2222f,  49.8110f,16.3702f,38.7543f,  
	50.3021f,17.9729f,37.3487f,  48.3480f,18.3305f,37.1131f,  48.7488f,19.7815f,35.7411f,  46.8112f,20.2792f,35.7312f,  47.1298f,21.5870f,34.3971f,  
	45.2471f,22.2323f,34.5998f,  45.4908f,23.4046f,33.3078f,  43.6999f,24.2035f,33.7036f,  43.8766f,25.2521f,32.4531f,  42.2122f,26.2082f,33.0169f,  
	42.3286f,27.1441f,31.8069f,  40.8222f,28.2582f,32.5083f,  40.8851f,29.0960f,31.3314f,  39.5647f,30.3656f,32.1356f,  39.5793f,31.1180f,30.9838f,  
	38.4682f,32.5365f,31.8531f,  38.4388f,33.2165f,30.7150f,  37.5556f,34.7729f,31.6093f,  37.4847f,35.3922f,30.4711f,  36.8430f,37.0708f,31.3500f,  
	36.7318f,37.6395f,30.1948f,  36.3398f,39.4198f,31.0188f,  36.1879f,39.9439f,29.8317f,  36.0486f,41.8004f,30.5639f,  35.8544f,42.2843f,29.3277f,  
	35.9658f,44.1871f,29.9350f,  35.7267f,44.6303f,28.6356f,  36.0822f,46.5456f,29.0908f,  35.7953f,46.9452f,27.7157f,  36.3840f,48.8361f,27.9981f,  
	36.0463f,49.1849f,26.5396f,  36.8538f,51.0127f,26.6364f,  36.4628f,51.3027f,25.0885f,  37.4717f,53.0276f,24.9952f,  37.0261f,53.2478f,23.3593f,  
	38.2166f,54.8311f,23.0801f,  37.7166f,54.9698f,21.3625f,  39.0667f,56.3749f,20.9099f,  38.5139f,56.4220f,19.1185f,  40.0007f,57.6157f,18.5130f,  
	39.3987f,57.5614f,16.6625f,  40.9981f,58.5155f,15.9319f,  40.3517f,58.3522f,14.0413f,  42.0389f,59.0444f,13.2188f,  41.3546f,58.7677f,11.3054f,  
	43.1042f,59.1825f,10.4287f,  42.3894f,58.7909f,8.5156f,  44.1762f,58.9204f,7.6253f,  43.4380f,58.4155f,5.7328f,  45.2363f,58.2593f,4.8704f,  
	44.4823f,57.6464f,3.0206f,  46.2666f,57.2120f,2.2270f,  45.5042f,56.4981f,0.4369f,  47.2490f,55.8002f,-0.2492f,  46.4851f,54.9963f,-1.9610f,  
	48.1654f,54.0563f,-2.5041f,  47.4053f,53.1742f,-4.1238f,  48.9968f,52.0199f,-4.4936f,  48.2436f,51.0739f,-6.0102f,  49.7228f,49.7379f,-6.1822f,  
	48.9808f,48.7426f,-7.5869f,  50.3251f,47.2617f,-7.5434f,  49.5944f,46.2307f,-8.8338f,  50.7832f,44.6444f,-8.5642f,  55.7570f,6.3331f,52.0226f,  
	56.8158f,8.7666f,50.5628f,  55.5067f,8.4058f,49.0938f,  56.4468f,10.6917f,47.6107f,  54.9460f,10.4197f,46.3162f,  55.7703f,12.5511f,44.8234f,  
	54.1068f,12.3770f,43.7259f,  54.8191f,14.3486f,42.2370f,  53.0255f,14.2822f,41.3535f,  53.6313f,16.0924f,39.8804f,  51.7428f,16.1441f,39.2222f,  
	52.2480f,17.7925f,37.7766f,  50.3021f,17.9729f,37.3487f,  50.7137f,19.4623f,35.9404f,  48.7488f,19.7815f,35.7411f,  49.0749f,21.1196f,34.3759f,  
	47.1298f,21.5870f,34.3971f,  47.3780f,22.7808f,33.0810f,  45.4908f,23.4046f,33.3078f,  45.6694f,24.4657f,32.0416f,  43.8766f,25.2521f,32.4531f,  
	43.9930f,26.1914f,31.2376f,  42.3286f,27.1441f,31.8069f,  42.3902f,27.9765f,30.6360f,  40.8851f,29.0960f,31.3314f,  40.8981f,29.8351f,30.1982f,  
	39.5793f,31.1180f,30.9838f,  39.5485f,31.7778f,29.8779f,  38.4388f,33.2165f,30.7150f,  38.3677f,33.8100f,29.6231f,  37.4847f,35.3922f,30.4711f,  
	37.3753f,35.9313f,29.3761f,  36.7318f,37.6395f,30.1948f,  36.5844f,38.1323f,29.0813f,  36.1879f,39.9439f,29.8317f,  36.0013f,40.3967f,28.6815f,  
	35.8544f,42.2843f,29.3277f,  35.6264f,42.6987f,28.1247f,  35.7267f,44.6303f,28.6356f,  35.4545f,45.0053f,27.3651f,  35.7953f,46.9452f,27.7157f,  
	35.4756f,47.2753f,26.3671f,  36.0463f,49.1849f,26.5396f,  35.6761f,49.4634f,25.1041f,  36.4628f,51.3027f,25.0885f,  36.0402f,51.5191f,23.5641f,  
	37.0261f,53.2478f,23.3593f,  36.5506f,53.3908f,21.7494f,  37.7166f,54.9698f,21.3625f,  37.1887f,55.0292f,19.6717f,  38.5139f,56.4220f,19.1185f,  
	37.9364f,56.3872f,17.3588f,  39.3987f,57.5614f,16.6625f,  38.7756f,57.4234f,14.8508f,  40.3517f,58.3522f,14.0413f,  39.6887f,58.1052f,12.1931f,  
	41.3546f,58.7677f,11.3054f,  40.6583f,58.4084f,9.4426f,  42.3894f,58.7909f,8.5156f,  41.6669f,58.3191f,6.6581f,  43.4380f,58.4155f,5.7328f,  
	42.6967f,57.8345f,3.9028f,  44.4823f,57.6464f,3.0206f,  43.7296f,56.9617f,1.2355f,  45.5042f,56.4981f,0.4369f,  44.7469f,55.7190f,-1.2837f,  
	46.4851f,54.9963f,-1.9610f,  45.7291f,54.1333f,-3.6018f,  47.4053f,53.1742f,-4.1238f,  46.6542f,52.2406f,-5.6718f,  48.2436f,51.0739f,-6.0102f,  
	47.5021f,50.0836f,-7.4547f,  48.9808f,48.7426f,-7.5869f,  48.2495f,47.7090f,-8.9235f,  49.5944f,46.2307f,-8.8338f,  56.8158f,8.7666f,50.5628f,  
	57.7828f,11.1184f,49.0372f,  56.4468f,10.6917f,47.6107f,  57.2958f,12.8873f,46.0732f,  55.7703f,12.5511f,44.8234f,  56.5047f,14.5840f,43.2884f,  
	54.8191f,14.3486f,42.2370f,  55.4440f,16.2158f,40.7176f,  53.6313f,16.0924f,39.8804f,  54.1523f,17.7924f,38.3897f,  52.2480f,17.7925f,37.7766f,  
	52.6721f,19.3271f,36.3257f,  50.7137f,19.4623f,35.9404f,  51.0490f,20.8380f,34.5367f,  49.0749f,21.1196f,34.3759f,  49.3292f,22.3427f,33.0271f,  
	47.3780f,22.7808f,33.0810f,  47.5601f,23.8623f,31.7895f,  45.6694f,24.4657f,32.0416f,  45.7870f,25.4163f,30.8101f,  43.9930f,26.1914f,31.2376f,  
	44.0542f,27.0262f,30.0614f,  42.3902f,27.9765f,30.6360f,  42.4019f,28.7097f,29.5096f,  40.8981f,29.8351f,30.1982f,  40.8659f,30.4813f,29.1126f,  
	39.5485f,31.7778f,29.8779f,  39.4768f,32.3513f,28.8205f,  38.3677f,33.8100f,29.6231f,  38.2592f,34.3245f,28.5768f,  37.3753f,35.9313f,29.3761f,  
	37.2312f,36.3966f,28.3256f,  36.5844f,38.1323f,29.0813f,  36.4041f,38.5567f,28.0075f,  36.0013f,40.3967f,28.6815f,  35.7830f,40.7840f,27.5666f,  
	35.6264f,42.6987f,28.1247f,  35.3673f,43.0498f,26.9523f,  35.4545f,45.0053f,27.3651f,  35.1514f,45.3165f,26.1220f,  35.4756f,47.2753f,26.3671f,  
	35.1251f,47.5413f,25.0417f,  35.6761f,49.4634f,25.1041f,  35.2758f,49.6748f,23.6911f,  36.0402f,51.5191f,23.5641f,  35.5889f,51.6651f,22.0639f,  
	36.5506f,53.3908f,21.7494f,  36.0478f,53.4612f,20.1632f,  37.1887f,55.0292f,19.6717f,  36.6362f,55.0127f,18.0088f,  37.9364f,56.3872f,17.3588f,  
	37.3375f,56.2739f,15.6334f,  38.7756f,57.4234f,14.8508f,  38.1351f,57.2064f,13.0766f,  39.6887f,58.1052f,12.1931f,  39.0126f,57.7794f,10.3903f,  
	40.6583f,58.4084f,9.4426f,  39.9534f,57.9718f,7.6303f,  41.6669f,58.3191f,6.6581f,  40.9401f,57.7731f,4.8582f,  42.6967f,57.8345f,3.9028f,  
	41.9554f,57.1831f,2.1330f,  43.7296f,56.9617f,1.2355f,  42.9808f,56.2123f,-0.4839f,  44.7469f,55.7190f,-1.2837f,  43.9968f,54.8807f,-2.9364f,  
	45.7291f,54.1333f,-3.6018f,  44.9816f,53.2181f,-5.1729f,  46.6542f,52.2406f,-5.6718f,  45.9144f,51.2617f,-7.1492f,  47.5021f,50.0836f,-7.4547f,  
	46.7713f,49.0539f,-8.8317f,  48.2495f,47.7090f,-8.9235f,  57.7828f,11.1184f,49.0372f,  58.6573f,13.3800f,47.4536f,  57.2958f,12.8873f,46.0732f,  
	58.0534f,14.9846f,44.4898f,  56.5047f,14.5840f,43.2884f,  57.1500f,16.5124f,41.7195f,  55.4440f,16.2158f,40.7176f,  55.9824f,17.9726f,39.1773f,  
	54.1523f,17.7924f,38.3897f,  54.5906f,19.3781f,36.8904f,  52.6721f,19.3271f,36.3257f,  53.0183f,20.7471f,34.8766f,  51.0490f,20.8380f,34.5367f,  
	51.3111f,22.0977f,33.1470f,  49.3292f,22.3427f,33.0271f,  49.5161f,23.4524f,31.7012f,  47.5601f,23.8623f,31.7895f,  47.6800f,24.8322f,30.5316f,  
	45.7870f,25.4163f,30.8101f,  45.8486f,26.2614f,29.6174f,  44.0542f,27.0262f,30.0614f,  44.0650f,27.7610f,28.9299f,  42.4019f,28.7097f,29.5096f,  
	42.3683f,29.3492f,28.4319f,  40.8659f,30.4813f,29.1126f,  40.7932f,31.0412f,28.0770f,  39.4768f,32.3513f,28.8205f,  39.3686f,32.8465f,27.8106f,  
	38.2592f,34.3245f,28.5768f,  38.1174f,34.7666f,27.5774f,  37.2312f,36.3966f,28.3256f,  37.0558f,36.7957f,27.3171f,  36.4041f,38.5567f,28.0075f,  
	36.1938f,38.9188f,26.9711f,  35.7830f,40.7840f,27.5666f,  35.5354f,41.1120f,26.4844f,  35.3673f,43.0498f,26.9523f,  35.0792f,43.3420f,25.8084f,  
	35.1514f,45.3165f,26.1220f,  34.8191f,45.5692f,24.9025f,  35.1251f,47.5413f,25.0417f,  34.7458f,47.7463f,23.7381f,  35.2758f,49.6748f,23.6911f,  
	34.8477f,49.8221f,22.3007f,  35.5889f,51.6651f,22.0639f,  35.1106f,51.7447f,20.5846f,  36.0478f,53.4612f,20.1632f,  35.5203f,53.4617f,18.6013f,  
	36.6362f,55.0127f,18.0088f,  36.0619f,54.9234f,16.3758f,  37.3375f,56.2739f,15.6334f,  36.7203f,56.0866f,13.9408f,  38.1351f,57.2064f,13.0766f,  
	37.4805f,56.9146f,11.3425f,  39.0126f,57.7794f,10.3903f,  38.3266f,57.3795f,8.6328f,  39.9534f,57.9718f,7.6303f,  39.2428f,57.4633f,5.8707f,  
	40.9401f,57.7731f,4.8582f,  40.2121f,57.1583f,3.1140f,  41.9554f,57.1831f,2.1330f,  41.2169f,56.4675f,0.4247f,  42.9808f,56.2123f,-0.4839f,  
	42.2382f,55.4037f,-2.1391f,  43.9968f,54.8807f,-2.9364f,  43.2544f,53.9897f,-4.5220f,  44.9816f,53.2181f,-5.1729f,  44.2449f,52.2569f,-6.6754f,  
	45.9144f,51.2617f,-7.1492f,  45.1852f,50.2424f,-8.5599f,  46.7713f,49.0539f,-8.8317f,  58.6573f,13.3800f,47.4536f,  59.4386f,15.5433f,45.8206f,  
	58.0534f,14.9846f,44.4898f,  58.7201f,16.9771f,42.8689f,  57.1500f,16.5124f,41.7195f,  57.7070f,18.3300f,40.1260f,  55.9824f,17.9726f,39.1773f,  
	56.4363f,19.6146f,37.6249f,  54.5906f,19.3781f,36.8904f,  54.9492f,20.8485f,35.3899f,  53.0183f,20.7471f,34.8766f,  53.2897f,22.0501f,33.4388f,  
	51.3111f,22.0977f,33.1470f,  51.5042f,23.2428f,31.7780f,  49.5161f,23.4524f,31.7012f,  49.6396f,24.4491f,30.4071f,  47.6800f,24.8322f,30.5316f,  
	47.7428f,25.6952f,29.3116f,  45.8486f,26.2614f,29.6174f,  45.8591f,27.0052f,28.4691f,  44.0650f,27.7610f,28.9299f,  44.0301f,28.4010f,27.8476f,  
	42.3683f,29.3492f,28.4319f,  42.2942f,29.9019f,27.4053f,  40.7932f,31.0412f,28.0770f,  40.6843f,31.5227f,27.0905f,  39.3686f,32.8465f,27.8106f,  
	39.2277f,33.2700f,26.8495f,  38.1174f,34.7666f,27.5774f,  37.9452f,35.1440f,26.6224f,  37.0558f,36.7957f,27.3171f,  36.8517f,37.1352f,26.3483f,  
	36.1938f,38.9188f,26.9711f,  35.9557f,39.2252f,25.9693f,  35.5354f,41.1120f,26.4844f,  35.2604f,41.3857f,25.4317f,  35.0792f,43.3420f,25.8084f,  
	34.7635f,43.5809f,24.6887f,  34.8191f,45.5692f,24.9025f,  34.4594f,45.7667f,23.7042f,  34.7458f,47.7463f,23.7381f,  34.3399f,47.8933f,22.4557f,  
	34.8477f,49.8221f,22.3007f,  34.3933f,49.9092f,20.9288f,  35.1106f,51.7447f,20.5846f,  34.6078f,51.7605f,19.1263f,  35.5203f,53.4617f,18.6013f,  
	34.9708f,53.3949f,17.0651f,  36.0619f,54.9234f,16.3758f,  35.4687f,54.7651f,14.7708f,  36.7203f,56.0866f,13.9408f,  36.0880f,55.8289f,12.2829f,  
	37.4805f,56.9146f,11.3425f,  36.8146f,56.5523f,9.6485f,  38.3266f,57.3795f,8.6328f,  37.6336f,56.9103f,6.9228f,  39.2428f,57.4633f,5.8707f,  
	38.5294f,56.8879f,4.1617f,  40.2121f,57.1583f,3.1140f,  39.4854f,56.4805f,1.4265f,  41.2169f,56.4675f,0.4247f,  40.4832f,55.6933f,-1.2234f,  
	42.2382f,55.4037f,-2.1391f,  41.5023f,54.5422f,-3.7302f,  43.2544f,53.9897f,-4.5220f,  42.5221f,53.0520f,-6.0415f,  44.2449f,52.2569f,-6.6754f,  
	43.5183f,51.2542f,-8.1129f,  45.1852f,50.2424f,-8.5599f,  59.4386f,15.5433f,45.8206f,  60.1273f,17.6018f,44.1461f,  58.7201f,16.9771f,42.8689f,  
	59.2967f,18.8584f,41.2195f,  57.7070f,18.3300f,40.1260f,  58.1777f,20.0323f,38.5166f,  56.4363f,19.6146f,37.6249f,  56.8087f,21.1403f,36.0677f,  
	54.9492f,20.8485f,35.3899f,  55.2311f,22.2007f,33.8973f,  53.2897f,22.0501f,33.4388f,  53.4902f,23.2372f,32.0187f,  51.5042f,23.2428f,31.7780f,  
	51.6321f,24.2734f,30.4385f,  49.6396f,24.4491f,30.4071f,  49.7044f,25.3373f,29.1492f,  47.7428f,25.6952f,29.3116f,  47.7533f,26.4554f,28.1349f,  
	45.8591f,27.0052f,28.4691f,  45.8229f,27.6529f,27.3697f,  44.0301f,28.4010f,27.8476f,  43.9542f,28.9531f,26.8169f,  42.2942f,29.9019f,27.4053f,  
	42.1838f,30.3757f,26.4289f,  40.6843f,31.5227f,27.0905f,  40.5431f,31.9326f,26.1545f,  39.2277f,33.2700f,26.8495f,  39.0572f,33.6297f,25.9350f,  
	37.9452f,35.1440f,26.6224f,  37.7456f,35.4636f,25.7089f,  36.8517f,37.1352f,26.3483f,  36.6212f,37.4216f,25.4159f,  35.9557f,39.2252f,25.9693f,  
	35.6918f,39.4809f,24.9984f,  35.2604f,41.3857f,25.4317f,  34.9592f,41.6104f,24.4041f,  34.7635f,43.5809f,24.6887f,  34.4215f,43.7698f,23.5901f,  
	34.4594f,45.7667f,23.7042f,  34.0741f,45.9118f,22.5262f,  34.3399f,47.8933f,22.4557f,  33.9081f,47.9860f,21.1897f,  34.3933f,49.9092f,20.9288f,  
	33.9144f,49.9384f,19.5747f,  34.6078f,51.7605f,19.1263f,  34.0827f,51.7146f,17.6897f,  34.9708f,53.3949f,17.0651f,  34.4015f,53.2644f,15.5526f,  
	35.4687f,54.7651f,14.7708f,  34.8591f,54.5409f,13.1955f,  36.0880f,55.8289f,12.2829f,  35.4431f,55.5045f,10.6600f,  36.8146f,56.5523f,9.6485f,  
	36.1402f,56.1237f,7.9966f,  37.6336f,56.9103f,6.9228f,  36.9363f,56.3765f,5.2582f,  38.5294f,56.8879f,4.1617f,  37.8159f,56.2508f,2.5045f,  
	39.4854f,56.4805f,1.4265f,  38.7620f,55.7446f,-0.2051f,  40.4832f,55.6933f,-1.2234f,  39.7552f,54.8662f,-2.8112f,  41.5023f,54.5422f,-3.7302f,  
	40.7754f,53.6335f,-5.2584f,  42.5221f,53.0520f,-6.0415f,  41.7992f,52.0719f,-7.4982f,  43.5183f,51.2542f,-8.1129f,  60.1273f,17.6018f,44.1461f,  
	60.7241f,19.5489f,42.4392f,  59.2967f,18.8584f,41.2195f,  59.7850f,20.6243f,39.5502f,  58.1777f,20.0323f,38.5166f,  58.5648f,21.6176f,36.8985f,  
	56.8087f,21.1403f,36.0677f,  57.1022f,22.5469f,34.5148f,  55.2311f,22.2007f,33.8973f,  55.4401f,23.4357f,32.4192f,  53.4902f,23.2372f,32.0187f,  
	53.6236f,24.3083f,30.6253f,  51.6321f,24.2734f,30.4385f,  51.6997f,25.1937f,29.1330f,  49.7044f,25.3373f,29.1492f,  49.7153f,26.1208f,27.9330f,  
	47.7533f,26.4554f,28.1349f,  47.7159f,27.1175f,27.0064f,  45.8229f,27.6529f,27.3697f,  45.7448f,28.2109f,26.3220f,  43.9542f,28.9531f,26.8169f,  
	43.8416f,29.4250f,25.8371f,  42.1838f,30.3757f,26.4289f,  42.0411f,30.7772f,25.5042f,  40.5431f,31.9326f,26.1545f,  40.3727f,32.2788f,25.2668f,  
	39.0572f,33.6297f,25.9350f,  38.8600f,33.9326f,25.0637f,  37.7456f,35.4636f,25.7089f,  37.5204f,35.7321f,24.8336f,  36.6212f,37.4216f,25.4159f,  
	36.3660f,37.6603f,24.5157f,  35.6918f,39.4809f,24.9984f,  35.4027f,39.6914f,24.0535f,  34.9592f,41.6104f,24.4041f,  34.6327f,41.7898f,23.3977f,  
	34.4215f,43.7698f,23.5901f,  34.0548f,43.9114f,22.5111f,  34.0741f,45.9118f,22.5262f,  33.6633f,46.0081f,21.3629f,  33.9081f,47.9860f,21.1897f,  
	33.4519f,48.0264f,19.9389f,  33.9144f,49.9384f,19.5747f,  33.4130f,49.9113f,18.2389f,  34.0827f,51.7146f,17.6897f,  33.5372f,51.6102f,16.2727f,  
	34.4015f,53.2644f,15.5526f,  33.8148f,53.0725f,14.0646f,  34.8591f,54.5409f,13.1955f,  34.2356f,54.2538f,11.6499f,  35.4431f,55.5045f,10.6600f,  
	34.7881f,55.1168f,9.0737f,  36.1402f,56.1237f,7.9966f,  35.4598f,55.6328f,6.3847f,  36.9363f,56.3765f,5.2582f,  36.2369f,55.7826f,3.6402f,  
	37.8159f,56.2508f,2.5045f,  37.1040f,55.5567f,0.8984f,  38.7620f,55.7446f,-0.2051f,  38.0429f,54.9560f,-1.7805f,  39.7552f,54.8662f,-2.8112f,  
	39.0349f,53.9914f,-4.3395f,  40.7754f,53.6335f,-5.2584f,  40.0572f,52.6817f,-6.7264f,  41.7992f,52.0719f,-7.4982f,  60.7241f,19.5489f,42.4392f,  
	61.2306f,21.3803f,40.7080f,  59.7850f,20.6243f,39.5502f,  60.1876f,22.2723f,37.8680f,  58.5648f,21.6176f,36.8985f,  58.8709f,23.0828f,35.2806f,  
	57.1022f,22.5469f,34.5148f,  57.3207f,23.8349f,32.9727f,  55.4401f,23.4357f,32.4192f,  55.5799f,24.5530f,30.9645f,  53.6236f,24.3083f,30.6253f,  
	53.6946f,25.2670f,29.2633f,  51.6997f,25.1937f,29.1330f,  51.7115f,26.0073f,27.8671f,  49.7153f,26.1208f,27.9330f,  49.6768f,26.8042f,26.7638f,  
	47.7159f,27.1175f,27.0064f,  47.6354f,27.6880f,25.9289f,  45.7448f,28.2109f,26.3220f,  45.6292f,28.6871f,25.3252f,  43.8416f,29.4250f,25.8371f,  
	43.6962f,29.8235f,24.9097f,  42.0411f,30.7772f,25.5042f,  41.8693f,31.1143f,24.6291f,  40.3727f,32.2788f,25.2668f,  40.1760f,32.5685f,24.4239f,  
	38.8600f,33.9326f,25.0637f,  38.6380f,34.1855f,24.2325f,  37.5204f,35.7321f,24.8336f,  37.2715f,35.9550f,23.9921f,  36.3660f,37.6603f,24.5157f,  
	36.0866f,37.8568f,23.6428f,  35.4027f,39.6914f,24.0535f,  35.0894f,39.8605f,23.1303f,  34.6327f,41.7898f,23.3977f,  34.2824f,41.9264f,22.4105f,  
	34.0548f,43.9114f,22.5111f,  33.6631f,44.0093f,21.4455f,  33.6633f,46.0081f,21.3629f,  33.2282f,46.0574f,20.2127f,  33.4519f,48.0264f,19.9389f,  
	32.9729f,48.0158f,18.7032f,  33.4130f,49.9113f,18.2389f,  32.8903f,49.8308f,16.9187f,  33.5372f,51.6102f,16.2727f,  32.9729f,51.4489f,14.8754f,  
	33.8148f,53.0725f,14.0646f,  33.2128f,52.8219f,12.6014f,  34.2356f,54.2538f,11.6499f,  33.6004f,53.9069f,10.1356f,  34.7881f,55.1168f,9.0737f,  
	34.1253f,54.6696f,7.5220f,  35.4598f,55.6328f,6.3847f,  34.7756f,55.0837f,4.8142f,  36.2369f,55.7826f,3.6402f,  35.5375f,55.1327f,2.0682f,  
	37.1040f,55.5567f,0.8984f,  36.3949f,54.8102f,-0.6558f,  38.0429f,54.9560f,-1.7805f,  37.3302f,54.1196f,-3.3004f,  39.0349f,53.9914f,-4.3395f,  
	38.3221f,53.0730f,-5.8107f,  40.0572f,52.6817f,-6.7264f,  61.2306f,21.3803f,40.7080f,  61.6493f,23.0931f,38.9597f,  60.1876f,22.2723f,37.8680f,  
	60.5070f,23.7992f,36.1818f,  58.8709f,23.0828f,35.2806f,  59.0996f,24.4279f,33.6696f,  57.3207f,23.8349f,32.9727f,  57.4676f,25.0035f,31.4500f,  
	55.5799f,24.5530f,30.9645f,  55.6551f,25.5559f,29.5379f,  53.6946f,25.2670f,29.2633f,  53.7078f,26.1168f,27.9381f,  51.7115f,26.0073f,27.8671f,  
	51.6720f,26.7182f,26.6462f,  49.6768f,26.8042f,26.7638f,  49.5935f,27.3934f,25.6444f,  47.6354f,27.6880f,25.9289f,  47.5160f,28.1744f,24.9020f,  
	45.6292f,28.6871f,25.3252f,  45.4800f,29.0879f,24.3812f,  43.6962f,29.8235f,24.9097f,  43.5212f,30.1563f,24.0330f,  41.8693f,31.1143f,24.6291f,  
	41.6710f,31.3945f,23.8003f,  40.1760f,32.5685f,24.4239f,  39.9548f,32.8084f,23.6227f,  38.6380f,34.1855f,24.2325f,  38.3928f,34.3942f,23.4364f,  
	37.2715f,35.9550f,23.9921f,  36.9992f,36.1380f,23.1793f,  36.0866f,37.8568f,23.6428f,  35.7838f,38.0150f,22.7924f,  35.0894f,39.8605f,23.1303f,  
	34.7529f,39.9907f,22.2263f,  34.2824f,41.9264f,22.4105f,  33.9074f,42.0237f,21.4361f,  33.6631f,44.0093f,21.4455f,  33.2472f,44.0650f,20.3913f,  
	33.2282f,46.0574f,20.2127f,  32.7702f,46.0605f,19.0747f,  32.9729f,48.0158f,18.7032f,  32.4721f,47.9566f,17.4797f,  32.8903f,49.8308f,16.9187f,  
	32.3479f,49.6979f,15.6140f,  32.9729f,51.4489f,14.8754f,  32.3921f,51.2330f,13.4982f,  33.2128f,52.8219f,12.6014f,  32.5975f,52.5147f,11.1642f,  
	33.6004f,53.9069f,10.1356f,  32.9557f,53.5033f,8.6507f,  34.1253f,54.6696f,7.5220f,  33.4568f,54.1662f,6.0063f,  34.7756f,55.0837f,4.8142f,  
	34.0894f,54.4801f,3.2844f,  35.5375f,55.1327f,2.0682f,  34.8392f,54.4312f,0.5435f,  36.3949f,54.8102f,-0.6558f,  35.6905f,54.0159f,-2.1590f,  
	37.3302f,54.1196f,-3.3004f,  36.6237f,53.2392f,-4.7668f,  38.3221f,53.0730f,-5.8107f,  61.6493f,23.0931f,38.9597f,  61.9825f,24.6838f,37.2028f,  
	60.5070f,23.7992f,36.1818f,  60.7466f,25.2047f,34.4980f,  59.0996f,24.4279f,33.6696f,  59.2544f,25.6520f,32.0738f,  57.4676f,25.0035f,31.4500f,  
	57.5474f,26.0554f,29.9520f,  55.6551f,25.5559f,29.5379f,  55.6699f,26.4473f,28.1449f,  53.7078f,26.1168f,27.9381f,  53.6674f,26.8611f,26.6555f,  
	51.6720f,26.7182f,26.6462f,  51.5858f,27.3322f,25.4733f,  49.5935f,27.3934f,25.6444f,  49.4699f,27.8959f,24.5747f,  47.5160f,28.1744f,24.9020f,  
	47.3619f,28.5830f,23.9278f,  45.4800f,29.0879f,24.3812f,  45.3003f,29.4210f,23.4887f,  43.5212f,30.1563f,24.0330f,  43.3195f,30.4307f,23.2037f,  
	41.6710f,31.3945f,23.8003f,  41.4483f,31.6244f,23.0148f,  39.9548f,32.8084f,23.6227f,  39.7107f,33.0046f,22.8582f,  38.3928f,34.3942f,23.4364f,  
	38.1248f,34.5643f,22.6706f,  36.9992f,36.1380f,23.1793f,  36.7040f,36.2849f,22.3899f,  35.7838f,38.0150f,22.7924f,  35.4584f,38.1375f,21.9616f,  
	34.7529f,39.9907f,22.2263f,  34.3922f,40.0855f,21.3347f,  33.9074f,42.0237f,21.4361f,  33.5085f,42.0832f,20.4717f,  33.2472f,44.0650f,20.3913f,  
	32.8081f,44.0790f,19.3470f,  32.7702f,46.0605f,19.0747f,  32.2896f,46.0198f,17.9461f,  32.4721f,47.9566f,17.4797f,  31.9505f,47.8495f,16.2677f,  
	32.3479f,49.6979f,15.6140f,  31.7877f,49.5145f,14.3249f,  32.3921f,51.2330f,13.4982f,  31.7965f,50.9640f,12.1419f,  32.5975f,52.5147f,11.1642f,  
	31.9709f,52.1538f,9.7513f,  32.9557f,53.5033f,8.6507f,  32.3033f,53.0457f,7.1962f,  33.4568f,54.1662f,6.0063f,  32.7844f,53.6098f,4.5261f,  
	34.0894f,54.4801f,3.2844f,  33.4026f,53.8256f,1.7970f,  34.8392f,54.4312f,0.5435f,  34.1439f,53.6820f,-0.9348f,  35.6905f,54.0159f,-2.1590f,  
	34.9909f,53.1774f,-3.6125f,  36.6237f,53.2392f,-4.7668f,  61.9825f,24.6838f,37.2028f,  62.2333f,26.1518f,35.4438f,  60.7466f,25.2047f,34.4980f,  
	60.9097f,26.4873f,32.8250f,  59.2544f,25.6520f,32.0738f,  59.3394f,26.7570f,30.4985f,  57.5474f,26.0554f,29.9520f,  57.5644f,26.9933f,28.4838f,  
	55.6699f,26.4473f,28.1449f,  55.6288f,27.2303f,26.7915f,  53.6674f,26.8611f,26.6555f,  53.5781f,27.5053f,25.4186f,  51.5858f,27.3322f,25.4733f,  
	51.4572f,27.8561f,24.3489f,  49.4699f,27.8959f,24.5747f,  49.3097f,28.3174f,23.5572f,  47.3619f,28.5830f,23.9278f,  47.1761f,28.9212f,23.0053f,  
	45.3003f,29.4210f,23.4887f,  45.0931f,29.6938f,22.6442f,  43.3195f,30.4307f,23.2037f,  43.0929f,30.6536f,22.4191f,  41.4483f,31.6244f,23.0148f,  
	41.2027f,31.8101f,22.2674f,  39.7107f,33.0046f,22.8582f,  39.4441f,33.1627f,22.1257f,  38.1248f,34.5643f,22.6706f,  37.8343f,34.6998f,21.9296f,  
	36.7040f,36.2849f,22.3899f,  36.3868f,36.3985f,21.6208f,  35.4584f,38.1375f,21.9616f,  35.1091f,38.2277f,21.1433f,  34.3922f,40.0855f,21.3347f,  
	34.0076f,40.1460f,20.4526f,  33.5085f,42.0832f,20.4717f,  33.0862f,42.1051f,19.5156f,  32.8081f,44.0790f,19.3470f,  32.3461f,44.0534f,18.3098f,  
	32.2896f,46.0198f,17.9461f,  31.7874f,45.9352f,16.8254f,  31.9505f,47.8495f,16.2677f,  31.4098f,47.6957f,15.0674f,  31.7877f,49.5145f,14.3249f,  
	31.2112f,49.2814f,13.0520f,  31.7965f,50.9640f,12.1419f,  31.1878f,50.6444f,10.8050f,  31.9709f,52.1538f,9.7513f,  31.3346f,51.7414f,8.3635f,  
	32.3033f,53.0457f,7.1962f,  31.6449f,52.5369f,5.7721f,  32.7844f,53.6098f,4.5261f,  32.1095f,53.0036f,3.0833f,  33.4026f,53.8256f,1.7970f,  
	32.7168f,53.1238f,0.3512f,  34.1439f,53.6820f,-0.9348f,  33.4518f,52.8887f,-2.3675f,  34.9909f,53.1774f,-3.6125f,  62.2333f,26.1518f,35.4438f,  
	62.4051f,27.4953f,33.6908f,  60.9097f,26.4873f,32.8250f,  61.0003f,27.6487f,31.1682f,  59.3394f,26.7570f,30.4985f,  59.3588f,27.7454f,28.9493f,  
	57.5644f,26.9933f,28.4838f,  57.5228f,27.8197f,27.0520f,  55.6288f,27.2303f,26.7915f,  55.5363f,27.9099f,25.4813f,  53.5781f,27.5053f,25.4186f,  
	53.4441f,28.0559f,24.2286f,  51.4572f,27.8561f,24.3489f,  51.2903f,28.2957f,23.2757f,  49.3097f,28.3174f,23.5572f,  49.1165f,28.6652f,22.5912f,  
	47.1761f,28.9212f,23.0053f,  46.9617f,29.1961f,22.1315f,  45.0931f,29.6938f,22.6442f,  44.8605f,29.9126f,21.8458f,  43.0929f,30.6536f,22.4191f,  
	42.8432f,30.8309f,21.6741f,  41.2027f,31.8101f,22.2674f,  40.9346f,31.9571f,21.5538f,  39.4441f,33.1627f,22.1257f,  39.1553f,33.2867f,21.4194f,  
	37.8343f,34.6998f,21.9296f,  37.5221f,34.8034f,21.2099f,  36.3868f,36.3985f,21.6208f,  36.0459f,36.4821f,20.8649f,  35.1091f,38.2277f,21.1433f,  
	34.7361f,38.2866f,20.3342f,  34.0076f,40.1460f,20.4526f,  33.5996f,40.1725f,19.5774f,  33.0862f,42.1051f,19.5156f,  32.6404f,42.0911f,18.5646f,  
	32.3461f,44.0534f,18.3098f,  31.8614f,43.9878f,17.2775f,  31.7874f,45.9352f,16.8254f,  31.2650f,45.8077f,15.7129f,  31.4098f,47.6957f,15.0674f,  
	30.8514f,47.4956f,13.8789f,  31.2112f,49.2814f,13.0520f,  30.6197f,49.0008f,11.7939f,  31.1878f,50.6444f,10.8050f,  30.5674f,50.2756f,9.4883f,  
	31.3346f,51.7414f,8.3635f,  30.6904f,51.2796f,7.0010f,  31.6449f,52.5369f,5.7721f,  30.9822f,51.9794f,4.3803f,  32.1095f,53.0036f,3.0833f,  
	31.4337f,52.3507f,1.6771f,  32.7168f,53.1238f,0.3512f,  32.0327f,52.3779f,-1.0532f,  33.4518f,52.8887f,-2.3675f,  62.4051f,27.4953f,33.6908f,  
	62.5016f,28.7152f,31.9495f,  61.0003f,27.6487f,31.1682f,  61.0224f,28.6907f,29.5331f,  59.3588f,27.7454f,28.9493f,  59.3168f,28.6190f,27.4325f,  
	57.5228f,27.8197f,27.0520f,  57.4269f,28.5389f,25.6602f,  55.5363f,27.9099f,25.4813f,  55.3965f,28.4918f,24.2157f,  53.4441f,28.0559f,24.2286f,  
	53.2695f,28.5182f,23.0881f,  51.2903f,28.2957f,23.2757f,  51.0883f,28.6576f,22.2536f,  49.1165f,28.6652f,22.5912f,  48.8932f,28.9463f,21.6743f,  
	46.9617f,29.1961f,22.1315f,  46.7209f,29.4143f,21.3048f,  44.8605f,29.9126f,21.8458f,  44.6042f,30.0839f,21.0883f,  42.8432f,30.8309f,21.6741f,  
	42.5708f,30.9683f,20.9646f,  40.9346f,31.9571f,21.5538f,  40.6444f,32.0699f,20.8681f,  39.1553f,33.2867f,21.4194f,  38.8450f,33.3796f,20.7358f,  
	37.5221f,34.8034f,21.2099f,  37.1865f,34.8786f,20.5045f,  36.0459f,36.4821f,20.8649f,  35.6814f,36.5367f,20.1187f,  34.7361f,38.2866f,20.3342f,  
	34.3395f,38.3144f,19.5315f,  33.5996f,40.1725f,19.5774f,  33.1677f,40.1665f,18.7062f,  32.6404f,42.0911f,18.5646f,  32.1714f,42.0407f,17.6163f,  
	31.8614f,43.9878f,17.2775f,  31.3556f,43.8829f,16.2505f,  31.2650f,45.8077f,15.7129f,  30.7235f,45.6372f,14.6084f,  30.8514f,47.4956f,13.8789f,  
	30.2765f,47.2511f,12.7010f,  30.6197f,49.0008f,11.7939f,  30.0148f,48.6735f,10.5513f,  30.5674f,50.2756f,9.4883f,  29.9371f,49.8596f,8.1918f,  
	30.6904f,51.2796f,7.0010f,  30.0398f,50.7705f,5.6659f,  30.9822f,51.9794f,4.3803f,  30.3165f,51.3760f,3.0202f,  31.4337f,52.3507f,1.6771f,  
	30.7578f,51.6540f,0.3079f,  32.0327f,52.3779f,-1.0532f,  62.5016f,28.7152f,31.9495f,  62.5267f,29.8131f,30.2254f,  61.0224f,28.6907f,29.5331f,  
	60.9801f,29.6146f,27.9265f,  59.3168f,28.6190f,27.4325f,  59.2176f,29.3817f,25.9523f,  57.4269f,28.5389f,25.6602f,  57.2811f,29.1563f,24.3104f,  
	55.3965f,28.4918f,24.2157f,  55.2136f,28.9811f,22.9978f,  53.2695f,28.5182f,23.0881f,  53.0578f,28.8984f,21.9980f,  51.0883f,28.6576f,22.2536f,  
	50.8546f,28.9487f,21.2804f,  48.8932f,28.9463f,21.6743f,  48.6422f,29.1670f,20.8052f,  46.7209f,29.4143f,21.3048f,  46.4555f,29.5819f,20.5202f,  
	44.6042f,30.0839f,21.0883f,  44.3247f,30.2130f,20.3683f,  42.5708f,30.9683f,20.9646f,  42.2762f,31.0701f,20.2851f,  40.6444f,32.0699f,20.8681f,  
	40.3328f,32.1513f,20.2068f,  38.8450f,33.3796f,20.7358f,  38.5114f,33.4446f,20.0681f,  37.1865f,34.8786f,20.5045f,  36.8274f,34.9264f,19.8095f,  
	35.6814f,36.5367f,20.1187f,  35.2933f,36.5626f,19.3786f,  34.3395f,38.3144f,19.5315f,  33.9188f,38.3125f,18.7322f,  33.1677f,40.1665f,18.7062f,  
	32.7119f,40.1271f,17.8360f,  32.1714f,42.0407f,17.6163f,  31.6802f,41.9542f,16.6709f,  31.3556f,43.8829f,16.2505f,  30.8295f,43.7381f,15.2281f,  
	30.7235f,45.6372f,14.6084f,  30.1639f,45.4253f,13.5108f,  30.2765f,47.2511f,12.7010f,  29.6863f,46.9624f,11.5342f,  30.0148f,48.6735f,10.5513f,  
	29.3980f,48.3010f,9.3242f,  29.9371f,49.8596f,8.1918f,  29.2986f,49.3976f,6.9180f,  30.0398f,50.7705f,5.6659f,  29.3843f,50.2163f,4.3575f,  
	30.3165f,51.3760f,3.0202f,  29.6490f,50.7292f,1.6926f,  30.7578f,51.6540f,0.3079f,  62.5267f,29.8131f,30.2254f,  62.4844f,30.7895f,28.5255f,  
	60.9801f,29.6146f,27.9265f,  60.8777f,30.4237f,26.3526f,  59.2176f,29.3817f,25.9523f,  59.0655f,30.0382f,24.5112f,  57.2811f,29.1563f,24.3104f,  
	57.0894f,29.6765f,23.0062f,  55.2136f,28.9811f,22.9978f,  54.9912f,29.3835f,21.8291f,  53.0578f,28.8984f,21.9980f,  52.8123f,29.2032f,20.9566f,  
	50.8546f,28.9487f,21.2804f,  50.5916f,29.1750f,20.3557f,  48.6422f,29.1670f,20.8052f,  48.3655f,29.3334f,19.9795f,  46.4555f,29.5819f,20.5202f,  
	46.1663f,29.7044f,19.7751f,  44.3247f,30.2130f,20.3683f,  44.0227f,30.3046f,19.6802f,  42.2762f,31.0701f,20.2851f,  41.9600f,31.1396f,19.6317f,  
	40.3328f,32.1513f,20.2068f,  39.9980f,32.2047f,19.5632f,  38.5114f,33.4446f,20.0681f,  38.1544f,33.4831f,19.4122f,  36.8274f,34.9264f,19.8095f,  
	36.4447f,34.9470f,19.1214f,  35.2933f,36.5626f,19.3786f,  34.8809f,36.5609f,18.6421f,  33.9188f,38.3125f,18.7322f,  33.4737f,38.2799f,17.9329f,  
	32.7119f,40.1271f,17.8360f,  32.2332f,40.0545f,16.9669f,  31.6802f,41.9542f,16.6709f,  31.1677f,41.8306f,15.7273f,  30.8295f,43.7381f,15.2281f,  
	30.2840f,43.5545f,14.2095f,  30.1639f,45.4253f,13.5108f,  29.5875f,45.1716f,12.4204f,  29.6863f,46.9624f,11.5342f,  29.0823f,46.6305f,10.3787f,  
	29.3980f,48.3010f,9.3242f,  28.7712f,47.8841f,8.1152f,  29.2986f,49.3976f,6.9180f,  28.6531f,48.8915f,5.6661f,  29.3843f,50.2163f,4.3575f,  
	28.7252f,49.6193f,3.0771f,  29.6490f,50.7292f,1.6926f,  62.4844f,30.7895f,28.5255f,  62.3788f,31.6473f,26.8542f,  60.8777f,30.4237f,26.3526f,  
	60.7193f,31.1222f,24.8146f,  59.0655f,30.0382f,24.5112f,  58.8647f,30.5925f,23.1131f,  57.0894f,29.6765f,23.0062f,  56.8556f,30.1046f,21.7495f,  
	54.9912f,29.3835f,21.8291f,  54.7328f,29.7054f,20.7084f,  52.8123f,29.2032f,20.9566f,  52.5356f,29.4382f,19.9639f,  50.5916f,29.1750f,20.3557f,  
	50.3015f,29.3428f,19.4754f,  48.3655f,29.3334f,19.9795f,  48.0641f,29.4509f,19.1950f,  46.1663f,29.7044f,19.7751f,  45.8540f,29.7863f,19.0640f,  
	44.0227f,30.3046f,19.6802f,  43.6989f,30.3619f,19.0205f,  41.9600f,31.1396f,19.6317f,  41.6207f,31.1801f,18.9985f,  39.9980f,32.2047f,19.5632f,  
	39.6398f,32.2315f,18.9334f,  38.1544f,33.4831f,19.4122f,  37.7739f,33.4952f,18.7642f,  36.4447f,34.9470f,19.1214f,  36.0376f,34.9415f,18.4374f,  
	34.8809f,36.5609f,18.6421f,  34.4438f,36.5305f,17.9052f,  33.4737f,38.2799f,17.9329f,  33.0051f,38.2164f,17.1335f,  32.2332f,40.0545f,16.9669f,  
	31.7323f,39.9474f,16.0975f,  31.1677f,41.8306f,15.7273f,  30.6346f,41.6709f,14.7850f,  30.2840f,43.5545f,14.2095f,  29.7200f,43.3317f,13.1946f,  
	29.5875f,45.1716f,12.4204f,  28.9954f,44.8768f,11.3374f,  29.0823f,46.6305f,10.3787f,  28.4666f,46.2560f,9.2371f,  28.7712f,47.8841f,8.1152f,  
	28.1354f,47.4243f,6.9236f,  28.6531f,48.8915f,5.6661f,  28.0023f,48.3434f,4.4378f,  28.7252f,49.6193f,3.0771f,  62.3788f,31.6473f,26.8542f,  
	62.2141f,32.3898f,25.2155f,  60.7193f,31.1222f,24.8146f,  60.5091f,31.7135f,23.3168f,  58.8647f,30.5925f,23.1131f,  58.6189f,31.0494f,21.7607f,  
	56.8556f,30.1046f,21.7495f,  56.5832f,30.4468f,20.5400f,  54.7328f,29.7054f,20.7084f,  54.4411f,29.9521f,19.6365f,  52.5356f,29.4382f,19.9639f,  
	52.2303f,29.6098f,19.0166f,  50.3015f,29.3428f,19.4754f,  49.9855f,29.4570f,18.6382f,  48.0641f,29.4509f,19.1950f,  47.7388f,29.5242f,18.4470f,  
	45.8540f,29.7863f,19.0640f,  45.5196f,29.8313f,18.3836f,  43.6989f,30.3619f,19.0205f,  43.3519f,30.3882f,18.3836f,  41.6207f,31.1801f,18.9985f,  
	41.2582f,31.1931f,18.3814f,  39.6398f,32.2315f,18.9334f,  39.2583f,32.2321f,18.3131f,  37.7739f,33.4952f,18.7642f,  37.3691f,33.4822f,18.1217f,  
	36.0376f,34.9415f,18.4374f,  35.6056f,34.9090f,17.7534f,  34.4438f,36.5305f,17.9052f,  33.9828f,36.4714f,17.1678f,  33.0051f,38.2164f,17.1335f,  
	32.5136f,38.1208f,16.3325f,  31.7323f,39.9474f,16.0975f,  31.2097f,39.8067f,15.2275f,  30.6346f,41.6709f,14.7850f,  30.0816f,41.4742f,13.8436f,  
	29.7200f,43.3317f,13.1946f,  29.1389f,43.0699f,12.1838f,  28.9954f,44.8768f,11.3374f,  28.3901f,44.5411f,10.2644f,  28.4666f,46.2560f,9.2371f,  
	27.8400f,45.8397f,8.1085f,  28.1354f,47.4243f,6.9236f,  27.4926f,46.9233f,5.7514f,  28.0023f,48.3434f,4.4378f,  62.2141f,32.3898f,25.2155f,  
	61.9945f,33.0201f,23.6137f,  60.5091f,31.7135f,23.3168f,  60.2510f,32.2018f,21.8625f,  58.6189f,31.0494f,21.7607f,  58.3319f,31.4146f,20.4544f,  
	56.5832f,30.4468f,20.5400f,  56.2754f,30.7079f,19.3792f,  54.4411f,29.9521f,19.6365f,  54.1189f,30.1298f,18.6108f,  52.2303f,29.6098f,19.0166f,  
	51.8978f,29.7226f,18.1139f,  49.9855f,29.4570f,18.6382f,  49.6449f,29.5227f,17.8396f,  47.7388f,29.5242f,18.4470f,  47.3911f,29.5571f,17.7320f,  
	45.5196f,29.8313f,18.3836f,  45.1620f,29.8428f,17.7290f,  43.3519f,30.3882f,18.3836f,  42.9818f,30.3856f,17.7655f,  41.2582f,31.1931f,18.3814f,  
	40.8724f,31.1794f,17.7760f,  39.2583f,32.2321f,18.3131f,  38.8525f,32.2078f,17.7003f,  37.3691f,33.4822f,18.1217f,  36.9392f,33.4431f,17.4802f,  
	35.6056f,34.9090f,17.7534f,  35.1494f,34.8493f,17.0692f,  33.9828f,36.4714f,17.1678f,  33.4983f,36.3820f,16.4280f,  32.5136f,38.1208f,16.3325f,  
	31.9995f,37.9936f,15.5295f,  31.2097f,39.8067f,15.2275f,  30.6661f,39.6309f,14.3562f,  30.0816f,41.4742f,13.8436f,  29.5101f,41.2405f,12.9035f,  
	29.1389f,43.0699f,12.1838f,  28.5429f,42.7688f,11.1795f,  28.3901f,44.5411f,10.2644f,  27.7721f,44.1649f,9.2005f,  27.8400f,45.8397f,8.1085f,  
	27.2048f,45.3832f,6.9954f,  27.4926f,46.9233f,5.7514f,  61.9945f,33.0201f,23.6137f,  61.7240f,33.5419f,22.0532f,  60.2510f,32.2018f,21.8625f,  
	59.9490f,32.5923f,20.4530f,  58.3319f,31.4146f,20.4544f,  58.0072f,31.6927f,19.1963f,  56.2754f,30.7079f,19.3792f,  55.9351f,30.8940f,18.2651f,  
	54.1189f,30.1298f,18.6108f,  53.7682f,30.2431f,17.6314f,  51.8978f,29.7226f,18.1139f,  51.5397f,29.7818f,17.2523f,  49.6449f,29.5227f,17.8396f,  
	49.2812f,29.5438f,17.0767f,  47.3911f,29.5571f,17.7320f,  47.0198f,29.5530f,17.0460f,  45.1620f,29.8428f,17.7290f,  44.7812f,29.8229f,17.0965f,  
	42.9818f,30.3856f,17.7655f,  42.5886f,30.3550f,17.1619f,  40.8724f,31.1794f,17.7760f,  40.4625f,31.1403f,17.1805f,  38.8525f,32.2078f,17.7003f,  
	38.4218f,32.1578f,17.0902f,  36.9392f,33.4431f,17.4802f,  36.4851f,33.3778f,16.8395f,  35.1494f,34.8493f,17.0692f,  34.6694f,34.7608f,16.3827f,  
	33.4983f,36.3820f,16.4280f,  32.9906f,36.2629f,15.6858f,  31.9995f,37.9936f,15.5295f,  31.4636f,37.8334f,14.7238f,  30.6661f,39.6309f,14.3562f,  
	30.1028f,39.4202f,13.4840f,  29.5101f,41.2405f,12.9035f,  28.9224f,40.9693f,11.9669f,  28.5429f,42.7688f,11.1795f,  27.9327f,42.4286f,10.1810f,  
	27.7721f,44.1649f,9.2005f,  27.1439f,43.7496f,8.1486f,  27.2048f,45.3832f,6.9954f,  61.7240f,33.5419f,22.0532f,  61.4068f,33.9598f,20.5358f,  
	59.9490f,32.5923f,20.4530f,  59.6067f,32.8895f,19.0909f,  58.0072f,31.6927f,19.1963f,  57.6478f,31.8894f,17.9853f,  55.9351f,30.8940f,18.2651f,  
	55.5646f,31.0097f,17.1987f,  53.7682f,30.2431f,17.6314f,  53.3907f,30.2972f,16.6952f,  51.5397f,29.7818f,17.2523f,  51.1576f,29.7918f,16.4290f,  
	49.2812f,29.5438f,17.0767f,  48.8938f,29.5239f,16.3463f,  47.0198f,29.5530f,17.0460f,  46.6255f,29.5145f,16.3855f,  44.7812f,29.8229f,17.0965f,  
	44.3776f,29.7730f,16.4814f,  42.5886f,30.3550f,17.1619f,  42.1716f,30.2977f,16.5711f,  40.4625f,31.1403f,17.1805f,  40.0281f,31.0752f,16.5901f,  
	38.4218f,32.1578f,17.0902f,  37.9670f,32.0821f,16.4825f,  36.4851f,33.3778f,16.8395f,  36.0071f,33.2848f,16.1971f,  34.6694f,34.7608f,16.3827f,  
	34.1659f,34.6440f,15.6938f,  32.9906f,36.2629f,15.6858f,  32.4603f,36.1124f,14.9399f,  31.4636f,37.8334f,14.7238f,  30.9069f,37.6399f,13.9156f,  
	30.1028f,39.4202f,13.4840f,  29.5222f,39.1735f,12.6130f,  28.9224f,40.9693f,11.9669f,  28.3191f,40.6603f,11.0331f,  27.9327f,42.4286f,10.1810f,  
	27.3108f,42.0504f,9.1911f,  27.1439f,43.7496f,8.1486f,  61.4068f,33.9598f,20.5358f,  61.0467f,34.2779f,19.0650f,  59.6067f,32.8895f,19.0909f,  
	59.2276f,33.0988f,17.7763f,  57.6478f,31.8894f,17.9853f,  57.2565f,32.0091f,16.8233f,  55.5646f,31.0097f,17.1987f,  55.1662f,31.0601f,16.1777f,  
	53.3907f,30.2972f,16.6952f,  52.9883f,30.2967f,15.8000f,  51.1576f,29.7918f,16.4290f,  50.7517f,29.7559f,15.6417f,  48.8938f,29.5239f,16.3463f,  
	48.4833f,29.4658f,15.6450f,  46.6255f,29.5145f,16.3855f,  46.2086f,29.4434f,15.7461f,  44.3776f,29.7730f,16.4814f,  43.9505f,29.6947f,15.8827f,  
	42.1716f,30.2977f,16.5711f,  41.7304f,30.2138f,15.9884f,  40.0281f,31.0752f,16.5901f,  39.5697f,30.9845f,16.0044f,  37.9670f,32.0821f,16.4825f,  
	37.4883f,31.9794f,15.8745f,  36.0071f,33.2848f,16.1971f,  35.5053f,33.1647f,15.5532f,  34.1659f,34.6440f,15.6938f,  33.6393f,34.4973f,15.0010f,  
	32.4603f,36.1124f,14.9399f,  31.9086f,35.9302f,14.1908f,  30.9069f,37.6399f,13.9156f,  30.3320f,37.4121f,13.1068f,  29.5222f,39.1735f,12.6130f,  
	28.9247f,38.8905f,11.7423f,  28.3191f,40.6603f,11.0331f,  27.7028f,40.3146f,10.1052f,  27.3108f,42.0504f,9.1911f,  61.0467f,34.2779f,19.0650f,  
	60.6475f,34.5015f,17.6416f,  59.2276f,33.0988f,17.7763f,  58.8147f,33.2244f,16.5116f,  57.2565f,32.0091f,16.8233f,  56.8359f,32.0572f,15.7088f,  
	55.1662f,31.0601f,16.1777f,  54.7420f,31.0500f,15.2004f,  52.9883f,30.2967f,15.8000f,  52.5618f,30.2451f,14.9444f,  50.7517f,29.7559f,15.6417f,  
	50.3227f,29.6776f,14.8874f,  48.4833f,29.4658f,15.6450f,  48.0505f,29.3718f,14.9686f,  46.2086f,29.4434f,15.7461f,  45.7688f,29.3414f,15.1269f,  
	43.9505f,29.6947f,15.8827f,  43.4998f,29.5883f,15.2955f,  41.7304f,30.2138f,15.9884f,  41.2658f,30.1036f,15.4133f,  39.5697f,30.9845f,16.0044f,  
	39.0877f,30.8669f,15.4204f,  37.4883f,31.9794f,15.8745f,  36.9859f,31.8503f,15.2664f,  35.5053f,33.1647f,15.5532f,  34.9803f,33.0158f,14.9060f,  
	33.6393f,34.4973f,15.0010f,  33.0907f,34.3204f,14.3049f,  31.9086f,35.9302f,14.1908f,  31.3379f,35.7152f,13.4398f,  30.3320f,37.4121f,13.1068f,  
	29.7391f,37.1493f,12.2965f,  28.9247f,38.8905f,11.7423f,  28.3132f,38.5720f,10.8752f,  27.7028f,40.3146f,10.1052f,  60.6475f,34.5015f,17.6416f,  
	60.2127f,34.6346f,16.2690f,  58.8147f,33.2244f,16.5116f,  58.3712f,33.2717f,15.2964f,  56.8359f,32.0572f,15.7088f,  56.3885f,32.0387f,14.6405f,  
	54.7420f,31.0500f,15.2004f,  54.2932f,30.9831f,14.2663f,  52.5618f,30.2451f,14.9444f,  52.1123f,30.1462f,14.1259f,  50.3227f,29.6776f,14.8874f,  
	49.8717f,29.5595f,14.1622f,  48.0505f,29.3718f,14.9686f,  47.5953f,29.2440f,14.3167f,  45.7688f,29.3414f,15.1269f,  45.3060f,29.2094f,14.5232f,  
	43.4998f,29.5883f,15.2955f,  43.0263f,29.4545f,14.7194f,  41.2658f,30.1036f,15.4133f,  40.7780f,29.9663f,14.8425f,  39.0877f,30.8669f,15.4204f,  
	38.5824f,30.7233f,14.8385f,  36.9859f,31.8503f,15.2664f,  36.4604f,31.6933f,14.6565f,  34.9803f,33.0158f,14.9060f,  34.4331f,32.8379f,14.2559f,  
	33.0907f,34.3204f,14.3049f,  32.5227f,34.1119f,13.6063f,  31.3379f,35.7152f,13.4398f,  30.7485f,35.4666f,12.6862f,  29.7391f,37.1493f,12.2965f,  
	29.1314f,36.8524f,11.4879f,  28.3132f,38.5720f,10.8752f,  60.2127f,34.6346f,16.2690f,  59.7458f,34.6826f,14.9476f,  58.3712f,33.2717f,15.2964f,  
	57.8998f,33.2459f,14.1298f,  56.3885f,32.0387f,14.6405f,  55.9161f,31.9573f,13.6190f,  54.2932f,30.9831f,14.2663f,  53.8214f,30.8637f,13.3733f,  
	52.1123f,30.1462f,14.1259f,  51.6410f,30.0032f,13.3407f,  49.8717f,29.5595f,14.1622f,  49.3989f,29.4041f,13.4660f,  47.5953f,29.2440f,14.3167f,  
	47.1179f,29.0837f,13.6847f,  45.3060f,29.2094f,14.5232f,  44.8210f,29.0485f,13.9344f,  43.0263f,29.4545f,14.7194f,  42.5303f,29.2931f,14.1508f,  
	40.7780f,29.9663f,14.8425f,  40.2675f,29.8029f,14.2767f,  38.5824f,30.7233f,14.8385f,  38.0542f,30.5522f,14.2567f,  36.4604f,31.6933f,14.6565f,  
	35.9127f,31.5081f,14.0447f,  34.4331f,32.8379f,14.2559f,  33.8662f,32.6296f,13.6036f,  32.5227f,34.1119f,13.6063f,  31.9355f,33.8712f,12.9046f,  
	30.7485f,35.4666f,12.6862f,  30.1436f,35.1851f,11.9329f,  29.1314f,36.8524f,11.4879f,  59.7458f,34.6826f,14.9476f,  59.2499f,34.6510f,13.6772f,  
	57.8998f,33.2459f,14.1298f,  57.4030f,33.1510f,13.0135f,  55.9161f,31.9573f,13.6190f,  55.4208f,31.8177f,12.6427f,  53.8214f,30.8637f,13.3733f,  
	53.3281f,30.6954f,12.5182f,  51.6410f,30.0032f,13.3407f,  51.1485f,29.8189f,12.5892f,  49.3989f,29.4041f,13.4660f,  48.9047f,29.2131f,12.7944f,  
	47.1179f,29.0837f,13.6847f,  46.6193f,28.8923f,13.0720f,  44.8210f,29.0485f,13.9344f,  44.3145f,28.8587f,13.3571f,  42.5303f,29.2931f,14.1508f,  
	42.0122f,29.1049f,13.5907f,  40.2675f,29.8029f,14.2767f,  39.7346f,29.6123f,13.7135f,  38.0542f,30.5522f,14.2567f,  37.5042f,30.3538f,13.6750f,  
	35.9127f,31.5081f,14.0447f,  35.3453f,31.2937f,13.4315f,  33.8662f,32.6296f,13.6036f,  33.2798f,32.3903f,12.9481f,  31.9355f,33.8712f,12.9046f,  
	31.3323f,33.5988f,12.2026f,  30.1436f,35.1851f,11.9329f,  59.2499f,34.6510f,13.6772f,  58.7281f,34.5438f,12.4603f,  57.4030f,33.1510f,13.0135f,  
	56.8831f,32.9920f,11.9464f,  55.4208f,31.8177f,12.6427f,  54.9042f,31.6242f,11.7087f,  53.3281f,30.6954f,12.5182f,  52.8144f,30.4814f,11.7015f,  
	51.1485f,29.8189f,12.5892f,  50.6358f,29.5957f,11.8675f,  48.9047f,29.2131f,12.7944f,  48.3904f,28.9887f,12.1467f,  46.6193f,28.8923f,13.0720f,  
	46.1002f,28.6706f,12.4750f,  44.3145f,28.8587f,13.3571f,  43.7869f,28.6413f,12.7921f,  42.0122f,29.1049f,13.5907f,  41.4727f,28.8895f,13.0363f,  
	39.7346f,29.6123f,13.7135f,  39.1804f,29.3947f,13.1529f,  37.5042f,30.3538f,13.6750f,  36.9348f,30.1269f,13.0931f,  35.3453f,31.2937f,13.4315f,  
	34.7585f,31.0493f,12.8159f,  33.2798f,32.3903f,12.9481f,  32.6772f,32.1206f,12.2923f,  31.3323f,33.5988f,12.2026f,  58.7281f,34.5438f,12.4603f,  
	58.1833f,34.3664f,11.2965f,  56.8831f,32.9920f,11.9464f,  56.3424f,32.7737f,10.9261f,  54.9042f,31.6242f,11.7087f,  54.3680f,31.3802f,10.8180f,  
	52.8144f,30.4814f,11.7015f,  52.2815f,30.2247f,10.9198f,  50.6358f,29.5957f,11.8675f,  50.1040f,29.3360f,11.1745f,  48.3904f,28.9887f,12.1467f,  
	47.8567f,28.7319f,11.5195f,  46.1002f,28.6706f,12.4750f,  45.5611f,28.4200f,11.8946f,  43.7869f,28.6413f,12.7921f,  43.2388f,28.3963f,12.2365f,  
	41.4727f,28.8895f,13.0363f,  40.9127f,28.6473f,12.4875f,  39.1804f,29.3947f,13.1529f,  38.6074f,29.1494f,12.5942f,  36.9348f,30.1269f,13.0931f,  
	36.3462f,29.8712f,12.5104f,  34.7585f,31.0493f,12.8159f,  34.1554f,30.7757f,12.2004f,  32.6772f,32.1206f,12.2923f,  58.1833f,34.3664f,11.2965f,  
	57.6179f,34.1243f,10.1837f,  56.3424f,32.7737f,10.9261f,  55.7828f,32.5000f,9.9539f,  54.3680f,31.3802f,10.8180f,  53.8137f,31.0895f,9.9676f,  
	52.2815f,30.2247f,10.9198f,  51.7307f,29.9283f,10.1720f,  50.1040f,29.3360f,11.1745f,  49.5541f,29.0418f,10.5071f,  47.8567f,28.7319f,11.5195f,  
	47.3044f,28.4448f,10.9136f,  45.5611f,28.4200f,11.8946f,  45.0027f,28.1410f,11.3279f,  43.2388f,28.3963f,12.2365f,  42.6714f,28.1243f,11.6902f,  
	40.9127f,28.6473f,12.4875f,  40.3346f,28.3778f,11.9433f,  38.6074f,29.1494f,12.5942f,  38.0157f,28.8760f,12.0366f,  36.3462f,29.8712f,12.5104f,  
	35.7415f,29.5872f,11.9287f,  34.1554f,30.7757f,12.2004f,  57.6179f,34.1243f,10.1837f,  57.0344f,33.8215f,9.1238f,  55.7828f,32.5000f,9.9539f,  
	55.2061f,32.1752f,9.0271f,  53.8137f,31.0895f,9.9676f,  53.2427f,30.7555f,9.1563f,  51.7307f,29.9283f,10.1720f,  51.1632f,29.5948f,9.4550f,  
	49.5541f,29.0418f,10.5071f,  48.9870f,28.7152f,9.8659f,  47.3044f,28.4448f,10.9136f,  46.7343f,28.1281f,10.3260f,  45.0027f,28.1410f,11.3279f,  
	44.4263f,27.8347f,10.7745f,  42.6714f,28.1243f,11.6902f,  42.0867f,27.8253f,11.1516f,  40.3346f,28.3778f,11.9433f,  39.7385f,28.0809f,11.4027f,  
	38.0157f,28.8760f,12.0366f,  37.4084f,28.5754f,11.4816f,  35.7415f,29.5872f,11.9287f,  57.0344f,33.8215f,9.1238f,  56.4350f,33.4632f,8.1145f,  
	55.2061f,32.1752f,9.0271f,  54.6142f,31.8033f,8.1448f,  53.2427f,30.7555f,9.1563f,  52.6566f,30.3814f,8.3812f,  51.1632f,29.5948f,9.4550f,  
	50.5802f,29.2266f,8.7694f,  48.9870f,28.7152f,9.8659f,  48.4038f,28.3578f,9.2480f,  46.7343f,28.1281f,10.3260f,  46.1475f,27.7834f,9.7562f,  
	44.4263f,27.8347f,10.7745f,  43.8338f,27.5014f,10.2325f,  42.0867f,27.8253f,11.1516f,  41.4852f,27.4994f,10.6196f,  39.7385f,28.0809f,11.4027f,  
	39.1276f,27.7576f,10.8670f,  37.4084f,28.5754f,11.4816f,  56.4350f,33.4632f,8.1145f,  55.8217f,33.0539f,7.1549f,  54.6142f,31.8033f,8.1448f,  
	54.0087f,31.3883f,7.3041f,  52.6566f,30.3814f,8.3812f,  52.0567f,29.9703f,7.6428f,  50.5802f,29.2266f,8.7694f,  49.9827f,28.8260f,8.1123f,  
	48.4038f,28.3578f,9.2480f,  47.8055f,27.9713f,8.6527f,  46.1475f,27.7834f,9.7562f,  45.5461f,27.4115f,9.2018f,  43.8338f,27.5014f,10.2325f,  
	43.2257f,27.1415f,9.7007f,  41.4852f,27.4994f,10.6196f,  40.8696f,27.1479f,10.0952f,  39.1276f,27.7576f,10.8670f,  55.8217f,33.0539f,7.1549f,  
	55.1964f,32.5982f,6.2422f,  54.0087f,31.3883f,7.3041f,  53.3911f,30.9335f,6.5055f,  52.0567f,29.9703f,7.6428f,  51.4441f,29.5249f,6.9384f,  
	49.9827f,28.8260f,8.1123f,  49.3720f,28.3953f,7.4829f,  47.8055f,27.9713f,8.6527f,  47.1941f,27.5574f,8.0771f,  45.5461f,27.4115f,9.2018f,  
	44.9305f,27.0133f,8.6616f,  43.2257f,27.1415f,9.7007f,  42.6047f,26.7566f,9.1795f,  40.8696f,27.1479f,10.0952f,  55.1964f,32.5982f,6.2422f,  
	54.5608f,32.0998f,5.3769f,  53.3911f,30.9335f,6.5055f,  52.7628f,30.4424f,5.7462f,  51.4441f,29.5249f,6.9384f,  50.8201f,29.0480f,6.2667f,  
	49.3720f,28.3953f,7.4829f,  48.7497f,27.9363f,6.8778f,  47.1941f,27.5574f,8.0771f,  46.5701f,27.1170f,7.5202f,  44.9305f,27.0133f,8.6616f,  
	44.3031f,26.5904f,8.1355f,  42.6047f,26.7566f,9.1795f,  54.5608f,32.0998f,5.3769f,  53.9163f,31.5629f,4.5564f,  52.7628f,30.4424f,5.7462f,  
	52.1251f,29.9182f,5.0250f,  50.8201f,29.0480f,6.2667f,  50.1864f,28.5420f,5.6243f,  48.7497f,27.9363f,6.8778f,  48.1168f,27.4508f,6.2960f,  
	46.5701f,27.1170f,7.5202f,  45.9358f,26.6522f,6.9812f,  44.3031f,26.5904f,8.1355f,  53.9163f,31.5629f,4.5564f,  53.2646f,30.9911f,3.7791f,  
	52.1251f,29.9182f,5.0250f,  51.4795f,29.3639f,4.3380f,  50.1864f,28.5420f,5.6243f,  49.5438f,28.0091f,5.0100f,  48.1168f,27.4508f,6.2960f,  
	47.4749f,26.9409f,5.7362f,  45.9358f,26.6522f,6.9812f,  53.2646f,30.9911f,3.7791f,  52.6068f,30.3881f,3.0410f,  51.4795f,29.3639f,4.3380f,  
	50.8269f,28.7822f,3.6840f,  49.5438f,28.0091f,5.0100f,  48.8941f,27.4516f,4.4219f,  47.4749f,26.9409f,5.7362f,  52.6068f,30.3881f,3.0410f,  
	51.9441f,29.7569f,2.3410f,  50.8269f,28.7822f,3.6840f,  50.1689f,28.1756f,3.0608f,  48.8941f,27.4516f,4.4219f,  51.2777f,29.1004f,1.6762f,  
	50.1689f,28.1756f,3.0608f,  51.9441f,29.7569f,2.3410f,  
_NULL_ }; /* End of torus_knot01_Vertices */

float torus_knot01_UV[] = {
	25.0001f,-4.0000f,24.9512f,-4.0000f,24.9512f,-3.8958f,24.9024f,-3.8958f,24.9024f,-3.7917f,24.8536f,-3.7917f,24.8536f,-3.6875f,24.8047f,-3.6875f,24.8047f,-3.5833f,24.7559f,-3.5833f,24.7559f,-3.4792f,24.7071f,-3.4792f,24.7071f,-3.3750f,24.6583f,-3.3750f,24.6583f,-3.2708f,
	24.6094f,-3.2708f,24.6094f,-3.1667f,24.5606f,-3.1667f,24.5606f,-3.0625f,24.5118f,-3.0625f,24.5118f,-2.9583f,24.4629f,-2.9583f,24.4629f,-2.8542f,24.4141f,-2.8542f,24.4141f,-2.7500f,24.3653f,-2.7500f,24.3653f,-2.6458f,24.3165f,-2.6458f,24.3165f,-2.5417f,24.2676f,-2.5417f,
	24.2676f,-2.4375f,24.2188f,-2.4375f,24.2188f,-2.3333f,24.1700f,-2.3333f,24.1700f,-2.2292f,24.1211f,-2.2292f,24.1211f,-2.1250f,24.0723f,-2.1250f,24.0723f,-2.0208f,24.0235f,-2.0208f,24.0235f,-1.9167f,23.9747f,-1.9167f,23.9747f,-1.8125f,23.9258f,-1.8125f,23.9258f,-1.7083f,
	23.8770f,-1.7083f,23.8770f,-1.6042f,23.8282f,-1.6042f,23.8282f,-1.5000f,23.7793f,-1.5000f,23.7793f,-1.3958f,23.7305f,-1.3958f,23.7305f,-1.2917f,23.6817f,-1.2917f,23.6817f,-1.1875f,23.6329f,-1.1875f,23.6329f,-1.0833f,23.5840f,-1.0833f,23.5840f,-0.9792f,23.5352f,-0.9792f,
	23.5352f,-0.8750f,23.4864f,-0.8750f,23.4864f,-0.7708f,23.4375f,-0.7708f,23.4375f,-0.6667f,23.3887f,-0.6667f,23.3887f,-0.5625f,23.3399f,-0.5625f,23.3399f,-0.4583f,23.2911f,-0.4583f,23.2911f,-0.3542f,23.2422f,-0.3542f,23.2422f,-0.2500f,23.1934f,-0.2500f,23.1934f,-0.1458f,
	23.1446f,-0.1458f,23.1446f,-0.0417f,23.0958f,-0.0417f,23.0958f,0.0625f,23.0469f,0.0625f,23.0469f,0.1667f,22.9981f,0.1667f,22.9981f,0.2708f,22.9493f,0.2708f,22.9493f,0.3750f,22.9004f,0.3750f,22.9004f,0.4792f,22.8516f,0.4792f,22.8516f,0.5833f,22.8028f,0.5833f,
	22.8028f,0.6875f,22.7540f,0.6875f,22.7540f,0.7917f,22.7051f,0.7917f,22.7051f,0.8958f,22.6563f,0.8958f,22.6563f,1.0000f,22.6075f,1.0000f,25.0001f,-4.0000f,24.9512f,-3.8958f,25.0001f,-3.8958f,24.9512f,-3.7917f,25.0001f,-3.7917f,24.9512f,-3.6875f,25.0001f,-3.6875f,
	24.9512f,-3.5833f,25.0001f,-3.5833f,24.9512f,-3.4792f,25.0001f,-3.4792f,24.9512f,-3.3750f,25.0001f,-3.3750f,24.9512f,-3.2708f,25.0001f,-3.2708f,24.9512f,-3.1667f,25.0001f,-3.1667f,24.9512f,-3.0625f,25.0001f,-3.0625f,24.9512f,-2.9583f,25.0001f,-2.9583f,24.9512f,-2.8542f,
	25.0001f,-2.8542f,24.9512f,-2.7500f,25.0001f,-2.7500f,24.9512f,-2.6458f,25.0001f,-2.6458f,24.9512f,-2.5417f,25.0001f,-2.5417f,24.9512f,-2.4375f,25.0001f,-2.4375f,24.9512f,-2.3333f,25.0001f,-2.3333f,24.9512f,-2.2292f,25.0001f,-2.2292f,24.9512f,-2.1250f,25.0001f,-2.1250f,
	24.9512f,-2.0208f,25.0001f,-2.0208f,24.9512f,-1.9167f,25.0001f,-1.9167f,24.9512f,-1.8125f,25.0001f,-1.8125f,24.9512f,-1.7083f,25.0001f,-1.7083f,24.9512f,-1.6042f,25.0001f,-1.6042f,24.9512f,-1.5000f,25.0001f,-1.5000f,24.9512f,-1.3958f,25.0001f,-1.3958f,24.9512f,-1.2917f,
	25.0001f,-1.2917f,24.9512f,-1.1875f,25.0001f,-1.1875f,24.9512f,-1.0833f,25.0001f,-1.0833f,24.9512f,-0.9792f,25.0001f,-0.9792f,24.9512f,-0.8750f,25.0001f,-0.8750f,24.9512f,-0.7708f,25.0001f,-0.7708f,24.9512f,-0.6667f,25.0001f,-0.6667f,24.9512f,-0.5625f,25.0001f,-0.5625f,
	24.9512f,-0.4583f,25.0001f,-0.4583f,24.9512f,-0.3542f,25.0001f,-0.3542f,24.9512f,-0.2500f,25.0001f,-0.2500f,24.9512f,-0.1458f,25.0001f,-0.1458f,24.9512f,-0.0417f,25.0001f,-0.0417f,24.9512f,0.0625f,25.0001f,0.0625f,24.9512f,0.1667f,25.0001f,0.1667f,24.9512f,0.2708f,
	25.0001f,0.2708f,24.9512f,0.3750f,25.0001f,0.3750f,24.9512f,0.4792f,25.0001f,0.4792f,24.9512f,0.5833f,25.0001f,0.5833f,24.9512f,0.6875f,25.0001f,0.6875f,24.9512f,0.7917f,25.0001f,0.7917f,24.9512f,0.8958f,25.0001f,0.8958f,24.9512f,1.0000f,25.0001f,1.0000f,
	24.9512f,-4.0000f,24.9024f,-4.0000f,24.9024f,-3.8958f,24.8536f,-3.8958f,24.8536f,-3.7917f,24.8047f,-3.7917f,24.8047f,-3.6875f,24.7559f,-3.6875f,24.7559f,-3.5833f,24.7071f,-3.5833f,24.7071f,-3.4792f,24.6583f,-3.4792f,24.6583f,-3.3750f,24.6094f,-3.3750f,24.6094f,-3.2708f,
	24.5606f,-3.2708f,24.5606f,-3.1667f,24.5118f,-3.1667f,24.5118f,-3.0625f,24.4629f,-3.0625f,24.4629f,-2.9583f,24.4141f,-2.9583f,24.4141f,-2.8542f,24.3653f,-2.8542f,24.3653f,-2.7500f,24.3165f,-2.7500f,24.3165f,-2.6458f,24.2676f,-2.6458f,24.2676f,-2.5417f,24.2188f,-2.5417f,
	24.2188f,-2.4375f,24.1700f,-2.4375f,24.1700f,-2.3333f,24.1211f,-2.3333f,24.1211f,-2.2292f,24.0723f,-2.2292f,24.0723f,-2.1250f,24.0235f,-2.1250f,24.0235f,-2.0208f,23.9747f,-2.0208f,23.9747f,-1.9167f,23.9258f,-1.9167f,23.9258f,-1.8125f,23.8770f,-1.8125f,23.8770f,-1.7083f,
	23.8282f,-1.7083f,23.8282f,-1.6042f,23.7793f,-1.6042f,23.7793f,-1.5000f,23.7305f,-1.5000f,23.7305f,-1.3958f,23.6817f,-1.3958f,23.6817f,-1.2917f,23.6329f,-1.2917f,23.6329f,-1.1875f,23.5840f,-1.1875f,23.5840f,-1.0833f,23.5352f,-1.0833f,23.5352f,-0.9792f,23.4864f,-0.9792f,
	23.4864f,-0.8750f,23.4375f,-0.8750f,23.4375f,-0.7708f,23.3887f,-0.7708f,23.3887f,-0.6667f,23.3399f,-0.6667f,23.3399f,-0.5625f,23.2911f,-0.5625f,23.2911f,-0.4583f,23.2422f,-0.4583f,23.2422f,-0.3542f,23.1934f,-0.3542f,23.1934f,-0.2500f,23.1446f,-0.2500f,23.1446f,-0.1458f,
	23.0958f,-0.1458f,23.0958f,-0.0417f,23.0469f,-0.0417f,23.0469f,0.0625f,22.9981f,0.0625f,22.9981f,0.1667f,22.9493f,0.1667f,22.9493f,0.2708f,22.9004f,0.2708f,22.9004f,0.3750f,22.8516f,0.3750f,22.8516f,0.4792f,22.8028f,0.4792f,22.8028f,0.5833f,22.7540f,0.5833f,
	22.7540f,0.6875f,22.7051f,0.6875f,22.7051f,0.7917f,22.6563f,0.7917f,22.6563f,0.8958f,22.6075f,0.8958f,22.6075f,1.0000f,22.5586f,1.0000f,24.9512f,-3.8958f,24.9024f,-3.7917f,24.9512f,-3.7917f,24.9024f,-3.6875f,24.9512f,-3.6875f,24.9024f,-3.5833f,24.9512f,-3.5833f,
	24.9024f,-3.4792f,24.9512f,-3.4792f,24.9024f,-3.3750f,24.9512f,-3.3750f,24.9024f,-3.2708f,24.9512f,-3.2708f,24.9024f,-3.1667f,24.9512f,-3.1667f,24.9024f,-3.0625f,24.9512f,-3.0625f,24.9024f,-2.9583f,24.9512f,-2.9583f,24.9024f,-2.8542f,24.9512f,-2.8542f,24.9024f,-2.7500f,
	24.9512f,-2.7500f,24.9024f,-2.6458f,24.9512f,-2.6458f,24.9024f,-2.5417f,24.9512f,-2.5417f,24.9024f,-2.4375f,24.9512f,-2.4375f,24.9024f,-2.3333f,24.9512f,-2.3333f,24.9024f,-2.2292f,24.9512f,-2.2292f,24.9024f,-2.1250f,24.9512f,-2.1250f,24.9024f,-2.0208f,24.9512f,-2.0208f,
	24.9024f,-1.9167f,24.9512f,-1.9167f,24.9024f,-1.8125f,24.9512f,-1.8125f,24.9024f,-1.7083f,24.9512f,-1.7083f,24.9024f,-1.6042f,24.9512f,-1.6042f,24.9024f,-1.5000f,24.9512f,-1.5000f,24.9024f,-1.3958f,24.9512f,-1.3958f,24.9024f,-1.2917f,24.9512f,-1.2917f,24.9024f,-1.1875f,
	24.9512f,-1.1875f,24.9024f,-1.0833f,24.9512f,-1.0833f,24.9024f,-0.9792f,24.9512f,-0.9792f,24.9024f,-0.8750f,24.9512f,-0.8750f,24.9024f,-0.7708f,24.9512f,-0.7708f,24.9024f,-0.6667f,24.9512f,-0.6667f,24.9024f,-0.5625f,24.9512f,-0.5625f,24.9024f,-0.4583f,24.9512f,-0.4583f,
	24.9024f,-0.3542f,24.9512f,-0.3542f,24.9024f,-0.2500f,24.9512f,-0.2500f,24.9024f,-0.1458f,24.9512f,-0.1458f,24.9024f,-0.0417f,24.9512f,-0.0417f,24.9024f,0.0625f,24.9512f,0.0625f,24.9024f,0.1667f,24.9512f,0.1667f,24.9024f,0.2708f,24.9512f,0.2708f,24.9024f,0.3750f,
	24.9512f,0.3750f,24.9024f,0.4792f,24.9512f,0.4792f,24.9024f,0.5833f,24.9512f,0.5833f,24.9024f,0.6875f,24.9512f,0.6875f,24.9024f,0.7917f,24.9512f,0.7917f,24.9024f,0.8958f,24.9512f,0.8958f,24.9024f,1.0000f,24.9512f,1.0000f,24.9024f,-4.0000f,24.8536f,-4.0000f,
	24.8536f,-3.8958f,24.8047f,-3.8958f,24.8047f,-3.7917f,24.7559f,-3.7917f,24.7559f,-3.6875f,24.7071f,-3.6875f,24.7071f,-3.5833f,24.6583f,-3.5833f,24.6583f,-3.4792f,24.6094f,-3.4792f,24.6094f,-3.3750f,24.5606f,-3.3750f,24.5606f,-3.2708f,24.5118f,-3.2708f,24.5118f,-3.1667f,
	24.4629f,-3.1667f,24.4629f,-3.0625f,24.4141f,-3.0625f,24.4141f,-2.9583f,24.3653f,-2.9583f,24.3653f,-2.8542f,24.3165f,-2.8542f,24.3165f,-2.7500f,24.2676f,-2.7500f,24.2676f,-2.6458f,24.2188f,-2.6458f,24.2188f,-2.5417f,24.1700f,-2.5417f,24.1700f,-2.4375f,24.1211f,-2.4375f,
	24.1211f,-2.3333f,24.0723f,-2.3333f,24.0723f,-2.2292f,24.0235f,-2.2292f,24.0235f,-2.1250f,23.9747f,-2.1250f,23.9747f,-2.0208f,23.9258f,-2.0208f,23.9258f,-1.9167f,23.8770f,-1.9167f,23.8770f,-1.8125f,23.8282f,-1.8125f,23.8282f,-1.7083f,23.7793f,-1.7083f,23.7793f,-1.6042f,
	23.7305f,-1.6042f,23.7305f,-1.5000f,23.6817f,-1.5000f,23.6817f,-1.3958f,23.6329f,-1.3958f,23.6329f,-1.2917f,23.5840f,-1.2917f,23.5840f,-1.1875f,23.5352f,-1.1875f,23.5352f,-1.0833f,23.4864f,-1.0833f,23.4864f,-0.9792f,23.4375f,-0.9792f,23.4375f,-0.8750f,23.3887f,-0.8750f,
	23.3887f,-0.7708f,23.3399f,-0.7708f,23.3399f,-0.6667f,23.2911f,-0.6667f,23.2911f,-0.5625f,23.2422f,-0.5625f,23.2422f,-0.4583f,23.1934f,-0.4583f,23.1934f,-0.3542f,23.1446f,-0.3542f,23.1446f,-0.2500f,23.0958f,-0.2500f,23.0958f,-0.1458f,23.0469f,-0.1458f,23.0469f,-0.0417f,
	22.9981f,-0.0417f,22.9981f,0.0625f,22.9493f,0.0625f,22.9493f,0.1667f,22.9004f,0.1667f,22.9004f,0.2708f,22.8516f,0.2708f,22.8516f,0.3750f,22.8028f,0.3750f,22.8028f,0.4792f,22.7540f,0.4792f,22.7540f,0.5833f,22.7051f,0.5833f,22.7051f,0.6875f,22.6563f,0.6875f,
	22.6563f,0.7917f,22.6075f,0.7917f,22.6075f,0.8958f,22.5586f,0.8958f,22.5586f,1.0000f,22.5098f,1.0000f,24.9024f,-3.7917f,24.8536f,-3.6875f,24.9024f,-3.6875f,24.8536f,-3.5833f,24.9024f,-3.5833f,24.8536f,-3.4792f,24.9024f,-3.4792f,24.8536f,-3.3750f,24.9024f,-3.3750f,
	24.8536f,-3.2708f,24.9024f,-3.2708f,24.8536f,-3.1667f,24.9024f,-3.1667f,24.8536f,-3.0625f,24.9024f,-3.0625f,24.8536f,-2.9583f,24.9024f,-2.9583f,24.8536f,-2.8542f,24.9024f,-2.8542f,24.8536f,-2.7500f,24.9024f,-2.7500f,24.8536f,-2.6458f,24.9024f,-2.6458f,24.8536f,-2.5417f,
	24.9024f,-2.5417f,24.8536f,-2.4375f,24.9024f,-2.4375f,24.8536f,-2.3333f,24.9024f,-2.3333f,24.8536f,-2.2292f,24.9024f,-2.2292f,24.8536f,-2.1250f,24.9024f,-2.1250f,24.8536f,-2.0208f,24.9024f,-2.0208f,24.8536f,-1.9167f,24.9024f,-1.9167f,24.8536f,-1.8125f,24.9024f,-1.8125f,
	24.8536f,-1.7083f,24.9024f,-1.7083f,24.8536f,-1.6042f,24.9024f,-1.6042f,24.8536f,-1.5000f,24.9024f,-1.5000f,24.8536f,-1.3958f,24.9024f,-1.3958f,24.8536f,-1.2917f,24.9024f,-1.2917f,24.8536f,-1.1875f,24.9024f,-1.1875f,24.8536f,-1.0833f,24.9024f,-1.0833f,24.8536f,-0.9792f,
	24.9024f,-0.9792f,24.8536f,-0.8750f,24.9024f,-0.8750f,24.8536f,-0.7708f,24.9024f,-0.7708f,24.8536f,-0.6667f,24.9024f,-0.6667f,24.8536f,-0.5625f,24.9024f,-0.5625f,24.8536f,-0.4583f,24.9024f,-0.4583f,24.8536f,-0.3542f,24.9024f,-0.3542f,24.8536f,-0.2500f,24.9024f,-0.2500f,
	24.8536f,-0.1458f,24.9024f,-0.1458f,24.8536f,-0.0417f,24.9024f,-0.0417f,24.8536f,0.0625f,24.9024f,0.0625f,24.8536f,0.1667f,24.9024f,0.1667f,24.8536f,0.2708f,24.9024f,0.2708f,24.8536f,0.3750f,24.9024f,0.3750f,24.8536f,0.4792f,24.9024f,0.4792f,24.8536f,0.5833f,
	24.9024f,0.5833f,24.8536f,0.6875f,24.9024f,0.6875f,24.8536f,0.7917f,24.9024f,0.7917f,24.8536f,0.8958f,24.9024f,0.8958f,24.8536f,1.0000f,24.9024f,1.0000f,24.8536f,-4.0000f,24.8047f,-4.0000f,24.8047f,-3.8958f,24.7559f,-3.8958f,24.7559f,-3.7917f,24.7071f,-3.7917f,
	24.7071f,-3.6875f,24.6583f,-3.6875f,24.6583f,-3.5833f,24.6094f,-3.5833f,24.6094f,-3.4792f,24.5606f,-3.4792f,24.5606f,-3.3750f,24.5118f,-3.3750f,24.5118f,-3.2708f,24.4629f,-3.2708f,24.4629f,-3.1667f,24.4141f,-3.1667f,24.4141f,-3.0625f,24.3653f,-3.0625f,24.3653f,-2.9583f,
	24.3165f,-2.9583f,24.3165f,-2.8542f,24.2676f,-2.8542f,24.2676f,-2.7500f,24.2188f,-2.7500f,24.2188f,-2.6458f,24.1700f,-2.6458f,24.1700f,-2.5417f,24.1211f,-2.5417f,24.1211f,-2.4375f,24.0723f,-2.4375f,24.0723f,-2.3333f,24.0235f,-2.3333f,24.0235f,-2.2292f,23.9747f,-2.2292f,
	23.9747f,-2.1250f,23.9258f,-2.1250f,23.9258f,-2.0208f,23.8770f,-2.0208f,23.8770f,-1.9167f,23.8282f,-1.9167f,23.8282f,-1.8125f,23.7793f,-1.8125f,23.7793f,-1.7083f,23.7305f,-1.7083f,23.7305f,-1.6042f,23.6817f,-1.6042f,23.6817f,-1.5000f,23.6329f,-1.5000f,23.6329f,-1.3958f,
	23.5840f,-1.3958f,23.5840f,-1.2917f,23.5352f,-1.2917f,23.5352f,-1.1875f,23.4864f,-1.1875f,23.4864f,-1.0833f,23.4375f,-1.0833f,23.4375f,-0.9792f,23.3887f,-0.9792f,23.3887f,-0.8750f,23.3399f,-0.8750f,23.3399f,-0.7708f,23.2911f,-0.7708f,23.2911f,-0.6667f,23.2422f,-0.6667f,
	23.2422f,-0.5625f,23.1934f,-0.5625f,23.1934f,-0.4583f,23.1446f,-0.4583f,23.1446f,-0.3542f,23.0958f,-0.3542f,23.0958f,-0.2500f,23.0469f,-0.2500f,23.0469f,-0.1458f,22.9981f,-0.1458f,22.9981f,-0.0417f,22.9493f,-0.0417f,22.9493f,0.0625f,22.9004f,0.0625f,22.9004f,0.1667f,
	22.8516f,0.1667f,22.8516f,0.2708f,22.8028f,0.2708f,22.8028f,0.3750f,22.7540f,0.3750f,22.7540f,0.4792f,22.7051f,0.4792f,22.7051f,0.5833f,22.6563f,0.5833f,22.6563f,0.6875f,22.6075f,0.6875f,22.6075f,0.7917f,22.5586f,0.7917f,22.5586f,0.8958f,22.5098f,0.8958f,
	22.5098f,1.0000f,22.4610f,1.0000f,24.8536f,-3.6875f,24.8047f,-3.5833f,24.8536f,-3.5833f,24.8047f,-3.4792f,24.8536f,-3.4792f,24.8047f,-3.3750f,24.8536f,-3.3750f,24.8047f,-3.2708f,24.8536f,-3.2708f,24.8047f,-3.1667f,24.8536f,-3.1667f,24.8047f,-3.0625f,24.8536f,-3.0625f,
	24.8047f,-2.9583f,24.8536f,-2.9583f,24.8047f,-2.8542f,24.8536f,-2.8542f,24.8047f,-2.7500f,24.8536f,-2.7500f,24.8047f,-2.6458f,24.8536f,-2.6458f,24.8047f,-2.5417f,24.8536f,-2.5417f,24.8047f,-2.4375f,24.8536f,-2.4375f,24.8047f,-2.3333f,24.8536f,-2.3333f,24.8047f,-2.2292f,
	24.8536f,-2.2292f,24.8047f,-2.1250f,24.8536f,-2.1250f,24.8047f,-2.0208f,24.8536f,-2.0208f,24.8047f,-1.9167f,24.8536f,-1.9167f,24.8047f,-1.8125f,24.8536f,-1.8125f,24.8047f,-1.7083f,24.8536f,-1.7083f,24.8047f,-1.6042f,24.8536f,-1.6042f,24.8047f,-1.5000f,24.8536f,-1.5000f,
	24.8047f,-1.3958f,24.8536f,-1.3958f,24.8047f,-1.2917f,24.8536f,-1.2917f,24.8047f,-1.1875f,24.8536f,-1.1875f,24.8047f,-1.0833f,24.8536f,-1.0833f,24.8047f,-0.9792f,24.8536f,-0.9792f,24.8047f,-0.8750f,24.8536f,-0.8750f,24.8047f,-0.7708f,24.8536f,-0.7708f,24.8047f,-0.6667f,
	24.8536f,-0.6667f,24.8047f,-0.5625f,24.8536f,-0.5625f,24.8047f,-0.4583f,24.8536f,-0.4583f,24.8047f,-0.3542f,24.8536f,-0.3542f,24.8047f,-0.2500f,24.8536f,-0.2500f,24.8047f,-0.1458f,24.8536f,-0.1458f,24.8047f,-0.0417f,24.8536f,-0.0417f,24.8047f,0.0625f,24.8536f,0.0625f,
	24.8047f,0.1667f,24.8536f,0.1667f,24.8047f,0.2708f,24.8536f,0.2708f,24.8047f,0.3750f,24.8536f,0.3750f,24.8047f,0.4792f,24.8536f,0.4792f,24.8047f,0.5833f,24.8536f,0.5833f,24.8047f,0.6875f,24.8536f,0.6875f,24.8047f,0.7917f,24.8536f,0.7917f,24.8047f,0.8958f,
	24.8536f,0.8958f,24.8047f,1.0000f,24.8536f,1.0000f,24.8047f,-4.0000f,24.7559f,-4.0000f,24.7559f,-3.8958f,24.7071f,-3.8958f,24.7071f,-3.7917f,24.6583f,-3.7917f,24.6583f,-3.6875f,24.6094f,-3.6875f,24.6094f,-3.5833f,24.5606f,-3.5833f,24.5606f,-3.4792f,24.5118f,-3.4792f,
	24.5118f,-3.3750f,24.4629f,-3.3750f,24.4629f,-3.2708f,24.4141f,-3.2708f,24.4141f,-3.1667f,24.3653f,-3.1667f,24.3653f,-3.0625f,24.3165f,-3.0625f,24.3165f,-2.9583f,24.2676f,-2.9583f,24.2676f,-2.8542f,24.2188f,-2.8542f,24.2188f,-2.7500f,24.1700f,-2.7500f,24.1700f,-2.6458f,
	24.1211f,-2.6458f,24.1211f,-2.5417f,24.0723f,-2.5417f,24.0723f,-2.4375f,24.0235f,-2.4375f,24.0235f,-2.3333f,23.9747f,-2.3333f,23.9747f,-2.2292f,23.9258f,-2.2292f,23.9258f,-2.1250f,23.8770f,-2.1250f,23.8770f,-2.0208f,23.8282f,-2.0208f,23.8282f,-1.9167f,23.7793f,-1.9167f,
	23.7793f,-1.8125f,23.7305f,-1.8125f,23.7305f,-1.7083f,23.6817f,-1.7083f,23.6817f,-1.6042f,23.6329f,-1.6042f,23.6329f,-1.5000f,23.5840f,-1.5000f,23.5840f,-1.3958f,23.5352f,-1.3958f,23.5352f,-1.2917f,23.4864f,-1.2917f,23.4864f,-1.1875f,23.4375f,-1.1875f,23.4375f,-1.0833f,
	23.3887f,-1.0833f,23.3887f,-0.9792f,23.3399f,-0.9792f,23.3399f,-0.8750f,23.2911f,-0.8750f,23.2911f,-0.7708f,23.2422f,-0.7708f,23.2422f,-0.6667f,23.1934f,-0.6667f,23.1934f,-0.5625f,23.1446f,-0.5625f,23.1446f,-0.4583f,23.0958f,-0.4583f,23.0958f,-0.3542f,23.0469f,-0.3542f,
	23.0469f,-0.2500f,22.9981f,-0.2500f,22.9981f,-0.1458f,22.9493f,-0.1458f,22.9493f,-0.0417f,22.9004f,-0.0417f,22.9004f,0.0625f,22.8516f,0.0625f,22.8516f,0.1667f,22.8028f,0.1667f,22.8028f,0.2708f,22.7540f,0.2708f,22.7540f,0.3750f,22.7051f,0.3750f,22.7051f,0.4792f,
	22.6563f,0.4792f,22.6563f,0.5833f,22.6075f,0.5833f,22.6075f,0.6875f,22.5586f,0.6875f,22.5586f,0.7917f,22.5098f,0.7917f,22.5098f,0.8958f,22.4610f,0.8958f,22.4610f,1.0000f,22.4122f,1.0000f,24.8047f,-3.5833f,24.7559f,-3.4792f,24.8047f,-3.4792f,24.7559f,-3.3750f,
	24.8047f,-3.3750f,24.7559f,-3.2708f,24.8047f,-3.2708f,24.7559f,-3.1667f,24.8047f,-3.1667f,24.7559f,-3.0625f,24.8047f,-3.0625f,24.7559f,-2.9583f,24.8047f,-2.9583f,24.7559f,-2.8542f,24.8047f,-2.8542f,24.7559f,-2.7500f,24.8047f,-2.7500f,24.7559f,-2.6458f,24.8047f,-2.6458f,
	24.7559f,-2.5417f,24.8047f,-2.5417f,24.7559f,-2.4375f,24.8047f,-2.4375f,24.7559f,-2.3333f,24.8047f,-2.3333f,24.7559f,-2.2292f,24.8047f,-2.2292f,24.7559f,-2.1250f,24.8047f,-2.1250f,24.7559f,-2.0208f,24.8047f,-2.0208f,24.7559f,-1.9167f,24.8047f,-1.9167f,24.7559f,-1.8125f,
	24.8047f,-1.8125f,24.7559f,-1.7083f,24.8047f,-1.7083f,24.7559f,-1.6042f,24.8047f,-1.6042f,24.7559f,-1.5000f,24.8047f,-1.5000f,24.7559f,-1.3958f,24.8047f,-1.3958f,24.7559f,-1.2917f,24.8047f,-1.2917f,24.7559f,-1.1875f,24.8047f,-1.1875f,24.7559f,-1.0833f,24.8047f,-1.0833f,
	24.7559f,-0.9792f,24.8047f,-0.9792f,24.7559f,-0.8750f,24.8047f,-0.8750f,24.7559f,-0.7708f,24.8047f,-0.7708f,24.7559f,-0.6667f,24.8047f,-0.6667f,24.7559f,-0.5625f,24.8047f,-0.5625f,24.7559f,-0.4583f,24.8047f,-0.4583f,24.7559f,-0.3542f,24.8047f,-0.3542f,24.7559f,-0.2500f,
	24.8047f,-0.2500f,24.7559f,-0.1458f,24.8047f,-0.1458f,24.7559f,-0.0417f,24.8047f,-0.0417f,24.7559f,0.0625f,24.8047f,0.0625f,24.7559f,0.1667f,24.8047f,0.1667f,24.7559f,0.2708f,24.8047f,0.2708f,24.7559f,0.3750f,24.8047f,0.3750f,24.7559f,0.4792f,24.8047f,0.4792f,
	24.7559f,0.5833f,24.8047f,0.5833f,24.7559f,0.6875f,24.8047f,0.6875f,24.7559f,0.7917f,24.8047f,0.7917f,24.7559f,0.8958f,24.8047f,0.8958f,24.7559f,1.0000f,24.8047f,1.0000f,24.7559f,-4.0000f,24.7071f,-4.0000f,24.7071f,-3.8958f,24.6583f,-3.8958f,24.6583f,-3.7917f,
	24.6094f,-3.7917f,24.6094f,-3.6875f,24.5606f,-3.6875f,24.5606f,-3.5833f,24.5118f,-3.5833f,24.5118f,-3.4792f,24.4629f,-3.4792f,24.4629f,-3.3750f,24.4141f,-3.3750f,24.4141f,-3.2708f,24.3653f,-3.2708f,24.3653f,-3.1667f,24.3165f,-3.1667f,24.3165f,-3.0625f,24.2676f,-3.0625f,
	24.2676f,-2.9583f,24.2188f,-2.9583f,24.2188f,-2.8542f,24.1700f,-2.8542f,24.1700f,-2.7500f,24.1211f,-2.7500f,24.1211f,-2.6458f,24.0723f,-2.6458f,24.0723f,-2.5417f,24.0235f,-2.5417f,24.0235f,-2.4375f,23.9747f,-2.4375f,23.9747f,-2.3333f,23.9258f,-2.3333f,23.9258f,-2.2292f,
	23.8770f,-2.2292f,23.8770f,-2.1250f,23.8282f,-2.1250f,23.8282f,-2.0208f,23.7793f,-2.0208f,23.7793f,-1.9167f,23.7305f,-1.9167f,23.7305f,-1.8125f,23.6817f,-1.8125f,23.6817f,-1.7083f,23.6329f,-1.7083f,23.6329f,-1.6042f,23.5840f,-1.6042f,23.5840f,-1.5000f,23.5352f,-1.5000f,
	23.5352f,-1.3958f,23.4864f,-1.3958f,23.4864f,-1.2917f,23.4375f,-1.2917f,23.4375f,-1.1875f,23.3887f,-1.1875f,23.3887f,-1.0833f,23.3399f,-1.0833f,23.3399f,-0.9792f,23.2911f,-0.9792f,23.2911f,-0.8750f,23.2422f,-0.8750f,23.2422f,-0.7708f,23.1934f,-0.7708f,23.1934f,-0.6667f,
	23.1446f,-0.6667f,23.1446f,-0.5625f,23.0958f,-0.5625f,23.0958f,-0.4583f,23.0469f,-0.4583f,23.0469f,-0.3542f,22.9981f,-0.3542f,22.9981f,-0.2500f,22.9493f,-0.2500f,22.9493f,-0.1458f,22.9004f,-0.1458f,22.9004f,-0.0417f,22.8516f,-0.0417f,22.8516f,0.0625f,22.8028f,0.0625f,
	22.8028f,0.1667f,22.7540f,0.1667f,22.7540f,0.2708f,22.7051f,0.2708f,22.7051f,0.3750f,22.6563f,0.3750f,22.6563f,0.4792f,22.6075f,0.4792f,22.6075f,0.5833f,22.5586f,0.5833f,22.5586f,0.6875f,22.5098f,0.6875f,22.5098f,0.7917f,22.4610f,0.7917f,22.4610f,0.8958f,
	22.4122f,0.8958f,22.4122f,1.0000f,22.3633f,1.0000f,24.7559f,-3.4792f,24.7071f,-3.3750f,24.7559f,-3.3750f,24.7071f,-3.2708f,24.7559f,-3.2708f,24.7071f,-3.1667f,24.7559f,-3.1667f,24.7071f,-3.0625f,24.7559f,-3.0625f,24.7071f,-2.9583f,24.7559f,-2.9583f,24.7071f,-2.8542f,
	24.7559f,-2.8542f,24.7071f,-2.7500f,24.7559f,-2.7500f,24.7071f,-2.6458f,24.7559f,-2.6458f,24.7071f,-2.5417f,24.7559f,-2.5417f,24.7071f,-2.4375f,24.7559f,-2.4375f,24.7071f,-2.3333f,24.7559f,-2.3333f,24.7071f,-2.2292f,24.7559f,-2.2292f,24.7071f,-2.1250f,24.7559f,-2.1250f,
	24.7071f,-2.0208f,24.7559f,-2.0208f,24.7071f,-1.9167f,24.7559f,-1.9167f,24.7071f,-1.8125f,24.7559f,-1.8125f,24.7071f,-1.7083f,24.7559f,-1.7083f,24.7071f,-1.6042f,24.7559f,-1.6042f,24.7071f,-1.5000f,24.7559f,-1.5000f,24.7071f,-1.3958f,24.7559f,-1.3958f,24.7071f,-1.2917f,
	24.7559f,-1.2917f,24.7071f,-1.1875f,24.7559f,-1.1875f,24.7071f,-1.0833f,24.7559f,-1.0833f,24.7071f,-0.9792f,24.7559f,-0.9792f,24.7071f,-0.8750f,24.7559f,-0.8750f,24.7071f,-0.7708f,24.7559f,-0.7708f,24.7071f,-0.6667f,24.7559f,-0.6667f,24.7071f,-0.5625f,24.7559f,-0.5625f,
	24.7071f,-0.4583f,24.7559f,-0.4583f,24.7071f,-0.3542f,24.7559f,-0.3542f,24.7071f,-0.2500f,24.7559f,-0.2500f,24.7071f,-0.1458f,24.7559f,-0.1458f,24.7071f,-0.0417f,24.7559f,-0.0417f,24.7071f,0.0625f,24.7559f,0.0625f,24.7071f,0.1667f,24.7559f,0.1667f,24.7071f,0.2708f,
	24.7559f,0.2708f,24.7071f,0.3750f,24.7559f,0.3750f,24.7071f,0.4792f,24.7559f,0.4792f,24.7071f,0.5833f,24.7559f,0.5833f,24.7071f,0.6875f,24.7559f,0.6875f,24.7071f,0.7917f,24.7559f,0.7917f,24.7071f,0.8958f,24.7559f,0.8958f,24.7071f,1.0000f,24.7559f,1.0000f,
	24.7071f,-4.0000f,24.6583f,-4.0000f,24.6583f,-3.8958f,24.6094f,-3.8958f,24.6094f,-3.7917f,24.5606f,-3.7917f,24.5606f,-3.6875f,24.5118f,-3.6875f,24.5118f,-3.5833f,24.4629f,-3.5833f,24.4629f,-3.4792f,24.4141f,-3.4792f,24.4141f,-3.3750f,24.3653f,-3.3750f,24.3653f,-3.2708f,
	24.3165f,-3.2708f,24.3165f,-3.1667f,24.2676f,-3.1667f,24.2676f,-3.0625f,24.2188f,-3.0625f,24.2188f,-2.9583f,24.1700f,-2.9583f,24.1700f,-2.8542f,24.1211f,-2.8542f,24.1211f,-2.7500f,24.0723f,-2.7500f,24.0723f,-2.6458f,24.0235f,-2.6458f,24.0235f,-2.5417f,23.9747f,-2.5417f,
	23.9747f,-2.4375f,23.9258f,-2.4375f,23.9258f,-2.3333f,23.8770f,-2.3333f,23.8770f,-2.2292f,23.8282f,-2.2292f,23.8282f,-2.1250f,23.7793f,-2.1250f,23.7793f,-2.0208f,23.7305f,-2.0208f,23.7305f,-1.9167f,23.6817f,-1.9167f,23.6817f,-1.8125f,23.6329f,-1.8125f,23.6329f,-1.7083f,
	23.5840f,-1.7083f,23.5840f,-1.6042f,23.5352f,-1.6042f,23.5352f,-1.5000f,23.4864f,-1.5000f,23.4864f,-1.3958f,23.4375f,-1.3958f,23.4375f,-1.2917f,23.3887f,-1.2917f,23.3887f,-1.1875f,23.3399f,-1.1875f,23.3399f,-1.0833f,23.2911f,-1.0833f,23.2911f,-0.9792f,23.2422f,-0.9792f,
	23.2422f,-0.8750f,23.1934f,-0.8750f,23.1934f,-0.7708f,23.1446f,-0.7708f,23.1446f,-0.6667f,23.0958f,-0.6667f,23.0958f,-0.5625f,23.0469f,-0.5625f,23.0469f,-0.4583f,22.9981f,-0.4583f,22.9981f,-0.3542f,22.9493f,-0.3542f,22.9493f,-0.2500f,22.9004f,-0.2500f,22.9004f,-0.1458f,
	22.8516f,-0.1458f,22.8516f,-0.0417f,22.8028f,-0.0417f,22.8028f,0.0625f,22.7540f,0.0625f,22.7540f,0.1667f,22.7051f,0.1667f,22.7051f,0.2708f,22.6563f,0.2708f,22.6563f,0.3750f,22.6075f,0.3750f,22.6075f,0.4792f,22.5586f,0.4792f,22.5586f,0.5833f,22.5098f,0.5833f,
	22.5098f,0.6875f,22.4610f,0.6875f,22.4610f,0.7917f,22.4122f,0.7917f,22.4122f,0.8958f,22.3633f,0.8958f,22.3633f,1.0000f,22.3145f,1.0000f,24.7071f,-3.3750f,24.6583f,-3.2708f,24.7071f,-3.2708f,24.6583f,-3.1667f,24.7071f,-3.1667f,24.6583f,-3.0625f,24.7071f,-3.0625f,
	24.6583f,-2.9583f,24.7071f,-2.9583f,24.6583f,-2.8542f,24.7071f,-2.8542f,24.6583f,-2.7500f,24.7071f,-2.7500f,24.6583f,-2.6458f,24.7071f,-2.6458f,24.6583f,-2.5417f,24.7071f,-2.5417f,24.6583f,-2.4375f,24.7071f,-2.4375f,24.6583f,-2.3333f,24.7071f,-2.3333f,24.6583f,-2.2292f,
	24.7071f,-2.2292f,24.6583f,-2.1250f,24.7071f,-2.1250f,24.6583f,-2.0208f,24.7071f,-2.0208f,24.6583f,-1.9167f,24.7071f,-1.9167f,24.6583f,-1.8125f,24.7071f,-1.8125f,24.6583f,-1.7083f,24.7071f,-1.7083f,24.6583f,-1.6042f,24.7071f,-1.6042f,24.6583f,-1.5000f,24.7071f,-1.5000f,
	24.6583f,-1.3958f,24.7071f,-1.3958f,24.6583f,-1.2917f,24.7071f,-1.2917f,24.6583f,-1.1875f,24.7071f,-1.1875f,24.6583f,-1.0833f,24.7071f,-1.0833f,24.6583f,-0.9792f,24.7071f,-0.9792f,24.6583f,-0.8750f,24.7071f,-0.8750f,24.6583f,-0.7708f,24.7071f,-0.7708f,24.6583f,-0.6667f,
	24.7071f,-0.6667f,24.6583f,-0.5625f,24.7071f,-0.5625f,24.6583f,-0.4583f,24.7071f,-0.4583f,24.6583f,-0.3542f,24.7071f,-0.3542f,24.6583f,-0.2500f,24.7071f,-0.2500f,24.6583f,-0.1458f,24.7071f,-0.1458f,24.6583f,-0.0417f,24.7071f,-0.0417f,24.6583f,0.0625f,24.7071f,0.0625f,
	24.6583f,0.1667f,24.7071f,0.1667f,24.6583f,0.2708f,24.7071f,0.2708f,24.6583f,0.3750f,24.7071f,0.3750f,24.6583f,0.4792f,24.7071f,0.4792f,24.6583f,0.5833f,24.7071f,0.5833f,24.6583f,0.6875f,24.7071f,0.6875f,24.6583f,0.7917f,24.7071f,0.7917f,24.6583f,0.8958f,
	24.7071f,0.8958f,24.6583f,1.0000f,24.7071f,1.0000f,24.6583f,-4.0000f,24.6094f,-4.0000f,24.6094f,-3.8958f,24.5606f,-3.8958f,24.5606f,-3.7917f,24.5118f,-3.7917f,24.5118f,-3.6875f,24.4629f,-3.6875f,24.4629f,-3.5833f,24.4141f,-3.5833f,24.4141f,-3.4792f,24.3653f,-3.4792f,
	24.3653f,-3.3750f,24.3165f,-3.3750f,24.3165f,-3.2708f,24.2676f,-3.2708f,24.2676f,-3.1667f,24.2188f,-3.1667f,24.2188f,-3.0625f,24.1700f,-3.0625f,24.1700f,-2.9583f,24.1211f,-2.9583f,24.1211f,-2.8542f,24.0723f,-2.8542f,24.0723f,-2.7500f,24.0235f,-2.7500f,24.0235f,-2.6458f,
	23.9747f,-2.6458f,23.9747f,-2.5417f,23.9258f,-2.5417f,23.9258f,-2.4375f,23.8770f,-2.4375f,23.8770f,-2.3333f,23.8282f,-2.3333f,23.8282f,-2.2292f,23.7793f,-2.2292f,23.7793f,-2.1250f,23.7305f,-2.1250f,23.7305f,-2.0208f,23.6817f,-2.0208f,23.6817f,-1.9167f,23.6329f,-1.9167f,
	23.6329f,-1.8125f,23.5840f,-1.8125f,23.5840f,-1.7083f,23.5352f,-1.7083f,23.5352f,-1.6042f,23.4864f,-1.6042f,23.4864f,-1.5000f,23.4375f,-1.5000f,23.4375f,-1.3958f,23.3887f,-1.3958f,23.3887f,-1.2917f,23.3399f,-1.2917f,23.3399f,-1.1875f,23.2911f,-1.1875f,23.2911f,-1.0833f,
	23.2422f,-1.0833f,23.2422f,-0.9792f,23.1934f,-0.9792f,23.1934f,-0.8750f,23.1446f,-0.8750f,23.1446f,-0.7708f,23.0958f,-0.7708f,23.0958f,-0.6667f,23.0469f,-0.6667f,23.0469f,-0.5625f,22.9981f,-0.5625f,22.9981f,-0.4583f,22.9493f,-0.4583f,22.9493f,-0.3542f,22.9004f,-0.3542f,
	22.9004f,-0.2500f,22.8516f,-0.2500f,22.8516f,-0.1458f,22.8028f,-0.1458f,22.8028f,-0.0417f,22.7540f,-0.0417f,22.7540f,0.0625f,22.7051f,0.0625f,22.7051f,0.1667f,22.6563f,0.1667f,22.6563f,0.2708f,22.6075f,0.2708f,22.6075f,0.3750f,22.5586f,0.3750f,22.5586f,0.4792f,
	22.5098f,0.4792f,22.5098f,0.5833f,22.4610f,0.5833f,22.4610f,0.6875f,22.4122f,0.6875f,22.4122f,0.7917f,22.3633f,0.7917f,22.3633f,0.8958f,22.3145f,0.8958f,22.3145f,1.0000f,22.2657f,1.0000f,24.6583f,-3.2708f,24.6094f,-3.1667f,24.6583f,-3.1667f,24.6094f,-3.0625f,
	24.6583f,-3.0625f,24.6094f,-2.9583f,24.6583f,-2.9583f,24.6094f,-2.8542f,24.6583f,-2.8542f,24.6094f,-2.7500f,24.6583f,-2.7500f,24.6094f,-2.6458f,24.6583f,-2.6458f,24.6094f,-2.5417f,24.6583f,-2.5417f,24.6094f,-2.4375f,24.6583f,-2.4375f,24.6094f,-2.3333f,24.6583f,-2.3333f,
	24.6094f,-2.2292f,24.6583f,-2.2292f,24.6094f,-2.1250f,24.6583f,-2.1250f,24.6094f,-2.0208f,24.6583f,-2.0208f,24.6094f,-1.9167f,24.6583f,-1.9167f,24.6094f,-1.8125f,24.6583f,-1.8125f,24.6094f,-1.7083f,24.6583f,-1.7083f,24.6094f,-1.6042f,24.6583f,-1.6042f,24.6094f,-1.5000f,
	24.6583f,-1.5000f,24.6094f,-1.3958f,24.6583f,-1.3958f,24.6094f,-1.2917f,24.6583f,-1.2917f,24.6094f,-1.1875f,24.6583f,-1.1875f,24.6094f,-1.0833f,24.6583f,-1.0833f,24.6094f,-0.9792f,24.6583f,-0.9792f,24.6094f,-0.8750f,24.6583f,-0.8750f,24.6094f,-0.7708f,24.6583f,-0.7708f,
	24.6094f,-0.6667f,24.6583f,-0.6667f,24.6094f,-0.5625f,24.6583f,-0.5625f,24.6094f,-0.4583f,24.6583f,-0.4583f,24.6094f,-0.3542f,24.6583f,-0.3542f,24.6094f,-0.2500f,24.6583f,-0.2500f,24.6094f,-0.1458f,24.6583f,-0.1458f,24.6094f,-0.0417f,24.6583f,-0.0417f,24.6094f,0.0625f,
	24.6583f,0.0625f,24.6094f,0.1667f,24.6583f,0.1667f,24.6094f,0.2708f,24.6583f,0.2708f,24.6094f,0.3750f,24.6583f,0.3750f,24.6094f,0.4792f,24.6583f,0.4792f,24.6094f,0.5833f,24.6583f,0.5833f,24.6094f,0.6875f,24.6583f,0.6875f,24.6094f,0.7917f,24.6583f,0.7917f,
	24.6094f,0.8958f,24.6583f,0.8958f,24.6094f,1.0000f,24.6583f,1.0000f,24.6094f,-4.0000f,24.5606f,-4.0000f,24.5606f,-3.8958f,24.5118f,-3.8958f,24.5118f,-3.7917f,24.4629f,-3.7917f,24.4629f,-3.6875f,24.4141f,-3.6875f,24.4141f,-3.5833f,24.3653f,-3.5833f,24.3653f,-3.4792f,
	24.3165f,-3.4792f,24.3165f,-3.3750f,24.2676f,-3.3750f,24.2676f,-3.2708f,24.2188f,-3.2708f,24.2188f,-3.1667f,24.1700f,-3.1667f,24.1700f,-3.0625f,24.1211f,-3.0625f,24.1211f,-2.9583f,24.0723f,-2.9583f,24.0723f,-2.8542f,24.0235f,-2.8542f,24.0235f,-2.7500f,23.9747f,-2.7500f,
	23.9747f,-2.6458f,23.9258f,-2.6458f,23.9258f,-2.5417f,23.8770f,-2.5417f,23.8770f,-2.4375f,23.8282f,-2.4375f,23.8282f,-2.3333f,23.7793f,-2.3333f,23.7793f,-2.2292f,23.7305f,-2.2292f,23.7305f,-2.1250f,23.6817f,-2.1250f,23.6817f,-2.0208f,23.6329f,-2.0208f,23.6329f,-1.9167f,
	23.5840f,-1.9167f,23.5840f,-1.8125f,23.5352f,-1.8125f,23.5352f,-1.7083f,23.4864f,-1.7083f,23.4864f,-1.6042f,23.4375f,-1.6042f,23.4375f,-1.5000f,23.3887f,-1.5000f,23.3887f,-1.3958f,23.3399f,-1.3958f,23.3399f,-1.2917f,23.2911f,-1.2917f,23.2911f,-1.1875f,23.2422f,-1.1875f,
	23.2422f,-1.0833f,23.1934f,-1.0833f,23.1934f,-0.9792f,23.1446f,-0.9792f,23.1446f,-0.8750f,23.0958f,-0.8750f,23.0958f,-0.7708f,23.0469f,-0.7708f,23.0469f,-0.6667f,22.9981f,-0.6667f,22.9981f,-0.5625f,22.9493f,-0.5625f,22.9493f,-0.4583f,22.9004f,-0.4583f,22.9004f,-0.3542f,
	22.8516f,-0.3542f,22.8516f,-0.2500f,22.8028f,-0.2500f,22.8028f,-0.1458f,22.7540f,-0.1458f,22.7540f,-0.0417f,22.7051f,-0.0417f,22.7051f,0.0625f,22.6563f,0.0625f,22.6563f,0.1667f,22.6075f,0.1667f,22.6075f,0.2708f,22.5586f,0.2708f,22.5586f,0.3750f,22.5098f,0.3750f,
	22.5098f,0.4792f,22.4610f,0.4792f,22.4610f,0.5833f,22.4122f,0.5833f,22.4122f,0.6875f,22.3633f,0.6875f,22.3633f,0.7917f,22.3145f,0.7917f,22.3145f,0.8958f,22.2657f,0.8958f,22.2657f,1.0000f,22.2168f,1.0000f,24.6094f,-3.1667f,24.5606f,-3.0625f,24.6094f,-3.0625f,
	24.5606f,-2.9583f,24.6094f,-2.9583f,24.5606f,-2.8542f,24.6094f,-2.8542f,24.5606f,-2.7500f,24.6094f,-2.7500f,24.5606f,-2.6458f,24.6094f,-2.6458f,24.5606f,-2.5417f,24.6094f,-2.5417f,24.5606f,-2.4375f,24.6094f,-2.4375f,24.5606f,-2.3333f,24.6094f,-2.3333f,24.5606f,-2.2292f,
	24.6094f,-2.2292f,24.5606f,-2.1250f,24.6094f,-2.1250f,24.5606f,-2.0208f,24.6094f,-2.0208f,24.5606f,-1.9167f,24.6094f,-1.9167f,24.5606f,-1.8125f,24.6094f,-1.8125f,24.5606f,-1.7083f,24.6094f,-1.7083f,24.5606f,-1.6042f,24.6094f,-1.6042f,24.5606f,-1.5000f,24.6094f,-1.5000f,
	24.5606f,-1.3958f,24.6094f,-1.3958f,24.5606f,-1.2917f,24.6094f,-1.2917f,24.5606f,-1.1875f,24.6094f,-1.1875f,24.5606f,-1.0833f,24.6094f,-1.0833f,24.5606f,-0.9792f,24.6094f,-0.9792f,24.5606f,-0.8750f,24.6094f,-0.8750f,24.5606f,-0.7708f,24.6094f,-0.7708f,24.5606f,-0.6667f,
	24.6094f,-0.6667f,24.5606f,-0.5625f,24.6094f,-0.5625f,24.5606f,-0.4583f,24.6094f,-0.4583f,24.5606f,-0.3542f,24.6094f,-0.3542f,24.5606f,-0.2500f,24.6094f,-0.2500f,24.5606f,-0.1458f,24.6094f,-0.1458f,24.5606f,-0.0417f,24.6094f,-0.0417f,24.5606f,0.0625f,24.6094f,0.0625f,
	24.5606f,0.1667f,24.6094f,0.1667f,24.5606f,0.2708f,24.6094f,0.2708f,24.5606f,0.3750f,24.6094f,0.3750f,24.5606f,0.4792f,24.6094f,0.4792f,24.5606f,0.5833f,24.6094f,0.5833f,24.5606f,0.6875f,24.6094f,0.6875f,24.5606f,0.7917f,24.6094f,0.7917f,24.5606f,0.8958f,
	24.6094f,0.8958f,24.5606f,1.0000f,24.6094f,1.0000f,24.5606f,-4.0000f,24.5118f,-4.0000f,24.5118f,-3.8958f,24.4629f,-3.8958f,24.4629f,-3.7917f,24.4141f,-3.7917f,24.4141f,-3.6875f,24.3653f,-3.6875f,24.3653f,-3.5833f,24.3165f,-3.5833f,24.3165f,-3.4792f,24.2676f,-3.4792f,
	24.2676f,-3.3750f,24.2188f,-3.3750f,24.2188f,-3.2708f,24.1700f,-3.2708f,24.1700f,-3.1667f,24.1211f,-3.1667f,24.1211f,-3.0625f,24.0723f,-3.0625f,24.0723f,-2.9583f,24.0235f,-2.9583f,24.0235f,-2.8542f,23.9747f,-2.8542f,23.9747f,-2.7500f,23.9258f,-2.7500f,23.9258f,-2.6458f,
	23.8770f,-2.6458f,23.8770f,-2.5417f,23.8282f,-2.5417f,23.8282f,-2.4375f,23.7793f,-2.4375f,23.7793f,-2.3333f,23.7305f,-2.3333f,23.7305f,-2.2292f,23.6817f,-2.2292f,23.6817f,-2.1250f,23.6329f,-2.1250f,23.6329f,-2.0208f,23.5840f,-2.0208f,23.5840f,-1.9167f,23.5352f,-1.9167f,
	23.5352f,-1.8125f,23.4864f,-1.8125f,23.4864f,-1.7083f,23.4375f,-1.7083f,23.4375f,-1.6042f,23.3887f,-1.6042f,23.3887f,-1.5000f,23.3399f,-1.5000f,23.3399f,-1.3958f,23.2911f,-1.3958f,23.2911f,-1.2917f,23.2422f,-1.2917f,23.2422f,-1.1875f,23.1934f,-1.1875f,23.1934f,-1.0833f,
	23.1446f,-1.0833f,23.1446f,-0.9792f,23.0958f,-0.9792f,23.0958f,-0.8750f,23.0469f,-0.8750f,23.0469f,-0.7708f,22.9981f,-0.7708f,22.9981f,-0.6667f,22.9493f,-0.6667f,22.9493f,-0.5625f,22.9004f,-0.5625f,22.9004f,-0.4583f,22.8516f,-0.4583f,22.8516f,-0.3542f,22.8028f,-0.3542f,
	22.8028f,-0.2500f,22.7540f,-0.2500f,22.7540f,-0.1458f,22.7051f,-0.1458f,22.7051f,-0.0417f,22.6563f,-0.0417f,22.6563f,0.0625f,22.6075f,0.0625f,22.6075f,0.1667f,22.5586f,0.1667f,22.5586f,0.2708f,22.5098f,0.2708f,22.5098f,0.3750f,22.4610f,0.3750f,22.4610f,0.4792f,
	22.4122f,0.4792f,22.4122f,0.5833f,22.3633f,0.5833f,22.3633f,0.6875f,22.3145f,0.6875f,22.3145f,0.7917f,22.2657f,0.7917f,22.2657f,0.8958f,22.2168f,0.8958f,22.2168f,1.0000f,22.1680f,1.0000f,24.5606f,-3.0625f,24.5118f,-2.9583f,24.5606f,-2.9583f,24.5118f,-2.8542f,
	24.5606f,-2.8542f,24.5118f,-2.7500f,24.5606f,-2.7500f,24.5118f,-2.6458f,24.5606f,-2.6458f,24.5118f,-2.5417f,24.5606f,-2.5417f,24.5118f,-2.4375f,24.5606f,-2.4375f,24.5118f,-2.3333f,24.5606f,-2.3333f,24.5118f,-2.2292f,24.5606f,-2.2292f,24.5118f,-2.1250f,24.5606f,-2.1250f,
	24.5118f,-2.0208f,24.5606f,-2.0208f,24.5118f,-1.9167f,24.5606f,-1.9167f,24.5118f,-1.8125f,24.5606f,-1.8125f,24.5118f,-1.7083f,24.5606f,-1.7083f,24.5118f,-1.6042f,24.5606f,-1.6042f,24.5118f,-1.5000f,24.5606f,-1.5000f,24.5118f,-1.3958f,24.5606f,-1.3958f,24.5118f,-1.2917f,
	24.5606f,-1.2917f,24.5118f,-1.1875f,24.5606f,-1.1875f,24.5118f,-1.0833f,24.5606f,-1.0833f,24.5118f,-0.9792f,24.5606f,-0.9792f,24.5118f,-0.8750f,24.5606f,-0.8750f,24.5118f,-0.7708f,24.5606f,-0.7708f,24.5118f,-0.6667f,24.5606f,-0.6667f,24.5118f,-0.5625f,24.5606f,-0.5625f,
	24.5118f,-0.4583f,24.5606f,-0.4583f,24.5118f,-0.3542f,24.5606f,-0.3542f,24.5118f,-0.2500f,24.5606f,-0.2500f,24.5118f,-0.1458f,24.5606f,-0.1458f,24.5118f,-0.0417f,24.5606f,-0.0417f,24.5118f,0.0625f,24.5606f,0.0625f,24.5118f,0.1667f,24.5606f,0.1667f,24.5118f,0.2708f,
	24.5606f,0.2708f,24.5118f,0.3750f,24.5606f,0.3750f,24.5118f,0.4792f,24.5606f,0.4792f,24.5118f,0.5833f,24.5606f,0.5833f,24.5118f,0.6875f,24.5606f,0.6875f,24.5118f,0.7917f,24.5606f,0.7917f,24.5118f,0.8958f,24.5606f,0.8958f,24.5118f,1.0000f,24.5606f,1.0000f,
	24.5118f,-4.0000f,24.4629f,-4.0000f,24.4629f,-3.8958f,24.4141f,-3.8958f,24.4141f,-3.7917f,24.3653f,-3.7917f,24.3653f,-3.6875f,24.3165f,-3.6875f,24.3165f,-3.5833f,24.2676f,-3.5833f,24.2676f,-3.4792f,24.2188f,-3.4792f,24.2188f,-3.3750f,24.1700f,-3.3750f,24.1700f,-3.2708f,
	24.1211f,-3.2708f,24.1211f,-3.1667f,24.0723f,-3.1667f,24.0723f,-3.0625f,24.0235f,-3.0625f,24.0235f,-2.9583f,23.9747f,-2.9583f,23.9747f,-2.8542f,23.9258f,-2.8542f,23.9258f,-2.7500f,23.8770f,-2.7500f,23.8770f,-2.6458f,23.8282f,-2.6458f,23.8282f,-2.5417f,23.7793f,-2.5417f,
	23.7793f,-2.4375f,23.7305f,-2.4375f,23.7305f,-2.3333f,23.6817f,-2.3333f,23.6817f,-2.2292f,23.6329f,-2.2292f,23.6329f,-2.1250f,23.5840f,-2.1250f,23.5840f,-2.0208f,23.5352f,-2.0208f,23.5352f,-1.9167f,23.4864f,-1.9167f,23.4864f,-1.8125f,23.4375f,-1.8125f,23.4375f,-1.7083f,
	23.3887f,-1.7083f,23.3887f,-1.6042f,23.3399f,-1.6042f,23.3399f,-1.5000f,23.2911f,-1.5000f,23.2911f,-1.3958f,23.2422f,-1.3958f,23.2422f,-1.2917f,23.1934f,-1.2917f,23.1934f,-1.1875f,23.1446f,-1.1875f,23.1446f,-1.0833f,23.0958f,-1.0833f,23.0958f,-0.9792f,23.0469f,-0.9792f,
	23.0469f,-0.8750f,22.9981f,-0.8750f,22.9981f,-0.7708f,22.9493f,-0.7708f,22.9493f,-0.6667f,22.9004f,-0.6667f,22.9004f,-0.5625f,22.8516f,-0.5625f,22.8516f,-0.4583f,22.8028f,-0.4583f,22.8028f,-0.3542f,22.7540f,-0.3542f,22.7540f,-0.2500f,22.7051f,-0.2500f,22.7051f,-0.1458f,
	22.6563f,-0.1458f,22.6563f,-0.0417f,22.6075f,-0.0417f,22.6075f,0.0625f,22.5586f,0.0625f,22.5586f,0.1667f,22.5098f,0.1667f,22.5098f,0.2708f,22.4610f,0.2708f,22.4610f,0.3750f,22.4122f,0.3750f,22.4122f,0.4792f,22.3633f,0.4792f,22.3633f,0.5833f,22.3145f,0.5833f,
	22.3145f,0.6875f,22.2657f,0.6875f,22.2657f,0.7917f,22.2168f,0.7917f,22.2168f,0.8958f,22.1680f,0.8958f,22.1680f,1.0000f,22.1192f,1.0000f,24.5118f,-2.9583f,24.4629f,-2.8542f,24.5118f,-2.8542f,24.4629f,-2.7500f,24.5118f,-2.7500f,24.4629f,-2.6458f,24.5118f,-2.6458f,
	24.4629f,-2.5417f,24.5118f,-2.5417f,24.4629f,-2.4375f,24.5118f,-2.4375f,24.4629f,-2.3333f,24.5118f,-2.3333f,24.4629f,-2.2292f,24.5118f,-2.2292f,24.4629f,-2.1250f,24.5118f,-2.1250f,24.4629f,-2.0208f,24.5118f,-2.0208f,24.4629f,-1.9167f,24.5118f,-1.9167f,24.4629f,-1.8125f,
	24.5118f,-1.8125f,24.4629f,-1.7083f,24.5118f,-1.7083f,24.4629f,-1.6042f,24.5118f,-1.6042f,24.4629f,-1.5000f,24.5118f,-1.5000f,24.4629f,-1.3958f,24.5118f,-1.3958f,24.4629f,-1.2917f,24.5118f,-1.2917f,24.4629f,-1.1875f,24.5118f,-1.1875f,24.4629f,-1.0833f,24.5118f,-1.0833f,
	24.4629f,-0.9792f,24.5118f,-0.9792f,24.4629f,-0.8750f,24.5118f,-0.8750f,24.4629f,-0.7708f,24.5118f,-0.7708f,24.4629f,-0.6667f,24.5118f,-0.6667f,24.4629f,-0.5625f,24.5118f,-0.5625f,24.4629f,-0.4583f,24.5118f,-0.4583f,24.4629f,-0.3542f,24.5118f,-0.3542f,24.4629f,-0.2500f,
	24.5118f,-0.2500f,24.4629f,-0.1458f,24.5118f,-0.1458f,24.4629f,-0.0417f,24.5118f,-0.0417f,24.4629f,0.0625f,24.5118f,0.0625f,24.4629f,0.1667f,24.5118f,0.1667f,24.4629f,0.2708f,24.5118f,0.2708f,24.4629f,0.3750f,24.5118f,0.3750f,24.4629f,0.4792f,24.5118f,0.4792f,
	24.4629f,0.5833f,24.5118f,0.5833f,24.4629f,0.6875f,24.5118f,0.6875f,24.4629f,0.7917f,24.5118f,0.7917f,24.4629f,0.8958f,24.5118f,0.8958f,24.4629f,1.0000f,24.5118f,1.0000f,24.4629f,-4.0000f,24.4141f,-4.0000f,24.4141f,-3.8958f,24.3653f,-3.8958f,24.3653f,-3.7917f,
	24.3165f,-3.7917f,24.3165f,-3.6875f,24.2676f,-3.6875f,24.2676f,-3.5833f,24.2188f,-3.5833f,24.2188f,-3.4792f,24.1700f,-3.4792f,24.1700f,-3.3750f,24.1211f,-3.3750f,24.1211f,-3.2708f,24.0723f,-3.2708f,24.0723f,-3.1667f,24.0235f,-3.1667f,24.0235f,-3.0625f,23.9747f,-3.0625f,
	23.9747f,-2.9583f,23.9258f,-2.9583f,23.9258f,-2.8542f,23.8770f,-2.8542f,23.8770f,-2.7500f,23.8282f,-2.7500f,23.8282f,-2.6458f,23.7793f,-2.6458f,23.7793f,-2.5417f,23.7305f,-2.5417f,23.7305f,-2.4375f,23.6817f,-2.4375f,23.6817f,-2.3333f,23.6329f,-2.3333f,23.6329f,-2.2292f,
	23.5840f,-2.2292f,23.5840f,-2.1250f,23.5352f,-2.1250f,23.5352f,-2.0208f,23.4864f,-2.0208f,23.4864f,-1.9167f,23.4375f,-1.9167f,23.4375f,-1.8125f,23.3887f,-1.8125f,23.3887f,-1.7083f,23.3399f,-1.7083f,23.3399f,-1.6042f,23.2911f,-1.6042f,23.2911f,-1.5000f,23.2422f,-1.5000f,
	23.2422f,-1.3958f,23.1934f,-1.3958f,23.1934f,-1.2917f,23.1446f,-1.2917f,23.1446f,-1.1875f,23.0958f,-1.1875f,23.0958f,-1.0833f,23.0469f,-1.0833f,23.0469f,-0.9792f,22.9981f,-0.9792f,22.9981f,-0.8750f,22.9493f,-0.8750f,22.9493f,-0.7708f,22.9004f,-0.7708f,22.9004f,-0.6667f,
	22.8516f,-0.6667f,22.8516f,-0.5625f,22.8028f,-0.5625f,22.8028f,-0.4583f,22.7540f,-0.4583f,22.7540f,-0.3542f,22.7051f,-0.3542f,22.7051f,-0.2500f,22.6563f,-0.2500f,22.6563f,-0.1458f,22.6075f,-0.1458f,22.6075f,-0.0417f,22.5586f,-0.0417f,22.5586f,0.0625f,22.5098f,0.0625f,
	22.5098f,0.1667f,22.4610f,0.1667f,22.4610f,0.2708f,22.4122f,0.2708f,22.4122f,0.3750f,22.3633f,0.3750f,22.3633f,0.4792f,22.3145f,0.4792f,22.3145f,0.5833f,22.2657f,0.5833f,22.2657f,0.6875f,22.2168f,0.6875f,22.2168f,0.7917f,22.1680f,0.7917f,22.1680f,0.8958f,
	22.1192f,0.8958f,22.1192f,1.0000f,22.0704f,1.0000f,24.4629f,-2.8542f,24.4141f,-2.7500f,24.4629f,-2.7500f,24.4141f,-2.6458f,24.4629f,-2.6458f,24.4141f,-2.5417f,24.4629f,-2.5417f,24.4141f,-2.4375f,24.4629f,-2.4375f,24.4141f,-2.3333f,24.4629f,-2.3333f,24.4141f,-2.2292f,
	24.4629f,-2.2292f,24.4141f,-2.1250f,24.4629f,-2.1250f,24.4141f,-2.0208f,24.4629f,-2.0208f,24.4141f,-1.9167f,24.4629f,-1.9167f,24.4141f,-1.8125f,24.4629f,-1.8125f,24.4141f,-1.7083f,24.4629f,-1.7083f,24.4141f,-1.6042f,24.4629f,-1.6042f,24.4141f,-1.5000f,24.4629f,-1.5000f,
	24.4141f,-1.3958f,24.4629f,-1.3958f,24.4141f,-1.2917f,24.4629f,-1.2917f,24.4141f,-1.1875f,24.4629f,-1.1875f,24.4141f,-1.0833f,24.4629f,-1.0833f,24.4141f,-0.9792f,24.4629f,-0.9792f,24.4141f,-0.8750f,24.4629f,-0.8750f,24.4141f,-0.7708f,24.4629f,-0.7708f,24.4141f,-0.6667f,
	24.4629f,-0.6667f,24.4141f,-0.5625f,24.4629f,-0.5625f,24.4141f,-0.4583f,24.4629f,-0.4583f,24.4141f,-0.3542f,24.4629f,-0.3542f,24.4141f,-0.2500f,24.4629f,-0.2500f,24.4141f,-0.1458f,24.4629f,-0.1458f,24.4141f,-0.0417f,24.4629f,-0.0417f,24.4141f,0.0625f,24.4629f,0.0625f,
	24.4141f,0.1667f,24.4629f,0.1667f,24.4141f,0.2708f,24.4629f,0.2708f,24.4141f,0.3750f,24.4629f,0.3750f,24.4141f,0.4792f,24.4629f,0.4792f,24.4141f,0.5833f,24.4629f,0.5833f,24.4141f,0.6875f,24.4629f,0.6875f,24.4141f,0.7917f,24.4629f,0.7917f,24.4141f,0.8958f,
	24.4629f,0.8958f,24.4141f,1.0000f,24.4629f,1.0000f,24.4141f,-4.0000f,24.3653f,-4.0000f,24.3653f,-3.8958f,24.3165f,-3.8958f,24.3165f,-3.7917f,24.2676f,-3.7917f,24.2676f,-3.6875f,24.2188f,-3.6875f,24.2188f,-3.5833f,24.1700f,-3.5833f,24.1700f,-3.4792f,24.1211f,-3.4792f,
	24.1211f,-3.3750f,24.0723f,-3.3750f,24.0723f,-3.2708f,24.0235f,-3.2708f,24.0235f,-3.1667f,23.9747f,-3.1667f,23.9747f,-3.0625f,23.9258f,-3.0625f,23.9258f,-2.9583f,23.8770f,-2.9583f,23.8770f,-2.8542f,23.8282f,-2.8542f,23.8282f,-2.7500f,23.7793f,-2.7500f,23.7793f,-2.6458f,
	23.7305f,-2.6458f,23.7305f,-2.5417f,23.6817f,-2.5417f,23.6817f,-2.4375f,23.6329f,-2.4375f,23.6329f,-2.3333f,23.5840f,-2.3333f,23.5840f,-2.2292f,23.5352f,-2.2292f,23.5352f,-2.1250f,23.4864f,-2.1250f,23.4864f,-2.0208f,23.4375f,-2.0208f,23.4375f,-1.9167f,23.3887f,-1.9167f,
	23.3887f,-1.8125f,23.3399f,-1.8125f,23.3399f,-1.7083f,23.2911f,-1.7083f,23.2911f,-1.6042f,23.2422f,-1.6042f,23.2422f,-1.5000f,23.1934f,-1.5000f,23.1934f,-1.3958f,23.1446f,-1.3958f,23.1446f,-1.2917f,23.0958f,-1.2917f,23.0958f,-1.1875f,23.0469f,-1.1875f,23.0469f,-1.0833f,
	22.9981f,-1.0833f,22.9981f,-0.9792f,22.9493f,-0.9792f,22.9493f,-0.8750f,22.9004f,-0.8750f,22.9004f,-0.7708f,22.8516f,-0.7708f,22.8516f,-0.6667f,22.8028f,-0.6667f,22.8028f,-0.5625f,22.7540f,-0.5625f,22.7540f,-0.4583f,22.7051f,-0.4583f,22.7051f,-0.3542f,22.6563f,-0.3542f,
	22.6563f,-0.2500f,22.6075f,-0.2500f,22.6075f,-0.1458f,22.5586f,-0.1458f,22.5586f,-0.0417f,22.5098f,-0.0417f,22.5098f,0.0625f,22.4610f,0.0625f,22.4610f,0.1667f,22.4122f,0.1667f,22.4122f,0.2708f,22.3633f,0.2708f,22.3633f,0.3750f,22.3145f,0.3750f,22.3145f,0.4792f,
	22.2657f,0.4792f,22.2657f,0.5833f,22.2168f,0.5833f,22.2168f,0.6875f,22.1680f,0.6875f,22.1680f,0.7917f,22.1192f,0.7917f,22.1192f,0.8958f,22.0704f,0.8958f,22.0704f,1.0000f,22.0215f,1.0000f,24.4141f,-2.7500f,24.3653f,-2.6458f,24.4141f,-2.6458f,24.3653f,-2.5417f,
	24.4141f,-2.5417f,24.3653f,-2.4375f,24.4141f,-2.4375f,24.3653f,-2.3333f,24.4141f,-2.3333f,24.3653f,-2.2292f,24.4141f,-2.2292f,24.3653f,-2.1250f,24.4141f,-2.1250f,24.3653f,-2.0208f,24.4141f,-2.0208f,24.3653f,-1.9167f,24.4141f,-1.9167f,24.3653f,-1.8125f,24.4141f,-1.8125f,
	24.3653f,-1.7083f,24.4141f,-1.7083f,24.3653f,-1.6042f,24.4141f,-1.6042f,24.3653f,-1.5000f,24.4141f,-1.5000f,24.3653f,-1.3958f,24.4141f,-1.3958f,24.3653f,-1.2917f,24.4141f,-1.2917f,24.3653f,-1.1875f,24.4141f,-1.1875f,24.3653f,-1.0833f,24.4141f,-1.0833f,24.3653f,-0.9792f,
	24.4141f,-0.9792f,24.3653f,-0.8750f,24.4141f,-0.8750f,24.3653f,-0.7708f,24.4141f,-0.7708f,24.3653f,-0.6667f,24.4141f,-0.6667f,24.3653f,-0.5625f,24.4141f,-0.5625f,24.3653f,-0.4583f,24.4141f,-0.4583f,24.3653f,-0.3542f,24.4141f,-0.3542f,24.3653f,-0.2500f,24.4141f,-0.2500f,
	24.3653f,-0.1458f,24.4141f,-0.1458f,24.3653f,-0.0417f,24.4141f,-0.0417f,24.3653f,0.0625f,24.4141f,0.0625f,24.3653f,0.1667f,24.4141f,0.1667f,24.3653f,0.2708f,24.4141f,0.2708f,24.3653f,0.3750f,24.4141f,0.3750f,24.3653f,0.4792f,24.4141f,0.4792f,24.3653f,0.5833f,
	24.4141f,0.5833f,24.3653f,0.6875f,24.4141f,0.6875f,24.3653f,0.7917f,24.4141f,0.7917f,24.3653f,0.8958f,24.4141f,0.8958f,24.3653f,1.0000f,24.4141f,1.0000f,24.3653f,-4.0000f,24.3165f,-4.0000f,24.3165f,-3.8958f,24.2676f,-3.8958f,24.2676f,-3.7917f,24.2188f,-3.7917f,
	24.2188f,-3.6875f,24.1700f,-3.6875f,24.1700f,-3.5833f,24.1211f,-3.5833f,24.1211f,-3.4792f,24.0723f,-3.4792f,24.0723f,-3.3750f,24.0235f,-3.3750f,24.0235f,-3.2708f,23.9747f,-3.2708f,23.9747f,-3.1667f,23.9258f,-3.1667f,23.9258f,-3.0625f,23.8770f,-3.0625f,23.8770f,-2.9583f,
	23.8282f,-2.9583f,23.8282f,-2.8542f,23.7793f,-2.8542f,23.7793f,-2.7500f,23.7305f,-2.7500f,23.7305f,-2.6458f,23.6817f,-2.6458f,23.6817f,-2.5417f,23.6329f,-2.5417f,23.6329f,-2.4375f,23.5840f,-2.4375f,23.5840f,-2.3333f,23.5352f,-2.3333f,23.5352f,-2.2292f,23.4864f,-2.2292f,
	23.4864f,-2.1250f,23.4375f,-2.1250f,23.4375f,-2.0208f,23.3887f,-2.0208f,23.3887f,-1.9167f,23.3399f,-1.9167f,23.3399f,-1.8125f,23.2911f,-1.8125f,23.2911f,-1.7083f,23.2422f,-1.7083f,23.2422f,-1.6042f,23.1934f,-1.6042f,23.1934f,-1.5000f,23.1446f,-1.5000f,23.1446f,-1.3958f,
	23.0958f,-1.3958f,23.0958f,-1.2917f,23.0469f,-1.2917f,23.0469f,-1.1875f,22.9981f,-1.1875f,22.9981f,-1.0833f,22.9493f,-1.0833f,22.9493f,-0.9792f,22.9004f,-0.9792f,22.9004f,-0.8750f,22.8516f,-0.8750f,22.8516f,-0.7708f,22.8028f,-0.7708f,22.8028f,-0.6667f,22.7540f,-0.6667f,
	22.7540f,-0.5625f,22.7051f,-0.5625f,22.7051f,-0.4583f,22.6563f,-0.4583f,22.6563f,-0.3542f,22.6075f,-0.3542f,22.6075f,-0.2500f,22.5586f,-0.2500f,22.5586f,-0.1458f,22.5098f,-0.1458f,22.5098f,-0.0417f,22.4610f,-0.0417f,22.4610f,0.0625f,22.4122f,0.0625f,22.4122f,0.1667f,
	22.3633f,0.1667f,22.3633f,0.2708f,22.3145f,0.2708f,22.3145f,0.3750f,22.2657f,0.3750f,22.2657f,0.4792f,22.2168f,0.4792f,22.2168f,0.5833f,22.1680f,0.5833f,22.1680f,0.6875f,22.1192f,0.6875f,22.1192f,0.7917f,22.0704f,0.7917f,22.0704f,0.8958f,22.0215f,0.8958f,
	22.0215f,1.0000f,21.9727f,1.0000f,24.3653f,-2.6458f,24.3165f,-2.5417f,24.3653f,-2.5417f,24.3165f,-2.4375f,24.3653f,-2.4375f,24.3165f,-2.3333f,24.3653f,-2.3333f,24.3165f,-2.2292f,24.3653f,-2.2292f,24.3165f,-2.1250f,24.3653f,-2.1250f,24.3165f,-2.0208f,24.3653f,-2.0208f,
	24.3165f,-1.9167f,24.3653f,-1.9167f,24.3165f,-1.8125f,24.3653f,-1.8125f,24.3165f,-1.7083f,24.3653f,-1.7083f,24.3165f,-1.6042f,24.3653f,-1.6042f,24.3165f,-1.5000f,24.3653f,-1.5000f,24.3165f,-1.3958f,24.3653f,-1.3958f,24.3165f,-1.2917f,24.3653f,-1.2917f,24.3165f,-1.1875f,
	24.3653f,-1.1875f,24.3165f,-1.0833f,24.3653f,-1.0833f,24.3165f,-0.9792f,24.3653f,-0.9792f,24.3165f,-0.8750f,24.3653f,-0.8750f,24.3165f,-0.7708f,24.3653f,-0.7708f,24.3165f,-0.6667f,24.3653f,-0.6667f,24.3165f,-0.5625f,24.3653f,-0.5625f,24.3165f,-0.4583f,24.3653f,-0.4583f,
	24.3165f,-0.3542f,24.3653f,-0.3542f,24.3165f,-0.2500f,24.3653f,-0.2500f,24.3165f,-0.1458f,24.3653f,-0.1458f,24.3165f,-0.0417f,24.3653f,-0.0417f,24.3165f,0.0625f,24.3653f,0.0625f,24.3165f,0.1667f,24.3653f,0.1667f,24.3165f,0.2708f,24.3653f,0.2708f,24.3165f,0.3750f,
	24.3653f,0.3750f,24.3165f,0.4792f,24.3653f,0.4792f,24.3165f,0.5833f,24.3653f,0.5833f,24.3165f,0.6875f,24.3653f,0.6875f,24.3165f,0.7917f,24.3653f,0.7917f,24.3165f,0.8958f,24.3653f,0.8958f,24.3165f,1.0000f,24.3653f,1.0000f,24.3165f,-4.0000f,24.2676f,-4.0000f,
	24.2676f,-3.8958f,24.2188f,-3.8958f,24.2188f,-3.7917f,24.1700f,-3.7917f,24.1700f,-3.6875f,24.1211f,-3.6875f,24.1211f,-3.5833f,24.0723f,-3.5833f,24.0723f,-3.4792f,24.0235f,-3.4792f,24.0235f,-3.3750f,23.9747f,-3.3750f,23.9747f,-3.2708f,23.9258f,-3.2708f,23.9258f,-3.1667f,
	23.8770f,-3.1667f,23.8770f,-3.0625f,23.8282f,-3.0625f,23.8282f,-2.9583f,23.7793f,-2.9583f,23.7793f,-2.8542f,23.7305f,-2.8542f,23.7305f,-2.7500f,23.6817f,-2.7500f,23.6817f,-2.6458f,23.6329f,-2.6458f,23.6329f,-2.5417f,23.5840f,-2.5417f,23.5840f,-2.4375f,23.5352f,-2.4375f,
	23.5352f,-2.3333f,23.4864f,-2.3333f,23.4864f,-2.2292f,23.4375f,-2.2292f,23.4375f,-2.1250f,23.3887f,-2.1250f,23.3887f,-2.0208f,23.3399f,-2.0208f,23.3399f,-1.9167f,23.2911f,-1.9167f,23.2911f,-1.8125f,23.2422f,-1.8125f,23.2422f,-1.7083f,23.1934f,-1.7083f,23.1934f,-1.6042f,
	23.1446f,-1.6042f,23.1446f,-1.5000f,23.0958f,-1.5000f,23.0958f,-1.3958f,23.0469f,-1.3958f,23.0469f,-1.2917f,22.9981f,-1.2917f,22.9981f,-1.1875f,22.9493f,-1.1875f,22.9493f,-1.0833f,22.9004f,-1.0833f,22.9004f,-0.9792f,22.8516f,-0.9792f,22.8516f,-0.8750f,22.8028f,-0.8750f,
	22.8028f,-0.7708f,22.7540f,-0.7708f,22.7540f,-0.6667f,22.7051f,-0.6667f,22.7051f,-0.5625f,22.6563f,-0.5625f,22.6563f,-0.4583f,22.6075f,-0.4583f,22.6075f,-0.3542f,22.5586f,-0.3542f,22.5586f,-0.2500f,22.5098f,-0.2500f,22.5098f,-0.1458f,22.4610f,-0.1458f,22.4610f,-0.0417f,
	22.4122f,-0.0417f,22.4122f,0.0625f,22.3633f,0.0625f,22.3633f,0.1667f,22.3145f,0.1667f,22.3145f,0.2708f,22.2657f,0.2708f,22.2657f,0.3750f,22.2168f,0.3750f,22.2168f,0.4792f,22.1680f,0.4792f,22.1680f,0.5833f,22.1192f,0.5833f,22.1192f,0.6875f,22.0704f,0.6875f,
	22.0704f,0.7917f,22.0215f,0.7917f,22.0215f,0.8958f,21.9727f,0.8958f,21.9727f,1.0000f,21.9239f,1.0000f,24.3165f,-2.5417f,24.2676f,-2.4375f,24.3165f,-2.4375f,24.2676f,-2.3333f,24.3165f,-2.3333f,24.2676f,-2.2292f,24.3165f,-2.2292f,24.2676f,-2.1250f,24.3165f,-2.1250f,
	24.2676f,-2.0208f,24.3165f,-2.0208f,24.2676f,-1.9167f,24.3165f,-1.9167f,24.2676f,-1.8125f,24.3165f,-1.8125f,24.2676f,-1.7083f,24.3165f,-1.7083f,24.2676f,-1.6042f,24.3165f,-1.6042f,24.2676f,-1.5000f,24.3165f,-1.5000f,24.2676f,-1.3958f,24.3165f,-1.3958f,24.2676f,-1.2917f,
	24.3165f,-1.2917f,24.2676f,-1.1875f,24.3165f,-1.1875f,24.2676f,-1.0833f,24.3165f,-1.0833f,24.2676f,-0.9792f,24.3165f,-0.9792f,24.2676f,-0.8750f,24.3165f,-0.8750f,24.2676f,-0.7708f,24.3165f,-0.7708f,24.2676f,-0.6667f,24.3165f,-0.6667f,24.2676f,-0.5625f,24.3165f,-0.5625f,
	24.2676f,-0.4583f,24.3165f,-0.4583f,24.2676f,-0.3542f,24.3165f,-0.3542f,24.2676f,-0.2500f,24.3165f,-0.2500f,24.2676f,-0.1458f,24.3165f,-0.1458f,24.2676f,-0.0417f,24.3165f,-0.0417f,24.2676f,0.0625f,24.3165f,0.0625f,24.2676f,0.1667f,24.3165f,0.1667f,24.2676f,0.2708f,
	24.3165f,0.2708f,24.2676f,0.3750f,24.3165f,0.3750f,24.2676f,0.4792f,24.3165f,0.4792f,24.2676f,0.5833f,24.3165f,0.5833f,24.2676f,0.6875f,24.3165f,0.6875f,24.2676f,0.7917f,24.3165f,0.7917f,24.2676f,0.8958f,24.3165f,0.8958f,24.2676f,1.0000f,24.3165f,1.0000f,
	24.2676f,-4.0000f,24.2188f,-4.0000f,24.2188f,-3.8958f,24.1700f,-3.8958f,24.1700f,-3.7917f,24.1211f,-3.7917f,24.1211f,-3.6875f,24.0723f,-3.6875f,24.0723f,-3.5833f,24.0235f,-3.5833f,24.0235f,-3.4792f,23.9747f,-3.4792f,23.9747f,-3.3750f,23.9258f,-3.3750f,23.9258f,-3.2708f,
	23.8770f,-3.2708f,23.8770f,-3.1667f,23.8282f,-3.1667f,23.8282f,-3.0625f,23.7793f,-3.0625f,23.7793f,-2.9583f,23.7305f,-2.9583f,23.7305f,-2.8542f,23.6817f,-2.8542f,23.6817f,-2.7500f,23.6329f,-2.7500f,23.6329f,-2.6458f,23.5840f,-2.6458f,23.5840f,-2.5417f,23.5352f,-2.5417f,
	23.5352f,-2.4375f,23.4864f,-2.4375f,23.4864f,-2.3333f,23.4375f,-2.3333f,23.4375f,-2.2292f,23.3887f,-2.2292f,23.3887f,-2.1250f,23.3399f,-2.1250f,23.3399f,-2.0208f,23.2911f,-2.0208f,23.2911f,-1.9167f,23.2422f,-1.9167f,23.2422f,-1.8125f,23.1934f,-1.8125f,23.1934f,-1.7083f,
	23.1446f,-1.7083f,23.1446f,-1.6042f,23.0958f,-1.6042f,23.0958f,-1.5000f,23.0469f,-1.5000f,23.0469f,-1.3958f,22.9981f,-1.3958f,22.9981f,-1.2917f,22.9493f,-1.2917f,22.9493f,-1.1875f,22.9004f,-1.1875f,22.9004f,-1.0833f,22.8516f,-1.0833f,22.8516f,-0.9792f,22.8028f,-0.9792f,
	22.8028f,-0.8750f,22.7540f,-0.8750f,22.7540f,-0.7708f,22.7051f,-0.7708f,22.7051f,-0.6667f,22.6563f,-0.6667f,22.6563f,-0.5625f,22.6075f,-0.5625f,22.6075f,-0.4583f,22.5586f,-0.4583f,22.5586f,-0.3542f,22.5098f,-0.3542f,22.5098f,-0.2500f,22.4610f,-0.2500f,22.4610f,-0.1458f,
	22.4122f,-0.1458f,22.4122f,-0.0417f,22.3633f,-0.0417f,22.3633f,0.0625f,22.3145f,0.0625f,22.3145f,0.1667f,22.2657f,0.1667f,22.2657f,0.2708f,22.2168f,0.2708f,22.2168f,0.3750f,22.1680f,0.3750f,22.1680f,0.4792f,22.1192f,0.4792f,22.1192f,0.5833f,22.0704f,0.5833f,
	22.0704f,0.6875f,22.0215f,0.6875f,22.0215f,0.7917f,21.9727f,0.7917f,21.9727f,0.8958f,21.9239f,0.8958f,21.9239f,1.0000f,21.8750f,1.0000f,24.2676f,-2.4375f,24.2188f,-2.3333f,24.2676f,-2.3333f,24.2188f,-2.2292f,24.2676f,-2.2292f,24.2188f,-2.1250f,24.2676f,-2.1250f,
	24.2188f,-2.0208f,24.2676f,-2.0208f,24.2188f,-1.9167f,24.2676f,-1.9167f,24.2188f,-1.8125f,24.2676f,-1.8125f,24.2188f,-1.7083f,24.2676f,-1.7083f,24.2188f,-1.6042f,24.2676f,-1.6042f,24.2188f,-1.5000f,24.2676f,-1.5000f,24.2188f,-1.3958f,24.2676f,-1.3958f,24.2188f,-1.2917f,
	24.2676f,-1.2917f,24.2188f,-1.1875f,24.2676f,-1.1875f,24.2188f,-1.0833f,24.2676f,-1.0833f,24.2188f,-0.9792f,24.2676f,-0.9792f,24.2188f,-0.8750f,24.2676f,-0.8750f,24.2188f,-0.7708f,24.2676f,-0.7708f,24.2188f,-0.6667f,24.2676f,-0.6667f,24.2188f,-0.5625f,24.2676f,-0.5625f,
	24.2188f,-0.4583f,24.2676f,-0.4583f,24.2188f,-0.3542f,24.2676f,-0.3542f,24.2188f,-0.2500f,24.2676f,-0.2500f,24.2188f,-0.1458f,24.2676f,-0.1458f,24.2188f,-0.0417f,24.2676f,-0.0417f,24.2188f,0.0625f,24.2676f,0.0625f,24.2188f,0.1667f,24.2676f,0.1667f,24.2188f,0.2708f,
	24.2676f,0.2708f,24.2188f,0.3750f,24.2676f,0.3750f,24.2188f,0.4792f,24.2676f,0.4792f,24.2188f,0.5833f,24.2676f,0.5833f,24.2188f,0.6875f,24.2676f,0.6875f,24.2188f,0.7917f,24.2676f,0.7917f,24.2188f,0.8958f,24.2676f,0.8958f,24.2188f,1.0000f,24.2676f,1.0000f,
	24.2188f,-4.0000f,24.1700f,-4.0000f,24.1700f,-3.8958f,24.1211f,-3.8958f,24.1211f,-3.7917f,24.0723f,-3.7917f,24.0723f,-3.6875f,24.0235f,-3.6875f,24.0235f,-3.5833f,23.9747f,-3.5833f,23.9747f,-3.4792f,23.9258f,-3.4792f,23.9258f,-3.3750f,23.8770f,-3.3750f,23.8770f,-3.2708f,
	23.8282f,-3.2708f,23.8282f,-3.1667f,23.7793f,-3.1667f,23.7793f,-3.0625f,23.7305f,-3.0625f,23.7305f,-2.9583f,23.6817f,-2.9583f,23.6817f,-2.8542f,23.6329f,-2.8542f,23.6329f,-2.7500f,23.5840f,-2.7500f,23.5840f,-2.6458f,23.5352f,-2.6458f,23.5352f,-2.5417f,23.4864f,-2.5417f,
	23.4864f,-2.4375f,23.4375f,-2.4375f,23.4375f,-2.3333f,23.3887f,-2.3333f,23.3887f,-2.2292f,23.3399f,-2.2292f,23.3399f,-2.1250f,23.2911f,-2.1250f,23.2911f,-2.0208f,23.2422f,-2.0208f,23.2422f,-1.9167f,23.1934f,-1.9167f,23.1934f,-1.8125f,23.1446f,-1.8125f,23.1446f,-1.7083f,
	23.0958f,-1.7083f,23.0958f,-1.6042f,23.0469f,-1.6042f,23.0469f,-1.5000f,22.9981f,-1.5000f,22.9981f,-1.3958f,22.9493f,-1.3958f,22.9493f,-1.2917f,22.9004f,-1.2917f,22.9004f,-1.1875f,22.8516f,-1.1875f,22.8516f,-1.0833f,22.8028f,-1.0833f,22.8028f,-0.9792f,22.7540f,-0.9792f,
	22.7540f,-0.8750f,22.7051f,-0.8750f,22.7051f,-0.7708f,22.6563f,-0.7708f,22.6563f,-0.6667f,22.6075f,-0.6667f,22.6075f,-0.5625f,22.5586f,-0.5625f,22.5586f,-0.4583f,22.5098f,-0.4583f,22.5098f,-0.3542f,22.4610f,-0.3542f,22.4610f,-0.2500f,22.4122f,-0.2500f,22.4122f,-0.1458f,
	22.3633f,-0.1458f,22.3633f,-0.0417f,22.3145f,-0.0417f,22.3145f,0.0625f,22.2657f,0.0625f,22.2657f,0.1667f,22.2168f,0.1667f,22.2168f,0.2708f,22.1680f,0.2708f,22.1680f,0.3750f,22.1192f,0.3750f,22.1192f,0.4792f,22.0704f,0.4792f,22.0704f,0.5833f,22.0215f,0.5833f,
	22.0215f,0.6875f,21.9727f,0.6875f,21.9727f,0.7917f,21.9239f,0.7917f,21.9239f,0.8958f,21.8750f,0.8958f,21.8750f,1.0000f,21.8262f,1.0000f,24.2188f,-2.3333f,24.1700f,-2.2292f,24.2188f,-2.2292f,24.1700f,-2.1250f,24.2188f,-2.1250f,24.1700f,-2.0208f,24.2188f,-2.0208f,
	24.1700f,-1.9167f,24.2188f,-1.9167f,24.1700f,-1.8125f,24.2188f,-1.8125f,24.1700f,-1.7083f,24.2188f,-1.7083f,24.1700f,-1.6042f,24.2188f,-1.6042f,24.1700f,-1.5000f,24.2188f,-1.5000f,24.1700f,-1.3958f,24.2188f,-1.3958f,24.1700f,-1.2917f,24.2188f,-1.2917f,24.1700f,-1.1875f,
	24.2188f,-1.1875f,24.1700f,-1.0833f,24.2188f,-1.0833f,24.1700f,-0.9792f,24.2188f,-0.9792f,24.1700f,-0.8750f,24.2188f,-0.8750f,24.1700f,-0.7708f,24.2188f,-0.7708f,24.1700f,-0.6667f,24.2188f,-0.6667f,24.1700f,-0.5625f,24.2188f,-0.5625f,24.1700f,-0.4583f,24.2188f,-0.4583f,
	24.1700f,-0.3542f,24.2188f,-0.3542f,24.1700f,-0.2500f,24.2188f,-0.2500f,24.1700f,-0.1458f,24.2188f,-0.1458f,24.1700f,-0.0417f,24.2188f,-0.0417f,24.1700f,0.0625f,24.2188f,0.0625f,24.1700f,0.1667f,24.2188f,0.1667f,24.1700f,0.2708f,24.2188f,0.2708f,24.1700f,0.3750f,
	24.2188f,0.3750f,24.1700f,0.4792f,24.2188f,0.4792f,24.1700f,0.5833f,24.2188f,0.5833f,24.1700f,0.6875f,24.2188f,0.6875f,24.1700f,0.7917f,24.2188f,0.7917f,24.1700f,0.8958f,24.2188f,0.8958f,24.1700f,1.0000f,24.2188f,1.0000f,24.1700f,-4.0000f,24.1211f,-4.0000f,
	24.1211f,-3.8958f,24.0723f,-3.8958f,24.0723f,-3.7917f,24.0235f,-3.7917f,24.0235f,-3.6875f,23.9747f,-3.6875f,23.9747f,-3.5833f,23.9258f,-3.5833f,23.9258f,-3.4792f,23.8770f,-3.4792f,23.8770f,-3.3750f,23.8282f,-3.3750f,23.8282f,-3.2708f,23.7793f,-3.2708f,23.7793f,-3.1667f,
	23.7305f,-3.1667f,23.7305f,-3.0625f,23.6817f,-3.0625f,23.6817f,-2.9583f,23.6329f,-2.9583f,23.6329f,-2.8542f,23.5840f,-2.8542f,23.5840f,-2.7500f,23.5352f,-2.7500f,23.5352f,-2.6458f,23.4864f,-2.6458f,23.4864f,-2.5417f,23.4375f,-2.5417f,23.4375f,-2.4375f,23.3887f,-2.4375f,
	23.3887f,-2.3333f,23.3399f,-2.3333f,23.3399f,-2.2292f,23.2911f,-2.2292f,23.2911f,-2.1250f,23.2422f,-2.1250f,23.2422f,-2.0208f,23.1934f,-2.0208f,23.1934f,-1.9167f,23.1446f,-1.9167f,23.1446f,-1.8125f,23.0958f,-1.8125f,23.0958f,-1.7083f,23.0469f,-1.7083f,23.0469f,-1.6042f,
	22.9981f,-1.6042f,22.9981f,-1.5000f,22.9493f,-1.5000f,22.9493f,-1.3958f,22.9004f,-1.3958f,22.9004f,-1.2917f,22.8516f,-1.2917f,22.8516f,-1.1875f,22.8028f,-1.1875f,22.8028f,-1.0833f,22.7540f,-1.0833f,22.7540f,-0.9792f,22.7051f,-0.9792f,22.7051f,-0.8750f,22.6563f,-0.8750f,
	22.6563f,-0.7708f,22.6075f,-0.7708f,22.6075f,-0.6667f,22.5586f,-0.6667f,22.5586f,-0.5625f,22.5098f,-0.5625f,22.5098f,-0.4583f,22.4610f,-0.4583f,22.4610f,-0.3542f,22.4122f,-0.3542f,22.4122f,-0.2500f,22.3633f,-0.2500f,22.3633f,-0.1458f,22.3145f,-0.1458f,22.3145f,-0.0417f,
	22.2657f,-0.0417f,22.2657f,0.0625f,22.2168f,0.0625f,22.2168f,0.1667f,22.1680f,0.1667f,22.1680f,0.2708f,22.1192f,0.2708f,22.1192f,0.3750f,22.0704f,0.3750f,22.0704f,0.4792f,22.0215f,0.4792f,22.0215f,0.5833f,21.9727f,0.5833f,21.9727f,0.6875f,21.9239f,0.6875f,
	21.9239f,0.7917f,21.8750f,0.7917f,21.8750f,0.8958f,21.8262f,0.8958f,21.8262f,1.0000f,21.7774f,1.0000f,24.1700f,-2.2292f,24.1211f,-2.1250f,24.1700f,-2.1250f,24.1211f,-2.0208f,24.1700f,-2.0208f,24.1211f,-1.9167f,24.1700f,-1.9167f,24.1211f,-1.8125f,24.1700f,-1.8125f,
	24.1211f,-1.7083f,24.1700f,-1.7083f,24.1211f,-1.6042f,24.1700f,-1.6042f,24.1211f,-1.5000f,24.1700f,-1.5000f,24.1211f,-1.3958f,24.1700f,-1.3958f,24.1211f,-1.2917f,24.1700f,-1.2917f,24.1211f,-1.1875f,24.1700f,-1.1875f,24.1211f,-1.0833f,24.1700f,-1.0833f,24.1211f,-0.9792f,
	24.1700f,-0.9792f,24.1211f,-0.8750f,24.1700f,-0.8750f,24.1211f,-0.7708f,24.1700f,-0.7708f,24.1211f,-0.6667f,24.1700f,-0.6667f,24.1211f,-0.5625f,24.1700f,-0.5625f,24.1211f,-0.4583f,24.1700f,-0.4583f,24.1211f,-0.3542f,24.1700f,-0.3542f,24.1211f,-0.2500f,24.1700f,-0.2500f,
	24.1211f,-0.1458f,24.1700f,-0.1458f,24.1211f,-0.0417f,24.1700f,-0.0417f,24.1211f,0.0625f,24.1700f,0.0625f,24.1211f,0.1667f,24.1700f,0.1667f,24.1211f,0.2708f,24.1700f,0.2708f,24.1211f,0.3750f,24.1700f,0.3750f,24.1211f,0.4792f,24.1700f,0.4792f,24.1211f,0.5833f,
	24.1700f,0.5833f,24.1211f,0.6875f,24.1700f,0.6875f,24.1211f,0.7917f,24.1700f,0.7917f,24.1211f,0.8958f,24.1700f,0.8958f,24.1211f,1.0000f,24.1700f,1.0000f,24.1211f,-4.0000f,24.0723f,-4.0000f,24.0723f,-3.8958f,24.0235f,-3.8958f,24.0235f,-3.7917f,23.9747f,-3.7917f,
	23.9747f,-3.6875f,23.9258f,-3.6875f,23.9258f,-3.5833f,23.8770f,-3.5833f,23.8770f,-3.4792f,23.8282f,-3.4792f,23.8282f,-3.3750f,23.7793f,-3.3750f,23.7793f,-3.2708f,23.7305f,-3.2708f,23.7305f,-3.1667f,23.6817f,-3.1667f,23.6817f,-3.0625f,23.6329f,-3.0625f,23.6329f,-2.9583f,
	23.5840f,-2.9583f,23.5840f,-2.8542f,23.5352f,-2.8542f,23.5352f,-2.7500f,23.4864f,-2.7500f,23.4864f,-2.6458f,23.4375f,-2.6458f,23.4375f,-2.5417f,23.3887f,-2.5417f,23.3887f,-2.4375f,23.3399f,-2.4375f,23.3399f,-2.3333f,23.2911f,-2.3333f,23.2911f,-2.2292f,23.2422f,-2.2292f,
	23.2422f,-2.1250f,23.1934f,-2.1250f,23.1934f,-2.0208f,23.1446f,-2.0208f,23.1446f,-1.9167f,23.0958f,-1.9167f,23.0958f,-1.8125f,23.0469f,-1.8125f,23.0469f,-1.7083f,22.9981f,-1.7083f,22.9981f,-1.6042f,22.9493f,-1.6042f,22.9493f,-1.5000f,22.9004f,-1.5000f,22.9004f,-1.3958f,
	22.8516f,-1.3958f,22.8516f,-1.2917f,22.8028f,-1.2917f,22.8028f,-1.1875f,22.7540f,-1.1875f,22.7540f,-1.0833f,22.7051f,-1.0833f,22.7051f,-0.9792f,22.6563f,-0.9792f,22.6563f,-0.8750f,22.6075f,-0.8750f,22.6075f,-0.7708f,22.5586f,-0.7708f,22.5586f,-0.6667f,22.5098f,-0.6667f,
	22.5098f,-0.5625f,22.4610f,-0.5625f,22.4610f,-0.4583f,22.4122f,-0.4583f,22.4122f,-0.3542f,22.3633f,-0.3542f,22.3633f,-0.2500f,22.3145f,-0.2500f,22.3145f,-0.1458f,22.2657f,-0.1458f,22.2657f,-0.0417f,22.2168f,-0.0417f,22.2168f,0.0625f,22.1680f,0.0625f,22.1680f,0.1667f,
	22.1192f,0.1667f,22.1192f,0.2708f,22.0704f,0.2708f,22.0704f,0.3750f,22.0215f,0.3750f,22.0215f,0.4792f,21.9727f,0.4792f,21.9727f,0.5833f,21.9239f,0.5833f,21.9239f,0.6875f,21.8750f,0.6875f,21.8750f,0.7917f,21.8262f,0.7917f,21.8262f,0.8958f,21.7774f,0.8958f,
	21.7774f,1.0000f,21.7286f,1.0000f,24.1211f,-2.1250f,24.0723f,-2.0208f,24.1211f,-2.0208f,24.0723f,-1.9167f,24.1211f,-1.9167f,24.0723f,-1.8125f,24.1211f,-1.8125f,24.0723f,-1.7083f,24.1211f,-1.7083f,24.0723f,-1.6042f,24.1211f,-1.6042f,24.0723f,-1.5000f,24.1211f,-1.5000f,
	24.0723f,-1.3958f,24.1211f,-1.3958f,24.0723f,-1.2917f,24.1211f,-1.2917f,24.0723f,-1.1875f,24.1211f,-1.1875f,24.0723f,-1.0833f,24.1211f,-1.0833f,24.0723f,-0.9792f,24.1211f,-0.9792f,24.0723f,-0.8750f,24.1211f,-0.8750f,24.0723f,-0.7708f,24.1211f,-0.7708f,24.0723f,-0.6667f,
	24.1211f,-0.6667f,24.0723f,-0.5625f,24.1211f,-0.5625f,24.0723f,-0.4583f,24.1211f,-0.4583f,24.0723f,-0.3542f,24.1211f,-0.3542f,24.0723f,-0.2500f,24.1211f,-0.2500f,24.0723f,-0.1458f,24.1211f,-0.1458f,24.0723f,-0.0417f,24.1211f,-0.0417f,24.0723f,0.0625f,24.1211f,0.0625f,
	24.0723f,0.1667f,24.1211f,0.1667f,24.0723f,0.2708f,24.1211f,0.2708f,24.0723f,0.3750f,24.1211f,0.3750f,24.0723f,0.4792f,24.1211f,0.4792f,24.0723f,0.5833f,24.1211f,0.5833f,24.0723f,0.6875f,24.1211f,0.6875f,24.0723f,0.7917f,24.1211f,0.7917f,24.0723f,0.8958f,
	24.1211f,0.8958f,24.0723f,1.0000f,24.1211f,1.0000f,24.0723f,-4.0000f,24.0235f,-4.0000f,24.0235f,-3.8958f,23.9747f,-3.8958f,23.9747f,-3.7917f,23.9258f,-3.7917f,23.9258f,-3.6875f,23.8770f,-3.6875f,23.8770f,-3.5833f,23.8282f,-3.5833f,23.8282f,-3.4792f,23.7793f,-3.4792f,
	23.7793f,-3.3750f,23.7305f,-3.3750f,23.7305f,-3.2708f,23.6817f,-3.2708f,23.6817f,-3.1667f,23.6329f,-3.1667f,23.6329f,-3.0625f,23.5840f,-3.0625f,23.5840f,-2.9583f,23.5352f,-2.9583f,23.5352f,-2.8542f,23.4864f,-2.8542f,23.4864f,-2.7500f,23.4375f,-2.7500f,23.4375f,-2.6458f,
	23.3887f,-2.6458f,23.3887f,-2.5417f,23.3399f,-2.5417f,23.3399f,-2.4375f,23.2911f,-2.4375f,23.2911f,-2.3333f,23.2422f,-2.3333f,23.2422f,-2.2292f,23.1934f,-2.2292f,23.1934f,-2.1250f,23.1446f,-2.1250f,23.1446f,-2.0208f,23.0958f,-2.0208f,23.0958f,-1.9167f,23.0469f,-1.9167f,
	23.0469f,-1.8125f,22.9981f,-1.8125f,22.9981f,-1.7083f,22.9493f,-1.7083f,22.9493f,-1.6042f,22.9004f,-1.6042f,22.9004f,-1.5000f,22.8516f,-1.5000f,22.8516f,-1.3958f,22.8028f,-1.3958f,22.8028f,-1.2917f,22.7540f,-1.2917f,22.7540f,-1.1875f,22.7051f,-1.1875f,22.7051f,-1.0833f,
	22.6563f,-1.0833f,22.6563f,-0.9792f,22.6075f,-0.9792f,22.6075f,-0.8750f,22.5586f,-0.8750f,22.5586f,-0.7708f,22.5098f,-0.7708f,22.5098f,-0.6667f,22.4610f,-0.6667f,22.4610f,-0.5625f,22.4122f,-0.5625f,22.4122f,-0.4583f,22.3633f,-0.4583f,22.3633f,-0.3542f,22.3145f,-0.3542f,
	22.3145f,-0.2500f,22.2657f,-0.2500f,22.2657f,-0.1458f,22.2168f,-0.1458f,22.2168f,-0.0417f,22.1680f,-0.0417f,22.1680f,0.0625f,22.1192f,0.0625f,22.1192f,0.1667f,22.0704f,0.1667f,22.0704f,0.2708f,22.0215f,0.2708f,22.0215f,0.3750f,21.9727f,0.3750f,21.9727f,0.4792f,
	21.9239f,0.4792f,21.9239f,0.5833f,21.8750f,0.5833f,21.8750f,0.6875f,21.8262f,0.6875f,21.8262f,0.7917f,21.7774f,0.7917f,21.7774f,0.8958f,21.7286f,0.8958f,21.7286f,1.0000f,21.6797f,1.0000f,24.0723f,-2.0208f,24.0235f,-1.9167f,24.0723f,-1.9167f,24.0235f,-1.8125f,
	24.0723f,-1.8125f,24.0235f,-1.7083f,24.0723f,-1.7083f,24.0235f,-1.6042f,24.0723f,-1.6042f,24.0235f,-1.5000f,24.0723f,-1.5000f,24.0235f,-1.3958f,24.0723f,-1.3958f,24.0235f,-1.2917f,24.0723f,-1.2917f,24.0235f,-1.1875f,24.0723f,-1.1875f,24.0235f,-1.0833f,24.0723f,-1.0833f,
	24.0235f,-0.9792f,24.0723f,-0.9792f,24.0235f,-0.8750f,24.0723f,-0.8750f,24.0235f,-0.7708f,24.0723f,-0.7708f,24.0235f,-0.6667f,24.0723f,-0.6667f,24.0235f,-0.5625f,24.0723f,-0.5625f,24.0235f,-0.4583f,24.0723f,-0.4583f,24.0235f,-0.3542f,24.0723f,-0.3542f,24.0235f,-0.2500f,
	24.0723f,-0.2500f,24.0235f,-0.1458f,24.0723f,-0.1458f,24.0235f,-0.0417f,24.0723f,-0.0417f,24.0235f,0.0625f,24.0723f,0.0625f,24.0235f,0.1667f,24.0723f,0.1667f,24.0235f,0.2708f,24.0723f,0.2708f,24.0235f,0.3750f,24.0723f,0.3750f,24.0235f,0.4792f,24.0723f,0.4792f,
	24.0235f,0.5833f,24.0723f,0.5833f,24.0235f,0.6875f,24.0723f,0.6875f,24.0235f,0.7917f,24.0723f,0.7917f,24.0235f,0.8958f,24.0723f,0.8958f,24.0235f,1.0000f,24.0723f,1.0000f,24.0235f,-4.0000f,23.9747f,-4.0000f,23.9747f,-3.8958f,23.9258f,-3.8958f,23.9258f,-3.7917f,
	23.8770f,-3.7917f,23.8770f,-3.6875f,23.8282f,-3.6875f,23.8282f,-3.5833f,23.7793f,-3.5833f,23.7793f,-3.4792f,23.7305f,-3.4792f,23.7305f,-3.3750f,23.6817f,-3.3750f,23.6817f,-3.2708f,23.6329f,-3.2708f,23.6329f,-3.1667f,23.5840f,-3.1667f,23.5840f,-3.0625f,23.5352f,-3.0625f,
	23.5352f,-2.9583f,23.4864f,-2.9583f,23.4864f,-2.8542f,23.4375f,-2.8542f,23.4375f,-2.7500f,23.3887f,-2.7500f,23.3887f,-2.6458f,23.3399f,-2.6458f,23.3399f,-2.5417f,23.2911f,-2.5417f,23.2911f,-2.4375f,23.2422f,-2.4375f,23.2422f,-2.3333f,23.1934f,-2.3333f,23.1934f,-2.2292f,
	23.1446f,-2.2292f,23.1446f,-2.1250f,23.0958f,-2.1250f,23.0958f,-2.0208f,23.0469f,-2.0208f,23.0469f,-1.9167f,22.9981f,-1.9167f,22.9981f,-1.8125f,22.9493f,-1.8125f,22.9493f,-1.7083f,22.9004f,-1.7083f,22.9004f,-1.6042f,22.8516f,-1.6042f,22.8516f,-1.5000f,22.8028f,-1.5000f,
	22.8028f,-1.3958f,22.7540f,-1.3958f,22.7540f,-1.2917f,22.7051f,-1.2917f,22.7051f,-1.1875f,22.6563f,-1.1875f,22.6563f,-1.0833f,22.6075f,-1.0833f,22.6075f,-0.9792f,22.5586f,-0.9792f,22.5586f,-0.8750f,22.5098f,-0.8750f,22.5098f,-0.7708f,22.4610f,-0.7708f,22.4610f,-0.6667f,
	22.4122f,-0.6667f,22.4122f,-0.5625f,22.3633f,-0.5625f,22.3633f,-0.4583f,22.3145f,-0.4583f,22.3145f,-0.3542f,22.2657f,-0.3542f,22.2657f,-0.2500f,22.2168f,-0.2500f,22.2168f,-0.1458f,22.1680f,-0.1458f,22.1680f,-0.0417f,22.1192f,-0.0417f,22.1192f,0.0625f,22.0704f,0.0625f,
	22.0704f,0.1667f,22.0215f,0.1667f,22.0215f,0.2708f,21.9727f,0.2708f,21.9727f,0.3750f,21.9239f,0.3750f,21.9239f,0.4792f,21.8750f,0.4792f,21.8750f,0.5833f,21.8262f,0.5833f,21.8262f,0.6875f,21.7774f,0.6875f,21.7774f,0.7917f,21.7286f,0.7917f,21.7286f,0.8958f,
	21.6797f,0.8958f,21.6797f,1.0000f,21.6309f,1.0000f,24.0235f,-1.9167f,23.9747f,-1.8125f,24.0235f,-1.8125f,23.9747f,-1.7083f,24.0235f,-1.7083f,23.9747f,-1.6042f,24.0235f,-1.6042f,23.9747f,-1.5000f,24.0235f,-1.5000f,23.9747f,-1.3958f,24.0235f,-1.3958f,23.9747f,-1.2917f,
	24.0235f,-1.2917f,23.9747f,-1.1875f,24.0235f,-1.1875f,23.9747f,-1.0833f,24.0235f,-1.0833f,23.9747f,-0.9792f,24.0235f,-0.9792f,23.9747f,-0.8750f,24.0235f,-0.8750f,23.9747f,-0.7708f,24.0235f,-0.7708f,23.9747f,-0.6667f,24.0235f,-0.6667f,23.9747f,-0.5625f,24.0235f,-0.5625f,
	23.9747f,-0.4583f,24.0235f,-0.4583f,23.9747f,-0.3542f,24.0235f,-0.3542f,23.9747f,-0.2500f,24.0235f,-0.2500f,23.9747f,-0.1458f,24.0235f,-0.1458f,23.9747f,-0.0417f,24.0235f,-0.0417f,23.9747f,0.0625f,24.0235f,0.0625f,23.9747f,0.1667f,24.0235f,0.1667f,23.9747f,0.2708f,
	24.0235f,0.2708f,23.9747f,0.3750f,24.0235f,0.3750f,23.9747f,0.4792f,24.0235f,0.4792f,23.9747f,0.5833f,24.0235f,0.5833f,23.9747f,0.6875f,24.0235f,0.6875f,23.9747f,0.7917f,24.0235f,0.7917f,23.9747f,0.8958f,24.0235f,0.8958f,23.9747f,1.0000f,24.0235f,1.0000f,
	23.9747f,-4.0000f,23.9258f,-4.0000f,23.9258f,-3.8958f,23.8770f,-3.8958f,23.8770f,-3.7917f,23.8282f,-3.7917f,23.8282f,-3.6875f,23.7793f,-3.6875f,23.7793f,-3.5833f,23.7305f,-3.5833f,23.7305f,-3.4792f,23.6817f,-3.4792f,23.6817f,-3.3750f,23.6329f,-3.3750f,23.6329f,-3.2708f,
	23.5840f,-3.2708f,23.5840f,-3.1667f,23.5352f,-3.1667f,23.5352f,-3.0625f,23.4864f,-3.0625f,23.4864f,-2.9583f,23.4375f,-2.9583f,23.4375f,-2.8542f,23.3887f,-2.8542f,23.3887f,-2.7500f,23.3399f,-2.7500f,23.3399f,-2.6458f,23.2911f,-2.6458f,23.2911f,-2.5417f,23.2422f,-2.5417f,
	23.2422f,-2.4375f,23.1934f,-2.4375f,23.1934f,-2.3333f,23.1446f,-2.3333f,23.1446f,-2.2292f,23.0958f,-2.2292f,23.0958f,-2.1250f,23.0469f,-2.1250f,23.0469f,-2.0208f,22.9981f,-2.0208f,22.9981f,-1.9167f,22.9493f,-1.9167f,22.9493f,-1.8125f,22.9004f,-1.8125f,22.9004f,-1.7083f,
	22.8516f,-1.7083f,22.8516f,-1.6042f,22.8028f,-1.6042f,22.8028f,-1.5000f,22.7540f,-1.5000f,22.7540f,-1.3958f,22.7051f,-1.3958f,22.7051f,-1.2917f,22.6563f,-1.2917f,22.6563f,-1.1875f,22.6075f,-1.1875f,22.6075f,-1.0833f,22.5586f,-1.0833f,22.5586f,-0.9792f,22.5098f,-0.9792f,
	22.5098f,-0.8750f,22.4610f,-0.8750f,22.4610f,-0.7708f,22.4122f,-0.7708f,22.4122f,-0.6667f,22.3633f,-0.6667f,22.3633f,-0.5625f,22.3145f,-0.5625f,22.3145f,-0.4583f,22.2657f,-0.4583f,22.2657f,-0.3542f,22.2168f,-0.3542f,22.2168f,-0.2500f,22.1680f,-0.2500f,22.1680f,-0.1458f,
	22.1192f,-0.1458f,22.1192f,-0.0417f,22.0704f,-0.0417f,22.0704f,0.0625f,22.0215f,0.0625f,22.0215f,0.1667f,21.9727f,0.1667f,21.9727f,0.2708f,21.9239f,0.2708f,21.9239f,0.3750f,21.8750f,0.3750f,21.8750f,0.4792f,21.8262f,0.4792f,21.8262f,0.5833f,21.7774f,0.5833f,
	21.7774f,0.6875f,21.7286f,0.6875f,21.7286f,0.7917f,21.6797f,0.7917f,21.6797f,0.8958f,21.6309f,0.8958f,21.6309f,1.0000f,21.5821f,1.0000f,23.9747f,-1.8125f,23.9258f,-1.7083f,23.9747f,-1.7083f,23.9258f,-1.6042f,23.9747f,-1.6042f,23.9258f,-1.5000f,23.9747f,-1.5000f,
	23.9258f,-1.3958f,23.9747f,-1.3958f,23.9258f,-1.2917f,23.9747f,-1.2917f,23.9258f,-1.1875f,23.9747f,-1.1875f,23.9258f,-1.0833f,23.9747f,-1.0833f,23.9258f,-0.9792f,23.9747f,-0.9792f,23.9258f,-0.8750f,23.9747f,-0.8750f,23.9258f,-0.7708f,23.9747f,-0.7708f,23.9258f,-0.6667f,
	23.9747f,-0.6667f,23.9258f,-0.5625f,23.9747f,-0.5625f,23.9258f,-0.4583f,23.9747f,-0.4583f,23.9258f,-0.3542f,23.9747f,-0.3542f,23.9258f,-0.2500f,23.9747f,-0.2500f,23.9258f,-0.1458f,23.9747f,-0.1458f,23.9258f,-0.0417f,23.9747f,-0.0417f,23.9258f,0.0625f,23.9747f,0.0625f,
	23.9258f,0.1667f,23.9747f,0.1667f,23.9258f,0.2708f,23.9747f,0.2708f,23.9258f,0.3750f,23.9747f,0.3750f,23.9258f,0.4792f,23.9747f,0.4792f,23.9258f,0.5833f,23.9747f,0.5833f,23.9258f,0.6875f,23.9747f,0.6875f,23.9258f,0.7917f,23.9747f,0.7917f,23.9258f,0.8958f,
	23.9747f,0.8958f,23.9258f,1.0000f,23.9747f,1.0000f,23.9258f,-4.0000f,23.8770f,-4.0000f,23.8770f,-3.8958f,23.8282f,-3.8958f,23.8282f,-3.7917f,23.7793f,-3.7917f,23.7793f,-3.6875f,23.7305f,-3.6875f,23.7305f,-3.5833f,23.6817f,-3.5833f,23.6817f,-3.4792f,23.6329f,-3.4792f,
	23.6329f,-3.3750f,23.5840f,-3.3750f,23.5840f,-3.2708f,23.5352f,-3.2708f,23.5352f,-3.1667f,23.4864f,-3.1667f,23.4864f,-3.0625f,23.4375f,-3.0625f,23.4375f,-2.9583f,23.3887f,-2.9583f,23.3887f,-2.8542f,23.3399f,-2.8542f,23.3399f,-2.7500f,23.2911f,-2.7500f,23.2911f,-2.6458f,
	23.2422f,-2.6458f,23.2422f,-2.5417f,23.1934f,-2.5417f,23.1934f,-2.4375f,23.1446f,-2.4375f,23.1446f,-2.3333f,23.0958f,-2.3333f,23.0958f,-2.2292f,23.0469f,-2.2292f,23.0469f,-2.1250f,22.9981f,-2.1250f,22.9981f,-2.0208f,22.9493f,-2.0208f,22.9493f,-1.9167f,22.9004f,-1.9167f,
	22.9004f,-1.8125f,22.8516f,-1.8125f,22.8516f,-1.7083f,22.8028f,-1.7083f,22.8028f,-1.6042f,22.7540f,-1.6042f,22.7540f,-1.5000f,22.7051f,-1.5000f,22.7051f,-1.3958f,22.6563f,-1.3958f,22.6563f,-1.2917f,22.6075f,-1.2917f,22.6075f,-1.1875f,22.5586f,-1.1875f,22.5586f,-1.0833f,
	22.5098f,-1.0833f,22.5098f,-0.9792f,22.4610f,-0.9792f,22.4610f,-0.8750f,22.4122f,-0.8750f,22.4122f,-0.7708f,22.3633f,-0.7708f,22.3633f,-0.6667f,22.3145f,-0.6667f,22.3145f,-0.5625f,22.2657f,-0.5625f,22.2657f,-0.4583f,22.2168f,-0.4583f,22.2168f,-0.3542f,22.1680f,-0.3542f,
	22.1680f,-0.2500f,22.1192f,-0.2500f,22.1192f,-0.1458f,22.0704f,-0.1458f,22.0704f,-0.0417f,22.0215f,-0.0417f,22.0215f,0.0625f,21.9727f,0.0625f,21.9727f,0.1667f,21.9239f,0.1667f,21.9239f,0.2708f,21.8750f,0.2708f,21.8750f,0.3750f,21.8262f,0.3750f,21.8262f,0.4792f,
	21.7774f,0.4792f,21.7774f,0.5833f,21.7286f,0.5833f,21.7286f,0.6875f,21.6797f,0.6875f,21.6797f,0.7917f,21.6309f,0.7917f,21.6309f,0.8958f,21.5821f,0.8958f,21.5821f,1.0000f,21.5332f,1.0000f,23.9258f,-1.7083f,23.8770f,-1.6042f,23.9258f,-1.6042f,23.8770f,-1.5000f,
	23.9258f,-1.5000f,23.8770f,-1.3958f,23.9258f,-1.3958f,23.8770f,-1.2917f,23.9258f,-1.2917f,23.8770f,-1.1875f,23.9258f,-1.1875f,23.8770f,-1.0833f,23.9258f,-1.0833f,23.8770f,-0.9792f,23.9258f,-0.9792f,23.8770f,-0.8750f,23.9258f,-0.8750f,23.8770f,-0.7708f,23.9258f,-0.7708f,
	23.8770f,-0.6667f,23.9258f,-0.6667f,23.8770f,-0.5625f,23.9258f,-0.5625f,23.8770f,-0.4583f,23.9258f,-0.4583f,23.8770f,-0.3542f,23.9258f,-0.3542f,23.8770f,-0.2500f,23.9258f,-0.2500f,23.8770f,-0.1458f,23.9258f,-0.1458f,23.8770f,-0.0417f,23.9258f,-0.0417f,23.8770f,0.0625f,
	23.9258f,0.0625f,23.8770f,0.1667f,23.9258f,0.1667f,23.8770f,0.2708f,23.9258f,0.2708f,23.8770f,0.3750f,23.9258f,0.3750f,23.8770f,0.4792f,23.9258f,0.4792f,23.8770f,0.5833f,23.9258f,0.5833f,23.8770f,0.6875f,23.9258f,0.6875f,23.8770f,0.7917f,23.9258f,0.7917f,
	23.8770f,0.8958f,23.9258f,0.8958f,23.8770f,1.0000f,23.9258f,1.0000f,23.8770f,-4.0000f,23.8282f,-4.0000f,23.8282f,-3.8958f,23.7793f,-3.8958f,23.7793f,-3.7917f,23.7305f,-3.7917f,23.7305f,-3.6875f,23.6817f,-3.6875f,23.6817f,-3.5833f,23.6329f,-3.5833f,23.6329f,-3.4792f,
	23.5840f,-3.4792f,23.5840f,-3.3750f,23.5352f,-3.3750f,23.5352f,-3.2708f,23.4864f,-3.2708f,23.4864f,-3.1667f,23.4375f,-3.1667f,23.4375f,-3.0625f,23.3887f,-3.0625f,23.3887f,-2.9583f,23.3399f,-2.9583f,23.3399f,-2.8542f,23.2911f,-2.8542f,23.2911f,-2.7500f,23.2422f,-2.7500f,
	23.2422f,-2.6458f,23.1934f,-2.6458f,23.1934f,-2.5417f,23.1446f,-2.5417f,23.1446f,-2.4375f,23.0958f,-2.4375f,23.0958f,-2.3333f,23.0469f,-2.3333f,23.0469f,-2.2292f,22.9981f,-2.2292f,22.9981f,-2.1250f,22.9493f,-2.1250f,22.9493f,-2.0208f,22.9004f,-2.0208f,22.9004f,-1.9167f,
	22.8516f,-1.9167f,22.8516f,-1.8125f,22.8028f,-1.8125f,22.8028f,-1.7083f,22.7540f,-1.7083f,22.7540f,-1.6042f,22.7051f,-1.6042f,22.7051f,-1.5000f,22.6563f,-1.5000f,22.6563f,-1.3958f,22.6075f,-1.3958f,22.6075f,-1.2917f,22.5586f,-1.2917f,22.5586f,-1.1875f,22.5098f,-1.1875f,
	22.5098f,-1.0833f,22.4610f,-1.0833f,22.4610f,-0.9792f,22.4122f,-0.9792f,22.4122f,-0.8750f,22.3633f,-0.8750f,22.3633f,-0.7708f,22.3145f,-0.7708f,22.3145f,-0.6667f,22.2657f,-0.6667f,22.2657f,-0.5625f,22.2168f,-0.5625f,22.2168f,-0.4583f,22.1680f,-0.4583f,22.1680f,-0.3542f,
	22.1192f,-0.3542f,22.1192f,-0.2500f,22.0704f,-0.2500f,22.0704f,-0.1458f,22.0215f,-0.1458f,22.0215f,-0.0417f,21.9727f,-0.0417f,21.9727f,0.0625f,21.9239f,0.0625f,21.9239f,0.1667f,21.8750f,0.1667f,21.8750f,0.2708f,21.8262f,0.2708f,21.8262f,0.3750f,21.7774f,0.3750f,
	21.7774f,0.4792f,21.7286f,0.4792f,21.7286f,0.5833f,21.6797f,0.5833f,21.6797f,0.6875f,21.6309f,0.6875f,21.6309f,0.7917f,21.5821f,0.7917f,21.5821f,0.8958f,21.5332f,0.8958f,21.5332f,1.0000f,21.4844f,1.0000f,23.8770f,-1.6042f,23.8282f,-1.5000f,23.8770f,-1.5000f,
	23.8282f,-1.3958f,23.8770f,-1.3958f,23.8282f,-1.2917f,23.8770f,-1.2917f,23.8282f,-1.1875f,23.8770f,-1.1875f,23.8282f,-1.0833f,23.8770f,-1.0833f,23.8282f,-0.9792f,23.8770f,-0.9792f,23.8282f,-0.8750f,23.8770f,-0.8750f,23.8282f,-0.7708f,23.8770f,-0.7708f,23.8282f,-0.6667f,
	23.8770f,-0.6667f,23.8282f,-0.5625f,23.8770f,-0.5625f,23.8282f,-0.4583f,23.8770f,-0.4583f,23.8282f,-0.3542f,23.8770f,-0.3542f,23.8282f,-0.2500f,23.8770f,-0.2500f,23.8282f,-0.1458f,23.8770f,-0.1458f,23.8282f,-0.0417f,23.8770f,-0.0417f,23.8282f,0.0625f,23.8770f,0.0625f,
	23.8282f,0.1667f,23.8770f,0.1667f,23.8282f,0.2708f,23.8770f,0.2708f,23.8282f,0.3750f,23.8770f,0.3750f,23.8282f,0.4792f,23.8770f,0.4792f,23.8282f,0.5833f,23.8770f,0.5833f,23.8282f,0.6875f,23.8770f,0.6875f,23.8282f,0.7917f,23.8770f,0.7917f,23.8282f,0.8958f,
	23.8770f,0.8958f,23.8282f,1.0000f,23.8770f,1.0000f,23.8282f,-4.0000f,23.7793f,-4.0000f,23.7793f,-3.8958f,23.7305f,-3.8958f,23.7305f,-3.7917f,23.6817f,-3.7917f,23.6817f,-3.6875f,23.6329f,-3.6875f,23.6329f,-3.5833f,23.5840f,-3.5833f,23.5840f,-3.4792f,23.5352f,-3.4792f,
	23.5352f,-3.3750f,23.4864f,-3.3750f,23.4864f,-3.2708f,23.4375f,-3.2708f,23.4375f,-3.1667f,23.3887f,-3.1667f,23.3887f,-3.0625f,23.3399f,-3.0625f,23.3399f,-2.9583f,23.2911f,-2.9583f,23.2911f,-2.8542f,23.2422f,-2.8542f,23.2422f,-2.7500f,23.1934f,-2.7500f,23.1934f,-2.6458f,
	23.1446f,-2.6458f,23.1446f,-2.5417f,23.0958f,-2.5417f,23.0958f,-2.4375f,23.0469f,-2.4375f,23.0469f,-2.3333f,22.9981f,-2.3333f,22.9981f,-2.2292f,22.9493f,-2.2292f,22.9493f,-2.1250f,22.9004f,-2.1250f,22.9004f,-2.0208f,22.8516f,-2.0208f,22.8516f,-1.9167f,22.8028f,-1.9167f,
	22.8028f,-1.8125f,22.7540f,-1.8125f,22.7540f,-1.7083f,22.7051f,-1.7083f,22.7051f,-1.6042f,22.6563f,-1.6042f,22.6563f,-1.5000f,22.6075f,-1.5000f,22.6075f,-1.3958f,22.5586f,-1.3958f,22.5586f,-1.2917f,22.5098f,-1.2917f,22.5098f,-1.1875f,22.4610f,-1.1875f,22.4610f,-1.0833f,
	22.4122f,-1.0833f,22.4122f,-0.9792f,22.3633f,-0.9792f,22.3633f,-0.8750f,22.3145f,-0.8750f,22.3145f,-0.7708f,22.2657f,-0.7708f,22.2657f,-0.6667f,22.2168f,-0.6667f,22.2168f,-0.5625f,22.1680f,-0.5625f,22.1680f,-0.4583f,22.1192f,-0.4583f,22.1192f,-0.3542f,22.0704f,-0.3542f,
	22.0704f,-0.2500f,22.0215f,-0.2500f,22.0215f,-0.1458f,21.9727f,-0.1458f,21.9727f,-0.0417f,21.9239f,-0.0417f,21.9239f,0.0625f,21.8750f,0.0625f,21.8750f,0.1667f,21.8262f,0.1667f,21.8262f,0.2708f,21.7774f,0.2708f,21.7774f,0.3750f,21.7286f,0.3750f,21.7286f,0.4792f,
	21.6797f,0.4792f,21.6797f,0.5833f,21.6309f,0.5833f,21.6309f,0.6875f,21.5821f,0.6875f,21.5821f,0.7917f,21.5332f,0.7917f,21.5332f,0.8958f,21.4844f,0.8958f,21.4844f,1.0000f,21.4356f,1.0000f,23.8282f,-1.5000f,23.7793f,-1.3958f,23.8282f,-1.3958f,23.7793f,-1.2917f,
	23.8282f,-1.2917f,23.7793f,-1.1875f,23.8282f,-1.1875f,23.7793f,-1.0833f,23.8282f,-1.0833f,23.7793f,-0.9792f,23.8282f,-0.9792f,23.7793f,-0.8750f,23.8282f,-0.8750f,23.7793f,-0.7708f,23.8282f,-0.7708f,23.7793f,-0.6667f,23.8282f,-0.6667f,23.7793f,-0.5625f,23.8282f,-0.5625f,
	23.7793f,-0.4583f,23.8282f,-0.4583f,23.7793f,-0.3542f,23.8282f,-0.3542f,23.7793f,-0.2500f,23.8282f,-0.2500f,23.7793f,-0.1458f,23.8282f,-0.1458f,23.7793f,-0.0417f,23.8282f,-0.0417f,23.7793f,0.0625f,23.8282f,0.0625f,23.7793f,0.1667f,23.8282f,0.1667f,23.7793f,0.2708f,
	23.8282f,0.2708f,23.7793f,0.3750f,23.8282f,0.3750f,23.7793f,0.4792f,23.8282f,0.4792f,23.7793f,0.5833f,23.8282f,0.5833f,23.7793f,0.6875f,23.8282f,0.6875f,23.7793f,0.7917f,23.8282f,0.7917f,23.7793f,0.8958f,23.8282f,0.8958f,23.7793f,1.0000f,23.8282f,1.0000f,
	23.7793f,-4.0000f,23.7305f,-4.0000f,23.7305f,-3.8958f,23.6817f,-3.8958f,23.6817f,-3.7917f,23.6329f,-3.7917f,23.6329f,-3.6875f,23.5840f,-3.6875f,23.5840f,-3.5833f,23.5352f,-3.5833f,23.5352f,-3.4792f,23.4864f,-3.4792f,23.4864f,-3.3750f,23.4375f,-3.3750f,23.4375f,-3.2708f,
	23.3887f,-3.2708f,23.3887f,-3.1667f,23.3399f,-3.1667f,23.3399f,-3.0625f,23.2911f,-3.0625f,23.2911f,-2.9583f,23.2422f,-2.9583f,23.2422f,-2.8542f,23.1934f,-2.8542f,23.1934f,-2.7500f,23.1446f,-2.7500f,23.1446f,-2.6458f,23.0958f,-2.6458f,23.0958f,-2.5417f,23.0469f,-2.5417f,
	23.0469f,-2.4375f,22.9981f,-2.4375f,22.9981f,-2.3333f,22.9493f,-2.3333f,22.9493f,-2.2292f,22.9004f,-2.2292f,22.9004f,-2.1250f,22.8516f,-2.1250f,22.8516f,-2.0208f,22.8028f,-2.0208f,22.8028f,-1.9167f,22.7540f,-1.9167f,22.7540f,-1.8125f,22.7051f,-1.8125f,22.7051f,-1.7083f,
	22.6563f,-1.7083f,22.6563f,-1.6042f,22.6075f,-1.6042f,22.6075f,-1.5000f,22.5586f,-1.5000f,22.5586f,-1.3958f,22.5098f,-1.3958f,22.5098f,-1.2917f,22.4610f,-1.2917f,22.4610f,-1.1875f,22.4122f,-1.1875f,22.4122f,-1.0833f,22.3633f,-1.0833f,22.3633f,-0.9792f,22.3145f,-0.9792f,
	22.3145f,-0.8750f,22.2657f,-0.8750f,22.2657f,-0.7708f,22.2168f,-0.7708f,22.2168f,-0.6667f,22.1680f,-0.6667f,22.1680f,-0.5625f,22.1192f,-0.5625f,22.1192f,-0.4583f,22.0704f,-0.4583f,22.0704f,-0.3542f,22.0215f,-0.3542f,22.0215f,-0.2500f,21.9727f,-0.2500f,21.9727f,-0.1458f,
	21.9239f,-0.1458f,21.9239f,-0.0417f,21.8750f,-0.0417f,21.8750f,0.0625f,21.8262f,0.0625f,21.8262f,0.1667f,21.7774f,0.1667f,21.7774f,0.2708f,21.7286f,0.2708f,21.7286f,0.3750f,21.6797f,0.3750f,21.6797f,0.4792f,21.6309f,0.4792f,21.6309f,0.5833f,21.5821f,0.5833f,
	21.5821f,0.6875f,21.5332f,0.6875f,21.5332f,0.7917f,21.4844f,0.7917f,21.4844f,0.8958f,21.4356f,0.8958f,21.4356f,1.0000f,21.3868f,1.0000f,23.7793f,-1.3958f,23.7305f,-1.2917f,23.7793f,-1.2917f,23.7305f,-1.1875f,23.7793f,-1.1875f,23.7305f,-1.0833f,23.7793f,-1.0833f,
	23.7305f,-0.9792f,23.7793f,-0.9792f,23.7305f,-0.8750f,23.7793f,-0.8750f,23.7305f,-0.7708f,23.7793f,-0.7708f,23.7305f,-0.6667f,23.7793f,-0.6667f,23.7305f,-0.5625f,23.7793f,-0.5625f,23.7305f,-0.4583f,23.7793f,-0.4583f,23.7305f,-0.3542f,23.7793f,-0.3542f,23.7305f,-0.2500f,
	23.7793f,-0.2500f,23.7305f,-0.1458f,23.7793f,-0.1458f,23.7305f,-0.0417f,23.7793f,-0.0417f,23.7305f,0.0625f,23.7793f,0.0625f,23.7305f,0.1667f,23.7793f,0.1667f,23.7305f,0.2708f,23.7793f,0.2708f,23.7305f,0.3750f,23.7793f,0.3750f,23.7305f,0.4792f,23.7793f,0.4792f,
	23.7305f,0.5833f,23.7793f,0.5833f,23.7305f,0.6875f,23.7793f,0.6875f,23.7305f,0.7917f,23.7793f,0.7917f,23.7305f,0.8958f,23.7793f,0.8958f,23.7305f,1.0000f,23.7793f,1.0000f,23.7305f,-4.0000f,23.6817f,-4.0000f,23.6817f,-3.8958f,23.6329f,-3.8958f,23.6329f,-3.7917f,
	23.5840f,-3.7917f,23.5840f,-3.6875f,23.5352f,-3.6875f,23.5352f,-3.5833f,23.4864f,-3.5833f,23.4864f,-3.4792f,23.4375f,-3.4792f,23.4375f,-3.3750f,23.3887f,-3.3750f,23.3887f,-3.2708f,23.3399f,-3.2708f,23.3399f,-3.1667f,23.2911f,-3.1667f,23.2911f,-3.0625f,23.2422f,-3.0625f,
	23.2422f,-2.9583f,23.1934f,-2.9583f,23.1934f,-2.8542f,23.1446f,-2.8542f,23.1446f,-2.7500f,23.0958f,-2.7500f,23.0958f,-2.6458f,23.0469f,-2.6458f,23.0469f,-2.5417f,22.9981f,-2.5417f,22.9981f,-2.4375f,22.9493f,-2.4375f,22.9493f,-2.3333f,22.9004f,-2.3333f,22.9004f,-2.2292f,
	22.8516f,-2.2292f,22.8516f,-2.1250f,22.8028f,-2.1250f,22.8028f,-2.0208f,22.7540f,-2.0208f,22.7540f,-1.9167f,22.7051f,-1.9167f,22.7051f,-1.8125f,22.6563f,-1.8125f,22.6563f,-1.7083f,22.6075f,-1.7083f,22.6075f,-1.6042f,22.5586f,-1.6042f,22.5586f,-1.5000f,22.5098f,-1.5000f,
	22.5098f,-1.3958f,22.4610f,-1.3958f,22.4610f,-1.2917f,22.4122f,-1.2917f,22.4122f,-1.1875f,22.3633f,-1.1875f,22.3633f,-1.0833f,22.3145f,-1.0833f,22.3145f,-0.9792f,22.2657f,-0.9792f,22.2657f,-0.8750f,22.2168f,-0.8750f,22.2168f,-0.7708f,22.1680f,-0.7708f,22.1680f,-0.6667f,
	22.1192f,-0.6667f,22.1192f,-0.5625f,22.0704f,-0.5625f,22.0704f,-0.4583f,22.0215f,-0.4583f,22.0215f,-0.3542f,21.9727f,-0.3542f,21.9727f,-0.2500f,21.9239f,-0.2500f,21.9239f,-0.1458f,21.8750f,-0.1458f,21.8750f,-0.0417f,21.8262f,-0.0417f,21.8262f,0.0625f,21.7774f,0.0625f,
	21.7774f,0.1667f,21.7286f,0.1667f,21.7286f,0.2708f,21.6797f,0.2708f,21.6797f,0.3750f,21.6309f,0.3750f,21.6309f,0.4792f,21.5821f,0.4792f,21.5821f,0.5833f,21.5332f,0.5833f,21.5332f,0.6875f,21.4844f,0.6875f,21.4844f,0.7917f,21.4356f,0.7917f,21.4356f,0.8958f,
	21.3868f,0.8958f,21.3868f,1.0000f,21.3379f,1.0000f,23.7305f,-1.2917f,23.6817f,-1.1875f,23.7305f,-1.1875f,23.6817f,-1.0833f,23.7305f,-1.0833f,23.6817f,-0.9792f,23.7305f,-0.9792f,23.6817f,-0.8750f,23.7305f,-0.8750f,23.6817f,-0.7708f,23.7305f,-0.7708f,23.6817f,-0.6667f,
	23.7305f,-0.6667f,23.6817f,-0.5625f,23.7305f,-0.5625f,23.6817f,-0.4583f,23.7305f,-0.4583f,23.6817f,-0.3542f,23.7305f,-0.3542f,23.6817f,-0.2500f,23.7305f,-0.2500f,23.6817f,-0.1458f,23.7305f,-0.1458f,23.6817f,-0.0417f,23.7305f,-0.0417f,23.6817f,0.0625f,23.7305f,0.0625f,
	23.6817f,0.1667f,23.7305f,0.1667f,23.6817f,0.2708f,23.7305f,0.2708f,23.6817f,0.3750f,23.7305f,0.3750f,23.6817f,0.4792f,23.7305f,0.4792f,23.6817f,0.5833f,23.7305f,0.5833f,23.6817f,0.6875f,23.7305f,0.6875f,23.6817f,0.7917f,23.7305f,0.7917f,23.6817f,0.8958f,
	23.7305f,0.8958f,23.6817f,1.0000f,23.7305f,1.0000f,23.6817f,-4.0000f,23.6329f,-4.0000f,23.6329f,-3.8958f,23.5840f,-3.8958f,23.5840f,-3.7917f,23.5352f,-3.7917f,23.5352f,-3.6875f,23.4864f,-3.6875f,23.4864f,-3.5833f,23.4375f,-3.5833f,23.4375f,-3.4792f,23.3887f,-3.4792f,
	23.3887f,-3.3750f,23.3399f,-3.3750f,23.3399f,-3.2708f,23.2911f,-3.2708f,23.2911f,-3.1667f,23.2422f,-3.1667f,23.2422f,-3.0625f,23.1934f,-3.0625f,23.1934f,-2.9583f,23.1446f,-2.9583f,23.1446f,-2.8542f,23.0958f,-2.8542f,23.0958f,-2.7500f,23.0469f,-2.7500f,23.0469f,-2.6458f,
	22.9981f,-2.6458f,22.9981f,-2.5417f,22.9493f,-2.5417f,22.9493f,-2.4375f,22.9004f,-2.4375f,22.9004f,-2.3333f,22.8516f,-2.3333f,22.8516f,-2.2292f,22.8028f,-2.2292f,22.8028f,-2.1250f,22.7540f,-2.1250f,22.7540f,-2.0208f,22.7051f,-2.0208f,22.7051f,-1.9167f,22.6563f,-1.9167f,
	22.6563f,-1.8125f,22.6075f,-1.8125f,22.6075f,-1.7083f,22.5586f,-1.7083f,22.5586f,-1.6042f,22.5098f,-1.6042f,22.5098f,-1.5000f,22.4610f,-1.5000f,22.4610f,-1.3958f,22.4122f,-1.3958f,22.4122f,-1.2917f,22.3633f,-1.2917f,22.3633f,-1.1875f,22.3145f,-1.1875f,22.3145f,-1.0833f,
	22.2657f,-1.0833f,22.2657f,-0.9792f,22.2168f,-0.9792f,22.2168f,-0.8750f,22.1680f,-0.8750f,22.1680f,-0.7708f,22.1192f,-0.7708f,22.1192f,-0.6667f,22.0704f,-0.6667f,22.0704f,-0.5625f,22.0215f,-0.5625f,22.0215f,-0.4583f,21.9727f,-0.4583f,21.9727f,-0.3542f,21.9239f,-0.3542f,
	21.9239f,-0.2500f,21.8750f,-0.2500f,21.8750f,-0.1458f,21.8262f,-0.1458f,21.8262f,-0.0417f,21.7774f,-0.0417f,21.7774f,0.0625f,21.7286f,0.0625f,21.7286f,0.1667f,21.6797f,0.1667f,21.6797f,0.2708f,21.6309f,0.2708f,21.6309f,0.3750f,21.5821f,0.3750f,21.5821f,0.4792f,
	21.5332f,0.4792f,21.5332f,0.5833f,21.4844f,0.5833f,21.4844f,0.6875f,21.4356f,0.6875f,21.4356f,0.7917f,21.3868f,0.7917f,21.3868f,0.8958f,21.3379f,0.8958f,21.3379f,1.0000f,21.2891f,1.0000f,23.6817f,-1.1875f,23.6329f,-1.0833f,23.6817f,-1.0833f,23.6329f,-0.9792f,
	23.6817f,-0.9792f,23.6329f,-0.8750f,23.6817f,-0.8750f,23.6329f,-0.7708f,23.6817f,-0.7708f,23.6329f,-0.6667f,23.6817f,-0.6667f,23.6329f,-0.5625f,23.6817f,-0.5625f,23.6329f,-0.4583f,23.6817f,-0.4583f,23.6329f,-0.3542f,23.6817f,-0.3542f,23.6329f,-0.2500f,23.6817f,-0.2500f,
	23.6329f,-0.1458f,23.6817f,-0.1458f,23.6329f,-0.0417f,23.6817f,-0.0417f,23.6329f,0.0625f,23.6817f,0.0625f,23.6329f,0.1667f,23.6817f,0.1667f,23.6329f,0.2708f,23.6817f,0.2708f,23.6329f,0.3750f,23.6817f,0.3750f,23.6329f,0.4792f,23.6817f,0.4792f,23.6329f,0.5833f,
	23.6817f,0.5833f,23.6329f,0.6875f,23.6817f,0.6875f,23.6329f,0.7917f,23.6817f,0.7917f,23.6329f,0.8958f,23.6817f,0.8958f,23.6329f,1.0000f,23.6817f,1.0000f,23.6329f,-4.0000f,23.5840f,-4.0000f,23.5840f,-3.8958f,23.5352f,-3.8958f,23.5352f,-3.7917f,23.4864f,-3.7917f,
	23.4864f,-3.6875f,23.4375f,-3.6875f,23.4375f,-3.5833f,23.3887f,-3.5833f,23.3887f,-3.4792f,23.3399f,-3.4792f,23.3399f,-3.3750f,23.2911f,-3.3750f,23.2911f,-3.2708f,23.2422f,-3.2708f,23.2422f,-3.1667f,23.1934f,-3.1667f,23.1934f,-3.0625f,23.1446f,-3.0625f,23.1446f,-2.9583f,
	23.0958f,-2.9583f,23.0958f,-2.8542f,23.0469f,-2.8542f,23.0469f,-2.7500f,22.9981f,-2.7500f,22.9981f,-2.6458f,22.9493f,-2.6458f,22.9493f,-2.5417f,22.9004f,-2.5417f,22.9004f,-2.4375f,22.8516f,-2.4375f,22.8516f,-2.3333f,22.8028f,-2.3333f,22.8028f,-2.2292f,22.7540f,-2.2292f,
	22.7540f,-2.1250f,22.7051f,-2.1250f,22.7051f,-2.0208f,22.6563f,-2.0208f,22.6563f,-1.9167f,22.6075f,-1.9167f,22.6075f,-1.8125f,22.5586f,-1.8125f,22.5586f,-1.7083f,22.5098f,-1.7083f,22.5098f,-1.6042f,22.4610f,-1.6042f,22.4610f,-1.5000f,22.4122f,-1.5000f,22.4122f,-1.3958f,
	22.3633f,-1.3958f,22.3633f,-1.2917f,22.3145f,-1.2917f,22.3145f,-1.1875f,22.2657f,-1.1875f,22.2657f,-1.0833f,22.2168f,-1.0833f,22.2168f,-0.9792f,22.1680f,-0.9792f,22.1680f,-0.8750f,22.1192f,-0.8750f,22.1192f,-0.7708f,22.0704f,-0.7708f,22.0704f,-0.6667f,22.0215f,-0.6667f,
	22.0215f,-0.5625f,21.9727f,-0.5625f,21.9727f,-0.4583f,21.9239f,-0.4583f,21.9239f,-0.3542f,21.8750f,-0.3542f,21.8750f,-0.2500f,21.8262f,-0.2500f,21.8262f,-0.1458f,21.7774f,-0.1458f,21.7774f,-0.0417f,21.7286f,-0.0417f,21.7286f,0.0625f,21.6797f,0.0625f,21.6797f,0.1667f,
	21.6309f,0.1667f,21.6309f,0.2708f,21.5821f,0.2708f,21.5821f,0.3750f,21.5332f,0.3750f,21.5332f,0.4792f,21.4844f,0.4792f,21.4844f,0.5833f,21.4356f,0.5833f,21.4356f,0.6875f,21.3868f,0.6875f,21.3868f,0.7917f,21.3379f,0.7917f,21.3379f,0.8958f,21.2891f,0.8958f,
	21.2891f,1.0000f,21.2403f,1.0000f,23.6329f,-1.0833f,23.5840f,-0.9792f,23.6329f,-0.9792f,23.5840f,-0.8750f,23.6329f,-0.8750f,23.5840f,-0.7708f,23.6329f,-0.7708f,23.5840f,-0.6667f,23.6329f,-0.6667f,23.5840f,-0.5625f,23.6329f,-0.5625f,23.5840f,-0.4583f,23.6329f,-0.4583f,
	23.5840f,-0.3542f,23.6329f,-0.3542f,23.5840f,-0.2500f,23.6329f,-0.2500f,23.5840f,-0.1458f,23.6329f,-0.1458f,23.5840f,-0.0417f,23.6329f,-0.0417f,23.5840f,0.0625f,23.6329f,0.0625f,23.5840f,0.1667f,23.6329f,0.1667f,23.5840f,0.2708f,23.6329f,0.2708f,23.5840f,0.3750f,
	23.6329f,0.3750f,23.5840f,0.4792f,23.6329f,0.4792f,23.5840f,0.5833f,23.6329f,0.5833f,23.5840f,0.6875f,23.6329f,0.6875f,23.5840f,0.7917f,23.6329f,0.7917f,23.5840f,0.8958f,23.6329f,0.8958f,23.5840f,1.0000f,23.6329f,1.0000f,23.5840f,-4.0000f,23.5352f,-4.0000f,
	23.5352f,-3.8958f,23.4864f,-3.8958f,23.4864f,-3.7917f,23.4375f,-3.7917f,23.4375f,-3.6875f,23.3887f,-3.6875f,23.3887f,-3.5833f,23.3399f,-3.5833f,23.3399f,-3.4792f,23.2911f,-3.4792f,23.2911f,-3.3750f,23.2422f,-3.3750f,23.2422f,-3.2708f,23.1934f,-3.2708f,23.1934f,-3.1667f,
	23.1446f,-3.1667f,23.1446f,-3.0625f,23.0958f,-3.0625f,23.0958f,-2.9583f,23.0469f,-2.9583f,23.0469f,-2.8542f,22.9981f,-2.8542f,22.9981f,-2.7500f,22.9493f,-2.7500f,22.9493f,-2.6458f,22.9004f,-2.6458f,22.9004f,-2.5417f,22.8516f,-2.5417f,22.8516f,-2.4375f,22.8028f,-2.4375f,
	22.8028f,-2.3333f,22.7540f,-2.3333f,22.7540f,-2.2292f,22.7051f,-2.2292f,22.7051f,-2.1250f,22.6563f,-2.1250f,22.6563f,-2.0208f,22.6075f,-2.0208f,22.6075f,-1.9167f,22.5586f,-1.9167f,22.5586f,-1.8125f,22.5098f,-1.8125f,22.5098f,-1.7083f,22.4610f,-1.7083f,22.4610f,-1.6042f,
	22.4122f,-1.6042f,22.4122f,-1.5000f,22.3633f,-1.5000f,22.3633f,-1.3958f,22.3145f,-1.3958f,22.3145f,-1.2917f,22.2657f,-1.2917f,22.2657f,-1.1875f,22.2168f,-1.1875f,22.2168f,-1.0833f,22.1680f,-1.0833f,22.1680f,-0.9792f,22.1192f,-0.9792f,22.1192f,-0.8750f,22.0704f,-0.8750f,
	22.0704f,-0.7708f,22.0215f,-0.7708f,22.0215f,-0.6667f,21.9727f,-0.6667f,21.9727f,-0.5625f,21.9239f,-0.5625f,21.9239f,-0.4583f,21.8750f,-0.4583f,21.8750f,-0.3542f,21.8262f,-0.3542f,21.8262f,-0.2500f,21.7774f,-0.2500f,21.7774f,-0.1458f,21.7286f,-0.1458f,21.7286f,-0.0417f,
	21.6797f,-0.0417f,21.6797f,0.0625f,21.6309f,0.0625f,21.6309f,0.1667f,21.5821f,0.1667f,21.5821f,0.2708f,21.5332f,0.2708f,21.5332f,0.3750f,21.4844f,0.3750f,21.4844f,0.4792f,21.4356f,0.4792f,21.4356f,0.5833f,21.3868f,0.5833f,21.3868f,0.6875f,21.3379f,0.6875f,
	21.3379f,0.7917f,21.2891f,0.7917f,21.2891f,0.8958f,21.2403f,0.8958f,21.2403f,1.0000f,21.1915f,1.0000f,23.5840f,-0.9792f,23.5352f,-0.8750f,23.5840f,-0.8750f,23.5352f,-0.7708f,23.5840f,-0.7708f,23.5352f,-0.6667f,23.5840f,-0.6667f,23.5352f,-0.5625f,23.5840f,-0.5625f,
	23.5352f,-0.4583f,23.5840f,-0.4583f,23.5352f,-0.3542f,23.5840f,-0.3542f,23.5352f,-0.2500f,23.5840f,-0.2500f,23.5352f,-0.1458f,23.5840f,-0.1458f,23.5352f,-0.0417f,23.5840f,-0.0417f,23.5352f,0.0625f,23.5840f,0.0625f,23.5352f,0.1667f,23.5840f,0.1667f,23.5352f,0.2708f,
	23.5840f,0.2708f,23.5352f,0.3750f,23.5840f,0.3750f,23.5352f,0.4792f,23.5840f,0.4792f,23.5352f,0.5833f,23.5840f,0.5833f,23.5352f,0.6875f,23.5840f,0.6875f,23.5352f,0.7917f,23.5840f,0.7917f,23.5352f,0.8958f,23.5840f,0.8958f,23.5352f,1.0000f,23.5840f,1.0000f,
	23.5352f,-4.0000f,23.4864f,-4.0000f,23.4864f,-3.8958f,23.4375f,-3.8958f,23.4375f,-3.7917f,23.3887f,-3.7917f,23.3887f,-3.6875f,23.3399f,-3.6875f,23.3399f,-3.5833f,23.2911f,-3.5833f,23.2911f,-3.4792f,23.2422f,-3.4792f,23.2422f,-3.3750f,23.1934f,-3.3750f,23.1934f,-3.2708f,
	23.1446f,-3.2708f,23.1446f,-3.1667f,23.0958f,-3.1667f,23.0958f,-3.0625f,23.0469f,-3.0625f,23.0469f,-2.9583f,22.9981f,-2.9583f,22.9981f,-2.8542f,22.9493f,-2.8542f,22.9493f,-2.7500f,22.9004f,-2.7500f,22.9004f,-2.6458f,22.8516f,-2.6458f,22.8516f,-2.5417f,22.8028f,-2.5417f,
	22.8028f,-2.4375f,22.7540f,-2.4375f,22.7540f,-2.3333f,22.7051f,-2.3333f,22.7051f,-2.2292f,22.6563f,-2.2292f,22.6563f,-2.1250f,22.6075f,-2.1250f,22.6075f,-2.0208f,22.5586f,-2.0208f,22.5586f,-1.9167f,22.5098f,-1.9167f,22.5098f,-1.8125f,22.4610f,-1.8125f,22.4610f,-1.7083f,
	22.4122f,-1.7083f,22.4122f,-1.6042f,22.3633f,-1.6042f,22.3633f,-1.5000f,22.3145f,-1.5000f,22.3145f,-1.3958f,22.2657f,-1.3958f,22.2657f,-1.2917f,22.2168f,-1.2917f,22.2168f,-1.1875f,22.1680f,-1.1875f,22.1680f,-1.0833f,22.1192f,-1.0833f,22.1192f,-0.9792f,22.0704f,-0.9792f,
	22.0704f,-0.8750f,22.0215f,-0.8750f,22.0215f,-0.7708f,21.9727f,-0.7708f,21.9727f,-0.6667f,21.9239f,-0.6667f,21.9239f,-0.5625f,21.8750f,-0.5625f,21.8750f,-0.4583f,21.8262f,-0.4583f,21.8262f,-0.3542f,21.7774f,-0.3542f,21.7774f,-0.2500f,21.7286f,-0.2500f,21.7286f,-0.1458f,
	21.6797f,-0.1458f,21.6797f,-0.0417f,21.6309f,-0.0417f,21.6309f,0.0625f,21.5821f,0.0625f,21.5821f,0.1667f,21.5332f,0.1667f,21.5332f,0.2708f,21.4844f,0.2708f,21.4844f,0.3750f,21.4356f,0.3750f,21.4356f,0.4792f,21.3868f,0.4792f,21.3868f,0.5833f,21.3379f,0.5833f,
	21.3379f,0.6875f,21.2891f,0.6875f,21.2891f,0.7917f,21.2403f,0.7917f,21.2403f,0.8958f,21.1915f,0.8958f,21.1915f,1.0000f,21.1426f,1.0000f,23.5352f,-0.8750f,23.4864f,-0.7708f,23.5352f,-0.7708f,23.4864f,-0.6667f,23.5352f,-0.6667f,23.4864f,-0.5625f,23.5352f,-0.5625f,
	23.4864f,-0.4583f,23.5352f,-0.4583f,23.4864f,-0.3542f,23.5352f,-0.3542f,23.4864f,-0.2500f,23.5352f,-0.2500f,23.4864f,-0.1458f,23.5352f,-0.1458f,23.4864f,-0.0417f,23.5352f,-0.0417f,23.4864f,0.0625f,23.5352f,0.0625f,23.4864f,0.1667f,23.5352f,0.1667f,23.4864f,0.2708f,
	23.5352f,0.2708f,23.4864f,0.3750f,23.5352f,0.3750f,23.4864f,0.4792f,23.5352f,0.4792f,23.4864f,0.5833f,23.5352f,0.5833f,23.4864f,0.6875f,23.5352f,0.6875f,23.4864f,0.7917f,23.5352f,0.7917f,23.4864f,0.8958f,23.5352f,0.8958f,23.4864f,1.0000f,23.5352f,1.0000f,
	23.4864f,-4.0000f,23.4375f,-4.0000f,23.4375f,-3.8958f,23.3887f,-3.8958f,23.3887f,-3.7917f,23.3399f,-3.7917f,23.3399f,-3.6875f,23.2911f,-3.6875f,23.2911f,-3.5833f,23.2422f,-3.5833f,23.2422f,-3.4792f,23.1934f,-3.4792f,23.1934f,-3.3750f,23.1446f,-3.3750f,23.1446f,-3.2708f,
	23.0958f,-3.2708f,23.0958f,-3.1667f,23.0469f,-3.1667f,23.0469f,-3.0625f,22.9981f,-3.0625f,22.9981f,-2.9583f,22.9493f,-2.9583f,22.9493f,-2.8542f,22.9004f,-2.8542f,22.9004f,-2.7500f,22.8516f,-2.7500f,22.8516f,-2.6458f,22.8028f,-2.6458f,22.8028f,-2.5417f,22.7540f,-2.5417f,
	22.7540f,-2.4375f,22.7051f,-2.4375f,22.7051f,-2.3333f,22.6563f,-2.3333f,22.6563f,-2.2292f,22.6075f,-2.2292f,22.6075f,-2.1250f,22.5586f,-2.1250f,22.5586f,-2.0208f,22.5098f,-2.0208f,22.5098f,-1.9167f,22.4610f,-1.9167f,22.4610f,-1.8125f,22.4122f,-1.8125f,22.4122f,-1.7083f,
	22.3633f,-1.7083f,22.3633f,-1.6042f,22.3145f,-1.6042f,22.3145f,-1.5000f,22.2657f,-1.5000f,22.2657f,-1.3958f,22.2168f,-1.3958f,22.2168f,-1.2917f,22.1680f,-1.2917f,22.1680f,-1.1875f,22.1192f,-1.1875f,22.1192f,-1.0833f,22.0704f,-1.0833f,22.0704f,-0.9792f,22.0215f,-0.9792f,
	22.0215f,-0.8750f,21.9727f,-0.8750f,21.9727f,-0.7708f,21.9239f,-0.7708f,21.9239f,-0.6667f,21.8750f,-0.6667f,21.8750f,-0.5625f,21.8262f,-0.5625f,21.8262f,-0.4583f,21.7774f,-0.4583f,21.7774f,-0.3542f,21.7286f,-0.3542f,21.7286f,-0.2500f,21.6797f,-0.2500f,21.6797f,-0.1458f,
	21.6309f,-0.1458f,21.6309f,-0.0417f,21.5821f,-0.0417f,21.5821f,0.0625f,21.5332f,0.0625f,21.5332f,0.1667f,21.4844f,0.1667f,21.4844f,0.2708f,21.4356f,0.2708f,21.4356f,0.3750f,21.3868f,0.3750f,21.3868f,0.4792f,21.3379f,0.4792f,21.3379f,0.5833f,21.2891f,0.5833f,
	21.2891f,0.6875f,21.2403f,0.6875f,21.2403f,0.7917f,21.1915f,0.7917f,21.1915f,0.8958f,21.1426f,0.8958f,21.1426f,1.0000f,21.0938f,1.0000f,23.4864f,-0.7708f,23.4375f,-0.6667f,23.4864f,-0.6667f,23.4375f,-0.5625f,23.4864f,-0.5625f,23.4375f,-0.4583f,23.4864f,-0.4583f,
	23.4375f,-0.3542f,23.4864f,-0.3542f,23.4375f,-0.2500f,23.4864f,-0.2500f,23.4375f,-0.1458f,23.4864f,-0.1458f,23.4375f,-0.0417f,23.4864f,-0.0417f,23.4375f,0.0625f,23.4864f,0.0625f,23.4375f,0.1667f,23.4864f,0.1667f,23.4375f,0.2708f,23.4864f,0.2708f,23.4375f,0.3750f,
	23.4864f,0.3750f,23.4375f,0.4792f,23.4864f,0.4792f,23.4375f,0.5833f,23.4864f,0.5833f,23.4375f,0.6875f,23.4864f,0.6875f,23.4375f,0.7917f,23.4864f,0.7917f,23.4375f,0.8958f,23.4864f,0.8958f,23.4375f,1.0000f,23.4864f,1.0000f,23.4375f,-4.0000f,23.3887f,-4.0000f,
	23.3887f,-3.8958f,23.3399f,-3.8958f,23.3399f,-3.7917f,23.2911f,-3.7917f,23.2911f,-3.6875f,23.2422f,-3.6875f,23.2422f,-3.5833f,23.1934f,-3.5833f,23.1934f,-3.4792f,23.1446f,-3.4792f,23.1446f,-3.3750f,23.0958f,-3.3750f,23.0958f,-3.2708f,23.0469f,-3.2708f,23.0469f,-3.1667f,
	22.9981f,-3.1667f,22.9981f,-3.0625f,22.9493f,-3.0625f,22.9493f,-2.9583f,22.9004f,-2.9583f,22.9004f,-2.8542f,22.8516f,-2.8542f,22.8516f,-2.7500f,22.8028f,-2.7500f,22.8028f,-2.6458f,22.7540f,-2.6458f,22.7540f,-2.5417f,22.7051f,-2.5417f,22.7051f,-2.4375f,22.6563f,-2.4375f,
	22.6563f,-2.3333f,22.6075f,-2.3333f,22.6075f,-2.2292f,22.5586f,-2.2292f,22.5586f,-2.1250f,22.5098f,-2.1250f,22.5098f,-2.0208f,22.4610f,-2.0208f,22.4610f,-1.9167f,22.4122f,-1.9167f,22.4122f,-1.8125f,22.3633f,-1.8125f,22.3633f,-1.7083f,22.3145f,-1.7083f,22.3145f,-1.6042f,
	22.2657f,-1.6042f,22.2657f,-1.5000f,22.2168f,-1.5000f,22.2168f,-1.3958f,22.1680f,-1.3958f,22.1680f,-1.2917f,22.1192f,-1.2917f,22.1192f,-1.1875f,22.0704f,-1.1875f,22.0704f,-1.0833f,22.0215f,-1.0833f,22.0215f,-0.9792f,21.9727f,-0.9792f,21.9727f,-0.8750f,21.9239f,-0.8750f,
	21.9239f,-0.7708f,21.8750f,-0.7708f,21.8750f,-0.6667f,21.8262f,-0.6667f,21.8262f,-0.5625f,21.7774f,-0.5625f,21.7774f,-0.4583f,21.7286f,-0.4583f,21.7286f,-0.3542f,21.6797f,-0.3542f,21.6797f,-0.2500f,21.6309f,-0.2500f,21.6309f,-0.1458f,21.5821f,-0.1458f,21.5821f,-0.0417f,
	21.5332f,-0.0417f,21.5332f,0.0625f,21.4844f,0.0625f,21.4844f,0.1667f,21.4356f,0.1667f,21.4356f,0.2708f,21.3868f,0.2708f,21.3868f,0.3750f,21.3379f,0.3750f,21.3379f,0.4792f,21.2891f,0.4792f,21.2891f,0.5833f,21.2403f,0.5833f,21.2403f,0.6875f,21.1915f,0.6875f,
	21.1915f,0.7917f,21.1426f,0.7917f,21.1426f,0.8958f,21.0938f,0.8958f,21.0938f,1.0000f,21.0450f,1.0000f,23.4375f,-0.6667f,23.3887f,-0.5625f,23.4375f,-0.5625f,23.3887f,-0.4583f,23.4375f,-0.4583f,23.3887f,-0.3542f,23.4375f,-0.3542f,23.3887f,-0.2500f,23.4375f,-0.2500f,
	23.3887f,-0.1458f,23.4375f,-0.1458f,23.3887f,-0.0417f,23.4375f,-0.0417f,23.3887f,0.0625f,23.4375f,0.0625f,23.3887f,0.1667f,23.4375f,0.1667f,23.3887f,0.2708f,23.4375f,0.2708f,23.3887f,0.3750f,23.4375f,0.3750f,23.3887f,0.4792f,23.4375f,0.4792f,23.3887f,0.5833f,
	23.4375f,0.5833f,23.3887f,0.6875f,23.4375f,0.6875f,23.3887f,0.7917f,23.4375f,0.7917f,23.3887f,0.8958f,23.4375f,0.8958f,23.3887f,1.0000f,23.4375f,1.0000f,23.3887f,-4.0000f,23.3399f,-4.0000f,23.3399f,-3.8958f,23.2911f,-3.8958f,23.2911f,-3.7917f,23.2422f,-3.7917f,
	23.2422f,-3.6875f,23.1934f,-3.6875f,23.1934f,-3.5833f,23.1446f,-3.5833f,23.1446f,-3.4792f,23.0958f,-3.4792f,23.0958f,-3.3750f,23.0469f,-3.3750f,23.0469f,-3.2708f,22.9981f,-3.2708f,22.9981f,-3.1667f,22.9493f,-3.1667f,22.9493f,-3.0625f,22.9004f,-3.0625f,22.9004f,-2.9583f,
	22.8516f,-2.9583f,22.8516f,-2.8542f,22.8028f,-2.8542f,22.8028f,-2.7500f,22.7540f,-2.7500f,22.7540f,-2.6458f,22.7051f,-2.6458f,22.7051f,-2.5417f,22.6563f,-2.5417f,22.6563f,-2.4375f,22.6075f,-2.4375f,22.6075f,-2.3333f,22.5586f,-2.3333f,22.5586f,-2.2292f,22.5098f,-2.2292f,
	22.5098f,-2.1250f,22.4610f,-2.1250f,22.4610f,-2.0208f,22.4122f,-2.0208f,22.4122f,-1.9167f,22.3633f,-1.9167f,22.3633f,-1.8125f,22.3145f,-1.8125f,22.3145f,-1.7083f,22.2657f,-1.7083f,22.2657f,-1.6042f,22.2168f,-1.6042f,22.2168f,-1.5000f,22.1680f,-1.5000f,22.1680f,-1.3958f,
	22.1192f,-1.3958f,22.1192f,-1.2917f,22.0704f,-1.2917f,22.0704f,-1.1875f,22.0215f,-1.1875f,22.0215f,-1.0833f,21.9727f,-1.0833f,21.9727f,-0.9792f,21.9239f,-0.9792f,21.9239f,-0.8750f,21.8750f,-0.8750f,21.8750f,-0.7708f,21.8262f,-0.7708f,21.8262f,-0.6667f,21.7774f,-0.6667f,
	21.7774f,-0.5625f,21.7286f,-0.5625f,21.7286f,-0.4583f,21.6797f,-0.4583f,21.6797f,-0.3542f,21.6309f,-0.3542f,21.6309f,-0.2500f,21.5821f,-0.2500f,21.5821f,-0.1458f,21.5332f,-0.1458f,21.5332f,-0.0417f,21.4844f,-0.0417f,21.4844f,0.0625f,21.4356f,0.0625f,21.4356f,0.1667f,
	21.3868f,0.1667f,21.3868f,0.2708f,21.3379f,0.2708f,21.3379f,0.3750f,21.2891f,0.3750f,21.2891f,0.4792f,21.2403f,0.4792f,21.2403f,0.5833f,21.1915f,0.5833f,21.1915f,0.6875f,21.1426f,0.6875f,21.1426f,0.7917f,21.0938f,0.7917f,21.0938f,0.8958f,21.0450f,0.8958f,
	21.0450f,1.0000f,20.9961f,1.0000f,23.3887f,-0.5625f,23.3399f,-0.4583f,23.3887f,-0.4583f,23.3399f,-0.3542f,23.3887f,-0.3542f,23.3399f,-0.2500f,23.3887f,-0.2500f,23.3399f,-0.1458f,23.3887f,-0.1458f,23.3399f,-0.0417f,23.3887f,-0.0417f,23.3399f,0.0625f,23.3887f,0.0625f,
	23.3399f,0.1667f,23.3887f,0.1667f,23.3399f,0.2708f,23.3887f,0.2708f,23.3399f,0.3750f,23.3887f,0.3750f,23.3399f,0.4792f,23.3887f,0.4792f,23.3399f,0.5833f,23.3887f,0.5833f,23.3399f,0.6875f,23.3887f,0.6875f,23.3399f,0.7917f,23.3887f,0.7917f,23.3399f,0.8958f,
	23.3887f,0.8958f,23.3399f,1.0000f,23.3887f,1.0000f,23.3399f,-4.0000f,23.2911f,-4.0000f,23.2911f,-3.8958f,23.2422f,-3.8958f,23.2422f,-3.7917f,23.1934f,-3.7917f,23.1934f,-3.6875f,23.1446f,-3.6875f,23.1446f,-3.5833f,23.0958f,-3.5833f,23.0958f,-3.4792f,23.0469f,-3.4792f,
	23.0469f,-3.3750f,22.9981f,-3.3750f,22.9981f,-3.2708f,22.9493f,-3.2708f,22.9493f,-3.1667f,22.9004f,-3.1667f,22.9004f,-3.0625f,22.8516f,-3.0625f,22.8516f,-2.9583f,22.8028f,-2.9583f,22.8028f,-2.8542f,22.7540f,-2.8542f,22.7540f,-2.7500f,22.7051f,-2.7500f,22.7051f,-2.6458f,
	22.6563f,-2.6458f,22.6563f,-2.5417f,22.6075f,-2.5417f,22.6075f,-2.4375f,22.5586f,-2.4375f,22.5586f,-2.3333f,22.5098f,-2.3333f,22.5098f,-2.2292f,22.4610f,-2.2292f,22.4610f,-2.1250f,22.4122f,-2.1250f,22.4122f,-2.0208f,22.3633f,-2.0208f,22.3633f,-1.9167f,22.3145f,-1.9167f,
	22.3145f,-1.8125f,22.2657f,-1.8125f,22.2657f,-1.7083f,22.2168f,-1.7083f,22.2168f,-1.6042f,22.1680f,-1.6042f,22.1680f,-1.5000f,22.1192f,-1.5000f,22.1192f,-1.3958f,22.0704f,-1.3958f,22.0704f,-1.2917f,22.0215f,-1.2917f,22.0215f,-1.1875f,21.9727f,-1.1875f,21.9727f,-1.0833f,
	21.9239f,-1.0833f,21.9239f,-0.9792f,21.8750f,-0.9792f,21.8750f,-0.8750f,21.8262f,-0.8750f,21.8262f,-0.7708f,21.7774f,-0.7708f,21.7774f,-0.6667f,21.7286f,-0.6667f,21.7286f,-0.5625f,21.6797f,-0.5625f,21.6797f,-0.4583f,21.6309f,-0.4583f,21.6309f,-0.3542f,21.5821f,-0.3542f,
	21.5821f,-0.2500f,21.5332f,-0.2500f,21.5332f,-0.1458f,21.4844f,-0.1458f,21.4844f,-0.0417f,21.4356f,-0.0417f,21.4356f,0.0625f,21.3868f,0.0625f,21.3868f,0.1667f,21.3379f,0.1667f,21.3379f,0.2708f,21.2891f,0.2708f,21.2891f,0.3750f,21.2403f,0.3750f,21.2403f,0.4792f,
	21.1915f,0.4792f,21.1915f,0.5833f,21.1426f,0.5833f,21.1426f,0.6875f,21.0938f,0.6875f,21.0938f,0.7917f,21.0450f,0.7917f,21.0450f,0.8958f,20.9961f,0.8958f,20.9961f,1.0000f,20.9473f,1.0000f,23.3399f,-0.4583f,23.2911f,-0.3542f,23.3399f,-0.3542f,23.2911f,-0.2500f,
	23.3399f,-0.2500f,23.2911f,-0.1458f,23.3399f,-0.1458f,23.2911f,-0.0417f,23.3399f,-0.0417f,23.2911f,0.0625f,23.3399f,0.0625f,23.2911f,0.1667f,23.3399f,0.1667f,23.2911f,0.2708f,23.3399f,0.2708f,23.2911f,0.3750f,23.3399f,0.3750f,23.2911f,0.4792f,23.3399f,0.4792f,
	23.2911f,0.5833f,23.3399f,0.5833f,23.2911f,0.6875f,23.3399f,0.6875f,23.2911f,0.7917f,23.3399f,0.7917f,23.2911f,0.8958f,23.3399f,0.8958f,23.2911f,1.0000f,23.3399f,1.0000f,23.2911f,-4.0000f,23.2422f,-4.0000f,23.2422f,-3.8958f,23.1934f,-3.8958f,23.1934f,-3.7917f,
	23.1446f,-3.7917f,23.1446f,-3.6875f,23.0958f,-3.6875f,23.0958f,-3.5833f,23.0469f,-3.5833f,23.0469f,-3.4792f,22.9981f,-3.4792f,22.9981f,-3.3750f,22.9493f,-3.3750f,22.9493f,-3.2708f,22.9004f,-3.2708f,22.9004f,-3.1667f,22.8516f,-3.1667f,22.8516f,-3.0625f,22.8028f,-3.0625f,
	22.8028f,-2.9583f,22.7540f,-2.9583f,22.7540f,-2.8542f,22.7051f,-2.8542f,22.7051f,-2.7500f,22.6563f,-2.7500f,22.6563f,-2.6458f,22.6075f,-2.6458f,22.6075f,-2.5417f,22.5586f,-2.5417f,22.5586f,-2.4375f,22.5098f,-2.4375f,22.5098f,-2.3333f,22.4610f,-2.3333f,22.4610f,-2.2292f,
	22.4122f,-2.2292f,22.4122f,-2.1250f,22.3633f,-2.1250f,22.3633f,-2.0208f,22.3145f,-2.0208f,22.3145f,-1.9167f,22.2657f,-1.9167f,22.2657f,-1.8125f,22.2168f,-1.8125f,22.2168f,-1.7083f,22.1680f,-1.7083f,22.1680f,-1.6042f,22.1192f,-1.6042f,22.1192f,-1.5000f,22.0704f,-1.5000f,
	22.0704f,-1.3958f,22.0215f,-1.3958f,22.0215f,-1.2917f,21.9727f,-1.2917f,21.9727f,-1.1875f,21.9239f,-1.1875f,21.9239f,-1.0833f,21.8750f,-1.0833f,21.8750f,-0.9792f,21.8262f,-0.9792f,21.8262f,-0.8750f,21.7774f,-0.8750f,21.7774f,-0.7708f,21.7286f,-0.7708f,21.7286f,-0.6667f,
	21.6797f,-0.6667f,21.6797f,-0.5625f,21.6309f,-0.5625f,21.6309f,-0.4583f,21.5821f,-0.4583f,21.5821f,-0.3542f,21.5332f,-0.3542f,21.5332f,-0.2500f,21.4844f,-0.2500f,21.4844f,-0.1458f,21.4356f,-0.1458f,21.4356f,-0.0417f,21.3868f,-0.0417f,21.3868f,0.0625f,21.3379f,0.0625f,
	21.3379f,0.1667f,21.2891f,0.1667f,21.2891f,0.2708f,21.2403f,0.2708f,21.2403f,0.3750f,21.1915f,0.3750f,21.1915f,0.4792f,21.1426f,0.4792f,21.1426f,0.5833f,21.0938f,0.5833f,21.0938f,0.6875f,21.0450f,0.6875f,21.0450f,0.7917f,20.9961f,0.7917f,20.9961f,0.8958f,
	20.9473f,0.8958f,20.9473f,1.0000f,20.8985f,1.0000f,23.2911f,-0.3542f,23.2422f,-0.2500f,23.2911f,-0.2500f,23.2422f,-0.1458f,23.2911f,-0.1458f,23.2422f,-0.0417f,23.2911f,-0.0417f,23.2422f,0.0625f,23.2911f,0.0625f,23.2422f,0.1667f,23.2911f,0.1667f,23.2422f,0.2708f,
	23.2911f,0.2708f,23.2422f,0.3750f,23.2911f,0.3750f,23.2422f,0.4792f,23.2911f,0.4792f,23.2422f,0.5833f,23.2911f,0.5833f,23.2422f,0.6875f,23.2911f,0.6875f,23.2422f,0.7917f,23.2911f,0.7917f,23.2422f,0.8958f,23.2911f,0.8958f,23.2422f,1.0000f,23.2911f,1.0000f,
	23.2422f,-4.0000f,23.1934f,-4.0000f,23.1934f,-3.8958f,23.1446f,-3.8958f,23.1446f,-3.7917f,23.0958f,-3.7917f,23.0958f,-3.6875f,23.0469f,-3.6875f,23.0469f,-3.5833f,22.9981f,-3.5833f,22.9981f,-3.4792f,22.9493f,-3.4792f,22.9493f,-3.3750f,22.9004f,-3.3750f,22.9004f,-3.2708f,
	22.8516f,-3.2708f,22.8516f,-3.1667f,22.8028f,-3.1667f,22.8028f,-3.0625f,22.7540f,-3.0625f,22.7540f,-2.9583f,22.7051f,-2.9583f,22.7051f,-2.8542f,22.6563f,-2.8542f,22.6563f,-2.7500f,22.6075f,-2.7500f,22.6075f,-2.6458f,22.5586f,-2.6458f,22.5586f,-2.5417f,22.5098f,-2.5417f,
	22.5098f,-2.4375f,22.4610f,-2.4375f,22.4610f,-2.3333f,22.4122f,-2.3333f,22.4122f,-2.2292f,22.3633f,-2.2292f,22.3633f,-2.1250f,22.3145f,-2.1250f,22.3145f,-2.0208f,22.2657f,-2.0208f,22.2657f,-1.9167f,22.2168f,-1.9167f,22.2168f,-1.8125f,22.1680f,-1.8125f,22.1680f,-1.7083f,
	22.1192f,-1.7083f,22.1192f,-1.6042f,22.0704f,-1.6042f,22.0704f,-1.5000f,22.0215f,-1.5000f,22.0215f,-1.3958f,21.9727f,-1.3958f,21.9727f,-1.2917f,21.9239f,-1.2917f,21.9239f,-1.1875f,21.8750f,-1.1875f,21.8750f,-1.0833f,21.8262f,-1.0833f,21.8262f,-0.9792f,21.7774f,-0.9792f,
	21.7774f,-0.8750f,21.7286f,-0.8750f,21.7286f,-0.7708f,21.6797f,-0.7708f,21.6797f,-0.6667f,21.6309f,-0.6667f,21.6309f,-0.5625f,21.5821f,-0.5625f,21.5821f,-0.4583f,21.5332f,-0.4583f,21.5332f,-0.3542f,21.4844f,-0.3542f,21.4844f,-0.2500f,21.4356f,-0.2500f,21.4356f,-0.1458f,
	21.3868f,-0.1458f,21.3868f,-0.0417f,21.3379f,-0.0417f,21.3379f,0.0625f,21.2891f,0.0625f,21.2891f,0.1667f,21.2403f,0.1667f,21.2403f,0.2708f,21.1915f,0.2708f,21.1915f,0.3750f,21.1426f,0.3750f,21.1426f,0.4792f,21.0938f,0.4792f,21.0938f,0.5833f,21.0450f,0.5833f,
	21.0450f,0.6875f,20.9961f,0.6875f,20.9961f,0.7917f,20.9473f,0.7917f,20.9473f,0.8958f,20.8985f,0.8958f,20.8985f,1.0000f,20.8497f,1.0000f,23.2422f,-0.2500f,23.1934f,-0.1458f,23.2422f,-0.1458f,23.1934f,-0.0417f,23.2422f,-0.0417f,23.1934f,0.0625f,23.2422f,0.0625f,
	23.1934f,0.1667f,23.2422f,0.1667f,23.1934f,0.2708f,23.2422f,0.2708f,23.1934f,0.3750f,23.2422f,0.3750f,23.1934f,0.4792f,23.2422f,0.4792f,23.1934f,0.5833f,23.2422f,0.5833f,23.1934f,0.6875f,23.2422f,0.6875f,23.1934f,0.7917f,23.2422f,0.7917f,23.1934f,0.8958f,
	23.2422f,0.8958f,23.1934f,1.0000f,23.2422f,1.0000f,23.1934f,-4.0000f,23.1446f,-4.0000f,23.1446f,-3.8958f,23.0958f,-3.8958f,23.0958f,-3.7917f,23.0469f,-3.7917f,23.0469f,-3.6875f,22.9981f,-3.6875f,22.9981f,-3.5833f,22.9493f,-3.5833f,22.9493f,-3.4792f,22.9004f,-3.4792f,
	22.9004f,-3.3750f,22.8516f,-3.3750f,22.8516f,-3.2708f,22.8028f,-3.2708f,22.8028f,-3.1667f,22.7540f,-3.1667f,22.7540f,-3.0625f,22.7051f,-3.0625f,22.7051f,-2.9583f,22.6563f,-2.9583f,22.6563f,-2.8542f,22.6075f,-2.8542f,22.6075f,-2.7500f,22.5586f,-2.7500f,22.5586f,-2.6458f,
	22.5098f,-2.6458f,22.5098f,-2.5417f,22.4610f,-2.5417f,22.4610f,-2.4375f,22.4122f,-2.4375f,22.4122f,-2.3333f,22.3633f,-2.3333f,22.3633f,-2.2292f,22.3145f,-2.2292f,22.3145f,-2.1250f,22.2657f,-2.1250f,22.2657f,-2.0208f,22.2168f,-2.0208f,22.2168f,-1.9167f,22.1680f,-1.9167f,
	22.1680f,-1.8125f,22.1192f,-1.8125f,22.1192f,-1.7083f,22.0704f,-1.7083f,22.0704f,-1.6042f,22.0215f,-1.6042f,22.0215f,-1.5000f,21.9727f,-1.5000f,21.9727f,-1.3958f,21.9239f,-1.3958f,21.9239f,-1.2917f,21.8750f,-1.2917f,21.8750f,-1.1875f,21.8262f,-1.1875f,21.8262f,-1.0833f,
	21.7774f,-1.0833f,21.7774f,-0.9792f,21.7286f,-0.9792f,21.7286f,-0.8750f,21.6797f,-0.8750f,21.6797f,-0.7708f,21.6309f,-0.7708f,21.6309f,-0.6667f,21.5821f,-0.6667f,21.5821f,-0.5625f,21.5332f,-0.5625f,21.5332f,-0.4583f,21.4844f,-0.4583f,21.4844f,-0.3542f,21.4356f,-0.3542f,
	21.4356f,-0.2500f,21.3868f,-0.2500f,21.3868f,-0.1458f,21.3379f,-0.1458f,21.3379f,-0.0417f,21.2891f,-0.0417f,21.2891f,0.0625f,21.2403f,0.0625f,21.2403f,0.1667f,21.1915f,0.1667f,21.1915f,0.2708f,21.1426f,0.2708f,21.1426f,0.3750f,21.0938f,0.3750f,21.0938f,0.4792f,
	21.0450f,0.4792f,21.0450f,0.5833f,20.9961f,0.5833f,20.9961f,0.6875f,20.9473f,0.6875f,20.9473f,0.7917f,20.8985f,0.7917f,20.8985f,0.8958f,20.8497f,0.8958f,20.8497f,1.0000f,20.8008f,1.0000f,23.1934f,-0.1458f,23.1446f,-0.0417f,23.1934f,-0.0417f,23.1446f,0.0625f,
	23.1934f,0.0625f,23.1446f,0.1667f,23.1934f,0.1667f,23.1446f,0.2708f,23.1934f,0.2708f,23.1446f,0.3750f,23.1934f,0.3750f,23.1446f,0.4792f,23.1934f,0.4792f,23.1446f,0.5833f,23.1934f,0.5833f,23.1446f,0.6875f,23.1934f,0.6875f,23.1446f,0.7917f,23.1934f,0.7917f,
	23.1446f,0.8958f,23.1934f,0.8958f,23.1446f,1.0000f,23.1934f,1.0000f,23.1446f,-4.0000f,23.0958f,-4.0000f,23.0958f,-3.8958f,23.0469f,-3.8958f,23.0469f,-3.7917f,22.9981f,-3.7917f,22.9981f,-3.6875f,22.9493f,-3.6875f,22.9493f,-3.5833f,22.9004f,-3.5833f,22.9004f,-3.4792f,
	22.8516f,-3.4792f,22.8516f,-3.3750f,22.8028f,-3.3750f,22.8028f,-3.2708f,22.7540f,-3.2708f,22.7540f,-3.1667f,22.7051f,-3.1667f,22.7051f,-3.0625f,22.6563f,-3.0625f,22.6563f,-2.9583f,22.6075f,-2.9583f,22.6075f,-2.8542f,22.5586f,-2.8542f,22.5586f,-2.7500f,22.5098f,-2.7500f,
	22.5098f,-2.6458f,22.4610f,-2.6458f,22.4610f,-2.5417f,22.4122f,-2.5417f,22.4122f,-2.4375f,22.3633f,-2.4375f,22.3633f,-2.3333f,22.3145f,-2.3333f,22.3145f,-2.2292f,22.2657f,-2.2292f,22.2657f,-2.1250f,22.2168f,-2.1250f,22.2168f,-2.0208f,22.1680f,-2.0208f,22.1680f,-1.9167f,
	22.1192f,-1.9167f,22.1192f,-1.8125f,22.0704f,-1.8125f,22.0704f,-1.7083f,22.0215f,-1.7083f,22.0215f,-1.6042f,21.9727f,-1.6042f,21.9727f,-1.5000f,21.9239f,-1.5000f,21.9239f,-1.3958f,21.8750f,-1.3958f,21.8750f,-1.2917f,21.8262f,-1.2917f,21.8262f,-1.1875f,21.7774f,-1.1875f,
	21.7774f,-1.0833f,21.7286f,-1.0833f,21.7286f,-0.9792f,21.6797f,-0.9792f,21.6797f,-0.8750f,21.6309f,-0.8750f,21.6309f,-0.7708f,21.5821f,-0.7708f,21.5821f,-0.6667f,21.5332f,-0.6667f,21.5332f,-0.5625f,21.4844f,-0.5625f,21.4844f,-0.4583f,21.4356f,-0.4583f,21.4356f,-0.3542f,
	21.3868f,-0.3542f,21.3868f,-0.2500f,21.3379f,-0.2500f,21.3379f,-0.1458f,21.2891f,-0.1458f,21.2891f,-0.0417f,21.2403f,-0.0417f,21.2403f,0.0625f,21.1915f,0.0625f,21.1915f,0.1667f,21.1426f,0.1667f,21.1426f,0.2708f,21.0938f,0.2708f,21.0938f,0.3750f,21.0450f,0.3750f,
	21.0450f,0.4792f,20.9961f,0.4792f,20.9961f,0.5833f,20.9473f,0.5833f,20.9473f,0.6875f,20.8985f,0.6875f,20.8985f,0.7917f,20.8497f,0.7917f,20.8497f,0.8958f,20.8008f,0.8958f,20.8008f,1.0000f,20.7520f,1.0000f,23.1446f,-0.0417f,23.0958f,0.0625f,23.1446f,0.0625f,
	23.0958f,0.1667f,23.1446f,0.1667f,23.0958f,0.2708f,23.1446f,0.2708f,23.0958f,0.3750f,23.1446f,0.3750f,23.0958f,0.4792f,23.1446f,0.4792f,23.0958f,0.5833f,23.1446f,0.5833f,23.0958f,0.6875f,23.1446f,0.6875f,23.0958f,0.7917f,23.1446f,0.7917f,23.0958f,0.8958f,
	23.1446f,0.8958f,23.0958f,1.0000f,23.1446f,1.0000f,23.0958f,-4.0000f,23.0469f,-4.0000f,23.0469f,-3.8958f,22.9981f,-3.8958f,22.9981f,-3.7917f,22.9493f,-3.7917f,22.9493f,-3.6875f,22.9004f,-3.6875f,22.9004f,-3.5833f,22.8516f,-3.5833f,22.8516f,-3.4792f,22.8028f,-3.4792f,
	22.8028f,-3.3750f,22.7540f,-3.3750f,22.7540f,-3.2708f,22.7051f,-3.2708f,22.7051f,-3.1667f,22.6563f,-3.1667f,22.6563f,-3.0625f,22.6075f,-3.0625f,22.6075f,-2.9583f,22.5586f,-2.9583f,22.5586f,-2.8542f,22.5098f,-2.8542f,22.5098f,-2.7500f,22.4610f,-2.7500f,22.4610f,-2.6458f,
	22.4122f,-2.6458f,22.4122f,-2.5417f,22.3633f,-2.5417f,22.3633f,-2.4375f,22.3145f,-2.4375f,22.3145f,-2.3333f,22.2657f,-2.3333f,22.2657f,-2.2292f,22.2168f,-2.2292f,22.2168f,-2.1250f,22.1680f,-2.1250f,22.1680f,-2.0208f,22.1192f,-2.0208f,22.1192f,-1.9167f,22.0704f,-1.9167f,
	22.0704f,-1.8125f,22.0215f,-1.8125f,22.0215f,-1.7083f,21.9727f,-1.7083f,21.9727f,-1.6042f,21.9239f,-1.6042f,21.9239f,-1.5000f,21.8750f,-1.5000f,21.8750f,-1.3958f,21.8262f,-1.3958f,21.8262f,-1.2917f,21.7774f,-1.2917f,21.7774f,-1.1875f,21.7286f,-1.1875f,21.7286f,-1.0833f,
	21.6797f,-1.0833f,21.6797f,-0.9792f,21.6309f,-0.9792f,21.6309f,-0.8750f,21.5821f,-0.8750f,21.5821f,-0.7708f,21.5332f,-0.7708f,21.5332f,-0.6667f,21.4844f,-0.6667f,21.4844f,-0.5625f,21.4356f,-0.5625f,21.4356f,-0.4583f,21.3868f,-0.4583f,21.3868f,-0.3542f,21.3379f,-0.3542f,
	21.3379f,-0.2500f,21.2891f,-0.2500f,21.2891f,-0.1458f,21.2403f,-0.1458f,21.2403f,-0.0417f,21.1915f,-0.0417f,21.1915f,0.0625f,21.1426f,0.0625f,21.1426f,0.1667f,21.0938f,0.1667f,21.0938f,0.2708f,21.0450f,0.2708f,21.0450f,0.3750f,20.9961f,0.3750f,20.9961f,0.4792f,
	20.9473f,0.4792f,20.9473f,0.5833f,20.8985f,0.5833f,20.8985f,0.6875f,20.8497f,0.6875f,20.8497f,0.7917f,20.8008f,0.7917f,20.8008f,0.8958f,20.7520f,0.8958f,20.7520f,1.0000f,20.7032f,1.0000f,23.0958f,0.0625f,23.0469f,0.1667f,23.0958f,0.1667f,23.0469f,0.2708f,
	23.0958f,0.2708f,23.0469f,0.3750f,23.0958f,0.3750f,23.0469f,0.4792f,23.0958f,0.4792f,23.0469f,0.5833f,23.0958f,0.5833f,23.0469f,0.6875f,23.0958f,0.6875f,23.0469f,0.7917f,23.0958f,0.7917f,23.0469f,0.8958f,23.0958f,0.8958f,23.0469f,1.0000f,23.0958f,1.0000f,
	23.0469f,-4.0000f,22.9981f,-4.0000f,22.9981f,-3.8958f,22.9493f,-3.8958f,22.9493f,-3.7917f,22.9004f,-3.7917f,22.9004f,-3.6875f,22.8516f,-3.6875f,22.8516f,-3.5833f,22.8028f,-3.5833f,22.8028f,-3.4792f,22.7540f,-3.4792f,22.7540f,-3.3750f,22.7051f,-3.3750f,22.7051f,-3.2708f,
	22.6563f,-3.2708f,22.6563f,-3.1667f,22.6075f,-3.1667f,22.6075f,-3.0625f,22.5586f,-3.0625f,22.5586f,-2.9583f,22.5098f,-2.9583f,22.5098f,-2.8542f,22.4610f,-2.8542f,22.4610f,-2.7500f,22.4122f,-2.7500f,22.4122f,-2.6458f,22.3633f,-2.6458f,22.3633f,-2.5417f,22.3145f,-2.5417f,
	22.3145f,-2.4375f,22.2657f,-2.4375f,22.2657f,-2.3333f,22.2168f,-2.3333f,22.2168f,-2.2292f,22.1680f,-2.2292f,22.1680f,-2.1250f,22.1192f,-2.1250f,22.1192f,-2.0208f,22.0704f,-2.0208f,22.0704f,-1.9167f,22.0215f,-1.9167f,22.0215f,-1.8125f,21.9727f,-1.8125f,21.9727f,-1.7083f,
	21.9239f,-1.7083f,21.9239f,-1.6042f,21.8750f,-1.6042f,21.8750f,-1.5000f,21.8262f,-1.5000f,21.8262f,-1.3958f,21.7774f,-1.3958f,21.7774f,-1.2917f,21.7286f,-1.2917f,21.7286f,-1.1875f,21.6797f,-1.1875f,21.6797f,-1.0833f,21.6309f,-1.0833f,21.6309f,-0.9792f,21.5821f,-0.9792f,
	21.5821f,-0.8750f,21.5332f,-0.8750f,21.5332f,-0.7708f,21.4844f,-0.7708f,21.4844f,-0.6667f,21.4356f,-0.6667f,21.4356f,-0.5625f,21.3868f,-0.5625f,21.3868f,-0.4583f,21.3379f,-0.4583f,21.3379f,-0.3542f,21.2891f,-0.3542f,21.2891f,-0.2500f,21.2403f,-0.2500f,21.2403f,-0.1458f,
	21.1915f,-0.1458f,21.1915f,-0.0417f,21.1426f,-0.0417f,21.1426f,0.0625f,21.0938f,0.0625f,21.0938f,0.1667f,21.0450f,0.1667f,21.0450f,0.2708f,20.9961f,0.2708f,20.9961f,0.3750f,20.9473f,0.3750f,20.9473f,0.4792f,20.8985f,0.4792f,20.8985f,0.5833f,20.8497f,0.5833f,
	20.8497f,0.6875f,20.8008f,0.6875f,20.8008f,0.7917f,20.7520f,0.7917f,20.7520f,0.8958f,20.7032f,0.8958f,20.7032f,1.0000f,20.6543f,1.0000f,23.0469f,0.1667f,22.9981f,0.2708f,23.0469f,0.2708f,22.9981f,0.3750f,23.0469f,0.3750f,22.9981f,0.4792f,23.0469f,0.4792f,
	22.9981f,0.5833f,23.0469f,0.5833f,22.9981f,0.6875f,23.0469f,0.6875f,22.9981f,0.7917f,23.0469f,0.7917f,22.9981f,0.8958f,23.0469f,0.8958f,22.9981f,1.0000f,23.0469f,1.0000f,22.9981f,-4.0000f,22.9493f,-4.0000f,22.9493f,-3.8958f,22.9004f,-3.8958f,22.9004f,-3.7917f,
	22.8516f,-3.7917f,22.8516f,-3.6875f,22.8028f,-3.6875f,22.8028f,-3.5833f,22.7540f,-3.5833f,22.7540f,-3.4792f,22.7051f,-3.4792f,22.7051f,-3.3750f,22.6563f,-3.3750f,22.6563f,-3.2708f,22.6075f,-3.2708f,22.6075f,-3.1667f,22.5586f,-3.1667f,22.5586f,-3.0625f,22.5098f,-3.0625f,
	22.5098f,-2.9583f,22.4610f,-2.9583f,22.4610f,-2.8542f,22.4122f,-2.8542f,22.4122f,-2.7500f,22.3633f,-2.7500f,22.3633f,-2.6458f,22.3145f,-2.6458f,22.3145f,-2.5417f,22.2657f,-2.5417f,22.2657f,-2.4375f,22.2168f,-2.4375f,22.2168f,-2.3333f,22.1680f,-2.3333f,22.1680f,-2.2292f,
	22.1192f,-2.2292f,22.1192f,-2.1250f,22.0704f,-2.1250f,22.0704f,-2.0208f,22.0215f,-2.0208f,22.0215f,-1.9167f,21.9727f,-1.9167f,21.9727f,-1.8125f,21.9239f,-1.8125f,21.9239f,-1.7083f,21.8750f,-1.7083f,21.8750f,-1.6042f,21.8262f,-1.6042f,21.8262f,-1.5000f,21.7774f,-1.5000f,
	21.7774f,-1.3958f,21.7286f,-1.3958f,21.7286f,-1.2917f,21.6797f,-1.2917f,21.6797f,-1.1875f,21.6309f,-1.1875f,21.6309f,-1.0833f,21.5821f,-1.0833f,21.5821f,-0.9792f,21.5332f,-0.9792f,21.5332f,-0.8750f,21.4844f,-0.8750f,21.4844f,-0.7708f,21.4356f,-0.7708f,21.4356f,-0.6667f,
	21.3868f,-0.6667f,21.3868f,-0.5625f,21.3379f,-0.5625f,21.3379f,-0.4583f,21.2891f,-0.4583f,21.2891f,-0.3542f,21.2403f,-0.3542f,21.2403f,-0.2500f,21.1915f,-0.2500f,21.1915f,-0.1458f,21.1426f,-0.1458f,21.1426f,-0.0417f,21.0938f,-0.0417f,21.0938f,0.0625f,21.0450f,0.0625f,
	21.0450f,0.1667f,20.9961f,0.1667f,20.9961f,0.2708f,20.9473f,0.2708f,20.9473f,0.3750f,20.8985f,0.3750f,20.8985f,0.4792f,20.8497f,0.4792f,20.8497f,0.5833f,20.8008f,0.5833f,20.8008f,0.6875f,20.7520f,0.6875f,20.7520f,0.7917f,20.7032f,0.7917f,20.7032f,0.8958f,
	20.6543f,0.8958f,20.6543f,1.0000f,20.6055f,1.0000f,22.9981f,0.2708f,22.9493f,0.3750f,22.9981f,0.3750f,22.9493f,0.4792f,22.9981f,0.4792f,22.9493f,0.5833f,22.9981f,0.5833f,22.9493f,0.6875f,22.9981f,0.6875f,22.9493f,0.7917f,22.9981f,0.7917f,22.9493f,0.8958f,
	22.9981f,0.8958f,22.9493f,1.0000f,22.9981f,1.0000f,22.9493f,-4.0000f,22.9004f,-4.0000f,22.9004f,-3.8958f,22.8516f,-3.8958f,22.8516f,-3.7917f,22.8028f,-3.7917f,22.8028f,-3.6875f,22.7540f,-3.6875f,22.7540f,-3.5833f,22.7051f,-3.5833f,22.7051f,-3.4792f,22.6563f,-3.4792f,
	22.6563f,-3.3750f,22.6075f,-3.3750f,22.6075f,-3.2708f,22.5586f,-3.2708f,22.5586f,-3.1667f,22.5098f,-3.1667f,22.5098f,-3.0625f,22.4610f,-3.0625f,22.4610f,-2.9583f,22.4122f,-2.9583f,22.4122f,-2.8542f,22.3633f,-2.8542f,22.3633f,-2.7500f,22.3145f,-2.7500f,22.3145f,-2.6458f,
	22.2657f,-2.6458f,22.2657f,-2.5417f,22.2168f,-2.5417f,22.2168f,-2.4375f,22.1680f,-2.4375f,22.1680f,-2.3333f,22.1192f,-2.3333f,22.1192f,-2.2292f,22.0704f,-2.2292f,22.0704f,-2.1250f,22.0215f,-2.1250f,22.0215f,-2.0208f,21.9727f,-2.0208f,21.9727f,-1.9167f,21.9239f,-1.9167f,
	21.9239f,-1.8125f,21.8750f,-1.8125f,21.8750f,-1.7083f,21.8262f,-1.7083f,21.8262f,-1.6042f,21.7774f,-1.6042f,21.7774f,-1.5000f,21.7286f,-1.5000f,21.7286f,-1.3958f,21.6797f,-1.3958f,21.6797f,-1.2917f,21.6309f,-1.2917f,21.6309f,-1.1875f,21.5821f,-1.1875f,21.5821f,-1.0833f,
	21.5332f,-1.0833f,21.5332f,-0.9792f,21.4844f,-0.9792f,21.4844f,-0.8750f,21.4356f,-0.8750f,21.4356f,-0.7708f,21.3868f,-0.7708f,21.3868f,-0.6667f,21.3379f,-0.6667f,21.3379f,-0.5625f,21.2891f,-0.5625f,21.2891f,-0.4583f,21.2403f,-0.4583f,21.2403f,-0.3542f,21.1915f,-0.3542f,
	21.1915f,-0.2500f,21.1426f,-0.2500f,21.1426f,-0.1458f,21.0938f,-0.1458f,21.0938f,-0.0417f,21.0450f,-0.0417f,21.0450f,0.0625f,20.9961f,0.0625f,20.9961f,0.1667f,20.9473f,0.1667f,20.9473f,0.2708f,20.8985f,0.2708f,20.8985f,0.3750f,20.8497f,0.3750f,20.8497f,0.4792f,
	20.8008f,0.4792f,20.8008f,0.5833f,20.7520f,0.5833f,20.7520f,0.6875f,20.7032f,0.6875f,20.7032f,0.7917f,20.6543f,0.7917f,20.6543f,0.8958f,20.6055f,0.8958f,20.6055f,1.0000f,20.5567f,1.0000f,22.9493f,0.3750f,22.9004f,0.4792f,22.9493f,0.4792f,22.9004f,0.5833f,
	22.9493f,0.5833f,22.9004f,0.6875f,22.9493f,0.6875f,22.9004f,0.7917f,22.9493f,0.7917f,22.9004f,0.8958f,22.9493f,0.8958f,22.9004f,1.0000f,22.9493f,1.0000f,22.9004f,-4.0000f,22.8516f,-4.0000f,22.8516f,-3.8958f,22.8028f,-3.8958f,22.8028f,-3.7917f,22.7540f,-3.7917f,
	22.7540f,-3.6875f,22.7051f,-3.6875f,22.7051f,-3.5833f,22.6563f,-3.5833f,22.6563f,-3.4792f,22.6075f,-3.4792f,22.6075f,-3.3750f,22.5586f,-3.3750f,22.5586f,-3.2708f,22.5098f,-3.2708f,22.5098f,-3.1667f,22.4610f,-3.1667f,22.4610f,-3.0625f,22.4122f,-3.0625f,22.4122f,-2.9583f,
	22.3633f,-2.9583f,22.3633f,-2.8542f,22.3145f,-2.8542f,22.3145f,-2.7500f,22.2657f,-2.7500f,22.2657f,-2.6458f,22.2168f,-2.6458f,22.2168f,-2.5417f,22.1680f,-2.5417f,22.1680f,-2.4375f,22.1192f,-2.4375f,22.1192f,-2.3333f,22.0704f,-2.3333f,22.0704f,-2.2292f,22.0215f,-2.2292f,
	22.0215f,-2.1250f,21.9727f,-2.1250f,21.9727f,-2.0208f,21.9239f,-2.0208f,21.9239f,-1.9167f,21.8750f,-1.9167f,21.8750f,-1.8125f,21.8262f,-1.8125f,21.8262f,-1.7083f,21.7774f,-1.7083f,21.7774f,-1.6042f,21.7286f,-1.6042f,21.7286f,-1.5000f,21.6797f,-1.5000f,21.6797f,-1.3958f,
	21.6309f,-1.3958f,21.6309f,-1.2917f,21.5821f,-1.2917f,21.5821f,-1.1875f,21.5332f,-1.1875f,21.5332f,-1.0833f,21.4844f,-1.0833f,21.4844f,-0.9792f,21.4356f,-0.9792f,21.4356f,-0.8750f,21.3868f,-0.8750f,21.3868f,-0.7708f,21.3379f,-0.7708f,21.3379f,-0.6667f,21.2891f,-0.6667f,
	21.2891f,-0.5625f,21.2403f,-0.5625f,21.2403f,-0.4583f,21.1915f,-0.4583f,21.1915f,-0.3542f,21.1426f,-0.3542f,21.1426f,-0.2500f,21.0938f,-0.2500f,21.0938f,-0.1458f,21.0450f,-0.1458f,21.0450f,-0.0417f,20.9961f,-0.0417f,20.9961f,0.0625f,20.9473f,0.0625f,20.9473f,0.1667f,
	20.8985f,0.1667f,20.8985f,0.2708f,20.8497f,0.2708f,20.8497f,0.3750f,20.8008f,0.3750f,20.8008f,0.4792f,20.7520f,0.4792f,20.7520f,0.5833f,20.7032f,0.5833f,20.7032f,0.6875f,20.6543f,0.6875f,20.6543f,0.7917f,20.6055f,0.7917f,20.6055f,0.8958f,20.5567f,0.8958f,
	20.5567f,1.0000f,20.5079f,1.0000f,22.9004f,0.4792f,22.8516f,0.5833f,22.9004f,0.5833f,22.8516f,0.6875f,22.9004f,0.6875f,22.8516f,0.7917f,22.9004f,0.7917f,22.8516f,0.8958f,22.9004f,0.8958f,22.8516f,1.0000f,22.9004f,1.0000f,22.8516f,-4.0000f,22.8028f,-4.0000f,
	22.8028f,-3.8958f,22.7540f,-3.8958f,22.7540f,-3.7917f,22.7051f,-3.7917f,22.7051f,-3.6875f,22.6563f,-3.6875f,22.6563f,-3.5833f,22.6075f,-3.5833f,22.6075f,-3.4792f,22.5586f,-3.4792f,22.5586f,-3.3750f,22.5098f,-3.3750f,22.5098f,-3.2708f,22.4610f,-3.2708f,22.4610f,-3.1667f,
	22.4122f,-3.1667f,22.4122f,-3.0625f,22.3633f,-3.0625f,22.3633f,-2.9583f,22.3145f,-2.9583f,22.3145f,-2.8542f,22.2657f,-2.8542f,22.2657f,-2.7500f,22.2168f,-2.7500f,22.2168f,-2.6458f,22.1680f,-2.6458f,22.1680f,-2.5417f,22.1192f,-2.5417f,22.1192f,-2.4375f,22.0704f,-2.4375f,
	22.0704f,-2.3333f,22.0215f,-2.3333f,22.0215f,-2.2292f,21.9727f,-2.2292f,21.9727f,-2.1250f,21.9239f,-2.1250f,21.9239f,-2.0208f,21.8750f,-2.0208f,21.8750f,-1.9167f,21.8262f,-1.9167f,21.8262f,-1.8125f,21.7774f,-1.8125f,21.7774f,-1.7083f,21.7286f,-1.7083f,21.7286f,-1.6042f,
	21.6797f,-1.6042f,21.6797f,-1.5000f,21.6309f,-1.5000f,21.6309f,-1.3958f,21.5821f,-1.3958f,21.5821f,-1.2917f,21.5332f,-1.2917f,21.5332f,-1.1875f,21.4844f,-1.1875f,21.4844f,-1.0833f,21.4356f,-1.0833f,21.4356f,-0.9792f,21.3868f,-0.9792f,21.3868f,-0.8750f,21.3379f,-0.8750f,
	21.3379f,-0.7708f,21.2891f,-0.7708f,21.2891f,-0.6667f,21.2403f,-0.6667f,21.2403f,-0.5625f,21.1915f,-0.5625f,21.1915f,-0.4583f,21.1426f,-0.4583f,21.1426f,-0.3542f,21.0938f,-0.3542f,21.0938f,-0.2500f,21.0450f,-0.2500f,21.0450f,-0.1458f,20.9961f,-0.1458f,20.9961f,-0.0417f,
	20.9473f,-0.0417f,20.9473f,0.0625f,20.8985f,0.0625f,20.8985f,0.1667f,20.8497f,0.1667f,20.8497f,0.2708f,20.8008f,0.2708f,20.8008f,0.3750f,20.7520f,0.3750f,20.7520f,0.4792f,20.7032f,0.4792f,20.7032f,0.5833f,20.6543f,0.5833f,20.6543f,0.6875f,20.6055f,0.6875f,
	20.6055f,0.7917f,20.5567f,0.7917f,20.5567f,0.8958f,20.5079f,0.8958f,20.5079f,1.0000f,20.4590f,1.0000f,22.8516f,0.5833f,22.8028f,0.6875f,22.8516f,0.6875f,22.8028f,0.7917f,22.8516f,0.7917f,22.8028f,0.8958f,22.8516f,0.8958f,22.8028f,1.0000f,22.8516f,1.0000f,
	22.8028f,-4.0000f,22.7540f,-4.0000f,22.7540f,-3.8958f,22.7051f,-3.8958f,22.7051f,-3.7917f,22.6563f,-3.7917f,22.6563f,-3.6875f,22.6075f,-3.6875f,22.6075f,-3.5833f,22.5586f,-3.5833f,22.5586f,-3.4792f,22.5098f,-3.4792f,22.5098f,-3.3750f,22.4610f,-3.3750f,22.4610f,-3.2708f,
	22.4122f,-3.2708f,22.4122f,-3.1667f,22.3633f,-3.1667f,22.3633f,-3.0625f,22.3145f,-3.0625f,22.3145f,-2.9583f,22.2657f,-2.9583f,22.2657f,-2.8542f,22.2168f,-2.8542f,22.2168f,-2.7500f,22.1680f,-2.7500f,22.1680f,-2.6458f,22.1192f,-2.6458f,22.1192f,-2.5417f,22.0704f,-2.5417f,
	22.0704f,-2.4375f,22.0215f,-2.4375f,22.0215f,-2.3333f,21.9727f,-2.3333f,21.9727f,-2.2292f,21.9239f,-2.2292f,21.9239f,-2.1250f,21.8750f,-2.1250f,21.8750f,-2.0208f,21.8262f,-2.0208f,21.8262f,-1.9167f,21.7774f,-1.9167f,21.7774f,-1.8125f,21.7286f,-1.8125f,21.7286f,-1.7083f,
	21.6797f,-1.7083f,21.6797f,-1.6042f,21.6309f,-1.6042f,21.6309f,-1.5000f,21.5821f,-1.5000f,21.5821f,-1.3958f,21.5332f,-1.3958f,21.5332f,-1.2917f,21.4844f,-1.2917f,21.4844f,-1.1875f,21.4356f,-1.1875f,21.4356f,-1.0833f,21.3868f,-1.0833f,21.3868f,-0.9792f,21.3379f,-0.9792f,
	21.3379f,-0.8750f,21.2891f,-0.8750f,21.2891f,-0.7708f,21.2403f,-0.7708f,21.2403f,-0.6667f,21.1915f,-0.6667f,21.1915f,-0.5625f,21.1426f,-0.5625f,21.1426f,-0.4583f,21.0938f,-0.4583f,21.0938f,-0.3542f,21.0450f,-0.3542f,21.0450f,-0.2500f,20.9961f,-0.2500f,20.9961f,-0.1458f,
	20.9473f,-0.1458f,20.9473f,-0.0417f,20.8985f,-0.0417f,20.8985f,0.0625f,20.8497f,0.0625f,20.8497f,0.1667f,20.8008f,0.1667f,20.8008f,0.2708f,20.7520f,0.2708f,20.7520f,0.3750f,20.7032f,0.3750f,20.7032f,0.4792f,20.6543f,0.4792f,20.6543f,0.5833f,20.6055f,0.5833f,
	20.6055f,0.6875f,20.5567f,0.6875f,20.5567f,0.7917f,20.5079f,0.7917f,20.5079f,0.8958f,20.4590f,0.8958f,20.4590f,1.0000f,20.4102f,1.0000f,22.8028f,0.6875f,22.7540f,0.7917f,22.8028f,0.7917f,22.7540f,0.8958f,22.8028f,0.8958f,22.7540f,1.0000f,22.8028f,1.0000f,
	22.7540f,-4.0000f,22.7051f,-4.0000f,22.7051f,-3.8958f,22.6563f,-3.8958f,22.6563f,-3.7917f,22.6075f,-3.7917f,22.6075f,-3.6875f,22.5586f,-3.6875f,22.5586f,-3.5833f,22.5098f,-3.5833f,22.5098f,-3.4792f,22.4610f,-3.4792f,22.4610f,-3.3750f,22.4122f,-3.3750f,22.4122f,-3.2708f,
	22.3633f,-3.2708f,22.3633f,-3.1667f,22.3145f,-3.1667f,22.3145f,-3.0625f,22.2657f,-3.0625f,22.2657f,-2.9583f,22.2168f,-2.9583f,22.2168f,-2.8542f,22.1680f,-2.8542f,22.1680f,-2.7500f,22.1192f,-2.7500f,22.1192f,-2.6458f,22.0704f,-2.6458f,22.0704f,-2.5417f,22.0215f,-2.5417f,
	22.0215f,-2.4375f,21.9727f,-2.4375f,21.9727f,-2.3333f,21.9239f,-2.3333f,21.9239f,-2.2292f,21.8750f,-2.2292f,21.8750f,-2.1250f,21.8262f,-2.1250f,21.8262f,-2.0208f,21.7774f,-2.0208f,21.7774f,-1.9167f,21.7286f,-1.9167f,21.7286f,-1.8125f,21.6797f,-1.8125f,21.6797f,-1.7083f,
	21.6309f,-1.7083f,21.6309f,-1.6042f,21.5821f,-1.6042f,21.5821f,-1.5000f,21.5332f,-1.5000f,21.5332f,-1.3958f,21.4844f,-1.3958f,21.4844f,-1.2917f,21.4356f,-1.2917f,21.4356f,-1.1875f,21.3868f,-1.1875f,21.3868f,-1.0833f,21.3379f,-1.0833f,21.3379f,-0.9792f,21.2891f,-0.9792f,
	21.2891f,-0.8750f,21.2403f,-0.8750f,21.2403f,-0.7708f,21.1915f,-0.7708f,21.1915f,-0.6667f,21.1426f,-0.6667f,21.1426f,-0.5625f,21.0938f,-0.5625f,21.0938f,-0.4583f,21.0450f,-0.4583f,21.0450f,-0.3542f,20.9961f,-0.3542f,20.9961f,-0.2500f,20.9473f,-0.2500f,20.9473f,-0.1458f,
	20.8985f,-0.1458f,20.8985f,-0.0417f,20.8497f,-0.0417f,20.8497f,0.0625f,20.8008f,0.0625f,20.8008f,0.1667f,20.7520f,0.1667f,20.7520f,0.2708f,20.7032f,0.2708f,20.7032f,0.3750f,20.6543f,0.3750f,20.6543f,0.4792f,20.6055f,0.4792f,20.6055f,0.5833f,20.5567f,0.5833f,
	20.5567f,0.6875f,20.5079f,0.6875f,20.5079f,0.7917f,20.4590f,0.7917f,20.4590f,0.8958f,20.4102f,0.8958f,20.4102f,1.0000f,20.3614f,1.0000f,22.7540f,0.7917f,22.7051f,0.8958f,22.7540f,0.8958f,22.7051f,1.0000f,22.7540f,1.0000f,22.7051f,-4.0000f,22.6563f,-4.0000f,
	22.6563f,-3.8958f,22.6075f,-3.8958f,22.6075f,-3.7917f,22.5586f,-3.7917f,22.5586f,-3.6875f,22.5098f,-3.6875f,22.5098f,-3.5833f,22.4610f,-3.5833f,22.4610f,-3.4792f,22.4122f,-3.4792f,22.4122f,-3.3750f,22.3633f,-3.3750f,22.3633f,-3.2708f,22.3145f,-3.2708f,22.3145f,-3.1667f,
	22.2657f,-3.1667f,22.2657f,-3.0625f,22.2168f,-3.0625f,22.2168f,-2.9583f,22.1680f,-2.9583f,22.1680f,-2.8542f,22.1192f,-2.8542f,22.1192f,-2.7500f,22.0704f,-2.7500f,22.0704f,-2.6458f,22.0215f,-2.6458f,22.0215f,-2.5417f,21.9727f,-2.5417f,21.9727f,-2.4375f,21.9239f,-2.4375f,
	21.9239f,-2.3333f,21.8750f,-2.3333f,21.8750f,-2.2292f,21.8262f,-2.2292f,21.8262f,-2.1250f,21.7774f,-2.1250f,21.7774f,-2.0208f,21.7286f,-2.0208f,21.7286f,-1.9167f,21.6797f,-1.9167f,21.6797f,-1.8125f,21.6309f,-1.8125f,21.6309f,-1.7083f,21.5821f,-1.7083f,21.5821f,-1.6042f,
	21.5332f,-1.6042f,21.5332f,-1.5000f,21.4844f,-1.5000f,21.4844f,-1.3958f,21.4356f,-1.3958f,21.4356f,-1.2917f,21.3868f,-1.2917f,21.3868f,-1.1875f,21.3379f,-1.1875f,21.3379f,-1.0833f,21.2891f,-1.0833f,21.2891f,-0.9792f,21.2403f,-0.9792f,21.2403f,-0.8750f,21.1915f,-0.8750f,
	21.1915f,-0.7708f,21.1426f,-0.7708f,21.1426f,-0.6667f,21.0938f,-0.6667f,21.0938f,-0.5625f,21.0450f,-0.5625f,21.0450f,-0.4583f,20.9961f,-0.4583f,20.9961f,-0.3542f,20.9473f,-0.3542f,20.9473f,-0.2500f,20.8985f,-0.2500f,20.8985f,-0.1458f,20.8497f,-0.1458f,20.8497f,-0.0417f,
	20.8008f,-0.0417f,20.8008f,0.0625f,20.7520f,0.0625f,20.7520f,0.1667f,20.7032f,0.1667f,20.7032f,0.2708f,20.6543f,0.2708f,20.6543f,0.3750f,20.6055f,0.3750f,20.6055f,0.4792f,20.5567f,0.4792f,20.5567f,0.5833f,20.5079f,0.5833f,20.5079f,0.6875f,20.4590f,0.6875f,
	20.4590f,0.7917f,20.4102f,0.7917f,20.4102f,0.8958f,20.3614f,0.8958f,20.3614f,1.0000f,20.3125f,1.0000f,22.7051f,0.8958f,22.6563f,1.0000f,22.7051f,1.0000f,22.6563f,-4.0000f,22.6075f,-4.0000f,22.6075f,-3.8958f,22.5586f,-3.8958f,22.5586f,-3.7917f,22.5098f,-3.7917f,
	22.5098f,-3.6875f,22.4610f,-3.6875f,22.4610f,-3.5833f,22.4122f,-3.5833f,22.4122f,-3.4792f,22.3633f,-3.4792f,22.3633f,-3.3750f,22.3145f,-3.3750f,22.3145f,-3.2708f,22.2657f,-3.2708f,22.2657f,-3.1667f,22.2168f,-3.1667f,22.2168f,-3.0625f,22.1680f,-3.0625f,22.1680f,-2.9583f,
	22.1192f,-2.9583f,22.1192f,-2.8542f,22.0704f,-2.8542f,22.0704f,-2.7500f,22.0215f,-2.7500f,22.0215f,-2.6458f,21.9727f,-2.6458f,21.9727f,-2.5417f,21.9239f,-2.5417f,21.9239f,-2.4375f,21.8750f,-2.4375f,21.8750f,-2.3333f,21.8262f,-2.3333f,21.8262f,-2.2292f,21.7774f,-2.2292f,
	21.7774f,-2.1250f,21.7286f,-2.1250f,21.7286f,-2.0208f,21.6797f,-2.0208f,21.6797f,-1.9167f,21.6309f,-1.9167f,21.6309f,-1.8125f,21.5821f,-1.8125f,21.5821f,-1.7083f,21.5332f,-1.7083f,21.5332f,-1.6042f,21.4844f,-1.6042f,21.4844f,-1.5000f,21.4356f,-1.5000f,21.4356f,-1.3958f,
	21.3868f,-1.3958f,21.3868f,-1.2917f,21.3379f,-1.2917f,21.3379f,-1.1875f,21.2891f,-1.1875f,21.2891f,-1.0833f,21.2403f,-1.0833f,21.2403f,-0.9792f,21.1915f,-0.9792f,21.1915f,-0.8750f,21.1426f,-0.8750f,21.1426f,-0.7708f,21.0938f,-0.7708f,21.0938f,-0.6667f,21.0450f,-0.6667f,
	21.0450f,-0.5625f,20.9961f,-0.5625f,20.9961f,-0.4583f,20.9473f,-0.4583f,20.9473f,-0.3542f,20.8985f,-0.3542f,20.8985f,-0.2500f,20.8497f,-0.2500f,20.8497f,-0.1458f,20.8008f,-0.1458f,20.8008f,-0.0417f,20.7520f,-0.0417f,20.7520f,0.0625f,20.7032f,0.0625f,20.7032f,0.1667f,
	20.6543f,0.1667f,20.6543f,0.2708f,20.6055f,0.2708f,20.6055f,0.3750f,20.5567f,0.3750f,20.5567f,0.4792f,20.5079f,0.4792f,20.5079f,0.5833f,20.4590f,0.5833f,20.4590f,0.6875f,20.4102f,0.6875f,20.4102f,0.7917f,20.3614f,0.7917f,20.3614f,0.8958f,20.3125f,0.8958f,
	20.3125f,1.0000f,20.2637f,1.0000f,22.6075f,-4.0000f,22.5586f,-4.0000f,22.5586f,-3.8958f,22.5098f,-3.8958f,22.5098f,-3.7917f,22.4610f,-3.7917f,22.4610f,-3.6875f,22.4122f,-3.6875f,22.4122f,-3.5833f,22.3633f,-3.5833f,22.3633f,-3.4792f,22.3145f,-3.4792f,22.3145f,-3.3750f,
	22.2657f,-3.3750f,22.2657f,-3.2708f,22.2168f,-3.2708f,22.2168f,-3.1667f,22.1680f,-3.1667f,22.1680f,-3.0625f,22.1192f,-3.0625f,22.1192f,-2.9583f,22.0704f,-2.9583f,22.0704f,-2.8542f,22.0215f,-2.8542f,22.0215f,-2.7500f,21.9727f,-2.7500f,21.9727f,-2.6458f,21.9239f,-2.6458f,
	21.9239f,-2.5417f,21.8750f,-2.5417f,21.8750f,-2.4375f,21.8262f,-2.4375f,21.8262f,-2.3333f,21.7774f,-2.3333f,21.7774f,-2.2292f,21.7286f,-2.2292f,21.7286f,-2.1250f,21.6797f,-2.1250f,21.6797f,-2.0208f,21.6309f,-2.0208f,21.6309f,-1.9167f,21.5821f,-1.9167f,21.5821f,-1.8125f,
	21.5332f,-1.8125f,21.5332f,-1.7083f,21.4844f,-1.7083f,21.4844f,-1.6042f,21.4356f,-1.6042f,21.4356f,-1.5000f,21.3868f,-1.5000f,21.3868f,-1.3958f,21.3379f,-1.3958f,21.3379f,-1.2917f,21.2891f,-1.2917f,21.2891f,-1.1875f,21.2403f,-1.1875f,21.2403f,-1.0833f,21.1915f,-1.0833f,
	21.1915f,-0.9792f,21.1426f,-0.9792f,21.1426f,-0.8750f,21.0938f,-0.8750f,21.0938f,-0.7708f,21.0450f,-0.7708f,21.0450f,-0.6667f,20.9961f,-0.6667f,20.9961f,-0.5625f,20.9473f,-0.5625f,20.9473f,-0.4583f,20.8985f,-0.4583f,20.8985f,-0.3542f,20.8497f,-0.3542f,20.8497f,-0.2500f,
	20.8008f,-0.2500f,20.8008f,-0.1458f,20.7520f,-0.1458f,20.7520f,-0.0417f,20.7032f,-0.0417f,20.7032f,0.0625f,20.6543f,0.0625f,20.6543f,0.1667f,20.6055f,0.1667f,20.6055f,0.2708f,20.5567f,0.2708f,20.5567f,0.3750f,20.5079f,0.3750f,20.5079f,0.4792f,20.4590f,0.4792f,
	20.4590f,0.5833f,20.4102f,0.5833f,20.4102f,0.6875f,20.3614f,0.6875f,20.3614f,0.7917f,20.3125f,0.7917f,20.3125f,0.8958f,20.2637f,0.8958f,20.2637f,1.0000f,20.2149f,1.0000f,22.5586f,-4.0000f,22.5098f,-4.0000f,22.5098f,-3.8958f,22.4610f,-3.8958f,22.4610f,-3.7917f,
	22.4122f,-3.7917f,22.4122f,-3.6875f,22.3633f,-3.6875f,22.3633f,-3.5833f,22.3145f,-3.5833f,22.3145f,-3.4792f,22.2657f,-3.4792f,22.2657f,-3.3750f,22.2168f,-3.3750f,22.2168f,-3.2708f,22.1680f,-3.2708f,22.1680f,-3.1667f,22.1192f,-3.1667f,22.1192f,-3.0625f,22.0704f,-3.0625f,
	22.0704f,-2.9583f,22.0215f,-2.9583f,22.0215f,-2.8542f,21.9727f,-2.8542f,21.9727f,-2.7500f,21.9239f,-2.7500f,21.9239f,-2.6458f,21.8750f,-2.6458f,21.8750f,-2.5417f,21.8262f,-2.5417f,21.8262f,-2.4375f,21.7774f,-2.4375f,21.7774f,-2.3333f,21.7286f,-2.3333f,21.7286f,-2.2292f,
	21.6797f,-2.2292f,21.6797f,-2.1250f,21.6309f,-2.1250f,21.6309f,-2.0208f,21.5821f,-2.0208f,21.5821f,-1.9167f,21.5332f,-1.9167f,21.5332f,-1.8125f,21.4844f,-1.8125f,21.4844f,-1.7083f,21.4356f,-1.7083f,21.4356f,-1.6042f,21.3868f,-1.6042f,21.3868f,-1.5000f,21.3379f,-1.5000f,
	21.3379f,-1.3958f,21.2891f,-1.3958f,21.2891f,-1.2917f,21.2403f,-1.2917f,21.2403f,-1.1875f,21.1915f,-1.1875f,21.1915f,-1.0833f,21.1426f,-1.0833f,21.1426f,-0.9792f,21.0938f,-0.9792f,21.0938f,-0.8750f,21.0450f,-0.8750f,21.0450f,-0.7708f,20.9961f,-0.7708f,20.9961f,-0.6667f,
	20.9473f,-0.6667f,20.9473f,-0.5625f,20.8985f,-0.5625f,20.8985f,-0.4583f,20.8497f,-0.4583f,20.8497f,-0.3542f,20.8008f,-0.3542f,20.8008f,-0.2500f,20.7520f,-0.2500f,20.7520f,-0.1458f,20.7032f,-0.1458f,20.7032f,-0.0417f,20.6543f,-0.0417f,20.6543f,0.0625f,20.6055f,0.0625f,
	20.6055f,0.1667f,20.5567f,0.1667f,20.5567f,0.2708f,20.5079f,0.2708f,20.5079f,0.3750f,20.4590f,0.3750f,20.4590f,0.4792f,20.4102f,0.4792f,20.4102f,0.5833f,20.3614f,0.5833f,20.3614f,0.6875f,20.3125f,0.6875f,20.3125f,0.7917f,20.2637f,0.7917f,20.2637f,0.8958f,
	20.2149f,0.8958f,20.2149f,1.0000f,20.1661f,1.0000f,22.5098f,-4.0000f,22.4610f,-4.0000f,22.4610f,-3.8958f,22.4122f,-3.8958f,22.4122f,-3.7917f,22.3633f,-3.7917f,22.3633f,-3.6875f,22.3145f,-3.6875f,22.3145f,-3.5833f,22.2657f,-3.5833f,22.2657f,-3.4792f,22.2168f,-3.4792f,
	22.2168f,-3.3750f,22.1680f,-3.3750f,22.1680f,-3.2708f,22.1192f,-3.2708f,22.1192f,-3.1667f,22.0704f,-3.1667f,22.0704f,-3.0625f,22.0215f,-3.0625f,22.0215f,-2.9583f,21.9727f,-2.9583f,21.9727f,-2.8542f,21.9239f,-2.8542f,21.9239f,-2.7500f,21.8750f,-2.7500f,21.8750f,-2.6458f,
	21.8262f,-2.6458f,21.8262f,-2.5417f,21.7774f,-2.5417f,21.7774f,-2.4375f,21.7286f,-2.4375f,21.7286f,-2.3333f,21.6797f,-2.3333f,21.6797f,-2.2292f,21.6309f,-2.2292f,21.6309f,-2.1250f,21.5821f,-2.1250f,21.5821f,-2.0208f,21.5332f,-2.0208f,21.5332f,-1.9167f,21.4844f,-1.9167f,
	21.4844f,-1.8125f,21.4356f,-1.8125f,21.4356f,-1.7083f,21.3868f,-1.7083f,21.3868f,-1.6042f,21.3379f,-1.6042f,21.3379f,-1.5000f,21.2891f,-1.5000f,21.2891f,-1.3958f,21.2403f,-1.3958f,21.2403f,-1.2917f,21.1915f,-1.2917f,21.1915f,-1.1875f,21.1426f,-1.1875f,21.1426f,-1.0833f,
	21.0938f,-1.0833f,21.0938f,-0.9792f,21.0450f,-0.9792f,21.0450f,-0.8750f,20.9961f,-0.8750f,20.9961f,-0.7708f,20.9473f,-0.7708f,20.9473f,-0.6667f,20.8985f,-0.6667f,20.8985f,-0.5625f,20.8497f,-0.5625f,20.8497f,-0.4583f,20.8008f,-0.4583f,20.8008f,-0.3542f,20.7520f,-0.3542f,
	20.7520f,-0.2500f,20.7032f,-0.2500f,20.7032f,-0.1458f,20.6543f,-0.1458f,20.6543f,-0.0417f,20.6055f,-0.0417f,20.6055f,0.0625f,20.5567f,0.0625f,20.5567f,0.1667f,20.5079f,0.1667f,20.5079f,0.2708f,20.4590f,0.2708f,20.4590f,0.3750f,20.4102f,0.3750f,20.4102f,0.4792f,
	20.3614f,0.4792f,20.3614f,0.5833f,20.3125f,0.5833f,20.3125f,0.6875f,20.2637f,0.6875f,20.2637f,0.7917f,20.2149f,0.7917f,20.2149f,0.8958f,20.1661f,0.8958f,20.1661f,1.0000f,20.1172f,1.0000f,22.4610f,-4.0000f,22.4122f,-4.0000f,22.4122f,-3.8958f,22.3633f,-3.8958f,
	22.3633f,-3.7917f,22.3145f,-3.7917f,22.3145f,-3.6875f,22.2657f,-3.6875f,22.2657f,-3.5833f,22.2168f,-3.5833f,22.2168f,-3.4792f,22.1680f,-3.4792f,22.1680f,-3.3750f,22.1192f,-3.3750f,22.1192f,-3.2708f,22.0704f,-3.2708f,22.0704f,-3.1667f,22.0215f,-3.1667f,22.0215f,-3.0625f,
	21.9727f,-3.0625f,21.9727f,-2.9583f,21.9239f,-2.9583f,21.9239f,-2.8542f,21.8750f,-2.8542f,21.8750f,-2.7500f,21.8262f,-2.7500f,21.8262f,-2.6458f,21.7774f,-2.6458f,21.7774f,-2.5417f,21.7286f,-2.5417f,21.7286f,-2.4375f,21.6797f,-2.4375f,21.6797f,-2.3333f,21.6309f,-2.3333f,
	21.6309f,-2.2292f,21.5821f,-2.2292f,21.5821f,-2.1250f,21.5332f,-2.1250f,21.5332f,-2.0208f,21.4844f,-2.0208f,21.4844f,-1.9167f,21.4356f,-1.9167f,21.4356f,-1.8125f,21.3868f,-1.8125f,21.3868f,-1.7083f,21.3379f,-1.7083f,21.3379f,-1.6042f,21.2891f,-1.6042f,21.2891f,-1.5000f,
	21.2403f,-1.5000f,21.2403f,-1.3958f,21.1915f,-1.3958f,21.1915f,-1.2917f,21.1426f,-1.2917f,21.1426f,-1.1875f,21.0938f,-1.1875f,21.0938f,-1.0833f,21.0450f,-1.0833f,21.0450f,-0.9792f,20.9961f,-0.9792f,20.9961f,-0.8750f,20.9473f,-0.8750f,20.9473f,-0.7708f,20.8985f,-0.7708f,
	20.8985f,-0.6667f,20.8497f,-0.6667f,20.8497f,-0.5625f,20.8008f,-0.5625f,20.8008f,-0.4583f,20.7520f,-0.4583f,20.7520f,-0.3542f,20.7032f,-0.3542f,20.7032f,-0.2500f,20.6543f,-0.2500f,20.6543f,-0.1458f,20.6055f,-0.1458f,20.6055f,-0.0417f,20.5567f,-0.0417f,20.5567f,0.0625f,
	20.5079f,0.0625f,20.5079f,0.1667f,20.4590f,0.1667f,20.4590f,0.2708f,20.4102f,0.2708f,20.4102f,0.3750f,20.3614f,0.3750f,20.3614f,0.4792f,20.3125f,0.4792f,20.3125f,0.5833f,20.2637f,0.5833f,20.2637f,0.6875f,20.2149f,0.6875f,20.2149f,0.7917f,20.1661f,0.7917f,
	20.1661f,0.8958f,20.1172f,0.8958f,20.1172f,1.0000f,20.0684f,1.0000f,22.4122f,-4.0000f,22.3633f,-4.0000f,22.3633f,-3.8958f,22.3145f,-3.8958f,22.3145f,-3.7917f,22.2657f,-3.7917f,22.2657f,-3.6875f,22.2168f,-3.6875f,22.2168f,-3.5833f,22.1680f,-3.5833f,22.1680f,-3.4792f,
	22.1192f,-3.4792f,22.1192f,-3.3750f,22.0704f,-3.3750f,22.0704f,-3.2708f,22.0215f,-3.2708f,22.0215f,-3.1667f,21.9727f,-3.1667f,21.9727f,-3.0625f,21.9239f,-3.0625f,21.9239f,-2.9583f,21.8750f,-2.9583f,21.8750f,-2.8542f,21.8262f,-2.8542f,21.8262f,-2.7500f,21.7774f,-2.7500f,
	21.7774f,-2.6458f,21.7286f,-2.6458f,21.7286f,-2.5417f,21.6797f,-2.5417f,21.6797f,-2.4375f,21.6309f,-2.4375f,21.6309f,-2.3333f,21.5821f,-2.3333f,21.5821f,-2.2292f,21.5332f,-2.2292f,21.5332f,-2.1250f,21.4844f,-2.1250f,21.4844f,-2.0208f,21.4356f,-2.0208f,21.4356f,-1.9167f,
	21.3868f,-1.9167f,21.3868f,-1.8125f,21.3379f,-1.8125f,21.3379f,-1.7083f,21.2891f,-1.7083f,21.2891f,-1.6042f,21.2403f,-1.6042f,21.2403f,-1.5000f,21.1915f,-1.5000f,21.1915f,-1.3958f,21.1426f,-1.3958f,21.1426f,-1.2917f,21.0938f,-1.2917f,21.0938f,-1.1875f,21.0450f,-1.1875f,
	21.0450f,-1.0833f,20.9961f,-1.0833f,20.9961f,-0.9792f,20.9473f,-0.9792f,20.9473f,-0.8750f,20.8985f,-0.8750f,20.8985f,-0.7708f,20.8497f,-0.7708f,20.8497f,-0.6667f,20.8008f,-0.6667f,20.8008f,-0.5625f,20.7520f,-0.5625f,20.7520f,-0.4583f,20.7032f,-0.4583f,20.7032f,-0.3542f,
	20.6543f,-0.3542f,20.6543f,-0.2500f,20.6055f,-0.2500f,20.6055f,-0.1458f,20.5567f,-0.1458f,20.5567f,-0.0417f,20.5079f,-0.0417f,20.5079f,0.0625f,20.4590f,0.0625f,20.4590f,0.1667f,20.4102f,0.1667f,20.4102f,0.2708f,20.3614f,0.2708f,20.3614f,0.3750f,20.3125f,0.3750f,
	20.3125f,0.4792f,20.2637f,0.4792f,20.2637f,0.5833f,20.2149f,0.5833f,20.2149f,0.6875f,20.1661f,0.6875f,20.1661f,0.7917f,20.1172f,0.7917f,20.1172f,0.8958f,20.0684f,0.8958f,20.0684f,1.0000f,20.0196f,1.0000f,22.3633f,-4.0000f,22.3145f,-4.0000f,22.3145f,-3.8958f,
	22.2657f,-3.8958f,22.2657f,-3.7917f,22.2168f,-3.7917f,22.2168f,-3.6875f,22.1680f,-3.6875f,22.1680f,-3.5833f,22.1192f,-3.5833f,22.1192f,-3.4792f,22.0704f,-3.4792f,22.0704f,-3.3750f,22.0215f,-3.3750f,22.0215f,-3.2708f,21.9727f,-3.2708f,21.9727f,-3.1667f,21.9239f,-3.1667f,
	21.9239f,-3.0625f,21.8750f,-3.0625f,21.8750f,-2.9583f,21.8262f,-2.9583f,21.8262f,-2.8542f,21.7774f,-2.8542f,21.7774f,-2.7500f,21.7286f,-2.7500f,21.7286f,-2.6458f,21.6797f,-2.6458f,21.6797f,-2.5417f,21.6309f,-2.5417f,21.6309f,-2.4375f,21.5821f,-2.4375f,21.5821f,-2.3333f,
	21.5332f,-2.3333f,21.5332f,-2.2292f,21.4844f,-2.2292f,21.4844f,-2.1250f,21.4356f,-2.1250f,21.4356f,-2.0208f,21.3868f,-2.0208f,21.3868f,-1.9167f,21.3379f,-1.9167f,21.3379f,-1.8125f,21.2891f,-1.8125f,21.2891f,-1.7083f,21.2403f,-1.7083f,21.2403f,-1.6042f,21.1915f,-1.6042f,
	21.1915f,-1.5000f,21.1426f,-1.5000f,21.1426f,-1.3958f,21.0938f,-1.3958f,21.0938f,-1.2917f,21.0450f,-1.2917f,21.0450f,-1.1875f,20.9961f,-1.1875f,20.9961f,-1.0833f,20.9473f,-1.0833f,20.9473f,-0.9792f,20.8985f,-0.9792f,20.8985f,-0.8750f,20.8497f,-0.8750f,20.8497f,-0.7708f,
	20.8008f,-0.7708f,20.8008f,-0.6667f,20.7520f,-0.6667f,20.7520f,-0.5625f,20.7032f,-0.5625f,20.7032f,-0.4583f,20.6543f,-0.4583f,20.6543f,-0.3542f,20.6055f,-0.3542f,20.6055f,-0.2500f,20.5567f,-0.2500f,20.5567f,-0.1458f,20.5079f,-0.1458f,20.5079f,-0.0417f,20.4590f,-0.0417f,
	20.4590f,0.0625f,20.4102f,0.0625f,20.4102f,0.1667f,20.3614f,0.1667f,20.3614f,0.2708f,20.3125f,0.2708f,20.3125f,0.3750f,20.2637f,0.3750f,20.2637f,0.4792f,20.2149f,0.4792f,20.2149f,0.5833f,20.1661f,0.5833f,20.1661f,0.6875f,20.1172f,0.6875f,20.1172f,0.7917f,
	20.0684f,0.7917f,20.0684f,0.8958f,20.0196f,0.8958f,20.0196f,1.0000f,19.9707f,1.0000f,22.3145f,-4.0000f,22.2657f,-4.0000f,22.2657f,-3.8958f,22.2168f,-3.8958f,22.2168f,-3.7917f,22.1680f,-3.7917f,22.1680f,-3.6875f,22.1192f,-3.6875f,22.1192f,-3.5833f,22.0704f,-3.5833f,
	22.0704f,-3.4792f,22.0215f,-3.4792f,22.0215f,-3.3750f,21.9727f,-3.3750f,21.9727f,-3.2708f,21.9239f,-3.2708f,21.9239f,-3.1667f,21.8750f,-3.1667f,21.8750f,-3.0625f,21.8262f,-3.0625f,21.8262f,-2.9583f,21.7774f,-2.9583f,21.7774f,-2.8542f,21.7286f,-2.8542f,21.7286f,-2.7500f,
	21.6797f,-2.7500f,21.6797f,-2.6458f,21.6309f,-2.6458f,21.6309f,-2.5417f,21.5821f,-2.5417f,21.5821f,-2.4375f,21.5332f,-2.4375f,21.5332f,-2.3333f,21.4844f,-2.3333f,21.4844f,-2.2292f,21.4356f,-2.2292f,21.4356f,-2.1250f,21.3868f,-2.1250f,21.3868f,-2.0208f,21.3379f,-2.0208f,
	21.3379f,-1.9167f,21.2891f,-1.9167f,21.2891f,-1.8125f,21.2403f,-1.8125f,21.2403f,-1.7083f,21.1915f,-1.7083f,21.1915f,-1.6042f,21.1426f,-1.6042f,21.1426f,-1.5000f,21.0938f,-1.5000f,21.0938f,-1.3958f,21.0450f,-1.3958f,21.0450f,-1.2917f,20.9961f,-1.2917f,20.9961f,-1.1875f,
	20.9473f,-1.1875f,20.9473f,-1.0833f,20.8985f,-1.0833f,20.8985f,-0.9792f,20.8497f,-0.9792f,20.8497f,-0.8750f,20.8008f,-0.8750f,20.8008f,-0.7708f,20.7520f,-0.7708f,20.7520f,-0.6667f,20.7032f,-0.6667f,20.7032f,-0.5625f,20.6543f,-0.5625f,20.6543f,-0.4583f,20.6055f,-0.4583f,
	20.6055f,-0.3542f,20.5567f,-0.3542f,20.5567f,-0.2500f,20.5079f,-0.2500f,20.5079f,-0.1458f,20.4590f,-0.1458f,20.4590f,-0.0417f,20.4102f,-0.0417f,20.4102f,0.0625f,20.3614f,0.0625f,20.3614f,0.1667f,20.3125f,0.1667f,20.3125f,0.2708f,20.2637f,0.2708f,20.2637f,0.3750f,
	20.2149f,0.3750f,20.2149f,0.4792f,20.1661f,0.4792f,20.1661f,0.5833f,20.1172f,0.5833f,20.1172f,0.6875f,20.0684f,0.6875f,20.0684f,0.7917f,20.0196f,0.7917f,20.0196f,0.8958f,19.9707f,0.8958f,19.9707f,1.0000f,19.9219f,1.0000f,22.2657f,-4.0000f,22.2168f,-4.0000f,
	22.2168f,-3.8958f,22.1680f,-3.8958f,22.1680f,-3.7917f,22.1192f,-3.7917f,22.1192f,-3.6875f,22.0704f,-3.6875f,22.0704f,-3.5833f,22.0215f,-3.5833f,22.0215f,-3.4792f,21.9727f,-3.4792f,21.9727f,-3.3750f,21.9239f,-3.3750f,21.9239f,-3.2708f,21.8750f,-3.2708f,21.8750f,-3.1667f,
	21.8262f,-3.1667f,21.8262f,-3.0625f,21.7774f,-3.0625f,21.7774f,-2.9583f,21.7286f,-2.9583f,21.7286f,-2.8542f,21.6797f,-2.8542f,21.6797f,-2.7500f,21.6309f,-2.7500f,21.6309f,-2.6458f,21.5821f,-2.6458f,21.5821f,-2.5417f,21.5332f,-2.5417f,21.5332f,-2.4375f,21.4844f,-2.4375f,
	21.4844f,-2.3333f,21.4356f,-2.3333f,21.4356f,-2.2292f,21.3868f,-2.2292f,21.3868f,-2.1250f,21.3379f,-2.1250f,21.3379f,-2.0208f,21.2891f,-2.0208f,21.2891f,-1.9167f,21.2403f,-1.9167f,21.2403f,-1.8125f,21.1915f,-1.8125f,21.1915f,-1.7083f,21.1426f,-1.7083f,21.1426f,-1.6042f,
	21.0938f,-1.6042f,21.0938f,-1.5000f,21.0450f,-1.5000f,21.0450f,-1.3958f,20.9961f,-1.3958f,20.9961f,-1.2917f,20.9473f,-1.2917f,20.9473f,-1.1875f,20.8985f,-1.1875f,20.8985f,-1.0833f,20.8497f,-1.0833f,20.8497f,-0.9792f,20.8008f,-0.9792f,20.8008f,-0.8750f,20.7520f,-0.8750f,
	20.7520f,-0.7708f,20.7032f,-0.7708f,20.7032f,-0.6667f,20.6543f,-0.6667f,20.6543f,-0.5625f,20.6055f,-0.5625f,20.6055f,-0.4583f,20.5567f,-0.4583f,20.5567f,-0.3542f,20.5079f,-0.3542f,20.5079f,-0.2500f,20.4590f,-0.2500f,20.4590f,-0.1458f,20.4102f,-0.1458f,20.4102f,-0.0417f,
	20.3614f,-0.0417f,20.3614f,0.0625f,20.3125f,0.0625f,20.3125f,0.1667f,20.2637f,0.1667f,20.2637f,0.2708f,20.2149f,0.2708f,20.2149f,0.3750f,20.1661f,0.3750f,20.1661f,0.4792f,20.1172f,0.4792f,20.1172f,0.5833f,20.0684f,0.5833f,20.0684f,0.6875f,20.0196f,0.6875f,
	20.0196f,0.7917f,19.9707f,0.7917f,19.9707f,0.8958f,19.9219f,0.8958f,19.9219f,1.0000f,19.8731f,1.0000f,22.2168f,-4.0000f,22.1680f,-4.0000f,22.1680f,-3.8958f,22.1192f,-3.8958f,22.1192f,-3.7917f,22.0704f,-3.7917f,22.0704f,-3.6875f,22.0215f,-3.6875f,22.0215f,-3.5833f,
	21.9727f,-3.5833f,21.9727f,-3.4792f,21.9239f,-3.4792f,21.9239f,-3.3750f,21.8750f,-3.3750f,21.8750f,-3.2708f,21.8262f,-3.2708f,21.8262f,-3.1667f,21.7774f,-3.1667f,21.7774f,-3.0625f,21.7286f,-3.0625f,21.7286f,-2.9583f,21.6797f,-2.9583f,21.6797f,-2.8542f,21.6309f,-2.8542f,
	21.6309f,-2.7500f,21.5821f,-2.7500f,21.5821f,-2.6458f,21.5332f,-2.6458f,21.5332f,-2.5417f,21.4844f,-2.5417f,21.4844f,-2.4375f,21.4356f,-2.4375f,21.4356f,-2.3333f,21.3868f,-2.3333f,21.3868f,-2.2292f,21.3379f,-2.2292f,21.3379f,-2.1250f,21.2891f,-2.1250f,21.2891f,-2.0208f,
	21.2403f,-2.0208f,21.2403f,-1.9167f,21.1915f,-1.9167f,21.1915f,-1.8125f,21.1426f,-1.8125f,21.1426f,-1.7083f,21.0938f,-1.7083f,21.0938f,-1.6042f,21.0450f,-1.6042f,21.0450f,-1.5000f,20.9961f,-1.5000f,20.9961f,-1.3958f,20.9473f,-1.3958f,20.9473f,-1.2917f,20.8985f,-1.2917f,
	20.8985f,-1.1875f,20.8497f,-1.1875f,20.8497f,-1.0833f,20.8008f,-1.0833f,20.8008f,-0.9792f,20.7520f,-0.9792f,20.7520f,-0.8750f,20.7032f,-0.8750f,20.7032f,-0.7708f,20.6543f,-0.7708f,20.6543f,-0.6667f,20.6055f,-0.6667f,20.6055f,-0.5625f,20.5567f,-0.5625f,20.5567f,-0.4583f,
	20.5079f,-0.4583f,20.5079f,-0.3542f,20.4590f,-0.3542f,20.4590f,-0.2500f,20.4102f,-0.2500f,20.4102f,-0.1458f,20.3614f,-0.1458f,20.3614f,-0.0417f,20.3125f,-0.0417f,20.3125f,0.0625f,20.2637f,0.0625f,20.2637f,0.1667f,20.2149f,0.1667f,20.2149f,0.2708f,20.1661f,0.2708f,
	20.1661f,0.3750f,20.1172f,0.3750f,20.1172f,0.4792f,20.0684f,0.4792f,20.0684f,0.5833f,20.0196f,0.5833f,20.0196f,0.6875f,19.9707f,0.6875f,19.9707f,0.7917f,19.9219f,0.7917f,19.9219f,0.8958f,19.8731f,0.8958f,19.8731f,1.0000f,19.8243f,1.0000f,22.1680f,-4.0000f,
	22.1192f,-4.0000f,22.1192f,-3.8958f,22.0704f,-3.8958f,22.0704f,-3.7917f,22.0215f,-3.7917f,22.0215f,-3.6875f,21.9727f,-3.6875f,21.9727f,-3.5833f,21.9239f,-3.5833f,21.9239f,-3.4792f,21.8750f,-3.4792f,21.8750f,-3.3750f,21.8262f,-3.3750f,21.8262f,-3.2708f,21.7774f,-3.2708f,
	21.7774f,-3.1667f,21.7286f,-3.1667f,21.7286f,-3.0625f,21.6797f,-3.0625f,21.6797f,-2.9583f,21.6309f,-2.9583f,21.6309f,-2.8542f,21.5821f,-2.8542f,21.5821f,-2.7500f,21.5332f,-2.7500f,21.5332f,-2.6458f,21.4844f,-2.6458f,21.4844f,-2.5417f,21.4356f,-2.5417f,21.4356f,-2.4375f,
	21.3868f,-2.4375f,21.3868f,-2.3333f,21.3379f,-2.3333f,21.3379f,-2.2292f,21.2891f,-2.2292f,21.2891f,-2.1250f,21.2403f,-2.1250f,21.2403f,-2.0208f,21.1915f,-2.0208f,21.1915f,-1.9167f,21.1426f,-1.9167f,21.1426f,-1.8125f,21.0938f,-1.8125f,21.0938f,-1.7083f,21.0450f,-1.7083f,
	21.0450f,-1.6042f,20.9961f,-1.6042f,20.9961f,-1.5000f,20.9473f,-1.5000f,20.9473f,-1.3958f,20.8985f,-1.3958f,20.8985f,-1.2917f,20.8497f,-1.2917f,20.8497f,-1.1875f,20.8008f,-1.1875f,20.8008f,-1.0833f,20.7520f,-1.0833f,20.7520f,-0.9792f,20.7032f,-0.9792f,20.7032f,-0.8750f,
	20.6543f,-0.8750f,20.6543f,-0.7708f,20.6055f,-0.7708f,20.6055f,-0.6667f,20.5567f,-0.6667f,20.5567f,-0.5625f,20.5079f,-0.5625f,20.5079f,-0.4583f,20.4590f,-0.4583f,20.4590f,-0.3542f,20.4102f,-0.3542f,20.4102f,-0.2500f,20.3614f,-0.2500f,20.3614f,-0.1458f,20.3125f,-0.1458f,
	20.3125f,-0.0417f,20.2637f,-0.0417f,20.2637f,0.0625f,20.2149f,0.0625f,20.2149f,0.1667f,20.1661f,0.1667f,20.1661f,0.2708f,20.1172f,0.2708f,20.1172f,0.3750f,20.0684f,0.3750f,20.0684f,0.4792f,20.0196f,0.4792f,20.0196f,0.5833f,19.9707f,0.5833f,19.9707f,0.6875f,
	19.9219f,0.6875f,19.9219f,0.7917f,19.8731f,0.7917f,19.8731f,0.8958f,19.8243f,0.8958f,19.8243f,1.0000f,19.7754f,1.0000f,22.1192f,-4.0000f,22.0704f,-4.0000f,22.0704f,-3.8958f,22.0215f,-3.8958f,22.0215f,-3.7917f,21.9727f,-3.7917f,21.9727f,-3.6875f,21.9239f,-3.6875f,
	21.9239f,-3.5833f,21.8750f,-3.5833f,21.8750f,-3.4792f,21.8262f,-3.4792f,21.8262f,-3.3750f,21.7774f,-3.3750f,21.7774f,-3.2708f,21.7286f,-3.2708f,21.7286f,-3.1667f,21.6797f,-3.1667f,21.6797f,-3.0625f,21.6309f,-3.0625f,21.6309f,-2.9583f,21.5821f,-2.9583f,21.5821f,-2.8542f,
	21.5332f,-2.8542f,21.5332f,-2.7500f,21.4844f,-2.7500f,21.4844f,-2.6458f,21.4356f,-2.6458f,21.4356f,-2.5417f,21.3868f,-2.5417f,21.3868f,-2.4375f,21.3379f,-2.4375f,21.3379f,-2.3333f,21.2891f,-2.3333f,21.2891f,-2.2292f,21.2403f,-2.2292f,21.2403f,-2.1250f,21.1915f,-2.1250f,
	21.1915f,-2.0208f,21.1426f,-2.0208f,21.1426f,-1.9167f,21.0938f,-1.9167f,21.0938f,-1.8125f,21.0450f,-1.8125f,21.0450f,-1.7083f,20.9961f,-1.7083f,20.9961f,-1.6042f,20.9473f,-1.6042f,20.9473f,-1.5000f,20.8985f,-1.5000f,20.8985f,-1.3958f,20.8497f,-1.3958f,20.8497f,-1.2917f,
	20.8008f,-1.2917f,20.8008f,-1.1875f,20.7520f,-1.1875f,20.7520f,-1.0833f,20.7032f,-1.0833f,20.7032f,-0.9792f,20.6543f,-0.9792f,20.6543f,-0.8750f,20.6055f,-0.8750f,20.6055f,-0.7708f,20.5567f,-0.7708f,20.5567f,-0.6667f,20.5079f,-0.6667f,20.5079f,-0.5625f,20.4590f,-0.5625f,
	20.4590f,-0.4583f,20.4102f,-0.4583f,20.4102f,-0.3542f,20.3614f,-0.3542f,20.3614f,-0.2500f,20.3125f,-0.2500f,20.3125f,-0.1458f,20.2637f,-0.1458f,20.2637f,-0.0417f,20.2149f,-0.0417f,20.2149f,0.0625f,20.1661f,0.0625f,20.1661f,0.1667f,20.1172f,0.1667f,20.1172f,0.2708f,
	20.0684f,0.2708f,20.0684f,0.3750f,20.0196f,0.3750f,20.0196f,0.4792f,19.9707f,0.4792f,19.9707f,0.5833f,19.9219f,0.5833f,19.9219f,0.6875f,19.8731f,0.6875f,19.8731f,0.7917f,19.8243f,0.7917f,19.8243f,0.8958f,19.7754f,0.8958f,19.7754f,1.0000f,19.7266f,1.0000f,
	22.0704f,-4.0000f,22.0215f,-4.0000f,22.0215f,-3.8958f,21.9727f,-3.8958f,21.9727f,-3.7917f,21.9239f,-3.7917f,21.9239f,-3.6875f,21.8750f,-3.6875f,21.8750f,-3.5833f,21.8262f,-3.5833f,21.8262f,-3.4792f,21.7774f,-3.4792f,21.7774f,-3.3750f,21.7286f,-3.3750f,21.7286f,-3.2708f,
	21.6797f,-3.2708f,21.6797f,-3.1667f,21.6309f,-3.1667f,21.6309f,-3.0625f,21.5821f,-3.0625f,21.5821f,-2.9583f,21.5332f,-2.9583f,21.5332f,-2.8542f,21.4844f,-2.8542f,21.4844f,-2.7500f,21.4356f,-2.7500f,21.4356f,-2.6458f,21.3868f,-2.6458f,21.3868f,-2.5417f,21.3379f,-2.5417f,
	21.3379f,-2.4375f,21.2891f,-2.4375f,21.2891f,-2.3333f,21.2403f,-2.3333f,21.2403f,-2.2292f,21.1915f,-2.2292f,21.1915f,-2.1250f,21.1426f,-2.1250f,21.1426f,-2.0208f,21.0938f,-2.0208f,21.0938f,-1.9167f,21.0450f,-1.9167f,21.0450f,-1.8125f,20.9961f,-1.8125f,20.9961f,-1.7083f,
	20.9473f,-1.7083f,20.9473f,-1.6042f,20.8985f,-1.6042f,20.8985f,-1.5000f,20.8497f,-1.5000f,20.8497f,-1.3958f,20.8008f,-1.3958f,20.8008f,-1.2917f,20.7520f,-1.2917f,20.7520f,-1.1875f,20.7032f,-1.1875f,20.7032f,-1.0833f,20.6543f,-1.0833f,20.6543f,-0.9792f,20.6055f,-0.9792f,
	20.6055f,-0.8750f,20.5567f,-0.8750f,20.5567f,-0.7708f,20.5079f,-0.7708f,20.5079f,-0.6667f,20.4590f,-0.6667f,20.4590f,-0.5625f,20.4102f,-0.5625f,20.4102f,-0.4583f,20.3614f,-0.4583f,20.3614f,-0.3542f,20.3125f,-0.3542f,20.3125f,-0.2500f,20.2637f,-0.2500f,20.2637f,-0.1458f,
	20.2149f,-0.1458f,20.2149f,-0.0417f,20.1661f,-0.0417f,20.1661f,0.0625f,20.1172f,0.0625f,20.1172f,0.1667f,20.0684f,0.1667f,20.0684f,0.2708f,20.0196f,0.2708f,20.0196f,0.3750f,19.9707f,0.3750f,19.9707f,0.4792f,19.9219f,0.4792f,19.9219f,0.5833f,19.8731f,0.5833f,
	19.8731f,0.6875f,19.8243f,0.6875f,19.8243f,0.7917f,19.7754f,0.7917f,19.7754f,0.8958f,19.7266f,0.8958f,19.7266f,1.0000f,19.6778f,1.0000f,22.0215f,-4.0000f,21.9727f,-4.0000f,21.9727f,-3.8958f,21.9239f,-3.8958f,21.9239f,-3.7917f,21.8750f,-3.7917f,21.8750f,-3.6875f,
	21.8262f,-3.6875f,21.8262f,-3.5833f,21.7774f,-3.5833f,21.7774f,-3.4792f,21.7286f,-3.4792f,21.7286f,-3.3750f,21.6797f,-3.3750f,21.6797f,-3.2708f,21.6309f,-3.2708f,21.6309f,-3.1667f,21.5821f,-3.1667f,21.5821f,-3.0625f,21.5332f,-3.0625f,21.5332f,-2.9583f,21.4844f,-2.9583f,
	21.4844f,-2.8542f,21.4356f,-2.8542f,21.4356f,-2.7500f,21.3868f,-2.7500f,21.3868f,-2.6458f,21.3379f,-2.6458f,21.3379f,-2.5417f,21.2891f,-2.5417f,21.2891f,-2.4375f,21.2403f,-2.4375f,21.2403f,-2.3333f,21.1915f,-2.3333f,21.1915f,-2.2292f,21.1426f,-2.2292f,21.1426f,-2.1250f,
	21.0938f,-2.1250f,21.0938f,-2.0208f,21.0450f,-2.0208f,21.0450f,-1.9167f,20.9961f,-1.9167f,20.9961f,-1.8125f,20.9473f,-1.8125f,20.9473f,-1.7083f,20.8985f,-1.7083f,20.8985f,-1.6042f,20.8497f,-1.6042f,20.8497f,-1.5000f,20.8008f,-1.5000f,20.8008f,-1.3958f,20.7520f,-1.3958f,
	20.7520f,-1.2917f,20.7032f,-1.2917f,20.7032f,-1.1875f,20.6543f,-1.1875f,20.6543f,-1.0833f,20.6055f,-1.0833f,20.6055f,-0.9792f,20.5567f,-0.9792f,20.5567f,-0.8750f,20.5079f,-0.8750f,20.5079f,-0.7708f,20.4590f,-0.7708f,20.4590f,-0.6667f,20.4102f,-0.6667f,20.4102f,-0.5625f,
	20.3614f,-0.5625f,20.3614f,-0.4583f,20.3125f,-0.4583f,20.3125f,-0.3542f,20.2637f,-0.3542f,20.2637f,-0.2500f,20.2149f,-0.2500f,20.2149f,-0.1458f,20.1661f,-0.1458f,20.1661f,-0.0417f,20.1172f,-0.0417f,20.1172f,0.0625f,20.0684f,0.0625f,20.0684f,0.1667f,20.0196f,0.1667f,
	20.0196f,0.2708f,19.9707f,0.2708f,19.9707f,0.3750f,19.9219f,0.3750f,19.9219f,0.4792f,19.8731f,0.4792f,19.8731f,0.5833f,19.8243f,0.5833f,19.8243f,0.6875f,19.7754f,0.6875f,19.7754f,0.7917f,19.7266f,0.7917f,19.7266f,0.8958f,19.6778f,0.8958f,19.6778f,1.0000f,
	19.6289f,1.0000f,21.9727f,-4.0000f,21.9239f,-4.0000f,21.9239f,-3.8958f,21.8750f,-3.8958f,21.8750f,-3.7917f,21.8262f,-3.7917f,21.8262f,-3.6875f,21.7774f,-3.6875f,21.7774f,-3.5833f,21.7286f,-3.5833f,21.7286f,-3.4792f,21.6797f,-3.4792f,21.6797f,-3.3750f,21.6309f,-3.3750f,
	21.6309f,-3.2708f,21.5821f,-3.2708f,21.5821f,-3.1667f,21.5332f,-3.1667f,21.5332f,-3.0625f,21.4844f,-3.0625f,21.4844f,-2.9583f,21.4356f,-2.9583f,21.4356f,-2.8542f,21.3868f,-2.8542f,21.3868f,-2.7500f,21.3379f,-2.7500f,21.3379f,-2.6458f,21.2891f,-2.6458f,21.2891f,-2.5417f,
	21.2403f,-2.5417f,21.2403f,-2.4375f,21.1915f,-2.4375f,21.1915f,-2.3333f,21.1426f,-2.3333f,21.1426f,-2.2292f,21.0938f,-2.2292f,21.0938f,-2.1250f,21.0450f,-2.1250f,21.0450f,-2.0208f,20.9961f,-2.0208f,20.9961f,-1.9167f,20.9473f,-1.9167f,20.9473f,-1.8125f,20.8985f,-1.8125f,
	20.8985f,-1.7083f,20.8497f,-1.7083f,20.8497f,-1.6042f,20.8008f,-1.6042f,20.8008f,-1.5000f,20.7520f,-1.5000f,20.7520f,-1.3958f,20.7032f,-1.3958f,20.7032f,-1.2917f,20.6543f,-1.2917f,20.6543f,-1.1875f,20.6055f,-1.1875f,20.6055f,-1.0833f,20.5567f,-1.0833f,20.5567f,-0.9792f,
	20.5079f,-0.9792f,20.5079f,-0.8750f,20.4590f,-0.8750f,20.4590f,-0.7708f,20.4102f,-0.7708f,20.4102f,-0.6667f,20.3614f,-0.6667f,20.3614f,-0.5625f,20.3125f,-0.5625f,20.3125f,-0.4583f,20.2637f,-0.4583f,20.2637f,-0.3542f,20.2149f,-0.3542f,20.2149f,-0.2500f,20.1661f,-0.2500f,
	20.1661f,-0.1458f,20.1172f,-0.1458f,20.1172f,-0.0417f,20.0684f,-0.0417f,20.0684f,0.0625f,20.0196f,0.0625f,20.0196f,0.1667f,19.9707f,0.1667f,19.9707f,0.2708f,19.9219f,0.2708f,19.9219f,0.3750f,19.8731f,0.3750f,19.8731f,0.4792f,19.8243f,0.4792f,19.8243f,0.5833f,
	19.7754f,0.5833f,19.7754f,0.6875f,19.7266f,0.6875f,19.7266f,0.7917f,19.6778f,0.7917f,19.6778f,0.8958f,19.6289f,0.8958f,19.6289f,1.0000f,19.5801f,1.0000f,21.9239f,-4.0000f,21.8750f,-4.0000f,21.8750f,-3.8958f,21.8262f,-3.8958f,21.8262f,-3.7917f,21.7774f,-3.7917f,
	21.7774f,-3.6875f,21.7286f,-3.6875f,21.7286f,-3.5833f,21.6797f,-3.5833f,21.6797f,-3.4792f,21.6309f,-3.4792f,21.6309f,-3.3750f,21.5821f,-3.3750f,21.5821f,-3.2708f,21.5332f,-3.2708f,21.5332f,-3.1667f,21.4844f,-3.1667f,21.4844f,-3.0625f,21.4356f,-3.0625f,21.4356f,-2.9583f,
	21.3868f,-2.9583f,21.3868f,-2.8542f,21.3379f,-2.8542f,21.3379f,-2.7500f,21.2891f,-2.7500f,21.2891f,-2.6458f,21.2403f,-2.6458f,21.2403f,-2.5417f,21.1915f,-2.5417f,21.1915f,-2.4375f,21.1426f,-2.4375f,21.1426f,-2.3333f,21.0938f,-2.3333f,21.0938f,-2.2292f,21.0450f,-2.2292f,
	21.0450f,-2.1250f,20.9961f,-2.1250f,20.9961f,-2.0208f,20.9473f,-2.0208f,20.9473f,-1.9167f,20.8985f,-1.9167f,20.8985f,-1.8125f,20.8497f,-1.8125f,20.8497f,-1.7083f,20.8008f,-1.7083f,20.8008f,-1.6042f,20.7520f,-1.6042f,20.7520f,-1.5000f,20.7032f,-1.5000f,20.7032f,-1.3958f,
	20.6543f,-1.3958f,20.6543f,-1.2917f,20.6055f,-1.2917f,20.6055f,-1.1875f,20.5567f,-1.1875f,20.5567f,-1.0833f,20.5079f,-1.0833f,20.5079f,-0.9792f,20.4590f,-0.9792f,20.4590f,-0.8750f,20.4102f,-0.8750f,20.4102f,-0.7708f,20.3614f,-0.7708f,20.3614f,-0.6667f,20.3125f,-0.6667f,
	20.3125f,-0.5625f,20.2637f,-0.5625f,20.2637f,-0.4583f,20.2149f,-0.4583f,20.2149f,-0.3542f,20.1661f,-0.3542f,20.1661f,-0.2500f,20.1172f,-0.2500f,20.1172f,-0.1458f,20.0684f,-0.1458f,20.0684f,-0.0417f,20.0196f,-0.0417f,20.0196f,0.0625f,19.9707f,0.0625f,19.9707f,0.1667f,
	19.9219f,0.1667f,19.9219f,0.2708f,19.8731f,0.2708f,19.8731f,0.3750f,19.8243f,0.3750f,19.8243f,0.4792f,19.7754f,0.4792f,19.7754f,0.5833f,19.7266f,0.5833f,19.7266f,0.6875f,19.6778f,0.6875f,19.6778f,0.7917f,19.6289f,0.7917f,19.6289f,0.8958f,19.5801f,0.8958f,
	19.5801f,1.0000f,19.5313f,1.0000f,21.8750f,-4.0000f,21.8262f,-4.0000f,21.8262f,-3.8958f,21.7774f,-3.8958f,21.7774f,-3.7917f,21.7286f,-3.7917f,21.7286f,-3.6875f,21.6797f,-3.6875f,21.6797f,-3.5833f,21.6309f,-3.5833f,21.6309f,-3.4792f,21.5821f,-3.4792f,21.5821f,-3.3750f,
	21.5332f,-3.3750f,21.5332f,-3.2708f,21.4844f,-3.2708f,21.4844f,-3.1667f,21.4356f,-3.1667f,21.4356f,-3.0625f,21.3868f,-3.0625f,21.3868f,-2.9583f,21.3379f,-2.9583f,21.3379f,-2.8542f,21.2891f,-2.8542f,21.2891f,-2.7500f,21.2403f,-2.7500f,21.2403f,-2.6458f,21.1915f,-2.6458f,
	21.1915f,-2.5417f,21.1426f,-2.5417f,21.1426f,-2.4375f,21.0938f,-2.4375f,21.0938f,-2.3333f,21.0450f,-2.3333f,21.0450f,-2.2292f,20.9961f,-2.2292f,20.9961f,-2.1250f,20.9473f,-2.1250f,20.9473f,-2.0208f,20.8985f,-2.0208f,20.8985f,-1.9167f,20.8497f,-1.9167f,20.8497f,-1.8125f,
	20.8008f,-1.8125f,20.8008f,-1.7083f,20.7520f,-1.7083f,20.7520f,-1.6042f,20.7032f,-1.6042f,20.7032f,-1.5000f,20.6543f,-1.5000f,20.6543f,-1.3958f,20.6055f,-1.3958f,20.6055f,-1.2917f,20.5567f,-1.2917f,20.5567f,-1.1875f,20.5079f,-1.1875f,20.5079f,-1.0833f,20.4590f,-1.0833f,
	20.4590f,-0.9792f,20.4102f,-0.9792f,20.4102f,-0.8750f,20.3614f,-0.8750f,20.3614f,-0.7708f,20.3125f,-0.7708f,20.3125f,-0.6667f,20.2637f,-0.6667f,20.2637f,-0.5625f,20.2149f,-0.5625f,20.2149f,-0.4583f,20.1661f,-0.4583f,20.1661f,-0.3542f,20.1172f,-0.3542f,20.1172f,-0.2500f,
	20.0684f,-0.2500f,20.0684f,-0.1458f,20.0196f,-0.1458f,20.0196f,-0.0417f,19.9707f,-0.0417f,19.9707f,0.0625f,19.9219f,0.0625f,19.9219f,0.1667f,19.8731f,0.1667f,19.8731f,0.2708f,19.8243f,0.2708f,19.8243f,0.3750f,19.7754f,0.3750f,19.7754f,0.4792f,19.7266f,0.4792f,
	19.7266f,0.5833f,19.6778f,0.5833f,19.6778f,0.6875f,19.6289f,0.6875f,19.6289f,0.7917f,19.5801f,0.7917f,19.5801f,0.8958f,19.5313f,0.8958f,19.5313f,1.0000f,19.4825f,1.0000f,21.8262f,-4.0000f,21.7774f,-4.0000f,21.7774f,-3.8958f,21.7286f,-3.8958f,21.7286f,-3.7917f,
	21.6797f,-3.7917f,21.6797f,-3.6875f,21.6309f,-3.6875f,21.6309f,-3.5833f,21.5821f,-3.5833f,21.5821f,-3.4792f,21.5332f,-3.4792f,21.5332f,-3.3750f,21.4844f,-3.3750f,21.4844f,-3.2708f,21.4356f,-3.2708f,21.4356f,-3.1667f,21.3868f,-3.1667f,21.3868f,-3.0625f,21.3379f,-3.0625f,
	21.3379f,-2.9583f,21.2891f,-2.9583f,21.2891f,-2.8542f,21.2403f,-2.8542f,21.2403f,-2.7500f,21.1915f,-2.7500f,21.1915f,-2.6458f,21.1426f,-2.6458f,21.1426f,-2.5417f,21.0938f,-2.5417f,21.0938f,-2.4375f,21.0450f,-2.4375f,21.0450f,-2.3333f,20.9961f,-2.3333f,20.9961f,-2.2292f,
	20.9473f,-2.2292f,20.9473f,-2.1250f,20.8985f,-2.1250f,20.8985f,-2.0208f,20.8497f,-2.0208f,20.8497f,-1.9167f,20.8008f,-1.9167f,20.8008f,-1.8125f,20.7520f,-1.8125f,20.7520f,-1.7083f,20.7032f,-1.7083f,20.7032f,-1.6042f,20.6543f,-1.6042f,20.6543f,-1.5000f,20.6055f,-1.5000f,
	20.6055f,-1.3958f,20.5567f,-1.3958f,20.5567f,-1.2917f,20.5079f,-1.2917f,20.5079f,-1.1875f,20.4590f,-1.1875f,20.4590f,-1.0833f,20.4102f,-1.0833f,20.4102f,-0.9792f,20.3614f,-0.9792f,20.3614f,-0.8750f,20.3125f,-0.8750f,20.3125f,-0.7708f,20.2637f,-0.7708f,20.2637f,-0.6667f,
	20.2149f,-0.6667f,20.2149f,-0.5625f,20.1661f,-0.5625f,20.1661f,-0.4583f,20.1172f,-0.4583f,20.1172f,-0.3542f,20.0684f,-0.3542f,20.0684f,-0.2500f,20.0196f,-0.2500f,20.0196f,-0.1458f,19.9707f,-0.1458f,19.9707f,-0.0417f,19.9219f,-0.0417f,19.9219f,0.0625f,19.8731f,0.0625f,
	19.8731f,0.1667f,19.8243f,0.1667f,19.8243f,0.2708f,19.7754f,0.2708f,19.7754f,0.3750f,19.7266f,0.3750f,19.7266f,0.4792f,19.6778f,0.4792f,19.6778f,0.5833f,19.6289f,0.5833f,19.6289f,0.6875f,19.5801f,0.6875f,19.5801f,0.7917f,19.5313f,0.7917f,19.5313f,0.8958f,
	19.4825f,0.8958f,19.4825f,1.0000f,19.4336f,1.0000f,21.7774f,-4.0000f,21.7286f,-4.0000f,21.7286f,-3.8958f,21.6797f,-3.8958f,21.6797f,-3.7917f,21.6309f,-3.7917f,21.6309f,-3.6875f,21.5821f,-3.6875f,21.5821f,-3.5833f,21.5332f,-3.5833f,21.5332f,-3.4792f,21.4844f,-3.4792f,
	21.4844f,-3.3750f,21.4356f,-3.3750f,21.4356f,-3.2708f,21.3868f,-3.2708f,21.3868f,-3.1667f,21.3379f,-3.1667f,21.3379f,-3.0625f,21.2891f,-3.0625f,21.2891f,-2.9583f,21.2403f,-2.9583f,21.2403f,-2.8542f,21.1915f,-2.8542f,21.1915f,-2.7500f,21.1426f,-2.7500f,21.1426f,-2.6458f,
	21.0938f,-2.6458f,21.0938f,-2.5417f,21.0450f,-2.5417f,21.0450f,-2.4375f,20.9961f,-2.4375f,20.9961f,-2.3333f,20.9473f,-2.3333f,20.9473f,-2.2292f,20.8985f,-2.2292f,20.8985f,-2.1250f,20.8497f,-2.1250f,20.8497f,-2.0208f,20.8008f,-2.0208f,20.8008f,-1.9167f,20.7520f,-1.9167f,
	20.7520f,-1.8125f,20.7032f,-1.8125f,20.7032f,-1.7083f,20.6543f,-1.7083f,20.6543f,-1.6042f,20.6055f,-1.6042f,20.6055f,-1.5000f,20.5567f,-1.5000f,20.5567f,-1.3958f,20.5079f,-1.3958f,20.5079f,-1.2917f,20.4590f,-1.2917f,20.4590f,-1.1875f,20.4102f,-1.1875f,20.4102f,-1.0833f,
	20.3614f,-1.0833f,20.3614f,-0.9792f,20.3125f,-0.9792f,20.3125f,-0.8750f,20.2637f,-0.8750f,20.2637f,-0.7708f,20.2149f,-0.7708f,20.2149f,-0.6667f,20.1661f,-0.6667f,20.1661f,-0.5625f,20.1172f,-0.5625f,20.1172f,-0.4583f,20.0684f,-0.4583f,20.0684f,-0.3542f,20.0196f,-0.3542f,
	20.0196f,-0.2500f,19.9707f,-0.2500f,19.9707f,-0.1458f,19.9219f,-0.1458f,19.9219f,-0.0417f,19.8731f,-0.0417f,19.8731f,0.0625f,19.8243f,0.0625f,19.8243f,0.1667f,19.7754f,0.1667f,19.7754f,0.2708f,19.7266f,0.2708f,19.7266f,0.3750f,19.6778f,0.3750f,19.6778f,0.4792f,
	19.6289f,0.4792f,19.6289f,0.5833f,19.5801f,0.5833f,19.5801f,0.6875f,19.5313f,0.6875f,19.5313f,0.7917f,19.4825f,0.7917f,19.4825f,0.8958f,19.4336f,0.8958f,19.4336f,1.0000f,19.3848f,1.0000f,21.7286f,-4.0000f,21.6797f,-4.0000f,21.6797f,-3.8958f,21.6309f,-3.8958f,
	21.6309f,-3.7917f,21.5821f,-3.7917f,21.5821f,-3.6875f,21.5332f,-3.6875f,21.5332f,-3.5833f,21.4844f,-3.5833f,21.4844f,-3.4792f,21.4356f,-3.4792f,21.4356f,-3.3750f,21.3868f,-3.3750f,21.3868f,-3.2708f,21.3379f,-3.2708f,21.3379f,-3.1667f,21.2891f,-3.1667f,21.2891f,-3.0625f,
	21.2403f,-3.0625f,21.2403f,-2.9583f,21.1915f,-2.9583f,21.1915f,-2.8542f,21.1426f,-2.8542f,21.1426f,-2.7500f,21.0938f,-2.7500f,21.0938f,-2.6458f,21.0450f,-2.6458f,21.0450f,-2.5417f,20.9961f,-2.5417f,20.9961f,-2.4375f,20.9473f,-2.4375f,20.9473f,-2.3333f,20.8985f,-2.3333f,
	20.8985f,-2.2292f,20.8497f,-2.2292f,20.8497f,-2.1250f,20.8008f,-2.1250f,20.8008f,-2.0208f,20.7520f,-2.0208f,20.7520f,-1.9167f,20.7032f,-1.9167f,20.7032f,-1.8125f,20.6543f,-1.8125f,20.6543f,-1.7083f,20.6055f,-1.7083f,20.6055f,-1.6042f,20.5567f,-1.6042f,20.5567f,-1.5000f,
	20.5079f,-1.5000f,20.5079f,-1.3958f,20.4590f,-1.3958f,20.4590f,-1.2917f,20.4102f,-1.2917f,20.4102f,-1.1875f,20.3614f,-1.1875f,20.3614f,-1.0833f,20.3125f,-1.0833f,20.3125f,-0.9792f,20.2637f,-0.9792f,20.2637f,-0.8750f,20.2149f,-0.8750f,20.2149f,-0.7708f,20.1661f,-0.7708f,
	20.1661f,-0.6667f,20.1172f,-0.6667f,20.1172f,-0.5625f,20.0684f,-0.5625f,20.0684f,-0.4583f,20.0196f,-0.4583f,20.0196f,-0.3542f,19.9707f,-0.3542f,19.9707f,-0.2500f,19.9219f,-0.2500f,19.9219f,-0.1458f,19.8731f,-0.1458f,19.8731f,-0.0417f,19.8243f,-0.0417f,19.8243f,0.0625f,
	19.7754f,0.0625f,19.7754f,0.1667f,19.7266f,0.1667f,19.7266f,0.2708f,19.6778f,0.2708f,19.6778f,0.3750f,19.6289f,0.3750f,19.6289f,0.4792f,19.5801f,0.4792f,19.5801f,0.5833f,19.5313f,0.5833f,19.5313f,0.6875f,19.4825f,0.6875f,19.4825f,0.7917f,19.4336f,0.7917f,
	19.4336f,0.8958f,19.3848f,0.8958f,19.3848f,1.0000f,19.3360f,1.0000f,21.6797f,-4.0000f,21.6309f,-4.0000f,21.6309f,-3.8958f,21.5821f,-3.8958f,21.5821f,-3.7917f,21.5332f,-3.7917f,21.5332f,-3.6875f,21.4844f,-3.6875f,21.4844f,-3.5833f,21.4356f,-3.5833f,21.4356f,-3.4792f,
	21.3868f,-3.4792f,21.3868f,-3.3750f,21.3379f,-3.3750f,21.3379f,-3.2708f,21.2891f,-3.2708f,21.2891f,-3.1667f,21.2403f,-3.1667f,21.2403f,-3.0625f,21.1915f,-3.0625f,21.1915f,-2.9583f,21.1426f,-2.9583f,21.1426f,-2.8542f,21.0938f,-2.8542f,21.0938f,-2.7500f,21.0450f,-2.7500f,
	21.0450f,-2.6458f,20.9961f,-2.6458f,20.9961f,-2.5417f,20.9473f,-2.5417f,20.9473f,-2.4375f,20.8985f,-2.4375f,20.8985f,-2.3333f,20.8497f,-2.3333f,20.8497f,-2.2292f,20.8008f,-2.2292f,20.8008f,-2.1250f,20.7520f,-2.1250f,20.7520f,-2.0208f,20.7032f,-2.0208f,20.7032f,-1.9167f,
	20.6543f,-1.9167f,20.6543f,-1.8125f,20.6055f,-1.8125f,20.6055f,-1.7083f,20.5567f,-1.7083f,20.5567f,-1.6042f,20.5079f,-1.6042f,20.5079f,-1.5000f,20.4590f,-1.5000f,20.4590f,-1.3958f,20.4102f,-1.3958f,20.4102f,-1.2917f,20.3614f,-1.2917f,20.3614f,-1.1875f,20.3125f,-1.1875f,
	20.3125f,-1.0833f,20.2637f,-1.0833f,20.2637f,-0.9792f,20.2149f,-0.9792f,20.2149f,-0.8750f,20.1661f,-0.8750f,20.1661f,-0.7708f,20.1172f,-0.7708f,20.1172f,-0.6667f,20.0684f,-0.6667f,20.0684f,-0.5625f,20.0196f,-0.5625f,20.0196f,-0.4583f,19.9707f,-0.4583f,19.9707f,-0.3542f,
	19.9219f,-0.3542f,19.9219f,-0.2500f,19.8731f,-0.2500f,19.8731f,-0.1458f,19.8243f,-0.1458f,19.8243f,-0.0417f,19.7754f,-0.0417f,19.7754f,0.0625f,19.7266f,0.0625f,19.7266f,0.1667f,19.6778f,0.1667f,19.6778f,0.2708f,19.6289f,0.2708f,19.6289f,0.3750f,19.5801f,0.3750f,
	19.5801f,0.4792f,19.5313f,0.4792f,19.5313f,0.5833f,19.4825f,0.5833f,19.4825f,0.6875f,19.4336f,0.6875f,19.4336f,0.7917f,19.3848f,0.7917f,19.3848f,0.8958f,19.3360f,0.8958f,19.3360f,1.0000f,19.2871f,1.0000f,21.6309f,-4.0000f,21.5821f,-4.0000f,21.5821f,-3.8958f,
	21.5332f,-3.8958f,21.5332f,-3.7917f,21.4844f,-3.7917f,21.4844f,-3.6875f,21.4356f,-3.6875f,21.4356f,-3.5833f,21.3868f,-3.5833f,21.3868f,-3.4792f,21.3379f,-3.4792f,21.3379f,-3.3750f,21.2891f,-3.3750f,21.2891f,-3.2708f,21.2403f,-3.2708f,21.2403f,-3.1667f,21.1915f,-3.1667f,
	21.1915f,-3.0625f,21.1426f,-3.0625f,21.1426f,-2.9583f,21.0938f,-2.9583f,21.0938f,-2.8542f,21.0450f,-2.8542f,21.0450f,-2.7500f,20.9961f,-2.7500f,20.9961f,-2.6458f,20.9473f,-2.6458f,20.9473f,-2.5417f,20.8985f,-2.5417f,20.8985f,-2.4375f,20.8497f,-2.4375f,20.8497f,-2.3333f,
	20.8008f,-2.3333f,20.8008f,-2.2292f,20.7520f,-2.2292f,20.7520f,-2.1250f,20.7032f,-2.1250f,20.7032f,-2.0208f,20.6543f,-2.0208f,20.6543f,-1.9167f,20.6055f,-1.9167f,20.6055f,-1.8125f,20.5567f,-1.8125f,20.5567f,-1.7083f,20.5079f,-1.7083f,20.5079f,-1.6042f,20.4590f,-1.6042f,
	20.4590f,-1.5000f,20.4102f,-1.5000f,20.4102f,-1.3958f,20.3614f,-1.3958f,20.3614f,-1.2917f,20.3125f,-1.2917f,20.3125f,-1.1875f,20.2637f,-1.1875f,20.2637f,-1.0833f,20.2149f,-1.0833f,20.2149f,-0.9792f,20.1661f,-0.9792f,20.1661f,-0.8750f,20.1172f,-0.8750f,20.1172f,-0.7708f,
	20.0684f,-0.7708f,20.0684f,-0.6667f,20.0196f,-0.6667f,20.0196f,-0.5625f,19.9707f,-0.5625f,19.9707f,-0.4583f,19.9219f,-0.4583f,19.9219f,-0.3542f,19.8731f,-0.3542f,19.8731f,-0.2500f,19.8243f,-0.2500f,19.8243f,-0.1458f,19.7754f,-0.1458f,19.7754f,-0.0417f,19.7266f,-0.0417f,
	19.7266f,0.0625f,19.6778f,0.0625f,19.6778f,0.1667f,19.6289f,0.1667f,19.6289f,0.2708f,19.5801f,0.2708f,19.5801f,0.3750f,19.5313f,0.3750f,19.5313f,0.4792f,19.4825f,0.4792f,19.4825f,0.5833f,19.4336f,0.5833f,19.4336f,0.6875f,19.3848f,0.6875f,19.3848f,0.7917f,
	19.3360f,0.7917f,19.3360f,0.8958f,19.2871f,0.8958f,19.2871f,1.0000f,19.2383f,1.0000f,21.5821f,-4.0000f,21.5332f,-4.0000f,21.5332f,-3.8958f,21.4844f,-3.8958f,21.4844f,-3.7917f,21.4356f,-3.7917f,21.4356f,-3.6875f,21.3868f,-3.6875f,21.3868f,-3.5833f,21.3379f,-3.5833f,
	21.3379f,-3.4792f,21.2891f,-3.4792f,21.2891f,-3.3750f,21.2403f,-3.3750f,21.2403f,-3.2708f,21.1915f,-3.2708f,21.1915f,-3.1667f,21.1426f,-3.1667f,21.1426f,-3.0625f,21.0938f,-3.0625f,21.0938f,-2.9583f,21.0450f,-2.9583f,21.0450f,-2.8542f,20.9961f,-2.8542f,20.9961f,-2.7500f,
	20.9473f,-2.7500f,20.9473f,-2.6458f,20.8985f,-2.6458f,20.8985f,-2.5417f,20.8497f,-2.5417f,20.8497f,-2.4375f,20.8008f,-2.4375f,20.8008f,-2.3333f,20.7520f,-2.3333f,20.7520f,-2.2292f,20.7032f,-2.2292f,20.7032f,-2.1250f,20.6543f,-2.1250f,20.6543f,-2.0208f,20.6055f,-2.0208f,
	20.6055f,-1.9167f,20.5567f,-1.9167f,20.5567f,-1.8125f,20.5079f,-1.8125f,20.5079f,-1.7083f,20.4590f,-1.7083f,20.4590f,-1.6042f,20.4102f,-1.6042f,20.4102f,-1.5000f,20.3614f,-1.5000f,20.3614f,-1.3958f,20.3125f,-1.3958f,20.3125f,-1.2917f,20.2637f,-1.2917f,20.2637f,-1.1875f,
	20.2149f,-1.1875f,20.2149f,-1.0833f,20.1661f,-1.0833f,20.1661f,-0.9792f,20.1172f,-0.9792f,20.1172f,-0.8750f,20.0684f,-0.8750f,20.0684f,-0.7708f,20.0196f,-0.7708f,20.0196f,-0.6667f,19.9707f,-0.6667f,19.9707f,-0.5625f,19.9219f,-0.5625f,19.9219f,-0.4583f,19.8731f,-0.4583f,
	19.8731f,-0.3542f,19.8243f,-0.3542f,19.8243f,-0.2500f,19.7754f,-0.2500f,19.7754f,-0.1458f,19.7266f,-0.1458f,19.7266f,-0.0417f,19.6778f,-0.0417f,19.6778f,0.0625f,19.6289f,0.0625f,19.6289f,0.1667f,19.5801f,0.1667f,19.5801f,0.2708f,19.5313f,0.2708f,19.5313f,0.3750f,
	19.4825f,0.3750f,19.4825f,0.4792f,19.4336f,0.4792f,19.4336f,0.5833f,19.3848f,0.5833f,19.3848f,0.6875f,19.3360f,0.6875f,19.3360f,0.7917f,19.2871f,0.7917f,19.2871f,0.8958f,19.2383f,0.8958f,19.2383f,1.0000f,19.1895f,1.0000f,21.5332f,-4.0000f,21.4844f,-4.0000f,
	21.4844f,-3.8958f,21.4356f,-3.8958f,21.4356f,-3.7917f,21.3868f,-3.7917f,21.3868f,-3.6875f,21.3379f,-3.6875f,21.3379f,-3.5833f,21.2891f,-3.5833f,21.2891f,-3.4792f,21.2403f,-3.4792f,21.2403f,-3.3750f,21.1915f,-3.3750f,21.1915f,-3.2708f,21.1426f,-3.2708f,21.1426f,-3.1667f,
	21.0938f,-3.1667f,21.0938f,-3.0625f,21.0450f,-3.0625f,21.0450f,-2.9583f,20.9961f,-2.9583f,20.9961f,-2.8542f,20.9473f,-2.8542f,20.9473f,-2.7500f,20.8985f,-2.7500f,20.8985f,-2.6458f,20.8497f,-2.6458f,20.8497f,-2.5417f,20.8008f,-2.5417f,20.8008f,-2.4375f,20.7520f,-2.4375f,
	20.7520f,-2.3333f,20.7032f,-2.3333f,20.7032f,-2.2292f,20.6543f,-2.2292f,20.6543f,-2.1250f,20.6055f,-2.1250f,20.6055f,-2.0208f,20.5567f,-2.0208f,20.5567f,-1.9167f,20.5079f,-1.9167f,20.5079f,-1.8125f,20.4590f,-1.8125f,20.4590f,-1.7083f,20.4102f,-1.7083f,20.4102f,-1.6042f,
	20.3614f,-1.6042f,20.3614f,-1.5000f,20.3125f,-1.5000f,20.3125f,-1.3958f,20.2637f,-1.3958f,20.2637f,-1.2917f,20.2149f,-1.2917f,20.2149f,-1.1875f,20.1661f,-1.1875f,20.1661f,-1.0833f,20.1172f,-1.0833f,20.1172f,-0.9792f,20.0684f,-0.9792f,20.0684f,-0.8750f,20.0196f,-0.8750f,
	20.0196f,-0.7708f,19.9707f,-0.7708f,19.9707f,-0.6667f,19.9219f,-0.6667f,19.9219f,-0.5625f,19.8731f,-0.5625f,19.8731f,-0.4583f,19.8243f,-0.4583f,19.8243f,-0.3542f,19.7754f,-0.3542f,19.7754f,-0.2500f,19.7266f,-0.2500f,19.7266f,-0.1458f,19.6778f,-0.1458f,19.6778f,-0.0417f,
	19.6289f,-0.0417f,19.6289f,0.0625f,19.5801f,0.0625f,19.5801f,0.1667f,19.5313f,0.1667f,19.5313f,0.2708f,19.4825f,0.2708f,19.4825f,0.3750f,19.4336f,0.3750f,19.4336f,0.4792f,19.3848f,0.4792f,19.3848f,0.5833f,19.3360f,0.5833f,19.3360f,0.6875f,19.2871f,0.6875f,
	19.2871f,0.7917f,19.2383f,0.7917f,19.2383f,0.8958f,19.1895f,0.8958f,19.1895f,1.0000f,19.1407f,1.0000f,21.4844f,-4.0000f,21.4356f,-4.0000f,21.4356f,-3.8958f,21.3868f,-3.8958f,21.3868f,-3.7917f,21.3379f,-3.7917f,21.3379f,-3.6875f,21.2891f,-3.6875f,21.2891f,-3.5833f,
	21.2403f,-3.5833f,21.2403f,-3.4792f,21.1915f,-3.4792f,21.1915f,-3.3750f,21.1426f,-3.3750f,21.1426f,-3.2708f,21.0938f,-3.2708f,21.0938f,-3.1667f,21.0450f,-3.1667f,21.0450f,-3.0625f,20.9961f,-3.0625f,20.9961f,-2.9583f,20.9473f,-2.9583f,20.9473f,-2.8542f,20.8985f,-2.8542f,
	20.8985f,-2.7500f,20.8497f,-2.7500f,20.8497f,-2.6458f,20.8008f,-2.6458f,20.8008f,-2.5417f,20.7520f,-2.5417f,20.7520f,-2.4375f,20.7032f,-2.4375f,20.7032f,-2.3333f,20.6543f,-2.3333f,20.6543f,-2.2292f,20.6055f,-2.2292f,20.6055f,-2.1250f,20.5567f,-2.1250f,20.5567f,-2.0208f,
	20.5079f,-2.0208f,20.5079f,-1.9167f,20.4590f,-1.9167f,20.4590f,-1.8125f,20.4102f,-1.8125f,20.4102f,-1.7083f,20.3614f,-1.7083f,20.3614f,-1.6042f,20.3125f,-1.6042f,20.3125f,-1.5000f,20.2637f,-1.5000f,20.2637f,-1.3958f,20.2149f,-1.3958f,20.2149f,-1.2917f,20.1661f,-1.2917f,
	20.1661f,-1.1875f,20.1172f,-1.1875f,20.1172f,-1.0833f,20.0684f,-1.0833f,20.0684f,-0.9792f,20.0196f,-0.9792f,20.0196f,-0.8750f,19.9707f,-0.8750f,19.9707f,-0.7708f,19.9219f,-0.7708f,19.9219f,-0.6667f,19.8731f,-0.6667f,19.8731f,-0.5625f,19.8243f,-0.5625f,19.8243f,-0.4583f,
	19.7754f,-0.4583f,19.7754f,-0.3542f,19.7266f,-0.3542f,19.7266f,-0.2500f,19.6778f,-0.2500f,19.6778f,-0.1458f,19.6289f,-0.1458f,19.6289f,-0.0417f,19.5801f,-0.0417f,19.5801f,0.0625f,19.5313f,0.0625f,19.5313f,0.1667f,19.4825f,0.1667f,19.4825f,0.2708f,19.4336f,0.2708f,
	19.4336f,0.3750f,19.3848f,0.3750f,19.3848f,0.4792f,19.3360f,0.4792f,19.3360f,0.5833f,19.2871f,0.5833f,19.2871f,0.6875f,19.2383f,0.6875f,19.2383f,0.7917f,19.1895f,0.7917f,19.1895f,0.8958f,19.1407f,0.8958f,19.1407f,1.0000f,19.0918f,1.0000f,21.4356f,-4.0000f,
	21.3868f,-4.0000f,21.3868f,-3.8958f,21.3379f,-3.8958f,21.3379f,-3.7917f,21.2891f,-3.7917f,21.2891f,-3.6875f,21.2403f,-3.6875f,21.2403f,-3.5833f,21.1915f,-3.5833f,21.1915f,-3.4792f,21.1426f,-3.4792f,21.1426f,-3.3750f,21.0938f,-3.3750f,21.0938f,-3.2708f,21.0450f,-3.2708f,
	21.0450f,-3.1667f,20.9961f,-3.1667f,20.9961f,-3.0625f,20.9473f,-3.0625f,20.9473f,-2.9583f,20.8985f,-2.9583f,20.8985f,-2.8542f,20.8497f,-2.8542f,20.8497f,-2.7500f,20.8008f,-2.7500f,20.8008f,-2.6458f,20.7520f,-2.6458f,20.7520f,-2.5417f,20.7032f,-2.5417f,20.7032f,-2.4375f,
	20.6543f,-2.4375f,20.6543f,-2.3333f,20.6055f,-2.3333f,20.6055f,-2.2292f,20.5567f,-2.2292f,20.5567f,-2.1250f,20.5079f,-2.1250f,20.5079f,-2.0208f,20.4590f,-2.0208f,20.4590f,-1.9167f,20.4102f,-1.9167f,20.4102f,-1.8125f,20.3614f,-1.8125f,20.3614f,-1.7083f,20.3125f,-1.7083f,
	20.3125f,-1.6042f,20.2637f,-1.6042f,20.2637f,-1.5000f,20.2149f,-1.5000f,20.2149f,-1.3958f,20.1661f,-1.3958f,20.1661f,-1.2917f,20.1172f,-1.2917f,20.1172f,-1.1875f,20.0684f,-1.1875f,20.0684f,-1.0833f,20.0196f,-1.0833f,20.0196f,-0.9792f,19.9707f,-0.9792f,19.9707f,-0.8750f,
	19.9219f,-0.8750f,19.9219f,-0.7708f,19.8731f,-0.7708f,19.8731f,-0.6667f,19.8243f,-0.6667f,19.8243f,-0.5625f,19.7754f,-0.5625f,19.7754f,-0.4583f,19.7266f,-0.4583f,19.7266f,-0.3542f,19.6778f,-0.3542f,19.6778f,-0.2500f,19.6289f,-0.2500f,19.6289f,-0.1458f,19.5801f,-0.1458f,
	19.5801f,-0.0417f,19.5313f,-0.0417f,19.5313f,0.0625f,19.4825f,0.0625f,19.4825f,0.1667f,19.4336f,0.1667f,19.4336f,0.2708f,19.3848f,0.2708f,19.3848f,0.3750f,19.3360f,0.3750f,19.3360f,0.4792f,19.2871f,0.4792f,19.2871f,0.5833f,19.2383f,0.5833f,19.2383f,0.6875f,
	19.1895f,0.6875f,19.1895f,0.7917f,19.1407f,0.7917f,19.1407f,0.8958f,19.0918f,0.8958f,19.0918f,1.0000f,19.0430f,1.0000f,21.3868f,-4.0000f,21.3379f,-4.0000f,21.3379f,-3.8958f,21.2891f,-3.8958f,21.2891f,-3.7917f,21.2403f,-3.7917f,21.2403f,-3.6875f,21.1915f,-3.6875f,
	21.1915f,-3.5833f,21.1426f,-3.5833f,21.1426f,-3.4792f,21.0938f,-3.4792f,21.0938f,-3.3750f,21.0450f,-3.3750f,21.0450f,-3.2708f,20.9961f,-3.2708f,20.9961f,-3.1667f,20.9473f,-3.1667f,20.9473f,-3.0625f,20.8985f,-3.0625f,20.8985f,-2.9583f,20.8497f,-2.9583f,20.8497f,-2.8542f,
	20.8008f,-2.8542f,20.8008f,-2.7500f,20.7520f,-2.7500f,20.7520f,-2.6458f,20.7032f,-2.6458f,20.7032f,-2.5417f,20.6543f,-2.5417f,20.6543f,-2.4375f,20.6055f,-2.4375f,20.6055f,-2.3333f,20.5567f,-2.3333f,20.5567f,-2.2292f,20.5079f,-2.2292f,20.5079f,-2.1250f,20.4590f,-2.1250f,
	20.4590f,-2.0208f,20.4102f,-2.0208f,20.4102f,-1.9167f,20.3614f,-1.9167f,20.3614f,-1.8125f,20.3125f,-1.8125f,20.3125f,-1.7083f,20.2637f,-1.7083f,20.2637f,-1.6042f,20.2149f,-1.6042f,20.2149f,-1.5000f,20.1661f,-1.5000f,20.1661f,-1.3958f,20.1172f,-1.3958f,20.1172f,-1.2917f,
	20.0684f,-1.2917f,20.0684f,-1.1875f,20.0196f,-1.1875f,20.0196f,-1.0833f,19.9707f,-1.0833f,19.9707f,-0.9792f,19.9219f,-0.9792f,19.9219f,-0.8750f,19.8731f,-0.8750f,19.8731f,-0.7708f,19.8243f,-0.7708f,19.8243f,-0.6667f,19.7754f,-0.6667f,19.7754f,-0.5625f,19.7266f,-0.5625f,
	19.7266f,-0.4583f,19.6778f,-0.4583f,19.6778f,-0.3542f,19.6289f,-0.3542f,19.6289f,-0.2500f,19.5801f,-0.2500f,19.5801f,-0.1458f,19.5313f,-0.1458f,19.5313f,-0.0417f,19.4825f,-0.0417f,19.4825f,0.0625f,19.4336f,0.0625f,19.4336f,0.1667f,19.3848f,0.1667f,19.3848f,0.2708f,
	19.3360f,0.2708f,19.3360f,0.3750f,19.2871f,0.3750f,19.2871f,0.4792f,19.2383f,0.4792f,19.2383f,0.5833f,19.1895f,0.5833f,19.1895f,0.6875f,19.1407f,0.6875f,19.1407f,0.7917f,19.0918f,0.7917f,19.0918f,0.8958f,19.0430f,0.8958f,19.0430f,1.0000f,18.9942f,1.0000f,
	21.3379f,-4.0000f,21.2891f,-4.0000f,21.2891f,-3.8958f,21.2403f,-3.8958f,21.2403f,-3.7917f,21.1915f,-3.7917f,21.1915f,-3.6875f,21.1426f,-3.6875f,21.1426f,-3.5833f,21.0938f,-3.5833f,21.0938f,-3.4792f,21.0450f,-3.4792f,21.0450f,-3.3750f,20.9961f,-3.3750f,20.9961f,-3.2708f,
	20.9473f,-3.2708f,20.9473f,-3.1667f,20.8985f,-3.1667f,20.8985f,-3.0625f,20.8497f,-3.0625f,20.8497f,-2.9583f,20.8008f,-2.9583f,20.8008f,-2.8542f,20.7520f,-2.8542f,20.7520f,-2.7500f,20.7032f,-2.7500f,20.7032f,-2.6458f,20.6543f,-2.6458f,20.6543f,-2.5417f,20.6055f,-2.5417f,
	20.6055f,-2.4375f,20.5567f,-2.4375f,20.5567f,-2.3333f,20.5079f,-2.3333f,20.5079f,-2.2292f,20.4590f,-2.2292f,20.4590f,-2.1250f,20.4102f,-2.1250f,20.4102f,-2.0208f,20.3614f,-2.0208f,20.3614f,-1.9167f,20.3125f,-1.9167f,20.3125f,-1.8125f,20.2637f,-1.8125f,20.2637f,-1.7083f,
	20.2149f,-1.7083f,20.2149f,-1.6042f,20.1661f,-1.6042f,20.1661f,-1.5000f,20.1172f,-1.5000f,20.1172f,-1.3958f,20.0684f,-1.3958f,20.0684f,-1.2917f,20.0196f,-1.2917f,20.0196f,-1.1875f,19.9707f,-1.1875f,19.9707f,-1.0833f,19.9219f,-1.0833f,19.9219f,-0.9792f,19.8731f,-0.9792f,
	19.8731f,-0.8750f,19.8243f,-0.8750f,19.8243f,-0.7708f,19.7754f,-0.7708f,19.7754f,-0.6667f,19.7266f,-0.6667f,19.7266f,-0.5625f,19.6778f,-0.5625f,19.6778f,-0.4583f,19.6289f,-0.4583f,19.6289f,-0.3542f,19.5801f,-0.3542f,19.5801f,-0.2500f,19.5313f,-0.2500f,19.5313f,-0.1458f,
	19.4825f,-0.1458f,19.4825f,-0.0417f,19.4336f,-0.0417f,19.4336f,0.0625f,19.3848f,0.0625f,19.3848f,0.1667f,19.3360f,0.1667f,19.3360f,0.2708f,19.2871f,0.2708f,19.2871f,0.3750f,19.2383f,0.3750f,19.2383f,0.4792f,19.1895f,0.4792f,19.1895f,0.5833f,19.1407f,0.5833f,
	19.1407f,0.6875f,19.0918f,0.6875f,19.0918f,0.7917f,19.0430f,0.7917f,19.0430f,0.8958f,18.9942f,0.8958f,18.9942f,1.0000f,18.9454f,1.0000f,21.2891f,-4.0000f,21.2403f,-4.0000f,21.2403f,-3.8958f,21.1915f,-3.8958f,21.1915f,-3.7917f,21.1426f,-3.7917f,21.1426f,-3.6875f,
	21.0938f,-3.6875f,21.0938f,-3.5833f,21.0450f,-3.5833f,21.0450f,-3.4792f,20.9961f,-3.4792f,20.9961f,-3.3750f,20.9473f,-3.3750f,20.9473f,-3.2708f,20.8985f,-3.2708f,20.8985f,-3.1667f,20.8497f,-3.1667f,20.8497f,-3.0625f,20.8008f,-3.0625f,20.8008f,-2.9583f,20.7520f,-2.9583f,
	20.7520f,-2.8542f,20.7032f,-2.8542f,20.7032f,-2.7500f,20.6543f,-2.7500f,20.6543f,-2.6458f,20.6055f,-2.6458f,20.6055f,-2.5417f,20.5567f,-2.5417f,20.5567f,-2.4375f,20.5079f,-2.4375f,20.5079f,-2.3333f,20.4590f,-2.3333f,20.4590f,-2.2292f,20.4102f,-2.2292f,20.4102f,-2.1250f,
	20.3614f,-2.1250f,20.3614f,-2.0208f,20.3125f,-2.0208f,20.3125f,-1.9167f,20.2637f,-1.9167f,20.2637f,-1.8125f,20.2149f,-1.8125f,20.2149f,-1.7083f,20.1661f,-1.7083f,20.1661f,-1.6042f,20.1172f,-1.6042f,20.1172f,-1.5000f,20.0684f,-1.5000f,20.0684f,-1.3958f,20.0196f,-1.3958f,
	20.0196f,-1.2917f,19.9707f,-1.2917f,19.9707f,-1.1875f,19.9219f,-1.1875f,19.9219f,-1.0833f,19.8731f,-1.0833f,19.8731f,-0.9792f,19.8243f,-0.9792f,19.8243f,-0.8750f,19.7754f,-0.8750f,19.7754f,-0.7708f,19.7266f,-0.7708f,19.7266f,-0.6667f,19.6778f,-0.6667f,19.6778f,-0.5625f,
	19.6289f,-0.5625f,19.6289f,-0.4583f,19.5801f,-0.4583f,19.5801f,-0.3542f,19.5313f,-0.3542f,19.5313f,-0.2500f,19.4825f,-0.2500f,19.4825f,-0.1458f,19.4336f,-0.1458f,19.4336f,-0.0417f,19.3848f,-0.0417f,19.3848f,0.0625f,19.3360f,0.0625f,19.3360f,0.1667f,19.2871f,0.1667f,
	19.2871f,0.2708f,19.2383f,0.2708f,19.2383f,0.3750f,19.1895f,0.3750f,19.1895f,0.4792f,19.1407f,0.4792f,19.1407f,0.5833f,19.0918f,0.5833f,19.0918f,0.6875f,19.0430f,0.6875f,19.0430f,0.7917f,18.9942f,0.7917f,18.9942f,0.8958f,18.9454f,0.8958f,18.9454f,1.0000f,
	18.8965f,1.0000f,21.2403f,-4.0000f,21.1915f,-4.0000f,21.1915f,-3.8958f,21.1426f,-3.8958f,21.1426f,-3.7917f,21.0938f,-3.7917f,21.0938f,-3.6875f,21.0450f,-3.6875f,21.0450f,-3.5833f,20.9961f,-3.5833f,20.9961f,-3.4792f,20.9473f,-3.4792f,20.9473f,-3.3750f,20.8985f,-3.3750f,
	20.8985f,-3.2708f,20.8497f,-3.2708f,20.8497f,-3.1667f,20.8008f,-3.1667f,20.8008f,-3.0625f,20.7520f,-3.0625f,20.7520f,-2.9583f,20.7032f,-2.9583f,20.7032f,-2.8542f,20.6543f,-2.8542f,20.6543f,-2.7500f,20.6055f,-2.7500f,20.6055f,-2.6458f,20.5567f,-2.6458f,20.5567f,-2.5417f,
	20.5079f,-2.5417f,20.5079f,-2.4375f,20.4590f,-2.4375f,20.4590f,-2.3333f,20.4102f,-2.3333f,20.4102f,-2.2292f,20.3614f,-2.2292f,20.3614f,-2.1250f,20.3125f,-2.1250f,20.3125f,-2.0208f,20.2637f,-2.0208f,20.2637f,-1.9167f,20.2149f,-1.9167f,20.2149f,-1.8125f,20.1661f,-1.8125f,
	20.1661f,-1.7083f,20.1172f,-1.7083f,20.1172f,-1.6042f,20.0684f,-1.6042f,20.0684f,-1.5000f,20.0196f,-1.5000f,20.0196f,-1.3958f,19.9707f,-1.3958f,19.9707f,-1.2917f,19.9219f,-1.2917f,19.9219f,-1.1875f,19.8731f,-1.1875f,19.8731f,-1.0833f,19.8243f,-1.0833f,19.8243f,-0.9792f,
	19.7754f,-0.9792f,19.7754f,-0.8750f,19.7266f,-0.8750f,19.7266f,-0.7708f,19.6778f,-0.7708f,19.6778f,-0.6667f,19.6289f,-0.6667f,19.6289f,-0.5625f,19.5801f,-0.5625f,19.5801f,-0.4583f,19.5313f,-0.4583f,19.5313f,-0.3542f,19.4825f,-0.3542f,19.4825f,-0.2500f,19.4336f,-0.2500f,
	19.4336f,-0.1458f,19.3848f,-0.1458f,19.3848f,-0.0417f,19.3360f,-0.0417f,19.3360f,0.0625f,19.2871f,0.0625f,19.2871f,0.1667f,19.2383f,0.1667f,19.2383f,0.2708f,19.1895f,0.2708f,19.1895f,0.3750f,19.1407f,0.3750f,19.1407f,0.4792f,19.0918f,0.4792f,19.0918f,0.5833f,
	19.0430f,0.5833f,19.0430f,0.6875f,18.9942f,0.6875f,18.9942f,0.7917f,18.9454f,0.7917f,18.9454f,0.8958f,18.8965f,0.8958f,18.8965f,1.0000f,18.8477f,1.0000f,21.1915f,-4.0000f,21.1426f,-4.0000f,21.1426f,-3.8958f,21.0938f,-3.8958f,21.0938f,-3.7917f,21.0450f,-3.7917f,
	21.0450f,-3.6875f,20.9961f,-3.6875f,20.9961f,-3.5833f,20.9473f,-3.5833f,20.9473f,-3.4792f,20.8985f,-3.4792f,20.8985f,-3.3750f,20.8497f,-3.3750f,20.8497f,-3.2708f,20.8008f,-3.2708f,20.8008f,-3.1667f,20.7520f,-3.1667f,20.7520f,-3.0625f,20.7032f,-3.0625f,20.7032f,-2.9583f,
	20.6543f,-2.9583f,20.6543f,-2.8542f,20.6055f,-2.8542f,20.6055f,-2.7500f,20.5567f,-2.7500f,20.5567f,-2.6458f,20.5079f,-2.6458f,20.5079f,-2.5417f,20.4590f,-2.5417f,20.4590f,-2.4375f,20.4102f,-2.4375f,20.4102f,-2.3333f,20.3614f,-2.3333f,20.3614f,-2.2292f,20.3125f,-2.2292f,
	20.3125f,-2.1250f,20.2637f,-2.1250f,20.2637f,-2.0208f,20.2149f,-2.0208f,20.2149f,-1.9167f,20.1661f,-1.9167f,20.1661f,-1.8125f,20.1172f,-1.8125f,20.1172f,-1.7083f,20.0684f,-1.7083f,20.0684f,-1.6042f,20.0196f,-1.6042f,20.0196f,-1.5000f,19.9707f,-1.5000f,19.9707f,-1.3958f,
	19.9219f,-1.3958f,19.9219f,-1.2917f,19.8731f,-1.2917f,19.8731f,-1.1875f,19.8243f,-1.1875f,19.8243f,-1.0833f,19.7754f,-1.0833f,19.7754f,-0.9792f,19.7266f,-0.9792f,19.7266f,-0.8750f,19.6778f,-0.8750f,19.6778f,-0.7708f,19.6289f,-0.7708f,19.6289f,-0.6667f,19.5801f,-0.6667f,
	19.5801f,-0.5625f,19.5313f,-0.5625f,19.5313f,-0.4583f,19.4825f,-0.4583f,19.4825f,-0.3542f,19.4336f,-0.3542f,19.4336f,-0.2500f,19.3848f,-0.2500f,19.3848f,-0.1458f,19.3360f,-0.1458f,19.3360f,-0.0417f,19.2871f,-0.0417f,19.2871f,0.0625f,19.2383f,0.0625f,19.2383f,0.1667f,
	19.1895f,0.1667f,19.1895f,0.2708f,19.1407f,0.2708f,19.1407f,0.3750f,19.0918f,0.3750f,19.0918f,0.4792f,19.0430f,0.4792f,19.0430f,0.5833f,18.9942f,0.5833f,18.9942f,0.6875f,18.9454f,0.6875f,18.9454f,0.7917f,18.8965f,0.7917f,18.8965f,0.8958f,18.8477f,0.8958f,
	18.8477f,1.0000f,18.7989f,1.0000f,21.1426f,-4.0000f,21.0938f,-4.0000f,21.0938f,-3.8958f,21.0450f,-3.8958f,21.0450f,-3.7917f,20.9961f,-3.7917f,20.9961f,-3.6875f,20.9473f,-3.6875f,20.9473f,-3.5833f,20.8985f,-3.5833f,20.8985f,-3.4792f,20.8497f,-3.4792f,20.8497f,-3.3750f,
	20.8008f,-3.3750f,20.8008f,-3.2708f,20.7520f,-3.2708f,20.7520f,-3.1667f,20.7032f,-3.1667f,20.7032f,-3.0625f,20.6543f,-3.0625f,20.6543f,-2.9583f,20.6055f,-2.9583f,20.6055f,-2.8542f,20.5567f,-2.8542f,20.5567f,-2.7500f,20.5079f,-2.7500f,20.5079f,-2.6458f,20.4590f,-2.6458f,
	20.4590f,-2.5417f,20.4102f,-2.5417f,20.4102f,-2.4375f,20.3614f,-2.4375f,20.3614f,-2.3333f,20.3125f,-2.3333f,20.3125f,-2.2292f,20.2637f,-2.2292f,20.2637f,-2.1250f,20.2149f,-2.1250f,20.2149f,-2.0208f,20.1661f,-2.0208f,20.1661f,-1.9167f,20.1172f,-1.9167f,20.1172f,-1.8125f,
	20.0684f,-1.8125f,20.0684f,-1.7083f,20.0196f,-1.7083f,20.0196f,-1.6042f,19.9707f,-1.6042f,19.9707f,-1.5000f,19.9219f,-1.5000f,19.9219f,-1.3958f,19.8731f,-1.3958f,19.8731f,-1.2917f,19.8243f,-1.2917f,19.8243f,-1.1875f,19.7754f,-1.1875f,19.7754f,-1.0833f,19.7266f,-1.0833f,
	19.7266f,-0.9792f,19.6778f,-0.9792f,19.6778f,-0.8750f,19.6289f,-0.8750f,19.6289f,-0.7708f,19.5801f,-0.7708f,19.5801f,-0.6667f,19.5313f,-0.6667f,19.5313f,-0.5625f,19.4825f,-0.5625f,19.4825f,-0.4583f,19.4336f,-0.4583f,19.4336f,-0.3542f,19.3848f,-0.3542f,19.3848f,-0.2500f,
	19.3360f,-0.2500f,19.3360f,-0.1458f,19.2871f,-0.1458f,19.2871f,-0.0417f,19.2383f,-0.0417f,19.2383f,0.0625f,19.1895f,0.0625f,19.1895f,0.1667f,19.1407f,0.1667f,19.1407f,0.2708f,19.0918f,0.2708f,19.0918f,0.3750f,19.0430f,0.3750f,19.0430f,0.4792f,18.9942f,0.4792f,
	18.9942f,0.5833f,18.9454f,0.5833f,18.9454f,0.6875f,18.8965f,0.6875f,18.8965f,0.7917f,18.8477f,0.7917f,18.8477f,0.8958f,18.7989f,0.8958f,18.7989f,1.0000f,18.7500f,1.0000f,21.0938f,-4.0000f,21.0450f,-4.0000f,21.0450f,-3.8958f,20.9961f,-3.8958f,20.9961f,-3.7917f,
	20.9473f,-3.7917f,20.9473f,-3.6875f,20.8985f,-3.6875f,20.8985f,-3.5833f,20.8497f,-3.5833f,20.8497f,-3.4792f,20.8008f,-3.4792f,20.8008f,-3.3750f,20.7520f,-3.3750f,20.7520f,-3.2708f,20.7032f,-3.2708f,20.7032f,-3.1667f,20.6543f,-3.1667f,20.6543f,-3.0625f,20.6055f,-3.0625f,
	20.6055f,-2.9583f,20.5567f,-2.9583f,20.5567f,-2.8542f,20.5079f,-2.8542f,20.5079f,-2.7500f,20.4590f,-2.7500f,20.4590f,-2.6458f,20.4102f,-2.6458f,20.4102f,-2.5417f,20.3614f,-2.5417f,20.3614f,-2.4375f,20.3125f,-2.4375f,20.3125f,-2.3333f,20.2637f,-2.3333f,20.2637f,-2.2292f,
	20.2149f,-2.2292f,20.2149f,-2.1250f,20.1661f,-2.1250f,20.1661f,-2.0208f,20.1172f,-2.0208f,20.1172f,-1.9167f,20.0684f,-1.9167f,20.0684f,-1.8125f,20.0196f,-1.8125f,20.0196f,-1.7083f,19.9707f,-1.7083f,19.9707f,-1.6042f,19.9219f,-1.6042f,19.9219f,-1.5000f,19.8731f,-1.5000f,
	19.8731f,-1.3958f,19.8243f,-1.3958f,19.8243f,-1.2917f,19.7754f,-1.2917f,19.7754f,-1.1875f,19.7266f,-1.1875f,19.7266f,-1.0833f,19.6778f,-1.0833f,19.6778f,-0.9792f,19.6289f,-0.9792f,19.6289f,-0.8750f,19.5801f,-0.8750f,19.5801f,-0.7708f,19.5313f,-0.7708f,19.5313f,-0.6667f,
	19.4825f,-0.6667f,19.4825f,-0.5625f,19.4336f,-0.5625f,19.4336f,-0.4583f,19.3848f,-0.4583f,19.3848f,-0.3542f,19.3360f,-0.3542f,19.3360f,-0.2500f,19.2871f,-0.2500f,19.2871f,-0.1458f,19.2383f,-0.1458f,19.2383f,-0.0417f,19.1895f,-0.0417f,19.1895f,0.0625f,19.1407f,0.0625f,
	19.1407f,0.1667f,19.0918f,0.1667f,19.0918f,0.2708f,19.0430f,0.2708f,19.0430f,0.3750f,18.9942f,0.3750f,18.9942f,0.4792f,18.9454f,0.4792f,18.9454f,0.5833f,18.8965f,0.5833f,18.8965f,0.6875f,18.8477f,0.6875f,18.8477f,0.7917f,18.7989f,0.7917f,18.7989f,0.8958f,
	18.7500f,0.8958f,18.7500f,1.0000f,18.7012f,1.0000f,21.0450f,-4.0000f,20.9961f,-4.0000f,20.9961f,-3.8958f,20.9473f,-3.8958f,20.9473f,-3.7917f,20.8985f,-3.7917f,20.8985f,-3.6875f,20.8497f,-3.6875f,20.8497f,-3.5833f,20.8008f,-3.5833f,20.8008f,-3.4792f,20.7520f,-3.4792f,
	20.7520f,-3.3750f,20.7032f,-3.3750f,20.7032f,-3.2708f,20.6543f,-3.2708f,20.6543f,-3.1667f,20.6055f,-3.1667f,20.6055f,-3.0625f,20.5567f,-3.0625f,20.5567f,-2.9583f,20.5079f,-2.9583f,20.5079f,-2.8542f,20.4590f,-2.8542f,20.4590f,-2.7500f,20.4102f,-2.7500f,20.4102f,-2.6458f,
	20.3614f,-2.6458f,20.3614f,-2.5417f,20.3125f,-2.5417f,20.3125f,-2.4375f,20.2637f,-2.4375f,20.2637f,-2.3333f,20.2149f,-2.3333f,20.2149f,-2.2292f,20.1661f,-2.2292f,20.1661f,-2.1250f,20.1172f,-2.1250f,20.1172f,-2.0208f,20.0684f,-2.0208f,20.0684f,-1.9167f,20.0196f,-1.9167f,
	20.0196f,-1.8125f,19.9707f,-1.8125f,19.9707f,-1.7083f,19.9219f,-1.7083f,19.9219f,-1.6042f,19.8731f,-1.6042f,19.8731f,-1.5000f,19.8243f,-1.5000f,19.8243f,-1.3958f,19.7754f,-1.3958f,19.7754f,-1.2917f,19.7266f,-1.2917f,19.7266f,-1.1875f,19.6778f,-1.1875f,19.6778f,-1.0833f,
	19.6289f,-1.0833f,19.6289f,-0.9792f,19.5801f,-0.9792f,19.5801f,-0.8750f,19.5313f,-0.8750f,19.5313f,-0.7708f,19.4825f,-0.7708f,19.4825f,-0.6667f,19.4336f,-0.6667f,19.4336f,-0.5625f,19.3848f,-0.5625f,19.3848f,-0.4583f,19.3360f,-0.4583f,19.3360f,-0.3542f,19.2871f,-0.3542f,
	19.2871f,-0.2500f,19.2383f,-0.2500f,19.2383f,-0.1458f,19.1895f,-0.1458f,19.1895f,-0.0417f,19.1407f,-0.0417f,19.1407f,0.0625f,19.0918f,0.0625f,19.0918f,0.1667f,19.0430f,0.1667f,19.0430f,0.2708f,18.9942f,0.2708f,18.9942f,0.3750f,18.9454f,0.3750f,18.9454f,0.4792f,
	18.8965f,0.4792f,18.8965f,0.5833f,18.8477f,0.5833f,18.8477f,0.6875f,18.7989f,0.6875f,18.7989f,0.7917f,18.7500f,0.7917f,18.7500f,0.8958f,18.7012f,0.8958f,18.7012f,1.0000f,18.6524f,1.0000f,20.9961f,-4.0000f,20.9473f,-4.0000f,20.9473f,-3.8958f,20.8985f,-3.8958f,
	20.8985f,-3.7917f,20.8497f,-3.7917f,20.8497f,-3.6875f,20.8008f,-3.6875f,20.8008f,-3.5833f,20.7520f,-3.5833f,20.7520f,-3.4792f,20.7032f,-3.4792f,20.7032f,-3.3750f,20.6543f,-3.3750f,20.6543f,-3.2708f,20.6055f,-3.2708f,20.6055f,-3.1667f,20.5567f,-3.1667f,20.5567f,-3.0625f,
	20.5079f,-3.0625f,20.5079f,-2.9583f,20.4590f,-2.9583f,20.4590f,-2.8542f,20.4102f,-2.8542f,20.4102f,-2.7500f,20.3614f,-2.7500f,20.3614f,-2.6458f,20.3125f,-2.6458f,20.3125f,-2.5417f,20.2637f,-2.5417f,20.2637f,-2.4375f,20.2149f,-2.4375f,20.2149f,-2.3333f,20.1661f,-2.3333f,
	20.1661f,-2.2292f,20.1172f,-2.2292f,20.1172f,-2.1250f,20.0684f,-2.1250f,20.0684f,-2.0208f,20.0196f,-2.0208f,20.0196f,-1.9167f,19.9707f,-1.9167f,19.9707f,-1.8125f,19.9219f,-1.8125f,19.9219f,-1.7083f,19.8731f,-1.7083f,19.8731f,-1.6042f,19.8243f,-1.6042f,19.8243f,-1.5000f,
	19.7754f,-1.5000f,19.7754f,-1.3958f,19.7266f,-1.3958f,19.7266f,-1.2917f,19.6778f,-1.2917f,19.6778f,-1.1875f,19.6289f,-1.1875f,19.6289f,-1.0833f,19.5801f,-1.0833f,19.5801f,-0.9792f,19.5313f,-0.9792f,19.5313f,-0.8750f,19.4825f,-0.8750f,19.4825f,-0.7708f,19.4336f,-0.7708f,
	19.4336f,-0.6667f,19.3848f,-0.6667f,19.3848f,-0.5625f,19.3360f,-0.5625f,19.3360f,-0.4583f,19.2871f,-0.4583f,19.2871f,-0.3542f,19.2383f,-0.3542f,19.2383f,-0.2500f,19.1895f,-0.2500f,19.1895f,-0.1458f,19.1407f,-0.1458f,19.1407f,-0.0417f,19.0918f,-0.0417f,19.0918f,0.0625f,
	19.0430f,0.0625f,19.0430f,0.1667f,18.9942f,0.1667f,18.9942f,0.2708f,18.9454f,0.2708f,18.9454f,0.3750f,18.8965f,0.3750f,18.8965f,0.4792f,18.8477f,0.4792f,18.8477f,0.5833f,18.7989f,0.5833f,18.7989f,0.6875f,18.7500f,0.6875f,18.7500f,0.7917f,18.7012f,0.7917f,
	18.7012f,0.8958f,18.6524f,0.8958f,18.6524f,1.0000f,18.6036f,1.0000f,20.9473f,-4.0000f,20.8985f,-4.0000f,20.8985f,-3.8958f,20.8497f,-3.8958f,20.8497f,-3.7917f,20.8008f,-3.7917f,20.8008f,-3.6875f,20.7520f,-3.6875f,20.7520f,-3.5833f,20.7032f,-3.5833f,20.7032f,-3.4792f,
	20.6543f,-3.4792f,20.6543f,-3.3750f,20.6055f,-3.3750f,20.6055f,-3.2708f,20.5567f,-3.2708f,20.5567f,-3.1667f,20.5079f,-3.1667f,20.5079f,-3.0625f,20.4590f,-3.0625f,20.4590f,-2.9583f,20.4102f,-2.9583f,20.4102f,-2.8542f,20.3614f,-2.8542f,20.3614f,-2.7500f,20.3125f,-2.7500f,
	20.3125f,-2.6458f,20.2637f,-2.6458f,20.2637f,-2.5417f,20.2149f,-2.5417f,20.2149f,-2.4375f,20.1661f,-2.4375f,20.1661f,-2.3333f,20.1172f,-2.3333f,20.1172f,-2.2292f,20.0684f,-2.2292f,20.0684f,-2.1250f,20.0196f,-2.1250f,20.0196f,-2.0208f,19.9707f,-2.0208f,19.9707f,-1.9167f,
	19.9219f,-1.9167f,19.9219f,-1.8125f,19.8731f,-1.8125f,19.8731f,-1.7083f,19.8243f,-1.7083f,19.8243f,-1.6042f,19.7754f,-1.6042f,19.7754f,-1.5000f,19.7266f,-1.5000f,19.7266f,-1.3958f,19.6778f,-1.3958f,19.6778f,-1.2917f,19.6289f,-1.2917f,19.6289f,-1.1875f,19.5801f,-1.1875f,
	19.5801f,-1.0833f,19.5313f,-1.0833f,19.5313f,-0.9792f,19.4825f,-0.9792f,19.4825f,-0.8750f,19.4336f,-0.8750f,19.4336f,-0.7708f,19.3848f,-0.7708f,19.3848f,-0.6667f,19.3360f,-0.6667f,19.3360f,-0.5625f,19.2871f,-0.5625f,19.2871f,-0.4583f,19.2383f,-0.4583f,19.2383f,-0.3542f,
	19.1895f,-0.3542f,19.1895f,-0.2500f,19.1407f,-0.2500f,19.1407f,-0.1458f,19.0918f,-0.1458f,19.0918f,-0.0417f,19.0430f,-0.0417f,19.0430f,0.0625f,18.9942f,0.0625f,18.9942f,0.1667f,18.9454f,0.1667f,18.9454f,0.2708f,18.8965f,0.2708f,18.8965f,0.3750f,18.8477f,0.3750f,
	18.8477f,0.4792f,18.7989f,0.4792f,18.7989f,0.5833f,18.7500f,0.5833f,18.7500f,0.6875f,18.7012f,0.6875f,18.7012f,0.7917f,18.6524f,0.7917f,18.6524f,0.8958f,18.6036f,0.8958f,18.6036f,1.0000f,18.5547f,1.0000f,20.8985f,-4.0000f,20.8497f,-4.0000f,20.8497f,-3.8958f,
	20.8008f,-3.8958f,20.8008f,-3.7917f,20.7520f,-3.7917f,20.7520f,-3.6875f,20.7032f,-3.6875f,20.7032f,-3.5833f,20.6543f,-3.5833f,20.6543f,-3.4792f,20.6055f,-3.4792f,20.6055f,-3.3750f,20.5567f,-3.3750f,20.5567f,-3.2708f,20.5079f,-3.2708f,20.5079f,-3.1667f,20.4590f,-3.1667f,
	20.4590f,-3.0625f,20.4102f,-3.0625f,20.4102f,-2.9583f,20.3614f,-2.9583f,20.3614f,-2.8542f,20.3125f,-2.8542f,20.3125f,-2.7500f,20.2637f,-2.7500f,20.2637f,-2.6458f,20.2149f,-2.6458f,20.2149f,-2.5417f,20.1661f,-2.5417f,20.1661f,-2.4375f,20.1172f,-2.4375f,20.1172f,-2.3333f,
	20.0684f,-2.3333f,20.0684f,-2.2292f,20.0196f,-2.2292f,20.0196f,-2.1250f,19.9707f,-2.1250f,19.9707f,-2.0208f,19.9219f,-2.0208f,19.9219f,-1.9167f,19.8731f,-1.9167f,19.8731f,-1.8125f,19.8243f,-1.8125f,19.8243f,-1.7083f,19.7754f,-1.7083f,19.7754f,-1.6042f,19.7266f,-1.6042f,
	19.7266f,-1.5000f,19.6778f,-1.5000f,19.6778f,-1.3958f,19.6289f,-1.3958f,19.6289f,-1.2917f,19.5801f,-1.2917f,19.5801f,-1.1875f,19.5313f,-1.1875f,19.5313f,-1.0833f,19.4825f,-1.0833f,19.4825f,-0.9792f,19.4336f,-0.9792f,19.4336f,-0.8750f,19.3848f,-0.8750f,19.3848f,-0.7708f,
	19.3360f,-0.7708f,19.3360f,-0.6667f,19.2871f,-0.6667f,19.2871f,-0.5625f,19.2383f,-0.5625f,19.2383f,-0.4583f,19.1895f,-0.4583f,19.1895f,-0.3542f,19.1407f,-0.3542f,19.1407f,-0.2500f,19.0918f,-0.2500f,19.0918f,-0.1458f,19.0430f,-0.1458f,19.0430f,-0.0417f,18.9942f,-0.0417f,
	18.9942f,0.0625f,18.9454f,0.0625f,18.9454f,0.1667f,18.8965f,0.1667f,18.8965f,0.2708f,18.8477f,0.2708f,18.8477f,0.3750f,18.7989f,0.3750f,18.7989f,0.4792f,18.7500f,0.4792f,18.7500f,0.5833f,18.7012f,0.5833f,18.7012f,0.6875f,18.6524f,0.6875f,18.6524f,0.7917f,
	18.6036f,0.7917f,18.6036f,0.8958f,18.5547f,0.8958f,18.5547f,1.0000f,18.5059f,1.0000f,20.8497f,-4.0000f,20.8008f,-4.0000f,20.8008f,-3.8958f,20.7520f,-3.8958f,20.7520f,-3.7917f,20.7032f,-3.7917f,20.7032f,-3.6875f,20.6543f,-3.6875f,20.6543f,-3.5833f,20.6055f,-3.5833f,
	20.6055f,-3.4792f,20.5567f,-3.4792f,20.5567f,-3.3750f,20.5079f,-3.3750f,20.5079f,-3.2708f,20.4590f,-3.2708f,20.4590f,-3.1667f,20.4102f,-3.1667f,20.4102f,-3.0625f,20.3614f,-3.0625f,20.3614f,-2.9583f,20.3125f,-2.9583f,20.3125f,-2.8542f,20.2637f,-2.8542f,20.2637f,-2.7500f,
	20.2149f,-2.7500f,20.2149f,-2.6458f,20.1661f,-2.6458f,20.1661f,-2.5417f,20.1172f,-2.5417f,20.1172f,-2.4375f,20.0684f,-2.4375f,20.0684f,-2.3333f,20.0196f,-2.3333f,20.0196f,-2.2292f,19.9707f,-2.2292f,19.9707f,-2.1250f,19.9219f,-2.1250f,19.9219f,-2.0208f,19.8731f,-2.0208f,
	19.8731f,-1.9167f,19.8243f,-1.9167f,19.8243f,-1.8125f,19.7754f,-1.8125f,19.7754f,-1.7083f,19.7266f,-1.7083f,19.7266f,-1.6042f,19.6778f,-1.6042f,19.6778f,-1.5000f,19.6289f,-1.5000f,19.6289f,-1.3958f,19.5801f,-1.3958f,19.5801f,-1.2917f,19.5313f,-1.2917f,19.5313f,-1.1875f,
	19.4825f,-1.1875f,19.4825f,-1.0833f,19.4336f,-1.0833f,19.4336f,-0.9792f,19.3848f,-0.9792f,19.3848f,-0.8750f,19.3360f,-0.8750f,19.3360f,-0.7708f,19.2871f,-0.7708f,19.2871f,-0.6667f,19.2383f,-0.6667f,19.2383f,-0.5625f,19.1895f,-0.5625f,19.1895f,-0.4583f,19.1407f,-0.4583f,
	19.1407f,-0.3542f,19.0918f,-0.3542f,19.0918f,-0.2500f,19.0430f,-0.2500f,19.0430f,-0.1458f,18.9942f,-0.1458f,18.9942f,-0.0417f,18.9454f,-0.0417f,18.9454f,0.0625f,18.8965f,0.0625f,18.8965f,0.1667f,18.8477f,0.1667f,18.8477f,0.2708f,18.7989f,0.2708f,18.7989f,0.3750f,
	18.7500f,0.3750f,18.7500f,0.4792f,18.7012f,0.4792f,18.7012f,0.5833f,18.6524f,0.5833f,18.6524f,0.6875f,18.6036f,0.6875f,18.6036f,0.7917f,18.5547f,0.7917f,18.5547f,0.8958f,18.5059f,0.8958f,18.5059f,1.0000f,18.4571f,1.0000f,20.8008f,-4.0000f,20.7520f,-4.0000f,
	20.7520f,-3.8958f,20.7032f,-3.8958f,20.7032f,-3.7917f,20.6543f,-3.7917f,20.6543f,-3.6875f,20.6055f,-3.6875f,20.6055f,-3.5833f,20.5567f,-3.5833f,20.5567f,-3.4792f,20.5079f,-3.4792f,20.5079f,-3.3750f,20.4590f,-3.3750f,20.4590f,-3.2708f,20.4102f,-3.2708f,20.4102f,-3.1667f,
	20.3614f,-3.1667f,20.3614f,-3.0625f,20.3125f,-3.0625f,20.3125f,-2.9583f,20.2637f,-2.9583f,20.2637f,-2.8542f,20.2149f,-2.8542f,20.2149f,-2.7500f,20.1661f,-2.7500f,20.1661f,-2.6458f,20.1172f,-2.6458f,20.1172f,-2.5417f,20.0684f,-2.5417f,20.0684f,-2.4375f,20.0196f,-2.4375f,
	20.0196f,-2.3333f,19.9707f,-2.3333f,19.9707f,-2.2292f,19.9219f,-2.2292f,19.9219f,-2.1250f,19.8731f,-2.1250f,19.8731f,-2.0208f,19.8243f,-2.0208f,19.8243f,-1.9167f,19.7754f,-1.9167f,19.7754f,-1.8125f,19.7266f,-1.8125f,19.7266f,-1.7083f,19.6778f,-1.7083f,19.6778f,-1.6042f,
	19.6289f,-1.6042f,19.6289f,-1.5000f,19.5801f,-1.5000f,19.5801f,-1.3958f,19.5313f,-1.3958f,19.5313f,-1.2917f,19.4825f,-1.2917f,19.4825f,-1.1875f,19.4336f,-1.1875f,19.4336f,-1.0833f,19.3848f,-1.0833f,19.3848f,-0.9792f,19.3360f,-0.9792f,19.3360f,-0.8750f,19.2871f,-0.8750f,
	19.2871f,-0.7708f,19.2383f,-0.7708f,19.2383f,-0.6667f,19.1895f,-0.6667f,19.1895f,-0.5625f,19.1407f,-0.5625f,19.1407f,-0.4583f,19.0918f,-0.4583f,19.0918f,-0.3542f,19.0430f,-0.3542f,19.0430f,-0.2500f,18.9942f,-0.2500f,18.9942f,-0.1458f,18.9454f,-0.1458f,18.9454f,-0.0417f,
	18.8965f,-0.0417f,18.8965f,0.0625f,18.8477f,0.0625f,18.8477f,0.1667f,18.7989f,0.1667f,18.7989f,0.2708f,18.7500f,0.2708f,18.7500f,0.3750f,18.7012f,0.3750f,18.7012f,0.4792f,18.6524f,0.4792f,18.6524f,0.5833f,18.6036f,0.5833f,18.6036f,0.6875f,18.5547f,0.6875f,
	18.5547f,0.7917f,18.5059f,0.7917f,18.5059f,0.8958f,18.4571f,0.8958f,18.4571f,1.0000f,18.4082f,1.0000f,20.7520f,-4.0000f,20.7032f,-4.0000f,20.7032f,-3.8958f,20.6543f,-3.8958f,20.6543f,-3.7917f,20.6055f,-3.7917f,20.6055f,-3.6875f,20.5567f,-3.6875f,20.5567f,-3.5833f,
	20.5079f,-3.5833f,20.5079f,-3.4792f,20.4590f,-3.4792f,20.4590f,-3.3750f,20.4102f,-3.3750f,20.4102f,-3.2708f,20.3614f,-3.2708f,20.3614f,-3.1667f,20.3125f,-3.1667f,20.3125f,-3.0625f,20.2637f,-3.0625f,20.2637f,-2.9583f,20.2149f,-2.9583f,20.2149f,-2.8542f,20.1661f,-2.8542f,
	20.1661f,-2.7500f,20.1172f,-2.7500f,20.1172f,-2.6458f,20.0684f,-2.6458f,20.0684f,-2.5417f,20.0196f,-2.5417f,20.0196f,-2.4375f,19.9707f,-2.4375f,19.9707f,-2.3333f,19.9219f,-2.3333f,19.9219f,-2.2292f,19.8731f,-2.2292f,19.8731f,-2.1250f,19.8243f,-2.1250f,19.8243f,-2.0208f,
	19.7754f,-2.0208f,19.7754f,-1.9167f,19.7266f,-1.9167f,19.7266f,-1.8125f,19.6778f,-1.8125f,19.6778f,-1.7083f,19.6289f,-1.7083f,19.6289f,-1.6042f,19.5801f,-1.6042f,19.5801f,-1.5000f,19.5313f,-1.5000f,19.5313f,-1.3958f,19.4825f,-1.3958f,19.4825f,-1.2917f,19.4336f,-1.2917f,
	19.4336f,-1.1875f,19.3848f,-1.1875f,19.3848f,-1.0833f,19.3360f,-1.0833f,19.3360f,-0.9792f,19.2871f,-0.9792f,19.2871f,-0.8750f,19.2383f,-0.8750f,19.2383f,-0.7708f,19.1895f,-0.7708f,19.1895f,-0.6667f,19.1407f,-0.6667f,19.1407f,-0.5625f,19.0918f,-0.5625f,19.0918f,-0.4583f,
	19.0430f,-0.4583f,19.0430f,-0.3542f,18.9942f,-0.3542f,18.9942f,-0.2500f,18.9454f,-0.2500f,18.9454f,-0.1458f,18.8965f,-0.1458f,18.8965f,-0.0417f,18.8477f,-0.0417f,18.8477f,0.0625f,18.7989f,0.0625f,18.7989f,0.1667f,18.7500f,0.1667f,18.7500f,0.2708f,18.7012f,0.2708f,
	18.7012f,0.3750f,18.6524f,0.3750f,18.6524f,0.4792f,18.6036f,0.4792f,18.6036f,0.5833f,18.5547f,0.5833f,18.5547f,0.6875f,18.5059f,0.6875f,18.5059f,0.7917f,18.4571f,0.7917f,18.4571f,0.8958f,18.4082f,0.8958f,18.4082f,1.0000f,18.3594f,1.0000f,20.7032f,-4.0000f,
	20.6543f,-4.0000f,20.6543f,-3.8958f,20.6055f,-3.8958f,20.6055f,-3.7917f,20.5567f,-3.7917f,20.5567f,-3.6875f,20.5079f,-3.6875f,20.5079f,-3.5833f,20.4590f,-3.5833f,20.4590f,-3.4792f,20.4102f,-3.4792f,20.4102f,-3.3750f,20.3614f,-3.3750f,20.3614f,-3.2708f,20.3125f,-3.2708f,
	20.3125f,-3.1667f,20.2637f,-3.1667f,20.2637f,-3.0625f,20.2149f,-3.0625f,20.2149f,-2.9583f,20.1661f,-2.9583f,20.1661f,-2.8542f,20.1172f,-2.8542f,20.1172f,-2.7500f,20.0684f,-2.7500f,20.0684f,-2.6458f,20.0196f,-2.6458f,20.0196f,-2.5417f,19.9707f,-2.5417f,19.9707f,-2.4375f,
	19.9219f,-2.4375f,19.9219f,-2.3333f,19.8731f,-2.3333f,19.8731f,-2.2292f,19.8243f,-2.2292f,19.8243f,-2.1250f,19.7754f,-2.1250f,19.7754f,-2.0208f,19.7266f,-2.0208f,19.7266f,-1.9167f,19.6778f,-1.9167f,19.6778f,-1.8125f,19.6289f,-1.8125f,19.6289f,-1.7083f,19.5801f,-1.7083f,
	19.5801f,-1.6042f,19.5313f,-1.6042f,19.5313f,-1.5000f,19.4825f,-1.5000f,19.4825f,-1.3958f,19.4336f,-1.3958f,19.4336f,-1.2917f,19.3848f,-1.2917f,19.3848f,-1.1875f,19.3360f,-1.1875f,19.3360f,-1.0833f,19.2871f,-1.0833f,19.2871f,-0.9792f,19.2383f,-0.9792f,19.2383f,-0.8750f,
	19.1895f,-0.8750f,19.1895f,-0.7708f,19.1407f,-0.7708f,19.1407f,-0.6667f,19.0918f,-0.6667f,19.0918f,-0.5625f,19.0430f,-0.5625f,19.0430f,-0.4583f,18.9942f,-0.4583f,18.9942f,-0.3542f,18.9454f,-0.3542f,18.9454f,-0.2500f,18.8965f,-0.2500f,18.8965f,-0.1458f,18.8477f,-0.1458f,
	18.8477f,-0.0417f,18.7989f,-0.0417f,18.7989f,0.0625f,18.7500f,0.0625f,18.7500f,0.1667f,18.7012f,0.1667f,18.7012f,0.2708f,18.6524f,0.2708f,18.6524f,0.3750f,18.6036f,0.3750f,18.6036f,0.4792f,18.5547f,0.4792f,18.5547f,0.5833f,18.5059f,0.5833f,18.5059f,0.6875f,
	18.4571f,0.6875f,18.4571f,0.7917f,18.4082f,0.7917f,18.4082f,0.8958f,18.3594f,0.8958f,18.3594f,1.0000f,18.3106f,1.0000f,20.6543f,-4.0000f,20.6055f,-4.0000f,20.6055f,-3.8958f,20.5567f,-3.8958f,20.5567f,-3.7917f,20.5079f,-3.7917f,20.5079f,-3.6875f,20.4590f,-3.6875f,
	20.4590f,-3.5833f,20.4102f,-3.5833f,20.4102f,-3.4792f,20.3614f,-3.4792f,20.3614f,-3.3750f,20.3125f,-3.3750f,20.3125f,-3.2708f,20.2637f,-3.2708f,20.2637f,-3.1667f,20.2149f,-3.1667f,20.2149f,-3.0625f,20.1661f,-3.0625f,20.1661f,-2.9583f,20.1172f,-2.9583f,20.1172f,-2.8542f,
	20.0684f,-2.8542f,20.0684f,-2.7500f,20.0196f,-2.7500f,20.0196f,-2.6458f,19.9707f,-2.6458f,19.9707f,-2.5417f,19.9219f,-2.5417f,19.9219f,-2.4375f,19.8731f,-2.4375f,19.8731f,-2.3333f,19.8243f,-2.3333f,19.8243f,-2.2292f,19.7754f,-2.2292f,19.7754f,-2.1250f,19.7266f,-2.1250f,
	19.7266f,-2.0208f,19.6778f,-2.0208f,19.6778f,-1.9167f,19.6289f,-1.9167f,19.6289f,-1.8125f,19.5801f,-1.8125f,19.5801f,-1.7083f,19.5313f,-1.7083f,19.5313f,-1.6042f,19.4825f,-1.6042f,19.4825f,-1.5000f,19.4336f,-1.5000f,19.4336f,-1.3958f,19.3848f,-1.3958f,19.3848f,-1.2917f,
	19.3360f,-1.2917f,19.3360f,-1.1875f,19.2871f,-1.1875f,19.2871f,-1.0833f,19.2383f,-1.0833f,19.2383f,-0.9792f,19.1895f,-0.9792f,19.1895f,-0.8750f,19.1407f,-0.8750f,19.1407f,-0.7708f,19.0918f,-0.7708f,19.0918f,-0.6667f,19.0430f,-0.6667f,19.0430f,-0.5625f,18.9942f,-0.5625f,
	18.9942f,-0.4583f,18.9454f,-0.4583f,18.9454f,-0.3542f,18.8965f,-0.3542f,18.8965f,-0.2500f,18.8477f,-0.2500f,18.8477f,-0.1458f,18.7989f,-0.1458f,18.7989f,-0.0417f,18.7500f,-0.0417f,18.7500f,0.0625f,18.7012f,0.0625f,18.7012f,0.1667f,18.6524f,0.1667f,18.6524f,0.2708f,
	18.6036f,0.2708f,18.6036f,0.3750f,18.5547f,0.3750f,18.5547f,0.4792f,18.5059f,0.4792f,18.5059f,0.5833f,18.4571f,0.5833f,18.4571f,0.6875f,18.4082f,0.6875f,18.4082f,0.7917f,18.3594f,0.7917f,18.3594f,0.8958f,18.3106f,0.8958f,18.3106f,1.0000f,18.2618f,1.0000f,
	20.6055f,-4.0000f,20.5567f,-4.0000f,20.5567f,-3.8958f,20.5079f,-3.8958f,20.5079f,-3.7917f,20.4590f,-3.7917f,20.4590f,-3.6875f,20.4102f,-3.6875f,20.4102f,-3.5833f,20.3614f,-3.5833f,20.3614f,-3.4792f,20.3125f,-3.4792f,20.3125f,-3.3750f,20.2637f,-3.3750f,20.2637f,-3.2708f,
	20.2149f,-3.2708f,20.2149f,-3.1667f,20.1661f,-3.1667f,20.1661f,-3.0625f,20.1172f,-3.0625f,20.1172f,-2.9583f,20.0684f,-2.9583f,20.0684f,-2.8542f,20.0196f,-2.8542f,20.0196f,-2.7500f,19.9707f,-2.7500f,19.9707f,-2.6458f,19.9219f,-2.6458f,19.9219f,-2.5417f,19.8731f,-2.5417f,
	19.8731f,-2.4375f,19.8243f,-2.4375f,19.8243f,-2.3333f,19.7754f,-2.3333f,19.7754f,-2.2292f,19.7266f,-2.2292f,19.7266f,-2.1250f,19.6778f,-2.1250f,19.6778f,-2.0208f,19.6289f,-2.0208f,19.6289f,-1.9167f,19.5801f,-1.9167f,19.5801f,-1.8125f,19.5313f,-1.8125f,19.5313f,-1.7083f,
	19.4825f,-1.7083f,19.4825f,-1.6042f,19.4336f,-1.6042f,19.4336f,-1.5000f,19.3848f,-1.5000f,19.3848f,-1.3958f,19.3360f,-1.3958f,19.3360f,-1.2917f,19.2871f,-1.2917f,19.2871f,-1.1875f,19.2383f,-1.1875f,19.2383f,-1.0833f,19.1895f,-1.0833f,19.1895f,-0.9792f,19.1407f,-0.9792f,
	19.1407f,-0.8750f,19.0918f,-0.8750f,19.0918f,-0.7708f,19.0430f,-0.7708f,19.0430f,-0.6667f,18.9942f,-0.6667f,18.9942f,-0.5625f,18.9454f,-0.5625f,18.9454f,-0.4583f,18.8965f,-0.4583f,18.8965f,-0.3542f,18.8477f,-0.3542f,18.8477f,-0.2500f,18.7989f,-0.2500f,18.7989f,-0.1458f,
	18.7500f,-0.1458f,18.7500f,-0.0417f,18.7012f,-0.0417f,18.7012f,0.0625f,18.6524f,0.0625f,18.6524f,0.1667f,18.6036f,0.1667f,18.6036f,0.2708f,18.5547f,0.2708f,18.5547f,0.3750f,18.5059f,0.3750f,18.5059f,0.4792f,18.4571f,0.4792f,18.4571f,0.5833f,18.4082f,0.5833f,
	18.4082f,0.6875f,18.3594f,0.6875f,18.3594f,0.7917f,18.3106f,0.7917f,18.3106f,0.8958f,18.2618f,0.8958f,18.2618f,1.0000f,18.2129f,1.0000f,20.5567f,-4.0000f,20.5079f,-4.0000f,20.5079f,-3.8958f,20.4590f,-3.8958f,20.4590f,-3.7917f,20.4102f,-3.7917f,20.4102f,-3.6875f,
	20.3614f,-3.6875f,20.3614f,-3.5833f,20.3125f,-3.5833f,20.3125f,-3.4792f,20.2637f,-3.4792f,20.2637f,-3.3750f,20.2149f,-3.3750f,20.2149f,-3.2708f,20.1661f,-3.2708f,20.1661f,-3.1667f,20.1172f,-3.1667f,20.1172f,-3.0625f,20.0684f,-3.0625f,20.0684f,-2.9583f,20.0196f,-2.9583f,
	20.0196f,-2.8542f,19.9707f,-2.8542f,19.9707f,-2.7500f,19.9219f,-2.7500f,19.9219f,-2.6458f,19.8731f,-2.6458f,19.8731f,-2.5417f,19.8243f,-2.5417f,19.8243f,-2.4375f,19.7754f,-2.4375f,19.7754f,-2.3333f,19.7266f,-2.3333f,19.7266f,-2.2292f,19.6778f,-2.2292f,19.6778f,-2.1250f,
	19.6289f,-2.1250f,19.6289f,-2.0208f,19.5801f,-2.0208f,19.5801f,-1.9167f,19.5313f,-1.9167f,19.5313f,-1.8125f,19.4825f,-1.8125f,19.4825f,-1.7083f,19.4336f,-1.7083f,19.4336f,-1.6042f,19.3848f,-1.6042f,19.3848f,-1.5000f,19.3360f,-1.5000f,19.3360f,-1.3958f,19.2871f,-1.3958f,
	19.2871f,-1.2917f,19.2383f,-1.2917f,19.2383f,-1.1875f,19.1895f,-1.1875f,19.1895f,-1.0833f,19.1407f,-1.0833f,19.1407f,-0.9792f,19.0918f,-0.9792f,19.0918f,-0.8750f,19.0430f,-0.8750f,19.0430f,-0.7708f,18.9942f,-0.7708f,18.9942f,-0.6667f,18.9454f,-0.6667f,18.9454f,-0.5625f,
	18.8965f,-0.5625f,18.8965f,-0.4583f,18.8477f,-0.4583f,18.8477f,-0.3542f,18.7989f,-0.3542f,18.7989f,-0.2500f,18.7500f,-0.2500f,18.7500f,-0.1458f,18.7012f,-0.1458f,18.7012f,-0.0417f,18.6524f,-0.0417f,18.6524f,0.0625f,18.6036f,0.0625f,18.6036f,0.1667f,18.5547f,0.1667f,
	18.5547f,0.2708f,18.5059f,0.2708f,18.5059f,0.3750f,18.4571f,0.3750f,18.4571f,0.4792f,18.4082f,0.4792f,18.4082f,0.5833f,18.3594f,0.5833f,18.3594f,0.6875f,18.3106f,0.6875f,18.3106f,0.7917f,18.2618f,0.7917f,18.2618f,0.8958f,18.2129f,0.8958f,18.2129f,1.0000f,
	18.1641f,1.0000f,20.5079f,-4.0000f,20.4590f,-4.0000f,20.4590f,-3.8958f,20.4102f,-3.8958f,20.4102f,-3.7917f,20.3614f,-3.7917f,20.3614f,-3.6875f,20.3125f,-3.6875f,20.3125f,-3.5833f,20.2637f,-3.5833f,20.2637f,-3.4792f,20.2149f,-3.4792f,20.2149f,-3.3750f,20.1661f,-3.3750f,
	20.1661f,-3.2708f,20.1172f,-3.2708f,20.1172f,-3.1667f,20.0684f,-3.1667f,20.0684f,-3.0625f,20.0196f,-3.0625f,20.0196f,-2.9583f,19.9707f,-2.9583f,19.9707f,-2.8542f,19.9219f,-2.8542f,19.9219f,-2.7500f,19.8731f,-2.7500f,19.8731f,-2.6458f,19.8243f,-2.6458f,19.8243f,-2.5417f,
	19.7754f,-2.5417f,19.7754f,-2.4375f,19.7266f,-2.4375f,19.7266f,-2.3333f,19.6778f,-2.3333f,19.6778f,-2.2292f,19.6289f,-2.2292f,19.6289f,-2.1250f,19.5801f,-2.1250f,19.5801f,-2.0208f,19.5313f,-2.0208f,19.5313f,-1.9167f,19.4825f,-1.9167f,19.4825f,-1.8125f,19.4336f,-1.8125f,
	19.4336f,-1.7083f,19.3848f,-1.7083f,19.3848f,-1.6042f,19.3360f,-1.6042f,19.3360f,-1.5000f,19.2871f,-1.5000f,19.2871f,-1.3958f,19.2383f,-1.3958f,19.2383f,-1.2917f,19.1895f,-1.2917f,19.1895f,-1.1875f,19.1407f,-1.1875f,19.1407f,-1.0833f,19.0918f,-1.0833f,19.0918f,-0.9792f,
	19.0430f,-0.9792f,19.0430f,-0.8750f,18.9942f,-0.8750f,18.9942f,-0.7708f,18.9454f,-0.7708f,18.9454f,-0.6667f,18.8965f,-0.6667f,18.8965f,-0.5625f,18.8477f,-0.5625f,18.8477f,-0.4583f,18.7989f,-0.4583f,18.7989f,-0.3542f,18.7500f,-0.3542f,18.7500f,-0.2500f,18.7012f,-0.2500f,
	18.7012f,-0.1458f,18.6524f,-0.1458f,18.6524f,-0.0417f,18.6036f,-0.0417f,18.6036f,0.0625f,18.5547f,0.0625f,18.5547f,0.1667f,18.5059f,0.1667f,18.5059f,0.2708f,18.4571f,0.2708f,18.4571f,0.3750f,18.4082f,0.3750f,18.4082f,0.4792f,18.3594f,0.4792f,18.3594f,0.5833f,
	18.3106f,0.5833f,18.3106f,0.6875f,18.2618f,0.6875f,18.2618f,0.7917f,18.2129f,0.7917f,18.2129f,0.8958f,18.1641f,0.8958f,18.1641f,1.0000f,18.1153f,1.0000f,20.4590f,-4.0000f,20.4102f,-4.0000f,20.4102f,-3.8958f,20.3614f,-3.8958f,20.3614f,-3.7917f,20.3125f,-3.7917f,
	20.3125f,-3.6875f,20.2637f,-3.6875f,20.2637f,-3.5833f,20.2149f,-3.5833f,20.2149f,-3.4792f,20.1661f,-3.4792f,20.1661f,-3.3750f,20.1172f,-3.3750f,20.1172f,-3.2708f,20.0684f,-3.2708f,20.0684f,-3.1667f,20.0196f,-3.1667f,20.0196f,-3.0625f,19.9707f,-3.0625f,19.9707f,-2.9583f,
	19.9219f,-2.9583f,19.9219f,-2.8542f,19.8731f,-2.8542f,19.8731f,-2.7500f,19.8243f,-2.7500f,19.8243f,-2.6458f,19.7754f,-2.6458f,19.7754f,-2.5417f,19.7266f,-2.5417f,19.7266f,-2.4375f,19.6778f,-2.4375f,19.6778f,-2.3333f,19.6289f,-2.3333f,19.6289f,-2.2292f,19.5801f,-2.2292f,
	19.5801f,-2.1250f,19.5313f,-2.1250f,19.5313f,-2.0208f,19.4825f,-2.0208f,19.4825f,-1.9167f,19.4336f,-1.9167f,19.4336f,-1.8125f,19.3848f,-1.8125f,19.3848f,-1.7083f,19.3360f,-1.7083f,19.3360f,-1.6042f,19.2871f,-1.6042f,19.2871f,-1.5000f,19.2383f,-1.5000f,19.2383f,-1.3958f,
	19.1895f,-1.3958f,19.1895f,-1.2917f,19.1407f,-1.2917f,19.1407f,-1.1875f,19.0918f,-1.1875f,19.0918f,-1.0833f,19.0430f,-1.0833f,19.0430f,-0.9792f,18.9942f,-0.9792f,18.9942f,-0.8750f,18.9454f,-0.8750f,18.9454f,-0.7708f,18.8965f,-0.7708f,18.8965f,-0.6667f,18.8477f,-0.6667f,
	18.8477f,-0.5625f,18.7989f,-0.5625f,18.7989f,-0.4583f,18.7500f,-0.4583f,18.7500f,-0.3542f,18.7012f,-0.3542f,18.7012f,-0.2500f,18.6524f,-0.2500f,18.6524f,-0.1458f,18.6036f,-0.1458f,18.6036f,-0.0417f,18.5547f,-0.0417f,18.5547f,0.0625f,18.5059f,0.0625f,18.5059f,0.1667f,
	18.4571f,0.1667f,18.4571f,0.2708f,18.4082f,0.2708f,18.4082f,0.3750f,18.3594f,0.3750f,18.3594f,0.4792f,18.3106f,0.4792f,18.3106f,0.5833f,18.2618f,0.5833f,18.2618f,0.6875f,18.2129f,0.6875f,18.2129f,0.7917f,18.1641f,0.7917f,18.1641f,0.8958f,18.1153f,0.8958f,
	18.1153f,1.0000f,18.0664f,1.0000f,20.4102f,-4.0000f,20.3614f,-4.0000f,20.3614f,-3.8958f,20.3125f,-3.8958f,20.3125f,-3.7917f,20.2637f,-3.7917f,20.2637f,-3.6875f,20.2149f,-3.6875f,20.2149f,-3.5833f,20.1661f,-3.5833f,20.1661f,-3.4792f,20.1172f,-3.4792f,20.1172f,-3.3750f,
	20.0684f,-3.3750f,20.0684f,-3.2708f,20.0196f,-3.2708f,20.0196f,-3.1667f,19.9707f,-3.1667f,19.9707f,-3.0625f,19.9219f,-3.0625f,19.9219f,-2.9583f,19.8731f,-2.9583f,19.8731f,-2.8542f,19.8243f,-2.8542f,19.8243f,-2.7500f,19.7754f,-2.7500f,19.7754f,-2.6458f,19.7266f,-2.6458f,
	19.7266f,-2.5417f,19.6778f,-2.5417f,19.6778f,-2.4375f,19.6289f,-2.4375f,19.6289f,-2.3333f,19.5801f,-2.3333f,19.5801f,-2.2292f,19.5313f,-2.2292f,19.5313f,-2.1250f,19.4825f,-2.1250f,19.4825f,-2.0208f,19.4336f,-2.0208f,19.4336f,-1.9167f,19.3848f,-1.9167f,19.3848f,-1.8125f,
	19.3360f,-1.8125f,19.3360f,-1.7083f,19.2871f,-1.7083f,19.2871f,-1.6042f,19.2383f,-1.6042f,19.2383f,-1.5000f,19.1895f,-1.5000f,19.1895f,-1.3958f,19.1407f,-1.3958f,19.1407f,-1.2917f,19.0918f,-1.2917f,19.0918f,-1.1875f,19.0430f,-1.1875f,19.0430f,-1.0833f,18.9942f,-1.0833f,
	18.9942f,-0.9792f,18.9454f,-0.9792f,18.9454f,-0.8750f,18.8965f,-0.8750f,18.8965f,-0.7708f,18.8477f,-0.7708f,18.8477f,-0.6667f,18.7989f,-0.6667f,18.7989f,-0.5625f,18.7500f,-0.5625f,18.7500f,-0.4583f,18.7012f,-0.4583f,18.7012f,-0.3542f,18.6524f,-0.3542f,18.6524f,-0.2500f,
	18.6036f,-0.2500f,18.6036f,-0.1458f,18.5547f,-0.1458f,18.5547f,-0.0417f,18.5059f,-0.0417f,18.5059f,0.0625f,18.4571f,0.0625f,18.4571f,0.1667f,18.4082f,0.1667f,18.4082f,0.2708f,18.3594f,0.2708f,18.3594f,0.3750f,18.3106f,0.3750f,18.3106f,0.4792f,18.2618f,0.4792f,
	18.2618f,0.5833f,18.2129f,0.5833f,18.2129f,0.6875f,18.1641f,0.6875f,18.1641f,0.7917f,18.1153f,0.7917f,18.1153f,0.8958f,18.0664f,0.8958f,18.0664f,1.0000f,18.0176f,1.0000f,20.3614f,-4.0000f,20.3125f,-4.0000f,20.3125f,-3.8958f,20.2637f,-3.8958f,20.2637f,-3.7917f,
	20.2149f,-3.7917f,20.2149f,-3.6875f,20.1661f,-3.6875f,20.1661f,-3.5833f,20.1172f,-3.5833f,20.1172f,-3.4792f,20.0684f,-3.4792f,20.0684f,-3.3750f,20.0196f,-3.3750f,20.0196f,-3.2708f,19.9707f,-3.2708f,19.9707f,-3.1667f,19.9219f,-3.1667f,19.9219f,-3.0625f,19.8731f,-3.0625f,
	19.8731f,-2.9583f,19.8243f,-2.9583f,19.8243f,-2.8542f,19.7754f,-2.8542f,19.7754f,-2.7500f,19.7266f,-2.7500f,19.7266f,-2.6458f,19.6778f,-2.6458f,19.6778f,-2.5417f,19.6289f,-2.5417f,19.6289f,-2.4375f,19.5801f,-2.4375f,19.5801f,-2.3333f,19.5313f,-2.3333f,19.5313f,-2.2292f,
	19.4825f,-2.2292f,19.4825f,-2.1250f,19.4336f,-2.1250f,19.4336f,-2.0208f,19.3848f,-2.0208f,19.3848f,-1.9167f,19.3360f,-1.9167f,19.3360f,-1.8125f,19.2871f,-1.8125f,19.2871f,-1.7083f,19.2383f,-1.7083f,19.2383f,-1.6042f,19.1895f,-1.6042f,19.1895f,-1.5000f,19.1407f,-1.5000f,
	19.1407f,-1.3958f,19.0918f,-1.3958f,19.0918f,-1.2917f,19.0430f,-1.2917f,19.0430f,-1.1875f,18.9942f,-1.1875f,18.9942f,-1.0833f,18.9454f,-1.0833f,18.9454f,-0.9792f,18.8965f,-0.9792f,18.8965f,-0.8750f,18.8477f,-0.8750f,18.8477f,-0.7708f,18.7989f,-0.7708f,18.7989f,-0.6667f,
	18.7500f,-0.6667f,18.7500f,-0.5625f,18.7012f,-0.5625f,18.7012f,-0.4583f,18.6524f,-0.4583f,18.6524f,-0.3542f,18.6036f,-0.3542f,18.6036f,-0.2500f,18.5547f,-0.2500f,18.5547f,-0.1458f,18.5059f,-0.1458f,18.5059f,-0.0417f,18.4571f,-0.0417f,18.4571f,0.0625f,18.4082f,0.0625f,
	18.4082f,0.1667f,18.3594f,0.1667f,18.3594f,0.2708f,18.3106f,0.2708f,18.3106f,0.3750f,18.2618f,0.3750f,18.2618f,0.4792f,18.2129f,0.4792f,18.2129f,0.5833f,18.1641f,0.5833f,18.1641f,0.6875f,18.1153f,0.6875f,18.1153f,0.7917f,18.0664f,0.7917f,18.0664f,0.8958f,
	18.0176f,0.8958f,18.0176f,1.0000f,17.9688f,1.0000f,20.3125f,-4.0000f,20.2637f,-4.0000f,20.2637f,-3.8958f,20.2149f,-3.8958f,20.2149f,-3.7917f,20.1661f,-3.7917f,20.1661f,-3.6875f,20.1172f,-3.6875f,20.1172f,-3.5833f,20.0684f,-3.5833f,20.0684f,-3.4792f,20.0196f,-3.4792f,
	20.0196f,-3.3750f,19.9707f,-3.3750f,19.9707f,-3.2708f,19.9219f,-3.2708f,19.9219f,-3.1667f,19.8731f,-3.1667f,19.8731f,-3.0625f,19.8243f,-3.0625f,19.8243f,-2.9583f,19.7754f,-2.9583f,19.7754f,-2.8542f,19.7266f,-2.8542f,19.7266f,-2.7500f,19.6778f,-2.7500f,19.6778f,-2.6458f,
	19.6289f,-2.6458f,19.6289f,-2.5417f,19.5801f,-2.5417f,19.5801f,-2.4375f,19.5313f,-2.4375f,19.5313f,-2.3333f,19.4825f,-2.3333f,19.4825f,-2.2292f,19.4336f,-2.2292f,19.4336f,-2.1250f,19.3848f,-2.1250f,19.3848f,-2.0208f,19.3360f,-2.0208f,19.3360f,-1.9167f,19.2871f,-1.9167f,
	19.2871f,-1.8125f,19.2383f,-1.8125f,19.2383f,-1.7083f,19.1895f,-1.7083f,19.1895f,-1.6042f,19.1407f,-1.6042f,19.1407f,-1.5000f,19.0918f,-1.5000f,19.0918f,-1.3958f,19.0430f,-1.3958f,19.0430f,-1.2917f,18.9942f,-1.2917f,18.9942f,-1.1875f,18.9454f,-1.1875f,18.9454f,-1.0833f,
	18.8965f,-1.0833f,18.8965f,-0.9792f,18.8477f,-0.9792f,18.8477f,-0.8750f,18.7989f,-0.8750f,18.7989f,-0.7708f,18.7500f,-0.7708f,18.7500f,-0.6667f,18.7012f,-0.6667f,18.7012f,-0.5625f,18.6524f,-0.5625f,18.6524f,-0.4583f,18.6036f,-0.4583f,18.6036f,-0.3542f,18.5547f,-0.3542f,
	18.5547f,-0.2500f,18.5059f,-0.2500f,18.5059f,-0.1458f,18.4571f,-0.1458f,18.4571f,-0.0417f,18.4082f,-0.0417f,18.4082f,0.0625f,18.3594f,0.0625f,18.3594f,0.1667f,18.3106f,0.1667f,18.3106f,0.2708f,18.2618f,0.2708f,18.2618f,0.3750f,18.2129f,0.3750f,18.2129f,0.4792f,
	18.1641f,0.4792f,18.1641f,0.5833f,18.1153f,0.5833f,18.1153f,0.6875f,18.0664f,0.6875f,18.0664f,0.7917f,18.0176f,0.7917f,18.0176f,0.8958f,17.9688f,0.8958f,17.9688f,1.0000f,17.9200f,1.0000f,20.2637f,-4.0000f,20.2149f,-4.0000f,20.2149f,-3.8958f,20.1661f,-3.8958f,
	20.1661f,-3.7917f,20.1172f,-3.7917f,20.1172f,-3.6875f,20.0684f,-3.6875f,20.0684f,-3.5833f,20.0196f,-3.5833f,20.0196f,-3.4792f,19.9707f,-3.4792f,19.9707f,-3.3750f,19.9219f,-3.3750f,19.9219f,-3.2708f,19.8731f,-3.2708f,19.8731f,-3.1667f,19.8243f,-3.1667f,19.8243f,-3.0625f,
	19.7754f,-3.0625f,19.7754f,-2.9583f,19.7266f,-2.9583f,19.7266f,-2.8542f,19.6778f,-2.8542f,19.6778f,-2.7500f,19.6289f,-2.7500f,19.6289f,-2.6458f,19.5801f,-2.6458f,19.5801f,-2.5417f,19.5313f,-2.5417f,19.5313f,-2.4375f,19.4825f,-2.4375f,19.4825f,-2.3333f,19.4336f,-2.3333f,
	19.4336f,-2.2292f,19.3848f,-2.2292f,19.3848f,-2.1250f,19.3360f,-2.1250f,19.3360f,-2.0208f,19.2871f,-2.0208f,19.2871f,-1.9167f,19.2383f,-1.9167f,19.2383f,-1.8125f,19.1895f,-1.8125f,19.1895f,-1.7083f,19.1407f,-1.7083f,19.1407f,-1.6042f,19.0918f,-1.6042f,19.0918f,-1.5000f,
	19.0430f,-1.5000f,19.0430f,-1.3958f,18.9942f,-1.3958f,18.9942f,-1.2917f,18.9454f,-1.2917f,18.9454f,-1.1875f,18.8965f,-1.1875f,18.8965f,-1.0833f,18.8477f,-1.0833f,18.8477f,-0.9792f,18.7989f,-0.9792f,18.7989f,-0.8750f,18.7500f,-0.8750f,18.7500f,-0.7708f,18.7012f,-0.7708f,
	18.7012f,-0.6667f,18.6524f,-0.6667f,18.6524f,-0.5625f,18.6036f,-0.5625f,18.6036f,-0.4583f,18.5547f,-0.4583f,18.5547f,-0.3542f,18.5059f,-0.3542f,18.5059f,-0.2500f,18.4571f,-0.2500f,18.4571f,-0.1458f,18.4082f,-0.1458f,18.4082f,-0.0417f,18.3594f,-0.0417f,18.3594f,0.0625f,
	18.3106f,0.0625f,18.3106f,0.1667f,18.2618f,0.1667f,18.2618f,0.2708f,18.2129f,0.2708f,18.2129f,0.3750f,18.1641f,0.3750f,18.1641f,0.4792f,18.1153f,0.4792f,18.1153f,0.5833f,18.0664f,0.5833f,18.0664f,0.6875f,18.0176f,0.6875f,18.0176f,0.7917f,17.9688f,0.7917f,
	17.9688f,0.8958f,17.9200f,0.8958f,17.9200f,1.0000f,17.8711f,1.0000f,20.2149f,-4.0000f,20.1661f,-4.0000f,20.1661f,-3.8958f,20.1172f,-3.8958f,20.1172f,-3.7917f,20.0684f,-3.7917f,20.0684f,-3.6875f,20.0196f,-3.6875f,20.0196f,-3.5833f,19.9707f,-3.5833f,19.9707f,-3.4792f,
	19.9219f,-3.4792f,19.9219f,-3.3750f,19.8731f,-3.3750f,19.8731f,-3.2708f,19.8243f,-3.2708f,19.8243f,-3.1667f,19.7754f,-3.1667f,19.7754f,-3.0625f,19.7266f,-3.0625f,19.7266f,-2.9583f,19.6778f,-2.9583f,19.6778f,-2.8542f,19.6289f,-2.8542f,19.6289f,-2.7500f,19.5801f,-2.7500f,
	19.5801f,-2.6458f,19.5313f,-2.6458f,19.5313f,-2.5417f,19.4825f,-2.5417f,19.4825f,-2.4375f,19.4336f,-2.4375f,19.4336f,-2.3333f,19.3848f,-2.3333f,19.3848f,-2.2292f,19.3360f,-2.2292f,19.3360f,-2.1250f,19.2871f,-2.1250f,19.2871f,-2.0208f,19.2383f,-2.0208f,19.2383f,-1.9167f,
	19.1895f,-1.9167f,19.1895f,-1.8125f,19.1407f,-1.8125f,19.1407f,-1.7083f,19.0918f,-1.7083f,19.0918f,-1.6042f,19.0430f,-1.6042f,19.0430f,-1.5000f,18.9942f,-1.5000f,18.9942f,-1.3958f,18.9454f,-1.3958f,18.9454f,-1.2917f,18.8965f,-1.2917f,18.8965f,-1.1875f,18.8477f,-1.1875f,
	18.8477f,-1.0833f,18.7989f,-1.0833f,18.7989f,-0.9792f,18.7500f,-0.9792f,18.7500f,-0.8750f,18.7012f,-0.8750f,18.7012f,-0.7708f,18.6524f,-0.7708f,18.6524f,-0.6667f,18.6036f,-0.6667f,18.6036f,-0.5625f,18.5547f,-0.5625f,18.5547f,-0.4583f,18.5059f,-0.4583f,18.5059f,-0.3542f,
	18.4571f,-0.3542f,18.4571f,-0.2500f,18.4082f,-0.2500f,18.4082f,-0.1458f,18.3594f,-0.1458f,18.3594f,-0.0417f,18.3106f,-0.0417f,18.3106f,0.0625f,18.2618f,0.0625f,18.2618f,0.1667f,18.2129f,0.1667f,18.2129f,0.2708f,18.1641f,0.2708f,18.1641f,0.3750f,18.1153f,0.3750f,
	18.1153f,0.4792f,18.0664f,0.4792f,18.0664f,0.5833f,18.0176f,0.5833f,18.0176f,0.6875f,17.9688f,0.6875f,17.9688f,0.7917f,17.9200f,0.7917f,17.9200f,0.8958f,17.8711f,0.8958f,17.8711f,1.0000f,17.8223f,1.0000f,20.1661f,-4.0000f,20.1172f,-4.0000f,20.1172f,-3.8958f,
	20.0684f,-3.8958f,20.0684f,-3.7917f,20.0196f,-3.7917f,20.0196f,-3.6875f,19.9707f,-3.6875f,19.9707f,-3.5833f,19.9219f,-3.5833f,19.9219f,-3.4792f,19.8731f,-3.4792f,19.8731f,-3.3750f,19.8243f,-3.3750f,19.8243f,-3.2708f,19.7754f,-3.2708f,19.7754f,-3.1667f,19.7266f,-3.1667f,
	19.7266f,-3.0625f,19.6778f,-3.0625f,19.6778f,-2.9583f,19.6289f,-2.9583f,19.6289f,-2.8542f,19.5801f,-2.8542f,19.5801f,-2.7500f,19.5313f,-2.7500f,19.5313f,-2.6458f,19.4825f,-2.6458f,19.4825f,-2.5417f,19.4336f,-2.5417f,19.4336f,-2.4375f,19.3848f,-2.4375f,19.3848f,-2.3333f,
	19.3360f,-2.3333f,19.3360f,-2.2292f,19.2871f,-2.2292f,19.2871f,-2.1250f,19.2383f,-2.1250f,19.2383f,-2.0208f,19.1895f,-2.0208f,19.1895f,-1.9167f,19.1407f,-1.9167f,19.1407f,-1.8125f,19.0918f,-1.8125f,19.0918f,-1.7083f,19.0430f,-1.7083f,19.0430f,-1.6042f,18.9942f,-1.6042f,
	18.9942f,-1.5000f,18.9454f,-1.5000f,18.9454f,-1.3958f,18.8965f,-1.3958f,18.8965f,-1.2917f,18.8477f,-1.2917f,18.8477f,-1.1875f,18.7989f,-1.1875f,18.7989f,-1.0833f,18.7500f,-1.0833f,18.7500f,-0.9792f,18.7012f,-0.9792f,18.7012f,-0.8750f,18.6524f,-0.8750f,18.6524f,-0.7708f,
	18.6036f,-0.7708f,18.6036f,-0.6667f,18.5547f,-0.6667f,18.5547f,-0.5625f,18.5059f,-0.5625f,18.5059f,-0.4583f,18.4571f,-0.4583f,18.4571f,-0.3542f,18.4082f,-0.3542f,18.4082f,-0.2500f,18.3594f,-0.2500f,18.3594f,-0.1458f,18.3106f,-0.1458f,18.3106f,-0.0417f,18.2618f,-0.0417f,
	18.2618f,0.0625f,18.2129f,0.0625f,18.2129f,0.1667f,18.1641f,0.1667f,18.1641f,0.2708f,18.1153f,0.2708f,18.1153f,0.3750f,18.0664f,0.3750f,18.0664f,0.4792f,18.0176f,0.4792f,18.0176f,0.5833f,17.9688f,0.5833f,17.9688f,0.6875f,17.9200f,0.6875f,17.9200f,0.7917f,
	17.8711f,0.7917f,17.8711f,0.8958f,17.8223f,0.8958f,17.8223f,1.0000f,17.7735f,1.0000f,20.1172f,-4.0000f,20.0684f,-4.0000f,20.0684f,-3.8958f,20.0196f,-3.8958f,20.0196f,-3.7917f,19.9707f,-3.7917f,19.9707f,-3.6875f,19.9219f,-3.6875f,19.9219f,-3.5833f,19.8731f,-3.5833f,
	19.8731f,-3.4792f,19.8243f,-3.4792f,19.8243f,-3.3750f,19.7754f,-3.3750f,19.7754f,-3.2708f,19.7266f,-3.2708f,19.7266f,-3.1667f,19.6778f,-3.1667f,19.6778f,-3.0625f,19.6289f,-3.0625f,19.6289f,-2.9583f,19.5801f,-2.9583f,19.5801f,-2.8542f,19.5313f,-2.8542f,19.5313f,-2.7500f,
	19.4825f,-2.7500f,19.4825f,-2.6458f,19.4336f,-2.6458f,19.4336f,-2.5417f,19.3848f,-2.5417f,19.3848f,-2.4375f,19.3360f,-2.4375f,19.3360f,-2.3333f,19.2871f,-2.3333f,19.2871f,-2.2292f,19.2383f,-2.2292f,19.2383f,-2.1250f,19.1895f,-2.1250f,19.1895f,-2.0208f,19.1407f,-2.0208f,
	19.1407f,-1.9167f,19.0918f,-1.9167f,19.0918f,-1.8125f,19.0430f,-1.8125f,19.0430f,-1.7083f,18.9942f,-1.7083f,18.9942f,-1.6042f,18.9454f,-1.6042f,18.9454f,-1.5000f,18.8965f,-1.5000f,18.8965f,-1.3958f,18.8477f,-1.3958f,18.8477f,-1.2917f,18.7989f,-1.2917f,18.7989f,-1.1875f,
	18.7500f,-1.1875f,18.7500f,-1.0833f,18.7012f,-1.0833f,18.7012f,-0.9792f,18.6524f,-0.9792f,18.6524f,-0.8750f,18.6036f,-0.8750f,18.6036f,-0.7708f,18.5547f,-0.7708f,18.5547f,-0.6667f,18.5059f,-0.6667f,18.5059f,-0.5625f,18.4571f,-0.5625f,18.4571f,-0.4583f,18.4082f,-0.4583f,
	18.4082f,-0.3542f,18.3594f,-0.3542f,18.3594f,-0.2500f,18.3106f,-0.2500f,18.3106f,-0.1458f,18.2618f,-0.1458f,18.2618f,-0.0417f,18.2129f,-0.0417f,18.2129f,0.0625f,18.1641f,0.0625f,18.1641f,0.1667f,18.1153f,0.1667f,18.1153f,0.2708f,18.0664f,0.2708f,18.0664f,0.3750f,
	18.0176f,0.3750f,18.0176f,0.4792f,17.9688f,0.4792f,17.9688f,0.5833f,17.9200f,0.5833f,17.9200f,0.6875f,17.8711f,0.6875f,17.8711f,0.7917f,17.8223f,0.7917f,17.8223f,0.8958f,17.7735f,0.8958f,17.7735f,1.0000f,17.7246f,1.0000f,20.0684f,-4.0000f,20.0196f,-4.0000f,
	20.0196f,-3.8958f,19.9707f,-3.8958f,19.9707f,-3.7917f,19.9219f,-3.7917f,19.9219f,-3.6875f,19.8731f,-3.6875f,19.8731f,-3.5833f,19.8243f,-3.5833f,19.8243f,-3.4792f,19.7754f,-3.4792f,19.7754f,-3.3750f,19.7266f,-3.3750f,19.7266f,-3.2708f,19.6778f,-3.2708f,19.6778f,-3.1667f,
	19.6289f,-3.1667f,19.6289f,-3.0625f,19.5801f,-3.0625f,19.5801f,-2.9583f,19.5313f,-2.9583f,19.5313f,-2.8542f,19.4825f,-2.8542f,19.4825f,-2.7500f,19.4336f,-2.7500f,19.4336f,-2.6458f,19.3848f,-2.6458f,19.3848f,-2.5417f,19.3360f,-2.5417f,19.3360f,-2.4375f,19.2871f,-2.4375f,
	19.2871f,-2.3333f,19.2383f,-2.3333f,19.2383f,-2.2292f,19.1895f,-2.2292f,19.1895f,-2.1250f,19.1407f,-2.1250f,19.1407f,-2.0208f,19.0918f,-2.0208f,19.0918f,-1.9167f,19.0430f,-1.9167f,19.0430f,-1.8125f,18.9942f,-1.8125f,18.9942f,-1.7083f,18.9454f,-1.7083f,18.9454f,-1.6042f,
	18.8965f,-1.6042f,18.8965f,-1.5000f,18.8477f,-1.5000f,18.8477f,-1.3958f,18.7989f,-1.3958f,18.7989f,-1.2917f,18.7500f,-1.2917f,18.7500f,-1.1875f,18.7012f,-1.1875f,18.7012f,-1.0833f,18.6524f,-1.0833f,18.6524f,-0.9792f,18.6036f,-0.9792f,18.6036f,-0.8750f,18.5547f,-0.8750f,
	18.5547f,-0.7708f,18.5059f,-0.7708f,18.5059f,-0.6667f,18.4571f,-0.6667f,18.4571f,-0.5625f,18.4082f,-0.5625f,18.4082f,-0.4583f,18.3594f,-0.4583f,18.3594f,-0.3542f,18.3106f,-0.3542f,18.3106f,-0.2500f,18.2618f,-0.2500f,18.2618f,-0.1458f,18.2129f,-0.1458f,18.2129f,-0.0417f,
	18.1641f,-0.0417f,18.1641f,0.0625f,18.1153f,0.0625f,18.1153f,0.1667f,18.0664f,0.1667f,18.0664f,0.2708f,18.0176f,0.2708f,18.0176f,0.3750f,17.9688f,0.3750f,17.9688f,0.4792f,17.9200f,0.4792f,17.9200f,0.5833f,17.8711f,0.5833f,17.8711f,0.6875f,17.8223f,0.6875f,
	17.8223f,0.7917f,17.7735f,0.7917f,17.7735f,0.8958f,17.7246f,0.8958f,17.7246f,1.0000f,17.6758f,1.0000f,20.0196f,-4.0000f,19.9707f,-4.0000f,19.9707f,-3.8958f,19.9219f,-3.8958f,19.9219f,-3.7917f,19.8731f,-3.7917f,19.8731f,-3.6875f,19.8243f,-3.6875f,19.8243f,-3.5833f,
	19.7754f,-3.5833f,19.7754f,-3.4792f,19.7266f,-3.4792f,19.7266f,-3.3750f,19.6778f,-3.3750f,19.6778f,-3.2708f,19.6289f,-3.2708f,19.6289f,-3.1667f,19.5801f,-3.1667f,19.5801f,-3.0625f,19.5313f,-3.0625f,19.5313f,-2.9583f,19.4825f,-2.9583f,19.4825f,-2.8542f,19.4336f,-2.8542f,
	19.4336f,-2.7500f,19.3848f,-2.7500f,19.3848f,-2.6458f,19.3360f,-2.6458f,19.3360f,-2.5417f,19.2871f,-2.5417f,19.2871f,-2.4375f,19.2383f,-2.4375f,19.2383f,-2.3333f,19.1895f,-2.3333f,19.1895f,-2.2292f,19.1407f,-2.2292f,19.1407f,-2.1250f,19.0918f,-2.1250f,19.0918f,-2.0208f,
	19.0430f,-2.0208f,19.0430f,-1.9167f,18.9942f,-1.9167f,18.9942f,-1.8125f,18.9454f,-1.8125f,18.9454f,-1.7083f,18.8965f,-1.7083f,18.8965f,-1.6042f,18.8477f,-1.6042f,18.8477f,-1.5000f,18.7989f,-1.5000f,18.7989f,-1.3958f,18.7500f,-1.3958f,18.7500f,-1.2917f,18.7012f,-1.2917f,
	18.7012f,-1.1875f,18.6524f,-1.1875f,18.6524f,-1.0833f,18.6036f,-1.0833f,18.6036f,-0.9792f,18.5547f,-0.9792f,18.5547f,-0.8750f,18.5059f,-0.8750f,18.5059f,-0.7708f,18.4571f,-0.7708f,18.4571f,-0.6667f,18.4082f,-0.6667f,18.4082f,-0.5625f,18.3594f,-0.5625f,18.3594f,-0.4583f,
	18.3106f,-0.4583f,18.3106f,-0.3542f,18.2618f,-0.3542f,18.2618f,-0.2500f,18.2129f,-0.2500f,18.2129f,-0.1458f,18.1641f,-0.1458f,18.1641f,-0.0417f,18.1153f,-0.0417f,18.1153f,0.0625f,18.0664f,0.0625f,18.0664f,0.1667f,18.0176f,0.1667f,18.0176f,0.2708f,17.9688f,0.2708f,
	17.9688f,0.3750f,17.9200f,0.3750f,17.9200f,0.4792f,17.8711f,0.4792f,17.8711f,0.5833f,17.8223f,0.5833f,17.8223f,0.6875f,17.7735f,0.6875f,17.7735f,0.7917f,17.7246f,0.7917f,17.7246f,0.8958f,17.6758f,0.8958f,17.6758f,1.0000f,17.6270f,1.0000f,19.9707f,-4.0000f,
	19.9219f,-4.0000f,19.9219f,-3.8958f,19.8731f,-3.8958f,19.8731f,-3.7917f,19.8243f,-3.7917f,19.8243f,-3.6875f,19.7754f,-3.6875f,19.7754f,-3.5833f,19.7266f,-3.5833f,19.7266f,-3.4792f,19.6778f,-3.4792f,19.6778f,-3.3750f,19.6289f,-3.3750f,19.6289f,-3.2708f,19.5801f,-3.2708f,
	19.5801f,-3.1667f,19.5313f,-3.1667f,19.5313f,-3.0625f,19.4825f,-3.0625f,19.4825f,-2.9583f,19.4336f,-2.9583f,19.4336f,-2.8542f,19.3848f,-2.8542f,19.3848f,-2.7500f,19.3360f,-2.7500f,19.3360f,-2.6458f,19.2871f,-2.6458f,19.2871f,-2.5417f,19.2383f,-2.5417f,19.2383f,-2.4375f,
	19.1895f,-2.4375f,19.1895f,-2.3333f,19.1407f,-2.3333f,19.1407f,-2.2292f,19.0918f,-2.2292f,19.0918f,-2.1250f,19.0430f,-2.1250f,19.0430f,-2.0208f,18.9942f,-2.0208f,18.9942f,-1.9167f,18.9454f,-1.9167f,18.9454f,-1.8125f,18.8965f,-1.8125f,18.8965f,-1.7083f,18.8477f,-1.7083f,
	18.8477f,-1.6042f,18.7989f,-1.6042f,18.7989f,-1.5000f,18.7500f,-1.5000f,18.7500f,-1.3958f,18.7012f,-1.3958f,18.7012f,-1.2917f,18.6524f,-1.2917f,18.6524f,-1.1875f,18.6036f,-1.1875f,18.6036f,-1.0833f,18.5547f,-1.0833f,18.5547f,-0.9792f,18.5059f,-0.9792f,18.5059f,-0.8750f,
	18.4571f,-0.8750f,18.4571f,-0.7708f,18.4082f,-0.7708f,18.4082f,-0.6667f,18.3594f,-0.6667f,18.3594f,-0.5625f,18.3106f,-0.5625f,18.3106f,-0.4583f,18.2618f,-0.4583f,18.2618f,-0.3542f,18.2129f,-0.3542f,18.2129f,-0.2500f,18.1641f,-0.2500f,18.1641f,-0.1458f,18.1153f,-0.1458f,
	18.1153f,-0.0417f,18.0664f,-0.0417f,18.0664f,0.0625f,18.0176f,0.0625f,18.0176f,0.1667f,17.9688f,0.1667f,17.9688f,0.2708f,17.9200f,0.2708f,17.9200f,0.3750f,17.8711f,0.3750f,17.8711f,0.4792f,17.8223f,0.4792f,17.8223f,0.5833f,17.7735f,0.5833f,17.7735f,0.6875f,
	17.7246f,0.6875f,17.7246f,0.7917f,17.6758f,0.7917f,17.6758f,0.8958f,17.6270f,0.8958f,17.6270f,1.0000f,17.5782f,1.0000f,19.9219f,-4.0000f,19.8731f,-4.0000f,19.8731f,-3.8958f,19.8243f,-3.8958f,19.8243f,-3.7917f,19.7754f,-3.7917f,19.7754f,-3.6875f,19.7266f,-3.6875f,
	19.7266f,-3.5833f,19.6778f,-3.5833f,19.6778f,-3.4792f,19.6289f,-3.4792f,19.6289f,-3.3750f,19.5801f,-3.3750f,19.5801f,-3.2708f,19.5313f,-3.2708f,19.5313f,-3.1667f,19.4825f,-3.1667f,19.4825f,-3.0625f,19.4336f,-3.0625f,19.4336f,-2.9583f,19.3848f,-2.9583f,19.3848f,-2.8542f,
	19.3360f,-2.8542f,19.3360f,-2.7500f,19.2871f,-2.7500f,19.2871f,-2.6458f,19.2383f,-2.6458f,19.2383f,-2.5417f,19.1895f,-2.5417f,19.1895f,-2.4375f,19.1407f,-2.4375f,19.1407f,-2.3333f,19.0918f,-2.3333f,19.0918f,-2.2292f,19.0430f,-2.2292f,19.0430f,-2.1250f,18.9942f,-2.1250f,
	18.9942f,-2.0208f,18.9454f,-2.0208f,18.9454f,-1.9167f,18.8965f,-1.9167f,18.8965f,-1.8125f,18.8477f,-1.8125f,18.8477f,-1.7083f,18.7989f,-1.7083f,18.7989f,-1.6042f,18.7500f,-1.6042f,18.7500f,-1.5000f,18.7012f,-1.5000f,18.7012f,-1.3958f,18.6524f,-1.3958f,18.6524f,-1.2917f,
	18.6036f,-1.2917f,18.6036f,-1.1875f,18.5547f,-1.1875f,18.5547f,-1.0833f,18.5059f,-1.0833f,18.5059f,-0.9792f,18.4571f,-0.9792f,18.4571f,-0.8750f,18.4082f,-0.8750f,18.4082f,-0.7708f,18.3594f,-0.7708f,18.3594f,-0.6667f,18.3106f,-0.6667f,18.3106f,-0.5625f,18.2618f,-0.5625f,
	18.2618f,-0.4583f,18.2129f,-0.4583f,18.2129f,-0.3542f,18.1641f,-0.3542f,18.1641f,-0.2500f,18.1153f,-0.2500f,18.1153f,-0.1458f,18.0664f,-0.1458f,18.0664f,-0.0417f,18.0176f,-0.0417f,18.0176f,0.0625f,17.9688f,0.0625f,17.9688f,0.1667f,17.9200f,0.1667f,17.9200f,0.2708f,
	17.8711f,0.2708f,17.8711f,0.3750f,17.8223f,0.3750f,17.8223f,0.4792f,17.7735f,0.4792f,17.7735f,0.5833f,17.7246f,0.5833f,17.7246f,0.6875f,17.6758f,0.6875f,17.6758f,0.7917f,17.6270f,0.7917f,17.6270f,0.8958f,17.5782f,0.8958f,17.5782f,1.0000f,17.5293f,1.0000f,
	19.8731f,-4.0000f,19.8243f,-4.0000f,19.8243f,-3.8958f,19.7754f,-3.8958f,19.7754f,-3.7917f,19.7266f,-3.7917f,19.7266f,-3.6875f,19.6778f,-3.6875f,19.6778f,-3.5833f,19.6289f,-3.5833f,19.6289f,-3.4792f,19.5801f,-3.4792f,19.5801f,-3.3750f,19.5313f,-3.3750f,19.5313f,-3.2708f,
	19.4825f,-3.2708f,19.4825f,-3.1667f,19.4336f,-3.1667f,19.4336f,-3.0625f,19.3848f,-3.0625f,19.3848f,-2.9583f,19.3360f,-2.9583f,19.3360f,-2.8542f,19.2871f,-2.8542f,19.2871f,-2.7500f,19.2383f,-2.7500f,19.2383f,-2.6458f,19.1895f,-2.6458f,19.1895f,-2.5417f,19.1407f,-2.5417f,
	19.1407f,-2.4375f,19.0918f,-2.4375f,19.0918f,-2.3333f,19.0430f,-2.3333f,19.0430f,-2.2292f,18.9942f,-2.2292f,18.9942f,-2.1250f,18.9454f,-2.1250f,18.9454f,-2.0208f,18.8965f,-2.0208f,18.8965f,-1.9167f,18.8477f,-1.9167f,18.8477f,-1.8125f,18.7989f,-1.8125f,18.7989f,-1.7083f,
	18.7500f,-1.7083f,18.7500f,-1.6042f,18.7012f,-1.6042f,18.7012f,-1.5000f,18.6524f,-1.5000f,18.6524f,-1.3958f,18.6036f,-1.3958f,18.6036f,-1.2917f,18.5547f,-1.2917f,18.5547f,-1.1875f,18.5059f,-1.1875f,18.5059f,-1.0833f,18.4571f,-1.0833f,18.4571f,-0.9792f,18.4082f,-0.9792f,
	18.4082f,-0.8750f,18.3594f,-0.8750f,18.3594f,-0.7708f,18.3106f,-0.7708f,18.3106f,-0.6667f,18.2618f,-0.6667f,18.2618f,-0.5625f,18.2129f,-0.5625f,18.2129f,-0.4583f,18.1641f,-0.4583f,18.1641f,-0.3542f,18.1153f,-0.3542f,18.1153f,-0.2500f,18.0664f,-0.2500f,18.0664f,-0.1458f,
	18.0176f,-0.1458f,18.0176f,-0.0417f,17.9688f,-0.0417f,17.9688f,0.0625f,17.9200f,0.0625f,17.9200f,0.1667f,17.8711f,0.1667f,17.8711f,0.2708f,17.8223f,0.2708f,17.8223f,0.3750f,17.7735f,0.3750f,17.7735f,0.4792f,17.7246f,0.4792f,17.7246f,0.5833f,17.6758f,0.5833f,
	17.6758f,0.6875f,17.6270f,0.6875f,17.6270f,0.7917f,17.5782f,0.7917f,17.5782f,0.8958f,17.5293f,0.8958f,17.5293f,1.0000f,17.4805f,1.0000f,19.8243f,-4.0000f,19.7754f,-4.0000f,19.7754f,-3.8958f,19.7266f,-3.8958f,19.7266f,-3.7917f,19.6778f,-3.7917f,19.6778f,-3.6875f,
	19.6289f,-3.6875f,19.6289f,-3.5833f,19.5801f,-3.5833f,19.5801f,-3.4792f,19.5313f,-3.4792f,19.5313f,-3.3750f,19.4825f,-3.3750f,19.4825f,-3.2708f,19.4336f,-3.2708f,19.4336f,-3.1667f,19.3848f,-3.1667f,19.3848f,-3.0625f,19.3360f,-3.0625f,19.3360f,-2.9583f,19.2871f,-2.9583f,
	19.2871f,-2.8542f,19.2383f,-2.8542f,19.2383f,-2.7500f,19.1895f,-2.7500f,19.1895f,-2.6458f,19.1407f,-2.6458f,19.1407f,-2.5417f,19.0918f,-2.5417f,19.0918f,-2.4375f,19.0430f,-2.4375f,19.0430f,-2.3333f,18.9942f,-2.3333f,18.9942f,-2.2292f,18.9454f,-2.2292f,18.9454f,-2.1250f,
	18.8965f,-2.1250f,18.8965f,-2.0208f,18.8477f,-2.0208f,18.8477f,-1.9167f,18.7989f,-1.9167f,18.7989f,-1.8125f,18.7500f,-1.8125f,18.7500f,-1.7083f,18.7012f,-1.7083f,18.7012f,-1.6042f,18.6524f,-1.6042f,18.6524f,-1.5000f,18.6036f,-1.5000f,18.6036f,-1.3958f,18.5547f,-1.3958f,
	18.5547f,-1.2917f,18.5059f,-1.2917f,18.5059f,-1.1875f,18.4571f,-1.1875f,18.4571f,-1.0833f,18.4082f,-1.0833f,18.4082f,-0.9792f,18.3594f,-0.9792f,18.3594f,-0.8750f,18.3106f,-0.8750f,18.3106f,-0.7708f,18.2618f,-0.7708f,18.2618f,-0.6667f,18.2129f,-0.6667f,18.2129f,-0.5625f,
	18.1641f,-0.5625f,18.1641f,-0.4583f,18.1153f,-0.4583f,18.1153f,-0.3542f,18.0664f,-0.3542f,18.0664f,-0.2500f,18.0176f,-0.2500f,18.0176f,-0.1458f,17.9688f,-0.1458f,17.9688f,-0.0417f,17.9200f,-0.0417f,17.9200f,0.0625f,17.8711f,0.0625f,17.8711f,0.1667f,17.8223f,0.1667f,
	17.8223f,0.2708f,17.7735f,0.2708f,17.7735f,0.3750f,17.7246f,0.3750f,17.7246f,0.4792f,17.6758f,0.4792f,17.6758f,0.5833f,17.6270f,0.5833f,17.6270f,0.6875f,17.5782f,0.6875f,17.5782f,0.7917f,17.5293f,0.7917f,17.5293f,0.8958f,17.4805f,0.8958f,17.4805f,1.0000f,
	17.4317f,1.0000f,19.7754f,-4.0000f,19.7266f,-4.0000f,19.7266f,-3.8958f,19.6778f,-3.8958f,19.6778f,-3.7917f,19.6289f,-3.7917f,19.6289f,-3.6875f,19.5801f,-3.6875f,19.5801f,-3.5833f,19.5313f,-3.5833f,19.5313f,-3.4792f,19.4825f,-3.4792f,19.4825f,-3.3750f,19.4336f,-3.3750f,
	19.4336f,-3.2708f,19.3848f,-3.2708f,19.3848f,-3.1667f,19.3360f,-3.1667f,19.3360f,-3.0625f,19.2871f,-3.0625f,19.2871f,-2.9583f,19.2383f,-2.9583f,19.2383f,-2.8542f,19.1895f,-2.8542f,19.1895f,-2.7500f,19.1407f,-2.7500f,19.1407f,-2.6458f,19.0918f,-2.6458f,19.0918f,-2.5417f,
	19.0430f,-2.5417f,19.0430f,-2.4375f,18.9942f,-2.4375f,18.9942f,-2.3333f,18.9454f,-2.3333f,18.9454f,-2.2292f,18.8965f,-2.2292f,18.8965f,-2.1250f,18.8477f,-2.1250f,18.8477f,-2.0208f,18.7989f,-2.0208f,18.7989f,-1.9167f,18.7500f,-1.9167f,18.7500f,-1.8125f,18.7012f,-1.8125f,
	18.7012f,-1.7083f,18.6524f,-1.7083f,18.6524f,-1.6042f,18.6036f,-1.6042f,18.6036f,-1.5000f,18.5547f,-1.5000f,18.5547f,-1.3958f,18.5059f,-1.3958f,18.5059f,-1.2917f,18.4571f,-1.2917f,18.4571f,-1.1875f,18.4082f,-1.1875f,18.4082f,-1.0833f,18.3594f,-1.0833f,18.3594f,-0.9792f,
	18.3106f,-0.9792f,18.3106f,-0.8750f,18.2618f,-0.8750f,18.2618f,-0.7708f,18.2129f,-0.7708f,18.2129f,-0.6667f,18.1641f,-0.6667f,18.1641f,-0.5625f,18.1153f,-0.5625f,18.1153f,-0.4583f,18.0664f,-0.4583f,18.0664f,-0.3542f,18.0176f,-0.3542f,18.0176f,-0.2500f,17.9688f,-0.2500f,
	17.9688f,-0.1458f,17.9200f,-0.1458f,17.9200f,-0.0417f,17.8711f,-0.0417f,17.8711f,0.0625f,17.8223f,0.0625f,17.8223f,0.1667f,17.7735f,0.1667f,17.7735f,0.2708f,17.7246f,0.2708f,17.7246f,0.3750f,17.6758f,0.3750f,17.6758f,0.4792f,17.6270f,0.4792f,17.6270f,0.5833f,
	17.5782f,0.5833f,17.5782f,0.6875f,17.5293f,0.6875f,17.5293f,0.7917f,17.4805f,0.7917f,17.4805f,0.8958f,17.4317f,0.8958f,17.4317f,1.0000f,17.3828f,1.0000f,19.7266f,-4.0000f,19.6778f,-4.0000f,19.6778f,-3.8958f,19.6289f,-3.8958f,19.6289f,-3.7917f,19.5801f,-3.7917f,
	19.5801f,-3.6875f,19.5313f,-3.6875f,19.5313f,-3.5833f,19.4825f,-3.5833f,19.4825f,-3.4792f,19.4336f,-3.4792f,19.4336f,-3.3750f,19.3848f,-3.3750f,19.3848f,-3.2708f,19.3360f,-3.2708f,19.3360f,-3.1667f,19.2871f,-3.1667f,19.2871f,-3.0625f,19.2383f,-3.0625f,19.2383f,-2.9583f,
	19.1895f,-2.9583f,19.1895f,-2.8542f,19.1407f,-2.8542f,19.1407f,-2.7500f,19.0918f,-2.7500f,19.0918f,-2.6458f,19.0430f,-2.6458f,19.0430f,-2.5417f,18.9942f,-2.5417f,18.9942f,-2.4375f,18.9454f,-2.4375f,18.9454f,-2.3333f,18.8965f,-2.3333f,18.8965f,-2.2292f,18.8477f,-2.2292f,
	18.8477f,-2.1250f,18.7989f,-2.1250f,18.7989f,-2.0208f,18.7500f,-2.0208f,18.7500f,-1.9167f,18.7012f,-1.9167f,18.7012f,-1.8125f,18.6524f,-1.8125f,18.6524f,-1.7083f,18.6036f,-1.7083f,18.6036f,-1.6042f,18.5547f,-1.6042f,18.5547f,-1.5000f,18.5059f,-1.5000f,18.5059f,-1.3958f,
	18.4571f,-1.3958f,18.4571f,-1.2917f,18.4082f,-1.2917f,18.4082f,-1.1875f,18.3594f,-1.1875f,18.3594f,-1.0833f,18.3106f,-1.0833f,18.3106f,-0.9792f,18.2618f,-0.9792f,18.2618f,-0.8750f,18.2129f,-0.8750f,18.2129f,-0.7708f,18.1641f,-0.7708f,18.1641f,-0.6667f,18.1153f,-0.6667f,
	18.1153f,-0.5625f,18.0664f,-0.5625f,18.0664f,-0.4583f,18.0176f,-0.4583f,18.0176f,-0.3542f,17.9688f,-0.3542f,17.9688f,-0.2500f,17.9200f,-0.2500f,17.9200f,-0.1458f,17.8711f,-0.1458f,17.8711f,-0.0417f,17.8223f,-0.0417f,17.8223f,0.0625f,17.7735f,0.0625f,17.7735f,0.1667f,
	17.7246f,0.1667f,17.7246f,0.2708f,17.6758f,0.2708f,17.6758f,0.3750f,17.6270f,0.3750f,17.6270f,0.4792f,17.5782f,0.4792f,17.5782f,0.5833f,17.5293f,0.5833f,17.5293f,0.6875f,17.4805f,0.6875f,17.4805f,0.7917f,17.4317f,0.7917f,17.4317f,0.8958f,17.3828f,0.8958f,
	17.3828f,1.0000f,17.3340f,1.0000f,19.6778f,-4.0000f,19.6289f,-4.0000f,19.6289f,-3.8958f,19.5801f,-3.8958f,19.5801f,-3.7917f,19.5313f,-3.7917f,19.5313f,-3.6875f,19.4825f,-3.6875f,19.4825f,-3.5833f,19.4336f,-3.5833f,19.4336f,-3.4792f,19.3848f,-3.4792f,19.3848f,-3.3750f,
	19.3360f,-3.3750f,19.3360f,-3.2708f,19.2871f,-3.2708f,19.2871f,-3.1667f,19.2383f,-3.1667f,19.2383f,-3.0625f,19.1895f,-3.0625f,19.1895f,-2.9583f,19.1407f,-2.9583f,19.1407f,-2.8542f,19.0918f,-2.8542f,19.0918f,-2.7500f,19.0430f,-2.7500f,19.0430f,-2.6458f,18.9942f,-2.6458f,
	18.9942f,-2.5417f,18.9454f,-2.5417f,18.9454f,-2.4375f,18.8965f,-2.4375f,18.8965f,-2.3333f,18.8477f,-2.3333f,18.8477f,-2.2292f,18.7989f,-2.2292f,18.7989f,-2.1250f,18.7500f,-2.1250f,18.7500f,-2.0208f,18.7012f,-2.0208f,18.7012f,-1.9167f,18.6524f,-1.9167f,18.6524f,-1.8125f,
	18.6036f,-1.8125f,18.6036f,-1.7083f,18.5547f,-1.7083f,18.5547f,-1.6042f,18.5059f,-1.6042f,18.5059f,-1.5000f,18.4571f,-1.5000f,18.4571f,-1.3958f,18.4082f,-1.3958f,18.4082f,-1.2917f,18.3594f,-1.2917f,18.3594f,-1.1875f,18.3106f,-1.1875f,18.3106f,-1.0833f,18.2618f,-1.0833f,
	18.2618f,-0.9792f,18.2129f,-0.9792f,18.2129f,-0.8750f,18.1641f,-0.8750f,18.1641f,-0.7708f,18.1153f,-0.7708f,18.1153f,-0.6667f,18.0664f,-0.6667f,18.0664f,-0.5625f,18.0176f,-0.5625f,18.0176f,-0.4583f,17.9688f,-0.4583f,17.9688f,-0.3542f,17.9200f,-0.3542f,17.9200f,-0.2500f,
	17.8711f,-0.2500f,17.8711f,-0.1458f,17.8223f,-0.1458f,17.8223f,-0.0417f,17.7735f,-0.0417f,17.7735f,0.0625f,17.7246f,0.0625f,17.7246f,0.1667f,17.6758f,0.1667f,17.6758f,0.2708f,17.6270f,0.2708f,17.6270f,0.3750f,17.5782f,0.3750f,17.5782f,0.4792f,17.5293f,0.4792f,
	17.5293f,0.5833f,17.4805f,0.5833f,17.4805f,0.6875f,17.4317f,0.6875f,17.4317f,0.7917f,17.3828f,0.7917f,17.3828f,0.8958f,17.3340f,0.8958f,17.3340f,1.0000f,17.2852f,1.0000f,19.6289f,-4.0000f,19.5801f,-4.0000f,19.5801f,-3.8958f,19.5313f,-3.8958f,19.5313f,-3.7917f,
	19.4825f,-3.7917f,19.4825f,-3.6875f,19.4336f,-3.6875f,19.4336f,-3.5833f,19.3848f,-3.5833f,19.3848f,-3.4792f,19.3360f,-3.4792f,19.3360f,-3.3750f,19.2871f,-3.3750f,19.2871f,-3.2708f,19.2383f,-3.2708f,19.2383f,-3.1667f,19.1895f,-3.1667f,19.1895f,-3.0625f,19.1407f,-3.0625f,
	19.1407f,-2.9583f,19.0918f,-2.9583f,19.0918f,-2.8542f,19.0430f,-2.8542f,19.0430f,-2.7500f,18.9942f,-2.7500f,18.9942f,-2.6458f,18.9454f,-2.6458f,18.9454f,-2.5417f,18.8965f,-2.5417f,18.8965f,-2.4375f,18.8477f,-2.4375f,18.8477f,-2.3333f,18.7989f,-2.3333f,18.7989f,-2.2292f,
	18.7500f,-2.2292f,18.7500f,-2.1250f,18.7012f,-2.1250f,18.7012f,-2.0208f,18.6524f,-2.0208f,18.6524f,-1.9167f,18.6036f,-1.9167f,18.6036f,-1.8125f,18.5547f,-1.8125f,18.5547f,-1.7083f,18.5059f,-1.7083f,18.5059f,-1.6042f,18.4571f,-1.6042f,18.4571f,-1.5000f,18.4082f,-1.5000f,
	18.4082f,-1.3958f,18.3594f,-1.3958f,18.3594f,-1.2917f,18.3106f,-1.2917f,18.3106f,-1.1875f,18.2618f,-1.1875f,18.2618f,-1.0833f,18.2129f,-1.0833f,18.2129f,-0.9792f,18.1641f,-0.9792f,18.1641f,-0.8750f,18.1153f,-0.8750f,18.1153f,-0.7708f,18.0664f,-0.7708f,18.0664f,-0.6667f,
	18.0176f,-0.6667f,18.0176f,-0.5625f,17.9688f,-0.5625f,17.9688f,-0.4583f,17.9200f,-0.4583f,17.9200f,-0.3542f,17.8711f,-0.3542f,17.8711f,-0.2500f,17.8223f,-0.2500f,17.8223f,-0.1458f,17.7735f,-0.1458f,17.7735f,-0.0417f,17.7246f,-0.0417f,17.7246f,0.0625f,17.6758f,0.0625f,
	17.6758f,0.1667f,17.6270f,0.1667f,17.6270f,0.2708f,17.5782f,0.2708f,17.5782f,0.3750f,17.5293f,0.3750f,17.5293f,0.4792f,17.4805f,0.4792f,17.4805f,0.5833f,17.4317f,0.5833f,17.4317f,0.6875f,17.3828f,0.6875f,17.3828f,0.7917f,17.3340f,0.7917f,17.3340f,0.8958f,
	17.2852f,0.8958f,17.2852f,1.0000f,17.2364f,1.0000f,19.5801f,-4.0000f,19.5313f,-4.0000f,19.5313f,-3.8958f,19.4825f,-3.8958f,19.4825f,-3.7917f,19.4336f,-3.7917f,19.4336f,-3.6875f,19.3848f,-3.6875f,19.3848f,-3.5833f,19.3360f,-3.5833f,19.3360f,-3.4792f,19.2871f,-3.4792f,
	19.2871f,-3.3750f,19.2383f,-3.3750f,19.2383f,-3.2708f,19.1895f,-3.2708f,19.1895f,-3.1667f,19.1407f,-3.1667f,19.1407f,-3.0625f,19.0918f,-3.0625f,19.0918f,-2.9583f,19.0430f,-2.9583f,19.0430f,-2.8542f,18.9942f,-2.8542f,18.9942f,-2.7500f,18.9454f,-2.7500f,18.9454f,-2.6458f,
	18.8965f,-2.6458f,18.8965f,-2.5417f,18.8477f,-2.5417f,18.8477f,-2.4375f,18.7989f,-2.4375f,18.7989f,-2.3333f,18.7500f,-2.3333f,18.7500f,-2.2292f,18.7012f,-2.2292f,18.7012f,-2.1250f,18.6524f,-2.1250f,18.6524f,-2.0208f,18.6036f,-2.0208f,18.6036f,-1.9167f,18.5547f,-1.9167f,
	18.5547f,-1.8125f,18.5059f,-1.8125f,18.5059f,-1.7083f,18.4571f,-1.7083f,18.4571f,-1.6042f,18.4082f,-1.6042f,18.4082f,-1.5000f,18.3594f,-1.5000f,18.3594f,-1.3958f,18.3106f,-1.3958f,18.3106f,-1.2917f,18.2618f,-1.2917f,18.2618f,-1.1875f,18.2129f,-1.1875f,18.2129f,-1.0833f,
	18.1641f,-1.0833f,18.1641f,-0.9792f,18.1153f,-0.9792f,18.1153f,-0.8750f,18.0664f,-0.8750f,18.0664f,-0.7708f,18.0176f,-0.7708f,18.0176f,-0.6667f,17.9688f,-0.6667f,17.9688f,-0.5625f,17.9200f,-0.5625f,17.9200f,-0.4583f,17.8711f,-0.4583f,17.8711f,-0.3542f,17.8223f,-0.3542f,
	17.8223f,-0.2500f,17.7735f,-0.2500f,17.7735f,-0.1458f,17.7246f,-0.1458f,17.7246f,-0.0417f,17.6758f,-0.0417f,17.6758f,0.0625f,17.6270f,0.0625f,17.6270f,0.1667f,17.5782f,0.1667f,17.5782f,0.2708f,17.5293f,0.2708f,17.5293f,0.3750f,17.4805f,0.3750f,17.4805f,0.4792f,
	17.4317f,0.4792f,17.4317f,0.5833f,17.3828f,0.5833f,17.3828f,0.6875f,17.3340f,0.6875f,17.3340f,0.7917f,17.2852f,0.7917f,17.2852f,0.8958f,17.2364f,0.8958f,17.2364f,1.0000f,17.1875f,1.0000f,19.5313f,-4.0000f,19.4825f,-4.0000f,19.4825f,-3.8958f,19.4336f,-3.8958f,
	19.4336f,-3.7917f,19.3848f,-3.7917f,19.3848f,-3.6875f,19.3360f,-3.6875f,19.3360f,-3.5833f,19.2871f,-3.5833f,19.2871f,-3.4792f,19.2383f,-3.4792f,19.2383f,-3.3750f,19.1895f,-3.3750f,19.1895f,-3.2708f,19.1407f,-3.2708f,19.1407f,-3.1667f,19.0918f,-3.1667f,19.0918f,-3.0625f,
	19.0430f,-3.0625f,19.0430f,-2.9583f,18.9942f,-2.9583f,18.9942f,-2.8542f,18.9454f,-2.8542f,18.9454f,-2.7500f,18.8965f,-2.7500f,18.8965f,-2.6458f,18.8477f,-2.6458f,18.8477f,-2.5417f,18.7989f,-2.5417f,18.7989f,-2.4375f,18.7500f,-2.4375f,18.7500f,-2.3333f,18.7012f,-2.3333f,
	18.7012f,-2.2292f,18.6524f,-2.2292f,18.6524f,-2.1250f,18.6036f,-2.1250f,18.6036f,-2.0208f,18.5547f,-2.0208f,18.5547f,-1.9167f,18.5059f,-1.9167f,18.5059f,-1.8125f,18.4571f,-1.8125f,18.4571f,-1.7083f,18.4082f,-1.7083f,18.4082f,-1.6042f,18.3594f,-1.6042f,18.3594f,-1.5000f,
	18.3106f,-1.5000f,18.3106f,-1.3958f,18.2618f,-1.3958f,18.2618f,-1.2917f,18.2129f,-1.2917f,18.2129f,-1.1875f,18.1641f,-1.1875f,18.1641f,-1.0833f,18.1153f,-1.0833f,18.1153f,-0.9792f,18.0664f,-0.9792f,18.0664f,-0.8750f,18.0176f,-0.8750f,18.0176f,-0.7708f,17.9688f,-0.7708f,
	17.9688f,-0.6667f,17.9200f,-0.6667f,17.9200f,-0.5625f,17.8711f,-0.5625f,17.8711f,-0.4583f,17.8223f,-0.4583f,17.8223f,-0.3542f,17.7735f,-0.3542f,17.7735f,-0.2500f,17.7246f,-0.2500f,17.7246f,-0.1458f,17.6758f,-0.1458f,17.6758f,-0.0417f,17.6270f,-0.0417f,17.6270f,0.0625f,
	17.5782f,0.0625f,17.5782f,0.1667f,17.5293f,0.1667f,17.5293f,0.2708f,17.4805f,0.2708f,17.4805f,0.3750f,17.4317f,0.3750f,17.4317f,0.4792f,17.3828f,0.4792f,17.3828f,0.5833f,17.3340f,0.5833f,17.3340f,0.6875f,17.2852f,0.6875f,17.2852f,0.7917f,17.2364f,0.7917f,
	17.2364f,0.8958f,17.1875f,0.8958f,17.1875f,1.0000f,17.1387f,1.0000f,19.4825f,-4.0000f,19.4336f,-4.0000f,19.4336f,-3.8958f,19.3848f,-3.8958f,19.3848f,-3.7917f,19.3360f,-3.7917f,19.3360f,-3.6875f,19.2871f,-3.6875f,19.2871f,-3.5833f,19.2383f,-3.5833f,19.2383f,-3.4792f,
	19.1895f,-3.4792f,19.1895f,-3.3750f,19.1407f,-3.3750f,19.1407f,-3.2708f,19.0918f,-3.2708f,19.0918f,-3.1667f,19.0430f,-3.1667f,19.0430f,-3.0625f,18.9942f,-3.0625f,18.9942f,-2.9583f,18.9454f,-2.9583f,18.9454f,-2.8542f,18.8965f,-2.8542f,18.8965f,-2.7500f,18.8477f,-2.7500f,
	18.8477f,-2.6458f,18.7989f,-2.6458f,18.7989f,-2.5417f,18.7500f,-2.5417f,18.7500f,-2.4375f,18.7012f,-2.4375f,18.7012f,-2.3333f,18.6524f,-2.3333f,18.6524f,-2.2292f,18.6036f,-2.2292f,18.6036f,-2.1250f,18.5547f,-2.1250f,18.5547f,-2.0208f,18.5059f,-2.0208f,18.5059f,-1.9167f,
	18.4571f,-1.9167f,18.4571f,-1.8125f,18.4082f,-1.8125f,18.4082f,-1.7083f,18.3594f,-1.7083f,18.3594f,-1.6042f,18.3106f,-1.6042f,18.3106f,-1.5000f,18.2618f,-1.5000f,18.2618f,-1.3958f,18.2129f,-1.3958f,18.2129f,-1.2917f,18.1641f,-1.2917f,18.1641f,-1.1875f,18.1153f,-1.1875f,
	18.1153f,-1.0833f,18.0664f,-1.0833f,18.0664f,-0.9792f,18.0176f,-0.9792f,18.0176f,-0.8750f,17.9688f,-0.8750f,17.9688f,-0.7708f,17.9200f,-0.7708f,17.9200f,-0.6667f,17.8711f,-0.6667f,17.8711f,-0.5625f,17.8223f,-0.5625f,17.8223f,-0.4583f,17.7735f,-0.4583f,17.7735f,-0.3542f,
	17.7246f,-0.3542f,17.7246f,-0.2500f,17.6758f,-0.2500f,17.6758f,-0.1458f,17.6270f,-0.1458f,17.6270f,-0.0417f,17.5782f,-0.0417f,17.5782f,0.0625f,17.5293f,0.0625f,17.5293f,0.1667f,17.4805f,0.1667f,17.4805f,0.2708f,17.4317f,0.2708f,17.4317f,0.3750f,17.3828f,0.3750f,
	17.3828f,0.4792f,17.3340f,0.4792f,17.3340f,0.5833f,17.2852f,0.5833f,17.2852f,0.6875f,17.2364f,0.6875f,17.2364f,0.7917f,17.1875f,0.7917f,17.1875f,0.8958f,17.1387f,0.8958f,17.1387f,1.0000f,17.0899f,1.0000f,19.4336f,-4.0000f,19.3848f,-4.0000f,19.3848f,-3.8958f,
	19.3360f,-3.8958f,19.3360f,-3.7917f,19.2871f,-3.7917f,19.2871f,-3.6875f,19.2383f,-3.6875f,19.2383f,-3.5833f,19.1895f,-3.5833f,19.1895f,-3.4792f,19.1407f,-3.4792f,19.1407f,-3.3750f,19.0918f,-3.3750f,19.0918f,-3.2708f,19.0430f,-3.2708f,19.0430f,-3.1667f,18.9942f,-3.1667f,
	18.9942f,-3.0625f,18.9454f,-3.0625f,18.9454f,-2.9583f,18.8965f,-2.9583f,18.8965f,-2.8542f,18.8477f,-2.8542f,18.8477f,-2.7500f,18.7989f,-2.7500f,18.7989f,-2.6458f,18.7500f,-2.6458f,18.7500f,-2.5417f,18.7012f,-2.5417f,18.7012f,-2.4375f,18.6524f,-2.4375f,18.6524f,-2.3333f,
	18.6036f,-2.3333f,18.6036f,-2.2292f,18.5547f,-2.2292f,18.5547f,-2.1250f,18.5059f,-2.1250f,18.5059f,-2.0208f,18.4571f,-2.0208f,18.4571f,-1.9167f,18.4082f,-1.9167f,18.4082f,-1.8125f,18.3594f,-1.8125f,18.3594f,-1.7083f,18.3106f,-1.7083f,18.3106f,-1.6042f,18.2618f,-1.6042f,
	18.2618f,-1.5000f,18.2129f,-1.5000f,18.2129f,-1.3958f,18.1641f,-1.3958f,18.1641f,-1.2917f,18.1153f,-1.2917f,18.1153f,-1.1875f,18.0664f,-1.1875f,18.0664f,-1.0833f,18.0176f,-1.0833f,18.0176f,-0.9792f,17.9688f,-0.9792f,17.9688f,-0.8750f,17.9200f,-0.8750f,17.9200f,-0.7708f,
	17.8711f,-0.7708f,17.8711f,-0.6667f,17.8223f,-0.6667f,17.8223f,-0.5625f,17.7735f,-0.5625f,17.7735f,-0.4583f,17.7246f,-0.4583f,17.7246f,-0.3542f,17.6758f,-0.3542f,17.6758f,-0.2500f,17.6270f,-0.2500f,17.6270f,-0.1458f,17.5782f,-0.1458f,17.5782f,-0.0417f,17.5293f,-0.0417f,
	17.5293f,0.0625f,17.4805f,0.0625f,17.4805f,0.1667f,17.4317f,0.1667f,17.4317f,0.2708f,17.3828f,0.2708f,17.3828f,0.3750f,17.3340f,0.3750f,17.3340f,0.4792f,17.2852f,0.4792f,17.2852f,0.5833f,17.2364f,0.5833f,17.2364f,0.6875f,17.1875f,0.6875f,17.1875f,0.7917f,
	17.1387f,0.7917f,17.1387f,0.8958f,17.0899f,0.8958f,17.0899f,1.0000f,17.0410f,1.0000f,19.3848f,-4.0000f,19.3360f,-4.0000f,19.3360f,-3.8958f,19.2871f,-3.8958f,19.2871f,-3.7917f,19.2383f,-3.7917f,19.2383f,-3.6875f,19.1895f,-3.6875f,19.1895f,-3.5833f,19.1407f,-3.5833f,
	19.1407f,-3.4792f,19.0918f,-3.4792f,19.0918f,-3.3750f,19.0430f,-3.3750f,19.0430f,-3.2708f,18.9942f,-3.2708f,18.9942f,-3.1667f,18.9454f,-3.1667f,18.9454f,-3.0625f,18.8965f,-3.0625f,18.8965f,-2.9583f,18.8477f,-2.9583f,18.8477f,-2.8542f,18.7989f,-2.8542f,18.7989f,-2.7500f,
	18.7500f,-2.7500f,18.7500f,-2.6458f,18.7012f,-2.6458f,18.7012f,-2.5417f,18.6524f,-2.5417f,18.6524f,-2.4375f,18.6036f,-2.4375f,18.6036f,-2.3333f,18.5547f,-2.3333f,18.5547f,-2.2292f,18.5059f,-2.2292f,18.5059f,-2.1250f,18.4571f,-2.1250f,18.4571f,-2.0208f,18.4082f,-2.0208f,
	18.4082f,-1.9167f,18.3594f,-1.9167f,18.3594f,-1.8125f,18.3106f,-1.8125f,18.3106f,-1.7083f,18.2618f,-1.7083f,18.2618f,-1.6042f,18.2129f,-1.6042f,18.2129f,-1.5000f,18.1641f,-1.5000f,18.1641f,-1.3958f,18.1153f,-1.3958f,18.1153f,-1.2917f,18.0664f,-1.2917f,18.0664f,-1.1875f,
	18.0176f,-1.1875f,18.0176f,-1.0833f,17.9688f,-1.0833f,17.9688f,-0.9792f,17.9200f,-0.9792f,17.9200f,-0.8750f,17.8711f,-0.8750f,17.8711f,-0.7708f,17.8223f,-0.7708f,17.8223f,-0.6667f,17.7735f,-0.6667f,17.7735f,-0.5625f,17.7246f,-0.5625f,17.7246f,-0.4583f,17.6758f,-0.4583f,
	17.6758f,-0.3542f,17.6270f,-0.3542f,17.6270f,-0.2500f,17.5782f,-0.2500f,17.5782f,-0.1458f,17.5293f,-0.1458f,17.5293f,-0.0417f,17.4805f,-0.0417f,17.4805f,0.0625f,17.4317f,0.0625f,17.4317f,0.1667f,17.3828f,0.1667f,17.3828f,0.2708f,17.3340f,0.2708f,17.3340f,0.3750f,
	17.2852f,0.3750f,17.2852f,0.4792f,17.2364f,0.4792f,17.2364f,0.5833f,17.1875f,0.5833f,17.1875f,0.6875f,17.1387f,0.6875f,17.1387f,0.7917f,17.0899f,0.7917f,17.0899f,0.8958f,17.0410f,0.8958f,17.0410f,1.0000f,16.9922f,1.0000f,19.3360f,-4.0000f,19.2871f,-4.0000f,
	19.2871f,-3.8958f,19.2383f,-3.8958f,19.2383f,-3.7917f,19.1895f,-3.7917f,19.1895f,-3.6875f,19.1407f,-3.6875f,19.1407f,-3.5833f,19.0918f,-3.5833f,19.0918f,-3.4792f,19.0430f,-3.4792f,19.0430f,-3.3750f,18.9942f,-3.3750f,18.9942f,-3.2708f,18.9454f,-3.2708f,18.9454f,-3.1667f,
	18.8965f,-3.1667f,18.8965f,-3.0625f,18.8477f,-3.0625f,18.8477f,-2.9583f,18.7989f,-2.9583f,18.7989f,-2.8542f,18.7500f,-2.8542f,18.7500f,-2.7500f,18.7012f,-2.7500f,18.7012f,-2.6458f,18.6524f,-2.6458f,18.6524f,-2.5417f,18.6036f,-2.5417f,18.6036f,-2.4375f,18.5547f,-2.4375f,
	18.5547f,-2.3333f,18.5059f,-2.3333f,18.5059f,-2.2292f,18.4571f,-2.2292f,18.4571f,-2.1250f,18.4082f,-2.1250f,18.4082f,-2.0208f,18.3594f,-2.0208f,18.3594f,-1.9167f,18.3106f,-1.9167f,18.3106f,-1.8125f,18.2618f,-1.8125f,18.2618f,-1.7083f,18.2129f,-1.7083f,18.2129f,-1.6042f,
	18.1641f,-1.6042f,18.1641f,-1.5000f,18.1153f,-1.5000f,18.1153f,-1.3958f,18.0664f,-1.3958f,18.0664f,-1.2917f,18.0176f,-1.2917f,18.0176f,-1.1875f,17.9688f,-1.1875f,17.9688f,-1.0833f,17.9200f,-1.0833f,17.9200f,-0.9792f,17.8711f,-0.9792f,17.8711f,-0.8750f,17.8223f,-0.8750f,
	17.8223f,-0.7708f,17.7735f,-0.7708f,17.7735f,-0.6667f,17.7246f,-0.6667f,17.7246f,-0.5625f,17.6758f,-0.5625f,17.6758f,-0.4583f,17.6270f,-0.4583f,17.6270f,-0.3542f,17.5782f,-0.3542f,17.5782f,-0.2500f,17.5293f,-0.2500f,17.5293f,-0.1458f,17.4805f,-0.1458f,17.4805f,-0.0417f,
	17.4317f,-0.0417f,17.4317f,0.0625f,17.3828f,0.0625f,17.3828f,0.1667f,17.3340f,0.1667f,17.3340f,0.2708f,17.2852f,0.2708f,17.2852f,0.3750f,17.2364f,0.3750f,17.2364f,0.4792f,17.1875f,0.4792f,17.1875f,0.5833f,17.1387f,0.5833f,17.1387f,0.6875f,17.0899f,0.6875f,
	17.0899f,0.7917f,17.0410f,0.7917f,17.0410f,0.8958f,16.9922f,0.8958f,16.9922f,1.0000f,16.9434f,1.0000f,19.2871f,-4.0000f,19.2383f,-4.0000f,19.2383f,-3.8958f,19.1895f,-3.8958f,19.1895f,-3.7917f,19.1407f,-3.7917f,19.1407f,-3.6875f,19.0918f,-3.6875f,19.0918f,-3.5833f,
	19.0430f,-3.5833f,19.0430f,-3.4792f,18.9942f,-3.4792f,18.9942f,-3.3750f,18.9454f,-3.3750f,18.9454f,-3.2708f,18.8965f,-3.2708f,18.8965f,-3.1667f,18.8477f,-3.1667f,18.8477f,-3.0625f,18.7989f,-3.0625f,18.7989f,-2.9583f,18.7500f,-2.9583f,18.7500f,-2.8542f,18.7012f,-2.8542f,
	18.7012f,-2.7500f,18.6524f,-2.7500f,18.6524f,-2.6458f,18.6036f,-2.6458f,18.6036f,-2.5417f,18.5547f,-2.5417f,18.5547f,-2.4375f,18.5059f,-2.4375f,18.5059f,-2.3333f,18.4571f,-2.3333f,18.4571f,-2.2292f,18.4082f,-2.2292f,18.4082f,-2.1250f,18.3594f,-2.1250f,18.3594f,-2.0208f,
	18.3106f,-2.0208f,18.3106f,-1.9167f,18.2618f,-1.9167f,18.2618f,-1.8125f,18.2129f,-1.8125f,18.2129f,-1.7083f,18.1641f,-1.7083f,18.1641f,-1.6042f,18.1153f,-1.6042f,18.1153f,-1.5000f,18.0664f,-1.5000f,18.0664f,-1.3958f,18.0176f,-1.3958f,18.0176f,-1.2917f,17.9688f,-1.2917f,
	17.9688f,-1.1875f,17.9200f,-1.1875f,17.9200f,-1.0833f,17.8711f,-1.0833f,17.8711f,-0.9792f,17.8223f,-0.9792f,17.8223f,-0.8750f,17.7735f,-0.8750f,17.7735f,-0.7708f,17.7246f,-0.7708f,17.7246f,-0.6667f,17.6758f,-0.6667f,17.6758f,-0.5625f,17.6270f,-0.5625f,17.6270f,-0.4583f,
	17.5782f,-0.4583f,17.5782f,-0.3542f,17.5293f,-0.3542f,17.5293f,-0.2500f,17.4805f,-0.2500f,17.4805f,-0.1458f,17.4317f,-0.1458f,17.4317f,-0.0417f,17.3828f,-0.0417f,17.3828f,0.0625f,17.3340f,0.0625f,17.3340f,0.1667f,17.2852f,0.1667f,17.2852f,0.2708f,17.2364f,0.2708f,
	17.2364f,0.3750f,17.1875f,0.3750f,17.1875f,0.4792f,17.1387f,0.4792f,17.1387f,0.5833f,17.0899f,0.5833f,17.0899f,0.6875f,17.0410f,0.6875f,17.0410f,0.7917f,16.9922f,0.7917f,16.9922f,0.8958f,16.9434f,0.8958f,16.9434f,1.0000f,16.8946f,1.0000f,19.2383f,-4.0000f,
	19.1895f,-4.0000f,19.1895f,-3.8958f,19.1407f,-3.8958f,19.1407f,-3.7917f,19.0918f,-3.7917f,19.0918f,-3.6875f,19.0430f,-3.6875f,19.0430f,-3.5833f,18.9942f,-3.5833f,18.9942f,-3.4792f,18.9454f,-3.4792f,18.9454f,-3.3750f,18.8965f,-3.3750f,18.8965f,-3.2708f,18.8477f,-3.2708f,
	18.8477f,-3.1667f,18.7989f,-3.1667f,18.7989f,-3.0625f,18.7500f,-3.0625f,18.7500f,-2.9583f,18.7012f,-2.9583f,18.7012f,-2.8542f,18.6524f,-2.8542f,18.6524f,-2.7500f,18.6036f,-2.7500f,18.6036f,-2.6458f,18.5547f,-2.6458f,18.5547f,-2.5417f,18.5059f,-2.5417f,18.5059f,-2.4375f,
	18.4571f,-2.4375f,18.4571f,-2.3333f,18.4082f,-2.3333f,18.4082f,-2.2292f,18.3594f,-2.2292f,18.3594f,-2.1250f,18.3106f,-2.1250f,18.3106f,-2.0208f,18.2618f,-2.0208f,18.2618f,-1.9167f,18.2129f,-1.9167f,18.2129f,-1.8125f,18.1641f,-1.8125f,18.1641f,-1.7083f,18.1153f,-1.7083f,
	18.1153f,-1.6042f,18.0664f,-1.6042f,18.0664f,-1.5000f,18.0176f,-1.5000f,18.0176f,-1.3958f,17.9688f,-1.3958f,17.9688f,-1.2917f,17.9200f,-1.2917f,17.9200f,-1.1875f,17.8711f,-1.1875f,17.8711f,-1.0833f,17.8223f,-1.0833f,17.8223f,-0.9792f,17.7735f,-0.9792f,17.7735f,-0.8750f,
	17.7246f,-0.8750f,17.7246f,-0.7708f,17.6758f,-0.7708f,17.6758f,-0.6667f,17.6270f,-0.6667f,17.6270f,-0.5625f,17.5782f,-0.5625f,17.5782f,-0.4583f,17.5293f,-0.4583f,17.5293f,-0.3542f,17.4805f,-0.3542f,17.4805f,-0.2500f,17.4317f,-0.2500f,17.4317f,-0.1458f,17.3828f,-0.1458f,
	17.3828f,-0.0417f,17.3340f,-0.0417f,17.3340f,0.0625f,17.2852f,0.0625f,17.2852f,0.1667f,17.2364f,0.1667f,17.2364f,0.2708f,17.1875f,0.2708f,17.1875f,0.3750f,17.1387f,0.3750f,17.1387f,0.4792f,17.0899f,0.4792f,17.0899f,0.5833f,17.0410f,0.5833f,17.0410f,0.6875f,
	16.9922f,0.6875f,16.9922f,0.7917f,16.9434f,0.7917f,16.9434f,0.8958f,16.8946f,0.8958f,16.8946f,1.0000f,16.8457f,1.0000f,19.1895f,-4.0000f,19.1407f,-4.0000f,19.1407f,-3.8958f,19.0918f,-3.8958f,19.0918f,-3.7917f,19.0430f,-3.7917f,19.0430f,-3.6875f,18.9942f,-3.6875f,
	18.9942f,-3.5833f,18.9454f,-3.5833f,18.9454f,-3.4792f,18.8965f,-3.4792f,18.8965f,-3.3750f,18.8477f,-3.3750f,18.8477f,-3.2708f,18.7989f,-3.2708f,18.7989f,-3.1667f,18.7500f,-3.1667f,18.7500f,-3.0625f,18.7012f,-3.0625f,18.7012f,-2.9583f,18.6524f,-2.9583f,18.6524f,-2.8542f,
	18.6036f,-2.8542f,18.6036f,-2.7500f,18.5547f,-2.7500f,18.5547f,-2.6458f,18.5059f,-2.6458f,18.5059f,-2.5417f,18.4571f,-2.5417f,18.4571f,-2.4375f,18.4082f,-2.4375f,18.4082f,-2.3333f,18.3594f,-2.3333f,18.3594f,-2.2292f,18.3106f,-2.2292f,18.3106f,-2.1250f,18.2618f,-2.1250f,
	18.2618f,-2.0208f,18.2129f,-2.0208f,18.2129f,-1.9167f,18.1641f,-1.9167f,18.1641f,-1.8125f,18.1153f,-1.8125f,18.1153f,-1.7083f,18.0664f,-1.7083f,18.0664f,-1.6042f,18.0176f,-1.6042f,18.0176f,-1.5000f,17.9688f,-1.5000f,17.9688f,-1.3958f,17.9200f,-1.3958f,17.9200f,-1.2917f,
	17.8711f,-1.2917f,17.8711f,-1.1875f,17.8223f,-1.1875f,17.8223f,-1.0833f,17.7735f,-1.0833f,17.7735f,-0.9792f,17.7246f,-0.9792f,17.7246f,-0.8750f,17.6758f,-0.8750f,17.6758f,-0.7708f,17.6270f,-0.7708f,17.6270f,-0.6667f,17.5782f,-0.6667f,17.5782f,-0.5625f,17.5293f,-0.5625f,
	17.5293f,-0.4583f,17.4805f,-0.4583f,17.4805f,-0.3542f,17.4317f,-0.3542f,17.4317f,-0.2500f,17.3828f,-0.2500f,17.3828f,-0.1458f,17.3340f,-0.1458f,17.3340f,-0.0417f,17.2852f,-0.0417f,17.2852f,0.0625f,17.2364f,0.0625f,17.2364f,0.1667f,17.1875f,0.1667f,17.1875f,0.2708f,
	17.1387f,0.2708f,17.1387f,0.3750f,17.0899f,0.3750f,17.0899f,0.4792f,17.0410f,0.4792f,17.0410f,0.5833f,16.9922f,0.5833f,16.9922f,0.6875f,16.9434f,0.6875f,16.9434f,0.7917f,16.8946f,0.7917f,16.8946f,0.8958f,16.8457f,0.8958f,16.8457f,1.0000f,16.7969f,1.0000f,
	19.1407f,-4.0000f,19.0918f,-4.0000f,19.0918f,-3.8958f,19.0430f,-3.8958f,19.0430f,-3.7917f,18.9942f,-3.7917f,18.9942f,-3.6875f,18.9454f,-3.6875f,18.9454f,-3.5833f,18.8965f,-3.5833f,18.8965f,-3.4792f,18.8477f,-3.4792f,18.8477f,-3.3750f,18.7989f,-3.3750f,18.7989f,-3.2708f,
	18.7500f,-3.2708f,18.7500f,-3.1667f,18.7012f,-3.1667f,18.7012f,-3.0625f,18.6524f,-3.0625f,18.6524f,-2.9583f,18.6036f,-2.9583f,18.6036f,-2.8542f,18.5547f,-2.8542f,18.5547f,-2.7500f,18.5059f,-2.7500f,18.5059f,-2.6458f,18.4571f,-2.6458f,18.4571f,-2.5417f,18.4082f,-2.5417f,
	18.4082f,-2.4375f,18.3594f,-2.4375f,18.3594f,-2.3333f,18.3106f,-2.3333f,18.3106f,-2.2292f,18.2618f,-2.2292f,18.2618f,-2.1250f,18.2129f,-2.1250f,18.2129f,-2.0208f,18.1641f,-2.0208f,18.1641f,-1.9167f,18.1153f,-1.9167f,18.1153f,-1.8125f,18.0664f,-1.8125f,18.0664f,-1.7083f,
	18.0176f,-1.7083f,18.0176f,-1.6042f,17.9688f,-1.6042f,17.9688f,-1.5000f,17.9200f,-1.5000f,17.9200f,-1.3958f,17.8711f,-1.3958f,17.8711f,-1.2917f,17.8223f,-1.2917f,17.8223f,-1.1875f,17.7735f,-1.1875f,17.7735f,-1.0833f,17.7246f,-1.0833f,17.7246f,-0.9792f,17.6758f,-0.9792f,
	17.6758f,-0.8750f,17.6270f,-0.8750f,17.6270f,-0.7708f,17.5782f,-0.7708f,17.5782f,-0.6667f,17.5293f,-0.6667f,17.5293f,-0.5625f,17.4805f,-0.5625f,17.4805f,-0.4583f,17.4317f,-0.4583f,17.4317f,-0.3542f,17.3828f,-0.3542f,17.3828f,-0.2500f,17.3340f,-0.2500f,17.3340f,-0.1458f,
	17.2852f,-0.1458f,17.2852f,-0.0417f,17.2364f,-0.0417f,17.2364f,0.0625f,17.1875f,0.0625f,17.1875f,0.1667f,17.1387f,0.1667f,17.1387f,0.2708f,17.0899f,0.2708f,17.0899f,0.3750f,17.0410f,0.3750f,17.0410f,0.4792f,16.9922f,0.4792f,16.9922f,0.5833f,16.9434f,0.5833f,
	16.9434f,0.6875f,16.8946f,0.6875f,16.8946f,0.7917f,16.8457f,0.7917f,16.8457f,0.8958f,16.7969f,0.8958f,16.7969f,1.0000f,16.7481f,1.0000f,19.0918f,-4.0000f,19.0430f,-4.0000f,19.0430f,-3.8958f,18.9942f,-3.8958f,18.9942f,-3.7917f,18.9454f,-3.7917f,18.9454f,-3.6875f,
	18.8965f,-3.6875f,18.8965f,-3.5833f,18.8477f,-3.5833f,18.8477f,-3.4792f,18.7989f,-3.4792f,18.7989f,-3.3750f,18.7500f,-3.3750f,18.7500f,-3.2708f,18.7012f,-3.2708f,18.7012f,-3.1667f,18.6524f,-3.1667f,18.6524f,-3.0625f,18.6036f,-3.0625f,18.6036f,-2.9583f,18.5547f,-2.9583f,
	18.5547f,-2.8542f,18.5059f,-2.8542f,18.5059f,-2.7500f,18.4571f,-2.7500f,18.4571f,-2.6458f,18.4082f,-2.6458f,18.4082f,-2.5417f,18.3594f,-2.5417f,18.3594f,-2.4375f,18.3106f,-2.4375f,18.3106f,-2.3333f,18.2618f,-2.3333f,18.2618f,-2.2292f,18.2129f,-2.2292f,18.2129f,-2.1250f,
	18.1641f,-2.1250f,18.1641f,-2.0208f,18.1153f,-2.0208f,18.1153f,-1.9167f,18.0664f,-1.9167f,18.0664f,-1.8125f,18.0176f,-1.8125f,18.0176f,-1.7083f,17.9688f,-1.7083f,17.9688f,-1.6042f,17.9200f,-1.6042f,17.9200f,-1.5000f,17.8711f,-1.5000f,17.8711f,-1.3958f,17.8223f,-1.3958f,
	17.8223f,-1.2917f,17.7735f,-1.2917f,17.7735f,-1.1875f,17.7246f,-1.1875f,17.7246f,-1.0833f,17.6758f,-1.0833f,17.6758f,-0.9792f,17.6270f,-0.9792f,17.6270f,-0.8750f,17.5782f,-0.8750f,17.5782f,-0.7708f,17.5293f,-0.7708f,17.5293f,-0.6667f,17.4805f,-0.6667f,17.4805f,-0.5625f,
	17.4317f,-0.5625f,17.4317f,-0.4583f,17.3828f,-0.4583f,17.3828f,-0.3542f,17.3340f,-0.3542f,17.3340f,-0.2500f,17.2852f,-0.2500f,17.2852f,-0.1458f,17.2364f,-0.1458f,17.2364f,-0.0417f,17.1875f,-0.0417f,17.1875f,0.0625f,17.1387f,0.0625f,17.1387f,0.1667f,17.0899f,0.1667f,
	17.0899f,0.2708f,17.0410f,0.2708f,17.0410f,0.3750f,16.9922f,0.3750f,16.9922f,0.4792f,16.9434f,0.4792f,16.9434f,0.5833f,16.8946f,0.5833f,16.8946f,0.6875f,16.8457f,0.6875f,16.8457f,0.7917f,16.7969f,0.7917f,16.7969f,0.8958f,16.7481f,0.8958f,16.7481f,1.0000f,
	16.6993f,1.0000f,19.0430f,-4.0000f,18.9942f,-4.0000f,18.9942f,-3.8958f,18.9454f,-3.8958f,18.9454f,-3.7917f,18.8965f,-3.7917f,18.8965f,-3.6875f,18.8477f,-3.6875f,18.8477f,-3.5833f,18.7989f,-3.5833f,18.7989f,-3.4792f,18.7500f,-3.4792f,18.7500f,-3.3750f,18.7012f,-3.3750f,
	18.7012f,-3.2708f,18.6524f,-3.2708f,18.6524f,-3.1667f,18.6036f,-3.1667f,18.6036f,-3.0625f,18.5547f,-3.0625f,18.5547f,-2.9583f,18.5059f,-2.9583f,18.5059f,-2.8542f,18.4571f,-2.8542f,18.4571f,-2.7500f,18.4082f,-2.7500f,18.4082f,-2.6458f,18.3594f,-2.6458f,18.3594f,-2.5417f,
	18.3106f,-2.5417f,18.3106f,-2.4375f,18.2618f,-2.4375f,18.2618f,-2.3333f,18.2129f,-2.3333f,18.2129f,-2.2292f,18.1641f,-2.2292f,18.1641f,-2.1250f,18.1153f,-2.1250f,18.1153f,-2.0208f,18.0664f,-2.0208f,18.0664f,-1.9167f,18.0176f,-1.9167f,18.0176f,-1.8125f,17.9688f,-1.8125f,
	17.9688f,-1.7083f,17.9200f,-1.7083f,17.9200f,-1.6042f,17.8711f,-1.6042f,17.8711f,-1.5000f,17.8223f,-1.5000f,17.8223f,-1.3958f,17.7735f,-1.3958f,17.7735f,-1.2917f,17.7246f,-1.2917f,17.7246f,-1.1875f,17.6758f,-1.1875f,17.6758f,-1.0833f,17.6270f,-1.0833f,17.6270f,-0.9792f,
	17.5782f,-0.9792f,17.5782f,-0.8750f,17.5293f,-0.8750f,17.5293f,-0.7708f,17.4805f,-0.7708f,17.4805f,-0.6667f,17.4317f,-0.6667f,17.4317f,-0.5625f,17.3828f,-0.5625f,17.3828f,-0.4583f,17.3340f,-0.4583f,17.3340f,-0.3542f,17.2852f,-0.3542f,17.2852f,-0.2500f,17.2364f,-0.2500f,
	17.2364f,-0.1458f,17.1875f,-0.1458f,17.1875f,-0.0417f,17.1387f,-0.0417f,17.1387f,0.0625f,17.0899f,0.0625f,17.0899f,0.1667f,17.0410f,0.1667f,17.0410f,0.2708f,16.9922f,0.2708f,16.9922f,0.3750f,16.9434f,0.3750f,16.9434f,0.4792f,16.8946f,0.4792f,16.8946f,0.5833f,
	16.8457f,0.5833f,16.8457f,0.6875f,16.7969f,0.6875f,16.7969f,0.7917f,16.7481f,0.7917f,16.7481f,0.8958f,16.6993f,0.8958f,16.6993f,1.0000f,16.6504f,1.0000f,18.9942f,-4.0000f,18.9454f,-4.0000f,18.9454f,-3.8958f,18.8965f,-3.8958f,18.8965f,-3.7917f,18.8477f,-3.7917f,
	18.8477f,-3.6875f,18.7989f,-3.6875f,18.7989f,-3.5833f,18.7500f,-3.5833f,18.7500f,-3.4792f,18.7012f,-3.4792f,18.7012f,-3.3750f,18.6524f,-3.3750f,18.6524f,-3.2708f,18.6036f,-3.2708f,18.6036f,-3.1667f,18.5547f,-3.1667f,18.5547f,-3.0625f,18.5059f,-3.0625f,18.5059f,-2.9583f,
	18.4571f,-2.9583f,18.4571f,-2.8542f,18.4082f,-2.8542f,18.4082f,-2.7500f,18.3594f,-2.7500f,18.3594f,-2.6458f,18.3106f,-2.6458f,18.3106f,-2.5417f,18.2618f,-2.5417f,18.2618f,-2.4375f,18.2129f,-2.4375f,18.2129f,-2.3333f,18.1641f,-2.3333f,18.1641f,-2.2292f,18.1153f,-2.2292f,
	18.1153f,-2.1250f,18.0664f,-2.1250f,18.0664f,-2.0208f,18.0176f,-2.0208f,18.0176f,-1.9167f,17.9688f,-1.9167f,17.9688f,-1.8125f,17.9200f,-1.8125f,17.9200f,-1.7083f,17.8711f,-1.7083f,17.8711f,-1.6042f,17.8223f,-1.6042f,17.8223f,-1.5000f,17.7735f,-1.5000f,17.7735f,-1.3958f,
	17.7246f,-1.3958f,17.7246f,-1.2917f,17.6758f,-1.2917f,17.6758f,-1.1875f,17.6270f,-1.1875f,17.6270f,-1.0833f,17.5782f,-1.0833f,17.5782f,-0.9792f,17.5293f,-0.9792f,17.5293f,-0.8750f,17.4805f,-0.8750f,17.4805f,-0.7708f,17.4317f,-0.7708f,17.4317f,-0.6667f,17.3828f,-0.6667f,
	17.3828f,-0.5625f,17.3340f,-0.5625f,17.3340f,-0.4583f,17.2852f,-0.4583f,17.2852f,-0.3542f,17.2364f,-0.3542f,17.2364f,-0.2500f,17.1875f,-0.2500f,17.1875f,-0.1458f,17.1387f,-0.1458f,17.1387f,-0.0417f,17.0899f,-0.0417f,17.0899f,0.0625f,17.0410f,0.0625f,17.0410f,0.1667f,
	16.9922f,0.1667f,16.9922f,0.2708f,16.9434f,0.2708f,16.9434f,0.3750f,16.8946f,0.3750f,16.8946f,0.4792f,16.8457f,0.4792f,16.8457f,0.5833f,16.7969f,0.5833f,16.7969f,0.6875f,16.7481f,0.6875f,16.7481f,0.7917f,16.6993f,0.7917f,16.6993f,0.8958f,16.6504f,0.8958f,
	16.6504f,1.0000f,16.6016f,1.0000f,18.9454f,-4.0000f,18.8965f,-4.0000f,18.8965f,-3.8958f,18.8477f,-3.8958f,18.8477f,-3.7917f,18.7989f,-3.7917f,18.7989f,-3.6875f,18.7500f,-3.6875f,18.7500f,-3.5833f,18.7012f,-3.5833f,18.7012f,-3.4792f,18.6524f,-3.4792f,18.6524f,-3.3750f,
	18.6036f,-3.3750f,18.6036f,-3.2708f,18.5547f,-3.2708f,18.5547f,-3.1667f,18.5059f,-3.1667f,18.5059f,-3.0625f,18.4571f,-3.0625f,18.4571f,-2.9583f,18.4082f,-2.9583f,18.4082f,-2.8542f,18.3594f,-2.8542f,18.3594f,-2.7500f,18.3106f,-2.7500f,18.3106f,-2.6458f,18.2618f,-2.6458f,
	18.2618f,-2.5417f,18.2129f,-2.5417f,18.2129f,-2.4375f,18.1641f,-2.4375f,18.1641f,-2.3333f,18.1153f,-2.3333f,18.1153f,-2.2292f,18.0664f,-2.2292f,18.0664f,-2.1250f,18.0176f,-2.1250f,18.0176f,-2.0208f,17.9688f,-2.0208f,17.9688f,-1.9167f,17.9200f,-1.9167f,17.9200f,-1.8125f,
	17.8711f,-1.8125f,17.8711f,-1.7083f,17.8223f,-1.7083f,17.8223f,-1.6042f,17.7735f,-1.6042f,17.7735f,-1.5000f,17.7246f,-1.5000f,17.7246f,-1.3958f,17.6758f,-1.3958f,17.6758f,-1.2917f,17.6270f,-1.2917f,17.6270f,-1.1875f,17.5782f,-1.1875f,17.5782f,-1.0833f,17.5293f,-1.0833f,
	17.5293f,-0.9792f,17.4805f,-0.9792f,17.4805f,-0.8750f,17.4317f,-0.8750f,17.4317f,-0.7708f,17.3828f,-0.7708f,17.3828f,-0.6667f,17.3340f,-0.6667f,17.3340f,-0.5625f,17.2852f,-0.5625f,17.2852f,-0.4583f,17.2364f,-0.4583f,17.2364f,-0.3542f,17.1875f,-0.3542f,17.1875f,-0.2500f,
	17.1387f,-0.2500f,17.1387f,-0.1458f,17.0899f,-0.1458f,17.0899f,-0.0417f,17.0410f,-0.0417f,17.0410f,0.0625f,16.9922f,0.0625f,16.9922f,0.1667f,16.9434f,0.1667f,16.9434f,0.2708f,16.8946f,0.2708f,16.8946f,0.3750f,16.8457f,0.3750f,16.8457f,0.4792f,16.7969f,0.4792f,
	16.7969f,0.5833f,16.7481f,0.5833f,16.7481f,0.6875f,16.6993f,0.6875f,16.6993f,0.7917f,16.6504f,0.7917f,16.6504f,0.8958f,16.6016f,0.8958f,16.6016f,1.0000f,16.5528f,1.0000f,18.8965f,-4.0000f,18.8477f,-4.0000f,18.8477f,-3.8958f,18.7989f,-3.8958f,18.7989f,-3.7917f,
	18.7500f,-3.7917f,18.7500f,-3.6875f,18.7012f,-3.6875f,18.7012f,-3.5833f,18.6524f,-3.5833f,18.6524f,-3.4792f,18.6036f,-3.4792f,18.6036f,-3.3750f,18.5547f,-3.3750f,18.5547f,-3.2708f,18.5059f,-3.2708f,18.5059f,-3.1667f,18.4571f,-3.1667f,18.4571f,-3.0625f,18.4082f,-3.0625f,
	18.4082f,-2.9583f,18.3594f,-2.9583f,18.3594f,-2.8542f,18.3106f,-2.8542f,18.3106f,-2.7500f,18.2618f,-2.7500f,18.2618f,-2.6458f,18.2129f,-2.6458f,18.2129f,-2.5417f,18.1641f,-2.5417f,18.1641f,-2.4375f,18.1153f,-2.4375f,18.1153f,-2.3333f,18.0664f,-2.3333f,18.0664f,-2.2292f,
	18.0176f,-2.2292f,18.0176f,-2.1250f,17.9688f,-2.1250f,17.9688f,-2.0208f,17.9200f,-2.0208f,17.9200f,-1.9167f,17.8711f,-1.9167f,17.8711f,-1.8125f,17.8223f,-1.8125f,17.8223f,-1.7083f,17.7735f,-1.7083f,17.7735f,-1.6042f,17.7246f,-1.6042f,17.7246f,-1.5000f,17.6758f,-1.5000f,
	17.6758f,-1.3958f,17.6270f,-1.3958f,17.6270f,-1.2917f,17.5782f,-1.2917f,17.5782f,-1.1875f,17.5293f,-1.1875f,17.5293f,-1.0833f,17.4805f,-1.0833f,17.4805f,-0.9792f,17.4317f,-0.9792f,17.4317f,-0.8750f,17.3828f,-0.8750f,17.3828f,-0.7708f,17.3340f,-0.7708f,17.3340f,-0.6667f,
	17.2852f,-0.6667f,17.2852f,-0.5625f,17.2364f,-0.5625f,17.2364f,-0.4583f,17.1875f,-0.4583f,17.1875f,-0.3542f,17.1387f,-0.3542f,17.1387f,-0.2500f,17.0899f,-0.2500f,17.0899f,-0.1458f,17.0410f,-0.1458f,17.0410f,-0.0417f,16.9922f,-0.0417f,16.9922f,0.0625f,16.9434f,0.0625f,
	16.9434f,0.1667f,16.8946f,0.1667f,16.8946f,0.2708f,16.8457f,0.2708f,16.8457f,0.3750f,16.7969f,0.3750f,16.7969f,0.4792f,16.7481f,0.4792f,16.7481f,0.5833f,16.6993f,0.5833f,16.6993f,0.6875f,16.6504f,0.6875f,16.6504f,0.7917f,16.6016f,0.7917f,16.6016f,0.8958f,
	16.5528f,0.8958f,16.5528f,1.0000f,16.5039f,1.0000f,18.8477f,-4.0000f,18.7989f,-4.0000f,18.7989f,-3.8958f,18.7500f,-3.8958f,18.7500f,-3.7917f,18.7012f,-3.7917f,18.7012f,-3.6875f,18.6524f,-3.6875f,18.6524f,-3.5833f,18.6036f,-3.5833f,18.6036f,-3.4792f,18.5547f,-3.4792f,
	18.5547f,-3.3750f,18.5059f,-3.3750f,18.5059f,-3.2708f,18.4571f,-3.2708f,18.4571f,-3.1667f,18.4082f,-3.1667f,18.4082f,-3.0625f,18.3594f,-3.0625f,18.3594f,-2.9583f,18.3106f,-2.9583f,18.3106f,-2.8542f,18.2618f,-2.8542f,18.2618f,-2.7500f,18.2129f,-2.7500f,18.2129f,-2.6458f,
	18.1641f,-2.6458f,18.1641f,-2.5417f,18.1153f,-2.5417f,18.1153f,-2.4375f,18.0664f,-2.4375f,18.0664f,-2.3333f,18.0176f,-2.3333f,18.0176f,-2.2292f,17.9688f,-2.2292f,17.9688f,-2.1250f,17.9200f,-2.1250f,17.9200f,-2.0208f,17.8711f,-2.0208f,17.8711f,-1.9167f,17.8223f,-1.9167f,
	17.8223f,-1.8125f,17.7735f,-1.8125f,17.7735f,-1.7083f,17.7246f,-1.7083f,17.7246f,-1.6042f,17.6758f,-1.6042f,17.6758f,-1.5000f,17.6270f,-1.5000f,17.6270f,-1.3958f,17.5782f,-1.3958f,17.5782f,-1.2917f,17.5293f,-1.2917f,17.5293f,-1.1875f,17.4805f,-1.1875f,17.4805f,-1.0833f,
	17.4317f,-1.0833f,17.4317f,-0.9792f,17.3828f,-0.9792f,17.3828f,-0.8750f,17.3340f,-0.8750f,17.3340f,-0.7708f,17.2852f,-0.7708f,17.2852f,-0.6667f,17.2364f,-0.6667f,17.2364f,-0.5625f,17.1875f,-0.5625f,17.1875f,-0.4583f,17.1387f,-0.4583f,17.1387f,-0.3542f,17.0899f,-0.3542f,
	17.0899f,-0.2500f,17.0410f,-0.2500f,17.0410f,-0.1458f,16.9922f,-0.1458f,16.9922f,-0.0417f,16.9434f,-0.0417f,16.9434f,0.0625f,16.8946f,0.0625f,16.8946f,0.1667f,16.8457f,0.1667f,16.8457f,0.2708f,16.7969f,0.2708f,16.7969f,0.3750f,16.7481f,0.3750f,16.7481f,0.4792f,
	16.6993f,0.4792f,16.6993f,0.5833f,16.6504f,0.5833f,16.6504f,0.6875f,16.6016f,0.6875f,16.6016f,0.7917f,16.5528f,0.7917f,16.5528f,0.8958f,16.5039f,0.8958f,16.5039f,1.0000f,16.4551f,1.0000f,18.7989f,-4.0000f,18.7500f,-4.0000f,18.7500f,-3.8958f,18.7012f,-3.8958f,
	18.7012f,-3.7917f,18.6524f,-3.7917f,18.6524f,-3.6875f,18.6036f,-3.6875f,18.6036f,-3.5833f,18.5547f,-3.5833f,18.5547f,-3.4792f,18.5059f,-3.4792f,18.5059f,-3.3750f,18.4571f,-3.3750f,18.4571f,-3.2708f,18.4082f,-3.2708f,18.4082f,-3.1667f,18.3594f,-3.1667f,18.3594f,-3.0625f,
	18.3106f,-3.0625f,18.3106f,-2.9583f,18.2618f,-2.9583f,18.2618f,-2.8542f,18.2129f,-2.8542f,18.2129f,-2.7500f,18.1641f,-2.7500f,18.1641f,-2.6458f,18.1153f,-2.6458f,18.1153f,-2.5417f,18.0664f,-2.5417f,18.0664f,-2.4375f,18.0176f,-2.4375f,18.0176f,-2.3333f,17.9688f,-2.3333f,
	17.9688f,-2.2292f,17.9200f,-2.2292f,17.9200f,-2.1250f,17.8711f,-2.1250f,17.8711f,-2.0208f,17.8223f,-2.0208f,17.8223f,-1.9167f,17.7735f,-1.9167f,17.7735f,-1.8125f,17.7246f,-1.8125f,17.7246f,-1.7083f,17.6758f,-1.7083f,17.6758f,-1.6042f,17.6270f,-1.6042f,17.6270f,-1.5000f,
	17.5782f,-1.5000f,17.5782f,-1.3958f,17.5293f,-1.3958f,17.5293f,-1.2917f,17.4805f,-1.2917f,17.4805f,-1.1875f,17.4317f,-1.1875f,17.4317f,-1.0833f,17.3828f,-1.0833f,17.3828f,-0.9792f,17.3340f,-0.9792f,17.3340f,-0.8750f,17.2852f,-0.8750f,17.2852f,-0.7708f,17.2364f,-0.7708f,
	17.2364f,-0.6667f,17.1875f,-0.6667f,17.1875f,-0.5625f,17.1387f,-0.5625f,17.1387f,-0.4583f,17.0899f,-0.4583f,17.0899f,-0.3542f,17.0410f,-0.3542f,17.0410f,-0.2500f,16.9922f,-0.2500f,16.9922f,-0.1458f,16.9434f,-0.1458f,16.9434f,-0.0417f,16.8946f,-0.0417f,16.8946f,0.0625f,
	16.8457f,0.0625f,16.8457f,0.1667f,16.7969f,0.1667f,16.7969f,0.2708f,16.7481f,0.2708f,16.7481f,0.3750f,16.6993f,0.3750f,16.6993f,0.4792f,16.6504f,0.4792f,16.6504f,0.5833f,16.6016f,0.5833f,16.6016f,0.6875f,16.5528f,0.6875f,16.5528f,0.7917f,16.5039f,0.7917f,
	16.5039f,0.8958f,16.4551f,0.8958f,16.4551f,1.0000f,16.4063f,1.0000f,18.7500f,-4.0000f,18.7012f,-4.0000f,18.7012f,-3.8958f,18.6524f,-3.8958f,18.6524f,-3.7917f,18.6036f,-3.7917f,18.6036f,-3.6875f,18.5547f,-3.6875f,18.5547f,-3.5833f,18.5059f,-3.5833f,18.5059f,-3.4792f,
	18.4571f,-3.4792f,18.4571f,-3.3750f,18.4082f,-3.3750f,18.4082f,-3.2708f,18.3594f,-3.2708f,18.3594f,-3.1667f,18.3106f,-3.1667f,18.3106f,-3.0625f,18.2618f,-3.0625f,18.2618f,-2.9583f,18.2129f,-2.9583f,18.2129f,-2.8542f,18.1641f,-2.8542f,18.1641f,-2.7500f,18.1153f,-2.7500f,
	18.1153f,-2.6458f,18.0664f,-2.6458f,18.0664f,-2.5417f,18.0176f,-2.5417f,18.0176f,-2.4375f,17.9688f,-2.4375f,17.9688f,-2.3333f,17.9200f,-2.3333f,17.9200f,-2.2292f,17.8711f,-2.2292f,17.8711f,-2.1250f,17.8223f,-2.1250f,17.8223f,-2.0208f,17.7735f,-2.0208f,17.7735f,-1.9167f,
	17.7246f,-1.9167f,17.7246f,-1.8125f,17.6758f,-1.8125f,17.6758f,-1.7083f,17.6270f,-1.7083f,17.6270f,-1.6042f,17.5782f,-1.6042f,17.5782f,-1.5000f,17.5293f,-1.5000f,17.5293f,-1.3958f,17.4805f,-1.3958f,17.4805f,-1.2917f,17.4317f,-1.2917f,17.4317f,-1.1875f,17.3828f,-1.1875f,
	17.3828f,-1.0833f,17.3340f,-1.0833f,17.3340f,-0.9792f,17.2852f,-0.9792f,17.2852f,-0.8750f,17.2364f,-0.8750f,17.2364f,-0.7708f,17.1875f,-0.7708f,17.1875f,-0.6667f,17.1387f,-0.6667f,17.1387f,-0.5625f,17.0899f,-0.5625f,17.0899f,-0.4583f,17.0410f,-0.4583f,17.0410f,-0.3542f,
	16.9922f,-0.3542f,16.9922f,-0.2500f,16.9434f,-0.2500f,16.9434f,-0.1458f,16.8946f,-0.1458f,16.8946f,-0.0417f,16.8457f,-0.0417f,16.8457f,0.0625f,16.7969f,0.0625f,16.7969f,0.1667f,16.7481f,0.1667f,16.7481f,0.2708f,16.6993f,0.2708f,16.6993f,0.3750f,16.6504f,0.3750f,
	16.6504f,0.4792f,16.6016f,0.4792f,16.6016f,0.5833f,16.5528f,0.5833f,16.5528f,0.6875f,16.5039f,0.6875f,16.5039f,0.7917f,16.4551f,0.7917f,16.4551f,0.8958f,16.4063f,0.8958f,16.4063f,1.0000f,16.3575f,1.0000f,18.7012f,-4.0000f,18.6524f,-4.0000f,18.6524f,-3.8958f,
	18.6036f,-3.8958f,18.6036f,-3.7917f,18.5547f,-3.7917f,18.5547f,-3.6875f,18.5059f,-3.6875f,18.5059f,-3.5833f,18.4571f,-3.5833f,18.4571f,-3.4792f,18.4082f,-3.4792f,18.4082f,-3.3750f,18.3594f,-3.3750f,18.3594f,-3.2708f,18.3106f,-3.2708f,18.3106f,-3.1667f,18.2618f,-3.1667f,
	18.2618f,-3.0625f,18.2129f,-3.0625f,18.2129f,-2.9583f,18.1641f,-2.9583f,18.1641f,-2.8542f,18.1153f,-2.8542f,18.1153f,-2.7500f,18.0664f,-2.7500f,18.0664f,-2.6458f,18.0176f,-2.6458f,18.0176f,-2.5417f,17.9688f,-2.5417f,17.9688f,-2.4375f,17.9200f,-2.4375f,17.9200f,-2.3333f,
	17.8711f,-2.3333f,17.8711f,-2.2292f,17.8223f,-2.2292f,17.8223f,-2.1250f,17.7735f,-2.1250f,17.7735f,-2.0208f,17.7246f,-2.0208f,17.7246f,-1.9167f,17.6758f,-1.9167f,17.6758f,-1.8125f,17.6270f,-1.8125f,17.6270f,-1.7083f,17.5782f,-1.7083f,17.5782f,-1.6042f,17.5293f,-1.6042f,
	17.5293f,-1.5000f,17.4805f,-1.5000f,17.4805f,-1.3958f,17.4317f,-1.3958f,17.4317f,-1.2917f,17.3828f,-1.2917f,17.3828f,-1.1875f,17.3340f,-1.1875f,17.3340f,-1.0833f,17.2852f,-1.0833f,17.2852f,-0.9792f,17.2364f,-0.9792f,17.2364f,-0.8750f,17.1875f,-0.8750f,17.1875f,-0.7708f,
	17.1387f,-0.7708f,17.1387f,-0.6667f,17.0899f,-0.6667f,17.0899f,-0.5625f,17.0410f,-0.5625f,17.0410f,-0.4583f,16.9922f,-0.4583f,16.9922f,-0.3542f,16.9434f,-0.3542f,16.9434f,-0.2500f,16.8946f,-0.2500f,16.8946f,-0.1458f,16.8457f,-0.1458f,16.8457f,-0.0417f,16.7969f,-0.0417f,
	16.7969f,0.0625f,16.7481f,0.0625f,16.7481f,0.1667f,16.6993f,0.1667f,16.6993f,0.2708f,16.6504f,0.2708f,16.6504f,0.3750f,16.6016f,0.3750f,16.6016f,0.4792f,16.5528f,0.4792f,16.5528f,0.5833f,16.5039f,0.5833f,16.5039f,0.6875f,16.4551f,0.6875f,16.4551f,0.7917f,
	16.4063f,0.7917f,16.4063f,0.8958f,16.3575f,0.8958f,16.3575f,1.0000f,16.3086f,1.0000f,18.6524f,-4.0000f,18.6036f,-4.0000f,18.6036f,-3.8958f,18.5547f,-3.8958f,18.5547f,-3.7917f,18.5059f,-3.7917f,18.5059f,-3.6875f,18.4571f,-3.6875f,18.4571f,-3.5833f,18.4082f,-3.5833f,
	18.4082f,-3.4792f,18.3594f,-3.4792f,18.3594f,-3.3750f,18.3106f,-3.3750f,18.3106f,-3.2708f,18.2618f,-3.2708f,18.2618f,-3.1667f,18.2129f,-3.1667f,18.2129f,-3.0625f,18.1641f,-3.0625f,18.1641f,-2.9583f,18.1153f,-2.9583f,18.1153f,-2.8542f,18.0664f,-2.8542f,18.0664f,-2.7500f,
	18.0176f,-2.7500f,18.0176f,-2.6458f,17.9688f,-2.6458f,17.9688f,-2.5417f,17.9200f,-2.5417f,17.9200f,-2.4375f,17.8711f,-2.4375f,17.8711f,-2.3333f,17.8223f,-2.3333f,17.8223f,-2.2292f,17.7735f,-2.2292f,17.7735f,-2.1250f,17.7246f,-2.1250f,17.7246f,-2.0208f,17.6758f,-2.0208f,
	17.6758f,-1.9167f,17.6270f,-1.9167f,17.6270f,-1.8125f,17.5782f,-1.8125f,17.5782f,-1.7083f,17.5293f,-1.7083f,17.5293f,-1.6042f,17.4805f,-1.6042f,17.4805f,-1.5000f,17.4317f,-1.5000f,17.4317f,-1.3958f,17.3828f,-1.3958f,17.3828f,-1.2917f,17.3340f,-1.2917f,17.3340f,-1.1875f,
	17.2852f,-1.1875f,17.2852f,-1.0833f,17.2364f,-1.0833f,17.2364f,-0.9792f,17.1875f,-0.9792f,17.1875f,-0.8750f,17.1387f,-0.8750f,17.1387f,-0.7708f,17.0899f,-0.7708f,17.0899f,-0.6667f,17.0410f,-0.6667f,17.0410f,-0.5625f,16.9922f,-0.5625f,16.9922f,-0.4583f,16.9434f,-0.4583f,
	16.9434f,-0.3542f,16.8946f,-0.3542f,16.8946f,-0.2500f,16.8457f,-0.2500f,16.8457f,-0.1458f,16.7969f,-0.1458f,16.7969f,-0.0417f,16.7481f,-0.0417f,16.7481f,0.0625f,16.6993f,0.0625f,16.6993f,0.1667f,16.6504f,0.1667f,16.6504f,0.2708f,16.6016f,0.2708f,16.6016f,0.3750f,
	16.5528f,0.3750f,16.5528f,0.4792f,16.5039f,0.4792f,16.5039f,0.5833f,16.4551f,0.5833f,16.4551f,0.6875f,16.4063f,0.6875f,16.4063f,0.7917f,16.3575f,0.7917f,16.3575f,0.8958f,16.3086f,0.8958f,16.3086f,1.0000f,16.2598f,1.0000f,18.6036f,-4.0000f,18.5547f,-4.0000f,
	18.5547f,-3.8958f,18.5059f,-3.8958f,18.5059f,-3.7917f,18.4571f,-3.7917f,18.4571f,-3.6875f,18.4082f,-3.6875f,18.4082f,-3.5833f,18.3594f,-3.5833f,18.3594f,-3.4792f,18.3106f,-3.4792f,18.3106f,-3.3750f,18.2618f,-3.3750f,18.2618f,-3.2708f,18.2129f,-3.2708f,18.2129f,-3.1667f,
	18.1641f,-3.1667f,18.1641f,-3.0625f,18.1153f,-3.0625f,18.1153f,-2.9583f,18.0664f,-2.9583f,18.0664f,-2.8542f,18.0176f,-2.8542f,18.0176f,-2.7500f,17.9688f,-2.7500f,17.9688f,-2.6458f,17.9200f,-2.6458f,17.9200f,-2.5417f,17.8711f,-2.5417f,17.8711f,-2.4375f,17.8223f,-2.4375f,
	17.8223f,-2.3333f,17.7735f,-2.3333f,17.7735f,-2.2292f,17.7246f,-2.2292f,17.7246f,-2.1250f,17.6758f,-2.1250f,17.6758f,-2.0208f,17.6270f,-2.0208f,17.6270f,-1.9167f,17.5782f,-1.9167f,17.5782f,-1.8125f,17.5293f,-1.8125f,17.5293f,-1.7083f,17.4805f,-1.7083f,17.4805f,-1.6042f,
	17.4317f,-1.6042f,17.4317f,-1.5000f,17.3828f,-1.5000f,17.3828f,-1.3958f,17.3340f,-1.3958f,17.3340f,-1.2917f,17.2852f,-1.2917f,17.2852f,-1.1875f,17.2364f,-1.1875f,17.2364f,-1.0833f,17.1875f,-1.0833f,17.1875f,-0.9792f,17.1387f,-0.9792f,17.1387f,-0.8750f,17.0899f,-0.8750f,
	17.0899f,-0.7708f,17.0410f,-0.7708f,17.0410f,-0.6667f,16.9922f,-0.6667f,16.9922f,-0.5625f,16.9434f,-0.5625f,16.9434f,-0.4583f,16.8946f,-0.4583f,16.8946f,-0.3542f,16.8457f,-0.3542f,16.8457f,-0.2500f,16.7969f,-0.2500f,16.7969f,-0.1458f,16.7481f,-0.1458f,16.7481f,-0.0417f,
	16.6993f,-0.0417f,16.6993f,0.0625f,16.6504f,0.0625f,16.6504f,0.1667f,16.6016f,0.1667f,16.6016f,0.2708f,16.5528f,0.2708f,16.5528f,0.3750f,16.5039f,0.3750f,16.5039f,0.4792f,16.4551f,0.4792f,16.4551f,0.5833f,16.4063f,0.5833f,16.4063f,0.6875f,16.3575f,0.6875f,
	16.3575f,0.7917f,16.3086f,0.7917f,16.3086f,0.8958f,16.2598f,0.8958f,16.2598f,1.0000f,16.2110f,1.0000f,18.5547f,-4.0000f,18.5059f,-4.0000f,18.5059f,-3.8958f,18.4571f,-3.8958f,18.4571f,-3.7917f,18.4082f,-3.7917f,18.4082f,-3.6875f,18.3594f,-3.6875f,18.3594f,-3.5833f,
	18.3106f,-3.5833f,18.3106f,-3.4792f,18.2618f,-3.4792f,18.2618f,-3.3750f,18.2129f,-3.3750f,18.2129f,-3.2708f,18.1641f,-3.2708f,18.1641f,-3.1667f,18.1153f,-3.1667f,18.1153f,-3.0625f,18.0664f,-3.0625f,18.0664f,-2.9583f,18.0176f,-2.9583f,18.0176f,-2.8542f,17.9688f,-2.8542f,
	17.9688f,-2.7500f,17.9200f,-2.7500f,17.9200f,-2.6458f,17.8711f,-2.6458f,17.8711f,-2.5417f,17.8223f,-2.5417f,17.8223f,-2.4375f,17.7735f,-2.4375f,17.7735f,-2.3333f,17.7246f,-2.3333f,17.7246f,-2.2292f,17.6758f,-2.2292f,17.6758f,-2.1250f,17.6270f,-2.1250f,17.6270f,-2.0208f,
	17.5782f,-2.0208f,17.5782f,-1.9167f,17.5293f,-1.9167f,17.5293f,-1.8125f,17.4805f,-1.8125f,17.4805f,-1.7083f,17.4317f,-1.7083f,17.4317f,-1.6042f,17.3828f,-1.6042f,17.3828f,-1.5000f,17.3340f,-1.5000f,17.3340f,-1.3958f,17.2852f,-1.3958f,17.2852f,-1.2917f,17.2364f,-1.2917f,
	17.2364f,-1.1875f,17.1875f,-1.1875f,17.1875f,-1.0833f,17.1387f,-1.0833f,17.1387f,-0.9792f,17.0899f,-0.9792f,17.0899f,-0.8750f,17.0410f,-0.8750f,17.0410f,-0.7708f,16.9922f,-0.7708f,16.9922f,-0.6667f,16.9434f,-0.6667f,16.9434f,-0.5625f,16.8946f,-0.5625f,16.8946f,-0.4583f,
	16.8457f,-0.4583f,16.8457f,-0.3542f,16.7969f,-0.3542f,16.7969f,-0.2500f,16.7481f,-0.2500f,16.7481f,-0.1458f,16.6993f,-0.1458f,16.6993f,-0.0417f,16.6504f,-0.0417f,16.6504f,0.0625f,16.6016f,0.0625f,16.6016f,0.1667f,16.5528f,0.1667f,16.5528f,0.2708f,16.5039f,0.2708f,
	16.5039f,0.3750f,16.4551f,0.3750f,16.4551f,0.4792f,16.4063f,0.4792f,16.4063f,0.5833f,16.3575f,0.5833f,16.3575f,0.6875f,16.3086f,0.6875f,16.3086f,0.7917f,16.2598f,0.7917f,16.2598f,0.8958f,16.2110f,0.8958f,16.2110f,1.0000f,16.1621f,1.0000f,18.5059f,-4.0000f,
	18.4571f,-4.0000f,18.4571f,-3.8958f,18.4082f,-3.8958f,18.4082f,-3.7917f,18.3594f,-3.7917f,18.3594f,-3.6875f,18.3106f,-3.6875f,18.3106f,-3.5833f,18.2618f,-3.5833f,18.2618f,-3.4792f,18.2129f,-3.4792f,18.2129f,-3.3750f,18.1641f,-3.3750f,18.1641f,-3.2708f,18.1153f,-3.2708f,
	18.1153f,-3.1667f,18.0664f,-3.1667f,18.0664f,-3.0625f,18.0176f,-3.0625f,18.0176f,-2.9583f,17.9688f,-2.9583f,17.9688f,-2.8542f,17.9200f,-2.8542f,17.9200f,-2.7500f,17.8711f,-2.7500f,17.8711f,-2.6458f,17.8223f,-2.6458f,17.8223f,-2.5417f,17.7735f,-2.5417f,17.7735f,-2.4375f,
	17.7246f,-2.4375f,17.7246f,-2.3333f,17.6758f,-2.3333f,17.6758f,-2.2292f,17.6270f,-2.2292f,17.6270f,-2.1250f,17.5782f,-2.1250f,17.5782f,-2.0208f,17.5293f,-2.0208f,17.5293f,-1.9167f,17.4805f,-1.9167f,17.4805f,-1.8125f,17.4317f,-1.8125f,17.4317f,-1.7083f,17.3828f,-1.7083f,
	17.3828f,-1.6042f,17.3340f,-1.6042f,17.3340f,-1.5000f,17.2852f,-1.5000f,17.2852f,-1.3958f,17.2364f,-1.3958f,17.2364f,-1.2917f,17.1875f,-1.2917f,17.1875f,-1.1875f,17.1387f,-1.1875f,17.1387f,-1.0833f,17.0899f,-1.0833f,17.0899f,-0.9792f,17.0410f,-0.9792f,17.0410f,-0.8750f,
	16.9922f,-0.8750f,16.9922f,-0.7708f,16.9434f,-0.7708f,16.9434f,-0.6667f,16.8946f,-0.6667f,16.8946f,-0.5625f,16.8457f,-0.5625f,16.8457f,-0.4583f,16.7969f,-0.4583f,16.7969f,-0.3542f,16.7481f,-0.3542f,16.7481f,-0.2500f,16.6993f,-0.2500f,16.6993f,-0.1458f,16.6504f,-0.1458f,
	16.6504f,-0.0417f,16.6016f,-0.0417f,16.6016f,0.0625f,16.5528f,0.0625f,16.5528f,0.1667f,16.5039f,0.1667f,16.5039f,0.2708f,16.4551f,0.2708f,16.4551f,0.3750f,16.4063f,0.3750f,16.4063f,0.4792f,16.3575f,0.4792f,16.3575f,0.5833f,16.3086f,0.5833f,16.3086f,0.6875f,
	16.2598f,0.6875f,16.2598f,0.7917f,16.2110f,0.7917f,16.2110f,0.8958f,16.1621f,0.8958f,16.1621f,1.0000f,16.1133f,1.0000f,18.4571f,-4.0000f,18.4082f,-4.0000f,18.4082f,-3.8958f,18.3594f,-3.8958f,18.3594f,-3.7917f,18.3106f,-3.7917f,18.3106f,-3.6875f,18.2618f,-3.6875f,
	18.2618f,-3.5833f,18.2129f,-3.5833f,18.2129f,-3.4792f,18.1641f,-3.4792f,18.1641f,-3.3750f,18.1153f,-3.3750f,18.1153f,-3.2708f,18.0664f,-3.2708f,18.0664f,-3.1667f,18.0176f,-3.1667f,18.0176f,-3.0625f,17.9688f,-3.0625f,17.9688f,-2.9583f,17.9200f,-2.9583f,17.9200f,-2.8542f,
	17.8711f,-2.8542f,17.8711f,-2.7500f,17.8223f,-2.7500f,17.8223f,-2.6458f,17.7735f,-2.6458f,17.7735f,-2.5417f,17.7246f,-2.5417f,17.7246f,-2.4375f,17.6758f,-2.4375f,17.6758f,-2.3333f,17.6270f,-2.3333f,17.6270f,-2.2292f,17.5782f,-2.2292f,17.5782f,-2.1250f,17.5293f,-2.1250f,
	17.5293f,-2.0208f,17.4805f,-2.0208f,17.4805f,-1.9167f,17.4317f,-1.9167f,17.4317f,-1.8125f,17.3828f,-1.8125f,17.3828f,-1.7083f,17.3340f,-1.7083f,17.3340f,-1.6042f,17.2852f,-1.6042f,17.2852f,-1.5000f,17.2364f,-1.5000f,17.2364f,-1.3958f,17.1875f,-1.3958f,17.1875f,-1.2917f,
	17.1387f,-1.2917f,17.1387f,-1.1875f,17.0899f,-1.1875f,17.0899f,-1.0833f,17.0410f,-1.0833f,17.0410f,-0.9792f,16.9922f,-0.9792f,16.9922f,-0.8750f,16.9434f,-0.8750f,16.9434f,-0.7708f,16.8946f,-0.7708f,16.8946f,-0.6667f,16.8457f,-0.6667f,16.8457f,-0.5625f,16.7969f,-0.5625f,
	16.7969f,-0.4583f,16.7481f,-0.4583f,16.7481f,-0.3542f,16.6993f,-0.3542f,16.6993f,-0.2500f,16.6504f,-0.2500f,16.6504f,-0.1458f,16.6016f,-0.1458f,16.6016f,-0.0417f,16.5528f,-0.0417f,16.5528f,0.0625f,16.5039f,0.0625f,16.5039f,0.1667f,16.4551f,0.1667f,16.4551f,0.2708f,
	16.4063f,0.2708f,16.4063f,0.3750f,16.3575f,0.3750f,16.3575f,0.4792f,16.3086f,0.4792f,16.3086f,0.5833f,16.2598f,0.5833f,16.2598f,0.6875f,16.2110f,0.6875f,16.2110f,0.7917f,16.1621f,0.7917f,16.1621f,0.8958f,16.1133f,0.8958f,16.1133f,1.0000f,16.0645f,1.0000f,
	18.4082f,-4.0000f,18.3594f,-4.0000f,18.3594f,-3.8958f,18.3106f,-3.8958f,18.3106f,-3.7917f,18.2618f,-3.7917f,18.2618f,-3.6875f,18.2129f,-3.6875f,18.2129f,-3.5833f,18.1641f,-3.5833f,18.1641f,-3.4792f,18.1153f,-3.4792f,18.1153f,-3.3750f,18.0664f,-3.3750f,18.0664f,-3.2708f,
	18.0176f,-3.2708f,18.0176f,-3.1667f,17.9688f,-3.1667f,17.9688f,-3.0625f,17.9200f,-3.0625f,17.9200f,-2.9583f,17.8711f,-2.9583f,17.8711f,-2.8542f,17.8223f,-2.8542f,17.8223f,-2.7500f,17.7735f,-2.7500f,17.7735f,-2.6458f,17.7246f,-2.6458f,17.7246f,-2.5417f,17.6758f,-2.5417f,
	17.6758f,-2.4375f,17.6270f,-2.4375f,17.6270f,-2.3333f,17.5782f,-2.3333f,17.5782f,-2.2292f,17.5293f,-2.2292f,17.5293f,-2.1250f,17.4805f,-2.1250f,17.4805f,-2.0208f,17.4317f,-2.0208f,17.4317f,-1.9167f,17.3828f,-1.9167f,17.3828f,-1.8125f,17.3340f,-1.8125f,17.3340f,-1.7083f,
	17.2852f,-1.7083f,17.2852f,-1.6042f,17.2364f,-1.6042f,17.2364f,-1.5000f,17.1875f,-1.5000f,17.1875f,-1.3958f,17.1387f,-1.3958f,17.1387f,-1.2917f,17.0899f,-1.2917f,17.0899f,-1.1875f,17.0410f,-1.1875f,17.0410f,-1.0833f,16.9922f,-1.0833f,16.9922f,-0.9792f,16.9434f,-0.9792f,
	16.9434f,-0.8750f,16.8946f,-0.8750f,16.8946f,-0.7708f,16.8457f,-0.7708f,16.8457f,-0.6667f,16.7969f,-0.6667f,16.7969f,-0.5625f,16.7481f,-0.5625f,16.7481f,-0.4583f,16.6993f,-0.4583f,16.6993f,-0.3542f,16.6504f,-0.3542f,16.6504f,-0.2500f,16.6016f,-0.2500f,16.6016f,-0.1458f,
	16.5528f,-0.1458f,16.5528f,-0.0417f,16.5039f,-0.0417f,16.5039f,0.0625f,16.4551f,0.0625f,16.4551f,0.1667f,16.4063f,0.1667f,16.4063f,0.2708f,16.3575f,0.2708f,16.3575f,0.3750f,16.3086f,0.3750f,16.3086f,0.4792f,16.2598f,0.4792f,16.2598f,0.5833f,16.2110f,0.5833f,
	16.2110f,0.6875f,16.1621f,0.6875f,16.1621f,0.7917f,16.1133f,0.7917f,16.1133f,0.8958f,16.0645f,0.8958f,16.0645f,1.0000f,16.0157f,1.0000f,18.3594f,-4.0000f,18.3106f,-4.0000f,18.3106f,-3.8958f,18.2618f,-3.8958f,18.2618f,-3.7917f,18.2129f,-3.7917f,18.2129f,-3.6875f,
	18.1641f,-3.6875f,18.1641f,-3.5833f,18.1153f,-3.5833f,18.1153f,-3.4792f,18.0664f,-3.4792f,18.0664f,-3.3750f,18.0176f,-3.3750f,18.0176f,-3.2708f,17.9688f,-3.2708f,17.9688f,-3.1667f,17.9200f,-3.1667f,17.9200f,-3.0625f,17.8711f,-3.0625f,17.8711f,-2.9583f,17.8223f,-2.9583f,
	17.8223f,-2.8542f,17.7735f,-2.8542f,17.7735f,-2.7500f,17.7246f,-2.7500f,17.7246f,-2.6458f,17.6758f,-2.6458f,17.6758f,-2.5417f,17.6270f,-2.5417f,17.6270f,-2.4375f,17.5782f,-2.4375f,17.5782f,-2.3333f,17.5293f,-2.3333f,17.5293f,-2.2292f,17.4805f,-2.2292f,17.4805f,-2.1250f,
	17.4317f,-2.1250f,17.4317f,-2.0208f,17.3828f,-2.0208f,17.3828f,-1.9167f,17.3340f,-1.9167f,17.3340f,-1.8125f,17.2852f,-1.8125f,17.2852f,-1.7083f,17.2364f,-1.7083f,17.2364f,-1.6042f,17.1875f,-1.6042f,17.1875f,-1.5000f,17.1387f,-1.5000f,17.1387f,-1.3958f,17.0899f,-1.3958f,
	17.0899f,-1.2917f,17.0410f,-1.2917f,17.0410f,-1.1875f,16.9922f,-1.1875f,16.9922f,-1.0833f,16.9434f,-1.0833f,16.9434f,-0.9792f,16.8946f,-0.9792f,16.8946f,-0.8750f,16.8457f,-0.8750f,16.8457f,-0.7708f,16.7969f,-0.7708f,16.7969f,-0.6667f,16.7481f,-0.6667f,16.7481f,-0.5625f,
	16.6993f,-0.5625f,16.6993f,-0.4583f,16.6504f,-0.4583f,16.6504f,-0.3542f,16.6016f,-0.3542f,16.6016f,-0.2500f,16.5528f,-0.2500f,16.5528f,-0.1458f,16.5039f,-0.1458f,16.5039f,-0.0417f,16.4551f,-0.0417f,16.4551f,0.0625f,16.4063f,0.0625f,16.4063f,0.1667f,16.3575f,0.1667f,
	16.3575f,0.2708f,16.3086f,0.2708f,16.3086f,0.3750f,16.2598f,0.3750f,16.2598f,0.4792f,16.2110f,0.4792f,16.2110f,0.5833f,16.1621f,0.5833f,16.1621f,0.6875f,16.1133f,0.6875f,16.1133f,0.7917f,16.0645f,0.7917f,16.0645f,0.8958f,16.0157f,0.8958f,16.0157f,1.0000f,
	15.9668f,1.0000f,18.3106f,-4.0000f,18.2618f,-4.0000f,18.2618f,-3.8958f,18.2129f,-3.8958f,18.2129f,-3.7917f,18.1641f,-3.7917f,18.1641f,-3.6875f,18.1153f,-3.6875f,18.1153f,-3.5833f,18.0664f,-3.5833f,18.0664f,-3.4792f,18.0176f,-3.4792f,18.0176f,-3.3750f,17.9688f,-3.3750f,
	17.9688f,-3.2708f,17.9200f,-3.2708f,17.9200f,-3.1667f,17.8711f,-3.1667f,17.8711f,-3.0625f,17.8223f,-3.0625f,17.8223f,-2.9583f,17.7735f,-2.9583f,17.7735f,-2.8542f,17.7246f,-2.8542f,17.7246f,-2.7500f,17.6758f,-2.7500f,17.6758f,-2.6458f,17.6270f,-2.6458f,17.6270f,-2.5417f,
	17.5782f,-2.5417f,17.5782f,-2.4375f,17.5293f,-2.4375f,17.5293f,-2.3333f,17.4805f,-2.3333f,17.4805f,-2.2292f,17.4317f,-2.2292f,17.4317f,-2.1250f,17.3828f,-2.1250f,17.3828f,-2.0208f,17.3340f,-2.0208f,17.3340f,-1.9167f,17.2852f,-1.9167f,17.2852f,-1.8125f,17.2364f,-1.8125f,
	17.2364f,-1.7083f,17.1875f,-1.7083f,17.1875f,-1.6042f,17.1387f,-1.6042f,17.1387f,-1.5000f,17.0899f,-1.5000f,17.0899f,-1.3958f,17.0410f,-1.3958f,17.0410f,-1.2917f,16.9922f,-1.2917f,16.9922f,-1.1875f,16.9434f,-1.1875f,16.9434f,-1.0833f,16.8946f,-1.0833f,16.8946f,-0.9792f,
	16.8457f,-0.9792f,16.8457f,-0.8750f,16.7969f,-0.8750f,16.7969f,-0.7708f,16.7481f,-0.7708f,16.7481f,-0.6667f,16.6993f,-0.6667f,16.6993f,-0.5625f,16.6504f,-0.5625f,16.6504f,-0.4583f,16.6016f,-0.4583f,16.6016f,-0.3542f,16.5528f,-0.3542f,16.5528f,-0.2500f,16.5039f,-0.2500f,
	16.5039f,-0.1458f,16.4551f,-0.1458f,16.4551f,-0.0417f,16.4063f,-0.0417f,16.4063f,0.0625f,16.3575f,0.0625f,16.3575f,0.1667f,16.3086f,0.1667f,16.3086f,0.2708f,16.2598f,0.2708f,16.2598f,0.3750f,16.2110f,0.3750f,16.2110f,0.4792f,16.1621f,0.4792f,16.1621f,0.5833f,
	16.1133f,0.5833f,16.1133f,0.6875f,16.0645f,0.6875f,16.0645f,0.7917f,16.0157f,0.7917f,16.0157f,0.8958f,15.9668f,0.8958f,15.9668f,1.0000f,15.9180f,1.0000f,18.2618f,-4.0000f,18.2129f,-4.0000f,18.2129f,-3.8958f,18.1641f,-3.8958f,18.1641f,-3.7917f,18.1153f,-3.7917f,
	18.1153f,-3.6875f,18.0664f,-3.6875f,18.0664f,-3.5833f,18.0176f,-3.5833f,18.0176f,-3.4792f,17.9688f,-3.4792f,17.9688f,-3.3750f,17.9200f,-3.3750f,17.9200f,-3.2708f,17.8711f,-3.2708f,17.8711f,-3.1667f,17.8223f,-3.1667f,17.8223f,-3.0625f,17.7735f,-3.0625f,17.7735f,-2.9583f,
	17.7246f,-2.9583f,17.7246f,-2.8542f,17.6758f,-2.8542f,17.6758f,-2.7500f,17.6270f,-2.7500f,17.6270f,-2.6458f,17.5782f,-2.6458f,17.5782f,-2.5417f,17.5293f,-2.5417f,17.5293f,-2.4375f,17.4805f,-2.4375f,17.4805f,-2.3333f,17.4317f,-2.3333f,17.4317f,-2.2292f,17.3828f,-2.2292f,
	17.3828f,-2.1250f,17.3340f,-2.1250f,17.3340f,-2.0208f,17.2852f,-2.0208f,17.2852f,-1.9167f,17.2364f,-1.9167f,17.2364f,-1.8125f,17.1875f,-1.8125f,17.1875f,-1.7083f,17.1387f,-1.7083f,17.1387f,-1.6042f,17.0899f,-1.6042f,17.0899f,-1.5000f,17.0410f,-1.5000f,17.0410f,-1.3958f,
	16.9922f,-1.3958f,16.9922f,-1.2917f,16.9434f,-1.2917f,16.9434f,-1.1875f,16.8946f,-1.1875f,16.8946f,-1.0833f,16.8457f,-1.0833f,16.8457f,-0.9792f,16.7969f,-0.9792f,16.7969f,-0.8750f,16.7481f,-0.8750f,16.7481f,-0.7708f,16.6993f,-0.7708f,16.6993f,-0.6667f,16.6504f,-0.6667f,
	16.6504f,-0.5625f,16.6016f,-0.5625f,16.6016f,-0.4583f,16.5528f,-0.4583f,16.5528f,-0.3542f,16.5039f,-0.3542f,16.5039f,-0.2500f,16.4551f,-0.2500f,16.4551f,-0.1458f,16.4063f,-0.1458f,16.4063f,-0.0417f,16.3575f,-0.0417f,16.3575f,0.0625f,16.3086f,0.0625f,16.3086f,0.1667f,
	16.2598f,0.1667f,16.2598f,0.2708f,16.2110f,0.2708f,16.2110f,0.3750f,16.1621f,0.3750f,16.1621f,0.4792f,16.1133f,0.4792f,16.1133f,0.5833f,16.0645f,0.5833f,16.0645f,0.6875f,16.0157f,0.6875f,16.0157f,0.7917f,15.9668f,0.7917f,15.9668f,0.8958f,15.9180f,0.8958f,
	15.9180f,1.0000f,15.8692f,1.0000f,18.2129f,-4.0000f,18.1641f,-4.0000f,18.1641f,-3.8958f,18.1153f,-3.8958f,18.1153f,-3.7917f,18.0664f,-3.7917f,18.0664f,-3.6875f,18.0176f,-3.6875f,18.0176f,-3.5833f,17.9688f,-3.5833f,17.9688f,-3.4792f,17.9200f,-3.4792f,17.9200f,-3.3750f,
	17.8711f,-3.3750f,17.8711f,-3.2708f,17.8223f,-3.2708f,17.8223f,-3.1667f,17.7735f,-3.1667f,17.7735f,-3.0625f,17.7246f,-3.0625f,17.7246f,-2.9583f,17.6758f,-2.9583f,17.6758f,-2.8542f,17.6270f,-2.8542f,17.6270f,-2.7500f,17.5782f,-2.7500f,17.5782f,-2.6458f,17.5293f,-2.6458f,
	17.5293f,-2.5417f,17.4805f,-2.5417f,17.4805f,-2.4375f,17.4317f,-2.4375f,17.4317f,-2.3333f,17.3828f,-2.3333f,17.3828f,-2.2292f,17.3340f,-2.2292f,17.3340f,-2.1250f,17.2852f,-2.1250f,17.2852f,-2.0208f,17.2364f,-2.0208f,17.2364f,-1.9167f,17.1875f,-1.9167f,17.1875f,-1.8125f,
	17.1387f,-1.8125f,17.1387f,-1.7083f,17.0899f,-1.7083f,17.0899f,-1.6042f,17.0410f,-1.6042f,17.0410f,-1.5000f,16.9922f,-1.5000f,16.9922f,-1.3958f,16.9434f,-1.3958f,16.9434f,-1.2917f,16.8946f,-1.2917f,16.8946f,-1.1875f,16.8457f,-1.1875f,16.8457f,-1.0833f,16.7969f,-1.0833f,
	16.7969f,-0.9792f,16.7481f,-0.9792f,16.7481f,-0.8750f,16.6993f,-0.8750f,16.6993f,-0.7708f,16.6504f,-0.7708f,16.6504f,-0.6667f,16.6016f,-0.6667f,16.6016f,-0.5625f,16.5528f,-0.5625f,16.5528f,-0.4583f,16.5039f,-0.4583f,16.5039f,-0.3542f,16.4551f,-0.3542f,16.4551f,-0.2500f,
	16.4063f,-0.2500f,16.4063f,-0.1458f,16.3575f,-0.1458f,16.3575f,-0.0417f,16.3086f,-0.0417f,16.3086f,0.0625f,16.2598f,0.0625f,16.2598f,0.1667f,16.2110f,0.1667f,16.2110f,0.2708f,16.1621f,0.2708f,16.1621f,0.3750f,16.1133f,0.3750f,16.1133f,0.4792f,16.0645f,0.4792f,
	16.0645f,0.5833f,16.0157f,0.5833f,16.0157f,0.6875f,15.9668f,0.6875f,15.9668f,0.7917f,15.9180f,0.7917f,15.9180f,0.8958f,15.8692f,0.8958f,15.8692f,1.0000f,15.8203f,1.0000f,18.1641f,-4.0000f,18.1153f,-4.0000f,18.1153f,-3.8958f,18.0664f,-3.8958f,18.0664f,-3.7917f,
	18.0176f,-3.7917f,18.0176f,-3.6875f,17.9688f,-3.6875f,17.9688f,-3.5833f,17.9200f,-3.5833f,17.9200f,-3.4792f,17.8711f,-3.4792f,17.8711f,-3.3750f,17.8223f,-3.3750f,17.8223f,-3.2708f,17.7735f,-3.2708f,17.7735f,-3.1667f,17.7246f,-3.1667f,17.7246f,-3.0625f,17.6758f,-3.0625f,
	17.6758f,-2.9583f,17.6270f,-2.9583f,17.6270f,-2.8542f,17.5782f,-2.8542f,17.5782f,-2.7500f,17.5293f,-2.7500f,17.5293f,-2.6458f,17.4805f,-2.6458f,17.4805f,-2.5417f,17.4317f,-2.5417f,17.4317f,-2.4375f,17.3828f,-2.4375f,17.3828f,-2.3333f,17.3340f,-2.3333f,17.3340f,-2.2292f,
	17.2852f,-2.2292f,17.2852f,-2.1250f,17.2364f,-2.1250f,17.2364f,-2.0208f,17.1875f,-2.0208f,17.1875f,-1.9167f,17.1387f,-1.9167f,17.1387f,-1.8125f,17.0899f,-1.8125f,17.0899f,-1.7083f,17.0410f,-1.7083f,17.0410f,-1.6042f,16.9922f,-1.6042f,16.9922f,-1.5000f,16.9434f,-1.5000f,
	16.9434f,-1.3958f,16.8946f,-1.3958f,16.8946f,-1.2917f,16.8457f,-1.2917f,16.8457f,-1.1875f,16.7969f,-1.1875f,16.7969f,-1.0833f,16.7481f,-1.0833f,16.7481f,-0.9792f,16.6993f,-0.9792f,16.6993f,-0.8750f,16.6504f,-0.8750f,16.6504f,-0.7708f,16.6016f,-0.7708f,16.6016f,-0.6667f,
	16.5528f,-0.6667f,16.5528f,-0.5625f,16.5039f,-0.5625f,16.5039f,-0.4583f,16.4551f,-0.4583f,16.4551f,-0.3542f,16.4063f,-0.3542f,16.4063f,-0.2500f,16.3575f,-0.2500f,16.3575f,-0.1458f,16.3086f,-0.1458f,16.3086f,-0.0417f,16.2598f,-0.0417f,16.2598f,0.0625f,16.2110f,0.0625f,
	16.2110f,0.1667f,16.1621f,0.1667f,16.1621f,0.2708f,16.1133f,0.2708f,16.1133f,0.3750f,16.0645f,0.3750f,16.0645f,0.4792f,16.0157f,0.4792f,16.0157f,0.5833f,15.9668f,0.5833f,15.9668f,0.6875f,15.9180f,0.6875f,15.9180f,0.7917f,15.8692f,0.7917f,15.8692f,0.8958f,
	15.8203f,0.8958f,15.8203f,1.0000f,15.7715f,1.0000f,18.1153f,-4.0000f,18.0664f,-4.0000f,18.0664f,-3.8958f,18.0176f,-3.8958f,18.0176f,-3.7917f,17.9688f,-3.7917f,17.9688f,-3.6875f,17.9200f,-3.6875f,17.9200f,-3.5833f,17.8711f,-3.5833f,17.8711f,-3.4792f,17.8223f,-3.4792f,
	17.8223f,-3.3750f,17.7735f,-3.3750f,17.7735f,-3.2708f,17.7246f,-3.2708f,17.7246f,-3.1667f,17.6758f,-3.1667f,17.6758f,-3.0625f,17.6270f,-3.0625f,17.6270f,-2.9583f,17.5782f,-2.9583f,17.5782f,-2.8542f,17.5293f,-2.8542f,17.5293f,-2.7500f,17.4805f,-2.7500f,17.4805f,-2.6458f,
	17.4317f,-2.6458f,17.4317f,-2.5417f,17.3828f,-2.5417f,17.3828f,-2.4375f,17.3340f,-2.4375f,17.3340f,-2.3333f,17.2852f,-2.3333f,17.2852f,-2.2292f,17.2364f,-2.2292f,17.2364f,-2.1250f,17.1875f,-2.1250f,17.1875f,-2.0208f,17.1387f,-2.0208f,17.1387f,-1.9167f,17.0899f,-1.9167f,
	17.0899f,-1.8125f,17.0410f,-1.8125f,17.0410f,-1.7083f,16.9922f,-1.7083f,16.9922f,-1.6042f,16.9434f,-1.6042f,16.9434f,-1.5000f,16.8946f,-1.5000f,16.8946f,-1.3958f,16.8457f,-1.3958f,16.8457f,-1.2917f,16.7969f,-1.2917f,16.7969f,-1.1875f,16.7481f,-1.1875f,16.7481f,-1.0833f,
	16.6993f,-1.0833f,16.6993f,-0.9792f,16.6504f,-0.9792f,16.6504f,-0.8750f,16.6016f,-0.8750f,16.6016f,-0.7708f,16.5528f,-0.7708f,16.5528f,-0.6667f,16.5039f,-0.6667f,16.5039f,-0.5625f,16.4551f,-0.5625f,16.4551f,-0.4583f,16.4063f,-0.4583f,16.4063f,-0.3542f,16.3575f,-0.3542f,
	16.3575f,-0.2500f,16.3086f,-0.2500f,16.3086f,-0.1458f,16.2598f,-0.1458f,16.2598f,-0.0417f,16.2110f,-0.0417f,16.2110f,0.0625f,16.1621f,0.0625f,16.1621f,0.1667f,16.1133f,0.1667f,16.1133f,0.2708f,16.0645f,0.2708f,16.0645f,0.3750f,16.0157f,0.3750f,16.0157f,0.4792f,
	15.9668f,0.4792f,15.9668f,0.5833f,15.9180f,0.5833f,15.9180f,0.6875f,15.8692f,0.6875f,15.8692f,0.7917f,15.8203f,0.7917f,15.8203f,0.8958f,15.7715f,0.8958f,15.7715f,1.0000f,15.7227f,1.0000f,18.0664f,-4.0000f,18.0176f,-4.0000f,18.0176f,-3.8958f,17.9688f,-3.8958f,
	17.9688f,-3.7917f,17.9200f,-3.7917f,17.9200f,-3.6875f,17.8711f,-3.6875f,17.8711f,-3.5833f,17.8223f,-3.5833f,17.8223f,-3.4792f,17.7735f,-3.4792f,17.7735f,-3.3750f,17.7246f,-3.3750f,17.7246f,-3.2708f,17.6758f,-3.2708f,17.6758f,-3.1667f,17.6270f,-3.1667f,17.6270f,-3.0625f,
	17.5782f,-3.0625f,17.5782f,-2.9583f,17.5293f,-2.9583f,17.5293f,-2.8542f,17.4805f,-2.8542f,17.4805f,-2.7500f,17.4317f,-2.7500f,17.4317f,-2.6458f,17.3828f,-2.6458f,17.3828f,-2.5417f,17.3340f,-2.5417f,17.3340f,-2.4375f,17.2852f,-2.4375f,17.2852f,-2.3333f,17.2364f,-2.3333f,
	17.2364f,-2.2292f,17.1875f,-2.2292f,17.1875f,-2.1250f,17.1387f,-2.1250f,17.1387f,-2.0208f,17.0899f,-2.0208f,17.0899f,-1.9167f,17.0410f,-1.9167f,17.0410f,-1.8125f,16.9922f,-1.8125f,16.9922f,-1.7083f,16.9434f,-1.7083f,16.9434f,-1.6042f,16.8946f,-1.6042f,16.8946f,-1.5000f,
	16.8457f,-1.5000f,16.8457f,-1.3958f,16.7969f,-1.3958f,16.7969f,-1.2917f,16.7481f,-1.2917f,16.7481f,-1.1875f,16.6993f,-1.1875f,16.6993f,-1.0833f,16.6504f,-1.0833f,16.6504f,-0.9792f,16.6016f,-0.9792f,16.6016f,-0.8750f,16.5528f,-0.8750f,16.5528f,-0.7708f,16.5039f,-0.7708f,
	16.5039f,-0.6667f,16.4551f,-0.6667f,16.4551f,-0.5625f,16.4063f,-0.5625f,16.4063f,-0.4583f,16.3575f,-0.4583f,16.3575f,-0.3542f,16.3086f,-0.3542f,16.3086f,-0.2500f,16.2598f,-0.2500f,16.2598f,-0.1458f,16.2110f,-0.1458f,16.2110f,-0.0417f,16.1621f,-0.0417f,16.1621f,0.0625f,
	16.1133f,0.0625f,16.1133f,0.1667f,16.0645f,0.1667f,16.0645f,0.2708f,16.0157f,0.2708f,16.0157f,0.3750f,15.9668f,0.3750f,15.9668f,0.4792f,15.9180f,0.4792f,15.9180f,0.5833f,15.8692f,0.5833f,15.8692f,0.6875f,15.8203f,0.6875f,15.8203f,0.7917f,15.7715f,0.7917f,
	15.7715f,0.8958f,15.7227f,0.8958f,15.7227f,1.0000f,15.6739f,1.0000f,18.0176f,-4.0000f,17.9688f,-4.0000f,17.9688f,-3.8958f,17.9200f,-3.8958f,17.9200f,-3.7917f,17.8711f,-3.7917f,17.8711f,-3.6875f,17.8223f,-3.6875f,17.8223f,-3.5833f,17.7735f,-3.5833f,17.7735f,-3.4792f,
	17.7246f,-3.4792f,17.7246f,-3.3750f,17.6758f,-3.3750f,17.6758f,-3.2708f,17.6270f,-3.2708f,17.6270f,-3.1667f,17.5782f,-3.1667f,17.5782f,-3.0625f,17.5293f,-3.0625f,17.5293f,-2.9583f,17.4805f,-2.9583f,17.4805f,-2.8542f,17.4317f,-2.8542f,17.4317f,-2.7500f,17.3828f,-2.7500f,
	17.3828f,-2.6458f,17.3340f,-2.6458f,17.3340f,-2.5417f,17.2852f,-2.5417f,17.2852f,-2.4375f,17.2364f,-2.4375f,17.2364f,-2.3333f,17.1875f,-2.3333f,17.1875f,-2.2292f,17.1387f,-2.2292f,17.1387f,-2.1250f,17.0899f,-2.1250f,17.0899f,-2.0208f,17.0410f,-2.0208f,17.0410f,-1.9167f,
	16.9922f,-1.9167f,16.9922f,-1.8125f,16.9434f,-1.8125f,16.9434f,-1.7083f,16.8946f,-1.7083f,16.8946f,-1.6042f,16.8457f,-1.6042f,16.8457f,-1.5000f,16.7969f,-1.5000f,16.7969f,-1.3958f,16.7481f,-1.3958f,16.7481f,-1.2917f,16.6993f,-1.2917f,16.6993f,-1.1875f,16.6504f,-1.1875f,
	16.6504f,-1.0833f,16.6016f,-1.0833f,16.6016f,-0.9792f,16.5528f,-0.9792f,16.5528f,-0.8750f,16.5039f,-0.8750f,16.5039f,-0.7708f,16.4551f,-0.7708f,16.4551f,-0.6667f,16.4063f,-0.6667f,16.4063f,-0.5625f,16.3575f,-0.5625f,16.3575f,-0.4583f,16.3086f,-0.4583f,16.3086f,-0.3542f,
	16.2598f,-0.3542f,16.2598f,-0.2500f,16.2110f,-0.2500f,16.2110f,-0.1458f,16.1621f,-0.1458f,16.1621f,-0.0417f,16.1133f,-0.0417f,16.1133f,0.0625f,16.0645f,0.0625f,16.0645f,0.1667f,16.0157f,0.1667f,16.0157f,0.2708f,15.9668f,0.2708f,15.9668f,0.3750f,15.9180f,0.3750f,
	15.9180f,0.4792f,15.8692f,0.4792f,15.8692f,0.5833f,15.8203f,0.5833f,15.8203f,0.6875f,15.7715f,0.6875f,15.7715f,0.7917f,15.7227f,0.7917f,15.7227f,0.8958f,15.6739f,0.8958f,15.6739f,1.0000f,15.6250f,1.0000f,17.9688f,-4.0000f,17.9200f,-4.0000f,17.9200f,-3.8958f,
	17.8711f,-3.8958f,17.8711f,-3.7917f,17.8223f,-3.7917f,17.8223f,-3.6875f,17.7735f,-3.6875f,17.7735f,-3.5833f,17.7246f,-3.5833f,17.7246f,-3.4792f,17.6758f,-3.4792f,17.6758f,-3.3750f,17.6270f,-3.3750f,17.6270f,-3.2708f,17.5782f,-3.2708f,17.5782f,-3.1667f,17.5293f,-3.1667f,
	17.5293f,-3.0625f,17.4805f,-3.0625f,17.4805f,-2.9583f,17.4317f,-2.9583f,17.4317f,-2.8542f,17.3828f,-2.8542f,17.3828f,-2.7500f,17.3340f,-2.7500f,17.3340f,-2.6458f,17.2852f,-2.6458f,17.2852f,-2.5417f,17.2364f,-2.5417f,17.2364f,-2.4375f,17.1875f,-2.4375f,17.1875f,-2.3333f,
	17.1387f,-2.3333f,17.1387f,-2.2292f,17.0899f,-2.2292f,17.0899f,-2.1250f,17.0410f,-2.1250f,17.0410f,-2.0208f,16.9922f,-2.0208f,16.9922f,-1.9167f,16.9434f,-1.9167f,16.9434f,-1.8125f,16.8946f,-1.8125f,16.8946f,-1.7083f,16.8457f,-1.7083f,16.8457f,-1.6042f,16.7969f,-1.6042f,
	16.7969f,-1.5000f,16.7481f,-1.5000f,16.7481f,-1.3958f,16.6993f,-1.3958f,16.6993f,-1.2917f,16.6504f,-1.2917f,16.6504f,-1.1875f,16.6016f,-1.1875f,16.6016f,-1.0833f,16.5528f,-1.0833f,16.5528f,-0.9792f,16.5039f,-0.9792f,16.5039f,-0.8750f,16.4551f,-0.8750f,16.4551f,-0.7708f,
	16.4063f,-0.7708f,16.4063f,-0.6667f,16.3575f,-0.6667f,16.3575f,-0.5625f,16.3086f,-0.5625f,16.3086f,-0.4583f,16.2598f,-0.4583f,16.2598f,-0.3542f,16.2110f,-0.3542f,16.2110f,-0.2500f,16.1621f,-0.2500f,16.1621f,-0.1458f,16.1133f,-0.1458f,16.1133f,-0.0417f,16.0645f,-0.0417f,
	16.0645f,0.0625f,16.0157f,0.0625f,16.0157f,0.1667f,15.9668f,0.1667f,15.9668f,0.2708f,15.9180f,0.2708f,15.9180f,0.3750f,15.8692f,0.3750f,15.8692f,0.4792f,15.8203f,0.4792f,15.8203f,0.5833f,15.7715f,0.5833f,15.7715f,0.6875f,15.7227f,0.6875f,15.7227f,0.7917f,
	15.6739f,0.7917f,15.6739f,0.8958f,15.6250f,0.8958f,15.6250f,1.0000f,15.5762f,1.0000f,17.9200f,-4.0000f,17.8711f,-4.0000f,17.8711f,-3.8958f,17.8223f,-3.8958f,17.8223f,-3.7917f,17.7735f,-3.7917f,17.7735f,-3.6875f,17.7246f,-3.6875f,17.7246f,-3.5833f,17.6758f,-3.5833f,
	17.6758f,-3.4792f,17.6270f,-3.4792f,17.6270f,-3.3750f,17.5782f,-3.3750f,17.5782f,-3.2708f,17.5293f,-3.2708f,17.5293f,-3.1667f,17.4805f,-3.1667f,17.4805f,-3.0625f,17.4317f,-3.0625f,17.4317f,-2.9583f,17.3828f,-2.9583f,17.3828f,-2.8542f,17.3340f,-2.8542f,17.3340f,-2.7500f,
	17.2852f,-2.7500f,17.2852f,-2.6458f,17.2364f,-2.6458f,17.2364f,-2.5417f,17.1875f,-2.5417f,17.1875f,-2.4375f,17.1387f,-2.4375f,17.1387f,-2.3333f,17.0899f,-2.3333f,17.0899f,-2.2292f,17.0410f,-2.2292f,17.0410f,-2.1250f,16.9922f,-2.1250f,16.9922f,-2.0208f,16.9434f,-2.0208f,
	16.9434f,-1.9167f,16.8946f,-1.9167f,16.8946f,-1.8125f,16.8457f,-1.8125f,16.8457f,-1.7083f,16.7969f,-1.7083f,16.7969f,-1.6042f,16.7481f,-1.6042f,16.7481f,-1.5000f,16.6993f,-1.5000f,16.6993f,-1.3958f,16.6504f,-1.3958f,16.6504f,-1.2917f,16.6016f,-1.2917f,16.6016f,-1.1875f,
	16.5528f,-1.1875f,16.5528f,-1.0833f,16.5039f,-1.0833f,16.5039f,-0.9792f,16.4551f,-0.9792f,16.4551f,-0.8750f,16.4063f,-0.8750f,16.4063f,-0.7708f,16.3575f,-0.7708f,16.3575f,-0.6667f,16.3086f,-0.6667f,16.3086f,-0.5625f,16.2598f,-0.5625f,16.2598f,-0.4583f,16.2110f,-0.4583f,
	16.2110f,-0.3542f,16.1621f,-0.3542f,16.1621f,-0.2500f,16.1133f,-0.2500f,16.1133f,-0.1458f,16.0645f,-0.1458f,16.0645f,-0.0417f,16.0157f,-0.0417f,16.0157f,0.0625f,15.9668f,0.0625f,15.9668f,0.1667f,15.9180f,0.1667f,15.9180f,0.2708f,15.8692f,0.2708f,15.8692f,0.3750f,
	15.8203f,0.3750f,15.8203f,0.4792f,15.7715f,0.4792f,15.7715f,0.5833f,15.7227f,0.5833f,15.7227f,0.6875f,15.6739f,0.6875f,15.6739f,0.7917f,15.6250f,0.7917f,15.6250f,0.8958f,15.5762f,0.8958f,15.5762f,1.0000f,15.5274f,1.0000f,17.8711f,-4.0000f,17.8223f,-4.0000f,
	17.8223f,-3.8958f,17.7735f,-3.8958f,17.7735f,-3.7917f,17.7246f,-3.7917f,17.7246f,-3.6875f,17.6758f,-3.6875f,17.6758f,-3.5833f,17.6270f,-3.5833f,17.6270f,-3.4792f,17.5782f,-3.4792f,17.5782f,-3.3750f,17.5293f,-3.3750f,17.5293f,-3.2708f,17.4805f,-3.2708f,17.4805f,-3.1667f,
	17.4317f,-3.1667f,17.4317f,-3.0625f,17.3828f,-3.0625f,17.3828f,-2.9583f,17.3340f,-2.9583f,17.3340f,-2.8542f,17.2852f,-2.8542f,17.2852f,-2.7500f,17.2364f,-2.7500f,17.2364f,-2.6458f,17.1875f,-2.6458f,17.1875f,-2.5417f,17.1387f,-2.5417f,17.1387f,-2.4375f,17.0899f,-2.4375f,
	17.0899f,-2.3333f,17.0410f,-2.3333f,17.0410f,-2.2292f,16.9922f,-2.2292f,16.9922f,-2.1250f,16.9434f,-2.1250f,16.9434f,-2.0208f,16.8946f,-2.0208f,16.8946f,-1.9167f,16.8457f,-1.9167f,16.8457f,-1.8125f,16.7969f,-1.8125f,16.7969f,-1.7083f,16.7481f,-1.7083f,16.7481f,-1.6042f,
	16.6993f,-1.6042f,16.6993f,-1.5000f,16.6504f,-1.5000f,16.6504f,-1.3958f,16.6016f,-1.3958f,16.6016f,-1.2917f,16.5528f,-1.2917f,16.5528f,-1.1875f,16.5039f,-1.1875f,16.5039f,-1.0833f,16.4551f,-1.0833f,16.4551f,-0.9792f,16.4063f,-0.9792f,16.4063f,-0.8750f,16.3575f,-0.8750f,
	16.3575f,-0.7708f,16.3086f,-0.7708f,16.3086f,-0.6667f,16.2598f,-0.6667f,16.2598f,-0.5625f,16.2110f,-0.5625f,16.2110f,-0.4583f,16.1621f,-0.4583f,16.1621f,-0.3542f,16.1133f,-0.3542f,16.1133f,-0.2500f,16.0645f,-0.2500f,16.0645f,-0.1458f,16.0157f,-0.1458f,16.0157f,-0.0417f,
	15.9668f,-0.0417f,15.9668f,0.0625f,15.9180f,0.0625f,15.9180f,0.1667f,15.8692f,0.1667f,15.8692f,0.2708f,15.8203f,0.2708f,15.8203f,0.3750f,15.7715f,0.3750f,15.7715f,0.4792f,15.7227f,0.4792f,15.7227f,0.5833f,15.6739f,0.5833f,15.6739f,0.6875f,15.6250f,0.6875f,
	15.6250f,0.7917f,15.5762f,0.7917f,15.5762f,0.8958f,15.5274f,0.8958f,15.5274f,1.0000f,15.4785f,1.0000f,17.8223f,-4.0000f,17.7735f,-4.0000f,17.7735f,-3.8958f,17.7246f,-3.8958f,17.7246f,-3.7917f,17.6758f,-3.7917f,17.6758f,-3.6875f,17.6270f,-3.6875f,17.6270f,-3.5833f,
	17.5782f,-3.5833f,17.5782f,-3.4792f,17.5293f,-3.4792f,17.5293f,-3.3750f,17.4805f,-3.3750f,17.4805f,-3.2708f,17.4317f,-3.2708f,17.4317f,-3.1667f,17.3828f,-3.1667f,17.3828f,-3.0625f,17.3340f,-3.0625f,17.3340f,-2.9583f,17.2852f,-2.9583f,17.2852f,-2.8542f,17.2364f,-2.8542f,
	17.2364f,-2.7500f,17.1875f,-2.7500f,17.1875f,-2.6458f,17.1387f,-2.6458f,17.1387f,-2.5417f,17.0899f,-2.5417f,17.0899f,-2.4375f,17.0410f,-2.4375f,17.0410f,-2.3333f,16.9922f,-2.3333f,16.9922f,-2.2292f,16.9434f,-2.2292f,16.9434f,-2.1250f,16.8946f,-2.1250f,16.8946f,-2.0208f,
	16.8457f,-2.0208f,16.8457f,-1.9167f,16.7969f,-1.9167f,16.7969f,-1.8125f,16.7481f,-1.8125f,16.7481f,-1.7083f,16.6993f,-1.7083f,16.6993f,-1.6042f,16.6504f,-1.6042f,16.6504f,-1.5000f,16.6016f,-1.5000f,16.6016f,-1.3958f,16.5528f,-1.3958f,16.5528f,-1.2917f,16.5039f,-1.2917f,
	16.5039f,-1.1875f,16.4551f,-1.1875f,16.4551f,-1.0833f,16.4063f,-1.0833f,16.4063f,-0.9792f,16.3575f,-0.9792f,16.3575f,-0.8750f,16.3086f,-0.8750f,16.3086f,-0.7708f,16.2598f,-0.7708f,16.2598f,-0.6667f,16.2110f,-0.6667f,16.2110f,-0.5625f,16.1621f,-0.5625f,16.1621f,-0.4583f,
	16.1133f,-0.4583f,16.1133f,-0.3542f,16.0645f,-0.3542f,16.0645f,-0.2500f,16.0157f,-0.2500f,16.0157f,-0.1458f,15.9668f,-0.1458f,15.9668f,-0.0417f,15.9180f,-0.0417f,15.9180f,0.0625f,15.8692f,0.0625f,15.8692f,0.1667f,15.8203f,0.1667f,15.8203f,0.2708f,15.7715f,0.2708f,
	15.7715f,0.3750f,15.7227f,0.3750f,15.7227f,0.4792f,15.6739f,0.4792f,15.6739f,0.5833f,15.6250f,0.5833f,15.6250f,0.6875f,15.5762f,0.6875f,15.5762f,0.7917f,15.5274f,0.7917f,15.5274f,0.8958f,15.4785f,0.8958f,15.4785f,1.0000f,15.4297f,1.0000f,17.7735f,-4.0000f,
	17.7246f,-4.0000f,17.7246f,-3.8958f,17.6758f,-3.8958f,17.6758f,-3.7917f,17.6270f,-3.7917f,17.6270f,-3.6875f,17.5782f,-3.6875f,17.5782f,-3.5833f,17.5293f,-3.5833f,17.5293f,-3.4792f,17.4805f,-3.4792f,17.4805f,-3.3750f,17.4317f,-3.3750f,17.4317f,-3.2708f,17.3828f,-3.2708f,
	17.3828f,-3.1667f,17.3340f,-3.1667f,17.3340f,-3.0625f,17.2852f,-3.0625f,17.2852f,-2.9583f,17.2364f,-2.9583f,17.2364f,-2.8542f,17.1875f,-2.8542f,17.1875f,-2.7500f,17.1387f,-2.7500f,17.1387f,-2.6458f,17.0899f,-2.6458f,17.0899f,-2.5417f,17.0410f,-2.5417f,17.0410f,-2.4375f,
	16.9922f,-2.4375f,16.9922f,-2.3333f,16.9434f,-2.3333f,16.9434f,-2.2292f,16.8946f,-2.2292f,16.8946f,-2.1250f,16.8457f,-2.1250f,16.8457f,-2.0208f,16.7969f,-2.0208f,16.7969f,-1.9167f,16.7481f,-1.9167f,16.7481f,-1.8125f,16.6993f,-1.8125f,16.6993f,-1.7083f,16.6504f,-1.7083f,
	16.6504f,-1.6042f,16.6016f,-1.6042f,16.6016f,-1.5000f,16.5528f,-1.5000f,16.5528f,-1.3958f,16.5039f,-1.3958f,16.5039f,-1.2917f,16.4551f,-1.2917f,16.4551f,-1.1875f,16.4063f,-1.1875f,16.4063f,-1.0833f,16.3575f,-1.0833f,16.3575f,-0.9792f,16.3086f,-0.9792f,16.3086f,-0.8750f,
	16.2598f,-0.8750f,16.2598f,-0.7708f,16.2110f,-0.7708f,16.2110f,-0.6667f,16.1621f,-0.6667f,16.1621f,-0.5625f,16.1133f,-0.5625f,16.1133f,-0.4583f,16.0645f,-0.4583f,16.0645f,-0.3542f,16.0157f,-0.3542f,16.0157f,-0.2500f,15.9668f,-0.2500f,15.9668f,-0.1458f,15.9180f,-0.1458f,
	15.9180f,-0.0417f,15.8692f,-0.0417f,15.8692f,0.0625f,15.8203f,0.0625f,15.8203f,0.1667f,15.7715f,0.1667f,15.7715f,0.2708f,15.7227f,0.2708f,15.7227f,0.3750f,15.6739f,0.3750f,15.6739f,0.4792f,15.6250f,0.4792f,15.6250f,0.5833f,15.5762f,0.5833f,15.5762f,0.6875f,
	15.5274f,0.6875f,15.5274f,0.7917f,15.4785f,0.7917f,15.4785f,0.8958f,15.4297f,0.8958f,15.4297f,1.0000f,15.3809f,1.0000f,17.7246f,-4.0000f,17.6758f,-4.0000f,17.6758f,-3.8958f,17.6270f,-3.8958f,17.6270f,-3.7917f,17.5782f,-3.7917f,17.5782f,-3.6875f,17.5293f,-3.6875f,
	17.5293f,-3.5833f,17.4805f,-3.5833f,17.4805f,-3.4792f,17.4317f,-3.4792f,17.4317f,-3.3750f,17.3828f,-3.3750f,17.3828f,-3.2708f,17.3340f,-3.2708f,17.3340f,-3.1667f,17.2852f,-3.1667f,17.2852f,-3.0625f,17.2364f,-3.0625f,17.2364f,-2.9583f,17.1875f,-2.9583f,17.1875f,-2.8542f,
	17.1387f,-2.8542f,17.1387f,-2.7500f,17.0899f,-2.7500f,17.0899f,-2.6458f,17.0410f,-2.6458f,17.0410f,-2.5417f,16.9922f,-2.5417f,16.9922f,-2.4375f,16.9434f,-2.4375f,16.9434f,-2.3333f,16.8946f,-2.3333f,16.8946f,-2.2292f,16.8457f,-2.2292f,16.8457f,-2.1250f,16.7969f,-2.1250f,
	16.7969f,-2.0208f,16.7481f,-2.0208f,16.7481f,-1.9167f,16.6993f,-1.9167f,16.6993f,-1.8125f,16.6504f,-1.8125f,16.6504f,-1.7083f,16.6016f,-1.7083f,16.6016f,-1.6042f,16.5528f,-1.6042f,16.5528f,-1.5000f,16.5039f,-1.5000f,16.5039f,-1.3958f,16.4551f,-1.3958f,16.4551f,-1.2917f,
	16.4063f,-1.2917f,16.4063f,-1.1875f,16.3575f,-1.1875f,16.3575f,-1.0833f,16.3086f,-1.0833f,16.3086f,-0.9792f,16.2598f,-0.9792f,16.2598f,-0.8750f,16.2110f,-0.8750f,16.2110f,-0.7708f,16.1621f,-0.7708f,16.1621f,-0.6667f,16.1133f,-0.6667f,16.1133f,-0.5625f,16.0645f,-0.5625f,
	16.0645f,-0.4583f,16.0157f,-0.4583f,16.0157f,-0.3542f,15.9668f,-0.3542f,15.9668f,-0.2500f,15.9180f,-0.2500f,15.9180f,-0.1458f,15.8692f,-0.1458f,15.8692f,-0.0417f,15.8203f,-0.0417f,15.8203f,0.0625f,15.7715f,0.0625f,15.7715f,0.1667f,15.7227f,0.1667f,15.7227f,0.2708f,
	15.6739f,0.2708f,15.6739f,0.3750f,15.6250f,0.3750f,15.6250f,0.4792f,15.5762f,0.4792f,15.5762f,0.5833f,15.5274f,0.5833f,15.5274f,0.6875f,15.4785f,0.6875f,15.4785f,0.7917f,15.4297f,0.7917f,15.4297f,0.8958f,15.3809f,0.8958f,15.3809f,1.0000f,15.3321f,1.0000f,
	17.6758f,-4.0000f,17.6270f,-4.0000f,17.6270f,-3.8958f,17.5782f,-3.8958f,17.5782f,-3.7917f,17.5293f,-3.7917f,17.5293f,-3.6875f,17.4805f,-3.6875f,17.4805f,-3.5833f,17.4317f,-3.5833f,17.4317f,-3.4792f,17.3828f,-3.4792f,17.3828f,-3.3750f,17.3340f,-3.3750f,17.3340f,-3.2708f,
	17.2852f,-3.2708f,17.2852f,-3.1667f,17.2364f,-3.1667f,17.2364f,-3.0625f,17.1875f,-3.0625f,17.1875f,-2.9583f,17.1387f,-2.9583f,17.1387f,-2.8542f,17.0899f,-2.8542f,17.0899f,-2.7500f,17.0410f,-2.7500f,17.0410f,-2.6458f,16.9922f,-2.6458f,16.9922f,-2.5417f,16.9434f,-2.5417f,
	16.9434f,-2.4375f,16.8946f,-2.4375f,16.8946f,-2.3333f,16.8457f,-2.3333f,16.8457f,-2.2292f,16.7969f,-2.2292f,16.7969f,-2.1250f,16.7481f,-2.1250f,16.7481f,-2.0208f,16.6993f,-2.0208f,16.6993f,-1.9167f,16.6504f,-1.9167f,16.6504f,-1.8125f,16.6016f,-1.8125f,16.6016f,-1.7083f,
	16.5528f,-1.7083f,16.5528f,-1.6042f,16.5039f,-1.6042f,16.5039f,-1.5000f,16.4551f,-1.5000f,16.4551f,-1.3958f,16.4063f,-1.3958f,16.4063f,-1.2917f,16.3575f,-1.2917f,16.3575f,-1.1875f,16.3086f,-1.1875f,16.3086f,-1.0833f,16.2598f,-1.0833f,16.2598f,-0.9792f,16.2110f,-0.9792f,
	16.2110f,-0.8750f,16.1621f,-0.8750f,16.1621f,-0.7708f,16.1133f,-0.7708f,16.1133f,-0.6667f,16.0645f,-0.6667f,16.0645f,-0.5625f,16.0157f,-0.5625f,16.0157f,-0.4583f,15.9668f,-0.4583f,15.9668f,-0.3542f,15.9180f,-0.3542f,15.9180f,-0.2500f,15.8692f,-0.2500f,15.8692f,-0.1458f,
	15.8203f,-0.1458f,15.8203f,-0.0417f,15.7715f,-0.0417f,15.7715f,0.0625f,15.7227f,0.0625f,15.7227f,0.1667f,15.6739f,0.1667f,15.6739f,0.2708f,15.6250f,0.2708f,15.6250f,0.3750f,15.5762f,0.3750f,15.5762f,0.4792f,15.5274f,0.4792f,15.5274f,0.5833f,15.4785f,0.5833f,
	15.4785f,0.6875f,15.4297f,0.6875f,15.4297f,0.7917f,15.3809f,0.7917f,15.3809f,0.8958f,15.3321f,0.8958f,15.3321f,1.0000f,15.2832f,1.0000f,17.6270f,-4.0000f,17.5782f,-4.0000f,17.5782f,-3.8958f,17.5293f,-3.8958f,17.5293f,-3.7917f,17.4805f,-3.7917f,17.4805f,-3.6875f,
	17.4317f,-3.6875f,17.4317f,-3.5833f,17.3828f,-3.5833f,17.3828f,-3.4792f,17.3340f,-3.4792f,17.3340f,-3.3750f,17.2852f,-3.3750f,17.2852f,-3.2708f,17.2364f,-3.2708f,17.2364f,-3.1667f,17.1875f,-3.1667f,17.1875f,-3.0625f,17.1387f,-3.0625f,17.1387f,-2.9583f,17.0899f,-2.9583f,
	17.0899f,-2.8542f,17.0410f,-2.8542f,17.0410f,-2.7500f,16.9922f,-2.7500f,16.9922f,-2.6458f,16.9434f,-2.6458f,16.9434f,-2.5417f,16.8946f,-2.5417f,16.8946f,-2.4375f,16.8457f,-2.4375f,16.8457f,-2.3333f,16.7969f,-2.3333f,16.7969f,-2.2292f,16.7481f,-2.2292f,16.7481f,-2.1250f,
	16.6993f,-2.1250f,16.6993f,-2.0208f,16.6504f,-2.0208f,16.6504f,-1.9167f,16.6016f,-1.9167f,16.6016f,-1.8125f,16.5528f,-1.8125f,16.5528f,-1.7083f,16.5039f,-1.7083f,16.5039f,-1.6042f,16.4551f,-1.6042f,16.4551f,-1.5000f,16.4063f,-1.5000f,16.4063f,-1.3958f,16.3575f,-1.3958f,
	16.3575f,-1.2917f,16.3086f,-1.2917f,16.3086f,-1.1875f,16.2598f,-1.1875f,16.2598f,-1.0833f,16.2110f,-1.0833f,16.2110f,-0.9792f,16.1621f,-0.9792f,16.1621f,-0.8750f,16.1133f,-0.8750f,16.1133f,-0.7708f,16.0645f,-0.7708f,16.0645f,-0.6667f,16.0157f,-0.6667f,16.0157f,-0.5625f,
	15.9668f,-0.5625f,15.9668f,-0.4583f,15.9180f,-0.4583f,15.9180f,-0.3542f,15.8692f,-0.3542f,15.8692f,-0.2500f,15.8203f,-0.2500f,15.8203f,-0.1458f,15.7715f,-0.1458f,15.7715f,-0.0417f,15.7227f,-0.0417f,15.7227f,0.0625f,15.6739f,0.0625f,15.6739f,0.1667f,15.6250f,0.1667f,
	15.6250f,0.2708f,15.5762f,0.2708f,15.5762f,0.3750f,15.5274f,0.3750f,15.5274f,0.4792f,15.4785f,0.4792f,15.4785f,0.5833f,15.4297f,0.5833f,15.4297f,0.6875f,15.3809f,0.6875f,15.3809f,0.7917f,15.3321f,0.7917f,15.3321f,0.8958f,15.2832f,0.8958f,15.2832f,1.0000f,
	15.2344f,1.0000f,17.5782f,-4.0000f,17.5293f,-4.0000f,17.5293f,-3.8958f,17.4805f,-3.8958f,17.4805f,-3.7917f,17.4317f,-3.7917f,17.4317f,-3.6875f,17.3828f,-3.6875f,17.3828f,-3.5833f,17.3340f,-3.5833f,17.3340f,-3.4792f,17.2852f,-3.4792f,17.2852f,-3.3750f,17.2364f,-3.3750f,
	17.2364f,-3.2708f,17.1875f,-3.2708f,17.1875f,-3.1667f,17.1387f,-3.1667f,17.1387f,-3.0625f,17.0899f,-3.0625f,17.0899f,-2.9583f,17.0410f,-2.9583f,17.0410f,-2.8542f,16.9922f,-2.8542f,16.9922f,-2.7500f,16.9434f,-2.7500f,16.9434f,-2.6458f,16.8946f,-2.6458f,16.8946f,-2.5417f,
	16.8457f,-2.5417f,16.8457f,-2.4375f,16.7969f,-2.4375f,16.7969f,-2.3333f,16.7481f,-2.3333f,16.7481f,-2.2292f,16.6993f,-2.2292f,16.6993f,-2.1250f,16.6504f,-2.1250f,16.6504f,-2.0208f,16.6016f,-2.0208f,16.6016f,-1.9167f,16.5528f,-1.9167f,16.5528f,-1.8125f,16.5039f,-1.8125f,
	16.5039f,-1.7083f,16.4551f,-1.7083f,16.4551f,-1.6042f,16.4063f,-1.6042f,16.4063f,-1.5000f,16.3575f,-1.5000f,16.3575f,-1.3958f,16.3086f,-1.3958f,16.3086f,-1.2917f,16.2598f,-1.2917f,16.2598f,-1.1875f,16.2110f,-1.1875f,16.2110f,-1.0833f,16.1621f,-1.0833f,16.1621f,-0.9792f,
	16.1133f,-0.9792f,16.1133f,-0.8750f,16.0645f,-0.8750f,16.0645f,-0.7708f,16.0157f,-0.7708f,16.0157f,-0.6667f,15.9668f,-0.6667f,15.9668f,-0.5625f,15.9180f,-0.5625f,15.9180f,-0.4583f,15.8692f,-0.4583f,15.8692f,-0.3542f,15.8203f,-0.3542f,15.8203f,-0.2500f,15.7715f,-0.2500f,
	15.7715f,-0.1458f,15.7227f,-0.1458f,15.7227f,-0.0417f,15.6739f,-0.0417f,15.6739f,0.0625f,15.6250f,0.0625f,15.6250f,0.1667f,15.5762f,0.1667f,15.5762f,0.2708f,15.5274f,0.2708f,15.5274f,0.3750f,15.4785f,0.3750f,15.4785f,0.4792f,15.4297f,0.4792f,15.4297f,0.5833f,
	15.3809f,0.5833f,15.3809f,0.6875f,15.3321f,0.6875f,15.3321f,0.7917f,15.2832f,0.7917f,15.2832f,0.8958f,15.2344f,0.8958f,15.2344f,1.0000f,15.1856f,1.0000f,17.5293f,-4.0000f,17.4805f,-4.0000f,17.4805f,-3.8958f,17.4317f,-3.8958f,17.4317f,-3.7917f,17.3828f,-3.7917f,
	17.3828f,-3.6875f,17.3340f,-3.6875f,17.3340f,-3.5833f,17.2852f,-3.5833f,17.2852f,-3.4792f,17.2364f,-3.4792f,17.2364f,-3.3750f,17.1875f,-3.3750f,17.1875f,-3.2708f,17.1387f,-3.2708f,17.1387f,-3.1667f,17.0899f,-3.1667f,17.0899f,-3.0625f,17.0410f,-3.0625f,17.0410f,-2.9583f,
	16.9922f,-2.9583f,16.9922f,-2.8542f,16.9434f,-2.8542f,16.9434f,-2.7500f,16.8946f,-2.7500f,16.8946f,-2.6458f,16.8457f,-2.6458f,16.8457f,-2.5417f,16.7969f,-2.5417f,16.7969f,-2.4375f,16.7481f,-2.4375f,16.7481f,-2.3333f,16.6993f,-2.3333f,16.6993f,-2.2292f,16.6504f,-2.2292f,
	16.6504f,-2.1250f,16.6016f,-2.1250f,16.6016f,-2.0208f,16.5528f,-2.0208f,16.5528f,-1.9167f,16.5039f,-1.9167f,16.5039f,-1.8125f,16.4551f,-1.8125f,16.4551f,-1.7083f,16.4063f,-1.7083f,16.4063f,-1.6042f,16.3575f,-1.6042f,16.3575f,-1.5000f,16.3086f,-1.5000f,16.3086f,-1.3958f,
	16.2598f,-1.3958f,16.2598f,-1.2917f,16.2110f,-1.2917f,16.2110f,-1.1875f,16.1621f,-1.1875f,16.1621f,-1.0833f,16.1133f,-1.0833f,16.1133f,-0.9792f,16.0645f,-0.9792f,16.0645f,-0.8750f,16.0157f,-0.8750f,16.0157f,-0.7708f,15.9668f,-0.7708f,15.9668f,-0.6667f,15.9180f,-0.6667f,
	15.9180f,-0.5625f,15.8692f,-0.5625f,15.8692f,-0.4583f,15.8203f,-0.4583f,15.8203f,-0.3542f,15.7715f,-0.3542f,15.7715f,-0.2500f,15.7227f,-0.2500f,15.7227f,-0.1458f,15.6739f,-0.1458f,15.6739f,-0.0417f,15.6250f,-0.0417f,15.6250f,0.0625f,15.5762f,0.0625f,15.5762f,0.1667f,
	15.5274f,0.1667f,15.5274f,0.2708f,15.4785f,0.2708f,15.4785f,0.3750f,15.4297f,0.3750f,15.4297f,0.4792f,15.3809f,0.4792f,15.3809f,0.5833f,15.3321f,0.5833f,15.3321f,0.6875f,15.2832f,0.6875f,15.2832f,0.7917f,15.2344f,0.7917f,15.2344f,0.8958f,15.1856f,0.8958f,
	15.1856f,1.0000f,15.1367f,1.0000f,17.4805f,-4.0000f,17.4317f,-4.0000f,17.4317f,-3.8958f,17.3828f,-3.8958f,17.3828f,-3.7917f,17.3340f,-3.7917f,17.3340f,-3.6875f,17.2852f,-3.6875f,17.2852f,-3.5833f,17.2364f,-3.5833f,17.2364f,-3.4792f,17.1875f,-3.4792f,17.1875f,-3.3750f,
	17.1387f,-3.3750f,17.1387f,-3.2708f,17.0899f,-3.2708f,17.0899f,-3.1667f,17.0410f,-3.1667f,17.0410f,-3.0625f,16.9922f,-3.0625f,16.9922f,-2.9583f,16.9434f,-2.9583f,16.9434f,-2.8542f,16.8946f,-2.8542f,16.8946f,-2.7500f,16.8457f,-2.7500f,16.8457f,-2.6458f,16.7969f,-2.6458f,
	16.7969f,-2.5417f,16.7481f,-2.5417f,16.7481f,-2.4375f,16.6993f,-2.4375f,16.6993f,-2.3333f,16.6504f,-2.3333f,16.6504f,-2.2292f,16.6016f,-2.2292f,16.6016f,-2.1250f,16.5528f,-2.1250f,16.5528f,-2.0208f,16.5039f,-2.0208f,16.5039f,-1.9167f,16.4551f,-1.9167f,16.4551f,-1.8125f,
	16.4063f,-1.8125f,16.4063f,-1.7083f,16.3575f,-1.7083f,16.3575f,-1.6042f,16.3086f,-1.6042f,16.3086f,-1.5000f,16.2598f,-1.5000f,16.2598f,-1.3958f,16.2110f,-1.3958f,16.2110f,-1.2917f,16.1621f,-1.2917f,16.1621f,-1.1875f,16.1133f,-1.1875f,16.1133f,-1.0833f,16.0645f,-1.0833f,
	16.0645f,-0.9792f,16.0157f,-0.9792f,16.0157f,-0.8750f,15.9668f,-0.8750f,15.9668f,-0.7708f,15.9180f,-0.7708f,15.9180f,-0.6667f,15.8692f,-0.6667f,15.8692f,-0.5625f,15.8203f,-0.5625f,15.8203f,-0.4583f,15.7715f,-0.4583f,15.7715f,-0.3542f,15.7227f,-0.3542f,15.7227f,-0.2500f,
	15.6739f,-0.2500f,15.6739f,-0.1458f,15.6250f,-0.1458f,15.6250f,-0.0417f,15.5762f,-0.0417f,15.5762f,0.0625f,15.5274f,0.0625f,15.5274f,0.1667f,15.4785f,0.1667f,15.4785f,0.2708f,15.4297f,0.2708f,15.4297f,0.3750f,15.3809f,0.3750f,15.3809f,0.4792f,15.3321f,0.4792f,
	15.3321f,0.5833f,15.2832f,0.5833f,15.2832f,0.6875f,15.2344f,0.6875f,15.2344f,0.7917f,15.1856f,0.7917f,15.1856f,0.8958f,15.1367f,0.8958f,15.1367f,1.0000f,15.0879f,1.0000f,17.4317f,-4.0000f,17.3828f,-4.0000f,17.3828f,-3.8958f,17.3340f,-3.8958f,17.3340f,-3.7917f,
	17.2852f,-3.7917f,17.2852f,-3.6875f,17.2364f,-3.6875f,17.2364f,-3.5833f,17.1875f,-3.5833f,17.1875f,-3.4792f,17.1387f,-3.4792f,17.1387f,-3.3750f,17.0899f,-3.3750f,17.0899f,-3.2708f,17.0410f,-3.2708f,17.0410f,-3.1667f,16.9922f,-3.1667f,16.9922f,-3.0625f,16.9434f,-3.0625f,
	16.9434f,-2.9583f,16.8946f,-2.9583f,16.8946f,-2.8542f,16.8457f,-2.8542f,16.8457f,-2.7500f,16.7969f,-2.7500f,16.7969f,-2.6458f,16.7481f,-2.6458f,16.7481f,-2.5417f,16.6993f,-2.5417f,16.6993f,-2.4375f,16.6504f,-2.4375f,16.6504f,-2.3333f,16.6016f,-2.3333f,16.6016f,-2.2292f,
	16.5528f,-2.2292f,16.5528f,-2.1250f,16.5039f,-2.1250f,16.5039f,-2.0208f,16.4551f,-2.0208f,16.4551f,-1.9167f,16.4063f,-1.9167f,16.4063f,-1.8125f,16.3575f,-1.8125f,16.3575f,-1.7083f,16.3086f,-1.7083f,16.3086f,-1.6042f,16.2598f,-1.6042f,16.2598f,-1.5000f,16.2110f,-1.5000f,
	16.2110f,-1.3958f,16.1621f,-1.3958f,16.1621f,-1.2917f,16.1133f,-1.2917f,16.1133f,-1.1875f,16.0645f,-1.1875f,16.0645f,-1.0833f,16.0157f,-1.0833f,16.0157f,-0.9792f,15.9668f,-0.9792f,15.9668f,-0.8750f,15.9180f,-0.8750f,15.9180f,-0.7708f,15.8692f,-0.7708f,15.8692f,-0.6667f,
	15.8203f,-0.6667f,15.8203f,-0.5625f,15.7715f,-0.5625f,15.7715f,-0.4583f,15.7227f,-0.4583f,15.7227f,-0.3542f,15.6739f,-0.3542f,15.6739f,-0.2500f,15.6250f,-0.2500f,15.6250f,-0.1458f,15.5762f,-0.1458f,15.5762f,-0.0417f,15.5274f,-0.0417f,15.5274f,0.0625f,15.4785f,0.0625f,
	15.4785f,0.1667f,15.4297f,0.1667f,15.4297f,0.2708f,15.3809f,0.2708f,15.3809f,0.3750f,15.3321f,0.3750f,15.3321f,0.4792f,15.2832f,0.4792f,15.2832f,0.5833f,15.2344f,0.5833f,15.2344f,0.6875f,15.1856f,0.6875f,15.1856f,0.7917f,15.1367f,0.7917f,15.1367f,0.8958f,
	15.0879f,0.8958f,15.0879f,1.0000f,15.0391f,1.0000f,17.3828f,-4.0000f,17.3340f,-4.0000f,17.3340f,-3.8958f,17.2852f,-3.8958f,17.2852f,-3.7917f,17.2364f,-3.7917f,17.2364f,-3.6875f,17.1875f,-3.6875f,17.1875f,-3.5833f,17.1387f,-3.5833f,17.1387f,-3.4792f,17.0899f,-3.4792f,
	17.0899f,-3.3750f,17.0410f,-3.3750f,17.0410f,-3.2708f,16.9922f,-3.2708f,16.9922f,-3.1667f,16.9434f,-3.1667f,16.9434f,-3.0625f,16.8946f,-3.0625f,16.8946f,-2.9583f,16.8457f,-2.9583f,16.8457f,-2.8542f,16.7969f,-2.8542f,16.7969f,-2.7500f,16.7481f,-2.7500f,16.7481f,-2.6458f,
	16.6993f,-2.6458f,16.6993f,-2.5417f,16.6504f,-2.5417f,16.6504f,-2.4375f,16.6016f,-2.4375f,16.6016f,-2.3333f,16.5528f,-2.3333f,16.5528f,-2.2292f,16.5039f,-2.2292f,16.5039f,-2.1250f,16.4551f,-2.1250f,16.4551f,-2.0208f,16.4063f,-2.0208f,16.4063f,-1.9167f,16.3575f,-1.9167f,
	16.3575f,-1.8125f,16.3086f,-1.8125f,16.3086f,-1.7083f,16.2598f,-1.7083f,16.2598f,-1.6042f,16.2110f,-1.6042f,16.2110f,-1.5000f,16.1621f,-1.5000f,16.1621f,-1.3958f,16.1133f,-1.3958f,16.1133f,-1.2917f,16.0645f,-1.2917f,16.0645f,-1.1875f,16.0157f,-1.1875f,16.0157f,-1.0833f,
	15.9668f,-1.0833f,15.9668f,-0.9792f,15.9180f,-0.9792f,15.9180f,-0.8750f,15.8692f,-0.8750f,15.8692f,-0.7708f,15.8203f,-0.7708f,15.8203f,-0.6667f,15.7715f,-0.6667f,15.7715f,-0.5625f,15.7227f,-0.5625f,15.7227f,-0.4583f,15.6739f,-0.4583f,15.6739f,-0.3542f,15.6250f,-0.3542f,
	15.6250f,-0.2500f,15.5762f,-0.2500f,15.5762f,-0.1458f,15.5274f,-0.1458f,15.5274f,-0.0417f,15.4785f,-0.0417f,15.4785f,0.0625f,15.4297f,0.0625f,15.4297f,0.1667f,15.3809f,0.1667f,15.3809f,0.2708f,15.3321f,0.2708f,15.3321f,0.3750f,15.2832f,0.3750f,15.2832f,0.4792f,
	15.2344f,0.4792f,15.2344f,0.5833f,15.1856f,0.5833f,15.1856f,0.6875f,15.1367f,0.6875f,15.1367f,0.7917f,15.0879f,0.7917f,15.0879f,0.8958f,15.0391f,0.8958f,15.0391f,1.0000f,14.9903f,1.0000f,17.3340f,-4.0000f,17.2852f,-4.0000f,17.2852f,-3.8958f,17.2364f,-3.8958f,
	17.2364f,-3.7917f,17.1875f,-3.7917f,17.1875f,-3.6875f,17.1387f,-3.6875f,17.1387f,-3.5833f,17.0899f,-3.5833f,17.0899f,-3.4792f,17.0410f,-3.4792f,17.0410f,-3.3750f,16.9922f,-3.3750f,16.9922f,-3.2708f,16.9434f,-3.2708f,16.9434f,-3.1667f,16.8946f,-3.1667f,16.8946f,-3.0625f,
	16.8457f,-3.0625f,16.8457f,-2.9583f,16.7969f,-2.9583f,16.7969f,-2.8542f,16.7481f,-2.8542f,16.7481f,-2.7500f,16.6993f,-2.7500f,16.6993f,-2.6458f,16.6504f,-2.6458f,16.6504f,-2.5417f,16.6016f,-2.5417f,16.6016f,-2.4375f,16.5528f,-2.4375f,16.5528f,-2.3333f,16.5039f,-2.3333f,
	16.5039f,-2.2292f,16.4551f,-2.2292f,16.4551f,-2.1250f,16.4063f,-2.1250f,16.4063f,-2.0208f,16.3575f,-2.0208f,16.3575f,-1.9167f,16.3086f,-1.9167f,16.3086f,-1.8125f,16.2598f,-1.8125f,16.2598f,-1.7083f,16.2110f,-1.7083f,16.2110f,-1.6042f,16.1621f,-1.6042f,16.1621f,-1.5000f,
	16.1133f,-1.5000f,16.1133f,-1.3958f,16.0645f,-1.3958f,16.0645f,-1.2917f,16.0157f,-1.2917f,16.0157f,-1.1875f,15.9668f,-1.1875f,15.9668f,-1.0833f,15.9180f,-1.0833f,15.9180f,-0.9792f,15.8692f,-0.9792f,15.8692f,-0.8750f,15.8203f,-0.8750f,15.8203f,-0.7708f,15.7715f,-0.7708f,
	15.7715f,-0.6667f,15.7227f,-0.6667f,15.7227f,-0.5625f,15.6739f,-0.5625f,15.6739f,-0.4583f,15.6250f,-0.4583f,15.6250f,-0.3542f,15.5762f,-0.3542f,15.5762f,-0.2500f,15.5274f,-0.2500f,15.5274f,-0.1458f,15.4785f,-0.1458f,15.4785f,-0.0417f,15.4297f,-0.0417f,15.4297f,0.0625f,
	15.3809f,0.0625f,15.3809f,0.1667f,15.3321f,0.1667f,15.3321f,0.2708f,15.2832f,0.2708f,15.2832f,0.3750f,15.2344f,0.3750f,15.2344f,0.4792f,15.1856f,0.4792f,15.1856f,0.5833f,15.1367f,0.5833f,15.1367f,0.6875f,15.0879f,0.6875f,15.0879f,0.7917f,15.0391f,0.7917f,
	15.0391f,0.8958f,14.9903f,0.8958f,14.9903f,1.0000f,14.9414f,1.0000f,17.2852f,-4.0000f,17.2364f,-4.0000f,17.2364f,-3.8958f,17.1875f,-3.8958f,17.1875f,-3.7917f,17.1387f,-3.7917f,17.1387f,-3.6875f,17.0899f,-3.6875f,17.0899f,-3.5833f,17.0410f,-3.5833f,17.0410f,-3.4792f,
	16.9922f,-3.4792f,16.9922f,-3.3750f,16.9434f,-3.3750f,16.9434f,-3.2708f,16.8946f,-3.2708f,16.8946f,-3.1667f,16.8457f,-3.1667f,16.8457f,-3.0625f,16.7969f,-3.0625f,16.7969f,-2.9583f,16.7481f,-2.9583f,16.7481f,-2.8542f,16.6993f,-2.8542f,16.6993f,-2.7500f,16.6504f,-2.7500f,
	16.6504f,-2.6458f,16.6016f,-2.6458f,16.6016f,-2.5417f,16.5528f,-2.5417f,16.5528f,-2.4375f,16.5039f,-2.4375f,16.5039f,-2.3333f,16.4551f,-2.3333f,16.4551f,-2.2292f,16.4063f,-2.2292f,16.4063f,-2.1250f,16.3575f,-2.1250f,16.3575f,-2.0208f,16.3086f,-2.0208f,16.3086f,-1.9167f,
	16.2598f,-1.9167f,16.2598f,-1.8125f,16.2110f,-1.8125f,16.2110f,-1.7083f,16.1621f,-1.7083f,16.1621f,-1.6042f,16.1133f,-1.6042f,16.1133f,-1.5000f,16.0645f,-1.5000f,16.0645f,-1.3958f,16.0157f,-1.3958f,16.0157f,-1.2917f,15.9668f,-1.2917f,15.9668f,-1.1875f,15.9180f,-1.1875f,
	15.9180f,-1.0833f,15.8692f,-1.0833f,15.8692f,-0.9792f,15.8203f,-0.9792f,15.8203f,-0.8750f,15.7715f,-0.8750f,15.7715f,-0.7708f,15.7227f,-0.7708f,15.7227f,-0.6667f,15.6739f,-0.6667f,15.6739f,-0.5625f,15.6250f,-0.5625f,15.6250f,-0.4583f,15.5762f,-0.4583f,15.5762f,-0.3542f,
	15.5274f,-0.3542f,15.5274f,-0.2500f,15.4785f,-0.2500f,15.4785f,-0.1458f,15.4297f,-0.1458f,15.4297f,-0.0417f,15.3809f,-0.0417f,15.3809f,0.0625f,15.3321f,0.0625f,15.3321f,0.1667f,15.2832f,0.1667f,15.2832f,0.2708f,15.2344f,0.2708f,15.2344f,0.3750f,15.1856f,0.3750f,
	15.1856f,0.4792f,15.1367f,0.4792f,15.1367f,0.5833f,15.0879f,0.5833f,15.0879f,0.6875f,15.0391f,0.6875f,15.0391f,0.7917f,14.9903f,0.7917f,14.9903f,0.8958f,14.9414f,0.8958f,14.9414f,1.0000f,14.8926f,1.0000f,17.2364f,-4.0000f,17.1875f,-4.0000f,17.1875f,-3.8958f,
	17.1387f,-3.8958f,17.1387f,-3.7917f,17.0899f,-3.7917f,17.0899f,-3.6875f,17.0410f,-3.6875f,17.0410f,-3.5833f,16.9922f,-3.5833f,16.9922f,-3.4792f,16.9434f,-3.4792f,16.9434f,-3.3750f,16.8946f,-3.3750f,16.8946f,-3.2708f,16.8457f,-3.2708f,16.8457f,-3.1667f,16.7969f,-3.1667f,
	16.7969f,-3.0625f,16.7481f,-3.0625f,16.7481f,-2.9583f,16.6993f,-2.9583f,16.6993f,-2.8542f,16.6504f,-2.8542f,16.6504f,-2.7500f,16.6016f,-2.7500f,16.6016f,-2.6458f,16.5528f,-2.6458f,16.5528f,-2.5417f,16.5039f,-2.5417f,16.5039f,-2.4375f,16.4551f,-2.4375f,16.4551f,-2.3333f,
	16.4063f,-2.3333f,16.4063f,-2.2292f,16.3575f,-2.2292f,16.3575f,-2.1250f,16.3086f,-2.1250f,16.3086f,-2.0208f,16.2598f,-2.0208f,16.2598f,-1.9167f,16.2110f,-1.9167f,16.2110f,-1.8125f,16.1621f,-1.8125f,16.1621f,-1.7083f,16.1133f,-1.7083f,16.1133f,-1.6042f,16.0645f,-1.6042f,
	16.0645f,-1.5000f,16.0157f,-1.5000f,16.0157f,-1.3958f,15.9668f,-1.3958f,15.9668f,-1.2917f,15.9180f,-1.2917f,15.9180f,-1.1875f,15.8692f,-1.1875f,15.8692f,-1.0833f,15.8203f,-1.0833f,15.8203f,-0.9792f,15.7715f,-0.9792f,15.7715f,-0.8750f,15.7227f,-0.8750f,15.7227f,-0.7708f,
	15.6739f,-0.7708f,15.6739f,-0.6667f,15.6250f,-0.6667f,15.6250f,-0.5625f,15.5762f,-0.5625f,15.5762f,-0.4583f,15.5274f,-0.4583f,15.5274f,-0.3542f,15.4785f,-0.3542f,15.4785f,-0.2500f,15.4297f,-0.2500f,15.4297f,-0.1458f,15.3809f,-0.1458f,15.3809f,-0.0417f,15.3321f,-0.0417f,
	15.3321f,0.0625f,15.2832f,0.0625f,15.2832f,0.1667f,15.2344f,0.1667f,15.2344f,0.2708f,15.1856f,0.2708f,15.1856f,0.3750f,15.1367f,0.3750f,15.1367f,0.4792f,15.0879f,0.4792f,15.0879f,0.5833f,15.0391f,0.5833f,15.0391f,0.6875f,14.9903f,0.6875f,14.9903f,0.7917f,
	14.9414f,0.7917f,14.9414f,0.8958f,14.8926f,0.8958f,14.8926f,1.0000f,14.8438f,1.0000f,17.1875f,-4.0000f,17.1387f,-4.0000f,17.1387f,-3.8958f,17.0899f,-3.8958f,17.0899f,-3.7917f,17.0410f,-3.7917f,17.0410f,-3.6875f,16.9922f,-3.6875f,16.9922f,-3.5833f,16.9434f,-3.5833f,
	16.9434f,-3.4792f,16.8946f,-3.4792f,16.8946f,-3.3750f,16.8457f,-3.3750f,16.8457f,-3.2708f,16.7969f,-3.2708f,16.7969f,-3.1667f,16.7481f,-3.1667f,16.7481f,-3.0625f,16.6993f,-3.0625f,16.6993f,-2.9583f,16.6504f,-2.9583f,16.6504f,-2.8542f,16.6016f,-2.8542f,16.6016f,-2.7500f,
	16.5528f,-2.7500f,16.5528f,-2.6458f,16.5039f,-2.6458f,16.5039f,-2.5417f,16.4551f,-2.5417f,16.4551f,-2.4375f,16.4063f,-2.4375f,16.4063f,-2.3333f,16.3575f,-2.3333f,16.3575f,-2.2292f,16.3086f,-2.2292f,16.3086f,-2.1250f,16.2598f,-2.1250f,16.2598f,-2.0208f,16.2110f,-2.0208f,
	16.2110f,-1.9167f,16.1621f,-1.9167f,16.1621f,-1.8125f,16.1133f,-1.8125f,16.1133f,-1.7083f,16.0645f,-1.7083f,16.0645f,-1.6042f,16.0157f,-1.6042f,16.0157f,-1.5000f,15.9668f,-1.5000f,15.9668f,-1.3958f,15.9180f,-1.3958f,15.9180f,-1.2917f,15.8692f,-1.2917f,15.8692f,-1.1875f,
	15.8203f,-1.1875f,15.8203f,-1.0833f,15.7715f,-1.0833f,15.7715f,-0.9792f,15.7227f,-0.9792f,15.7227f,-0.8750f,15.6739f,-0.8750f,15.6739f,-0.7708f,15.6250f,-0.7708f,15.6250f,-0.6667f,15.5762f,-0.6667f,15.5762f,-0.5625f,15.5274f,-0.5625f,15.5274f,-0.4583f,15.4785f,-0.4583f,
	15.4785f,-0.3542f,15.4297f,-0.3542f,15.4297f,-0.2500f,15.3809f,-0.2500f,15.3809f,-0.1458f,15.3321f,-0.1458f,15.3321f,-0.0417f,15.2832f,-0.0417f,15.2832f,0.0625f,15.2344f,0.0625f,15.2344f,0.1667f,15.1856f,0.1667f,15.1856f,0.2708f,15.1367f,0.2708f,15.1367f,0.3750f,
	15.0879f,0.3750f,15.0879f,0.4792f,15.0391f,0.4792f,15.0391f,0.5833f,14.9903f,0.5833f,14.9903f,0.6875f,14.9414f,0.6875f,14.9414f,0.7917f,14.8926f,0.7917f,14.8926f,0.8958f,14.8438f,0.8958f,14.8438f,1.0000f,14.7949f,1.0000f,17.1387f,-4.0000f,17.0899f,-4.0000f,
	17.0899f,-3.8958f,17.0410f,-3.8958f,17.0410f,-3.7917f,16.9922f,-3.7917f,16.9922f,-3.6875f,16.9434f,-3.6875f,16.9434f,-3.5833f,16.8946f,-3.5833f,16.8946f,-3.4792f,16.8457f,-3.4792f,16.8457f,-3.3750f,16.7969f,-3.3750f,16.7969f,-3.2708f,16.7481f,-3.2708f,16.7481f,-3.1667f,
	16.6993f,-3.1667f,16.6993f,-3.0625f,16.6504f,-3.0625f,16.6504f,-2.9583f,16.6016f,-2.9583f,16.6016f,-2.8542f,16.5528f,-2.8542f,16.5528f,-2.7500f,16.5039f,-2.7500f,16.5039f,-2.6458f,16.4551f,-2.6458f,16.4551f,-2.5417f,16.4063f,-2.5417f,16.4063f,-2.4375f,16.3575f,-2.4375f,
	16.3575f,-2.3333f,16.3086f,-2.3333f,16.3086f,-2.2292f,16.2598f,-2.2292f,16.2598f,-2.1250f,16.2110f,-2.1250f,16.2110f,-2.0208f,16.1621f,-2.0208f,16.1621f,-1.9167f,16.1133f,-1.9167f,16.1133f,-1.8125f,16.0645f,-1.8125f,16.0645f,-1.7083f,16.0157f,-1.7083f,16.0157f,-1.6042f,
	15.9668f,-1.6042f,15.9668f,-1.5000f,15.9180f,-1.5000f,15.9180f,-1.3958f,15.8692f,-1.3958f,15.8692f,-1.2917f,15.8203f,-1.2917f,15.8203f,-1.1875f,15.7715f,-1.1875f,15.7715f,-1.0833f,15.7227f,-1.0833f,15.7227f,-0.9792f,15.6739f,-0.9792f,15.6739f,-0.8750f,15.6250f,-0.8750f,
	15.6250f,-0.7708f,15.5762f,-0.7708f,15.5762f,-0.6667f,15.5274f,-0.6667f,15.5274f,-0.5625f,15.4785f,-0.5625f,15.4785f,-0.4583f,15.4297f,-0.4583f,15.4297f,-0.3542f,15.3809f,-0.3542f,15.3809f,-0.2500f,15.3321f,-0.2500f,15.3321f,-0.1458f,15.2832f,-0.1458f,15.2832f,-0.0417f,
	15.2344f,-0.0417f,15.2344f,0.0625f,15.1856f,0.0625f,15.1856f,0.1667f,15.1367f,0.1667f,15.1367f,0.2708f,15.0879f,0.2708f,15.0879f,0.3750f,15.0391f,0.3750f,15.0391f,0.4792f,14.9903f,0.4792f,14.9903f,0.5833f,14.9414f,0.5833f,14.9414f,0.6875f,14.8926f,0.6875f,
	14.8926f,0.7917f,14.8438f,0.7917f,14.8438f,0.8958f,14.7949f,0.8958f,14.7949f,1.0000f,14.7461f,1.0000f,17.0899f,-4.0000f,17.0410f,-4.0000f,17.0410f,-3.8958f,16.9922f,-3.8958f,16.9922f,-3.7917f,16.9434f,-3.7917f,16.9434f,-3.6875f,16.8946f,-3.6875f,16.8946f,-3.5833f,
	16.8457f,-3.5833f,16.8457f,-3.4792f,16.7969f,-3.4792f,16.7969f,-3.3750f,16.7481f,-3.3750f,16.7481f,-3.2708f,16.6993f,-3.2708f,16.6993f,-3.1667f,16.6504f,-3.1667f,16.6504f,-3.0625f,16.6016f,-3.0625f,16.6016f,-2.9583f,16.5528f,-2.9583f,16.5528f,-2.8542f,16.5039f,-2.8542f,
	16.5039f,-2.7500f,16.4551f,-2.7500f,16.4551f,-2.6458f,16.4063f,-2.6458f,16.4063f,-2.5417f,16.3575f,-2.5417f,16.3575f,-2.4375f,16.3086f,-2.4375f,16.3086f,-2.3333f,16.2598f,-2.3333f,16.2598f,-2.2292f,16.2110f,-2.2292f,16.2110f,-2.1250f,16.1621f,-2.1250f,16.1621f,-2.0208f,
	16.1133f,-2.0208f,16.1133f,-1.9167f,16.0645f,-1.9167f,16.0645f,-1.8125f,16.0157f,-1.8125f,16.0157f,-1.7083f,15.9668f,-1.7083f,15.9668f,-1.6042f,15.9180f,-1.6042f,15.9180f,-1.5000f,15.8692f,-1.5000f,15.8692f,-1.3958f,15.8203f,-1.3958f,15.8203f,-1.2917f,15.7715f,-1.2917f,
	15.7715f,-1.1875f,15.7227f,-1.1875f,15.7227f,-1.0833f,15.6739f,-1.0833f,15.6739f,-0.9792f,15.6250f,-0.9792f,15.6250f,-0.8750f,15.5762f,-0.8750f,15.5762f,-0.7708f,15.5274f,-0.7708f,15.5274f,-0.6667f,15.4785f,-0.6667f,15.4785f,-0.5625f,15.4297f,-0.5625f,15.4297f,-0.4583f,
	15.3809f,-0.4583f,15.3809f,-0.3542f,15.3321f,-0.3542f,15.3321f,-0.2500f,15.2832f,-0.2500f,15.2832f,-0.1458f,15.2344f,-0.1458f,15.2344f,-0.0417f,15.1856f,-0.0417f,15.1856f,0.0625f,15.1367f,0.0625f,15.1367f,0.1667f,15.0879f,0.1667f,15.0879f,0.2708f,15.0391f,0.2708f,
	15.0391f,0.3750f,14.9903f,0.3750f,14.9903f,0.4792f,14.9414f,0.4792f,14.9414f,0.5833f,14.8926f,0.5833f,14.8926f,0.6875f,14.8438f,0.6875f,14.8438f,0.7917f,14.7949f,0.7917f,14.7949f,0.8958f,14.7461f,0.8958f,14.7461f,1.0000f,14.6973f,1.0000f,17.0410f,-4.0000f,
	16.9922f,-4.0000f,16.9922f,-3.8958f,16.9434f,-3.8958f,16.9434f,-3.7917f,16.8946f,-3.7917f,16.8946f,-3.6875f,16.8457f,-3.6875f,16.8457f,-3.5833f,16.7969f,-3.5833f,16.7969f,-3.4792f,16.7481f,-3.4792f,16.7481f,-3.3750f,16.6993f,-3.3750f,16.6993f,-3.2708f,16.6504f,-3.2708f,
	16.6504f,-3.1667f,16.6016f,-3.1667f,16.6016f,-3.0625f,16.5528f,-3.0625f,16.5528f,-2.9583f,16.5039f,-2.9583f,16.5039f,-2.8542f,16.4551f,-2.8542f,16.4551f,-2.7500f,16.4063f,-2.7500f,16.4063f,-2.6458f,16.3575f,-2.6458f,16.3575f,-2.5417f,16.3086f,-2.5417f,16.3086f,-2.4375f,
	16.2598f,-2.4375f,16.2598f,-2.3333f,16.2110f,-2.3333f,16.2110f,-2.2292f,16.1621f,-2.2292f,16.1621f,-2.1250f,16.1133f,-2.1250f,16.1133f,-2.0208f,16.0645f,-2.0208f,16.0645f,-1.9167f,16.0157f,-1.9167f,16.0157f,-1.8125f,15.9668f,-1.8125f,15.9668f,-1.7083f,15.9180f,-1.7083f,
	15.9180f,-1.6042f,15.8692f,-1.6042f,15.8692f,-1.5000f,15.8203f,-1.5000f,15.8203f,-1.3958f,15.7715f,-1.3958f,15.7715f,-1.2917f,15.7227f,-1.2917f,15.7227f,-1.1875f,15.6739f,-1.1875f,15.6739f,-1.0833f,15.6250f,-1.0833f,15.6250f,-0.9792f,15.5762f,-0.9792f,15.5762f,-0.8750f,
	15.5274f,-0.8750f,15.5274f,-0.7708f,15.4785f,-0.7708f,15.4785f,-0.6667f,15.4297f,-0.6667f,15.4297f,-0.5625f,15.3809f,-0.5625f,15.3809f,-0.4583f,15.3321f,-0.4583f,15.3321f,-0.3542f,15.2832f,-0.3542f,15.2832f,-0.2500f,15.2344f,-0.2500f,15.2344f,-0.1458f,15.1856f,-0.1458f,
	15.1856f,-0.0417f,15.1367f,-0.0417f,15.1367f,0.0625f,15.0879f,0.0625f,15.0879f,0.1667f,15.0391f,0.1667f,15.0391f,0.2708f,14.9903f,0.2708f,14.9903f,0.3750f,14.9414f,0.3750f,14.9414f,0.4792f,14.8926f,0.4792f,14.8926f,0.5833f,14.8438f,0.5833f,14.8438f,0.6875f,
	14.7949f,0.6875f,14.7949f,0.7917f,14.7461f,0.7917f,14.7461f,0.8958f,14.6973f,0.8958f,14.6973f,1.0000f,14.6485f,1.0000f,16.9922f,-4.0000f,16.9434f,-4.0000f,16.9434f,-3.8958f,16.8946f,-3.8958f,16.8946f,-3.7917f,16.8457f,-3.7917f,16.8457f,-3.6875f,16.7969f,-3.6875f,
	16.7969f,-3.5833f,16.7481f,-3.5833f,16.7481f,-3.4792f,16.6993f,-3.4792f,16.6993f,-3.3750f,16.6504f,-3.3750f,16.6504f,-3.2708f,16.6016f,-3.2708f,16.6016f,-3.1667f,16.5528f,-3.1667f,16.5528f,-3.0625f,16.5039f,-3.0625f,16.5039f,-2.9583f,16.4551f,-2.9583f,16.4551f,-2.8542f,
	16.4063f,-2.8542f,16.4063f,-2.7500f,16.3575f,-2.7500f,16.3575f,-2.6458f,16.3086f,-2.6458f,16.3086f,-2.5417f,16.2598f,-2.5417f,16.2598f,-2.4375f,16.2110f,-2.4375f,16.2110f,-2.3333f,16.1621f,-2.3333f,16.1621f,-2.2292f,16.1133f,-2.2292f,16.1133f,-2.1250f,16.0645f,-2.1250f,
	16.0645f,-2.0208f,16.0157f,-2.0208f,16.0157f,-1.9167f,15.9668f,-1.9167f,15.9668f,-1.8125f,15.9180f,-1.8125f,15.9180f,-1.7083f,15.8692f,-1.7083f,15.8692f,-1.6042f,15.8203f,-1.6042f,15.8203f,-1.5000f,15.7715f,-1.5000f,15.7715f,-1.3958f,15.7227f,-1.3958f,15.7227f,-1.2917f,
	15.6739f,-1.2917f,15.6739f,-1.1875f,15.6250f,-1.1875f,15.6250f,-1.0833f,15.5762f,-1.0833f,15.5762f,-0.9792f,15.5274f,-0.9792f,15.5274f,-0.8750f,15.4785f,-0.8750f,15.4785f,-0.7708f,15.4297f,-0.7708f,15.4297f,-0.6667f,15.3809f,-0.6667f,15.3809f,-0.5625f,15.3321f,-0.5625f,
	15.3321f,-0.4583f,15.2832f,-0.4583f,15.2832f,-0.3542f,15.2344f,-0.3542f,15.2344f,-0.2500f,15.1856f,-0.2500f,15.1856f,-0.1458f,15.1367f,-0.1458f,15.1367f,-0.0417f,15.0879f,-0.0417f,15.0879f,0.0625f,15.0391f,0.0625f,15.0391f,0.1667f,14.9903f,0.1667f,14.9903f,0.2708f,
	14.9414f,0.2708f,14.9414f,0.3750f,14.8926f,0.3750f,14.8926f,0.4792f,14.8438f,0.4792f,14.8438f,0.5833f,14.7949f,0.5833f,14.7949f,0.6875f,14.7461f,0.6875f,14.7461f,0.7917f,14.6973f,0.7917f,14.6973f,0.8958f,14.6485f,0.8958f,14.6485f,1.0000f,14.5996f,1.0000f,
	16.9434f,-4.0000f,16.8946f,-4.0000f,16.8946f,-3.8958f,16.8457f,-3.8958f,16.8457f,-3.7917f,16.7969f,-3.7917f,16.7969f,-3.6875f,16.7481f,-3.6875f,16.7481f,-3.5833f,16.6993f,-3.5833f,16.6993f,-3.4792f,16.6504f,-3.4792f,16.6504f,-3.3750f,16.6016f,-3.3750f,16.6016f,-3.2708f,
	16.5528f,-3.2708f,16.5528f,-3.1667f,16.5039f,-3.1667f,16.5039f,-3.0625f,16.4551f,-3.0625f,16.4551f,-2.9583f,16.4063f,-2.9583f,16.4063f,-2.8542f,16.3575f,-2.8542f,16.3575f,-2.7500f,16.3086f,-2.7500f,16.3086f,-2.6458f,16.2598f,-2.6458f,16.2598f,-2.5417f,16.2110f,-2.5417f,
	16.2110f,-2.4375f,16.1621f,-2.4375f,16.1621f,-2.3333f,16.1133f,-2.3333f,16.1133f,-2.2292f,16.0645f,-2.2292f,16.0645f,-2.1250f,16.0157f,-2.1250f,16.0157f,-2.0208f,15.9668f,-2.0208f,15.9668f,-1.9167f,15.9180f,-1.9167f,15.9180f,-1.8125f,15.8692f,-1.8125f,15.8692f,-1.7083f,
	15.8203f,-1.7083f,15.8203f,-1.6042f,15.7715f,-1.6042f,15.7715f,-1.5000f,15.7227f,-1.5000f,15.7227f,-1.3958f,15.6739f,-1.3958f,15.6739f,-1.2917f,15.6250f,-1.2917f,15.6250f,-1.1875f,15.5762f,-1.1875f,15.5762f,-1.0833f,15.5274f,-1.0833f,15.5274f,-0.9792f,15.4785f,-0.9792f,
	15.4785f,-0.8750f,15.4297f,-0.8750f,15.4297f,-0.7708f,15.3809f,-0.7708f,15.3809f,-0.6667f,15.3321f,-0.6667f,15.3321f,-0.5625f,15.2832f,-0.5625f,15.2832f,-0.4583f,15.2344f,-0.4583f,15.2344f,-0.3542f,15.1856f,-0.3542f,15.1856f,-0.2500f,15.1367f,-0.2500f,15.1367f,-0.1458f,
	15.0879f,-0.1458f,15.0879f,-0.0417f,15.0391f,-0.0417f,15.0391f,0.0625f,14.9903f,0.0625f,14.9903f,0.1667f,14.9414f,0.1667f,14.9414f,0.2708f,14.8926f,0.2708f,14.8926f,0.3750f,14.8438f,0.3750f,14.8438f,0.4792f,14.7949f,0.4792f,14.7949f,0.5833f,14.7461f,0.5833f,
	14.7461f,0.6875f,14.6973f,0.6875f,14.6973f,0.7917f,14.6485f,0.7917f,14.6485f,0.8958f,14.5996f,0.8958f,14.5996f,1.0000f,14.5508f,1.0000f,16.8946f,-4.0000f,16.8457f,-4.0000f,16.8457f,-3.8958f,16.7969f,-3.8958f,16.7969f,-3.7917f,16.7481f,-3.7917f,16.7481f,-3.6875f,
	16.6993f,-3.6875f,16.6993f,-3.5833f,16.6504f,-3.5833f,16.6504f,-3.4792f,16.6016f,-3.4792f,16.6016f,-3.3750f,16.5528f,-3.3750f,16.5528f,-3.2708f,16.5039f,-3.2708f,16.5039f,-3.1667f,16.4551f,-3.1667f,16.4551f,-3.0625f,16.4063f,-3.0625f,16.4063f,-2.9583f,16.3575f,-2.9583f,
	16.3575f,-2.8542f,16.3086f,-2.8542f,16.3086f,-2.7500f,16.2598f,-2.7500f,16.2598f,-2.6458f,16.2110f,-2.6458f,16.2110f,-2.5417f,16.1621f,-2.5417f,16.1621f,-2.4375f,16.1133f,-2.4375f,16.1133f,-2.3333f,16.0645f,-2.3333f,16.0645f,-2.2292f,16.0157f,-2.2292f,16.0157f,-2.1250f,
	15.9668f,-2.1250f,15.9668f,-2.0208f,15.9180f,-2.0208f,15.9180f,-1.9167f,15.8692f,-1.9167f,15.8692f,-1.8125f,15.8203f,-1.8125f,15.8203f,-1.7083f,15.7715f,-1.7083f,15.7715f,-1.6042f,15.7227f,-1.6042f,15.7227f,-1.5000f,15.6739f,-1.5000f,15.6739f,-1.3958f,15.6250f,-1.3958f,
	15.6250f,-1.2917f,15.5762f,-1.2917f,15.5762f,-1.1875f,15.5274f,-1.1875f,15.5274f,-1.0833f,15.4785f,-1.0833f,15.4785f,-0.9792f,15.4297f,-0.9792f,15.4297f,-0.8750f,15.3809f,-0.8750f,15.3809f,-0.7708f,15.3321f,-0.7708f,15.3321f,-0.6667f,15.2832f,-0.6667f,15.2832f,-0.5625f,
	15.2344f,-0.5625f,15.2344f,-0.4583f,15.1856f,-0.4583f,15.1856f,-0.3542f,15.1367f,-0.3542f,15.1367f,-0.2500f,15.0879f,-0.2500f,15.0879f,-0.1458f,15.0391f,-0.1458f,15.0391f,-0.0417f,14.9903f,-0.0417f,14.9903f,0.0625f,14.9414f,0.0625f,14.9414f,0.1667f,14.8926f,0.1667f,
	14.8926f,0.2708f,14.8438f,0.2708f,14.8438f,0.3750f,14.7949f,0.3750f,14.7949f,0.4792f,14.7461f,0.4792f,14.7461f,0.5833f,14.6973f,0.5833f,14.6973f,0.6875f,14.6485f,0.6875f,14.6485f,0.7917f,14.5996f,0.7917f,14.5996f,0.8958f,14.5508f,0.8958f,14.5508f,1.0000f,
	14.5020f,1.0000f,16.8457f,-4.0000f,16.7969f,-4.0000f,16.7969f,-3.8958f,16.7481f,-3.8958f,16.7481f,-3.7917f,16.6993f,-3.7917f,16.6993f,-3.6875f,16.6504f,-3.6875f,16.6504f,-3.5833f,16.6016f,-3.5833f,16.6016f,-3.4792f,16.5528f,-3.4792f,16.5528f,-3.3750f,16.5039f,-3.3750f,
	16.5039f,-3.2708f,16.4551f,-3.2708f,16.4551f,-3.1667f,16.4063f,-3.1667f,16.4063f,-3.0625f,16.3575f,-3.0625f,16.3575f,-2.9583f,16.3086f,-2.9583f,16.3086f,-2.8542f,16.2598f,-2.8542f,16.2598f,-2.7500f,16.2110f,-2.7500f,16.2110f,-2.6458f,16.1621f,-2.6458f,16.1621f,-2.5417f,
	16.1133f,-2.5417f,16.1133f,-2.4375f,16.0645f,-2.4375f,16.0645f,-2.3333f,16.0157f,-2.3333f,16.0157f,-2.2292f,15.9668f,-2.2292f,15.9668f,-2.1250f,15.9180f,-2.1250f,15.9180f,-2.0208f,15.8692f,-2.0208f,15.8692f,-1.9167f,15.8203f,-1.9167f,15.8203f,-1.8125f,15.7715f,-1.8125f,
	15.7715f,-1.7083f,15.7227f,-1.7083f,15.7227f,-1.6042f,15.6739f,-1.6042f,15.6739f,-1.5000f,15.6250f,-1.5000f,15.6250f,-1.3958f,15.5762f,-1.3958f,15.5762f,-1.2917f,15.5274f,-1.2917f,15.5274f,-1.1875f,15.4785f,-1.1875f,15.4785f,-1.0833f,15.4297f,-1.0833f,15.4297f,-0.9792f,
	15.3809f,-0.9792f,15.3809f,-0.8750f,15.3321f,-0.8750f,15.3321f,-0.7708f,15.2832f,-0.7708f,15.2832f,-0.6667f,15.2344f,-0.6667f,15.2344f,-0.5625f,15.1856f,-0.5625f,15.1856f,-0.4583f,15.1367f,-0.4583f,15.1367f,-0.3542f,15.0879f,-0.3542f,15.0879f,-0.2500f,15.0391f,-0.2500f,
	15.0391f,-0.1458f,14.9903f,-0.1458f,14.9903f,-0.0417f,14.9414f,-0.0417f,14.9414f,0.0625f,14.8926f,0.0625f,14.8926f,0.1667f,14.8438f,0.1667f,14.8438f,0.2708f,14.7949f,0.2708f,14.7949f,0.3750f,14.7461f,0.3750f,14.7461f,0.4792f,14.6973f,0.4792f,14.6973f,0.5833f,
	14.6485f,0.5833f,14.6485f,0.6875f,14.5996f,0.6875f,14.5996f,0.7917f,14.5508f,0.7917f,14.5508f,0.8958f,14.5020f,0.8958f,14.5020f,1.0000f,14.4531f,1.0000f,16.7969f,-4.0000f,16.7481f,-4.0000f,16.7481f,-3.8958f,16.6993f,-3.8958f,16.6993f,-3.7917f,16.6504f,-3.7917f,
	16.6504f,-3.6875f,16.6016f,-3.6875f,16.6016f,-3.5833f,16.5528f,-3.5833f,16.5528f,-3.4792f,16.5039f,-3.4792f,16.5039f,-3.3750f,16.4551f,-3.3750f,16.4551f,-3.2708f,16.4063f,-3.2708f,16.4063f,-3.1667f,16.3575f,-3.1667f,16.3575f,-3.0625f,16.3086f,-3.0625f,16.3086f,-2.9583f,
	16.2598f,-2.9583f,16.2598f,-2.8542f,16.2110f,-2.8542f,16.2110f,-2.7500f,16.1621f,-2.7500f,16.1621f,-2.6458f,16.1133f,-2.6458f,16.1133f,-2.5417f,16.0645f,-2.5417f,16.0645f,-2.4375f,16.0157f,-2.4375f,16.0157f,-2.3333f,15.9668f,-2.3333f,15.9668f,-2.2292f,15.9180f,-2.2292f,
	15.9180f,-2.1250f,15.8692f,-2.1250f,15.8692f,-2.0208f,15.8203f,-2.0208f,15.8203f,-1.9167f,15.7715f,-1.9167f,15.7715f,-1.8125f,15.7227f,-1.8125f,15.7227f,-1.7083f,15.6739f,-1.7083f,15.6739f,-1.6042f,15.6250f,-1.6042f,15.6250f,-1.5000f,15.5762f,-1.5000f,15.5762f,-1.3958f,
	15.5274f,-1.3958f,15.5274f,-1.2917f,15.4785f,-1.2917f,15.4785f,-1.1875f,15.4297f,-1.1875f,15.4297f,-1.0833f,15.3809f,-1.0833f,15.3809f,-0.9792f,15.3321f,-0.9792f,15.3321f,-0.8750f,15.2832f,-0.8750f,15.2832f,-0.7708f,15.2344f,-0.7708f,15.2344f,-0.6667f,15.1856f,-0.6667f,
	15.1856f,-0.5625f,15.1367f,-0.5625f,15.1367f,-0.4583f,15.0879f,-0.4583f,15.0879f,-0.3542f,15.0391f,-0.3542f,15.0391f,-0.2500f,14.9903f,-0.2500f,14.9903f,-0.1458f,14.9414f,-0.1458f,14.9414f,-0.0417f,14.8926f,-0.0417f,14.8926f,0.0625f,14.8438f,0.0625f,14.8438f,0.1667f,
	14.7949f,0.1667f,14.7949f,0.2708f,14.7461f,0.2708f,14.7461f,0.3750f,14.6973f,0.3750f,14.6973f,0.4792f,14.6485f,0.4792f,14.6485f,0.5833f,14.5996f,0.5833f,14.5996f,0.6875f,14.5508f,0.6875f,14.5508f,0.7917f,14.5020f,0.7917f,14.5020f,0.8958f,14.4531f,0.8958f,
	14.4531f,1.0000f,14.4043f,1.0000f,16.7481f,-4.0000f,16.6993f,-4.0000f,16.6993f,-3.8958f,16.6504f,-3.8958f,16.6504f,-3.7917f,16.6016f,-3.7917f,16.6016f,-3.6875f,16.5528f,-3.6875f,16.5528f,-3.5833f,16.5039f,-3.5833f,16.5039f,-3.4792f,16.4551f,-3.4792f,16.4551f,-3.3750f,
	16.4063f,-3.3750f,16.4063f,-3.2708f,16.3575f,-3.2708f,16.3575f,-3.1667f,16.3086f,-3.1667f,16.3086f,-3.0625f,16.2598f,-3.0625f,16.2598f,-2.9583f,16.2110f,-2.9583f,16.2110f,-2.8542f,16.1621f,-2.8542f,16.1621f,-2.7500f,16.1133f,-2.7500f,16.1133f,-2.6458f,16.0645f,-2.6458f,
	16.0645f,-2.5417f,16.0157f,-2.5417f,16.0157f,-2.4375f,15.9668f,-2.4375f,15.9668f,-2.3333f,15.9180f,-2.3333f,15.9180f,-2.2292f,15.8692f,-2.2292f,15.8692f,-2.1250f,15.8203f,-2.1250f,15.8203f,-2.0208f,15.7715f,-2.0208f,15.7715f,-1.9167f,15.7227f,-1.9167f,15.7227f,-1.8125f,
	15.6739f,-1.8125f,15.6739f,-1.7083f,15.6250f,-1.7083f,15.6250f,-1.6042f,15.5762f,-1.6042f,15.5762f,-1.5000f,15.5274f,-1.5000f,15.5274f,-1.3958f,15.4785f,-1.3958f,15.4785f,-1.2917f,15.4297f,-1.2917f,15.4297f,-1.1875f,15.3809f,-1.1875f,15.3809f,-1.0833f,15.3321f,-1.0833f,
	15.3321f,-0.9792f,15.2832f,-0.9792f,15.2832f,-0.8750f,15.2344f,-0.8750f,15.2344f,-0.7708f,15.1856f,-0.7708f,15.1856f,-0.6667f,15.1367f,-0.6667f,15.1367f,-0.5625f,15.0879f,-0.5625f,15.0879f,-0.4583f,15.0391f,-0.4583f,15.0391f,-0.3542f,14.9903f,-0.3542f,14.9903f,-0.2500f,
	14.9414f,-0.2500f,14.9414f,-0.1458f,14.8926f,-0.1458f,14.8926f,-0.0417f,14.8438f,-0.0417f,14.8438f,0.0625f,14.7949f,0.0625f,14.7949f,0.1667f,14.7461f,0.1667f,14.7461f,0.2708f,14.6973f,0.2708f,14.6973f,0.3750f,14.6485f,0.3750f,14.6485f,0.4792f,14.5996f,0.4792f,
	14.5996f,0.5833f,14.5508f,0.5833f,14.5508f,0.6875f,14.5020f,0.6875f,14.5020f,0.7917f,14.4531f,0.7917f,14.4531f,0.8958f,14.4043f,0.8958f,14.4043f,1.0000f,14.3555f,1.0000f,16.6993f,-4.0000f,16.6504f,-4.0000f,16.6504f,-3.8958f,16.6016f,-3.8958f,16.6016f,-3.7917f,
	16.5528f,-3.7917f,16.5528f,-3.6875f,16.5039f,-3.6875f,16.5039f,-3.5833f,16.4551f,-3.5833f,16.4551f,-3.4792f,16.4063f,-3.4792f,16.4063f,-3.3750f,16.3575f,-3.3750f,16.3575f,-3.2708f,16.3086f,-3.2708f,16.3086f,-3.1667f,16.2598f,-3.1667f,16.2598f,-3.0625f,16.2110f,-3.0625f,
	16.2110f,-2.9583f,16.1621f,-2.9583f,16.1621f,-2.8542f,16.1133f,-2.8542f,16.1133f,-2.7500f,16.0645f,-2.7500f,16.0645f,-2.6458f,16.0157f,-2.6458f,16.0157f,-2.5417f,15.9668f,-2.5417f,15.9668f,-2.4375f,15.9180f,-2.4375f,15.9180f,-2.3333f,15.8692f,-2.3333f,15.8692f,-2.2292f,
	15.8203f,-2.2292f,15.8203f,-2.1250f,15.7715f,-2.1250f,15.7715f,-2.0208f,15.7227f,-2.0208f,15.7227f,-1.9167f,15.6739f,-1.9167f,15.6739f,-1.8125f,15.6250f,-1.8125f,15.6250f,-1.7083f,15.5762f,-1.7083f,15.5762f,-1.6042f,15.5274f,-1.6042f,15.5274f,-1.5000f,15.4785f,-1.5000f,
	15.4785f,-1.3958f,15.4297f,-1.3958f,15.4297f,-1.2917f,15.3809f,-1.2917f,15.3809f,-1.1875f,15.3321f,-1.1875f,15.3321f,-1.0833f,15.2832f,-1.0833f,15.2832f,-0.9792f,15.2344f,-0.9792f,15.2344f,-0.8750f,15.1856f,-0.8750f,15.1856f,-0.7708f,15.1367f,-0.7708f,15.1367f,-0.6667f,
	15.0879f,-0.6667f,15.0879f,-0.5625f,15.0391f,-0.5625f,15.0391f,-0.4583f,14.9903f,-0.4583f,14.9903f,-0.3542f,14.9414f,-0.3542f,14.9414f,-0.2500f,14.8926f,-0.2500f,14.8926f,-0.1458f,14.8438f,-0.1458f,14.8438f,-0.0417f,14.7949f,-0.0417f,14.7949f,0.0625f,14.7461f,0.0625f,
	14.7461f,0.1667f,14.6973f,0.1667f,14.6973f,0.2708f,14.6485f,0.2708f,14.6485f,0.3750f,14.5996f,0.3750f,14.5996f,0.4792f,14.5508f,0.4792f,14.5508f,0.5833f,14.5020f,0.5833f,14.5020f,0.6875f,14.4531f,0.6875f,14.4531f,0.7917f,14.4043f,0.7917f,14.4043f,0.8958f,
	14.3555f,0.8958f,14.3555f,1.0000f,14.3067f,1.0000f,16.6504f,-4.0000f,16.6016f,-4.0000f,16.6016f,-3.8958f,16.5528f,-3.8958f,16.5528f,-3.7917f,16.5039f,-3.7917f,16.5039f,-3.6875f,16.4551f,-3.6875f,16.4551f,-3.5833f,16.4063f,-3.5833f,16.4063f,-3.4792f,16.3575f,-3.4792f,
	16.3575f,-3.3750f,16.3086f,-3.3750f,16.3086f,-3.2708f,16.2598f,-3.2708f,16.2598f,-3.1667f,16.2110f,-3.1667f,16.2110f,-3.0625f,16.1621f,-3.0625f,16.1621f,-2.9583f,16.1133f,-2.9583f,16.1133f,-2.8542f,16.0645f,-2.8542f,16.0645f,-2.7500f,16.0157f,-2.7500f,16.0157f,-2.6458f,
	15.9668f,-2.6458f,15.9668f,-2.5417f,15.9180f,-2.5417f,15.9180f,-2.4375f,15.8692f,-2.4375f,15.8692f,-2.3333f,15.8203f,-2.3333f,15.8203f,-2.2292f,15.7715f,-2.2292f,15.7715f,-2.1250f,15.7227f,-2.1250f,15.7227f,-2.0208f,15.6739f,-2.0208f,15.6739f,-1.9167f,15.6250f,-1.9167f,
	15.6250f,-1.8125f,15.5762f,-1.8125f,15.5762f,-1.7083f,15.5274f,-1.7083f,15.5274f,-1.6042f,15.4785f,-1.6042f,15.4785f,-1.5000f,15.4297f,-1.5000f,15.4297f,-1.3958f,15.3809f,-1.3958f,15.3809f,-1.2917f,15.3321f,-1.2917f,15.3321f,-1.1875f,15.2832f,-1.1875f,15.2832f,-1.0833f,
	15.2344f,-1.0833f,15.2344f,-0.9792f,15.1856f,-0.9792f,15.1856f,-0.8750f,15.1367f,-0.8750f,15.1367f,-0.7708f,15.0879f,-0.7708f,15.0879f,-0.6667f,15.0391f,-0.6667f,15.0391f,-0.5625f,14.9903f,-0.5625f,14.9903f,-0.4583f,14.9414f,-0.4583f,14.9414f,-0.3542f,14.8926f,-0.3542f,
	14.8926f,-0.2500f,14.8438f,-0.2500f,14.8438f,-0.1458f,14.7949f,-0.1458f,14.7949f,-0.0417f,14.7461f,-0.0417f,14.7461f,0.0625f,14.6973f,0.0625f,14.6973f,0.1667f,14.6485f,0.1667f,14.6485f,0.2708f,14.5996f,0.2708f,14.5996f,0.3750f,14.5508f,0.3750f,14.5508f,0.4792f,
	14.5020f,0.4792f,14.5020f,0.5833f,14.4531f,0.5833f,14.4531f,0.6875f,14.4043f,0.6875f,14.4043f,0.7917f,14.3555f,0.7917f,14.3555f,0.8958f,14.3067f,0.8958f,14.3067f,1.0000f,14.2578f,1.0000f,16.6016f,-4.0000f,16.5528f,-4.0000f,16.5528f,-3.8958f,16.5039f,-3.8958f,
	16.5039f,-3.7917f,16.4551f,-3.7917f,16.4551f,-3.6875f,16.4063f,-3.6875f,16.4063f,-3.5833f,16.3575f,-3.5833f,16.3575f,-3.4792f,16.3086f,-3.4792f,16.3086f,-3.3750f,16.2598f,-3.3750f,16.2598f,-3.2708f,16.2110f,-3.2708f,16.2110f,-3.1667f,16.1621f,-3.1667f,16.1621f,-3.0625f,
	16.1133f,-3.0625f,16.1133f,-2.9583f,16.0645f,-2.9583f,16.0645f,-2.8542f,16.0157f,-2.8542f,16.0157f,-2.7500f,15.9668f,-2.7500f,15.9668f,-2.6458f,15.9180f,-2.6458f,15.9180f,-2.5417f,15.8692f,-2.5417f,15.8692f,-2.4375f,15.8203f,-2.4375f,15.8203f,-2.3333f,15.7715f,-2.3333f,
	15.7715f,-2.2292f,15.7227f,-2.2292f,15.7227f,-2.1250f,15.6739f,-2.1250f,15.6739f,-2.0208f,15.6250f,-2.0208f,15.6250f,-1.9167f,15.5762f,-1.9167f,15.5762f,-1.8125f,15.5274f,-1.8125f,15.5274f,-1.7083f,15.4785f,-1.7083f,15.4785f,-1.6042f,15.4297f,-1.6042f,15.4297f,-1.5000f,
	15.3809f,-1.5000f,15.3809f,-1.3958f,15.3321f,-1.3958f,15.3321f,-1.2917f,15.2832f,-1.2917f,15.2832f,-1.1875f,15.2344f,-1.1875f,15.2344f,-1.0833f,15.1856f,-1.0833f,15.1856f,-0.9792f,15.1367f,-0.9792f,15.1367f,-0.8750f,15.0879f,-0.8750f,15.0879f,-0.7708f,15.0391f,-0.7708f,
	15.0391f,-0.6667f,14.9903f,-0.6667f,14.9903f,-0.5625f,14.9414f,-0.5625f,14.9414f,-0.4583f,14.8926f,-0.4583f,14.8926f,-0.3542f,14.8438f,-0.3542f,14.8438f,-0.2500f,14.7949f,-0.2500f,14.7949f,-0.1458f,14.7461f,-0.1458f,14.7461f,-0.0417f,14.6973f,-0.0417f,14.6973f,0.0625f,
	14.6485f,0.0625f,14.6485f,0.1667f,14.5996f,0.1667f,14.5996f,0.2708f,14.5508f,0.2708f,14.5508f,0.3750f,14.5020f,0.3750f,14.5020f,0.4792f,14.4531f,0.4792f,14.4531f,0.5833f,14.4043f,0.5833f,14.4043f,0.6875f,14.3555f,0.6875f,14.3555f,0.7917f,14.3067f,0.7917f,
	14.3067f,0.8958f,14.2578f,0.8958f,14.2578f,1.0000f,14.2090f,1.0000f,16.5528f,-4.0000f,16.5039f,-4.0000f,16.5039f,-3.8958f,16.4551f,-3.8958f,16.4551f,-3.7917f,16.4063f,-3.7917f,16.4063f,-3.6875f,16.3575f,-3.6875f,16.3575f,-3.5833f,16.3086f,-3.5833f,16.3086f,-3.4792f,
	16.2598f,-3.4792f,16.2598f,-3.3750f,16.2110f,-3.3750f,16.2110f,-3.2708f,16.1621f,-3.2708f,16.1621f,-3.1667f,16.1133f,-3.1667f,16.1133f,-3.0625f,16.0645f,-3.0625f,16.0645f,-2.9583f,16.0157f,-2.9583f,16.0157f,-2.8542f,15.9668f,-2.8542f,15.9668f,-2.7500f,15.9180f,-2.7500f,
	15.9180f,-2.6458f,15.8692f,-2.6458f,15.8692f,-2.5417f,15.8203f,-2.5417f,15.8203f,-2.4375f,15.7715f,-2.4375f,15.7715f,-2.3333f,15.7227f,-2.3333f,15.7227f,-2.2292f,15.6739f,-2.2292f,15.6739f,-2.1250f,15.6250f,-2.1250f,15.6250f,-2.0208f,15.5762f,-2.0208f,15.5762f,-1.9167f,
	15.5274f,-1.9167f,15.5274f,-1.8125f,15.4785f,-1.8125f,15.4785f,-1.7083f,15.4297f,-1.7083f,15.4297f,-1.6042f,15.3809f,-1.6042f,15.3809f,-1.5000f,15.3321f,-1.5000f,15.3321f,-1.3958f,15.2832f,-1.3958f,15.2832f,-1.2917f,15.2344f,-1.2917f,15.2344f,-1.1875f,15.1856f,-1.1875f,
	15.1856f,-1.0833f,15.1367f,-1.0833f,15.1367f,-0.9792f,15.0879f,-0.9792f,15.0879f,-0.8750f,15.0391f,-0.8750f,15.0391f,-0.7708f,14.9903f,-0.7708f,14.9903f,-0.6667f,14.9414f,-0.6667f,14.9414f,-0.5625f,14.8926f,-0.5625f,14.8926f,-0.4583f,14.8438f,-0.4583f,14.8438f,-0.3542f,
	14.7949f,-0.3542f,14.7949f,-0.2500f,14.7461f,-0.2500f,14.7461f,-0.1458f,14.6973f,-0.1458f,14.6973f,-0.0417f,14.6485f,-0.0417f,14.6485f,0.0625f,14.5996f,0.0625f,14.5996f,0.1667f,14.5508f,0.1667f,14.5508f,0.2708f,14.5020f,0.2708f,14.5020f,0.3750f,14.4531f,0.3750f,
	14.4531f,0.4792f,14.4043f,0.4792f,14.4043f,0.5833f,14.3555f,0.5833f,14.3555f,0.6875f,14.3067f,0.6875f,14.3067f,0.7917f,14.2578f,0.7917f,14.2578f,0.8958f,14.2090f,0.8958f,14.2090f,1.0000f,14.1602f,1.0000f,16.5039f,-4.0000f,16.4551f,-4.0000f,16.4551f,-3.8958f,
	16.4063f,-3.8958f,16.4063f,-3.7917f,16.3575f,-3.7917f,16.3575f,-3.6875f,16.3086f,-3.6875f,16.3086f,-3.5833f,16.2598f,-3.5833f,16.2598f,-3.4792f,16.2110f,-3.4792f,16.2110f,-3.3750f,16.1621f,-3.3750f,16.1621f,-3.2708f,16.1133f,-3.2708f,16.1133f,-3.1667f,16.0645f,-3.1667f,
	16.0645f,-3.0625f,16.0157f,-3.0625f,16.0157f,-2.9583f,15.9668f,-2.9583f,15.9668f,-2.8542f,15.9180f,-2.8542f,15.9180f,-2.7500f,15.8692f,-2.7500f,15.8692f,-2.6458f,15.8203f,-2.6458f,15.8203f,-2.5417f,15.7715f,-2.5417f,15.7715f,-2.4375f,15.7227f,-2.4375f,15.7227f,-2.3333f,
	15.6739f,-2.3333f,15.6739f,-2.2292f,15.6250f,-2.2292f,15.6250f,-2.1250f,15.5762f,-2.1250f,15.5762f,-2.0208f,15.5274f,-2.0208f,15.5274f,-1.9167f,15.4785f,-1.9167f,15.4785f,-1.8125f,15.4297f,-1.8125f,15.4297f,-1.7083f,15.3809f,-1.7083f,15.3809f,-1.6042f,15.3321f,-1.6042f,
	15.3321f,-1.5000f,15.2832f,-1.5000f,15.2832f,-1.3958f,15.2344f,-1.3958f,15.2344f,-1.2917f,15.1856f,-1.2917f,15.1856f,-1.1875f,15.1367f,-1.1875f,15.1367f,-1.0833f,15.0879f,-1.0833f,15.0879f,-0.9792f,15.0391f,-0.9792f,15.0391f,-0.8750f,14.9903f,-0.8750f,14.9903f,-0.7708f,
	14.9414f,-0.7708f,14.9414f,-0.6667f,14.8926f,-0.6667f,14.8926f,-0.5625f,14.8438f,-0.5625f,14.8438f,-0.4583f,14.7949f,-0.4583f,14.7949f,-0.3542f,14.7461f,-0.3542f,14.7461f,-0.2500f,14.6973f,-0.2500f,14.6973f,-0.1458f,14.6485f,-0.1458f,14.6485f,-0.0417f,14.5996f,-0.0417f,
	14.5996f,0.0625f,14.5508f,0.0625f,14.5508f,0.1667f,14.5020f,0.1667f,14.5020f,0.2708f,14.4531f,0.2708f,14.4531f,0.3750f,14.4043f,0.3750f,14.4043f,0.4792f,14.3555f,0.4792f,14.3555f,0.5833f,14.3067f,0.5833f,14.3067f,0.6875f,14.2578f,0.6875f,14.2578f,0.7917f,
	14.2090f,0.7917f,14.2090f,0.8958f,14.1602f,0.8958f,14.1602f,1.0000f,14.1114f,1.0000f,16.4551f,-4.0000f,16.4063f,-4.0000f,16.4063f,-3.8958f,16.3575f,-3.8958f,16.3575f,-3.7917f,16.3086f,-3.7917f,16.3086f,-3.6875f,16.2598f,-3.6875f,16.2598f,-3.5833f,16.2110f,-3.5833f,
	16.2110f,-3.4792f,16.1621f,-3.4792f,16.1621f,-3.3750f,16.1133f,-3.3750f,16.1133f,-3.2708f,16.0645f,-3.2708f,16.0645f,-3.1667f,16.0157f,-3.1667f,16.0157f,-3.0625f,15.9668f,-3.0625f,15.9668f,-2.9583f,15.9180f,-2.9583f,15.9180f,-2.8542f,15.8692f,-2.8542f,15.8692f,-2.7500f,
	15.8203f,-2.7500f,15.8203f,-2.6458f,15.7715f,-2.6458f,15.7715f,-2.5417f,15.7227f,-2.5417f,15.7227f,-2.4375f,15.6739f,-2.4375f,15.6739f,-2.3333f,15.6250f,-2.3333f,15.6250f,-2.2292f,15.5762f,-2.2292f,15.5762f,-2.1250f,15.5274f,-2.1250f,15.5274f,-2.0208f,15.4785f,-2.0208f,
	15.4785f,-1.9167f,15.4297f,-1.9167f,15.4297f,-1.8125f,15.3809f,-1.8125f,15.3809f,-1.7083f,15.3321f,-1.7083f,15.3321f,-1.6042f,15.2832f,-1.6042f,15.2832f,-1.5000f,15.2344f,-1.5000f,15.2344f,-1.3958f,15.1856f,-1.3958f,15.1856f,-1.2917f,15.1367f,-1.2917f,15.1367f,-1.1875f,
	15.0879f,-1.1875f,15.0879f,-1.0833f,15.0391f,-1.0833f,15.0391f,-0.9792f,14.9903f,-0.9792f,14.9903f,-0.8750f,14.9414f,-0.8750f,14.9414f,-0.7708f,14.8926f,-0.7708f,14.8926f,-0.6667f,14.8438f,-0.6667f,14.8438f,-0.5625f,14.7949f,-0.5625f,14.7949f,-0.4583f,14.7461f,-0.4583f,
	14.7461f,-0.3542f,14.6973f,-0.3542f,14.6973f,-0.2500f,14.6485f,-0.2500f,14.6485f,-0.1458f,14.5996f,-0.1458f,14.5996f,-0.0417f,14.5508f,-0.0417f,14.5508f,0.0625f,14.5020f,0.0625f,14.5020f,0.1667f,14.4531f,0.1667f,14.4531f,0.2708f,14.4043f,0.2708f,14.4043f,0.3750f,
	14.3555f,0.3750f,14.3555f,0.4792f,14.3067f,0.4792f,14.3067f,0.5833f,14.2578f,0.5833f,14.2578f,0.6875f,14.2090f,0.6875f,14.2090f,0.7917f,14.1602f,0.7917f,14.1602f,0.8958f,14.1114f,0.8958f,14.1114f,1.0000f,14.0625f,1.0000f,16.4063f,-4.0000f,16.3575f,-4.0000f,
	16.3575f,-3.8958f,16.3086f,-3.8958f,16.3086f,-3.7917f,16.2598f,-3.7917f,16.2598f,-3.6875f,16.2110f,-3.6875f,16.2110f,-3.5833f,16.1621f,-3.5833f,16.1621f,-3.4792f,16.1133f,-3.4792f,16.1133f,-3.3750f,16.0645f,-3.3750f,16.0645f,-3.2708f,16.0157f,-3.2708f,16.0157f,-3.1667f,
	15.9668f,-3.1667f,15.9668f,-3.0625f,15.9180f,-3.0625f,15.9180f,-2.9583f,15.8692f,-2.9583f,15.8692f,-2.8542f,15.8203f,-2.8542f,15.8203f,-2.7500f,15.7715f,-2.7500f,15.7715f,-2.6458f,15.7227f,-2.6458f,15.7227f,-2.5417f,15.6739f,-2.5417f,15.6739f,-2.4375f,15.6250f,-2.4375f,
	15.6250f,-2.3333f,15.5762f,-2.3333f,15.5762f,-2.2292f,15.5274f,-2.2292f,15.5274f,-2.1250f,15.4785f,-2.1250f,15.4785f,-2.0208f,15.4297f,-2.0208f,15.4297f,-1.9167f,15.3809f,-1.9167f,15.3809f,-1.8125f,15.3321f,-1.8125f,15.3321f,-1.7083f,15.2832f,-1.7083f,15.2832f,-1.6042f,
	15.2344f,-1.6042f,15.2344f,-1.5000f,15.1856f,-1.5000f,15.1856f,-1.3958f,15.1367f,-1.3958f,15.1367f,-1.2917f,15.0879f,-1.2917f,15.0879f,-1.1875f,15.0391f,-1.1875f,15.0391f,-1.0833f,14.9903f,-1.0833f,14.9903f,-0.9792f,14.9414f,-0.9792f,14.9414f,-0.8750f,14.8926f,-0.8750f,
	14.8926f,-0.7708f,14.8438f,-0.7708f,14.8438f,-0.6667f,14.7949f,-0.6667f,14.7949f,-0.5625f,14.7461f,-0.5625f,14.7461f,-0.4583f,14.6973f,-0.4583f,14.6973f,-0.3542f,14.6485f,-0.3542f,14.6485f,-0.2500f,14.5996f,-0.2500f,14.5996f,-0.1458f,14.5508f,-0.1458f,14.5508f,-0.0417f,
	14.5020f,-0.0417f,14.5020f,0.0625f,14.4531f,0.0625f,14.4531f,0.1667f,14.4043f,0.1667f,14.4043f,0.2708f,14.3555f,0.2708f,14.3555f,0.3750f,14.3067f,0.3750f,14.3067f,0.4792f,14.2578f,0.4792f,14.2578f,0.5833f,14.2090f,0.5833f,14.2090f,0.6875f,14.1602f,0.6875f,
	14.1602f,0.7917f,14.1114f,0.7917f,14.1114f,0.8958f,14.0625f,0.8958f,14.0625f,1.0000f,14.0137f,1.0000f,16.3575f,-4.0000f,16.3086f,-4.0000f,16.3086f,-3.8958f,16.2598f,-3.8958f,16.2598f,-3.7917f,16.2110f,-3.7917f,16.2110f,-3.6875f,16.1621f,-3.6875f,16.1621f,-3.5833f,
	16.1133f,-3.5833f,16.1133f,-3.4792f,16.0645f,-3.4792f,16.0645f,-3.3750f,16.0157f,-3.3750f,16.0157f,-3.2708f,15.9668f,-3.2708f,15.9668f,-3.1667f,15.9180f,-3.1667f,15.9180f,-3.0625f,15.8692f,-3.0625f,15.8692f,-2.9583f,15.8203f,-2.9583f,15.8203f,-2.8542f,15.7715f,-2.8542f,
	15.7715f,-2.7500f,15.7227f,-2.7500f,15.7227f,-2.6458f,15.6739f,-2.6458f,15.6739f,-2.5417f,15.6250f,-2.5417f,15.6250f,-2.4375f,15.5762f,-2.4375f,15.5762f,-2.3333f,15.5274f,-2.3333f,15.5274f,-2.2292f,15.4785f,-2.2292f,15.4785f,-2.1250f,15.4297f,-2.1250f,15.4297f,-2.0208f,
	15.3809f,-2.0208f,15.3809f,-1.9167f,15.3321f,-1.9167f,15.3321f,-1.8125f,15.2832f,-1.8125f,15.2832f,-1.7083f,15.2344f,-1.7083f,15.2344f,-1.6042f,15.1856f,-1.6042f,15.1856f,-1.5000f,15.1367f,-1.5000f,15.1367f,-1.3958f,15.0879f,-1.3958f,15.0879f,-1.2917f,15.0391f,-1.2917f,
	15.0391f,-1.1875f,14.9903f,-1.1875f,14.9903f,-1.0833f,14.9414f,-1.0833f,14.9414f,-0.9792f,14.8926f,-0.9792f,14.8926f,-0.8750f,14.8438f,-0.8750f,14.8438f,-0.7708f,14.7949f,-0.7708f,14.7949f,-0.6667f,14.7461f,-0.6667f,14.7461f,-0.5625f,14.6973f,-0.5625f,14.6973f,-0.4583f,
	14.6485f,-0.4583f,14.6485f,-0.3542f,14.5996f,-0.3542f,14.5996f,-0.2500f,14.5508f,-0.2500f,14.5508f,-0.1458f,14.5020f,-0.1458f,14.5020f,-0.0417f,14.4531f,-0.0417f,14.4531f,0.0625f,14.4043f,0.0625f,14.4043f,0.1667f,14.3555f,0.1667f,14.3555f,0.2708f,14.3067f,0.2708f,
	14.3067f,0.3750f,14.2578f,0.3750f,14.2578f,0.4792f,14.2090f,0.4792f,14.2090f,0.5833f,14.1602f,0.5833f,14.1602f,0.6875f,14.1114f,0.6875f,14.1114f,0.7917f,14.0625f,0.7917f,14.0625f,0.8958f,14.0137f,0.8958f,14.0137f,1.0000f,13.9649f,1.0000f,16.3086f,-4.0000f,
	16.2598f,-4.0000f,16.2598f,-3.8958f,16.2110f,-3.8958f,16.2110f,-3.7917f,16.1621f,-3.7917f,16.1621f,-3.6875f,16.1133f,-3.6875f,16.1133f,-3.5833f,16.0645f,-3.5833f,16.0645f,-3.4792f,16.0157f,-3.4792f,16.0157f,-3.3750f,15.9668f,-3.3750f,15.9668f,-3.2708f,15.9180f,-3.2708f,
	15.9180f,-3.1667f,15.8692f,-3.1667f,15.8692f,-3.0625f,15.8203f,-3.0625f,15.8203f,-2.9583f,15.7715f,-2.9583f,15.7715f,-2.8542f,15.7227f,-2.8542f,15.7227f,-2.7500f,15.6739f,-2.7500f,15.6739f,-2.6458f,15.6250f,-2.6458f,15.6250f,-2.5417f,15.5762f,-2.5417f,15.5762f,-2.4375f,
	15.5274f,-2.4375f,15.5274f,-2.3333f,15.4785f,-2.3333f,15.4785f,-2.2292f,15.4297f,-2.2292f,15.4297f,-2.1250f,15.3809f,-2.1250f,15.3809f,-2.0208f,15.3321f,-2.0208f,15.3321f,-1.9167f,15.2832f,-1.9167f,15.2832f,-1.8125f,15.2344f,-1.8125f,15.2344f,-1.7083f,15.1856f,-1.7083f,
	15.1856f,-1.6042f,15.1367f,-1.6042f,15.1367f,-1.5000f,15.0879f,-1.5000f,15.0879f,-1.3958f,15.0391f,-1.3958f,15.0391f,-1.2917f,14.9903f,-1.2917f,14.9903f,-1.1875f,14.9414f,-1.1875f,14.9414f,-1.0833f,14.8926f,-1.0833f,14.8926f,-0.9792f,14.8438f,-0.9792f,14.8438f,-0.8750f,
	14.7949f,-0.8750f,14.7949f,-0.7708f,14.7461f,-0.7708f,14.7461f,-0.6667f,14.6973f,-0.6667f,14.6973f,-0.5625f,14.6485f,-0.5625f,14.6485f,-0.4583f,14.5996f,-0.4583f,14.5996f,-0.3542f,14.5508f,-0.3542f,14.5508f,-0.2500f,14.5020f,-0.2500f,14.5020f,-0.1458f,14.4531f,-0.1458f,
	14.4531f,-0.0417f,14.4043f,-0.0417f,14.4043f,0.0625f,14.3555f,0.0625f,14.3555f,0.1667f,14.3067f,0.1667f,14.3067f,0.2708f,14.2578f,0.2708f,14.2578f,0.3750f,14.2090f,0.3750f,14.2090f,0.4792f,14.1602f,0.4792f,14.1602f,0.5833f,14.1114f,0.5833f,14.1114f,0.6875f,
	14.0625f,0.6875f,14.0625f,0.7917f,14.0137f,0.7917f,14.0137f,0.8958f,13.9649f,0.8958f,13.9649f,1.0000f,13.9160f,1.0000f,16.2598f,-4.0000f,16.2110f,-4.0000f,16.2110f,-3.8958f,16.1621f,-3.8958f,16.1621f,-3.7917f,16.1133f,-3.7917f,16.1133f,-3.6875f,16.0645f,-3.6875f,
	16.0645f,-3.5833f,16.0157f,-3.5833f,16.0157f,-3.4792f,15.9668f,-3.4792f,15.9668f,-3.3750f,15.9180f,-3.3750f,15.9180f,-3.2708f,15.8692f,-3.2708f,15.8692f,-3.1667f,15.8203f,-3.1667f,15.8203f,-3.0625f,15.7715f,-3.0625f,15.7715f,-2.9583f,15.7227f,-2.9583f,15.7227f,-2.8542f,
	15.6739f,-2.8542f,15.6739f,-2.7500f,15.6250f,-2.7500f,15.6250f,-2.6458f,15.5762f,-2.6458f,15.5762f,-2.5417f,15.5274f,-2.5417f,15.5274f,-2.4375f,15.4785f,-2.4375f,15.4785f,-2.3333f,15.4297f,-2.3333f,15.4297f,-2.2292f,15.3809f,-2.2292f,15.3809f,-2.1250f,15.3321f,-2.1250f,
	15.3321f,-2.0208f,15.2832f,-2.0208f,15.2832f,-1.9167f,15.2344f,-1.9167f,15.2344f,-1.8125f,15.1856f,-1.8125f,15.1856f,-1.7083f,15.1367f,-1.7083f,15.1367f,-1.6042f,15.0879f,-1.6042f,15.0879f,-1.5000f,15.0391f,-1.5000f,15.0391f,-1.3958f,14.9903f,-1.3958f,14.9903f,-1.2917f,
	14.9414f,-1.2917f,14.9414f,-1.1875f,14.8926f,-1.1875f,14.8926f,-1.0833f,14.8438f,-1.0833f,14.8438f,-0.9792f,14.7949f,-0.9792f,14.7949f,-0.8750f,14.7461f,-0.8750f,14.7461f,-0.7708f,14.6973f,-0.7708f,14.6973f,-0.6667f,14.6485f,-0.6667f,14.6485f,-0.5625f,14.5996f,-0.5625f,
	14.5996f,-0.4583f,14.5508f,-0.4583f,14.5508f,-0.3542f,14.5020f,-0.3542f,14.5020f,-0.2500f,14.4531f,-0.2500f,14.4531f,-0.1458f,14.4043f,-0.1458f,14.4043f,-0.0417f,14.3555f,-0.0417f,14.3555f,0.0625f,14.3067f,0.0625f,14.3067f,0.1667f,14.2578f,0.1667f,14.2578f,0.2708f,
	14.2090f,0.2708f,14.2090f,0.3750f,14.1602f,0.3750f,14.1602f,0.4792f,14.1114f,0.4792f,14.1114f,0.5833f,14.0625f,0.5833f,14.0625f,0.6875f,14.0137f,0.6875f,14.0137f,0.7917f,13.9649f,0.7917f,13.9649f,0.8958f,13.9160f,0.8958f,13.9160f,1.0000f,13.8672f,1.0000f,
	16.2110f,-4.0000f,16.1621f,-4.0000f,16.1621f,-3.8958f,16.1133f,-3.8958f,16.1133f,-3.7917f,16.0645f,-3.7917f,16.0645f,-3.6875f,16.0157f,-3.6875f,16.0157f,-3.5833f,15.9668f,-3.5833f,15.9668f,-3.4792f,15.9180f,-3.4792f,15.9180f,-3.3750f,15.8692f,-3.3750f,15.8692f,-3.2708f,
	15.8203f,-3.2708f,15.8203f,-3.1667f,15.7715f,-3.1667f,15.7715f,-3.0625f,15.7227f,-3.0625f,15.7227f,-2.9583f,15.6739f,-2.9583f,15.6739f,-2.8542f,15.6250f,-2.8542f,15.6250f,-2.7500f,15.5762f,-2.7500f,15.5762f,-2.6458f,15.5274f,-2.6458f,15.5274f,-2.5417f,15.4785f,-2.5417f,
	15.4785f,-2.4375f,15.4297f,-2.4375f,15.4297f,-2.3333f,15.3809f,-2.3333f,15.3809f,-2.2292f,15.3321f,-2.2292f,15.3321f,-2.1250f,15.2832f,-2.1250f,15.2832f,-2.0208f,15.2344f,-2.0208f,15.2344f,-1.9167f,15.1856f,-1.9167f,15.1856f,-1.8125f,15.1367f,-1.8125f,15.1367f,-1.7083f,
	15.0879f,-1.7083f,15.0879f,-1.6042f,15.0391f,-1.6042f,15.0391f,-1.5000f,14.9903f,-1.5000f,14.9903f,-1.3958f,14.9414f,-1.3958f,14.9414f,-1.2917f,14.8926f,-1.2917f,14.8926f,-1.1875f,14.8438f,-1.1875f,14.8438f,-1.0833f,14.7949f,-1.0833f,14.7949f,-0.9792f,14.7461f,-0.9792f,
	14.7461f,-0.8750f,14.6973f,-0.8750f,14.6973f,-0.7708f,14.6485f,-0.7708f,14.6485f,-0.6667f,14.5996f,-0.6667f,14.5996f,-0.5625f,14.5508f,-0.5625f,14.5508f,-0.4583f,14.5020f,-0.4583f,14.5020f,-0.3542f,14.4531f,-0.3542f,14.4531f,-0.2500f,14.4043f,-0.2500f,14.4043f,-0.1458f,
	14.3555f,-0.1458f,14.3555f,-0.0417f,14.3067f,-0.0417f,14.3067f,0.0625f,14.2578f,0.0625f,14.2578f,0.1667f,14.2090f,0.1667f,14.2090f,0.2708f,14.1602f,0.2708f,14.1602f,0.3750f,14.1114f,0.3750f,14.1114f,0.4792f,14.0625f,0.4792f,14.0625f,0.5833f,14.0137f,0.5833f,
	14.0137f,0.6875f,13.9649f,0.6875f,13.9649f,0.7917f,13.9160f,0.7917f,13.9160f,0.8958f,13.8672f,0.8958f,13.8672f,1.0000f,13.8184f,1.0000f,16.1621f,-4.0000f,16.1133f,-4.0000f,16.1133f,-3.8958f,16.0645f,-3.8958f,16.0645f,-3.7917f,16.0157f,-3.7917f,16.0157f,-3.6875f,
	15.9668f,-3.6875f,15.9668f,-3.5833f,15.9180f,-3.5833f,15.9180f,-3.4792f,15.8692f,-3.4792f,15.8692f,-3.3750f,15.8203f,-3.3750f,15.8203f,-3.2708f,15.7715f,-3.2708f,15.7715f,-3.1667f,15.7227f,-3.1667f,15.7227f,-3.0625f,15.6739f,-3.0625f,15.6739f,-2.9583f,15.6250f,-2.9583f,
	15.6250f,-2.8542f,15.5762f,-2.8542f,15.5762f,-2.7500f,15.5274f,-2.7500f,15.5274f,-2.6458f,15.4785f,-2.6458f,15.4785f,-2.5417f,15.4297f,-2.5417f,15.4297f,-2.4375f,15.3809f,-2.4375f,15.3809f,-2.3333f,15.3321f,-2.3333f,15.3321f,-2.2292f,15.2832f,-2.2292f,15.2832f,-2.1250f,
	15.2344f,-2.1250f,15.2344f,-2.0208f,15.1856f,-2.0208f,15.1856f,-1.9167f,15.1367f,-1.9167f,15.1367f,-1.8125f,15.0879f,-1.8125f,15.0879f,-1.7083f,15.0391f,-1.7083f,15.0391f,-1.6042f,14.9903f,-1.6042f,14.9903f,-1.5000f,14.9414f,-1.5000f,14.9414f,-1.3958f,14.8926f,-1.3958f,
	14.8926f,-1.2917f,14.8438f,-1.2917f,14.8438f,-1.1875f,14.7949f,-1.1875f,14.7949f,-1.0833f,14.7461f,-1.0833f,14.7461f,-0.9792f,14.6973f,-0.9792f,14.6973f,-0.8750f,14.6485f,-0.8750f,14.6485f,-0.7708f,14.5996f,-0.7708f,14.5996f,-0.6667f,14.5508f,-0.6667f,14.5508f,-0.5625f,
	14.5020f,-0.5625f,14.5020f,-0.4583f,14.4531f,-0.4583f,14.4531f,-0.3542f,14.4043f,-0.3542f,14.4043f,-0.2500f,14.3555f,-0.2500f,14.3555f,-0.1458f,14.3067f,-0.1458f,14.3067f,-0.0417f,14.2578f,-0.0417f,14.2578f,0.0625f,14.2090f,0.0625f,14.2090f,0.1667f,14.1602f,0.1667f,
	14.1602f,0.2708f,14.1114f,0.2708f,14.1114f,0.3750f,14.0625f,0.3750f,14.0625f,0.4792f,14.0137f,0.4792f,14.0137f,0.5833f,13.9649f,0.5833f,13.9649f,0.6875f,13.9160f,0.6875f,13.9160f,0.7917f,13.8672f,0.7917f,13.8672f,0.8958f,13.8184f,0.8958f,13.8184f,1.0000f,
	13.7696f,1.0000f,16.1133f,-4.0000f,16.0645f,-4.0000f,16.0645f,-3.8958f,16.0157f,-3.8958f,16.0157f,-3.7917f,15.9668f,-3.7917f,15.9668f,-3.6875f,15.9180f,-3.6875f,15.9180f,-3.5833f,15.8692f,-3.5833f,15.8692f,-3.4792f,15.8203f,-3.4792f,15.8203f,-3.3750f,15.7715f,-3.3750f,
	15.7715f,-3.2708f,15.7227f,-3.2708f,15.7227f,-3.1667f,15.6739f,-3.1667f,15.6739f,-3.0625f,15.6250f,-3.0625f,15.6250f,-2.9583f,15.5762f,-2.9583f,15.5762f,-2.8542f,15.5274f,-2.8542f,15.5274f,-2.7500f,15.4785f,-2.7500f,15.4785f,-2.6458f,15.4297f,-2.6458f,15.4297f,-2.5417f,
	15.3809f,-2.5417f,15.3809f,-2.4375f,15.3321f,-2.4375f,15.3321f,-2.3333f,15.2832f,-2.3333f,15.2832f,-2.2292f,15.2344f,-2.2292f,15.2344f,-2.1250f,15.1856f,-2.1250f,15.1856f,-2.0208f,15.1367f,-2.0208f,15.1367f,-1.9167f,15.0879f,-1.9167f,15.0879f,-1.8125f,15.0391f,-1.8125f,
	15.0391f,-1.7083f,14.9903f,-1.7083f,14.9903f,-1.6042f,14.9414f,-1.6042f,14.9414f,-1.5000f,14.8926f,-1.5000f,14.8926f,-1.3958f,14.8438f,-1.3958f,14.8438f,-1.2917f,14.7949f,-1.2917f,14.7949f,-1.1875f,14.7461f,-1.1875f,14.7461f,-1.0833f,14.6973f,-1.0833f,14.6973f,-0.9792f,
	14.6485f,-0.9792f,14.6485f,-0.8750f,14.5996f,-0.8750f,14.5996f,-0.7708f,14.5508f,-0.7708f,14.5508f,-0.6667f,14.5020f,-0.6667f,14.5020f,-0.5625f,14.4531f,-0.5625f,14.4531f,-0.4583f,14.4043f,-0.4583f,14.4043f,-0.3542f,14.3555f,-0.3542f,14.3555f,-0.2500f,14.3067f,-0.2500f,
	14.3067f,-0.1458f,14.2578f,-0.1458f,14.2578f,-0.0417f,14.2090f,-0.0417f,14.2090f,0.0625f,14.1602f,0.0625f,14.1602f,0.1667f,14.1114f,0.1667f,14.1114f,0.2708f,14.0625f,0.2708f,14.0625f,0.3750f,14.0137f,0.3750f,14.0137f,0.4792f,13.9649f,0.4792f,13.9649f,0.5833f,
	13.9160f,0.5833f,13.9160f,0.6875f,13.8672f,0.6875f,13.8672f,0.7917f,13.8184f,0.7917f,13.8184f,0.8958f,13.7696f,0.8958f,13.7696f,1.0000f,13.7207f,1.0000f,16.0645f,-4.0000f,16.0157f,-4.0000f,16.0157f,-3.8958f,15.9668f,-3.8958f,15.9668f,-3.7917f,15.9180f,-3.7917f,
	15.9180f,-3.6875f,15.8692f,-3.6875f,15.8692f,-3.5833f,15.8203f,-3.5833f,15.8203f,-3.4792f,15.7715f,-3.4792f,15.7715f,-3.3750f,15.7227f,-3.3750f,15.7227f,-3.2708f,15.6739f,-3.2708f,15.6739f,-3.1667f,15.6250f,-3.1667f,15.6250f,-3.0625f,15.5762f,-3.0625f,15.5762f,-2.9583f,
	15.5274f,-2.9583f,15.5274f,-2.8542f,15.4785f,-2.8542f,15.4785f,-2.7500f,15.4297f,-2.7500f,15.4297f,-2.6458f,15.3809f,-2.6458f,15.3809f,-2.5417f,15.3321f,-2.5417f,15.3321f,-2.4375f,15.2832f,-2.4375f,15.2832f,-2.3333f,15.2344f,-2.3333f,15.2344f,-2.2292f,15.1856f,-2.2292f,
	15.1856f,-2.1250f,15.1367f,-2.1250f,15.1367f,-2.0208f,15.0879f,-2.0208f,15.0879f,-1.9167f,15.0391f,-1.9167f,15.0391f,-1.8125f,14.9903f,-1.8125f,14.9903f,-1.7083f,14.9414f,-1.7083f,14.9414f,-1.6042f,14.8926f,-1.6042f,14.8926f,-1.5000f,14.8438f,-1.5000f,14.8438f,-1.3958f,
	14.7949f,-1.3958f,14.7949f,-1.2917f,14.7461f,-1.2917f,14.7461f,-1.1875f,14.6973f,-1.1875f,14.6973f,-1.0833f,14.6485f,-1.0833f,14.6485f,-0.9792f,14.5996f,-0.9792f,14.5996f,-0.8750f,14.5508f,-0.8750f,14.5508f,-0.7708f,14.5020f,-0.7708f,14.5020f,-0.6667f,14.4531f,-0.6667f,
	14.4531f,-0.5625f,14.4043f,-0.5625f,14.4043f,-0.4583f,14.3555f,-0.4583f,14.3555f,-0.3542f,14.3067f,-0.3542f,14.3067f,-0.2500f,14.2578f,-0.2500f,14.2578f,-0.1458f,14.2090f,-0.1458f,14.2090f,-0.0417f,14.1602f,-0.0417f,14.1602f,0.0625f,14.1114f,0.0625f,14.1114f,0.1667f,
	14.0625f,0.1667f,14.0625f,0.2708f,14.0137f,0.2708f,14.0137f,0.3750f,13.9649f,0.3750f,13.9649f,0.4792f,13.9160f,0.4792f,13.9160f,0.5833f,13.8672f,0.5833f,13.8672f,0.6875f,13.8184f,0.6875f,13.8184f,0.7917f,13.7696f,0.7917f,13.7696f,0.8958f,13.7207f,0.8958f,
	13.7207f,1.0000f,13.6719f,1.0000f,16.0157f,-4.0000f,15.9668f,-4.0000f,15.9668f,-3.8958f,15.9180f,-3.8958f,15.9180f,-3.7917f,15.8692f,-3.7917f,15.8692f,-3.6875f,15.8203f,-3.6875f,15.8203f,-3.5833f,15.7715f,-3.5833f,15.7715f,-3.4792f,15.7227f,-3.4792f,15.7227f,-3.3750f,
	15.6739f,-3.3750f,15.6739f,-3.2708f,15.6250f,-3.2708f,15.6250f,-3.1667f,15.5762f,-3.1667f,15.5762f,-3.0625f,15.5274f,-3.0625f,15.5274f,-2.9583f,15.4785f,-2.9583f,15.4785f,-2.8542f,15.4297f,-2.8542f,15.4297f,-2.7500f,15.3809f,-2.7500f,15.3809f,-2.6458f,15.3321f,-2.6458f,
	15.3321f,-2.5417f,15.2832f,-2.5417f,15.2832f,-2.4375f,15.2344f,-2.4375f,15.2344f,-2.3333f,15.1856f,-2.3333f,15.1856f,-2.2292f,15.1367f,-2.2292f,15.1367f,-2.1250f,15.0879f,-2.1250f,15.0879f,-2.0208f,15.0391f,-2.0208f,15.0391f,-1.9167f,14.9903f,-1.9167f,14.9903f,-1.8125f,
	14.9414f,-1.8125f,14.9414f,-1.7083f,14.8926f,-1.7083f,14.8926f,-1.6042f,14.8438f,-1.6042f,14.8438f,-1.5000f,14.7949f,-1.5000f,14.7949f,-1.3958f,14.7461f,-1.3958f,14.7461f,-1.2917f,14.6973f,-1.2917f,14.6973f,-1.1875f,14.6485f,-1.1875f,14.6485f,-1.0833f,14.5996f,-1.0833f,
	14.5996f,-0.9792f,14.5508f,-0.9792f,14.5508f,-0.8750f,14.5020f,-0.8750f,14.5020f,-0.7708f,14.4531f,-0.7708f,14.4531f,-0.6667f,14.4043f,-0.6667f,14.4043f,-0.5625f,14.3555f,-0.5625f,14.3555f,-0.4583f,14.3067f,-0.4583f,14.3067f,-0.3542f,14.2578f,-0.3542f,14.2578f,-0.2500f,
	14.2090f,-0.2500f,14.2090f,-0.1458f,14.1602f,-0.1458f,14.1602f,-0.0417f,14.1114f,-0.0417f,14.1114f,0.0625f,14.0625f,0.0625f,14.0625f,0.1667f,14.0137f,0.1667f,14.0137f,0.2708f,13.9649f,0.2708f,13.9649f,0.3750f,13.9160f,0.3750f,13.9160f,0.4792f,13.8672f,0.4792f,
	13.8672f,0.5833f,13.8184f,0.5833f,13.8184f,0.6875f,13.7696f,0.6875f,13.7696f,0.7917f,13.7207f,0.7917f,13.7207f,0.8958f,13.6719f,0.8958f,13.6719f,1.0000f,13.6231f,1.0000f,15.9668f,-4.0000f,15.9180f,-4.0000f,15.9180f,-3.8958f,15.8692f,-3.8958f,15.8692f,-3.7917f,
	15.8203f,-3.7917f,15.8203f,-3.6875f,15.7715f,-3.6875f,15.7715f,-3.5833f,15.7227f,-3.5833f,15.7227f,-3.4792f,15.6739f,-3.4792f,15.6739f,-3.3750f,15.6250f,-3.3750f,15.6250f,-3.2708f,15.5762f,-3.2708f,15.5762f,-3.1667f,15.5274f,-3.1667f,15.5274f,-3.0625f,15.4785f,-3.0625f,
	15.4785f,-2.9583f,15.4297f,-2.9583f,15.4297f,-2.8542f,15.3809f,-2.8542f,15.3809f,-2.7500f,15.3321f,-2.7500f,15.3321f,-2.6458f,15.2832f,-2.6458f,15.2832f,-2.5417f,15.2344f,-2.5417f,15.2344f,-2.4375f,15.1856f,-2.4375f,15.1856f,-2.3333f,15.1367f,-2.3333f,15.1367f,-2.2292f,
	15.0879f,-2.2292f,15.0879f,-2.1250f,15.0391f,-2.1250f,15.0391f,-2.0208f,14.9903f,-2.0208f,14.9903f,-1.9167f,14.9414f,-1.9167f,14.9414f,-1.8125f,14.8926f,-1.8125f,14.8926f,-1.7083f,14.8438f,-1.7083f,14.8438f,-1.6042f,14.7949f,-1.6042f,14.7949f,-1.5000f,14.7461f,-1.5000f,
	14.7461f,-1.3958f,14.6973f,-1.3958f,14.6973f,-1.2917f,14.6485f,-1.2917f,14.6485f,-1.1875f,14.5996f,-1.1875f,14.5996f,-1.0833f,14.5508f,-1.0833f,14.5508f,-0.9792f,14.5020f,-0.9792f,14.5020f,-0.8750f,14.4531f,-0.8750f,14.4531f,-0.7708f,14.4043f,-0.7708f,14.4043f,-0.6667f,
	14.3555f,-0.6667f,14.3555f,-0.5625f,14.3067f,-0.5625f,14.3067f,-0.4583f,14.2578f,-0.4583f,14.2578f,-0.3542f,14.2090f,-0.3542f,14.2090f,-0.2500f,14.1602f,-0.2500f,14.1602f,-0.1458f,14.1114f,-0.1458f,14.1114f,-0.0417f,14.0625f,-0.0417f,14.0625f,0.0625f,14.0137f,0.0625f,
	14.0137f,0.1667f,13.9649f,0.1667f,13.9649f,0.2708f,13.9160f,0.2708f,13.9160f,0.3750f,13.8672f,0.3750f,13.8672f,0.4792f,13.8184f,0.4792f,13.8184f,0.5833f,13.7696f,0.5833f,13.7696f,0.6875f,13.7207f,0.6875f,13.7207f,0.7917f,13.6719f,0.7917f,13.6719f,0.8958f,
	13.6231f,0.8958f,13.6231f,1.0000f,13.5742f,1.0000f,15.9180f,-4.0000f,15.8692f,-4.0000f,15.8692f,-3.8958f,15.8203f,-3.8958f,15.8203f,-3.7917f,15.7715f,-3.7917f,15.7715f,-3.6875f,15.7227f,-3.6875f,15.7227f,-3.5833f,15.6739f,-3.5833f,15.6739f,-3.4792f,15.6250f,-3.4792f,
	15.6250f,-3.3750f,15.5762f,-3.3750f,15.5762f,-3.2708f,15.5274f,-3.2708f,15.5274f,-3.1667f,15.4785f,-3.1667f,15.4785f,-3.0625f,15.4297f,-3.0625f,15.4297f,-2.9583f,15.3809f,-2.9583f,15.3809f,-2.8542f,15.3321f,-2.8542f,15.3321f,-2.7500f,15.2832f,-2.7500f,15.2832f,-2.6458f,
	15.2344f,-2.6458f,15.2344f,-2.5417f,15.1856f,-2.5417f,15.1856f,-2.4375f,15.1367f,-2.4375f,15.1367f,-2.3333f,15.0879f,-2.3333f,15.0879f,-2.2292f,15.0391f,-2.2292f,15.0391f,-2.1250f,14.9903f,-2.1250f,14.9903f,-2.0208f,14.9414f,-2.0208f,14.9414f,-1.9167f,14.8926f,-1.9167f,
	14.8926f,-1.8125f,14.8438f,-1.8125f,14.8438f,-1.7083f,14.7949f,-1.7083f,14.7949f,-1.6042f,14.7461f,-1.6042f,14.7461f,-1.5000f,14.6973f,-1.5000f,14.6973f,-1.3958f,14.6485f,-1.3958f,14.6485f,-1.2917f,14.5996f,-1.2917f,14.5996f,-1.1875f,14.5508f,-1.1875f,14.5508f,-1.0833f,
	14.5020f,-1.0833f,14.5020f,-0.9792f,14.4531f,-0.9792f,14.4531f,-0.8750f,14.4043f,-0.8750f,14.4043f,-0.7708f,14.3555f,-0.7708f,14.3555f,-0.6667f,14.3067f,-0.6667f,14.3067f,-0.5625f,14.2578f,-0.5625f,14.2578f,-0.4583f,14.2090f,-0.4583f,14.2090f,-0.3542f,14.1602f,-0.3542f,
	14.1602f,-0.2500f,14.1114f,-0.2500f,14.1114f,-0.1458f,14.0625f,-0.1458f,14.0625f,-0.0417f,14.0137f,-0.0417f,14.0137f,0.0625f,13.9649f,0.0625f,13.9649f,0.1667f,13.9160f,0.1667f,13.9160f,0.2708f,13.8672f,0.2708f,13.8672f,0.3750f,13.8184f,0.3750f,13.8184f,0.4792f,
	13.7696f,0.4792f,13.7696f,0.5833f,13.7207f,0.5833f,13.7207f,0.6875f,13.6719f,0.6875f,13.6719f,0.7917f,13.6231f,0.7917f,13.6231f,0.8958f,13.5742f,0.8958f,13.5742f,1.0000f,13.5254f,1.0000f,15.8692f,-4.0000f,15.8203f,-4.0000f,15.8203f,-3.8958f,15.7715f,-3.8958f,
	15.7715f,-3.7917f,15.7227f,-3.7917f,15.7227f,-3.6875f,15.6739f,-3.6875f,15.6739f,-3.5833f,15.6250f,-3.5833f,15.6250f,-3.4792f,15.5762f,-3.4792f,15.5762f,-3.3750f,15.5274f,-3.3750f,15.5274f,-3.2708f,15.4785f,-3.2708f,15.4785f,-3.1667f,15.4297f,-3.1667f,15.4297f,-3.0625f,
	15.3809f,-3.0625f,15.3809f,-2.9583f,15.3321f,-2.9583f,15.3321f,-2.8542f,15.2832f,-2.8542f,15.2832f,-2.7500f,15.2344f,-2.7500f,15.2344f,-2.6458f,15.1856f,-2.6458f,15.1856f,-2.5417f,15.1367f,-2.5417f,15.1367f,-2.4375f,15.0879f,-2.4375f,15.0879f,-2.3333f,15.0391f,-2.3333f,
	15.0391f,-2.2292f,14.9903f,-2.2292f,14.9903f,-2.1250f,14.9414f,-2.1250f,14.9414f,-2.0208f,14.8926f,-2.0208f,14.8926f,-1.9167f,14.8438f,-1.9167f,14.8438f,-1.8125f,14.7949f,-1.8125f,14.7949f,-1.7083f,14.7461f,-1.7083f,14.7461f,-1.6042f,14.6973f,-1.6042f,14.6973f,-1.5000f,
	14.6485f,-1.5000f,14.6485f,-1.3958f,14.5996f,-1.3958f,14.5996f,-1.2917f,14.5508f,-1.2917f,14.5508f,-1.1875f,14.5020f,-1.1875f,14.5020f,-1.0833f,14.4531f,-1.0833f,14.4531f,-0.9792f,14.4043f,-0.9792f,14.4043f,-0.8750f,14.3555f,-0.8750f,14.3555f,-0.7708f,14.3067f,-0.7708f,
	14.3067f,-0.6667f,14.2578f,-0.6667f,14.2578f,-0.5625f,14.2090f,-0.5625f,14.2090f,-0.4583f,14.1602f,-0.4583f,14.1602f,-0.3542f,14.1114f,-0.3542f,14.1114f,-0.2500f,14.0625f,-0.2500f,14.0625f,-0.1458f,14.0137f,-0.1458f,14.0137f,-0.0417f,13.9649f,-0.0417f,13.9649f,0.0625f,
	13.9160f,0.0625f,13.9160f,0.1667f,13.8672f,0.1667f,13.8672f,0.2708f,13.8184f,0.2708f,13.8184f,0.3750f,13.7696f,0.3750f,13.7696f,0.4792f,13.7207f,0.4792f,13.7207f,0.5833f,13.6719f,0.5833f,13.6719f,0.6875f,13.6231f,0.6875f,13.6231f,0.7917f,13.5742f,0.7917f,
	13.5742f,0.8958f,13.5254f,0.8958f,13.5254f,1.0000f,13.4766f,1.0000f,15.8203f,-4.0000f,15.7715f,-4.0000f,15.7715f,-3.8958f,15.7227f,-3.8958f,15.7227f,-3.7917f,15.6739f,-3.7917f,15.6739f,-3.6875f,15.6250f,-3.6875f,15.6250f,-3.5833f,15.5762f,-3.5833f,15.5762f,-3.4792f,
	15.5274f,-3.4792f,15.5274f,-3.3750f,15.4785f,-3.3750f,15.4785f,-3.2708f,15.4297f,-3.2708f,15.4297f,-3.1667f,15.3809f,-3.1667f,15.3809f,-3.0625f,15.3321f,-3.0625f,15.3321f,-2.9583f,15.2832f,-2.9583f,15.2832f,-2.8542f,15.2344f,-2.8542f,15.2344f,-2.7500f,15.1856f,-2.7500f,
	15.1856f,-2.6458f,15.1367f,-2.6458f,15.1367f,-2.5417f,15.0879f,-2.5417f,15.0879f,-2.4375f,15.0391f,-2.4375f,15.0391f,-2.3333f,14.9903f,-2.3333f,14.9903f,-2.2292f,14.9414f,-2.2292f,14.9414f,-2.1250f,14.8926f,-2.1250f,14.8926f,-2.0208f,14.8438f,-2.0208f,14.8438f,-1.9167f,
	14.7949f,-1.9167f,14.7949f,-1.8125f,14.7461f,-1.8125f,14.7461f,-1.7083f,14.6973f,-1.7083f,14.6973f,-1.6042f,14.6485f,-1.6042f,14.6485f,-1.5000f,14.5996f,-1.5000f,14.5996f,-1.3958f,14.5508f,-1.3958f,14.5508f,-1.2917f,14.5020f,-1.2917f,14.5020f,-1.1875f,14.4531f,-1.1875f,
	14.4531f,-1.0833f,14.4043f,-1.0833f,14.4043f,-0.9792f,14.3555f,-0.9792f,14.3555f,-0.8750f,14.3067f,-0.8750f,14.3067f,-0.7708f,14.2578f,-0.7708f,14.2578f,-0.6667f,14.2090f,-0.6667f,14.2090f,-0.5625f,14.1602f,-0.5625f,14.1602f,-0.4583f,14.1114f,-0.4583f,14.1114f,-0.3542f,
	14.0625f,-0.3542f,14.0625f,-0.2500f,14.0137f,-0.2500f,14.0137f,-0.1458f,13.9649f,-0.1458f,13.9649f,-0.0417f,13.9160f,-0.0417f,13.9160f,0.0625f,13.8672f,0.0625f,13.8672f,0.1667f,13.8184f,0.1667f,13.8184f,0.2708f,13.7696f,0.2708f,13.7696f,0.3750f,13.7207f,0.3750f,
	13.7207f,0.4792f,13.6719f,0.4792f,13.6719f,0.5833f,13.6231f,0.5833f,13.6231f,0.6875f,13.5742f,0.6875f,13.5742f,0.7917f,13.5254f,0.7917f,13.5254f,0.8958f,13.4766f,0.8958f,13.4766f,1.0000f,13.4278f,1.0000f,15.7715f,-4.0000f,15.7227f,-4.0000f,15.7227f,-3.8958f,
	15.6739f,-3.8958f,15.6739f,-3.7917f,15.6250f,-3.7917f,15.6250f,-3.6875f,15.5762f,-3.6875f,15.5762f,-3.5833f,15.5274f,-3.5833f,15.5274f,-3.4792f,15.4785f,-3.4792f,15.4785f,-3.3750f,15.4297f,-3.3750f,15.4297f,-3.2708f,15.3809f,-3.2708f,15.3809f,-3.1667f,15.3321f,-3.1667f,
	15.3321f,-3.0625f,15.2832f,-3.0625f,15.2832f,-2.9583f,15.2344f,-2.9583f,15.2344f,-2.8542f,15.1856f,-2.8542f,15.1856f,-2.7500f,15.1367f,-2.7500f,15.1367f,-2.6458f,15.0879f,-2.6458f,15.0879f,-2.5417f,15.0391f,-2.5417f,15.0391f,-2.4375f,14.9903f,-2.4375f,14.9903f,-2.3333f,
	14.9414f,-2.3333f,14.9414f,-2.2292f,14.8926f,-2.2292f,14.8926f,-2.1250f,14.8438f,-2.1250f,14.8438f,-2.0208f,14.7949f,-2.0208f,14.7949f,-1.9167f,14.7461f,-1.9167f,14.7461f,-1.8125f,14.6973f,-1.8125f,14.6973f,-1.7083f,14.6485f,-1.7083f,14.6485f,-1.6042f,14.5996f,-1.6042f,
	14.5996f,-1.5000f,14.5508f,-1.5000f,14.5508f,-1.3958f,14.5020f,-1.3958f,14.5020f,-1.2917f,14.4531f,-1.2917f,14.4531f,-1.1875f,14.4043f,-1.1875f,14.4043f,-1.0833f,14.3555f,-1.0833f,14.3555f,-0.9792f,14.3067f,-0.9792f,14.3067f,-0.8750f,14.2578f,-0.8750f,14.2578f,-0.7708f,
	14.2090f,-0.7708f,14.2090f,-0.6667f,14.1602f,-0.6667f,14.1602f,-0.5625f,14.1114f,-0.5625f,14.1114f,-0.4583f,14.0625f,-0.4583f,14.0625f,-0.3542f,14.0137f,-0.3542f,14.0137f,-0.2500f,13.9649f,-0.2500f,13.9649f,-0.1458f,13.9160f,-0.1458f,13.9160f,-0.0417f,13.8672f,-0.0417f,
	13.8672f,0.0625f,13.8184f,0.0625f,13.8184f,0.1667f,13.7696f,0.1667f,13.7696f,0.2708f,13.7207f,0.2708f,13.7207f,0.3750f,13.6719f,0.3750f,13.6719f,0.4792f,13.6231f,0.4792f,13.6231f,0.5833f,13.5742f,0.5833f,13.5742f,0.6875f,13.5254f,0.6875f,13.5254f,0.7917f,
	13.4766f,0.7917f,13.4766f,0.8958f,13.4278f,0.8958f,13.4278f,1.0000f,13.3789f,1.0000f,15.7227f,-4.0000f,15.6739f,-4.0000f,15.6739f,-3.8958f,15.6250f,-3.8958f,15.6250f,-3.7917f,15.5762f,-3.7917f,15.5762f,-3.6875f,15.5274f,-3.6875f,15.5274f,-3.5833f,15.4785f,-3.5833f,
	15.4785f,-3.4792f,15.4297f,-3.4792f,15.4297f,-3.3750f,15.3809f,-3.3750f,15.3809f,-3.2708f,15.3321f,-3.2708f,15.3321f,-3.1667f,15.2832f,-3.1667f,15.2832f,-3.0625f,15.2344f,-3.0625f,15.2344f,-2.9583f,15.1856f,-2.9583f,15.1856f,-2.8542f,15.1367f,-2.8542f,15.1367f,-2.7500f,
	15.0879f,-2.7500f,15.0879f,-2.6458f,15.0391f,-2.6458f,15.0391f,-2.5417f,14.9903f,-2.5417f,14.9903f,-2.4375f,14.9414f,-2.4375f,14.9414f,-2.3333f,14.8926f,-2.3333f,14.8926f,-2.2292f,14.8438f,-2.2292f,14.8438f,-2.1250f,14.7949f,-2.1250f,14.7949f,-2.0208f,14.7461f,-2.0208f,
	14.7461f,-1.9167f,14.6973f,-1.9167f,14.6973f,-1.8125f,14.6485f,-1.8125f,14.6485f,-1.7083f,14.5996f,-1.7083f,14.5996f,-1.6042f,14.5508f,-1.6042f,14.5508f,-1.5000f,14.5020f,-1.5000f,14.5020f,-1.3958f,14.4531f,-1.3958f,14.4531f,-1.2917f,14.4043f,-1.2917f,14.4043f,-1.1875f,
	14.3555f,-1.1875f,14.3555f,-1.0833f,14.3067f,-1.0833f,14.3067f,-0.9792f,14.2578f,-0.9792f,14.2578f,-0.8750f,14.2090f,-0.8750f,14.2090f,-0.7708f,14.1602f,-0.7708f,14.1602f,-0.6667f,14.1114f,-0.6667f,14.1114f,-0.5625f,14.0625f,-0.5625f,14.0625f,-0.4583f,14.0137f,-0.4583f,
	14.0137f,-0.3542f,13.9649f,-0.3542f,13.9649f,-0.2500f,13.9160f,-0.2500f,13.9160f,-0.1458f,13.8672f,-0.1458f,13.8672f,-0.0417f,13.8184f,-0.0417f,13.8184f,0.0625f,13.7696f,0.0625f,13.7696f,0.1667f,13.7207f,0.1667f,13.7207f,0.2708f,13.6719f,0.2708f,13.6719f,0.3750f,
	13.6231f,0.3750f,13.6231f,0.4792f,13.5742f,0.4792f,13.5742f,0.5833f,13.5254f,0.5833f,13.5254f,0.6875f,13.4766f,0.6875f,13.4766f,0.7917f,13.4278f,0.7917f,13.4278f,0.8958f,13.3789f,0.8958f,13.3789f,1.0000f,13.3301f,1.0000f,15.6739f,-4.0000f,15.6250f,-4.0000f,
	15.6250f,-3.8958f,15.5762f,-3.8958f,15.5762f,-3.7917f,15.5274f,-3.7917f,15.5274f,-3.6875f,15.4785f,-3.6875f,15.4785f,-3.5833f,15.4297f,-3.5833f,15.4297f,-3.4792f,15.3809f,-3.4792f,15.3809f,-3.3750f,15.3321f,-3.3750f,15.3321f,-3.2708f,15.2832f,-3.2708f,15.2832f,-3.1667f,
	15.2344f,-3.1667f,15.2344f,-3.0625f,15.1856f,-3.0625f,15.1856f,-2.9583f,15.1367f,-2.9583f,15.1367f,-2.8542f,15.0879f,-2.8542f,15.0879f,-2.7500f,15.0391f,-2.7500f,15.0391f,-2.6458f,14.9903f,-2.6458f,14.9903f,-2.5417f,14.9414f,-2.5417f,14.9414f,-2.4375f,14.8926f,-2.4375f,
	14.8926f,-2.3333f,14.8438f,-2.3333f,14.8438f,-2.2292f,14.7949f,-2.2292f,14.7949f,-2.1250f,14.7461f,-2.1250f,14.7461f,-2.0208f,14.6973f,-2.0208f,14.6973f,-1.9167f,14.6485f,-1.9167f,14.6485f,-1.8125f,14.5996f,-1.8125f,14.5996f,-1.7083f,14.5508f,-1.7083f,14.5508f,-1.6042f,
	14.5020f,-1.6042f,14.5020f,-1.5000f,14.4531f,-1.5000f,14.4531f,-1.3958f,14.4043f,-1.3958f,14.4043f,-1.2917f,14.3555f,-1.2917f,14.3555f,-1.1875f,14.3067f,-1.1875f,14.3067f,-1.0833f,14.2578f,-1.0833f,14.2578f,-0.9792f,14.2090f,-0.9792f,14.2090f,-0.8750f,14.1602f,-0.8750f,
	14.1602f,-0.7708f,14.1114f,-0.7708f,14.1114f,-0.6667f,14.0625f,-0.6667f,14.0625f,-0.5625f,14.0137f,-0.5625f,14.0137f,-0.4583f,13.9649f,-0.4583f,13.9649f,-0.3542f,13.9160f,-0.3542f,13.9160f,-0.2500f,13.8672f,-0.2500f,13.8672f,-0.1458f,13.8184f,-0.1458f,13.8184f,-0.0417f,
	13.7696f,-0.0417f,13.7696f,0.0625f,13.7207f,0.0625f,13.7207f,0.1667f,13.6719f,0.1667f,13.6719f,0.2708f,13.6231f,0.2708f,13.6231f,0.3750f,13.5742f,0.3750f,13.5742f,0.4792f,13.5254f,0.4792f,13.5254f,0.5833f,13.4766f,0.5833f,13.4766f,0.6875f,13.4278f,0.6875f,
	13.4278f,0.7917f,13.3789f,0.7917f,13.3789f,0.8958f,13.3301f,0.8958f,13.3301f,1.0000f,13.2813f,1.0000f,15.6250f,-4.0000f,15.5762f,-4.0000f,15.5762f,-3.8958f,15.5274f,-3.8958f,15.5274f,-3.7917f,15.4785f,-3.7917f,15.4785f,-3.6875f,15.4297f,-3.6875f,15.4297f,-3.5833f,
	15.3809f,-3.5833f,15.3809f,-3.4792f,15.3321f,-3.4792f,15.3321f,-3.3750f,15.2832f,-3.3750f,15.2832f,-3.2708f,15.2344f,-3.2708f,15.2344f,-3.1667f,15.1856f,-3.1667f,15.1856f,-3.0625f,15.1367f,-3.0625f,15.1367f,-2.9583f,15.0879f,-2.9583f,15.0879f,-2.8542f,15.0391f,-2.8542f,
	15.0391f,-2.7500f,14.9903f,-2.7500f,14.9903f,-2.6458f,14.9414f,-2.6458f,14.9414f,-2.5417f,14.8926f,-2.5417f,14.8926f,-2.4375f,14.8438f,-2.4375f,14.8438f,-2.3333f,14.7949f,-2.3333f,14.7949f,-2.2292f,14.7461f,-2.2292f,14.7461f,-2.1250f,14.6973f,-2.1250f,14.6973f,-2.0208f,
	14.6485f,-2.0208f,14.6485f,-1.9167f,14.5996f,-1.9167f,14.5996f,-1.8125f,14.5508f,-1.8125f,14.5508f,-1.7083f,14.5020f,-1.7083f,14.5020f,-1.6042f,14.4531f,-1.6042f,14.4531f,-1.5000f,14.4043f,-1.5000f,14.4043f,-1.3958f,14.3555f,-1.3958f,14.3555f,-1.2917f,14.3067f,-1.2917f,
	14.3067f,-1.1875f,14.2578f,-1.1875f,14.2578f,-1.0833f,14.2090f,-1.0833f,14.2090f,-0.9792f,14.1602f,-0.9792f,14.1602f,-0.8750f,14.1114f,-0.8750f,14.1114f,-0.7708f,14.0625f,-0.7708f,14.0625f,-0.6667f,14.0137f,-0.6667f,14.0137f,-0.5625f,13.9649f,-0.5625f,13.9649f,-0.4583f,
	13.9160f,-0.4583f,13.9160f,-0.3542f,13.8672f,-0.3542f,13.8672f,-0.2500f,13.8184f,-0.2500f,13.8184f,-0.1458f,13.7696f,-0.1458f,13.7696f,-0.0417f,13.7207f,-0.0417f,13.7207f,0.0625f,13.6719f,0.0625f,13.6719f,0.1667f,13.6231f,0.1667f,13.6231f,0.2708f,13.5742f,0.2708f,
	13.5742f,0.3750f,13.5254f,0.3750f,13.5254f,0.4792f,13.4766f,0.4792f,13.4766f,0.5833f,13.4278f,0.5833f,13.4278f,0.6875f,13.3789f,0.6875f,13.3789f,0.7917f,13.3301f,0.7917f,13.3301f,0.8958f,13.2813f,0.8958f,13.2813f,1.0000f,13.2324f,1.0000f,15.5762f,-4.0000f,
	15.5274f,-4.0000f,15.5274f,-3.8958f,15.4785f,-3.8958f,15.4785f,-3.7917f,15.4297f,-3.7917f,15.4297f,-3.6875f,15.3809f,-3.6875f,15.3809f,-3.5833f,15.3321f,-3.5833f,15.3321f,-3.4792f,15.2832f,-3.4792f,15.2832f,-3.3750f,15.2344f,-3.3750f,15.2344f,-3.2708f,15.1856f,-3.2708f,
	15.1856f,-3.1667f,15.1367f,-3.1667f,15.1367f,-3.0625f,15.0879f,-3.0625f,15.0879f,-2.9583f,15.0391f,-2.9583f,15.0391f,-2.8542f,14.9903f,-2.8542f,14.9903f,-2.7500f,14.9414f,-2.7500f,14.9414f,-2.6458f,14.8926f,-2.6458f,14.8926f,-2.5417f,14.8438f,-2.5417f,14.8438f,-2.4375f,
	14.7949f,-2.4375f,14.7949f,-2.3333f,14.7461f,-2.3333f,14.7461f,-2.2292f,14.6973f,-2.2292f,14.6973f,-2.1250f,14.6485f,-2.1250f,14.6485f,-2.0208f,14.5996f,-2.0208f,14.5996f,-1.9167f,14.5508f,-1.9167f,14.5508f,-1.8125f,14.5020f,-1.8125f,14.5020f,-1.7083f,14.4531f,-1.7083f,
	14.4531f,-1.6042f,14.4043f,-1.6042f,14.4043f,-1.5000f,14.3555f,-1.5000f,14.3555f,-1.3958f,14.3067f,-1.3958f,14.3067f,-1.2917f,14.2578f,-1.2917f,14.2578f,-1.1875f,14.2090f,-1.1875f,14.2090f,-1.0833f,14.1602f,-1.0833f,14.1602f,-0.9792f,14.1114f,-0.9792f,14.1114f,-0.8750f,
	14.0625f,-0.8750f,14.0625f,-0.7708f,14.0137f,-0.7708f,14.0137f,-0.6667f,13.9649f,-0.6667f,13.9649f,-0.5625f,13.9160f,-0.5625f,13.9160f,-0.4583f,13.8672f,-0.4583f,13.8672f,-0.3542f,13.8184f,-0.3542f,13.8184f,-0.2500f,13.7696f,-0.2500f,13.7696f,-0.1458f,13.7207f,-0.1458f,
	13.7207f,-0.0417f,13.6719f,-0.0417f,13.6719f,0.0625f,13.6231f,0.0625f,13.6231f,0.1667f,13.5742f,0.1667f,13.5742f,0.2708f,13.5254f,0.2708f,13.5254f,0.3750f,13.4766f,0.3750f,13.4766f,0.4792f,13.4278f,0.4792f,13.4278f,0.5833f,13.3789f,0.5833f,13.3789f,0.6875f,
	13.3301f,0.6875f,13.3301f,0.7917f,13.2813f,0.7917f,13.2813f,0.8958f,13.2324f,0.8958f,13.2324f,1.0000f,13.1836f,1.0000f,15.5274f,-4.0000f,15.4785f,-4.0000f,15.4785f,-3.8958f,15.4297f,-3.8958f,15.4297f,-3.7917f,15.3809f,-3.7917f,15.3809f,-3.6875f,15.3321f,-3.6875f,
	15.3321f,-3.5833f,15.2832f,-3.5833f,15.2832f,-3.4792f,15.2344f,-3.4792f,15.2344f,-3.3750f,15.1856f,-3.3750f,15.1856f,-3.2708f,15.1367f,-3.2708f,15.1367f,-3.1667f,15.0879f,-3.1667f,15.0879f,-3.0625f,15.0391f,-3.0625f,15.0391f,-2.9583f,14.9903f,-2.9583f,14.9903f,-2.8542f,
	14.9414f,-2.8542f,14.9414f,-2.7500f,14.8926f,-2.7500f,14.8926f,-2.6458f,14.8438f,-2.6458f,14.8438f,-2.5417f,14.7949f,-2.5417f,14.7949f,-2.4375f,14.7461f,-2.4375f,14.7461f,-2.3333f,14.6973f,-2.3333f,14.6973f,-2.2292f,14.6485f,-2.2292f,14.6485f,-2.1250f,14.5996f,-2.1250f,
	14.5996f,-2.0208f,14.5508f,-2.0208f,14.5508f,-1.9167f,14.5020f,-1.9167f,14.5020f,-1.8125f,14.4531f,-1.8125f,14.4531f,-1.7083f,14.4043f,-1.7083f,14.4043f,-1.6042f,14.3555f,-1.6042f,14.3555f,-1.5000f,14.3067f,-1.5000f,14.3067f,-1.3958f,14.2578f,-1.3958f,14.2578f,-1.2917f,
	14.2090f,-1.2917f,14.2090f,-1.1875f,14.1602f,-1.1875f,14.1602f,-1.0833f,14.1114f,-1.0833f,14.1114f,-0.9792f,14.0625f,-0.9792f,14.0625f,-0.8750f,14.0137f,-0.8750f,14.0137f,-0.7708f,13.9649f,-0.7708f,13.9649f,-0.6667f,13.9160f,-0.6667f,13.9160f,-0.5625f,13.8672f,-0.5625f,
	13.8672f,-0.4583f,13.8184f,-0.4583f,13.8184f,-0.3542f,13.7696f,-0.3542f,13.7696f,-0.2500f,13.7207f,-0.2500f,13.7207f,-0.1458f,13.6719f,-0.1458f,13.6719f,-0.0417f,13.6231f,-0.0417f,13.6231f,0.0625f,13.5742f,0.0625f,13.5742f,0.1667f,13.5254f,0.1667f,13.5254f,0.2708f,
	13.4766f,0.2708f,13.4766f,0.3750f,13.4278f,0.3750f,13.4278f,0.4792f,13.3789f,0.4792f,13.3789f,0.5833f,13.3301f,0.5833f,13.3301f,0.6875f,13.2813f,0.6875f,13.2813f,0.7917f,13.2324f,0.7917f,13.2324f,0.8958f,13.1836f,0.8958f,13.1836f,1.0000f,13.1348f,1.0000f,
	15.4785f,-4.0000f,15.4297f,-4.0000f,15.4297f,-3.8958f,15.3809f,-3.8958f,15.3809f,-3.7917f,15.3321f,-3.7917f,15.3321f,-3.6875f,15.2832f,-3.6875f,15.2832f,-3.5833f,15.2344f,-3.5833f,15.2344f,-3.4792f,15.1856f,-3.4792f,15.1856f,-3.3750f,15.1367f,-3.3750f,15.1367f,-3.2708f,
	15.0879f,-3.2708f,15.0879f,-3.1667f,15.0391f,-3.1667f,15.0391f,-3.0625f,14.9903f,-3.0625f,14.9903f,-2.9583f,14.9414f,-2.9583f,14.9414f,-2.8542f,14.8926f,-2.8542f,14.8926f,-2.7500f,14.8438f,-2.7500f,14.8438f,-2.6458f,14.7949f,-2.6458f,14.7949f,-2.5417f,14.7461f,-2.5417f,
	14.7461f,-2.4375f,14.6973f,-2.4375f,14.6973f,-2.3333f,14.6485f,-2.3333f,14.6485f,-2.2292f,14.5996f,-2.2292f,14.5996f,-2.1250f,14.5508f,-2.1250f,14.5508f,-2.0208f,14.5020f,-2.0208f,14.5020f,-1.9167f,14.4531f,-1.9167f,14.4531f,-1.8125f,14.4043f,-1.8125f,14.4043f,-1.7083f,
	14.3555f,-1.7083f,14.3555f,-1.6042f,14.3067f,-1.6042f,14.3067f,-1.5000f,14.2578f,-1.5000f,14.2578f,-1.3958f,14.2090f,-1.3958f,14.2090f,-1.2917f,14.1602f,-1.2917f,14.1602f,-1.1875f,14.1114f,-1.1875f,14.1114f,-1.0833f,14.0625f,-1.0833f,14.0625f,-0.9792f,14.0137f,-0.9792f,
	14.0137f,-0.8750f,13.9649f,-0.8750f,13.9649f,-0.7708f,13.9160f,-0.7708f,13.9160f,-0.6667f,13.8672f,-0.6667f,13.8672f,-0.5625f,13.8184f,-0.5625f,13.8184f,-0.4583f,13.7696f,-0.4583f,13.7696f,-0.3542f,13.7207f,-0.3542f,13.7207f,-0.2500f,13.6719f,-0.2500f,13.6719f,-0.1458f,
	13.6231f,-0.1458f,13.6231f,-0.0417f,13.5742f,-0.0417f,13.5742f,0.0625f,13.5254f,0.0625f,13.5254f,0.1667f,13.4766f,0.1667f,13.4766f,0.2708f,13.4278f,0.2708f,13.4278f,0.3750f,13.3789f,0.3750f,13.3789f,0.4792f,13.3301f,0.4792f,13.3301f,0.5833f,13.2813f,0.5833f,
	13.2813f,0.6875f,13.2324f,0.6875f,13.2324f,0.7917f,13.1836f,0.7917f,13.1836f,0.8958f,13.1348f,0.8958f,13.1348f,1.0000f,13.0860f,1.0000f,15.4297f,-4.0000f,15.3809f,-4.0000f,15.3809f,-3.8958f,15.3321f,-3.8958f,15.3321f,-3.7917f,15.2832f,-3.7917f,15.2832f,-3.6875f,
	15.2344f,-3.6875f,15.2344f,-3.5833f,15.1856f,-3.5833f,15.1856f,-3.4792f,15.1367f,-3.4792f,15.1367f,-3.3750f,15.0879f,-3.3750f,15.0879f,-3.2708f,15.0391f,-3.2708f,15.0391f,-3.1667f,14.9903f,-3.1667f,14.9903f,-3.0625f,14.9414f,-3.0625f,14.9414f,-2.9583f,14.8926f,-2.9583f,
	14.8926f,-2.8542f,14.8438f,-2.8542f,14.8438f,-2.7500f,14.7949f,-2.7500f,14.7949f,-2.6458f,14.7461f,-2.6458f,14.7461f,-2.5417f,14.6973f,-2.5417f,14.6973f,-2.4375f,14.6485f,-2.4375f,14.6485f,-2.3333f,14.5996f,-2.3333f,14.5996f,-2.2292f,14.5508f,-2.2292f,14.5508f,-2.1250f,
	14.5020f,-2.1250f,14.5020f,-2.0208f,14.4531f,-2.0208f,14.4531f,-1.9167f,14.4043f,-1.9167f,14.4043f,-1.8125f,14.3555f,-1.8125f,14.3555f,-1.7083f,14.3067f,-1.7083f,14.3067f,-1.6042f,14.2578f,-1.6042f,14.2578f,-1.5000f,14.2090f,-1.5000f,14.2090f,-1.3958f,14.1602f,-1.3958f,
	14.1602f,-1.2917f,14.1114f,-1.2917f,14.1114f,-1.1875f,14.0625f,-1.1875f,14.0625f,-1.0833f,14.0137f,-1.0833f,14.0137f,-0.9792f,13.9649f,-0.9792f,13.9649f,-0.8750f,13.9160f,-0.8750f,13.9160f,-0.7708f,13.8672f,-0.7708f,13.8672f,-0.6667f,13.8184f,-0.6667f,13.8184f,-0.5625f,
	13.7696f,-0.5625f,13.7696f,-0.4583f,13.7207f,-0.4583f,13.7207f,-0.3542f,13.6719f,-0.3542f,13.6719f,-0.2500f,13.6231f,-0.2500f,13.6231f,-0.1458f,13.5742f,-0.1458f,13.5742f,-0.0417f,13.5254f,-0.0417f,13.5254f,0.0625f,13.4766f,0.0625f,13.4766f,0.1667f,13.4278f,0.1667f,
	13.4278f,0.2708f,13.3789f,0.2708f,13.3789f,0.3750f,13.3301f,0.3750f,13.3301f,0.4792f,13.2813f,0.4792f,13.2813f,0.5833f,13.2324f,0.5833f,13.2324f,0.6875f,13.1836f,0.6875f,13.1836f,0.7917f,13.1348f,0.7917f,13.1348f,0.8958f,13.0860f,0.8958f,13.0860f,1.0000f,
	13.0371f,1.0000f,15.3809f,-4.0000f,15.3321f,-4.0000f,15.3321f,-3.8958f,15.2832f,-3.8958f,15.2832f,-3.7917f,15.2344f,-3.7917f,15.2344f,-3.6875f,15.1856f,-3.6875f,15.1856f,-3.5833f,15.1367f,-3.5833f,15.1367f,-3.4792f,15.0879f,-3.4792f,15.0879f,-3.3750f,15.0391f,-3.3750f,
	15.0391f,-3.2708f,14.9903f,-3.2708f,14.9903f,-3.1667f,14.9414f,-3.1667f,14.9414f,-3.0625f,14.8926f,-3.0625f,14.8926f,-2.9583f,14.8438f,-2.9583f,14.8438f,-2.8542f,14.7949f,-2.8542f,14.7949f,-2.7500f,14.7461f,-2.7500f,14.7461f,-2.6458f,14.6973f,-2.6458f,14.6973f,-2.5417f,
	14.6485f,-2.5417f,14.6485f,-2.4375f,14.5996f,-2.4375f,14.5996f,-2.3333f,14.5508f,-2.3333f,14.5508f,-2.2292f,14.5020f,-2.2292f,14.5020f,-2.1250f,14.4531f,-2.1250f,14.4531f,-2.0208f,14.4043f,-2.0208f,14.4043f,-1.9167f,14.3555f,-1.9167f,14.3555f,-1.8125f,14.3067f,-1.8125f,
	14.3067f,-1.7083f,14.2578f,-1.7083f,14.2578f,-1.6042f,14.2090f,-1.6042f,14.2090f,-1.5000f,14.1602f,-1.5000f,14.1602f,-1.3958f,14.1114f,-1.3958f,14.1114f,-1.2917f,14.0625f,-1.2917f,14.0625f,-1.1875f,14.0137f,-1.1875f,14.0137f,-1.0833f,13.9649f,-1.0833f,13.9649f,-0.9792f,
	13.9160f,-0.9792f,13.9160f,-0.8750f,13.8672f,-0.8750f,13.8672f,-0.7708f,13.8184f,-0.7708f,13.8184f,-0.6667f,13.7696f,-0.6667f,13.7696f,-0.5625f,13.7207f,-0.5625f,13.7207f,-0.4583f,13.6719f,-0.4583f,13.6719f,-0.3542f,13.6231f,-0.3542f,13.6231f,-0.2500f,13.5742f,-0.2500f,
	13.5742f,-0.1458f,13.5254f,-0.1458f,13.5254f,-0.0417f,13.4766f,-0.0417f,13.4766f,0.0625f,13.4278f,0.0625f,13.4278f,0.1667f,13.3789f,0.1667f,13.3789f,0.2708f,13.3301f,0.2708f,13.3301f,0.3750f,13.2813f,0.3750f,13.2813f,0.4792f,13.2324f,0.4792f,13.2324f,0.5833f,
	13.1836f,0.5833f,13.1836f,0.6875f,13.1348f,0.6875f,13.1348f,0.7917f,13.0860f,0.7917f,13.0860f,0.8958f,13.0371f,0.8958f,13.0371f,1.0000f,12.9883f,1.0000f,15.3321f,-4.0000f,15.2832f,-4.0000f,15.2832f,-3.8958f,15.2344f,-3.8958f,15.2344f,-3.7917f,15.1856f,-3.7917f,
	15.1856f,-3.6875f,15.1367f,-3.6875f,15.1367f,-3.5833f,15.0879f,-3.5833f,15.0879f,-3.4792f,15.0391f,-3.4792f,15.0391f,-3.3750f,14.9903f,-3.3750f,14.9903f,-3.2708f,14.9414f,-3.2708f,14.9414f,-3.1667f,14.8926f,-3.1667f,14.8926f,-3.0625f,14.8438f,-3.0625f,14.8438f,-2.9583f,
	14.7949f,-2.9583f,14.7949f,-2.8542f,14.7461f,-2.8542f,14.7461f,-2.7500f,14.6973f,-2.7500f,14.6973f,-2.6458f,14.6485f,-2.6458f,14.6485f,-2.5417f,14.5996f,-2.5417f,14.5996f,-2.4375f,14.5508f,-2.4375f,14.5508f,-2.3333f,14.5020f,-2.3333f,14.5020f,-2.2292f,14.4531f,-2.2292f,
	14.4531f,-2.1250f,14.4043f,-2.1250f,14.4043f,-2.0208f,14.3555f,-2.0208f,14.3555f,-1.9167f,14.3067f,-1.9167f,14.3067f,-1.8125f,14.2578f,-1.8125f,14.2578f,-1.7083f,14.2090f,-1.7083f,14.2090f,-1.6042f,14.1602f,-1.6042f,14.1602f,-1.5000f,14.1114f,-1.5000f,14.1114f,-1.3958f,
	14.0625f,-1.3958f,14.0625f,-1.2917f,14.0137f,-1.2917f,14.0137f,-1.1875f,13.9649f,-1.1875f,13.9649f,-1.0833f,13.9160f,-1.0833f,13.9160f,-0.9792f,13.8672f,-0.9792f,13.8672f,-0.8750f,13.8184f,-0.8750f,13.8184f,-0.7708f,13.7696f,-0.7708f,13.7696f,-0.6667f,13.7207f,-0.6667f,
	13.7207f,-0.5625f,13.6719f,-0.5625f,13.6719f,-0.4583f,13.6231f,-0.4583f,13.6231f,-0.3542f,13.5742f,-0.3542f,13.5742f,-0.2500f,13.5254f,-0.2500f,13.5254f,-0.1458f,13.4766f,-0.1458f,13.4766f,-0.0417f,13.4278f,-0.0417f,13.4278f,0.0625f,13.3789f,0.0625f,13.3789f,0.1667f,
	13.3301f,0.1667f,13.3301f,0.2708f,13.2813f,0.2708f,13.2813f,0.3750f,13.2324f,0.3750f,13.2324f,0.4792f,13.1836f,0.4792f,13.1836f,0.5833f,13.1348f,0.5833f,13.1348f,0.6875f,13.0860f,0.6875f,13.0860f,0.7917f,13.0371f,0.7917f,13.0371f,0.8958f,12.9883f,0.8958f,
	12.9883f,1.0000f,12.9395f,1.0000f,15.2832f,-4.0000f,15.2344f,-4.0000f,15.2344f,-3.8958f,15.1856f,-3.8958f,15.1856f,-3.7917f,15.1367f,-3.7917f,15.1367f,-3.6875f,15.0879f,-3.6875f,15.0879f,-3.5833f,15.0391f,-3.5833f,15.0391f,-3.4792f,14.9903f,-3.4792f,14.9903f,-3.3750f,
	14.9414f,-3.3750f,14.9414f,-3.2708f,14.8926f,-3.2708f,14.8926f,-3.1667f,14.8438f,-3.1667f,14.8438f,-3.0625f,14.7949f,-3.0625f,14.7949f,-2.9583f,14.7461f,-2.9583f,14.7461f,-2.8542f,14.6973f,-2.8542f,14.6973f,-2.7500f,14.6485f,-2.7500f,14.6485f,-2.6458f,14.5996f,-2.6458f,
	14.5996f,-2.5417f,14.5508f,-2.5417f,14.5508f,-2.4375f,14.5020f,-2.4375f,14.5020f,-2.3333f,14.4531f,-2.3333f,14.4531f,-2.2292f,14.4043f,-2.2292f,14.4043f,-2.1250f,14.3555f,-2.1250f,14.3555f,-2.0208f,14.3067f,-2.0208f,14.3067f,-1.9167f,14.2578f,-1.9167f,14.2578f,-1.8125f,
	14.2090f,-1.8125f,14.2090f,-1.7083f,14.1602f,-1.7083f,14.1602f,-1.6042f,14.1114f,-1.6042f,14.1114f,-1.5000f,14.0625f,-1.5000f,14.0625f,-1.3958f,14.0137f,-1.3958f,14.0137f,-1.2917f,13.9649f,-1.2917f,13.9649f,-1.1875f,13.9160f,-1.1875f,13.9160f,-1.0833f,13.8672f,-1.0833f,
	13.8672f,-0.9792f,13.8184f,-0.9792f,13.8184f,-0.8750f,13.7696f,-0.8750f,13.7696f,-0.7708f,13.7207f,-0.7708f,13.7207f,-0.6667f,13.6719f,-0.6667f,13.6719f,-0.5625f,13.6231f,-0.5625f,13.6231f,-0.4583f,13.5742f,-0.4583f,13.5742f,-0.3542f,13.5254f,-0.3542f,13.5254f,-0.2500f,
	13.4766f,-0.2500f,13.4766f,-0.1458f,13.4278f,-0.1458f,13.4278f,-0.0417f,13.3789f,-0.0417f,13.3789f,0.0625f,13.3301f,0.0625f,13.3301f,0.1667f,13.2813f,0.1667f,13.2813f,0.2708f,13.2324f,0.2708f,13.2324f,0.3750f,13.1836f,0.3750f,13.1836f,0.4792f,13.1348f,0.4792f,
	13.1348f,0.5833f,13.0860f,0.5833f,13.0860f,0.6875f,13.0371f,0.6875f,13.0371f,0.7917f,12.9883f,0.7917f,12.9883f,0.8958f,12.9395f,0.8958f,12.9395f,1.0000f,12.8906f,1.0000f,15.2344f,-4.0000f,15.1856f,-4.0000f,15.1856f,-3.8958f,15.1367f,-3.8958f,15.1367f,-3.7917f,
	15.0879f,-3.7917f,15.0879f,-3.6875f,15.0391f,-3.6875f,15.0391f,-3.5833f,14.9903f,-3.5833f,14.9903f,-3.4792f,14.9414f,-3.4792f,14.9414f,-3.3750f,14.8926f,-3.3750f,14.8926f,-3.2708f,14.8438f,-3.2708f,14.8438f,-3.1667f,14.7949f,-3.1667f,14.7949f,-3.0625f,14.7461f,-3.0625f,
	14.7461f,-2.9583f,14.6973f,-2.9583f,14.6973f,-2.8542f,14.6485f,-2.8542f,14.6485f,-2.7500f,14.5996f,-2.7500f,14.5996f,-2.6458f,14.5508f,-2.6458f,14.5508f,-2.5417f,14.5020f,-2.5417f,14.5020f,-2.4375f,14.4531f,-2.4375f,14.4531f,-2.3333f,14.4043f,-2.3333f,14.4043f,-2.2292f,
	14.3555f,-2.2292f,14.3555f,-2.1250f,14.3067f,-2.1250f,14.3067f,-2.0208f,14.2578f,-2.0208f,14.2578f,-1.9167f,14.2090f,-1.9167f,14.2090f,-1.8125f,14.1602f,-1.8125f,14.1602f,-1.7083f,14.1114f,-1.7083f,14.1114f,-1.6042f,14.0625f,-1.6042f,14.0625f,-1.5000f,14.0137f,-1.5000f,
	14.0137f,-1.3958f,13.9649f,-1.3958f,13.9649f,-1.2917f,13.9160f,-1.2917f,13.9160f,-1.1875f,13.8672f,-1.1875f,13.8672f,-1.0833f,13.8184f,-1.0833f,13.8184f,-0.9792f,13.7696f,-0.9792f,13.7696f,-0.8750f,13.7207f,-0.8750f,13.7207f,-0.7708f,13.6719f,-0.7708f,13.6719f,-0.6667f,
	13.6231f,-0.6667f,13.6231f,-0.5625f,13.5742f,-0.5625f,13.5742f,-0.4583f,13.5254f,-0.4583f,13.5254f,-0.3542f,13.4766f,-0.3542f,13.4766f,-0.2500f,13.4278f,-0.2500f,13.4278f,-0.1458f,13.3789f,-0.1458f,13.3789f,-0.0417f,13.3301f,-0.0417f,13.3301f,0.0625f,13.2813f,0.0625f,
	13.2813f,0.1667f,13.2324f,0.1667f,13.2324f,0.2708f,13.1836f,0.2708f,13.1836f,0.3750f,13.1348f,0.3750f,13.1348f,0.4792f,13.0860f,0.4792f,13.0860f,0.5833f,13.0371f,0.5833f,13.0371f,0.6875f,12.9883f,0.6875f,12.9883f,0.7917f,12.9395f,0.7917f,12.9395f,0.8958f,
	12.8906f,0.8958f,12.8906f,1.0000f,12.8418f,1.0000f,15.1856f,-4.0000f,15.1367f,-4.0000f,15.1367f,-3.8958f,15.0879f,-3.8958f,15.0879f,-3.7917f,15.0391f,-3.7917f,15.0391f,-3.6875f,14.9903f,-3.6875f,14.9903f,-3.5833f,14.9414f,-3.5833f,14.9414f,-3.4792f,14.8926f,-3.4792f,
	14.8926f,-3.3750f,14.8438f,-3.3750f,14.8438f,-3.2708f,14.7949f,-3.2708f,14.7949f,-3.1667f,14.7461f,-3.1667f,14.7461f,-3.0625f,14.6973f,-3.0625f,14.6973f,-2.9583f,14.6485f,-2.9583f,14.6485f,-2.8542f,14.5996f,-2.8542f,14.5996f,-2.7500f,14.5508f,-2.7500f,14.5508f,-2.6458f,
	14.5020f,-2.6458f,14.5020f,-2.5417f,14.4531f,-2.5417f,14.4531f,-2.4375f,14.4043f,-2.4375f,14.4043f,-2.3333f,14.3555f,-2.3333f,14.3555f,-2.2292f,14.3067f,-2.2292f,14.3067f,-2.1250f,14.2578f,-2.1250f,14.2578f,-2.0208f,14.2090f,-2.0208f,14.2090f,-1.9167f,14.1602f,-1.9167f,
	14.1602f,-1.8125f,14.1114f,-1.8125f,14.1114f,-1.7083f,14.0625f,-1.7083f,14.0625f,-1.6042f,14.0137f,-1.6042f,14.0137f,-1.5000f,13.9649f,-1.5000f,13.9649f,-1.3958f,13.9160f,-1.3958f,13.9160f,-1.2917f,13.8672f,-1.2917f,13.8672f,-1.1875f,13.8184f,-1.1875f,13.8184f,-1.0833f,
	13.7696f,-1.0833f,13.7696f,-0.9792f,13.7207f,-0.9792f,13.7207f,-0.8750f,13.6719f,-0.8750f,13.6719f,-0.7708f,13.6231f,-0.7708f,13.6231f,-0.6667f,13.5742f,-0.6667f,13.5742f,-0.5625f,13.5254f,-0.5625f,13.5254f,-0.4583f,13.4766f,-0.4583f,13.4766f,-0.3542f,13.4278f,-0.3542f,
	13.4278f,-0.2500f,13.3789f,-0.2500f,13.3789f,-0.1458f,13.3301f,-0.1458f,13.3301f,-0.0417f,13.2813f,-0.0417f,13.2813f,0.0625f,13.2324f,0.0625f,13.2324f,0.1667f,13.1836f,0.1667f,13.1836f,0.2708f,13.1348f,0.2708f,13.1348f,0.3750f,13.0860f,0.3750f,13.0860f,0.4792f,
	13.0371f,0.4792f,13.0371f,0.5833f,12.9883f,0.5833f,12.9883f,0.6875f,12.9395f,0.6875f,12.9395f,0.7917f,12.8906f,0.7917f,12.8906f,0.8958f,12.8418f,0.8958f,12.8418f,1.0000f,12.7930f,1.0000f,15.1367f,-4.0000f,15.0879f,-4.0000f,15.0879f,-3.8958f,15.0391f,-3.8958f,
	15.0391f,-3.7917f,14.9903f,-3.7917f,14.9903f,-3.6875f,14.9414f,-3.6875f,14.9414f,-3.5833f,14.8926f,-3.5833f,14.8926f,-3.4792f,14.8438f,-3.4792f,14.8438f,-3.3750f,14.7949f,-3.3750f,14.7949f,-3.2708f,14.7461f,-3.2708f,14.7461f,-3.1667f,14.6973f,-3.1667f,14.6973f,-3.0625f,
	14.6485f,-3.0625f,14.6485f,-2.9583f,14.5996f,-2.9583f,14.5996f,-2.8542f,14.5508f,-2.8542f,14.5508f,-2.7500f,14.5020f,-2.7500f,14.5020f,-2.6458f,14.4531f,-2.6458f,14.4531f,-2.5417f,14.4043f,-2.5417f,14.4043f,-2.4375f,14.3555f,-2.4375f,14.3555f,-2.3333f,14.3067f,-2.3333f,
	14.3067f,-2.2292f,14.2578f,-2.2292f,14.2578f,-2.1250f,14.2090f,-2.1250f,14.2090f,-2.0208f,14.1602f,-2.0208f,14.1602f,-1.9167f,14.1114f,-1.9167f,14.1114f,-1.8125f,14.0625f,-1.8125f,14.0625f,-1.7083f,14.0137f,-1.7083f,14.0137f,-1.6042f,13.9649f,-1.6042f,13.9649f,-1.5000f,
	13.9160f,-1.5000f,13.9160f,-1.3958f,13.8672f,-1.3958f,13.8672f,-1.2917f,13.8184f,-1.2917f,13.8184f,-1.1875f,13.7696f,-1.1875f,13.7696f,-1.0833f,13.7207f,-1.0833f,13.7207f,-0.9792f,13.6719f,-0.9792f,13.6719f,-0.8750f,13.6231f,-0.8750f,13.6231f,-0.7708f,13.5742f,-0.7708f,
	13.5742f,-0.6667f,13.5254f,-0.6667f,13.5254f,-0.5625f,13.4766f,-0.5625f,13.4766f,-0.4583f,13.4278f,-0.4583f,13.4278f,-0.3542f,13.3789f,-0.3542f,13.3789f,-0.2500f,13.3301f,-0.2500f,13.3301f,-0.1458f,13.2813f,-0.1458f,13.2813f,-0.0417f,13.2324f,-0.0417f,13.2324f,0.0625f,
	13.1836f,0.0625f,13.1836f,0.1667f,13.1348f,0.1667f,13.1348f,0.2708f,13.0860f,0.2708f,13.0860f,0.3750f,13.0371f,0.3750f,13.0371f,0.4792f,12.9883f,0.4792f,12.9883f,0.5833f,12.9395f,0.5833f,12.9395f,0.6875f,12.8906f,0.6875f,12.8906f,0.7917f,12.8418f,0.7917f,
	12.8418f,0.8958f,12.7930f,0.8958f,12.7930f,1.0000f,12.7442f,1.0000f,15.0879f,-4.0000f,15.0391f,-4.0000f,15.0391f,-3.8958f,14.9903f,-3.8958f,14.9903f,-3.7917f,14.9414f,-3.7917f,14.9414f,-3.6875f,14.8926f,-3.6875f,14.8926f,-3.5833f,14.8438f,-3.5833f,14.8438f,-3.4792f,
	14.7949f,-3.4792f,14.7949f,-3.3750f,14.7461f,-3.3750f,14.7461f,-3.2708f,14.6973f,-3.2708f,14.6973f,-3.1667f,14.6485f,-3.1667f,14.6485f,-3.0625f,14.5996f,-3.0625f,14.5996f,-2.9583f,14.5508f,-2.9583f,14.5508f,-2.8542f,14.5020f,-2.8542f,14.5020f,-2.7500f,14.4531f,-2.7500f,
	14.4531f,-2.6458f,14.4043f,-2.6458f,14.4043f,-2.5417f,14.3555f,-2.5417f,14.3555f,-2.4375f,14.3067f,-2.4375f,14.3067f,-2.3333f,14.2578f,-2.3333f,14.2578f,-2.2292f,14.2090f,-2.2292f,14.2090f,-2.1250f,14.1602f,-2.1250f,14.1602f,-2.0208f,14.1114f,-2.0208f,14.1114f,-1.9167f,
	14.0625f,-1.9167f,14.0625f,-1.8125f,14.0137f,-1.8125f,14.0137f,-1.7083f,13.9649f,-1.7083f,13.9649f,-1.6042f,13.9160f,-1.6042f,13.9160f,-1.5000f,13.8672f,-1.5000f,13.8672f,-1.3958f,13.8184f,-1.3958f,13.8184f,-1.2917f,13.7696f,-1.2917f,13.7696f,-1.1875f,13.7207f,-1.1875f,
	13.7207f,-1.0833f,13.6719f,-1.0833f,13.6719f,-0.9792f,13.6231f,-0.9792f,13.6231f,-0.8750f,13.5742f,-0.8750f,13.5742f,-0.7708f,13.5254f,-0.7708f,13.5254f,-0.6667f,13.4766f,-0.6667f,13.4766f,-0.5625f,13.4278f,-0.5625f,13.4278f,-0.4583f,13.3789f,-0.4583f,13.3789f,-0.3542f,
	13.3301f,-0.3542f,13.3301f,-0.2500f,13.2813f,-0.2500f,13.2813f,-0.1458f,13.2324f,-0.1458f,13.2324f,-0.0417f,13.1836f,-0.0417f,13.1836f,0.0625f,13.1348f,0.0625f,13.1348f,0.1667f,13.0860f,0.1667f,13.0860f,0.2708f,13.0371f,0.2708f,13.0371f,0.3750f,12.9883f,0.3750f,
	12.9883f,0.4792f,12.9395f,0.4792f,12.9395f,0.5833f,12.8906f,0.5833f,12.8906f,0.6875f,12.8418f,0.6875f,12.8418f,0.7917f,12.7930f,0.7917f,12.7930f,0.8958f,12.7442f,0.8958f,12.7442f,1.0000f,12.6953f,1.0000f,15.0391f,-4.0000f,14.9903f,-4.0000f,14.9903f,-3.8958f,
	14.9414f,-3.8958f,14.9414f,-3.7917f,14.8926f,-3.7917f,14.8926f,-3.6875f,14.8438f,-3.6875f,14.8438f,-3.5833f,14.7949f,-3.5833f,14.7949f,-3.4792f,14.7461f,-3.4792f,14.7461f,-3.3750f,14.6973f,-3.3750f,14.6973f,-3.2708f,14.6485f,-3.2708f,14.6485f,-3.1667f,14.5996f,-3.1667f,
	14.5996f,-3.0625f,14.5508f,-3.0625f,14.5508f,-2.9583f,14.5020f,-2.9583f,14.5020f,-2.8542f,14.4531f,-2.8542f,14.4531f,-2.7500f,14.4043f,-2.7500f,14.4043f,-2.6458f,14.3555f,-2.6458f,14.3555f,-2.5417f,14.3067f,-2.5417f,14.3067f,-2.4375f,14.2578f,-2.4375f,14.2578f,-2.3333f,
	14.2090f,-2.3333f,14.2090f,-2.2292f,14.1602f,-2.2292f,14.1602f,-2.1250f,14.1114f,-2.1250f,14.1114f,-2.0208f,14.0625f,-2.0208f,14.0625f,-1.9167f,14.0137f,-1.9167f,14.0137f,-1.8125f,13.9649f,-1.8125f,13.9649f,-1.7083f,13.9160f,-1.7083f,13.9160f,-1.6042f,13.8672f,-1.6042f,
	13.8672f,-1.5000f,13.8184f,-1.5000f,13.8184f,-1.3958f,13.7696f,-1.3958f,13.7696f,-1.2917f,13.7207f,-1.2917f,13.7207f,-1.1875f,13.6719f,-1.1875f,13.6719f,-1.0833f,13.6231f,-1.0833f,13.6231f,-0.9792f,13.5742f,-0.9792f,13.5742f,-0.8750f,13.5254f,-0.8750f,13.5254f,-0.7708f,
	13.4766f,-0.7708f,13.4766f,-0.6667f,13.4278f,-0.6667f,13.4278f,-0.5625f,13.3789f,-0.5625f,13.3789f,-0.4583f,13.3301f,-0.4583f,13.3301f,-0.3542f,13.2813f,-0.3542f,13.2813f,-0.2500f,13.2324f,-0.2500f,13.2324f,-0.1458f,13.1836f,-0.1458f,13.1836f,-0.0417f,13.1348f,-0.0417f,
	13.1348f,0.0625f,13.0860f,0.0625f,13.0860f,0.1667f,13.0371f,0.1667f,13.0371f,0.2708f,12.9883f,0.2708f,12.9883f,0.3750f,12.9395f,0.3750f,12.9395f,0.4792f,12.8906f,0.4792f,12.8906f,0.5833f,12.8418f,0.5833f,12.8418f,0.6875f,12.7930f,0.6875f,12.7930f,0.7917f,
	12.7442f,0.7917f,12.7442f,0.8958f,12.6953f,0.8958f,12.6953f,1.0000f,12.6465f,1.0000f,14.9903f,-4.0000f,14.9414f,-4.0000f,14.9414f,-3.8958f,14.8926f,-3.8958f,14.8926f,-3.7917f,14.8438f,-3.7917f,14.8438f,-3.6875f,14.7949f,-3.6875f,14.7949f,-3.5833f,14.7461f,-3.5833f,
	14.7461f,-3.4792f,14.6973f,-3.4792f,14.6973f,-3.3750f,14.6485f,-3.3750f,14.6485f,-3.2708f,14.5996f,-3.2708f,14.5996f,-3.1667f,14.5508f,-3.1667f,14.5508f,-3.0625f,14.5020f,-3.0625f,14.5020f,-2.9583f,14.4531f,-2.9583f,14.4531f,-2.8542f,14.4043f,-2.8542f,14.4043f,-2.7500f,
	14.3555f,-2.7500f,14.3555f,-2.6458f,14.3067f,-2.6458f,14.3067f,-2.5417f,14.2578f,-2.5417f,14.2578f,-2.4375f,14.2090f,-2.4375f,14.2090f,-2.3333f,14.1602f,-2.3333f,14.1602f,-2.2292f,14.1114f,-2.2292f,14.1114f,-2.1250f,14.0625f,-2.1250f,14.0625f,-2.0208f,14.0137f,-2.0208f,
	14.0137f,-1.9167f,13.9649f,-1.9167f,13.9649f,-1.8125f,13.9160f,-1.8125f,13.9160f,-1.7083f,13.8672f,-1.7083f,13.8672f,-1.6042f,13.8184f,-1.6042f,13.8184f,-1.5000f,13.7696f,-1.5000f,13.7696f,-1.3958f,13.7207f,-1.3958f,13.7207f,-1.2917f,13.6719f,-1.2917f,13.6719f,-1.1875f,
	13.6231f,-1.1875f,13.6231f,-1.0833f,13.5742f,-1.0833f,13.5742f,-0.9792f,13.5254f,-0.9792f,13.5254f,-0.8750f,13.4766f,-0.8750f,13.4766f,-0.7708f,13.4278f,-0.7708f,13.4278f,-0.6667f,13.3789f,-0.6667f,13.3789f,-0.5625f,13.3301f,-0.5625f,13.3301f,-0.4583f,13.2813f,-0.4583f,
	13.2813f,-0.3542f,13.2324f,-0.3542f,13.2324f,-0.2500f,13.1836f,-0.2500f,13.1836f,-0.1458f,13.1348f,-0.1458f,13.1348f,-0.0417f,13.0860f,-0.0417f,13.0860f,0.0625f,13.0371f,0.0625f,13.0371f,0.1667f,12.9883f,0.1667f,12.9883f,0.2708f,12.9395f,0.2708f,12.9395f,0.3750f,
	12.8906f,0.3750f,12.8906f,0.4792f,12.8418f,0.4792f,12.8418f,0.5833f,12.7930f,0.5833f,12.7930f,0.6875f,12.7442f,0.6875f,12.7442f,0.7917f,12.6953f,0.7917f,12.6953f,0.8958f,12.6465f,0.8958f,12.6465f,1.0000f,12.5977f,1.0000f,14.9414f,-4.0000f,14.8926f,-4.0000f,
	14.8926f,-3.8958f,14.8438f,-3.8958f,14.8438f,-3.7917f,14.7949f,-3.7917f,14.7949f,-3.6875f,14.7461f,-3.6875f,14.7461f,-3.5833f,14.6973f,-3.5833f,14.6973f,-3.4792f,14.6485f,-3.4792f,14.6485f,-3.3750f,14.5996f,-3.3750f,14.5996f,-3.2708f,14.5508f,-3.2708f,14.5508f,-3.1667f,
	14.5020f,-3.1667f,14.5020f,-3.0625f,14.4531f,-3.0625f,14.4531f,-2.9583f,14.4043f,-2.9583f,14.4043f,-2.8542f,14.3555f,-2.8542f,14.3555f,-2.7500f,14.3067f,-2.7500f,14.3067f,-2.6458f,14.2578f,-2.6458f,14.2578f,-2.5417f,14.2090f,-2.5417f,14.2090f,-2.4375f,14.1602f,-2.4375f,
	14.1602f,-2.3333f,14.1114f,-2.3333f,14.1114f,-2.2292f,14.0625f,-2.2292f,14.0625f,-2.1250f,14.0137f,-2.1250f,14.0137f,-2.0208f,13.9649f,-2.0208f,13.9649f,-1.9167f,13.9160f,-1.9167f,13.9160f,-1.8125f,13.8672f,-1.8125f,13.8672f,-1.7083f,13.8184f,-1.7083f,13.8184f,-1.6042f,
	13.7696f,-1.6042f,13.7696f,-1.5000f,13.7207f,-1.5000f,13.7207f,-1.3958f,13.6719f,-1.3958f,13.6719f,-1.2917f,13.6231f,-1.2917f,13.6231f,-1.1875f,13.5742f,-1.1875f,13.5742f,-1.0833f,13.5254f,-1.0833f,13.5254f,-0.9792f,13.4766f,-0.9792f,13.4766f,-0.8750f,13.4278f,-0.8750f,
	13.4278f,-0.7708f,13.3789f,-0.7708f,13.3789f,-0.6667f,13.3301f,-0.6667f,13.3301f,-0.5625f,13.2813f,-0.5625f,13.2813f,-0.4583f,13.2324f,-0.4583f,13.2324f,-0.3542f,13.1836f,-0.3542f,13.1836f,-0.2500f,13.1348f,-0.2500f,13.1348f,-0.1458f,13.0860f,-0.1458f,13.0860f,-0.0417f,
	13.0371f,-0.0417f,13.0371f,0.0625f,12.9883f,0.0625f,12.9883f,0.1667f,12.9395f,0.1667f,12.9395f,0.2708f,12.8906f,0.2708f,12.8906f,0.3750f,12.8418f,0.3750f,12.8418f,0.4792f,12.7930f,0.4792f,12.7930f,0.5833f,12.7442f,0.5833f,12.7442f,0.6875f,12.6953f,0.6875f,
	12.6953f,0.7917f,12.6465f,0.7917f,12.6465f,0.8958f,12.5977f,0.8958f,12.5977f,1.0000f,12.5488f,1.0000f,14.8926f,-4.0000f,14.8438f,-4.0000f,14.8438f,-3.8958f,14.7949f,-3.8958f,14.7949f,-3.7917f,14.7461f,-3.7917f,14.7461f,-3.6875f,14.6973f,-3.6875f,14.6973f,-3.5833f,
	14.6485f,-3.5833f,14.6485f,-3.4792f,14.5996f,-3.4792f,14.5996f,-3.3750f,14.5508f,-3.3750f,14.5508f,-3.2708f,14.5020f,-3.2708f,14.5020f,-3.1667f,14.4531f,-3.1667f,14.4531f,-3.0625f,14.4043f,-3.0625f,14.4043f,-2.9583f,14.3555f,-2.9583f,14.3555f,-2.8542f,14.3067f,-2.8542f,
	14.3067f,-2.7500f,14.2578f,-2.7500f,14.2578f,-2.6458f,14.2090f,-2.6458f,14.2090f,-2.5417f,14.1602f,-2.5417f,14.1602f,-2.4375f,14.1114f,-2.4375f,14.1114f,-2.3333f,14.0625f,-2.3333f,14.0625f,-2.2292f,14.0137f,-2.2292f,14.0137f,-2.1250f,13.9649f,-2.1250f,13.9649f,-2.0208f,
	13.9160f,-2.0208f,13.9160f,-1.9167f,13.8672f,-1.9167f,13.8672f,-1.8125f,13.8184f,-1.8125f,13.8184f,-1.7083f,13.7696f,-1.7083f,13.7696f,-1.6042f,13.7207f,-1.6042f,13.7207f,-1.5000f,13.6719f,-1.5000f,13.6719f,-1.3958f,13.6231f,-1.3958f,13.6231f,-1.2917f,13.5742f,-1.2917f,
	13.5742f,-1.1875f,13.5254f,-1.1875f,13.5254f,-1.0833f,13.4766f,-1.0833f,13.4766f,-0.9792f,13.4278f,-0.9792f,13.4278f,-0.8750f,13.3789f,-0.8750f,13.3789f,-0.7708f,13.3301f,-0.7708f,13.3301f,-0.6667f,13.2813f,-0.6667f,13.2813f,-0.5625f,13.2324f,-0.5625f,13.2324f,-0.4583f,
	13.1836f,-0.4583f,13.1836f,-0.3542f,13.1348f,-0.3542f,13.1348f,-0.2500f,13.0860f,-0.2500f,13.0860f,-0.1458f,13.0371f,-0.1458f,13.0371f,-0.0417f,12.9883f,-0.0417f,12.9883f,0.0625f,12.9395f,0.0625f,12.9395f,0.1667f,12.8906f,0.1667f,12.8906f,0.2708f,12.8418f,0.2708f,
	12.8418f,0.3750f,12.7930f,0.3750f,12.7930f,0.4792f,12.7442f,0.4792f,12.7442f,0.5833f,12.6953f,0.5833f,12.6953f,0.6875f,12.6465f,0.6875f,12.6465f,0.7917f,12.5977f,0.7917f,12.5977f,0.8958f,12.5488f,0.8958f,12.5488f,1.0000f,12.5000f,1.0000f,14.8438f,-4.0000f,
	14.7949f,-4.0000f,14.7949f,-3.8958f,14.7461f,-3.8958f,14.7461f,-3.7917f,14.6973f,-3.7917f,14.6973f,-3.6875f,14.6485f,-3.6875f,14.6485f,-3.5833f,14.5996f,-3.5833f,14.5996f,-3.4792f,14.5508f,-3.4792f,14.5508f,-3.3750f,14.5020f,-3.3750f,14.5020f,-3.2708f,14.4531f,-3.2708f,
	14.4531f,-3.1667f,14.4043f,-3.1667f,14.4043f,-3.0625f,14.3555f,-3.0625f,14.3555f,-2.9583f,14.3067f,-2.9583f,14.3067f,-2.8542f,14.2578f,-2.8542f,14.2578f,-2.7500f,14.2090f,-2.7500f,14.2090f,-2.6458f,14.1602f,-2.6458f,14.1602f,-2.5417f,14.1114f,-2.5417f,14.1114f,-2.4375f,
	14.0625f,-2.4375f,14.0625f,-2.3333f,14.0137f,-2.3333f,14.0137f,-2.2292f,13.9649f,-2.2292f,13.9649f,-2.1250f,13.9160f,-2.1250f,13.9160f,-2.0208f,13.8672f,-2.0208f,13.8672f,-1.9167f,13.8184f,-1.9167f,13.8184f,-1.8125f,13.7696f,-1.8125f,13.7696f,-1.7083f,13.7207f,-1.7083f,
	13.7207f,-1.6042f,13.6719f,-1.6042f,13.6719f,-1.5000f,13.6231f,-1.5000f,13.6231f,-1.3958f,13.5742f,-1.3958f,13.5742f,-1.2917f,13.5254f,-1.2917f,13.5254f,-1.1875f,13.4766f,-1.1875f,13.4766f,-1.0833f,13.4278f,-1.0833f,13.4278f,-0.9792f,13.3789f,-0.9792f,13.3789f,-0.8750f,
	13.3301f,-0.8750f,13.3301f,-0.7708f,13.2813f,-0.7708f,13.2813f,-0.6667f,13.2324f,-0.6667f,13.2324f,-0.5625f,13.1836f,-0.5625f,13.1836f,-0.4583f,13.1348f,-0.4583f,13.1348f,-0.3542f,13.0860f,-0.3542f,13.0860f,-0.2500f,13.0371f,-0.2500f,13.0371f,-0.1458f,12.9883f,-0.1458f,
	12.9883f,-0.0417f,12.9395f,-0.0417f,12.9395f,0.0625f,12.8906f,0.0625f,12.8906f,0.1667f,12.8418f,0.1667f,12.8418f,0.2708f,12.7930f,0.2708f,12.7930f,0.3750f,12.7442f,0.3750f,12.7442f,0.4792f,12.6953f,0.4792f,12.6953f,0.5833f,12.6465f,0.5833f,12.6465f,0.6875f,
	12.5977f,0.6875f,12.5977f,0.7917f,12.5488f,0.7917f,12.5488f,0.8958f,12.5000f,0.8958f,12.5000f,1.0000f,12.4512f,1.0000f,14.7949f,-4.0000f,14.7461f,-4.0000f,14.7461f,-3.8958f,14.6973f,-3.8958f,14.6973f,-3.7917f,14.6485f,-3.7917f,14.6485f,-3.6875f,14.5996f,-3.6875f,
	14.5996f,-3.5833f,14.5508f,-3.5833f,14.5508f,-3.4792f,14.5020f,-3.4792f,14.5020f,-3.3750f,14.4531f,-3.3750f,14.4531f,-3.2708f,14.4043f,-3.2708f,14.4043f,-3.1667f,14.3555f,-3.1667f,14.3555f,-3.0625f,14.3067f,-3.0625f,14.3067f,-2.9583f,14.2578f,-2.9583f,14.2578f,-2.8542f,
	14.2090f,-2.8542f,14.2090f,-2.7500f,14.1602f,-2.7500f,14.1602f,-2.6458f,14.1114f,-2.6458f,14.1114f,-2.5417f,14.0625f,-2.5417f,14.0625f,-2.4375f,14.0137f,-2.4375f,14.0137f,-2.3333f,13.9649f,-2.3333f,13.9649f,-2.2292f,13.9160f,-2.2292f,13.9160f,-2.1250f,13.8672f,-2.1250f,
	13.8672f,-2.0208f,13.8184f,-2.0208f,13.8184f,-1.9167f,13.7696f,-1.9167f,13.7696f,-1.8125f,13.7207f,-1.8125f,13.7207f,-1.7083f,13.6719f,-1.7083f,13.6719f,-1.6042f,13.6231f,-1.6042f,13.6231f,-1.5000f,13.5742f,-1.5000f,13.5742f,-1.3958f,13.5254f,-1.3958f,13.5254f,-1.2917f,
	13.4766f,-1.2917f,13.4766f,-1.1875f,13.4278f,-1.1875f,13.4278f,-1.0833f,13.3789f,-1.0833f,13.3789f,-0.9792f,13.3301f,-0.9792f,13.3301f,-0.8750f,13.2813f,-0.8750f,13.2813f,-0.7708f,13.2324f,-0.7708f,13.2324f,-0.6667f,13.1836f,-0.6667f,13.1836f,-0.5625f,13.1348f,-0.5625f,
	13.1348f,-0.4583f,13.0860f,-0.4583f,13.0860f,-0.3542f,13.0371f,-0.3542f,13.0371f,-0.2500f,12.9883f,-0.2500f,12.9883f,-0.1458f,12.9395f,-0.1458f,12.9395f,-0.0417f,12.8906f,-0.0417f,12.8906f,0.0625f,12.8418f,0.0625f,12.8418f,0.1667f,12.7930f,0.1667f,12.7930f,0.2708f,
	12.7442f,0.2708f,12.7442f,0.3750f,12.6953f,0.3750f,12.6953f,0.4792f,12.6465f,0.4792f,12.6465f,0.5833f,12.5977f,0.5833f,12.5977f,0.6875f,12.5488f,0.6875f,12.5488f,0.7917f,12.5000f,0.7917f,12.5000f,0.8958f,12.4512f,0.8958f,12.4512f,1.0000f,12.4024f,1.0000f,
	14.7461f,-4.0000f,14.6973f,-4.0000f,14.6973f,-3.8958f,14.6485f,-3.8958f,14.6485f,-3.7917f,14.5996f,-3.7917f,14.5996f,-3.6875f,14.5508f,-3.6875f,14.5508f,-3.5833f,14.5020f,-3.5833f,14.5020f,-3.4792f,14.4531f,-3.4792f,14.4531f,-3.3750f,14.4043f,-3.3750f,14.4043f,-3.2708f,
	14.3555f,-3.2708f,14.3555f,-3.1667f,14.3067f,-3.1667f,14.3067f,-3.0625f,14.2578f,-3.0625f,14.2578f,-2.9583f,14.2090f,-2.9583f,14.2090f,-2.8542f,14.1602f,-2.8542f,14.1602f,-2.7500f,14.1114f,-2.7500f,14.1114f,-2.6458f,14.0625f,-2.6458f,14.0625f,-2.5417f,14.0137f,-2.5417f,
	14.0137f,-2.4375f,13.9649f,-2.4375f,13.9649f,-2.3333f,13.9160f,-2.3333f,13.9160f,-2.2292f,13.8672f,-2.2292f,13.8672f,-2.1250f,13.8184f,-2.1250f,13.8184f,-2.0208f,13.7696f,-2.0208f,13.7696f,-1.9167f,13.7207f,-1.9167f,13.7207f,-1.8125f,13.6719f,-1.8125f,13.6719f,-1.7083f,
	13.6231f,-1.7083f,13.6231f,-1.6042f,13.5742f,-1.6042f,13.5742f,-1.5000f,13.5254f,-1.5000f,13.5254f,-1.3958f,13.4766f,-1.3958f,13.4766f,-1.2917f,13.4278f,-1.2917f,13.4278f,-1.1875f,13.3789f,-1.1875f,13.3789f,-1.0833f,13.3301f,-1.0833f,13.3301f,-0.9792f,13.2813f,-0.9792f,
	13.2813f,-0.8750f,13.2324f,-0.8750f,13.2324f,-0.7708f,13.1836f,-0.7708f,13.1836f,-0.6667f,13.1348f,-0.6667f,13.1348f,-0.5625f,13.0860f,-0.5625f,13.0860f,-0.4583f,13.0371f,-0.4583f,13.0371f,-0.3542f,12.9883f,-0.3542f,12.9883f,-0.2500f,12.9395f,-0.2500f,12.9395f,-0.1458f,
	12.8906f,-0.1458f,12.8906f,-0.0417f,12.8418f,-0.0417f,12.8418f,0.0625f,12.7930f,0.0625f,12.7930f,0.1667f,12.7442f,0.1667f,12.7442f,0.2708f,12.6953f,0.2708f,12.6953f,0.3750f,12.6465f,0.3750f,12.6465f,0.4792f,12.5977f,0.4792f,12.5977f,0.5833f,12.5488f,0.5833f,
	12.5488f,0.6875f,12.5000f,0.6875f,12.5000f,0.7917f,12.4512f,0.7917f,12.4512f,0.8958f,12.4024f,0.8958f,12.4024f,1.0000f,12.3535f,1.0000f,14.6973f,-4.0000f,14.6485f,-4.0000f,14.6485f,-3.8958f,14.5996f,-3.8958f,14.5996f,-3.7917f,14.5508f,-3.7917f,14.5508f,-3.6875f,
	14.5020f,-3.6875f,14.5020f,-3.5833f,14.4531f,-3.5833f,14.4531f,-3.4792f,14.4043f,-3.4792f,14.4043f,-3.3750f,14.3555f,-3.3750f,14.3555f,-3.2708f,14.3067f,-3.2708f,14.3067f,-3.1667f,14.2578f,-3.1667f,14.2578f,-3.0625f,14.2090f,-3.0625f,14.2090f,-2.9583f,14.1602f,-2.9583f,
	14.1602f,-2.8542f,14.1114f,-2.8542f,14.1114f,-2.7500f,14.0625f,-2.7500f,14.0625f,-2.6458f,14.0137f,-2.6458f,14.0137f,-2.5417f,13.9649f,-2.5417f,13.9649f,-2.4375f,13.9160f,-2.4375f,13.9160f,-2.3333f,13.8672f,-2.3333f,13.8672f,-2.2292f,13.8184f,-2.2292f,13.8184f,-2.1250f,
	13.7696f,-2.1250f,13.7696f,-2.0208f,13.7207f,-2.0208f,13.7207f,-1.9167f,13.6719f,-1.9167f,13.6719f,-1.8125f,13.6231f,-1.8125f,13.6231f,-1.7083f,13.5742f,-1.7083f,13.5742f,-1.6042f,13.5254f,-1.6042f,13.5254f,-1.5000f,13.4766f,-1.5000f,13.4766f,-1.3958f,13.4278f,-1.3958f,
	13.4278f,-1.2917f,13.3789f,-1.2917f,13.3789f,-1.1875f,13.3301f,-1.1875f,13.3301f,-1.0833f,13.2813f,-1.0833f,13.2813f,-0.9792f,13.2324f,-0.9792f,13.2324f,-0.8750f,13.1836f,-0.8750f,13.1836f,-0.7708f,13.1348f,-0.7708f,13.1348f,-0.6667f,13.0860f,-0.6667f,13.0860f,-0.5625f,
	13.0371f,-0.5625f,13.0371f,-0.4583f,12.9883f,-0.4583f,12.9883f,-0.3542f,12.9395f,-0.3542f,12.9395f,-0.2500f,12.8906f,-0.2500f,12.8906f,-0.1458f,12.8418f,-0.1458f,12.8418f,-0.0417f,12.7930f,-0.0417f,12.7930f,0.0625f,12.7442f,0.0625f,12.7442f,0.1667f,12.6953f,0.1667f,
	12.6953f,0.2708f,12.6465f,0.2708f,12.6465f,0.3750f,12.5977f,0.3750f,12.5977f,0.4792f,12.5488f,0.4792f,12.5488f,0.5833f,12.5000f,0.5833f,12.5000f,0.6875f,12.4512f,0.6875f,12.4512f,0.7917f,12.4024f,0.7917f,12.4024f,0.8958f,12.3535f,0.8958f,12.3535f,1.0000f,
	12.3047f,1.0000f,14.6485f,-4.0000f,14.5996f,-4.0000f,14.5996f,-3.8958f,14.5508f,-3.8958f,14.5508f,-3.7917f,14.5020f,-3.7917f,14.5020f,-3.6875f,14.4531f,-3.6875f,14.4531f,-3.5833f,14.4043f,-3.5833f,14.4043f,-3.4792f,14.3555f,-3.4792f,14.3555f,-3.3750f,14.3067f,-3.3750f,
	14.3067f,-3.2708f,14.2578f,-3.2708f,14.2578f,-3.1667f,14.2090f,-3.1667f,14.2090f,-3.0625f,14.1602f,-3.0625f,14.1602f,-2.9583f,14.1114f,-2.9583f,14.1114f,-2.8542f,14.0625f,-2.8542f,14.0625f,-2.7500f,14.0137f,-2.7500f,14.0137f,-2.6458f,13.9649f,-2.6458f,13.9649f,-2.5417f,
	13.9160f,-2.5417f,13.9160f,-2.4375f,13.8672f,-2.4375f,13.8672f,-2.3333f,13.8184f,-2.3333f,13.8184f,-2.2292f,13.7696f,-2.2292f,13.7696f,-2.1250f,13.7207f,-2.1250f,13.7207f,-2.0208f,13.6719f,-2.0208f,13.6719f,-1.9167f,13.6231f,-1.9167f,13.6231f,-1.8125f,13.5742f,-1.8125f,
	13.5742f,-1.7083f,13.5254f,-1.7083f,13.5254f,-1.6042f,13.4766f,-1.6042f,13.4766f,-1.5000f,13.4278f,-1.5000f,13.4278f,-1.3958f,13.3789f,-1.3958f,13.3789f,-1.2917f,13.3301f,-1.2917f,13.3301f,-1.1875f,13.2813f,-1.1875f,13.2813f,-1.0833f,13.2324f,-1.0833f,13.2324f,-0.9792f,
	13.1836f,-0.9792f,13.1836f,-0.8750f,13.1348f,-0.8750f,13.1348f,-0.7708f,13.0860f,-0.7708f,13.0860f,-0.6667f,13.0371f,-0.6667f,13.0371f,-0.5625f,12.9883f,-0.5625f,12.9883f,-0.4583f,12.9395f,-0.4583f,12.9395f,-0.3542f,12.8906f,-0.3542f,12.8906f,-0.2500f,12.8418f,-0.2500f,
	12.8418f,-0.1458f,12.7930f,-0.1458f,12.7930f,-0.0417f,12.7442f,-0.0417f,12.7442f,0.0625f,12.6953f,0.0625f,12.6953f,0.1667f,12.6465f,0.1667f,12.6465f,0.2708f,12.5977f,0.2708f,12.5977f,0.3750f,12.5488f,0.3750f,12.5488f,0.4792f,12.5000f,0.4792f,12.5000f,0.5833f,
	12.4512f,0.5833f,12.4512f,0.6875f,12.4024f,0.6875f,12.4024f,0.7917f,12.3535f,0.7917f,12.3535f,0.8958f,12.3047f,0.8958f,12.3047f,1.0000f,12.2559f,1.0000f,14.5996f,-4.0000f,14.5508f,-4.0000f,14.5508f,-3.8958f,14.5020f,-3.8958f,14.5020f,-3.7917f,14.4531f,-3.7917f,
	14.4531f,-3.6875f,14.4043f,-3.6875f,14.4043f,-3.5833f,14.3555f,-3.5833f,14.3555f,-3.4792f,14.3067f,-3.4792f,14.3067f,-3.3750f,14.2578f,-3.3750f,14.2578f,-3.2708f,14.2090f,-3.2708f,14.2090f,-3.1667f,14.1602f,-3.1667f,14.1602f,-3.0625f,14.1114f,-3.0625f,14.1114f,-2.9583f,
	14.0625f,-2.9583f,14.0625f,-2.8542f,14.0137f,-2.8542f,14.0137f,-2.7500f,13.9649f,-2.7500f,13.9649f,-2.6458f,13.9160f,-2.6458f,13.9160f,-2.5417f,13.8672f,-2.5417f,13.8672f,-2.4375f,13.8184f,-2.4375f,13.8184f,-2.3333f,13.7696f,-2.3333f,13.7696f,-2.2292f,13.7207f,-2.2292f,
	13.7207f,-2.1250f,13.6719f,-2.1250f,13.6719f,-2.0208f,13.6231f,-2.0208f,13.6231f,-1.9167f,13.5742f,-1.9167f,13.5742f,-1.8125f,13.5254f,-1.8125f,13.5254f,-1.7083f,13.4766f,-1.7083f,13.4766f,-1.6042f,13.4278f,-1.6042f,13.4278f,-1.5000f,13.3789f,-1.5000f,13.3789f,-1.3958f,
	13.3301f,-1.3958f,13.3301f,-1.2917f,13.2813f,-1.2917f,13.2813f,-1.1875f,13.2324f,-1.1875f,13.2324f,-1.0833f,13.1836f,-1.0833f,13.1836f,-0.9792f,13.1348f,-0.9792f,13.1348f,-0.8750f,13.0860f,-0.8750f,13.0860f,-0.7708f,13.0371f,-0.7708f,13.0371f,-0.6667f,12.9883f,-0.6667f,
	12.9883f,-0.5625f,12.9395f,-0.5625f,12.9395f,-0.4583f,12.8906f,-0.4583f,12.8906f,-0.3542f,12.8418f,-0.3542f,12.8418f,-0.2500f,12.7930f,-0.2500f,12.7930f,-0.1458f,12.7442f,-0.1458f,12.7442f,-0.0417f,12.6953f,-0.0417f,12.6953f,0.0625f,12.6465f,0.0625f,12.6465f,0.1667f,
	12.5977f,0.1667f,12.5977f,0.2708f,12.5488f,0.2708f,12.5488f,0.3750f,12.5000f,0.3750f,12.5000f,0.4792f,12.4512f,0.4792f,12.4512f,0.5833f,12.4024f,0.5833f,12.4024f,0.6875f,12.3535f,0.6875f,12.3535f,0.7917f,12.3047f,0.7917f,12.3047f,0.8958f,12.2559f,0.8958f,
	12.2559f,1.0000f,12.2071f,1.0000f,14.5508f,-4.0000f,14.5020f,-4.0000f,14.5020f,-3.8958f,14.4531f,-3.8958f,14.4531f,-3.7917f,14.4043f,-3.7917f,14.4043f,-3.6875f,14.3555f,-3.6875f,14.3555f,-3.5833f,14.3067f,-3.5833f,14.3067f,-3.4792f,14.2578f,-3.4792f,14.2578f,-3.3750f,
	14.2090f,-3.3750f,14.2090f,-3.2708f,14.1602f,-3.2708f,14.1602f,-3.1667f,14.1114f,-3.1667f,14.1114f,-3.0625f,14.0625f,-3.0625f,14.0625f,-2.9583f,14.0137f,-2.9583f,14.0137f,-2.8542f,13.9649f,-2.8542f,13.9649f,-2.7500f,13.9160f,-2.7500f,13.9160f,-2.6458f,13.8672f,-2.6458f,
	13.8672f,-2.5417f,13.8184f,-2.5417f,13.8184f,-2.4375f,13.7696f,-2.4375f,13.7696f,-2.3333f,13.7207f,-2.3333f,13.7207f,-2.2292f,13.6719f,-2.2292f,13.6719f,-2.1250f,13.6231f,-2.1250f,13.6231f,-2.0208f,13.5742f,-2.0208f,13.5742f,-1.9167f,13.5254f,-1.9167f,13.5254f,-1.8125f,
	13.4766f,-1.8125f,13.4766f,-1.7083f,13.4278f,-1.7083f,13.4278f,-1.6042f,13.3789f,-1.6042f,13.3789f,-1.5000f,13.3301f,-1.5000f,13.3301f,-1.3958f,13.2813f,-1.3958f,13.2813f,-1.2917f,13.2324f,-1.2917f,13.2324f,-1.1875f,13.1836f,-1.1875f,13.1836f,-1.0833f,13.1348f,-1.0833f,
	13.1348f,-0.9792f,13.0860f,-0.9792f,13.0860f,-0.8750f,13.0371f,-0.8750f,13.0371f,-0.7708f,12.9883f,-0.7708f,12.9883f,-0.6667f,12.9395f,-0.6667f,12.9395f,-0.5625f,12.8906f,-0.5625f,12.8906f,-0.4583f,12.8418f,-0.4583f,12.8418f,-0.3542f,12.7930f,-0.3542f,12.7930f,-0.2500f,
	12.7442f,-0.2500f,12.7442f,-0.1458f,12.6953f,-0.1458f,12.6953f,-0.0417f,12.6465f,-0.0417f,12.6465f,0.0625f,12.5977f,0.0625f,12.5977f,0.1667f,12.5488f,0.1667f,12.5488f,0.2708f,12.5000f,0.2708f,12.5000f,0.3750f,12.4512f,0.3750f,12.4512f,0.4792f,12.4024f,0.4792f,
	12.4024f,0.5833f,12.3535f,0.5833f,12.3535f,0.6875f,12.3047f,0.6875f,12.3047f,0.7917f,12.2559f,0.7917f,12.2559f,0.8958f,12.2071f,0.8958f,12.2071f,1.0000f,12.1582f,1.0000f,14.5020f,-4.0000f,14.4531f,-4.0000f,14.4531f,-3.8958f,14.4043f,-3.8958f,14.4043f,-3.7917f,
	14.3555f,-3.7917f,14.3555f,-3.6875f,14.3067f,-3.6875f,14.3067f,-3.5833f,14.2578f,-3.5833f,14.2578f,-3.4792f,14.2090f,-3.4792f,14.2090f,-3.3750f,14.1602f,-3.3750f,14.1602f,-3.2708f,14.1114f,-3.2708f,14.1114f,-3.1667f,14.0625f,-3.1667f,14.0625f,-3.0625f,14.0137f,-3.0625f,
	14.0137f,-2.9583f,13.9649f,-2.9583f,13.9649f,-2.8542f,13.9160f,-2.8542f,13.9160f,-2.7500f,13.8672f,-2.7500f,13.8672f,-2.6458f,13.8184f,-2.6458f,13.8184f,-2.5417f,13.7696f,-2.5417f,13.7696f,-2.4375f,13.7207f,-2.4375f,13.7207f,-2.3333f,13.6719f,-2.3333f,13.6719f,-2.2292f,
	13.6231f,-2.2292f,13.6231f,-2.1250f,13.5742f,-2.1250f,13.5742f,-2.0208f,13.5254f,-2.0208f,13.5254f,-1.9167f,13.4766f,-1.9167f,13.4766f,-1.8125f,13.4278f,-1.8125f,13.4278f,-1.7083f,13.3789f,-1.7083f,13.3789f,-1.6042f,13.3301f,-1.6042f,13.3301f,-1.5000f,13.2813f,-1.5000f,
	13.2813f,-1.3958f,13.2324f,-1.3958f,13.2324f,-1.2917f,13.1836f,-1.2917f,13.1836f,-1.1875f,13.1348f,-1.1875f,13.1348f,-1.0833f,13.0860f,-1.0833f,13.0860f,-0.9792f,13.0371f,-0.9792f,13.0371f,-0.8750f,12.9883f,-0.8750f,12.9883f,-0.7708f,12.9395f,-0.7708f,12.9395f,-0.6667f,
	12.8906f,-0.6667f,12.8906f,-0.5625f,12.8418f,-0.5625f,12.8418f,-0.4583f,12.7930f,-0.4583f,12.7930f,-0.3542f,12.7442f,-0.3542f,12.7442f,-0.2500f,12.6953f,-0.2500f,12.6953f,-0.1458f,12.6465f,-0.1458f,12.6465f,-0.0417f,12.5977f,-0.0417f,12.5977f,0.0625f,12.5488f,0.0625f,
	12.5488f,0.1667f,12.5000f,0.1667f,12.5000f,0.2708f,12.4512f,0.2708f,12.4512f,0.3750f,12.4024f,0.3750f,12.4024f,0.4792f,12.3535f,0.4792f,12.3535f,0.5833f,12.3047f,0.5833f,12.3047f,0.6875f,12.2559f,0.6875f,12.2559f,0.7917f,12.2071f,0.7917f,12.2071f,0.8958f,
	12.1582f,0.8958f,12.1582f,1.0000f,12.1094f,1.0000f,14.4531f,-4.0000f,14.4043f,-4.0000f,14.4043f,-3.8958f,14.3555f,-3.8958f,14.3555f,-3.7917f,14.3067f,-3.7917f,14.3067f,-3.6875f,14.2578f,-3.6875f,14.2578f,-3.5833f,14.2090f,-3.5833f,14.2090f,-3.4792f,14.1602f,-3.4792f,
	14.1602f,-3.3750f,14.1114f,-3.3750f,14.1114f,-3.2708f,14.0625f,-3.2708f,14.0625f,-3.1667f,14.0137f,-3.1667f,14.0137f,-3.0625f,13.9649f,-3.0625f,13.9649f,-2.9583f,13.9160f,-2.9583f,13.9160f,-2.8542f,13.8672f,-2.8542f,13.8672f,-2.7500f,13.8184f,-2.7500f,13.8184f,-2.6458f,
	13.7696f,-2.6458f,13.7696f,-2.5417f,13.7207f,-2.5417f,13.7207f,-2.4375f,13.6719f,-2.4375f,13.6719f,-2.3333f,13.6231f,-2.3333f,13.6231f,-2.2292f,13.5742f,-2.2292f,13.5742f,-2.1250f,13.5254f,-2.1250f,13.5254f,-2.0208f,13.4766f,-2.0208f,13.4766f,-1.9167f,13.4278f,-1.9167f,
	13.4278f,-1.8125f,13.3789f,-1.8125f,13.3789f,-1.7083f,13.3301f,-1.7083f,13.3301f,-1.6042f,13.2813f,-1.6042f,13.2813f,-1.5000f,13.2324f,-1.5000f,13.2324f,-1.3958f,13.1836f,-1.3958f,13.1836f,-1.2917f,13.1348f,-1.2917f,13.1348f,-1.1875f,13.0860f,-1.1875f,13.0860f,-1.0833f,
	13.0371f,-1.0833f,13.0371f,-0.9792f,12.9883f,-0.9792f,12.9883f,-0.8750f,12.9395f,-0.8750f,12.9395f,-0.7708f,12.8906f,-0.7708f,12.8906f,-0.6667f,12.8418f,-0.6667f,12.8418f,-0.5625f,12.7930f,-0.5625f,12.7930f,-0.4583f,12.7442f,-0.4583f,12.7442f,-0.3542f,12.6953f,-0.3542f,
	12.6953f,-0.2500f,12.6465f,-0.2500f,12.6465f,-0.1458f,12.5977f,-0.1458f,12.5977f,-0.0417f,12.5488f,-0.0417f,12.5488f,0.0625f,12.5000f,0.0625f,12.5000f,0.1667f,12.4512f,0.1667f,12.4512f,0.2708f,12.4024f,0.2708f,12.4024f,0.3750f,12.3535f,0.3750f,12.3535f,0.4792f,
	12.3047f,0.4792f,12.3047f,0.5833f,12.2559f,0.5833f,12.2559f,0.6875f,12.2071f,0.6875f,12.2071f,0.7917f,12.1582f,0.7917f,12.1582f,0.8958f,12.1094f,0.8958f,12.1094f,1.0000f,12.0606f,1.0000f,14.4043f,-4.0000f,14.3555f,-4.0000f,14.3555f,-3.8958f,14.3067f,-3.8958f,
	14.3067f,-3.7917f,14.2578f,-3.7917f,14.2578f,-3.6875f,14.2090f,-3.6875f,14.2090f,-3.5833f,14.1602f,-3.5833f,14.1602f,-3.4792f,14.1114f,-3.4792f,14.1114f,-3.3750f,14.0625f,-3.3750f,14.0625f,-3.2708f,14.0137f,-3.2708f,14.0137f,-3.1667f,13.9649f,-3.1667f,13.9649f,-3.0625f,
	13.9160f,-3.0625f,13.9160f,-2.9583f,13.8672f,-2.9583f,13.8672f,-2.8542f,13.8184f,-2.8542f,13.8184f,-2.7500f,13.7696f,-2.7500f,13.7696f,-2.6458f,13.7207f,-2.6458f,13.7207f,-2.5417f,13.6719f,-2.5417f,13.6719f,-2.4375f,13.6231f,-2.4375f,13.6231f,-2.3333f,13.5742f,-2.3333f,
	13.5742f,-2.2292f,13.5254f,-2.2292f,13.5254f,-2.1250f,13.4766f,-2.1250f,13.4766f,-2.0208f,13.4278f,-2.0208f,13.4278f,-1.9167f,13.3789f,-1.9167f,13.3789f,-1.8125f,13.3301f,-1.8125f,13.3301f,-1.7083f,13.2813f,-1.7083f,13.2813f,-1.6042f,13.2324f,-1.6042f,13.2324f,-1.5000f,
	13.1836f,-1.5000f,13.1836f,-1.3958f,13.1348f,-1.3958f,13.1348f,-1.2917f,13.0860f,-1.2917f,13.0860f,-1.1875f,13.0371f,-1.1875f,13.0371f,-1.0833f,12.9883f,-1.0833f,12.9883f,-0.9792f,12.9395f,-0.9792f,12.9395f,-0.8750f,12.8906f,-0.8750f,12.8906f,-0.7708f,12.8418f,-0.7708f,
	12.8418f,-0.6667f,12.7930f,-0.6667f,12.7930f,-0.5625f,12.7442f,-0.5625f,12.7442f,-0.4583f,12.6953f,-0.4583f,12.6953f,-0.3542f,12.6465f,-0.3542f,12.6465f,-0.2500f,12.5977f,-0.2500f,12.5977f,-0.1458f,12.5488f,-0.1458f,12.5488f,-0.0417f,12.5000f,-0.0417f,12.5000f,0.0625f,
	12.4512f,0.0625f,12.4512f,0.1667f,12.4024f,0.1667f,12.4024f,0.2708f,12.3535f,0.2708f,12.3535f,0.3750f,12.3047f,0.3750f,12.3047f,0.4792f,12.2559f,0.4792f,12.2559f,0.5833f,12.2071f,0.5833f,12.2071f,0.6875f,12.1582f,0.6875f,12.1582f,0.7917f,12.1094f,0.7917f,
	12.1094f,0.8958f,12.0606f,0.8958f,12.0606f,1.0000f,12.0117f,1.0000f,14.3555f,-4.0000f,14.3067f,-4.0000f,14.3067f,-3.8958f,14.2578f,-3.8958f,14.2578f,-3.7917f,14.2090f,-3.7917f,14.2090f,-3.6875f,14.1602f,-3.6875f,14.1602f,-3.5833f,14.1114f,-3.5833f,14.1114f,-3.4792f,
	14.0625f,-3.4792f,14.0625f,-3.3750f,14.0137f,-3.3750f,14.0137f,-3.2708f,13.9649f,-3.2708f,13.9649f,-3.1667f,13.9160f,-3.1667f,13.9160f,-3.0625f,13.8672f,-3.0625f,13.8672f,-2.9583f,13.8184f,-2.9583f,13.8184f,-2.8542f,13.7696f,-2.8542f,13.7696f,-2.7500f,13.7207f,-2.7500f,
	13.7207f,-2.6458f,13.6719f,-2.6458f,13.6719f,-2.5417f,13.6231f,-2.5417f,13.6231f,-2.4375f,13.5742f,-2.4375f,13.5742f,-2.3333f,13.5254f,-2.3333f,13.5254f,-2.2292f,13.4766f,-2.2292f,13.4766f,-2.1250f,13.4278f,-2.1250f,13.4278f,-2.0208f,13.3789f,-2.0208f,13.3789f,-1.9167f,
	13.3301f,-1.9167f,13.3301f,-1.8125f,13.2813f,-1.8125f,13.2813f,-1.7083f,13.2324f,-1.7083f,13.2324f,-1.6042f,13.1836f,-1.6042f,13.1836f,-1.5000f,13.1348f,-1.5000f,13.1348f,-1.3958f,13.0860f,-1.3958f,13.0860f,-1.2917f,13.0371f,-1.2917f,13.0371f,-1.1875f,12.9883f,-1.1875f,
	12.9883f,-1.0833f,12.9395f,-1.0833f,12.9395f,-0.9792f,12.8906f,-0.9792f,12.8906f,-0.8750f,12.8418f,-0.8750f,12.8418f,-0.7708f,12.7930f,-0.7708f,12.7930f,-0.6667f,12.7442f,-0.6667f,12.7442f,-0.5625f,12.6953f,-0.5625f,12.6953f,-0.4583f,12.6465f,-0.4583f,12.6465f,-0.3542f,
	12.5977f,-0.3542f,12.5977f,-0.2500f,12.5488f,-0.2500f,12.5488f,-0.1458f,12.5000f,-0.1458f,12.5000f,-0.0417f,12.4512f,-0.0417f,12.4512f,0.0625f,12.4024f,0.0625f,12.4024f,0.1667f,12.3535f,0.1667f,12.3535f,0.2708f,12.3047f,0.2708f,12.3047f,0.3750f,12.2559f,0.3750f,
	12.2559f,0.4792f,12.2071f,0.4792f,12.2071f,0.5833f,12.1582f,0.5833f,12.1582f,0.6875f,12.1094f,0.6875f,12.1094f,0.7917f,12.0606f,0.7917f,12.0606f,0.8958f,12.0117f,0.8958f,12.0117f,1.0000f,11.9629f,1.0000f,14.3067f,-4.0000f,14.2578f,-4.0000f,14.2578f,-3.8958f,
	14.2090f,-3.8958f,14.2090f,-3.7917f,14.1602f,-3.7917f,14.1602f,-3.6875f,14.1114f,-3.6875f,14.1114f,-3.5833f,14.0625f,-3.5833f,14.0625f,-3.4792f,14.0137f,-3.4792f,14.0137f,-3.3750f,13.9649f,-3.3750f,13.9649f,-3.2708f,13.9160f,-3.2708f,13.9160f,-3.1667f,13.8672f,-3.1667f,
	13.8672f,-3.0625f,13.8184f,-3.0625f,13.8184f,-2.9583f,13.7696f,-2.9583f,13.7696f,-2.8542f,13.7207f,-2.8542f,13.7207f,-2.7500f,13.6719f,-2.7500f,13.6719f,-2.6458f,13.6231f,-2.6458f,13.6231f,-2.5417f,13.5742f,-2.5417f,13.5742f,-2.4375f,13.5254f,-2.4375f,13.5254f,-2.3333f,
	13.4766f,-2.3333f,13.4766f,-2.2292f,13.4278f,-2.2292f,13.4278f,-2.1250f,13.3789f,-2.1250f,13.3789f,-2.0208f,13.3301f,-2.0208f,13.3301f,-1.9167f,13.2813f,-1.9167f,13.2813f,-1.8125f,13.2324f,-1.8125f,13.2324f,-1.7083f,13.1836f,-1.7083f,13.1836f,-1.6042f,13.1348f,-1.6042f,
	13.1348f,-1.5000f,13.0860f,-1.5000f,13.0860f,-1.3958f,13.0371f,-1.3958f,13.0371f,-1.2917f,12.9883f,-1.2917f,12.9883f,-1.1875f,12.9395f,-1.1875f,12.9395f,-1.0833f,12.8906f,-1.0833f,12.8906f,-0.9792f,12.8418f,-0.9792f,12.8418f,-0.8750f,12.7930f,-0.8750f,12.7930f,-0.7708f,
	12.7442f,-0.7708f,12.7442f,-0.6667f,12.6953f,-0.6667f,12.6953f,-0.5625f,12.6465f,-0.5625f,12.6465f,-0.4583f,12.5977f,-0.4583f,12.5977f,-0.3542f,12.5488f,-0.3542f,12.5488f,-0.2500f,12.5000f,-0.2500f,12.5000f,-0.1458f,12.4512f,-0.1458f,12.4512f,-0.0417f,12.4024f,-0.0417f,
	12.4024f,0.0625f,12.3535f,0.0625f,12.3535f,0.1667f,12.3047f,0.1667f,12.3047f,0.2708f,12.2559f,0.2708f,12.2559f,0.3750f,12.2071f,0.3750f,12.2071f,0.4792f,12.1582f,0.4792f,12.1582f,0.5833f,12.1094f,0.5833f,12.1094f,0.6875f,12.0606f,0.6875f,12.0606f,0.7917f,
	12.0117f,0.7917f,12.0117f,0.8958f,11.9629f,0.8958f,11.9629f,1.0000f,11.9141f,1.0000f,14.2578f,-4.0000f,14.2090f,-4.0000f,14.2090f,-3.8958f,14.1602f,-3.8958f,14.1602f,-3.7917f,14.1114f,-3.7917f,14.1114f,-3.6875f,14.0625f,-3.6875f,14.0625f,-3.5833f,14.0137f,-3.5833f,
	14.0137f,-3.4792f,13.9649f,-3.4792f,13.9649f,-3.3750f,13.9160f,-3.3750f,13.9160f,-3.2708f,13.8672f,-3.2708f,13.8672f,-3.1667f,13.8184f,-3.1667f,13.8184f,-3.0625f,13.7696f,-3.0625f,13.7696f,-2.9583f,13.7207f,-2.9583f,13.7207f,-2.8542f,13.6719f,-2.8542f,13.6719f,-2.7500f,
	13.6231f,-2.7500f,13.6231f,-2.6458f,13.5742f,-2.6458f,13.5742f,-2.5417f,13.5254f,-2.5417f,13.5254f,-2.4375f,13.4766f,-2.4375f,13.4766f,-2.3333f,13.4278f,-2.3333f,13.4278f,-2.2292f,13.3789f,-2.2292f,13.3789f,-2.1250f,13.3301f,-2.1250f,13.3301f,-2.0208f,13.2813f,-2.0208f,
	13.2813f,-1.9167f,13.2324f,-1.9167f,13.2324f,-1.8125f,13.1836f,-1.8125f,13.1836f,-1.7083f,13.1348f,-1.7083f,13.1348f,-1.6042f,13.0860f,-1.6042f,13.0860f,-1.5000f,13.0371f,-1.5000f,13.0371f,-1.3958f,12.9883f,-1.3958f,12.9883f,-1.2917f,12.9395f,-1.2917f,12.9395f,-1.1875f,
	12.8906f,-1.1875f,12.8906f,-1.0833f,12.8418f,-1.0833f,12.8418f,-0.9792f,12.7930f,-0.9792f,12.7930f,-0.8750f,12.7442f,-0.8750f,12.7442f,-0.7708f,12.6953f,-0.7708f,12.6953f,-0.6667f,12.6465f,-0.6667f,12.6465f,-0.5625f,12.5977f,-0.5625f,12.5977f,-0.4583f,12.5488f,-0.4583f,
	12.5488f,-0.3542f,12.5000f,-0.3542f,12.5000f,-0.2500f,12.4512f,-0.2500f,12.4512f,-0.1458f,12.4024f,-0.1458f,12.4024f,-0.0417f,12.3535f,-0.0417f,12.3535f,0.0625f,12.3047f,0.0625f,12.3047f,0.1667f,12.2559f,0.1667f,12.2559f,0.2708f,12.2071f,0.2708f,12.2071f,0.3750f,
	12.1582f,0.3750f,12.1582f,0.4792f,12.1094f,0.4792f,12.1094f,0.5833f,12.0606f,0.5833f,12.0606f,0.6875f,12.0117f,0.6875f,12.0117f,0.7917f,11.9629f,0.7917f,11.9629f,0.8958f,11.9141f,0.8958f,11.9141f,1.0000f,11.8653f,1.0000f,14.2090f,-4.0000f,14.1602f,-4.0000f,
	14.1602f,-3.8958f,14.1114f,-3.8958f,14.1114f,-3.7917f,14.0625f,-3.7917f,14.0625f,-3.6875f,14.0137f,-3.6875f,14.0137f,-3.5833f,13.9649f,-3.5833f,13.9649f,-3.4792f,13.9160f,-3.4792f,13.9160f,-3.3750f,13.8672f,-3.3750f,13.8672f,-3.2708f,13.8184f,-3.2708f,13.8184f,-3.1667f,
	13.7696f,-3.1667f,13.7696f,-3.0625f,13.7207f,-3.0625f,13.7207f,-2.9583f,13.6719f,-2.9583f,13.6719f,-2.8542f,13.6231f,-2.8542f,13.6231f,-2.7500f,13.5742f,-2.7500f,13.5742f,-2.6458f,13.5254f,-2.6458f,13.5254f,-2.5417f,13.4766f,-2.5417f,13.4766f,-2.4375f,13.4278f,-2.4375f,
	13.4278f,-2.3333f,13.3789f,-2.3333f,13.3789f,-2.2292f,13.3301f,-2.2292f,13.3301f,-2.1250f,13.2813f,-2.1250f,13.2813f,-2.0208f,13.2324f,-2.0208f,13.2324f,-1.9167f,13.1836f,-1.9167f,13.1836f,-1.8125f,13.1348f,-1.8125f,13.1348f,-1.7083f,13.0860f,-1.7083f,13.0860f,-1.6042f,
	13.0371f,-1.6042f,13.0371f,-1.5000f,12.9883f,-1.5000f,12.9883f,-1.3958f,12.9395f,-1.3958f,12.9395f,-1.2917f,12.8906f,-1.2917f,12.8906f,-1.1875f,12.8418f,-1.1875f,12.8418f,-1.0833f,12.7930f,-1.0833f,12.7930f,-0.9792f,12.7442f,-0.9792f,12.7442f,-0.8750f,12.6953f,-0.8750f,
	12.6953f,-0.7708f,12.6465f,-0.7708f,12.6465f,-0.6667f,12.5977f,-0.6667f,12.5977f,-0.5625f,12.5488f,-0.5625f,12.5488f,-0.4583f,12.5000f,-0.4583f,12.5000f,-0.3542f,12.4512f,-0.3542f,12.4512f,-0.2500f,12.4024f,-0.2500f,12.4024f,-0.1458f,12.3535f,-0.1458f,12.3535f,-0.0417f,
	12.3047f,-0.0417f,12.3047f,0.0625f,12.2559f,0.0625f,12.2559f,0.1667f,12.2071f,0.1667f,12.2071f,0.2708f,12.1582f,0.2708f,12.1582f,0.3750f,12.1094f,0.3750f,12.1094f,0.4792f,12.0606f,0.4792f,12.0606f,0.5833f,12.0117f,0.5833f,12.0117f,0.6875f,11.9629f,0.6875f,
	11.9629f,0.7917f,11.9141f,0.7917f,11.9141f,0.8958f,11.8653f,0.8958f,11.8653f,1.0000f,11.8164f,1.0000f,14.1602f,-4.0000f,14.1114f,-4.0000f,14.1114f,-3.8958f,14.0625f,-3.8958f,14.0625f,-3.7917f,14.0137f,-3.7917f,14.0137f,-3.6875f,13.9649f,-3.6875f,13.9649f,-3.5833f,
	13.9160f,-3.5833f,13.9160f,-3.4792f,13.8672f,-3.4792f,13.8672f,-3.3750f,13.8184f,-3.3750f,13.8184f,-3.2708f,13.7696f,-3.2708f,13.7696f,-3.1667f,13.7207f,-3.1667f,13.7207f,-3.0625f,13.6719f,-3.0625f,13.6719f,-2.9583f,13.6231f,-2.9583f,13.6231f,-2.8542f,13.5742f,-2.8542f,
	13.5742f,-2.7500f,13.5254f,-2.7500f,13.5254f,-2.6458f,13.4766f,-2.6458f,13.4766f,-2.5417f,13.4278f,-2.5417f,13.4278f,-2.4375f,13.3789f,-2.4375f,13.3789f,-2.3333f,13.3301f,-2.3333f,13.3301f,-2.2292f,13.2813f,-2.2292f,13.2813f,-2.1250f,13.2324f,-2.1250f,13.2324f,-2.0208f,
	13.1836f,-2.0208f,13.1836f,-1.9167f,13.1348f,-1.9167f,13.1348f,-1.8125f,13.0860f,-1.8125f,13.0860f,-1.7083f,13.0371f,-1.7083f,13.0371f,-1.6042f,12.9883f,-1.6042f,12.9883f,-1.5000f,12.9395f,-1.5000f,12.9395f,-1.3958f,12.8906f,-1.3958f,12.8906f,-1.2917f,12.8418f,-1.2917f,
	12.8418f,-1.1875f,12.7930f,-1.1875f,12.7930f,-1.0833f,12.7442f,-1.0833f,12.7442f,-0.9792f,12.6953f,-0.9792f,12.6953f,-0.8750f,12.6465f,-0.8750f,12.6465f,-0.7708f,12.5977f,-0.7708f,12.5977f,-0.6667f,12.5488f,-0.6667f,12.5488f,-0.5625f,12.5000f,-0.5625f,12.5000f,-0.4583f,
	12.4512f,-0.4583f,12.4512f,-0.3542f,12.4024f,-0.3542f,12.4024f,-0.2500f,12.3535f,-0.2500f,12.3535f,-0.1458f,12.3047f,-0.1458f,12.3047f,-0.0417f,12.2559f,-0.0417f,12.2559f,0.0625f,12.2071f,0.0625f,12.2071f,0.1667f,12.1582f,0.1667f,12.1582f,0.2708f,12.1094f,0.2708f,
	12.1094f,0.3750f,12.0606f,0.3750f,12.0606f,0.4792f,12.0117f,0.4792f,12.0117f,0.5833f,11.9629f,0.5833f,11.9629f,0.6875f,11.9141f,0.6875f,11.9141f,0.7917f,11.8653f,0.7917f,11.8653f,0.8958f,11.8164f,0.8958f,11.8164f,1.0000f,11.7676f,1.0000f,14.1114f,-4.0000f,
	14.0625f,-4.0000f,14.0625f,-3.8958f,14.0137f,-3.8958f,14.0137f,-3.7917f,13.9649f,-3.7917f,13.9649f,-3.6875f,13.9160f,-3.6875f,13.9160f,-3.5833f,13.8672f,-3.5833f,13.8672f,-3.4792f,13.8184f,-3.4792f,13.8184f,-3.3750f,13.7696f,-3.3750f,13.7696f,-3.2708f,13.7207f,-3.2708f,
	13.7207f,-3.1667f,13.6719f,-3.1667f,13.6719f,-3.0625f,13.6231f,-3.0625f,13.6231f,-2.9583f,13.5742f,-2.9583f,13.5742f,-2.8542f,13.5254f,-2.8542f,13.5254f,-2.7500f,13.4766f,-2.7500f,13.4766f,-2.6458f,13.4278f,-2.6458f,13.4278f,-2.5417f,13.3789f,-2.5417f,13.3789f,-2.4375f,
	13.3301f,-2.4375f,13.3301f,-2.3333f,13.2813f,-2.3333f,13.2813f,-2.2292f,13.2324f,-2.2292f,13.2324f,-2.1250f,13.1836f,-2.1250f,13.1836f,-2.0208f,13.1348f,-2.0208f,13.1348f,-1.9167f,13.0860f,-1.9167f,13.0860f,-1.8125f,13.0371f,-1.8125f,13.0371f,-1.7083f,12.9883f,-1.7083f,
	12.9883f,-1.6042f,12.9395f,-1.6042f,12.9395f,-1.5000f,12.8906f,-1.5000f,12.8906f,-1.3958f,12.8418f,-1.3958f,12.8418f,-1.2917f,12.7930f,-1.2917f,12.7930f,-1.1875f,12.7442f,-1.1875f,12.7442f,-1.0833f,12.6953f,-1.0833f,12.6953f,-0.9792f,12.6465f,-0.9792f,12.6465f,-0.8750f,
	12.5977f,-0.8750f,12.5977f,-0.7708f,12.5488f,-0.7708f,12.5488f,-0.6667f,12.5000f,-0.6667f,12.5000f,-0.5625f,12.4512f,-0.5625f,12.4512f,-0.4583f,12.4024f,-0.4583f,12.4024f,-0.3542f,12.3535f,-0.3542f,12.3535f,-0.2500f,12.3047f,-0.2500f,12.3047f,-0.1458f,12.2559f,-0.1458f,
	12.2559f,-0.0417f,12.2071f,-0.0417f,12.2071f,0.0625f,12.1582f,0.0625f,12.1582f,0.1667f,12.1094f,0.1667f,12.1094f,0.2708f,12.0606f,0.2708f,12.0606f,0.3750f,12.0117f,0.3750f,12.0117f,0.4792f,11.9629f,0.4792f,11.9629f,0.5833f,11.9141f,0.5833f,11.9141f,0.6875f,
	11.8653f,0.6875f,11.8653f,0.7917f,11.8164f,0.7917f,11.8164f,0.8958f,11.7676f,0.8958f,11.7676f,1.0000f,11.7188f,1.0000f,14.0625f,-4.0000f,14.0137f,-4.0000f,14.0137f,-3.8958f,13.9649f,-3.8958f,13.9649f,-3.7917f,13.9160f,-3.7917f,13.9160f,-3.6875f,13.8672f,-3.6875f,
	13.8672f,-3.5833f,13.8184f,-3.5833f,13.8184f,-3.4792f,13.7696f,-3.4792f,13.7696f,-3.3750f,13.7207f,-3.3750f,13.7207f,-3.2708f,13.6719f,-3.2708f,13.6719f,-3.1667f,13.6231f,-3.1667f,13.6231f,-3.0625f,13.5742f,-3.0625f,13.5742f,-2.9583f,13.5254f,-2.9583f,13.5254f,-2.8542f,
	13.4766f,-2.8542f,13.4766f,-2.7500f,13.4278f,-2.7500f,13.4278f,-2.6458f,13.3789f,-2.6458f,13.3789f,-2.5417f,13.3301f,-2.5417f,13.3301f,-2.4375f,13.2813f,-2.4375f,13.2813f,-2.3333f,13.2324f,-2.3333f,13.2324f,-2.2292f,13.1836f,-2.2292f,13.1836f,-2.1250f,13.1348f,-2.1250f,
	13.1348f,-2.0208f,13.0860f,-2.0208f,13.0860f,-1.9167f,13.0371f,-1.9167f,13.0371f,-1.8125f,12.9883f,-1.8125f,12.9883f,-1.7083f,12.9395f,-1.7083f,12.9395f,-1.6042f,12.8906f,-1.6042f,12.8906f,-1.5000f,12.8418f,-1.5000f,12.8418f,-1.3958f,12.7930f,-1.3958f,12.7930f,-1.2917f,
	12.7442f,-1.2917f,12.7442f,-1.1875f,12.6953f,-1.1875f,12.6953f,-1.0833f,12.6465f,-1.0833f,12.6465f,-0.9792f,12.5977f,-0.9792f,12.5977f,-0.8750f,12.5488f,-0.8750f,12.5488f,-0.7708f,12.5000f,-0.7708f,12.5000f,-0.6667f,12.4512f,-0.6667f,12.4512f,-0.5625f,12.4024f,-0.5625f,
	12.4024f,-0.4583f,12.3535f,-0.4583f,12.3535f,-0.3542f,12.3047f,-0.3542f,12.3047f,-0.2500f,12.2559f,-0.2500f,12.2559f,-0.1458f,12.2071f,-0.1458f,12.2071f,-0.0417f,12.1582f,-0.0417f,12.1582f,0.0625f,12.1094f,0.0625f,12.1094f,0.1667f,12.0606f,0.1667f,12.0606f,0.2708f,
	12.0117f,0.2708f,12.0117f,0.3750f,11.9629f,0.3750f,11.9629f,0.4792f,11.9141f,0.4792f,11.9141f,0.5833f,11.8653f,0.5833f,11.8653f,0.6875f,11.8164f,0.6875f,11.8164f,0.7917f,11.7676f,0.7917f,11.7676f,0.8958f,11.7188f,0.8958f,11.7188f,1.0000f,11.6699f,1.0000f,
	14.0137f,-4.0000f,13.9649f,-4.0000f,13.9649f,-3.8958f,13.9160f,-3.8958f,13.9160f,-3.7917f,13.8672f,-3.7917f,13.8672f,-3.6875f,13.8184f,-3.6875f,13.8184f,-3.5833f,13.7696f,-3.5833f,13.7696f,-3.4792f,13.7207f,-3.4792f,13.7207f,-3.3750f,13.6719f,-3.3750f,13.6719f,-3.2708f,
	13.6231f,-3.2708f,13.6231f,-3.1667f,13.5742f,-3.1667f,13.5742f,-3.0625f,13.5254f,-3.0625f,13.5254f,-2.9583f,13.4766f,-2.9583f,13.4766f,-2.8542f,13.4278f,-2.8542f,13.4278f,-2.7500f,13.3789f,-2.7500f,13.3789f,-2.6458f,13.3301f,-2.6458f,13.3301f,-2.5417f,13.2813f,-2.5417f,
	13.2813f,-2.4375f,13.2324f,-2.4375f,13.2324f,-2.3333f,13.1836f,-2.3333f,13.1836f,-2.2292f,13.1348f,-2.2292f,13.1348f,-2.1250f,13.0860f,-2.1250f,13.0860f,-2.0208f,13.0371f,-2.0208f,13.0371f,-1.9167f,12.9883f,-1.9167f,12.9883f,-1.8125f,12.9395f,-1.8125f,12.9395f,-1.7083f,
	12.8906f,-1.7083f,12.8906f,-1.6042f,12.8418f,-1.6042f,12.8418f,-1.5000f,12.7930f,-1.5000f,12.7930f,-1.3958f,12.7442f,-1.3958f,12.7442f,-1.2917f,12.6953f,-1.2917f,12.6953f,-1.1875f,12.6465f,-1.1875f,12.6465f,-1.0833f,12.5977f,-1.0833f,12.5977f,-0.9792f,12.5488f,-0.9792f,
	12.5488f,-0.8750f,12.5000f,-0.8750f,12.5000f,-0.7708f,12.4512f,-0.7708f,12.4512f,-0.6667f,12.4024f,-0.6667f,12.4024f,-0.5625f,12.3535f,-0.5625f,12.3535f,-0.4583f,12.3047f,-0.4583f,12.3047f,-0.3542f,12.2559f,-0.3542f,12.2559f,-0.2500f,12.2071f,-0.2500f,12.2071f,-0.1458f,
	12.1582f,-0.1458f,12.1582f,-0.0417f,12.1094f,-0.0417f,12.1094f,0.0625f,12.0606f,0.0625f,12.0606f,0.1667f,12.0117f,0.1667f,12.0117f,0.2708f,11.9629f,0.2708f,11.9629f,0.3750f,11.9141f,0.3750f,11.9141f,0.4792f,11.8653f,0.4792f,11.8653f,0.5833f,11.8164f,0.5833f,
	11.8164f,0.6875f,11.7676f,0.6875f,11.7676f,0.7917f,11.7188f,0.7917f,11.7188f,0.8958f,11.6699f,0.8958f,11.6699f,1.0000f,11.6211f,1.0000f,13.9649f,-4.0000f,13.9160f,-4.0000f,13.9160f,-3.8958f,13.8672f,-3.8958f,13.8672f,-3.7917f,13.8184f,-3.7917f,13.8184f,-3.6875f,
	13.7696f,-3.6875f,13.7696f,-3.5833f,13.7207f,-3.5833f,13.7207f,-3.4792f,13.6719f,-3.4792f,13.6719f,-3.3750f,13.6231f,-3.3750f,13.6231f,-3.2708f,13.5742f,-3.2708f,13.5742f,-3.1667f,13.5254f,-3.1667f,13.5254f,-3.0625f,13.4766f,-3.0625f,13.4766f,-2.9583f,13.4278f,-2.9583f,
	13.4278f,-2.8542f,13.3789f,-2.8542f,13.3789f,-2.7500f,13.3301f,-2.7500f,13.3301f,-2.6458f,13.2813f,-2.6458f,13.2813f,-2.5417f,13.2324f,-2.5417f,13.2324f,-2.4375f,13.1836f,-2.4375f,13.1836f,-2.3333f,13.1348f,-2.3333f,13.1348f,-2.2292f,13.0860f,-2.2292f,13.0860f,-2.1250f,
	13.0371f,-2.1250f,13.0371f,-2.0208f,12.9883f,-2.0208f,12.9883f,-1.9167f,12.9395f,-1.9167f,12.9395f,-1.8125f,12.8906f,-1.8125f,12.8906f,-1.7083f,12.8418f,-1.7083f,12.8418f,-1.6042f,12.7930f,-1.6042f,12.7930f,-1.5000f,12.7442f,-1.5000f,12.7442f,-1.3958f,12.6953f,-1.3958f,
	12.6953f,-1.2917f,12.6465f,-1.2917f,12.6465f,-1.1875f,12.5977f,-1.1875f,12.5977f,-1.0833f,12.5488f,-1.0833f,12.5488f,-0.9792f,12.5000f,-0.9792f,12.5000f,-0.8750f,12.4512f,-0.8750f,12.4512f,-0.7708f,12.4024f,-0.7708f,12.4024f,-0.6667f,12.3535f,-0.6667f,12.3535f,-0.5625f,
	12.3047f,-0.5625f,12.3047f,-0.4583f,12.2559f,-0.4583f,12.2559f,-0.3542f,12.2071f,-0.3542f,12.2071f,-0.2500f,12.1582f,-0.2500f,12.1582f,-0.1458f,12.1094f,-0.1458f,12.1094f,-0.0417f,12.0606f,-0.0417f,12.0606f,0.0625f,12.0117f,0.0625f,12.0117f,0.1667f,11.9629f,0.1667f,
	11.9629f,0.2708f,11.9141f,0.2708f,11.9141f,0.3750f,11.8653f,0.3750f,11.8653f,0.4792f,11.8164f,0.4792f,11.8164f,0.5833f,11.7676f,0.5833f,11.7676f,0.6875f,11.7188f,0.6875f,11.7188f,0.7917f,11.6699f,0.7917f,11.6699f,0.8958f,11.6211f,0.8958f,11.6211f,1.0000f,
	11.5723f,1.0000f,13.9160f,-4.0000f,13.8672f,-4.0000f,13.8672f,-3.8958f,13.8184f,-3.8958f,13.8184f,-3.7917f,13.7696f,-3.7917f,13.7696f,-3.6875f,13.7207f,-3.6875f,13.7207f,-3.5833f,13.6719f,-3.5833f,13.6719f,-3.4792f,13.6231f,-3.4792f,13.6231f,-3.3750f,13.5742f,-3.3750f,
	13.5742f,-3.2708f,13.5254f,-3.2708f,13.5254f,-3.1667f,13.4766f,-3.1667f,13.4766f,-3.0625f,13.4278f,-3.0625f,13.4278f,-2.9583f,13.3789f,-2.9583f,13.3789f,-2.8542f,13.3301f,-2.8542f,13.3301f,-2.7500f,13.2813f,-2.7500f,13.2813f,-2.6458f,13.2324f,-2.6458f,13.2324f,-2.5417f,
	13.1836f,-2.5417f,13.1836f,-2.4375f,13.1348f,-2.4375f,13.1348f,-2.3333f,13.0860f,-2.3333f,13.0860f,-2.2292f,13.0371f,-2.2292f,13.0371f,-2.1250f,12.9883f,-2.1250f,12.9883f,-2.0208f,12.9395f,-2.0208f,12.9395f,-1.9167f,12.8906f,-1.9167f,12.8906f,-1.8125f,12.8418f,-1.8125f,
	12.8418f,-1.7083f,12.7930f,-1.7083f,12.7930f,-1.6042f,12.7442f,-1.6042f,12.7442f,-1.5000f,12.6953f,-1.5000f,12.6953f,-1.3958f,12.6465f,-1.3958f,12.6465f,-1.2917f,12.5977f,-1.2917f,12.5977f,-1.1875f,12.5488f,-1.1875f,12.5488f,-1.0833f,12.5000f,-1.0833f,12.5000f,-0.9792f,
	12.4512f,-0.9792f,12.4512f,-0.8750f,12.4024f,-0.8750f,12.4024f,-0.7708f,12.3535f,-0.7708f,12.3535f,-0.6667f,12.3047f,-0.6667f,12.3047f,-0.5625f,12.2559f,-0.5625f,12.2559f,-0.4583f,12.2071f,-0.4583f,12.2071f,-0.3542f,12.1582f,-0.3542f,12.1582f,-0.2500f,12.1094f,-0.2500f,
	12.1094f,-0.1458f,12.0606f,-0.1458f,12.0606f,-0.0417f,12.0117f,-0.0417f,12.0117f,0.0625f,11.9629f,0.0625f,11.9629f,0.1667f,11.9141f,0.1667f,11.9141f,0.2708f,11.8653f,0.2708f,11.8653f,0.3750f,11.8164f,0.3750f,11.8164f,0.4792f,11.7676f,0.4792f,11.7676f,0.5833f,
	11.7188f,0.5833f,11.7188f,0.6875f,11.6699f,0.6875f,11.6699f,0.7917f,11.6211f,0.7917f,11.6211f,0.8958f,11.5723f,0.8958f,11.5723f,1.0000f,11.5235f,1.0000f,13.8672f,-4.0000f,13.8184f,-4.0000f,13.8184f,-3.8958f,13.7696f,-3.8958f,13.7696f,-3.7917f,13.7207f,-3.7917f,
	13.7207f,-3.6875f,13.6719f,-3.6875f,13.6719f,-3.5833f,13.6231f,-3.5833f,13.6231f,-3.4792f,13.5742f,-3.4792f,13.5742f,-3.3750f,13.5254f,-3.3750f,13.5254f,-3.2708f,13.4766f,-3.2708f,13.4766f,-3.1667f,13.4278f,-3.1667f,13.4278f,-3.0625f,13.3789f,-3.0625f,13.3789f,-2.9583f,
	13.3301f,-2.9583f,13.3301f,-2.8542f,13.2813f,-2.8542f,13.2813f,-2.7500f,13.2324f,-2.7500f,13.2324f,-2.6458f,13.1836f,-2.6458f,13.1836f,-2.5417f,13.1348f,-2.5417f,13.1348f,-2.4375f,13.0860f,-2.4375f,13.0860f,-2.3333f,13.0371f,-2.3333f,13.0371f,-2.2292f,12.9883f,-2.2292f,
	12.9883f,-2.1250f,12.9395f,-2.1250f,12.9395f,-2.0208f,12.8906f,-2.0208f,12.8906f,-1.9167f,12.8418f,-1.9167f,12.8418f,-1.8125f,12.7930f,-1.8125f,12.7930f,-1.7083f,12.7442f,-1.7083f,12.7442f,-1.6042f,12.6953f,-1.6042f,12.6953f,-1.5000f,12.6465f,-1.5000f,12.6465f,-1.3958f,
	12.5977f,-1.3958f,12.5977f,-1.2917f,12.5488f,-1.2917f,12.5488f,-1.1875f,12.5000f,-1.1875f,12.5000f,-1.0833f,12.4512f,-1.0833f,12.4512f,-0.9792f,12.4024f,-0.9792f,12.4024f,-0.8750f,12.3535f,-0.8750f,12.3535f,-0.7708f,12.3047f,-0.7708f,12.3047f,-0.6667f,12.2559f,-0.6667f,
	12.2559f,-0.5625f,12.2071f,-0.5625f,12.2071f,-0.4583f,12.1582f,-0.4583f,12.1582f,-0.3542f,12.1094f,-0.3542f,12.1094f,-0.2500f,12.0606f,-0.2500f,12.0606f,-0.1458f,12.0117f,-0.1458f,12.0117f,-0.0417f,11.9629f,-0.0417f,11.9629f,0.0625f,11.9141f,0.0625f,11.9141f,0.1667f,
	11.8653f,0.1667f,11.8653f,0.2708f,11.8164f,0.2708f,11.8164f,0.3750f,11.7676f,0.3750f,11.7676f,0.4792f,11.7188f,0.4792f,11.7188f,0.5833f,11.6699f,0.5833f,11.6699f,0.6875f,11.6211f,0.6875f,11.6211f,0.7917f,11.5723f,0.7917f,11.5723f,0.8958f,11.5235f,0.8958f,
	11.5235f,1.0000f,11.4746f,1.0000f,13.8184f,-4.0000f,13.7696f,-4.0000f,13.7696f,-3.8958f,13.7207f,-3.8958f,13.7207f,-3.7917f,13.6719f,-3.7917f,13.6719f,-3.6875f,13.6231f,-3.6875f,13.6231f,-3.5833f,13.5742f,-3.5833f,13.5742f,-3.4792f,13.5254f,-3.4792f,13.5254f,-3.3750f,
	13.4766f,-3.3750f,13.4766f,-3.2708f,13.4278f,-3.2708f,13.4278f,-3.1667f,13.3789f,-3.1667f,13.3789f,-3.0625f,13.3301f,-3.0625f,13.3301f,-2.9583f,13.2813f,-2.9583f,13.2813f,-2.8542f,13.2324f,-2.8542f,13.2324f,-2.7500f,13.1836f,-2.7500f,13.1836f,-2.6458f,13.1348f,-2.6458f,
	13.1348f,-2.5417f,13.0860f,-2.5417f,13.0860f,-2.4375f,13.0371f,-2.4375f,13.0371f,-2.3333f,12.9883f,-2.3333f,12.9883f,-2.2292f,12.9395f,-2.2292f,12.9395f,-2.1250f,12.8906f,-2.1250f,12.8906f,-2.0208f,12.8418f,-2.0208f,12.8418f,-1.9167f,12.7930f,-1.9167f,12.7930f,-1.8125f,
	12.7442f,-1.8125f,12.7442f,-1.7083f,12.6953f,-1.7083f,12.6953f,-1.6042f,12.6465f,-1.6042f,12.6465f,-1.5000f,12.5977f,-1.5000f,12.5977f,-1.3958f,12.5488f,-1.3958f,12.5488f,-1.2917f,12.5000f,-1.2917f,12.5000f,-1.1875f,12.4512f,-1.1875f,12.4512f,-1.0833f,12.4024f,-1.0833f,
	12.4024f,-0.9792f,12.3535f,-0.9792f,12.3535f,-0.8750f,12.3047f,-0.8750f,12.3047f,-0.7708f,12.2559f,-0.7708f,12.2559f,-0.6667f,12.2071f,-0.6667f,12.2071f,-0.5625f,12.1582f,-0.5625f,12.1582f,-0.4583f,12.1094f,-0.4583f,12.1094f,-0.3542f,12.0606f,-0.3542f,12.0606f,-0.2500f,
	12.0117f,-0.2500f,12.0117f,-0.1458f,11.9629f,-0.1458f,11.9629f,-0.0417f,11.9141f,-0.0417f,11.9141f,0.0625f,11.8653f,0.0625f,11.8653f,0.1667f,11.8164f,0.1667f,11.8164f,0.2708f,11.7676f,0.2708f,11.7676f,0.3750f,11.7188f,0.3750f,11.7188f,0.4792f,11.6699f,0.4792f,
	11.6699f,0.5833f,11.6211f,0.5833f,11.6211f,0.6875f,11.5723f,0.6875f,11.5723f,0.7917f,11.5235f,0.7917f,11.5235f,0.8958f,11.4746f,0.8958f,11.4746f,1.0000f,11.4258f,1.0000f,13.7696f,-4.0000f,13.7207f,-4.0000f,13.7207f,-3.8958f,13.6719f,-3.8958f,13.6719f,-3.7917f,
	13.6231f,-3.7917f,13.6231f,-3.6875f,13.5742f,-3.6875f,13.5742f,-3.5833f,13.5254f,-3.5833f,13.5254f,-3.4792f,13.4766f,-3.4792f,13.4766f,-3.3750f,13.4278f,-3.3750f,13.4278f,-3.2708f,13.3789f,-3.2708f,13.3789f,-3.1667f,13.3301f,-3.1667f,13.3301f,-3.0625f,13.2813f,-3.0625f,
	13.2813f,-2.9583f,13.2324f,-2.9583f,13.2324f,-2.8542f,13.1836f,-2.8542f,13.1836f,-2.7500f,13.1348f,-2.7500f,13.1348f,-2.6458f,13.0860f,-2.6458f,13.0860f,-2.5417f,13.0371f,-2.5417f,13.0371f,-2.4375f,12.9883f,-2.4375f,12.9883f,-2.3333f,12.9395f,-2.3333f,12.9395f,-2.2292f,
	12.8906f,-2.2292f,12.8906f,-2.1250f,12.8418f,-2.1250f,12.8418f,-2.0208f,12.7930f,-2.0208f,12.7930f,-1.9167f,12.7442f,-1.9167f,12.7442f,-1.8125f,12.6953f,-1.8125f,12.6953f,-1.7083f,12.6465f,-1.7083f,12.6465f,-1.6042f,12.5977f,-1.6042f,12.5977f,-1.5000f,12.5488f,-1.5000f,
	12.5488f,-1.3958f,12.5000f,-1.3958f,12.5000f,-1.2917f,12.4512f,-1.2917f,12.4512f,-1.1875f,12.4024f,-1.1875f,12.4024f,-1.0833f,12.3535f,-1.0833f,12.3535f,-0.9792f,12.3047f,-0.9792f,12.3047f,-0.8750f,12.2559f,-0.8750f,12.2559f,-0.7708f,12.2071f,-0.7708f,12.2071f,-0.6667f,
	12.1582f,-0.6667f,12.1582f,-0.5625f,12.1094f,-0.5625f,12.1094f,-0.4583f,12.0606f,-0.4583f,12.0606f,-0.3542f,12.0117f,-0.3542f,12.0117f,-0.2500f,11.9629f,-0.2500f,11.9629f,-0.1458f,11.9141f,-0.1458f,11.9141f,-0.0417f,11.8653f,-0.0417f,11.8653f,0.0625f,11.8164f,0.0625f,
	11.8164f,0.1667f,11.7676f,0.1667f,11.7676f,0.2708f,11.7188f,0.2708f,11.7188f,0.3750f,11.6699f,0.3750f,11.6699f,0.4792f,11.6211f,0.4792f,11.6211f,0.5833f,11.5723f,0.5833f,11.5723f,0.6875f,11.5235f,0.6875f,11.5235f,0.7917f,11.4746f,0.7917f,11.4746f,0.8958f,
	11.4258f,0.8958f,11.4258f,1.0000f,11.3770f,1.0000f,13.7207f,-4.0000f,13.6719f,-4.0000f,13.6719f,-3.8958f,13.6231f,-3.8958f,13.6231f,-3.7917f,13.5742f,-3.7917f,13.5742f,-3.6875f,13.5254f,-3.6875f,13.5254f,-3.5833f,13.4766f,-3.5833f,13.4766f,-3.4792f,13.4278f,-3.4792f,
	13.4278f,-3.3750f,13.3789f,-3.3750f,13.3789f,-3.2708f,13.3301f,-3.2708f,13.3301f,-3.1667f,13.2813f,-3.1667f,13.2813f,-3.0625f,13.2324f,-3.0625f,13.2324f,-2.9583f,13.1836f,-2.9583f,13.1836f,-2.8542f,13.1348f,-2.8542f,13.1348f,-2.7500f,13.0860f,-2.7500f,13.0860f,-2.6458f,
	13.0371f,-2.6458f,13.0371f,-2.5417f,12.9883f,-2.5417f,12.9883f,-2.4375f,12.9395f,-2.4375f,12.9395f,-2.3333f,12.8906f,-2.3333f,12.8906f,-2.2292f,12.8418f,-2.2292f,12.8418f,-2.1250f,12.7930f,-2.1250f,12.7930f,-2.0208f,12.7442f,-2.0208f,12.7442f,-1.9167f,12.6953f,-1.9167f,
	12.6953f,-1.8125f,12.6465f,-1.8125f,12.6465f,-1.7083f,12.5977f,-1.7083f,12.5977f,-1.6042f,12.5488f,-1.6042f,12.5488f,-1.5000f,12.5000f,-1.5000f,12.5000f,-1.3958f,12.4512f,-1.3958f,12.4512f,-1.2917f,12.4024f,-1.2917f,12.4024f,-1.1875f,12.3535f,-1.1875f,12.3535f,-1.0833f,
	12.3047f,-1.0833f,12.3047f,-0.9792f,12.2559f,-0.9792f,12.2559f,-0.8750f,12.2071f,-0.8750f,12.2071f,-0.7708f,12.1582f,-0.7708f,12.1582f,-0.6667f,12.1094f,-0.6667f,12.1094f,-0.5625f,12.0606f,-0.5625f,12.0606f,-0.4583f,12.0117f,-0.4583f,12.0117f,-0.3542f,11.9629f,-0.3542f,
	11.9629f,-0.2500f,11.9141f,-0.2500f,11.9141f,-0.1458f,11.8653f,-0.1458f,11.8653f,-0.0417f,11.8164f,-0.0417f,11.8164f,0.0625f,11.7676f,0.0625f,11.7676f,0.1667f,11.7188f,0.1667f,11.7188f,0.2708f,11.6699f,0.2708f,11.6699f,0.3750f,11.6211f,0.3750f,11.6211f,0.4792f,
	11.5723f,0.4792f,11.5723f,0.5833f,11.5235f,0.5833f,11.5235f,0.6875f,11.4746f,0.6875f,11.4746f,0.7917f,11.4258f,0.7917f,11.4258f,0.8958f,11.3770f,0.8958f,11.3770f,1.0000f,11.3281f,1.0000f,13.6719f,-4.0000f,13.6231f,-4.0000f,13.6231f,-3.8958f,13.5742f,-3.8958f,
	13.5742f,-3.7917f,13.5254f,-3.7917f,13.5254f,-3.6875f,13.4766f,-3.6875f,13.4766f,-3.5833f,13.4278f,-3.5833f,13.4278f,-3.4792f,13.3789f,-3.4792f,13.3789f,-3.3750f,13.3301f,-3.3750f,13.3301f,-3.2708f,13.2813f,-3.2708f,13.2813f,-3.1667f,13.2324f,-3.1667f,13.2324f,-3.0625f,
	13.1836f,-3.0625f,13.1836f,-2.9583f,13.1348f,-2.9583f,13.1348f,-2.8542f,13.0860f,-2.8542f,13.0860f,-2.7500f,13.0371f,-2.7500f,13.0371f,-2.6458f,12.9883f,-2.6458f,12.9883f,-2.5417f,12.9395f,-2.5417f,12.9395f,-2.4375f,12.8906f,-2.4375f,12.8906f,-2.3333f,12.8418f,-2.3333f,
	12.8418f,-2.2292f,12.7930f,-2.2292f,12.7930f,-2.1250f,12.7442f,-2.1250f,12.7442f,-2.0208f,12.6953f,-2.0208f,12.6953f,-1.9167f,12.6465f,-1.9167f,12.6465f,-1.8125f,12.5977f,-1.8125f,12.5977f,-1.7083f,12.5488f,-1.7083f,12.5488f,-1.6042f,12.5000f,-1.6042f,12.5000f,-1.5000f,
	12.4512f,-1.5000f,12.4512f,-1.3958f,12.4024f,-1.3958f,12.4024f,-1.2917f,12.3535f,-1.2917f,12.3535f,-1.1875f,12.3047f,-1.1875f,12.3047f,-1.0833f,12.2559f,-1.0833f,12.2559f,-0.9792f,12.2071f,-0.9792f,12.2071f,-0.8750f,12.1582f,-0.8750f,12.1582f,-0.7708f,12.1094f,-0.7708f,
	12.1094f,-0.6667f,12.0606f,-0.6667f,12.0606f,-0.5625f,12.0117f,-0.5625f,12.0117f,-0.4583f,11.9629f,-0.4583f,11.9629f,-0.3542f,11.9141f,-0.3542f,11.9141f,-0.2500f,11.8653f,-0.2500f,11.8653f,-0.1458f,11.8164f,-0.1458f,11.8164f,-0.0417f,11.7676f,-0.0417f,11.7676f,0.0625f,
	11.7188f,0.0625f,11.7188f,0.1667f,11.6699f,0.1667f,11.6699f,0.2708f,11.6211f,0.2708f,11.6211f,0.3750f,11.5723f,0.3750f,11.5723f,0.4792f,11.5235f,0.4792f,11.5235f,0.5833f,11.4746f,0.5833f,11.4746f,0.6875f,11.4258f,0.6875f,11.4258f,0.7917f,11.3770f,0.7917f,
	11.3770f,0.8958f,11.3281f,0.8958f,11.3281f,1.0000f,11.2793f,1.0000f,13.6231f,-4.0000f,13.5742f,-4.0000f,13.5742f,-3.8958f,13.5254f,-3.8958f,13.5254f,-3.7917f,13.4766f,-3.7917f,13.4766f,-3.6875f,13.4278f,-3.6875f,13.4278f,-3.5833f,13.3789f,-3.5833f,13.3789f,-3.4792f,
	13.3301f,-3.4792f,13.3301f,-3.3750f,13.2813f,-3.3750f,13.2813f,-3.2708f,13.2324f,-3.2708f,13.2324f,-3.1667f,13.1836f,-3.1667f,13.1836f,-3.0625f,13.1348f,-3.0625f,13.1348f,-2.9583f,13.0860f,-2.9583f,13.0860f,-2.8542f,13.0371f,-2.8542f,13.0371f,-2.7500f,12.9883f,-2.7500f,
	12.9883f,-2.6458f,12.9395f,-2.6458f,12.9395f,-2.5417f,12.8906f,-2.5417f,12.8906f,-2.4375f,12.8418f,-2.4375f,12.8418f,-2.3333f,12.7930f,-2.3333f,12.7930f,-2.2292f,12.7442f,-2.2292f,12.7442f,-2.1250f,12.6953f,-2.1250f,12.6953f,-2.0208f,12.6465f,-2.0208f,12.6465f,-1.9167f,
	12.5977f,-1.9167f,12.5977f,-1.8125f,12.5488f,-1.8125f,12.5488f,-1.7083f,12.5000f,-1.7083f,12.5000f,-1.6042f,12.4512f,-1.6042f,12.4512f,-1.5000f,12.4024f,-1.5000f,12.4024f,-1.3958f,12.3535f,-1.3958f,12.3535f,-1.2917f,12.3047f,-1.2917f,12.3047f,-1.1875f,12.2559f,-1.1875f,
	12.2559f,-1.0833f,12.2071f,-1.0833f,12.2071f,-0.9792f,12.1582f,-0.9792f,12.1582f,-0.8750f,12.1094f,-0.8750f,12.1094f,-0.7708f,12.0606f,-0.7708f,12.0606f,-0.6667f,12.0117f,-0.6667f,12.0117f,-0.5625f,11.9629f,-0.5625f,11.9629f,-0.4583f,11.9141f,-0.4583f,11.9141f,-0.3542f,
	11.8653f,-0.3542f,11.8653f,-0.2500f,11.8164f,-0.2500f,11.8164f,-0.1458f,11.7676f,-0.1458f,11.7676f,-0.0417f,11.7188f,-0.0417f,11.7188f,0.0625f,11.6699f,0.0625f,11.6699f,0.1667f,11.6211f,0.1667f,11.6211f,0.2708f,11.5723f,0.2708f,11.5723f,0.3750f,11.5235f,0.3750f,
	11.5235f,0.4792f,11.4746f,0.4792f,11.4746f,0.5833f,11.4258f,0.5833f,11.4258f,0.6875f,11.3770f,0.6875f,11.3770f,0.7917f,11.3281f,0.7917f,11.3281f,0.8958f,11.2793f,0.8958f,11.2793f,1.0000f,11.2305f,1.0000f,13.5742f,-4.0000f,13.5254f,-4.0000f,13.5254f,-3.8958f,
	13.4766f,-3.8958f,13.4766f,-3.7917f,13.4278f,-3.7917f,13.4278f,-3.6875f,13.3789f,-3.6875f,13.3789f,-3.5833f,13.3301f,-3.5833f,13.3301f,-3.4792f,13.2813f,-3.4792f,13.2813f,-3.3750f,13.2324f,-3.3750f,13.2324f,-3.2708f,13.1836f,-3.2708f,13.1836f,-3.1667f,13.1348f,-3.1667f,
	13.1348f,-3.0625f,13.0860f,-3.0625f,13.0860f,-2.9583f,13.0371f,-2.9583f,13.0371f,-2.8542f,12.9883f,-2.8542f,12.9883f,-2.7500f,12.9395f,-2.7500f,12.9395f,-2.6458f,12.8906f,-2.6458f,12.8906f,-2.5417f,12.8418f,-2.5417f,12.8418f,-2.4375f,12.7930f,-2.4375f,12.7930f,-2.3333f,
	12.7442f,-2.3333f,12.7442f,-2.2292f,12.6953f,-2.2292f,12.6953f,-2.1250f,12.6465f,-2.1250f,12.6465f,-2.0208f,12.5977f,-2.0208f,12.5977f,-1.9167f,12.5488f,-1.9167f,12.5488f,-1.8125f,12.5000f,-1.8125f,12.5000f,-1.7083f,12.4512f,-1.7083f,12.4512f,-1.6042f,12.4024f,-1.6042f,
	12.4024f,-1.5000f,12.3535f,-1.5000f,12.3535f,-1.3958f,12.3047f,-1.3958f,12.3047f,-1.2917f,12.2559f,-1.2917f,12.2559f,-1.1875f,12.2071f,-1.1875f,12.2071f,-1.0833f,12.1582f,-1.0833f,12.1582f,-0.9792f,12.1094f,-0.9792f,12.1094f,-0.8750f,12.0606f,-0.8750f,12.0606f,-0.7708f,
	12.0117f,-0.7708f,12.0117f,-0.6667f,11.9629f,-0.6667f,11.9629f,-0.5625f,11.9141f,-0.5625f,11.9141f,-0.4583f,11.8653f,-0.4583f,11.8653f,-0.3542f,11.8164f,-0.3542f,11.8164f,-0.2500f,11.7676f,-0.2500f,11.7676f,-0.1458f,11.7188f,-0.1458f,11.7188f,-0.0417f,11.6699f,-0.0417f,
	11.6699f,0.0625f,11.6211f,0.0625f,11.6211f,0.1667f,11.5723f,0.1667f,11.5723f,0.2708f,11.5235f,0.2708f,11.5235f,0.3750f,11.4746f,0.3750f,11.4746f,0.4792f,11.4258f,0.4792f,11.4258f,0.5833f,11.3770f,0.5833f,11.3770f,0.6875f,11.3281f,0.6875f,11.3281f,0.7917f,
	11.2793f,0.7917f,11.2793f,0.8958f,11.2305f,0.8958f,11.2305f,1.0000f,11.1817f,1.0000f,13.5254f,-4.0000f,13.4766f,-4.0000f,13.4766f,-3.8958f,13.4278f,-3.8958f,13.4278f,-3.7917f,13.3789f,-3.7917f,13.3789f,-3.6875f,13.3301f,-3.6875f,13.3301f,-3.5833f,13.2813f,-3.5833f,
	13.2813f,-3.4792f,13.2324f,-3.4792f,13.2324f,-3.3750f,13.1836f,-3.3750f,13.1836f,-3.2708f,13.1348f,-3.2708f,13.1348f,-3.1667f,13.0860f,-3.1667f,13.0860f,-3.0625f,13.0371f,-3.0625f,13.0371f,-2.9583f,12.9883f,-2.9583f,12.9883f,-2.8542f,12.9395f,-2.8542f,12.9395f,-2.7500f,
	12.8906f,-2.7500f,12.8906f,-2.6458f,12.8418f,-2.6458f,12.8418f,-2.5417f,12.7930f,-2.5417f,12.7930f,-2.4375f,12.7442f,-2.4375f,12.7442f,-2.3333f,12.6953f,-2.3333f,12.6953f,-2.2292f,12.6465f,-2.2292f,12.6465f,-2.1250f,12.5977f,-2.1250f,12.5977f,-2.0208f,12.5488f,-2.0208f,
	12.5488f,-1.9167f,12.5000f,-1.9167f,12.5000f,-1.8125f,12.4512f,-1.8125f,12.4512f,-1.7083f,12.4024f,-1.7083f,12.4024f,-1.6042f,12.3535f,-1.6042f,12.3535f,-1.5000f,12.3047f,-1.5000f,12.3047f,-1.3958f,12.2559f,-1.3958f,12.2559f,-1.2917f,12.2071f,-1.2917f,12.2071f,-1.1875f,
	12.1582f,-1.1875f,12.1582f,-1.0833f,12.1094f,-1.0833f,12.1094f,-0.9792f,12.0606f,-0.9792f,12.0606f,-0.8750f,12.0117f,-0.8750f,12.0117f,-0.7708f,11.9629f,-0.7708f,11.9629f,-0.6667f,11.9141f,-0.6667f,11.9141f,-0.5625f,11.8653f,-0.5625f,11.8653f,-0.4583f,11.8164f,-0.4583f,
	11.8164f,-0.3542f,11.7676f,-0.3542f,11.7676f,-0.2500f,11.7188f,-0.2500f,11.7188f,-0.1458f,11.6699f,-0.1458f,11.6699f,-0.0417f,11.6211f,-0.0417f,11.6211f,0.0625f,11.5723f,0.0625f,11.5723f,0.1667f,11.5235f,0.1667f,11.5235f,0.2708f,11.4746f,0.2708f,11.4746f,0.3750f,
	11.4258f,0.3750f,11.4258f,0.4792f,11.3770f,0.4792f,11.3770f,0.5833f,11.3281f,0.5833f,11.3281f,0.6875f,11.2793f,0.6875f,11.2793f,0.7917f,11.2305f,0.7917f,11.2305f,0.8958f,11.1817f,0.8958f,11.1817f,1.0000f,11.1328f,1.0000f,13.4766f,-4.0000f,13.4278f,-4.0000f,
	13.4278f,-3.8958f,13.3789f,-3.8958f,13.3789f,-3.7917f,13.3301f,-3.7917f,13.3301f,-3.6875f,13.2813f,-3.6875f,13.2813f,-3.5833f,13.2324f,-3.5833f,13.2324f,-3.4792f,13.1836f,-3.4792f,13.1836f,-3.3750f,13.1348f,-3.3750f,13.1348f,-3.2708f,13.0860f,-3.2708f,13.0860f,-3.1667f,
	13.0371f,-3.1667f,13.0371f,-3.0625f,12.9883f,-3.0625f,12.9883f,-2.9583f,12.9395f,-2.9583f,12.9395f,-2.8542f,12.8906f,-2.8542f,12.8906f,-2.7500f,12.8418f,-2.7500f,12.8418f,-2.6458f,12.7930f,-2.6458f,12.7930f,-2.5417f,12.7442f,-2.5417f,12.7442f,-2.4375f,12.6953f,-2.4375f,
	12.6953f,-2.3333f,12.6465f,-2.3333f,12.6465f,-2.2292f,12.5977f,-2.2292f,12.5977f,-2.1250f,12.5488f,-2.1250f,12.5488f,-2.0208f,12.5000f,-2.0208f,12.5000f,-1.9167f,12.4512f,-1.9167f,12.4512f,-1.8125f,12.4024f,-1.8125f,12.4024f,-1.7083f,12.3535f,-1.7083f,12.3535f,-1.6042f,
	12.3047f,-1.6042f,12.3047f,-1.5000f,12.2559f,-1.5000f,12.2559f,-1.3958f,12.2071f,-1.3958f,12.2071f,-1.2917f,12.1582f,-1.2917f,12.1582f,-1.1875f,12.1094f,-1.1875f,12.1094f,-1.0833f,12.0606f,-1.0833f,12.0606f,-0.9792f,12.0117f,-0.9792f,12.0117f,-0.8750f,11.9629f,-0.8750f,
	11.9629f,-0.7708f,11.9141f,-0.7708f,11.9141f,-0.6667f,11.8653f,-0.6667f,11.8653f,-0.5625f,11.8164f,-0.5625f,11.8164f,-0.4583f,11.7676f,-0.4583f,11.7676f,-0.3542f,11.7188f,-0.3542f,11.7188f,-0.2500f,11.6699f,-0.2500f,11.6699f,-0.1458f,11.6211f,-0.1458f,11.6211f,-0.0417f,
	11.5723f,-0.0417f,11.5723f,0.0625f,11.5235f,0.0625f,11.5235f,0.1667f,11.4746f,0.1667f,11.4746f,0.2708f,11.4258f,0.2708f,11.4258f,0.3750f,11.3770f,0.3750f,11.3770f,0.4792f,11.3281f,0.4792f,11.3281f,0.5833f,11.2793f,0.5833f,11.2793f,0.6875f,11.2305f,0.6875f,
	11.2305f,0.7917f,11.1817f,0.7917f,11.1817f,0.8958f,11.1328f,0.8958f,11.1328f,1.0000f,11.0840f,1.0000f,13.4278f,-4.0000f,13.3789f,-4.0000f,13.3789f,-3.8958f,13.3301f,-3.8958f,13.3301f,-3.7917f,13.2813f,-3.7917f,13.2813f,-3.6875f,13.2324f,-3.6875f,13.2324f,-3.5833f,
	13.1836f,-3.5833f,13.1836f,-3.4792f,13.1348f,-3.4792f,13.1348f,-3.3750f,13.0860f,-3.3750f,13.0860f,-3.2708f,13.0371f,-3.2708f,13.0371f,-3.1667f,12.9883f,-3.1667f,12.9883f,-3.0625f,12.9395f,-3.0625f,12.9395f,-2.9583f,12.8906f,-2.9583f,12.8906f,-2.8542f,12.8418f,-2.8542f,
	12.8418f,-2.7500f,12.7930f,-2.7500f,12.7930f,-2.6458f,12.7442f,-2.6458f,12.7442f,-2.5417f,12.6953f,-2.5417f,12.6953f,-2.4375f,12.6465f,-2.4375f,12.6465f,-2.3333f,12.5977f,-2.3333f,12.5977f,-2.2292f,12.5488f,-2.2292f,12.5488f,-2.1250f,12.5000f,-2.1250f,12.5000f,-2.0208f,
	12.4512f,-2.0208f,12.4512f,-1.9167f,12.4024f,-1.9167f,12.4024f,-1.8125f,12.3535f,-1.8125f,12.3535f,-1.7083f,12.3047f,-1.7083f,12.3047f,-1.6042f,12.2559f,-1.6042f,12.2559f,-1.5000f,12.2071f,-1.5000f,12.2071f,-1.3958f,12.1582f,-1.3958f,12.1582f,-1.2917f,12.1094f,-1.2917f,
	12.1094f,-1.1875f,12.0606f,-1.1875f,12.0606f,-1.0833f,12.0117f,-1.0833f,12.0117f,-0.9792f,11.9629f,-0.9792f,11.9629f,-0.8750f,11.9141f,-0.8750f,11.9141f,-0.7708f,11.8653f,-0.7708f,11.8653f,-0.6667f,11.8164f,-0.6667f,11.8164f,-0.5625f,11.7676f,-0.5625f,11.7676f,-0.4583f,
	11.7188f,-0.4583f,11.7188f,-0.3542f,11.6699f,-0.3542f,11.6699f,-0.2500f,11.6211f,-0.2500f,11.6211f,-0.1458f,11.5723f,-0.1458f,11.5723f,-0.0417f,11.5235f,-0.0417f,11.5235f,0.0625f,11.4746f,0.0625f,11.4746f,0.1667f,11.4258f,0.1667f,11.4258f,0.2708f,11.3770f,0.2708f,
	11.3770f,0.3750f,11.3281f,0.3750f,11.3281f,0.4792f,11.2793f,0.4792f,11.2793f,0.5833f,11.2305f,0.5833f,11.2305f,0.6875f,11.1817f,0.6875f,11.1817f,0.7917f,11.1328f,0.7917f,11.1328f,0.8958f,11.0840f,0.8958f,11.0840f,1.0000f,11.0352f,1.0000f,13.3789f,-4.0000f,
	13.3301f,-4.0000f,13.3301f,-3.8958f,13.2813f,-3.8958f,13.2813f,-3.7917f,13.2324f,-3.7917f,13.2324f,-3.6875f,13.1836f,-3.6875f,13.1836f,-3.5833f,13.1348f,-3.5833f,13.1348f,-3.4792f,13.0860f,-3.4792f,13.0860f,-3.3750f,13.0371f,-3.3750f,13.0371f,-3.2708f,12.9883f,-3.2708f,
	12.9883f,-3.1667f,12.9395f,-3.1667f,12.9395f,-3.0625f,12.8906f,-3.0625f,12.8906f,-2.9583f,12.8418f,-2.9583f,12.8418f,-2.8542f,12.7930f,-2.8542f,12.7930f,-2.7500f,12.7442f,-2.7500f,12.7442f,-2.6458f,12.6953f,-2.6458f,12.6953f,-2.5417f,12.6465f,-2.5417f,12.6465f,-2.4375f,
	12.5977f,-2.4375f,12.5977f,-2.3333f,12.5488f,-2.3333f,12.5488f,-2.2292f,12.5000f,-2.2292f,12.5000f,-2.1250f,12.4512f,-2.1250f,12.4512f,-2.0208f,12.4024f,-2.0208f,12.4024f,-1.9167f,12.3535f,-1.9167f,12.3535f,-1.8125f,12.3047f,-1.8125f,12.3047f,-1.7083f,12.2559f,-1.7083f,
	12.2559f,-1.6042f,12.2071f,-1.6042f,12.2071f,-1.5000f,12.1582f,-1.5000f,12.1582f,-1.3958f,12.1094f,-1.3958f,12.1094f,-1.2917f,12.0606f,-1.2917f,12.0606f,-1.1875f,12.0117f,-1.1875f,12.0117f,-1.0833f,11.9629f,-1.0833f,11.9629f,-0.9792f,11.9141f,-0.9792f,11.9141f,-0.8750f,
	11.8653f,-0.8750f,11.8653f,-0.7708f,11.8164f,-0.7708f,11.8164f,-0.6667f,11.7676f,-0.6667f,11.7676f,-0.5625f,11.7188f,-0.5625f,11.7188f,-0.4583f,11.6699f,-0.4583f,11.6699f,-0.3542f,11.6211f,-0.3542f,11.6211f,-0.2500f,11.5723f,-0.2500f,11.5723f,-0.1458f,11.5235f,-0.1458f,
	11.5235f,-0.0417f,11.4746f,-0.0417f,11.4746f,0.0625f,11.4258f,0.0625f,11.4258f,0.1667f,11.3770f,0.1667f,11.3770f,0.2708f,11.3281f,0.2708f,11.3281f,0.3750f,11.2793f,0.3750f,11.2793f,0.4792f,11.2305f,0.4792f,11.2305f,0.5833f,11.1817f,0.5833f,11.1817f,0.6875f,
	11.1328f,0.6875f,11.1328f,0.7917f,11.0840f,0.7917f,11.0840f,0.8958f,11.0352f,0.8958f,11.0352f,1.0000f,10.9863f,1.0000f,13.3301f,-4.0000f,13.2813f,-4.0000f,13.2813f,-3.8958f,13.2324f,-3.8958f,13.2324f,-3.7917f,13.1836f,-3.7917f,13.1836f,-3.6875f,13.1348f,-3.6875f,
	13.1348f,-3.5833f,13.0860f,-3.5833f,13.0860f,-3.4792f,13.0371f,-3.4792f,13.0371f,-3.3750f,12.9883f,-3.3750f,12.9883f,-3.2708f,12.9395f,-3.2708f,12.9395f,-3.1667f,12.8906f,-3.1667f,12.8906f,-3.0625f,12.8418f,-3.0625f,12.8418f,-2.9583f,12.7930f,-2.9583f,12.7930f,-2.8542f,
	12.7442f,-2.8542f,12.7442f,-2.7500f,12.6953f,-2.7500f,12.6953f,-2.6458f,12.6465f,-2.6458f,12.6465f,-2.5417f,12.5977f,-2.5417f,12.5977f,-2.4375f,12.5488f,-2.4375f,12.5488f,-2.3333f,12.5000f,-2.3333f,12.5000f,-2.2292f,12.4512f,-2.2292f,12.4512f,-2.1250f,12.4024f,-2.1250f,
	12.4024f,-2.0208f,12.3535f,-2.0208f,12.3535f,-1.9167f,12.3047f,-1.9167f,12.3047f,-1.8125f,12.2559f,-1.8125f,12.2559f,-1.7083f,12.2071f,-1.7083f,12.2071f,-1.6042f,12.1582f,-1.6042f,12.1582f,-1.5000f,12.1094f,-1.5000f,12.1094f,-1.3958f,12.0606f,-1.3958f,12.0606f,-1.2917f,
	12.0117f,-1.2917f,12.0117f,-1.1875f,11.9629f,-1.1875f,11.9629f,-1.0833f,11.9141f,-1.0833f,11.9141f,-0.9792f,11.8653f,-0.9792f,11.8653f,-0.8750f,11.8164f,-0.8750f,11.8164f,-0.7708f,11.7676f,-0.7708f,11.7676f,-0.6667f,11.7188f,-0.6667f,11.7188f,-0.5625f,11.6699f,-0.5625f,
	11.6699f,-0.4583f,11.6211f,-0.4583f,11.6211f,-0.3542f,11.5723f,-0.3542f,11.5723f,-0.2500f,11.5235f,-0.2500f,11.5235f,-0.1458f,11.4746f,-0.1458f,11.4746f,-0.0417f,11.4258f,-0.0417f,11.4258f,0.0625f,11.3770f,0.0625f,11.3770f,0.1667f,11.3281f,0.1667f,11.3281f,0.2708f,
	11.2793f,0.2708f,11.2793f,0.3750f,11.2305f,0.3750f,11.2305f,0.4792f,11.1817f,0.4792f,11.1817f,0.5833f,11.1328f,0.5833f,11.1328f,0.6875f,11.0840f,0.6875f,11.0840f,0.7917f,11.0352f,0.7917f,11.0352f,0.8958f,10.9863f,0.8958f,10.9863f,1.0000f,10.9375f,1.0000f,
	13.2813f,-4.0000f,13.2324f,-4.0000f,13.2324f,-3.8958f,13.1836f,-3.8958f,13.1836f,-3.7917f,13.1348f,-3.7917f,13.1348f,-3.6875f,13.0860f,-3.6875f,13.0860f,-3.5833f,13.0371f,-3.5833f,13.0371f,-3.4792f,12.9883f,-3.4792f,12.9883f,-3.3750f,12.9395f,-3.3750f,12.9395f,-3.2708f,
	12.8906f,-3.2708f,12.8906f,-3.1667f,12.8418f,-3.1667f,12.8418f,-3.0625f,12.7930f,-3.0625f,12.7930f,-2.9583f,12.7442f,-2.9583f,12.7442f,-2.8542f,12.6953f,-2.8542f,12.6953f,-2.7500f,12.6465f,-2.7500f,12.6465f,-2.6458f,12.5977f,-2.6458f,12.5977f,-2.5417f,12.5488f,-2.5417f,
	12.5488f,-2.4375f,12.5000f,-2.4375f,12.5000f,-2.3333f,12.4512f,-2.3333f,12.4512f,-2.2292f,12.4024f,-2.2292f,12.4024f,-2.1250f,12.3535f,-2.1250f,12.3535f,-2.0208f,12.3047f,-2.0208f,12.3047f,-1.9167f,12.2559f,-1.9167f,12.2559f,-1.8125f,12.2071f,-1.8125f,12.2071f,-1.7083f,
	12.1582f,-1.7083f,12.1582f,-1.6042f,12.1094f,-1.6042f,12.1094f,-1.5000f,12.0606f,-1.5000f,12.0606f,-1.3958f,12.0117f,-1.3958f,12.0117f,-1.2917f,11.9629f,-1.2917f,11.9629f,-1.1875f,11.9141f,-1.1875f,11.9141f,-1.0833f,11.8653f,-1.0833f,11.8653f,-0.9792f,11.8164f,-0.9792f,
	11.8164f,-0.8750f,11.7676f,-0.8750f,11.7676f,-0.7708f,11.7188f,-0.7708f,11.7188f,-0.6667f,11.6699f,-0.6667f,11.6699f,-0.5625f,11.6211f,-0.5625f,11.6211f,-0.4583f,11.5723f,-0.4583f,11.5723f,-0.3542f,11.5235f,-0.3542f,11.5235f,-0.2500f,11.4746f,-0.2500f,11.4746f,-0.1458f,
	11.4258f,-0.1458f,11.4258f,-0.0417f,11.3770f,-0.0417f,11.3770f,0.0625f,11.3281f,0.0625f,11.3281f,0.1667f,11.2793f,0.1667f,11.2793f,0.2708f,11.2305f,0.2708f,11.2305f,0.3750f,11.1817f,0.3750f,11.1817f,0.4792f,11.1328f,0.4792f,11.1328f,0.5833f,11.0840f,0.5833f,
	11.0840f,0.6875f,11.0352f,0.6875f,11.0352f,0.7917f,10.9863f,0.7917f,10.9863f,0.8958f,10.9375f,0.8958f,10.9375f,1.0000f,10.8887f,1.0000f,13.2324f,-4.0000f,13.1836f,-4.0000f,13.1836f,-3.8958f,13.1348f,-3.8958f,13.1348f,-3.7917f,13.0860f,-3.7917f,13.0860f,-3.6875f,
	13.0371f,-3.6875f,13.0371f,-3.5833f,12.9883f,-3.5833f,12.9883f,-3.4792f,12.9395f,-3.4792f,12.9395f,-3.3750f,12.8906f,-3.3750f,12.8906f,-3.2708f,12.8418f,-3.2708f,12.8418f,-3.1667f,12.7930f,-3.1667f,12.7930f,-3.0625f,12.7442f,-3.0625f,12.7442f,-2.9583f,12.6953f,-2.9583f,
	12.6953f,-2.8542f,12.6465f,-2.8542f,12.6465f,-2.7500f,12.5977f,-2.7500f,12.5977f,-2.6458f,12.5488f,-2.6458f,12.5488f,-2.5417f,12.5000f,-2.5417f,12.5000f,-2.4375f,12.4512f,-2.4375f,12.4512f,-2.3333f,12.4024f,-2.3333f,12.4024f,-2.2292f,12.3535f,-2.2292f,12.3535f,-2.1250f,
	12.3047f,-2.1250f,12.3047f,-2.0208f,12.2559f,-2.0208f,12.2559f,-1.9167f,12.2071f,-1.9167f,12.2071f,-1.8125f,12.1582f,-1.8125f,12.1582f,-1.7083f,12.1094f,-1.7083f,12.1094f,-1.6042f,12.0606f,-1.6042f,12.0606f,-1.5000f,12.0117f,-1.5000f,12.0117f,-1.3958f,11.9629f,-1.3958f,
	11.9629f,-1.2917f,11.9141f,-1.2917f,11.9141f,-1.1875f,11.8653f,-1.1875f,11.8653f,-1.0833f,11.8164f,-1.0833f,11.8164f,-0.9792f,11.7676f,-0.9792f,11.7676f,-0.8750f,11.7188f,-0.8750f,11.7188f,-0.7708f,11.6699f,-0.7708f,11.6699f,-0.6667f,11.6211f,-0.6667f,11.6211f,-0.5625f,
	11.5723f,-0.5625f,11.5723f,-0.4583f,11.5235f,-0.4583f,11.5235f,-0.3542f,11.4746f,-0.3542f,11.4746f,-0.2500f,11.4258f,-0.2500f,11.4258f,-0.1458f,11.3770f,-0.1458f,11.3770f,-0.0417f,11.3281f,-0.0417f,11.3281f,0.0625f,11.2793f,0.0625f,11.2793f,0.1667f,11.2305f,0.1667f,
	11.2305f,0.2708f,11.1817f,0.2708f,11.1817f,0.3750f,11.1328f,0.3750f,11.1328f,0.4792f,11.0840f,0.4792f,11.0840f,0.5833f,11.0352f,0.5833f,11.0352f,0.6875f,10.9863f,0.6875f,10.9863f,0.7917f,10.9375f,0.7917f,10.9375f,0.8958f,10.8887f,0.8958f,10.8887f,1.0000f,
	10.8399f,1.0000f,13.1836f,-4.0000f,13.1348f,-4.0000f,13.1348f,-3.8958f,13.0860f,-3.8958f,13.0860f,-3.7917f,13.0371f,-3.7917f,13.0371f,-3.6875f,12.9883f,-3.6875f,12.9883f,-3.5833f,12.9395f,-3.5833f,12.9395f,-3.4792f,12.8906f,-3.4792f,12.8906f,-3.3750f,12.8418f,-3.3750f,
	12.8418f,-3.2708f,12.7930f,-3.2708f,12.7930f,-3.1667f,12.7442f,-3.1667f,12.7442f,-3.0625f,12.6953f,-3.0625f,12.6953f,-2.9583f,12.6465f,-2.9583f,12.6465f,-2.8542f,12.5977f,-2.8542f,12.5977f,-2.7500f,12.5488f,-2.7500f,12.5488f,-2.6458f,12.5000f,-2.6458f,12.5000f,-2.5417f,
	12.4512f,-2.5417f,12.4512f,-2.4375f,12.4024f,-2.4375f,12.4024f,-2.3333f,12.3535f,-2.3333f,12.3535f,-2.2292f,12.3047f,-2.2292f,12.3047f,-2.1250f,12.2559f,-2.1250f,12.2559f,-2.0208f,12.2071f,-2.0208f,12.2071f,-1.9167f,12.1582f,-1.9167f,12.1582f,-1.8125f,12.1094f,-1.8125f,
	12.1094f,-1.7083f,12.0606f,-1.7083f,12.0606f,-1.6042f,12.0117f,-1.6042f,12.0117f,-1.5000f,11.9629f,-1.5000f,11.9629f,-1.3958f,11.9141f,-1.3958f,11.9141f,-1.2917f,11.8653f,-1.2917f,11.8653f,-1.1875f,11.8164f,-1.1875f,11.8164f,-1.0833f,11.7676f,-1.0833f,11.7676f,-0.9792f,
	11.7188f,-0.9792f,11.7188f,-0.8750f,11.6699f,-0.8750f,11.6699f,-0.7708f,11.6211f,-0.7708f,11.6211f,-0.6667f,11.5723f,-0.6667f,11.5723f,-0.5625f,11.5235f,-0.5625f,11.5235f,-0.4583f,11.4746f,-0.4583f,11.4746f,-0.3542f,11.4258f,-0.3542f,11.4258f,-0.2500f,11.3770f,-0.2500f,
	11.3770f,-0.1458f,11.3281f,-0.1458f,11.3281f,-0.0417f,11.2793f,-0.0417f,11.2793f,0.0625f,11.2305f,0.0625f,11.2305f,0.1667f,11.1817f,0.1667f,11.1817f,0.2708f,11.1328f,0.2708f,11.1328f,0.3750f,11.0840f,0.3750f,11.0840f,0.4792f,11.0352f,0.4792f,11.0352f,0.5833f,
	10.9863f,0.5833f,10.9863f,0.6875f,10.9375f,0.6875f,10.9375f,0.7917f,10.8887f,0.7917f,10.8887f,0.8958f,10.8399f,0.8958f,10.8399f,1.0000f,10.7910f,1.0000f,13.1348f,-4.0000f,13.0860f,-4.0000f,13.0860f,-3.8958f,13.0371f,-3.8958f,13.0371f,-3.7917f,12.9883f,-3.7917f,
	12.9883f,-3.6875f,12.9395f,-3.6875f,12.9395f,-3.5833f,12.8906f,-3.5833f,12.8906f,-3.4792f,12.8418f,-3.4792f,12.8418f,-3.3750f,12.7930f,-3.3750f,12.7930f,-3.2708f,12.7442f,-3.2708f,12.7442f,-3.1667f,12.6953f,-3.1667f,12.6953f,-3.0625f,12.6465f,-3.0625f,12.6465f,-2.9583f,
	12.5977f,-2.9583f,12.5977f,-2.8542f,12.5488f,-2.8542f,12.5488f,-2.7500f,12.5000f,-2.7500f,12.5000f,-2.6458f,12.4512f,-2.6458f,12.4512f,-2.5417f,12.4024f,-2.5417f,12.4024f,-2.4375f,12.3535f,-2.4375f,12.3535f,-2.3333f,12.3047f,-2.3333f,12.3047f,-2.2292f,12.2559f,-2.2292f,
	12.2559f,-2.1250f,12.2071f,-2.1250f,12.2071f,-2.0208f,12.1582f,-2.0208f,12.1582f,-1.9167f,12.1094f,-1.9167f,12.1094f,-1.8125f,12.0606f,-1.8125f,12.0606f,-1.7083f,12.0117f,-1.7083f,12.0117f,-1.6042f,11.9629f,-1.6042f,11.9629f,-1.5000f,11.9141f,-1.5000f,11.9141f,-1.3958f,
	11.8653f,-1.3958f,11.8653f,-1.2917f,11.8164f,-1.2917f,11.8164f,-1.1875f,11.7676f,-1.1875f,11.7676f,-1.0833f,11.7188f,-1.0833f,11.7188f,-0.9792f,11.6699f,-0.9792f,11.6699f,-0.8750f,11.6211f,-0.8750f,11.6211f,-0.7708f,11.5723f,-0.7708f,11.5723f,-0.6667f,11.5235f,-0.6667f,
	11.5235f,-0.5625f,11.4746f,-0.5625f,11.4746f,-0.4583f,11.4258f,-0.4583f,11.4258f,-0.3542f,11.3770f,-0.3542f,11.3770f,-0.2500f,11.3281f,-0.2500f,11.3281f,-0.1458f,11.2793f,-0.1458f,11.2793f,-0.0417f,11.2305f,-0.0417f,11.2305f,0.0625f,11.1817f,0.0625f,11.1817f,0.1667f,
	11.1328f,0.1667f,11.1328f,0.2708f,11.0840f,0.2708f,11.0840f,0.3750f,11.0352f,0.3750f,11.0352f,0.4792f,10.9863f,0.4792f,10.9863f,0.5833f,10.9375f,0.5833f,10.9375f,0.6875f,10.8887f,0.6875f,10.8887f,0.7917f,10.8399f,0.7917f,10.8399f,0.8958f,10.7910f,0.8958f,
	10.7910f,1.0000f,10.7422f,1.0000f,13.0860f,-4.0000f,13.0371f,-4.0000f,13.0371f,-3.8958f,12.9883f,-3.8958f,12.9883f,-3.7917f,12.9395f,-3.7917f,12.9395f,-3.6875f,12.8906f,-3.6875f,12.8906f,-3.5833f,12.8418f,-3.5833f,12.8418f,-3.4792f,12.7930f,-3.4792f,12.7930f,-3.3750f,
	12.7442f,-3.3750f,12.7442f,-3.2708f,12.6953f,-3.2708f,12.6953f,-3.1667f,12.6465f,-3.1667f,12.6465f,-3.0625f,12.5977f,-3.0625f,12.5977f,-2.9583f,12.5488f,-2.9583f,12.5488f,-2.8542f,12.5000f,-2.8542f,12.5000f,-2.7500f,12.4512f,-2.7500f,12.4512f,-2.6458f,12.4024f,-2.6458f,
	12.4024f,-2.5417f,12.3535f,-2.5417f,12.3535f,-2.4375f,12.3047f,-2.4375f,12.3047f,-2.3333f,12.2559f,-2.3333f,12.2559f,-2.2292f,12.2071f,-2.2292f,12.2071f,-2.1250f,12.1582f,-2.1250f,12.1582f,-2.0208f,12.1094f,-2.0208f,12.1094f,-1.9167f,12.0606f,-1.9167f,12.0606f,-1.8125f,
	12.0117f,-1.8125f,12.0117f,-1.7083f,11.9629f,-1.7083f,11.9629f,-1.6042f,11.9141f,-1.6042f,11.9141f,-1.5000f,11.8653f,-1.5000f,11.8653f,-1.3958f,11.8164f,-1.3958f,11.8164f,-1.2917f,11.7676f,-1.2917f,11.7676f,-1.1875f,11.7188f,-1.1875f,11.7188f,-1.0833f,11.6699f,-1.0833f,
	11.6699f,-0.9792f,11.6211f,-0.9792f,11.6211f,-0.8750f,11.5723f,-0.8750f,11.5723f,-0.7708f,11.5235f,-0.7708f,11.5235f,-0.6667f,11.4746f,-0.6667f,11.4746f,-0.5625f,11.4258f,-0.5625f,11.4258f,-0.4583f,11.3770f,-0.4583f,11.3770f,-0.3542f,11.3281f,-0.3542f,11.3281f,-0.2500f,
	11.2793f,-0.2500f,11.2793f,-0.1458f,11.2305f,-0.1458f,11.2305f,-0.0417f,11.1817f,-0.0417f,11.1817f,0.0625f,11.1328f,0.0625f,11.1328f,0.1667f,11.0840f,0.1667f,11.0840f,0.2708f,11.0352f,0.2708f,11.0352f,0.3750f,10.9863f,0.3750f,10.9863f,0.4792f,10.9375f,0.4792f,
	10.9375f,0.5833f,10.8887f,0.5833f,10.8887f,0.6875f,10.8399f,0.6875f,10.8399f,0.7917f,10.7910f,0.7917f,10.7910f,0.8958f,10.7422f,0.8958f,10.7422f,1.0000f,10.6934f,1.0000f,13.0371f,-4.0000f,12.9883f,-4.0000f,12.9883f,-3.8958f,12.9395f,-3.8958f,12.9395f,-3.7917f,
	12.8906f,-3.7917f,12.8906f,-3.6875f,12.8418f,-3.6875f,12.8418f,-3.5833f,12.7930f,-3.5833f,12.7930f,-3.4792f,12.7442f,-3.4792f,12.7442f,-3.3750f,12.6953f,-3.3750f,12.6953f,-3.2708f,12.6465f,-3.2708f,12.6465f,-3.1667f,12.5977f,-3.1667f,12.5977f,-3.0625f,12.5488f,-3.0625f,
	12.5488f,-2.9583f,12.5000f,-2.9583f,12.5000f,-2.8542f,12.4512f,-2.8542f,12.4512f,-2.7500f,12.4024f,-2.7500f,12.4024f,-2.6458f,12.3535f,-2.6458f,12.3535f,-2.5417f,12.3047f,-2.5417f,12.3047f,-2.4375f,12.2559f,-2.4375f,12.2559f,-2.3333f,12.2071f,-2.3333f,12.2071f,-2.2292f,
	12.1582f,-2.2292f,12.1582f,-2.1250f,12.1094f,-2.1250f,12.1094f,-2.0208f,12.0606f,-2.0208f,12.0606f,-1.9167f,12.0117f,-1.9167f,12.0117f,-1.8125f,11.9629f,-1.8125f,11.9629f,-1.7083f,11.9141f,-1.7083f,11.9141f,-1.6042f,11.8653f,-1.6042f,11.8653f,-1.5000f,11.8164f,-1.5000f,
	11.8164f,-1.3958f,11.7676f,-1.3958f,11.7676f,-1.2917f,11.7188f,-1.2917f,11.7188f,-1.1875f,11.6699f,-1.1875f,11.6699f,-1.0833f,11.6211f,-1.0833f,11.6211f,-0.9792f,11.5723f,-0.9792f,11.5723f,-0.8750f,11.5235f,-0.8750f,11.5235f,-0.7708f,11.4746f,-0.7708f,11.4746f,-0.6667f,
	11.4258f,-0.6667f,11.4258f,-0.5625f,11.3770f,-0.5625f,11.3770f,-0.4583f,11.3281f,-0.4583f,11.3281f,-0.3542f,11.2793f,-0.3542f,11.2793f,-0.2500f,11.2305f,-0.2500f,11.2305f,-0.1458f,11.1817f,-0.1458f,11.1817f,-0.0417f,11.1328f,-0.0417f,11.1328f,0.0625f,11.0840f,0.0625f,
	11.0840f,0.1667f,11.0352f,0.1667f,11.0352f,0.2708f,10.9863f,0.2708f,10.9863f,0.3750f,10.9375f,0.3750f,10.9375f,0.4792f,10.8887f,0.4792f,10.8887f,0.5833f,10.8399f,0.5833f,10.8399f,0.6875f,10.7910f,0.6875f,10.7910f,0.7917f,10.7422f,0.7917f,10.7422f,0.8958f,
	10.6934f,0.8958f,10.6934f,1.0000f,10.6445f,1.0000f,12.9883f,-4.0000f,12.9395f,-4.0000f,12.9395f,-3.8958f,12.8906f,-3.8958f,12.8906f,-3.7917f,12.8418f,-3.7917f,12.8418f,-3.6875f,12.7930f,-3.6875f,12.7930f,-3.5833f,12.7442f,-3.5833f,12.7442f,-3.4792f,12.6953f,-3.4792f,
	12.6953f,-3.3750f,12.6465f,-3.3750f,12.6465f,-3.2708f,12.5977f,-3.2708f,12.5977f,-3.1667f,12.5488f,-3.1667f,12.5488f,-3.0625f,12.5000f,-3.0625f,12.5000f,-2.9583f,12.4512f,-2.9583f,12.4512f,-2.8542f,12.4024f,-2.8542f,12.4024f,-2.7500f,12.3535f,-2.7500f,12.3535f,-2.6458f,
	12.3047f,-2.6458f,12.3047f,-2.5417f,12.2559f,-2.5417f,12.2559f,-2.4375f,12.2071f,-2.4375f,12.2071f,-2.3333f,12.1582f,-2.3333f,12.1582f,-2.2292f,12.1094f,-2.2292f,12.1094f,-2.1250f,12.0606f,-2.1250f,12.0606f,-2.0208f,12.0117f,-2.0208f,12.0117f,-1.9167f,11.9629f,-1.9167f,
	11.9629f,-1.8125f,11.9141f,-1.8125f,11.9141f,-1.7083f,11.8653f,-1.7083f,11.8653f,-1.6042f,11.8164f,-1.6042f,11.8164f,-1.5000f,11.7676f,-1.5000f,11.7676f,-1.3958f,11.7188f,-1.3958f,11.7188f,-1.2917f,11.6699f,-1.2917f,11.6699f,-1.1875f,11.6211f,-1.1875f,11.6211f,-1.0833f,
	11.5723f,-1.0833f,11.5723f,-0.9792f,11.5235f,-0.9792f,11.5235f,-0.8750f,11.4746f,-0.8750f,11.4746f,-0.7708f,11.4258f,-0.7708f,11.4258f,-0.6667f,11.3770f,-0.6667f,11.3770f,-0.5625f,11.3281f,-0.5625f,11.3281f,-0.4583f,11.2793f,-0.4583f,11.2793f,-0.3542f,11.2305f,-0.3542f,
	11.2305f,-0.2500f,11.1817f,-0.2500f,11.1817f,-0.1458f,11.1328f,-0.1458f,11.1328f,-0.0417f,11.0840f,-0.0417f,11.0840f,0.0625f,11.0352f,0.0625f,11.0352f,0.1667f,10.9863f,0.1667f,10.9863f,0.2708f,10.9375f,0.2708f,10.9375f,0.3750f,10.8887f,0.3750f,10.8887f,0.4792f,
	10.8399f,0.4792f,10.8399f,0.5833f,10.7910f,0.5833f,10.7910f,0.6875f,10.7422f,0.6875f,10.7422f,0.7917f,10.6934f,0.7917f,10.6934f,0.8958f,10.6445f,0.8958f,10.6445f,1.0000f,10.5957f,1.0000f,12.9395f,-4.0000f,12.8906f,-4.0000f,12.8906f,-3.8958f,12.8418f,-3.8958f,
	12.8418f,-3.7917f,12.7930f,-3.7917f,12.7930f,-3.6875f,12.7442f,-3.6875f,12.7442f,-3.5833f,12.6953f,-3.5833f,12.6953f,-3.4792f,12.6465f,-3.4792f,12.6465f,-3.3750f,12.5977f,-3.3750f,12.5977f,-3.2708f,12.5488f,-3.2708f,12.5488f,-3.1667f,12.5000f,-3.1667f,12.5000f,-3.0625f,
	12.4512f,-3.0625f,12.4512f,-2.9583f,12.4024f,-2.9583f,12.4024f,-2.8542f,12.3535f,-2.8542f,12.3535f,-2.7500f,12.3047f,-2.7500f,12.3047f,-2.6458f,12.2559f,-2.6458f,12.2559f,-2.5417f,12.2071f,-2.5417f,12.2071f,-2.4375f,12.1582f,-2.4375f,12.1582f,-2.3333f,12.1094f,-2.3333f,
	12.1094f,-2.2292f,12.0606f,-2.2292f,12.0606f,-2.1250f,12.0117f,-2.1250f,12.0117f,-2.0208f,11.9629f,-2.0208f,11.9629f,-1.9167f,11.9141f,-1.9167f,11.9141f,-1.8125f,11.8653f,-1.8125f,11.8653f,-1.7083f,11.8164f,-1.7083f,11.8164f,-1.6042f,11.7676f,-1.6042f,11.7676f,-1.5000f,
	11.7188f,-1.5000f,11.7188f,-1.3958f,11.6699f,-1.3958f,11.6699f,-1.2917f,11.6211f,-1.2917f,11.6211f,-1.1875f,11.5723f,-1.1875f,11.5723f,-1.0833f,11.5235f,-1.0833f,11.5235f,-0.9792f,11.4746f,-0.9792f,11.4746f,-0.8750f,11.4258f,-0.8750f,11.4258f,-0.7708f,11.3770f,-0.7708f,
	11.3770f,-0.6667f,11.3281f,-0.6667f,11.3281f,-0.5625f,11.2793f,-0.5625f,11.2793f,-0.4583f,11.2305f,-0.4583f,11.2305f,-0.3542f,11.1817f,-0.3542f,11.1817f,-0.2500f,11.1328f,-0.2500f,11.1328f,-0.1458f,11.0840f,-0.1458f,11.0840f,-0.0417f,11.0352f,-0.0417f,11.0352f,0.0625f,
	10.9863f,0.0625f,10.9863f,0.1667f,10.9375f,0.1667f,10.9375f,0.2708f,10.8887f,0.2708f,10.8887f,0.3750f,10.8399f,0.3750f,10.8399f,0.4792f,10.7910f,0.4792f,10.7910f,0.5833f,10.7422f,0.5833f,10.7422f,0.6875f,10.6934f,0.6875f,10.6934f,0.7917f,10.6445f,0.7917f,
	10.6445f,0.8958f,10.5957f,0.8958f,10.5957f,1.0000f,10.5469f,1.0000f,12.8906f,-4.0000f,12.8418f,-4.0000f,12.8418f,-3.8958f,12.7930f,-3.8958f,12.7930f,-3.7917f,12.7442f,-3.7917f,12.7442f,-3.6875f,12.6953f,-3.6875f,12.6953f,-3.5833f,12.6465f,-3.5833f,12.6465f,-3.4792f,
	12.5977f,-3.4792f,12.5977f,-3.3750f,12.5488f,-3.3750f,12.5488f,-3.2708f,12.5000f,-3.2708f,12.5000f,-3.1667f,12.4512f,-3.1667f,12.4512f,-3.0625f,12.4024f,-3.0625f,12.4024f,-2.9583f,12.3535f,-2.9583f,12.3535f,-2.8542f,12.3047f,-2.8542f,12.3047f,-2.7500f,12.2559f,-2.7500f,
	12.2559f,-2.6458f,12.2071f,-2.6458f,12.2071f,-2.5417f,12.1582f,-2.5417f,12.1582f,-2.4375f,12.1094f,-2.4375f,12.1094f,-2.3333f,12.0606f,-2.3333f,12.0606f,-2.2292f,12.0117f,-2.2292f,12.0117f,-2.1250f,11.9629f,-2.1250f,11.9629f,-2.0208f,11.9141f,-2.0208f,11.9141f,-1.9167f,
	11.8653f,-1.9167f,11.8653f,-1.8125f,11.8164f,-1.8125f,11.8164f,-1.7083f,11.7676f,-1.7083f,11.7676f,-1.6042f,11.7188f,-1.6042f,11.7188f,-1.5000f,11.6699f,-1.5000f,11.6699f,-1.3958f,11.6211f,-1.3958f,11.6211f,-1.2917f,11.5723f,-1.2917f,11.5723f,-1.1875f,11.5235f,-1.1875f,
	11.5235f,-1.0833f,11.4746f,-1.0833f,11.4746f,-0.9792f,11.4258f,-0.9792f,11.4258f,-0.8750f,11.3770f,-0.8750f,11.3770f,-0.7708f,11.3281f,-0.7708f,11.3281f,-0.6667f,11.2793f,-0.6667f,11.2793f,-0.5625f,11.2305f,-0.5625f,11.2305f,-0.4583f,11.1817f,-0.4583f,11.1817f,-0.3542f,
	11.1328f,-0.3542f,11.1328f,-0.2500f,11.0840f,-0.2500f,11.0840f,-0.1458f,11.0352f,-0.1458f,11.0352f,-0.0417f,10.9863f,-0.0417f,10.9863f,0.0625f,10.9375f,0.0625f,10.9375f,0.1667f,10.8887f,0.1667f,10.8887f,0.2708f,10.8399f,0.2708f,10.8399f,0.3750f,10.7910f,0.3750f,
	10.7910f,0.4792f,10.7422f,0.4792f,10.7422f,0.5833f,10.6934f,0.5833f,10.6934f,0.6875f,10.6445f,0.6875f,10.6445f,0.7917f,10.5957f,0.7917f,10.5957f,0.8958f,10.5469f,0.8958f,10.5469f,1.0000f,10.4981f,1.0000f,12.8418f,-4.0000f,12.7930f,-4.0000f,12.7930f,-3.8958f,
	12.7442f,-3.8958f,12.7442f,-3.7917f,12.6953f,-3.7917f,12.6953f,-3.6875f,12.6465f,-3.6875f,12.6465f,-3.5833f,12.5977f,-3.5833f,12.5977f,-3.4792f,12.5488f,-3.4792f,12.5488f,-3.3750f,12.5000f,-3.3750f,12.5000f,-3.2708f,12.4512f,-3.2708f,12.4512f,-3.1667f,12.4024f,-3.1667f,
	12.4024f,-3.0625f,12.3535f,-3.0625f,12.3535f,-2.9583f,12.3047f,-2.9583f,12.3047f,-2.8542f,12.2559f,-2.8542f,12.2559f,-2.7500f,12.2071f,-2.7500f,12.2071f,-2.6458f,12.1582f,-2.6458f,12.1582f,-2.5417f,12.1094f,-2.5417f,12.1094f,-2.4375f,12.0606f,-2.4375f,12.0606f,-2.3333f,
	12.0117f,-2.3333f,12.0117f,-2.2292f,11.9629f,-2.2292f,11.9629f,-2.1250f,11.9141f,-2.1250f,11.9141f,-2.0208f,11.8653f,-2.0208f,11.8653f,-1.9167f,11.8164f,-1.9167f,11.8164f,-1.8125f,11.7676f,-1.8125f,11.7676f,-1.7083f,11.7188f,-1.7083f,11.7188f,-1.6042f,11.6699f,-1.6042f,
	11.6699f,-1.5000f,11.6211f,-1.5000f,11.6211f,-1.3958f,11.5723f,-1.3958f,11.5723f,-1.2917f,11.5235f,-1.2917f,11.5235f,-1.1875f,11.4746f,-1.1875f,11.4746f,-1.0833f,11.4258f,-1.0833f,11.4258f,-0.9792f,11.3770f,-0.9792f,11.3770f,-0.8750f,11.3281f,-0.8750f,11.3281f,-0.7708f,
	11.2793f,-0.7708f,11.2793f,-0.6667f,11.2305f,-0.6667f,11.2305f,-0.5625f,11.1817f,-0.5625f,11.1817f,-0.4583f,11.1328f,-0.4583f,11.1328f,-0.3542f,11.0840f,-0.3542f,11.0840f,-0.2500f,11.0352f,-0.2500f,11.0352f,-0.1458f,10.9863f,-0.1458f,10.9863f,-0.0417f,10.9375f,-0.0417f,
	10.9375f,0.0625f,10.8887f,0.0625f,10.8887f,0.1667f,10.8399f,0.1667f,10.8399f,0.2708f,10.7910f,0.2708f,10.7910f,0.3750f,10.7422f,0.3750f,10.7422f,0.4792f,10.6934f,0.4792f,10.6934f,0.5833f,10.6445f,0.5833f,10.6445f,0.6875f,10.5957f,0.6875f,10.5957f,0.7917f,
	10.5469f,0.7917f,10.5469f,0.8958f,10.4981f,0.8958f,10.4981f,1.0000f,10.4492f,1.0000f,12.7930f,-4.0000f,12.7442f,-4.0000f,12.7442f,-3.8958f,12.6953f,-3.8958f,12.6953f,-3.7917f,12.6465f,-3.7917f,12.6465f,-3.6875f,12.5977f,-3.6875f,12.5977f,-3.5833f,12.5488f,-3.5833f,
	12.5488f,-3.4792f,12.5000f,-3.4792f,12.5000f,-3.3750f,12.4512f,-3.3750f,12.4512f,-3.2708f,12.4024f,-3.2708f,12.4024f,-3.1667f,12.3535f,-3.1667f,12.3535f,-3.0625f,12.3047f,-3.0625f,12.3047f,-2.9583f,12.2559f,-2.9583f,12.2559f,-2.8542f,12.2071f,-2.8542f,12.2071f,-2.7500f,
	12.1582f,-2.7500f,12.1582f,-2.6458f,12.1094f,-2.6458f,12.1094f,-2.5417f,12.0606f,-2.5417f,12.0606f,-2.4375f,12.0117f,-2.4375f,12.0117f,-2.3333f,11.9629f,-2.3333f,11.9629f,-2.2292f,11.9141f,-2.2292f,11.9141f,-2.1250f,11.8653f,-2.1250f,11.8653f,-2.0208f,11.8164f,-2.0208f,
	11.8164f,-1.9167f,11.7676f,-1.9167f,11.7676f,-1.8125f,11.7188f,-1.8125f,11.7188f,-1.7083f,11.6699f,-1.7083f,11.6699f,-1.6042f,11.6211f,-1.6042f,11.6211f,-1.5000f,11.5723f,-1.5000f,11.5723f,-1.3958f,11.5235f,-1.3958f,11.5235f,-1.2917f,11.4746f,-1.2917f,11.4746f,-1.1875f,
	11.4258f,-1.1875f,11.4258f,-1.0833f,11.3770f,-1.0833f,11.3770f,-0.9792f,11.3281f,-0.9792f,11.3281f,-0.8750f,11.2793f,-0.8750f,11.2793f,-0.7708f,11.2305f,-0.7708f,11.2305f,-0.6667f,11.1817f,-0.6667f,11.1817f,-0.5625f,11.1328f,-0.5625f,11.1328f,-0.4583f,11.0840f,-0.4583f,
	11.0840f,-0.3542f,11.0352f,-0.3542f,11.0352f,-0.2500f,10.9863f,-0.2500f,10.9863f,-0.1458f,10.9375f,-0.1458f,10.9375f,-0.0417f,10.8887f,-0.0417f,10.8887f,0.0625f,10.8399f,0.0625f,10.8399f,0.1667f,10.7910f,0.1667f,10.7910f,0.2708f,10.7422f,0.2708f,10.7422f,0.3750f,
	10.6934f,0.3750f,10.6934f,0.4792f,10.6445f,0.4792f,10.6445f,0.5833f,10.5957f,0.5833f,10.5957f,0.6875f,10.5469f,0.6875f,10.5469f,0.7917f,10.4981f,0.7917f,10.4981f,0.8958f,10.4492f,0.8958f,10.4492f,1.0000f,10.4004f,1.0000f,12.7442f,-4.0000f,12.6953f,-4.0000f,
	12.6953f,-3.8958f,12.6465f,-3.8958f,12.6465f,-3.7917f,12.5977f,-3.7917f,12.5977f,-3.6875f,12.5488f,-3.6875f,12.5488f,-3.5833f,12.5000f,-3.5833f,12.5000f,-3.4792f,12.4512f,-3.4792f,12.4512f,-3.3750f,12.4024f,-3.3750f,12.4024f,-3.2708f,12.3535f,-3.2708f,12.3535f,-3.1667f,
	12.3047f,-3.1667f,12.3047f,-3.0625f,12.2559f,-3.0625f,12.2559f,-2.9583f,12.2071f,-2.9583f,12.2071f,-2.8542f,12.1582f,-2.8542f,12.1582f,-2.7500f,12.1094f,-2.7500f,12.1094f,-2.6458f,12.0606f,-2.6458f,12.0606f,-2.5417f,12.0117f,-2.5417f,12.0117f,-2.4375f,11.9629f,-2.4375f,
	11.9629f,-2.3333f,11.9141f,-2.3333f,11.9141f,-2.2292f,11.8653f,-2.2292f,11.8653f,-2.1250f,11.8164f,-2.1250f,11.8164f,-2.0208f,11.7676f,-2.0208f,11.7676f,-1.9167f,11.7188f,-1.9167f,11.7188f,-1.8125f,11.6699f,-1.8125f,11.6699f,-1.7083f,11.6211f,-1.7083f,11.6211f,-1.6042f,
	11.5723f,-1.6042f,11.5723f,-1.5000f,11.5235f,-1.5000f,11.5235f,-1.3958f,11.4746f,-1.3958f,11.4746f,-1.2917f,11.4258f,-1.2917f,11.4258f,-1.1875f,11.3770f,-1.1875f,11.3770f,-1.0833f,11.3281f,-1.0833f,11.3281f,-0.9792f,11.2793f,-0.9792f,11.2793f,-0.8750f,11.2305f,-0.8750f,
	11.2305f,-0.7708f,11.1817f,-0.7708f,11.1817f,-0.6667f,11.1328f,-0.6667f,11.1328f,-0.5625f,11.0840f,-0.5625f,11.0840f,-0.4583f,11.0352f,-0.4583f,11.0352f,-0.3542f,10.9863f,-0.3542f,10.9863f,-0.2500f,10.9375f,-0.2500f,10.9375f,-0.1458f,10.8887f,-0.1458f,10.8887f,-0.0417f,
	10.8399f,-0.0417f,10.8399f,0.0625f,10.7910f,0.0625f,10.7910f,0.1667f,10.7422f,0.1667f,10.7422f,0.2708f,10.6934f,0.2708f,10.6934f,0.3750f,10.6445f,0.3750f,10.6445f,0.4792f,10.5957f,0.4792f,10.5957f,0.5833f,10.5469f,0.5833f,10.5469f,0.6875f,10.4981f,0.6875f,
	10.4981f,0.7917f,10.4492f,0.7917f,10.4492f,0.8958f,10.4004f,0.8958f,10.4004f,1.0000f,10.3516f,1.0000f,12.6953f,-4.0000f,12.6465f,-4.0000f,12.6465f,-3.8958f,12.5977f,-3.8958f,12.5977f,-3.7917f,12.5488f,-3.7917f,12.5488f,-3.6875f,12.5000f,-3.6875f,12.5000f,-3.5833f,
	12.4512f,-3.5833f,12.4512f,-3.4792f,12.4024f,-3.4792f,12.4024f,-3.3750f,12.3535f,-3.3750f,12.3535f,-3.2708f,12.3047f,-3.2708f,12.3047f,-3.1667f,12.2559f,-3.1667f,12.2559f,-3.0625f,12.2071f,-3.0625f,12.2071f,-2.9583f,12.1582f,-2.9583f,12.1582f,-2.8542f,12.1094f,-2.8542f,
	12.1094f,-2.7500f,12.0606f,-2.7500f,12.0606f,-2.6458f,12.0117f,-2.6458f,12.0117f,-2.5417f,11.9629f,-2.5417f,11.9629f,-2.4375f,11.9141f,-2.4375f,11.9141f,-2.3333f,11.8653f,-2.3333f,11.8653f,-2.2292f,11.8164f,-2.2292f,11.8164f,-2.1250f,11.7676f,-2.1250f,11.7676f,-2.0208f,
	11.7188f,-2.0208f,11.7188f,-1.9167f,11.6699f,-1.9167f,11.6699f,-1.8125f,11.6211f,-1.8125f,11.6211f,-1.7083f,11.5723f,-1.7083f,11.5723f,-1.6042f,11.5235f,-1.6042f,11.5235f,-1.5000f,11.4746f,-1.5000f,11.4746f,-1.3958f,11.4258f,-1.3958f,11.4258f,-1.2917f,11.3770f,-1.2917f,
	11.3770f,-1.1875f,11.3281f,-1.1875f,11.3281f,-1.0833f,11.2793f,-1.0833f,11.2793f,-0.9792f,11.2305f,-0.9792f,11.2305f,-0.8750f,11.1817f,-0.8750f,11.1817f,-0.7708f,11.1328f,-0.7708f,11.1328f,-0.6667f,11.0840f,-0.6667f,11.0840f,-0.5625f,11.0352f,-0.5625f,11.0352f,-0.4583f,
	10.9863f,-0.4583f,10.9863f,-0.3542f,10.9375f,-0.3542f,10.9375f,-0.2500f,10.8887f,-0.2500f,10.8887f,-0.1458f,10.8399f,-0.1458f,10.8399f,-0.0417f,10.7910f,-0.0417f,10.7910f,0.0625f,10.7422f,0.0625f,10.7422f,0.1667f,10.6934f,0.1667f,10.6934f,0.2708f,10.6445f,0.2708f,
	10.6445f,0.3750f,10.5957f,0.3750f,10.5957f,0.4792f,10.5469f,0.4792f,10.5469f,0.5833f,10.4981f,0.5833f,10.4981f,0.6875f,10.4492f,0.6875f,10.4492f,0.7917f,10.4004f,0.7917f,10.4004f,0.8958f,10.3516f,0.8958f,10.3516f,1.0000f,10.3027f,1.0000f,12.6465f,-4.0000f,
	12.5977f,-4.0000f,12.5977f,-3.8958f,12.5488f,-3.8958f,12.5488f,-3.7917f,12.5000f,-3.7917f,12.5000f,-3.6875f,12.4512f,-3.6875f,12.4512f,-3.5833f,12.4024f,-3.5833f,12.4024f,-3.4792f,12.3535f,-3.4792f,12.3535f,-3.3750f,12.3047f,-3.3750f,12.3047f,-3.2708f,12.2559f,-3.2708f,
	12.2559f,-3.1667f,12.2071f,-3.1667f,12.2071f,-3.0625f,12.1582f,-3.0625f,12.1582f,-2.9583f,12.1094f,-2.9583f,12.1094f,-2.8542f,12.0606f,-2.8542f,12.0606f,-2.7500f,12.0117f,-2.7500f,12.0117f,-2.6458f,11.9629f,-2.6458f,11.9629f,-2.5417f,11.9141f,-2.5417f,11.9141f,-2.4375f,
	11.8653f,-2.4375f,11.8653f,-2.3333f,11.8164f,-2.3333f,11.8164f,-2.2292f,11.7676f,-2.2292f,11.7676f,-2.1250f,11.7188f,-2.1250f,11.7188f,-2.0208f,11.6699f,-2.0208f,11.6699f,-1.9167f,11.6211f,-1.9167f,11.6211f,-1.8125f,11.5723f,-1.8125f,11.5723f,-1.7083f,11.5235f,-1.7083f,
	11.5235f,-1.6042f,11.4746f,-1.6042f,11.4746f,-1.5000f,11.4258f,-1.5000f,11.4258f,-1.3958f,11.3770f,-1.3958f,11.3770f,-1.2917f,11.3281f,-1.2917f,11.3281f,-1.1875f,11.2793f,-1.1875f,11.2793f,-1.0833f,11.2305f,-1.0833f,11.2305f,-0.9792f,11.1817f,-0.9792f,11.1817f,-0.8750f,
	11.1328f,-0.8750f,11.1328f,-0.7708f,11.0840f,-0.7708f,11.0840f,-0.6667f,11.0352f,-0.6667f,11.0352f,-0.5625f,10.9863f,-0.5625f,10.9863f,-0.4583f,10.9375f,-0.4583f,10.9375f,-0.3542f,10.8887f,-0.3542f,10.8887f,-0.2500f,10.8399f,-0.2500f,10.8399f,-0.1458f,10.7910f,-0.1458f,
	10.7910f,-0.0417f,10.7422f,-0.0417f,10.7422f,0.0625f,10.6934f,0.0625f,10.6934f,0.1667f,10.6445f,0.1667f,10.6445f,0.2708f,10.5957f,0.2708f,10.5957f,0.3750f,10.5469f,0.3750f,10.5469f,0.4792f,10.4981f,0.4792f,10.4981f,0.5833f,10.4492f,0.5833f,10.4492f,0.6875f,
	10.4004f,0.6875f,10.4004f,0.7917f,10.3516f,0.7917f,10.3516f,0.8958f,10.3027f,0.8958f,10.3027f,1.0000f,10.2539f,1.0000f,12.5977f,-4.0000f,12.5488f,-4.0000f,12.5488f,-3.8958f,12.5000f,-3.8958f,12.5000f,-3.7917f,12.4512f,-3.7917f,12.4512f,-3.6875f,12.4024f,-3.6875f,
	12.4024f,-3.5833f,12.3535f,-3.5833f,12.3535f,-3.4792f,12.3047f,-3.4792f,12.3047f,-3.3750f,12.2559f,-3.3750f,12.2559f,-3.2708f,12.2071f,-3.2708f,12.2071f,-3.1667f,12.1582f,-3.1667f,12.1582f,-3.0625f,12.1094f,-3.0625f,12.1094f,-2.9583f,12.0606f,-2.9583f,12.0606f,-2.8542f,
	12.0117f,-2.8542f,12.0117f,-2.7500f,11.9629f,-2.7500f,11.9629f,-2.6458f,11.9141f,-2.6458f,11.9141f,-2.5417f,11.8653f,-2.5417f,11.8653f,-2.4375f,11.8164f,-2.4375f,11.8164f,-2.3333f,11.7676f,-2.3333f,11.7676f,-2.2292f,11.7188f,-2.2292f,11.7188f,-2.1250f,11.6699f,-2.1250f,
	11.6699f,-2.0208f,11.6211f,-2.0208f,11.6211f,-1.9167f,11.5723f,-1.9167f,11.5723f,-1.8125f,11.5235f,-1.8125f,11.5235f,-1.7083f,11.4746f,-1.7083f,11.4746f,-1.6042f,11.4258f,-1.6042f,11.4258f,-1.5000f,11.3770f,-1.5000f,11.3770f,-1.3958f,11.3281f,-1.3958f,11.3281f,-1.2917f,
	11.2793f,-1.2917f,11.2793f,-1.1875f,11.2305f,-1.1875f,11.2305f,-1.0833f,11.1817f,-1.0833f,11.1817f,-0.9792f,11.1328f,-0.9792f,11.1328f,-0.8750f,11.0840f,-0.8750f,11.0840f,-0.7708f,11.0352f,-0.7708f,11.0352f,-0.6667f,10.9863f,-0.6667f,10.9863f,-0.5625f,10.9375f,-0.5625f,
	10.9375f,-0.4583f,10.8887f,-0.4583f,10.8887f,-0.3542f,10.8399f,-0.3542f,10.8399f,-0.2500f,10.7910f,-0.2500f,10.7910f,-0.1458f,10.7422f,-0.1458f,10.7422f,-0.0417f,10.6934f,-0.0417f,10.6934f,0.0625f,10.6445f,0.0625f,10.6445f,0.1667f,10.5957f,0.1667f,10.5957f,0.2708f,
	10.5469f,0.2708f,10.5469f,0.3750f,10.4981f,0.3750f,10.4981f,0.4792f,10.4492f,0.4792f,10.4492f,0.5833f,10.4004f,0.5833f,10.4004f,0.6875f,10.3516f,0.6875f,10.3516f,0.7917f,10.3027f,0.7917f,10.3027f,0.8958f,10.2539f,0.8958f,10.2539f,1.0000f,10.2051f,1.0000f,
	12.5488f,-4.0000f,12.5000f,-4.0000f,12.5000f,-3.8958f,12.4512f,-3.8958f,12.4512f,-3.7917f,12.4024f,-3.7917f,12.4024f,-3.6875f,12.3535f,-3.6875f,12.3535f,-3.5833f,12.3047f,-3.5833f,12.3047f,-3.4792f,12.2559f,-3.4792f,12.2559f,-3.3750f,12.2071f,-3.3750f,12.2071f,-3.2708f,
	12.1582f,-3.2708f,12.1582f,-3.1667f,12.1094f,-3.1667f,12.1094f,-3.0625f,12.0606f,-3.0625f,12.0606f,-2.9583f,12.0117f,-2.9583f,12.0117f,-2.8542f,11.9629f,-2.8542f,11.9629f,-2.7500f,11.9141f,-2.7500f,11.9141f,-2.6458f,11.8653f,-2.6458f,11.8653f,-2.5417f,11.8164f,-2.5417f,
	11.8164f,-2.4375f,11.7676f,-2.4375f,11.7676f,-2.3333f,11.7188f,-2.3333f,11.7188f,-2.2292f,11.6699f,-2.2292f,11.6699f,-2.1250f,11.6211f,-2.1250f,11.6211f,-2.0208f,11.5723f,-2.0208f,11.5723f,-1.9167f,11.5235f,-1.9167f,11.5235f,-1.8125f,11.4746f,-1.8125f,11.4746f,-1.7083f,
	11.4258f,-1.7083f,11.4258f,-1.6042f,11.3770f,-1.6042f,11.3770f,-1.5000f,11.3281f,-1.5000f,11.3281f,-1.3958f,11.2793f,-1.3958f,11.2793f,-1.2917f,11.2305f,-1.2917f,11.2305f,-1.1875f,11.1817f,-1.1875f,11.1817f,-1.0833f,11.1328f,-1.0833f,11.1328f,-0.9792f,11.0840f,-0.9792f,
	11.0840f,-0.8750f,11.0352f,-0.8750f,11.0352f,-0.7708f,10.9863f,-0.7708f,10.9863f,-0.6667f,10.9375f,-0.6667f,10.9375f,-0.5625f,10.8887f,-0.5625f,10.8887f,-0.4583f,10.8399f,-0.4583f,10.8399f,-0.3542f,10.7910f,-0.3542f,10.7910f,-0.2500f,10.7422f,-0.2500f,10.7422f,-0.1458f,
	10.6934f,-0.1458f,10.6934f,-0.0417f,10.6445f,-0.0417f,10.6445f,0.0625f,10.5957f,0.0625f,10.5957f,0.1667f,10.5469f,0.1667f,10.5469f,0.2708f,10.4981f,0.2708f,10.4981f,0.3750f,10.4492f,0.3750f,10.4492f,0.4792f,10.4004f,0.4792f,10.4004f,0.5833f,10.3516f,0.5833f,
	10.3516f,0.6875f,10.3027f,0.6875f,10.3027f,0.7917f,10.2539f,0.7917f,10.2539f,0.8958f,10.2051f,0.8958f,10.2051f,1.0000f,10.1563f,1.0000f,12.5000f,-4.0000f,12.4512f,-4.0000f,12.4512f,-3.8958f,12.4024f,-3.8958f,12.4024f,-3.7917f,12.3535f,-3.7917f,12.3535f,-3.6875f,
	12.3047f,-3.6875f,12.3047f,-3.5833f,12.2559f,-3.5833f,12.2559f,-3.4792f,12.2071f,-3.4792f,12.2071f,-3.3750f,12.1582f,-3.3750f,12.1582f,-3.2708f,12.1094f,-3.2708f,12.1094f,-3.1667f,12.0606f,-3.1667f,12.0606f,-3.0625f,12.0117f,-3.0625f,12.0117f,-2.9583f,11.9629f,-2.9583f,
	11.9629f,-2.8542f,11.9141f,-2.8542f,11.9141f,-2.7500f,11.8653f,-2.7500f,11.8653f,-2.6458f,11.8164f,-2.6458f,11.8164f,-2.5417f,11.7676f,-2.5417f,11.7676f,-2.4375f,11.7188f,-2.4375f,11.7188f,-2.3333f,11.6699f,-2.3333f,11.6699f,-2.2292f,11.6211f,-2.2292f,11.6211f,-2.1250f,
	11.5723f,-2.1250f,11.5723f,-2.0208f,11.5235f,-2.0208f,11.5235f,-1.9167f,11.4746f,-1.9167f,11.4746f,-1.8125f,11.4258f,-1.8125f,11.4258f,-1.7083f,11.3770f,-1.7083f,11.3770f,-1.6042f,11.3281f,-1.6042f,11.3281f,-1.5000f,11.2793f,-1.5000f,11.2793f,-1.3958f,11.2305f,-1.3958f,
	11.2305f,-1.2917f,11.1817f,-1.2917f,11.1817f,-1.1875f,11.1328f,-1.1875f,11.1328f,-1.0833f,11.0840f,-1.0833f,11.0840f,-0.9792f,11.0352f,-0.9792f,11.0352f,-0.8750f,10.9863f,-0.8750f,10.9863f,-0.7708f,10.9375f,-0.7708f,10.9375f,-0.6667f,10.8887f,-0.6667f,10.8887f,-0.5625f,
	10.8399f,-0.5625f,10.8399f,-0.4583f,10.7910f,-0.4583f,10.7910f,-0.3542f,10.7422f,-0.3542f,10.7422f,-0.2500f,10.6934f,-0.2500f,10.6934f,-0.1458f,10.6445f,-0.1458f,10.6445f,-0.0417f,10.5957f,-0.0417f,10.5957f,0.0625f,10.5469f,0.0625f,10.5469f,0.1667f,10.4981f,0.1667f,
	10.4981f,0.2708f,10.4492f,0.2708f,10.4492f,0.3750f,10.4004f,0.3750f,10.4004f,0.4792f,10.3516f,0.4792f,10.3516f,0.5833f,10.3027f,0.5833f,10.3027f,0.6875f,10.2539f,0.6875f,10.2539f,0.7917f,10.2051f,0.7917f,10.2051f,0.8958f,10.1563f,0.8958f,10.1563f,1.0000f,
	10.1074f,1.0000f,12.4512f,-4.0000f,12.4024f,-4.0000f,12.4024f,-3.8958f,12.3535f,-3.8958f,12.3535f,-3.7917f,12.3047f,-3.7917f,12.3047f,-3.6875f,12.2559f,-3.6875f,12.2559f,-3.5833f,12.2071f,-3.5833f,12.2071f,-3.4792f,12.1582f,-3.4792f,12.1582f,-3.3750f,12.1094f,-3.3750f,
	12.1094f,-3.2708f,12.0606f,-3.2708f,12.0606f,-3.1667f,12.0117f,-3.1667f,12.0117f,-3.0625f,11.9629f,-3.0625f,11.9629f,-2.9583f,11.9141f,-2.9583f,11.9141f,-2.8542f,11.8653f,-2.8542f,11.8653f,-2.7500f,11.8164f,-2.7500f,11.8164f,-2.6458f,11.7676f,-2.6458f,11.7676f,-2.5417f,
	11.7188f,-2.5417f,11.7188f,-2.4375f,11.6699f,-2.4375f,11.6699f,-2.3333f,11.6211f,-2.3333f,11.6211f,-2.2292f,11.5723f,-2.2292f,11.5723f,-2.1250f,11.5235f,-2.1250f,11.5235f,-2.0208f,11.4746f,-2.0208f,11.4746f,-1.9167f,11.4258f,-1.9167f,11.4258f,-1.8125f,11.3770f,-1.8125f,
	11.3770f,-1.7083f,11.3281f,-1.7083f,11.3281f,-1.6042f,11.2793f,-1.6042f,11.2793f,-1.5000f,11.2305f,-1.5000f,11.2305f,-1.3958f,11.1817f,-1.3958f,11.1817f,-1.2917f,11.1328f,-1.2917f,11.1328f,-1.1875f,11.0840f,-1.1875f,11.0840f,-1.0833f,11.0352f,-1.0833f,11.0352f,-0.9792f,
	10.9863f,-0.9792f,10.9863f,-0.8750f,10.9375f,-0.8750f,10.9375f,-0.7708f,10.8887f,-0.7708f,10.8887f,-0.6667f,10.8399f,-0.6667f,10.8399f,-0.5625f,10.7910f,-0.5625f,10.7910f,-0.4583f,10.7422f,-0.4583f,10.7422f,-0.3542f,10.6934f,-0.3542f,10.6934f,-0.2500f,10.6445f,-0.2500f,
	10.6445f,-0.1458f,10.5957f,-0.1458f,10.5957f,-0.0417f,10.5469f,-0.0417f,10.5469f,0.0625f,10.4981f,0.0625f,10.4981f,0.1667f,10.4492f,0.1667f,10.4492f,0.2708f,10.4004f,0.2708f,10.4004f,0.3750f,10.3516f,0.3750f,10.3516f,0.4792f,10.3027f,0.4792f,10.3027f,0.5833f,
	10.2539f,0.5833f,10.2539f,0.6875f,10.2051f,0.6875f,10.2051f,0.7917f,10.1563f,0.7917f,10.1563f,0.8958f,10.1074f,0.8958f,10.1074f,1.0000f,10.0586f,1.0000f,12.4024f,-4.0000f,12.3535f,-4.0000f,12.3535f,-3.8958f,12.3047f,-3.8958f,12.3047f,-3.7917f,12.2559f,-3.7917f,
	12.2559f,-3.6875f,12.2071f,-3.6875f,12.2071f,-3.5833f,12.1582f,-3.5833f,12.1582f,-3.4792f,12.1094f,-3.4792f,12.1094f,-3.3750f,12.0606f,-3.3750f,12.0606f,-3.2708f,12.0117f,-3.2708f,12.0117f,-3.1667f,11.9629f,-3.1667f,11.9629f,-3.0625f,11.9141f,-3.0625f,11.9141f,-2.9583f,
	11.8653f,-2.9583f,11.8653f,-2.8542f,11.8164f,-2.8542f,11.8164f,-2.7500f,11.7676f,-2.7500f,11.7676f,-2.6458f,11.7188f,-2.6458f,11.7188f,-2.5417f,11.6699f,-2.5417f,11.6699f,-2.4375f,11.6211f,-2.4375f,11.6211f,-2.3333f,11.5723f,-2.3333f,11.5723f,-2.2292f,11.5235f,-2.2292f,
	11.5235f,-2.1250f,11.4746f,-2.1250f,11.4746f,-2.0208f,11.4258f,-2.0208f,11.4258f,-1.9167f,11.3770f,-1.9167f,11.3770f,-1.8125f,11.3281f,-1.8125f,11.3281f,-1.7083f,11.2793f,-1.7083f,11.2793f,-1.6042f,11.2305f,-1.6042f,11.2305f,-1.5000f,11.1817f,-1.5000f,11.1817f,-1.3958f,
	11.1328f,-1.3958f,11.1328f,-1.2917f,11.0840f,-1.2917f,11.0840f,-1.1875f,11.0352f,-1.1875f,11.0352f,-1.0833f,10.9863f,-1.0833f,10.9863f,-0.9792f,10.9375f,-0.9792f,10.9375f,-0.8750f,10.8887f,-0.8750f,10.8887f,-0.7708f,10.8399f,-0.7708f,10.8399f,-0.6667f,10.7910f,-0.6667f,
	10.7910f,-0.5625f,10.7422f,-0.5625f,10.7422f,-0.4583f,10.6934f,-0.4583f,10.6934f,-0.3542f,10.6445f,-0.3542f,10.6445f,-0.2500f,10.5957f,-0.2500f,10.5957f,-0.1458f,10.5469f,-0.1458f,10.5469f,-0.0417f,10.4981f,-0.0417f,10.4981f,0.0625f,10.4492f,0.0625f,10.4492f,0.1667f,
	10.4004f,0.1667f,10.4004f,0.2708f,10.3516f,0.2708f,10.3516f,0.3750f,10.3027f,0.3750f,10.3027f,0.4792f,10.2539f,0.4792f,10.2539f,0.5833f,10.2051f,0.5833f,10.2051f,0.6875f,10.1563f,0.6875f,10.1563f,0.7917f,10.1074f,0.7917f,10.1074f,0.8958f,10.0586f,0.8958f,
	10.0586f,1.0000f,10.0098f,1.0000f,12.3535f,-4.0000f,12.3047f,-4.0000f,12.3047f,-3.8958f,12.2559f,-3.8958f,12.2559f,-3.7917f,12.2071f,-3.7917f,12.2071f,-3.6875f,12.1582f,-3.6875f,12.1582f,-3.5833f,12.1094f,-3.5833f,12.1094f,-3.4792f,12.0606f,-3.4792f,12.0606f,-3.3750f,
	12.0117f,-3.3750f,12.0117f,-3.2708f,11.9629f,-3.2708f,11.9629f,-3.1667f,11.9141f,-3.1667f,11.9141f,-3.0625f,11.8653f,-3.0625f,11.8653f,-2.9583f,11.8164f,-2.9583f,11.8164f,-2.8542f,11.7676f,-2.8542f,11.7676f,-2.7500f,11.7188f,-2.7500f,11.7188f,-2.6458f,11.6699f,-2.6458f,
	11.6699f,-2.5417f,11.6211f,-2.5417f,11.6211f,-2.4375f,11.5723f,-2.4375f,11.5723f,-2.3333f,11.5235f,-2.3333f,11.5235f,-2.2292f,11.4746f,-2.2292f,11.4746f,-2.1250f,11.4258f,-2.1250f,11.4258f,-2.0208f,11.3770f,-2.0208f,11.3770f,-1.9167f,11.3281f,-1.9167f,11.3281f,-1.8125f,
	11.2793f,-1.8125f,11.2793f,-1.7083f,11.2305f,-1.7083f,11.2305f,-1.6042f,11.1817f,-1.6042f,11.1817f,-1.5000f,11.1328f,-1.5000f,11.1328f,-1.3958f,11.0840f,-1.3958f,11.0840f,-1.2917f,11.0352f,-1.2917f,11.0352f,-1.1875f,10.9863f,-1.1875f,10.9863f,-1.0833f,10.9375f,-1.0833f,
	10.9375f,-0.9792f,10.8887f,-0.9792f,10.8887f,-0.8750f,10.8399f,-0.8750f,10.8399f,-0.7708f,10.7910f,-0.7708f,10.7910f,-0.6667f,10.7422f,-0.6667f,10.7422f,-0.5625f,10.6934f,-0.5625f,10.6934f,-0.4583f,10.6445f,-0.4583f,10.6445f,-0.3542f,10.5957f,-0.3542f,10.5957f,-0.2500f,
	10.5469f,-0.2500f,10.5469f,-0.1458f,10.4981f,-0.1458f,10.4981f,-0.0417f,10.4492f,-0.0417f,10.4492f,0.0625f,10.4004f,0.0625f,10.4004f,0.1667f,10.3516f,0.1667f,10.3516f,0.2708f,10.3027f,0.2708f,10.3027f,0.3750f,10.2539f,0.3750f,10.2539f,0.4792f,10.2051f,0.4792f,
	10.2051f,0.5833f,10.1563f,0.5833f,10.1563f,0.6875f,10.1074f,0.6875f,10.1074f,0.7917f,10.0586f,0.7917f,10.0586f,0.8958f,10.0098f,0.8958f,10.0098f,1.0000f,9.9609f,1.0000f,12.3047f,-4.0000f,12.2559f,-4.0000f,12.2559f,-3.8958f,12.2071f,-3.8958f,12.2071f,-3.7917f,
	12.1582f,-3.7917f,12.1582f,-3.6875f,12.1094f,-3.6875f,12.1094f,-3.5833f,12.0606f,-3.5833f,12.0606f,-3.4792f,12.0117f,-3.4792f,12.0117f,-3.3750f,11.9629f,-3.3750f,11.9629f,-3.2708f,11.9141f,-3.2708f,11.9141f,-3.1667f,11.8653f,-3.1667f,11.8653f,-3.0625f,11.8164f,-3.0625f,
	11.8164f,-2.9583f,11.7676f,-2.9583f,11.7676f,-2.8542f,11.7188f,-2.8542f,11.7188f,-2.7500f,11.6699f,-2.7500f,11.6699f,-2.6458f,11.6211f,-2.6458f,11.6211f,-2.5417f,11.5723f,-2.5417f,11.5723f,-2.4375f,11.5235f,-2.4375f,11.5235f,-2.3333f,11.4746f,-2.3333f,11.4746f,-2.2292f,
	11.4258f,-2.2292f,11.4258f,-2.1250f,11.3770f,-2.1250f,11.3770f,-2.0208f,11.3281f,-2.0208f,11.3281f,-1.9167f,11.2793f,-1.9167f,11.2793f,-1.8125f,11.2305f,-1.8125f,11.2305f,-1.7083f,11.1817f,-1.7083f,11.1817f,-1.6042f,11.1328f,-1.6042f,11.1328f,-1.5000f,11.0840f,-1.5000f,
	11.0840f,-1.3958f,11.0352f,-1.3958f,11.0352f,-1.2917f,10.9863f,-1.2917f,10.9863f,-1.1875f,10.9375f,-1.1875f,10.9375f,-1.0833f,10.8887f,-1.0833f,10.8887f,-0.9792f,10.8399f,-0.9792f,10.8399f,-0.8750f,10.7910f,-0.8750f,10.7910f,-0.7708f,10.7422f,-0.7708f,10.7422f,-0.6667f,
	10.6934f,-0.6667f,10.6934f,-0.5625f,10.6445f,-0.5625f,10.6445f,-0.4583f,10.5957f,-0.4583f,10.5957f,-0.3542f,10.5469f,-0.3542f,10.5469f,-0.2500f,10.4981f,-0.2500f,10.4981f,-0.1458f,10.4492f,-0.1458f,10.4492f,-0.0417f,10.4004f,-0.0417f,10.4004f,0.0625f,10.3516f,0.0625f,
	10.3516f,0.1667f,10.3027f,0.1667f,10.3027f,0.2708f,10.2539f,0.2708f,10.2539f,0.3750f,10.2051f,0.3750f,10.2051f,0.4792f,10.1563f,0.4792f,10.1563f,0.5833f,10.1074f,0.5833f,10.1074f,0.6875f,10.0586f,0.6875f,10.0586f,0.7917f,10.0098f,0.7917f,10.0098f,0.8958f,
	9.9609f,0.8958f,9.9609f,1.0000f,9.9121f,1.0000f,12.2559f,-4.0000f,12.2071f,-4.0000f,12.2071f,-3.8958f,12.1582f,-3.8958f,12.1582f,-3.7917f,12.1094f,-3.7917f,12.1094f,-3.6875f,12.0606f,-3.6875f,12.0606f,-3.5833f,12.0117f,-3.5833f,12.0117f,-3.4792f,11.9629f,-3.4792f,
	11.9629f,-3.3750f,11.9141f,-3.3750f,11.9141f,-3.2708f,11.8653f,-3.2708f,11.8653f,-3.1667f,11.8164f,-3.1667f,11.8164f,-3.0625f,11.7676f,-3.0625f,11.7676f,-2.9583f,11.7188f,-2.9583f,11.7188f,-2.8542f,11.6699f,-2.8542f,11.6699f,-2.7500f,11.6211f,-2.7500f,11.6211f,-2.6458f,
	11.5723f,-2.6458f,11.5723f,-2.5417f,11.5235f,-2.5417f,11.5235f,-2.4375f,11.4746f,-2.4375f,11.4746f,-2.3333f,11.4258f,-2.3333f,11.4258f,-2.2292f,11.3770f,-2.2292f,11.3770f,-2.1250f,11.3281f,-2.1250f,11.3281f,-2.0208f,11.2793f,-2.0208f,11.2793f,-1.9167f,11.2305f,-1.9167f,
	11.2305f,-1.8125f,11.1817f,-1.8125f,11.1817f,-1.7083f,11.1328f,-1.7083f,11.1328f,-1.6042f,11.0840f,-1.6042f,11.0840f,-1.5000f,11.0352f,-1.5000f,11.0352f,-1.3958f,10.9863f,-1.3958f,10.9863f,-1.2917f,10.9375f,-1.2917f,10.9375f,-1.1875f,10.8887f,-1.1875f,10.8887f,-1.0833f,
	10.8399f,-1.0833f,10.8399f,-0.9792f,10.7910f,-0.9792f,10.7910f,-0.8750f,10.7422f,-0.8750f,10.7422f,-0.7708f,10.6934f,-0.7708f,10.6934f,-0.6667f,10.6445f,-0.6667f,10.6445f,-0.5625f,10.5957f,-0.5625f,10.5957f,-0.4583f,10.5469f,-0.4583f,10.5469f,-0.3542f,10.4981f,-0.3542f,
	10.4981f,-0.2500f,10.4492f,-0.2500f,10.4492f,-0.1458f,10.4004f,-0.1458f,10.4004f,-0.0417f,10.3516f,-0.0417f,10.3516f,0.0625f,10.3027f,0.0625f,10.3027f,0.1667f,10.2539f,0.1667f,10.2539f,0.2708f,10.2051f,0.2708f,10.2051f,0.3750f,10.1563f,0.3750f,10.1563f,0.4792f,
	10.1074f,0.4792f,10.1074f,0.5833f,10.0586f,0.5833f,10.0586f,0.6875f,10.0098f,0.6875f,10.0098f,0.7917f,9.9609f,0.7917f,9.9609f,0.8958f,9.9121f,0.8958f,9.9121f,1.0000f,9.8633f,1.0000f,12.2071f,-4.0000f,12.1582f,-4.0000f,12.1582f,-3.8958f,12.1094f,-3.8958f,
	12.1094f,-3.7917f,12.0606f,-3.7917f,12.0606f,-3.6875f,12.0117f,-3.6875f,12.0117f,-3.5833f,11.9629f,-3.5833f,11.9629f,-3.4792f,11.9141f,-3.4792f,11.9141f,-3.3750f,11.8653f,-3.3750f,11.8653f,-3.2708f,11.8164f,-3.2708f,11.8164f,-3.1667f,11.7676f,-3.1667f,11.7676f,-3.0625f,
	11.7188f,-3.0625f,11.7188f,-2.9583f,11.6699f,-2.9583f,11.6699f,-2.8542f,11.6211f,-2.8542f,11.6211f,-2.7500f,11.5723f,-2.7500f,11.5723f,-2.6458f,11.5235f,-2.6458f,11.5235f,-2.5417f,11.4746f,-2.5417f,11.4746f,-2.4375f,11.4258f,-2.4375f,11.4258f,-2.3333f,11.3770f,-2.3333f,
	11.3770f,-2.2292f,11.3281f,-2.2292f,11.3281f,-2.1250f,11.2793f,-2.1250f,11.2793f,-2.0208f,11.2305f,-2.0208f,11.2305f,-1.9167f,11.1817f,-1.9167f,11.1817f,-1.8125f,11.1328f,-1.8125f,11.1328f,-1.7083f,11.0840f,-1.7083f,11.0840f,-1.6042f,11.0352f,-1.6042f,11.0352f,-1.5000f,
	10.9863f,-1.5000f,10.9863f,-1.3958f,10.9375f,-1.3958f,10.9375f,-1.2917f,10.8887f,-1.2917f,10.8887f,-1.1875f,10.8399f,-1.1875f,10.8399f,-1.0833f,10.7910f,-1.0833f,10.7910f,-0.9792f,10.7422f,-0.9792f,10.7422f,-0.8750f,10.6934f,-0.8750f,10.6934f,-0.7708f,10.6445f,-0.7708f,
	10.6445f,-0.6667f,10.5957f,-0.6667f,10.5957f,-0.5625f,10.5469f,-0.5625f,10.5469f,-0.4583f,10.4981f,-0.4583f,10.4981f,-0.3542f,10.4492f,-0.3542f,10.4492f,-0.2500f,10.4004f,-0.2500f,10.4004f,-0.1458f,10.3516f,-0.1458f,10.3516f,-0.0417f,10.3027f,-0.0417f,10.3027f,0.0625f,
	10.2539f,0.0625f,10.2539f,0.1667f,10.2051f,0.1667f,10.2051f,0.2708f,10.1563f,0.2708f,10.1563f,0.3750f,10.1074f,0.3750f,10.1074f,0.4792f,10.0586f,0.4792f,10.0586f,0.5833f,10.0098f,0.5833f,10.0098f,0.6875f,9.9609f,0.6875f,9.9609f,0.7917f,9.9121f,0.7917f,
	9.9121f,0.8958f,9.8633f,0.8958f,9.8633f,1.0000f,9.8145f,1.0000f,12.1582f,-4.0000f,12.1094f,-4.0000f,12.1094f,-3.8958f,12.0606f,-3.8958f,12.0606f,-3.7917f,12.0117f,-3.7917f,12.0117f,-3.6875f,11.9629f,-3.6875f,11.9629f,-3.5833f,11.9141f,-3.5833f,11.9141f,-3.4792f,
	11.8653f,-3.4792f,11.8653f,-3.3750f,11.8164f,-3.3750f,11.8164f,-3.2708f,11.7676f,-3.2708f,11.7676f,-3.1667f,11.7188f,-3.1667f,11.7188f,-3.0625f,11.6699f,-3.0625f,11.6699f,-2.9583f,11.6211f,-2.9583f,11.6211f,-2.8542f,11.5723f,-2.8542f,11.5723f,-2.7500f,11.5235f,-2.7500f,
	11.5235f,-2.6458f,11.4746f,-2.6458f,11.4746f,-2.5417f,11.4258f,-2.5417f,11.4258f,-2.4375f,11.3770f,-2.4375f,11.3770f,-2.3333f,11.3281f,-2.3333f,11.3281f,-2.2292f,11.2793f,-2.2292f,11.2793f,-2.1250f,11.2305f,-2.1250f,11.2305f,-2.0208f,11.1817f,-2.0208f,11.1817f,-1.9167f,
	11.1328f,-1.9167f,11.1328f,-1.8125f,11.0840f,-1.8125f,11.0840f,-1.7083f,11.0352f,-1.7083f,11.0352f,-1.6042f,10.9863f,-1.6042f,10.9863f,-1.5000f,10.9375f,-1.5000f,10.9375f,-1.3958f,10.8887f,-1.3958f,10.8887f,-1.2917f,10.8399f,-1.2917f,10.8399f,-1.1875f,10.7910f,-1.1875f,
	10.7910f,-1.0833f,10.7422f,-1.0833f,10.7422f,-0.9792f,10.6934f,-0.9792f,10.6934f,-0.8750f,10.6445f,-0.8750f,10.6445f,-0.7708f,10.5957f,-0.7708f,10.5957f,-0.6667f,10.5469f,-0.6667f,10.5469f,-0.5625f,10.4981f,-0.5625f,10.4981f,-0.4583f,10.4492f,-0.4583f,10.4492f,-0.3542f,
	10.4004f,-0.3542f,10.4004f,-0.2500f,10.3516f,-0.2500f,10.3516f,-0.1458f,10.3027f,-0.1458f,10.3027f,-0.0417f,10.2539f,-0.0417f,10.2539f,0.0625f,10.2051f,0.0625f,10.2051f,0.1667f,10.1563f,0.1667f,10.1563f,0.2708f,10.1074f,0.2708f,10.1074f,0.3750f,10.0586f,0.3750f,
	10.0586f,0.4792f,10.0098f,0.4792f,10.0098f,0.5833f,9.9609f,0.5833f,9.9609f,0.6875f,9.9121f,0.6875f,9.9121f,0.7917f,9.8633f,0.7917f,9.8633f,0.8958f,9.8145f,0.8958f,9.8145f,1.0000f,9.7656f,1.0000f,12.1094f,-4.0000f,12.0606f,-4.0000f,12.0606f,-3.8958f,
	12.0117f,-3.8958f,12.0117f,-3.7917f,11.9629f,-3.7917f,11.9629f,-3.6875f,11.9141f,-3.6875f,11.9141f,-3.5833f,11.8653f,-3.5833f,11.8653f,-3.4792f,11.8164f,-3.4792f,11.8164f,-3.3750f,11.7676f,-3.3750f,11.7676f,-3.2708f,11.7188f,-3.2708f,11.7188f,-3.1667f,11.6699f,-3.1667f,
	11.6699f,-3.0625f,11.6211f,-3.0625f,11.6211f,-2.9583f,11.5723f,-2.9583f,11.5723f,-2.8542f,11.5235f,-2.8542f,11.5235f,-2.7500f,11.4746f,-2.7500f,11.4746f,-2.6458f,11.4258f,-2.6458f,11.4258f,-2.5417f,11.3770f,-2.5417f,11.3770f,-2.4375f,11.3281f,-2.4375f,11.3281f,-2.3333f,
	11.2793f,-2.3333f,11.2793f,-2.2292f,11.2305f,-2.2292f,11.2305f,-2.1250f,11.1817f,-2.1250f,11.1817f,-2.0208f,11.1328f,-2.0208f,11.1328f,-1.9167f,11.0840f,-1.9167f,11.0840f,-1.8125f,11.0352f,-1.8125f,11.0352f,-1.7083f,10.9863f,-1.7083f,10.9863f,-1.6042f,10.9375f,-1.6042f,
	10.9375f,-1.5000f,10.8887f,-1.5000f,10.8887f,-1.3958f,10.8399f,-1.3958f,10.8399f,-1.2917f,10.7910f,-1.2917f,10.7910f,-1.1875f,10.7422f,-1.1875f,10.7422f,-1.0833f,10.6934f,-1.0833f,10.6934f,-0.9792f,10.6445f,-0.9792f,10.6445f,-0.8750f,10.5957f,-0.8750f,10.5957f,-0.7708f,
	10.5469f,-0.7708f,10.5469f,-0.6667f,10.4981f,-0.6667f,10.4981f,-0.5625f,10.4492f,-0.5625f,10.4492f,-0.4583f,10.4004f,-0.4583f,10.4004f,-0.3542f,10.3516f,-0.3542f,10.3516f,-0.2500f,10.3027f,-0.2500f,10.3027f,-0.1458f,10.2539f,-0.1458f,10.2539f,-0.0417f,10.2051f,-0.0417f,
	10.2051f,0.0625f,10.1563f,0.0625f,10.1563f,0.1667f,10.1074f,0.1667f,10.1074f,0.2708f,10.0586f,0.2708f,10.0586f,0.3750f,10.0098f,0.3750f,10.0098f,0.4792f,9.9609f,0.4792f,9.9609f,0.5833f,9.9121f,0.5833f,9.9121f,0.6875f,9.8633f,0.6875f,9.8633f,0.7917f,
	9.8145f,0.7917f,9.8145f,0.8958f,9.7656f,0.8958f,9.7656f,1.0000f,9.7168f,1.0000f,12.0606f,-4.0000f,12.0117f,-4.0000f,12.0117f,-3.8958f,11.9629f,-3.8958f,11.9629f,-3.7917f,11.9141f,-3.7917f,11.9141f,-3.6875f,11.8653f,-3.6875f,11.8653f,-3.5833f,11.8164f,-3.5833f,
	11.8164f,-3.4792f,11.7676f,-3.4792f,11.7676f,-3.3750f,11.7188f,-3.3750f,11.7188f,-3.2708f,11.6699f,-3.2708f,11.6699f,-3.1667f,11.6211f,-3.1667f,11.6211f,-3.0625f,11.5723f,-3.0625f,11.5723f,-2.9583f,11.5235f,-2.9583f,11.5235f,-2.8542f,11.4746f,-2.8542f,11.4746f,-2.7500f,
	11.4258f,-2.7500f,11.4258f,-2.6458f,11.3770f,-2.6458f,11.3770f,-2.5417f,11.3281f,-2.5417f,11.3281f,-2.4375f,11.2793f,-2.4375f,11.2793f,-2.3333f,11.2305f,-2.3333f,11.2305f,-2.2292f,11.1817f,-2.2292f,11.1817f,-2.1250f,11.1328f,-2.1250f,11.1328f,-2.0208f,11.0840f,-2.0208f,
	11.0840f,-1.9167f,11.0352f,-1.9167f,11.0352f,-1.8125f,10.9863f,-1.8125f,10.9863f,-1.7083f,10.9375f,-1.7083f,10.9375f,-1.6042f,10.8887f,-1.6042f,10.8887f,-1.5000f,10.8399f,-1.5000f,10.8399f,-1.3958f,10.7910f,-1.3958f,10.7910f,-1.2917f,10.7422f,-1.2917f,10.7422f,-1.1875f,
	10.6934f,-1.1875f,10.6934f,-1.0833f,10.6445f,-1.0833f,10.6445f,-0.9792f,10.5957f,-0.9792f,10.5957f,-0.8750f,10.5469f,-0.8750f,10.5469f,-0.7708f,10.4981f,-0.7708f,10.4981f,-0.6667f,10.4492f,-0.6667f,10.4492f,-0.5625f,10.4004f,-0.5625f,10.4004f,-0.4583f,10.3516f,-0.4583f,
	10.3516f,-0.3542f,10.3027f,-0.3542f,10.3027f,-0.2500f,10.2539f,-0.2500f,10.2539f,-0.1458f,10.2051f,-0.1458f,10.2051f,-0.0417f,10.1563f,-0.0417f,10.1563f,0.0625f,10.1074f,0.0625f,10.1074f,0.1667f,10.0586f,0.1667f,10.0586f,0.2708f,10.0098f,0.2708f,10.0098f,0.3750f,
	9.9609f,0.3750f,9.9609f,0.4792f,9.9121f,0.4792f,9.9121f,0.5833f,9.8633f,0.5833f,9.8633f,0.6875f,9.8145f,0.6875f,9.8145f,0.7917f,9.7656f,0.7917f,9.7656f,0.8958f,9.7168f,0.8958f,9.7168f,1.0000f,9.6680f,1.0000f,12.0117f,-4.0000f,11.9629f,-4.0000f,
	11.9629f,-3.8958f,11.9141f,-3.8958f,11.9141f,-3.7917f,11.8653f,-3.7917f,11.8653f,-3.6875f,11.8164f,-3.6875f,11.8164f,-3.5833f,11.7676f,-3.5833f,11.7676f,-3.4792f,11.7188f,-3.4792f,11.7188f,-3.3750f,11.6699f,-3.3750f,11.6699f,-3.2708f,11.6211f,-3.2708f,11.6211f,-3.1667f,
	11.5723f,-3.1667f,11.5723f,-3.0625f,11.5235f,-3.0625f,11.5235f,-2.9583f,11.4746f,-2.9583f,11.4746f,-2.8542f,11.4258f,-2.8542f,11.4258f,-2.7500f,11.3770f,-2.7500f,11.3770f,-2.6458f,11.3281f,-2.6458f,11.3281f,-2.5417f,11.2793f,-2.5417f,11.2793f,-2.4375f,11.2305f,-2.4375f,
	11.2305f,-2.3333f,11.1817f,-2.3333f,11.1817f,-2.2292f,11.1328f,-2.2292f,11.1328f,-2.1250f,11.0840f,-2.1250f,11.0840f,-2.0208f,11.0352f,-2.0208f,11.0352f,-1.9167f,10.9863f,-1.9167f,10.9863f,-1.8125f,10.9375f,-1.8125f,10.9375f,-1.7083f,10.8887f,-1.7083f,10.8887f,-1.6042f,
	10.8399f,-1.6042f,10.8399f,-1.5000f,10.7910f,-1.5000f,10.7910f,-1.3958f,10.7422f,-1.3958f,10.7422f,-1.2917f,10.6934f,-1.2917f,10.6934f,-1.1875f,10.6445f,-1.1875f,10.6445f,-1.0833f,10.5957f,-1.0833f,10.5957f,-0.9792f,10.5469f,-0.9792f,10.5469f,-0.8750f,10.4981f,-0.8750f,
	10.4981f,-0.7708f,10.4492f,-0.7708f,10.4492f,-0.6667f,10.4004f,-0.6667f,10.4004f,-0.5625f,10.3516f,-0.5625f,10.3516f,-0.4583f,10.3027f,-0.4583f,10.3027f,-0.3542f,10.2539f,-0.3542f,10.2539f,-0.2500f,10.2051f,-0.2500f,10.2051f,-0.1458f,10.1563f,-0.1458f,10.1563f,-0.0417f,
	10.1074f,-0.0417f,10.1074f,0.0625f,10.0586f,0.0625f,10.0586f,0.1667f,10.0098f,0.1667f,10.0098f,0.2708f,9.9609f,0.2708f,9.9609f,0.3750f,9.9121f,0.3750f,9.9121f,0.4792f,9.8633f,0.4792f,9.8633f,0.5833f,9.8145f,0.5833f,9.8145f,0.6875f,9.7656f,0.6875f,
	9.7656f,0.7917f,9.7168f,0.7917f,9.7168f,0.8958f,9.6680f,0.8958f,9.6680f,1.0000f,9.6192f,1.0000f,11.9629f,-4.0000f,11.9141f,-4.0000f,11.9141f,-3.8958f,11.8653f,-3.8958f,11.8653f,-3.7917f,11.8164f,-3.7917f,11.8164f,-3.6875f,11.7676f,-3.6875f,11.7676f,-3.5833f,
	11.7188f,-3.5833f,11.7188f,-3.4792f,11.6699f,-3.4792f,11.6699f,-3.3750f,11.6211f,-3.3750f,11.6211f,-3.2708f,11.5723f,-3.2708f,11.5723f,-3.1667f,11.5235f,-3.1667f,11.5235f,-3.0625f,11.4746f,-3.0625f,11.4746f,-2.9583f,11.4258f,-2.9583f,11.4258f,-2.8542f,11.3770f,-2.8542f,
	11.3770f,-2.7500f,11.3281f,-2.7500f,11.3281f,-2.6458f,11.2793f,-2.6458f,11.2793f,-2.5417f,11.2305f,-2.5417f,11.2305f,-2.4375f,11.1817f,-2.4375f,11.1817f,-2.3333f,11.1328f,-2.3333f,11.1328f,-2.2292f,11.0840f,-2.2292f,11.0840f,-2.1250f,11.0352f,-2.1250f,11.0352f,-2.0208f,
	10.9863f,-2.0208f,10.9863f,-1.9167f,10.9375f,-1.9167f,10.9375f,-1.8125f,10.8887f,-1.8125f,10.8887f,-1.7083f,10.8399f,-1.7083f,10.8399f,-1.6042f,10.7910f,-1.6042f,10.7910f,-1.5000f,10.7422f,-1.5000f,10.7422f,-1.3958f,10.6934f,-1.3958f,10.6934f,-1.2917f,10.6445f,-1.2917f,
	10.6445f,-1.1875f,10.5957f,-1.1875f,10.5957f,-1.0833f,10.5469f,-1.0833f,10.5469f,-0.9792f,10.4981f,-0.9792f,10.4981f,-0.8750f,10.4492f,-0.8750f,10.4492f,-0.7708f,10.4004f,-0.7708f,10.4004f,-0.6667f,10.3516f,-0.6667f,10.3516f,-0.5625f,10.3027f,-0.5625f,10.3027f,-0.4583f,
	10.2539f,-0.4583f,10.2539f,-0.3542f,10.2051f,-0.3542f,10.2051f,-0.2500f,10.1563f,-0.2500f,10.1563f,-0.1458f,10.1074f,-0.1458f,10.1074f,-0.0417f,10.0586f,-0.0417f,10.0586f,0.0625f,10.0098f,0.0625f,10.0098f,0.1667f,9.9609f,0.1667f,9.9609f,0.2708f,9.9121f,0.2708f,
	9.9121f,0.3750f,9.8633f,0.3750f,9.8633f,0.4792f,9.8145f,0.4792f,9.8145f,0.5833f,9.7656f,0.5833f,9.7656f,0.6875f,9.7168f,0.6875f,9.7168f,0.7917f,9.6680f,0.7917f,9.6680f,0.8958f,9.6192f,0.8958f,9.6192f,1.0000f,9.5703f,1.0000f,11.9141f,-4.0000f,
	11.8653f,-4.0000f,11.8653f,-3.8958f,11.8164f,-3.8958f,11.8164f,-3.7917f,11.7676f,-3.7917f,11.7676f,-3.6875f,11.7188f,-3.6875f,11.7188f,-3.5833f,11.6699f,-3.5833f,11.6699f,-3.4792f,11.6211f,-3.4792f,11.6211f,-3.3750f,11.5723f,-3.3750f,11.5723f,-3.2708f,11.5235f,-3.2708f,
	11.5235f,-3.1667f,11.4746f,-3.1667f,11.4746f,-3.0625f,11.4258f,-3.0625f,11.4258f,-2.9583f,11.3770f,-2.9583f,11.3770f,-2.8542f,11.3281f,-2.8542f,11.3281f,-2.7500f,11.2793f,-2.7500f,11.2793f,-2.6458f,11.2305f,-2.6458f,11.2305f,-2.5417f,11.1817f,-2.5417f,11.1817f,-2.4375f,
	11.1328f,-2.4375f,11.1328f,-2.3333f,11.0840f,-2.3333f,11.0840f,-2.2292f,11.0352f,-2.2292f,11.0352f,-2.1250f,10.9863f,-2.1250f,10.9863f,-2.0208f,10.9375f,-2.0208f,10.9375f,-1.9167f,10.8887f,-1.9167f,10.8887f,-1.8125f,10.8399f,-1.8125f,10.8399f,-1.7083f,10.7910f,-1.7083f,
	10.7910f,-1.6042f,10.7422f,-1.6042f,10.7422f,-1.5000f,10.6934f,-1.5000f,10.6934f,-1.3958f,10.6445f,-1.3958f,10.6445f,-1.2917f,10.5957f,-1.2917f,10.5957f,-1.1875f,10.5469f,-1.1875f,10.5469f,-1.0833f,10.4981f,-1.0833f,10.4981f,-0.9792f,10.4492f,-0.9792f,10.4492f,-0.8750f,
	10.4004f,-0.8750f,10.4004f,-0.7708f,10.3516f,-0.7708f,10.3516f,-0.6667f,10.3027f,-0.6667f,10.3027f,-0.5625f,10.2539f,-0.5625f,10.2539f,-0.4583f,10.2051f,-0.4583f,10.2051f,-0.3542f,10.1563f,-0.3542f,10.1563f,-0.2500f,10.1074f,-0.2500f,10.1074f,-0.1458f,10.0586f,-0.1458f,
	10.0586f,-0.0417f,10.0098f,-0.0417f,10.0098f,0.0625f,9.9609f,0.0625f,9.9609f,0.1667f,9.9121f,0.1667f,9.9121f,0.2708f,9.8633f,0.2708f,9.8633f,0.3750f,9.8145f,0.3750f,9.8145f,0.4792f,9.7656f,0.4792f,9.7656f,0.5833f,9.7168f,0.5833f,9.7168f,0.6875f,
	9.6680f,0.6875f,9.6680f,0.7917f,9.6192f,0.7917f,9.6192f,0.8958f,9.5703f,0.8958f,9.5703f,1.0000f,9.5215f,1.0000f,11.8653f,-4.0000f,11.8164f,-4.0000f,11.8164f,-3.8958f,11.7676f,-3.8958f,11.7676f,-3.7917f,11.7188f,-3.7917f,11.7188f,-3.6875f,11.6699f,-3.6875f,
	11.6699f,-3.5833f,11.6211f,-3.5833f,11.6211f,-3.4792f,11.5723f,-3.4792f,11.5723f,-3.3750f,11.5235f,-3.3750f,11.5235f,-3.2708f,11.4746f,-3.2708f,11.4746f,-3.1667f,11.4258f,-3.1667f,11.4258f,-3.0625f,11.3770f,-3.0625f,11.3770f,-2.9583f,11.3281f,-2.9583f,11.3281f,-2.8542f,
	11.2793f,-2.8542f,11.2793f,-2.7500f,11.2305f,-2.7500f,11.2305f,-2.6458f,11.1817f,-2.6458f,11.1817f,-2.5417f,11.1328f,-2.5417f,11.1328f,-2.4375f,11.0840f,-2.4375f,11.0840f,-2.3333f,11.0352f,-2.3333f,11.0352f,-2.2292f,10.9863f,-2.2292f,10.9863f,-2.1250f,10.9375f,-2.1250f,
	10.9375f,-2.0208f,10.8887f,-2.0208f,10.8887f,-1.9167f,10.8399f,-1.9167f,10.8399f,-1.8125f,10.7910f,-1.8125f,10.7910f,-1.7083f,10.7422f,-1.7083f,10.7422f,-1.6042f,10.6934f,-1.6042f,10.6934f,-1.5000f,10.6445f,-1.5000f,10.6445f,-1.3958f,10.5957f,-1.3958f,10.5957f,-1.2917f,
	10.5469f,-1.2917f,10.5469f,-1.1875f,10.4981f,-1.1875f,10.4981f,-1.0833f,10.4492f,-1.0833f,10.4492f,-0.9792f,10.4004f,-0.9792f,10.4004f,-0.8750f,10.3516f,-0.8750f,10.3516f,-0.7708f,10.3027f,-0.7708f,10.3027f,-0.6667f,10.2539f,-0.6667f,10.2539f,-0.5625f,10.2051f,-0.5625f,
	10.2051f,-0.4583f,10.1563f,-0.4583f,10.1563f,-0.3542f,10.1074f,-0.3542f,10.1074f,-0.2500f,10.0586f,-0.2500f,10.0586f,-0.1458f,10.0098f,-0.1458f,10.0098f,-0.0417f,9.9609f,-0.0417f,9.9609f,0.0625f,9.9121f,0.0625f,9.9121f,0.1667f,9.8633f,0.1667f,9.8633f,0.2708f,
	9.8145f,0.2708f,9.8145f,0.3750f,9.7656f,0.3750f,9.7656f,0.4792f,9.7168f,0.4792f,9.7168f,0.5833f,9.6680f,0.5833f,9.6680f,0.6875f,9.6192f,0.6875f,9.6192f,0.7917f,9.5703f,0.7917f,9.5703f,0.8958f,9.5215f,0.8958f,9.5215f,1.0000f,9.4727f,1.0000f,
	11.8164f,-4.0000f,11.7676f,-4.0000f,11.7676f,-3.8958f,11.7188f,-3.8958f,11.7188f,-3.7917f,11.6699f,-3.7917f,11.6699f,-3.6875f,11.6211f,-3.6875f,11.6211f,-3.5833f,11.5723f,-3.5833f,11.5723f,-3.4792f,11.5235f,-3.4792f,11.5235f,-3.3750f,11.4746f,-3.3750f,11.4746f,-3.2708f,
	11.4258f,-3.2708f,11.4258f,-3.1667f,11.3770f,-3.1667f,11.3770f,-3.0625f,11.3281f,-3.0625f,11.3281f,-2.9583f,11.2793f,-2.9583f,11.2793f,-2.8542f,11.2305f,-2.8542f,11.2305f,-2.7500f,11.1817f,-2.7500f,11.1817f,-2.6458f,11.1328f,-2.6458f,11.1328f,-2.5417f,11.0840f,-2.5417f,
	11.0840f,-2.4375f,11.0352f,-2.4375f,11.0352f,-2.3333f,10.9863f,-2.3333f,10.9863f,-2.2292f,10.9375f,-2.2292f,10.9375f,-2.1250f,10.8887f,-2.1250f,10.8887f,-2.0208f,10.8399f,-2.0208f,10.8399f,-1.9167f,10.7910f,-1.9167f,10.7910f,-1.8125f,10.7422f,-1.8125f,10.7422f,-1.7083f,
	10.6934f,-1.7083f,10.6934f,-1.6042f,10.6445f,-1.6042f,10.6445f,-1.5000f,10.5957f,-1.5000f,10.5957f,-1.3958f,10.5469f,-1.3958f,10.5469f,-1.2917f,10.4981f,-1.2917f,10.4981f,-1.1875f,10.4492f,-1.1875f,10.4492f,-1.0833f,10.4004f,-1.0833f,10.4004f,-0.9792f,10.3516f,-0.9792f,
	10.3516f,-0.8750f,10.3027f,-0.8750f,10.3027f,-0.7708f,10.2539f,-0.7708f,10.2539f,-0.6667f,10.2051f,-0.6667f,10.2051f,-0.5625f,10.1563f,-0.5625f,10.1563f,-0.4583f,10.1074f,-0.4583f,10.1074f,-0.3542f,10.0586f,-0.3542f,10.0586f,-0.2500f,10.0098f,-0.2500f,10.0098f,-0.1458f,
	9.9609f,-0.1458f,9.9609f,-0.0417f,9.9121f,-0.0417f,9.9121f,0.0625f,9.8633f,0.0625f,9.8633f,0.1667f,9.8145f,0.1667f,9.8145f,0.2708f,9.7656f,0.2708f,9.7656f,0.3750f,9.7168f,0.3750f,9.7168f,0.4792f,9.6680f,0.4792f,9.6680f,0.5833f,9.6192f,0.5833f,
	9.6192f,0.6875f,9.5703f,0.6875f,9.5703f,0.7917f,9.5215f,0.7917f,9.5215f,0.8958f,9.4727f,0.8958f,9.4727f,1.0000f,9.4238f,1.0000f,11.7676f,-4.0000f,11.7188f,-4.0000f,11.7188f,-3.8958f,11.6699f,-3.8958f,11.6699f,-3.7917f,11.6211f,-3.7917f,11.6211f,-3.6875f,
	11.5723f,-3.6875f,11.5723f,-3.5833f,11.5235f,-3.5833f,11.5235f,-3.4792f,11.4746f,-3.4792f,11.4746f,-3.3750f,11.4258f,-3.3750f,11.4258f,-3.2708f,11.3770f,-3.2708f,11.3770f,-3.1667f,11.3281f,-3.1667f,11.3281f,-3.0625f,11.2793f,-3.0625f,11.2793f,-2.9583f,11.2305f,-2.9583f,
	11.2305f,-2.8542f,11.1817f,-2.8542f,11.1817f,-2.7500f,11.1328f,-2.7500f,11.1328f,-2.6458f,11.0840f,-2.6458f,11.0840f,-2.5417f,11.0352f,-2.5417f,11.0352f,-2.4375f,10.9863f,-2.4375f,10.9863f,-2.3333f,10.9375f,-2.3333f,10.9375f,-2.2292f,10.8887f,-2.2292f,10.8887f,-2.1250f,
	10.8399f,-2.1250f,10.8399f,-2.0208f,10.7910f,-2.0208f,10.7910f,-1.9167f,10.7422f,-1.9167f,10.7422f,-1.8125f,10.6934f,-1.8125f,10.6934f,-1.7083f,10.6445f,-1.7083f,10.6445f,-1.6042f,10.5957f,-1.6042f,10.5957f,-1.5000f,10.5469f,-1.5000f,10.5469f,-1.3958f,10.4981f,-1.3958f,
	10.4981f,-1.2917f,10.4492f,-1.2917f,10.4492f,-1.1875f,10.4004f,-1.1875f,10.4004f,-1.0833f,10.3516f,-1.0833f,10.3516f,-0.9792f,10.3027f,-0.9792f,10.3027f,-0.8750f,10.2539f,-0.8750f,10.2539f,-0.7708f,10.2051f,-0.7708f,10.2051f,-0.6667f,10.1563f,-0.6667f,10.1563f,-0.5625f,
	10.1074f,-0.5625f,10.1074f,-0.4583f,10.0586f,-0.4583f,10.0586f,-0.3542f,10.0098f,-0.3542f,10.0098f,-0.2500f,9.9609f,-0.2500f,9.9609f,-0.1458f,9.9121f,-0.1458f,9.9121f,-0.0417f,9.8633f,-0.0417f,9.8633f,0.0625f,9.8145f,0.0625f,9.8145f,0.1667f,9.7656f,0.1667f,
	9.7656f,0.2708f,9.7168f,0.2708f,9.7168f,0.3750f,9.6680f,0.3750f,9.6680f,0.4792f,9.6192f,0.4792f,9.6192f,0.5833f,9.5703f,0.5833f,9.5703f,0.6875f,9.5215f,0.6875f,9.5215f,0.7917f,9.4727f,0.7917f,9.4727f,0.8958f,9.4238f,0.8958f,9.4238f,1.0000f,
	9.3750f,1.0000f,11.7188f,-4.0000f,11.6699f,-4.0000f,11.6699f,-3.8958f,11.6211f,-3.8958f,11.6211f,-3.7917f,11.5723f,-3.7917f,11.5723f,-3.6875f,11.5235f,-3.6875f,11.5235f,-3.5833f,11.4746f,-3.5833f,11.4746f,-3.4792f,11.4258f,-3.4792f,11.4258f,-3.3750f,11.3770f,-3.3750f,
	11.3770f,-3.2708f,11.3281f,-3.2708f,11.3281f,-3.1667f,11.2793f,-3.1667f,11.2793f,-3.0625f,11.2305f,-3.0625f,11.2305f,-2.9583f,11.1817f,-2.9583f,11.1817f,-2.8542f,11.1328f,-2.8542f,11.1328f,-2.7500f,11.0840f,-2.7500f,11.0840f,-2.6458f,11.0352f,-2.6458f,11.0352f,-2.5417f,
	10.9863f,-2.5417f,10.9863f,-2.4375f,10.9375f,-2.4375f,10.9375f,-2.3333f,10.8887f,-2.3333f,10.8887f,-2.2292f,10.8399f,-2.2292f,10.8399f,-2.1250f,10.7910f,-2.1250f,10.7910f,-2.0208f,10.7422f,-2.0208f,10.7422f,-1.9167f,10.6934f,-1.9167f,10.6934f,-1.8125f,10.6445f,-1.8125f,
	10.6445f,-1.7083f,10.5957f,-1.7083f,10.5957f,-1.6042f,10.5469f,-1.6042f,10.5469f,-1.5000f,10.4981f,-1.5000f,10.4981f,-1.3958f,10.4492f,-1.3958f,10.4492f,-1.2917f,10.4004f,-1.2917f,10.4004f,-1.1875f,10.3516f,-1.1875f,10.3516f,-1.0833f,10.3027f,-1.0833f,10.3027f,-0.9792f,
	10.2539f,-0.9792f,10.2539f,-0.8750f,10.2051f,-0.8750f,10.2051f,-0.7708f,10.1563f,-0.7708f,10.1563f,-0.6667f,10.1074f,-0.6667f,10.1074f,-0.5625f,10.0586f,-0.5625f,10.0586f,-0.4583f,10.0098f,-0.4583f,10.0098f,-0.3542f,9.9609f,-0.3542f,9.9609f,-0.2500f,9.9121f,-0.2500f,
	9.9121f,-0.1458f,9.8633f,-0.1458f,9.8633f,-0.0417f,9.8145f,-0.0417f,9.8145f,0.0625f,9.7656f,0.0625f,9.7656f,0.1667f,9.7168f,0.1667f,9.7168f,0.2708f,9.6680f,0.2708f,9.6680f,0.3750f,9.6192f,0.3750f,9.6192f,0.4792f,9.5703f,0.4792f,9.5703f,0.5833f,
	9.5215f,0.5833f,9.5215f,0.6875f,9.4727f,0.6875f,9.4727f,0.7917f,9.4238f,0.7917f,9.4238f,0.8958f,9.3750f,0.8958f,9.3750f,1.0000f,9.3262f,1.0000f,11.6699f,-4.0000f,11.6211f,-4.0000f,11.6211f,-3.8958f,11.5723f,-3.8958f,11.5723f,-3.7917f,11.5235f,-3.7917f,
	11.5235f,-3.6875f,11.4746f,-3.6875f,11.4746f,-3.5833f,11.4258f,-3.5833f,11.4258f,-3.4792f,11.3770f,-3.4792f,11.3770f,-3.3750f,11.3281f,-3.3750f,11.3281f,-3.2708f,11.2793f,-3.2708f,11.2793f,-3.1667f,11.2305f,-3.1667f,11.2305f,-3.0625f,11.1817f,-3.0625f,11.1817f,-2.9583f,
	11.1328f,-2.9583f,11.1328f,-2.8542f,11.0840f,-2.8542f,11.0840f,-2.7500f,11.0352f,-2.7500f,11.0352f,-2.6458f,10.9863f,-2.6458f,10.9863f,-2.5417f,10.9375f,-2.5417f,10.9375f,-2.4375f,10.8887f,-2.4375f,10.8887f,-2.3333f,10.8399f,-2.3333f,10.8399f,-2.2292f,10.7910f,-2.2292f,
	10.7910f,-2.1250f,10.7422f,-2.1250f,10.7422f,-2.0208f,10.6934f,-2.0208f,10.6934f,-1.9167f,10.6445f,-1.9167f,10.6445f,-1.8125f,10.5957f,-1.8125f,10.5957f,-1.7083f,10.5469f,-1.7083f,10.5469f,-1.6042f,10.4981f,-1.6042f,10.4981f,-1.5000f,10.4492f,-1.5000f,10.4492f,-1.3958f,
	10.4004f,-1.3958f,10.4004f,-1.2917f,10.3516f,-1.2917f,10.3516f,-1.1875f,10.3027f,-1.1875f,10.3027f,-1.0833f,10.2539f,-1.0833f,10.2539f,-0.9792f,10.2051f,-0.9792f,10.2051f,-0.8750f,10.1563f,-0.8750f,10.1563f,-0.7708f,10.1074f,-0.7708f,10.1074f,-0.6667f,10.0586f,-0.6667f,
	10.0586f,-0.5625f,10.0098f,-0.5625f,10.0098f,-0.4583f,9.9609f,-0.4583f,9.9609f,-0.3542f,9.9121f,-0.3542f,9.9121f,-0.2500f,9.8633f,-0.2500f,9.8633f,-0.1458f,9.8145f,-0.1458f,9.8145f,-0.0417f,9.7656f,-0.0417f,9.7656f,0.0625f,9.7168f,0.0625f,9.7168f,0.1667f,
	9.6680f,0.1667f,9.6680f,0.2708f,9.6192f,0.2708f,9.6192f,0.3750f,9.5703f,0.3750f,9.5703f,0.4792f,9.5215f,0.4792f,9.5215f,0.5833f,9.4727f,0.5833f,9.4727f,0.6875f,9.4238f,0.6875f,9.4238f,0.7917f,9.3750f,0.7917f,9.3750f,0.8958f,9.3262f,0.8958f,
	9.3262f,1.0000f,9.2774f,1.0000f,11.6211f,-4.0000f,11.5723f,-4.0000f,11.5723f,-3.8958f,11.5235f,-3.8958f,11.5235f,-3.7917f,11.4746f,-3.7917f,11.4746f,-3.6875f,11.4258f,-3.6875f,11.4258f,-3.5833f,11.3770f,-3.5833f,11.3770f,-3.4792f,11.3281f,-3.4792f,11.3281f,-3.3750f,
	11.2793f,-3.3750f,11.2793f,-3.2708f,11.2305f,-3.2708f,11.2305f,-3.1667f,11.1817f,-3.1667f,11.1817f,-3.0625f,11.1328f,-3.0625f,11.1328f,-2.9583f,11.0840f,-2.9583f,11.0840f,-2.8542f,11.0352f,-2.8542f,11.0352f,-2.7500f,10.9863f,-2.7500f,10.9863f,-2.6458f,10.9375f,-2.6458f,
	10.9375f,-2.5417f,10.8887f,-2.5417f,10.8887f,-2.4375f,10.8399f,-2.4375f,10.8399f,-2.3333f,10.7910f,-2.3333f,10.7910f,-2.2292f,10.7422f,-2.2292f,10.7422f,-2.1250f,10.6934f,-2.1250f,10.6934f,-2.0208f,10.6445f,-2.0208f,10.6445f,-1.9167f,10.5957f,-1.9167f,10.5957f,-1.8125f,
	10.5469f,-1.8125f,10.5469f,-1.7083f,10.4981f,-1.7083f,10.4981f,-1.6042f,10.4492f,-1.6042f,10.4492f,-1.5000f,10.4004f,-1.5000f,10.4004f,-1.3958f,10.3516f,-1.3958f,10.3516f,-1.2917f,10.3027f,-1.2917f,10.3027f,-1.1875f,10.2539f,-1.1875f,10.2539f,-1.0833f,10.2051f,-1.0833f,
	10.2051f,-0.9792f,10.1563f,-0.9792f,10.1563f,-0.8750f,10.1074f,-0.8750f,10.1074f,-0.7708f,10.0586f,-0.7708f,10.0586f,-0.6667f,10.0098f,-0.6667f,10.0098f,-0.5625f,9.9609f,-0.5625f,9.9609f,-0.4583f,9.9121f,-0.4583f,9.9121f,-0.3542f,9.8633f,-0.3542f,9.8633f,-0.2500f,
	9.8145f,-0.2500f,9.8145f,-0.1458f,9.7656f,-0.1458f,9.7656f,-0.0417f,9.7168f,-0.0417f,9.7168f,0.0625f,9.6680f,0.0625f,9.6680f,0.1667f,9.6192f,0.1667f,9.6192f,0.2708f,9.5703f,0.2708f,9.5703f,0.3750f,9.5215f,0.3750f,9.5215f,0.4792f,9.4727f,0.4792f,
	9.4727f,0.5833f,9.4238f,0.5833f,9.4238f,0.6875f,9.3750f,0.6875f,9.3750f,0.7917f,9.3262f,0.7917f,9.3262f,0.8958f,9.2774f,0.8958f,9.2774f,1.0000f,9.2285f,1.0000f,11.5723f,-4.0000f,11.5235f,-4.0000f,11.5235f,-3.8958f,11.4746f,-3.8958f,11.4746f,-3.7917f,
	11.4258f,-3.7917f,11.4258f,-3.6875f,11.3770f,-3.6875f,11.3770f,-3.5833f,11.3281f,-3.5833f,11.3281f,-3.4792f,11.2793f,-3.4792f,11.2793f,-3.3750f,11.2305f,-3.3750f,11.2305f,-3.2708f,11.1817f,-3.2708f,11.1817f,-3.1667f,11.1328f,-3.1667f,11.1328f,-3.0625f,11.0840f,-3.0625f,
	11.0840f,-2.9583f,11.0352f,-2.9583f,11.0352f,-2.8542f,10.9863f,-2.8542f,10.9863f,-2.7500f,10.9375f,-2.7500f,10.9375f,-2.6458f,10.8887f,-2.6458f,10.8887f,-2.5417f,10.8399f,-2.5417f,10.8399f,-2.4375f,10.7910f,-2.4375f,10.7910f,-2.3333f,10.7422f,-2.3333f,10.7422f,-2.2292f,
	10.6934f,-2.2292f,10.6934f,-2.1250f,10.6445f,-2.1250f,10.6445f,-2.0208f,10.5957f,-2.0208f,10.5957f,-1.9167f,10.5469f,-1.9167f,10.5469f,-1.8125f,10.4981f,-1.8125f,10.4981f,-1.7083f,10.4492f,-1.7083f,10.4492f,-1.6042f,10.4004f,-1.6042f,10.4004f,-1.5000f,10.3516f,-1.5000f,
	10.3516f,-1.3958f,10.3027f,-1.3958f,10.3027f,-1.2917f,10.2539f,-1.2917f,10.2539f,-1.1875f,10.2051f,-1.1875f,10.2051f,-1.0833f,10.1563f,-1.0833f,10.1563f,-0.9792f,10.1074f,-0.9792f,10.1074f,-0.8750f,10.0586f,-0.8750f,10.0586f,-0.7708f,10.0098f,-0.7708f,10.0098f,-0.6667f,
	9.9609f,-0.6667f,9.9609f,-0.5625f,9.9121f,-0.5625f,9.9121f,-0.4583f,9.8633f,-0.4583f,9.8633f,-0.3542f,9.8145f,-0.3542f,9.8145f,-0.2500f,9.7656f,-0.2500f,9.7656f,-0.1458f,9.7168f,-0.1458f,9.7168f,-0.0417f,9.6680f,-0.0417f,9.6680f,0.0625f,9.6192f,0.0625f,
	9.6192f,0.1667f,9.5703f,0.1667f,9.5703f,0.2708f,9.5215f,0.2708f,9.5215f,0.3750f,9.4727f,0.3750f,9.4727f,0.4792f,9.4238f,0.4792f,9.4238f,0.5833f,9.3750f,0.5833f,9.3750f,0.6875f,9.3262f,0.6875f,9.3262f,0.7917f,9.2774f,0.7917f,9.2774f,0.8958f,
	9.2285f,0.8958f,9.2285f,1.0000f,9.1797f,1.0000f,11.5235f,-4.0000f,11.4746f,-4.0000f,11.4746f,-3.8958f,11.4258f,-3.8958f,11.4258f,-3.7917f,11.3770f,-3.7917f,11.3770f,-3.6875f,11.3281f,-3.6875f,11.3281f,-3.5833f,11.2793f,-3.5833f,11.2793f,-3.4792f,11.2305f,-3.4792f,
	11.2305f,-3.3750f,11.1817f,-3.3750f,11.1817f,-3.2708f,11.1328f,-3.2708f,11.1328f,-3.1667f,11.0840f,-3.1667f,11.0840f,-3.0625f,11.0352f,-3.0625f,11.0352f,-2.9583f,10.9863f,-2.9583f,10.9863f,-2.8542f,10.9375f,-2.8542f,10.9375f,-2.7500f,10.8887f,-2.7500f,10.8887f,-2.6458f,
	10.8399f,-2.6458f,10.8399f,-2.5417f,10.7910f,-2.5417f,10.7910f,-2.4375f,10.7422f,-2.4375f,10.7422f,-2.3333f,10.6934f,-2.3333f,10.6934f,-2.2292f,10.6445f,-2.2292f,10.6445f,-2.1250f,10.5957f,-2.1250f,10.5957f,-2.0208f,10.5469f,-2.0208f,10.5469f,-1.9167f,10.4981f,-1.9167f,
	10.4981f,-1.8125f,10.4492f,-1.8125f,10.4492f,-1.7083f,10.4004f,-1.7083f,10.4004f,-1.6042f,10.3516f,-1.6042f,10.3516f,-1.5000f,10.3027f,-1.5000f,10.3027f,-1.3958f,10.2539f,-1.3958f,10.2539f,-1.2917f,10.2051f,-1.2917f,10.2051f,-1.1875f,10.1563f,-1.1875f,10.1563f,-1.0833f,
	10.1074f,-1.0833f,10.1074f,-0.9792f,10.0586f,-0.9792f,10.0586f,-0.8750f,10.0098f,-0.8750f,10.0098f,-0.7708f,9.9609f,-0.7708f,9.9609f,-0.6667f,9.9121f,-0.6667f,9.9121f,-0.5625f,9.8633f,-0.5625f,9.8633f,-0.4583f,9.8145f,-0.4583f,9.8145f,-0.3542f,9.7656f,-0.3542f,
	9.7656f,-0.2500f,9.7168f,-0.2500f,9.7168f,-0.1458f,9.6680f,-0.1458f,9.6680f,-0.0417f,9.6192f,-0.0417f,9.6192f,0.0625f,9.5703f,0.0625f,9.5703f,0.1667f,9.5215f,0.1667f,9.5215f,0.2708f,9.4727f,0.2708f,9.4727f,0.3750f,9.4238f,0.3750f,9.4238f,0.4792f,
	9.3750f,0.4792f,9.3750f,0.5833f,9.3262f,0.5833f,9.3262f,0.6875f,9.2774f,0.6875f,9.2774f,0.7917f,9.2285f,0.7917f,9.2285f,0.8958f,9.1797f,0.8958f,9.1797f,1.0000f,9.1309f,1.0000f,11.4746f,-4.0000f,11.4258f,-4.0000f,11.4258f,-3.8958f,11.3770f,-3.8958f,
	11.3770f,-3.7917f,11.3281f,-3.7917f,11.3281f,-3.6875f,11.2793f,-3.6875f,11.2793f,-3.5833f,11.2305f,-3.5833f,11.2305f,-3.4792f,11.1817f,-3.4792f,11.1817f,-3.3750f,11.1328f,-3.3750f,11.1328f,-3.2708f,11.0840f,-3.2708f,11.0840f,-3.1667f,11.0352f,-3.1667f,11.0352f,-3.0625f,
	10.9863f,-3.0625f,10.9863f,-2.9583f,10.9375f,-2.9583f,10.9375f,-2.8542f,10.8887f,-2.8542f,10.8887f,-2.7500f,10.8399f,-2.7500f,10.8399f,-2.6458f,10.7910f,-2.6458f,10.7910f,-2.5417f,10.7422f,-2.5417f,10.7422f,-2.4375f,10.6934f,-2.4375f,10.6934f,-2.3333f,10.6445f,-2.3333f,
	10.6445f,-2.2292f,10.5957f,-2.2292f,10.5957f,-2.1250f,10.5469f,-2.1250f,10.5469f,-2.0208f,10.4981f,-2.0208f,10.4981f,-1.9167f,10.4492f,-1.9167f,10.4492f,-1.8125f,10.4004f,-1.8125f,10.4004f,-1.7083f,10.3516f,-1.7083f,10.3516f,-1.6042f,10.3027f,-1.6042f,10.3027f,-1.5000f,
	10.2539f,-1.5000f,10.2539f,-1.3958f,10.2051f,-1.3958f,10.2051f,-1.2917f,10.1563f,-1.2917f,10.1563f,-1.1875f,10.1074f,-1.1875f,10.1074f,-1.0833f,10.0586f,-1.0833f,10.0586f,-0.9792f,10.0098f,-0.9792f,10.0098f,-0.8750f,9.9609f,-0.8750f,9.9609f,-0.7708f,9.9121f,-0.7708f,
	9.9121f,-0.6667f,9.8633f,-0.6667f,9.8633f,-0.5625f,9.8145f,-0.5625f,9.8145f,-0.4583f,9.7656f,-0.4583f,9.7656f,-0.3542f,9.7168f,-0.3542f,9.7168f,-0.2500f,9.6680f,-0.2500f,9.6680f,-0.1458f,9.6192f,-0.1458f,9.6192f,-0.0417f,9.5703f,-0.0417f,9.5703f,0.0625f,
	9.5215f,0.0625f,9.5215f,0.1667f,9.4727f,0.1667f,9.4727f,0.2708f,9.4238f,0.2708f,9.4238f,0.3750f,9.3750f,0.3750f,9.3750f,0.4792f,9.3262f,0.4792f,9.3262f,0.5833f,9.2774f,0.5833f,9.2774f,0.6875f,9.2285f,0.6875f,9.2285f,0.7917f,9.1797f,0.7917f,
	9.1797f,0.8958f,9.1309f,0.8958f,9.1309f,1.0000f,9.0820f,1.0000f,11.4258f,-4.0000f,11.3770f,-4.0000f,11.3770f,-3.8958f,11.3281f,-3.8958f,11.3281f,-3.7917f,11.2793f,-3.7917f,11.2793f,-3.6875f,11.2305f,-3.6875f,11.2305f,-3.5833f,11.1817f,-3.5833f,11.1817f,-3.4792f,
	11.1328f,-3.4792f,11.1328f,-3.3750f,11.0840f,-3.3750f,11.0840f,-3.2708f,11.0352f,-3.2708f,11.0352f,-3.1667f,10.9863f,-3.1667f,10.9863f,-3.0625f,10.9375f,-3.0625f,10.9375f,-2.9583f,10.8887f,-2.9583f,10.8887f,-2.8542f,10.8399f,-2.8542f,10.8399f,-2.7500f,10.7910f,-2.7500f,
	10.7910f,-2.6458f,10.7422f,-2.6458f,10.7422f,-2.5417f,10.6934f,-2.5417f,10.6934f,-2.4375f,10.6445f,-2.4375f,10.6445f,-2.3333f,10.5957f,-2.3333f,10.5957f,-2.2292f,10.5469f,-2.2292f,10.5469f,-2.1250f,10.4981f,-2.1250f,10.4981f,-2.0208f,10.4492f,-2.0208f,10.4492f,-1.9167f,
	10.4004f,-1.9167f,10.4004f,-1.8125f,10.3516f,-1.8125f,10.3516f,-1.7083f,10.3027f,-1.7083f,10.3027f,-1.6042f,10.2539f,-1.6042f,10.2539f,-1.5000f,10.2051f,-1.5000f,10.2051f,-1.3958f,10.1563f,-1.3958f,10.1563f,-1.2917f,10.1074f,-1.2917f,10.1074f,-1.1875f,10.0586f,-1.1875f,
	10.0586f,-1.0833f,10.0098f,-1.0833f,10.0098f,-0.9792f,9.9609f,-0.9792f,9.9609f,-0.8750f,9.9121f,-0.8750f,9.9121f,-0.7708f,9.8633f,-0.7708f,9.8633f,-0.6667f,9.8145f,-0.6667f,9.8145f,-0.5625f,9.7656f,-0.5625f,9.7656f,-0.4583f,9.7168f,-0.4583f,9.7168f,-0.3542f,
	9.6680f,-0.3542f,9.6680f,-0.2500f,9.6192f,-0.2500f,9.6192f,-0.1458f,9.5703f,-0.1458f,9.5703f,-0.0417f,9.5215f,-0.0417f,9.5215f,0.0625f,9.4727f,0.0625f,9.4727f,0.1667f,9.4238f,0.1667f,9.4238f,0.2708f,9.3750f,0.2708f,9.3750f,0.3750f,9.3262f,0.3750f,
	9.3262f,0.4792f,9.2774f,0.4792f,9.2774f,0.5833f,9.2285f,0.5833f,9.2285f,0.6875f,9.1797f,0.6875f,9.1797f,0.7917f,9.1309f,0.7917f,9.1309f,0.8958f,9.0820f,0.8958f,9.0820f,1.0000f,9.0332f,1.0000f,11.3770f,-4.0000f,11.3281f,-4.0000f,11.3281f,-3.8958f,
	11.2793f,-3.8958f,11.2793f,-3.7917f,11.2305f,-3.7917f,11.2305f,-3.6875f,11.1817f,-3.6875f,11.1817f,-3.5833f,11.1328f,-3.5833f,11.1328f,-3.4792f,11.0840f,-3.4792f,11.0840f,-3.3750f,11.0352f,-3.3750f,11.0352f,-3.2708f,10.9863f,-3.2708f,10.9863f,-3.1667f,10.9375f,-3.1667f,
	10.9375f,-3.0625f,10.8887f,-3.0625f,10.8887f,-2.9583f,10.8399f,-2.9583f,10.8399f,-2.8542f,10.7910f,-2.8542f,10.7910f,-2.7500f,10.7422f,-2.7500f,10.7422f,-2.6458f,10.6934f,-2.6458f,10.6934f,-2.5417f,10.6445f,-2.5417f,10.6445f,-2.4375f,10.5957f,-2.4375f,10.5957f,-2.3333f,
	10.5469f,-2.3333f,10.5469f,-2.2292f,10.4981f,-2.2292f,10.4981f,-2.1250f,10.4492f,-2.1250f,10.4492f,-2.0208f,10.4004f,-2.0208f,10.4004f,-1.9167f,10.3516f,-1.9167f,10.3516f,-1.8125f,10.3027f,-1.8125f,10.3027f,-1.7083f,10.2539f,-1.7083f,10.2539f,-1.6042f,10.2051f,-1.6042f,
	10.2051f,-1.5000f,10.1563f,-1.5000f,10.1563f,-1.3958f,10.1074f,-1.3958f,10.1074f,-1.2917f,10.0586f,-1.2917f,10.0586f,-1.1875f,10.0098f,-1.1875f,10.0098f,-1.0833f,9.9609f,-1.0833f,9.9609f,-0.9792f,9.9121f,-0.9792f,9.9121f,-0.8750f,9.8633f,-0.8750f,9.8633f,-0.7708f,
	9.8145f,-0.7708f,9.8145f,-0.6667f,9.7656f,-0.6667f,9.7656f,-0.5625f,9.7168f,-0.5625f,9.7168f,-0.4583f,9.6680f,-0.4583f,9.6680f,-0.3542f,9.6192f,-0.3542f,9.6192f,-0.2500f,9.5703f,-0.2500f,9.5703f,-0.1458f,9.5215f,-0.1458f,9.5215f,-0.0417f,9.4727f,-0.0417f,
	9.4727f,0.0625f,9.4238f,0.0625f,9.4238f,0.1667f,9.3750f,0.1667f,9.3750f,0.2708f,9.3262f,0.2708f,9.3262f,0.3750f,9.2774f,0.3750f,9.2774f,0.4792f,9.2285f,0.4792f,9.2285f,0.5833f,9.1797f,0.5833f,9.1797f,0.6875f,9.1309f,0.6875f,9.1309f,0.7917f,
	9.0820f,0.7917f,9.0820f,0.8958f,9.0332f,0.8958f,9.0332f,1.0000f,8.9844f,1.0000f,11.3281f,-4.0000f,11.2793f,-4.0000f,11.2793f,-3.8958f,11.2305f,-3.8958f,11.2305f,-3.7917f,11.1817f,-3.7917f,11.1817f,-3.6875f,11.1328f,-3.6875f,11.1328f,-3.5833f,11.0840f,-3.5833f,
	11.0840f,-3.4792f,11.0352f,-3.4792f,11.0352f,-3.3750f,10.9863f,-3.3750f,10.9863f,-3.2708f,10.9375f,-3.2708f,10.9375f,-3.1667f,10.8887f,-3.1667f,10.8887f,-3.0625f,10.8399f,-3.0625f,10.8399f,-2.9583f,10.7910f,-2.9583f,10.7910f,-2.8542f,10.7422f,-2.8542f,10.7422f,-2.7500f,
	10.6934f,-2.7500f,10.6934f,-2.6458f,10.6445f,-2.6458f,10.6445f,-2.5417f,10.5957f,-2.5417f,10.5957f,-2.4375f,10.5469f,-2.4375f,10.5469f,-2.3333f,10.4981f,-2.3333f,10.4981f,-2.2292f,10.4492f,-2.2292f,10.4492f,-2.1250f,10.4004f,-2.1250f,10.4004f,-2.0208f,10.3516f,-2.0208f,
	10.3516f,-1.9167f,10.3027f,-1.9167f,10.3027f,-1.8125f,10.2539f,-1.8125f,10.2539f,-1.7083f,10.2051f,-1.7083f,10.2051f,-1.6042f,10.1563f,-1.6042f,10.1563f,-1.5000f,10.1074f,-1.5000f,10.1074f,-1.3958f,10.0586f,-1.3958f,10.0586f,-1.2917f,10.0098f,-1.2917f,10.0098f,-1.1875f,
	9.9609f,-1.1875f,9.9609f,-1.0833f,9.9121f,-1.0833f,9.9121f,-0.9792f,9.8633f,-0.9792f,9.8633f,-0.8750f,9.8145f,-0.8750f,9.8145f,-0.7708f,9.7656f,-0.7708f,9.7656f,-0.6667f,9.7168f,-0.6667f,9.7168f,-0.5625f,9.6680f,-0.5625f,9.6680f,-0.4583f,9.6192f,-0.4583f,
	9.6192f,-0.3542f,9.5703f,-0.3542f,9.5703f,-0.2500f,9.5215f,-0.2500f,9.5215f,-0.1458f,9.4727f,-0.1458f,9.4727f,-0.0417f,9.4238f,-0.0417f,9.4238f,0.0625f,9.3750f,0.0625f,9.3750f,0.1667f,9.3262f,0.1667f,9.3262f,0.2708f,9.2774f,0.2708f,9.2774f,0.3750f,
	9.2285f,0.3750f,9.2285f,0.4792f,9.1797f,0.4792f,9.1797f,0.5833f,9.1309f,0.5833f,9.1309f,0.6875f,9.0820f,0.6875f,9.0820f,0.7917f,9.0332f,0.7917f,9.0332f,0.8958f,8.9844f,0.8958f,8.9844f,1.0000f,8.9356f,1.0000f,11.2793f,-4.0000f,11.2305f,-4.0000f,
	11.2305f,-3.8958f,11.1817f,-3.8958f,11.1817f,-3.7917f,11.1328f,-3.7917f,11.1328f,-3.6875f,11.0840f,-3.6875f,11.0840f,-3.5833f,11.0352f,-3.5833f,11.0352f,-3.4792f,10.9863f,-3.4792f,10.9863f,-3.3750f,10.9375f,-3.3750f,10.9375f,-3.2708f,10.8887f,-3.2708f,10.8887f,-3.1667f,
	10.8399f,-3.1667f,10.8399f,-3.0625f,10.7910f,-3.0625f,10.7910f,-2.9583f,10.7422f,-2.9583f,10.7422f,-2.8542f,10.6934f,-2.8542f,10.6934f,-2.7500f,10.6445f,-2.7500f,10.6445f,-2.6458f,10.5957f,-2.6458f,10.5957f,-2.5417f,10.5469f,-2.5417f,10.5469f,-2.4375f,10.4981f,-2.4375f,
	10.4981f,-2.3333f,10.4492f,-2.3333f,10.4492f,-2.2292f,10.4004f,-2.2292f,10.4004f,-2.1250f,10.3516f,-2.1250f,10.3516f,-2.0208f,10.3027f,-2.0208f,10.3027f,-1.9167f,10.2539f,-1.9167f,10.2539f,-1.8125f,10.2051f,-1.8125f,10.2051f,-1.7083f,10.1563f,-1.7083f,10.1563f,-1.6042f,
	10.1074f,-1.6042f,10.1074f,-1.5000f,10.0586f,-1.5000f,10.0586f,-1.3958f,10.0098f,-1.3958f,10.0098f,-1.2917f,9.9609f,-1.2917f,9.9609f,-1.1875f,9.9121f,-1.1875f,9.9121f,-1.0833f,9.8633f,-1.0833f,9.8633f,-0.9792f,9.8145f,-0.9792f,9.8145f,-0.8750f,9.7656f,-0.8750f,
	9.7656f,-0.7708f,9.7168f,-0.7708f,9.7168f,-0.6667f,9.6680f,-0.6667f,9.6680f,-0.5625f,9.6192f,-0.5625f,9.6192f,-0.4583f,9.5703f,-0.4583f,9.5703f,-0.3542f,9.5215f,-0.3542f,9.5215f,-0.2500f,9.4727f,-0.2500f,9.4727f,-0.1458f,9.4238f,-0.1458f,9.4238f,-0.0417f,
	9.3750f,-0.0417f,9.3750f,0.0625f,9.3262f,0.0625f,9.3262f,0.1667f,9.2774f,0.1667f,9.2774f,0.2708f,9.2285f,0.2708f,9.2285f,0.3750f,9.1797f,0.3750f,9.1797f,0.4792f,9.1309f,0.4792f,9.1309f,0.5833f,9.0820f,0.5833f,9.0820f,0.6875f,9.0332f,0.6875f,
	9.0332f,0.7917f,8.9844f,0.7917f,8.9844f,0.8958f,8.9356f,0.8958f,8.9356f,1.0000f,8.8867f,1.0000f,11.2305f,-4.0000f,11.1817f,-4.0000f,11.1817f,-3.8958f,11.1328f,-3.8958f,11.1328f,-3.7917f,11.0840f,-3.7917f,11.0840f,-3.6875f,11.0352f,-3.6875f,11.0352f,-3.5833f,
	10.9863f,-3.5833f,10.9863f,-3.4792f,10.9375f,-3.4792f,10.9375f,-3.3750f,10.8887f,-3.3750f,10.8887f,-3.2708f,10.8399f,-3.2708f,10.8399f,-3.1667f,10.7910f,-3.1667f,10.7910f,-3.0625f,10.7422f,-3.0625f,10.7422f,-2.9583f,10.6934f,-2.9583f,10.6934f,-2.8542f,10.6445f,-2.8542f,
	10.6445f,-2.7500f,10.5957f,-2.7500f,10.5957f,-2.6458f,10.5469f,-2.6458f,10.5469f,-2.5417f,10.4981f,-2.5417f,10.4981f,-2.4375f,10.4492f,-2.4375f,10.4492f,-2.3333f,10.4004f,-2.3333f,10.4004f,-2.2292f,10.3516f,-2.2292f,10.3516f,-2.1250f,10.3027f,-2.1250f,10.3027f,-2.0208f,
	10.2539f,-2.0208f,10.2539f,-1.9167f,10.2051f,-1.9167f,10.2051f,-1.8125f,10.1563f,-1.8125f,10.1563f,-1.7083f,10.1074f,-1.7083f,10.1074f,-1.6042f,10.0586f,-1.6042f,10.0586f,-1.5000f,10.0098f,-1.5000f,10.0098f,-1.3958f,9.9609f,-1.3958f,9.9609f,-1.2917f,9.9121f,-1.2917f,
	9.9121f,-1.1875f,9.8633f,-1.1875f,9.8633f,-1.0833f,9.8145f,-1.0833f,9.8145f,-0.9792f,9.7656f,-0.9792f,9.7656f,-0.8750f,9.7168f,-0.8750f,9.7168f,-0.7708f,9.6680f,-0.7708f,9.6680f,-0.6667f,9.6192f,-0.6667f,9.6192f,-0.5625f,9.5703f,-0.5625f,9.5703f,-0.4583f,
	9.5215f,-0.4583f,9.5215f,-0.3542f,9.4727f,-0.3542f,9.4727f,-0.2500f,9.4238f,-0.2500f,9.4238f,-0.1458f,9.3750f,-0.1458f,9.3750f,-0.0417f,9.3262f,-0.0417f,9.3262f,0.0625f,9.2774f,0.0625f,9.2774f,0.1667f,9.2285f,0.1667f,9.2285f,0.2708f,9.1797f,0.2708f,
	9.1797f,0.3750f,9.1309f,0.3750f,9.1309f,0.4792f,9.0820f,0.4792f,9.0820f,0.5833f,9.0332f,0.5833f,9.0332f,0.6875f,8.9844f,0.6875f,8.9844f,0.7917f,8.9356f,0.7917f,8.9356f,0.8958f,8.8867f,0.8958f,8.8867f,1.0000f,8.8379f,1.0000f,11.1817f,-4.0000f,
	11.1328f,-4.0000f,11.1328f,-3.8958f,11.0840f,-3.8958f,11.0840f,-3.7917f,11.0352f,-3.7917f,11.0352f,-3.6875f,10.9863f,-3.6875f,10.9863f,-3.5833f,10.9375f,-3.5833f,10.9375f,-3.4792f,10.8887f,-3.4792f,10.8887f,-3.3750f,10.8399f,-3.3750f,10.8399f,-3.2708f,10.7910f,-3.2708f,
	10.7910f,-3.1667f,10.7422f,-3.1667f,10.7422f,-3.0625f,10.6934f,-3.0625f,10.6934f,-2.9583f,10.6445f,-2.9583f,10.6445f,-2.8542f,10.5957f,-2.8542f,10.5957f,-2.7500f,10.5469f,-2.7500f,10.5469f,-2.6458f,10.4981f,-2.6458f,10.4981f,-2.5417f,10.4492f,-2.5417f,10.4492f,-2.4375f,
	10.4004f,-2.4375f,10.4004f,-2.3333f,10.3516f,-2.3333f,10.3516f,-2.2292f,10.3027f,-2.2292f,10.3027f,-2.1250f,10.2539f,-2.1250f,10.2539f,-2.0208f,10.2051f,-2.0208f,10.2051f,-1.9167f,10.1563f,-1.9167f,10.1563f,-1.8125f,10.1074f,-1.8125f,10.1074f,-1.7083f,10.0586f,-1.7083f,
	10.0586f,-1.6042f,10.0098f,-1.6042f,10.0098f,-1.5000f,9.9609f,-1.5000f,9.9609f,-1.3958f,9.9121f,-1.3958f,9.9121f,-1.2917f,9.8633f,-1.2917f,9.8633f,-1.1875f,9.8145f,-1.1875f,9.8145f,-1.0833f,9.7656f,-1.0833f,9.7656f,-0.9792f,9.7168f,-0.9792f,9.7168f,-0.8750f,
	9.6680f,-0.8750f,9.6680f,-0.7708f,9.6192f,-0.7708f,9.6192f,-0.6667f,9.5703f,-0.6667f,9.5703f,-0.5625f,9.5215f,-0.5625f,9.5215f,-0.4583f,9.4727f,-0.4583f,9.4727f,-0.3542f,9.4238f,-0.3542f,9.4238f,-0.2500f,9.3750f,-0.2500f,9.3750f,-0.1458f,9.3262f,-0.1458f,
	9.3262f,-0.0417f,9.2774f,-0.0417f,9.2774f,0.0625f,9.2285f,0.0625f,9.2285f,0.1667f,9.1797f,0.1667f,9.1797f,0.2708f,9.1309f,0.2708f,9.1309f,0.3750f,9.0820f,0.3750f,9.0820f,0.4792f,9.0332f,0.4792f,9.0332f,0.5833f,8.9844f,0.5833f,8.9844f,0.6875f,
	8.9356f,0.6875f,8.9356f,0.7917f,8.8867f,0.7917f,8.8867f,0.8958f,8.8379f,0.8958f,8.8379f,1.0000f,8.7891f,1.0000f,11.1328f,-4.0000f,11.0840f,-4.0000f,11.0840f,-3.8958f,11.0352f,-3.8958f,11.0352f,-3.7917f,10.9863f,-3.7917f,10.9863f,-3.6875f,10.9375f,-3.6875f,
	10.9375f,-3.5833f,10.8887f,-3.5833f,10.8887f,-3.4792f,10.8399f,-3.4792f,10.8399f,-3.3750f,10.7910f,-3.3750f,10.7910f,-3.2708f,10.7422f,-3.2708f,10.7422f,-3.1667f,10.6934f,-3.1667f,10.6934f,-3.0625f,10.6445f,-3.0625f,10.6445f,-2.9583f,10.5957f,-2.9583f,10.5957f,-2.8542f,
	10.5469f,-2.8542f,10.5469f,-2.7500f,10.4981f,-2.7500f,10.4981f,-2.6458f,10.4492f,-2.6458f,10.4492f,-2.5417f,10.4004f,-2.5417f,10.4004f,-2.4375f,10.3516f,-2.4375f,10.3516f,-2.3333f,10.3027f,-2.3333f,10.3027f,-2.2292f,10.2539f,-2.2292f,10.2539f,-2.1250f,10.2051f,-2.1250f,
	10.2051f,-2.0208f,10.1563f,-2.0208f,10.1563f,-1.9167f,10.1074f,-1.9167f,10.1074f,-1.8125f,10.0586f,-1.8125f,10.0586f,-1.7083f,10.0098f,-1.7083f,10.0098f,-1.6042f,9.9609f,-1.6042f,9.9609f,-1.5000f,9.9121f,-1.5000f,9.9121f,-1.3958f,9.8633f,-1.3958f,9.8633f,-1.2917f,
	9.8145f,-1.2917f,9.8145f,-1.1875f,9.7656f,-1.1875f,9.7656f,-1.0833f,9.7168f,-1.0833f,9.7168f,-0.9792f,9.6680f,-0.9792f,9.6680f,-0.8750f,9.6192f,-0.8750f,9.6192f,-0.7708f,9.5703f,-0.7708f,9.5703f,-0.6667f,9.5215f,-0.6667f,9.5215f,-0.5625f,9.4727f,-0.5625f,
	9.4727f,-0.4583f,9.4238f,-0.4583f,9.4238f,-0.3542f,9.3750f,-0.3542f,9.3750f,-0.2500f,9.3262f,-0.2500f,9.3262f,-0.1458f,9.2774f,-0.1458f,9.2774f,-0.0417f,9.2285f,-0.0417f,9.2285f,0.0625f,9.1797f,0.0625f,9.1797f,0.1667f,9.1309f,0.1667f,9.1309f,0.2708f,
	9.0820f,0.2708f,9.0820f,0.3750f,9.0332f,0.3750f,9.0332f,0.4792f,8.9844f,0.4792f,8.9844f,0.5833f,8.9356f,0.5833f,8.9356f,0.6875f,8.8867f,0.6875f,8.8867f,0.7917f,8.8379f,0.7917f,8.8379f,0.8958f,8.7891f,0.8958f,8.7891f,1.0000f,8.7402f,1.0000f,
	11.0840f,-4.0000f,11.0352f,-4.0000f,11.0352f,-3.8958f,10.9863f,-3.8958f,10.9863f,-3.7917f,10.9375f,-3.7917f,10.9375f,-3.6875f,10.8887f,-3.6875f,10.8887f,-3.5833f,10.8399f,-3.5833f,10.8399f,-3.4792f,10.7910f,-3.4792f,10.7910f,-3.3750f,10.7422f,-3.3750f,10.7422f,-3.2708f,
	10.6934f,-3.2708f,10.6934f,-3.1667f,10.6445f,-3.1667f,10.6445f,-3.0625f,10.5957f,-3.0625f,10.5957f,-2.9583f,10.5469f,-2.9583f,10.5469f,-2.8542f,10.4981f,-2.8542f,10.4981f,-2.7500f,10.4492f,-2.7500f,10.4492f,-2.6458f,10.4004f,-2.6458f,10.4004f,-2.5417f,10.3516f,-2.5417f,
	10.3516f,-2.4375f,10.3027f,-2.4375f,10.3027f,-2.3333f,10.2539f,-2.3333f,10.2539f,-2.2292f,10.2051f,-2.2292f,10.2051f,-2.1250f,10.1563f,-2.1250f,10.1563f,-2.0208f,10.1074f,-2.0208f,10.1074f,-1.9167f,10.0586f,-1.9167f,10.0586f,-1.8125f,10.0098f,-1.8125f,10.0098f,-1.7083f,
	9.9609f,-1.7083f,9.9609f,-1.6042f,9.9121f,-1.6042f,9.9121f,-1.5000f,9.8633f,-1.5000f,9.8633f,-1.3958f,9.8145f,-1.3958f,9.8145f,-1.2917f,9.7656f,-1.2917f,9.7656f,-1.1875f,9.7168f,-1.1875f,9.7168f,-1.0833f,9.6680f,-1.0833f,9.6680f,-0.9792f,9.6192f,-0.9792f,
	9.6192f,-0.8750f,9.5703f,-0.8750f,9.5703f,-0.7708f,9.5215f,-0.7708f,9.5215f,-0.6667f,9.4727f,-0.6667f,9.4727f,-0.5625f,9.4238f,-0.5625f,9.4238f,-0.4583f,9.3750f,-0.4583f,9.3750f,-0.3542f,9.3262f,-0.3542f,9.3262f,-0.2500f,9.2774f,-0.2500f,9.2774f,-0.1458f,
	9.2285f,-0.1458f,9.2285f,-0.0417f,9.1797f,-0.0417f,9.1797f,0.0625f,9.1309f,0.0625f,9.1309f,0.1667f,9.0820f,0.1667f,9.0820f,0.2708f,9.0332f,0.2708f,9.0332f,0.3750f,8.9844f,0.3750f,8.9844f,0.4792f,8.9356f,0.4792f,8.9356f,0.5833f,8.8867f,0.5833f,
	8.8867f,0.6875f,8.8379f,0.6875f,8.8379f,0.7917f,8.7891f,0.7917f,8.7891f,0.8958f,8.7402f,0.8958f,8.7402f,1.0000f,8.6914f,1.0000f,11.0352f,-4.0000f,10.9863f,-4.0000f,10.9863f,-3.8958f,10.9375f,-3.8958f,10.9375f,-3.7917f,10.8887f,-3.7917f,10.8887f,-3.6875f,
	10.8399f,-3.6875f,10.8399f,-3.5833f,10.7910f,-3.5833f,10.7910f,-3.4792f,10.7422f,-3.4792f,10.7422f,-3.3750f,10.6934f,-3.3750f,10.6934f,-3.2708f,10.6445f,-3.2708f,10.6445f,-3.1667f,10.5957f,-3.1667f,10.5957f,-3.0625f,10.5469f,-3.0625f,10.5469f,-2.9583f,10.4981f,-2.9583f,
	10.4981f,-2.8542f,10.4492f,-2.8542f,10.4492f,-2.7500f,10.4004f,-2.7500f,10.4004f,-2.6458f,10.3516f,-2.6458f,10.3516f,-2.5417f,10.3027f,-2.5417f,10.3027f,-2.4375f,10.2539f,-2.4375f,10.2539f,-2.3333f,10.2051f,-2.3333f,10.2051f,-2.2292f,10.1563f,-2.2292f,10.1563f,-2.1250f,
	10.1074f,-2.1250f,10.1074f,-2.0208f,10.0586f,-2.0208f,10.0586f,-1.9167f,10.0098f,-1.9167f,10.0098f,-1.8125f,9.9609f,-1.8125f,9.9609f,-1.7083f,9.9121f,-1.7083f,9.9121f,-1.6042f,9.8633f,-1.6042f,9.8633f,-1.5000f,9.8145f,-1.5000f,9.8145f,-1.3958f,9.7656f,-1.3958f,
	9.7656f,-1.2917f,9.7168f,-1.2917f,9.7168f,-1.1875f,9.6680f,-1.1875f,9.6680f,-1.0833f,9.6192f,-1.0833f,9.6192f,-0.9792f,9.5703f,-0.9792f,9.5703f,-0.8750f,9.5215f,-0.8750f,9.5215f,-0.7708f,9.4727f,-0.7708f,9.4727f,-0.6667f,9.4238f,-0.6667f,9.4238f,-0.5625f,
	9.3750f,-0.5625f,9.3750f,-0.4583f,9.3262f,-0.4583f,9.3262f,-0.3542f,9.2774f,-0.3542f,9.2774f,-0.2500f,9.2285f,-0.2500f,9.2285f,-0.1458f,9.1797f,-0.1458f,9.1797f,-0.0417f,9.1309f,-0.0417f,9.1309f,0.0625f,9.0820f,0.0625f,9.0820f,0.1667f,9.0332f,0.1667f,
	9.0332f,0.2708f,8.9844f,0.2708f,8.9844f,0.3750f,8.9356f,0.3750f,8.9356f,0.4792f,8.8867f,0.4792f,8.8867f,0.5833f,8.8379f,0.5833f,8.8379f,0.6875f,8.7891f,0.6875f,8.7891f,0.7917f,8.7402f,0.7917f,8.7402f,0.8958f,8.6914f,0.8958f,8.6914f,1.0000f,
	8.6426f,1.0000f,10.9863f,-4.0000f,10.9375f,-4.0000f,10.9375f,-3.8958f,10.8887f,-3.8958f,10.8887f,-3.7917f,10.8399f,-3.7917f,10.8399f,-3.6875f,10.7910f,-3.6875f,10.7910f,-3.5833f,10.7422f,-3.5833f,10.7422f,-3.4792f,10.6934f,-3.4792f,10.6934f,-3.3750f,10.6445f,-3.3750f,
	10.6445f,-3.2708f,10.5957f,-3.2708f,10.5957f,-3.1667f,10.5469f,-3.1667f,10.5469f,-3.0625f,10.4981f,-3.0625f,10.4981f,-2.9583f,10.4492f,-2.9583f,10.4492f,-2.8542f,10.4004f,-2.8542f,10.4004f,-2.7500f,10.3516f,-2.7500f,10.3516f,-2.6458f,10.3027f,-2.6458f,10.3027f,-2.5417f,
	10.2539f,-2.5417f,10.2539f,-2.4375f,10.2051f,-2.4375f,10.2051f,-2.3333f,10.1563f,-2.3333f,10.1563f,-2.2292f,10.1074f,-2.2292f,10.1074f,-2.1250f,10.0586f,-2.1250f,10.0586f,-2.0208f,10.0098f,-2.0208f,10.0098f,-1.9167f,9.9609f,-1.9167f,9.9609f,-1.8125f,9.9121f,-1.8125f,
	9.9121f,-1.7083f,9.8633f,-1.7083f,9.8633f,-1.6042f,9.8145f,-1.6042f,9.8145f,-1.5000f,9.7656f,-1.5000f,9.7656f,-1.3958f,9.7168f,-1.3958f,9.7168f,-1.2917f,9.6680f,-1.2917f,9.6680f,-1.1875f,9.6192f,-1.1875f,9.6192f,-1.0833f,9.5703f,-1.0833f,9.5703f,-0.9792f,
	9.5215f,-0.9792f,9.5215f,-0.8750f,9.4727f,-0.8750f,9.4727f,-0.7708f,9.4238f,-0.7708f,9.4238f,-0.6667f,9.3750f,-0.6667f,9.3750f,-0.5625f,9.3262f,-0.5625f,9.3262f,-0.4583f,9.2774f,-0.4583f,9.2774f,-0.3542f,9.2285f,-0.3542f,9.2285f,-0.2500f,9.1797f,-0.2500f,
	9.1797f,-0.1458f,9.1309f,-0.1458f,9.1309f,-0.0417f,9.0820f,-0.0417f,9.0820f,0.0625f,9.0332f,0.0625f,9.0332f,0.1667f,8.9844f,0.1667f,8.9844f,0.2708f,8.9356f,0.2708f,8.9356f,0.3750f,8.8867f,0.3750f,8.8867f,0.4792f,8.8379f,0.4792f,8.8379f,0.5833f,
	8.7891f,0.5833f,8.7891f,0.6875f,8.7402f,0.6875f,8.7402f,0.7917f,8.6914f,0.7917f,8.6914f,0.8958f,8.6426f,0.8958f,8.6426f,1.0000f,8.5938f,1.0000f,10.9375f,-4.0000f,10.8887f,-4.0000f,10.8887f,-3.8958f,10.8399f,-3.8958f,10.8399f,-3.7917f,10.7910f,-3.7917f,
	10.7910f,-3.6875f,10.7422f,-3.6875f,10.7422f,-3.5833f,10.6934f,-3.5833f,10.6934f,-3.4792f,10.6445f,-3.4792f,10.6445f,-3.3750f,10.5957f,-3.3750f,10.5957f,-3.2708f,10.5469f,-3.2708f,10.5469f,-3.1667f,10.4981f,-3.1667f,10.4981f,-3.0625f,10.4492f,-3.0625f,10.4492f,-2.9583f,
	10.4004f,-2.9583f,10.4004f,-2.8542f,10.3516f,-2.8542f,10.3516f,-2.7500f,10.3027f,-2.7500f,10.3027f,-2.6458f,10.2539f,-2.6458f,10.2539f,-2.5417f,10.2051f,-2.5417f,10.2051f,-2.4375f,10.1563f,-2.4375f,10.1563f,-2.3333f,10.1074f,-2.3333f,10.1074f,-2.2292f,10.0586f,-2.2292f,
	10.0586f,-2.1250f,10.0098f,-2.1250f,10.0098f,-2.0208f,9.9609f,-2.0208f,9.9609f,-1.9167f,9.9121f,-1.9167f,9.9121f,-1.8125f,9.8633f,-1.8125f,9.8633f,-1.7083f,9.8145f,-1.7083f,9.8145f,-1.6042f,9.7656f,-1.6042f,9.7656f,-1.5000f,9.7168f,-1.5000f,9.7168f,-1.3958f,
	9.6680f,-1.3958f,9.6680f,-1.2917f,9.6192f,-1.2917f,9.6192f,-1.1875f,9.5703f,-1.1875f,9.5703f,-1.0833f,9.5215f,-1.0833f,9.5215f,-0.9792f,9.4727f,-0.9792f,9.4727f,-0.8750f,9.4238f,-0.8750f,9.4238f,-0.7708f,9.3750f,-0.7708f,9.3750f,-0.6667f,9.3262f,-0.6667f,
	9.3262f,-0.5625f,9.2774f,-0.5625f,9.2774f,-0.4583f,9.2285f,-0.4583f,9.2285f,-0.3542f,9.1797f,-0.3542f,9.1797f,-0.2500f,9.1309f,-0.2500f,9.1309f,-0.1458f,9.0820f,-0.1458f,9.0820f,-0.0417f,9.0332f,-0.0417f,9.0332f,0.0625f,8.9844f,0.0625f,8.9844f,0.1667f,
	8.9356f,0.1667f,8.9356f,0.2708f,8.8867f,0.2708f,8.8867f,0.3750f,8.8379f,0.3750f,8.8379f,0.4792f,8.7891f,0.4792f,8.7891f,0.5833f,8.7402f,0.5833f,8.7402f,0.6875f,8.6914f,0.6875f,8.6914f,0.7917f,8.6426f,0.7917f,8.6426f,0.8958f,8.5938f,0.8958f,
	8.5938f,1.0000f,8.5449f,1.0000f,10.8887f,-4.0000f,10.8399f,-4.0000f,10.8399f,-3.8958f,10.7910f,-3.8958f,10.7910f,-3.7917f,10.7422f,-3.7917f,10.7422f,-3.6875f,10.6934f,-3.6875f,10.6934f,-3.5833f,10.6445f,-3.5833f,10.6445f,-3.4792f,10.5957f,-3.4792f,10.5957f,-3.3750f,
	10.5469f,-3.3750f,10.5469f,-3.2708f,10.4981f,-3.2708f,10.4981f,-3.1667f,10.4492f,-3.1667f,10.4492f,-3.0625f,10.4004f,-3.0625f,10.4004f,-2.9583f,10.3516f,-2.9583f,10.3516f,-2.8542f,10.3027f,-2.8542f,10.3027f,-2.7500f,10.2539f,-2.7500f,10.2539f,-2.6458f,10.2051f,-2.6458f,
	10.2051f,-2.5417f,10.1563f,-2.5417f,10.1563f,-2.4375f,10.1074f,-2.4375f,10.1074f,-2.3333f,10.0586f,-2.3333f,10.0586f,-2.2292f,10.0098f,-2.2292f,10.0098f,-2.1250f,9.9609f,-2.1250f,9.9609f,-2.0208f,9.9121f,-2.0208f,9.9121f,-1.9167f,9.8633f,-1.9167f,9.8633f,-1.8125f,
	9.8145f,-1.8125f,9.8145f,-1.7083f,9.7656f,-1.7083f,9.7656f,-1.6042f,9.7168f,-1.6042f,9.7168f,-1.5000f,9.6680f,-1.5000f,9.6680f,-1.3958f,9.6192f,-1.3958f,9.6192f,-1.2917f,9.5703f,-1.2917f,9.5703f,-1.1875f,9.5215f,-1.1875f,9.5215f,-1.0833f,9.4727f,-1.0833f,
	9.4727f,-0.9792f,9.4238f,-0.9792f,9.4238f,-0.8750f,9.3750f,-0.8750f,9.3750f,-0.7708f,9.3262f,-0.7708f,9.3262f,-0.6667f,9.2774f,-0.6667f,9.2774f,-0.5625f,9.2285f,-0.5625f,9.2285f,-0.4583f,9.1797f,-0.4583f,9.1797f,-0.3542f,9.1309f,-0.3542f,9.1309f,-0.2500f,
	9.0820f,-0.2500f,9.0820f,-0.1458f,9.0332f,-0.1458f,9.0332f,-0.0417f,8.9844f,-0.0417f,8.9844f,0.0625f,8.9356f,0.0625f,8.9356f,0.1667f,8.8867f,0.1667f,8.8867f,0.2708f,8.8379f,0.2708f,8.8379f,0.3750f,8.7891f,0.3750f,8.7891f,0.4792f,8.7402f,0.4792f,
	8.7402f,0.5833f,8.6914f,0.5833f,8.6914f,0.6875f,8.6426f,0.6875f,8.6426f,0.7917f,8.5938f,0.7917f,8.5938f,0.8958f,8.5449f,0.8958f,8.5449f,1.0000f,8.4961f,1.0000f,10.8399f,-4.0000f,10.7910f,-4.0000f,10.7910f,-3.8958f,10.7422f,-3.8958f,10.7422f,-3.7917f,
	10.6934f,-3.7917f,10.6934f,-3.6875f,10.6445f,-3.6875f,10.6445f,-3.5833f,10.5957f,-3.5833f,10.5957f,-3.4792f,10.5469f,-3.4792f,10.5469f,-3.3750f,10.4981f,-3.3750f,10.4981f,-3.2708f,10.4492f,-3.2708f,10.4492f,-3.1667f,10.4004f,-3.1667f,10.4004f,-3.0625f,10.3516f,-3.0625f,
	10.3516f,-2.9583f,10.3027f,-2.9583f,10.3027f,-2.8542f,10.2539f,-2.8542f,10.2539f,-2.7500f,10.2051f,-2.7500f,10.2051f,-2.6458f,10.1563f,-2.6458f,10.1563f,-2.5417f,10.1074f,-2.5417f,10.1074f,-2.4375f,10.0586f,-2.4375f,10.0586f,-2.3333f,10.0098f,-2.3333f,10.0098f,-2.2292f,
	9.9609f,-2.2292f,9.9609f,-2.1250f,9.9121f,-2.1250f,9.9121f,-2.0208f,9.8633f,-2.0208f,9.8633f,-1.9167f,9.8145f,-1.9167f,9.8145f,-1.8125f,9.7656f,-1.8125f,9.7656f,-1.7083f,9.7168f,-1.7083f,9.7168f,-1.6042f,9.6680f,-1.6042f,9.6680f,-1.5000f,9.6192f,-1.5000f,
	9.6192f,-1.3958f,9.5703f,-1.3958f,9.5703f,-1.2917f,9.5215f,-1.2917f,9.5215f,-1.1875f,9.4727f,-1.1875f,9.4727f,-1.0833f,9.4238f,-1.0833f,9.4238f,-0.9792f,9.3750f,-0.9792f,9.3750f,-0.8750f,9.3262f,-0.8750f,9.3262f,-0.7708f,9.2774f,-0.7708f,9.2774f,-0.6667f,
	9.2285f,-0.6667f,9.2285f,-0.5625f,9.1797f,-0.5625f,9.1797f,-0.4583f,9.1309f,-0.4583f,9.1309f,-0.3542f,9.0820f,-0.3542f,9.0820f,-0.2500f,9.0332f,-0.2500f,9.0332f,-0.1458f,8.9844f,-0.1458f,8.9844f,-0.0417f,8.9356f,-0.0417f,8.9356f,0.0625f,8.8867f,0.0625f,
	8.8867f,0.1667f,8.8379f,0.1667f,8.8379f,0.2708f,8.7891f,0.2708f,8.7891f,0.3750f,8.7402f,0.3750f,8.7402f,0.4792f,8.6914f,0.4792f,8.6914f,0.5833f,8.6426f,0.5833f,8.6426f,0.6875f,8.5938f,0.6875f,8.5938f,0.7917f,8.5449f,0.7917f,8.5449f,0.8958f,
	8.4961f,0.8958f,8.4961f,1.0000f,8.4473f,1.0000f,10.7910f,-4.0000f,10.7422f,-4.0000f,10.7422f,-3.8958f,10.6934f,-3.8958f,10.6934f,-3.7917f,10.6445f,-3.7917f,10.6445f,-3.6875f,10.5957f,-3.6875f,10.5957f,-3.5833f,10.5469f,-3.5833f,10.5469f,-3.4792f,10.4981f,-3.4792f,
	10.4981f,-3.3750f,10.4492f,-3.3750f,10.4492f,-3.2708f,10.4004f,-3.2708f,10.4004f,-3.1667f,10.3516f,-3.1667f,10.3516f,-3.0625f,10.3027f,-3.0625f,10.3027f,-2.9583f,10.2539f,-2.9583f,10.2539f,-2.8542f,10.2051f,-2.8542f,10.2051f,-2.7500f,10.1563f,-2.7500f,10.1563f,-2.6458f,
	10.1074f,-2.6458f,10.1074f,-2.5417f,10.0586f,-2.5417f,10.0586f,-2.4375f,10.0098f,-2.4375f,10.0098f,-2.3333f,9.9609f,-2.3333f,9.9609f,-2.2292f,9.9121f,-2.2292f,9.9121f,-2.1250f,9.8633f,-2.1250f,9.8633f,-2.0208f,9.8145f,-2.0208f,9.8145f,-1.9167f,9.7656f,-1.9167f,
	9.7656f,-1.8125f,9.7168f,-1.8125f,9.7168f,-1.7083f,9.6680f,-1.7083f,9.6680f,-1.6042f,9.6192f,-1.6042f,9.6192f,-1.5000f,9.5703f,-1.5000f,9.5703f,-1.3958f,9.5215f,-1.3958f,9.5215f,-1.2917f,9.4727f,-1.2917f,9.4727f,-1.1875f,9.4238f,-1.1875f,9.4238f,-1.0833f,
	9.3750f,-1.0833f,9.3750f,-0.9792f,9.3262f,-0.9792f,9.3262f,-0.8750f,9.2774f,-0.8750f,9.2774f,-0.7708f,9.2285f,-0.7708f,9.2285f,-0.6667f,9.1797f,-0.6667f,9.1797f,-0.5625f,9.1309f,-0.5625f,9.1309f,-0.4583f,9.0820f,-0.4583f,9.0820f,-0.3542f,9.0332f,-0.3542f,
	9.0332f,-0.2500f,8.9844f,-0.2500f,8.9844f,-0.1458f,8.9356f,-0.1458f,8.9356f,-0.0417f,8.8867f,-0.0417f,8.8867f,0.0625f,8.8379f,0.0625f,8.8379f,0.1667f,8.7891f,0.1667f,8.7891f,0.2708f,8.7402f,0.2708f,8.7402f,0.3750f,8.6914f,0.3750f,8.6914f,0.4792f,
	8.6426f,0.4792f,8.6426f,0.5833f,8.5938f,0.5833f,8.5938f,0.6875f,8.5449f,0.6875f,8.5449f,0.7917f,8.4961f,0.7917f,8.4961f,0.8958f,8.4473f,0.8958f,8.4473f,1.0000f,8.3984f,1.0000f,10.7422f,-4.0000f,10.6934f,-4.0000f,10.6934f,-3.8958f,10.6445f,-3.8958f,
	10.6445f,-3.7917f,10.5957f,-3.7917f,10.5957f,-3.6875f,10.5469f,-3.6875f,10.5469f,-3.5833f,10.4981f,-3.5833f,10.4981f,-3.4792f,10.4492f,-3.4792f,10.4492f,-3.3750f,10.4004f,-3.3750f,10.4004f,-3.2708f,10.3516f,-3.2708f,10.3516f,-3.1667f,10.3027f,-3.1667f,10.3027f,-3.0625f,
	10.2539f,-3.0625f,10.2539f,-2.9583f,10.2051f,-2.9583f,10.2051f,-2.8542f,10.1563f,-2.8542f,10.1563f,-2.7500f,10.1074f,-2.7500f,10.1074f,-2.6458f,10.0586f,-2.6458f,10.0586f,-2.5417f,10.0098f,-2.5417f,10.0098f,-2.4375f,9.9609f,-2.4375f,9.9609f,-2.3333f,9.9121f,-2.3333f,
	9.9121f,-2.2292f,9.8633f,-2.2292f,9.8633f,-2.1250f,9.8145f,-2.1250f,9.8145f,-2.0208f,9.7656f,-2.0208f,9.7656f,-1.9167f,9.7168f,-1.9167f,9.7168f,-1.8125f,9.6680f,-1.8125f,9.6680f,-1.7083f,9.6192f,-1.7083f,9.6192f,-1.6042f,9.5703f,-1.6042f,9.5703f,-1.5000f,
	9.5215f,-1.5000f,9.5215f,-1.3958f,9.4727f,-1.3958f,9.4727f,-1.2917f,9.4238f,-1.2917f,9.4238f,-1.1875f,9.3750f,-1.1875f,9.3750f,-1.0833f,9.3262f,-1.0833f,9.3262f,-0.9792f,9.2774f,-0.9792f,9.2774f,-0.8750f,9.2285f,-0.8750f,9.2285f,-0.7708f,9.1797f,-0.7708f,
	9.1797f,-0.6667f,9.1309f,-0.6667f,9.1309f,-0.5625f,9.0820f,-0.5625f,9.0820f,-0.4583f,9.0332f,-0.4583f,9.0332f,-0.3542f,8.9844f,-0.3542f,8.9844f,-0.2500f,8.9356f,-0.2500f,8.9356f,-0.1458f,8.8867f,-0.1458f,8.8867f,-0.0417f,8.8379f,-0.0417f,8.8379f,0.0625f,
	8.7891f,0.0625f,8.7891f,0.1667f,8.7402f,0.1667f,8.7402f,0.2708f,8.6914f,0.2708f,8.6914f,0.3750f,8.6426f,0.3750f,8.6426f,0.4792f,8.5938f,0.4792f,8.5938f,0.5833f,8.5449f,0.5833f,8.5449f,0.6875f,8.4961f,0.6875f,8.4961f,0.7917f,8.4473f,0.7917f,
	8.4473f,0.8958f,8.3984f,0.8958f,8.3984f,1.0000f,8.3496f,1.0000f,10.6934f,-4.0000f,10.6445f,-4.0000f,10.6445f,-3.8958f,10.5957f,-3.8958f,10.5957f,-3.7917f,10.5469f,-3.7917f,10.5469f,-3.6875f,10.4981f,-3.6875f,10.4981f,-3.5833f,10.4492f,-3.5833f,10.4492f,-3.4792f,
	10.4004f,-3.4792f,10.4004f,-3.3750f,10.3516f,-3.3750f,10.3516f,-3.2708f,10.3027f,-3.2708f,10.3027f,-3.1667f,10.2539f,-3.1667f,10.2539f,-3.0625f,10.2051f,-3.0625f,10.2051f,-2.9583f,10.1563f,-2.9583f,10.1563f,-2.8542f,10.1074f,-2.8542f,10.1074f,-2.7500f,10.0586f,-2.7500f,
	10.0586f,-2.6458f,10.0098f,-2.6458f,10.0098f,-2.5417f,9.9609f,-2.5417f,9.9609f,-2.4375f,9.9121f,-2.4375f,9.9121f,-2.3333f,9.8633f,-2.3333f,9.8633f,-2.2292f,9.8145f,-2.2292f,9.8145f,-2.1250f,9.7656f,-2.1250f,9.7656f,-2.0208f,9.7168f,-2.0208f,9.7168f,-1.9167f,
	9.6680f,-1.9167f,9.6680f,-1.8125f,9.6192f,-1.8125f,9.6192f,-1.7083f,9.5703f,-1.7083f,9.5703f,-1.6042f,9.5215f,-1.6042f,9.5215f,-1.5000f,9.4727f,-1.5000f,9.4727f,-1.3958f,9.4238f,-1.3958f,9.4238f,-1.2917f,9.3750f,-1.2917f,9.3750f,-1.1875f,9.3262f,-1.1875f,
	9.3262f,-1.0833f,9.2774f,-1.0833f,9.2774f,-0.9792f,9.2285f,-0.9792f,9.2285f,-0.8750f,9.1797f,-0.8750f,9.1797f,-0.7708f,9.1309f,-0.7708f,9.1309f,-0.6667f,9.0820f,-0.6667f,9.0820f,-0.5625f,9.0332f,-0.5625f,9.0332f,-0.4583f,8.9844f,-0.4583f,8.9844f,-0.3542f,
	8.9356f,-0.3542f,8.9356f,-0.2500f,8.8867f,-0.2500f,8.8867f,-0.1458f,8.8379f,-0.1458f,8.8379f,-0.0417f,8.7891f,-0.0417f,8.7891f,0.0625f,8.7402f,0.0625f,8.7402f,0.1667f,8.6914f,0.1667f,8.6914f,0.2708f,8.6426f,0.2708f,8.6426f,0.3750f,8.5938f,0.3750f,
	8.5938f,0.4792f,8.5449f,0.4792f,8.5449f,0.5833f,8.4961f,0.5833f,8.4961f,0.6875f,8.4473f,0.6875f,8.4473f,0.7917f,8.3984f,0.7917f,8.3984f,0.8958f,8.3496f,0.8958f,8.3496f,1.0000f,8.3008f,1.0000f,10.6445f,-4.0000f,10.5957f,-4.0000f,10.5957f,-3.8958f,
	10.5469f,-3.8958f,10.5469f,-3.7917f,10.4981f,-3.7917f,10.4981f,-3.6875f,10.4492f,-3.6875f,10.4492f,-3.5833f,10.4004f,-3.5833f,10.4004f,-3.4792f,10.3516f,-3.4792f,10.3516f,-3.3750f,10.3027f,-3.3750f,10.3027f,-3.2708f,10.2539f,-3.2708f,10.2539f,-3.1667f,10.2051f,-3.1667f,
	10.2051f,-3.0625f,10.1563f,-3.0625f,10.1563f,-2.9583f,10.1074f,-2.9583f,10.1074f,-2.8542f,10.0586f,-2.8542f,10.0586f,-2.7500f,10.0098f,-2.7500f,10.0098f,-2.6458f,9.9609f,-2.6458f,9.9609f,-2.5417f,9.9121f,-2.5417f,9.9121f,-2.4375f,9.8633f,-2.4375f,9.8633f,-2.3333f,
	9.8145f,-2.3333f,9.8145f,-2.2292f,9.7656f,-2.2292f,9.7656f,-2.1250f,9.7168f,-2.1250f,9.7168f,-2.0208f,9.6680f,-2.0208f,9.6680f,-1.9167f,9.6192f,-1.9167f,9.6192f,-1.8125f,9.5703f,-1.8125f,9.5703f,-1.7083f,9.5215f,-1.7083f,9.5215f,-1.6042f,9.4727f,-1.6042f,
	9.4727f,-1.5000f,9.4238f,-1.5000f,9.4238f,-1.3958f,9.3750f,-1.3958f,9.3750f,-1.2917f,9.3262f,-1.2917f,9.3262f,-1.1875f,9.2774f,-1.1875f,9.2774f,-1.0833f,9.2285f,-1.0833f,9.2285f,-0.9792f,9.1797f,-0.9792f,9.1797f,-0.8750f,9.1309f,-0.8750f,9.1309f,-0.7708f,
	9.0820f,-0.7708f,9.0820f,-0.6667f,9.0332f,-0.6667f,9.0332f,-0.5625f,8.9844f,-0.5625f,8.9844f,-0.4583f,8.9356f,-0.4583f,8.9356f,-0.3542f,8.8867f,-0.3542f,8.8867f,-0.2500f,8.8379f,-0.2500f,8.8379f,-0.1458f,8.7891f,-0.1458f,8.7891f,-0.0417f,8.7402f,-0.0417f,
	8.7402f,0.0625f,8.6914f,0.0625f,8.6914f,0.1667f,8.6426f,0.1667f,8.6426f,0.2708f,8.5938f,0.2708f,8.5938f,0.3750f,8.5449f,0.3750f,8.5449f,0.4792f,8.4961f,0.4792f,8.4961f,0.5833f,8.4473f,0.5833f,8.4473f,0.6875f,8.3984f,0.6875f,8.3984f,0.7917f,
	8.3496f,0.7917f,8.3496f,0.8958f,8.3008f,0.8958f,8.3008f,1.0000f,8.2520f,1.0000f,10.5957f,-4.0000f,10.5469f,-4.0000f,10.5469f,-3.8958f,10.4981f,-3.8958f,10.4981f,-3.7917f,10.4492f,-3.7917f,10.4492f,-3.6875f,10.4004f,-3.6875f,10.4004f,-3.5833f,10.3516f,-3.5833f,
	10.3516f,-3.4792f,10.3027f,-3.4792f,10.3027f,-3.3750f,10.2539f,-3.3750f,10.2539f,-3.2708f,10.2051f,-3.2708f,10.2051f,-3.1667f,10.1563f,-3.1667f,10.1563f,-3.0625f,10.1074f,-3.0625f,10.1074f,-2.9583f,10.0586f,-2.9583f,10.0586f,-2.8542f,10.0098f,-2.8542f,10.0098f,-2.7500f,
	9.9609f,-2.7500f,9.9609f,-2.6458f,9.9121f,-2.6458f,9.9121f,-2.5417f,9.8633f,-2.5417f,9.8633f,-2.4375f,9.8145f,-2.4375f,9.8145f,-2.3333f,9.7656f,-2.3333f,9.7656f,-2.2292f,9.7168f,-2.2292f,9.7168f,-2.1250f,9.6680f,-2.1250f,9.6680f,-2.0208f,9.6192f,-2.0208f,
	9.6192f,-1.9167f,9.5703f,-1.9167f,9.5703f,-1.8125f,9.5215f,-1.8125f,9.5215f,-1.7083f,9.4727f,-1.7083f,9.4727f,-1.6042f,9.4238f,-1.6042f,9.4238f,-1.5000f,9.3750f,-1.5000f,9.3750f,-1.3958f,9.3262f,-1.3958f,9.3262f,-1.2917f,9.2774f,-1.2917f,9.2774f,-1.1875f,
	9.2285f,-1.1875f,9.2285f,-1.0833f,9.1797f,-1.0833f,9.1797f,-0.9792f,9.1309f,-0.9792f,9.1309f,-0.8750f,9.0820f,-0.8750f,9.0820f,-0.7708f,9.0332f,-0.7708f,9.0332f,-0.6667f,8.9844f,-0.6667f,8.9844f,-0.5625f,8.9356f,-0.5625f,8.9356f,-0.4583f,8.8867f,-0.4583f,
	8.8867f,-0.3542f,8.8379f,-0.3542f,8.8379f,-0.2500f,8.7891f,-0.2500f,8.7891f,-0.1458f,8.7402f,-0.1458f,8.7402f,-0.0417f,8.6914f,-0.0417f,8.6914f,0.0625f,8.6426f,0.0625f,8.6426f,0.1667f,8.5938f,0.1667f,8.5938f,0.2708f,8.5449f,0.2708f,8.5449f,0.3750f,
	8.4961f,0.3750f,8.4961f,0.4792f,8.4473f,0.4792f,8.4473f,0.5833f,8.3984f,0.5833f,8.3984f,0.6875f,8.3496f,0.6875f,8.3496f,0.7917f,8.3008f,0.7917f,8.3008f,0.8958f,8.2520f,0.8958f,8.2520f,1.0000f,8.2031f,1.0000f,10.5469f,-4.0000f,10.4981f,-4.0000f,
	10.4981f,-3.8958f,10.4492f,-3.8958f,10.4492f,-3.7917f,10.4004f,-3.7917f,10.4004f,-3.6875f,10.3516f,-3.6875f,10.3516f,-3.5833f,10.3027f,-3.5833f,10.3027f,-3.4792f,10.2539f,-3.4792f,10.2539f,-3.3750f,10.2051f,-3.3750f,10.2051f,-3.2708f,10.1563f,-3.2708f,10.1563f,-3.1667f,
	10.1074f,-3.1667f,10.1074f,-3.0625f,10.0586f,-3.0625f,10.0586f,-2.9583f,10.0098f,-2.9583f,10.0098f,-2.8542f,9.9609f,-2.8542f,9.9609f,-2.7500f,9.9121f,-2.7500f,9.9121f,-2.6458f,9.8633f,-2.6458f,9.8633f,-2.5417f,9.8145f,-2.5417f,9.8145f,-2.4375f,9.7656f,-2.4375f,
	9.7656f,-2.3333f,9.7168f,-2.3333f,9.7168f,-2.2292f,9.6680f,-2.2292f,9.6680f,-2.1250f,9.6192f,-2.1250f,9.6192f,-2.0208f,9.5703f,-2.0208f,9.5703f,-1.9167f,9.5215f,-1.9167f,9.5215f,-1.8125f,9.4727f,-1.8125f,9.4727f,-1.7083f,9.4238f,-1.7083f,9.4238f,-1.6042f,
	9.3750f,-1.6042f,9.3750f,-1.5000f,9.3262f,-1.5000f,9.3262f,-1.3958f,9.2774f,-1.3958f,9.2774f,-1.2917f,9.2285f,-1.2917f,9.2285f,-1.1875f,9.1797f,-1.1875f,9.1797f,-1.0833f,9.1309f,-1.0833f,9.1309f,-0.9792f,9.0820f,-0.9792f,9.0820f,-0.8750f,9.0332f,-0.8750f,
	9.0332f,-0.7708f,8.9844f,-0.7708f,8.9844f,-0.6667f,8.9356f,-0.6667f,8.9356f,-0.5625f,8.8867f,-0.5625f,8.8867f,-0.4583f,8.8379f,-0.4583f,8.8379f,-0.3542f,8.7891f,-0.3542f,8.7891f,-0.2500f,8.7402f,-0.2500f,8.7402f,-0.1458f,8.6914f,-0.1458f,8.6914f,-0.0417f,
	8.6426f,-0.0417f,8.6426f,0.0625f,8.5938f,0.0625f,8.5938f,0.1667f,8.5449f,0.1667f,8.5449f,0.2708f,8.4961f,0.2708f,8.4961f,0.3750f,8.4473f,0.3750f,8.4473f,0.4792f,8.3984f,0.4792f,8.3984f,0.5833f,8.3496f,0.5833f,8.3496f,0.6875f,8.3008f,0.6875f,
	8.3008f,0.7917f,8.2520f,0.7917f,8.2520f,0.8958f,8.2031f,0.8958f,8.2031f,1.0000f,8.1543f,1.0000f,10.4981f,-4.0000f,10.4492f,-4.0000f,10.4492f,-3.8958f,10.4004f,-3.8958f,10.4004f,-3.7917f,10.3516f,-3.7917f,10.3516f,-3.6875f,10.3027f,-3.6875f,10.3027f,-3.5833f,
	10.2539f,-3.5833f,10.2539f,-3.4792f,10.2051f,-3.4792f,10.2051f,-3.3750f,10.1563f,-3.3750f,10.1563f,-3.2708f,10.1074f,-3.2708f,10.1074f,-3.1667f,10.0586f,-3.1667f,10.0586f,-3.0625f,10.0098f,-3.0625f,10.0098f,-2.9583f,9.9609f,-2.9583f,9.9609f,-2.8542f,9.9121f,-2.8542f,
	9.9121f,-2.7500f,9.8633f,-2.7500f,9.8633f,-2.6458f,9.8145f,-2.6458f,9.8145f,-2.5417f,9.7656f,-2.5417f,9.7656f,-2.4375f,9.7168f,-2.4375f,9.7168f,-2.3333f,9.6680f,-2.3333f,9.6680f,-2.2292f,9.6192f,-2.2292f,9.6192f,-2.1250f,9.5703f,-2.1250f,9.5703f,-2.0208f,
	9.5215f,-2.0208f,9.5215f,-1.9167f,9.4727f,-1.9167f,9.4727f,-1.8125f,9.4238f,-1.8125f,9.4238f,-1.7083f,9.3750f,-1.7083f,9.3750f,-1.6042f,9.3262f,-1.6042f,9.3262f,-1.5000f,9.2774f,-1.5000f,9.2774f,-1.3958f,9.2285f,-1.3958f,9.2285f,-1.2917f,9.1797f,-1.2917f,
	9.1797f,-1.1875f,9.1309f,-1.1875f,9.1309f,-1.0833f,9.0820f,-1.0833f,9.0820f,-0.9792f,9.0332f,-0.9792f,9.0332f,-0.8750f,8.9844f,-0.8750f,8.9844f,-0.7708f,8.9356f,-0.7708f,8.9356f,-0.6667f,8.8867f,-0.6667f,8.8867f,-0.5625f,8.8379f,-0.5625f,8.8379f,-0.4583f,
	8.7891f,-0.4583f,8.7891f,-0.3542f,8.7402f,-0.3542f,8.7402f,-0.2500f,8.6914f,-0.2500f,8.6914f,-0.1458f,8.6426f,-0.1458f,8.6426f,-0.0417f,8.5938f,-0.0417f,8.5938f,0.0625f,8.5449f,0.0625f,8.5449f,0.1667f,8.4961f,0.1667f,8.4961f,0.2708f,8.4473f,0.2708f,
	8.4473f,0.3750f,8.3984f,0.3750f,8.3984f,0.4792f,8.3496f,0.4792f,8.3496f,0.5833f,8.3008f,0.5833f,8.3008f,0.6875f,8.2520f,0.6875f,8.2520f,0.7917f,8.2031f,0.7917f,8.2031f,0.8958f,8.1543f,0.8958f,8.1543f,1.0000f,8.1055f,1.0000f,10.4492f,-4.0000f,
	10.4004f,-4.0000f,10.4004f,-3.8958f,10.3516f,-3.8958f,10.3516f,-3.7917f,10.3027f,-3.7917f,10.3027f,-3.6875f,10.2539f,-3.6875f,10.2539f,-3.5833f,10.2051f,-3.5833f,10.2051f,-3.4792f,10.1563f,-3.4792f,10.1563f,-3.3750f,10.1074f,-3.3750f,10.1074f,-3.2708f,10.0586f,-3.2708f,
	10.0586f,-3.1667f,10.0098f,-3.1667f,10.0098f,-3.0625f,9.9609f,-3.0625f,9.9609f,-2.9583f,9.9121f,-2.9583f,9.9121f,-2.8542f,9.8633f,-2.8542f,9.8633f,-2.7500f,9.8145f,-2.7500f,9.8145f,-2.6458f,9.7656f,-2.6458f,9.7656f,-2.5417f,9.7168f,-2.5417f,9.7168f,-2.4375f,
	9.6680f,-2.4375f,9.6680f,-2.3333f,9.6192f,-2.3333f,9.6192f,-2.2292f,9.5703f,-2.2292f,9.5703f,-2.1250f,9.5215f,-2.1250f,9.5215f,-2.0208f,9.4727f,-2.0208f,9.4727f,-1.9167f,9.4238f,-1.9167f,9.4238f,-1.8125f,9.3750f,-1.8125f,9.3750f,-1.7083f,9.3262f,-1.7083f,
	9.3262f,-1.6042f,9.2774f,-1.6042f,9.2774f,-1.5000f,9.2285f,-1.5000f,9.2285f,-1.3958f,9.1797f,-1.3958f,9.1797f,-1.2917f,9.1309f,-1.2917f,9.1309f,-1.1875f,9.0820f,-1.1875f,9.0820f,-1.0833f,9.0332f,-1.0833f,9.0332f,-0.9792f,8.9844f,-0.9792f,8.9844f,-0.8750f,
	8.9356f,-0.8750f,8.9356f,-0.7708f,8.8867f,-0.7708f,8.8867f,-0.6667f,8.8379f,-0.6667f,8.8379f,-0.5625f,8.7891f,-0.5625f,8.7891f,-0.4583f,8.7402f,-0.4583f,8.7402f,-0.3542f,8.6914f,-0.3542f,8.6914f,-0.2500f,8.6426f,-0.2500f,8.6426f,-0.1458f,8.5938f,-0.1458f,
	8.5938f,-0.0417f,8.5449f,-0.0417f,8.5449f,0.0625f,8.4961f,0.0625f,8.4961f,0.1667f,8.4473f,0.1667f,8.4473f,0.2708f,8.3984f,0.2708f,8.3984f,0.3750f,8.3496f,0.3750f,8.3496f,0.4792f,8.3008f,0.4792f,8.3008f,0.5833f,8.2520f,0.5833f,8.2520f,0.6875f,
	8.2031f,0.6875f,8.2031f,0.7917f,8.1543f,0.7917f,8.1543f,0.8958f,8.1055f,0.8958f,8.1055f,1.0000f,8.0566f,1.0000f,10.4004f,-4.0000f,10.3516f,-4.0000f,10.3516f,-3.8958f,10.3027f,-3.8958f,10.3027f,-3.7917f,10.2539f,-3.7917f,10.2539f,-3.6875f,10.2051f,-3.6875f,
	10.2051f,-3.5833f,10.1563f,-3.5833f,10.1563f,-3.4792f,10.1074f,-3.4792f,10.1074f,-3.3750f,10.0586f,-3.3750f,10.0586f,-3.2708f,10.0098f,-3.2708f,10.0098f,-3.1667f,9.9609f,-3.1667f,9.9609f,-3.0625f,9.9121f,-3.0625f,9.9121f,-2.9583f,9.8633f,-2.9583f,9.8633f,-2.8542f,
	9.8145f,-2.8542f,9.8145f,-2.7500f,9.7656f,-2.7500f,9.7656f,-2.6458f,9.7168f,-2.6458f,9.7168f,-2.5417f,9.6680f,-2.5417f,9.6680f,-2.4375f,9.6192f,-2.4375f,9.6192f,-2.3333f,9.5703f,-2.3333f,9.5703f,-2.2292f,9.5215f,-2.2292f,9.5215f,-2.1250f,9.4727f,-2.1250f,
	9.4727f,-2.0208f,9.4238f,-2.0208f,9.4238f,-1.9167f,9.3750f,-1.9167f,9.3750f,-1.8125f,9.3262f,-1.8125f,9.3262f,-1.7083f,9.2774f,-1.7083f,9.2774f,-1.6042f,9.2285f,-1.6042f,9.2285f,-1.5000f,9.1797f,-1.5000f,9.1797f,-1.3958f,9.1309f,-1.3958f,9.1309f,-1.2917f,
	9.0820f,-1.2917f,9.0820f,-1.1875f,9.0332f,-1.1875f,9.0332f,-1.0833f,8.9844f,-1.0833f,8.9844f,-0.9792f,8.9356f,-0.9792f,8.9356f,-0.8750f,8.8867f,-0.8750f,8.8867f,-0.7708f,8.8379f,-0.7708f,8.8379f,-0.6667f,8.7891f,-0.6667f,8.7891f,-0.5625f,8.7402f,-0.5625f,
	8.7402f,-0.4583f,8.6914f,-0.4583f,8.6914f,-0.3542f,8.6426f,-0.3542f,8.6426f,-0.2500f,8.5938f,-0.2500f,8.5938f,-0.1458f,8.5449f,-0.1458f,8.5449f,-0.0417f,8.4961f,-0.0417f,8.4961f,0.0625f,8.4473f,0.0625f,8.4473f,0.1667f,8.3984f,0.1667f,8.3984f,0.2708f,
	8.3496f,0.2708f,8.3496f,0.3750f,8.3008f,0.3750f,8.3008f,0.4792f,8.2520f,0.4792f,8.2520f,0.5833f,8.2031f,0.5833f,8.2031f,0.6875f,8.1543f,0.6875f,8.1543f,0.7917f,8.1055f,0.7917f,8.1055f,0.8958f,8.0566f,0.8958f,8.0566f,1.0000f,8.0078f,1.0000f,
	10.3516f,-4.0000f,10.3027f,-4.0000f,10.3027f,-3.8958f,10.2539f,-3.8958f,10.2539f,-3.7917f,10.2051f,-3.7917f,10.2051f,-3.6875f,10.1563f,-3.6875f,10.1563f,-3.5833f,10.1074f,-3.5833f,10.1074f,-3.4792f,10.0586f,-3.4792f,10.0586f,-3.3750f,10.0098f,-3.3750f,10.0098f,-3.2708f,
	9.9609f,-3.2708f,9.9609f,-3.1667f,9.9121f,-3.1667f,9.9121f,-3.0625f,9.8633f,-3.0625f,9.8633f,-2.9583f,9.8145f,-2.9583f,9.8145f,-2.8542f,9.7656f,-2.8542f,9.7656f,-2.7500f,9.7168f,-2.7500f,9.7168f,-2.6458f,9.6680f,-2.6458f,9.6680f,-2.5417f,9.6192f,-2.5417f,
	9.6192f,-2.4375f,9.5703f,-2.4375f,9.5703f,-2.3333f,9.5215f,-2.3333f,9.5215f,-2.2292f,9.4727f,-2.2292f,9.4727f,-2.1250f,9.4238f,-2.1250f,9.4238f,-2.0208f,9.3750f,-2.0208f,9.3750f,-1.9167f,9.3262f,-1.9167f,9.3262f,-1.8125f,9.2774f,-1.8125f,9.2774f,-1.7083f,
	9.2285f,-1.7083f,9.2285f,-1.6042f,9.1797f,-1.6042f,9.1797f,-1.5000f,9.1309f,-1.5000f,9.1309f,-1.3958f,9.0820f,-1.3958f,9.0820f,-1.2917f,9.0332f,-1.2917f,9.0332f,-1.1875f,8.9844f,-1.1875f,8.9844f,-1.0833f,8.9356f,-1.0833f,8.9356f,-0.9792f,8.8867f,-0.9792f,
	8.8867f,-0.8750f,8.8379f,-0.8750f,8.8379f,-0.7708f,8.7891f,-0.7708f,8.7891f,-0.6667f,8.7402f,-0.6667f,8.7402f,-0.5625f,8.6914f,-0.5625f,8.6914f,-0.4583f,8.6426f,-0.4583f,8.6426f,-0.3542f,8.5938f,-0.3542f,8.5938f,-0.2500f,8.5449f,-0.2500f,8.5449f,-0.1458f,
	8.4961f,-0.1458f,8.4961f,-0.0417f,8.4473f,-0.0417f,8.4473f,0.0625f,8.3984f,0.0625f,8.3984f,0.1667f,8.3496f,0.1667f,8.3496f,0.2708f,8.3008f,0.2708f,8.3008f,0.3750f,8.2520f,0.3750f,8.2520f,0.4792f,8.2031f,0.4792f,8.2031f,0.5833f,8.1543f,0.5833f,
	8.1543f,0.6875f,8.1055f,0.6875f,8.1055f,0.7917f,8.0566f,0.7917f,8.0566f,0.8958f,8.0078f,0.8958f,8.0078f,1.0000f,7.9590f,1.0000f,10.3027f,-4.0000f,10.2539f,-4.0000f,10.2539f,-3.8958f,10.2051f,-3.8958f,10.2051f,-3.7917f,10.1563f,-3.7917f,10.1563f,-3.6875f,
	10.1074f,-3.6875f,10.1074f,-3.5833f,10.0586f,-3.5833f,10.0586f,-3.4792f,10.0098f,-3.4792f,10.0098f,-3.3750f,9.9609f,-3.3750f,9.9609f,-3.2708f,9.9121f,-3.2708f,9.9121f,-3.1667f,9.8633f,-3.1667f,9.8633f,-3.0625f,9.8145f,-3.0625f,9.8145f,-2.9583f,9.7656f,-2.9583f,
	9.7656f,-2.8542f,9.7168f,-2.8542f,9.7168f,-2.7500f,9.6680f,-2.7500f,9.6680f,-2.6458f,9.6192f,-2.6458f,9.6192f,-2.5417f,9.5703f,-2.5417f,9.5703f,-2.4375f,9.5215f,-2.4375f,9.5215f,-2.3333f,9.4727f,-2.3333f,9.4727f,-2.2292f,9.4238f,-2.2292f,9.4238f,-2.1250f,
	9.3750f,-2.1250f,9.3750f,-2.0208f,9.3262f,-2.0208f,9.3262f,-1.9167f,9.2774f,-1.9167f,9.2774f,-1.8125f,9.2285f,-1.8125f,9.2285f,-1.7083f,9.1797f,-1.7083f,9.1797f,-1.6042f,9.1309f,-1.6042f,9.1309f,-1.5000f,9.0820f,-1.5000f,9.0820f,-1.3958f,9.0332f,-1.3958f,
	9.0332f,-1.2917f,8.9844f,-1.2917f,8.9844f,-1.1875f,8.9356f,-1.1875f,8.9356f,-1.0833f,8.8867f,-1.0833f,8.8867f,-0.9792f,8.8379f,-0.9792f,8.8379f,-0.8750f,8.7891f,-0.8750f,8.7891f,-0.7708f,8.7402f,-0.7708f,8.7402f,-0.6667f,8.6914f,-0.6667f,8.6914f,-0.5625f,
	8.6426f,-0.5625f,8.6426f,-0.4583f,8.5938f,-0.4583f,8.5938f,-0.3542f,8.5449f,-0.3542f,8.5449f,-0.2500f,8.4961f,-0.2500f,8.4961f,-0.1458f,8.4473f,-0.1458f,8.4473f,-0.0417f,8.3984f,-0.0417f,8.3984f,0.0625f,8.3496f,0.0625f,8.3496f,0.1667f,8.3008f,0.1667f,
	8.3008f,0.2708f,8.2520f,0.2708f,8.2520f,0.3750f,8.2031f,0.3750f,8.2031f,0.4792f,8.1543f,0.4792f,8.1543f,0.5833f,8.1055f,0.5833f,8.1055f,0.6875f,8.0566f,0.6875f,8.0566f,0.7917f,8.0078f,0.7917f,8.0078f,0.8958f,7.9590f,0.8958f,7.9590f,1.0000f,
	7.9102f,1.0000f,10.2539f,-4.0000f,10.2051f,-4.0000f,10.2051f,-3.8958f,10.1563f,-3.8958f,10.1563f,-3.7917f,10.1074f,-3.7917f,10.1074f,-3.6875f,10.0586f,-3.6875f,10.0586f,-3.5833f,10.0098f,-3.5833f,10.0098f,-3.4792f,9.9609f,-3.4792f,9.9609f,-3.3750f,9.9121f,-3.3750f,
	9.9121f,-3.2708f,9.8633f,-3.2708f,9.8633f,-3.1667f,9.8145f,-3.1667f,9.8145f,-3.0625f,9.7656f,-3.0625f,9.7656f,-2.9583f,9.7168f,-2.9583f,9.7168f,-2.8542f,9.6680f,-2.8542f,9.6680f,-2.7500f,9.6192f,-2.7500f,9.6192f,-2.6458f,9.5703f,-2.6458f,9.5703f,-2.5417f,
	9.5215f,-2.5417f,9.5215f,-2.4375f,9.4727f,-2.4375f,9.4727f,-2.3333f,9.4238f,-2.3333f,9.4238f,-2.2292f,9.3750f,-2.2292f,9.3750f,-2.1250f,9.3262f,-2.1250f,9.3262f,-2.0208f,9.2774f,-2.0208f,9.2774f,-1.9167f,9.2285f,-1.9167f,9.2285f,-1.8125f,9.1797f,-1.8125f,
	9.1797f,-1.7083f,9.1309f,-1.7083f,9.1309f,-1.6042f,9.0820f,-1.6042f,9.0820f,-1.5000f,9.0332f,-1.5000f,9.0332f,-1.3958f,8.9844f,-1.3958f,8.9844f,-1.2917f,8.9356f,-1.2917f,8.9356f,-1.1875f,8.8867f,-1.1875f,8.8867f,-1.0833f,8.8379f,-1.0833f,8.8379f,-0.9792f,
	8.7891f,-0.9792f,8.7891f,-0.8750f,8.7402f,-0.8750f,8.7402f,-0.7708f,8.6914f,-0.7708f,8.6914f,-0.6667f,8.6426f,-0.6667f,8.6426f,-0.5625f,8.5938f,-0.5625f,8.5938f,-0.4583f,8.5449f,-0.4583f,8.5449f,-0.3542f,8.4961f,-0.3542f,8.4961f,-0.2500f,8.4473f,-0.2500f,
	8.4473f,-0.1458f,8.3984f,-0.1458f,8.3984f,-0.0417f,8.3496f,-0.0417f,8.3496f,0.0625f,8.3008f,0.0625f,8.3008f,0.1667f,8.2520f,0.1667f,8.2520f,0.2708f,8.2031f,0.2708f,8.2031f,0.3750f,8.1543f,0.3750f,8.1543f,0.4792f,8.1055f,0.4792f,8.1055f,0.5833f,
	8.0566f,0.5833f,8.0566f,0.6875f,8.0078f,0.6875f,8.0078f,0.7917f,7.9590f,0.7917f,7.9590f,0.8958f,7.9102f,0.8958f,7.9102f,1.0000f,7.8613f,1.0000f,10.2051f,-4.0000f,10.1563f,-4.0000f,10.1563f,-3.8958f,10.1074f,-3.8958f,10.1074f,-3.7917f,10.0586f,-3.7917f,
	10.0586f,-3.6875f,10.0098f,-3.6875f,10.0098f,-3.5833f,9.9609f,-3.5833f,9.9609f,-3.4792f,9.9121f,-3.4792f,9.9121f,-3.3750f,9.8633f,-3.3750f,9.8633f,-3.2708f,9.8145f,-3.2708f,9.8145f,-3.1667f,9.7656f,-3.1667f,9.7656f,-3.0625f,9.7168f,-3.0625f,9.7168f,-2.9583f,
	9.6680f,-2.9583f,9.6680f,-2.8542f,9.6192f,-2.8542f,9.6192f,-2.7500f,9.5703f,-2.7500f,9.5703f,-2.6458f,9.5215f,-2.6458f,9.5215f,-2.5417f,9.4727f,-2.5417f,9.4727f,-2.4375f,9.4238f,-2.4375f,9.4238f,-2.3333f,9.3750f,-2.3333f,9.3750f,-2.2292f,9.3262f,-2.2292f,
	9.3262f,-2.1250f,9.2774f,-2.1250f,9.2774f,-2.0208f,9.2285f,-2.0208f,9.2285f,-1.9167f,9.1797f,-1.9167f,9.1797f,-1.8125f,9.1309f,-1.8125f,9.1309f,-1.7083f,9.0820f,-1.7083f,9.0820f,-1.6042f,9.0332f,-1.6042f,9.0332f,-1.5000f,8.9844f,-1.5000f,8.9844f,-1.3958f,
	8.9356f,-1.3958f,8.9356f,-1.2917f,8.8867f,-1.2917f,8.8867f,-1.1875f,8.8379f,-1.1875f,8.8379f,-1.0833f,8.7891f,-1.0833f,8.7891f,-0.9792f,8.7402f,-0.9792f,8.7402f,-0.8750f,8.6914f,-0.8750f,8.6914f,-0.7708f,8.6426f,-0.7708f,8.6426f,-0.6667f,8.5938f,-0.6667f,
	8.5938f,-0.5625f,8.5449f,-0.5625f,8.5449f,-0.4583f,8.4961f,-0.4583f,8.4961f,-0.3542f,8.4473f,-0.3542f,8.4473f,-0.2500f,8.3984f,-0.2500f,8.3984f,-0.1458f,8.3496f,-0.1458f,8.3496f,-0.0417f,8.3008f,-0.0417f,8.3008f,0.0625f,8.2520f,0.0625f,8.2520f,0.1667f,
	8.2031f,0.1667f,8.2031f,0.2708f,8.1543f,0.2708f,8.1543f,0.3750f,8.1055f,0.3750f,8.1055f,0.4792f,8.0566f,0.4792f,8.0566f,0.5833f,8.0078f,0.5833f,8.0078f,0.6875f,7.9590f,0.6875f,7.9590f,0.7917f,7.9102f,0.7917f,7.9102f,0.8958f,7.8613f,0.8958f,
	7.8613f,1.0000f,7.8125f,1.0000f,10.1563f,-4.0000f,10.1074f,-4.0000f,10.1074f,-3.8958f,10.0586f,-3.8958f,10.0586f,-3.7917f,10.0098f,-3.7917f,10.0098f,-3.6875f,9.9609f,-3.6875f,9.9609f,-3.5833f,9.9121f,-3.5833f,9.9121f,-3.4792f,9.8633f,-3.4792f,9.8633f,-3.3750f,
	9.8145f,-3.3750f,9.8145f,-3.2708f,9.7656f,-3.2708f,9.7656f,-3.1667f,9.7168f,-3.1667f,9.7168f,-3.0625f,9.6680f,-3.0625f,9.6680f,-2.9583f,9.6192f,-2.9583f,9.6192f,-2.8542f,9.5703f,-2.8542f,9.5703f,-2.7500f,9.5215f,-2.7500f,9.5215f,-2.6458f,9.4727f,-2.6458f,
	9.4727f,-2.5417f,9.4238f,-2.5417f,9.4238f,-2.4375f,9.3750f,-2.4375f,9.3750f,-2.3333f,9.3262f,-2.3333f,9.3262f,-2.2292f,9.2774f,-2.2292f,9.2774f,-2.1250f,9.2285f,-2.1250f,9.2285f,-2.0208f,9.1797f,-2.0208f,9.1797f,-1.9167f,9.1309f,-1.9167f,9.1309f,-1.8125f,
	9.0820f,-1.8125f,9.0820f,-1.7083f,9.0332f,-1.7083f,9.0332f,-1.6042f,8.9844f,-1.6042f,8.9844f,-1.5000f,8.9356f,-1.5000f,8.9356f,-1.3958f,8.8867f,-1.3958f,8.8867f,-1.2917f,8.8379f,-1.2917f,8.8379f,-1.1875f,8.7891f,-1.1875f,8.7891f,-1.0833f,8.7402f,-1.0833f,
	8.7402f,-0.9792f,8.6914f,-0.9792f,8.6914f,-0.8750f,8.6426f,-0.8750f,8.6426f,-0.7708f,8.5938f,-0.7708f,8.5938f,-0.6667f,8.5449f,-0.6667f,8.5449f,-0.5625f,8.4961f,-0.5625f,8.4961f,-0.4583f,8.4473f,-0.4583f,8.4473f,-0.3542f,8.3984f,-0.3542f,8.3984f,-0.2500f,
	8.3496f,-0.2500f,8.3496f,-0.1458f,8.3008f,-0.1458f,8.3008f,-0.0417f,8.2520f,-0.0417f,8.2520f,0.0625f,8.2031f,0.0625f,8.2031f,0.1667f,8.1543f,0.1667f,8.1543f,0.2708f,8.1055f,0.2708f,8.1055f,0.3750f,8.0566f,0.3750f,8.0566f,0.4792f,8.0078f,0.4792f,
	8.0078f,0.5833f,7.9590f,0.5833f,7.9590f,0.6875f,7.9102f,0.6875f,7.9102f,0.7917f,7.8613f,0.7917f,7.8613f,0.8958f,7.8125f,0.8958f,7.8125f,1.0000f,7.7637f,1.0000f,10.1074f,-4.0000f,10.0586f,-4.0000f,10.0586f,-3.8958f,10.0098f,-3.8958f,10.0098f,-3.7917f,
	9.9609f,-3.7917f,9.9609f,-3.6875f,9.9121f,-3.6875f,9.9121f,-3.5833f,9.8633f,-3.5833f,9.8633f,-3.4792f,9.8145f,-3.4792f,9.8145f,-3.3750f,9.7656f,-3.3750f,9.7656f,-3.2708f,9.7168f,-3.2708f,9.7168f,-3.1667f,9.6680f,-3.1667f,9.6680f,-3.0625f,9.6192f,-3.0625f,
	9.6192f,-2.9583f,9.5703f,-2.9583f,9.5703f,-2.8542f,9.5215f,-2.8542f,9.5215f,-2.7500f,9.4727f,-2.7500f,9.4727f,-2.6458f,9.4238f,-2.6458f,9.4238f,-2.5417f,9.3750f,-2.5417f,9.3750f,-2.4375f,9.3262f,-2.4375f,9.3262f,-2.3333f,9.2774f,-2.3333f,9.2774f,-2.2292f,
	9.2285f,-2.2292f,9.2285f,-2.1250f,9.1797f,-2.1250f,9.1797f,-2.0208f,9.1309f,-2.0208f,9.1309f,-1.9167f,9.0820f,-1.9167f,9.0820f,-1.8125f,9.0332f,-1.8125f,9.0332f,-1.7083f,8.9844f,-1.7083f,8.9844f,-1.6042f,8.9356f,-1.6042f,8.9356f,-1.5000f,8.8867f,-1.5000f,
	8.8867f,-1.3958f,8.8379f,-1.3958f,8.8379f,-1.2917f,8.7891f,-1.2917f,8.7891f,-1.1875f,8.7402f,-1.1875f,8.7402f,-1.0833f,8.6914f,-1.0833f,8.6914f,-0.9792f,8.6426f,-0.9792f,8.6426f,-0.8750f,8.5938f,-0.8750f,8.5938f,-0.7708f,8.5449f,-0.7708f,8.5449f,-0.6667f,
	8.4961f,-0.6667f,8.4961f,-0.5625f,8.4473f,-0.5625f,8.4473f,-0.4583f,8.3984f,-0.4583f,8.3984f,-0.3542f,8.3496f,-0.3542f,8.3496f,-0.2500f,8.3008f,-0.2500f,8.3008f,-0.1458f,8.2520f,-0.1458f,8.2520f,-0.0417f,8.2031f,-0.0417f,8.2031f,0.0625f,8.1543f,0.0625f,
	8.1543f,0.1667f,8.1055f,0.1667f,8.1055f,0.2708f,8.0566f,0.2708f,8.0566f,0.3750f,8.0078f,0.3750f,8.0078f,0.4792f,7.9590f,0.4792f,7.9590f,0.5833f,7.9102f,0.5833f,7.9102f,0.6875f,7.8613f,0.6875f,7.8613f,0.7917f,7.8125f,0.7917f,7.8125f,0.8958f,
	7.7637f,0.8958f,7.7637f,1.0000f,7.7149f,1.0000f,10.0586f,-4.0000f,10.0098f,-4.0000f,10.0098f,-3.8958f,9.9609f,-3.8958f,9.9609f,-3.7917f,9.9121f,-3.7917f,9.9121f,-3.6875f,9.8633f,-3.6875f,9.8633f,-3.5833f,9.8145f,-3.5833f,9.8145f,-3.4792f,9.7656f,-3.4792f,
	9.7656f,-3.3750f,9.7168f,-3.3750f,9.7168f,-3.2708f,9.6680f,-3.2708f,9.6680f,-3.1667f,9.6192f,-3.1667f,9.6192f,-3.0625f,9.5703f,-3.0625f,9.5703f,-2.9583f,9.5215f,-2.9583f,9.5215f,-2.8542f,9.4727f,-2.8542f,9.4727f,-2.7500f,9.4238f,-2.7500f,9.4238f,-2.6458f,
	9.3750f,-2.6458f,9.3750f,-2.5417f,9.3262f,-2.5417f,9.3262f,-2.4375f,9.2774f,-2.4375f,9.2774f,-2.3333f,9.2285f,-2.3333f,9.2285f,-2.2292f,9.1797f,-2.2292f,9.1797f,-2.1250f,9.1309f,-2.1250f,9.1309f,-2.0208f,9.0820f,-2.0208f,9.0820f,-1.9167f,9.0332f,-1.9167f,
	9.0332f,-1.8125f,8.9844f,-1.8125f,8.9844f,-1.7083f,8.9356f,-1.7083f,8.9356f,-1.6042f,8.8867f,-1.6042f,8.8867f,-1.5000f,8.8379f,-1.5000f,8.8379f,-1.3958f,8.7891f,-1.3958f,8.7891f,-1.2917f,8.7402f,-1.2917f,8.7402f,-1.1875f,8.6914f,-1.1875f,8.6914f,-1.0833f,
	8.6426f,-1.0833f,8.6426f,-0.9792f,8.5938f,-0.9792f,8.5938f,-0.8750f,8.5449f,-0.8750f,8.5449f,-0.7708f,8.4961f,-0.7708f,8.4961f,-0.6667f,8.4473f,-0.6667f,8.4473f,-0.5625f,8.3984f,-0.5625f,8.3984f,-0.4583f,8.3496f,-0.4583f,8.3496f,-0.3542f,8.3008f,-0.3542f,
	8.3008f,-0.2500f,8.2520f,-0.2500f,8.2520f,-0.1458f,8.2031f,-0.1458f,8.2031f,-0.0417f,8.1543f,-0.0417f,8.1543f,0.0625f,8.1055f,0.0625f,8.1055f,0.1667f,8.0566f,0.1667f,8.0566f,0.2708f,8.0078f,0.2708f,8.0078f,0.3750f,7.9590f,0.3750f,7.9590f,0.4792f,
	7.9102f,0.4792f,7.9102f,0.5833f,7.8613f,0.5833f,7.8613f,0.6875f,7.8125f,0.6875f,7.8125f,0.7917f,7.7637f,0.7917f,7.7637f,0.8958f,7.7149f,0.8958f,7.7149f,1.0000f,7.6660f,1.0000f,10.0098f,-4.0000f,9.9609f,-4.0000f,9.9609f,-3.8958f,9.9121f,-3.8958f,
	9.9121f,-3.7917f,9.8633f,-3.7917f,9.8633f,-3.6875f,9.8145f,-3.6875f,9.8145f,-3.5833f,9.7656f,-3.5833f,9.7656f,-3.4792f,9.7168f,-3.4792f,9.7168f,-3.3750f,9.6680f,-3.3750f,9.6680f,-3.2708f,9.6192f,-3.2708f,9.6192f,-3.1667f,9.5703f,-3.1667f,9.5703f,-3.0625f,
	9.5215f,-3.0625f,9.5215f,-2.9583f,9.4727f,-2.9583f,9.4727f,-2.8542f,9.4238f,-2.8542f,9.4238f,-2.7500f,9.3750f,-2.7500f,9.3750f,-2.6458f,9.3262f,-2.6458f,9.3262f,-2.5417f,9.2774f,-2.5417f,9.2774f,-2.4375f,9.2285f,-2.4375f,9.2285f,-2.3333f,9.1797f,-2.3333f,
	9.1797f,-2.2292f,9.1309f,-2.2292f,9.1309f,-2.1250f,9.0820f,-2.1250f,9.0820f,-2.0208f,9.0332f,-2.0208f,9.0332f,-1.9167f,8.9844f,-1.9167f,8.9844f,-1.8125f,8.9356f,-1.8125f,8.9356f,-1.7083f,8.8867f,-1.7083f,8.8867f,-1.6042f,8.8379f,-1.6042f,8.8379f,-1.5000f,
	8.7891f,-1.5000f,8.7891f,-1.3958f,8.7402f,-1.3958f,8.7402f,-1.2917f,8.6914f,-1.2917f,8.6914f,-1.1875f,8.6426f,-1.1875f,8.6426f,-1.0833f,8.5938f,-1.0833f,8.5938f,-0.9792f,8.5449f,-0.9792f,8.5449f,-0.8750f,8.4961f,-0.8750f,8.4961f,-0.7708f,8.4473f,-0.7708f,
	8.4473f,-0.6667f,8.3984f,-0.6667f,8.3984f,-0.5625f,8.3496f,-0.5625f,8.3496f,-0.4583f,8.3008f,-0.4583f,8.3008f,-0.3542f,8.2520f,-0.3542f,8.2520f,-0.2500f,8.2031f,-0.2500f,8.2031f,-0.1458f,8.1543f,-0.1458f,8.1543f,-0.0417f,8.1055f,-0.0417f,8.1055f,0.0625f,
	8.0566f,0.0625f,8.0566f,0.1667f,8.0078f,0.1667f,8.0078f,0.2708f,7.9590f,0.2708f,7.9590f,0.3750f,7.9102f,0.3750f,7.9102f,0.4792f,7.8613f,0.4792f,7.8613f,0.5833f,7.8125f,0.5833f,7.8125f,0.6875f,7.7637f,0.6875f,7.7637f,0.7917f,7.7149f,0.7917f,
	7.7149f,0.8958f,7.6660f,0.8958f,7.6660f,1.0000f,7.6172f,1.0000f,9.9609f,-4.0000f,9.9121f,-4.0000f,9.9121f,-3.8958f,9.8633f,-3.8958f,9.8633f,-3.7917f,9.8145f,-3.7917f,9.8145f,-3.6875f,9.7656f,-3.6875f,9.7656f,-3.5833f,9.7168f,-3.5833f,9.7168f,-3.4792f,
	9.6680f,-3.4792f,9.6680f,-3.3750f,9.6192f,-3.3750f,9.6192f,-3.2708f,9.5703f,-3.2708f,9.5703f,-3.1667f,9.5215f,-3.1667f,9.5215f,-3.0625f,9.4727f,-3.0625f,9.4727f,-2.9583f,9.4238f,-2.9583f,9.4238f,-2.8542f,9.3750f,-2.8542f,9.3750f,-2.7500f,9.3262f,-2.7500f,
	9.3262f,-2.6458f,9.2774f,-2.6458f,9.2774f,-2.5417f,9.2285f,-2.5417f,9.2285f,-2.4375f,9.1797f,-2.4375f,9.1797f,-2.3333f,9.1309f,-2.3333f,9.1309f,-2.2292f,9.0820f,-2.2292f,9.0820f,-2.1250f,9.0332f,-2.1250f,9.0332f,-2.0208f,8.9844f,-2.0208f,8.9844f,-1.9167f,
	8.9356f,-1.9167f,8.9356f,-1.8125f,8.8867f,-1.8125f,8.8867f,-1.7083f,8.8379f,-1.7083f,8.8379f,-1.6042f,8.7891f,-1.6042f,8.7891f,-1.5000f,8.7402f,-1.5000f,8.7402f,-1.3958f,8.6914f,-1.3958f,8.6914f,-1.2917f,8.6426f,-1.2917f,8.6426f,-1.1875f,8.5938f,-1.1875f,
	8.5938f,-1.0833f,8.5449f,-1.0833f,8.5449f,-0.9792f,8.4961f,-0.9792f,8.4961f,-0.8750f,8.4473f,-0.8750f,8.4473f,-0.7708f,8.3984f,-0.7708f,8.3984f,-0.6667f,8.3496f,-0.6667f,8.3496f,-0.5625f,8.3008f,-0.5625f,8.3008f,-0.4583f,8.2520f,-0.4583f,8.2520f,-0.3542f,
	8.2031f,-0.3542f,8.2031f,-0.2500f,8.1543f,-0.2500f,8.1543f,-0.1458f,8.1055f,-0.1458f,8.1055f,-0.0417f,8.0566f,-0.0417f,8.0566f,0.0625f,8.0078f,0.0625f,8.0078f,0.1667f,7.9590f,0.1667f,7.9590f,0.2708f,7.9102f,0.2708f,7.9102f,0.3750f,7.8613f,0.3750f,
	7.8613f,0.4792f,7.8125f,0.4792f,7.8125f,0.5833f,7.7637f,0.5833f,7.7637f,0.6875f,7.7149f,0.6875f,7.7149f,0.7917f,7.6660f,0.7917f,7.6660f,0.8958f,7.6172f,0.8958f,7.6172f,1.0000f,7.5684f,1.0000f,9.9121f,-4.0000f,9.8633f,-4.0000f,9.8633f,-3.8958f,
	9.8145f,-3.8958f,9.8145f,-3.7917f,9.7656f,-3.7917f,9.7656f,-3.6875f,9.7168f,-3.6875f,9.7168f,-3.5833f,9.6680f,-3.5833f,9.6680f,-3.4792f,9.6192f,-3.4792f,9.6192f,-3.3750f,9.5703f,-3.3750f,9.5703f,-3.2708f,9.5215f,-3.2708f,9.5215f,-3.1667f,9.4727f,-3.1667f,
	9.4727f,-3.0625f,9.4238f,-3.0625f,9.4238f,-2.9583f,9.3750f,-2.9583f,9.3750f,-2.8542f,9.3262f,-2.8542f,9.3262f,-2.7500f,9.2774f,-2.7500f,9.2774f,-2.6458f,9.2285f,-2.6458f,9.2285f,-2.5417f,9.1797f,-2.5417f,9.1797f,-2.4375f,9.1309f,-2.4375f,9.1309f,-2.3333f,
	9.0820f,-2.3333f,9.0820f,-2.2292f,9.0332f,-2.2292f,9.0332f,-2.1250f,8.9844f,-2.1250f,8.9844f,-2.0208f,8.9356f,-2.0208f,8.9356f,-1.9167f,8.8867f,-1.9167f,8.8867f,-1.8125f,8.8379f,-1.8125f,8.8379f,-1.7083f,8.7891f,-1.7083f,8.7891f,-1.6042f,8.7402f,-1.6042f,
	8.7402f,-1.5000f,8.6914f,-1.5000f,8.6914f,-1.3958f,8.6426f,-1.3958f,8.6426f,-1.2917f,8.5938f,-1.2917f,8.5938f,-1.1875f,8.5449f,-1.1875f,8.5449f,-1.0833f,8.4961f,-1.0833f,8.4961f,-0.9792f,8.4473f,-0.9792f,8.4473f,-0.8750f,8.3984f,-0.8750f,8.3984f,-0.7708f,
	8.3496f,-0.7708f,8.3496f,-0.6667f,8.3008f,-0.6667f,8.3008f,-0.5625f,8.2520f,-0.5625f,8.2520f,-0.4583f,8.2031f,-0.4583f,8.2031f,-0.3542f,8.1543f,-0.3542f,8.1543f,-0.2500f,8.1055f,-0.2500f,8.1055f,-0.1458f,8.0566f,-0.1458f,8.0566f,-0.0417f,8.0078f,-0.0417f,
	8.0078f,0.0625f,7.9590f,0.0625f,7.9590f,0.1667f,7.9102f,0.1667f,7.9102f,0.2708f,7.8613f,0.2708f,7.8613f,0.3750f,7.8125f,0.3750f,7.8125f,0.4792f,7.7637f,0.4792f,7.7637f,0.5833f,7.7149f,0.5833f,7.7149f,0.6875f,7.6660f,0.6875f,7.6660f,0.7917f,
	7.6172f,0.7917f,7.6172f,0.8958f,7.5684f,0.8958f,7.5684f,1.0000f,7.5195f,1.0000f,9.8633f,-4.0000f,9.8145f,-4.0000f,9.8145f,-3.8958f,9.7656f,-3.8958f,9.7656f,-3.7917f,9.7168f,-3.7917f,9.7168f,-3.6875f,9.6680f,-3.6875f,9.6680f,-3.5833f,9.6192f,-3.5833f,
	9.6192f,-3.4792f,9.5703f,-3.4792f,9.5703f,-3.3750f,9.5215f,-3.3750f,9.5215f,-3.2708f,9.4727f,-3.2708f,9.4727f,-3.1667f,9.4238f,-3.1667f,9.4238f,-3.0625f,9.3750f,-3.0625f,9.3750f,-2.9583f,9.3262f,-2.9583f,9.3262f,-2.8542f,9.2774f,-2.8542f,9.2774f,-2.7500f,
	9.2285f,-2.7500f,9.2285f,-2.6458f,9.1797f,-2.6458f,9.1797f,-2.5417f,9.1309f,-2.5417f,9.1309f,-2.4375f,9.0820f,-2.4375f,9.0820f,-2.3333f,9.0332f,-2.3333f,9.0332f,-2.2292f,8.9844f,-2.2292f,8.9844f,-2.1250f,8.9356f,-2.1250f,8.9356f,-2.0208f,8.8867f,-2.0208f,
	8.8867f,-1.9167f,8.8379f,-1.9167f,8.8379f,-1.8125f,8.7891f,-1.8125f,8.7891f,-1.7083f,8.7402f,-1.7083f,8.7402f,-1.6042f,8.6914f,-1.6042f,8.6914f,-1.5000f,8.6426f,-1.5000f,8.6426f,-1.3958f,8.5938f,-1.3958f,8.5938f,-1.2917f,8.5449f,-1.2917f,8.5449f,-1.1875f,
	8.4961f,-1.1875f,8.4961f,-1.0833f,8.4473f,-1.0833f,8.4473f,-0.9792f,8.3984f,-0.9792f,8.3984f,-0.8750f,8.3496f,-0.8750f,8.3496f,-0.7708f,8.3008f,-0.7708f,8.3008f,-0.6667f,8.2520f,-0.6667f,8.2520f,-0.5625f,8.2031f,-0.5625f,8.2031f,-0.4583f,8.1543f,-0.4583f,
	8.1543f,-0.3542f,8.1055f,-0.3542f,8.1055f,-0.2500f,8.0566f,-0.2500f,8.0566f,-0.1458f,8.0078f,-0.1458f,8.0078f,-0.0417f,7.9590f,-0.0417f,7.9590f,0.0625f,7.9102f,0.0625f,7.9102f,0.1667f,7.8613f,0.1667f,7.8613f,0.2708f,7.8125f,0.2708f,7.8125f,0.3750f,
	7.7637f,0.3750f,7.7637f,0.4792f,7.7149f,0.4792f,7.7149f,0.5833f,7.6660f,0.5833f,7.6660f,0.6875f,7.6172f,0.6875f,7.6172f,0.7917f,7.5684f,0.7917f,7.5684f,0.8958f,7.5195f,0.8958f,7.5195f,1.0000f,7.4707f,1.0000f,9.8145f,-4.0000f,9.7656f,-4.0000f,
	9.7656f,-3.8958f,9.7168f,-3.8958f,9.7168f,-3.7917f,9.6680f,-3.7917f,9.6680f,-3.6875f,9.6192f,-3.6875f,9.6192f,-3.5833f,9.5703f,-3.5833f,9.5703f,-3.4792f,9.5215f,-3.4792f,9.5215f,-3.3750f,9.4727f,-3.3750f,9.4727f,-3.2708f,9.4238f,-3.2708f,9.4238f,-3.1667f,
	9.3750f,-3.1667f,9.3750f,-3.0625f,9.3262f,-3.0625f,9.3262f,-2.9583f,9.2774f,-2.9583f,9.2774f,-2.8542f,9.2285f,-2.8542f,9.2285f,-2.7500f,9.1797f,-2.7500f,9.1797f,-2.6458f,9.1309f,-2.6458f,9.1309f,-2.5417f,9.0820f,-2.5417f,9.0820f,-2.4375f,9.0332f,-2.4375f,
	9.0332f,-2.3333f,8.9844f,-2.3333f,8.9844f,-2.2292f,8.9356f,-2.2292f,8.9356f,-2.1250f,8.8867f,-2.1250f,8.8867f,-2.0208f,8.8379f,-2.0208f,8.8379f,-1.9167f,8.7891f,-1.9167f,8.7891f,-1.8125f,8.7402f,-1.8125f,8.7402f,-1.7083f,8.6914f,-1.7083f,8.6914f,-1.6042f,
	8.6426f,-1.6042f,8.6426f,-1.5000f,8.5938f,-1.5000f,8.5938f,-1.3958f,8.5449f,-1.3958f,8.5449f,-1.2917f,8.4961f,-1.2917f,8.4961f,-1.1875f,8.4473f,-1.1875f,8.4473f,-1.0833f,8.3984f,-1.0833f,8.3984f,-0.9792f,8.3496f,-0.9792f,8.3496f,-0.8750f,8.3008f,-0.8750f,
	8.3008f,-0.7708f,8.2520f,-0.7708f,8.2520f,-0.6667f,8.2031f,-0.6667f,8.2031f,-0.5625f,8.1543f,-0.5625f,8.1543f,-0.4583f,8.1055f,-0.4583f,8.1055f,-0.3542f,8.0566f,-0.3542f,8.0566f,-0.2500f,8.0078f,-0.2500f,8.0078f,-0.1458f,7.9590f,-0.1458f,7.9590f,-0.0417f,
	7.9102f,-0.0417f,7.9102f,0.0625f,7.8613f,0.0625f,7.8613f,0.1667f,7.8125f,0.1667f,7.8125f,0.2708f,7.7637f,0.2708f,7.7637f,0.3750f,7.7149f,0.3750f,7.7149f,0.4792f,7.6660f,0.4792f,7.6660f,0.5833f,7.6172f,0.5833f,7.6172f,0.6875f,7.5684f,0.6875f,
	7.5684f,0.7917f,7.5195f,0.7917f,7.5195f,0.8958f,7.4707f,0.8958f,7.4707f,1.0000f,7.4219f,1.0000f,9.7656f,-4.0000f,9.7168f,-4.0000f,9.7168f,-3.8958f,9.6680f,-3.8958f,9.6680f,-3.7917f,9.6192f,-3.7917f,9.6192f,-3.6875f,9.5703f,-3.6875f,9.5703f,-3.5833f,
	9.5215f,-3.5833f,9.5215f,-3.4792f,9.4727f,-3.4792f,9.4727f,-3.3750f,9.4238f,-3.3750f,9.4238f,-3.2708f,9.3750f,-3.2708f,9.3750f,-3.1667f,9.3262f,-3.1667f,9.3262f,-3.0625f,9.2774f,-3.0625f,9.2774f,-2.9583f,9.2285f,-2.9583f,9.2285f,-2.8542f,9.1797f,-2.8542f,
	9.1797f,-2.7500f,9.1309f,-2.7500f,9.1309f,-2.6458f,9.0820f,-2.6458f,9.0820f,-2.5417f,9.0332f,-2.5417f,9.0332f,-2.4375f,8.9844f,-2.4375f,8.9844f,-2.3333f,8.9356f,-2.3333f,8.9356f,-2.2292f,8.8867f,-2.2292f,8.8867f,-2.1250f,8.8379f,-2.1250f,8.8379f,-2.0208f,
	8.7891f,-2.0208f,8.7891f,-1.9167f,8.7402f,-1.9167f,8.7402f,-1.8125f,8.6914f,-1.8125f,8.6914f,-1.7083f,8.6426f,-1.7083f,8.6426f,-1.6042f,8.5938f,-1.6042f,8.5938f,-1.5000f,8.5449f,-1.5000f,8.5449f,-1.3958f,8.4961f,-1.3958f,8.4961f,-1.2917f,8.4473f,-1.2917f,
	8.4473f,-1.1875f,8.3984f,-1.1875f,8.3984f,-1.0833f,8.3496f,-1.0833f,8.3496f,-0.9792f,8.3008f,-0.9792f,8.3008f,-0.8750f,8.2520f,-0.8750f,8.2520f,-0.7708f,8.2031f,-0.7708f,8.2031f,-0.6667f,8.1543f,-0.6667f,8.1543f,-0.5625f,8.1055f,-0.5625f,8.1055f,-0.4583f,
	8.0566f,-0.4583f,8.0566f,-0.3542f,8.0078f,-0.3542f,8.0078f,-0.2500f,7.9590f,-0.2500f,7.9590f,-0.1458f,7.9102f,-0.1458f,7.9102f,-0.0417f,7.8613f,-0.0417f,7.8613f,0.0625f,7.8125f,0.0625f,7.8125f,0.1667f,7.7637f,0.1667f,7.7637f,0.2708f,7.7149f,0.2708f,
	7.7149f,0.3750f,7.6660f,0.3750f,7.6660f,0.4792f,7.6172f,0.4792f,7.6172f,0.5833f,7.5684f,0.5833f,7.5684f,0.6875f,7.5195f,0.6875f,7.5195f,0.7917f,7.4707f,0.7917f,7.4707f,0.8958f,7.4219f,0.8958f,7.4219f,1.0000f,7.3731f,1.0000f,9.7168f,-4.0000f,
	9.6680f,-4.0000f,9.6680f,-3.8958f,9.6192f,-3.8958f,9.6192f,-3.7917f,9.5703f,-3.7917f,9.5703f,-3.6875f,9.5215f,-3.6875f,9.5215f,-3.5833f,9.4727f,-3.5833f,9.4727f,-3.4792f,9.4238f,-3.4792f,9.4238f,-3.3750f,9.3750f,-3.3750f,9.3750f,-3.2708f,9.3262f,-3.2708f,
	9.3262f,-3.1667f,9.2774f,-3.1667f,9.2774f,-3.0625f,9.2285f,-3.0625f,9.2285f,-2.9583f,9.1797f,-2.9583f,9.1797f,-2.8542f,9.1309f,-2.8542f,9.1309f,-2.7500f,9.0820f,-2.7500f,9.0820f,-2.6458f,9.0332f,-2.6458f,9.0332f,-2.5417f,8.9844f,-2.5417f,8.9844f,-2.4375f,
	8.9356f,-2.4375f,8.9356f,-2.3333f,8.8867f,-2.3333f,8.8867f,-2.2292f,8.8379f,-2.2292f,8.8379f,-2.1250f,8.7891f,-2.1250f,8.7891f,-2.0208f,8.7402f,-2.0208f,8.7402f,-1.9167f,8.6914f,-1.9167f,8.6914f,-1.8125f,8.6426f,-1.8125f,8.6426f,-1.7083f,8.5938f,-1.7083f,
	8.5938f,-1.6042f,8.5449f,-1.6042f,8.5449f,-1.5000f,8.4961f,-1.5000f,8.4961f,-1.3958f,8.4473f,-1.3958f,8.4473f,-1.2917f,8.3984f,-1.2917f,8.3984f,-1.1875f,8.3496f,-1.1875f,8.3496f,-1.0833f,8.3008f,-1.0833f,8.3008f,-0.9792f,8.2520f,-0.9792f,8.2520f,-0.8750f,
	8.2031f,-0.8750f,8.2031f,-0.7708f,8.1543f,-0.7708f,8.1543f,-0.6667f,8.1055f,-0.6667f,8.1055f,-0.5625f,8.0566f,-0.5625f,8.0566f,-0.4583f,8.0078f,-0.4583f,8.0078f,-0.3542f,7.9590f,-0.3542f,7.9590f,-0.2500f,7.9102f,-0.2500f,7.9102f,-0.1458f,7.8613f,-0.1458f,
	7.8613f,-0.0417f,7.8125f,-0.0417f,7.8125f,0.0625f,7.7637f,0.0625f,7.7637f,0.1667f,7.7149f,0.1667f,7.7149f,0.2708f,7.6660f,0.2708f,7.6660f,0.3750f,7.6172f,0.3750f,7.6172f,0.4792f,7.5684f,0.4792f,7.5684f,0.5833f,7.5195f,0.5833f,7.5195f,0.6875f,
	7.4707f,0.6875f,7.4707f,0.7917f,7.4219f,0.7917f,7.4219f,0.8958f,7.3731f,0.8958f,7.3731f,1.0000f,7.3242f,1.0000f,9.6680f,-4.0000f,9.6192f,-4.0000f,9.6192f,-3.8958f,9.5703f,-3.8958f,9.5703f,-3.7917f,9.5215f,-3.7917f,9.5215f,-3.6875f,9.4727f,-3.6875f,
	9.4727f,-3.5833f,9.4238f,-3.5833f,9.4238f,-3.4792f,9.3750f,-3.4792f,9.3750f,-3.3750f,9.3262f,-3.3750f,9.3262f,-3.2708f,9.2774f,-3.2708f,9.2774f,-3.1667f,9.2285f,-3.1667f,9.2285f,-3.0625f,9.1797f,-3.0625f,9.1797f,-2.9583f,9.1309f,-2.9583f,9.1309f,-2.8542f,
	9.0820f,-2.8542f,9.0820f,-2.7500f,9.0332f,-2.7500f,9.0332f,-2.6458f,8.9844f,-2.6458f,8.9844f,-2.5417f,8.9356f,-2.5417f,8.9356f,-2.4375f,8.8867f,-2.4375f,8.8867f,-2.3333f,8.8379f,-2.3333f,8.8379f,-2.2292f,8.7891f,-2.2292f,8.7891f,-2.1250f,8.7402f,-2.1250f,
	8.7402f,-2.0208f,8.6914f,-2.0208f,8.6914f,-1.9167f,8.6426f,-1.9167f,8.6426f,-1.8125f,8.5938f,-1.8125f,8.5938f,-1.7083f,8.5449f,-1.7083f,8.5449f,-1.6042f,8.4961f,-1.6042f,8.4961f,-1.5000f,8.4473f,-1.5000f,8.4473f,-1.3958f,8.3984f,-1.3958f,8.3984f,-1.2917f,
	8.3496f,-1.2917f,8.3496f,-1.1875f,8.3008f,-1.1875f,8.3008f,-1.0833f,8.2520f,-1.0833f,8.2520f,-0.9792f,8.2031f,-0.9792f,8.2031f,-0.8750f,8.1543f,-0.8750f,8.1543f,-0.7708f,8.1055f,-0.7708f,8.1055f,-0.6667f,8.0566f,-0.6667f,8.0566f,-0.5625f,8.0078f,-0.5625f,
	8.0078f,-0.4583f,7.9590f,-0.4583f,7.9590f,-0.3542f,7.9102f,-0.3542f,7.9102f,-0.2500f,7.8613f,-0.2500f,7.8613f,-0.1458f,7.8125f,-0.1458f,7.8125f,-0.0417f,7.7637f,-0.0417f,7.7637f,0.0625f,7.7149f,0.0625f,7.7149f,0.1667f,7.6660f,0.1667f,7.6660f,0.2708f,
	7.6172f,0.2708f,7.6172f,0.3750f,7.5684f,0.3750f,7.5684f,0.4792f,7.5195f,0.4792f,7.5195f,0.5833f,7.4707f,0.5833f,7.4707f,0.6875f,7.4219f,0.6875f,7.4219f,0.7917f,7.3731f,0.7917f,7.3731f,0.8958f,7.3242f,0.8958f,7.3242f,1.0000f,7.2754f,1.0000f,
	9.6192f,-4.0000f,9.5703f,-4.0000f,9.5703f,-3.8958f,9.5215f,-3.8958f,9.5215f,-3.7917f,9.4727f,-3.7917f,9.4727f,-3.6875f,9.4238f,-3.6875f,9.4238f,-3.5833f,9.3750f,-3.5833f,9.3750f,-3.4792f,9.3262f,-3.4792f,9.3262f,-3.3750f,9.2774f,-3.3750f,9.2774f,-3.2708f,
	9.2285f,-3.2708f,9.2285f,-3.1667f,9.1797f,-3.1667f,9.1797f,-3.0625f,9.1309f,-3.0625f,9.1309f,-2.9583f,9.0820f,-2.9583f,9.0820f,-2.8542f,9.0332f,-2.8542f,9.0332f,-2.7500f,8.9844f,-2.7500f,8.9844f,-2.6458f,8.9356f,-2.6458f,8.9356f,-2.5417f,8.8867f,-2.5417f,
	8.8867f,-2.4375f,8.8379f,-2.4375f,8.8379f,-2.3333f,8.7891f,-2.3333f,8.7891f,-2.2292f,8.7402f,-2.2292f,8.7402f,-2.1250f,8.6914f,-2.1250f,8.6914f,-2.0208f,8.6426f,-2.0208f,8.6426f,-1.9167f,8.5938f,-1.9167f,8.5938f,-1.8125f,8.5449f,-1.8125f,8.5449f,-1.7083f,
	8.4961f,-1.7083f,8.4961f,-1.6042f,8.4473f,-1.6042f,8.4473f,-1.5000f,8.3984f,-1.5000f,8.3984f,-1.3958f,8.3496f,-1.3958f,8.3496f,-1.2917f,8.3008f,-1.2917f,8.3008f,-1.1875f,8.2520f,-1.1875f,8.2520f,-1.0833f,8.2031f,-1.0833f,8.2031f,-0.9792f,8.1543f,-0.9792f,
	8.1543f,-0.8750f,8.1055f,-0.8750f,8.1055f,-0.7708f,8.0566f,-0.7708f,8.0566f,-0.6667f,8.0078f,-0.6667f,8.0078f,-0.5625f,7.9590f,-0.5625f,7.9590f,-0.4583f,7.9102f,-0.4583f,7.9102f,-0.3542f,7.8613f,-0.3542f,7.8613f,-0.2500f,7.8125f,-0.2500f,7.8125f,-0.1458f,
	7.7637f,-0.1458f,7.7637f,-0.0417f,7.7149f,-0.0417f,7.7149f,0.0625f,7.6660f,0.0625f,7.6660f,0.1667f,7.6172f,0.1667f,7.6172f,0.2708f,7.5684f,0.2708f,7.5684f,0.3750f,7.5195f,0.3750f,7.5195f,0.4792f,7.4707f,0.4792f,7.4707f,0.5833f,7.4219f,0.5833f,
	7.4219f,0.6875f,7.3731f,0.6875f,7.3731f,0.7917f,7.3242f,0.7917f,7.3242f,0.8958f,7.2754f,0.8958f,7.2754f,1.0000f,7.2266f,1.0000f,9.5703f,-4.0000f,9.5215f,-4.0000f,9.5215f,-3.8958f,9.4727f,-3.8958f,9.4727f,-3.7917f,9.4238f,-3.7917f,9.4238f,-3.6875f,
	9.3750f,-3.6875f,9.3750f,-3.5833f,9.3262f,-3.5833f,9.3262f,-3.4792f,9.2774f,-3.4792f,9.2774f,-3.3750f,9.2285f,-3.3750f,9.2285f,-3.2708f,9.1797f,-3.2708f,9.1797f,-3.1667f,9.1309f,-3.1667f,9.1309f,-3.0625f,9.0820f,-3.0625f,9.0820f,-2.9583f,9.0332f,-2.9583f,
	9.0332f,-2.8542f,8.9844f,-2.8542f,8.9844f,-2.7500f,8.9356f,-2.7500f,8.9356f,-2.6458f,8.8867f,-2.6458f,8.8867f,-2.5417f,8.8379f,-2.5417f,8.8379f,-2.4375f,8.7891f,-2.4375f,8.7891f,-2.3333f,8.7402f,-2.3333f,8.7402f,-2.2292f,8.6914f,-2.2292f,8.6914f,-2.1250f,
	8.6426f,-2.1250f,8.6426f,-2.0208f,8.5938f,-2.0208f,8.5938f,-1.9167f,8.5449f,-1.9167f,8.5449f,-1.8125f,8.4961f,-1.8125f,8.4961f,-1.7083f,8.4473f,-1.7083f,8.4473f,-1.6042f,8.3984f,-1.6042f,8.3984f,-1.5000f,8.3496f,-1.5000f,8.3496f,-1.3958f,8.3008f,-1.3958f,
	8.3008f,-1.2917f,8.2520f,-1.2917f,8.2520f,-1.1875f,8.2031f,-1.1875f,8.2031f,-1.0833f,8.1543f,-1.0833f,8.1543f,-0.9792f,8.1055f,-0.9792f,8.1055f,-0.8750f,8.0566f,-0.8750f,8.0566f,-0.7708f,8.0078f,-0.7708f,8.0078f,-0.6667f,7.9590f,-0.6667f,7.9590f,-0.5625f,
	7.9102f,-0.5625f,7.9102f,-0.4583f,7.8613f,-0.4583f,7.8613f,-0.3542f,7.8125f,-0.3542f,7.8125f,-0.2500f,7.7637f,-0.2500f,7.7637f,-0.1458f,7.7149f,-0.1458f,7.7149f,-0.0417f,7.6660f,-0.0417f,7.6660f,0.0625f,7.6172f,0.0625f,7.6172f,0.1667f,7.5684f,0.1667f,
	7.5684f,0.2708f,7.5195f,0.2708f,7.5195f,0.3750f,7.4707f,0.3750f,7.4707f,0.4792f,7.4219f,0.4792f,7.4219f,0.5833f,7.3731f,0.5833f,7.3731f,0.6875f,7.3242f,0.6875f,7.3242f,0.7917f,7.2754f,0.7917f,7.2754f,0.8958f,7.2266f,0.8958f,7.2266f,1.0000f,
	7.1777f,1.0000f,9.5215f,-4.0000f,9.4727f,-4.0000f,9.4727f,-3.8958f,9.4238f,-3.8958f,9.4238f,-3.7917f,9.3750f,-3.7917f,9.3750f,-3.6875f,9.3262f,-3.6875f,9.3262f,-3.5833f,9.2774f,-3.5833f,9.2774f,-3.4792f,9.2285f,-3.4792f,9.2285f,-3.3750f,9.1797f,-3.3750f,
	9.1797f,-3.2708f,9.1309f,-3.2708f,9.1309f,-3.1667f,9.0820f,-3.1667f,9.0820f,-3.0625f,9.0332f,-3.0625f,9.0332f,-2.9583f,8.9844f,-2.9583f,8.9844f,-2.8542f,8.9356f,-2.8542f,8.9356f,-2.7500f,8.8867f,-2.7500f,8.8867f,-2.6458f,8.8379f,-2.6458f,8.8379f,-2.5417f,
	8.7891f,-2.5417f,8.7891f,-2.4375f,8.7402f,-2.4375f,8.7402f,-2.3333f,8.6914f,-2.3333f,8.6914f,-2.2292f,8.6426f,-2.2292f,8.6426f,-2.1250f,8.5938f,-2.1250f,8.5938f,-2.0208f,8.5449f,-2.0208f,8.5449f,-1.9167f,8.4961f,-1.9167f,8.4961f,-1.8125f,8.4473f,-1.8125f,
	8.4473f,-1.7083f,8.3984f,-1.7083f,8.3984f,-1.6042f,8.3496f,-1.6042f,8.3496f,-1.5000f,8.3008f,-1.5000f,8.3008f,-1.3958f,8.2520f,-1.3958f,8.2520f,-1.2917f,8.2031f,-1.2917f,8.2031f,-1.1875f,8.1543f,-1.1875f,8.1543f,-1.0833f,8.1055f,-1.0833f,8.1055f,-0.9792f,
	8.0566f,-0.9792f,8.0566f,-0.8750f,8.0078f,-0.8750f,8.0078f,-0.7708f,7.9590f,-0.7708f,7.9590f,-0.6667f,7.9102f,-0.6667f,7.9102f,-0.5625f,7.8613f,-0.5625f,7.8613f,-0.4583f,7.8125f,-0.4583f,7.8125f,-0.3542f,7.7637f,-0.3542f,7.7637f,-0.2500f,7.7149f,-0.2500f,
	7.7149f,-0.1458f,7.6660f,-0.1458f,7.6660f,-0.0417f,7.6172f,-0.0417f,7.6172f,0.0625f,7.5684f,0.0625f,7.5684f,0.1667f,7.5195f,0.1667f,7.5195f,0.2708f,7.4707f,0.2708f,7.4707f,0.3750f,7.4219f,0.3750f,7.4219f,0.4792f,7.3731f,0.4792f,7.3731f,0.5833f,
	7.3242f,0.5833f,7.3242f,0.6875f,7.2754f,0.6875f,7.2754f,0.7917f,7.2266f,0.7917f,7.2266f,0.8958f,7.1777f,0.8958f,7.1777f,1.0000f,7.1289f,1.0000f,9.4727f,-4.0000f,9.4238f,-4.0000f,9.4238f,-3.8958f,9.3750f,-3.8958f,9.3750f,-3.7917f,9.3262f,-3.7917f,
	9.3262f,-3.6875f,9.2774f,-3.6875f,9.2774f,-3.5833f,9.2285f,-3.5833f,9.2285f,-3.4792f,9.1797f,-3.4792f,9.1797f,-3.3750f,9.1309f,-3.3750f,9.1309f,-3.2708f,9.0820f,-3.2708f,9.0820f,-3.1667f,9.0332f,-3.1667f,9.0332f,-3.0625f,8.9844f,-3.0625f,8.9844f,-2.9583f,
	8.9356f,-2.9583f,8.9356f,-2.8542f,8.8867f,-2.8542f,8.8867f,-2.7500f,8.8379f,-2.7500f,8.8379f,-2.6458f,8.7891f,-2.6458f,8.7891f,-2.5417f,8.7402f,-2.5417f,8.7402f,-2.4375f,8.6914f,-2.4375f,8.6914f,-2.3333f,8.6426f,-2.3333f,8.6426f,-2.2292f,8.5938f,-2.2292f,
	8.5938f,-2.1250f,8.5449f,-2.1250f,8.5449f,-2.0208f,8.4961f,-2.0208f,8.4961f,-1.9167f,8.4473f,-1.9167f,8.4473f,-1.8125f,8.3984f,-1.8125f,8.3984f,-1.7083f,8.3496f,-1.7083f,8.3496f,-1.6042f,8.3008f,-1.6042f,8.3008f,-1.5000f,8.2520f,-1.5000f,8.2520f,-1.3958f,
	8.2031f,-1.3958f,8.2031f,-1.2917f,8.1543f,-1.2917f,8.1543f,-1.1875f,8.1055f,-1.1875f,8.1055f,-1.0833f,8.0566f,-1.0833f,8.0566f,-0.9792f,8.0078f,-0.9792f,8.0078f,-0.8750f,7.9590f,-0.8750f,7.9590f,-0.7708f,7.9102f,-0.7708f,7.9102f,-0.6667f,7.8613f,-0.6667f,
	7.8613f,-0.5625f,7.8125f,-0.5625f,7.8125f,-0.4583f,7.7637f,-0.4583f,7.7637f,-0.3542f,7.7149f,-0.3542f,7.7149f,-0.2500f,7.6660f,-0.2500f,7.6660f,-0.1458f,7.6172f,-0.1458f,7.6172f,-0.0417f,7.5684f,-0.0417f,7.5684f,0.0625f,7.5195f,0.0625f,7.5195f,0.1667f,
	7.4707f,0.1667f,7.4707f,0.2708f,7.4219f,0.2708f,7.4219f,0.3750f,7.3731f,0.3750f,7.3731f,0.4792f,7.3242f,0.4792f,7.3242f,0.5833f,7.2754f,0.5833f,7.2754f,0.6875f,7.2266f,0.6875f,7.2266f,0.7917f,7.1777f,0.7917f,7.1777f,0.8958f,7.1289f,0.8958f,
	7.1289f,1.0000f,7.0801f,1.0000f,9.4238f,-4.0000f,9.3750f,-4.0000f,9.3750f,-3.8958f,9.3262f,-3.8958f,9.3262f,-3.7917f,9.2774f,-3.7917f,9.2774f,-3.6875f,9.2285f,-3.6875f,9.2285f,-3.5833f,9.1797f,-3.5833f,9.1797f,-3.4792f,9.1309f,-3.4792f,9.1309f,-3.3750f,
	9.0820f,-3.3750f,9.0820f,-3.2708f,9.0332f,-3.2708f,9.0332f,-3.1667f,8.9844f,-3.1667f,8.9844f,-3.0625f,8.9356f,-3.0625f,8.9356f,-2.9583f,8.8867f,-2.9583f,8.8867f,-2.8542f,8.8379f,-2.8542f,8.8379f,-2.7500f,8.7891f,-2.7500f,8.7891f,-2.6458f,8.7402f,-2.6458f,
	8.7402f,-2.5417f,8.6914f,-2.5417f,8.6914f,-2.4375f,8.6426f,-2.4375f,8.6426f,-2.3333f,8.5938f,-2.3333f,8.5938f,-2.2292f,8.5449f,-2.2292f,8.5449f,-2.1250f,8.4961f,-2.1250f,8.4961f,-2.0208f,8.4473f,-2.0208f,8.4473f,-1.9167f,8.3984f,-1.9167f,8.3984f,-1.8125f,
	8.3496f,-1.8125f,8.3496f,-1.7083f,8.3008f,-1.7083f,8.3008f,-1.6042f,8.2520f,-1.6042f,8.2520f,-1.5000f,8.2031f,-1.5000f,8.2031f,-1.3958f,8.1543f,-1.3958f,8.1543f,-1.2917f,8.1055f,-1.2917f,8.1055f,-1.1875f,8.0566f,-1.1875f,8.0566f,-1.0833f,8.0078f,-1.0833f,
	8.0078f,-0.9792f,7.9590f,-0.9792f,7.9590f,-0.8750f,7.9102f,-0.8750f,7.9102f,-0.7708f,7.8613f,-0.7708f,7.8613f,-0.6667f,7.8125f,-0.6667f,7.8125f,-0.5625f,7.7637f,-0.5625f,7.7637f,-0.4583f,7.7149f,-0.4583f,7.7149f,-0.3542f,7.6660f,-0.3542f,7.6660f,-0.2500f,
	7.6172f,-0.2500f,7.6172f,-0.1458f,7.5684f,-0.1458f,7.5684f,-0.0417f,7.5195f,-0.0417f,7.5195f,0.0625f,7.4707f,0.0625f,7.4707f,0.1667f,7.4219f,0.1667f,7.4219f,0.2708f,7.3731f,0.2708f,7.3731f,0.3750f,7.3242f,0.3750f,7.3242f,0.4792f,7.2754f,0.4792f,
	7.2754f,0.5833f,7.2266f,0.5833f,7.2266f,0.6875f,7.1777f,0.6875f,7.1777f,0.7917f,7.1289f,0.7917f,7.1289f,0.8958f,7.0801f,0.8958f,7.0801f,1.0000f,7.0313f,1.0000f,9.3750f,-4.0000f,9.3262f,-4.0000f,9.3262f,-3.8958f,9.2774f,-3.8958f,9.2774f,-3.7917f,
	9.2285f,-3.7917f,9.2285f,-3.6875f,9.1797f,-3.6875f,9.1797f,-3.5833f,9.1309f,-3.5833f,9.1309f,-3.4792f,9.0820f,-3.4792f,9.0820f,-3.3750f,9.0332f,-3.3750f,9.0332f,-3.2708f,8.9844f,-3.2708f,8.9844f,-3.1667f,8.9356f,-3.1667f,8.9356f,-3.0625f,8.8867f,-3.0625f,
	8.8867f,-2.9583f,8.8379f,-2.9583f,8.8379f,-2.8542f,8.7891f,-2.8542f,8.7891f,-2.7500f,8.7402f,-2.7500f,8.7402f,-2.6458f,8.6914f,-2.6458f,8.6914f,-2.5417f,8.6426f,-2.5417f,8.6426f,-2.4375f,8.5938f,-2.4375f,8.5938f,-2.3333f,8.5449f,-2.3333f,8.5449f,-2.2292f,
	8.4961f,-2.2292f,8.4961f,-2.1250f,8.4473f,-2.1250f,8.4473f,-2.0208f,8.3984f,-2.0208f,8.3984f,-1.9167f,8.3496f,-1.9167f,8.3496f,-1.8125f,8.3008f,-1.8125f,8.3008f,-1.7083f,8.2520f,-1.7083f,8.2520f,-1.6042f,8.2031f,-1.6042f,8.2031f,-1.5000f,8.1543f,-1.5000f,
	8.1543f,-1.3958f,8.1055f,-1.3958f,8.1055f,-1.2917f,8.0566f,-1.2917f,8.0566f,-1.1875f,8.0078f,-1.1875f,8.0078f,-1.0833f,7.9590f,-1.0833f,7.9590f,-0.9792f,7.9102f,-0.9792f,7.9102f,-0.8750f,7.8613f,-0.8750f,7.8613f,-0.7708f,7.8125f,-0.7708f,7.8125f,-0.6667f,
	7.7637f,-0.6667f,7.7637f,-0.5625f,7.7149f,-0.5625f,7.7149f,-0.4583f,7.6660f,-0.4583f,7.6660f,-0.3542f,7.6172f,-0.3542f,7.6172f,-0.2500f,7.5684f,-0.2500f,7.5684f,-0.1458f,7.5195f,-0.1458f,7.5195f,-0.0417f,7.4707f,-0.0417f,7.4707f,0.0625f,7.4219f,0.0625f,
	7.4219f,0.1667f,7.3731f,0.1667f,7.3731f,0.2708f,7.3242f,0.2708f,7.3242f,0.3750f,7.2754f,0.3750f,7.2754f,0.4792f,7.2266f,0.4792f,7.2266f,0.5833f,7.1777f,0.5833f,7.1777f,0.6875f,7.1289f,0.6875f,7.1289f,0.7917f,7.0801f,0.7917f,7.0801f,0.8958f,
	7.0313f,0.8958f,7.0313f,1.0000f,6.9824f,1.0000f,9.3262f,-4.0000f,9.2774f,-4.0000f,9.2774f,-3.8958f,9.2285f,-3.8958f,9.2285f,-3.7917f,9.1797f,-3.7917f,9.1797f,-3.6875f,9.1309f,-3.6875f,9.1309f,-3.5833f,9.0820f,-3.5833f,9.0820f,-3.4792f,9.0332f,-3.4792f,
	9.0332f,-3.3750f,8.9844f,-3.3750f,8.9844f,-3.2708f,8.9356f,-3.2708f,8.9356f,-3.1667f,8.8867f,-3.1667f,8.8867f,-3.0625f,8.8379f,-3.0625f,8.8379f,-2.9583f,8.7891f,-2.9583f,8.7891f,-2.8542f,8.7402f,-2.8542f,8.7402f,-2.7500f,8.6914f,-2.7500f,8.6914f,-2.6458f,
	8.6426f,-2.6458f,8.6426f,-2.5417f,8.5938f,-2.5417f,8.5938f,-2.4375f,8.5449f,-2.4375f,8.5449f,-2.3333f,8.4961f,-2.3333f,8.4961f,-2.2292f,8.4473f,-2.2292f,8.4473f,-2.1250f,8.3984f,-2.1250f,8.3984f,-2.0208f,8.3496f,-2.0208f,8.3496f,-1.9167f,8.3008f,-1.9167f,
	8.3008f,-1.8125f,8.2520f,-1.8125f,8.2520f,-1.7083f,8.2031f,-1.7083f,8.2031f,-1.6042f,8.1543f,-1.6042f,8.1543f,-1.5000f,8.1055f,-1.5000f,8.1055f,-1.3958f,8.0566f,-1.3958f,8.0566f,-1.2917f,8.0078f,-1.2917f,8.0078f,-1.1875f,7.9590f,-1.1875f,7.9590f,-1.0833f,
	7.9102f,-1.0833f,7.9102f,-0.9792f,7.8613f,-0.9792f,7.8613f,-0.8750f,7.8125f,-0.8750f,7.8125f,-0.7708f,7.7637f,-0.7708f,7.7637f,-0.6667f,7.7149f,-0.6667f,7.7149f,-0.5625f,7.6660f,-0.5625f,7.6660f,-0.4583f,7.6172f,-0.4583f,7.6172f,-0.3542f,7.5684f,-0.3542f,
	7.5684f,-0.2500f,7.5195f,-0.2500f,7.5195f,-0.1458f,7.4707f,-0.1458f,7.4707f,-0.0417f,7.4219f,-0.0417f,7.4219f,0.0625f,7.3731f,0.0625f,7.3731f,0.1667f,7.3242f,0.1667f,7.3242f,0.2708f,7.2754f,0.2708f,7.2754f,0.3750f,7.2266f,0.3750f,7.2266f,0.4792f,
	7.1777f,0.4792f,7.1777f,0.5833f,7.1289f,0.5833f,7.1289f,0.6875f,7.0801f,0.6875f,7.0801f,0.7917f,7.0313f,0.7917f,7.0313f,0.8958f,6.9824f,0.8958f,6.9824f,1.0000f,6.9336f,1.0000f,9.2774f,-4.0000f,9.2285f,-4.0000f,9.2285f,-3.8958f,9.1797f,-3.8958f,
	9.1797f,-3.7917f,9.1309f,-3.7917f,9.1309f,-3.6875f,9.0820f,-3.6875f,9.0820f,-3.5833f,9.0332f,-3.5833f,9.0332f,-3.4792f,8.9844f,-3.4792f,8.9844f,-3.3750f,8.9356f,-3.3750f,8.9356f,-3.2708f,8.8867f,-3.2708f,8.8867f,-3.1667f,8.8379f,-3.1667f,8.8379f,-3.0625f,
	8.7891f,-3.0625f,8.7891f,-2.9583f,8.7402f,-2.9583f,8.7402f,-2.8542f,8.6914f,-2.8542f,8.6914f,-2.7500f,8.6426f,-2.7500f,8.6426f,-2.6458f,8.5938f,-2.6458f,8.5938f,-2.5417f,8.5449f,-2.5417f,8.5449f,-2.4375f,8.4961f,-2.4375f,8.4961f,-2.3333f,8.4473f,-2.3333f,
	8.4473f,-2.2292f,8.3984f,-2.2292f,8.3984f,-2.1250f,8.3496f,-2.1250f,8.3496f,-2.0208f,8.3008f,-2.0208f,8.3008f,-1.9167f,8.2520f,-1.9167f,8.2520f,-1.8125f,8.2031f,-1.8125f,8.2031f,-1.7083f,8.1543f,-1.7083f,8.1543f,-1.6042f,8.1055f,-1.6042f,8.1055f,-1.5000f,
	8.0566f,-1.5000f,8.0566f,-1.3958f,8.0078f,-1.3958f,8.0078f,-1.2917f,7.9590f,-1.2917f,7.9590f,-1.1875f,7.9102f,-1.1875f,7.9102f,-1.0833f,7.8613f,-1.0833f,7.8613f,-0.9792f,7.8125f,-0.9792f,7.8125f,-0.8750f,7.7637f,-0.8750f,7.7637f,-0.7708f,7.7149f,-0.7708f,
	7.7149f,-0.6667f,7.6660f,-0.6667f,7.6660f,-0.5625f,7.6172f,-0.5625f,7.6172f,-0.4583f,7.5684f,-0.4583f,7.5684f,-0.3542f,7.5195f,-0.3542f,7.5195f,-0.2500f,7.4707f,-0.2500f,7.4707f,-0.1458f,7.4219f,-0.1458f,7.4219f,-0.0417f,7.3731f,-0.0417f,7.3731f,0.0625f,
	7.3242f,0.0625f,7.3242f,0.1667f,7.2754f,0.1667f,7.2754f,0.2708f,7.2266f,0.2708f,7.2266f,0.3750f,7.1777f,0.3750f,7.1777f,0.4792f,7.1289f,0.4792f,7.1289f,0.5833f,7.0801f,0.5833f,7.0801f,0.6875f,7.0313f,0.6875f,7.0313f,0.7917f,6.9824f,0.7917f,
	6.9824f,0.8958f,6.9336f,0.8958f,6.9336f,1.0000f,6.8848f,1.0000f,9.2285f,-4.0000f,9.1797f,-4.0000f,9.1797f,-3.8958f,9.1309f,-3.8958f,9.1309f,-3.7917f,9.0820f,-3.7917f,9.0820f,-3.6875f,9.0332f,-3.6875f,9.0332f,-3.5833f,8.9844f,-3.5833f,8.9844f,-3.4792f,
	8.9356f,-3.4792f,8.9356f,-3.3750f,8.8867f,-3.3750f,8.8867f,-3.2708f,8.8379f,-3.2708f,8.8379f,-3.1667f,8.7891f,-3.1667f,8.7891f,-3.0625f,8.7402f,-3.0625f,8.7402f,-2.9583f,8.6914f,-2.9583f,8.6914f,-2.8542f,8.6426f,-2.8542f,8.6426f,-2.7500f,8.5938f,-2.7500f,
	8.5938f,-2.6458f,8.5449f,-2.6458f,8.5449f,-2.5417f,8.4961f,-2.5417f,8.4961f,-2.4375f,8.4473f,-2.4375f,8.4473f,-2.3333f,8.3984f,-2.3333f,8.3984f,-2.2292f,8.3496f,-2.2292f,8.3496f,-2.1250f,8.3008f,-2.1250f,8.3008f,-2.0208f,8.2520f,-2.0208f,8.2520f,-1.9167f,
	8.2031f,-1.9167f,8.2031f,-1.8125f,8.1543f,-1.8125f,8.1543f,-1.7083f,8.1055f,-1.7083f,8.1055f,-1.6042f,8.0566f,-1.6042f,8.0566f,-1.5000f,8.0078f,-1.5000f,8.0078f,-1.3958f,7.9590f,-1.3958f,7.9590f,-1.2917f,7.9102f,-1.2917f,7.9102f,-1.1875f,7.8613f,-1.1875f,
	7.8613f,-1.0833f,7.8125f,-1.0833f,7.8125f,-0.9792f,7.7637f,-0.9792f,7.7637f,-0.8750f,7.7149f,-0.8750f,7.7149f,-0.7708f,7.6660f,-0.7708f,7.6660f,-0.6667f,7.6172f,-0.6667f,7.6172f,-0.5625f,7.5684f,-0.5625f,7.5684f,-0.4583f,7.5195f,-0.4583f,7.5195f,-0.3542f,
	7.4707f,-0.3542f,7.4707f,-0.2500f,7.4219f,-0.2500f,7.4219f,-0.1458f,7.3731f,-0.1458f,7.3731f,-0.0417f,7.3242f,-0.0417f,7.3242f,0.0625f,7.2754f,0.0625f,7.2754f,0.1667f,7.2266f,0.1667f,7.2266f,0.2708f,7.1777f,0.2708f,7.1777f,0.3750f,7.1289f,0.3750f,
	7.1289f,0.4792f,7.0801f,0.4792f,7.0801f,0.5833f,7.0313f,0.5833f,7.0313f,0.6875f,6.9824f,0.6875f,6.9824f,0.7917f,6.9336f,0.7917f,6.9336f,0.8958f,6.8848f,0.8958f,6.8848f,1.0000f,6.8359f,1.0000f,9.1797f,-4.0000f,9.1309f,-4.0000f,9.1309f,-3.8958f,
	9.0820f,-3.8958f,9.0820f,-3.7917f,9.0332f,-3.7917f,9.0332f,-3.6875f,8.9844f,-3.6875f,8.9844f,-3.5833f,8.9356f,-3.5833f,8.9356f,-3.4792f,8.8867f,-3.4792f,8.8867f,-3.3750f,8.8379f,-3.3750f,8.8379f,-3.2708f,8.7891f,-3.2708f,8.7891f,-3.1667f,8.7402f,-3.1667f,
	8.7402f,-3.0625f,8.6914f,-3.0625f,8.6914f,-2.9583f,8.6426f,-2.9583f,8.6426f,-2.8542f,8.5938f,-2.8542f,8.5938f,-2.7500f,8.5449f,-2.7500f,8.5449f,-2.6458f,8.4961f,-2.6458f,8.4961f,-2.5417f,8.4473f,-2.5417f,8.4473f,-2.4375f,8.3984f,-2.4375f,8.3984f,-2.3333f,
	8.3496f,-2.3333f,8.3496f,-2.2292f,8.3008f,-2.2292f,8.3008f,-2.1250f,8.2520f,-2.1250f,8.2520f,-2.0208f,8.2031f,-2.0208f,8.2031f,-1.9167f,8.1543f,-1.9167f,8.1543f,-1.8125f,8.1055f,-1.8125f,8.1055f,-1.7083f,8.0566f,-1.7083f,8.0566f,-1.6042f,8.0078f,-1.6042f,
	8.0078f,-1.5000f,7.9590f,-1.5000f,7.9590f,-1.3958f,7.9102f,-1.3958f,7.9102f,-1.2917f,7.8613f,-1.2917f,7.8613f,-1.1875f,7.8125f,-1.1875f,7.8125f,-1.0833f,7.7637f,-1.0833f,7.7637f,-0.9792f,7.7149f,-0.9792f,7.7149f,-0.8750f,7.6660f,-0.8750f,7.6660f,-0.7708f,
	7.6172f,-0.7708f,7.6172f,-0.6667f,7.5684f,-0.6667f,7.5684f,-0.5625f,7.5195f,-0.5625f,7.5195f,-0.4583f,7.4707f,-0.4583f,7.4707f,-0.3542f,7.4219f,-0.3542f,7.4219f,-0.2500f,7.3731f,-0.2500f,7.3731f,-0.1458f,7.3242f,-0.1458f,7.3242f,-0.0417f,7.2754f,-0.0417f,
	7.2754f,0.0625f,7.2266f,0.0625f,7.2266f,0.1667f,7.1777f,0.1667f,7.1777f,0.2708f,7.1289f,0.2708f,7.1289f,0.3750f,7.0801f,0.3750f,7.0801f,0.4792f,7.0313f,0.4792f,7.0313f,0.5833f,6.9824f,0.5833f,6.9824f,0.6875f,6.9336f,0.6875f,6.9336f,0.7917f,
	6.8848f,0.7917f,6.8848f,0.8958f,6.8359f,0.8958f,6.8359f,1.0000f,6.7871f,1.0000f,9.1309f,-4.0000f,9.0820f,-4.0000f,9.0820f,-3.8958f,9.0332f,-3.8958f,9.0332f,-3.7917f,8.9844f,-3.7917f,8.9844f,-3.6875f,8.9356f,-3.6875f,8.9356f,-3.5833f,8.8867f,-3.5833f,
	8.8867f,-3.4792f,8.8379f,-3.4792f,8.8379f,-3.3750f,8.7891f,-3.3750f,8.7891f,-3.2708f,8.7402f,-3.2708f,8.7402f,-3.1667f,8.6914f,-3.1667f,8.6914f,-3.0625f,8.6426f,-3.0625f,8.6426f,-2.9583f,8.5938f,-2.9583f,8.5938f,-2.8542f,8.5449f,-2.8542f,8.5449f,-2.7500f,
	8.4961f,-2.7500f,8.4961f,-2.6458f,8.4473f,-2.6458f,8.4473f,-2.5417f,8.3984f,-2.5417f,8.3984f,-2.4375f,8.3496f,-2.4375f,8.3496f,-2.3333f,8.3008f,-2.3333f,8.3008f,-2.2292f,8.2520f,-2.2292f,8.2520f,-2.1250f,8.2031f,-2.1250f,8.2031f,-2.0208f,8.1543f,-2.0208f,
	8.1543f,-1.9167f,8.1055f,-1.9167f,8.1055f,-1.8125f,8.0566f,-1.8125f,8.0566f,-1.7083f,8.0078f,-1.7083f,8.0078f,-1.6042f,7.9590f,-1.6042f,7.9590f,-1.5000f,7.9102f,-1.5000f,7.9102f,-1.3958f,7.8613f,-1.3958f,7.8613f,-1.2917f,7.8125f,-1.2917f,7.8125f,-1.1875f,
	7.7637f,-1.1875f,7.7637f,-1.0833f,7.7149f,-1.0833f,7.7149f,-0.9792f,7.6660f,-0.9792f,7.6660f,-0.8750f,7.6172f,-0.8750f,7.6172f,-0.7708f,7.5684f,-0.7708f,7.5684f,-0.6667f,7.5195f,-0.6667f,7.5195f,-0.5625f,7.4707f,-0.5625f,7.4707f,-0.4583f,7.4219f,-0.4583f,
	7.4219f,-0.3542f,7.3731f,-0.3542f,7.3731f,-0.2500f,7.3242f,-0.2500f,7.3242f,-0.1458f,7.2754f,-0.1458f,7.2754f,-0.0417f,7.2266f,-0.0417f,7.2266f,0.0625f,7.1777f,0.0625f,7.1777f,0.1667f,7.1289f,0.1667f,7.1289f,0.2708f,7.0801f,0.2708f,7.0801f,0.3750f,
	7.0313f,0.3750f,7.0313f,0.4792f,6.9824f,0.4792f,6.9824f,0.5833f,6.9336f,0.5833f,6.9336f,0.6875f,6.8848f,0.6875f,6.8848f,0.7917f,6.8359f,0.7917f,6.8359f,0.8958f,6.7871f,0.8958f,6.7871f,1.0000f,6.7383f,1.0000f,9.0820f,-4.0000f,9.0332f,-4.0000f,
	9.0332f,-3.8958f,8.9844f,-3.8958f,8.9844f,-3.7917f,8.9356f,-3.7917f,8.9356f,-3.6875f,8.8867f,-3.6875f,8.8867f,-3.5833f,8.8379f,-3.5833f,8.8379f,-3.4792f,8.7891f,-3.4792f,8.7891f,-3.3750f,8.7402f,-3.3750f,8.7402f,-3.2708f,8.6914f,-3.2708f,8.6914f,-3.1667f,
	8.6426f,-3.1667f,8.6426f,-3.0625f,8.5938f,-3.0625f,8.5938f,-2.9583f,8.5449f,-2.9583f,8.5449f,-2.8542f,8.4961f,-2.8542f,8.4961f,-2.7500f,8.4473f,-2.7500f,8.4473f,-2.6458f,8.3984f,-2.6458f,8.3984f,-2.5417f,8.3496f,-2.5417f,8.3496f,-2.4375f,8.3008f,-2.4375f,
	8.3008f,-2.3333f,8.2520f,-2.3333f,8.2520f,-2.2292f,8.2031f,-2.2292f,8.2031f,-2.1250f,8.1543f,-2.1250f,8.1543f,-2.0208f,8.1055f,-2.0208f,8.1055f,-1.9167f,8.0566f,-1.9167f,8.0566f,-1.8125f,8.0078f,-1.8125f,8.0078f,-1.7083f,7.9590f,-1.7083f,7.9590f,-1.6042f,
	7.9102f,-1.6042f,7.9102f,-1.5000f,7.8613f,-1.5000f,7.8613f,-1.3958f,7.8125f,-1.3958f,7.8125f,-1.2917f,7.7637f,-1.2917f,7.7637f,-1.1875f,7.7149f,-1.1875f,7.7149f,-1.0833f,7.6660f,-1.0833f,7.6660f,-0.9792f,7.6172f,-0.9792f,7.6172f,-0.8750f,7.5684f,-0.8750f,
	7.5684f,-0.7708f,7.5195f,-0.7708f,7.5195f,-0.6667f,7.4707f,-0.6667f,7.4707f,-0.5625f,7.4219f,-0.5625f,7.4219f,-0.4583f,7.3731f,-0.4583f,7.3731f,-0.3542f,7.3242f,-0.3542f,7.3242f,-0.2500f,7.2754f,-0.2500f,7.2754f,-0.1458f,7.2266f,-0.1458f,7.2266f,-0.0417f,
	7.1777f,-0.0417f,7.1777f,0.0625f,7.1289f,0.0625f,7.1289f,0.1667f,7.0801f,0.1667f,7.0801f,0.2708f,7.0313f,0.2708f,7.0313f,0.3750f,6.9824f,0.3750f,6.9824f,0.4792f,6.9336f,0.4792f,6.9336f,0.5833f,6.8848f,0.5833f,6.8848f,0.6875f,6.8359f,0.6875f,
	6.8359f,0.7917f,6.7871f,0.7917f,6.7871f,0.8958f,6.7383f,0.8958f,6.7383f,1.0000f,6.6895f,1.0000f,9.0332f,-4.0000f,8.9844f,-4.0000f,8.9844f,-3.8958f,8.9356f,-3.8958f,8.9356f,-3.7917f,8.8867f,-3.7917f,8.8867f,-3.6875f,8.8379f,-3.6875f,8.8379f,-3.5833f,
	8.7891f,-3.5833f,8.7891f,-3.4792f,8.7402f,-3.4792f,8.7402f,-3.3750f,8.6914f,-3.3750f,8.6914f,-3.2708f,8.6426f,-3.2708f,8.6426f,-3.1667f,8.5938f,-3.1667f,8.5938f,-3.0625f,8.5449f,-3.0625f,8.5449f,-2.9583f,8.4961f,-2.9583f,8.4961f,-2.8542f,8.4473f,-2.8542f,
	8.4473f,-2.7500f,8.3984f,-2.7500f,8.3984f,-2.6458f,8.3496f,-2.6458f,8.3496f,-2.5417f,8.3008f,-2.5417f,8.3008f,-2.4375f,8.2520f,-2.4375f,8.2520f,-2.3333f,8.2031f,-2.3333f,8.2031f,-2.2292f,8.1543f,-2.2292f,8.1543f,-2.1250f,8.1055f,-2.1250f,8.1055f,-2.0208f,
	8.0566f,-2.0208f,8.0566f,-1.9167f,8.0078f,-1.9167f,8.0078f,-1.8125f,7.9590f,-1.8125f,7.9590f,-1.7083f,7.9102f,-1.7083f,7.9102f,-1.6042f,7.8613f,-1.6042f,7.8613f,-1.5000f,7.8125f,-1.5000f,7.8125f,-1.3958f,7.7637f,-1.3958f,7.7637f,-1.2917f,7.7149f,-1.2917f,
	7.7149f,-1.1875f,7.6660f,-1.1875f,7.6660f,-1.0833f,7.6172f,-1.0833f,7.6172f,-0.9792f,7.5684f,-0.9792f,7.5684f,-0.8750f,7.5195f,-0.8750f,7.5195f,-0.7708f,7.4707f,-0.7708f,7.4707f,-0.6667f,7.4219f,-0.6667f,7.4219f,-0.5625f,7.3731f,-0.5625f,7.3731f,-0.4583f,
	7.3242f,-0.4583f,7.3242f,-0.3542f,7.2754f,-0.3542f,7.2754f,-0.2500f,7.2266f,-0.2500f,7.2266f,-0.1458f,7.1777f,-0.1458f,7.1777f,-0.0417f,7.1289f,-0.0417f,7.1289f,0.0625f,7.0801f,0.0625f,7.0801f,0.1667f,7.0313f,0.1667f,7.0313f,0.2708f,6.9824f,0.2708f,
	6.9824f,0.3750f,6.9336f,0.3750f,6.9336f,0.4792f,6.8848f,0.4792f,6.8848f,0.5833f,6.8359f,0.5833f,6.8359f,0.6875f,6.7871f,0.6875f,6.7871f,0.7917f,6.7383f,0.7917f,6.7383f,0.8958f,6.6895f,0.8958f,6.6895f,1.0000f,6.6406f,1.0000f,8.9844f,-4.0000f,
	8.9356f,-4.0000f,8.9356f,-3.8958f,8.8867f,-3.8958f,8.8867f,-3.7917f,8.8379f,-3.7917f,8.8379f,-3.6875f,8.7891f,-3.6875f,8.7891f,-3.5833f,8.7402f,-3.5833f,8.7402f,-3.4792f,8.6914f,-3.4792f,8.6914f,-3.3750f,8.6426f,-3.3750f,8.6426f,-3.2708f,8.5938f,-3.2708f,
	8.5938f,-3.1667f,8.5449f,-3.1667f,8.5449f,-3.0625f,8.4961f,-3.0625f,8.4961f,-2.9583f,8.4473f,-2.9583f,8.4473f,-2.8542f,8.3984f,-2.8542f,8.3984f,-2.7500f,8.3496f,-2.7500f,8.3496f,-2.6458f,8.3008f,-2.6458f,8.3008f,-2.5417f,8.2520f,-2.5417f,8.2520f,-2.4375f,
	8.2031f,-2.4375f,8.2031f,-2.3333f,8.1543f,-2.3333f,8.1543f,-2.2292f,8.1055f,-2.2292f,8.1055f,-2.1250f,8.0566f,-2.1250f,8.0566f,-2.0208f,8.0078f,-2.0208f,8.0078f,-1.9167f,7.9590f,-1.9167f,7.9590f,-1.8125f,7.9102f,-1.8125f,7.9102f,-1.7083f,7.8613f,-1.7083f,
	7.8613f,-1.6042f,7.8125f,-1.6042f,7.8125f,-1.5000f,7.7637f,-1.5000f,7.7637f,-1.3958f,7.7149f,-1.3958f,7.7149f,-1.2917f,7.6660f,-1.2917f,7.6660f,-1.1875f,7.6172f,-1.1875f,7.6172f,-1.0833f,7.5684f,-1.0833f,7.5684f,-0.9792f,7.5195f,-0.9792f,7.5195f,-0.8750f,
	7.4707f,-0.8750f,7.4707f,-0.7708f,7.4219f,-0.7708f,7.4219f,-0.6667f,7.3731f,-0.6667f,7.3731f,-0.5625f,7.3242f,-0.5625f,7.3242f,-0.4583f,7.2754f,-0.4583f,7.2754f,-0.3542f,7.2266f,-0.3542f,7.2266f,-0.2500f,7.1777f,-0.2500f,7.1777f,-0.1458f,7.1289f,-0.1458f,
	7.1289f,-0.0417f,7.0801f,-0.0417f,7.0801f,0.0625f,7.0313f,0.0625f,7.0313f,0.1667f,6.9824f,0.1667f,6.9824f,0.2708f,6.9336f,0.2708f,6.9336f,0.3750f,6.8848f,0.3750f,6.8848f,0.4792f,6.8359f,0.4792f,6.8359f,0.5833f,6.7871f,0.5833f,6.7871f,0.6875f,
	6.7383f,0.6875f,6.7383f,0.7917f,6.6895f,0.7917f,6.6895f,0.8958f,6.6406f,0.8958f,6.6406f,1.0000f,6.5918f,1.0000f,8.9356f,-4.0000f,8.8867f,-4.0000f,8.8867f,-3.8958f,8.8379f,-3.8958f,8.8379f,-3.7917f,8.7891f,-3.7917f,8.7891f,-3.6875f,8.7402f,-3.6875f,
	8.7402f,-3.5833f,8.6914f,-3.5833f,8.6914f,-3.4792f,8.6426f,-3.4792f,8.6426f,-3.3750f,8.5938f,-3.3750f,8.5938f,-3.2708f,8.5449f,-3.2708f,8.5449f,-3.1667f,8.4961f,-3.1667f,8.4961f,-3.0625f,8.4473f,-3.0625f,8.4473f,-2.9583f,8.3984f,-2.9583f,8.3984f,-2.8542f,
	8.3496f,-2.8542f,8.3496f,-2.7500f,8.3008f,-2.7500f,8.3008f,-2.6458f,8.2520f,-2.6458f,8.2520f,-2.5417f,8.2031f,-2.5417f,8.2031f,-2.4375f,8.1543f,-2.4375f,8.1543f,-2.3333f,8.1055f,-2.3333f,8.1055f,-2.2292f,8.0566f,-2.2292f,8.0566f,-2.1250f,8.0078f,-2.1250f,
	8.0078f,-2.0208f,7.9590f,-2.0208f,7.9590f,-1.9167f,7.9102f,-1.9167f,7.9102f,-1.8125f,7.8613f,-1.8125f,7.8613f,-1.7083f,7.8125f,-1.7083f,7.8125f,-1.6042f,7.7637f,-1.6042f,7.7637f,-1.5000f,7.7149f,-1.5000f,7.7149f,-1.3958f,7.6660f,-1.3958f,7.6660f,-1.2917f,
	7.6172f,-1.2917f,7.6172f,-1.1875f,7.5684f,-1.1875f,7.5684f,-1.0833f,7.5195f,-1.0833f,7.5195f,-0.9792f,7.4707f,-0.9792f,7.4707f,-0.8750f,7.4219f,-0.8750f,7.4219f,-0.7708f,7.3731f,-0.7708f,7.3731f,-0.6667f,7.3242f,-0.6667f,7.3242f,-0.5625f,7.2754f,-0.5625f,
	7.2754f,-0.4583f,7.2266f,-0.4583f,7.2266f,-0.3542f,7.1777f,-0.3542f,7.1777f,-0.2500f,7.1289f,-0.2500f,7.1289f,-0.1458f,7.0801f,-0.1458f,7.0801f,-0.0417f,7.0313f,-0.0417f,7.0313f,0.0625f,6.9824f,0.0625f,6.9824f,0.1667f,6.9336f,0.1667f,6.9336f,0.2708f,
	6.8848f,0.2708f,6.8848f,0.3750f,6.8359f,0.3750f,6.8359f,0.4792f,6.7871f,0.4792f,6.7871f,0.5833f,6.7383f,0.5833f,6.7383f,0.6875f,6.6895f,0.6875f,6.6895f,0.7917f,6.6406f,0.7917f,6.6406f,0.8958f,6.5918f,0.8958f,6.5918f,1.0000f,6.5430f,1.0000f,
	8.8867f,-4.0000f,8.8379f,-4.0000f,8.8379f,-3.8958f,8.7891f,-3.8958f,8.7891f,-3.7917f,8.7402f,-3.7917f,8.7402f,-3.6875f,8.6914f,-3.6875f,8.6914f,-3.5833f,8.6426f,-3.5833f,8.6426f,-3.4792f,8.5938f,-3.4792f,8.5938f,-3.3750f,8.5449f,-3.3750f,8.5449f,-3.2708f,
	8.4961f,-3.2708f,8.4961f,-3.1667f,8.4473f,-3.1667f,8.4473f,-3.0625f,8.3984f,-3.0625f,8.3984f,-2.9583f,8.3496f,-2.9583f,8.3496f,-2.8542f,8.3008f,-2.8542f,8.3008f,-2.7500f,8.2520f,-2.7500f,8.2520f,-2.6458f,8.2031f,-2.6458f,8.2031f,-2.5417f,8.1543f,-2.5417f,
	8.1543f,-2.4375f,8.1055f,-2.4375f,8.1055f,-2.3333f,8.0566f,-2.3333f,8.0566f,-2.2292f,8.0078f,-2.2292f,8.0078f,-2.1250f,7.9590f,-2.1250f,7.9590f,-2.0208f,7.9102f,-2.0208f,7.9102f,-1.9167f,7.8613f,-1.9167f,7.8613f,-1.8125f,7.8125f,-1.8125f,7.8125f,-1.7083f,
	7.7637f,-1.7083f,7.7637f,-1.6042f,7.7149f,-1.6042f,7.7149f,-1.5000f,7.6660f,-1.5000f,7.6660f,-1.3958f,7.6172f,-1.3958f,7.6172f,-1.2917f,7.5684f,-1.2917f,7.5684f,-1.1875f,7.5195f,-1.1875f,7.5195f,-1.0833f,7.4707f,-1.0833f,7.4707f,-0.9792f,7.4219f,-0.9792f,
	7.4219f,-0.8750f,7.3731f,-0.8750f,7.3731f,-0.7708f,7.3242f,-0.7708f,7.3242f,-0.6667f,7.2754f,-0.6667f,7.2754f,-0.5625f,7.2266f,-0.5625f,7.2266f,-0.4583f,7.1777f,-0.4583f,7.1777f,-0.3542f,7.1289f,-0.3542f,7.1289f,-0.2500f,7.0801f,-0.2500f,7.0801f,-0.1458f,
	7.0313f,-0.1458f,7.0313f,-0.0417f,6.9824f,-0.0417f,6.9824f,0.0625f,6.9336f,0.0625f,6.9336f,0.1667f,6.8848f,0.1667f,6.8848f,0.2708f,6.8359f,0.2708f,6.8359f,0.3750f,6.7871f,0.3750f,6.7871f,0.4792f,6.7383f,0.4792f,6.7383f,0.5833f,6.6895f,0.5833f,
	6.6895f,0.6875f,6.6406f,0.6875f,6.6406f,0.7917f,6.5918f,0.7917f,6.5918f,0.8958f,6.5430f,0.8958f,6.5430f,1.0000f,6.4941f,1.0000f,8.8379f,-4.0000f,8.7891f,-4.0000f,8.7891f,-3.8958f,8.7402f,-3.8958f,8.7402f,-3.7917f,8.6914f,-3.7917f,8.6914f,-3.6875f,
	8.6426f,-3.6875f,8.6426f,-3.5833f,8.5938f,-3.5833f,8.5938f,-3.4792f,8.5449f,-3.4792f,8.5449f,-3.3750f,8.4961f,-3.3750f,8.4961f,-3.2708f,8.4473f,-3.2708f,8.4473f,-3.1667f,8.3984f,-3.1667f,8.3984f,-3.0625f,8.3496f,-3.0625f,8.3496f,-2.9583f,8.3008f,-2.9583f,
	8.3008f,-2.8542f,8.2520f,-2.8542f,8.2520f,-2.7500f,8.2031f,-2.7500f,8.2031f,-2.6458f,8.1543f,-2.6458f,8.1543f,-2.5417f,8.1055f,-2.5417f,8.1055f,-2.4375f,8.0566f,-2.4375f,8.0566f,-2.3333f,8.0078f,-2.3333f,8.0078f,-2.2292f,7.9590f,-2.2292f,7.9590f,-2.1250f,
	7.9102f,-2.1250f,7.9102f,-2.0208f,7.8613f,-2.0208f,7.8613f,-1.9167f,7.8125f,-1.9167f,7.8125f,-1.8125f,7.7637f,-1.8125f,7.7637f,-1.7083f,7.7149f,-1.7083f,7.7149f,-1.6042f,7.6660f,-1.6042f,7.6660f,-1.5000f,7.6172f,-1.5000f,7.6172f,-1.3958f,7.5684f,-1.3958f,
	7.5684f,-1.2917f,7.5195f,-1.2917f,7.5195f,-1.1875f,7.4707f,-1.1875f,7.4707f,-1.0833f,7.4219f,-1.0833f,7.4219f,-0.9792f,7.3731f,-0.9792f,7.3731f,-0.8750f,7.3242f,-0.8750f,7.3242f,-0.7708f,7.2754f,-0.7708f,7.2754f,-0.6667f,7.2266f,-0.6667f,7.2266f,-0.5625f,
	7.1777f,-0.5625f,7.1777f,-0.4583f,7.1289f,-0.4583f,7.1289f,-0.3542f,7.0801f,-0.3542f,7.0801f,-0.2500f,7.0313f,-0.2500f,7.0313f,-0.1458f,6.9824f,-0.1458f,6.9824f,-0.0417f,6.9336f,-0.0417f,6.9336f,0.0625f,6.8848f,0.0625f,6.8848f,0.1667f,6.8359f,0.1667f,
	6.8359f,0.2708f,6.7871f,0.2708f,6.7871f,0.3750f,6.7383f,0.3750f,6.7383f,0.4792f,6.6895f,0.4792f,6.6895f,0.5833f,6.6406f,0.5833f,6.6406f,0.6875f,6.5918f,0.6875f,6.5918f,0.7917f,6.5430f,0.7917f,6.5430f,0.8958f,6.4941f,0.8958f,6.4941f,1.0000f,
	6.4453f,1.0000f,8.7891f,-4.0000f,8.7402f,-4.0000f,8.7402f,-3.8958f,8.6914f,-3.8958f,8.6914f,-3.7917f,8.6426f,-3.7917f,8.6426f,-3.6875f,8.5938f,-3.6875f,8.5938f,-3.5833f,8.5449f,-3.5833f,8.5449f,-3.4792f,8.4961f,-3.4792f,8.4961f,-3.3750f,8.4473f,-3.3750f,
	8.4473f,-3.2708f,8.3984f,-3.2708f,8.3984f,-3.1667f,8.3496f,-3.1667f,8.3496f,-3.0625f,8.3008f,-3.0625f,8.3008f,-2.9583f,8.2520f,-2.9583f,8.2520f,-2.8542f,8.2031f,-2.8542f,8.2031f,-2.7500f,8.1543f,-2.7500f,8.1543f,-2.6458f,8.1055f,-2.6458f,8.1055f,-2.5417f,
	8.0566f,-2.5417f,8.0566f,-2.4375f,8.0078f,-2.4375f,8.0078f,-2.3333f,7.9590f,-2.3333f,7.9590f,-2.2292f,7.9102f,-2.2292f,7.9102f,-2.1250f,7.8613f,-2.1250f,7.8613f,-2.0208f,7.8125f,-2.0208f,7.8125f,-1.9167f,7.7637f,-1.9167f,7.7637f,-1.8125f,7.7149f,-1.8125f,
	7.7149f,-1.7083f,7.6660f,-1.7083f,7.6660f,-1.6042f,7.6172f,-1.6042f,7.6172f,-1.5000f,7.5684f,-1.5000f,7.5684f,-1.3958f,7.5195f,-1.3958f,7.5195f,-1.2917f,7.4707f,-1.2917f,7.4707f,-1.1875f,7.4219f,-1.1875f,7.4219f,-1.0833f,7.3731f,-1.0833f,7.3731f,-0.9792f,
	7.3242f,-0.9792f,7.3242f,-0.8750f,7.2754f,-0.8750f,7.2754f,-0.7708f,7.2266f,-0.7708f,7.2266f,-0.6667f,7.1777f,-0.6667f,7.1777f,-0.5625f,7.1289f,-0.5625f,7.1289f,-0.4583f,7.0801f,-0.4583f,7.0801f,-0.3542f,7.0313f,-0.3542f,7.0313f,-0.2500f,6.9824f,-0.2500f,
	6.9824f,-0.1458f,6.9336f,-0.1458f,6.9336f,-0.0417f,6.8848f,-0.0417f,6.8848f,0.0625f,6.8359f,0.0625f,6.8359f,0.1667f,6.7871f,0.1667f,6.7871f,0.2708f,6.7383f,0.2708f,6.7383f,0.3750f,6.6895f,0.3750f,6.6895f,0.4792f,6.6406f,0.4792f,6.6406f,0.5833f,
	6.5918f,0.5833f,6.5918f,0.6875f,6.5430f,0.6875f,6.5430f,0.7917f,6.4941f,0.7917f,6.4941f,0.8958f,6.4453f,0.8958f,6.4453f,1.0000f,6.3965f,1.0000f,8.7402f,-4.0000f,8.6914f,-4.0000f,8.6914f,-3.8958f,8.6426f,-3.8958f,8.6426f,-3.7917f,8.5938f,-3.7917f,
	8.5938f,-3.6875f,8.5449f,-3.6875f,8.5449f,-3.5833f,8.4961f,-3.5833f,8.4961f,-3.4792f,8.4473f,-3.4792f,8.4473f,-3.3750f,8.3984f,-3.3750f,8.3984f,-3.2708f,8.3496f,-3.2708f,8.3496f,-3.1667f,8.3008f,-3.1667f,8.3008f,-3.0625f,8.2520f,-3.0625f,8.2520f,-2.9583f,
	8.2031f,-2.9583f,8.2031f,-2.8542f,8.1543f,-2.8542f,8.1543f,-2.7500f,8.1055f,-2.7500f,8.1055f,-2.6458f,8.0566f,-2.6458f,8.0566f,-2.5417f,8.0078f,-2.5417f,8.0078f,-2.4375f,7.9590f,-2.4375f,7.9590f,-2.3333f,7.9102f,-2.3333f,7.9102f,-2.2292f,7.8613f,-2.2292f,
	7.8613f,-2.1250f,7.8125f,-2.1250f,7.8125f,-2.0208f,7.7637f,-2.0208f,7.7637f,-1.9167f,7.7149f,-1.9167f,7.7149f,-1.8125f,7.6660f,-1.8125f,7.6660f,-1.7083f,7.6172f,-1.7083f,7.6172f,-1.6042f,7.5684f,-1.6042f,7.5684f,-1.5000f,7.5195f,-1.5000f,7.5195f,-1.3958f,
	7.4707f,-1.3958f,7.4707f,-1.2917f,7.4219f,-1.2917f,7.4219f,-1.1875f,7.3731f,-1.1875f,7.3731f,-1.0833f,7.3242f,-1.0833f,7.3242f,-0.9792f,7.2754f,-0.9792f,7.2754f,-0.8750f,7.2266f,-0.8750f,7.2266f,-0.7708f,7.1777f,-0.7708f,7.1777f,-0.6667f,7.1289f,-0.6667f,
	7.1289f,-0.5625f,7.0801f,-0.5625f,7.0801f,-0.4583f,7.0313f,-0.4583f,7.0313f,-0.3542f,6.9824f,-0.3542f,6.9824f,-0.2500f,6.9336f,-0.2500f,6.9336f,-0.1458f,6.8848f,-0.1458f,6.8848f,-0.0417f,6.8359f,-0.0417f,6.8359f,0.0625f,6.7871f,0.0625f,6.7871f,0.1667f,
	6.7383f,0.1667f,6.7383f,0.2708f,6.6895f,0.2708f,6.6895f,0.3750f,6.6406f,0.3750f,6.6406f,0.4792f,6.5918f,0.4792f,6.5918f,0.5833f,6.5430f,0.5833f,6.5430f,0.6875f,6.4941f,0.6875f,6.4941f,0.7917f,6.4453f,0.7917f,6.4453f,0.8958f,6.3965f,0.8958f,
	6.3965f,1.0000f,6.3477f,1.0000f,8.6914f,-4.0000f,8.6426f,-4.0000f,8.6426f,-3.8958f,8.5938f,-3.8958f,8.5938f,-3.7917f,8.5449f,-3.7917f,8.5449f,-3.6875f,8.4961f,-3.6875f,8.4961f,-3.5833f,8.4473f,-3.5833f,8.4473f,-3.4792f,8.3984f,-3.4792f,8.3984f,-3.3750f,
	8.3496f,-3.3750f,8.3496f,-3.2708f,8.3008f,-3.2708f,8.3008f,-3.1667f,8.2520f,-3.1667f,8.2520f,-3.0625f,8.2031f,-3.0625f,8.2031f,-2.9583f,8.1543f,-2.9583f,8.1543f,-2.8542f,8.1055f,-2.8542f,8.1055f,-2.7500f,8.0566f,-2.7500f,8.0566f,-2.6458f,8.0078f,-2.6458f,
	8.0078f,-2.5417f,7.9590f,-2.5417f,7.9590f,-2.4375f,7.9102f,-2.4375f,7.9102f,-2.3333f,7.8613f,-2.3333f,7.8613f,-2.2292f,7.8125f,-2.2292f,7.8125f,-2.1250f,7.7637f,-2.1250f,7.7637f,-2.0208f,7.7149f,-2.0208f,7.7149f,-1.9167f,7.6660f,-1.9167f,7.6660f,-1.8125f,
	7.6172f,-1.8125f,7.6172f,-1.7083f,7.5684f,-1.7083f,7.5684f,-1.6042f,7.5195f,-1.6042f,7.5195f,-1.5000f,7.4707f,-1.5000f,7.4707f,-1.3958f,7.4219f,-1.3958f,7.4219f,-1.2917f,7.3731f,-1.2917f,7.3731f,-1.1875f,7.3242f,-1.1875f,7.3242f,-1.0833f,7.2754f,-1.0833f,
	7.2754f,-0.9792f,7.2266f,-0.9792f,7.2266f,-0.8750f,7.1777f,-0.8750f,7.1777f,-0.7708f,7.1289f,-0.7708f,7.1289f,-0.6667f,7.0801f,-0.6667f,7.0801f,-0.5625f,7.0313f,-0.5625f,7.0313f,-0.4583f,6.9824f,-0.4583f,6.9824f,-0.3542f,6.9336f,-0.3542f,6.9336f,-0.2500f,
	6.8848f,-0.2500f,6.8848f,-0.1458f,6.8359f,-0.1458f,6.8359f,-0.0417f,6.7871f,-0.0417f,6.7871f,0.0625f,6.7383f,0.0625f,6.7383f,0.1667f,6.6895f,0.1667f,6.6895f,0.2708f,6.6406f,0.2708f,6.6406f,0.3750f,6.5918f,0.3750f,6.5918f,0.4792f,6.5430f,0.4792f,
	6.5430f,0.5833f,6.4941f,0.5833f,6.4941f,0.6875f,6.4453f,0.6875f,6.4453f,0.7917f,6.3965f,0.7917f,6.3965f,0.8958f,6.3477f,0.8958f,6.3477f,1.0000f,6.2988f,1.0000f,8.6426f,-4.0000f,8.5938f,-4.0000f,8.5938f,-3.8958f,8.5449f,-3.8958f,8.5449f,-3.7917f,
	8.4961f,-3.7917f,8.4961f,-3.6875f,8.4473f,-3.6875f,8.4473f,-3.5833f,8.3984f,-3.5833f,8.3984f,-3.4792f,8.3496f,-3.4792f,8.3496f,-3.3750f,8.3008f,-3.3750f,8.3008f,-3.2708f,8.2520f,-3.2708f,8.2520f,-3.1667f,8.2031f,-3.1667f,8.2031f,-3.0625f,8.1543f,-3.0625f,
	8.1543f,-2.9583f,8.1055f,-2.9583f,8.1055f,-2.8542f,8.0566f,-2.8542f,8.0566f,-2.7500f,8.0078f,-2.7500f,8.0078f,-2.6458f,7.9590f,-2.6458f,7.9590f,-2.5417f,7.9102f,-2.5417f,7.9102f,-2.4375f,7.8613f,-2.4375f,7.8613f,-2.3333f,7.8125f,-2.3333f,7.8125f,-2.2292f,
	7.7637f,-2.2292f,7.7637f,-2.1250f,7.7149f,-2.1250f,7.7149f,-2.0208f,7.6660f,-2.0208f,7.6660f,-1.9167f,7.6172f,-1.9167f,7.6172f,-1.8125f,7.5684f,-1.8125f,7.5684f,-1.7083f,7.5195f,-1.7083f,7.5195f,-1.6042f,7.4707f,-1.6042f,7.4707f,-1.5000f,7.4219f,-1.5000f,
	7.4219f,-1.3958f,7.3731f,-1.3958f,7.3731f,-1.2917f,7.3242f,-1.2917f,7.3242f,-1.1875f,7.2754f,-1.1875f,7.2754f,-1.0833f,7.2266f,-1.0833f,7.2266f,-0.9792f,7.1777f,-0.9792f,7.1777f,-0.8750f,7.1289f,-0.8750f,7.1289f,-0.7708f,7.0801f,-0.7708f,7.0801f,-0.6667f,
	7.0313f,-0.6667f,7.0313f,-0.5625f,6.9824f,-0.5625f,6.9824f,-0.4583f,6.9336f,-0.4583f,6.9336f,-0.3542f,6.8848f,-0.3542f,6.8848f,-0.2500f,6.8359f,-0.2500f,6.8359f,-0.1458f,6.7871f,-0.1458f,6.7871f,-0.0417f,6.7383f,-0.0417f,6.7383f,0.0625f,6.6895f,0.0625f,
	6.6895f,0.1667f,6.6406f,0.1667f,6.6406f,0.2708f,6.5918f,0.2708f,6.5918f,0.3750f,6.5430f,0.3750f,6.5430f,0.4792f,6.4941f,0.4792f,6.4941f,0.5833f,6.4453f,0.5833f,6.4453f,0.6875f,6.3965f,0.6875f,6.3965f,0.7917f,6.3477f,0.7917f,6.3477f,0.8958f,
	6.2988f,0.8958f,6.2988f,1.0000f,6.2500f,1.0000f,8.5938f,-4.0000f,8.5449f,-4.0000f,8.5449f,-3.8958f,8.4961f,-3.8958f,8.4961f,-3.7917f,8.4473f,-3.7917f,8.4473f,-3.6875f,8.3984f,-3.6875f,8.3984f,-3.5833f,8.3496f,-3.5833f,8.3496f,-3.4792f,8.3008f,-3.4792f,
	8.3008f,-3.3750f,8.2520f,-3.3750f,8.2520f,-3.2708f,8.2031f,-3.2708f,8.2031f,-3.1667f,8.1543f,-3.1667f,8.1543f,-3.0625f,8.1055f,-3.0625f,8.1055f,-2.9583f,8.0566f,-2.9583f,8.0566f,-2.8542f,8.0078f,-2.8542f,8.0078f,-2.7500f,7.9590f,-2.7500f,7.9590f,-2.6458f,
	7.9102f,-2.6458f,7.9102f,-2.5417f,7.8613f,-2.5417f,7.8613f,-2.4375f,7.8125f,-2.4375f,7.8125f,-2.3333f,7.7637f,-2.3333f,7.7637f,-2.2292f,7.7149f,-2.2292f,7.7149f,-2.1250f,7.6660f,-2.1250f,7.6660f,-2.0208f,7.6172f,-2.0208f,7.6172f,-1.9167f,7.5684f,-1.9167f,
	7.5684f,-1.8125f,7.5195f,-1.8125f,7.5195f,-1.7083f,7.4707f,-1.7083f,7.4707f,-1.6042f,7.4219f,-1.6042f,7.4219f,-1.5000f,7.3731f,-1.5000f,7.3731f,-1.3958f,7.3242f,-1.3958f,7.3242f,-1.2917f,7.2754f,-1.2917f,7.2754f,-1.1875f,7.2266f,-1.1875f,7.2266f,-1.0833f,
	7.1777f,-1.0833f,7.1777f,-0.9792f,7.1289f,-0.9792f,7.1289f,-0.8750f,7.0801f,-0.8750f,7.0801f,-0.7708f,7.0313f,-0.7708f,7.0313f,-0.6667f,6.9824f,-0.6667f,6.9824f,-0.5625f,6.9336f,-0.5625f,6.9336f,-0.4583f,6.8848f,-0.4583f,6.8848f,-0.3542f,6.8359f,-0.3542f,
	6.8359f,-0.2500f,6.7871f,-0.2500f,6.7871f,-0.1458f,6.7383f,-0.1458f,6.7383f,-0.0417f,6.6895f,-0.0417f,6.6895f,0.0625f,6.6406f,0.0625f,6.6406f,0.1667f,6.5918f,0.1667f,6.5918f,0.2708f,6.5430f,0.2708f,6.5430f,0.3750f,6.4941f,0.3750f,6.4941f,0.4792f,
	6.4453f,0.4792f,6.4453f,0.5833f,6.3965f,0.5833f,6.3965f,0.6875f,6.3477f,0.6875f,6.3477f,0.7917f,6.2988f,0.7917f,6.2988f,0.8958f,6.2500f,0.8958f,6.2500f,1.0000f,6.2012f,1.0000f,8.5449f,-4.0000f,8.4961f,-4.0000f,8.4961f,-3.8958f,8.4473f,-3.8958f,
	8.4473f,-3.7917f,8.3984f,-3.7917f,8.3984f,-3.6875f,8.3496f,-3.6875f,8.3496f,-3.5833f,8.3008f,-3.5833f,8.3008f,-3.4792f,8.2520f,-3.4792f,8.2520f,-3.3750f,8.2031f,-3.3750f,8.2031f,-3.2708f,8.1543f,-3.2708f,8.1543f,-3.1667f,8.1055f,-3.1667f,8.1055f,-3.0625f,
	8.0566f,-3.0625f,8.0566f,-2.9583f,8.0078f,-2.9583f,8.0078f,-2.8542f,7.9590f,-2.8542f,7.9590f,-2.7500f,7.9102f,-2.7500f,7.9102f,-2.6458f,7.8613f,-2.6458f,7.8613f,-2.5417f,7.8125f,-2.5417f,7.8125f,-2.4375f,7.7637f,-2.4375f,7.7637f,-2.3333f,7.7149f,-2.3333f,
	7.7149f,-2.2292f,7.6660f,-2.2292f,7.6660f,-2.1250f,7.6172f,-2.1250f,7.6172f,-2.0208f,7.5684f,-2.0208f,7.5684f,-1.9167f,7.5195f,-1.9167f,7.5195f,-1.8125f,7.4707f,-1.8125f,7.4707f,-1.7083f,7.4219f,-1.7083f,7.4219f,-1.6042f,7.3731f,-1.6042f,7.3731f,-1.5000f,
	7.3242f,-1.5000f,7.3242f,-1.3958f,7.2754f,-1.3958f,7.2754f,-1.2917f,7.2266f,-1.2917f,7.2266f,-1.1875f,7.1777f,-1.1875f,7.1777f,-1.0833f,7.1289f,-1.0833f,7.1289f,-0.9792f,7.0801f,-0.9792f,7.0801f,-0.8750f,7.0313f,-0.8750f,7.0313f,-0.7708f,6.9824f,-0.7708f,
	6.9824f,-0.6667f,6.9336f,-0.6667f,6.9336f,-0.5625f,6.8848f,-0.5625f,6.8848f,-0.4583f,6.8359f,-0.4583f,6.8359f,-0.3542f,6.7871f,-0.3542f,6.7871f,-0.2500f,6.7383f,-0.2500f,6.7383f,-0.1458f,6.6895f,-0.1458f,6.6895f,-0.0417f,6.6406f,-0.0417f,6.6406f,0.0625f,
	6.5918f,0.0625f,6.5918f,0.1667f,6.5430f,0.1667f,6.5430f,0.2708f,6.4941f,0.2708f,6.4941f,0.3750f,6.4453f,0.3750f,6.4453f,0.4792f,6.3965f,0.4792f,6.3965f,0.5833f,6.3477f,0.5833f,6.3477f,0.6875f,6.2988f,0.6875f,6.2988f,0.7917f,6.2500f,0.7917f,
	6.2500f,0.8958f,6.2012f,0.8958f,6.2012f,1.0000f,6.1523f,1.0000f,8.4961f,-4.0000f,8.4473f,-4.0000f,8.4473f,-3.8958f,8.3984f,-3.8958f,8.3984f,-3.7917f,8.3496f,-3.7917f,8.3496f,-3.6875f,8.3008f,-3.6875f,8.3008f,-3.5833f,8.2520f,-3.5833f,8.2520f,-3.4792f,
	8.2031f,-3.4792f,8.2031f,-3.3750f,8.1543f,-3.3750f,8.1543f,-3.2708f,8.1055f,-3.2708f,8.1055f,-3.1667f,8.0566f,-3.1667f,8.0566f,-3.0625f,8.0078f,-3.0625f,8.0078f,-2.9583f,7.9590f,-2.9583f,7.9590f,-2.8542f,7.9102f,-2.8542f,7.9102f,-2.7500f,7.8613f,-2.7500f,
	7.8613f,-2.6458f,7.8125f,-2.6458f,7.8125f,-2.5417f,7.7637f,-2.5417f,7.7637f,-2.4375f,7.7149f,-2.4375f,7.7149f,-2.3333f,7.6660f,-2.3333f,7.6660f,-2.2292f,7.6172f,-2.2292f,7.6172f,-2.1250f,7.5684f,-2.1250f,7.5684f,-2.0208f,7.5195f,-2.0208f,7.5195f,-1.9167f,
	7.4707f,-1.9167f,7.4707f,-1.8125f,7.4219f,-1.8125f,7.4219f,-1.7083f,7.3731f,-1.7083f,7.3731f,-1.6042f,7.3242f,-1.6042f,7.3242f,-1.5000f,7.2754f,-1.5000f,7.2754f,-1.3958f,7.2266f,-1.3958f,7.2266f,-1.2917f,7.1777f,-1.2917f,7.1777f,-1.1875f,7.1289f,-1.1875f,
	7.1289f,-1.0833f,7.0801f,-1.0833f,7.0801f,-0.9792f,7.0313f,-0.9792f,7.0313f,-0.8750f,6.9824f,-0.8750f,6.9824f,-0.7708f,6.9336f,-0.7708f,6.9336f,-0.6667f,6.8848f,-0.6667f,6.8848f,-0.5625f,6.8359f,-0.5625f,6.8359f,-0.4583f,6.7871f,-0.4583f,6.7871f,-0.3542f,
	6.7383f,-0.3542f,6.7383f,-0.2500f,6.6895f,-0.2500f,6.6895f,-0.1458f,6.6406f,-0.1458f,6.6406f,-0.0417f,6.5918f,-0.0417f,6.5918f,0.0625f,6.5430f,0.0625f,6.5430f,0.1667f,6.4941f,0.1667f,6.4941f,0.2708f,6.4453f,0.2708f,6.4453f,0.3750f,6.3965f,0.3750f,
	6.3965f,0.4792f,6.3477f,0.4792f,6.3477f,0.5833f,6.2988f,0.5833f,6.2988f,0.6875f,6.2500f,0.6875f,6.2500f,0.7917f,6.2012f,0.7917f,6.2012f,0.8958f,6.1523f,0.8958f,6.1523f,1.0000f,6.1035f,1.0000f,8.4473f,-4.0000f,8.3984f,-4.0000f,8.3984f,-3.8958f,
	8.3496f,-3.8958f,8.3496f,-3.7917f,8.3008f,-3.7917f,8.3008f,-3.6875f,8.2520f,-3.6875f,8.2520f,-3.5833f,8.2031f,-3.5833f,8.2031f,-3.4792f,8.1543f,-3.4792f,8.1543f,-3.3750f,8.1055f,-3.3750f,8.1055f,-3.2708f,8.0566f,-3.2708f,8.0566f,-3.1667f,8.0078f,-3.1667f,
	8.0078f,-3.0625f,7.9590f,-3.0625f,7.9590f,-2.9583f,7.9102f,-2.9583f,7.9102f,-2.8542f,7.8613f,-2.8542f,7.8613f,-2.7500f,7.8125f,-2.7500f,7.8125f,-2.6458f,7.7637f,-2.6458f,7.7637f,-2.5417f,7.7149f,-2.5417f,7.7149f,-2.4375f,7.6660f,-2.4375f,7.6660f,-2.3333f,
	7.6172f,-2.3333f,7.6172f,-2.2292f,7.5684f,-2.2292f,7.5684f,-2.1250f,7.5195f,-2.1250f,7.5195f,-2.0208f,7.4707f,-2.0208f,7.4707f,-1.9167f,7.4219f,-1.9167f,7.4219f,-1.8125f,7.3731f,-1.8125f,7.3731f,-1.7083f,7.3242f,-1.7083f,7.3242f,-1.6042f,7.2754f,-1.6042f,
	7.2754f,-1.5000f,7.2266f,-1.5000f,7.2266f,-1.3958f,7.1777f,-1.3958f,7.1777f,-1.2917f,7.1289f,-1.2917f,7.1289f,-1.1875f,7.0801f,-1.1875f,7.0801f,-1.0833f,7.0313f,-1.0833f,7.0313f,-0.9792f,6.9824f,-0.9792f,6.9824f,-0.8750f,6.9336f,-0.8750f,6.9336f,-0.7708f,
	6.8848f,-0.7708f,6.8848f,-0.6667f,6.8359f,-0.6667f,6.8359f,-0.5625f,6.7871f,-0.5625f,6.7871f,-0.4583f,6.7383f,-0.4583f,6.7383f,-0.3542f,6.6895f,-0.3542f,6.6895f,-0.2500f,6.6406f,-0.2500f,6.6406f,-0.1458f,6.5918f,-0.1458f,6.5918f,-0.0417f,6.5430f,-0.0417f,
	6.5430f,0.0625f,6.4941f,0.0625f,6.4941f,0.1667f,6.4453f,0.1667f,6.4453f,0.2708f,6.3965f,0.2708f,6.3965f,0.3750f,6.3477f,0.3750f,6.3477f,0.4792f,6.2988f,0.4792f,6.2988f,0.5833f,6.2500f,0.5833f,6.2500f,0.6875f,6.2012f,0.6875f,6.2012f,0.7917f,
	6.1523f,0.7917f,6.1523f,0.8958f,6.1035f,0.8958f,6.1035f,1.0000f,6.0547f,1.0000f,8.3984f,-4.0000f,8.3496f,-4.0000f,8.3496f,-3.8958f,8.3008f,-3.8958f,8.3008f,-3.7917f,8.2520f,-3.7917f,8.2520f,-3.6875f,8.2031f,-3.6875f,8.2031f,-3.5833f,8.1543f,-3.5833f,
	8.1543f,-3.4792f,8.1055f,-3.4792f,8.1055f,-3.3750f,8.0566f,-3.3750f,8.0566f,-3.2708f,8.0078f,-3.2708f,8.0078f,-3.1667f,7.9590f,-3.1667f,7.9590f,-3.0625f,7.9102f,-3.0625f,7.9102f,-2.9583f,7.8613f,-2.9583f,7.8613f,-2.8542f,7.8125f,-2.8542f,7.8125f,-2.7500f,
	7.7637f,-2.7500f,7.7637f,-2.6458f,7.7149f,-2.6458f,7.7149f,-2.5417f,7.6660f,-2.5417f,7.6660f,-2.4375f,7.6172f,-2.4375f,7.6172f,-2.3333f,7.5684f,-2.3333f,7.5684f,-2.2292f,7.5195f,-2.2292f,7.5195f,-2.1250f,7.4707f,-2.1250f,7.4707f,-2.0208f,7.4219f,-2.0208f,
	7.4219f,-1.9167f,7.3731f,-1.9167f,7.3731f,-1.8125f,7.3242f,-1.8125f,7.3242f,-1.7083f,7.2754f,-1.7083f,7.2754f,-1.6042f,7.2266f,-1.6042f,7.2266f,-1.5000f,7.1777f,-1.5000f,7.1777f,-1.3958f,7.1289f,-1.3958f,7.1289f,-1.2917f,7.0801f,-1.2917f,7.0801f,-1.1875f,
	7.0313f,-1.1875f,7.0313f,-1.0833f,6.9824f,-1.0833f,6.9824f,-0.9792f,6.9336f,-0.9792f,6.9336f,-0.8750f,6.8848f,-0.8750f,6.8848f,-0.7708f,6.8359f,-0.7708f,6.8359f,-0.6667f,6.7871f,-0.6667f,6.7871f,-0.5625f,6.7383f,-0.5625f,6.7383f,-0.4583f,6.6895f,-0.4583f,
	6.6895f,-0.3542f,6.6406f,-0.3542f,6.6406f,-0.2500f,6.5918f,-0.2500f,6.5918f,-0.1458f,6.5430f,-0.1458f,6.5430f,-0.0417f,6.4941f,-0.0417f,6.4941f,0.0625f,6.4453f,0.0625f,6.4453f,0.1667f,6.3965f,0.1667f,6.3965f,0.2708f,6.3477f,0.2708f,6.3477f,0.3750f,
	6.2988f,0.3750f,6.2988f,0.4792f,6.2500f,0.4792f,6.2500f,0.5833f,6.2012f,0.5833f,6.2012f,0.6875f,6.1523f,0.6875f,6.1523f,0.7917f,6.1035f,0.7917f,6.1035f,0.8958f,6.0547f,0.8958f,6.0547f,1.0000f,6.0059f,1.0000f,8.3496f,-4.0000f,8.3008f,-4.0000f,
	8.3008f,-3.8958f,8.2520f,-3.8958f,8.2520f,-3.7917f,8.2031f,-3.7917f,8.2031f,-3.6875f,8.1543f,-3.6875f,8.1543f,-3.5833f,8.1055f,-3.5833f,8.1055f,-3.4792f,8.0566f,-3.4792f,8.0566f,-3.3750f,8.0078f,-3.3750f,8.0078f,-3.2708f,7.9590f,-3.2708f,7.9590f,-3.1667f,
	7.9102f,-3.1667f,7.9102f,-3.0625f,7.8613f,-3.0625f,7.8613f,-2.9583f,7.8125f,-2.9583f,7.8125f,-2.8542f,7.7637f,-2.8542f,7.7637f,-2.7500f,7.7149f,-2.7500f,7.7149f,-2.6458f,7.6660f,-2.6458f,7.6660f,-2.5417f,7.6172f,-2.5417f,7.6172f,-2.4375f,7.5684f,-2.4375f,
	7.5684f,-2.3333f,7.5195f,-2.3333f,7.5195f,-2.2292f,7.4707f,-2.2292f,7.4707f,-2.1250f,7.4219f,-2.1250f,7.4219f,-2.0208f,7.3731f,-2.0208f,7.3731f,-1.9167f,7.3242f,-1.9167f,7.3242f,-1.8125f,7.2754f,-1.8125f,7.2754f,-1.7083f,7.2266f,-1.7083f,7.2266f,-1.6042f,
	7.1777f,-1.6042f,7.1777f,-1.5000f,7.1289f,-1.5000f,7.1289f,-1.3958f,7.0801f,-1.3958f,7.0801f,-1.2917f,7.0313f,-1.2917f,7.0313f,-1.1875f,6.9824f,-1.1875f,6.9824f,-1.0833f,6.9336f,-1.0833f,6.9336f,-0.9792f,6.8848f,-0.9792f,6.8848f,-0.8750f,6.8359f,-0.8750f,
	6.8359f,-0.7708f,6.7871f,-0.7708f,6.7871f,-0.6667f,6.7383f,-0.6667f,6.7383f,-0.5625f,6.6895f,-0.5625f,6.6895f,-0.4583f,6.6406f,-0.4583f,6.6406f,-0.3542f,6.5918f,-0.3542f,6.5918f,-0.2500f,6.5430f,-0.2500f,6.5430f,-0.1458f,6.4941f,-0.1458f,6.4941f,-0.0417f,
	6.4453f,-0.0417f,6.4453f,0.0625f,6.3965f,0.0625f,6.3965f,0.1667f,6.3477f,0.1667f,6.3477f,0.2708f,6.2988f,0.2708f,6.2988f,0.3750f,6.2500f,0.3750f,6.2500f,0.4792f,6.2012f,0.4792f,6.2012f,0.5833f,6.1523f,0.5833f,6.1523f,0.6875f,6.1035f,0.6875f,
	6.1035f,0.7917f,6.0547f,0.7917f,6.0547f,0.8958f,6.0059f,0.8958f,6.0059f,1.0000f,5.9570f,1.0000f,8.3008f,-4.0000f,8.2520f,-4.0000f,8.2520f,-3.8958f,8.2031f,-3.8958f,8.2031f,-3.7917f,8.1543f,-3.7917f,8.1543f,-3.6875f,8.1055f,-3.6875f,8.1055f,-3.5833f,
	8.0566f,-3.5833f,8.0566f,-3.4792f,8.0078f,-3.4792f,8.0078f,-3.3750f,7.9590f,-3.3750f,7.9590f,-3.2708f,7.9102f,-3.2708f,7.9102f,-3.1667f,7.8613f,-3.1667f,7.8613f,-3.0625f,7.8125f,-3.0625f,7.8125f,-2.9583f,7.7637f,-2.9583f,7.7637f,-2.8542f,7.7149f,-2.8542f,
	7.7149f,-2.7500f,7.6660f,-2.7500f,7.6660f,-2.6458f,7.6172f,-2.6458f,7.6172f,-2.5417f,7.5684f,-2.5417f,7.5684f,-2.4375f,7.5195f,-2.4375f,7.5195f,-2.3333f,7.4707f,-2.3333f,7.4707f,-2.2292f,7.4219f,-2.2292f,7.4219f,-2.1250f,7.3731f,-2.1250f,7.3731f,-2.0208f,
	7.3242f,-2.0208f,7.3242f,-1.9167f,7.2754f,-1.9167f,7.2754f,-1.8125f,7.2266f,-1.8125f,7.2266f,-1.7083f,7.1777f,-1.7083f,7.1777f,-1.6042f,7.1289f,-1.6042f,7.1289f,-1.5000f,7.0801f,-1.5000f,7.0801f,-1.3958f,7.0313f,-1.3958f,7.0313f,-1.2917f,6.9824f,-1.2917f,
	6.9824f,-1.1875f,6.9336f,-1.1875f,6.9336f,-1.0833f,6.8848f,-1.0833f,6.8848f,-0.9792f,6.8359f,-0.9792f,6.8359f,-0.8750f,6.7871f,-0.8750f,6.7871f,-0.7708f,6.7383f,-0.7708f,6.7383f,-0.6667f,6.6895f,-0.6667f,6.6895f,-0.5625f,6.6406f,-0.5625f,6.6406f,-0.4583f,
	6.5918f,-0.4583f,6.5918f,-0.3542f,6.5430f,-0.3542f,6.5430f,-0.2500f,6.4941f,-0.2500f,6.4941f,-0.1458f,6.4453f,-0.1458f,6.4453f,-0.0417f,6.3965f,-0.0417f,6.3965f,0.0625f,6.3477f,0.0625f,6.3477f,0.1667f,6.2988f,0.1667f,6.2988f,0.2708f,6.2500f,0.2708f,
	6.2500f,0.3750f,6.2012f,0.3750f,6.2012f,0.4792f,6.1523f,0.4792f,6.1523f,0.5833f,6.1035f,0.5833f,6.1035f,0.6875f,6.0547f,0.6875f,6.0547f,0.7917f,6.0059f,0.7917f,6.0059f,0.8958f,5.9570f,0.8958f,5.9570f,1.0000f,5.9082f,1.0000f,8.2520f,-4.0000f,
	8.2031f,-4.0000f,8.2031f,-3.8958f,8.1543f,-3.8958f,8.1543f,-3.7917f,8.1055f,-3.7917f,8.1055f,-3.6875f,8.0566f,-3.6875f,8.0566f,-3.5833f,8.0078f,-3.5833f,8.0078f,-3.4792f,7.9590f,-3.4792f,7.9590f,-3.3750f,7.9102f,-3.3750f,7.9102f,-3.2708f,7.8613f,-3.2708f,
	7.8613f,-3.1667f,7.8125f,-3.1667f,7.8125f,-3.0625f,7.7637f,-3.0625f,7.7637f,-2.9583f,7.7149f,-2.9583f,7.7149f,-2.8542f,7.6660f,-2.8542f,7.6660f,-2.7500f,7.6172f,-2.7500f,7.6172f,-2.6458f,7.5684f,-2.6458f,7.5684f,-2.5417f,7.5195f,-2.5417f,7.5195f,-2.4375f,
	7.4707f,-2.4375f,7.4707f,-2.3333f,7.4219f,-2.3333f,7.4219f,-2.2292f,7.3731f,-2.2292f,7.3731f,-2.1250f,7.3242f,-2.1250f,7.3242f,-2.0208f,7.2754f,-2.0208f,7.2754f,-1.9167f,7.2266f,-1.9167f,7.2266f,-1.8125f,7.1777f,-1.8125f,7.1777f,-1.7083f,7.1289f,-1.7083f,
	7.1289f,-1.6042f,7.0801f,-1.6042f,7.0801f,-1.5000f,7.0313f,-1.5000f,7.0313f,-1.3958f,6.9824f,-1.3958f,6.9824f,-1.2917f,6.9336f,-1.2917f,6.9336f,-1.1875f,6.8848f,-1.1875f,6.8848f,-1.0833f,6.8359f,-1.0833f,6.8359f,-0.9792f,6.7871f,-0.9792f,6.7871f,-0.8750f,
	6.7383f,-0.8750f,6.7383f,-0.7708f,6.6895f,-0.7708f,6.6895f,-0.6667f,6.6406f,-0.6667f,6.6406f,-0.5625f,6.5918f,-0.5625f,6.5918f,-0.4583f,6.5430f,-0.4583f,6.5430f,-0.3542f,6.4941f,-0.3542f,6.4941f,-0.2500f,6.4453f,-0.2500f,6.4453f,-0.1458f,6.3965f,-0.1458f,
	6.3965f,-0.0417f,6.3477f,-0.0417f,6.3477f,0.0625f,6.2988f,0.0625f,6.2988f,0.1667f,6.2500f,0.1667f,6.2500f,0.2708f,6.2012f,0.2708f,6.2012f,0.3750f,6.1523f,0.3750f,6.1523f,0.4792f,6.1035f,0.4792f,6.1035f,0.5833f,6.0547f,0.5833f,6.0547f,0.6875f,
	6.0059f,0.6875f,6.0059f,0.7917f,5.9570f,0.7917f,5.9570f,0.8958f,5.9082f,0.8958f,5.9082f,1.0000f,5.8594f,1.0000f,8.2031f,-4.0000f,8.1543f,-4.0000f,8.1543f,-3.8958f,8.1055f,-3.8958f,8.1055f,-3.7917f,8.0566f,-3.7917f,8.0566f,-3.6875f,8.0078f,-3.6875f,
	8.0078f,-3.5833f,7.9590f,-3.5833f,7.9590f,-3.4792f,7.9102f,-3.4792f,7.9102f,-3.3750f,7.8613f,-3.3750f,7.8613f,-3.2708f,7.8125f,-3.2708f,7.8125f,-3.1667f,7.7637f,-3.1667f,7.7637f,-3.0625f,7.7149f,-3.0625f,7.7149f,-2.9583f,7.6660f,-2.9583f,7.6660f,-2.8542f,
	7.6172f,-2.8542f,7.6172f,-2.7500f,7.5684f,-2.7500f,7.5684f,-2.6458f,7.5195f,-2.6458f,7.5195f,-2.5417f,7.4707f,-2.5417f,7.4707f,-2.4375f,7.4219f,-2.4375f,7.4219f,-2.3333f,7.3731f,-2.3333f,7.3731f,-2.2292f,7.3242f,-2.2292f,7.3242f,-2.1250f,7.2754f,-2.1250f,
	7.2754f,-2.0208f,7.2266f,-2.0208f,7.2266f,-1.9167f,7.1777f,-1.9167f,7.1777f,-1.8125f,7.1289f,-1.8125f,7.1289f,-1.7083f,7.0801f,-1.7083f,7.0801f,-1.6042f,7.0313f,-1.6042f,7.0313f,-1.5000f,6.9824f,-1.5000f,6.9824f,-1.3958f,6.9336f,-1.3958f,6.9336f,-1.2917f,
	6.8848f,-1.2917f,6.8848f,-1.1875f,6.8359f,-1.1875f,6.8359f,-1.0833f,6.7871f,-1.0833f,6.7871f,-0.9792f,6.7383f,-0.9792f,6.7383f,-0.8750f,6.6895f,-0.8750f,6.6895f,-0.7708f,6.6406f,-0.7708f,6.6406f,-0.6667f,6.5918f,-0.6667f,6.5918f,-0.5625f,6.5430f,-0.5625f,
	6.5430f,-0.4583f,6.4941f,-0.4583f,6.4941f,-0.3542f,6.4453f,-0.3542f,6.4453f,-0.2500f,6.3965f,-0.2500f,6.3965f,-0.1458f,6.3477f,-0.1458f,6.3477f,-0.0417f,6.2988f,-0.0417f,6.2988f,0.0625f,6.2500f,0.0625f,6.2500f,0.1667f,6.2012f,0.1667f,6.2012f,0.2708f,
	6.1523f,0.2708f,6.1523f,0.3750f,6.1035f,0.3750f,6.1035f,0.4792f,6.0547f,0.4792f,6.0547f,0.5833f,6.0059f,0.5833f,6.0059f,0.6875f,5.9570f,0.6875f,5.9570f,0.7917f,5.9082f,0.7917f,5.9082f,0.8958f,5.8594f,0.8958f,5.8594f,1.0000f,5.8105f,1.0000f,
	8.1543f,-4.0000f,8.1055f,-4.0000f,8.1055f,-3.8958f,8.0566f,-3.8958f,8.0566f,-3.7917f,8.0078f,-3.7917f,8.0078f,-3.6875f,7.9590f,-3.6875f,7.9590f,-3.5833f,7.9102f,-3.5833f,7.9102f,-3.4792f,7.8613f,-3.4792f,7.8613f,-3.3750f,7.8125f,-3.3750f,7.8125f,-3.2708f,
	7.7637f,-3.2708f,7.7637f,-3.1667f,7.7149f,-3.1667f,7.7149f,-3.0625f,7.6660f,-3.0625f,7.6660f,-2.9583f,7.6172f,-2.9583f,7.6172f,-2.8542f,7.5684f,-2.8542f,7.5684f,-2.7500f,7.5195f,-2.7500f,7.5195f,-2.6458f,7.4707f,-2.6458f,7.4707f,-2.5417f,7.4219f,-2.5417f,
	7.4219f,-2.4375f,7.3731f,-2.4375f,7.3731f,-2.3333f,7.3242f,-2.3333f,7.3242f,-2.2292f,7.2754f,-2.2292f,7.2754f,-2.1250f,7.2266f,-2.1250f,7.2266f,-2.0208f,7.1777f,-2.0208f,7.1777f,-1.9167f,7.1289f,-1.9167f,7.1289f,-1.8125f,7.0801f,-1.8125f,7.0801f,-1.7083f,
	7.0313f,-1.7083f,7.0313f,-1.6042f,6.9824f,-1.6042f,6.9824f,-1.5000f,6.9336f,-1.5000f,6.9336f,-1.3958f,6.8848f,-1.3958f,6.8848f,-1.2917f,6.8359f,-1.2917f,6.8359f,-1.1875f,6.7871f,-1.1875f,6.7871f,-1.0833f,6.7383f,-1.0833f,6.7383f,-0.9792f,6.6895f,-0.9792f,
	6.6895f,-0.8750f,6.6406f,-0.8750f,6.6406f,-0.7708f,6.5918f,-0.7708f,6.5918f,-0.6667f,6.5430f,-0.6667f,6.5430f,-0.5625f,6.4941f,-0.5625f,6.4941f,-0.4583f,6.4453f,-0.4583f,6.4453f,-0.3542f,6.3965f,-0.3542f,6.3965f,-0.2500f,6.3477f,-0.2500f,6.3477f,-0.1458f,
	6.2988f,-0.1458f,6.2988f,-0.0417f,6.2500f,-0.0417f,6.2500f,0.0625f,6.2012f,0.0625f,6.2012f,0.1667f,6.1523f,0.1667f,6.1523f,0.2708f,6.1035f,0.2708f,6.1035f,0.3750f,6.0547f,0.3750f,6.0547f,0.4792f,6.0059f,0.4792f,6.0059f,0.5833f,5.9570f,0.5833f,
	5.9570f,0.6875f,5.9082f,0.6875f,5.9082f,0.7917f,5.8594f,0.7917f,5.8594f,0.8958f,5.8105f,0.8958f,5.8105f,1.0000f,5.7617f,1.0000f,8.1055f,-4.0000f,8.0566f,-4.0000f,8.0566f,-3.8958f,8.0078f,-3.8958f,8.0078f,-3.7917f,7.9590f,-3.7917f,7.9590f,-3.6875f,
	7.9102f,-3.6875f,7.9102f,-3.5833f,7.8613f,-3.5833f,7.8613f,-3.4792f,7.8125f,-3.4792f,7.8125f,-3.3750f,7.7637f,-3.3750f,7.7637f,-3.2708f,7.7149f,-3.2708f,7.7149f,-3.1667f,7.6660f,-3.1667f,7.6660f,-3.0625f,7.6172f,-3.0625f,7.6172f,-2.9583f,7.5684f,-2.9583f,
	7.5684f,-2.8542f,7.5195f,-2.8542f,7.5195f,-2.7500f,7.4707f,-2.7500f,7.4707f,-2.6458f,7.4219f,-2.6458f,7.4219f,-2.5417f,7.3731f,-2.5417f,7.3731f,-2.4375f,7.3242f,-2.4375f,7.3242f,-2.3333f,7.2754f,-2.3333f,7.2754f,-2.2292f,7.2266f,-2.2292f,7.2266f,-2.1250f,
	7.1777f,-2.1250f,7.1777f,-2.0208f,7.1289f,-2.0208f,7.1289f,-1.9167f,7.0801f,-1.9167f,7.0801f,-1.8125f,7.0313f,-1.8125f,7.0313f,-1.7083f,6.9824f,-1.7083f,6.9824f,-1.6042f,6.9336f,-1.6042f,6.9336f,-1.5000f,6.8848f,-1.5000f,6.8848f,-1.3958f,6.8359f,-1.3958f,
	6.8359f,-1.2917f,6.7871f,-1.2917f,6.7871f,-1.1875f,6.7383f,-1.1875f,6.7383f,-1.0833f,6.6895f,-1.0833f,6.6895f,-0.9792f,6.6406f,-0.9792f,6.6406f,-0.8750f,6.5918f,-0.8750f,6.5918f,-0.7708f,6.5430f,-0.7708f,6.5430f,-0.6667f,6.4941f,-0.6667f,6.4941f,-0.5625f,
	6.4453f,-0.5625f,6.4453f,-0.4583f,6.3965f,-0.4583f,6.3965f,-0.3542f,6.3477f,-0.3542f,6.3477f,-0.2500f,6.2988f,-0.2500f,6.2988f,-0.1458f,6.2500f,-0.1458f,6.2500f,-0.0417f,6.2012f,-0.0417f,6.2012f,0.0625f,6.1523f,0.0625f,6.1523f,0.1667f,6.1035f,0.1667f,
	6.1035f,0.2708f,6.0547f,0.2708f,6.0547f,0.3750f,6.0059f,0.3750f,6.0059f,0.4792f,5.9570f,0.4792f,5.9570f,0.5833f,5.9082f,0.5833f,5.9082f,0.6875f,5.8594f,0.6875f,5.8594f,0.7917f,5.8105f,0.7917f,5.8105f,0.8958f,5.7617f,0.8958f,5.7617f,1.0000f,
	5.7129f,1.0000f,8.0566f,-4.0000f,8.0078f,-4.0000f,8.0078f,-3.8958f,7.9590f,-3.8958f,7.9590f,-3.7917f,7.9102f,-3.7917f,7.9102f,-3.6875f,7.8613f,-3.6875f,7.8613f,-3.5833f,7.8125f,-3.5833f,7.8125f,-3.4792f,7.7637f,-3.4792f,7.7637f,-3.3750f,7.7149f,-3.3750f,
	7.7149f,-3.2708f,7.6660f,-3.2708f,7.6660f,-3.1667f,7.6172f,-3.1667f,7.6172f,-3.0625f,7.5684f,-3.0625f,7.5684f,-2.9583f,7.5195f,-2.9583f,7.5195f,-2.8542f,7.4707f,-2.8542f,7.4707f,-2.7500f,7.4219f,-2.7500f,7.4219f,-2.6458f,7.3731f,-2.6458f,7.3731f,-2.5417f,
	7.3242f,-2.5417f,7.3242f,-2.4375f,7.2754f,-2.4375f,7.2754f,-2.3333f,7.2266f,-2.3333f,7.2266f,-2.2292f,7.1777f,-2.2292f,7.1777f,-2.1250f,7.1289f,-2.1250f,7.1289f,-2.0208f,7.0801f,-2.0208f,7.0801f,-1.9167f,7.0313f,-1.9167f,7.0313f,-1.8125f,6.9824f,-1.8125f,
	6.9824f,-1.7083f,6.9336f,-1.7083f,6.9336f,-1.6042f,6.8848f,-1.6042f,6.8848f,-1.5000f,6.8359f,-1.5000f,6.8359f,-1.3958f,6.7871f,-1.3958f,6.7871f,-1.2917f,6.7383f,-1.2917f,6.7383f,-1.1875f,6.6895f,-1.1875f,6.6895f,-1.0833f,6.6406f,-1.0833f,6.6406f,-0.9792f,
	6.5918f,-0.9792f,6.5918f,-0.8750f,6.5430f,-0.8750f,6.5430f,-0.7708f,6.4941f,-0.7708f,6.4941f,-0.6667f,6.4453f,-0.6667f,6.4453f,-0.5625f,6.3965f,-0.5625f,6.3965f,-0.4583f,6.3477f,-0.4583f,6.3477f,-0.3542f,6.2988f,-0.3542f,6.2988f,-0.2500f,6.2500f,-0.2500f,
	6.2500f,-0.1458f,6.2012f,-0.1458f,6.2012f,-0.0417f,6.1523f,-0.0417f,6.1523f,0.0625f,6.1035f,0.0625f,6.1035f,0.1667f,6.0547f,0.1667f,6.0547f,0.2708f,6.0059f,0.2708f,6.0059f,0.3750f,5.9570f,0.3750f,5.9570f,0.4792f,5.9082f,0.4792f,5.9082f,0.5833f,
	5.8594f,0.5833f,5.8594f,0.6875f,5.8105f,0.6875f,5.8105f,0.7917f,5.7617f,0.7917f,5.7617f,0.8958f,5.7129f,0.8958f,5.7129f,1.0000f,5.6641f,1.0000f,8.0078f,-4.0000f,7.9590f,-4.0000f,7.9590f,-3.8958f,7.9102f,-3.8958f,7.9102f,-3.7917f,7.8613f,-3.7917f,
	7.8613f,-3.6875f,7.8125f,-3.6875f,7.8125f,-3.5833f,7.7637f,-3.5833f,7.7637f,-3.4792f,7.7149f,-3.4792f,7.7149f,-3.3750f,7.6660f,-3.3750f,7.6660f,-3.2708f,7.6172f,-3.2708f,7.6172f,-3.1667f,7.5684f,-3.1667f,7.5684f,-3.0625f,7.5195f,-3.0625f,7.5195f,-2.9583f,
	7.4707f,-2.9583f,7.4707f,-2.8542f,7.4219f,-2.8542f,7.4219f,-2.7500f,7.3731f,-2.7500f,7.3731f,-2.6458f,7.3242f,-2.6458f,7.3242f,-2.5417f,7.2754f,-2.5417f,7.2754f,-2.4375f,7.2266f,-2.4375f,7.2266f,-2.3333f,7.1777f,-2.3333f,7.1777f,-2.2292f,7.1289f,-2.2292f,
	7.1289f,-2.1250f,7.0801f,-2.1250f,7.0801f,-2.0208f,7.0313f,-2.0208f,7.0313f,-1.9167f,6.9824f,-1.9167f,6.9824f,-1.8125f,6.9336f,-1.8125f,6.9336f,-1.7083f,6.8848f,-1.7083f,6.8848f,-1.6042f,6.8359f,-1.6042f,6.8359f,-1.5000f,6.7871f,-1.5000f,6.7871f,-1.3958f,
	6.7383f,-1.3958f,6.7383f,-1.2917f,6.6895f,-1.2917f,6.6895f,-1.1875f,6.6406f,-1.1875f,6.6406f,-1.0833f,6.5918f,-1.0833f,6.5918f,-0.9792f,6.5430f,-0.9792f,6.5430f,-0.8750f,6.4941f,-0.8750f,6.4941f,-0.7708f,6.4453f,-0.7708f,6.4453f,-0.6667f,6.3965f,-0.6667f,
	6.3965f,-0.5625f,6.3477f,-0.5625f,6.3477f,-0.4583f,6.2988f,-0.4583f,6.2988f,-0.3542f,6.2500f,-0.3542f,6.2500f,-0.2500f,6.2012f,-0.2500f,6.2012f,-0.1458f,6.1523f,-0.1458f,6.1523f,-0.0417f,6.1035f,-0.0417f,6.1035f,0.0625f,6.0547f,0.0625f,6.0547f,0.1667f,
	6.0059f,0.1667f,6.0059f,0.2708f,5.9570f,0.2708f,5.9570f,0.3750f,5.9082f,0.3750f,5.9082f,0.4792f,5.8594f,0.4792f,5.8594f,0.5833f,5.8105f,0.5833f,5.8105f,0.6875f,5.7617f,0.6875f,5.7617f,0.7917f,5.7129f,0.7917f,5.7129f,0.8958f,5.6641f,0.8958f,
	5.6641f,1.0000f,5.6152f,1.0000f,7.9590f,-4.0000f,7.9102f,-4.0000f,7.9102f,-3.8958f,7.8613f,-3.8958f,7.8613f,-3.7917f,7.8125f,-3.7917f,7.8125f,-3.6875f,7.7637f,-3.6875f,7.7637f,-3.5833f,7.7149f,-3.5833f,7.7149f,-3.4792f,7.6660f,-3.4792f,7.6660f,-3.3750f,
	7.6172f,-3.3750f,7.6172f,-3.2708f,7.5684f,-3.2708f,7.5684f,-3.1667f,7.5195f,-3.1667f,7.5195f,-3.0625f,7.4707f,-3.0625f,7.4707f,-2.9583f,7.4219f,-2.9583f,7.4219f,-2.8542f,7.3731f,-2.8542f,7.3731f,-2.7500f,7.3242f,-2.7500f,7.3242f,-2.6458f,7.2754f,-2.6458f,
	7.2754f,-2.5417f,7.2266f,-2.5417f,7.2266f,-2.4375f,7.1777f,-2.4375f,7.1777f,-2.3333f,7.1289f,-2.3333f,7.1289f,-2.2292f,7.0801f,-2.2292f,7.0801f,-2.1250f,7.0313f,-2.1250f,7.0313f,-2.0208f,6.9824f,-2.0208f,6.9824f,-1.9167f,6.9336f,-1.9167f,6.9336f,-1.8125f,
	6.8848f,-1.8125f,6.8848f,-1.7083f,6.8359f,-1.7083f,6.8359f,-1.6042f,6.7871f,-1.6042f,6.7871f,-1.5000f,6.7383f,-1.5000f,6.7383f,-1.3958f,6.6895f,-1.3958f,6.6895f,-1.2917f,6.6406f,-1.2917f,6.6406f,-1.1875f,6.5918f,-1.1875f,6.5918f,-1.0833f,6.5430f,-1.0833f,
	6.5430f,-0.9792f,6.4941f,-0.9792f,6.4941f,-0.8750f,6.4453f,-0.8750f,6.4453f,-0.7708f,6.3965f,-0.7708f,6.3965f,-0.6667f,6.3477f,-0.6667f,6.3477f,-0.5625f,6.2988f,-0.5625f,6.2988f,-0.4583f,6.2500f,-0.4583f,6.2500f,-0.3542f,6.2012f,-0.3542f,6.2012f,-0.2500f,
	6.1523f,-0.2500f,6.1523f,-0.1458f,6.1035f,-0.1458f,6.1035f,-0.0417f,6.0547f,-0.0417f,6.0547f,0.0625f,6.0059f,0.0625f,6.0059f,0.1667f,5.9570f,0.1667f,5.9570f,0.2708f,5.9082f,0.2708f,5.9082f,0.3750f,5.8594f,0.3750f,5.8594f,0.4792f,5.8105f,0.4792f,
	5.8105f,0.5833f,5.7617f,0.5833f,5.7617f,0.6875f,5.7129f,0.6875f,5.7129f,0.7917f,5.6641f,0.7917f,5.6641f,0.8958f,5.6152f,0.8958f,5.6152f,1.0000f,5.5664f,1.0000f,7.9102f,-4.0000f,7.8613f,-4.0000f,7.8613f,-3.8958f,7.8125f,-3.8958f,7.8125f,-3.7917f,
	7.7637f,-3.7917f,7.7637f,-3.6875f,7.7149f,-3.6875f,7.7149f,-3.5833f,7.6660f,-3.5833f,7.6660f,-3.4792f,7.6172f,-3.4792f,7.6172f,-3.3750f,7.5684f,-3.3750f,7.5684f,-3.2708f,7.5195f,-3.2708f,7.5195f,-3.1667f,7.4707f,-3.1667f,7.4707f,-3.0625f,7.4219f,-3.0625f,
	7.4219f,-2.9583f,7.3731f,-2.9583f,7.3731f,-2.8542f,7.3242f,-2.8542f,7.3242f,-2.7500f,7.2754f,-2.7500f,7.2754f,-2.6458f,7.2266f,-2.6458f,7.2266f,-2.5417f,7.1777f,-2.5417f,7.1777f,-2.4375f,7.1289f,-2.4375f,7.1289f,-2.3333f,7.0801f,-2.3333f,7.0801f,-2.2292f,
	7.0313f,-2.2292f,7.0313f,-2.1250f,6.9824f,-2.1250f,6.9824f,-2.0208f,6.9336f,-2.0208f,6.9336f,-1.9167f,6.8848f,-1.9167f,6.8848f,-1.8125f,6.8359f,-1.8125f,6.8359f,-1.7083f,6.7871f,-1.7083f,6.7871f,-1.6042f,6.7383f,-1.6042f,6.7383f,-1.5000f,6.6895f,-1.5000f,
	6.6895f,-1.3958f,6.6406f,-1.3958f,6.6406f,-1.2917f,6.5918f,-1.2917f,6.5918f,-1.1875f,6.5430f,-1.1875f,6.5430f,-1.0833f,6.4941f,-1.0833f,6.4941f,-0.9792f,6.4453f,-0.9792f,6.4453f,-0.8750f,6.3965f,-0.8750f,6.3965f,-0.7708f,6.3477f,-0.7708f,6.3477f,-0.6667f,
	6.2988f,-0.6667f,6.2988f,-0.5625f,6.2500f,-0.5625f,6.2500f,-0.4583f,6.2012f,-0.4583f,6.2012f,-0.3542f,6.1523f,-0.3542f,6.1523f,-0.2500f,6.1035f,-0.2500f,6.1035f,-0.1458f,6.0547f,-0.1458f,6.0547f,-0.0417f,6.0059f,-0.0417f,6.0059f,0.0625f,5.9570f,0.0625f,
	5.9570f,0.1667f,5.9082f,0.1667f,5.9082f,0.2708f,5.8594f,0.2708f,5.8594f,0.3750f,5.8105f,0.3750f,5.8105f,0.4792f,5.7617f,0.4792f,5.7617f,0.5833f,5.7129f,0.5833f,5.7129f,0.6875f,5.6641f,0.6875f,5.6641f,0.7917f,5.6152f,0.7917f,5.6152f,0.8958f,
	5.5664f,0.8958f,5.5664f,1.0000f,5.5176f,1.0000f,7.8613f,-4.0000f,7.8125f,-4.0000f,7.8125f,-3.8958f,7.7637f,-3.8958f,7.7637f,-3.7917f,7.7149f,-3.7917f,7.7149f,-3.6875f,7.6660f,-3.6875f,7.6660f,-3.5833f,7.6172f,-3.5833f,7.6172f,-3.4792f,7.5684f,-3.4792f,
	7.5684f,-3.3750f,7.5195f,-3.3750f,7.5195f,-3.2708f,7.4707f,-3.2708f,7.4707f,-3.1667f,7.4219f,-3.1667f,7.4219f,-3.0625f,7.3731f,-3.0625f,7.3731f,-2.9583f,7.3242f,-2.9583f,7.3242f,-2.8542f,7.2754f,-2.8542f,7.2754f,-2.7500f,7.2266f,-2.7500f,7.2266f,-2.6458f,
	7.1777f,-2.6458f,7.1777f,-2.5417f,7.1289f,-2.5417f,7.1289f,-2.4375f,7.0801f,-2.4375f,7.0801f,-2.3333f,7.0313f,-2.3333f,7.0313f,-2.2292f,6.9824f,-2.2292f,6.9824f,-2.1250f,6.9336f,-2.1250f,6.9336f,-2.0208f,6.8848f,-2.0208f,6.8848f,-1.9167f,6.8359f,-1.9167f,
	6.8359f,-1.8125f,6.7871f,-1.8125f,6.7871f,-1.7083f,6.7383f,-1.7083f,6.7383f,-1.6042f,6.6895f,-1.6042f,6.6895f,-1.5000f,6.6406f,-1.5000f,6.6406f,-1.3958f,6.5918f,-1.3958f,6.5918f,-1.2917f,6.5430f,-1.2917f,6.5430f,-1.1875f,6.4941f,-1.1875f,6.4941f,-1.0833f,
	6.4453f,-1.0833f,6.4453f,-0.9792f,6.3965f,-0.9792f,6.3965f,-0.8750f,6.3477f,-0.8750f,6.3477f,-0.7708f,6.2988f,-0.7708f,6.2988f,-0.6667f,6.2500f,-0.6667f,6.2500f,-0.5625f,6.2012f,-0.5625f,6.2012f,-0.4583f,6.1523f,-0.4583f,6.1523f,-0.3542f,6.1035f,-0.3542f,
	6.1035f,-0.2500f,6.0547f,-0.2500f,6.0547f,-0.1458f,6.0059f,-0.1458f,6.0059f,-0.0417f,5.9570f,-0.0417f,5.9570f,0.0625f,5.9082f,0.0625f,5.9082f,0.1667f,5.8594f,0.1667f,5.8594f,0.2708f,5.8105f,0.2708f,5.8105f,0.3750f,5.7617f,0.3750f,5.7617f,0.4792f,
	5.7129f,0.4792f,5.7129f,0.5833f,5.6641f,0.5833f,5.6641f,0.6875f,5.6152f,0.6875f,5.6152f,0.7917f,5.5664f,0.7917f,5.5664f,0.8958f,5.5176f,0.8958f,5.5176f,1.0000f,5.4688f,1.0000f,7.8125f,-4.0000f,7.7637f,-4.0000f,7.7637f,-3.8958f,7.7149f,-3.8958f,
	7.7149f,-3.7917f,7.6660f,-3.7917f,7.6660f,-3.6875f,7.6172f,-3.6875f,7.6172f,-3.5833f,7.5684f,-3.5833f,7.5684f,-3.4792f,7.5195f,-3.4792f,7.5195f,-3.3750f,7.4707f,-3.3750f,7.4707f,-3.2708f,7.4219f,-3.2708f,7.4219f,-3.1667f,7.3731f,-3.1667f,7.3731f,-3.0625f,
	7.3242f,-3.0625f,7.3242f,-2.9583f,7.2754f,-2.9583f,7.2754f,-2.8542f,7.2266f,-2.8542f,7.2266f,-2.7500f,7.1777f,-2.7500f,7.1777f,-2.6458f,7.1289f,-2.6458f,7.1289f,-2.5417f,7.0801f,-2.5417f,7.0801f,-2.4375f,7.0313f,-2.4375f,7.0313f,-2.3333f,6.9824f,-2.3333f,
	6.9824f,-2.2292f,6.9336f,-2.2292f,6.9336f,-2.1250f,6.8848f,-2.1250f,6.8848f,-2.0208f,6.8359f,-2.0208f,6.8359f,-1.9167f,6.7871f,-1.9167f,6.7871f,-1.8125f,6.7383f,-1.8125f,6.7383f,-1.7083f,6.6895f,-1.7083f,6.6895f,-1.6042f,6.6406f,-1.6042f,6.6406f,-1.5000f,
	6.5918f,-1.5000f,6.5918f,-1.3958f,6.5430f,-1.3958f,6.5430f,-1.2917f,6.4941f,-1.2917f,6.4941f,-1.1875f,6.4453f,-1.1875f,6.4453f,-1.0833f,6.3965f,-1.0833f,6.3965f,-0.9792f,6.3477f,-0.9792f,6.3477f,-0.8750f,6.2988f,-0.8750f,6.2988f,-0.7708f,6.2500f,-0.7708f,
	6.2500f,-0.6667f,6.2012f,-0.6667f,6.2012f,-0.5625f,6.1523f,-0.5625f,6.1523f,-0.4583f,6.1035f,-0.4583f,6.1035f,-0.3542f,6.0547f,-0.3542f,6.0547f,-0.2500f,6.0059f,-0.2500f,6.0059f,-0.1458f,5.9570f,-0.1458f,5.9570f,-0.0417f,5.9082f,-0.0417f,5.9082f,0.0625f,
	5.8594f,0.0625f,5.8594f,0.1667f,5.8105f,0.1667f,5.8105f,0.2708f,5.7617f,0.2708f,5.7617f,0.3750f,5.7129f,0.3750f,5.7129f,0.4792f,5.6641f,0.4792f,5.6641f,0.5833f,5.6152f,0.5833f,5.6152f,0.6875f,5.5664f,0.6875f,5.5664f,0.7917f,5.5176f,0.7917f,
	5.5176f,0.8958f,5.4688f,0.8958f,5.4688f,1.0000f,5.4199f,1.0000f,7.7637f,-4.0000f,7.7149f,-4.0000f,7.7149f,-3.8958f,7.6660f,-3.8958f,7.6660f,-3.7917f,7.6172f,-3.7917f,7.6172f,-3.6875f,7.5684f,-3.6875f,7.5684f,-3.5833f,7.5195f,-3.5833f,7.5195f,-3.4792f,
	7.4707f,-3.4792f,7.4707f,-3.3750f,7.4219f,-3.3750f,7.4219f,-3.2708f,7.3731f,-3.2708f,7.3731f,-3.1667f,7.3242f,-3.1667f,7.3242f,-3.0625f,7.2754f,-3.0625f,7.2754f,-2.9583f,7.2266f,-2.9583f,7.2266f,-2.8542f,7.1777f,-2.8542f,7.1777f,-2.7500f,7.1289f,-2.7500f,
	7.1289f,-2.6458f,7.0801f,-2.6458f,7.0801f,-2.5417f,7.0313f,-2.5417f,7.0313f,-2.4375f,6.9824f,-2.4375f,6.9824f,-2.3333f,6.9336f,-2.3333f,6.9336f,-2.2292f,6.8848f,-2.2292f,6.8848f,-2.1250f,6.8359f,-2.1250f,6.8359f,-2.0208f,6.7871f,-2.0208f,6.7871f,-1.9167f,
	6.7383f,-1.9167f,6.7383f,-1.8125f,6.6895f,-1.8125f,6.6895f,-1.7083f,6.6406f,-1.7083f,6.6406f,-1.6042f,6.5918f,-1.6042f,6.5918f,-1.5000f,6.5430f,-1.5000f,6.5430f,-1.3958f,6.4941f,-1.3958f,6.4941f,-1.2917f,6.4453f,-1.2917f,6.4453f,-1.1875f,6.3965f,-1.1875f,
	6.3965f,-1.0833f,6.3477f,-1.0833f,6.3477f,-0.9792f,6.2988f,-0.9792f,6.2988f,-0.8750f,6.2500f,-0.8750f,6.2500f,-0.7708f,6.2012f,-0.7708f,6.2012f,-0.6667f,6.1523f,-0.6667f,6.1523f,-0.5625f,6.1035f,-0.5625f,6.1035f,-0.4583f,6.0547f,-0.4583f,6.0547f,-0.3542f,
	6.0059f,-0.3542f,6.0059f,-0.2500f,5.9570f,-0.2500f,5.9570f,-0.1458f,5.9082f,-0.1458f,5.9082f,-0.0417f,5.8594f,-0.0417f,5.8594f,0.0625f,5.8105f,0.0625f,5.8105f,0.1667f,5.7617f,0.1667f,5.7617f,0.2708f,5.7129f,0.2708f,5.7129f,0.3750f,5.6641f,0.3750f,
	5.6641f,0.4792f,5.6152f,0.4792f,5.6152f,0.5833f,5.5664f,0.5833f,5.5664f,0.6875f,5.5176f,0.6875f,5.5176f,0.7917f,5.4688f,0.7917f,5.4688f,0.8958f,5.4199f,0.8958f,5.4199f,1.0000f,5.3711f,1.0000f,7.7149f,-4.0000f,7.6660f,-4.0000f,7.6660f,-3.8958f,
	7.6172f,-3.8958f,7.6172f,-3.7917f,7.5684f,-3.7917f,7.5684f,-3.6875f,7.5195f,-3.6875f,7.5195f,-3.5833f,7.4707f,-3.5833f,7.4707f,-3.4792f,7.4219f,-3.4792f,7.4219f,-3.3750f,7.3731f,-3.3750f,7.3731f,-3.2708f,7.3242f,-3.2708f,7.3242f,-3.1667f,7.2754f,-3.1667f,
	7.2754f,-3.0625f,7.2266f,-3.0625f,7.2266f,-2.9583f,7.1777f,-2.9583f,7.1777f,-2.8542f,7.1289f,-2.8542f,7.1289f,-2.7500f,7.0801f,-2.7500f,7.0801f,-2.6458f,7.0313f,-2.6458f,7.0313f,-2.5417f,6.9824f,-2.5417f,6.9824f,-2.4375f,6.9336f,-2.4375f,6.9336f,-2.3333f,
	6.8848f,-2.3333f,6.8848f,-2.2292f,6.8359f,-2.2292f,6.8359f,-2.1250f,6.7871f,-2.1250f,6.7871f,-2.0208f,6.7383f,-2.0208f,6.7383f,-1.9167f,6.6895f,-1.9167f,6.6895f,-1.8125f,6.6406f,-1.8125f,6.6406f,-1.7083f,6.5918f,-1.7083f,6.5918f,-1.6042f,6.5430f,-1.6042f,
	6.5430f,-1.5000f,6.4941f,-1.5000f,6.4941f,-1.3958f,6.4453f,-1.3958f,6.4453f,-1.2917f,6.3965f,-1.2917f,6.3965f,-1.1875f,6.3477f,-1.1875f,6.3477f,-1.0833f,6.2988f,-1.0833f,6.2988f,-0.9792f,6.2500f,-0.9792f,6.2500f,-0.8750f,6.2012f,-0.8750f,6.2012f,-0.7708f,
	6.1523f,-0.7708f,6.1523f,-0.6667f,6.1035f,-0.6667f,6.1035f,-0.5625f,6.0547f,-0.5625f,6.0547f,-0.4583f,6.0059f,-0.4583f,6.0059f,-0.3542f,5.9570f,-0.3542f,5.9570f,-0.2500f,5.9082f,-0.2500f,5.9082f,-0.1458f,5.8594f,-0.1458f,5.8594f,-0.0417f,5.8105f,-0.0417f,
	5.8105f,0.0625f,5.7617f,0.0625f,5.7617f,0.1667f,5.7129f,0.1667f,5.7129f,0.2708f,5.6641f,0.2708f,5.6641f,0.3750f,5.6152f,0.3750f,5.6152f,0.4792f,5.5664f,0.4792f,5.5664f,0.5833f,5.5176f,0.5833f,5.5176f,0.6875f,5.4688f,0.6875f,5.4688f,0.7917f,
	5.4199f,0.7917f,5.4199f,0.8958f,5.3711f,0.8958f,5.3711f,1.0000f,5.3223f,1.0000f,7.6660f,-4.0000f,7.6172f,-4.0000f,7.6172f,-3.8958f,7.5684f,-3.8958f,7.5684f,-3.7917f,7.5195f,-3.7917f,7.5195f,-3.6875f,7.4707f,-3.6875f,7.4707f,-3.5833f,7.4219f,-3.5833f,
	7.4219f,-3.4792f,7.3731f,-3.4792f,7.3731f,-3.3750f,7.3242f,-3.3750f,7.3242f,-3.2708f,7.2754f,-3.2708f,7.2754f,-3.1667f,7.2266f,-3.1667f,7.2266f,-3.0625f,7.1777f,-3.0625f,7.1777f,-2.9583f,7.1289f,-2.9583f,7.1289f,-2.8542f,7.0801f,-2.8542f,7.0801f,-2.7500f,
	7.0313f,-2.7500f,7.0313f,-2.6458f,6.9824f,-2.6458f,6.9824f,-2.5417f,6.9336f,-2.5417f,6.9336f,-2.4375f,6.8848f,-2.4375f,6.8848f,-2.3333f,6.8359f,-2.3333f,6.8359f,-2.2292f,6.7871f,-2.2292f,6.7871f,-2.1250f,6.7383f,-2.1250f,6.7383f,-2.0208f,6.6895f,-2.0208f,
	6.6895f,-1.9167f,6.6406f,-1.9167f,6.6406f,-1.8125f,6.5918f,-1.8125f,6.5918f,-1.7083f,6.5430f,-1.7083f,6.5430f,-1.6042f,6.4941f,-1.6042f,6.4941f,-1.5000f,6.4453f,-1.5000f,6.4453f,-1.3958f,6.3965f,-1.3958f,6.3965f,-1.2917f,6.3477f,-1.2917f,6.3477f,-1.1875f,
	6.2988f,-1.1875f,6.2988f,-1.0833f,6.2500f,-1.0833f,6.2500f,-0.9792f,6.2012f,-0.9792f,6.2012f,-0.8750f,6.1523f,-0.8750f,6.1523f,-0.7708f,6.1035f,-0.7708f,6.1035f,-0.6667f,6.0547f,-0.6667f,6.0547f,-0.5625f,6.0059f,-0.5625f,6.0059f,-0.4583f,5.9570f,-0.4583f,
	5.9570f,-0.3542f,5.9082f,-0.3542f,5.9082f,-0.2500f,5.8594f,-0.2500f,5.8594f,-0.1458f,5.8105f,-0.1458f,5.8105f,-0.0417f,5.7617f,-0.0417f,5.7617f,0.0625f,5.7129f,0.0625f,5.7129f,0.1667f,5.6641f,0.1667f,5.6641f,0.2708f,5.6152f,0.2708f,5.6152f,0.3750f,
	5.5664f,0.3750f,5.5664f,0.4792f,5.5176f,0.4792f,5.5176f,0.5833f,5.4688f,0.5833f,5.4688f,0.6875f,5.4199f,0.6875f,5.4199f,0.7917f,5.3711f,0.7917f,5.3711f,0.8958f,5.3223f,0.8958f,5.3223f,1.0000f,5.2734f,1.0000f,7.6172f,-4.0000f,7.5684f,-4.0000f,
	7.5684f,-3.8958f,7.5195f,-3.8958f,7.5195f,-3.7917f,7.4707f,-3.7917f,7.4707f,-3.6875f,7.4219f,-3.6875f,7.4219f,-3.5833f,7.3731f,-3.5833f,7.3731f,-3.4792f,7.3242f,-3.4792f,7.3242f,-3.3750f,7.2754f,-3.3750f,7.2754f,-3.2708f,7.2266f,-3.2708f,7.2266f,-3.1667f,
	7.1777f,-3.1667f,7.1777f,-3.0625f,7.1289f,-3.0625f,7.1289f,-2.9583f,7.0801f,-2.9583f,7.0801f,-2.8542f,7.0313f,-2.8542f,7.0313f,-2.7500f,6.9824f,-2.7500f,6.9824f,-2.6458f,6.9336f,-2.6458f,6.9336f,-2.5417f,6.8848f,-2.5417f,6.8848f,-2.4375f,6.8359f,-2.4375f,
	6.8359f,-2.3333f,6.7871f,-2.3333f,6.7871f,-2.2292f,6.7383f,-2.2292f,6.7383f,-2.1250f,6.6895f,-2.1250f,6.6895f,-2.0208f,6.6406f,-2.0208f,6.6406f,-1.9167f,6.5918f,-1.9167f,6.5918f,-1.8125f,6.5430f,-1.8125f,6.5430f,-1.7083f,6.4941f,-1.7083f,6.4941f,-1.6042f,
	6.4453f,-1.6042f,6.4453f,-1.5000f,6.3965f,-1.5000f,6.3965f,-1.3958f,6.3477f,-1.3958f,6.3477f,-1.2917f,6.2988f,-1.2917f,6.2988f,-1.1875f,6.2500f,-1.1875f,6.2500f,-1.0833f,6.2012f,-1.0833f,6.2012f,-0.9792f,6.1523f,-0.9792f,6.1523f,-0.8750f,6.1035f,-0.8750f,
	6.1035f,-0.7708f,6.0547f,-0.7708f,6.0547f,-0.6667f,6.0059f,-0.6667f,6.0059f,-0.5625f,5.9570f,-0.5625f,5.9570f,-0.4583f,5.9082f,-0.4583f,5.9082f,-0.3542f,5.8594f,-0.3542f,5.8594f,-0.2500f,5.8105f,-0.2500f,5.8105f,-0.1458f,5.7617f,-0.1458f,5.7617f,-0.0417f,
	5.7129f,-0.0417f,5.7129f,0.0625f,5.6641f,0.0625f,5.6641f,0.1667f,5.6152f,0.1667f,5.6152f,0.2708f,5.5664f,0.2708f,5.5664f,0.3750f,5.5176f,0.3750f,5.5176f,0.4792f,5.4688f,0.4792f,5.4688f,0.5833f,5.4199f,0.5833f,5.4199f,0.6875f,5.3711f,0.6875f,
	5.3711f,0.7917f,5.3223f,0.7917f,5.3223f,0.8958f,5.2734f,0.8958f,5.2734f,1.0000f,5.2246f,1.0000f,7.5684f,-4.0000f,7.5195f,-4.0000f,7.5195f,-3.8958f,7.4707f,-3.8958f,7.4707f,-3.7917f,7.4219f,-3.7917f,7.4219f,-3.6875f,7.3731f,-3.6875f,7.3731f,-3.5833f,
	7.3242f,-3.5833f,7.3242f,-3.4792f,7.2754f,-3.4792f,7.2754f,-3.3750f,7.2266f,-3.3750f,7.2266f,-3.2708f,7.1777f,-3.2708f,7.1777f,-3.1667f,7.1289f,-3.1667f,7.1289f,-3.0625f,7.0801f,-3.0625f,7.0801f,-2.9583f,7.0313f,-2.9583f,7.0313f,-2.8542f,6.9824f,-2.8542f,
	6.9824f,-2.7500f,6.9336f,-2.7500f,6.9336f,-2.6458f,6.8848f,-2.6458f,6.8848f,-2.5417f,6.8359f,-2.5417f,6.8359f,-2.4375f,6.7871f,-2.4375f,6.7871f,-2.3333f,6.7383f,-2.3333f,6.7383f,-2.2292f,6.6895f,-2.2292f,6.6895f,-2.1250f,6.6406f,-2.1250f,6.6406f,-2.0208f,
	6.5918f,-2.0208f,6.5918f,-1.9167f,6.5430f,-1.9167f,6.5430f,-1.8125f,6.4941f,-1.8125f,6.4941f,-1.7083f,6.4453f,-1.7083f,6.4453f,-1.6042f,6.3965f,-1.6042f,6.3965f,-1.5000f,6.3477f,-1.5000f,6.3477f,-1.3958f,6.2988f,-1.3958f,6.2988f,-1.2917f,6.2500f,-1.2917f,
	6.2500f,-1.1875f,6.2012f,-1.1875f,6.2012f,-1.0833f,6.1523f,-1.0833f,6.1523f,-0.9792f,6.1035f,-0.9792f,6.1035f,-0.8750f,6.0547f,-0.8750f,6.0547f,-0.7708f,6.0059f,-0.7708f,6.0059f,-0.6667f,5.9570f,-0.6667f,5.9570f,-0.5625f,5.9082f,-0.5625f,5.9082f,-0.4583f,
	5.8594f,-0.4583f,5.8594f,-0.3542f,5.8105f,-0.3542f,5.8105f,-0.2500f,5.7617f,-0.2500f,5.7617f,-0.1458f,5.7129f,-0.1458f,5.7129f,-0.0417f,5.6641f,-0.0417f,5.6641f,0.0625f,5.6152f,0.0625f,5.6152f,0.1667f,5.5664f,0.1667f,5.5664f,0.2708f,5.5176f,0.2708f,
	5.5176f,0.3750f,5.4688f,0.3750f,5.4688f,0.4792f,5.4199f,0.4792f,5.4199f,0.5833f,5.3711f,0.5833f,5.3711f,0.6875f,5.3223f,0.6875f,5.3223f,0.7917f,5.2734f,0.7917f,5.2734f,0.8958f,5.2246f,0.8958f,5.2246f,1.0000f,5.1758f,1.0000f,7.5195f,-4.0000f,
	7.4707f,-4.0000f,7.4707f,-3.8958f,7.4219f,-3.8958f,7.4219f,-3.7917f,7.3731f,-3.7917f,7.3731f,-3.6875f,7.3242f,-3.6875f,7.3242f,-3.5833f,7.2754f,-3.5833f,7.2754f,-3.4792f,7.2266f,-3.4792f,7.2266f,-3.3750f,7.1777f,-3.3750f,7.1777f,-3.2708f,7.1289f,-3.2708f,
	7.1289f,-3.1667f,7.0801f,-3.1667f,7.0801f,-3.0625f,7.0313f,-3.0625f,7.0313f,-2.9583f,6.9824f,-2.9583f,6.9824f,-2.8542f,6.9336f,-2.8542f,6.9336f,-2.7500f,6.8848f,-2.7500f,6.8848f,-2.6458f,6.8359f,-2.6458f,6.8359f,-2.5417f,6.7871f,-2.5417f,6.7871f,-2.4375f,
	6.7383f,-2.4375f,6.7383f,-2.3333f,6.6895f,-2.3333f,6.6895f,-2.2292f,6.6406f,-2.2292f,6.6406f,-2.1250f,6.5918f,-2.1250f,6.5918f,-2.0208f,6.5430f,-2.0208f,6.5430f,-1.9167f,6.4941f,-1.9167f,6.4941f,-1.8125f,6.4453f,-1.8125f,6.4453f,-1.7083f,6.3965f,-1.7083f,
	6.3965f,-1.6042f,6.3477f,-1.6042f,6.3477f,-1.5000f,6.2988f,-1.5000f,6.2988f,-1.3958f,6.2500f,-1.3958f,6.2500f,-1.2917f,6.2012f,-1.2917f,6.2012f,-1.1875f,6.1523f,-1.1875f,6.1523f,-1.0833f,6.1035f,-1.0833f,6.1035f,-0.9792f,6.0547f,-0.9792f,6.0547f,-0.8750f,
	6.0059f,-0.8750f,6.0059f,-0.7708f,5.9570f,-0.7708f,5.9570f,-0.6667f,5.9082f,-0.6667f,5.9082f,-0.5625f,5.8594f,-0.5625f,5.8594f,-0.4583f,5.8105f,-0.4583f,5.8105f,-0.3542f,5.7617f,-0.3542f,5.7617f,-0.2500f,5.7129f,-0.2500f,5.7129f,-0.1458f,5.6641f,-0.1458f,
	5.6641f,-0.0417f,5.6152f,-0.0417f,5.6152f,0.0625f,5.5664f,0.0625f,5.5664f,0.1667f,5.5176f,0.1667f,5.5176f,0.2708f,5.4688f,0.2708f,5.4688f,0.3750f,5.4199f,0.3750f,5.4199f,0.4792f,5.3711f,0.4792f,5.3711f,0.5833f,5.3223f,0.5833f,5.3223f,0.6875f,
	5.2734f,0.6875f,5.2734f,0.7917f,5.2246f,0.7917f,5.2246f,0.8958f,5.1758f,0.8958f,5.1758f,1.0000f,5.1270f,1.0000f,7.4707f,-4.0000f,7.4219f,-4.0000f,7.4219f,-3.8958f,7.3731f,-3.8958f,7.3731f,-3.7917f,7.3242f,-3.7917f,7.3242f,-3.6875f,7.2754f,-3.6875f,
	7.2754f,-3.5833f,7.2266f,-3.5833f,7.2266f,-3.4792f,7.1777f,-3.4792f,7.1777f,-3.3750f,7.1289f,-3.3750f,7.1289f,-3.2708f,7.0801f,-3.2708f,7.0801f,-3.1667f,7.0313f,-3.1667f,7.0313f,-3.0625f,6.9824f,-3.0625f,6.9824f,-2.9583f,6.9336f,-2.9583f,6.9336f,-2.8542f,
	6.8848f,-2.8542f,6.8848f,-2.7500f,6.8359f,-2.7500f,6.8359f,-2.6458f,6.7871f,-2.6458f,6.7871f,-2.5417f,6.7383f,-2.5417f,6.7383f,-2.4375f,6.6895f,-2.4375f,6.6895f,-2.3333f,6.6406f,-2.3333f,6.6406f,-2.2292f,6.5918f,-2.2292f,6.5918f,-2.1250f,6.5430f,-2.1250f,
	6.5430f,-2.0208f,6.4941f,-2.0208f,6.4941f,-1.9167f,6.4453f,-1.9167f,6.4453f,-1.8125f,6.3965f,-1.8125f,6.3965f,-1.7083f,6.3477f,-1.7083f,6.3477f,-1.6042f,6.2988f,-1.6042f,6.2988f,-1.5000f,6.2500f,-1.5000f,6.2500f,-1.3958f,6.2012f,-1.3958f,6.2012f,-1.2917f,
	6.1523f,-1.2917f,6.1523f,-1.1875f,6.1035f,-1.1875f,6.1035f,-1.0833f,6.0547f,-1.0833f,6.0547f,-0.9792f,6.0059f,-0.9792f,6.0059f,-0.8750f,5.9570f,-0.8750f,5.9570f,-0.7708f,5.9082f,-0.7708f,5.9082f,-0.6667f,5.8594f,-0.6667f,5.8594f,-0.5625f,5.8105f,-0.5625f,
	5.8105f,-0.4583f,5.7617f,-0.4583f,5.7617f,-0.3542f,5.7129f,-0.3542f,5.7129f,-0.2500f,5.6641f,-0.2500f,5.6641f,-0.1458f,5.6152f,-0.1458f,5.6152f,-0.0417f,5.5664f,-0.0417f,5.5664f,0.0625f,5.5176f,0.0625f,5.5176f,0.1667f,5.4688f,0.1667f,5.4688f,0.2708f,
	5.4199f,0.2708f,5.4199f,0.3750f,5.3711f,0.3750f,5.3711f,0.4792f,5.3223f,0.4792f,5.3223f,0.5833f,5.2734f,0.5833f,5.2734f,0.6875f,5.2246f,0.6875f,5.2246f,0.7917f,5.1758f,0.7917f,5.1758f,0.8958f,5.1270f,0.8958f,5.1270f,1.0000f,5.0781f,1.0000f,
	7.4219f,-4.0000f,7.3731f,-4.0000f,7.3731f,-3.8958f,7.3242f,-3.8958f,7.3242f,-3.7917f,7.2754f,-3.7917f,7.2754f,-3.6875f,7.2266f,-3.6875f,7.2266f,-3.5833f,7.1777f,-3.5833f,7.1777f,-3.4792f,7.1289f,-3.4792f,7.1289f,-3.3750f,7.0801f,-3.3750f,7.0801f,-3.2708f,
	7.0313f,-3.2708f,7.0313f,-3.1667f,6.9824f,-3.1667f,6.9824f,-3.0625f,6.9336f,-3.0625f,6.9336f,-2.9583f,6.8848f,-2.9583f,6.8848f,-2.8542f,6.8359f,-2.8542f,6.8359f,-2.7500f,6.7871f,-2.7500f,6.7871f,-2.6458f,6.7383f,-2.6458f,6.7383f,-2.5417f,6.6895f,-2.5417f,
	6.6895f,-2.4375f,6.6406f,-2.4375f,6.6406f,-2.3333f,6.5918f,-2.3333f,6.5918f,-2.2292f,6.5430f,-2.2292f,6.5430f,-2.1250f,6.4941f,-2.1250f,6.4941f,-2.0208f,6.4453f,-2.0208f,6.4453f,-1.9167f,6.3965f,-1.9167f,6.3965f,-1.8125f,6.3477f,-1.8125f,6.3477f,-1.7083f,
	6.2988f,-1.7083f,6.2988f,-1.6042f,6.2500f,-1.6042f,6.2500f,-1.5000f,6.2012f,-1.5000f,6.2012f,-1.3958f,6.1523f,-1.3958f,6.1523f,-1.2917f,6.1035f,-1.2917f,6.1035f,-1.1875f,6.0547f,-1.1875f,6.0547f,-1.0833f,6.0059f,-1.0833f,6.0059f,-0.9792f,5.9570f,-0.9792f,
	5.9570f,-0.8750f,5.9082f,-0.8750f,5.9082f,-0.7708f,5.8594f,-0.7708f,5.8594f,-0.6667f,5.8105f,-0.6667f,5.8105f,-0.5625f,5.7617f,-0.5625f,5.7617f,-0.4583f,5.7129f,-0.4583f,5.7129f,-0.3542f,5.6641f,-0.3542f,5.6641f,-0.2500f,5.6152f,-0.2500f,5.6152f,-0.1458f,
	5.5664f,-0.1458f,5.5664f,-0.0417f,5.5176f,-0.0417f,5.5176f,0.0625f,5.4688f,0.0625f,5.4688f,0.1667f,5.4199f,0.1667f,5.4199f,0.2708f,5.3711f,0.2708f,5.3711f,0.3750f,5.3223f,0.3750f,5.3223f,0.4792f,5.2734f,0.4792f,5.2734f,0.5833f,5.2246f,0.5833f,
	5.2246f,0.6875f,5.1758f,0.6875f,5.1758f,0.7917f,5.1270f,0.7917f,5.1270f,0.8958f,5.0781f,0.8958f,5.0781f,1.0000f,5.0293f,1.0000f,7.3731f,-4.0000f,7.3242f,-4.0000f,7.3242f,-3.8958f,7.2754f,-3.8958f,7.2754f,-3.7917f,7.2266f,-3.7917f,7.2266f,-3.6875f,
	7.1777f,-3.6875f,7.1777f,-3.5833f,7.1289f,-3.5833f,7.1289f,-3.4792f,7.0801f,-3.4792f,7.0801f,-3.3750f,7.0313f,-3.3750f,7.0313f,-3.2708f,6.9824f,-3.2708f,6.9824f,-3.1667f,6.9336f,-3.1667f,6.9336f,-3.0625f,6.8848f,-3.0625f,6.8848f,-2.9583f,6.8359f,-2.9583f,
	6.8359f,-2.8542f,6.7871f,-2.8542f,6.7871f,-2.7500f,6.7383f,-2.7500f,6.7383f,-2.6458f,6.6895f,-2.6458f,6.6895f,-2.5417f,6.6406f,-2.5417f,6.6406f,-2.4375f,6.5918f,-2.4375f,6.5918f,-2.3333f,6.5430f,-2.3333f,6.5430f,-2.2292f,6.4941f,-2.2292f,6.4941f,-2.1250f,
	6.4453f,-2.1250f,6.4453f,-2.0208f,6.3965f,-2.0208f,6.3965f,-1.9167f,6.3477f,-1.9167f,6.3477f,-1.8125f,6.2988f,-1.8125f,6.2988f,-1.7083f,6.2500f,-1.7083f,6.2500f,-1.6042f,6.2012f,-1.6042f,6.2012f,-1.5000f,6.1523f,-1.5000f,6.1523f,-1.3958f,6.1035f,-1.3958f,
	6.1035f,-1.2917f,6.0547f,-1.2917f,6.0547f,-1.1875f,6.0059f,-1.1875f,6.0059f,-1.0833f,5.9570f,-1.0833f,5.9570f,-0.9792f,5.9082f,-0.9792f,5.9082f,-0.8750f,5.8594f,-0.8750f,5.8594f,-0.7708f,5.8105f,-0.7708f,5.8105f,-0.6667f,5.7617f,-0.6667f,5.7617f,-0.5625f,
	5.7129f,-0.5625f,5.7129f,-0.4583f,5.6641f,-0.4583f,5.6641f,-0.3542f,5.6152f,-0.3542f,5.6152f,-0.2500f,5.5664f,-0.2500f,5.5664f,-0.1458f,5.5176f,-0.1458f,5.5176f,-0.0417f,5.4688f,-0.0417f,5.4688f,0.0625f,5.4199f,0.0625f,5.4199f,0.1667f,5.3711f,0.1667f,
	5.3711f,0.2708f,5.3223f,0.2708f,5.3223f,0.3750f,5.2734f,0.3750f,5.2734f,0.4792f,5.2246f,0.4792f,5.2246f,0.5833f,5.1758f,0.5833f,5.1758f,0.6875f,5.1270f,0.6875f,5.1270f,0.7917f,5.0781f,0.7917f,5.0781f,0.8958f,5.0293f,0.8958f,5.0293f,1.0000f,
	4.9805f,1.0000f,7.3242f,-4.0000f,7.2754f,-4.0000f,7.2754f,-3.8958f,7.2266f,-3.8958f,7.2266f,-3.7917f,7.1777f,-3.7917f,7.1777f,-3.6875f,7.1289f,-3.6875f,7.1289f,-3.5833f,7.0801f,-3.5833f,7.0801f,-3.4792f,7.0313f,-3.4792f,7.0313f,-3.3750f,6.9824f,-3.3750f,
	6.9824f,-3.2708f,6.9336f,-3.2708f,6.9336f,-3.1667f,6.8848f,-3.1667f,6.8848f,-3.0625f,6.8359f,-3.0625f,6.8359f,-2.9583f,6.7871f,-2.9583f,6.7871f,-2.8542f,6.7383f,-2.8542f,6.7383f,-2.7500f,6.6895f,-2.7500f,6.6895f,-2.6458f,6.6406f,-2.6458f,6.6406f,-2.5417f,
	6.5918f,-2.5417f,6.5918f,-2.4375f,6.5430f,-2.4375f,6.5430f,-2.3333f,6.4941f,-2.3333f,6.4941f,-2.2292f,6.4453f,-2.2292f,6.4453f,-2.1250f,6.3965f,-2.1250f,6.3965f,-2.0208f,6.3477f,-2.0208f,6.3477f,-1.9167f,6.2988f,-1.9167f,6.2988f,-1.8125f,6.2500f,-1.8125f,
	6.2500f,-1.7083f,6.2012f,-1.7083f,6.2012f,-1.6042f,6.1523f,-1.6042f,6.1523f,-1.5000f,6.1035f,-1.5000f,6.1035f,-1.3958f,6.0547f,-1.3958f,6.0547f,-1.2917f,6.0059f,-1.2917f,6.0059f,-1.1875f,5.9570f,-1.1875f,5.9570f,-1.0833f,5.9082f,-1.0833f,5.9082f,-0.9792f,
	5.8594f,-0.9792f,5.8594f,-0.8750f,5.8105f,-0.8750f,5.8105f,-0.7708f,5.7617f,-0.7708f,5.7617f,-0.6667f,5.7129f,-0.6667f,5.7129f,-0.5625f,5.6641f,-0.5625f,5.6641f,-0.4583f,5.6152f,-0.4583f,5.6152f,-0.3542f,5.5664f,-0.3542f,5.5664f,-0.2500f,5.5176f,-0.2500f,
	5.5176f,-0.1458f,5.4688f,-0.1458f,5.4688f,-0.0417f,5.4199f,-0.0417f,5.4199f,0.0625f,5.3711f,0.0625f,5.3711f,0.1667f,5.3223f,0.1667f,5.3223f,0.2708f,5.2734f,0.2708f,5.2734f,0.3750f,5.2246f,0.3750f,5.2246f,0.4792f,5.1758f,0.4792f,5.1758f,0.5833f,
	5.1270f,0.5833f,5.1270f,0.6875f,5.0781f,0.6875f,5.0781f,0.7917f,5.0293f,0.7917f,5.0293f,0.8958f,4.9805f,0.8958f,4.9805f,1.0000f,4.9316f,1.0000f,7.2754f,-4.0000f,7.2266f,-4.0000f,7.2266f,-3.8958f,7.1777f,-3.8958f,7.1777f,-3.7917f,7.1289f,-3.7917f,
	7.1289f,-3.6875f,7.0801f,-3.6875f,7.0801f,-3.5833f,7.0313f,-3.5833f,7.0313f,-3.4792f,6.9824f,-3.4792f,6.9824f,-3.3750f,6.9336f,-3.3750f,6.9336f,-3.2708f,6.8848f,-3.2708f,6.8848f,-3.1667f,6.8359f,-3.1667f,6.8359f,-3.0625f,6.7871f,-3.0625f,6.7871f,-2.9583f,
	6.7383f,-2.9583f,6.7383f,-2.8542f,6.6895f,-2.8542f,6.6895f,-2.7500f,6.6406f,-2.7500f,6.6406f,-2.6458f,6.5918f,-2.6458f,6.5918f,-2.5417f,6.5430f,-2.5417f,6.5430f,-2.4375f,6.4941f,-2.4375f,6.4941f,-2.3333f,6.4453f,-2.3333f,6.4453f,-2.2292f,6.3965f,-2.2292f,
	6.3965f,-2.1250f,6.3477f,-2.1250f,6.3477f,-2.0208f,6.2988f,-2.0208f,6.2988f,-1.9167f,6.2500f,-1.9167f,6.2500f,-1.8125f,6.2012f,-1.8125f,6.2012f,-1.7083f,6.1523f,-1.7083f,6.1523f,-1.6042f,6.1035f,-1.6042f,6.1035f,-1.5000f,6.0547f,-1.5000f,6.0547f,-1.3958f,
	6.0059f,-1.3958f,6.0059f,-1.2917f,5.9570f,-1.2917f,5.9570f,-1.1875f,5.9082f,-1.1875f,5.9082f,-1.0833f,5.8594f,-1.0833f,5.8594f,-0.9792f,5.8105f,-0.9792f,5.8105f,-0.8750f,5.7617f,-0.8750f,5.7617f,-0.7708f,5.7129f,-0.7708f,5.7129f,-0.6667f,5.6641f,-0.6667f,
	5.6641f,-0.5625f,5.6152f,-0.5625f,5.6152f,-0.4583f,5.5664f,-0.4583f,5.5664f,-0.3542f,5.5176f,-0.3542f,5.5176f,-0.2500f,5.4688f,-0.2500f,5.4688f,-0.1458f,5.4199f,-0.1458f,5.4199f,-0.0417f,5.3711f,-0.0417f,5.3711f,0.0625f,5.3223f,0.0625f,5.3223f,0.1667f,
	5.2734f,0.1667f,5.2734f,0.2708f,5.2246f,0.2708f,5.2246f,0.3750f,5.1758f,0.3750f,5.1758f,0.4792f,5.1270f,0.4792f,5.1270f,0.5833f,5.0781f,0.5833f,5.0781f,0.6875f,5.0293f,0.6875f,5.0293f,0.7917f,4.9805f,0.7917f,4.9805f,0.8958f,4.9316f,0.8958f,
	4.9316f,1.0000f,4.8828f,1.0000f,7.2266f,-4.0000f,7.1777f,-4.0000f,7.1777f,-3.8958f,7.1289f,-3.8958f,7.1289f,-3.7917f,7.0801f,-3.7917f,7.0801f,-3.6875f,7.0313f,-3.6875f,7.0313f,-3.5833f,6.9824f,-3.5833f,6.9824f,-3.4792f,6.9336f,-3.4792f,6.9336f,-3.3750f,
	6.8848f,-3.3750f,6.8848f,-3.2708f,6.8359f,-3.2708f,6.8359f,-3.1667f,6.7871f,-3.1667f,6.7871f,-3.0625f,6.7383f,-3.0625f,6.7383f,-2.9583f,6.6895f,-2.9583f,6.6895f,-2.8542f,6.6406f,-2.8542f,6.6406f,-2.7500f,6.5918f,-2.7500f,6.5918f,-2.6458f,6.5430f,-2.6458f,
	6.5430f,-2.5417f,6.4941f,-2.5417f,6.4941f,-2.4375f,6.4453f,-2.4375f,6.4453f,-2.3333f,6.3965f,-2.3333f,6.3965f,-2.2292f,6.3477f,-2.2292f,6.3477f,-2.1250f,6.2988f,-2.1250f,6.2988f,-2.0208f,6.2500f,-2.0208f,6.2500f,-1.9167f,6.2012f,-1.9167f,6.2012f,-1.8125f,
	6.1523f,-1.8125f,6.1523f,-1.7083f,6.1035f,-1.7083f,6.1035f,-1.6042f,6.0547f,-1.6042f,6.0547f,-1.5000f,6.0059f,-1.5000f,6.0059f,-1.3958f,5.9570f,-1.3958f,5.9570f,-1.2917f,5.9082f,-1.2917f,5.9082f,-1.1875f,5.8594f,-1.1875f,5.8594f,-1.0833f,5.8105f,-1.0833f,
	5.8105f,-0.9792f,5.7617f,-0.9792f,5.7617f,-0.8750f,5.7129f,-0.8750f,5.7129f,-0.7708f,5.6641f,-0.7708f,5.6641f,-0.6667f,5.6152f,-0.6667f,5.6152f,-0.5625f,5.5664f,-0.5625f,5.5664f,-0.4583f,5.5176f,-0.4583f,5.5176f,-0.3542f,5.4688f,-0.3542f,5.4688f,-0.2500f,
	5.4199f,-0.2500f,5.4199f,-0.1458f,5.3711f,-0.1458f,5.3711f,-0.0417f,5.3223f,-0.0417f,5.3223f,0.0625f,5.2734f,0.0625f,5.2734f,0.1667f,5.2246f,0.1667f,5.2246f,0.2708f,5.1758f,0.2708f,5.1758f,0.3750f,5.1270f,0.3750f,5.1270f,0.4792f,5.0781f,0.4792f,
	5.0781f,0.5833f,5.0293f,0.5833f,5.0293f,0.6875f,4.9805f,0.6875f,4.9805f,0.7917f,4.9316f,0.7917f,4.9316f,0.8958f,4.8828f,0.8958f,4.8828f,1.0000f,4.8340f,1.0000f,7.1777f,-4.0000f,7.1289f,-4.0000f,7.1289f,-3.8958f,7.0801f,-3.8958f,7.0801f,-3.7917f,
	7.0313f,-3.7917f,7.0313f,-3.6875f,6.9824f,-3.6875f,6.9824f,-3.5833f,6.9336f,-3.5833f,6.9336f,-3.4792f,6.8848f,-3.4792f,6.8848f,-3.3750f,6.8359f,-3.3750f,6.8359f,-3.2708f,6.7871f,-3.2708f,6.7871f,-3.1667f,6.7383f,-3.1667f,6.7383f,-3.0625f,6.6895f,-3.0625f,
	6.6895f,-2.9583f,6.6406f,-2.9583f,6.6406f,-2.8542f,6.5918f,-2.8542f,6.5918f,-2.7500f,6.5430f,-2.7500f,6.5430f,-2.6458f,6.4941f,-2.6458f,6.4941f,-2.5417f,6.4453f,-2.5417f,6.4453f,-2.4375f,6.3965f,-2.4375f,6.3965f,-2.3333f,6.3477f,-2.3333f,6.3477f,-2.2292f,
	6.2988f,-2.2292f,6.2988f,-2.1250f,6.2500f,-2.1250f,6.2500f,-2.0208f,6.2012f,-2.0208f,6.2012f,-1.9167f,6.1523f,-1.9167f,6.1523f,-1.8125f,6.1035f,-1.8125f,6.1035f,-1.7083f,6.0547f,-1.7083f,6.0547f,-1.6042f,6.0059f,-1.6042f,6.0059f,-1.5000f,5.9570f,-1.5000f,
	5.9570f,-1.3958f,5.9082f,-1.3958f,5.9082f,-1.2917f,5.8594f,-1.2917f,5.8594f,-1.1875f,5.8105f,-1.1875f,5.8105f,-1.0833f,5.7617f,-1.0833f,5.7617f,-0.9792f,5.7129f,-0.9792f,5.7129f,-0.8750f,5.6641f,-0.8750f,5.6641f,-0.7708f,5.6152f,-0.7708f,5.6152f,-0.6667f,
	5.5664f,-0.6667f,5.5664f,-0.5625f,5.5176f,-0.5625f,5.5176f,-0.4583f,5.4688f,-0.4583f,5.4688f,-0.3542f,5.4199f,-0.3542f,5.4199f,-0.2500f,5.3711f,-0.2500f,5.3711f,-0.1458f,5.3223f,-0.1458f,5.3223f,-0.0417f,5.2734f,-0.0417f,5.2734f,0.0625f,5.2246f,0.0625f,
	5.2246f,0.1667f,5.1758f,0.1667f,5.1758f,0.2708f,5.1270f,0.2708f,5.1270f,0.3750f,5.0781f,0.3750f,5.0781f,0.4792f,5.0293f,0.4792f,5.0293f,0.5833f,4.9805f,0.5833f,4.9805f,0.6875f,4.9316f,0.6875f,4.9316f,0.7917f,4.8828f,0.7917f,4.8828f,0.8958f,
	4.8340f,0.8958f,4.8340f,1.0000f,4.7852f,1.0000f,7.1289f,-4.0000f,7.0801f,-4.0000f,7.0801f,-3.8958f,7.0313f,-3.8958f,7.0313f,-3.7917f,6.9824f,-3.7917f,6.9824f,-3.6875f,6.9336f,-3.6875f,6.9336f,-3.5833f,6.8848f,-3.5833f,6.8848f,-3.4792f,6.8359f,-3.4792f,
	6.8359f,-3.3750f,6.7871f,-3.3750f,6.7871f,-3.2708f,6.7383f,-3.2708f,6.7383f,-3.1667f,6.6895f,-3.1667f,6.6895f,-3.0625f,6.6406f,-3.0625f,6.6406f,-2.9583f,6.5918f,-2.9583f,6.5918f,-2.8542f,6.5430f,-2.8542f,6.5430f,-2.7500f,6.4941f,-2.7500f,6.4941f,-2.6458f,
	6.4453f,-2.6458f,6.4453f,-2.5417f,6.3965f,-2.5417f,6.3965f,-2.4375f,6.3477f,-2.4375f,6.3477f,-2.3333f,6.2988f,-2.3333f,6.2988f,-2.2292f,6.2500f,-2.2292f,6.2500f,-2.1250f,6.2012f,-2.1250f,6.2012f,-2.0208f,6.1523f,-2.0208f,6.1523f,-1.9167f,6.1035f,-1.9167f,
	6.1035f,-1.8125f,6.0547f,-1.8125f,6.0547f,-1.7083f,6.0059f,-1.7083f,6.0059f,-1.6042f,5.9570f,-1.6042f,5.9570f,-1.5000f,5.9082f,-1.5000f,5.9082f,-1.3958f,5.8594f,-1.3958f,5.8594f,-1.2917f,5.8105f,-1.2917f,5.8105f,-1.1875f,5.7617f,-1.1875f,5.7617f,-1.0833f,
	5.7129f,-1.0833f,5.7129f,-0.9792f,5.6641f,-0.9792f,5.6641f,-0.8750f,5.6152f,-0.8750f,5.6152f,-0.7708f,5.5664f,-0.7708f,5.5664f,-0.6667f,5.5176f,-0.6667f,5.5176f,-0.5625f,5.4688f,-0.5625f,5.4688f,-0.4583f,5.4199f,-0.4583f,5.4199f,-0.3542f,5.3711f,-0.3542f,
	5.3711f,-0.2500f,5.3223f,-0.2500f,5.3223f,-0.1458f,5.2734f,-0.1458f,5.2734f,-0.0417f,5.2246f,-0.0417f,5.2246f,0.0625f,5.1758f,0.0625f,5.1758f,0.1667f,5.1270f,0.1667f,5.1270f,0.2708f,5.0781f,0.2708f,5.0781f,0.3750f,5.0293f,0.3750f,5.0293f,0.4792f,
	4.9805f,0.4792f,4.9805f,0.5833f,4.9316f,0.5833f,4.9316f,0.6875f,4.8828f,0.6875f,4.8828f,0.7917f,4.8340f,0.7917f,4.8340f,0.8958f,4.7852f,0.8958f,4.7852f,1.0000f,4.7363f,1.0000f,7.0801f,-4.0000f,7.0313f,-4.0000f,7.0313f,-3.8958f,6.9824f,-3.8958f,
	6.9824f,-3.7917f,6.9336f,-3.7917f,6.9336f,-3.6875f,6.8848f,-3.6875f,6.8848f,-3.5833f,6.8359f,-3.5833f,6.8359f,-3.4792f,6.7871f,-3.4792f,6.7871f,-3.3750f,6.7383f,-3.3750f,6.7383f,-3.2708f,6.6895f,-3.2708f,6.6895f,-3.1667f,6.6406f,-3.1667f,6.6406f,-3.0625f,
	6.5918f,-3.0625f,6.5918f,-2.9583f,6.5430f,-2.9583f,6.5430f,-2.8542f,6.4941f,-2.8542f,6.4941f,-2.7500f,6.4453f,-2.7500f,6.4453f,-2.6458f,6.3965f,-2.6458f,6.3965f,-2.5417f,6.3477f,-2.5417f,6.3477f,-2.4375f,6.2988f,-2.4375f,6.2988f,-2.3333f,6.2500f,-2.3333f,
	6.2500f,-2.2292f,6.2012f,-2.2292f,6.2012f,-2.1250f,6.1523f,-2.1250f,6.1523f,-2.0208f,6.1035f,-2.0208f,6.1035f,-1.9167f,6.0547f,-1.9167f,6.0547f,-1.8125f,6.0059f,-1.8125f,6.0059f,-1.7083f,5.9570f,-1.7083f,5.9570f,-1.6042f,5.9082f,-1.6042f,5.9082f,-1.5000f,
	5.8594f,-1.5000f,5.8594f,-1.3958f,5.8105f,-1.3958f,5.8105f,-1.2917f,5.7617f,-1.2917f,5.7617f,-1.1875f,5.7129f,-1.1875f,5.7129f,-1.0833f,5.6641f,-1.0833f,5.6641f,-0.9792f,5.6152f,-0.9792f,5.6152f,-0.8750f,5.5664f,-0.8750f,5.5664f,-0.7708f,5.5176f,-0.7708f,
	5.5176f,-0.6667f,5.4688f,-0.6667f,5.4688f,-0.5625f,5.4199f,-0.5625f,5.4199f,-0.4583f,5.3711f,-0.4583f,5.3711f,-0.3542f,5.3223f,-0.3542f,5.3223f,-0.2500f,5.2734f,-0.2500f,5.2734f,-0.1458f,5.2246f,-0.1458f,5.2246f,-0.0417f,5.1758f,-0.0417f,5.1758f,0.0625f,
	5.1270f,0.0625f,5.1270f,0.1667f,5.0781f,0.1667f,5.0781f,0.2708f,5.0293f,0.2708f,5.0293f,0.3750f,4.9805f,0.3750f,4.9805f,0.4792f,4.9316f,0.4792f,4.9316f,0.5833f,4.8828f,0.5833f,4.8828f,0.6875f,4.8340f,0.6875f,4.8340f,0.7917f,4.7852f,0.7917f,
	4.7852f,0.8958f,4.7363f,0.8958f,4.7363f,1.0000f,4.6875f,1.0000f,7.0313f,-4.0000f,6.9824f,-4.0000f,6.9824f,-3.8958f,6.9336f,-3.8958f,6.9336f,-3.7917f,6.8848f,-3.7917f,6.8848f,-3.6875f,6.8359f,-3.6875f,6.8359f,-3.5833f,6.7871f,-3.5833f,6.7871f,-3.4792f,
	6.7383f,-3.4792f,6.7383f,-3.3750f,6.6895f,-3.3750f,6.6895f,-3.2708f,6.6406f,-3.2708f,6.6406f,-3.1667f,6.5918f,-3.1667f,6.5918f,-3.0625f,6.5430f,-3.0625f,6.5430f,-2.9583f,6.4941f,-2.9583f,6.4941f,-2.8542f,6.4453f,-2.8542f,6.4453f,-2.7500f,6.3965f,-2.7500f,
	6.3965f,-2.6458f,6.3477f,-2.6458f,6.3477f,-2.5417f,6.2988f,-2.5417f,6.2988f,-2.4375f,6.2500f,-2.4375f,6.2500f,-2.3333f,6.2012f,-2.3333f,6.2012f,-2.2292f,6.1523f,-2.2292f,6.1523f,-2.1250f,6.1035f,-2.1250f,6.1035f,-2.0208f,6.0547f,-2.0208f,6.0547f,-1.9167f,
	6.0059f,-1.9167f,6.0059f,-1.8125f,5.9570f,-1.8125f,5.9570f,-1.7083f,5.9082f,-1.7083f,5.9082f,-1.6042f,5.8594f,-1.6042f,5.8594f,-1.5000f,5.8105f,-1.5000f,5.8105f,-1.3958f,5.7617f,-1.3958f,5.7617f,-1.2917f,5.7129f,-1.2917f,5.7129f,-1.1875f,5.6641f,-1.1875f,
	5.6641f,-1.0833f,5.6152f,-1.0833f,5.6152f,-0.9792f,5.5664f,-0.9792f,5.5664f,-0.8750f,5.5176f,-0.8750f,5.5176f,-0.7708f,5.4688f,-0.7708f,5.4688f,-0.6667f,5.4199f,-0.6667f,5.4199f,-0.5625f,5.3711f,-0.5625f,5.3711f,-0.4583f,5.3223f,-0.4583f,5.3223f,-0.3542f,
	5.2734f,-0.3542f,5.2734f,-0.2500f,5.2246f,-0.2500f,5.2246f,-0.1458f,5.1758f,-0.1458f,5.1758f,-0.0417f,5.1270f,-0.0417f,5.1270f,0.0625f,5.0781f,0.0625f,5.0781f,0.1667f,5.0293f,0.1667f,5.0293f,0.2708f,4.9805f,0.2708f,4.9805f,0.3750f,4.9316f,0.3750f,
	4.9316f,0.4792f,4.8828f,0.4792f,4.8828f,0.5833f,4.8340f,0.5833f,4.8340f,0.6875f,4.7852f,0.6875f,4.7852f,0.7917f,4.7363f,0.7917f,4.7363f,0.8958f,4.6875f,0.8958f,4.6875f,1.0000f,4.6387f,1.0000f,6.9824f,-4.0000f,6.9336f,-4.0000f,6.9336f,-3.8958f,
	6.8848f,-3.8958f,6.8848f,-3.7917f,6.8359f,-3.7917f,6.8359f,-3.6875f,6.7871f,-3.6875f,6.7871f,-3.5833f,6.7383f,-3.5833f,6.7383f,-3.4792f,6.6895f,-3.4792f,6.6895f,-3.3750f,6.6406f,-3.3750f,6.6406f,-3.2708f,6.5918f,-3.2708f,6.5918f,-3.1667f,6.5430f,-3.1667f,
	6.5430f,-3.0625f,6.4941f,-3.0625f,6.4941f,-2.9583f,6.4453f,-2.9583f,6.4453f,-2.8542f,6.3965f,-2.8542f,6.3965f,-2.7500f,6.3477f,-2.7500f,6.3477f,-2.6458f,6.2988f,-2.6458f,6.2988f,-2.5417f,6.2500f,-2.5417f,6.2500f,-2.4375f,6.2012f,-2.4375f,6.2012f,-2.3333f,
	6.1523f,-2.3333f,6.1523f,-2.2292f,6.1035f,-2.2292f,6.1035f,-2.1250f,6.0547f,-2.1250f,6.0547f,-2.0208f,6.0059f,-2.0208f,6.0059f,-1.9167f,5.9570f,-1.9167f,5.9570f,-1.8125f,5.9082f,-1.8125f,5.9082f,-1.7083f,5.8594f,-1.7083f,5.8594f,-1.6042f,5.8105f,-1.6042f,
	5.8105f,-1.5000f,5.7617f,-1.5000f,5.7617f,-1.3958f,5.7129f,-1.3958f,5.7129f,-1.2917f,5.6641f,-1.2917f,5.6641f,-1.1875f,5.6152f,-1.1875f,5.6152f,-1.0833f,5.5664f,-1.0833f,5.5664f,-0.9792f,5.5176f,-0.9792f,5.5176f,-0.8750f,5.4688f,-0.8750f,5.4688f,-0.7708f,
	5.4199f,-0.7708f,5.4199f,-0.6667f,5.3711f,-0.6667f,5.3711f,-0.5625f,5.3223f,-0.5625f,5.3223f,-0.4583f,5.2734f,-0.4583f,5.2734f,-0.3542f,5.2246f,-0.3542f,5.2246f,-0.2500f,5.1758f,-0.2500f,5.1758f,-0.1458f,5.1270f,-0.1458f,5.1270f,-0.0417f,5.0781f,-0.0417f,
	5.0781f,0.0625f,5.0293f,0.0625f,5.0293f,0.1667f,4.9805f,0.1667f,4.9805f,0.2708f,4.9316f,0.2708f,4.9316f,0.3750f,4.8828f,0.3750f,4.8828f,0.4792f,4.8340f,0.4792f,4.8340f,0.5833f,4.7852f,0.5833f,4.7852f,0.6875f,4.7363f,0.6875f,4.7363f,0.7917f,
	4.6875f,0.7917f,4.6875f,0.8958f,4.6387f,0.8958f,4.6387f,1.0000f,4.5898f,1.0000f,6.9336f,-4.0000f,6.8848f,-4.0000f,6.8848f,-3.8958f,6.8359f,-3.8958f,6.8359f,-3.7917f,6.7871f,-3.7917f,6.7871f,-3.6875f,6.7383f,-3.6875f,6.7383f,-3.5833f,6.6895f,-3.5833f,
	6.6895f,-3.4792f,6.6406f,-3.4792f,6.6406f,-3.3750f,6.5918f,-3.3750f,6.5918f,-3.2708f,6.5430f,-3.2708f,6.5430f,-3.1667f,6.4941f,-3.1667f,6.4941f,-3.0625f,6.4453f,-3.0625f,6.4453f,-2.9583f,6.3965f,-2.9583f,6.3965f,-2.8542f,6.3477f,-2.8542f,6.3477f,-2.7500f,
	6.2988f,-2.7500f,6.2988f,-2.6458f,6.2500f,-2.6458f,6.2500f,-2.5417f,6.2012f,-2.5417f,6.2012f,-2.4375f,6.1523f,-2.4375f,6.1523f,-2.3333f,6.1035f,-2.3333f,6.1035f,-2.2292f,6.0547f,-2.2292f,6.0547f,-2.1250f,6.0059f,-2.1250f,6.0059f,-2.0208f,5.9570f,-2.0208f,
	5.9570f,-1.9167f,5.9082f,-1.9167f,5.9082f,-1.8125f,5.8594f,-1.8125f,5.8594f,-1.7083f,5.8105f,-1.7083f,5.8105f,-1.6042f,5.7617f,-1.6042f,5.7617f,-1.5000f,5.7129f,-1.5000f,5.7129f,-1.3958f,5.6641f,-1.3958f,5.6641f,-1.2917f,5.6152f,-1.2917f,5.6152f,-1.1875f,
	5.5664f,-1.1875f,5.5664f,-1.0833f,5.5176f,-1.0833f,5.5176f,-0.9792f,5.4688f,-0.9792f,5.4688f,-0.8750f,5.4199f,-0.8750f,5.4199f,-0.7708f,5.3711f,-0.7708f,5.3711f,-0.6667f,5.3223f,-0.6667f,5.3223f,-0.5625f,5.2734f,-0.5625f,5.2734f,-0.4583f,5.2246f,-0.4583f,
	5.2246f,-0.3542f,5.1758f,-0.3542f,5.1758f,-0.2500f,5.1270f,-0.2500f,5.1270f,-0.1458f,5.0781f,-0.1458f,5.0781f,-0.0417f,5.0293f,-0.0417f,5.0293f,0.0625f,4.9805f,0.0625f,4.9805f,0.1667f,4.9316f,0.1667f,4.9316f,0.2708f,4.8828f,0.2708f,4.8828f,0.3750f,
	4.8340f,0.3750f,4.8340f,0.4792f,4.7852f,0.4792f,4.7852f,0.5833f,4.7363f,0.5833f,4.7363f,0.6875f,4.6875f,0.6875f,4.6875f,0.7917f,4.6387f,0.7917f,4.6387f,0.8958f,4.5898f,0.8958f,4.5898f,1.0000f,4.5410f,1.0000f,6.8848f,-4.0000f,6.8359f,-4.0000f,
	6.8359f,-3.8958f,6.7871f,-3.8958f,6.7871f,-3.7917f,6.7383f,-3.7917f,6.7383f,-3.6875f,6.6895f,-3.6875f,6.6895f,-3.5833f,6.6406f,-3.5833f,6.6406f,-3.4792f,6.5918f,-3.4792f,6.5918f,-3.3750f,6.5430f,-3.3750f,6.5430f,-3.2708f,6.4941f,-3.2708f,6.4941f,-3.1667f,
	6.4453f,-3.1667f,6.4453f,-3.0625f,6.3965f,-3.0625f,6.3965f,-2.9583f,6.3477f,-2.9583f,6.3477f,-2.8542f,6.2988f,-2.8542f,6.2988f,-2.7500f,6.2500f,-2.7500f,6.2500f,-2.6458f,6.2012f,-2.6458f,6.2012f,-2.5417f,6.1523f,-2.5417f,6.1523f,-2.4375f,6.1035f,-2.4375f,
	6.1035f,-2.3333f,6.0547f,-2.3333f,6.0547f,-2.2292f,6.0059f,-2.2292f,6.0059f,-2.1250f,5.9570f,-2.1250f,5.9570f,-2.0208f,5.9082f,-2.0208f,5.9082f,-1.9167f,5.8594f,-1.9167f,5.8594f,-1.8125f,5.8105f,-1.8125f,5.8105f,-1.7083f,5.7617f,-1.7083f,5.7617f,-1.6042f,
	5.7129f,-1.6042f,5.7129f,-1.5000f,5.6641f,-1.5000f,5.6641f,-1.3958f,5.6152f,-1.3958f,5.6152f,-1.2917f,5.5664f,-1.2917f,5.5664f,-1.1875f,5.5176f,-1.1875f,5.5176f,-1.0833f,5.4688f,-1.0833f,5.4688f,-0.9792f,5.4199f,-0.9792f,5.4199f,-0.8750f,5.3711f,-0.8750f,
	5.3711f,-0.7708f,5.3223f,-0.7708f,5.3223f,-0.6667f,5.2734f,-0.6667f,5.2734f,-0.5625f,5.2246f,-0.5625f,5.2246f,-0.4583f,5.1758f,-0.4583f,5.1758f,-0.3542f,5.1270f,-0.3542f,5.1270f,-0.2500f,5.0781f,-0.2500f,5.0781f,-0.1458f,5.0293f,-0.1458f,5.0293f,-0.0417f,
	4.9805f,-0.0417f,4.9805f,0.0625f,4.9316f,0.0625f,4.9316f,0.1667f,4.8828f,0.1667f,4.8828f,0.2708f,4.8340f,0.2708f,4.8340f,0.3750f,4.7852f,0.3750f,4.7852f,0.4792f,4.7363f,0.4792f,4.7363f,0.5833f,4.6875f,0.5833f,4.6875f,0.6875f,4.6387f,0.6875f,
	4.6387f,0.7917f,4.5898f,0.7917f,4.5898f,0.8958f,4.5410f,0.8958f,4.5410f,1.0000f,4.4922f,1.0000f,6.8359f,-4.0000f,6.7871f,-4.0000f,6.7871f,-3.8958f,6.7383f,-3.8958f,6.7383f,-3.7917f,6.6895f,-3.7917f,6.6895f,-3.6875f,6.6406f,-3.6875f,6.6406f,-3.5833f,
	6.5918f,-3.5833f,6.5918f,-3.4792f,6.5430f,-3.4792f,6.5430f,-3.3750f,6.4941f,-3.3750f,6.4941f,-3.2708f,6.4453f,-3.2708f,6.4453f,-3.1667f,6.3965f,-3.1667f,6.3965f,-3.0625f,6.3477f,-3.0625f,6.3477f,-2.9583f,6.2988f,-2.9583f,6.2988f,-2.8542f,6.2500f,-2.8542f,
	6.2500f,-2.7500f,6.2012f,-2.7500f,6.2012f,-2.6458f,6.1523f,-2.6458f,6.1523f,-2.5417f,6.1035f,-2.5417f,6.1035f,-2.4375f,6.0547f,-2.4375f,6.0547f,-2.3333f,6.0059f,-2.3333f,6.0059f,-2.2292f,5.9570f,-2.2292f,5.9570f,-2.1250f,5.9082f,-2.1250f,5.9082f,-2.0208f,
	5.8594f,-2.0208f,5.8594f,-1.9167f,5.8105f,-1.9167f,5.8105f,-1.8125f,5.7617f,-1.8125f,5.7617f,-1.7083f,5.7129f,-1.7083f,5.7129f,-1.6042f,5.6641f,-1.6042f,5.6641f,-1.5000f,5.6152f,-1.5000f,5.6152f,-1.3958f,5.5664f,-1.3958f,5.5664f,-1.2917f,5.5176f,-1.2917f,
	5.5176f,-1.1875f,5.4688f,-1.1875f,5.4688f,-1.0833f,5.4199f,-1.0833f,5.4199f,-0.9792f,5.3711f,-0.9792f,5.3711f,-0.8750f,5.3223f,-0.8750f,5.3223f,-0.7708f,5.2734f,-0.7708f,5.2734f,-0.6667f,5.2246f,-0.6667f,5.2246f,-0.5625f,5.1758f,-0.5625f,5.1758f,-0.4583f,
	5.1270f,-0.4583f,5.1270f,-0.3542f,5.0781f,-0.3542f,5.0781f,-0.2500f,5.0293f,-0.2500f,5.0293f,-0.1458f,4.9805f,-0.1458f,4.9805f,-0.0417f,4.9316f,-0.0417f,4.9316f,0.0625f,4.8828f,0.0625f,4.8828f,0.1667f,4.8340f,0.1667f,4.8340f,0.2708f,4.7852f,0.2708f,
	4.7852f,0.3750f,4.7363f,0.3750f,4.7363f,0.4792f,4.6875f,0.4792f,4.6875f,0.5833f,4.6387f,0.5833f,4.6387f,0.6875f,4.5898f,0.6875f,4.5898f,0.7917f,4.5410f,0.7917f,4.5410f,0.8958f,4.4922f,0.8958f,4.4922f,1.0000f,4.4434f,1.0000f,6.7871f,-4.0000f,
	6.7383f,-4.0000f,6.7383f,-3.8958f,6.6895f,-3.8958f,6.6895f,-3.7917f,6.6406f,-3.7917f,6.6406f,-3.6875f,6.5918f,-3.6875f,6.5918f,-3.5833f,6.5430f,-3.5833f,6.5430f,-3.4792f,6.4941f,-3.4792f,6.4941f,-3.3750f,6.4453f,-3.3750f,6.4453f,-3.2708f,6.3965f,-3.2708f,
	6.3965f,-3.1667f,6.3477f,-3.1667f,6.3477f,-3.0625f,6.2988f,-3.0625f,6.2988f,-2.9583f,6.2500f,-2.9583f,6.2500f,-2.8542f,6.2012f,-2.8542f,6.2012f,-2.7500f,6.1523f,-2.7500f,6.1523f,-2.6458f,6.1035f,-2.6458f,6.1035f,-2.5417f,6.0547f,-2.5417f,6.0547f,-2.4375f,
	6.0059f,-2.4375f,6.0059f,-2.3333f,5.9570f,-2.3333f,5.9570f,-2.2292f,5.9082f,-2.2292f,5.9082f,-2.1250f,5.8594f,-2.1250f,5.8594f,-2.0208f,5.8105f,-2.0208f,5.8105f,-1.9167f,5.7617f,-1.9167f,5.7617f,-1.8125f,5.7129f,-1.8125f,5.7129f,-1.7083f,5.6641f,-1.7083f,
	5.6641f,-1.6042f,5.6152f,-1.6042f,5.6152f,-1.5000f,5.5664f,-1.5000f,5.5664f,-1.3958f,5.5176f,-1.3958f,5.5176f,-1.2917f,5.4688f,-1.2917f,5.4688f,-1.1875f,5.4199f,-1.1875f,5.4199f,-1.0833f,5.3711f,-1.0833f,5.3711f,-0.9792f,5.3223f,-0.9792f,5.3223f,-0.8750f,
	5.2734f,-0.8750f,5.2734f,-0.7708f,5.2246f,-0.7708f,5.2246f,-0.6667f,5.1758f,-0.6667f,5.1758f,-0.5625f,5.1270f,-0.5625f,5.1270f,-0.4583f,5.0781f,-0.4583f,5.0781f,-0.3542f,5.0293f,-0.3542f,5.0293f,-0.2500f,4.9805f,-0.2500f,4.9805f,-0.1458f,4.9316f,-0.1458f,
	4.9316f,-0.0417f,4.8828f,-0.0417f,4.8828f,0.0625f,4.8340f,0.0625f,4.8340f,0.1667f,4.7852f,0.1667f,4.7852f,0.2708f,4.7363f,0.2708f,4.7363f,0.3750f,4.6875f,0.3750f,4.6875f,0.4792f,4.6387f,0.4792f,4.6387f,0.5833f,4.5898f,0.5833f,4.5898f,0.6875f,
	4.5410f,0.6875f,4.5410f,0.7917f,4.4922f,0.7917f,4.4922f,0.8958f,4.4434f,0.8958f,4.4434f,1.0000f,4.3945f,1.0000f,6.7383f,-4.0000f,6.6895f,-4.0000f,6.6895f,-3.8958f,6.6406f,-3.8958f,6.6406f,-3.7917f,6.5918f,-3.7917f,6.5918f,-3.6875f,6.5430f,-3.6875f,
	6.5430f,-3.5833f,6.4941f,-3.5833f,6.4941f,-3.4792f,6.4453f,-3.4792f,6.4453f,-3.3750f,6.3965f,-3.3750f,6.3965f,-3.2708f,6.3477f,-3.2708f,6.3477f,-3.1667f,6.2988f,-3.1667f,6.2988f,-3.0625f,6.2500f,-3.0625f,6.2500f,-2.9583f,6.2012f,-2.9583f,6.2012f,-2.8542f,
	6.1523f,-2.8542f,6.1523f,-2.7500f,6.1035f,-2.7500f,6.1035f,-2.6458f,6.0547f,-2.6458f,6.0547f,-2.5417f,6.0059f,-2.5417f,6.0059f,-2.4375f,5.9570f,-2.4375f,5.9570f,-2.3333f,5.9082f,-2.3333f,5.9082f,-2.2292f,5.8594f,-2.2292f,5.8594f,-2.1250f,5.8105f,-2.1250f,
	5.8105f,-2.0208f,5.7617f,-2.0208f,5.7617f,-1.9167f,5.7129f,-1.9167f,5.7129f,-1.8125f,5.6641f,-1.8125f,5.6641f,-1.7083f,5.6152f,-1.7083f,5.6152f,-1.6042f,5.5664f,-1.6042f,5.5664f,-1.5000f,5.5176f,-1.5000f,5.5176f,-1.3958f,5.4688f,-1.3958f,5.4688f,-1.2917f,
	5.4199f,-1.2917f,5.4199f,-1.1875f,5.3711f,-1.1875f,5.3711f,-1.0833f,5.3223f,-1.0833f,5.3223f,-0.9792f,5.2734f,-0.9792f,5.2734f,-0.8750f,5.2246f,-0.8750f,5.2246f,-0.7708f,5.1758f,-0.7708f,5.1758f,-0.6667f,5.1270f,-0.6667f,5.1270f,-0.5625f,5.0781f,-0.5625f,
	5.0781f,-0.4583f,5.0293f,-0.4583f,5.0293f,-0.3542f,4.9805f,-0.3542f,4.9805f,-0.2500f,4.9316f,-0.2500f,4.9316f,-0.1458f,4.8828f,-0.1458f,4.8828f,-0.0417f,4.8340f,-0.0417f,4.8340f,0.0625f,4.7852f,0.0625f,4.7852f,0.1667f,4.7363f,0.1667f,4.7363f,0.2708f,
	4.6875f,0.2708f,4.6875f,0.3750f,4.6387f,0.3750f,4.6387f,0.4792f,4.5898f,0.4792f,4.5898f,0.5833f,4.5410f,0.5833f,4.5410f,0.6875f,4.4922f,0.6875f,4.4922f,0.7917f,4.4434f,0.7917f,4.4434f,0.8958f,4.3945f,0.8958f,4.3945f,1.0000f,4.3457f,1.0000f,
	6.6895f,-4.0000f,6.6406f,-4.0000f,6.6406f,-3.8958f,6.5918f,-3.8958f,6.5918f,-3.7917f,6.5430f,-3.7917f,6.5430f,-3.6875f,6.4941f,-3.6875f,6.4941f,-3.5833f,6.4453f,-3.5833f,6.4453f,-3.4792f,6.3965f,-3.4792f,6.3965f,-3.3750f,6.3477f,-3.3750f,6.3477f,-3.2708f,
	6.2988f,-3.2708f,6.2988f,-3.1667f,6.2500f,-3.1667f,6.2500f,-3.0625f,6.2012f,-3.0625f,6.2012f,-2.9583f,6.1523f,-2.9583f,6.1523f,-2.8542f,6.1035f,-2.8542f,6.1035f,-2.7500f,6.0547f,-2.7500f,6.0547f,-2.6458f,6.0059f,-2.6458f,6.0059f,-2.5417f,5.9570f,-2.5417f,
	5.9570f,-2.4375f,5.9082f,-2.4375f,5.9082f,-2.3333f,5.8594f,-2.3333f,5.8594f,-2.2292f,5.8105f,-2.2292f,5.8105f,-2.1250f,5.7617f,-2.1250f,5.7617f,-2.0208f,5.7129f,-2.0208f,5.7129f,-1.9167f,5.6641f,-1.9167f,5.6641f,-1.8125f,5.6152f,-1.8125f,5.6152f,-1.7083f,
	5.5664f,-1.7083f,5.5664f,-1.6042f,5.5176f,-1.6042f,5.5176f,-1.5000f,5.4688f,-1.5000f,5.4688f,-1.3958f,5.4199f,-1.3958f,5.4199f,-1.2917f,5.3711f,-1.2917f,5.3711f,-1.1875f,5.3223f,-1.1875f,5.3223f,-1.0833f,5.2734f,-1.0833f,5.2734f,-0.9792f,5.2246f,-0.9792f,
	5.2246f,-0.8750f,5.1758f,-0.8750f,5.1758f,-0.7708f,5.1270f,-0.7708f,5.1270f,-0.6667f,5.0781f,-0.6667f,5.0781f,-0.5625f,5.0293f,-0.5625f,5.0293f,-0.4583f,4.9805f,-0.4583f,4.9805f,-0.3542f,4.9316f,-0.3542f,4.9316f,-0.2500f,4.8828f,-0.2500f,4.8828f,-0.1458f,
	4.8340f,-0.1458f,4.8340f,-0.0417f,4.7852f,-0.0417f,4.7852f,0.0625f,4.7363f,0.0625f,4.7363f,0.1667f,4.6875f,0.1667f,4.6875f,0.2708f,4.6387f,0.2708f,4.6387f,0.3750f,4.5898f,0.3750f,4.5898f,0.4792f,4.5410f,0.4792f,4.5410f,0.5833f,4.4922f,0.5833f,
	4.4922f,0.6875f,4.4434f,0.6875f,4.4434f,0.7917f,4.3945f,0.7917f,4.3945f,0.8958f,4.3457f,0.8958f,4.3457f,1.0000f,4.2969f,1.0000f,6.6406f,-4.0000f,6.5918f,-4.0000f,6.5918f,-3.8958f,6.5430f,-3.8958f,6.5430f,-3.7917f,6.4941f,-3.7917f,6.4941f,-3.6875f,
	6.4453f,-3.6875f,6.4453f,-3.5833f,6.3965f,-3.5833f,6.3965f,-3.4792f,6.3477f,-3.4792f,6.3477f,-3.3750f,6.2988f,-3.3750f,6.2988f,-3.2708f,6.2500f,-3.2708f,6.2500f,-3.1667f,6.2012f,-3.1667f,6.2012f,-3.0625f,6.1523f,-3.0625f,6.1523f,-2.9583f,6.1035f,-2.9583f,
	6.1035f,-2.8542f,6.0547f,-2.8542f,6.0547f,-2.7500f,6.0059f,-2.7500f,6.0059f,-2.6458f,5.9570f,-2.6458f,5.9570f,-2.5417f,5.9082f,-2.5417f,5.9082f,-2.4375f,5.8594f,-2.4375f,5.8594f,-2.3333f,5.8105f,-2.3333f,5.8105f,-2.2292f,5.7617f,-2.2292f,5.7617f,-2.1250f,
	5.7129f,-2.1250f,5.7129f,-2.0208f,5.6641f,-2.0208f,5.6641f,-1.9167f,5.6152f,-1.9167f,5.6152f,-1.8125f,5.5664f,-1.8125f,5.5664f,-1.7083f,5.5176f,-1.7083f,5.5176f,-1.6042f,5.4688f,-1.6042f,5.4688f,-1.5000f,5.4199f,-1.5000f,5.4199f,-1.3958f,5.3711f,-1.3958f,
	5.3711f,-1.2917f,5.3223f,-1.2917f,5.3223f,-1.1875f,5.2734f,-1.1875f,5.2734f,-1.0833f,5.2246f,-1.0833f,5.2246f,-0.9792f,5.1758f,-0.9792f,5.1758f,-0.8750f,5.1270f,-0.8750f,5.1270f,-0.7708f,5.0781f,-0.7708f,5.0781f,-0.6667f,5.0293f,-0.6667f,5.0293f,-0.5625f,
	4.9805f,-0.5625f,4.9805f,-0.4583f,4.9316f,-0.4583f,4.9316f,-0.3542f,4.8828f,-0.3542f,4.8828f,-0.2500f,4.8340f,-0.2500f,4.8340f,-0.1458f,4.7852f,-0.1458f,4.7852f,-0.0417f,4.7363f,-0.0417f,4.7363f,0.0625f,4.6875f,0.0625f,4.6875f,0.1667f,4.6387f,0.1667f,
	4.6387f,0.2708f,4.5898f,0.2708f,4.5898f,0.3750f,4.5410f,0.3750f,4.5410f,0.4792f,4.4922f,0.4792f,4.4922f,0.5833f,4.4434f,0.5833f,4.4434f,0.6875f,4.3945f,0.6875f,4.3945f,0.7917f,4.3457f,0.7917f,4.3457f,0.8958f,4.2969f,0.8958f,4.2969f,1.0000f,
	4.2480f,1.0000f,6.5918f,-4.0000f,6.5430f,-4.0000f,6.5430f,-3.8958f,6.4941f,-3.8958f,6.4941f,-3.7917f,6.4453f,-3.7917f,6.4453f,-3.6875f,6.3965f,-3.6875f,6.3965f,-3.5833f,6.3477f,-3.5833f,6.3477f,-3.4792f,6.2988f,-3.4792f,6.2988f,-3.3750f,6.2500f,-3.3750f,
	6.2500f,-3.2708f,6.2012f,-3.2708f,6.2012f,-3.1667f,6.1523f,-3.1667f,6.1523f,-3.0625f,6.1035f,-3.0625f,6.1035f,-2.9583f,6.0547f,-2.9583f,6.0547f,-2.8542f,6.0059f,-2.8542f,6.0059f,-2.7500f,5.9570f,-2.7500f,5.9570f,-2.6458f,5.9082f,-2.6458f,5.9082f,-2.5417f,
	5.8594f,-2.5417f,5.8594f,-2.4375f,5.8105f,-2.4375f,5.8105f,-2.3333f,5.7617f,-2.3333f,5.7617f,-2.2292f,5.7129f,-2.2292f,5.7129f,-2.1250f,5.6641f,-2.1250f,5.6641f,-2.0208f,5.6152f,-2.0208f,5.6152f,-1.9167f,5.5664f,-1.9167f,5.5664f,-1.8125f,5.5176f,-1.8125f,
	5.5176f,-1.7083f,5.4688f,-1.7083f,5.4688f,-1.6042f,5.4199f,-1.6042f,5.4199f,-1.5000f,5.3711f,-1.5000f,5.3711f,-1.3958f,5.3223f,-1.3958f,5.3223f,-1.2917f,5.2734f,-1.2917f,5.2734f,-1.1875f,5.2246f,-1.1875f,5.2246f,-1.0833f,5.1758f,-1.0833f,5.1758f,-0.9792f,
	5.1270f,-0.9792f,5.1270f,-0.8750f,5.0781f,-0.8750f,5.0781f,-0.7708f,5.0293f,-0.7708f,5.0293f,-0.6667f,4.9805f,-0.6667f,4.9805f,-0.5625f,4.9316f,-0.5625f,4.9316f,-0.4583f,4.8828f,-0.4583f,4.8828f,-0.3542f,4.8340f,-0.3542f,4.8340f,-0.2500f,4.7852f,-0.2500f,
	4.7852f,-0.1458f,4.7363f,-0.1458f,4.7363f,-0.0417f,4.6875f,-0.0417f,4.6875f,0.0625f,4.6387f,0.0625f,4.6387f,0.1667f,4.5898f,0.1667f,4.5898f,0.2708f,4.5410f,0.2708f,4.5410f,0.3750f,4.4922f,0.3750f,4.4922f,0.4792f,4.4434f,0.4792f,4.4434f,0.5833f,
	4.3945f,0.5833f,4.3945f,0.6875f,4.3457f,0.6875f,4.3457f,0.7917f,4.2969f,0.7917f,4.2969f,0.8958f,4.2480f,0.8958f,4.2480f,1.0000f,4.1992f,1.0000f,6.5430f,-4.0000f,6.4941f,-4.0000f,6.4941f,-3.8958f,6.4453f,-3.8958f,6.4453f,-3.7917f,6.3965f,-3.7917f,
	6.3965f,-3.6875f,6.3477f,-3.6875f,6.3477f,-3.5833f,6.2988f,-3.5833f,6.2988f,-3.4792f,6.2500f,-3.4792f,6.2500f,-3.3750f,6.2012f,-3.3750f,6.2012f,-3.2708f,6.1523f,-3.2708f,6.1523f,-3.1667f,6.1035f,-3.1667f,6.1035f,-3.0625f,6.0547f,-3.0625f,6.0547f,-2.9583f,
	6.0059f,-2.9583f,6.0059f,-2.8542f,5.9570f,-2.8542f,5.9570f,-2.7500f,5.9082f,-2.7500f,5.9082f,-2.6458f,5.8594f,-2.6458f,5.8594f,-2.5417f,5.8105f,-2.5417f,5.8105f,-2.4375f,5.7617f,-2.4375f,5.7617f,-2.3333f,5.7129f,-2.3333f,5.7129f,-2.2292f,5.6641f,-2.2292f,
	5.6641f,-2.1250f,5.6152f,-2.1250f,5.6152f,-2.0208f,5.5664f,-2.0208f,5.5664f,-1.9167f,5.5176f,-1.9167f,5.5176f,-1.8125f,5.4688f,-1.8125f,5.4688f,-1.7083f,5.4199f,-1.7083f,5.4199f,-1.6042f,5.3711f,-1.6042f,5.3711f,-1.5000f,5.3223f,-1.5000f,5.3223f,-1.3958f,
	5.2734f,-1.3958f,5.2734f,-1.2917f,5.2246f,-1.2917f,5.2246f,-1.1875f,5.1758f,-1.1875f,5.1758f,-1.0833f,5.1270f,-1.0833f,5.1270f,-0.9792f,5.0781f,-0.9792f,5.0781f,-0.8750f,5.0293f,-0.8750f,5.0293f,-0.7708f,4.9805f,-0.7708f,4.9805f,-0.6667f,4.9316f,-0.6667f,
	4.9316f,-0.5625f,4.8828f,-0.5625f,4.8828f,-0.4583f,4.8340f,-0.4583f,4.8340f,-0.3542f,4.7852f,-0.3542f,4.7852f,-0.2500f,4.7363f,-0.2500f,4.7363f,-0.1458f,4.6875f,-0.1458f,4.6875f,-0.0417f,4.6387f,-0.0417f,4.6387f,0.0625f,4.5898f,0.0625f,4.5898f,0.1667f,
	4.5410f,0.1667f,4.5410f,0.2708f,4.4922f,0.2708f,4.4922f,0.3750f,4.4434f,0.3750f,4.4434f,0.4792f,4.3945f,0.4792f,4.3945f,0.5833f,4.3457f,0.5833f,4.3457f,0.6875f,4.2969f,0.6875f,4.2969f,0.7917f,4.2480f,0.7917f,4.2480f,0.8958f,4.1992f,0.8958f,
	4.1992f,1.0000f,4.1504f,1.0000f,6.4941f,-4.0000f,6.4453f,-4.0000f,6.4453f,-3.8958f,6.3965f,-3.8958f,6.3965f,-3.7917f,6.3477f,-3.7917f,6.3477f,-3.6875f,6.2988f,-3.6875f,6.2988f,-3.5833f,6.2500f,-3.5833f,6.2500f,-3.4792f,6.2012f,-3.4792f,6.2012f,-3.3750f,
	6.1523f,-3.3750f,6.1523f,-3.2708f,6.1035f,-3.2708f,6.1035f,-3.1667f,6.0547f,-3.1667f,6.0547f,-3.0625f,6.0059f,-3.0625f,6.0059f,-2.9583f,5.9570f,-2.9583f,5.9570f,-2.8542f,5.9082f,-2.8542f,5.9082f,-2.7500f,5.8594f,-2.7500f,5.8594f,-2.6458f,5.8105f,-2.6458f,
	5.8105f,-2.5417f,5.7617f,-2.5417f,5.7617f,-2.4375f,5.7129f,-2.4375f,5.7129f,-2.3333f,5.6641f,-2.3333f,5.6641f,-2.2292f,5.6152f,-2.2292f,5.6152f,-2.1250f,5.5664f,-2.1250f,5.5664f,-2.0208f,5.5176f,-2.0208f,5.5176f,-1.9167f,5.4688f,-1.9167f,5.4688f,-1.8125f,
	5.4199f,-1.8125f,5.4199f,-1.7083f,5.3711f,-1.7083f,5.3711f,-1.6042f,5.3223f,-1.6042f,5.3223f,-1.5000f,5.2734f,-1.5000f,5.2734f,-1.3958f,5.2246f,-1.3958f,5.2246f,-1.2917f,5.1758f,-1.2917f,5.1758f,-1.1875f,5.1270f,-1.1875f,5.1270f,-1.0833f,5.0781f,-1.0833f,
	5.0781f,-0.9792f,5.0293f,-0.9792f,5.0293f,-0.8750f,4.9805f,-0.8750f,4.9805f,-0.7708f,4.9316f,-0.7708f,4.9316f,-0.6667f,4.8828f,-0.6667f,4.8828f,-0.5625f,4.8340f,-0.5625f,4.8340f,-0.4583f,4.7852f,-0.4583f,4.7852f,-0.3542f,4.7363f,-0.3542f,4.7363f,-0.2500f,
	4.6875f,-0.2500f,4.6875f,-0.1458f,4.6387f,-0.1458f,4.6387f,-0.0417f,4.5898f,-0.0417f,4.5898f,0.0625f,4.5410f,0.0625f,4.5410f,0.1667f,4.4922f,0.1667f,4.4922f,0.2708f,4.4434f,0.2708f,4.4434f,0.3750f,4.3945f,0.3750f,4.3945f,0.4792f,4.3457f,0.4792f,
	4.3457f,0.5833f,4.2969f,0.5833f,4.2969f,0.6875f,4.2480f,0.6875f,4.2480f,0.7917f,4.1992f,0.7917f,4.1992f,0.8958f,4.1504f,0.8958f,4.1504f,1.0000f,4.1016f,1.0000f,6.4453f,-4.0000f,6.3965f,-4.0000f,6.3965f,-3.8958f,6.3477f,-3.8958f,6.3477f,-3.7917f,
	6.2988f,-3.7917f,6.2988f,-3.6875f,6.2500f,-3.6875f,6.2500f,-3.5833f,6.2012f,-3.5833f,6.2012f,-3.4792f,6.1523f,-3.4792f,6.1523f,-3.3750f,6.1035f,-3.3750f,6.1035f,-3.2708f,6.0547f,-3.2708f,6.0547f,-3.1667f,6.0059f,-3.1667f,6.0059f,-3.0625f,5.9570f,-3.0625f,
	5.9570f,-2.9583f,5.9082f,-2.9583f,5.9082f,-2.8542f,5.8594f,-2.8542f,5.8594f,-2.7500f,5.8105f,-2.7500f,5.8105f,-2.6458f,5.7617f,-2.6458f,5.7617f,-2.5417f,5.7129f,-2.5417f,5.7129f,-2.4375f,5.6641f,-2.4375f,5.6641f,-2.3333f,5.6152f,-2.3333f,5.6152f,-2.2292f,
	5.5664f,-2.2292f,5.5664f,-2.1250f,5.5176f,-2.1250f,5.5176f,-2.0208f,5.4688f,-2.0208f,5.4688f,-1.9167f,5.4199f,-1.9167f,5.4199f,-1.8125f,5.3711f,-1.8125f,5.3711f,-1.7083f,5.3223f,-1.7083f,5.3223f,-1.6042f,5.2734f,-1.6042f,5.2734f,-1.5000f,5.2246f,-1.5000f,
	5.2246f,-1.3958f,5.1758f,-1.3958f,5.1758f,-1.2917f,5.1270f,-1.2917f,5.1270f,-1.1875f,5.0781f,-1.1875f,5.0781f,-1.0833f,5.0293f,-1.0833f,5.0293f,-0.9792f,4.9805f,-0.9792f,4.9805f,-0.8750f,4.9316f,-0.8750f,4.9316f,-0.7708f,4.8828f,-0.7708f,4.8828f,-0.6667f,
	4.8340f,-0.6667f,4.8340f,-0.5625f,4.7852f,-0.5625f,4.7852f,-0.4583f,4.7363f,-0.4583f,4.7363f,-0.3542f,4.6875f,-0.3542f,4.6875f,-0.2500f,4.6387f,-0.2500f,4.6387f,-0.1458f,4.5898f,-0.1458f,4.5898f,-0.0417f,4.5410f,-0.0417f,4.5410f,0.0625f,4.4922f,0.0625f,
	4.4922f,0.1667f,4.4434f,0.1667f,4.4434f,0.2708f,4.3945f,0.2708f,4.3945f,0.3750f,4.3457f,0.3750f,4.3457f,0.4792f,4.2969f,0.4792f,4.2969f,0.5833f,4.2480f,0.5833f,4.2480f,0.6875f,4.1992f,0.6875f,4.1992f,0.7917f,4.1504f,0.7917f,4.1504f,0.8958f,
	4.1016f,0.8958f,4.1016f,1.0000f,4.0527f,1.0000f,6.3965f,-4.0000f,6.3477f,-4.0000f,6.3477f,-3.8958f,6.2988f,-3.8958f,6.2988f,-3.7917f,6.2500f,-3.7917f,6.2500f,-3.6875f,6.2012f,-3.6875f,6.2012f,-3.5833f,6.1523f,-3.5833f,6.1523f,-3.4792f,6.1035f,-3.4792f,
	6.1035f,-3.3750f,6.0547f,-3.3750f,6.0547f,-3.2708f,6.0059f,-3.2708f,6.0059f,-3.1667f,5.9570f,-3.1667f,5.9570f,-3.0625f,5.9082f,-3.0625f,5.9082f,-2.9583f,5.8594f,-2.9583f,5.8594f,-2.8542f,5.8105f,-2.8542f,5.8105f,-2.7500f,5.7617f,-2.7500f,5.7617f,-2.6458f,
	5.7129f,-2.6458f,5.7129f,-2.5417f,5.6641f,-2.5417f,5.6641f,-2.4375f,5.6152f,-2.4375f,5.6152f,-2.3333f,5.5664f,-2.3333f,5.5664f,-2.2292f,5.5176f,-2.2292f,5.5176f,-2.1250f,5.4688f,-2.1250f,5.4688f,-2.0208f,5.4199f,-2.0208f,5.4199f,-1.9167f,5.3711f,-1.9167f,
	5.3711f,-1.8125f,5.3223f,-1.8125f,5.3223f,-1.7083f,5.2734f,-1.7083f,5.2734f,-1.6042f,5.2246f,-1.6042f,5.2246f,-1.5000f,5.1758f,-1.5000f,5.1758f,-1.3958f,5.1270f,-1.3958f,5.1270f,-1.2917f,5.0781f,-1.2917f,5.0781f,-1.1875f,5.0293f,-1.1875f,5.0293f,-1.0833f,
	4.9805f,-1.0833f,4.9805f,-0.9792f,4.9316f,-0.9792f,4.9316f,-0.8750f,4.8828f,-0.8750f,4.8828f,-0.7708f,4.8340f,-0.7708f,4.8340f,-0.6667f,4.7852f,-0.6667f,4.7852f,-0.5625f,4.7363f,-0.5625f,4.7363f,-0.4583f,4.6875f,-0.4583f,4.6875f,-0.3542f,4.6387f,-0.3542f,
	4.6387f,-0.2500f,4.5898f,-0.2500f,4.5898f,-0.1458f,4.5410f,-0.1458f,4.5410f,-0.0417f,4.4922f,-0.0417f,4.4922f,0.0625f,4.4434f,0.0625f,4.4434f,0.1667f,4.3945f,0.1667f,4.3945f,0.2708f,4.3457f,0.2708f,4.3457f,0.3750f,4.2969f,0.3750f,4.2969f,0.4792f,
	4.2480f,0.4792f,4.2480f,0.5833f,4.1992f,0.5833f,4.1992f,0.6875f,4.1504f,0.6875f,4.1504f,0.7917f,4.1016f,0.7917f,4.1016f,0.8958f,4.0527f,0.8958f,4.0527f,1.0000f,4.0039f,1.0000f,6.3477f,-4.0000f,6.2988f,-4.0000f,6.2988f,-3.8958f,6.2500f,-3.8958f,
	6.2500f,-3.7917f,6.2012f,-3.7917f,6.2012f,-3.6875f,6.1523f,-3.6875f,6.1523f,-3.5833f,6.1035f,-3.5833f,6.1035f,-3.4792f,6.0547f,-3.4792f,6.0547f,-3.3750f,6.0059f,-3.3750f,6.0059f,-3.2708f,5.9570f,-3.2708f,5.9570f,-3.1667f,5.9082f,-3.1667f,5.9082f,-3.0625f,
	5.8594f,-3.0625f,5.8594f,-2.9583f,5.8105f,-2.9583f,5.8105f,-2.8542f,5.7617f,-2.8542f,5.7617f,-2.7500f,5.7129f,-2.7500f,5.7129f,-2.6458f,5.6641f,-2.6458f,5.6641f,-2.5417f,5.6152f,-2.5417f,5.6152f,-2.4375f,5.5664f,-2.4375f,5.5664f,-2.3333f,5.5176f,-2.3333f,
	5.5176f,-2.2292f,5.4688f,-2.2292f,5.4688f,-2.1250f,5.4199f,-2.1250f,5.4199f,-2.0208f,5.3711f,-2.0208f,5.3711f,-1.9167f,5.3223f,-1.9167f,5.3223f,-1.8125f,5.2734f,-1.8125f,5.2734f,-1.7083f,5.2246f,-1.7083f,5.2246f,-1.6042f,5.1758f,-1.6042f,5.1758f,-1.5000f,
	5.1270f,-1.5000f,5.1270f,-1.3958f,5.0781f,-1.3958f,5.0781f,-1.2917f,5.0293f,-1.2917f,5.0293f,-1.1875f,4.9805f,-1.1875f,4.9805f,-1.0833f,4.9316f,-1.0833f,4.9316f,-0.9792f,4.8828f,-0.9792f,4.8828f,-0.8750f,4.8340f,-0.8750f,4.8340f,-0.7708f,4.7852f,-0.7708f,
	4.7852f,-0.6667f,4.7363f,-0.6667f,4.7363f,-0.5625f,4.6875f,-0.5625f,4.6875f,-0.4583f,4.6387f,-0.4583f,4.6387f,-0.3542f,4.5898f,-0.3542f,4.5898f,-0.2500f,4.5410f,-0.2500f,4.5410f,-0.1458f,4.4922f,-0.1458f,4.4922f,-0.0417f,4.4434f,-0.0417f,4.4434f,0.0625f,
	4.3945f,0.0625f,4.3945f,0.1667f,4.3457f,0.1667f,4.3457f,0.2708f,4.2969f,0.2708f,4.2969f,0.3750f,4.2480f,0.3750f,4.2480f,0.4792f,4.1992f,0.4792f,4.1992f,0.5833f,4.1504f,0.5833f,4.1504f,0.6875f,4.1016f,0.6875f,4.1016f,0.7917f,4.0527f,0.7917f,
	4.0527f,0.8958f,4.0039f,0.8958f,4.0039f,1.0000f,3.9551f,1.0000f,6.2988f,-4.0000f,6.2500f,-4.0000f,6.2500f,-3.8958f,6.2012f,-3.8958f,6.2012f,-3.7917f,6.1523f,-3.7917f,6.1523f,-3.6875f,6.1035f,-3.6875f,6.1035f,-3.5833f,6.0547f,-3.5833f,6.0547f,-3.4792f,
	6.0059f,-3.4792f,6.0059f,-3.3750f,5.9570f,-3.3750f,5.9570f,-3.2708f,5.9082f,-3.2708f,5.9082f,-3.1667f,5.8594f,-3.1667f,5.8594f,-3.0625f,5.8105f,-3.0625f,5.8105f,-2.9583f,5.7617f,-2.9583f,5.7617f,-2.8542f,5.7129f,-2.8542f,5.7129f,-2.7500f,5.6641f,-2.7500f,
	5.6641f,-2.6458f,5.6152f,-2.6458f,5.6152f,-2.5417f,5.5664f,-2.5417f,5.5664f,-2.4375f,5.5176f,-2.4375f,5.5176f,-2.3333f,5.4688f,-2.3333f,5.4688f,-2.2292f,5.4199f,-2.2292f,5.4199f,-2.1250f,5.3711f,-2.1250f,5.3711f,-2.0208f,5.3223f,-2.0208f,5.3223f,-1.9167f,
	5.2734f,-1.9167f,5.2734f,-1.8125f,5.2246f,-1.8125f,5.2246f,-1.7083f,5.1758f,-1.7083f,5.1758f,-1.6042f,5.1270f,-1.6042f,5.1270f,-1.5000f,5.0781f,-1.5000f,5.0781f,-1.3958f,5.0293f,-1.3958f,5.0293f,-1.2917f,4.9805f,-1.2917f,4.9805f,-1.1875f,4.9316f,-1.1875f,
	4.9316f,-1.0833f,4.8828f,-1.0833f,4.8828f,-0.9792f,4.8340f,-0.9792f,4.8340f,-0.8750f,4.7852f,-0.8750f,4.7852f,-0.7708f,4.7363f,-0.7708f,4.7363f,-0.6667f,4.6875f,-0.6667f,4.6875f,-0.5625f,4.6387f,-0.5625f,4.6387f,-0.4583f,4.5898f,-0.4583f,4.5898f,-0.3542f,
	4.5410f,-0.3542f,4.5410f,-0.2500f,4.4922f,-0.2500f,4.4922f,-0.1458f,4.4434f,-0.1458f,4.4434f,-0.0417f,4.3945f,-0.0417f,4.3945f,0.0625f,4.3457f,0.0625f,4.3457f,0.1667f,4.2969f,0.1667f,4.2969f,0.2708f,4.2480f,0.2708f,4.2480f,0.3750f,4.1992f,0.3750f,
	4.1992f,0.4792f,4.1504f,0.4792f,4.1504f,0.5833f,4.1016f,0.5833f,4.1016f,0.6875f,4.0527f,0.6875f,4.0527f,0.7917f,4.0039f,0.7917f,4.0039f,0.8958f,3.9551f,0.8958f,3.9551f,1.0000f,3.9062f,1.0000f,6.2500f,-4.0000f,6.2012f,-4.0000f,6.2012f,-3.8958f,
	6.1523f,-3.8958f,6.1523f,-3.7917f,6.1035f,-3.7917f,6.1035f,-3.6875f,6.0547f,-3.6875f,6.0547f,-3.5833f,6.0059f,-3.5833f,6.0059f,-3.4792f,5.9570f,-3.4792f,5.9570f,-3.3750f,5.9082f,-3.3750f,5.9082f,-3.2708f,5.8594f,-3.2708f,5.8594f,-3.1667f,5.8105f,-3.1667f,
	5.8105f,-3.0625f,5.7617f,-3.0625f,5.7617f,-2.9583f,5.7129f,-2.9583f,5.7129f,-2.8542f,5.6641f,-2.8542f,5.6641f,-2.7500f,5.6152f,-2.7500f,5.6152f,-2.6458f,5.5664f,-2.6458f,5.5664f,-2.5417f,5.5176f,-2.5417f,5.5176f,-2.4375f,5.4688f,-2.4375f,5.4688f,-2.3333f,
	5.4199f,-2.3333f,5.4199f,-2.2292f,5.3711f,-2.2292f,5.3711f,-2.1250f,5.3223f,-2.1250f,5.3223f,-2.0208f,5.2734f,-2.0208f,5.2734f,-1.9167f,5.2246f,-1.9167f,5.2246f,-1.8125f,5.1758f,-1.8125f,5.1758f,-1.7083f,5.1270f,-1.7083f,5.1270f,-1.6042f,5.0781f,-1.6042f,
	5.0781f,-1.5000f,5.0293f,-1.5000f,5.0293f,-1.3958f,4.9805f,-1.3958f,4.9805f,-1.2917f,4.9316f,-1.2917f,4.9316f,-1.1875f,4.8828f,-1.1875f,4.8828f,-1.0833f,4.8340f,-1.0833f,4.8340f,-0.9792f,4.7852f,-0.9792f,4.7852f,-0.8750f,4.7363f,-0.8750f,4.7363f,-0.7708f,
	4.6875f,-0.7708f,4.6875f,-0.6667f,4.6387f,-0.6667f,4.6387f,-0.5625f,4.5898f,-0.5625f,4.5898f,-0.4583f,4.5410f,-0.4583f,4.5410f,-0.3542f,4.4922f,-0.3542f,4.4922f,-0.2500f,4.4434f,-0.2500f,4.4434f,-0.1458f,4.3945f,-0.1458f,4.3945f,-0.0417f,4.3457f,-0.0417f,
	4.3457f,0.0625f,4.2969f,0.0625f,4.2969f,0.1667f,4.2480f,0.1667f,4.2480f,0.2708f,4.1992f,0.2708f,4.1992f,0.3750f,4.1504f,0.3750f,4.1504f,0.4792f,4.1016f,0.4792f,4.1016f,0.5833f,4.0527f,0.5833f,4.0527f,0.6875f,4.0039f,0.6875f,4.0039f,0.7917f,
	3.9551f,0.7917f,3.9551f,0.8958f,3.9062f,0.8958f,3.9062f,1.0000f,3.8574f,1.0000f,6.2012f,-4.0000f,6.1523f,-4.0000f,6.1523f,-3.8958f,6.1035f,-3.8958f,6.1035f,-3.7917f,6.0547f,-3.7917f,6.0547f,-3.6875f,6.0059f,-3.6875f,6.0059f,-3.5833f,5.9570f,-3.5833f,
	5.9570f,-3.4792f,5.9082f,-3.4792f,5.9082f,-3.3750f,5.8594f,-3.3750f,5.8594f,-3.2708f,5.8105f,-3.2708f,5.8105f,-3.1667f,5.7617f,-3.1667f,5.7617f,-3.0625f,5.7129f,-3.0625f,5.7129f,-2.9583f,5.6641f,-2.9583f,5.6641f,-2.8542f,5.6152f,-2.8542f,5.6152f,-2.7500f,
	5.5664f,-2.7500f,5.5664f,-2.6458f,5.5176f,-2.6458f,5.5176f,-2.5417f,5.4688f,-2.5417f,5.4688f,-2.4375f,5.4199f,-2.4375f,5.4199f,-2.3333f,5.3711f,-2.3333f,5.3711f,-2.2292f,5.3223f,-2.2292f,5.3223f,-2.1250f,5.2734f,-2.1250f,5.2734f,-2.0208f,5.2246f,-2.0208f,
	5.2246f,-1.9167f,5.1758f,-1.9167f,5.1758f,-1.8125f,5.1270f,-1.8125f,5.1270f,-1.7083f,5.0781f,-1.7083f,5.0781f,-1.6042f,5.0293f,-1.6042f,5.0293f,-1.5000f,4.9805f,-1.5000f,4.9805f,-1.3958f,4.9316f,-1.3958f,4.9316f,-1.2917f,4.8828f,-1.2917f,4.8828f,-1.1875f,
	4.8340f,-1.1875f,4.8340f,-1.0833f,4.7852f,-1.0833f,4.7852f,-0.9792f,4.7363f,-0.9792f,4.7363f,-0.8750f,4.6875f,-0.8750f,4.6875f,-0.7708f,4.6387f,-0.7708f,4.6387f,-0.6667f,4.5898f,-0.6667f,4.5898f,-0.5625f,4.5410f,-0.5625f,4.5410f,-0.4583f,4.4922f,-0.4583f,
	4.4922f,-0.3542f,4.4434f,-0.3542f,4.4434f,-0.2500f,4.3945f,-0.2500f,4.3945f,-0.1458f,4.3457f,-0.1458f,4.3457f,-0.0417f,4.2969f,-0.0417f,4.2969f,0.0625f,4.2480f,0.0625f,4.2480f,0.1667f,4.1992f,0.1667f,4.1992f,0.2708f,4.1504f,0.2708f,4.1504f,0.3750f,
	4.1016f,0.3750f,4.1016f,0.4792f,4.0527f,0.4792f,4.0527f,0.5833f,4.0039f,0.5833f,4.0039f,0.6875f,3.9551f,0.6875f,3.9551f,0.7917f,3.9062f,0.7917f,3.9062f,0.8958f,3.8574f,0.8958f,3.8574f,1.0000f,3.8086f,1.0000f,6.1523f,-4.0000f,6.1035f,-4.0000f,
	6.1035f,-3.8958f,6.0547f,-3.8958f,6.0547f,-3.7917f,6.0059f,-3.7917f,6.0059f,-3.6875f,5.9570f,-3.6875f,5.9570f,-3.5833f,5.9082f,-3.5833f,5.9082f,-3.4792f,5.8594f,-3.4792f,5.8594f,-3.3750f,5.8105f,-3.3750f,5.8105f,-3.2708f,5.7617f,-3.2708f,5.7617f,-3.1667f,
	5.7129f,-3.1667f,5.7129f,-3.0625f,5.6641f,-3.0625f,5.6641f,-2.9583f,5.6152f,-2.9583f,5.6152f,-2.8542f,5.5664f,-2.8542f,5.5664f,-2.7500f,5.5176f,-2.7500f,5.5176f,-2.6458f,5.4688f,-2.6458f,5.4688f,-2.5417f,5.4199f,-2.5417f,5.4199f,-2.4375f,5.3711f,-2.4375f,
	5.3711f,-2.3333f,5.3223f,-2.3333f,5.3223f,-2.2292f,5.2734f,-2.2292f,5.2734f,-2.1250f,5.2246f,-2.1250f,5.2246f,-2.0208f,5.1758f,-2.0208f,5.1758f,-1.9167f,5.1270f,-1.9167f,5.1270f,-1.8125f,5.0781f,-1.8125f,5.0781f,-1.7083f,5.0293f,-1.7083f,5.0293f,-1.6042f,
	4.9805f,-1.6042f,4.9805f,-1.5000f,4.9316f,-1.5000f,4.9316f,-1.3958f,4.8828f,-1.3958f,4.8828f,-1.2917f,4.8340f,-1.2917f,4.8340f,-1.1875f,4.7852f,-1.1875f,4.7852f,-1.0833f,4.7363f,-1.0833f,4.7363f,-0.9792f,4.6875f,-0.9792f,4.6875f,-0.8750f,4.6387f,-0.8750f,
	4.6387f,-0.7708f,4.5898f,-0.7708f,4.5898f,-0.6667f,4.5410f,-0.6667f,4.5410f,-0.5625f,4.4922f,-0.5625f,4.4922f,-0.4583f,4.4434f,-0.4583f,4.4434f,-0.3542f,4.3945f,-0.3542f,4.3945f,-0.2500f,4.3457f,-0.2500f,4.3457f,-0.1458f,4.2969f,-0.1458f,4.2969f,-0.0417f,
	4.2480f,-0.0417f,4.2480f,0.0625f,4.1992f,0.0625f,4.1992f,0.1667f,4.1504f,0.1667f,4.1504f,0.2708f,4.1016f,0.2708f,4.1016f,0.3750f,4.0527f,0.3750f,4.0527f,0.4792f,4.0039f,0.4792f,4.0039f,0.5833f,3.9551f,0.5833f,3.9551f,0.6875f,3.9062f,0.6875f,
	3.9062f,0.7917f,3.8574f,0.7917f,3.8574f,0.8958f,3.8086f,0.8958f,3.8086f,1.0000f,3.7598f,1.0000f,6.1035f,-4.0000f,6.0547f,-4.0000f,6.0547f,-3.8958f,6.0059f,-3.8958f,6.0059f,-3.7917f,5.9570f,-3.7917f,5.9570f,-3.6875f,5.9082f,-3.6875f,5.9082f,-3.5833f,
	5.8594f,-3.5833f,5.8594f,-3.4792f,5.8105f,-3.4792f,5.8105f,-3.3750f,5.7617f,-3.3750f,5.7617f,-3.2708f,5.7129f,-3.2708f,5.7129f,-3.1667f,5.6641f,-3.1667f,5.6641f,-3.0625f,5.6152f,-3.0625f,5.6152f,-2.9583f,5.5664f,-2.9583f,5.5664f,-2.8542f,5.5176f,-2.8542f,
	5.5176f,-2.7500f,5.4688f,-2.7500f,5.4688f,-2.6458f,5.4199f,-2.6458f,5.4199f,-2.5417f,5.3711f,-2.5417f,5.3711f,-2.4375f,5.3223f,-2.4375f,5.3223f,-2.3333f,5.2734f,-2.3333f,5.2734f,-2.2292f,5.2246f,-2.2292f,5.2246f,-2.1250f,5.1758f,-2.1250f,5.1758f,-2.0208f,
	5.1270f,-2.0208f,5.1270f,-1.9167f,5.0781f,-1.9167f,5.0781f,-1.8125f,5.0293f,-1.8125f,5.0293f,-1.7083f,4.9805f,-1.7083f,4.9805f,-1.6042f,4.9316f,-1.6042f,4.9316f,-1.5000f,4.8828f,-1.5000f,4.8828f,-1.3958f,4.8340f,-1.3958f,4.8340f,-1.2917f,4.7852f,-1.2917f,
	4.7852f,-1.1875f,4.7363f,-1.1875f,4.7363f,-1.0833f,4.6875f,-1.0833f,4.6875f,-0.9792f,4.6387f,-0.9792f,4.6387f,-0.8750f,4.5898f,-0.8750f,4.5898f,-0.7708f,4.5410f,-0.7708f,4.5410f,-0.6667f,4.4922f,-0.6667f,4.4922f,-0.5625f,4.4434f,-0.5625f,4.4434f,-0.4583f,
	4.3945f,-0.4583f,4.3945f,-0.3542f,4.3457f,-0.3542f,4.3457f,-0.2500f,4.2969f,-0.2500f,4.2969f,-0.1458f,4.2480f,-0.1458f,4.2480f,-0.0417f,4.1992f,-0.0417f,4.1992f,0.0625f,4.1504f,0.0625f,4.1504f,0.1667f,4.1016f,0.1667f,4.1016f,0.2708f,4.0527f,0.2708f,
	4.0527f,0.3750f,4.0039f,0.3750f,4.0039f,0.4792f,3.9551f,0.4792f,3.9551f,0.5833f,3.9062f,0.5833f,3.9062f,0.6875f,3.8574f,0.6875f,3.8574f,0.7917f,3.8086f,0.7917f,3.8086f,0.8958f,3.7598f,0.8958f,3.7598f,1.0000f,3.7109f,1.0000f,6.0547f,-4.0000f,
	6.0059f,-4.0000f,6.0059f,-3.8958f,5.9570f,-3.8958f,5.9570f,-3.7917f,5.9082f,-3.7917f,5.9082f,-3.6875f,5.8594f,-3.6875f,5.8594f,-3.5833f,5.8105f,-3.5833f,5.8105f,-3.4792f,5.7617f,-3.4792f,5.7617f,-3.3750f,5.7129f,-3.3750f,5.7129f,-3.2708f,5.6641f,-3.2708f,
	5.6641f,-3.1667f,5.6152f,-3.1667f,5.6152f,-3.0625f,5.5664f,-3.0625f,5.5664f,-2.9583f,5.5176f,-2.9583f,5.5176f,-2.8542f,5.4688f,-2.8542f,5.4688f,-2.7500f,5.4199f,-2.7500f,5.4199f,-2.6458f,5.3711f,-2.6458f,5.3711f,-2.5417f,5.3223f,-2.5417f,5.3223f,-2.4375f,
	5.2734f,-2.4375f,5.2734f,-2.3333f,5.2246f,-2.3333f,5.2246f,-2.2292f,5.1758f,-2.2292f,5.1758f,-2.1250f,5.1270f,-2.1250f,5.1270f,-2.0208f,5.0781f,-2.0208f,5.0781f,-1.9167f,5.0293f,-1.9167f,5.0293f,-1.8125f,4.9805f,-1.8125f,4.9805f,-1.7083f,4.9316f,-1.7083f,
	4.9316f,-1.6042f,4.8828f,-1.6042f,4.8828f,-1.5000f,4.8340f,-1.5000f,4.8340f,-1.3958f,4.7852f,-1.3958f,4.7852f,-1.2917f,4.7363f,-1.2917f,4.7363f,-1.1875f,4.6875f,-1.1875f,4.6875f,-1.0833f,4.6387f,-1.0833f,4.6387f,-0.9792f,4.5898f,-0.9792f,4.5898f,-0.8750f,
	4.5410f,-0.8750f,4.5410f,-0.7708f,4.4922f,-0.7708f,4.4922f,-0.6667f,4.4434f,-0.6667f,4.4434f,-0.5625f,4.3945f,-0.5625f,4.3945f,-0.4583f,4.3457f,-0.4583f,4.3457f,-0.3542f,4.2969f,-0.3542f,4.2969f,-0.2500f,4.2480f,-0.2500f,4.2480f,-0.1458f,4.1992f,-0.1458f,
	4.1992f,-0.0417f,4.1504f,-0.0417f,4.1504f,0.0625f,4.1016f,0.0625f,4.1016f,0.1667f,4.0527f,0.1667f,4.0527f,0.2708f,4.0039f,0.2708f,4.0039f,0.3750f,3.9551f,0.3750f,3.9551f,0.4792f,3.9062f,0.4792f,3.9062f,0.5833f,3.8574f,0.5833f,3.8574f,0.6875f,
	3.8086f,0.6875f,3.8086f,0.7917f,3.7598f,0.7917f,3.7598f,0.8958f,3.7109f,0.8958f,3.7109f,1.0000f,3.6621f,1.0000f,6.0059f,-4.0000f,5.9570f,-4.0000f,5.9570f,-3.8958f,5.9082f,-3.8958f,5.9082f,-3.7917f,5.8594f,-3.7917f,5.8594f,-3.6875f,5.8105f,-3.6875f,
	5.8105f,-3.5833f,5.7617f,-3.5833f,5.7617f,-3.4792f,5.7129f,-3.4792f,5.7129f,-3.3750f,5.6641f,-3.3750f,5.6641f,-3.2708f,5.6152f,-3.2708f,5.6152f,-3.1667f,5.5664f,-3.1667f,5.5664f,-3.0625f,5.5176f,-3.0625f,5.5176f,-2.9583f,5.4688f,-2.9583f,5.4688f,-2.8542f,
	5.4199f,-2.8542f,5.4199f,-2.7500f,5.3711f,-2.7500f,5.3711f,-2.6458f,5.3223f,-2.6458f,5.3223f,-2.5417f,5.2734f,-2.5417f,5.2734f,-2.4375f,5.2246f,-2.4375f,5.2246f,-2.3333f,5.1758f,-2.3333f,5.1758f,-2.2292f,5.1270f,-2.2292f,5.1270f,-2.1250f,5.0781f,-2.1250f,
	5.0781f,-2.0208f,5.0293f,-2.0208f,5.0293f,-1.9167f,4.9805f,-1.9167f,4.9805f,-1.8125f,4.9316f,-1.8125f,4.9316f,-1.7083f,4.8828f,-1.7083f,4.8828f,-1.6042f,4.8340f,-1.6042f,4.8340f,-1.5000f,4.7852f,-1.5000f,4.7852f,-1.3958f,4.7363f,-1.3958f,4.7363f,-1.2917f,
	4.6875f,-1.2917f,4.6875f,-1.1875f,4.6387f,-1.1875f,4.6387f,-1.0833f,4.5898f,-1.0833f,4.5898f,-0.9792f,4.5410f,-0.9792f,4.5410f,-0.8750f,4.4922f,-0.8750f,4.4922f,-0.7708f,4.4434f,-0.7708f,4.4434f,-0.6667f,4.3945f,-0.6667f,4.3945f,-0.5625f,4.3457f,-0.5625f,
	4.3457f,-0.4583f,4.2969f,-0.4583f,4.2969f,-0.3542f,4.2480f,-0.3542f,4.2480f,-0.2500f,4.1992f,-0.2500f,4.1992f,-0.1458f,4.1504f,-0.1458f,4.1504f,-0.0417f,4.1016f,-0.0417f,4.1016f,0.0625f,4.0527f,0.0625f,4.0527f,0.1667f,4.0039f,0.1667f,4.0039f,0.2708f,
	3.9551f,0.2708f,3.9551f,0.3750f,3.9062f,0.3750f,3.9062f,0.4792f,3.8574f,0.4792f,3.8574f,0.5833f,3.8086f,0.5833f,3.8086f,0.6875f,3.7598f,0.6875f,3.7598f,0.7917f,3.7109f,0.7917f,3.7109f,0.8958f,3.6621f,0.8958f,3.6621f,1.0000f,3.6133f,1.0000f,
	5.9570f,-4.0000f,5.9082f,-4.0000f,5.9082f,-3.8958f,5.8594f,-3.8958f,5.8594f,-3.7917f,5.8105f,-3.7917f,5.8105f,-3.6875f,5.7617f,-3.6875f,5.7617f,-3.5833f,5.7129f,-3.5833f,5.7129f,-3.4792f,5.6641f,-3.4792f,5.6641f,-3.3750f,5.6152f,-3.3750f,5.6152f,-3.2708f,
	5.5664f,-3.2708f,5.5664f,-3.1667f,5.5176f,-3.1667f,5.5176f,-3.0625f,5.4688f,-3.0625f,5.4688f,-2.9583f,5.4199f,-2.9583f,5.4199f,-2.8542f,5.3711f,-2.8542f,5.3711f,-2.7500f,5.3223f,-2.7500f,5.3223f,-2.6458f,5.2734f,-2.6458f,5.2734f,-2.5417f,5.2246f,-2.5417f,
	5.2246f,-2.4375f,5.1758f,-2.4375f,5.1758f,-2.3333f,5.1270f,-2.3333f,5.1270f,-2.2292f,5.0781f,-2.2292f,5.0781f,-2.1250f,5.0293f,-2.1250f,5.0293f,-2.0208f,4.9805f,-2.0208f,4.9805f,-1.9167f,4.9316f,-1.9167f,4.9316f,-1.8125f,4.8828f,-1.8125f,4.8828f,-1.7083f,
	4.8340f,-1.7083f,4.8340f,-1.6042f,4.7852f,-1.6042f,4.7852f,-1.5000f,4.7363f,-1.5000f,4.7363f,-1.3958f,4.6875f,-1.3958f,4.6875f,-1.2917f,4.6387f,-1.2917f,4.6387f,-1.1875f,4.5898f,-1.1875f,4.5898f,-1.0833f,4.5410f,-1.0833f,4.5410f,-0.9792f,4.4922f,-0.9792f,
	4.4922f,-0.8750f,4.4434f,-0.8750f,4.4434f,-0.7708f,4.3945f,-0.7708f,4.3945f,-0.6667f,4.3457f,-0.6667f,4.3457f,-0.5625f,4.2969f,-0.5625f,4.2969f,-0.4583f,4.2480f,-0.4583f,4.2480f,-0.3542f,4.1992f,-0.3542f,4.1992f,-0.2500f,4.1504f,-0.2500f,4.1504f,-0.1458f,
	4.1016f,-0.1458f,4.1016f,-0.0417f,4.0527f,-0.0417f,4.0527f,0.0625f,4.0039f,0.0625f,4.0039f,0.1667f,3.9551f,0.1667f,3.9551f,0.2708f,3.9062f,0.2708f,3.9062f,0.3750f,3.8574f,0.3750f,3.8574f,0.4792f,3.8086f,0.4792f,3.8086f,0.5833f,3.7598f,0.5833f,
	3.7598f,0.6875f,3.7109f,0.6875f,3.7109f,0.7917f,3.6621f,0.7917f,3.6621f,0.8958f,3.6133f,0.8958f,3.6133f,1.0000f,3.5645f,1.0000f,5.9082f,-4.0000f,5.8594f,-4.0000f,5.8594f,-3.8958f,5.8105f,-3.8958f,5.8105f,-3.7917f,5.7617f,-3.7917f,5.7617f,-3.6875f,
	5.7129f,-3.6875f,5.7129f,-3.5833f,5.6641f,-3.5833f,5.6641f,-3.4792f,5.6152f,-3.4792f,5.6152f,-3.3750f,5.5664f,-3.3750f,5.5664f,-3.2708f,5.5176f,-3.2708f,5.5176f,-3.1667f,5.4688f,-3.1667f,5.4688f,-3.0625f,5.4199f,-3.0625f,5.4199f,-2.9583f,5.3711f,-2.9583f,
	5.3711f,-2.8542f,5.3223f,-2.8542f,5.3223f,-2.7500f,5.2734f,-2.7500f,5.2734f,-2.6458f,5.2246f,-2.6458f,5.2246f,-2.5417f,5.1758f,-2.5417f,5.1758f,-2.4375f,5.1270f,-2.4375f,5.1270f,-2.3333f,5.0781f,-2.3333f,5.0781f,-2.2292f,5.0293f,-2.2292f,5.0293f,-2.1250f,
	4.9805f,-2.1250f,4.9805f,-2.0208f,4.9316f,-2.0208f,4.9316f,-1.9167f,4.8828f,-1.9167f,4.8828f,-1.8125f,4.8340f,-1.8125f,4.8340f,-1.7083f,4.7852f,-1.7083f,4.7852f,-1.6042f,4.7363f,-1.6042f,4.7363f,-1.5000f,4.6875f,-1.5000f,4.6875f,-1.3958f,4.6387f,-1.3958f,
	4.6387f,-1.2917f,4.5898f,-1.2917f,4.5898f,-1.1875f,4.5410f,-1.1875f,4.5410f,-1.0833f,4.4922f,-1.0833f,4.4922f,-0.9792f,4.4434f,-0.9792f,4.4434f,-0.8750f,4.3945f,-0.8750f,4.3945f,-0.7708f,4.3457f,-0.7708f,4.3457f,-0.6667f,4.2969f,-0.6667f,4.2969f,-0.5625f,
	4.2480f,-0.5625f,4.2480f,-0.4583f,4.1992f,-0.4583f,4.1992f,-0.3542f,4.1504f,-0.3542f,4.1504f,-0.2500f,4.1016f,-0.2500f,4.1016f,-0.1458f,4.0527f,-0.1458f,4.0527f,-0.0417f,4.0039f,-0.0417f,4.0039f,0.0625f,3.9551f,0.0625f,3.9551f,0.1667f,3.9062f,0.1667f,
	3.9062f,0.2708f,3.8574f,0.2708f,3.8574f,0.3750f,3.8086f,0.3750f,3.8086f,0.4792f,3.7598f,0.4792f,3.7598f,0.5833f,3.7109f,0.5833f,3.7109f,0.6875f,3.6621f,0.6875f,3.6621f,0.7917f,3.6133f,0.7917f,3.6133f,0.8958f,3.5645f,0.8958f,3.5645f,1.0000f,
	3.5156f,1.0000f,5.8594f,-4.0000f,5.8105f,-4.0000f,5.8105f,-3.8958f,5.7617f,-3.8958f,5.7617f,-3.7917f,5.7129f,-3.7917f,5.7129f,-3.6875f,5.6641f,-3.6875f,5.6641f,-3.5833f,5.6152f,-3.5833f,5.6152f,-3.4792f,5.5664f,-3.4792f,5.5664f,-3.3750f,5.5176f,-3.3750f,
	5.5176f,-3.2708f,5.4688f,-3.2708f,5.4688f,-3.1667f,5.4199f,-3.1667f,5.4199f,-3.0625f,5.3711f,-3.0625f,5.3711f,-2.9583f,5.3223f,-2.9583f,5.3223f,-2.8542f,5.2734f,-2.8542f,5.2734f,-2.7500f,5.2246f,-2.7500f,5.2246f,-2.6458f,5.1758f,-2.6458f,5.1758f,-2.5417f,
	5.1270f,-2.5417f,5.1270f,-2.4375f,5.0781f,-2.4375f,5.0781f,-2.3333f,5.0293f,-2.3333f,5.0293f,-2.2292f,4.9805f,-2.2292f,4.9805f,-2.1250f,4.9316f,-2.1250f,4.9316f,-2.0208f,4.8828f,-2.0208f,4.8828f,-1.9167f,4.8340f,-1.9167f,4.8340f,-1.8125f,4.7852f,-1.8125f,
	4.7852f,-1.7083f,4.7363f,-1.7083f,4.7363f,-1.6042f,4.6875f,-1.6042f,4.6875f,-1.5000f,4.6387f,-1.5000f,4.6387f,-1.3958f,4.5898f,-1.3958f,4.5898f,-1.2917f,4.5410f,-1.2917f,4.5410f,-1.1875f,4.4922f,-1.1875f,4.4922f,-1.0833f,4.4434f,-1.0833f,4.4434f,-0.9792f,
	4.3945f,-0.9792f,4.3945f,-0.8750f,4.3457f,-0.8750f,4.3457f,-0.7708f,4.2969f,-0.7708f,4.2969f,-0.6667f,4.2480f,-0.6667f,4.2480f,-0.5625f,4.1992f,-0.5625f,4.1992f,-0.4583f,4.1504f,-0.4583f,4.1504f,-0.3542f,4.1016f,-0.3542f,4.1016f,-0.2500f,4.0527f,-0.2500f,
	4.0527f,-0.1458f,4.0039f,-0.1458f,4.0039f,-0.0417f,3.9551f,-0.0417f,3.9551f,0.0625f,3.9062f,0.0625f,3.9062f,0.1667f,3.8574f,0.1667f,3.8574f,0.2708f,3.8086f,0.2708f,3.8086f,0.3750f,3.7598f,0.3750f,3.7598f,0.4792f,3.7109f,0.4792f,3.7109f,0.5833f,
	3.6621f,0.5833f,3.6621f,0.6875f,3.6133f,0.6875f,3.6133f,0.7917f,3.5645f,0.7917f,3.5645f,0.8958f,3.5156f,0.8958f,3.5156f,1.0000f,3.4668f,1.0000f,5.8105f,-4.0000f,5.7617f,-4.0000f,5.7617f,-3.8958f,5.7129f,-3.8958f,5.7129f,-3.7917f,5.6641f,-3.7917f,
	5.6641f,-3.6875f,5.6152f,-3.6875f,5.6152f,-3.5833f,5.5664f,-3.5833f,5.5664f,-3.4792f,5.5176f,-3.4792f,5.5176f,-3.3750f,5.4688f,-3.3750f,5.4688f,-3.2708f,5.4199f,-3.2708f,5.4199f,-3.1667f,5.3711f,-3.1667f,5.3711f,-3.0625f,5.3223f,-3.0625f,5.3223f,-2.9583f,
	5.2734f,-2.9583f,5.2734f,-2.8542f,5.2246f,-2.8542f,5.2246f,-2.7500f,5.1758f,-2.7500f,5.1758f,-2.6458f,5.1270f,-2.6458f,5.1270f,-2.5417f,5.0781f,-2.5417f,5.0781f,-2.4375f,5.0293f,-2.4375f,5.0293f,-2.3333f,4.9805f,-2.3333f,4.9805f,-2.2292f,4.9316f,-2.2292f,
	4.9316f,-2.1250f,4.8828f,-2.1250f,4.8828f,-2.0208f,4.8340f,-2.0208f,4.8340f,-1.9167f,4.7852f,-1.9167f,4.7852f,-1.8125f,4.7363f,-1.8125f,4.7363f,-1.7083f,4.6875f,-1.7083f,4.6875f,-1.6042f,4.6387f,-1.6042f,4.6387f,-1.5000f,4.5898f,-1.5000f,4.5898f,-1.3958f,
	4.5410f,-1.3958f,4.5410f,-1.2917f,4.4922f,-1.2917f,4.4922f,-1.1875f,4.4434f,-1.1875f,4.4434f,-1.0833f,4.3945f,-1.0833f,4.3945f,-0.9792f,4.3457f,-0.9792f,4.3457f,-0.8750f,4.2969f,-0.8750f,4.2969f,-0.7708f,4.2480f,-0.7708f,4.2480f,-0.6667f,4.1992f,-0.6667f,
	4.1992f,-0.5625f,4.1504f,-0.5625f,4.1504f,-0.4583f,4.1016f,-0.4583f,4.1016f,-0.3542f,4.0527f,-0.3542f,4.0527f,-0.2500f,4.0039f,-0.2500f,4.0039f,-0.1458f,3.9551f,-0.1458f,3.9551f,-0.0417f,3.9062f,-0.0417f,3.9062f,0.0625f,3.8574f,0.0625f,3.8574f,0.1667f,
	3.8086f,0.1667f,3.8086f,0.2708f,3.7598f,0.2708f,3.7598f,0.3750f,3.7109f,0.3750f,3.7109f,0.4792f,3.6621f,0.4792f,3.6621f,0.5833f,3.6133f,0.5833f,3.6133f,0.6875f,3.5645f,0.6875f,3.5645f,0.7917f,3.5156f,0.7917f,3.5156f,0.8958f,3.4668f,0.8958f,
	3.4668f,1.0000f,3.4180f,1.0000f,5.7617f,-4.0000f,5.7129f,-4.0000f,5.7129f,-3.8958f,5.6641f,-3.8958f,5.6641f,-3.7917f,5.6152f,-3.7917f,5.6152f,-3.6875f,5.5664f,-3.6875f,5.5664f,-3.5833f,5.5176f,-3.5833f,5.5176f,-3.4792f,5.4688f,-3.4792f,5.4688f,-3.3750f,
	5.4199f,-3.3750f,5.4199f,-3.2708f,5.3711f,-3.2708f,5.3711f,-3.1667f,5.3223f,-3.1667f,5.3223f,-3.0625f,5.2734f,-3.0625f,5.2734f,-2.9583f,5.2246f,-2.9583f,5.2246f,-2.8542f,5.1758f,-2.8542f,5.1758f,-2.7500f,5.1270f,-2.7500f,5.1270f,-2.6458f,5.0781f,-2.6458f,
	5.0781f,-2.5417f,5.0293f,-2.5417f,5.0293f,-2.4375f,4.9805f,-2.4375f,4.9805f,-2.3333f,4.9316f,-2.3333f,4.9316f,-2.2292f,4.8828f,-2.2292f,4.8828f,-2.1250f,4.8340f,-2.1250f,4.8340f,-2.0208f,4.7852f,-2.0208f,4.7852f,-1.9167f,4.7363f,-1.9167f,4.7363f,-1.8125f,
	4.6875f,-1.8125f,4.6875f,-1.7083f,4.6387f,-1.7083f,4.6387f,-1.6042f,4.5898f,-1.6042f,4.5898f,-1.5000f,4.5410f,-1.5000f,4.5410f,-1.3958f,4.4922f,-1.3958f,4.4922f,-1.2917f,4.4434f,-1.2917f,4.4434f,-1.1875f,4.3945f,-1.1875f,4.3945f,-1.0833f,4.3457f,-1.0833f,
	4.3457f,-0.9792f,4.2969f,-0.9792f,4.2969f,-0.8750f,4.2480f,-0.8750f,4.2480f,-0.7708f,4.1992f,-0.7708f,4.1992f,-0.6667f,4.1504f,-0.6667f,4.1504f,-0.5625f,4.1016f,-0.5625f,4.1016f,-0.4583f,4.0527f,-0.4583f,4.0527f,-0.3542f,4.0039f,-0.3542f,4.0039f,-0.2500f,
	3.9551f,-0.2500f,3.9551f,-0.1458f,3.9062f,-0.1458f,3.9062f,-0.0417f,3.8574f,-0.0417f,3.8574f,0.0625f,3.8086f,0.0625f,3.8086f,0.1667f,3.7598f,0.1667f,3.7598f,0.2708f,3.7109f,0.2708f,3.7109f,0.3750f,3.6621f,0.3750f,3.6621f,0.4792f,3.6133f,0.4792f,
	3.6133f,0.5833f,3.5645f,0.5833f,3.5645f,0.6875f,3.5156f,0.6875f,3.5156f,0.7917f,3.4668f,0.7917f,3.4668f,0.8958f,3.4180f,0.8958f,3.4180f,1.0000f,3.3691f,1.0000f,5.7129f,-4.0000f,5.6641f,-4.0000f,5.6641f,-3.8958f,5.6152f,-3.8958f,5.6152f,-3.7917f,
	5.5664f,-3.7917f,5.5664f,-3.6875f,5.5176f,-3.6875f,5.5176f,-3.5833f,5.4688f,-3.5833f,5.4688f,-3.4792f,5.4199f,-3.4792f,5.4199f,-3.3750f,5.3711f,-3.3750f,5.3711f,-3.2708f,5.3223f,-3.2708f,5.3223f,-3.1667f,5.2734f,-3.1667f,5.2734f,-3.0625f,5.2246f,-3.0625f,
	5.2246f,-2.9583f,5.1758f,-2.9583f,5.1758f,-2.8542f,5.1270f,-2.8542f,5.1270f,-2.7500f,5.0781f,-2.7500f,5.0781f,-2.6458f,5.0293f,-2.6458f,5.0293f,-2.5417f,4.9805f,-2.5417f,4.9805f,-2.4375f,4.9316f,-2.4375f,4.9316f,-2.3333f,4.8828f,-2.3333f,4.8828f,-2.2292f,
	4.8340f,-2.2292f,4.8340f,-2.1250f,4.7852f,-2.1250f,4.7852f,-2.0208f,4.7363f,-2.0208f,4.7363f,-1.9167f,4.6875f,-1.9167f,4.6875f,-1.8125f,4.6387f,-1.8125f,4.6387f,-1.7083f,4.5898f,-1.7083f,4.5898f,-1.6042f,4.5410f,-1.6042f,4.5410f,-1.5000f,4.4922f,-1.5000f,
	4.4922f,-1.3958f,4.4434f,-1.3958f,4.4434f,-1.2917f,4.3945f,-1.2917f,4.3945f,-1.1875f,4.3457f,-1.1875f,4.3457f,-1.0833f,4.2969f,-1.0833f,4.2969f,-0.9792f,4.2480f,-0.9792f,4.2480f,-0.8750f,4.1992f,-0.8750f,4.1992f,-0.7708f,4.1504f,-0.7708f,4.1504f,-0.6667f,
	4.1016f,-0.6667f,4.1016f,-0.5625f,4.0527f,-0.5625f,4.0527f,-0.4583f,4.0039f,-0.4583f,4.0039f,-0.3542f,3.9551f,-0.3542f,3.9551f,-0.2500f,3.9062f,-0.2500f,3.9062f,-0.1458f,3.8574f,-0.1458f,3.8574f,-0.0417f,3.8086f,-0.0417f,3.8086f,0.0625f,3.7598f,0.0625f,
	3.7598f,0.1667f,3.7109f,0.1667f,3.7109f,0.2708f,3.6621f,0.2708f,3.6621f,0.3750f,3.6133f,0.3750f,3.6133f,0.4792f,3.5645f,0.4792f,3.5645f,0.5833f,3.5156f,0.5833f,3.5156f,0.6875f,3.4668f,0.6875f,3.4668f,0.7917f,3.4180f,0.7917f,3.4180f,0.8958f,
	3.3691f,0.8958f,3.3691f,1.0000f,3.3203f,1.0000f,5.6641f,-4.0000f,5.6152f,-4.0000f,5.6152f,-3.8958f,5.5664f,-3.8958f,5.5664f,-3.7917f,5.5176f,-3.7917f,5.5176f,-3.6875f,5.4688f,-3.6875f,5.4688f,-3.5833f,5.4199f,-3.5833f,5.4199f,-3.4792f,5.3711f,-3.4792f,
	5.3711f,-3.3750f,5.3223f,-3.3750f,5.3223f,-3.2708f,5.2734f,-3.2708f,5.2734f,-3.1667f,5.2246f,-3.1667f,5.2246f,-3.0625f,5.1758f,-3.0625f,5.1758f,-2.9583f,5.1270f,-2.9583f,5.1270f,-2.8542f,5.0781f,-2.8542f,5.0781f,-2.7500f,5.0293f,-2.7500f,5.0293f,-2.6458f,
	4.9805f,-2.6458f,4.9805f,-2.5417f,4.9316f,-2.5417f,4.9316f,-2.4375f,4.8828f,-2.4375f,4.8828f,-2.3333f,4.8340f,-2.3333f,4.8340f,-2.2292f,4.7852f,-2.2292f,4.7852f,-2.1250f,4.7363f,-2.1250f,4.7363f,-2.0208f,4.6875f,-2.0208f,4.6875f,-1.9167f,4.6387f,-1.9167f,
	4.6387f,-1.8125f,4.5898f,-1.8125f,4.5898f,-1.7083f,4.5410f,-1.7083f,4.5410f,-1.6042f,4.4922f,-1.6042f,4.4922f,-1.5000f,4.4434f,-1.5000f,4.4434f,-1.3958f,4.3945f,-1.3958f,4.3945f,-1.2917f,4.3457f,-1.2917f,4.3457f,-1.1875f,4.2969f,-1.1875f,4.2969f,-1.0833f,
	4.2480f,-1.0833f,4.2480f,-0.9792f,4.1992f,-0.9792f,4.1992f,-0.8750f,4.1504f,-0.8750f,4.1504f,-0.7708f,4.1016f,-0.7708f,4.1016f,-0.6667f,4.0527f,-0.6667f,4.0527f,-0.5625f,4.0039f,-0.5625f,4.0039f,-0.4583f,3.9551f,-0.4583f,3.9551f,-0.3542f,3.9062f,-0.3542f,
	3.9062f,-0.2500f,3.8574f,-0.2500f,3.8574f,-0.1458f,3.8086f,-0.1458f,3.8086f,-0.0417f,3.7598f,-0.0417f,3.7598f,0.0625f,3.7109f,0.0625f,3.7109f,0.1667f,3.6621f,0.1667f,3.6621f,0.2708f,3.6133f,0.2708f,3.6133f,0.3750f,3.5645f,0.3750f,3.5645f,0.4792f,
	3.5156f,0.4792f,3.5156f,0.5833f,3.4668f,0.5833f,3.4668f,0.6875f,3.4180f,0.6875f,3.4180f,0.7917f,3.3691f,0.7917f,3.3691f,0.8958f,3.3203f,0.8958f,3.3203f,1.0000f,3.2715f,1.0000f,5.6152f,-4.0000f,5.5664f,-4.0000f,5.5664f,-3.8958f,5.5176f,-3.8958f,
	5.5176f,-3.7917f,5.4688f,-3.7917f,5.4688f,-3.6875f,5.4199f,-3.6875f,5.4199f,-3.5833f,5.3711f,-3.5833f,5.3711f,-3.4792f,5.3223f,-3.4792f,5.3223f,-3.3750f,5.2734f,-3.3750f,5.2734f,-3.2708f,5.2246f,-3.2708f,5.2246f,-3.1667f,5.1758f,-3.1667f,5.1758f,-3.0625f,
	5.1270f,-3.0625f,5.1270f,-2.9583f,5.0781f,-2.9583f,5.0781f,-2.8542f,5.0293f,-2.8542f,5.0293f,-2.7500f,4.9805f,-2.7500f,4.9805f,-2.6458f,4.9316f,-2.6458f,4.9316f,-2.5417f,4.8828f,-2.5417f,4.8828f,-2.4375f,4.8340f,-2.4375f,4.8340f,-2.3333f,4.7852f,-2.3333f,
	4.7852f,-2.2292f,4.7363f,-2.2292f,4.7363f,-2.1250f,4.6875f,-2.1250f,4.6875f,-2.0208f,4.6387f,-2.0208f,4.6387f,-1.9167f,4.5898f,-1.9167f,4.5898f,-1.8125f,4.5410f,-1.8125f,4.5410f,-1.7083f,4.4922f,-1.7083f,4.4922f,-1.6042f,4.4434f,-1.6042f,4.4434f,-1.5000f,
	4.3945f,-1.5000f,4.3945f,-1.3958f,4.3457f,-1.3958f,4.3457f,-1.2917f,4.2969f,-1.2917f,4.2969f,-1.1875f,4.2480f,-1.1875f,4.2480f,-1.0833f,4.1992f,-1.0833f,4.1992f,-0.9792f,4.1504f,-0.9792f,4.1504f,-0.8750f,4.1016f,-0.8750f,4.1016f,-0.7708f,4.0527f,-0.7708f,
	4.0527f,-0.6667f,4.0039f,-0.6667f,4.0039f,-0.5625f,3.9551f,-0.5625f,3.9551f,-0.4583f,3.9062f,-0.4583f,3.9062f,-0.3542f,3.8574f,-0.3542f,3.8574f,-0.2500f,3.8086f,-0.2500f,3.8086f,-0.1458f,3.7598f,-0.1458f,3.7598f,-0.0417f,3.7109f,-0.0417f,3.7109f,0.0625f,
	3.6621f,0.0625f,3.6621f,0.1667f,3.6133f,0.1667f,3.6133f,0.2708f,3.5645f,0.2708f,3.5645f,0.3750f,3.5156f,0.3750f,3.5156f,0.4792f,3.4668f,0.4792f,3.4668f,0.5833f,3.4180f,0.5833f,3.4180f,0.6875f,3.3691f,0.6875f,3.3691f,0.7917f,3.3203f,0.7917f,
	3.3203f,0.8958f,3.2715f,0.8958f,3.2715f,1.0000f,3.2227f,1.0000f,5.5664f,-4.0000f,5.5176f,-4.0000f,5.5176f,-3.8958f,5.4688f,-3.8958f,5.4688f,-3.7917f,5.4199f,-3.7917f,5.4199f,-3.6875f,5.3711f,-3.6875f,5.3711f,-3.5833f,5.3223f,-3.5833f,5.3223f,-3.4792f,
	5.2734f,-3.4792f,5.2734f,-3.3750f,5.2246f,-3.3750f,5.2246f,-3.2708f,5.1758f,-3.2708f,5.1758f,-3.1667f,5.1270f,-3.1667f,5.1270f,-3.0625f,5.0781f,-3.0625f,5.0781f,-2.9583f,5.0293f,-2.9583f,5.0293f,-2.8542f,4.9805f,-2.8542f,4.9805f,-2.7500f,4.9316f,-2.7500f,
	4.9316f,-2.6458f,4.8828f,-2.6458f,4.8828f,-2.5417f,4.8340f,-2.5417f,4.8340f,-2.4375f,4.7852f,-2.4375f,4.7852f,-2.3333f,4.7363f,-2.3333f,4.7363f,-2.2292f,4.6875f,-2.2292f,4.6875f,-2.1250f,4.6387f,-2.1250f,4.6387f,-2.0208f,4.5898f,-2.0208f,4.5898f,-1.9167f,
	4.5410f,-1.9167f,4.5410f,-1.8125f,4.4922f,-1.8125f,4.4922f,-1.7083f,4.4434f,-1.7083f,4.4434f,-1.6042f,4.3945f,-1.6042f,4.3945f,-1.5000f,4.3457f,-1.5000f,4.3457f,-1.3958f,4.2969f,-1.3958f,4.2969f,-1.2917f,4.2480f,-1.2917f,4.2480f,-1.1875f,4.1992f,-1.1875f,
	4.1992f,-1.0833f,4.1504f,-1.0833f,4.1504f,-0.9792f,4.1016f,-0.9792f,4.1016f,-0.8750f,4.0527f,-0.8750f,4.0527f,-0.7708f,4.0039f,-0.7708f,4.0039f,-0.6667f,3.9551f,-0.6667f,3.9551f,-0.5625f,3.9062f,-0.5625f,3.9062f,-0.4583f,3.8574f,-0.4583f,3.8574f,-0.3542f,
	3.8086f,-0.3542f,3.8086f,-0.2500f,3.7598f,-0.2500f,3.7598f,-0.1458f,3.7109f,-0.1458f,3.7109f,-0.0417f,3.6621f,-0.0417f,3.6621f,0.0625f,3.6133f,0.0625f,3.6133f,0.1667f,3.5645f,0.1667f,3.5645f,0.2708f,3.5156f,0.2708f,3.5156f,0.3750f,3.4668f,0.3750f,
	3.4668f,0.4792f,3.4180f,0.4792f,3.4180f,0.5833f,3.3691f,0.5833f,3.3691f,0.6875f,3.3203f,0.6875f,3.3203f,0.7917f,3.2715f,0.7917f,3.2715f,0.8958f,3.2227f,0.8958f,3.2227f,1.0000f,3.1738f,1.0000f,5.5176f,-4.0000f,5.4688f,-4.0000f,5.4688f,-3.8958f,
	5.4199f,-3.8958f,5.4199f,-3.7917f,5.3711f,-3.7917f,5.3711f,-3.6875f,5.3223f,-3.6875f,5.3223f,-3.5833f,5.2734f,-3.5833f,5.2734f,-3.4792f,5.2246f,-3.4792f,5.2246f,-3.3750f,5.1758f,-3.3750f,5.1758f,-3.2708f,5.1270f,-3.2708f,5.1270f,-3.1667f,5.0781f,-3.1667f,
	5.0781f,-3.0625f,5.0293f,-3.0625f,5.0293f,-2.9583f,4.9805f,-2.9583f,4.9805f,-2.8542f,4.9316f,-2.8542f,4.9316f,-2.7500f,4.8828f,-2.7500f,4.8828f,-2.6458f,4.8340f,-2.6458f,4.8340f,-2.5417f,4.7852f,-2.5417f,4.7852f,-2.4375f,4.7363f,-2.4375f,4.7363f,-2.3333f,
	4.6875f,-2.3333f,4.6875f,-2.2292f,4.6387f,-2.2292f,4.6387f,-2.1250f,4.5898f,-2.1250f,4.5898f,-2.0208f,4.5410f,-2.0208f,4.5410f,-1.9167f,4.4922f,-1.9167f,4.4922f,-1.8125f,4.4434f,-1.8125f,4.4434f,-1.7083f,4.3945f,-1.7083f,4.3945f,-1.6042f,4.3457f,-1.6042f,
	4.3457f,-1.5000f,4.2969f,-1.5000f,4.2969f,-1.3958f,4.2480f,-1.3958f,4.2480f,-1.2917f,4.1992f,-1.2917f,4.1992f,-1.1875f,4.1504f,-1.1875f,4.1504f,-1.0833f,4.1016f,-1.0833f,4.1016f,-0.9792f,4.0527f,-0.9792f,4.0527f,-0.8750f,4.0039f,-0.8750f,4.0039f,-0.7708f,
	3.9551f,-0.7708f,3.9551f,-0.6667f,3.9062f,-0.6667f,3.9062f,-0.5625f,3.8574f,-0.5625f,3.8574f,-0.4583f,3.8086f,-0.4583f,3.8086f,-0.3542f,3.7598f,-0.3542f,3.7598f,-0.2500f,3.7109f,-0.2500f,3.7109f,-0.1458f,3.6621f,-0.1458f,3.6621f,-0.0417f,3.6133f,-0.0417f,
	3.6133f,0.0625f,3.5645f,0.0625f,3.5645f,0.1667f,3.5156f,0.1667f,3.5156f,0.2708f,3.4668f,0.2708f,3.4668f,0.3750f,3.4180f,0.3750f,3.4180f,0.4792f,3.3691f,0.4792f,3.3691f,0.5833f,3.3203f,0.5833f,3.3203f,0.6875f,3.2715f,0.6875f,3.2715f,0.7917f,
	3.2227f,0.7917f,3.2227f,0.8958f,3.1738f,0.8958f,3.1738f,1.0000f,3.1250f,1.0000f,5.4688f,-4.0000f,5.4199f,-4.0000f,5.4199f,-3.8958f,5.3711f,-3.8958f,5.3711f,-3.7917f,5.3223f,-3.7917f,5.3223f,-3.6875f,5.2734f,-3.6875f,5.2734f,-3.5833f,5.2246f,-3.5833f,
	5.2246f,-3.4792f,5.1758f,-3.4792f,5.1758f,-3.3750f,5.1270f,-3.3750f,5.1270f,-3.2708f,5.0781f,-3.2708f,5.0781f,-3.1667f,5.0293f,-3.1667f,5.0293f,-3.0625f,4.9805f,-3.0625f,4.9805f,-2.9583f,4.9316f,-2.9583f,4.9316f,-2.8542f,4.8828f,-2.8542f,4.8828f,-2.7500f,
	4.8340f,-2.7500f,4.8340f,-2.6458f,4.7852f,-2.6458f,4.7852f,-2.5417f,4.7363f,-2.5417f,4.7363f,-2.4375f,4.6875f,-2.4375f,4.6875f,-2.3333f,4.6387f,-2.3333f,4.6387f,-2.2292f,4.5898f,-2.2292f,4.5898f,-2.1250f,4.5410f,-2.1250f,4.5410f,-2.0208f,4.4922f,-2.0208f,
	4.4922f,-1.9167f,4.4434f,-1.9167f,4.4434f,-1.8125f,4.3945f,-1.8125f,4.3945f,-1.7083f,4.3457f,-1.7083f,4.3457f,-1.6042f,4.2969f,-1.6042f,4.2969f,-1.5000f,4.2480f,-1.5000f,4.2480f,-1.3958f,4.1992f,-1.3958f,4.1992f,-1.2917f,4.1504f,-1.2917f,4.1504f,-1.1875f,
	4.1016f,-1.1875f,4.1016f,-1.0833f,4.0527f,-1.0833f,4.0527f,-0.9792f,4.0039f,-0.9792f,4.0039f,-0.8750f,3.9551f,-0.8750f,3.9551f,-0.7708f,3.9062f,-0.7708f,3.9062f,-0.6667f,3.8574f,-0.6667f,3.8574f,-0.5625f,3.8086f,-0.5625f,3.8086f,-0.4583f,3.7598f,-0.4583f,
	3.7598f,-0.3542f,3.7109f,-0.3542f,3.7109f,-0.2500f,3.6621f,-0.2500f,3.6621f,-0.1458f,3.6133f,-0.1458f,3.6133f,-0.0417f,3.5645f,-0.0417f,3.5645f,0.0625f,3.5156f,0.0625f,3.5156f,0.1667f,3.4668f,0.1667f,3.4668f,0.2708f,3.4180f,0.2708f,3.4180f,0.3750f,
	3.3691f,0.3750f,3.3691f,0.4792f,3.3203f,0.4792f,3.3203f,0.5833f,3.2715f,0.5833f,3.2715f,0.6875f,3.2227f,0.6875f,3.2227f,0.7917f,3.1738f,0.7917f,3.1738f,0.8958f,3.1250f,0.8958f,3.1250f,1.0000f,3.0762f,1.0000f,5.4199f,-4.0000f,5.3711f,-4.0000f,
	5.3711f,-3.8958f,5.3223f,-3.8958f,5.3223f,-3.7917f,5.2734f,-3.7917f,5.2734f,-3.6875f,5.2246f,-3.6875f,5.2246f,-3.5833f,5.1758f,-3.5833f,5.1758f,-3.4792f,5.1270f,-3.4792f,5.1270f,-3.3750f,5.0781f,-3.3750f,5.0781f,-3.2708f,5.0293f,-3.2708f,5.0293f,-3.1667f,
	4.9805f,-3.1667f,4.9805f,-3.0625f,4.9316f,-3.0625f,4.9316f,-2.9583f,4.8828f,-2.9583f,4.8828f,-2.8542f,4.8340f,-2.8542f,4.8340f,-2.7500f,4.7852f,-2.7500f,4.7852f,-2.6458f,4.7363f,-2.6458f,4.7363f,-2.5417f,4.6875f,-2.5417f,4.6875f,-2.4375f,4.6387f,-2.4375f,
	4.6387f,-2.3333f,4.5898f,-2.3333f,4.5898f,-2.2292f,4.5410f,-2.2292f,4.5410f,-2.1250f,4.4922f,-2.1250f,4.4922f,-2.0208f,4.4434f,-2.0208f,4.4434f,-1.9167f,4.3945f,-1.9167f,4.3945f,-1.8125f,4.3457f,-1.8125f,4.3457f,-1.7083f,4.2969f,-1.7083f,4.2969f,-1.6042f,
	4.2480f,-1.6042f,4.2480f,-1.5000f,4.1992f,-1.5000f,4.1992f,-1.3958f,4.1504f,-1.3958f,4.1504f,-1.2917f,4.1016f,-1.2917f,4.1016f,-1.1875f,4.0527f,-1.1875f,4.0527f,-1.0833f,4.0039f,-1.0833f,4.0039f,-0.9792f,3.9551f,-0.9792f,3.9551f,-0.8750f,3.9062f,-0.8750f,
	3.9062f,-0.7708f,3.8574f,-0.7708f,3.8574f,-0.6667f,3.8086f,-0.6667f,3.8086f,-0.5625f,3.7598f,-0.5625f,3.7598f,-0.4583f,3.7109f,-0.4583f,3.7109f,-0.3542f,3.6621f,-0.3542f,3.6621f,-0.2500f,3.6133f,-0.2500f,3.6133f,-0.1458f,3.5645f,-0.1458f,3.5645f,-0.0417f,
	3.5156f,-0.0417f,3.5156f,0.0625f,3.4668f,0.0625f,3.4668f,0.1667f,3.4180f,0.1667f,3.4180f,0.2708f,3.3691f,0.2708f,3.3691f,0.3750f,3.3203f,0.3750f,3.3203f,0.4792f,3.2715f,0.4792f,3.2715f,0.5833f,3.2227f,0.5833f,3.2227f,0.6875f,3.1738f,0.6875f,
	3.1738f,0.7917f,3.1250f,0.7917f,3.1250f,0.8958f,3.0762f,0.8958f,3.0762f,1.0000f,3.0273f,1.0000f,5.3711f,-4.0000f,5.3223f,-4.0000f,5.3223f,-3.8958f,5.2734f,-3.8958f,5.2734f,-3.7917f,5.2246f,-3.7917f,5.2246f,-3.6875f,5.1758f,-3.6875f,5.1758f,-3.5833f,
	5.1270f,-3.5833f,5.1270f,-3.4792f,5.0781f,-3.4792f,5.0781f,-3.3750f,5.0293f,-3.3750f,5.0293f,-3.2708f,4.9805f,-3.2708f,4.9805f,-3.1667f,4.9316f,-3.1667f,4.9316f,-3.0625f,4.8828f,-3.0625f,4.8828f,-2.9583f,4.8340f,-2.9583f,4.8340f,-2.8542f,4.7852f,-2.8542f,
	4.7852f,-2.7500f,4.7363f,-2.7500f,4.7363f,-2.6458f,4.6875f,-2.6458f,4.6875f,-2.5417f,4.6387f,-2.5417f,4.6387f,-2.4375f,4.5898f,-2.4375f,4.5898f,-2.3333f,4.5410f,-2.3333f,4.5410f,-2.2292f,4.4922f,-2.2292f,4.4922f,-2.1250f,4.4434f,-2.1250f,4.4434f,-2.0208f,
	4.3945f,-2.0208f,4.3945f,-1.9167f,4.3457f,-1.9167f,4.3457f,-1.8125f,4.2969f,-1.8125f,4.2969f,-1.7083f,4.2480f,-1.7083f,4.2480f,-1.6042f,4.1992f,-1.6042f,4.1992f,-1.5000f,4.1504f,-1.5000f,4.1504f,-1.3958f,4.1016f,-1.3958f,4.1016f,-1.2917f,4.0527f,-1.2917f,
	4.0527f,-1.1875f,4.0039f,-1.1875f,4.0039f,-1.0833f,3.9551f,-1.0833f,3.9551f,-0.9792f,3.9062f,-0.9792f,3.9062f,-0.8750f,3.8574f,-0.8750f,3.8574f,-0.7708f,3.8086f,-0.7708f,3.8086f,-0.6667f,3.7598f,-0.6667f,3.7598f,-0.5625f,3.7109f,-0.5625f,3.7109f,-0.4583f,
	3.6621f,-0.4583f,3.6621f,-0.3542f,3.6133f,-0.3542f,3.6133f,-0.2500f,3.5645f,-0.2500f,3.5645f,-0.1458f,3.5156f,-0.1458f,3.5156f,-0.0417f,3.4668f,-0.0417f,3.4668f,0.0625f,3.4180f,0.0625f,3.4180f,0.1667f,3.3691f,0.1667f,3.3691f,0.2708f,3.3203f,0.2708f,
	3.3203f,0.3750f,3.2715f,0.3750f,3.2715f,0.4792f,3.2227f,0.4792f,3.2227f,0.5833f,3.1738f,0.5833f,3.1738f,0.6875f,3.1250f,0.6875f,3.1250f,0.7917f,3.0762f,0.7917f,3.0762f,0.8958f,3.0273f,0.8958f,3.0273f,1.0000f,2.9785f,1.0000f,5.3223f,-4.0000f,
	5.2734f,-4.0000f,5.2734f,-3.8958f,5.2246f,-3.8958f,5.2246f,-3.7917f,5.1758f,-3.7917f,5.1758f,-3.6875f,5.1270f,-3.6875f,5.1270f,-3.5833f,5.0781f,-3.5833f,5.0781f,-3.4792f,5.0293f,-3.4792f,5.0293f,-3.3750f,4.9805f,-3.3750f,4.9805f,-3.2708f,4.9316f,-3.2708f,
	4.9316f,-3.1667f,4.8828f,-3.1667f,4.8828f,-3.0625f,4.8340f,-3.0625f,4.8340f,-2.9583f,4.7852f,-2.9583f,4.7852f,-2.8542f,4.7363f,-2.8542f,4.7363f,-2.7500f,4.6875f,-2.7500f,4.6875f,-2.6458f,4.6387f,-2.6458f,4.6387f,-2.5417f,4.5898f,-2.5417f,4.5898f,-2.4375f,
	4.5410f,-2.4375f,4.5410f,-2.3333f,4.4922f,-2.3333f,4.4922f,-2.2292f,4.4434f,-2.2292f,4.4434f,-2.1250f,4.3945f,-2.1250f,4.3945f,-2.0208f,4.3457f,-2.0208f,4.3457f,-1.9167f,4.2969f,-1.9167f,4.2969f,-1.8125f,4.2480f,-1.8125f,4.2480f,-1.7083f,4.1992f,-1.7083f,
	4.1992f,-1.6042f,4.1504f,-1.6042f,4.1504f,-1.5000f,4.1016f,-1.5000f,4.1016f,-1.3958f,4.0527f,-1.3958f,4.0527f,-1.2917f,4.0039f,-1.2917f,4.0039f,-1.1875f,3.9551f,-1.1875f,3.9551f,-1.0833f,3.9062f,-1.0833f,3.9062f,-0.9792f,3.8574f,-0.9792f,3.8574f,-0.8750f,
	3.8086f,-0.8750f,3.8086f,-0.7708f,3.7598f,-0.7708f,3.7598f,-0.6667f,3.7109f,-0.6667f,3.7109f,-0.5625f,3.6621f,-0.5625f,3.6621f,-0.4583f,3.6133f,-0.4583f,3.6133f,-0.3542f,3.5645f,-0.3542f,3.5645f,-0.2500f,3.5156f,-0.2500f,3.5156f,-0.1458f,3.4668f,-0.1458f,
	3.4668f,-0.0417f,3.4180f,-0.0417f,3.4180f,0.0625f,3.3691f,0.0625f,3.3691f,0.1667f,3.3203f,0.1667f,3.3203f,0.2708f,3.2715f,0.2708f,3.2715f,0.3750f,3.2227f,0.3750f,3.2227f,0.4792f,3.1738f,0.4792f,3.1738f,0.5833f,3.1250f,0.5833f,3.1250f,0.6875f,
	3.0762f,0.6875f,3.0762f,0.7917f,3.0273f,0.7917f,3.0273f,0.8958f,2.9785f,0.8958f,2.9785f,1.0000f,2.9297f,1.0000f,5.2734f,-4.0000f,5.2246f,-4.0000f,5.2246f,-3.8958f,5.1758f,-3.8958f,5.1758f,-3.7917f,5.1270f,-3.7917f,5.1270f,-3.6875f,5.0781f,-3.6875f,
	5.0781f,-3.5833f,5.0293f,-3.5833f,5.0293f,-3.4792f,4.9805f,-3.4792f,4.9805f,-3.3750f,4.9316f,-3.3750f,4.9316f,-3.2708f,4.8828f,-3.2708f,4.8828f,-3.1667f,4.8340f,-3.1667f,4.8340f,-3.0625f,4.7852f,-3.0625f,4.7852f,-2.9583f,4.7363f,-2.9583f,4.7363f,-2.8542f,
	4.6875f,-2.8542f,4.6875f,-2.7500f,4.6387f,-2.7500f,4.6387f,-2.6458f,4.5898f,-2.6458f,4.5898f,-2.5417f,4.5410f,-2.5417f,4.5410f,-2.4375f,4.4922f,-2.4375f,4.4922f,-2.3333f,4.4434f,-2.3333f,4.4434f,-2.2292f,4.3945f,-2.2292f,4.3945f,-2.1250f,4.3457f,-2.1250f,
	4.3457f,-2.0208f,4.2969f,-2.0208f,4.2969f,-1.9167f,4.2480f,-1.9167f,4.2480f,-1.8125f,4.1992f,-1.8125f,4.1992f,-1.7083f,4.1504f,-1.7083f,4.1504f,-1.6042f,4.1016f,-1.6042f,4.1016f,-1.5000f,4.0527f,-1.5000f,4.0527f,-1.3958f,4.0039f,-1.3958f,4.0039f,-1.2917f,
	3.9551f,-1.2917f,3.9551f,-1.1875f,3.9062f,-1.1875f,3.9062f,-1.0833f,3.8574f,-1.0833f,3.8574f,-0.9792f,3.8086f,-0.9792f,3.8086f,-0.8750f,3.7598f,-0.8750f,3.7598f,-0.7708f,3.7109f,-0.7708f,3.7109f,-0.6667f,3.6621f,-0.6667f,3.6621f,-0.5625f,3.6133f,-0.5625f,
	3.6133f,-0.4583f,3.5645f,-0.4583f,3.5645f,-0.3542f,3.5156f,-0.3542f,3.5156f,-0.2500f,3.4668f,-0.2500f,3.4668f,-0.1458f,3.4180f,-0.1458f,3.4180f,-0.0417f,3.3691f,-0.0417f,3.3691f,0.0625f,3.3203f,0.0625f,3.3203f,0.1667f,3.2715f,0.1667f,3.2715f,0.2708f,
	3.2227f,0.2708f,3.2227f,0.3750f,3.1738f,0.3750f,3.1738f,0.4792f,3.1250f,0.4792f,3.1250f,0.5833f,3.0762f,0.5833f,3.0762f,0.6875f,3.0273f,0.6875f,3.0273f,0.7917f,2.9785f,0.7917f,2.9785f,0.8958f,2.9297f,0.8958f,2.9297f,1.0000f,2.8809f,1.0000f,
	5.2246f,-4.0000f,5.1758f,-4.0000f,5.1758f,-3.8958f,5.1270f,-3.8958f,5.1270f,-3.7917f,5.0781f,-3.7917f,5.0781f,-3.6875f,5.0293f,-3.6875f,5.0293f,-3.5833f,4.9805f,-3.5833f,4.9805f,-3.4792f,4.9316f,-3.4792f,4.9316f,-3.3750f,4.8828f,-3.3750f,4.8828f,-3.2708f,
	4.8340f,-3.2708f,4.8340f,-3.1667f,4.7852f,-3.1667f,4.7852f,-3.0625f,4.7363f,-3.0625f,4.7363f,-2.9583f,4.6875f,-2.9583f,4.6875f,-2.8542f,4.6387f,-2.8542f,4.6387f,-2.7500f,4.5898f,-2.7500f,4.5898f,-2.6458f,4.5410f,-2.6458f,4.5410f,-2.5417f,4.4922f,-2.5417f,
	4.4922f,-2.4375f,4.4434f,-2.4375f,4.4434f,-2.3333f,4.3945f,-2.3333f,4.3945f,-2.2292f,4.3457f,-2.2292f,4.3457f,-2.1250f,4.2969f,-2.1250f,4.2969f,-2.0208f,4.2480f,-2.0208f,4.2480f,-1.9167f,4.1992f,-1.9167f,4.1992f,-1.8125f,4.1504f,-1.8125f,4.1504f,-1.7083f,
	4.1016f,-1.7083f,4.1016f,-1.6042f,4.0527f,-1.6042f,4.0527f,-1.5000f,4.0039f,-1.5000f,4.0039f,-1.3958f,3.9551f,-1.3958f,3.9551f,-1.2917f,3.9062f,-1.2917f,3.9062f,-1.1875f,3.8574f,-1.1875f,3.8574f,-1.0833f,3.8086f,-1.0833f,3.8086f,-0.9792f,3.7598f,-0.9792f,
	3.7598f,-0.8750f,3.7109f,-0.8750f,3.7109f,-0.7708f,3.6621f,-0.7708f,3.6621f,-0.6667f,3.6133f,-0.6667f,3.6133f,-0.5625f,3.5645f,-0.5625f,3.5645f,-0.4583f,3.5156f,-0.4583f,3.5156f,-0.3542f,3.4668f,-0.3542f,3.4668f,-0.2500f,3.4180f,-0.2500f,3.4180f,-0.1458f,
	3.3691f,-0.1458f,3.3691f,-0.0417f,3.3203f,-0.0417f,3.3203f,0.0625f,3.2715f,0.0625f,3.2715f,0.1667f,3.2227f,0.1667f,3.2227f,0.2708f,3.1738f,0.2708f,3.1738f,0.3750f,3.1250f,0.3750f,3.1250f,0.4792f,3.0762f,0.4792f,3.0762f,0.5833f,3.0273f,0.5833f,
	3.0273f,0.6875f,2.9785f,0.6875f,2.9785f,0.7917f,2.9297f,0.7917f,2.9297f,0.8958f,2.8809f,0.8958f,2.8809f,1.0000f,2.8320f,1.0000f,5.1758f,-4.0000f,5.1270f,-4.0000f,5.1270f,-3.8958f,5.0781f,-3.8958f,5.0781f,-3.7917f,5.0293f,-3.7917f,5.0293f,-3.6875f,
	4.9805f,-3.6875f,4.9805f,-3.5833f,4.9316f,-3.5833f,4.9316f,-3.4792f,4.8828f,-3.4792f,4.8828f,-3.3750f,4.8340f,-3.3750f,4.8340f,-3.2708f,4.7852f,-3.2708f,4.7852f,-3.1667f,4.7363f,-3.1667f,4.7363f,-3.0625f,4.6875f,-3.0625f,4.6875f,-2.9583f,4.6387f,-2.9583f,
	4.6387f,-2.8542f,4.5898f,-2.8542f,4.5898f,-2.7500f,4.5410f,-2.7500f,4.5410f,-2.6458f,4.4922f,-2.6458f,4.4922f,-2.5417f,4.4434f,-2.5417f,4.4434f,-2.4375f,4.3945f,-2.4375f,4.3945f,-2.3333f,4.3457f,-2.3333f,4.3457f,-2.2292f,4.2969f,-2.2292f,4.2969f,-2.1250f,
	4.2480f,-2.1250f,4.2480f,-2.0208f,4.1992f,-2.0208f,4.1992f,-1.9167f,4.1504f,-1.9167f,4.1504f,-1.8125f,4.1016f,-1.8125f,4.1016f,-1.7083f,4.0527f,-1.7083f,4.0527f,-1.6042f,4.0039f,-1.6042f,4.0039f,-1.5000f,3.9551f,-1.5000f,3.9551f,-1.3958f,3.9062f,-1.3958f,
	3.9062f,-1.2917f,3.8574f,-1.2917f,3.8574f,-1.1875f,3.8086f,-1.1875f,3.8086f,-1.0833f,3.7598f,-1.0833f,3.7598f,-0.9792f,3.7109f,-0.9792f,3.7109f,-0.8750f,3.6621f,-0.8750f,3.6621f,-0.7708f,3.6133f,-0.7708f,3.6133f,-0.6667f,3.5645f,-0.6667f,3.5645f,-0.5625f,
	3.5156f,-0.5625f,3.5156f,-0.4583f,3.4668f,-0.4583f,3.4668f,-0.3542f,3.4180f,-0.3542f,3.4180f,-0.2500f,3.3691f,-0.2500f,3.3691f,-0.1458f,3.3203f,-0.1458f,3.3203f,-0.0417f,3.2715f,-0.0417f,3.2715f,0.0625f,3.2227f,0.0625f,3.2227f,0.1667f,3.1738f,0.1667f,
	3.1738f,0.2708f,3.1250f,0.2708f,3.1250f,0.3750f,3.0762f,0.3750f,3.0762f,0.4792f,3.0273f,0.4792f,3.0273f,0.5833f,2.9785f,0.5833f,2.9785f,0.6875f,2.9297f,0.6875f,2.9297f,0.7917f,2.8809f,0.7917f,2.8809f,0.8958f,2.8320f,0.8958f,2.8320f,1.0000f,
	2.7832f,1.0000f,5.1270f,-4.0000f,5.0781f,-4.0000f,5.0781f,-3.8958f,5.0293f,-3.8958f,5.0293f,-3.7917f,4.9805f,-3.7917f,4.9805f,-3.6875f,4.9316f,-3.6875f,4.9316f,-3.5833f,4.8828f,-3.5833f,4.8828f,-3.4792f,4.8340f,-3.4792f,4.8340f,-3.3750f,4.7852f,-3.3750f,
	4.7852f,-3.2708f,4.7363f,-3.2708f,4.7363f,-3.1667f,4.6875f,-3.1667f,4.6875f,-3.0625f,4.6387f,-3.0625f,4.6387f,-2.9583f,4.5898f,-2.9583f,4.5898f,-2.8542f,4.5410f,-2.8542f,4.5410f,-2.7500f,4.4922f,-2.7500f,4.4922f,-2.6458f,4.4434f,-2.6458f,4.4434f,-2.5417f,
	4.3945f,-2.5417f,4.3945f,-2.4375f,4.3457f,-2.4375f,4.3457f,-2.3333f,4.2969f,-2.3333f,4.2969f,-2.2292f,4.2480f,-2.2292f,4.2480f,-2.1250f,4.1992f,-2.1250f,4.1992f,-2.0208f,4.1504f,-2.0208f,4.1504f,-1.9167f,4.1016f,-1.9167f,4.1016f,-1.8125f,4.0527f,-1.8125f,
	4.0527f,-1.7083f,4.0039f,-1.7083f,4.0039f,-1.6042f,3.9551f,-1.6042f,3.9551f,-1.5000f,3.9062f,-1.5000f,3.9062f,-1.3958f,3.8574f,-1.3958f,3.8574f,-1.2917f,3.8086f,-1.2917f,3.8086f,-1.1875f,3.7598f,-1.1875f,3.7598f,-1.0833f,3.7109f,-1.0833f,3.7109f,-0.9792f,
	3.6621f,-0.9792f,3.6621f,-0.8750f,3.6133f,-0.8750f,3.6133f,-0.7708f,3.5645f,-0.7708f,3.5645f,-0.6667f,3.5156f,-0.6667f,3.5156f,-0.5625f,3.4668f,-0.5625f,3.4668f,-0.4583f,3.4180f,-0.4583f,3.4180f,-0.3542f,3.3691f,-0.3542f,3.3691f,-0.2500f,3.3203f,-0.2500f,
	3.3203f,-0.1458f,3.2715f,-0.1458f,3.2715f,-0.0417f,3.2227f,-0.0417f,3.2227f,0.0625f,3.1738f,0.0625f,3.1738f,0.1667f,3.1250f,0.1667f,3.1250f,0.2708f,3.0762f,0.2708f,3.0762f,0.3750f,3.0273f,0.3750f,3.0273f,0.4792f,2.9785f,0.4792f,2.9785f,0.5833f,
	2.9297f,0.5833f,2.9297f,0.6875f,2.8809f,0.6875f,2.8809f,0.7917f,2.8320f,0.7917f,2.8320f,0.8958f,2.7832f,0.8958f,2.7832f,1.0000f,2.7344f,1.0000f,5.0781f,-4.0000f,5.0293f,-4.0000f,5.0293f,-3.8958f,4.9805f,-3.8958f,4.9805f,-3.7917f,4.9316f,-3.7917f,
	4.9316f,-3.6875f,4.8828f,-3.6875f,4.8828f,-3.5833f,4.8340f,-3.5833f,4.8340f,-3.4792f,4.7852f,-3.4792f,4.7852f,-3.3750f,4.7363f,-3.3750f,4.7363f,-3.2708f,4.6875f,-3.2708f,4.6875f,-3.1667f,4.6387f,-3.1667f,4.6387f,-3.0625f,4.5898f,-3.0625f,4.5898f,-2.9583f,
	4.5410f,-2.9583f,4.5410f,-2.8542f,4.4922f,-2.8542f,4.4922f,-2.7500f,4.4434f,-2.7500f,4.4434f,-2.6458f,4.3945f,-2.6458f,4.3945f,-2.5417f,4.3457f,-2.5417f,4.3457f,-2.4375f,4.2969f,-2.4375f,4.2969f,-2.3333f,4.2480f,-2.3333f,4.2480f,-2.2292f,4.1992f,-2.2292f,
	4.1992f,-2.1250f,4.1504f,-2.1250f,4.1504f,-2.0208f,4.1016f,-2.0208f,4.1016f,-1.9167f,4.0527f,-1.9167f,4.0527f,-1.8125f,4.0039f,-1.8125f,4.0039f,-1.7083f,3.9551f,-1.7083f,3.9551f,-1.6042f,3.9062f,-1.6042f,3.9062f,-1.5000f,3.8574f,-1.5000f,3.8574f,-1.3958f,
	3.8086f,-1.3958f,3.8086f,-1.2917f,3.7598f,-1.2917f,3.7598f,-1.1875f,3.7109f,-1.1875f,3.7109f,-1.0833f,3.6621f,-1.0833f,3.6621f,-0.9792f,3.6133f,-0.9792f,3.6133f,-0.8750f,3.5645f,-0.8750f,3.5645f,-0.7708f,3.5156f,-0.7708f,3.5156f,-0.6667f,3.4668f,-0.6667f,
	3.4668f,-0.5625f,3.4180f,-0.5625f,3.4180f,-0.4583f,3.3691f,-0.4583f,3.3691f,-0.3542f,3.3203f,-0.3542f,3.3203f,-0.2500f,3.2715f,-0.2500f,3.2715f,-0.1458f,3.2227f,-0.1458f,3.2227f,-0.0417f,3.1738f,-0.0417f,3.1738f,0.0625f,3.1250f,0.0625f,3.1250f,0.1667f,
	3.0762f,0.1667f,3.0762f,0.2708f,3.0273f,0.2708f,3.0273f,0.3750f,2.9785f,0.3750f,2.9785f,0.4792f,2.9297f,0.4792f,2.9297f,0.5833f,2.8809f,0.5833f,2.8809f,0.6875f,2.8320f,0.6875f,2.8320f,0.7917f,2.7832f,0.7917f,2.7832f,0.8958f,2.7344f,0.8958f,
	2.7344f,1.0000f,2.6855f,1.0000f,5.0293f,-4.0000f,4.9805f,-4.0000f,4.9805f,-3.8958f,4.9316f,-3.8958f,4.9316f,-3.7917f,4.8828f,-3.7917f,4.8828f,-3.6875f,4.8340f,-3.6875f,4.8340f,-3.5833f,4.7852f,-3.5833f,4.7852f,-3.4792f,4.7363f,-3.4792f,4.7363f,-3.3750f,
	4.6875f,-3.3750f,4.6875f,-3.2708f,4.6387f,-3.2708f,4.6387f,-3.1667f,4.5898f,-3.1667f,4.5898f,-3.0625f,4.5410f,-3.0625f,4.5410f,-2.9583f,4.4922f,-2.9583f,4.4922f,-2.8542f,4.4434f,-2.8542f,4.4434f,-2.7500f,4.3945f,-2.7500f,4.3945f,-2.6458f,4.3457f,-2.6458f,
	4.3457f,-2.5417f,4.2969f,-2.5417f,4.2969f,-2.4375f,4.2480f,-2.4375f,4.2480f,-2.3333f,4.1992f,-2.3333f,4.1992f,-2.2292f,4.1504f,-2.2292f,4.1504f,-2.1250f,4.1016f,-2.1250f,4.1016f,-2.0208f,4.0527f,-2.0208f,4.0527f,-1.9167f,4.0039f,-1.9167f,4.0039f,-1.8125f,
	3.9551f,-1.8125f,3.9551f,-1.7083f,3.9062f,-1.7083f,3.9062f,-1.6042f,3.8574f,-1.6042f,3.8574f,-1.5000f,3.8086f,-1.5000f,3.8086f,-1.3958f,3.7598f,-1.3958f,3.7598f,-1.2917f,3.7109f,-1.2917f,3.7109f,-1.1875f,3.6621f,-1.1875f,3.6621f,-1.0833f,3.6133f,-1.0833f,
	3.6133f,-0.9792f,3.5645f,-0.9792f,3.5645f,-0.8750f,3.5156f,-0.8750f,3.5156f,-0.7708f,3.4668f,-0.7708f,3.4668f,-0.6667f,3.4180f,-0.6667f,3.4180f,-0.5625f,3.3691f,-0.5625f,3.3691f,-0.4583f,3.3203f,-0.4583f,3.3203f,-0.3542f,3.2715f,-0.3542f,3.2715f,-0.2500f,
	3.2227f,-0.2500f,3.2227f,-0.1458f,3.1738f,-0.1458f,3.1738f,-0.0417f,3.1250f,-0.0417f,3.1250f,0.0625f,3.0762f,0.0625f,3.0762f,0.1667f,3.0273f,0.1667f,3.0273f,0.2708f,2.9785f,0.2708f,2.9785f,0.3750f,2.9297f,0.3750f,2.9297f,0.4792f,2.8809f,0.4792f,
	2.8809f,0.5833f,2.8320f,0.5833f,2.8320f,0.6875f,2.7832f,0.6875f,2.7832f,0.7917f,2.7344f,0.7917f,2.7344f,0.8958f,2.6855f,0.8958f,2.6855f,1.0000f,2.6367f,1.0000f,4.9805f,-4.0000f,4.9316f,-4.0000f,4.9316f,-3.8958f,4.8828f,-3.8958f,4.8828f,-3.7917f,
	4.8340f,-3.7917f,4.8340f,-3.6875f,4.7852f,-3.6875f,4.7852f,-3.5833f,4.7363f,-3.5833f,4.7363f,-3.4792f,4.6875f,-3.4792f,4.6875f,-3.3750f,4.6387f,-3.3750f,4.6387f,-3.2708f,4.5898f,-3.2708f,4.5898f,-3.1667f,4.5410f,-3.1667f,4.5410f,-3.0625f,4.4922f,-3.0625f,
	4.4922f,-2.9583f,4.4434f,-2.9583f,4.4434f,-2.8542f,4.3945f,-2.8542f,4.3945f,-2.7500f,4.3457f,-2.7500f,4.3457f,-2.6458f,4.2969f,-2.6458f,4.2969f,-2.5417f,4.2480f,-2.5417f,4.2480f,-2.4375f,4.1992f,-2.4375f,4.1992f,-2.3333f,4.1504f,-2.3333f,4.1504f,-2.2292f,
	4.1016f,-2.2292f,4.1016f,-2.1250f,4.0527f,-2.1250f,4.0527f,-2.0208f,4.0039f,-2.0208f,4.0039f,-1.9167f,3.9551f,-1.9167f,3.9551f,-1.8125f,3.9062f,-1.8125f,3.9062f,-1.7083f,3.8574f,-1.7083f,3.8574f,-1.6042f,3.8086f,-1.6042f,3.8086f,-1.5000f,3.7598f,-1.5000f,
	3.7598f,-1.3958f,3.7109f,-1.3958f,3.7109f,-1.2917f,3.6621f,-1.2917f,3.6621f,-1.1875f,3.6133f,-1.1875f,3.6133f,-1.0833f,3.5645f,-1.0833f,3.5645f,-0.9792f,3.5156f,-0.9792f,3.5156f,-0.8750f,3.4668f,-0.8750f,3.4668f,-0.7708f,3.4180f,-0.7708f,3.4180f,-0.6667f,
	3.3691f,-0.6667f,3.3691f,-0.5625f,3.3203f,-0.5625f,3.3203f,-0.4583f,3.2715f,-0.4583f,3.2715f,-0.3542f,3.2227f,-0.3542f,3.2227f,-0.2500f,3.1738f,-0.2500f,3.1738f,-0.1458f,3.1250f,-0.1458f,3.1250f,-0.0417f,3.0762f,-0.0417f,3.0762f,0.0625f,3.0273f,0.0625f,
	3.0273f,0.1667f,2.9785f,0.1667f,2.9785f,0.2708f,2.9297f,0.2708f,2.9297f,0.3750f,2.8809f,0.3750f,2.8809f,0.4792f,2.8320f,0.4792f,2.8320f,0.5833f,2.7832f,0.5833f,2.7832f,0.6875f,2.7344f,0.6875f,2.7344f,0.7917f,2.6855f,0.7917f,2.6855f,0.8958f,
	2.6367f,0.8958f,2.6367f,1.0000f,2.5879f,1.0000f,4.9316f,-4.0000f,4.8828f,-4.0000f,4.8828f,-3.8958f,4.8340f,-3.8958f,4.8340f,-3.7917f,4.7852f,-3.7917f,4.7852f,-3.6875f,4.7363f,-3.6875f,4.7363f,-3.5833f,4.6875f,-3.5833f,4.6875f,-3.4792f,4.6387f,-3.4792f,
	4.6387f,-3.3750f,4.5898f,-3.3750f,4.5898f,-3.2708f,4.5410f,-3.2708f,4.5410f,-3.1667f,4.4922f,-3.1667f,4.4922f,-3.0625f,4.4434f,-3.0625f,4.4434f,-2.9583f,4.3945f,-2.9583f,4.3945f,-2.8542f,4.3457f,-2.8542f,4.3457f,-2.7500f,4.2969f,-2.7500f,4.2969f,-2.6458f,
	4.2480f,-2.6458f,4.2480f,-2.5417f,4.1992f,-2.5417f,4.1992f,-2.4375f,4.1504f,-2.4375f,4.1504f,-2.3333f,4.1016f,-2.3333f,4.1016f,-2.2292f,4.0527f,-2.2292f,4.0527f,-2.1250f,4.0039f,-2.1250f,4.0039f,-2.0208f,3.9551f,-2.0208f,3.9551f,-1.9167f,3.9062f,-1.9167f,
	3.9062f,-1.8125f,3.8574f,-1.8125f,3.8574f,-1.7083f,3.8086f,-1.7083f,3.8086f,-1.6042f,3.7598f,-1.6042f,3.7598f,-1.5000f,3.7109f,-1.5000f,3.7109f,-1.3958f,3.6621f,-1.3958f,3.6621f,-1.2917f,3.6133f,-1.2917f,3.6133f,-1.1875f,3.5645f,-1.1875f,3.5645f,-1.0833f,
	3.5156f,-1.0833f,3.5156f,-0.9792f,3.4668f,-0.9792f,3.4668f,-0.8750f,3.4180f,-0.8750f,3.4180f,-0.7708f,3.3691f,-0.7708f,3.3691f,-0.6667f,3.3203f,-0.6667f,3.3203f,-0.5625f,3.2715f,-0.5625f,3.2715f,-0.4583f,3.2227f,-0.4583f,3.2227f,-0.3542f,3.1738f,-0.3542f,
	3.1738f,-0.2500f,3.1250f,-0.2500f,3.1250f,-0.1458f,3.0762f,-0.1458f,3.0762f,-0.0417f,3.0273f,-0.0417f,3.0273f,0.0625f,2.9785f,0.0625f,2.9785f,0.1667f,2.9297f,0.1667f,2.9297f,0.2708f,2.8809f,0.2708f,2.8809f,0.3750f,2.8320f,0.3750f,2.8320f,0.4792f,
	2.7832f,0.4792f,2.7832f,0.5833f,2.7344f,0.5833f,2.7344f,0.6875f,2.6855f,0.6875f,2.6855f,0.7917f,2.6367f,0.7917f,2.6367f,0.8958f,2.5879f,0.8958f,2.5879f,1.0000f,2.5391f,1.0000f,4.8828f,-4.0000f,4.8340f,-4.0000f,4.8340f,-3.8958f,4.7852f,-3.8958f,
	4.7852f,-3.7917f,4.7363f,-3.7917f,4.7363f,-3.6875f,4.6875f,-3.6875f,4.6875f,-3.5833f,4.6387f,-3.5833f,4.6387f,-3.4792f,4.5898f,-3.4792f,4.5898f,-3.3750f,4.5410f,-3.3750f,4.5410f,-3.2708f,4.4922f,-3.2708f,4.4922f,-3.1667f,4.4434f,-3.1667f,4.4434f,-3.0625f,
	4.3945f,-3.0625f,4.3945f,-2.9583f,4.3457f,-2.9583f,4.3457f,-2.8542f,4.2969f,-2.8542f,4.2969f,-2.7500f,4.2480f,-2.7500f,4.2480f,-2.6458f,4.1992f,-2.6458f,4.1992f,-2.5417f,4.1504f,-2.5417f,4.1504f,-2.4375f,4.1016f,-2.4375f,4.1016f,-2.3333f,4.0527f,-2.3333f,
	4.0527f,-2.2292f,4.0039f,-2.2292f,4.0039f,-2.1250f,3.9551f,-2.1250f,3.9551f,-2.0208f,3.9062f,-2.0208f,3.9062f,-1.9167f,3.8574f,-1.9167f,3.8574f,-1.8125f,3.8086f,-1.8125f,3.8086f,-1.7083f,3.7598f,-1.7083f,3.7598f,-1.6042f,3.7109f,-1.6042f,3.7109f,-1.5000f,
	3.6621f,-1.5000f,3.6621f,-1.3958f,3.6133f,-1.3958f,3.6133f,-1.2917f,3.5645f,-1.2917f,3.5645f,-1.1875f,3.5156f,-1.1875f,3.5156f,-1.0833f,3.4668f,-1.0833f,3.4668f,-0.9792f,3.4180f,-0.9792f,3.4180f,-0.8750f,3.3691f,-0.8750f,3.3691f,-0.7708f,3.3203f,-0.7708f,
	3.3203f,-0.6667f,3.2715f,-0.6667f,3.2715f,-0.5625f,3.2227f,-0.5625f,3.2227f,-0.4583f,3.1738f,-0.4583f,3.1738f,-0.3542f,3.1250f,-0.3542f,3.1250f,-0.2500f,3.0762f,-0.2500f,3.0762f,-0.1458f,3.0273f,-0.1458f,3.0273f,-0.0417f,2.9785f,-0.0417f,2.9785f,0.0625f,
	2.9297f,0.0625f,2.9297f,0.1667f,2.8809f,0.1667f,2.8809f,0.2708f,2.8320f,0.2708f,2.8320f,0.3750f,2.7832f,0.3750f,2.7832f,0.4792f,2.7344f,0.4792f,2.7344f,0.5833f,2.6855f,0.5833f,2.6855f,0.6875f,2.6367f,0.6875f,2.6367f,0.7917f,2.5879f,0.7917f,
	2.5879f,0.8958f,2.5391f,0.8958f,2.5391f,1.0000f,2.4902f,1.0000f,4.8340f,-4.0000f,4.7852f,-4.0000f,4.7852f,-3.8958f,4.7363f,-3.8958f,4.7363f,-3.7917f,4.6875f,-3.7917f,4.6875f,-3.6875f,4.6387f,-3.6875f,4.6387f,-3.5833f,4.5898f,-3.5833f,4.5898f,-3.4792f,
	4.5410f,-3.4792f,4.5410f,-3.3750f,4.4922f,-3.3750f,4.4922f,-3.2708f,4.4434f,-3.2708f,4.4434f,-3.1667f,4.3945f,-3.1667f,4.3945f,-3.0625f,4.3457f,-3.0625f,4.3457f,-2.9583f,4.2969f,-2.9583f,4.2969f,-2.8542f,4.2480f,-2.8542f,4.2480f,-2.7500f,4.1992f,-2.7500f,
	4.1992f,-2.6458f,4.1504f,-2.6458f,4.1504f,-2.5417f,4.1016f,-2.5417f,4.1016f,-2.4375f,4.0527f,-2.4375f,4.0527f,-2.3333f,4.0039f,-2.3333f,4.0039f,-2.2292f,3.9551f,-2.2292f,3.9551f,-2.1250f,3.9062f,-2.1250f,3.9062f,-2.0208f,3.8574f,-2.0208f,3.8574f,-1.9167f,
	3.8086f,-1.9167f,3.8086f,-1.8125f,3.7598f,-1.8125f,3.7598f,-1.7083f,3.7109f,-1.7083f,3.7109f,-1.6042f,3.6621f,-1.6042f,3.6621f,-1.5000f,3.6133f,-1.5000f,3.6133f,-1.3958f,3.5645f,-1.3958f,3.5645f,-1.2917f,3.5156f,-1.2917f,3.5156f,-1.1875f,3.4668f,-1.1875f,
	3.4668f,-1.0833f,3.4180f,-1.0833f,3.4180f,-0.9792f,3.3691f,-0.9792f,3.3691f,-0.8750f,3.3203f,-0.8750f,3.3203f,-0.7708f,3.2715f,-0.7708f,3.2715f,-0.6667f,3.2227f,-0.6667f,3.2227f,-0.5625f,3.1738f,-0.5625f,3.1738f,-0.4583f,3.1250f,-0.4583f,3.1250f,-0.3542f,
	3.0762f,-0.3542f,3.0762f,-0.2500f,3.0273f,-0.2500f,3.0273f,-0.1458f,2.9785f,-0.1458f,2.9785f,-0.0417f,2.9297f,-0.0417f,2.9297f,0.0625f,2.8809f,0.0625f,2.8809f,0.1667f,2.8320f,0.1667f,2.8320f,0.2708f,2.7832f,0.2708f,2.7832f,0.3750f,2.7344f,0.3750f,
	2.7344f,0.4792f,2.6855f,0.4792f,2.6855f,0.5833f,2.6367f,0.5833f,2.6367f,0.6875f,2.5879f,0.6875f,2.5879f,0.7917f,2.5391f,0.7917f,2.5391f,0.8958f,2.4902f,0.8958f,2.4902f,1.0000f,2.4414f,1.0000f,4.7852f,-4.0000f,4.7363f,-4.0000f,4.7363f,-3.8958f,
	4.6875f,-3.8958f,4.6875f,-3.7917f,4.6387f,-3.7917f,4.6387f,-3.6875f,4.5898f,-3.6875f,4.5898f,-3.5833f,4.5410f,-3.5833f,4.5410f,-3.4792f,4.4922f,-3.4792f,4.4922f,-3.3750f,4.4434f,-3.3750f,4.4434f,-3.2708f,4.3945f,-3.2708f,4.3945f,-3.1667f,4.3457f,-3.1667f,
	4.3457f,-3.0625f,4.2969f,-3.0625f,4.2969f,-2.9583f,4.2480f,-2.9583f,4.2480f,-2.8542f,4.1992f,-2.8542f,4.1992f,-2.7500f,4.1504f,-2.7500f,4.1504f,-2.6458f,4.1016f,-2.6458f,4.1016f,-2.5417f,4.0527f,-2.5417f,4.0527f,-2.4375f,4.0039f,-2.4375f,4.0039f,-2.3333f,
	3.9551f,-2.3333f,3.9551f,-2.2292f,3.9062f,-2.2292f,3.9062f,-2.1250f,3.8574f,-2.1250f,3.8574f,-2.0208f,3.8086f,-2.0208f,3.8086f,-1.9167f,3.7598f,-1.9167f,3.7598f,-1.8125f,3.7109f,-1.8125f,3.7109f,-1.7083f,3.6621f,-1.7083f,3.6621f,-1.6042f,3.6133f,-1.6042f,
	3.6133f,-1.5000f,3.5645f,-1.5000f,3.5645f,-1.3958f,3.5156f,-1.3958f,3.5156f,-1.2917f,3.4668f,-1.2917f,3.4668f,-1.1875f,3.4180f,-1.1875f,3.4180f,-1.0833f,3.3691f,-1.0833f,3.3691f,-0.9792f,3.3203f,-0.9792f,3.3203f,-0.8750f,3.2715f,-0.8750f,3.2715f,-0.7708f,
	3.2227f,-0.7708f,3.2227f,-0.6667f,3.1738f,-0.6667f,3.1738f,-0.5625f,3.1250f,-0.5625f,3.1250f,-0.4583f,3.0762f,-0.4583f,3.0762f,-0.3542f,3.0273f,-0.3542f,3.0273f,-0.2500f,2.9785f,-0.2500f,2.9785f,-0.1458f,2.9297f,-0.1458f,2.9297f,-0.0417f,2.8809f,-0.0417f,
	2.8809f,0.0625f,2.8320f,0.0625f,2.8320f,0.1667f,2.7832f,0.1667f,2.7832f,0.2708f,2.7344f,0.2708f,2.7344f,0.3750f,2.6855f,0.3750f,2.6855f,0.4792f,2.6367f,0.4792f,2.6367f,0.5833f,2.5879f,0.5833f,2.5879f,0.6875f,2.5391f,0.6875f,2.5391f,0.7917f,
	2.4902f,0.7917f,2.4902f,0.8958f,2.4414f,0.8958f,2.4414f,1.0000f,2.3926f,1.0000f,4.7363f,-4.0000f,4.6875f,-4.0000f,4.6875f,-3.8958f,4.6387f,-3.8958f,4.6387f,-3.7917f,4.5898f,-3.7917f,4.5898f,-3.6875f,4.5410f,-3.6875f,4.5410f,-3.5833f,4.4922f,-3.5833f,
	4.4922f,-3.4792f,4.4434f,-3.4792f,4.4434f,-3.3750f,4.3945f,-3.3750f,4.3945f,-3.2708f,4.3457f,-3.2708f,4.3457f,-3.1667f,4.2969f,-3.1667f,4.2969f,-3.0625f,4.2480f,-3.0625f,4.2480f,-2.9583f,4.1992f,-2.9583f,4.1992f,-2.8542f,4.1504f,-2.8542f,4.1504f,-2.7500f,
	4.1016f,-2.7500f,4.1016f,-2.6458f,4.0527f,-2.6458f,4.0527f,-2.5417f,4.0039f,-2.5417f,4.0039f,-2.4375f,3.9551f,-2.4375f,3.9551f,-2.3333f,3.9062f,-2.3333f,3.9062f,-2.2292f,3.8574f,-2.2292f,3.8574f,-2.1250f,3.8086f,-2.1250f,3.8086f,-2.0208f,3.7598f,-2.0208f,
	3.7598f,-1.9167f,3.7109f,-1.9167f,3.7109f,-1.8125f,3.6621f,-1.8125f,3.6621f,-1.7083f,3.6133f,-1.7083f,3.6133f,-1.6042f,3.5645f,-1.6042f,3.5645f,-1.5000f,3.5156f,-1.5000f,3.5156f,-1.3958f,3.4668f,-1.3958f,3.4668f,-1.2917f,3.4180f,-1.2917f,3.4180f,-1.1875f,
	3.3691f,-1.1875f,3.3691f,-1.0833f,3.3203f,-1.0833f,3.3203f,-0.9792f,3.2715f,-0.9792f,3.2715f,-0.8750f,3.2227f,-0.8750f,3.2227f,-0.7708f,3.1738f,-0.7708f,3.1738f,-0.6667f,3.1250f,-0.6667f,3.1250f,-0.5625f,3.0762f,-0.5625f,3.0762f,-0.4583f,3.0273f,-0.4583f,
	3.0273f,-0.3542f,2.9785f,-0.3542f,2.9785f,-0.2500f,2.9297f,-0.2500f,2.9297f,-0.1458f,2.8809f,-0.1458f,2.8809f,-0.0417f,2.8320f,-0.0417f,2.8320f,0.0625f,2.7832f,0.0625f,2.7832f,0.1667f,2.7344f,0.1667f,2.7344f,0.2708f,2.6855f,0.2708f,2.6855f,0.3750f,
	2.6367f,0.3750f,2.6367f,0.4792f,2.5879f,0.4792f,2.5879f,0.5833f,2.5391f,0.5833f,2.5391f,0.6875f,2.4902f,0.6875f,2.4902f,0.7917f,2.4414f,0.7917f,2.4414f,0.8958f,2.3926f,0.8958f,2.3926f,1.0000f,2.3437f,1.0000f,4.6875f,-4.0000f,4.6387f,-4.0000f,
	4.6387f,-3.8958f,4.5898f,-3.8958f,4.5898f,-3.7917f,4.5410f,-3.7917f,4.5410f,-3.6875f,4.4922f,-3.6875f,4.4922f,-3.5833f,4.4434f,-3.5833f,4.4434f,-3.4792f,4.3945f,-3.4792f,4.3945f,-3.3750f,4.3457f,-3.3750f,4.3457f,-3.2708f,4.2969f,-3.2708f,4.2969f,-3.1667f,
	4.2480f,-3.1667f,4.2480f,-3.0625f,4.1992f,-3.0625f,4.1992f,-2.9583f,4.1504f,-2.9583f,4.1504f,-2.8542f,4.1016f,-2.8542f,4.1016f,-2.7500f,4.0527f,-2.7500f,4.0527f,-2.6458f,4.0039f,-2.6458f,4.0039f,-2.5417f,3.9551f,-2.5417f,3.9551f,-2.4375f,3.9062f,-2.4375f,
	3.9062f,-2.3333f,3.8574f,-2.3333f,3.8574f,-2.2292f,3.8086f,-2.2292f,3.8086f,-2.1250f,3.7598f,-2.1250f,3.7598f,-2.0208f,3.7109f,-2.0208f,3.7109f,-1.9167f,3.6621f,-1.9167f,3.6621f,-1.8125f,3.6133f,-1.8125f,3.6133f,-1.7083f,3.5645f,-1.7083f,3.5645f,-1.6042f,
	3.5156f,-1.6042f,3.5156f,-1.5000f,3.4668f,-1.5000f,3.4668f,-1.3958f,3.4180f,-1.3958f,3.4180f,-1.2917f,3.3691f,-1.2917f,3.3691f,-1.1875f,3.3203f,-1.1875f,3.3203f,-1.0833f,3.2715f,-1.0833f,3.2715f,-0.9792f,3.2227f,-0.9792f,3.2227f,-0.8750f,3.1738f,-0.8750f,
	3.1738f,-0.7708f,3.1250f,-0.7708f,3.1250f,-0.6667f,3.0762f,-0.6667f,3.0762f,-0.5625f,3.0273f,-0.5625f,3.0273f,-0.4583f,2.9785f,-0.4583f,2.9785f,-0.3542f,2.9297f,-0.3542f,2.9297f,-0.2500f,2.8809f,-0.2500f,2.8809f,-0.1458f,2.8320f,-0.1458f,2.8320f,-0.0417f,
	2.7832f,-0.0417f,2.7832f,0.0625f,2.7344f,0.0625f,2.7344f,0.1667f,2.6855f,0.1667f,2.6855f,0.2708f,2.6367f,0.2708f,2.6367f,0.3750f,2.5879f,0.3750f,2.5879f,0.4792f,2.5391f,0.4792f,2.5391f,0.5833f,2.4902f,0.5833f,2.4902f,0.6875f,2.4414f,0.6875f,
	2.4414f,0.7917f,2.3926f,0.7917f,2.3926f,0.8958f,2.3437f,0.8958f,2.3437f,1.0000f,2.2949f,1.0000f,4.6387f,-4.0000f,4.5898f,-4.0000f,4.5898f,-3.8958f,4.5410f,-3.8958f,4.5410f,-3.7917f,4.4922f,-3.7917f,4.4922f,-3.6875f,4.4434f,-3.6875f,4.4434f,-3.5833f,
	4.3945f,-3.5833f,4.3945f,-3.4792f,4.3457f,-3.4792f,4.3457f,-3.3750f,4.2969f,-3.3750f,4.2969f,-3.2708f,4.2480f,-3.2708f,4.2480f,-3.1667f,4.1992f,-3.1667f,4.1992f,-3.0625f,4.1504f,-3.0625f,4.1504f,-2.9583f,4.1016f,-2.9583f,4.1016f,-2.8542f,4.0527f,-2.8542f,
	4.0527f,-2.7500f,4.0039f,-2.7500f,4.0039f,-2.6458f,3.9551f,-2.6458f,3.9551f,-2.5417f,3.9062f,-2.5417f,3.9062f,-2.4375f,3.8574f,-2.4375f,3.8574f,-2.3333f,3.8086f,-2.3333f,3.8086f,-2.2292f,3.7598f,-2.2292f,3.7598f,-2.1250f,3.7109f,-2.1250f,3.7109f,-2.0208f,
	3.6621f,-2.0208f,3.6621f,-1.9167f,3.6133f,-1.9167f,3.6133f,-1.8125f,3.5645f,-1.8125f,3.5645f,-1.7083f,3.5156f,-1.7083f,3.5156f,-1.6042f,3.4668f,-1.6042f,3.4668f,-1.5000f,3.4180f,-1.5000f,3.4180f,-1.3958f,3.3691f,-1.3958f,3.3691f,-1.2917f,3.3203f,-1.2917f,
	3.3203f,-1.1875f,3.2715f,-1.1875f,3.2715f,-1.0833f,3.2227f,-1.0833f,3.2227f,-0.9792f,3.1738f,-0.9792f,3.1738f,-0.8750f,3.1250f,-0.8750f,3.1250f,-0.7708f,3.0762f,-0.7708f,3.0762f,-0.6667f,3.0273f,-0.6667f,3.0273f,-0.5625f,2.9785f,-0.5625f,2.9785f,-0.4583f,
	2.9297f,-0.4583f,2.9297f,-0.3542f,2.8809f,-0.3542f,2.8809f,-0.2500f,2.8320f,-0.2500f,2.8320f,-0.1458f,2.7832f,-0.1458f,2.7832f,-0.0417f,2.7344f,-0.0417f,2.7344f,0.0625f,2.6855f,0.0625f,2.6855f,0.1667f,2.6367f,0.1667f,2.6367f,0.2708f,2.5879f,0.2708f,
	2.5879f,0.3750f,2.5391f,0.3750f,2.5391f,0.4792f,2.4902f,0.4792f,2.4902f,0.5833f,2.4414f,0.5833f,2.4414f,0.6875f,2.3926f,0.6875f,2.3926f,0.7917f,2.3437f,0.7917f,2.3437f,0.8958f,2.2949f,0.8958f,2.2949f,1.0000f,2.2461f,1.0000f,4.5898f,-4.0000f,
	4.5410f,-4.0000f,4.5410f,-3.8958f,4.4922f,-3.8958f,4.4922f,-3.7917f,4.4434f,-3.7917f,4.4434f,-3.6875f,4.3945f,-3.6875f,4.3945f,-3.5833f,4.3457f,-3.5833f,4.3457f,-3.4792f,4.2969f,-3.4792f,4.2969f,-3.3750f,4.2480f,-3.3750f,4.2480f,-3.2708f,4.1992f,-3.2708f,
	4.1992f,-3.1667f,4.1504f,-3.1667f,4.1504f,-3.0625f,4.1016f,-3.0625f,4.1016f,-2.9583f,4.0527f,-2.9583f,4.0527f,-2.8542f,4.0039f,-2.8542f,4.0039f,-2.7500f,3.9551f,-2.7500f,3.9551f,-2.6458f,3.9062f,-2.6458f,3.9062f,-2.5417f,3.8574f,-2.5417f,3.8574f,-2.4375f,
	3.8086f,-2.4375f,3.8086f,-2.3333f,3.7598f,-2.3333f,3.7598f,-2.2292f,3.7109f,-2.2292f,3.7109f,-2.1250f,3.6621f,-2.1250f,3.6621f,-2.0208f,3.6133f,-2.0208f,3.6133f,-1.9167f,3.5645f,-1.9167f,3.5645f,-1.8125f,3.5156f,-1.8125f,3.5156f,-1.7083f,3.4668f,-1.7083f,
	3.4668f,-1.6042f,3.4180f,-1.6042f,3.4180f,-1.5000f,3.3691f,-1.5000f,3.3691f,-1.3958f,3.3203f,-1.3958f,3.3203f,-1.2917f,3.2715f,-1.2917f,3.2715f,-1.1875f,3.2227f,-1.1875f,3.2227f,-1.0833f,3.1738f,-1.0833f,3.1738f,-0.9792f,3.1250f,-0.9792f,3.1250f,-0.8750f,
	3.0762f,-0.8750f,3.0762f,-0.7708f,3.0273f,-0.7708f,3.0273f,-0.6667f,2.9785f,-0.6667f,2.9785f,-0.5625f,2.9297f,-0.5625f,2.9297f,-0.4583f,2.8809f,-0.4583f,2.8809f,-0.3542f,2.8320f,-0.3542f,2.8320f,-0.2500f,2.7832f,-0.2500f,2.7832f,-0.1458f,2.7344f,-0.1458f,
	2.7344f,-0.0417f,2.6855f,-0.0417f,2.6855f,0.0625f,2.6367f,0.0625f,2.6367f,0.1667f,2.5879f,0.1667f,2.5879f,0.2708f,2.5391f,0.2708f,2.5391f,0.3750f,2.4902f,0.3750f,2.4902f,0.4792f,2.4414f,0.4792f,2.4414f,0.5833f,2.3926f,0.5833f,2.3926f,0.6875f,
	2.3437f,0.6875f,2.3437f,0.7917f,2.2949f,0.7917f,2.2949f,0.8958f,2.2461f,0.8958f,2.2461f,1.0000f,2.1973f,1.0000f,4.5410f,-4.0000f,4.4922f,-4.0000f,4.4922f,-3.8958f,4.4434f,-3.8958f,4.4434f,-3.7917f,4.3945f,-3.7917f,4.3945f,-3.6875f,4.3457f,-3.6875f,
	4.3457f,-3.5833f,4.2969f,-3.5833f,4.2969f,-3.4792f,4.2480f,-3.4792f,4.2480f,-3.3750f,4.1992f,-3.3750f,4.1992f,-3.2708f,4.1504f,-3.2708f,4.1504f,-3.1667f,4.1016f,-3.1667f,4.1016f,-3.0625f,4.0527f,-3.0625f,4.0527f,-2.9583f,4.0039f,-2.9583f,4.0039f,-2.8542f,
	3.9551f,-2.8542f,3.9551f,-2.7500f,3.9062f,-2.7500f,3.9062f,-2.6458f,3.8574f,-2.6458f,3.8574f,-2.5417f,3.8086f,-2.5417f,3.8086f,-2.4375f,3.7598f,-2.4375f,3.7598f,-2.3333f,3.7109f,-2.3333f,3.7109f,-2.2292f,3.6621f,-2.2292f,3.6621f,-2.1250f,3.6133f,-2.1250f,
	3.6133f,-2.0208f,3.5645f,-2.0208f,3.5645f,-1.9167f,3.5156f,-1.9167f,3.5156f,-1.8125f,3.4668f,-1.8125f,3.4668f,-1.7083f,3.4180f,-1.7083f,3.4180f,-1.6042f,3.3691f,-1.6042f,3.3691f,-1.5000f,3.3203f,-1.5000f,3.3203f,-1.3958f,3.2715f,-1.3958f,3.2715f,-1.2917f,
	3.2227f,-1.2917f,3.2227f,-1.1875f,3.1738f,-1.1875f,3.1738f,-1.0833f,3.1250f,-1.0833f,3.1250f,-0.9792f,3.0762f,-0.9792f,3.0762f,-0.8750f,3.0273f,-0.8750f,3.0273f,-0.7708f,2.9785f,-0.7708f,2.9785f,-0.6667f,2.9297f,-0.6667f,2.9297f,-0.5625f,2.8809f,-0.5625f,
	2.8809f,-0.4583f,2.8320f,-0.4583f,2.8320f,-0.3542f,2.7832f,-0.3542f,2.7832f,-0.2500f,2.7344f,-0.2500f,2.7344f,-0.1458f,2.6855f,-0.1458f,2.6855f,-0.0417f,2.6367f,-0.0417f,2.6367f,0.0625f,2.5879f,0.0625f,2.5879f,0.1667f,2.5391f,0.1667f,2.5391f,0.2708f,
	2.4902f,0.2708f,2.4902f,0.3750f,2.4414f,0.3750f,2.4414f,0.4792f,2.3926f,0.4792f,2.3926f,0.5833f,2.3437f,0.5833f,2.3437f,0.6875f,2.2949f,0.6875f,2.2949f,0.7917f,2.2461f,0.7917f,2.2461f,0.8958f,2.1973f,0.8958f,2.1973f,1.0000f,2.1484f,1.0000f,
	4.4922f,-4.0000f,4.4434f,-4.0000f,4.4434f,-3.8958f,4.3945f,-3.8958f,4.3945f,-3.7917f,4.3457f,-3.7917f,4.3457f,-3.6875f,4.2969f,-3.6875f,4.2969f,-3.5833f,4.2480f,-3.5833f,4.2480f,-3.4792f,4.1992f,-3.4792f,4.1992f,-3.3750f,4.1504f,-3.3750f,4.1504f,-3.2708f,
	4.1016f,-3.2708f,4.1016f,-3.1667f,4.0527f,-3.1667f,4.0527f,-3.0625f,4.0039f,-3.0625f,4.0039f,-2.9583f,3.9551f,-2.9583f,3.9551f,-2.8542f,3.9062f,-2.8542f,3.9062f,-2.7500f,3.8574f,-2.7500f,3.8574f,-2.6458f,3.8086f,-2.6458f,3.8086f,-2.5417f,3.7598f,-2.5417f,
	3.7598f,-2.4375f,3.7109f,-2.4375f,3.7109f,-2.3333f,3.6621f,-2.3333f,3.6621f,-2.2292f,3.6133f,-2.2292f,3.6133f,-2.1250f,3.5645f,-2.1250f,3.5645f,-2.0208f,3.5156f,-2.0208f,3.5156f,-1.9167f,3.4668f,-1.9167f,3.4668f,-1.8125f,3.4180f,-1.8125f,3.4180f,-1.7083f,
	3.3691f,-1.7083f,3.3691f,-1.6042f,3.3203f,-1.6042f,3.3203f,-1.5000f,3.2715f,-1.5000f,3.2715f,-1.3958f,3.2227f,-1.3958f,3.2227f,-1.2917f,3.1738f,-1.2917f,3.1738f,-1.1875f,3.1250f,-1.1875f,3.1250f,-1.0833f,3.0762f,-1.0833f,3.0762f,-0.9792f,3.0273f,-0.9792f,
	3.0273f,-0.8750f,2.9785f,-0.8750f,2.9785f,-0.7708f,2.9297f,-0.7708f,2.9297f,-0.6667f,2.8809f,-0.6667f,2.8809f,-0.5625f,2.8320f,-0.5625f,2.8320f,-0.4583f,2.7832f,-0.4583f,2.7832f,-0.3542f,2.7344f,-0.3542f,2.7344f,-0.2500f,2.6855f,-0.2500f,2.6855f,-0.1458f,
	2.6367f,-0.1458f,2.6367f,-0.0417f,2.5879f,-0.0417f,2.5879f,0.0625f,2.5391f,0.0625f,2.5391f,0.1667f,2.4902f,0.1667f,2.4902f,0.2708f,2.4414f,0.2708f,2.4414f,0.3750f,2.3926f,0.3750f,2.3926f,0.4792f,2.3437f,0.4792f,2.3437f,0.5833f,2.2949f,0.5833f,
	2.2949f,0.6875f,2.2461f,0.6875f,2.2461f,0.7917f,2.1973f,0.7917f,2.1973f,0.8958f,2.1484f,0.8958f,2.1484f,1.0000f,2.0996f,1.0000f,4.4434f,-4.0000f,4.3945f,-4.0000f,4.3945f,-3.8958f,4.3457f,-3.8958f,4.3457f,-3.7917f,4.2969f,-3.7917f,4.2969f,-3.6875f,
	4.2480f,-3.6875f,4.2480f,-3.5833f,4.1992f,-3.5833f,4.1992f,-3.4792f,4.1504f,-3.4792f,4.1504f,-3.3750f,4.1016f,-3.3750f,4.1016f,-3.2708f,4.0527f,-3.2708f,4.0527f,-3.1667f,4.0039f,-3.1667f,4.0039f,-3.0625f,3.9551f,-3.0625f,3.9551f,-2.9583f,3.9062f,-2.9583f,
	3.9062f,-2.8542f,3.8574f,-2.8542f,3.8574f,-2.7500f,3.8086f,-2.7500f,3.8086f,-2.6458f,3.7598f,-2.6458f,3.7598f,-2.5417f,3.7109f,-2.5417f,3.7109f,-2.4375f,3.6621f,-2.4375f,3.6621f,-2.3333f,3.6133f,-2.3333f,3.6133f,-2.2292f,3.5645f,-2.2292f,3.5645f,-2.1250f,
	3.5156f,-2.1250f,3.5156f,-2.0208f,3.4668f,-2.0208f,3.4668f,-1.9167f,3.4180f,-1.9167f,3.4180f,-1.8125f,3.3691f,-1.8125f,3.3691f,-1.7083f,3.3203f,-1.7083f,3.3203f,-1.6042f,3.2715f,-1.6042f,3.2715f,-1.5000f,3.2227f,-1.5000f,3.2227f,-1.3958f,3.1738f,-1.3958f,
	3.1738f,-1.2917f,3.1250f,-1.2917f,3.1250f,-1.1875f,3.0762f,-1.1875f,3.0762f,-1.0833f,3.0273f,-1.0833f,3.0273f,-0.9792f,2.9785f,-0.9792f,2.9785f,-0.8750f,2.9297f,-0.8750f,2.9297f,-0.7708f,2.8809f,-0.7708f,2.8809f,-0.6667f,2.8320f,-0.6667f,2.8320f,-0.5625f,
	2.7832f,-0.5625f,2.7832f,-0.4583f,2.7344f,-0.4583f,2.7344f,-0.3542f,2.6855f,-0.3542f,2.6855f,-0.2500f,2.6367f,-0.2500f,2.6367f,-0.1458f,2.5879f,-0.1458f,2.5879f,-0.0417f,2.5391f,-0.0417f,2.5391f,0.0625f,2.4902f,0.0625f,2.4902f,0.1667f,2.4414f,0.1667f,
	2.4414f,0.2708f,2.3926f,0.2708f,2.3926f,0.3750f,2.3437f,0.3750f,2.3437f,0.4792f,2.2949f,0.4792f,2.2949f,0.5833f,2.2461f,0.5833f,2.2461f,0.6875f,2.1973f,0.6875f,2.1973f,0.7917f,2.1484f,0.7917f,2.1484f,0.8958f,2.0996f,0.8958f,2.0996f,1.0000f,
	2.0508f,1.0000f,4.3945f,-4.0000f,4.3457f,-4.0000f,4.3457f,-3.8958f,4.2969f,-3.8958f,4.2969f,-3.7917f,4.2480f,-3.7917f,4.2480f,-3.6875f,4.1992f,-3.6875f,4.1992f,-3.5833f,4.1504f,-3.5833f,4.1504f,-3.4792f,4.1016f,-3.4792f,4.1016f,-3.3750f,4.0527f,-3.3750f,
	4.0527f,-3.2708f,4.0039f,-3.2708f,4.0039f,-3.1667f,3.9551f,-3.1667f,3.9551f,-3.0625f,3.9062f,-3.0625f,3.9062f,-2.9583f,3.8574f,-2.9583f,3.8574f,-2.8542f,3.8086f,-2.8542f,3.8086f,-2.7500f,3.7598f,-2.7500f,3.7598f,-2.6458f,3.7109f,-2.6458f,3.7109f,-2.5417f,
	3.6621f,-2.5417f,3.6621f,-2.4375f,3.6133f,-2.4375f,3.6133f,-2.3333f,3.5645f,-2.3333f,3.5645f,-2.2292f,3.5156f,-2.2292f,3.5156f,-2.1250f,3.4668f,-2.1250f,3.4668f,-2.0208f,3.4180f,-2.0208f,3.4180f,-1.9167f,3.3691f,-1.9167f,3.3691f,-1.8125f,3.3203f,-1.8125f,
	3.3203f,-1.7083f,3.2715f,-1.7083f,3.2715f,-1.6042f,3.2227f,-1.6042f,3.2227f,-1.5000f,3.1738f,-1.5000f,3.1738f,-1.3958f,3.1250f,-1.3958f,3.1250f,-1.2917f,3.0762f,-1.2917f,3.0762f,-1.1875f,3.0273f,-1.1875f,3.0273f,-1.0833f,2.9785f,-1.0833f,2.9785f,-0.9792f,
	2.9297f,-0.9792f,2.9297f,-0.8750f,2.8809f,-0.8750f,2.8809f,-0.7708f,2.8320f,-0.7708f,2.8320f,-0.6667f,2.7832f,-0.6667f,2.7832f,-0.5625f,2.7344f,-0.5625f,2.7344f,-0.4583f,2.6855f,-0.4583f,2.6855f,-0.3542f,2.6367f,-0.3542f,2.6367f,-0.2500f,2.5879f,-0.2500f,
	2.5879f,-0.1458f,2.5391f,-0.1458f,2.5391f,-0.0417f,2.4902f,-0.0417f,2.4902f,0.0625f,2.4414f,0.0625f,2.4414f,0.1667f,2.3926f,0.1667f,2.3926f,0.2708f,2.3437f,0.2708f,2.3437f,0.3750f,2.2949f,0.3750f,2.2949f,0.4792f,2.2461f,0.4792f,2.2461f,0.5833f,
	2.1973f,0.5833f,2.1973f,0.6875f,2.1484f,0.6875f,2.1484f,0.7917f,2.0996f,0.7917f,2.0996f,0.8958f,2.0508f,0.8958f,2.0508f,1.0000f,2.0020f,1.0000f,4.3457f,-4.0000f,4.2969f,-4.0000f,4.2969f,-3.8958f,4.2480f,-3.8958f,4.2480f,-3.7917f,4.1992f,-3.7917f,
	4.1992f,-3.6875f,4.1504f,-3.6875f,4.1504f,-3.5833f,4.1016f,-3.5833f,4.1016f,-3.4792f,4.0527f,-3.4792f,4.0527f,-3.3750f,4.0039f,-3.3750f,4.0039f,-3.2708f,3.9551f,-3.2708f,3.9551f,-3.1667f,3.9062f,-3.1667f,3.9062f,-3.0625f,3.8574f,-3.0625f,3.8574f,-2.9583f,
	3.8086f,-2.9583f,3.8086f,-2.8542f,3.7598f,-2.8542f,3.7598f,-2.7500f,3.7109f,-2.7500f,3.7109f,-2.6458f,3.6621f,-2.6458f,3.6621f,-2.5417f,3.6133f,-2.5417f,3.6133f,-2.4375f,3.5645f,-2.4375f,3.5645f,-2.3333f,3.5156f,-2.3333f,3.5156f,-2.2292f,3.4668f,-2.2292f,
	3.4668f,-2.1250f,3.4180f,-2.1250f,3.4180f,-2.0208f,3.3691f,-2.0208f,3.3691f,-1.9167f,3.3203f,-1.9167f,3.3203f,-1.8125f,3.2715f,-1.8125f,3.2715f,-1.7083f,3.2227f,-1.7083f,3.2227f,-1.6042f,3.1738f,-1.6042f,3.1738f,-1.5000f,3.1250f,-1.5000f,3.1250f,-1.3958f,
	3.0762f,-1.3958f,3.0762f,-1.2917f,3.0273f,-1.2917f,3.0273f,-1.1875f,2.9785f,-1.1875f,2.9785f,-1.0833f,2.9297f,-1.0833f,2.9297f,-0.9792f,2.8809f,-0.9792f,2.8809f,-0.8750f,2.8320f,-0.8750f,2.8320f,-0.7708f,2.7832f,-0.7708f,2.7832f,-0.6667f,2.7344f,-0.6667f,
	2.7344f,-0.5625f,2.6855f,-0.5625f,2.6855f,-0.4583f,2.6367f,-0.4583f,2.6367f,-0.3542f,2.5879f,-0.3542f,2.5879f,-0.2500f,2.5391f,-0.2500f,2.5391f,-0.1458f,2.4902f,-0.1458f,2.4902f,-0.0417f,2.4414f,-0.0417f,2.4414f,0.0625f,2.3926f,0.0625f,2.3926f,0.1667f,
	2.3437f,0.1667f,2.3437f,0.2708f,2.2949f,0.2708f,2.2949f,0.3750f,2.2461f,0.3750f,2.2461f,0.4792f,2.1973f,0.4792f,2.1973f,0.5833f,2.1484f,0.5833f,2.1484f,0.6875f,2.0996f,0.6875f,2.0996f,0.7917f,2.0508f,0.7917f,2.0508f,0.8958f,2.0020f,0.8958f,
	2.0020f,1.0000f,1.9531f,1.0000f,4.2969f,-4.0000f,4.2480f,-4.0000f,4.2480f,-3.8958f,4.1992f,-3.8958f,4.1992f,-3.7917f,4.1504f,-3.7917f,4.1504f,-3.6875f,4.1016f,-3.6875f,4.1016f,-3.5833f,4.0527f,-3.5833f,4.0527f,-3.4792f,4.0039f,-3.4792f,4.0039f,-3.3750f,
	3.9551f,-3.3750f,3.9551f,-3.2708f,3.9062f,-3.2708f,3.9062f,-3.1667f,3.8574f,-3.1667f,3.8574f,-3.0625f,3.8086f,-3.0625f,3.8086f,-2.9583f,3.7598f,-2.9583f,3.7598f,-2.8542f,3.7109f,-2.8542f,3.7109f,-2.7500f,3.6621f,-2.7500f,3.6621f,-2.6458f,3.6133f,-2.6458f,
	3.6133f,-2.5417f,3.5645f,-2.5417f,3.5645f,-2.4375f,3.5156f,-2.4375f,3.5156f,-2.3333f,3.4668f,-2.3333f,3.4668f,-2.2292f,3.4180f,-2.2292f,3.4180f,-2.1250f,3.3691f,-2.1250f,3.3691f,-2.0208f,3.3203f,-2.0208f,3.3203f,-1.9167f,3.2715f,-1.9167f,3.2715f,-1.8125f,
	3.2227f,-1.8125f,3.2227f,-1.7083f,3.1738f,-1.7083f,3.1738f,-1.6042f,3.1250f,-1.6042f,3.1250f,-1.5000f,3.0762f,-1.5000f,3.0762f,-1.3958f,3.0273f,-1.3958f,3.0273f,-1.2917f,2.9785f,-1.2917f,2.9785f,-1.1875f,2.9297f,-1.1875f,2.9297f,-1.0833f,2.8809f,-1.0833f,
	2.8809f,-0.9792f,2.8320f,-0.9792f,2.8320f,-0.8750f,2.7832f,-0.8750f,2.7832f,-0.7708f,2.7344f,-0.7708f,2.7344f,-0.6667f,2.6855f,-0.6667f,2.6855f,-0.5625f,2.6367f,-0.5625f,2.6367f,-0.4583f,2.5879f,-0.4583f,2.5879f,-0.3542f,2.5391f,-0.3542f,2.5391f,-0.2500f,
	2.4902f,-0.2500f,2.4902f,-0.1458f,2.4414f,-0.1458f,2.4414f,-0.0417f,2.3926f,-0.0417f,2.3926f,0.0625f,2.3437f,0.0625f,2.3437f,0.1667f,2.2949f,0.1667f,2.2949f,0.2708f,2.2461f,0.2708f,2.2461f,0.3750f,2.1973f,0.3750f,2.1973f,0.4792f,2.1484f,0.4792f,
	2.1484f,0.5833f,2.0996f,0.5833f,2.0996f,0.6875f,2.0508f,0.6875f,2.0508f,0.7917f,2.0020f,0.7917f,2.0020f,0.8958f,1.9531f,0.8958f,1.9531f,1.0000f,1.9043f,1.0000f,4.2480f,-4.0000f,4.1992f,-4.0000f,4.1992f,-3.8958f,4.1504f,-3.8958f,4.1504f,-3.7917f,
	4.1016f,-3.7917f,4.1016f,-3.6875f,4.0527f,-3.6875f,4.0527f,-3.5833f,4.0039f,-3.5833f,4.0039f,-3.4792f,3.9551f,-3.4792f,3.9551f,-3.3750f,3.9062f,-3.3750f,3.9062f,-3.2708f,3.8574f,-3.2708f,3.8574f,-3.1667f,3.8086f,-3.1667f,3.8086f,-3.0625f,3.7598f,-3.0625f,
	3.7598f,-2.9583f,3.7109f,-2.9583f,3.7109f,-2.8542f,3.6621f,-2.8542f,3.6621f,-2.7500f,3.6133f,-2.7500f,3.6133f,-2.6458f,3.5645f,-2.6458f,3.5645f,-2.5417f,3.5156f,-2.5417f,3.5156f,-2.4375f,3.4668f,-2.4375f,3.4668f,-2.3333f,3.4180f,-2.3333f,3.4180f,-2.2292f,
	3.3691f,-2.2292f,3.3691f,-2.1250f,3.3203f,-2.1250f,3.3203f,-2.0208f,3.2715f,-2.0208f,3.2715f,-1.9167f,3.2227f,-1.9167f,3.2227f,-1.8125f,3.1738f,-1.8125f,3.1738f,-1.7083f,3.1250f,-1.7083f,3.1250f,-1.6042f,3.0762f,-1.6042f,3.0762f,-1.5000f,3.0273f,-1.5000f,
	3.0273f,-1.3958f,2.9785f,-1.3958f,2.9785f,-1.2917f,2.9297f,-1.2917f,2.9297f,-1.1875f,2.8809f,-1.1875f,2.8809f,-1.0833f,2.8320f,-1.0833f,2.8320f,-0.9792f,2.7832f,-0.9792f,2.7832f,-0.8750f,2.7344f,-0.8750f,2.7344f,-0.7708f,2.6855f,-0.7708f,2.6855f,-0.6667f,
	2.6367f,-0.6667f,2.6367f,-0.5625f,2.5879f,-0.5625f,2.5879f,-0.4583f,2.5391f,-0.4583f,2.5391f,-0.3542f,2.4902f,-0.3542f,2.4902f,-0.2500f,2.4414f,-0.2500f,2.4414f,-0.1458f,2.3926f,-0.1458f,2.3926f,-0.0417f,2.3437f,-0.0417f,2.3437f,0.0625f,2.2949f,0.0625f,
	2.2949f,0.1667f,2.2461f,0.1667f,2.2461f,0.2708f,2.1973f,0.2708f,2.1973f,0.3750f,2.1484f,0.3750f,2.1484f,0.4792f,2.0996f,0.4792f,2.0996f,0.5833f,2.0508f,0.5833f,2.0508f,0.6875f,2.0020f,0.6875f,2.0020f,0.7917f,1.9531f,0.7917f,1.9531f,0.8958f,
	1.9043f,0.8958f,1.9043f,1.0000f,1.8555f,1.0000f,4.1992f,-4.0000f,4.1504f,-4.0000f,4.1504f,-3.8958f,4.1016f,-3.8958f,4.1016f,-3.7917f,4.0527f,-3.7917f,4.0527f,-3.6875f,4.0039f,-3.6875f,4.0039f,-3.5833f,3.9551f,-3.5833f,3.9551f,-3.4792f,3.9062f,-3.4792f,
	3.9062f,-3.3750f,3.8574f,-3.3750f,3.8574f,-3.2708f,3.8086f,-3.2708f,3.8086f,-3.1667f,3.7598f,-3.1667f,3.7598f,-3.0625f,3.7109f,-3.0625f,3.7109f,-2.9583f,3.6621f,-2.9583f,3.6621f,-2.8542f,3.6133f,-2.8542f,3.6133f,-2.7500f,3.5645f,-2.7500f,3.5645f,-2.6458f,
	3.5156f,-2.6458f,3.5156f,-2.5417f,3.4668f,-2.5417f,3.4668f,-2.4375f,3.4180f,-2.4375f,3.4180f,-2.3333f,3.3691f,-2.3333f,3.3691f,-2.2292f,3.3203f,-2.2292f,3.3203f,-2.1250f,3.2715f,-2.1250f,3.2715f,-2.0208f,3.2227f,-2.0208f,3.2227f,-1.9167f,3.1738f,-1.9167f,
	3.1738f,-1.8125f,3.1250f,-1.8125f,3.1250f,-1.7083f,3.0762f,-1.7083f,3.0762f,-1.6042f,3.0273f,-1.6042f,3.0273f,-1.5000f,2.9785f,-1.5000f,2.9785f,-1.3958f,2.9297f,-1.3958f,2.9297f,-1.2917f,2.8809f,-1.2917f,2.8809f,-1.1875f,2.8320f,-1.1875f,2.8320f,-1.0833f,
	2.7832f,-1.0833f,2.7832f,-0.9792f,2.7344f,-0.9792f,2.7344f,-0.8750f,2.6855f,-0.8750f,2.6855f,-0.7708f,2.6367f,-0.7708f,2.6367f,-0.6667f,2.5879f,-0.6667f,2.5879f,-0.5625f,2.5391f,-0.5625f,2.5391f,-0.4583f,2.4902f,-0.4583f,2.4902f,-0.3542f,2.4414f,-0.3542f,
	2.4414f,-0.2500f,2.3926f,-0.2500f,2.3926f,-0.1458f,2.3437f,-0.1458f,2.3437f,-0.0417f,2.2949f,-0.0417f,2.2949f,0.0625f,2.2461f,0.0625f,2.2461f,0.1667f,2.1973f,0.1667f,2.1973f,0.2708f,2.1484f,0.2708f,2.1484f,0.3750f,2.0996f,0.3750f,2.0996f,0.4792f,
	2.0508f,0.4792f,2.0508f,0.5833f,2.0020f,0.5833f,2.0020f,0.6875f,1.9531f,0.6875f,1.9531f,0.7917f,1.9043f,0.7917f,1.9043f,0.8958f,1.8555f,0.8958f,1.8555f,1.0000f,1.8066f,1.0000f,4.1504f,-4.0000f,4.1016f,-4.0000f,4.1016f,-3.8958f,4.0527f,-3.8958f,
	4.0527f,-3.7917f,4.0039f,-3.7917f,4.0039f,-3.6875f,3.9551f,-3.6875f,3.9551f,-3.5833f,3.9062f,-3.5833f,3.9062f,-3.4792f,3.8574f,-3.4792f,3.8574f,-3.3750f,3.8086f,-3.3750f,3.8086f,-3.2708f,3.7598f,-3.2708f,3.7598f,-3.1667f,3.7109f,-3.1667f,3.7109f,-3.0625f,
	3.6621f,-3.0625f,3.6621f,-2.9583f,3.6133f,-2.9583f,3.6133f,-2.8542f,3.5645f,-2.8542f,3.5645f,-2.7500f,3.5156f,-2.7500f,3.5156f,-2.6458f,3.4668f,-2.6458f,3.4668f,-2.5417f,3.4180f,-2.5417f,3.4180f,-2.4375f,3.3691f,-2.4375f,3.3691f,-2.3333f,3.3203f,-2.3333f,
	3.3203f,-2.2292f,3.2715f,-2.2292f,3.2715f,-2.1250f,3.2227f,-2.1250f,3.2227f,-2.0208f,3.1738f,-2.0208f,3.1738f,-1.9167f,3.1250f,-1.9167f,3.1250f,-1.8125f,3.0762f,-1.8125f,3.0762f,-1.7083f,3.0273f,-1.7083f,3.0273f,-1.6042f,2.9785f,-1.6042f,2.9785f,-1.5000f,
	2.9297f,-1.5000f,2.9297f,-1.3958f,2.8809f,-1.3958f,2.8809f,-1.2917f,2.8320f,-1.2917f,2.8320f,-1.1875f,2.7832f,-1.1875f,2.7832f,-1.0833f,2.7344f,-1.0833f,2.7344f,-0.9792f,2.6855f,-0.9792f,2.6855f,-0.8750f,2.6367f,-0.8750f,2.6367f,-0.7708f,2.5879f,-0.7708f,
	2.5879f,-0.6667f,2.5391f,-0.6667f,2.5391f,-0.5625f,2.4902f,-0.5625f,2.4902f,-0.4583f,2.4414f,-0.4583f,2.4414f,-0.3542f,2.3926f,-0.3542f,2.3926f,-0.2500f,2.3437f,-0.2500f,2.3437f,-0.1458f,2.2949f,-0.1458f,2.2949f,-0.0417f,2.2461f,-0.0417f,2.2461f,0.0625f,
	2.1973f,0.0625f,2.1973f,0.1667f,2.1484f,0.1667f,2.1484f,0.2708f,2.0996f,0.2708f,2.0996f,0.3750f,2.0508f,0.3750f,2.0508f,0.4792f,2.0020f,0.4792f,2.0020f,0.5833f,1.9531f,0.5833f,1.9531f,0.6875f,1.9043f,0.6875f,1.9043f,0.7917f,1.8555f,0.7917f,
	1.8555f,0.8958f,1.8066f,0.8958f,1.8066f,1.0000f,1.7578f,1.0000f,4.1016f,-4.0000f,4.0527f,-4.0000f,4.0527f,-3.8958f,4.0039f,-3.8958f,4.0039f,-3.7917f,3.9551f,-3.7917f,3.9551f,-3.6875f,3.9062f,-3.6875f,3.9062f,-3.5833f,3.8574f,-3.5833f,3.8574f,-3.4792f,
	3.8086f,-3.4792f,3.8086f,-3.3750f,3.7598f,-3.3750f,3.7598f,-3.2708f,3.7109f,-3.2708f,3.7109f,-3.1667f,3.6621f,-3.1667f,3.6621f,-3.0625f,3.6133f,-3.0625f,3.6133f,-2.9583f,3.5645f,-2.9583f,3.5645f,-2.8542f,3.5156f,-2.8542f,3.5156f,-2.7500f,3.4668f,-2.7500f,
	3.4668f,-2.6458f,3.4180f,-2.6458f,3.4180f,-2.5417f,3.3691f,-2.5417f,3.3691f,-2.4375f,3.3203f,-2.4375f,3.3203f,-2.3333f,3.2715f,-2.3333f,3.2715f,-2.2292f,3.2227f,-2.2292f,3.2227f,-2.1250f,3.1738f,-2.1250f,3.1738f,-2.0208f,3.1250f,-2.0208f,3.1250f,-1.9167f,
	3.0762f,-1.9167f,3.0762f,-1.8125f,3.0273f,-1.8125f,3.0273f,-1.7083f,2.9785f,-1.7083f,2.9785f,-1.6042f,2.9297f,-1.6042f,2.9297f,-1.5000f,2.8809f,-1.5000f,2.8809f,-1.3958f,2.8320f,-1.3958f,2.8320f,-1.2917f,2.7832f,-1.2917f,2.7832f,-1.1875f,2.7344f,-1.1875f,
	2.7344f,-1.0833f,2.6855f,-1.0833f,2.6855f,-0.9792f,2.6367f,-0.9792f,2.6367f,-0.8750f,2.5879f,-0.8750f,2.5879f,-0.7708f,2.5391f,-0.7708f,2.5391f,-0.6667f,2.4902f,-0.6667f,2.4902f,-0.5625f,2.4414f,-0.5625f,2.4414f,-0.4583f,2.3926f,-0.4583f,2.3926f,-0.3542f,
	2.3437f,-0.3542f,2.3437f,-0.2500f,2.2949f,-0.2500f,2.2949f,-0.1458f,2.2461f,-0.1458f,2.2461f,-0.0417f,2.1973f,-0.0417f,2.1973f,0.0625f,2.1484f,0.0625f,2.1484f,0.1667f,2.0996f,0.1667f,2.0996f,0.2708f,2.0508f,0.2708f,2.0508f,0.3750f,2.0020f,0.3750f,
	2.0020f,0.4792f,1.9531f,0.4792f,1.9531f,0.5833f,1.9043f,0.5833f,1.9043f,0.6875f,1.8555f,0.6875f,1.8555f,0.7917f,1.8066f,0.7917f,1.8066f,0.8958f,1.7578f,0.8958f,1.7578f,1.0000f,1.7090f,1.0000f,4.0527f,-4.0000f,4.0039f,-4.0000f,4.0039f,-3.8958f,
	3.9551f,-3.8958f,3.9551f,-3.7917f,3.9062f,-3.7917f,3.9062f,-3.6875f,3.8574f,-3.6875f,3.8574f,-3.5833f,3.8086f,-3.5833f,3.8086f,-3.4792f,3.7598f,-3.4792f,3.7598f,-3.3750f,3.7109f,-3.3750f,3.7109f,-3.2708f,3.6621f,-3.2708f,3.6621f,-3.1667f,3.6133f,-3.1667f,
	3.6133f,-3.0625f,3.5645f,-3.0625f,3.5645f,-2.9583f,3.5156f,-2.9583f,3.5156f,-2.8542f,3.4668f,-2.8542f,3.4668f,-2.7500f,3.4180f,-2.7500f,3.4180f,-2.6458f,3.3691f,-2.6458f,3.3691f,-2.5417f,3.3203f,-2.5417f,3.3203f,-2.4375f,3.2715f,-2.4375f,3.2715f,-2.3333f,
	3.2227f,-2.3333f,3.2227f,-2.2292f,3.1738f,-2.2292f,3.1738f,-2.1250f,3.1250f,-2.1250f,3.1250f,-2.0208f,3.0762f,-2.0208f,3.0762f,-1.9167f,3.0273f,-1.9167f,3.0273f,-1.8125f,2.9785f,-1.8125f,2.9785f,-1.7083f,2.9297f,-1.7083f,2.9297f,-1.6042f,2.8809f,-1.6042f,
	2.8809f,-1.5000f,2.8320f,-1.5000f,2.8320f,-1.3958f,2.7832f,-1.3958f,2.7832f,-1.2917f,2.7344f,-1.2917f,2.7344f,-1.1875f,2.6855f,-1.1875f,2.6855f,-1.0833f,2.6367f,-1.0833f,2.6367f,-0.9792f,2.5879f,-0.9792f,2.5879f,-0.8750f,2.5391f,-0.8750f,2.5391f,-0.7708f,
	2.4902f,-0.7708f,2.4902f,-0.6667f,2.4414f,-0.6667f,2.4414f,-0.5625f,2.3926f,-0.5625f,2.3926f,-0.4583f,2.3437f,-0.4583f,2.3437f,-0.3542f,2.2949f,-0.3542f,2.2949f,-0.2500f,2.2461f,-0.2500f,2.2461f,-0.1458f,2.1973f,-0.1458f,2.1973f,-0.0417f,2.1484f,-0.0417f,
	2.1484f,0.0625f,2.0996f,0.0625f,2.0996f,0.1667f,2.0508f,0.1667f,2.0508f,0.2708f,2.0020f,0.2708f,2.0020f,0.3750f,1.9531f,0.3750f,1.9531f,0.4792f,1.9043f,0.4792f,1.9043f,0.5833f,1.8555f,0.5833f,1.8555f,0.6875f,1.8066f,0.6875f,1.8066f,0.7917f,
	1.7578f,0.7917f,1.7578f,0.8958f,1.7090f,0.8958f,1.7090f,1.0000f,1.6602f,1.0000f,4.0039f,-4.0000f,3.9551f,-4.0000f,3.9551f,-3.8958f,3.9062f,-3.8958f,3.9062f,-3.7917f,3.8574f,-3.7917f,3.8574f,-3.6875f,3.8086f,-3.6875f,3.8086f,-3.5833f,3.7598f,-3.5833f,
	3.7598f,-3.4792f,3.7109f,-3.4792f,3.7109f,-3.3750f,3.6621f,-3.3750f,3.6621f,-3.2708f,3.6133f,-3.2708f,3.6133f,-3.1667f,3.5645f,-3.1667f,3.5645f,-3.0625f,3.5156f,-3.0625f,3.5156f,-2.9583f,3.4668f,-2.9583f,3.4668f,-2.8542f,3.4180f,-2.8542f,3.4180f,-2.7500f,
	3.3691f,-2.7500f,3.3691f,-2.6458f,3.3203f,-2.6458f,3.3203f,-2.5417f,3.2715f,-2.5417f,3.2715f,-2.4375f,3.2227f,-2.4375f,3.2227f,-2.3333f,3.1738f,-2.3333f,3.1738f,-2.2292f,3.1250f,-2.2292f,3.1250f,-2.1250f,3.0762f,-2.1250f,3.0762f,-2.0208f,3.0273f,-2.0208f,
	3.0273f,-1.9167f,2.9785f,-1.9167f,2.9785f,-1.8125f,2.9297f,-1.8125f,2.9297f,-1.7083f,2.8809f,-1.7083f,2.8809f,-1.6042f,2.8320f,-1.6042f,2.8320f,-1.5000f,2.7832f,-1.5000f,2.7832f,-1.3958f,2.7344f,-1.3958f,2.7344f,-1.2917f,2.6855f,-1.2917f,2.6855f,-1.1875f,
	2.6367f,-1.1875f,2.6367f,-1.0833f,2.5879f,-1.0833f,2.5879f,-0.9792f,2.5391f,-0.9792f,2.5391f,-0.8750f,2.4902f,-0.8750f,2.4902f,-0.7708f,2.4414f,-0.7708f,2.4414f,-0.6667f,2.3926f,-0.6667f,2.3926f,-0.5625f,2.3437f,-0.5625f,2.3437f,-0.4583f,2.2949f,-0.4583f,
	2.2949f,-0.3542f,2.2461f,-0.3542f,2.2461f,-0.2500f,2.1973f,-0.2500f,2.1973f,-0.1458f,2.1484f,-0.1458f,2.1484f,-0.0417f,2.0996f,-0.0417f,2.0996f,0.0625f,2.0508f,0.0625f,2.0508f,0.1667f,2.0020f,0.1667f,2.0020f,0.2708f,1.9531f,0.2708f,1.9531f,0.3750f,
	1.9043f,0.3750f,1.9043f,0.4792f,1.8555f,0.4792f,1.8555f,0.5833f,1.8066f,0.5833f,1.8066f,0.6875f,1.7578f,0.6875f,1.7578f,0.7917f,1.7090f,0.7917f,1.7090f,0.8958f,1.6602f,0.8958f,1.6602f,1.0000f,1.6113f,1.0000f,3.9551f,-4.0000f,3.9062f,-4.0000f,
	3.9062f,-3.8958f,3.8574f,-3.8958f,3.8574f,-3.7917f,3.8086f,-3.7917f,3.8086f,-3.6875f,3.7598f,-3.6875f,3.7598f,-3.5833f,3.7109f,-3.5833f,3.7109f,-3.4792f,3.6621f,-3.4792f,3.6621f,-3.3750f,3.6133f,-3.3750f,3.6133f,-3.2708f,3.5645f,-3.2708f,3.5645f,-3.1667f,
	3.5156f,-3.1667f,3.5156f,-3.0625f,3.4668f,-3.0625f,3.4668f,-2.9583f,3.4180f,-2.9583f,3.4180f,-2.8542f,3.3691f,-2.8542f,3.3691f,-2.7500f,3.3203f,-2.7500f,3.3203f,-2.6458f,3.2715f,-2.6458f,3.2715f,-2.5417f,3.2227f,-2.5417f,3.2227f,-2.4375f,3.1738f,-2.4375f,
	3.1738f,-2.3333f,3.1250f,-2.3333f,3.1250f,-2.2292f,3.0762f,-2.2292f,3.0762f,-2.1250f,3.0273f,-2.1250f,3.0273f,-2.0208f,2.9785f,-2.0208f,2.9785f,-1.9167f,2.9297f,-1.9167f,2.9297f,-1.8125f,2.8809f,-1.8125f,2.8809f,-1.7083f,2.8320f,-1.7083f,2.8320f,-1.6042f,
	2.7832f,-1.6042f,2.7832f,-1.5000f,2.7344f,-1.5000f,2.7344f,-1.3958f,2.6855f,-1.3958f,2.6855f,-1.2917f,2.6367f,-1.2917f,2.6367f,-1.1875f,2.5879f,-1.1875f,2.5879f,-1.0833f,2.5391f,-1.0833f,2.5391f,-0.9792f,2.4902f,-0.9792f,2.4902f,-0.8750f,2.4414f,-0.8750f,
	2.4414f,-0.7708f,2.3926f,-0.7708f,2.3926f,-0.6667f,2.3437f,-0.6667f,2.3437f,-0.5625f,2.2949f,-0.5625f,2.2949f,-0.4583f,2.2461f,-0.4583f,2.2461f,-0.3542f,2.1973f,-0.3542f,2.1973f,-0.2500f,2.1484f,-0.2500f,2.1484f,-0.1458f,2.0996f,-0.1458f,2.0996f,-0.0417f,
	2.0508f,-0.0417f,2.0508f,0.0625f,2.0020f,0.0625f,2.0020f,0.1667f,1.9531f,0.1667f,1.9531f,0.2708f,1.9043f,0.2708f,1.9043f,0.3750f,1.8555f,0.3750f,1.8555f,0.4792f,1.8066f,0.4792f,1.8066f,0.5833f,1.7578f,0.5833f,1.7578f,0.6875f,1.7090f,0.6875f,
	1.7090f,0.7917f,1.6602f,0.7917f,1.6602f,0.8958f,1.6113f,0.8958f,1.6113f,1.0000f,1.5625f,1.0000f,3.9062f,-4.0000f,3.8574f,-4.0000f,3.8574f,-3.8958f,3.8086f,-3.8958f,3.8086f,-3.7917f,3.7598f,-3.7917f,3.7598f,-3.6875f,3.7109f,-3.6875f,3.7109f,-3.5833f,
	3.6621f,-3.5833f,3.6621f,-3.4792f,3.6133f,-3.4792f,3.6133f,-3.3750f,3.5645f,-3.3750f,3.5645f,-3.2708f,3.5156f,-3.2708f,3.5156f,-3.1667f,3.4668f,-3.1667f,3.4668f,-3.0625f,3.4180f,-3.0625f,3.4180f,-2.9583f,3.3691f,-2.9583f,3.3691f,-2.8542f,3.3203f,-2.8542f,
	3.3203f,-2.7500f,3.2715f,-2.7500f,3.2715f,-2.6458f,3.2227f,-2.6458f,3.2227f,-2.5417f,3.1738f,-2.5417f,3.1738f,-2.4375f,3.1250f,-2.4375f,3.1250f,-2.3333f,3.0762f,-2.3333f,3.0762f,-2.2292f,3.0273f,-2.2292f,3.0273f,-2.1250f,2.9785f,-2.1250f,2.9785f,-2.0208f,
	2.9297f,-2.0208f,2.9297f,-1.9167f,2.8809f,-1.9167f,2.8809f,-1.8125f,2.8320f,-1.8125f,2.8320f,-1.7083f,2.7832f,-1.7083f,2.7832f,-1.6042f,2.7344f,-1.6042f,2.7344f,-1.5000f,2.6855f,-1.5000f,2.6855f,-1.3958f,2.6367f,-1.3958f,2.6367f,-1.2917f,2.5879f,-1.2917f,
	2.5879f,-1.1875f,2.5391f,-1.1875f,2.5391f,-1.0833f,2.4902f,-1.0833f,2.4902f,-0.9792f,2.4414f,-0.9792f,2.4414f,-0.8750f,2.3926f,-0.8750f,2.3926f,-0.7708f,2.3437f,-0.7708f,2.3437f,-0.6667f,2.2949f,-0.6667f,2.2949f,-0.5625f,2.2461f,-0.5625f,2.2461f,-0.4583f,
	2.1973f,-0.4583f,2.1973f,-0.3542f,2.1484f,-0.3542f,2.1484f,-0.2500f,2.0996f,-0.2500f,2.0996f,-0.1458f,2.0508f,-0.1458f,2.0508f,-0.0417f,2.0020f,-0.0417f,2.0020f,0.0625f,1.9531f,0.0625f,1.9531f,0.1667f,1.9043f,0.1667f,1.9043f,0.2708f,1.8555f,0.2708f,
	1.8555f,0.3750f,1.8066f,0.3750f,1.8066f,0.4792f,1.7578f,0.4792f,1.7578f,0.5833f,1.7090f,0.5833f,1.7090f,0.6875f,1.6602f,0.6875f,1.6602f,0.7917f,1.6113f,0.7917f,1.6113f,0.8958f,1.5625f,0.8958f,1.5625f,1.0000f,1.5137f,1.0000f,3.8574f,-4.0000f,
	3.8086f,-4.0000f,3.8086f,-3.8958f,3.7598f,-3.8958f,3.7598f,-3.7917f,3.7109f,-3.7917f,3.7109f,-3.6875f,3.6621f,-3.6875f,3.6621f,-3.5833f,3.6133f,-3.5833f,3.6133f,-3.4792f,3.5645f,-3.4792f,3.5645f,-3.3750f,3.5156f,-3.3750f,3.5156f,-3.2708f,3.4668f,-3.2708f,
	3.4668f,-3.1667f,3.4180f,-3.1667f,3.4180f,-3.0625f,3.3691f,-3.0625f,3.3691f,-2.9583f,3.3203f,-2.9583f,3.3203f,-2.8542f,3.2715f,-2.8542f,3.2715f,-2.7500f,3.2227f,-2.7500f,3.2227f,-2.6458f,3.1738f,-2.6458f,3.1738f,-2.5417f,3.1250f,-2.5417f,3.1250f,-2.4375f,
	3.0762f,-2.4375f,3.0762f,-2.3333f,3.0273f,-2.3333f,3.0273f,-2.2292f,2.9785f,-2.2292f,2.9785f,-2.1250f,2.9297f,-2.1250f,2.9297f,-2.0208f,2.8809f,-2.0208f,2.8809f,-1.9167f,2.8320f,-1.9167f,2.8320f,-1.8125f,2.7832f,-1.8125f,2.7832f,-1.7083f,2.7344f,-1.7083f,
	2.7344f,-1.6042f,2.6855f,-1.6042f,2.6855f,-1.5000f,2.6367f,-1.5000f,2.6367f,-1.3958f,2.5879f,-1.3958f,2.5879f,-1.2917f,2.5391f,-1.2917f,2.5391f,-1.1875f,2.4902f,-1.1875f,2.4902f,-1.0833f,2.4414f,-1.0833f,2.4414f,-0.9792f,2.3926f,-0.9792f,2.3926f,-0.8750f,
	2.3437f,-0.8750f,2.3437f,-0.7708f,2.2949f,-0.7708f,2.2949f,-0.6667f,2.2461f,-0.6667f,2.2461f,-0.5625f,2.1973f,-0.5625f,2.1973f,-0.4583f,2.1484f,-0.4583f,2.1484f,-0.3542f,2.0996f,-0.3542f,2.0996f,-0.2500f,2.0508f,-0.2500f,2.0508f,-0.1458f,2.0020f,-0.1458f,
	2.0020f,-0.0417f,1.9531f,-0.0417f,1.9531f,0.0625f,1.9043f,0.0625f,1.9043f,0.1667f,1.8555f,0.1667f,1.8555f,0.2708f,1.8066f,0.2708f,1.8066f,0.3750f,1.7578f,0.3750f,1.7578f,0.4792f,1.7090f,0.4792f,1.7090f,0.5833f,1.6602f,0.5833f,1.6602f,0.6875f,
	1.6113f,0.6875f,1.6113f,0.7917f,1.5625f,0.7917f,1.5625f,0.8958f,1.5137f,0.8958f,1.5137f,1.0000f,1.4648f,1.0000f,3.8086f,-4.0000f,3.7598f,-4.0000f,3.7598f,-3.8958f,3.7109f,-3.8958f,3.7109f,-3.7917f,3.6621f,-3.7917f,3.6621f,-3.6875f,3.6133f,-3.6875f,
	3.6133f,-3.5833f,3.5645f,-3.5833f,3.5645f,-3.4792f,3.5156f,-3.4792f,3.5156f,-3.3750f,3.4668f,-3.3750f,3.4668f,-3.2708f,3.4180f,-3.2708f,3.4180f,-3.1667f,3.3691f,-3.1667f,3.3691f,-3.0625f,3.3203f,-3.0625f,3.3203f,-2.9583f,3.2715f,-2.9583f,3.2715f,-2.8542f,
	3.2227f,-2.8542f,3.2227f,-2.7500f,3.1738f,-2.7500f,3.1738f,-2.6458f,3.1250f,-2.6458f,3.1250f,-2.5417f,3.0762f,-2.5417f,3.0762f,-2.4375f,3.0273f,-2.4375f,3.0273f,-2.3333f,2.9785f,-2.3333f,2.9785f,-2.2292f,2.9297f,-2.2292f,2.9297f,-2.1250f,2.8809f,-2.1250f,
	2.8809f,-2.0208f,2.8320f,-2.0208f,2.8320f,-1.9167f,2.7832f,-1.9167f,2.7832f,-1.8125f,2.7344f,-1.8125f,2.7344f,-1.7083f,2.6855f,-1.7083f,2.6855f,-1.6042f,2.6367f,-1.6042f,2.6367f,-1.5000f,2.5879f,-1.5000f,2.5879f,-1.3958f,2.5391f,-1.3958f,2.5391f,-1.2917f,
	2.4902f,-1.2917f,2.4902f,-1.1875f,2.4414f,-1.1875f,2.4414f,-1.0833f,2.3926f,-1.0833f,2.3926f,-0.9792f,2.3437f,-0.9792f,2.3437f,-0.8750f,2.2949f,-0.8750f,2.2949f,-0.7708f,2.2461f,-0.7708f,2.2461f,-0.6667f,2.1973f,-0.6667f,2.1973f,-0.5625f,2.1484f,-0.5625f,
	2.1484f,-0.4583f,2.0996f,-0.4583f,2.0996f,-0.3542f,2.0508f,-0.3542f,2.0508f,-0.2500f,2.0020f,-0.2500f,2.0020f,-0.1458f,1.9531f,-0.1458f,1.9531f,-0.0417f,1.9043f,-0.0417f,1.9043f,0.0625f,1.8555f,0.0625f,1.8555f,0.1667f,1.8066f,0.1667f,1.8066f,0.2708f,
	1.7578f,0.2708f,1.7578f,0.3750f,1.7090f,0.3750f,1.7090f,0.4792f,1.6602f,0.4792f,1.6602f,0.5833f,1.6113f,0.5833f,1.6113f,0.6875f,1.5625f,0.6875f,1.5625f,0.7917f,1.5137f,0.7917f,1.5137f,0.8958f,1.4648f,0.8958f,1.4648f,1.0000f,1.4160f,1.0000f,
	3.7598f,-4.0000f,3.7109f,-4.0000f,3.7109f,-3.8958f,3.6621f,-3.8958f,3.6621f,-3.7917f,3.6133f,-3.7917f,3.6133f,-3.6875f,3.5645f,-3.6875f,3.5645f,-3.5833f,3.5156f,-3.5833f,3.5156f,-3.4792f,3.4668f,-3.4792f,3.4668f,-3.3750f,3.4180f,-3.3750f,3.4180f,-3.2708f,
	3.3691f,-3.2708f,3.3691f,-3.1667f,3.3203f,-3.1667f,3.3203f,-3.0625f,3.2715f,-3.0625f,3.2715f,-2.9583f,3.2227f,-2.9583f,3.2227f,-2.8542f,3.1738f,-2.8542f,3.1738f,-2.7500f,3.1250f,-2.7500f,3.1250f,-2.6458f,3.0762f,-2.6458f,3.0762f,-2.5417f,3.0273f,-2.5417f,
	3.0273f,-2.4375f,2.9785f,-2.4375f,2.9785f,-2.3333f,2.9297f,-2.3333f,2.9297f,-2.2292f,2.8809f,-2.2292f,2.8809f,-2.1250f,2.8320f,-2.1250f,2.8320f,-2.0208f,2.7832f,-2.0208f,2.7832f,-1.9167f,2.7344f,-1.9167f,2.7344f,-1.8125f,2.6855f,-1.8125f,2.6855f,-1.7083f,
	2.6367f,-1.7083f,2.6367f,-1.6042f,2.5879f,-1.6042f,2.5879f,-1.5000f,2.5391f,-1.5000f,2.5391f,-1.3958f,2.4902f,-1.3958f,2.4902f,-1.2917f,2.4414f,-1.2917f,2.4414f,-1.1875f,2.3926f,-1.1875f,2.3926f,-1.0833f,2.3437f,-1.0833f,2.3437f,-0.9792f,2.2949f,-0.9792f,
	2.2949f,-0.8750f,2.2461f,-0.8750f,2.2461f,-0.7708f,2.1973f,-0.7708f,2.1973f,-0.6667f,2.1484f,-0.6667f,2.1484f,-0.5625f,2.0996f,-0.5625f,2.0996f,-0.4583f,2.0508f,-0.4583f,2.0508f,-0.3542f,2.0020f,-0.3542f,2.0020f,-0.2500f,1.9531f,-0.2500f,1.9531f,-0.1458f,
	1.9043f,-0.1458f,1.9043f,-0.0417f,1.8555f,-0.0417f,1.8555f,0.0625f,1.8066f,0.0625f,1.8066f,0.1667f,1.7578f,0.1667f,1.7578f,0.2708f,1.7090f,0.2708f,1.7090f,0.3750f,1.6602f,0.3750f,1.6602f,0.4792f,1.6113f,0.4792f,1.6113f,0.5833f,1.5625f,0.5833f,
	1.5625f,0.6875f,1.5137f,0.6875f,1.5137f,0.7917f,1.4648f,0.7917f,1.4648f,0.8958f,1.4160f,0.8958f,1.4160f,1.0000f,1.3672f,1.0000f,3.7109f,-4.0000f,3.6621f,-4.0000f,3.6621f,-3.8958f,3.6133f,-3.8958f,3.6133f,-3.7917f,3.5645f,-3.7917f,3.5645f,-3.6875f,
	3.5156f,-3.6875f,3.5156f,-3.5833f,3.4668f,-3.5833f,3.4668f,-3.4792f,3.4180f,-3.4792f,3.4180f,-3.3750f,3.3691f,-3.3750f,3.3691f,-3.2708f,3.3203f,-3.2708f,3.3203f,-3.1667f,3.2715f,-3.1667f,3.2715f,-3.0625f,3.2227f,-3.0625f,3.2227f,-2.9583f,3.1738f,-2.9583f,
	3.1738f,-2.8542f,3.1250f,-2.8542f,3.1250f,-2.7500f,3.0762f,-2.7500f,3.0762f,-2.6458f,3.0273f,-2.6458f,3.0273f,-2.5417f,2.9785f,-2.5417f,2.9785f,-2.4375f,2.9297f,-2.4375f,2.9297f,-2.3333f,2.8809f,-2.3333f,2.8809f,-2.2292f,2.8320f,-2.2292f,2.8320f,-2.1250f,
	2.7832f,-2.1250f,2.7832f,-2.0208f,2.7344f,-2.0208f,2.7344f,-1.9167f,2.6855f,-1.9167f,2.6855f,-1.8125f,2.6367f,-1.8125f,2.6367f,-1.7083f,2.5879f,-1.7083f,2.5879f,-1.6042f,2.5391f,-1.6042f,2.5391f,-1.5000f,2.4902f,-1.5000f,2.4902f,-1.3958f,2.4414f,-1.3958f,
	2.4414f,-1.2917f,2.3926f,-1.2917f,2.3926f,-1.1875f,2.3437f,-1.1875f,2.3437f,-1.0833f,2.2949f,-1.0833f,2.2949f,-0.9792f,2.2461f,-0.9792f,2.2461f,-0.8750f,2.1973f,-0.8750f,2.1973f,-0.7708f,2.1484f,-0.7708f,2.1484f,-0.6667f,2.0996f,-0.6667f,2.0996f,-0.5625f,
	2.0508f,-0.5625f,2.0508f,-0.4583f,2.0020f,-0.4583f,2.0020f,-0.3542f,1.9531f,-0.3542f,1.9531f,-0.2500f,1.9043f,-0.2500f,1.9043f,-0.1458f,1.8555f,-0.1458f,1.8555f,-0.0417f,1.8066f,-0.0417f,1.8066f,0.0625f,1.7578f,0.0625f,1.7578f,0.1667f,1.7090f,0.1667f,
	1.7090f,0.2708f,1.6602f,0.2708f,1.6602f,0.3750f,1.6113f,0.3750f,1.6113f,0.4792f,1.5625f,0.4792f,1.5625f,0.5833f,1.5137f,0.5833f,1.5137f,0.6875f,1.4648f,0.6875f,1.4648f,0.7917f,1.4160f,0.7917f,1.4160f,0.8958f,1.3672f,0.8958f,1.3672f,1.0000f,
	1.3184f,1.0000f,3.6621f,-4.0000f,3.6133f,-4.0000f,3.6133f,-3.8958f,3.5645f,-3.8958f,3.5645f,-3.7917f,3.5156f,-3.7917f,3.5156f,-3.6875f,3.4668f,-3.6875f,3.4668f,-3.5833f,3.4180f,-3.5833f,3.4180f,-3.4792f,3.3691f,-3.4792f,3.3691f,-3.3750f,3.3203f,-3.3750f,
	3.3203f,-3.2708f,3.2715f,-3.2708f,3.2715f,-3.1667f,3.2227f,-3.1667f,3.2227f,-3.0625f,3.1738f,-3.0625f,3.1738f,-2.9583f,3.1250f,-2.9583f,3.1250f,-2.8542f,3.0762f,-2.8542f,3.0762f,-2.7500f,3.0273f,-2.7500f,3.0273f,-2.6458f,2.9785f,-2.6458f,2.9785f,-2.5417f,
	2.9297f,-2.5417f,2.9297f,-2.4375f,2.8809f,-2.4375f,2.8809f,-2.3333f,2.8320f,-2.3333f,2.8320f,-2.2292f,2.7832f,-2.2292f,2.7832f,-2.1250f,2.7344f,-2.1250f,2.7344f,-2.0208f,2.6855f,-2.0208f,2.6855f,-1.9167f,2.6367f,-1.9167f,2.6367f,-1.8125f,2.5879f,-1.8125f,
	2.5879f,-1.7083f,2.5391f,-1.7083f,2.5391f,-1.6042f,2.4902f,-1.6042f,2.4902f,-1.5000f,2.4414f,-1.5000f,2.4414f,-1.3958f,2.3926f,-1.3958f,2.3926f,-1.2917f,2.3437f,-1.2917f,2.3437f,-1.1875f,2.2949f,-1.1875f,2.2949f,-1.0833f,2.2461f,-1.0833f,2.2461f,-0.9792f,
	2.1973f,-0.9792f,2.1973f,-0.8750f,2.1484f,-0.8750f,2.1484f,-0.7708f,2.0996f,-0.7708f,2.0996f,-0.6667f,2.0508f,-0.6667f,2.0508f,-0.5625f,2.0020f,-0.5625f,2.0020f,-0.4583f,1.9531f,-0.4583f,1.9531f,-0.3542f,1.9043f,-0.3542f,1.9043f,-0.2500f,1.8555f,-0.2500f,
	1.8555f,-0.1458f,1.8066f,-0.1458f,1.8066f,-0.0417f,1.7578f,-0.0417f,1.7578f,0.0625f,1.7090f,0.0625f,1.7090f,0.1667f,1.6602f,0.1667f,1.6602f,0.2708f,1.6113f,0.2708f,1.6113f,0.3750f,1.5625f,0.3750f,1.5625f,0.4792f,1.5137f,0.4792f,1.5137f,0.5833f,
	1.4648f,0.5833f,1.4648f,0.6875f,1.4160f,0.6875f,1.4160f,0.7917f,1.3672f,0.7917f,1.3672f,0.8958f,1.3184f,0.8958f,1.3184f,1.0000f,1.2695f,1.0000f,3.6133f,-4.0000f,3.5645f,-4.0000f,3.5645f,-3.8958f,3.5156f,-3.8958f,3.5156f,-3.7917f,3.4668f,-3.7917f,
	3.4668f,-3.6875f,3.4180f,-3.6875f,3.4180f,-3.5833f,3.3691f,-3.5833f,3.3691f,-3.4792f,3.3203f,-3.4792f,3.3203f,-3.3750f,3.2715f,-3.3750f,3.2715f,-3.2708f,3.2227f,-3.2708f,3.2227f,-3.1667f,3.1738f,-3.1667f,3.1738f,-3.0625f,3.1250f,-3.0625f,3.1250f,-2.9583f,
	3.0762f,-2.9583f,3.0762f,-2.8542f,3.0273f,-2.8542f,3.0273f,-2.7500f,2.9785f,-2.7500f,2.9785f,-2.6458f,2.9297f,-2.6458f,2.9297f,-2.5417f,2.8809f,-2.5417f,2.8809f,-2.4375f,2.8320f,-2.4375f,2.8320f,-2.3333f,2.7832f,-2.3333f,2.7832f,-2.2292f,2.7344f,-2.2292f,
	2.7344f,-2.1250f,2.6855f,-2.1250f,2.6855f,-2.0208f,2.6367f,-2.0208f,2.6367f,-1.9167f,2.5879f,-1.9167f,2.5879f,-1.8125f,2.5391f,-1.8125f,2.5391f,-1.7083f,2.4902f,-1.7083f,2.4902f,-1.6042f,2.4414f,-1.6042f,2.4414f,-1.5000f,2.3926f,-1.5000f,2.3926f,-1.3958f,
	2.3437f,-1.3958f,2.3437f,-1.2917f,2.2949f,-1.2917f,2.2949f,-1.1875f,2.2461f,-1.1875f,2.2461f,-1.0833f,2.1973f,-1.0833f,2.1973f,-0.9792f,2.1484f,-0.9792f,2.1484f,-0.8750f,2.0996f,-0.8750f,2.0996f,-0.7708f,2.0508f,-0.7708f,2.0508f,-0.6667f,2.0020f,-0.6667f,
	2.0020f,-0.5625f,1.9531f,-0.5625f,1.9531f,-0.4583f,1.9043f,-0.4583f,1.9043f,-0.3542f,1.8555f,-0.3542f,1.8555f,-0.2500f,1.8066f,-0.2500f,1.8066f,-0.1458f,1.7578f,-0.1458f,1.7578f,-0.0417f,1.7090f,-0.0417f,1.7090f,0.0625f,1.6602f,0.0625f,1.6602f,0.1667f,
	1.6113f,0.1667f,1.6113f,0.2708f,1.5625f,0.2708f,1.5625f,0.3750f,1.5137f,0.3750f,1.5137f,0.4792f,1.4648f,0.4792f,1.4648f,0.5833f,1.4160f,0.5833f,1.4160f,0.6875f,1.3672f,0.6875f,1.3672f,0.7917f,1.3184f,0.7917f,1.3184f,0.8958f,1.2695f,0.8958f,
	1.2695f,1.0000f,1.2207f,1.0000f,3.5645f,-4.0000f,3.5156f,-4.0000f,3.5156f,-3.8958f,3.4668f,-3.8958f,3.4668f,-3.7917f,3.4180f,-3.7917f,3.4180f,-3.6875f,3.3691f,-3.6875f,3.3691f,-3.5833f,3.3203f,-3.5833f,3.3203f,-3.4792f,3.2715f,-3.4792f,3.2715f,-3.3750f,
	3.2227f,-3.3750f,3.2227f,-3.2708f,3.1738f,-3.2708f,3.1738f,-3.1667f,3.1250f,-3.1667f,3.1250f,-3.0625f,3.0762f,-3.0625f,3.0762f,-2.9583f,3.0273f,-2.9583f,3.0273f,-2.8542f,2.9785f,-2.8542f,2.9785f,-2.7500f,2.9297f,-2.7500f,2.9297f,-2.6458f,2.8809f,-2.6458f,
	2.8809f,-2.5417f,2.8320f,-2.5417f,2.8320f,-2.4375f,2.7832f,-2.4375f,2.7832f,-2.3333f,2.7344f,-2.3333f,2.7344f,-2.2292f,2.6855f,-2.2292f,2.6855f,-2.1250f,2.6367f,-2.1250f,2.6367f,-2.0208f,2.5879f,-2.0208f,2.5879f,-1.9167f,2.5391f,-1.9167f,2.5391f,-1.8125f,
	2.4902f,-1.8125f,2.4902f,-1.7083f,2.4414f,-1.7083f,2.4414f,-1.6042f,2.3926f,-1.6042f,2.3926f,-1.5000f,2.3437f,-1.5000f,2.3437f,-1.3958f,2.2949f,-1.3958f,2.2949f,-1.2917f,2.2461f,-1.2917f,2.2461f,-1.1875f,2.1973f,-1.1875f,2.1973f,-1.0833f,2.1484f,-1.0833f,
	2.1484f,-0.9792f,2.0996f,-0.9792f,2.0996f,-0.8750f,2.0508f,-0.8750f,2.0508f,-0.7708f,2.0020f,-0.7708f,2.0020f,-0.6667f,1.9531f,-0.6667f,1.9531f,-0.5625f,1.9043f,-0.5625f,1.9043f,-0.4583f,1.8555f,-0.4583f,1.8555f,-0.3542f,1.8066f,-0.3542f,1.8066f,-0.2500f,
	1.7578f,-0.2500f,1.7578f,-0.1458f,1.7090f,-0.1458f,1.7090f,-0.0417f,1.6602f,-0.0417f,1.6602f,0.0625f,1.6113f,0.0625f,1.6113f,0.1667f,1.5625f,0.1667f,1.5625f,0.2708f,1.5137f,0.2708f,1.5137f,0.3750f,1.4648f,0.3750f,1.4648f,0.4792f,1.4160f,0.4792f,
	1.4160f,0.5833f,1.3672f,0.5833f,1.3672f,0.6875f,1.3184f,0.6875f,1.3184f,0.7917f,1.2695f,0.7917f,1.2695f,0.8958f,1.2207f,0.8958f,1.2207f,1.0000f,1.1719f,1.0000f,3.5156f,-4.0000f,3.4668f,-4.0000f,3.4668f,-3.8958f,3.4180f,-3.8958f,3.4180f,-3.7917f,
	3.3691f,-3.7917f,3.3691f,-3.6875f,3.3203f,-3.6875f,3.3203f,-3.5833f,3.2715f,-3.5833f,3.2715f,-3.4792f,3.2227f,-3.4792f,3.2227f,-3.3750f,3.1738f,-3.3750f,3.1738f,-3.2708f,3.1250f,-3.2708f,3.1250f,-3.1667f,3.0762f,-3.1667f,3.0762f,-3.0625f,3.0273f,-3.0625f,
	3.0273f,-2.9583f,2.9785f,-2.9583f,2.9785f,-2.8542f,2.9297f,-2.8542f,2.9297f,-2.7500f,2.8809f,-2.7500f,2.8809f,-2.6458f,2.8320f,-2.6458f,2.8320f,-2.5417f,2.7832f,-2.5417f,2.7832f,-2.4375f,2.7344f,-2.4375f,2.7344f,-2.3333f,2.6855f,-2.3333f,2.6855f,-2.2292f,
	2.6367f,-2.2292f,2.6367f,-2.1250f,2.5879f,-2.1250f,2.5879f,-2.0208f,2.5391f,-2.0208f,2.5391f,-1.9167f,2.4902f,-1.9167f,2.4902f,-1.8125f,2.4414f,-1.8125f,2.4414f,-1.7083f,2.3926f,-1.7083f,2.3926f,-1.6042f,2.3437f,-1.6042f,2.3437f,-1.5000f,2.2949f,-1.5000f,
	2.2949f,-1.3958f,2.2461f,-1.3958f,2.2461f,-1.2917f,2.1973f,-1.2917f,2.1973f,-1.1875f,2.1484f,-1.1875f,2.1484f,-1.0833f,2.0996f,-1.0833f,2.0996f,-0.9792f,2.0508f,-0.9792f,2.0508f,-0.8750f,2.0020f,-0.8750f,2.0020f,-0.7708f,1.9531f,-0.7708f,1.9531f,-0.6667f,
	1.9043f,-0.6667f,1.9043f,-0.5625f,1.8555f,-0.5625f,1.8555f,-0.4583f,1.8066f,-0.4583f,1.8066f,-0.3542f,1.7578f,-0.3542f,1.7578f,-0.2500f,1.7090f,-0.2500f,1.7090f,-0.1458f,1.6602f,-0.1458f,1.6602f,-0.0417f,1.6113f,-0.0417f,1.6113f,0.0625f,1.5625f,0.0625f,
	1.5625f,0.1667f,1.5137f,0.1667f,1.5137f,0.2708f,1.4648f,0.2708f,1.4648f,0.3750f,1.4160f,0.3750f,1.4160f,0.4792f,1.3672f,0.4792f,1.3672f,0.5833f,1.3184f,0.5833f,1.3184f,0.6875f,1.2695f,0.6875f,1.2695f,0.7917f,1.2207f,0.7917f,1.2207f,0.8958f,
	1.1719f,0.8958f,1.1719f,1.0000f,1.1230f,1.0000f,3.4668f,-4.0000f,3.4180f,-4.0000f,3.4180f,-3.8958f,3.3691f,-3.8958f,3.3691f,-3.7917f,3.3203f,-3.7917f,3.3203f,-3.6875f,3.2715f,-3.6875f,3.2715f,-3.5833f,3.2227f,-3.5833f,3.2227f,-3.4792f,3.1738f,-3.4792f,
	3.1738f,-3.3750f,3.1250f,-3.3750f,3.1250f,-3.2708f,3.0762f,-3.2708f,3.0762f,-3.1667f,3.0273f,-3.1667f,3.0273f,-3.0625f,2.9785f,-3.0625f,2.9785f,-2.9583f,2.9297f,-2.9583f,2.9297f,-2.8542f,2.8809f,-2.8542f,2.8809f,-2.7500f,2.8320f,-2.7500f,2.8320f,-2.6458f,
	2.7832f,-2.6458f,2.7832f,-2.5417f,2.7344f,-2.5417f,2.7344f,-2.4375f,2.6855f,-2.4375f,2.6855f,-2.3333f,2.6367f,-2.3333f,2.6367f,-2.2292f,2.5879f,-2.2292f,2.5879f,-2.1250f,2.5391f,-2.1250f,2.5391f,-2.0208f,2.4902f,-2.0208f,2.4902f,-1.9167f,2.4414f,-1.9167f,
	2.4414f,-1.8125f,2.3926f,-1.8125f,2.3926f,-1.7083f,2.3437f,-1.7083f,2.3437f,-1.6042f,2.2949f,-1.6042f,2.2949f,-1.5000f,2.2461f,-1.5000f,2.2461f,-1.3958f,2.1973f,-1.3958f,2.1973f,-1.2917f,2.1484f,-1.2917f,2.1484f,-1.1875f,2.0996f,-1.1875f,2.0996f,-1.0833f,
	2.0508f,-1.0833f,2.0508f,-0.9792f,2.0020f,-0.9792f,2.0020f,-0.8750f,1.9531f,-0.8750f,1.9531f,-0.7708f,1.9043f,-0.7708f,1.9043f,-0.6667f,1.8555f,-0.6667f,1.8555f,-0.5625f,1.8066f,-0.5625f,1.8066f,-0.4583f,1.7578f,-0.4583f,1.7578f,-0.3542f,1.7090f,-0.3542f,
	1.7090f,-0.2500f,1.6602f,-0.2500f,1.6602f,-0.1458f,1.6113f,-0.1458f,1.6113f,-0.0417f,1.5625f,-0.0417f,1.5625f,0.0625f,1.5137f,0.0625f,1.5137f,0.1667f,1.4648f,0.1667f,1.4648f,0.2708f,1.4160f,0.2708f,1.4160f,0.3750f,1.3672f,0.3750f,1.3672f,0.4792f,
	1.3184f,0.4792f,1.3184f,0.5833f,1.2695f,0.5833f,1.2695f,0.6875f,1.2207f,0.6875f,1.2207f,0.7917f,1.1719f,0.7917f,1.1719f,0.8958f,1.1230f,0.8958f,1.1230f,1.0000f,1.0742f,1.0000f,3.4180f,-4.0000f,3.3691f,-4.0000f,3.3691f,-3.8958f,3.3203f,-3.8958f,
	3.3203f,-3.7917f,3.2715f,-3.7917f,3.2715f,-3.6875f,3.2227f,-3.6875f,3.2227f,-3.5833f,3.1738f,-3.5833f,3.1738f,-3.4792f,3.1250f,-3.4792f,3.1250f,-3.3750f,3.0762f,-3.3750f,3.0762f,-3.2708f,3.0273f,-3.2708f,3.0273f,-3.1667f,2.9785f,-3.1667f,2.9785f,-3.0625f,
	2.9297f,-3.0625f,2.9297f,-2.9583f,2.8809f,-2.9583f,2.8809f,-2.8542f,2.8320f,-2.8542f,2.8320f,-2.7500f,2.7832f,-2.7500f,2.7832f,-2.6458f,2.7344f,-2.6458f,2.7344f,-2.5417f,2.6855f,-2.5417f,2.6855f,-2.4375f,2.6367f,-2.4375f,2.6367f,-2.3333f,2.5879f,-2.3333f,
	2.5879f,-2.2292f,2.5391f,-2.2292f,2.5391f,-2.1250f,2.4902f,-2.1250f,2.4902f,-2.0208f,2.4414f,-2.0208f,2.4414f,-1.9167f,2.3926f,-1.9167f,2.3926f,-1.8125f,2.3437f,-1.8125f,2.3437f,-1.7083f,2.2949f,-1.7083f,2.2949f,-1.6042f,2.2461f,-1.6042f,2.2461f,-1.5000f,
	2.1973f,-1.5000f,2.1973f,-1.3958f,2.1484f,-1.3958f,2.1484f,-1.2917f,2.0996f,-1.2917f,2.0996f,-1.1875f,2.0508f,-1.1875f,2.0508f,-1.0833f,2.0020f,-1.0833f,2.0020f,-0.9792f,1.9531f,-0.9792f,1.9531f,-0.8750f,1.9043f,-0.8750f,1.9043f,-0.7708f,1.8555f,-0.7708f,
	1.8555f,-0.6667f,1.8066f,-0.6667f,1.8066f,-0.5625f,1.7578f,-0.5625f,1.7578f,-0.4583f,1.7090f,-0.4583f,1.7090f,-0.3542f,1.6602f,-0.3542f,1.6602f,-0.2500f,1.6113f,-0.2500f,1.6113f,-0.1458f,1.5625f,-0.1458f,1.5625f,-0.0417f,1.5137f,-0.0417f,1.5137f,0.0625f,
	1.4648f,0.0625f,1.4648f,0.1667f,1.4160f,0.1667f,1.4160f,0.2708f,1.3672f,0.2708f,1.3672f,0.3750f,1.3184f,0.3750f,1.3184f,0.4792f,1.2695f,0.4792f,1.2695f,0.5833f,1.2207f,0.5833f,1.2207f,0.6875f,1.1719f,0.6875f,1.1719f,0.7917f,1.1230f,0.7917f,
	1.1230f,0.8958f,1.0742f,0.8958f,1.0742f,1.0000f,1.0254f,1.0000f,3.3691f,-4.0000f,3.3203f,-4.0000f,3.3203f,-3.8958f,3.2715f,-3.8958f,3.2715f,-3.7917f,3.2227f,-3.7917f,3.2227f,-3.6875f,3.1738f,-3.6875f,3.1738f,-3.5833f,3.1250f,-3.5833f,3.1250f,-3.4792f,
	3.0762f,-3.4792f,3.0762f,-3.3750f,3.0273f,-3.3750f,3.0273f,-3.2708f,2.9785f,-3.2708f,2.9785f,-3.1667f,2.9297f,-3.1667f,2.9297f,-3.0625f,2.8809f,-3.0625f,2.8809f,-2.9583f,2.8320f,-2.9583f,2.8320f,-2.8542f,2.7832f,-2.8542f,2.7832f,-2.7500f,2.7344f,-2.7500f,
	2.7344f,-2.6458f,2.6855f,-2.6458f,2.6855f,-2.5417f,2.6367f,-2.5417f,2.6367f,-2.4375f,2.5879f,-2.4375f,2.5879f,-2.3333f,2.5391f,-2.3333f,2.5391f,-2.2292f,2.4902f,-2.2292f,2.4902f,-2.1250f,2.4414f,-2.1250f,2.4414f,-2.0208f,2.3926f,-2.0208f,2.3926f,-1.9167f,
	2.3437f,-1.9167f,2.3437f,-1.8125f,2.2949f,-1.8125f,2.2949f,-1.7083f,2.2461f,-1.7083f,2.2461f,-1.6042f,2.1973f,-1.6042f,2.1973f,-1.5000f,2.1484f,-1.5000f,2.1484f,-1.3958f,2.0996f,-1.3958f,2.0996f,-1.2917f,2.0508f,-1.2917f,2.0508f,-1.1875f,2.0020f,-1.1875f,
	2.0020f,-1.0833f,1.9531f,-1.0833f,1.9531f,-0.9792f,1.9043f,-0.9792f,1.9043f,-0.8750f,1.8555f,-0.8750f,1.8555f,-0.7708f,1.8066f,-0.7708f,1.8066f,-0.6667f,1.7578f,-0.6667f,1.7578f,-0.5625f,1.7090f,-0.5625f,1.7090f,-0.4583f,1.6602f,-0.4583f,1.6602f,-0.3542f,
	1.6113f,-0.3542f,1.6113f,-0.2500f,1.5625f,-0.2500f,1.5625f,-0.1458f,1.5137f,-0.1458f,1.5137f,-0.0417f,1.4648f,-0.0417f,1.4648f,0.0625f,1.4160f,0.0625f,1.4160f,0.1667f,1.3672f,0.1667f,1.3672f,0.2708f,1.3184f,0.2708f,1.3184f,0.3750f,1.2695f,0.3750f,
	1.2695f,0.4792f,1.2207f,0.4792f,1.2207f,0.5833f,1.1719f,0.5833f,1.1719f,0.6875f,1.1230f,0.6875f,1.1230f,0.7917f,1.0742f,0.7917f,1.0742f,0.8958f,1.0254f,0.8958f,1.0254f,1.0000f,0.9766f,1.0000f,3.3203f,-4.0000f,3.2715f,-4.0000f,3.2715f,-3.8958f,
	3.2227f,-3.8958f,3.2227f,-3.7917f,3.1738f,-3.7917f,3.1738f,-3.6875f,3.1250f,-3.6875f,3.1250f,-3.5833f,3.0762f,-3.5833f,3.0762f,-3.4792f,3.0273f,-3.4792f,3.0273f,-3.3750f,2.9785f,-3.3750f,2.9785f,-3.2708f,2.9297f,-3.2708f,2.9297f,-3.1667f,2.8809f,-3.1667f,
	2.8809f,-3.0625f,2.8320f,-3.0625f,2.8320f,-2.9583f,2.7832f,-2.9583f,2.7832f,-2.8542f,2.7344f,-2.8542f,2.7344f,-2.7500f,2.6855f,-2.7500f,2.6855f,-2.6458f,2.6367f,-2.6458f,2.6367f,-2.5417f,2.5879f,-2.5417f,2.5879f,-2.4375f,2.5391f,-2.4375f,2.5391f,-2.3333f,
	2.4902f,-2.3333f,2.4902f,-2.2292f,2.4414f,-2.2292f,2.4414f,-2.1250f,2.3926f,-2.1250f,2.3926f,-2.0208f,2.3437f,-2.0208f,2.3437f,-1.9167f,2.2949f,-1.9167f,2.2949f,-1.8125f,2.2461f,-1.8125f,2.2461f,-1.7083f,2.1973f,-1.7083f,2.1973f,-1.6042f,2.1484f,-1.6042f,
	2.1484f,-1.5000f,2.0996f,-1.5000f,2.0996f,-1.3958f,2.0508f,-1.3958f,2.0508f,-1.2917f,2.0020f,-1.2917f,2.0020f,-1.1875f,1.9531f,-1.1875f,1.9531f,-1.0833f,1.9043f,-1.0833f,1.9043f,-0.9792f,1.8555f,-0.9792f,1.8555f,-0.8750f,1.8066f,-0.8750f,1.8066f,-0.7708f,
	1.7578f,-0.7708f,1.7578f,-0.6667f,1.7090f,-0.6667f,1.7090f,-0.5625f,1.6602f,-0.5625f,1.6602f,-0.4583f,1.6113f,-0.4583f,1.6113f,-0.3542f,1.5625f,-0.3542f,1.5625f,-0.2500f,1.5137f,-0.2500f,1.5137f,-0.1458f,1.4648f,-0.1458f,1.4648f,-0.0417f,1.4160f,-0.0417f,
	1.4160f,0.0625f,1.3672f,0.0625f,1.3672f,0.1667f,1.3184f,0.1667f,1.3184f,0.2708f,1.2695f,0.2708f,1.2695f,0.3750f,1.2207f,0.3750f,1.2207f,0.4792f,1.1719f,0.4792f,1.1719f,0.5833f,1.1230f,0.5833f,1.1230f,0.6875f,1.0742f,0.6875f,1.0742f,0.7917f,
	1.0254f,0.7917f,1.0254f,0.8958f,0.9766f,0.8958f,0.9766f,1.0000f,0.9277f,1.0000f,3.2715f,-4.0000f,3.2227f,-4.0000f,3.2227f,-3.8958f,3.1738f,-3.8958f,3.1738f,-3.7917f,3.1250f,-3.7917f,3.1250f,-3.6875f,3.0762f,-3.6875f,3.0762f,-3.5833f,3.0273f,-3.5833f,
	3.0273f,-3.4792f,2.9785f,-3.4792f,2.9785f,-3.3750f,2.9297f,-3.3750f,2.9297f,-3.2708f,2.8809f,-3.2708f,2.8809f,-3.1667f,2.8320f,-3.1667f,2.8320f,-3.0625f,2.7832f,-3.0625f,2.7832f,-2.9583f,2.7344f,-2.9583f,2.7344f,-2.8542f,2.6855f,-2.8542f,2.6855f,-2.7500f,
	2.6367f,-2.7500f,2.6367f,-2.6458f,2.5879f,-2.6458f,2.5879f,-2.5417f,2.5391f,-2.5417f,2.5391f,-2.4375f,2.4902f,-2.4375f,2.4902f,-2.3333f,2.4414f,-2.3333f,2.4414f,-2.2292f,2.3926f,-2.2292f,2.3926f,-2.1250f,2.3437f,-2.1250f,2.3437f,-2.0208f,2.2949f,-2.0208f,
	2.2949f,-1.9167f,2.2461f,-1.9167f,2.2461f,-1.8125f,2.1973f,-1.8125f,2.1973f,-1.7083f,2.1484f,-1.7083f,2.1484f,-1.6042f,2.0996f,-1.6042f,2.0996f,-1.5000f,2.0508f,-1.5000f,2.0508f,-1.3958f,2.0020f,-1.3958f,2.0020f,-1.2917f,1.9531f,-1.2917f,1.9531f,-1.1875f,
	1.9043f,-1.1875f,1.9043f,-1.0833f,1.8555f,-1.0833f,1.8555f,-0.9792f,1.8066f,-0.9792f,1.8066f,-0.8750f,1.7578f,-0.8750f,1.7578f,-0.7708f,1.7090f,-0.7708f,1.7090f,-0.6667f,1.6602f,-0.6667f,1.6602f,-0.5625f,1.6113f,-0.5625f,1.6113f,-0.4583f,1.5625f,-0.4583f,
	1.5625f,-0.3542f,1.5137f,-0.3542f,1.5137f,-0.2500f,1.4648f,-0.2500f,1.4648f,-0.1458f,1.4160f,-0.1458f,1.4160f,-0.0417f,1.3672f,-0.0417f,1.3672f,0.0625f,1.3184f,0.0625f,1.3184f,0.1667f,1.2695f,0.1667f,1.2695f,0.2708f,1.2207f,0.2708f,1.2207f,0.3750f,
	1.1719f,0.3750f,1.1719f,0.4792f,1.1230f,0.4792f,1.1230f,0.5833f,1.0742f,0.5833f,1.0742f,0.6875f,1.0254f,0.6875f,1.0254f,0.7917f,0.9766f,0.7917f,0.9766f,0.8958f,0.9277f,0.8958f,0.9277f,1.0000f,0.8789f,1.0000f,3.2227f,-4.0000f,3.1738f,-4.0000f,
	3.1738f,-3.8958f,3.1250f,-3.8958f,3.1250f,-3.7917f,3.0762f,-3.7917f,3.0762f,-3.6875f,3.0273f,-3.6875f,3.0273f,-3.5833f,2.9785f,-3.5833f,2.9785f,-3.4792f,2.9297f,-3.4792f,2.9297f,-3.3750f,2.8809f,-3.3750f,2.8809f,-3.2708f,2.8320f,-3.2708f,2.8320f,-3.1667f,
	2.7832f,-3.1667f,2.7832f,-3.0625f,2.7344f,-3.0625f,2.7344f,-2.9583f,2.6855f,-2.9583f,2.6855f,-2.8542f,2.6367f,-2.8542f,2.6367f,-2.7500f,2.5879f,-2.7500f,2.5879f,-2.6458f,2.5391f,-2.6458f,2.5391f,-2.5417f,2.4902f,-2.5417f,2.4902f,-2.4375f,2.4414f,-2.4375f,
	2.4414f,-2.3333f,2.3926f,-2.3333f,2.3926f,-2.2292f,2.3437f,-2.2292f,2.3437f,-2.1250f,2.2949f,-2.1250f,2.2949f,-2.0208f,2.2461f,-2.0208f,2.2461f,-1.9167f,2.1973f,-1.9167f,2.1973f,-1.8125f,2.1484f,-1.8125f,2.1484f,-1.7083f,2.0996f,-1.7083f,2.0996f,-1.6042f,
	2.0508f,-1.6042f,2.0508f,-1.5000f,2.0020f,-1.5000f,2.0020f,-1.3958f,1.9531f,-1.3958f,1.9531f,-1.2917f,1.9043f,-1.2917f,1.9043f,-1.1875f,1.8555f,-1.1875f,1.8555f,-1.0833f,1.8066f,-1.0833f,1.8066f,-0.9792f,1.7578f,-0.9792f,1.7578f,-0.8750f,1.7090f,-0.8750f,
	1.7090f,-0.7708f,1.6602f,-0.7708f,1.6602f,-0.6667f,1.6113f,-0.6667f,1.6113f,-0.5625f,1.5625f,-0.5625f,1.5625f,-0.4583f,1.5137f,-0.4583f,1.5137f,-0.3542f,1.4648f,-0.3542f,1.4648f,-0.2500f,1.4160f,-0.2500f,1.4160f,-0.1458f,1.3672f,-0.1458f,1.3672f,-0.0417f,
	1.3184f,-0.0417f,1.3184f,0.0625f,1.2695f,0.0625f,1.2695f,0.1667f,1.2207f,0.1667f,1.2207f,0.2708f,1.1719f,0.2708f,1.1719f,0.3750f,1.1230f,0.3750f,1.1230f,0.4792f,1.0742f,0.4792f,1.0742f,0.5833f,1.0254f,0.5833f,1.0254f,0.6875f,0.9766f,0.6875f,
	0.9766f,0.7917f,0.9277f,0.7917f,0.9277f,0.8958f,0.8789f,0.8958f,0.8789f,1.0000f,0.8301f,1.0000f,3.1738f,-4.0000f,3.1250f,-4.0000f,3.1250f,-3.8958f,3.0762f,-3.8958f,3.0762f,-3.7917f,3.0273f,-3.7917f,3.0273f,-3.6875f,2.9785f,-3.6875f,2.9785f,-3.5833f,
	2.9297f,-3.5833f,2.9297f,-3.4792f,2.8809f,-3.4792f,2.8809f,-3.3750f,2.8320f,-3.3750f,2.8320f,-3.2708f,2.7832f,-3.2708f,2.7832f,-3.1667f,2.7344f,-3.1667f,2.7344f,-3.0625f,2.6855f,-3.0625f,2.6855f,-2.9583f,2.6367f,-2.9583f,2.6367f,-2.8542f,2.5879f,-2.8542f,
	2.5879f,-2.7500f,2.5391f,-2.7500f,2.5391f,-2.6458f,2.4902f,-2.6458f,2.4902f,-2.5417f,2.4414f,-2.5417f,2.4414f,-2.4375f,2.3926f,-2.4375f,2.3926f,-2.3333f,2.3437f,-2.3333f,2.3437f,-2.2292f,2.2949f,-2.2292f,2.2949f,-2.1250f,2.2461f,-2.1250f,2.2461f,-2.0208f,
	2.1973f,-2.0208f,2.1973f,-1.9167f,2.1484f,-1.9167f,2.1484f,-1.8125f,2.0996f,-1.8125f,2.0996f,-1.7083f,2.0508f,-1.7083f,2.0508f,-1.6042f,2.0020f,-1.6042f,2.0020f,-1.5000f,1.9531f,-1.5000f,1.9531f,-1.3958f,1.9043f,-1.3958f,1.9043f,-1.2917f,1.8555f,-1.2917f,
	1.8555f,-1.1875f,1.8066f,-1.1875f,1.8066f,-1.0833f,1.7578f,-1.0833f,1.7578f,-0.9792f,1.7090f,-0.9792f,1.7090f,-0.8750f,1.6602f,-0.8750f,1.6602f,-0.7708f,1.6113f,-0.7708f,1.6113f,-0.6667f,1.5625f,-0.6667f,1.5625f,-0.5625f,1.5137f,-0.5625f,1.5137f,-0.4583f,
	1.4648f,-0.4583f,1.4648f,-0.3542f,1.4160f,-0.3542f,1.4160f,-0.2500f,1.3672f,-0.2500f,1.3672f,-0.1458f,1.3184f,-0.1458f,1.3184f,-0.0417f,1.2695f,-0.0417f,1.2695f,0.0625f,1.2207f,0.0625f,1.2207f,0.1667f,1.1719f,0.1667f,1.1719f,0.2708f,1.1230f,0.2708f,
	1.1230f,0.3750f,1.0742f,0.3750f,1.0742f,0.4792f,1.0254f,0.4792f,1.0254f,0.5833f,0.9766f,0.5833f,0.9766f,0.6875f,0.9277f,0.6875f,0.9277f,0.7917f,0.8789f,0.7917f,0.8789f,0.8958f,0.8301f,0.8958f,0.8301f,1.0000f,0.7813f,1.0000f,3.1250f,-4.0000f,
	3.0762f,-4.0000f,3.0762f,-3.8958f,3.0273f,-3.8958f,3.0273f,-3.7917f,2.9785f,-3.7917f,2.9785f,-3.6875f,2.9297f,-3.6875f,2.9297f,-3.5833f,2.8809f,-3.5833f,2.8809f,-3.4792f,2.8320f,-3.4792f,2.8320f,-3.3750f,2.7832f,-3.3750f,2.7832f,-3.2708f,2.7344f,-3.2708f,
	2.7344f,-3.1667f,2.6855f,-3.1667f,2.6855f,-3.0625f,2.6367f,-3.0625f,2.6367f,-2.9583f,2.5879f,-2.9583f,2.5879f,-2.8542f,2.5391f,-2.8542f,2.5391f,-2.7500f,2.4902f,-2.7500f,2.4902f,-2.6458f,2.4414f,-2.6458f,2.4414f,-2.5417f,2.3926f,-2.5417f,2.3926f,-2.4375f,
	2.3437f,-2.4375f,2.3437f,-2.3333f,2.2949f,-2.3333f,2.2949f,-2.2292f,2.2461f,-2.2292f,2.2461f,-2.1250f,2.1973f,-2.1250f,2.1973f,-2.0208f,2.1484f,-2.0208f,2.1484f,-1.9167f,2.0996f,-1.9167f,2.0996f,-1.8125f,2.0508f,-1.8125f,2.0508f,-1.7083f,2.0020f,-1.7083f,
	2.0020f,-1.6042f,1.9531f,-1.6042f,1.9531f,-1.5000f,1.9043f,-1.5000f,1.9043f,-1.3958f,1.8555f,-1.3958f,1.8555f,-1.2917f,1.8066f,-1.2917f,1.8066f,-1.1875f,1.7578f,-1.1875f,1.7578f,-1.0833f,1.7090f,-1.0833f,1.7090f,-0.9792f,1.6602f,-0.9792f,1.6602f,-0.8750f,
	1.6113f,-0.8750f,1.6113f,-0.7708f,1.5625f,-0.7708f,1.5625f,-0.6667f,1.5137f,-0.6667f,1.5137f,-0.5625f,1.4648f,-0.5625f,1.4648f,-0.4583f,1.4160f,-0.4583f,1.4160f,-0.3542f,1.3672f,-0.3542f,1.3672f,-0.2500f,1.3184f,-0.2500f,1.3184f,-0.1458f,1.2695f,-0.1458f,
	1.2695f,-0.0417f,1.2207f,-0.0417f,1.2207f,0.0625f,1.1719f,0.0625f,1.1719f,0.1667f,1.1230f,0.1667f,1.1230f,0.2708f,1.0742f,0.2708f,1.0742f,0.3750f,1.0254f,0.3750f,1.0254f,0.4792f,0.9766f,0.4792f,0.9766f,0.5833f,0.9277f,0.5833f,0.9277f,0.6875f,
	0.8789f,0.6875f,0.8789f,0.7917f,0.8301f,0.7917f,0.8301f,0.8958f,0.7813f,0.8958f,0.7813f,1.0000f,0.7324f,1.0000f,3.0762f,-4.0000f,3.0273f,-4.0000f,3.0273f,-3.8958f,2.9785f,-3.8958f,2.9785f,-3.7917f,2.9297f,-3.7917f,2.9297f,-3.6875f,2.8809f,-3.6875f,
	2.8809f,-3.5833f,2.8320f,-3.5833f,2.8320f,-3.4792f,2.7832f,-3.4792f,2.7832f,-3.3750f,2.7344f,-3.3750f,2.7344f,-3.2708f,2.6855f,-3.2708f,2.6855f,-3.1667f,2.6367f,-3.1667f,2.6367f,-3.0625f,2.5879f,-3.0625f,2.5879f,-2.9583f,2.5391f,-2.9583f,2.5391f,-2.8542f,
	2.4902f,-2.8542f,2.4902f,-2.7500f,2.4414f,-2.7500f,2.4414f,-2.6458f,2.3926f,-2.6458f,2.3926f,-2.5417f,2.3437f,-2.5417f,2.3437f,-2.4375f,2.2949f,-2.4375f,2.2949f,-2.3333f,2.2461f,-2.3333f,2.2461f,-2.2292f,2.1973f,-2.2292f,2.1973f,-2.1250f,2.1484f,-2.1250f,
	2.1484f,-2.0208f,2.0996f,-2.0208f,2.0996f,-1.9167f,2.0508f,-1.9167f,2.0508f,-1.8125f,2.0020f,-1.8125f,2.0020f,-1.7083f,1.9531f,-1.7083f,1.9531f,-1.6042f,1.9043f,-1.6042f,1.9043f,-1.5000f,1.8555f,-1.5000f,1.8555f,-1.3958f,1.8066f,-1.3958f,1.8066f,-1.2917f,
	1.7578f,-1.2917f,1.7578f,-1.1875f,1.7090f,-1.1875f,1.7090f,-1.0833f,1.6602f,-1.0833f,1.6602f,-0.9792f,1.6113f,-0.9792f,1.6113f,-0.8750f,1.5625f,-0.8750f,1.5625f,-0.7708f,1.5137f,-0.7708f,1.5137f,-0.6667f,1.4648f,-0.6667f,1.4648f,-0.5625f,1.4160f,-0.5625f,
	1.4160f,-0.4583f,1.3672f,-0.4583f,1.3672f,-0.3542f,1.3184f,-0.3542f,1.3184f,-0.2500f,1.2695f,-0.2500f,1.2695f,-0.1458f,1.2207f,-0.1458f,1.2207f,-0.0417f,1.1719f,-0.0417f,1.1719f,0.0625f,1.1230f,0.0625f,1.1230f,0.1667f,1.0742f,0.1667f,1.0742f,0.2708f,
	1.0254f,0.2708f,1.0254f,0.3750f,0.9766f,0.3750f,0.9766f,0.4792f,0.9277f,0.4792f,0.9277f,0.5833f,0.8789f,0.5833f,0.8789f,0.6875f,0.8301f,0.6875f,0.8301f,0.7917f,0.7813f,0.7917f,0.7813f,0.8958f,0.7324f,0.8958f,0.7324f,1.0000f,0.6836f,1.0000f,
	3.0273f,-4.0000f,2.9785f,-4.0000f,2.9785f,-3.8958f,2.9297f,-3.8958f,2.9297f,-3.7917f,2.8809f,-3.7917f,2.8809f,-3.6875f,2.8320f,-3.6875f,2.8320f,-3.5833f,2.7832f,-3.5833f,2.7832f,-3.4792f,2.7344f,-3.4792f,2.7344f,-3.3750f,2.6855f,-3.3750f,2.6855f,-3.2708f,
	2.6367f,-3.2708f,2.6367f,-3.1667f,2.5879f,-3.1667f,2.5879f,-3.0625f,2.5391f,-3.0625f,2.5391f,-2.9583f,2.4902f,-2.9583f,2.4902f,-2.8542f,2.4414f,-2.8542f,2.4414f,-2.7500f,2.3926f,-2.7500f,2.3926f,-2.6458f,2.3437f,-2.6458f,2.3437f,-2.5417f,2.2949f,-2.5417f,
	2.2949f,-2.4375f,2.2461f,-2.4375f,2.2461f,-2.3333f,2.1973f,-2.3333f,2.1973f,-2.2292f,2.1484f,-2.2292f,2.1484f,-2.1250f,2.0996f,-2.1250f,2.0996f,-2.0208f,2.0508f,-2.0208f,2.0508f,-1.9167f,2.0020f,-1.9167f,2.0020f,-1.8125f,1.9531f,-1.8125f,1.9531f,-1.7083f,
	1.9043f,-1.7083f,1.9043f,-1.6042f,1.8555f,-1.6042f,1.8555f,-1.5000f,1.8066f,-1.5000f,1.8066f,-1.3958f,1.7578f,-1.3958f,1.7578f,-1.2917f,1.7090f,-1.2917f,1.7090f,-1.1875f,1.6602f,-1.1875f,1.6602f,-1.0833f,1.6113f,-1.0833f,1.6113f,-0.9792f,1.5625f,-0.9792f,
	1.5625f,-0.8750f,1.5137f,-0.8750f,1.5137f,-0.7708f,1.4648f,-0.7708f,1.4648f,-0.6667f,1.4160f,-0.6667f,1.4160f,-0.5625f,1.3672f,-0.5625f,1.3672f,-0.4583f,1.3184f,-0.4583f,1.3184f,-0.3542f,1.2695f,-0.3542f,1.2695f,-0.2500f,1.2207f,-0.2500f,1.2207f,-0.1458f,
	1.1719f,-0.1458f,1.1719f,-0.0417f,1.1230f,-0.0417f,1.1230f,0.0625f,1.0742f,0.0625f,1.0742f,0.1667f,1.0254f,0.1667f,1.0254f,0.2708f,0.9766f,0.2708f,0.9766f,0.3750f,0.9277f,0.3750f,0.9277f,0.4792f,0.8789f,0.4792f,0.8789f,0.5833f,0.8301f,0.5833f,
	0.8301f,0.6875f,0.7813f,0.6875f,0.7813f,0.7917f,0.7324f,0.7917f,0.7324f,0.8958f,0.6836f,0.8958f,0.6836f,1.0000f,0.6348f,1.0000f,2.9785f,-4.0000f,2.9297f,-4.0000f,2.9297f,-3.8958f,2.8809f,-3.8958f,2.8809f,-3.7917f,2.8320f,-3.7917f,2.8320f,-3.6875f,
	2.7832f,-3.6875f,2.7832f,-3.5833f,2.7344f,-3.5833f,2.7344f,-3.4792f,2.6855f,-3.4792f,2.6855f,-3.3750f,2.6367f,-3.3750f,2.6367f,-3.2708f,2.5879f,-3.2708f,2.5879f,-3.1667f,2.5391f,-3.1667f,2.5391f,-3.0625f,2.4902f,-3.0625f,2.4902f,-2.9583f,2.4414f,-2.9583f,
	2.4414f,-2.8542f,2.3926f,-2.8542f,2.3926f,-2.7500f,2.3437f,-2.7500f,2.3437f,-2.6458f,2.2949f,-2.6458f,2.2949f,-2.5417f,2.2461f,-2.5417f,2.2461f,-2.4375f,2.1973f,-2.4375f,2.1973f,-2.3333f,2.1484f,-2.3333f,2.1484f,-2.2292f,2.0996f,-2.2292f,2.0996f,-2.1250f,
	2.0508f,-2.1250f,2.0508f,-2.0208f,2.0020f,-2.0208f,2.0020f,-1.9167f,1.9531f,-1.9167f,1.9531f,-1.8125f,1.9043f,-1.8125f,1.9043f,-1.7083f,1.8555f,-1.7083f,1.8555f,-1.6042f,1.8066f,-1.6042f,1.8066f,-1.5000f,1.7578f,-1.5000f,1.7578f,-1.3958f,1.7090f,-1.3958f,
	1.7090f,-1.2917f,1.6602f,-1.2917f,1.6602f,-1.1875f,1.6113f,-1.1875f,1.6113f,-1.0833f,1.5625f,-1.0833f,1.5625f,-0.9792f,1.5137f,-0.9792f,1.5137f,-0.8750f,1.4648f,-0.8750f,1.4648f,-0.7708f,1.4160f,-0.7708f,1.4160f,-0.6667f,1.3672f,-0.6667f,1.3672f,-0.5625f,
	1.3184f,-0.5625f,1.3184f,-0.4583f,1.2695f,-0.4583f,1.2695f,-0.3542f,1.2207f,-0.3542f,1.2207f,-0.2500f,1.1719f,-0.2500f,1.1719f,-0.1458f,1.1230f,-0.1458f,1.1230f,-0.0417f,1.0742f,-0.0417f,1.0742f,0.0625f,1.0254f,0.0625f,1.0254f,0.1667f,0.9766f,0.1667f,
	0.9766f,0.2708f,0.9277f,0.2708f,0.9277f,0.3750f,0.8789f,0.3750f,0.8789f,0.4792f,0.8301f,0.4792f,0.8301f,0.5833f,0.7813f,0.5833f,0.7813f,0.6875f,0.7324f,0.6875f,0.7324f,0.7917f,0.6836f,0.7917f,0.6836f,0.8958f,0.6348f,0.8958f,0.6348f,1.0000f,
	0.5859f,1.0000f,2.9297f,-4.0000f,2.8809f,-4.0000f,2.8809f,-3.8958f,2.8320f,-3.8958f,2.8320f,-3.7917f,2.7832f,-3.7917f,2.7832f,-3.6875f,2.7344f,-3.6875f,2.7344f,-3.5833f,2.6855f,-3.5833f,2.6855f,-3.4792f,2.6367f,-3.4792f,2.6367f,-3.3750f,2.5879f,-3.3750f,
	2.5879f,-3.2708f,2.5391f,-3.2708f,2.5391f,-3.1667f,2.4902f,-3.1667f,2.4902f,-3.0625f,2.4414f,-3.0625f,2.4414f,-2.9583f,2.3926f,-2.9583f,2.3926f,-2.8542f,2.3437f,-2.8542f,2.3437f,-2.7500f,2.2949f,-2.7500f,2.2949f,-2.6458f,2.2461f,-2.6458f,2.2461f,-2.5417f,
	2.1973f,-2.5417f,2.1973f,-2.4375f,2.1484f,-2.4375f,2.1484f,-2.3333f,2.0996f,-2.3333f,2.0996f,-2.2292f,2.0508f,-2.2292f,2.0508f,-2.1250f,2.0020f,-2.1250f,2.0020f,-2.0208f,1.9531f,-2.0208f,1.9531f,-1.9167f,1.9043f,-1.9167f,1.9043f,-1.8125f,1.8555f,-1.8125f,
	1.8555f,-1.7083f,1.8066f,-1.7083f,1.8066f,-1.6042f,1.7578f,-1.6042f,1.7578f,-1.5000f,1.7090f,-1.5000f,1.7090f,-1.3958f,1.6602f,-1.3958f,1.6602f,-1.2917f,1.6113f,-1.2917f,1.6113f,-1.1875f,1.5625f,-1.1875f,1.5625f,-1.0833f,1.5137f,-1.0833f,1.5137f,-0.9792f,
	1.4648f,-0.9792f,1.4648f,-0.8750f,1.4160f,-0.8750f,1.4160f,-0.7708f,1.3672f,-0.7708f,1.3672f,-0.6667f,1.3184f,-0.6667f,1.3184f,-0.5625f,1.2695f,-0.5625f,1.2695f,-0.4583f,1.2207f,-0.4583f,1.2207f,-0.3542f,1.1719f,-0.3542f,1.1719f,-0.2500f,1.1230f,-0.2500f,
	1.1230f,-0.1458f,1.0742f,-0.1458f,1.0742f,-0.0417f,1.0254f,-0.0417f,1.0254f,0.0625f,0.9766f,0.0625f,0.9766f,0.1667f,0.9277f,0.1667f,0.9277f,0.2708f,0.8789f,0.2708f,0.8789f,0.3750f,0.8301f,0.3750f,0.8301f,0.4792f,0.7813f,0.4792f,0.7813f,0.5833f,
	0.7324f,0.5833f,0.7324f,0.6875f,0.6836f,0.6875f,0.6836f,0.7917f,0.6348f,0.7917f,0.6348f,0.8958f,0.5859f,0.8958f,0.5859f,1.0000f,0.5371f,1.0000f,2.8809f,-4.0000f,2.8320f,-4.0000f,2.8320f,-3.8958f,2.7832f,-3.8958f,2.7832f,-3.7917f,2.7344f,-3.7917f,
	2.7344f,-3.6875f,2.6855f,-3.6875f,2.6855f,-3.5833f,2.6367f,-3.5833f,2.6367f,-3.4792f,2.5879f,-3.4792f,2.5879f,-3.3750f,2.5391f,-3.3750f,2.5391f,-3.2708f,2.4902f,-3.2708f,2.4902f,-3.1667f,2.4414f,-3.1667f,2.4414f,-3.0625f,2.3926f,-3.0625f,2.3926f,-2.9583f,
	2.3437f,-2.9583f,2.3437f,-2.8542f,2.2949f,-2.8542f,2.2949f,-2.7500f,2.2461f,-2.7500f,2.2461f,-2.6458f,2.1973f,-2.6458f,2.1973f,-2.5417f,2.1484f,-2.5417f,2.1484f,-2.4375f,2.0996f,-2.4375f,2.0996f,-2.3333f,2.0508f,-2.3333f,2.0508f,-2.2292f,2.0020f,-2.2292f,
	2.0020f,-2.1250f,1.9531f,-2.1250f,1.9531f,-2.0208f,1.9043f,-2.0208f,1.9043f,-1.9167f,1.8555f,-1.9167f,1.8555f,-1.8125f,1.8066f,-1.8125f,1.8066f,-1.7083f,1.7578f,-1.7083f,1.7578f,-1.6042f,1.7090f,-1.6042f,1.7090f,-1.5000f,1.6602f,-1.5000f,1.6602f,-1.3958f,
	1.6113f,-1.3958f,1.6113f,-1.2917f,1.5625f,-1.2917f,1.5625f,-1.1875f,1.5137f,-1.1875f,1.5137f,-1.0833f,1.4648f,-1.0833f,1.4648f,-0.9792f,1.4160f,-0.9792f,1.4160f,-0.8750f,1.3672f,-0.8750f,1.3672f,-0.7708f,1.3184f,-0.7708f,1.3184f,-0.6667f,1.2695f,-0.6667f,
	1.2695f,-0.5625f,1.2207f,-0.5625f,1.2207f,-0.4583f,1.1719f,-0.4583f,1.1719f,-0.3542f,1.1230f,-0.3542f,1.1230f,-0.2500f,1.0742f,-0.2500f,1.0742f,-0.1458f,1.0254f,-0.1458f,1.0254f,-0.0417f,0.9766f,-0.0417f,0.9766f,0.0625f,0.9277f,0.0625f,0.9277f,0.1667f,
	0.8789f,0.1667f,0.8789f,0.2708f,0.8301f,0.2708f,0.8301f,0.3750f,0.7813f,0.3750f,0.7813f,0.4792f,0.7324f,0.4792f,0.7324f,0.5833f,0.6836f,0.5833f,0.6836f,0.6875f,0.6348f,0.6875f,0.6348f,0.7917f,0.5859f,0.7917f,0.5859f,0.8958f,0.5371f,0.8958f,
	0.5371f,1.0000f,0.4883f,1.0000f,2.8320f,-4.0000f,2.7832f,-4.0000f,2.7832f,-3.8958f,2.7344f,-3.8958f,2.7344f,-3.7917f,2.6855f,-3.7917f,2.6855f,-3.6875f,2.6367f,-3.6875f,2.6367f,-3.5833f,2.5879f,-3.5833f,2.5879f,-3.4792f,2.5391f,-3.4792f,2.5391f,-3.3750f,
	2.4902f,-3.3750f,2.4902f,-3.2708f,2.4414f,-3.2708f,2.4414f,-3.1667f,2.3926f,-3.1667f,2.3926f,-3.0625f,2.3437f,-3.0625f,2.3437f,-2.9583f,2.2949f,-2.9583f,2.2949f,-2.8542f,2.2461f,-2.8542f,2.2461f,-2.7500f,2.1973f,-2.7500f,2.1973f,-2.6458f,2.1484f,-2.6458f,
	2.1484f,-2.5417f,2.0996f,-2.5417f,2.0996f,-2.4375f,2.0508f,-2.4375f,2.0508f,-2.3333f,2.0020f,-2.3333f,2.0020f,-2.2292f,1.9531f,-2.2292f,1.9531f,-2.1250f,1.9043f,-2.1250f,1.9043f,-2.0208f,1.8555f,-2.0208f,1.8555f,-1.9167f,1.8066f,-1.9167f,1.8066f,-1.8125f,
	1.7578f,-1.8125f,1.7578f,-1.7083f,1.7090f,-1.7083f,1.7090f,-1.6042f,1.6602f,-1.6042f,1.6602f,-1.5000f,1.6113f,-1.5000f,1.6113f,-1.3958f,1.5625f,-1.3958f,1.5625f,-1.2917f,1.5137f,-1.2917f,1.5137f,-1.1875f,1.4648f,-1.1875f,1.4648f,-1.0833f,1.4160f,-1.0833f,
	1.4160f,-0.9792f,1.3672f,-0.9792f,1.3672f,-0.8750f,1.3184f,-0.8750f,1.3184f,-0.7708f,1.2695f,-0.7708f,1.2695f,-0.6667f,1.2207f,-0.6667f,1.2207f,-0.5625f,1.1719f,-0.5625f,1.1719f,-0.4583f,1.1230f,-0.4583f,1.1230f,-0.3542f,1.0742f,-0.3542f,1.0742f,-0.2500f,
	1.0254f,-0.2500f,1.0254f,-0.1458f,0.9766f,-0.1458f,0.9766f,-0.0417f,0.9277f,-0.0417f,0.9277f,0.0625f,0.8789f,0.0625f,0.8789f,0.1667f,0.8301f,0.1667f,0.8301f,0.2708f,0.7813f,0.2708f,0.7813f,0.3750f,0.7324f,0.3750f,0.7324f,0.4792f,0.6836f,0.4792f,
	0.6836f,0.5833f,0.6348f,0.5833f,0.6348f,0.6875f,0.5859f,0.6875f,0.5859f,0.7917f,0.5371f,0.7917f,0.5371f,0.8958f,0.4883f,0.8958f,0.4883f,1.0000f,0.4395f,1.0000f,2.7832f,-4.0000f,2.7344f,-4.0000f,2.7344f,-3.8958f,2.6855f,-3.8958f,2.6855f,-3.7917f,
	2.6367f,-3.7917f,2.6367f,-3.6875f,2.5879f,-3.6875f,2.5879f,-3.5833f,2.5391f,-3.5833f,2.5391f,-3.4792f,2.4902f,-3.4792f,2.4902f,-3.3750f,2.4414f,-3.3750f,2.4414f,-3.2708f,2.3926f,-3.2708f,2.3926f,-3.1667f,2.3437f,-3.1667f,2.3437f,-3.0625f,2.2949f,-3.0625f,
	2.2949f,-2.9583f,2.2461f,-2.9583f,2.2461f,-2.8542f,2.1973f,-2.8542f,2.1973f,-2.7500f,2.1484f,-2.7500f,2.1484f,-2.6458f,2.0996f,-2.6458f,2.0996f,-2.5417f,2.0508f,-2.5417f,2.0508f,-2.4375f,2.0020f,-2.4375f,2.0020f,-2.3333f,1.9531f,-2.3333f,1.9531f,-2.2292f,
	1.9043f,-2.2292f,1.9043f,-2.1250f,1.8555f,-2.1250f,1.8555f,-2.0208f,1.8066f,-2.0208f,1.8066f,-1.9167f,1.7578f,-1.9167f,1.7578f,-1.8125f,1.7090f,-1.8125f,1.7090f,-1.7083f,1.6602f,-1.7083f,1.6602f,-1.6042f,1.6113f,-1.6042f,1.6113f,-1.5000f,1.5625f,-1.5000f,
	1.5625f,-1.3958f,1.5137f,-1.3958f,1.5137f,-1.2917f,1.4648f,-1.2917f,1.4648f,-1.1875f,1.4160f,-1.1875f,1.4160f,-1.0833f,1.3672f,-1.0833f,1.3672f,-0.9792f,1.3184f,-0.9792f,1.3184f,-0.8750f,1.2695f,-0.8750f,1.2695f,-0.7708f,1.2207f,-0.7708f,1.2207f,-0.6667f,
	1.1719f,-0.6667f,1.1719f,-0.5625f,1.1230f,-0.5625f,1.1230f,-0.4583f,1.0742f,-0.4583f,1.0742f,-0.3542f,1.0254f,-0.3542f,1.0254f,-0.2500f,0.9766f,-0.2500f,0.9766f,-0.1458f,0.9277f,-0.1458f,0.9277f,-0.0417f,0.8789f,-0.0417f,0.8789f,0.0625f,0.8301f,0.0625f,
	0.8301f,0.1667f,0.7813f,0.1667f,0.7813f,0.2708f,0.7324f,0.2708f,0.7324f,0.3750f,0.6836f,0.3750f,0.6836f,0.4792f,0.6348f,0.4792f,0.6348f,0.5833f,0.5859f,0.5833f,0.5859f,0.6875f,0.5371f,0.6875f,0.5371f,0.7917f,0.4883f,0.7917f,0.4883f,0.8958f,
	0.4395f,0.8958f,0.4395f,1.0000f,0.3906f,1.0000f,2.7344f,-4.0000f,2.6855f,-4.0000f,2.6855f,-3.8958f,2.6367f,-3.8958f,2.6367f,-3.7917f,2.5879f,-3.7917f,2.5879f,-3.6875f,2.5391f,-3.6875f,2.5391f,-3.5833f,2.4902f,-3.5833f,2.4902f,-3.4792f,2.4414f,-3.4792f,
	2.4414f,-3.3750f,2.3926f,-3.3750f,2.3926f,-3.2708f,2.3437f,-3.2708f,2.3437f,-3.1667f,2.2949f,-3.1667f,2.2949f,-3.0625f,2.2461f,-3.0625f,2.2461f,-2.9583f,2.1973f,-2.9583f,2.1973f,-2.8542f,2.1484f,-2.8542f,2.1484f,-2.7500f,2.0996f,-2.7500f,2.0996f,-2.6458f,
	2.0508f,-2.6458f,2.0508f,-2.5417f,2.0020f,-2.5417f,2.0020f,-2.4375f,1.9531f,-2.4375f,1.9531f,-2.3333f,1.9043f,-2.3333f,1.9043f,-2.2292f,1.8555f,-2.2292f,1.8555f,-2.1250f,1.8066f,-2.1250f,1.8066f,-2.0208f,1.7578f,-2.0208f,1.7578f,-1.9167f,1.7090f,-1.9167f,
	1.7090f,-1.8125f,1.6602f,-1.8125f,1.6602f,-1.7083f,1.6113f,-1.7083f,1.6113f,-1.6042f,1.5625f,-1.6042f,1.5625f,-1.5000f,1.5137f,-1.5000f,1.5137f,-1.3958f,1.4648f,-1.3958f,1.4648f,-1.2917f,1.4160f,-1.2917f,1.4160f,-1.1875f,1.3672f,-1.1875f,1.3672f,-1.0833f,
	1.3184f,-1.0833f,1.3184f,-0.9792f,1.2695f,-0.9792f,1.2695f,-0.8750f,1.2207f,-0.8750f,1.2207f,-0.7708f,1.1719f,-0.7708f,1.1719f,-0.6667f,1.1230f,-0.6667f,1.1230f,-0.5625f,1.0742f,-0.5625f,1.0742f,-0.4583f,1.0254f,-0.4583f,1.0254f,-0.3542f,0.9766f,-0.3542f,
	0.9766f,-0.2500f,0.9277f,-0.2500f,0.9277f,-0.1458f,0.8789f,-0.1458f,0.8789f,-0.0417f,0.8301f,-0.0417f,0.8301f,0.0625f,0.7813f,0.0625f,0.7813f,0.1667f,0.7324f,0.1667f,0.7324f,0.2708f,0.6836f,0.2708f,0.6836f,0.3750f,0.6348f,0.3750f,0.6348f,0.4792f,
	0.5859f,0.4792f,0.5859f,0.5833f,0.5371f,0.5833f,0.5371f,0.6875f,0.4883f,0.6875f,0.4883f,0.7917f,0.4395f,0.7917f,0.4395f,0.8958f,0.3906f,0.8958f,0.3906f,1.0000f,0.3418f,1.0000f,2.6855f,-4.0000f,2.6367f,-4.0000f,2.6367f,-3.8958f,2.5879f,-3.8958f,
	2.5879f,-3.7917f,2.5391f,-3.7917f,2.5391f,-3.6875f,2.4902f,-3.6875f,2.4902f,-3.5833f,2.4414f,-3.5833f,2.4414f,-3.4792f,2.3926f,-3.4792f,2.3926f,-3.3750f,2.3437f,-3.3750f,2.3437f,-3.2708f,2.2949f,-3.2708f,2.2949f,-3.1667f,2.2461f,-3.1667f,2.2461f,-3.0625f,
	2.1973f,-3.0625f,2.1973f,-2.9583f,2.1484f,-2.9583f,2.1484f,-2.8542f,2.0996f,-2.8542f,2.0996f,-2.7500f,2.0508f,-2.7500f,2.0508f,-2.6458f,2.0020f,-2.6458f,2.0020f,-2.5417f,1.9531f,-2.5417f,1.9531f,-2.4375f,1.9043f,-2.4375f,1.9043f,-2.3333f,1.8555f,-2.3333f,
	1.8555f,-2.2292f,1.8066f,-2.2292f,1.8066f,-2.1250f,1.7578f,-2.1250f,1.7578f,-2.0208f,1.7090f,-2.0208f,1.7090f,-1.9167f,1.6602f,-1.9167f,1.6602f,-1.8125f,1.6113f,-1.8125f,1.6113f,-1.7083f,1.5625f,-1.7083f,1.5625f,-1.6042f,1.5137f,-1.6042f,1.5137f,-1.5000f,
	1.4648f,-1.5000f,1.4648f,-1.3958f,1.4160f,-1.3958f,1.4160f,-1.2917f,1.3672f,-1.2917f,1.3672f,-1.1875f,1.3184f,-1.1875f,1.3184f,-1.0833f,1.2695f,-1.0833f,1.2695f,-0.9792f,1.2207f,-0.9792f,1.2207f,-0.8750f,1.1719f,-0.8750f,1.1719f,-0.7708f,1.1230f,-0.7708f,
	1.1230f,-0.6667f,1.0742f,-0.6667f,1.0742f,-0.5625f,1.0254f,-0.5625f,1.0254f,-0.4583f,0.9766f,-0.4583f,0.9766f,-0.3542f,0.9277f,-0.3542f,0.9277f,-0.2500f,0.8789f,-0.2500f,0.8789f,-0.1458f,0.8301f,-0.1458f,0.8301f,-0.0417f,0.7813f,-0.0417f,0.7813f,0.0625f,
	0.7324f,0.0625f,0.7324f,0.1667f,0.6836f,0.1667f,0.6836f,0.2708f,0.6348f,0.2708f,0.6348f,0.3750f,0.5859f,0.3750f,0.5859f,0.4792f,0.5371f,0.4792f,0.5371f,0.5833f,0.4883f,0.5833f,0.4883f,0.6875f,0.4395f,0.6875f,0.4395f,0.7917f,0.3906f,0.7917f,
	0.3906f,0.8958f,0.3418f,0.8958f,0.3418f,1.0000f,0.2930f,1.0000f,2.6367f,-4.0000f,2.5879f,-4.0000f,2.5879f,-3.8958f,2.5391f,-3.8958f,2.5391f,-3.7917f,2.4902f,-3.7917f,2.4902f,-3.6875f,2.4414f,-3.6875f,2.4414f,-3.5833f,2.3926f,-3.5833f,2.3926f,-3.4792f,
	2.3437f,-3.4792f,2.3437f,-3.3750f,2.2949f,-3.3750f,2.2949f,-3.2708f,2.2461f,-3.2708f,2.2461f,-3.1667f,2.1973f,-3.1667f,2.1973f,-3.0625f,2.1484f,-3.0625f,2.1484f,-2.9583f,2.0996f,-2.9583f,2.0996f,-2.8542f,2.0508f,-2.8542f,2.0508f,-2.7500f,2.0020f,-2.7500f,
	2.0020f,-2.6458f,1.9531f,-2.6458f,1.9531f,-2.5417f,1.9043f,-2.5417f,1.9043f,-2.4375f,1.8555f,-2.4375f,1.8555f,-2.3333f,1.8066f,-2.3333f,1.8066f,-2.2292f,1.7578f,-2.2292f,1.7578f,-2.1250f,1.7090f,-2.1250f,1.7090f,-2.0208f,1.6602f,-2.0208f,1.6602f,-1.9167f,
	1.6113f,-1.9167f,1.6113f,-1.8125f,1.5625f,-1.8125f,1.5625f,-1.7083f,1.5137f,-1.7083f,1.5137f,-1.6042f,1.4648f,-1.6042f,1.4648f,-1.5000f,1.4160f,-1.5000f,1.4160f,-1.3958f,1.3672f,-1.3958f,1.3672f,-1.2917f,1.3184f,-1.2917f,1.3184f,-1.1875f,1.2695f,-1.1875f,
	1.2695f,-1.0833f,1.2207f,-1.0833f,1.2207f,-0.9792f,1.1719f,-0.9792f,1.1719f,-0.8750f,1.1230f,-0.8750f,1.1230f,-0.7708f,1.0742f,-0.7708f,1.0742f,-0.6667f,1.0254f,-0.6667f,1.0254f,-0.5625f,0.9766f,-0.5625f,0.9766f,-0.4583f,0.9277f,-0.4583f,0.9277f,-0.3542f,
	0.8789f,-0.3542f,0.8789f,-0.2500f,0.8301f,-0.2500f,0.8301f,-0.1458f,0.7813f,-0.1458f,0.7813f,-0.0417f,0.7324f,-0.0417f,0.7324f,0.0625f,0.6836f,0.0625f,0.6836f,0.1667f,0.6348f,0.1667f,0.6348f,0.2708f,0.5859f,0.2708f,0.5859f,0.3750f,0.5371f,0.3750f,
	0.5371f,0.4792f,0.4883f,0.4792f,0.4883f,0.5833f,0.4395f,0.5833f,0.4395f,0.6875f,0.3906f,0.6875f,0.3906f,0.7917f,0.3418f,0.7917f,0.3418f,0.8958f,0.2930f,0.8958f,0.2930f,1.0000f,0.2441f,1.0000f,2.5879f,-4.0000f,2.5391f,-4.0000f,2.5391f,-3.8958f,
	2.4902f,-3.8958f,2.4902f,-3.7917f,2.4414f,-3.7917f,2.4414f,-3.6875f,2.3926f,-3.6875f,2.3926f,-3.5833f,2.3437f,-3.5833f,2.3437f,-3.4792f,2.2949f,-3.4792f,2.2949f,-3.3750f,2.2461f,-3.3750f,2.2461f,-3.2708f,2.1973f,-3.2708f,2.1973f,-3.1667f,2.1484f,-3.1667f,
	2.1484f,-3.0625f,2.0996f,-3.0625f,2.0996f,-2.9583f,2.0508f,-2.9583f,2.0508f,-2.8542f,2.0020f,-2.8542f,2.0020f,-2.7500f,1.9531f,-2.7500f,1.9531f,-2.6458f,1.9043f,-2.6458f,1.9043f,-2.5417f,1.8555f,-2.5417f,1.8555f,-2.4375f,1.8066f,-2.4375f,1.8066f,-2.3333f,
	1.7578f,-2.3333f,1.7578f,-2.2292f,1.7090f,-2.2292f,1.7090f,-2.1250f,1.6602f,-2.1250f,1.6602f,-2.0208f,1.6113f,-2.0208f,1.6113f,-1.9167f,1.5625f,-1.9167f,1.5625f,-1.8125f,1.5137f,-1.8125f,1.5137f,-1.7083f,1.4648f,-1.7083f,1.4648f,-1.6042f,1.4160f,-1.6042f,
	1.4160f,-1.5000f,1.3672f,-1.5000f,1.3672f,-1.3958f,1.3184f,-1.3958f,1.3184f,-1.2917f,1.2695f,-1.2917f,1.2695f,-1.1875f,1.2207f,-1.1875f,1.2207f,-1.0833f,1.1719f,-1.0833f,1.1719f,-0.9792f,1.1230f,-0.9792f,1.1230f,-0.8750f,1.0742f,-0.8750f,1.0742f,-0.7708f,
	1.0254f,-0.7708f,1.0254f,-0.6667f,0.9766f,-0.6667f,0.9766f,-0.5625f,0.9277f,-0.5625f,0.9277f,-0.4583f,0.8789f,-0.4583f,0.8789f,-0.3542f,0.8301f,-0.3542f,0.8301f,-0.2500f,0.7813f,-0.2500f,0.7813f,-0.1458f,0.7324f,-0.1458f,0.7324f,-0.0417f,0.6836f,-0.0417f,
	0.6836f,0.0625f,0.6348f,0.0625f,0.6348f,0.1667f,0.5859f,0.1667f,0.5859f,0.2708f,0.5371f,0.2708f,0.5371f,0.3750f,0.4883f,0.3750f,0.4883f,0.4792f,0.4395f,0.4792f,0.4395f,0.5833f,0.3906f,0.5833f,0.3906f,0.6875f,0.3418f,0.6875f,0.3418f,0.7917f,
	0.2930f,0.7917f,0.2930f,0.8958f,0.2441f,0.8958f,0.2441f,1.0000f,0.1953f,1.0000f,2.5391f,-4.0000f,2.4902f,-4.0000f,2.4902f,-3.8958f,2.4414f,-3.8958f,2.4414f,-3.7917f,2.3926f,-3.7917f,2.3926f,-3.6875f,2.3437f,-3.6875f,2.3437f,-3.5833f,2.2949f,-3.5833f,
	2.2949f,-3.4792f,2.2461f,-3.4792f,2.2461f,-3.3750f,2.1973f,-3.3750f,2.1973f,-3.2708f,2.1484f,-3.2708f,2.1484f,-3.1667f,2.0996f,-3.1667f,2.0996f,-3.0625f,2.0508f,-3.0625f,2.0508f,-2.9583f,2.0020f,-2.9583f,2.0020f,-2.8542f,1.9531f,-2.8542f,1.9531f,-2.7500f,
	1.9043f,-2.7500f,1.9043f,-2.6458f,1.8555f,-2.6458f,1.8555f,-2.5417f,1.8066f,-2.5417f,1.8066f,-2.4375f,1.7578f,-2.4375f,1.7578f,-2.3333f,1.7090f,-2.3333f,1.7090f,-2.2292f,1.6602f,-2.2292f,1.6602f,-2.1250f,1.6113f,-2.1250f,1.6113f,-2.0208f,1.5625f,-2.0208f,
	1.5625f,-1.9167f,1.5137f,-1.9167f,1.5137f,-1.8125f,1.4648f,-1.8125f,1.4648f,-1.7083f,1.4160f,-1.7083f,1.4160f,-1.6042f,1.3672f,-1.6042f,1.3672f,-1.5000f,1.3184f,-1.5000f,1.3184f,-1.3958f,1.2695f,-1.3958f,1.2695f,-1.2917f,1.2207f,-1.2917f,1.2207f,-1.1875f,
	1.1719f,-1.1875f,1.1719f,-1.0833f,1.1230f,-1.0833f,1.1230f,-0.9792f,1.0742f,-0.9792f,1.0742f,-0.8750f,1.0254f,-0.8750f,1.0254f,-0.7708f,0.9766f,-0.7708f,0.9766f,-0.6667f,0.9277f,-0.6667f,0.9277f,-0.5625f,0.8789f,-0.5625f,0.8789f,-0.4583f,0.8301f,-0.4583f,
	0.8301f,-0.3542f,0.7813f,-0.3542f,0.7813f,-0.2500f,0.7324f,-0.2500f,0.7324f,-0.1458f,0.6836f,-0.1458f,0.6836f,-0.0417f,0.6348f,-0.0417f,0.6348f,0.0625f,0.5859f,0.0625f,0.5859f,0.1667f,0.5371f,0.1667f,0.5371f,0.2708f,0.4883f,0.2708f,0.4883f,0.3750f,
	0.4395f,0.3750f,0.4395f,0.4792f,0.3906f,0.4792f,0.3906f,0.5833f,0.3418f,0.5833f,0.3418f,0.6875f,0.2930f,0.6875f,0.2930f,0.7917f,0.2441f,0.7917f,0.2441f,0.8958f,0.1953f,0.8958f,0.1953f,1.0000f,0.1465f,1.0000f,2.4902f,-4.0000f,2.4414f,-4.0000f,
	2.4414f,-3.8958f,2.3926f,-3.8958f,2.3926f,-3.7917f,2.3437f,-3.7917f,2.3437f,-3.6875f,2.2949f,-3.6875f,2.2949f,-3.5833f,2.2461f,-3.5833f,2.2461f,-3.4792f,2.1973f,-3.4792f,2.1973f,-3.3750f,2.1484f,-3.3750f,2.1484f,-3.2708f,2.0996f,-3.2708f,2.0996f,-3.1667f,
	2.0508f,-3.1667f,2.0508f,-3.0625f,2.0020f,-3.0625f,2.0020f,-2.9583f,1.9531f,-2.9583f,1.9531f,-2.8542f,1.9043f,-2.8542f,1.9043f,-2.7500f,1.8555f,-2.7500f,1.8555f,-2.6458f,1.8066f,-2.6458f,1.8066f,-2.5417f,1.7578f,-2.5417f,1.7578f,-2.4375f,1.7090f,-2.4375f,
	1.7090f,-2.3333f,1.6602f,-2.3333f,1.6602f,-2.2292f,1.6113f,-2.2292f,1.6113f,-2.1250f,1.5625f,-2.1250f,1.5625f,-2.0208f,1.5137f,-2.0208f,1.5137f,-1.9167f,1.4648f,-1.9167f,1.4648f,-1.8125f,1.4160f,-1.8125f,1.4160f,-1.7083f,1.3672f,-1.7083f,1.3672f,-1.6042f,
	1.3184f,-1.6042f,1.3184f,-1.5000f,1.2695f,-1.5000f,1.2695f,-1.3958f,1.2207f,-1.3958f,1.2207f,-1.2917f,1.1719f,-1.2917f,1.1719f,-1.1875f,1.1230f,-1.1875f,1.1230f,-1.0833f,1.0742f,-1.0833f,1.0742f,-0.9792f,1.0254f,-0.9792f,1.0254f,-0.8750f,0.9766f,-0.8750f,
	0.9766f,-0.7708f,0.9277f,-0.7708f,0.9277f,-0.6667f,0.8789f,-0.6667f,0.8789f,-0.5625f,0.8301f,-0.5625f,0.8301f,-0.4583f,0.7813f,-0.4583f,0.7813f,-0.3542f,0.7324f,-0.3542f,0.7324f,-0.2500f,0.6836f,-0.2500f,0.6836f,-0.1458f,0.6348f,-0.1458f,0.6348f,-0.0417f,
	0.5859f,-0.0417f,0.5859f,0.0625f,0.5371f,0.0625f,0.5371f,0.1667f,0.4883f,0.1667f,0.4883f,0.2708f,0.4395f,0.2708f,0.4395f,0.3750f,0.3906f,0.3750f,0.3906f,0.4792f,0.3418f,0.4792f,0.3418f,0.5833f,0.2930f,0.5833f,0.2930f,0.6875f,0.2441f,0.6875f,
	0.2441f,0.7917f,0.1953f,0.7917f,0.1953f,0.8958f,0.1465f,0.8958f,0.1465f,1.0000f,0.0977f,1.0000f,2.4414f,-4.0000f,2.3926f,-4.0000f,2.3926f,-3.8958f,2.3437f,-3.8958f,2.3437f,-3.7917f,2.2949f,-3.7917f,2.2949f,-3.6875f,2.2461f,-3.6875f,2.2461f,-3.5833f,
	2.1973f,-3.5833f,2.1973f,-3.4792f,2.1484f,-3.4792f,2.1484f,-3.3750f,2.0996f,-3.3750f,2.0996f,-3.2708f,2.0508f,-3.2708f,2.0508f,-3.1667f,2.0020f,-3.1667f,2.0020f,-3.0625f,1.9531f,-3.0625f,1.9531f,-2.9583f,1.9043f,-2.9583f,1.9043f,-2.8542f,1.8555f,-2.8542f,
	1.8555f,-2.7500f,1.8066f,-2.7500f,1.8066f,-2.6458f,1.7578f,-2.6458f,1.7578f,-2.5417f,1.7090f,-2.5417f,1.7090f,-2.4375f,1.6602f,-2.4375f,1.6602f,-2.3333f,1.6113f,-2.3333f,1.6113f,-2.2292f,1.5625f,-2.2292f,1.5625f,-2.1250f,1.5137f,-2.1250f,1.5137f,-2.0208f,
	1.4648f,-2.0208f,1.4648f,-1.9167f,1.4160f,-1.9167f,1.4160f,-1.8125f,1.3672f,-1.8125f,1.3672f,-1.7083f,1.3184f,-1.7083f,1.3184f,-1.6042f,1.2695f,-1.6042f,1.2695f,-1.5000f,1.2207f,-1.5000f,1.2207f,-1.3958f,1.1719f,-1.3958f,1.1719f,-1.2917f,1.1230f,-1.2917f,
	1.1230f,-1.1875f,1.0742f,-1.1875f,1.0742f,-1.0833f,1.0254f,-1.0833f,1.0254f,-0.9792f,0.9766f,-0.9792f,0.9766f,-0.8750f,0.9277f,-0.8750f,0.9277f,-0.7708f,0.8789f,-0.7708f,0.8789f,-0.6667f,0.8301f,-0.6667f,0.8301f,-0.5625f,0.7813f,-0.5625f,0.7813f,-0.4583f,
	0.7324f,-0.4583f,0.7324f,-0.3542f,0.6836f,-0.3542f,0.6836f,-0.2500f,0.6348f,-0.2500f,0.6348f,-0.1458f,0.5859f,-0.1458f,0.5859f,-0.0417f,0.5371f,-0.0417f,0.5371f,0.0625f,0.4883f,0.0625f,0.4883f,0.1667f,0.4395f,0.1667f,0.4395f,0.2708f,0.3906f,0.2708f,
	0.3906f,0.3750f,0.3418f,0.3750f,0.3418f,0.4792f,0.2930f,0.4792f,0.2930f,0.5833f,0.2441f,0.5833f,0.2441f,0.6875f,0.1953f,0.6875f,0.1953f,0.7917f,0.1465f,0.7917f,0.1465f,0.8958f,0.0977f,0.8958f,0.0977f,1.0000f,0.0488f,1.0000f,2.3926f,-4.0000f,
	2.3437f,-4.0000f,2.3437f,-3.8958f,2.2949f,-3.8958f,2.2949f,-3.7917f,2.2461f,-3.7917f,2.2461f,-3.6875f,2.1973f,-3.6875f,2.1973f,-3.5833f,2.1484f,-3.5833f,2.1484f,-3.4792f,2.0996f,-3.4792f,2.0996f,-3.3750f,2.0508f,-3.3750f,2.0508f,-3.2708f,2.0020f,-3.2708f,
	2.0020f,-3.1667f,1.9531f,-3.1667f,1.9531f,-3.0625f,1.9043f,-3.0625f,1.9043f,-2.9583f,1.8555f,-2.9583f,1.8555f,-2.8542f,1.8066f,-2.8542f,1.8066f,-2.7500f,1.7578f,-2.7500f,1.7578f,-2.6458f,1.7090f,-2.6458f,1.7090f,-2.5417f,1.6602f,-2.5417f,1.6602f,-2.4375f,
	1.6113f,-2.4375f,1.6113f,-2.3333f,1.5625f,-2.3333f,1.5625f,-2.2292f,1.5137f,-2.2292f,1.5137f,-2.1250f,1.4648f,-2.1250f,1.4648f,-2.0208f,1.4160f,-2.0208f,1.4160f,-1.9167f,1.3672f,-1.9167f,1.3672f,-1.8125f,1.3184f,-1.8125f,1.3184f,-1.7083f,1.2695f,-1.7083f,
	1.2695f,-1.6042f,1.2207f,-1.6042f,1.2207f,-1.5000f,1.1719f,-1.5000f,1.1719f,-1.3958f,1.1230f,-1.3958f,1.1230f,-1.2917f,1.0742f,-1.2917f,1.0742f,-1.1875f,1.0254f,-1.1875f,1.0254f,-1.0833f,0.9766f,-1.0833f,0.9766f,-0.9792f,0.9277f,-0.9792f,0.9277f,-0.8750f,
	0.8789f,-0.8750f,0.8789f,-0.7708f,0.8301f,-0.7708f,0.8301f,-0.6667f,0.7813f,-0.6667f,0.7813f,-0.5625f,0.7324f,-0.5625f,0.7324f,-0.4583f,0.6836f,-0.4583f,0.6836f,-0.3542f,0.6348f,-0.3542f,0.6348f,-0.2500f,0.5859f,-0.2500f,0.5859f,-0.1458f,0.5371f,-0.1458f,
	0.5371f,-0.0417f,0.4883f,-0.0417f,0.4883f,0.0625f,0.4395f,0.0625f,0.4395f,0.1667f,0.3906f,0.1667f,0.3906f,0.2708f,0.3418f,0.2708f,0.3418f,0.3750f,0.2930f,0.3750f,0.2930f,0.4792f,0.2441f,0.4792f,0.2441f,0.5833f,0.1953f,0.5833f,0.1953f,0.6875f,
	0.1465f,0.6875f,0.1465f,0.7917f,0.0977f,0.7917f,0.0977f,0.8958f,0.0488f,0.8958f,0.0488f,1.0000f,0.0000f,1.0000f,2.3437f,-4.0000f,2.2949f,-4.0000f,2.2949f,-3.8958f,2.2461f,-3.8958f,2.2461f,-3.7917f,2.1973f,-3.7917f,2.1973f,-3.6875f,2.1484f,-3.6875f,
	2.1484f,-3.5833f,2.0996f,-3.5833f,2.0996f,-3.4792f,2.0508f,-3.4792f,2.0508f,-3.3750f,2.0020f,-3.3750f,2.0020f,-3.2708f,1.9531f,-3.2708f,1.9531f,-3.1667f,1.9043f,-3.1667f,1.9043f,-3.0625f,1.8555f,-3.0625f,1.8555f,-2.9583f,1.8066f,-2.9583f,1.8066f,-2.8542f,
	1.7578f,-2.8542f,1.7578f,-2.7500f,1.7090f,-2.7500f,1.7090f,-2.6458f,1.6602f,-2.6458f,1.6602f,-2.5417f,1.6113f,-2.5417f,1.6113f,-2.4375f,1.5625f,-2.4375f,1.5625f,-2.3333f,1.5137f,-2.3333f,1.5137f,-2.2292f,1.4648f,-2.2292f,1.4648f,-2.1250f,1.4160f,-2.1250f,
	1.4160f,-2.0208f,1.3672f,-2.0208f,1.3672f,-1.9167f,1.3184f,-1.9167f,1.3184f,-1.8125f,1.2695f,-1.8125f,1.2695f,-1.7083f,1.2207f,-1.7083f,1.2207f,-1.6042f,1.1719f,-1.6042f,1.1719f,-1.5000f,1.1230f,-1.5000f,1.1230f,-1.3958f,1.0742f,-1.3958f,1.0742f,-1.2917f,
	1.0254f,-1.2917f,1.0254f,-1.1875f,0.9766f,-1.1875f,0.9766f,-1.0833f,0.9277f,-1.0833f,0.9277f,-0.9792f,0.8789f,-0.9792f,0.8789f,-0.8750f,0.8301f,-0.8750f,0.8301f,-0.7708f,0.7813f,-0.7708f,0.7813f,-0.6667f,0.7324f,-0.6667f,0.7324f,-0.5625f,0.6836f,-0.5625f,
	0.6836f,-0.4583f,0.6348f,-0.4583f,0.6348f,-0.3542f,0.5859f,-0.3542f,0.5859f,-0.2500f,0.5371f,-0.2500f,0.5371f,-0.1458f,0.4883f,-0.1458f,0.4883f,-0.0417f,0.4395f,-0.0417f,0.4395f,0.0625f,0.3906f,0.0625f,0.3906f,0.1667f,0.3418f,0.1667f,0.3418f,0.2708f,
	0.2930f,0.2708f,0.2930f,0.3750f,0.2441f,0.3750f,0.2441f,0.4792f,0.1953f,0.4792f,0.1953f,0.5833f,0.1465f,0.5833f,0.1465f,0.6875f,0.0977f,0.6875f,0.0977f,0.7917f,0.0488f,0.7917f,0.0488f,0.8958f,0.0000f,0.8958f,0.0000f,1.0000f,2.2949f,-4.0000f,
	2.2461f,-4.0000f,2.2461f,-3.8958f,2.1973f,-3.8958f,2.1973f,-3.7917f,2.1484f,-3.7917f,2.1484f,-3.6875f,2.0996f,-3.6875f,2.0996f,-3.5833f,2.0508f,-3.5833f,2.0508f,-3.4792f,2.0020f,-3.4792f,2.0020f,-3.3750f,1.9531f,-3.3750f,1.9531f,-3.2708f,1.9043f,-3.2708f,
	1.9043f,-3.1667f,1.8555f,-3.1667f,1.8555f,-3.0625f,1.8066f,-3.0625f,1.8066f,-2.9583f,1.7578f,-2.9583f,1.7578f,-2.8542f,1.7090f,-2.8542f,1.7090f,-2.7500f,1.6602f,-2.7500f,1.6602f,-2.6458f,1.6113f,-2.6458f,1.6113f,-2.5417f,1.5625f,-2.5417f,1.5625f,-2.4375f,
	1.5137f,-2.4375f,1.5137f,-2.3333f,1.4648f,-2.3333f,1.4648f,-2.2292f,1.4160f,-2.2292f,1.4160f,-2.1250f,1.3672f,-2.1250f,1.3672f,-2.0208f,1.3184f,-2.0208f,1.3184f,-1.9167f,1.2695f,-1.9167f,1.2695f,-1.8125f,1.2207f,-1.8125f,1.2207f,-1.7083f,1.1719f,-1.7083f,
	1.1719f,-1.6042f,1.1230f,-1.6042f,1.1230f,-1.5000f,1.0742f,-1.5000f,1.0742f,-1.3958f,1.0254f,-1.3958f,1.0254f,-1.2917f,0.9766f,-1.2917f,0.9766f,-1.1875f,0.9277f,-1.1875f,0.9277f,-1.0833f,0.8789f,-1.0833f,0.8789f,-0.9792f,0.8301f,-0.9792f,0.8301f,-0.8750f,
	0.7813f,-0.8750f,0.7813f,-0.7708f,0.7324f,-0.7708f,0.7324f,-0.6667f,0.6836f,-0.6667f,0.6836f,-0.5625f,0.6348f,-0.5625f,0.6348f,-0.4583f,0.5859f,-0.4583f,0.5859f,-0.3542f,0.5371f,-0.3542f,0.5371f,-0.2500f,0.4883f,-0.2500f,0.4883f,-0.1458f,0.4395f,-0.1458f,
	0.4395f,-0.0417f,0.3906f,-0.0417f,0.3906f,0.0625f,0.3418f,0.0625f,0.3418f,0.1667f,0.2930f,0.1667f,0.2930f,0.2708f,0.2441f,0.2708f,0.2441f,0.3750f,0.1953f,0.3750f,0.1953f,0.4792f,0.1465f,0.4792f,0.1465f,0.5833f,0.0977f,0.5833f,0.0977f,0.6875f,
	0.0488f,0.6875f,0.0488f,0.7917f,0.0000f,0.7917f,0.0000f,0.8958f,2.2461f,-4.0000f,2.1973f,-4.0000f,2.1973f,-3.8958f,2.1484f,-3.8958f,2.1484f,-3.7917f,2.0996f,-3.7917f,2.0996f,-3.6875f,2.0508f,-3.6875f,2.0508f,-3.5833f,2.0020f,-3.5833f,2.0020f,-3.4792f,
	1.9531f,-3.4792f,1.9531f,-3.3750f,1.9043f,-3.3750f,1.9043f,-3.2708f,1.8555f,-3.2708f,1.8555f,-3.1667f,1.8066f,-3.1667f,1.8066f,-3.0625f,1.7578f,-3.0625f,1.7578f,-2.9583f,1.7090f,-2.9583f,1.7090f,-2.8542f,1.6602f,-2.8542f,1.6602f,-2.7500f,1.6113f,-2.7500f,
	1.6113f,-2.6458f,1.5625f,-2.6458f,1.5625f,-2.5417f,1.5137f,-2.5417f,1.5137f,-2.4375f,1.4648f,-2.4375f,1.4648f,-2.3333f,1.4160f,-2.3333f,1.4160f,-2.2292f,1.3672f,-2.2292f,1.3672f,-2.1250f,1.3184f,-2.1250f,1.3184f,-2.0208f,1.2695f,-2.0208f,1.2695f,-1.9167f,
	1.2207f,-1.9167f,1.2207f,-1.8125f,1.1719f,-1.8125f,1.1719f,-1.7083f,1.1230f,-1.7083f,1.1230f,-1.6042f,1.0742f,-1.6042f,1.0742f,-1.5000f,1.0254f,-1.5000f,1.0254f,-1.3958f,0.9766f,-1.3958f,0.9766f,-1.2917f,0.9277f,-1.2917f,0.9277f,-1.1875f,0.8789f,-1.1875f,
	0.8789f,-1.0833f,0.8301f,-1.0833f,0.8301f,-0.9792f,0.7813f,-0.9792f,0.7813f,-0.8750f,0.7324f,-0.8750f,0.7324f,-0.7708f,0.6836f,-0.7708f,0.6836f,-0.6667f,0.6348f,-0.6667f,0.6348f,-0.5625f,0.5859f,-0.5625f,0.5859f,-0.4583f,0.5371f,-0.4583f,0.5371f,-0.3542f,
	0.4883f,-0.3542f,0.4883f,-0.2500f,0.4395f,-0.2500f,0.4395f,-0.1458f,0.3906f,-0.1458f,0.3906f,-0.0417f,0.3418f,-0.0417f,0.3418f,0.0625f,0.2930f,0.0625f,0.2930f,0.1667f,0.2441f,0.1667f,0.2441f,0.2708f,0.1953f,0.2708f,0.1953f,0.3750f,0.1465f,0.3750f,
	0.1465f,0.4792f,0.0977f,0.4792f,0.0977f,0.5833f,0.0488f,0.5833f,0.0488f,0.6875f,0.0000f,0.6875f,0.0000f,0.7917f,2.1973f,-4.0000f,2.1484f,-4.0000f,2.1484f,-3.8958f,2.0996f,-3.8958f,2.0996f,-3.7917f,2.0508f,-3.7917f,2.0508f,-3.6875f,2.0020f,-3.6875f,
	2.0020f,-3.5833f,1.9531f,-3.5833f,1.9531f,-3.4792f,1.9043f,-3.4792f,1.9043f,-3.3750f,1.8555f,-3.3750f,1.8555f,-3.2708f,1.8066f,-3.2708f,1.8066f,-3.1667f,1.7578f,-3.1667f,1.7578f,-3.0625f,1.7090f,-3.0625f,1.7090f,-2.9583f,1.6602f,-2.9583f,1.6602f,-2.8542f,
	1.6113f,-2.8542f,1.6113f,-2.7500f,1.5625f,-2.7500f,1.5625f,-2.6458f,1.5137f,-2.6458f,1.5137f,-2.5417f,1.4648f,-2.5417f,1.4648f,-2.4375f,1.4160f,-2.4375f,1.4160f,-2.3333f,1.3672f,-2.3333f,1.3672f,-2.2292f,1.3184f,-2.2292f,1.3184f,-2.1250f,1.2695f,-2.1250f,
	1.2695f,-2.0208f,1.2207f,-2.0208f,1.2207f,-1.9167f,1.1719f,-1.9167f,1.1719f,-1.8125f,1.1230f,-1.8125f,1.1230f,-1.7083f,1.0742f,-1.7083f,1.0742f,-1.6042f,1.0254f,-1.6042f,1.0254f,-1.5000f,0.9766f,-1.5000f,0.9766f,-1.3958f,0.9277f,-1.3958f,0.9277f,-1.2917f,
	0.8789f,-1.2917f,0.8789f,-1.1875f,0.8301f,-1.1875f,0.8301f,-1.0833f,0.7813f,-1.0833f,0.7813f,-0.9792f,0.7324f,-0.9792f,0.7324f,-0.8750f,0.6836f,-0.8750f,0.6836f,-0.7708f,0.6348f,-0.7708f,0.6348f,-0.6667f,0.5859f,-0.6667f,0.5859f,-0.5625f,0.5371f,-0.5625f,
	0.5371f,-0.4583f,0.4883f,-0.4583f,0.4883f,-0.3542f,0.4395f,-0.3542f,0.4395f,-0.2500f,0.3906f,-0.2500f,0.3906f,-0.1458f,0.3418f,-0.1458f,0.3418f,-0.0417f,0.2930f,-0.0417f,0.2930f,0.0625f,0.2441f,0.0625f,0.2441f,0.1667f,0.1953f,0.1667f,0.1953f,0.2708f,
	0.1465f,0.2708f,0.1465f,0.3750f,0.0977f,0.3750f,0.0977f,0.4792f,0.0488f,0.4792f,0.0488f,0.5833f,0.0000f,0.5833f,0.0000f,0.6875f,2.1484f,-4.0000f,2.0996f,-4.0000f,2.0996f,-3.8958f,2.0508f,-3.8958f,2.0508f,-3.7917f,2.0020f,-3.7917f,2.0020f,-3.6875f,
	1.9531f,-3.6875f,1.9531f,-3.5833f,1.9043f,-3.5833f,1.9043f,-3.4792f,1.8555f,-3.4792f,1.8555f,-3.3750f,1.8066f,-3.3750f,1.8066f,-3.2708f,1.7578f,-3.2708f,1.7578f,-3.1667f,1.7090f,-3.1667f,1.7090f,-3.0625f,1.6602f,-3.0625f,1.6602f,-2.9583f,1.6113f,-2.9583f,
	1.6113f,-2.8542f,1.5625f,-2.8542f,1.5625f,-2.7500f,1.5137f,-2.7500f,1.5137f,-2.6458f,1.4648f,-2.6458f,1.4648f,-2.5417f,1.4160f,-2.5417f,1.4160f,-2.4375f,1.3672f,-2.4375f,1.3672f,-2.3333f,1.3184f,-2.3333f,1.3184f,-2.2292f,1.2695f,-2.2292f,1.2695f,-2.1250f,
	1.2207f,-2.1250f,1.2207f,-2.0208f,1.1719f,-2.0208f,1.1719f,-1.9167f,1.1230f,-1.9167f,1.1230f,-1.8125f,1.0742f,-1.8125f,1.0742f,-1.7083f,1.0254f,-1.7083f,1.0254f,-1.6042f,0.9766f,-1.6042f,0.9766f,-1.5000f,0.9277f,-1.5000f,0.9277f,-1.3958f,0.8789f,-1.3958f,
	0.8789f,-1.2917f,0.8301f,-1.2917f,0.8301f,-1.1875f,0.7813f,-1.1875f,0.7813f,-1.0833f,0.7324f,-1.0833f,0.7324f,-0.9792f,0.6836f,-0.9792f,0.6836f,-0.8750f,0.6348f,-0.8750f,0.6348f,-0.7708f,0.5859f,-0.7708f,0.5859f,-0.6667f,0.5371f,-0.6667f,0.5371f,-0.5625f,
	0.4883f,-0.5625f,0.4883f,-0.4583f,0.4395f,-0.4583f,0.4395f,-0.3542f,0.3906f,-0.3542f,0.3906f,-0.2500f,0.3418f,-0.2500f,0.3418f,-0.1458f,0.2930f,-0.1458f,0.2930f,-0.0417f,0.2441f,-0.0417f,0.2441f,0.0625f,0.1953f,0.0625f,0.1953f,0.1667f,0.1465f,0.1667f,
	0.1465f,0.2708f,0.0977f,0.2708f,0.0977f,0.3750f,0.0488f,0.3750f,0.0488f,0.4792f,0.0000f,0.4792f,0.0000f,0.5833f,2.0996f,-4.0000f,2.0508f,-4.0000f,2.0508f,-3.8958f,2.0020f,-3.8958f,2.0020f,-3.7917f,1.9531f,-3.7917f,1.9531f,-3.6875f,1.9043f,-3.6875f,
	1.9043f,-3.5833f,1.8555f,-3.5833f,1.8555f,-3.4792f,1.8066f,-3.4792f,1.8066f,-3.3750f,1.7578f,-3.3750f,1.7578f,-3.2708f,1.7090f,-3.2708f,1.7090f,-3.1667f,1.6602f,-3.1667f,1.6602f,-3.0625f,1.6113f,-3.0625f,1.6113f,-2.9583f,1.5625f,-2.9583f,1.5625f,-2.8542f,
	1.5137f,-2.8542f,1.5137f,-2.7500f,1.4648f,-2.7500f,1.4648f,-2.6458f,1.4160f,-2.6458f,1.4160f,-2.5417f,1.3672f,-2.5417f,1.3672f,-2.4375f,1.3184f,-2.4375f,1.3184f,-2.3333f,1.2695f,-2.3333f,1.2695f,-2.2292f,1.2207f,-2.2292f,1.2207f,-2.1250f,1.1719f,-2.1250f,
	1.1719f,-2.0208f,1.1230f,-2.0208f,1.1230f,-1.9167f,1.0742f,-1.9167f,1.0742f,-1.8125f,1.0254f,-1.8125f,1.0254f,-1.7083f,0.9766f,-1.7083f,0.9766f,-1.6042f,0.9277f,-1.6042f,0.9277f,-1.5000f,0.8789f,-1.5000f,0.8789f,-1.3958f,0.8301f,-1.3958f,0.8301f,-1.2917f,
	0.7813f,-1.2917f,0.7813f,-1.1875f,0.7324f,-1.1875f,0.7324f,-1.0833f,0.6836f,-1.0833f,0.6836f,-0.9792f,0.6348f,-0.9792f,0.6348f,-0.8750f,0.5859f,-0.8750f,0.5859f,-0.7708f,0.5371f,-0.7708f,0.5371f,-0.6667f,0.4883f,-0.6667f,0.4883f,-0.5625f,0.4395f,-0.5625f,
	0.4395f,-0.4583f,0.3906f,-0.4583f,0.3906f,-0.3542f,0.3418f,-0.3542f,0.3418f,-0.2500f,0.2930f,-0.2500f,0.2930f,-0.1458f,0.2441f,-0.1458f,0.2441f,-0.0417f,0.1953f,-0.0417f,0.1953f,0.0625f,0.1465f,0.0625f,0.1465f,0.1667f,0.0977f,0.1667f,0.0977f,0.2708f,
	0.0488f,0.2708f,0.0488f,0.3750f,0.0000f,0.3750f,0.0000f,0.4792f,2.0508f,-4.0000f,2.0020f,-4.0000f,2.0020f,-3.8958f,1.9531f,-3.8958f,1.9531f,-3.7917f,1.9043f,-3.7917f,1.9043f,-3.6875f,1.8555f,-3.6875f,1.8555f,-3.5833f,1.8066f,-3.5833f,1.8066f,-3.4792f,
	1.7578f,-3.4792f,1.7578f,-3.3750f,1.7090f,-3.3750f,1.7090f,-3.2708f,1.6602f,-3.2708f,1.6602f,-3.1667f,1.6113f,-3.1667f,1.6113f,-3.0625f,1.5625f,-3.0625f,1.5625f,-2.9583f,1.5137f,-2.9583f,1.5137f,-2.8542f,1.4648f,-2.8542f,1.4648f,-2.7500f,1.4160f,-2.7500f,
	1.4160f,-2.6458f,1.3672f,-2.6458f,1.3672f,-2.5417f,1.3184f,-2.5417f,1.3184f,-2.4375f,1.2695f,-2.4375f,1.2695f,-2.3333f,1.2207f,-2.3333f,1.2207f,-2.2292f,1.1719f,-2.2292f,1.1719f,-2.1250f,1.1230f,-2.1250f,1.1230f,-2.0208f,1.0742f,-2.0208f,1.0742f,-1.9167f,
	1.0254f,-1.9167f,1.0254f,-1.8125f,0.9766f,-1.8125f,0.9766f,-1.7083f,0.9277f,-1.7083f,0.9277f,-1.6042f,0.8789f,-1.6042f,0.8789f,-1.5000f,0.8301f,-1.5000f,0.8301f,-1.3958f,0.7813f,-1.3958f,0.7813f,-1.2917f,0.7324f,-1.2917f,0.7324f,-1.1875f,0.6836f,-1.1875f,
	0.6836f,-1.0833f,0.6348f,-1.0833f,0.6348f,-0.9792f,0.5859f,-0.9792f,0.5859f,-0.8750f,0.5371f,-0.8750f,0.5371f,-0.7708f,0.4883f,-0.7708f,0.4883f,-0.6667f,0.4395f,-0.6667f,0.4395f,-0.5625f,0.3906f,-0.5625f,0.3906f,-0.4583f,0.3418f,-0.4583f,0.3418f,-0.3542f,
	0.2930f,-0.3542f,0.2930f,-0.2500f,0.2441f,-0.2500f,0.2441f,-0.1458f,0.1953f,-0.1458f,0.1953f,-0.0417f,0.1465f,-0.0417f,0.1465f,0.0625f,0.0977f,0.0625f,0.0977f,0.1667f,0.0488f,0.1667f,0.0488f,0.2708f,0.0000f,0.2708f,0.0000f,0.3750f,2.0020f,-4.0000f,
	1.9531f,-4.0000f,1.9531f,-3.8958f,1.9043f,-3.8958f,1.9043f,-3.7917f,1.8555f,-3.7917f,1.8555f,-3.6875f,1.8066f,-3.6875f,1.8066f,-3.5833f,1.7578f,-3.5833f,1.7578f,-3.4792f,1.7090f,-3.4792f,1.7090f,-3.3750f,1.6602f,-3.3750f,1.6602f,-3.2708f,1.6113f,-3.2708f,
	1.6113f,-3.1667f,1.5625f,-3.1667f,1.5625f,-3.0625f,1.5137f,-3.0625f,1.5137f,-2.9583f,1.4648f,-2.9583f,1.4648f,-2.8542f,1.4160f,-2.8542f,1.4160f,-2.7500f,1.3672f,-2.7500f,1.3672f,-2.6458f,1.3184f,-2.6458f,1.3184f,-2.5417f,1.2695f,-2.5417f,1.2695f,-2.4375f,
	1.2207f,-2.4375f,1.2207f,-2.3333f,1.1719f,-2.3333f,1.1719f,-2.2292f,1.1230f,-2.2292f,1.1230f,-2.1250f,1.0742f,-2.1250f,1.0742f,-2.0208f,1.0254f,-2.0208f,1.0254f,-1.9167f,0.9766f,-1.9167f,0.9766f,-1.8125f,0.9277f,-1.8125f,0.9277f,-1.7083f,0.8789f,-1.7083f,
	0.8789f,-1.6042f,0.8301f,-1.6042f,0.8301f,-1.5000f,0.7813f,-1.5000f,0.7813f,-1.3958f,0.7324f,-1.3958f,0.7324f,-1.2917f,0.6836f,-1.2917f,0.6836f,-1.1875f,0.6348f,-1.1875f,0.6348f,-1.0833f,0.5859f,-1.0833f,0.5859f,-0.9792f,0.5371f,-0.9792f,0.5371f,-0.8750f,
	0.4883f,-0.8750f,0.4883f,-0.7708f,0.4395f,-0.7708f,0.4395f,-0.6667f,0.3906f,-0.6667f,0.3906f,-0.5625f,0.3418f,-0.5625f,0.3418f,-0.4583f,0.2930f,-0.4583f,0.2930f,-0.3542f,0.2441f,-0.3542f,0.2441f,-0.2500f,0.1953f,-0.2500f,0.1953f,-0.1458f,0.1465f,-0.1458f,
	0.1465f,-0.0417f,0.0977f,-0.0417f,0.0977f,0.0625f,0.0488f,0.0625f,0.0488f,0.1667f,0.0000f,0.1667f,0.0000f,0.2708f,1.9531f,-4.0000f,1.9043f,-4.0000f,1.9043f,-3.8958f,1.8555f,-3.8958f,1.8555f,-3.7917f,1.8066f,-3.7917f,1.8066f,-3.6875f,1.7578f,-3.6875f,
	1.7578f,-3.5833f,1.7090f,-3.5833f,1.7090f,-3.4792f,1.6602f,-3.4792f,1.6602f,-3.3750f,1.6113f,-3.3750f,1.6113f,-3.2708f,1.5625f,-3.2708f,1.5625f,-3.1667f,1.5137f,-3.1667f,1.5137f,-3.0625f,1.4648f,-3.0625f,1.4648f,-2.9583f,1.4160f,-2.9583f,1.4160f,-2.8542f,
	1.3672f,-2.8542f,1.3672f,-2.7500f,1.3184f,-2.7500f,1.3184f,-2.6458f,1.2695f,-2.6458f,1.2695f,-2.5417f,1.2207f,-2.5417f,1.2207f,-2.4375f,1.1719f,-2.4375f,1.1719f,-2.3333f,1.1230f,-2.3333f,1.1230f,-2.2292f,1.0742f,-2.2292f,1.0742f,-2.1250f,1.0254f,-2.1250f,
	1.0254f,-2.0208f,0.9766f,-2.0208f,0.9766f,-1.9167f,0.9277f,-1.9167f,0.9277f,-1.8125f,0.8789f,-1.8125f,0.8789f,-1.7083f,0.8301f,-1.7083f,0.8301f,-1.6042f,0.7813f,-1.6042f,0.7813f,-1.5000f,0.7324f,-1.5000f,0.7324f,-1.3958f,0.6836f,-1.3958f,0.6836f,-1.2917f,
	0.6348f,-1.2917f,0.6348f,-1.1875f,0.5859f,-1.1875f,0.5859f,-1.0833f,0.5371f,-1.0833f,0.5371f,-0.9792f,0.4883f,-0.9792f,0.4883f,-0.8750f,0.4395f,-0.8750f,0.4395f,-0.7708f,0.3906f,-0.7708f,0.3906f,-0.6667f,0.3418f,-0.6667f,0.3418f,-0.5625f,0.2930f,-0.5625f,
	0.2930f,-0.4583f,0.2441f,-0.4583f,0.2441f,-0.3542f,0.1953f,-0.3542f,0.1953f,-0.2500f,0.1465f,-0.2500f,0.1465f,-0.1458f,0.0977f,-0.1458f,0.0977f,-0.0417f,0.0488f,-0.0417f,0.0488f,0.0625f,0.0000f,0.0625f,0.0000f,0.1667f,1.9043f,-4.0000f,1.8555f,-4.0000f,
	1.8555f,-3.8958f,1.8066f,-3.8958f,1.8066f,-3.7917f,1.7578f,-3.7917f,1.7578f,-3.6875f,1.7090f,-3.6875f,1.7090f,-3.5833f,1.6602f,-3.5833f,1.6602f,-3.4792f,1.6113f,-3.4792f,1.6113f,-3.3750f,1.5625f,-3.3750f,1.5625f,-3.2708f,1.5137f,-3.2708f,1.5137f,-3.1667f,
	1.4648f,-3.1667f,1.4648f,-3.0625f,1.4160f,-3.0625f,1.4160f,-2.9583f,1.3672f,-2.9583f,1.3672f,-2.8542f,1.3184f,-2.8542f,1.3184f,-2.7500f,1.2695f,-2.7500f,1.2695f,-2.6458f,1.2207f,-2.6458f,1.2207f,-2.5417f,1.1719f,-2.5417f,1.1719f,-2.4375f,1.1230f,-2.4375f,
	1.1230f,-2.3333f,1.0742f,-2.3333f,1.0742f,-2.2292f,1.0254f,-2.2292f,1.0254f,-2.1250f,0.9766f,-2.1250f,0.9766f,-2.0208f,0.9277f,-2.0208f,0.9277f,-1.9167f,0.8789f,-1.9167f,0.8789f,-1.8125f,0.8301f,-1.8125f,0.8301f,-1.7083f,0.7813f,-1.7083f,0.7813f,-1.6042f,
	0.7324f,-1.6042f,0.7324f,-1.5000f,0.6836f,-1.5000f,0.6836f,-1.3958f,0.6348f,-1.3958f,0.6348f,-1.2917f,0.5859f,-1.2917f,0.5859f,-1.1875f,0.5371f,-1.1875f,0.5371f,-1.0833f,0.4883f,-1.0833f,0.4883f,-0.9792f,0.4395f,-0.9792f,0.4395f,-0.8750f,0.3906f,-0.8750f,
	0.3906f,-0.7708f,0.3418f,-0.7708f,0.3418f,-0.6667f,0.2930f,-0.6667f,0.2930f,-0.5625f,0.2441f,-0.5625f,0.2441f,-0.4583f,0.1953f,-0.4583f,0.1953f,-0.3542f,0.1465f,-0.3542f,0.1465f,-0.2500f,0.0977f,-0.2500f,0.0977f,-0.1458f,0.0488f,-0.1458f,0.0488f,-0.0417f,
	0.0000f,-0.0417f,0.0000f,0.0625f,1.8555f,-4.0000f,1.8066f,-4.0000f,1.8066f,-3.8958f,1.7578f,-3.8958f,1.7578f,-3.7917f,1.7090f,-3.7917f,1.7090f,-3.6875f,1.6602f,-3.6875f,1.6602f,-3.5833f,1.6113f,-3.5833f,1.6113f,-3.4792f,1.5625f,-3.4792f,1.5625f,-3.3750f,
	1.5137f,-3.3750f,1.5137f,-3.2708f,1.4648f,-3.2708f,1.4648f,-3.1667f,1.4160f,-3.1667f,1.4160f,-3.0625f,1.3672f,-3.0625f,1.3672f,-2.9583f,1.3184f,-2.9583f,1.3184f,-2.8542f,1.2695f,-2.8542f,1.2695f,-2.7500f,1.2207f,-2.7500f,1.2207f,-2.6458f,1.1719f,-2.6458f,
	1.1719f,-2.5417f,1.1230f,-2.5417f,1.1230f,-2.4375f,1.0742f,-2.4375f,1.0742f,-2.3333f,1.0254f,-2.3333f,1.0254f,-2.2292f,0.9766f,-2.2292f,0.9766f,-2.1250f,0.9277f,-2.1250f,0.9277f,-2.0208f,0.8789f,-2.0208f,0.8789f,-1.9167f,0.8301f,-1.9167f,0.8301f,-1.8125f,
	0.7813f,-1.8125f,0.7813f,-1.7083f,0.7324f,-1.7083f,0.7324f,-1.6042f,0.6836f,-1.6042f,0.6836f,-1.5000f,0.6348f,-1.5000f,0.6348f,-1.3958f,0.5859f,-1.3958f,0.5859f,-1.2917f,0.5371f,-1.2917f,0.5371f,-1.1875f,0.4883f,-1.1875f,0.4883f,-1.0833f,0.4395f,-1.0833f,
	0.4395f,-0.9792f,0.3906f,-0.9792f,0.3906f,-0.8750f,0.3418f,-0.8750f,0.3418f,-0.7708f,0.2930f,-0.7708f,0.2930f,-0.6667f,0.2441f,-0.6667f,0.2441f,-0.5625f,0.1953f,-0.5625f,0.1953f,-0.4583f,0.1465f,-0.4583f,0.1465f,-0.3542f,0.0977f,-0.3542f,0.0977f,-0.2500f,
	0.0488f,-0.2500f,0.0488f,-0.1458f,0.0000f,-0.1458f,0.0000f,-0.0417f,1.8066f,-4.0000f,1.7578f,-4.0000f,1.7578f,-3.8958f,1.7090f,-3.8958f,1.7090f,-3.7917f,1.6602f,-3.7917f,1.6602f,-3.6875f,1.6113f,-3.6875f,1.6113f,-3.5833f,1.5625f,-3.5833f,1.5625f,-3.4792f,
	1.5137f,-3.4792f,1.5137f,-3.3750f,1.4648f,-3.3750f,1.4648f,-3.2708f,1.4160f,-3.2708f,1.4160f,-3.1667f,1.3672f,-3.1667f,1.3672f,-3.0625f,1.3184f,-3.0625f,1.3184f,-2.9583f,1.2695f,-2.9583f,1.2695f,-2.8542f,1.2207f,-2.8542f,1.2207f,-2.7500f,1.1719f,-2.7500f,
	1.1719f,-2.6458f,1.1230f,-2.6458f,1.1230f,-2.5417f,1.0742f,-2.5417f,1.0742f,-2.4375f,1.0254f,-2.4375f,1.0254f,-2.3333f,0.9766f,-2.3333f,0.9766f,-2.2292f,0.9277f,-2.2292f,0.9277f,-2.1250f,0.8789f,-2.1250f,0.8789f,-2.0208f,0.8301f,-2.0208f,0.8301f,-1.9167f,
	0.7813f,-1.9167f,0.7813f,-1.8125f,0.7324f,-1.8125f,0.7324f,-1.7083f,0.6836f,-1.7083f,0.6836f,-1.6042f,0.6348f,-1.6042f,0.6348f,-1.5000f,0.5859f,-1.5000f,0.5859f,-1.3958f,0.5371f,-1.3958f,0.5371f,-1.2917f,0.4883f,-1.2917f,0.4883f,-1.1875f,0.4395f,-1.1875f,
	0.4395f,-1.0833f,0.3906f,-1.0833f,0.3906f,-0.9792f,0.3418f,-0.9792f,0.3418f,-0.8750f,0.2930f,-0.8750f,0.2930f,-0.7708f,0.2441f,-0.7708f,0.2441f,-0.6667f,0.1953f,-0.6667f,0.1953f,-0.5625f,0.1465f,-0.5625f,0.1465f,-0.4583f,0.0977f,-0.4583f,0.0977f,-0.3542f,
	0.0488f,-0.3542f,0.0488f,-0.2500f,0.0000f,-0.2500f,0.0000f,-0.1458f,1.7578f,-4.0000f,1.7090f,-4.0000f,1.7090f,-3.8958f,1.6602f,-3.8958f,1.6602f,-3.7917f,1.6113f,-3.7917f,1.6113f,-3.6875f,1.5625f,-3.6875f,1.5625f,-3.5833f,1.5137f,-3.5833f,1.5137f,-3.4792f,
	1.4648f,-3.4792f,1.4648f,-3.3750f,1.4160f,-3.3750f,1.4160f,-3.2708f,1.3672f,-3.2708f,1.3672f,-3.1667f,1.3184f,-3.1667f,1.3184f,-3.0625f,1.2695f,-3.0625f,1.2695f,-2.9583f,1.2207f,-2.9583f,1.2207f,-2.8542f,1.1719f,-2.8542f,1.1719f,-2.7500f,1.1230f,-2.7500f,
	1.1230f,-2.6458f,1.0742f,-2.6458f,1.0742f,-2.5417f,1.0254f,-2.5417f,1.0254f,-2.4375f,0.9766f,-2.4375f,0.9766f,-2.3333f,0.9277f,-2.3333f,0.9277f,-2.2292f,0.8789f,-2.2292f,0.8789f,-2.1250f,0.8301f,-2.1250f,0.8301f,-2.0208f,0.7813f,-2.0208f,0.7813f,-1.9167f,
	0.7324f,-1.9167f,0.7324f,-1.8125f,0.6836f,-1.8125f,0.6836f,-1.7083f,0.6348f,-1.7083f,0.6348f,-1.6042f,0.5859f,-1.6042f,0.5859f,-1.5000f,0.5371f,-1.5000f,0.5371f,-1.3958f,0.4883f,-1.3958f,0.4883f,-1.2917f,0.4395f,-1.2917f,0.4395f,-1.1875f,0.3906f,-1.1875f,
	0.3906f,-1.0833f,0.3418f,-1.0833f,0.3418f,-0.9792f,0.2930f,-0.9792f,0.2930f,-0.8750f,0.2441f,-0.8750f,0.2441f,-0.7708f,0.1953f,-0.7708f,0.1953f,-0.6667f,0.1465f,-0.6667f,0.1465f,-0.5625f,0.0977f,-0.5625f,0.0977f,-0.4583f,0.0488f,-0.4583f,0.0488f,-0.3542f,
	0.0000f,-0.3542f,0.0000f,-0.2500f,1.7090f,-4.0000f,1.6602f,-4.0000f,1.6602f,-3.8958f,1.6113f,-3.8958f,1.6113f,-3.7917f,1.5625f,-3.7917f,1.5625f,-3.6875f,1.5137f,-3.6875f,1.5137f,-3.5833f,1.4648f,-3.5833f,1.4648f,-3.4792f,1.4160f,-3.4792f,1.4160f,-3.3750f,
	1.3672f,-3.3750f,1.3672f,-3.2708f,1.3184f,-3.2708f,1.3184f,-3.1667f,1.2695f,-3.1667f,1.2695f,-3.0625f,1.2207f,-3.0625f,1.2207f,-2.9583f,1.1719f,-2.9583f,1.1719f,-2.8542f,1.1230f,-2.8542f,1.1230f,-2.7500f,1.0742f,-2.7500f,1.0742f,-2.6458f,1.0254f,-2.6458f,
	1.0254f,-2.5417f,0.9766f,-2.5417f,0.9766f,-2.4375f,0.9277f,-2.4375f,0.9277f,-2.3333f,0.8789f,-2.3333f,0.8789f,-2.2292f,0.8301f,-2.2292f,0.8301f,-2.1250f,0.7813f,-2.1250f,0.7813f,-2.0208f,0.7324f,-2.0208f,0.7324f,-1.9167f,0.6836f,-1.9167f,0.6836f,-1.8125f,
	0.6348f,-1.8125f,0.6348f,-1.7083f,0.5859f,-1.7083f,0.5859f,-1.6042f,0.5371f,-1.6042f,0.5371f,-1.5000f,0.4883f,-1.5000f,0.4883f,-1.3958f,0.4395f,-1.3958f,0.4395f,-1.2917f,0.3906f,-1.2917f,0.3906f,-1.1875f,0.3418f,-1.1875f,0.3418f,-1.0833f,0.2930f,-1.0833f,
	0.2930f,-0.9792f,0.2441f,-0.9792f,0.2441f,-0.8750f,0.1953f,-0.8750f,0.1953f,-0.7708f,0.1465f,-0.7708f,0.1465f,-0.6667f,0.0977f,-0.6667f,0.0977f,-0.5625f,0.0488f,-0.5625f,0.0488f,-0.4583f,0.0000f,-0.4583f,0.0000f,-0.3542f,1.6602f,-4.0000f,1.6113f,-4.0000f,
	1.6113f,-3.8958f,1.5625f,-3.8958f,1.5625f,-3.7917f,1.5137f,-3.7917f,1.5137f,-3.6875f,1.4648f,-3.6875f,1.4648f,-3.5833f,1.4160f,-3.5833f,1.4160f,-3.4792f,1.3672f,-3.4792f,1.3672f,-3.3750f,1.3184f,-3.3750f,1.3184f,-3.2708f,1.2695f,-3.2708f,1.2695f,-3.1667f,
	1.2207f,-3.1667f,1.2207f,-3.0625f,1.1719f,-3.0625f,1.1719f,-2.9583f,1.1230f,-2.9583f,1.1230f,-2.8542f,1.0742f,-2.8542f,1.0742f,-2.7500f,1.0254f,-2.7500f,1.0254f,-2.6458f,0.9766f,-2.6458f,0.9766f,-2.5417f,0.9277f,-2.5417f,0.9277f,-2.4375f,0.8789f,-2.4375f,
	0.8789f,-2.3333f,0.8301f,-2.3333f,0.8301f,-2.2292f,0.7813f,-2.2292f,0.7813f,-2.1250f,0.7324f,-2.1250f,0.7324f,-2.0208f,0.6836f,-2.0208f,0.6836f,-1.9167f,0.6348f,-1.9167f,0.6348f,-1.8125f,0.5859f,-1.8125f,0.5859f,-1.7083f,0.5371f,-1.7083f,0.5371f,-1.6042f,
	0.4883f,-1.6042f,0.4883f,-1.5000f,0.4395f,-1.5000f,0.4395f,-1.3958f,0.3906f,-1.3958f,0.3906f,-1.2917f,0.3418f,-1.2917f,0.3418f,-1.1875f,0.2930f,-1.1875f,0.2930f,-1.0833f,0.2441f,-1.0833f,0.2441f,-0.9792f,0.1953f,-0.9792f,0.1953f,-0.8750f,0.1465f,-0.8750f,
	0.1465f,-0.7708f,0.0977f,-0.7708f,0.0977f,-0.6667f,0.0488f,-0.6667f,0.0488f,-0.5625f,0.0000f,-0.5625f,0.0000f,-0.4583f,1.6113f,-4.0000f,1.5625f,-4.0000f,1.5625f,-3.8958f,1.5137f,-3.8958f,1.5137f,-3.7917f,1.4648f,-3.7917f,1.4648f,-3.6875f,1.4160f,-3.6875f,
	1.4160f,-3.5833f,1.3672f,-3.5833f,1.3672f,-3.4792f,1.3184f,-3.4792f,1.3184f,-3.3750f,1.2695f,-3.3750f,1.2695f,-3.2708f,1.2207f,-3.2708f,1.2207f,-3.1667f,1.1719f,-3.1667f,1.1719f,-3.0625f,1.1230f,-3.0625f,1.1230f,-2.9583f,1.0742f,-2.9583f,1.0742f,-2.8542f,
	1.0254f,-2.8542f,1.0254f,-2.7500f,0.9766f,-2.7500f,0.9766f,-2.6458f,0.9277f,-2.6458f,0.9277f,-2.5417f,0.8789f,-2.5417f,0.8789f,-2.4375f,0.8301f,-2.4375f,0.8301f,-2.3333f,0.7813f,-2.3333f,0.7813f,-2.2292f,0.7324f,-2.2292f,0.7324f,-2.1250f,0.6836f,-2.1250f,
	0.6836f,-2.0208f,0.6348f,-2.0208f,0.6348f,-1.9167f,0.5859f,-1.9167f,0.5859f,-1.8125f,0.5371f,-1.8125f,0.5371f,-1.7083f,0.4883f,-1.7083f,0.4883f,-1.6042f,0.4395f,-1.6042f,0.4395f,-1.5000f,0.3906f,-1.5000f,0.3906f,-1.3958f,0.3418f,-1.3958f,0.3418f,-1.2917f,
	0.2930f,-1.2917f,0.2930f,-1.1875f,0.2441f,-1.1875f,0.2441f,-1.0833f,0.1953f,-1.0833f,0.1953f,-0.9792f,0.1465f,-0.9792f,0.1465f,-0.8750f,0.0977f,-0.8750f,0.0977f,-0.7708f,0.0488f,-0.7708f,0.0488f,-0.6667f,0.0000f,-0.6667f,0.0000f,-0.5625f,1.5625f,-4.0000f,
	1.5137f,-4.0000f,1.5137f,-3.8958f,1.4648f,-3.8958f,1.4648f,-3.7917f,1.4160f,-3.7917f,1.4160f,-3.6875f,1.3672f,-3.6875f,1.3672f,-3.5833f,1.3184f,-3.5833f,1.3184f,-3.4792f,1.2695f,-3.4792f,1.2695f,-3.3750f,1.2207f,-3.3750f,1.2207f,-3.2708f,1.1719f,-3.2708f,
	1.1719f,-3.1667f,1.1230f,-3.1667f,1.1230f,-3.0625f,1.0742f,-3.0625f,1.0742f,-2.9583f,1.0254f,-2.9583f,1.0254f,-2.8542f,0.9766f,-2.8542f,0.9766f,-2.7500f,0.9277f,-2.7500f,0.9277f,-2.6458f,0.8789f,-2.6458f,0.8789f,-2.5417f,0.8301f,-2.5417f,0.8301f,-2.4375f,
	0.7813f,-2.4375f,0.7813f,-2.3333f,0.7324f,-2.3333f,0.7324f,-2.2292f,0.6836f,-2.2292f,0.6836f,-2.1250f,0.6348f,-2.1250f,0.6348f,-2.0208f,0.5859f,-2.0208f,0.5859f,-1.9167f,0.5371f,-1.9167f,0.5371f,-1.8125f,0.4883f,-1.8125f,0.4883f,-1.7083f,0.4395f,-1.7083f,
	0.4395f,-1.6042f,0.3906f,-1.6042f,0.3906f,-1.5000f,0.3418f,-1.5000f,0.3418f,-1.3958f,0.2930f,-1.3958f,0.2930f,-1.2917f,0.2441f,-1.2917f,0.2441f,-1.1875f,0.1953f,-1.1875f,0.1953f,-1.0833f,0.1465f,-1.0833f,0.1465f,-0.9792f,0.0977f,-0.9792f,0.0977f,-0.8750f,
	0.0488f,-0.8750f,0.0488f,-0.7708f,0.0000f,-0.7708f,0.0000f,-0.6667f,1.5137f,-4.0000f,1.4648f,-4.0000f,1.4648f,-3.8958f,1.4160f,-3.8958f,1.4160f,-3.7917f,1.3672f,-3.7917f,1.3672f,-3.6875f,1.3184f,-3.6875f,1.3184f,-3.5833f,1.2695f,-3.5833f,1.2695f,-3.4792f,
	1.2207f,-3.4792f,1.2207f,-3.3750f,1.1719f,-3.3750f,1.1719f,-3.2708f,1.1230f,-3.2708f,1.1230f,-3.1667f,1.0742f,-3.1667f,1.0742f,-3.0625f,1.0254f,-3.0625f,1.0254f,-2.9583f,0.9766f,-2.9583f,0.9766f,-2.8542f,0.9277f,-2.8542f,0.9277f,-2.7500f,0.8789f,-2.7500f,
	0.8789f,-2.6458f,0.8301f,-2.6458f,0.8301f,-2.5417f,0.7813f,-2.5417f,0.7813f,-2.4375f,0.7324f,-2.4375f,0.7324f,-2.3333f,0.6836f,-2.3333f,0.6836f,-2.2292f,0.6348f,-2.2292f,0.6348f,-2.1250f,0.5859f,-2.1250f,0.5859f,-2.0208f,0.5371f,-2.0208f,0.5371f,-1.9167f,
	0.4883f,-1.9167f,0.4883f,-1.8125f,0.4395f,-1.8125f,0.4395f,-1.7083f,0.3906f,-1.7083f,0.3906f,-1.6042f,0.3418f,-1.6042f,0.3418f,-1.5000f,0.2930f,-1.5000f,0.2930f,-1.3958f,0.2441f,-1.3958f,0.2441f,-1.2917f,0.1953f,-1.2917f,0.1953f,-1.1875f,0.1465f,-1.1875f,
	0.1465f,-1.0833f,0.0977f,-1.0833f,0.0977f,-0.9792f,0.0488f,-0.9792f,0.0488f,-0.8750f,0.0000f,-0.8750f,0.0000f,-0.7708f,1.4648f,-4.0000f,1.4160f,-4.0000f,1.4160f,-3.8958f,1.3672f,-3.8958f,1.3672f,-3.7917f,1.3184f,-3.7917f,1.3184f,-3.6875f,1.2695f,-3.6875f,
	1.2695f,-3.5833f,1.2207f,-3.5833f,1.2207f,-3.4792f,1.1719f,-3.4792f,1.1719f,-3.3750f,1.1230f,-3.3750f,1.1230f,-3.2708f,1.0742f,-3.2708f,1.0742f,-3.1667f,1.0254f,-3.1667f,1.0254f,-3.0625f,0.9766f,-3.0625f,0.9766f,-2.9583f,0.9277f,-2.9583f,0.9277f,-2.8542f,
	0.8789f,-2.8542f,0.8789f,-2.7500f,0.8301f,-2.7500f,0.8301f,-2.6458f,0.7813f,-2.6458f,0.7813f,-2.5417f,0.7324f,-2.5417f,0.7324f,-2.4375f,0.6836f,-2.4375f,0.6836f,-2.3333f,0.6348f,-2.3333f,0.6348f,-2.2292f,0.5859f,-2.2292f,0.5859f,-2.1250f,0.5371f,-2.1250f,
	0.5371f,-2.0208f,0.4883f,-2.0208f,0.4883f,-1.9167f,0.4395f,-1.9167f,0.4395f,-1.8125f,0.3906f,-1.8125f,0.3906f,-1.7083f,0.3418f,-1.7083f,0.3418f,-1.6042f,0.2930f,-1.6042f,0.2930f,-1.5000f,0.2441f,-1.5000f,0.2441f,-1.3958f,0.1953f,-1.3958f,0.1953f,-1.2917f,
	0.1465f,-1.2917f,0.1465f,-1.1875f,0.0977f,-1.1875f,0.0977f,-1.0833f,0.0488f,-1.0833f,0.0488f,-0.9792f,0.0000f,-0.9792f,0.0000f,-0.8750f,1.4160f,-4.0000f,1.3672f,-4.0000f,1.3672f,-3.8958f,1.3184f,-3.8958f,1.3184f,-3.7917f,1.2695f,-3.7917f,1.2695f,-3.6875f,
	1.2207f,-3.6875f,1.2207f,-3.5833f,1.1719f,-3.5833f,1.1719f,-3.4792f,1.1230f,-3.4792f,1.1230f,-3.3750f,1.0742f,-3.3750f,1.0742f,-3.2708f,1.0254f,-3.2708f,1.0254f,-3.1667f,0.9766f,-3.1667f,0.9766f,-3.0625f,0.9277f,-3.0625f,0.9277f,-2.9583f,0.8789f,-2.9583f,
	0.8789f,-2.8542f,0.8301f,-2.8542f,0.8301f,-2.7500f,0.7813f,-2.7500f,0.7813f,-2.6458f,0.7324f,-2.6458f,0.7324f,-2.5417f,0.6836f,-2.5417f,0.6836f,-2.4375f,0.6348f,-2.4375f,0.6348f,-2.3333f,0.5859f,-2.3333f,0.5859f,-2.2292f,0.5371f,-2.2292f,0.5371f,-2.1250f,
	0.4883f,-2.1250f,0.4883f,-2.0208f,0.4395f,-2.0208f,0.4395f,-1.9167f,0.3906f,-1.9167f,0.3906f,-1.8125f,0.3418f,-1.8125f,0.3418f,-1.7083f,0.2930f,-1.7083f,0.2930f,-1.6042f,0.2441f,-1.6042f,0.2441f,-1.5000f,0.1953f,-1.5000f,0.1953f,-1.3958f,0.1465f,-1.3958f,
	0.1465f,-1.2917f,0.0977f,-1.2917f,0.0977f,-1.1875f,0.0488f,-1.1875f,0.0488f,-1.0833f,0.0000f,-1.0833f,0.0000f,-0.9792f,1.3672f,-4.0000f,1.3184f,-4.0000f,1.3184f,-3.8958f,1.2695f,-3.8958f,1.2695f,-3.7917f,1.2207f,-3.7917f,1.2207f,-3.6875f,1.1719f,-3.6875f,
	1.1719f,-3.5833f,1.1230f,-3.5833f,1.1230f,-3.4792f,1.0742f,-3.4792f,1.0742f,-3.3750f,1.0254f,-3.3750f,1.0254f,-3.2708f,0.9766f,-3.2708f,0.9766f,-3.1667f,0.9277f,-3.1667f,0.9277f,-3.0625f,0.8789f,-3.0625f,0.8789f,-2.9583f,0.8301f,-2.9583f,0.8301f,-2.8542f,
	0.7813f,-2.8542f,0.7813f,-2.7500f,0.7324f,-2.7500f,0.7324f,-2.6458f,0.6836f,-2.6458f,0.6836f,-2.5417f,0.6348f,-2.5417f,0.6348f,-2.4375f,0.5859f,-2.4375f,0.5859f,-2.3333f,0.5371f,-2.3333f,0.5371f,-2.2292f,0.4883f,-2.2292f,0.4883f,-2.1250f,0.4395f,-2.1250f,
	0.4395f,-2.0208f,0.3906f,-2.0208f,0.3906f,-1.9167f,0.3418f,-1.9167f,0.3418f,-1.8125f,0.2930f,-1.8125f,0.2930f,-1.7083f,0.2441f,-1.7083f,0.2441f,-1.6042f,0.1953f,-1.6042f,0.1953f,-1.5000f,0.1465f,-1.5000f,0.1465f,-1.3958f,0.0977f,-1.3958f,0.0977f,-1.2917f,
	0.0488f,-1.2917f,0.0488f,-1.1875f,0.0000f,-1.1875f,0.0000f,-1.0833f,1.3184f,-4.0000f,1.2695f,-4.0000f,1.2695f,-3.8958f,1.2207f,-3.8958f,1.2207f,-3.7917f,1.1719f,-3.7917f,1.1719f,-3.6875f,1.1230f,-3.6875f,1.1230f,-3.5833f,1.0742f,-3.5833f,1.0742f,-3.4792f,
	1.0254f,-3.4792f,1.0254f,-3.3750f,0.9766f,-3.3750f,0.9766f,-3.2708f,0.9277f,-3.2708f,0.9277f,-3.1667f,0.8789f,-3.1667f,0.8789f,-3.0625f,0.8301f,-3.0625f,0.8301f,-2.9583f,0.7813f,-2.9583f,0.7813f,-2.8542f,0.7324f,-2.8542f,0.7324f,-2.7500f,0.6836f,-2.7500f,
	0.6836f,-2.6458f,0.6348f,-2.6458f,0.6348f,-2.5417f,0.5859f,-2.5417f,0.5859f,-2.4375f,0.5371f,-2.4375f,0.5371f,-2.3333f,0.4883f,-2.3333f,0.4883f,-2.2292f,0.4395f,-2.2292f,0.4395f,-2.1250f,0.3906f,-2.1250f,0.3906f,-2.0208f,0.3418f,-2.0208f,0.3418f,-1.9167f,
	0.2930f,-1.9167f,0.2930f,-1.8125f,0.2441f,-1.8125f,0.2441f,-1.7083f,0.1953f,-1.7083f,0.1953f,-1.6042f,0.1465f,-1.6042f,0.1465f,-1.5000f,0.0977f,-1.5000f,0.0977f,-1.3958f,0.0488f,-1.3958f,0.0488f,-1.2917f,0.0000f,-1.2917f,0.0000f,-1.1875f,1.2695f,-4.0000f,
	1.2207f,-4.0000f,1.2207f,-3.8958f,1.1719f,-3.8958f,1.1719f,-3.7917f,1.1230f,-3.7917f,1.1230f,-3.6875f,1.0742f,-3.6875f,1.0742f,-3.5833f,1.0254f,-3.5833f,1.0254f,-3.4792f,0.9766f,-3.4792f,0.9766f,-3.3750f,0.9277f,-3.3750f,0.9277f,-3.2708f,0.8789f,-3.2708f,
	0.8789f,-3.1667f,0.8301f,-3.1667f,0.8301f,-3.0625f,0.7813f,-3.0625f,0.7813f,-2.9583f,0.7324f,-2.9583f,0.7324f,-2.8542f,0.6836f,-2.8542f,0.6836f,-2.7500f,0.6348f,-2.7500f,0.6348f,-2.6458f,0.5859f,-2.6458f,0.5859f,-2.5417f,0.5371f,-2.5417f,0.5371f,-2.4375f,
	0.4883f,-2.4375f,0.4883f,-2.3333f,0.4395f,-2.3333f,0.4395f,-2.2292f,0.3906f,-2.2292f,0.3906f,-2.1250f,0.3418f,-2.1250f,0.3418f,-2.0208f,0.2930f,-2.0208f,0.2930f,-1.9167f,0.2441f,-1.9167f,0.2441f,-1.8125f,0.1953f,-1.8125f,0.1953f,-1.7083f,0.1465f,-1.7083f,
	0.1465f,-1.6042f,0.0977f,-1.6042f,0.0977f,-1.5000f,0.0488f,-1.5000f,0.0488f,-1.3958f,0.0000f,-1.3958f,0.0000f,-1.2917f,1.2207f,-4.0000f,1.1719f,-4.0000f,1.1719f,-3.8958f,1.1230f,-3.8958f,1.1230f,-3.7917f,1.0742f,-3.7917f,1.0742f,-3.6875f,1.0254f,-3.6875f,
	1.0254f,-3.5833f,0.9766f,-3.5833f,0.9766f,-3.4792f,0.9277f,-3.4792f,0.9277f,-3.3750f,0.8789f,-3.3750f,0.8789f,-3.2708f,0.8301f,-3.2708f,0.8301f,-3.1667f,0.7813f,-3.1667f,0.7813f,-3.0625f,0.7324f,-3.0625f,0.7324f,-2.9583f,0.6836f,-2.9583f,0.6836f,-2.8542f,
	0.6348f,-2.8542f,0.6348f,-2.7500f,0.5859f,-2.7500f,0.5859f,-2.6458f,0.5371f,-2.6458f,0.5371f,-2.5417f,0.4883f,-2.5417f,0.4883f,-2.4375f,0.4395f,-2.4375f,0.4395f,-2.3333f,0.3906f,-2.3333f,0.3906f,-2.2292f,0.3418f,-2.2292f,0.3418f,-2.1250f,0.2930f,-2.1250f,
	0.2930f,-2.0208f,0.2441f,-2.0208f,0.2441f,-1.9167f,0.1953f,-1.9167f,0.1953f,-1.8125f,0.1465f,-1.8125f,0.1465f,-1.7083f,0.0977f,-1.7083f,0.0977f,-1.6042f,0.0488f,-1.6042f,0.0488f,-1.5000f,0.0000f,-1.5000f,0.0000f,-1.3958f,1.1719f,-4.0000f,1.1230f,-4.0000f,
	1.1230f,-3.8958f,1.0742f,-3.8958f,1.0742f,-3.7917f,1.0254f,-3.7917f,1.0254f,-3.6875f,0.9766f,-3.6875f,0.9766f,-3.5833f,0.9277f,-3.5833f,0.9277f,-3.4792f,0.8789f,-3.4792f,0.8789f,-3.3750f,0.8301f,-3.3750f,0.8301f,-3.2708f,0.7813f,-3.2708f,0.7813f,-3.1667f,
	0.7324f,-3.1667f,0.7324f,-3.0625f,0.6836f,-3.0625f,0.6836f,-2.9583f,0.6348f,-2.9583f,0.6348f,-2.8542f,0.5859f,-2.8542f,0.5859f,-2.7500f,0.5371f,-2.7500f,0.5371f,-2.6458f,0.4883f,-2.6458f,0.4883f,-2.5417f,0.4395f,-2.5417f,0.4395f,-2.4375f,0.3906f,-2.4375f,
	0.3906f,-2.3333f,0.3418f,-2.3333f,0.3418f,-2.2292f,0.2930f,-2.2292f,0.2930f,-2.1250f,0.2441f,-2.1250f,0.2441f,-2.0208f,0.1953f,-2.0208f,0.1953f,-1.9167f,0.1465f,-1.9167f,0.1465f,-1.8125f,0.0977f,-1.8125f,0.0977f,-1.7083f,0.0488f,-1.7083f,0.0488f,-1.6042f,
	0.0000f,-1.6042f,0.0000f,-1.5000f,1.1230f,-4.0000f,1.0742f,-4.0000f,1.0742f,-3.8958f,1.0254f,-3.8958f,1.0254f,-3.7917f,0.9766f,-3.7917f,0.9766f,-3.6875f,0.9277f,-3.6875f,0.9277f,-3.5833f,0.8789f,-3.5833f,0.8789f,-3.4792f,0.8301f,-3.4792f,0.8301f,-3.3750f,
	0.7813f,-3.3750f,0.7813f,-3.2708f,0.7324f,-3.2708f,0.7324f,-3.1667f,0.6836f,-3.1667f,0.6836f,-3.0625f,0.6348f,-3.0625f,0.6348f,-2.9583f,0.5859f,-2.9583f,0.5859f,-2.8542f,0.5371f,-2.8542f,0.5371f,-2.7500f,0.4883f,-2.7500f,0.4883f,-2.6458f,0.4395f,-2.6458f,
	0.4395f,-2.5417f,0.3906f,-2.5417f,0.3906f,-2.4375f,0.3418f,-2.4375f,0.3418f,-2.3333f,0.2930f,-2.3333f,0.2930f,-2.2292f,0.2441f,-2.2292f,0.2441f,-2.1250f,0.1953f,-2.1250f,0.1953f,-2.0208f,0.1465f,-2.0208f,0.1465f,-1.9167f,0.0977f,-1.9167f,0.0977f,-1.8125f,
	0.0488f,-1.8125f,0.0488f,-1.7083f,0.0000f,-1.7083f,0.0000f,-1.6042f,1.0742f,-4.0000f,1.0254f,-4.0000f,1.0254f,-3.8958f,0.9766f,-3.8958f,0.9766f,-3.7917f,0.9277f,-3.7917f,0.9277f,-3.6875f,0.8789f,-3.6875f,0.8789f,-3.5833f,0.8301f,-3.5833f,0.8301f,-3.4792f,
	0.7813f,-3.4792f,0.7813f,-3.3750f,0.7324f,-3.3750f,0.7324f,-3.2708f,0.6836f,-3.2708f,0.6836f,-3.1667f,0.6348f,-3.1667f,0.6348f,-3.0625f,0.5859f,-3.0625f,0.5859f,-2.9583f,0.5371f,-2.9583f,0.5371f,-2.8542f,0.4883f,-2.8542f,0.4883f,-2.7500f,0.4395f,-2.7500f,
	0.4395f,-2.6458f,0.3906f,-2.6458f,0.3906f,-2.5417f,0.3418f,-2.5417f,0.3418f,-2.4375f,0.2930f,-2.4375f,0.2930f,-2.3333f,0.2441f,-2.3333f,0.2441f,-2.2292f,0.1953f,-2.2292f,0.1953f,-2.1250f,0.1465f,-2.1250f,0.1465f,-2.0208f,0.0977f,-2.0208f,0.0977f,-1.9167f,
	0.0488f,-1.9167f,0.0488f,-1.8125f,0.0000f,-1.8125f,0.0000f,-1.7083f,1.0254f,-4.0000f,0.9766f,-4.0000f,0.9766f,-3.8958f,0.9277f,-3.8958f,0.9277f,-3.7917f,0.8789f,-3.7917f,0.8789f,-3.6875f,0.8301f,-3.6875f,0.8301f,-3.5833f,0.7813f,-3.5833f,0.7813f,-3.4792f,
	0.7324f,-3.4792f,0.7324f,-3.3750f,0.6836f,-3.3750f,0.6836f,-3.2708f,0.6348f,-3.2708f,0.6348f,-3.1667f,0.5859f,-3.1667f,0.5859f,-3.0625f,0.5371f,-3.0625f,0.5371f,-2.9583f,0.4883f,-2.9583f,0.4883f,-2.8542f,0.4395f,-2.8542f,0.4395f,-2.7500f,0.3906f,-2.7500f,
	0.3906f,-2.6458f,0.3418f,-2.6458f,0.3418f,-2.5417f,0.2930f,-2.5417f,0.2930f,-2.4375f,0.2441f,-2.4375f,0.2441f,-2.3333f,0.1953f,-2.3333f,0.1953f,-2.2292f,0.1465f,-2.2292f,0.1465f,-2.1250f,0.0977f,-2.1250f,0.0977f,-2.0208f,0.0488f,-2.0208f,0.0488f,-1.9167f,
	0.0000f,-1.9167f,0.0000f,-1.8125f,0.9766f,-4.0000f,0.9277f,-4.0000f,0.9277f,-3.8958f,0.8789f,-3.8958f,0.8789f,-3.7917f,0.8301f,-3.7917f,0.8301f,-3.6875f,0.7813f,-3.6875f,0.7813f,-3.5833f,0.7324f,-3.5833f,0.7324f,-3.4792f,0.6836f,-3.4792f,0.6836f,-3.3750f,
	0.6348f,-3.3750f,0.6348f,-3.2708f,0.5859f,-3.2708f,0.5859f,-3.1667f,0.5371f,-3.1667f,0.5371f,-3.0625f,0.4883f,-3.0625f,0.4883f,-2.9583f,0.4395f,-2.9583f,0.4395f,-2.8542f,0.3906f,-2.8542f,0.3906f,-2.7500f,0.3418f,-2.7500f,0.3418f,-2.6458f,0.2930f,-2.6458f,
	0.2930f,-2.5417f,0.2441f,-2.5417f,0.2441f,-2.4375f,0.1953f,-2.4375f,0.1953f,-2.3333f,0.1465f,-2.3333f,0.1465f,-2.2292f,0.0977f,-2.2292f,0.0977f,-2.1250f,0.0488f,-2.1250f,0.0488f,-2.0208f,0.0000f,-2.0208f,0.0000f,-1.9167f,0.9277f,-4.0000f,0.8789f,-4.0000f,
	0.8789f,-3.8958f,0.8301f,-3.8958f,0.8301f,-3.7917f,0.7813f,-3.7917f,0.7813f,-3.6875f,0.7324f,-3.6875f,0.7324f,-3.5833f,0.6836f,-3.5833f,0.6836f,-3.4792f,0.6348f,-3.4792f,0.6348f,-3.3750f,0.5859f,-3.3750f,0.5859f,-3.2708f,0.5371f,-3.2708f,0.5371f,-3.1667f,
	0.4883f,-3.1667f,0.4883f,-3.0625f,0.4395f,-3.0625f,0.4395f,-2.9583f,0.3906f,-2.9583f,0.3906f,-2.8542f,0.3418f,-2.8542f,0.3418f,-2.7500f,0.2930f,-2.7500f,0.2930f,-2.6458f,0.2441f,-2.6458f,0.2441f,-2.5417f,0.1953f,-2.5417f,0.1953f,-2.4375f,0.1465f,-2.4375f,
	0.1465f,-2.3333f,0.0977f,-2.3333f,0.0977f,-2.2292f,0.0488f,-2.2292f,0.0488f,-2.1250f,0.0000f,-2.1250f,0.0000f,-2.0208f,0.8789f,-4.0000f,0.8301f,-4.0000f,0.8301f,-3.8958f,0.7813f,-3.8958f,0.7813f,-3.7917f,0.7324f,-3.7917f,0.7324f,-3.6875f,0.6836f,-3.6875f,
	0.6836f,-3.5833f,0.6348f,-3.5833f,0.6348f,-3.4792f,0.5859f,-3.4792f,0.5859f,-3.3750f,0.5371f,-3.3750f,0.5371f,-3.2708f,0.4883f,-3.2708f,0.4883f,-3.1667f,0.4395f,-3.1667f,0.4395f,-3.0625f,0.3906f,-3.0625f,0.3906f,-2.9583f,0.3418f,-2.9583f,0.3418f,-2.8542f,
	0.2930f,-2.8542f,0.2930f,-2.7500f,0.2441f,-2.7500f,0.2441f,-2.6458f,0.1953f,-2.6458f,0.1953f,-2.5417f,0.1465f,-2.5417f,0.1465f,-2.4375f,0.0977f,-2.4375f,0.0977f,-2.3333f,0.0488f,-2.3333f,0.0488f,-2.2292f,0.0000f,-2.2292f,0.0000f,-2.1250f,0.8301f,-4.0000f,
	0.7813f,-4.0000f,0.7813f,-3.8958f,0.7324f,-3.8958f,0.7324f,-3.7917f,0.6836f,-3.7917f,0.6836f,-3.6875f,0.6348f,-3.6875f,0.6348f,-3.5833f,0.5859f,-3.5833f,0.5859f,-3.4792f,0.5371f,-3.4792f,0.5371f,-3.3750f,0.4883f,-3.3750f,0.4883f,-3.2708f,0.4395f,-3.2708f,
	0.4395f,-3.1667f,0.3906f,-3.1667f,0.3906f,-3.0625f,0.3418f,-3.0625f,0.3418f,-2.9583f,0.2930f,-2.9583f,0.2930f,-2.8542f,0.2441f,-2.8542f,0.2441f,-2.7500f,0.1953f,-2.7500f,0.1953f,-2.6458f,0.1465f,-2.6458f,0.1465f,-2.5417f,0.0977f,-2.5417f,0.0977f,-2.4375f,
	0.0488f,-2.4375f,0.0488f,-2.3333f,0.0000f,-2.3333f,0.0000f,-2.2292f,0.7813f,-4.0000f,0.7324f,-4.0000f,0.7324f,-3.8958f,0.6836f,-3.8958f,0.6836f,-3.7917f,0.6348f,-3.7917f,0.6348f,-3.6875f,0.5859f,-3.6875f,0.5859f,-3.5833f,0.5371f,-3.5833f,0.5371f,-3.4792f,
	0.4883f,-3.4792f,0.4883f,-3.3750f,0.4395f,-3.3750f,0.4395f,-3.2708f,0.3906f,-3.2708f,0.3906f,-3.1667f,0.3418f,-3.1667f,0.3418f,-3.0625f,0.2930f,-3.0625f,0.2930f,-2.9583f,0.2441f,-2.9583f,0.2441f,-2.8542f,0.1953f,-2.8542f,0.1953f,-2.7500f,0.1465f,-2.7500f,
	0.1465f,-2.6458f,0.0977f,-2.6458f,0.0977f,-2.5417f,0.0488f,-2.5417f,0.0488f,-2.4375f,0.0000f,-2.4375f,0.0000f,-2.3333f,0.7324f,-4.0000f,0.6836f,-4.0000f,0.6836f,-3.8958f,0.6348f,-3.8958f,0.6348f,-3.7917f,0.5859f,-3.7917f,0.5859f,-3.6875f,0.5371f,-3.6875f,
	0.5371f,-3.5833f,0.4883f,-3.5833f,0.4883f,-3.4792f,0.4395f,-3.4792f,0.4395f,-3.3750f,0.3906f,-3.3750f,0.3906f,-3.2708f,0.3418f,-3.2708f,0.3418f,-3.1667f,0.2930f,-3.1667f,0.2930f,-3.0625f,0.2441f,-3.0625f,0.2441f,-2.9583f,0.1953f,-2.9583f,0.1953f,-2.8542f,
	0.1465f,-2.8542f,0.1465f,-2.7500f,0.0977f,-2.7500f,0.0977f,-2.6458f,0.0488f,-2.6458f,0.0488f,-2.5417f,0.0000f,-2.5417f,0.0000f,-2.4375f,0.6836f,-4.0000f,0.6348f,-4.0000f,0.6348f,-3.8958f,0.5859f,-3.8958f,0.5859f,-3.7917f,0.5371f,-3.7917f,0.5371f,-3.6875f,
	0.4883f,-3.6875f,0.4883f,-3.5833f,0.4395f,-3.5833f,0.4395f,-3.4792f,0.3906f,-3.4792f,0.3906f,-3.3750f,0.3418f,-3.3750f,0.3418f,-3.2708f,0.2930f,-3.2708f,0.2930f,-3.1667f,0.2441f,-3.1667f,0.2441f,-3.0625f,0.1953f,-3.0625f,0.1953f,-2.9583f,0.1465f,-2.9583f,
	0.1465f,-2.8542f,0.0977f,-2.8542f,0.0977f,-2.7500f,0.0488f,-2.7500f,0.0488f,-2.6458f,0.0000f,-2.6458f,0.0000f,-2.5417f,0.6348f,-4.0000f,0.5859f,-4.0000f,0.5859f,-3.8958f,0.5371f,-3.8958f,0.5371f,-3.7917f,0.4883f,-3.7917f,0.4883f,-3.6875f,0.4395f,-3.6875f,
	0.4395f,-3.5833f,0.3906f,-3.5833f,0.3906f,-3.4792f,0.3418f,-3.4792f,0.3418f,-3.3750f,0.2930f,-3.3750f,0.2930f,-3.2708f,0.2441f,-3.2708f,0.2441f,-3.1667f,0.1953f,-3.1667f,0.1953f,-3.0625f,0.1465f,-3.0625f,0.1465f,-2.9583f,0.0977f,-2.9583f,0.0977f,-2.8542f,
	0.0488f,-2.8542f,0.0488f,-2.7500f,0.0000f,-2.7500f,0.0000f,-2.6458f,0.5859f,-4.0000f,0.5371f,-4.0000f,0.5371f,-3.8958f,0.4883f,-3.8958f,0.4883f,-3.7917f,0.4395f,-3.7917f,0.4395f,-3.6875f,0.3906f,-3.6875f,0.3906f,-3.5833f,0.3418f,-3.5833f,0.3418f,-3.4792f,
	0.2930f,-3.4792f,0.2930f,-3.3750f,0.2441f,-3.3750f,0.2441f,-3.2708f,0.1953f,-3.2708f,0.1953f,-3.1667f,0.1465f,-3.1667f,0.1465f,-3.0625f,0.0977f,-3.0625f,0.0977f,-2.9583f,0.0488f,-2.9583f,0.0488f,-2.8542f,0.0000f,-2.8542f,0.0000f,-2.7500f,0.5371f,-4.0000f,
	0.4883f,-4.0000f,0.4883f,-3.8958f,0.4395f,-3.8958f,0.4395f,-3.7917f,0.3906f,-3.7917f,0.3906f,-3.6875f,0.3418f,-3.6875f,0.3418f,-3.5833f,0.2930f,-3.5833f,0.2930f,-3.4792f,0.2441f,-3.4792f,0.2441f,-3.3750f,0.1953f,-3.3750f,0.1953f,-3.2708f,0.1465f,-3.2708f,
	0.1465f,-3.1667f,0.0977f,-3.1667f,0.0977f,-3.0625f,0.0488f,-3.0625f,0.0488f,-2.9583f,0.0000f,-2.9583f,0.0000f,-2.8542f,0.4883f,-4.0000f,0.4395f,-4.0000f,0.4395f,-3.8958f,0.3906f,-3.8958f,0.3906f,-3.7917f,0.3418f,-3.7917f,0.3418f,-3.6875f,0.2930f,-3.6875f,
	0.2930f,-3.5833f,0.2441f,-3.5833f,0.2441f,-3.4792f,0.1953f,-3.4792f,0.1953f,-3.3750f,0.1465f,-3.3750f,0.1465f,-3.2708f,0.0977f,-3.2708f,0.0977f,-3.1667f,0.0488f,-3.1667f,0.0488f,-3.0625f,0.0000f,-3.0625f,0.0000f,-2.9583f,0.4395f,-4.0000f,0.3906f,-4.0000f,
	0.3906f,-3.8958f,0.3418f,-3.8958f,0.3418f,-3.7917f,0.2930f,-3.7917f,0.2930f,-3.6875f,0.2441f,-3.6875f,0.2441f,-3.5833f,0.1953f,-3.5833f,0.1953f,-3.4792f,0.1465f,-3.4792f,0.1465f,-3.3750f,0.0977f,-3.3750f,0.0977f,-3.2708f,0.0488f,-3.2708f,0.0488f,-3.1667f,
	0.0000f,-3.1667f,0.0000f,-3.0625f,0.3906f,-4.0000f,0.3418f,-4.0000f,0.3418f,-3.8958f,0.2930f,-3.8958f,0.2930f,-3.7917f,0.2441f,-3.7917f,0.2441f,-3.6875f,0.1953f,-3.6875f,0.1953f,-3.5833f,0.1465f,-3.5833f,0.1465f,-3.4792f,0.0977f,-3.4792f,0.0977f,-3.3750f,
	0.0488f,-3.3750f,0.0488f,-3.2708f,0.0000f,-3.2708f,0.0000f,-3.1667f,0.3418f,-4.0000f,0.2930f,-4.0000f,0.2930f,-3.8958f,0.2441f,-3.8958f,0.2441f,-3.7917f,0.1953f,-3.7917f,0.1953f,-3.6875f,0.1465f,-3.6875f,0.1465f,-3.5833f,0.0977f,-3.5833f,0.0977f,-3.4792f,
	0.0488f,-3.4792f,0.0488f,-3.3750f,0.0000f,-3.3750f,0.0000f,-3.2708f,0.2930f,-4.0000f,0.2441f,-4.0000f,0.2441f,-3.8958f,0.1953f,-3.8958f,0.1953f,-3.7917f,0.1465f,-3.7917f,0.1465f,-3.6875f,0.0977f,-3.6875f,0.0977f,-3.5833f,0.0488f,-3.5833f,0.0488f,-3.4792f,
	0.0000f,-3.4792f,0.0000f,-3.3750f,0.2441f,-4.0000f,0.1953f,-4.0000f,0.1953f,-3.8958f,0.1465f,-3.8958f,0.1465f,-3.7917f,0.0977f,-3.7917f,0.0977f,-3.6875f,0.0488f,-3.6875f,0.0488f,-3.5833f,0.0000f,-3.5833f,0.0000f,-3.4792f,0.1953f,-4.0000f,0.1465f,-4.0000f,
	0.1465f,-3.8958f,0.0977f,-3.8958f,0.0977f,-3.7917f,0.0488f,-3.7917f,0.0488f,-3.6875f,0.0000f,-3.6875f,0.0000f,-3.5833f,0.1465f,-4.0000f,0.0977f,-4.0000f,0.0977f,-3.8958f,0.0488f,-3.8958f,0.0488f,-3.7917f,0.0000f,-3.7917f,0.0000f,-3.6875f,0.0977f,-4.0000f,
	0.0488f,-4.0000f,0.0488f,-3.8958f,0.0000f,-3.8958f,0.0000f,-3.7917f,0.0000f,-4.0000f,0.0000f,-3.8958f,0.0488f,-4.0000f,
_NULL_ }; /* End of torus_knot01_UV */

float torus_knot01_Normals[] = {
	0.7076f,-0.7066f,-0.0009f,  0.7075f,-0.7064f,0.0207f,  0.6356f,-0.7636f,0.1138f,  0.6358f,-0.7595f,0.1376f,  0.5532f,-0.8005f,0.2305f,  0.5535f,-0.7928f,0.2554f,  0.4616f,-0.8171f,0.3454f,  0.4618f,-0.8060f,0.3703f,  0.3622f,-0.8135f,0.4549f,  0.3622f,-0.7995f,0.4792f,  0.2564f,-0.7908f,0.5558f,  0.2561f,-0.7744f,0.5786f,  0.1460f,-0.7501f,0.6450f,  0.1453f,-0.7318f,0.6658f,  0.0327f,-0.6931f,0.7201f,  
	0.0317f,-0.6737f,0.7383f,  -0.0815f,-0.6220f,0.7787f,  -0.0827f,-0.6019f,0.7943f,  -0.1946f,-0.5389f,0.8196f,  -0.1959f,-0.5189f,0.8321f,  -0.3045f,-0.4464f,0.8414f,  -0.3061f,-0.4269f,0.8509f,  -0.4094f,-0.3469f,0.8438f,  -0.4110f,-0.3284f,0.8504f,  -0.5073f,-0.2431f,0.8268f,  -0.5091f,-0.2259f,0.8306f,  -0.5966f,-0.1373f,0.7907f,  -0.5983f,-0.1220f,0.7920f,  -0.6756f,-0.0321f,0.7366f,  -0.6771f,-0.0187f,0.7356f,  
	-0.7428f,0.0704f,0.6658f,  -0.7442f,0.0815f,0.6629f,  -0.7971f,0.1678f,0.5801f,  -0.7984f,0.1767f,0.5756f,  -0.8375f,0.2586f,0.4814f,  -0.8387f,0.2653f,0.4757f,  -0.8633f,0.3411f,0.3720f,  -0.8643f,0.3455f,0.3654f,  -0.8741f,0.4138f,0.2543f,  -0.8751f,0.4162f,0.2472f,  -0.8698f,0.4758f,0.1308f,  -0.8706f,0.4762f,0.1234f,  -0.8504f,0.5261f,0.0042f,  -0.8512f,0.5249f,-0.0032f,  -0.8163f,0.5644f,-0.1229f,  
	-0.8170f,0.5617f,-0.1302f,  -0.7682f,0.5902f,-0.2481f,  -0.7689f,0.5863f,-0.2550f,  -0.7069f,0.6035f,-0.3689f,  -0.7076f,0.5986f,-0.3753f,  -0.6335f,0.6045f,-0.4829f,  -0.6343f,0.5990f,-0.4888f,  -0.5493f,0.5937f,-0.5880f,  -0.5502f,0.5876f,-0.5933f,  -0.4558f,0.5715f,-0.6824f,  -0.4568f,0.5651f,-0.6870f,  -0.3546f,0.5387f,-0.7642f,  -0.3557f,0.5322f,-0.7683f,  -0.2475f,0.4962f,-0.8322f,  -0.2487f,0.4898f,-0.8356f,  
	-0.1361f,0.4451f,-0.8851f,  -0.1375f,0.4388f,-0.8880f,  -0.0226f,0.3864f,-0.9221f,  -0.0240f,0.3804f,-0.9245f,  0.0912f,0.3213f,-0.9426f,  0.0897f,0.3158f,-0.9446f,  0.2035f,0.2512f,-0.9463f,  0.2019f,0.2461f,-0.9480f,  0.3122f,0.1774f,-0.9333f,  0.3106f,0.1727f,-0.9347f,  0.4157f,0.1011f,-0.9039f,  0.4140f,0.0968f,-0.9051f,  0.5121f,0.0238f,-0.8586f,  0.5104f,0.0199f,-0.8597f,  0.5998f,-0.0533f,-0.7984f,  
	0.5982f,-0.0568f,-0.7993f,  0.6775f,-0.1287f,-0.7242f,  0.6759f,-0.1319f,-0.7251f,  0.7437f,-0.2014f,-0.6375f,  0.7422f,-0.2043f,-0.6383f,  0.7974f,-0.2700f,-0.5397f,  0.7959f,-0.2727f,-0.5405f,  0.8377f,-0.3335f,-0.4325f,  0.8363f,-0.3359f,-0.4333f,  0.8639f,-0.3907f,-0.3178f,  0.8626f,-0.3931f,-0.3185f,  0.8756f,-0.4408f,-0.1975f,  0.8743f,-0.4430f,-0.1982f,  0.8726f,-0.4829f,-0.0736f,  0.8713f,-0.4851f,-0.0743f,  
	0.8549f,-0.5163f,0.0517f,  0.8536f,-0.5184f,0.0512f,  0.8228f,-0.5403f,0.1763f,  0.8215f,-0.5423f,0.1759f,  0.7768f,-0.5547f,0.2982f,  0.7756f,-0.5565f,0.2979f,  0.7178f,-0.5589f,0.4152f,  0.7165f,-0.5606f,0.4151f,  0.7076f,-0.7066f,-0.0009f,  0.6356f,-0.7636f,0.1138f,  0.6352f,-0.7671f,0.0898f,  0.5527f,-0.8078f,0.2050f,  0.5519f,-0.8145f,0.1789f,  0.4604f,-0.8381f,0.2926f,  0.4591f,-0.8479f,0.2650f,  
	0.3601f,-0.8541f,0.3753f,  0.3585f,-0.8668f,0.3466f,  0.2537f,-0.8554f,0.4515f,  0.2517f,-0.8708f,0.4223f,  0.1429f,-0.8421f,0.5200f,  0.1406f,-0.8599f,0.4907f,  0.0297f,-0.8144f,0.5796f,  0.0271f,-0.8343f,0.5507f,  -0.0840f,-0.7726f,0.6293f,  -0.0868f,-0.7943f,0.6012f,  -0.1963f,-0.7177f,0.6681f,  -0.1993f,-0.7408f,0.6415f,  -0.3052f,-0.6505f,0.6955f,  -0.3084f,-0.6746f,0.6707f,  -0.4090f,-0.5721f,0.7109f,  
	-0.4122f,-0.5968f,0.6884f,  -0.5057f,-0.4840f,0.7142f,  -0.5090f,-0.5088f,0.6943f,  -0.5937f,-0.3875f,0.7052f,  -0.5970f,-0.4121f,0.6883f,  -0.6715f,-0.2845f,0.6842f,  -0.6747f,-0.3084f,0.6705f,  -0.7379f,-0.1766f,0.6514f,  -0.7409f,-0.1994f,0.6413f,  -0.7916f,-0.0656f,0.6075f,  -0.7945f,-0.0870f,0.6011f,  -0.8317f,0.0465f,0.5532f,  -0.8344f,0.0269f,0.5506f,  -0.8576f,0.1577f,0.4895f,  -0.8600f,0.1404f,0.4906f,  
	-0.8688f,0.2663f,0.4174f,  -0.8709f,0.2514f,0.4224f,  -0.8652f,0.3703f,0.3381f,  -0.8668f,0.3582f,0.3469f,  -0.8467f,0.4680f,0.2531f,  -0.8480f,0.4588f,0.2655f,  -0.8138f,0.5576f,0.1638f,  -0.8146f,0.5515f,0.1795f,  -0.7669f,0.6378f,0.0717f,  -0.7673f,0.6349f,0.0905f,  -0.7069f,0.7070f,-0.0216f,  -0.7069f,0.7073f,-0.0000f,  -0.6348f,0.7641f,-0.1146f,  -0.6344f,0.7677f,-0.0905f,  -0.5519f,0.8082f,-0.2055f,  
	-0.5510f,0.8150f,-0.1795f,  -0.4596f,0.8384f,-0.2930f,  -0.4583f,0.8483f,-0.2654f,  -0.3594f,0.8543f,-0.3754f,  -0.3577f,0.8671f,-0.3467f,  -0.2530f,0.8557f,-0.4515f,  -0.2510f,0.8711f,-0.4222f,  -0.1424f,0.8424f,-0.5198f,  -0.1400f,0.8602f,-0.4904f,  -0.0293f,0.8147f,-0.5792f,  -0.0266f,0.8346f,-0.5502f,  0.0843f,0.7730f,-0.6287f,  0.0872f,0.7948f,-0.6006f,  0.1964f,0.7182f,-0.6675f,  0.1995f,0.7413f,-0.6408f,  
	0.3052f,0.6511f,-0.6949f,  0.3084f,0.6752f,-0.6700f,  0.4088f,0.5729f,-0.7105f,  0.4121f,0.5976f,-0.6878f,  0.5054f,0.4848f,-0.7138f,  0.5087f,0.5098f,-0.6938f,  0.5933f,0.3885f,-0.7050f,  0.5966f,0.4132f,-0.6880f,  0.6711f,0.2855f,-0.6842f,  0.6743f,0.3095f,-0.6704f,  0.7375f,0.1777f,-0.6516f,  0.7405f,0.2006f,-0.6414f,  0.7912f,0.0668f,-0.6079f,  0.7941f,0.0882f,-0.6014f,  0.8314f,-0.0453f,-0.5538f,  
	0.8341f,-0.0256f,-0.5511f,  0.8574f,-0.1565f,-0.4902f,  0.8598f,-0.1391f,-0.4914f,  0.8688f,-0.2651f,-0.4183f,  0.8708f,-0.2502f,-0.4232f,  0.8653f,-0.3692f,-0.3391f,  0.8669f,-0.3570f,-0.3478f,  0.8469f,-0.4670f,-0.2542f,  0.8482f,-0.4577f,-0.2665f,  0.8141f,-0.5568f,-0.1649f,  0.8150f,-0.5506f,-0.1806f,  0.7674f,-0.6370f,-0.0727f,  0.7679f,-0.6340f,-0.0915f,  0.7075f,-0.7064f,0.0207f,  0.7076f,-0.7066f,-0.0009f,  
	0.7075f,-0.7064f,0.0207f,  0.7074f,-0.7055f,0.0424f,  0.6358f,-0.7595f,0.1376f,  0.6358f,-0.7549f,0.1610f,  0.5535f,-0.7928f,0.2554f,  0.5535f,-0.7846f,0.2794f,  0.4618f,-0.8060f,0.3703f,  0.4617f,-0.7946f,0.3943f,  0.3622f,-0.7995f,0.4792f,  0.3618f,-0.7854f,0.5022f,  0.2561f,-0.7744f,0.5786f,  0.2555f,-0.7580f,0.6002f,  0.1453f,-0.7318f,0.6658f,  0.1444f,-0.7139f,0.6852f,  0.0317f,-0.6737f,0.7383f,  
	0.0306f,-0.6546f,0.7553f,  -0.0827f,-0.6019f,0.7943f,  -0.0840f,-0.5825f,0.8085f,  -0.1959f,-0.5189f,0.8321f,  -0.1975f,-0.4995f,0.8435f,  -0.3061f,-0.4269f,0.8509f,  -0.3077f,-0.4082f,0.8595f,  -0.4110f,-0.3284f,0.8504f,  -0.4128f,-0.3107f,0.8562f,  -0.5091f,-0.2259f,0.8306f,  -0.5107f,-0.2098f,0.8338f,  -0.5983f,-0.1220f,0.7920f,  -0.5999f,-0.1075f,0.7928f,  -0.6771f,-0.0187f,0.7356f,  -0.6787f,-0.0063f,0.7344f,  
	-0.7442f,0.0815f,0.6629f,  -0.7456f,0.0918f,0.6600f,  -0.7984f,0.1767f,0.5756f,  -0.7996f,0.1849f,0.5713f,  -0.8387f,0.2653f,0.4757f,  -0.8398f,0.2713f,0.4703f,  -0.8643f,0.3455f,0.3654f,  -0.8653f,0.3495f,0.3593f,  -0.8751f,0.4162f,0.2472f,  -0.8759f,0.4182f,0.2405f,  -0.8706f,0.4762f,0.1234f,  -0.8714f,0.4765f,0.1166f,  -0.8512f,0.5249f,-0.0032f,  -0.8519f,0.5236f,-0.0100f,  -0.8170f,0.5617f,-0.1302f,  
	-0.8178f,0.5590f,-0.1368f,  -0.7689f,0.5863f,-0.2550f,  -0.7697f,0.5825f,-0.2613f,  -0.7076f,0.5986f,-0.3753f,  -0.7084f,0.5940f,-0.3811f,  -0.6343f,0.5990f,-0.4888f,  -0.6352f,0.5937f,-0.4941f,  -0.5502f,0.5876f,-0.5933f,  -0.5512f,0.5819f,-0.5980f,  -0.4568f,0.5651f,-0.6870f,  -0.4579f,0.5591f,-0.6912f,  -0.3557f,0.5322f,-0.7683f,  -0.3569f,0.5262f,-0.7719f,  -0.2487f,0.4898f,-0.8356f,  -0.2500f,0.4838f,-0.8387f,  
	-0.1375f,0.4388f,-0.8880f,  -0.1389f,0.4330f,-0.8906f,  -0.0240f,0.3804f,-0.9245f,  -0.0256f,0.3749f,-0.9267f,  0.0897f,0.3158f,-0.9446f,  0.0881f,0.3106f,-0.9465f,  0.2019f,0.2461f,-0.9480f,  0.2002f,0.2412f,-0.9496f,  0.3106f,0.1727f,-0.9347f,  0.3089f,0.1682f,-0.9361f,  0.4140f,0.0968f,-0.9051f,  0.4123f,0.0928f,-0.9063f,  0.5104f,0.0199f,-0.8597f,  0.5087f,0.0162f,-0.8608f,  0.5982f,-0.0568f,-0.7993f,  
	0.5965f,-0.0602f,-0.8003f,  0.6759f,-0.1319f,-0.7251f,  0.6742f,-0.1351f,-0.7261f,  0.7422f,-0.2043f,-0.6383f,  0.7405f,-0.2072f,-0.6393f,  0.7959f,-0.2727f,-0.5405f,  0.7944f,-0.2754f,-0.5414f,  0.8363f,-0.3359f,-0.4333f,  0.8348f,-0.3386f,-0.4342f,  0.8626f,-0.3931f,-0.3185f,  0.8611f,-0.3956f,-0.3194f,  0.8743f,-0.4430f,-0.1982f,  0.8729f,-0.4455f,-0.1989f,  0.8713f,-0.4851f,-0.0743f,  0.8699f,-0.4875f,-0.0749f,  
	0.8536f,-0.5184f,0.0512f,  0.8522f,-0.5207f,0.0506f,  0.8215f,-0.5423f,0.1759f,  0.8201f,-0.5446f,0.1755f,  0.7756f,-0.5565f,0.2979f,  0.7741f,-0.5587f,0.2977f,  0.7165f,-0.5606f,0.4151f,  0.7150f,-0.5625f,0.4151f,  0.6356f,-0.7636f,0.1138f,  0.5532f,-0.8005f,0.2305f,  0.5527f,-0.8078f,0.2050f,  0.4612f,-0.8278f,0.3195f,  0.4604f,-0.8381f,0.2926f,  0.3612f,-0.8409f,0.4030f,  0.3601f,-0.8541f,0.3753f,  
	0.2551f,-0.8396f,0.4796f,  0.2537f,-0.8554f,0.4515f,  0.1445f,-0.8239f,0.5480f,  0.1429f,-0.8421f,0.5200f,  0.0315f,-0.7941f,0.6070f,  0.0297f,-0.8144f,0.5796f,  -0.0820f,-0.7506f,0.6556f,  -0.0840f,-0.7726f,0.6293f,  -0.1941f,-0.6944f,0.6929f,  -0.1963f,-0.7177f,0.6681f,  -0.3030f,-0.6262f,0.7184f,  -0.3052f,-0.6505f,0.6955f,  -0.4066f,-0.5473f,0.7315f,  -0.4090f,-0.5721f,0.7109f,  -0.5032f,-0.4591f,0.7321f,  
	-0.5057f,-0.4840f,0.7142f,  -0.5913f,-0.3630f,0.7202f,  -0.5937f,-0.3875f,0.7052f,  -0.6692f,-0.2607f,0.6959f,  -0.6715f,-0.2845f,0.6842f,  -0.7356f,-0.1539f,0.6597f,  -0.7379f,-0.1766f,0.6514f,  -0.7894f,-0.0445f,0.6122f,  -0.7916f,-0.0656f,0.6075f,  -0.8297f,0.0656f,0.5543f,  -0.8317f,0.0465f,0.5532f,  -0.8558f,0.1747f,0.4869f,  -0.8576f,0.1577f,0.4895f,  -0.8673f,0.2807f,0.4112f,  -0.8688f,0.2663f,0.4174f,  
	-0.8639f,0.3820f,0.3284f,  -0.8652f,0.3703f,0.3381f,  -0.8457f,0.4767f,0.2401f,  -0.8467f,0.4680f,0.2531f,  -0.8130f,0.5632f,0.1476f,  -0.8138f,0.5576f,0.1638f,  -0.7664f,0.6401f,0.0527f,  -0.7669f,0.6378f,0.0717f,  -0.7068f,0.7061f,-0.0432f,  -0.7069f,0.7070f,-0.0216f,  -0.6350f,0.7600f,-0.1383f,  -0.6348f,0.7641f,-0.1146f,  -0.5524f,0.8009f,-0.2310f,  -0.5519f,0.8082f,-0.2055f,  -0.4604f,0.8281f,-0.3198f,  
	-0.4596f,0.8384f,-0.2930f,  -0.3605f,0.8412f,-0.4031f,  -0.3594f,0.8543f,-0.3754f,  -0.2544f,0.8399f,-0.4795f,  -0.2530f,0.8557f,-0.4515f,  -0.1440f,0.8242f,-0.5477f,  -0.1424f,0.8424f,-0.5198f,  -0.0311f,0.7944f,-0.6066f,  -0.0293f,0.8147f,-0.5792f,  0.0823f,0.7511f,-0.6550f,  0.0843f,0.7730f,-0.6287f,  0.1942f,0.6949f,-0.6923f,  0.1964f,0.7182f,-0.6675f,  0.3029f,0.6269f,-0.7178f,  0.3052f,0.6511f,-0.6949f,  
	0.4064f,0.5481f,-0.7310f,  0.4088f,0.5729f,-0.7105f,  0.5030f,0.4600f,-0.7317f,  0.5054f,0.4848f,-0.7138f,  0.5909f,0.3640f,-0.7200f,  0.5933f,0.3885f,-0.7050f,  0.6688f,0.2618f,-0.6959f,  0.6711f,0.2855f,-0.6842f,  0.7352f,0.1551f,-0.6599f,  0.7375f,0.1777f,-0.6516f,  0.7891f,0.0457f,-0.6126f,  0.7912f,0.0668f,-0.6079f,  0.8295f,-0.0645f,-0.5548f,  0.8314f,-0.0453f,-0.5538f,  0.8557f,-0.1735f,-0.4876f,  
	0.8574f,-0.1565f,-0.4902f,  0.8672f,-0.2796f,-0.4120f,  0.8688f,-0.2651f,-0.4183f,  0.8640f,-0.3809f,-0.3293f,  0.8653f,-0.3692f,-0.3391f,  0.8459f,-0.4757f,-0.2410f,  0.8469f,-0.4670f,-0.2542f,  0.8134f,-0.5624f,-0.1486f,  0.8141f,-0.5568f,-0.1649f,  0.7670f,-0.6394f,-0.0535f,  0.7674f,-0.6370f,-0.0727f,  0.7074f,-0.7055f,0.0424f,  0.7075f,-0.7064f,0.0207f,  0.7074f,-0.7055f,0.0424f,  0.7073f,-0.7041f,0.0638f,  
	0.6358f,-0.7549f,0.1610f,  0.6357f,-0.7497f,0.1837f,  0.5535f,-0.7846f,0.2794f,  0.5534f,-0.7760f,0.3028f,  0.4617f,-0.7946f,0.3943f,  0.4614f,-0.7829f,0.4173f,  0.3618f,-0.7854f,0.5022f,  0.3612f,-0.7712f,0.5242f,  0.2555f,-0.7580f,0.6002f,  0.2547f,-0.7418f,0.6204f,  0.1444f,-0.7139f,0.6852f,  0.1433f,-0.6961f,0.7035f,  0.0306f,-0.6546f,0.7553f,  0.0293f,-0.6362f,0.7710f,  -0.0840f,-0.5825f,0.8085f,  
	-0.0855f,-0.5636f,0.8216f,  -0.1975f,-0.4995f,0.8435f,  -0.1991f,-0.4809f,0.8539f,  -0.3077f,-0.4082f,0.8595f,  -0.3094f,-0.3902f,0.8672f,  -0.4128f,-0.3107f,0.8562f,  -0.4145f,-0.2940f,0.8613f,  -0.5107f,-0.2098f,0.8338f,  -0.5124f,-0.1944f,0.8364f,  -0.5999f,-0.1075f,0.7928f,  -0.6015f,-0.0940f,0.7933f,  -0.6787f,-0.0063f,0.7344f,  -0.6801f,0.0053f,0.7331f,  -0.7456f,0.0918f,0.6600f,  -0.7470f,0.1014f,0.6571f,  
	-0.7996f,0.1849f,0.5713f,  -0.8008f,0.1924f,0.5671f,  -0.8398f,0.2713f,0.4703f,  -0.8408f,0.2768f,0.4652f,  -0.8653f,0.3495f,0.3593f,  -0.8663f,0.3530f,0.3535f,  -0.8759f,0.4182f,0.2405f,  -0.8768f,0.4199f,0.2344f,  -0.8714f,0.4765f,0.1166f,  -0.8722f,0.4765f,0.1103f,  -0.8519f,0.5236f,-0.0100f,  -0.8527f,0.5222f,-0.0163f,  -0.8178f,0.5590f,-0.1368f,  -0.8185f,0.5564f,-0.1428f,  -0.7697f,0.5825f,-0.2613f,  
	-0.7705f,0.5789f,-0.2670f,  -0.7084f,0.5940f,-0.3811f,  -0.7093f,0.5896f,-0.3864f,  -0.6352f,0.5937f,-0.4941f,  -0.6361f,0.5886f,-0.4988f,  -0.5512f,0.5819f,-0.5980f,  -0.5522f,0.5765f,-0.6023f,  -0.4579f,0.5591f,-0.6912f,  -0.4591f,0.5535f,-0.6949f,  -0.3569f,0.5262f,-0.7719f,  -0.3582f,0.5205f,-0.7751f,  -0.2500f,0.4838f,-0.8387f,  -0.2514f,0.4782f,-0.8415f,  -0.1389f,0.4330f,-0.8906f,  -0.1404f,0.4276f,-0.8930f,  
	-0.0256f,0.3749f,-0.9267f,  -0.0272f,0.3697f,-0.9287f,  0.0881f,0.3106f,-0.9465f,  0.0865f,0.3057f,-0.9482f,  0.2002f,0.2412f,-0.9496f,  0.1985f,0.2367f,-0.9511f,  0.3089f,0.1682f,-0.9361f,  0.3071f,0.1640f,-0.9374f,  0.4123f,0.0928f,-0.9063f,  0.4105f,0.0889f,-0.9075f,  0.5087f,0.0162f,-0.8608f,  0.5069f,0.0126f,-0.8619f,  0.5965f,-0.0602f,-0.8003f,  0.5947f,-0.0635f,-0.8014f,  0.6742f,-0.1351f,-0.7261f,  
	0.6725f,-0.1382f,-0.7271f,  0.7405f,-0.2072f,-0.6393f,  0.7388f,-0.2102f,-0.6402f,  0.7944f,-0.2754f,-0.5414f,  0.7927f,-0.2783f,-0.5424f,  0.8348f,-0.3386f,-0.4342f,  0.8332f,-0.3414f,-0.4351f,  0.8611f,-0.3956f,-0.3194f,  0.8595f,-0.3984f,-0.3202f,  0.8729f,-0.4455f,-0.1989f,  0.8713f,-0.4483f,-0.1997f,  0.8699f,-0.4875f,-0.0749f,  0.8683f,-0.4902f,-0.0756f,  0.8522f,-0.5207f,0.0506f,  0.8506f,-0.5234f,0.0501f,  
	0.8201f,-0.5446f,0.1755f,  0.8184f,-0.5473f,0.1752f,  0.7741f,-0.5587f,0.2977f,  0.7724f,-0.5612f,0.2975f,  0.7150f,-0.5625f,0.4151f,  0.7132f,-0.5648f,0.4151f,  0.5532f,-0.8005f,0.2305f,  0.4616f,-0.8171f,0.3454f,  0.4612f,-0.8278f,0.3195f,  0.3619f,-0.8274f,0.4296f,  0.3612f,-0.8409f,0.4030f,  0.2559f,-0.8235f,0.5063f,  0.2551f,-0.8396f,0.4796f,  0.1456f,-0.8055f,0.5745f,  0.1445f,-0.8239f,0.5480f,  
	0.0328f,-0.7737f,0.6327f,  0.0315f,-0.7941f,0.6070f,  -0.0806f,-0.7287f,0.6801f,  -0.0820f,-0.7506f,0.6556f,  -0.1926f,-0.6711f,0.7159f,  -0.1941f,-0.6944f,0.6929f,  -0.3014f,-0.6021f,0.7393f,  -0.3030f,-0.6262f,0.7184f,  -0.4049f,-0.5228f,0.7501f,  -0.4066f,-0.5473f,0.7315f,  -0.5015f,-0.4346f,0.7481f,  -0.5032f,-0.4591f,0.7321f,  -0.5896f,-0.3389f,0.7332f,  -0.5913f,-0.3630f,0.7202f,  -0.6675f,-0.2374f,0.7058f,  
	-0.6692f,-0.2607f,0.6959f,  -0.7340f,-0.1318f,0.6663f,  -0.7356f,-0.1539f,0.6597f,  -0.7879f,-0.0240f,0.6154f,  -0.7894f,-0.0445f,0.6122f,  -0.8283f,0.0842f,0.5540f,  -0.8297f,0.0656f,0.5543f,  -0.8545f,0.1910f,0.4831f,  -0.8558f,0.1747f,0.4869f,  -0.8661f,0.2945f,0.4039f,  -0.8673f,0.2807f,0.4112f,  -0.8629f,0.3929f,0.3179f,  -0.8639f,0.3820f,0.3284f,  -0.8449f,0.4847f,0.2264f,  -0.8457f,0.4767f,0.2401f,  
	-0.8124f,0.5681f,0.1311f,  -0.8130f,0.5632f,0.1476f,  -0.7661f,0.6419f,0.0336f,  -0.7664f,0.6401f,0.0527f,  -0.7066f,0.7046f,-0.0645f,  -0.7068f,0.7061f,-0.0432f,  -0.6351f,0.7553f,-0.1615f,  -0.6350f,0.7600f,-0.1383f,  -0.5527f,0.7931f,-0.2558f,  -0.5524f,0.8009f,-0.2310f,  -0.4609f,0.8174f,-0.3456f,  -0.4604f,0.8281f,-0.3198f,  -0.3612f,0.8276f,-0.4296f,  -0.3605f,0.8412f,-0.4031f,  -0.2553f,0.8238f,-0.5062f,  
	-0.2544f,0.8399f,-0.4795f,  -0.1451f,0.8058f,-0.5741f,  -0.1440f,0.8242f,-0.5477f,  -0.0324f,0.7741f,-0.6323f,  -0.0311f,0.7944f,-0.6066f,  0.0808f,0.7291f,-0.6796f,  0.0823f,0.7511f,-0.6550f,  0.1927f,0.6717f,-0.7153f,  0.1942f,0.6949f,-0.6923f,  0.3013f,0.6028f,-0.7388f,  0.3029f,0.6269f,-0.7178f,  0.4047f,0.5236f,-0.7497f,  0.4064f,0.5481f,-0.7310f,  0.5013f,0.4355f,-0.7477f,  0.5030f,0.4600f,-0.7317f,  
	0.5892f,0.3398f,-0.7330f,  0.5909f,0.3640f,-0.7200f,  0.6671f,0.2384f,-0.7058f,  0.6688f,0.2618f,-0.6959f,  0.7336f,0.1329f,-0.6665f,  0.7352f,0.1551f,-0.6599f,  0.7875f,0.0252f,-0.6158f,  0.7891f,0.0457f,-0.6126f,  0.8280f,-0.0831f,-0.5545f,  0.8295f,-0.0645f,-0.5548f,  0.8544f,-0.1899f,-0.4838f,  0.8557f,-0.1735f,-0.4876f,  0.8661f,-0.2934f,-0.4048f,  0.8672f,-0.2796f,-0.4120f,  0.8630f,-0.3920f,-0.3188f,  
	0.8640f,-0.3809f,-0.3293f,  0.8451f,-0.4838f,-0.2274f,  0.8459f,-0.4757f,-0.2410f,  0.8128f,-0.5673f,-0.1320f,  0.8134f,-0.5624f,-0.1486f,  0.7666f,-0.6412f,-0.0344f,  0.7670f,-0.6394f,-0.0535f,  0.7073f,-0.7041f,0.0638f,  0.7074f,-0.7055f,0.0424f,  0.7073f,-0.7041f,0.0638f,  0.7070f,-0.7020f,0.0849f,  0.6357f,-0.7497f,0.1837f,  0.6355f,-0.7441f,0.2060f,  0.5534f,-0.7760f,0.3028f,  0.5530f,-0.7671f,0.3252f,  
	0.4614f,-0.7829f,0.4173f,  0.4608f,-0.7711f,0.4393f,  0.3612f,-0.7712f,0.5242f,  0.3605f,-0.7570f,0.5449f,  0.2547f,-0.7418f,0.6204f,  0.2536f,-0.7257f,0.6395f,  0.1433f,-0.6961f,0.7035f,  0.1422f,-0.6789f,0.7203f,  0.0293f,-0.6362f,0.7710f,  0.0279f,-0.6181f,0.7856f,  -0.0855f,-0.5636f,0.8216f,  -0.0871f,-0.5455f,0.8336f,  -0.1991f,-0.4809f,0.8539f,  -0.2008f,-0.4630f,0.8633f,  -0.3094f,-0.3902f,0.8672f,  
	-0.3111f,-0.3732f,0.8741f,  -0.4145f,-0.2940f,0.8613f,  -0.4161f,-0.2780f,0.8658f,  -0.5124f,-0.1944f,0.8364f,  -0.5140f,-0.1800f,0.8387f,  -0.6015f,-0.0940f,0.7933f,  -0.6030f,-0.0813f,0.7936f,  -0.6801f,0.0053f,0.7331f,  -0.6815f,0.0162f,0.7317f,  -0.7470f,0.1014f,0.6571f,  -0.7482f,0.1103f,0.6543f,  -0.8008f,0.1924f,0.5671f,  -0.8019f,0.1994f,0.5632f,  -0.8408f,0.2768f,0.4652f,  -0.8418f,0.2818f,0.4604f,  
	-0.8663f,0.3530f,0.3535f,  -0.8671f,0.3561f,0.3482f,  -0.8768f,0.4199f,0.2344f,  -0.8776f,0.4213f,0.2288f,  -0.8722f,0.4765f,0.1103f,  -0.8730f,0.4764f,0.1046f,  -0.8527f,0.5222f,-0.0163f,  -0.8534f,0.5208f,-0.0219f,  -0.8185f,0.5564f,-0.1428f,  -0.8193f,0.5539f,-0.1483f,  -0.7705f,0.5789f,-0.2670f,  -0.7713f,0.5754f,-0.2721f,  -0.7093f,0.5896f,-0.3864f,  -0.7102f,0.5854f,-0.3911f,  -0.6361f,0.5886f,-0.4988f,  
	-0.6371f,0.5839f,-0.5031f,  -0.5522f,0.5765f,-0.6023f,  -0.5533f,0.5714f,-0.6061f,  -0.4591f,0.5535f,-0.6949f,  -0.4603f,0.5483f,-0.6982f,  -0.3582f,0.5205f,-0.7751f,  -0.3596f,0.5152f,-0.7780f,  -0.2514f,0.4782f,-0.8415f,  -0.2528f,0.4730f,-0.8440f,  -0.1404f,0.4276f,-0.8930f,  -0.1420f,0.4226f,-0.8951f,  -0.0272f,0.3697f,-0.9287f,  -0.0288f,0.3649f,-0.9306f,  0.0865f,0.3057f,-0.9482f,  0.0847f,0.3012f,-0.9498f,  
	0.1985f,0.2367f,-0.9511f,  0.1967f,0.2325f,-0.9525f,  0.3071f,0.1640f,-0.9374f,  0.3053f,0.1600f,-0.9387f,  0.4105f,0.0889f,-0.9075f,  0.4087f,0.0852f,-0.9087f,  0.5069f,0.0126f,-0.8619f,  0.5051f,0.0091f,-0.8630f,  0.5947f,-0.0635f,-0.8014f,  0.5929f,-0.0669f,-0.8025f,  0.6725f,-0.1382f,-0.7271f,  0.6707f,-0.1414f,-0.7281f,  0.7388f,-0.2102f,-0.6402f,  0.7371f,-0.2133f,-0.6413f,  0.7927f,-0.2783f,-0.5424f,  
	0.7910f,-0.2814f,-0.5433f,  0.8332f,-0.3414f,-0.4351f,  0.8314f,-0.3445f,-0.4360f,  0.8595f,-0.3984f,-0.3202f,  0.8578f,-0.4014f,-0.3211f,  0.8713f,-0.4483f,-0.1997f,  0.8695f,-0.4514f,-0.2004f,  0.8683f,-0.4902f,-0.0756f,  0.8665f,-0.4933f,-0.0762f,  0.8506f,-0.5234f,0.0501f,  0.8488f,-0.5265f,0.0497f,  0.8184f,-0.5473f,0.1752f,  0.8165f,-0.5502f,0.1749f,  0.7724f,-0.5612f,0.2975f,  0.7704f,-0.5639f,0.2975f,  
	0.7132f,-0.5648f,0.4151f,  0.7111f,-0.5673f,0.4153f,  0.4616f,-0.8171f,0.3454f,  0.3622f,-0.8135f,0.4549f,  0.3619f,-0.8274f,0.4296f,  0.2564f,-0.8072f,0.5317f,  0.2559f,-0.8235f,0.5063f,  0.1462f,-0.7870f,0.5994f,  0.1456f,-0.8055f,0.5745f,  0.0335f,-0.7533f,0.6568f,  0.0328f,-0.7737f,0.6327f,  -0.0798f,-0.7068f,0.7029f,  -0.0806f,-0.7287f,0.6801f,  -0.1918f,-0.6481f,0.7370f,  -0.1926f,-0.6711f,0.7159f,  
	-0.3004f,-0.5783f,0.7585f,  -0.3014f,-0.6021f,0.7393f,  -0.4039f,-0.4987f,0.7669f,  -0.4049f,-0.5228f,0.7501f,  -0.5005f,-0.4105f,0.7622f,  -0.5015f,-0.4346f,0.7481f,  -0.5885f,-0.3153f,0.7445f,  -0.5896f,-0.3389f,0.7332f,  -0.6664f,-0.2147f,0.7140f,  -0.6675f,-0.2374f,0.7058f,  -0.7329f,-0.1104f,0.6713f,  -0.7340f,-0.1318f,0.6663f,  -0.7869f,-0.0042f,0.6171f,  -0.7879f,-0.0240f,0.6154f,  -0.8273f,0.1020f,0.5524f,  
	-0.8283f,0.0842f,0.5540f,  -0.8536f,0.2065f,0.4782f,  -0.8545f,0.1910f,0.4831f,  -0.8653f,0.3075f,0.3959f,  -0.8661f,0.2945f,0.4039f,  -0.8622f,0.4032f,0.3068f,  -0.8629f,0.3929f,0.3179f,  -0.8443f,0.4920f,0.2124f,  -0.8449f,0.4847f,0.2264f,  -0.8120f,0.5724f,0.1145f,  -0.8124f,0.5681f,0.1311f,  -0.7658f,0.6430f,0.0145f,  -0.7661f,0.6419f,0.0336f,  -0.7065f,0.7026f,-0.0856f,  -0.7066f,0.7046f,-0.0645f,  
	-0.6351f,0.7502f,-0.1843f,  -0.6351f,0.7553f,-0.1615f,  -0.5528f,0.7849f,-0.2798f,  -0.5527f,0.7931f,-0.2558f,  -0.4611f,0.8063f,-0.3705f,  -0.4609f,0.8174f,-0.3456f,  -0.3616f,0.8138f,-0.4549f,  -0.3612f,0.8276f,-0.4296f,  -0.2558f,0.8075f,-0.5315f,  -0.2553f,0.8238f,-0.5062f,  -0.1457f,0.7873f,-0.5991f,  -0.1451f,0.8058f,-0.5741f,  -0.0331f,0.7537f,-0.6563f,  -0.0324f,0.7741f,-0.6323f,  0.0800f,0.7072f,-0.7024f,  
	0.0808f,0.7291f,-0.6796f,  0.1918f,0.6487f,-0.7365f,  0.1927f,0.6717f,-0.7153f,  0.3003f,0.5790f,-0.7580f,  0.3013f,0.6028f,-0.7388f,  0.4037f,0.4994f,-0.7665f,  0.4047f,0.5236f,-0.7497f,  0.5002f,0.4113f,-0.7620f,  0.5013f,0.4355f,-0.7477f,  0.5882f,0.3162f,-0.7444f,  0.5892f,0.3398f,-0.7330f,  0.6660f,0.2157f,-0.7141f,  0.6671f,0.2384f,-0.7058f,  0.7326f,0.1114f,-0.6715f,  0.7336f,0.1329f,-0.6665f,  
	0.7865f,0.0053f,-0.6175f,  0.7875f,0.0252f,-0.6158f,  0.8271f,-0.1010f,-0.5529f,  0.8280f,-0.0831f,-0.5545f,  0.8535f,-0.2055f,-0.4789f,  0.8544f,-0.1899f,-0.4838f,  0.8653f,-0.3065f,-0.3967f,  0.8661f,-0.2934f,-0.4048f,  0.8623f,-0.4023f,-0.3076f,  0.8630f,-0.3920f,-0.3188f,  0.8446f,-0.4912f,-0.2133f,  0.8451f,-0.4838f,-0.2274f,  0.8124f,-0.5716f,-0.1153f,  0.8128f,-0.5673f,-0.1320f,  0.7663f,-0.6423f,-0.0153f,  
	0.7666f,-0.6412f,-0.0344f,  0.7070f,-0.7020f,0.0849f,  0.7073f,-0.7041f,0.0638f,  0.7070f,-0.7020f,0.0849f,  0.7068f,-0.6995f,0.1057f,  0.6355f,-0.7441f,0.2060f,  0.6352f,-0.7381f,0.2276f,  0.5530f,-0.7671f,0.3252f,  0.5525f,-0.7579f,0.3469f,  0.4608f,-0.7711f,0.4393f,  0.4602f,-0.7593f,0.4602f,  0.3605f,-0.7570f,0.5449f,  0.3596f,-0.7429f,0.5647f,  0.2536f,-0.7257f,0.6395f,  0.2526f,-0.7100f,0.6573f,  
	0.1422f,-0.6789f,0.7203f,  0.1408f,-0.6620f,0.7362f,  0.0279f,-0.6181f,0.7856f,  0.0264f,-0.6007f,0.7991f,  -0.0871f,-0.5455f,0.8336f,  -0.0887f,-0.5279f,0.8447f,  -0.2008f,-0.4630f,0.8633f,  -0.2024f,-0.4460f,0.8719f,  -0.3111f,-0.3732f,0.8741f,  -0.3127f,-0.3569f,0.8802f,  -0.4161f,-0.2780f,0.8658f,  -0.4177f,-0.2630f,0.8697f,  -0.5140f,-0.1800f,0.8387f,  -0.5155f,-0.1664f,0.8406f,  -0.6030f,-0.0813f,0.7936f,  
	-0.6043f,-0.0693f,0.7937f,  -0.6815f,0.0162f,0.7317f,  -0.6827f,0.0263f,0.7302f,  -0.7482f,0.1103f,0.6543f,  -0.7493f,0.1185f,0.6516f,  -0.8019f,0.1994f,0.5632f,  -0.8029f,0.2057f,0.5595f,  -0.8418f,0.2818f,0.4604f,  -0.8427f,0.2863f,0.4560f,  -0.8671f,0.3561f,0.3482f,  -0.8679f,0.3589f,0.3433f,  -0.8776f,0.4213f,0.2288f,  -0.8783f,0.4225f,0.2236f,  -0.8730f,0.4764f,0.1046f,  -0.8737f,0.4762f,0.0994f,  
	-0.8534f,0.5208f,-0.0219f,  -0.8541f,0.5194f,-0.0270f,  -0.8193f,0.5539f,-0.1483f,  -0.8201f,0.5514f,-0.1531f,  -0.7713f,0.5754f,-0.2721f,  -0.7721f,0.5721f,-0.2766f,  -0.7102f,0.5854f,-0.3911f,  -0.7111f,0.5814f,-0.3953f,  -0.6371f,0.5839f,-0.5031f,  -0.6382f,0.5795f,-0.5068f,  -0.5533f,0.5714f,-0.6061f,  -0.5545f,0.5667f,-0.6094f,  -0.4603f,0.5483f,-0.6982f,  -0.4616f,0.5434f,-0.7012f,  -0.3596f,0.5152f,-0.7780f,  
	-0.3610f,0.5103f,-0.7805f,  -0.2528f,0.4730f,-0.8440f,  -0.2544f,0.4682f,-0.8462f,  -0.1420f,0.4226f,-0.8951f,  -0.1436f,0.4179f,-0.8971f,  -0.0288f,0.3649f,-0.9306f,  -0.0305f,0.3605f,-0.9323f,  0.0847f,0.3012f,-0.9498f,  0.0829f,0.2969f,-0.9513f,  0.1967f,0.2325f,-0.9525f,  0.1949f,0.2285f,-0.9539f,  0.3053f,0.1600f,-0.9387f,  0.3034f,0.1562f,-0.9400f,  0.4087f,0.0852f,-0.9087f,  0.4068f,0.0816f,-0.9099f,  
	0.5051f,0.0091f,-0.8630f,  0.5032f,0.0057f,-0.8641f,  0.5929f,-0.0669f,-0.8025f,  0.5910f,-0.0702f,-0.8036f,  0.6707f,-0.1414f,-0.7281f,  0.6688f,-0.1447f,-0.7292f,  0.7371f,-0.2133f,-0.6413f,  0.7352f,-0.2165f,-0.6423f,  0.7910f,-0.2814f,-0.5433f,  0.7891f,-0.2846f,-0.5443f,  0.8314f,-0.3445f,-0.4360f,  0.8296f,-0.3477f,-0.4369f,  0.8578f,-0.4014f,-0.3211f,  0.8559f,-0.4047f,-0.3219f,  0.8695f,-0.4514f,-0.2004f,  
	0.8676f,-0.4547f,-0.2011f,  0.8665f,-0.4933f,-0.0762f,  0.8646f,-0.4967f,-0.0767f,  0.8488f,-0.5265f,0.0497f,  0.8467f,-0.5298f,0.0493f,  0.8165f,-0.5502f,0.1749f,  0.8144f,-0.5534f,0.1748f,  0.7704f,-0.5639f,0.2975f,  0.7681f,-0.5670f,0.2976f,  0.7111f,-0.5673f,0.4153f,  0.7087f,-0.5701f,0.4156f,  0.3622f,-0.8135f,0.4549f,  0.2564f,-0.7908f,0.5558f,  0.2564f,-0.8072f,0.5317f,  0.1463f,-0.7685f,0.6229f,  
	0.1462f,-0.7870f,0.5994f,  0.0336f,-0.7331f,0.6793f,  0.0335f,-0.7533f,0.6568f,  -0.0796f,-0.6851f,0.7241f,  -0.0798f,-0.7068f,0.7029f,  -0.1915f,-0.6253f,0.7565f,  -0.1918f,-0.6481f,0.7370f,  -0.3001f,-0.5549f,0.7759f,  -0.3004f,-0.5783f,0.7585f,  -0.4036f,-0.4750f,0.7820f,  -0.4039f,-0.4987f,0.7669f,  -0.5001f,-0.3869f,0.7747f,  -0.5005f,-0.4105f,0.7622f,  -0.5881f,-0.2922f,0.7542f,  -0.5885f,-0.3153f,0.7445f,  
	-0.6659f,-0.1926f,0.7207f,  -0.6664f,-0.2147f,0.7140f,  -0.7324f,-0.0896f,0.6749f,  -0.7329f,-0.1104f,0.6713f,  -0.7864f,0.0149f,0.6176f,  -0.7869f,-0.0042f,0.6171f,  -0.8268f,0.1192f,0.5497f,  -0.8273f,0.1020f,0.5524f,  -0.8531f,0.2214f,0.4724f,  -0.8536f,0.2065f,0.4782f,  -0.8648f,0.3198f,0.3870f,  -0.8653f,0.3075f,0.3959f,  -0.8617f,0.4127f,0.2951f,  -0.8622f,0.4032f,0.3068f,  -0.8439f,0.4986f,0.1981f,  
	-0.8443f,0.4920f,0.2124f,  -0.8116f,0.5760f,0.0977f,  -0.8120f,0.5724f,0.1145f,  -0.7654f,0.6435f,-0.0044f,  -0.7658f,0.6430f,0.0145f,  -0.7062f,0.7000f,-0.1063f,  -0.7065f,0.7026f,-0.0856f,  -0.6349f,0.7445f,-0.2064f,  -0.6351f,0.7502f,-0.1843f,  -0.5527f,0.7763f,-0.3030f,  -0.5528f,0.7849f,-0.2798f,  -0.4611f,0.7949f,-0.3945f,  -0.4611f,0.8063f,-0.3705f,  -0.3616f,0.7998f,-0.4791f,  -0.3616f,0.8138f,-0.4549f,  
	-0.2559f,0.7911f,-0.5556f,  -0.2558f,0.8075f,-0.5315f,  -0.1458f,0.7689f,-0.6226f,  -0.1457f,0.7873f,-0.5991f,  -0.0333f,0.7335f,-0.6789f,  -0.0331f,0.7537f,-0.6563f,  0.0798f,0.6855f,-0.7236f,  0.0800f,0.7072f,-0.7024f,  0.1915f,0.6259f,-0.7560f,  0.1918f,0.6487f,-0.7365f,  0.3000f,0.5556f,-0.7755f,  0.3003f,0.5790f,-0.7580f,  0.4034f,0.4757f,-0.7817f,  0.4037f,0.4994f,-0.7665f,  0.4998f,0.3878f,-0.7745f,  
	0.5002f,0.4113f,-0.7620f,  0.5877f,0.2932f,-0.7541f,  0.5882f,0.3162f,-0.7444f,  0.6656f,0.1935f,-0.7208f,  0.6660f,0.2157f,-0.7141f,  0.7321f,0.0906f,-0.6752f,  0.7326f,0.1114f,-0.6715f,  0.7861f,-0.0139f,-0.6180f,  0.7865f,0.0053f,-0.6175f,  0.8266f,-0.1181f,-0.5502f,  0.8271f,-0.1010f,-0.5529f,  0.8530f,-0.2204f,-0.4731f,  0.8535f,-0.2055f,-0.4789f,  0.8648f,-0.3188f,-0.3878f,  0.8653f,-0.3065f,-0.3967f,  
	0.8619f,-0.4119f,-0.2959f,  0.8623f,-0.4023f,-0.3076f,  0.8442f,-0.4978f,-0.1989f,  0.8446f,-0.4912f,-0.2133f,  0.8120f,-0.5753f,-0.0985f,  0.8124f,-0.5716f,-0.1153f,  0.7659f,-0.6429f,0.0036f,  0.7663f,-0.6423f,-0.0153f,  0.7068f,-0.6995f,0.1057f,  0.7070f,-0.7020f,0.0849f,  0.7068f,-0.6995f,0.1057f,  0.7065f,-0.6964f,0.1260f,  0.6352f,-0.7381f,0.2276f,  0.6348f,-0.7317f,0.2485f,  0.5525f,-0.7579f,0.3469f,  
	0.5520f,-0.7485f,0.3675f,  0.4602f,-0.7593f,0.4602f,  0.4594f,-0.7473f,0.4801f,  0.3596f,-0.7429f,0.5647f,  0.3587f,-0.7290f,0.5831f,  0.2526f,-0.7100f,0.6573f,  0.2514f,-0.6945f,0.6741f,  0.1408f,-0.6620f,0.7362f,  0.1395f,-0.6456f,0.7509f,  0.0264f,-0.6007f,0.7991f,  0.0249f,-0.5838f,0.8115f,  -0.0887f,-0.5279f,0.8447f,  -0.0902f,-0.5111f,0.8548f,  -0.2024f,-0.4460f,0.8719f,  -0.2040f,-0.4296f,0.8797f,  
	-0.3127f,-0.3569f,0.8802f,  -0.3142f,-0.3415f,0.8858f,  -0.4177f,-0.2630f,0.8697f,  -0.4192f,-0.2488f,0.8732f,  -0.5155f,-0.1664f,0.8406f,  -0.5168f,-0.1536f,0.8422f,  -0.6043f,-0.0693f,0.7937f,  -0.6056f,-0.0582f,0.7937f,  -0.6827f,0.0263f,0.7302f,  -0.6838f,0.0357f,0.7288f,  -0.7493f,0.1185f,0.6516f,  -0.7503f,0.1261f,0.6490f,  -0.8029f,0.2057f,0.5595f,  -0.8038f,0.2115f,0.5561f,  -0.8427f,0.2863f,0.4560f,  
	-0.8434f,0.2904f,0.4520f,  -0.8679f,0.3589f,0.3433f,  -0.8686f,0.3614f,0.3389f,  -0.8783f,0.4225f,0.2236f,  -0.8790f,0.4236f,0.2190f,  -0.8737f,0.4762f,0.0994f,  -0.8743f,0.4760f,0.0947f,  -0.8541f,0.5194f,-0.0270f,  -0.8548f,0.5179f,-0.0316f,  -0.8201f,0.5514f,-0.1531f,  -0.8208f,0.5490f,-0.1575f,  -0.7721f,0.5721f,-0.2766f,  -0.7730f,0.5690f,-0.2807f,  -0.7111f,0.5814f,-0.3953f,  -0.7121f,0.5777f,-0.3990f,  
	-0.6382f,0.5795f,-0.5068f,  -0.6392f,0.5754f,-0.5102f,  -0.5545f,0.5667f,-0.6094f,  -0.5557f,0.5623f,-0.6123f,  -0.4616f,0.5434f,-0.7012f,  -0.4629f,0.5389f,-0.7038f,  -0.3610f,0.5103f,-0.7805f,  -0.3624f,0.5058f,-0.7828f,  -0.2544f,0.4682f,-0.8462f,  -0.2559f,0.4638f,-0.8482f,  -0.1436f,0.4179f,-0.8971f,  -0.1453f,0.4136f,-0.8988f,  -0.0305f,0.3605f,-0.9323f,  -0.0323f,0.3563f,-0.9338f,  0.0829f,0.2969f,-0.9513f,  
	0.0811f,0.2930f,-0.9527f,  0.1949f,0.2285f,-0.9539f,  0.1930f,0.2247f,-0.9551f,  0.3034f,0.1562f,-0.9400f,  0.3015f,0.1526f,-0.9412f,  0.4068f,0.0816f,-0.9099f,  0.4049f,0.0781f,-0.9110f,  0.5032f,0.0057f,-0.8641f,  0.5013f,0.0023f,-0.8653f,  0.5910f,-0.0702f,-0.8036f,  0.5891f,-0.0735f,-0.8047f,  0.6688f,-0.1447f,-0.7292f,  0.6669f,-0.1480f,-0.7303f,  0.7352f,-0.2165f,-0.6423f,  0.7333f,-0.2199f,-0.6434f,  
	0.7891f,-0.2846f,-0.5443f,  0.7872f,-0.2880f,-0.5453f,  0.8296f,-0.3477f,-0.4369f,  0.8276f,-0.3512f,-0.4378f,  0.8559f,-0.4047f,-0.3219f,  0.8539f,-0.4083f,-0.3227f,  0.8676f,-0.4547f,-0.2011f,  0.8656f,-0.4584f,-0.2018f,  0.8646f,-0.4967f,-0.0767f,  0.8624f,-0.5003f,-0.0772f,  0.8467f,-0.5298f,0.0493f,  0.8444f,-0.5335f,0.0490f,  0.8144f,-0.5534f,0.1748f,  0.8119f,-0.5570f,0.1747f,  0.7681f,-0.5670f,0.2976f,  
	0.7655f,-0.5704f,0.2978f,  0.7087f,-0.5701f,0.4156f,  0.7059f,-0.5732f,0.4161f,  0.2564f,-0.7908f,0.5558f,  0.1460f,-0.7501f,0.6450f,  0.1463f,-0.7685f,0.6229f,  0.0333f,-0.7130f,0.7004f,  0.0336f,-0.7331f,0.6793f,  -0.0799f,-0.6636f,0.7438f,  -0.0796f,-0.6851f,0.7241f,  -0.1917f,-0.6030f,0.7744f,  -0.1915f,-0.6253f,0.7565f,  -0.3003f,-0.5319f,0.7917f,  -0.3001f,-0.5549f,0.7759f,  -0.4037f,-0.4518f,0.7955f,  
	-0.4036f,-0.4750f,0.7820f,  -0.5002f,-0.3640f,0.7857f,  -0.5001f,-0.3869f,0.7747f,  -0.5881f,-0.2699f,0.7624f,  -0.5881f,-0.2922f,0.7542f,  -0.6660f,-0.1712f,0.7261f,  -0.6659f,-0.1926f,0.7207f,  -0.7324f,-0.0696f,0.6773f,  -0.7324f,-0.0896f,0.6749f,  -0.7863f,0.0332f,0.6170f,  -0.7864f,0.0149f,0.6176f,  -0.8267f,0.1355f,0.5461f,  -0.8268f,0.1192f,0.5497f,  -0.8530f,0.2354f,0.4658f,  -0.8531f,0.2214f,0.4724f,  
	-0.8646f,0.3313f,0.3777f,  -0.8648f,0.3198f,0.3870f,  -0.8615f,0.4215f,0.2830f,  -0.8617f,0.4127f,0.2951f,  -0.8436f,0.5045f,0.1836f,  -0.8439f,0.4986f,0.1981f,  -0.8113f,0.5789f,0.0810f,  -0.8116f,0.5760f,0.0977f,  -0.7652f,0.6434f,-0.0230f,  -0.7654f,0.6435f,-0.0044f,  -0.7059f,0.6969f,-0.1266f,  -0.7062f,0.7000f,-0.1063f,  -0.6346f,0.7385f,-0.2280f,  -0.6349f,0.7445f,-0.2064f,  -0.5524f,0.7674f,-0.3255f,  
	-0.5527f,0.7763f,-0.3030f,  -0.4608f,0.7832f,-0.4174f,  -0.4611f,0.7949f,-0.3945f,  -0.3613f,0.7857f,-0.5022f,  -0.3616f,0.7998f,-0.4791f,  -0.2556f,0.7747f,-0.5784f,  -0.2559f,0.7911f,-0.5556f,  -0.1456f,0.7505f,-0.6447f,  -0.1458f,0.7689f,-0.6226f,  -0.0331f,0.7134f,-0.7000f,  -0.0333f,0.7335f,-0.6789f,  0.0800f,0.6641f,-0.7433f,  0.0798f,0.6855f,-0.7236f,  0.1917f,0.6035f,-0.7739f,  0.1915f,0.6259f,-0.7560f,  
	0.3002f,0.5326f,-0.7913f,  0.3000f,0.5556f,-0.7755f,  0.4035f,0.4526f,-0.7952f,  0.4034f,0.4757f,-0.7817f,  0.4999f,0.3648f,-0.7855f,  0.4998f,0.3878f,-0.7745f,  0.5878f,0.2708f,-0.7624f,  0.5877f,0.2932f,-0.7541f,  0.6656f,0.1721f,-0.7262f,  0.6656f,0.1935f,-0.7208f,  0.7321f,0.0706f,-0.6776f,  0.7321f,0.0906f,-0.6752f,  0.7860f,-0.0322f,-0.6174f,  0.7861f,-0.0139f,-0.6180f,  0.8265f,-0.1345f,-0.5466f,  
	0.8266f,-0.1181f,-0.5502f,  0.8529f,-0.2345f,-0.4665f,  0.8530f,-0.2204f,-0.4731f,  0.8647f,-0.3304f,-0.3784f,  0.8648f,-0.3188f,-0.3878f,  0.8617f,-0.4207f,-0.2838f,  0.8619f,-0.4119f,-0.2959f,  0.8439f,-0.5038f,-0.1843f,  0.8442f,-0.4978f,-0.1989f,  0.8117f,-0.5783f,-0.0817f,  0.8120f,-0.5753f,-0.0985f,  0.7657f,-0.6429f,0.0224f,  0.7659f,-0.6429f,0.0036f,  0.7065f,-0.6964f,0.1260f,  0.7068f,-0.6995f,0.1057f,  
	0.7065f,-0.6964f,0.1260f,  0.7061f,-0.6929f,0.1459f,  0.6348f,-0.7317f,0.2485f,  0.6343f,-0.7249f,0.2686f,  0.5520f,-0.7485f,0.3675f,  0.5513f,-0.7389f,0.3873f,  0.4594f,-0.7473f,0.4801f,  0.4586f,-0.7354f,0.4988f,  0.3587f,-0.7290f,0.5831f,  0.3576f,-0.7152f,0.6006f,  0.2514f,-0.6945f,0.6741f,  0.2502f,-0.6794f,0.6898f,  0.1395f,-0.6456f,0.7509f,  0.1381f,-0.6296f,0.7645f,  0.0249f,-0.5838f,0.8115f,  
	0.0235f,-0.5675f,0.8230f,  -0.0902f,-0.5111f,0.8548f,  -0.0917f,-0.4950f,0.8640f,  -0.2040f,-0.4296f,0.8797f,  -0.2054f,-0.4141f,0.8867f,  -0.3142f,-0.3415f,0.8858f,  -0.3157f,-0.3269f,0.8908f,  -0.4192f,-0.2488f,0.8732f,  -0.4205f,-0.2353f,0.8762f,  -0.5168f,-0.1536f,0.8422f,  -0.5180f,-0.1417f,0.8435f,  -0.6056f,-0.0582f,0.7937f,  -0.6066f,-0.0478f,0.7935f,  -0.6838f,0.0357f,0.7288f,  -0.6848f,0.0444f,0.7274f,  
	-0.7503f,0.1261f,0.6490f,  -0.7511f,0.1332f,0.6466f,  -0.8038f,0.2115f,0.5561f,  -0.8045f,0.2169f,0.5529f,  -0.8434f,0.2904f,0.4520f,  -0.8441f,0.2941f,0.4483f,  -0.8686f,0.3614f,0.3389f,  -0.8693f,0.3636f,0.3349f,  -0.8790f,0.4236f,0.2190f,  -0.8796f,0.4244f,0.2148f,  -0.8743f,0.4760f,0.0947f,  -0.8750f,0.4756f,0.0906f,  -0.8548f,0.5179f,-0.0316f,  -0.8555f,0.5166f,-0.0356f,  -0.8208f,0.5490f,-0.1575f,  
	-0.8216f,0.5468f,-0.1613f,  -0.7730f,0.5690f,-0.2807f,  -0.7738f,0.5661f,-0.2842f,  -0.7121f,0.5777f,-0.3990f,  -0.7131f,0.5743f,-0.4022f,  -0.6392f,0.5754f,-0.5102f,  -0.6403f,0.5716f,-0.5131f,  -0.5557f,0.5623f,-0.6123f,  -0.5569f,0.5583f,-0.6149f,  -0.4629f,0.5389f,-0.7038f,  -0.4643f,0.5348f,-0.7060f,  -0.3624f,0.5058f,-0.7828f,  -0.3639f,0.5017f,-0.7848f,  -0.2559f,0.4638f,-0.8482f,  -0.2575f,0.4597f,-0.8499f,  
	-0.1453f,0.4136f,-0.8988f,  -0.1470f,0.4096f,-0.9003f,  -0.0323f,0.3563f,-0.9338f,  -0.0341f,0.3525f,-0.9352f,  0.0811f,0.2930f,-0.9527f,  0.0793f,0.2892f,-0.9540f,  0.1930f,0.2247f,-0.9551f,  0.1911f,0.2211f,-0.9563f,  0.3015f,0.1526f,-0.9412f,  0.2996f,0.1491f,-0.9423f,  0.4049f,0.0781f,-0.9110f,  0.4030f,0.0747f,-0.9122f,  0.5013f,0.0023f,-0.8653f,  0.4994f,-0.0011f,-0.8664f,  0.5891f,-0.0735f,-0.8047f,  
	0.5872f,-0.0769f,-0.8058f,  0.6669f,-0.1480f,-0.7303f,  0.6650f,-0.1514f,-0.7314f,  0.7333f,-0.2199f,-0.6434f,  0.7313f,-0.2234f,-0.6444f,  0.7872f,-0.2880f,-0.5453f,  0.7852f,-0.2917f,-0.5463f,  0.8276f,-0.3512f,-0.4378f,  0.8256f,-0.3550f,-0.4387f,  0.8539f,-0.4083f,-0.3227f,  0.8518f,-0.4122f,-0.3234f,  0.8656f,-0.4584f,-0.2018f,  0.8633f,-0.4623f,-0.2023f,  0.8624f,-0.5003f,-0.0772f,  0.8600f,-0.5044f,-0.0775f,  
	0.8444f,-0.5335f,0.0490f,  0.8419f,-0.5375f,0.0489f,  0.8119f,-0.5570f,0.1747f,  0.8092f,-0.5609f,0.1749f,  0.7655f,-0.5704f,0.2978f,  0.7625f,-0.5741f,0.2983f,  0.7059f,-0.5732f,0.4161f,  0.7027f,-0.5765f,0.4168f,  0.1460f,-0.7501f,0.6450f,  0.0327f,-0.6931f,0.7201f,  0.0333f,-0.7130f,0.7004f,  -0.0806f,-0.6426f,0.7620f,  -0.0799f,-0.6636f,0.7438f,  -0.1924f,-0.5810f,0.7908f,  -0.1917f,-0.6030f,0.7744f,  
	-0.3009f,-0.5095f,0.8061f,  -0.3003f,-0.5319f,0.7917f,  -0.4043f,-0.4293f,0.8076f,  -0.4037f,-0.4518f,0.7955f,  -0.5007f,-0.3417f,0.7953f,  -0.5002f,-0.3640f,0.7857f,  -0.5886f,-0.2483f,0.7693f,  -0.5881f,-0.2699f,0.7624f,  -0.6664f,-0.1506f,0.7302f,  -0.6660f,-0.1712f,0.7261f,  -0.7328f,-0.0504f,0.6786f,  -0.7324f,-0.0696f,0.6773f,  -0.7866f,0.0507f,0.6154f,  -0.7863f,0.0332f,0.6170f,  -0.8269f,0.1510f,0.5417f,  
	-0.8267f,0.1355f,0.5461f,  -0.8531f,0.2486f,0.4587f,  -0.8530f,0.2354f,0.4658f,  -0.8647f,0.3420f,0.3678f,  -0.8646f,0.3313f,0.3777f,  -0.8615f,0.4296f,0.2707f,  -0.8615f,0.4215f,0.2830f,  -0.8435f,0.5098f,0.1690f,  -0.8436f,0.5045f,0.1836f,  -0.8112f,0.5813f,0.0644f,  -0.8113f,0.5789f,0.0810f,  -0.7649f,0.6428f,-0.0413f,  -0.7652f,0.6434f,-0.0230f,  -0.7056f,0.6934f,-0.1463f,  -0.7059f,0.6969f,-0.1266f,  
	-0.6342f,0.7321f,-0.2488f,  -0.6346f,0.7385f,-0.2280f,  -0.5519f,0.7583f,-0.3470f,  -0.5524f,0.7674f,-0.3255f,  -0.4603f,0.7715f,-0.4393f,  -0.4608f,0.7832f,-0.4174f,  -0.3607f,0.7715f,-0.5241f,  -0.3613f,0.7857f,-0.5022f,  -0.2550f,0.7583f,-0.5999f,  -0.2556f,0.7747f,-0.5784f,  -0.1449f,0.7322f,-0.6655f,  -0.1456f,0.7505f,-0.6447f,  -0.0324f,0.6936f,-0.7196f,  -0.0331f,0.7134f,-0.7000f,  0.0807f,0.6431f,-0.7615f,  
	0.0800f,0.6641f,-0.7433f,  0.1924f,0.5816f,-0.7904f,  0.1917f,0.6035f,-0.7739f,  0.3008f,0.5102f,-0.8057f,  0.3002f,0.5326f,-0.7913f,  0.4041f,0.4300f,-0.8073f,  0.4035f,0.4526f,-0.7952f,  0.5005f,0.3425f,-0.7951f,  0.4999f,0.3648f,-0.7855f,  0.5883f,0.2492f,-0.7693f,  0.5878f,0.2708f,-0.7624f,  0.6661f,0.1516f,-0.7303f,  0.6656f,0.1721f,-0.7262f,  0.7325f,0.0513f,-0.6789f,  0.7321f,0.0706f,-0.6776f,  
	0.7863f,-0.0498f,-0.6158f,  0.7860f,-0.0322f,-0.6174f,  0.8268f,-0.1500f,-0.5422f,  0.8265f,-0.1345f,-0.5466f,  0.8531f,-0.2477f,-0.4593f,  0.8529f,-0.2345f,-0.4665f,  0.8648f,-0.3412f,-0.3685f,  0.8647f,-0.3304f,-0.3784f,  0.8617f,-0.4288f,-0.2714f,  0.8617f,-0.4207f,-0.2838f,  0.8438f,-0.5091f,-0.1697f,  0.8439f,-0.5038f,-0.1843f,  0.8116f,-0.5806f,-0.0650f,  0.8117f,-0.5783f,-0.0817f,  0.7654f,-0.6423f,0.0408f,  
	0.7657f,-0.6429f,0.0224f,  0.7061f,-0.6929f,0.1459f,  0.7065f,-0.6964f,0.1260f,  0.7061f,-0.6929f,0.1459f,  0.7058f,-0.6889f,0.1650f,  0.6343f,-0.7249f,0.2686f,  0.6338f,-0.7179f,0.2879f,  0.5513f,-0.7389f,0.3873f,  0.5507f,-0.7293f,0.4061f,  0.4586f,-0.7354f,0.4988f,  0.4577f,-0.7236f,0.5167f,  0.3576f,-0.7152f,0.6006f,  0.3566f,-0.7016f,0.6169f,  0.2502f,-0.6794f,0.6898f,  0.2490f,-0.6647f,0.7044f,  
	0.1381f,-0.6296f,0.7645f,  0.1368f,-0.6142f,0.7772f,  0.0235f,-0.5675f,0.8230f,  0.0221f,-0.5519f,0.8336f,  -0.0917f,-0.4950f,0.8640f,  -0.0931f,-0.4797f,0.8725f,  -0.2054f,-0.4141f,0.8867f,  -0.2068f,-0.3993f,0.8932f,  -0.3157f,-0.3269f,0.8908f,  -0.3169f,-0.3130f,0.8953f,  -0.4205f,-0.2353f,0.8762f,  -0.4217f,-0.2227f,0.8790f,  -0.5180f,-0.1417f,0.8435f,  -0.5191f,-0.1304f,0.8447f,  -0.6066f,-0.0478f,0.7935f,  
	-0.6076f,-0.0381f,0.7933f,  -0.6848f,0.0444f,0.7274f,  -0.6856f,0.0525f,0.7261f,  -0.7511f,0.1332f,0.6466f,  -0.7518f,0.1396f,0.6444f,  -0.8045f,0.2169f,0.5529f,  -0.8051f,0.2218f,0.5501f,  -0.8441f,0.2941f,0.4483f,  -0.8447f,0.2975f,0.4450f,  -0.8693f,0.3636f,0.3349f,  -0.8698f,0.3657f,0.3313f,  -0.8796f,0.4244f,0.2148f,  -0.8801f,0.4252f,0.2111f,  -0.8750f,0.4756f,0.0906f,  -0.8755f,0.4753f,0.0869f,  
	-0.8555f,0.5166f,-0.0356f,  -0.8562f,0.5152f,-0.0392f,  -0.8216f,0.5468f,-0.1613f,  -0.8223f,0.5447f,-0.1647f,  -0.7738f,0.5661f,-0.2842f,  -0.7747f,0.5633f,-0.2873f,  -0.7131f,0.5743f,-0.4022f,  -0.7140f,0.5711f,-0.4050f,  -0.6403f,0.5716f,-0.5131f,  -0.6414f,0.5681f,-0.5156f,  -0.5569f,0.5583f,-0.6149f,  -0.5582f,0.5546f,-0.6171f,  -0.4643f,0.5348f,-0.7060f,  -0.4656f,0.5310f,-0.7080f,  -0.3639f,0.5017f,-0.7848f,  
	-0.3654f,0.4979f,-0.7865f,  -0.2575f,0.4597f,-0.8499f,  -0.2591f,0.4559f,-0.8515f,  -0.1470f,0.4096f,-0.9003f,  -0.1486f,0.4059f,-0.9017f,  -0.0341f,0.3525f,-0.9352f,  -0.0358f,0.3489f,-0.9365f,  0.0793f,0.2892f,-0.9540f,  0.0775f,0.2857f,-0.9552f,  0.1911f,0.2211f,-0.9563f,  0.1893f,0.2177f,-0.9575f,  0.2996f,0.1491f,-0.9423f,  0.2977f,0.1458f,-0.9434f,  0.4030f,0.0747f,-0.9122f,  0.4010f,0.0713f,-0.9133f,  
	0.4994f,-0.0011f,-0.8664f,  0.4974f,-0.0045f,-0.8675f,  0.5872f,-0.0769f,-0.8058f,  0.5852f,-0.0804f,-0.8069f,  0.6650f,-0.1514f,-0.7314f,  0.6630f,-0.1550f,-0.7324f,  0.7313f,-0.2234f,-0.6444f,  0.7293f,-0.2271f,-0.6454f,  0.7852f,-0.2917f,-0.5463f,  0.7831f,-0.2956f,-0.5472f,  0.8256f,-0.3550f,-0.4387f,  0.8234f,-0.3590f,-0.4395f,  0.8518f,-0.4122f,-0.3234f,  0.8495f,-0.4164f,-0.3240f,  0.8633f,-0.4623f,-0.2023f,  
	0.8609f,-0.4667f,-0.2027f,  0.8600f,-0.5044f,-0.0775f,  0.8574f,-0.5088f,-0.0777f,  0.8419f,-0.5375f,0.0489f,  0.8390f,-0.5419f,0.0490f,  0.8092f,-0.5609f,0.1749f,  0.8061f,-0.5652f,0.1753f,  0.7625f,-0.5741f,0.2983f,  0.7593f,-0.5781f,0.2989f,  0.7027f,-0.5765f,0.4168f,  0.6992f,-0.5802f,0.4178f,  0.0327f,-0.6931f,0.7201f,  -0.0815f,-0.6220f,0.7787f,  -0.0806f,-0.6426f,0.7620f,  -0.1933f,-0.5597f,0.8058f,  
	-0.1924f,-0.5810f,0.7908f,  -0.3019f,-0.4878f,0.8191f,  -0.3009f,-0.5095f,0.8061f,  -0.4052f,-0.4075f,0.8184f,  -0.4043f,-0.4293f,0.8076f,  -0.5016f,-0.3203f,0.8036f,  -0.5007f,-0.3417f,0.7953f,  -0.5894f,-0.2276f,0.7751f,  -0.5886f,-0.2483f,0.7693f,  -0.6672f,-0.1310f,0.7333f,  -0.6664f,-0.1506f,0.7302f,  -0.7334f,-0.0321f,0.6790f,  -0.7328f,-0.0504f,0.6786f,  -0.7872f,0.0673f,0.6130f,  -0.7866f,0.0507f,0.6154f,  
	-0.8274f,0.1655f,0.5366f,  -0.8269f,0.1510f,0.5417f,  -0.8535f,0.2610f,0.4510f,  -0.8531f,0.2486f,0.4587f,  -0.8650f,0.3519f,0.3577f,  -0.8647f,0.3420f,0.3678f,  -0.8617f,0.4368f,0.2583f,  -0.8615f,0.4296f,0.2707f,  -0.8436f,0.5143f,0.1545f,  -0.8435f,0.5098f,0.1690f,  -0.8111f,0.5830f,0.0480f,  -0.8112f,0.5813f,0.0644f,  -0.7647f,0.6416f,-0.0592f,  -0.7649f,0.6428f,-0.0413f,  -0.7053f,0.6894f,-0.1655f,  
	-0.7056f,0.6934f,-0.1463f,  -0.6338f,0.7253f,-0.2689f,  -0.6342f,0.7321f,-0.2488f,  -0.5514f,0.7488f,-0.3677f,  -0.5519f,0.7583f,-0.3470f,  -0.4596f,0.7596f,-0.4602f,  -0.4603f,0.7715f,-0.4393f,  -0.3600f,0.7573f,-0.5448f,  -0.3607f,0.7715f,-0.5241f,  -0.2542f,0.7421f,-0.6202f,  -0.2550f,0.7583f,-0.5999f,  -0.1441f,0.7142f,-0.6849f,  -0.1449f,0.7322f,-0.6655f,  -0.0315f,0.6741f,-0.7379f,  -0.0324f,0.6936f,-0.7196f,  
	0.0816f,0.6225f,-0.7783f,  0.0807f,0.6431f,-0.7615f,  0.1933f,0.5603f,-0.8054f,  0.1924f,0.5816f,-0.7904f,  0.3017f,0.4884f,-0.8188f,  0.3008f,0.5102f,-0.8057f,  0.4050f,0.4082f,-0.8181f,  0.4041f,0.4300f,-0.8073f,  0.5014f,0.3211f,-0.8035f,  0.5005f,0.3425f,-0.7951f,  0.5891f,0.2284f,-0.7751f,  0.5883f,0.2492f,-0.7693f,  0.6668f,0.1318f,-0.7334f,  0.6661f,0.1516f,-0.7303f,  0.7332f,0.0330f,-0.6793f,  
	0.7325f,0.0513f,-0.6789f,  0.7869f,-0.0664f,-0.6134f,  0.7863f,-0.0498f,-0.6158f,  0.8273f,-0.1647f,-0.5371f,  0.8268f,-0.1500f,-0.5422f,  0.8534f,-0.2601f,-0.4516f,  0.8531f,-0.2477f,-0.4593f,  0.8650f,-0.3511f,-0.3584f,  0.8648f,-0.3412f,-0.3685f,  0.8618f,-0.4361f,-0.2590f,  0.8617f,-0.4288f,-0.2714f,  0.8439f,-0.5136f,-0.1552f,  0.8438f,-0.5091f,-0.1697f,  0.8115f,-0.5824f,-0.0487f,  0.8116f,-0.5806f,-0.0650f,  
	0.7652f,-0.6411f,0.0587f,  0.7654f,-0.6423f,0.0408f,  0.7058f,-0.6889f,0.1650f,  0.7061f,-0.6929f,0.1459f,  0.7058f,-0.6889f,0.1650f,  0.7054f,-0.6846f,0.1836f,  0.6338f,-0.7179f,0.2879f,  0.6333f,-0.7107f,0.3063f,  0.5507f,-0.7293f,0.4061f,  0.5500f,-0.7196f,0.4240f,  0.4577f,-0.7236f,0.5167f,  0.4569f,-0.7118f,0.5334f,  0.3566f,-0.7016f,0.6169f,  0.3555f,-0.6884f,0.6323f,  0.2490f,-0.6647f,0.7044f,  
	0.2478f,-0.6504f,0.7180f,  0.1368f,-0.6142f,0.7772f,  0.1356f,-0.5994f,0.7889f,  0.0221f,-0.5519f,0.8336f,  0.0209f,-0.5370f,0.8433f,  -0.0931f,-0.4797f,0.8725f,  -0.0943f,-0.4650f,0.8803f,  -0.2068f,-0.3993f,0.8932f,  -0.2080f,-0.3853f,0.8990f,  -0.3169f,-0.3130f,0.8953f,  -0.3180f,-0.3000f,0.8994f,  -0.4217f,-0.2227f,0.8790f,  -0.4226f,-0.2109f,0.8814f,  -0.5191f,-0.1304f,0.8447f,  -0.5200f,-0.1199f,0.8457f,  
	-0.6076f,-0.0381f,0.7933f,  -0.6083f,-0.0290f,0.7931f,  -0.6856f,0.0525f,0.7261f,  -0.6862f,0.0600f,0.7249f,  -0.7518f,0.1396f,0.6444f,  -0.7524f,0.1456f,0.6425f,  -0.8051f,0.2218f,0.5501f,  -0.8056f,0.2263f,0.5475f,  -0.8447f,0.2975f,0.4450f,  -0.8451f,0.3006f,0.4420f,  -0.8698f,0.3657f,0.3313f,  -0.8702f,0.3675f,0.3281f,  -0.8801f,0.4252f,0.2111f,  -0.8806f,0.4258f,0.2079f,  -0.8755f,0.4753f,0.0869f,  
	-0.8761f,0.4749f,0.0837f,  -0.8562f,0.5152f,-0.0392f,  -0.8567f,0.5140f,-0.0423f,  -0.8223f,0.5447f,-0.1647f,  -0.8230f,0.5428f,-0.1675f,  -0.7747f,0.5633f,-0.2873f,  -0.7755f,0.5609f,-0.2900f,  -0.7140f,0.5711f,-0.4050f,  -0.7149f,0.5682f,-0.4074f,  -0.6414f,0.5681f,-0.5156f,  -0.6425f,0.5649f,-0.5177f,  -0.5582f,0.5546f,-0.6171f,  -0.5594f,0.5512f,-0.6191f,  -0.4656f,0.5310f,-0.7080f,  -0.4669f,0.5276f,-0.7097f,  
	-0.3654f,0.4979f,-0.7865f,  -0.3668f,0.4944f,-0.7880f,  -0.2591f,0.4559f,-0.8515f,  -0.2607f,0.4525f,-0.8528f,  -0.1486f,0.4059f,-0.9017f,  -0.1503f,0.4025f,-0.9030f,  -0.0358f,0.3489f,-0.9365f,  -0.0376f,0.3455f,-0.9377f,  0.0775f,0.2857f,-0.9552f,  0.0757f,0.2825f,-0.9563f,  0.1893f,0.2177f,-0.9575f,  0.1874f,0.2144f,-0.9586f,  0.2977f,0.1458f,-0.9434f,  0.2959f,0.1425f,-0.9445f,  0.4010f,0.0713f,-0.9133f,  
	0.3991f,0.0680f,-0.9144f,  0.4974f,-0.0045f,-0.8675f,  0.4955f,-0.0079f,-0.8686f,  0.5852f,-0.0804f,-0.8069f,  0.5833f,-0.0839f,-0.8079f,  0.6630f,-0.1550f,-0.7324f,  0.6610f,-0.1587f,-0.7334f,  0.7293f,-0.2271f,-0.6454f,  0.7272f,-0.2311f,-0.6463f,  0.7831f,-0.2956f,-0.5472f,  0.7809f,-0.2997f,-0.5480f,  0.8234f,-0.3590f,-0.4395f,  0.8211f,-0.3634f,-0.4401f,  0.8495f,-0.4164f,-0.3240f,  0.8470f,-0.4210f,-0.3245f,  
	0.8609f,-0.4667f,-0.2027f,  0.8583f,-0.4713f,-0.2030f,  0.8574f,-0.5088f,-0.0777f,  0.8546f,-0.5135f,-0.0777f,  0.8390f,-0.5419f,0.0490f,  0.8360f,-0.5466f,0.0493f,  0.8061f,-0.5652f,0.1753f,  0.8028f,-0.5698f,0.1758f,  0.7593f,-0.5781f,0.2989f,  0.7556f,-0.5824f,0.2998f,  0.6992f,-0.5802f,0.4178f,  0.6953f,-0.5840f,0.4189f,  -0.0815f,-0.6220f,0.7787f,  -0.1946f,-0.5389f,0.8196f,  -0.1933f,-0.5597f,0.8058f,  
	-0.3031f,-0.4667f,0.8309f,  -0.3019f,-0.4878f,0.8191f,  -0.4064f,-0.3864f,0.8279f,  -0.4052f,-0.4075f,0.8184f,  -0.5028f,-0.2996f,0.8108f,  -0.5016f,-0.3203f,0.8036f,  -0.5906f,-0.2076f,0.7798f,  -0.5894f,-0.2276f,0.7751f,  -0.6682f,-0.1121f,0.7355f,  -0.6672f,-0.1310f,0.7333f,  -0.7344f,-0.0147f,0.6786f,  -0.7334f,-0.0321f,0.6790f,  -0.7880f,0.0830f,0.6100f,  -0.7872f,0.0673f,0.6130f,  -0.8281f,0.1793f,0.5311f,  
	-0.8274f,0.1655f,0.5366f,  -0.8541f,0.2725f,0.4430f,  -0.8535f,0.2610f,0.4510f,  -0.8654f,0.3610f,0.3474f,  -0.8650f,0.3519f,0.3577f,  -0.8619f,0.4434f,0.2459f,  -0.8617f,0.4368f,0.2583f,  -0.8437f,0.5182f,0.1402f,  -0.8436f,0.5143f,0.1545f,  -0.8110f,0.5841f,0.0320f,  -0.8111f,0.5830f,0.0480f,  -0.7645f,0.6400f,-0.0766f,  -0.7647f,0.6416f,-0.0592f,  -0.7049f,0.6850f,-0.1840f,  -0.7053f,0.6894f,-0.1655f,  
	-0.6332f,0.7183f,-0.2882f,  -0.6338f,0.7253f,-0.2689f,  -0.5508f,0.7393f,-0.3874f,  -0.5514f,0.7488f,-0.3677f,  -0.4589f,0.7477f,-0.4801f,  -0.4596f,0.7596f,-0.4602f,  -0.3591f,0.7432f,-0.5645f,  -0.3600f,0.7573f,-0.5448f,  -0.2532f,0.7261f,-0.6392f,  -0.2542f,0.7421f,-0.6202f,  -0.1430f,0.6966f,-0.7031f,  -0.1441f,0.7142f,-0.6849f,  -0.0304f,0.6551f,-0.7549f,  -0.0315f,0.6741f,-0.7379f,  0.0828f,0.6025f,-0.7938f,  
	0.0816f,0.6225f,-0.7783f,  0.1946f,0.5395f,-0.8192f,  0.1933f,0.5603f,-0.8054f,  0.3030f,0.4674f,-0.8305f,  0.3017f,0.4884f,-0.8188f,  0.4063f,0.3872f,-0.8277f,  0.4050f,0.4082f,-0.8181f,  0.5026f,0.3004f,-0.8107f,  0.5014f,0.3211f,-0.8035f,  0.5903f,0.2085f,-0.7798f,  0.5891f,0.2284f,-0.7751f,  0.6680f,0.1130f,-0.7356f,  0.6668f,0.1318f,-0.7334f,  0.7342f,0.0156f,-0.6788f,  0.7332f,0.0330f,-0.6793f,  
	0.7878f,-0.0821f,-0.6104f,  0.7869f,-0.0664f,-0.6134f,  0.8280f,-0.1784f,-0.5315f,  0.8273f,-0.1647f,-0.5371f,  0.8541f,-0.2716f,-0.4436f,  0.8534f,-0.2601f,-0.4516f,  0.8655f,-0.3602f,-0.3480f,  0.8650f,-0.3511f,-0.3584f,  0.8621f,-0.4427f,-0.2465f,  0.8618f,-0.4361f,-0.2590f,  0.8440f,-0.5175f,-0.1407f,  0.8439f,-0.5136f,-0.1552f,  0.8114f,-0.5835f,-0.0326f,  0.8115f,-0.5824f,-0.0487f,  0.7650f,-0.6395f,0.0762f,  
	0.7652f,-0.6411f,0.0587f,  0.7054f,-0.6846f,0.1836f,  0.7058f,-0.6889f,0.1650f,  0.7054f,-0.6846f,0.1836f,  0.7051f,-0.6799f,0.2013f,  0.6333f,-0.7107f,0.3063f,  0.6328f,-0.7032f,0.3240f,  0.5500f,-0.7196f,0.4240f,  0.5493f,-0.7099f,0.4409f,  0.4569f,-0.7118f,0.5334f,  0.4560f,-0.7003f,0.5493f,  0.3555f,-0.6884f,0.6323f,  0.3545f,-0.6755f,0.6466f,  0.2478f,-0.6504f,0.7180f,  0.2467f,-0.6366f,0.7307f,  
	0.1356f,-0.5994f,0.7889f,  0.1345f,-0.5851f,0.7997f,  0.0209f,-0.5370f,0.8433f,  0.0197f,-0.5227f,0.8523f,  -0.0943f,-0.4650f,0.8803f,  -0.0954f,-0.4510f,0.8874f,  -0.2080f,-0.3853f,0.8990f,  -0.2090f,-0.3721f,0.9044f,  -0.3180f,-0.3000f,0.8994f,  -0.3190f,-0.2877f,0.9031f,  -0.4226f,-0.2109f,0.8814f,  -0.4235f,-0.1997f,0.8836f,  -0.5200f,-0.1199f,0.8457f,  -0.5207f,-0.1101f,0.8466f,  -0.6083f,-0.0290f,0.7931f,  
	-0.6089f,-0.0206f,0.7930f,  -0.6862f,0.0600f,0.7249f,  -0.6867f,0.0670f,0.7238f,  -0.7524f,0.1456f,0.6425f,  -0.7528f,0.1512f,0.6407f,  -0.8056f,0.2263f,0.5475f,  -0.8060f,0.2304f,0.5452f,  -0.8451f,0.3006f,0.4420f,  -0.8455f,0.3035f,0.4394f,  -0.8702f,0.3675f,0.3281f,  -0.8706f,0.3691f,0.3253f,  -0.8806f,0.4258f,0.2079f,  -0.8810f,0.4264f,0.2051f,  -0.8761f,0.4749f,0.0837f,  -0.8765f,0.4745f,0.0809f,  
	-0.8567f,0.5140f,-0.0423f,  -0.8573f,0.5129f,-0.0449f,  -0.8230f,0.5428f,-0.1675f,  -0.8237f,0.5410f,-0.1700f,  -0.7755f,0.5609f,-0.2900f,  -0.7762f,0.5586f,-0.2923f,  -0.7149f,0.5682f,-0.4074f,  -0.7159f,0.5656f,-0.4094f,  -0.6425f,0.5649f,-0.5177f,  -0.6435f,0.5621f,-0.5196f,  -0.5594f,0.5512f,-0.6191f,  -0.5605f,0.5482f,-0.6207f,  -0.4669f,0.5276f,-0.7097f,  -0.4683f,0.5244f,-0.7111f,  -0.3668f,0.4944f,-0.7880f,  
	-0.3682f,0.4912f,-0.7893f,  -0.2607f,0.4525f,-0.8528f,  -0.2622f,0.4493f,-0.8540f,  -0.1503f,0.4025f,-0.9030f,  -0.1519f,0.3994f,-0.9041f,  -0.0376f,0.3455f,-0.9377f,  -0.0392f,0.3424f,-0.9387f,  0.0757f,0.2825f,-0.9563f,  0.0739f,0.2793f,-0.9573f,  0.1874f,0.2144f,-0.9586f,  0.1856f,0.2113f,-0.9596f,  0.2959f,0.1425f,-0.9445f,  0.2940f,0.1393f,-0.9456f,  0.3991f,0.0680f,-0.9144f,  0.3973f,0.0647f,-0.9154f,  
	0.4955f,-0.0079f,-0.8686f,  0.4936f,-0.0114f,-0.8696f,  0.5833f,-0.0839f,-0.8079f,  0.5813f,-0.0876f,-0.8090f,  0.6610f,-0.1587f,-0.7334f,  0.6589f,-0.1626f,-0.7344f,  0.7272f,-0.2311f,-0.6463f,  0.7251f,-0.2352f,-0.6472f,  0.7809f,-0.2997f,-0.5480f,  0.7787f,-0.3041f,-0.5488f,  0.8211f,-0.3634f,-0.4401f,  0.8187f,-0.3681f,-0.4407f,  0.8470f,-0.4210f,-0.3245f,  0.8445f,-0.4259f,-0.3249f,  0.8583f,-0.4713f,-0.2030f,  
	0.8554f,-0.4764f,-0.2031f,  0.8546f,-0.5135f,-0.0777f,  0.8515f,-0.5186f,-0.0775f,  0.8360f,-0.5466f,0.0493f,  0.8326f,-0.5517f,0.0498f,  0.8028f,-0.5698f,0.1758f,  0.7991f,-0.5747f,0.1767f,  0.7556f,-0.5824f,0.2998f,  0.7516f,-0.5870f,0.3009f,  0.6953f,-0.5840f,0.4189f,  0.6909f,-0.5881f,0.4204f,  -0.1946f,-0.5389f,0.8196f,  -0.3045f,-0.4464f,0.8414f,  -0.3031f,-0.4667f,0.8309f,  -0.4078f,-0.3663f,0.8364f,  
	-0.4064f,-0.3864f,0.8279f,  -0.5042f,-0.2799f,0.8170f,  -0.5028f,-0.2996f,0.8108f,  -0.5919f,-0.1887f,0.7836f,  -0.5906f,-0.2076f,0.7798f,  -0.6694f,-0.0943f,0.7369f,  -0.6682f,-0.1121f,0.7355f,  -0.7355f,0.0017f,0.6775f,  -0.7344f,-0.0147f,0.6786f,  -0.7890f,0.0977f,0.6065f,  -0.7880f,0.0830f,0.6100f,  -0.8290f,0.1920f,0.5252f,  -0.8281f,0.1793f,0.5311f,  -0.8549f,0.2831f,0.4349f,  -0.8541f,0.2725f,0.4430f,  
	-0.8660f,0.3693f,0.3371f,  -0.8654f,0.3610f,0.3474f,  -0.8624f,0.4491f,0.2336f,  -0.8619f,0.4434f,0.2459f,  -0.8440f,0.5213f,0.1261f,  -0.8437f,0.5182f,0.1402f,  -0.8111f,0.5846f,0.0165f,  -0.8110f,0.5841f,0.0320f,  -0.7644f,0.6379f,-0.0934f,  -0.7645f,0.6400f,-0.0766f,  -0.7046f,0.6803f,-0.2017f,  -0.7049f,0.6850f,-0.1840f,  -0.6328f,0.7110f,-0.3066f,  -0.6332f,0.7183f,-0.2882f,  -0.5502f,0.7296f,-0.4062f,  
	-0.5508f,0.7393f,-0.3874f,  -0.4581f,0.7357f,-0.4988f,  -0.4589f,0.7477f,-0.4801f,  -0.3582f,0.7293f,-0.5830f,  -0.3591f,0.7432f,-0.5645f,  -0.2522f,0.7103f,-0.6571f,  -0.2532f,0.7261f,-0.6392f,  -0.1419f,0.6793f,-0.7200f,  -0.1430f,0.6966f,-0.7031f,  -0.0292f,0.6366f,-0.7707f,  -0.0304f,0.6551f,-0.7549f,  0.0841f,0.5830f,-0.8081f,  0.0828f,0.6025f,-0.7938f,  0.1959f,0.5195f,-0.8317f,  0.1946f,0.5395f,-0.8192f,  
	0.3044f,0.4471f,-0.8411f,  0.3030f,0.4674f,-0.8305f,  0.4076f,0.3670f,-0.8362f,  0.4063f,0.3872f,-0.8277f,  0.5039f,0.2807f,-0.8169f,  0.5026f,0.3004f,-0.8107f,  0.5916f,0.1895f,-0.7836f,  0.5903f,0.2085f,-0.7798f,  0.6692f,0.0951f,-0.7370f,  0.6680f,0.1130f,-0.7356f,  0.7353f,-0.0009f,-0.6778f,  0.7342f,0.0156f,-0.6788f,  0.7888f,-0.0969f,-0.6069f,  0.7878f,-0.0821f,-0.6104f,  0.8289f,-0.1912f,-0.5257f,  
	0.8280f,-0.1784f,-0.5315f,  0.8548f,-0.2823f,-0.4354f,  0.8541f,-0.2716f,-0.4436f,  0.8661f,-0.3685f,-0.3377f,  0.8655f,-0.3602f,-0.3480f,  0.8626f,-0.4485f,-0.2342f,  0.8621f,-0.4427f,-0.2465f,  0.8443f,-0.5207f,-0.1267f,  0.8440f,-0.5175f,-0.1407f,  0.8115f,-0.5841f,-0.0171f,  0.8114f,-0.5835f,-0.0326f,  0.7649f,-0.6374f,0.0929f,  0.7650f,-0.6395f,0.0762f,  0.7051f,-0.6799f,0.2013f,  0.7054f,-0.6846f,0.1836f,  
	0.7051f,-0.6799f,0.2013f,  0.7048f,-0.6749f,0.2185f,  0.6328f,-0.7032f,0.3240f,  0.6324f,-0.6957f,0.3407f,  0.5493f,-0.7099f,0.4409f,  0.5487f,-0.7002f,0.4569f,  0.4560f,-0.7003f,0.5493f,  0.4552f,-0.6889f,0.5641f,  0.3545f,-0.6755f,0.6466f,  0.3536f,-0.6629f,0.6600f,  0.2467f,-0.6366f,0.7307f,  0.2458f,-0.6232f,0.7424f,  0.1345f,-0.5851f,0.7997f,  0.1335f,-0.5714f,0.8097f,  0.0197f,-0.5227f,0.8523f,  
	0.0188f,-0.5090f,0.8605f,  -0.0954f,-0.4510f,0.8874f,  -0.0963f,-0.4378f,0.8939f,  -0.2090f,-0.3721f,0.9044f,  -0.2098f,-0.3595f,0.9092f,  -0.3190f,-0.2877f,0.9031f,  -0.3197f,-0.2761f,0.9064f,  -0.4235f,-0.1997f,0.8836f,  -0.4241f,-0.1893f,0.8856f,  -0.5207f,-0.1101f,0.8466f,  -0.5212f,-0.1009f,0.8475f,  -0.6089f,-0.0206f,0.7930f,  -0.6093f,-0.0128f,0.7928f,  -0.6867f,0.0670f,0.7238f,  -0.6871f,0.0735f,0.7229f,  
	-0.7528f,0.1512f,0.6407f,  -0.7530f,0.1563f,0.6391f,  -0.8060f,0.2304f,0.5452f,  -0.8062f,0.2343f,0.5433f,  -0.8455f,0.3035f,0.4394f,  -0.8457f,0.3061f,0.4372f,  -0.8706f,0.3691f,0.3253f,  -0.8708f,0.3706f,0.3229f,  -0.8810f,0.4264f,0.2051f,  -0.8813f,0.4270f,0.2026f,  -0.8765f,0.4745f,0.0809f,  -0.8769f,0.4742f,0.0785f,  -0.8573f,0.5129f,-0.0449f,  -0.8578f,0.5119f,-0.0471f,  -0.8237f,0.5410f,-0.1700f,  
	-0.8243f,0.5394f,-0.1721f,  -0.7762f,0.5586f,-0.2923f,  -0.7770f,0.5566f,-0.2941f,  -0.7159f,0.5656f,-0.4094f,  -0.7167f,0.5633f,-0.4112f,  -0.6435f,0.5621f,-0.5196f,  -0.6446f,0.5595f,-0.5211f,  -0.5605f,0.5482f,-0.6207f,  -0.5617f,0.5455f,-0.6221f,  -0.4683f,0.5244f,-0.7111f,  -0.4695f,0.5216f,-0.7124f,  -0.3682f,0.4912f,-0.7893f,  -0.3696f,0.4884f,-0.7905f,  -0.2622f,0.4493f,-0.8540f,  -0.2637f,0.4464f,-0.8551f,  
	-0.1519f,0.3994f,-0.9041f,  -0.1535f,0.3965f,-0.9051f,  -0.0392f,0.3424f,-0.9387f,  -0.0409f,0.3395f,-0.9397f,  0.0739f,0.2793f,-0.9573f,  0.0722f,0.2764f,-0.9583f,  0.1856f,0.2113f,-0.9596f,  0.1839f,0.2082f,-0.9606f,  0.2940f,0.1393f,-0.9456f,  0.2923f,0.1361f,-0.9466f,  0.3973f,0.0647f,-0.9154f,  0.3955f,0.0613f,-0.9164f,  0.4936f,-0.0114f,-0.8696f,  0.4917f,-0.0150f,-0.8706f,  0.5813f,-0.0876f,-0.8090f,  
	0.5794f,-0.0914f,-0.8099f,  0.6589f,-0.1626f,-0.7344f,  0.6569f,-0.1668f,-0.7353f,  0.7251f,-0.2352f,-0.6472f,  0.7230f,-0.2397f,-0.6480f,  0.7787f,-0.3041f,-0.5488f,  0.7764f,-0.3089f,-0.5494f,  0.8187f,-0.3681f,-0.4407f,  0.8162f,-0.3731f,-0.4412f,  0.8445f,-0.4259f,-0.3249f,  0.8417f,-0.4311f,-0.3251f,  0.8554f,-0.4764f,-0.2031f,  0.8524f,-0.4818f,-0.2030f,  0.8515f,-0.5186f,-0.0775f,  0.8481f,-0.5242f,-0.0771f,  
	0.8326f,-0.5517f,0.0498f,  0.8289f,-0.5572f,0.0505f,  0.7991f,-0.5747f,0.1767f,  0.7950f,-0.5800f,0.1778f,  0.7516f,-0.5870f,0.3009f,  0.7471f,-0.5919f,0.3023f,  0.6909f,-0.5881f,0.4204f,  0.6861f,-0.5925f,0.4221f,  -0.3045f,-0.4464f,0.8414f,  -0.4094f,-0.3469f,0.8438f,  -0.4078f,-0.3663f,0.8364f,  -0.5057f,-0.2610f,0.8223f,  -0.5042f,-0.2799f,0.8170f,  -0.5934f,-0.1706f,0.7866f,  -0.5919f,-0.1887f,0.7836f,  
	-0.6709f,-0.0774f,0.7375f,  -0.6694f,-0.0943f,0.7369f,  -0.7369f,0.0173f,0.6758f,  -0.7355f,0.0017f,0.6775f,  -0.7902f,0.1116f,0.6026f,  -0.7890f,0.0977f,0.6065f,  -0.8301f,0.2040f,0.5190f,  -0.8290f,0.1920f,0.5252f,  -0.8557f,0.2929f,0.4265f,  -0.8549f,0.2831f,0.4349f,  -0.8667f,0.3768f,0.3268f,  -0.8660f,0.3693f,0.3371f,  -0.8629f,0.4543f,0.2215f,  -0.8624f,0.4491f,0.2336f,  -0.8443f,0.5240f,0.1124f,  
	-0.8440f,0.5213f,0.1261f,  -0.8113f,0.5847f,0.0014f,  -0.8111f,0.5846f,0.0165f,  -0.7643f,0.6354f,-0.1096f,  -0.7644f,0.6379f,-0.0934f,  -0.7043f,0.6753f,-0.2188f,  -0.7046f,0.6803f,-0.2017f,  -0.6323f,0.7036f,-0.3242f,  -0.6328f,0.7110f,-0.3066f,  -0.5495f,0.7199f,-0.4240f,  -0.5502f,0.7296f,-0.4062f,  -0.4572f,0.7239f,-0.5166f,  -0.4581f,0.7357f,-0.4988f,  -0.3572f,0.7155f,-0.6004f,  -0.3582f,0.7293f,-0.5830f,  
	-0.2510f,0.6949f,-0.6739f,  -0.2522f,0.7103f,-0.6571f,  -0.1406f,0.6624f,-0.7359f,  -0.1419f,0.6793f,-0.7200f,  -0.0277f,0.6186f,-0.7852f,  -0.0292f,0.6366f,-0.7707f,  0.0856f,0.5642f,-0.8212f,  0.0841f,0.5830f,-0.8081f,  0.1975f,0.5001f,-0.8431f,  0.1959f,0.5195f,-0.8317f,  0.3060f,0.4275f,-0.8507f,  0.3044f,0.4471f,-0.8411f,  0.4092f,0.3476f,-0.8436f,  0.4076f,0.3670f,-0.8362f,  0.5055f,0.2618f,-0.8222f,  
	0.5039f,0.2807f,-0.8169f,  0.5931f,0.1714f,-0.7867f,  0.5916f,0.1895f,-0.7836f,  0.6706f,0.0782f,-0.7377f,  0.6692f,0.0951f,-0.7370f,  0.7366f,-0.0164f,-0.6761f,  0.7353f,-0.0009f,-0.6778f,  0.7901f,-0.1108f,-0.6029f,  0.7888f,-0.0969f,-0.6069f,  0.8300f,-0.2032f,-0.5194f,  0.8289f,-0.1912f,-0.5257f,  0.8557f,-0.2921f,-0.4271f,  0.8548f,-0.2823f,-0.4354f,  0.8668f,-0.3761f,-0.3274f,  0.8661f,-0.3685f,-0.3377f,  
	0.8631f,-0.4536f,-0.2220f,  0.8626f,-0.4485f,-0.2342f,  0.8446f,-0.5234f,-0.1129f,  0.8443f,-0.5207f,-0.1267f,  0.8116f,-0.5842f,-0.0019f,  0.8115f,-0.5841f,-0.0171f,  0.7648f,-0.6350f,0.1092f,  0.7649f,-0.6374f,0.0929f,  0.7048f,-0.6749f,0.2185f,  0.7051f,-0.6799f,0.2013f,  0.7048f,-0.6749f,0.2185f,  0.7046f,-0.6697f,0.2348f,  0.6324f,-0.6957f,0.3407f,  0.6319f,-0.6881f,0.3567f,  0.5487f,-0.7002f,0.4569f,  
	0.5481f,-0.6906f,0.4719f,  0.4552f,-0.6889f,0.5641f,  0.4545f,-0.6778f,0.5779f,  0.3536f,-0.6629f,0.6600f,  0.3528f,-0.6507f,0.6724f,  0.2458f,-0.6232f,0.7424f,  0.2449f,-0.6104f,0.7533f,  0.1335f,-0.5714f,0.8097f,  0.1326f,-0.5583f,0.8190f,  0.0188f,-0.5090f,0.8605f,  0.0179f,-0.4961f,0.8681f,  -0.0963f,-0.4378f,0.8939f,  -0.0971f,-0.4253f,0.8998f,  -0.2098f,-0.3595f,0.9092f,  -0.2105f,-0.3477f,0.9137f,  
	-0.3197f,-0.2761f,0.9064f,  -0.3203f,-0.2652f,0.9095f,  -0.4241f,-0.1893f,0.8856f,  -0.4245f,-0.1795f,0.8875f,  -0.5212f,-0.1009f,0.8475f,  -0.5215f,-0.0923f,0.8482f,  -0.6093f,-0.0128f,0.7928f,  -0.6096f,-0.0055f,0.7927f,  -0.6871f,0.0735f,0.7229f,  -0.6872f,0.0795f,0.7221f,  -0.7530f,0.1563f,0.6391f,  -0.7532f,0.1611f,0.6378f,  -0.8062f,0.2343f,0.5433f,  -0.8063f,0.2378f,0.5416f,  -0.8457f,0.3061f,0.4372f,  
	-0.8458f,0.3085f,0.4353f,  -0.8708f,0.3706f,0.3229f,  -0.8710f,0.3721f,0.3209f,  -0.8813f,0.4270f,0.2026f,  -0.8815f,0.4275f,0.2006f,  -0.8769f,0.4742f,0.0785f,  -0.8772f,0.4740f,0.0765f,  -0.8578f,0.5119f,-0.0471f,  -0.8582f,0.5110f,-0.0490f,  -0.8243f,0.5394f,-0.1721f,  -0.8248f,0.5380f,-0.1738f,  -0.7770f,0.5566f,-0.2941f,  -0.7776f,0.5548f,-0.2957f,  -0.7167f,0.5633f,-0.4112f,  -0.7175f,0.5612f,-0.4125f,  
	-0.6446f,0.5595f,-0.5211f,  -0.6455f,0.5572f,-0.5223f,  -0.5617f,0.5455f,-0.6221f,  -0.5628f,0.5431f,-0.6232f,  -0.4695f,0.5216f,-0.7124f,  -0.4707f,0.5191f,-0.7134f,  -0.3696f,0.4884f,-0.7905f,  -0.3709f,0.4859f,-0.7914f,  -0.2637f,0.4464f,-0.8551f,  -0.2651f,0.4439f,-0.8560f,  -0.1535f,0.3965f,-0.9051f,  -0.1550f,0.3939f,-0.9060f,  -0.0409f,0.3395f,-0.9397f,  -0.0424f,0.3368f,-0.9406f,  0.0722f,0.2764f,-0.9583f,  
	0.0706f,0.2736f,-0.9593f,  0.1839f,0.2082f,-0.9606f,  0.1823f,0.2053f,-0.9616f,  0.2923f,0.1361f,-0.9466f,  0.2906f,0.1330f,-0.9476f,  0.3955f,0.0613f,-0.9164f,  0.3937f,0.0579f,-0.9174f,  0.4917f,-0.0150f,-0.8706f,  0.4899f,-0.0186f,-0.8716f,  0.5794f,-0.0914f,-0.8099f,  0.5774f,-0.0954f,-0.8108f,  0.6569f,-0.1668f,-0.7353f,  0.6549f,-0.1711f,-0.7361f,  0.7230f,-0.2397f,-0.6480f,  0.7208f,-0.2444f,-0.6487f,  
	0.7764f,-0.3089f,-0.5494f,  0.7740f,-0.3139f,-0.5499f,  0.8162f,-0.3731f,-0.4412f,  0.8136f,-0.3785f,-0.4414f,  0.8417f,-0.4311f,-0.3251f,  0.8388f,-0.4368f,-0.3251f,  0.8524f,-0.4818f,-0.2030f,  0.8492f,-0.4877f,-0.2027f,  0.8481f,-0.5242f,-0.0771f,  0.8445f,-0.5301f,-0.0765f,  0.8289f,-0.5572f,0.0505f,  0.8248f,-0.5630f,0.0516f,  0.7950f,-0.5800f,0.1778f,  0.7906f,-0.5856f,0.1791f,  0.7471f,-0.5919f,0.3023f,  
	0.7423f,-0.5971f,0.3041f,  0.6861f,-0.5925f,0.4221f,  0.6809f,-0.5971f,0.4242f,  -0.4094f,-0.3469f,0.8438f,  -0.5073f,-0.2431f,0.8268f,  -0.5057f,-0.2610f,0.8223f,  -0.5950f,-0.1535f,0.7890f,  -0.5934f,-0.1706f,0.7866f,  -0.6724f,-0.0614f,0.7377f,  -0.6709f,-0.0774f,0.7375f,  -0.7383f,0.0318f,0.6738f,  -0.7369f,0.0173f,0.6758f,  -0.7915f,0.1245f,0.5983f,  -0.7902f,0.1116f,0.6026f,  -0.8312f,0.2150f,0.5127f,  
	-0.8301f,0.2040f,0.5190f,  -0.8567f,0.3019f,0.4182f,  -0.8557f,0.2929f,0.4265f,  -0.8675f,0.3836f,0.3166f,  -0.8667f,0.3768f,0.3268f,  -0.8635f,0.4587f,0.2097f,  -0.8629f,0.4543f,0.2215f,  -0.8447f,0.5260f,0.0991f,  -0.8443f,0.5240f,0.1124f,  -0.8115f,0.5843f,-0.0132f,  -0.8113f,0.5847f,0.0014f,  -0.7643f,0.6326f,-0.1252f,  -0.7643f,0.6354f,-0.1096f,  -0.7041f,0.6700f,-0.2351f,  -0.7043f,0.6753f,-0.2188f,  
	-0.6319f,0.6961f,-0.3409f,  -0.6323f,0.7036f,-0.3242f,  -0.5488f,0.7102f,-0.4409f,  -0.5495f,0.7199f,-0.4240f,  -0.4564f,0.7122f,-0.5334f,  -0.4572f,0.7239f,-0.5166f,  -0.3562f,0.7020f,-0.6167f,  -0.3572f,0.7155f,-0.6004f,  -0.2499f,0.6798f,-0.6895f,  -0.2510f,0.6949f,-0.6739f,  -0.1393f,0.6460f,-0.7506f,  -0.1406f,0.6624f,-0.7359f,  -0.0263f,0.6011f,-0.7987f,  -0.0277f,0.6186f,-0.7852f,  0.0871f,0.5460f,-0.8333f,  
	0.0856f,0.5642f,-0.8212f,  0.1991f,0.4815f,-0.8536f,  0.1975f,0.5001f,-0.8431f,  0.3076f,0.4088f,-0.8592f,  0.3060f,0.4275f,-0.8507f,  0.4109f,0.3291f,-0.8502f,  0.4092f,0.3476f,-0.8436f,  0.5071f,0.2438f,-0.8267f,  0.5055f,0.2618f,-0.8222f,  0.5947f,0.1543f,-0.7890f,  0.5931f,0.1714f,-0.7867f,  0.6721f,0.0621f,-0.7378f,  0.6706f,0.0782f,-0.7377f,  0.7381f,-0.0311f,-0.6740f,  0.7366f,-0.0164f,-0.6761f,  
	0.7914f,-0.1237f,-0.5987f,  0.7901f,-0.1108f,-0.6029f,  0.8311f,-0.2143f,-0.5131f,  0.8300f,-0.2032f,-0.5194f,  0.8567f,-0.3012f,-0.4187f,  0.8557f,-0.2921f,-0.4271f,  0.8676f,-0.3829f,-0.3172f,  0.8668f,-0.3761f,-0.3274f,  0.8637f,-0.4581f,-0.2102f,  0.8631f,-0.4536f,-0.2220f,  0.8450f,-0.5254f,-0.0996f,  0.8446f,-0.5234f,-0.1129f,  0.8118f,-0.5838f,0.0127f,  0.8116f,-0.5842f,-0.0019f,  0.7647f,-0.6321f,0.1248f,  
	0.7648f,-0.6350f,0.1092f,  0.7046f,-0.6697f,0.2348f,  0.7048f,-0.6749f,0.2185f,  0.7046f,-0.6697f,0.2348f,  0.7044f,-0.6642f,0.2503f,  0.6319f,-0.6881f,0.3567f,  0.6316f,-0.6805f,0.3716f,  0.5481f,-0.6906f,0.4719f,  0.5475f,-0.6812f,0.4860f,  0.4545f,-0.6778f,0.5779f,  0.4539f,-0.6670f,0.5909f,  0.3528f,-0.6507f,0.6724f,  0.3522f,-0.6389f,0.6840f,  0.2449f,-0.6104f,0.7533f,  0.2442f,-0.5980f,0.7634f,  
	0.1326f,-0.5583f,0.8190f,  0.1319f,-0.5458f,0.8275f,  0.0179f,-0.4961f,0.8681f,  0.0173f,-0.4837f,0.8751f,  -0.0971f,-0.4253f,0.8998f,  -0.0976f,-0.4134f,0.9053f,  -0.2105f,-0.3477f,0.9137f,  -0.2109f,-0.3366f,0.9177f,  -0.3203f,-0.2652f,0.9095f,  -0.3206f,-0.2550f,0.9123f,  -0.4245f,-0.1795f,0.8875f,  -0.4247f,-0.1703f,0.8891f,  -0.5215f,-0.0923f,0.8482f,  -0.5216f,-0.0843f,0.8490f,  -0.6096f,-0.0055f,0.7927f,  
	-0.6096f,0.0014f,0.7927f,  -0.6872f,0.0795f,0.7221f,  -0.6872f,0.0851f,0.7215f,  -0.7532f,0.1611f,0.6378f,  -0.7531f,0.1655f,0.6367f,  -0.8063f,0.2378f,0.5416f,  -0.8063f,0.2411f,0.5402f,  -0.8458f,0.3085f,0.4353f,  -0.8458f,0.3108f,0.4337f,  -0.8710f,0.3721f,0.3209f,  -0.8710f,0.3734f,0.3192f,  -0.8815f,0.4275f,0.2006f,  -0.8816f,0.4280f,0.1989f,  -0.8772f,0.4740f,0.0765f,  -0.8774f,0.4739f,0.0749f,  
	-0.8582f,0.5110f,-0.0490f,  -0.8585f,0.5103f,-0.0505f,  -0.8248f,0.5380f,-0.1738f,  -0.8253f,0.5369f,-0.1752f,  -0.7776f,0.5548f,-0.2957f,  -0.7783f,0.5533f,-0.2970f,  -0.7175f,0.5612f,-0.4125f,  -0.7183f,0.5594f,-0.4137f,  -0.6455f,0.5572f,-0.5223f,  -0.6464f,0.5552f,-0.5233f,  -0.5628f,0.5431f,-0.6232f,  -0.5638f,0.5410f,-0.6241f,  -0.4707f,0.5191f,-0.7134f,  -0.4719f,0.5169f,-0.7142f,  -0.3709f,0.4859f,-0.7914f,  
	-0.3722f,0.4836f,-0.7922f,  -0.2651f,0.4439f,-0.8560f,  -0.2664f,0.4415f,-0.8568f,  -0.1550f,0.3939f,-0.9060f,  -0.1563f,0.3915f,-0.9068f,  -0.0424f,0.3368f,-0.9406f,  -0.0439f,0.3343f,-0.9415f,  0.0706f,0.2736f,-0.9593f,  0.0691f,0.2709f,-0.9601f,  0.1823f,0.2053f,-0.9616f,  0.1807f,0.2024f,-0.9625f,  0.2906f,0.1330f,-0.9476f,  0.2890f,0.1298f,-0.9485f,  0.3937f,0.0579f,-0.9174f,  0.3920f,0.0545f,-0.9183f,  
	0.4899f,-0.0186f,-0.8716f,  0.4881f,-0.0225f,-0.8725f,  0.5774f,-0.0954f,-0.8108f,  0.5756f,-0.0996f,-0.8117f,  0.6549f,-0.1711f,-0.7361f,  0.6528f,-0.1757f,-0.7369f,  0.7208f,-0.2444f,-0.6487f,  0.7185f,-0.2494f,-0.6492f,  0.7740f,-0.3139f,-0.5499f,  0.7715f,-0.3194f,-0.5503f,  0.8136f,-0.3785f,-0.4414f,  0.8108f,-0.3843f,-0.4415f,  0.8388f,-0.4368f,-0.3251f,  0.8357f,-0.4429f,-0.3249f,  0.8492f,-0.4877f,-0.2027f,  
	0.8457f,-0.4939f,-0.2022f,  0.8445f,-0.5301f,-0.0765f,  0.8406f,-0.5364f,-0.0755f,  0.8248f,-0.5630f,0.0516f,  0.8205f,-0.5692f,0.0529f,  0.7906f,-0.5856f,0.1791f,  0.7857f,-0.5916f,0.1809f,  0.7423f,-0.5971f,0.3041f,  0.7369f,-0.6026f,0.3062f,  0.6809f,-0.5971f,0.4242f,  0.6751f,-0.6019f,0.4266f,  -0.5073f,-0.2431f,0.8268f,  -0.5966f,-0.1373f,0.7907f,  -0.5950f,-0.1535f,0.7890f,  -0.6740f,-0.0462f,0.7373f,  
	-0.6724f,-0.0614f,0.7377f,  -0.7398f,0.0456f,0.6713f,  -0.7383f,0.0318f,0.6738f,  -0.7929f,0.1366f,0.5939f,  -0.7915f,0.1245f,0.5983f,  -0.8325f,0.2253f,0.5062f,  -0.8312f,0.2150f,0.5127f,  -0.8578f,0.3101f,0.4099f,  -0.8567f,0.3019f,0.4182f,  -0.8684f,0.3897f,0.3067f,  -0.8675f,0.3836f,0.3166f,  -0.8642f,0.4626f,0.1981f,  -0.8635f,0.4587f,0.2097f,  -0.8452f,0.5275f,0.0862f,  -0.8447f,0.5260f,0.0991f,  
	-0.8117f,0.5835f,-0.0272f,  -0.8115f,0.5843f,-0.0132f,  -0.7643f,0.6294f,-0.1401f,  -0.7643f,0.6326f,-0.1252f,  -0.7039f,0.6646f,-0.2506f,  -0.7041f,0.6700f,-0.2351f,  -0.6314f,0.6885f,-0.3568f,  -0.6319f,0.6961f,-0.3409f,  -0.5482f,0.7005f,-0.4569f,  -0.5488f,0.7102f,-0.4409f,  -0.4556f,0.7006f,-0.5492f,  -0.4564f,0.7122f,-0.5334f,  -0.3551f,0.6887f,-0.6321f,  -0.3562f,0.7020f,-0.6167f,  -0.2486f,0.6651f,-0.7042f,  
	-0.2499f,0.6798f,-0.6895f,  -0.1379f,0.6300f,-0.7642f,  -0.1393f,0.6460f,-0.7506f,  -0.0248f,0.5842f,-0.8112f,  -0.0263f,0.6011f,-0.7987f,  0.0887f,0.5284f,-0.8443f,  0.0871f,0.5460f,-0.8333f,  0.2007f,0.4636f,-0.8630f,  0.1991f,0.4815f,-0.8536f,  0.3093f,0.3908f,-0.8669f,  0.3076f,0.4088f,-0.8592f,  0.4126f,0.3114f,-0.8560f,  0.4109f,0.3291f,-0.8502f,  0.5088f,0.2266f,-0.8305f,  0.5071f,0.2438f,-0.8267f,  
	0.5964f,0.1380f,-0.7907f,  0.5947f,0.1543f,-0.7890f,  0.6737f,0.0470f,-0.7375f,  0.6721f,0.0621f,-0.7378f,  0.7396f,-0.0448f,-0.6716f,  0.7381f,-0.0311f,-0.6740f,  0.7928f,-0.1359f,-0.5942f,  0.7914f,-0.1237f,-0.5987f,  0.8324f,-0.2246f,-0.5066f,  0.8311f,-0.2143f,-0.5131f,  0.8578f,-0.3095f,-0.4104f,  0.8567f,-0.3012f,-0.4187f,  0.8685f,-0.3890f,-0.3071f,  0.8676f,-0.3829f,-0.3172f,  0.8644f,-0.4620f,-0.1986f,  
	0.8637f,-0.4581f,-0.2102f,  0.8454f,-0.5270f,-0.0867f,  0.8450f,-0.5254f,-0.0996f,  0.8120f,-0.5830f,0.0268f,  0.8118f,-0.5838f,0.0127f,  0.7648f,-0.6290f,0.1397f,  0.7647f,-0.6321f,0.1248f,  0.7044f,-0.6642f,0.2503f,  0.7046f,-0.6697f,0.2348f,  0.7044f,-0.6642f,0.2503f,  0.7042f,-0.6587f,0.2650f,  0.6316f,-0.6805f,0.3716f,  0.6312f,-0.6728f,0.3858f,  0.5475f,-0.6812f,0.4860f,  0.5471f,-0.6719f,0.4992f,  
	0.4539f,-0.6670f,0.5909f,  0.4534f,-0.6565f,0.6029f,  0.3522f,-0.6389f,0.6840f,  0.3516f,-0.6275f,0.6947f,  0.2442f,-0.5980f,0.7634f,  0.2437f,-0.5862f,0.7726f,  0.1319f,-0.5458f,0.8275f,  0.1314f,-0.5339f,0.8353f,  0.0173f,-0.4837f,0.8751f,  0.0169f,-0.4720f,0.8814f,  -0.0976f,-0.4134f,0.9053f,  -0.0980f,-0.4022f,0.9103f,  -0.2109f,-0.3366f,0.9177f,  -0.2112f,-0.3261f,0.9215f,  -0.3206f,-0.2550f,0.9123f,  
	-0.3207f,-0.2454f,0.9148f,  -0.4247f,-0.1703f,0.8891f,  -0.4248f,-0.1617f,0.8907f,  -0.5216f,-0.0843f,0.8490f,  -0.5216f,-0.0768f,0.8497f,  -0.6096f,0.0014f,0.7927f,  -0.6095f,0.0077f,0.7927f,  -0.6872f,0.0851f,0.7215f,  -0.6870f,0.0903f,0.7210f,  -0.7531f,0.1655f,0.6367f,  -0.7529f,0.1696f,0.6359f,  -0.8063f,0.2411f,0.5402f,  -0.8061f,0.2442f,0.5391f,  -0.8458f,0.3108f,0.4337f,  -0.8456f,0.3130f,0.4324f,  
	-0.8710f,0.3734f,0.3192f,  -0.8709f,0.3747f,0.3179f,  -0.8816f,0.4280f,0.1989f,  -0.8816f,0.4286f,0.1976f,  -0.8774f,0.4739f,0.0749f,  -0.8775f,0.4738f,0.0736f,  -0.8585f,0.5103f,-0.0505f,  -0.8588f,0.5098f,-0.0517f,  -0.8253f,0.5369f,-0.1752f,  -0.8256f,0.5359f,-0.1763f,  -0.7783f,0.5533f,-0.2970f,  -0.7788f,0.5520f,-0.2979f,  -0.7183f,0.5594f,-0.4137f,  -0.7189f,0.5579f,-0.4145f,  -0.6464f,0.5552f,-0.5233f,  
	-0.6472f,0.5536f,-0.5241f,  -0.5638f,0.5410f,-0.6241f,  -0.5647f,0.5392f,-0.6248f,  -0.4719f,0.5169f,-0.7142f,  -0.4729f,0.5150f,-0.7149f,  -0.3722f,0.4836f,-0.7922f,  -0.3733f,0.4816f,-0.7929f,  -0.2664f,0.4415f,-0.8568f,  -0.2676f,0.4394f,-0.8575f,  -0.1563f,0.3915f,-0.9068f,  -0.1576f,0.3892f,-0.9076f,  -0.0439f,0.3343f,-0.9415f,  -0.0452f,0.3319f,-0.9422f,  0.0691f,0.2709f,-0.9601f,  0.0677f,0.2683f,-0.9610f,  
	0.1807f,0.2024f,-0.9625f,  0.1793f,0.1995f,-0.9634f,  0.2890f,0.1298f,-0.9485f,  0.2874f,0.1266f,-0.9494f,  0.3920f,0.0545f,-0.9183f,  0.3904f,0.0509f,-0.9192f,  0.4881f,-0.0225f,-0.8725f,  0.4864f,-0.0264f,-0.8733f,  0.5756f,-0.0996f,-0.8117f,  0.5737f,-0.1041f,-0.8124f,  0.6528f,-0.1757f,-0.7369f,  0.6508f,-0.1806f,-0.7375f,  0.7185f,-0.2494f,-0.6492f,  0.7163f,-0.2548f,-0.6497f,  0.7715f,-0.3194f,-0.5503f,  
	0.7689f,-0.3252f,-0.5504f,  0.8108f,-0.3843f,-0.4415f,  0.8079f,-0.3905f,-0.4414f,  0.8357f,-0.4429f,-0.3249f,  0.8324f,-0.4494f,-0.3244f,  0.8457f,-0.4939f,-0.2022f,  0.8419f,-0.5006f,-0.2013f,  0.8406f,-0.5364f,-0.0755f,  0.8363f,-0.5432f,-0.0743f,  0.8205f,-0.5692f,0.0529f,  0.8157f,-0.5759f,0.0545f,  0.7857f,-0.5916f,0.1809f,  0.7804f,-0.5979f,0.1829f,  0.7369f,-0.6026f,0.3062f,  0.7312f,-0.6084f,0.3087f,  
	0.6751f,-0.6019f,0.4266f,  0.6688f,-0.6069f,0.4294f,  -0.5966f,-0.1373f,0.7907f,  -0.6756f,-0.0321f,0.7366f,  -0.6740f,-0.0462f,0.7373f,  -0.7413f,0.0584f,0.6687f,  -0.7398f,0.0456f,0.6713f,  -0.7943f,0.1478f,0.5893f,  -0.7929f,0.1366f,0.5939f,  -0.8337f,0.2347f,0.4998f,  -0.8325f,0.2253f,0.5062f,  -0.8589f,0.3176f,0.4018f,  -0.8578f,0.3101f,0.4099f,  -0.8693f,0.3951f,0.2970f,  -0.8684f,0.3897f,0.3067f,  
	-0.8649f,0.4658f,0.1870f,  -0.8642f,0.4626f,0.1981f,  -0.8457f,0.5285f,0.0739f,  -0.8452f,0.5275f,0.0862f,  -0.8120f,0.5822f,-0.0405f,  -0.8117f,0.5835f,-0.0272f,  -0.7644f,0.6260f,-0.1542f,  -0.7643f,0.6294f,-0.1401f,  -0.7038f,0.6590f,-0.2653f,  -0.7039f,0.6646f,-0.2506f,  -0.6311f,0.6808f,-0.3718f,  -0.6314f,0.6885f,-0.3568f,  -0.5476f,0.6909f,-0.4719f,  -0.5482f,0.7005f,-0.4569f,  -0.4548f,0.6893f,-0.5640f,  
	-0.4556f,0.7006f,-0.5492f,  -0.3542f,0.6758f,-0.6464f,  -0.3551f,0.6887f,-0.6321f,  -0.2475f,0.6508f,-0.7178f,  -0.2486f,0.6651f,-0.7042f,  -0.1366f,0.6146f,-0.7769f,  -0.1379f,0.6300f,-0.7642f,  -0.0234f,0.5680f,-0.8227f,  -0.0248f,0.5842f,-0.8112f,  0.0902f,0.5116f,-0.8545f,  0.0887f,0.5284f,-0.8443f,  0.2023f,0.4465f,-0.8716f,  0.2007f,0.4636f,-0.8630f,  0.3110f,0.3737f,-0.8739f,  0.3093f,0.3908f,-0.8669f,  
	0.4143f,0.2946f,-0.8611f,  0.4126f,0.3114f,-0.8560f,  0.5105f,0.2104f,-0.8337f,  0.5088f,0.2266f,-0.8305f,  0.5981f,0.1226f,-0.7920f,  0.5964f,0.1380f,-0.7907f,  0.6753f,0.0328f,-0.7368f,  0.6737f,0.0470f,-0.7375f,  0.7411f,-0.0577f,-0.6689f,  0.7396f,-0.0448f,-0.6716f,  0.7942f,-0.1471f,-0.5896f,  0.7928f,-0.1359f,-0.5942f,  0.8337f,-0.2341f,-0.5002f,  0.8324f,-0.2246f,-0.5066f,  0.8589f,-0.3170f,-0.4023f,  
	0.8578f,-0.3095f,-0.4104f,  0.8694f,-0.3945f,-0.2974f,  0.8685f,-0.3890f,-0.3071f,  0.8651f,-0.4653f,-0.1875f,  0.8644f,-0.4620f,-0.1986f,  0.8460f,-0.5280f,-0.0743f,  0.8454f,-0.5270f,-0.0867f,  0.8123f,-0.5818f,0.0401f,  0.8120f,-0.5830f,0.0268f,  0.7648f,-0.6256f,0.1539f,  0.7648f,-0.6290f,0.1397f,  0.7042f,-0.6587f,0.2650f,  0.7044f,-0.6642f,0.2503f,  0.7042f,-0.6587f,0.2650f,  0.7041f,-0.6530f,0.2790f,  
	0.6312f,-0.6728f,0.3858f,  0.6310f,-0.6653f,0.3990f,  0.5471f,-0.6719f,0.4992f,  0.5468f,-0.6628f,0.5116f,  0.4534f,-0.6565f,0.6029f,  0.4530f,-0.6462f,0.6141f,  0.3516f,-0.6275f,0.6947f,  0.3512f,-0.6166f,0.7046f,  0.2437f,-0.5862f,0.7726f,  0.2433f,-0.5749f,0.7812f,  0.1314f,-0.5339f,0.8353f,  0.1311f,-0.5226f,0.8425f,  0.0169f,-0.4720f,0.8814f,  0.0166f,-0.4609f,0.8873f,  -0.0980f,-0.4022f,0.9103f,  
	-0.0981f,-0.3916f,0.9149f,  -0.2112f,-0.3261f,0.9215f,  -0.2112f,-0.3162f,0.9249f,  -0.3207f,-0.2454f,0.9148f,  -0.3207f,-0.2364f,0.9172f,  -0.4248f,-0.1617f,0.8907f,  -0.4246f,-0.1537f,0.8922f,  -0.5216f,-0.0768f,0.8497f,  -0.5214f,-0.0699f,0.8505f,  -0.6095f,0.0077f,0.7927f,  -0.6092f,0.0136f,0.7929f,  -0.6870f,0.0903f,0.7210f,  -0.6867f,0.0952f,0.7207f,  -0.7529f,0.1696f,0.6359f,  -0.7526f,0.1735f,0.6352f,  
	-0.8061f,0.2442f,0.5391f,  -0.8058f,0.2471f,0.5382f,  -0.8456f,0.3130f,0.4324f,  -0.8454f,0.3150f,0.4314f,  -0.8709f,0.3747f,0.3179f,  -0.8708f,0.3760f,0.3169f,  -0.8816f,0.4286f,0.1976f,  -0.8815f,0.4293f,0.1966f,  -0.8775f,0.4738f,0.0736f,  -0.8776f,0.4739f,0.0727f,  -0.8588f,0.5098f,-0.0517f,  -0.8589f,0.5094f,-0.0526f,  -0.8256f,0.5359f,-0.1763f,  -0.8259f,0.5352f,-0.1771f,  -0.7788f,0.5520f,-0.2979f,  
	-0.7792f,0.5510f,-0.2986f,  -0.7189f,0.5579f,-0.4145f,  -0.7195f,0.5567f,-0.4152f,  -0.6472f,0.5536f,-0.5241f,  -0.6479f,0.5522f,-0.5247f,  -0.5647f,0.5392f,-0.6248f,  -0.5655f,0.5377f,-0.6254f,  -0.4729f,0.5150f,-0.7149f,  -0.4738f,0.5134f,-0.7155f,  -0.3733f,0.4816f,-0.7929f,  -0.3743f,0.4799f,-0.7935f,  -0.2676f,0.4394f,-0.8575f,  -0.2687f,0.4376f,-0.8581f,  -0.1576f,0.3892f,-0.9076f,  -0.1587f,0.3872f,-0.9082f,  
	-0.0452f,0.3319f,-0.9422f,  -0.0464f,0.3296f,-0.9430f,  0.0677f,0.2683f,-0.9610f,  0.0665f,0.2657f,-0.9617f,  0.1793f,0.1995f,-0.9634f,  0.1779f,0.1966f,-0.9642f,  0.2874f,0.1266f,-0.9494f,  0.2861f,0.1234f,-0.9502f,  0.3904f,0.0509f,-0.9192f,  0.3889f,0.0472f,-0.9201f,  0.4864f,-0.0264f,-0.8733f,  0.4848f,-0.0306f,-0.8741f,  0.5737f,-0.1041f,-0.8124f,  0.5719f,-0.1087f,-0.8131f,  0.6508f,-0.1806f,-0.7375f,  
	0.6487f,-0.1858f,-0.7380f,  0.7163f,-0.2548f,-0.6497f,  0.7139f,-0.2605f,-0.6499f,  0.7689f,-0.3252f,-0.5504f,  0.7663f,-0.3314f,-0.5504f,  0.8079f,-0.3905f,-0.4414f,  0.8048f,-0.3971f,-0.4411f,  0.8324f,-0.4494f,-0.3244f,  0.8289f,-0.4563f,-0.3237f,  0.8419f,-0.5006f,-0.2013f,  0.8379f,-0.5078f,-0.2002f,  0.8363f,-0.5432f,-0.0743f,  0.8318f,-0.5503f,-0.0727f,  0.8157f,-0.5759f,0.0545f,  0.8106f,-0.5829f,0.0566f,  
	0.7804f,-0.5979f,0.1829f,  0.7747f,-0.6045f,0.1854f,  0.7312f,-0.6084f,0.3087f,  0.7249f,-0.6144f,0.3115f,  0.6688f,-0.6069f,0.4294f,  0.6620f,-0.6120f,0.4326f,  -0.6756f,-0.0321f,0.7366f,  -0.7428f,0.0704f,0.6658f,  -0.7413f,0.0584f,0.6687f,  -0.7957f,0.1582f,0.5847f,  -0.7943f,0.1478f,0.5893f,  -0.8350f,0.2434f,0.4935f,  -0.8337f,0.2347f,0.4998f,  -0.8600f,0.3244f,0.3939f,  -0.8589f,0.3176f,0.4018f,  
	-0.8703f,0.3999f,0.2876f,  -0.8693f,0.3951f,0.2970f,  -0.8657f,0.4685f,0.1763f,  -0.8649f,0.4658f,0.1870f,  -0.8462f,0.5292f,0.0621f,  -0.8457f,0.5285f,0.0739f,  -0.8124f,0.5807f,-0.0532f,  -0.8120f,0.5822f,-0.0405f,  -0.7646f,0.6224f,-0.1676f,  -0.7644f,0.6260f,-0.1542f,  -0.7037f,0.6534f,-0.2792f,  -0.7038f,0.6590f,-0.2653f,  -0.6308f,0.6732f,-0.3859f,  -0.6311f,0.6808f,-0.3718f,  -0.5471f,0.6815f,-0.4860f,  
	-0.5476f,0.6909f,-0.4719f,  -0.4541f,0.6781f,-0.5779f,  -0.4548f,0.6893f,-0.5640f,  -0.3533f,0.6632f,-0.6598f,  -0.3542f,0.6758f,-0.6464f,  -0.2464f,0.6369f,-0.7305f,  -0.2475f,0.6508f,-0.7178f,  -0.1354f,0.5998f,-0.7886f,  -0.1366f,0.6146f,-0.7769f,  -0.0220f,0.5523f,-0.8333f,  -0.0234f,0.5680f,-0.8227f,  0.0917f,0.4955f,-0.8638f,  0.0902f,0.5116f,-0.8545f,  0.2039f,0.4301f,-0.8794f,  0.2023f,0.4465f,-0.8716f,  
	0.3126f,0.3575f,-0.8801f,  0.3110f,0.3737f,-0.8739f,  0.4160f,0.2786f,-0.8656f,  0.4143f,0.2946f,-0.8611f,  0.5122f,0.1951f,-0.8364f,  0.5105f,0.2104f,-0.8337f,  0.5997f,0.1082f,-0.7929f,  0.5981f,0.1226f,-0.7920f,  0.6770f,0.0194f,-0.7358f,  0.6753f,0.0328f,-0.7368f,  0.7426f,-0.0697f,-0.6661f,  0.7411f,-0.0577f,-0.6689f,  0.7956f,-0.1576f,-0.5850f,  0.7942f,-0.1471f,-0.5896f,  0.8349f,-0.2428f,-0.4939f,  
	0.8337f,-0.2341f,-0.5002f,  0.8600f,-0.3238f,-0.3943f,  0.8589f,-0.3170f,-0.4023f,  0.8704f,-0.3993f,-0.2880f,  0.8694f,-0.3945f,-0.2974f,  0.8659f,-0.4680f,-0.1768f,  0.8651f,-0.4653f,-0.1875f,  0.8465f,-0.5287f,-0.0625f,  0.8460f,-0.5280f,-0.0743f,  0.8127f,-0.5803f,0.0529f,  0.8123f,-0.5818f,0.0401f,  0.7650f,-0.6220f,0.1674f,  0.7648f,-0.6256f,0.1539f,  0.7041f,-0.6530f,0.2790f,  0.7042f,-0.6587f,0.2650f,  
	0.7041f,-0.6530f,0.2790f,  0.7041f,-0.6472f,0.2920f,  0.6310f,-0.6653f,0.3990f,  0.6309f,-0.6578f,0.4114f,  0.5468f,-0.6628f,0.5116f,  0.5466f,-0.6539f,0.5231f,  0.4530f,-0.6462f,0.6141f,  0.4528f,-0.6364f,0.6245f,  0.3512f,-0.6166f,0.7046f,  0.3510f,-0.6061f,0.7138f,  0.2433f,-0.5749f,0.7812f,  0.2431f,-0.5641f,0.7891f,  0.1311f,-0.5226f,0.8425f,  0.1310f,-0.5118f,0.8491f,  0.0166f,-0.4609f,0.8873f,  
	0.0166f,-0.4505f,0.8926f,  -0.0981f,-0.3916f,0.9149f,  -0.0981f,-0.3817f,0.9191f,  -0.2112f,-0.3162f,0.9249f,  -0.2110f,-0.3070f,0.9280f,  -0.3207f,-0.2364f,0.9172f,  -0.3204f,-0.2279f,0.9194f,  -0.4246f,-0.1537f,0.8922f,  -0.4243f,-0.1462f,0.8936f,  -0.5214f,-0.0699f,0.8505f,  -0.5209f,-0.0633f,0.8512f,  -0.6092f,0.0136f,0.7929f,  -0.6087f,0.0191f,0.7931f,  -0.6867f,0.0952f,0.7207f,  -0.6862f,0.0997f,0.7205f,  
	-0.7526f,0.1735f,0.6352f,  -0.7521f,0.1771f,0.6348f,  -0.8058f,0.2471f,0.5382f,  -0.8053f,0.2499f,0.5376f,  -0.8454f,0.3150f,0.4314f,  -0.8450f,0.3170f,0.4307f,  -0.8708f,0.3760f,0.3169f,  -0.8705f,0.3773f,0.3161f,  -0.8815f,0.4293f,0.1966f,  -0.8813f,0.4300f,0.1958f,  -0.8776f,0.4739f,0.0727f,  -0.8775f,0.4742f,0.0720f,  -0.8589f,0.5094f,-0.0526f,  -0.8590f,0.5092f,-0.0532f,  -0.8259f,0.5352f,-0.1771f,  
	-0.8261f,0.5347f,-0.1776f,  -0.7792f,0.5510f,-0.2986f,  -0.7795f,0.5503f,-0.2991f,  -0.7195f,0.5567f,-0.4152f,  -0.7200f,0.5558f,-0.4156f,  -0.6479f,0.5522f,-0.5247f,  -0.6484f,0.5511f,-0.5251f,  -0.5655f,0.5377f,-0.6254f,  -0.5662f,0.5365f,-0.6258f,  -0.4738f,0.5134f,-0.7155f,  -0.4745f,0.5121f,-0.7159f,  -0.3743f,0.4799f,-0.7935f,  -0.3751f,0.4784f,-0.7940f,  -0.2687f,0.4376f,-0.8581f,  -0.2696f,0.4359f,-0.8587f,  
	-0.1587f,0.3872f,-0.9082f,  -0.1597f,0.3853f,-0.9089f,  -0.0464f,0.3296f,-0.9430f,  -0.0474f,0.3275f,-0.9437f,  0.0665f,0.2657f,-0.9617f,  0.0654f,0.2633f,-0.9625f,  0.1779f,0.1966f,-0.9642f,  0.1768f,0.1937f,-0.9650f,  0.2861f,0.1234f,-0.9502f,  0.2848f,0.1200f,-0.9510f,  0.3889f,0.0472f,-0.9201f,  0.3875f,0.0434f,-0.9208f,  0.4848f,-0.0306f,-0.8741f,  0.4832f,-0.0350f,-0.8748f,  0.5719f,-0.1087f,-0.8131f,  
	0.5701f,-0.1137f,-0.8136f,  0.6487f,-0.1858f,-0.7380f,  0.6467f,-0.1914f,-0.7383f,  0.7139f,-0.2605f,-0.6499f,  0.7116f,-0.2666f,-0.6500f,  0.7663f,-0.3314f,-0.5504f,  0.7635f,-0.3380f,-0.5502f,  0.8048f,-0.3971f,-0.4411f,  0.8016f,-0.4042f,-0.4405f,  0.8289f,-0.4563f,-0.3237f,  0.8251f,-0.4637f,-0.3227f,  0.8379f,-0.5078f,-0.2002f,  0.8336f,-0.5154f,-0.1987f,  0.8318f,-0.5503f,-0.0727f,  0.8268f,-0.5580f,-0.0708f,  
	0.8106f,-0.5829f,0.0566f,  0.8050f,-0.5903f,0.0590f,  0.7747f,-0.6045f,0.1854f,  0.7685f,-0.6115f,0.1883f,  0.7249f,-0.6144f,0.3115f,  0.7181f,-0.6208f,0.3148f,  0.6620f,-0.6120f,0.4326f,  0.6547f,-0.6174f,0.4362f,  -0.7428f,0.0704f,0.6658f,  -0.7971f,0.1678f,0.5801f,  -0.7957f,0.1582f,0.5847f,  -0.8362f,0.2514f,0.4874f,  -0.8350f,0.2434f,0.4935f,  -0.8611f,0.3306f,0.3863f,  -0.8600f,0.3244f,0.3939f,  
	-0.8712f,0.4041f,0.2786f,  -0.8703f,0.3999f,0.2876f,  -0.8665f,0.4708f,0.1662f,  -0.8657f,0.4685f,0.1763f,  -0.8469f,0.5294f,0.0509f,  -0.8462f,0.5292f,0.0621f,  -0.8128f,0.5789f,-0.0653f,  -0.8124f,0.5807f,-0.0532f,  -0.7648f,0.6185f,-0.1803f,  -0.7646f,0.6224f,-0.1676f,  -0.7037f,0.6476f,-0.2922f,  -0.7037f,0.6534f,-0.2792f,  -0.6306f,0.6656f,-0.3991f,  -0.6308f,0.6732f,-0.3859f,  -0.5467f,0.6722f,-0.4992f,  
	-0.5471f,0.6815f,-0.4860f,  -0.4535f,0.6673f,-0.5908f,  -0.4541f,0.6781f,-0.5779f,  -0.3525f,0.6510f,-0.6723f,  -0.3533f,0.6632f,-0.6598f,  -0.2455f,0.6236f,-0.7422f,  -0.2464f,0.6369f,-0.7305f,  -0.1343f,0.5855f,-0.7995f,  -0.1354f,0.5998f,-0.7886f,  -0.0208f,0.5374f,-0.8431f,  -0.0220f,0.5523f,-0.8333f,  0.0931f,0.4801f,-0.8723f,  0.0917f,0.4955f,-0.8638f,  0.2054f,0.4146f,-0.8865f,  0.2039f,0.4301f,-0.8794f,  
	0.3141f,0.3420f,-0.8856f,  0.3126f,0.3575f,-0.8801f,  0.4175f,0.2636f,-0.8696f,  0.4160f,0.2786f,-0.8656f,  0.5138f,0.1806f,-0.8387f,  0.5122f,0.1951f,-0.8364f,  0.6013f,0.0946f,-0.7934f,  0.5997f,0.1082f,-0.7929f,  0.6785f,0.0069f,-0.7346f,  0.6770f,0.0194f,-0.7358f,  0.7441f,-0.0808f,-0.6632f,  0.7426f,-0.0697f,-0.6661f,  0.7970f,-0.1672f,-0.5804f,  0.7956f,-0.1576f,-0.5850f,  0.8362f,-0.2508f,-0.4877f,  
	0.8349f,-0.2428f,-0.4939f,  0.8611f,-0.3300f,-0.3867f,  0.8600f,-0.3238f,-0.3943f,  0.8714f,-0.4036f,-0.2790f,  0.8704f,-0.3993f,-0.2880f,  0.8667f,-0.4703f,-0.1666f,  0.8659f,-0.4680f,-0.1768f,  0.8471f,-0.5289f,-0.0513f,  0.8465f,-0.5287f,-0.0625f,  0.8131f,-0.5785f,0.0649f,  0.8127f,-0.5803f,0.0529f,  0.7652f,-0.6182f,0.1800f,  0.7650f,-0.6220f,0.1674f,  0.7041f,-0.6472f,0.2920f,  0.7041f,-0.6530f,0.2790f,  
	0.7041f,-0.6472f,0.2920f,  0.7042f,-0.6415f,0.3043f,  0.6309f,-0.6578f,0.4114f,  0.6309f,-0.6504f,0.4230f,  0.5466f,-0.6539f,0.5231f,  0.5466f,-0.6453f,0.5337f,  0.4528f,-0.6364f,0.6245f,  0.4527f,-0.6269f,0.6341f,  0.3510f,-0.6061f,0.7138f,  0.3509f,-0.5961f,0.7222f,  0.2431f,-0.5641f,0.7891f,  0.2431f,-0.5539f,0.7963f,  0.1310f,-0.5118f,0.8491f,  0.1310f,-0.5016f,0.8551f,  0.0166f,-0.4505f,0.8926f,  
	0.0168f,-0.4406f,0.8975f,  -0.0981f,-0.3817f,0.9191f,  -0.0978f,-0.3723f,0.9229f,  -0.2110f,-0.3070f,0.9280f,  -0.2107f,-0.2983f,0.9309f,  -0.3204f,-0.2279f,0.9194f,  -0.3200f,-0.2201f,0.9215f,  -0.4243f,-0.1462f,0.8936f,  -0.4238f,-0.1392f,0.8950f,  -0.5209f,-0.0633f,0.8512f,  -0.5204f,-0.0572f,0.8520f,  -0.6087f,0.0191f,0.7931f,  -0.6081f,0.0243f,0.7935f,  -0.6862f,0.0997f,0.7205f,  -0.6856f,0.1040f,0.7205f,  
	-0.7521f,0.1771f,0.6348f,  -0.7515f,0.1805f,0.6346f,  -0.8053f,0.2499f,0.5376f,  -0.8048f,0.2526f,0.5372f,  -0.8450f,0.3170f,0.4307f,  -0.8445f,0.3190f,0.4302f,  -0.8705f,0.3773f,0.3161f,  -0.8701f,0.3787f,0.3156f,  -0.8813f,0.4300f,0.1958f,  -0.8811f,0.4308f,0.1954f,  -0.8775f,0.4742f,0.0720f,  -0.8773f,0.4745f,0.0716f,  -0.8590f,0.5092f,-0.0532f,  -0.8589f,0.5093f,-0.0536f,  -0.8261f,0.5347f,-0.1776f,  
	-0.8262f,0.5345f,-0.1780f,  -0.7795f,0.5503f,-0.2991f,  -0.7797f,0.5499f,-0.2994f,  -0.7200f,0.5558f,-0.4156f,  -0.7203f,0.5552f,-0.4159f,  -0.6484f,0.5511f,-0.5251f,  -0.6489f,0.5504f,-0.5254f,  -0.5662f,0.5365f,-0.6258f,  -0.5667f,0.5356f,-0.6261f,  -0.4745f,0.5121f,-0.7159f,  -0.4751f,0.5110f,-0.7163f,  -0.3751f,0.4784f,-0.7940f,  -0.3758f,0.4771f,-0.7944f,  -0.2696f,0.4359f,-0.8587f,  -0.2703f,0.4344f,-0.8592f,  
	-0.1597f,0.3853f,-0.9089f,  -0.1605f,0.3836f,-0.9095f,  -0.0474f,0.3275f,-0.9437f,  -0.0482f,0.3254f,-0.9443f,  0.0654f,0.2633f,-0.9625f,  0.0645f,0.2608f,-0.9632f,  0.1768f,0.1937f,-0.9650f,  0.1758f,0.1908f,-0.9658f,  0.2848f,0.1200f,-0.9510f,  0.2837f,0.1166f,-0.9518f,  0.3875f,0.0434f,-0.9208f,  0.3863f,0.0393f,-0.9215f,  0.4832f,-0.0350f,-0.8748f,  0.4818f,-0.0397f,-0.8754f,  0.5701f,-0.1137f,-0.8136f,  
	0.5684f,-0.1190f,-0.8141f,  0.6467f,-0.1914f,-0.7383f,  0.6447f,-0.1973f,-0.7385f,  0.7116f,-0.2666f,-0.6500f,  0.7092f,-0.2732f,-0.6500f,  0.7635f,-0.3380f,-0.5502f,  0.7607f,-0.3451f,-0.5498f,  0.8016f,-0.4042f,-0.4405f,  0.7982f,-0.4117f,-0.4396f,  0.8251f,-0.4637f,-0.3227f,  0.8212f,-0.4716f,-0.3214f,  0.8336f,-0.5154f,-0.1987f,  0.8290f,-0.5235f,-0.1969f,  0.8268f,-0.5580f,-0.0708f,  0.8215f,-0.5660f,-0.0685f,  
	0.8050f,-0.5903f,0.0590f,  0.7990f,-0.5981f,0.0618f,  0.7685f,-0.6115f,0.1883f,  0.7618f,-0.6188f,0.1915f,  0.7181f,-0.6208f,0.3148f,  0.7107f,-0.6273f,0.3185f,  0.6547f,-0.6174f,0.4362f,  0.6467f,-0.6229f,0.4402f,  -0.7971f,0.1678f,0.5801f,  -0.8375f,0.2586f,0.4814f,  -0.8362f,0.2514f,0.4874f,  -0.8622f,0.3361f,0.3790f,  -0.8611f,0.3306f,0.3863f,  -0.8722f,0.4078f,0.2700f,  -0.8712f,0.4041f,0.2786f,  
	-0.8673f,0.4726f,0.1565f,  -0.8665f,0.4708f,0.1662f,  -0.8475f,0.5292f,0.0403f,  -0.8469f,0.5294f,0.0509f,  -0.8133f,0.5768f,-0.0766f,  -0.8128f,0.5789f,-0.0653f,  -0.7651f,0.6146f,-0.1922f,  -0.7648f,0.6185f,-0.1803f,  -0.7038f,0.6418f,-0.3045f,  -0.7037f,0.6476f,-0.2922f,  -0.6305f,0.6581f,-0.4115f,  -0.6306f,0.6656f,-0.3991f,  -0.5464f,0.6631f,-0.5116f,  -0.5467f,0.6722f,-0.4992f,  -0.4530f,0.6568f,-0.6029f,  
	-0.4535f,0.6673f,-0.5908f,  -0.3519f,0.6392f,-0.6838f,  -0.3525f,0.6510f,-0.6723f,  -0.2447f,0.6107f,-0.7531f,  -0.2455f,0.6236f,-0.7422f,  -0.1333f,0.5718f,-0.8095f,  -0.1343f,0.5855f,-0.7995f,  -0.0197f,0.5231f,-0.8521f,  -0.0208f,0.5374f,-0.8431f,  0.0943f,0.4654f,-0.8801f,  0.0931f,0.4801f,-0.8723f,  0.2067f,0.3998f,-0.8930f,  0.2054f,0.4146f,-0.8865f,  0.3156f,0.3274f,-0.8907f,  0.3141f,0.3420f,-0.8856f,  
	0.4190f,0.2493f,-0.8731f,  0.4175f,0.2636f,-0.8696f,  0.5153f,0.1670f,-0.8406f,  0.5138f,0.1806f,-0.8387f,  0.6028f,0.0818f,-0.7937f,  0.6013f,0.0946f,-0.7934f,  0.6799f,-0.0047f,-0.7332f,  0.6785f,0.0069f,-0.7346f,  0.7455f,-0.0912f,-0.6602f,  0.7441f,-0.0808f,-0.6632f,  0.7983f,-0.1761f,-0.5759f,  0.7970f,-0.1672f,-0.5804f,  0.8374f,-0.2581f,-0.4818f,  0.8362f,-0.2508f,-0.4877f,  0.8623f,-0.3356f,-0.3794f,  
	0.8611f,-0.3300f,-0.3867f,  0.8723f,-0.4073f,-0.2704f,  0.8714f,-0.4036f,-0.2790f,  0.8675f,-0.4721f,-0.1569f,  0.8667f,-0.4703f,-0.1666f,  0.8478f,-0.5288f,-0.0407f,  0.8471f,-0.5289f,-0.0513f,  0.8136f,-0.5764f,0.0763f,  0.8131f,-0.5785f,0.0649f,  0.7654f,-0.6142f,0.1919f,  0.7652f,-0.6182f,0.1800f,  0.7042f,-0.6415f,0.3043f,  0.7041f,-0.6472f,0.2920f,  0.7042f,-0.6415f,0.3043f,  0.7044f,-0.6357f,0.3158f,  
	0.6309f,-0.6504f,0.4230f,  0.6310f,-0.6432f,0.4338f,  0.5466f,-0.6453f,0.5337f,  0.5466f,-0.6370f,0.5436f,  0.4527f,-0.6269f,0.6341f,  0.4527f,-0.6178f,0.6429f,  0.3509f,-0.5961f,0.7222f,  0.3510f,-0.5865f,0.7300f,  0.2431f,-0.5539f,0.7963f,  0.2432f,-0.5441f,0.8030f,  0.1310f,-0.5016f,0.8551f,  0.1313f,-0.4920f,0.8607f,  0.0168f,-0.4406f,0.8975f,  0.0171f,-0.4313f,0.9021f,  -0.0978f,-0.3723f,0.9229f,  
	-0.0974f,-0.3635f,0.9265f,  -0.2107f,-0.2983f,0.9309f,  -0.2102f,-0.2901f,0.9336f,  -0.3200f,-0.2201f,0.9215f,  -0.3193f,-0.2127f,0.9235f,  -0.4238f,-0.1392f,0.8950f,  -0.4231f,-0.1326f,0.8963f,  -0.5204f,-0.0572f,0.8520f,  -0.5196f,-0.0515f,0.8528f,  -0.6081f,0.0243f,0.7935f,  -0.6073f,0.0292f,0.7939f,  -0.6856f,0.1040f,0.7205f,  -0.6848f,0.1081f,0.7207f,  -0.7515f,0.1805f,0.6346f,  -0.7508f,0.1838f,0.6345f,  
	-0.8048f,0.2526f,0.5372f,  -0.8041f,0.2551f,0.5370f,  -0.8445f,0.3190f,0.4302f,  -0.8439f,0.3209f,0.4300f,  -0.8701f,0.3787f,0.3156f,  -0.8695f,0.3801f,0.3154f,  -0.8811f,0.4308f,0.1954f,  -0.8806f,0.4317f,0.1951f,  -0.8773f,0.4745f,0.0716f,  -0.8770f,0.4751f,0.0714f,  -0.8589f,0.5093f,-0.0536f,  -0.8588f,0.5095f,-0.0538f,  -0.8262f,0.5345f,-0.1780f,  -0.8262f,0.5345f,-0.1781f,  -0.7797f,0.5499f,-0.2994f,  
	-0.7798f,0.5497f,-0.2996f,  -0.7203f,0.5552f,-0.4159f,  -0.7205f,0.5548f,-0.4161f,  -0.6489f,0.5504f,-0.5254f,  -0.6491f,0.5499f,-0.5256f,  -0.5667f,0.5356f,-0.6261f,  -0.5670f,0.5349f,-0.6264f,  -0.4751f,0.5110f,-0.7163f,  -0.4756f,0.5102f,-0.7166f,  -0.3758f,0.4771f,-0.7944f,  -0.3763f,0.4761f,-0.7948f,  -0.2703f,0.4344f,-0.8592f,  -0.2708f,0.4331f,-0.8597f,  -0.1605f,0.3836f,-0.9095f,  -0.1611f,0.3820f,-0.9100f,  
	-0.0482f,0.3254f,-0.9443f,  -0.0489f,0.3234f,-0.9450f,  0.0645f,0.2608f,-0.9632f,  0.0638f,0.2583f,-0.9639f,  0.1758f,0.1908f,-0.9658f,  0.1750f,0.1878f,-0.9665f,  0.2837f,0.1166f,-0.9518f,  0.2827f,0.1130f,-0.9525f,  0.3863f,0.0393f,-0.9215f,  0.3852f,0.0351f,-0.9222f,  0.4818f,-0.0397f,-0.8754f,  0.4804f,-0.0446f,-0.8759f,  0.5684f,-0.1190f,-0.8141f,  0.5668f,-0.1247f,-0.8144f,  0.6447f,-0.1973f,-0.7385f,  
	0.6427f,-0.2037f,-0.7386f,  0.7092f,-0.2732f,-0.6500f,  0.7067f,-0.2802f,-0.6496f,  0.7607f,-0.3451f,-0.5498f,  0.7577f,-0.3527f,-0.5491f,  0.7982f,-0.4117f,-0.4396f,  0.7947f,-0.4198f,-0.4385f,  0.8212f,-0.4716f,-0.3214f,  0.8169f,-0.4800f,-0.3197f,  0.8290f,-0.5235f,-0.1969f,  0.8240f,-0.5320f,-0.1947f,  0.8215f,-0.5660f,-0.0685f,  0.8158f,-0.5745f,-0.0657f,  0.7990f,-0.5981f,0.0618f,  0.7925f,-0.6064f,0.0650f,  
	0.7618f,-0.6188f,0.1915f,  0.7546f,-0.6265f,0.1953f,  0.7107f,-0.6273f,0.3185f,  0.7028f,-0.6341f,0.3226f,  0.6467f,-0.6229f,0.4402f,  0.6382f,-0.6285f,0.4447f,  -0.8375f,0.2586f,0.4814f,  -0.8633f,0.3411f,0.3720f,  -0.8622f,0.3361f,0.3790f,  -0.8732f,0.4110f,0.2619f,  -0.8722f,0.4078f,0.2700f,  -0.8681f,0.4740f,0.1474f,  -0.8673f,0.4726f,0.1565f,  -0.8482f,0.5288f,0.0303f,  -0.8475f,0.5292f,0.0403f,  
	-0.8138f,0.5746f,-0.0873f,  -0.8133f,0.5768f,-0.0766f,  -0.7654f,0.6105f,-0.2033f,  -0.7651f,0.6146f,-0.1922f,  -0.7040f,0.6361f,-0.3159f,  -0.7038f,0.6418f,-0.3045f,  -0.6305f,0.6507f,-0.4231f,  -0.6305f,0.6581f,-0.4115f,  -0.5463f,0.6542f,-0.5231f,  -0.5464f,0.6631f,-0.5116f,  -0.4527f,0.6465f,-0.6141f,  -0.4530f,0.6568f,-0.6029f,  -0.3513f,0.6278f,-0.6946f,  -0.3519f,0.6392f,-0.6838f,  -0.2440f,0.5984f,-0.7632f,  
	-0.2447f,0.6107f,-0.7531f,  -0.1324f,0.5587f,-0.8188f,  -0.1333f,0.5718f,-0.8095f,  -0.0187f,0.5094f,-0.8603f,  -0.0197f,0.5231f,-0.8521f,  0.0954f,0.4515f,-0.8872f,  0.0943f,0.4654f,-0.8801f,  0.2079f,0.3858f,-0.8989f,  0.2067f,0.3998f,-0.8930f,  0.3168f,0.3135f,-0.8952f,  0.3156f,0.3274f,-0.8907f,  0.4204f,0.2359f,-0.8762f,  0.4190f,0.2493f,-0.8731f,  0.5167f,0.1542f,-0.8422f,  0.5153f,0.1670f,-0.8406f,  
	0.6042f,0.0699f,-0.7938f,  0.6028f,0.0818f,-0.7937f,  0.6813f,-0.0156f,-0.7318f,  0.6799f,-0.0047f,-0.7332f,  0.7468f,-0.1009f,-0.6573f,  0.7455f,-0.0912f,-0.6602f,  0.7996f,-0.1844f,-0.5716f,  0.7983f,-0.1761f,-0.5759f,  0.8386f,-0.2647f,-0.4760f,  0.8374f,-0.2581f,-0.4818f,  0.8633f,-0.3406f,-0.3724f,  0.8623f,-0.3356f,-0.3794f,  0.8733f,-0.4106f,-0.2623f,  0.8723f,-0.4073f,-0.2704f,  0.8683f,-0.4735f,-0.1477f,  
	0.8675f,-0.4721f,-0.1569f,  0.8484f,-0.5284f,-0.0307f,  0.8478f,-0.5288f,-0.0407f,  0.8141f,-0.5742f,0.0870f,  0.8136f,-0.5764f,0.0763f,  0.7658f,-0.6102f,0.2031f,  0.7654f,-0.6142f,0.1919f,  0.7044f,-0.6357f,0.3158f,  0.7042f,-0.6415f,0.3043f,  0.7044f,-0.6357f,0.3158f,  0.7046f,-0.6300f,0.3264f,  0.6310f,-0.6432f,0.4338f,  0.6312f,-0.6362f,0.4438f,  0.5466f,-0.6370f,0.5436f,  0.5468f,-0.6289f,0.5527f,  
	0.4527f,-0.6178f,0.6429f,  0.4529f,-0.6091f,0.6511f,  0.3510f,-0.5865f,0.7300f,  0.3513f,-0.5774f,0.7371f,  0.2432f,-0.5441f,0.8030f,  0.2436f,-0.5349f,0.8090f,  0.1313f,-0.4920f,0.8607f,  0.1317f,-0.4829f,0.8657f,  0.0171f,-0.4313f,0.9021f,  0.0176f,-0.4225f,0.9062f,  -0.0974f,-0.3635f,0.9265f,  -0.0968f,-0.3553f,0.9297f,  -0.2102f,-0.2901f,0.9336f,  -0.2094f,-0.2825f,0.9361f,  -0.3193f,-0.2127f,0.9235f,  
	-0.3185f,-0.2058f,0.9253f,  -0.4231f,-0.1326f,0.8963f,  -0.4222f,-0.1265f,0.8976f,  -0.5196f,-0.0515f,0.8528f,  -0.5187f,-0.0462f,0.8537f,  -0.6073f,0.0292f,0.7939f,  -0.6064f,0.0337f,0.7944f,  -0.6848f,0.1081f,0.7207f,  -0.6839f,0.1119f,0.7209f,  -0.7508f,0.1838f,0.6345f,  -0.7499f,0.1869f,0.6346f,  -0.8041f,0.2551f,0.5370f,  -0.8032f,0.2576f,0.5371f,  -0.8439f,0.3209f,0.4300f,  -0.8431f,0.3228f,0.4300f,  
	-0.8695f,0.3801f,0.3154f,  -0.8689f,0.3815f,0.3153f,  -0.8806f,0.4317f,0.1951f,  -0.8801f,0.4328f,0.1951f,  -0.8770f,0.4751f,0.0714f,  -0.8766f,0.4759f,0.0714f,  -0.8588f,0.5095f,-0.0538f,  -0.8585f,0.5100f,-0.0537f,  -0.8262f,0.5345f,-0.1781f,  -0.8260f,0.5348f,-0.1781f,  -0.7798f,0.5497f,-0.2996f,  -0.7797f,0.5498f,-0.2996f,  -0.7205f,0.5548f,-0.4161f,  -0.7205f,0.5548f,-0.4161f,  -0.6491f,0.5499f,-0.5256f,  
	-0.6493f,0.5497f,-0.5257f,  -0.5670f,0.5349f,-0.6264f,  -0.5672f,0.5345f,-0.6265f,  -0.4756f,0.5102f,-0.7166f,  -0.4758f,0.5096f,-0.7169f,  -0.3763f,0.4761f,-0.7948f,  -0.3766f,0.4753f,-0.7952f,  -0.2708f,0.4331f,-0.8597f,  -0.2712f,0.4320f,-0.8601f,  -0.1611f,0.3820f,-0.9100f,  -0.1615f,0.3804f,-0.9106f,  -0.0489f,0.3234f,-0.9450f,  -0.0493f,0.3214f,-0.9456f,  0.0638f,0.2583f,-0.9639f,  0.0633f,0.2558f,-0.9646f,  
	0.1750f,0.1878f,-0.9665f,  0.1743f,0.1847f,-0.9672f,  0.2827f,0.1130f,-0.9525f,  0.2820f,0.1092f,-0.9532f,  0.3852f,0.0351f,-0.9222f,  0.3842f,0.0306f,-0.9227f,  0.4804f,-0.0446f,-0.8759f,  0.4792f,-0.0498f,-0.8763f,  0.5668f,-0.1247f,-0.8144f,  0.5652f,-0.1307f,-0.8145f,  0.6427f,-0.2037f,-0.7386f,  0.6407f,-0.2104f,-0.7384f,  0.7067f,-0.2802f,-0.6496f,  0.7042f,-0.2876f,-0.6491f,  0.7577f,-0.3527f,-0.5491f,  
	0.7546f,-0.3608f,-0.5481f,  0.7947f,-0.4198f,-0.4385f,  0.7909f,-0.4284f,-0.4370f,  0.8169f,-0.4800f,-0.3197f,  0.8124f,-0.4890f,-0.3177f,  0.8240f,-0.5320f,-0.1947f,  0.8187f,-0.5411f,-0.1921f,  0.8158f,-0.5745f,-0.0657f,  0.8097f,-0.5835f,-0.0626f,  0.7925f,-0.6064f,0.0650f,  0.7855f,-0.6150f,0.0687f,  0.7546f,-0.6265f,0.1953f,  0.7468f,-0.6345f,0.1994f,  0.7028f,-0.6341f,0.3226f,  0.6942f,-0.6411f,0.3272f,  
	0.6382f,-0.6285f,0.4447f,  0.6290f,-0.6342f,0.4496f,  -0.8633f,0.3411f,0.3720f,  -0.8741f,0.4138f,0.2543f,  -0.8732f,0.4110f,0.2619f,  -0.8690f,0.4750f,0.1388f,  -0.8681f,0.4740f,0.1474f,  -0.8489f,0.5281f,0.0210f,  -0.8482f,0.5288f,0.0303f,  -0.8144f,0.5722f,-0.0972f,  -0.8138f,0.5746f,-0.0873f,  -0.7659f,0.6064f,-0.2137f,  -0.7654f,0.6105f,-0.2033f,  -0.7043f,0.6303f,-0.3266f,  -0.7040f,0.6361f,-0.3159f,  
	-0.6306f,0.6435f,-0.4339f,  -0.6305f,0.6507f,-0.4231f,  -0.5462f,0.6456f,-0.5337f,  -0.5463f,0.6542f,-0.5231f,  -0.4525f,0.6367f,-0.6244f,  -0.4527f,0.6465f,-0.6141f,  -0.3510f,0.6169f,-0.7045f,  -0.3513f,0.6278f,-0.6946f,  -0.2435f,0.5865f,-0.7725f,  -0.2440f,0.5984f,-0.7632f,  -0.1318f,0.5461f,-0.8273f,  -0.1324f,0.5587f,-0.8188f,  -0.0179f,0.4964f,-0.8679f,  -0.0187f,0.5094f,-0.8603f,  0.0964f,0.4382f,-0.8937f,  
	0.0954f,0.4515f,-0.8872f,  0.2089f,0.3725f,-0.9042f,  0.2079f,0.3858f,-0.8989f,  0.3179f,0.3004f,-0.8993f,  0.3168f,0.3135f,-0.8952f,  0.4215f,0.2232f,-0.8789f,  0.4204f,0.2359f,-0.8762f,  0.5179f,0.1422f,-0.8436f,  0.5167f,0.1542f,-0.8422f,  0.6054f,0.0587f,-0.7938f,  0.6042f,0.0699f,-0.7938f,  0.6826f,-0.0258f,-0.7304f,  0.6813f,-0.0156f,-0.7318f,  0.7481f,-0.1098f,-0.6545f,  0.7468f,-0.1009f,-0.6573f,  
	0.8007f,-0.1919f,-0.5674f,  0.7996f,-0.1844f,-0.5716f,  0.8398f,-0.2708f,-0.4706f,  0.8386f,-0.2647f,-0.4760f,  0.8644f,-0.3450f,-0.3658f,  0.8633f,-0.3406f,-0.3724f,  0.8742f,-0.4133f,-0.2547f,  0.8733f,-0.4106f,-0.2623f,  0.8691f,-0.4746f,-0.1392f,  0.8683f,-0.4735f,-0.1477f,  0.8491f,-0.5277f,-0.0213f,  0.8484f,-0.5284f,-0.0307f,  0.8146f,-0.5718f,0.0969f,  0.8141f,-0.5742f,0.0870f,  0.7662f,-0.6061f,0.2135f,  
	0.7658f,-0.6102f,0.2031f,  0.7046f,-0.6300f,0.3264f,  0.7044f,-0.6357f,0.3158f,  0.7046f,-0.6300f,0.3264f,  0.7050f,-0.6244f,0.3363f,  0.6312f,-0.6362f,0.4438f,  0.6315f,-0.6293f,0.4530f,  0.5468f,-0.6289f,0.5527f,  0.5471f,-0.6212f,0.5611f,  0.4529f,-0.6091f,0.6511f,  0.4533f,-0.6007f,0.6585f,  0.3513f,-0.5774f,0.7371f,  0.3517f,-0.5687f,0.7436f,  0.2436f,-0.5349f,0.8090f,  0.2441f,-0.5262f,0.8146f,  
	0.1317f,-0.4829f,0.8657f,  0.1323f,-0.4743f,0.8704f,  0.0176f,-0.4225f,0.9062f,  0.0183f,-0.4143f,0.9100f,  -0.0968f,-0.3553f,0.9297f,  -0.0960f,-0.3475f,0.9328f,  -0.2094f,-0.2825f,0.9361f,  -0.2085f,-0.2754f,0.9384f,  -0.3185f,-0.2058f,0.9253f,  -0.3176f,-0.1993f,0.9271f,  -0.4222f,-0.1265f,0.8976f,  -0.4212f,-0.1207f,0.8989f,  -0.5187f,-0.0462f,0.8537f,  -0.5176f,-0.0412f,0.8546f,  -0.6064f,0.0337f,0.7944f,  
	-0.6054f,0.0381f,0.7950f,  -0.6839f,0.1119f,0.7209f,  -0.6829f,0.1155f,0.7214f,  -0.7499f,0.1869f,0.6346f,  -0.7489f,0.1899f,0.6349f,  -0.8032f,0.2576f,0.5371f,  -0.8023f,0.2601f,0.5373f,  -0.8431f,0.3228f,0.4300f,  -0.8423f,0.3248f,0.4302f,  -0.8689f,0.3815f,0.3153f,  -0.8681f,0.3831f,0.3155f,  -0.8801f,0.4328f,0.1951f,  -0.8795f,0.4340f,0.1953f,  -0.8766f,0.4759f,0.0714f,  -0.8761f,0.4768f,0.0715f,  
	-0.8585f,0.5100f,-0.0537f,  -0.8581f,0.5108f,-0.0536f,  -0.8260f,0.5348f,-0.1781f,  -0.8257f,0.5353f,-0.1779f,  -0.7797f,0.5498f,-0.2996f,  -0.7795f,0.5502f,-0.2994f,  -0.7205f,0.5548f,-0.4161f,  -0.7204f,0.5550f,-0.4160f,  -0.6493f,0.5497f,-0.5257f,  -0.6492f,0.5497f,-0.5257f,  -0.5672f,0.5345f,-0.6265f,  -0.5672f,0.5344f,-0.6266f,  -0.4758f,0.5096f,-0.7169f,  -0.4758f,0.5093f,-0.7171f,  -0.3766f,0.4753f,-0.7952f,  
	-0.3766f,0.4746f,-0.7955f,  -0.2712f,0.4320f,-0.8601f,  -0.2713f,0.4310f,-0.8606f,  -0.1615f,0.3804f,-0.9106f,  -0.1616f,0.3790f,-0.9112f,  -0.0493f,0.3214f,-0.9456f,  -0.0495f,0.3195f,-0.9463f,  0.0633f,0.2558f,-0.9646f,  0.0630f,0.2533f,-0.9653f,  0.1743f,0.1847f,-0.9672f,  0.1740f,0.1815f,-0.9679f,  0.2820f,0.1092f,-0.9532f,  0.2814f,0.1052f,-0.9538f,  0.3842f,0.0306f,-0.9227f,  0.3834f,0.0258f,-0.9232f,  
	0.4792f,-0.0498f,-0.8763f,  0.4780f,-0.0554f,-0.8766f,  0.5652f,-0.1307f,-0.8145f,  0.5637f,-0.1371f,-0.8145f,  0.6407f,-0.2104f,-0.7384f,  0.6387f,-0.2177f,-0.7381f,  0.7042f,-0.2876f,-0.6491f,  0.7016f,-0.2956f,-0.6483f,  0.7546f,-0.3608f,-0.5481f,  0.7513f,-0.3694f,-0.5468f,  0.7909f,-0.4284f,-0.4370f,  0.7869f,-0.4375f,-0.4352f,  0.8124f,-0.4890f,-0.3177f,  0.8076f,-0.4984f,-0.3153f,  0.8187f,-0.5411f,-0.1921f,  
	0.8130f,-0.5507f,-0.1891f,  0.8097f,-0.5835f,-0.0626f,  0.8031f,-0.5929f,-0.0590f,  0.7855f,-0.6150f,0.0687f,  0.7780f,-0.6240f,0.0729f,  0.7468f,-0.6345f,0.1994f,  0.7384f,-0.6427f,0.2041f,  0.6942f,-0.6411f,0.3272f,  0.6851f,-0.6482f,0.3323f,  0.6290f,-0.6342f,0.4496f,  0.6193f,-0.6399f,0.4550f,  -0.8741f,0.4138f,0.2543f,  -0.8698f,0.4758f,0.1308f,  -0.8690f,0.4750f,0.1388f,  -0.8497f,0.5272f,0.0123f,  
	-0.8489f,0.5281f,0.0210f,  -0.8150f,0.5696f,-0.1065f,  -0.8144f,0.5722f,-0.0972f,  -0.7664f,0.6023f,-0.2234f,  -0.7659f,0.6064f,-0.2137f,  -0.7046f,0.6247f,-0.3365f,  -0.7043f,0.6303f,-0.3266f,  -0.6309f,0.6364f,-0.4438f,  -0.6306f,0.6435f,-0.4339f,  -0.5463f,0.6372f,-0.5436f,  -0.5462f,0.6456f,-0.5337f,  -0.4524f,0.6272f,-0.6340f,  -0.4525f,0.6367f,-0.6244f,  -0.3507f,0.6064f,-0.7137f,  -0.3510f,0.6169f,-0.7045f,  
	-0.2431f,0.5752f,-0.7811f,  -0.2435f,0.5865f,-0.7725f,  -0.1313f,0.5342f,-0.8351f,  -0.1318f,0.5461f,-0.8273f,  -0.0172f,0.4841f,-0.8749f,  -0.0179f,0.4964f,-0.8679f,  0.0971f,0.4256f,-0.8997f,  0.0964f,0.4382f,-0.8937f,  0.2098f,0.3599f,-0.9091f,  0.2089f,0.3725f,-0.9042f,  0.3189f,0.2881f,-0.9030f,  0.3179f,0.3004f,-0.8993f,  0.4225f,0.2113f,-0.8814f,  0.4215f,0.2232f,-0.8789f,  0.5189f,0.1309f,-0.8447f,  
	0.5179f,0.1422f,-0.8436f,  0.6065f,0.0483f,-0.7936f,  0.6054f,0.0587f,-0.7938f,  0.6837f,-0.0352f,-0.7289f,  0.6826f,-0.0258f,-0.7304f,  0.7492f,-0.1180f,-0.6518f,  0.7481f,-0.1098f,-0.6545f,  0.8018f,-0.1989f,-0.5635f,  0.8007f,-0.1919f,-0.5674f,  0.8408f,-0.2763f,-0.4655f,  0.8398f,-0.2708f,-0.4706f,  0.8654f,-0.3490f,-0.3596f,  0.8644f,-0.3450f,-0.3658f,  0.8752f,-0.4157f,-0.2475f,  0.8742f,-0.4133f,-0.2547f,  
	0.8700f,-0.4754f,-0.1312f,  0.8691f,-0.4746f,-0.1392f,  0.8499f,-0.5268f,-0.0126f,  0.8491f,-0.5277f,-0.0213f,  0.8152f,-0.5693f,0.1062f,  0.8146f,-0.5718f,0.0969f,  0.7667f,-0.6020f,0.2232f,  0.7662f,-0.6061f,0.2135f,  0.7050f,-0.6244f,0.3363f,  0.7046f,-0.6300f,0.3264f,  0.7050f,-0.6244f,0.3363f,  0.7054f,-0.6189f,0.3455f,  0.6315f,-0.6293f,0.4530f,  0.6319f,-0.6227f,0.4615f,  0.5471f,-0.6212f,0.5611f,  
	0.5475f,-0.6138f,0.5688f,  0.4533f,-0.6007f,0.6585f,  0.4538f,-0.5928f,0.6654f,  0.3517f,-0.5687f,0.7436f,  0.3522f,-0.5605f,0.7495f,  0.2441f,-0.5262f,0.8146f,  0.2447f,-0.5179f,0.8197f,  0.1323f,-0.4743f,0.8704f,  0.1331f,-0.4662f,0.8746f,  0.0183f,-0.4143f,0.9100f,  0.0192f,-0.4066f,0.9134f,  -0.0960f,-0.3475f,0.9328f,  -0.0950f,-0.3403f,0.9355f,  -0.2085f,-0.2754f,0.9384f,  -0.2075f,-0.2687f,0.9406f,  
	-0.3176f,-0.1993f,0.9271f,  -0.3164f,-0.1932f,0.9287f,  -0.4212f,-0.1207f,0.8989f,  -0.4200f,-0.1154f,0.9002f,  -0.5176f,-0.0412f,0.8546f,  -0.5164f,-0.0364f,0.8556f,  -0.6054f,0.0381f,0.7950f,  -0.6042f,0.0421f,0.7958f,  -0.6829f,0.1155f,0.7214f,  -0.6817f,0.1190f,0.7219f,  -0.7489f,0.1899f,0.6349f,  -0.7478f,0.1929f,0.6353f,  -0.8023f,0.2601f,0.5373f,  -0.8013f,0.2625f,0.5376f,  -0.8423f,0.3248f,0.4302f,  
	-0.8413f,0.3269f,0.4305f,  -0.8681f,0.3831f,0.3155f,  -0.8673f,0.3848f,0.3158f,  -0.8795f,0.4340f,0.1953f,  -0.8787f,0.4354f,0.1956f,  -0.8761f,0.4768f,0.0715f,  -0.8754f,0.4780f,0.0719f,  -0.8581f,0.5108f,-0.0536f,  -0.8575f,0.5117f,-0.0533f,  -0.8257f,0.5353f,-0.1779f,  -0.8252f,0.5362f,-0.1777f,  -0.7795f,0.5502f,-0.2994f,  -0.7791f,0.5508f,-0.2992f,  -0.7204f,0.5550f,-0.4160f,  -0.7200f,0.5555f,-0.4159f,  
	-0.6492f,0.5497f,-0.5257f,  -0.6489f,0.5501f,-0.5257f,  -0.5672f,0.5344f,-0.6266f,  -0.5670f,0.5346f,-0.6267f,  -0.4758f,0.5093f,-0.7171f,  -0.4756f,0.5092f,-0.7173f,  -0.3766f,0.4746f,-0.7955f,  -0.3764f,0.4742f,-0.7959f,  -0.2713f,0.4310f,-0.8606f,  -0.2711f,0.4301f,-0.8611f,  -0.1616f,0.3790f,-0.9112f,  -0.1615f,0.3777f,-0.9118f,  -0.0495f,0.3195f,-0.9463f,  -0.0495f,0.3175f,-0.9470f,  0.0630f,0.2533f,-0.9653f,  
	0.0630f,0.2507f,-0.9660f,  0.1740f,0.1815f,-0.9679f,  0.1738f,0.1781f,-0.9685f,  0.2814f,0.1052f,-0.9538f,  0.2810f,0.1010f,-0.9544f,  0.3834f,0.0258f,-0.9232f,  0.3827f,0.0207f,-0.9236f,  0.4780f,-0.0554f,-0.8766f,  0.4770f,-0.0614f,-0.8767f,  0.5637f,-0.1371f,-0.8145f,  0.5622f,-0.1440f,-0.8144f,  0.6387f,-0.2177f,-0.7381f,  0.6366f,-0.2254f,-0.7375f,  0.7016f,-0.2956f,-0.6483f,  0.6989f,-0.3041f,-0.6473f,  
	0.7513f,-0.3694f,-0.5468f,  0.7479f,-0.3786f,-0.5452f,  0.7869f,-0.4375f,-0.4352f,  0.7826f,-0.4472f,-0.4330f,  0.8076f,-0.4984f,-0.3153f,  0.8024f,-0.5084f,-0.3124f,  0.8130f,-0.5507f,-0.1891f,  0.8069f,-0.5608f,-0.1857f,  0.8031f,-0.5929f,-0.0590f,  0.7960f,-0.6028f,-0.0549f,  0.7780f,-0.6240f,0.0729f,  0.7700f,-0.6333f,0.0776f,  0.7384f,-0.6427f,0.2041f,  0.7294f,-0.6512f,0.2093f,  0.6851f,-0.6482f,0.3323f,  
	0.6753f,-0.6556f,0.3379f,  0.6193f,-0.6399f,0.4550f,  0.6088f,-0.6457f,0.4609f,  -0.8698f,0.4758f,0.1308f,  -0.8504f,0.5261f,0.0042f,  -0.8497f,0.5272f,0.0123f,  -0.8156f,0.5670f,-0.1150f,  -0.8150f,0.5696f,-0.1065f,  -0.7669f,0.5982f,-0.2323f,  -0.7664f,0.6023f,-0.2234f,  -0.7051f,0.6192f,-0.3456f,  -0.7046f,0.6247f,-0.3365f,  -0.6312f,0.6296f,-0.4530f,  -0.6309f,0.6364f,-0.4438f,  -0.5465f,0.6292f,-0.5527f,  
	-0.5463f,0.6372f,-0.5436f,  -0.4525f,0.6180f,-0.6429f,  -0.4524f,0.6272f,-0.6340f,  -0.3507f,0.5963f,-0.7221f,  -0.3507f,0.6064f,-0.7137f,  -0.2429f,0.5644f,-0.7889f,  -0.2431f,0.5752f,-0.7811f,  -0.1310f,0.5228f,-0.8423f,  -0.1313f,0.5342f,-0.8351f,  -0.0168f,0.4723f,-0.8813f,  -0.0172f,0.4841f,-0.8749f,  0.0976f,0.4138f,-0.9051f,  0.0971f,0.4256f,-0.8997f,  0.2104f,0.3481f,-0.9135f,  0.2098f,0.3599f,-0.9091f,  
	0.3196f,0.2765f,-0.9063f,  0.3189f,0.2881f,-0.9030f,  0.4233f,0.2001f,-0.8836f,  0.4225f,0.2113f,-0.8814f,  0.5198f,0.1204f,-0.8457f,  0.5189f,0.1309f,-0.8447f,  0.6074f,0.0386f,-0.7934f,  0.6065f,0.0483f,-0.7936f,  0.6846f,-0.0439f,-0.7276f,  0.6837f,-0.0352f,-0.7289f,  0.7501f,-0.1257f,-0.6492f,  0.7492f,-0.1180f,-0.6518f,  0.8028f,-0.2052f,-0.5598f,  0.8018f,-0.1989f,-0.5635f,  0.8418f,-0.2813f,-0.4607f,  
	0.8408f,-0.2763f,-0.4655f,  0.8663f,-0.3526f,-0.3538f,  0.8654f,-0.3490f,-0.3596f,  0.8760f,-0.4178f,-0.2409f,  0.8752f,-0.4157f,-0.2475f,  0.8708f,-0.4759f,-0.1238f,  0.8700f,-0.4754f,-0.1312f,  0.8506f,-0.5258f,-0.0045f,  0.8499f,-0.5268f,-0.0126f,  0.8159f,-0.5667f,0.1148f,  0.8152f,-0.5693f,0.1062f,  0.7672f,-0.5979f,0.2321f,  0.7667f,-0.6020f,0.2232f,  0.7054f,-0.6189f,0.3455f,  0.7050f,-0.6244f,0.3363f,  
	0.7054f,-0.6189f,0.3455f,  0.7059f,-0.6135f,0.3539f,  0.6319f,-0.6227f,0.4615f,  0.6324f,-0.6163f,0.4692f,  0.5475f,-0.6138f,0.5688f,  0.5481f,-0.6067f,0.5758f,  0.4538f,-0.5928f,0.6654f,  0.4544f,-0.5852f,0.6716f,  0.3522f,-0.5605f,0.7495f,  0.3529f,-0.5527f,0.7549f,  0.2447f,-0.5179f,0.8197f,  0.2455f,-0.5102f,0.8243f,  0.1331f,-0.4662f,0.8746f,  0.1340f,-0.4587f,0.8785f,  0.0192f,-0.4066f,0.9134f,  
	0.0202f,-0.3993f,0.9166f,  -0.0950f,-0.3403f,0.9355f,  -0.0939f,-0.3335f,0.9381f,  -0.2075f,-0.2687f,0.9406f,  -0.2063f,-0.2624f,0.9426f,  -0.3164f,-0.1932f,0.9287f,  -0.3152f,-0.1876f,0.9303f,  -0.4200f,-0.1154f,0.9002f,  -0.4187f,-0.1103f,0.9014f,  -0.5164f,-0.0364f,0.8556f,  -0.5151f,-0.0320f,0.8565f,  -0.6042f,0.0421f,0.7958f,  -0.6028f,0.0460f,0.7965f,  -0.6817f,0.1190f,0.7219f,  -0.6804f,0.1224f,0.7226f,  
	-0.7478f,0.1929f,0.6353f,  -0.7465f,0.1958f,0.6359f,  -0.8013f,0.2625f,0.5376f,  -0.8001f,0.2650f,0.5382f,  -0.8413f,0.3269f,0.4305f,  -0.8402f,0.3290f,0.4310f,  -0.8673f,0.3848f,0.3158f,  -0.8663f,0.3866f,0.3163f,  -0.8787f,0.4354f,0.1956f,  -0.8778f,0.4370f,0.1961f,  -0.8754f,0.4780f,0.0719f,  -0.8746f,0.4794f,0.0723f,  -0.8575f,0.5117f,-0.0533f,  -0.8568f,0.5130f,-0.0529f,  -0.8252f,0.5362f,-0.1777f,  
	-0.8246f,0.5373f,-0.1773f,  -0.7791f,0.5508f,-0.2992f,  -0.7785f,0.5518f,-0.2990f,  -0.7200f,0.5555f,-0.4159f,  -0.7195f,0.5563f,-0.4157f,  -0.6489f,0.5501f,-0.5257f,  -0.6484f,0.5507f,-0.5256f,  -0.5670f,0.5346f,-0.6267f,  -0.5665f,0.5350f,-0.6268f,  -0.4756f,0.5092f,-0.7173f,  -0.4751f,0.5092f,-0.7176f,  -0.3764f,0.4742f,-0.7959f,  -0.3760f,0.4739f,-0.7963f,  -0.2711f,0.4301f,-0.8611f,  -0.2707f,0.4294f,-0.8616f,  
	-0.1615f,0.3777f,-0.9118f,  -0.1611f,0.3763f,-0.9124f,  -0.0495f,0.3175f,-0.9470f,  -0.0491f,0.3155f,-0.9476f,  0.0630f,0.2507f,-0.9660f,  0.0632f,0.2479f,-0.9667f,  0.1738f,0.1781f,-0.9685f,  0.1738f,0.1745f,-0.9692f,  0.2810f,0.1010f,-0.9544f,  0.2808f,0.0966f,-0.9549f,  0.3827f,0.0207f,-0.9236f,  0.3822f,0.0153f,-0.9240f,  0.4770f,-0.0614f,-0.8767f,  0.4761f,-0.0678f,-0.8768f,  0.5622f,-0.1440f,-0.8144f,  
	0.5608f,-0.1513f,-0.8140f,  0.6366f,-0.2254f,-0.7375f,  0.6346f,-0.2336f,-0.7367f,  0.6989f,-0.3041f,-0.6473f,  0.6962f,-0.3132f,-0.6460f,  0.7479f,-0.3786f,-0.5452f,  0.7443f,-0.3884f,-0.5433f,  0.7826f,-0.4472f,-0.4330f,  0.7781f,-0.4575f,-0.4304f,  0.8024f,-0.5084f,-0.3124f,  0.7969f,-0.5190f,-0.3092f,  0.8069f,-0.5608f,-0.1857f,  0.8003f,-0.5714f,-0.1818f,  0.7960f,-0.6028f,-0.0549f,  0.7884f,-0.6132f,-0.0503f,  
	0.7700f,-0.6333f,0.0776f,  0.7613f,-0.6431f,0.0827f,  0.7294f,-0.6512f,0.2093f,  0.7198f,-0.6600f,0.2150f,  0.6753f,-0.6556f,0.3379f,  0.6649f,-0.6630f,0.3440f,  0.6088f,-0.6457f,0.4609f,  0.5978f,-0.6515f,0.4672f,  -0.8504f,0.5261f,0.0042f,  -0.8163f,0.5644f,-0.1229f,  -0.8156f,0.5670f,-0.1150f,  -0.7675f,0.5942f,-0.2406f,  -0.7669f,0.5982f,-0.2323f,  -0.7056f,0.6138f,-0.3541f,  -0.7051f,0.6192f,-0.3456f,  
	-0.6316f,0.6229f,-0.4615f,  -0.6312f,0.6296f,-0.4530f,  -0.5468f,0.6214f,-0.5611f,  -0.5465f,0.6292f,-0.5527f,  -0.4527f,0.6093f,-0.6510f,  -0.4525f,0.6180f,-0.6429f,  -0.3508f,0.5867f,-0.7299f,  -0.3507f,0.5963f,-0.7221f,  -0.2429f,0.5541f,-0.7962f,  -0.2429f,0.5644f,-0.7889f,  -0.1309f,0.5121f,-0.8489f,  -0.1310f,0.5228f,-0.8423f,  -0.0166f,0.4612f,-0.8871f,  -0.0168f,0.4723f,-0.8813f,  0.0980f,0.4025f,-0.9101f,  
	0.0976f,0.4138f,-0.9051f,  0.2109f,0.3369f,-0.9176f,  0.2104f,0.3481f,-0.9135f,  0.3202f,0.2656f,-0.9094f,  0.3196f,0.2765f,-0.9063f,  0.4240f,0.1897f,-0.8856f,  0.4233f,0.2001f,-0.8836f,  0.5205f,0.1105f,-0.8467f,  0.5198f,0.1204f,-0.8457f,  0.6082f,0.0295f,-0.7932f,  0.6074f,0.0386f,-0.7934f,  0.6855f,-0.0521f,-0.7262f,  0.6846f,-0.0439f,-0.7276f,  0.7510f,-0.1327f,-0.6468f,  0.7501f,-0.1257f,-0.6492f,  
	0.8037f,-0.2111f,-0.5563f,  0.8028f,-0.2052f,-0.5598f,  0.8426f,-0.2859f,-0.4563f,  0.8418f,-0.2813f,-0.4607f,  0.8672f,-0.3557f,-0.3485f,  0.8663f,-0.3526f,-0.3538f,  0.8769f,-0.4195f,-0.2347f,  0.8760f,-0.4178f,-0.2409f,  0.8716f,-0.4761f,-0.1169f,  0.8708f,-0.4759f,-0.1238f,  0.8514f,-0.5246f,0.0029f,  0.8506f,-0.5258f,-0.0045f,  0.8166f,-0.5641f,0.1227f,  0.8159f,-0.5667f,0.1148f,  0.7678f,-0.5939f,0.2404f,  
	0.7672f,-0.5979f,0.2321f,  0.7059f,-0.6135f,0.3539f,  0.7054f,-0.6189f,0.3455f,  0.7059f,-0.6135f,0.3539f,  0.7065f,-0.6083f,0.3617f,  0.6324f,-0.6163f,0.4692f,  0.6330f,-0.6102f,0.4763f,  0.5481f,-0.6067f,0.5758f,  0.5488f,-0.5999f,0.5822f,  0.4544f,-0.5852f,0.6716f,  0.4551f,-0.5781f,0.6773f,  0.3529f,-0.5527f,0.7549f,  0.3538f,-0.5454f,0.7598f,  0.2455f,-0.5102f,0.8243f,  0.2465f,-0.5029f,0.8285f,  
	0.1340f,-0.4587f,0.8785f,  0.1350f,-0.4516f,0.8820f,  0.0202f,-0.3993f,0.9166f,  0.0214f,-0.3925f,0.9195f,  -0.0939f,-0.3335f,0.9381f,  -0.0926f,-0.3271f,0.9404f,  -0.2063f,-0.2624f,0.9426f,  -0.2049f,-0.2566f,0.9445f,  -0.3152f,-0.1876f,0.9303f,  -0.3138f,-0.1822f,0.9318f,  -0.4187f,-0.1103f,0.9014f,  -0.4172f,-0.1055f,0.9026f,  -0.5151f,-0.0320f,0.8565f,  -0.5137f,-0.0277f,0.8575f,  -0.6028f,0.0460f,0.7965f,  
	-0.6014f,0.0498f,0.7974f,  -0.6804f,0.1224f,0.7226f,  -0.6790f,0.1256f,0.7233f,  -0.7465f,0.1958f,0.6359f,  -0.7452f,0.1986f,0.6366f,  -0.8001f,0.2650f,0.5382f,  -0.7988f,0.2675f,0.5388f,  -0.8402f,0.3290f,0.4310f,  -0.8391f,0.3312f,0.4316f,  -0.8663f,0.3866f,0.3163f,  -0.8652f,0.3886f,0.3169f,  -0.8778f,0.4370f,0.1961f,  -0.8768f,0.4388f,0.1967f,  -0.8746f,0.4794f,0.0723f,  -0.8737f,0.4810f,0.0729f,  
	-0.8568f,0.5130f,-0.0529f,  -0.8559f,0.5145f,-0.0524f,  -0.8246f,0.5373f,-0.1773f,  -0.8237f,0.5387f,-0.1769f,  -0.7785f,0.5518f,-0.2990f,  -0.7778f,0.5531f,-0.2987f,  -0.7195f,0.5563f,-0.4157f,  -0.7187f,0.5574f,-0.4156f,  -0.6484f,0.5507f,-0.5256f,  -0.6476f,0.5516f,-0.5256f,  -0.5665f,0.5350f,-0.6268f,  -0.5657f,0.5356f,-0.6270f,  -0.4751f,0.5092f,-0.7176f,  -0.4744f,0.5095f,-0.7179f,  -0.3760f,0.4739f,-0.7963f,  
	-0.3752f,0.4737f,-0.7967f,  -0.2707f,0.4294f,-0.8616f,  -0.2699f,0.4287f,-0.8622f,  -0.1611f,0.3763f,-0.9124f,  -0.1604f,0.3750f,-0.9130f,  -0.0491f,0.3155f,-0.9476f,  -0.0485f,0.3135f,-0.9484f,  0.0632f,0.2479f,-0.9667f,  0.0637f,0.2450f,-0.9674f,  0.1738f,0.1745f,-0.9692f,  0.1741f,0.1708f,-0.9698f,  0.2808f,0.0966f,-0.9549f,  0.2808f,0.0918f,-0.9554f,  0.3822f,0.0153f,-0.9240f,  0.3819f,0.0095f,-0.9242f,  
	0.4761f,-0.0678f,-0.8768f,  0.4753f,-0.0746f,-0.8767f,  0.5608f,-0.1513f,-0.8140f,  0.5594f,-0.1591f,-0.8135f,  0.6346f,-0.2336f,-0.7367f,  0.6325f,-0.2424f,-0.7356f,  0.6962f,-0.3132f,-0.6460f,  0.6933f,-0.3229f,-0.6443f,  0.7443f,-0.3884f,-0.5433f,  0.7405f,-0.3987f,-0.5410f,  0.7781f,-0.4575f,-0.4304f,  0.7733f,-0.4684f,-0.4274f,  0.7969f,-0.5190f,-0.3092f,  0.7910f,-0.5301f,-0.3055f,  0.8003f,-0.5714f,-0.1818f,  
	0.7932f,-0.5825f,-0.1774f,  0.7884f,-0.6132f,-0.0503f,  0.7802f,-0.6240f,-0.0452f,  0.7613f,-0.6431f,0.0827f,  0.7520f,-0.6532f,0.0884f,  0.7198f,-0.6600f,0.2150f,  0.7095f,-0.6691f,0.2212f,  0.6649f,-0.6630f,0.3440f,  0.6537f,-0.6706f,0.3506f,  0.5978f,-0.6515f,0.4672f,  0.5860f,-0.6572f,0.4740f,  -0.8163f,0.5644f,-0.1229f,  -0.7682f,0.5902f,-0.2481f,  -0.7675f,0.5942f,-0.2406f,  -0.7062f,0.6086f,-0.3618f,  
	-0.7056f,0.6138f,-0.3541f,  -0.6321f,0.6166f,-0.4693f,  -0.6316f,0.6229f,-0.4615f,  -0.5473f,0.6140f,-0.5688f,  -0.5468f,0.6214f,-0.5611f,  -0.4530f,0.6009f,-0.6585f,  -0.4527f,0.6093f,-0.6510f,  -0.3510f,0.5776f,-0.7370f,  -0.3508f,0.5867f,-0.7299f,  -0.2431f,0.5444f,-0.8028f,  -0.2429f,0.5541f,-0.7962f,  -0.1309f,0.5019f,-0.8550f,  -0.1309f,0.5121f,-0.8489f,  -0.0165f,0.4508f,-0.8925f,  -0.0166f,0.4612f,-0.8871f,  
	0.0981f,0.3919f,-0.9147f,  0.0980f,0.4025f,-0.9101f,  0.2111f,0.3264f,-0.9213f,  0.2109f,0.3369f,-0.9176f,  0.3205f,0.2553f,-0.9122f,  0.3202f,0.2656f,-0.9094f,  0.4244f,0.1798f,-0.8874f,  0.4240f,0.1897f,-0.8856f,  0.5211f,0.1013f,-0.8475f,  0.5205f,0.1105f,-0.8467f,  0.6088f,0.0210f,-0.7930f,  0.6082f,0.0295f,-0.7932f,  0.6861f,-0.0596f,-0.7250f,  0.6855f,-0.0521f,-0.7262f,  0.7517f,-0.1392f,-0.6446f,  
	0.7510f,-0.1327f,-0.6468f,  0.8045f,-0.2165f,-0.5532f,  0.8037f,-0.2111f,-0.5563f,  0.8434f,-0.2900f,-0.4523f,  0.8426f,-0.2859f,-0.4563f,  0.8680f,-0.3586f,-0.3436f,  0.8672f,-0.3557f,-0.3485f,  0.8777f,-0.4210f,-0.2291f,  0.8769f,-0.4195f,-0.2347f,  0.8723f,-0.4762f,-0.1106f,  0.8716f,-0.4761f,-0.1169f,  0.8521f,-0.5233f,0.0098f,  0.8514f,-0.5246f,0.0029f,  0.8173f,-0.5614f,0.1300f,  0.8166f,-0.5641f,0.1227f,  
	0.7685f,-0.5899f,0.2479f,  0.7678f,-0.5939f,0.2404f,  0.7065f,-0.6083f,0.3617f,  0.7059f,-0.6135f,0.3539f,  0.7065f,-0.6083f,0.3617f,  0.7072f,-0.6033f,0.3687f,  0.6330f,-0.6102f,0.4763f,  0.6338f,-0.6044f,0.4828f,  0.5488f,-0.5999f,0.5822f,  0.5495f,-0.5935f,0.5880f,  0.4551f,-0.5781f,0.6773f,  0.4560f,-0.5713f,0.6824f,  0.3538f,-0.5454f,0.7598f,  0.3548f,-0.5385f,0.7643f,  0.2465f,-0.5029f,0.8285f,  
	0.2476f,-0.4961f,0.8322f,  0.1350f,-0.4516f,0.8820f,  0.1362f,-0.4449f,0.8852f,  0.0214f,-0.3925f,0.9195f,  0.0227f,-0.3862f,0.9221f,  -0.0926f,-0.3271f,0.9404f,  -0.0912f,-0.3212f,0.9426f,  -0.2049f,-0.2566f,0.9445f,  -0.2035f,-0.2511f,0.9463f,  -0.3138f,-0.1822f,0.9318f,  -0.3122f,-0.1773f,0.9333f,  -0.4172f,-0.1055f,0.9026f,  -0.4157f,-0.1010f,0.9039f,  -0.5137f,-0.0277f,0.8575f,  -0.5121f,-0.0237f,0.8586f,  
	-0.6014f,0.0498f,0.7974f,  -0.5999f,0.0534f,0.7983f,  -0.6790f,0.1256f,0.7233f,  -0.6775f,0.1288f,0.7241f,  -0.7452f,0.1986f,0.6366f,  -0.7437f,0.2015f,0.6374f,  -0.7988f,0.2675f,0.5388f,  -0.7975f,0.2701f,0.5396f,  -0.8391f,0.3312f,0.4316f,  -0.8377f,0.3335f,0.4324f,  -0.8652f,0.3886f,0.3169f,  -0.8640f,0.3907f,0.3176f,  -0.8768f,0.4388f,0.1967f,  -0.8756f,0.4408f,0.1973f,  -0.8737f,0.4810f,0.0729f,  
	-0.8726f,0.4829f,0.0735f,  -0.8559f,0.5145f,-0.0524f,  -0.8549f,0.5163f,-0.0518f,  -0.8237f,0.5387f,-0.1769f,  -0.8227f,0.5403f,-0.1765f,  -0.7778f,0.5531f,-0.2987f,  -0.7768f,0.5546f,-0.2984f,  -0.7187f,0.5574f,-0.4156f,  -0.7177f,0.5588f,-0.4154f,  -0.6476f,0.5516f,-0.5256f,  -0.6466f,0.5528f,-0.5256f,  -0.5657f,0.5356f,-0.6270f,  -0.5647f,0.5365f,-0.6272f,  -0.4744f,0.5095f,-0.7179f,  -0.4733f,0.5100f,-0.7182f,  
	-0.3752f,0.4737f,-0.7967f,  -0.3742f,0.4737f,-0.7972f,  -0.2699f,0.4287f,-0.8622f,  -0.2689f,0.4281f,-0.8628f,  -0.1604f,0.3750f,-0.9130f,  -0.1593f,0.3737f,-0.9138f,  -0.0485f,0.3135f,-0.9484f,  -0.0476f,0.3114f,-0.9491f,  0.0637f,0.2450f,-0.9674f,  0.0644f,0.2420f,-0.9681f,  0.1741f,0.1708f,-0.9698f,  0.1747f,0.1667f,-0.9704f,  0.2808f,0.0918f,-0.9554f,  0.2811f,0.0867f,-0.9558f,  0.3819f,0.0095f,-0.9242f,  
	0.3817f,0.0033f,-0.9243f,  0.4753f,-0.0746f,-0.8767f,  0.4746f,-0.0819f,-0.8764f,  0.5594f,-0.1591f,-0.8135f,  0.5580f,-0.1675f,-0.8127f,  0.6325f,-0.2424f,-0.7356f,  0.6303f,-0.2518f,-0.7343f,  0.6933f,-0.3229f,-0.6443f,  0.6902f,-0.3331f,-0.6424f,  0.7405f,-0.3987f,-0.5410f,  0.7364f,-0.4097f,-0.5384f,  0.7733f,-0.4684f,-0.4274f,  0.7681f,-0.4798f,-0.4241f,  0.7910f,-0.5301f,-0.3055f,  0.7846f,-0.5419f,-0.3014f,  
	0.7932f,-0.5825f,-0.1774f,  0.7857f,-0.5941f,-0.1725f,  0.7802f,-0.6240f,-0.0452f,  0.7713f,-0.6352f,-0.0396f,  0.7520f,-0.6532f,0.0884f,  0.7421f,-0.6636f,0.0947f,  0.7095f,-0.6691f,0.2212f,  0.6985f,-0.6783f,0.2280f,  0.6537f,-0.6706f,0.3506f,  0.6419f,-0.6782f,0.3577f,  0.5860f,-0.6572f,0.4740f,  0.5736f,-0.6628f,0.4813f,  -0.7682f,0.5902f,-0.2481f,  -0.7069f,0.6035f,-0.3689f,  -0.7062f,0.6086f,-0.3618f,  
	-0.6328f,0.6104f,-0.4764f,  -0.6321f,0.6166f,-0.4693f,  -0.5479f,0.6069f,-0.5758f,  -0.5473f,0.6140f,-0.5688f,  -0.4535f,0.5930f,-0.6653f,  -0.4530f,0.6009f,-0.6585f,  -0.3515f,0.5689f,-0.7435f,  -0.3510f,0.5776f,-0.7370f,  -0.2434f,0.5351f,-0.8089f,  -0.2431f,0.5444f,-0.8028f,  -0.1312f,0.4922f,-0.8605f,  -0.1309f,0.5019f,-0.8550f,  -0.0167f,0.4409f,-0.8974f,  -0.0165f,0.4508f,-0.8925f,  0.0981f,0.3820f,-0.9190f,  
	0.0981f,0.3919f,-0.9147f,  0.2112f,0.3165f,-0.9248f,  0.2111f,0.3264f,-0.9213f,  0.3206f,0.2457f,-0.9148f,  0.3205f,0.2553f,-0.9122f,  0.4246f,0.1707f,-0.8891f,  0.4244f,0.1798f,-0.8874f,  0.5214f,0.0927f,-0.8483f,  0.5211f,0.1013f,-0.8475f,  0.6092f,0.0132f,-0.7929f,  0.6088f,0.0210f,-0.7930f,  0.6866f,-0.0666f,-0.7240f,  0.6861f,-0.0596f,-0.7250f,  0.7523f,-0.1452f,-0.6426f,  0.7517f,-0.1392f,-0.6446f,  
	0.8051f,-0.2214f,-0.5503f,  0.8045f,-0.2165f,-0.5532f,  0.8441f,-0.2937f,-0.4486f,  0.8434f,-0.2900f,-0.4523f,  0.8687f,-0.3611f,-0.3392f,  0.8680f,-0.3586f,-0.3436f,  0.8784f,-0.4222f,-0.2239f,  0.8777f,-0.4210f,-0.2291f,  0.8731f,-0.4761f,-0.1049f,  0.8723f,-0.4762f,-0.1106f,  0.8528f,-0.5219f,0.0160f,  0.8521f,-0.5233f,0.0098f,  0.8180f,-0.5588f,0.1366f,  0.8173f,-0.5614f,0.1300f,  0.7692f,-0.5860f,0.2548f,  
	0.7685f,-0.5899f,0.2479f,  0.7072f,-0.6033f,0.3687f,  0.7065f,-0.6083f,0.3617f,  0.7072f,-0.6033f,0.3687f,  0.7079f,-0.5984f,0.3752f,  0.6338f,-0.6044f,0.4828f,  0.6345f,-0.5988f,0.4887f,  0.5495f,-0.5935f,0.5880f,  0.5504f,-0.5874f,0.5933f,  0.4560f,-0.5713f,0.6824f,  0.4570f,-0.5650f,0.6870f,  0.3548f,-0.5385f,0.7643f,  0.3559f,-0.5321f,0.7683f,  0.2476f,-0.4961f,0.8322f,  0.2488f,-0.4897f,0.8357f,  
	0.1362f,-0.4449f,0.8852f,  0.1376f,-0.4387f,0.8880f,  0.0227f,-0.3862f,0.9221f,  0.0241f,-0.3803f,0.9245f,  -0.0912f,-0.3212f,0.9426f,  -0.0897f,-0.3156f,0.9446f,  -0.2035f,-0.2511f,0.9463f,  -0.2019f,-0.2460f,0.9480f,  -0.3122f,-0.1773f,0.9333f,  -0.3106f,-0.1725f,0.9347f,  -0.4157f,-0.1010f,0.9039f,  -0.4141f,-0.0967f,0.9051f,  -0.5121f,-0.0237f,0.8586f,  -0.5105f,-0.0198f,0.8597f,  -0.5999f,0.0534f,0.7983f,  
	-0.5982f,0.0568f,0.7993f,  -0.6775f,0.1288f,0.7241f,  -0.6759f,0.1320f,0.7251f,  -0.7437f,0.2015f,0.6374f,  -0.7422f,0.2043f,0.6383f,  -0.7975f,0.2701f,0.5396f,  -0.7960f,0.2727f,0.5404f,  -0.8377f,0.3335f,0.4324f,  -0.8363f,0.3360f,0.4332f,  -0.8640f,0.3907f,0.3176f,  -0.8626f,0.3931f,0.3184f,  -0.8756f,0.4408f,0.1973f,  -0.8743f,0.4431f,0.1981f,  -0.8726f,0.4829f,0.0735f,  -0.8713f,0.4851f,0.0741f,  
	-0.8549f,0.5163f,-0.0518f,  -0.8536f,0.5184f,-0.0513f,  -0.8227f,0.5403f,-0.1765f,  -0.8215f,0.5423f,-0.1761f,  -0.7768f,0.5546f,-0.2984f,  -0.7755f,0.5565f,-0.2981f,  -0.7177f,0.5588f,-0.4154f,  -0.7165f,0.5605f,-0.4153f,  -0.6466f,0.5528f,-0.5256f,  -0.6454f,0.5542f,-0.5257f,  -0.5647f,0.5365f,-0.6272f,  -0.5634f,0.5375f,-0.6274f,  -0.4733f,0.5100f,-0.7182f,  -0.4720f,0.5106f,-0.7187f,  -0.3742f,0.4737f,-0.7972f,  
	-0.3728f,0.4738f,-0.7978f,  -0.2689f,0.4281f,-0.8628f,  -0.2675f,0.4275f,-0.8635f,  -0.1593f,0.3737f,-0.9138f,  -0.1580f,0.3724f,-0.9145f,  -0.0476f,0.3114f,-0.9491f,  -0.0463f,0.3091f,-0.9499f,  0.0644f,0.2420f,-0.9681f,  0.0655f,0.2388f,-0.9689f,  0.1747f,0.1667f,-0.9704f,  0.1755f,0.1624f,-0.9710f,  0.2811f,0.0867f,-0.9558f,  0.2815f,0.0812f,-0.9561f,  0.3817f,0.0033f,-0.9243f,  0.3817f,-0.0033f,-0.9243f,  
	0.4746f,-0.0819f,-0.8764f,  0.4740f,-0.0898f,-0.8760f,  0.5580f,-0.1675f,-0.8127f,  0.5566f,-0.1765f,-0.8118f,  0.6303f,-0.2518f,-0.7343f,  0.6281f,-0.2619f,-0.7327f,  0.6902f,-0.3331f,-0.6424f,  0.6870f,-0.3441f,-0.6401f,  0.7364f,-0.4097f,-0.5384f,  0.7320f,-0.4214f,-0.5353f,  0.7681f,-0.4798f,-0.4241f,  0.7625f,-0.4920f,-0.4202f,  0.7846f,-0.5419f,-0.3014f,  0.7777f,-0.5542f,-0.2967f,  0.7857f,-0.5941f,-0.1725f,  
	0.7775f,-0.6063f,-0.1670f,  0.7713f,-0.6352f,-0.0396f,  0.7619f,-0.6468f,-0.0335f,  0.7421f,-0.6636f,0.0947f,  0.7314f,-0.6744f,0.1014f,  0.6985f,-0.6783f,0.2280f,  0.6868f,-0.6877f,0.2352f,  0.6419f,-0.6782f,0.3577f,  0.6294f,-0.6859f,0.3653f,  0.5736f,-0.6628f,0.4813f,  0.5605f,-0.6683f,0.4890f,  -0.7069f,0.6035f,-0.3689f,  -0.6335f,0.6045f,-0.4829f,  -0.6328f,0.6104f,-0.4764f,  -0.5485f,0.6001f,-0.5822f,  
	-0.5479f,0.6069f,-0.5758f,  -0.4542f,0.5854f,-0.6716f,  -0.4535f,0.5930f,-0.6653f,  -0.3521f,0.5607f,-0.7494f,  -0.3515f,0.5689f,-0.7435f,  -0.2439f,0.5264f,-0.8145f,  -0.2434f,0.5351f,-0.8089f,  -0.1316f,0.4831f,-0.8656f,  -0.1312f,0.4922f,-0.8605f,  -0.0170f,0.4315f,-0.9019f,  -0.0167f,0.4409f,-0.8974f,  0.0978f,0.3726f,-0.9228f,  0.0981f,0.3820f,-0.9190f,  0.2110f,0.3073f,-0.9279f,  0.2112f,0.3165f,-0.9248f,  
	0.3206f,0.2367f,-0.9172f,  0.3206f,0.2457f,-0.9148f,  0.4247f,0.1621f,-0.8907f,  0.4246f,0.1707f,-0.8891f,  0.5215f,0.0847f,-0.8490f,  0.5214f,0.0927f,-0.8483f,  0.6094f,0.0058f,-0.7928f,  0.6092f,0.0132f,-0.7929f,  0.6869f,-0.0731f,-0.7230f,  0.6866f,-0.0666f,-0.7240f,  0.7527f,-0.1508f,-0.6409f,  0.7523f,-0.1452f,-0.6426f,  0.8056f,-0.2259f,-0.5477f,  0.8051f,-0.2214f,-0.5503f,  0.8447f,-0.2972f,-0.4453f,  
	0.8441f,-0.2937f,-0.4486f,  0.8693f,-0.3633f,-0.3351f,  0.8687f,-0.3611f,-0.3392f,  0.8791f,-0.4233f,-0.2193f,  0.8784f,-0.4222f,-0.2239f,  0.8738f,-0.4760f,-0.0997f,  0.8731f,-0.4761f,-0.1049f,  0.8536f,-0.5205f,0.0217f,  0.8528f,-0.5219f,0.0160f,  0.8187f,-0.5562f,0.1426f,  0.8180f,-0.5588f,0.1366f,  0.7699f,-0.5823f,0.2611f,  0.7692f,-0.5860f,0.2548f,  0.7079f,-0.5984f,0.3752f,  0.7072f,-0.6033f,0.3687f,  
	0.7079f,-0.5984f,0.3752f,  0.7087f,-0.5938f,0.3810f,  0.6345f,-0.5988f,0.4887f,  0.6354f,-0.5935f,0.4940f,  0.5504f,-0.5874f,0.5933f,  0.5514f,-0.5817f,0.5980f,  0.4570f,-0.5650f,0.6870f,  0.4581f,-0.5590f,0.6912f,  0.3559f,-0.5321f,0.7683f,  0.3571f,-0.5260f,0.7719f,  0.2488f,-0.4897f,0.8357f,  0.2501f,-0.4837f,0.8388f,  0.1376f,-0.4387f,0.8880f,  0.1390f,-0.4329f,0.8907f,  0.0241f,-0.3803f,0.9245f,  
	0.0256f,-0.3748f,0.9268f,  -0.0897f,-0.3156f,0.9446f,  -0.0881f,-0.3105f,0.9465f,  -0.2019f,-0.2460f,0.9480f,  -0.2002f,-0.2411f,0.9496f,  -0.3106f,-0.1725f,0.9347f,  -0.3089f,-0.1681f,0.9361f,  -0.4141f,-0.0967f,0.9051f,  -0.4123f,-0.0927f,0.9063f,  -0.5105f,-0.0198f,0.8597f,  -0.5087f,-0.0161f,0.8608f,  -0.5982f,0.0568f,0.7993f,  -0.5965f,0.0602f,0.8003f,  -0.6759f,0.1320f,0.7251f,  -0.6743f,0.1351f,0.7260f,  
	-0.7422f,0.2043f,0.6383f,  -0.7406f,0.2073f,0.6392f,  -0.7960f,0.2727f,0.5404f,  -0.7944f,0.2755f,0.5413f,  -0.8363f,0.3360f,0.4332f,  -0.8348f,0.3386f,0.4341f,  -0.8626f,0.3931f,0.3184f,  -0.8611f,0.3956f,0.3192f,  -0.8743f,0.4431f,0.1981f,  -0.8729f,0.4456f,0.1988f,  -0.8713f,0.4851f,0.0741f,  -0.8699f,0.4875f,0.0748f,  -0.8536f,0.5184f,-0.0513f,  -0.8522f,0.5207f,-0.0507f,  -0.8215f,0.5423f,-0.1761f,  
	-0.8201f,0.5446f,-0.1757f,  -0.7755f,0.5565f,-0.2981f,  -0.7741f,0.5586f,-0.2979f,  -0.7165f,0.5605f,-0.4153f,  -0.7150f,0.5625f,-0.4153f,  -0.6454f,0.5542f,-0.5257f,  -0.6438f,0.5559f,-0.5259f,  -0.5634f,0.5375f,-0.6274f,  -0.5618f,0.5388f,-0.6278f,  -0.4720f,0.5106f,-0.7187f,  -0.4703f,0.5115f,-0.7192f,  -0.3728f,0.4738f,-0.7978f,  -0.3710f,0.4740f,-0.7985f,  -0.2675f,0.4275f,-0.8635f,  -0.2657f,0.4270f,-0.8643f,  
	-0.1580f,0.3724f,-0.9145f,  -0.1563f,0.3709f,-0.9154f,  -0.0463f,0.3091f,-0.9499f,  -0.0447f,0.3067f,-0.9507f,  0.0655f,0.2388f,-0.9689f,  0.0669f,0.2353f,-0.9696f,  0.1755f,0.1624f,-0.9710f,  0.1766f,0.1577f,-0.9716f,  0.2815f,0.0812f,-0.9561f,  0.2822f,0.0753f,-0.9564f,  0.3817f,-0.0033f,-0.9243f,  0.3818f,-0.0105f,-0.9242f,  0.4740f,-0.0898f,-0.8760f,  0.4734f,-0.0982f,-0.8754f,  0.5566f,-0.1765f,-0.8118f,  
	0.5553f,-0.1861f,-0.8106f,  0.6281f,-0.2619f,-0.7327f,  0.6258f,-0.2725f,-0.7309f,  0.6870f,-0.3441f,-0.6401f,  0.6835f,-0.3556f,-0.6375f,  0.7320f,-0.4214f,-0.5353f,  0.7274f,-0.4337f,-0.5319f,  0.7625f,-0.4920f,-0.4202f,  0.7565f,-0.5047f,-0.4159f,  0.7777f,-0.5542f,-0.2967f,  0.7703f,-0.5671f,-0.2916f,  0.7775f,-0.6063f,-0.1670f,  0.7687f,-0.6190f,-0.1611f,  0.7619f,-0.6468f,-0.0335f,  0.7517f,-0.6589f,-0.0268f,  
	0.7314f,-0.6744f,0.1014f,  0.7200f,-0.6854f,0.1088f,  0.6868f,-0.6877f,0.2352f,  0.6744f,-0.6972f,0.2430f,  0.6294f,-0.6859f,0.3653f,  0.6161f,-0.6935f,0.3734f,  0.5605f,-0.6683f,0.4890f,  0.5468f,-0.6737f,0.4972f,  -0.6335f,0.6045f,-0.4829f,  -0.5493f,0.5937f,-0.5880f,  -0.5485f,0.6001f,-0.5822f,  -0.4549f,0.5782f,-0.6773f,  -0.4542f,0.5854f,-0.6716f,  -0.3528f,0.5529f,-0.7549f,  -0.3521f,0.5607f,-0.7494f,  
	-0.2446f,0.5181f,-0.8196f,  -0.2439f,0.5264f,-0.8145f,  -0.1322f,0.4745f,-0.8703f,  -0.1316f,0.4831f,-0.8656f,  -0.0175f,0.4227f,-0.9061f,  -0.0170f,0.4315f,-0.9019f,  0.0974f,0.3638f,-0.9264f,  0.0978f,0.3726f,-0.9228f,  0.2107f,0.2985f,-0.9309f,  0.2110f,0.3073f,-0.9279f,  0.3204f,0.2282f,-0.9194f,  0.3206f,0.2367f,-0.9172f,  0.4246f,0.1540f,-0.8922f,  0.4247f,0.1621f,-0.8907f,  0.5215f,0.0772f,-0.8498f,  
	0.5215f,0.0847f,-0.8490f,  0.6095f,-0.0010f,-0.7928f,  0.6094f,0.0058f,-0.7928f,  0.6871f,-0.0792f,-0.7222f,  0.6869f,-0.0731f,-0.7230f,  0.7530f,-0.1560f,-0.6393f,  0.7527f,-0.1508f,-0.6409f,  0.8059f,-0.2301f,-0.5455f,  0.8056f,-0.2259f,-0.5477f,  0.8451f,-0.3003f,-0.4423f,  0.8447f,-0.2972f,-0.4453f,  0.8698f,-0.3654f,-0.3315f,  0.8693f,-0.3633f,-0.3351f,  0.8797f,-0.4242f,-0.2151f,  0.8791f,-0.4233f,-0.2193f,  
	0.8745f,-0.4757f,-0.0950f,  0.8738f,-0.4760f,-0.0997f,  0.8543f,-0.5191f,0.0268f,  0.8536f,-0.5205f,0.0217f,  0.8195f,-0.5536f,0.1481f,  0.8187f,-0.5562f,0.1426f,  0.7707f,-0.5787f,0.2668f,  0.7699f,-0.5823f,0.2611f,  0.7087f,-0.5938f,0.3810f,  0.7079f,-0.5984f,0.3752f,  0.7087f,-0.5938f,0.3810f,  0.7095f,-0.5894f,0.3863f,  0.6354f,-0.5935f,0.4940f,  0.6363f,-0.5885f,0.4987f,  0.5514f,-0.5817f,0.5980f,  
	0.5524f,-0.5763f,0.6022f,  0.4581f,-0.5590f,0.6912f,  0.4592f,-0.5534f,0.6949f,  0.3571f,-0.5260f,0.7719f,  0.3583f,-0.5204f,0.7751f,  0.2501f,-0.4837f,0.8388f,  0.2515f,-0.4781f,0.8415f,  0.1390f,-0.4329f,0.8907f,  0.1405f,-0.4275f,0.8930f,  0.0256f,-0.3748f,0.9268f,  0.0272f,-0.3697f,0.9288f,  -0.0881f,-0.3105f,0.9465f,  -0.0864f,-0.3056f,0.9482f,  -0.2002f,-0.2411f,0.9496f,  -0.1985f,-0.2366f,0.9511f,  
	-0.3089f,-0.1681f,0.9361f,  -0.3071f,-0.1639f,0.9374f,  -0.4123f,-0.0927f,0.9063f,  -0.4105f,-0.0888f,0.9075f,  -0.5087f,-0.0161f,0.8608f,  -0.5070f,-0.0125f,0.8619f,  -0.5965f,0.0602f,0.8003f,  -0.5948f,0.0636f,0.8014f,  -0.6743f,0.1351f,0.7260f,  -0.6725f,0.1383f,0.7271f,  -0.7406f,0.2073f,0.6392f,  -0.7389f,0.2103f,0.6402f,  -0.7944f,0.2755f,0.5413f,  -0.7927f,0.2784f,0.5423f,  -0.8348f,0.3386f,0.4341f,  
	-0.8332f,0.3414f,0.4350f,  -0.8611f,0.3956f,0.3192f,  -0.8595f,0.3984f,0.3201f,  -0.8729f,0.4456f,0.1988f,  -0.8713f,0.4483f,0.1996f,  -0.8699f,0.4875f,0.0748f,  -0.8683f,0.4902f,0.0754f,  -0.8522f,0.5207f,-0.0507f,  -0.8506f,0.5234f,-0.0502f,  -0.8201f,0.5446f,-0.1757f,  -0.8184f,0.5472f,-0.1753f,  -0.7741f,0.5586f,-0.2979f,  -0.7724f,0.5611f,-0.2977f,  -0.7150f,0.5625f,-0.4153f,  -0.7132f,0.5647f,-0.4153f,  
	-0.6438f,0.5559f,-0.5259f,  -0.6419f,0.5578f,-0.5261f,  -0.5618f,0.5388f,-0.6278f,  -0.5598f,0.5403f,-0.6282f,  -0.4703f,0.5115f,-0.7192f,  -0.4682f,0.5124f,-0.7199f,  -0.3710f,0.4740f,-0.7985f,  -0.3689f,0.4743f,-0.7993f,  -0.2657f,0.4270f,-0.8643f,  -0.2636f,0.4264f,-0.8652f,  -0.1563f,0.3709f,-0.9154f,  -0.1543f,0.3695f,-0.9164f,  -0.0447f,0.3067f,-0.9507f,  -0.0428f,0.3042f,-0.9517f,  0.0669f,0.2353f,-0.9696f,  
	0.0686f,0.2315f,-0.9704f,  0.1766f,0.1577f,-0.9716f,  0.1780f,0.1527f,-0.9721f,  0.2822f,0.0753f,-0.9564f,  0.2831f,0.0690f,-0.9566f,  0.3818f,-0.0105f,-0.9242f,  0.3821f,-0.0181f,-0.9239f,  0.4734f,-0.0982f,-0.8754f,  0.4729f,-0.1071f,-0.8746f,  0.5553f,-0.1861f,-0.8106f,  0.5539f,-0.1963f,-0.8091f,  0.6258f,-0.2725f,-0.7309f,  0.6233f,-0.2839f,-0.7287f,  0.6835f,-0.3556f,-0.6375f,  0.6798f,-0.3679f,-0.6344f,  
	0.7274f,-0.4337f,-0.5319f,  0.7223f,-0.4467f,-0.5280f,  0.7565f,-0.5047f,-0.4159f,  0.7500f,-0.5182f,-0.4111f,  0.7703f,-0.5671f,-0.2916f,  0.7624f,-0.5806f,-0.2859f,  0.7687f,-0.6190f,-0.1611f,  0.7592f,-0.6322f,-0.1546f,  0.7517f,-0.6589f,-0.0268f,  0.7409f,-0.6714f,-0.0196f,  0.7200f,-0.6854f,0.1088f,  0.7078f,-0.6967f,0.1166f,  0.6744f,-0.6972f,0.2430f,  0.6611f,-0.7069f,0.2514f,  0.6161f,-0.6935f,0.3734f,  
	0.6021f,-0.7012f,0.3819f,  0.5468f,-0.6737f,0.4972f,  0.5323f,-0.6788f,0.5058f,  -0.5493f,0.5937f,-0.5880f,  -0.4558f,0.5715f,-0.6824f,  -0.4549f,0.5782f,-0.6773f,  -0.3536f,0.5456f,-0.7598f,  -0.3528f,0.5529f,-0.7549f,  -0.2454f,0.5104f,-0.8242f,  -0.2446f,0.5181f,-0.8196f,  -0.1330f,0.4664f,-0.8745f,  -0.1322f,0.4745f,-0.8703f,  -0.0183f,0.4145f,-0.9099f,  -0.0175f,0.4227f,-0.9061f,  0.0968f,0.3555f,-0.9297f,  
	0.0974f,0.3638f,-0.9264f,  0.2101f,0.2904f,-0.9336f,  0.2107f,0.2985f,-0.9309f,  0.3199f,0.2203f,-0.9215f,  0.3204f,0.2282f,-0.9194f,  0.4242f,0.1465f,-0.8936f,  0.4246f,0.1540f,-0.8922f,  0.5213f,0.0702f,-0.8505f,  0.5215f,0.0772f,-0.8498f,  0.6094f,-0.0074f,-0.7928f,  0.6095f,-0.0010f,-0.7928f,  0.6871f,-0.0848f,-0.7216f,  0.6871f,-0.0792f,-0.7222f,  0.7531f,-0.1608f,-0.6380f,  0.7530f,-0.1560f,-0.6393f,  
	0.8061f,-0.2340f,-0.5435f,  0.8059f,-0.2301f,-0.5455f,  0.8454f,-0.3032f,-0.4397f,  0.8451f,-0.3003f,-0.4423f,  0.8703f,-0.3672f,-0.3283f,  0.8698f,-0.3654f,-0.3315f,  0.8802f,-0.4249f,-0.2114f,  0.8797f,-0.4242f,-0.2151f,  0.8751f,-0.4754f,-0.0908f,  0.8745f,-0.4757f,-0.0950f,  0.8550f,-0.5177f,0.0313f,  0.8543f,-0.5191f,0.0268f,  0.8203f,-0.5512f,0.1529f,  0.8195f,-0.5536f,0.1481f,  0.7715f,-0.5752f,0.2719f,  
	0.7707f,-0.5787f,0.2668f,  0.7095f,-0.5894f,0.3863f,  0.7087f,-0.5938f,0.3810f,  0.7095f,-0.5894f,0.3863f,  0.7104f,-0.5852f,0.3910f,  0.6363f,-0.5885f,0.4987f,  0.6373f,-0.5838f,0.5030f,  0.5524f,-0.5763f,0.6022f,  0.5535f,-0.5713f,0.6060f,  0.4592f,-0.5534f,0.6949f,  0.4605f,-0.5482f,0.6982f,  0.3583f,-0.5204f,0.7751f,  0.3597f,-0.5151f,0.7780f,  0.2515f,-0.4781f,0.8415f,  0.2530f,-0.4729f,0.8440f,  
	0.1405f,-0.4275f,0.8930f,  0.1421f,-0.4225f,0.8952f,  0.0272f,-0.3697f,0.9288f,  0.0289f,-0.3649f,0.9306f,  -0.0864f,-0.3056f,0.9482f,  -0.0847f,-0.3011f,0.9498f,  -0.1985f,-0.2366f,0.9511f,  -0.1967f,-0.2324f,0.9525f,  -0.3071f,-0.1639f,0.9374f,  -0.3053f,-0.1600f,0.9387f,  -0.4105f,-0.0888f,0.9075f,  -0.4087f,-0.0851f,0.9087f,  -0.5070f,-0.0125f,0.8619f,  -0.5051f,-0.0090f,0.8630f,  -0.5948f,0.0636f,0.8014f,  
	-0.5929f,0.0669f,0.8025f,  -0.6725f,0.1383f,0.7271f,  -0.6707f,0.1414f,0.7281f,  -0.7389f,0.2103f,0.6402f,  -0.7371f,0.2133f,0.6412f,  -0.7927f,0.2784f,0.5423f,  -0.7910f,0.2814f,0.5433f,  -0.8332f,0.3414f,0.4350f,  -0.8315f,0.3445f,0.4359f,  -0.8595f,0.3984f,0.3201f,  -0.8578f,0.4015f,0.3210f,  -0.8713f,0.4483f,0.1996f,  -0.8696f,0.4514f,0.2003f,  -0.8683f,0.4902f,0.0754f,  -0.8665f,0.4933f,0.0761f,  
	-0.8506f,0.5234f,-0.0502f,  -0.8488f,0.5264f,-0.0498f,  -0.8184f,0.5472f,-0.1753f,  -0.8165f,0.5501f,-0.1751f,  -0.7724f,0.5611f,-0.2977f,  -0.7704f,0.5639f,-0.2977f,  -0.7132f,0.5647f,-0.4153f,  -0.7111f,0.5672f,-0.4155f,  -0.6419f,0.5578f,-0.5261f,  -0.6397f,0.5600f,-0.5266f,  -0.5598f,0.5403f,-0.6282f,  -0.5574f,0.5420f,-0.6289f,  -0.4682f,0.5124f,-0.7199f,  -0.4658f,0.5135f,-0.7207f,  -0.3689f,0.4743f,-0.7993f,  
	-0.3665f,0.4746f,-0.8003f,  -0.2636f,0.4264f,-0.8652f,  -0.2611f,0.4258f,-0.8663f,  -0.1543f,0.3695f,-0.9164f,  -0.1518f,0.3678f,-0.9174f,  -0.0428f,0.3042f,-0.9517f,  -0.0406f,0.3014f,-0.9526f,  0.0686f,0.2315f,-0.9704f,  0.0706f,0.2275f,-0.9712f,  0.1780f,0.1527f,-0.9721f,  0.1796f,0.1473f,-0.9726f,  0.2831f,0.0690f,-0.9566f,  0.2842f,0.0622f,-0.9567f,  0.3821f,-0.0181f,-0.9239f,  0.3825f,-0.0264f,-0.9236f,  
	0.4729f,-0.1071f,-0.8746f,  0.4725f,-0.1168f,-0.8736f,  0.5539f,-0.1963f,-0.8091f,  0.5524f,-0.2072f,-0.8074f,  0.6233f,-0.2839f,-0.7287f,  0.6206f,-0.2959f,-0.7261f,  0.6798f,-0.3679f,-0.6344f,  0.6758f,-0.3810f,-0.6310f,  0.7223f,-0.4467f,-0.5280f,  0.7168f,-0.4604f,-0.5236f,  0.7500f,-0.5182f,-0.4111f,  0.7430f,-0.5322f,-0.4058f,  0.7624f,-0.5806f,-0.2859f,  0.7537f,-0.5947f,-0.2797f,  0.7592f,-0.6322f,-0.1546f,  
	0.7491f,-0.6459f,-0.1475f,  0.7409f,-0.6714f,-0.0196f,  0.7292f,-0.6842f,-0.0118f,  0.7078f,-0.6967f,0.1166f,  0.6949f,-0.7082f,0.1250f,  0.6611f,-0.7069f,0.2514f,  0.6471f,-0.7166f,0.2602f,  0.6021f,-0.7012f,0.3819f,  0.5873f,-0.7086f,0.3910f,  0.5323f,-0.6788f,0.5058f,  0.5173f,-0.6836f,0.5149f,  -0.4558f,0.5715f,-0.6824f,  -0.3546f,0.5387f,-0.7642f,  -0.3536f,0.5456f,-0.7598f,  -0.2464f,0.5031f,-0.8284f,  
	-0.2454f,0.5104f,-0.8242f,  -0.1339f,0.4588f,-0.8784f,  -0.1330f,0.4664f,-0.8745f,  -0.0191f,0.4068f,-0.9133f,  -0.0183f,0.4145f,-0.9099f,  0.0960f,0.3477f,-0.9327f,  0.0968f,0.3555f,-0.9297f,  0.2094f,0.2827f,-0.9361f,  0.2101f,0.2904f,-0.9336f,  0.3193f,0.2129f,-0.9234f,  0.3199f,0.2203f,-0.9215f,  0.4237f,0.1395f,-0.8950f,  0.4242f,0.1465f,-0.8936f,  0.5209f,0.0636f,-0.8513f,  0.5213f,0.0702f,-0.8505f,  
	0.6091f,-0.0133f,-0.7930f,  0.6094f,-0.0074f,-0.7928f,  0.6869f,-0.0900f,-0.7211f,  0.6871f,-0.0848f,-0.7216f,  0.7530f,-0.1652f,-0.6369f,  0.7531f,-0.1608f,-0.6380f,  0.8062f,-0.2375f,-0.5418f,  0.8061f,-0.2340f,-0.5435f,  0.8457f,-0.3058f,-0.4374f,  0.8454f,-0.3032f,-0.4397f,  0.8706f,-0.3689f,-0.3256f,  0.8703f,-0.3672f,-0.3283f,  0.8807f,-0.4256f,-0.2081f,  0.8802f,-0.4249f,-0.2114f,  0.8756f,-0.4750f,-0.0871f,  
	0.8751f,-0.4754f,-0.0908f,  0.8557f,-0.5163f,0.0354f,  0.8550f,-0.5177f,0.0313f,  0.8210f,-0.5488f,0.1573f,  0.8203f,-0.5512f,0.1529f,  0.7723f,-0.5719f,0.2765f,  0.7715f,-0.5752f,0.2719f,  0.7104f,-0.5852f,0.3910f,  0.7095f,-0.5894f,0.3863f,  0.7104f,-0.5852f,0.3910f,  0.7113f,-0.5813f,0.3951f,  0.6373f,-0.5838f,0.5030f,  0.6384f,-0.5794f,0.5068f,  0.5535f,-0.5713f,0.6060f,  0.5547f,-0.5666f,0.6093f,  
	0.4605f,-0.5482f,0.6982f,  0.4617f,-0.5433f,0.7011f,  0.3597f,-0.5151f,0.7780f,  0.3611f,-0.5103f,0.7805f,  0.2530f,-0.4729f,0.8440f,  0.2545f,-0.4681f,0.8462f,  0.1421f,-0.4225f,0.8952f,  0.1437f,-0.4179f,0.8971f,  0.0289f,-0.3649f,0.9306f,  0.0306f,-0.3604f,0.9323f,  -0.0847f,-0.3011f,0.9498f,  -0.0829f,-0.2969f,0.9513f,  -0.1967f,-0.2324f,0.9525f,  -0.1948f,-0.2284f,0.9539f,  -0.3053f,-0.1600f,0.9387f,  
	-0.3034f,-0.1562f,0.9400f,  -0.4087f,-0.0851f,0.9087f,  -0.4068f,-0.0815f,0.9099f,  -0.5051f,-0.0090f,0.8630f,  -0.5032f,-0.0056f,0.8641f,  -0.5929f,0.0669f,0.8025f,  -0.5911f,0.0702f,0.8036f,  -0.6707f,0.1414f,0.7281f,  -0.6688f,0.1447f,0.7292f,  -0.7371f,0.2133f,0.6412f,  -0.7352f,0.2166f,0.6423f,  -0.7910f,0.2814f,0.5433f,  -0.7891f,0.2846f,0.5443f,  -0.8315f,0.3445f,0.4359f,  -0.8296f,0.3477f,0.4369f,  
	-0.8578f,0.4015f,0.3210f,  -0.8559f,0.4047f,0.3218f,  -0.8696f,0.4514f,0.2003f,  -0.8677f,0.4547f,0.2010f,  -0.8665f,0.4933f,0.0761f,  -0.8646f,0.4966f,0.0766f,  -0.8488f,0.5264f,-0.0498f,  -0.8467f,0.5298f,-0.0494f,  -0.8165f,0.5501f,-0.1751f,  -0.8144f,0.5534f,-0.1749f,  -0.7704f,0.5639f,-0.2977f,  -0.7681f,0.5669f,-0.2978f,  -0.7111f,0.5672f,-0.4155f,  -0.7086f,0.5700f,-0.4158f,  -0.6397f,0.5600f,-0.5266f,  
	-0.6371f,0.5624f,-0.5271f,  -0.5574f,0.5420f,-0.6289f,  -0.5547f,0.5439f,-0.6297f,  -0.4658f,0.5135f,-0.7207f,  -0.4630f,0.5147f,-0.7216f,  -0.3665f,0.4746f,-0.8003f,  -0.3636f,0.4750f,-0.8014f,  -0.2611f,0.4258f,-0.8663f,  -0.2582f,0.4252f,-0.8675f,  -0.1518f,0.3678f,-0.9174f,  -0.1490f,0.3661f,-0.9186f,  -0.0406f,0.3014f,-0.9526f,  -0.0379f,0.2984f,-0.9537f,  0.0706f,0.2275f,-0.9712f,  0.0730f,0.2231f,-0.9721f,  
	0.1796f,0.1473f,-0.9726f,  0.1815f,0.1415f,-0.9732f,  0.2842f,0.0622f,-0.9567f,  0.2855f,0.0549f,-0.9568f,  0.3825f,-0.0264f,-0.9236f,  0.3830f,-0.0352f,-0.9231f,  0.4725f,-0.1168f,-0.8736f,  0.4720f,-0.1270f,-0.8724f,  0.5524f,-0.2072f,-0.8074f,  0.5508f,-0.2189f,-0.8054f,  0.6206f,-0.2959f,-0.7261f,  0.6177f,-0.3087f,-0.7233f,  0.6758f,-0.3810f,-0.6310f,  0.6714f,-0.3947f,-0.6272f,  0.7168f,-0.4604f,-0.5236f,  
	0.7109f,-0.4748f,-0.5188f,  0.7430f,-0.5322f,-0.4058f,  0.7354f,-0.5470f,-0.4001f,  0.7537f,-0.5947f,-0.2797f,  0.7445f,-0.6093f,-0.2730f,  0.7491f,-0.6459f,-0.1475f,  0.7381f,-0.6600f,-0.1400f,  0.7292f,-0.6842f,-0.0118f,  0.7168f,-0.6973f,-0.0034f,  0.6949f,-0.7082f,0.1250f,  0.6811f,-0.7198f,0.1339f,  0.6471f,-0.7166f,0.2602f,  0.6323f,-0.7263f,0.2695f,  0.5873f,-0.7086f,0.3910f,  0.5719f,-0.7159f,0.4005f,  
	0.5173f,-0.6836f,0.5149f,  0.5017f,-0.6881f,0.5242f,  -0.3546f,0.5387f,-0.7642f,  -0.2475f,0.4962f,-0.8322f,  -0.2464f,0.5031f,-0.8284f,  -0.1350f,0.4517f,-0.8819f,  -0.1339f,0.4588f,-0.8784f,  -0.0202f,0.3995f,-0.9165f,  -0.0191f,0.4068f,-0.9133f,  0.0950f,0.3404f,-0.9355f,  0.0960f,0.3477f,-0.9327f,  0.2085f,0.2756f,-0.9384f,  0.2094f,0.2827f,-0.9361f,  0.3185f,0.2060f,-0.9253f,  0.3193f,0.2129f,-0.9234f,  
	0.4230f,0.1329f,-0.8963f,  0.4237f,0.1395f,-0.8950f,  0.5203f,0.0575f,-0.8521f,  0.5209f,0.0636f,-0.8513f,  0.6087f,-0.0189f,-0.7932f,  0.6091f,-0.0133f,-0.7930f,  0.6866f,-0.0949f,-0.7208f,  0.6869f,-0.0900f,-0.7211f,  0.7529f,-0.1693f,-0.6360f,  0.7530f,-0.1652f,-0.6369f,  0.8062f,-0.2409f,-0.5404f,  0.8062f,-0.2375f,-0.5418f,  0.8458f,-0.3083f,-0.4355f,  0.8457f,-0.3058f,-0.4374f,  0.8708f,-0.3704f,-0.3232f,  
	0.8706f,-0.3689f,-0.3256f,  0.8810f,-0.4262f,-0.2053f,  0.8807f,-0.4256f,-0.2081f,  0.8762f,-0.4747f,-0.0839f,  0.8756f,-0.4750f,-0.0871f,  0.8563f,-0.5150f,0.0389f,  0.8557f,-0.5163f,0.0354f,  0.8217f,-0.5466f,0.1611f,  0.8210f,-0.5488f,0.1573f,  0.7732f,-0.5688f,0.2805f,  0.7723f,-0.5719f,0.2765f,  0.7113f,-0.5813f,0.3951f,  0.7104f,-0.5852f,0.3910f,  0.7113f,-0.5813f,0.3951f,  0.7123f,-0.5776f,0.3988f,  
	0.6384f,-0.5794f,0.5068f,  0.6394f,-0.5753f,0.5101f,  0.5547f,-0.5666f,0.6093f,  0.5559f,-0.5622f,0.6123f,  0.4617f,-0.5433f,0.7011f,  0.4631f,-0.5388f,0.7037f,  0.3611f,-0.5103f,0.7805f,  0.3625f,-0.5058f,0.7828f,  0.2545f,-0.4681f,0.8462f,  0.2560f,-0.4637f,0.8482f,  0.1437f,-0.4179f,0.8971f,  0.1454f,-0.4136f,0.8988f,  0.0306f,-0.3604f,0.9323f,  0.0324f,-0.3563f,0.9338f,  -0.0829f,-0.2969f,0.9513f,  
	-0.0810f,-0.2929f,0.9527f,  -0.1948f,-0.2284f,0.9539f,  -0.1930f,-0.2246f,0.9551f,  -0.3034f,-0.1562f,0.9400f,  -0.3015f,-0.1526f,0.9412f,  -0.4068f,-0.0815f,0.9099f,  -0.4049f,-0.0780f,0.9110f,  -0.5032f,-0.0056f,0.8641f,  -0.5013f,-0.0022f,0.8653f,  -0.5911f,0.0702f,0.8036f,  -0.5891f,0.0735f,0.8047f,  -0.6688f,0.1447f,0.7292f,  -0.6669f,0.1480f,0.7303f,  -0.7352f,0.2166f,0.6423f,  -0.7333f,0.2199f,0.6433f,  
	-0.7891f,0.2846f,0.5443f,  -0.7872f,0.2880f,0.5453f,  -0.8296f,0.3477f,0.4369f,  -0.8277f,0.3512f,0.4378f,  -0.8559f,0.4047f,0.3218f,  -0.8539f,0.4083f,0.3226f,  -0.8677f,0.4547f,0.2010f,  -0.8656f,0.4583f,0.2017f,  -0.8646f,0.4966f,0.0766f,  -0.8624f,0.5003f,0.0771f,  -0.8467f,0.5298f,-0.0494f,  -0.8444f,0.5334f,-0.0492f,  -0.8144f,0.5534f,-0.1749f,  -0.8119f,0.5569f,-0.1749f,  -0.7681f,0.5669f,-0.2978f,  
	-0.7655f,0.5703f,-0.2980f,  -0.7086f,0.5700f,-0.4158f,  -0.7058f,0.5731f,-0.4163f,  -0.6371f,0.5624f,-0.5271f,  -0.6341f,0.5650f,-0.5279f,  -0.5547f,0.5439f,-0.6297f,  -0.5516f,0.5459f,-0.6306f,  -0.4630f,0.5147f,-0.7216f,  -0.4597f,0.5159f,-0.7228f,  -0.3636f,0.4750f,-0.8014f,  -0.3602f,0.4753f,-0.8027f,  -0.2582f,0.4252f,-0.8675f,  -0.2549f,0.4245f,-0.8688f,  -0.1490f,0.3661f,-0.9186f,  -0.1458f,0.3641f,-0.9199f,  
	-0.0379f,0.2984f,-0.9537f,  -0.0349f,0.2951f,-0.9548f,  0.0730f,0.2231f,-0.9721f,  0.0757f,0.2183f,-0.9729f,  0.1815f,0.1415f,-0.9732f,  0.1836f,0.1351f,-0.9737f,  0.2855f,0.0549f,-0.9568f,  0.2870f,0.0469f,-0.9568f,  0.3830f,-0.0352f,-0.9231f,  0.3836f,-0.0447f,-0.9224f,  0.4720f,-0.1270f,-0.8724f,  0.4715f,-0.1380f,-0.8710f,  0.5508f,-0.2189f,-0.8054f,  0.5490f,-0.2312f,-0.8032f,  0.6177f,-0.3087f,-0.7233f,  
	0.6145f,-0.3223f,-0.7201f,  0.6714f,-0.3947f,-0.6272f,  0.6667f,-0.4092f,-0.6230f,  0.7109f,-0.4748f,-0.5188f,  0.7044f,-0.4899f,-0.5136f,  0.7354f,-0.5470f,-0.4001f,  0.7271f,-0.5623f,-0.3938f,  0.7445f,-0.6093f,-0.2730f,  0.7344f,-0.6245f,-0.2658f,  0.7381f,-0.6600f,-0.1400f,  0.7264f,-0.6746f,-0.1318f,  0.7168f,-0.6973f,-0.0034f,  0.7034f,-0.7108f,0.0054f,  0.6811f,-0.7198f,0.1339f,  0.6664f,-0.7317f,0.1434f,  
	0.6323f,-0.7263f,0.2695f,  0.6166f,-0.7360f,0.2793f,  0.5719f,-0.7159f,0.4005f,  0.5557f,-0.7231f,0.4104f,  0.5017f,-0.6881f,0.5242f,  0.4855f,-0.6923f,0.5339f,  -0.2475f,0.4962f,-0.8322f,  -0.1361f,0.4451f,-0.8851f,  -0.1350f,0.4517f,-0.8819f,  -0.0213f,0.3927f,-0.9194f,  -0.0202f,0.3995f,-0.9165f,  0.0939f,0.3336f,-0.9380f,  0.0950f,0.3404f,-0.9355f,  0.2075f,0.2689f,-0.9406f,  0.2085f,0.2756f,-0.9384f,  
	0.3175f,0.1995f,-0.9270f,  0.3185f,0.2060f,-0.9253f,  0.4221f,0.1267f,-0.8976f,  0.4230f,0.1329f,-0.8963f,  0.5195f,0.0518f,-0.8529f,  0.5203f,0.0575f,-0.8521f,  0.6080f,-0.0241f,-0.7935f,  0.6087f,-0.0189f,-0.7932f,  0.6862f,-0.0995f,-0.7206f,  0.6866f,-0.0949f,-0.7208f,  0.7525f,-0.1732f,-0.6354f,  0.7529f,-0.1693f,-0.6360f,  0.8060f,-0.2440f,-0.5392f,  0.8062f,-0.2409f,-0.5404f,  0.8457f,-0.3106f,-0.4339f,  
	0.8458f,-0.3083f,-0.4355f,  0.8710f,-0.3718f,-0.3211f,  0.8708f,-0.3704f,-0.3232f,  0.8813f,-0.4267f,-0.2028f,  0.8810f,-0.4262f,-0.2053f,  0.8766f,-0.4743f,-0.0811f,  0.8762f,-0.4747f,-0.0839f,  0.8569f,-0.5138f,0.0420f,  0.8563f,-0.5150f,0.0389f,  0.8225f,-0.5445f,0.1645f,  0.8217f,-0.5466f,0.1611f,  0.7740f,-0.5659f,0.2841f,  0.7732f,-0.5688f,0.2805f,  0.7123f,-0.5776f,0.3988f,  0.7113f,-0.5813f,0.3951f,  
	0.7123f,-0.5776f,0.3988f,  0.7132f,-0.5742f,0.4021f,  0.6394f,-0.5753f,0.5101f,  0.6405f,-0.5715f,0.5130f,  0.5559f,-0.5622f,0.6123f,  0.5571f,-0.5582f,0.6149f,  0.4631f,-0.5388f,0.7037f,  0.4644f,-0.5347f,0.7060f,  0.3625f,-0.5058f,0.7828f,  0.3640f,-0.5016f,0.7848f,  0.2560f,-0.4637f,0.8482f,  0.2576f,-0.4596f,0.8499f,  0.1454f,-0.4136f,0.8988f,  0.1471f,-0.4096f,0.9003f,  0.0324f,-0.3563f,0.9338f,  
	0.0341f,-0.3524f,0.9352f,  -0.0810f,-0.2929f,0.9527f,  -0.0792f,-0.2892f,0.9540f,  -0.1930f,-0.2246f,0.9551f,  -0.1911f,-0.2211f,0.9564f,  -0.3015f,-0.1526f,0.9412f,  -0.2996f,-0.1491f,0.9423f,  -0.4049f,-0.0780f,0.9110f,  -0.4029f,-0.0746f,0.9122f,  -0.5013f,-0.0022f,0.8653f,  -0.4994f,0.0011f,0.8664f,  -0.5891f,0.0735f,0.8047f,  -0.5872f,0.0769f,0.8058f,  -0.6669f,0.1480f,0.7303f,  -0.6650f,0.1514f,0.7314f,  
	-0.7333f,0.2199f,0.6433f,  -0.7314f,0.2234f,0.6444f,  -0.7872f,0.2880f,0.5453f,  -0.7852f,0.2917f,0.5462f,  -0.8277f,0.3512f,0.4378f,  -0.8256f,0.3550f,0.4386f,  -0.8539f,0.4083f,0.3226f,  -0.8518f,0.4122f,0.3233f,  -0.8656f,0.4583f,0.2017f,  -0.8634f,0.4623f,0.2022f,  -0.8624f,0.5003f,0.0771f,  -0.8600f,0.5043f,0.0774f,  -0.8444f,0.5334f,-0.0492f,  -0.8419f,0.5374f,-0.0491f,  -0.8119f,0.5569f,-0.1749f,  
	-0.8092f,0.5609f,-0.1751f,  -0.7655f,0.5703f,-0.2980f,  -0.7625f,0.5740f,-0.2984f,  -0.7058f,0.5731f,-0.4163f,  -0.7027f,0.5764f,-0.4171f,  -0.6341f,0.5650f,-0.5279f,  -0.6308f,0.5678f,-0.5288f,  -0.5516f,0.5459f,-0.6306f,  -0.5481f,0.5481f,-0.6318f,  -0.4597f,0.5159f,-0.7228f,  -0.4561f,0.5173f,-0.7241f,  -0.3602f,0.4753f,-0.8027f,  -0.3565f,0.4757f,-0.8041f,  -0.2549f,0.4245f,-0.8688f,  -0.2512f,0.4237f,-0.8703f,  
	-0.1458f,0.3641f,-0.9199f,  -0.1421f,0.3620f,-0.9213f,  -0.0349f,0.2951f,-0.9548f,  -0.0315f,0.2914f,-0.9561f,  0.0757f,0.2183f,-0.9729f,  0.0786f,0.2131f,-0.9739f,  0.1836f,0.1351f,-0.9737f,  0.1861f,0.1283f,-0.9741f,  0.2870f,0.0469f,-0.9568f,  0.2886f,0.0384f,-0.9567f,  0.3836f,-0.0447f,-0.9224f,  0.3842f,-0.0549f,-0.9216f,  0.4715f,-0.1380f,-0.8710f,  0.4710f,-0.1498f,-0.8693f,  0.5490f,-0.2312f,-0.8032f,  
	0.5471f,-0.2444f,-0.8006f,  0.6145f,-0.3223f,-0.7201f,  0.6110f,-0.3367f,-0.7165f,  0.6667f,-0.4092f,-0.6230f,  0.6614f,-0.4245f,-0.6183f,  0.7044f,-0.4899f,-0.5136f,  0.6973f,-0.5058f,-0.5079f,  0.7271f,-0.5623f,-0.3938f,  0.7181f,-0.5784f,-0.3870f,  0.7344f,-0.6245f,-0.2658f,  0.7235f,-0.6403f,-0.2580f,  0.7264f,-0.6746f,-0.1318f,  0.7137f,-0.6895f,-0.1232f,  0.7034f,-0.7108f,0.0054f,  0.6892f,-0.7245f,0.0148f,  
	0.6664f,-0.7317f,0.1434f,  0.6509f,-0.7435f,0.1534f,  0.6166f,-0.7360f,0.2793f,  0.6002f,-0.7456f,0.2895f,  0.5557f,-0.7231f,0.4104f,  0.5388f,-0.7300f,0.4205f,  0.4855f,-0.6923f,0.5339f,  0.4687f,-0.6960f,0.5440f,  -0.1361f,0.4451f,-0.8851f,  -0.0226f,0.3864f,-0.9221f,  -0.0213f,0.3927f,-0.9194f,  0.0926f,0.3273f,-0.9404f,  0.0939f,0.3336f,-0.9380f,  0.2063f,0.2626f,-0.9426f,  0.2075f,0.2689f,-0.9406f,  
	0.3164f,0.1934f,-0.9287f,  0.3175f,0.1995f,-0.9270f,  0.4211f,0.1209f,-0.8989f,  0.4221f,0.1267f,-0.8976f,  0.5186f,0.0464f,-0.8537f,  0.5195f,0.0518f,-0.8529f,  0.6073f,-0.0290f,-0.7940f,  0.6080f,-0.0241f,-0.7935f,  0.6855f,-0.1038f,-0.7206f,  0.6862f,-0.0995f,-0.7206f,  0.7521f,-0.1769f,-0.6349f,  0.7525f,-0.1732f,-0.6354f,  0.8057f,-0.2469f,-0.5384f,  0.8060f,-0.2440f,-0.5392f,  0.8456f,-0.3127f,-0.4326f,  
	0.8457f,-0.3106f,-0.4339f,  0.8710f,-0.3732f,-0.3194f,  0.8710f,-0.3718f,-0.3211f,  0.8815f,-0.4273f,-0.2008f,  0.8813f,-0.4267f,-0.2028f,  0.8770f,-0.4741f,-0.0787f,  0.8766f,-0.4743f,-0.0811f,  0.8574f,-0.5127f,0.0447f,  0.8569f,-0.5138f,0.0420f,  0.8231f,-0.5426f,0.1674f,  0.8225f,-0.5445f,0.1645f,  0.7748f,-0.5632f,0.2872f,  0.7740f,-0.5659f,0.2841f,  0.7132f,-0.5742f,0.4021f,  0.7123f,-0.5776f,0.3988f,  
	0.7132f,-0.5742f,0.4021f,  0.7142f,-0.5710f,0.4049f,  0.6405f,-0.5715f,0.5130f,  0.6416f,-0.5680f,0.5155f,  0.5571f,-0.5582f,0.6149f,  0.5583f,-0.5545f,0.6171f,  0.4644f,-0.5347f,0.7060f,  0.4657f,-0.5309f,0.7079f,  0.3640f,-0.5016f,0.7848f,  0.3655f,-0.4978f,0.7865f,  0.2576f,-0.4596f,0.8499f,  0.2592f,-0.4559f,0.8515f,  0.1471f,-0.4096f,0.9003f,  0.1487f,-0.4059f,0.9017f,  0.0341f,-0.3524f,0.9352f,  
	0.0359f,-0.3489f,0.9365f,  -0.0792f,-0.2892f,0.9540f,  -0.0774f,-0.2857f,0.9552f,  -0.1911f,-0.2211f,0.9564f,  -0.1892f,-0.2177f,0.9575f,  -0.2996f,-0.1491f,0.9423f,  -0.2977f,-0.1458f,0.9435f,  -0.4029f,-0.0746f,0.9122f,  -0.4010f,-0.0713f,0.9133f,  -0.4994f,0.0011f,0.8664f,  -0.4974f,0.0045f,0.8675f,  -0.5872f,0.0769f,0.8058f,  -0.5852f,0.0803f,0.8069f,  -0.6650f,0.1514f,0.7314f,  -0.6630f,0.1550f,0.7324f,  
	-0.7314f,0.2234f,0.6444f,  -0.7293f,0.2271f,0.6454f,  -0.7852f,0.2917f,0.5462f,  -0.7831f,0.2955f,0.5472f,  -0.8256f,0.3550f,0.4386f,  -0.8234f,0.3590f,0.4394f,  -0.8518f,0.4122f,0.3233f,  -0.8495f,0.4164f,0.3239f,  -0.8634f,0.4623f,0.2022f,  -0.8609f,0.4666f,0.2027f,  -0.8600f,0.5043f,0.0774f,  -0.8574f,0.5087f,0.0776f,  -0.8419f,0.5374f,-0.0491f,  -0.8391f,0.5418f,-0.0492f,  -0.8092f,0.5609f,-0.1751f,  
	-0.8062f,0.5651f,-0.1754f,  -0.7625f,0.5740f,-0.2984f,  -0.7593f,0.5780f,-0.2991f,  -0.7027f,0.5764f,-0.4171f,  -0.6992f,0.5800f,-0.4180f,  -0.6308f,0.5678f,-0.5288f,  -0.6270f,0.5709f,-0.5300f,  -0.5481f,0.5481f,-0.6318f,  -0.5441f,0.5504f,-0.6332f,  -0.4561f,0.5173f,-0.7241f,  -0.4520f,0.5187f,-0.7257f,  -0.3565f,0.4757f,-0.8041f,  -0.3523f,0.4760f,-0.8058f,  -0.2512f,0.4237f,-0.8703f,  -0.2470f,0.4227f,-0.8720f,  
	-0.1421f,0.3620f,-0.9213f,  -0.1381f,0.3596f,-0.9228f,  -0.0315f,0.2914f,-0.9561f,  -0.0277f,0.2875f,-0.9574f,  0.0786f,0.2131f,-0.9739f,  0.0819f,0.2075f,-0.9748f,  0.1861f,0.1283f,-0.9741f,  0.1887f,0.1209f,-0.9746f,  0.2886f,0.0384f,-0.9567f,  0.2903f,0.0293f,-0.9565f,  0.3842f,-0.0549f,-0.9216f,  0.3849f,-0.0658f,-0.9206f,  0.4710f,-0.1498f,-0.8693f,  0.4703f,-0.1623f,-0.8674f,  0.5471f,-0.2444f,-0.8006f,  
	0.5449f,-0.2583f,-0.7977f,  0.6110f,-0.3367f,-0.7165f,  0.6071f,-0.3518f,-0.7125f,  0.6614f,-0.4245f,-0.6183f,  0.6556f,-0.4405f,-0.6133f,  0.6973f,-0.5058f,-0.5079f,  0.6896f,-0.5223f,-0.5016f,  0.7181f,-0.5784f,-0.3870f,  0.7084f,-0.5950f,-0.3797f,  0.7235f,-0.6403f,-0.2580f,  0.7118f,-0.6565f,-0.2497f,  0.7137f,-0.6895f,-0.1232f,  0.7001f,-0.7049f,-0.1140f,  0.6892f,-0.7245f,0.0148f,  0.6740f,-0.7383f,0.0248f,  
	0.6509f,-0.7435f,0.1534f,  0.6344f,-0.7555f,0.1637f,  0.6002f,-0.7456f,0.2895f,  0.5829f,-0.7552f,0.3000f,  0.5388f,-0.7300f,0.4205f,  0.5212f,-0.7365f,0.4313f,  0.4687f,-0.6960f,0.5440f,  0.4515f,-0.6993f,0.5542f,  -0.0226f,0.3864f,-0.9221f,  0.0912f,0.3213f,-0.9426f,  0.0926f,0.3273f,-0.9404f,  0.2049f,0.2567f,-0.9445f,  0.2063f,0.2626f,-0.9426f,  0.3151f,0.1877f,-0.9303f,  0.3164f,0.1934f,-0.9287f,  
	0.4199f,0.1155f,-0.9002f,  0.4211f,0.1209f,-0.8989f,  0.5176f,0.0413f,-0.8546f,  0.5186f,0.0464f,-0.8537f,  0.6063f,-0.0335f,-0.7945f,  0.6073f,-0.0290f,-0.7940f,  0.6847f,-0.1079f,-0.7208f,  0.6855f,-0.1038f,-0.7206f,  0.7514f,-0.1803f,-0.6347f,  0.7521f,-0.1769f,-0.6349f,  0.8053f,-0.2497f,-0.5378f,  0.8057f,-0.2469f,-0.5384f,  0.8453f,-0.3148f,-0.4316f,  0.8456f,-0.3127f,-0.4326f,  0.8709f,-0.3745f,-0.3181f,  
	0.8710f,-0.3732f,-0.3194f,  0.8816f,-0.4279f,-0.1991f,  0.8815f,-0.4273f,-0.2008f,  0.8773f,-0.4739f,-0.0767f,  0.8770f,-0.4741f,-0.0787f,  0.8579f,-0.5117f,0.0469f,  0.8574f,-0.5127f,0.0447f,  0.8238f,-0.5409f,0.1698f,  0.8231f,-0.5426f,0.1674f,  0.7756f,-0.5607f,0.2898f,  0.7748f,-0.5632f,0.2872f,  0.7142f,-0.5710f,0.4049f,  0.7132f,-0.5742f,0.4021f,  0.7142f,-0.5710f,0.4049f,  0.7151f,-0.5681f,0.4073f,  
	0.6416f,-0.5680f,0.5155f,  0.6427f,-0.5649f,0.5176f,  0.5583f,-0.5545f,0.6171f,  0.5595f,-0.5512f,0.6190f,  0.4657f,-0.5309f,0.7079f,  0.4671f,-0.5275f,0.7096f,  0.3655f,-0.4978f,0.7865f,  0.3669f,-0.4944f,0.7880f,  0.2592f,-0.4559f,0.8515f,  0.2608f,-0.4524f,0.8528f,  0.1487f,-0.4059f,0.9017f,  0.1504f,-0.4025f,0.9030f,  0.0359f,-0.3489f,0.9365f,  0.0376f,-0.3455f,0.9377f,  -0.0774f,-0.2857f,0.9552f,  
	-0.0756f,-0.2825f,0.9563f,  -0.1892f,-0.2177f,0.9575f,  -0.1874f,-0.2144f,0.9586f,  -0.2977f,-0.1458f,0.9435f,  -0.2958f,-0.1425f,0.9446f,  -0.4010f,-0.0713f,0.9133f,  -0.3991f,-0.0680f,0.9144f,  -0.4974f,0.0045f,0.8675f,  -0.4955f,0.0079f,0.8686f,  -0.5852f,0.0803f,0.8069f,  -0.5833f,0.0839f,0.8079f,  -0.6630f,0.1550f,0.7324f,  -0.6610f,0.1587f,0.7334f,  -0.7293f,0.2271f,0.6454f,  -0.7273f,0.2310f,0.6463f,  
	-0.7831f,0.2955f,0.5472f,  -0.7810f,0.2997f,0.5480f,  -0.8234f,0.3590f,0.4394f,  -0.8211f,0.3633f,0.4401f,  -0.8495f,0.4164f,0.3239f,  -0.8471f,0.4209f,0.3245f,  -0.8609f,0.4666f,0.2027f,  -0.8583f,0.4713f,0.2030f,  -0.8574f,0.5087f,0.0776f,  -0.8546f,0.5135f,0.0776f,  -0.8391f,0.5418f,-0.0492f,  -0.8360f,0.5465f,-0.0494f,  -0.8062f,0.5651f,-0.1754f,  -0.8028f,0.5697f,-0.1760f,  -0.7593f,0.5780f,-0.2991f,  
	-0.7556f,0.5823f,-0.3000f,  -0.6992f,0.5800f,-0.4180f,  -0.6953f,0.5839f,-0.4192f,  -0.6270f,0.5709f,-0.5300f,  -0.6228f,0.5741f,-0.5315f,  -0.5441f,0.5504f,-0.6332f,  -0.5397f,0.5528f,-0.6349f,  -0.4520f,0.5187f,-0.7257f,  -0.4474f,0.5201f,-0.7276f,  -0.3523f,0.4760f,-0.8058f,  -0.3476f,0.4762f,-0.8077f,  -0.2470f,0.4227f,-0.8720f,  -0.2423f,0.4215f,-0.8738f,  -0.1381f,0.3596f,-0.9228f,  -0.1336f,0.3569f,-0.9245f,  
	-0.0277f,0.2875f,-0.9574f,  -0.0236f,0.2831f,-0.9588f,  0.0819f,0.2075f,-0.9748f,  0.0855f,0.2013f,-0.9758f,  0.1887f,0.1209f,-0.9746f,  0.1915f,0.1129f,-0.9750f,  0.2903f,0.0293f,-0.9565f,  0.2922f,0.0194f,-0.9562f,  0.3849f,-0.0658f,-0.9206f,  0.3855f,-0.0774f,-0.9195f,  0.4703f,-0.1623f,-0.8674f,  0.4695f,-0.1756f,-0.8653f,  0.5449f,-0.2583f,-0.7977f,  0.5424f,-0.2731f,-0.7945f,  0.6071f,-0.3518f,-0.7125f,  
	0.6027f,-0.3678f,-0.7081f,  0.6556f,-0.4405f,-0.6133f,  0.6492f,-0.4574f,-0.6077f,  0.6896f,-0.5223f,-0.5016f,  0.6811f,-0.5396f,-0.4949f,  0.7084f,-0.5950f,-0.3797f,  0.6977f,-0.6123f,-0.3719f,  0.7118f,-0.6565f,-0.2497f,  0.6991f,-0.6732f,-0.2408f,  0.7001f,-0.7049f,-0.1140f,  0.6855f,-0.7205f,-0.1042f,  0.6740f,-0.7383f,0.0248f,  0.6578f,-0.7524f,0.0351f,  0.6344f,-0.7555f,0.1637f,  0.6170f,-0.7674f,0.1744f,  
	0.5829f,-0.7552f,0.3000f,  0.5647f,-0.7644f,0.3112f,  0.5212f,-0.7365f,0.4313f,  0.5029f,-0.7427f,0.4421f,  0.4515f,-0.6993f,0.5542f,  0.4338f,-0.7020f,0.5648f,  0.0912f,0.3213f,-0.9426f,  0.2035f,0.2512f,-0.9463f,  0.2049f,0.2567f,-0.9445f,  0.3137f,0.1824f,-0.9318f,  0.3151f,0.1877f,-0.9303f,  0.4186f,0.1104f,-0.9014f,  0.4199f,0.1155f,-0.9002f,  0.5164f,0.0366f,-0.8556f,  0.5176f,0.0413f,-0.8546f,  
	0.6053f,-0.0379f,-0.7951f,  0.6063f,-0.0335f,-0.7945f,  0.6838f,-0.1117f,-0.7210f,  0.6847f,-0.1079f,-0.7208f,  0.7507f,-0.1836f,-0.6346f,  0.7514f,-0.1803f,-0.6347f,  0.8047f,-0.2524f,-0.5373f,  0.8053f,-0.2497f,-0.5378f,  0.8450f,-0.3168f,-0.4309f,  0.8453f,-0.3148f,-0.4316f,  0.8708f,-0.3759f,-0.3170f,  0.8709f,-0.3745f,-0.3181f,  0.8817f,-0.4285f,-0.1978f,  0.8816f,-0.4279f,-0.1991f,  0.8775f,-0.4737f,-0.0751f,  
	0.8773f,-0.4739f,-0.0767f,  0.8583f,-0.5109f,0.0488f,  0.8579f,-0.5117f,0.0469f,  0.8244f,-0.5393f,0.1719f,  0.8238f,-0.5409f,0.1698f,  0.7764f,-0.5585f,0.2921f,  0.7756f,-0.5607f,0.2898f,  0.7151f,-0.5681f,0.4073f,  0.7142f,-0.5710f,0.4049f,  0.7151f,-0.5681f,0.4073f,  0.7160f,-0.5655f,0.4093f,  0.6427f,-0.5649f,0.5176f,  0.6437f,-0.5620f,0.5195f,  0.5595f,-0.5512f,0.6190f,  0.5607f,-0.5482f,0.6206f,  
	0.4671f,-0.5275f,0.7096f,  0.4684f,-0.5244f,0.7111f,  0.3669f,-0.4944f,0.7880f,  0.3684f,-0.4912f,0.7893f,  0.2608f,-0.4524f,0.8528f,  0.2623f,-0.4493f,0.8540f,  0.1504f,-0.4025f,0.9030f,  0.1520f,-0.3994f,0.9041f,  0.0376f,-0.3455f,0.9377f,  0.0393f,-0.3424f,0.9387f,  -0.0756f,-0.2825f,0.9563f,  -0.0738f,-0.2794f,0.9573f,  -0.1874f,-0.2144f,0.9586f,  -0.1856f,-0.2113f,0.9596f,  -0.2958f,-0.1425f,0.9446f,  
	-0.2940f,-0.1393f,0.9456f,  -0.3991f,-0.0680f,0.9144f,  -0.3972f,-0.0647f,0.9154f,  -0.4955f,0.0079f,0.8686f,  -0.4936f,0.0114f,0.8696f,  -0.5833f,0.0839f,0.8079f,  -0.5813f,0.0876f,0.8090f,  -0.6610f,0.1587f,0.7334f,  -0.6589f,0.1626f,0.7344f,  -0.7273f,0.2310f,0.6463f,  -0.7251f,0.2352f,0.6472f,  -0.7810f,0.2997f,0.5480f,  -0.7787f,0.3041f,0.5488f,  -0.8211f,0.3633f,0.4401f,  -0.8188f,0.3680f,0.4407f,  
	-0.8471f,0.4209f,0.3245f,  -0.8445f,0.4258f,0.3248f,  -0.8583f,0.4713f,0.2030f,  -0.8555f,0.4763f,0.2031f,  -0.8546f,0.5135f,0.0776f,  -0.8515f,0.5186f,0.0774f,  -0.8360f,0.5465f,-0.0494f,  -0.8326f,0.5516f,-0.0499f,  -0.8028f,0.5697f,-0.1760f,  -0.7991f,0.5746f,-0.1768f,  -0.7556f,0.5823f,-0.3000f,  -0.7516f,0.5869f,-0.3011f,  -0.6953f,0.5839f,-0.4192f,  -0.6909f,0.5880f,-0.4206f,  -0.6228f,0.5741f,-0.5315f,  
	-0.6182f,0.5775f,-0.5332f,  -0.5397f,0.5528f,-0.6349f,  -0.5348f,0.5554f,-0.6369f,  -0.4474f,0.5201f,-0.7276f,  -0.4423f,0.5215f,-0.7297f,  -0.3476f,0.4762f,-0.8077f,  -0.3424f,0.4763f,-0.8099f,  -0.2423f,0.4215f,-0.8738f,  -0.2372f,0.4202f,-0.8759f,  -0.1336f,0.3569f,-0.9245f,  -0.1287f,0.3538f,-0.9264f,  -0.0236f,0.2831f,-0.9588f,  -0.0191f,0.2782f,-0.9603f,  0.0855f,0.2013f,-0.9758f,  0.0894f,0.1946f,-0.9768f,  
	0.1915f,0.1129f,-0.9750f,  0.1945f,0.1042f,-0.9754f,  0.2922f,0.0194f,-0.9562f,  0.2941f,0.0088f,-0.9557f,  0.3855f,-0.0774f,-0.9195f,  0.3860f,-0.0899f,-0.9181f,  0.4695f,-0.1756f,-0.8653f,  0.4684f,-0.1898f,-0.8629f,  0.5424f,-0.2731f,-0.7945f,  0.5395f,-0.2887f,-0.7909f,  0.6027f,-0.3678f,-0.7081f,  0.5978f,-0.3846f,-0.7034f,  0.6492f,-0.4574f,-0.6077f,  0.6421f,-0.4750f,-0.6017f,  0.6811f,-0.5396f,-0.4949f,  
	0.6717f,-0.5576f,-0.4878f,  0.6977f,-0.6123f,-0.3719f,  0.6861f,-0.6301f,-0.3636f,  0.6991f,-0.6732f,-0.2408f,  0.6854f,-0.6904f,-0.2315f,  0.6855f,-0.7205f,-0.1042f,  0.6699f,-0.7365f,-0.0941f,  0.6578f,-0.7524f,0.0351f,  0.6405f,-0.7666f,0.0458f,  0.6170f,-0.7674f,0.1744f,  0.5986f,-0.7792f,0.1858f,  0.5647f,-0.7644f,0.3112f,  0.5457f,-0.7734f,0.3225f,  0.5029f,-0.7427f,0.4421f,  0.4840f,-0.7485f,0.4533f,  
	0.4338f,-0.7020f,0.5648f,  0.4158f,-0.7043f,0.5754f,  0.2035f,0.2512f,-0.9463f,  0.3122f,0.1774f,-0.9333f,  0.3137f,0.1824f,-0.9318f,  0.4172f,0.1056f,-0.9027f,  0.4186f,0.1104f,-0.9014f,  0.5151f,0.0321f,-0.8566f,  0.5164f,0.0366f,-0.8556f,  0.6041f,-0.0420f,-0.7958f,  0.6053f,-0.0379f,-0.7951f,  0.6828f,-0.1154f,-0.7214f,  0.6838f,-0.1117f,-0.7210f,  0.7498f,-0.1868f,-0.6347f,  0.7507f,-0.1836f,-0.6346f,  
	0.8040f,-0.2550f,-0.5372f,  0.8047f,-0.2524f,-0.5373f,  0.8445f,-0.3188f,-0.4304f,  0.8450f,-0.3168f,-0.4309f,  0.8705f,-0.3772f,-0.3163f,  0.8708f,-0.3759f,-0.3170f,  0.8816f,-0.4291f,-0.1967f,  0.8817f,-0.4285f,-0.1978f,  0.8776f,-0.4737f,-0.0738f,  0.8775f,-0.4737f,-0.0751f,  0.8586f,-0.5102f,0.0503f,  0.8583f,-0.5109f,0.0488f,  0.8249f,-0.5379f,0.1736f,  0.8244f,-0.5393f,0.1719f,  0.7771f,-0.5565f,0.2940f,  
	0.7764f,-0.5585f,0.2921f,  0.7160f,-0.5655f,0.4093f,  0.7151f,-0.5681f,0.4073f,  0.7160f,-0.5655f,0.4093f,  0.7169f,-0.5632f,0.4110f,  0.6437f,-0.5620f,0.5195f,  0.6447f,-0.5594f,0.5210f,  0.5607f,-0.5482f,0.6206f,  0.5618f,-0.5454f,0.6220f,  0.4684f,-0.5244f,0.7111f,  0.4697f,-0.5216f,0.7123f,  0.3684f,-0.4912f,0.7893f,  0.3698f,-0.4884f,0.7904f,  0.2623f,-0.4493f,0.8540f,  0.2638f,-0.4464f,0.8550f,  
	0.1520f,-0.3994f,0.9041f,  0.1536f,-0.3965f,0.9051f,  0.0393f,-0.3424f,0.9387f,  0.0410f,-0.3395f,0.9397f,  -0.0738f,-0.2794f,0.9573f,  -0.0721f,-0.2764f,0.9583f,  -0.1856f,-0.2113f,0.9596f,  -0.1838f,-0.2083f,0.9606f,  -0.2940f,-0.1393f,0.9456f,  -0.2922f,-0.1362f,0.9466f,  -0.3972f,-0.0647f,0.9154f,  -0.3954f,-0.0614f,0.9165f,  -0.4936f,0.0114f,0.8696f,  -0.4917f,0.0149f,0.8707f,  -0.5813f,0.0876f,0.8090f,  
	-0.5793f,0.0914f,0.8099f,  -0.6589f,0.1626f,0.7344f,  -0.6569f,0.1667f,0.7353f,  -0.7251f,0.2352f,0.6472f,  -0.7230f,0.2396f,0.6480f,  -0.7787f,0.3041f,0.5488f,  -0.7764f,0.3088f,0.5494f,  -0.8188f,0.3680f,0.4407f,  -0.8162f,0.3730f,0.4411f,  -0.8445f,0.4258f,0.3248f,  -0.8418f,0.4311f,0.3250f,  -0.8555f,0.4763f,0.2031f,  -0.8525f,0.4818f,0.2030f,  -0.8515f,0.5186f,0.0774f,  -0.8482f,0.5241f,0.0770f,  
	-0.8326f,0.5516f,-0.0499f,  -0.8289f,0.5571f,-0.0507f,  -0.7991f,0.5746f,-0.1768f,  -0.7950f,0.5799f,-0.1779f,  -0.7516f,0.5869f,-0.3011f,  -0.7471f,0.5918f,-0.3026f,  -0.6909f,0.5880f,-0.4206f,  -0.6861f,0.5923f,-0.4224f,  -0.6182f,0.5775f,-0.5332f,  -0.6131f,0.5811f,-0.5352f,  -0.5348f,0.5554f,-0.6369f,  -0.5294f,0.5580f,-0.6391f,  -0.4423f,0.5215f,-0.7297f,  -0.4367f,0.5229f,-0.7320f,  -0.3424f,0.4763f,-0.8099f,  
	-0.3368f,0.4763f,-0.8122f,  -0.2372f,0.4202f,-0.8759f,  -0.2317f,0.4186f,-0.8781f,  -0.1287f,0.3538f,-0.9264f,  -0.1235f,0.3504f,-0.9284f,  -0.0191f,0.2782f,-0.9603f,  -0.0143f,0.2729f,-0.9619f,  0.0894f,0.1946f,-0.9768f,  0.0935f,0.1872f,-0.9779f,  0.1945f,0.1042f,-0.9754f,  0.1976f,0.0948f,-0.9757f,  0.2941f,0.0088f,-0.9557f,  0.2960f,-0.0026f,-0.9552f,  0.3860f,-0.0899f,-0.9181f,  0.3864f,-0.1031f,-0.9165f,  
	0.4684f,-0.1898f,-0.8629f,  0.4670f,-0.2048f,-0.8602f,  0.5395f,-0.2887f,-0.7909f,  0.5361f,-0.3052f,-0.7870f,  0.5978f,-0.3846f,-0.7034f,  0.5922f,-0.4022f,-0.6982f,  0.6421f,-0.4750f,-0.6017f,  0.6342f,-0.4933f,-0.5953f,  0.6717f,-0.5576f,-0.4878f,  0.6614f,-0.5762f,-0.4801f,  0.6861f,-0.6301f,-0.3636f,  0.6735f,-0.6485f,-0.3547f,  0.6854f,-0.6904f,-0.2315f,  0.6706f,-0.7079f,-0.2217f,  0.6699f,-0.7365f,-0.0941f,  
	0.6531f,-0.7526f,-0.0836f,  0.6405f,-0.7666f,0.0458f,  0.6222f,-0.7807f,0.0572f,  0.5986f,-0.7792f,0.1858f,  0.5792f,-0.7909f,0.1973f,  0.5457f,-0.7734f,0.3225f,  0.5259f,-0.7821f,0.3341f,  0.4840f,-0.7485f,0.4533f,  0.4645f,-0.7538f,0.4647f,  0.4158f,-0.7043f,0.5754f,  0.3974f,-0.7060f,0.5862f,  0.3122f,0.1774f,-0.9333f,  0.4157f,0.1011f,-0.9039f,  0.4172f,0.1056f,-0.9027f,  0.5136f,0.0278f,-0.8576f,  
	0.5151f,0.0321f,-0.8566f,  0.6028f,-0.0459f,-0.7966f,  0.6041f,-0.0420f,-0.7958f,  0.6816f,-0.1189f,-0.7220f,  0.6828f,-0.1154f,-0.7214f,  0.7488f,-0.1898f,-0.6350f,  0.7498f,-0.1868f,-0.6347f,  0.8032f,-0.2575f,-0.5372f,  0.8040f,-0.2550f,-0.5372f,  0.8438f,-0.3207f,-0.4302f,  0.8445f,-0.3188f,-0.4304f,  0.8701f,-0.3785f,-0.3158f,  0.8705f,-0.3772f,-0.3163f,  0.8814f,-0.4299f,-0.1960f,  0.8816f,-0.4291f,-0.1967f,  
	0.8776f,-0.4738f,-0.0729f,  0.8776f,-0.4737f,-0.0738f,  0.8588f,-0.5097f,0.0515f,  0.8586f,-0.5102f,0.0503f,  0.8254f,-0.5368f,0.1750f,  0.8249f,-0.5379f,0.1736f,  0.7778f,-0.5547f,0.2956f,  0.7771f,-0.5565f,0.2940f,  0.7169f,-0.5632f,0.4110f,  0.7160f,-0.5655f,0.4093f,  0.7169f,-0.5632f,0.4110f,  0.7177f,-0.5611f,0.4124f,  0.6447f,-0.5594f,0.5210f,  0.6456f,-0.5572f,0.5222f,  0.5618f,-0.5454f,0.6220f,  
	0.5629f,-0.5431f,0.6231f,  0.4697f,-0.5216f,0.7123f,  0.4709f,-0.5191f,0.7133f,  0.3698f,-0.4884f,0.7904f,  0.3711f,-0.4859f,0.7914f,  0.2638f,-0.4464f,0.8550f,  0.2652f,-0.4439f,0.8559f,  0.1536f,-0.3965f,0.9051f,  0.1551f,-0.3939f,0.9060f,  0.0410f,-0.3395f,0.9397f,  0.0425f,-0.3368f,0.9406f,  -0.0721f,-0.2764f,0.9583f,  -0.0705f,-0.2736f,0.9592f,  -0.1838f,-0.2083f,0.9606f,  -0.1822f,-0.2053f,0.9616f,  
	-0.2922f,-0.1362f,0.9466f,  -0.2905f,-0.1330f,0.9476f,  -0.3954f,-0.0614f,0.9165f,  -0.3936f,-0.0580f,0.9174f,  -0.4917f,0.0149f,0.8707f,  -0.4898f,0.0186f,0.8716f,  -0.5793f,0.0914f,0.8099f,  -0.5774f,0.0953f,0.8109f,  -0.6569f,0.1667f,0.7353f,  -0.6549f,0.1710f,0.7362f,  -0.7230f,0.2396f,0.6480f,  -0.7208f,0.2443f,0.6487f,  -0.7764f,0.3088f,0.5494f,  -0.7740f,0.3138f,0.5499f,  -0.8162f,0.3730f,0.4411f,  
	-0.8136f,0.3784f,0.4414f,  -0.8418f,0.4311f,0.3250f,  -0.8388f,0.4367f,0.3250f,  -0.8525f,0.4818f,0.2030f,  -0.8492f,0.4876f,0.2027f,  -0.8482f,0.5241f,0.0770f,  -0.8446f,0.5300f,0.0764f,  -0.8289f,0.5571f,-0.0507f,  -0.8249f,0.5629f,-0.0517f,  -0.7950f,0.5799f,-0.1779f,  -0.7906f,0.5855f,-0.1793f,  -0.7471f,0.5918f,-0.3026f,  -0.7423f,0.5970f,-0.3043f,  -0.6861f,0.5923f,-0.4224f,  -0.6808f,0.5969f,-0.4245f,  
	-0.6131f,0.5811f,-0.5352f,  -0.6074f,0.5848f,-0.5376f,  -0.5294f,0.5580f,-0.6391f,  -0.5235f,0.5606f,-0.6417f,  -0.4367f,0.5229f,-0.7320f,  -0.4306f,0.5243f,-0.7347f,  -0.3368f,0.4763f,-0.8122f,  -0.3306f,0.4761f,-0.8149f,  -0.2317f,0.4186f,-0.8781f,  -0.2257f,0.4166f,-0.8806f,  -0.1235f,0.3504f,-0.9284f,  -0.1177f,0.3466f,-0.9306f,  -0.0143f,0.2729f,-0.9619f,  -0.0092f,0.2671f,-0.9636f,  0.0935f,0.1872f,-0.9779f,  
	0.0978f,0.1793f,-0.9789f,  0.1976f,0.0948f,-0.9757f,  0.2009f,0.0847f,-0.9759f,  0.2960f,-0.0026f,-0.9552f,  0.2978f,-0.0148f,-0.9545f,  0.3864f,-0.1031f,-0.9165f,  0.3866f,-0.1173f,-0.9148f,  0.4670f,-0.2048f,-0.8602f,  0.4652f,-0.2207f,-0.8572f,  0.5361f,-0.3052f,-0.7870f,  0.5321f,-0.3226f,-0.7828f,  0.5922f,-0.4022f,-0.6982f,  0.5858f,-0.4207f,-0.6927f,  0.6342f,-0.4933f,-0.5953f,  0.6254f,-0.5124f,-0.5884f,  
	0.6614f,-0.5762f,-0.4801f,  0.6502f,-0.5955f,-0.4719f,  0.6735f,-0.6485f,-0.3547f,  0.6598f,-0.6673f,-0.3455f,  0.6706f,-0.7079f,-0.2217f,  0.6546f,-0.7258f,-0.2115f,  0.6531f,-0.7526f,-0.0836f,  0.6352f,-0.7689f,-0.0724f,  0.6222f,-0.7807f,0.0572f,  0.6028f,-0.7949f,0.0688f,  0.5792f,-0.7909f,0.1973f,  0.5589f,-0.8024f,0.2093f,  0.5259f,-0.7821f,0.3341f,  0.5054f,-0.7905f,0.3460f,  0.4645f,-0.7538f,0.4647f,  
	0.4445f,-0.7587f,0.4762f,  0.3974f,-0.7060f,0.5862f,  0.3788f,-0.7071f,0.5971f,  0.4157f,0.1011f,-0.9039f,  0.5121f,0.0238f,-0.8586f,  0.5136f,0.0278f,-0.8576f,  0.6014f,-0.0496f,-0.7974f,  0.6028f,-0.0459f,-0.7966f,  0.6803f,-0.1222f,-0.7226f,  0.6816f,-0.1189f,-0.7220f,  0.7477f,-0.1927f,-0.6354f,  0.7488f,-0.1898f,-0.6350f,  0.8023f,-0.2600f,-0.5374f,  0.8032f,-0.2575f,-0.5372f,  0.8431f,-0.3227f,-0.4301f,  
	0.8438f,-0.3207f,-0.4302f,  0.8695f,-0.3799f,-0.3155f,  0.8701f,-0.3785f,-0.3158f,  0.8811f,-0.4307f,-0.1955f,  0.8814f,-0.4299f,-0.1960f,  0.8775f,-0.4741f,-0.0722f,  0.8776f,-0.4738f,-0.0729f,  0.8590f,-0.5093f,0.0524f,  0.8588f,-0.5097f,0.0515f,  0.8257f,-0.5358f,0.1761f,  0.8254f,-0.5368f,0.1750f,  0.7784f,-0.5532f,0.2968f,  0.7778f,-0.5547f,0.2956f,  0.7177f,-0.5611f,0.4124f,  0.7169f,-0.5632f,0.4110f,  
	0.7177f,-0.5611f,0.4124f,  0.7184f,-0.5594f,0.4135f,  0.6456f,-0.5572f,0.5222f,  0.6465f,-0.5552f,0.5232f,  0.5629f,-0.5431f,0.6231f,  0.5639f,-0.5410f,0.6240f,  0.4709f,-0.5191f,0.7133f,  0.4720f,-0.5169f,0.7142f,  0.3711f,-0.4859f,0.7914f,  0.3723f,-0.4836f,0.7922f,  0.2652f,-0.4439f,0.8559f,  0.2665f,-0.4415f,0.8567f,  0.1551f,-0.3939f,0.9060f,  0.1565f,-0.3915f,0.9068f,  0.0425f,-0.3368f,0.9406f,  
	0.0440f,-0.3343f,0.9414f,  -0.0705f,-0.2736f,0.9592f,  -0.0690f,-0.2709f,0.9601f,  -0.1822f,-0.2053f,0.9616f,  -0.1806f,-0.2024f,0.9625f,  -0.2905f,-0.1330f,0.9476f,  -0.2889f,-0.1299f,0.9485f,  -0.3936f,-0.0580f,0.9174f,  -0.3920f,-0.0545f,0.9184f,  -0.4898f,0.0186f,0.8716f,  -0.4881f,0.0224f,0.8725f,  -0.5774f,0.0953f,0.8109f,  -0.5755f,0.0995f,0.8117f,  -0.6549f,0.1710f,0.7362f,  -0.6528f,0.1756f,0.7369f,  
	-0.7208f,0.2443f,0.6487f,  -0.7185f,0.2493f,0.6493f,  -0.7740f,0.3138f,0.5499f,  -0.7715f,0.3193f,0.5503f,  -0.8136f,0.3784f,0.4414f,  -0.8108f,0.3842f,0.4415f,  -0.8388f,0.4367f,0.3250f,  -0.8357f,0.4428f,0.3248f,  -0.8492f,0.4876f,0.2027f,  -0.8457f,0.4939f,0.2021f,  -0.8446f,0.5300f,0.0764f,  -0.8406f,0.5363f,0.0754f,  -0.8249f,0.5629f,-0.0517f,  -0.8205f,0.5691f,-0.0530f,  -0.7906f,0.5855f,-0.1793f,  
	-0.7858f,0.5914f,-0.1810f,  -0.7423f,0.5970f,-0.3043f,  -0.7370f,0.6025f,-0.3064f,  -0.6808f,0.5969f,-0.4245f,  -0.6751f,0.6017f,-0.4269f,  -0.6074f,0.5848f,-0.5376f,  -0.6013f,0.5887f,-0.5403f,  -0.5235f,0.5606f,-0.6417f,  -0.5170f,0.5633f,-0.6445f,  -0.4306f,0.5243f,-0.7347f,  -0.4239f,0.5255f,-0.7376f,  -0.3306f,0.4761f,-0.8149f,  -0.3240f,0.4757f,-0.8178f,  -0.2257f,0.4166f,-0.8806f,  -0.2192f,0.4144f,-0.8833f,  
	-0.1177f,0.3466f,-0.9306f,  -0.1117f,0.3423f,-0.9329f,  -0.0092f,0.2671f,-0.9636f,  -0.0037f,0.2606f,-0.9654f,  0.0978f,0.1793f,-0.9789f,  0.1023f,0.1706f,-0.9800f,  0.2009f,0.0847f,-0.9759f,  0.2041f,0.0739f,-0.9762f,  0.2978f,-0.0148f,-0.9545f,  0.2995f,-0.0278f,-0.9537f,  0.3866f,-0.1173f,-0.9148f,  0.3864f,-0.1323f,-0.9128f,  0.4652f,-0.2207f,-0.8572f,  0.4630f,-0.2374f,-0.8540f,  0.5321f,-0.3226f,-0.7828f,  
	0.5275f,-0.3408f,-0.7782f,  0.5858f,-0.4207f,-0.6927f,  0.5787f,-0.4399f,-0.6867f,  0.6254f,-0.5124f,-0.5884f,  0.6157f,-0.5323f,-0.5811f,  0.6502f,-0.5955f,-0.4719f,  0.6377f,-0.6153f,-0.4634f,  0.6598f,-0.6673f,-0.3455f,  0.6448f,-0.6866f,-0.3359f,  0.6546f,-0.7258f,-0.2115f,  0.6374f,-0.7439f,-0.2007f,  0.6352f,-0.7689f,-0.0724f,  0.6161f,-0.7853f,-0.0609f,  0.6028f,-0.7949f,0.0688f,  0.5822f,-0.8090f,0.0808f,  
	0.5589f,-0.8024f,0.2093f,  0.5375f,-0.8136f,0.2214f,  0.5054f,-0.7905f,0.3460f,  0.4840f,-0.7984f,0.3581f,  0.4445f,-0.7587f,0.4762f,  0.4240f,-0.7630f,0.4879f,  0.3788f,-0.7071f,0.5971f,  0.3601f,-0.7076f,0.6079f,  0.5121f,0.0238f,-0.8586f,  0.5998f,-0.0533f,-0.7984f,  0.6014f,-0.0496f,-0.7974f,  0.6790f,-0.1255f,-0.7234f,  0.6803f,-0.1222f,-0.7226f,  0.7465f,-0.1956f,-0.6360f,  0.7477f,-0.1927f,-0.6354f,  
	0.8012f,-0.2624f,-0.5378f,  0.8023f,-0.2600f,-0.5374f,  0.8423f,-0.3247f,-0.4303f,  0.8431f,-0.3227f,-0.4301f,  0.8689f,-0.3814f,-0.3155f,  0.8695f,-0.3799f,-0.3155f,  0.8807f,-0.4316f,-0.1953f,  0.8811f,-0.4307f,-0.1955f,  0.8774f,-0.4745f,-0.0717f,  0.8775f,-0.4741f,-0.0722f,  0.8590f,-0.5092f,0.0530f,  0.8590f,-0.5093f,0.0524f,  0.8260f,-0.5351f,0.1769f,  0.8257f,-0.5358f,0.1761f,  0.7789f,-0.5520f,0.2978f,  
	0.7784f,-0.5532f,0.2968f,  0.7184f,-0.5594f,0.4135f,  0.7177f,-0.5611f,0.4124f,  0.7184f,-0.5594f,0.4135f,  0.7191f,-0.5579f,0.4144f,  0.6465f,-0.5552f,0.5232f,  0.6473f,-0.5536f,0.5240f,  0.5639f,-0.5410f,0.6240f,  0.5648f,-0.5392f,0.6247f,  0.4720f,-0.5169f,0.7142f,  0.4730f,-0.5151f,0.7148f,  0.3723f,-0.4836f,0.7922f,  0.3734f,-0.4816f,0.7929f,  0.2665f,-0.4415f,0.8567f,  0.2677f,-0.4395f,0.8574f,  
	0.1565f,-0.3915f,0.9068f,  0.1577f,-0.3893f,0.9075f,  0.0440f,-0.3343f,0.9414f,  0.0453f,-0.3319f,0.9422f,  -0.0690f,-0.2709f,0.9601f,  -0.0676f,-0.2684f,0.9609f,  -0.1806f,-0.2024f,0.9625f,  -0.1792f,-0.1996f,0.9634f,  -0.2889f,-0.1299f,0.9485f,  -0.2874f,-0.1267f,0.9494f,  -0.3920f,-0.0545f,0.9184f,  -0.3903f,-0.0510f,0.9193f,  -0.4881f,0.0224f,0.8725f,  -0.4863f,0.0263f,0.8734f,  -0.5755f,0.0995f,0.8117f,  
	-0.5737f,0.1040f,0.8125f,  -0.6528f,0.1756f,0.7369f,  -0.6508f,0.1805f,0.7375f,  -0.7185f,0.2493f,0.6493f,  -0.7163f,0.2547f,0.6497f,  -0.7715f,0.3193f,0.5503f,  -0.7690f,0.3251f,0.5505f,  -0.8108f,0.3842f,0.4415f,  -0.8079f,0.3904f,0.4414f,  -0.8357f,0.4428f,0.3248f,  -0.8324f,0.4493f,0.3244f,  -0.8457f,0.4939f,0.2021f,  -0.8420f,0.5005f,0.2013f,  -0.8406f,0.5363f,0.0754f,  -0.8364f,0.5431f,0.0742f,  
	-0.8205f,0.5691f,-0.0530f,  -0.8158f,0.5758f,-0.0547f,  -0.7858f,0.5914f,-0.1810f,  -0.7805f,0.5977f,-0.1831f,  -0.7370f,0.6025f,-0.3064f,  -0.7312f,0.6082f,-0.3089f,  -0.6751f,0.6017f,-0.4269f,  -0.6688f,0.6067f,-0.4297f,  -0.6013f,0.5887f,-0.5403f,  -0.5946f,0.5926f,-0.5434f,  -0.5170f,0.5633f,-0.6445f,  -0.5100f,0.5659f,-0.6478f,  -0.4239f,0.5255f,-0.7376f,  -0.4168f,0.5266f,-0.7409f,  -0.3240f,0.4757f,-0.8178f,  
	-0.3168f,0.4751f,-0.8209f,  -0.2192f,0.4144f,-0.8833f,  -0.2122f,0.4117f,-0.8862f,  -0.1117f,0.3423f,-0.9329f,  -0.1052f,0.3375f,-0.9354f,  -0.0037f,0.2606f,-0.9654f,  0.0020f,0.2536f,-0.9673f,  0.1023f,0.1706f,-0.9800f,  0.1069f,0.1612f,-0.9811f,  0.2041f,0.0739f,-0.9762f,  0.2073f,0.0622f,-0.9763f,  0.2995f,-0.0278f,-0.9537f,  0.3010f,-0.0417f,-0.9527f,  0.3864f,-0.1323f,-0.9128f,  0.3859f,-0.1482f,-0.9106f,  
	0.4630f,-0.2374f,-0.8540f,  0.4601f,-0.2551f,-0.8504f,  0.5275f,-0.3408f,-0.7782f,  0.5221f,-0.3599f,-0.7732f,  0.5787f,-0.4399f,-0.6867f,  0.5706f,-0.4600f,-0.6803f,  0.6157f,-0.5323f,-0.5811f,  0.6047f,-0.5527f,-0.5734f,  0.6377f,-0.6153f,-0.4634f,  0.6240f,-0.6356f,-0.4545f,  0.6448f,-0.6866f,-0.3359f,  0.6286f,-0.7063f,-0.3256f,  0.6374f,-0.7439f,-0.2007f,  0.6188f,-0.7623f,-0.1896f,  0.6161f,-0.7853f,-0.0609f,  
	0.5957f,-0.8017f,-0.0490f,  0.5822f,-0.8090f,0.0808f,  0.5605f,-0.8229f,0.0931f,  0.5375f,-0.8136f,0.2214f,  0.5152f,-0.8245f,0.2338f,  0.4840f,-0.7984f,0.3581f,  0.4619f,-0.8059f,0.3703f,  0.4240f,-0.7630f,0.4879f,  0.4031f,-0.7668f,0.4996f,  0.3601f,-0.7076f,0.6079f,  0.3413f,-0.7075f,0.6188f,  0.5998f,-0.0533f,-0.7984f,  0.6775f,-0.1287f,-0.7242f,  0.6790f,-0.1255f,-0.7234f,  0.7451f,-0.1985f,-0.6367f,  
	0.7465f,-0.1956f,-0.6360f,  0.8001f,-0.2649f,-0.5383f,  0.8012f,-0.2624f,-0.5378f,  0.8413f,-0.3268f,-0.4306f,  0.8423f,-0.3247f,-0.4303f,  0.8681f,-0.3830f,-0.3157f,  0.8689f,-0.3814f,-0.3155f,  0.8801f,-0.4327f,-0.1953f,  0.8807f,-0.4316f,-0.1953f,  0.8771f,-0.4750f,-0.0715f,  0.8774f,-0.4745f,-0.0717f,  0.8590f,-0.5092f,0.0534f,  0.8590f,-0.5092f,0.0530f,  0.8262f,-0.5347f,0.1775f,  0.8260f,-0.5351f,0.1769f,  
	0.7793f,-0.5510f,0.2985f,  0.7789f,-0.5520f,0.2978f,  0.7191f,-0.5579f,0.4144f,  0.7184f,-0.5594f,0.4135f,  0.7191f,-0.5579f,0.4144f,  0.7196f,-0.5567f,0.4150f,  0.6473f,-0.5536f,0.5240f,  0.6480f,-0.5522f,0.5246f,  0.5648f,-0.5392f,0.6247f,  0.5656f,-0.5377f,0.6253f,  0.4730f,-0.5151f,0.7148f,  0.4739f,-0.5134f,0.7154f,  0.3734f,-0.4816f,0.7929f,  0.3744f,-0.4799f,0.7934f,  0.2677f,-0.4395f,0.8574f,  
	0.2688f,-0.4376f,0.8581f,  0.1577f,-0.3893f,0.9075f,  0.1589f,-0.3873f,0.9082f,  0.0453f,-0.3319f,0.9422f,  0.0465f,-0.3297f,0.9429f,  -0.0676f,-0.2684f,0.9609f,  -0.0664f,-0.2658f,0.9617f,  -0.1792f,-0.1996f,0.9634f,  -0.1778f,-0.1967f,0.9642f,  -0.2874f,-0.1267f,0.9494f,  -0.2860f,-0.1235f,0.9503f,  -0.3903f,-0.0510f,0.9193f,  -0.3888f,-0.0473f,0.9201f,  -0.4863f,0.0263f,0.8734f,  -0.4847f,0.0305f,0.8741f,  
	-0.5737f,0.1040f,0.8125f,  -0.5718f,0.1086f,0.8131f,  -0.6508f,0.1805f,0.7375f,  -0.6487f,0.1857f,0.7380f,  -0.7163f,0.2547f,0.6497f,  -0.7139f,0.2604f,0.6500f,  -0.7690f,0.3251f,0.5505f,  -0.7663f,0.3313f,0.5505f,  -0.8079f,0.3904f,0.4414f,  -0.8049f,0.3970f,0.4411f,  -0.8324f,0.4493f,0.3244f,  -0.8289f,0.4562f,0.3237f,  -0.8420f,0.5005f,0.2013f,  -0.8380f,0.5077f,0.2001f,  -0.8364f,0.5431f,0.0742f,  
	-0.8319f,0.5502f,0.0726f,  -0.8158f,0.5758f,-0.0547f,  -0.8106f,0.5828f,-0.0567f,  -0.7805f,0.5977f,-0.1831f,  -0.7748f,0.6044f,-0.1856f,  -0.7312f,0.6082f,-0.3089f,  -0.7249f,0.6143f,-0.3118f,  -0.6688f,0.6067f,-0.4297f,  -0.6620f,0.6118f,-0.4329f,  -0.5946f,0.5926f,-0.5434f,  -0.5873f,0.5967f,-0.5468f,  -0.5100f,0.5659f,-0.6478f,  -0.5025f,0.5686f,-0.6514f,  -0.4168f,0.5266f,-0.7409f,  -0.4091f,0.5276f,-0.7445f,  
	-0.3168f,0.4751f,-0.8209f,  -0.3092f,0.4741f,-0.8244f,  -0.2122f,0.4117f,-0.8862f,  -0.2049f,0.4087f,-0.8894f,  -0.1052f,0.3375f,-0.9354f,  -0.0984f,0.3322f,-0.9381f,  0.0020f,0.2536f,-0.9673f,  0.0078f,0.2459f,-0.9693f,  0.1069f,0.1612f,-0.9811f,  0.1116f,0.1511f,-0.9822f,  0.2073f,0.0622f,-0.9763f,  0.2104f,0.0497f,-0.9763f,  0.3010f,-0.0417f,-0.9527f,  0.3022f,-0.0565f,-0.9516f,  0.3859f,-0.1482f,-0.9106f,  
	0.3848f,-0.1650f,-0.9081f,  0.4601f,-0.2551f,-0.8504f,  0.4566f,-0.2737f,-0.8465f,  0.5221f,-0.3599f,-0.7732f,  0.5159f,-0.3798f,-0.7679f,  0.5706f,-0.4600f,-0.6803f,  0.5614f,-0.4807f,-0.6736f,  0.6047f,-0.5527f,-0.5734f,  0.5926f,-0.5738f,-0.5653f,  0.6240f,-0.6356f,-0.4545f,  0.6091f,-0.6565f,-0.4449f,  0.6286f,-0.7063f,-0.3256f,  0.6110f,-0.7263f,-0.3150f,  0.6188f,-0.7623f,-0.1896f,  0.5989f,-0.7808f,-0.1780f,  
	0.5957f,-0.8017f,-0.0490f,  0.5739f,-0.8181f,-0.0369f,  0.5605f,-0.8229f,0.0931f,  0.5376f,-0.8365f,0.1056f,  0.5152f,-0.8245f,0.2338f,  0.4919f,-0.8350f,0.2464f,  0.4619f,-0.8059f,0.3703f,  0.4391f,-0.8129f,0.3826f,  0.4031f,-0.7668f,0.4996f,  0.3818f,-0.7700f,0.5112f,  0.3413f,-0.7075f,0.6188f,  0.3226f,-0.7069f,0.6295f,  0.6775f,-0.1287f,-0.7242f,  0.7437f,-0.2014f,-0.6375f,  0.7451f,-0.1985f,-0.6367f,  
	0.7988f,-0.2674f,-0.5389f,  0.8001f,-0.2649f,-0.5383f,  0.8402f,-0.3289f,-0.4311f,  0.8413f,-0.3268f,-0.4306f,  0.8673f,-0.3847f,-0.3160f,  0.8681f,-0.3830f,-0.3157f,  0.8795f,-0.4340f,-0.1954f,  0.8801f,-0.4327f,-0.1953f,  0.8766f,-0.4758f,-0.0715f,  0.8771f,-0.4750f,-0.0715f,  0.8588f,-0.5095f,0.0536f,  0.8590f,-0.5092f,0.0534f,  0.8263f,-0.5344f,0.1778f,  0.8262f,-0.5347f,0.1775f,  0.7796f,-0.5503f,0.2990f,  
	0.7793f,-0.5510f,0.2985f,  0.7196f,-0.5567f,0.4150f,  0.7191f,-0.5579f,0.4144f,  0.7196f,-0.5567f,0.4150f,  0.7201f,-0.5558f,0.4155f,  0.6480f,-0.5522f,0.5246f,  0.6485f,-0.5511f,0.5250f,  0.5656f,-0.5377f,0.6253f,  0.5663f,-0.5365f,0.6257f,  0.4739f,-0.5134f,0.7154f,  0.4747f,-0.5121f,0.7158f,  0.3744f,-0.4799f,0.7934f,  0.3752f,-0.4784f,0.7939f,  0.2688f,-0.4376f,0.8581f,  0.2697f,-0.4360f,0.8586f,  
	0.1589f,-0.3873f,0.9082f,  0.1598f,-0.3854f,0.9088f,  0.0465f,-0.3297f,0.9429f,  0.0475f,-0.3276f,0.9436f,  -0.0664f,-0.2658f,0.9617f,  -0.0653f,-0.2634f,0.9625f,  -0.1778f,-0.1967f,0.9642f,  -0.1767f,-0.1939f,0.9650f,  -0.2860f,-0.1235f,0.9503f,  -0.2847f,-0.1202f,0.9511f,  -0.3888f,-0.0473f,0.9201f,  -0.3875f,-0.0435f,0.9209f,  -0.4847f,0.0305f,0.8741f,  -0.4832f,0.0349f,0.8748f,  -0.5718f,0.1086f,0.8131f,  
	-0.5701f,0.1136f,0.8137f,  -0.6487f,0.1857f,0.7380f,  -0.6467f,0.1913f,0.7384f,  -0.7139f,0.2604f,0.6500f,  -0.7116f,0.2665f,0.6501f,  -0.7663f,0.3313f,0.5505f,  -0.7636f,0.3379f,0.5503f,  -0.8049f,0.3970f,0.4411f,  -0.8017f,0.4041f,0.4405f,  -0.8289f,0.4562f,0.3237f,  -0.8252f,0.4636f,0.3227f,  -0.8380f,0.5077f,0.2001f,  -0.8337f,0.5153f,0.1987f,  -0.8319f,0.5502f,0.0726f,  -0.8269f,0.5578f,0.0707f,  
	-0.8106f,0.5828f,-0.0567f,  -0.8051f,0.5902f,-0.0591f,  -0.7748f,0.6044f,-0.1856f,  -0.7686f,0.6114f,-0.1885f,  -0.7249f,0.6143f,-0.3118f,  -0.7181f,0.6206f,-0.3150f,  -0.6620f,0.6118f,-0.4329f,  -0.6546f,0.6172f,-0.4365f,  -0.5873f,0.5967f,-0.5468f,  -0.5796f,0.6007f,-0.5507f,  -0.5025f,0.5686f,-0.6514f,  -0.4943f,0.5711f,-0.6553f,  -0.4091f,0.5276f,-0.7445f,  -0.4008f,0.5284f,-0.7484f,  -0.3092f,0.4741f,-0.8244f,  
	-0.3010f,0.4728f,-0.8281f,  -0.2049f,0.4087f,-0.8894f,  -0.1971f,0.4051f,-0.8928f,  -0.0984f,0.3322f,-0.9381f,  -0.0913f,0.3263f,-0.9409f,  0.0078f,0.2459f,-0.9693f,  0.0139f,0.2374f,-0.9713f,  0.1116f,0.1511f,-0.9822f,  0.1163f,0.1401f,-0.9833f,  0.2104f,0.0497f,-0.9763f,  0.2134f,0.0362f,-0.9763f,  0.3022f,-0.0565f,-0.9516f,  0.3030f,-0.0722f,-0.9503f,  0.3848f,-0.1650f,-0.9081f,  0.3832f,-0.1828f,-0.9054f,  
	0.4566f,-0.2737f,-0.8465f,  0.4523f,-0.2932f,-0.8423f,  0.5159f,-0.3798f,-0.7679f,  0.5085f,-0.4005f,-0.7622f,  0.5614f,-0.4807f,-0.6736f,  0.5510f,-0.5022f,-0.6665f,  0.5926f,-0.5738f,-0.5653f,  0.5792f,-0.5956f,-0.5566f,  0.6091f,-0.6565f,-0.4449f,  0.5927f,-0.6778f,-0.4351f,  0.6110f,-0.7263f,-0.3150f,  0.5919f,-0.7465f,-0.3040f,  0.5989f,-0.7808f,-0.1780f,  0.5775f,-0.7993f,-0.1662f,  0.5739f,-0.8181f,-0.0369f,  
	0.5509f,-0.8342f,-0.0244f,  0.5376f,-0.8365f,0.1056f,  0.5135f,-0.8499f,0.1184f,  0.4919f,-0.8350f,0.2464f,  0.4677f,-0.8450f,0.2592f,  0.4391f,-0.8129f,0.3826f,  0.4157f,-0.8193f,0.3950f,  0.3818f,-0.7700f,0.5112f,  0.3602f,-0.7725f,0.5229f,  0.3226f,-0.7069f,0.6295f,  0.3039f,-0.7056f,0.6401f,  0.7437f,-0.2014f,-0.6375f,  0.7974f,-0.2700f,-0.5397f,  0.7988f,-0.2674f,-0.5389f,  0.8390f,-0.3311f,-0.4318f,  
	0.8402f,-0.3289f,-0.4311f,  0.8663f,-0.3865f,-0.3165f,  0.8673f,-0.3847f,-0.3160f,  0.8787f,-0.4354f,-0.1958f,  0.8795f,-0.4340f,-0.1954f,  0.8761f,-0.4768f,-0.0717f,  0.8766f,-0.4758f,-0.0715f,  0.8585f,-0.5100f,0.0536f,  0.8588f,-0.5095f,0.0536f,  0.8262f,-0.5345f,0.1780f,  0.8263f,-0.5344f,0.1778f,  0.7798f,-0.5498f,0.2993f,  0.7796f,-0.5503f,0.2990f,  0.7201f,-0.5558f,0.4155f,  0.7196f,-0.5567f,0.4150f,  
	0.7201f,-0.5558f,0.4155f,  0.7204f,-0.5551f,0.4158f,  0.6485f,-0.5511f,0.5250f,  0.6490f,-0.5504f,0.5253f,  0.5663f,-0.5365f,0.6257f,  0.5668f,-0.5356f,0.6260f,  0.4747f,-0.5121f,0.7158f,  0.4753f,-0.5111f,0.7162f,  0.3752f,-0.4784f,0.7939f,  0.3759f,-0.4772f,0.7943f,  0.2697f,-0.4360f,0.8586f,  0.2704f,-0.4345f,0.8591f,  0.1598f,-0.3854f,0.9088f,  0.1606f,-0.3837f,0.9094f,  0.0475f,-0.3276f,0.9436f,  
	0.0484f,-0.3255f,0.9443f,  -0.0653f,-0.2634f,0.9625f,  -0.0644f,-0.2609f,0.9632f,  -0.1767f,-0.1939f,0.9650f,  -0.1757f,-0.1909f,0.9658f,  -0.2847f,-0.1202f,0.9511f,  -0.2836f,-0.1167f,0.9518f,  -0.3875f,-0.0435f,0.9209f,  -0.3862f,-0.0395f,0.9216f,  -0.4832f,0.0349f,0.8748f,  -0.4817f,0.0395f,0.8754f,  -0.5701f,0.1136f,0.8137f,  -0.5684f,0.1189f,0.8141f,  -0.6467f,0.1913f,0.7384f,  -0.6447f,0.1972f,0.7386f,  
	-0.7116f,0.2665f,0.6501f,  -0.7092f,0.2730f,0.6500f,  -0.7636f,0.3379f,0.5503f,  -0.7607f,0.3450f,0.5498f,  -0.8017f,0.4041f,0.4405f,  -0.7983f,0.4116f,0.4397f,  -0.8252f,0.4636f,0.3227f,  -0.8212f,0.4715f,0.3213f,  -0.8337f,0.5153f,0.1987f,  -0.8291f,0.5233f,0.1969f,  -0.8269f,0.5578f,0.0707f,  -0.8216f,0.5659f,0.0684f,  -0.8051f,0.5902f,-0.0591f,  -0.7991f,0.5980f,-0.0620f,  -0.7686f,0.6114f,-0.1885f,  
	-0.7619f,0.6187f,-0.1918f,  -0.7181f,0.6206f,-0.3150f,  -0.7107f,0.6271f,-0.3187f,  -0.6546f,0.6172f,-0.4365f,  -0.6467f,0.6227f,-0.4405f,  -0.5796f,0.6007f,-0.5507f,  -0.5711f,0.6049f,-0.5549f,  -0.4943f,0.5711f,-0.6553f,  -0.4856f,0.5736f,-0.6597f,  -0.4008f,0.5284f,-0.7484f,  -0.3920f,0.5289f,-0.7527f,  -0.3010f,0.4728f,-0.8281f,  -0.2924f,0.4712f,-0.8322f,  -0.1971f,0.4051f,-0.8928f,  -0.1890f,0.4011f,-0.8963f,  
	-0.0913f,0.3263f,-0.9409f,  -0.0840f,0.3197f,-0.9438f,  0.0139f,0.2374f,-0.9713f,  0.0201f,0.2282f,-0.9734f,  0.1163f,0.1401f,-0.9833f,  0.1209f,0.1283f,-0.9843f,  0.2134f,0.0362f,-0.9763f,  0.2160f,0.0220f,-0.9762f,  0.3030f,-0.0722f,-0.9503f,  0.3033f,-0.0888f,-0.9487f,  0.3832f,-0.1828f,-0.9054f,  0.3809f,-0.2015f,-0.9024f,  0.4523f,-0.2932f,-0.8423f,  0.4469f,-0.3135f,-0.8379f,  0.5085f,-0.4005f,-0.7622f,  
	0.5000f,-0.4220f,-0.7562f,  0.5510f,-0.5022f,-0.6665f,  0.5394f,-0.5244f,-0.6588f,  0.5792f,-0.5956f,-0.5566f,  0.5643f,-0.6178f,-0.5477f,  0.5927f,-0.6778f,-0.4351f,  0.5748f,-0.6994f,-0.4248f,  0.5919f,-0.7465f,-0.3040f,  0.5713f,-0.7668f,-0.2927f,  0.5775f,-0.7993f,-0.1662f,  0.5547f,-0.8177f,-0.1540f,  0.5509f,-0.8342f,-0.0244f,  0.5264f,-0.8502f,-0.0117f,  0.5135f,-0.8499f,0.1184f,  0.4883f,-0.8628f,0.1313f,  
	0.4677f,-0.8450f,0.2592f,  0.4425f,-0.8545f,0.2720f,  0.4157f,-0.8193f,0.3950f,  0.3916f,-0.8250f,0.4074f,  0.3602f,-0.7725f,0.5229f,  0.3384f,-0.7745f,0.5345f,  0.3039f,-0.7056f,0.6401f,  0.2854f,-0.7038f,0.6505f,  0.7974f,-0.2700f,-0.5397f,  0.8377f,-0.3335f,-0.4325f,  0.8390f,-0.3311f,-0.4318f,  0.8652f,-0.3885f,-0.3171f,  0.8663f,-0.3865f,-0.3165f,  0.8778f,-0.4370f,-0.1962f,  0.8787f,-0.4354f,-0.1958f,  
	0.8754f,-0.4779f,-0.0720f,  0.8761f,-0.4768f,-0.0717f,  0.8581f,-0.5107f,0.0534f,  0.8585f,-0.5100f,0.0536f,  0.8261f,-0.5348f,0.1779f,  0.8262f,-0.5345f,0.1780f,  0.7799f,-0.5497f,0.2994f,  0.7798f,-0.5498f,0.2993f,  0.7204f,-0.5551f,0.4158f,  0.7201f,-0.5558f,0.4155f,  0.7204f,-0.5551f,0.4158f,  0.7206f,-0.5548f,0.4159f,  0.6490f,-0.5504f,0.5253f,  0.6492f,-0.5499f,0.5255f,  0.5668f,-0.5356f,0.6260f,  
	0.5672f,-0.5349f,0.6262f,  0.4753f,-0.5111f,0.7162f,  0.4757f,-0.5102f,0.7165f,  0.3759f,-0.4772f,0.7943f,  0.3764f,-0.4762f,0.7947f,  0.2704f,-0.4345f,0.8591f,  0.2710f,-0.4332f,0.8596f,  0.1606f,-0.3837f,0.9094f,  0.1612f,-0.3821f,0.9100f,  0.0484f,-0.3255f,0.9443f,  0.0490f,-0.3235f,0.9450f,  -0.0644f,-0.2609f,0.9632f,  -0.0636f,-0.2585f,0.9639f,  -0.1757f,-0.1909f,0.9658f,  -0.1748f,-0.1880f,0.9665f,  
	-0.2836f,-0.1167f,0.9518f,  -0.2826f,-0.1132f,0.9525f,  -0.3862f,-0.0395f,0.9216f,  -0.3851f,-0.0352f,0.9222f,  -0.4817f,0.0395f,0.8754f,  -0.4803f,0.0444f,0.8760f,  -0.5684f,0.1189f,0.8141f,  -0.5667f,0.1245f,0.8144f,  -0.6447f,0.1972f,0.7386f,  -0.6427f,0.2035f,0.7386f,  -0.7092f,0.2730f,0.6500f,  -0.7067f,0.2800f,0.6497f,  -0.7607f,0.3450f,0.5498f,  -0.7577f,0.3525f,0.5491f,  -0.7983f,0.4116f,0.4397f,  
	-0.7947f,0.4196f,0.4385f,  -0.8212f,0.4715f,0.3213f,  -0.8170f,0.4799f,0.3197f,  -0.8291f,0.5233f,0.1969f,  -0.8241f,0.5319f,0.1947f,  -0.8216f,0.5659f,0.0684f,  -0.8159f,0.5744f,0.0656f,  -0.7991f,0.5980f,-0.0620f,  -0.7926f,0.6062f,-0.0652f,  -0.7619f,0.6187f,-0.1918f,  -0.7546f,0.6263f,-0.1955f,  -0.7107f,0.6271f,-0.3187f,  -0.7028f,0.6339f,-0.3229f,  -0.6467f,0.6227f,-0.4405f,  -0.6382f,0.6282f,-0.4450f,  
	-0.5711f,0.6049f,-0.5549f,  -0.5621f,0.6090f,-0.5596f,  -0.4856f,0.5736f,-0.6597f,  -0.4764f,0.5759f,-0.6644f,  -0.3920f,0.5289f,-0.7527f,  -0.3827f,0.5291f,-0.7573f,  -0.2924f,0.4712f,-0.8322f,  -0.2833f,0.4691f,-0.8365f,  -0.1890f,0.4011f,-0.8963f,  -0.1805f,0.3965f,-0.9001f,  -0.0840f,0.3197f,-0.9438f,  -0.0764f,0.3124f,-0.9469f,  0.0201f,0.2282f,-0.9734f,  0.0263f,0.2183f,-0.9755f,  0.1209f,0.1283f,-0.9843f,  
	0.1252f,0.1157f,-0.9854f,  0.2160f,0.0220f,-0.9762f,  0.2182f,0.0067f,-0.9759f,  0.3033f,-0.0888f,-0.9487f,  0.3031f,-0.1064f,-0.9470f,  0.3809f,-0.2015f,-0.9024f,  0.3777f,-0.2210f,-0.8992f,  0.4469f,-0.3135f,-0.8379f,  0.4404f,-0.3346f,-0.8331f,  0.5000f,-0.4220f,-0.7562f,  0.4904f,-0.4443f,-0.7498f,  0.5394f,-0.5244f,-0.6588f,  0.5263f,-0.5472f,-0.6509f,  0.5643f,-0.6178f,-0.5477f,  0.5479f,-0.6404f,-0.5383f,  
	0.5748f,-0.6994f,-0.4248f,  0.5552f,-0.7212f,-0.4141f,  0.5713f,-0.7668f,-0.2927f,  0.5490f,-0.7872f,-0.2810f,  0.5547f,-0.8177f,-0.1540f,  0.5302f,-0.8360f,-0.1416f,  0.5264f,-0.8502f,-0.0117f,  0.5005f,-0.8658f,0.0012f,  0.4883f,-0.8628f,0.1313f,  0.4618f,-0.8752f,0.1444f,  0.4425f,-0.8545f,0.2720f,  0.4164f,-0.8634f,0.2849f,  0.3916f,-0.8250f,0.4074f,  0.3670f,-0.8301f,0.4197f,  0.3384f,-0.7745f,0.5345f,  
	0.3164f,-0.7758f,0.5459f,  0.2854f,-0.7038f,0.6505f,  0.2672f,-0.7014f,0.6607f,  0.8377f,-0.3335f,-0.4325f,  0.8639f,-0.3907f,-0.3178f,  0.8652f,-0.3885f,-0.3171f,  0.8768f,-0.4388f,-0.1968f,  0.8778f,-0.4370f,-0.1962f,  0.8746f,-0.4793f,-0.0725f,  0.8754f,-0.4779f,-0.0720f,  0.8575f,-0.5117f,0.0531f,  0.8581f,-0.5107f,0.0534f,  0.8257f,-0.5353f,0.1778f,  0.8261f,-0.5348f,0.1779f,  0.7798f,-0.5498f,0.2994f,  
	0.7799f,-0.5497f,0.2994f,  0.7206f,-0.5548f,0.4159f,  0.7204f,-0.5551f,0.4158f,  0.7206f,-0.5548f,0.4159f,  0.7206f,-0.5548f,0.4159f,  0.6492f,-0.5499f,0.5255f,  0.6494f,-0.5497f,0.5255f,  0.5672f,-0.5349f,0.6262f,  0.5673f,-0.5346f,0.6264f,  0.4757f,-0.5102f,0.7165f,  0.4759f,-0.5097f,0.7167f,  0.3764f,-0.4762f,0.7947f,  0.3767f,-0.4753f,0.7951f,  0.2710f,-0.4332f,0.8596f,  0.2713f,-0.4321f,0.8601f,  
	0.1612f,-0.3821f,0.9100f,  0.1616f,-0.3806f,0.9105f,  0.0490f,-0.3235f,0.9450f,  0.0495f,-0.3216f,0.9456f,  -0.0636f,-0.2585f,0.9639f,  -0.0631f,-0.2560f,0.9646f,  -0.1748f,-0.1880f,0.9665f,  -0.1742f,-0.1849f,0.9672f,  -0.2826f,-0.1132f,0.9525f,  -0.2818f,-0.1094f,0.9532f,  -0.3851f,-0.0352f,0.9222f,  -0.3841f,-0.0308f,0.9228f,  -0.4803f,0.0444f,0.8760f,  -0.4791f,0.0497f,0.8764f,  -0.5667f,0.1245f,0.8144f,  
	-0.5651f,0.1305f,0.8146f,  -0.6427f,0.2035f,0.7386f,  -0.6407f,0.2102f,0.7385f,  -0.7067f,0.2800f,0.6497f,  -0.7042f,0.2875f,0.6492f,  -0.7577f,0.3525f,0.5491f,  -0.7546f,0.3606f,0.5482f,  -0.7947f,0.4196f,0.4385f,  -0.7910f,0.4282f,0.4370f,  -0.8170f,0.4799f,0.3197f,  -0.8125f,0.4888f,0.3177f,  -0.8241f,0.5319f,0.1947f,  -0.8188f,0.5410f,0.1921f,  -0.8159f,0.5744f,0.0656f,  -0.8098f,0.5834f,0.0625f,  
	-0.7926f,0.6062f,-0.0652f,  -0.7856f,0.6148f,-0.0689f,  -0.7546f,0.6263f,-0.1955f,  -0.7469f,0.6343f,-0.1997f,  -0.7028f,0.6339f,-0.3229f,  -0.6943f,0.6409f,-0.3275f,  -0.6382f,0.6282f,-0.4450f,  -0.6290f,0.6339f,-0.4500f,  -0.5621f,0.6090f,-0.5596f,  -0.5525f,0.6130f,-0.5647f,  -0.4764f,0.5759f,-0.6644f,  -0.4665f,0.5780f,-0.6696f,  -0.3827f,0.5291f,-0.7573f,  -0.3729f,0.5290f,-0.7623f,  -0.2833f,0.4691f,-0.8365f,  
	-0.2738f,0.4665f,-0.8411f,  -0.1805f,0.3965f,-0.9001f,  -0.1717f,0.3913f,-0.9041f,  -0.0764f,0.3124f,-0.9469f,  -0.0688f,0.3044f,-0.9500f,  0.0263f,0.2183f,-0.9755f,  0.0323f,0.2075f,-0.9777f,  0.1252f,0.1157f,-0.9854f,  0.1294f,0.1021f,-0.9863f,  0.2182f,0.0067f,-0.9759f,  0.2200f,-0.0095f,-0.9755f,  0.3031f,-0.1064f,-0.9470f,  0.3019f,-0.1248f,-0.9451f,  0.3777f,-0.2210f,-0.8992f,  0.3734f,-0.2414f,-0.8957f,  
	0.4404f,-0.3346f,-0.8331f,  0.4330f,-0.3566f,-0.8279f,  0.4904f,-0.4443f,-0.7498f,  0.4792f,-0.4672f,-0.7430f,  0.5263f,-0.5472f,-0.6509f,  0.5116f,-0.5704f,-0.6426f,  0.5479f,-0.6404f,-0.5383f,  0.5297f,-0.6634f,-0.5285f,  0.5552f,-0.7212f,-0.4141f,  0.5339f,-0.7432f,-0.4032f,  0.5490f,-0.7872f,-0.2810f,  0.5250f,-0.8075f,-0.2691f,  0.5302f,-0.8360f,-0.1416f,  0.5041f,-0.8539f,-0.1290f,  0.5005f,-0.8658f,0.0012f,  
	0.4731f,-0.8809f,0.0143f,  0.4618f,-0.8752f,0.1444f,  0.4341f,-0.8870f,0.1575f,  0.4164f,-0.8634f,0.2849f,  0.3894f,-0.8716f,0.2977f,  0.3670f,-0.8301f,0.4197f,  0.3419f,-0.8346f,0.4319f,  0.3164f,-0.7758f,0.5459f,  0.2943f,-0.7765f,0.5572f,  0.2672f,-0.7014f,0.6607f,  0.2493f,-0.6985f,0.6707f,  0.8639f,-0.3907f,-0.3178f,  0.8756f,-0.4408f,-0.1975f,  0.8768f,-0.4388f,-0.1968f,  0.8737f,-0.4810f,-0.0730f,  
	0.8746f,-0.4793f,-0.0725f,  0.8568f,-0.5130f,0.0527f,  0.8575f,-0.5117f,0.0531f,  0.8253f,-0.5361f,0.1775f,  0.8257f,-0.5353f,0.1778f,  0.7796f,-0.5502f,0.2993f,  0.7798f,-0.5498f,0.2994f,  0.7206f,-0.5548f,0.4159f,  0.7206f,-0.5548f,0.4159f,  0.7206f,-0.5548f,0.4159f,  0.7204f,-0.5550f,0.4159f,  0.6494f,-0.5497f,0.5255f,  0.6493f,-0.5498f,0.5256f,  0.5673f,-0.5346f,0.6264f,  0.5673f,-0.5345f,0.6265f,  
	0.4759f,-0.5097f,0.7167f,  0.4759f,-0.5093f,0.7170f,  0.3767f,-0.4753f,0.7951f,  0.3767f,-0.4747f,0.7954f,  0.2713f,-0.4321f,0.8601f,  0.2714f,-0.4311f,0.8605f,  0.1616f,-0.3806f,0.9105f,  0.1617f,-0.3792f,0.9111f,  0.0495f,-0.3216f,0.9456f,  0.0497f,-0.3196f,0.9462f,  -0.0631f,-0.2560f,0.9646f,  -0.0628f,-0.2535f,0.9653f,  -0.1742f,-0.1849f,0.9672f,  -0.1738f,-0.1817f,0.9679f,  -0.2818f,-0.1094f,0.9532f,  
	-0.2813f,-0.1054f,0.9538f,  -0.3841f,-0.0308f,0.9228f,  -0.3833f,-0.0260f,0.9233f,  -0.4791f,0.0497f,0.8764f,  -0.4779f,0.0552f,0.8767f,  -0.5651f,0.1305f,0.8146f,  -0.5636f,0.1369f,0.8146f,  -0.6407f,0.2102f,0.7385f,  -0.6386f,0.2175f,0.7381f,  -0.7042f,0.2875f,0.6492f,  -0.7016f,0.2954f,0.6484f,  -0.7546f,0.3606f,0.5482f,  -0.7514f,0.3692f,0.5469f,  -0.7910f,0.4282f,0.4370f,  -0.7870f,0.4374f,0.4352f,  
	-0.8125f,0.4888f,0.3177f,  -0.8077f,0.4983f,0.3153f,  -0.8188f,0.5410f,0.1921f,  -0.8131f,0.5505f,0.1891f,  -0.8098f,0.5834f,0.0625f,  -0.8032f,0.5928f,0.0589f,  -0.7856f,0.6148f,-0.0689f,  -0.7781f,0.6238f,-0.0731f,  -0.7469f,0.6343f,-0.1997f,  -0.7385f,0.6425f,-0.2044f,  -0.6943f,0.6409f,-0.3275f,  -0.6851f,0.6480f,-0.3326f,  -0.6290f,0.6339f,-0.4500f,  -0.6192f,0.6397f,-0.4554f,  -0.5525f,0.6130f,-0.5647f,  
	-0.5423f,0.6170f,-0.5703f,  -0.4665f,0.5780f,-0.6696f,  -0.4561f,0.5799f,-0.6751f,  -0.3729f,0.5290f,-0.7623f,  -0.3625f,0.5286f,-0.7676f,  -0.2738f,0.4665f,-0.8411f,  -0.2640f,0.4634f,-0.8459f,  -0.1717f,0.3913f,-0.9041f,  -0.1626f,0.3854f,-0.9083f,  -0.0688f,0.3044f,-0.9500f,  -0.0611f,0.2957f,-0.9533f,  0.0323f,0.2075f,-0.9777f,  0.0383f,0.1958f,-0.9799f,  0.1294f,0.1021f,-0.9863f,  0.1332f,0.0877f,-0.9872f,  
	0.2200f,-0.0095f,-0.9755f,  0.2211f,-0.0265f,-0.9749f,  0.3019f,-0.1248f,-0.9451f,  0.2999f,-0.1442f,-0.9430f,  0.3734f,-0.2414f,-0.8957f,  0.3681f,-0.2628f,-0.8919f,  0.4330f,-0.3566f,-0.8279f,  0.4240f,-0.3793f,-0.8224f,  0.4792f,-0.4672f,-0.7430f,  0.4665f,-0.4907f,-0.7359f,  0.5116f,-0.5704f,-0.6426f,  0.4951f,-0.5941f,-0.6339f,  0.5297f,-0.6634f,-0.5285f,  0.5097f,-0.6865f,-0.5185f,  0.5339f,-0.7432f,-0.4032f,  
	0.5108f,-0.7652f,-0.3920f,  0.5250f,-0.8075f,-0.2691f,  0.4992f,-0.8275f,-0.2569f,  0.5041f,-0.8539f,-0.1290f,  0.4764f,-0.8715f,-0.1161f,  0.4731f,-0.8809f,0.0143f,  0.4443f,-0.8955f,0.0275f,  0.4341f,-0.8870f,0.1575f,  0.4052f,-0.8981f,0.1707f,  0.3894f,-0.8716f,0.2977f,  0.3616f,-0.8791f,0.3106f,  0.3419f,-0.8346f,0.4319f,  0.3164f,-0.8383f,0.4441f,  0.2943f,-0.7765f,0.5572f,  0.2722f,-0.7765f,0.5683f,  
	0.2493f,-0.6985f,0.6707f,  0.2319f,-0.6951f,0.6804f,  0.8756f,-0.4408f,-0.1975f,  0.8726f,-0.4829f,-0.0736f,  0.8737f,-0.4810f,-0.0730f,  0.8559f,-0.5145f,0.0522f,  0.8568f,-0.5130f,0.0527f,  0.8246f,-0.5373f,0.1772f,  0.8253f,-0.5361f,0.1775f,  0.7792f,-0.5508f,0.2991f,  0.7796f,-0.5502f,0.2993f,  0.7204f,-0.5550f,0.4159f,  0.7206f,-0.5548f,0.4159f,  0.7204f,-0.5550f,0.4159f,  0.7201f,-0.5555f,0.4157f,  
	0.6493f,-0.5498f,0.5256f,  0.6490f,-0.5501f,0.5255f,  0.5673f,-0.5345f,0.6265f,  0.5671f,-0.5346f,0.6266f,  0.4759f,-0.5093f,0.7170f,  0.4757f,-0.5092f,0.7172f,  0.3767f,-0.4747f,0.7954f,  0.3765f,-0.4743f,0.7958f,  0.2714f,-0.4311f,0.8605f,  0.2712f,-0.4303f,0.8610f,  0.1617f,-0.3792f,0.9111f,  0.1616f,-0.3778f,0.9117f,  0.0497f,-0.3196f,0.9462f,  0.0496f,-0.3177f,0.9469f,  -0.0628f,-0.2535f,0.9653f,  
	-0.0628f,-0.2508f,0.9660f,  -0.1738f,-0.1817f,0.9679f,  -0.1736f,-0.1783f,0.9685f,  -0.2813f,-0.1054f,0.9538f,  -0.2809f,-0.1013f,0.9544f,  -0.3833f,-0.0260f,0.9233f,  -0.3826f,-0.0210f,0.9237f,  -0.4779f,0.0552f,0.8767f,  -0.4769f,0.0612f,0.8768f,  -0.5636f,0.1369f,0.8146f,  -0.5621f,0.1437f,0.8145f,  -0.6386f,0.2175f,0.7381f,  -0.6366f,0.2252f,0.7376f,  -0.7016f,0.2954f,0.6484f,  -0.6990f,0.3039f,0.6474f,  
	-0.7514f,0.3692f,0.5469f,  -0.7480f,0.3784f,0.5453f,  -0.7870f,0.4374f,0.4352f,  -0.7827f,0.4470f,0.4330f,  -0.8077f,0.4983f,0.3153f,  -0.8025f,0.5083f,0.3125f,  -0.8131f,0.5505f,0.1891f,  -0.8070f,0.5606f,0.1856f,  -0.8032f,0.5928f,0.0589f,  -0.7961f,0.6027f,0.0548f,  -0.7781f,0.6238f,-0.0731f,  -0.7701f,0.6332f,-0.0777f,  -0.7385f,0.6425f,-0.2044f,  -0.7295f,0.6511f,-0.2096f,  -0.6851f,0.6480f,-0.3326f,  
	-0.6753f,0.6554f,-0.3382f,  -0.6192f,0.6397f,-0.4554f,  -0.6088f,0.6455f,-0.4613f,  -0.5423f,0.6170f,-0.5703f,  -0.5314f,0.6209f,-0.5763f,  -0.4561f,0.5799f,-0.6751f,  -0.4451f,0.5815f,-0.6810f,  -0.3625f,0.5286f,-0.7676f,  -0.3517f,0.5277f,-0.7732f,  -0.2640f,0.4634f,-0.8459f,  -0.2537f,0.4598f,-0.8510f,  -0.1626f,0.3854f,-0.9083f,  -0.1535f,0.3789f,-0.9126f,  -0.0611f,0.2957f,-0.9533f,  -0.0534f,0.2861f,-0.9567f,  
	0.0383f,0.1958f,-0.9799f,  0.0441f,0.1833f,-0.9821f,  0.1332f,0.0877f,-0.9872f,  0.1364f,0.0723f,-0.9880f,  0.2211f,-0.0265f,-0.9749f,  0.2213f,-0.0444f,-0.9742f,  0.2999f,-0.1442f,-0.9430f,  0.2970f,-0.1645f,-0.9406f,  0.3681f,-0.2628f,-0.8919f,  0.3615f,-0.2848f,-0.8878f,  0.4240f,-0.3793f,-0.8224f,  0.4135f,-0.4026f,-0.8166f,  0.4665f,-0.4907f,-0.7359f,  0.4521f,-0.5148f,-0.7284f,  0.4951f,-0.5941f,-0.6339f,  
	0.4768f,-0.6181f,-0.6249f,  0.5097f,-0.6865f,-0.5185f,  0.4879f,-0.7098f,-0.5081f,  0.5108f,-0.7652f,-0.3920f,  0.4857f,-0.7870f,-0.3804f,  0.4992f,-0.8275f,-0.2569f,  0.4715f,-0.8473f,-0.2444f,  0.4764f,-0.8715f,-0.1161f,  0.4470f,-0.8886f,-0.1030f,  0.4443f,-0.8955f,0.0275f,  0.4140f,-0.9094f,0.0408f,  0.4052f,-0.8981f,0.1707f,  0.3752f,-0.9085f,0.1839f,  0.3616f,-0.8791f,0.3106f,  0.3330f,-0.8857f,0.3234f,  
	0.3164f,-0.8383f,0.4441f,  0.2904f,-0.8412f,0.4561f,  0.2722f,-0.7765f,0.5683f,  0.2502f,-0.7758f,0.5792f,  0.2319f,-0.6951f,0.6804f,  0.2149f,-0.6914f,0.6898f,  0.8726f,-0.4829f,-0.0736f,  0.8549f,-0.5163f,0.0517f,  0.8559f,-0.5145f,0.0522f,  0.8238f,-0.5387f,0.1768f,  0.8246f,-0.5373f,0.1772f,  0.7786f,-0.5518f,0.2988f,  0.7792f,-0.5508f,0.2991f,  0.7201f,-0.5555f,0.4157f,  0.7204f,-0.5550f,0.4159f,  
	0.7201f,-0.5555f,0.4157f,  0.7196f,-0.5564f,0.4156f,  0.6490f,-0.5501f,0.5255f,  0.6485f,-0.5508f,0.5255f,  0.5671f,-0.5346f,0.6266f,  0.5666f,-0.5350f,0.6267f,  0.4757f,-0.5092f,0.7172f,  0.4753f,-0.5093f,0.7174f,  0.3765f,-0.4743f,0.7958f,  0.3761f,-0.4740f,0.7962f,  0.2712f,-0.4303f,0.8610f,  0.2708f,-0.4295f,0.8615f,  0.1616f,-0.3778f,0.9117f,  0.1612f,-0.3765f,0.9123f,  0.0496f,-0.3177f,0.9469f,  
	0.0493f,-0.3157f,0.9476f,  -0.0628f,-0.2508f,0.9660f,  -0.0630f,-0.2481f,0.9667f,  -0.1736f,-0.1783f,0.9685f,  -0.1737f,-0.1748f,0.9692f,  -0.2809f,-0.1013f,0.9544f,  -0.2807f,-0.0968f,0.9549f,  -0.3826f,-0.0210f,0.9237f,  -0.3821f,-0.0156f,0.9240f,  -0.4769f,0.0612f,0.8768f,  -0.4760f,0.0675f,0.8769f,  -0.5621f,0.1437f,0.8145f,  -0.5607f,0.1510f,0.8141f,  -0.6366f,0.2252f,0.7376f,  -0.6346f,0.2334f,0.7368f,  
	-0.6990f,0.3039f,0.6474f,  -0.6962f,0.3130f,0.6461f,  -0.7480f,0.3784f,0.5453f,  -0.7444f,0.3881f,0.5434f,  -0.7827f,0.4470f,0.4330f,  -0.7782f,0.4573f,0.4305f,  -0.8025f,0.5083f,0.3125f,  -0.7970f,0.5188f,0.3092f,  -0.8070f,0.5606f,0.1856f,  -0.8004f,0.5712f,0.1817f,  -0.7961f,0.6027f,0.0548f,  -0.7885f,0.6130f,0.0502f,  -0.7701f,0.6332f,-0.0777f,  -0.7614f,0.6429f,-0.0829f,  -0.7295f,0.6511f,-0.2096f,  
	-0.7199f,0.6599f,-0.2153f,  -0.6753f,0.6554f,-0.3382f,  -0.6649f,0.6628f,-0.3444f,  -0.6088f,0.6455f,-0.4613f,  -0.5977f,0.6512f,-0.4676f,  -0.5314f,0.6209f,-0.5763f,  -0.5199f,0.6246f,-0.5827f,  -0.4451f,0.5815f,-0.6810f,  -0.4335f,0.5829f,-0.6873f,  -0.3517f,0.5277f,-0.7732f,  -0.3405f,0.5263f,-0.7791f,  -0.2537f,0.4598f,-0.8510f,  -0.2433f,0.4556f,-0.8563f,  -0.1535f,0.3789f,-0.9126f,  -0.1442f,0.3716f,-0.9171f,  
	-0.0534f,0.2861f,-0.9567f,  -0.0457f,0.2758f,-0.9601f,  0.0441f,0.1833f,-0.9821f,  0.0494f,0.1699f,-0.9842f,  0.1364f,0.0723f,-0.9880f,  0.1389f,0.0561f,-0.9887f,  0.2213f,-0.0444f,-0.9742f,  0.2208f,-0.0633f,-0.9733f,  0.2970f,-0.1645f,-0.9406f,  0.2928f,-0.1855f,-0.9380f,  0.3615f,-0.2848f,-0.8878f,  0.3534f,-0.3077f,-0.8834f,  0.4135f,-0.4026f,-0.8166f,  0.4013f,-0.4266f,-0.8105f,  0.4521f,-0.5148f,-0.7284f,  
	0.4358f,-0.5392f,-0.7207f,  0.4768f,-0.6181f,-0.6249f,  0.4566f,-0.6423f,-0.6156f,  0.4879f,-0.7098f,-0.5081f,  0.4639f,-0.7330f,-0.4974f,  0.4857f,-0.7870f,-0.3804f,  0.4586f,-0.8086f,-0.3686f,  0.4715f,-0.8473f,-0.2444f,  0.4420f,-0.8666f,-0.2318f,  0.4470f,-0.8886f,-0.1030f,  0.4158f,-0.9050f,-0.0898f,  0.4140f,-0.9094f,0.0408f,  0.3823f,-0.9225f,0.0542f,  0.3752f,-0.9085f,0.1839f,  0.3440f,-0.9180f,0.1972f,  
	0.3330f,-0.8857f,0.3234f,  0.3037f,-0.8915f,0.3362f,  0.2904f,-0.8412f,0.4561f,  0.2642f,-0.8433f,0.4681f,  0.2502f,-0.7758f,0.5792f,  0.2283f,-0.7746f,0.5898f,  0.2149f,-0.6914f,0.6898f,  0.1985f,-0.6872f,0.6988f,  0.8549f,-0.5163f,0.0517f,  0.8228f,-0.5403f,0.1763f,  0.8238f,-0.5387f,0.1768f,  0.7778f,-0.5531f,0.2985f,  0.7786f,-0.5518f,0.2988f,  0.7196f,-0.5564f,0.4156f,  0.7201f,-0.5555f,0.4157f,  
	0.7196f,-0.5564f,0.4156f,  0.7188f,-0.5575f,0.4154f,  0.6485f,-0.5508f,0.5255f,  0.6477f,-0.5517f,0.5255f,  0.5666f,-0.5350f,0.6267f,  0.5658f,-0.5357f,0.6268f,  0.4753f,-0.5093f,0.7174f,  0.4745f,-0.5096f,0.7177f,  0.3761f,-0.4740f,0.7962f,  0.3753f,-0.4739f,0.7966f,  0.2708f,-0.4295f,0.8615f,  0.2701f,-0.4289f,0.8621f,  0.1612f,-0.3765f,0.9123f,  0.1605f,-0.3752f,0.9129f,  0.0493f,-0.3157f,0.9476f,  
	0.0487f,-0.3137f,0.9483f,  -0.0630f,-0.2481f,0.9667f,  -0.0635f,-0.2453f,0.9674f,  -0.1737f,-0.1748f,0.9692f,  -0.1739f,-0.1710f,0.9698f,  -0.2807f,-0.0968f,0.9549f,  -0.2807f,-0.0921f,0.9554f,  -0.3821f,-0.0156f,0.9240f,  -0.3817f,-0.0098f,0.9242f,  -0.4760f,0.0675f,0.8769f,  -0.4752f,0.0743f,0.8767f,  -0.5607f,0.1510f,0.8141f,  -0.5593f,0.1589f,0.8136f,  -0.6346f,0.2334f,0.7368f,  -0.6325f,0.2422f,0.7358f,  
	-0.6962f,0.3130f,0.6461f,  -0.6933f,0.3226f,0.6444f,  -0.7444f,0.3881f,0.5434f,  -0.7405f,0.3985f,0.5411f,  -0.7782f,0.4573f,0.4305f,  -0.7733f,0.4682f,0.4275f,  -0.7970f,0.5188f,0.3092f,  -0.7911f,0.5300f,0.3055f,  -0.8004f,0.5712f,0.1817f,  -0.7934f,0.5823f,0.1773f,  -0.7885f,0.6130f,0.0502f,  -0.7803f,0.6238f,0.0451f,  -0.7614f,0.6429f,-0.0829f,  -0.7521f,0.6530f,-0.0887f,  -0.7199f,0.6599f,-0.2153f,  
	-0.7096f,0.6689f,-0.2215f,  -0.6649f,0.6628f,-0.3444f,  -0.6538f,0.6704f,-0.3509f,  -0.5977f,0.6512f,-0.4676f,  -0.5859f,0.6569f,-0.4745f,  -0.5199f,0.6246f,-0.5827f,  -0.5078f,0.6281f,-0.5896f,  -0.4335f,0.5829f,-0.6873f,  -0.4214f,0.5838f,-0.6939f,  -0.3405f,0.5263f,-0.7791f,  -0.3289f,0.5245f,-0.7853f,  -0.2433f,0.4556f,-0.8563f,  -0.2325f,0.4507f,-0.8619f,  -0.1442f,0.3716f,-0.9171f,  -0.1348f,0.3635f,-0.9218f,  
	-0.0457f,0.2758f,-0.9601f,  -0.0385f,0.2646f,-0.9636f,  0.0494f,0.1699f,-0.9842f,  0.0541f,0.1557f,-0.9863f,  0.1389f,0.0561f,-0.9887f,  0.1408f,0.0389f,-0.9893f,  0.2208f,-0.0633f,-0.9733f,  0.2191f,-0.0831f,-0.9722f,  0.2928f,-0.1855f,-0.9380f,  0.2873f,-0.2074f,-0.9351f,  0.3534f,-0.3077f,-0.8834f,  0.3437f,-0.3311f,-0.8788f,  0.4013f,-0.4266f,-0.8105f,  0.3873f,-0.4510f,-0.8041f,  0.4358f,-0.5392f,-0.7207f,  
	0.4175f,-0.5638f,-0.7126f,  0.4566f,-0.6423f,-0.6156f,  0.4342f,-0.6665f,-0.6060f,  0.4639f,-0.7330f,-0.4974f,  0.4378f,-0.7561f,-0.4865f,  0.4586f,-0.8086f,-0.3686f,  0.4294f,-0.8298f,-0.3565f,  0.4420f,-0.8666f,-0.2318f,  0.4104f,-0.8852f,-0.2189f,  0.4158f,-0.9050f,-0.0898f,  0.3829f,-0.9206f,-0.0765f,  0.3823f,-0.9225f,0.0542f,  0.3491f,-0.9346f,0.0677f,  0.3440f,-0.9180f,0.1972f,  0.3118f,-0.9265f,0.2105f,  
	0.3037f,-0.8915f,0.3362f,  0.2737f,-0.8963f,0.3489f,  0.2642f,-0.8433f,0.4681f,  0.2377f,-0.8446f,0.4798f,  0.2283f,-0.7746f,0.5898f,  0.2066f,-0.7727f,0.6003f,  0.1985f,-0.6872f,0.6988f,  0.1827f,-0.6826f,0.7076f,  0.8228f,-0.5403f,0.1763f,  0.7768f,-0.5547f,0.2982f,  0.7778f,-0.5531f,0.2985f,  0.7188f,-0.5575f,0.4154f,  0.7196f,-0.5564f,0.4156f,  0.7188f,-0.5575f,0.4154f,  0.7178f,-0.5589f,0.4152f,  
	0.6477f,-0.5517f,0.5255f,  0.6467f,-0.5528f,0.5255f,  0.5658f,-0.5357f,0.6268f,  0.5648f,-0.5365f,0.6270f,  0.4745f,-0.5096f,0.7177f,  0.4735f,-0.5101f,0.7181f,  0.3753f,-0.4739f,0.7966f,  0.3743f,-0.4739f,0.7971f,  0.2701f,-0.4289f,0.8621f,  0.2690f,-0.4283f,0.8627f,  0.1605f,-0.3752f,0.9129f,  0.1595f,-0.3739f,0.9136f,  0.0487f,-0.3137f,0.9483f,  0.0478f,-0.3116f,0.9490f,  -0.0635f,-0.2453f,0.9674f,  
	-0.0643f,-0.2423f,0.9681f,  -0.1739f,-0.1710f,0.9698f,  -0.1745f,-0.1670f,0.9704f,  -0.2807f,-0.0921f,0.9554f,  -0.2809f,-0.0870f,0.9558f,  -0.3817f,-0.0098f,0.9242f,  -0.3815f,-0.0036f,0.9243f,  -0.4752f,0.0743f,0.8767f,  -0.4745f,0.0816f,0.8765f,  -0.5593f,0.1589f,0.8136f,  -0.5579f,0.1673f,0.8129f,  -0.6325f,0.2422f,0.7358f,  -0.6303f,0.2516f,0.7345f,  -0.6933f,0.3226f,0.6444f,  -0.6902f,0.3329f,0.6425f,  
	-0.7405f,0.3985f,0.5411f,  -0.7365f,0.4095f,0.5385f,  -0.7733f,0.4682f,0.4275f,  -0.7682f,0.4796f,0.4241f,  -0.7911f,0.5300f,0.3055f,  -0.7847f,0.5417f,0.3014f,  -0.7934f,0.5823f,0.1773f,  -0.7858f,0.5940f,0.1724f,  -0.7803f,0.6238f,0.0451f,  -0.7715f,0.6350f,0.0395f,  -0.7521f,0.6530f,-0.0887f,  -0.7422f,0.6635f,-0.0949f,  -0.7096f,0.6689f,-0.2215f,  -0.6986f,0.6781f,-0.2283f,  -0.6538f,0.6704f,-0.3509f,  
	-0.6419f,0.6780f,-0.3581f,  -0.5859f,0.6569f,-0.4745f,  -0.5735f,0.6626f,-0.4818f,  -0.5078f,0.6281f,-0.5896f,  -0.4951f,0.6314f,-0.5969f,  -0.4214f,0.5838f,-0.6939f,  -0.4089f,0.5844f,-0.7009f,  -0.3289f,0.5245f,-0.7853f,  -0.3169f,0.5220f,-0.7919f,  -0.2325f,0.4507f,-0.8619f,  -0.2216f,0.4451f,-0.8676f,  -0.1348f,0.3635f,-0.9218f,  -0.1257f,0.3547f,-0.9265f,  -0.0385f,0.2646f,-0.9636f,  -0.0316f,0.2526f,-0.9671f,  
	0.0541f,0.1557f,-0.9863f,  0.0584f,0.1404f,-0.9884f,  0.1408f,0.0389f,-0.9893f,  0.1416f,0.0208f,-0.9897f,  0.2191f,-0.0831f,-0.9722f,  0.2162f,-0.1036f,-0.9708f,  0.2873f,-0.2074f,-0.9351f,  0.2802f,-0.2300f,-0.9320f,  0.3437f,-0.3311f,-0.8788f,  0.3322f,-0.3551f,-0.8738f,  0.3873f,-0.4510f,-0.8041f,  0.3713f,-0.4757f,-0.7974f,  0.4175f,-0.5638f,-0.7126f,  0.3971f,-0.5886f,-0.7042f,  0.4342f,-0.6665f,-0.6060f,  
	0.4096f,-0.6906f,-0.5960f,  0.4378f,-0.7561f,-0.4865f,  0.4095f,-0.7787f,-0.4752f,  0.4294f,-0.8298f,-0.3565f,  0.3981f,-0.8503f,-0.3443f,  0.4104f,-0.8852f,-0.2189f,  0.3769f,-0.9031f,-0.2059f,  0.3829f,-0.9206f,-0.0765f,  0.3482f,-0.9353f,-0.0630f,  0.3491f,-0.9346f,0.0677f,  0.3145f,-0.9458f,0.0813f,  0.3118f,-0.9265f,0.2105f,  0.2785f,-0.9340f,0.2239f,  0.2737f,-0.8963f,0.3489f,  0.2431f,-0.9001f,0.3616f,  
	0.2377f,-0.8446f,0.4798f,  0.2111f,-0.8450f,0.4914f,  0.2066f,-0.7727f,0.6003f,  0.1851f,-0.7701f,0.6105f,  0.1827f,-0.6826f,0.7076f,  0.1675f,-0.6777f,0.7160f,  0.7768f,-0.5547f,0.2982f,  0.7178f,-0.5589f,0.4152f,  0.7188f,-0.5575f,0.4154f,  0.7178f,-0.5589f,0.4152f,  0.7165f,-0.5606f,0.4151f,  0.6467f,-0.5528f,0.5255f,  0.6454f,-0.5543f,0.5255f,  0.5648f,-0.5365f,0.6270f,  0.5635f,-0.5376f,0.6272f,  
	0.4735f,-0.5101f,0.7181f,  0.4721f,-0.5108f,0.7185f,  0.3743f,-0.4739f,0.7971f,  0.3729f,-0.4740f,0.7977f,  0.2690f,-0.4283f,0.8627f,  0.2676f,-0.4277f,0.8634f,  0.1595f,-0.3739f,0.9136f,  0.1582f,-0.3726f,0.9144f,  0.0478f,-0.3116f,0.9490f,  0.0465f,-0.3094f,0.9498f,  -0.0643f,-0.2423f,0.9681f,  -0.0653f,-0.2390f,0.9688f,  -0.1745f,-0.1670f,0.9704f,  -0.1753f,-0.1627f,0.9710f,  -0.2809f,-0.0870f,0.9558f,  
	-0.2814f,-0.0815f,0.9561f,  -0.3815f,-0.0036f,0.9243f,  -0.3815f,0.0030f,0.9244f,  -0.4745f,0.0816f,0.8765f,  -0.4738f,0.0895f,0.8761f,  -0.5579f,0.1673f,0.8129f,  -0.5566f,0.1762f,0.8119f,  -0.6303f,0.2516f,0.7345f,  -0.6281f,0.2616f,0.7329f,  -0.6902f,0.3329f,0.6425f,  -0.6870f,0.3438f,0.6402f,  -0.7365f,0.4095f,0.5385f,  -0.7321f,0.4211f,0.5354f,  -0.7682f,0.4796f,0.4241f,  -0.7626f,0.4918f,0.4203f,  
	-0.7847f,0.5417f,0.3014f,  -0.7778f,0.5540f,0.2967f,  -0.7858f,0.5940f,0.1724f,  -0.7776f,0.6062f,0.1670f,  -0.7715f,0.6350f,0.0395f,  -0.7620f,0.6467f,0.0333f,  -0.7422f,0.6635f,-0.0949f,  -0.7315f,0.6742f,-0.1017f,  -0.6986f,0.6781f,-0.2283f,  -0.6869f,0.6875f,-0.2356f,  -0.6419f,0.6780f,-0.3581f,  -0.6293f,0.6857f,-0.3657f,  -0.5735f,0.6626f,-0.4818f,  -0.5604f,0.6681f,-0.4896f,  -0.4951f,0.6314f,-0.5969f,  
	-0.4819f,0.6343f,-0.6045f,  -0.4089f,0.5844f,-0.7009f,  -0.3958f,0.5845f,-0.7083f,  -0.3169f,0.5220f,-0.7919f,  -0.3046f,0.5190f,-0.7986f,  -0.2216f,0.4451f,-0.8676f,  -0.2107f,0.4388f,-0.8735f,  -0.1257f,0.3547f,-0.9265f,  -0.1168f,0.3452f,-0.9313f,  -0.0316f,0.2526f,-0.9671f,  -0.0250f,0.2396f,-0.9706f,  0.0584f,0.1404f,-0.9884f,  0.0618f,0.1244f,-0.9903f,  0.1416f,0.0208f,-0.9897f,  0.1414f,0.0020f,-0.9899f,  
	0.2162f,-0.1036f,-0.9708f,  0.2119f,-0.1249f,-0.9693f,  0.2802f,-0.2300f,-0.9320f,  0.2714f,-0.2531f,-0.9286f,  0.3322f,-0.3551f,-0.8738f,  0.3188f,-0.3795f,-0.8686f,  0.3713f,-0.4757f,-0.7974f,  0.3532f,-0.5007f,-0.7903f,  0.3971f,-0.5886f,-0.7042f,  0.3744f,-0.6134f,-0.6954f,  0.4096f,-0.6906f,-0.5960f,  0.3827f,-0.7144f,-0.5858f,  0.4095f,-0.7787f,-0.4752f,  0.3789f,-0.8009f,-0.4637f,  0.3981f,-0.8503f,-0.3443f,  
	0.3645f,-0.8701f,-0.3318f,  0.3769f,-0.9031f,-0.2059f,  0.3414f,-0.9200f,-0.1926f,  0.3482f,-0.9353f,-0.0630f,  0.3119f,-0.9488f,-0.0494f,  0.3145f,-0.9458f,0.0813f,  0.2786f,-0.9557f,0.0950f,  0.2785f,-0.9340f,0.2239f,  0.2443f,-0.9402f,0.2372f,  0.2431f,-0.9001f,0.3616f,  0.2120f,-0.9028f,0.3742f,  0.2111f,-0.8450f,0.4914f,  0.1844f,-0.8444f,0.5029f,  0.1851f,-0.7701f,0.6105f,  0.1640f,-0.7669f,0.6204f,  
	0.1675f,-0.6777f,0.7160f,  0.1530f,-0.6726f,0.7240f,  0.7165f,-0.5606f,0.4151f,  0.7150f,-0.5625f,0.4151f,  0.6454f,-0.5543f,0.5255f,  0.6439f,-0.5560f,0.5257f,  0.5635f,-0.5376f,0.6272f,  0.5619f,-0.5390f,0.6276f,  0.4721f,-0.5108f,0.7185f,  0.4704f,-0.5116f,0.7190f,  0.3729f,-0.4740f,0.7977f,  0.3712f,-0.4742f,0.7984f,  0.2676f,-0.4277f,0.8634f,  0.2659f,-0.4272f,0.8642f,  0.1582f,-0.3726f,0.9144f,  
	0.1565f,-0.3712f,0.9153f,  0.0465f,-0.3094f,0.9498f,  0.0449f,-0.3070f,0.9507f,  -0.0653f,-0.2390f,0.9688f,  -0.0667f,-0.2356f,0.9696f,  -0.1753f,-0.1627f,0.9710f,  -0.1764f,-0.1581f,0.9715f,  -0.2814f,-0.0815f,0.9561f,  -0.2820f,-0.0757f,0.9564f,  -0.3815f,0.0030f,0.9244f,  -0.3817f,0.0101f,0.9242f,  -0.4738f,0.0895f,0.8761f,  -0.4733f,0.0978f,0.8755f,  -0.5566f,0.1762f,0.8119f,  -0.5552f,0.1858f,0.8107f,  
	-0.6281f,0.2616f,0.7329f,  -0.6257f,0.2722f,0.7310f,  -0.6870f,0.3438f,0.6402f,  -0.6835f,0.3554f,0.6376f,  -0.7321f,0.4211f,0.5354f,  -0.7274f,0.4334f,0.5320f,  -0.7626f,0.4918f,0.4203f,  -0.7566f,0.5045f,0.4160f,  -0.7778f,0.5540f,0.2967f,  -0.7705f,0.5669f,0.2916f,  -0.7776f,0.6062f,0.1670f,  -0.7688f,0.6189f,0.1610f,  -0.7620f,0.6467f,0.0333f,  -0.7519f,0.6588f,0.0266f,  -0.7315f,0.6742f,-0.1017f,  
	-0.7201f,0.6853f,-0.1090f,  -0.6869f,0.6875f,-0.2356f,  -0.6744f,0.6971f,-0.2434f,  -0.6293f,0.6857f,-0.3657f,  -0.6160f,0.6933f,-0.3739f,  -0.5604f,0.6681f,-0.4896f,  -0.5466f,0.6734f,-0.4977f,  -0.4819f,0.6343f,-0.6045f,  -0.4680f,0.6369f,-0.6126f,  -0.3958f,0.5845f,-0.7083f,  -0.3824f,0.5841f,-0.7160f,  -0.3046f,0.5190f,-0.7986f,  -0.2922f,0.5154f,-0.8056f,  -0.2107f,0.4388f,-0.8735f,  -0.2000f,0.4318f,-0.8795f,  
	-0.1168f,0.3452f,-0.9313f,  -0.1079f,0.3347f,-0.9361f,  -0.0250f,0.2396f,-0.9706f,  -0.0191f,0.2258f,-0.9740f,  0.0618f,0.1244f,-0.9903f,  0.0643f,0.1075f,-0.9921f,  0.1414f,0.0020f,-0.9899f,  0.1400f,-0.0177f,-0.9900f,  0.2119f,-0.1249f,-0.9693f,  0.2060f,-0.1468f,-0.9675f,  0.2714f,-0.2531f,-0.9286f,  0.2607f,-0.2767f,-0.9249f,  0.3188f,-0.3795f,-0.8686f,  0.3032f,-0.4041f,-0.8630f,  0.3532f,-0.5007f,-0.7903f,  
	0.3327f,-0.5257f,-0.7829f,  0.3744f,-0.6134f,-0.6954f,  0.3492f,-0.6379f,-0.6864f,  0.3827f,-0.7144f,-0.5858f,  0.3533f,-0.7378f,-0.5753f,  0.3789f,-0.8009f,-0.4637f,  0.3459f,-0.8222f,-0.4520f,  0.3645f,-0.8701f,-0.3318f,  0.3288f,-0.8889f,-0.3190f,  0.3414f,-0.9200f,-0.1926f,  0.3039f,-0.9357f,-0.1792f,  0.3119f,-0.9488f,-0.0494f,  0.2739f,-0.9611f,-0.0355f,  0.2786f,-0.9557f,0.0950f,  0.2414f,-0.9643f,0.1088f,  
	0.2443f,-0.9402f,0.2372f,  0.2093f,-0.9452f,0.2506f,  0.2120f,-0.9028f,0.3742f,  0.1805f,-0.9043f,0.3868f,  0.1844f,-0.8444f,0.5029f,  0.1577f,-0.8430f,0.5143f,  0.1640f,-0.7669f,0.6204f,  0.1433f,-0.7631f,0.6302f,  0.1530f,-0.6726f,0.7240f,  0.1392f,-0.6672f,0.7317f,  0.7150f,-0.5625f,0.4151f,  0.7132f,-0.5648f,0.4151f,  0.6439f,-0.5560f,0.5257f,  0.6420f,-0.5579f,0.5259f,  0.5619f,-0.5390f,0.6276f,  
	0.5599f,-0.5405f,0.6281f,  0.4704f,-0.5116f,0.7190f,  0.4684f,-0.5126f,0.7197f,  0.3712f,-0.4742f,0.7984f,  0.3691f,-0.4745f,0.7992f,  0.2659f,-0.4272f,0.8642f,  0.2638f,-0.4266f,0.8651f,  0.1565f,-0.3712f,0.9153f,  0.1545f,-0.3697f,0.9162f,  0.0449f,-0.3070f,0.9507f,  0.0430f,-0.3045f,0.9516f,  -0.0667f,-0.2356f,0.9696f,  -0.0684f,-0.2319f,0.9703f,  -0.1764f,-0.1581f,0.9715f,  -0.1777f,-0.1531f,0.9721f,  
	-0.2820f,-0.0757f,0.9564f,  -0.2829f,-0.0694f,0.9566f,  -0.3817f,0.0101f,0.9242f,  -0.3819f,0.0178f,0.9240f,  -0.4733f,0.0978f,0.8755f,  -0.4728f,0.1068f,0.8747f,  -0.5552f,0.1858f,0.8107f,  -0.5538f,0.1960f,0.8093f,  -0.6257f,0.2722f,0.7310f,  -0.6232f,0.2835f,0.7288f,  -0.6835f,0.3554f,0.6376f,  -0.6798f,0.3677f,0.6346f,  -0.7274f,0.4334f,0.5320f,  -0.7224f,0.4464f,0.5281f,  -0.7566f,0.5045f,0.4160f,  
	-0.7501f,0.5179f,0.4112f,  -0.7705f,0.5669f,0.2916f,  -0.7625f,0.5804f,0.2859f,  -0.7688f,0.6189f,0.1610f,  -0.7594f,0.6320f,0.1545f,  -0.7519f,0.6588f,0.0266f,  -0.7410f,0.6712f,0.0194f,  -0.7201f,0.6853f,-0.1090f,  -0.7079f,0.6965f,-0.1169f,  -0.6744f,0.6971f,-0.2434f,  -0.6611f,0.7068f,-0.2518f,  -0.6160f,0.6933f,-0.3739f,  -0.6020f,0.7009f,-0.3824f,  -0.5466f,0.6734f,-0.4977f,  -0.5322f,0.6785f,-0.5064f,  
	-0.4680f,0.6369f,-0.6126f,  -0.4536f,0.6392f,-0.6211f,  -0.3824f,0.5841f,-0.7160f,  -0.3686f,0.5832f,-0.7239f,  -0.2922f,0.5154f,-0.8056f,  -0.2797f,0.5112f,-0.8127f,  -0.2000f,0.4318f,-0.8795f,  -0.1891f,0.4240f,-0.8857f,  -0.1079f,0.3347f,-0.9361f,  -0.0997f,0.3234f,-0.9410f,  -0.0191f,0.2258f,-0.9740f,  -0.0140f,0.2112f,-0.9773f,  0.0643f,0.1075f,-0.9921f,  0.0656f,0.0898f,-0.9938f,  0.1400f,-0.0177f,-0.9900f,  
	0.1370f,-0.0380f,-0.9898f,  0.2060f,-0.1468f,-0.9675f,  0.1983f,-0.1693f,-0.9654f,  0.2607f,-0.2767f,-0.9249f,  0.2481f,-0.3007f,-0.9209f,  0.3032f,-0.4041f,-0.8630f,  0.2854f,-0.4289f,-0.8571f,  0.3327f,-0.5257f,-0.7829f,  0.3099f,-0.5505f,-0.7752f,  0.3492f,-0.6379f,-0.6864f,  0.3216f,-0.6620f,-0.6770f,  0.3533f,-0.7378f,-0.5753f,  0.3214f,-0.7604f,-0.5644f,  0.3459f,-0.8222f,-0.4520f,  0.3105f,-0.8427f,-0.4398f,  
	0.3288f,-0.8889f,-0.3190f,  0.2908f,-0.9066f,-0.3059f,  0.3039f,-0.9357f,-0.1792f,  0.2645f,-0.9501f,-0.1654f,  0.2739f,-0.9611f,-0.0355f,  0.2343f,-0.9719f,-0.0215f,  0.2414f,-0.9643f,0.1088f,  0.2030f,-0.9715f,0.1227f,  0.2093f,-0.9452f,0.2506f,  0.1735f,-0.9488f,0.2641f,  0.1805f,-0.9043f,0.3868f,  0.1487f,-0.9046f,0.3994f,  0.1577f,-0.8430f,0.5143f,  0.1311f,-0.8406f,0.5256f,  0.1433f,-0.7631f,0.6302f,  
	0.1230f,-0.7587f,0.6397f,  0.1392f,-0.6672f,0.7317f,  0.1261f,-0.6617f,0.7391f,  0.7132f,-0.5648f,0.4151f,  0.7111f,-0.5673f,0.4153f,  0.6420f,-0.5579f,0.5259f,  0.6397f,-0.5601f,0.5264f,  0.5599f,-0.5405f,0.6281f,  0.5575f,-0.5422f,0.6287f,  0.4684f,-0.5126f,0.7197f,  0.4659f,-0.5136f,0.7205f,  0.3691f,-0.4745f,0.7992f,  0.3666f,-0.4748f,0.8001f,  0.2638f,-0.4266f,0.8651f,  0.2613f,-0.4261f,0.8661f,  
	0.1545f,-0.3697f,0.9162f,  0.1520f,-0.3681f,0.9173f,  0.0430f,-0.3045f,0.9516f,  0.0408f,-0.3017f,0.9525f,  -0.0684f,-0.2319f,0.9703f,  -0.0704f,-0.2278f,0.9712f,  -0.1777f,-0.1531f,0.9721f,  -0.1794f,-0.1477f,0.9726f,  -0.2829f,-0.0694f,0.9566f,  -0.2840f,-0.0626f,0.9568f,  -0.3819f,0.0178f,0.9240f,  -0.3823f,0.0260f,0.9237f,  -0.4728f,0.1068f,0.8747f,  -0.4723f,0.1164f,0.8737f,  -0.5538f,0.1960f,0.8093f,  
	-0.5523f,0.2069f,0.8076f,  -0.6232f,0.2835f,0.7288f,  -0.6206f,0.2956f,0.7263f,  -0.6798f,0.3677f,0.6346f,  -0.6758f,0.3807f,0.6312f,  -0.7224f,0.4464f,0.5281f,  -0.7169f,0.4601f,0.5238f,  -0.7501f,0.5179f,0.4112f,  -0.7431f,0.5320f,0.4059f,  -0.7625f,0.5804f,0.2859f,  -0.7539f,0.5945f,0.2797f,  -0.7594f,0.6320f,0.1545f,  -0.7492f,0.6457f,0.1475f,  -0.7410f,0.6712f,0.0194f,  -0.7293f,0.6841f,0.0116f,  
	-0.7079f,0.6965f,-0.1169f,  -0.6949f,0.7081f,-0.1253f,  -0.6611f,0.7068f,-0.2518f,  -0.6471f,0.7165f,-0.2606f,  -0.6020f,0.7009f,-0.3824f,  -0.5872f,0.7084f,-0.3915f,  -0.5322f,0.6785f,-0.5064f,  -0.5171f,0.6833f,-0.5155f,  -0.4536f,0.6392f,-0.6211f,  -0.4388f,0.6410f,-0.6298f,  -0.3686f,0.5832f,-0.7239f,  -0.3547f,0.5817f,-0.7320f,  -0.2797f,0.5112f,-0.8127f,  -0.2670f,0.5061f,-0.8201f,  -0.1891f,0.4240f,-0.8857f,  
	-0.1786f,0.4155f,-0.8919f,  -0.0997f,0.3234f,-0.9410f,  -0.0920f,0.3114f,-0.9458f,  -0.0140f,0.2112f,-0.9773f,  -0.0098f,0.1959f,-0.9806f,  0.0656f,0.0898f,-0.9938f,  0.0657f,0.0715f,-0.9953f,  0.1370f,-0.0380f,-0.9898f,  0.1324f,-0.0589f,-0.9894f,  0.1983f,-0.1693f,-0.9654f,  0.1887f,-0.1921f,-0.9631f,  0.2481f,-0.3007f,-0.9209f,  0.2332f,-0.3249f,-0.9166f,  0.2854f,-0.4289f,-0.8571f,  0.2652f,-0.4537f,-0.8508f,  
	0.3099f,-0.5505f,-0.7752f,  0.2844f,-0.5750f,-0.7671f,  0.3216f,-0.6620f,-0.6770f,  0.2913f,-0.6855f,-0.6673f,  0.3214f,-0.7604f,-0.5644f,  0.2869f,-0.7821f,-0.5532f,  0.3105f,-0.8427f,-0.4398f,  0.2727f,-0.8620f,-0.4274f,  0.2908f,-0.9066f,-0.3059f,  0.2507f,-0.9228f,-0.2925f,  0.2645f,-0.9501f,-0.1654f,  0.2233f,-0.9629f,-0.1514f,  0.2343f,-0.9719f,-0.0215f,  0.1933f,-0.9811f,-0.0073f,  0.2030f,-0.9715f,0.1227f,  
	0.1636f,-0.9770f,0.1368f,  0.1735f,-0.9488f,0.2641f,  0.1372f,-0.9508f,0.2777f,  0.1487f,-0.9046f,0.3994f,  0.1167f,-0.9037f,0.4120f,  0.1311f,-0.8406f,0.5256f,  0.1048f,-0.8372f,0.5368f,  0.1230f,-0.7587f,0.6397f,  0.1033f,-0.7537f,0.6490f,  0.1261f,-0.6617f,0.7391f,  0.1137f,-0.6560f,0.7461f,  0.7111f,-0.5673f,0.4153f,  0.7087f,-0.5701f,0.4156f,  0.6397f,-0.5601f,0.5264f,  0.6372f,-0.5625f,0.5269f,  
	0.5575f,-0.5422f,0.6287f,  0.5548f,-0.5440f,0.6294f,  0.4659f,-0.5136f,0.7205f,  0.4631f,-0.5149f,0.7214f,  0.3666f,-0.4748f,0.8001f,  0.3637f,-0.4752f,0.8012f,  0.2613f,-0.4261f,0.8661f,  0.2584f,-0.4255f,0.8673f,  0.1520f,-0.3681f,0.9173f,  0.1492f,-0.3664f,0.9184f,  0.0408f,-0.3017f,0.9525f,  0.0381f,-0.2987f,0.9536f,  -0.0704f,-0.2278f,0.9712f,  -0.0728f,-0.2235f,0.9720f,  -0.1794f,-0.1477f,0.9726f,  
	-0.1813f,-0.1418f,0.9732f,  -0.2840f,-0.0626f,0.9568f,  -0.2853f,-0.0553f,0.9568f,  -0.3823f,0.0260f,0.9237f,  -0.3828f,0.0348f,0.9232f,  -0.4723f,0.1164f,0.8737f,  -0.4719f,0.1266f,0.8725f,  -0.5523f,0.2069f,0.8076f,  -0.5507f,0.2185f,0.8056f,  -0.6206f,0.2956f,0.7263f,  -0.6177f,0.3084f,0.7235f,  -0.6758f,0.3807f,0.6312f,  -0.6714f,0.3944f,0.6274f,  -0.7169f,0.4601f,0.5238f,  -0.7110f,0.4745f,0.5190f,  
	-0.7431f,0.5320f,0.4059f,  -0.7355f,0.5467f,0.4002f,  -0.7539f,0.5945f,0.2797f,  -0.7446f,0.6092f,0.2730f,  -0.7492f,0.6457f,0.1475f,  -0.7382f,0.6599f,0.1399f,  -0.7293f,0.6841f,0.0116f,  -0.7168f,0.6972f,0.0032f,  -0.6949f,0.7081f,-0.1253f,  -0.6812f,0.7197f,-0.1342f,  -0.6471f,0.7165f,-0.2606f,  -0.6323f,0.7262f,-0.2699f,  -0.5872f,0.7084f,-0.3915f,  -0.5717f,0.7157f,-0.4011f,  -0.5171f,0.6833f,-0.5155f,  
	-0.5014f,0.6878f,-0.5249f,  -0.4388f,0.6410f,-0.6298f,  -0.4236f,0.6423f,-0.6387f,  -0.3547f,0.5817f,-0.7320f,  -0.3403f,0.5796f,-0.7404f,  -0.2670f,0.5061f,-0.8201f,  -0.2545f,0.5004f,-0.8275f,  -0.1786f,0.4155f,-0.8919f,  -0.1685f,0.4062f,-0.8981f,  -0.0920f,0.3114f,-0.9458f,  -0.0851f,0.2985f,-0.9506f,  -0.0098f,0.1959f,-0.9806f,  -0.0067f,0.1798f,-0.9837f,  0.0657f,0.0715f,-0.9953f,  0.0643f,0.0525f,-0.9965f,  
	0.1324f,-0.0589f,-0.9894f,  0.1261f,-0.0803f,-0.9888f,  0.1887f,-0.1921f,-0.9631f,  0.1770f,-0.2153f,-0.9604f,  0.2332f,-0.3249f,-0.9166f,  0.2160f,-0.3491f,-0.9119f,  0.2652f,-0.4537f,-0.8508f,  0.2424f,-0.4781f,-0.8442f,  0.2844f,-0.5750f,-0.7671f,  0.2563f,-0.5989f,-0.7587f,  0.2913f,-0.6855f,-0.6673f,  0.2585f,-0.7081f,-0.6571f,  0.2869f,-0.7821f,-0.5532f,  0.2499f,-0.8027f,-0.5416f,  0.2727f,-0.8620f,-0.4274f,  
	0.2325f,-0.8798f,-0.4145f,  0.2507f,-0.9228f,-0.2925f,  0.2084f,-0.9375f,-0.2787f,  0.2233f,-0.9629f,-0.1514f,  0.1802f,-0.9740f,-0.1371f,  0.1933f,-0.9811f,-0.0073f,  0.1509f,-0.9885f,0.0073f,  0.1636f,-0.9770f,0.1368f,  0.1233f,-0.9808f,0.1512f,  0.1372f,-0.9508f,0.2777f,  0.1004f,-0.9513f,0.2914f,  0.1167f,-0.9037f,0.4120f,  0.0848f,-0.9014f,0.4247f,  0.1048f,-0.8372f,0.5368f,  0.0788f,-0.8328f,0.5479f,  
	0.1033f,-0.7537f,0.6490f,  0.0842f,-0.7482f,0.6581f,  0.1137f,-0.6560f,0.7461f,  0.1021f,-0.6503f,0.7528f,  0.7087f,-0.5701f,0.4156f,  0.7059f,-0.5732f,0.4161f,  0.6372f,-0.5625f,0.5269f,  0.6342f,-0.5651f,0.5277f,  0.5548f,-0.5440f,0.6294f,  0.5517f,-0.5461f,0.6304f,  0.4631f,-0.5149f,0.7214f,  0.4599f,-0.5161f,0.7226f,  0.3637f,-0.4752f,0.8012f,  0.3604f,-0.4756f,0.8025f,  0.2584f,-0.4255f,0.8673f,  
	0.2551f,-0.4248f,0.8686f,  0.1492f,-0.3664f,0.9184f,  0.1460f,-0.3645f,0.9197f,  0.0381f,-0.2987f,0.9536f,  0.0351f,-0.2954f,0.9547f,  -0.0728f,-0.2235f,0.9720f,  -0.0754f,-0.2187f,0.9729f,  -0.1813f,-0.1418f,0.9732f,  -0.1834f,-0.1356f,0.9736f,  -0.2853f,-0.0553f,0.9568f,  -0.2868f,-0.0474f,0.9568f,  -0.3828f,0.0348f,0.9232f,  -0.3834f,0.0442f,0.9225f,  -0.4719f,0.1266f,0.8725f,  -0.4714f,0.1376f,0.8711f,  
	-0.5507f,0.2185f,0.8056f,  -0.5489f,0.2308f,0.8034f,  -0.6177f,0.3084f,0.7235f,  -0.6145f,0.3219f,0.7203f,  -0.6714f,0.3944f,0.6274f,  -0.6667f,0.4089f,0.6232f,  -0.7110f,0.4745f,0.5190f,  -0.7045f,0.4897f,0.5138f,  -0.7355f,0.5467f,0.4002f,  -0.7272f,0.5621f,0.3939f,  -0.7446f,0.6092f,0.2730f,  -0.7345f,0.6244f,0.2658f,  -0.7382f,0.6599f,0.1399f,  -0.7265f,0.6745f,0.1317f,  -0.7168f,0.6972f,0.0032f,  
	-0.7035f,0.7107f,-0.0056f,  -0.6812f,0.7197f,-0.1342f,  -0.6664f,0.7316f,-0.1437f,  -0.6323f,0.7262f,-0.2699f,  -0.6165f,0.7359f,-0.2798f,  -0.5717f,0.7157f,-0.4011f,  -0.5555f,0.7229f,-0.4110f,  -0.5014f,0.6878f,-0.5249f,  -0.4853f,0.6920f,-0.5345f,  -0.4236f,0.6423f,-0.6387f,  -0.4078f,0.6431f,-0.6481f,  -0.3403f,0.5796f,-0.7404f,  -0.3260f,0.5769f,-0.7490f,  -0.2545f,0.5004f,-0.8275f,  -0.2421f,0.4940f,-0.8351f,  
	-0.1685f,0.4062f,-0.8981f,  -0.1589f,0.3961f,-0.9044f,  -0.0851f,0.2985f,-0.9506f,  -0.0791f,0.2850f,-0.9553f,  -0.0067f,0.1798f,-0.9837f,  -0.0049f,0.1630f,-0.9866f,  0.0643f,0.0525f,-0.9965f,  0.0613f,0.0330f,-0.9976f,  0.1261f,-0.0803f,-0.9888f,  0.1178f,-0.1020f,-0.9878f,  0.1770f,-0.2153f,-0.9604f,  0.1631f,-0.2385f,-0.9573f,  0.2160f,-0.3491f,-0.9119f,  0.1962f,-0.3731f,-0.9068f,  0.2424f,-0.4781f,-0.8442f,  
	0.2169f,-0.5021f,-0.8372f,  0.2563f,-0.5989f,-0.7587f,  0.2256f,-0.6220f,-0.7498f,  0.2585f,-0.7081f,-0.6571f,  0.2229f,-0.7296f,-0.6465f,  0.2499f,-0.8027f,-0.5416f,  0.2103f,-0.8218f,-0.5295f,  0.2325f,-0.8798f,-0.4145f,  0.1899f,-0.8960f,-0.4013f,  0.2084f,-0.9375f,-0.2787f,  0.1641f,-0.9503f,-0.2646f,  0.1802f,-0.9740f,-0.1371f,  0.1356f,-0.9832f,-0.1224f,  0.1509f,-0.9885f,0.0073f,  0.1073f,-0.9940f,0.0222f,  
	0.1233f,-0.9808f,0.1512f,  0.0823f,-0.9827f,0.1658f,  0.1004f,-0.9513f,0.2914f,  0.0633f,-0.9501f,0.3053f,  0.0848f,-0.9014f,0.4247f,  0.0530f,-0.8977f,0.4374f,  0.0788f,-0.8328f,0.5479f,  0.0533f,-0.8275f,0.5589f,  0.0842f,-0.7482f,0.6581f,  0.0658f,-0.7421f,0.6670f,  0.1021f,-0.6503f,0.7528f,  0.0912f,-0.6445f,0.7591f,  0.7059f,-0.5732f,0.4161f,  0.7027f,-0.5765f,0.4168f,  0.6342f,-0.5651f,0.5277f,  
	0.6309f,-0.5680f,0.5286f,  0.5517f,-0.5461f,0.6304f,  0.5482f,-0.5483f,0.6316f,  0.4599f,-0.5161f,0.7226f,  0.4562f,-0.5175f,0.7239f,  0.3604f,-0.4756f,0.8025f,  0.3567f,-0.4759f,0.8039f,  0.2551f,-0.4248f,0.8686f,  0.2514f,-0.4240f,0.8701f,  0.1460f,-0.3645f,0.9197f,  0.1424f,-0.3623f,0.9211f,  0.0351f,-0.2954f,0.9547f,  0.0317f,-0.2918f,0.9559f,  -0.0754f,-0.2187f,0.9729f,  -0.0784f,-0.2135f,0.9738f,  
	-0.1834f,-0.1356f,0.9736f,  -0.1858f,-0.1287f,0.9741f,  -0.2868f,-0.0474f,0.9568f,  -0.2884f,-0.0389f,0.9567f,  -0.3834f,0.0442f,0.9225f,  -0.3840f,0.0544f,0.9217f,  -0.4714f,0.1376f,0.8711f,  -0.4708f,0.1493f,0.8695f,  -0.5489f,0.2308f,0.8034f,  -0.5470f,0.2440f,0.8008f,  -0.6145f,0.3219f,0.7203f,  -0.6110f,0.3363f,0.7167f,  -0.6667f,0.4089f,0.6232f,  -0.6614f,0.4242f,0.6185f,  -0.7045f,0.4897f,0.5138f,  
	-0.6974f,0.5055f,0.5080f,  -0.7272f,0.5621f,0.3939f,  -0.7182f,0.5782f,0.3871f,  -0.7345f,0.6244f,0.2658f,  -0.7236f,0.6401f,0.2580f,  -0.7265f,0.6745f,0.1317f,  -0.7138f,0.6894f,0.1231f,  -0.7035f,0.7107f,-0.0056f,  -0.6892f,0.7244f,-0.0151f,  -0.6664f,0.7316f,-0.1437f,  -0.6508f,0.7435f,-0.1537f,  -0.6165f,0.7359f,-0.2798f,  -0.6000f,0.7456f,-0.2900f,  -0.5555f,0.7229f,-0.4110f,  -0.5386f,0.7297f,-0.4212f,  
	-0.4853f,0.6920f,-0.5345f,  -0.4683f,0.6957f,-0.5447f,  -0.4078f,0.6431f,-0.6481f,  -0.3919f,0.6434f,-0.6576f,  -0.3260f,0.5769f,-0.7490f,  -0.3115f,0.5734f,-0.7577f,  -0.2421f,0.4940f,-0.8351f,  -0.2300f,0.4869f,-0.8426f,  -0.1589f,0.3961f,-0.9044f,  -0.1500f,0.3853f,-0.9105f,  -0.0791f,0.2850f,-0.9553f,  -0.0740f,0.2707f,-0.9598f,  -0.0049f,0.1630f,-0.9866f,  -0.0045f,0.1457f,-0.9893f,  0.0613f,0.0330f,-0.9976f,  
	0.0566f,0.0131f,-0.9983f,  0.1178f,-0.1020f,-0.9878f,  0.1074f,-0.1239f,-0.9865f,  0.1631f,-0.2385f,-0.9573f,  0.1467f,-0.2617f,-0.9539f,  0.1962f,-0.3731f,-0.9068f,  0.1739f,-0.3967f,-0.9013f,  0.2169f,-0.5021f,-0.8372f,  0.1888f,-0.5253f,-0.8297f,  0.2256f,-0.6220f,-0.7498f,  0.1920f,-0.6441f,-0.7404f,  0.2229f,-0.7296f,-0.6465f,  0.1846f,-0.7498f,-0.6354f,  0.2103f,-0.8218f,-0.5295f,  0.1682f,-0.8393f,-0.5170f,  
	0.1899f,-0.8960f,-0.4013f,  0.1452f,-0.9103f,-0.3876f,  0.1641f,-0.9503f,-0.2646f,  0.1180f,-0.9610f,-0.2500f,  0.1356f,-0.9832f,-0.1224f,  0.0895f,-0.9902f,-0.1072f,  0.1073f,-0.9940f,0.0222f,  0.0628f,-0.9973f,0.0374f,  0.0823f,-0.9827f,0.1658f,  0.0408f,-0.9827f,0.1807f,  0.0633f,-0.9501f,0.3053f,  0.0262f,-0.9473f,0.3194f,  0.0530f,-0.8977f,0.4374f,  0.0215f,-0.8927f,0.4502f,  0.0533f,-0.8275f,0.5589f,  
	0.0284f,-0.8212f,0.5699f,  0.0658f,-0.7421f,0.6670f,  0.0481f,-0.7356f,0.6758f,  0.0912f,-0.6445f,0.7591f,  0.0811f,-0.6388f,0.7651f,  0.7027f,-0.5765f,0.4168f,  0.6992f,-0.5802f,0.4178f,  0.6309f,-0.5680f,0.5286f,  0.6271f,-0.5710f,0.5298f,  0.5482f,-0.5483f,0.6316f,  0.5442f,-0.5506f,0.6330f,  0.4562f,-0.5175f,0.7239f,  0.4521f,-0.5189f,0.7255f,  0.3567f,-0.4759f,0.8039f,  0.3525f,-0.4762f,0.8056f,  
	0.2514f,-0.4240f,0.8701f,  0.2472f,-0.4230f,0.8718f,  0.1424f,-0.3623f,0.9211f,  0.1383f,-0.3599f,0.9227f,  0.0317f,-0.2918f,0.9559f,  0.0280f,-0.2879f,0.9573f,  -0.0784f,-0.2135f,0.9738f,  -0.0817f,-0.2079f,0.9747f,  -0.1858f,-0.1287f,0.9741f,  -0.1884f,-0.1214f,0.9746f,  -0.2884f,-0.0389f,0.9567f,  -0.2901f,-0.0297f,0.9565f,  -0.3840f,0.0544f,0.9217f,  -0.3847f,0.0653f,0.9207f,  -0.4708f,0.1493f,0.8695f,  
	-0.4702f,0.1618f,0.8676f,  -0.5470f,0.2440f,0.8008f,  -0.5448f,0.2579f,0.7979f,  -0.6110f,0.3363f,0.7167f,  -0.6070f,0.3514f,0.7127f,  -0.6614f,0.4242f,0.6185f,  -0.6556f,0.4402f,0.6135f,  -0.6974f,0.5055f,0.5080f,  -0.6896f,0.5221f,0.5018f,  -0.7182f,0.5782f,0.3871f,  -0.7084f,0.5949f,0.3798f,  -0.7236f,0.6401f,0.2580f,  -0.7119f,0.6564f,0.2497f,  -0.7138f,0.6894f,0.1231f,  -0.7002f,0.7048f,0.1139f,  
	-0.6892f,0.7244f,-0.0151f,  -0.6740f,0.7383f,-0.0250f,  -0.6508f,0.7435f,-0.1537f,  -0.6343f,0.7555f,-0.1641f,  -0.6000f,0.7456f,-0.2900f,  -0.5828f,0.7550f,-0.3006f,  -0.5386f,0.7297f,-0.4212f,  -0.5208f,0.7363f,-0.4319f,  -0.4683f,0.6957f,-0.5447f,  -0.4511f,0.6989f,-0.5550f,  -0.3919f,0.6434f,-0.6576f,  -0.3756f,0.6431f,-0.6674f,  -0.3115f,0.5734f,-0.7577f,  -0.2972f,0.5694f,-0.7665f,  -0.2300f,0.4869f,-0.8426f,  
	-0.2184f,0.4791f,-0.8502f,  -0.1500f,0.3853f,-0.9105f,  -0.1419f,0.3739f,-0.9166f,  -0.0740f,0.2707f,-0.9598f,  -0.0702f,0.2559f,-0.9641f,  -0.0045f,0.1457f,-0.9893f,  -0.0056f,0.1279f,-0.9918f,  0.0566f,0.0131f,-0.9983f,  0.0499f,-0.0071f,-0.9987f,  0.1074f,-0.1239f,-0.9865f,  0.0947f,-0.1458f,-0.9848f,  0.1467f,-0.2617f,-0.9539f,  0.1278f,-0.2845f,-0.9501f,  0.1739f,-0.3967f,-0.9013f,  0.1488f,-0.4197f,-0.8954f,  
	0.1888f,-0.5253f,-0.8297f,  0.1579f,-0.5476f,-0.8217f,  0.1920f,-0.6441f,-0.7404f,  0.1557f,-0.6649f,-0.7305f,  0.1846f,-0.7498f,-0.6354f,  0.1437f,-0.7682f,-0.6238f,  0.1682f,-0.8393f,-0.5170f,  0.1238f,-0.8548f,-0.5039f,  0.1452f,-0.9103f,-0.3876f,  0.0983f,-0.9225f,-0.3733f,  0.1180f,-0.9610f,-0.2500f,  0.0702f,-0.9695f,-0.2348f,  0.0895f,-0.9902f,-0.1072f,  0.0422f,-0.9949f,-0.0916f,  0.0628f,-0.9973f,0.0374f,  
	0.0175f,-0.9984f,0.0531f,  0.0408f,-0.9827f,0.1807f,  -0.0010f,-0.9806f,0.1960f,  0.0262f,-0.9473f,0.3194f,  -0.0107f,-0.9426f,0.3337f,  0.0215f,-0.8927f,0.4502f,  -0.0095f,-0.8863f,0.4630f,  0.0284f,-0.8212f,0.5699f,  0.0042f,-0.8140f,0.5808f,  0.0481f,-0.7356f,0.6758f,  0.0312f,-0.7286f,0.6842f,  0.0811f,-0.6388f,0.7651f,  0.0717f,-0.6330f,0.7708f,  0.6992f,-0.5802f,0.4178f,  0.6953f,-0.5840f,0.4189f,  
	0.6271f,-0.5710f,0.5298f,  0.6229f,-0.5743f,0.5312f,  0.5442f,-0.5506f,0.6330f,  0.5398f,-0.5530f,0.6346f,  0.4521f,-0.5189f,0.7255f,  0.4475f,-0.5204f,0.7273f,  0.3525f,-0.4762f,0.8056f,  0.3478f,-0.4765f,0.8075f,  0.2472f,-0.4230f,0.8718f,  0.2426f,-0.4219f,0.8736f,  0.1383f,-0.3599f,0.9227f,  0.1339f,-0.3573f,0.9244f,  0.0280f,-0.2879f,0.9573f,  0.0239f,-0.2835f,0.9587f,  -0.0817f,-0.2079f,0.9747f,  
	-0.0852f,-0.2018f,0.9757f,  -0.1884f,-0.1214f,0.9746f,  -0.1912f,-0.1134f,0.9750f,  -0.2901f,-0.0297f,0.9565f,  -0.2919f,-0.0199f,0.9562f,  -0.3847f,0.0653f,0.9207f,  -0.3853f,0.0769f,0.9196f,  -0.4702f,0.1618f,0.8676f,  -0.4693f,0.1751f,0.8655f,  -0.5448f,0.2579f,0.7979f,  -0.5423f,0.2727f,0.7947f,  -0.6070f,0.3514f,0.7127f,  -0.6027f,0.3674f,0.7084f,  -0.6556f,0.4402f,0.6135f,  -0.6492f,0.4571f,0.6079f,  
	-0.6896f,0.5221f,0.5018f,  -0.6811f,0.5394f,0.4951f,  -0.7084f,0.5949f,0.3798f,  -0.6978f,0.6121f,0.3720f,  -0.7119f,0.6564f,0.2497f,  -0.6992f,0.6732f,0.2408f,  -0.7002f,0.7048f,0.1139f,  -0.6856f,0.7205f,0.1041f,  -0.6740f,0.7383f,-0.0250f,  -0.6577f,0.7524f,-0.0354f,  -0.6343f,0.7555f,-0.1641f,  -0.6169f,0.7674f,-0.1748f,  -0.5828f,0.7550f,-0.3006f,  -0.5644f,0.7643f,-0.3118f,  -0.5208f,0.7363f,-0.4319f,  
	-0.5026f,0.7425f,-0.4428f,  -0.4511f,0.6989f,-0.5550f,  -0.4333f,0.7017f,-0.5656f,  -0.3756f,0.6431f,-0.6674f,  -0.3593f,0.6421f,-0.6772f,  -0.2972f,0.5694f,-0.7665f,  -0.2831f,0.5647f,-0.7753f,  -0.2184f,0.4791f,-0.8502f,  -0.2073f,0.4706f,-0.8576f,  -0.1419f,0.3739f,-0.9166f,  -0.1347f,0.3618f,-0.9225f,  -0.0702f,0.2559f,-0.9641f,  -0.0677f,0.2406f,-0.9683f,  -0.0056f,0.1279f,-0.9918f,  -0.0085f,0.1098f,-0.9939f,  
	0.0499f,-0.0071f,-0.9987f,  0.0410f,-0.0274f,-0.9988f,  0.0947f,-0.1458f,-0.9848f,  0.0796f,-0.1674f,-0.9827f,  0.1278f,-0.2845f,-0.9501f,  0.1063f,-0.3068f,-0.9458f,  0.1488f,-0.4197f,-0.8954f,  0.1211f,-0.4418f,-0.8889f,  0.1579f,-0.5476f,-0.8217f,  0.1242f,-0.5687f,-0.8131f,  0.1557f,-0.6649f,-0.7305f,  0.1167f,-0.6840f,-0.7201f,  0.1437f,-0.7682f,-0.6238f,  0.1002f,-0.7848f,-0.6117f,  0.1238f,-0.8548f,-0.5039f,  
	0.0770f,-0.8682f,-0.4902f,  0.0983f,-0.9225f,-0.3733f,  0.0496f,-0.9323f,-0.3584f,  0.0702f,-0.9695f,-0.2348f,  0.0209f,-0.9755f,-0.2191f,  0.0422f,-0.9949f,-0.0916f,  -0.0060f,-0.9971f,-0.0754f,  0.0175f,-0.9984f,0.0531f,  -0.0281f,-0.9972f,0.0693f,  -0.0010f,-0.9806f,0.1960f,  -0.0427f,-0.9764f,0.2116f,  -0.0107f,-0.9426f,0.3337f,  -0.0472f,-0.9362f,0.3483f,  -0.0095f,-0.8863f,0.4630f,  -0.0398f,-0.8785f,0.4760f,  
	0.0042f,-0.8140f,0.5808f,  -0.0192f,-0.8059f,0.5917f,  0.0312f,-0.7286f,0.6842f,  0.0151f,-0.7212f,0.6926f,  0.0717f,-0.6330f,0.7708f,  0.0630f,-0.6273f,0.7762f,  0.6953f,-0.5840f,0.4189f,  0.6909f,-0.5881f,0.4204f,  0.6229f,-0.5743f,0.5312f,  0.6183f,-0.5777f,0.5329f,  0.5398f,-0.5530f,0.6346f,  0.5349f,-0.5556f,0.6366f,  0.4475f,-0.5204f,0.7273f,  0.4424f,-0.5218f,0.7294f,  0.3478f,-0.4765f,0.8075f,  
	0.3426f,-0.4766f,0.8096f,  0.2426f,-0.4219f,0.8736f,  0.2375f,-0.4205f,0.8756f,  0.1339f,-0.3573f,0.9244f,  0.1290f,-0.3542f,0.9262f,  0.0239f,-0.2835f,0.9587f,  0.0194f,-0.2787f,0.9602f,  -0.0852f,-0.2018f,0.9757f,  -0.0891f,-0.1951f,0.9767f,  -0.1912f,-0.1134f,0.9750f,  -0.1942f,-0.1047f,0.9754f,  -0.2919f,-0.0199f,0.9562f,  -0.2938f,-0.0093f,0.9558f,  -0.3853f,0.0769f,0.9196f,  -0.3858f,0.0893f,0.9182f,  
	-0.4693f,0.1751f,0.8655f,  -0.4682f,0.1892f,0.8631f,  -0.5423f,0.2727f,0.7947f,  -0.5394f,0.2883f,0.7912f,  -0.6027f,0.3674f,0.7084f,  -0.5977f,0.3842f,0.7037f,  -0.6492f,0.4571f,0.6079f,  -0.6421f,0.4747f,0.6020f,  -0.6811f,0.5394f,0.4951f,  -0.6717f,0.5573f,0.4880f,  -0.6978f,0.6121f,0.3720f,  -0.6862f,0.6300f,0.3637f,  -0.6992f,0.6732f,0.2408f,  -0.6854f,0.6904f,0.2315f,  -0.6856f,0.7205f,0.1041f,  
	-0.6699f,0.7365f,0.0939f,  -0.6577f,0.7524f,-0.0354f,  -0.6405f,0.7666f,-0.0461f,  -0.6169f,0.7674f,-0.1748f,  -0.5984f,0.7793f,-0.1862f,  -0.5644f,0.7643f,-0.3118f,  -0.5455f,0.7734f,-0.3231f,  -0.5026f,0.7425f,-0.4428f,  -0.4836f,0.7483f,-0.4541f,  -0.4333f,0.7017f,-0.5656f,  -0.4152f,0.7039f,-0.5763f,  -0.3593f,0.6421f,-0.6772f,  -0.3429f,0.6406f,-0.6871f,  -0.2831f,0.5647f,-0.7753f,  -0.2692f,0.5593f,-0.7840f,  
	-0.2073f,0.4706f,-0.8576f,  -0.1970f,0.4615f,-0.8650f,  -0.1347f,0.3618f,-0.9225f,  -0.1287f,0.3491f,-0.9282f,  -0.0677f,0.2406f,-0.9683f,  -0.0667f,0.2248f,-0.9721f,  -0.0085f,0.1098f,-0.9939f,  -0.0133f,0.0915f,-0.9957f,  0.0410f,-0.0274f,-0.9988f,  0.0300f,-0.0475f,-0.9984f,  0.0796f,-0.1674f,-0.9827f,  0.0620f,-0.1887f,-0.9801f,  0.1063f,-0.3068f,-0.9458f,  0.0822f,-0.3284f,-0.9410f,  0.1211f,-0.4418f,-0.8889f,  
	0.0905f,-0.4627f,-0.8819f,  0.1242f,-0.5687f,-0.8131f,  0.0877f,-0.5881f,-0.8040f,  0.1167f,-0.6840f,-0.7201f,  0.0750f,-0.7013f,-0.7090f,  0.1002f,-0.7848f,-0.6117f,  0.0544f,-0.7991f,-0.5987f,  0.0770f,-0.8682f,-0.4902f,  0.0282f,-0.8791f,-0.4758f,  0.0496f,-0.9323f,-0.3584f,  -0.0007f,-0.9394f,-0.3428f,  0.0209f,-0.9755f,-0.2191f,  -0.0294f,-0.9788f,-0.2026f,  -0.0060f,-0.9971f,-0.0754f,  -0.0548f,-0.9968f,-0.0586f,  
	-0.0281f,-0.9972f,0.0693f,  -0.0740f,-0.9936f,0.0859f,  -0.0427f,-0.9764f,0.2116f,  -0.0841f,-0.9701f,0.2276f,  -0.0472f,-0.9362f,0.3483f,  -0.0831f,-0.9281f,0.3630f,  -0.0398f,-0.8785f,0.4760f,  -0.0693f,-0.8695f,0.4890f,  -0.0192f,-0.8059f,0.5917f,  -0.0417f,-0.7970f,0.6025f,  0.0151f,-0.7212f,0.6926f,  -0.0000f,-0.7135f,0.7007f,  0.0630f,-0.6273f,0.7762f,  0.0550f,-0.6217f,0.7813f,  0.6909f,-0.5881f,0.4204f,  
	0.6861f,-0.5925f,0.4221f,  0.6183f,-0.5777f,0.5329f,  0.6131f,-0.5813f,0.5350f,  0.5349f,-0.5556f,0.6366f,  0.5295f,-0.5582f,0.6388f,  0.4424f,-0.5218f,0.7294f,  0.4368f,-0.5232f,0.7317f,  0.3426f,-0.4766f,0.8096f,  0.3370f,-0.4766f,0.8119f,  0.2375f,-0.4205f,0.8756f,  0.2319f,-0.4189f,0.8779f,  0.1290f,-0.3542f,0.9262f,  0.1237f,-0.3509f,0.9282f,  0.0194f,-0.2787f,0.9602f,  0.0146f,-0.2734f,0.9618f,  
	-0.0891f,-0.1951f,0.9767f,  -0.0932f,-0.1878f,0.9778f,  -0.1942f,-0.1047f,0.9754f,  -0.1973f,-0.0954f,0.9757f,  -0.2938f,-0.0093f,0.9558f,  -0.2957f,0.0020f,0.9553f,  -0.3858f,0.0893f,0.9182f,  -0.3862f,0.1026f,0.9167f,  -0.4682f,0.1892f,0.8631f,  -0.4668f,0.2042f,0.8604f,  -0.5394f,0.2883f,0.7912f,  -0.5360f,0.3048f,0.7873f,  -0.5977f,0.3842f,0.7037f,  -0.5921f,0.4019f,0.6985f,  -0.6421f,0.4747f,0.6020f,  
	-0.6342f,0.4930f,0.5956f,  -0.6717f,0.5573f,0.4880f,  -0.6615f,0.5760f,0.4803f,  -0.6862f,0.6300f,0.3637f,  -0.6735f,0.6484f,0.3548f,  -0.6854f,0.6904f,0.2315f,  -0.6706f,0.7080f,0.2217f,  -0.6699f,0.7365f,0.0939f,  -0.6531f,0.7526f,0.0834f,  -0.6405f,0.7666f,-0.0461f,  -0.6220f,0.7809f,-0.0575f,  -0.5984f,0.7793f,-0.1862f,  -0.5790f,0.7910f,-0.1978f,  -0.5455f,0.7734f,-0.3231f,  -0.5256f,0.7821f,-0.3348f,  
	-0.4836f,0.7483f,-0.4541f,  -0.4640f,0.7536f,-0.4655f,  -0.4152f,0.7039f,-0.5763f,  -0.3968f,0.7056f,-0.5871f,  -0.3429f,0.6406f,-0.6871f,  -0.3265f,0.6384f,-0.6970f,  -0.2692f,0.5593f,-0.7840f,  -0.2558f,0.5533f,-0.7928f,  -0.1970f,0.4615f,-0.8650f,  -0.1874f,0.4517f,-0.8722f,  -0.1287f,0.3491f,-0.9282f,  -0.1238f,0.3360f,-0.9337f,  -0.0667f,0.2248f,-0.9721f,  -0.0674f,0.2089f,-0.9756f,  -0.0133f,0.0915f,-0.9957f,  
	-0.0202f,0.0733f,-0.9971f,  0.0300f,-0.0475f,-0.9984f,  0.0166f,-0.0674f,-0.9976f,  0.0620f,-0.1887f,-0.9801f,  0.0419f,-0.2093f,-0.9769f,  0.0822f,-0.3284f,-0.9410f,  0.0553f,-0.3489f,-0.9355f,  0.0905f,-0.4627f,-0.8819f,  0.0572f,-0.4822f,-0.8742f,  0.0877f,-0.5881f,-0.8040f,  0.0485f,-0.6058f,-0.7942f,  0.0750f,-0.7013f,-0.7090f,  0.0309f,-0.7163f,-0.6971f,  0.0544f,-0.7991f,-0.5987f,  0.0064f,-0.8109f,-0.5851f,  
	0.0282f,-0.8791f,-0.4758f,  -0.0223f,-0.8873f,-0.4606f,  -0.0007f,-0.9394f,-0.3428f,  -0.0523f,-0.9438f,-0.3264f,  -0.0294f,-0.9788f,-0.2026f,  -0.0806f,-0.9793f,-0.1856f,  -0.0548f,-0.9968f,-0.0586f,  -0.1040f,-0.9937f,-0.0413f,  -0.0740f,-0.9936f,0.0859f,  -0.1196f,-0.9875f,0.1030f,  -0.0841f,-0.9701f,0.2276f,  -0.1250f,-0.9617f,0.2439f,  -0.0831f,-0.9281f,0.3630f,  -0.1182f,-0.9182f,0.3780f,  -0.0693f,-0.8695f,0.4890f,  
	-0.0977f,-0.8593f,0.5021f,  -0.0417f,-0.7970f,0.6025f,  -0.0631f,-0.7874f,0.6132f,  -0.0000f,-0.7135f,0.7007f,  -0.0143f,-0.7055f,0.7086f,  0.0550f,-0.6217f,0.7813f,  0.0477f,-0.6163f,0.7861f,  0.6861f,-0.5925f,0.4221f,  0.6809f,-0.5971f,0.4242f,  0.6131f,-0.5813f,0.5350f,  0.6075f,-0.5850f,0.5373f,  0.5295f,-0.5582f,0.6388f,  0.5236f,-0.5608f,0.6414f,  0.4368f,-0.5232f,0.7317f,  0.4307f,-0.5246f,0.7344f,  
	0.3370f,-0.4766f,0.8119f,  0.3308f,-0.4765f,0.8146f,  0.2319f,-0.4189f,0.8779f,  0.2259f,-0.4171f,0.8804f,  0.1237f,-0.3509f,0.9282f,  0.1180f,-0.3471f,0.9304f,  0.0146f,-0.2734f,0.9618f,  0.0095f,-0.2676f,0.9635f,  -0.0932f,-0.1878f,0.9778f,  -0.0975f,-0.1798f,0.9789f,  -0.1973f,-0.0954f,0.9757f,  -0.2005f,-0.0853f,0.9760f,  -0.2957f,0.0020f,0.9553f,  -0.2975f,0.0142f,0.9546f,  -0.3862f,0.1026f,0.9167f,  
	-0.3863f,0.1167f,0.9149f,  -0.4668f,0.2042f,0.8604f,  -0.4650f,0.2201f,0.8575f,  -0.5360f,0.3048f,0.7873f,  -0.5320f,0.3221f,0.7831f,  -0.5921f,0.4019f,0.6985f,  -0.5858f,0.4203f,0.6930f,  -0.6342f,0.4930f,0.5956f,  -0.6254f,0.5122f,0.5887f,  -0.6615f,0.5760f,0.4803f,  -0.6501f,0.5953f,0.4721f,  -0.6735f,0.6484f,0.3548f,  -0.6598f,0.6673f,0.3456f,  -0.6706f,0.7080f,0.2217f,  -0.6546f,0.7258f,0.2115f,  
	-0.6531f,0.7526f,0.0834f,  -0.6351f,0.7691f,0.0722f,  -0.6220f,0.7809f,-0.0575f,  -0.6026f,0.7950f,-0.0692f,  -0.5790f,0.7910f,-0.1978f,  -0.5586f,0.8025f,-0.2098f,  -0.5256f,0.7821f,-0.3348f,  -0.5049f,0.7904f,-0.3468f,  -0.4640f,0.7536f,-0.4655f,  -0.4440f,0.7585f,-0.4771f,  -0.3968f,0.7056f,-0.5871f,  -0.3782f,0.7066f,-0.5980f,  -0.3265f,0.6384f,-0.6970f,  -0.3103f,0.6356f,-0.7069f,  -0.2558f,0.5533f,-0.7928f,  
	-0.2429f,0.5466f,-0.8014f,  -0.1874f,0.4517f,-0.8722f,  -0.1788f,0.4415f,-0.8793f,  -0.1238f,0.3360f,-0.9337f,  -0.1204f,0.3226f,-0.9388f,  -0.0674f,0.2089f,-0.9756f,  -0.0699f,0.1928f,-0.9787f,  -0.0202f,0.0733f,-0.9971f,  -0.0290f,0.0552f,-0.9981f,  0.0166f,-0.0674f,-0.9976f,  0.0009f,-0.0867f,-0.9962f,  0.0419f,-0.2093f,-0.9769f,  0.0191f,-0.2291f,-0.9732f,  0.0553f,-0.3489f,-0.9355f,  0.0257f,-0.3680f,-0.9295f,  
	0.0572f,-0.4822f,-0.8742f,  0.0211f,-0.5000f,-0.8658f,  0.0485f,-0.6058f,-0.7942f,  0.0068f,-0.6213f,-0.7835f,  0.0309f,-0.7163f,-0.6971f,  -0.0156f,-0.7289f,-0.6844f,  0.0064f,-0.8109f,-0.5851f,  -0.0436f,-0.8201f,-0.5706f,  -0.0223f,-0.8873f,-0.4606f,  -0.0744f,-0.8926f,-0.4446f,  -0.0523f,-0.9438f,-0.3264f,  -0.1049f,-0.9452f,-0.3093f,  -0.0806f,-0.9793f,-0.1856f,  -0.1322f,-0.9769f,-0.1678f,  -0.1040f,-0.9937f,-0.0413f,  
	-0.1530f,-0.9879f,-0.0233f,  -0.1196f,-0.9875f,0.1030f,  -0.1647f,-0.9789f,0.1206f,  -0.1250f,-0.9617f,0.2439f,  -0.1650f,-0.9513f,0.2605f,  -0.1182f,-0.9182f,0.3780f,  -0.1521f,-0.9068f,0.3932f,  -0.0977f,-0.8593f,0.5021f,  -0.1250f,-0.8479f,0.5152f,  -0.0631f,-0.7874f,0.6132f,  -0.0834f,-0.7771f,0.6238f,  -0.0143f,-0.7055f,0.7086f,  -0.0276f,-0.6972f,0.7163f,  0.0477f,-0.6163f,0.7861f,  0.0411f,-0.6109f,0.7906f,  
	0.6809f,-0.5971f,0.4242f,  0.6751f,-0.6019f,0.4266f,  0.6075f,-0.5850f,0.5373f,  0.6013f,-0.5889f,0.5400f,  0.5236f,-0.5608f,0.6414f,  0.5171f,-0.5635f,0.6442f,  0.4307f,-0.5246f,0.7344f,  0.4241f,-0.5258f,0.7373f,  0.3308f,-0.4765f,0.8146f,  0.3242f,-0.4761f,0.8174f,  0.2259f,-0.4171f,0.8804f,  0.2194f,-0.4148f,0.8830f,  0.1180f,-0.3471f,0.9304f,  0.1119f,-0.3428f,0.9327f,  0.0095f,-0.2676f,0.9635f,  
	0.0041f,-0.2612f,0.9653f,  -0.0975f,-0.1798f,0.9789f,  -0.1020f,-0.1712f,0.9799f,  -0.2005f,-0.0853f,0.9760f,  -0.2038f,-0.0745f,0.9762f,  -0.2975f,0.0142f,0.9546f,  -0.2992f,0.0272f,0.9538f,  -0.3863f,0.1167f,0.9149f,  -0.3862f,0.1317f,0.9130f,  -0.4650f,0.2201f,0.8575f,  -0.4628f,0.2369f,0.8542f,  -0.5320f,0.3221f,0.7831f,  -0.5274f,0.3403f,0.7785f,  -0.5858f,0.4203f,0.6930f,  -0.5786f,0.4396f,0.6870f,  
	-0.6254f,0.5122f,0.5887f,  -0.6156f,0.5321f,0.5814f,  -0.6501f,0.5953f,0.4721f,  -0.6377f,0.6152f,0.4636f,  -0.6598f,0.6673f,0.3456f,  -0.6448f,0.6865f,0.3360f,  -0.6546f,0.7258f,0.2115f,  -0.6372f,0.7441f,0.2006f,  -0.6351f,0.7691f,0.0722f,  -0.6159f,0.7855f,0.0607f,  -0.6026f,0.7950f,-0.0692f,  -0.5820f,0.8091f,-0.0812f,  -0.5586f,0.8025f,-0.2098f,  -0.5372f,0.8137f,-0.2221f,  -0.5049f,0.7904f,-0.3468f,  
	-0.4835f,0.7984f,-0.3589f,  -0.4440f,0.7585f,-0.4771f,  -0.4234f,0.7628f,-0.4888f,  -0.3782f,0.7066f,-0.5980f,  -0.3594f,0.7071f,-0.6090f,  -0.3103f,0.6356f,-0.7069f,  -0.2943f,0.6322f,-0.7167f,  -0.2429f,0.5466f,-0.8014f,  -0.2307f,0.5395f,-0.8098f,  -0.1788f,0.4415f,-0.8793f,  -0.1713f,0.4309f,-0.8860f,  -0.1204f,0.3226f,-0.9388f,  -0.1185f,0.3090f,-0.9436f,  -0.0699f,0.1928f,-0.9787f,  -0.0742f,0.1767f,-0.9815f,  
	-0.0290f,0.0552f,-0.9981f,  -0.0402f,0.0374f,-0.9985f,  0.0009f,-0.0867f,-0.9962f,  -0.0173f,-0.1053f,-0.9943f,  0.0191f,-0.2291f,-0.9732f,  -0.0062f,-0.2476f,-0.9689f,  0.0257f,-0.3680f,-0.9295f,  -0.0066f,-0.3856f,-0.9226f,  0.0211f,-0.5000f,-0.8658f,  -0.0174f,-0.5157f,-0.8566f,  0.0068f,-0.6213f,-0.7835f,  -0.0373f,-0.6345f,-0.7720f,  -0.0156f,-0.7289f,-0.6844f,  -0.0641f,-0.7388f,-0.6708f,  -0.0436f,-0.8201f,-0.5706f,  
	-0.0951f,-0.8262f,-0.5552f,  -0.0744f,-0.8926f,-0.4446f,  -0.1275f,-0.8949f,-0.4277f,  -0.1049f,-0.9452f,-0.3093f,  -0.1581f,-0.9435f,-0.2914f,  -0.1322f,-0.9769f,-0.1678f,  -0.1838f,-0.9716f,-0.1492f,  -0.1530f,-0.9879f,-0.0233f,  -0.2016f,-0.9794f,-0.0047f,  -0.1647f,-0.9789f,0.1206f,  -0.2090f,-0.9680f,0.1387f,  -0.1650f,-0.9513f,0.2605f,  -0.2039f,-0.9388f,0.2775f,  -0.1521f,-0.9068f,0.3932f,  -0.1848f,-0.8939f,0.4085f,  
	-0.1250f,-0.8479f,0.5152f,  -0.1510f,-0.8355f,0.5283f,  -0.0834f,-0.7771f,0.6238f,  -0.1025f,-0.7663f,0.6343f,  -0.0276f,-0.6972f,0.7163f,  -0.0400f,-0.6888f,0.7238f,  0.0411f,-0.6109f,0.7906f,  0.0351f,-0.6058f,0.7949f,  0.6751f,-0.6019f,0.4266f,  0.6688f,-0.6069f,0.4294f,  0.6013f,-0.5889f,0.5400f,  0.5947f,-0.5929f,0.5431f,  0.5171f,-0.5635f,0.6442f,  0.5101f,-0.5662f,0.6474f,  0.4241f,-0.5258f,0.7373f,  
	0.4170f,-0.5270f,0.7406f,  0.3242f,-0.4761f,0.8174f,  0.3171f,-0.4755f,0.8206f,  0.2194f,-0.4148f,0.8830f,  0.2125f,-0.4122f,0.8860f,  0.1119f,-0.3428f,0.9327f,  0.1055f,-0.3381f,0.9352f,  0.0041f,-0.2612f,0.9653f,  -0.0016f,-0.2542f,0.9672f,  -0.1020f,-0.1712f,0.9799f,  -0.1065f,-0.1619f,0.9810f,  -0.2038f,-0.0745f,0.9762f,  -0.2070f,-0.0628f,0.9763f,  -0.2992f,0.0272f,0.9538f,  -0.3007f,0.0410f,0.9528f,  
	-0.3862f,0.1317f,0.9130f,  -0.3856f,0.1476f,0.9108f,  -0.4628f,0.2369f,0.8542f,  -0.4599f,0.2545f,0.8507f,  -0.5274f,0.3403f,0.7785f,  -0.5219f,0.3594f,0.7736f,  -0.5786f,0.4396f,0.6870f,  -0.5705f,0.4597f,0.6806f,  -0.6156f,0.5321f,0.5814f,  -0.6046f,0.5525f,0.5737f,  -0.6377f,0.6152f,0.4636f,  -0.6240f,0.6355f,0.4547f,  -0.6448f,0.6865f,0.3360f,  -0.6285f,0.7064f,0.3257f,  -0.6372f,0.7441f,0.2006f,  
	-0.6187f,0.7624f,0.1895f,  -0.6159f,0.7855f,0.0607f,  -0.5954f,0.8019f,0.0488f,  -0.5820f,0.8091f,-0.0812f,  -0.5602f,0.8231f,-0.0936f,  -0.5372f,0.8137f,-0.2221f,  -0.5148f,0.8246f,-0.2345f,  -0.4835f,0.7984f,-0.3589f,  -0.4614f,0.8058f,-0.3712f,  -0.4234f,0.7628f,-0.4888f,  -0.4024f,0.7665f,-0.5006f,  -0.3594f,0.7071f,-0.6090f,  -0.3406f,0.7070f,-0.6198f,  -0.2943f,0.6322f,-0.7167f,  -0.2786f,0.6282f,-0.7265f,  
	-0.2307f,0.5395f,-0.8098f,  -0.2192f,0.5318f,-0.8180f,  -0.1713f,0.4309f,-0.8860f,  -0.1650f,0.4200f,-0.8924f,  -0.1185f,0.3090f,-0.9436f,  -0.1181f,0.2953f,-0.9481f,  -0.0742f,0.1767f,-0.9815f,  -0.0805f,0.1609f,-0.9837f,  -0.0402f,0.0374f,-0.9985f,  -0.0535f,0.0203f,-0.9984f,  -0.0173f,-0.1053f,-0.9943f,  -0.0381f,-0.1228f,-0.9917f,  -0.0062f,-0.2476f,-0.9689f,  -0.0342f,-0.2646f,-0.9637f,  -0.0066f,-0.3856f,-0.9226f,  
	-0.0413f,-0.4013f,-0.9150f,  -0.0174f,-0.5157f,-0.8566f,  -0.0584f,-0.5292f,-0.8465f,  -0.0373f,-0.6345f,-0.7720f,  -0.0835f,-0.6450f,-0.7596f,  -0.0641f,-0.7388f,-0.6708f,  -0.1143f,-0.7458f,-0.6563f,  -0.0951f,-0.8262f,-0.5552f,  -0.1479f,-0.8293f,-0.5389f,  -0.1275f,-0.8949f,-0.4277f,  -0.1814f,-0.8939f,-0.4100f,  -0.1581f,-0.9435f,-0.2914f,  -0.2115f,-0.9386f,-0.2726f,  -0.1838f,-0.9716f,-0.1492f,  -0.2351f,-0.9632f,-0.1300f,  
	-0.2016f,-0.9794f,-0.0047f,  -0.2495f,-0.9683f,0.0144f,  -0.2090f,-0.9680f,0.1387f,  -0.2522f,-0.9548f,0.1571f,  -0.2039f,-0.9388f,0.2775f,  -0.2414f,-0.9246f,0.2947f,  -0.1848f,-0.8939f,0.4085f,  -0.2160f,-0.8796f,0.4239f,  -0.1510f,-0.8355f,0.5283f,  -0.1755f,-0.8222f,0.5414f,  -0.1025f,-0.7663f,0.6343f,  -0.1204f,-0.7550f,0.6446f,  -0.0400f,-0.6888f,0.7238f,  -0.0514f,-0.6803f,0.7311f,  0.0351f,-0.6058f,0.7949f,  
	0.0296f,-0.6008f,0.7988f,  0.6688f,-0.6069f,0.4294f,  0.6620f,-0.6120f,0.4326f,  0.5947f,-0.5929f,0.5431f,  0.5874f,-0.5969f,0.5465f,  0.5101f,-0.5662f,0.6474f,  0.5026f,-0.5689f,0.6510f,  0.4170f,-0.5270f,0.7406f,  0.4093f,-0.5280f,0.7441f,  0.3171f,-0.4755f,0.8206f,  0.3094f,-0.4745f,0.8241f,  0.2125f,-0.4122f,0.8860f,  0.2051f,-0.4092f,0.8891f,  0.1055f,-0.3381f,0.9352f,  0.0987f,-0.3328f,0.9378f,  
	-0.0016f,-0.2542f,0.9672f,  -0.0075f,-0.2465f,0.9691f,  -0.1065f,-0.1619f,0.9810f,  -0.1112f,-0.1518f,0.9821f,  -0.2070f,-0.0628f,0.9763f,  -0.2101f,-0.0503f,0.9764f,  -0.3007f,0.0410f,0.9528f,  -0.3019f,0.0558f,0.9517f,  -0.3856f,0.1476f,0.9108f,  -0.3846f,0.1644f,0.9083f,  -0.4599f,0.2545f,0.8507f,  -0.4563f,0.2731f,0.8468f,  -0.5219f,0.3594f,0.7736f,  -0.5156f,0.3794f,0.7682f,  -0.5705f,0.4597f,0.6806f,  
	-0.5613f,0.4804f,0.6739f,  -0.6046f,0.5525f,0.5737f,  -0.5925f,0.5736f,0.5656f,  -0.6240f,0.6355f,0.4547f,  -0.6090f,0.6565f,0.4451f,  -0.6285f,0.7064f,0.3257f,  -0.6108f,0.7263f,0.3151f,  -0.6187f,0.7624f,0.1895f,  -0.5987f,0.7810f,0.1779f,  -0.5954f,0.8019f,0.0488f,  -0.5736f,0.8183f,0.0365f,  -0.5602f,0.8231f,-0.0936f,  -0.5372f,0.8367f,-0.1062f,  -0.5148f,0.8246f,-0.2345f,  -0.4914f,0.8351f,-0.2472f,  
	-0.4614f,0.8058f,-0.3712f,  -0.4385f,0.8128f,-0.3836f,  -0.4024f,0.7665f,-0.5006f,  -0.3811f,0.7696f,-0.5123f,  -0.3406f,0.7070f,-0.6198f,  -0.3217f,0.7062f,-0.6307f,  -0.2786f,0.6282f,-0.7265f,  -0.2634f,0.6237f,-0.7360f,  -0.2192f,0.5318f,-0.8180f,  -0.2087f,0.5238f,-0.8259f,  -0.1650f,0.4200f,-0.8924f,  -0.1599f,0.4089f,-0.8985f,  -0.1181f,0.2953f,-0.9481f,  -0.1194f,0.2817f,-0.9520f,  -0.0805f,0.1609f,-0.9837f,  
	-0.0888f,0.1455f,-0.9854f,  -0.0535f,0.0203f,-0.9984f,  -0.0692f,0.0040f,-0.9976f,  -0.0381f,-0.1228f,-0.9917f,  -0.0612f,-0.1391f,-0.9884f,  -0.0342f,-0.2646f,-0.9637f,  -0.0645f,-0.2800f,-0.9578f,  -0.0413f,-0.4013f,-0.9150f,  -0.0785f,-0.4148f,-0.9065f,  -0.0584f,-0.5292f,-0.8465f,  -0.1015f,-0.5401f,-0.8355f,  -0.0835f,-0.6450f,-0.7596f,  -0.1315f,-0.6525f,-0.7463f,  -0.1143f,-0.7458f,-0.6563f,  -0.1658f,-0.7496f,-0.6408f,  
	-0.1479f,-0.8293f,-0.5389f,  -0.2016f,-0.8290f,-0.5216f,  -0.1814f,-0.8939f,-0.4100f,  -0.2355f,-0.8896f,-0.3913f,  -0.2115f,-0.9386f,-0.2726f,  -0.2646f,-0.9306f,-0.2530f,  -0.2351f,-0.9632f,-0.1300f,  -0.2857f,-0.9520f,-0.1101f,  -0.2495f,-0.9683f,0.0144f,  -0.2961f,-0.9545f,0.0341f,  -0.2522f,-0.9548f,0.1571f,  -0.2939f,-0.9395f,0.1760f,  -0.2414f,-0.9246f,0.2947f,  -0.2773f,-0.9087f,0.3122f,  -0.2160f,-0.8796f,0.4239f,  
	-0.2456f,-0.8641f,0.4393f,  -0.1755f,-0.8222f,0.5414f,  -0.1986f,-0.8082f,0.5545f,  -0.1204f,-0.7550f,0.6446f,  -0.1370f,-0.7433f,0.6548f,  -0.0514f,-0.6803f,0.7311f,  -0.0619f,-0.6718f,0.7381f,  0.0296f,-0.6008f,0.7988f,  0.0248f,-0.5961f,0.8026f,  0.6620f,-0.6120f,0.4326f,  0.6547f,-0.6174f,0.4362f,  0.5874f,-0.5969f,0.5465f,  0.5796f,-0.6010f,0.5503f,  0.5026f,-0.5689f,0.6510f,  0.4945f,-0.5714f,0.6550f,  
	0.4093f,-0.5280f,0.7441f,  0.4010f,-0.5287f,0.7481f,  0.3094f,-0.4745f,0.8241f,  0.3013f,-0.4733f,0.8278f,  0.2051f,-0.4092f,0.8891f,  0.1974f,-0.4057f,0.8925f,  0.0987f,-0.3328f,0.9378f,  0.0917f,-0.3269f,0.9406f,  -0.0075f,-0.2465f,0.9691f,  -0.0135f,-0.2381f,0.9712f,  -0.1112f,-0.1518f,0.9821f,  -0.1159f,-0.1408f,0.9832f,  -0.2101f,-0.0503f,0.9764f,  -0.2130f,-0.0369f,0.9764f,  -0.3019f,0.0558f,0.9517f,  
	-0.3027f,0.0715f,0.9504f,  -0.3846f,0.1644f,0.9083f,  -0.3829f,0.1822f,0.9056f,  -0.4563f,0.2731f,0.8468f,  -0.4520f,0.2926f,0.8427f,  -0.5156f,0.3794f,0.7682f,  -0.5083f,0.4001f,0.7626f,  -0.5613f,0.4804f,0.6739f,  -0.5509f,0.5018f,0.6668f,  -0.5925f,0.5736f,0.5656f,  -0.5790f,0.5955f,0.5569f,  -0.6090f,0.6565f,0.4451f,  -0.5925f,0.6778f,0.4353f,  -0.6108f,0.7263f,0.3151f,  -0.5917f,0.7466f,0.3041f,  
	-0.5987f,0.7810f,0.1779f,  -0.5773f,0.7995f,0.1660f,  -0.5736f,0.8183f,0.0365f,  -0.5505f,0.8345f,0.0241f,  -0.5372f,0.8367f,-0.1062f,  -0.5131f,0.8501f,-0.1190f,  -0.4914f,0.8351f,-0.2472f,  -0.4671f,0.8451f,-0.2600f,  -0.4385f,0.8128f,-0.3836f,  -0.4150f,0.8191f,-0.3960f,  -0.3811f,0.7696f,-0.5123f,  -0.3594f,0.7721f,-0.5241f,  -0.3217f,0.7062f,-0.6307f,  -0.3031f,0.7049f,-0.6413f,  -0.2634f,0.6237f,-0.7360f,  
	-0.2488f,0.6187f,-0.7452f,  -0.2087f,0.5238f,-0.8259f,  -0.1990f,0.5154f,-0.8335f,  -0.1599f,0.4089f,-0.8985f,  -0.1563f,0.3976f,-0.9041f,  -0.1194f,0.2817f,-0.9520f,  -0.1225f,0.2683f,-0.9555f,  -0.0888f,0.1455f,-0.9854f,  -0.0992f,0.1308f,-0.9864f,  -0.0692f,0.0040f,-0.9976f,  -0.0871f,-0.0112f,-0.9961f,  -0.0612f,-0.1391f,-0.9884f,  -0.0866f,-0.1539f,-0.9843f,  -0.0645f,-0.2800f,-0.9578f,  -0.0972f,-0.2934f,-0.9510f,  
	-0.0785f,-0.4148f,-0.9065f,  -0.1178f,-0.4259f,-0.8971f,  -0.1015f,-0.5401f,-0.8355f,  -0.1464f,-0.5482f,-0.8235f,  -0.1315f,-0.6525f,-0.7463f,  -0.1809f,-0.6570f,-0.7318f,  -0.1658f,-0.7496f,-0.6408f,  -0.2183f,-0.7500f,-0.6243f,  -0.2016f,-0.8290f,-0.5216f,  -0.2556f,-0.8254f,-0.5033f,  -0.2355f,-0.8896f,-0.3913f,  -0.2895f,-0.8820f,-0.3717f,  -0.2646f,-0.9306f,-0.2530f,  -0.3170f,-0.9194f,-0.2327f,  -0.2857f,-0.9520f,-0.1101f,  
	-0.3351f,-0.9379f,-0.0895f,  -0.2961f,-0.9545f,0.0341f,  -0.3413f,-0.9384f,0.0542f,  -0.2939f,-0.9395f,0.1760f,  -0.3339f,-0.9222f,0.1951f,  -0.2773f,-0.9087f,0.3122f,  -0.3115f,-0.8912f,0.3297f,  -0.2456f,-0.8641f,0.4393f,  -0.2735f,-0.8476f,0.4548f,  -0.1986f,-0.8082f,0.5545f,  -0.2202f,-0.7935f,0.5674f,  -0.1370f,-0.7433f,0.6548f,  -0.1524f,-0.7313f,0.6648f,  -0.0619f,-0.6718f,0.7381f,  -0.0715f,-0.6633f,0.7450f,  
	0.0248f,-0.5961f,0.8026f,  0.0205f,-0.5915f,0.8060f,  0.6547f,-0.6174f,0.4362f,  0.6467f,-0.6229f,0.4402f,  0.5796f,-0.6010f,0.5503f,  0.5712f,-0.6051f,0.5546f,  0.4945f,-0.5714f,0.6550f,  0.4858f,-0.5739f,0.6593f,  0.4010f,-0.5287f,0.7481f,  0.3922f,-0.5293f,0.7523f,  0.3013f,-0.4733f,0.8278f,  0.2927f,-0.4716f,0.8318f,  0.1974f,-0.4057f,0.8925f,  0.1893f,-0.4017f,0.8960f,  0.0917f,-0.3269f,0.9406f,  
	0.0844f,-0.3203f,0.9435f,  -0.0135f,-0.2381f,0.9712f,  -0.0197f,-0.2289f,0.9732f,  -0.1159f,-0.1408f,0.9832f,  -0.1204f,-0.1291f,0.9843f,  -0.2130f,-0.0369f,0.9764f,  -0.2156f,-0.0227f,0.9762f,  -0.3027f,0.0715f,0.9504f,  -0.3030f,0.0881f,0.9489f,  -0.3829f,0.1822f,0.9056f,  -0.3806f,0.2009f,0.9027f,  -0.4520f,0.2926f,0.8427f,  -0.4466f,0.3129f,0.8382f,  -0.5083f,0.4001f,0.7626f,  -0.4998f,0.4215f,0.7566f,  
	-0.5509f,0.5018f,0.6668f,  -0.5391f,0.5242f,0.6592f,  -0.5790f,0.5955f,0.5569f,  -0.5641f,0.6177f,0.5480f,  -0.5925f,0.6778f,0.4353f,  -0.5746f,0.6995f,0.4250f,  -0.5917f,0.7466f,0.3041f,  -0.5710f,0.7670f,0.2927f,  -0.5773f,0.7995f,0.1660f,  -0.5543f,0.8180f,0.1539f,  -0.5505f,0.8345f,0.0241f,  -0.5259f,0.8505f,0.0113f,  -0.5131f,0.8501f,-0.1190f,  -0.4877f,0.8630f,-0.1320f,  -0.4671f,0.8451f,-0.2600f,  
	-0.4418f,0.8546f,-0.2729f,  -0.4150f,0.8191f,-0.3960f,  -0.3909f,0.8248f,-0.4085f,  -0.3594f,0.7721f,-0.5241f,  -0.3375f,0.7740f,-0.5357f,  -0.3031f,0.7049f,-0.6413f,  -0.2846f,0.7030f,-0.6518f,  -0.2488f,0.6187f,-0.7452f,  -0.2347f,0.6132f,-0.7542f,  -0.1990f,0.5154f,-0.8335f,  -0.1905f,0.5068f,-0.8407f,  -0.1563f,0.3976f,-0.9041f,  -0.1541f,0.3865f,-0.9093f,  -0.1225f,0.2683f,-0.9555f,  -0.1274f,0.2553f,-0.9584f,  
	-0.0992f,0.1308f,-0.9864f,  -0.1116f,0.1169f,-0.9869f,  -0.0871f,-0.0112f,-0.9961f,  -0.1072f,-0.0252f,-0.9939f,  -0.0866f,-0.1539f,-0.9843f,  -0.1143f,-0.1669f,-0.9793f,  -0.0972f,-0.2934f,-0.9510f,  -0.1320f,-0.3045f,-0.9433f,  -0.1178f,-0.4259f,-0.8971f,  -0.1589f,-0.4344f,-0.8866f,  -0.1464f,-0.5482f,-0.8235f,  -0.1929f,-0.5533f,-0.8104f,  -0.1809f,-0.6570f,-0.7318f,  -0.2313f,-0.6582f,-0.7164f,  -0.2183f,-0.7500f,-0.6243f,  
	-0.2713f,-0.7472f,-0.6068f,  -0.2556f,-0.8254f,-0.5033f,  -0.3095f,-0.8185f,-0.4841f,  -0.2895f,-0.8820f,-0.3717f,  -0.3429f,-0.8712f,-0.3513f,  -0.3170f,-0.9194f,-0.2327f,  -0.3683f,-0.9053f,-0.2116f,  -0.3351f,-0.9379f,-0.0895f,  -0.3830f,-0.9212f,-0.0684f,  -0.3413f,-0.9384f,0.0542f,  -0.3847f,-0.9200f,0.0748f,  -0.3339f,-0.9222f,0.1951f,  -0.3720f,-0.9031f,0.2145f,  -0.3115f,-0.8912f,0.3297f,  -0.3437f,-0.8725f,0.3474f,  
	-0.2735f,-0.8476f,0.4548f,  -0.2996f,-0.8301f,0.4702f,  -0.2202f,-0.7935f,0.5674f,  -0.2402f,-0.7783f,0.5802f,  -0.1524f,-0.7313f,0.6648f,  -0.1665f,-0.7191f,0.6746f,  -0.0715f,-0.6633f,0.7450f,  -0.0801f,-0.6548f,0.7515f,  0.0205f,-0.5915f,0.8060f,  0.0167f,-0.5872f,0.8093f,  0.6467f,-0.6229f,0.4402f,  0.6382f,-0.6285f,0.4447f,  0.5712f,-0.6051f,0.5546f,  0.5622f,-0.6092f,0.5592f,  0.4858f,-0.5739f,0.6593f,  
	0.4765f,-0.5762f,0.6640f,  0.3922f,-0.5293f,0.7523f,  0.3829f,-0.5295f,0.7569f,  0.2927f,-0.4716f,0.8318f,  0.2836f,-0.4696f,0.8361f,  0.1893f,-0.4017f,0.8960f,  0.1808f,-0.3971f,0.8998f,  0.0844f,-0.3203f,0.9435f,  0.0768f,-0.3131f,0.9466f,  -0.0197f,-0.2289f,0.9732f,  -0.0258f,-0.2190f,0.9754f,  -0.1204f,-0.1291f,0.9843f,  -0.1248f,-0.1165f,0.9853f,  -0.2156f,-0.0227f,0.9762f,  -0.2178f,-0.0075f,0.9760f,  
	-0.3030f,0.0881f,0.9489f,  -0.3026f,0.1057f,0.9472f,  -0.3806f,0.2009f,0.9027f,  -0.3773f,0.2204f,0.8995f,  -0.4466f,0.3129f,0.8382f,  -0.4402f,0.3340f,0.8335f,  -0.4998f,0.4215f,0.7566f,  -0.4900f,0.4439f,0.7502f,  -0.5391f,0.5242f,0.6592f,  -0.5260f,0.5469f,0.6513f,  -0.5641f,0.6177f,0.5480f,  -0.5476f,0.6404f,0.5386f,  -0.5746f,0.6995f,0.4250f,  -0.5549f,0.7214f,0.4143f,  -0.5710f,0.7670f,0.2927f,  
	-0.5486f,0.7874f,0.2810f,  -0.5543f,0.8180f,0.1539f,  -0.5297f,0.8363f,0.1414f,  -0.5259f,0.8505f,0.0113f,  -0.4999f,0.8661f,-0.0017f,  -0.4877f,0.8630f,-0.1320f,  -0.4611f,0.8754f,-0.1451f,  -0.4418f,0.8546f,-0.2729f,  -0.4156f,0.8634f,-0.2859f,  -0.3909f,0.8248f,-0.4085f,  -0.3662f,0.8299f,-0.4209f,  -0.3375f,0.7740f,-0.5357f,  -0.3156f,0.7753f,-0.5472f,  -0.2846f,0.7030f,-0.6518f,  -0.2664f,0.7005f,-0.6621f,  
	-0.2347f,0.6132f,-0.7542f,  -0.2214f,0.6073f,-0.7630f,  -0.1905f,0.5068f,-0.8407f,  -0.1831f,0.4981f,-0.8476f,  -0.1541f,0.3865f,-0.9093f,  -0.1535f,0.3755f,-0.9140f,  -0.1274f,0.2553f,-0.9584f,  -0.1341f,0.2430f,-0.9607f,  -0.1116f,0.1169f,-0.9869f,  -0.1260f,0.1040f,-0.9866f,  -0.1072f,-0.0252f,-0.9939f,  -0.1294f,-0.0376f,-0.9909f,  -0.1143f,-0.1669f,-0.9793f,  -0.1440f,-0.1779f,-0.9734f,  -0.1320f,-0.3045f,-0.9433f,  
	-0.1686f,-0.3133f,-0.9346f,  -0.1589f,-0.4344f,-0.8866f,  -0.2015f,-0.4401f,-0.8751f,  -0.1929f,-0.5533f,-0.8104f,  -0.2404f,-0.5552f,-0.7962f,  -0.2313f,-0.6582f,-0.7164f,  -0.2823f,-0.6561f,-0.6998f,  -0.2713f,-0.7472f,-0.6068f,  -0.3242f,-0.7409f,-0.5882f,  -0.3095f,-0.8185f,-0.4841f,  -0.3629f,-0.8081f,-0.4639f,  -0.3429f,-0.8712f,-0.3513f,  -0.3952f,-0.8573f,-0.3301f,  -0.3683f,-0.9053f,-0.2116f,  -0.4180f,-0.8884f,-0.1899f,  
	-0.3830f,-0.9212f,-0.0684f,  -0.4290f,-0.9021f,-0.0468f,  -0.3847f,-0.9200f,0.0748f,  -0.4261f,-0.8996f,0.0957f,  -0.3720f,-0.9031f,0.2145f,  -0.4080f,-0.8825f,0.2341f,  -0.3437f,-0.8725f,0.3474f,  -0.3739f,-0.8526f,0.3651f,  -0.2996f,-0.8301f,0.4702f,  -0.3239f,-0.8120f,0.4856f,  -0.2402f,-0.7783f,0.5802f,  -0.2586f,-0.7627f,0.5928f,  -0.1665f,-0.7191f,0.6746f,  -0.1794f,-0.7069f,0.6842f,  -0.0801f,-0.6548f,0.7515f,  
	-0.0879f,-0.6464f,0.7579f,  0.0167f,-0.5872f,0.8093f,  0.0134f,-0.5831f,0.8123f,  0.6382f,-0.6285f,0.4447f,  0.6290f,-0.6342f,0.4496f,  0.5622f,-0.6092f,0.5592f,  0.5526f,-0.6133f,0.5643f,  0.4765f,-0.5762f,0.6640f,  0.4667f,-0.5784f,0.6691f,  0.3829f,-0.5295f,0.7569f,  0.3731f,-0.5295f,0.7619f,  0.2836f,-0.4696f,0.8361f,  0.2741f,-0.4670f,0.8407f,  0.1808f,-0.3971f,0.8998f,  0.1720f,-0.3919f,0.9038f,  
	0.0768f,-0.3131f,0.9466f,  0.0692f,-0.3051f,0.9498f,  -0.0258f,-0.2190f,0.9754f,  -0.0319f,-0.2082f,0.9776f,  -0.1248f,-0.1165f,0.9853f,  -0.1289f,-0.1029f,0.9863f,  -0.2178f,-0.0075f,0.9760f,  -0.2195f,0.0087f,0.9756f,  -0.3026f,0.1057f,0.9472f,  -0.3015f,0.1241f,0.9453f,  -0.3773f,0.2204f,0.8995f,  -0.3730f,0.2407f,0.8960f,  -0.4402f,0.3340f,0.8335f,  -0.4326f,0.3561f,0.8283f,  -0.4900f,0.4439f,0.7502f,  
	-0.4789f,0.4668f,0.7435f,  -0.5260f,0.5469f,0.6513f,  -0.5112f,0.5703f,0.6430f,  -0.5476f,0.6404f,0.5386f,  -0.5294f,0.6634f,0.5289f,  -0.5549f,0.7214f,0.4143f,  -0.5336f,0.7434f,0.4034f,  -0.5486f,0.7874f,0.2810f,  -0.5245f,0.8078f,0.2690f,  -0.5297f,0.8363f,0.1414f,  -0.5036f,0.8543f,0.1287f,  -0.4999f,0.8661f,-0.0017f,  -0.4725f,0.8812f,-0.0148f,  -0.4611f,0.8754f,-0.1451f,  -0.4333f,0.8872f,-0.1584f,  
	-0.4156f,0.8634f,-0.2859f,  -0.3886f,0.8716f,-0.2989f,  -0.3662f,0.8299f,-0.4209f,  -0.3411f,0.8343f,-0.4332f,  -0.3156f,0.7753f,-0.5472f,  -0.2935f,0.7758f,-0.5586f,  -0.2664f,0.7005f,-0.6621f,  -0.2486f,0.6975f,-0.6721f,  -0.2214f,0.6073f,-0.7630f,  -0.2089f,0.6012f,-0.7713f,  -0.1831f,0.4981f,-0.8476f,  -0.1769f,0.4893f,-0.8540f,  -0.1535f,0.3755f,-0.9140f,  -0.1544f,0.3650f,-0.9181f,  -0.1341f,0.2430f,-0.9607f,  
	-0.1426f,0.2314f,-0.9624f,  -0.1260f,0.1040f,-0.9866f,  -0.1423f,0.0924f,-0.9855f,  -0.1294f,-0.0376f,-0.9909f,  -0.1535f,-0.0483f,-0.9870f,  -0.1440f,-0.1779f,-0.9734f,  -0.1755f,-0.1867f,-0.9666f,  -0.1686f,-0.3133f,-0.9346f,  -0.2068f,-0.3193f,-0.9248f,  -0.2015f,-0.4401f,-0.8751f,  -0.2453f,-0.4427f,-0.8625f,  -0.2404f,-0.5552f,-0.7962f,  -0.2886f,-0.5539f,-0.7810f,  -0.2823f,-0.6561f,-0.6998f,  -0.3334f,-0.6507f,-0.6822f,  
	-0.3242f,-0.7409f,-0.5882f,  -0.3768f,-0.7312f,-0.5686f,  -0.3629f,-0.8081f,-0.4639f,  -0.4153f,-0.7946f,-0.4429f,  -0.3952f,-0.8573f,-0.3301f,  -0.4460f,-0.8403f,-0.3081f,  -0.4180f,-0.8884f,-0.1899f,  -0.4660f,-0.8688f,-0.1676f,  -0.4290f,-0.9021f,-0.0468f,  -0.4730f,-0.8807f,-0.0248f,  -0.4261f,-0.8996f,0.0957f,  -0.4653f,-0.8774f,0.1167f,  -0.4080f,-0.8825f,0.2341f,  -0.4418f,-0.8605f,0.2537f,  -0.3739f,-0.8526f,0.3651f,  
	-0.4021f,-0.8318f,0.3827f,  -0.3239f,-0.8120f,0.4856f,  -0.3463f,-0.7933f,0.5008f,  -0.2586f,-0.7627f,0.5928f,  -0.2755f,-0.7469f,0.6052f,  -0.1794f,-0.7069f,0.6842f,  -0.1910f,-0.6946f,0.6935f,  -0.0879f,-0.6464f,0.7579f,  -0.0949f,-0.6382f,0.7640f,  0.0134f,-0.5831f,0.8123f,  0.0106f,-0.5793f,0.8151f,  0.6290f,-0.6342f,0.4496f,  0.6193f,-0.6399f,0.4550f,  0.5526f,-0.6133f,0.5643f,  0.5424f,-0.6173f,0.5699f,  
	0.4667f,-0.5784f,0.6691f,  0.4562f,-0.5803f,0.6746f,  0.3731f,-0.5295f,0.7619f,  0.3628f,-0.5290f,0.7671f,  0.2741f,-0.4670f,0.8407f,  0.2643f,-0.4640f,0.8455f,  0.1720f,-0.3919f,0.9038f,  0.1630f,-0.3861f,0.9080f,  0.0692f,-0.3051f,0.9498f,  0.0615f,-0.2964f,0.9531f,  -0.0319f,-0.2082f,0.9776f,  -0.0379f,-0.1966f,0.9798f,  -0.1289f,-0.1029f,0.9863f,  -0.1327f,-0.0884f,0.9872f,  -0.2195f,0.0087f,0.9756f,  
	-0.2206f,0.0257f,0.9750f,  -0.3015f,0.1241f,0.9453f,  -0.2995f,0.1433f,0.9433f,  -0.3730f,0.2407f,0.8960f,  -0.3677f,0.2622f,0.8922f,  -0.4326f,0.3561f,0.8283f,  -0.4236f,0.3787f,0.8229f,  -0.4789f,0.4668f,0.7435f,  -0.4661f,0.4904f,0.7363f,  -0.5112f,0.5703f,0.6430f,  -0.4948f,0.5940f,0.6343f,  -0.5294f,0.6634f,0.5289f,  -0.5093f,0.6866f,0.5188f,  -0.5336f,0.7434f,0.4034f,  -0.5103f,0.7654f,0.3921f,  
	-0.5245f,0.8078f,0.2690f,  -0.4986f,0.8279f,0.2568f,  -0.5036f,0.8543f,0.1287f,  -0.4757f,0.8719f,0.1157f,  -0.4725f,0.8812f,-0.0148f,  -0.4435f,0.8958f,-0.0282f,  -0.4333f,0.8872f,-0.1584f,  -0.4044f,0.8983f,-0.1717f,  -0.3886f,0.8716f,-0.2989f,  -0.3608f,0.8790f,-0.3118f,  -0.3411f,0.8343f,-0.4332f,  -0.3155f,0.8379f,-0.4455f,  -0.2935f,0.7758f,-0.5586f,  -0.2714f,0.7757f,-0.5697f,  -0.2486f,0.6975f,-0.6721f,  
	-0.2311f,0.6940f,-0.6819f,  -0.2089f,0.6012f,-0.7713f,  -0.1973f,0.5948f,-0.7793f,  -0.1769f,0.4893f,-0.8540f,  -0.1719f,0.4807f,-0.8598f,  -0.1544f,0.3650f,-0.9181f,  -0.1567f,0.3549f,-0.9217f,  -0.1426f,0.2314f,-0.9624f,  -0.1528f,0.2208f,-0.9633f,  -0.1423f,0.0924f,-0.9855f,  -0.1604f,0.0822f,-0.9836f,  -0.1535f,-0.0483f,-0.9870f,  -0.1793f,-0.0570f,-0.9821f,  -0.1755f,-0.1867f,-0.9666f,  -0.2085f,-0.1931f,-0.9588f,  
	-0.2068f,-0.3193f,-0.9248f,  -0.2461f,-0.3226f,-0.9140f,  -0.2453f,-0.4427f,-0.8625f,  -0.2899f,-0.4422f,-0.8488f,  -0.2886f,-0.5539f,-0.7810f,  -0.3369f,-0.5493f,-0.7647f,  -0.3334f,-0.6507f,-0.6822f,  -0.3842f,-0.6418f,-0.6636f,  -0.3768f,-0.7312f,-0.5686f,  -0.4284f,-0.7183f,-0.5482f,  -0.4153f,-0.7946f,-0.4429f,  -0.4664f,-0.7779f,-0.4211f,  -0.4460f,-0.8403f,-0.3081f,  -0.4950f,-0.8206f,-0.2856f,  -0.4660f,-0.8688f,-0.1676f,  
	-0.5118f,-0.8468f,-0.1449f,  -0.4730f,-0.8807f,-0.0248f,  -0.5147f,-0.8574f,-0.0026f,  -0.4653f,-0.8774f,0.1167f,  -0.5022f,-0.8537f,0.1380f,  -0.4418f,-0.8605f,0.2537f,  -0.4734f,-0.8374f,0.2733f,  -0.4021f,-0.8318f,0.3827f,  -0.4281f,-0.8103f,0.4002f,  -0.3463f,-0.7933f,0.5008f,  -0.3669f,-0.7742f,0.5158f,  -0.2755f,-0.7469f,0.6052f,  -0.2909f,-0.7309f,0.6173f,  -0.1910f,-0.6946f,0.6935f,  -0.2016f,-0.6824f,0.7026f,  
	-0.0949f,-0.6382f,0.7640f,  -0.1011f,-0.6302f,0.7698f,  0.0106f,-0.5793f,0.8151f,  0.0081f,-0.5757f,0.8176f,  0.6193f,-0.6399f,0.4550f,  0.6088f,-0.6457f,0.4609f,  0.5424f,-0.6173f,0.5699f,  0.5315f,-0.6212f,0.5758f,  0.4562f,-0.5803f,0.6746f,  0.4453f,-0.5819f,0.6805f,  0.3628f,-0.5290f,0.7671f,  0.3520f,-0.5282f,0.7727f,  0.2643f,-0.4640f,0.8455f,  0.2541f,-0.4604f,0.8506f,  0.1630f,-0.3861f,0.9080f,  
	0.1539f,-0.3796f,0.9123f,  0.0615f,-0.2964f,0.9531f,  0.0538f,-0.2869f,0.9564f,  -0.0379f,-0.1966f,0.9798f,  -0.0436f,-0.1841f,0.9819f,  -0.1327f,-0.0884f,0.9872f,  -0.1359f,-0.0731f,0.9880f,  -0.2206f,0.0257f,0.9750f,  -0.2208f,0.0435f,0.9743f,  -0.2995f,0.1433f,0.9433f,  -0.2965f,0.1637f,0.9409f,  -0.3677f,0.2622f,0.8922f,  -0.3610f,0.2842f,0.8882f,  -0.4236f,0.3787f,0.8229f,  -0.4131f,0.4022f,0.8171f,  
	-0.4661f,0.4904f,0.7363f,  -0.4517f,0.5145f,0.7289f,  -0.4948f,0.5940f,0.6343f,  -0.4764f,0.6180f,0.6254f,  -0.5093f,0.6866f,0.5188f,  -0.4874f,0.7099f,0.5084f,  -0.5103f,0.7654f,0.3921f,  -0.4851f,0.7873f,0.3805f,  -0.4986f,0.8279f,0.2568f,  -0.4709f,0.8477f,0.2443f,  -0.4757f,0.8719f,0.1157f,  -0.4462f,0.8891f,0.1025f,  -0.4435f,0.8958f,-0.0282f,  -0.4132f,0.9097f,-0.0416f,  -0.4044f,0.8983f,-0.1717f,  
	-0.3743f,0.9087f,-0.1850f,  -0.3608f,0.8790f,-0.3118f,  -0.3321f,0.8856f,-0.3247f,  -0.3155f,0.8379f,-0.4455f,  -0.2896f,0.8407f,-0.4576f,  -0.2714f,0.7757f,-0.5697f,  -0.2494f,0.7750f,-0.5807f,  -0.2311f,0.6940f,-0.6819f,  -0.2142f,0.6901f,-0.6913f,  -0.1973f,0.5948f,-0.7793f,  -0.1865f,0.5883f,-0.7868f,  -0.1719f,0.4807f,-0.8598f,  -0.1681f,0.4723f,-0.8652f,  -0.1567f,0.3549f,-0.9217f,  -0.1606f,0.3455f,-0.9246f,  
	-0.1528f,0.2208f,-0.9633f,  -0.1645f,0.2114f,-0.9635f,  -0.1604f,0.0822f,-0.9836f,  -0.1801f,0.0737f,-0.9809f,  -0.1793f,-0.0570f,-0.9821f,  -0.2066f,-0.0636f,-0.9764f,  -0.2085f,-0.1931f,-0.9588f,  -0.2427f,-0.1970f,-0.9499f,  -0.2461f,-0.3226f,-0.9140f,  -0.2862f,-0.3230f,-0.9021f,  -0.2899f,-0.4422f,-0.8488f,  -0.3347f,-0.4386f,-0.8340f,  -0.3369f,-0.5493f,-0.7647f,  -0.3851f,-0.5415f,-0.7473f,  -0.3842f,-0.6418f,-0.6636f,  
	-0.4342f,-0.6298f,-0.6441f,  -0.4284f,-0.7183f,-0.5482f,  -0.4788f,-0.7023f,-0.5269f,  -0.4664f,-0.7779f,-0.4211f,  -0.5156f,-0.7585f,-0.3986f,  -0.4950f,-0.8206f,-0.2856f,  -0.5419f,-0.7984f,-0.2625f,  -0.5118f,-0.8468f,-0.1449f,  -0.5553f,-0.8227f,-0.1218f,  -0.5147f,-0.8574f,-0.0026f,  -0.5539f,-0.8323f,0.0199f,  -0.5022f,-0.8537f,0.1380f,  -0.5366f,-0.8287f,0.1593f,  -0.4734f,-0.8374f,0.2733f,  -0.5026f,-0.8134f,0.2929f,  
	-0.4281f,-0.8103f,0.4002f,  -0.4521f,-0.7882f,0.4176f,  -0.3669f,-0.7742f,0.5158f,  -0.3857f,-0.7548f,0.5306f,  -0.2909f,-0.7309f,0.6173f,  -0.3048f,-0.7149f,0.6293f,  -0.2016f,-0.6824f,0.7026f,  -0.2110f,-0.6703f,0.7115f,  -0.1011f,-0.6302f,0.7698f,  -0.1066f,-0.6224f,0.7754f,  0.0081f,-0.5757f,0.8176f,  0.0061f,-0.5724f,0.8199f,  0.6088f,-0.6457f,0.4609f,  0.5978f,-0.6515f,0.4672f,  0.5315f,-0.6212f,0.5758f,  
	0.5201f,-0.6249f,0.5822f,  0.4453f,-0.5819f,0.6805f,  0.4337f,-0.5833f,0.6868f,  0.3520f,-0.5282f,0.7727f,  0.3408f,-0.5268f,0.7787f,  0.2541f,-0.4604f,0.8506f,  0.2436f,-0.4562f,0.8559f,  0.1539f,-0.3796f,0.9123f,  0.1446f,-0.3723f,0.9168f,  0.0538f,-0.2869f,0.9564f,  0.0463f,-0.2765f,0.9599f,  -0.0436f,-0.1841f,0.9819f,  -0.0488f,-0.1707f,0.9841f,  -0.1359f,-0.0731f,0.9880f,  -0.1384f,-0.0570f,0.9887f,  
	-0.2208f,0.0435f,0.9743f,  -0.2202f,0.0625f,0.9734f,  -0.2965f,0.1637f,0.9409f,  -0.2923f,0.1848f,0.9383f,  -0.3610f,0.2842f,0.8882f,  -0.3529f,0.3070f,0.8839f,  -0.4131f,0.4022f,0.8171f,  -0.4008f,0.4261f,0.8110f,  -0.4517f,0.5145f,0.7289f,  -0.4353f,0.5389f,0.7212f,  -0.4764f,0.6180f,0.6254f,  -0.4561f,0.6423f,0.6160f,  -0.4874f,0.7099f,0.5084f,  -0.4633f,0.7332f,0.4977f,  -0.4851f,0.7873f,0.3805f,  
	-0.4579f,0.8089f,0.3687f,  -0.4709f,0.8477f,0.2443f,  -0.4412f,0.8671f,0.2315f,  -0.4462f,0.8891f,0.1025f,  -0.4149f,0.9055f,0.0893f,  -0.4132f,0.9097f,-0.0416f,  -0.3813f,0.9228f,-0.0550f,  -0.3743f,0.9087f,-0.1850f,  -0.3430f,0.9181f,-0.1984f,  -0.3321f,0.8856f,-0.3247f,  -0.3027f,0.8913f,-0.3376f,  -0.2896f,0.8407f,-0.4576f,  -0.2633f,0.8427f,-0.4696f,  -0.2494f,0.7750f,-0.5807f,  -0.2275f,0.7736f,-0.5914f,  
	-0.2142f,0.6901f,-0.6913f,  -0.1979f,0.6858f,-0.7004f,  -0.1865f,0.5883f,-0.7868f,  -0.1767f,0.5817f,-0.7940f,  -0.1681f,0.4723f,-0.8652f,  -0.1656f,0.4643f,-0.8701f,  -0.1606f,0.3455f,-0.9246f,  -0.1658f,0.3369f,-0.9268f,  -0.1645f,0.2114f,-0.9635f,  -0.1778f,0.2032f,-0.9629f,  -0.1801f,0.0737f,-0.9809f,  -0.2012f,0.0670f,-0.9773f,  -0.2066f,-0.0636f,-0.9764f,  -0.2351f,-0.0680f,-0.9696f,  -0.2427f,-0.1970f,-0.9499f,  
	-0.2777f,-0.1982f,-0.9400f,  -0.2862f,-0.3230f,-0.9021f,  -0.3267f,-0.3204f,-0.8891f,  -0.3347f,-0.4386f,-0.8340f,  -0.3794f,-0.4319f,-0.8182f,  -0.3851f,-0.5415f,-0.7473f,  -0.4326f,-0.5305f,-0.7290f,  -0.4342f,-0.6298f,-0.6441f,  -0.4830f,-0.6146f,-0.6237f,  -0.4788f,-0.7023f,-0.5269f,  -0.5274f,-0.6833f,-0.5049f,  -0.5156f,-0.7585f,-0.3986f,  -0.5628f,-0.7364f,-0.3756f,  -0.5419f,-0.7984f,-0.2625f,  -0.5864f,-0.7739f,-0.2390f,  
	-0.5553f,-0.8227f,-0.1218f,  -0.5963f,-0.7967f,-0.0985f,  -0.5539f,-0.8323f,0.0199f,  -0.5906f,-0.8058f,0.0425f,  -0.5366f,-0.8287f,0.1593f,  -0.5685f,-0.8027f,0.1805f,  -0.5026f,-0.8134f,0.2929f,  -0.5295f,-0.7887f,0.3123f,  -0.4521f,-0.7882f,0.4176f,  -0.4740f,-0.7657f,0.4348f,  -0.3857f,-0.7548f,0.5306f,  -0.4028f,-0.7352f,0.5451f,  -0.3048f,-0.7149f,0.6293f,  -0.3173f,-0.6990f,0.6409f,  -0.2110f,-0.6703f,0.7115f,  
	-0.2194f,-0.6583f,0.7200f,  -0.1066f,-0.6224f,0.7754f,  -0.1113f,-0.6148f,0.7808f,  0.0061f,-0.5724f,0.8199f,  0.0043f,-0.5694f,0.8220f,  0.5978f,-0.6515f,0.4672f,  0.5860f,-0.6572f,0.4740f,  0.5201f,-0.6249f,0.5822f,  0.5080f,-0.6284f,0.5891f,  0.4337f,-0.5833f,0.6868f,  0.4217f,-0.5843f,0.6934f,  0.3408f,-0.5268f,0.7787f,  0.3292f,-0.5250f,0.7848f,  0.2436f,-0.4562f,0.8559f,  0.2329f,-0.4513f,0.8614f,  
	0.1446f,-0.3723f,0.9168f,  0.1353f,-0.3643f,0.9214f,  0.0463f,-0.2765f,0.9599f,  0.0390f,-0.2654f,0.9633f,  -0.0488f,-0.1707f,0.9841f,  -0.0536f,-0.1566f,0.9862f,  -0.1384f,-0.0570f,0.9887f,  -0.1402f,-0.0397f,0.9893f,  -0.2202f,0.0625f,0.9734f,  -0.2185f,0.0822f,0.9724f,  -0.2923f,0.1848f,0.9383f,  -0.2867f,0.2066f,0.9355f,  -0.3529f,0.3070f,0.8839f,  -0.3432f,0.3305f,0.8792f,  -0.4008f,0.4261f,0.8110f,  
	-0.3868f,0.4505f,0.8046f,  -0.4353f,0.5389f,0.7212f,  -0.4170f,0.5636f,0.7131f,  -0.4561f,0.6423f,0.6160f,  -0.4336f,0.6665f,0.6064f,  -0.4633f,0.7332f,0.4977f,  -0.4371f,0.7563f,0.4867f,  -0.4579f,0.8089f,0.3687f,  -0.4286f,0.8302f,0.3565f,  -0.4412f,0.8671f,0.2315f,  -0.4095f,0.8857f,0.2186f,  -0.4149f,0.9055f,0.0893f,  -0.3819f,0.9211f,0.0759f,  -0.3813f,0.9228f,-0.0550f,  -0.3480f,0.9350f,-0.0687f,  
	-0.3430f,0.9181f,-0.1984f,  -0.3107f,0.9266f,-0.2118f,  -0.3027f,0.8913f,-0.3376f,  -0.2727f,0.8960f,-0.3505f,  -0.2633f,0.8427f,-0.4696f,  -0.2369f,0.8439f,-0.4814f,  -0.2275f,0.7736f,-0.5914f,  -0.2059f,0.7716f,-0.6019f,  -0.1979f,0.6858f,-0.7004f,  -0.1822f,0.6811f,-0.7092f,  -0.1767f,0.5817f,-0.7940f,  -0.1678f,0.5751f,-0.8007f,  -0.1656f,0.4643f,-0.8701f,  -0.1643f,0.4567f,-0.8743f,  -0.1658f,0.3369f,-0.9268f,  
	-0.1724f,0.3292f,-0.9284f,  -0.1778f,0.2032f,-0.9629f,  -0.1923f,0.1965f,-0.9615f,  -0.2012f,0.0670f,-0.9773f,  -0.2235f,0.0622f,-0.9727f,  -0.2351f,-0.0680f,-0.9696f,  -0.2644f,-0.0700f,-0.9619f,  -0.2777f,-0.1982f,-0.9400f,  -0.3132f,-0.1967f,-0.9291f,  -0.3267f,-0.3204f,-0.8891f,  -0.3672f,-0.3149f,-0.8752f,  -0.3794f,-0.4319f,-0.8182f,  -0.4236f,-0.4221f,-0.8015f,  -0.4326f,-0.5305f,-0.7290f,  -0.4790f,-0.5165f,-0.7098f,  
	-0.4830f,-0.6146f,-0.6237f,  -0.5302f,-0.5966f,-0.6025f,  -0.5274f,-0.6833f,-0.5049f,  -0.5740f,-0.6618f,-0.4823f,  -0.5628f,-0.7364f,-0.3756f,  -0.6076f,-0.7120f,-0.3521f,  -0.5864f,-0.7739f,-0.2390f,  -0.6284f,-0.7475f,-0.2152f,  -0.5963f,-0.7967f,-0.0985f,  -0.6345f,-0.7692f,-0.0750f,  -0.5906f,-0.8058f,0.0425f,  -0.6246f,-0.7782f,0.0651f,  -0.5685f,-0.8027f,0.1805f,  -0.5979f,-0.7758f,0.2017f,  -0.5295f,-0.7887f,0.3123f,  
	-0.5541f,-0.7636f,0.3315f,  -0.4740f,-0.7657f,0.4348f,  -0.4939f,-0.7430f,0.4517f,  -0.4028f,-0.7352f,0.5451f,  -0.4182f,-0.7157f,0.5594f,  -0.3173f,-0.6990f,0.6409f,  -0.3285f,-0.6831f,0.6523f,  -0.2194f,-0.6583f,0.7200f,  -0.2268f,-0.6466f,0.7283f,  -0.1113f,-0.6148f,0.7808f,  -0.1155f,-0.6074f,0.7859f,  0.0043f,-0.5694f,0.8220f,  0.0029f,-0.5667f,0.8239f,  0.5860f,-0.6572f,0.4740f,  0.5736f,-0.6628f,0.4813f,  
	0.5080f,-0.6284f,0.5891f,  0.4953f,-0.6317f,0.5963f,  0.4217f,-0.5843f,0.6934f,  0.4091f,-0.5849f,0.7004f,  0.3292f,-0.5250f,0.7848f,  0.3172f,-0.5226f,0.7913f,  0.2329f,-0.4513f,0.8614f,  0.2221f,-0.4457f,0.8672f,  0.1353f,-0.3643f,0.9214f,  0.1262f,-0.3555f,0.9261f,  0.0390f,-0.2654f,0.9633f,  0.0321f,-0.2535f,0.9668f,  -0.0536f,-0.1566f,0.9862f,  -0.0578f,-0.1413f,0.9883f,  -0.1402f,-0.0397f,0.9893f,  
	-0.1411f,-0.0218f,0.9898f,  -0.2185f,0.0822f,0.9724f,  -0.2156f,0.1027f,0.9711f,  -0.2867f,0.2066f,0.9355f,  -0.2796f,0.2292f,0.9324f,  -0.3432f,0.3305f,0.8792f,  -0.3316f,0.3545f,0.8743f,  -0.3868f,0.4505f,0.8046f,  -0.3707f,0.4753f,0.7979f,  -0.4170f,0.5636f,0.7131f,  -0.3964f,0.5885f,0.7047f,  -0.4336f,0.6665f,0.6064f,  -0.4089f,0.6907f,0.5965f,  -0.4371f,0.7563f,0.4867f,  -0.4087f,0.7791f,0.4754f,  
	-0.4286f,0.8302f,0.3565f,  -0.3971f,0.8508f,0.3442f,  -0.4095f,0.8857f,0.2186f,  -0.3759f,0.9036f,0.2055f,  -0.3819f,0.9211f,0.0759f,  -0.3471f,0.9358f,0.0623f,  -0.3480f,0.9350f,-0.0687f,  -0.3134f,0.9460f,-0.0824f,  -0.3107f,0.9266f,-0.2118f,  -0.2774f,0.9340f,-0.2253f,  -0.2727f,0.8960f,-0.3505f,  -0.2421f,0.8997f,-0.3632f,  -0.2369f,0.8439f,-0.4814f,  -0.2103f,0.8442f,-0.4931f,  -0.2059f,0.7716f,-0.6019f,  
	-0.1845f,0.7689f,-0.6122f,  -0.1822f,0.6811f,-0.7092f,  -0.1671f,0.6761f,-0.7176f,  -0.1678f,0.5751f,-0.8007f,  -0.1599f,0.5686f,-0.8069f,  -0.1643f,0.4567f,-0.8743f,  -0.1641f,0.4496f,-0.8780f,  -0.1724f,0.3292f,-0.9284f,  -0.1802f,0.3226f,-0.9292f,  -0.1923f,0.1965f,-0.9615f,  -0.2080f,0.1913f,-0.9592f,  -0.2235f,0.0622f,-0.9727f,  -0.2466f,0.0594f,-0.9673f,  -0.2644f,-0.0700f,-0.9619f,  -0.2943f,-0.0696f,-0.9532f,  
	-0.3132f,-0.1967f,-0.9291f,  -0.3488f,-0.1925f,-0.9172f,  -0.3672f,-0.3149f,-0.8752f,  -0.4073f,-0.3066f,-0.8603f,  -0.4236f,-0.4221f,-0.8015f,  -0.4668f,-0.4095f,-0.7839f,  -0.4790f,-0.5165f,-0.7098f,  -0.5239f,-0.4996f,-0.6899f,  -0.5302f,-0.5966f,-0.6025f,  -0.5754f,-0.5759f,-0.5807f,  -0.5740f,-0.6618f,-0.4823f,  -0.6183f,-0.6379f,-0.4591f,  -0.6076f,-0.7120f,-0.3521f,  -0.6498f,-0.6856f,-0.3282f,  -0.6284f,-0.7475f,-0.2152f,  
	-0.6677f,-0.7195f,-0.1912f,  -0.6345f,-0.7692f,-0.0750f,  -0.6701f,-0.7405f,-0.0515f,  -0.6246f,-0.7782f,0.0651f,  -0.6560f,-0.7496f,0.0876f,  -0.5979f,-0.7758f,0.2017f,  -0.6248f,-0.7484f,0.2227f,  -0.5541f,-0.7636f,0.3315f,  -0.5765f,-0.7380f,0.3506f,  -0.4939f,-0.7430f,0.4517f,  -0.5119f,-0.7201f,0.4684f,  -0.4182f,-0.7157f,0.5594f,  -0.4319f,-0.6961f,0.5734f,  -0.3285f,-0.6831f,0.6523f,  -0.3384f,-0.6674f,0.6634f,  
	-0.2268f,-0.6466f,0.7283f,  -0.2333f,-0.6351f,0.7364f,  -0.1155f,-0.6074f,0.7859f,  -0.1190f,-0.6004f,0.7908f,  0.0029f,-0.5667f,0.8239f,  0.0018f,-0.5642f,0.8256f,  0.5736f,-0.6628f,0.4813f,  0.5605f,-0.6683f,0.4890f,  0.4953f,-0.6317f,0.5963f,  0.4821f,-0.6347f,0.6039f,  0.4091f,-0.5849f,0.7004f,  0.3961f,-0.5850f,0.7077f,  0.3172f,-0.5226f,0.7913f,  0.3050f,-0.5196f,0.7981f,  0.2221f,-0.4457f,0.8672f,  
	0.2112f,-0.4395f,0.8730f,  0.1262f,-0.3555f,0.9261f,  0.1173f,-0.3460f,0.9309f,  0.0321f,-0.2535f,0.9668f,  0.0256f,-0.2405f,0.9703f,  -0.0578f,-0.1413f,0.9883f,  -0.0612f,-0.1254f,0.9902f,  -0.1411f,-0.0218f,0.9898f,  -0.1408f,-0.0029f,0.9900f,  -0.2156f,0.1027f,0.9711f,  -0.2112f,0.1240f,0.9695f,  -0.2796f,0.2292f,0.9324f,  -0.2708f,0.2523f,0.9290f,  -0.3316f,0.3545f,0.8743f,  -0.3181f,0.3789f,0.8691f,  
	-0.3707f,0.4753f,0.7979f,  -0.3525f,0.5003f,0.7909f,  -0.3964f,0.5885f,0.7047f,  -0.3736f,0.6133f,0.6959f,  -0.4089f,0.6907f,0.5965f,  -0.3818f,0.7146f,0.5862f,  -0.4087f,0.7791f,0.4754f,  -0.3780f,0.8012f,0.4639f,  -0.3971f,0.8508f,0.3442f,  -0.3635f,0.8706f,0.3316f,  -0.3759f,0.9036f,0.2055f,  -0.3402f,0.9205f,0.1921f,  -0.3471f,0.9358f,0.0623f,  -0.3107f,0.9493f,0.0485f,  -0.3134f,0.9460f,-0.0824f,  
	-0.2774f,0.9559f,-0.0962f,  -0.2774f,0.9340f,-0.2253f,  -0.2432f,0.9401f,-0.2387f,  -0.2421f,0.8997f,-0.3632f,  -0.2110f,0.9023f,-0.3759f,  -0.2103f,0.8442f,-0.4931f,  -0.1836f,0.8435f,-0.5047f,  -0.1845f,0.7689f,-0.6122f,  -0.1635f,0.7656f,-0.6222f,  -0.1671f,0.6761f,-0.7176f,  -0.1527f,0.6709f,-0.7256f,  -0.1599f,0.5686f,-0.8069f,  -0.1529f,0.5624f,-0.8126f,  -0.1641f,0.4496f,-0.8780f,  -0.1649f,0.4433f,-0.8811f,  
	-0.1802f,0.3226f,-0.9292f,  -0.1890f,0.3172f,-0.9293f,  -0.2080f,0.1913f,-0.9592f,  -0.2246f,0.1878f,-0.9562f,  -0.2466f,0.0594f,-0.9673f,  -0.2703f,0.0586f,-0.9610f,  -0.2943f,-0.0696f,-0.9532f,  -0.3244f,-0.0669f,-0.9436f,  -0.3488f,-0.1925f,-0.9172f,  -0.3841f,-0.1857f,-0.9044f,  -0.4073f,-0.3066f,-0.8603f,  -0.4466f,-0.2955f,-0.8445f,  -0.4668f,-0.4095f,-0.7839f,  -0.5087f,-0.3941f,-0.7655f,  -0.5239f,-0.4996f,-0.6899f,  
	-0.5670f,-0.4802f,-0.6692f,  -0.5754f,-0.5759f,-0.5807f,  -0.6185f,-0.5529f,-0.5584f,  -0.6183f,-0.6379f,-0.4591f,  -0.6601f,-0.6119f,-0.4356f,  -0.6498f,-0.6856f,-0.3282f,  -0.6894f,-0.6575f,-0.3041f,  -0.6677f,-0.7195f,-0.1912f,  -0.7042f,-0.6901f,-0.1671f,  -0.6701f,-0.7405f,-0.0515f,  -0.7030f,-0.7107f,-0.0280f,  -0.6560f,-0.7496f,0.0876f,  -0.6848f,-0.7204f,0.1100f,  -0.6248f,-0.7484f,0.2227f,  -0.6493f,-0.7205f,0.2434f,  
	-0.5765f,-0.7380f,0.3506f,  -0.5968f,-0.7123f,0.3693f,  -0.5119f,-0.7201f,0.4684f,  -0.5280f,-0.6973f,0.4848f,  -0.4319f,-0.6961f,0.5734f,  -0.4442f,-0.6767f,0.5871f,  -0.3384f,-0.6674f,0.6634f,  -0.3471f,-0.6519f,0.6742f,  -0.2333f,-0.6351f,0.7364f,  -0.2389f,-0.6238f,0.7442f,  -0.1190f,-0.6004f,0.7908f,  -0.1220f,-0.5936f,0.7955f,  0.0018f,-0.5642f,0.8256f,  0.0009f,-0.5620f,0.8271f,  0.5605f,-0.6683f,0.4890f,  
	0.5468f,-0.6737f,0.4972f,  0.4821f,-0.6347f,0.6039f,  0.4682f,-0.6373f,0.6120f,  0.3961f,-0.5850f,0.7077f,  0.3827f,-0.5846f,0.7154f,  0.3050f,-0.5196f,0.7981f,  0.2926f,-0.5160f,0.8050f,  0.2112f,-0.4395f,0.8730f,  0.2004f,-0.4327f,0.8790f,  0.1173f,-0.3460f,0.9309f,  0.1086f,-0.3355f,0.9358f,  0.0256f,-0.2405f,0.9703f,  0.0198f,-0.2268f,0.9737f,  -0.0612f,-0.1254f,0.9902f,  -0.0636f,-0.1085f,0.9921f,  
	-0.1408f,-0.0029f,0.9900f,  -0.1393f,0.0167f,0.9901f,  -0.2112f,0.1240f,0.9695f,  -0.2053f,0.1459f,0.9678f,  -0.2708f,0.2523f,0.9290f,  -0.2601f,0.2760f,0.9253f,  -0.3181f,0.3789f,0.8691f,  -0.3025f,0.4036f,0.8635f,  -0.3525f,0.5003f,0.7909f,  -0.3320f,0.5253f,0.7835f,  -0.3736f,0.6133f,0.6959f,  -0.3484f,0.6379f,0.6868f,  -0.3818f,0.7146f,0.5862f,  -0.3523f,0.7379f,0.5756f,  -0.3780f,0.8012f,0.4639f,  
	-0.3448f,0.8226f,0.4521f,  -0.3635f,0.8706f,0.3316f,  -0.3275f,0.8895f,0.3187f,  -0.3402f,0.9205f,0.1921f,  -0.3026f,0.9362f,0.1786f,  -0.3107f,0.9493f,0.0485f,  -0.2726f,0.9615f,0.0345f,  -0.2774f,0.9559f,-0.0962f,  -0.2401f,0.9645f,-0.1101f,  -0.2432f,0.9401f,-0.2387f,  -0.2082f,0.9450f,-0.2522f,  -0.2110f,0.9023f,-0.3759f,  -0.1795f,0.9037f,-0.3886f,  -0.1836f,0.8435f,-0.5047f,  -0.1570f,0.8420f,-0.5161f,  
	-0.1635f,0.7656f,-0.6222f,  -0.1429f,0.7617f,-0.6320f,  -0.1527f,0.6709f,-0.7256f,  -0.1390f,0.6655f,-0.7334f,  -0.1529f,0.5624f,-0.8126f,  -0.1468f,0.5563f,-0.8179f,  -0.1649f,0.4433f,-0.8811f,  -0.1667f,0.4377f,-0.8836f,  -0.1890f,0.3172f,-0.9293f,  -0.1987f,0.3130f,-0.9287f,  -0.2246f,0.1878f,-0.9562f,  -0.2418f,0.1859f,-0.9524f,  -0.2703f,0.0586f,-0.9610f,  -0.2944f,0.0600f,-0.9538f,  -0.3244f,-0.0669f,-0.9436f,  
	-0.3543f,-0.0617f,-0.9331f,  -0.3841f,-0.1857f,-0.9044f,  -0.4188f,-0.1765f,-0.8908f,  -0.4466f,-0.2955f,-0.8445f,  -0.4848f,-0.2819f,-0.8280f,  -0.5087f,-0.3941f,-0.7655f,  -0.5490f,-0.3763f,-0.7464f,  -0.5670f,-0.4802f,-0.6692f,  -0.6081f,-0.4585f,-0.6481f,  -0.6185f,-0.5529f,-0.5584f,  -0.6592f,-0.5278f,-0.5356f,  -0.6601f,-0.6119f,-0.4356f,  -0.6993f,-0.5842f,-0.4118f,  -0.6894f,-0.6575f,-0.3041f,  -0.7262f,-0.6279f,-0.2799f,  
	-0.7042f,-0.6901f,-0.1671f,  -0.7379f,-0.6596f,-0.1430f,  -0.7030f,-0.7107f,-0.0280f,  -0.7331f,-0.6801f,-0.0046f,  -0.6848f,-0.7204f,0.1100f,  -0.7110f,-0.6906f,0.1322f,  -0.6493f,-0.7205f,0.2434f,  -0.6715f,-0.6923f,0.2640f,  -0.5968f,-0.7123f,0.3693f,  -0.6150f,-0.6865f,0.3878f,  -0.5280f,-0.6973f,0.4848f,  -0.5424f,-0.6745f,0.5009f,  -0.4442f,-0.6767f,0.5871f,  -0.4550f,-0.6575f,0.6006f,  -0.3471f,-0.6519f,0.6742f,  
	-0.3547f,-0.6366f,0.6848f,  -0.2389f,-0.6238f,0.7442f,  -0.2437f,-0.6128f,0.7517f,  -0.1220f,-0.5936f,0.7955f,  -0.1245f,-0.5871f,0.7999f,  0.0009f,-0.5620f,0.8271f,  0.0003f,-0.5601f,0.8284f,  0.5468f,-0.6737f,0.4972f,  0.5323f,-0.6788f,0.5058f,  0.4682f,-0.6373f,0.6120f,  0.4539f,-0.6395f,0.6204f,  0.3827f,-0.5846f,0.7154f,  0.3690f,-0.5837f,0.7233f,  0.2926f,-0.5160f,0.8050f,  0.2801f,-0.5119f,0.8121f,  
	0.2004f,-0.4327f,0.8790f,  0.1897f,-0.4248f,0.8852f,  0.1086f,-0.3355f,0.9358f,  0.1004f,-0.3244f,0.9406f,  0.0198f,-0.2268f,0.9737f,  0.0147f,-0.2122f,0.9771f,  -0.0636f,-0.1085f,0.9921f,  -0.0649f,-0.0909f,0.9937f,  -0.1393f,0.0167f,0.9901f,  -0.1363f,0.0370f,0.9900f,  -0.2053f,0.1459f,0.9678f,  -0.1976f,0.1683f,0.9657f,  -0.2601f,0.2760f,0.9253f,  -0.2473f,0.2999f,0.9213f,  -0.3025f,0.4036f,0.8635f,  
	-0.2846f,0.4284f,0.8576f,  -0.3320f,0.5253f,0.7835f,  -0.3090f,0.5502f,0.7757f,  -0.3484f,0.6379f,0.6868f,  -0.3206f,0.6620f,0.6774f,  -0.3523f,0.7379f,0.5756f,  -0.3203f,0.7606f,0.5647f,  -0.3448f,0.8226f,0.4521f,  -0.3092f,0.8432f,0.4398f,  -0.3275f,0.8895f,0.3187f,  -0.2894f,0.9071f,0.3055f,  -0.3026f,0.9362f,0.1786f,  -0.2631f,0.9506f,0.1646f,  -0.2726f,0.9615f,0.0345f,  -0.2329f,0.9723f,0.0204f,  
	-0.2401f,0.9645f,-0.1101f,  -0.2017f,0.9715f,-0.1242f,  -0.2082f,0.9450f,-0.2522f,  -0.1724f,0.9485f,-0.2658f,  -0.1795f,0.9037f,-0.3886f,  -0.1478f,0.9040f,-0.4012f,  -0.1570f,0.8420f,-0.5161f,  -0.1305f,0.8395f,-0.5275f,  -0.1429f,0.7617f,-0.6320f,  -0.1228f,0.7572f,-0.6415f,  -0.1390f,0.6655f,-0.7334f,  -0.1261f,0.6599f,-0.7407f,  -0.1468f,0.5563f,-0.8179f,  -0.1415f,0.5506f,-0.8227f,  -0.1667f,0.4377f,-0.8836f,  
	-0.1693f,0.4328f,-0.8854f,  -0.1987f,0.3130f,-0.9287f,  -0.2090f,0.3101f,-0.9275f,  -0.2418f,0.1859f,-0.9524f,  -0.2594f,0.1858f,-0.9477f,  -0.2944f,0.0600f,-0.9538f,  -0.3185f,0.0633f,-0.9458f,  -0.3543f,-0.0617f,-0.9331f,  -0.3839f,-0.0544f,-0.9218f,  -0.4188f,-0.1765f,-0.8908f,  -0.4526f,-0.1648f,-0.8764f,  -0.4848f,-0.2819f,-0.8280f,  -0.5215f,-0.2659f,-0.8107f,  -0.5490f,-0.3763f,-0.7464f,  -0.5874f,-0.3562f,-0.7267f,  
	-0.6081f,-0.4585f,-0.6481f,  -0.6470f,-0.4347f,-0.6264f,  -0.6592f,-0.5278f,-0.5356f,  -0.6974f,-0.5010f,-0.5126f,  -0.6993f,-0.5842f,-0.4118f,  -0.7359f,-0.5550f,-0.3879f,  -0.7262f,-0.6279f,-0.2799f,  -0.7603f,-0.5972f,-0.2557f,  -0.7379f,-0.6596f,-0.1430f,  -0.7689f,-0.6282f,-0.1190f,  -0.7331f,-0.6801f,-0.0046f,  -0.7606f,-0.6490f,0.0187f,  -0.7110f,-0.6906f,0.1322f,  -0.7348f,-0.6605f,0.1542f,  -0.6715f,-0.6923f,0.2640f,  
	-0.6915f,-0.6641f,0.2843f,  -0.6150f,-0.6865f,0.3878f,  -0.6313f,-0.6608f,0.4060f,  -0.5424f,-0.6745f,0.5009f,  -0.5551f,-0.6519f,0.5167f,  -0.4550f,-0.6575f,0.6006f,  -0.4644f,-0.6385f,0.6137f,  -0.3547f,-0.6366f,0.6848f,  -0.3613f,-0.6216f,0.6951f,  -0.2437f,-0.6128f,0.7517f,  -0.2478f,-0.6021f,0.7590f,  -0.1245f,-0.5871f,0.7999f,  -0.1265f,-0.5809f,0.8041f,  0.0003f,-0.5601f,0.8284f,  -0.0002f,-0.5585f,0.8295f,  
	0.5323f,-0.6788f,0.5058f,  0.5173f,-0.6836f,0.5149f,  0.4539f,-0.6395f,0.6204f,  0.4391f,-0.6414f,0.6291f,  0.3690f,-0.5837f,0.7233f,  0.3551f,-0.5823f,0.7313f,  0.2801f,-0.5119f,0.8121f,  0.2675f,-0.5068f,0.8195f,  0.1897f,-0.4248f,0.8852f,  0.1792f,-0.4164f,0.8914f,  0.1004f,-0.3244f,0.9406f,  0.0927f,-0.3123f,0.9454f,  0.0147f,-0.2122f,0.9771f,  0.0105f,-0.1969f,0.9804f,  -0.0649f,-0.0909f,0.9937f,  
	-0.0650f,-0.0726f,0.9952f,  -0.1363f,0.0370f,0.9900f,  -0.1317f,0.0578f,0.9896f,  -0.1976f,0.1683f,0.9657f,  -0.1879f,0.1912f,0.9634f,  -0.2473f,0.2999f,0.9213f,  -0.2324f,0.3241f,0.9170f,  -0.2846f,0.4284f,0.8576f,  -0.2643f,0.4532f,0.8513f,  -0.3090f,0.5502f,0.7757f,  -0.2835f,0.5747f,0.7677f,  -0.3206f,0.6620f,0.6774f,  -0.2903f,0.6855f,0.6677f,  -0.3203f,0.7606f,0.5647f,  -0.2857f,0.7824f,0.5534f,  
	-0.3092f,0.8432f,0.4398f,  -0.2713f,0.8624f,0.4273f,  -0.2894f,0.9071f,0.3055f,  -0.2491f,0.9234f,0.2920f,  -0.2631f,0.9506f,0.1646f,  -0.2217f,0.9634f,0.1506f,  -0.2329f,0.9723f,0.0204f,  -0.1918f,0.9814f,0.0060f,  -0.2017f,0.9715f,-0.1242f,  -0.1622f,0.9770f,-0.1384f,  -0.1724f,0.9485f,-0.2658f,  -0.1360f,0.9505f,-0.2795f,  -0.1478f,0.9040f,-0.4012f,  -0.1159f,0.9029f,-0.4139f,  -0.1305f,0.8395f,-0.5275f,  
	-0.1043f,0.8360f,-0.5387f,  -0.1228f,0.7572f,-0.6415f,  -0.1032f,0.7522f,-0.6508f,  -0.1261f,0.6599f,-0.7407f,  -0.1139f,0.6542f,-0.7477f,  -0.1415f,0.5506f,-0.8227f,  -0.1371f,0.5453f,-0.8269f,  -0.1693f,0.4328f,-0.8854f,  -0.1727f,0.4289f,-0.8867f,  -0.2090f,0.3101f,-0.9275f,  -0.2199f,0.3085f,-0.9255f,  -0.2594f,0.1858f,-0.9477f,  -0.2772f,0.1873f,-0.9424f,  -0.3185f,0.0633f,-0.9458f,  -0.3423f,0.0686f,-0.9371f,  
	-0.3839f,-0.0544f,-0.9218f,  -0.4127f,-0.0449f,-0.9098f,  -0.4526f,-0.1648f,-0.8764f,  -0.4852f,-0.1510f,-0.8613f,  -0.5215f,-0.2659f,-0.8107f,  -0.5566f,-0.2478f,-0.7929f,  -0.5874f,-0.3562f,-0.7267f,  -0.6238f,-0.3341f,-0.7066f,  -0.6470f,-0.4347f,-0.6264f,  -0.6835f,-0.4092f,-0.6045f,  -0.6974f,-0.5010f,-0.5126f,  -0.7329f,-0.4726f,-0.4893f,  -0.7359f,-0.5550f,-0.3879f,  -0.7696f,-0.5246f,-0.3639f,  -0.7603f,-0.5972f,-0.2557f,  
	-0.7915f,-0.5656f,-0.2315f,  -0.7689f,-0.6282f,-0.1190f,  -0.7972f,-0.5962f,-0.0952f,  -0.7606f,-0.6490f,0.0187f,  -0.7855f,-0.6174f,0.0417f,  -0.7348f,-0.6605f,0.1542f,  -0.7562f,-0.6302f,0.1759f,  -0.6915f,-0.6641f,0.2843f,  -0.7094f,-0.6358f,0.3042f,  -0.6313f,-0.6608f,0.4060f,  -0.6457f,-0.6352f,0.4239f,  -0.5551f,-0.6519f,0.5167f,  -0.5662f,-0.6295f,0.5321f,  -0.4644f,-0.6385f,0.6137f,  -0.4726f,-0.6197f,0.6265f,  
	-0.3613f,-0.6216f,0.6951f,  -0.3668f,-0.6069f,0.7051f,  -0.2478f,-0.6021f,0.7590f,  -0.2512f,-0.5918f,0.7660f,  -0.1265f,-0.5809f,0.8041f,  -0.1281f,-0.5750f,0.8081f,  -0.0002f,-0.5585f,0.8295f,  -0.0005f,-0.5572f,0.8304f,  0.5173f,-0.6836f,0.5149f,  0.5017f,-0.6881f,0.5242f,  0.4391f,-0.6414f,0.6291f,  0.4239f,-0.6428f,0.6380f,  0.3551f,-0.5823f,0.7313f,  0.3408f,-0.5802f,0.7398f,  0.2675f,-0.5068f,0.8195f,  
	0.2550f,-0.5012f,0.8269f,  0.1792f,-0.4164f,0.8914f,  0.1692f,-0.4071f,0.8976f,  0.0927f,-0.3123f,0.9454f,  0.0858f,-0.2995f,0.9502f,  0.0105f,-0.1969f,0.9804f,  0.0075f,-0.1809f,0.9835f,  -0.0650f,-0.0726f,0.9952f,  -0.0635f,-0.0536f,0.9965f,  -0.1317f,0.0578f,0.9896f,  -0.1253f,0.0792f,0.9890f,  -0.1879f,0.1912f,0.9634f,  -0.1762f,0.2143f,0.9607f,  -0.2324f,0.3241f,0.9170f,  -0.2151f,0.3484f,0.9124f,  
	-0.2643f,0.4532f,0.8513f,  -0.2415f,0.4776f,0.8447f,  -0.2835f,0.5747f,0.7677f,  -0.2553f,0.5987f,0.7592f,  -0.2903f,0.6855f,0.6677f,  -0.2572f,0.7082f,0.6575f,  -0.2857f,0.7824f,0.5534f,  -0.2485f,0.8030f,0.5417f,  -0.2713f,0.8624f,0.4273f,  -0.2309f,0.8804f,0.4143f,  -0.2491f,0.9234f,0.2920f,  -0.2067f,0.9380f,0.2782f,  -0.2217f,0.9634f,0.1506f,  -0.1785f,0.9745f,0.1361f,  -0.1918f,0.9814f,0.0060f,  
	-0.1493f,0.9888f,-0.0087f,  -0.1622f,0.9770f,-0.1384f,  -0.1219f,0.9807f,-0.1529f,  -0.1360f,0.9505f,-0.2795f,  -0.0993f,0.9509f,-0.2933f,  -0.1159f,0.9029f,-0.4139f,  -0.0841f,0.9005f,-0.4266f,  -0.1043f,0.8360f,-0.5387f,  -0.0785f,0.8316f,-0.5498f,  -0.1032f,0.7522f,-0.6508f,  -0.0842f,0.7466f,-0.6599f,  -0.1139f,0.6542f,-0.7477f,  -0.1024f,0.6485f,-0.7543f,  -0.1371f,0.5453f,-0.8269f,  -0.1334f,0.5404f,-0.8308f,  
	-0.1727f,0.4289f,-0.8867f,  -0.1766f,0.4258f,-0.8874f,  -0.2199f,0.3085f,-0.9255f,  -0.2311f,0.3081f,-0.9228f,  -0.2772f,0.1873f,-0.9424f,  -0.2949f,0.1905f,-0.9363f,  -0.3423f,0.0686f,-0.9371f,  -0.3657f,0.0758f,-0.9277f,  -0.4127f,-0.0449f,-0.9098f,  -0.4405f,-0.0335f,-0.8971f,  -0.4852f,-0.1510f,-0.8613f,  -0.5163f,-0.1352f,-0.8457f,  -0.5566f,-0.2478f,-0.7929f,  -0.5899f,-0.2279f,-0.7747f,  -0.6238f,-0.3341f,-0.7066f,  
	-0.6579f,-0.3104f,-0.6861f,  -0.6835f,-0.4092f,-0.6045f,  -0.7175f,-0.3822f,-0.5823f,  -0.7329f,-0.4726f,-0.4893f,  -0.7659f,-0.4431f,-0.4660f,  -0.7696f,-0.5246f,-0.3639f,  -0.8007f,-0.4933f,-0.3399f,  -0.7915f,-0.5656f,-0.2315f,  -0.8201f,-0.5333f,-0.2074f,  -0.7972f,-0.5962f,-0.0952f,  -0.8228f,-0.5638f,-0.0715f,  -0.7855f,-0.6174f,0.0417f,  -0.8080f,-0.5857f,0.0645f,  -0.7562f,-0.6302f,0.1759f,  -0.7754f,-0.5999f,0.1973f,  
	-0.7094f,-0.6358f,0.3042f,  -0.7252f,-0.6076f,0.3239f,  -0.6457f,-0.6352f,0.4239f,  -0.6583f,-0.6097f,0.4415f,  -0.5662f,-0.6295f,0.5321f,  -0.5759f,-0.6073f,0.5473f,  -0.4726f,-0.6197f,0.6265f,  -0.4796f,-0.6013f,0.6391f,  -0.3668f,-0.6069f,0.7051f,  -0.3715f,-0.5925f,0.7148f,  -0.2512f,-0.5918f,0.7660f,  -0.2539f,-0.5817f,0.7728f,  -0.1281f,-0.5750f,0.8081f,  -0.1294f,-0.5694f,0.8118f,  -0.0005f,-0.5572f,0.8304f,  
	-0.0007f,-0.5561f,0.8311f,  0.5017f,-0.6881f,0.5242f,  0.4855f,-0.6923f,0.5339f,  0.4239f,-0.6428f,0.6380f,  0.4083f,-0.6436f,0.6474f,  0.3408f,-0.5802f,0.7398f,  0.3265f,-0.5775f,0.7483f,  0.2550f,-0.5012f,0.8269f,  0.2427f,-0.4948f,0.8344f,  0.1692f,-0.4071f,0.8976f,  0.1596f,-0.3970f,0.9038f,  0.0858f,-0.2995f,0.9502f,  0.0798f,-0.2861f,0.9549f,  0.0075f,-0.1809f,0.9835f,  0.0057f,-0.1642f,0.9864f,  
	-0.0635f,-0.0536f,0.9965f,  -0.0605f,-0.0342f,0.9976f,  -0.1253f,0.0792f,0.9890f,  -0.1170f,0.1009f,0.9880f,  -0.1762f,0.2143f,0.9607f,  -0.1622f,0.2376f,0.9577f,  -0.2151f,0.3484f,0.9124f,  -0.1953f,0.3724f,0.9073f,  -0.2415f,0.4776f,0.8447f,  -0.2159f,0.5016f,0.8377f,  -0.2553f,0.5987f,0.7592f,  -0.2244f,0.6219f,0.7503f,  -0.2572f,0.7082f,0.6575f,  -0.2215f,0.7297f,0.6469f,  -0.2485f,0.8030f,0.5417f,  
	-0.2088f,0.8222f,0.5295f,  -0.2309f,0.8804f,0.4143f,  -0.1882f,0.8965f,0.4010f,  -0.2067f,0.9380f,0.2782f,  -0.1623f,0.9508f,0.2639f,  -0.1785f,0.9745f,0.1361f,  -0.1338f,0.9836f,0.1212f,  -0.1493f,0.9888f,-0.0087f,  -0.1057f,0.9941f,-0.0237f,  -0.1219f,0.9807f,-0.1529f,  -0.0809f,0.9825f,-0.1676f,  -0.0993f,0.9509f,-0.2933f,  -0.0623f,0.9496f,-0.3073f,  -0.0841f,0.9005f,-0.4266f,  -0.0524f,0.8968f,-0.4394f,  
	-0.0785f,0.8316f,-0.5498f,  -0.0531f,0.8262f,-0.5608f,  -0.0842f,0.7466f,-0.6599f,  -0.0660f,0.7405f,-0.6688f,  -0.1024f,0.6485f,-0.7543f,  -0.0917f,0.6427f,-0.7606f,  -0.1334f,0.5404f,-0.8308f,  -0.1303f,0.5360f,-0.8341f,  -0.1766f,0.4258f,-0.8874f,  -0.1811f,0.4237f,-0.8875f,  -0.2311f,0.3081f,-0.9228f,  -0.2425f,0.3091f,-0.9196f,  -0.2949f,0.1905f,-0.9363f,  -0.3124f,0.1951f,-0.9297f,  -0.3657f,0.0758f,-0.9277f,  
	-0.3883f,0.0846f,-0.9176f,  -0.4405f,-0.0335f,-0.8971f,  -0.4672f,-0.0203f,-0.8839f,  -0.5163f,-0.1352f,-0.8457f,  -0.5459f,-0.1177f,-0.8296f,  -0.5899f,-0.2279f,-0.7747f,  -0.6211f,-0.2063f,-0.7561f,  -0.6579f,-0.3104f,-0.6861f,  -0.6898f,-0.2852f,-0.6654f,  -0.7175f,-0.3822f,-0.5823f,  -0.7490f,-0.3540f,-0.5601f,  -0.7659f,-0.4431f,-0.4660f,  -0.7962f,-0.4126f,-0.4426f,  -0.8007f,-0.4933f,-0.3399f,  -0.8291f,-0.4613f,-0.3160f,  
	-0.8201f,-0.5333f,-0.2074f,  -0.8461f,-0.5005f,-0.1834f,  -0.8228f,-0.5638f,-0.0715f,  -0.8460f,-0.5311f,-0.0480f,  -0.8080f,-0.5857f,0.0645f,  -0.8281f,-0.5538f,0.0870f,  -0.7754f,-0.5999f,0.1973f,  -0.7924f,-0.5696f,0.2184f,  -0.7252f,-0.6076f,0.3239f,  -0.7391f,-0.5795f,0.3433f,  -0.6583f,-0.6097f,0.4415f,  -0.6693f,-0.5845f,0.4588f,  -0.5759f,-0.6073f,0.5473f,  -0.5841f,-0.5854f,0.5622f,  -0.4796f,-0.6013f,0.6391f,  
	-0.4855f,-0.5831f,0.6514f,  -0.3715f,-0.5925f,0.7148f,  -0.3753f,-0.5784f,0.7243f,  -0.2539f,-0.5817f,0.7728f,  -0.2560f,-0.5719f,0.7794f,  -0.1294f,-0.5694f,0.8118f,  -0.1303f,-0.5641f,0.8154f,  -0.0007f,-0.5561f,0.8311f,  -0.0008f,-0.5554f,0.8316f,  0.4855f,-0.6923f,0.5339f,  0.4687f,-0.6960f,0.5440f,  0.4083f,-0.6436f,0.6474f,  0.3923f,-0.6439f,0.6569f,  0.3265f,-0.5775f,0.7483f,  0.3121f,-0.5741f,0.7570f,  
	0.2427f,-0.4948f,0.8344f,  0.2307f,-0.4877f,0.8420f,  0.1596f,-0.3970f,0.9038f,  0.1508f,-0.3863f,0.9100f,  0.0798f,-0.2861f,0.9549f,  0.0748f,-0.2719f,0.9594f,  0.0057f,-0.1642f,0.9864f,  0.0053f,-0.1469f,0.9891f,  -0.0605f,-0.0342f,0.9976f,  -0.0557f,-0.0143f,0.9983f,  -0.1170f,0.1009f,0.9880f,  -0.1065f,0.1228f,0.9867f,  -0.1622f,0.2376f,0.9577f,  -0.1458f,0.2607f,0.9543f,  -0.1953f,0.3724f,0.9073f,  
	-0.1729f,0.3959f,0.9019f,  -0.2159f,0.5016f,0.8377f,  -0.1877f,0.5249f,0.8302f,  -0.2244f,0.6219f,0.7503f,  -0.1907f,0.6440f,0.7409f,  -0.2215f,0.7297f,0.6469f,  -0.1831f,0.7499f,0.6357f,  -0.2088f,0.8222f,0.5295f,  -0.1665f,0.8397f,0.5170f,  -0.1882f,0.8965f,0.4010f,  -0.1433f,0.9108f,0.3871f,  -0.1623f,0.9508f,0.2639f,  -0.1160f,0.9615f,0.2491f,  -0.1338f,0.9836f,0.1212f,  -0.0876f,0.9905f,0.1060f,  
	-0.1057f,0.9941f,-0.0237f,  -0.0611f,0.9974f,-0.0390f,  -0.0809f,0.9825f,-0.1676f,  -0.0395f,0.9824f,-0.1826f,  -0.0623f,0.9496f,-0.3073f,  -0.0254f,0.9466f,-0.3214f,  -0.0524f,0.8968f,-0.4394f,  -0.0210f,0.8917f,-0.4522f,  -0.0531f,0.8262f,-0.5608f,  -0.0284f,0.8199f,-0.5718f,  -0.0660f,0.7405f,-0.6688f,  -0.0485f,0.7339f,-0.6775f,  -0.0917f,0.6427f,-0.7606f,  -0.0817f,0.6369f,-0.7666f,  -0.1303f,0.5360f,-0.8341f,  
	-0.1279f,0.5320f,-0.8370f,  -0.1811f,0.4237f,-0.8875f,  -0.1858f,0.4224f,-0.8872f,  -0.2425f,0.3091f,-0.9196f,  -0.2538f,0.3112f,-0.9158f,  -0.3124f,0.1951f,-0.9297f,  -0.3295f,0.2013f,-0.9225f,  -0.3883f,0.0846f,-0.9176f,  -0.4101f,0.0950f,-0.9071f,  -0.4672f,-0.0203f,-0.8839f,  -0.4926f,-0.0055f,-0.8702f,  -0.5459f,-0.1177f,-0.8296f,  -0.5737f,-0.0987f,-0.8131f,  -0.6211f,-0.2063f,-0.7561f,  -0.6503f,-0.1834f,-0.7372f,  
	-0.6898f,-0.2852f,-0.6654f,  -0.7194f,-0.2588f,-0.6446f,  -0.7490f,-0.3540f,-0.5601f,  -0.7780f,-0.3248f,-0.5377f,  -0.7962f,-0.4126f,-0.4426f,  -0.8239f,-0.3813f,-0.4193f,  -0.8291f,-0.4613f,-0.3160f,  -0.8549f,-0.4287f,-0.2923f,  -0.8461f,-0.5005f,-0.1834f,  -0.8695f,-0.4674f,-0.1597f,  -0.8460f,-0.5311f,-0.0480f,  -0.8667f,-0.4982f,-0.0248f,  -0.8281f,-0.5538f,0.0870f,  -0.8460f,-0.5219f,0.1092f,  -0.7924f,-0.5696f,0.2184f,  
	-0.8073f,-0.5394f,0.2393f,  -0.7391f,-0.5795f,0.3433f,  -0.7512f,-0.5517f,0.3624f,  -0.6693f,-0.5845f,0.4588f,  -0.6787f,-0.5595f,0.4758f,  -0.5841f,-0.5854f,0.5622f,  -0.5911f,-0.5637f,0.5769f,  -0.4855f,-0.5831f,0.6514f,  -0.4903f,-0.5652f,0.6634f,  -0.3753f,-0.5784f,0.7243f,  -0.3783f,-0.5646f,0.7336f,  -0.2560f,-0.5719f,0.7794f,  -0.2576f,-0.5624f,0.7857f,  -0.1303f,-0.5641f,0.8154f,  -0.1309f,-0.5591f,0.8187f,  
	-0.0008f,-0.5554f,0.8316f,  -0.0008f,-0.5549f,0.8319f,  0.4687f,-0.6960f,0.5440f,  0.4515f,-0.6993f,0.5542f,  0.3923f,-0.6439f,0.6569f,  0.3762f,-0.6436f,0.6666f,  0.3121f,-0.5741f,0.7570f,  0.2978f,-0.5701f,0.7657f,  0.2307f,-0.4877f,0.8420f,  0.2191f,-0.4800f,0.8495f,  0.1508f,-0.3863f,0.9100f,  0.1427f,-0.3749f,0.9160f,  0.0748f,-0.2719f,0.9594f,  0.0710f,-0.2571f,0.9638f,  0.0053f,-0.1469f,0.9891f,  
	0.0065f,-0.1292f,0.9916f,  -0.0557f,-0.0143f,0.9983f,  -0.0490f,0.0059f,0.9988f,  -0.1065f,0.1228f,0.9867f,  -0.0938f,0.1446f,0.9850f,  -0.1458f,0.2607f,0.9543f,  -0.1269f,0.2835f,0.9506f,  -0.1729f,0.3959f,0.9019f,  -0.1478f,0.4189f,0.8959f,  -0.1877f,0.5249f,0.8302f,  -0.1566f,0.5472f,0.8222f,  -0.1907f,0.6440f,0.7409f,  -0.1542f,0.6648f,0.7309f,  -0.1831f,0.7499f,0.6357f,  -0.1420f,0.7684f,0.6240f,  
	-0.1665f,0.8397f,0.5170f,  -0.1219f,0.8552f,0.5038f,  -0.1433f,0.9108f,0.3871f,  -0.0963f,0.9229f,0.3727f,  -0.1160f,0.9615f,0.2491f,  -0.0681f,0.9699f,0.2338f,  -0.0876f,0.9905f,0.1060f,  -0.0403f,0.9951f,0.0902f,  -0.0611f,0.9974f,-0.0390f,  -0.0159f,0.9984f,-0.0548f,  -0.0395f,0.9824f,-0.1826f,  0.0022f,0.9802f,-0.1980f,  -0.0254f,0.9466f,-0.3214f,  0.0115f,0.9419f,-0.3357f,  -0.0210f,0.8917f,-0.4522f,  
	0.0098f,0.8852f,-0.4650f,  -0.0284f,0.8199f,-0.5718f,  -0.0043f,0.8127f,-0.5827f,  -0.0485f,0.7339f,-0.6775f,  -0.0317f,0.7270f,-0.6859f,  -0.0817f,0.6369f,-0.7666f,  -0.0725f,0.6312f,-0.7722f,  -0.1279f,0.5320f,-0.8370f,  -0.1260f,0.5285f,-0.8395f,  -0.1858f,0.4224f,-0.8872f,  -0.1908f,0.4220f,-0.8863f,  -0.2538f,0.3112f,-0.9158f,  -0.2650f,0.3145f,-0.9115f,  -0.3295f,0.2013f,-0.9225f,  -0.3460f,0.2087f,-0.9147f,  
	-0.4101f,0.0950f,-0.9071f,  -0.4308f,0.1068f,-0.8961f,  -0.4926f,-0.0055f,-0.8702f,  -0.5165f,0.0107f,-0.8562f,  -0.5737f,-0.0987f,-0.8131f,  -0.5998f,-0.0783f,-0.7963f,  -0.6503f,-0.1834f,-0.7372f,  -0.6775f,-0.1593f,-0.7181f,  -0.7194f,-0.2588f,-0.6446f,  -0.7466f,-0.2315f,-0.6236f,  -0.7780f,-0.3248f,-0.5377f,  -0.8046f,-0.2949f,-0.5154f,  -0.8239f,-0.3813f,-0.4193f,  -0.8491f,-0.3495f,-0.3961f,  -0.8549f,-0.4287f,-0.2923f,  
	-0.8782f,-0.3958f,-0.2687f,  -0.8695f,-0.4674f,-0.1597f,  -0.8905f,-0.4341f,-0.1362f,  -0.8667f,-0.4982f,-0.0248f,  -0.8851f,-0.4653f,-0.0019f,  -0.8460f,-0.5219f,0.1092f,  -0.8617f,-0.4901f,0.1311f,  -0.8073f,-0.5394f,0.2393f,  -0.8204f,-0.5094f,0.2598f,  -0.7512f,-0.5517f,0.3624f,  -0.7616f,-0.5240f,0.3812f,  -0.6787f,-0.5595f,0.4758f,  -0.6866f,-0.5348f,0.4925f,  -0.5911f,-0.5637f,0.5769f,  -0.5968f,-0.5424f,0.5912f,  
	-0.4903f,-0.5652f,0.6634f,  -0.4941f,-0.5477f,0.6752f,  -0.3783f,-0.5646f,0.7336f,  -0.3806f,-0.5511f,0.7426f,  -0.2576f,-0.5624f,0.7857f,  -0.2587f,-0.5532f,0.7918f,  -0.1309f,-0.5591f,0.8187f,  -0.1312f,-0.5544f,0.8219f,  -0.0008f,-0.5549f,0.8319f,  -0.0008f,-0.5547f,0.8320f,  0.4515f,-0.6993f,0.5542f,  0.4338f,-0.7020f,0.5648f,  0.3762f,-0.6436f,0.6666f,  0.3599f,-0.6427f,0.6763f,  0.2978f,-0.5701f,0.7657f,  
	0.2838f,-0.5654f,0.7745f,  0.2191f,-0.4800f,0.8495f,  0.2081f,-0.4716f,0.8569f,  0.1427f,-0.3749f,0.9160f,  0.1355f,-0.3629f,0.9219f,  0.0710f,-0.2571f,0.9638f,  0.0686f,-0.2419f,0.9679f,  0.0065f,-0.1292f,0.9916f,  0.0094f,-0.1111f,0.9938f,  -0.0490f,0.0059f,0.9988f,  -0.0401f,0.0261f,0.9989f,  -0.0938f,0.1446f,0.9850f,  -0.0786f,0.1662f,0.9830f,  -0.1269f,0.2835f,0.9506f,  -0.1053f,0.3058f,0.9462f,  
	-0.1478f,0.4189f,0.8959f,  -0.1199f,0.4410f,0.8894f,  -0.1566f,0.5472f,0.8222f,  -0.1228f,0.5683f,0.8136f,  -0.1542f,0.6648f,0.7309f,  -0.1151f,0.6839f,0.7204f,  -0.1420f,0.7684f,0.6240f,  -0.0984f,0.7849f,0.6118f,  -0.1219f,0.8552f,0.5038f,  -0.0749f,0.8685f,0.4899f,  -0.0963f,0.9229f,0.3727f,  -0.0474f,0.9326f,0.3577f,  -0.0681f,0.9699f,0.2338f,  -0.0188f,0.9758f,0.2179f,  -0.0403f,0.9951f,0.0902f,  
	0.0079f,0.9972f,0.0739f,  -0.0159f,0.9984f,-0.0548f,  0.0297f,0.9970f,-0.0711f,  0.0022f,0.9802f,-0.1980f,  0.0438f,0.9759f,-0.2136f,  0.0115f,0.9419f,-0.3357f,  0.0478f,0.9354f,-0.3503f,  0.0098f,0.8852f,-0.4650f,  0.0399f,0.8775f,-0.4780f,  -0.0043f,0.8127f,-0.5827f,  0.0189f,0.8046f,-0.5935f,  -0.0317f,0.7270f,-0.6859f,  -0.0158f,0.7196f,-0.6942f,  -0.0725f,0.6312f,-0.7722f,  -0.0640f,0.6256f,-0.7775f,  
	-0.1260f,0.5285f,-0.8395f,  -0.1246f,0.5255f,-0.8416f,  -0.1908f,0.4220f,-0.8863f,  -0.1959f,0.4223f,-0.8850f,  -0.2650f,0.3145f,-0.9115f,  -0.2759f,0.3188f,-0.9068f,  -0.3460f,0.2087f,-0.9147f,  -0.3617f,0.2173f,-0.9066f,  -0.4308f,0.1068f,-0.8961f,  -0.4505f,0.1198f,-0.8847f,  -0.5165f,0.0107f,-0.8562f,  -0.5389f,0.0280f,-0.8419f,  -0.5998f,-0.0783f,-0.7963f,  -0.6240f,-0.0570f,-0.7794f,  -0.6775f,-0.1593f,-0.7181f,  
	-0.7025f,-0.1342f,-0.6989f,  -0.7466f,-0.2315f,-0.6236f,  -0.7716f,-0.2034f,-0.6027f,  -0.8046f,-0.2949f,-0.5154f,  -0.8288f,-0.2644f,-0.4931f,  -0.8491f,-0.3495f,-0.3961f,  -0.8719f,-0.3174f,-0.3730f,  -0.8782f,-0.3958f,-0.2687f,  -0.8991f,-0.3626f,-0.2453f,  -0.8905f,-0.4341f,-0.1362f,  -0.9092f,-0.4008f,-0.1130f,  -0.8851f,-0.4653f,-0.0019f,  -0.9014f,-0.4325f,0.0208f,  -0.8617f,-0.4901f,0.1311f,  -0.8755f,-0.4585f,0.1528f,  
	-0.8204f,-0.5094f,0.2598f,  -0.8316f,-0.4796f,0.2801f,  -0.7616f,-0.5240f,0.3812f,  -0.7705f,-0.4966f,0.3997f,  -0.6866f,-0.5348f,0.4925f,  -0.6932f,-0.5103f,0.5089f,  -0.5968f,-0.5424f,0.5912f,  -0.6014f,-0.5214f,0.6054f,  -0.4941f,-0.5477f,0.6752f,  -0.4970f,-0.5304f,0.6868f,  -0.3806f,-0.5511f,0.7426f,  -0.3822f,-0.5379f,0.7514f,  -0.2587f,-0.5532f,0.7918f,  -0.2594f,-0.5443f,0.7978f,  -0.1312f,-0.5544f,0.8219f,  
	-0.1313f,-0.5500f,0.8248f,  -0.0008f,-0.5547f,0.8320f,  -0.0008f,-0.5549f,0.8319f,  0.4338f,-0.7020f,0.5648f,  0.4158f,-0.7043f,0.5754f,  0.3599f,-0.6427f,0.6763f,  0.3435f,-0.6412f,0.6862f,  0.2838f,-0.5654f,0.7745f,  0.2700f,-0.5601f,0.7832f,  0.2081f,-0.4716f,0.8569f,  0.1977f,-0.4625f,0.8643f,  0.1355f,-0.3629f,0.9219f,  0.1295f,-0.3504f,0.9276f,  0.0686f,-0.2419f,0.9679f,  0.0676f,-0.2262f,0.9717f,  
	0.0094f,-0.1111f,0.9938f,  0.0142f,-0.0929f,0.9956f,  -0.0401f,0.0261f,0.9989f,  -0.0291f,0.0461f,0.9985f,  -0.0786f,0.1662f,0.9830f,  -0.0610f,0.1875f,0.9804f,  -0.1053f,0.3058f,0.9462f,  -0.0811f,0.3273f,0.9414f,  -0.1199f,0.4410f,0.8894f,  -0.0892f,0.4620f,0.8824f,  -0.1228f,0.5683f,0.8136f,  -0.0862f,0.5877f,0.8045f,  -0.1151f,0.6839f,0.7204f,  -0.0732f,0.7012f,0.7092f,  -0.0984f,0.7849f,0.6118f,  
	-0.0524f,0.7992f,0.5987f,  -0.0749f,0.8685f,0.4899f,  -0.0260f,0.8794f,0.4754f,  -0.0474f,0.9326f,0.3577f,  0.0030f,0.9397f,0.3419f,  -0.0188f,0.9758f,0.2179f,  0.0316f,0.9790f,0.2014f,  0.0079f,0.9972f,0.0739f,  0.0567f,0.9968f,0.0570f,  0.0297f,0.9970f,-0.0711f,  0.0754f,0.9933f,-0.0877f,  0.0438f,0.9759f,-0.2136f,  0.0851f,0.9696f,-0.2296f,  0.0478f,0.9354f,-0.3503f,  0.0836f,0.9272f,-0.3651f,  
	0.0399f,0.8775f,-0.4780f,  0.0692f,0.8684f,-0.4910f,  0.0189f,0.8046f,-0.5935f,  0.0412f,0.7957f,-0.6043f,  -0.0158f,0.7196f,-0.6942f,  -0.0008f,0.7119f,-0.7022f,  -0.0640f,0.6256f,-0.7775f,  -0.0561f,0.6200f,-0.7826f,  -0.1246f,0.5255f,-0.8416f,  -0.1235f,0.5230f,-0.8434f,  -0.1959f,0.4223f,-0.8850f,  -0.2011f,0.4235f,-0.8833f,  -0.2759f,0.3188f,-0.9068f,  -0.2864f,0.3241f,-0.9016f,  -0.3617f,0.2173f,-0.9066f,  
	-0.3767f,0.2269f,-0.8981f,  -0.4505f,0.1198f,-0.8847f,  -0.4689f,0.1339f,-0.8731f,  -0.5389f,0.0280f,-0.8419f,  -0.5598f,0.0463f,-0.8273f,  -0.6240f,-0.0570f,-0.7794f,  -0.6463f,-0.0347f,-0.7623f,  -0.7025f,-0.1342f,-0.6989f,  -0.7254f,-0.1085f,-0.6797f,  -0.7716f,-0.2034f,-0.6027f,  -0.7943f,-0.1748f,-0.5818f,  -0.8288f,-0.2644f,-0.4931f,  -0.8507f,-0.2335f,-0.4710f,  -0.8719f,-0.3174f,-0.3730f,  -0.8923f,-0.2849f,-0.3501f,  
	-0.8991f,-0.3626f,-0.2453f,  -0.9177f,-0.3294f,-0.2221f,  -0.9092f,-0.4008f,-0.1130f,  -0.9257f,-0.3674f,-0.0899f,  -0.9014f,-0.4325f,0.0208f,  -0.9156f,-0.3997f,0.0432f,  -0.8755f,-0.4585f,0.1528f,  -0.8873f,-0.4270f,0.1742f,  -0.8316f,-0.4796f,0.2801f,  -0.8411f,-0.4500f,0.3001f,  -0.7705f,-0.4966f,0.3997f,  -0.7777f,-0.4695f,0.4179f,  -0.6932f,-0.5103f,0.5089f,  -0.6985f,-0.4862f,0.5251f,  -0.6014f,-0.5214f,0.6054f,  
	-0.6049f,-0.5006f,0.6192f,  -0.4970f,-0.5304f,0.6868f,  -0.4990f,-0.5134f,0.6981f,  -0.3822f,-0.5379f,0.7514f,  -0.3831f,-0.5250f,0.7600f,  -0.2594f,-0.5443f,0.7978f,  -0.2596f,-0.5357f,0.8035f,  -0.1313f,-0.5500f,0.8248f,  -0.1312f,-0.5458f,0.8276f,  -0.0008f,-0.5549f,0.8319f,  -0.0009f,-0.5553f,0.8317f,  0.4158f,-0.7043f,0.5754f,  0.3974f,-0.7060f,0.5862f,  0.3435f,-0.6412f,0.6862f,  0.3272f,-0.6391f,0.6961f,  
	0.2700f,-0.5601f,0.7832f,  0.2566f,-0.5542f,0.7919f,  0.1977f,-0.4625f,0.8643f,  0.1882f,-0.4529f,0.8715f,  0.1295f,-0.3504f,0.9276f,  0.1247f,-0.3373f,0.9331f,  0.0676f,-0.2262f,0.9717f,  0.0683f,-0.2103f,0.9753f,  0.0142f,-0.0929f,0.9956f,  0.0210f,-0.0747f,0.9970f,  -0.0291f,0.0461f,0.9985f,  -0.0157f,0.0660f,0.9977f,  -0.0610f,0.1875f,0.9804f,  -0.0408f,0.2081f,0.9773f,  -0.0811f,0.3273f,0.9414f,  
	-0.0541f,0.3479f,0.9360f,  -0.0892f,0.4620f,0.8824f,  -0.0558f,0.4815f,0.8747f,  -0.0862f,0.5877f,0.8045f,  -0.0469f,0.6054f,0.7946f,  -0.0732f,0.7012f,0.7092f,  -0.0289f,0.7162f,0.6973f,  -0.0524f,0.7992f,0.5987f,  -0.0042f,0.8110f,0.5850f,  -0.0260f,0.8794f,0.4754f,  0.0247f,0.8875f,0.4601f,  0.0030f,0.9397f,0.3419f,  0.0547f,0.9440f,0.3255f,  0.0316f,0.9790f,0.2014f,  0.0827f,0.9794f,0.1842f,  
	0.0567f,0.9968f,0.0570f,  0.1058f,0.9936f,0.0396f,  0.0754f,0.9933f,-0.0877f,  0.1209f,0.9871f,-0.1049f,  0.0851f,0.9696f,-0.2296f,  0.1258f,0.9611f,-0.2459f,  0.0836f,0.9272f,-0.3651f,  0.1184f,0.9174f,-0.3800f,  0.0692f,0.8684f,-0.4910f,  0.0975f,0.8582f,-0.5040f,  0.0412f,0.7957f,-0.6043f,  0.0624f,0.7861f,-0.6149f,  -0.0008f,0.7119f,-0.7022f,  0.0133f,0.7040f,-0.7101f,  -0.0561f,0.6200f,-0.7826f,  
	-0.0489f,0.6146f,-0.7873f,  -0.1235f,0.5230f,-0.8434f,  -0.1228f,0.5209f,-0.8447f,  -0.2011f,0.4235f,-0.8833f,  -0.2062f,0.4253f,-0.8813f,  -0.2864f,0.3241f,-0.9016f,  -0.2965f,0.3302f,-0.8962f,  -0.3767f,0.2269f,-0.8981f,  -0.3908f,0.2375f,-0.8893f,  -0.4689f,0.1339f,-0.8731f,  -0.4861f,0.1488f,-0.8612f,  -0.5598f,0.0463f,-0.8273f,  -0.5791f,0.0655f,-0.8126f,  -0.6463f,-0.0347f,-0.7623f,  -0.6668f,-0.0117f,-0.7451f,  
	-0.7254f,-0.1085f,-0.6797f,  -0.7463f,-0.0821f,-0.6605f,  -0.7943f,-0.1748f,-0.5818f,  -0.8149f,-0.1457f,-0.5609f,  -0.8507f,-0.2335f,-0.4710f,  -0.8703f,-0.2023f,-0.4490f,  -0.8923f,-0.2849f,-0.3501f,  -0.9106f,-0.2523f,-0.3274f,  -0.9177f,-0.3294f,-0.2221f,  -0.9342f,-0.2961f,-0.1991f,  -0.9257f,-0.3674f,-0.0899f,  -0.9401f,-0.3341f,-0.0672f,  -0.9156f,-0.3997f,0.0432f,  -0.9279f,-0.3671f,0.0653f,  -0.8873f,-0.4270f,0.1742f,  
	-0.8973f,-0.3958f,0.1953f,  -0.8411f,-0.4500f,0.3001f,  -0.8490f,-0.4207f,0.3198f,  -0.7777f,-0.4695f,0.4179f,  -0.7836f,-0.4427f,0.4360f,  -0.6985f,-0.4862f,0.5251f,  -0.7025f,-0.4623f,0.5411f,  -0.6049f,-0.5006f,0.6192f,  -0.6074f,-0.4801f,0.6329f,  -0.4990f,-0.5134f,0.6981f,  -0.5002f,-0.4967f,0.7093f,  -0.3831f,-0.5250f,0.7600f,  -0.3834f,-0.5124f,0.7684f,  -0.2596f,-0.5357f,0.8035f,  -0.2594f,-0.5274f,0.8090f,  
	-0.1312f,-0.5458f,0.8276f,  -0.1310f,-0.5420f,0.8301f,  -0.0009f,-0.5553f,0.8317f,  -0.0010f,-0.5560f,0.8312f,  0.3974f,-0.7060f,0.5862f,  0.3788f,-0.7071f,0.5971f,  0.3272f,-0.6391f,0.6961f,  0.3110f,-0.6363f,0.7059f,  0.2566f,-0.5542f,0.7919f,  0.2437f,-0.5476f,0.8004f,  0.1882f,-0.4529f,0.8715f,  0.1796f,-0.4427f,0.8785f,  0.1247f,-0.3373f,0.9331f,  0.1213f,-0.3240f,0.9382f,  0.0683f,-0.2103f,0.9753f,  
	0.0708f,-0.1943f,0.9784f,  0.0210f,-0.0747f,0.9970f,  0.0300f,-0.0567f,0.9979f,  -0.0157f,0.0660f,0.9977f,  0.0001f,0.0852f,0.9964f,  -0.0408f,0.2081f,0.9773f,  -0.0180f,0.2278f,0.9735f,  -0.0541f,0.3479f,0.9360f,  -0.0244f,0.3670f,0.9299f,  -0.0558f,0.4815f,0.8747f,  -0.0197f,0.4992f,0.8662f,  -0.0469f,0.6054f,0.7946f,  -0.0050f,0.6209f,0.7839f,  -0.0289f,0.7162f,0.6973f,  0.0177f,0.7288f,0.6845f,  
	-0.0042f,0.8110f,0.5850f,  0.0459f,0.8201f,0.5704f,  0.0247f,0.8875f,0.4601f,  0.0768f,0.8927f,0.4440f,  0.0547f,0.9440f,0.3255f,  0.1073f,0.9452f,0.3082f,  0.0827f,0.9794f,0.1842f,  0.1343f,0.9769f,0.1663f,  0.1058f,0.9936f,0.0396f,  0.1547f,0.9877f,0.0216f,  0.1209f,0.9871f,-0.1049f,  0.1659f,0.9785f,-0.1225f,  0.1258f,0.9611f,-0.2459f,  0.1657f,0.9506f,-0.2625f,  0.1184f,0.9174f,-0.3800f,  
	0.1522f,0.9059f,-0.3951f,  0.0975f,0.8582f,-0.5040f,  0.1246f,0.8468f,-0.5171f,  0.0624f,0.7861f,-0.6149f,  0.0826f,0.7759f,-0.6255f,  0.0133f,0.7040f,-0.7101f,  0.0265f,0.6958f,-0.7177f,  -0.0489f,0.6146f,-0.7873f,  -0.0424f,0.6094f,-0.7917f,  -0.1228f,0.5209f,-0.8447f,  -0.1224f,0.5193f,-0.8458f,  -0.2062f,0.4253f,-0.8813f,  -0.2112f,0.4278f,-0.8789f,  -0.2965f,0.3302f,-0.8962f,  -0.3060f,0.3370f,-0.8904f,  
	-0.3908f,0.2375f,-0.8893f,  -0.4040f,0.2488f,-0.8803f,  -0.4861f,0.1488f,-0.8612f,  -0.5020f,0.1646f,-0.8491f,  -0.5791f,0.0655f,-0.8126f,  -0.5968f,0.0854f,-0.7978f,  -0.6668f,-0.0117f,-0.7451f,  -0.6856f,0.0119f,-0.7279f,  -0.7463f,-0.0821f,-0.6605f,  -0.7653f,-0.0554f,-0.6413f,  -0.8149f,-0.1457f,-0.5609f,  -0.8335f,-0.1163f,-0.5402f,  -0.8703f,-0.2023f,-0.4490f,  -0.8879f,-0.1710f,-0.4271f,  -0.9106f,-0.2523f,-0.3274f,  
	-0.9267f,-0.2197f,-0.3049f,  -0.9342f,-0.2961f,-0.1991f,  -0.9486f,-0.2628f,-0.1764f,  -0.9401f,-0.3341f,-0.0672f,  -0.9526f,-0.3010f,-0.0446f,  -0.9279f,-0.3671f,0.0653f,  -0.9383f,-0.3347f,0.0873f,  -0.8973f,-0.3958f,0.1953f,  -0.9057f,-0.3647f,0.2162f,  -0.8490f,-0.4207f,0.3198f,  -0.8553f,-0.3916f,0.3393f,  -0.7836f,-0.4427f,0.4360f,  -0.7880f,-0.4161f,0.4538f,  -0.7025f,-0.4623f,0.5411f,  -0.7053f,-0.4387f,0.5569f,  
	-0.6074f,-0.4801f,0.6329f,  -0.6088f,-0.4599f,0.6464f,  -0.5002f,-0.4967f,0.7093f,  -0.5006f,-0.4803f,0.7202f,  -0.3834f,-0.5124f,0.7684f,  -0.3831f,-0.5000f,0.7767f,  -0.2594f,-0.5274f,0.8090f,  -0.2588f,-0.5194f,0.8144f,  -0.1310f,-0.5420f,0.8301f,  -0.1306f,-0.5384f,0.8325f,  -0.0010f,-0.5560f,0.8312f,  -0.0012f,-0.5570f,0.8305f,  0.3788f,-0.7071f,0.5971f,  0.3601f,-0.7076f,0.6079f,  0.3110f,-0.6363f,0.7059f,  
	0.2950f,-0.6330f,0.7157f,  0.2437f,-0.5476f,0.8004f,  0.2315f,-0.5405f,0.8088f,  0.1796f,-0.4427f,0.8785f,  0.1721f,-0.4322f,0.8852f,  0.1213f,-0.3240f,0.9382f,  0.1193f,-0.3105f,0.9430f,  0.0708f,-0.1943f,0.9784f,  0.0751f,-0.1783f,0.9811f,  0.0300f,-0.0567f,0.9979f,  0.0411f,-0.0390f,0.9984f,  0.0001f,0.0852f,0.9964f,  0.0184f,0.1038f,0.9944f,  -0.0180f,0.2278f,0.9735f,  0.0074f,0.2463f,0.9692f,  
	-0.0244f,0.3670f,0.9299f,  0.0079f,0.3845f,0.9231f,  -0.0197f,0.4992f,0.8662f,  0.0190f,0.5150f,0.8570f,  -0.0050f,0.6209f,0.7839f,  0.0392f,0.6340f,0.7723f,  0.0177f,0.7288f,0.6845f,  0.0663f,0.7386f,0.6709f,  0.0459f,0.8201f,0.5704f,  0.0975f,0.8262f,0.5549f,  0.0768f,0.8927f,0.4440f,  0.1300f,0.8949f,0.4270f,  0.1073f,0.9452f,0.3082f,  0.1605f,0.9434f,0.2902f,  0.1343f,0.9769f,0.1663f,  
	0.1859f,0.9714f,0.1478f,  0.1547f,0.9877f,0.0216f,  0.2032f,0.9791f,0.0030f,  0.1659f,0.9785f,-0.1225f,  0.2101f,0.9675f,-0.1405f,  0.1657f,0.9506f,-0.2625f,  0.2044f,0.9381f,-0.2795f,  0.1522f,0.9059f,-0.3951f,  0.1848f,0.8930f,-0.4104f,  0.1246f,0.8468f,-0.5171f,  0.1505f,0.8344f,-0.5301f,  0.0826f,0.7759f,-0.6255f,  0.1016f,0.7651f,-0.6359f,  0.0265f,0.6958f,-0.7177f,  0.0388f,0.6875f,-0.7252f,  
	-0.0424f,0.6094f,-0.7917f,  -0.0365f,0.6043f,-0.7959f,  -0.1224f,0.5193f,-0.8458f,  -0.1222f,0.5180f,-0.8466f,  -0.2112f,0.4278f,-0.8789f,  -0.2160f,0.4308f,-0.8762f,  -0.3060f,0.3370f,-0.8904f,  -0.3150f,0.3445f,-0.8843f,  -0.4040f,0.2488f,-0.8803f,  -0.4163f,0.2608f,-0.8710f,  -0.5020f,0.1646f,-0.8491f,  -0.5166f,0.1809f,-0.8369f,  -0.5968f,0.0854f,-0.7978f,  -0.6131f,0.1057f,-0.7829f,  -0.6856f,0.0119f,-0.7279f,  
	-0.7025f,0.0359f,-0.7107f,  -0.7653f,-0.0554f,-0.6413f,  -0.7823f,-0.0283f,-0.6222f,  -0.8335f,-0.1163f,-0.5402f,  -0.8500f,-0.0867f,-0.5196f,  -0.8879f,-0.1710f,-0.4271f,  -0.9034f,-0.1395f,-0.4055f,  -0.9267f,-0.2197f,-0.3049f,  -0.9408f,-0.1870f,-0.2826f,  -0.9486f,-0.2628f,-0.1764f,  -0.9610f,-0.2297f,-0.1538f,  -0.9526f,-0.3010f,-0.0446f,  -0.9632f,-0.2679f,-0.0223f,  -0.9383f,-0.3347f,0.0873f,  -0.9469f,-0.3025f,0.1090f,  
	-0.9057f,-0.3647f,0.2162f,  -0.9124f,-0.3338f,0.2369f,  -0.8553f,-0.3916f,0.3393f,  -0.8601f,-0.3627f,0.3586f,  -0.7880f,-0.4161f,0.4538f,  -0.7912f,-0.3897f,0.4714f,  -0.7053f,-0.4387f,0.5569f,  -0.7070f,-0.4153f,0.5725f,  -0.6088f,-0.4599f,0.6464f,  -0.6093f,-0.4400f,0.6597f,  -0.5006f,-0.4803f,0.7202f,  -0.5002f,-0.4641f,0.7310f,  -0.3831f,-0.5000f,0.7767f,  -0.3823f,-0.4879f,0.7847f,  -0.2588f,-0.5194f,0.8144f,  
	-0.2579f,-0.5116f,0.8196f,  -0.1306f,-0.5384f,0.8325f,  -0.1301f,-0.5351f,0.8347f,  -0.0012f,-0.5570f,0.8305f,  -0.0015f,-0.5582f,0.8297f,  0.3601f,-0.7076f,0.6079f,  0.3413f,-0.7075f,0.6188f,  0.2950f,-0.6330f,0.7157f,  0.2794f,-0.6291f,0.7254f,  0.2315f,-0.5405f,0.8088f,  0.2201f,-0.5330f,0.8170f,  0.1721f,-0.4322f,0.8852f,  0.1658f,-0.4214f,0.8916f,  0.1193f,-0.3105f,0.9430f,  0.1189f,-0.2969f,0.9475f,  
	0.0751f,-0.1783f,0.9811f,  0.0814f,-0.1626f,0.9833f,  0.0411f,-0.0390f,0.9984f,  0.0545f,-0.0219f,0.9983f,  0.0184f,0.1038f,0.9944f,  0.0390f,0.1213f,0.9918f,  0.0074f,0.2463f,0.9692f,  0.0353f,0.2633f,0.9641f,  0.0079f,0.3845f,0.9231f,  0.0427f,0.4002f,0.9154f,  0.0190f,0.5150f,0.8570f,  0.0601f,0.5284f,0.8469f,  0.0392f,0.6340f,0.7723f,  0.0855f,0.6445f,0.7599f,  0.0663f,0.7386f,0.6709f,  
	0.1165f,0.7455f,0.6563f,  0.0975f,0.8262f,0.5549f,  0.1504f,0.8292f,0.5384f,  0.1300f,0.8949f,0.4270f,  0.1839f,0.8938f,0.4091f,  0.1605f,0.9434f,0.2902f,  0.2138f,0.9384f,0.2714f,  0.1859f,0.9714f,0.1478f,  0.2370f,0.9630f,0.1285f,  0.2032f,0.9791f,0.0030f,  0.2509f,0.9679f,-0.0161f,  0.2101f,0.9675f,-0.1405f,  0.2531f,0.9543f,-0.1590f,  0.2044f,0.9381f,-0.2795f,  0.2417f,0.9239f,-0.2967f,  
	0.1848f,0.8930f,-0.4104f,  0.2158f,0.8787f,-0.4257f,  0.1505f,0.8344f,-0.5301f,  0.1749f,0.8212f,-0.5432f,  0.1016f,0.7651f,-0.6359f,  0.1193f,0.7538f,-0.6462f,  0.0388f,0.6875f,-0.7252f,  0.0501f,0.6791f,-0.7323f,  -0.0365f,0.6043f,-0.7959f,  -0.0311f,0.5995f,-0.7998f,  -0.1222f,0.5180f,-0.8466f,  -0.1222f,0.5172f,-0.8471f,  -0.2160f,0.4308f,-0.8762f,  -0.2206f,0.4344f,-0.8733f,  -0.3150f,0.3445f,-0.8843f,  
	-0.3234f,0.3526f,-0.8781f,  -0.4163f,0.2608f,-0.8710f,  -0.4276f,0.2734f,-0.8616f,  -0.5166f,0.1809f,-0.8369f,  -0.5301f,0.1978f,-0.8246f,  -0.6131f,0.1057f,-0.7829f,  -0.6278f,0.1266f,-0.7680f,  -0.7025f,0.0359f,-0.7107f,  -0.7179f,0.0602f,-0.6936f,  -0.7823f,-0.0283f,-0.6222f,  -0.7976f,-0.0010f,-0.6032f,  -0.8500f,-0.0867f,-0.5196f,  -0.8647f,-0.0570f,-0.4991f,  -0.9034f,-0.1395f,-0.4055f,  -0.9170f,-0.1080f,-0.3839f,  
	-0.9408f,-0.1870f,-0.2826f,  -0.9531f,-0.1544f,-0.2604f,  -0.9610f,-0.2297f,-0.1538f,  -0.9716f,-0.1966f,-0.1315f,  -0.9632f,-0.2679f,-0.0223f,  -0.9720f,-0.2350f,-0.0002f,  -0.9469f,-0.3025f,0.1090f,  -0.9539f,-0.2704f,0.1305f,  -0.9124f,-0.3338f,0.2369f,  -0.9175f,-0.3032f,0.2574f,  -0.8601f,-0.3627f,0.3586f,  -0.8635f,-0.3341f,0.3778f,  -0.7912f,-0.3897f,0.4714f,  -0.7930f,-0.3635f,0.4888f,  -0.7070f,-0.4153f,0.5725f,  
	-0.7075f,-0.3921f,0.5879f,  -0.6093f,-0.4400f,0.6597f,  -0.6088f,-0.4202f,0.6729f,  -0.5002f,-0.4641f,0.7310f,  -0.4991f,-0.4481f,0.7416f,  -0.3823f,-0.4879f,0.7847f,  -0.3809f,-0.4761f,0.7926f,  -0.2579f,-0.5116f,0.8196f,  -0.2567f,-0.5041f,0.8246f,  -0.1301f,-0.5351f,0.8347f,  -0.1294f,-0.5321f,0.8368f,  -0.0015f,-0.5582f,0.8297f,  -0.0020f,-0.5598f,0.8286f,  0.3413f,-0.7075f,0.6188f,  0.3226f,-0.7069f,0.6295f,  
	0.2794f,-0.6291f,0.7254f,  0.2642f,-0.6247f,0.7348f,  0.2201f,-0.5330f,0.8170f,  0.2095f,-0.5251f,0.8249f,  0.1658f,-0.4214f,0.8916f,  0.1608f,-0.4104f,0.8976f,  0.1189f,-0.2969f,0.9475f,  0.1203f,-0.2833f,0.9514f,  0.0814f,-0.1626f,0.9833f,  0.0897f,-0.1472f,0.9850f,  0.0545f,-0.0219f,0.9983f,  0.0701f,-0.0057f,0.9975f,  0.0390f,0.1213f,0.9918f,  0.0622f,0.1375f,0.9885f,  0.0353f,0.2633f,0.9641f,  
	0.0657f,0.2786f,0.9581f,  0.0427f,0.4002f,0.9154f,  0.0799f,0.4137f,0.9069f,  0.0601f,0.5284f,0.8469f,  0.1032f,0.5392f,0.8358f,  0.0855f,0.6445f,0.7599f,  0.1335f,0.6520f,0.7464f,  0.1165f,0.7455f,0.6563f,  0.1681f,0.7492f,0.6406f,  0.1504f,0.8292f,0.5384f,  0.2040f,0.8288f,0.5211f,  0.1839f,0.8938f,0.4091f,  0.2380f,0.8894f,0.3904f,  0.2138f,0.9384f,0.2714f,  0.2667f,0.9303f,0.2517f,  
	0.2370f,0.9630f,0.1285f,  0.2874f,0.9516f,0.1085f,  0.2509f,0.9679f,-0.0161f,  0.2974f,0.9541f,-0.0358f,  0.2531f,0.9543f,-0.1590f,  0.2946f,0.9389f,-0.1778f,  0.2417f,0.9239f,-0.2967f,  0.2775f,0.9080f,-0.3140f,  0.2158f,0.8787f,-0.4257f,  0.2453f,0.8633f,-0.4411f,  0.1749f,0.8212f,-0.5432f,  0.1979f,0.8072f,-0.5561f,  0.1193f,0.7538f,-0.6462f,  0.1359f,0.7422f,-0.6563f,  0.0501f,0.6791f,-0.7323f,  
	0.0605f,0.6706f,-0.7393f,  -0.0311f,0.5995f,-0.7998f,  -0.0264f,0.5948f,-0.8034f,  -0.1222f,0.5172f,-0.8471f,  -0.1224f,0.5168f,-0.8473f,  -0.2206f,0.4344f,-0.8733f,  -0.2250f,0.4384f,-0.8701f,  -0.3234f,0.3526f,-0.8781f,  -0.3312f,0.3612f,-0.8717f,  -0.4276f,0.2734f,-0.8616f,  -0.4380f,0.2865f,-0.8521f,  -0.5301f,0.1978f,-0.8246f,  -0.5423f,0.2151f,-0.8122f,  -0.6278f,0.1266f,-0.7680f,  -0.6411f,0.1477f,-0.7531f,  
	-0.7179f,0.0602f,-0.6936f,  -0.7316f,0.0848f,-0.6764f,  -0.7976f,-0.0010f,-0.6032f,  -0.8112f,0.0265f,-0.5842f,  -0.8647f,-0.0570f,-0.4991f,  -0.8775f,-0.0272f,-0.4787f,  -0.9170f,-0.1080f,-0.3839f,  -0.9288f,-0.0766f,-0.3625f,  -0.9531f,-0.1544f,-0.2604f,  -0.9635f,-0.1219f,-0.2384f,  -0.9716f,-0.1966f,-0.1315f,  -0.9804f,-0.1636f,-0.1093f,  -0.9720f,-0.2350f,-0.0002f,  -0.9791f,-0.2023f,0.0218f,  -0.9539f,-0.2704f,0.1305f,  
	-0.9592f,-0.2385f,0.1518f,  -0.9175f,-0.3032f,0.2574f,  -0.9211f,-0.2727f,0.2778f,  -0.8635f,-0.3341f,0.3778f,  -0.8656f,-0.3056f,0.3968f,  -0.7930f,-0.3635f,0.4888f,  -0.7937f,-0.3376f,0.5061f,  -0.7075f,-0.3921f,0.5879f,  -0.7070f,-0.3692f,0.6032f,  -0.6088f,-0.4202f,0.6729f,  -0.6075f,-0.4007f,0.6859f,  -0.4991f,-0.4481f,0.7416f,  -0.4973f,-0.4324f,0.7522f,  -0.3809f,-0.4761f,0.7926f,  -0.3790f,-0.4644f,0.8004f,  
	-0.2567f,-0.5041f,0.8246f,  -0.2552f,-0.4968f,0.8295f,  -0.1294f,-0.5321f,0.8368f,  -0.1288f,-0.5293f,0.8386f,  -0.0020f,-0.5598f,0.8286f,  -0.0026f,-0.5616f,0.8274f,  0.3226f,-0.7069f,0.6295f,  0.3039f,-0.7056f,0.6401f,  0.2642f,-0.6247f,0.7348f,  0.2496f,-0.6198f,0.7440f,  0.2095f,-0.5251f,0.8249f,  0.1999f,-0.5168f,0.8325f,  0.1608f,-0.4104f,0.8976f,  0.1571f,-0.3992f,0.9033f,  0.1203f,-0.2833f,0.9514f,  
	0.1233f,-0.2700f,0.9549f,  0.0897f,-0.1472f,0.9850f,  0.1000f,-0.1326f,0.9861f,  0.0701f,-0.0057f,0.9975f,  0.0880f,0.0095f,0.9961f,  0.0622f,0.1375f,0.9885f,  0.0876f,0.1523f,0.9844f,  0.0657f,0.2786f,0.9581f,  0.0984f,0.2920f,0.9514f,  0.0799f,0.4137f,0.9069f,  0.1192f,0.4248f,0.8974f,  0.1032f,0.5392f,0.8358f,  0.1482f,0.5473f,0.8237f,  0.1335f,0.6520f,0.7464f,  0.1830f,0.6564f,0.7319f,  
	0.1681f,0.7492f,0.6406f,  0.2206f,0.7496f,0.6241f,  0.2040f,0.8288f,0.5211f,  0.2580f,0.8251f,0.5027f,  0.2380f,0.8894f,0.3904f,  0.2918f,0.8817f,0.3707f,  0.2667f,0.9303f,0.2517f,  0.3190f,0.9191f,0.2314f,  0.2874f,0.9516f,0.1085f,  0.3367f,0.9375f,0.0880f,  0.2974f,0.9541f,-0.0358f,  0.3424f,0.9379f,-0.0560f,  0.2946f,0.9389f,-0.1778f,  0.3345f,0.9216f,-0.1969f,  0.2775f,0.9080f,-0.3140f,  
	0.3115f,0.8905f,-0.3316f,  0.2453f,0.8633f,-0.4411f,  0.2731f,0.8468f,-0.4565f,  0.1979f,0.8072f,-0.5561f,  0.2193f,0.7926f,-0.5690f,  0.1359f,0.7422f,-0.6563f,  0.1511f,0.7303f,-0.6662f,  0.0605f,0.6706f,-0.7393f,  0.0700f,0.6622f,-0.7461f,  -0.0264f,0.5948f,-0.8034f,  -0.0221f,0.5904f,-0.8068f,  -0.1224f,0.5168f,-0.8473f,  -0.1228f,0.5167f,-0.8473f,  -0.2250f,0.4384f,-0.8701f,  -0.2292f,0.4429f,-0.8668f,  
	-0.3312f,0.3612f,-0.8717f,  -0.3385f,0.3702f,-0.8651f,  -0.4380f,0.2865f,-0.8521f,  -0.4476f,0.2999f,-0.8425f,  -0.5423f,0.2151f,-0.8122f,  -0.5534f,0.2327f,-0.7997f,  -0.6411f,0.1477f,-0.7531f,  -0.6531f,0.1692f,-0.7382f,  -0.7316f,0.0848f,-0.6764f,  -0.7438f,0.1096f,-0.6594f,  -0.8112f,0.0265f,-0.5842f,  -0.8231f,0.0541f,-0.5654f,  -0.8775f,-0.0272f,-0.4787f,  -0.8887f,0.0026f,-0.4585f,  -0.9288f,-0.0766f,-0.3625f,  
	-0.9389f,-0.0451f,-0.3413f,  -0.9635f,-0.1219f,-0.2384f,  -0.9722f,-0.0894f,-0.2166f,  -0.9804f,-0.1636f,-0.1093f,  -0.9876f,-0.1308f,-0.0873f,  -0.9791f,-0.2023f,0.0218f,  -0.9845f,-0.1697f,0.0436f,  -0.9592f,-0.2385f,0.1518f,  -0.9630f,-0.2067f,0.1730f,  -0.9211f,-0.2727f,0.2778f,  -0.9233f,-0.2424f,0.2980f,  -0.8656f,-0.3056f,0.3968f,  -0.8662f,-0.2773f,0.4156f,  -0.7937f,-0.3376f,0.5061f,  -0.7931f,-0.3118f,0.5233f,  
	-0.7070f,-0.3692f,0.6032f,  -0.7054f,-0.3464f,0.6184f,  -0.6075f,-0.4007f,0.6859f,  -0.6052f,-0.3814f,0.6988f,  -0.4973f,-0.4324f,0.7522f,  -0.4947f,-0.4169f,0.7625f,  -0.3790f,-0.4644f,0.8004f,  -0.3765f,-0.4531f,0.8081f,  -0.2552f,-0.4968f,0.8295f,  -0.2534f,-0.4898f,0.8342f,  -0.1288f,-0.5293f,0.8386f,  -0.1280f,-0.5268f,0.8403f,  -0.0026f,-0.5616f,0.8274f,  -0.0035f,-0.5637f,0.8260f,  0.3039f,-0.7056f,0.6401f,  
	0.2854f,-0.7038f,0.6505f,  0.2496f,-0.6198f,0.7440f,  0.2356f,-0.6144f,0.7530f,  0.1999f,-0.5168f,0.8325f,  0.1913f,-0.5083f,0.8397f,  0.1571f,-0.3992f,0.9033f,  0.1549f,-0.3881f,0.9085f,  0.1233f,-0.2700f,0.9549f,  0.1282f,-0.2572f,0.9578f,  0.1000f,-0.1326f,0.9861f,  0.1123f,-0.1188f,0.9865f,  0.0880f,0.0095f,0.9961f,  0.1081f,0.0234f,0.9939f,  0.0876f,0.1523f,0.9844f,  0.1153f,0.1652f,0.9795f,  
	0.0984f,0.2920f,0.9514f,  0.1332f,0.3031f,0.9436f,  0.1192f,0.4248f,0.8974f,  0.1604f,0.4332f,0.8869f,  0.1482f,0.5473f,0.8237f,  0.1947f,0.5523f,0.8106f,  0.1830f,0.6564f,0.7319f,  0.2334f,0.6575f,0.7164f,  0.2206f,0.7496f,0.6241f,  0.2736f,0.7466f,0.6064f,  0.2580f,0.8251f,0.5027f,  0.3119f,0.8180f,0.4834f,  0.2918f,0.8817f,0.3707f,  0.3451f,0.8708f,0.3503f,  0.3190f,0.9191f,0.2314f,  
	0.3701f,0.9049f,0.2103f,  0.3367f,0.9375f,0.0880f,  0.3844f,0.9207f,0.0669f,  0.3424f,0.9379f,-0.0560f,  0.3857f,0.9195f,-0.0765f,  0.3345f,0.9216f,-0.1969f,  0.3724f,0.9025f,-0.2163f,  0.3115f,0.8905f,-0.3316f,  0.3436f,0.8718f,-0.3492f,  0.2731f,0.8468f,-0.4565f,  0.2991f,0.8294f,-0.4719f,  0.2193f,0.7926f,-0.5690f,  0.2392f,0.7775f,-0.5817f,  0.1511f,0.7303f,-0.6662f,  0.1652f,0.7182f,-0.6759f,  
	0.0700f,0.6622f,-0.7461f,  0.0786f,0.6538f,-0.7526f,  -0.0221f,0.5904f,-0.8068f,  -0.0184f,0.5861f,-0.8100f,  -0.1228f,0.5167f,-0.8473f,  -0.1232f,0.5169f,-0.8471f,  -0.2292f,0.4429f,-0.8668f,  -0.2331f,0.4477f,-0.8633f,  -0.3385f,0.3702f,-0.8651f,  -0.3451f,0.3796f,-0.8583f,  -0.4476f,0.2999f,-0.8425f,  -0.4562f,0.3137f,-0.8327f,  -0.5534f,0.2327f,-0.7997f,  -0.5634f,0.2506f,-0.7873f,  -0.6531f,0.1692f,-0.7382f,  
	-0.6637f,0.1908f,-0.7232f,  -0.7438f,0.1096f,-0.6594f,  -0.7545f,0.1345f,-0.6423f,  -0.8231f,0.0541f,-0.5654f,  -0.8334f,0.0818f,-0.5466f,  -0.8887f,0.0026f,-0.4585f,  -0.8982f,0.0325f,-0.4383f,  -0.9389f,-0.0451f,-0.3413f,  -0.9473f,-0.0137f,-0.3201f,  -0.9722f,-0.0894f,-0.2166f,  -0.9792f,-0.0570f,-0.1948f,  -0.9876f,-0.1308f,-0.0873f,  -0.9930f,-0.0980f,-0.0654f,  -0.9845f,-0.1697f,0.0436f,  -0.9884f,-0.1372f,0.0653f,  
	-0.9630f,-0.2067f,0.1730f,  -0.9652f,-0.1751f,0.1941f,  -0.9233f,-0.2424f,0.2980f,  -0.9240f,-0.2123f,0.3181f,  -0.8662f,-0.2773f,0.4156f,  -0.8656f,-0.2491f,0.4344f,  -0.7931f,-0.3118f,0.5233f,  -0.7913f,-0.2862f,0.5404f,  -0.7054f,-0.3464f,0.6184f,  -0.7027f,-0.3238f,0.6335f,  -0.6052f,-0.3814f,0.6988f,  -0.6020f,-0.3622f,0.7116f,  -0.4947f,-0.4169f,0.7625f,  -0.4915f,-0.4016f,0.7728f,  -0.3765f,-0.4531f,0.8081f,  
	-0.3736f,-0.4419f,0.8156f,  -0.2534f,-0.4898f,0.8342f,  -0.2513f,-0.4830f,0.8388f,  -0.1280f,-0.5268f,0.8403f,  -0.1273f,-0.5246f,0.8418f,  -0.0035f,-0.5637f,0.8260f,  -0.0046f,-0.5661f,0.8243f,  0.2854f,-0.7038f,0.6505f,  0.2672f,-0.7014f,0.6607f,  0.2356f,-0.6144f,0.7530f,  0.2222f,-0.6086f,0.7617f,  0.1913f,-0.5083f,0.8397f,  0.1839f,-0.4996f,0.8465f,  0.1549f,-0.3881f,0.9085f,  0.1542f,-0.3773f,0.9132f,  
	0.1282f,-0.2572f,0.9578f,  0.1348f,-0.2449f,0.9601f,  0.1123f,-0.1188f,0.9865f,  0.1267f,-0.1060f,0.9863f,  0.1081f,0.0234f,0.9939f,  0.1302f,0.0357f,0.9908f,  0.1153f,0.1652f,0.9795f,  0.1449f,0.1762f,0.9736f,  0.1332f,0.3031f,0.9436f,  0.1698f,0.3118f,0.9349f,  0.1604f,0.4332f,0.8869f,  0.2031f,0.4388f,0.8753f,  0.1947f,0.5523f,0.8106f,  0.2422f,0.5542f,0.7964f,  0.2334f,0.6575f,0.7164f,  
	0.2844f,0.6553f,0.6998f,  0.2736f,0.7466f,0.6064f,  0.3265f,0.7402f,0.5878f,  0.3119f,0.8180f,0.4834f,  0.3652f,0.8075f,0.4632f,  0.3451f,0.8708f,0.3503f,  0.3972f,0.8567f,0.3291f,  0.3701f,0.9049f,0.2103f,  0.4198f,0.8878f,0.1886f,  0.3844f,0.9207f,0.0669f,  0.4303f,0.9015f,0.0453f,  0.3857f,0.9195f,-0.0765f,  0.4269f,0.8990f,-0.0973f,  0.3724f,0.9025f,-0.2163f,  0.4083f,0.8819f,-0.2358f,  
	0.3436f,0.8718f,-0.3492f,  0.3737f,0.8520f,-0.3668f,  0.2991f,0.8294f,-0.4719f,  0.3232f,0.8113f,-0.4872f,  0.2392f,0.7775f,-0.5817f,  0.2576f,0.7620f,-0.5942f,  0.1652f,0.7182f,-0.6759f,  0.1780f,0.7061f,-0.6854f,  0.0786f,0.6538f,-0.7526f,  0.0864f,0.6455f,-0.7588f,  -0.0184f,0.5861f,-0.8100f,  -0.0151f,0.5822f,-0.8129f,  -0.1232f,0.5169f,-0.8471f,  -0.1237f,0.5175f,-0.8467f,  -0.2331f,0.4477f,-0.8633f,  
	-0.2368f,0.4529f,-0.8596f,  -0.3451f,0.3796f,-0.8583f,  -0.3512f,0.3894f,-0.8515f,  -0.4562f,0.3137f,-0.8327f,  -0.4640f,0.3278f,-0.8229f,  -0.5634f,0.2506f,-0.7873f,  -0.5723f,0.2688f,-0.7748f,  -0.6637f,0.1908f,-0.7232f,  -0.6731f,0.2126f,-0.7083f,  -0.7545f,0.1345f,-0.6423f,  -0.7639f,0.1595f,-0.6253f,  -0.8334f,0.0818f,-0.5466f,  -0.8423f,0.1095f,-0.5278f,  -0.8982f,0.0325f,-0.4383f,  -0.9062f,0.0623f,-0.4182f,  
	-0.9473f,-0.0137f,-0.3201f,  -0.9541f,0.0177f,-0.2991f,  -0.9792f,-0.0570f,-0.1948f,  -0.9846f,-0.0247f,-0.1732f,  -0.9930f,-0.0980f,-0.0654f,  -0.9969f,-0.0654f,-0.0436f,  -0.9884f,-0.1372f,0.0653f,  -0.9907f,-0.1048f,0.0869f,  -0.9652f,-0.1751f,0.1941f,  -0.9660f,-0.1436f,0.2151f,  -0.9240f,-0.2123f,0.3181f,  -0.9233f,-0.1822f,0.3381f,  -0.8656f,-0.2491f,0.4344f,  -0.8636f,-0.2211f,0.4531f,  -0.7913f,-0.2862f,0.5404f,  
	-0.7883f,-0.2607f,0.5574f,  -0.7027f,-0.3238f,0.6335f,  -0.6990f,-0.3013f,0.6485f,  -0.6020f,-0.3622f,0.7116f,  -0.5980f,-0.3432f,0.7243f,  -0.4915f,-0.4016f,0.7728f,  -0.4875f,-0.3864f,0.7830f,  -0.3736f,-0.4419f,0.8156f,  -0.3702f,-0.4309f,0.8230f,  -0.2513f,-0.4830f,0.8388f,  -0.2489f,-0.4764f,0.8432f,  -0.1273f,-0.5246f,0.8418f,  -0.1265f,-0.5226f,0.8431f,  -0.0046f,-0.5661f,0.8243f,  -0.0060f,-0.5688f,0.8225f,  
	0.2672f,-0.7014f,0.6607f,  0.2493f,-0.6985f,0.6707f,  0.2222f,-0.6086f,0.7617f,  0.2097f,-0.6025f,0.7700f,  0.1839f,-0.4996f,0.8465f,  0.1776f,-0.4910f,0.8529f,  0.1542f,-0.3773f,0.9132f,  0.1550f,-0.3669f,0.9173f,  0.1348f,-0.2449f,0.9601f,  0.1432f,-0.2334f,0.9618f,  0.1267f,-0.1060f,0.9863f,  0.1429f,-0.0944f,0.9852f,  0.1302f,0.0357f,0.9908f,  0.1542f,0.0463f,0.9869f,  0.1449f,0.1762f,0.9736f,  
	0.1764f,0.1850f,0.9668f,  0.1698f,0.3118f,0.9349f,  0.2080f,0.3178f,0.9251f,  0.2031f,0.4388f,0.8753f,  0.2468f,0.4414f,0.8627f,  0.2422f,0.5542f,0.7964f,  0.2904f,0.5528f,0.7811f,  0.2844f,0.6553f,0.6998f,  0.3355f,0.6498f,0.6821f,  0.3265f,0.7402f,0.5878f,  0.3789f,0.7304f,0.5682f,  0.3652f,0.8075f,0.4632f,  0.4174f,0.7939f,0.4422f,  0.3972f,0.8567f,0.3291f,  0.4479f,0.8397f,0.3071f,  
	0.4198f,0.8878f,0.1886f,  0.4675f,0.8682f,0.1663f,  0.4303f,0.9015f,0.0453f,  0.4741f,0.8801f,0.0234f,  0.4269f,0.8990f,-0.0973f,  0.4659f,0.8769f,-0.1183f,  0.4083f,0.8819f,-0.2358f,  0.4419f,0.8599f,-0.2553f,  0.3737f,0.8520f,-0.3668f,  0.4017f,0.8312f,-0.3843f,  0.3232f,0.8113f,-0.4872f,  0.3456f,0.7927f,-0.5023f,  0.2576f,0.7620f,-0.5942f,  0.2744f,0.7462f,-0.6065f,  0.1780f,0.7061f,-0.6854f,  
	0.1897f,0.6939f,-0.6947f,  0.0864f,0.6455f,-0.7588f,  0.0933f,0.6374f,-0.7648f,  -0.0151f,0.5822f,-0.8129f,  -0.0122f,0.5784f,-0.8156f,  -0.1237f,0.5175f,-0.8467f,  -0.1244f,0.5183f,-0.8461f,  -0.2368f,0.4529f,-0.8596f,  -0.2402f,0.4583f,-0.8557f,  -0.3512f,0.3894f,-0.8515f,  -0.3568f,0.3994f,-0.8445f,  -0.4640f,0.3278f,-0.8229f,  -0.4710f,0.3421f,-0.8131f,  -0.5723f,0.2688f,-0.7748f,  -0.5801f,0.2871f,-0.7622f,  
	-0.6731f,0.2126f,-0.7083f,  -0.6813f,0.2346f,-0.6934f,  -0.7639f,0.1595f,-0.6253f,  -0.7719f,0.1846f,-0.6084f,  -0.8423f,0.1095f,-0.5278f,  -0.8496f,0.1373f,-0.5092f,  -0.9062f,0.0623f,-0.4182f,  -0.9127f,0.0922f,-0.3982f,  -0.9541f,0.0177f,-0.2991f,  -0.9593f,0.0491f,-0.2781f,  -0.9846f,-0.0247f,-0.1732f,  -0.9884f,0.0076f,-0.1517f,  -0.9969f,-0.0654f,-0.0436f,  -0.9992f,-0.0327f,-0.0218f,  -0.9907f,-0.1048f,0.0869f,  
	-0.9915f,-0.0725f,0.1084f,  -0.9660f,-0.1436f,0.2151f,  -0.9653f,-0.1122f,0.2360f,  -0.9233f,-0.1822f,0.3381f,  -0.9212f,-0.1522f,0.3581f,  -0.8636f,-0.2211f,0.4531f,  -0.8603f,-0.1932f,0.4717f,  -0.7883f,-0.2607f,0.5574f,  -0.7841f,-0.2353f,0.5743f,  -0.6990f,-0.3013f,0.6485f,  -0.6942f,-0.2790f,0.6635f,  -0.5980f,-0.3432f,0.7243f,  -0.5930f,-0.3244f,0.7370f,  -0.4875f,-0.3864f,0.7830f,  -0.4828f,-0.3715f,0.7931f,  
	-0.3702f,-0.4309f,0.8230f,  -0.3662f,-0.4202f,0.8303f,  -0.2489f,-0.4764f,0.8432f,  -0.2463f,-0.4702f,0.8475f,  -0.1265f,-0.5226f,0.8431f,  -0.1258f,-0.5209f,0.8443f,  -0.0060f,-0.5688f,0.8225f,  -0.0077f,-0.5717f,0.8204f,  0.2493f,-0.6985f,0.6707f,  0.2319f,-0.6951f,0.6804f,  0.2097f,-0.6025f,0.7700f,  0.1979f,-0.5963f,0.7780f,  0.1776f,-0.4910f,0.8529f,  0.1725f,-0.4825f,0.8587f,  0.1550f,-0.3669f,0.9173f,  
	0.1572f,-0.3569f,0.9208f,  0.1432f,-0.2334f,0.9618f,  0.1533f,-0.2229f,0.9627f,  0.1429f,-0.0944f,0.9852f,  0.1609f,-0.0843f,0.9834f,  0.1542f,0.0463f,0.9869f,  0.1800f,0.0550f,0.9821f,  0.1764f,0.1850f,0.9668f,  0.2094f,0.1913f,0.9589f,  0.2080f,0.3178f,0.9251f,  0.2473f,0.3210f,0.9142f,  0.2468f,0.4414f,0.8627f,  0.2913f,0.4408f,0.8490f,  0.2904f,0.5528f,0.7811f,  0.3387f,0.5482f,0.7647f,  
	0.3355f,0.6498f,0.6821f,  0.3862f,0.6408f,0.6635f,  0.3789f,0.7304f,0.5682f,  0.4305f,0.7174f,0.5477f,  0.4174f,0.7939f,0.4422f,  0.4683f,0.7772f,0.4204f,  0.4479f,0.8397f,0.3071f,  0.4967f,0.8199f,0.2845f,  0.4675f,0.8682f,0.1663f,  0.5132f,0.8462f,0.1436f,  0.4741f,0.8801f,0.0234f,  0.5156f,0.8568f,0.0011f,  0.4659f,0.8769f,-0.1183f,  0.5026f,0.8532f,-0.1395f,  0.4419f,0.8599f,-0.2553f,  
	0.4733f,0.8369f,-0.2749f,  0.4017f,0.8312f,-0.3843f,  0.4277f,0.8097f,-0.4018f,  0.3456f,0.7927f,-0.5023f,  0.3661f,0.7736f,-0.5172f,  0.2744f,0.7462f,-0.6065f,  0.2897f,0.7303f,-0.6186f,  0.1897f,0.6939f,-0.6947f,  0.2002f,0.6818f,-0.7037f,  0.0933f,0.6374f,-0.7648f,  0.0995f,0.6295f,-0.7706f,  -0.0122f,0.5784f,-0.8156f,  -0.0098f,0.5750f,-0.8181f,  -0.1244f,0.5183f,-0.8461f,  -0.1251f,0.5195f,-0.8453f,  
	-0.2402f,0.4583f,-0.8557f,  -0.2434f,0.4641f,-0.8517f,  -0.3568f,0.3994f,-0.8445f,  -0.3617f,0.4096f,-0.8375f,  -0.4710f,0.3421f,-0.8131f,  -0.4773f,0.3567f,-0.8031f,  -0.5801f,0.2871f,-0.7622f,  -0.5870f,0.3056f,-0.7497f,  -0.6813f,0.2346f,-0.6934f,  -0.6883f,0.2566f,-0.6785f,  -0.7719f,0.1846f,-0.6084f,  -0.7786f,0.2098f,-0.5914f,  -0.8496f,0.1373f,-0.5092f,  -0.8556f,0.1651f,-0.4905f,  -0.9127f,0.0922f,-0.3982f,  
	-0.9176f,0.1221f,-0.3782f,  -0.9593f,0.0491f,-0.2781f,  -0.9630f,0.0805f,-0.2572f,  -0.9884f,0.0076f,-0.1517f,  -0.9907f,0.0399f,-0.1301f,  -0.9992f,-0.0327f,-0.0218f,  -1.0000f,-0.0002f,-0.0001f,  -0.9915f,-0.0725f,0.1084f,  -0.9907f,-0.0402f,0.1299f,  -0.9653f,-0.1122f,0.2360f,  -0.9630f,-0.0808f,0.2570f,  -0.9212f,-0.1522f,0.3581f,  -0.9177f,-0.1223f,0.3780f,  -0.8603f,-0.1932f,0.4717f,  -0.8557f,-0.1653f,0.4904f,  
	-0.7841f,-0.2353f,0.5743f,  -0.7786f,-0.2101f,0.5913f,  -0.6942f,-0.2790f,0.6635f,  -0.6883f,-0.2568f,0.6784f,  -0.5930f,-0.3244f,0.7370f,  -0.5871f,-0.3057f,0.7496f,  -0.4828f,-0.3715f,0.7931f,  -0.4773f,-0.3567f,0.8031f,  -0.3662f,-0.4202f,0.8303f,  -0.3618f,-0.4097f,0.8374f,  -0.2463f,-0.4702f,0.8475f,  -0.2434f,-0.4641f,0.8517f,  -0.1258f,-0.5209f,0.8443f,  -0.1251f,-0.5195f,0.8453f,  -0.0077f,-0.5717f,0.8204f,  
	-0.0098f,-0.5750f,0.8181f,  0.2319f,-0.6951f,0.6804f,  0.2149f,-0.6914f,0.6898f,  0.1979f,-0.5963f,0.7780f,  0.1871f,-0.5899f,0.7855f,  0.1725f,-0.4825f,0.8587f,  0.1687f,-0.4742f,0.8641f,  0.1572f,-0.3569f,0.9208f,  0.1610f,-0.3476f,0.9237f,  0.1533f,-0.2229f,0.9627f,  0.1649f,-0.2135f,0.9629f,  0.1609f,-0.0843f,0.9834f,  0.1806f,-0.0759f,0.9806f,  0.1800f,0.0550f,0.9821f,  0.2072f,0.0616f,0.9764f,  
	0.2094f,0.1913f,0.9589f,  0.2435f,0.1951f,0.9501f,  0.2473f,0.3210f,0.9142f,  0.2873f,0.3213f,0.9023f,  0.2913f,0.4408f,0.8490f,  0.3361f,0.4372f,0.8342f,  0.3387f,0.5482f,0.7647f,  0.3868f,0.5402f,0.7474f,  0.3862f,0.6408f,0.6635f,  0.4361f,0.6287f,0.6439f,  0.4305f,0.7174f,0.5477f,  0.4806f,0.7013f,0.5264f,  0.4683f,0.7772f,0.4204f,  0.5174f,0.7576f,0.3979f,  0.4967f,0.8199f,0.2845f,  
	0.5434f,0.7977f,0.2615f,  0.5132f,0.8462f,0.1436f,  0.5565f,0.8221f,0.1206f,  0.5156f,0.8568f,0.0011f,  0.5547f,0.8318f,-0.0213f,  0.5026f,0.8532f,-0.1395f,  0.5369f,0.8282f,-0.1607f,  0.4733f,0.8369f,-0.2749f,  0.5025f,0.8129f,-0.2944f,  0.4277f,0.8097f,-0.4018f,  0.4515f,0.7877f,-0.4191f,  0.3661f,0.7736f,-0.5172f,  0.3848f,0.7543f,-0.5319f,  0.2897f,0.7303f,-0.6186f,  0.3036f,0.7144f,-0.6304f,  
	0.2002f,0.6818f,-0.7037f,  0.2096f,0.6697f,-0.7124f,  0.0995f,0.6295f,-0.7706f,  0.1050f,0.6217f,-0.7762f,  -0.0098f,0.5750f,-0.8181f,  -0.0077f,0.5717f,-0.8204f,  -0.1251f,0.5195f,-0.8453f,  -0.1258f,0.5209f,-0.8443f,  -0.2434f,0.4641f,-0.8517f,  -0.2463f,0.4701f,-0.8475f,  -0.3617f,0.4096f,-0.8375f,  -0.3662f,0.4201f,-0.8303f,  -0.4773f,0.3567f,-0.8031f,  -0.4827f,0.3714f,-0.7931f,  -0.5870f,0.3056f,-0.7497f,  
	-0.5929f,0.3243f,-0.7371f,  -0.6883f,0.2566f,-0.6785f,  -0.6942f,0.2789f,-0.6636f,  -0.7786f,0.2098f,-0.5914f,  -0.7840f,0.2351f,-0.5745f,  -0.8556f,0.1651f,-0.4905f,  -0.8603f,0.1929f,-0.4719f,  -0.9176f,0.1221f,-0.3782f,  -0.9212f,0.1520f,-0.3583f,  -0.9630f,0.0805f,-0.2572f,  -0.9652f,0.1119f,-0.2362f,  -0.9907f,0.0399f,-0.1301f,  -0.9915f,0.0722f,-0.1086f,  -1.0000f,-0.0002f,-0.0001f,  -0.9992f,0.0324f,0.0216f,  
	-0.9907f,-0.0402f,0.1299f,  -0.9884f,-0.0079f,0.1515f,  -0.9630f,-0.0808f,0.2570f,  -0.9593f,-0.0494f,0.2779f,  -0.9177f,-0.1223f,0.3780f,  -0.9127f,-0.0925f,0.3980f,  -0.8557f,-0.1653f,0.4904f,  -0.8497f,-0.1375f,0.5090f,  -0.7786f,-0.2101f,0.5913f,  -0.7719f,-0.1849f,0.6082f,  -0.6883f,-0.2568f,0.6784f,  -0.6813f,-0.2347f,0.6933f,  -0.5871f,-0.3057f,0.7496f,  -0.5802f,-0.2872f,0.7622f,  -0.4773f,-0.3567f,0.8031f,  
	-0.4711f,-0.3422f,0.8130f,  -0.3618f,-0.4097f,0.8374f,  -0.3568f,-0.3994f,0.8445f,  -0.2434f,-0.4641f,0.8517f,  -0.2402f,-0.4584f,0.8557f,  -0.1251f,-0.5195f,0.8453f,  -0.1244f,-0.5183f,0.8461f,  -0.0098f,-0.5750f,0.8181f,  -0.0122f,-0.5784f,0.8156f,  0.2149f,-0.6914f,0.6898f,  0.1985f,-0.6872f,0.6988f,  0.1871f,-0.5899f,0.7855f,  0.1772f,-0.5834f,0.7926f,  0.1687f,-0.4742f,0.8641f,  0.1660f,-0.4662f,0.8689f,  
	0.1610f,-0.3476f,0.9237f,  0.1661f,-0.3390f,0.9260f,  0.1649f,-0.2135f,0.9629f,  0.1781f,-0.2054f,0.9623f,  0.1806f,-0.0759f,0.9806f,  0.2016f,-0.0692f,0.9770f,  0.2072f,0.0616f,0.9764f,  0.2356f,0.0659f,0.9696f,  0.2435f,0.1951f,0.9501f,  0.2785f,0.1963f,0.9402f,  0.2873f,0.3213f,0.9023f,  0.3278f,0.3187f,0.8894f,  0.3361f,0.4372f,0.8342f,  0.3807f,0.4304f,0.8184f,  0.3868f,0.5402f,0.7474f,  
	0.4341f,0.5292f,0.7290f,  0.4361f,0.6287f,0.6439f,  0.4847f,0.6135f,0.6235f,  0.4806f,0.7013f,0.5264f,  0.5291f,0.6824f,0.5044f,  0.5174f,0.7576f,0.3979f,  0.5643f,0.7355f,0.3748f,  0.5434f,0.7977f,0.2615f,  0.5878f,0.7732f,0.2380f,  0.5565f,0.8221f,0.1206f,  0.5973f,0.7961f,0.0973f,  0.5547f,0.8318f,-0.0213f,  0.5912f,0.8053f,-0.0438f,  0.5369f,0.8282f,-0.1607f,  0.5686f,0.8022f,-0.1819f,  
	0.5025f,0.8129f,-0.2944f,  0.5293f,0.7883f,-0.3137f,  0.4515f,0.7877f,-0.4191f,  0.4734f,0.7653f,-0.4362f,  0.3848f,0.7543f,-0.5319f,  0.4018f,0.7348f,-0.5464f,  0.3036f,0.7144f,-0.6304f,  0.3160f,0.6985f,-0.6420f,  0.2096f,0.6697f,-0.7124f,  0.2179f,0.6578f,-0.7209f,  0.1050f,0.6217f,-0.7762f,  0.1097f,0.6142f,-0.7815f,  -0.0077f,0.5717f,-0.8204f,  -0.0060f,0.5688f,-0.8225f,  -0.1258f,0.5209f,-0.8443f,  
	-0.1265f,0.5226f,-0.8431f,  -0.2463f,0.4701f,-0.8475f,  -0.2489f,0.4764f,-0.8433f,  -0.3662f,0.4201f,-0.8303f,  -0.3702f,0.4309f,-0.8230f,  -0.4827f,0.3714f,-0.7931f,  -0.4874f,0.3863f,-0.7830f,  -0.5929f,0.3243f,-0.7371f,  -0.5979f,0.3431f,-0.7244f,  -0.6942f,0.2789f,-0.6636f,  -0.6990f,0.3012f,-0.6486f,  -0.7840f,0.2351f,-0.5745f,  -0.7882f,0.2605f,-0.5575f,  -0.8603f,0.1929f,-0.4719f,  -0.8636f,0.2209f,-0.4533f,  
	-0.9212f,0.1520f,-0.3583f,  -0.9233f,0.1819f,-0.3383f,  -0.9652f,0.1119f,-0.2362f,  -0.9660f,0.1433f,-0.2153f,  -0.9915f,0.0722f,-0.1086f,  -0.9907f,0.1045f,-0.0871f,  -0.9992f,0.0324f,0.0216f,  -0.9969f,0.0650f,0.0434f,  -0.9884f,-0.0079f,0.1515f,  -0.9846f,0.0244f,0.1730f,  -0.9593f,-0.0494f,0.2779f,  -0.9541f,-0.0180f,0.2989f,  -0.9127f,-0.0925f,0.3980f,  -0.9063f,-0.0626f,0.4180f,  -0.8497f,-0.1375f,0.5090f,  
	-0.8423f,-0.1097f,0.5277f,  -0.7719f,-0.1849f,0.6082f,  -0.7639f,-0.1598f,0.6252f,  -0.6813f,-0.2347f,0.6933f,  -0.6732f,-0.2128f,0.7082f,  -0.5802f,-0.2872f,0.7622f,  -0.5723f,-0.2689f,0.7747f,  -0.4711f,-0.3422f,0.8130f,  -0.4641f,-0.3279f,0.8229f,  -0.3568f,-0.3994f,0.8445f,  -0.3513f,-0.3894f,0.8515f,  -0.2402f,-0.4584f,0.8557f,  -0.2368f,-0.4529f,0.8595f,  -0.1244f,-0.5183f,0.8461f,  -0.1238f,-0.5175f,0.8467f,  
	-0.0122f,-0.5784f,0.8156f,  -0.0151f,-0.5822f,0.8129f,  0.1985f,-0.6872f,0.6988f,  0.1827f,-0.6826f,0.7076f,  0.1772f,-0.5834f,0.7926f,  0.1682f,-0.5769f,0.7993f,  0.1660f,-0.4662f,0.8689f,  0.1645f,-0.4587f,0.8732f,  0.1661f,-0.3390f,0.9260f,  0.1726f,-0.3314f,0.9276f,  0.1781f,-0.2054f,0.9623f,  0.1925f,-0.1987f,0.9610f,  0.2016f,-0.0692f,0.9770f,  0.2237f,-0.0644f,0.9725f,  0.2356f,0.0659f,0.9696f,  
	0.2648f,0.0679f,0.9619f,  0.2785f,0.1963f,0.9402f,  0.3139f,0.1948f,0.9293f,  0.3278f,0.3187f,0.8894f,  0.3682f,0.3132f,0.8754f,  0.3807f,0.4304f,0.8184f,  0.4248f,0.4206f,0.8017f,  0.4341f,0.5292f,0.7290f,  0.4804f,0.5151f,0.7098f,  0.4847f,0.6135f,0.6235f,  0.5317f,0.5954f,0.6023f,  0.5291f,0.6824f,0.5044f,  0.5755f,0.6608f,0.4818f,  0.5643f,0.7355f,0.3748f,  0.6090f,0.7111f,0.3514f,  
	0.5878f,0.7732f,0.2380f,  0.6296f,0.7468f,0.2142f,  0.5973f,0.7961f,0.0973f,  0.6354f,0.7687f,0.0739f,  0.5912f,0.8053f,-0.0438f,  0.6251f,0.7777f,-0.0663f,  0.5686f,0.8022f,-0.1819f,  0.5979f,0.7754f,-0.2030f,  0.5293f,0.7883f,-0.3137f,  0.5538f,0.7632f,-0.3329f,  0.4734f,0.7653f,-0.4362f,  0.4932f,0.7426f,-0.4530f,  0.4018f,0.7348f,-0.5464f,  0.4171f,0.7153f,-0.5606f,  0.3160f,0.6985f,-0.6420f,  
	0.3272f,0.6827f,-0.6534f,  0.2179f,0.6578f,-0.7209f,  0.2253f,0.6462f,-0.7292f,  0.1097f,0.6142f,-0.7815f,  0.1138f,0.6070f,-0.7865f,  -0.0060f,0.5688f,-0.8225f,  -0.0046f,0.5661f,-0.8243f,  -0.1265f,0.5226f,-0.8431f,  -0.1273f,0.5246f,-0.8418f,  -0.2489f,0.4764f,-0.8433f,  -0.2513f,0.4830f,-0.8388f,  -0.3702f,0.4309f,-0.8230f,  -0.3736f,0.4418f,-0.8156f,  -0.4874f,0.3863f,-0.7830f,  -0.4914f,0.4015f,-0.7729f,  
	-0.5979f,0.3431f,-0.7244f,  -0.6020f,0.3621f,-0.7117f,  -0.6990f,0.3012f,-0.6486f,  -0.7027f,0.3236f,-0.6336f,  -0.7882f,0.2605f,-0.5575f,  -0.7912f,0.2860f,-0.5405f,  -0.8636f,0.2209f,-0.4533f,  -0.8656f,0.2489f,-0.4346f,  -0.9233f,0.1819f,-0.3383f,  -0.9240f,0.2120f,-0.3182f,  -0.9660f,0.1433f,-0.2153f,  -0.9652f,0.1748f,-0.1943f,  -0.9907f,0.1045f,-0.0871f,  -0.9884f,0.1369f,-0.0655f,  -0.9969f,0.0650f,0.0434f,  
	-0.9931f,0.0977f,0.0652f,  -0.9846f,0.0244f,0.1730f,  -0.9792f,0.0567f,0.1946f,  -0.9541f,-0.0180f,0.2989f,  -0.9473f,0.0134f,0.3199f,  -0.9063f,-0.0626f,0.4180f,  -0.8983f,-0.0328f,0.4381f,  -0.8423f,-0.1097f,0.5277f,  -0.8335f,-0.0820f,0.5464f,  -0.7639f,-0.1598f,0.6252f,  -0.7546f,-0.1347f,0.6422f,  -0.6732f,-0.2128f,0.7082f,  -0.6638f,-0.1910f,0.7231f,  -0.5723f,-0.2689f,0.7747f,  -0.5634f,-0.2507f,0.7872f,  
	-0.4641f,-0.3279f,0.8229f,  -0.4563f,-0.3138f,0.8327f,  -0.3513f,-0.3894f,0.8515f,  -0.3452f,-0.3797f,0.8583f,  -0.2368f,-0.4529f,0.8595f,  -0.2331f,-0.4477f,0.8633f,  -0.1238f,-0.5175f,0.8467f,  -0.1232f,-0.5169f,0.8471f,  -0.0151f,-0.5822f,0.8129f,  -0.0184f,-0.5861f,0.8100f,  0.1827f,-0.6826f,0.7076f,  0.1675f,-0.6777f,0.7160f,  0.1682f,-0.5769f,0.7993f,  0.1601f,-0.5705f,0.8055f,  0.1645f,-0.4587f,0.8732f,  
	0.1642f,-0.4518f,0.8769f,  0.1726f,-0.3314f,0.9276f,  0.1802f,-0.3249f,0.9284f,  0.1925f,-0.1987f,0.9610f,  0.2080f,-0.1936f,0.9588f,  0.2237f,-0.0644f,0.9725f,  0.2467f,-0.0616f,0.9671f,  0.2648f,0.0679f,0.9619f,  0.2946f,0.0675f,0.9532f,  0.3139f,0.1948f,0.9293f,  0.3494f,0.1906f,0.9174f,  0.3682f,0.3132f,0.8754f,  0.4082f,0.3048f,0.8605f,  0.4248f,0.4206f,0.8017f,  0.4679f,0.4079f,0.7840f,  
	0.4804f,0.5151f,0.7098f,  0.5252f,0.4982f,0.6899f,  0.5317f,0.5954f,0.6023f,  0.5768f,0.5747f,0.5805f,  0.5755f,0.6608f,0.4818f,  0.6197f,0.6369f,0.4587f,  0.6090f,0.7111f,0.3514f,  0.6510f,0.6847f,0.3275f,  0.6296f,0.7468f,0.2142f,  0.6687f,0.7188f,0.1903f,  0.6354f,0.7687f,0.0739f,  0.6708f,0.7399f,0.0504f,  0.6251f,0.7777f,-0.0663f,  0.6564f,0.7492f,-0.0888f,  0.5979f,0.7754f,-0.2030f,  
	0.6248f,0.7480f,-0.2240f,  0.5538f,0.7632f,-0.3329f,  0.5761f,0.7377f,-0.3519f,  0.4932f,0.7426f,-0.4530f,  0.5111f,0.7199f,-0.4696f,  0.4171f,0.7153f,-0.5606f,  0.4309f,0.6958f,-0.5746f,  0.3272f,0.6827f,-0.6534f,  0.3371f,0.6670f,-0.6644f,  0.2253f,0.6462f,-0.7292f,  0.2318f,0.6347f,-0.7372f,  0.1138f,0.6070f,-0.7865f,  0.1174f,0.6000f,-0.7914f,  -0.0046f,0.5661f,-0.8243f,  -0.0035f,0.5637f,-0.8260f,  
	-0.1273f,0.5246f,-0.8418f,  -0.1280f,0.5268f,-0.8403f,  -0.2513f,0.4830f,-0.8388f,  -0.2534f,0.4897f,-0.8342f,  -0.3736f,0.4418f,-0.8156f,  -0.3765f,0.4530f,-0.8081f,  -0.4914f,0.4015f,-0.7729f,  -0.4947f,0.4168f,-0.7626f,  -0.6020f,0.3621f,-0.7117f,  -0.6052f,0.3812f,-0.6989f,  -0.7027f,0.3236f,-0.6336f,  -0.7054f,0.3462f,-0.6185f,  -0.7912f,0.2860f,-0.5405f,  -0.7931f,0.3116f,-0.5234f,  -0.8656f,0.2489f,-0.4346f,  
	-0.8662f,0.2771f,-0.4158f,  -0.9240f,0.2120f,-0.3182f,  -0.9233f,0.2422f,-0.2981f,  -0.9652f,0.1748f,-0.1943f,  -0.9630f,0.2064f,-0.1732f,  -0.9884f,0.1369f,-0.0655f,  -0.9846f,0.1694f,-0.0438f,  -0.9931f,0.0977f,0.0652f,  -0.9876f,0.1305f,0.0871f,  -0.9792f,0.0567f,0.1946f,  -0.9722f,0.0891f,0.2164f,  -0.9473f,0.0134f,0.3199f,  -0.9390f,0.0448f,0.3411f,  -0.8983f,-0.0328f,0.4381f,  -0.8888f,-0.0029f,0.4583f,  
	-0.8335f,-0.0820f,0.5464f,  -0.8232f,-0.0544f,0.5652f,  -0.7546f,-0.1347f,0.6422f,  -0.7439f,-0.1098f,0.6592f,  -0.6638f,-0.1910f,0.7231f,  -0.6532f,-0.1693f,0.7380f,  -0.5634f,-0.2507f,0.7872f,  -0.5535f,-0.2328f,0.7996f,  -0.4563f,-0.3138f,0.8327f,  -0.4476f,-0.3000f,0.8424f,  -0.3452f,-0.3797f,0.8583f,  -0.3385f,-0.3703f,0.8650f,  -0.2331f,-0.4477f,0.8633f,  -0.2292f,-0.4429f,0.8668f,  -0.1232f,-0.5169f,0.8471f,  
	-0.1228f,-0.5167f,0.8473f,  -0.0184f,-0.5861f,0.8100f,  -0.0221f,-0.5903f,0.8068f,  0.1675f,-0.6777f,0.7160f,  0.1530f,-0.6726f,0.7240f,  0.1601f,-0.5705f,0.8055f,  0.1530f,-0.5643f,0.8113f,  0.1642f,-0.4518f,0.8769f,  0.1649f,-0.4454f,0.8800f,  0.1802f,-0.3249f,0.9284f,  0.1889f,-0.3195f,0.9286f,  0.2080f,-0.1936f,0.9588f,  0.2245f,-0.1901f,0.9558f,  0.2467f,-0.0616f,0.9671f,  0.2704f,-0.0609f,0.9608f,  
	0.2946f,0.0675f,0.9532f,  0.3246f,0.0647f,0.9436f,  0.3494f,0.1906f,0.9174f,  0.3846f,0.1838f,0.9046f,  0.4082f,0.3048f,0.8605f,  0.4474f,0.2937f,0.8447f,  0.4679f,0.4079f,0.7840f,  0.5097f,0.3925f,0.7656f,  0.5252f,0.4982f,0.6899f,  0.5682f,0.4788f,0.6692f,  0.5768f,0.5747f,0.5805f,  0.6197f,0.5517f,0.5582f,  0.6197f,0.6369f,0.4587f,  0.6613f,0.6109f,0.4352f,  0.6510f,0.6847f,0.3275f,  
	0.6905f,0.6566f,0.3035f,  0.6687f,0.7188f,0.1903f,  0.7050f,0.6894f,0.1663f,  0.6708f,0.7399f,0.0504f,  0.7035f,0.7102f,0.0269f,  0.6564f,0.7492f,-0.0888f,  0.6850f,0.7200f,-0.1111f,  0.6248f,0.7480f,-0.2240f,  0.6492f,0.7202f,-0.2447f,  0.5761f,0.7377f,-0.3519f,  0.5963f,0.7121f,-0.3706f,  0.5111f,0.7199f,-0.4696f,  0.5272f,0.6971f,-0.4860f,  0.4309f,0.6958f,-0.5746f,  0.4431f,0.6765f,-0.5882f,  
	0.3371f,0.6670f,-0.6644f,  0.3458f,0.6516f,-0.6752f,  0.2318f,0.6347f,-0.7372f,  0.2374f,0.6235f,-0.7449f,  0.1174f,0.6000f,-0.7914f,  0.1204f,0.5932f,-0.7960f,  -0.0035f,0.5637f,-0.8260f,  -0.0026f,0.5616f,-0.8274f,  -0.1280f,0.5268f,-0.8403f,  -0.1288f,0.5293f,-0.8386f,  -0.2534f,0.4897f,-0.8342f,  -0.2552f,0.4968f,-0.8295f,  -0.3765f,0.4530f,-0.8081f,  -0.3790f,0.4644f,-0.8005f,  -0.4947f,0.4168f,-0.7626f,  
	-0.4973f,0.4323f,-0.7522f,  -0.6052f,0.3812f,-0.6989f,  -0.6074f,0.4006f,-0.6860f,  -0.7054f,0.3462f,-0.6185f,  -0.7070f,0.3690f,-0.6033f,  -0.7931f,0.3116f,-0.5234f,  -0.7937f,0.3374f,-0.5062f,  -0.8662f,0.2771f,-0.4158f,  -0.8656f,0.3054f,-0.3969f,  -0.9233f,0.2422f,-0.2981f,  -0.9211f,0.2725f,-0.2779f,  -0.9630f,0.2064f,-0.1732f,  -0.9593f,0.2382f,-0.1520f,  -0.9846f,0.1694f,-0.0438f,  -0.9791f,0.2020f,-0.0220f,  
	-0.9876f,0.1305f,0.0871f,  -0.9805f,0.1633f,0.1091f,  -0.9722f,0.0891f,0.2164f,  -0.9636f,0.1216f,0.2382f,  -0.9390f,0.0448f,0.3411f,  -0.9289f,0.0762f,0.3623f,  -0.8888f,-0.0029f,0.4583f,  -0.8777f,0.0269f,0.4785f,  -0.8232f,-0.0544f,0.5652f,  -0.8113f,-0.0268f,0.5840f,  -0.7439f,-0.1098f,0.6592f,  -0.7317f,-0.0851f,0.6763f,  -0.6532f,-0.1693f,0.7380f,  -0.6412f,-0.1479f,0.7530f,  -0.5535f,-0.2328f,0.7996f,  
	-0.5424f,-0.2152f,0.8121f,  -0.4476f,-0.3000f,0.8424f,  -0.4381f,-0.2866f,0.8520f,  -0.3385f,-0.3703f,0.8650f,  -0.3313f,-0.3613f,0.8716f,  -0.2292f,-0.4429f,0.8668f,  -0.2250f,-0.4384f,0.8701f,  -0.1228f,-0.5167f,0.8473f,  -0.1224f,-0.5168f,0.8473f,  -0.0221f,-0.5903f,0.8068f,  -0.0264f,-0.5948f,0.8034f,  0.1530f,-0.6726f,0.7240f,  0.1392f,-0.6672f,0.7317f,  0.1530f,-0.5643f,0.8113f,  0.1467f,-0.5583f,0.8165f,  
	0.1649f,-0.4454f,0.8800f,  0.1665f,-0.4399f,0.8825f,  0.1889f,-0.3195f,0.9286f,  0.1984f,-0.3153f,0.9280f,  0.2245f,-0.1901f,0.9558f,  0.2416f,-0.1882f,0.9520f,  0.2704f,-0.0609f,0.9608f,  0.2943f,-0.0622f,0.9537f,  0.3246f,0.0647f,0.9436f,  0.3544f,0.0596f,0.9332f,  0.3846f,0.1838f,0.9046f,  0.4192f,0.1745f,0.8910f,  0.4474f,0.2937f,0.8447f,  0.4854f,0.2801f,0.8282f,  0.5097f,0.3925f,0.7656f,  
	0.5498f,0.3747f,0.7465f,  0.5682f,0.4788f,0.6692f,  0.6091f,0.4571f,0.6481f,  0.6197f,0.5517f,0.5582f,  0.6603f,0.5266f,0.5354f,  0.6613f,0.6109f,0.4352f,  0.7004f,0.5832f,0.4115f,  0.6905f,0.6566f,0.3035f,  0.7271f,0.6271f,0.2793f,  0.7050f,0.6894f,0.1663f,  0.7386f,0.6590f,0.1422f,  0.7035f,0.7102f,0.0269f,  0.7335f,0.6796f,0.0036f,  0.6850f,0.7200f,-0.1111f,  0.7112f,0.6903f,-0.1333f,  
	0.6492f,0.7202f,-0.2447f,  0.6713f,0.6921f,-0.2652f,  0.5963f,0.7121f,-0.3706f,  0.6145f,0.6864f,-0.3890f,  0.5272f,0.6971f,-0.4860f,  0.5415f,0.6744f,-0.5020f,  0.4431f,0.6765f,-0.5882f,  0.4539f,0.6573f,-0.6016f,  0.3458f,0.6516f,-0.6752f,  0.3534f,0.6364f,-0.6857f,  0.2374f,0.6235f,-0.7449f,  0.2422f,0.6126f,-0.7524f,  0.1204f,0.5932f,-0.7960f,  0.1229f,0.5868f,-0.8003f,  -0.0026f,0.5616f,-0.8274f,  
	-0.0020f,0.5598f,-0.8286f,  -0.1288f,0.5293f,-0.8386f,  -0.1294f,0.5320f,-0.8368f,  -0.2552f,0.4968f,-0.8295f,  -0.2567f,0.5040f,-0.8246f,  -0.3790f,0.4644f,-0.8005f,  -0.3809f,0.4760f,-0.7927f,  -0.4973f,0.4323f,-0.7522f,  -0.4991f,0.4480f,-0.7417f,  -0.6074f,0.4006f,-0.6860f,  -0.6088f,0.4201f,-0.6730f,  -0.7070f,0.3690f,-0.6033f,  -0.7075f,0.3920f,-0.5880f,  -0.7937f,0.3374f,-0.5062f,  -0.7930f,0.3633f,-0.4889f,  
	-0.8656f,0.3054f,-0.3969f,  -0.8635f,0.3338f,-0.3780f,  -0.9211f,0.2725f,-0.2779f,  -0.9175f,0.3029f,-0.2576f,  -0.9593f,0.2382f,-0.1520f,  -0.9539f,0.2701f,-0.1307f,  -0.9791f,0.2020f,-0.0220f,  -0.9721f,0.2347f,-0.0000f,  -0.9805f,0.1633f,0.1091f,  -0.9717f,0.1962f,0.1313f,  -0.9636f,0.1216f,0.2382f,  -0.9532f,0.1541f,0.2602f,  -0.9289f,0.0762f,0.3623f,  -0.9172f,0.1077f,0.3837f,  -0.8777f,0.0269f,0.4785f,  
	-0.8648f,0.0567f,0.4989f,  -0.8113f,-0.0268f,0.5840f,  -0.7978f,0.0007f,0.6030f,  -0.7317f,-0.0851f,0.6763f,  -0.7180f,-0.0605f,0.6934f,  -0.6412f,-0.1479f,0.7530f,  -0.6279f,-0.1268f,0.7679f,  -0.5424f,-0.2152f,0.8121f,  -0.5302f,-0.1979f,0.8245f,  -0.4381f,-0.2866f,0.8520f,  -0.4277f,-0.2735f,0.8616f,  -0.3313f,-0.3613f,0.8716f,  -0.3235f,-0.3527f,0.8781f,  -0.2250f,-0.4384f,0.8701f,  -0.2206f,-0.4344f,0.8733f,  
	-0.1224f,-0.5168f,0.8473f,  -0.1222f,-0.5172f,0.8471f,  -0.0264f,-0.5948f,0.8034f,  -0.0312f,-0.5994f,0.7998f,  0.1392f,-0.6672f,0.7317f,  0.1261f,-0.6617f,0.7391f,  0.1467f,-0.5583f,0.8165f,  0.1413f,-0.5527f,0.8213f,  0.1665f,-0.4399f,0.8825f,  0.1690f,-0.4351f,0.8844f,  0.1984f,-0.3153f,0.9280f,  0.2086f,-0.3123f,0.9268f,  0.2416f,-0.1882f,0.9520f,  0.2590f,-0.1881f,0.9474f,  0.2943f,-0.0622f,0.9537f,  
	0.3183f,-0.0655f,0.9457f,  0.3544f,0.0596f,0.9332f,  0.3839f,0.0523f,0.9219f,  0.4192f,0.1745f,0.8910f,  0.4528f,0.1629f,0.8766f,  0.4854f,0.2801f,0.8282f,  0.5220f,0.2642f,0.8110f,  0.5498f,0.3747f,0.7465f,  0.5881f,0.3546f,0.7269f,  0.6091f,0.4571f,0.6481f,  0.6479f,0.4334f,0.6265f,  0.6603f,0.5266f,0.5354f,  0.6983f,0.4998f,0.5124f,  0.7004f,0.5832f,0.4115f,  0.7368f,0.5541f,0.3876f,  
	0.7271f,0.6271f,0.2793f,  0.7610f,0.5965f,0.2551f,  0.7386f,0.6590f,0.1422f,  0.7695f,0.6276f,0.1183f,  0.7335f,0.6796f,0.0036f,  0.7609f,0.6485f,-0.0196f,  0.7112f,0.6903f,-0.1333f,  0.7349f,0.6602f,-0.1552f,  0.6713f,0.6921f,-0.2652f,  0.6913f,0.6639f,-0.2854f,  0.6145f,0.6864f,-0.3890f,  0.6307f,0.6607f,-0.4072f,  0.5415f,0.6744f,-0.5020f,  0.5542f,0.6518f,-0.5177f,  0.4539f,0.6573f,-0.6016f,  
	0.4633f,0.6384f,-0.6147f,  0.3534f,0.6364f,-0.6857f,  0.3599f,0.6215f,-0.6959f,  0.2422f,0.6126f,-0.7524f,  0.2463f,0.6020f,-0.7596f,  0.1229f,0.5868f,-0.8003f,  0.1249f,0.5807f,-0.8045f,  -0.0020f,0.5598f,-0.8286f,  -0.0015f,0.5582f,-0.8297f,  -0.1294f,0.5320f,-0.8368f,  -0.1301f,0.5351f,-0.8347f,  -0.2567f,0.5040f,-0.8246f,  -0.2579f,0.5116f,-0.8196f,  -0.3809f,0.4760f,-0.7927f,  -0.3822f,0.4879f,-0.7848f,  
	-0.4991f,0.4480f,-0.7417f,  -0.5002f,0.4640f,-0.7311f,  -0.6088f,0.4201f,-0.6730f,  -0.6093f,0.4398f,-0.6598f,  -0.7075f,0.3920f,-0.5880f,  -0.7070f,0.4151f,-0.5726f,  -0.7930f,0.3633f,-0.4889f,  -0.7912f,0.3895f,-0.4715f,  -0.8635f,0.3338f,-0.3780f,  -0.8602f,0.3625f,-0.3588f,  -0.9175f,0.3029f,-0.2576f,  -0.9124f,0.3336f,-0.2371f,  -0.9539f,0.2701f,-0.1307f,  -0.9470f,0.3021f,-0.1092f,  -0.9721f,0.2347f,-0.0000f,  
	-0.9633f,0.2676f,0.0221f,  -0.9717f,0.1962f,0.1313f,  -0.9612f,0.2293f,0.1536f,  -0.9532f,0.1541f,0.2602f,  -0.9410f,0.1867f,0.2823f,  -0.9172f,0.1077f,0.3837f,  -0.9036f,0.1392f,0.4052f,  -0.8648f,0.0567f,0.4989f,  -0.8502f,0.0864f,0.5194f,  -0.7978f,0.0007f,0.6030f,  -0.7825f,0.0280f,0.6220f,  -0.7180f,-0.0605f,0.6934f,  -0.7027f,-0.0361f,0.7106f,  -0.6279f,-0.1268f,0.7679f,  -0.6132f,-0.1059f,0.7828f,  
	-0.5302f,-0.1979f,0.8245f,  -0.5168f,-0.1811f,0.8368f,  -0.4277f,-0.2735f,0.8616f,  -0.4164f,-0.2609f,0.8710f,  -0.3235f,-0.3527f,0.8781f,  -0.3151f,-0.3446f,0.8843f,  -0.2206f,-0.4344f,0.8733f,  -0.2160f,-0.4308f,0.8762f,  -0.1222f,-0.5172f,0.8471f,  -0.1222f,-0.5180f,0.8466f,  -0.0312f,-0.5994f,0.7998f,  -0.0365f,-0.6043f,0.7959f,  0.1261f,-0.6617f,0.7391f,  0.1137f,-0.6560f,0.7461f,  0.1413f,-0.5527f,0.8213f,  
	0.1367f,-0.5474f,0.8257f,  0.1690f,-0.4351f,0.8844f,  0.1722f,-0.4311f,0.8857f,  0.2086f,-0.3123f,0.9268f,  0.2194f,-0.3107f,0.9248f,  0.2590f,-0.1881f,0.9474f,  0.2767f,-0.1896f,0.9421f,  0.3183f,-0.0655f,0.9457f,  0.3420f,-0.0708f,0.9370f,  0.3839f,0.0523f,0.9219f,  0.4126f,0.0429f,0.9099f,  0.4528f,0.1629f,0.8766f,  0.4853f,0.1491f,0.8615f,  0.5220f,0.2642f,0.8110f,  0.5570f,0.2461f,0.7932f,  
	0.5881f,0.3546f,0.7269f,  0.6244f,0.3326f,0.7068f,  0.6479f,0.4334f,0.6265f,  0.6842f,0.4079f,0.6045f,  0.6983f,0.4998f,0.5124f,  0.7337f,0.4715f,0.4892f,  0.7368f,0.5541f,0.3876f,  0.7704f,0.5237f,0.3636f,  0.7610f,0.5965f,0.2551f,  0.7922f,0.5649f,0.2309f,  0.7695f,0.6276f,0.1183f,  0.7976f,0.5957f,0.0944f,  0.7609f,0.6485f,-0.0196f,  0.7858f,0.6170f,-0.0426f,  0.7349f,0.6602f,-0.1552f,  
	0.7562f,0.6300f,-0.1769f,  0.6913f,0.6639f,-0.2854f,  0.7090f,0.6356f,-0.3053f,  0.6307f,0.6607f,-0.4072f,  0.6450f,0.6351f,-0.4250f,  0.5542f,0.6518f,-0.5177f,  0.5653f,0.6294f,-0.5332f,  0.4633f,0.6384f,-0.6147f,  0.4715f,0.6197f,-0.6275f,  0.3599f,0.6215f,-0.6959f,  0.3655f,0.6068f,-0.7058f,  0.2463f,0.6020f,-0.7596f,  0.2496f,0.5916f,-0.7666f,  0.1249f,0.5807f,-0.8045f,  0.1265f,0.5748f,-0.8084f,  
	-0.0015f,0.5582f,-0.8297f,  -0.0012f,0.5570f,-0.8305f,  -0.1301f,0.5351f,-0.8347f,  -0.1306f,0.5384f,-0.8325f,  -0.2579f,0.5116f,-0.8196f,  -0.2588f,0.5194f,-0.8144f,  -0.3822f,0.4879f,-0.7848f,  -0.3831f,0.5000f,-0.7767f,  -0.5002f,0.4640f,-0.7311f,  -0.5006f,0.4802f,-0.7203f,  -0.6093f,0.4398f,-0.6598f,  -0.6088f,0.4598f,-0.6465f,  -0.7070f,0.4151f,-0.5726f,  -0.7053f,0.4385f,-0.5570f,  -0.7912f,0.3895f,-0.4715f,  
	-0.7881f,0.4158f,-0.4539f,  -0.8602f,0.3625f,-0.3588f,  -0.8553f,0.3913f,-0.3395f,  -0.9124f,0.3336f,-0.2371f,  -0.9057f,0.3644f,-0.2164f,  -0.9470f,0.3021f,-0.1092f,  -0.9384f,0.3344f,-0.0875f,  -0.9633f,0.2676f,0.0221f,  -0.9527f,0.3006f,0.0444f,  -0.9612f,0.2293f,0.1536f,  -0.9487f,0.2625f,0.1761f,  -0.9410f,0.1867f,0.2823f,  -0.9269f,0.2193f,0.3046f,  -0.9036f,0.1392f,0.4052f,  -0.8880f,0.1706f,0.4269f,  
	-0.8502f,0.0864f,0.5194f,  -0.8336f,0.1160f,0.5400f,  -0.7825f,0.0280f,0.6220f,  -0.7655f,0.0551f,0.6411f,  -0.7027f,-0.0361f,0.7106f,  -0.6857f,-0.0121f,0.7278f,  -0.6132f,-0.1059f,0.7828f,  -0.5970f,-0.0855f,0.7977f,  -0.5168f,-0.1811f,0.8368f,  -0.5021f,-0.1647f,0.8490f,  -0.4164f,-0.2609f,0.8710f,  -0.4041f,-0.2489f,0.8802f,  -0.3151f,-0.3446f,0.8843f,  -0.3061f,-0.3370f,0.8903f,  -0.2160f,-0.4308f,0.8762f,  
	-0.2112f,-0.4277f,0.8789f,  -0.1222f,-0.5180f,0.8466f,  -0.1224f,-0.5192f,0.8458f,  -0.0365f,-0.6043f,0.7959f,  -0.0425f,-0.6094f,0.7918f,  0.1137f,-0.6560f,0.7461f,  0.1021f,-0.6503f,0.7528f,  0.1367f,-0.5474f,0.8257f,  0.1328f,-0.5425f,0.8295f,  0.1722f,-0.4311f,0.8857f,  0.1760f,-0.4280f,0.8865f,  0.2194f,-0.3107f,0.9248f,  0.2304f,-0.3104f,0.9223f,  0.2767f,-0.1896f,0.9421f,  0.2943f,-0.1927f,0.9361f,  
	0.3420f,-0.0708f,0.9370f,  0.3652f,-0.0779f,0.9276f,  0.4126f,0.0429f,0.9099f,  0.4403f,0.0315f,0.8973f,  0.4853f,0.1491f,0.8615f,  0.5164f,0.1334f,0.8459f,  0.5570f,0.2461f,0.7932f,  0.5901f,0.2262f,0.7750f,  0.6244f,0.3326f,0.7068f,  0.6584f,0.3089f,0.6863f,  0.6842f,0.4079f,0.6045f,  0.7181f,0.3809f,0.5824f,  0.7337f,0.4715f,0.4892f,  0.7665f,0.4421f,0.4658f,  0.7704f,0.5237f,0.3636f,  
	0.8013f,0.4925f,0.3396f,  0.7922f,0.5649f,0.2309f,  0.8207f,0.5326f,0.2069f,  0.7976f,0.5957f,0.0944f,  0.8232f,0.5633f,0.0708f,  0.7858f,0.6170f,-0.0426f,  0.8082f,0.5853f,-0.0653f,  0.7562f,0.6300f,-0.1769f,  0.7753f,0.5997f,-0.1983f,  0.7090f,0.6356f,-0.3053f,  0.7248f,0.6075f,-0.3250f,  0.6450f,0.6351f,-0.4250f,  0.6576f,0.6097f,-0.4425f,  0.5653f,0.6294f,-0.5332f,  0.5749f,0.6073f,-0.5483f,  
	0.4715f,0.6197f,-0.6275f,  0.4784f,0.6013f,-0.6400f,  0.3655f,0.6068f,-0.7058f,  0.3701f,0.5925f,-0.7155f,  0.2496f,0.5916f,-0.7666f,  0.2524f,0.5816f,-0.7733f,  0.1265f,0.5748f,-0.8084f,  0.1278f,0.5693f,-0.8121f,  -0.0012f,0.5570f,-0.8305f,  -0.0010f,0.5560f,-0.8312f,  -0.1306f,0.5384f,-0.8325f,  -0.1310f,0.5420f,-0.8301f,  -0.2588f,0.5194f,-0.8144f,  -0.2594f,0.5274f,-0.8091f,  -0.3831f,0.5000f,-0.7767f,  
	-0.3834f,0.5123f,-0.7685f,  -0.5006f,0.4802f,-0.7203f,  -0.5002f,0.4966f,-0.7093f,  -0.6088f,0.4598f,-0.6465f,  -0.6074f,0.4800f,-0.6330f,  -0.7053f,0.4385f,-0.5570f,  -0.7025f,0.4621f,-0.5412f,  -0.7881f,0.4158f,-0.4539f,  -0.7836f,0.4424f,-0.4361f,  -0.8553f,0.3913f,-0.3395f,  -0.8490f,0.4204f,-0.3200f,  -0.9057f,0.3644f,-0.2164f,  -0.8974f,0.3954f,-0.1955f,  -0.9384f,0.3344f,-0.0875f,  -0.9280f,0.3668f,-0.0656f,  
	-0.9527f,0.3006f,0.0444f,  -0.9403f,0.3338f,0.0669f,  -0.9487f,0.2625f,0.1761f,  -0.9343f,0.2957f,0.1989f,  -0.9269f,0.2193f,0.3046f,  -0.9108f,0.2520f,0.3272f,  -0.8880f,0.1706f,0.4269f,  -0.8705f,0.2020f,0.4488f,  -0.8336f,0.1160f,0.5400f,  -0.8151f,0.1454f,0.5607f,  -0.7655f,0.0551f,0.6411f,  -0.7465f,0.0819f,0.6603f,  -0.6857f,-0.0121f,0.7278f,  -0.6670f,0.0114f,0.7449f,  -0.5970f,-0.0855f,0.7977f,  
	-0.5793f,-0.0657f,0.8125f,  -0.5021f,-0.1647f,0.8490f,  -0.4862f,-0.1490f,0.8610f,  -0.4041f,-0.2489f,0.8802f,  -0.3909f,-0.2375f,0.8892f,  -0.3061f,-0.3370f,0.8903f,  -0.2966f,-0.3302f,0.8961f,  -0.2112f,-0.4277f,0.8789f,  -0.2062f,-0.4253f,0.8813f,  -0.1224f,-0.5192f,0.8458f,  -0.1228f,-0.5208f,0.8448f,  -0.0425f,-0.6094f,0.7918f,  -0.0490f,-0.6146f,0.7873f,  0.1021f,-0.6503f,0.7528f,  0.0912f,-0.6445f,0.7591f,  
	0.1328f,-0.5425f,0.8295f,  0.1296f,-0.5380f,0.8329f,  0.1760f,-0.4280f,0.8865f,  0.1803f,-0.4258f,0.8867f,  0.2304f,-0.3104f,0.9223f,  0.2417f,-0.3112f,0.9191f,  0.2943f,-0.1927f,0.9361f,  0.3117f,-0.1973f,0.9295f,  0.3652f,-0.0779f,0.9276f,  0.3878f,-0.0867f,0.9177f,  0.4403f,0.0315f,0.8973f,  0.4669f,0.0184f,0.8841f,  0.5164f,0.1334f,0.8459f,  0.5458f,0.1159f,0.8298f,  0.5901f,0.2262f,0.7750f,  
	0.6213f,0.2047f,0.7564f,  0.6584f,0.3089f,0.6863f,  0.6902f,0.2838f,0.6657f,  0.7181f,0.3809f,0.5824f,  0.7495f,0.3528f,0.5601f,  0.7665f,0.4421f,0.4658f,  0.7967f,0.4116f,0.4425f,  0.8013f,0.4925f,0.3396f,  0.8296f,0.4605f,0.3157f,  0.8207f,0.5326f,0.2069f,  0.8465f,0.4999f,0.1830f,  0.8232f,0.5633f,0.0708f,  0.8463f,0.5306f,0.0474f,  0.8082f,0.5853f,-0.0653f,  0.8282f,0.5535f,-0.0878f,  
	0.7753f,0.5997f,-0.1983f,  0.7922f,0.5695f,-0.2194f,  0.7248f,0.6075f,-0.3250f,  0.7387f,0.5795f,-0.3443f,  0.6576f,0.6097f,-0.4425f,  0.6686f,0.5845f,-0.4598f,  0.5749f,0.6073f,-0.5483f,  0.5832f,0.5854f,-0.5632f,  0.4784f,0.6013f,-0.6400f,  0.4843f,0.5831f,-0.6522f,  0.3701f,0.5925f,-0.7155f,  0.3739f,0.5784f,-0.7250f,  0.2524f,0.5816f,-0.7733f,  0.2545f,0.5719f,-0.7799f,  0.1278f,0.5693f,-0.8121f,  
	0.1287f,0.5640f,-0.8157f,  -0.0010f,0.5560f,-0.8312f,  -0.0009f,0.5553f,-0.8317f,  -0.1310f,0.5420f,-0.8301f,  -0.1312f,0.5458f,-0.8276f,  -0.2594f,0.5274f,-0.8091f,  -0.2596f,0.5357f,-0.8035f,  -0.3834f,0.5123f,-0.7685f,  -0.3831f,0.5250f,-0.7600f,  -0.5002f,0.4966f,-0.7093f,  -0.4990f,0.5133f,-0.6982f,  -0.6074f,0.4800f,-0.6330f,  -0.6049f,0.5005f,-0.6193f,  -0.7025f,0.4621f,-0.5412f,  -0.6985f,0.4860f,-0.5253f,  
	-0.7836f,0.4424f,-0.4361f,  -0.7778f,0.4693f,-0.4181f,  -0.8490f,0.4204f,-0.3200f,  -0.8412f,0.4498f,-0.3003f,  -0.8974f,0.3954f,-0.1955f,  -0.8874f,0.4267f,-0.1744f,  -0.9280f,0.3668f,-0.0656f,  -0.9158f,0.3994f,-0.0434f,  -0.9403f,0.3338f,0.0669f,  -0.9259f,0.3670f,0.0897f,  -0.9343f,0.2957f,0.1989f,  -0.9179f,0.3290f,0.2218f,  -0.9108f,0.2520f,0.3272f,  -0.8925f,0.2845f,0.3499f,  -0.8705f,0.2020f,0.4488f,  
	-0.8509f,0.2332f,0.4707f,  -0.8151f,0.1454f,0.5607f,  -0.7946f,0.1744f,0.5815f,  -0.7465f,0.0819f,0.6603f,  -0.7257f,0.1082f,0.6795f,  -0.6670f,0.0114f,0.7449f,  -0.6465f,0.0344f,0.7621f,  -0.5793f,-0.0657f,0.8125f,  -0.5600f,-0.0465f,0.8272f,  -0.4862f,-0.1490f,0.8610f,  -0.4691f,-0.1340f,0.8729f,  -0.3909f,-0.2375f,0.8892f,  -0.3768f,-0.2270f,0.8980f,  -0.2966f,-0.3302f,0.8961f,  -0.2865f,-0.3240f,0.9016f,  
	-0.2062f,-0.4253f,0.8813f,  -0.2011f,-0.4234f,0.8833f,  -0.1228f,-0.5208f,0.8448f,  -0.1236f,-0.5229f,0.8434f,  -0.0490f,-0.6146f,0.7873f,  -0.0562f,-0.6200f,0.7826f,  0.0912f,-0.6445f,0.7591f,  0.0811f,-0.6388f,0.7651f,  0.1296f,-0.5380f,0.8329f,  0.1270f,-0.5340f,0.8359f,  0.1803f,-0.4258f,0.8867f,  0.1849f,-0.4245f,0.8864f,  0.2417f,-0.3112f,0.9191f,  0.2529f,-0.3133f,0.9154f,  0.3117f,-0.1973f,0.9295f,  
	0.3287f,-0.2033f,0.9223f,  0.3878f,-0.0867f,0.9177f,  0.4095f,-0.0970f,0.9072f,  0.4669f,0.0184f,0.8841f,  0.4922f,0.0036f,0.8705f,  0.5458f,0.1159f,0.8298f,  0.5736f,0.0970f,0.8134f,  0.6213f,0.2047f,0.7564f,  0.6504f,0.1818f,0.7375f,  0.6902f,0.2838f,0.6657f,  0.7197f,0.2575f,0.6448f,  0.7495f,0.3528f,0.5601f,  0.7784f,0.3237f,0.5378f,  0.7967f,0.4116f,0.4425f,  0.8243f,0.3804f,0.4192f,  
	0.8296f,0.4605f,0.3157f,  0.8553f,0.4279f,0.2920f,  0.8465f,0.4999f,0.1830f,  0.8699f,0.4669f,0.1593f,  0.8463f,0.5306f,0.0474f,  0.8669f,0.4978f,0.0242f,  0.8282f,0.5535f,-0.0878f,  0.8460f,0.5217f,-0.1100f,  0.7922f,0.5695f,-0.2194f,  0.8071f,0.5393f,-0.2402f,  0.7387f,0.5795f,-0.3443f,  0.7508f,0.5516f,-0.3633f,  0.6686f,0.5845f,-0.4598f,  0.6780f,0.5595f,-0.4767f,  0.5832f,0.5854f,-0.5632f,  
	0.5902f,0.5638f,-0.5778f,  0.4843f,0.5831f,-0.6522f,  0.4891f,0.5653f,-0.6642f,  0.3739f,0.5784f,-0.7250f,  0.3769f,0.5647f,-0.7342f,  0.2545f,0.5719f,-0.7799f,  0.2561f,0.5625f,-0.7862f,  0.1287f,0.5640f,-0.8157f,  0.1293f,0.5591f,-0.8190f,  -0.0009f,0.5553f,-0.8317f,  -0.0008f,0.5549f,-0.8319f,  -0.1312f,0.5458f,-0.8276f,  -0.1313f,0.5499f,-0.8248f,  -0.2596f,0.5357f,-0.8035f,  -0.2594f,0.5443f,-0.7978f,  
	-0.3831f,0.5250f,-0.7600f,  -0.3822f,0.5379f,-0.7514f,  -0.4990f,0.5133f,-0.6982f,  -0.4970f,0.5303f,-0.6869f,  -0.6049f,0.5005f,-0.6193f,  -0.6015f,0.5212f,-0.6055f,  -0.6985f,0.4860f,-0.5253f,  -0.6933f,0.5101f,-0.5091f,  -0.7778f,0.4693f,-0.4181f,  -0.7705f,0.4964f,-0.3998f,  -0.8412f,0.4498f,-0.3003f,  -0.8317f,0.4793f,-0.2803f,  -0.8874f,0.4267f,-0.1744f,  -0.8756f,0.4582f,-0.1530f,  -0.9158f,0.3994f,-0.0434f,  
	-0.9016f,0.4321f,-0.0210f,  -0.9259f,0.3670f,0.0897f,  -0.9094f,0.4004f,0.1127f,  -0.9179f,0.3290f,0.2218f,  -0.8993f,0.3622f,0.2450f,  -0.8925f,0.2845f,0.3499f,  -0.8721f,0.3170f,0.3727f,  -0.8509f,0.2332f,0.4707f,  -0.8291f,0.2640f,0.4929f,  -0.7946f,0.1744f,0.5815f,  -0.7719f,0.2031f,0.6024f,  -0.7257f,0.1082f,0.6795f,  -0.7028f,0.1339f,0.6987f,  -0.6465f,0.0344f,0.7621f,  -0.6242f,0.0567f,0.7792f,  
	-0.5600f,-0.0465f,0.8272f,  -0.5392f,-0.0282f,0.8417f,  -0.4691f,-0.1340f,0.8729f,  -0.4507f,-0.1199f,0.8846f,  -0.3768f,-0.2270f,0.8980f,  -0.3619f,-0.2173f,0.9065f,  -0.2865f,-0.3240f,0.9016f,  -0.2760f,-0.3188f,0.9068f,  -0.2011f,-0.4234f,0.8833f,  -0.1960f,-0.4223f,0.8850f,  -0.1236f,-0.5229f,0.8434f,  -0.1246f,-0.5254f,0.8417f,  -0.0562f,-0.6200f,0.7826f,  -0.0641f,-0.6255f,0.7776f,  0.0811f,-0.6388f,0.7651f,  
	0.0717f,-0.6330f,0.7708f,  0.1270f,-0.5340f,0.8359f,  0.1250f,-0.5304f,0.8385f,  0.1849f,-0.4245f,0.8864f,  0.1898f,-0.4240f,0.8856f,  0.2529f,-0.3133f,0.9154f,  0.2640f,-0.3165f,0.9111f,  0.3287f,-0.2033f,0.9223f,  0.3451f,-0.2107f,0.9146f,  0.4095f,-0.0970f,0.9072f,  0.4301f,-0.1087f,0.8962f,  0.4922f,0.0036f,0.8705f,  0.5160f,-0.0124f,0.8565f,  0.5736f,0.0970f,0.8134f,  0.5995f,0.0767f,0.7967f,  
	0.6504f,0.1818f,0.7375f,  0.6775f,0.1578f,0.7184f,  0.7197f,0.2575f,0.6448f,  0.7468f,0.2302f,0.6239f,  0.7784f,0.3237f,0.5378f,  0.8049f,0.2938f,0.5155f,  0.8243f,0.3804f,0.4192f,  0.8495f,0.3487f,0.3960f,  0.8553f,0.4279f,0.2920f,  0.8786f,0.3951f,0.2684f,  0.8699f,0.4669f,0.1593f,  0.8908f,0.4336f,0.1358f,  0.8669f,0.4978f,0.0242f,  0.8853f,0.4650f,0.0013f,  0.8460f,0.5217f,-0.1100f,  
	0.8617f,0.4899f,-0.1319f,  0.8071f,0.5393f,-0.2402f,  0.8201f,0.5093f,-0.2607f,  0.7508f,0.5516f,-0.3633f,  0.7612f,0.5240f,-0.3821f,  0.6780f,0.5595f,-0.4767f,  0.6859f,0.5348f,-0.4934f,  0.5902f,0.5638f,-0.5778f,  0.5959f,0.5425f,-0.5921f,  0.4891f,0.5653f,-0.6642f,  0.4929f,0.5478f,-0.6760f,  0.3769f,0.5647f,-0.7342f,  0.3792f,0.5512f,-0.7432f,  0.2561f,0.5625f,-0.7862f,  0.2572f,0.5533f,-0.7922f,  
	0.1293f,0.5591f,-0.8190f,  0.1296f,0.5544f,-0.8221f,  -0.0008f,0.5549f,-0.8319f,  -0.0008f,0.5547f,-0.8320f,  -0.1313f,0.5499f,-0.8248f,  -0.1312f,0.5544f,-0.8219f,  -0.2594f,0.5443f,-0.7978f,  -0.2587f,0.5532f,-0.7919f,  -0.3822f,0.5379f,-0.7514f,  -0.3806f,0.5510f,-0.7426f,  -0.4970f,0.5303f,-0.6869f,  -0.4941f,0.5476f,-0.6753f,  -0.6015f,0.5212f,-0.6055f,  -0.5969f,0.5423f,-0.5913f,  -0.6933f,0.5101f,-0.5091f,  
	-0.6867f,0.5346f,-0.4926f,  -0.7705f,0.4964f,-0.3998f,  -0.7617f,0.5238f,-0.3813f,  -0.8317f,0.4793f,-0.2803f,  -0.8205f,0.5091f,-0.2600f,  -0.8756f,0.4582f,-0.1530f,  -0.8619f,0.4898f,-0.1314f,  -0.9016f,0.4321f,-0.0210f,  -0.8853f,0.4649f,0.0016f,  -0.9094f,0.4004f,0.1127f,  -0.8907f,0.4337f,0.1359f,  -0.8993f,0.3622f,0.2450f,  -0.8784f,0.3954f,0.2684f,  -0.8721f,0.3170f,0.3727f,  -0.8494f,0.3491f,0.3958f,  
	-0.8291f,0.2640f,0.4929f,  -0.8049f,0.2945f,0.5151f,  -0.7719f,0.2031f,0.6024f,  -0.7470f,0.2312f,0.6234f,  -0.7028f,0.1339f,0.6987f,  -0.6778f,0.1590f,0.7179f,  -0.6242f,0.0567f,0.7792f,  -0.6001f,0.0781f,0.7961f,  -0.5392f,-0.0282f,0.8417f,  -0.5168f,-0.0108f,0.8560f,  -0.4507f,-0.1199f,0.8846f,  -0.4311f,-0.1069f,0.8960f,  -0.3619f,-0.2173f,0.9065f,  -0.3461f,-0.2087f,0.9147f,  -0.2760f,-0.3188f,0.9068f,  
	-0.2651f,-0.3144f,0.9115f,  -0.1960f,-0.4223f,0.8850f,  -0.1909f,-0.4219f,0.8863f,  -0.1246f,-0.5254f,0.8417f,  -0.1261f,-0.5284f,0.8396f,  -0.0641f,-0.6255f,0.7776f,  -0.0726f,-0.6311f,0.7723f,  0.0717f,-0.6330f,0.7708f,  0.0630f,-0.6273f,0.7762f,  0.1250f,-0.5304f,0.8385f,  0.1234f,-0.5274f,0.8406f,  0.1898f,-0.4240f,0.8856f,  0.1948f,-0.4243f,0.8843f,  0.2640f,-0.3165f,0.9111f,  0.2748f,-0.3207f,0.9064f,  
	0.3451f,-0.2107f,0.9146f,  0.3607f,-0.2192f,0.9066f,  0.4301f,-0.1087f,0.8962f,  0.4497f,-0.1216f,0.8849f,  0.5160f,-0.0124f,0.8565f,  0.5384f,-0.0297f,0.8422f,  0.5995f,0.0767f,0.7967f,  0.6237f,0.0554f,0.7797f,  0.6775f,0.1578f,0.7184f,  0.7024f,0.1329f,0.6993f,  0.7468f,0.2302f,0.6239f,  0.7717f,0.2022f,0.6030f,  0.8049f,0.2938f,0.5155f,  0.8290f,0.2634f,0.4933f,  0.8495f,0.3487f,0.3960f,  
	0.8722f,0.3165f,0.3730f,  0.8786f,0.3951f,0.2684f,  0.8994f,0.3620f,0.2450f,  0.8908f,0.4336f,0.1358f,  0.9094f,0.4003f,0.1125f,  0.8853f,0.4650f,0.0013f,  0.9015f,0.4322f,-0.0214f,  0.8617f,0.4899f,-0.1319f,  0.8754f,0.4583f,-0.1535f,  0.8201f,0.5093f,-0.2607f,  0.8313f,0.4796f,-0.2809f,  0.7612f,0.5240f,-0.3821f,  0.7700f,0.4967f,-0.4006f,  0.6859f,0.5348f,-0.4934f,  0.6925f,0.5104f,-0.5098f,  
	0.5959f,0.5425f,-0.5921f,  0.6005f,0.5215f,-0.6062f,  0.4929f,0.5478f,-0.6760f,  0.4958f,0.5306f,-0.6875f,  0.3792f,0.5512f,-0.7432f,  0.3808f,0.5381f,-0.7520f,  0.2572f,0.5533f,-0.7922f,  0.2579f,0.5445f,-0.7981f,  0.1296f,0.5544f,-0.8221f,  0.1297f,0.5501f,-0.8250f,  -0.0008f,0.5547f,-0.8320f,  -0.0008f,0.5549f,-0.8319f,  -0.1312f,0.5544f,-0.8219f,  -0.1309f,0.5591f,-0.8187f,  -0.2587f,0.5532f,-0.7919f,  
	-0.2576f,0.5624f,-0.7857f,  -0.3806f,0.5510f,-0.7426f,  -0.3783f,0.5645f,-0.7336f,  -0.4941f,0.5476f,-0.6753f,  -0.4903f,0.5651f,-0.6635f,  -0.5969f,0.5423f,-0.5913f,  -0.5912f,0.5636f,-0.5770f,  -0.6867f,0.5346f,-0.4926f,  -0.6788f,0.5593f,-0.4759f,  -0.7617f,0.5238f,-0.3813f,  -0.7513f,0.5514f,-0.3626f,  -0.8205f,0.5091f,-0.2600f,  -0.8075f,0.5391f,-0.2395f,  -0.8619f,0.4898f,-0.1314f,  -0.8462f,0.5216f,-0.1094f,  
	-0.8853f,0.4649f,0.0016f,  -0.8669f,0.4978f,0.0246f,  -0.8907f,0.4337f,0.1359f,  -0.8698f,0.4670f,0.1594f,  -0.8784f,0.3954f,0.2684f,  -0.8552f,0.4283f,0.2920f,  -0.8494f,0.3491f,0.3958f,  -0.8242f,0.3809f,0.4190f,  -0.8049f,0.2945f,0.5151f,  -0.7784f,0.3244f,0.5374f,  -0.7470f,0.2312f,0.6234f,  -0.7198f,0.2585f,0.6443f,  -0.6778f,0.1590f,0.7179f,  -0.6507f,0.1831f,0.7369f,  -0.6001f,0.0781f,0.7961f,  
	-0.5741f,0.0985f,0.8129f,  -0.5168f,-0.0108f,0.8560f,  -0.4929f,0.0054f,0.8701f,  -0.4311f,-0.1069f,0.8960f,  -0.4103f,-0.0951f,0.9070f,  -0.3461f,-0.2087f,0.9147f,  -0.3297f,-0.2012f,0.9224f,  -0.2651f,-0.3144f,0.9115f,  -0.2539f,-0.3111f,0.9158f,  -0.1909f,-0.4219f,0.8863f,  -0.1859f,-0.4223f,0.8872f,  -0.1261f,-0.5284f,0.8396f,  -0.1280f,-0.5318f,0.8371f,  -0.0726f,-0.6311f,0.7723f,  -0.0819f,-0.6368f,0.7667f,  
	0.0630f,-0.6273f,0.7762f,  0.0550f,-0.6217f,0.7813f,  0.1234f,-0.5274f,0.8406f,  0.1223f,-0.5248f,0.8424f,  0.1948f,-0.4243f,0.8843f,  0.1998f,-0.4253f,0.8827f,  0.2748f,-0.3207f,0.9064f,  0.2852f,-0.3259f,0.9014f,  0.3607f,-0.2192f,0.9066f,  0.3756f,-0.2287f,0.8981f,  0.4497f,-0.1216f,0.8849f,  0.4680f,-0.1356f,0.8733f,  0.5384f,-0.0297f,0.8422f,  0.5592f,-0.0479f,0.8277f,  0.6237f,0.0554f,0.7797f,  
	0.6459f,0.0332f,0.7627f,  0.7024f,0.1329f,0.6993f,  0.7253f,0.1072f,0.6801f,  0.7717f,0.2022f,0.6030f,  0.7944f,0.1736f,0.5821f,  0.8290f,0.2634f,0.4933f,  0.8508f,0.2326f,0.4711f,  0.8722f,0.3165f,0.3730f,  0.8926f,0.2841f,0.3501f,  0.8994f,0.3620f,0.2450f,  0.9180f,0.3288f,0.2219f,  0.9094f,0.4003f,0.1125f,  0.9259f,0.3670f,0.0895f,  0.9015f,0.4322f,-0.0214f,  0.9157f,0.3995f,-0.0438f,  
	0.8754f,0.4583f,-0.1535f,  0.8872f,0.4269f,-0.1749f,  0.8313f,0.4796f,-0.2809f,  0.8408f,0.4500f,-0.3009f,  0.7700f,0.4967f,-0.4006f,  0.7772f,0.4696f,-0.4188f,  0.6925f,0.5104f,-0.5098f,  0.6977f,0.4863f,-0.5260f,  0.6005f,0.5215f,-0.6062f,  0.6039f,0.5008f,-0.6200f,  0.4958f,0.5306f,-0.6875f,  0.4978f,0.5136f,-0.6988f,  0.3808f,0.5381f,-0.7520f,  0.3817f,0.5252f,-0.7605f,  0.2579f,0.5445f,-0.7981f,  
	0.2581f,0.5360f,-0.8038f,  0.1297f,0.5501f,-0.8250f,  0.1296f,0.5460f,-0.8277f,  -0.0008f,0.5549f,-0.8319f,  -0.0008f,0.5554f,-0.8316f,  -0.1309f,0.5591f,-0.8187f,  -0.1303f,0.5641f,-0.8154f,  -0.2576f,0.5624f,-0.7857f,  -0.2560f,0.5718f,-0.7794f,  -0.3783f,0.5645f,-0.7336f,  -0.3753f,0.5783f,-0.7244f,  -0.4903f,0.5651f,-0.6635f,  -0.4855f,0.5830f,-0.6515f,  -0.5912f,0.5636f,-0.5770f,  -0.5842f,0.5852f,-0.5624f,  
	-0.6788f,0.5593f,-0.4759f,  -0.6694f,0.5842f,-0.4589f,  -0.7513f,0.5514f,-0.3626f,  -0.7392f,0.5792f,-0.3435f,  -0.8075f,0.5391f,-0.2395f,  -0.7925f,0.5693f,-0.2186f,  -0.8462f,0.5216f,-0.1094f,  -0.8283f,0.5534f,-0.0872f,  -0.8669f,0.4978f,0.0246f,  -0.8462f,0.5307f,0.0478f,  -0.8698f,0.4670f,0.1594f,  -0.8464f,0.5001f,0.1831f,  -0.8552f,0.4283f,0.2920f,  -0.8294f,0.4608f,0.3157f,  -0.8242f,0.3809f,0.4190f,  
	-0.7966f,0.4122f,0.4423f,  -0.7784f,0.3244f,0.5374f,  -0.7494f,0.3536f,0.5597f,  -0.7198f,0.2585f,0.6443f,  -0.6902f,0.2849f,0.6651f,  -0.6507f,0.1831f,0.7369f,  -0.6215f,0.2060f,0.7558f,  -0.5741f,0.0985f,0.8129f,  -0.5463f,0.1175f,0.8293f,  -0.4929f,0.0054f,0.8701f,  -0.4675f,0.0202f,0.8837f,  -0.4103f,-0.0951f,0.9070f,  -0.3886f,-0.0846f,0.9175f,  -0.3297f,-0.2012f,0.9224f,  -0.3126f,-0.1951f,0.9296f,  
	-0.2539f,-0.3111f,0.9158f,  -0.2426f,-0.3090f,0.9196f,  -0.1859f,-0.4223f,0.8872f,  -0.1812f,-0.4235f,0.8876f,  -0.1280f,-0.5318f,0.8371f,  -0.1304f,-0.5358f,0.8342f,  -0.0819f,-0.6368f,0.7667f,  -0.0919f,-0.6425f,0.7607f,  0.0550f,-0.6217f,0.7813f,  0.0477f,-0.6163f,0.7861f,  0.1223f,-0.5248f,0.8424f,  0.1215f,-0.5226f,0.8439f,  0.1998f,-0.4253f,0.8827f,  0.2048f,-0.4270f,0.8807f,  0.2852f,-0.3259f,0.9014f,  
	0.2952f,-0.3319f,0.8959f,  0.3756f,-0.2287f,0.8981f,  0.3897f,-0.2392f,0.8894f,  0.4680f,-0.1356f,0.8733f,  0.4851f,-0.1505f,0.8614f,  0.5592f,-0.0479f,0.8277f,  0.5784f,-0.0670f,0.8130f,  0.6459f,0.0332f,0.7627f,  0.6664f,0.0103f,0.7455f,  0.7253f,0.1072f,0.6801f,  0.7461f,0.0809f,0.6609f,  0.7944f,0.1736f,0.5821f,  0.8149f,0.1446f,0.5612f,  0.8508f,0.2326f,0.4711f,  0.8704f,0.2014f,0.4491f,  
	0.8926f,0.2841f,0.3501f,  0.9108f,0.2516f,0.3274f,  0.9180f,0.3288f,0.2219f,  0.9344f,0.2955f,0.1989f,  0.9259f,0.3670f,0.0895f,  0.9403f,0.3337f,0.0668f,  0.9157f,0.3995f,-0.0438f,  0.9279f,0.3669f,-0.0659f,  0.8872f,0.4269f,-0.1749f,  0.8972f,0.3957f,-0.1960f,  0.8408f,0.4500f,-0.3009f,  0.8486f,0.4207f,-0.3206f,  0.7772f,0.4696f,-0.4188f,  0.7830f,0.4428f,-0.4368f,  0.6977f,0.4863f,-0.5260f,  
	0.7017f,0.4625f,-0.5420f,  0.6039f,0.5008f,-0.6200f,  0.6064f,0.4804f,-0.6337f,  0.4978f,0.5136f,-0.6988f,  0.4990f,0.4970f,-0.7099f,  0.3817f,0.5252f,-0.7605f,  0.3820f,0.5127f,-0.7689f,  0.2581f,0.5360f,-0.8038f,  0.2579f,0.5277f,-0.8094f,  0.1296f,0.5460f,-0.8277f,  0.1294f,0.5422f,-0.8302f,  -0.0008f,0.5554f,-0.8316f,  -0.0007f,0.5561f,-0.8311f,  -0.1303f,0.5641f,-0.8154f,  -0.1294f,0.5694f,-0.8118f,  
	-0.2560f,0.5718f,-0.7794f,  -0.2539f,0.5816f,-0.7728f,  -0.3753f,0.5783f,-0.7244f,  -0.3715f,0.5924f,-0.7149f,  -0.4855f,0.5830f,-0.6515f,  -0.4796f,0.6011f,-0.6392f,  -0.5842f,0.5852f,-0.5624f,  -0.5759f,0.6071f,-0.5475f,  -0.6694f,0.5842f,-0.4589f,  -0.6584f,0.6095f,-0.4417f,  -0.7392f,0.5792f,-0.3435f,  -0.7254f,0.6073f,-0.3241f,  -0.7925f,0.5693f,-0.2186f,  -0.7756f,0.5996f,-0.1975f,  -0.8283f,0.5534f,-0.0872f,  
	-0.8082f,0.5853f,-0.0647f,  -0.8462f,0.5307f,0.0478f,  -0.8231f,0.5634f,0.0712f,  -0.8464f,0.5001f,0.1831f,  -0.8205f,0.5329f,0.2070f,  -0.8294f,0.4608f,0.3157f,  -0.8011f,0.4929f,0.3396f,  -0.7966f,0.4122f,0.4423f,  -0.7663f,0.4427f,0.4656f,  -0.7494f,0.3536f,0.5597f,  -0.7180f,0.3818f,0.5820f,  -0.6902f,0.2849f,0.6651f,  -0.6584f,0.3101f,0.6858f,  -0.6215f,0.2060f,0.7558f,  -0.5903f,0.2276f,0.7744f,  
	-0.5463f,0.1175f,0.8293f,  -0.5167f,0.1350f,0.8454f,  -0.4675f,0.0202f,0.8837f,  -0.4409f,0.0334f,0.8969f,  -0.3886f,-0.0846f,0.9175f,  -0.3660f,-0.0758f,0.9275f,  -0.3126f,-0.1951f,0.9296f,  -0.2951f,-0.1904f,0.9363f,  -0.2426f,-0.3090f,0.9196f,  -0.2312f,-0.3080f,0.9229f,  -0.1812f,-0.4235f,0.8876f,  -0.1767f,-0.4257f,0.8875f,  -0.1304f,-0.5358f,0.8342f,  -0.1334f,-0.5402f,0.8309f,  -0.0919f,-0.6425f,0.7607f,  
	-0.1026f,-0.6483f,0.7545f,  0.0477f,-0.6163f,0.7861f,  0.0411f,-0.6109f,0.7906f,  0.1215f,-0.5226f,0.8439f,  0.1210f,-0.5209f,0.8450f,  0.2048f,-0.4270f,0.8807f,  0.2097f,-0.4294f,0.8784f,  0.2952f,-0.3319f,0.8959f,  0.3047f,-0.3387f,0.8902f,  0.3897f,-0.2392f,0.8894f,  0.4028f,-0.2504f,0.8804f,  0.4851f,-0.1505f,0.8614f,  0.5010f,-0.1661f,0.8493f,  0.5784f,-0.0670f,0.8130f,  0.5961f,-0.0868f,0.7982f,  
	0.6664f,0.0103f,0.7455f,  0.6851f,-0.0132f,0.7283f,  0.7461f,0.0809f,0.6609f,  0.7650f,0.0542f,0.6417f,  0.8149f,0.1446f,0.5612f,  0.8334f,0.1153f,0.5405f,  0.8704f,0.2014f,0.4491f,  0.8880f,0.1702f,0.4273f,  0.9108f,0.2516f,0.3274f,  0.9269f,0.2190f,0.3049f,  0.9344f,0.2955f,0.1989f,  0.9488f,0.2623f,0.1762f,  0.9403f,0.3337f,0.0668f,  0.9527f,0.3006f,0.0442f,  0.9279f,0.3669f,-0.0659f,  
	0.9383f,0.3345f,-0.0878f,  0.8972f,0.3957f,-0.1960f,  0.9055f,0.3646f,-0.2169f,  0.8486f,0.4207f,-0.3206f,  0.8549f,0.3916f,-0.3401f,  0.7830f,0.4428f,-0.4368f,  0.7875f,0.4162f,-0.4546f,  0.7017f,0.4625f,-0.5420f,  0.7045f,0.4389f,-0.5577f,  0.6064f,0.4804f,-0.6337f,  0.6078f,0.4602f,-0.6472f,  0.4990f,0.4970f,-0.7099f,  0.4994f,0.4806f,-0.7209f,  0.3820f,0.5127f,-0.7689f,  0.3817f,0.5004f,-0.7771f,  
	0.2579f,0.5277f,-0.8094f,  0.2573f,0.5197f,-0.8147f,  0.1294f,0.5422f,-0.8302f,  0.1290f,0.5387f,-0.8326f,  -0.0007f,0.5561f,-0.8311f,  -0.0005f,0.5572f,-0.8304f,  -0.1294f,0.5694f,-0.8118f,  -0.1281f,0.5750f,-0.8081f,  -0.2539f,0.5816f,-0.7728f,  -0.2512f,0.5917f,-0.7660f,  -0.3715f,0.5924f,-0.7149f,  -0.3669f,0.6068f,-0.7051f,  -0.4796f,0.6011f,-0.6392f,  -0.4727f,0.6196f,-0.6266f,  -0.5759f,0.6071f,-0.5475f,  
	-0.5663f,0.6293f,-0.5323f,  -0.6584f,0.6095f,-0.4417f,  -0.6458f,0.6349f,-0.4241f,  -0.7254f,0.6073f,-0.3241f,  -0.7095f,0.6355f,-0.3045f,  -0.7756f,0.5996f,-0.1975f,  -0.7565f,0.6299f,-0.1761f,  -0.8082f,0.5853f,-0.0647f,  -0.7858f,0.6170f,-0.0420f,  -0.8231f,0.5634f,0.0712f,  -0.7975f,0.5958f,0.0948f,  -0.8205f,0.5329f,0.2070f,  -0.7920f,0.5651f,0.2311f,  -0.8011f,0.4929f,0.3396f,  -0.7701f,0.5242f,0.3636f,  
	-0.7663f,0.4427f,0.4656f,  -0.7334f,0.4722f,0.4889f,  -0.7180f,0.3818f,0.5820f,  -0.6840f,0.4088f,0.6042f,  -0.6584f,0.3101f,0.6858f,  -0.6243f,0.3338f,0.7063f,  -0.5903f,0.2276f,0.7744f,  -0.5571f,0.2476f,0.7927f,  -0.5167f,0.1350f,0.8454f,  -0.4856f,0.1509f,0.8611f,  -0.4409f,0.0334f,0.8969f,  -0.4130f,0.0449f,0.9096f,  -0.3660f,-0.0758f,0.9275f,  -0.3426f,-0.0686f,0.9370f,  -0.2951f,-0.1904f,0.9363f,  
	-0.2774f,-0.1872f,0.9423f,  -0.2312f,-0.3080f,0.9229f,  -0.2200f,-0.3083f,0.9255f,  -0.1767f,-0.4257f,0.8875f,  -0.1728f,-0.4287f,0.8868f,  -0.1334f,-0.5402f,0.8309f,  -0.1371f,-0.5451f,0.8271f,  -0.1026f,-0.6483f,0.7545f,  -0.1140f,-0.6540f,0.7479f,  0.0411f,-0.6109f,0.7906f,  0.0351f,-0.6058f,0.7949f,  0.1210f,-0.5209f,0.8450f,  0.1207f,-0.5196f,0.8459f,  0.2097f,-0.4294f,0.8784f,  0.2145f,-0.4323f,0.8758f,  
	0.3047f,-0.3387f,0.8902f,  0.3136f,-0.3461f,0.8842f,  0.4028f,-0.2504f,0.8804f,  0.4151f,-0.2623f,0.8712f,  0.5010f,-0.1661f,0.8493f,  0.5157f,-0.1824f,0.8372f,  0.5961f,-0.0868f,0.7982f,  0.6123f,-0.1071f,0.7833f,  0.6851f,-0.0132f,0.7283f,  0.7020f,-0.0371f,0.7112f,  0.7650f,0.0542f,0.6417f,  0.7821f,0.0272f,0.6226f,  0.8334f,0.1153f,0.5405f,  0.8499f,0.0858f,0.5199f,  0.8880f,0.1702f,0.4273f,  
	0.9035f,0.1387f,0.4056f,  0.9269f,0.2190f,0.3049f,  0.9410f,0.1864f,0.2825f,  0.9488f,0.2623f,0.1762f,  0.9612f,0.2292f,0.1536f,  0.9527f,0.3006f,0.0442f,  0.9633f,0.2676f,0.0219f,  0.9383f,0.3345f,-0.0878f,  0.9469f,0.3023f,-0.1095f,  0.9055f,0.3646f,-0.2169f,  0.9122f,0.3338f,-0.2376f,  0.8549f,0.3916f,-0.3401f,  0.8598f,0.3628f,-0.3594f,  0.7875f,0.4162f,-0.4546f,  0.7906f,0.3899f,-0.4722f,  
	0.7045f,0.4389f,-0.5577f,  0.7062f,0.4155f,-0.5733f,  0.6078f,0.4602f,-0.6472f,  0.6082f,0.4403f,-0.6604f,  0.4994f,0.4806f,-0.7209f,  0.4990f,0.4644f,-0.7316f,  0.3817f,0.5004f,-0.7771f,  0.3808f,0.4883f,-0.7852f,  0.2573f,0.5197f,-0.8147f,  0.2564f,0.5120f,-0.8199f,  0.1290f,0.5387f,-0.8326f,  0.1284f,0.5354f,-0.8348f,  -0.0005f,0.5572f,-0.8304f,  -0.0002f,0.5585f,-0.8295f,  -0.1281f,0.5750f,-0.8081f,  
	-0.1265f,0.5809f,-0.8041f,  -0.2512f,0.5917f,-0.7660f,  -0.2478f,0.6021f,-0.7590f,  -0.3669f,0.6068f,-0.7051f,  -0.3613f,0.6215f,-0.6951f,  -0.4727f,0.6196f,-0.6266f,  -0.4645f,0.6383f,-0.6138f,  -0.5663f,0.6293f,-0.5323f,  -0.5552f,0.6517f,-0.5168f,  -0.6458f,0.6349f,-0.4241f,  -0.6314f,0.6605f,-0.4062f,  -0.7095f,0.6355f,-0.3045f,  -0.6917f,0.6638f,-0.2845f,  -0.7565f,0.6299f,-0.1761f,  -0.7351f,0.6602f,-0.1544f,  
	-0.7858f,0.6170f,-0.0420f,  -0.7609f,0.6485f,-0.0190f,  -0.7975f,0.5958f,0.0948f,  -0.7693f,0.6278f,0.1187f,  -0.7920f,0.5651f,0.2311f,  -0.7607f,0.5968f,0.2553f,  -0.7701f,0.5242f,0.3636f,  -0.7364f,0.5546f,0.3875f,  -0.7334f,0.4722f,0.4889f,  -0.6979f,0.5006f,0.5122f,  -0.6840f,0.4088f,0.6042f,  -0.6476f,0.4343f,0.6261f,  -0.6243f,0.3338f,0.7063f,  -0.5880f,0.3559f,0.7264f,  -0.5571f,0.2476f,0.7927f,  
	-0.5221f,0.2657f,0.8105f,  -0.4856f,0.1509f,0.8611f,  -0.4531f,0.1647f,0.8761f,  -0.4130f,0.0449f,0.9096f,  -0.3842f,0.0544f,0.9216f,  -0.3426f,-0.0686f,0.9370f,  -0.3188f,-0.0632f,0.9457f,  -0.2774f,-0.1872f,0.9423f,  -0.2596f,-0.1856f,0.9477f,  -0.2200f,-0.3083f,0.9255f,  -0.2091f,-0.3099f,0.9275f,  -0.1728f,-0.4287f,0.8868f,  -0.1694f,-0.4326f,0.8855f,  -0.1371f,-0.5451f,0.8271f,  -0.1416f,-0.5504f,0.8228f,  
	-0.1140f,-0.6540f,0.7479f,  -0.1262f,-0.6596f,0.7409f,  0.0351f,-0.6058f,0.7949f,  0.0296f,-0.6008f,0.7988f,  0.1207f,-0.5196f,0.8459f,  0.1207f,-0.5186f,0.8464f,  0.2145f,-0.4323f,0.8758f,  0.2191f,-0.4358f,0.8730f,  0.3136f,-0.3461f,0.8842f,  0.3220f,-0.3541f,0.8780f,  0.4151f,-0.2623f,0.8712f,  0.4264f,-0.2748f,0.8618f,  0.5157f,-0.1824f,0.8372f,  0.5291f,-0.1991f,0.8249f,  0.6123f,-0.1071f,0.7833f,  
	0.6270f,-0.1278f,0.7684f,  0.7020f,-0.0371f,0.7112f,  0.7173f,-0.0614f,0.6940f,  0.7821f,0.0272f,0.6226f,  0.7973f,-0.0001f,0.6036f,  0.8499f,0.0858f,0.5199f,  0.8646f,0.0561f,0.4994f,  0.9035f,0.1387f,0.4056f,  0.9170f,0.1073f,0.3841f,  0.9410f,0.1864f,0.2825f,  0.9532f,0.1538f,0.2604f,  0.9612f,0.2292f,0.1536f,  0.9717f,0.1961f,0.1313f,  0.9633f,0.2676f,0.0219f,  0.9721f,0.2348f,-0.0002f,  
	0.9469f,0.3023f,-0.1095f,  0.9538f,0.2702f,-0.1310f,  0.9122f,0.3338f,-0.2376f,  0.9173f,0.3032f,-0.2581f,  0.8598f,0.3628f,-0.3594f,  0.8632f,0.3342f,-0.3785f,  0.7906f,0.3899f,-0.4722f,  0.7924f,0.3637f,-0.4896f,  0.7062f,0.4155f,-0.5733f,  0.7067f,0.3924f,-0.5887f,  0.6082f,0.4403f,-0.6604f,  0.6078f,0.4206f,-0.6736f,  0.4990f,0.4644f,-0.7316f,  0.4979f,0.4485f,-0.7422f,  0.3808f,0.4883f,-0.7852f,  
	0.3795f,0.4765f,-0.7931f,  0.2564f,0.5120f,-0.8199f,  0.2552f,0.5045f,-0.8249f,  0.1284f,0.5354f,-0.8348f,  0.1278f,0.5324f,-0.8368f,  -0.0002f,0.5585f,-0.8295f,  0.0003f,0.5601f,-0.8284f,  -0.1265f,0.5809f,-0.8041f,  -0.1245f,0.5871f,-0.7999f,  -0.2478f,0.6021f,-0.7590f,  -0.2437f,0.6128f,-0.7517f,  -0.3613f,0.6215f,-0.6951f,  -0.3548f,0.6365f,-0.6849f,  -0.4645f,0.6383f,-0.6138f,  -0.4551f,0.6573f,-0.6007f,  
	-0.5552f,0.6517f,-0.5168f,  -0.5425f,0.6743f,-0.5010f,  -0.6314f,0.6605f,-0.4062f,  -0.6152f,0.6863f,-0.3880f,  -0.6917f,0.6638f,-0.2845f,  -0.6718f,0.6920f,-0.2642f,  -0.7351f,0.6602f,-0.1544f,  -0.7114f,0.6902f,-0.1325f,  -0.7609f,0.6485f,-0.0190f,  -0.7335f,0.6797f,0.0042f,  -0.7693f,0.6278f,0.1187f,  -0.7384f,0.6591f,0.1427f,  -0.7607f,0.5968f,0.2553f,  -0.7267f,0.6275f,0.2795f,  -0.7364f,0.5546f,0.3875f,  
	-0.6999f,0.5838f,0.4115f,  -0.6979f,0.5006f,0.5122f,  -0.6598f,0.5274f,0.5352f,  -0.6476f,0.4343f,0.6261f,  -0.6088f,0.4581f,0.6477f,  -0.5880f,0.3559f,0.7264f,  -0.5496f,0.3760f,0.7460f,  -0.5221f,0.2657f,0.8105f,  -0.4853f,0.2817f,0.8277f,  -0.4531f,0.1647f,0.8761f,  -0.4193f,0.1764f,0.8905f,  -0.3842f,0.0544f,0.9216f,  -0.3547f,0.0618f,0.9329f,  -0.3188f,-0.0632f,0.9457f,  -0.2947f,-0.0598f,0.9537f,  
	-0.2596f,-0.1856f,0.9477f,  -0.2420f,-0.1858f,0.9523f,  -0.2091f,-0.3099f,0.9275f,  -0.1988f,-0.3127f,0.9288f,  -0.1694f,-0.4326f,0.8855f,  -0.1667f,-0.4373f,0.8837f,  -0.1416f,-0.5504f,0.8228f,  -0.1468f,-0.5560f,0.8181f,  -0.1262f,-0.6596f,0.7409f,  -0.1392f,-0.6652f,0.7336f,  0.0296f,-0.6008f,0.7988f,  0.0248f,-0.5961f,0.8026f,  0.1207f,-0.5186f,0.8464f,  0.1208f,-0.5181f,0.8467f,  0.2191f,-0.4358f,0.8730f,  
	0.2235f,-0.4397f,0.8699f,  0.3220f,-0.3541f,0.8780f,  0.3298f,-0.3626f,0.8717f,  0.4264f,-0.2748f,0.8618f,  0.4368f,-0.2878f,0.8523f,  0.5291f,-0.1991f,0.8249f,  0.5413f,-0.2163f,0.8125f,  0.6270f,-0.1278f,0.7684f,  0.6403f,-0.1489f,0.7535f,  0.7173f,-0.0614f,0.6940f,  0.7310f,-0.0859f,0.6769f,  0.7973f,-0.0001f,0.6036f,  0.8108f,-0.0275f,0.5846f,  0.8646f,0.0561f,0.4994f,  0.8774f,0.0264f,0.4790f,  
	0.9170f,0.1073f,0.3841f,  0.9288f,0.0758f,0.3627f,  0.9532f,0.1538f,0.2604f,  0.9636f,0.1213f,0.2384f,  0.9717f,0.1961f,0.1313f,  0.9805f,0.1632f,0.1091f,  0.9721f,0.2348f,-0.0002f,  0.9791f,0.2020f,-0.0221f,  0.9538f,0.2702f,-0.1310f,  0.9592f,0.2384f,-0.1523f,  0.9173f,0.3032f,-0.2581f,  0.9209f,0.2727f,-0.2784f,  0.8632f,0.3342f,-0.3785f,  0.8652f,0.3057f,-0.3975f,  0.7924f,0.3637f,-0.4896f,  
	0.7931f,0.3378f,-0.5069f,  0.7067f,0.3924f,-0.5887f,  0.7062f,0.3695f,-0.6040f,  0.6078f,0.4206f,-0.6736f,  0.6064f,0.4011f,-0.6866f,  0.4979f,0.4485f,-0.7422f,  0.4960f,0.4329f,-0.7527f,  0.3795f,0.4765f,-0.7931f,  0.3775f,0.4649f,-0.8008f,  0.2552f,0.5045f,-0.8249f,  0.2536f,0.4973f,-0.8297f,  0.1278f,0.5324f,-0.8368f,  0.1271f,0.5297f,-0.8386f,  0.0003f,0.5601f,-0.8284f,  0.0009f,0.5620f,-0.8271f,  
	-0.1245f,0.5871f,-0.7999f,  -0.1220f,0.5936f,-0.7955f,  -0.2437f,0.6128f,-0.7517f,  -0.2389f,0.6238f,-0.7442f,  -0.3548f,0.6365f,-0.6849f,  -0.3472f,0.6518f,-0.6743f,  -0.4551f,0.6573f,-0.6007f,  -0.4443f,0.6766f,-0.5873f,  -0.5425f,0.6743f,-0.5010f,  -0.5281f,0.6971f,-0.4849f,  -0.6152f,0.6863f,-0.3880f,  -0.5970f,0.7120f,-0.3695f,  -0.6718f,0.6920f,-0.2642f,  -0.6496f,0.7201f,-0.2437f,  -0.7114f,0.6902f,-0.1325f,  
	-0.6852f,0.7200f,-0.1103f,  -0.7335f,0.6797f,0.0042f,  -0.7034f,0.7103f,0.0276f,  -0.7384f,0.6591f,0.1427f,  -0.7047f,0.6896f,0.1667f,  -0.7267f,0.6275f,0.2795f,  -0.6900f,0.6570f,0.3037f,  -0.6999f,0.5838f,0.4115f,  -0.6608f,0.6115f,0.4352f,  -0.6598f,0.5274f,0.5352f,  -0.6192f,0.5525f,0.5580f,  -0.6088f,0.4581f,0.6477f,  -0.5677f,0.4799f,0.6689f,  -0.5496f,0.3760f,0.7460f,  -0.5093f,0.3939f,0.7651f,  
	-0.4853f,0.2817f,0.8277f,  -0.4472f,0.2954f,0.8442f,  -0.4193f,0.1764f,0.8905f,  -0.3846f,0.1858f,0.9042f,  -0.3547f,0.0618f,0.9329f,  -0.3248f,0.0669f,0.9434f,  -0.2947f,-0.0598f,0.9537f,  -0.2706f,-0.0585f,0.9609f,  -0.2420f,-0.1858f,0.9523f,  -0.2247f,-0.1875f,0.9562f,  -0.1988f,-0.3127f,0.9288f,  -0.1891f,-0.3169f,0.9294f,  -0.1667f,-0.4373f,0.8837f,  -0.1649f,-0.4429f,0.8812f,  -0.1468f,-0.5560f,0.8181f,  
	-0.1529f,-0.5620f,0.8129f,  -0.1392f,-0.6652f,0.7336f,  -0.1528f,-0.6706f,0.7259f,  0.0248f,-0.5961f,0.8026f,  0.0205f,-0.5915f,0.8060f,  0.1208f,-0.5181f,0.8467f,  0.1211f,-0.5179f,0.8468f,  0.2235f,-0.4397f,0.8699f,  0.2276f,-0.4441f,0.8666f,  0.3298f,-0.3626f,0.8717f,  0.3370f,-0.3715f,0.8651f,  0.4368f,-0.2878f,0.8523f,  0.4463f,-0.3011f,0.8427f,  0.5413f,-0.2163f,0.8125f,  0.5524f,-0.2339f,0.8001f,  
	0.6403f,-0.1489f,0.7535f,  0.6523f,-0.1703f,0.7386f,  0.7310f,-0.0859f,0.6769f,  0.7432f,-0.1106f,0.6598f,  0.8108f,-0.0275f,0.5846f,  0.8227f,-0.0550f,0.5658f,  0.8774f,0.0264f,0.4790f,  0.8885f,-0.0034f,0.4588f,  0.9288f,0.0758f,0.3627f,  0.9389f,0.0444f,0.3414f,  0.9636f,0.1213f,0.2384f,  0.9722f,0.0889f,0.2166f,  0.9805f,0.1632f,0.1091f,  0.9876f,0.1304f,0.0871f,  0.9791f,0.2020f,-0.0221f,  
	0.9846f,0.1695f,-0.0439f,  0.9592f,0.2384f,-0.1523f,  0.9629f,0.2066f,-0.1735f,  0.9209f,0.2727f,-0.2784f,  0.9231f,0.2425f,-0.2986f,  0.8652f,0.3057f,-0.3975f,  0.8658f,0.2775f,-0.4164f,  0.7931f,0.3378f,-0.5069f,  0.7924f,0.3121f,-0.5241f,  0.7062f,0.3695f,-0.6040f,  0.7045f,0.3468f,-0.6192f,  0.6064f,0.4011f,-0.6866f,  0.6041f,0.3818f,-0.6995f,  0.4960f,0.4329f,-0.7527f,  0.4935f,0.4174f,-0.7631f,  
	0.3775f,0.4649f,-0.8008f,  0.3751f,0.4536f,-0.8084f,  0.2536f,0.4973f,-0.8297f,  0.2518f,0.4903f,-0.8344f,  0.1271f,0.5297f,-0.8386f,  0.1264f,0.5273f,-0.8402f,  0.0009f,0.5620f,-0.8271f,  0.0018f,0.5642f,-0.8256f,  -0.1220f,0.5936f,-0.7955f,  -0.1190f,0.6003f,-0.7908f,  -0.2389f,0.6238f,-0.7442f,  -0.2333f,0.6350f,-0.7364f,  -0.3472f,0.6518f,-0.6743f,  -0.3385f,0.6673f,-0.6635f,  -0.4443f,0.6766f,-0.5873f,  
	-0.4320f,0.6960f,-0.5735f,  -0.5281f,0.6971f,-0.4849f,  -0.5120f,0.7199f,-0.4685f,  -0.5970f,0.7120f,-0.3695f,  -0.5768f,0.7377f,-0.3508f,  -0.6496f,0.7201f,-0.2437f,  -0.6251f,0.7480f,-0.2229f,  -0.6852f,0.7200f,-0.1103f,  -0.6564f,0.7492f,-0.0879f,  -0.7034f,0.7103f,0.0276f,  -0.6706f,0.7400f,0.0511f,  -0.7047f,0.6896f,0.1667f,  -0.6682f,0.7191f,0.1908f,  -0.6900f,0.6570f,0.3037f,  -0.6505f,0.6852f,0.3278f,  
	-0.6608f,0.6115f,0.4352f,  -0.6190f,0.6375f,0.4587f,  -0.6192f,0.5525f,0.5580f,  -0.5762f,0.5756f,0.5803f,  -0.5677f,0.4799f,0.6689f,  -0.5246f,0.4994f,0.6895f,  -0.5093f,0.3939f,0.7651f,  -0.4675f,0.4093f,0.7835f,  -0.4472f,0.2954f,0.8442f,  -0.4079f,0.3065f,0.8600f,  -0.3846f,0.1858f,0.9042f,  -0.3493f,0.1926f,0.9170f,  -0.3248f,0.0669f,0.9434f,  -0.2947f,0.0698f,0.9530f,  -0.2706f,-0.0585f,0.9609f,  
	-0.2469f,-0.0592f,0.9672f,  -0.2247f,-0.1875f,0.9562f,  -0.2082f,-0.1910f,0.9593f,  -0.1891f,-0.3169f,0.9294f,  -0.1803f,-0.3223f,0.9293f,  -0.1649f,-0.4429f,0.8812f,  -0.1641f,-0.4493f,0.8782f,  -0.1529f,-0.5620f,0.8129f,  -0.1599f,-0.5683f,0.8072f,  -0.1528f,-0.6706f,0.7259f,  -0.1672f,-0.6757f,0.7179f,  0.0205f,-0.5915f,0.8060f,  0.0167f,-0.5872f,0.8093f,  0.1211f,-0.5179f,0.8468f,  0.1215f,-0.5180f,0.8467f,  
	0.2276f,-0.4441f,0.8666f,  0.2315f,-0.4488f,0.8631f,  0.3370f,-0.3715f,0.8651f,  0.3437f,-0.3808f,0.8584f,  0.4463f,-0.3011f,0.8427f,  0.4549f,-0.3148f,0.8330f,  0.5524f,-0.2339f,0.8001f,  0.5623f,-0.2517f,0.7877f,  0.6523f,-0.1703f,0.7386f,  0.6629f,-0.1918f,0.7237f,  0.7432f,-0.1106f,0.6598f,  0.7540f,-0.1355f,0.6428f,  0.8227f,-0.0550f,0.5658f,  0.8331f,-0.0826f,0.5470f,  0.8885f,-0.0034f,0.4588f,  
	0.8981f,-0.0332f,0.4386f,  0.9389f,0.0444f,0.3414f,  0.9472f,0.0130f,0.3203f,  0.9722f,0.0889f,0.2166f,  0.9792f,0.0565f,0.1948f,  0.9876f,0.1304f,0.0871f,  0.9931f,0.0977f,0.0652f,  0.9846f,0.1695f,-0.0439f,  0.9884f,0.1370f,-0.0656f,  0.9629f,0.2066f,-0.1735f,  0.9651f,0.1750f,-0.1946f,  0.9231f,0.2425f,-0.2986f,  0.9238f,0.2123f,-0.3187f,  0.8658f,0.2775f,-0.4164f,  0.8652f,0.2493f,-0.4351f,  
	0.7924f,0.3121f,-0.5241f,  0.7906f,0.2865f,-0.5411f,  0.7045f,0.3468f,-0.6192f,  0.7019f,0.3242f,-0.6342f,  0.6041f,0.3818f,-0.6995f,  0.6009f,0.3627f,-0.7123f,  0.4935f,0.4174f,-0.7631f,  0.4902f,0.4021f,-0.7733f,  0.3751f,0.4536f,-0.8084f,  0.3722f,0.4425f,-0.8159f,  0.2518f,0.4903f,-0.8344f,  0.2497f,0.4836f,-0.8389f,  0.1264f,0.5273f,-0.8402f,  0.1256f,0.5252f,-0.8417f,  0.0018f,0.5642f,-0.8256f,  
	0.0029f,0.5667f,-0.8239f,  -0.1190f,0.6003f,-0.7908f,  -0.1155f,0.6074f,-0.7859f,  -0.2333f,0.6350f,-0.7364f,  -0.2269f,0.6465f,-0.7284f,  -0.3385f,0.6673f,-0.6635f,  -0.3286f,0.6830f,-0.6524f,  -0.4320f,0.6960f,-0.5735f,  -0.4183f,0.7155f,-0.5595f,  -0.5120f,0.7199f,-0.4685f,  -0.4941f,0.7427f,-0.4519f,  -0.5768f,0.7377f,-0.3508f,  -0.5544f,0.7633f,-0.3318f,  -0.6251f,0.7480f,-0.2229f,  -0.5982f,0.7755f,-0.2020f,  
	-0.6564f,0.7492f,-0.0879f,  -0.6251f,0.7778f,-0.0654f,  -0.6706f,0.7400f,0.0511f,  -0.6351f,0.7688f,0.0746f,  -0.6682f,0.7191f,0.1908f,  -0.6290f,0.7471f,0.2148f,  -0.6505f,0.6852f,0.3278f,  -0.6083f,0.7116f,0.3516f,  -0.6190f,0.6375f,0.4587f,  -0.5748f,0.6615f,0.4818f,  -0.5762f,0.5756f,0.5803f,  -0.5309f,0.5963f,0.6021f,  -0.5246f,0.4994f,0.6895f,  -0.4797f,0.5163f,0.7095f,  -0.4675f,0.4093f,0.7835f,  
	-0.4243f,0.4220f,0.8012f,  -0.4079f,0.3065f,0.8600f,  -0.3679f,0.3149f,0.8749f,  -0.3493f,0.1926f,0.9170f,  -0.3137f,0.1968f,0.9289f,  -0.2947f,0.0698f,0.9530f,  -0.2648f,0.0702f,0.9617f,  -0.2469f,-0.0592f,0.9672f,  -0.2237f,-0.0619f,0.9727f,  -0.2082f,-0.1910f,0.9593f,  -0.1925f,-0.1961f,0.9615f,  -0.1803f,-0.3223f,0.9293f,  -0.1725f,-0.3288f,0.9285f,  -0.1641f,-0.4493f,0.8782f,  -0.1643f,-0.4562f,0.8746f,  
	-0.1599f,-0.5683f,0.8072f,  -0.1678f,-0.5747f,0.8010f,  -0.1672f,-0.6757f,0.7179f,  -0.1823f,-0.6807f,0.7095f,  0.0167f,-0.5872f,0.8093f,  0.0134f,-0.5831f,0.8123f,  0.1215f,-0.5180f,0.8467f,  0.1221f,-0.5185f,0.8463f,  0.2315f,-0.4488f,0.8631f,  0.2352f,-0.4539f,0.8595f,  0.3437f,-0.3808f,0.8584f,  0.3498f,-0.3904f,0.8516f,  0.4549f,-0.3148f,0.8330f,  0.4628f,-0.3288f,0.8232f,  0.5623f,-0.2517f,0.7877f,  
	0.5712f,-0.2698f,0.7752f,  0.6629f,-0.1918f,0.7237f,  0.6723f,-0.2136f,0.7088f,  0.7540f,-0.1355f,0.6428f,  0.7633f,-0.1604f,0.6258f,  0.8331f,-0.0826f,0.5470f,  0.8419f,-0.1103f,0.5283f,  0.8981f,-0.0332f,0.4386f,  0.9060f,-0.0630f,0.4185f,  0.9472f,0.0130f,0.3203f,  0.9540f,-0.0183f,0.2992f,  0.9792f,0.0565f,0.1948f,  0.9846f,0.0242f,0.1732f,  0.9931f,0.0977f,0.0652f,  0.9969f,0.0650f,0.0434f,  
	0.9884f,0.1370f,-0.0656f,  0.9907f,0.1046f,-0.0872f,  0.9651f,0.1750f,-0.1946f,  0.9659f,0.1435f,-0.2156f,  0.9238f,0.2123f,-0.3187f,  0.9230f,0.1823f,-0.3388f,  0.8652f,0.2493f,-0.4351f,  0.8632f,0.2213f,-0.4538f,  0.7906f,0.2865f,-0.5411f,  0.7876f,0.2610f,-0.5581f,  0.7019f,0.3242f,-0.6342f,  0.6981f,0.3018f,-0.6493f,  0.6009f,0.3627f,-0.7123f,  0.5969f,0.3438f,-0.7250f,  0.4902f,0.4021f,-0.7733f,  
	0.4862f,0.3871f,-0.7835f,  0.3722f,0.4425f,-0.8159f,  0.3687f,0.4316f,-0.8233f,  0.2497f,0.4836f,-0.8389f,  0.2473f,0.4771f,-0.8433f,  0.1256f,0.5252f,-0.8417f,  0.1249f,0.5233f,-0.8430f,  0.0029f,0.5667f,-0.8239f,  0.0043f,0.5694f,-0.8220f,  -0.1155f,0.6074f,-0.7859f,  -0.1114f,0.6147f,-0.7808f,  -0.2269f,0.6465f,-0.7284f,  -0.2195f,0.6583f,-0.7201f,  -0.3286f,0.6830f,-0.6524f,  -0.3174f,0.6988f,-0.6410f,  
	-0.4183f,0.7155f,-0.5595f,  -0.4029f,0.7351f,-0.5453f,  -0.4941f,0.7427f,-0.4519f,  -0.4742f,0.7655f,-0.4350f,  -0.5544f,0.7633f,-0.3318f,  -0.5298f,0.7884f,-0.3125f,  -0.5982f,0.7755f,-0.2020f,  -0.5689f,0.8023f,-0.1808f,  -0.6251f,0.7778f,-0.0654f,  -0.5911f,0.8055f,-0.0429f,  -0.6351f,0.7688f,0.0746f,  -0.5969f,0.7963f,0.0981f,  -0.6290f,0.7471f,0.2148f,  -0.5871f,0.7735f,0.2385f,  -0.6083f,0.7116f,0.3516f,  
	-0.5635f,0.7360f,0.3751f,  -0.5748f,0.6615f,0.4818f,  -0.5282f,0.6830f,0.5044f,  -0.5309f,0.5963f,0.6021f,  -0.4838f,0.6144f,0.6233f,  -0.4797f,0.5163f,0.7095f,  -0.4334f,0.5304f,0.7286f,  -0.4243f,0.4220f,0.8012f,  -0.3802f,0.4319f,0.8179f,  -0.3679f,0.3149f,0.8749f,  -0.3274f,0.3205f,0.8889f,  -0.3137f,0.1968f,0.9289f,  -0.2782f,0.1984f,0.9398f,  -0.2648f,0.0702f,0.9617f,  -0.2355f,0.0683f,0.9695f,  
	-0.2237f,-0.0619f,0.9727f,  -0.2015f,-0.0667f,0.9772f,  -0.1925f,-0.1961f,0.9615f,  -0.1779f,-0.2028f,0.9629f,  -0.1725f,-0.3288f,0.9285f,  -0.1659f,-0.3364f,0.9270f,  -0.1643f,-0.4562f,0.8746f,  -0.1656f,-0.4638f,0.8703f,  -0.1678f,-0.5747f,0.8010f,  -0.1767f,-0.5813f,0.7943f,  -0.1823f,-0.6807f,0.7095f,  -0.1980f,-0.6854f,0.7008f,  0.0134f,-0.5831f,0.8123f,  0.0106f,-0.5793f,0.8151f,  0.1221f,-0.5185f,0.8463f,  
	0.1227f,-0.5193f,0.8458f,  0.2352f,-0.4539f,0.8595f,  0.2386f,-0.4593f,0.8556f,  0.3498f,-0.3904f,0.8516f,  0.3553f,-0.4003f,0.8447f,  0.4628f,-0.3288f,0.8232f,  0.4698f,-0.3431f,0.8134f,  0.5712f,-0.2698f,0.7752f,  0.5791f,-0.2880f,0.7627f,  0.6723f,-0.2136f,0.7088f,  0.6804f,-0.2355f,0.6939f,  0.7633f,-0.1604f,0.6258f,  0.7713f,-0.1855f,0.6089f,  0.8419f,-0.1103f,0.5283f,  0.8493f,-0.1380f,0.5096f,  
	0.9060f,-0.0630f,0.4185f,  0.9124f,-0.0929f,0.3985f,  0.9540f,-0.0183f,0.2992f,  0.9592f,-0.0497f,0.2783f,  0.9846f,0.0242f,0.1732f,  0.9884f,-0.0081f,0.1516f,  0.9969f,0.0650f,0.0434f,  0.9992f,0.0324f,0.0216f,  0.9907f,0.1046f,-0.0872f,  0.9914f,0.0723f,-0.1088f,  0.9659f,0.1435f,-0.2156f,  0.9651f,0.1121f,-0.2366f,  0.9230f,0.1823f,-0.3388f,  0.9209f,0.1523f,-0.3588f,  0.8632f,0.2213f,-0.4538f,  
	0.8599f,0.1934f,-0.4725f,  0.7876f,0.2610f,-0.5581f,  0.7834f,0.2357f,-0.5751f,  0.6981f,0.3018f,-0.6493f,  0.6933f,0.2795f,-0.6642f,  0.5969f,0.3438f,-0.7250f,  0.5919f,0.3250f,-0.7376f,  0.4862f,0.3871f,-0.7835f,  0.4814f,0.3722f,-0.7935f,  0.3687f,0.4316f,-0.8233f,  0.3648f,0.4209f,-0.8305f,  0.2473f,0.4771f,-0.8433f,  0.2447f,0.4709f,-0.8476f,  0.1249f,0.5233f,-0.8430f,  0.1241f,0.5217f,-0.8441f,  
	0.0043f,0.5694f,-0.8220f,  0.0061f,0.5724f,-0.8199f,  -0.1114f,0.6147f,-0.7808f,  -0.1066f,0.6223f,-0.7755f,  -0.2195f,0.6583f,-0.7201f,  -0.2111f,0.6702f,-0.7115f,  -0.3174f,0.6988f,-0.6410f,  -0.3049f,0.7148f,-0.6294f,  -0.4029f,0.7351f,-0.5453f,  -0.3859f,0.7546f,-0.5307f,  -0.4742f,0.7655f,-0.4350f,  -0.4523f,0.7879f,-0.4178f,  -0.5298f,0.7884f,-0.3125f,  -0.5029f,0.8131f,-0.2931f,  -0.5689f,0.8023f,-0.1808f,  
	-0.5370f,0.8283f,-0.1596f,  -0.5911f,0.8055f,-0.0429f,  -0.5545f,0.8319f,-0.0203f,  -0.5969f,0.7963f,0.0981f,  -0.5560f,0.8223f,0.1214f,  -0.5871f,0.7735f,0.2385f,  -0.5427f,0.7980f,0.2620f,  -0.5635f,0.7360f,0.3751f,  -0.5164f,0.7581f,0.3982f,  -0.5282f,0.6830f,0.5044f,  -0.4796f,0.7020f,0.5264f,  -0.4838f,0.6144f,0.6233f,  -0.4351f,0.6296f,0.6437f,  -0.4334f,0.5304f,0.7286f,  -0.3859f,0.5414f,0.7469f,  
	-0.3802f,0.4319f,0.8179f,  -0.3355f,0.4387f,0.8337f,  -0.3274f,0.3205f,0.8889f,  -0.2869f,0.3231f,0.9018f,  -0.2782f,0.1984f,0.9398f,  -0.2432f,0.1973f,0.9497f,  -0.2355f,0.0683f,0.9695f,  -0.2070f,0.0640f,0.9763f,  -0.2015f,-0.0667f,0.9772f,  -0.1803f,-0.0733f,0.9809f,  -0.1779f,-0.2028f,0.9629f,  -0.1646f,-0.2109f,0.9635f,  -0.1659f,-0.3364f,0.9270f,  -0.1606f,-0.3450f,0.9248f,  -0.1656f,-0.4638f,0.8703f,  
	-0.1681f,-0.4719f,0.8655f,  -0.1767f,-0.5813f,0.7943f,  -0.1865f,-0.5878f,0.7872f,  -0.1980f,-0.6854f,0.7008f,  -0.2143f,-0.6896f,0.6917f,  0.0106f,-0.5793f,0.8151f,  0.0081f,-0.5757f,0.8176f,  0.1227f,-0.5193f,0.8458f,  0.1234f,-0.5203f,0.8450f,  0.2386f,-0.4593f,0.8556f,  0.2418f,-0.4650f,0.8517f,  0.3553f,-0.4003f,0.8447f,  0.3603f,-0.4105f,0.8377f,  0.4698f,-0.3431f,0.8134f,  0.4760f,-0.3575f,0.8035f,  
	0.5791f,-0.2880f,0.7627f,  0.5860f,-0.3065f,0.7501f,  0.6804f,-0.2355f,0.6939f,  0.6875f,-0.2575f,0.6790f,  0.7713f,-0.1855f,0.6089f,  0.7780f,-0.2106f,0.5919f,  0.8493f,-0.1380f,0.5096f,  0.8553f,-0.1658f,0.4910f,  0.9124f,-0.0929f,0.3985f,  0.9174f,-0.1227f,0.3785f,  0.9592f,-0.0497f,0.2783f,  0.9629f,-0.0810f,0.2573f,  0.9884f,-0.0081f,0.1516f,  0.9907f,-0.0404f,0.1301f,  0.9992f,0.0324f,0.0216f,  
	1.0000f,-0.0002f,-0.0001f,  0.9914f,0.0723f,-0.1088f,  0.9907f,0.0400f,-0.1303f,  0.9651f,0.1121f,-0.2366f,  0.9629f,0.0807f,-0.2575f,  0.9209f,0.1523f,-0.3588f,  0.9174f,0.1224f,-0.3787f,  0.8599f,0.1934f,-0.4725f,  0.8552f,0.1656f,-0.4911f,  0.7834f,0.2357f,-0.5751f,  0.7779f,0.2105f,-0.5921f,  0.6933f,0.2795f,-0.6642f,  0.6874f,0.2574f,-0.6791f,  0.5919f,0.3250f,-0.7376f,  0.5859f,0.3064f,-0.7502f,  
	0.4814f,0.3722f,-0.7935f,  0.4760f,0.3575f,-0.8035f,  0.3648f,0.4209f,-0.8305f,  0.3603f,0.4105f,-0.8377f,  0.2447f,0.4709f,-0.8476f,  0.2418f,0.4650f,-0.8517f,  0.1241f,0.5217f,-0.8441f,  0.1234f,0.5203f,-0.8450f,  0.0061f,0.5724f,-0.8199f,  0.0081f,0.5757f,-0.8176f,  -0.1066f,0.6223f,-0.7755f,  -0.1011f,0.6301f,-0.7699f,  -0.2111f,0.6702f,-0.7115f,  -0.2016f,0.6823f,-0.7027f,  -0.3049f,0.7148f,-0.6294f,  
	-0.2910f,0.7308f,-0.6174f,  -0.3859f,0.7546f,-0.5307f,  -0.3671f,0.7740f,-0.5159f,  -0.4523f,0.7879f,-0.4178f,  -0.4284f,0.8100f,-0.4005f,  -0.5029f,0.8131f,-0.2931f,  -0.4738f,0.8371f,-0.2736f,  -0.5370f,0.8283f,-0.1596f,  -0.5027f,0.8533f,-0.1383f,  -0.5545f,0.8319f,-0.0203f,  -0.5153f,0.8570f,0.0021f,  -0.5560f,0.8223f,0.1214f,  -0.5126f,0.8464f,0.1444f,  -0.5427f,0.7980f,0.2620f,  -0.4959f,0.8203f,0.2851f,  
	-0.5164f,0.7581f,0.3982f,  -0.4673f,0.7776f,0.4206f,  -0.4796f,0.7020f,0.5264f,  -0.4294f,0.7181f,0.5477f,  -0.4351f,0.6296f,0.6437f,  -0.3851f,0.6417f,0.6632f,  -0.3859f,0.5414f,0.7469f,  -0.3378f,0.5493f,0.7643f,  -0.3355f,0.4387f,0.8337f,  -0.2906f,0.4423f,0.8484f,  -0.2869f,0.3231f,0.9018f,  -0.2467f,0.3228f,0.9137f,  -0.2432f,0.1973f,0.9497f,  -0.2090f,0.1935f,0.9586f,  -0.2070f,0.0640f,0.9763f,  
	-0.1797f,0.0574f,0.9821f,  -0.1803f,-0.0733f,0.9809f,  -0.1606f,-0.0818f,0.9836f,  -0.1646f,-0.2109f,0.9635f,  -0.1528f,-0.2203f,0.9634f,  -0.1606f,-0.3450f,0.9248f,  -0.1567f,-0.3544f,0.9219f,  -0.1681f,-0.4719f,0.8655f,  -0.1718f,-0.4802f,0.8601f,  -0.1865f,-0.5878f,0.7872f,  -0.1972f,-0.5943f,0.7797f,  -0.2143f,-0.6896f,0.6917f,  -0.2312f,-0.6936f,0.6823f,  0.0081f,-0.5757f,0.8176f,  0.0061f,-0.5724f,0.8199f,  
	0.1234f,-0.5203f,0.8450f,  0.1241f,-0.5217f,0.8441f,  0.2418f,-0.4650f,0.8517f,  0.2447f,-0.4709f,0.8476f,  0.3603f,-0.4105f,0.8377f,  0.3648f,-0.4209f,0.8305f,  0.4760f,-0.3575f,0.8035f,  0.4815f,-0.3722f,0.7935f,  0.5860f,-0.3065f,0.7501f,  0.5919f,-0.3251f,0.7376f,  0.6875f,-0.2575f,0.6790f,  0.6934f,-0.2796f,0.6641f,  0.7780f,-0.2106f,0.5919f,  0.7834f,-0.2359f,0.5750f,  0.8553f,-0.1658f,0.4910f,  
	0.8599f,-0.1936f,0.4723f,  0.9174f,-0.1227f,0.3785f,  0.9209f,-0.1526f,0.3586f,  0.9629f,-0.0810f,0.2573f,  0.9651f,-0.1124f,0.2364f,  0.9907f,-0.0404f,0.1301f,  0.9914f,-0.0726f,0.1086f,  1.0000f,-0.0002f,-0.0001f,  0.9992f,-0.0327f,-0.0218f,  0.9907f,0.0400f,-0.1303f,  0.9884f,0.0078f,-0.1519f,  0.9629f,0.0807f,-0.2575f,  0.9592f,0.0494f,-0.2785f,  0.9174f,0.1224f,-0.3787f,  0.9124f,0.0926f,-0.3987f,  
	0.8552f,0.1656f,-0.4911f,  0.8492f,0.1378f,-0.5098f,  0.7779f,0.2105f,-0.5921f,  0.7712f,0.1853f,-0.6090f,  0.6874f,0.2574f,-0.6791f,  0.6804f,0.2353f,-0.6940f,  0.5859f,0.3064f,-0.7502f,  0.5790f,0.2879f,-0.7628f,  0.4760f,0.3575f,-0.8035f,  0.4697f,0.3430f,-0.8134f,  0.3603f,0.4105f,-0.8377f,  0.3553f,0.4003f,-0.8447f,  0.2418f,0.4650f,-0.8517f,  0.2386f,0.4593f,-0.8556f,  0.1234f,0.5203f,-0.8450f,  
	0.1227f,0.5193f,-0.8458f,  0.0081f,0.5757f,-0.8176f,  0.0106f,0.5793f,-0.8151f,  -0.1011f,0.6301f,-0.7699f,  -0.0949f,0.6382f,-0.7640f,  -0.2016f,0.6823f,-0.7027f,  -0.1911f,0.6945f,-0.6936f,  -0.2910f,0.7308f,-0.6174f,  -0.2756f,0.7468f,-0.6053f,  -0.3671f,0.7740f,-0.5159f,  -0.3466f,0.7931f,-0.5010f,  -0.4284f,0.8100f,-0.4005f,  -0.4024f,0.8315f,-0.3830f,  -0.4738f,0.8371f,-0.2736f,  -0.4423f,0.8602f,-0.2540f,  
	-0.5027f,0.8533f,-0.1383f,  -0.4659f,0.8771f,-0.1171f,  -0.5153f,0.8570f,0.0021f,  -0.4737f,0.8803f,0.0244f,  -0.5126f,0.8464f,0.1444f,  -0.4668f,0.8684f,0.1672f,  -0.4959f,0.8203f,0.2851f,  -0.4469f,0.8400f,0.3077f,  -0.4673f,0.7776f,0.4206f,  -0.4163f,0.7943f,0.4424f,  -0.4294f,0.7181f,0.5477f,  -0.3777f,0.7311f,0.5682f,  -0.3851f,0.6417f,0.6632f,  -0.3344f,0.6506f,0.6818f,  -0.3378f,0.5493f,0.7643f,  
	-0.2894f,0.5540f,0.7806f,  -0.2906f,0.4423f,0.8484f,  -0.2461f,0.4429f,0.8621f,  -0.2467f,0.3228f,0.9137f,  -0.2074f,0.3196f,0.9246f,  -0.2090f,0.1935f,0.9586f,  -0.1759f,0.1871f,0.9665f,  -0.1797f,0.0574f,0.9821f,  -0.1538f,0.0488f,0.9869f,  -0.1606f,-0.0818f,0.9836f,  -0.1424f,-0.0919f,0.9855f,  -0.1528f,-0.2203f,0.9634f,  -0.1426f,-0.2309f,0.9625f,  -0.1567f,-0.3544f,0.9219f,  -0.1543f,-0.3644f,0.9184f,  
	-0.1718f,-0.4802f,0.8601f,  -0.1768f,-0.4888f,0.8543f,  -0.1972f,-0.5943f,0.7797f,  -0.2089f,-0.6007f,0.7717f,  -0.2312f,-0.6936f,0.6823f,  -0.2485f,-0.6970f,0.6726f,  0.0061f,-0.5724f,0.8199f,  0.0043f,-0.5694f,0.8220f,  0.1241f,-0.5217f,0.8441f,  0.1249f,-0.5233f,0.8430f,  0.2447f,-0.4709f,0.8476f,  0.2473f,-0.4771f,0.8433f,  0.3648f,-0.4209f,0.8305f,  0.3687f,-0.4316f,0.8233f,  0.4815f,-0.3722f,0.7935f,  
	0.4862f,-0.3871f,0.7834f,  0.5919f,-0.3251f,0.7376f,  0.5969f,-0.3439f,0.7249f,  0.6934f,-0.2796f,0.6641f,  0.6982f,-0.3019f,0.6492f,  0.7834f,-0.2359f,0.5750f,  0.7876f,-0.2612f,0.5580f,  0.8599f,-0.1936f,0.4723f,  0.8632f,-0.2215f,0.4537f,  0.9209f,-0.1526f,0.3586f,  0.9231f,-0.1826f,0.3386f,  0.9651f,-0.1124f,0.2364f,  0.9659f,-0.1438f,0.2154f,  0.9914f,-0.0726f,0.1086f,  0.9907f,-0.1049f,0.0870f,  
	0.9992f,-0.0327f,-0.0218f,  0.9969f,-0.0653f,-0.0436f,  0.9884f,0.0078f,-0.1519f,  0.9845f,-0.0245f,-0.1734f,  0.9592f,0.0494f,-0.2785f,  0.9539f,0.0180f,-0.2995f,  0.9124f,0.0926f,-0.3987f,  0.9059f,0.0628f,-0.4187f,  0.8492f,0.1378f,-0.5098f,  0.8418f,0.1101f,-0.5284f,  0.7712f,0.1853f,-0.6090f,  0.7632f,0.1602f,-0.6260f,  0.6804f,0.2353f,-0.6940f,  0.6722f,0.2134f,-0.7089f,  0.5790f,0.2879f,-0.7628f,  
	0.5712f,0.2697f,-0.7753f,  0.4697f,0.3430f,-0.8134f,  0.4627f,0.3288f,-0.8233f,  0.3553f,0.4003f,-0.8447f,  0.3497f,0.3904f,-0.8516f,  0.2386f,0.4593f,-0.8556f,  0.2352f,0.4539f,-0.8595f,  0.1227f,0.5193f,-0.8458f,  0.1221f,0.5185f,-0.8463f,  0.0106f,0.5793f,-0.8151f,  0.0134f,0.5831f,-0.8123f,  -0.0949f,0.6382f,-0.7640f,  -0.0879f,0.6464f,-0.7579f,  -0.1911f,0.6945f,-0.6936f,  -0.1794f,0.7068f,-0.6843f,  
	-0.2756f,0.7468f,-0.6053f,  -0.2588f,0.7626f,-0.5929f,  -0.3466f,0.7931f,-0.5010f,  -0.3241f,0.8118f,-0.4858f,  -0.4024f,0.8315f,-0.3830f,  -0.3743f,0.8523f,-0.3653f,  -0.4423f,0.8602f,-0.2540f,  -0.4085f,0.8822f,-0.2344f,  -0.4659f,0.8771f,-0.1171f,  -0.4268f,0.8992f,-0.0961f,  -0.4737f,0.8803f,0.0244f,  -0.4298f,0.9017f,0.0464f,  -0.4668f,0.8684f,0.1672f,  -0.4189f,0.8881f,0.1894f,  -0.4469f,0.8400f,0.3077f,  
	-0.3961f,0.8570f,0.3296f,  -0.4163f,0.7943f,0.4424f,  -0.3639f,0.8080f,0.4634f,  -0.3777f,0.7311f,0.5682f,  -0.3252f,0.7408f,0.5877f,  -0.3344f,0.6506f,0.6818f,  -0.2833f,0.6562f,0.6994f,  -0.2894f,0.5540f,0.7806f,  -0.2412f,0.5554f,0.7958f,  -0.2461f,0.4429f,0.8621f,  -0.2023f,0.4403f,0.8747f,  -0.2074f,0.3196f,0.9246f,  -0.1692f,0.3136f,0.9343f,  -0.1759f,0.1871f,0.9665f,  -0.1444f,0.1784f,0.9733f,  
	-0.1538f,0.0488f,0.9869f,  -0.1296f,0.0381f,0.9908f,  -0.1424f,-0.0919f,0.9855f,  -0.1261f,-0.1035f,0.9866f,  -0.1426f,-0.2309f,0.9625f,  -0.1341f,-0.2424f,0.9609f,  -0.1543f,-0.3644f,0.9184f,  -0.1534f,-0.3750f,0.9143f,  -0.1768f,-0.4888f,0.8543f,  -0.1830f,-0.4976f,0.8479f,  -0.2089f,-0.6007f,0.7717f,  -0.2213f,-0.6068f,0.7634f,  -0.2485f,-0.6970f,0.6726f,  -0.2664f,-0.7000f,0.6626f,  0.0043f,-0.5694f,0.8220f,  
	0.0029f,-0.5667f,0.8239f,  0.1249f,-0.5233f,0.8430f,  0.1256f,-0.5252f,0.8417f,  0.2473f,-0.4771f,0.8433f,  0.2497f,-0.4836f,0.8389f,  0.3687f,-0.4316f,0.8233f,  0.3722f,-0.4425f,0.8159f,  0.4862f,-0.3871f,0.7834f,  0.4902f,-0.4022f,0.7733f,  0.5969f,-0.3439f,0.7249f,  0.6010f,-0.3628f,0.7122f,  0.6982f,-0.3019f,0.6492f,  0.7019f,-0.3243f,0.6342f,  0.7876f,-0.2612f,0.5580f,  0.7906f,-0.2867f,0.5410f,  
	0.8632f,-0.2215f,0.4537f,  0.8652f,-0.2496f,0.4350f,  0.9231f,-0.1826f,0.3386f,  0.9238f,-0.2126f,0.3186f,  0.9659f,-0.1438f,0.2154f,  0.9651f,-0.1753f,0.1944f,  0.9907f,-0.1049f,0.0870f,  0.9884f,-0.1373f,0.0654f,  0.9969f,-0.0653f,-0.0436f,  0.9930f,-0.0980f,-0.0654f,  0.9845f,-0.0245f,-0.1734f,  0.9791f,-0.0568f,-0.1951f,  0.9539f,0.0180f,-0.2995f,  0.9471f,-0.0134f,-0.3205f,  0.9059f,0.0628f,-0.4187f,  
	0.8980f,0.0329f,-0.4388f,  0.8418f,0.1101f,-0.5284f,  0.8330f,0.0824f,-0.5472f,  0.7632f,0.1602f,-0.6260f,  0.7539f,0.1352f,-0.6430f,  0.6722f,0.2134f,-0.7089f,  0.6628f,0.1917f,-0.7238f,  0.5712f,0.2697f,-0.7753f,  0.5623f,0.2516f,-0.7878f,  0.4627f,0.3288f,-0.8233f,  0.4549f,0.3148f,-0.8331f,  0.3497f,0.3904f,-0.8516f,  0.3437f,0.3807f,-0.8585f,  0.2352f,0.4539f,-0.8595f,  0.2315f,0.4488f,-0.8631f,  
	0.1221f,0.5185f,-0.8463f,  0.1216f,0.5180f,-0.8467f,  0.0134f,0.5831f,-0.8123f,  0.0167f,0.5872f,-0.8093f,  -0.0879f,0.6464f,-0.7579f,  -0.0801f,0.6547f,-0.7516f,  -0.1794f,0.7068f,-0.6843f,  -0.1666f,0.7190f,-0.6747f,  -0.2588f,0.7626f,-0.5929f,  -0.2404f,0.7781f,-0.5803f,  -0.3241f,0.8118f,-0.4858f,  -0.2999f,0.8299f,-0.4705f,  -0.3743f,0.8523f,-0.3653f,  -0.3441f,0.8722f,-0.3477f,  -0.4085f,0.8822f,-0.2344f,  
	-0.3725f,0.9028f,-0.2149f,  -0.4268f,0.8992f,-0.0961f,  -0.3854f,0.9197f,-0.0752f,  -0.4298f,0.9017f,0.0464f,  -0.3837f,0.9209f,0.0680f,  -0.4189f,0.8881f,0.1894f,  -0.3691f,0.9051f,0.2111f,  -0.3961f,0.8570f,0.3296f,  -0.3438f,0.8710f,0.3508f,  -0.3639f,0.8080f,0.4634f,  -0.3105f,0.8184f,0.4836f,  -0.3252f,0.7408f,0.5877f,  -0.2723f,0.7472f,0.6063f,  -0.2833f,0.6562f,0.6994f,  -0.2322f,0.6584f,0.7159f,  
	-0.2412f,0.5554f,0.7958f,  -0.1937f,0.5535f,0.8100f,  -0.2023f,0.4403f,0.8747f,  -0.1596f,0.4348f,0.8863f,  -0.1692f,0.3136f,0.9343f,  -0.1325f,0.3050f,0.9431f,  -0.1444f,0.1784f,0.9733f,  -0.1146f,0.1675f,0.9792f,  -0.1296f,0.0381f,0.9908f,  -0.1074f,0.0257f,0.9939f,  -0.1261f,-0.1035f,0.9866f,  -0.1116f,-0.1163f,0.9869f,  -0.1341f,-0.2424f,0.9609f,  -0.1273f,-0.2548f,0.9586f,  -0.1534f,-0.3750f,0.9143f,  
	-0.1540f,-0.3859f,0.9096f,  -0.1830f,-0.4976f,0.8479f,  -0.1903f,-0.5062f,0.8411f,  -0.2213f,-0.6068f,0.7634f,  -0.2346f,-0.6127f,0.7547f,  -0.2664f,-0.7000f,0.6626f,  -0.2846f,-0.7025f,0.6523f,  0.0029f,-0.5667f,0.8239f,  0.0018f,-0.5642f,0.8256f,  0.1256f,-0.5252f,0.8417f,  0.1264f,-0.5273f,0.8402f,  0.2497f,-0.4836f,0.8389f,  0.2518f,-0.4903f,0.8344f,  0.3722f,-0.4425f,0.8159f,  0.3751f,-0.4536f,0.8084f,  
	0.4902f,-0.4022f,0.7733f,  0.4935f,-0.4174f,0.7630f,  0.6010f,-0.3628f,0.7122f,  0.6041f,-0.3819f,0.6994f,  0.7019f,-0.3243f,0.6342f,  0.7046f,-0.3469f,0.6191f,  0.7906f,-0.2867f,0.5410f,  0.7924f,-0.3123f,0.5239f,  0.8652f,-0.2496f,0.4350f,  0.8658f,-0.2777f,0.4162f,  0.9238f,-0.2126f,0.3186f,  0.9230f,-0.2427f,0.2984f,  0.9651f,-0.1753f,0.1944f,  0.9629f,-0.2069f,0.1733f,  0.9884f,-0.1373f,0.0654f,  
	0.9845f,-0.1698f,0.0437f,  0.9930f,-0.0980f,-0.0654f,  0.9876f,-0.1307f,-0.0873f,  0.9791f,-0.0568f,-0.1951f,  0.9721f,-0.0892f,-0.2168f,  0.9471f,-0.0134f,-0.3205f,  0.9388f,-0.0448f,-0.3417f,  0.8980f,0.0329f,-0.4388f,  0.8884f,0.0031f,-0.4590f,  0.8330f,0.0824f,-0.5472f,  0.8226f,0.0548f,-0.5660f,  0.7539f,0.1352f,-0.6430f,  0.7431f,0.1104f,-0.6600f,  0.6628f,0.1917f,-0.7238f,  0.6522f,0.1701f,-0.7387f,  
	0.5623f,0.2516f,-0.7878f,  0.5523f,0.2338f,-0.8002f,  0.4549f,0.3148f,-0.8331f,  0.4462f,0.3011f,-0.8427f,  0.3437f,0.3807f,-0.8585f,  0.3370f,0.3714f,-0.8651f,  0.2315f,0.4488f,-0.8631f,  0.2276f,0.4441f,-0.8666f,  0.1216f,0.5180f,-0.8467f,  0.1211f,0.5179f,-0.8468f,  0.0167f,0.5872f,-0.8093f,  0.0205f,0.5915f,-0.8061f,  -0.0801f,0.6547f,-0.7516f,  -0.0715f,0.6632f,-0.7450f,  -0.1666f,0.7190f,-0.6747f,  
	-0.1524f,0.7312f,-0.6649f,  -0.2404f,0.7781f,-0.5803f,  -0.2204f,0.7933f,-0.5675f,  -0.2999f,0.8299f,-0.4705f,  -0.2738f,0.8473f,-0.4550f,  -0.3441f,0.8722f,-0.3477f,  -0.3119f,0.8909f,-0.3301f,  -0.3725f,0.9028f,-0.2149f,  -0.3345f,0.9219f,-0.1955f,  -0.3854f,0.9197f,-0.0752f,  -0.3420f,0.9381f,-0.0547f,  -0.3837f,0.9209f,0.0680f,  -0.3359f,0.9377f,0.0891f,  -0.3691f,0.9051f,0.2111f,  -0.3179f,0.9193f,0.2322f,  
	-0.3438f,0.8710f,0.3508f,  -0.2905f,0.8819f,0.3712f,  -0.3105f,0.8184f,0.4836f,  -0.2566f,0.8254f,0.5028f,  -0.2723f,0.7472f,0.6063f,  -0.2193f,0.7502f,0.6238f,  -0.2322f,0.6584f,0.7159f,  -0.1818f,0.6572f,0.7314f,  -0.1937f,0.5535f,0.8100f,  -0.1472f,0.5485f,0.8231f,  -0.1596f,0.4348f,0.8863f,  -0.1184f,0.4264f,0.8967f,  -0.1325f,0.3050f,0.9431f,  -0.0977f,0.2939f,0.9508f,  -0.1146f,0.1675f,0.9792f,  
	-0.0869f,0.1545f,0.9842f,  -0.1074f,0.0257f,0.9939f,  -0.0873f,0.0118f,0.9961f,  -0.1116f,-0.1163f,0.9869f,  -0.0991f,-0.1302f,0.9865f,  -0.1273f,-0.2548f,0.9586f,  -0.1224f,-0.2677f,0.9557f,  -0.1540f,-0.3859f,0.9096f,  -0.1561f,-0.3970f,0.9044f,  -0.1903f,-0.5062f,0.8411f,  -0.1989f,-0.5149f,0.8339f,  -0.2346f,-0.6127f,0.7547f,  -0.2486f,-0.6182f,0.7457f,  -0.2846f,-0.7025f,0.6523f,  -0.3030f,-0.7044f,0.6419f,  
	0.0018f,-0.5642f,0.8256f,  0.0009f,-0.5620f,0.8271f,  0.1264f,-0.5273f,0.8402f,  0.1271f,-0.5297f,0.8386f,  0.2518f,-0.4903f,0.8344f,  0.2536f,-0.4973f,0.8297f,  0.3751f,-0.4536f,0.8084f,  0.3775f,-0.4650f,0.8008f,  0.4935f,-0.4174f,0.7630f,  0.4960f,-0.4329f,0.7527f,  0.6041f,-0.3819f,0.6994f,  0.6064f,-0.4012f,0.6865f,  0.7046f,-0.3469f,0.6191f,  0.7062f,-0.3696f,0.6039f,  0.7924f,-0.3123f,0.5239f,  
	0.7931f,-0.3380f,0.5068f,  0.8658f,-0.2777f,0.4162f,  0.8652f,-0.3060f,0.3974f,  0.9230f,-0.2427f,0.2984f,  0.9209f,-0.2730f,0.2782f,  0.9629f,-0.2069f,0.1733f,  0.9591f,-0.2387f,0.1521f,  0.9845f,-0.1698f,0.0437f,  0.9791f,-0.2024f,0.0219f,  0.9876f,-0.1307f,-0.0873f,  0.9804f,-0.1636f,-0.1094f,  0.9721f,-0.0892f,-0.2168f,  0.9634f,-0.1217f,-0.2387f,  0.9388f,-0.0448f,-0.3417f,  0.9287f,-0.0762f,-0.3629f,  
	0.8884f,0.0031f,-0.4590f,  0.8773f,-0.0267f,-0.4793f,  0.8226f,0.0548f,-0.5660f,  0.8107f,0.0272f,-0.5848f,  0.7431f,0.1104f,-0.6600f,  0.7309f,0.0857f,-0.6771f,  0.6522f,0.1701f,-0.7387f,  0.6402f,0.1487f,-0.7537f,  0.5523f,0.2338f,-0.8002f,  0.5412f,0.2162f,-0.8126f,  0.4462f,0.3011f,-0.8427f,  0.4367f,0.2877f,-0.8524f,  0.3370f,0.3714f,-0.8651f,  0.3298f,0.3625f,-0.8717f,  0.2276f,0.4441f,-0.8666f,  
	0.2235f,0.4397f,-0.8699f,  0.1211f,0.5179f,-0.8468f,  0.1208f,0.5181f,-0.8468f,  0.0205f,0.5915f,-0.8061f,  0.0248f,0.5960f,-0.8026f,  -0.0715f,0.6632f,-0.7450f,  -0.0619f,0.6717f,-0.7382f,  -0.1524f,0.7312f,-0.6649f,  -0.1371f,0.7431f,-0.6549f,  -0.2204f,0.7933f,-0.5675f,  -0.1988f,0.8080f,-0.5546f,  -0.2738f,0.8473f,-0.4550f,  -0.2459f,0.8639f,-0.4396f,  -0.3119f,0.8909f,-0.3301f,  -0.2778f,0.9084f,-0.3125f,  
	-0.3345f,0.9219f,-0.1955f,  -0.2945f,0.9392f,-0.1764f,  -0.3420f,0.9381f,-0.0547f,  -0.2969f,0.9543f,-0.0345f,  -0.3359f,0.9377f,0.0891f,  -0.2865f,0.9518f,0.1096f,  -0.3179f,0.9193f,0.2322f,  -0.2655f,0.9304f,0.2525f,  -0.2905f,0.8819f,0.3712f,  -0.2366f,0.8896f,0.3908f,  -0.2566f,0.8254f,0.5028f,  -0.2026f,0.8291f,0.5211f,  -0.2193f,0.7502f,0.6238f,  -0.1668f,0.7497f,0.6404f,  -0.1818f,0.6572f,0.7314f,  
	-0.1324f,0.6528f,0.7458f,  -0.1472f,0.5485f,0.8231f,  -0.1022f,0.5405f,0.8351f,  -0.1184f,0.4264f,0.8967f,  -0.0791f,0.4154f,0.9062f,  -0.0977f,0.2939f,0.9508f,  -0.0650f,0.2806f,0.9576f,  -0.0869f,0.1545f,0.9842f,  -0.0614f,0.1397f,0.9883f,  -0.0873f,0.0118f,0.9961f,  -0.0693f,-0.0034f,0.9976f,  -0.0991f,-0.1302f,0.9865f,  -0.0888f,-0.1449f,0.9855f,  -0.1224f,-0.2677f,0.9557f,  -0.1192f,-0.2810f,0.9523f,  
	-0.1561f,-0.3970f,0.9044f,  -0.1597f,-0.4083f,0.8988f,  -0.1989f,-0.5149f,0.8339f,  -0.2084f,-0.5233f,0.8263f,  -0.2486f,-0.6182f,0.7457f,  -0.2632f,-0.6232f,0.7364f,  -0.3030f,-0.7044f,0.6419f,  -0.3217f,-0.7057f,0.6313f,  0.0009f,-0.5620f,0.8271f,  0.0003f,-0.5601f,0.8284f,  0.1271f,-0.5297f,0.8386f,  0.1278f,-0.5324f,0.8368f,  0.2536f,-0.4973f,0.8297f,  0.2552f,-0.5045f,0.8248f,  0.3775f,-0.4650f,0.8008f,  
	0.3795f,-0.4765f,0.7931f,  0.4960f,-0.4329f,0.7527f,  0.4979f,-0.4486f,0.7422f,  0.6064f,-0.4012f,0.6865f,  0.6078f,-0.4207f,0.6735f,  0.7062f,-0.3696f,0.6039f,  0.7067f,-0.3926f,0.5886f,  0.7931f,-0.3380f,0.5068f,  0.7924f,-0.3639f,0.4895f,  0.8652f,-0.3060f,0.3974f,  0.8631f,-0.3344f,0.3784f,  0.9209f,-0.2730f,0.2782f,  0.9173f,-0.3035f,0.2579f,  0.9591f,-0.2387f,0.1521f,  0.9538f,-0.2706f,0.1308f,  
	0.9791f,-0.2024f,0.0219f,  0.9720f,-0.2351f,-0.0001f,  0.9804f,-0.1636f,-0.1094f,  0.9716f,-0.1965f,-0.1316f,  0.9634f,-0.1217f,-0.2387f,  0.9530f,-0.1542f,-0.2607f,  0.9287f,-0.0762f,-0.3629f,  0.9169f,-0.1077f,-0.3843f,  0.8773f,-0.0267f,-0.4793f,  0.8644f,-0.0564f,-0.4996f,  0.8107f,0.0272f,-0.5848f,  0.7971f,-0.0002f,-0.6038f,  0.7309f,0.0857f,-0.6771f,  0.7172f,0.0611f,-0.6942f,  0.6402f,0.1487f,-0.7537f,  
	0.6269f,0.1276f,-0.7686f,  0.5412f,0.2162f,-0.8126f,  0.5290f,0.1990f,-0.8250f,  0.4367f,0.2877f,-0.8524f,  0.4263f,0.2747f,-0.8618f,  0.3298f,0.3625f,-0.8717f,  0.3220f,0.3540f,-0.8781f,  0.2235f,0.4397f,-0.8699f,  0.2191f,0.4358f,-0.8730f,  0.1208f,0.5181f,-0.8468f,  0.1207f,0.5186f,-0.8464f,  0.0248f,0.5960f,-0.8026f,  0.0297f,0.6008f,-0.7989f,  -0.0619f,0.6717f,-0.7382f,  -0.0515f,0.6803f,-0.7312f,  
	-0.1371f,0.7431f,-0.6549f,  -0.1205f,0.7548f,-0.6448f,  -0.1988f,0.8080f,-0.5546f,  -0.1758f,0.8220f,-0.5416f,  -0.2459f,0.8639f,-0.4396f,  -0.2164f,0.8794f,-0.4241f,  -0.2778f,0.9084f,-0.3125f,  -0.2419f,0.9243f,-0.2951f,  -0.2945f,0.9392f,-0.1764f,  -0.2528f,0.9546f,-0.1576f,  -0.2969f,0.9543f,-0.0345f,  -0.2503f,0.9681f,-0.0149f,  -0.2865f,0.9518f,0.1096f,  -0.2360f,0.9631f,0.1295f,  -0.2655f,0.9304f,0.2525f,  
	-0.2125f,0.9385f,0.2721f,  -0.2366f,0.8896f,0.3908f,  -0.1824f,0.8939f,0.4094f,  -0.2026f,0.8291f,0.5211f,  -0.1489f,0.8294f,0.5384f,  -0.1668f,0.7497f,0.6404f,  -0.1152f,0.7460f,0.6559f,  -0.1324f,0.6528f,0.7458f,  -0.0843f,0.6454f,0.7592f,  -0.1022f,0.5405f,0.8351f,  -0.0591f,0.5297f,0.8461f,  -0.0791f,0.4154f,0.9062f,  -0.0419f,0.4019f,0.9147f,  -0.0650f,0.2806f,0.9576f,  -0.0345f,0.2653f,0.9636f,  
	-0.0614f,0.1397f,0.9883f,  -0.0382f,0.1235f,0.9916f,  -0.0693f,-0.0034f,0.9976f,  -0.0536f,-0.0197f,0.9984f,  -0.0888f,-0.1449f,0.9855f,  -0.0803f,-0.1602f,0.9838f,  -0.1192f,-0.2810f,0.9523f,  -0.1179f,-0.2947f,0.9483f,  -0.1597f,-0.4083f,0.8988f,  -0.1647f,-0.4194f,0.8927f,  -0.2084f,-0.5233f,0.8263f,  -0.2190f,-0.5313f,0.8184f,  -0.2632f,-0.6232f,0.7364f,  -0.2784f,-0.6277f,0.7270f,  -0.3217f,-0.7057f,0.6313f,  
	-0.3405f,-0.7065f,0.6205f,  0.0003f,-0.5601f,0.8284f,  -0.0002f,-0.5585f,0.8295f,  0.1278f,-0.5324f,0.8368f,  0.1284f,-0.5354f,0.8348f,  0.2552f,-0.5045f,0.8248f,  0.2564f,-0.5120f,0.8198f,  0.3795f,-0.4765f,0.7931f,  0.3808f,-0.4883f,0.7852f,  0.4979f,-0.4486f,0.7422f,  0.4990f,-0.4645f,0.7316f,  0.6078f,-0.4207f,0.6735f,  0.6082f,-0.4404f,0.6604f,  0.7067f,-0.3926f,0.5886f,  0.7062f,-0.4157f,0.5732f,  
	0.7924f,-0.3639f,0.4895f,  0.7906f,-0.3901f,0.4721f,  0.8631f,-0.3344f,0.3784f,  0.8597f,-0.3630f,0.3593f,  0.9173f,-0.3035f,0.2579f,  0.9122f,-0.3341f,0.2374f,  0.9538f,-0.2706f,0.1308f,  0.9468f,-0.3026f,0.1093f,  0.9720f,-0.2351f,-0.0001f,  0.9632f,-0.2680f,-0.0222f,  0.9716f,-0.1965f,-0.1316f,  0.9610f,-0.2296f,-0.1539f,  0.9530f,-0.1542f,-0.2607f,  0.9408f,-0.1868f,-0.2828f,  0.9169f,-0.1077f,-0.3843f,  
	0.9033f,-0.1391f,-0.4059f,  0.8644f,-0.0564f,-0.4996f,  0.8497f,-0.0861f,-0.5202f,  0.7971f,-0.0002f,-0.6038f,  0.7819f,-0.0275f,-0.6228f,  0.7172f,0.0611f,-0.6942f,  0.7019f,0.0369f,-0.7114f,  0.6269f,0.1276f,-0.7686f,  0.6122f,0.1069f,-0.7835f,  0.5290f,0.1990f,-0.8250f,  0.5156f,0.1822f,-0.8373f,  0.4263f,0.2747f,-0.8618f,  0.4150f,0.2622f,-0.8712f,  0.3220f,0.3540f,-0.8781f,  0.3136f,0.3460f,-0.8843f,  
	0.2191f,0.4358f,-0.8730f,  0.2145f,0.4323f,-0.8758f,  0.1207f,0.5186f,-0.8464f,  0.1207f,0.5195f,-0.8459f,  0.0297f,0.6008f,-0.7989f,  0.0351f,0.6057f,-0.7949f,  -0.0515f,0.6803f,-0.7312f,  -0.0400f,0.6887f,-0.7239f,  -0.1205f,0.7548f,-0.6448f,  -0.1026f,0.7661f,-0.6344f,  -0.1758f,0.8220f,-0.5416f,  -0.1512f,0.8353f,-0.5286f,  -0.2164f,0.8794f,-0.4241f,  -0.1852f,0.8937f,-0.4088f,  -0.2419f,0.9243f,-0.2951f,  
	-0.2044f,0.9386f,-0.2779f,  -0.2528f,0.9546f,-0.1576f,  -0.2097f,0.9678f,-0.1391f,  -0.2503f,0.9681f,-0.0149f,  -0.2025f,0.9793f,0.0043f,  -0.2360f,0.9631f,0.1295f,  -0.1847f,0.9715f,0.1487f,  -0.2125f,0.9385f,0.2721f,  -0.1591f,0.9435f,0.2908f,  -0.1824f,0.8939f,0.4094f,  -0.1285f,0.8950f,0.4272f,  -0.1489f,0.8294f,0.5384f,  -0.0961f,0.8265f,0.5547f,  -0.1152f,0.7460f,0.6559f,  -0.0650f,0.7392f,0.6704f,  
	-0.0843f,0.6454f,0.7592f,  -0.0381f,0.6350f,0.7716f,  -0.0591f,0.5297f,0.8461f,  -0.0181f,0.5163f,0.8562f,  -0.0419f,0.4019f,0.9147f,  -0.0070f,0.3862f,0.9224f,  -0.0345f,0.2653f,0.9636f,  -0.0065f,0.2482f,0.9687f,  -0.0382f,0.1235f,0.9916f,  -0.0175f,0.1059f,0.9942f,  -0.0536f,-0.0197f,0.9984f,  -0.0401f,-0.0367f,0.9985f,  -0.0803f,-0.1602f,0.9838f,  -0.0740f,-0.1761f,0.9816f,  -0.1179f,-0.2947f,0.9483f,  
	-0.1182f,-0.3084f,0.9439f,  -0.1647f,-0.4194f,0.8927f,  -0.1710f,-0.4303f,0.8863f,  -0.2190f,-0.5313f,0.8184f,  -0.2304f,-0.5389f,0.8102f,  -0.2784f,-0.6277f,0.7270f,  -0.2941f,-0.6317f,0.7173f,  -0.3405f,-0.7065f,0.6205f,  -0.3592f,-0.7066f,0.6096f,  -0.0002f,-0.5585f,0.8295f,  -0.0005f,-0.5572f,0.8304f,  0.1284f,-0.5354f,0.8348f,  0.1290f,-0.5387f,0.8326f,  0.2564f,-0.5120f,0.8198f,  0.2573f,-0.5197f,0.8147f,  
	0.3808f,-0.4883f,0.7852f,  0.3817f,-0.5004f,0.7771f,  0.4990f,-0.4645f,0.7316f,  0.4994f,-0.4806f,0.7208f,  0.6082f,-0.4404f,0.6604f,  0.6078f,-0.4603f,0.6471f,  0.7062f,-0.4157f,0.5732f,  0.7045f,-0.4390f,0.5576f,  0.7906f,-0.3901f,0.4721f,  0.7874f,-0.4164f,0.4545f,  0.8597f,-0.3630f,0.3593f,  0.8549f,-0.3919f,0.3399f,  0.9122f,-0.3341f,0.2374f,  0.9055f,-0.3649f,0.2167f,  0.9468f,-0.3026f,0.1093f,  
	0.9382f,-0.3349f,0.0876f,  0.9632f,-0.2680f,-0.0222f,  0.9526f,-0.3010f,-0.0445f,  0.9610f,-0.2296f,-0.1539f,  0.9486f,-0.2627f,-0.1765f,  0.9408f,-0.1868f,-0.2828f,  0.9267f,-0.2194f,-0.3052f,  0.9033f,-0.1391f,-0.4059f,  0.8877f,-0.1706f,-0.4276f,  0.8497f,-0.0861f,-0.5202f,  0.8332f,-0.1157f,-0.5408f,  0.7819f,-0.0275f,-0.6228f,  0.7648f,-0.0546f,-0.6419f,  0.7019f,0.0369f,-0.7114f,  0.6849f,0.0129f,-0.7285f,  
	0.6122f,0.1069f,-0.7835f,  0.5960f,0.0866f,-0.7983f,  0.5156f,0.1822f,-0.8373f,  0.5009f,0.1660f,-0.8494f,  0.4150f,0.2622f,-0.8712f,  0.4028f,0.2503f,-0.8804f,  0.3136f,0.3460f,-0.8843f,  0.3047f,0.3386f,-0.8902f,  0.2145f,0.4323f,-0.8758f,  0.2098f,0.4294f,-0.8784f,  0.1207f,0.5195f,-0.8459f,  0.1210f,0.5208f,-0.8450f,  0.0351f,0.6057f,-0.7949f,  0.0411f,0.6109f,-0.7907f,  -0.0400f,0.6887f,-0.7239f,  
	-0.0277f,0.6971f,-0.7164f,  -0.1026f,0.7661f,-0.6344f,  -0.0835f,0.7770f,-0.6240f,  -0.1512f,0.8353f,-0.5286f,  -0.1253f,0.8477f,-0.5155f,  -0.1852f,0.8937f,-0.4088f,  -0.1525f,0.9066f,-0.3935f,  -0.2044f,0.9386f,-0.2779f,  -0.1656f,0.9510f,-0.2609f,  -0.2097f,0.9678f,-0.1391f,  -0.1655f,0.9788f,-0.1210f,  -0.2025f,0.9793f,0.0043f,  -0.1539f,0.9878f,0.0228f,  -0.1847f,0.9715f,0.1487f,  -0.1331f,0.9769f,0.1672f,  
	-0.1591f,0.9435f,0.2908f,  -0.1059f,0.9452f,0.3087f,  -0.1285f,0.8950f,0.4272f,  -0.0754f,0.8928f,0.4441f,  -0.0961f,0.8265f,0.5547f,  -0.0445f,0.8204f,0.5701f,  -0.0650f,0.7392f,0.6704f,  -0.0164f,0.7294f,0.6839f,  -0.0381f,0.6350f,0.7716f,  0.0060f,0.6219f,0.7831f,  -0.0181f,0.5163f,0.8562f,  0.0206f,0.5006f,0.8654f,  -0.0070f,0.3862f,0.9224f,  0.0253f,0.3687f,0.9292f,  -0.0065f,0.2482f,0.9687f,  
	0.0189f,0.2297f,0.9731f,  -0.0175f,0.1059f,0.9942f,  0.0009f,0.0874f,0.9962f,  -0.0401f,-0.0367f,0.9985f,  -0.0289f,-0.0544f,0.9981f,  -0.0740f,-0.1761f,0.9816f,  -0.0697f,-0.1921f,0.9789f,  -0.1182f,-0.3084f,0.9439f,  -0.1201f,-0.3220f,0.9391f,  -0.1710f,-0.4303f,0.8863f,  -0.1785f,-0.4410f,0.8796f,  -0.2304f,-0.5389f,0.8102f,  -0.2426f,-0.5461f,0.8018f,  -0.2941f,-0.6317f,0.7173f,  -0.3100f,-0.6351f,0.7075f,  
	-0.3592f,-0.7066f,0.6096f,  -0.3780f,-0.7062f,0.5987f,  -0.0005f,-0.5572f,0.8304f,  -0.0007f,-0.5561f,0.8311f,  0.1290f,-0.5387f,0.8326f,  0.1294f,-0.5422f,0.8302f,  0.2573f,-0.5197f,0.8147f,  0.2579f,-0.5277f,0.8093f,  0.3817f,-0.5004f,0.7771f,  0.3820f,-0.5127f,0.7689f,  0.4994f,-0.4806f,0.7208f,  0.4990f,-0.4970f,0.7099f,  0.6078f,-0.4603f,0.6471f,  0.6063f,-0.4805f,0.6336f,  0.7045f,-0.4390f,0.5576f,  
	0.7017f,-0.4626f,0.5419f,  0.7874f,-0.4164f,0.4545f,  0.7830f,-0.4430f,0.4367f,  0.8549f,-0.3919f,0.3399f,  0.8486f,-0.4210f,0.3204f,  0.9055f,-0.3649f,0.2167f,  0.8971f,-0.3960f,0.1958f,  0.9382f,-0.3349f,0.0876f,  0.9278f,-0.3673f,0.0656f,  0.9526f,-0.3010f,-0.0445f,  0.9401f,-0.3342f,-0.0671f,  0.9486f,-0.2627f,-0.1765f,  0.9342f,-0.2960f,-0.1992f,  0.9267f,-0.2194f,-0.3052f,  0.9105f,-0.2521f,-0.3277f,  
	0.8877f,-0.1706f,-0.4276f,  0.8702f,-0.2019f,-0.4495f,  0.8332f,-0.1157f,-0.5408f,  0.8146f,-0.1450f,-0.5615f,  0.7648f,-0.0546f,-0.6419f,  0.7459f,-0.0813f,-0.6611f,  0.6849f,0.0129f,-0.7285f,  0.6662f,-0.0106f,-0.7457f,  0.5960f,0.0866f,-0.7983f,  0.5782f,0.0668f,-0.8131f,  0.5009f,0.1660f,-0.8494f,  0.4850f,0.1503f,-0.8615f,  0.4028f,0.2503f,-0.8804f,  0.3896f,0.2391f,-0.8894f,  0.3047f,0.3386f,-0.8902f,  
	0.2952f,0.3318f,-0.8960f,  0.2098f,0.4294f,-0.8784f,  0.2048f,0.4270f,-0.8807f,  0.1210f,0.5208f,-0.8450f,  0.1215f,0.5225f,-0.8439f,  0.0411f,0.6109f,-0.7907f,  0.0477f,0.6162f,-0.7862f,  -0.0277f,0.6971f,-0.7164f,  -0.0144f,0.7053f,-0.7087f,  -0.0835f,0.7770f,-0.6240f,  -0.0632f,0.7873f,-0.6134f,  -0.1253f,0.8477f,-0.5155f,  -0.0980f,0.8591f,-0.5024f,  -0.1525f,0.9066f,-0.3935f,  -0.1186f,0.9180f,-0.3784f,  
	-0.1656f,0.9510f,-0.2609f,  -0.1256f,0.9615f,-0.2443f,  -0.1655f,0.9788f,-0.1210f,  -0.1203f,0.9873f,-0.1035f,  -0.1539f,0.9878f,0.0228f,  -0.1048f,0.9937f,0.0408f,  -0.1331f,0.9769f,0.1672f,  -0.0815f,0.9794f,0.1850f,  -0.1059f,0.9452f,0.3087f,  -0.0533f,0.9439f,0.3259f,  -0.0754f,0.8928f,0.4441f,  -0.0233f,0.8876f,0.4600f,  -0.0445f,0.8204f,0.5701f,  0.0055f,0.8113f,0.5845f,  -0.0164f,0.7294f,0.6839f,  
	0.0301f,0.7168f,0.6966f,  0.0060f,0.6219f,0.7831f,  0.0478f,0.6064f,0.7937f,  0.0206f,0.5006f,0.8654f,  0.0567f,0.4829f,0.8739f,  0.0253f,0.3687f,0.9292f,  0.0549f,0.3495f,0.9353f,  0.0189f,0.2297f,0.9731f,  0.0418f,0.2101f,0.9768f,  0.0009f,0.0874f,0.9962f,  0.0167f,0.0681f,0.9975f,  -0.0289f,-0.0544f,0.9981f,  -0.0200f,-0.0726f,0.9972f,  -0.0697f,-0.1921f,0.9789f,  -0.0672f,-0.2082f,0.9758f,  
	-0.1201f,-0.3220f,0.9391f,  -0.1235f,-0.3354f,0.9339f,  -0.1785f,-0.4410f,0.8796f,  -0.1871f,-0.4512f,0.8726f,  -0.2426f,-0.5461f,0.8018f,  -0.2554f,-0.5527f,0.7933f,  -0.3100f,-0.6351f,0.7075f,  -0.3262f,-0.6379f,0.6976f,  -0.3780f,-0.7062f,0.5987f,  -0.3966f,-0.7051f,0.5878f,  -0.0007f,-0.5561f,0.8311f,  -0.0008f,-0.5554f,0.8316f,  0.1294f,-0.5422f,0.8302f,  0.1296f,-0.5460f,0.8277f,  0.2579f,-0.5277f,0.8093f,  
	0.2581f,-0.5360f,0.8038f,  0.3820f,-0.5127f,0.7689f,  0.3817f,-0.5253f,0.7605f,  0.4990f,-0.4970f,0.7099f,  0.4978f,-0.5137f,0.6988f,  0.6063f,-0.4805f,0.6336f,  0.6039f,-0.5009f,0.6200f,  0.7017f,-0.4626f,0.5419f,  0.6977f,-0.4865f,0.5259f,  0.7830f,-0.4430f,0.4367f,  0.7772f,-0.4698f,0.4187f,  0.8486f,-0.4210f,0.3204f,  0.8407f,-0.4503f,0.3007f,  0.8971f,-0.3960f,0.1958f,  0.8871f,-0.4272f,0.1747f,  
	0.9278f,-0.3673f,0.0656f,  0.9155f,-0.3999f,0.0435f,  0.9401f,-0.3342f,-0.0671f,  0.9257f,-0.3674f,-0.0899f,  0.9342f,-0.2960f,-0.1992f,  0.9177f,-0.3293f,-0.2222f,  0.9105f,-0.2521f,-0.3277f,  0.8923f,-0.2846f,-0.3504f,  0.8702f,-0.2019f,-0.4495f,  0.8505f,-0.2331f,-0.4715f,  0.8146f,-0.1450f,-0.5615f,  0.7941f,-0.1741f,-0.5824f,  0.7459f,-0.0813f,-0.6611f,  0.7250f,-0.1076f,-0.6803f,  0.6662f,-0.0106f,-0.7457f,  
	0.6457f,-0.0335f,-0.7629f,  0.5782f,0.0668f,-0.8131f,  0.5590f,0.0477f,-0.8278f,  0.4850f,0.1503f,-0.8615f,  0.4679f,0.1354f,-0.8734f,  0.3896f,0.2391f,-0.8894f,  0.3756f,0.2286f,-0.8982f,  0.2952f,0.3318f,-0.8960f,  0.2852f,0.3258f,-0.9014f,  0.2048f,0.4270f,-0.8807f,  0.1999f,0.4252f,-0.8827f,  0.1215f,0.5225f,-0.8439f,  0.1223f,0.5247f,-0.8425f,  0.0477f,0.6162f,-0.7862f,  0.0550f,0.6216f,-0.7814f,  
	-0.0144f,0.7053f,-0.7087f,  -0.0001f,0.7133f,-0.7008f,  -0.0632f,0.7873f,-0.6134f,  -0.0418f,0.7969f,-0.6027f,  -0.0980f,0.8591f,-0.5024f,  -0.0696f,0.8693f,-0.4893f,  -0.1186f,0.9180f,-0.3784f,  -0.0836f,0.9279f,-0.3634f,  -0.1256f,0.9615f,-0.2443f,  -0.0848f,0.9700f,-0.2280f,  -0.1203f,0.9873f,-0.1035f,  -0.0747f,0.9935f,-0.0864f,  -0.1048f,0.9937f,0.0408f,  -0.0557f,0.9968f,0.0581f,  -0.0815f,0.9794f,0.1850f,  
	-0.0303f,0.9789f,0.2021f,  -0.0533f,0.9439f,0.3259f,  -0.0016f,0.9396f,0.3422f,  -0.0233f,0.8876f,0.4600f,  0.0273f,0.8795f,0.4752f,  0.0055f,0.8113f,0.5845f,  0.0536f,0.7995f,0.5982f,  0.0301f,0.7168f,0.6966f,  0.0743f,0.7018f,0.7085f,  0.0478f,0.6064f,0.7937f,  0.0871f,0.5888f,0.8036f,  0.0567f,0.4829f,0.8739f,  0.0901f,0.4634f,0.8816f,  0.0549f,0.3495f,0.9353f,  0.0820f,0.3291f,0.9407f,  
	0.0418f,0.2101f,0.9768f,  0.0619f,0.1895f,0.9799f,  0.0167f,0.0681f,0.9975f,  0.0301f,0.0482f,0.9984f,  -0.0200f,-0.0726f,0.9972f,  -0.0131f,-0.0908f,0.9958f,  -0.0672f,-0.2082f,0.9758f,  -0.0665f,-0.2242f,0.9723f,  -0.1235f,-0.3354f,0.9339f,  -0.1283f,-0.3485f,0.9285f,  -0.1871f,-0.4512f,0.8726f,  -0.1965f,-0.4608f,0.8654f,  -0.2554f,-0.5527f,0.7933f,  -0.2689f,-0.5587f,0.7846f,  -0.3262f,-0.6379f,0.6976f,  
	-0.3426f,-0.6401f,0.6877f,  -0.3966f,-0.7051f,0.5878f,  -0.4150f,-0.7035f,0.5770f,  -0.0008f,-0.5554f,0.8316f,  -0.0008f,-0.5549f,0.8319f,  0.1296f,-0.5460f,0.8277f,  0.1297f,-0.5501f,0.8250f,  0.2581f,-0.5360f,0.8038f,  0.2579f,-0.5445f,0.7981f,  0.3817f,-0.5253f,0.7605f,  0.3808f,-0.5381f,0.7520f,  0.4978f,-0.5137f,0.6988f,  0.4958f,-0.5306f,0.6875f,  0.6039f,-0.5009f,0.6200f,  0.6004f,-0.5216f,0.6061f,  
	0.6977f,-0.4865f,0.5259f,  0.6924f,-0.5106f,0.5097f,  0.7772f,-0.4698f,0.4187f,  0.7699f,-0.4969f,0.4004f,  0.8407f,-0.4503f,0.3007f,  0.8312f,-0.4799f,0.2807f,  0.8871f,-0.4272f,0.1747f,  0.8753f,-0.4587f,0.1533f,  0.9155f,-0.3999f,0.0435f,  0.9013f,-0.4326f,0.0211f,  0.9257f,-0.3674f,-0.0899f,  0.9092f,-0.4008f,-0.1129f,  0.9177f,-0.3293f,-0.2222f,  0.8991f,-0.3625f,-0.2454f,  0.8923f,-0.2846f,-0.3504f,  
	0.8718f,-0.3171f,-0.3734f,  0.8505f,-0.2331f,-0.4715f,  0.8287f,-0.2639f,-0.4936f,  0.7941f,-0.1741f,-0.5824f,  0.7713f,-0.2027f,-0.6033f,  0.7250f,-0.1076f,-0.6803f,  0.7020f,-0.1333f,-0.6996f,  0.6457f,-0.0335f,-0.7629f,  0.6233f,-0.0557f,-0.7800f,  0.5590f,0.0477f,-0.8278f,  0.5381f,0.0294f,-0.8423f,  0.4679f,0.1354f,-0.8734f,  0.4495f,0.1214f,-0.8850f,  0.3756f,0.2286f,-0.8982f,  0.3607f,0.2191f,-0.9066f,  
	0.2852f,0.3258f,-0.9014f,  0.2748f,0.3206f,-0.9065f,  0.1999f,0.4252f,-0.8827f,  0.1948f,0.4242f,-0.8844f,  0.1223f,0.5247f,-0.8425f,  0.1235f,0.5273f,-0.8407f,  0.0550f,0.6216f,-0.7814f,  0.0630f,0.6272f,-0.7763f,  -0.0001f,0.7133f,-0.7008f,  0.0151f,0.7210f,-0.6927f,  -0.0418f,0.7969f,-0.6027f,  -0.0194f,0.8058f,-0.5919f,  -0.0696f,0.8693f,-0.4893f,  -0.0402f,0.8784f,-0.4763f,  -0.0836f,0.9279f,-0.3634f,  
	-0.0477f,0.9360f,-0.3487f,  -0.0848f,0.9700f,-0.2280f,  -0.0433f,0.9763f,-0.2121f,  -0.0747f,0.9935f,-0.0864f,  -0.0289f,0.9971f,-0.0698f,  -0.0557f,0.9968f,0.0581f,  -0.0069f,0.9972f,0.0749f,  -0.0303f,0.9789f,0.2021f,  0.0200f,0.9756f,0.2184f,  -0.0016f,0.9396f,0.3422f,  0.0487f,0.9325f,0.3577f,  0.0273f,0.8795f,0.4752f,  0.0762f,0.8686f,0.4896f,  0.0536f,0.7995f,0.5982f,  0.0995f,0.7853f,0.6111f,  
	0.0743f,0.7018f,0.7085f,  0.1160f,0.6846f,0.7196f,  0.0871f,0.5888f,0.8036f,  0.1236f,0.5693f,0.8128f,  0.0901f,0.4634f,0.8816f,  0.1208f,0.4425f,0.8886f,  0.0820f,0.3291f,0.9407f,  0.1062f,0.3076f,0.9456f,  0.0619f,0.1895f,0.9799f,  0.0796f,0.1682f,0.9825f,  0.0301f,0.0482f,0.9984f,  0.0412f,0.0281f,0.9988f,  -0.0131f,-0.0908f,0.9958f,  -0.0083f,-0.1091f,0.9940f,  -0.0665f,-0.2242f,0.9723f,  
	-0.0674f,-0.2399f,0.9685f,  -0.1283f,-0.3485f,0.9285f,  -0.1343f,-0.3611f,0.9228f,  -0.1965f,-0.4608f,0.8654f,  -0.2069f,-0.4700f,0.8581f,  -0.2689f,-0.5587f,0.7846f,  -0.2827f,-0.5641f,0.7758f,  -0.3426f,-0.6401f,0.6877f,  -0.3590f,-0.6417f,0.6778f,  -0.4150f,-0.7035f,0.5770f,  -0.4331f,-0.7013f,0.5662f,  -0.0008f,-0.5549f,0.8319f,  -0.0008f,-0.5547f,0.8320f,  0.1297f,-0.5501f,0.8250f,  0.1296f,-0.5544f,0.8221f,  
	0.2579f,-0.5445f,0.7981f,  0.2572f,-0.5533f,0.7922f,  0.3808f,-0.5381f,0.7520f,  0.3792f,-0.5513f,0.7432f,  0.4958f,-0.5306f,0.6875f,  0.4929f,-0.5479f,0.6759f,  0.6004f,-0.5216f,0.6061f,  0.5958f,-0.5427f,0.5920f,  0.6924f,-0.5106f,0.5097f,  0.6859f,-0.5350f,0.4933f,  0.7699f,-0.4969f,0.4004f,  0.7611f,-0.5243f,0.3819f,  0.8312f,-0.4799f,0.2807f,  0.8200f,-0.5097f,0.2604f,  0.8753f,-0.4587f,0.1533f,  
	0.8615f,-0.4903f,0.1316f,  0.9013f,-0.4326f,0.0211f,  0.8851f,-0.4654f,-0.0016f,  0.9092f,-0.4008f,-0.1129f,  0.8905f,-0.4341f,-0.1361f,  0.8991f,-0.3625f,-0.2454f,  0.8782f,-0.3956f,-0.2688f,  0.8718f,-0.3171f,-0.3734f,  0.8490f,-0.3492f,-0.3964f,  0.8287f,-0.2639f,-0.4936f,  0.8045f,-0.2944f,-0.5159f,  0.7713f,-0.2027f,-0.6033f,  0.7464f,-0.2308f,-0.6243f,  0.7020f,-0.1333f,-0.6996f,  0.6770f,-0.1582f,-0.7187f,  
	0.6233f,-0.0557f,-0.7800f,  0.5992f,-0.0771f,-0.7969f,  0.5381f,0.0294f,-0.8423f,  0.5158f,0.0122f,-0.8567f,  0.4495f,0.1214f,-0.8850f,  0.4299f,0.1085f,-0.8963f,  0.3607f,0.2191f,-0.9066f,  0.3450f,0.2105f,-0.9147f,  0.2748f,0.3206f,-0.9065f,  0.2640f,0.3164f,-0.9112f,  0.1948f,0.4242f,-0.8844f,  0.1898f,0.4239f,-0.8856f,  0.1235f,0.5273f,-0.8407f,  0.1251f,0.5303f,-0.8385f,  0.0630f,0.6272f,-0.7763f,  
	0.0717f,0.6329f,-0.7709f,  0.0151f,0.7210f,-0.6927f,  0.0311f,0.7284f,-0.6844f,  -0.0194f,0.8058f,-0.5919f,  0.0039f,0.8138f,-0.5811f,  -0.0402f,0.8784f,-0.4763f,  -0.0099f,0.8861f,-0.4634f,  -0.0477f,0.9360f,-0.3487f,  -0.0112f,0.9424f,-0.3342f,  -0.0433f,0.9763f,-0.2121f,  -0.0016f,0.9805f,-0.1965f,  -0.0289f,0.9971f,-0.0698f,  0.0168f,0.9984f,-0.0537f,  -0.0069f,0.9972f,0.0749f,  0.0414f,0.9950f,0.0910f,  
	0.0200f,0.9756f,0.2184f,  0.0693f,0.9697f,0.2342f,  0.0487f,0.9325f,0.3577f,  0.0975f,0.9228f,0.3727f,  0.0762f,0.8686f,0.4896f,  0.1230f,0.8553f,0.5033f,  0.0995f,0.7853f,0.6111f,  0.1430f,0.7688f,0.6233f,  0.1160f,0.6846f,0.7196f,  0.1551f,0.6655f,0.7301f,  0.1236f,0.5693f,0.8128f,  0.1575f,0.5484f,0.8213f,  0.1208f,0.4425f,0.8886f,  0.1486f,0.4204f,0.8951f,  0.1062f,0.3076f,0.9456f,  
	0.1277f,0.2853f,0.9499f,  0.0796f,0.1682f,0.9825f,  0.0947f,0.1465f,0.9847f,  0.0412f,0.0281f,0.9988f,  0.0500f,0.0079f,0.9987f,  -0.0083f,-0.1091f,0.9940f,  -0.0053f,-0.1272f,0.9919f,  -0.0674f,-0.2399f,0.9685f,  -0.0698f,-0.2552f,0.9644f,  -0.1343f,-0.3611f,0.9228f,  -0.1415f,-0.3732f,0.9169f,  -0.2069f,-0.4700f,0.8581f,  -0.2180f,-0.4785f,0.8506f,  -0.2827f,-0.5641f,0.7758f,  -0.2968f,-0.5689f,0.7670f,  
	-0.3590f,-0.6417f,0.6778f,  -0.3753f,-0.6426f,0.6679f,  -0.4331f,-0.7013f,0.5662f,  -0.4509f,-0.6986f,0.5556f,  -0.0008f,-0.5547f,0.8320f,  -0.0008f,-0.5549f,0.8319f,  0.1296f,-0.5544f,0.8221f,  0.1293f,-0.5591f,0.8190f,  0.2572f,-0.5533f,0.7922f,  0.2561f,-0.5625f,0.7861f,  0.3792f,-0.5513f,0.7432f,  0.3769f,-0.5647f,0.7342f,  0.4929f,-0.5479f,0.6759f,  0.4891f,-0.5654f,0.6642f,  0.5958f,-0.5427f,0.5920f,  
	0.5901f,-0.5640f,0.5777f,  0.6859f,-0.5350f,0.4933f,  0.6779f,-0.5597f,0.4766f,  0.7611f,-0.5243f,0.3819f,  0.7507f,-0.5519f,0.3631f,  0.8200f,-0.5097f,0.2604f,  0.8070f,-0.5397f,0.2399f,  0.8615f,-0.4903f,0.1316f,  0.8458f,-0.5221f,0.1097f,  0.8851f,-0.4654f,-0.0016f,  0.8666f,-0.4983f,-0.0246f,  0.8905f,-0.4341f,-0.1361f,  0.8695f,-0.4674f,-0.1597f,  0.8782f,-0.3956f,-0.2688f,  0.8549f,-0.4286f,-0.2924f,  
	0.8490f,-0.3492f,-0.3964f,  0.8238f,-0.3810f,-0.4197f,  0.8045f,-0.2944f,-0.5159f,  0.7779f,-0.3243f,-0.5382f,  0.7464f,-0.2308f,-0.6243f,  0.7191f,-0.2581f,-0.6452f,  0.6770f,-0.1582f,-0.7187f,  0.6499f,-0.1823f,-0.7378f,  0.5992f,-0.0771f,-0.7969f,  0.5732f,-0.0973f,-0.8136f,  0.5158f,0.0122f,-0.8567f,  0.4919f,-0.0039f,-0.8707f,  0.4299f,0.1085f,-0.8963f,  0.4093f,0.0968f,-0.9073f,  0.3450f,0.2105f,-0.9147f,  
	0.3286f,0.2032f,-0.9224f,  0.2640f,0.3164f,-0.9112f,  0.2529f,0.3132f,-0.9154f,  0.1898f,0.4239f,-0.8856f,  0.1850f,0.4244f,-0.8864f,  0.1251f,0.5303f,-0.8385f,  0.1271f,0.5338f,-0.8360f,  0.0717f,0.6329f,-0.7709f,  0.0811f,0.6386f,-0.7653f,  0.0311f,0.7284f,-0.6844f,  0.0480f,0.7354f,-0.6759f,  0.0039f,0.8138f,-0.5811f,  0.0281f,0.8210f,-0.5702f,  -0.0099f,0.8861f,-0.4634f,  0.0211f,0.8925f,-0.4506f,  
	-0.0112f,0.9424f,-0.3342f,  0.0257f,0.9471f,-0.3199f,  -0.0016f,0.9805f,-0.1965f,  0.0401f,0.9826f,-0.1813f,  0.0168f,0.9984f,-0.0537f,  0.0620f,0.9974f,-0.0380f,  0.0414f,0.9950f,0.0910f,  0.0887f,0.9903f,0.1066f,  0.0693f,0.9697f,0.2342f,  0.1171f,0.9613f,0.2494f,  0.0975f,0.9228f,0.3727f,  0.1444f,0.9107f,0.3870f,  0.1230f,0.8553f,0.5033f,  0.1675f,0.8398f,0.5164f,  0.1430f,0.7688f,0.6233f,  
	0.1839f,0.7503f,0.6350f,  0.1551f,0.6655f,0.7301f,  0.1916f,0.6448f,0.7399f,  0.1575f,0.5484f,0.8213f,  0.1885f,0.5261f,0.8293f,  0.1486f,0.4204f,0.8951f,  0.1737f,0.3975f,0.9010f,  0.1277f,0.2853f,0.9499f,  0.1467f,0.2624f,0.9537f,  0.0947f,0.1465f,0.9847f,  0.1075f,0.1246f,0.9864f,  0.0500f,0.0079f,0.9987f,  0.0568f,-0.0123f,0.9983f,  -0.0053f,-0.1272f,0.9919f,  -0.0041f,-0.1449f,0.9894f,  
	-0.0698f,-0.2552f,0.9644f,  -0.0736f,-0.2700f,0.9600f,  -0.1415f,-0.3732f,0.9169f,  -0.1496f,-0.3847f,0.9108f,  -0.2180f,-0.4785f,0.8506f,  -0.2296f,-0.4864f,0.8431f,  -0.2968f,-0.5689f,0.7670f,  -0.3111f,-0.5730f,0.7582f,  -0.3753f,-0.6426f,0.6679f,  -0.3916f,-0.6430f,0.6582f,  -0.4509f,-0.6986f,0.5556f,  -0.4681f,-0.6953f,0.5453f,  -0.0008f,-0.5549f,0.8319f,  -0.0009f,-0.5553f,0.8317f,  0.1293f,-0.5591f,0.8190f,  
	0.1287f,-0.5640f,0.8157f,  0.2561f,-0.5625f,0.7861f,  0.2545f,-0.5719f,0.7798f,  0.3769f,-0.5647f,0.7342f,  0.3739f,-0.5785f,0.7250f,  0.4891f,-0.5654f,0.6642f,  0.4843f,-0.5832f,0.6522f,  0.5901f,-0.5640f,0.5777f,  0.5831f,-0.5856f,0.5631f,  0.6779f,-0.5597f,0.4766f,  0.6685f,-0.5847f,0.4596f,  0.7507f,-0.5519f,0.3631f,  0.7386f,-0.5798f,0.3441f,  0.8070f,-0.5397f,0.2399f,  0.7920f,-0.5698f,0.2191f,  
	0.8458f,-0.5221f,0.1097f,  0.8279f,-0.5540f,0.0875f,  0.8666f,-0.4983f,-0.0246f,  0.8459f,-0.5312f,-0.0478f,  0.8695f,-0.4674f,-0.1597f,  0.8461f,-0.5005f,-0.1834f,  0.8549f,-0.4286f,-0.2924f,  0.8291f,-0.4611f,-0.3162f,  0.8238f,-0.3810f,-0.4197f,  0.7961f,-0.4123f,-0.4430f,  0.7779f,-0.3243f,-0.5382f,  0.7489f,-0.3534f,-0.5606f,  0.7191f,-0.2581f,-0.6452f,  0.6895f,-0.2844f,-0.6661f,  0.6499f,-0.1823f,-0.7378f,  
	0.6207f,-0.2052f,-0.7567f,  0.5732f,-0.0973f,-0.8136f,  0.5453f,-0.1163f,-0.8301f,  0.4919f,-0.0039f,-0.8707f,  0.4666f,-0.0186f,-0.8843f,  0.4093f,0.0968f,-0.9073f,  0.3876f,0.0864f,-0.9178f,  0.3286f,0.2032f,-0.9224f,  0.3116f,0.1971f,-0.9295f,  0.2529f,0.3132f,-0.9154f,  0.2417f,0.3111f,-0.9191f,  0.1850f,0.4244f,-0.8864f,  0.1803f,0.4257f,-0.8867f,  0.1271f,0.5338f,-0.8360f,  0.1297f,0.5378f,-0.8330f,  
	0.0811f,0.6386f,-0.7653f,  0.0912f,0.6443f,-0.7593f,  0.0480f,0.7354f,-0.6759f,  0.0657f,0.7419f,-0.6673f,  0.0281f,0.8210f,-0.5702f,  0.0530f,0.8273f,-0.5593f,  0.0211f,0.8925f,-0.4506f,  0.0526f,0.8975f,-0.4378f,  0.0257f,0.9471f,-0.3199f,  0.0628f,0.9500f,-0.3058f,  0.0401f,0.9826f,-0.1813f,  0.0816f,0.9827f,-0.1664f,  0.0620f,0.9974f,-0.0380f,  0.1066f,0.9940f,-0.0228f,  0.0887f,0.9903f,0.1066f,  
	0.1348f,0.9834f,0.1218f,  0.1171f,0.9613f,0.2494f,  0.1633f,0.9506f,0.2640f,  0.1444f,0.9107f,0.3870f,  0.1892f,0.8965f,0.4007f,  0.1675f,0.8398f,0.5164f,  0.2096f,0.8223f,0.5290f,  0.1839f,0.7503f,0.6350f,  0.2223f,0.7303f,0.6460f,  0.1916f,0.6448f,0.7399f,  0.2251f,0.6228f,0.7493f,  0.1885f,0.5261f,0.8293f,  0.2167f,0.5028f,0.8368f,  0.1737f,0.3975f,0.9010f,  0.1961f,0.3739f,0.9065f,  
	0.1467f,0.2624f,0.9537f,  0.1631f,0.2393f,0.9571f,  0.1075f,0.1246f,0.9864f,  0.1180f,0.1028f,0.9877f,  0.0568f,-0.0123f,0.9983f,  0.0617f,-0.0322f,0.9976f,  -0.0041f,-0.1449f,0.9894f,  -0.0045f,-0.1623f,0.9867f,  -0.0736f,-0.2700f,0.9600f,  -0.0786f,-0.2843f,0.9555f,  -0.1496f,-0.3847f,0.9108f,  -0.1585f,-0.3955f,0.9047f,  -0.2296f,-0.4864f,0.8431f,  -0.2417f,-0.4935f,0.8355f,  -0.3111f,-0.5730f,0.7582f,  
	-0.3256f,-0.5764f,0.7495f,  -0.3916f,-0.6430f,0.6582f,  -0.4075f,-0.6428f,0.6487f,  -0.4681f,-0.6953f,0.5453f,  -0.4849f,-0.6917f,0.5352f,  -0.0009f,-0.5553f,0.8317f,  -0.0010f,-0.5560f,0.8312f,  0.1287f,-0.5640f,0.8157f,  0.1278f,-0.5693f,0.8121f,  0.2545f,-0.5719f,0.7798f,  0.2524f,-0.5816f,0.7733f,  0.3739f,-0.5785f,0.7250f,  0.3701f,-0.5925f,0.7155f,  0.4843f,-0.5832f,0.6522f,  0.4784f,-0.6014f,0.6399f,  
	0.5831f,-0.5856f,0.5631f,  0.5749f,-0.6074f,0.5482f,  0.6685f,-0.5847f,0.4596f,  0.6575f,-0.6099f,0.4424f,  0.7386f,-0.5798f,0.3441f,  0.7247f,-0.6078f,0.3247f,  0.7920f,-0.5698f,0.2191f,  0.7750f,-0.6001f,0.1980f,  0.8279f,-0.5540f,0.0875f,  0.8078f,-0.5858f,0.0650f,  0.8459f,-0.5312f,-0.0478f,  0.8228f,-0.5639f,-0.0712f,  0.8461f,-0.5005f,-0.1834f,  0.8201f,-0.5333f,-0.2073f,  0.8291f,-0.4611f,-0.3162f,  
	0.8007f,-0.4932f,-0.3401f,  0.7961f,-0.4123f,-0.4430f,  0.7658f,-0.4428f,-0.4664f,  0.7489f,-0.3534f,-0.5606f,  0.7174f,-0.3816f,-0.5829f,  0.6895f,-0.2844f,-0.6661f,  0.6577f,-0.3095f,-0.6868f,  0.6207f,-0.2052f,-0.7567f,  0.5895f,-0.2267f,-0.7753f,  0.5453f,-0.1163f,-0.8301f,  0.5158f,-0.1338f,-0.8462f,  0.4666f,-0.0186f,-0.8843f,  0.4399f,-0.0318f,-0.8975f,  0.3876f,0.0864f,-0.9178f,  0.3650f,0.0776f,-0.9278f,  
	0.3116f,0.1971f,-0.9295f,  0.2943f,0.1925f,-0.9361f,  0.2417f,0.3111f,-0.9191f,  0.2305f,0.3102f,-0.9223f,  0.1803f,0.4257f,-0.8867f,  0.1761f,0.4278f,-0.8865f,  0.1297f,0.5378f,-0.8330f,  0.1329f,0.5423f,-0.8296f,  0.0912f,0.6443f,-0.7593f,  0.1021f,0.6501f,-0.7530f,  0.0657f,0.7419f,-0.6673f,  0.0841f,0.7479f,-0.6584f,  0.0530f,0.8273f,-0.5593f,  0.0785f,0.8326f,-0.5482f,  0.0526f,0.8975f,-0.4378f,  
	0.0843f,0.9012f,-0.4251f,  0.0628f,0.9500f,-0.3058f,  0.0998f,0.9512f,-0.2919f,  0.0816f,0.9827f,-0.1664f,  0.1226f,0.9808f,-0.1518f,  0.1066f,0.9940f,-0.0228f,  0.1501f,0.9886f,-0.0079f,  0.1348f,0.9834f,0.1218f,  0.1795f,0.9743f,0.1365f,  0.1633f,0.9506f,0.2640f,  0.2076f,0.9378f,0.2781f,  0.1892f,0.8965f,0.4007f,  0.2318f,0.8803f,0.4140f,  0.2096f,0.8223f,0.5290f,  0.2493f,0.8032f,0.5410f,  
	0.2223f,0.7303f,0.6460f,  0.2580f,0.7088f,0.6566f,  0.2251f,0.6228f,0.7493f,  0.2560f,0.5996f,0.7582f,  0.2167f,0.5028f,0.8368f,  0.2422f,0.4789f,0.8438f,  0.1961f,0.3739f,0.9065f,  0.2159f,0.3499f,0.9116f,  0.1631f,0.2393f,0.9571f,  0.1771f,0.2161f,0.9602f,  0.1180f,0.1028f,0.9877f,  0.1264f,0.0811f,0.9887f,  0.0617f,-0.0322f,0.9976f,  0.0647f,-0.0517f,0.9966f,  -0.0045f,-0.1623f,0.9867f,  
	-0.0063f,-0.1791f,0.9838f,  -0.0786f,-0.2843f,0.9555f,  -0.0846f,-0.2979f,0.9508f,  -0.1585f,-0.3955f,0.9047f,  -0.1680f,-0.4056f,0.8985f,  -0.2417f,-0.4935f,0.8355f,  -0.2540f,-0.5000f,0.8279f,  -0.3256f,-0.5764f,0.7495f,  -0.3399f,-0.5792f,0.7409f,  -0.4075f,-0.6428f,0.6487f,  -0.4232f,-0.6420f,0.6393f,  -0.4849f,-0.6917f,0.5352f,  -0.5012f,-0.6875f,0.5254f,  -0.0010f,-0.5560f,0.8312f,  -0.0012f,-0.5570f,0.8305f,  
	0.1278f,-0.5693f,0.8121f,  0.1265f,-0.5748f,0.8084f,  0.2524f,-0.5816f,0.7733f,  0.2496f,-0.5917f,0.7666f,  0.3701f,-0.5925f,0.7155f,  0.3655f,-0.6069f,0.7058f,  0.4784f,-0.6014f,0.6399f,  0.4714f,-0.6198f,0.6274f,  0.5749f,-0.6074f,0.5482f,  0.5652f,-0.6296f,0.5331f,  0.6575f,-0.6099f,0.4424f,  0.6449f,-0.6353f,0.4248f,  0.7247f,-0.6078f,0.3247f,  0.7088f,-0.6360f,0.3051f,  0.7750f,-0.6001f,0.1980f,  
	0.7559f,-0.6305f,0.1766f,  0.8078f,-0.5858f,0.0650f,  0.7854f,-0.6176f,0.0422f,  0.8228f,-0.5639f,-0.0712f,  0.7971f,-0.5963f,-0.0949f,  0.8201f,-0.5333f,-0.2073f,  0.7915f,-0.5656f,-0.2314f,  0.8007f,-0.4932f,-0.3401f,  0.7696f,-0.5245f,-0.3641f,  0.7658f,-0.4428f,-0.4664f,  0.7329f,-0.4723f,-0.4897f,  0.7174f,-0.3816f,-0.5829f,  0.6833f,-0.4086f,-0.6051f,  0.6577f,-0.3095f,-0.6868f,  0.6235f,-0.3333f,-0.7072f,  
	0.5895f,-0.2267f,-0.7753f,  0.5563f,-0.2466f,-0.7936f,  0.5158f,-0.1338f,-0.8462f,  0.4847f,-0.1495f,-0.8618f,  0.4399f,-0.0318f,-0.8975f,  0.4121f,-0.0432f,-0.9101f,  0.3650f,0.0776f,-0.9278f,  0.3418f,0.0705f,-0.9371f,  0.2943f,0.1925f,-0.9361f,  0.2766f,0.1894f,-0.9421f,  0.2305f,0.3102f,-0.9223f,  0.2194f,0.3105f,-0.9249f,  0.1761f,0.4278f,-0.8865f,  0.1722f,0.4309f,-0.8858f,  0.1329f,0.5423f,-0.8296f,  
	0.1367f,0.5471f,-0.8258f,  0.1021f,0.6501f,-0.7530f,  0.1137f,0.6558f,-0.7463f,  0.0841f,0.7479f,-0.6584f,  0.1032f,0.7535f,-0.6493f,  0.0785f,0.8326f,-0.5482f,  0.1045f,0.8370f,-0.5372f,  0.0843f,0.9012f,-0.4251f,  0.1163f,0.9035f,-0.4125f,  0.0998f,0.9512f,-0.2919f,  0.1366f,0.9508f,-0.2782f,  0.1226f,0.9808f,-0.1518f,  0.1629f,0.9770f,-0.1374f,  0.1501f,0.9886f,-0.0079f,  0.1926f,0.9813f,0.0066f,  
	0.1795f,0.9743f,0.1365f,  0.2225f,0.9632f,0.1508f,  0.2076f,0.9378f,0.2781f,  0.2499f,0.9232f,0.2919f,  0.2318f,0.8803f,0.4140f,  0.2721f,0.8625f,0.4267f,  0.2493f,0.8032f,0.5410f,  0.2864f,0.7827f,0.5526f,  0.2580f,0.7088f,0.6566f,  0.2909f,0.6862f,0.6667f,  0.2560f,0.5996f,0.7582f,  0.2842f,0.5757f,0.7667f,  0.2422f,0.4789f,0.8438f,  0.2651f,0.4544f,0.8504f,  0.2159f,0.3499f,0.9116f,  
	0.2332f,0.3257f,0.9163f,  0.1771f,0.2161f,0.9602f,  0.1889f,0.1930f,0.9629f,  0.1264f,0.0811f,0.9887f,  0.1327f,0.0597f,0.9894f,  0.0647f,-0.0517f,0.9966f,  0.0661f,-0.0707f,0.9953f,  -0.0063f,-0.1791f,0.9838f,  -0.0094f,-0.1952f,0.9807f,  -0.0846f,-0.2979f,0.9508f,  -0.0916f,-0.3108f,0.9461f,  -0.1680f,-0.4056f,0.8985f,  -0.1782f,-0.4150f,0.8922f,  -0.2540f,-0.5000f,0.8279f,  -0.2666f,-0.5057f,0.8205f,  
	-0.3399f,-0.5792f,0.7409f,  -0.3542f,-0.5813f,0.7325f,  -0.4232f,-0.6420f,0.6393f,  -0.4385f,-0.6407f,0.6303f,  -0.5012f,-0.6875f,0.5254f,  -0.5169f,-0.6831f,0.5160f,  -0.0012f,-0.5570f,0.8305f,  -0.0015f,-0.5582f,0.8297f,  0.1265f,-0.5748f,0.8084f,  0.1249f,-0.5807f,0.8045f,  0.2496f,-0.5917f,0.7666f,  0.2463f,-0.6020f,0.7596f,  0.3655f,-0.6069f,0.7058f,  0.3599f,-0.6215f,0.6958f,  0.4714f,-0.6198f,0.6274f,  
	0.4632f,-0.6385f,0.6146f,  0.5652f,-0.6296f,0.5331f,  0.5541f,-0.6520f,0.5176f,  0.6449f,-0.6353f,0.4248f,  0.6305f,-0.6609f,0.4070f,  0.7088f,-0.6360f,0.3051f,  0.6910f,-0.6642f,0.2851f,  0.7559f,-0.6305f,0.1766f,  0.7345f,-0.6607f,0.1549f,  0.7854f,-0.6176f,0.0422f,  0.7604f,-0.6491f,0.0192f,  0.7971f,-0.5963f,-0.0949f,  0.7688f,-0.6283f,-0.1188f,  0.7915f,-0.5656f,-0.2314f,  0.7602f,-0.5972f,-0.2556f,  
	0.7696f,-0.5245f,-0.3641f,  0.7358f,-0.5549f,-0.3881f,  0.7329f,-0.4723f,-0.4897f,  0.6973f,-0.5006f,-0.5130f,  0.6833f,-0.4086f,-0.6051f,  0.6468f,-0.4341f,-0.6270f,  0.6235f,-0.3333f,-0.7072f,  0.5871f,-0.3553f,-0.7273f,  0.5563f,-0.2466f,-0.7936f,  0.5212f,-0.2647f,-0.8114f,  0.4847f,-0.1495f,-0.8618f,  0.4522f,-0.1633f,-0.8768f,  0.4121f,-0.0432f,-0.9101f,  0.3834f,-0.0526f,-0.9221f,  0.3418f,0.0705f,-0.9371f,  
	0.3180f,0.0653f,-0.9458f,  0.2766f,0.1894f,-0.9421f,  0.2590f,0.1878f,-0.9474f,  0.2194f,0.3105f,-0.9249f,  0.2087f,0.3121f,-0.9268f,  0.1722f,0.4309f,-0.8858f,  0.1690f,0.4348f,-0.8845f,  0.1367f,0.5471f,-0.8258f,  0.1413f,0.5524f,-0.8215f,  0.1137f,0.6558f,-0.7463f,  0.1260f,0.6614f,-0.7394f,  0.1032f,0.7535f,-0.6493f,  0.1229f,0.7584f,-0.6401f,  0.1045f,0.8370f,-0.5372f,  0.1308f,0.8403f,-0.5260f,  
	0.1163f,0.9035f,-0.4125f,  0.1483f,0.9045f,-0.3999f,  0.1366f,0.9508f,-0.2782f,  0.1730f,0.9487f,-0.2646f,  0.1629f,0.9770f,-0.1374f,  0.2024f,0.9715f,-0.1233f,  0.1926f,0.9813f,0.0066f,  0.2336f,0.9721f,0.0209f,  0.2225f,0.9632f,0.1508f,  0.2638f,0.9504f,0.1648f,  0.2499f,0.9232f,0.2919f,  0.2902f,0.9070f,0.3052f,  0.2721f,0.8625f,0.4267f,  0.3099f,0.8432f,0.4392f,  0.2864f,0.7827f,0.5526f,  
	0.3209f,0.7610f,0.5638f,  0.2909f,0.6862f,0.6667f,  0.3212f,0.6627f,0.6765f,  0.2842f,0.5757f,0.7667f,  0.3097f,0.5512f,0.7748f,  0.2651f,0.4544f,0.8504f,  0.2853f,0.4297f,0.8567f,  0.2332f,0.3257f,0.9163f,  0.2482f,0.3015f,0.9206f,  0.1889f,0.1930f,0.9629f,  0.1985f,0.1701f,0.9652f,  0.1327f,0.0597f,0.9894f,  0.1374f,0.0388f,0.9898f,  0.0661f,-0.0707f,0.9953f,  0.0660f,-0.0891f,0.9938f,  
	-0.0094f,-0.1952f,0.9807f,  -0.0135f,-0.2106f,0.9775f,  -0.0916f,-0.3108f,0.9461f,  -0.0993f,-0.3229f,0.9412f,  -0.1782f,-0.4150f,0.8922f,  -0.1886f,-0.4235f,0.8860f,  -0.2666f,-0.5057f,0.8205f,  -0.2792f,-0.5107f,0.8132f,  -0.3542f,-0.5813f,0.7325f,  -0.3683f,-0.5829f,0.7243f,  -0.4385f,-0.6407f,0.6303f,  -0.4533f,-0.6389f,0.6215f,  -0.5169f,-0.6831f,0.5160f,  -0.5320f,-0.6783f,0.5069f,  -0.0015f,-0.5582f,0.8297f,  
	-0.0020f,-0.5598f,0.8286f,  0.1249f,-0.5807f,0.8045f,  0.1229f,-0.5868f,0.8003f,  0.2463f,-0.6020f,0.7596f,  0.2422f,-0.6126f,0.7523f,  0.3599f,-0.6215f,0.6958f,  0.3533f,-0.6365f,0.6856f,  0.4632f,-0.6385f,0.6146f,  0.4538f,-0.6575f,0.6015f,  0.5541f,-0.6520f,0.5176f,  0.5414f,-0.6746f,0.5019f,  0.6305f,-0.6609f,0.4070f,  0.6143f,-0.6867f,0.3888f,  0.6910f,-0.6642f,0.2851f,  0.6710f,-0.6925f,0.2649f,  
	0.7345f,-0.6607f,0.1549f,  0.7107f,-0.6908f,0.1329f,  0.7604f,-0.6491f,0.0192f,  0.7329f,-0.6803f,-0.0041f,  0.7688f,-0.6283f,-0.1188f,  0.7378f,-0.6597f,-0.1428f,  0.7602f,-0.5972f,-0.2556f,  0.7262f,-0.6280f,-0.2799f,  0.7358f,-0.5549f,-0.3881f,  0.6993f,-0.5841f,-0.4121f,  0.6973f,-0.5006f,-0.5130f,  0.6591f,-0.5275f,-0.5361f,  0.6468f,-0.4341f,-0.6270f,  0.6080f,-0.4579f,-0.6486f,  0.5871f,-0.3553f,-0.7273f,  
	0.5487f,-0.3754f,-0.7470f,  0.5212f,-0.2647f,-0.8114f,  0.4845f,-0.2807f,-0.8286f,  0.4522f,-0.1633f,-0.8768f,  0.4185f,-0.1749f,-0.8912f,  0.3834f,-0.0526f,-0.9221f,  0.3540f,-0.0600f,-0.9333f,  0.3180f,0.0653f,-0.9458f,  0.2941f,0.0619f,-0.9538f,  0.2590f,0.1878f,-0.9474f,  0.2415f,0.1880f,-0.9520f,  0.2087f,0.3121f,-0.9268f,  0.1984f,0.3150f,-0.9281f,  0.1690f,0.4348f,-0.8845f,  0.1666f,0.4395f,-0.8826f,  
	0.1413f,0.5524f,-0.8215f,  0.1467f,0.5580f,-0.8167f,  0.1260f,0.6614f,-0.7394f,  0.1391f,0.6669f,-0.7320f,  0.1229f,0.7584f,-0.6401f,  0.1431f,0.7628f,-0.6306f,  0.1308f,0.8403f,-0.5260f,  0.1574f,0.8428f,-0.5148f,  0.1483f,0.9045f,-0.3999f,  0.1800f,0.9042f,-0.3873f,  0.1730f,0.9487f,-0.2646f,  0.2087f,0.9452f,-0.2512f,  0.2024f,0.9715f,-0.1233f,  0.2408f,0.9644f,-0.1094f,  0.2336f,0.9721f,0.0209f,  
	0.2732f,0.9613f,0.0349f,  0.2638f,0.9504f,0.1648f,  0.3033f,0.9360f,0.1784f,  0.2902f,0.9070f,0.3052f,  0.3282f,0.8894f,0.3183f,  0.3099f,0.8432f,0.4392f,  0.3454f,0.8228f,0.4513f,  0.3209f,0.7610f,0.5638f,  0.3529f,0.7384f,0.5747f,  0.3212f,0.6627f,0.6765f,  0.3490f,0.6386f,0.6858f,  0.3097f,0.5512f,0.7748f,  0.3326f,0.5264f,0.7825f,  0.2853f,0.4297f,0.8567f,  0.3033f,0.4050f,0.8626f,  
	0.2482f,0.3015f,0.9206f,  0.2609f,0.2776f,0.9246f,  0.1985f,0.1701f,0.9652f,  0.2063f,0.1476f,0.9673f,  0.1374f,0.0388f,0.9898f,  0.1403f,0.0184f,0.9899f,  0.0660f,-0.0891f,0.9938f,  0.0647f,-0.1068f,0.9922f,  -0.0135f,-0.2106f,0.9775f,  -0.0187f,-0.2252f,0.9741f,  -0.0993f,-0.3229f,0.9412f,  -0.1075f,-0.3341f,0.9364f,  -0.1886f,-0.4235f,0.8860f,  -0.1994f,-0.4314f,0.8799f,  -0.2792f,-0.5107f,0.8132f,  
	-0.2918f,-0.5150f,0.8060f,  -0.3683f,-0.5829f,0.7243f,  -0.3821f,-0.5838f,0.7164f,  -0.4533f,-0.6389f,0.6215f,  -0.4678f,-0.6367f,0.6130f,  -0.5320f,-0.6783f,0.5069f,  -0.5464f,-0.6732f,0.4982f,  -0.0020f,-0.5598f,0.8286f,  -0.0026f,-0.5616f,0.8274f,  0.1229f,-0.5868f,0.8003f,  0.1204f,-0.5933f,0.7960f,  0.2422f,-0.6126f,0.7523f,  0.2374f,-0.6236f,0.7449f,  0.3533f,-0.6365f,0.6856f,  0.3457f,-0.6517f,0.6751f,  
	0.4538f,-0.6575f,0.6015f,  0.4430f,-0.6766f,0.5881f,  0.5414f,-0.6746f,0.5019f,  0.5270f,-0.6973f,0.4858f,  0.6143f,-0.6867f,0.3888f,  0.5961f,-0.7124f,0.3703f,  0.6710f,-0.6925f,0.2649f,  0.6488f,-0.7206f,0.2444f,  0.7107f,-0.6908f,0.1329f,  0.6845f,-0.7206f,0.1107f,  0.7329f,-0.6803f,-0.0041f,  0.7028f,-0.7109f,-0.0274f,  0.7378f,-0.6597f,-0.1428f,  0.7041f,-0.6902f,-0.1669f,  0.7262f,-0.6280f,-0.2799f,  
	0.6893f,-0.6575f,-0.3041f,  0.6993f,-0.5841f,-0.4121f,  0.6601f,-0.6118f,-0.4359f,  0.6591f,-0.5275f,-0.5361f,  0.6184f,-0.5526f,-0.5588f,  0.6080f,-0.4579f,-0.6486f,  0.5668f,-0.4796f,-0.6698f,  0.5487f,-0.3754f,-0.7470f,  0.5084f,-0.3932f,-0.7661f,  0.4845f,-0.2807f,-0.8286f,  0.4463f,-0.2943f,-0.8451f,  0.4185f,-0.1749f,-0.8912f,  0.3838f,-0.1842f,-0.9048f,  0.3540f,-0.0600f,-0.9333f,  0.3241f,-0.0651f,-0.9438f,  
	0.2941f,0.0619f,-0.9538f,  0.2701f,0.0606f,-0.9609f,  0.2415f,0.1880f,-0.9520f,  0.2244f,0.1898f,-0.9558f,  0.1984f,0.3150f,-0.9281f,  0.1889f,0.3191f,-0.9287f,  0.1666f,0.4395f,-0.8826f,  0.1649f,0.4451f,-0.8802f,  0.1467f,0.5580f,-0.8167f,  0.1530f,0.5640f,-0.8115f,  0.1391f,0.6669f,-0.7320f,  0.1529f,0.6722f,-0.7244f,  0.1431f,0.7628f,-0.6306f,  0.1638f,0.7666f,-0.6209f,  0.1574f,0.8428f,-0.5148f,  
	0.1840f,0.8442f,-0.5034f,  0.1800f,0.9042f,-0.3873f,  0.2115f,0.9027f,-0.3748f,  0.2087f,0.9452f,-0.2512f,  0.2438f,0.9402f,-0.2378f,  0.2408f,0.9644f,-0.1094f,  0.2780f,0.9558f,-0.0956f,  0.2732f,0.9613f,0.0349f,  0.3113f,0.9491f,0.0486f,  0.3033f,0.9360f,0.1784f,  0.3408f,0.9203f,0.1919f,  0.3282f,0.8894f,0.3183f,  0.3640f,0.8706f,0.3311f,  0.3454f,0.8228f,0.4513f,  0.3785f,0.8014f,0.4631f,  
	0.3529f,0.7384f,0.5747f,  0.3823f,0.7151f,0.5852f,  0.3490f,0.6386f,0.6858f,  0.3741f,0.6141f,0.6949f,  0.3326f,0.5264f,0.7825f,  0.3531f,0.5015f,0.7898f,  0.3033f,0.4050f,0.8626f,  0.3188f,0.3803f,0.8682f,  0.2609f,0.2776f,0.9246f,  0.2716f,0.2539f,0.9283f,  0.2063f,0.1476f,0.9673f,  0.2122f,0.1257f,0.9691f,  0.1403f,0.0184f,0.9899f,  0.1418f,-0.0012f,0.9899f,  0.0647f,-0.1068f,0.9922f,  
	0.0622f,-0.1238f,0.9904f,  -0.0187f,-0.2252f,0.9741f,  -0.0245f,-0.2390f,0.9707f,  -0.1075f,-0.3341f,0.9364f,  -0.1162f,-0.3446f,0.9315f,  -0.1994f,-0.4314f,0.8799f,  -0.2103f,-0.4384f,0.8738f,  -0.2918f,-0.5150f,0.8060f,  -0.3042f,-0.5187f,0.7990f,  -0.3821f,-0.5838f,0.7164f,  -0.3955f,-0.5842f,0.7087f,  -0.4678f,-0.6367f,0.6130f,  -0.4816f,-0.6341f,0.6049f,  -0.5464f,-0.6732f,0.4982f,  -0.5602f,-0.6679f,0.4900f,  
	-0.0026f,-0.5616f,0.8274f,  -0.0035f,-0.5637f,0.8260f,  0.1204f,-0.5933f,0.7960f,  0.1174f,-0.6000f,0.7914f,  0.2374f,-0.6236f,0.7449f,  0.2318f,-0.6348f,0.7371f,  0.3457f,-0.6517f,0.6751f,  0.3370f,-0.6671f,0.6643f,  0.4430f,-0.6766f,0.5881f,  0.4308f,-0.6960f,0.5745f,  0.5270f,-0.6973f,0.4858f,  0.5109f,-0.7201f,0.4695f,  0.5961f,-0.7124f,0.3703f,  0.5758f,-0.7381f,0.3516f,  0.6488f,-0.7206f,0.2444f,  
	0.6243f,-0.7485f,0.2236f,  0.6845f,-0.7206f,0.1107f,  0.6557f,-0.7498f,0.0883f,  0.7028f,-0.7109f,-0.0274f,  0.6699f,-0.7407f,-0.0509f,  0.7041f,-0.6902f,-0.1669f,  0.6675f,-0.7197f,-0.1909f,  0.6893f,-0.6575f,-0.3041f,  0.6497f,-0.6857f,-0.3282f,  0.6601f,-0.6118f,-0.4359f,  0.6182f,-0.6378f,-0.4594f,  0.6184f,-0.5526f,-0.5588f,  0.5753f,-0.5756f,-0.5812f,  0.5668f,-0.4796f,-0.6698f,  0.5237f,-0.4990f,-0.6905f,  
	0.5084f,-0.3932f,-0.7661f,  0.4665f,-0.4086f,-0.7845f,  0.4463f,-0.2943f,-0.8451f,  0.4071f,-0.3053f,-0.8608f,  0.3838f,-0.1842f,-0.9048f,  0.3486f,-0.1910f,-0.9176f,  0.3241f,-0.0651f,-0.9438f,  0.2941f,-0.0679f,-0.9534f,  0.2701f,0.0606f,-0.9609f,  0.2465f,0.0613f,-0.9672f,  0.2244f,0.1898f,-0.9558f,  0.2080f,0.1933f,-0.9588f,  0.1889f,0.3191f,-0.9287f,  0.1802f,0.3245f,-0.9285f,  0.1649f,0.4451f,-0.8802f,  
	0.1642f,0.4514f,-0.8771f,  0.1530f,0.5640f,-0.8115f,  0.1601f,0.5702f,-0.8058f,  0.1529f,0.6722f,-0.7244f,  0.1674f,0.6774f,-0.7164f,  0.1638f,0.7666f,-0.6209f,  0.1849f,0.7698f,-0.6109f,  0.1840f,0.8442f,-0.5034f,  0.2107f,0.8447f,-0.4919f,  0.2115f,0.9027f,-0.3748f,  0.2427f,0.9000f,-0.3622f,  0.2438f,0.9402f,-0.2378f,  0.2780f,0.9340f,-0.2245f,  0.2780f,0.9558f,-0.0956f,  0.3140f,0.9459f,-0.0820f,  
	0.3113f,0.9491f,0.0486f,  0.3477f,0.9355f,0.0623f,  0.3408f,0.9203f,0.1919f,  0.3764f,0.9035f,0.2052f,  0.3640f,0.8706f,0.3311f,  0.3976f,0.8508f,0.3436f,  0.3785f,0.8014f,0.4631f,  0.4091f,0.7793f,0.4746f,  0.3823f,0.7151f,0.5852f,  0.4093f,0.6913f,0.5955f,  0.3741f,0.6141f,0.6949f,  0.3970f,0.5894f,0.7036f,  0.3531f,0.5015f,0.7898f,  0.3713f,0.4765f,0.7969f,  0.3188f,0.3803f,0.8682f,  
	0.3323f,0.3559f,0.8734f,  0.2716f,0.2539f,0.9283f,  0.2804f,0.2307f,0.9317f,  0.2122f,0.1257f,0.9691f,  0.2165f,0.1043f,0.9707f,  0.1418f,-0.0012f,0.9899f,  0.1421f,-0.0202f,0.9897f,  0.0622f,-0.1238f,0.9904f,  0.0589f,-0.1398f,0.9884f,  -0.0245f,-0.2390f,0.9707f,  -0.0310f,-0.2520f,0.9672f,  -0.1162f,-0.3446f,0.9315f,  -0.1253f,-0.3543f,0.9267f,  -0.2103f,-0.4384f,0.8738f,  -0.2212f,-0.4447f,0.8679f,  
	-0.3042f,-0.5187f,0.7990f,  -0.3165f,-0.5218f,0.7922f,  -0.3955f,-0.5842f,0.7087f,  -0.4086f,-0.5842f,0.7013f,  -0.4816f,-0.6341f,0.6049f,  -0.4949f,-0.6312f,0.5972f,  -0.5602f,-0.6679f,0.4900f,  -0.5733f,-0.6624f,0.4822f,  -0.0035f,-0.5637f,0.8260f,  -0.0046f,-0.5661f,0.8243f,  0.1174f,-0.6000f,0.7914f,  0.1138f,-0.6070f,0.7865f,  0.2318f,-0.6348f,0.7371f,  0.2253f,-0.6462f,0.7292f,  0.3370f,-0.6671f,0.6643f,  
	0.3271f,-0.6828f,0.6533f,  0.4308f,-0.6960f,0.5745f,  0.4170f,-0.7155f,0.5605f,  0.5109f,-0.7201f,0.4695f,  0.4930f,-0.7429f,0.4528f,  0.5758f,-0.7381f,0.3516f,  0.5534f,-0.7636f,0.3326f,  0.6243f,-0.7485f,0.2236f,  0.5974f,-0.7759f,0.2026f,  0.6557f,-0.7498f,0.0883f,  0.6243f,-0.7784f,0.0658f,  0.6699f,-0.7407f,-0.0509f,  0.6343f,-0.7695f,-0.0745f,  0.6675f,-0.7197f,-0.1909f,  0.6282f,-0.7477f,-0.2149f,  
	0.6497f,-0.6857f,-0.3282f,  0.6074f,-0.7121f,-0.3521f,  0.6182f,-0.6378f,-0.4594f,  0.5738f,-0.6617f,-0.4825f,  0.5753f,-0.5756f,-0.5812f,  0.5299f,-0.5963f,-0.6030f,  0.5237f,-0.4990f,-0.6905f,  0.4787f,-0.5159f,-0.7104f,  0.4665f,-0.4086f,-0.7845f,  0.4233f,-0.4212f,-0.8021f,  0.4071f,-0.3053f,-0.8608f,  0.3670f,-0.3137f,-0.8757f,  0.3486f,-0.1910f,-0.9176f,  0.3130f,-0.1952f,-0.9295f,  0.2941f,-0.0679f,-0.9534f,  
	0.2643f,-0.0683f,-0.9620f,  0.2465f,0.0613f,-0.9672f,  0.2235f,0.0640f,-0.9726f,  0.2080f,0.1933f,-0.9588f,  0.1924f,0.1984f,-0.9610f,  0.1802f,0.3245f,-0.9285f,  0.1726f,0.3310f,-0.9277f,  0.1642f,0.4514f,-0.8771f,  0.1645f,0.4583f,-0.8734f,  0.1601f,0.5702f,-0.8058f,  0.1681f,0.5765f,-0.7996f,  0.1674f,0.6774f,-0.7164f,  0.1826f,0.6822f,-0.7080f,  0.1849f,0.7698f,-0.6109f,  0.2063f,0.7724f,-0.6007f,  
	0.2107f,0.8447f,-0.4919f,  0.2374f,0.8443f,-0.4803f,  0.2427f,0.9000f,-0.3622f,  0.2732f,0.8962f,-0.3495f,  0.2780f,0.9340f,-0.2245f,  0.3113f,0.9266f,-0.2112f,  0.3140f,0.9459f,-0.0820f,  0.3486f,0.9348f,-0.0684f,  0.3477f,0.9355f,0.0623f,  0.3823f,0.9209f,0.0758f,  0.3764f,0.9035f,0.2052f,  0.4099f,0.8856f,0.2182f,  0.3976f,0.8508f,0.3436f,  0.4290f,0.8303f,0.3559f,  0.4091f,0.7793f,0.4746f,  
	0.4375f,0.7567f,0.4858f,  0.4093f,0.6913f,0.5955f,  0.4340f,0.6672f,0.6053f,  0.3970f,0.5894f,0.7036f,  0.4174f,0.5646f,0.7120f,  0.3713f,0.4765f,0.7969f,  0.3874f,0.4517f,0.8037f,  0.3323f,0.3559f,0.8734f,  0.3439f,0.3319f,0.8784f,  0.2804f,0.2307f,0.9317f,  0.2875f,0.2082f,0.9349f,  0.2165f,0.1043f,0.9707f,  0.2194f,0.0838f,0.9720f,  0.1421f,-0.0202f,0.9897f,  0.1412f,-0.0382f,0.9892f,  
	0.0589f,-0.1398f,0.9884f,  0.0546f,-0.1551f,0.9864f,  -0.0310f,-0.2520f,0.9672f,  -0.0381f,-0.2641f,0.9637f,  -0.1253f,-0.3543f,0.9267f,  -0.1344f,-0.3631f,0.9220f,  -0.2212f,-0.4447f,0.8679f,  -0.2321f,-0.4503f,0.8621f,  -0.3165f,-0.5218f,0.7922f,  -0.3286f,-0.5242f,0.7857f,  -0.4086f,-0.5842f,0.7013f,  -0.4211f,-0.5836f,0.6943f,  -0.4949f,-0.6312f,0.5972f,  -0.5076f,-0.6279f,0.5899f,  -0.5733f,-0.6624f,0.4822f,  
	-0.5858f,-0.6568f,0.4748f,  -0.0046f,-0.5661f,0.8243f,  -0.0060f,-0.5688f,0.8225f,  0.1138f,-0.6070f,0.7865f,  0.1097f,-0.6142f,0.7815f,  0.2253f,-0.6462f,0.7292f,  0.2179f,-0.6579f,0.7209f,  0.3271f,-0.6828f,0.6533f,  0.3160f,-0.6986f,0.6420f,  0.4170f,-0.7155f,0.5605f,  0.4017f,-0.7350f,0.5463f,  0.4930f,-0.7429f,0.4528f,  0.4731f,-0.7656f,0.4360f,  0.5534f,-0.7636f,0.3326f,  0.5288f,-0.7887f,0.3134f,  
	0.5974f,-0.7759f,0.2026f,  0.5680f,-0.8028f,0.1815f,  0.6243f,-0.7784f,0.0658f,  0.5903f,-0.8061f,0.0433f,  0.6343f,-0.7695f,-0.0745f,  0.5960f,-0.7970f,-0.0979f,  0.6282f,-0.7477f,-0.2149f,  0.5862f,-0.7742f,-0.2387f,  0.6074f,-0.7121f,-0.3521f,  0.5625f,-0.7365f,-0.3756f,  0.5738f,-0.6617f,-0.4825f,  0.5271f,-0.6833f,-0.5052f,  0.5299f,-0.5963f,-0.6030f,  0.4827f,-0.6143f,-0.6242f,  0.4787f,-0.5159f,-0.7104f,  
	0.4323f,-0.5299f,-0.7296f,  0.4233f,-0.4212f,-0.8021f,  0.3792f,-0.4310f,-0.8188f,  0.3670f,-0.3137f,-0.8757f,  0.3266f,-0.3192f,-0.8897f,  0.3130f,-0.1952f,-0.9295f,  0.2776f,-0.1967f,-0.9403f,  0.2643f,-0.0683f,-0.9620f,  0.2351f,-0.0663f,-0.9697f,  0.2235f,0.0640f,-0.9726f,  0.2013f,0.0688f,-0.9771f,  0.1924f,0.1984f,-0.9610f,  0.1780f,0.2050f,-0.9624f,  0.1726f,0.3310f,-0.9277f,  0.1661f,0.3386f,-0.9262f,  
	0.1645f,0.4583f,-0.8734f,  0.1660f,0.4658f,-0.8692f,  0.1681f,0.5765f,-0.7996f,  0.1771f,0.5830f,-0.7929f,  0.1826f,0.6822f,-0.7080f,  0.1984f,0.6868f,-0.6993f,  0.2063f,0.7724f,-0.6007f,  0.2280f,0.7743f,-0.5903f,  0.2374f,0.8443f,-0.4803f,  0.2638f,0.8431f,-0.4686f,  0.2732f,0.8962f,-0.3495f,  0.3033f,0.8914f,-0.3369f,  0.3113f,0.9266f,-0.2112f,  0.3435f,0.9181f,-0.1979f,  0.3486f,0.9348f,-0.0684f,  
	0.3818f,0.9226f,-0.0549f,  0.3823f,0.9209f,0.0758f,  0.4153f,0.9053f,0.0891f,  0.4099f,0.8856f,0.2182f,  0.4415f,0.8670f,0.2311f,  0.4290f,0.8303f,0.3559f,  0.4582f,0.8091f,0.3679f,  0.4375f,0.7567f,0.4858f,  0.4637f,0.7337f,0.4968f,  0.4340f,0.6672f,0.6053f,  0.4564f,0.6430f,0.6150f,  0.4174f,0.5646f,0.7120f,  0.4358f,0.5399f,0.7201f,  0.3874f,0.4517f,0.8037f,  0.4014f,0.4273f,0.8101f,  
	0.3439f,0.3319f,0.8784f,  0.3536f,0.3084f,0.8831f,  0.2875f,0.2082f,0.9349f,  0.2931f,0.1863f,0.9378f,  0.2194f,0.0838f,0.9720f,  0.2212f,0.0640f,0.9731f,  0.1412f,-0.0382f,0.9892f,  0.1394f,-0.0555f,0.9887f,  0.0546f,-0.1551f,0.9864f,  0.0498f,-0.1694f,0.9843f,  -0.0381f,-0.2641f,0.9637f,  -0.0454f,-0.2753f,0.9603f,  -0.1344f,-0.3631f,0.9220f,  -0.1438f,-0.3712f,0.9173f,  -0.2321f,-0.4503f,0.8621f,  
	-0.2429f,-0.4553f,0.8566f,  -0.3286f,-0.5242f,0.7857f,  -0.3402f,-0.5261f,0.7794f,  -0.4211f,-0.5836f,0.6943f,  -0.4332f,-0.5827f,0.6876f,  -0.5076f,-0.6279f,0.5899f,  -0.5198f,-0.6245f,0.5830f,  -0.5858f,-0.6568f,0.4748f,  -0.5976f,-0.6511f,0.4679f,  -0.0060f,-0.5688f,0.8225f,  -0.0077f,-0.5717f,0.8204f,  0.1097f,-0.6142f,0.7815f,  0.1049f,-0.6217f,0.7762f,  0.2179f,-0.6579f,0.7209f,  0.2095f,-0.6698f,0.7124f,  
	0.3160f,-0.6986f,0.6420f,  0.3035f,-0.7145f,0.6304f,  0.4017f,-0.7350f,0.5463f,  0.3846f,-0.7545f,0.5318f,  0.4731f,-0.7656f,0.4360f,  0.4512f,-0.7880f,0.4188f,  0.5288f,-0.7887f,0.3134f,  0.5019f,-0.8134f,0.2940f,  0.5680f,-0.8028f,0.1815f,  0.5361f,-0.8288f,0.1603f,  0.5903f,-0.8061f,0.0433f,  0.5536f,-0.8325f,0.0207f,  0.5960f,-0.7970f,-0.0979f,  0.5550f,-0.8229f,-0.1213f,  0.5862f,-0.7742f,-0.2387f,  
	0.5417f,-0.7987f,-0.2622f,  0.5625f,-0.7365f,-0.3756f,  0.5153f,-0.7586f,-0.3987f,  0.5271f,-0.6833f,-0.5052f,  0.4785f,-0.7023f,-0.5272f,  0.4827f,-0.6143f,-0.6242f,  0.4339f,-0.6295f,-0.6446f,  0.4323f,-0.5299f,-0.7296f,  0.3848f,-0.5409f,-0.7479f,  0.3792f,-0.4310f,-0.8188f,  0.3345f,-0.4377f,-0.8346f,  0.3266f,-0.3192f,-0.8897f,  0.2861f,-0.3218f,-0.9026f,  0.2776f,-0.1967f,-0.9403f,  0.2427f,-0.1955f,-0.9502f,  
	0.2351f,-0.0663f,-0.9697f,  0.2067f,-0.0620f,-0.9764f,  0.2013f,0.0688f,-0.9771f,  0.1803f,0.0755f,-0.9807f,  0.1780f,0.2050f,-0.9624f,  0.1649f,0.2131f,-0.9630f,  0.1661f,0.3386f,-0.9262f,  0.1610f,0.3471f,-0.9239f,  0.1660f,0.4658f,-0.8692f,  0.1686f,0.4737f,-0.8644f,  0.1771f,0.5830f,-0.7929f,  0.1870f,0.5895f,-0.7859f,  0.1984f,0.6868f,-0.6993f,  0.2148f,0.6910f,-0.6902f,  0.2280f,0.7743f,-0.5903f,  
	0.2499f,0.7756f,-0.5797f,  0.2638f,0.8431f,-0.4686f,  0.2901f,0.8410f,-0.4568f,  0.3033f,0.8914f,-0.3369f,  0.3326f,0.8856f,-0.3241f,  0.3435f,0.9181f,-0.1979f,  0.3747f,0.9086f,-0.1846f,  0.3818f,0.9226f,-0.0549f,  0.4135f,0.9095f,-0.0415f,  0.4153f,0.9053f,0.0891f,  0.4465f,0.8889f,0.1023f,  0.4415f,0.8670f,0.2311f,  0.4711f,0.8477f,0.2437f,  0.4582f,0.8091f,0.3679f,  0.4854f,0.7876f,0.3797f,  
	0.4637f,0.7337f,0.4968f,  0.4876f,0.7104f,0.5074f,  0.4564f,0.6430f,0.6150f,  0.4768f,0.6188f,0.6243f,  0.4358f,0.5399f,0.7201f,  0.4521f,0.5155f,0.7279f,  0.4014f,0.4273f,0.8101f,  0.4136f,0.4034f,0.8162f,  0.3536f,0.3084f,0.8831f,  0.3617f,0.2856f,0.8875f,  0.2931f,0.1863f,0.9378f,  0.2973f,0.1651f,0.9404f,  0.2212f,0.0640f,0.9731f,  0.2217f,0.0451f,0.9741f,  0.1394f,-0.0555f,0.9887f,  
	0.1368f,-0.0718f,0.9880f,  0.0498f,-0.1694f,0.9843f,  0.0445f,-0.1828f,0.9821f,  -0.0454f,-0.2753f,0.9603f,  -0.0530f,-0.2858f,0.9568f,  -0.1438f,-0.3712f,0.9173f,  -0.1531f,-0.3786f,0.9128f,  -0.2429f,-0.4553f,0.8566f,  -0.2534f,-0.4595f,0.8512f,  -0.3402f,-0.5261f,0.7794f,  -0.3514f,-0.5275f,0.7735f,  -0.4332f,-0.5827f,0.6876f,  -0.4448f,-0.5814f,0.6813f,  -0.5198f,-0.6245f,0.5830f,  -0.5312f,-0.6208f,0.5766f,  
	-0.5976f,-0.6511f,0.4679f,  -0.6086f,-0.6454f,0.4616f,  -0.0077f,-0.5717f,0.8204f,  -0.0098f,-0.5750f,0.8181f,  0.1049f,-0.6217f,0.7762f,  0.0995f,-0.6295f,0.7706f,  0.2095f,-0.6698f,0.7124f,  0.2001f,-0.6818f,0.7036f,  0.3035f,-0.7145f,0.6304f,  0.2896f,-0.7305f,0.6185f,  0.3846f,-0.7545f,0.5318f,  0.3658f,-0.7738f,0.5171f,  0.4512f,-0.7880f,0.4188f,  0.4272f,-0.8101f,0.4015f,  0.5019f,-0.8134f,0.2940f,  
	0.4727f,-0.8374f,0.2745f,  0.5361f,-0.8288f,0.1603f,  0.5017f,-0.8538f,0.1390f,  0.5536f,-0.8325f,0.0207f,  0.5143f,-0.8576f,-0.0017f,  0.5550f,-0.8229f,-0.1213f,  0.5115f,-0.8471f,-0.1443f,  0.5417f,-0.7987f,-0.2622f,  0.4947f,-0.8209f,-0.2853f,  0.5153f,-0.7586f,-0.3987f,  0.4660f,-0.7781f,-0.4212f,  0.4785f,-0.7023f,-0.5272f,  0.4281f,-0.7183f,-0.5485f,  0.4339f,-0.6295f,-0.6446f,  0.3839f,-0.6416f,-0.6641f,  
	0.3848f,-0.5409f,-0.7479f,  0.3366f,-0.5488f,-0.7652f,  0.3345f,-0.4377f,-0.8346f,  0.2897f,-0.4413f,-0.8493f,  0.2861f,-0.3218f,-0.9026f,  0.2461f,-0.3214f,-0.9144f,  0.2427f,-0.1955f,-0.9502f,  0.2086f,-0.1917f,-0.9590f,  0.2067f,-0.0620f,-0.9764f,  0.1795f,-0.0554f,-0.9822f,  0.1803f,0.0755f,-0.9807f,  0.1607f,0.0839f,-0.9834f,  0.1649f,0.2131f,-0.9630f,  0.1532f,0.2224f,-0.9628f,  0.1610f,0.3471f,-0.9239f,  
	0.1572f,0.3564f,-0.9210f,  0.1686f,0.4737f,-0.8644f,  0.1724f,0.4820f,-0.8590f,  0.1870f,0.5895f,-0.7859f,  0.1978f,0.5959f,-0.7783f,  0.2148f,0.6910f,-0.6902f,  0.2317f,0.6948f,-0.6809f,  0.2499f,0.7756f,-0.5797f,  0.2719f,0.7762f,-0.5689f,  0.2901f,0.8410f,-0.4568f,  0.3160f,0.8381f,-0.4447f,  0.3326f,0.8856f,-0.3241f,  0.3612f,0.8790f,-0.3113f,  0.3747f,0.9086f,-0.1846f,  0.4047f,0.8982f,-0.1714f,  
	0.4135f,0.9095f,-0.0415f,  0.4438f,0.8957f,-0.0282f,  0.4465f,0.8889f,0.1023f,  0.4760f,0.8719f,0.1153f,  0.4711f,0.8477f,0.2437f,  0.4988f,0.8280f,0.2561f,  0.4854f,0.7876f,0.3797f,  0.5105f,0.7657f,0.3912f,  0.4876f,0.7104f,0.5074f,  0.5096f,0.6872f,0.5178f,  0.4768f,0.6188f,0.6243f,  0.4951f,0.5948f,0.6333f,  0.4521f,0.5155f,0.7279f,  0.4666f,0.4914f,0.7354f,  0.4136f,0.4034f,0.8162f,  
	0.4241f,0.3800f,0.8220f,  0.3617f,0.2856f,0.8875f,  0.3684f,0.2634f,0.8916f,  0.2973f,0.1651f,0.9404f,  0.3003f,0.1448f,0.9428f,  0.2217f,0.0451f,0.9741f,  0.2214f,0.0270f,0.9748f,  0.1368f,-0.0718f,0.9880f,  0.1336f,-0.0872f,0.9872f,  0.0445f,-0.1828f,0.9821f,  0.0387f,-0.1954f,0.9800f,  -0.0530f,-0.2858f,0.9568f,  -0.0607f,-0.2954f,0.9535f,  -0.1531f,-0.3786f,0.9128f,  -0.1623f,-0.3851f,0.9085f,  
	-0.2534f,-0.4595f,0.8512f,  -0.2637f,-0.4632f,0.8461f,  -0.3514f,-0.5275f,0.7735f,  -0.3623f,-0.5284f,0.7678f,  -0.4448f,-0.5814f,0.6813f,  -0.4558f,-0.5798f,0.6753f,  -0.5312f,-0.6208f,0.5766f,  -0.5421f,-0.6169f,0.5706f,  -0.6086f,-0.6454f,0.4616f,  -0.6191f,-0.6396f,0.4557f,  -0.0098f,-0.5750f,0.8181f,  -0.0122f,-0.5784f,0.8156f,  0.0995f,-0.6295f,0.7706f,  0.0933f,-0.6374f,0.7648f,  0.2001f,-0.6818f,0.7036f,  
	0.1896f,-0.6939f,0.6946f,  0.2896f,-0.7305f,0.6185f,  0.2742f,-0.7464f,0.6064f,  0.3658f,-0.7738f,0.5171f,  0.3453f,-0.7929f,0.5021f,  0.4272f,-0.8101f,0.4015f,  0.4012f,-0.8316f,0.3841f,  0.4727f,-0.8374f,0.2745f,  0.4412f,-0.8605f,0.2550f,  0.5017f,-0.8538f,0.1390f,  0.4648f,-0.8775f,0.1178f,  0.5143f,-0.8576f,-0.0017f,  0.4726f,-0.8810f,-0.0240f,  0.5115f,-0.8471f,-0.1443f,  0.4656f,-0.8691f,-0.1671f,  
	0.4947f,-0.8209f,-0.2853f,  0.4456f,-0.8406f,-0.3079f,  0.4660f,-0.7781f,-0.4212f,  0.4149f,-0.7948f,-0.4429f,  0.4281f,-0.7183f,-0.5485f,  0.3763f,-0.7312f,-0.5689f,  0.3839f,-0.6416f,-0.6641f,  0.3330f,-0.6504f,-0.6827f,  0.3366f,-0.5488f,-0.7652f,  0.2882f,-0.5534f,-0.7815f,  0.2897f,-0.4413f,-0.8493f,  0.2452f,-0.4418f,-0.8629f,  0.2461f,-0.3214f,-0.9144f,  0.2068f,-0.3182f,-0.9252f,  0.2086f,-0.1917f,-0.9590f,  
	0.1756f,-0.1853f,-0.9669f,  0.1795f,-0.0554f,-0.9822f,  0.1538f,-0.0468f,-0.9870f,  0.1607f,0.0839f,-0.9834f,  0.1427f,0.0939f,-0.9853f,  0.1532f,0.2224f,-0.9628f,  0.1431f,0.2329f,-0.9619f,  0.1572f,0.3564f,-0.9210f,  0.1549f,0.3663f,-0.9175f,  0.1724f,0.4820f,-0.8590f,  0.1775f,0.4905f,-0.8532f,  0.1978f,0.5959f,-0.7783f,  0.2095f,0.6021f,-0.7704f,  0.2317f,0.6948f,-0.6809f,  0.2491f,0.6981f,-0.6712f,  
	0.2719f,0.7762f,-0.5689f,  0.2940f,0.7762f,-0.5578f,  0.3160f,0.8381f,-0.4447f,  0.3415f,0.8344f,-0.4326f,  0.3612f,0.8790f,-0.3113f,  0.3890f,0.8716f,-0.2984f,  0.4047f,0.8982f,-0.1714f,  0.4336f,0.8871f,-0.1582f,  0.4438f,0.8957f,-0.0282f,  0.4727f,0.8811f,-0.0150f,  0.4760f,0.8719f,0.1153f,  0.5037f,0.8543f,0.1282f,  0.4988f,0.8280f,0.2561f,  0.5247f,0.8079f,0.2683f,  0.5105f,0.7657f,0.3912f,  
	0.5337f,0.7438f,0.4025f,  0.5096f,0.6872f,0.5178f,  0.5296f,0.6640f,0.5279f,  0.4951f,0.5948f,0.6333f,  0.5115f,0.5711f,0.6420f,  0.4666f,0.4914f,0.7354f,  0.4793f,0.4679f,0.7425f,  0.4241f,0.3800f,0.8220f,  0.4331f,0.3572f,0.8275f,  0.3684f,0.2634f,0.8916f,  0.3737f,0.2421f,0.8954f,  0.3003f,0.1448f,0.9428f,  0.3022f,0.1254f,0.9449f,  0.2214f,0.0270f,0.9748f,  0.2203f,0.0099f,0.9754f,  
	0.1336f,-0.0872f,0.9872f,  0.1298f,-0.1017f,0.9863f,  0.0387f,-0.1954f,0.9800f,  0.0327f,-0.2071f,0.9778f,  -0.0607f,-0.2954f,0.9535f,  -0.0684f,-0.3041f,0.9502f,  -0.1623f,-0.3851f,0.9085f,  -0.1713f,-0.3910f,0.9043f,  -0.2637f,-0.4632f,0.8461f,  -0.2736f,-0.4663f,0.8413f,  -0.3623f,-0.5284f,0.7678f,  -0.3726f,-0.5289f,0.7625f,  -0.4558f,-0.5798f,0.6753f,  -0.4663f,-0.5779f,0.6698f,  -0.5421f,-0.6169f,0.5706f,  
	-0.5523f,-0.6129f,0.5650f,  -0.6191f,-0.6396f,0.4557f,  -0.6289f,-0.6339f,0.4502f,  -0.0122f,-0.5784f,0.8156f,  -0.0151f,-0.5822f,0.8129f,  0.0933f,-0.6374f,0.7648f,  0.0863f,-0.6456f,0.7588f,  0.1896f,-0.6939f,0.6946f,  0.1779f,-0.7061f,0.6854f,  0.2742f,-0.7464f,0.6064f,  0.2574f,-0.7621f,0.5941f,  0.3453f,-0.7929f,0.5021f,  0.3229f,-0.8115f,0.4870f,  0.4012f,-0.8316f,0.3841f,  0.3731f,-0.8523f,0.3665f,  
	0.4412f,-0.8605f,0.2550f,  0.4073f,-0.8824f,0.2354f,  0.4648f,-0.8775f,0.1178f,  0.4256f,-0.8997f,0.0968f,  0.4726f,-0.8810f,-0.0240f,  0.4286f,-0.9023f,-0.0459f,  0.4656f,-0.8691f,-0.1671f,  0.4176f,-0.8887f,-0.1893f,  0.4456f,-0.8406f,-0.3079f,  0.3947f,-0.8576f,-0.3298f,  0.4149f,-0.7948f,-0.4429f,  0.3624f,-0.8083f,-0.4640f,  0.3763f,-0.7312f,-0.5689f,  0.3237f,-0.7409f,-0.5884f,  0.3330f,-0.6504f,-0.6827f,  
	0.2819f,-0.6559f,-0.7002f,  0.2882f,-0.5534f,-0.7815f,  0.2401f,-0.5547f,-0.7967f,  0.2452f,-0.4418f,-0.8629f,  0.2014f,-0.4392f,-0.8755f,  0.2068f,-0.3182f,-0.9252f,  0.1687f,-0.3121f,-0.9349f,  0.1756f,-0.1853f,-0.9669f,  0.1442f,-0.1766f,-0.9737f,  0.1538f,-0.0468f,-0.9870f,  0.1298f,-0.0362f,-0.9909f,  0.1427f,0.0939f,-0.9853f,  0.1265f,0.1055f,-0.9864f,  0.1431f,0.2329f,-0.9619f,  0.1347f,0.2444f,-0.9603f,  
	0.1549f,0.3663f,-0.9175f,  0.1541f,0.3768f,-0.9134f,  0.1775f,0.4905f,-0.8532f,  0.1837f,0.4992f,-0.8468f,  0.2095f,0.6021f,-0.7704f,  0.2220f,0.6081f,-0.7622f,  0.2491f,0.6981f,-0.6712f,  0.2669f,0.7011f,-0.6613f,  0.2940f,0.7762f,-0.5578f,  0.3160f,0.7756f,-0.5465f,  0.3415f,0.8344f,-0.4326f,  0.3666f,0.8300f,-0.4203f,  0.3890f,0.8716f,-0.2984f,  0.4160f,0.8634f,-0.2856f,  0.4336f,0.8871f,-0.1582f,  
	0.4613f,0.8753f,-0.1451f,  0.4727f,0.8811f,-0.0150f,  0.5001f,0.8660f,-0.0020f,  0.5037f,0.8543f,0.1282f,  0.5298f,0.8363f,0.1408f,  0.5247f,0.8079f,0.2683f,  0.5487f,0.7876f,0.2803f,  0.5337f,0.7438f,0.4025f,  0.5550f,0.7218f,0.4134f,  0.5296f,0.6640f,0.5279f,  0.5478f,0.6410f,0.5376f,  0.5115f,0.5711f,0.6420f,  0.5263f,0.5478f,0.6504f,  0.4793f,0.4679f,0.7425f,  0.4905f,0.4449f,0.7493f,  
	0.4331f,0.3572f,0.8275f,  0.4407f,0.3352f,0.8327f,  0.3737f,0.2421f,0.8954f,  0.3779f,0.2216f,0.8989f,  0.3022f,0.1254f,0.9449f,  0.3033f,0.1069f,0.9469f,  0.2203f,0.0099f,0.9754f,  0.2186f,-0.0063f,0.9758f,  0.1298f,-0.1017f,0.9863f,  0.1256f,-0.1153f,0.9854f,  0.0327f,-0.2071f,0.9778f,  0.0266f,-0.2179f,0.9756f,  -0.0684f,-0.3041f,0.9502f,  -0.0761f,-0.3122f,0.9470f,  -0.1713f,-0.3910f,0.9043f,  
	-0.1802f,-0.3963f,0.9003f,  -0.2736f,-0.4663f,0.8413f,  -0.2831f,-0.4689f,0.8367f,  -0.3726f,-0.5289f,0.7625f,  -0.3825f,-0.5290f,0.7576f,  -0.4663f,-0.5779f,0.6698f,  -0.4761f,-0.5758f,0.6647f,  -0.5523f,-0.6129f,0.5650f,  -0.5620f,-0.6089f,0.5599f,  -0.6289f,-0.6339f,0.4502f,  -0.6380f,-0.6282f,0.4453f,  -0.0151f,-0.5822f,0.8129f,  -0.0184f,-0.5861f,0.8100f,  0.0863f,-0.6456f,0.7588f,  0.0786f,-0.6538f,0.7526f,  
	0.1779f,-0.7061f,0.6854f,  0.1651f,-0.7183f,0.6759f,  0.2574f,-0.7621f,0.5941f,  0.2390f,-0.7776f,0.5816f,  0.3229f,-0.8115f,0.4870f,  0.2986f,-0.8296f,0.4717f,  0.3731f,-0.8523f,0.3665f,  0.3429f,-0.8722f,0.3489f,  0.4073f,-0.8824f,0.2354f,  0.3713f,-0.9031f,0.2159f,  0.4256f,-0.8997f,0.0968f,  0.3842f,-0.9201f,0.0760f,  0.4286f,-0.9023f,-0.0459f,  0.3824f,-0.9215f,-0.0675f,  0.4176f,-0.8887f,-0.1893f,  
	0.3677f,-0.9057f,-0.2110f,  0.3947f,-0.8576f,-0.3298f,  0.3423f,-0.8716f,-0.3510f,  0.3624f,-0.8083f,-0.4640f,  0.3089f,-0.8187f,-0.4841f,  0.3237f,-0.7409f,-0.5884f,  0.2707f,-0.7472f,-0.6070f,  0.2819f,-0.6559f,-0.7002f,  0.2309f,-0.6580f,-0.7167f,  0.2401f,-0.5547f,-0.7967f,  0.1926f,-0.5527f,-0.8108f,  0.2014f,-0.4392f,-0.8755f,  0.1588f,-0.4336f,-0.8870f,  0.1687f,-0.3121f,-0.9349f,  0.1321f,-0.3035f,-0.9436f,  
	0.1442f,-0.1766f,-0.9737f,  0.1146f,-0.1657f,-0.9795f,  0.1298f,-0.0362f,-0.9909f,  0.1076f,-0.0238f,-0.9939f,  0.1265f,0.1055f,-0.9864f,  0.1121f,0.1182f,-0.9866f,  0.1347f,0.2444f,-0.9603f,  0.1280f,0.2566f,-0.9580f,  0.1541f,0.3768f,-0.9134f,  0.1548f,0.3877f,-0.9087f,  0.1837f,0.4992f,-0.8468f,  0.1911f,0.5077f,-0.8401f,  0.2220f,0.6081f,-0.7622f,  0.2353f,0.6139f,-0.7535f,  0.2669f,0.7011f,-0.6613f,  
	0.2851f,0.7034f,-0.6511f,  0.3160f,0.7756f,-0.5465f,  0.3380f,0.7743f,-0.5351f,  0.3666f,0.8300f,-0.4203f,  0.3912f,0.8249f,-0.4080f,  0.4160f,0.8634f,-0.2856f,  0.4421f,0.8545f,-0.2727f,  0.4613f,0.8753f,-0.1451f,  0.4878f,0.8629f,-0.1321f,  0.5001f,0.8660f,-0.0020f,  0.5260f,0.8504f,0.0109f,  0.5298f,0.8363f,0.1408f,  0.5543f,0.8181f,0.1533f,  0.5487f,0.7876f,0.2803f,  0.5710f,0.7673f,0.2919f,  
	0.5550f,0.7218f,0.4134f,  0.5747f,0.6999f,0.4241f,  0.5478f,0.6410f,0.5376f,  0.5643f,0.6183f,0.5470f,  0.5263f,0.5478f,0.6504f,  0.5394f,0.5250f,0.6583f,  0.4905f,0.4449f,0.7493f,  0.5002f,0.4226f,0.7558f,  0.4407f,0.3352f,0.8327f,  0.4471f,0.3140f,0.8375f,  0.3779f,0.2216f,0.8989f,  0.3811f,0.2020f,0.9022f,  0.3033f,0.1069f,0.9469f,  0.3036f,0.0893f,0.9486f,  0.2186f,-0.0063f,0.9758f,  
	0.2163f,-0.0215f,0.9761f,  0.1256f,-0.1153f,0.9854f,  0.1212f,-0.1280f,0.9843f,  0.0266f,-0.2179f,0.9756f,  0.0204f,-0.2279f,0.9735f,  -0.0761f,-0.3122f,0.9470f,  -0.0837f,-0.3195f,0.9439f,  -0.1802f,-0.3963f,0.9003f,  -0.1887f,-0.4009f,0.8965f,  -0.2831f,-0.4689f,0.8367f,  -0.2922f,-0.4710f,0.8323f,  -0.3825f,-0.5290f,0.7576f,  -0.3918f,-0.5287f,0.7529f,  -0.4761f,-0.5758f,0.6647f,  -0.4855f,-0.5735f,0.6599f,  
	-0.5620f,-0.6089f,0.5599f,  -0.5710f,-0.6048f,0.5552f,  -0.6380f,-0.6282f,0.4453f,  -0.6466f,-0.6226f,0.4408f,  -0.0184f,-0.5861f,0.8100f,  -0.0221f,-0.5903f,0.8068f,  0.0786f,-0.6538f,0.7526f,  0.0699f,-0.6622f,0.7461f,  0.1651f,-0.7183f,0.6759f,  0.1510f,-0.7304f,0.6662f,  0.2390f,-0.7776f,0.5816f,  0.2190f,-0.7927f,0.5689f,  0.2986f,-0.8296f,0.4717f,  0.2725f,-0.8470f,0.4564f,  0.3429f,-0.8722f,0.3489f,  
	0.3107f,-0.8909f,0.3313f,  0.3713f,-0.9031f,0.2159f,  0.3332f,-0.9221f,0.1965f,  0.3842f,-0.9201f,0.0760f,  0.3407f,-0.9385f,0.0554f,  0.3824f,-0.9215f,-0.0675f,  0.3345f,-0.9382f,-0.0886f,  0.3677f,-0.9057f,-0.2110f,  0.3163f,-0.9198f,-0.2321f,  0.3423f,-0.8716f,-0.3510f,  0.2888f,-0.8824f,-0.3714f,  0.3089f,-0.8187f,-0.4841f,  0.2549f,-0.8257f,-0.5033f,  0.2707f,-0.7472f,-0.6070f,  0.2177f,-0.7501f,-0.6245f,  
	0.2309f,-0.6580f,-0.7167f,  0.1804f,-0.6568f,-0.7322f,  0.1926f,-0.5527f,-0.8108f,  0.1461f,-0.5476f,-0.8239f,  0.1588f,-0.4336f,-0.8870f,  0.1177f,-0.4252f,-0.8974f,  0.1321f,-0.3035f,-0.9436f,  0.0974f,-0.2924f,-0.9513f,  0.1146f,-0.1657f,-0.9795f,  0.0870f,-0.1527f,-0.9844f,  0.1076f,-0.0238f,-0.9939f,  0.0876f,-0.0100f,-0.9961f,  0.1121f,0.1182f,-0.9866f,  0.0997f,0.1320f,-0.9862f,  0.1280f,0.2566f,-0.9580f,  
	0.1231f,0.2695f,-0.9551f,  0.1548f,0.3877f,-0.9087f,  0.1569f,0.3986f,-0.9036f,  0.1911f,0.5077f,-0.8401f,  0.1996f,0.5163f,-0.8328f,  0.2353f,0.6139f,-0.7535f,  0.2493f,0.6193f,-0.7445f,  0.2851f,0.7034f,-0.6511f,  0.3036f,0.7053f,-0.6407f,  0.3380f,0.7743f,-0.5351f,  0.3598f,0.7723f,-0.5235f,  0.3912f,0.8249f,-0.4080f,  0.4153f,0.8191f,-0.3957f,  0.4421f,0.8545f,-0.2727f,  0.4673f,0.8450f,-0.2600f,  
	0.4878f,0.8629f,-0.1321f,  0.5131f,0.8500f,-0.1192f,  0.5260f,0.8504f,0.0109f,  0.5505f,0.8345f,0.0236f,  0.5543f,0.8181f,0.1533f,  0.5772f,0.7996f,0.1654f,  0.5710f,0.7673f,0.2919f,  0.5917f,0.7469f,0.3033f,  0.5747f,0.6999f,0.4241f,  0.5926f,0.6783f,0.4344f,  0.5643f,0.6183f,0.5470f,  0.5792f,0.5961f,0.5560f,  0.5394f,0.5250f,0.6583f,  0.5511f,0.5028f,0.6659f,  0.5002f,0.4226f,0.7558f,  
	0.5086f,0.4011f,0.7618f,  0.4471f,0.3140f,0.8375f,  0.4524f,0.2937f,0.8421f,  0.3811f,0.2020f,0.9022f,  0.3835f,0.1833f,0.9052f,  0.3036f,0.0893f,0.9486f,  0.3033f,0.0726f,0.9501f,  0.2163f,-0.0215f,0.9761f,  0.2137f,-0.0359f,0.9763f,  0.1212f,-0.1280f,0.9843f,  0.1166f,-0.1398f,0.9833f,  0.0204f,-0.2279f,0.9735f,  0.0142f,-0.2372f,0.9714f,  -0.0837f,-0.3195f,0.9439f,  -0.0910f,-0.3261f,0.9410f,  
	-0.1887f,-0.4009f,0.8965f,  -0.1968f,-0.4050f,0.8929f,  -0.2922f,-0.4710f,0.8323f,  -0.3008f,-0.4727f,0.8283f,  -0.3918f,-0.5287f,0.7529f,  -0.4006f,-0.5282f,0.7486f,  -0.4855f,-0.5735f,0.6599f,  -0.4942f,-0.5710f,0.6555f,  -0.5710f,-0.6048f,0.5552f,  -0.5794f,-0.6007f,0.5509f,  -0.6466f,-0.6226f,0.4408f,  -0.6546f,-0.6171f,0.4367f,  -0.0221f,-0.5903f,0.8068f,  -0.0264f,-0.5948f,0.8034f,  0.0699f,-0.6622f,0.7461f,  
	0.0604f,-0.6706f,0.7393f,  0.1510f,-0.7304f,0.6662f,  0.1357f,-0.7423f,0.6562f,  0.2190f,-0.7927f,0.5689f,  0.1975f,-0.8074f,0.5560f,  0.2725f,-0.8470f,0.4564f,  0.2447f,-0.8635f,0.4410f,  0.3107f,-0.8909f,0.3313f,  0.2765f,-0.9083f,0.3138f,  0.3332f,-0.9221f,0.1965f,  0.2932f,-0.9394f,0.1774f,  0.3407f,-0.9385f,0.0554f,  0.2955f,-0.9547f,0.0353f,  0.3345f,-0.9382f,-0.0886f,  0.2850f,-0.9523f,-0.1091f,  
	0.3163f,-0.9198f,-0.2321f,  0.2639f,-0.9310f,-0.2524f,  0.2888f,-0.8824f,-0.3714f,  0.2348f,-0.8900f,-0.3910f,  0.2549f,-0.8257f,-0.5033f,  0.2008f,-0.8293f,-0.5215f,  0.2177f,-0.7501f,-0.6245f,  0.1652f,-0.7495f,-0.6410f,  0.1804f,-0.6568f,-0.7322f,  0.1310f,-0.6523f,-0.7466f,  0.1461f,-0.5476f,-0.8239f,  0.1013f,-0.5395f,-0.8359f,  0.1177f,-0.4252f,-0.8974f,  0.0785f,-0.4141f,-0.9069f,  0.0974f,-0.2924f,-0.9513f,  
	0.0647f,-0.2790f,-0.9581f,  0.0870f,-0.1527f,-0.9844f,  0.0615f,-0.1380f,-0.9885f,  0.0876f,-0.0100f,-0.9961f,  0.0697f,0.0052f,-0.9976f,  0.0997f,0.1320f,-0.9862f,  0.0894f,0.1467f,-0.9851f,  0.1231f,0.2695f,-0.9551f,  0.1201f,0.2827f,-0.9517f,  0.1569f,0.3986f,-0.9036f,  0.1606f,0.4098f,-0.8979f,  0.1996f,0.5163f,-0.8328f,  0.2092f,0.5245f,-0.8253f,  0.2493f,0.6193f,-0.7445f,  0.2639f,0.6242f,-0.7353f,  
	0.3036f,0.7053f,-0.6407f,  0.3222f,0.7065f,-0.6301f,  0.3598f,0.7723f,-0.5235f,  0.3814f,0.7698f,-0.5119f,  0.4153f,0.8191f,-0.3957f,  0.4387f,0.8127f,-0.3834f,  0.4673f,0.8450f,-0.2600f,  0.4915f,0.8350f,-0.2472f,  0.5131f,0.8500f,-0.1192f,  0.5372f,0.8367f,-0.1064f,  0.5505f,0.8345f,0.0236f,  0.5736f,0.8183f,0.0361f,  0.5772f,0.7996f,0.1654f,  0.5986f,0.7811f,0.1773f,  0.5917f,0.7469f,0.3033f,  
	0.6108f,0.7267f,0.3143f,  0.5926f,0.6783f,0.4344f,  0.6090f,0.6570f,0.4443f,  0.5792f,0.5961f,0.5560f,  0.5926f,0.5744f,0.5647f,  0.5511f,0.5028f,0.6659f,  0.5615f,0.4813f,0.6731f,  0.5086f,0.4011f,0.7618f,  0.5160f,0.3803f,0.7675f,  0.4524f,0.2937f,0.8421f,  0.4568f,0.2742f,0.8463f,  0.3835f,0.1833f,0.9052f,  0.3851f,0.1655f,0.9079f,  0.3033f,0.0726f,0.9501f,  0.3025f,0.0569f,0.9515f,  
	0.2137f,-0.0359f,0.9763f,  0.2107f,-0.0493f,0.9763f,  0.1166f,-0.1398f,0.9833f,  0.1118f,-0.1508f,0.9822f,  0.0142f,-0.2372f,0.9714f,  0.0081f,-0.2456f,0.9693f,  -0.0910f,-0.3261f,0.9410f,  -0.0981f,-0.3320f,0.9382f,  -0.1968f,-0.4050f,0.8929f,  -0.2046f,-0.4085f,0.8895f,  -0.3008f,-0.4727f,0.8283f,  -0.3090f,-0.4740f,0.8246f,  -0.4006f,-0.5282f,0.7486f,  -0.4089f,-0.5275f,0.7447f,  -0.4942f,-0.5710f,0.6555f,  
	-0.5023f,-0.5685f,0.6516f,  -0.5794f,-0.6007f,0.5509f,  -0.5873f,-0.5966f,0.5470f,  -0.6546f,-0.6171f,0.4367f,  -0.6619f,-0.6118f,0.4331f,  -0.0264f,-0.5948f,0.8034f,  -0.0312f,-0.5994f,0.7998f,  0.0604f,-0.6706f,0.7393f,  0.0500f,-0.6791f,0.7323f,  0.1357f,-0.7423f,0.6562f,  0.1191f,-0.7539f,0.6461f,  0.1975f,-0.8074f,0.5560f,  0.1745f,-0.8213f,0.5431f,  0.2447f,-0.8635f,0.4410f,  0.2151f,-0.8790f,0.4256f,  
	0.2765f,-0.9083f,0.3138f,  0.2407f,-0.9243f,0.2964f,  0.2932f,-0.9394f,0.1774f,  0.2515f,-0.9548f,0.1586f,  0.2955f,-0.9547f,0.0353f,  0.2488f,-0.9684f,0.0157f,  0.2850f,-0.9523f,-0.1091f,  0.2344f,-0.9635f,-0.1290f,  0.2639f,-0.9310f,-0.2524f,  0.2107f,-0.9390f,-0.2719f,  0.2348f,-0.8900f,-0.3910f,  0.1806f,-0.8942f,-0.4096f,  0.2008f,-0.8293f,-0.5215f,  0.1472f,-0.8295f,-0.5388f,  0.1652f,-0.7495f,-0.6410f,  
	0.1136f,-0.7457f,-0.6565f,  0.1310f,-0.6523f,-0.7466f,  0.0831f,-0.6447f,-0.7599f,  0.1013f,-0.5395f,-0.8359f,  0.0582f,-0.5287f,-0.8468f,  0.0785f,-0.4141f,-0.9069f,  0.0414f,-0.4005f,-0.9154f,  0.0647f,-0.2790f,-0.9581f,  0.0344f,-0.2637f,-0.9640f,  0.0615f,-0.1380f,-0.9885f,  0.0385f,-0.1218f,-0.9918f,  0.0697f,0.0052f,-0.9976f,  0.0541f,0.0214f,-0.9983f,  0.0894f,0.1467f,-0.9851f,  0.0811f,0.1619f,-0.9835f,  
	0.1201f,0.2827f,-0.9517f,  0.1187f,0.2963f,-0.9477f,  0.1606f,0.4098f,-0.8979f,  0.1656f,0.4208f,-0.8919f,  0.2092f,0.5245f,-0.8253f,  0.2198f,0.5325f,-0.8174f,  0.2639f,0.6242f,-0.7353f,  0.2791f,0.6286f,-0.7259f,  0.3222f,0.7065f,-0.6301f,  0.3409f,0.7072f,-0.6194f,  0.3814f,0.7698f,-0.5119f,  0.4026f,0.7666f,-0.5003f,  0.4387f,0.8127f,-0.3834f,  0.4615f,0.8058f,-0.3711f,  0.4915f,0.8350f,-0.2472f,  
	0.5148f,0.8246f,-0.2346f,  0.5372f,0.8367f,-0.1064f,  0.5602f,0.8231f,-0.0939f,  0.5736f,0.8183f,0.0361f,  0.5954f,0.8020f,0.0483f,  0.5986f,0.7811f,0.1773f,  0.6186f,0.7627f,0.1889f,  0.6108f,0.7267f,0.3143f,  0.6285f,0.7067f,0.3249f,  0.6090f,0.6570f,0.4443f,  0.6240f,0.6362f,0.4538f,  0.5926f,0.5744f,0.5647f,  0.6047f,0.5532f,0.5729f,  0.5615f,0.4813f,0.6731f,  0.5707f,0.4605f,0.6799f,  
	0.5160f,0.3803f,0.7675f,  0.5223f,0.3603f,0.7729f,  0.4568f,0.2742f,0.8463f,  0.4603f,0.2555f,0.8502f,  0.3851f,0.1655f,0.9079f,  0.3861f,0.1486f,0.9104f,  0.3025f,0.0569f,0.9515f,  0.3013f,0.0420f,0.9526f,  0.2107f,-0.0493f,0.9763f,  0.2076f,-0.0619f,0.9763f,  0.1118f,-0.1508f,0.9822f,  0.1072f,-0.1610f,0.9811f,  0.0081f,-0.2456f,0.9693f,  0.0022f,-0.2534f,0.9674f,  -0.0981f,-0.3320f,0.9382f,  
	-0.1049f,-0.3374f,0.9355f,  -0.2046f,-0.4085f,0.8895f,  -0.2120f,-0.4116f,0.8864f,  -0.3090f,-0.4740f,0.8246f,  -0.3166f,-0.4749f,0.8211f,  -0.4089f,-0.5275f,0.7447f,  -0.4166f,-0.5266f,0.7411f,  -0.5023f,-0.5685f,0.6516f,  -0.5099f,-0.5659f,0.6479f,  -0.5873f,-0.5966f,0.5470f,  -0.5945f,-0.5926f,0.5435f,  -0.6619f,-0.6118f,0.4331f,  -0.6687f,-0.6066f,0.4299f,  -0.0312f,-0.5994f,0.7998f,  -0.0365f,-0.6043f,0.7959f,  
	0.0500f,-0.6791f,0.7323f,  0.0387f,-0.6875f,0.7252f,  0.1191f,-0.7539f,0.6461f,  0.1013f,-0.7651f,0.6359f,  0.1745f,-0.8213f,0.5431f,  0.1500f,-0.8346f,0.5301f,  0.2151f,-0.8790f,0.4256f,  0.1840f,-0.8932f,0.4102f,  0.2407f,-0.9243f,0.2964f,  0.2032f,-0.9385f,0.2792f,  0.2515f,-0.9548f,0.1586f,  0.2084f,-0.9679f,0.1402f,  0.2488f,-0.9684f,0.0157f,  0.2010f,-0.9796f,-0.0034f,  0.2344f,-0.9635f,-0.1290f,  
	0.1831f,-0.9719f,-0.1482f,  0.2107f,-0.9390f,-0.2719f,  0.1573f,-0.9438f,-0.2906f,  0.1806f,-0.8942f,-0.4096f,  0.1267f,-0.8952f,-0.4273f,  0.1472f,-0.8295f,-0.5388f,  0.0944f,-0.8264f,-0.5551f,  0.1136f,-0.7457f,-0.6565f,  0.0635f,-0.7388f,-0.6709f,  0.0831f,-0.6447f,-0.7599f,  0.0369f,-0.6342f,-0.7723f,  0.0582f,-0.5287f,-0.8468f,  0.0173f,-0.5152f,-0.8569f,  0.0414f,-0.4005f,-0.9154f,  0.0067f,-0.3848f,-0.9230f,  
	0.0344f,-0.2637f,-0.9640f,  0.0065f,-0.2467f,-0.9691f,  0.0385f,-0.1218f,-0.9918f,  0.0178f,-0.1043f,-0.9944f,  0.0541f,0.0214f,-0.9983f,  0.0407f,0.0384f,-0.9984f,  0.0811f,0.1619f,-0.9835f,  0.0748f,0.1777f,-0.9812f,  0.1187f,0.2963f,-0.9477f,  0.1190f,0.3099f,-0.9433f,  0.1656f,0.4208f,-0.8919f,  0.1719f,0.4317f,-0.8855f,  0.2198f,0.5325f,-0.8174f,  0.2312f,0.5400f,-0.8093f,  0.2791f,0.6286f,-0.7259f,  
	0.2947f,0.6326f,-0.7163f,  0.3409f,0.7072f,-0.6194f,  0.3597f,0.7073f,-0.6086f,  0.4026f,0.7666f,-0.5003f,  0.4236f,0.7628f,-0.4886f,  0.4615f,0.8058f,-0.3711f,  0.4836f,0.7984f,-0.3589f,  0.5148f,0.8246f,-0.2346f,  0.5371f,0.8137f,-0.2222f,  0.5602f,0.8231f,-0.0939f,  0.5819f,0.8092f,-0.0815f,  0.5954f,0.8020f,0.0483f,  0.6158f,0.7856f,0.0602f,  0.6186f,0.7627f,0.1889f,  0.6372f,0.7443f,0.2000f,  
	0.6285f,0.7067f,0.3249f,  0.6447f,0.6870f,0.3352f,  0.6240f,0.6362f,0.4538f,  0.6377f,0.6157f,0.4629f,  0.6047f,0.5532f,0.5729f,  0.6157f,0.5327f,0.5807f,  0.5707f,0.4605f,0.6799f,  0.5788f,0.4404f,0.6863f,  0.5223f,0.3603f,0.7729f,  0.5276f,0.3412f,0.7779f,  0.4603f,0.2555f,0.8502f,  0.4632f,0.2378f,0.8538f,  0.3861f,0.1486f,0.9104f,  0.3867f,0.1327f,0.9126f,  0.3013f,0.0420f,0.9526f,  
	0.2997f,0.0281f,0.9536f,  0.2076f,-0.0619f,0.9763f,  0.2043f,-0.0736f,0.9761f,  0.1072f,-0.1610f,0.9811f,  0.1025f,-0.1704f,0.9800f,  0.0022f,-0.2534f,0.9674f,  -0.0035f,-0.2604f,0.9655f,  -0.1049f,-0.3374f,0.9355f,  -0.1114f,-0.3422f,0.9330f,  -0.2120f,-0.4116f,0.8864f,  -0.2189f,-0.4142f,0.8834f,  -0.3166f,-0.4749f,0.8211f,  -0.3238f,-0.4756f,0.8179f,  -0.4166f,-0.5266f,0.7411f,  -0.4238f,-0.5254f,0.7378f,  
	-0.5099f,-0.5659f,0.6479f,  -0.5169f,-0.5632f,0.6447f,  -0.5945f,-0.5926f,0.5435f,  -0.6012f,-0.5886f,0.5404f,  -0.6687f,-0.6066f,0.4299f,  -0.6750f,-0.6017f,0.4270f,  -0.0365f,-0.6043f,0.7959f,  -0.0425f,-0.6094f,0.7918f,  0.0387f,-0.6875f,0.7252f,  0.0264f,-0.6958f,0.7178f,  0.1013f,-0.7651f,0.6359f,  0.0823f,-0.7759f,0.6255f,  0.1500f,-0.8346f,0.5301f,  0.1241f,-0.8469f,0.5170f,  0.1840f,-0.8932f,0.4102f,  
	0.1514f,-0.9061f,0.3950f,  0.2032f,-0.9385f,0.2792f,  0.1643f,-0.9509f,0.2623f,  0.2084f,-0.9679f,0.1402f,  0.1641f,-0.9788f,0.1222f,  0.2010f,-0.9796f,-0.0034f,  0.1523f,-0.9881f,-0.0220f,  0.1831f,-0.9719f,-0.1482f,  0.1314f,-0.9772f,-0.1667f,  0.1573f,-0.9438f,-0.2906f,  0.1041f,-0.9455f,-0.3086f,  0.1267f,-0.8952f,-0.4273f,  0.0735f,-0.8929f,-0.4442f,  0.0944f,-0.8264f,-0.5551f,  0.0428f,-0.8202f,-0.5705f,  
	0.0635f,-0.7388f,-0.6709f,  0.0150f,-0.7289f,-0.6845f,  0.0369f,-0.6342f,-0.7723f,  -0.0071f,-0.6210f,-0.7838f,  0.0173f,-0.5152f,-0.8569f,  -0.0212f,-0.4994f,-0.8661f,  0.0067f,-0.3848f,-0.9230f,  -0.0255f,-0.3673f,-0.9297f,  0.0065f,-0.2467f,-0.9691f,  -0.0188f,-0.2281f,-0.9734f,  0.0178f,-0.1043f,-0.9944f,  -0.0004f,-0.0858f,-0.9963f,  0.0407f,0.0384f,-0.9984f,  0.0296f,0.0561f,-0.9980f,  0.0748f,0.1777f,-0.9812f,  
	0.0705f,0.1937f,-0.9785f,  0.1190f,0.3099f,-0.9433f,  0.1210f,0.3234f,-0.9385f,  0.1719f,0.4317f,-0.8855f,  0.1793f,0.4422f,-0.8788f,  0.2312f,0.5400f,-0.8093f,  0.2434f,0.5471f,-0.8009f,  0.2947f,0.6326f,-0.7163f,  0.3106f,0.6359f,-0.7065f,  0.3597f,0.7073f,-0.6086f,  0.3784f,0.7068f,-0.5977f,  0.4236f,0.7628f,-0.4886f,  0.4441f,0.7585f,-0.4769f,  0.4836f,0.7984f,-0.3589f,  0.5049f,0.7904f,-0.3468f,  
	0.5371f,0.8137f,-0.2222f,  0.5585f,0.8025f,-0.2100f,  0.5819f,0.8092f,-0.0815f,  0.6025f,0.7951f,-0.0695f,  0.6158f,0.7856f,0.0602f,  0.6350f,0.7692f,0.0717f,  0.6372f,0.7443f,0.2000f,  0.6544f,0.7262f,0.2108f,  0.6447f,0.6870f,0.3352f,  0.6597f,0.6677f,0.3449f,  0.6377f,0.6157f,0.4629f,  0.6501f,0.5959f,0.4715f,  0.6157f,0.5327f,0.5807f,  0.6255f,0.5129f,0.5880f,  0.5788f,0.4404f,0.6863f,  
	0.5860f,0.4211f,0.6924f,  0.5276f,0.3412f,0.7779f,  0.5323f,0.3230f,0.7825f,  0.4632f,0.2378f,0.8538f,  0.4654f,0.2210f,0.8570f,  0.3867f,0.1327f,0.9126f,  0.3868f,0.1176f,0.9146f,  0.2997f,0.0281f,0.9536f,  0.2980f,0.0150f,0.9544f,  0.2043f,-0.0736f,0.9761f,  0.2011f,-0.0845f,0.9759f,  0.1025f,-0.1704f,0.9800f,  0.0980f,-0.1790f,0.9789f,  -0.0035f,-0.2604f,0.9655f,  -0.0089f,-0.2669f,0.9637f,  
	-0.1114f,-0.3422f,0.9330f,  -0.1175f,-0.3465f,0.9307f,  -0.2189f,-0.4142f,0.8834f,  -0.2254f,-0.4165f,0.8807f,  -0.3238f,-0.4756f,0.8179f,  -0.3305f,-0.4760f,0.8150f,  -0.4238f,-0.5254f,0.7378f,  -0.4304f,-0.5242f,0.7348f,  -0.5169f,-0.5632f,0.6447f,  -0.5234f,-0.5606f,0.6418f,  -0.6012f,-0.5886f,0.5404f,  -0.6074f,-0.5848f,0.5377f,  -0.6750f,-0.6017f,0.4270f,  -0.6808f,-0.5969f,0.4246f,  -0.0425f,-0.6094f,0.7918f,  
	-0.0490f,-0.6146f,0.7873f,  0.0264f,-0.6958f,0.7178f,  0.0132f,-0.7040f,0.7101f,  0.0823f,-0.7759f,0.6255f,  0.0621f,-0.7861f,0.6149f,  0.1241f,-0.8469f,0.5170f,  0.0969f,-0.8583f,0.5040f,  0.1514f,-0.9061f,0.3950f,  0.1175f,-0.9175f,0.3799f,  0.1643f,-0.9509f,0.2623f,  0.1244f,-0.9613f,0.2457f,  0.1641f,-0.9788f,0.1222f,  0.1190f,-0.9874f,0.1046f,  0.1523f,-0.9881f,-0.0220f,  0.1032f,-0.9939f,-0.0399f,  
	0.1314f,-0.9772f,-0.1667f,  0.0798f,-0.9796f,-0.1845f,  0.1041f,-0.9455f,-0.3086f,  0.0515f,-0.9441f,-0.3257f,  0.0735f,-0.8929f,-0.4442f,  0.0215f,-0.8876f,-0.4602f,  0.0428f,-0.8202f,-0.5705f,  -0.0071f,-0.8111f,-0.5849f,  0.0150f,-0.7289f,-0.6845f,  -0.0314f,-0.7162f,-0.6972f,  -0.0071f,-0.6210f,-0.7838f,  -0.0488f,-0.6055f,-0.7944f,  -0.0212f,-0.4994f,-0.8661f,  -0.0572f,-0.4817f,-0.8745f,  -0.0255f,-0.3673f,-0.9297f,  
	-0.0551f,-0.3481f,-0.9358f,  -0.0188f,-0.2281f,-0.9734f,  -0.0415f,-0.2086f,-0.9771f,  -0.0004f,-0.0858f,-0.9963f,  -0.0162f,-0.0665f,-0.9977f,  0.0296f,0.0561f,-0.9980f,  0.0207f,0.0741f,-0.9970f,  0.0705f,0.1937f,-0.9785f,  0.0680f,0.2097f,-0.9754f,  0.1210f,0.3234f,-0.9385f,  0.1244f,0.3367f,-0.9333f,  0.1793f,0.4422f,-0.8788f,  0.1879f,0.4523f,-0.8719f,  0.2434f,0.5471f,-0.8009f,  0.2562f,0.5536f,-0.7924f,  
	0.3106f,0.6359f,-0.7065f,  0.3268f,0.6386f,-0.6967f,  0.3784f,0.7068f,-0.5977f,  0.3970f,0.7057f,-0.5869f,  0.4441f,0.7585f,-0.4769f,  0.4641f,0.7537f,-0.4654f,  0.5049f,0.7904f,-0.3468f,  0.5255f,0.7821f,-0.3349f,  0.5585f,0.8025f,-0.2100f,  0.5789f,0.7910f,-0.1980f,  0.6025f,0.7951f,-0.0695f,  0.6219f,0.7809f,-0.0579f,  0.6350f,0.7692f,0.0717f,  0.6529f,0.7529f,0.0828f,  0.6544f,0.7262f,0.2108f,  
	0.6704f,0.7083f,0.2211f,  0.6597f,0.6677f,0.3449f,  0.6734f,0.6488f,0.3542f,  0.6501f,0.5959f,0.4715f,  0.6614f,0.5766f,0.4797f,  0.6255f,0.5129f,0.5880f,  0.6342f,0.4937f,0.5950f,  0.5860f,0.4211f,0.6924f,  0.5923f,0.4026f,0.6979f,  0.5323f,0.3230f,0.7825f,  0.5362f,0.3056f,0.7868f,  0.4654f,0.2210f,0.8570f,  0.4672f,0.2051f,0.8600f,  0.3868f,0.1176f,0.9146f,  0.3866f,0.1034f,0.9164f,  
	0.2980f,0.0150f,0.9544f,  0.2962f,0.0028f,0.9551f,  0.2011f,-0.0845f,0.9759f,  0.1979f,-0.0946f,0.9757f,  0.0980f,-0.1790f,0.9789f,  0.0937f,-0.1870f,0.9779f,  -0.0089f,-0.2669f,0.9637f,  -0.0141f,-0.2728f,0.9620f,  -0.1175f,-0.3465f,0.9307f,  -0.1232f,-0.3503f,0.9285f,  -0.2254f,-0.4165f,0.8807f,  -0.2315f,-0.4185f,0.8782f,  -0.3305f,-0.4760f,0.8150f,  -0.3366f,-0.4762f,0.8123f,  -0.4304f,-0.5242f,0.7348f,  
	-0.4366f,-0.5229f,0.7321f,  -0.5234f,-0.5606f,0.6418f,  -0.5293f,-0.5579f,0.6392f,  -0.6074f,-0.5848f,0.5377f,  -0.6130f,-0.5811f,0.5353f,  -0.6808f,-0.5969f,0.4246f,  -0.6861f,-0.5923f,0.4225f,  -0.0490f,-0.6146f,0.7873f,  -0.0562f,-0.6200f,0.7826f,  0.0132f,-0.7040f,0.7101f,  -0.0010f,-0.7119f,0.7023f,  0.0621f,-0.7861f,0.6149f,  0.0408f,-0.7957f,0.6043f,  0.0969f,-0.8583f,0.5040f,  0.0686f,-0.8685f,0.4910f,  
	0.1175f,-0.9175f,0.3799f,  0.0825f,-0.9273f,0.3650f,  0.1244f,-0.9613f,0.2457f,  0.0836f,-0.9697f,0.2294f,  0.1190f,-0.9874f,0.1046f,  0.0733f,-0.9935f,0.0876f,  0.1032f,-0.9939f,-0.0399f,  0.0541f,-0.9969f,-0.0572f,  0.0798f,-0.9796f,-0.1845f,  0.0286f,-0.9791f,-0.2016f,  0.0515f,-0.9441f,-0.3257f,  -0.0001f,-0.9397f,-0.3420f,  0.0215f,-0.8876f,-0.4602f,  -0.0290f,-0.8793f,-0.4753f,  -0.0071f,-0.8111f,-0.5849f,  
	-0.0551f,-0.7991f,-0.5986f,  -0.0314f,-0.7162f,-0.6972f,  -0.0755f,-0.7011f,-0.7090f,  -0.0488f,-0.6055f,-0.7944f,  -0.0879f,-0.5878f,-0.8042f,  -0.0572f,-0.4817f,-0.8745f,  -0.0905f,-0.4622f,-0.8822f,  -0.0551f,-0.3481f,-0.9358f,  -0.0820f,-0.3278f,-0.9412f,  -0.0415f,-0.2086f,-0.9771f,  -0.0617f,-0.1880f,-0.9802f,  -0.0162f,-0.0665f,-0.9977f,  -0.0295f,-0.0467f,-0.9985f,  0.0207f,0.0741f,-0.9970f,  0.0138f,0.0923f,-0.9956f,  
	0.0680f,0.2097f,-0.9754f,  0.0673f,0.2255f,-0.9719f,  0.1244f,0.3367f,-0.9333f,  0.1291f,0.3497f,-0.9279f,  0.1879f,0.4523f,-0.8719f,  0.1974f,0.4619f,-0.8647f,  0.2562f,0.5536f,-0.7924f,  0.2695f,0.5596f,-0.7837f,  0.3268f,0.6386f,-0.6967f,  0.3431f,0.6408f,-0.6868f,  0.3970f,0.7057f,-0.5869f,  0.4153f,0.7040f,-0.5761f,  0.4641f,0.7537f,-0.4654f,  0.4836f,0.7483f,-0.4540f,  0.5255f,0.7821f,-0.3349f,  
	0.5453f,0.7734f,-0.3232f,  0.5789f,0.7910f,-0.1980f,  0.5982f,0.7793f,-0.1864f,  0.6219f,0.7809f,-0.0579f,  0.6403f,0.7668f,-0.0466f,  0.6529f,0.7529f,0.0828f,  0.6697f,0.7367f,0.0935f,  0.6704f,0.7083f,0.2211f,  0.6853f,0.6907f,0.2309f,  0.6734f,0.6488f,0.3542f,  0.6861f,0.6305f,0.3631f,  0.6614f,0.5766f,0.4797f,  0.6717f,0.5579f,0.4874f,  0.6342f,0.4937f,0.5950f,  0.6422f,0.4753f,0.6014f,  
	0.5923f,0.4026f,0.6979f,  0.5979f,0.3849f,0.7031f,  0.5362f,0.3056f,0.7868f,  0.5396f,0.2891f,0.7907f,  0.4672f,0.2051f,0.8600f,  0.4686f,0.1901f,0.8627f,  0.3866f,0.1034f,0.9164f,  0.3862f,0.0901f,0.9180f,  0.2962f,0.0028f,0.9551f,  0.2943f,-0.0085f,0.9557f,  0.1979f,-0.0946f,0.9757f,  0.1947f,-0.1040f,0.9753f,  0.0937f,-0.1870f,0.9779f,  0.0896f,-0.1944f,0.9768f,  -0.0141f,-0.2728f,0.9620f,  
	-0.0189f,-0.2781f,0.9604f,  -0.1232f,-0.3503f,0.9285f,  -0.1286f,-0.3537f,0.9265f,  -0.2315f,-0.4185f,0.8782f,  -0.2371f,-0.4201f,0.8760f,  -0.3366f,-0.4762f,0.8123f,  -0.3423f,-0.4763f,0.8099f,  -0.4366f,-0.5229f,0.7321f,  -0.4422f,-0.5215f,0.7297f,  -0.5293f,-0.5579f,0.6392f,  -0.5347f,-0.5553f,0.6369f,  -0.6130f,-0.5811f,0.5353f,  -0.6181f,-0.5775f,0.5333f,  -0.6861f,-0.5923f,0.4225f,  -0.6909f,-0.5880f,0.4207f,  
	-0.0562f,-0.6200f,0.7826f,  -0.0641f,-0.6255f,0.7776f,  -0.0010f,-0.7119f,0.7023f,  -0.0160f,-0.7195f,0.6943f,  0.0408f,-0.7957f,0.6043f,  0.0185f,-0.8046f,0.5936f,  0.0686f,-0.8685f,0.4910f,  0.0392f,-0.8775f,0.4780f,  0.0825f,-0.9273f,0.3650f,  0.0467f,-0.9355f,0.3503f,  0.0836f,-0.9697f,0.2294f,  0.0422f,-0.9760f,0.2135f,  0.0733f,-0.9935f,0.0876f,  0.0276f,-0.9971f,0.0710f,  0.0541f,-0.9969f,-0.0572f,  
	0.0053f,-0.9972f,-0.0740f,  0.0286f,-0.9791f,-0.2016f,  -0.0217f,-0.9757f,-0.2179f,  -0.0001f,-0.9397f,-0.3420f,  -0.0504f,-0.9325f,-0.3576f,  -0.0290f,-0.8793f,-0.4753f,  -0.0778f,-0.8684f,-0.4898f,  -0.0551f,-0.7991f,-0.5986f,  -0.1008f,-0.7848f,-0.6115f,  -0.0755f,-0.7011f,-0.7090f,  -0.1171f,-0.6839f,-0.7201f,  -0.0879f,-0.5878f,-0.8042f,  -0.1244f,-0.5683f,-0.8134f,  -0.0905f,-0.4622f,-0.8822f,  -0.1210f,-0.4413f,-0.8891f,  
	-0.0820f,-0.3278f,-0.9412f,  -0.1061f,-0.3062f,-0.9460f,  -0.0617f,-0.1880f,-0.9802f,  -0.0792f,-0.1668f,-0.9828f,  -0.0295f,-0.0467f,-0.9985f,  -0.0406f,-0.0266f,-0.9988f,  0.0138f,0.0923f,-0.9956f,  0.0090f,0.1105f,-0.9938f,  0.0673f,0.2255f,-0.9719f,  0.0682f,0.2412f,-0.9681f,  0.1291f,0.3497f,-0.9279f,  0.1351f,0.3622f,-0.9222f,  0.1974f,0.4619f,-0.8647f,  0.2077f,0.4710f,-0.8574f,  0.2695f,0.5596f,-0.7837f,  
	0.2833f,0.5649f,-0.7750f,  0.3431f,0.6408f,-0.6868f,  0.3594f,0.6423f,-0.6769f,  0.4153f,0.7040f,-0.5761f,  0.4334f,0.7018f,-0.5654f,  0.4836f,0.7483f,-0.4540f,  0.5025f,0.7426f,-0.4428f,  0.5453f,0.7734f,-0.3232f,  0.5643f,0.7644f,-0.3118f,  0.5982f,0.7793f,-0.1864f,  0.6167f,0.7675f,-0.1752f,  0.6403f,0.7668f,-0.0466f,  0.6575f,0.7526f,-0.0357f,  0.6697f,0.7367f,0.0935f,  0.6854f,0.7208f,0.1037f,  
	0.6853f,0.6907f,0.2309f,  0.6990f,0.6735f,0.2403f,  0.6861f,0.6305f,0.3631f,  0.6977f,0.6126f,0.3715f,  0.6717f,0.5579f,0.4874f,  0.6811f,0.5399f,0.4946f,  0.6422f,0.4753f,0.6014f,  0.6493f,0.4577f,0.6074f,  0.5979f,0.3849f,0.7031f,  0.6028f,0.3681f,0.7079f,  0.5396f,0.2891f,0.7907f,  0.5425f,0.2734f,0.7943f,  0.4686f,0.1901f,0.8627f,  0.4697f,0.1759f,0.8652f,  0.3862f,0.0901f,0.9180f,  
	0.3857f,0.0777f,0.9193f,  0.2943f,-0.0085f,0.9557f,  0.2924f,-0.0192f,0.9561f,  0.1947f,-0.1040f,0.9753f,  0.1917f,-0.1127f,0.9750f,  0.0896f,-0.1944f,0.9768f,  0.0857f,-0.2011f,0.9758f,  -0.0189f,-0.2781f,0.9604f,  -0.0235f,-0.2830f,0.9588f,  -0.1286f,-0.3537f,0.9265f,  -0.1335f,-0.3568f,0.9246f,  -0.2371f,-0.4201f,0.8760f,  -0.2422f,-0.4215f,0.8739f,  -0.3423f,-0.4763f,0.8099f,  -0.3475f,-0.4762f,0.8078f,  
	-0.4422f,-0.5215f,0.7297f,  -0.4473f,-0.5201f,0.7276f,  -0.5347f,-0.5553f,0.6369f,  -0.5396f,-0.5528f,0.6350f,  -0.6181f,-0.5775f,0.5333f,  -0.6228f,-0.5741f,0.5315f,  -0.6909f,-0.5880f,0.4207f,  -0.6952f,-0.5839f,0.4192f,  -0.0641f,-0.6255f,0.7776f,  -0.0726f,-0.6311f,0.7723f,  -0.0160f,-0.7195f,0.6943f,  -0.0320f,-0.7269f,0.6860f,  0.0185f,-0.8046f,0.5936f,  -0.0048f,-0.8126f,0.5828f,  0.0392f,-0.8775f,0.4780f,  
	0.0090f,-0.8852f,0.4651f,  0.0467f,-0.9355f,0.3503f,  0.0103f,-0.9419f,0.3358f,  0.0422f,-0.9760f,0.2135f,  0.0005f,-0.9802f,0.1979f,  0.0276f,-0.9971f,0.0710f,  -0.0181f,-0.9983f,0.0548f,  0.0053f,-0.9972f,-0.0740f,  -0.0429f,-0.9950f,-0.0901f,  -0.0217f,-0.9757f,-0.2179f,  -0.0709f,-0.9697f,-0.2337f,  -0.0504f,-0.9325f,-0.3576f,  -0.0991f,-0.9227f,-0.3725f,  -0.0778f,-0.8684f,-0.4898f,  -0.1244f,-0.8550f,-0.5035f,  
	-0.1008f,-0.7848f,-0.6115f,  -0.1442f,-0.7682f,-0.6237f,  -0.1171f,-0.6839f,-0.7201f,  -0.1561f,-0.6647f,-0.7306f,  -0.1244f,-0.5683f,-0.8134f,  -0.1580f,-0.5474f,-0.8218f,  -0.1210f,-0.4413f,-0.8891f,  -0.1488f,-0.4192f,-0.8956f,  -0.1061f,-0.3062f,-0.9460f,  -0.1276f,-0.2839f,-0.9503f,  -0.0792f,-0.1668f,-0.9828f,  -0.0943f,-0.1451f,-0.9849f,  -0.0406f,-0.0266f,-0.9988f,  -0.0494f,-0.0065f,-0.9988f,  0.0090f,0.1105f,-0.9938f,  
	0.0061f,0.1285f,-0.9917f,  0.0682f,0.2412f,-0.9681f,  0.0706f,0.2564f,-0.9640f,  0.1351f,0.3622f,-0.9222f,  0.1422f,0.3743f,-0.9163f,  0.2077f,0.4710f,-0.8574f,  0.2187f,0.4794f,-0.8499f,  0.2833f,0.5649f,-0.7750f,  0.2974f,0.5696f,-0.7662f,  0.3594f,0.6423f,-0.6769f,  0.3757f,0.6432f,-0.6671f,  0.4334f,0.7018f,-0.5654f,  0.4511f,0.6991f,-0.5549f,  0.5025f,0.7426f,-0.4428f,  0.5208f,0.7364f,-0.4319f,  
	0.5643f,0.7644f,-0.3118f,  0.5825f,0.7551f,-0.3008f,  0.6167f,0.7675f,-0.1752f,  0.6341f,0.7556f,-0.1643f,  0.6575f,0.7526f,-0.0357f,  0.6737f,0.7385f,-0.0253f,  0.6854f,0.7208f,0.1037f,  0.7000f,0.7051f,0.1135f,  0.6990f,0.6735f,0.2403f,  0.7117f,0.6568f,0.2492f,  0.6977f,0.6126f,0.3715f,  0.7083f,0.5953f,0.3793f,  0.6811f,0.5399f,0.4946f,  0.6896f,0.5226f,0.5013f,  0.6493f,0.4577f,0.6074f,  
	0.6557f,0.4408f,0.6130f,  0.6028f,0.3681f,0.7079f,  0.6072f,0.3521f,0.7123f,  0.5425f,0.2734f,0.7943f,  0.5450f,0.2586f,0.7975f,  0.4697f,0.1759f,0.8652f,  0.4705f,0.1626f,0.8673f,  0.3857f,0.0777f,0.9193f,  0.3851f,0.0660f,0.9205f,  0.2924f,-0.0192f,0.9561f,  0.2905f,-0.0290f,0.9564f,  0.1917f,-0.1127f,0.9750f,  0.1889f,-0.1207f,0.9746f,  0.0857f,-0.2011f,0.9758f,  0.0821f,-0.2073f,0.9748f,  
	-0.0235f,-0.2830f,0.9588f,  -0.0276f,-0.2873f,0.9574f,  -0.1335f,-0.3568f,0.9246f,  -0.1380f,-0.3595f,0.9229f,  -0.2422f,-0.4215f,0.8739f,  -0.2469f,-0.4227f,0.8720f,  -0.3475f,-0.4762f,0.8078f,  -0.3522f,-0.4759f,0.8059f,  -0.4473f,-0.5201f,0.7276f,  -0.4519f,-0.5187f,0.7258f,  -0.5396f,-0.5528f,0.6350f,  -0.5441f,-0.5504f,0.6333f,  -0.6228f,-0.5741f,0.5315f,  -0.6270f,-0.5709f,0.5301f,  -0.6952f,-0.5839f,0.4192f,  
	-0.6992f,-0.5800f,0.4180f,  -0.0726f,-0.6311f,0.7723f,  -0.0819f,-0.6368f,0.7667f,  -0.0320f,-0.7269f,0.6860f,  -0.0487f,-0.7338f,0.6776f,  -0.0048f,-0.8126f,0.5828f,  -0.0289f,-0.8198f,0.5719f,  0.0090f,-0.8852f,0.4651f,  -0.0219f,-0.8916f,0.4523f,  0.0103f,-0.9419f,0.3358f,  -0.0266f,-0.9465f,0.3215f,  0.0005f,-0.9802f,0.1979f,  -0.0412f,-0.9823f,0.1827f,  -0.0181f,-0.9983f,0.0548f,  -0.0633f,-0.9972f,0.0392f,  
	-0.0429f,-0.9950f,-0.0901f,  -0.0901f,-0.9903f,-0.1058f,  -0.0709f,-0.9697f,-0.2337f,  -0.1186f,-0.9612f,-0.2489f,  -0.0991f,-0.9227f,-0.3725f,  -0.1458f,-0.9105f,-0.3868f,  -0.1244f,-0.8550f,-0.5035f,  -0.1688f,-0.8394f,-0.5166f,  -0.1442f,-0.7682f,-0.6237f,  -0.1851f,-0.7497f,-0.6354f,  -0.1561f,-0.6647f,-0.7306f,  -0.1923f,-0.6440f,-0.7404f,  -0.1580f,-0.5474f,-0.8218f,  -0.1889f,-0.5251f,-0.8298f,  -0.1488f,-0.4192f,-0.8956f,  
	-0.1738f,-0.3963f,-0.9015f,  -0.1276f,-0.2839f,-0.9503f,  -0.1465f,-0.2611f,-0.9541f,  -0.0943f,-0.1451f,-0.9849f,  -0.1070f,-0.1233f,-0.9866f,  -0.0494f,-0.0065f,-0.9988f,  -0.0562f,0.0137f,-0.9983f,  0.0061f,0.1285f,-0.9917f,  0.0048f,0.1462f,-0.9892f,  0.0706f,0.2564f,-0.9640f,  0.0744f,0.2712f,-0.9596f,  0.1422f,0.3743f,-0.9163f,  0.1503f,0.3857f,-0.9103f,  0.2187f,0.4794f,-0.8499f,  0.2303f,0.4872f,-0.8424f,  
	0.2974f,0.5696f,-0.7662f,  0.3117f,0.5737f,-0.7575f,  0.3757f,0.6432f,-0.6671f,  0.3919f,0.6436f,-0.6574f,  0.4511f,0.6991f,-0.5549f,  0.4683f,0.6958f,-0.5446f,  0.5208f,0.7364f,-0.4319f,  0.5384f,0.7299f,-0.4212f,  0.5825f,0.7551f,-0.3008f,  0.5998f,0.7457f,-0.2901f,  0.6341f,0.7556f,-0.1643f,  0.6506f,0.7437f,-0.1539f,  0.6737f,0.7385f,-0.0253f,  0.6890f,0.7246f,-0.0154f,  0.7000f,0.7051f,0.1135f,  
	0.7136f,0.6898f,0.1227f,  0.7117f,0.6568f,0.2492f,  0.7235f,0.6405f,0.2575f,  0.7083f,0.5953f,0.3793f,  0.7181f,0.5786f,0.3866f,  0.6896f,0.5226f,0.5013f,  0.6973f,0.5060f,0.5076f,  0.6557f,0.4408f,0.6130f,  0.6615f,0.4248f,0.6181f,  0.6072f,0.3521f,0.7123f,  0.6111f,0.3369f,0.7163f,  0.5450f,0.2586f,0.7975f,  0.5472f,0.2446f,0.8004f,  0.4705f,0.1626f,0.8673f,  0.4711f,0.1500f,0.8692f,  
	0.3851f,0.0660f,0.9205f,  0.3844f,0.0551f,0.9215f,  0.2905f,-0.0290f,0.9564f,  0.2888f,-0.0382f,0.9566f,  0.1889f,-0.1207f,0.9746f,  0.1862f,-0.1282f,0.9741f,  0.0821f,-0.2073f,0.9748f,  0.0788f,-0.2130f,0.9739f,  -0.0276f,-0.2873f,0.9574f,  -0.0314f,-0.2913f,0.9561f,  -0.1380f,-0.3595f,0.9229f,  -0.1420f,-0.3619f,0.9213f,  -0.2469f,-0.4227f,0.8720f,  -0.2511f,-0.4236f,0.8703f,  -0.3522f,-0.4759f,0.8059f,  
	-0.3564f,-0.4756f,0.8042f,  -0.4519f,-0.5187f,0.7258f,  -0.4560f,-0.5173f,0.7242f,  -0.5441f,-0.5504f,0.6333f,  -0.5480f,-0.5481f,0.6319f,  -0.6270f,-0.5709f,0.5301f,  -0.6308f,-0.5678f,0.5289f,  -0.6992f,-0.5800f,0.4180f,  -0.7027f,-0.5764f,0.4171f,  -0.0819f,-0.6368f,0.7667f,  -0.0919f,-0.6425f,0.7607f,  -0.0487f,-0.7338f,0.6776f,  -0.0663f,-0.7404f,0.6689f,  -0.0289f,-0.8198f,0.5719f,  -0.0536f,-0.8261f,0.5610f,  
	-0.0219f,-0.8916f,0.4523f,  -0.0532f,-0.8966f,0.4395f,  -0.0266f,-0.9465f,0.3215f,  -0.0636f,-0.9494f,0.3074f,  -0.0412f,-0.9823f,0.1827f,  -0.0826f,-0.9824f,0.1678f,  -0.0633f,-0.9972f,0.0392f,  -0.1077f,-0.9939f,0.0239f,  -0.0901f,-0.9903f,-0.1058f,  -0.1361f,-0.9833f,-0.1210f,  -0.1186f,-0.9612f,-0.2489f,  -0.1647f,-0.9505f,-0.2635f,  -0.1458f,-0.9105f,-0.3868f,  -0.1905f,-0.8962f,-0.4006f,  -0.1688f,-0.8394f,-0.5166f,  
	-0.2109f,-0.8219f,-0.5292f,  -0.1851f,-0.7497f,-0.6354f,  -0.2233f,-0.7296f,-0.6463f,  -0.1923f,-0.6440f,-0.7404f,  -0.2258f,-0.6219f,-0.7498f,  -0.1889f,-0.5251f,-0.8298f,  -0.2171f,-0.5018f,-0.8373f,  -0.1738f,-0.3963f,-0.9015f,  -0.1962f,-0.3727f,-0.9070f,  -0.1465f,-0.2611f,-0.9541f,  -0.1629f,-0.2381f,-0.9575f,  -0.1070f,-0.1233f,-0.9866f,  -0.1175f,-0.1014f,-0.9879f,  -0.0562f,0.0137f,-0.9983f,  -0.0610f,0.0335f,-0.9976f,  
	0.0048f,0.1462f,-0.9892f,  0.0052f,0.1635f,-0.9865f,  0.0744f,0.2712f,-0.9596f,  0.0793f,0.2854f,-0.9551f,  0.1503f,0.3857f,-0.9103f,  0.1592f,0.3965f,-0.9041f,  0.2303f,0.4872f,-0.8424f,  0.2423f,0.4943f,-0.8348f,  0.3117f,0.5737f,-0.7575f,  0.3260f,0.5771f,-0.7488f,  0.3919f,0.6436f,-0.6574f,  0.4078f,0.6433f,-0.6479f,  0.4683f,0.6958f,-0.5446f,  0.4851f,0.6921f,-0.5345f,  0.5384f,0.7299f,-0.4212f,  
	0.5553f,0.7230f,-0.4109f,  0.5998f,0.7457f,-0.2901f,  0.6163f,0.7361f,-0.2799f,  0.6506f,0.7437f,-0.1539f,  0.6662f,0.7318f,-0.1439f,  0.6890f,0.7246f,-0.0154f,  0.7032f,0.7109f,-0.0059f,  0.7136f,0.6898f,0.1227f,  0.7262f,0.6748f,0.1314f,  0.7235f,0.6405f,0.2575f,  0.7343f,0.6248f,0.2654f,  0.7181f,0.5786f,0.3866f,  0.7271f,0.5626f,0.3935f,  0.6973f,0.5060f,0.5076f,  0.7044f,0.4902f,0.5133f,  
	0.6615f,0.4248f,0.6181f,  0.6667f,0.4095f,0.6227f,  0.6111f,0.3369f,0.7163f,  0.6146f,0.3226f,0.7199f,  0.5472f,0.2446f,0.8004f,  0.5492f,0.2315f,0.8030f,  0.4711f,0.1500f,0.8692f,  0.4717f,0.1383f,0.8709f,  0.3844f,0.0551f,0.9215f,  0.3838f,0.0449f,0.9223f,  0.2888f,-0.0382f,0.9566f,  0.2871f,-0.0468f,0.9568f,  0.1862f,-0.1282f,0.9741f,  0.1838f,-0.1350f,0.9736f,  0.0788f,-0.2130f,0.9739f,  
	0.0758f,-0.2182f,0.9730f,  -0.0314f,-0.2913f,0.9561f,  -0.0348f,-0.2950f,0.9549f,  -0.1420f,-0.3619f,0.9213f,  -0.1457f,-0.3641f,0.9199f,  -0.2511f,-0.4236f,0.8703f,  -0.2548f,-0.4245f,0.8688f,  -0.3564f,-0.4756f,0.8042f,  -0.3602f,-0.4753f,0.8027f,  -0.4560f,-0.5173f,0.7242f,  -0.4597f,-0.5159f,0.7228f,  -0.5480f,-0.5481f,0.6319f,  -0.5516f,-0.5459f,0.6307f,  -0.6308f,-0.5678f,0.5289f,  -0.6341f,-0.5650f,0.5279f,  
	-0.7027f,-0.5764f,0.4171f,  -0.7058f,-0.5731f,0.4164f,  -0.0919f,-0.6425f,0.7607f,  -0.1026f,-0.6483f,0.7545f,  -0.0663f,-0.7404f,0.6689f,  -0.0845f,-0.7464f,0.6601f,  -0.0536f,-0.8261f,0.5610f,  -0.0790f,-0.8314f,0.5500f,  -0.0532f,-0.8966f,0.4395f,  -0.0849f,-0.9003f,0.4268f,  -0.0636f,-0.9494f,0.3074f,  -0.1006f,-0.9506f,0.2935f,  -0.0826f,-0.9824f,0.1678f,  -0.1236f,-0.9804f,0.1532f,  -0.1077f,-0.9939f,0.0239f,  
	-0.1513f,-0.9885f,0.0090f,  -0.1361f,-0.9833f,-0.1210f,  -0.1807f,-0.9741f,-0.1357f,  -0.1647f,-0.9505f,-0.2635f,  -0.2089f,-0.9377f,-0.2777f,  -0.1905f,-0.8962f,-0.4006f,  -0.2331f,-0.8800f,-0.4139f,  -0.2109f,-0.8219f,-0.5292f,  -0.2504f,-0.8028f,-0.5412f,  -0.2233f,-0.7296f,-0.6463f,  -0.2588f,-0.7081f,-0.6570f,  -0.2258f,-0.6219f,-0.7498f,  -0.2566f,-0.5988f,-0.7587f,  -0.2171f,-0.5018f,-0.8373f,  -0.2425f,-0.4778f,-0.8443f,  
	-0.1962f,-0.3727f,-0.9070f,  -0.2159f,-0.3487f,-0.9120f,  -0.1629f,-0.2381f,-0.9575f,  -0.1768f,-0.2148f,-0.9605f,  -0.1175f,-0.1014f,-0.9879f,  -0.1259f,-0.0798f,-0.9888f,  -0.0610f,0.0335f,-0.9976f,  -0.0641f,0.0530f,-0.9965f,  0.0052f,0.1635f,-0.9865f,  0.0070f,0.1802f,-0.9836f,  0.0793f,0.2854f,-0.9551f,  0.0854f,0.2989f,-0.9505f,  0.1592f,0.3965f,-0.9041f,  0.1687f,0.4065f,-0.8979f,  0.2423f,0.4943f,-0.8348f,  
	0.2546f,0.5007f,-0.8273f,  0.3260f,0.5771f,-0.7488f,  0.3404f,0.5798f,-0.7402f,  0.4078f,0.6433f,-0.6479f,  0.4235f,0.6425f,-0.6386f,  0.4851f,0.6921f,-0.5345f,  0.5013f,0.6880f,-0.5247f,  0.5553f,0.7230f,-0.4109f,  0.5715f,0.7159f,-0.4010f,  0.6163f,0.7361f,-0.2799f,  0.6320f,0.7264f,-0.2700f,  0.6662f,0.7318f,-0.1439f,  0.6809f,0.7200f,-0.1344f,  0.7032f,0.7109f,-0.0059f,  0.7166f,0.6975f,0.0030f,  
	0.7262f,0.6748f,0.1314f,  0.7380f,0.6602f,0.1396f,  0.7343f,0.6248f,0.2654f,  0.7444f,0.6095f,0.2727f,  0.7271f,0.5626f,0.3935f,  0.7354f,0.5472f,0.3998f,  0.7044f,0.4902f,0.5133f,  0.7109f,0.4750f,0.5186f,  0.6667f,0.4095f,0.6227f,  0.6715f,0.3950f,0.6270f,  0.6146f,0.3226f,0.7199f,  0.6178f,0.3090f,0.7231f,  0.5492f,0.2315f,0.8030f,  0.5509f,0.2191f,0.8053f,  0.4717f,0.1383f,0.8709f,  
	0.4722f,0.1272f,0.8723f,  0.3838f,0.0449f,0.9223f,  0.3831f,0.0354f,0.9230f,  0.2871f,-0.0468f,0.9568f,  0.2857f,-0.0547f,0.9568f,  0.1838f,-0.1350f,0.9736f,  0.1816f,-0.1414f,0.9732f,  0.0758f,-0.2182f,0.9730f,  0.0731f,-0.2230f,0.9721f,  -0.0348f,-0.2950f,0.9549f,  -0.0378f,-0.2983f,0.9537f,  -0.1457f,-0.3641f,0.9199f,  -0.1489f,-0.3660f,0.9186f,  -0.2548f,-0.4245f,0.8688f,  -0.2582f,-0.4252f,0.8675f,  
	-0.3602f,-0.4753f,0.8027f,  -0.3635f,-0.4749f,0.8014f,  -0.4597f,-0.5159f,0.7228f,  -0.4630f,-0.5147f,0.7217f,  -0.5516f,-0.5459f,0.6307f,  -0.5547f,-0.5439f,0.6297f,  -0.6341f,-0.5650f,0.5279f,  -0.6371f,-0.5624f,0.5271f,  -0.7058f,-0.5731f,0.4164f,  -0.7086f,-0.5700f,0.4158f,  -0.1026f,-0.6483f,0.7545f,  -0.1140f,-0.6540f,0.7479f,  -0.0845f,-0.7464f,0.6601f,  -0.1035f,-0.7520f,0.6510f,  -0.0790f,-0.8314f,0.5500f,  
	-0.1049f,-0.8358f,0.5389f,  -0.0849f,-0.9003f,0.4268f,  -0.1168f,-0.9027f,0.4142f,  -0.1006f,-0.9506f,0.2935f,  -0.1373f,-0.9502f,0.2798f,  -0.1236f,-0.9804f,0.1532f,  -0.1638f,-0.9767f,0.1388f,  -0.1513f,-0.9885f,0.0090f,  -0.1936f,-0.9811f,-0.0056f,  -0.1807f,-0.9741f,-0.1357f,  -0.2237f,-0.9630f,-0.1501f,  -0.2089f,-0.9377f,-0.2777f,  -0.2512f,-0.9230f,-0.2916f,  -0.2331f,-0.8800f,-0.4139f,  -0.2732f,-0.8622f,-0.4267f,  
	-0.2504f,-0.8028f,-0.5412f,  -0.2874f,-0.7822f,-0.5528f,  -0.2588f,-0.7081f,-0.6570f,  -0.2917f,-0.6855f,-0.6671f,  -0.2566f,-0.5988f,-0.7587f,  -0.2846f,-0.5748f,-0.7672f,  -0.2425f,-0.4778f,-0.8443f,  -0.2652f,-0.4534f,-0.8509f,  -0.2159f,-0.3487f,-0.9120f,  -0.2331f,-0.3245f,-0.9167f,  -0.1768f,-0.2148f,-0.9605f,  -0.1886f,-0.1918f,-0.9632f,  -0.1259f,-0.0798f,-0.9888f,  -0.1322f,-0.0585f,-0.9895f,  -0.0641f,0.0530f,-0.9965f,  
	-0.0655f,0.0719f,-0.9953f,  0.0070f,0.1802f,-0.9836f,  0.0101f,0.1963f,-0.9805f,  0.0854f,0.2989f,-0.9505f,  0.0923f,0.3117f,-0.9457f,  0.1687f,0.4065f,-0.8979f,  0.1788f,0.4158f,-0.8917f,  0.2546f,0.5007f,-0.8273f,  0.2671f,0.5064f,-0.8199f,  0.3404f,0.5798f,-0.7402f,  0.3546f,0.5819f,-0.7319f,  0.4235f,0.6425f,-0.6386f,  0.4387f,0.6412f,-0.6296f,  0.5013f,0.6880f,-0.5247f,  0.5170f,0.6835f,-0.5154f,  
	0.5715f,0.7159f,-0.4010f,  0.5871f,0.7086f,-0.3915f,  0.6320f,0.7264f,-0.2700f,  0.6469f,0.7167f,-0.2606f,  0.6809f,0.7200f,-0.1344f,  0.6947f,0.7083f,-0.1255f,  0.7166f,0.6975f,0.0030f,  0.7291f,0.6843f,0.0114f,  0.7380f,0.6602f,0.1396f,  0.7490f,0.6460f,0.1472f,  0.7444f,0.6095f,0.2727f,  0.7537f,0.5949f,0.2794f,  0.7354f,0.5472f,0.3998f,  0.7430f,0.5325f,0.4056f,  0.7109f,0.4750f,0.5186f,  
	0.7169f,0.4606f,0.5234f,  0.6715f,0.3950f,0.6270f,  0.6758f,0.3812f,0.6308f,  0.6178f,0.3090f,0.7231f,  0.6207f,0.2962f,0.7260f,  0.5509f,0.2191f,0.8053f,  0.5525f,0.2074f,0.8073f,  0.4722f,0.1272f,0.8723f,  0.4726f,0.1169f,0.8735f,  0.3831f,0.0354f,0.9230f,  0.3826f,0.0265f,0.9235f,  0.2857f,-0.0547f,0.9568f,  0.2843f,-0.0621f,0.9567f,  0.1816f,-0.1414f,0.9732f,  0.1797f,-0.1472f,0.9726f,  
	0.0731f,-0.2230f,0.9721f,  0.0707f,-0.2274f,0.9712f,  -0.0378f,-0.2983f,0.9537f,  -0.0405f,-0.3013f,0.9527f,  -0.1489f,-0.3660f,0.9186f,  -0.1518f,-0.3678f,0.9174f,  -0.2582f,-0.4252f,0.8675f,  -0.2611f,-0.4258f,0.8663f,  -0.3635f,-0.4749f,0.8014f,  -0.3664f,-0.4746f,0.8003f,  -0.4630f,-0.5147f,0.7217f,  -0.4658f,-0.5135f,0.7207f,  -0.5547f,-0.5439f,0.6297f,  -0.5574f,-0.5420f,0.6289f,  -0.6371f,-0.5624f,0.5271f,  
	-0.6397f,-0.5600f,0.5266f,  -0.7086f,-0.5700f,0.4158f,  -0.7111f,-0.5672f,0.4155f,  -0.1140f,-0.6540f,0.7479f,  -0.1262f,-0.6596f,0.7409f,  -0.1035f,-0.7520f,0.6510f,  -0.1231f,-0.7570f,0.6418f,  -0.1049f,-0.8358f,0.5389f,  -0.1311f,-0.8392f,0.5278f,  -0.1168f,-0.9027f,0.4142f,  -0.1487f,-0.9037f,0.4016f,  -0.1373f,-0.9502f,0.2798f,  -0.1736f,-0.9482f,0.2662f,  -0.1638f,-0.9767f,0.1388f,  -0.2032f,-0.9712f,0.1246f,  
	-0.1936f,-0.9811f,-0.0056f,  -0.2346f,-0.9719f,-0.0199f,  -0.2237f,-0.9630f,-0.1501f,  -0.2649f,-0.9502f,-0.1642f,  -0.2512f,-0.9230f,-0.2916f,  -0.2912f,-0.9068f,-0.3049f,  -0.2732f,-0.8622f,-0.4267f,  -0.3110f,-0.8428f,-0.4392f,  -0.2874f,-0.7822f,-0.5528f,  -0.3218f,-0.7604f,-0.5641f,  -0.2917f,-0.6855f,-0.6671f,  -0.3219f,-0.6620f,-0.6769f,  -0.2846f,-0.5748f,-0.7672f,  -0.3100f,-0.5504f,-0.7752f,  -0.2652f,-0.4534f,-0.8509f,  
	-0.2855f,-0.4287f,-0.8572f,  -0.2331f,-0.3245f,-0.9167f,  -0.2480f,-0.3004f,-0.9210f,  -0.1886f,-0.1918f,-0.9632f,  -0.1982f,-0.1689f,-0.9655f,  -0.1322f,-0.0585f,-0.9895f,  -0.1368f,-0.0376f,-0.9899f,  -0.0655f,0.0719f,-0.9953f,  -0.0654f,0.0903f,-0.9938f,  0.0101f,0.1963f,-0.9805f,  0.0142f,0.2116f,-0.9772f,  0.0923f,0.3117f,-0.9457f,  0.0999f,0.3238f,-0.9408f,  0.1788f,0.4158f,-0.8917f,  0.1892f,0.4243f,-0.8855f,  
	0.2671f,0.5064f,-0.8199f,  0.2797f,0.5114f,-0.8126f,  0.3546f,0.5819f,-0.7319f,  0.3686f,0.5834f,-0.7237f,  0.4387f,0.6412f,-0.6296f,  0.4536f,0.6394f,-0.6209f,  0.5170f,0.6835f,-0.5154f,  0.5320f,0.6787f,-0.5063f,  0.5871f,0.7086f,-0.3915f,  0.6018f,0.7011f,-0.3824f,  0.6469f,0.7167f,-0.2606f,  0.6609f,0.7070f,-0.2518f,  0.6947f,0.7083f,-0.1255f,  0.7077f,0.6968f,-0.1170f,  0.7291f,0.6843f,0.0114f,  
	0.7407f,0.6715f,0.0192f,  0.7490f,0.6460f,0.1472f,  0.7592f,0.6324f,0.1542f,  0.7537f,0.5949f,0.2794f,  0.7623f,0.5808f,0.2856f,  0.7430f,0.5325f,0.4056f,  0.7500f,0.5184f,0.4108f,  0.7169f,0.4606f,0.5234f,  0.7223f,0.4469f,0.5277f,  0.6758f,0.3812f,0.6308f,  0.6799f,0.3682f,0.6342f,  0.6207f,0.2962f,0.7260f,  0.6234f,0.2841f,0.7285f,  0.5525f,0.2074f,0.8073f,  0.5539f,0.1965f,0.8090f,  
	0.4726f,0.1169f,0.8735f,  0.4730f,0.1073f,0.8745f,  0.3826f,0.0265f,0.9235f,  0.3822f,0.0183f,0.9239f,  0.2843f,-0.0621f,0.9567f,  0.2832f,-0.0689f,0.9566f,  0.1797f,-0.1472f,0.9726f,  0.1780f,-0.1526f,0.9721f,  0.0707f,-0.2274f,0.9712f,  0.0687f,-0.2315f,0.9704f,  -0.0405f,-0.3013f,0.9527f,  -0.0428f,-0.3041f,0.9517f,  -0.1518f,-0.3678f,0.9174f,  -0.1542f,-0.3694f,0.9164f,  -0.2611f,-0.4258f,0.8663f,  
	-0.2636f,-0.4264f,0.8653f,  -0.3664f,-0.4746f,0.8003f,  -0.3689f,-0.4743f,0.7994f,  -0.4658f,-0.5135f,0.7207f,  -0.4682f,-0.5124f,0.7199f,  -0.5574f,-0.5420f,0.6289f,  -0.5598f,-0.5403f,0.6283f,  -0.6397f,-0.5600f,0.5266f,  -0.6419f,-0.5578f,0.5261f,  -0.7111f,-0.5672f,0.4155f,  -0.7132f,-0.5647f,0.4153f,  -0.1262f,-0.6596f,0.7409f,  -0.1392f,-0.6652f,0.7336f,  -0.1231f,-0.7570f,0.6418f,  -0.1432f,-0.7614f,0.6323f,  
	-0.1311f,-0.8392f,0.5278f,  -0.1575f,-0.8417f,0.5165f,  -0.1487f,-0.9037f,0.4016f,  -0.1804f,-0.9034f,0.3890f,  -0.1736f,-0.9482f,0.2662f,  -0.2093f,-0.9447f,0.2527f,  -0.2032f,-0.9712f,0.1246f,  -0.2415f,-0.9641f,0.1106f,  -0.2346f,-0.9719f,-0.0199f,  -0.2742f,-0.9611f,-0.0340f,  -0.2649f,-0.9502f,-0.1642f,  -0.3043f,-0.9358f,-0.1779f,  -0.2912f,-0.9068f,-0.3049f,  -0.3292f,-0.8891f,-0.3181f,  -0.3110f,-0.8428f,-0.4392f,  
	-0.3463f,-0.8224f,-0.4514f,  -0.3218f,-0.7604f,-0.5641f,  -0.3536f,-0.7378f,-0.5750f,  -0.3219f,-0.6620f,-0.6769f,  -0.3495f,-0.6379f,-0.6863f,  -0.3100f,-0.5504f,-0.7752f,  -0.3329f,-0.5255f,-0.7830f,  -0.2855f,-0.4287f,-0.8572f,  -0.3033f,-0.4040f,-0.8630f,  -0.2480f,-0.3004f,-0.9210f,  -0.2607f,-0.2765f,-0.9250f,  -0.1982f,-0.1689f,-0.9655f,  -0.2059f,-0.1465f,-0.9676f,  -0.1368f,-0.0376f,-0.9899f,  -0.1398f,-0.0173f,-0.9900f,  
	-0.0654f,0.0903f,-0.9938f,  -0.0641f,0.1079f,-0.9921f,  0.0142f,0.2116f,-0.9772f,  0.0193f,0.2262f,-0.9739f,  0.0999f,0.3238f,-0.9408f,  0.1081f,0.3350f,-0.9360f,  0.1892f,0.4243f,-0.8855f,  0.1999f,0.4321f,-0.8794f,  0.2797f,0.5114f,-0.8126f,  0.2922f,0.5157f,-0.8054f,  0.3686f,0.5834f,-0.7237f,  0.3824f,0.5843f,-0.7158f,  0.4536f,0.6394f,-0.6209f,  0.4679f,0.6372f,-0.6124f,  0.5320f,0.6787f,-0.5063f,  
	0.5465f,0.6736f,-0.4976f,  0.6018f,0.7011f,-0.3824f,  0.6158f,0.6935f,-0.3738f,  0.6609f,0.7070f,-0.2518f,  0.6742f,0.6973f,-0.2434f,  0.7077f,0.6968f,-0.1170f,  0.7199f,0.6855f,-0.1091f,  0.7407f,0.6715f,0.0192f,  0.7516f,0.6590f,0.0265f,  0.7592f,0.6324f,0.1542f,  0.7686f,0.6192f,0.1608f,  0.7623f,0.5808f,0.2856f,  0.7703f,0.5673f,0.2913f,  0.7500f,0.5184f,0.4108f,  0.7565f,0.5049f,0.4156f,  
	0.7223f,0.4469f,0.5277f,  0.7274f,0.4339f,0.5316f,  0.6799f,0.3682f,0.6342f,  0.6836f,0.3558f,0.6373f,  0.6234f,0.2841f,0.7285f,  0.6258f,0.2727f,0.7308f,  0.5539f,0.1965f,0.8090f,  0.5554f,0.1862f,0.8105f,  0.4730f,0.1073f,0.8745f,  0.4735f,0.0983f,0.8753f,  0.3822f,0.0183f,0.9239f,  0.3819f,0.0106f,0.9242f,  0.2832f,-0.0689f,0.9566f,  0.2823f,-0.0753f,0.9564f,  0.1780f,-0.1526f,0.9721f,  
	0.1767f,-0.1577f,0.9716f,  0.0687f,-0.2315f,0.9704f,  0.0670f,-0.2352f,0.9696f,  -0.0428f,-0.3041f,0.9517f,  -0.0447f,-0.3067f,0.9508f,  -0.1542f,-0.3694f,0.9164f,  -0.1563f,-0.3709f,0.9154f,  -0.2636f,-0.4264f,0.8653f,  -0.2657f,-0.4270f,0.8643f,  -0.3689f,-0.4743f,0.7994f,  -0.3710f,-0.4740f,0.7985f,  -0.4682f,-0.5124f,0.7199f,  -0.4703f,-0.5115f,0.7192f,  -0.5598f,-0.5403f,0.6283f,  -0.5617f,-0.5388f,0.6278f,  
	-0.6419f,-0.5578f,0.5261f,  -0.6438f,-0.5559f,0.5259f,  -0.7132f,-0.5647f,0.4153f,  -0.7150f,-0.5625f,0.4153f,  -0.1392f,-0.6652f,0.7336f,  -0.1528f,-0.6706f,0.7259f,  -0.1432f,-0.7614f,0.6323f,  -0.1638f,-0.7652f,0.6226f,  -0.1575f,-0.8417f,0.5165f,  -0.1841f,-0.8432f,0.5051f,  -0.1804f,-0.9034f,0.3890f,  -0.2118f,-0.9019f,0.3764f,  -0.2093f,-0.9447f,0.2527f,  -0.2442f,-0.9397f,0.2392f,  -0.2415f,-0.9641f,0.1106f,  
	-0.2787f,-0.9555f,0.0967f,  -0.2742f,-0.9611f,-0.0340f,  -0.3121f,-0.9489f,-0.0478f,  -0.3043f,-0.9358f,-0.1779f,  -0.3417f,-0.9201f,-0.1915f,  -0.3292f,-0.8891f,-0.3181f,  -0.3649f,-0.8703f,-0.3309f,  -0.3463f,-0.8224f,-0.4514f,  -0.3793f,-0.8010f,-0.4632f,  -0.3536f,-0.7378f,-0.5750f,  -0.3830f,-0.7145f,-0.5855f,  -0.3495f,-0.6379f,-0.6863f,  -0.3746f,-0.6133f,-0.6953f,  -0.3329f,-0.5255f,-0.7830f,  -0.3533f,-0.5006f,-0.7903f,  
	-0.3033f,-0.4040f,-0.8630f,  -0.3189f,-0.3793f,-0.8686f,  -0.2607f,-0.2765f,-0.9250f,  -0.2714f,-0.2529f,-0.9287f,  -0.2059f,-0.1465f,-0.9676f,  -0.2118f,-0.1246f,-0.9693f,  -0.1398f,-0.0173f,-0.9900f,  -0.1413f,0.0023f,-0.9900f,  -0.0641f,0.1079f,-0.9921f,  -0.0617f,0.1248f,-0.9903f,  0.0193f,0.2262f,-0.9739f,  0.0251f,0.2400f,-0.9705f,  0.1081f,0.3350f,-0.9360f,  0.1168f,0.3455f,-0.9311f,  0.1999f,0.4321f,-0.8794f,  
	0.2108f,0.4392f,-0.8733f,  0.2922f,0.5157f,-0.8054f,  0.3046f,0.5193f,-0.7984f,  0.3824f,0.5843f,-0.7158f,  0.3958f,0.5848f,-0.7081f,  0.4679f,0.6372f,-0.6124f,  0.4818f,0.6346f,-0.6043f,  0.5465f,0.6736f,-0.4976f,  0.5602f,0.6683f,-0.4894f,  0.6158f,0.6935f,-0.3738f,  0.6291f,0.6859f,-0.3657f,  0.6742f,0.6973f,-0.2434f,  0.6867f,0.6878f,-0.2356f,  0.7199f,0.6855f,-0.1091f,  0.7313f,0.6745f,-0.1018f,  
	0.7516f,0.6590f,0.0265f,  0.7618f,0.6470f,0.0332f,  0.7686f,0.6192f,0.1608f,  0.7774f,0.6065f,0.1667f,  0.7703f,0.5673f,0.2913f,  0.7777f,0.5544f,0.2964f,  0.7565f,0.5049f,0.4156f,  0.7625f,0.4922f,0.4200f,  0.7274f,0.4339f,0.5316f,  0.7321f,0.4215f,0.5351f,  0.6836f,0.3558f,0.6373f,  0.6870f,0.3442f,0.6400f,  0.6258f,0.2727f,0.7308f,  0.6282f,0.2620f,0.7326f,  0.5554f,0.1862f,0.8105f,  
	0.5567f,0.1766f,0.8117f,  0.4735f,0.0983f,0.8753f,  0.4740f,0.0899f,0.8759f,  0.3819f,0.0106f,0.9242f,  0.3817f,0.0034f,0.9243f,  0.2823f,-0.0753f,0.9564f,  0.2816f,-0.0812f,0.9561f,  0.1767f,-0.1577f,0.9716f,  0.1756f,-0.1623f,0.9710f,  0.0670f,-0.2352f,0.9696f,  0.0656f,-0.2387f,0.9689f,  -0.0447f,-0.3067f,0.9508f,  -0.0463f,-0.3091f,0.9499f,  -0.1563f,-0.3709f,0.9154f,  -0.1580f,-0.3723f,0.9146f,  
	-0.2657f,-0.4270f,0.8643f,  -0.2675f,-0.4275f,0.8635f,  -0.3710f,-0.4740f,0.7985f,  -0.3728f,-0.4738f,0.7978f,  -0.4703f,-0.5115f,0.7192f,  -0.4720f,-0.5107f,0.7187f,  -0.5617f,-0.5388f,0.6278f,  -0.5634f,-0.5375f,0.6274f,  -0.6438f,-0.5559f,0.5259f,  -0.6454f,-0.5542f,0.5257f,  -0.7150f,-0.5625f,0.4153f,  -0.7165f,-0.5605f,0.4153f,  -0.1528f,-0.6706f,0.7259f,  -0.1672f,-0.6757f,0.7179f,  -0.1638f,-0.7652f,0.6226f,  
	-0.1848f,-0.7685f,0.6126f,  -0.1841f,-0.8432f,0.5051f,  -0.2107f,-0.8438f,0.4936f,  -0.2118f,-0.9019f,0.3764f,  -0.2428f,-0.8993f,0.3637f,  -0.2442f,-0.9397f,0.2392f,  -0.2784f,-0.9335f,0.2258f,  -0.2787f,-0.9555f,0.0967f,  -0.3145f,-0.9456f,0.0831f,  -0.3121f,-0.9489f,-0.0478f,  -0.3484f,-0.9353f,-0.0616f,  -0.3417f,-0.9201f,-0.1915f,  -0.3772f,-0.9032f,-0.2048f,  -0.3649f,-0.8703f,-0.3309f,  -0.3984f,-0.8505f,-0.3435f,  
	-0.3793f,-0.8010f,-0.4632f,  -0.4099f,-0.7789f,-0.4747f,  -0.3830f,-0.7145f,-0.5855f,  -0.4099f,-0.6906f,-0.5958f,  -0.3746f,-0.6133f,-0.6953f,  -0.3973f,-0.5886f,-0.7040f,  -0.3533f,-0.5006f,-0.7903f,  -0.3715f,-0.4756f,-0.7974f,  -0.3189f,-0.3793f,-0.8686f,  -0.3323f,-0.3549f,-0.8739f,  -0.2714f,-0.2529f,-0.9287f,  -0.2802f,-0.2297f,-0.9321f,  -0.2118f,-0.1246f,-0.9693f,  -0.2162f,-0.1033f,-0.9709f,  -0.1413f,0.0023f,-0.9900f,  
	-0.1416f,0.0212f,-0.9897f,  -0.0617f,0.1248f,-0.9903f,  -0.0583f,0.1408f,-0.9883f,  0.0251f,0.2400f,-0.9705f,  0.0316f,0.2529f,-0.9670f,  0.1168f,0.3455f,-0.9311f,  0.1258f,0.3551f,-0.9263f,  0.2108f,0.4392f,-0.8733f,  0.2217f,0.4454f,-0.8674f,  0.3046f,0.5193f,-0.7984f,  0.3169f,0.5224f,-0.7917f,  0.3958f,0.5848f,-0.7081f,  0.4088f,0.5847f,-0.7007f,  0.4818f,0.6346f,-0.6043f,  0.4950f,0.6316f,-0.5967f,  
	0.5602f,0.6683f,-0.4894f,  0.5733f,0.6628f,-0.4817f,  0.6291f,0.6859f,-0.3657f,  0.6417f,0.6783f,-0.3580f,  0.6867f,0.6878f,-0.2356f,  0.6984f,0.6783f,-0.2283f,  0.7313f,0.6745f,-0.1018f,  0.7419f,0.6637f,-0.0950f,  0.7618f,0.6470f,0.0332f,  0.7713f,0.6353f,0.0393f,  0.7774f,0.6065f,0.1667f,  0.7856f,0.5943f,0.1722f,  0.7777f,0.5544f,0.2964f,  0.7846f,0.5420f,0.3011f,  0.7625f,0.4922f,0.4200f,  
	0.7681f,0.4800f,0.4238f,  0.7321f,0.4215f,0.5351f,  0.7364f,0.4099f,0.5382f,  0.6870f,0.3442f,0.6400f,  0.6903f,0.3333f,0.6422f,  0.6282f,0.2620f,0.7326f,  0.6304f,0.2520f,0.7342f,  0.5567f,0.1766f,0.8117f,  0.5580f,0.1676f,0.8127f,  0.4740f,0.0899f,0.8759f,  0.4746f,0.0820f,0.8764f,  0.3817f,0.0034f,0.9243f,  0.3817f,-0.0033f,0.9243f,  0.2816f,-0.0812f,0.9561f,  0.2811f,-0.0866f,0.9557f,  
	0.1756f,-0.1623f,0.9710f,  0.1747f,-0.1667f,0.9704f,  0.0656f,-0.2387f,0.9689f,  0.0645f,-0.2420f,0.9681f,  -0.0463f,-0.3091f,0.9499f,  -0.0476f,-0.3113f,0.9491f,  -0.1580f,-0.3723f,0.9146f,  -0.1593f,-0.3737f,0.9138f,  -0.2675f,-0.4275f,0.8635f,  -0.2689f,-0.4281f,0.8628f,  -0.3728f,-0.4738f,0.7978f,  -0.3742f,-0.4737f,0.7972f,  -0.4720f,-0.5107f,0.7187f,  -0.4733f,-0.5100f,0.7182f,  -0.5634f,-0.5375f,0.6274f,  
	-0.5647f,-0.5365f,0.6271f,  -0.6454f,-0.5542f,0.5257f,  -0.6466f,-0.5528f,0.5256f,  -0.7165f,-0.5605f,0.4153f,  -0.7177f,-0.5588f,0.4154f,  -0.1672f,-0.6757f,0.7179f,  -0.1823f,-0.6807f,0.7095f,  -0.1848f,-0.7685f,0.6126f,  -0.2061f,-0.7712f,0.6024f,  -0.2107f,-0.8438f,0.4936f,  -0.2373f,-0.8435f,0.4819f,  -0.2428f,-0.8993f,0.3637f,  -0.2734f,-0.8956f,0.3509f,  -0.2784f,-0.9335f,0.2258f,  -0.3116f,-0.9262f,0.2125f,  
	-0.3145f,-0.9456f,0.0831f,  -0.3491f,-0.9345f,0.0694f,  -0.3484f,-0.9353f,-0.0616f,  -0.3830f,-0.9207f,-0.0751f,  -0.3772f,-0.9032f,-0.2048f,  -0.4107f,-0.8854f,-0.2179f,  -0.3984f,-0.8505f,-0.3435f,  -0.4297f,-0.8299f,-0.3558f,  -0.4099f,-0.7789f,-0.4747f,  -0.4382f,-0.7562f,-0.4860f,  -0.4099f,-0.6906f,-0.5958f,  -0.4345f,-0.6666f,-0.6057f,  -0.3973f,-0.5886f,-0.7040f,  -0.4178f,-0.5638f,-0.7124f,  -0.3715f,-0.4756f,-0.7974f,  
	-0.3875f,-0.4509f,-0.8041f,  -0.3323f,-0.3549f,-0.8739f,  -0.3438f,-0.3309f,-0.8788f,  -0.2802f,-0.2297f,-0.9321f,  -0.2873f,-0.2072f,-0.9352f,  -0.2162f,-0.1033f,-0.9709f,  -0.2191f,-0.0827f,-0.9722f,  -0.1416f,0.0212f,-0.9897f,  -0.1407f,0.0392f,-0.9893f,  -0.0583f,0.1408f,-0.9883f,  -0.0541f,0.1560f,-0.9863f,  0.0316f,0.2529f,-0.9670f,  0.0386f,0.2650f,-0.9635f,  0.1258f,0.3551f,-0.9263f,  0.1349f,0.3639f,-0.9216f,  
	0.2217f,0.4454f,-0.8674f,  0.2326f,0.4510f,-0.8617f,  0.3169f,0.5224f,-0.7917f,  0.3289f,0.5248f,-0.7851f,  0.4088f,0.5847f,-0.7007f,  0.4214f,0.5841f,-0.6937f,  0.4950f,0.6316f,-0.5967f,  0.5077f,0.6283f,-0.5894f,  0.5733f,0.6628f,-0.4817f,  0.5858f,0.6572f,-0.4743f,  0.6417f,0.6783f,-0.3580f,  0.6535f,0.6706f,-0.3509f,  0.6984f,0.6783f,-0.2283f,  0.7094f,0.6691f,-0.2215f,  0.7419f,0.6637f,-0.0950f,  
	0.7519f,0.6533f,-0.0888f,  0.7713f,0.6353f,0.0393f,  0.7801f,0.6241f,0.0450f,  0.7856f,0.5943f,0.1722f,  0.7932f,0.5826f,0.1771f,  0.7846f,0.5420f,0.3011f,  0.7909f,0.5303f,0.3053f,  0.7681f,0.4800f,0.4238f,  0.7733f,0.4685f,0.4273f,  0.7364f,0.4099f,0.5382f,  0.7405f,0.3989f,0.5409f,  0.6903f,0.3333f,0.6422f,  0.6933f,0.3230f,0.6442f,  0.6304f,0.2520f,0.7342f,  0.6325f,0.2425f,0.7356f,  
	0.5580f,0.1676f,0.8127f,  0.5594f,0.1592f,0.8134f,  0.4746f,0.0820f,0.8764f,  0.4753f,0.0747f,0.8766f,  0.3817f,-0.0033f,0.9243f,  0.3819f,-0.0095f,0.9242f,  0.2811f,-0.0866f,0.9557f,  0.2809f,-0.0918f,0.9553f,  0.1747f,-0.1667f,0.9704f,  0.1742f,-0.1707f,0.9698f,  0.0645f,-0.2420f,0.9681f,  0.0637f,-0.2450f,0.9674f,  -0.0476f,-0.3113f,0.9491f,  -0.0485f,-0.3135f,0.9484f,  -0.1593f,-0.3737f,0.9138f,  
	-0.1604f,-0.3750f,0.9130f,  -0.2689f,-0.4281f,0.8628f,  -0.2699f,-0.4287f,0.8622f,  -0.3742f,-0.4737f,0.7972f,  -0.3752f,-0.4737f,0.7967f,  -0.4733f,-0.5100f,0.7182f,  -0.4744f,-0.5095f,0.7179f,  -0.5647f,-0.5365f,0.6271f,  -0.5657f,-0.5356f,0.6270f,  -0.6466f,-0.5528f,0.5256f,  -0.6476f,-0.5516f,0.5256f,  -0.7177f,-0.5588f,0.4154f,  -0.7187f,-0.5574f,0.4155f,  -0.1823f,-0.6807f,0.7095f,  -0.1980f,-0.6854f,0.7008f,  
	-0.2061f,-0.7712f,0.6024f,  -0.2277f,-0.7732f,0.5919f,  -0.2373f,-0.8435f,0.4819f,  -0.2637f,-0.8423f,0.4701f,  -0.2734f,-0.8956f,0.3509f,  -0.3033f,-0.8908f,0.3383f,  -0.3116f,-0.9262f,0.2125f,  -0.3438f,-0.9177f,0.1991f,  -0.3491f,-0.9345f,0.0694f,  -0.3823f,-0.9224f,0.0558f,  -0.3830f,-0.9207f,-0.0751f,  -0.4159f,-0.9051f,-0.0886f,  -0.4107f,-0.8854f,-0.2179f,  -0.4422f,-0.8667f,-0.2308f,  -0.4297f,-0.8299f,-0.3558f,  
	-0.4589f,-0.8087f,-0.3679f,  -0.4382f,-0.7562f,-0.4860f,  -0.4643f,-0.7332f,-0.4969f,  -0.4345f,-0.6666f,-0.6057f,  -0.4569f,-0.6423f,-0.6153f,  -0.4178f,-0.5638f,-0.7124f,  -0.4361f,-0.5391f,-0.7205f,  -0.3875f,-0.4509f,-0.8041f,  -0.4015f,-0.4265f,-0.8105f,  -0.3438f,-0.3309f,-0.8788f,  -0.3535f,-0.3075f,-0.8835f,  -0.2873f,-0.2072f,-0.9352f,  -0.2928f,-0.1853f,-0.9380f,  -0.2191f,-0.0827f,-0.9722f,  -0.2208f,-0.0630f,-0.9733f,  
	-0.1407f,0.0392f,-0.9893f,  -0.1389f,0.0564f,-0.9887f,  -0.0541f,0.1560f,-0.9863f,  -0.0493f,0.1703f,-0.9842f,  0.0386f,0.2650f,-0.9635f,  0.0459f,0.2761f,-0.9600f,  0.1349f,0.3639f,-0.9216f,  0.1442f,0.3720f,-0.9170f,  0.2326f,0.4510f,-0.8617f,  0.2433f,0.4559f,-0.8561f,  0.3289f,0.5248f,-0.7851f,  0.3405f,0.5266f,-0.7789f,  0.4214f,0.5841f,-0.6937f,  0.4334f,0.5831f,-0.6871f,  0.5077f,0.6283f,-0.5894f,  
	0.5198f,0.6248f,-0.5825f,  0.5858f,0.6572f,-0.4743f,  0.5975f,0.6515f,-0.4675f,  0.6535f,0.6706f,-0.3509f,  0.6647f,0.6631f,-0.3443f,  0.7094f,0.6691f,-0.2215f,  0.7197f,0.6601f,-0.2153f,  0.7519f,0.6533f,-0.0888f,  0.7612f,0.6432f,-0.0830f,  0.7801f,0.6241f,0.0450f,  0.7883f,0.6133f,0.0501f,  0.7932f,0.5826f,0.1771f,  0.8003f,0.5715f,0.1815f,  0.7909f,0.5303f,0.3053f,  0.7969f,0.5191f,0.3090f,  
	0.7733f,0.4685f,0.4273f,  0.7781f,0.4576f,0.4302f,  0.7405f,0.3989f,0.5409f,  0.7443f,0.3885f,0.5432f,  0.6933f,0.3230f,0.6442f,  0.6962f,0.3133f,0.6459f,  0.6325f,0.2425f,0.7356f,  0.6346f,0.2337f,0.7366f,  0.5594f,0.1592f,0.8134f,  0.5608f,0.1514f,0.8140f,  0.4753f,0.0747f,0.8766f,  0.4761f,0.0679f,0.8767f,  0.3819f,-0.0095f,0.9242f,  0.3822f,-0.0153f,0.9239f,  0.2809f,-0.0918f,0.9553f,  
	0.2808f,-0.0965f,0.9549f,  0.1742f,-0.1707f,0.9698f,  0.1738f,-0.1745f,0.9692f,  0.0637f,-0.2450f,0.9674f,  0.0632f,-0.2479f,0.9667f,  -0.0485f,-0.3135f,0.9484f,  -0.0491f,-0.3155f,0.9476f,  -0.1604f,-0.3750f,0.9130f,  -0.1611f,-0.3763f,0.9124f,  -0.2699f,-0.4287f,0.8622f,  -0.2707f,-0.4294f,0.8616f,  -0.3752f,-0.4737f,0.7967f,  -0.3760f,-0.4739f,0.7963f,  -0.4744f,-0.5095f,0.7179f,  -0.4751f,-0.5092f,0.7176f,  
	-0.5657f,-0.5356f,0.6270f,  -0.5665f,-0.5350f,0.6268f,  -0.6476f,-0.5516f,0.5256f,  -0.6484f,-0.5507f,0.5256f,  -0.7187f,-0.5574f,0.4155f,  -0.7195f,-0.5563f,0.4157f,  -0.1980f,-0.6854f,0.7008f,  -0.2143f,-0.6896f,0.6917f,  -0.2277f,-0.7732f,0.5919f,  -0.2496f,-0.7746f,0.5812f,  -0.2637f,-0.8423f,0.4701f,  -0.2899f,-0.8402f,0.4582f,  -0.3033f,-0.8908f,0.3383f,  -0.3327f,-0.8851f,0.3254f,  -0.3438f,-0.9177f,0.1991f,  
	-0.3750f,-0.9082f,0.1857f,  -0.3823f,-0.9224f,0.0558f,  -0.4140f,-0.9093f,0.0423f,  -0.4159f,-0.9051f,-0.0886f,  -0.4471f,-0.8887f,-0.1018f,  -0.4422f,-0.8667f,-0.2308f,  -0.4718f,-0.8474f,-0.2435f,  -0.4589f,-0.8087f,-0.3679f,  -0.4860f,-0.7872f,-0.3797f,  -0.4643f,-0.7332f,-0.4969f,  -0.4882f,-0.7099f,-0.5076f,  -0.4569f,-0.6423f,-0.6153f,  -0.4772f,-0.6182f,-0.6246f,  -0.4361f,-0.5391f,-0.7205f,  -0.4524f,-0.5147f,-0.7283f,  
	-0.4015f,-0.4265f,-0.8105f,  -0.4137f,-0.4025f,-0.8166f,  -0.3535f,-0.3075f,-0.8835f,  -0.3616f,-0.2846f,-0.8878f,  -0.2928f,-0.1853f,-0.9380f,  -0.2970f,-0.1642f,-0.9406f,  -0.2208f,-0.0630f,-0.9733f,  -0.2214f,-0.0441f,-0.9742f,  -0.1389f,0.0564f,-0.9887f,  -0.1363f,0.0727f,-0.9880f,  -0.0493f,0.1703f,-0.9842f,  -0.0440f,0.1837f,-0.9820f,  0.0459f,0.2761f,-0.9600f,  0.0534f,0.2865f,-0.9566f,  0.1442f,0.3720f,-0.9170f,  
	0.1535f,0.3792f,-0.9125f,  0.2433f,0.4559f,-0.8561f,  0.2538f,0.4601f,-0.8508f,  0.3405f,0.5266f,-0.7789f,  0.3517f,0.5280f,-0.7730f,  0.4334f,0.5831f,-0.6871f,  0.4450f,0.5818f,-0.6808f,  0.5198f,0.6248f,-0.5825f,  0.5313f,0.6211f,-0.5761f,  0.5975f,0.6515f,-0.4675f,  0.6086f,0.6457f,-0.4612f,  0.6647f,0.6631f,-0.3443f,  0.6751f,0.6556f,-0.3382f,  0.7197f,0.6601f,-0.2153f,  0.7293f,0.6513f,-0.2096f,  
	0.7612f,0.6432f,-0.0830f,  0.7699f,0.6334f,-0.0778f,  0.7883f,0.6133f,0.0501f,  0.7959f,0.6029f,0.0547f,  0.8003f,0.5715f,0.1815f,  0.8068f,0.5609f,0.1855f,  0.7969f,0.5191f,0.3090f,  0.8024f,0.5085f,0.3123f,  0.7781f,0.4576f,0.4302f,  0.7826f,0.4473f,0.4329f,  0.7443f,0.3885f,0.5432f,  0.7479f,0.3787f,0.5451f,  0.6962f,0.3133f,0.6459f,  0.6990f,0.3042f,0.6472f,  0.6346f,0.2337f,0.7366f,  
	0.6367f,0.2255f,0.7374f,  0.5608f,0.1514f,0.8140f,  0.5622f,0.1440f,0.8143f,  0.4761f,0.0679f,0.8767f,  0.4770f,0.0615f,0.8767f,  0.3822f,-0.0153f,0.9239f,  0.3827f,-0.0207f,0.9236f,  0.2808f,-0.0965f,0.9549f,  0.2810f,-0.1010f,0.9544f,  0.1738f,-0.1745f,0.9692f,  0.1738f,-0.1781f,0.9685f,  0.0632f,-0.2479f,0.9667f,  0.0630f,-0.2506f,0.9660f,  -0.0491f,-0.3155f,0.9476f,  -0.0495f,-0.3175f,0.9470f,  
	-0.1611f,-0.3763f,0.9124f,  -0.1615f,-0.3777f,0.9118f,  -0.2707f,-0.4294f,0.8616f,  -0.2711f,-0.4301f,0.8611f,  -0.3760f,-0.4739f,0.7963f,  -0.3764f,-0.4742f,0.7959f,  -0.4751f,-0.5092f,0.7176f,  -0.4756f,-0.5092f,0.7173f,  -0.5665f,-0.5350f,0.6268f,  -0.5670f,-0.5346f,0.6267f,  -0.6484f,-0.5507f,0.5256f,  -0.6489f,-0.5501f,0.5257f,  -0.7195f,-0.5563f,0.4157f,  -0.7200f,-0.5555f,0.4159f,  -0.2143f,-0.6896f,0.6917f,  
	-0.2312f,-0.6936f,0.6823f,  -0.2496f,-0.7746f,0.5812f,  -0.2715f,-0.7753f,0.5703f,  -0.2899f,-0.8402f,0.4582f,  -0.3158f,-0.8374f,0.4461f,  -0.3327f,-0.8851f,0.3254f,  -0.3612f,-0.8786f,0.3125f,  -0.3750f,-0.9082f,0.1857f,  -0.4050f,-0.8979f,0.1724f,  -0.4140f,-0.9093f,0.0423f,  -0.4443f,-0.8954f,0.0289f,  -0.4471f,-0.8887f,-0.1018f,  -0.4766f,-0.8716f,-0.1149f,  -0.4718f,-0.8474f,-0.2435f,  -0.4994f,-0.8277f,-0.2559f,  
	-0.4860f,-0.7872f,-0.3797f,  -0.5111f,-0.7653f,-0.3913f,  -0.4882f,-0.7099f,-0.5076f,  -0.5101f,-0.6866f,-0.5180f,  -0.4772f,-0.6182f,-0.6246f,  -0.4955f,-0.5941f,-0.6336f,  -0.4524f,-0.5147f,-0.7283f,  -0.4668f,-0.4907f,-0.7358f,  -0.4137f,-0.4025f,-0.8166f,  -0.4242f,-0.3791f,-0.8224f,  -0.3616f,-0.2846f,-0.8878f,  -0.3682f,-0.2626f,-0.8919f,  -0.2970f,-0.1642f,-0.9406f,  -0.3000f,-0.1439f,-0.9430f,  -0.2214f,-0.0441f,-0.9742f,  
	-0.2211f,-0.0261f,-0.9749f,  -0.1363f,0.0727f,-0.9880f,  -0.1331f,0.0880f,-0.9872f,  -0.0440f,0.1837f,-0.9820f,  -0.0383f,0.1962f,-0.9798f,  0.0534f,0.2865f,-0.9566f,  0.0611f,0.2961f,-0.9532f,  0.1535f,0.3792f,-0.9125f,  0.1627f,0.3858f,-0.9081f,  0.2538f,0.4601f,-0.8508f,  0.2640f,0.4638f,-0.8457f,  0.3517f,0.5280f,-0.7730f,  0.3625f,0.5289f,-0.7674f,  0.4450f,0.5818f,-0.6808f,  0.4560f,0.5802f,-0.6749f,  
	0.5313f,0.6211f,-0.5761f,  0.5422f,0.6173f,-0.5701f,  0.6086f,0.6457f,-0.4612f,  0.6191f,0.6399f,-0.4553f,  0.6751f,0.6556f,-0.3382f,  0.6849f,0.6483f,-0.3326f,  0.7293f,0.6513f,-0.2096f,  0.7383f,0.6428f,-0.2044f,  0.7699f,0.6334f,-0.0778f,  0.7780f,0.6240f,-0.0731f,  0.7959f,0.6029f,0.0547f,  0.8030f,0.5930f,0.0588f,  0.8068f,0.5609f,0.1855f,  0.8130f,0.5508f,0.1890f,  0.8024f,0.5085f,0.3123f,  
	0.8076f,0.4985f,0.3151f,  0.7826f,0.4473f,0.4329f,  0.7869f,0.4376f,0.4351f,  0.7479f,0.3787f,0.5451f,  0.7514f,0.3695f,0.5467f,  0.6990f,0.3042f,0.6472f,  0.7016f,0.2957f,0.6483f,  0.6367f,0.2255f,0.7374f,  0.6387f,0.2177f,0.7380f,  0.5622f,0.1440f,0.8143f,  0.5637f,0.1372f,0.8145f,  0.4770f,0.0615f,0.8767f,  0.4781f,0.0555f,0.8766f,  0.3827f,-0.0207f,0.9236f,  0.3834f,-0.0258f,0.9232f,  
	0.2810f,-0.1010f,0.9544f,  0.2814f,-0.1052f,0.9538f,  0.1738f,-0.1781f,0.9685f,  0.1740f,-0.1815f,0.9679f,  0.0630f,-0.2506f,0.9660f,  0.0630f,-0.2533f,0.9653f,  -0.0495f,-0.3175f,0.9470f,  -0.0495f,-0.3195f,0.9463f,  -0.1615f,-0.3777f,0.9118f,  -0.1616f,-0.3790f,0.9112f,  -0.2711f,-0.4301f,0.8611f,  -0.2713f,-0.4310f,0.8606f,  -0.3764f,-0.4742f,0.7959f,  -0.3766f,-0.4746f,0.7955f,  -0.4756f,-0.5092f,0.7173f,  
	-0.4758f,-0.5093f,0.7171f,  -0.5670f,-0.5346f,0.6267f,  -0.5672f,-0.5344f,0.6266f,  -0.6489f,-0.5501f,0.5257f,  -0.6492f,-0.5497f,0.5257f,  -0.7200f,-0.5555f,0.4159f,  -0.7204f,-0.5550f,0.4160f,  -0.2312f,-0.6936f,0.6823f,  -0.2485f,-0.6970f,0.6726f,  -0.2715f,-0.7753f,0.5703f,  -0.2936f,-0.7753f,0.5591f,  -0.3158f,-0.8374f,0.4461f,  -0.3414f,-0.8338f,0.4339f,  -0.3612f,-0.8786f,0.3125f,  -0.3890f,-0.8712f,0.2995f,  
	-0.4050f,-0.8979f,0.1724f,  -0.4339f,-0.8868f,0.1592f,  -0.4443f,-0.8954f,0.0289f,  -0.4731f,-0.8809f,0.0156f,  -0.4766f,-0.8716f,-0.1149f,  -0.5043f,-0.8540f,-0.1277f,  -0.4994f,-0.8277f,-0.2559f,  -0.5253f,-0.8076f,-0.2682f,  -0.5111f,-0.7653f,-0.3913f,  -0.5343f,-0.7433f,-0.4025f,  -0.5101f,-0.6866f,-0.5180f,  -0.5301f,-0.6634f,-0.5281f,  -0.4955f,-0.5941f,-0.6336f,  -0.5119f,-0.5704f,-0.6423f,  -0.4668f,-0.4907f,-0.7358f,  
	-0.4795f,-0.4671f,-0.7429f,  -0.4242f,-0.3791f,-0.8224f,  -0.4331f,-0.3564f,-0.8279f,  -0.3682f,-0.2626f,-0.8919f,  -0.3736f,-0.2412f,-0.8957f,  -0.3000f,-0.1439f,-0.9430f,  -0.3020f,-0.1245f,-0.9451f,  -0.2211f,-0.0261f,-0.9749f,  -0.2200f,-0.0091f,-0.9755f,  -0.1331f,0.0880f,-0.9872f,  -0.1294f,0.1025f,-0.9863f,  -0.0383f,0.1962f,-0.9798f,  -0.0323f,0.2079f,-0.9776f,  0.0611f,0.2961f,-0.9532f,  0.0689f,0.3048f,-0.9499f,  
	0.1627f,0.3858f,-0.9081f,  0.1717f,0.3916f,-0.9040f,  0.2640f,0.4638f,-0.8457f,  0.2739f,0.4669f,-0.8409f,  0.3625f,0.5289f,-0.7674f,  0.3729f,0.5293f,-0.7621f,  0.4560f,0.5802f,-0.6749f,  0.4664f,0.5783f,-0.6694f,  0.5422f,0.6173f,-0.5701f,  0.5524f,0.6133f,-0.5646f,  0.6191f,0.6399f,-0.4553f,  0.6289f,0.6342f,-0.4499f,  0.6849f,0.6483f,-0.3326f,  0.6941f,0.6411f,-0.3275f,  0.7383f,0.6428f,-0.2044f,  
	0.7467f,0.6345f,-0.1997f,  0.7780f,0.6240f,-0.0731f,  0.7855f,0.6151f,-0.0689f,  0.8030f,0.5930f,0.0588f,  0.8096f,0.5836f,0.0624f,  0.8130f,0.5508f,0.1890f,  0.8187f,0.5412f,0.1920f,  0.8076f,0.4985f,0.3151f,  0.8124f,0.4890f,0.3176f,  0.7869f,0.4376f,0.4351f,  0.7909f,0.4285f,0.4369f,  0.7514f,0.3695f,0.5467f,  0.7546f,0.3609f,0.5480f,  0.7016f,0.2957f,0.6483f,  0.7042f,0.2877f,0.6491f,  
	0.6387f,0.2177f,0.7380f,  0.6407f,0.2105f,0.7384f,  0.5637f,0.1372f,0.8145f,  0.5652f,0.1307f,0.8145f,  0.4781f,0.0555f,0.8766f,  0.4792f,0.0499f,0.8763f,  0.3834f,-0.0258f,0.9232f,  0.3842f,-0.0306f,0.9227f,  0.2814f,-0.1052f,0.9538f,  0.2820f,-0.1092f,0.9532f,  0.1740f,-0.1815f,0.9679f,  0.1744f,-0.1847f,0.9672f,  0.0630f,-0.2533f,0.9653f,  0.0633f,-0.2558f,0.9647f,  -0.0495f,-0.3195f,0.9463f,  
	-0.0493f,-0.3214f,0.9456f,  -0.1616f,-0.3790f,0.9112f,  -0.1615f,-0.3804f,0.9106f,  -0.2713f,-0.4310f,0.8606f,  -0.2712f,-0.4320f,0.8601f,  -0.3766f,-0.4746f,0.7955f,  -0.3766f,-0.4753f,0.7952f,  -0.4758f,-0.5093f,0.7171f,  -0.4758f,-0.5096f,0.7169f,  -0.5672f,-0.5344f,0.6266f,  -0.5672f,-0.5345f,0.6265f,  -0.6492f,-0.5497f,0.5257f,  -0.6493f,-0.5497f,0.5257f,  -0.7204f,-0.5550f,0.4160f,  -0.7205f,-0.5548f,0.4161f,  
	-0.2485f,-0.6970f,0.6726f,  -0.2664f,-0.7000f,0.6626f,  -0.2936f,-0.7753f,0.5591f,  -0.3156f,-0.7748f,0.5478f,  -0.3414f,-0.8338f,0.4339f,  -0.3665f,-0.8295f,0.4215f,  -0.3890f,-0.8712f,0.2995f,  -0.4160f,-0.8630f,0.2866f,  -0.4339f,-0.8868f,0.1592f,  -0.4616f,-0.8750f,0.1459f,  -0.4731f,-0.8809f,0.0156f,  -0.5005f,-0.8658f,0.0026f,  -0.5043f,-0.8540f,-0.1277f,  -0.5304f,-0.8360f,-0.1405f,  -0.5253f,-0.8076f,-0.2682f,  
	-0.5493f,-0.7873f,-0.2801f,  -0.5343f,-0.7433f,-0.4025f,  -0.5556f,-0.7213f,-0.4135f,  -0.5301f,-0.6634f,-0.5281f,  -0.5482f,-0.6404f,-0.5379f,  -0.5119f,-0.5704f,-0.6423f,  -0.5266f,-0.5471f,-0.6507f,  -0.4795f,-0.4671f,-0.7429f,  -0.4906f,-0.4442f,-0.7497f,  -0.4331f,-0.3564f,-0.8279f,  -0.4407f,-0.3344f,-0.8330f,  -0.3736f,-0.2412f,-0.8957f,  -0.3778f,-0.2208f,-0.8992f,  -0.3020f,-0.1245f,-0.9451f,  -0.3031f,-0.1061f,-0.9470f,  
	-0.2200f,-0.0091f,-0.9755f,  -0.2182f,0.0071f,-0.9759f,  -0.1294f,0.1025f,-0.9863f,  -0.1252f,0.1161f,-0.9853f,  -0.0323f,0.2079f,-0.9776f,  -0.0262f,0.2187f,-0.9755f,  0.0689f,0.3048f,-0.9499f,  0.0765f,0.3128f,-0.9467f,  0.1717f,0.3916f,-0.9040f,  0.1805f,0.3969f,-0.9000f,  0.2739f,0.4669f,-0.8409f,  0.2834f,0.4694f,-0.8363f,  0.3729f,0.5293f,-0.7621f,  0.3827f,0.5294f,-0.7571f,  0.4664f,0.5783f,-0.6694f,  
	0.4763f,0.5761f,-0.6642f,  0.5524f,0.6133f,-0.5646f,  0.5620f,0.6092f,-0.5595f,  0.6289f,0.6342f,-0.4499f,  0.6380f,0.6285f,-0.4449f,  0.6941f,0.6411f,-0.3275f,  0.7026f,0.6341f,-0.3228f,  0.7467f,0.6345f,-0.1997f,  0.7545f,0.6265f,-0.1954f,  0.7855f,0.6151f,-0.0689f,  0.7925f,0.6064f,-0.0652f,  0.8096f,0.5836f,0.0624f,  0.8158f,0.5746f,0.0656f,  0.8187f,0.5412f,0.1920f,  0.8240f,0.5321f,0.1946f,  
	0.8124f,0.4890f,0.3176f,  0.8169f,0.4801f,0.3196f,  0.7909f,0.4285f,0.4369f,  0.7947f,0.4199f,0.4384f,  0.7546f,0.3609f,0.5480f,  0.7577f,0.3528f,0.5490f,  0.7042f,0.2877f,0.6491f,  0.7067f,0.2802f,0.6496f,  0.6407f,0.2105f,0.7384f,  0.6427f,0.2037f,0.7385f,  0.5652f,0.1307f,0.8145f,  0.5668f,0.1247f,0.8144f,  0.4792f,0.0499f,0.8763f,  0.4804f,0.0446f,0.8759f,  0.3842f,-0.0306f,0.9227f,  
	0.3852f,-0.0351f,0.9222f,  0.2820f,-0.1092f,0.9532f,  0.2827f,-0.1130f,0.9525f,  0.1744f,-0.1847f,0.9672f,  0.1750f,-0.1878f,0.9665f,  0.0633f,-0.2558f,0.9647f,  0.0638f,-0.2583f,0.9640f,  -0.0493f,-0.3214f,0.9456f,  -0.0489f,-0.3234f,0.9450f,  -0.1615f,-0.3804f,0.9106f,  -0.1611f,-0.3820f,0.9100f,  -0.2712f,-0.4320f,0.8601f,  -0.2709f,-0.4331f,0.8597f,  -0.3766f,-0.4753f,0.7952f,  -0.3763f,-0.4761f,0.7948f,  
	-0.4758f,-0.5096f,0.7169f,  -0.4756f,-0.5102f,0.7166f,  -0.5672f,-0.5345f,0.6265f,  -0.5671f,-0.5349f,0.6264f,  -0.6493f,-0.5497f,0.5257f,  -0.6491f,-0.5499f,0.5256f,  -0.7205f,-0.5548f,0.4161f,  -0.7205f,-0.5548f,0.4161f,  -0.2664f,-0.7000f,0.6626f,  -0.2846f,-0.7025f,0.6523f,  -0.3156f,-0.7748f,0.5478f,  -0.3376f,-0.7736f,0.5363f,  -0.3665f,-0.8295f,0.4215f,  -0.3911f,-0.8244f,0.4092f,  -0.4160f,-0.8630f,0.2866f,  
	-0.4422f,-0.8542f,0.2737f,  -0.4616f,-0.8750f,0.1459f,  -0.4881f,-0.8626f,0.1329f,  -0.5005f,-0.8658f,0.0026f,  -0.5264f,-0.8502f,-0.0104f,  -0.5304f,-0.8360f,-0.1405f,  -0.5548f,-0.8178f,-0.1530f,  -0.5493f,-0.7873f,-0.2801f,  -0.5716f,-0.7669f,-0.2919f,  -0.5556f,-0.7213f,-0.4135f,  -0.5751f,-0.6995f,-0.4242f,  -0.5482f,-0.6404f,-0.5379f,  -0.5647f,-0.6178f,-0.5473f,  -0.5266f,-0.5471f,-0.6507f,  -0.5397f,-0.5244f,-0.6586f,  
	-0.4906f,-0.4442f,-0.7497f,  -0.5003f,-0.4219f,-0.7561f,  -0.4407f,-0.3344f,-0.8330f,  -0.4471f,-0.3132f,-0.8378f,  -0.3778f,-0.2208f,-0.8992f,  -0.3810f,-0.2012f,-0.9024f,  -0.3031f,-0.1061f,-0.9470f,  -0.3034f,-0.0884f,-0.9488f,  -0.2182f,0.0071f,-0.9759f,  -0.2160f,0.0224f,-0.9761f,  -0.1252f,0.1161f,-0.9853f,  -0.1208f,0.1287f,-0.9843f,  -0.0262f,0.2187f,-0.9755f,  -0.0200f,0.2286f,-0.9733f,  0.0765f,0.3128f,-0.9467f,  
	0.0841f,0.3201f,-0.9437f,  0.1805f,0.3969f,-0.9000f,  0.1890f,0.4015f,-0.8962f,  0.2834f,0.4694f,-0.8363f,  0.2924f,0.4715f,-0.8320f,  0.3827f,0.5294f,-0.7571f,  0.3920f,0.5292f,-0.7525f,  0.4763f,0.5761f,-0.6642f,  0.4856f,0.5738f,-0.6595f,  0.5620f,0.6092f,-0.5595f,  0.5710f,0.6051f,-0.5548f,  0.6380f,0.6285f,-0.4449f,  0.6466f,0.6229f,-0.4404f,  0.7026f,0.6341f,-0.3228f,  0.7106f,0.6273f,-0.3186f,  
	0.7545f,0.6265f,-0.1954f,  0.7617f,0.6189f,-0.1917f,  0.7925f,0.6064f,-0.0652f,  0.7989f,0.5982f,-0.0619f,  0.8158f,0.5746f,0.0656f,  0.8215f,0.5661f,0.0684f,  0.8240f,0.5321f,0.1946f,  0.8290f,0.5235f,0.1968f,  0.8169f,0.4801f,0.3196f,  0.8211f,0.4717f,0.3213f,  0.7947f,0.4199f,0.4384f,  0.7982f,0.4118f,0.4396f,  0.7577f,0.3528f,0.5490f,  0.7607f,0.3452f,0.5497f,  0.7067f,0.2802f,0.6496f,  
	0.7092f,0.2732f,0.6499f,  0.6427f,0.2037f,0.7385f,  0.6447f,0.1974f,0.7385f,  0.5668f,0.1247f,0.8144f,  0.5684f,0.1190f,0.8141f,  0.4804f,0.0446f,0.8759f,  0.4818f,0.0397f,0.8754f,  0.3852f,-0.0351f,0.9222f,  0.3863f,-0.0393f,0.9215f,  0.2827f,-0.1130f,0.9525f,  0.2837f,-0.1166f,0.9518f,  0.1750f,-0.1878f,0.9665f,  0.1758f,-0.1908f,0.9658f,  0.0638f,-0.2583f,0.9640f,  0.0645f,-0.2608f,0.9632f,  
	-0.0489f,-0.3234f,0.9450f,  -0.0483f,-0.3254f,0.9443f,  -0.1611f,-0.3820f,0.9100f,  -0.1605f,-0.3836f,0.9095f,  -0.2709f,-0.4331f,0.8597f,  -0.2703f,-0.4344f,0.8592f,  -0.3763f,-0.4761f,0.7948f,  -0.3758f,-0.4771f,0.7944f,  -0.4756f,-0.5102f,0.7166f,  -0.4751f,-0.5110f,0.7163f,  -0.5671f,-0.5349f,0.6264f,  -0.5667f,-0.5356f,0.6261f,  -0.6491f,-0.5499f,0.5256f,  -0.6489f,-0.5504f,0.5254f,  -0.7205f,-0.5548f,0.4161f,  
	-0.7203f,-0.5552f,0.4159f,  -0.2846f,-0.7025f,0.6523f,  -0.3030f,-0.7044f,0.6419f,  -0.3376f,-0.7736f,0.5363f,  -0.3594f,-0.7717f,0.5247f,  -0.3911f,-0.8244f,0.4092f,  -0.4152f,-0.8187f,0.3968f,  -0.4422f,-0.8542f,0.2737f,  -0.4673f,-0.8447f,0.2609f,  -0.4881f,-0.8626f,0.1329f,  -0.5134f,-0.8497f,0.1199f,  -0.5264f,-0.8502f,-0.0104f,  -0.5509f,-0.8342f,-0.0232f,  -0.5548f,-0.8178f,-0.1530f,  -0.5777f,-0.7993f,-0.1652f,  
	-0.5716f,-0.7669f,-0.2919f,  -0.5922f,-0.7465f,-0.3032f,  -0.5751f,-0.6995f,-0.4242f,  -0.5931f,-0.6778f,-0.4345f,  -0.5647f,-0.6178f,-0.5473f,  -0.5795f,-0.5956f,-0.5563f,  -0.5397f,-0.5244f,-0.6586f,  -0.5514f,-0.5022f,-0.6662f,  -0.5003f,-0.4219f,-0.7561f,  -0.5088f,-0.4003f,-0.7622f,  -0.4471f,-0.3132f,-0.8378f,  -0.4524f,-0.2929f,-0.8423f,  -0.3810f,-0.2012f,-0.9024f,  -0.3833f,-0.1825f,-0.9054f,  -0.3034f,-0.0884f,-0.9488f,  
	-0.3030f,-0.0718f,-0.9503f,  -0.2160f,0.0224f,-0.9761f,  -0.2133f,0.0366f,-0.9763f,  -0.1208f,0.1287f,-0.9843f,  -0.1162f,0.1405f,-0.9832f,  -0.0200f,0.2286f,-0.9733f,  -0.0138f,0.2378f,-0.9712f,  0.0841f,0.3201f,-0.9437f,  0.0914f,0.3266f,-0.9407f,  0.1890f,0.4015f,-0.8962f,  0.1971f,0.4055f,-0.8926f,  0.2924f,0.4715f,-0.8320f,  0.3010f,0.4731f,-0.8280f,  0.3920f,0.5292f,-0.7525f,  0.4008f,0.5286f,-0.7483f,  
	0.4856f,0.5738f,-0.6595f,  0.4943f,0.5714f,-0.6552f,  0.5710f,0.6051f,-0.5548f,  0.5795f,0.6010f,-0.5505f,  0.6466f,0.6229f,-0.4404f,  0.6545f,0.6174f,-0.4364f,  0.7106f,0.6273f,-0.3186f,  0.7180f,0.6208f,-0.3149f,  0.7617f,0.6189f,-0.1917f,  0.7684f,0.6116f,-0.1884f,  0.7989f,0.5982f,-0.0619f,  0.8050f,0.5904f,-0.0591f,  0.8215f,0.5661f,0.0684f,  0.8268f,0.5580f,0.0707f,  0.8290f,0.5235f,0.1968f,  
	0.8336f,0.5155f,0.1986f,  0.8211f,0.4717f,0.3213f,  0.8251f,0.4638f,0.3226f,  0.7982f,0.4118f,0.4396f,  0.8016f,0.4042f,0.4405f,  0.7607f,0.3452f,0.5497f,  0.7635f,0.3381f,0.5502f,  0.7092f,0.2732f,0.6499f,  0.7116f,0.2667f,0.6500f,  0.6447f,0.1974f,0.7385f,  0.6467f,0.1914f,0.7383f,  0.5684f,0.1190f,0.8141f,  0.5701f,0.1137f,0.8136f,  0.4818f,0.0397f,0.8754f,  0.4832f,0.0350f,0.8748f,  
	0.3863f,-0.0393f,0.9215f,  0.3875f,-0.0433f,0.9208f,  0.2837f,-0.1166f,0.9518f,  0.2848f,-0.1200f,0.9510f,  0.1758f,-0.1908f,0.9658f,  0.1768f,-0.1937f,0.9650f,  0.0645f,-0.2608f,0.9632f,  0.0654f,-0.2633f,0.9625f,  -0.0483f,-0.3254f,0.9443f,  -0.0474f,-0.3275f,0.9437f,  -0.1605f,-0.3836f,0.9095f,  -0.1597f,-0.3853f,0.9089f,  -0.2703f,-0.4344f,0.8592f,  -0.2696f,-0.4359f,0.8587f,  -0.3758f,-0.4771f,0.7944f,  
	-0.3751f,-0.4784f,0.7940f,  -0.4751f,-0.5110f,0.7163f,  -0.4745f,-0.5121f,0.7159f,  -0.5667f,-0.5356f,0.6261f,  -0.5662f,-0.5365f,0.6258f,  -0.6489f,-0.5504f,0.5254f,  -0.6484f,-0.5511f,0.5252f,  -0.7203f,-0.5552f,0.4159f,  -0.7200f,-0.5558f,0.4157f,  -0.3030f,-0.7044f,0.6419f,  -0.3217f,-0.7057f,0.6313f,  -0.3594f,-0.7717f,0.5247f,  -0.3811f,-0.7692f,0.5130f,  -0.4152f,-0.8187f,0.3968f,  -0.4386f,-0.8123f,0.3844f,  
	-0.4673f,-0.8447f,0.2609f,  -0.4916f,-0.8347f,0.2481f,  -0.5134f,-0.8497f,0.1199f,  -0.5375f,-0.8364f,0.1070f,  -0.5509f,-0.8342f,-0.0232f,  -0.5740f,-0.8181f,-0.0357f,  -0.5777f,-0.7993f,-0.1652f,  -0.5991f,-0.7808f,-0.1771f,  -0.5922f,-0.7465f,-0.3032f,  -0.6113f,-0.7263f,-0.3143f,  -0.5931f,-0.6778f,-0.4345f,  -0.6095f,-0.6566f,-0.4444f,  -0.5795f,-0.5956f,-0.5563f,  -0.5930f,-0.5738f,-0.5649f,  -0.5514f,-0.5022f,-0.6662f,  
	-0.5617f,-0.4806f,-0.6734f,  -0.5088f,-0.4003f,-0.7622f,  -0.5161f,-0.3796f,-0.7678f,  -0.4524f,-0.2929f,-0.8423f,  -0.4567f,-0.2734f,-0.8465f,  -0.3833f,-0.1825f,-0.9054f,  -0.3849f,-0.1647f,-0.9081f,  -0.3030f,-0.0718f,-0.9503f,  -0.3022f,-0.0561f,-0.9516f,  -0.2133f,0.0366f,-0.9763f,  -0.2104f,0.0500f,-0.9763f,  -0.1162f,0.1405f,-0.9832f,  -0.1115f,0.1515f,-0.9821f,  -0.0138f,0.2378f,-0.9712f,  -0.0078f,0.2462f,-0.9692f,  
	0.0914f,0.3266f,-0.9407f,  0.0985f,0.3326f,-0.9379f,  0.1971f,0.4055f,-0.8926f,  0.2049f,0.4090f,-0.8892f,  0.3010f,0.4731f,-0.8280f,  0.3092f,0.4744f,-0.8242f,  0.4008f,0.5286f,-0.7483f,  0.4091f,0.5279f,-0.7443f,  0.4943f,0.5714f,-0.6552f,  0.5024f,0.5688f,-0.6512f,  0.5795f,0.6010f,-0.5505f,  0.5873f,0.5969f,-0.5466f,  0.6545f,0.6174f,-0.4364f,  0.6619f,0.6120f,-0.4328f,  0.7180f,0.6208f,-0.3149f,  
	0.7248f,0.6145f,-0.3116f,  0.7684f,0.6116f,-0.1884f,  0.7747f,0.6046f,-0.1855f,  0.8050f,0.5904f,-0.0591f,  0.8105f,0.5829f,-0.0567f,  0.8268f,0.5580f,0.0707f,  0.8317f,0.5504f,0.0726f,  0.8336f,0.5155f,0.1986f,  0.8379f,0.5078f,0.2001f,  0.8251f,0.4638f,0.3226f,  0.8289f,0.4564f,0.3236f,  0.8016f,0.4042f,0.4405f,  0.8048f,0.3971f,0.4410f,  0.7635f,0.3381f,0.5502f,  0.7663f,0.3314f,0.5504f,  
	0.7116f,0.2667f,0.6500f,  0.7139f,0.2605f,0.6499f,  0.6467f,0.1914f,0.7383f,  0.6487f,0.1859f,0.7380f,  0.5701f,0.1137f,0.8136f,  0.5719f,0.1088f,0.8131f,  0.4832f,0.0350f,0.8748f,  0.4848f,0.0306f,0.8741f,  0.3875f,-0.0433f,0.9208f,  0.3889f,-0.0472f,0.9201f,  0.2848f,-0.1200f,0.9510f,  0.2860f,-0.1234f,0.9502f,  0.1768f,-0.1937f,0.9650f,  0.1779f,-0.1966f,0.9642f,  0.0654f,-0.2633f,0.9625f,  
	0.0665f,-0.2657f,0.9617f,  -0.0474f,-0.3275f,0.9437f,  -0.0464f,-0.3296f,0.9430f,  -0.1597f,-0.3853f,0.9089f,  -0.1588f,-0.3872f,0.9082f,  -0.2696f,-0.4359f,0.8587f,  -0.2687f,-0.4375f,0.8581f,  -0.3751f,-0.4784f,0.7940f,  -0.3743f,-0.4799f,0.7935f,  -0.4745f,-0.5121f,0.7159f,  -0.4738f,-0.5134f,0.7155f,  -0.5662f,-0.5365f,0.6258f,  -0.5655f,-0.5377f,0.6254f,  -0.6484f,-0.5511f,0.5252f,  -0.6479f,-0.5522f,0.5247f,  
	-0.7200f,-0.5558f,0.4157f,  -0.7195f,-0.5567f,0.4152f,  -0.3217f,-0.7057f,0.6313f,  -0.3405f,-0.7065f,0.6205f,  -0.3811f,-0.7692f,0.5130f,  -0.4023f,-0.7660f,0.5013f,  -0.4386f,-0.8123f,0.3844f,  -0.4614f,-0.8054f,0.3720f,  -0.4916f,-0.8347f,0.2481f,  -0.5149f,-0.8243f,0.2354f,  -0.5375f,-0.8364f,0.1070f,  -0.5605f,-0.8228f,0.0944f,  -0.5740f,-0.8181f,-0.0357f,  -0.5958f,-0.8017f,-0.0479f,  -0.5991f,-0.7808f,-0.1771f,  
	-0.6191f,-0.7623f,-0.1887f,  -0.6113f,-0.7263f,-0.3143f,  -0.6289f,-0.7063f,-0.3249f,  -0.6095f,-0.6566f,-0.4444f,  -0.6244f,-0.6357f,-0.4539f,  -0.5930f,-0.5738f,-0.5649f,  -0.6051f,-0.5527f,-0.5731f,  -0.5617f,-0.4806f,-0.6734f,  -0.5709f,-0.4598f,-0.6802f,  -0.5161f,-0.3796f,-0.7678f,  -0.5223f,-0.3597f,-0.7732f,  -0.4567f,-0.2734f,-0.8465f,  -0.4603f,-0.2548f,-0.8504f,  -0.3849f,-0.1647f,-0.9081f,  -0.3860f,-0.1479f,-0.9106f,  
	-0.3022f,-0.0561f,-0.9516f,  -0.3010f,-0.0413f,-0.9527f,  -0.2104f,0.0500f,-0.9763f,  -0.2073f,0.0626f,-0.9763f,  -0.1115f,0.1515f,-0.9821f,  -0.1068f,0.1616f,-0.9811f,  -0.0078f,0.2462f,-0.9692f,  -0.0019f,0.2540f,-0.9672f,  0.0985f,0.3326f,-0.9379f,  0.1052f,0.3379f,-0.9353f,  0.2049f,0.4090f,-0.8892f,  0.2123f,0.4121f,-0.8861f,  0.3092f,0.4744f,-0.8242f,  0.3168f,0.4753f,-0.8208f,  0.4091f,0.5279f,-0.7443f,  
	0.4168f,0.5269f,-0.7407f,  0.5024f,0.5688f,-0.6512f,  0.5100f,0.5662f,-0.6476f,  0.5873f,0.5969f,-0.5466f,  0.5945f,0.5928f,-0.5432f,  0.6619f,0.6120f,-0.4328f,  0.6687f,0.6069f,-0.4296f,  0.7248f,0.6145f,-0.3116f,  0.7311f,0.6084f,-0.3088f,  0.7747f,0.6046f,-0.1855f,  0.7804f,0.5979f,-0.1830f,  0.8105f,0.5829f,-0.0567f,  0.8157f,0.5759f,-0.0546f,  0.8317f,0.5504f,0.0726f,  0.8363f,0.5432f,0.0742f,  
	0.8379f,0.5078f,0.2001f,  0.8419f,0.5007f,0.2013f,  0.8289f,0.4564f,0.3236f,  0.8324f,0.4494f,0.3244f,  0.8048f,0.3971f,0.4410f,  0.8079f,0.3905f,0.4414f,  0.7663f,0.3314f,0.5504f,  0.7689f,0.3252f,0.5504f,  0.7139f,0.2605f,0.6499f,  0.7162f,0.2548f,0.6497f,  0.6487f,0.1859f,0.7380f,  0.6508f,0.1806f,0.7375f,  0.5719f,0.1088f,0.8131f,  0.5737f,0.1041f,0.8124f,  0.4848f,0.0306f,0.8741f,  
	0.4864f,0.0264f,0.8733f,  0.3889f,-0.0472f,0.9201f,  0.3904f,-0.0509f,0.9192f,  0.2860f,-0.1234f,0.9502f,  0.2874f,-0.1266f,0.9494f,  0.1779f,-0.1966f,0.9642f,  0.1792f,-0.1995f,0.9634f,  0.0665f,-0.2657f,0.9617f,  0.0677f,-0.2683f,0.9610f,  -0.0464f,-0.3296f,0.9430f,  -0.0452f,-0.3319f,0.9422f,  -0.1588f,-0.3872f,0.9082f,  -0.1576f,-0.3892f,0.9076f,  -0.2687f,-0.4375f,0.8581f,  -0.2676f,-0.4394f,0.8575f,  
	-0.3743f,-0.4799f,0.7935f,  -0.3733f,-0.4816f,0.7929f,  -0.4738f,-0.5134f,0.7155f,  -0.4729f,-0.5150f,0.7149f,  -0.5655f,-0.5377f,0.6254f,  -0.5647f,-0.5392f,0.6248f,  -0.6479f,-0.5522f,0.5247f,  -0.6472f,-0.5536f,0.5242f,  -0.7195f,-0.5567f,0.4152f,  -0.7189f,-0.5579f,0.4146f,  -0.3405f,-0.7065f,0.6205f,  -0.3592f,-0.7066f,0.6096f,  -0.4023f,-0.7660f,0.5013f,  -0.4233f,-0.7623f,0.4896f,  -0.4614f,-0.8054f,0.3720f,  
	-0.4835f,-0.7980f,0.3597f,  -0.5149f,-0.8243f,0.2354f,  -0.5373f,-0.8134f,0.2229f,  -0.5605f,-0.8228f,0.0944f,  -0.5822f,-0.8089f,0.0821f,  -0.5958f,-0.8017f,-0.0479f,  -0.6162f,-0.7853f,-0.0599f,  -0.6191f,-0.7623f,-0.1887f,  -0.6376f,-0.7440f,-0.1999f,  -0.6289f,-0.7063f,-0.3249f,  -0.6452f,-0.6866f,-0.3352f,  -0.6244f,-0.6357f,-0.4539f,  -0.6381f,-0.6152f,-0.4630f,  -0.6051f,-0.5527f,-0.5731f,  -0.6159f,-0.5322f,-0.5809f,  
	-0.5709f,-0.4598f,-0.6802f,  -0.5790f,-0.4397f,-0.6866f,  -0.5223f,-0.3597f,-0.7732f,  -0.5277f,-0.3405f,-0.7782f,  -0.4603f,-0.2548f,-0.8504f,  -0.4631f,-0.2371f,-0.8540f,  -0.3860f,-0.1479f,-0.9106f,  -0.3865f,-0.1320f,-0.9128f,  -0.3010f,-0.0413f,-0.9527f,  -0.2995f,-0.0274f,-0.9537f,  -0.2073f,0.0626f,-0.9763f,  -0.2041f,0.0743f,-0.9761f,  -0.1068f,0.1616f,-0.9811f,  -0.1022f,0.1710f,-0.9800f,  -0.0019f,0.2540f,-0.9672f,  
	0.0038f,0.2610f,-0.9653f,  0.1052f,0.3379f,-0.9353f,  0.1117f,0.3427f,-0.9328f,  0.2123f,0.4121f,-0.8861f,  0.2192f,0.4147f,-0.8832f,  0.3168f,0.4753f,-0.8208f,  0.3240f,0.4760f,-0.8176f,  0.4168f,0.5269f,-0.7407f,  0.4240f,0.5258f,-0.7374f,  0.5100f,0.5662f,-0.6476f,  0.5170f,0.5635f,-0.6444f,  0.5945f,0.5928f,-0.5432f,  0.6012f,0.5889f,-0.5401f,  0.6687f,0.6069f,-0.4296f,  0.6750f,0.6019f,-0.4267f,  
	0.7311f,0.6084f,-0.3088f,  0.7369f,0.6026f,-0.3063f,  0.7804f,0.5979f,-0.1830f,  0.7857f,0.5916f,-0.1810f,  0.8157f,0.5759f,-0.0546f,  0.8204f,0.5693f,-0.0529f,  0.8363f,0.5432f,0.0742f,  0.8405f,0.5364f,0.0755f,  0.8419f,0.5007f,0.2013f,  0.8457f,0.4940f,0.2021f,  0.8324f,0.4494f,0.3244f,  0.8357f,0.4429f,0.3248f,  0.8079f,0.3905f,0.4414f,  0.8108f,0.3843f,0.4415f,  0.7689f,0.3252f,0.5504f,  
	0.7715f,0.3194f,0.5503f,  0.7162f,0.2548f,0.6497f,  0.7185f,0.2494f,0.6492f,  0.6508f,0.1806f,0.7375f,  0.6528f,0.1757f,0.7369f,  0.5737f,0.1041f,0.8124f,  0.5755f,0.0996f,0.8117f,  0.4864f,0.0264f,0.8733f,  0.4881f,0.0225f,0.8725f,  0.3904f,-0.0509f,0.9192f,  0.3920f,-0.0545f,0.9184f,  0.2874f,-0.1266f,0.9494f,  0.2889f,-0.1298f,0.9485f,  0.1792f,-0.1995f,0.9634f,  0.1807f,-0.2024f,0.9625f,  
	0.0677f,-0.2683f,0.9610f,  0.0691f,-0.2709f,0.9601f,  -0.0452f,-0.3319f,0.9422f,  -0.0439f,-0.3343f,0.9415f,  -0.1576f,-0.3892f,0.9076f,  -0.1564f,-0.3915f,0.9068f,  -0.2676f,-0.4394f,0.8575f,  -0.2664f,-0.4415f,0.8568f,  -0.3733f,-0.4816f,0.7929f,  -0.3722f,-0.4836f,0.7922f,  -0.4729f,-0.5150f,0.7149f,  -0.4718f,-0.5169f,0.7143f,  -0.5647f,-0.5392f,0.6248f,  -0.5638f,-0.5410f,0.6241f,  -0.6472f,-0.5536f,0.5242f,  
	-0.6464f,-0.5552f,0.5234f,  -0.7189f,-0.5579f,0.4146f,  -0.7183f,-0.5594f,0.4137f,  -0.3592f,-0.7066f,0.6096f,  -0.3780f,-0.7062f,0.5987f,  -0.4233f,-0.7623f,0.4896f,  -0.4439f,-0.7580f,0.4779f,  -0.4835f,-0.7980f,0.3597f,  -0.5049f,-0.7901f,0.3476f,  -0.5373f,-0.8134f,0.2229f,  -0.5587f,-0.8022f,0.2107f,  -0.5822f,-0.8089f,0.0821f,  -0.6028f,-0.7948f,0.0700f,  -0.6162f,-0.7853f,-0.0599f,  -0.6354f,-0.7689f,-0.0714f,  
	-0.6376f,-0.7440f,-0.1999f,  -0.6548f,-0.7258f,-0.2107f,  -0.6452f,-0.6866f,-0.3352f,  -0.6601f,-0.6673f,-0.3450f,  -0.6381f,-0.6152f,-0.4630f,  -0.6505f,-0.5954f,-0.4716f,  -0.6159f,-0.5322f,-0.5809f,  -0.6257f,-0.5123f,-0.5882f,  -0.5790f,-0.4397f,-0.6866f,  -0.5861f,-0.4204f,-0.6926f,  -0.5277f,-0.3405f,-0.7782f,  -0.5323f,-0.3223f,-0.7828f,  -0.4631f,-0.2371f,-0.8540f,  -0.4654f,-0.2204f,-0.8573f,  -0.3865f,-0.1320f,-0.9128f,  
	-0.3866f,-0.1169f,-0.9148f,  -0.2995f,-0.0274f,-0.9537f,  -0.2978f,-0.0144f,-0.9545f,  -0.2041f,0.0743f,-0.9761f,  -0.2008f,0.0851f,-0.9759f,  -0.1022f,0.1710f,-0.9800f,  -0.0977f,0.1796f,-0.9789f,  0.0038f,0.2610f,-0.9653f,  0.0092f,0.2674f,-0.9635f,  0.1117f,0.3427f,-0.9328f,  0.1178f,0.3469f,-0.9305f,  0.2192f,0.4147f,-0.8832f,  0.2257f,0.4169f,-0.8805f,  0.3240f,0.4760f,-0.8176f,  0.3307f,0.4764f,-0.8147f,  
	0.4240f,0.5258f,-0.7374f,  0.4306f,0.5245f,-0.7345f,  0.5170f,0.5635f,-0.6444f,  0.5235f,0.5608f,-0.6415f,  0.6012f,0.5889f,-0.5401f,  0.6074f,0.5850f,-0.5374f,  0.6750f,0.6019f,-0.4267f,  0.6808f,0.5971f,-0.4243f,  0.7369f,0.6026f,-0.3063f,  0.7422f,0.5971f,-0.3042f,  0.7857f,0.5916f,-0.1810f,  0.7905f,0.5856f,-0.1792f,  0.8204f,0.5693f,-0.0529f,  0.8248f,0.5630f,-0.0516f,  0.8405f,0.5364f,0.0755f,  
	0.8445f,0.5301f,0.0764f,  0.8457f,0.4940f,0.2021f,  0.8491f,0.4877f,0.2027f,  0.8357f,0.4429f,0.3248f,  0.8388f,0.4368f,0.3251f,  0.8108f,0.3843f,0.4415f,  0.8136f,0.3785f,0.4414f,  0.7715f,0.3194f,0.5503f,  0.7740f,0.3140f,0.5499f,  0.7185f,0.2494f,0.6492f,  0.7208f,0.2444f,0.6487f,  0.6528f,0.1757f,0.7369f,  0.6548f,0.1711f,0.7361f,  0.5755f,0.0996f,0.8117f,  0.5774f,0.0954f,0.8108f,  
	0.4881f,0.0225f,0.8725f,  0.4899f,0.0186f,0.8716f,  0.3920f,-0.0545f,0.9184f,  0.3937f,-0.0579f,0.9174f,  0.2889f,-0.1298f,0.9485f,  0.2905f,-0.1330f,0.9476f,  0.1807f,-0.2024f,0.9625f,  0.1822f,-0.2053f,0.9616f,  0.0691f,-0.2709f,0.9601f,  0.0706f,-0.2736f,0.9593f,  -0.0439f,-0.3343f,0.9415f,  -0.0424f,-0.3368f,0.9406f,  -0.1564f,-0.3915f,0.9068f,  -0.1550f,-0.3939f,0.9060f,  -0.2664f,-0.4415f,0.8568f,  
	-0.2651f,-0.4438f,0.8560f,  -0.3722f,-0.4836f,0.7922f,  -0.3709f,-0.4858f,0.7914f,  -0.4718f,-0.5169f,0.7143f,  -0.4707f,-0.5191f,0.7134f,  -0.5638f,-0.5410f,0.6241f,  -0.5627f,-0.5431f,0.6232f,  -0.6464f,-0.5552f,0.5234f,  -0.6455f,-0.5572f,0.5224f,  -0.7183f,-0.5594f,0.4137f,  -0.7175f,-0.5612f,0.4126f,  -0.3780f,-0.7062f,0.5987f,  -0.3966f,-0.7051f,0.5878f,  -0.4439f,-0.7580f,0.4779f,  -0.4639f,-0.7532f,0.4663f,  
	-0.5049f,-0.7901f,0.3476f,  -0.5256f,-0.7818f,0.3356f,  -0.5587f,-0.8022f,0.2107f,  -0.5791f,-0.7907f,0.1986f,  -0.6028f,-0.7948f,0.0700f,  -0.6222f,-0.7807f,0.0583f,  -0.6354f,-0.7689f,-0.0714f,  -0.6533f,-0.7526f,-0.0826f,  -0.6548f,-0.7258f,-0.2107f,  -0.6708f,-0.7079f,-0.2210f,  -0.6601f,-0.6673f,-0.3450f,  -0.6738f,-0.6484f,-0.3543f,  -0.6505f,-0.5954f,-0.4716f,  -0.6617f,-0.5761f,-0.4798f,  -0.6257f,-0.5123f,-0.5882f,  
	-0.6345f,-0.4932f,-0.5952f,  -0.5861f,-0.4204f,-0.6926f,  -0.5924f,-0.4020f,-0.6982f,  -0.5323f,-0.3223f,-0.7828f,  -0.5363f,-0.3050f,-0.7870f,  -0.4654f,-0.2204f,-0.8573f,  -0.4671f,-0.2044f,-0.8602f,  -0.3866f,-0.1169f,-0.9148f,  -0.3865f,-0.1028f,-0.9166f,  -0.2978f,-0.0144f,-0.9545f,  -0.2960f,-0.0022f,-0.9552f,  -0.2008f,0.0851f,-0.9759f,  -0.1976f,0.0952f,-0.9757f,  -0.0977f,0.1796f,-0.9789f,  -0.0934f,0.1876f,-0.9778f,  
	0.0092f,0.2674f,-0.9635f,  0.0144f,0.2732f,-0.9618f,  0.1178f,0.3469f,-0.9305f,  0.1235f,0.3507f,-0.9283f,  0.2257f,0.4169f,-0.8805f,  0.2317f,0.4189f,-0.8780f,  0.3307f,0.4764f,-0.8147f,  0.3368f,0.4766f,-0.8121f,  0.4306f,0.5245f,-0.7345f,  0.4367f,0.5232f,-0.7318f,  0.5235f,0.5608f,-0.6415f,  0.5294f,0.5582f,-0.6389f,  0.6074f,0.5850f,-0.5374f,  0.6130f,0.5813f,-0.5351f,  0.6808f,0.5971f,-0.4243f,  
	0.6861f,0.5925f,-0.4222f,  0.7422f,0.5971f,-0.3042f,  0.7471f,0.5919f,-0.3024f,  0.7905f,0.5856f,-0.1792f,  0.7950f,0.5800f,-0.1778f,  0.8248f,0.5630f,-0.0516f,  0.8289f,0.5572f,-0.0506f,  0.8445f,0.5301f,0.0764f,  0.8481f,0.5242f,0.0771f,  0.8491f,0.4877f,0.2027f,  0.8524f,0.4819f,0.2030f,  0.8388f,0.4368f,0.3251f,  0.8417f,0.4311f,0.3251f,  0.8136f,0.3785f,0.4414f,  0.8162f,0.3731f,0.4412f,  
	0.7740f,0.3140f,0.5499f,  0.7764f,0.3089f,0.5494f,  0.7208f,0.2444f,0.6487f,  0.7229f,0.2397f,0.6480f,  0.6548f,0.1711f,0.7361f,  0.6569f,0.1668f,0.7353f,  0.5774f,0.0954f,0.8108f,  0.5793f,0.0914f,0.8099f,  0.4899f,0.0186f,0.8716f,  0.4917f,0.0150f,0.8706f,  0.3937f,-0.0579f,0.9174f,  0.3954f,-0.0613f,0.9164f,  0.2905f,-0.1330f,0.9476f,  0.2922f,-0.1361f,0.9466f,  0.1822f,-0.2053f,0.9616f,  
	0.1839f,-0.2082f,0.9606f,  0.0706f,-0.2736f,0.9593f,  0.0722f,-0.2764f,0.9583f,  -0.0424f,-0.3368f,0.9406f,  -0.0409f,-0.3395f,0.9397f,  -0.1550f,-0.3939f,0.9060f,  -0.1535f,-0.3965f,0.9051f,  -0.2651f,-0.4438f,0.8560f,  -0.2637f,-0.4464f,0.8551f,  -0.3709f,-0.4858f,0.7914f,  -0.3696f,-0.4884f,0.7905f,  -0.4707f,-0.5191f,0.7134f,  -0.4695f,-0.5216f,0.7124f,  -0.5627f,-0.5431f,0.6232f,  -0.5617f,-0.5455f,0.6221f,  
	-0.6455f,-0.5572f,0.5224f,  -0.6445f,-0.5595f,0.5211f,  -0.7175f,-0.5612f,0.4126f,  -0.7167f,-0.5632f,0.4112f,  -0.3966f,-0.7051f,0.5878f,  -0.4150f,-0.7035f,0.5770f,  -0.4639f,-0.7532f,0.4663f,  -0.4835f,-0.7479f,0.4548f,  -0.5256f,-0.7818f,0.3356f,  -0.5454f,-0.7731f,0.3239f,  -0.5791f,-0.7907f,0.1986f,  -0.5985f,-0.7790f,0.1870f,  -0.6222f,-0.7807f,0.0583f,  -0.6406f,-0.7665f,0.0470f,  -0.6533f,-0.7526f,-0.0826f,  
	-0.6701f,-0.7364f,-0.0933f,  -0.6708f,-0.7079f,-0.2210f,  -0.6856f,-0.6904f,-0.2308f,  -0.6738f,-0.6484f,-0.3543f,  -0.6864f,-0.6300f,-0.3631f,  -0.6617f,-0.5761f,-0.4798f,  -0.6720f,-0.5574f,-0.4875f,  -0.6345f,-0.4932f,-0.5952f,  -0.6424f,-0.4748f,-0.6016f,  -0.5924f,-0.4020f,-0.6982f,  -0.5980f,-0.3844f,-0.7033f,  -0.5363f,-0.3050f,-0.7870f,  -0.5396f,-0.2885f,-0.7909f,  -0.4671f,-0.2044f,-0.8602f,  -0.4685f,-0.1894f,-0.8629f,  
	-0.3865f,-0.1028f,-0.9166f,  -0.3861f,-0.0895f,-0.9181f,  -0.2960f,-0.0022f,-0.9552f,  -0.2941f,0.0091f,-0.9557f,  -0.1976f,0.0952f,-0.9757f,  -0.1945f,0.1045f,-0.9753f,  -0.0934f,0.1876f,-0.9778f,  -0.0893f,0.1949f,-0.9768f,  0.0144f,0.2732f,-0.9618f,  0.0192f,0.2786f,-0.9602f,  0.1235f,0.3507f,-0.9283f,  0.1288f,0.3542f,-0.9263f,  0.2317f,0.4189f,-0.8780f,  0.2373f,0.4204f,-0.8757f,  0.3368f,0.4766f,-0.8121f,  
	0.3425f,0.4766f,-0.8097f,  0.4367f,0.5232f,-0.7318f,  0.4423f,0.5218f,-0.7295f,  0.5294f,0.5582f,-0.6389f,  0.5348f,0.5556f,-0.6367f,  0.6130f,0.5813f,-0.5351f,  0.6182f,0.5777f,-0.5330f,  0.6861f,0.5925f,-0.4222f,  0.6909f,0.5881f,-0.4205f,  0.7471f,0.5919f,-0.3024f,  0.7515f,0.5870f,-0.3010f,  0.7950f,0.5800f,-0.1778f,  0.7991f,0.5747f,-0.1767f,  0.8289f,0.5572f,-0.0506f,  0.8326f,0.5517f,-0.0498f,  
	0.8481f,0.5242f,0.0771f,  0.8515f,0.5187f,0.0775f,  0.8524f,0.4819f,0.2030f,  0.8554f,0.4764f,0.2031f,  0.8417f,0.4311f,0.3251f,  0.8444f,0.4259f,0.3249f,  0.8162f,0.3731f,0.4412f,  0.8187f,0.3681f,0.4407f,  0.7764f,0.3089f,0.5494f,  0.7787f,0.3041f,0.5488f,  0.7229f,0.2397f,0.6480f,  0.7251f,0.2352f,0.6472f,  0.6569f,0.1668f,0.7353f,  0.6589f,0.1626f,0.7344f,  0.5793f,0.0914f,0.8099f,  
	0.5813f,0.0876f,0.8090f,  0.4917f,0.0150f,0.8706f,  0.4936f,0.0114f,0.8696f,  0.3954f,-0.0613f,0.9164f,  0.3973f,-0.0646f,0.9154f,  0.2922f,-0.1361f,0.9466f,  0.2940f,-0.1393f,0.9456f,  0.1839f,-0.2082f,0.9606f,  0.1856f,-0.2112f,0.9596f,  0.0722f,-0.2764f,0.9583f,  0.0739f,-0.2793f,0.9573f,  -0.0409f,-0.3395f,0.9397f,  -0.0392f,-0.3424f,0.9387f,  -0.1535f,-0.3965f,0.9051f,  -0.1519f,-0.3994f,0.9041f,  
	-0.2637f,-0.4464f,0.8551f,  -0.2622f,-0.4493f,0.8541f,  -0.3696f,-0.4884f,0.7905f,  -0.3682f,-0.4912f,0.7894f,  -0.4695f,-0.5216f,0.7124f,  -0.4682f,-0.5244f,0.7112f,  -0.5617f,-0.5455f,0.6221f,  -0.5605f,-0.5482f,0.6208f,  -0.6445f,-0.5595f,0.5211f,  -0.6435f,-0.5620f,0.5196f,  -0.7167f,-0.5632f,0.4112f,  -0.7158f,-0.5656f,0.4095f,  -0.4150f,-0.7035f,0.5770f,  -0.4331f,-0.7013f,0.5662f,  -0.4835f,-0.7479f,0.4548f,  
	-0.5024f,-0.7422f,0.4436f,  -0.5454f,-0.7731f,0.3239f,  -0.5644f,-0.7641f,0.3125f,  -0.5985f,-0.7790f,0.1870f,  -0.6169f,-0.7672f,0.1757f,  -0.6406f,-0.7665f,0.0470f,  -0.6579f,-0.7523f,0.0361f,  -0.6701f,-0.7364f,-0.0933f,  -0.6857f,-0.7205f,-0.1035f,  -0.6856f,-0.6904f,-0.2308f,  -0.6994f,-0.6732f,-0.2403f,  -0.6864f,-0.6300f,-0.3631f,  -0.6980f,-0.6122f,-0.3715f,  -0.6720f,-0.5574f,-0.4875f,  -0.6813f,-0.5395f,-0.4947f,  
	-0.6424f,-0.4748f,-0.6016f,  -0.6495f,-0.4572f,-0.6076f,  -0.5980f,-0.3844f,-0.7033f,  -0.6029f,-0.3675f,-0.7081f,  -0.5396f,-0.2885f,-0.7909f,  -0.5425f,-0.2728f,-0.7945f,  -0.4685f,-0.1894f,-0.8629f,  -0.4696f,-0.1753f,-0.8653f,  -0.3861f,-0.0895f,-0.9181f,  -0.3855f,-0.0771f,-0.9195f,  -0.2941f,0.0091f,-0.9557f,  -0.2922f,0.0197f,-0.9562f,  -0.1945f,0.1045f,-0.9753f,  -0.1915f,0.1132f,-0.9750f,  -0.0893f,0.1949f,-0.9768f,  
	-0.0854f,0.2016f,-0.9757f,  0.0192f,0.2786f,-0.9602f,  0.0237f,0.2834f,-0.9587f,  0.1288f,0.3542f,-0.9263f,  0.1337f,0.3572f,-0.9244f,  0.2373f,0.4204f,-0.8757f,  0.2424f,0.4218f,-0.8737f,  0.3425f,0.4766f,-0.8097f,  0.3477f,0.4764f,-0.8075f,  0.4423f,0.5218f,-0.7295f,  0.4474f,0.5203f,-0.7274f,  0.5348f,0.5556f,-0.6367f,  0.5397f,0.5530f,-0.6347f,  0.6182f,0.5777f,-0.5330f,  0.6228f,0.5743f,-0.5313f,  
	0.6909f,0.5881f,-0.4205f,  0.6952f,0.5840f,-0.4190f,  0.7515f,0.5870f,-0.3010f,  0.7556f,0.5824f,-0.2998f,  0.7991f,0.5747f,-0.1767f,  0.8028f,0.5698f,-0.1759f,  0.8326f,0.5517f,-0.0498f,  0.8359f,0.5466f,-0.0493f,  0.8515f,0.5187f,0.0775f,  0.8545f,0.5135f,0.0777f,  0.8554f,0.4764f,0.2031f,  0.8583f,0.4714f,0.2030f,  0.8444f,0.4259f,0.3249f,  0.8470f,0.4210f,0.3245f,  0.8187f,0.3681f,0.4407f,  
	0.8211f,0.3634f,0.4402f,  0.7787f,0.3041f,0.5488f,  0.7809f,0.2997f,0.5480f,  0.7251f,0.2352f,0.6472f,  0.7272f,0.2311f,0.6463f,  0.6589f,0.1626f,0.7344f,  0.6609f,0.1587f,0.7335f,  0.5813f,0.0876f,0.8090f,  0.5832f,0.0839f,0.8079f,  0.4936f,0.0114f,0.8696f,  0.4955f,0.0079f,0.8686f,  0.3973f,-0.0646f,0.9154f,  0.3991f,-0.0680f,0.9144f,  0.2940f,-0.1393f,0.9456f,  0.2959f,-0.1425f,0.9445f,  
	0.1856f,-0.2112f,0.9596f,  0.1874f,-0.2144f,0.9586f,  0.0739f,-0.2793f,0.9573f,  0.0757f,-0.2824f,0.9563f,  -0.0392f,-0.3424f,0.9387f,  -0.0375f,-0.3455f,0.9377f,  -0.1519f,-0.3994f,0.9041f,  -0.1503f,-0.4025f,0.9030f,  -0.2622f,-0.4493f,0.8541f,  -0.2606f,-0.4524f,0.8529f,  -0.3682f,-0.4912f,0.7894f,  -0.3668f,-0.4944f,0.7881f,  -0.4682f,-0.5244f,0.7112f,  -0.4669f,-0.5275f,0.7097f,  -0.5605f,-0.5482f,0.6208f,  
	-0.5593f,-0.5512f,0.6191f,  -0.6435f,-0.5620f,0.5196f,  -0.6424f,-0.5649f,0.5178f,  -0.7158f,-0.5656f,0.4095f,  -0.7149f,-0.5682f,0.4075f,  -0.4331f,-0.7013f,0.5662f,  -0.4509f,-0.6986f,0.5556f,  -0.5024f,-0.7422f,0.4436f,  -0.5207f,-0.7360f,0.4326f,  -0.5644f,-0.7641f,0.3125f,  -0.5826f,-0.7548f,0.3014f,  -0.6169f,-0.7672f,0.1757f,  -0.6344f,-0.7553f,0.1648f,  -0.6579f,-0.7523f,0.0361f,  -0.6740f,-0.7382f,0.0257f,  
	-0.6857f,-0.7205f,-0.1035f,  -0.7003f,-0.7048f,-0.1133f,  -0.6994f,-0.6732f,-0.2403f,  -0.7121f,-0.6564f,-0.2492f,  -0.6980f,-0.6122f,-0.3715f,  -0.7086f,-0.5949f,-0.3794f,  -0.6813f,-0.5395f,-0.4947f,  -0.6898f,-0.5222f,-0.5014f,  -0.6495f,-0.4572f,-0.6076f,  -0.6559f,-0.4403f,-0.6131f,  -0.6029f,-0.3675f,-0.7081f,  -0.6073f,-0.3516f,-0.7125f,  -0.5425f,-0.2728f,-0.7945f,  -0.5450f,-0.2581f,-0.7977f,  -0.4696f,-0.1753f,-0.8653f,  
	-0.4704f,-0.1620f,-0.8675f,  -0.3855f,-0.0771f,-0.9195f,  -0.3849f,-0.0654f,-0.9206f,  -0.2922f,0.0197f,-0.9562f,  -0.2903f,0.0296f,-0.9565f,  -0.1915f,0.1132f,-0.9750f,  -0.1886f,0.1212f,-0.9745f,  -0.0854f,0.2016f,-0.9757f,  -0.0818f,0.2078f,-0.9747f,  0.0237f,0.2834f,-0.9587f,  0.0278f,0.2877f,-0.9573f,  0.1337f,0.3572f,-0.9244f,  0.1382f,0.3599f,-0.9227f,  0.2424f,0.4218f,-0.8737f,  0.2471f,0.4230f,-0.8718f,  
	0.3477f,0.4764f,-0.8075f,  0.3524f,0.4762f,-0.8056f,  0.4474f,0.5203f,-0.7274f,  0.4520f,0.5189f,-0.7256f,  0.5397f,0.5530f,-0.6347f,  0.5442f,0.5506f,-0.6330f,  0.6228f,0.5743f,-0.5313f,  0.6271f,0.5710f,-0.5298f,  0.6952f,0.5840f,-0.4190f,  0.6992f,0.5802f,-0.4178f,  0.7556f,0.5824f,-0.2998f,  0.7592f,0.5781f,-0.2989f,  0.8028f,0.5698f,-0.1759f,  0.8061f,0.5652f,-0.1753f,  0.8359f,0.5466f,-0.0493f,  
	0.8390f,0.5419f,-0.0490f,  0.8545f,0.5135f,0.0777f,  0.8574f,0.5088f,0.0777f,  0.8583f,0.4714f,0.2030f,  0.8609f,0.4667f,0.2028f,  0.8470f,0.4210f,0.3245f,  0.8495f,0.4164f,0.3240f,  0.8211f,0.3634f,0.4402f,  0.8234f,0.3590f,0.4395f,  0.7809f,0.2997f,0.5480f,  0.7831f,0.2956f,0.5472f,  0.7272f,0.2311f,0.6463f,  0.7293f,0.2271f,0.6454f,  0.6609f,0.1587f,0.7335f,  0.6630f,0.1550f,0.7324f,  
	0.5832f,0.0839f,0.8079f,  0.5852f,0.0804f,0.8069f,  0.4955f,0.0079f,0.8686f,  0.4974f,0.0045f,0.8675f,  0.3991f,-0.0680f,0.9144f,  0.4010f,-0.0713f,0.9133f,  0.2959f,-0.1425f,0.9445f,  0.2977f,-0.1458f,0.9435f,  0.1874f,-0.2144f,0.9586f,  0.1893f,-0.2176f,0.9575f,  0.0757f,-0.2824f,0.9563f,  0.0775f,-0.2857f,0.9552f,  -0.0375f,-0.3455f,0.9377f,  -0.0358f,-0.3488f,0.9365f,  -0.1503f,-0.4025f,0.9030f,  
	-0.1486f,-0.4059f,0.9018f,  -0.2606f,-0.4524f,0.8529f,  -0.2591f,-0.4559f,0.8515f,  -0.3668f,-0.4944f,0.7881f,  -0.3653f,-0.4978f,0.7866f,  -0.4669f,-0.5275f,0.7097f,  -0.4655f,-0.5310f,0.7081f,  -0.5593f,-0.5512f,0.6191f,  -0.5581f,-0.5546f,0.6172f,  -0.6424f,-0.5649f,0.5178f,  -0.6414f,-0.5681f,0.5157f,  -0.7149f,-0.5682f,0.4075f,  -0.7139f,-0.5711f,0.4051f,  -0.4509f,-0.6986f,0.5556f,  -0.4681f,-0.6953f,0.5453f,  
	-0.5207f,-0.7360f,0.4326f,  -0.5384f,-0.7295f,0.4219f,  -0.5826f,-0.7548f,0.3014f,  -0.6000f,-0.7453f,0.2907f,  -0.6344f,-0.7553f,0.1648f,  -0.6508f,-0.7434f,0.1543f,  -0.6740f,-0.7382f,0.0257f,  -0.6893f,-0.7243f,0.0157f,  -0.7003f,-0.7048f,-0.1133f,  -0.7139f,-0.6894f,-0.1225f,  -0.7121f,-0.6564f,-0.2492f,  -0.7238f,-0.6402f,-0.2575f,  -0.7086f,-0.5949f,-0.3794f,  -0.7184f,-0.5782f,-0.3867f,  -0.6898f,-0.5222f,-0.5014f,  
	-0.6976f,-0.5056f,-0.5077f,  -0.6559f,-0.4403f,-0.6131f,  -0.6616f,-0.4243f,-0.6182f,  -0.6073f,-0.3516f,-0.7125f,  -0.6112f,-0.3364f,-0.7164f,  -0.5450f,-0.2581f,-0.7977f,  -0.5472f,-0.2441f,-0.8006f,  -0.4704f,-0.1620f,-0.8675f,  -0.4711f,-0.1495f,-0.8693f,  -0.3849f,-0.0654f,-0.9206f,  -0.3843f,-0.0546f,-0.9216f,  -0.2903f,0.0296f,-0.9565f,  -0.2886f,0.0387f,-0.9567f,  -0.1886f,0.1212f,-0.9745f,  -0.1860f,0.1286f,-0.9741f,  
	-0.0818f,0.2078f,-0.9747f,  -0.0785f,0.2134f,-0.9738f,  0.0278f,0.2877f,-0.9573f,  0.0316f,0.2917f,-0.9560f,  0.1382f,0.3599f,-0.9227f,  0.1423f,0.3623f,-0.9212f,  0.2471f,0.4230f,-0.8718f,  0.2513f,0.4239f,-0.8701f,  0.3524f,0.4762f,-0.8056f,  0.3566f,0.4759f,-0.8040f,  0.4520f,0.5189f,-0.7256f,  0.4562f,0.5175f,-0.7240f,  0.5442f,0.5506f,-0.6330f,  0.5481f,0.5483f,-0.6316f,  0.6271f,0.5710f,-0.5298f,  
	0.6308f,0.5680f,-0.5286f,  0.6992f,0.5802f,-0.4178f,  0.7027f,0.5765f,-0.4169f,  0.7592f,0.5781f,-0.2989f,  0.7625f,0.5741f,-0.2983f,  0.8061f,0.5652f,-0.1753f,  0.8092f,0.5609f,-0.1749f,  0.8390f,0.5419f,-0.0490f,  0.8418f,0.5375f,-0.0489f,  0.8574f,0.5088f,0.0777f,  0.8600f,0.5044f,0.0775f,  0.8609f,0.4667f,0.2028f,  0.8633f,0.4623f,0.2023f,  0.8495f,0.4164f,0.3240f,  0.8518f,0.4122f,0.3234f,  
	0.8234f,0.3590f,0.4395f,  0.8256f,0.3550f,0.4387f,  0.7831f,0.2956f,0.5472f,  0.7852f,0.2917f,0.5463f,  0.7293f,0.2271f,0.6454f,  0.7313f,0.2234f,0.6444f,  0.6630f,0.1550f,0.7324f,  0.6649f,0.1514f,0.7314f,  0.5852f,0.0804f,0.8069f,  0.5872f,0.0769f,0.8058f,  0.4974f,0.0045f,0.8675f,  0.4994f,0.0011f,0.8664f,  0.4010f,-0.0713f,0.9133f,  0.4030f,-0.0746f,0.9122f,  0.2977f,-0.1458f,0.9435f,  
	0.2996f,-0.1491f,0.9423f,  0.1893f,-0.2176f,0.9575f,  0.1911f,-0.2210f,0.9563f,  0.0775f,-0.2857f,0.9552f,  0.0793f,-0.2892f,0.9540f,  -0.0358f,-0.3488f,0.9365f,  -0.0340f,-0.3524f,0.9352f,  -0.1486f,-0.4059f,0.9018f,  -0.1469f,-0.4096f,0.9004f,  -0.2591f,-0.4559f,0.8515f,  -0.2575f,-0.4596f,0.8500f,  -0.3653f,-0.4978f,0.7866f,  -0.3638f,-0.5016f,0.7849f,  -0.4655f,-0.5310f,0.7081f,  -0.4642f,-0.5348f,0.7061f,  
	-0.5581f,-0.5546f,0.6172f,  -0.5568f,-0.5583f,0.6150f,  -0.6414f,-0.5681f,0.5157f,  -0.6403f,-0.5716f,0.5132f,  -0.7139f,-0.5711f,0.4051f,  -0.7130f,-0.5743f,0.4023f,  -0.4681f,-0.6953f,0.5453f,  -0.4849f,-0.6917f,0.5352f,  -0.5384f,-0.7295f,0.4219f,  -0.5554f,-0.7226f,0.4116f,  -0.6000f,-0.7453f,0.2907f,  -0.6165f,-0.7358f,0.2804f,  -0.6508f,-0.7434f,0.1543f,  -0.6664f,-0.7315f,0.1443f,  -0.6893f,-0.7243f,0.0157f,  
	-0.7036f,-0.7106f,0.0062f,  -0.7139f,-0.6894f,-0.1225f,  -0.7266f,-0.6744f,-0.1312f,  -0.7238f,-0.6402f,-0.2575f,  -0.7347f,-0.6244f,-0.2653f,  -0.7184f,-0.5782f,-0.3867f,  -0.7274f,-0.5622f,-0.3935f,  -0.6976f,-0.5056f,-0.5077f,  -0.7047f,-0.4897f,-0.5134f,  -0.6616f,-0.4243f,-0.6182f,  -0.6669f,-0.4090f,-0.6229f,  -0.6112f,-0.3364f,-0.7164f,  -0.6147f,-0.3221f,-0.7200f,  -0.5472f,-0.2441f,-0.8006f,  -0.5491f,-0.2310f,-0.8032f,  
	-0.4711f,-0.1495f,-0.8693f,  -0.4716f,-0.1378f,-0.8710f,  -0.3843f,-0.0546f,-0.9216f,  -0.3836f,-0.0444f,-0.9224f,  -0.2886f,0.0387f,-0.9567f,  -0.2869f,0.0473f,-0.9568f,  -0.1860f,0.1286f,-0.9741f,  -0.1836f,0.1354f,-0.9736f,  -0.0785f,0.2134f,-0.9738f,  -0.0755f,0.2186f,-0.9729f,  0.0316f,0.2917f,-0.9560f,  0.0350f,0.2954f,-0.9547f,  0.1423f,0.3623f,-0.9212f,  0.1459f,0.3644f,-0.9197f,  0.2513f,0.4239f,-0.8701f,  
	0.2550f,0.4247f,-0.8686f,  0.3566f,0.4759f,-0.8040f,  0.3603f,0.4755f,-0.8025f,  0.4562f,0.5175f,-0.7240f,  0.4598f,0.5161f,-0.7226f,  0.5481f,0.5483f,-0.6316f,  0.5517f,0.5461f,-0.6304f,  0.6308f,0.5680f,-0.5286f,  0.6342f,0.5651f,-0.5277f,  0.7027f,0.5765f,-0.4169f,  0.7059f,0.5732f,-0.4162f,  0.7625f,0.5741f,-0.2983f,  0.7655f,0.5704f,-0.2978f,  0.8092f,0.5609f,-0.1749f,  0.8119f,0.5570f,-0.1748f,  
	0.8418f,0.5375f,-0.0489f,  0.8444f,0.5335f,-0.0490f,  0.8600f,0.5044f,0.0775f,  0.8624f,0.5004f,0.0772f,  0.8633f,0.4623f,0.2023f,  0.8656f,0.4584f,0.2018f,  0.8518f,0.4122f,0.3234f,  0.8539f,0.4083f,0.3227f,  0.8256f,0.3550f,0.4387f,  0.8276f,0.3512f,0.4378f,  0.7852f,0.2917f,0.5463f,  0.7872f,0.2880f,0.5453f,  0.7313f,0.2234f,0.6444f,  0.7333f,0.2199f,0.6434f,  0.6649f,0.1514f,0.7314f,  
	0.6669f,0.1480f,0.7303f,  0.5872f,0.0769f,0.8058f,  0.5891f,0.0735f,0.8047f,  0.4994f,0.0011f,0.8664f,  0.5013f,-0.0022f,0.8653f,  0.4030f,-0.0746f,0.9122f,  0.4049f,-0.0780f,0.9110f,  0.2996f,-0.1491f,0.9423f,  0.3016f,-0.1526f,0.9412f,  0.1911f,-0.2210f,0.9563f,  0.1930f,-0.2246f,0.9551f,  0.0793f,-0.2892f,0.9540f,  0.0811f,-0.2929f,0.9527f,  -0.0340f,-0.3524f,0.9352f,  -0.0323f,-0.3563f,0.9338f,  
	-0.1469f,-0.4096f,0.9004f,  -0.1452f,-0.4136f,0.8988f,  -0.2575f,-0.4596f,0.8500f,  -0.2559f,-0.4637f,0.8482f,  -0.3638f,-0.5016f,0.7849f,  -0.3623f,-0.5058f,0.7829f,  -0.4642f,-0.5348f,0.7061f,  -0.4628f,-0.5389f,0.7039f,  -0.5568f,-0.5583f,0.6150f,  -0.5556f,-0.5623f,0.6125f,  -0.6403f,-0.5716f,0.5132f,  -0.6392f,-0.5754f,0.5103f,  -0.7130f,-0.5743f,0.4023f,  -0.7120f,-0.5777f,0.3991f,  -0.4849f,-0.6917f,0.5352f,  
	-0.5012f,-0.6875f,0.5254f,  -0.5554f,-0.7226f,0.4116f,  -0.5716f,-0.7155f,0.4016f,  -0.6165f,-0.7358f,0.2804f,  -0.6322f,-0.7261f,0.2705f,  -0.6664f,-0.7315f,0.1443f,  -0.6811f,-0.7196f,0.1348f,  -0.7036f,-0.7106f,0.0062f,  -0.7169f,-0.6972f,-0.0027f,  -0.7266f,-0.6744f,-0.1312f,  -0.7383f,-0.6599f,-0.1394f,  -0.7347f,-0.6244f,-0.2653f,  -0.7447f,-0.6092f,-0.2726f,  -0.7274f,-0.5622f,-0.3935f,  -0.7356f,-0.5468f,-0.3998f,  
	-0.7047f,-0.4897f,-0.5134f,  -0.7111f,-0.4746f,-0.5187f,  -0.6669f,-0.4090f,-0.6229f,  -0.6716f,-0.3945f,-0.6271f,  -0.6147f,-0.3221f,-0.7200f,  -0.6178f,-0.3085f,-0.7232f,  -0.5491f,-0.2310f,-0.8032f,  -0.5509f,-0.2186f,-0.8054f,  -0.4716f,-0.1378f,-0.8710f,  -0.4721f,-0.1268f,-0.8724f,  -0.3836f,-0.0444f,-0.9224f,  -0.3830f,-0.0349f,-0.9231f,  -0.2869f,0.0473f,-0.9568f,  -0.2855f,0.0551f,-0.9568f,  -0.1836f,0.1354f,-0.9736f,  
	-0.1814f,0.1418f,-0.9731f,  -0.0755f,0.2186f,-0.9729f,  -0.0729f,0.2234f,-0.9720f,  0.0350f,0.2954f,-0.9547f,  0.0380f,0.2986f,-0.9536f,  0.1459f,0.3644f,-0.9197f,  0.1491f,0.3663f,-0.9185f,  0.2550f,0.4247f,-0.8686f,  0.2584f,0.4254f,-0.8673f,  0.3603f,0.4755f,-0.8025f,  0.3637f,0.4752f,-0.8012f,  0.4598f,0.5161f,-0.7226f,  0.4631f,0.5148f,-0.7215f,  0.5517f,0.5461f,-0.6304f,  0.5548f,0.5440f,-0.6295f,  
	0.6342f,0.5651f,-0.5277f,  0.6371f,0.5625f,-0.5269f,  0.7059f,0.5732f,-0.4162f,  0.7087f,0.5701f,-0.4156f,  0.7655f,0.5704f,-0.2978f,  0.7681f,0.5670f,-0.2976f,  0.8119f,0.5570f,-0.1748f,  0.8143f,0.5534f,-0.1748f,  0.8444f,0.5335f,-0.0490f,  0.8467f,0.5298f,-0.0493f,  0.8624f,0.5004f,0.0772f,  0.8645f,0.4967f,0.0767f,  0.8656f,0.4584f,0.2018f,  0.8676f,0.4547f,0.2012f,  0.8539f,0.4083f,0.3227f,  
	0.8559f,0.4048f,0.3219f,  0.8276f,0.3512f,0.4378f,  0.8296f,0.3477f,0.4369f,  0.7872f,0.2880f,0.5453f,  0.7891f,0.2846f,0.5443f,  0.7333f,0.2199f,0.6434f,  0.7352f,0.2165f,0.6423f,  0.6669f,0.1480f,0.7303f,  0.6688f,0.1447f,0.7292f,  0.5891f,0.0735f,0.8047f,  0.5910f,0.0702f,0.8036f,  0.5013f,-0.0022f,0.8653f,  0.5032f,-0.0056f,0.8641f,  0.4049f,-0.0780f,0.9110f,  0.4068f,-0.0815f,0.9099f,  
	0.3016f,-0.1526f,0.9412f,  0.3035f,-0.1562f,0.9400f,  0.1930f,-0.2246f,0.9551f,  0.1949f,-0.2284f,0.9539f,  0.0811f,-0.2929f,0.9527f,  0.0830f,-0.2969f,0.9513f,  -0.0323f,-0.3563f,0.9338f,  -0.0305f,-0.3604f,0.9323f,  -0.1452f,-0.4136f,0.8988f,  -0.1435f,-0.4179f,0.8971f,  -0.2559f,-0.4637f,0.8482f,  -0.2543f,-0.4682f,0.8463f,  -0.3623f,-0.5058f,0.7829f,  -0.3609f,-0.5103f,0.7806f,  -0.4628f,-0.5389f,0.7039f,  
	-0.4615f,-0.5434f,0.7013f,  -0.5556f,-0.5623f,0.6125f,  -0.5544f,-0.5666f,0.6095f,  -0.6392f,-0.5754f,0.5103f,  -0.6381f,-0.5795f,0.5070f,  -0.7120f,-0.5777f,0.3991f,  -0.7110f,-0.5814f,0.3955f,  -0.5012f,-0.6875f,0.5254f,  -0.5169f,-0.6831f,0.5160f,  -0.5716f,-0.7155f,0.4016f,  -0.5871f,-0.7082f,0.3920f,  -0.6322f,-0.7261f,0.2705f,  -0.6471f,-0.7163f,0.2611f,  -0.6811f,-0.7196f,0.1348f,  -0.6949f,-0.7080f,0.1258f,  
	-0.7169f,-0.6972f,-0.0027f,  -0.7294f,-0.6840f,-0.0111f,  -0.7383f,-0.6599f,-0.1394f,  -0.7493f,-0.6457f,-0.1471f,  -0.7447f,-0.6092f,-0.2726f,  -0.7540f,-0.5945f,-0.2794f,  -0.7356f,-0.5468f,-0.3998f,  -0.7432f,-0.5321f,-0.4056f,  -0.7111f,-0.4746f,-0.5187f,  -0.7171f,-0.4602f,-0.5235f,  -0.6716f,-0.3945f,-0.6271f,  -0.6760f,-0.3808f,-0.6309f,  -0.6178f,-0.3085f,-0.7232f,  -0.6207f,-0.2957f,-0.7261f,  -0.5509f,-0.2186f,-0.8054f,  
	-0.5524f,-0.2070f,-0.8074f,  -0.4721f,-0.1268f,-0.8724f,  -0.4725f,-0.1165f,-0.8736f,  -0.3830f,-0.0349f,-0.9231f,  -0.3825f,-0.0261f,-0.9236f,  -0.2855f,0.0551f,-0.9568f,  -0.2841f,0.0625f,-0.9567f,  -0.1814f,0.1418f,-0.9731f,  -0.1795f,0.1476f,-0.9726f,  -0.0729f,0.2234f,-0.9720f,  -0.0705f,0.2278f,-0.9712f,  0.0380f,0.2986f,-0.9536f,  0.0407f,0.3016f,-0.9526f,  0.1491f,0.3663f,-0.9185f,  0.1520f,0.3681f,-0.9173f,  
	0.2584f,0.4254f,-0.8673f,  0.2613f,0.4261f,-0.8661f,  0.3637f,0.4752f,-0.8012f,  0.3666f,0.4748f,-0.8001f,  0.4631f,0.5148f,-0.7215f,  0.4659f,0.5136f,-0.7205f,  0.5548f,0.5440f,-0.6295f,  0.5575f,0.5422f,-0.6287f,  0.6371f,0.5625f,-0.5269f,  0.6397f,0.5601f,-0.5264f,  0.7087f,0.5701f,-0.4156f,  0.7111f,0.5673f,-0.4153f,  0.7681f,0.5670f,-0.2976f,  0.7704f,0.5639f,-0.2975f,  0.8143f,0.5534f,-0.1748f,  
	0.8165f,0.5502f,-0.1749f,  0.8467f,0.5298f,-0.0493f,  0.8488f,0.5265f,-0.0496f,  0.8645f,0.4967f,0.0767f,  0.8665f,0.4933f,0.0762f,  0.8676f,0.4547f,0.2012f,  0.8695f,0.4514f,0.2005f,  0.8559f,0.4048f,0.3219f,  0.8578f,0.4014f,0.3211f,  0.8296f,0.3477f,0.4369f,  0.8314f,0.3445f,0.4360f,  0.7891f,0.2846f,0.5443f,  0.7909f,0.2814f,0.5433f,  0.7352f,0.2165f,0.6423f,  0.7371f,0.2133f,0.6413f,  
	0.6688f,0.1447f,0.7292f,  0.6707f,0.1414f,0.7281f,  0.5910f,0.0702f,0.8036f,  0.5929f,0.0669f,0.8025f,  0.5032f,-0.0056f,0.8641f,  0.5051f,-0.0091f,0.8630f,  0.4068f,-0.0815f,0.9099f,  0.4087f,-0.0851f,0.9087f,  0.3035f,-0.1562f,0.9400f,  0.3053f,-0.1600f,0.9387f,  0.1949f,-0.2284f,0.9539f,  0.1968f,-0.2324f,0.9525f,  0.0830f,-0.2969f,0.9513f,  0.0848f,-0.3011f,0.9498f,  -0.0305f,-0.3604f,0.9323f,  
	-0.0288f,-0.3649f,0.9306f,  -0.1435f,-0.4179f,0.8971f,  -0.1419f,-0.4225f,0.8952f,  -0.2543f,-0.4682f,0.8463f,  -0.2527f,-0.4730f,0.8441f,  -0.3609f,-0.5103f,0.7806f,  -0.3594f,-0.5152f,0.7781f,  -0.4615f,-0.5434f,0.7013f,  -0.4602f,-0.5482f,0.6984f,  -0.5544f,-0.5666f,0.6095f,  -0.5532f,-0.5713f,0.6062f,  -0.6381f,-0.5795f,0.5070f,  -0.6370f,-0.5839f,0.5033f,  -0.7110f,-0.5814f,0.3955f,  -0.7101f,-0.5854f,0.3913f,  
	-0.5169f,-0.6831f,0.5160f,  -0.5320f,-0.6783f,0.5069f,  -0.5871f,-0.7082f,0.3920f,  -0.6019f,-0.7008f,0.3829f,  -0.6471f,-0.7163f,0.2611f,  -0.6611f,-0.7067f,0.2522f,  -0.6949f,-0.7080f,0.1258f,  -0.7079f,-0.6965f,0.1174f,  -0.7294f,-0.6840f,-0.0111f,  -0.7410f,-0.6712f,-0.0190f,  -0.7493f,-0.6457f,-0.1471f,  -0.7594f,-0.6320f,-0.1541f,  -0.7540f,-0.5945f,-0.2794f,  -0.7626f,-0.5804f,-0.2856f,  -0.7432f,-0.5321f,-0.4056f,  
	-0.7502f,-0.5180f,-0.4109f,  -0.7171f,-0.4602f,-0.5235f,  -0.7225f,-0.4465f,-0.5278f,  -0.6760f,-0.3808f,-0.6309f,  -0.6800f,-0.3678f,-0.6344f,  -0.6207f,-0.2957f,-0.7261f,  -0.6234f,-0.2837f,-0.7286f,  -0.5524f,-0.2070f,-0.8074f,  -0.5539f,-0.1961f,-0.8092f,  -0.4725f,-0.1165f,-0.8736f,  -0.4729f,-0.1069f,-0.8746f,  -0.3825f,-0.0261f,-0.9236f,  -0.3820f,-0.0179f,-0.9240f,  -0.2841f,0.0625f,-0.9567f,  -0.2830f,0.0693f,-0.9566f,  
	-0.1795f,0.1476f,-0.9726f,  -0.1778f,0.1530f,-0.9721f,  -0.0705f,0.2278f,-0.9712f,  -0.0685f,0.2318f,-0.9703f,  0.0407f,0.3016f,-0.9526f,  0.0430f,0.3044f,-0.9516f,  0.1520f,0.3681f,-0.9173f,  0.1544f,0.3697f,-0.9162f,  0.2613f,0.4261f,-0.8661f,  0.2638f,0.4266f,-0.8651f,  0.3666f,0.4748f,-0.8001f,  0.3691f,0.4745f,-0.7992f,  0.4659f,0.5136f,-0.7205f,  0.4684f,0.5126f,-0.7197f,  0.5575f,0.5422f,-0.6287f,  
	0.5599f,0.5405f,-0.6281f,  0.6397f,0.5601f,-0.5264f,  0.6420f,0.5579f,-0.5259f,  0.7111f,0.5673f,-0.4153f,  0.7132f,0.5648f,-0.4151f,  0.7704f,0.5639f,-0.2975f,  0.7724f,0.5612f,-0.2975f,  0.8165f,0.5502f,-0.1749f,  0.8184f,0.5473f,-0.1752f,  0.8488f,0.5265f,-0.0496f,  0.8506f,0.5234f,-0.0501f,  0.8665f,0.4933f,0.0762f,  0.8683f,0.4902f,0.0756f,  0.8695f,0.4514f,0.2005f,  0.8713f,0.4483f,0.1997f,  
	0.8578f,0.4014f,0.3211f,  0.8595f,0.3984f,0.3202f,  0.8314f,0.3445f,0.4360f,  0.8332f,0.3414f,0.4351f,  0.7909f,0.2814f,0.5433f,  0.7927f,0.2783f,0.5424f,  0.7371f,0.2133f,0.6413f,  0.7388f,0.2102f,0.6402f,  0.6707f,0.1414f,0.7281f,  0.6725f,0.1382f,0.7271f,  0.5929f,0.0669f,0.8025f,  0.5948f,0.0635f,0.8014f,  0.5051f,-0.0091f,0.8630f,  0.5069f,-0.0126f,0.8619f,  0.4087f,-0.0851f,0.9087f,  
	0.4106f,-0.0888f,0.9075f,  0.3053f,-0.1600f,0.9387f,  0.3072f,-0.1640f,0.9374f,  0.1968f,-0.2324f,0.9525f,  0.1986f,-0.2367f,0.9511f,  0.0848f,-0.3011f,0.9498f,  0.0865f,-0.3056f,0.9482f,  -0.0288f,-0.3649f,0.9306f,  -0.0271f,-0.3697f,0.9288f,  -0.1419f,-0.4225f,0.8952f,  -0.1403f,-0.4275f,0.8930f,  -0.2527f,-0.4730f,0.8441f,  -0.2513f,-0.4781f,0.8416f,  -0.3594f,-0.5152f,0.7781f,  -0.3581f,-0.5204f,0.7752f,  
	-0.4602f,-0.5482f,0.6984f,  -0.4589f,-0.5534f,0.6950f,  -0.5532f,-0.5713f,0.6062f,  -0.5521f,-0.5764f,0.6025f,  -0.6370f,-0.5839f,0.5033f,  -0.6360f,-0.5886f,0.4990f,  -0.7101f,-0.5854f,0.3913f,  -0.7092f,-0.5896f,0.3866f,  -0.5320f,-0.6783f,0.5069f,  -0.5464f,-0.6732f,0.4982f,  -0.6019f,-0.7008f,0.3829f,  -0.6159f,-0.6932f,0.3743f,  -0.6611f,-0.7067f,0.2522f,  -0.6743f,-0.6970f,0.2438f,  -0.7079f,-0.6965f,0.1174f,  
	-0.7201f,-0.6852f,0.1094f,  -0.7410f,-0.6712f,-0.0190f,  -0.7519f,-0.6588f,-0.0263f,  -0.7594f,-0.6320f,-0.1541f,  -0.7689f,-0.6189f,-0.1606f,  -0.7626f,-0.5804f,-0.2856f,  -0.7705f,-0.5670f,-0.2912f,  -0.7502f,-0.5180f,-0.4109f,  -0.7567f,-0.5046f,-0.4157f,  -0.7225f,-0.4465f,-0.5278f,  -0.7276f,-0.4335f,-0.5317f,  -0.6800f,-0.3678f,-0.6344f,  -0.6837f,-0.3555f,-0.6374f,  -0.6234f,-0.2837f,-0.7286f,  -0.6259f,-0.2723f,-0.7309f,  
	-0.5539f,-0.1961f,-0.8092f,  -0.5553f,-0.1859f,-0.8106f,  -0.4729f,-0.1069f,-0.8746f,  -0.4734f,-0.0979f,-0.8754f,  -0.3820f,-0.0179f,-0.9240f,  -0.3818f,-0.0102f,-0.9242f,  -0.2830f,0.0693f,-0.9566f,  -0.2821f,0.0756f,-0.9564f,  -0.1778f,0.1530f,-0.9721f,  -0.1765f,0.1580f,-0.9715f,  -0.0685f,0.2318f,-0.9703f,  -0.0668f,0.2355f,-0.9696f,  0.0430f,0.3044f,-0.9516f,  0.0449f,0.3070f,-0.9507f,  0.1544f,0.3697f,-0.9162f,  
	0.1565f,0.3712f,-0.9153f,  0.2638f,0.4266f,-0.8651f,  0.2659f,0.4272f,-0.8642f,  0.3691f,0.4745f,-0.7992f,  0.3712f,0.4742f,-0.7984f,  0.4684f,0.5126f,-0.7197f,  0.4704f,0.5116f,-0.7190f,  0.5599f,0.5405f,-0.6281f,  0.5618f,0.5390f,-0.6276f,  0.6420f,0.5579f,-0.5259f,  0.6439f,0.5560f,-0.5257f,  0.7132f,0.5648f,-0.4151f,  0.7150f,0.5625f,-0.4151f,  0.7724f,0.5612f,-0.2975f,  0.7741f,0.5587f,-0.2977f,  
	0.8184f,0.5473f,-0.1752f,  0.8201f,0.5446f,-0.1755f,  0.8506f,0.5234f,-0.0501f,  0.8522f,0.5207f,-0.0506f,  0.8683f,0.4902f,0.0756f,  0.8699f,0.4875f,0.0749f,  0.8713f,0.4483f,0.1997f,  0.8729f,0.4455f,0.1989f,  0.8595f,0.3984f,0.3202f,  0.8611f,0.3956f,0.3194f,  0.8332f,0.3414f,0.4351f,  0.8348f,0.3386f,0.4342f,  0.7927f,0.2783f,0.5424f,  0.7944f,0.2754f,0.5414f,  0.7388f,0.2102f,0.6402f,  
	0.7405f,0.2072f,0.6393f,  0.6725f,0.1382f,0.7271f,  0.6742f,0.1351f,0.7261f,  0.5948f,0.0635f,0.8014f,  0.5965f,0.0602f,0.8003f,  0.5069f,-0.0126f,0.8619f,  0.5087f,-0.0162f,0.8608f,  0.4106f,-0.0888f,0.9075f,  0.4124f,-0.0927f,0.9063f,  0.3072f,-0.1640f,0.9374f,  0.3090f,-0.1682f,0.9361f,  0.1986f,-0.2367f,0.9511f,  0.2003f,-0.2412f,0.9496f,  0.0865f,-0.3056f,0.9482f,  0.0882f,-0.3105f,0.9465f,  
	-0.0271f,-0.3697f,0.9288f,  -0.0255f,-0.3748f,0.9268f,  -0.1403f,-0.4275f,0.8930f,  -0.1388f,-0.4329f,0.8907f,  -0.2513f,-0.4781f,0.8416f,  -0.2498f,-0.4837f,0.8388f,  -0.3581f,-0.5204f,0.7752f,  -0.3568f,-0.5261f,0.7720f,  -0.4589f,-0.5534f,0.6950f,  -0.4577f,-0.5590f,0.6913f,  -0.5521f,-0.5764f,0.6025f,  -0.5510f,-0.5818f,0.5982f,  -0.6360f,-0.5886f,0.4990f,  -0.6350f,-0.5936f,0.4943f,  -0.7092f,-0.5896f,0.3866f,  
	-0.7083f,-0.5940f,0.3814f,  -0.5464f,-0.6732f,0.4982f,  -0.5602f,-0.6679f,0.4900f,  -0.6159f,-0.6932f,0.3743f,  -0.6292f,-0.6856f,0.3661f,  -0.6743f,-0.6970f,0.2438f,  -0.6868f,-0.6875f,0.2360f,  -0.7201f,-0.6852f,0.1094f,  -0.7315f,-0.6742f,0.1021f,  -0.7519f,-0.6588f,-0.0263f,  -0.7620f,-0.6467f,-0.0330f,  -0.7689f,-0.6189f,-0.1606f,  -0.7777f,-0.6062f,-0.1666f,  -0.7705f,-0.5670f,-0.2912f,  -0.7779f,-0.5540f,-0.2964f,  
	-0.7567f,-0.5046f,-0.4157f,  -0.7627f,-0.4918f,-0.4200f,  -0.7276f,-0.4335f,-0.5317f,  -0.7322f,-0.4212f,-0.5352f,  -0.6837f,-0.3555f,-0.6374f,  -0.6871f,-0.3438f,-0.6401f,  -0.6259f,-0.2723f,-0.7309f,  -0.6282f,-0.2617f,-0.7327f,  -0.5553f,-0.1859f,-0.8106f,  -0.5567f,-0.1763f,-0.8118f,  -0.4734f,-0.0979f,-0.8754f,  -0.4739f,-0.0895f,-0.8760f,  -0.3818f,-0.0102f,-0.9242f,  -0.3816f,-0.0031f,-0.9243f,  -0.2821f,0.0756f,-0.9564f,  
	-0.2815f,0.0815f,-0.9561f,  -0.1765f,0.1580f,-0.9715f,  -0.1754f,0.1626f,-0.9710f,  -0.0668f,0.2355f,-0.9696f,  -0.0654f,0.2390f,-0.9688f,  0.0449f,0.3070f,-0.9507f,  0.0465f,0.3093f,-0.9498f,  0.1565f,0.3712f,-0.9153f,  0.1582f,0.3726f,-0.9144f,  0.2659f,0.4272f,-0.8642f,  0.2676f,0.4277f,-0.8634f,  0.3712f,0.4742f,-0.7984f,  0.3729f,0.4740f,-0.7977f,  0.4704f,0.5116f,-0.7190f,  0.4721f,0.5108f,-0.7185f,  
	0.5618f,0.5390f,-0.6276f,  0.5635f,0.5376f,-0.6272f,  0.6439f,0.5560f,-0.5257f,  0.6454f,0.5543f,-0.5255f,  0.7150f,0.5625f,-0.4151f,  0.7166f,0.5606f,-0.4151f,  0.7741f,0.5587f,-0.2977f,  0.7756f,0.5565f,-0.2979f,  0.8201f,0.5446f,-0.1755f,  0.8215f,0.5423f,-0.1759f,  0.8522f,0.5207f,-0.0506f,  0.8536f,0.5184f,-0.0511f,  0.8699f,0.4875f,0.0749f,  0.8713f,0.4851f,0.0743f,  0.8729f,0.4455f,0.1989f,  
	0.8743f,0.4430f,0.1982f,  0.8611f,0.3956f,0.3194f,  0.8626f,0.3931f,0.3185f,  0.8348f,0.3386f,0.4342f,  0.8363f,0.3359f,0.4333f,  0.7944f,0.2754f,0.5414f,  0.7959f,0.2727f,0.5405f,  0.7405f,0.2072f,0.6393f,  0.7422f,0.2043f,0.6383f,  0.6742f,0.1351f,0.7261f,  0.6759f,0.1319f,0.7251f,  0.5965f,0.0602f,0.8003f,  0.5982f,0.0568f,0.7993f,  0.5087f,-0.0162f,0.8608f,  0.5105f,-0.0199f,0.8597f,  
	0.4124f,-0.0927f,0.9063f,  0.4141f,-0.0968f,0.9051f,  0.3090f,-0.1682f,0.9361f,  0.3107f,-0.1726f,0.9347f,  0.2003f,-0.2412f,0.9496f,  0.2020f,-0.2460f,0.9480f,  0.0882f,-0.3105f,0.9465f,  0.0898f,-0.3157f,0.9446f,  -0.0255f,-0.3748f,0.9268f,  -0.0239f,-0.3803f,0.9245f,  -0.1388f,-0.4329f,0.8907f,  -0.1373f,-0.4387f,0.8881f,  -0.2498f,-0.4837f,0.8388f,  -0.2485f,-0.4897f,0.8357f,  -0.3568f,-0.5261f,0.7720f,  
	-0.3556f,-0.5321f,0.7684f,  -0.4577f,-0.5590f,0.6913f,  -0.4566f,-0.5650f,0.6872f,  -0.5510f,-0.5818f,0.5982f,  -0.5500f,-0.5875f,0.5935f,  -0.6350f,-0.5936f,0.4943f,  -0.6341f,-0.5989f,0.4890f,  -0.7083f,-0.5940f,0.3814f,  -0.7075f,-0.5986f,0.3756f,  -0.5602f,-0.6679f,0.4900f,  -0.5733f,-0.6624f,0.4822f,  -0.6292f,-0.6856f,0.3661f,  -0.6418f,-0.6779f,0.3585f,  -0.6868f,-0.6875f,0.2360f,  -0.6985f,-0.6781f,0.2286f,  
	-0.7315f,-0.6742f,0.1021f,  -0.7421f,-0.6634f,0.0953f,  -0.7620f,-0.6467f,-0.0330f,  -0.7715f,-0.6350f,-0.0391f,  -0.7777f,-0.6062f,-0.1666f,  -0.7858f,-0.5940f,-0.1721f,  -0.7779f,-0.5540f,-0.2964f,  -0.7848f,-0.5417f,-0.3011f,  -0.7627f,-0.4918f,-0.4200f,  -0.7683f,-0.4797f,-0.4239f,  -0.7322f,-0.4212f,-0.5352f,  -0.7366f,-0.4095f,-0.5383f,  -0.6871f,-0.3438f,-0.6401f,  -0.6903f,-0.3330f,-0.6423f,  -0.6282f,-0.2617f,-0.7327f,  
	-0.6304f,-0.2516f,-0.7343f,  -0.5567f,-0.1763f,-0.8118f,  -0.5580f,-0.1673f,-0.8128f,  -0.4739f,-0.0895f,-0.8760f,  -0.4745f,-0.0817f,-0.8764f,  -0.3816f,-0.0031f,-0.9243f,  -0.3816f,0.0036f,-0.9243f,  -0.2815f,0.0815f,-0.9561f,  -0.2810f,0.0869f,-0.9558f,  -0.1754f,0.1626f,-0.9710f,  -0.1745f,0.1669f,-0.9704f,  -0.0654f,0.2390f,-0.9688f,  -0.0643f,0.2422f,-0.9681f,  0.0465f,0.3093f,-0.9498f,  0.0477f,0.3116f,-0.9490f,  
	0.1582f,0.3726f,-0.9144f,  0.1595f,0.3739f,-0.9137f,  0.2676f,0.4277f,-0.8634f,  0.2690f,0.4282f,-0.8627f,  0.3729f,0.4740f,-0.7977f,  0.3743f,0.4739f,-0.7971f,  0.4721f,0.5108f,-0.7185f,  0.4735f,0.5101f,-0.7181f,  0.5635f,0.5376f,-0.6272f,  0.5648f,0.5365f,-0.6270f,  0.6454f,0.5543f,-0.5255f,  0.6467f,0.5528f,-0.5255f,  0.7166f,0.5606f,-0.4151f,  0.7178f,0.5589f,-0.4152f,  0.7756f,0.5565f,-0.2979f,  
	0.7768f,0.5547f,-0.2982f,  0.8215f,0.5423f,-0.1759f,  0.8228f,0.5403f,-0.1763f,  0.8536f,0.5184f,-0.0511f,  0.8549f,0.5163f,-0.0517f,  0.8713f,0.4851f,0.0743f,  0.8726f,0.4829f,0.0736f,  0.8743f,0.4430f,0.1982f,  0.8756f,0.4408f,0.1975f,  0.8626f,0.3931f,0.3185f,  0.8639f,0.3907f,0.3178f,  0.8363f,0.3359f,0.4333f,  0.8377f,0.3335f,0.4325f,  0.7959f,0.2727f,0.5405f,  0.7974f,0.2700f,0.5396f,  
	0.7422f,0.2043f,0.6383f,  0.7437f,0.2014f,0.6374f,  0.6759f,0.1319f,0.7251f,  0.6775f,0.1288f,0.7242f,  0.5982f,0.0568f,0.7993f,  0.5999f,0.0533f,0.7983f,  0.5105f,-0.0199f,0.8597f,  0.5121f,-0.0237f,0.8586f,  0.4141f,-0.0968f,0.9051f,  0.4157f,-0.1010f,0.9039f,  0.3107f,-0.1726f,0.9347f,  0.3123f,-0.1773f,0.9333f,  0.2020f,-0.2460f,0.9480f,  0.2036f,-0.2511f,0.9463f,  0.0898f,-0.3157f,0.9446f,  
	0.0913f,-0.3212f,0.9426f,  -0.0239f,-0.3803f,0.9245f,  -0.0225f,-0.3862f,0.9221f,  -0.1373f,-0.4387f,0.8881f,  -0.1360f,-0.4449f,0.8852f,  -0.2485f,-0.4897f,0.8357f,  -0.2473f,-0.4961f,0.8323f,  -0.3556f,-0.5321f,0.7684f,  -0.3544f,-0.5386f,0.7644f,  -0.4566f,-0.5650f,0.6872f,  -0.4556f,-0.5714f,0.6826f,  -0.5500f,-0.5875f,0.5935f,  -0.5491f,-0.5936f,0.5883f,  -0.6341f,-0.5989f,0.4890f,  -0.6333f,-0.6045f,0.4832f,  
	-0.7075f,-0.5986f,0.3756f,  -0.7067f,-0.6035f,0.3692f,  -0.5733f,-0.6624f,0.4822f,  -0.5858f,-0.6568f,0.4748f,  -0.6418f,-0.6779f,0.3585f,  -0.6536f,-0.6703f,0.3513f,  -0.6985f,-0.6781f,0.2286f,  -0.7095f,-0.6688f,0.2219f,  -0.7421f,-0.6634f,0.0953f,  -0.7521f,-0.6530f,0.0890f,  -0.7715f,-0.6350f,-0.0391f,  -0.7803f,-0.6238f,-0.0448f,  -0.7858f,-0.5940f,-0.1721f,  -0.7934f,-0.5824f,-0.1770f,  -0.7848f,-0.5417f,-0.3011f,  
	-0.7911f,-0.5300f,-0.3053f,  -0.7683f,-0.4797f,-0.4239f,  -0.7734f,-0.4682f,-0.4273f,  -0.7366f,-0.4095f,-0.5383f,  -0.7406f,-0.3986f,-0.5409f,  -0.6903f,-0.3330f,-0.6423f,  -0.6934f,-0.3227f,-0.6443f,  -0.6304f,-0.2516f,-0.7343f,  -0.6325f,-0.2422f,-0.7357f,  -0.5580f,-0.1673f,-0.8128f,  -0.5594f,-0.1589f,-0.8135f,  -0.4745f,-0.0817f,-0.8764f,  -0.4753f,-0.0744f,-0.8767f,  -0.3816f,0.0036f,-0.9243f,  -0.3818f,0.0098f,-0.9242f,  
	-0.2810f,0.0869f,-0.9558f,  -0.2807f,0.0920f,-0.9554f,  -0.1745f,0.1669f,-0.9704f,  -0.1740f,0.1710f,-0.9698f,  -0.0643f,0.2422f,-0.9681f,  -0.0635f,0.2453f,-0.9674f,  0.0477f,0.3116f,-0.9490f,  0.0487f,0.3137f,-0.9483f,  0.1595f,0.3739f,-0.9137f,  0.1605f,0.3752f,-0.9129f,  0.2690f,0.4282f,-0.8627f,  0.2701f,0.4288f,-0.8621f,  0.3743f,0.4739f,-0.7971f,  0.3753f,0.4739f,-0.7966f,  0.4735f,0.5101f,-0.7181f,  
	0.4745f,0.5096f,-0.7177f,  0.5648f,0.5365f,-0.6270f,  0.5658f,0.5357f,-0.6268f,  0.6467f,0.5528f,-0.5255f,  0.6477f,0.5517f,-0.5255f,  0.7178f,0.5589f,-0.4152f,  0.7188f,0.5575f,-0.4154f,  0.7768f,0.5547f,-0.2982f,  0.7778f,0.5531f,-0.2985f,  0.8228f,0.5403f,-0.1763f,  0.8238f,0.5387f,-0.1768f,  0.8549f,0.5163f,-0.0517f,  0.8559f,0.5145f,-0.0522f,  0.8726f,0.4829f,0.0736f,  0.8737f,0.4810f,0.0730f,  
	0.8756f,0.4408f,0.1975f,  0.8768f,0.4388f,0.1968f,  0.8639f,0.3907f,0.3178f,  0.8652f,0.3885f,0.3170f,  0.8377f,0.3335f,0.4325f,  0.8390f,0.3311f,0.4317f,  0.7974f,0.2700f,0.5396f,  0.7988f,0.2674f,0.5389f,  0.7437f,0.2014f,0.6374f,  0.7452f,0.1985f,0.6366f,  0.6775f,0.1288f,0.7242f,  0.6790f,0.1256f,0.7233f,  0.5999f,0.0533f,0.7983f,  0.6014f,0.0497f,0.7974f,  0.5121f,-0.0237f,0.8586f,  
	0.5137f,-0.0278f,0.8575f,  0.4157f,-0.1010f,0.9039f,  0.4173f,-0.1056f,0.9026f,  0.3123f,-0.1773f,0.9333f,  0.3138f,-0.1823f,0.9318f,  0.2036f,-0.2511f,0.9463f,  0.2050f,-0.2566f,0.9445f,  0.0913f,-0.3212f,0.9426f,  0.0928f,-0.3272f,0.9404f,  -0.0225f,-0.3862f,0.9221f,  -0.0212f,-0.3926f,0.9195f,  -0.1360f,-0.4449f,0.8852f,  -0.1348f,-0.4516f,0.8820f,  -0.2473f,-0.4961f,0.8323f,  -0.2462f,-0.5029f,0.8285f,  
	-0.3544f,-0.5386f,0.7644f,  -0.3534f,-0.5455f,0.7600f,  -0.4556f,-0.5714f,0.6826f,  -0.4547f,-0.5781f,0.6775f,  -0.5491f,-0.5936f,0.5883f,  -0.5483f,-0.6000f,0.5825f,  -0.6333f,-0.6045f,0.4832f,  -0.6326f,-0.6104f,0.4767f,  -0.7067f,-0.6035f,0.3692f,  -0.7060f,-0.6086f,0.3622f,  -0.5858f,-0.6568f,0.4748f,  -0.5976f,-0.6511f,0.4679f,  -0.6536f,-0.6703f,0.3513f,  -0.6648f,-0.6628f,0.3447f,  -0.7095f,-0.6688f,0.2219f,  
	-0.7198f,-0.6598f,0.2156f,  -0.7521f,-0.6530f,0.0890f,  -0.7614f,-0.6429f,0.0833f,  -0.7803f,-0.6238f,-0.0448f,  -0.7885f,-0.6130f,-0.0499f,  -0.7934f,-0.5824f,-0.1770f,  -0.8005f,-0.5712f,-0.1814f,  -0.7911f,-0.5300f,-0.3053f,  -0.7971f,-0.5188f,-0.3090f,  -0.7734f,-0.4682f,-0.4273f,  -0.7783f,-0.4574f,-0.4303f,  -0.7406f,-0.3986f,-0.5409f,  -0.7444f,-0.3882f,-0.5432f,  -0.6934f,-0.3227f,-0.6443f,  -0.6962f,-0.3130f,-0.6460f,  
	-0.6325f,-0.2422f,-0.7357f,  -0.6346f,-0.2334f,-0.7367f,  -0.5594f,-0.1589f,-0.8135f,  -0.5608f,-0.1511f,-0.8141f,  -0.4753f,-0.0744f,-0.8767f,  -0.4761f,-0.0676f,-0.8768f,  -0.3818f,0.0098f,-0.9242f,  -0.3821f,0.0156f,-0.9240f,  -0.2807f,0.0920f,-0.9554f,  -0.2807f,0.0968f,-0.9549f,  -0.1740f,0.1710f,-0.9698f,  -0.1737f,0.1747f,-0.9692f,  -0.0635f,0.2453f,-0.9674f,  -0.0630f,0.2481f,-0.9667f,  0.0487f,0.3137f,-0.9483f,  
	0.0493f,0.3157f,-0.9476f,  0.1605f,0.3752f,-0.9129f,  0.1612f,0.3765f,-0.9123f,  0.2701f,0.4288f,-0.8621f,  0.2708f,0.4295f,-0.8615f,  0.3753f,0.4739f,-0.7966f,  0.3761f,0.4740f,-0.7962f,  0.4745f,0.5096f,-0.7177f,  0.4753f,0.5093f,-0.7174f,  0.5658f,0.5357f,-0.6268f,  0.5666f,0.5350f,-0.6267f,  0.6477f,0.5517f,-0.5255f,  0.6485f,0.5508f,-0.5255f,  0.7188f,0.5575f,-0.4154f,  0.7196f,0.5564f,-0.4156f,  
	0.7778f,0.5531f,-0.2985f,  0.7786f,0.5518f,-0.2988f,  0.8238f,0.5387f,-0.1768f,  0.8246f,0.5373f,-0.1772f,  0.8559f,0.5145f,-0.0522f,  0.8568f,0.5130f,-0.0527f,  0.8737f,0.4810f,0.0730f,  0.8746f,0.4793f,0.0725f,  0.8768f,0.4388f,0.1968f,  0.8778f,0.4370f,0.1962f,  0.8652f,0.3885f,0.3170f,  0.8663f,0.3866f,0.3164f,  0.8390f,0.3311f,0.4317f,  0.8402f,0.3289f,0.4311f,  0.7988f,0.2674f,0.5389f,  
	0.8001f,0.2649f,0.5382f,  0.7452f,0.1985f,0.6366f,  0.7465f,0.1957f,0.6360f,  0.6790f,0.1256f,0.7233f,  0.6804f,0.1223f,0.7226f,  0.6014f,0.0497f,0.7974f,  0.6028f,0.0460f,0.7965f,  0.5137f,-0.0278f,0.8575f,  0.5151f,-0.0320f,0.8565f,  0.4173f,-0.1056f,0.9026f,  0.4187f,-0.1103f,0.9014f,  0.3138f,-0.1823f,0.9318f,  0.3152f,-0.1876f,0.9303f,  0.2050f,-0.2566f,0.9445f,  0.2064f,-0.2625f,0.9426f,  
	0.0928f,-0.3272f,0.9404f,  0.0940f,-0.3335f,0.9380f,  -0.0212f,-0.3926f,0.9195f,  -0.0200f,-0.3994f,0.9166f,  -0.1348f,-0.4516f,0.8820f,  -0.1337f,-0.4587f,0.8785f,  -0.2462f,-0.5029f,0.8285f,  -0.2452f,-0.5102f,0.8243f,  -0.3534f,-0.5455f,0.7600f,  -0.3526f,-0.5528f,0.7551f,  -0.4547f,-0.5781f,0.6775f,  -0.4539f,-0.5853f,0.6718f,  -0.5483f,-0.6000f,0.5825f,  -0.5476f,-0.6068f,0.5761f,  -0.6326f,-0.6104f,0.4767f,  
	-0.6319f,-0.6165f,0.4697f,  -0.7060f,-0.6086f,0.3622f,  -0.7054f,-0.6138f,0.3545f,  -0.5976f,-0.6511f,0.4679f,  -0.6086f,-0.6454f,0.4616f,  -0.6648f,-0.6628f,0.3447f,  -0.6752f,-0.6553f,0.3386f,  -0.7198f,-0.6598f,0.2156f,  -0.7295f,-0.6510f,0.2099f,  -0.7614f,-0.6429f,0.0833f,  -0.7700f,-0.6332f,0.0780f,  -0.7885f,-0.6130f,-0.0499f,  -0.7961f,-0.6027f,-0.0545f,  -0.8005f,-0.5712f,-0.1814f,  -0.8070f,-0.5606f,-0.1854f,  
	-0.7971f,-0.5188f,-0.3090f,  -0.8026f,-0.5083f,-0.3123f,  -0.7783f,-0.4574f,-0.4303f,  -0.7828f,-0.4471f,-0.4329f,  -0.7444f,-0.3882f,-0.5432f,  -0.7480f,-0.3784f,-0.5452f,  -0.6962f,-0.3130f,-0.6460f,  -0.6990f,-0.3040f,-0.6473f,  -0.6346f,-0.2334f,-0.7367f,  -0.6367f,-0.2252f,-0.7375f,  -0.5608f,-0.1511f,-0.8141f,  -0.5622f,-0.1438f,-0.8144f,  -0.4761f,-0.0676f,-0.8768f,  -0.4770f,-0.0612f,-0.8768f,  -0.3821f,0.0156f,-0.9240f,  
	-0.3826f,0.0209f,-0.9237f,  -0.2807f,0.0968f,-0.9549f,  -0.2809f,0.1012f,-0.9544f,  -0.1737f,0.1747f,-0.9692f,  -0.1736f,0.1783f,-0.9685f,  -0.0630f,0.2481f,-0.9667f,  -0.0628f,0.2508f,-0.9660f,  0.0493f,0.3157f,-0.9476f,  0.0496f,0.3177f,-0.9469f,  0.1612f,0.3765f,-0.9123f,  0.1616f,0.3778f,-0.9117f,  0.2708f,0.4295f,-0.8615f,  0.2712f,0.4303f,-0.8610f,  0.3761f,0.4740f,-0.7962f,  0.3766f,0.4743f,-0.7958f,  
	0.4753f,0.5093f,-0.7174f,  0.4757f,0.5092f,-0.7172f,  0.5666f,0.5350f,-0.6267f,  0.5671f,0.5346f,-0.6266f,  0.6485f,0.5508f,-0.5255f,  0.6490f,0.5501f,-0.5255f,  0.7196f,0.5564f,-0.4156f,  0.7201f,0.5555f,-0.4157f,  0.7786f,0.5518f,-0.2988f,  0.7792f,0.5508f,-0.2991f,  0.8246f,0.5373f,-0.1772f,  0.8253f,0.5361f,-0.1775f,  0.8568f,0.5130f,-0.0527f,  0.8575f,0.5117f,-0.0531f,  0.8746f,0.4793f,0.0725f,  
	0.8754f,0.4779f,0.0720f,  0.8778f,0.4370f,0.1962f,  0.8787f,0.4354f,0.1957f,  0.8663f,0.3866f,0.3164f,  0.8673f,0.3847f,0.3160f,  0.8402f,0.3289f,0.4311f,  0.8413f,0.3268f,0.4306f,  0.8001f,0.2649f,0.5382f,  0.8012f,0.2625f,0.5377f,  0.7465f,0.1957f,0.6360f,  0.7477f,0.1928f,0.6354f,  0.6804f,0.1223f,0.7226f,  0.6817f,0.1189f,0.7219f,  0.6028f,0.0460f,0.7965f,  0.6042f,0.0421f,0.7958f,  
	0.5151f,-0.0320f,0.8565f,  0.5165f,-0.0365f,0.8555f,  0.4187f,-0.1103f,0.9014f,  0.4200f,-0.1154f,0.9001f,  0.3152f,-0.1876f,0.9303f,  0.3165f,-0.1933f,0.9287f,  0.2064f,-0.2625f,0.9426f,  0.2076f,-0.2687f,0.9406f,  0.0940f,-0.3335f,0.9380f,  0.0952f,-0.3403f,0.9355f,  -0.0200f,-0.3994f,0.9166f,  -0.0189f,-0.4066f,0.9134f,  -0.1337f,-0.4587f,0.8785f,  -0.1327f,-0.4663f,0.8746f,  -0.2452f,-0.5102f,0.8243f,  
	-0.2443f,-0.5180f,0.8197f,  -0.3526f,-0.5528f,0.7551f,  -0.3518f,-0.5606f,0.7497f,  -0.4539f,-0.5853f,0.6718f,  -0.4533f,-0.5929f,0.6656f,  -0.5476f,-0.6068f,0.5761f,  -0.5470f,-0.6139f,0.5691f,  -0.6319f,-0.6165f,0.4697f,  -0.6314f,-0.6229f,0.4619f,  -0.7054f,-0.6138f,0.3545f,  -0.7049f,-0.6192f,0.3461f,  -0.6086f,-0.6454f,0.4616f,  -0.6191f,-0.6396f,0.4557f,  -0.6752f,-0.6553f,0.3386f,  -0.6850f,-0.6480f,0.3329f,  
	-0.7295f,-0.6510f,0.2099f,  -0.7384f,-0.6425f,0.2047f,  -0.7700f,-0.6332f,0.0780f,  -0.7781f,-0.6238f,0.0733f,  -0.7961f,-0.6027f,-0.0545f,  -0.8032f,-0.5928f,-0.0587f,  -0.8070f,-0.5606f,-0.1854f,  -0.8131f,-0.5506f,-0.1889f,  -0.8026f,-0.5083f,-0.3123f,  -0.8077f,-0.4983f,-0.3151f,  -0.7828f,-0.4471f,-0.4329f,  -0.7870f,-0.4374f,-0.4351f,  -0.7480f,-0.3784f,-0.5452f,  -0.7514f,-0.3693f,-0.5468f,  -0.6990f,-0.3040f,-0.6473f,  
	-0.7017f,-0.2955f,-0.6483f,  -0.6367f,-0.2252f,-0.7375f,  -0.6387f,-0.2175f,-0.7381f,  -0.5622f,-0.1438f,-0.8144f,  -0.5636f,-0.1369f,-0.8146f,  -0.4770f,-0.0612f,-0.8768f,  -0.4780f,-0.0552f,-0.8766f,  -0.3826f,0.0209f,-0.9237f,  -0.3833f,0.0260f,-0.9233f,  -0.2809f,0.1012f,-0.9544f,  -0.2813f,0.1054f,-0.9538f,  -0.1736f,0.1783f,-0.9685f,  -0.1738f,0.1817f,-0.9679f,  -0.0628f,0.2508f,-0.9660f,  -0.0628f,0.2535f,-0.9653f,  
	0.0496f,0.3177f,-0.9469f,  0.0497f,0.3196f,-0.9462f,  0.1616f,0.3778f,-0.9117f,  0.1617f,0.3792f,-0.9111f,  0.2712f,0.4303f,-0.8610f,  0.2714f,0.4311f,-0.8605f,  0.3766f,0.4743f,-0.7958f,  0.3767f,0.4747f,-0.7954f,  0.4757f,0.5092f,-0.7172f,  0.4760f,0.5093f,-0.7170f,  0.5671f,0.5346f,-0.6266f,  0.5673f,0.5345f,-0.6265f,  0.6490f,0.5501f,-0.5255f,  0.6493f,0.5498f,-0.5255f,  0.7201f,0.5555f,-0.4157f,  
	0.7204f,0.5550f,-0.4159f,  0.7792f,0.5508f,-0.2991f,  0.7796f,0.5502f,-0.2993f,  0.8253f,0.5361f,-0.1775f,  0.8257f,0.5353f,-0.1778f,  0.8575f,0.5117f,-0.0531f,  0.8581f,0.5107f,-0.0534f,  0.8754f,0.4779f,0.0720f,  0.8761f,0.4768f,0.0717f,  0.8787f,0.4354f,0.1957f,  0.8795f,0.4340f,0.1954f,  0.8673f,0.3847f,0.3160f,  0.8681f,0.3830f,0.3156f,  0.8413f,0.3268f,0.4306f,  0.8423f,0.3247f,0.4303f,  
	0.8012f,0.2625f,0.5377f,  0.8023f,0.2600f,0.5373f,  0.7477f,0.1928f,0.6354f,  0.7489f,0.1898f,0.6349f,  0.6817f,0.1189f,0.7219f,  0.6829f,0.1154f,0.7214f,  0.6042f,0.0421f,0.7958f,  0.6054f,0.0380f,0.7950f,  0.5165f,-0.0365f,0.8555f,  0.5177f,-0.0412f,0.8546f,  0.4200f,-0.1154f,0.9001f,  0.4212f,-0.1208f,0.8989f,  0.3165f,-0.1933f,0.9287f,  0.3177f,-0.1994f,0.9270f,  0.2076f,-0.2687f,0.9406f,  
	0.2087f,-0.2754f,0.9384f,  0.0952f,-0.3403f,0.9355f,  0.0962f,-0.3475f,0.9327f,  -0.0189f,-0.4066f,0.9134f,  -0.0180f,-0.4143f,0.9100f,  -0.1327f,-0.4663f,0.8746f,  -0.1320f,-0.4743f,0.8704f,  -0.2443f,-0.5180f,0.8197f,  -0.2436f,-0.5262f,0.8147f,  -0.3518f,-0.5606f,0.7497f,  -0.3512f,-0.5688f,0.7437f,  -0.4533f,-0.5929f,0.6656f,  -0.4528f,-0.6008f,0.6588f,  -0.5470f,-0.6139f,0.5691f,  -0.5465f,-0.6213f,0.5614f,  
	-0.6314f,-0.6229f,0.4619f,  -0.6309f,-0.6295f,0.4534f,  -0.7049f,-0.6192f,0.3461f,  -0.7044f,-0.6247f,0.3369f,  -0.6191f,-0.6396f,0.4557f,  -0.6289f,-0.6339f,0.4502f,  -0.6850f,-0.6480f,0.3329f,  -0.6942f,-0.6408f,0.3278f,  -0.7384f,-0.6425f,0.2047f,  -0.7468f,-0.6343f,0.1999f,  -0.7781f,-0.6238f,0.0733f,  -0.7856f,-0.6148f,0.0691f,  -0.8032f,-0.5928f,-0.0587f,  -0.8098f,-0.5834f,-0.0623f,  -0.8131f,-0.5506f,-0.1889f,  
	-0.8188f,-0.5410f,-0.1919f,  -0.8077f,-0.4983f,-0.3151f,  -0.8125f,-0.4888f,-0.3175f,  -0.7870f,-0.4374f,-0.4351f,  -0.7910f,-0.4283f,-0.4369f,  -0.7514f,-0.3693f,-0.5468f,  -0.7547f,-0.3607f,-0.5481f,  -0.7017f,-0.2955f,-0.6483f,  -0.7043f,-0.2875f,-0.6491f,  -0.6387f,-0.2175f,-0.7381f,  -0.6407f,-0.2103f,-0.7384f,  -0.5636f,-0.1369f,-0.8146f,  -0.5652f,-0.1305f,-0.8146f,  -0.4780f,-0.0552f,-0.8766f,  -0.4791f,-0.0497f,-0.8763f,  
	-0.3833f,0.0260f,-0.9233f,  -0.3841f,0.0308f,-0.9228f,  -0.2813f,0.1054f,-0.9538f,  -0.2819f,0.1094f,-0.9532f,  -0.1738f,0.1817f,-0.9679f,  -0.1742f,0.1849f,-0.9672f,  -0.0628f,0.2535f,-0.9653f,  -0.0631f,0.2560f,-0.9646f,  0.0497f,0.3196f,-0.9462f,  0.0495f,0.3216f,-0.9456f,  0.1617f,0.3792f,-0.9111f,  0.1616f,0.3806f,-0.9105f,  0.2714f,0.4311f,-0.8605f,  0.2713f,0.4321f,-0.8600f,  0.3767f,0.4747f,-0.7954f,  
	0.3767f,0.4753f,-0.7951f,  0.4760f,0.5093f,-0.7170f,  0.4759f,0.5097f,-0.7167f,  0.5673f,0.5345f,-0.6265f,  0.5673f,0.5346f,-0.6264f,  0.6493f,0.5498f,-0.5255f,  0.6494f,0.5497f,-0.5255f,  0.7204f,0.5550f,-0.4159f,  0.7206f,0.5548f,-0.4159f,  0.7796f,0.5502f,-0.2993f,  0.7798f,0.5498f,-0.2994f,  0.8257f,0.5353f,-0.1778f,  0.8261f,0.5348f,-0.1780f,  0.8581f,0.5107f,-0.0534f,  0.8585f,0.5100f,-0.0536f,  
	0.8761f,0.4768f,0.0717f,  0.8766f,0.4758f,0.0715f,  0.8795f,0.4340f,0.1954f,  0.8801f,0.4327f,0.1952f,  0.8681f,0.3830f,0.3156f,  0.8689f,0.3815f,0.3154f,  0.8423f,0.3247f,0.4303f,  0.8431f,0.3227f,0.4301f,  0.8023f,0.2600f,0.5373f,  0.8032f,0.2575f,0.5371f,  0.7489f,0.1898f,0.6349f,  0.7499f,0.1868f,0.6347f,  0.6829f,0.1154f,0.7214f,  0.6839f,0.1118f,0.7210f,  0.6054f,0.0380f,0.7950f,  
	0.6064f,0.0337f,0.7944f,  0.5177f,-0.0412f,0.8546f,  0.5187f,-0.0463f,0.8537f,  0.4212f,-0.1208f,0.8989f,  0.4223f,-0.1266f,0.8976f,  0.3177f,-0.1994f,0.9270f,  0.3187f,-0.2058f,0.9253f,  0.2087f,-0.2754f,0.9384f,  0.2096f,-0.2826f,0.9361f,  0.0962f,-0.3475f,0.9327f,  0.0970f,-0.3553f,0.9297f,  -0.0180f,-0.4143f,0.9100f,  -0.0173f,-0.4226f,0.9062f,  -0.1320f,-0.4743f,0.8704f,  -0.1313f,-0.4829f,0.8658f,  
	-0.2436f,-0.5262f,0.8147f,  -0.2431f,-0.5350f,0.8091f,  -0.3512f,-0.5688f,0.7437f,  -0.3508f,-0.5774f,0.7372f,  -0.4528f,-0.6008f,0.6588f,  -0.4524f,-0.6092f,0.6514f,  -0.5465f,-0.6213f,0.5614f,  -0.5462f,-0.6291f,0.5531f,  -0.6309f,-0.6295f,0.4534f,  -0.6306f,-0.6364f,0.4443f,  -0.7044f,-0.6247f,0.3369f,  -0.7040f,-0.6304f,0.3271f,  -0.6289f,-0.6339f,0.4502f,  -0.6380f,-0.6282f,0.4453f,  -0.6942f,-0.6408f,0.3278f,  
	-0.7027f,-0.6339f,0.3231f,  -0.7468f,-0.6343f,0.1999f,  -0.7546f,-0.6263f,0.1957f,  -0.7856f,-0.6148f,0.0691f,  -0.7926f,-0.6062f,0.0654f,  -0.8098f,-0.5834f,-0.0623f,  -0.8159f,-0.5744f,-0.0655f,  -0.8188f,-0.5410f,-0.1919f,  -0.8241f,-0.5319f,-0.1945f,  -0.8125f,-0.4888f,-0.3175f,  -0.8170f,-0.4799f,-0.3196f,  -0.7910f,-0.4283f,-0.4369f,  -0.7948f,-0.4197f,-0.4384f,  -0.7547f,-0.3607f,-0.5481f,  -0.7578f,-0.3526f,-0.5490f,  
	-0.7043f,-0.2875f,-0.6491f,  -0.7068f,-0.2800f,-0.6497f,  -0.6407f,-0.2103f,-0.7384f,  -0.6427f,-0.2035f,-0.7386f,  -0.5652f,-0.1305f,-0.8146f,  -0.5668f,-0.1245f,-0.8144f,  -0.4791f,-0.0497f,-0.8763f,  -0.4804f,-0.0444f,-0.8759f,  -0.3841f,0.0308f,-0.9228f,  -0.3851f,0.0352f,-0.9222f,  -0.2819f,0.1094f,-0.9532f,  -0.2826f,0.1132f,-0.9525f,  -0.1742f,0.1849f,-0.9672f,  -0.1748f,0.1880f,-0.9665f,  -0.0631f,0.2560f,-0.9646f,  
	-0.0636f,0.2585f,-0.9639f,  0.0495f,0.3216f,-0.9456f,  0.0490f,0.3235f,-0.9450f,  0.1616f,0.3806f,-0.9105f,  0.1612f,0.3821f,-0.9100f,  0.2713f,0.4321f,-0.8600f,  0.2710f,0.4332f,-0.8596f,  0.3767f,0.4753f,-0.7951f,  0.3764f,0.4762f,-0.7947f,  0.4759f,0.5097f,-0.7167f,  0.4757f,0.5102f,-0.7165f,  0.5673f,0.5346f,-0.6264f,  0.5672f,0.5349f,-0.6262f,  0.6494f,0.5497f,-0.5255f,  0.6492f,0.5499f,-0.5255f,  
	0.7206f,0.5548f,-0.4159f,  0.7206f,0.5548f,-0.4159f,  0.7798f,0.5498f,-0.2994f,  0.7799f,0.5497f,-0.2994f,  0.8261f,0.5348f,-0.1780f,  0.8262f,0.5345f,-0.1780f,  0.8585f,0.5100f,-0.0536f,  0.8588f,0.5095f,-0.0536f,  0.8766f,0.4758f,0.0715f,  0.8771f,0.4750f,0.0715f,  0.8801f,0.4327f,0.1952f,  0.8807f,0.4317f,0.1952f,  0.8689f,0.3815f,0.3154f,  0.8695f,0.3800f,0.3155f,  0.8431f,0.3227f,0.4301f,  
	0.8439f,0.3208f,0.4301f,  0.8032f,0.2575f,0.5371f,  0.8041f,0.2550f,0.5371f,  0.7499f,0.1868f,0.6347f,  0.7508f,0.1837f,0.6345f,  0.6839f,0.1118f,0.7210f,  0.6848f,0.1080f,0.7207f,  0.6064f,0.0337f,0.7944f,  0.6074f,0.0291f,0.7939f,  0.5187f,-0.0463f,0.8537f,  0.5197f,-0.0516f,0.8528f,  0.4223f,-0.1266f,0.8976f,  0.4232f,-0.1327f,0.8963f,  0.3187f,-0.2058f,0.9253f,  0.3195f,-0.2127f,0.9234f,  
	0.2096f,-0.2826f,0.9361f,  0.2104f,-0.2902f,0.9336f,  0.0970f,-0.3553f,0.9297f,  0.0976f,-0.3636f,0.9264f,  -0.0173f,-0.4226f,0.9062f,  -0.0168f,-0.4313f,0.9020f,  -0.1313f,-0.4829f,0.8658f,  -0.1309f,-0.4920f,0.8607f,  -0.2431f,-0.5350f,0.8091f,  -0.2428f,-0.5442f,0.8031f,  -0.3508f,-0.5774f,0.7372f,  -0.3505f,-0.5866f,0.7301f,  -0.4524f,-0.6092f,0.6514f,  -0.4522f,-0.6179f,0.6432f,  -0.5462f,-0.6291f,0.5531f,  
	-0.5460f,-0.6372f,0.5440f,  -0.6306f,-0.6364f,0.4443f,  -0.6304f,-0.6434f,0.4343f,  -0.7040f,-0.6304f,0.3271f,  -0.7038f,-0.6361f,0.3164f,  -0.6380f,-0.6282f,0.4453f,  -0.6466f,-0.6226f,0.4408f,  -0.7027f,-0.6339f,0.3231f,  -0.7107f,-0.6271f,0.3189f,  -0.7546f,-0.6263f,0.1957f,  -0.7618f,-0.6187f,0.1920f,  -0.7926f,-0.6062f,0.0654f,  -0.7991f,-0.5980f,0.0621f,  -0.8159f,-0.5744f,-0.0655f,  -0.8216f,-0.5659f,-0.0682f,  
	-0.8241f,-0.5319f,-0.1945f,  -0.8291f,-0.5234f,-0.1968f,  -0.8170f,-0.4799f,-0.3196f,  -0.8213f,-0.4715f,-0.3212f,  -0.7948f,-0.4197f,-0.4384f,  -0.7983f,-0.4116f,-0.4396f,  -0.7578f,-0.3526f,-0.5490f,  -0.7607f,-0.3450f,-0.5498f,  -0.7068f,-0.2800f,-0.6497f,  -0.7092f,-0.2730f,-0.6500f,  -0.6427f,-0.2035f,-0.7386f,  -0.6447f,-0.1972f,-0.7386f,  -0.5668f,-0.1245f,-0.8144f,  -0.5684f,-0.1189f,-0.8141f,  -0.4804f,-0.0444f,-0.8759f,  
	-0.4817f,-0.0395f,-0.8754f,  -0.3851f,0.0352f,-0.9222f,  -0.3862f,0.0395f,-0.9216f,  -0.2826f,0.1132f,-0.9525f,  -0.2836f,0.1167f,-0.9518f,  -0.1748f,0.1880f,-0.9665f,  -0.1757f,0.1910f,-0.9658f,  -0.0636f,0.2585f,-0.9639f,  -0.0644f,0.2609f,-0.9632f,  0.0490f,0.3235f,-0.9450f,  0.0484f,0.3255f,-0.9443f,  0.1612f,0.3821f,-0.9100f,  0.1606f,0.3837f,-0.9094f,  0.2710f,0.4332f,-0.8596f,  0.2704f,0.4345f,-0.8591f,  
	0.3764f,0.4762f,-0.7947f,  0.3759f,0.4772f,-0.7943f,  0.4757f,0.5102f,-0.7165f,  0.4753f,0.5111f,-0.7162f,  0.5672f,0.5349f,-0.6262f,  0.5668f,0.5356f,-0.6260f,  0.6492f,0.5499f,-0.5255f,  0.6490f,0.5504f,-0.5253f,  0.7206f,0.5548f,-0.4159f,  0.7204f,0.5551f,-0.4158f,  0.7799f,0.5497f,-0.2994f,  0.7798f,0.5498f,-0.2993f,  0.8262f,0.5345f,-0.1780f,  0.8263f,0.5344f,-0.1778f,  0.8588f,0.5095f,-0.0536f,  
	0.8590f,0.5092f,-0.0535f,  0.8771f,0.4750f,0.0715f,  0.8773f,0.4745f,0.0717f,  0.8807f,0.4317f,0.1952f,  0.8811f,0.4307f,0.1955f,  0.8695f,0.3800f,0.3155f,  0.8701f,0.3786f,0.3157f,  0.8439f,0.3208f,0.4301f,  0.8445f,0.3189f,0.4303f,  0.8041f,0.2550f,0.5371f,  0.8048f,0.2525f,0.5372f,  0.7508f,0.1837f,0.6345f,  0.7515f,0.1804f,0.6346f,  0.6848f,0.1080f,0.7207f,  0.6856f,0.1039f,0.7205f,  
	0.6074f,0.0291f,0.7939f,  0.6082f,0.0242f,0.7935f,  0.5197f,-0.0516f,0.8528f,  0.5204f,-0.0573f,0.8520f,  0.4232f,-0.1327f,0.8963f,  0.4239f,-0.1393f,0.8949f,  0.3195f,-0.2127f,0.9234f,  0.3201f,-0.2201f,0.9214f,  0.2104f,-0.2902f,0.9336f,  0.2109f,-0.2983f,0.9309f,  0.0976f,-0.3636f,0.9264f,  0.0981f,-0.3724f,0.9229f,  -0.0168f,-0.4313f,0.9020f,  -0.0164f,-0.4406f,0.8975f,  -0.1309f,-0.4920f,0.8607f,  
	-0.1306f,-0.5016f,0.8552f,  -0.2428f,-0.5442f,0.8031f,  -0.2426f,-0.5539f,0.7964f,  -0.3505f,-0.5866f,0.7301f,  -0.3504f,-0.5961f,0.7224f,  -0.4522f,-0.6179f,0.6432f,  -0.4521f,-0.6270f,0.6344f,  -0.5460f,-0.6372f,0.5440f,  -0.5459f,-0.6455f,0.5342f,  -0.6304f,-0.6434f,0.4343f,  -0.6303f,-0.6507f,0.4236f,  -0.7038f,-0.6361f,0.3164f,  -0.7035f,-0.6419f,0.3050f,  -0.6466f,-0.6226f,0.4408f,  -0.6546f,-0.6171f,0.4367f,  
	-0.7107f,-0.6271f,0.3189f,  -0.7180f,-0.6206f,0.3152f,  -0.7618f,-0.6187f,0.1920f,  -0.7685f,-0.6114f,0.1886f,  -0.7991f,-0.5980f,0.0621f,  -0.8051f,-0.5902f,0.0593f,  -0.8216f,-0.5659f,-0.0682f,  -0.8269f,-0.5578f,-0.0706f,  -0.8291f,-0.5234f,-0.1968f,  -0.8337f,-0.5153f,-0.1986f,  -0.8213f,-0.4715f,-0.3212f,  -0.8252f,-0.4636f,-0.3226f,  -0.7983f,-0.4116f,-0.4396f,  -0.8017f,-0.4041f,-0.4405f,  -0.7607f,-0.3450f,-0.5498f,  
	-0.7636f,-0.3379f,-0.5502f,  -0.7092f,-0.2730f,-0.6500f,  -0.7116f,-0.2665f,-0.6501f,  -0.6447f,-0.1972f,-0.7386f,  -0.6467f,-0.1913f,-0.7384f,  -0.5684f,-0.1189f,-0.8141f,  -0.5701f,-0.1136f,-0.8137f,  -0.4817f,-0.0395f,-0.8754f,  -0.4832f,-0.0349f,-0.8748f,  -0.3862f,0.0395f,-0.9216f,  -0.3875f,0.0435f,-0.9209f,  -0.2836f,0.1167f,-0.9518f,  -0.2847f,0.1202f,-0.9511f,  -0.1757f,0.1910f,-0.9658f,  -0.1767f,0.1939f,-0.9650f,  
	-0.0644f,0.2609f,-0.9632f,  -0.0653f,0.2634f,-0.9625f,  0.0484f,0.3255f,-0.9443f,  0.0475f,0.3276f,-0.9436f,  0.1606f,0.3837f,-0.9094f,  0.1598f,0.3854f,-0.9088f,  0.2704f,0.4345f,-0.8591f,  0.2697f,0.4360f,-0.8586f,  0.3759f,0.4772f,-0.7943f,  0.3752f,0.4784f,-0.7939f,  0.4753f,0.5111f,-0.7162f,  0.4747f,0.5121f,-0.7158f,  0.5668f,0.5356f,-0.6260f,  0.5663f,0.5365f,-0.6257f,  0.6490f,0.5504f,-0.5253f,  
	0.6485f,0.5511f,-0.5250f,  0.7204f,0.5551f,-0.4158f,  0.7201f,0.5558f,-0.4155f,  0.7798f,0.5498f,-0.2993f,  0.7796f,0.5503f,-0.2990f,  0.8263f,0.5344f,-0.1778f,  0.8262f,0.5347f,-0.1775f,  0.8590f,0.5092f,-0.0535f,  0.8590f,0.5092f,-0.0531f,  0.8773f,0.4745f,0.0717f,  0.8775f,0.4741f,0.0721f,  0.8811f,0.4307f,0.1955f,  0.8814f,0.4299f,0.1959f,  0.8701f,0.3786f,0.3157f,  0.8705f,0.3772f,0.3162f,  
	0.8445f,0.3189f,0.4303f,  0.8450f,0.3169f,0.4308f,  0.8048f,0.2525f,0.5372f,  0.8053f,0.2498f,0.5376f,  0.7515f,0.1804f,0.6346f,  0.7521f,0.1770f,0.6348f,  0.6856f,0.1039f,0.7205f,  0.6862f,0.0996f,0.7205f,  0.6082f,0.0242f,0.7935f,  0.6088f,0.0190f,0.7931f,  0.5204f,-0.0573f,0.8520f,  0.5210f,-0.0634f,0.8512f,  0.4239f,-0.1393f,0.8949f,  0.4244f,-0.1463f,0.8936f,  0.3201f,-0.2201f,0.9214f,  
	0.3206f,-0.2280f,0.9194f,  0.2109f,-0.2983f,0.9309f,  0.2113f,-0.3070f,0.9280f,  0.0981f,-0.3724f,0.9229f,  0.0984f,-0.3817f,0.9190f,  -0.0164f,-0.4406f,0.8975f,  -0.0162f,-0.4505f,0.8926f,  -0.1306f,-0.5016f,0.8552f,  -0.1305f,-0.5118f,0.8491f,  -0.2426f,-0.5539f,0.7964f,  -0.2426f,-0.5642f,0.7892f,  -0.3504f,-0.5961f,0.7224f,  -0.3504f,-0.6062f,0.7140f,  -0.4521f,-0.6270f,0.6344f,  -0.4521f,-0.6365f,0.6248f,  
	-0.5459f,-0.6455f,0.5342f,  -0.5460f,-0.6541f,0.5235f,  -0.6303f,-0.6507f,0.4236f,  -0.6302f,-0.6581f,0.4121f,  -0.7035f,-0.6419f,0.3050f,  -0.7034f,-0.6476f,0.2928f,  -0.6546f,-0.6171f,0.4367f,  -0.6619f,-0.6118f,0.4331f,  -0.7180f,-0.6206f,0.3152f,  -0.7248f,-0.6143f,0.3119f,  -0.7685f,-0.6114f,0.1886f,  -0.7747f,-0.6044f,0.1857f,  -0.8051f,-0.5902f,0.0593f,  -0.8106f,-0.5828f,0.0568f,  -0.8269f,-0.5578f,-0.0706f,  
	-0.8319f,-0.5502f,-0.0725f,  -0.8337f,-0.5153f,-0.1986f,  -0.8380f,-0.5077f,-0.2001f,  -0.8252f,-0.4636f,-0.3226f,  -0.8290f,-0.4562f,-0.3236f,  -0.8017f,-0.4041f,-0.4405f,  -0.8049f,-0.3970f,-0.4410f,  -0.7636f,-0.3379f,-0.5502f,  -0.7663f,-0.3313f,-0.5504f,  -0.7116f,-0.2665f,-0.6501f,  -0.7140f,-0.2604f,-0.6500f,  -0.6467f,-0.1913f,-0.7384f,  -0.6487f,-0.1857f,-0.7380f,  -0.5701f,-0.1136f,-0.8137f,  -0.5719f,-0.1086f,-0.8131f,  
	-0.4832f,-0.0349f,-0.8748f,  -0.4847f,-0.0305f,-0.8741f,  -0.3875f,0.0435f,-0.9209f,  -0.3888f,0.0473f,-0.9201f,  -0.2847f,0.1202f,-0.9511f,  -0.2859f,0.1235f,-0.9503f,  -0.1767f,0.1939f,-0.9650f,  -0.1778f,0.1967f,-0.9642f,  -0.0653f,0.2634f,-0.9625f,  -0.0664f,0.2658f,-0.9617f,  0.0475f,0.3276f,-0.9436f,  0.0465f,0.3297f,-0.9429f,  0.1598f,0.3854f,-0.9088f,  0.1589f,0.3873f,-0.9082f,  0.2697f,0.4360f,-0.8586f,  
	0.2688f,0.4376f,-0.8581f,  0.3752f,0.4784f,-0.7939f,  0.3744f,0.4799f,-0.7934f,  0.4747f,0.5121f,-0.7158f,  0.4739f,0.5135f,-0.7154f,  0.5663f,0.5365f,-0.6257f,  0.5656f,0.5377f,-0.6253f,  0.6485f,0.5511f,-0.5250f,  0.6480f,0.5522f,-0.5246f,  0.7201f,0.5558f,-0.4155f,  0.7196f,0.5567f,-0.4151f,  0.7796f,0.5503f,-0.2990f,  0.7793f,0.5510f,-0.2985f,  0.8262f,0.5347f,-0.1775f,  0.8260f,0.5352f,-0.1770f,  
	0.8590f,0.5092f,-0.0531f,  0.8590f,0.5093f,-0.0525f,  0.8775f,0.4741f,0.0721f,  0.8776f,0.4738f,0.0728f,  0.8814f,0.4299f,0.1959f,  0.8816f,0.4292f,0.1966f,  0.8705f,0.3772f,0.3162f,  0.8708f,0.3759f,0.3169f,  0.8450f,0.3169f,0.4308f,  0.8454f,0.3149f,0.4315f,  0.8053f,0.2498f,0.5376f,  0.8058f,0.2470f,0.5382f,  0.7521f,0.1770f,0.6348f,  0.7526f,0.1733f,0.6352f,  0.6862f,0.0996f,0.7205f,  
	0.6867f,0.0951f,0.7207f,  0.6088f,0.0190f,0.7931f,  0.6093f,0.0135f,0.7928f,  0.5210f,-0.0634f,0.8512f,  0.5215f,-0.0700f,0.8504f,  0.4244f,-0.1463f,0.8936f,  0.4248f,-0.1538f,0.8921f,  0.3206f,-0.2280f,0.9194f,  0.3209f,-0.2364f,0.9171f,  0.2113f,-0.3070f,0.9280f,  0.2115f,-0.3163f,0.9248f,  0.0984f,-0.3817f,0.9190f,  0.0985f,-0.3917f,0.9148f,  -0.0162f,-0.4505f,0.8926f,  -0.0162f,-0.4610f,0.8873f,  
	-0.1305f,-0.5118f,0.8491f,  -0.1307f,-0.5226f,0.8425f,  -0.2426f,-0.5642f,0.7892f,  -0.2427f,-0.5750f,0.7813f,  -0.3504f,-0.6062f,0.7140f,  -0.3506f,-0.6167f,0.7048f,  -0.4521f,-0.6365f,0.6248f,  -0.4523f,-0.6464f,0.6145f,  -0.5460f,-0.6541f,0.5235f,  -0.5461f,-0.6630f,0.5121f,  -0.6302f,-0.6581f,0.4121f,  -0.6303f,-0.6656f,0.3997f,  -0.7034f,-0.6476f,0.2928f,  -0.7034f,-0.6534f,0.2798f,  -0.6619f,-0.6118f,0.4331f,  
	-0.6687f,-0.6066f,0.4299f,  -0.7248f,-0.6143f,0.3119f,  -0.7311f,-0.6082f,0.3090f,  -0.7747f,-0.6044f,0.1857f,  -0.7805f,-0.5977f,0.1832f,  -0.8106f,-0.5828f,0.0568f,  -0.8158f,-0.5758f,0.0548f,  -0.8319f,-0.5502f,-0.0725f,  -0.8364f,-0.5431f,-0.0741f,  -0.8380f,-0.5077f,-0.2001f,  -0.8420f,-0.5005f,-0.2012f,  -0.8290f,-0.4562f,-0.3236f,  -0.8324f,-0.4493f,-0.3243f,  -0.8049f,-0.3970f,-0.4410f,  -0.8080f,-0.3904f,-0.4414f,  
	-0.7663f,-0.3313f,-0.5504f,  -0.7690f,-0.3251f,-0.5505f,  -0.7140f,-0.2604f,-0.6500f,  -0.7163f,-0.2547f,-0.6497f,  -0.6487f,-0.1857f,-0.7380f,  -0.6508f,-0.1805f,-0.7375f,  -0.5719f,-0.1086f,-0.8131f,  -0.5737f,-0.1039f,-0.8125f,  -0.4847f,-0.0305f,-0.8741f,  -0.4863f,-0.0263f,-0.8734f,  -0.3888f,0.0473f,-0.9201f,  -0.3903f,0.0510f,-0.9193f,  -0.2859f,0.1235f,-0.9503f,  -0.2873f,0.1267f,-0.9494f,  -0.1778f,0.1967f,-0.9642f,  
	-0.1791f,0.1996f,-0.9634f,  -0.0664f,0.2658f,-0.9617f,  -0.0676f,0.2684f,-0.9609f,  0.0465f,0.3297f,-0.9429f,  0.0453f,0.3320f,-0.9422f,  0.1589f,0.3873f,-0.9082f,  0.1577f,0.3893f,-0.9075f,  0.2688f,0.4376f,-0.8581f,  0.2677f,0.4395f,-0.8574f,  0.3744f,0.4799f,-0.7934f,  0.3734f,0.4816f,-0.7929f,  0.4739f,0.5135f,-0.7154f,  0.4730f,0.5151f,-0.7148f,  0.5656f,0.5377f,-0.6253f,  0.5648f,0.5392f,-0.6247f,  
	0.6480f,0.5522f,-0.5246f,  0.6473f,0.5536f,-0.5240f,  0.7196f,0.5567f,-0.4151f,  0.7190f,0.5579f,-0.4144f,  0.7793f,0.5510f,-0.2985f,  0.7789f,0.5520f,-0.2978f,  0.8260f,0.5352f,-0.1770f,  0.8257f,0.5359f,-0.1762f,  0.8590f,0.5093f,-0.0525f,  0.8588f,0.5097f,-0.0516f,  0.8776f,0.4738f,0.0728f,  0.8776f,0.4737f,0.0737f,  0.8816f,0.4292f,0.1966f,  0.8817f,0.4285f,0.1976f,  0.8708f,0.3759f,0.3169f,  
	0.8710f,0.3746f,0.3180f,  0.8454f,0.3149f,0.4315f,  0.8456f,0.3128f,0.4325f,  0.8058f,0.2470f,0.5382f,  0.8061f,0.2441f,0.5391f,  0.7526f,0.1733f,0.6352f,  0.7530f,0.1695f,0.6359f,  0.6867f,0.0951f,0.7207f,  0.6871f,0.0902f,0.7210f,  0.6093f,0.0135f,0.7928f,  0.6096f,0.0076f,0.7927f,  0.5215f,-0.0700f,0.8504f,  0.5217f,-0.0769f,0.8497f,  0.4248f,-0.1538f,0.8921f,  0.4249f,-0.1618f,0.8906f,  
	0.3209f,-0.2364f,0.9171f,  0.3209f,-0.2454f,0.9147f,  0.2115f,-0.3163f,0.9248f,  0.2114f,-0.3261f,0.9214f,  0.0985f,-0.3917f,0.9148f,  0.0983f,-0.4023f,0.9102f,  -0.0162f,-0.4610f,0.8873f,  -0.0165f,-0.4721f,0.8814f,  -0.1307f,-0.5226f,0.8425f,  -0.1309f,-0.5339f,0.8353f,  -0.2427f,-0.5750f,0.7813f,  -0.2431f,-0.5863f,0.7728f,  -0.3506f,-0.6167f,0.7048f,  -0.3510f,-0.6276f,0.6949f,  -0.4523f,-0.6464f,0.6145f,  
	-0.4526f,-0.6566f,0.6033f,  -0.5461f,-0.6630f,0.5121f,  -0.5464f,-0.6721f,0.4998f,  -0.6303f,-0.6656f,0.3997f,  -0.6305f,-0.6732f,0.3865f,  -0.7034f,-0.6534f,0.2798f,  -0.7035f,-0.6591f,0.2659f,  -0.6687f,-0.6066f,0.4299f,  -0.6750f,-0.6017f,0.4270f,  -0.7311f,-0.6082f,0.3090f,  -0.7369f,-0.6025f,0.3065f,  -0.7805f,-0.5977f,0.1832f,  -0.7857f,-0.5914f,0.1812f,  -0.8158f,-0.5758f,0.0548f,  -0.8205f,-0.5691f,0.0531f,  
	-0.8364f,-0.5431f,-0.0741f,  -0.8406f,-0.5363f,-0.0754f,  -0.8420f,-0.5005f,-0.2012f,  -0.8457f,-0.4939f,-0.2021f,  -0.8324f,-0.4493f,-0.3243f,  -0.8357f,-0.4428f,-0.3248f,  -0.8080f,-0.3904f,-0.4414f,  -0.8109f,-0.3842f,-0.4415f,  -0.7690f,-0.3251f,-0.5505f,  -0.7715f,-0.3193f,-0.5503f,  -0.7163f,-0.2547f,-0.6497f,  -0.7185f,-0.2493f,-0.6493f,  -0.6508f,-0.1805f,-0.7375f,  -0.6528f,-0.1756f,-0.7369f,  -0.5737f,-0.1039f,-0.8125f,  
	-0.5755f,-0.0995f,-0.8117f,  -0.4863f,-0.0263f,-0.8734f,  -0.4881f,-0.0224f,-0.8725f,  -0.3903f,0.0510f,-0.9193f,  -0.3919f,0.0546f,-0.9184f,  -0.2873f,0.1267f,-0.9494f,  -0.2889f,0.1299f,-0.9485f,  -0.1791f,0.1996f,-0.9634f,  -0.1806f,0.2024f,-0.9625f,  -0.0676f,0.2684f,-0.9609f,  -0.0690f,0.2710f,-0.9601f,  0.0453f,0.3320f,-0.9422f,  0.0440f,0.3343f,-0.9414f,  0.1577f,0.3893f,-0.9075f,  0.1565f,0.3915f,-0.9068f,  
	0.2677f,0.4395f,-0.8574f,  0.2665f,0.4415f,-0.8567f,  0.3734f,0.4816f,-0.7929f,  0.3723f,0.4836f,-0.7922f,  0.4730f,0.5151f,-0.7148f,  0.4720f,0.5169f,-0.7142f,  0.5648f,0.5392f,-0.6247f,  0.5639f,0.5410f,-0.6240f,  0.6473f,0.5536f,-0.5240f,  0.6465f,0.5552f,-0.5233f,  0.7190f,0.5579f,-0.4144f,  0.7184f,0.5594f,-0.4136f,  0.7789f,0.5520f,-0.2978f,  0.7783f,0.5532f,-0.2969f,  0.8257f,0.5359f,-0.1762f,  
	0.8253f,0.5368f,-0.1751f,  0.8588f,0.5097f,-0.0516f,  0.8586f,0.5102f,-0.0504f,  0.8776f,0.4737f,0.0737f,  0.8775f,0.4738f,0.0750f,  0.8817f,0.4285f,0.1976f,  0.8816f,0.4279f,0.1990f,  0.8710f,0.3746f,0.3180f,  0.8710f,0.3733f,0.3193f,  0.8456f,0.3128f,0.4325f,  0.8458f,0.3107f,0.4337f,  0.8061f,0.2441f,0.5391f,  0.8063f,0.2410f,0.5402f,  0.7530f,0.1695f,0.6359f,  0.7532f,0.1654f,0.6367f,  
	0.6871f,0.0902f,0.7210f,  0.6873f,0.0850f,0.7214f,  0.6096f,0.0076f,0.7927f,  0.6097f,0.0012f,0.7926f,  0.5217f,-0.0769f,0.8497f,  0.5218f,-0.0844f,0.8489f,  0.4249f,-0.1618f,0.8906f,  0.4249f,-0.1704f,0.8890f,  0.3209f,-0.2454f,0.9147f,  0.3208f,-0.2550f,0.9122f,  0.2114f,-0.3261f,0.9214f,  0.2112f,-0.3366f,0.9176f,  0.0983f,-0.4023f,0.9102f,  0.0980f,-0.4135f,0.9052f,  -0.0165f,-0.4721f,0.8814f,  
	-0.0169f,-0.4838f,0.8750f,  -0.1309f,-0.5339f,0.8353f,  -0.1314f,-0.5459f,0.8275f,  -0.2431f,-0.5863f,0.7728f,  -0.2436f,-0.5981f,0.7635f,  -0.3510f,-0.6276f,0.6949f,  -0.3514f,-0.6390f,0.6842f,  -0.4526f,-0.6566f,0.6033f,  -0.4531f,-0.6671f,0.5913f,  -0.5464f,-0.6721f,0.4998f,  -0.5467f,-0.6814f,0.4866f,  -0.6305f,-0.6732f,0.3865f,  -0.6307f,-0.6808f,0.3724f,  -0.7035f,-0.6591f,0.2659f,  -0.7036f,-0.6647f,0.2512f,  
	-0.6750f,-0.6017f,0.4270f,  -0.6808f,-0.5969f,0.4246f,  -0.7369f,-0.6025f,0.3065f,  -0.7422f,-0.5970f,0.3044f,  -0.7857f,-0.5914f,0.1812f,  -0.7906f,-0.5855f,0.1794f,  -0.8205f,-0.5691f,0.0531f,  -0.8249f,-0.5629f,0.0517f,  -0.8406f,-0.5363f,-0.0754f,  -0.8446f,-0.5300f,-0.0763f,  -0.8457f,-0.4939f,-0.2021f,  -0.8492f,-0.4876f,-0.2026f,  -0.8357f,-0.4428f,-0.3248f,  -0.8388f,-0.4367f,-0.3250f,  -0.8109f,-0.3842f,-0.4415f,  
	-0.8136f,-0.3784f,-0.4414f,  -0.7715f,-0.3193f,-0.5503f,  -0.7740f,-0.3138f,-0.5499f,  -0.7185f,-0.2493f,-0.6493f,  -0.7208f,-0.2443f,-0.6487f,  -0.6528f,-0.1756f,-0.7369f,  -0.6548f,-0.1710f,-0.7362f,  -0.5755f,-0.0995f,-0.8117f,  -0.5774f,-0.0953f,-0.8109f,  -0.4881f,-0.0224f,-0.8725f,  -0.4898f,-0.0186f,-0.8716f,  -0.3919f,0.0546f,-0.9184f,  -0.3936f,0.0580f,-0.9174f,  -0.2889f,0.1299f,-0.9485f,  -0.2905f,0.1330f,-0.9476f,  
	-0.1806f,0.2024f,-0.9625f,  -0.1822f,0.2053f,-0.9616f,  -0.0690f,0.2710f,-0.9601f,  -0.0705f,0.2736f,-0.9592f,  0.0440f,0.3343f,-0.9414f,  0.0425f,0.3369f,-0.9406f,  0.1565f,0.3915f,-0.9068f,  0.1551f,0.3939f,-0.9060f,  0.2665f,0.4415f,-0.8567f,  0.2652f,0.4439f,-0.8560f,  0.3723f,0.4836f,-0.7922f,  0.3710f,0.4858f,-0.7914f,  0.4720f,0.5169f,-0.7142f,  0.4708f,0.5191f,-0.7133f,  0.5639f,0.5410f,-0.6240f,  
	0.5629f,0.5431f,-0.6231f,  0.6465f,0.5552f,-0.5233f,  0.6456f,0.5572f,-0.5223f,  0.7184f,0.5594f,-0.4136f,  0.7176f,0.5611f,-0.4125f,  0.7783f,0.5532f,-0.2969f,  0.7777f,0.5547f,-0.2956f,  0.8253f,0.5368f,-0.1751f,  0.8249f,0.5380f,-0.1738f,  0.8586f,0.5102f,-0.0504f,  0.8582f,0.5109f,-0.0489f,  0.8775f,0.4738f,0.0750f,  0.8772f,0.4739f,0.0766f,  0.8816f,0.4279f,0.1990f,  0.8815f,0.4274f,0.2006f,  
	0.8710f,0.3733f,0.3193f,  0.8710f,0.3719f,0.3210f,  0.8458f,0.3107f,0.4337f,  0.8458f,0.3084f,0.4353f,  0.8063f,0.2410f,0.5402f,  0.8063f,0.2377f,0.5416f,  0.7532f,0.1654f,0.6367f,  0.7532f,0.1609f,0.6378f,  0.6873f,0.0850f,0.7214f,  0.6873f,0.0794f,0.7221f,  0.6097f,0.0012f,0.7926f,  0.6097f,-0.0056f,0.7926f,  0.5218f,-0.0844f,0.8489f,  0.5216f,-0.0924f,0.8481f,  0.4249f,-0.1704f,0.8890f,  
	0.4247f,-0.1796f,0.8873f,  0.3208f,-0.2550f,0.9122f,  0.3205f,-0.2653f,0.9093f,  0.2112f,-0.3366f,0.9176f,  0.2108f,-0.3478f,0.9136f,  0.0980f,-0.4135f,0.9052f,  0.0975f,-0.4253f,0.8998f,  -0.0169f,-0.4838f,0.8750f,  -0.0175f,-0.4961f,0.8681f,  -0.1314f,-0.5459f,0.8275f,  -0.1321f,-0.5584f,0.8190f,  -0.2436f,-0.5981f,0.7635f,  -0.2442f,-0.6104f,0.7535f,  -0.3514f,-0.6390f,0.6842f,  -0.3521f,-0.6508f,0.6727f,  
	-0.4531f,-0.6671f,0.5913f,  -0.4537f,-0.6780f,0.5784f,  -0.5467f,-0.6814f,0.4866f,  -0.5472f,-0.6909f,0.4725f,  -0.6307f,-0.6808f,0.3724f,  -0.6311f,-0.6885f,0.3574f,  -0.7036f,-0.6647f,0.2512f,  -0.7038f,-0.6702f,0.2357f,  -0.6808f,-0.5969f,0.4246f,  -0.6861f,-0.5923f,0.4225f,  -0.7422f,-0.5970f,0.3044f,  -0.7471f,-0.5918f,0.3026f,  -0.7906f,-0.5855f,0.1794f,  -0.7950f,-0.5799f,0.1780f,  -0.8249f,-0.5629f,0.0517f,  
	-0.8289f,-0.5571f,0.0507f,  -0.8446f,-0.5300f,-0.0763f,  -0.8482f,-0.5241f,-0.0770f,  -0.8492f,-0.4876f,-0.2026f,  -0.8525f,-0.4818f,-0.2030f,  -0.8388f,-0.4367f,-0.3250f,  -0.8418f,-0.4311f,-0.3250f,  -0.8136f,-0.3784f,-0.4414f,  -0.8162f,-0.3730f,-0.4411f,  -0.7740f,-0.3138f,-0.5499f,  -0.7764f,-0.3088f,-0.5494f,  -0.7208f,-0.2443f,-0.6487f,  -0.7230f,-0.2396f,-0.6480f,  -0.6548f,-0.1710f,-0.7362f,  -0.6569f,-0.1667f,-0.7353f,  
	-0.5774f,-0.0953f,-0.8109f,  -0.5793f,-0.0914f,-0.8099f,  -0.4898f,-0.0186f,-0.8716f,  -0.4917f,-0.0149f,-0.8707f,  -0.3936f,0.0580f,-0.9174f,  -0.3954f,0.0614f,-0.9165f,  -0.2905f,0.1330f,-0.9476f,  -0.2922f,0.1362f,-0.9466f,  -0.1822f,0.2053f,-0.9616f,  -0.1838f,0.2083f,-0.9606f,  -0.0705f,0.2736f,-0.9592f,  -0.0721f,0.2764f,-0.9583f,  0.0425f,0.3369f,-0.9406f,  0.0410f,0.3395f,-0.9397f,  0.1551f,0.3939f,-0.9060f,  
	0.1536f,0.3965f,-0.9051f,  0.2652f,0.4439f,-0.8560f,  0.2638f,0.4464f,-0.8550f,  0.3710f,0.4858f,-0.7914f,  0.3697f,0.4884f,-0.7904f,  0.4708f,0.5191f,-0.7133f,  0.4696f,0.5216f,-0.7123f,  0.5629f,0.5431f,-0.6231f,  0.5618f,0.5454f,-0.6220f,  0.6456f,0.5572f,-0.5223f,  0.6446f,0.5594f,-0.5210f,  0.7176f,0.5611f,-0.4125f,  0.7168f,0.5632f,-0.4111f,  0.7777f,0.5547f,-0.2956f,  0.7771f,0.5565f,-0.2941f,  
	0.8249f,0.5380f,-0.1738f,  0.8243f,0.5393f,-0.1721f,  0.8582f,0.5109f,-0.0489f,  0.8578f,0.5118f,-0.0471f,  0.8772f,0.4739f,0.0766f,  0.8769f,0.4741f,0.0786f,  0.8815f,0.4274f,0.2006f,  0.8813f,0.4268f,0.2027f,  0.8710f,0.3719f,0.3210f,  0.8709f,0.3705f,0.3230f,  0.8458f,0.3084f,0.4353f,  0.8457f,0.3059f,0.4372f,  0.8063f,0.2377f,0.5416f,  0.8063f,0.2341f,0.5433f,  0.7532f,0.1609f,0.6378f,  
	0.7531f,0.1562f,0.6391f,  0.6873f,0.0794f,0.7221f,  0.6871f,0.0733f,0.7228f,  0.6097f,-0.0056f,0.7926f,  0.6094f,-0.0129f,0.7927f,  0.5216f,-0.0924f,0.8481f,  0.5213f,-0.1010f,0.8474f,  0.4247f,-0.1796f,0.8873f,  0.4243f,-0.1894f,0.8855f,  0.3205f,-0.2653f,0.9093f,  0.3199f,-0.2762f,0.9063f,  0.2108f,-0.3478f,0.9136f,  0.2102f,-0.3596f,0.9091f,  0.0975f,-0.4253f,0.8998f,  0.0968f,-0.4379f,0.8938f,  
	-0.0175f,-0.4961f,0.8681f,  -0.0183f,-0.5091f,0.8605f,  -0.1321f,-0.5584f,0.8190f,  -0.1328f,-0.5715f,0.8098f,  -0.2442f,-0.6104f,0.7535f,  -0.2450f,-0.6233f,0.7426f,  -0.3521f,-0.6508f,0.6727f,  -0.3528f,-0.6630f,0.6603f,  -0.4537f,-0.6780f,0.5784f,  -0.4544f,-0.6891f,0.5645f,  -0.5472f,-0.6909f,0.4725f,  -0.5478f,-0.7005f,0.4575f,  -0.6311f,-0.6885f,0.3574f,  -0.6315f,-0.6961f,0.3416f,  -0.7038f,-0.6702f,0.2357f,  
	-0.7040f,-0.6754f,0.2195f,  -0.6861f,-0.5923f,0.4225f,  -0.6909f,-0.5880f,0.4207f,  -0.7471f,-0.5918f,0.3026f,  -0.7516f,-0.5869f,0.3012f,  -0.7950f,-0.5799f,0.1780f,  -0.7991f,-0.5746f,0.1769f,  -0.8289f,-0.5571f,0.0507f,  -0.8326f,-0.5516f,0.0499f,  -0.8482f,-0.5241f,-0.0770f,  -0.8515f,-0.5186f,-0.0774f,  -0.8525f,-0.4818f,-0.2030f,  -0.8555f,-0.4763f,-0.2030f,  -0.8418f,-0.4311f,-0.3250f,  -0.8445f,-0.4258f,-0.3248f,  
	-0.8162f,-0.3730f,-0.4411f,  -0.8188f,-0.3680f,-0.4407f,  -0.7764f,-0.3088f,-0.5494f,  -0.7787f,-0.3041f,-0.5488f,  -0.7230f,-0.2396f,-0.6480f,  -0.7251f,-0.2352f,-0.6472f,  -0.6569f,-0.1667f,-0.7353f,  -0.6589f,-0.1626f,-0.7344f,  -0.5793f,-0.0914f,-0.8099f,  -0.5813f,-0.0875f,-0.8090f,  -0.4917f,-0.0149f,-0.8707f,  -0.4935f,-0.0113f,-0.8696f,  -0.3954f,0.0614f,-0.9165f,  -0.3972f,0.0647f,-0.9154f,  -0.2922f,0.1362f,-0.9466f,  
	-0.2940f,0.1393f,-0.9456f,  -0.1838f,0.2083f,-0.9606f,  -0.1856f,0.2113f,-0.9596f,  -0.0721f,0.2764f,-0.9583f,  -0.0738f,0.2794f,-0.9573f,  0.0410f,0.3395f,-0.9397f,  0.0393f,0.3424f,-0.9387f,  0.1536f,0.3965f,-0.9051f,  0.1520f,0.3994f,-0.9041f,  0.2638f,0.4464f,-0.8550f,  0.2623f,0.4493f,-0.8540f,  0.3697f,0.4884f,-0.7904f,  0.3683f,0.4912f,-0.7893f,  0.4696f,0.5216f,-0.7123f,  0.4683f,0.5244f,-0.7111f,  
	0.5618f,0.5454f,-0.6220f,  0.5606f,0.5482f,-0.6207f,  0.6446f,0.5594f,-0.5210f,  0.6436f,0.5620f,-0.5195f,  0.7168f,0.5632f,-0.4111f,  0.7159f,0.5655f,-0.4094f,  0.7771f,0.5565f,-0.2941f,  0.7763f,0.5585f,-0.2922f,  0.8243f,0.5393f,-0.1721f,  0.8237f,0.5409f,-0.1700f,  0.8578f,0.5118f,-0.0471f,  0.8574f,0.5128f,-0.0449f,  0.8769f,0.4741f,0.0786f,  0.8766f,0.4744f,0.0809f,  0.8813f,0.4268f,0.2027f,  
	0.8810f,0.4263f,0.2051f,  0.8709f,0.3705f,0.3230f,  0.8706f,0.3690f,0.3253f,  0.8457f,0.3059f,0.4372f,  0.8455f,0.3033f,0.4394f,  0.8063f,0.2341f,0.5433f,  0.8060f,0.2303f,0.5452f,  0.7531f,0.1562f,0.6391f,  0.7528f,0.1510f,0.6406f,  0.6871f,0.0733f,0.7228f,  0.6868f,0.0669f,0.7237f,  0.6094f,-0.0129f,0.7927f,  0.6090f,-0.0208f,0.7929f,  0.5213f,-0.1010f,0.8474f,  0.5208f,-0.1102f,0.8465f,  
	0.4243f,-0.1894f,0.8855f,  0.4236f,-0.1998f,0.8835f,  0.3199f,-0.2762f,0.9063f,  0.3192f,-0.2877f,0.9029f,  0.2102f,-0.3596f,0.9091f,  0.2093f,-0.3721f,0.9043f,  0.0968f,-0.4379f,0.8938f,  0.0958f,-0.4511f,0.8873f,  -0.0183f,-0.5091f,0.8605f,  -0.0192f,-0.5227f,0.8523f,  -0.1328f,-0.5715f,0.8098f,  -0.1338f,-0.5852f,0.7998f,  -0.2450f,-0.6233f,0.7426f,  -0.2460f,-0.6366f,0.7309f,  -0.3528f,-0.6630f,0.6603f,  
	-0.3537f,-0.6756f,0.6469f,  -0.4544f,-0.6891f,0.5645f,  -0.4551f,-0.7005f,0.5498f,  -0.5478f,-0.7005f,0.4575f,  -0.5484f,-0.7101f,0.4416f,  -0.6315f,-0.6961f,0.3416f,  -0.6319f,-0.7036f,0.3249f,  -0.7040f,-0.6754f,0.2195f,  -0.7043f,-0.6805f,0.2025f,  -0.6909f,-0.5880f,0.4207f,  -0.6952f,-0.5839f,0.4192f,  -0.7516f,-0.5869f,0.3012f,  -0.7556f,-0.5823f,0.3000f,  -0.7991f,-0.5746f,0.1769f,  -0.8028f,-0.5697f,0.1760f,  
	-0.8326f,-0.5516f,0.0499f,  -0.8360f,-0.5465f,0.0494f,  -0.8515f,-0.5186f,-0.0774f,  -0.8546f,-0.5135f,-0.0776f,  -0.8555f,-0.4763f,-0.2030f,  -0.8583f,-0.4713f,-0.2029f,  -0.8445f,-0.4258f,-0.3248f,  -0.8471f,-0.4209f,-0.3245f,  -0.8188f,-0.3680f,-0.4407f,  -0.8211f,-0.3633f,-0.4401f,  -0.7787f,-0.3041f,-0.5488f,  -0.7810f,-0.2996f,-0.5480f,  -0.7251f,-0.2352f,-0.6472f,  -0.7273f,-0.2310f,-0.6463f,  -0.6589f,-0.1626f,-0.7344f,  
	-0.6610f,-0.1587f,-0.7334f,  -0.5813f,-0.0875f,-0.8090f,  -0.5833f,-0.0839f,-0.8079f,  -0.4935f,-0.0113f,-0.8696f,  -0.4955f,-0.0079f,-0.8686f,  -0.3972f,0.0647f,-0.9154f,  -0.3991f,0.0680f,-0.9144f,  -0.2940f,0.1393f,-0.9456f,  -0.2958f,0.1425f,-0.9445f,  -0.1856f,0.2113f,-0.9596f,  -0.1874f,0.2144f,-0.9586f,  -0.0738f,0.2794f,-0.9573f,  -0.0756f,0.2825f,-0.9563f,  0.0393f,0.3424f,-0.9387f,  0.0376f,0.3455f,-0.9377f,  
	0.1520f,0.3994f,-0.9041f,  0.1504f,0.4025f,-0.9030f,  0.2623f,0.4493f,-0.8540f,  0.2607f,0.4524f,-0.8528f,  0.3683f,0.4912f,-0.7893f,  0.3669f,0.4944f,-0.7880f,  0.4683f,0.5244f,-0.7111f,  0.4670f,0.5275f,-0.7097f,  0.5606f,0.5482f,-0.6207f,  0.5594f,0.5512f,-0.6191f,  0.6436f,0.5620f,-0.5195f,  0.6426f,0.5649f,-0.5177f,  0.7159f,0.5655f,-0.4094f,  0.7150f,0.5681f,-0.4074f,  0.7763f,0.5585f,-0.2922f,  
	0.7755f,0.5608f,-0.2900f,  0.8237f,0.5409f,-0.1700f,  0.8231f,0.5426f,-0.1675f,  0.8574f,0.5128f,-0.0449f,  0.8568f,0.5139f,-0.0422f,  0.8766f,0.4744f,0.0809f,  0.8761f,0.4748f,0.0837f,  0.8810f,0.4263f,0.2051f,  0.8807f,0.4257f,0.2079f,  0.8706f,0.3690f,0.3253f,  0.8703f,0.3673f,0.3281f,  0.8455f,0.3033f,0.4394f,  0.8452f,0.3005f,0.4420f,  0.8060f,0.2303f,0.5452f,  0.8057f,0.2261f,0.5475f,  
	0.7528f,0.1510f,0.6406f,  0.7524f,0.1455f,0.6424f,  0.6868f,0.0669f,0.7237f,  0.6863f,0.0599f,0.7248f,  0.6090f,-0.0208f,0.7929f,  0.6085f,-0.0292f,0.7930f,  0.5208f,-0.1102f,0.8465f,  0.5201f,-0.1201f,0.8456f,  0.4236f,-0.1998f,0.8835f,  0.4229f,-0.2110f,0.8813f,  0.3192f,-0.2877f,0.9029f,  0.3183f,-0.3001f,0.8992f,  0.2093f,-0.3721f,0.9043f,  0.2083f,-0.3854f,0.8989f,  0.0958f,-0.4511f,0.8873f,  
	0.0948f,-0.4650f,0.8802f,  -0.0192f,-0.5227f,0.8523f,  -0.0203f,-0.5370f,0.8433f,  -0.1338f,-0.5852f,0.7998f,  -0.1349f,-0.5994f,0.7890f,  -0.2460f,-0.6366f,0.7309f,  -0.2470f,-0.6505f,0.7182f,  -0.3537f,-0.6756f,0.6469f,  -0.3547f,-0.6885f,0.6326f,  -0.4551f,-0.7005f,0.5498f,  -0.4559f,-0.7120f,0.5340f,  -0.5484f,-0.7101f,0.4416f,  -0.5491f,-0.7199f,0.4247f,  -0.6319f,-0.7036f,0.3249f,  -0.6324f,-0.7111f,0.3073f,  
	-0.7043f,-0.6805f,0.2025f,  -0.7046f,-0.6852f,0.1847f,  -0.6952f,-0.5839f,0.4192f,  -0.6992f,-0.5800f,0.4180f,  -0.7556f,-0.5823f,0.3000f,  -0.7592f,-0.5780f,0.2991f,  -0.8028f,-0.5697f,0.1760f,  -0.8062f,-0.5651f,0.1755f,  -0.8360f,-0.5465f,0.0494f,  -0.8391f,-0.5418f,0.0492f,  -0.8546f,-0.5135f,-0.0776f,  -0.8574f,-0.5087f,-0.0776f,  -0.8583f,-0.4713f,-0.2029f,  -0.8609f,-0.4666f,-0.2027f,  -0.8471f,-0.4209f,-0.3245f,  
	-0.8495f,-0.4164f,-0.3240f,  -0.8211f,-0.3633f,-0.4401f,  -0.8234f,-0.3590f,-0.4394f,  -0.7810f,-0.2996f,-0.5480f,  -0.7831f,-0.2955f,-0.5472f,  -0.7273f,-0.2310f,-0.6463f,  -0.7293f,-0.2271f,-0.6454f,  -0.6610f,-0.1587f,-0.7334f,  -0.6630f,-0.1550f,-0.7324f,  -0.5833f,-0.0839f,-0.8079f,  -0.5852f,-0.0803f,-0.8069f,  -0.4955f,-0.0079f,-0.8686f,  -0.4974f,-0.0045f,-0.8675f,  -0.3991f,0.0680f,-0.9144f,  -0.4010f,0.0713f,-0.9133f,  
	-0.2958f,0.1425f,-0.9445f,  -0.2977f,0.1458f,-0.9435f,  -0.1874f,0.2144f,-0.9586f,  -0.1892f,0.2177f,-0.9575f,  -0.0756f,0.2825f,-0.9563f,  -0.0774f,0.2857f,-0.9552f,  0.0376f,0.3455f,-0.9377f,  0.0359f,0.3488f,-0.9365f,  0.1504f,0.4025f,-0.9030f,  0.1487f,0.4059f,-0.9017f,  0.2607f,0.4524f,-0.8528f,  0.2592f,0.4559f,-0.8515f,  0.3669f,0.4944f,-0.7880f,  0.3654f,0.4978f,-0.7865f,  0.4670f,0.5275f,-0.7097f,  
	0.4657f,0.5309f,-0.7080f,  0.5594f,0.5512f,-0.6191f,  0.5582f,0.5545f,-0.6172f,  0.6426f,0.5649f,-0.5177f,  0.6415f,0.5680f,-0.5156f,  0.7150f,0.5681f,-0.4074f,  0.7141f,0.5710f,-0.4050f,  0.7755f,0.5608f,-0.2900f,  0.7747f,0.5632f,-0.2873f,  0.8231f,0.5426f,-0.1675f,  0.8224f,0.5446f,-0.1646f,  0.8568f,0.5139f,-0.0422f,  0.8562f,0.5151f,-0.0391f,  0.8761f,0.4748f,0.0837f,  0.8756f,0.4751f,0.0869f,  
	0.8807f,0.4257f,0.2079f,  0.8802f,0.4250f,0.2111f,  0.8703f,0.3673f,0.3281f,  0.8699f,0.3655f,0.3313f,  0.8452f,0.3005f,0.4420f,  0.8447f,0.2974f,0.4450f,  0.8057f,0.2261f,0.5475f,  0.8052f,0.2216f,0.5500f,  0.7524f,0.1455f,0.6424f,  0.7519f,0.1395f,0.6444f,  0.6863f,0.0599f,0.7248f,  0.6857f,0.0524f,0.7260f,  0.6085f,-0.0292f,0.7930f,  0.6077f,-0.0382f,0.7933f,  0.5201f,-0.1201f,0.8456f,  
	0.5193f,-0.1306f,0.8446f,  0.4229f,-0.2110f,0.8813f,  0.4219f,-0.2228f,0.8788f,  0.3183f,-0.3001f,0.8992f,  0.3172f,-0.3131f,0.8952f,  0.2083f,-0.3854f,0.8989f,  0.2072f,-0.3994f,0.8931f,  0.0948f,-0.4650f,0.8802f,  0.0936f,-0.4797f,0.8724f,  -0.0203f,-0.5370f,0.8433f,  -0.0215f,-0.5520f,0.8336f,  -0.1349f,-0.5994f,0.7890f,  -0.1361f,-0.6143f,0.7773f,  -0.2470f,-0.6505f,0.7182f,  -0.2481f,-0.6648f,0.7046f,  
	-0.3547f,-0.6885f,0.6326f,  -0.3557f,-0.7017f,0.6173f,  -0.4559f,-0.7120f,0.5340f,  -0.4568f,-0.7237f,0.5172f,  -0.5491f,-0.7199f,0.4247f,  -0.5497f,-0.7296f,0.4069f,  -0.6324f,-0.7111f,0.3073f,  -0.6329f,-0.7184f,0.2888f,  -0.7046f,-0.6852f,0.1847f,  -0.7049f,-0.6895f,0.1662f,  -0.6992f,-0.5800f,0.4180f,  -0.7027f,-0.5764f,0.4171f,  -0.7592f,-0.5780f,0.2991f,  -0.7625f,-0.5740f,0.2985f,  -0.8062f,-0.5651f,0.1755f,  
	-0.8092f,-0.5609f,0.1751f,  -0.8391f,-0.5418f,0.0492f,  -0.8419f,-0.5374f,0.0491f,  -0.8574f,-0.5087f,-0.0776f,  -0.8600f,-0.5043f,-0.0774f,  -0.8609f,-0.4666f,-0.2027f,  -0.8634f,-0.4623f,-0.2022f,  -0.8495f,-0.4164f,-0.3240f,  -0.8518f,-0.4122f,-0.3233f,  -0.8234f,-0.3590f,-0.4394f,  -0.8256f,-0.3550f,-0.4386f,  -0.7831f,-0.2955f,-0.5472f,  -0.7852f,-0.2916f,-0.5462f,  -0.7293f,-0.2271f,-0.6454f,  -0.7314f,-0.2234f,-0.6444f,  
	-0.6630f,-0.1550f,-0.7324f,  -0.6650f,-0.1514f,-0.7314f,  -0.5852f,-0.0803f,-0.8069f,  -0.5872f,-0.0769f,-0.8058f,  -0.4974f,-0.0045f,-0.8675f,  -0.4994f,-0.0011f,-0.8664f,  -0.4010f,0.0713f,-0.9133f,  -0.4030f,0.0746f,-0.9122f,  -0.2977f,0.1458f,-0.9435f,  -0.2996f,0.1491f,-0.9423f,  -0.1892f,0.2177f,-0.9575f,  -0.1911f,0.2210f,-0.9564f,  -0.0774f,0.2857f,-0.9552f,  -0.0792f,0.2892f,-0.9540f,  0.0359f,0.3488f,-0.9365f,  
	0.0341f,0.3524f,-0.9352f,  0.1487f,0.4059f,-0.9017f,  0.1470f,0.4096f,-0.9004f,  0.2592f,0.4559f,-0.8515f,  0.2576f,0.4596f,-0.8500f,  0.3654f,0.4978f,-0.7865f,  0.3639f,0.5016f,-0.7848f,  0.4657f,0.5309f,-0.7080f,  0.4643f,0.5347f,-0.7060f,  0.5582f,0.5545f,-0.6172f,  0.5570f,0.5582f,-0.6150f,  0.6415f,0.5680f,-0.5156f,  0.6404f,0.5715f,-0.5131f,  0.7141f,0.5710f,-0.4050f,  0.7131f,0.5742f,-0.4022f,  
	0.7747f,0.5632f,-0.2873f,  0.7739f,0.5659f,-0.2842f,  0.8224f,0.5446f,-0.1646f,  0.8217f,0.5467f,-0.1613f,  0.8562f,0.5151f,-0.0391f,  0.8556f,0.5164f,-0.0356f,  0.8756f,0.4751f,0.0869f,  0.8751f,0.4755f,0.0905f,  0.8802f,0.4250f,0.2111f,  0.8797f,0.4243f,0.2148f,  0.8699f,0.3655f,0.3313f,  0.8693f,0.3635f,0.3348f,  0.8447f,0.2974f,0.4450f,  0.8442f,0.2940f,0.4483f,  0.8052f,0.2216f,0.5500f,  
	0.8046f,0.2167f,0.5529f,  0.7519f,0.1395f,0.6444f,  0.7512f,0.1330f,0.6465f,  0.6857f,0.0524f,0.7260f,  0.6849f,0.0442f,0.7273f,  0.6077f,-0.0382f,0.7933f,  0.6068f,-0.0479f,0.7934f,  0.5193f,-0.1306f,0.8446f,  0.5182f,-0.1418f,0.8434f,  0.4219f,-0.2228f,0.8788f,  0.4207f,-0.2355f,0.8761f,  0.3172f,-0.3131f,0.8952f,  0.3160f,-0.3269f,0.8907f,  0.2072f,-0.3994f,0.8931f,  0.2059f,-0.4142f,0.8866f,  
	0.0936f,-0.4797f,0.8724f,  0.0922f,-0.4951f,0.8640f,  -0.0215f,-0.5520f,0.8336f,  -0.0229f,-0.5676f,0.8230f,  -0.1361f,-0.6143f,0.7773f,  -0.1374f,-0.6297f,0.7646f,  -0.2481f,-0.6648f,0.7046f,  -0.2493f,-0.6795f,0.6900f,  -0.3557f,-0.7017f,0.6173f,  -0.3567f,-0.7153f,0.6010f,  -0.4568f,-0.7237f,0.5172f,  -0.4576f,-0.7356f,0.4995f,  -0.5497f,-0.7296f,0.4069f,  -0.5504f,-0.7393f,0.3881f,  -0.6329f,-0.7184f,0.2888f,  
	-0.6334f,-0.7254f,0.2696f,  -0.7049f,-0.6895f,0.1662f,  -0.7052f,-0.6936f,0.1470f,  -0.7027f,-0.5764f,0.4171f,  -0.7058f,-0.5731f,0.4164f,  -0.7625f,-0.5740f,0.2985f,  -0.7655f,-0.5703f,0.2980f,  -0.8092f,-0.5609f,0.1751f,  -0.8119f,-0.5570f,0.1749f,  -0.8419f,-0.5374f,0.0491f,  -0.8444f,-0.5334f,0.0492f,  -0.8600f,-0.5043f,-0.0774f,  -0.8624f,-0.5003f,-0.0771f,  -0.8634f,-0.4623f,-0.2022f,  -0.8656f,-0.4583f,-0.2017f,  
	-0.8518f,-0.4122f,-0.3233f,  -0.8539f,-0.4083f,-0.3226f,  -0.8256f,-0.3550f,-0.4386f,  -0.8277f,-0.3512f,-0.4378f,  -0.7852f,-0.2916f,-0.5462f,  -0.7872f,-0.2880f,-0.5453f,  -0.7314f,-0.2234f,-0.6444f,  -0.7333f,-0.2199f,-0.6433f,  -0.6650f,-0.1514f,-0.7314f,  -0.6669f,-0.1480f,-0.7303f,  -0.5872f,-0.0769f,-0.8058f,  -0.5891f,-0.0735f,-0.8047f,  -0.4994f,-0.0011f,-0.8664f,  -0.5013f,0.0022f,-0.8653f,  -0.4030f,0.0746f,-0.9122f,  
	-0.4049f,0.0780f,-0.9110f,  -0.2996f,0.1491f,-0.9423f,  -0.3015f,0.1526f,-0.9412f,  -0.1911f,0.2210f,-0.9564f,  -0.1930f,0.2246f,-0.9551f,  -0.0792f,0.2892f,-0.9540f,  -0.0811f,0.2929f,-0.9527f,  0.0341f,0.3524f,-0.9352f,  0.0323f,0.3563f,-0.9338f,  0.1470f,0.4096f,-0.9004f,  0.1453f,0.4135f,-0.8988f,  0.2576f,0.4596f,-0.8500f,  0.2560f,0.4637f,-0.8482f,  0.3639f,0.5016f,-0.7848f,  0.3624f,0.5058f,-0.7828f,  
	0.4643f,0.5347f,-0.7060f,  0.4629f,0.5388f,-0.7038f,  0.5570f,0.5582f,-0.6150f,  0.5558f,0.5622f,-0.6124f,  0.6404f,0.5715f,-0.5131f,  0.6393f,0.5753f,-0.5102f,  0.7131f,0.5742f,-0.4022f,  0.7122f,0.5776f,-0.3990f,  0.7739f,0.5659f,-0.2842f,  0.7731f,0.5688f,-0.2807f,  0.8217f,0.5467f,-0.1613f,  0.8209f,0.5489f,-0.1575f,  0.8556f,0.5164f,-0.0356f,  0.8549f,0.5178f,-0.0316f,  0.8751f,0.4755f,0.0905f,  
	0.8744f,0.4758f,0.0947f,  0.8797f,0.4243f,0.2148f,  0.8791f,0.4234f,0.2190f,  0.8693f,0.3635f,0.3348f,  0.8687f,0.3613f,0.3388f,  0.8442f,0.2940f,0.4483f,  0.8435f,0.2902f,0.4519f,  0.8046f,0.2167f,0.5529f,  0.8039f,0.2114f,0.5560f,  0.7512f,0.1330f,0.6465f,  0.7503f,0.1260f,0.6489f,  0.6849f,0.0442f,0.7273f,  0.6839f,0.0355f,0.7287f,  0.6068f,-0.0479f,0.7934f,  0.6057f,-0.0583f,0.7935f,  
	0.5182f,-0.1418f,0.8434f,  0.5170f,-0.1538f,0.8421f,  0.4207f,-0.2355f,0.8761f,  0.4194f,-0.2489f,0.8730f,  0.3160f,-0.3269f,0.8907f,  0.3146f,-0.3415f,0.8856f,  0.2059f,-0.4142f,0.8866f,  0.2044f,-0.4297f,0.8795f,  0.0922f,-0.4951f,0.8640f,  0.0908f,-0.5112f,0.8547f,  -0.0229f,-0.5676f,0.8230f,  -0.0243f,-0.5838f,0.8115f,  -0.1374f,-0.6297f,0.7646f,  -0.1387f,-0.6456f,0.7510f,  -0.2493f,-0.6795f,0.6900f,  
	-0.2505f,-0.6946f,0.6744f,  -0.3567f,-0.7153f,0.6010f,  -0.3576f,-0.7291f,0.5836f,  -0.4576f,-0.7356f,0.4995f,  -0.4584f,-0.7475f,0.4807f,  -0.5504f,-0.7393f,0.3881f,  -0.5510f,-0.7488f,0.3684f,  -0.6334f,-0.7254f,0.2696f,  -0.6338f,-0.7322f,0.2495f,  -0.7052f,-0.6936f,0.1470f,  -0.7056f,-0.6971f,0.1273f,  -0.7058f,-0.5731f,0.4164f,  -0.7086f,-0.5700f,0.4158f,  -0.7655f,-0.5703f,0.2980f,  -0.7681f,-0.5669f,0.2978f,  
	-0.8119f,-0.5570f,0.1749f,  -0.8144f,-0.5534f,0.1749f,  -0.8444f,-0.5334f,0.0492f,  -0.8467f,-0.5298f,0.0494f,  -0.8624f,-0.5003f,-0.0771f,  -0.8646f,-0.4966f,-0.0766f,  -0.8656f,-0.4583f,-0.2017f,  -0.8677f,-0.4547f,-0.2011f,  -0.8539f,-0.4083f,-0.3226f,  -0.8559f,-0.4047f,-0.3218f,  -0.8277f,-0.3512f,-0.4378f,  -0.8296f,-0.3477f,-0.4369f,  -0.7872f,-0.2880f,-0.5453f,  -0.7891f,-0.2846f,-0.5443f,  -0.7333f,-0.2199f,-0.6433f,  
	-0.7352f,-0.2165f,-0.6423f,  -0.6669f,-0.1480f,-0.7303f,  -0.6688f,-0.1447f,-0.7292f,  -0.5891f,-0.0735f,-0.8047f,  -0.5911f,-0.0702f,-0.8036f,  -0.5013f,0.0022f,-0.8653f,  -0.5032f,0.0056f,-0.8641f,  -0.4049f,0.0780f,-0.9110f,  -0.4068f,0.0815f,-0.9099f,  -0.3015f,0.1526f,-0.9412f,  -0.3035f,0.1562f,-0.9400f,  -0.1930f,0.2246f,-0.9551f,  -0.1949f,0.2284f,-0.9539f,  -0.0811f,0.2929f,-0.9527f,  -0.0829f,0.2969f,-0.9513f,  
	0.0323f,0.3563f,-0.9338f,  0.0305f,0.3604f,-0.9323f,  0.1453f,0.4135f,-0.8988f,  0.1436f,0.4178f,-0.8971f,  0.2560f,0.4637f,-0.8482f,  0.2544f,0.4681f,-0.8463f,  0.3624f,0.5058f,-0.7828f,  0.3610f,0.5102f,-0.7806f,  0.4629f,0.5388f,-0.7038f,  0.4616f,0.5433f,-0.7012f,  0.5558f,0.5622f,-0.6124f,  0.5546f,0.5666f,-0.6095f,  0.6393f,0.5753f,-0.5102f,  0.6382f,0.5794f,-0.5069f,  0.7122f,0.5776f,-0.3990f,  
	0.7112f,0.5813f,-0.3953f,  0.7731f,0.5688f,-0.2807f,  0.7722f,0.5720f,-0.2767f,  0.8209f,0.5489f,-0.1575f,  0.8202f,0.5512f,-0.1532f,  0.8549f,0.5178f,-0.0316f,  0.8542f,0.5192f,-0.0270f,  0.8744f,0.4758f,0.0947f,  0.8738f,0.4761f,0.0993f,  0.8791f,0.4234f,0.2190f,  0.8784f,0.4224f,0.2236f,  0.8687f,0.3613f,0.3388f,  0.8680f,0.3588f,0.3433f,  0.8435f,0.2902f,0.4519f,  0.8428f,0.2861f,0.4560f,  
	0.8039f,0.2114f,0.5560f,  0.8030f,0.2055f,0.5595f,  0.7503f,0.1260f,0.6489f,  0.7494f,0.1184f,0.6515f,  0.6839f,0.0355f,0.7287f,  0.6828f,0.0261f,0.7301f,  0.6057f,-0.0583f,0.7935f,  0.6045f,-0.0695f,0.7936f,  0.5170f,-0.1538f,0.8421f,  0.5157f,-0.1666f,0.8404f,  0.4194f,-0.2489f,0.8730f,  0.4180f,-0.2631f,0.8695f,  0.3146f,-0.3415f,0.8856f,  0.3131f,-0.3570f,0.8801f,  0.2044f,-0.4297f,0.8795f,  
	0.2028f,-0.4460f,0.8717f,  0.0908f,-0.5112f,0.8547f,  0.0893f,-0.5280f,0.8446f,  -0.0243f,-0.5838f,0.8115f,  -0.0257f,-0.6007f,0.7991f,  -0.1387f,-0.6456f,0.7510f,  -0.1400f,-0.6620f,0.7363f,  -0.2505f,-0.6946f,0.6744f,  -0.2516f,-0.7100f,0.6577f,  -0.3576f,-0.7291f,0.5836f,  -0.3586f,-0.7430f,0.5651f,  -0.4584f,-0.7475f,0.4807f,  -0.4592f,-0.7595f,0.4609f,  -0.5510f,-0.7488f,0.3684f,  -0.5515f,-0.7582f,0.3477f,  
	-0.6338f,-0.7322f,0.2495f,  -0.6342f,-0.7386f,0.2287f,  -0.7056f,-0.6971f,0.1273f,  -0.7058f,-0.7002f,0.1070f,  -0.7086f,-0.5700f,0.4158f,  -0.7111f,-0.5672f,0.4155f,  -0.7681f,-0.5669f,0.2978f,  -0.7704f,-0.5639f,0.2977f,  -0.8144f,-0.5534f,0.1749f,  -0.8165f,-0.5501f,0.1751f,  -0.8467f,-0.5298f,0.0494f,  -0.8488f,-0.5264f,0.0498f,  -0.8646f,-0.4966f,-0.0766f,  -0.8665f,-0.4933f,-0.0761f,  -0.8677f,-0.4547f,-0.2011f,  
	-0.8696f,-0.4513f,-0.2003f,  -0.8559f,-0.4047f,-0.3218f,  -0.8578f,-0.4014f,-0.3210f,  -0.8296f,-0.3477f,-0.4369f,  -0.8314f,-0.3445f,-0.4359f,  -0.7891f,-0.2846f,-0.5443f,  -0.7910f,-0.2814f,-0.5433f,  -0.7352f,-0.2165f,-0.6423f,  -0.7371f,-0.2133f,-0.6412f,  -0.6688f,-0.1447f,-0.7292f,  -0.6707f,-0.1414f,-0.7281f,  -0.5911f,-0.0702f,-0.8036f,  -0.5929f,-0.0669f,-0.8025f,  -0.5032f,0.0056f,-0.8641f,  -0.5051f,0.0090f,-0.8630f,  
	-0.4068f,0.0815f,-0.9099f,  -0.4087f,0.0851f,-0.9087f,  -0.3035f,0.1562f,-0.9400f,  -0.3053f,0.1600f,-0.9387f,  -0.1949f,0.2284f,-0.9539f,  -0.1967f,0.2324f,-0.9525f,  -0.0829f,0.2969f,-0.9513f,  -0.0847f,0.3011f,-0.9498f,  0.0305f,0.3604f,-0.9323f,  0.0288f,0.3648f,-0.9306f,  0.1436f,0.4178f,-0.8971f,  0.1420f,0.4225f,-0.8952f,  0.2544f,0.4681f,-0.8463f,  0.2528f,0.4729f,-0.8441f,  0.3610f,0.5102f,-0.7806f,  
	0.3596f,0.5151f,-0.7781f,  0.4616f,0.5433f,-0.7012f,  0.4603f,0.5482f,-0.6983f,  0.5546f,0.5666f,-0.6095f,  0.5534f,0.5713f,-0.6061f,  0.6382f,0.5794f,-0.5069f,  0.6372f,0.5838f,-0.5032f,  0.7112f,0.5813f,-0.3953f,  0.7103f,0.5852f,-0.3912f,  0.7722f,0.5720f,-0.2767f,  0.7714f,0.5753f,-0.2722f,  0.8202f,0.5512f,-0.1532f,  0.8194f,0.5537f,-0.1483f,  0.8542f,0.5192f,-0.0270f,  0.8535f,0.5206f,-0.0220f,  
	0.8738f,0.4761f,0.0993f,  0.8730f,0.4763f,0.1045f,  0.8784f,0.4224f,0.2236f,  0.8777f,0.4212f,0.2287f,  0.8680f,0.3588f,0.3433f,  0.8672f,0.3560f,0.3481f,  0.8428f,0.2861f,0.4560f,  0.8419f,0.2816f,0.4604f,  0.8030f,0.2055f,0.5595f,  0.8020f,0.1992f,0.5631f,  0.7494f,0.1184f,0.6515f,  0.7483f,0.1101f,0.6542f,  0.6828f,0.0261f,0.7301f,  0.6816f,0.0160f,0.7316f,  0.6045f,-0.0695f,0.7936f,  
	0.6031f,-0.0814f,0.7935f,  0.5157f,-0.1666f,0.8404f,  0.5142f,-0.1801f,0.8385f,  0.4180f,-0.2631f,0.8695f,  0.4164f,-0.2782f,0.8656f,  0.3131f,-0.3570f,0.8801f,  0.3115f,-0.3732f,0.8739f,  0.2028f,-0.4460f,0.8717f,  0.2013f,-0.4631f,0.8632f,  0.0893f,-0.5280f,0.8446f,  0.0877f,-0.5455f,0.8335f,  -0.0257f,-0.6007f,0.7991f,  -0.0272f,-0.6181f,0.7856f,  -0.1400f,-0.6620f,0.7363f,  -0.1413f,-0.6789f,0.7205f,  
	-0.2516f,-0.7100f,0.6577f,  -0.2527f,-0.7258f,0.6398f,  -0.3586f,-0.7430f,0.5651f,  -0.3595f,-0.7571f,0.5455f,  -0.4592f,-0.7595f,0.4609f,  -0.4598f,-0.7714f,0.4400f,  -0.5515f,-0.7582f,0.3477f,  -0.5519f,-0.7674f,0.3262f,  -0.6342f,-0.7386f,0.2287f,  -0.6345f,-0.7447f,0.2072f,  -0.7058f,-0.7002f,0.1070f,  -0.7061f,-0.7028f,0.0863f,  -0.7111f,-0.5672f,0.4155f,  -0.7132f,-0.5647f,0.4153f,  -0.7704f,-0.5639f,0.2977f,  
	-0.7724f,-0.5611f,0.2977f,  -0.8165f,-0.5501f,0.1751f,  -0.8184f,-0.5472f,0.1753f,  -0.8488f,-0.5264f,0.0498f,  -0.8506f,-0.5234f,0.0502f,  -0.8665f,-0.4933f,-0.0761f,  -0.8683f,-0.4902f,-0.0755f,  -0.8696f,-0.4513f,-0.2003f,  -0.8713f,-0.4483f,-0.1996f,  -0.8578f,-0.4014f,-0.3210f,  -0.8595f,-0.3984f,-0.3201f,  -0.8314f,-0.3445f,-0.4359f,  -0.8332f,-0.3414f,-0.4350f,  -0.7910f,-0.2814f,-0.5433f,  -0.7927f,-0.2784f,-0.5423f,  
	-0.7371f,-0.2133f,-0.6412f,  -0.7389f,-0.2103f,-0.6402f,  -0.6707f,-0.1414f,-0.7281f,  -0.6725f,-0.1383f,-0.7270f,  -0.5929f,-0.0669f,-0.8025f,  -0.5948f,-0.0636f,-0.8014f,  -0.5051f,0.0090f,-0.8630f,  -0.5070f,0.0125f,-0.8619f,  -0.4087f,0.0851f,-0.9087f,  -0.4106f,0.0888f,-0.9075f,  -0.3053f,0.1600f,-0.9387f,  -0.3072f,0.1639f,-0.9374f,  -0.1967f,0.2324f,-0.9525f,  -0.1986f,0.2366f,-0.9511f,  -0.0847f,0.3011f,-0.9498f,  
	-0.0865f,0.3056f,-0.9482f,  0.0288f,0.3648f,-0.9306f,  0.0271f,0.3696f,-0.9288f,  0.1420f,0.4225f,-0.8952f,  0.1404f,0.4275f,-0.8931f,  0.2528f,0.4729f,-0.8441f,  0.2513f,0.4781f,-0.8416f,  0.3596f,0.5151f,-0.7781f,  0.3582f,0.5204f,-0.7752f,  0.4603f,0.5482f,-0.6983f,  0.4591f,0.5534f,-0.6950f,  0.5534f,0.5713f,-0.6061f,  0.5523f,0.5763f,-0.6024f,  0.6372f,0.5838f,-0.5032f,  0.6362f,0.5885f,-0.4989f,  
	0.7103f,0.5852f,-0.3912f,  0.7094f,0.5894f,-0.3865f,  0.7714f,0.5753f,-0.2722f,  0.7705f,0.5787f,-0.2671f,  0.8194f,0.5537f,-0.1483f,  0.8186f,0.5563f,-0.1429f,  0.8535f,0.5206f,-0.0220f,  0.8528f,0.5221f,-0.0164f,  0.8730f,0.4763f,0.1045f,  0.8723f,0.4764f,0.1102f,  0.8777f,0.4212f,0.2287f,  0.8769f,0.4197f,0.2343f,  0.8672f,0.3560f,0.3481f,  0.8664f,0.3528f,0.3535f,  0.8419f,0.2816f,0.4604f,  
	0.8409f,0.2766f,0.4651f,  0.8020f,0.1992f,0.5631f,  0.8009f,0.1923f,0.5670f,  0.7483f,0.1101f,0.6542f,  0.7471f,0.1012f,0.6570f,  0.6816f,0.0160f,0.7316f,  0.6803f,0.0052f,0.7329f,  0.6031f,-0.0814f,0.7935f,  0.6017f,-0.0941f,0.7932f,  0.5142f,-0.1801f,0.8385f,  0.5126f,-0.1946f,0.8363f,  0.4164f,-0.2782f,0.8656f,  0.4148f,-0.2941f,0.8611f,  0.3115f,-0.3732f,0.8739f,  0.3098f,-0.3903f,0.8670f,  
	0.2013f,-0.4631f,0.8632f,  0.1996f,-0.4810f,0.8537f,  0.0877f,-0.5455f,0.8335f,  0.0861f,-0.5637f,0.8215f,  -0.0272f,-0.6181f,0.7856f,  -0.0285f,-0.6361f,0.7711f,  -0.1413f,-0.6789f,0.7205f,  -0.1425f,-0.6962f,0.7036f,  -0.2527f,-0.7258f,0.6398f,  -0.2537f,-0.7418f,0.6207f,  -0.3595f,-0.7571f,0.5455f,  -0.3602f,-0.7713f,0.5247f,  -0.4598f,-0.7714f,0.4400f,  -0.4603f,-0.7832f,0.4181f,  -0.5519f,-0.7674f,0.3262f,  
	-0.5522f,-0.7764f,0.3038f,  -0.6345f,-0.7447f,0.2072f,  -0.6347f,-0.7503f,0.1850f,  -0.7061f,-0.7028f,0.0863f,  -0.7063f,-0.7049f,0.0653f,  -0.7132f,-0.5647f,0.4153f,  -0.7150f,-0.5625f,0.4153f,  -0.7724f,-0.5611f,0.2977f,  -0.7741f,-0.5586f,0.2979f,  -0.8184f,-0.5472f,0.1753f,  -0.8201f,-0.5446f,0.1757f,  -0.8506f,-0.5234f,0.0502f,  -0.8522f,-0.5207f,0.0507f,  -0.8683f,-0.4902f,-0.0755f,  -0.8699f,-0.4875f,-0.0748f,  
	-0.8713f,-0.4483f,-0.1996f,  -0.8729f,-0.4456f,-0.1988f,  -0.8595f,-0.3984f,-0.3201f,  -0.8611f,-0.3956f,-0.3193f,  -0.8332f,-0.3414f,-0.4350f,  -0.8348f,-0.3386f,-0.4341f,  -0.7927f,-0.2784f,-0.5423f,  -0.7944f,-0.2755f,-0.5413f,  -0.7389f,-0.2103f,-0.6402f,  -0.7406f,-0.2073f,-0.6392f,  -0.6725f,-0.1383f,-0.7270f,  -0.6743f,-0.1351f,-0.7260f,  -0.5948f,-0.0636f,-0.8014f,  -0.5966f,-0.0603f,-0.8003f,  -0.5070f,0.0125f,-0.8619f,  
	-0.5088f,0.0161f,-0.8607f,  -0.4106f,0.0888f,-0.9075f,  -0.4124f,0.0927f,-0.9063f,  -0.3072f,0.1639f,-0.9374f,  -0.3090f,0.1681f,-0.9361f,  -0.1986f,0.2366f,-0.9511f,  -0.2003f,0.2411f,-0.9496f,  -0.0865f,0.3056f,-0.9482f,  -0.0882f,0.3104f,-0.9465f,  0.0271f,0.3696f,-0.9288f,  0.0255f,0.3747f,-0.9268f,  0.1404f,0.4275f,-0.8931f,  0.1388f,0.4329f,-0.8907f,  0.2513f,0.4781f,-0.8416f,  0.2499f,0.4836f,-0.8388f,  
	0.3582f,0.5204f,-0.7752f,  0.3569f,0.5260f,-0.7720f,  0.4591f,0.5534f,-0.6950f,  0.4579f,0.5590f,-0.6913f,  0.5523f,0.5763f,-0.6024f,  0.5512f,0.5817f,-0.5982f,  0.6362f,0.5885f,-0.4989f,  0.6352f,0.5935f,-0.4942f,  0.7094f,0.5894f,-0.3865f,  0.7085f,0.5939f,-0.3813f,  0.7705f,0.5787f,-0.2671f,  0.7697f,0.5824f,-0.2614f,  0.8186f,0.5563f,-0.1429f,  0.8179f,0.5589f,-0.1369f,  0.8528f,0.5221f,-0.0164f,  
	0.8520f,0.5234f,-0.0101f,  0.8723f,0.4764f,0.1102f,  0.8715f,0.4763f,0.1165f,  0.8769f,0.4197f,0.2343f,  0.8760f,0.4180f,0.2405f,  0.8664f,0.3528f,0.3535f,  0.8654f,0.3493f,0.3592f,  0.8409f,0.2766f,0.4651f,  0.8399f,0.2711f,0.4702f,  0.8009f,0.1923f,0.5670f,  0.7997f,0.1847f,0.5712f,  0.7471f,0.1012f,0.6570f,  0.7458f,0.0916f,0.6599f,  0.6803f,0.0052f,0.7329f,  0.6788f,-0.0065f,0.7343f,  
	0.6017f,-0.0941f,0.7932f,  0.6001f,-0.1077f,0.7926f,  0.5126f,-0.1946f,0.8363f,  0.5110f,-0.2099f,0.8336f,  0.4148f,-0.2941f,0.8611f,  0.4131f,-0.3108f,0.8560f,  0.3098f,-0.3903f,0.8670f,  0.3081f,-0.4082f,0.8593f,  0.1996f,-0.4810f,0.8537f,  0.1980f,-0.4996f,0.8433f,  0.0861f,-0.5637f,0.8215f,  0.0847f,-0.5825f,0.8084f,  -0.0285f,-0.6361f,0.7711f,  -0.0299f,-0.6547f,0.7553f,  -0.1425f,-0.6962f,0.7036f,  
	-0.1436f,-0.7139f,0.6854f,  -0.2537f,-0.7418f,0.6207f,  -0.2545f,-0.7580f,0.6005f,  -0.3602f,-0.7713f,0.5247f,  -0.3608f,-0.7855f,0.5029f,  -0.4603f,-0.7832f,0.4181f,  -0.4606f,-0.7948f,0.3952f,  -0.5522f,-0.7764f,0.3038f,  -0.5524f,-0.7849f,0.2805f,  -0.6347f,-0.7503f,0.1850f,  -0.6347f,-0.7555f,0.1623f,  -0.7063f,-0.7049f,0.0653f,  -0.7064f,-0.7064f,0.0439f,  -0.7150f,-0.5625f,0.4153f,  -0.7165f,-0.5605f,0.4153f,  
	-0.7741f,-0.5586f,0.2979f,  -0.7755f,-0.5565f,0.2981f,  -0.8201f,-0.5446f,0.1757f,  -0.8215f,-0.5423f,0.1761f,  -0.8522f,-0.5207f,0.0507f,  -0.8536f,-0.5184f,0.0513f,  -0.8699f,-0.4875f,-0.0748f,  -0.8713f,-0.4851f,-0.0741f,  -0.8729f,-0.4456f,-0.1988f,  -0.8743f,-0.4431f,-0.1981f,  -0.8611f,-0.3956f,-0.3193f,  -0.8626f,-0.3931f,-0.3184f,  -0.8348f,-0.3386f,-0.4341f,  -0.8363f,-0.3360f,-0.4332f,  -0.7944f,-0.2755f,-0.5413f,  
	-0.7960f,-0.2727f,-0.5404f,  -0.7406f,-0.2073f,-0.6392f,  -0.7422f,-0.2044f,-0.6382f,  -0.6743f,-0.1351f,-0.7260f,  -0.6759f,-0.1320f,-0.7250f,  -0.5966f,-0.0603f,-0.8003f,  -0.5983f,-0.0569f,-0.7993f,  -0.5088f,0.0161f,-0.8607f,  -0.5105f,0.0198f,-0.8596f,  -0.4124f,0.0927f,-0.9063f,  -0.4141f,0.0967f,-0.9051f,  -0.3090f,0.1681f,-0.9361f,  -0.3107f,0.1725f,-0.9347f,  -0.2003f,0.2411f,-0.9496f,  -0.2020f,0.2459f,-0.9480f,  
	-0.0882f,0.3104f,-0.9465f,  -0.0898f,0.3156f,-0.9446f,  0.0255f,0.3747f,-0.9268f,  0.0239f,0.3803f,-0.9246f,  0.1388f,0.4329f,-0.8907f,  0.1374f,0.4387f,-0.8881f,  0.2499f,0.4836f,-0.8388f,  0.2486f,0.4896f,-0.8357f,  0.3569f,0.5260f,-0.7720f,  0.3557f,0.5320f,-0.7684f,  0.4579f,0.5590f,-0.6913f,  0.4568f,0.5649f,-0.6872f,  0.5512f,0.5817f,-0.5982f,  0.5502f,0.5874f,-0.5935f,  0.6352f,0.5935f,-0.4942f,  
	0.6343f,0.5988f,-0.4889f,  0.7085f,0.5939f,-0.3813f,  0.7077f,0.5985f,-0.3755f,  0.7697f,0.5824f,-0.2614f,  0.7690f,0.5861f,-0.2552f,  0.8179f,0.5589f,-0.1369f,  0.8171f,0.5615f,-0.1303f,  0.8520f,0.5234f,-0.0101f,  0.8513f,0.5247f,-0.0033f,  0.8715f,0.4763f,0.1165f,  0.8707f,0.4761f,0.1234f,  0.8760f,0.4180f,0.2405f,  0.8752f,0.4160f,0.2471f,  0.8654f,0.3493f,0.3592f,  0.8645f,0.3453f,0.3653f,  
	0.8399f,0.2711f,0.4702f,  0.8388f,0.2651f,0.4756f,  0.7997f,0.1847f,0.5712f,  0.7985f,0.1765f,0.5755f,  0.7458f,0.0916f,0.6599f,  0.7444f,0.0813f,0.6628f,  0.6788f,-0.0065f,0.7343f,  0.6773f,-0.0189f,0.7355f,  0.6001f,-0.1077f,0.7926f,  0.5985f,-0.1221f,0.7918f,  0.5110f,-0.2099f,0.8336f,  0.5093f,-0.2261f,0.8304f,  0.4131f,-0.3108f,0.8560f,  0.4114f,-0.3285f,0.8502f,  0.3081f,-0.4082f,0.8593f,  
	0.3065f,-0.4269f,0.8508f,  0.1980f,-0.4996f,0.8433f,  0.1965f,-0.5189f,0.8319f,  0.0847f,-0.5825f,0.8084f,  0.0833f,-0.6020f,0.7942f,  -0.0299f,-0.6547f,0.7553f,  -0.0310f,-0.6737f,0.7384f,  -0.1436f,-0.7139f,0.6854f,  -0.1444f,-0.7318f,0.6660f,  -0.2545f,-0.7580f,0.6005f,  -0.2551f,-0.7744f,0.5790f,  -0.3608f,-0.7855f,0.5029f,  -0.3611f,-0.7996f,0.4798f,  -0.4606f,-0.7948f,0.3952f,  -0.4607f,-0.8062f,0.3712f,  
	-0.5524f,-0.7849f,0.2805f,  -0.5523f,-0.7932f,0.2565f,  -0.6347f,-0.7555f,0.1623f,  -0.6346f,-0.7602f,0.1390f,  -0.7064f,-0.7064f,0.0439f,  -0.7065f,-0.7073f,0.0223f,  -0.7165f,-0.5605f,0.4153f,  -0.7177f,-0.5588f,0.4154f,  -0.7755f,-0.5565f,0.2981f,  -0.7768f,-0.5546f,0.2984f,  -0.8215f,-0.5423f,0.1761f,  -0.8227f,-0.5403f,0.1765f,  -0.8536f,-0.5184f,0.0513f,  -0.8549f,-0.5163f,0.0518f,  -0.8713f,-0.4851f,-0.0741f,  
	-0.8726f,-0.4829f,-0.0735f,  -0.8743f,-0.4431f,-0.1981f,  -0.8756f,-0.4408f,-0.1973f,  -0.8626f,-0.3931f,-0.3184f,  -0.8640f,-0.3908f,-0.3176f,  -0.8363f,-0.3360f,-0.4332f,  -0.8378f,-0.3335f,-0.4324f,  -0.7960f,-0.2727f,-0.5404f,  -0.7975f,-0.2701f,-0.5395f,  -0.7422f,-0.2044f,-0.6382f,  -0.7438f,-0.2015f,-0.6374f,  -0.6759f,-0.1320f,-0.7250f,  -0.6775f,-0.1289f,-0.7241f,  -0.5983f,-0.0569f,-0.7993f,  -0.5999f,-0.0534f,-0.7983f,  
	-0.5105f,0.0198f,-0.8596f,  -0.5122f,0.0236f,-0.8586f,  -0.4141f,0.0967f,-0.9051f,  -0.4158f,0.1009f,-0.9038f,  -0.3107f,0.1725f,-0.9347f,  -0.3123f,0.1772f,-0.9333f,  -0.2020f,0.2459f,-0.9480f,  -0.2036f,0.2510f,-0.9463f,  -0.0898f,0.3156f,-0.9446f,  -0.0913f,0.3211f,-0.9426f,  0.0239f,0.3803f,-0.9246f,  0.0225f,0.3862f,-0.9222f,  0.1374f,0.4387f,-0.8881f,  0.1361f,0.4449f,-0.8852f,  0.2486f,0.4896f,-0.8357f,  
	0.2474f,0.4960f,-0.8323f,  0.3557f,0.5320f,-0.7684f,  0.3545f,0.5385f,-0.7644f,  0.4568f,0.5649f,-0.6872f,  0.4558f,0.5713f,-0.6826f,  0.5502f,0.5874f,-0.5935f,  0.5493f,0.5935f,-0.5882f,  0.6343f,0.5988f,-0.4889f,  0.6335f,0.6044f,-0.4831f,  0.7077f,0.5985f,-0.3755f,  0.7069f,0.6034f,-0.3691f,  0.7690f,0.5861f,-0.2552f,  0.7683f,0.5900f,-0.2483f,  0.8171f,0.5615f,-0.1303f,  0.8164f,0.5642f,-0.1231f,  
	0.8513f,0.5247f,-0.0033f,  0.8505f,0.5260f,0.0041f,  0.8707f,0.4761f,0.1234f,  0.8699f,0.4756f,0.1307f,  0.8752f,0.4160f,0.2471f,  0.8743f,0.4136f,0.2542f,  0.8645f,0.3453f,0.3653f,  0.8634f,0.3409f,0.3719f,  0.8388f,0.2651f,0.4756f,  0.8376f,0.2584f,0.4813f,  0.7985f,0.1765f,0.5755f,  0.7972f,0.1677f,0.5800f,  0.7444f,0.0813f,0.6628f,  0.7429f,0.0702f,0.6657f,  0.6773f,-0.0189f,0.7355f,  
	0.6757f,-0.0322f,0.7365f,  0.5985f,-0.1221f,0.7918f,  0.5968f,-0.1374f,0.7905f,  0.5093f,-0.2261f,0.8304f,  0.5076f,-0.2432f,0.8266f,  0.4114f,-0.3285f,0.8502f,  0.4097f,-0.3470f,0.8436f,  0.3065f,-0.4269f,0.8508f,  0.3050f,-0.4464f,0.8412f,  0.1965f,-0.5189f,0.8319f,  0.1951f,-0.5390f,0.8194f,  0.0833f,-0.6020f,0.7942f,  0.0821f,-0.6220f,0.7787f,  -0.0310f,-0.6737f,0.7384f,  -0.0319f,-0.6931f,0.7201f,  
	-0.1444f,-0.7318f,0.6660f,  -0.1450f,-0.7501f,0.6453f,  -0.2551f,-0.7744f,0.5790f,  -0.2554f,-0.7908f,0.5562f,  -0.3611f,-0.7996f,0.4798f,  -0.3611f,-0.8136f,0.4556f,  -0.4607f,-0.8062f,0.3712f,  -0.4604f,-0.8173f,0.3464f,  -0.5523f,-0.7932f,0.2565f,  -0.5520f,-0.8010f,0.2318f,  -0.6346f,-0.7602f,0.1390f,  -0.6344f,-0.7643f,0.1153f,  -0.7065f,-0.7073f,0.0223f,  -0.7066f,-0.7076f,0.0006f,  -0.7177f,-0.5588f,0.4154f,  
	-0.7187f,-0.5574f,0.4155f,  -0.7768f,-0.5546f,0.2984f,  -0.7778f,-0.5531f,0.2987f,  -0.8227f,-0.5403f,0.1765f,  -0.8237f,-0.5387f,0.1769f,  -0.8549f,-0.5163f,0.0518f,  -0.8559f,-0.5145f,0.0524f,  -0.8726f,-0.4829f,-0.0735f,  -0.8737f,-0.4810f,-0.0729f,  -0.8756f,-0.4408f,-0.1973f,  -0.8768f,-0.4388f,-0.1967f,  -0.8640f,-0.3908f,-0.3176f,  -0.8652f,-0.3886f,-0.3169f,  -0.8378f,-0.3335f,-0.4324f,  -0.8391f,-0.3312f,-0.4316f,  
	-0.7975f,-0.2701f,-0.5395f,  -0.7988f,-0.2675f,-0.5388f,  -0.7438f,-0.2015f,-0.6374f,  -0.7452f,-0.1986f,-0.6366f,  -0.6775f,-0.1289f,-0.7241f,  -0.6790f,-0.1257f,-0.7233f,  -0.5999f,-0.0534f,-0.7983f,  -0.6015f,-0.0498f,-0.7974f,  -0.5122f,0.0236f,-0.8586f,  -0.5137f,0.0277f,-0.8575f,  -0.4158f,0.1009f,-0.9038f,  -0.4173f,0.1055f,-0.9026f,  -0.3123f,0.1772f,-0.9333f,  -0.3139f,0.1822f,-0.9318f,  -0.2036f,0.2510f,-0.9463f,  
	-0.2051f,0.2565f,-0.9445f,  -0.0913f,0.3211f,-0.9426f,  -0.0928f,0.3271f,-0.9404f,  0.0225f,0.3862f,-0.9222f,  0.0212f,0.3925f,-0.9195f,  0.1361f,0.4449f,-0.8852f,  0.1348f,0.4515f,-0.8820f,  0.2474f,0.4960f,-0.8323f,  0.2462f,0.5029f,-0.8286f,  0.3545f,0.5385f,-0.7644f,  0.3535f,0.5454f,-0.7600f,  0.4558f,0.5713f,-0.6826f,  0.4549f,0.5781f,-0.6774f,  0.5493f,0.5935f,-0.5882f,  0.5485f,0.5999f,-0.5824f,  
	0.6335f,0.6044f,-0.4831f,  0.6328f,0.6102f,-0.4766f,  0.7069f,0.6034f,-0.3691f,  0.7063f,0.6084f,-0.3620f,  0.7683f,0.5900f,-0.2483f,  0.7676f,0.5940f,-0.2408f,  0.8164f,0.5642f,-0.1231f,  0.8157f,0.5669f,-0.1152f,  0.8505f,0.5260f,0.0041f,  0.8498f,0.5270f,0.0122f,  0.8699f,0.4756f,0.1307f,  0.8691f,0.4748f,0.1387f,  0.8743f,0.4136f,0.2542f,  0.8733f,0.4108f,0.2619f,  0.8634f,0.3409f,0.3719f,  
	0.8623f,0.3359f,0.3789f,  0.8376f,0.2584f,0.4813f,  0.8364f,0.2512f,0.4873f,  0.7972f,0.1677f,0.5800f,  0.7958f,0.1580f,0.5846f,  0.7429f,0.0702f,0.6657f,  0.7414f,0.0582f,0.6686f,  0.6757f,-0.0322f,0.7365f,  0.6741f,-0.0464f,0.7372f,  0.5968f,-0.1374f,0.7905f,  0.5951f,-0.1537f,0.7888f,  0.5076f,-0.2432f,0.8266f,  0.5059f,-0.2611f,0.8221f,  0.4097f,-0.3470f,0.8436f,  0.4082f,-0.3664f,0.8362f,  
	0.3050f,-0.4464f,0.8412f,  0.3035f,-0.4668f,0.8307f,  0.1951f,-0.5390f,0.8194f,  0.1938f,-0.5597f,0.8057f,  0.0821f,-0.6220f,0.7787f,  0.0812f,-0.6426f,0.7619f,  -0.0319f,-0.6931f,0.7201f,  -0.0325f,-0.7129f,0.7005f,  -0.1450f,-0.7501f,0.6453f,  -0.1453f,-0.7685f,0.6232f,  -0.2554f,-0.7908f,0.5562f,  -0.2554f,-0.8072f,0.5322f,  -0.3611f,-0.8136f,0.4556f,  -0.3607f,-0.8275f,0.4303f,  -0.4604f,-0.8173f,0.3464f,  
	-0.4599f,-0.8281f,0.3205f,  -0.5520f,-0.8010f,0.2318f,  -0.5515f,-0.8083f,0.2063f,  -0.6344f,-0.7643f,0.1153f,  -0.6340f,-0.7679f,0.0912f,  -0.7066f,-0.7076f,0.0006f,  -0.7065f,-0.7074f,-0.0210f,  -0.7187f,-0.5574f,0.4155f,  -0.7195f,-0.5563f,0.4157f,  -0.7778f,-0.5531f,0.2987f,  -0.7785f,-0.5518f,0.2990f,  -0.8237f,-0.5387f,0.1769f,  -0.8246f,-0.5373f,0.1773f,  -0.8559f,-0.5145f,0.0524f,  -0.8568f,-0.5130f,0.0529f,  
	-0.8737f,-0.4810f,-0.0729f,  -0.8746f,-0.4794f,-0.0723f,  -0.8768f,-0.4388f,-0.1967f,  -0.8778f,-0.4370f,-0.1961f,  -0.8652f,-0.3886f,-0.3169f,  -0.8663f,-0.3866f,-0.3163f,  -0.8391f,-0.3312f,-0.4316f,  -0.8403f,-0.3290f,-0.4310f,  -0.7988f,-0.2675f,-0.5388f,  -0.8001f,-0.2650f,-0.5381f,  -0.7452f,-0.1986f,-0.6366f,  -0.7466f,-0.1958f,-0.6359f,  -0.6790f,-0.1257f,-0.7233f,  -0.6804f,-0.1224f,-0.7225f,  -0.6015f,-0.0498f,-0.7974f,  
	-0.6029f,-0.0461f,-0.7965f,  -0.5137f,0.0277f,-0.8575f,  -0.5152f,0.0319f,-0.8565f,  -0.4173f,0.1055f,-0.9026f,  -0.4188f,0.1102f,-0.9014f,  -0.3139f,0.1822f,-0.9318f,  -0.3153f,0.1875f,-0.9303f,  -0.2051f,0.2565f,-0.9445f,  -0.2064f,0.2623f,-0.9426f,  -0.0928f,0.3271f,-0.9404f,  -0.0940f,0.3334f,-0.9381f,  0.0212f,0.3925f,-0.9195f,  0.0200f,0.3993f,-0.9166f,  0.1348f,0.4515f,-0.8820f,  0.1337f,0.4586f,-0.8785f,  
	0.2462f,0.5029f,-0.8286f,  0.2453f,0.5101f,-0.8244f,  0.3535f,0.5454f,-0.7600f,  0.3527f,0.5527f,-0.7551f,  0.4549f,0.5781f,-0.6774f,  0.4541f,0.5852f,-0.6718f,  0.5485f,0.5999f,-0.5824f,  0.5478f,0.6067f,-0.5760f,  0.6328f,0.6102f,-0.4766f,  0.6321f,0.6164f,-0.4695f,  0.7063f,0.6084f,-0.3620f,  0.7056f,0.6136f,-0.3543f,  0.7676f,0.5940f,-0.2408f,  0.7670f,0.5981f,-0.2325f,  0.8157f,0.5669f,-0.1152f,  
	0.8151f,0.5695f,-0.1066f,  0.8498f,0.5270f,0.0122f,  0.8490f,0.5279f,0.0208f,  0.8691f,0.4748f,0.1387f,  0.8682f,0.4738f,0.1473f,  0.8733f,0.4108f,0.2619f,  0.8723f,0.4076f,0.2699f,  0.8623f,0.3359f,0.3789f,  0.8612f,0.3304f,0.3862f,  0.8364f,0.2512f,0.4873f,  0.8351f,0.2432f,0.4934f,  0.7958f,0.1580f,0.5846f,  0.7944f,0.1476f,0.5892f,  0.7414f,0.0582f,0.6686f,  0.7399f,0.0454f,0.6712f,  
	0.6741f,-0.0464f,0.7372f,  0.6725f,-0.0615f,0.7375f,  0.5951f,-0.1537f,0.7888f,  0.5935f,-0.1708f,0.7865f,  0.5059f,-0.2611f,0.8221f,  0.5045f,-0.2800f,0.8168f,  0.4082f,-0.3664f,0.8362f,  0.4067f,-0.3866f,0.8277f,  0.3035f,-0.4668f,0.8307f,  0.3022f,-0.4878f,0.8189f,  0.1938f,-0.5597f,0.8057f,  0.1929f,-0.5810f,0.7907f,  0.0812f,-0.6426f,0.7619f,  0.0806f,-0.6636f,0.7437f,  -0.0325f,-0.7129f,0.7005f,  
	-0.0327f,-0.7330f,0.6794f,  -0.1453f,-0.7685f,0.6232f,  -0.1452f,-0.7869f,0.5997f,  -0.2554f,-0.8072f,0.5322f,  -0.2548f,-0.8235f,0.5069f,  -0.3607f,-0.8275f,0.4303f,  -0.3600f,-0.8410f,0.4038f,  -0.4599f,-0.8281f,0.3205f,  -0.4591f,-0.8384f,0.2938f,  -0.5515f,-0.8083f,0.2063f,  -0.5507f,-0.8151f,0.1802f,  -0.6340f,-0.7679f,0.0912f,  -0.6334f,-0.7709f,0.0668f,  -0.7065f,-0.7074f,-0.0210f,  -0.7064f,-0.7065f,-0.0426f,  
	-0.7195f,-0.5563f,0.4157f,  -0.7200f,-0.5555f,0.4159f,  -0.7785f,-0.5518f,0.2990f,  -0.7791f,-0.5508f,0.2992f,  -0.8246f,-0.5373f,0.1773f,  -0.8252f,-0.5362f,0.1777f,  -0.8568f,-0.5130f,0.0529f,  -0.8575f,-0.5117f,0.0533f,  -0.8746f,-0.4794f,-0.0723f,  -0.8754f,-0.4780f,-0.0719f,  -0.8778f,-0.4370f,-0.1961f,  -0.8787f,-0.4355f,-0.1956f,  -0.8663f,-0.3866f,-0.3163f,  -0.8673f,-0.3848f,-0.3158f,  -0.8403f,-0.3290f,-0.4310f,  
	-0.8413f,-0.3269f,-0.4305f,  -0.8001f,-0.2650f,-0.5381f,  -0.8013f,-0.2626f,-0.5376f,  -0.7466f,-0.1958f,-0.6359f,  -0.7478f,-0.1929f,-0.6353f,  -0.6804f,-0.1224f,-0.7225f,  -0.6817f,-0.1191f,-0.7218f,  -0.6029f,-0.0461f,-0.7965f,  -0.6042f,-0.0422f,-0.7957f,  -0.5152f,0.0319f,-0.8565f,  -0.5165f,0.0364f,-0.8555f,  -0.4188f,0.1102f,-0.9014f,  -0.4201f,0.1153f,-0.9001f,  -0.3153f,0.1875f,-0.9303f,  -0.3166f,0.1932f,-0.9287f,  
	-0.2064f,0.2623f,-0.9426f,  -0.2077f,0.2686f,-0.9406f,  -0.0940f,0.3334f,-0.9381f,  -0.0952f,0.3402f,-0.9355f,  0.0200f,0.3993f,-0.9166f,  0.0189f,0.4065f,-0.9135f,  0.1337f,0.4586f,-0.8785f,  0.1328f,0.4662f,-0.8747f,  0.2453f,0.5101f,-0.8244f,  0.2444f,0.5179f,-0.8198f,  0.3527f,0.5527f,-0.7551f,  0.3519f,0.5605f,-0.7497f,  0.4541f,0.5852f,-0.6718f,  0.4534f,0.5928f,-0.6656f,  0.5478f,0.6067f,-0.5760f,  
	0.5472f,0.6138f,-0.5690f,  0.6321f,0.6164f,-0.4695f,  0.6316f,0.6228f,-0.4618f,  0.7056f,0.6136f,-0.3543f,  0.7051f,0.6190f,-0.3459f,  0.7670f,0.5981f,-0.2325f,  0.7664f,0.6022f,-0.2236f,  0.8151f,0.5695f,-0.1066f,  0.8145f,0.5720f,-0.0974f,  0.8490f,0.5279f,0.0208f,  0.8483f,0.5286f,0.0302f,  0.8682f,0.4738f,0.1473f,  0.8674f,0.4724f,0.1564f,  0.8723f,0.4076f,0.2699f,  0.8714f,0.4039f,0.2785f,  
	0.8612f,0.3304f,0.3862f,  0.8601f,0.3242f,0.3938f,  0.8351f,0.2432f,0.4934f,  0.8338f,0.2345f,0.4997f,  0.7944f,0.1476f,0.5892f,  0.7930f,0.1364f,0.5937f,  0.7399f,0.0454f,0.6712f,  0.7384f,0.0317f,0.6736f,  0.6725f,-0.0615f,0.7375f,  0.6710f,-0.0775f,0.7374f,  0.5935f,-0.1708f,0.7865f,  0.5921f,-0.1888f,0.7834f,  0.5045f,-0.2800f,0.8168f,  0.5030f,-0.2998f,0.8106f,  0.4067f,-0.3866f,0.8277f,  
	0.4055f,-0.4076f,0.8182f,  0.3022f,-0.4878f,0.8189f,  0.3013f,-0.5096f,0.8059f,  0.1929f,-0.5810f,0.7907f,  0.1923f,-0.6029f,0.7743f,  0.0806f,-0.6636f,0.7437f,  0.0804f,-0.6850f,0.7241f,  -0.0327f,-0.7330f,0.6794f,  -0.0326f,-0.7533f,0.6569f,  -0.1452f,-0.7869f,0.5997f,  -0.1446f,-0.8054f,0.5748f,  -0.2548f,-0.8235f,0.5069f,  -0.2539f,-0.8396f,0.4802f,  -0.3600f,-0.8410f,0.4038f,  -0.3589f,-0.8542f,0.3762f,  
	-0.4591f,-0.8384f,0.2938f,  -0.4579f,-0.8483f,0.2661f,  -0.5507f,-0.8151f,0.1802f,  -0.5495f,-0.8213f,0.1535f,  -0.6334f,-0.7709f,0.0668f,  -0.6327f,-0.7733f,0.0423f,  -0.7064f,-0.7065f,-0.0426f,  -0.7062f,-0.7051f,-0.0640f,  -0.7200f,-0.5555f,0.4159f,  -0.7204f,-0.5550f,0.4160f,  -0.7791f,-0.5508f,0.2992f,  -0.7795f,-0.5502f,0.2994f,  -0.8252f,-0.5362f,0.1777f,  -0.8257f,-0.5353f,0.1779f,  -0.8575f,-0.5117f,0.0533f,  
	-0.8581f,-0.5108f,0.0536f,  -0.8754f,-0.4780f,-0.0719f,  -0.8761f,-0.4768f,-0.0715f,  -0.8787f,-0.4355f,-0.1956f,  -0.8795f,-0.4341f,-0.1952f,  -0.8673f,-0.3848f,-0.3158f,  -0.8682f,-0.3831f,-0.3155f,  -0.8413f,-0.3269f,-0.4305f,  -0.8423f,-0.3248f,-0.4301f,  -0.8013f,-0.2626f,-0.5376f,  -0.8023f,-0.2601f,-0.5372f,  -0.7478f,-0.1929f,-0.6353f,  -0.7489f,-0.1900f,-0.6348f,  -0.6817f,-0.1191f,-0.7218f,  -0.6829f,-0.1156f,-0.7213f,  
	-0.6042f,-0.0422f,-0.7957f,  -0.6054f,-0.0381f,-0.7950f,  -0.5165f,0.0364f,-0.8555f,  -0.5178f,0.0411f,-0.8545f,  -0.4201f,0.1153f,-0.9001f,  -0.4213f,0.1207f,-0.8989f,  -0.3166f,0.1932f,-0.9287f,  -0.3177f,0.1992f,-0.9270f,  -0.2077f,0.2686f,-0.9406f,  -0.2088f,0.2753f,-0.9384f,  -0.0952f,0.3402f,-0.9355f,  -0.0962f,0.3474f,-0.9328f,  0.0189f,0.4065f,-0.9135f,  0.0180f,0.4142f,-0.9100f,  0.1328f,0.4662f,-0.8747f,  
	0.1320f,0.4742f,-0.8705f,  0.2444f,0.5179f,-0.8198f,  0.2437f,0.5261f,-0.8147f,  0.3519f,0.5605f,-0.7497f,  0.3513f,0.5687f,-0.7438f,  0.4534f,0.5928f,-0.6656f,  0.4529f,0.6007f,-0.6588f,  0.5472f,0.6138f,-0.5690f,  0.5468f,0.6212f,-0.5614f,  0.6316f,0.6228f,-0.4618f,  0.6312f,0.6294f,-0.4533f,  0.7051f,0.6190f,-0.3459f,  0.7047f,0.6245f,-0.3367f,  0.7664f,0.6022f,-0.2236f,  0.7659f,0.6063f,-0.2140f,  
	0.8145f,0.5720f,-0.0974f,  0.8139f,0.5744f,-0.0874f,  0.8483f,0.5286f,0.0302f,  0.8476f,0.5290f,0.0401f,  0.8674f,0.4724f,0.1564f,  0.8666f,0.4706f,0.1660f,  0.8714f,0.4039f,0.2785f,  0.8704f,0.3997f,0.2875f,  0.8601f,0.3242f,0.3938f,  0.8590f,0.3174f,0.4017f,  0.8338f,0.2345f,0.4997f,  0.8326f,0.2251f,0.5061f,  0.7930f,0.1364f,0.5937f,  0.7916f,0.1243f,0.5982f,  0.7384f,0.0317f,0.6736f,  
	0.7369f,0.0170f,0.6757f,  0.6710f,-0.0775f,0.7374f,  0.6696f,-0.0945f,0.7367f,  0.5921f,-0.1888f,0.7834f,  0.5907f,-0.2079f,0.7797f,  0.5030f,-0.2998f,0.8106f,  0.5018f,-0.3204f,0.8034f,  0.4055f,-0.4076f,0.8182f,  0.4046f,-0.4294f,0.8074f,  0.3013f,-0.5096f,0.8059f,  0.3007f,-0.5319f,0.7916f,  0.1923f,-0.6029f,0.7743f,  0.1921f,-0.6253f,0.7564f,  0.0804f,-0.6850f,0.7241f,  0.0805f,-0.7067f,0.7029f,  
	-0.0326f,-0.7533f,0.6569f,  -0.0319f,-0.7736f,0.6329f,  -0.1446f,-0.8054f,0.5748f,  -0.1435f,-0.8238f,0.5484f,  -0.2539f,-0.8396f,0.4802f,  -0.2525f,-0.8554f,0.4522f,  -0.3589f,-0.8542f,0.3762f,  -0.3573f,-0.8670f,0.3475f,  -0.4579f,-0.8483f,0.2661f,  -0.4562f,-0.8576f,0.2376f,  -0.5495f,-0.8213f,0.1535f,  -0.5481f,-0.8268f,0.1264f,  -0.6327f,-0.7733f,0.0423f,  -0.6317f,-0.7750f,0.0176f,  -0.7062f,-0.7051f,-0.0640f,  
	-0.7060f,-0.7031f,-0.0851f,  -0.7204f,-0.5550f,0.4160f,  -0.7205f,-0.5548f,0.4161f,  -0.7795f,-0.5502f,0.2994f,  -0.7797f,-0.5498f,0.2996f,  -0.8257f,-0.5353f,0.1779f,  -0.8260f,-0.5348f,0.1781f,  -0.8581f,-0.5108f,0.0536f,  -0.8585f,-0.5100f,0.0538f,  -0.8761f,-0.4768f,-0.0715f,  -0.8766f,-0.4759f,-0.0713f,  -0.8795f,-0.4341f,-0.1952f,  -0.8801f,-0.4328f,-0.1951f,  -0.8682f,-0.3831f,-0.3155f,  -0.8689f,-0.3816f,-0.3153f,  
	-0.8423f,-0.3248f,-0.4301f,  -0.8432f,-0.3229f,-0.4299f,  -0.8023f,-0.2601f,-0.5372f,  -0.8033f,-0.2577f,-0.5370f,  -0.7489f,-0.1900f,-0.6348f,  -0.7499f,-0.1870f,-0.6345f,  -0.6829f,-0.1156f,-0.7213f,  -0.6840f,-0.1120f,-0.7209f,  -0.6054f,-0.0381f,-0.7950f,  -0.6065f,-0.0338f,-0.7943f,  -0.5178f,0.0411f,-0.8545f,  -0.5188f,0.0461f,-0.8536f,  -0.4213f,0.1207f,-0.8989f,  -0.4224f,0.1264f,-0.8976f,  -0.3177f,0.1992f,-0.9270f,  
	-0.3187f,0.2057f,-0.9253f,  -0.2088f,0.2753f,-0.9384f,  -0.2097f,0.2824f,-0.9361f,  -0.0962f,0.3474f,-0.9328f,  -0.0970f,0.3551f,-0.9298f,  0.0180f,0.4142f,-0.9100f,  0.0173f,0.4224f,-0.9062f,  0.1320f,0.4742f,-0.8705f,  0.1314f,0.4828f,-0.8658f,  0.2437f,0.5261f,-0.8147f,  0.2432f,0.5349f,-0.8092f,  0.3513f,0.5687f,-0.7438f,  0.3509f,0.5773f,-0.7373f,  0.4529f,0.6007f,-0.6588f,  0.4526f,0.6090f,-0.6513f,  
	0.5468f,0.6212f,-0.5614f,  0.5464f,0.6290f,-0.5530f,  0.6312f,0.6294f,-0.4533f,  0.6308f,0.6362f,-0.4441f,  0.7047f,0.6245f,-0.3367f,  0.7043f,0.6302f,-0.3269f,  0.7659f,0.6063f,-0.2140f,  0.7655f,0.6104f,-0.2035f,  0.8139f,0.5744f,-0.0874f,  0.8134f,0.5767f,-0.0768f,  0.8476f,0.5290f,0.0401f,  0.8470f,0.5292f,0.0507f,  0.8666f,0.4706f,0.1660f,  0.8658f,0.4683f,0.1762f,  0.8704f,0.3997f,0.2875f,  
	0.8694f,0.3949f,0.2969f,  0.8590f,0.3174f,0.4017f,  0.8579f,0.3099f,0.4099f,  0.8326f,0.2251f,0.5061f,  0.8313f,0.2148f,0.5126f,  0.7916f,0.1243f,0.5982f,  0.7903f,0.1113f,0.6025f,  0.7369f,0.0170f,0.6757f,  0.7357f,0.0015f,0.6773f,  0.6696f,-0.0945f,0.7367f,  0.6683f,-0.1124f,0.7354f,  0.5907f,-0.2079f,0.7797f,  0.5896f,-0.2277f,0.7750f,  0.5018f,-0.3204f,0.8034f,  0.5010f,-0.3419f,0.7951f,  
	0.4046f,-0.4294f,0.8074f,  0.4040f,-0.4519f,0.7954f,  0.3007f,-0.5319f,0.7916f,  0.3005f,-0.5549f,0.7757f,  0.1921f,-0.6253f,0.7564f,  0.1923f,-0.6481f,0.7369f,  0.0805f,-0.7067f,0.7029f,  0.0814f,-0.7286f,0.6801f,  -0.0319f,-0.7736f,0.6329f,  -0.0306f,-0.7940f,0.6072f,  -0.1435f,-0.8238f,0.5484f,  -0.1419f,-0.8420f,0.5205f,  -0.2525f,-0.8554f,0.4522f,  -0.2506f,-0.8709f,0.4229f,  -0.3573f,-0.8670f,0.3475f,  
	-0.3551f,-0.8792f,0.3176f,  -0.4562f,-0.8576f,0.2376f,  -0.4542f,-0.8662f,0.2083f,  -0.5481f,-0.8268f,0.1264f,  -0.5463f,-0.8317f,0.0988f,  -0.6317f,-0.7750f,0.0176f,  -0.6305f,-0.7761f,-0.0072f,  -0.7060f,-0.7031f,-0.0851f,  -0.7057f,-0.7005f,-0.1058f,  -0.7205f,-0.5548f,0.4161f,  -0.7205f,-0.5548f,0.4161f,  -0.7797f,-0.5498f,0.2996f,  -0.7798f,-0.5497f,0.2996f,  -0.8260f,-0.5348f,0.1781f,  -0.8262f,-0.5345f,0.1781f,  
	-0.8585f,-0.5100f,0.0538f,  -0.8588f,-0.5095f,0.0538f,  -0.8766f,-0.4759f,-0.0713f,  -0.8770f,-0.4751f,-0.0713f,  -0.8801f,-0.4328f,-0.1951f,  -0.8806f,-0.4318f,-0.1951f,  -0.8689f,-0.3816f,-0.3153f,  -0.8695f,-0.3801f,-0.3153f,  -0.8432f,-0.3229f,-0.4299f,  -0.8439f,-0.3209f,-0.4299f,  -0.8033f,-0.2577f,-0.5370f,  -0.8041f,-0.2552f,-0.5369f,  -0.7499f,-0.1870f,-0.6345f,  -0.7508f,-0.1839f,-0.6344f,  -0.6840f,-0.1120f,-0.7209f,  
	-0.6849f,-0.1081f,-0.7206f,  -0.6065f,-0.0338f,-0.7943f,  -0.6075f,-0.0293f,-0.7938f,  -0.5188f,0.0461f,-0.8536f,  -0.5198f,0.0514f,-0.8528f,  -0.4224f,0.1264f,-0.8976f,  -0.4233f,0.1325f,-0.8963f,  -0.3187f,0.2057f,-0.9253f,  -0.3196f,0.2125f,-0.9234f,  -0.2097f,0.2824f,-0.9361f,  -0.2104f,0.2900f,-0.9336f,  -0.0970f,0.3551f,-0.9298f,  -0.0977f,0.3634f,-0.9265f,  0.0173f,0.4224f,-0.9062f,  0.0167f,0.4312f,-0.9021f,  
	0.1314f,0.4828f,-0.8658f,  0.1309f,0.4919f,-0.8608f,  0.2432f,0.5349f,-0.8092f,  0.2428f,0.5441f,-0.8031f,  0.3509f,0.5773f,-0.7373f,  0.3506f,0.5864f,-0.7302f,  0.4526f,0.6090f,-0.6513f,  0.4523f,0.6178f,-0.6432f,  0.5464f,0.6290f,-0.5530f,  0.5462f,0.6370f,-0.5439f,  0.6308f,0.6362f,-0.4441f,  0.6306f,0.6433f,-0.4342f,  0.7043f,0.6302f,-0.3269f,  0.7041f,0.6359f,-0.3162f,  0.7655f,0.6104f,-0.2035f,  
	0.7652f,0.6144f,-0.1924f,  0.8134f,0.5767f,-0.0768f,  0.8129f,0.5787f,-0.0655f,  0.8470f,0.5292f,0.0507f,  0.8464f,0.5290f,0.0619f,  0.8658f,0.4683f,0.1762f,  0.8650f,0.4656f,0.1869f,  0.8694f,0.3949f,0.2969f,  0.8685f,0.3895f,0.3066f,  0.8579f,0.3099f,0.4099f,  0.8568f,0.3017f,0.4181f,  0.8313f,0.2148f,0.5126f,  0.8302f,0.2037f,0.5189f,  0.7903f,0.1113f,0.6025f,  0.7892f,0.0975f,0.6064f,  
	0.7357f,0.0015f,0.6773f,  0.7345f,-0.0150f,0.6785f,  0.6683f,-0.1124f,0.7354f,  0.6672f,-0.1312f,0.7332f,  0.5896f,-0.2277f,0.7750f,  0.5887f,-0.2485f,0.7692f,  0.5010f,-0.3419f,0.7951f,  0.5004f,-0.3641f,0.7855f,  0.4040f,-0.4519f,0.7954f,  0.4039f,-0.4750f,0.7818f,  0.3005f,-0.5549f,0.7757f,  0.3008f,-0.5783f,0.7583f,  0.1923f,-0.6481f,0.7369f,  0.1932f,-0.6711f,0.7158f,  0.0814f,-0.7286f,0.6801f,  
	0.0828f,-0.7505f,0.6556f,  -0.0306f,-0.7940f,0.6072f,  -0.0288f,-0.8142f,0.5799f,  -0.1419f,-0.8420f,0.5205f,  -0.1396f,-0.8599f,0.4911f,  -0.2506f,-0.8709f,0.4229f,  -0.2480f,-0.8858f,0.3922f,  -0.3551f,-0.8792f,0.3176f,  -0.3525f,-0.8908f,0.2867f,  -0.4542f,-0.8662f,0.2083f,  -0.4517f,-0.8742f,0.1783f,  -0.5463f,-0.8317f,0.0988f,  -0.5443f,-0.8359f,0.0708f,  -0.6305f,-0.7761f,-0.0072f,  -0.6292f,-0.7766f,-0.0319f,  
	-0.7057f,-0.7005f,-0.1058f,  -0.7054f,-0.6975f,-0.1261f,  -0.7205f,-0.5548f,0.4161f,  -0.7203f,-0.5552f,0.4159f,  -0.7798f,-0.5497f,0.2996f,  -0.7797f,-0.5499f,0.2995f,  -0.8262f,-0.5345f,0.1781f,  -0.8262f,-0.5345f,0.1780f,  -0.8588f,-0.5095f,0.0538f,  -0.8589f,-0.5093f,0.0536f,  -0.8770f,-0.4751f,-0.0713f,  -0.8773f,-0.4746f,-0.0715f,  -0.8806f,-0.4318f,-0.1951f,  -0.8811f,-0.4308f,-0.1953f,  -0.8695f,-0.3801f,-0.3153f,  
	-0.8701f,-0.3787f,-0.3155f,  -0.8439f,-0.3209f,-0.4299f,  -0.8445f,-0.3190f,-0.4302f,  -0.8041f,-0.2552f,-0.5369f,  -0.8048f,-0.2526f,-0.5371f,  -0.7508f,-0.1839f,-0.6344f,  -0.7516f,-0.1806f,-0.6344f,  -0.6849f,-0.1081f,-0.7206f,  -0.6857f,-0.1041f,-0.7204f,  -0.6075f,-0.0293f,-0.7938f,  -0.6083f,-0.0244f,-0.7934f,  -0.5198f,0.0514f,-0.8528f,  -0.5205f,0.0571f,-0.8519f,  -0.4233f,0.1325f,-0.8963f,  -0.4240f,0.1391f,-0.8949f,  
	-0.3196f,0.2125f,-0.9234f,  -0.3202f,0.2199f,-0.9215f,  -0.2104f,0.2900f,-0.9336f,  -0.2110f,0.2982f,-0.9309f,  -0.0977f,0.3634f,-0.9265f,  -0.0982f,0.3722f,-0.9230f,  0.0167f,0.4312f,-0.9021f,  0.0164f,0.4405f,-0.8976f,  0.1309f,0.4919f,-0.8608f,  0.1306f,0.5015f,-0.8552f,  0.2428f,0.5441f,-0.8031f,  0.2427f,0.5538f,-0.7965f,  0.3506f,0.5864f,-0.7302f,  0.3505f,0.5960f,-0.7224f,  0.4523f,0.6178f,-0.6432f,  
	0.4522f,0.6269f,-0.6344f,  0.5462f,0.6370f,-0.5439f,  0.5461f,0.6454f,-0.5341f,  0.6306f,0.6433f,-0.4342f,  0.6305f,0.6505f,-0.4234f,  0.7041f,0.6359f,-0.3162f,  0.7039f,0.6416f,-0.3048f,  0.7652f,0.6144f,-0.1924f,  0.7649f,0.6184f,-0.1806f,  0.8129f,0.5787f,-0.0655f,  0.8125f,0.5805f,-0.0535f,  0.8464f,0.5290f,0.0619f,  0.8458f,0.5284f,0.0737f,  0.8650f,0.4656f,0.1869f,  0.8643f,0.4624f,0.1980f,  
	0.8685f,0.3895f,0.3066f,  0.8676f,0.3834f,0.3166f,  0.8568f,0.3017f,0.4181f,  0.8558f,0.2927f,0.4265f,  0.8302f,0.2037f,0.5189f,  0.8292f,0.1918f,0.5251f,  0.7892f,0.0975f,0.6064f,  0.7881f,0.0827f,0.6100f,  0.7345f,-0.0150f,0.6785f,  0.7335f,-0.0324f,0.6789f,  0.6672f,-0.1312f,0.7332f,  0.6665f,-0.1509f,0.7301f,  0.5887f,-0.2485f,0.7692f,  0.5882f,-0.2701f,0.7623f,  0.5004f,-0.3641f,0.7855f,  
	0.5003f,-0.3870f,0.7746f,  0.4039f,-0.4750f,0.7818f,  0.4042f,-0.4987f,0.7667f,  0.3008f,-0.5783f,0.7583f,  0.3018f,-0.6021f,0.7392f,  0.1932f,-0.6711f,0.7158f,  0.1947f,-0.6943f,0.6928f,  0.0828f,-0.7505f,0.6556f,  0.0848f,-0.7725f,0.6293f,  -0.0288f,-0.8142f,0.5799f,  -0.0262f,-0.8342f,0.5509f,  -0.1396f,-0.8599f,0.4911f,  -0.1366f,-0.8773f,0.4601f,  -0.2480f,-0.8858f,0.3922f,  -0.2448f,-0.9002f,0.3602f,  
	-0.3525f,-0.8908f,0.2867f,  -0.3493f,-0.9017f,0.2548f,  -0.4517f,-0.8742f,0.1783f,  -0.4488f,-0.8814f,0.1476f,  -0.5443f,-0.8359f,0.0708f,  -0.5420f,-0.8393f,0.0425f,  -0.6292f,-0.7766f,-0.0319f,  -0.6277f,-0.7764f,-0.0566f,  -0.7054f,-0.6975f,-0.1261f,  -0.7051f,-0.6940f,-0.1459f,  -0.7203f,-0.5552f,0.4159f,  -0.7200f,-0.5558f,0.4157f,  -0.7797f,-0.5499f,0.2995f,  -0.7795f,-0.5503f,0.2992f,  -0.8262f,-0.5345f,0.1780f,  
	-0.8261f,-0.5347f,0.1777f,  -0.8589f,-0.5093f,0.0536f,  -0.8590f,-0.5093f,0.0533f,  -0.8773f,-0.4746f,-0.0715f,  -0.8775f,-0.4742f,-0.0719f,  -0.8811f,-0.4308f,-0.1953f,  -0.8813f,-0.4300f,-0.1957f,  -0.8701f,-0.3787f,-0.3155f,  -0.8705f,-0.3774f,-0.3160f,  -0.8445f,-0.3190f,-0.4302f,  -0.8450f,-0.3171f,-0.4306f,  -0.8048f,-0.2526f,-0.5371f,  -0.8054f,-0.2500f,-0.5375f,  -0.7516f,-0.1806f,-0.6344f,  -0.7522f,-0.1772f,-0.6347f,  
	-0.6857f,-0.1041f,-0.7204f,  -0.6863f,-0.0998f,-0.7204f,  -0.6083f,-0.0244f,-0.7934f,  -0.6089f,-0.0193f,-0.7930f,  -0.5205f,0.0571f,-0.8519f,  -0.5211f,0.0632f,-0.8511f,  -0.4240f,0.1391f,-0.8949f,  -0.4245f,0.1461f,-0.8936f,  -0.3202f,0.2199f,-0.9215f,  -0.3207f,0.2278f,-0.9194f,  -0.2110f,0.2982f,-0.9309f,  -0.2114f,0.3068f,-0.9280f,  -0.0982f,0.3722f,-0.9230f,  -0.0984f,0.3815f,-0.9191f,  0.0164f,0.4405f,-0.8976f,  
	0.0162f,0.4504f,-0.8927f,  0.1306f,0.5015f,-0.8552f,  0.1305f,0.5117f,-0.8492f,  0.2427f,0.5538f,-0.7965f,  0.2427f,0.5641f,-0.7892f,  0.3505f,0.5960f,-0.7224f,  0.3505f,0.6061f,-0.7140f,  0.4522f,0.6269f,-0.6344f,  0.4523f,0.6364f,-0.6248f,  0.5461f,0.6454f,-0.5341f,  0.5462f,0.6540f,-0.5234f,  0.6305f,0.6505f,-0.4234f,  0.6305f,0.6579f,-0.4119f,  0.7039f,0.6416f,-0.3048f,  0.7038f,0.6474f,-0.2925f,  
	0.7649f,0.6184f,-0.1806f,  0.7647f,0.6222f,-0.1679f,  0.8125f,0.5805f,-0.0535f,  0.8121f,0.5821f,-0.0407f,  0.8458f,0.5284f,0.0737f,  0.8453f,0.5273f,0.0861f,  0.8643f,0.4624f,0.1980f,  0.8636f,0.4585f,0.2096f,  0.8676f,0.3834f,0.3166f,  0.8668f,0.3766f,0.3268f,  0.8558f,0.2927f,0.4265f,  0.8550f,0.2829f,0.4348f,  0.8292f,0.1918f,0.5251f,  0.8282f,0.1790f,0.5311f,  0.7881f,0.0827f,0.6100f,  
	0.7872f,0.0670f,0.6130f,  0.7335f,-0.0324f,0.6789f,  0.7328f,-0.0506f,0.6786f,  0.6665f,-0.1509f,0.7301f,  0.6660f,-0.1714f,0.7260f,  0.5882f,-0.2701f,0.7623f,  0.5882f,-0.2924f,0.7540f,  0.5003f,-0.3870f,0.7746f,  0.5006f,-0.4106f,0.7621f,  0.4042f,-0.4987f,0.7667f,  0.4052f,-0.5229f,0.7499f,  0.3018f,-0.6021f,0.7392f,  0.3034f,-0.6262f,0.7182f,  0.1947f,-0.6943f,0.6928f,  0.1969f,-0.7176f,0.6681f,  
	0.0848f,-0.7725f,0.6293f,  0.0876f,-0.7943f,0.6012f,  -0.0262f,-0.8342f,0.5509f,  -0.0229f,-0.8538f,0.5201f,  -0.1366f,-0.8773f,0.4601f,  -0.1329f,-0.8942f,0.4276f,  -0.2448f,-0.9002f,0.3602f,  -0.2409f,-0.9138f,0.3269f,  -0.3493f,-0.9017f,0.2548f,  -0.3456f,-0.9118f,0.2218f,  -0.4488f,-0.8814f,0.1476f,  -0.4455f,-0.8877f,0.1162f,  -0.5420f,-0.8393f,0.0425f,  -0.5393f,-0.8420f,0.0139f,  -0.6277f,-0.7764f,-0.0566f,  
	-0.6261f,-0.7755f,-0.0810f,  -0.7051f,-0.6940f,-0.1459f,  -0.7047f,-0.6900f,-0.1651f,  -0.7200f,-0.5558f,0.4157f,  -0.7195f,-0.5567f,0.4152f,  -0.7795f,-0.5503f,0.2992f,  -0.7792f,-0.5510f,0.2987f,  -0.8261f,-0.5347f,0.1777f,  -0.8259f,-0.5352f,0.1772f,  -0.8590f,-0.5093f,0.0533f,  -0.8589f,-0.5094f,0.0527f,  -0.8775f,-0.4742f,-0.0719f,  -0.8776f,-0.4740f,-0.0726f,  -0.8813f,-0.4300f,-0.1957f,  -0.8815f,-0.4293f,-0.1965f,  
	-0.8705f,-0.3774f,-0.3160f,  -0.8708f,-0.3761f,-0.3167f,  -0.8450f,-0.3171f,-0.4306f,  -0.8454f,-0.3151f,-0.4313f,  -0.8054f,-0.2500f,-0.5375f,  -0.8058f,-0.2472f,-0.5381f,  -0.7522f,-0.1772f,-0.6347f,  -0.7527f,-0.1736f,-0.6351f,  -0.6863f,-0.0998f,-0.7204f,  -0.6868f,-0.0953f,-0.7205f,  -0.6089f,-0.0193f,-0.7930f,  -0.6094f,-0.0138f,-0.7928f,  -0.5211f,0.0632f,-0.8511f,  -0.5216f,0.0697f,-0.8504f,  -0.4245f,0.1461f,-0.8936f,  
	-0.4249f,0.1536f,-0.8921f,  -0.3207f,0.2278f,-0.9194f,  -0.3210f,0.2362f,-0.9172f,  -0.2114f,0.3068f,-0.9280f,  -0.2116f,0.3161f,-0.9248f,  -0.0984f,0.3815f,-0.9191f,  -0.0985f,0.3915f,-0.9149f,  0.0162f,0.4504f,-0.8927f,  0.0162f,0.4608f,-0.8874f,  0.1305f,0.5117f,-0.8492f,  0.1307f,0.5225f,-0.8426f,  0.2427f,0.5641f,-0.7892f,  0.2428f,0.5748f,-0.7814f,  0.3505f,0.6061f,-0.7140f,  0.3507f,0.6166f,-0.7049f,  
	0.4523f,0.6364f,-0.6248f,  0.4525f,0.6463f,-0.6144f,  0.5462f,0.6540f,-0.5234f,  0.5464f,0.6629f,-0.5120f,  0.6305f,0.6579f,-0.4119f,  0.6306f,0.6654f,-0.3995f,  0.7038f,0.6474f,-0.2925f,  0.7037f,0.6532f,-0.2795f,  0.7647f,0.6222f,-0.1679f,  0.7645f,0.6258f,-0.1545f,  0.8121f,0.5821f,-0.0407f,  0.8118f,0.5833f,-0.0274f,  0.8453f,0.5273f,0.0861f,  0.8448f,0.5258f,0.0990f,  0.8636f,0.4585f,0.2096f,  
	0.8630f,0.4541f,0.2214f,  0.8668f,0.3766f,0.3268f,  0.8661f,0.3691f,0.3370f,  0.8550f,0.2829f,0.4348f,  0.8542f,0.2722f,0.4431f,  0.8282f,0.1790f,0.5311f,  0.8275f,0.1653f,0.5367f,  0.7872f,0.0670f,0.6130f,  0.7866f,0.0504f,0.6154f,  0.7328f,-0.0506f,0.6786f,  0.7324f,-0.0698f,0.6773f,  0.6660f,-0.1714f,0.7260f,  0.6660f,-0.1928f,0.7206f,  0.5882f,-0.2924f,0.7540f,  0.5885f,-0.3155f,0.7444f,  
	0.5006f,-0.4106f,0.7621f,  0.5017f,-0.4347f,0.7479f,  0.4052f,-0.5229f,0.7499f,  0.4069f,-0.5474f,0.7313f,  0.3034f,-0.6262f,0.7182f,  0.3057f,-0.6504f,0.6954f,  0.1969f,-0.7176f,0.6681f,  0.1999f,-0.7408f,0.6413f,  0.0876f,-0.7943f,0.6012f,  0.0912f,-0.8157f,0.5712f,  -0.0229f,-0.8538f,0.5201f,  -0.0188f,-0.8729f,0.4876f,  -0.1329f,-0.8942f,0.4276f,  -0.1285f,-0.9103f,0.3935f,  -0.2409f,-0.9138f,0.3269f,  
	-0.2365f,-0.9266f,0.2923f,  -0.3456f,-0.9118f,0.2218f,  -0.3413f,-0.9210f,0.1879f,  -0.4455f,-0.8877f,0.1162f,  -0.4417f,-0.8932f,0.0842f,  -0.5393f,-0.8420f,0.0139f,  -0.5364f,-0.8438f,-0.0148f,  -0.6261f,-0.7755f,-0.0810f,  -0.6243f,-0.7740f,-0.1053f,  -0.7047f,-0.6900f,-0.1651f,  -0.7044f,-0.6857f,-0.1836f,  -0.7195f,-0.5567f,0.4152f,  -0.7189f,-0.5579f,0.4146f,  -0.7792f,-0.5510f,0.2987f,  -0.7788f,-0.5520f,0.2980f,  
	-0.8259f,-0.5352f,0.1772f,  -0.8256f,-0.5359f,0.1764f,  -0.8589f,-0.5094f,0.0527f,  -0.8587f,-0.5098f,0.0518f,  -0.8776f,-0.4740f,-0.0726f,  -0.8775f,-0.4739f,-0.0735f,  -0.8815f,-0.4293f,-0.1965f,  -0.8816f,-0.4287f,-0.1974f,  -0.8708f,-0.3761f,-0.3167f,  -0.8710f,-0.3748f,-0.3178f,  -0.8454f,-0.3151f,-0.4313f,  -0.8457f,-0.3130f,-0.4323f,  -0.8058f,-0.2472f,-0.5381f,  -0.8061f,-0.2443f,-0.5389f,  -0.7527f,-0.1736f,-0.6351f,  
	-0.7530f,-0.1697f,-0.6357f,  -0.6868f,-0.0953f,-0.7205f,  -0.6872f,-0.0905f,-0.7208f,  -0.6094f,-0.0138f,-0.7928f,  -0.6097f,-0.0079f,-0.7926f,  -0.5216f,0.0697f,-0.8504f,  -0.5218f,0.0767f,-0.8496f,  -0.4249f,0.1536f,-0.8921f,  -0.4251f,0.1615f,-0.8906f,  -0.3210f,0.2362f,-0.9172f,  -0.3211f,0.2452f,-0.9148f,  -0.2116f,0.3161f,-0.9248f,  -0.2115f,0.3259f,-0.9214f,  -0.0985f,0.3915f,-0.9149f,  -0.0984f,0.4020f,-0.9103f,  
	0.0162f,0.4608f,-0.8874f,  0.0164f,0.4719f,-0.8815f,  0.1307f,0.5225f,-0.8426f,  0.1309f,0.5338f,-0.8354f,  0.2428f,0.5748f,-0.7814f,  0.2432f,0.5861f,-0.7729f,  0.3507f,0.6166f,-0.7049f,  0.3511f,0.6275f,-0.6949f,  0.4525f,0.6463f,-0.6144f,  0.4529f,0.6565f,-0.6033f,  0.5464f,0.6629f,-0.5120f,  0.5466f,0.6720f,-0.4997f,  0.6306f,0.6654f,-0.3995f,  0.6308f,0.6730f,-0.3863f,  0.7037f,0.6532f,-0.2795f,  
	0.7038f,0.6589f,-0.2656f,  0.7645f,0.6258f,-0.1545f,  0.7644f,0.6293f,-0.1403f,  0.8118f,0.5833f,-0.0274f,  0.8116f,0.5841f,-0.0133f,  0.8448f,0.5258f,0.0990f,  0.8444f,0.5238f,0.1123f,  0.8630f,0.4541f,0.2214f,  0.8625f,0.4490f,0.2335f,  0.8661f,0.3691f,0.3370f,  0.8655f,0.3607f,0.3475f,  0.8542f,0.2722f,0.4431f,  0.8536f,0.2607f,0.4511f,  0.8275f,0.1653f,0.5367f,  0.8270f,0.1507f,0.5417f,  
	0.7866f,0.0504f,0.6154f,  0.7863f,0.0329f,0.6170f,  0.7324f,-0.0698f,0.6773f,  0.7324f,-0.0899f,0.6749f,  0.6660f,-0.1928f,0.7206f,  0.6664f,-0.2149f,0.7140f,  0.5885f,-0.3155f,0.7444f,  0.5896f,-0.3391f,0.7331f,  0.5017f,-0.4347f,0.7479f,  0.5034f,-0.4592f,0.7319f,  0.4069f,-0.5474f,0.7313f,  0.4092f,-0.5721f,0.7108f,  0.3057f,-0.6504f,0.6954f,  0.3088f,-0.6746f,0.6705f,  0.1999f,-0.7408f,0.6413f,  
	0.2038f,-0.7637f,0.6126f,  0.0912f,-0.8157f,0.5712f,  0.0957f,-0.8367f,0.5393f,  -0.0188f,-0.8729f,0.4876f,  -0.0138f,-0.8913f,0.4533f,  -0.1285f,-0.9103f,0.3935f,  -0.1234f,-0.9256f,0.3577f,  -0.2365f,-0.9266f,0.2923f,  -0.2314f,-0.9385f,0.2564f,  -0.3413f,-0.9210f,0.1879f,  -0.3365f,-0.9292f,0.1530f,  -0.4417f,-0.8932f,0.0842f,  -0.4376f,-0.8977f,0.0517f,  -0.5364f,-0.8438f,-0.0148f,  -0.5333f,-0.8448f,-0.0436f,  
	-0.6243f,-0.7740f,-0.1053f,  -0.6225f,-0.7719f,-0.1292f,  -0.7044f,-0.6857f,-0.1836f,  -0.7040f,-0.6810f,-0.2014f,  -0.7189f,-0.5579f,0.4146f,  -0.7183f,-0.5594f,0.4137f,  -0.7788f,-0.5520f,0.2980f,  -0.7782f,-0.5533f,0.2970f,  -0.8256f,-0.5359f,0.1764f,  -0.8252f,-0.5369f,0.1753f,  -0.8587f,-0.5098f,0.0518f,  -0.8585f,-0.5103f,0.0506f,  -0.8775f,-0.4739f,-0.0735f,  -0.8774f,-0.4739f,-0.0748f,  -0.8816f,-0.4287f,-0.1974f,  
	-0.8816f,-0.4281f,-0.1988f,  -0.8710f,-0.3748f,-0.3178f,  -0.8710f,-0.3735f,-0.3191f,  -0.8457f,-0.3130f,-0.4323f,  -0.8458f,-0.3109f,-0.4335f,  -0.8061f,-0.2443f,-0.5389f,  -0.8063f,-0.2412f,-0.5400f,  -0.7530f,-0.1697f,-0.6357f,  -0.7532f,-0.1656f,-0.6366f,  -0.6872f,-0.0905f,-0.7208f,  -0.6874f,-0.0853f,-0.7213f,  -0.6097f,-0.0079f,-0.7926f,  -0.6098f,-0.0015f,-0.7925f,  -0.5218f,0.0767f,-0.8496f,  -0.5219f,0.0841f,-0.8488f,  
	-0.4251f,0.1615f,-0.8906f,  -0.4251f,0.1701f,-0.8890f,  -0.3211f,0.2452f,-0.9148f,  -0.3209f,0.2548f,-0.9122f,  -0.2115f,0.3259f,-0.9214f,  -0.2113f,0.3364f,-0.9177f,  -0.0984f,0.4020f,-0.9103f,  -0.0981f,0.4133f,-0.9053f,  0.0164f,0.4719f,-0.8815f,  0.0168f,0.4836f,-0.8751f,  0.1309f,0.5338f,-0.8354f,  0.1314f,0.5457f,-0.8276f,  0.2432f,0.5861f,-0.7729f,  0.2437f,0.5980f,-0.7636f,  0.3511f,0.6275f,-0.6949f,  
	0.3516f,0.6389f,-0.6843f,  0.4529f,0.6565f,-0.6033f,  0.4533f,0.6670f,-0.5912f,  0.5466f,0.6720f,-0.4997f,  0.5470f,0.6813f,-0.4865f,  0.6308f,0.6730f,-0.3863f,  0.6311f,0.6806f,-0.3721f,  0.7038f,0.6589f,-0.2656f,  0.7040f,0.6645f,-0.2509f,  0.7644f,0.6293f,-0.1403f,  0.7644f,0.6324f,-0.1254f,  0.8116f,0.5841f,-0.0133f,  0.8114f,0.5845f,0.0013f,  0.8444f,0.5238f,0.1123f,  0.8441f,0.5212f,0.1260f,  
	0.8625f,0.4490f,0.2335f,  0.8621f,0.4431f,0.2459f,  0.8655f,0.3607f,0.3475f,  0.8651f,0.3516f,0.3578f,  0.8536f,0.2607f,0.4511f,  0.8532f,0.2483f,0.4587f,  0.8270f,0.1507f,0.5417f,  0.8267f,0.1352f,0.5462f,  0.7863f,0.0329f,0.6170f,  0.7863f,0.0146f,0.6176f,  0.7324f,-0.0899f,0.6749f,  0.7328f,-0.1107f,0.6713f,  0.6664f,-0.2149f,0.7140f,  0.6675f,-0.2376f,0.7057f,  0.5896f,-0.3391f,0.7331f,  
	0.5913f,-0.3632f,0.7200f,  0.5034f,-0.4592f,0.7319f,  0.5058f,-0.4841f,0.7141f,  0.4092f,-0.5721f,0.7108f,  0.4124f,-0.5969f,0.6882f,  0.3088f,-0.6746f,0.6705f,  0.3128f,-0.6986f,0.6435f,  0.2038f,-0.7637f,0.6126f,  0.2085f,-0.7862f,0.5818f,  0.0957f,-0.8367f,0.5393f,  0.1010f,-0.8570f,0.5053f,  -0.0138f,-0.8913f,0.4533f,  -0.0082f,-0.9088f,0.4171f,  -0.1234f,-0.9256f,0.3577f,  -0.1175f,-0.9399f,0.3205f,  
	-0.2314f,-0.9385f,0.2564f,  -0.2256f,-0.9492f,0.2192f,  -0.3365f,-0.9292f,0.1530f,  -0.3313f,-0.9362f,0.1173f,  -0.4376f,-0.8977f,0.0517f,  -0.4332f,-0.9011f,0.0187f,  -0.5333f,-0.8448f,-0.0436f,  -0.5300f,-0.8449f,-0.0725f,  -0.6225f,-0.7719f,-0.1292f,  -0.6206f,-0.7691f,-0.1527f,  -0.7040f,-0.6810f,-0.2014f,  -0.7037f,-0.6760f,-0.2185f,  -0.7183f,-0.5594f,0.4137f,  -0.7175f,-0.5612f,0.4126f,  -0.7782f,-0.5533f,0.2970f,  
	-0.7776f,-0.5548f,0.2958f,  -0.8252f,-0.5369f,0.1753f,  -0.8248f,-0.5381f,0.1739f,  -0.8585f,-0.5103f,0.0506f,  -0.8581f,-0.5110f,0.0491f,  -0.8774f,-0.4739f,-0.0748f,  -0.8772f,-0.4741f,-0.0764f,  -0.8816f,-0.4281f,-0.1988f,  -0.8815f,-0.4276f,-0.2004f,  -0.8710f,-0.3735f,-0.3191f,  -0.8710f,-0.3721f,-0.3207f,  -0.8458f,-0.3109f,-0.4335f,  -0.8458f,-0.3086f,-0.4351f,  -0.8063f,-0.2412f,-0.5400f,  -0.8064f,-0.2380f,-0.5414f,  
	-0.7532f,-0.1656f,-0.6366f,  -0.7533f,-0.1612f,-0.6376f,  -0.6874f,-0.0853f,-0.7213f,  -0.6874f,-0.0797f,-0.7219f,  -0.6098f,-0.0015f,-0.7925f,  -0.6098f,0.0053f,-0.7925f,  -0.5219f,0.0841f,-0.8488f,  -0.5218f,0.0921f,-0.8481f,  -0.4251f,0.1701f,-0.8890f,  -0.4249f,0.1793f,-0.8873f,  -0.3209f,0.2548f,-0.9122f,  -0.3206f,0.2650f,-0.9094f,  -0.2113f,0.3364f,-0.9177f,  -0.2109f,0.3476f,-0.9136f,  -0.0981f,0.4133f,-0.9053f,  
	-0.0976f,0.4251f,-0.8999f,  0.0168f,0.4836f,-0.8751f,  0.0174f,0.4959f,-0.8682f,  0.1314f,0.5457f,-0.8276f,  0.1321f,0.5582f,-0.8191f,  0.2437f,0.5980f,-0.7636f,  0.2443f,0.6103f,-0.7535f,  0.3516f,0.6389f,-0.6843f,  0.3522f,0.6507f,-0.6727f,  0.4533f,0.6670f,-0.5912f,  0.4539f,0.6779f,-0.5783f,  0.5470f,0.6813f,-0.4865f,  0.5475f,0.6907f,-0.4723f,  0.6311f,0.6806f,-0.3721f,  0.6314f,0.6883f,-0.3572f,  
	0.7040f,0.6645f,-0.2509f,  0.7042f,0.6699f,-0.2354f,  0.7644f,0.6324f,-0.1254f,  0.7644f,0.6353f,-0.1098f,  0.8114f,0.5845f,0.0013f,  0.8112f,0.5845f,0.0163f,  0.8441f,0.5212f,0.1260f,  0.8438f,0.5180f,0.1402f,  0.8621f,0.4431f,0.2459f,  0.8618f,0.4366f,0.2584f,  0.8651f,0.3516f,0.3578f,  0.8648f,0.3417f,0.3679f,  0.8532f,0.2483f,0.4587f,  0.8530f,0.2351f,0.4660f,  0.8267f,0.1352f,0.5462f,  
	0.8268f,0.1188f,0.5498f,  0.7863f,0.0146f,0.6176f,  0.7868f,-0.0046f,0.6172f,  0.7328f,-0.1107f,0.6713f,  0.7339f,-0.1321f,0.6663f,  0.6675f,-0.2376f,0.7057f,  0.6691f,-0.2610f,0.6958f,  0.5913f,-0.3632f,0.7200f,  0.5937f,-0.3877f,0.7051f,  0.5058f,-0.4841f,0.7141f,  0.5090f,-0.5090f,0.6941f,  0.4124f,-0.5969f,0.6882f,  0.4166f,-0.6216f,0.6634f,  0.3128f,-0.6986f,0.6435f,  0.3178f,-0.7223f,0.6143f,  
	0.2085f,-0.7862f,0.5818f,  0.2142f,-0.8081f,0.5487f,  0.1010f,-0.8570f,0.5053f,  0.1072f,-0.8765f,0.4693f,  -0.0082f,-0.9088f,0.4171f,  -0.0016f,-0.9253f,0.3791f,  -0.1175f,-0.9399f,0.3205f,  -0.1109f,-0.9531f,0.2817f,  -0.2256f,-0.9492f,0.2192f,  -0.2194f,-0.9587f,0.1809f,  -0.3313f,-0.9362f,0.1173f,  -0.3256f,-0.9420f,0.0807f,  -0.4332f,-0.9011f,0.0187f,  -0.4285f,-0.9034f,-0.0147f,  -0.5300f,-0.8449f,-0.0725f,  
	-0.5265f,-0.8441f,-0.1014f,  -0.6206f,-0.7691f,-0.1527f,  -0.6187f,-0.7657f,-0.1759f,  -0.7037f,-0.6760f,-0.2185f,  -0.7035f,-0.6708f,-0.2349f,  -0.7175f,-0.5612f,0.4126f,  -0.7167f,-0.5632f,0.4112f,  -0.7776f,-0.5548f,0.2958f,  -0.7769f,-0.5566f,0.2943f,  -0.8248f,-0.5381f,0.1739f,  -0.8242f,-0.5394f,0.1722f,  -0.8581f,-0.5110f,0.0491f,  -0.8577f,-0.5119f,0.0473f,  -0.8772f,-0.4741f,-0.0764f,  -0.8769f,-0.4743f,-0.0783f,  
	-0.8815f,-0.4276f,-0.2004f,  -0.8813f,-0.4270f,-0.2024f,  -0.8710f,-0.3721f,-0.3207f,  -0.8709f,-0.3707f,-0.3227f,  -0.8458f,-0.3086f,-0.4351f,  -0.8457f,-0.3062f,-0.4370f,  -0.8064f,-0.2380f,-0.5414f,  -0.8063f,-0.2344f,-0.5431f,  -0.7533f,-0.1612f,-0.6376f,  -0.7532f,-0.1565f,-0.6389f,  -0.6874f,-0.0797f,-0.7219f,  -0.6872f,-0.0737f,-0.7227f,  -0.6098f,0.0053f,-0.7925f,  -0.6096f,0.0126f,-0.7926f,  -0.5218f,0.0921f,-0.8481f,  
	-0.5215f,0.1007f,-0.8473f,  -0.4249f,0.1793f,-0.8873f,  -0.4245f,0.1890f,-0.8855f,  -0.3206f,0.2650f,-0.9094f,  -0.3201f,0.2759f,-0.9063f,  -0.2109f,0.3476f,-0.9136f,  -0.2103f,0.3593f,-0.9092f,  -0.0976f,0.4251f,-0.8999f,  -0.0969f,0.4376f,-0.8939f,  0.0174f,0.4959f,-0.8682f,  0.0182f,0.5089f,-0.8606f,  0.1321f,0.5582f,-0.8191f,  0.1329f,0.5713f,-0.8099f,  0.2443f,0.6103f,-0.7535f,  0.2451f,0.6232f,-0.7427f,  
	0.3522f,0.6507f,-0.6727f,  0.3530f,0.6629f,-0.6603f,  0.4539f,0.6779f,-0.5783f,  0.4546f,0.6890f,-0.5644f,  0.5475f,0.6907f,-0.4723f,  0.5481f,0.7003f,-0.4573f,  0.6314f,0.6883f,-0.3572f,  0.6318f,0.6959f,-0.3413f,  0.7042f,0.6699f,-0.2354f,  0.7044f,0.6752f,-0.2190f,  0.7644f,0.6353f,-0.1098f,  0.7645f,0.6378f,-0.0937f,  0.8112f,0.5845f,0.0163f,  0.8112f,0.5839f,0.0320f,  0.8438f,0.5180f,0.1402f,  
	0.8437f,0.5141f,0.1546f,  0.8618f,0.4366f,0.2584f,  0.8616f,0.4293f,0.2708f,  0.8648f,0.3417f,0.3679f,  0.8647f,0.3310f,0.3778f,  0.8530f,0.2351f,0.4660f,  0.8531f,0.2210f,0.4726f,  0.8268f,0.1188f,0.5498f,  0.8272f,0.1017f,0.5526f,  0.7868f,-0.0046f,0.6172f,  0.7878f,-0.0244f,0.6155f,  0.7339f,-0.1321f,0.6663f,  0.7355f,-0.1543f,0.6597f,  0.6691f,-0.2610f,0.6958f,  0.6715f,-0.2847f,0.6842f,  
	0.5937f,-0.3877f,0.7051f,  0.5969f,-0.4124f,0.6882f,  0.5090f,-0.5090f,0.6941f,  0.5132f,-0.5340f,0.6720f,  0.4166f,-0.6216f,0.6634f,  0.4216f,-0.6460f,0.6363f,  0.3178f,-0.7223f,0.6143f,  0.3237f,-0.7454f,0.5828f,  0.2142f,-0.8081f,0.5487f,  0.2207f,-0.8292f,0.5135f,  0.1072f,-0.8765f,0.4693f,  0.1143f,-0.8950f,0.4313f,  -0.0016f,-0.9253f,0.3791f,  0.0057f,-0.9407f,0.3393f,  -0.1109f,-0.9531f,0.2817f,  
	-0.1037f,-0.9649f,0.2414f,  -0.2194f,-0.9587f,0.1809f,  -0.2125f,-0.9669f,0.1414f,  -0.3256f,-0.9420f,0.0807f,  -0.3196f,-0.9466f,0.0435f,  -0.4285f,-0.9034f,-0.0147f,  -0.4235f,-0.9046f,-0.0483f,  -0.5265f,-0.8441f,-0.1014f,  -0.5231f,-0.8423f,-0.1302f,  -0.6187f,-0.7657f,-0.1759f,  -0.6168f,-0.7617f,-0.1985f,  -0.7035f,-0.6708f,-0.2349f,  -0.7032f,-0.6654f,-0.2504f,  -0.7167f,-0.5632f,0.4112f,  -0.7158f,-0.5656f,0.4095f,  
	-0.7769f,-0.5566f,0.2943f,  -0.7762f,-0.5586f,0.2924f,  -0.8242f,-0.5394f,0.1722f,  -0.8236f,-0.5410f,0.1702f,  -0.8577f,-0.5119f,0.0473f,  -0.8572f,-0.5129f,0.0451f,  -0.8769f,-0.4743f,-0.0783f,  -0.8765f,-0.4746f,-0.0807f,  -0.8813f,-0.4270f,-0.2024f,  -0.8810f,-0.4265f,-0.2048f,  -0.8709f,-0.3707f,-0.3227f,  -0.8706f,-0.3692f,-0.3251f,  -0.8457f,-0.3062f,-0.4370f,  -0.8455f,-0.3036f,-0.4392f,  -0.8063f,-0.2344f,-0.5431f,  
	-0.8061f,-0.2306f,-0.5450f,  -0.7532f,-0.1565f,-0.6389f,  -0.7529f,-0.1514f,-0.6404f,  -0.6872f,-0.0737f,-0.7227f,  -0.6869f,-0.0672f,-0.7236f,  -0.6096f,0.0126f,-0.7926f,  -0.6092f,0.0204f,-0.7928f,  -0.5215f,0.1007f,-0.8473f,  -0.5210f,0.1098f,-0.8465f,  -0.4245f,0.1890f,-0.8855f,  -0.4238f,0.1995f,-0.8835f,  -0.3201f,0.2759f,-0.9063f,  -0.3194f,0.2874f,-0.9030f,  -0.2103f,0.3593f,-0.9092f,  -0.2095f,0.3719f,-0.9043f,  
	-0.0969f,0.4376f,-0.8939f,  -0.0959f,0.4509f,-0.8874f,  0.0182f,0.5089f,-0.8606f,  0.0191f,0.5225f,-0.8524f,  0.1329f,0.5713f,-0.8099f,  0.1338f,0.5850f,-0.7999f,  0.2451f,0.6232f,-0.7427f,  0.2461f,0.6365f,-0.7309f,  0.3530f,0.6629f,-0.6603f,  0.3539f,0.6755f,-0.6469f,  0.4546f,0.6890f,-0.5644f,  0.4554f,0.7004f,-0.5497f,  0.5481f,0.7003f,-0.4573f,  0.5487f,0.7100f,-0.4413f,  0.6318f,0.6959f,-0.3413f,  
	0.6323f,0.7035f,-0.3245f,  0.7044f,0.6752f,-0.2190f,  0.7047f,0.6802f,-0.2020f,  0.7645f,0.6378f,-0.0937f,  0.7646f,0.6399f,-0.0767f,  0.8112f,0.5839f,0.0320f,  0.8112f,0.5828f,0.0481f,  0.8437f,0.5141f,0.1546f,  0.8437f,0.5096f,0.1690f,  0.8616f,0.4293f,0.2708f,  0.8616f,0.4213f,0.2832f,  0.8647f,0.3310f,0.3778f,  0.8649f,0.3195f,0.3872f,  0.8531f,0.2210f,0.4726f,  0.8536f,0.2061f,0.4785f,  
	0.8272f,0.1017f,0.5526f,  0.8282f,0.0838f,0.5541f,  0.7878f,-0.0244f,0.6155f,  0.7893f,-0.0449f,0.6123f,  0.7355f,-0.1543f,0.6597f,  0.7377f,-0.1769f,0.6515f,  0.6715f,-0.2847f,0.6842f,  0.6746f,-0.3088f,0.6705f,  0.5969f,-0.4124f,0.6882f,  0.6010f,-0.4372f,0.6690f,  0.5132f,-0.5340f,0.6720f,  0.5182f,-0.5587f,0.6475f,  0.4216f,-0.6460f,0.6363f,  0.4277f,-0.6699f,0.6069f,  0.3237f,-0.7454f,0.5828f,  
	0.3305f,-0.7677f,0.5490f,  0.2207f,-0.8292f,0.5135f,  0.2283f,-0.8493f,0.4761f,  0.1143f,-0.8950f,0.4313f,  0.1223f,-0.9122f,0.3912f,  0.0057f,-0.9407f,0.3393f,  0.0138f,-0.9545f,0.2977f,  -0.1037f,-0.9649f,0.2414f,  -0.0958f,-0.9752f,0.1996f,  -0.2125f,-0.9669f,0.1414f,  -0.2053f,-0.9735f,0.1008f,  -0.3196f,-0.9466f,0.0435f,  -0.3133f,-0.9496f,0.0055f,  -0.4235f,-0.9046f,-0.0483f,  -0.4185f,-0.9045f,-0.0822f,  
	-0.5231f,-0.8423f,-0.1302f,  -0.5196f,-0.8396f,-0.1587f,  -0.6168f,-0.7617f,-0.1985f,  -0.6149f,-0.7571f,-0.2206f,  -0.7032f,-0.6654f,-0.2504f,  -0.7031f,-0.6598f,-0.2652f,  -0.7158f,-0.5656f,0.4095f,  -0.7149f,-0.5682f,0.4075f,  -0.7762f,-0.5586f,0.2924f,  -0.7754f,-0.5609f,0.2901f,  -0.8236f,-0.5410f,0.1702f,  -0.8230f,-0.5428f,0.1677f,  -0.8572f,-0.5129f,0.0451f,  -0.8567f,-0.5141f,0.0424f,  -0.8765f,-0.4746f,-0.0807f,  
	-0.8760f,-0.4750f,-0.0834f,  -0.8810f,-0.4265f,-0.2048f,  -0.8806f,-0.4259f,-0.2076f,  -0.8706f,-0.3692f,-0.3251f,  -0.8703f,-0.3676f,-0.3278f,  -0.8455f,-0.3036f,-0.4392f,  -0.8452f,-0.3008f,-0.4418f,  -0.8061f,-0.2306f,-0.5450f,  -0.8057f,-0.2265f,-0.5473f,  -0.7529f,-0.1514f,-0.6404f,  -0.7525f,-0.1459f,-0.6422f,  -0.6869f,-0.0672f,-0.7236f,  -0.6865f,-0.0603f,-0.7247f,  -0.6092f,0.0204f,-0.7928f,  -0.6086f,0.0288f,-0.7929f,  
	-0.5210f,0.1098f,-0.8465f,  -0.5203f,0.1197f,-0.8456f,  -0.4238f,0.1995f,-0.8835f,  -0.4231f,0.2106f,-0.8813f,  -0.3194f,0.2874f,-0.9030f,  -0.3185f,0.2997f,-0.8993f,  -0.2095f,0.3719f,-0.9043f,  -0.2085f,0.3851f,-0.8990f,  -0.0959f,0.4509f,-0.8874f,  -0.0949f,0.4648f,-0.8803f,  0.0191f,0.5225f,-0.8524f,  0.0202f,0.5368f,-0.8435f,  0.1338f,0.5850f,-0.7999f,  0.1349f,0.5993f,-0.7891f,  0.2461f,0.6365f,-0.7309f,  
	0.2471f,0.6504f,-0.7183f,  0.3539f,0.6755f,-0.6469f,  0.3548f,0.6884f,-0.6326f,  0.4554f,0.7004f,-0.5497f,  0.4562f,0.7119f,-0.5339f,  0.5487f,0.7100f,-0.4413f,  0.5494f,0.7198f,-0.4244f,  0.6323f,0.7035f,-0.3245f,  0.6328f,0.7109f,-0.3070f,  0.7047f,0.6802f,-0.2020f,  0.7050f,0.6849f,-0.1841f,  0.7646f,0.6399f,-0.0767f,  0.7648f,0.6415f,-0.0593f,  0.8112f,0.5828f,0.0481f,  0.8113f,0.5811f,0.0644f,  
	0.8437f,0.5096f,0.1690f,  0.8438f,0.5043f,0.1837f,  0.8616f,0.4213f,0.2832f,  0.8618f,0.4124f,0.2953f,  0.8649f,0.3195f,0.3872f,  0.8653f,0.3071f,0.3961f,  0.8536f,0.2061f,0.4785f,  0.8545f,0.1906f,0.4833f,  0.8282f,0.0838f,0.5541f,  0.8296f,0.0652f,0.5545f,  0.7893f,-0.0449f,0.6123f,  0.7914f,-0.0660f,0.6077f,  0.7377f,-0.1769f,0.6515f,  0.7407f,-0.1999f,0.6414f,  0.6746f,-0.3088f,0.6705f,  
	0.6786f,-0.3330f,0.6547f,  0.6010f,-0.4372f,0.6690f,  0.6060f,-0.4619f,0.6476f,  0.5182f,-0.5587f,0.6475f,  0.5243f,-0.5830f,0.6206f,  0.4277f,-0.6699f,0.6069f,  0.4346f,-0.6932f,0.5750f,  0.3305f,-0.7677f,0.5490f,  0.3383f,-0.7891f,0.5127f,  0.2283f,-0.8493f,0.4761f,  0.2367f,-0.8681f,0.4363f,  0.1223f,-0.9122f,0.3912f,  0.1310f,-0.9279f,0.3490f,  0.0138f,-0.9545f,0.2977f,  0.0225f,-0.9668f,0.2544f,  
	-0.0958f,-0.9752f,0.1996f,  -0.0875f,-0.9838f,0.1565f,  -0.2053f,-0.9735f,0.1008f,  -0.1977f,-0.9785f,0.0593f,  -0.3133f,-0.9496f,0.0055f,  -0.3069f,-0.9512f,-0.0329f,  -0.4185f,-0.9045f,-0.0822f,  -0.4135f,-0.9031f,-0.1163f,  -0.5196f,-0.8396f,-0.1587f,  -0.5161f,-0.8359f,-0.1870f,  -0.6149f,-0.7571f,-0.2206f,  -0.6132f,-0.7519f,-0.2421f,  -0.7031f,-0.6598f,-0.2652f,  -0.7030f,-0.6542f,-0.2791f,  -0.7149f,-0.5682f,0.4075f,  
	-0.7139f,-0.5711f,0.4051f,  -0.7754f,-0.5609f,0.2901f,  -0.7746f,-0.5634f,0.2875f,  -0.8230f,-0.5428f,0.1677f,  -0.8223f,-0.5447f,0.1648f,  -0.8567f,-0.5141f,0.0424f,  -0.8561f,-0.5153f,0.0394f,  -0.8760f,-0.4750f,-0.0834f,  -0.8755f,-0.4753f,-0.0866f,  -0.8806f,-0.4259f,-0.2076f,  -0.8801f,-0.4253f,-0.2109f,  -0.8703f,-0.3676f,-0.3278f,  -0.8698f,-0.3658f,-0.3310f,  -0.8452f,-0.3008f,-0.4418f,  -0.8448f,-0.2977f,-0.4447f,  
	-0.8057f,-0.2265f,-0.5473f,  -0.8053f,-0.2220f,-0.5498f,  -0.7525f,-0.1459f,-0.6422f,  -0.7520f,-0.1399f,-0.6442f,  -0.6865f,-0.0603f,-0.7247f,  -0.6858f,-0.0528f,-0.7258f,  -0.6086f,0.0288f,-0.7929f,  -0.6079f,0.0378f,-0.7931f,  -0.5203f,0.1197f,-0.8456f,  -0.5195f,0.1302f,-0.8445f,  -0.4231f,0.2106f,-0.8813f,  -0.4221f,0.2224f,-0.8788f,  -0.3185f,0.2997f,-0.8993f,  -0.3174f,0.3128f,-0.8952f,  -0.2085f,0.3851f,-0.8990f,  
	-0.2073f,0.3991f,-0.8932f,  -0.0949f,0.4648f,-0.8803f,  -0.0937f,0.4794f,-0.8726f,  0.0202f,0.5368f,-0.8435f,  0.0215f,0.5518f,-0.8337f,  0.1349f,0.5993f,-0.7891f,  0.1361f,0.6142f,-0.7774f,  0.2471f,0.6504f,-0.7183f,  0.2482f,0.6647f,-0.7047f,  0.3548f,0.6884f,-0.6326f,  0.3559f,0.7017f,-0.6172f,  0.4562f,0.7119f,-0.5339f,  0.4571f,0.7237f,-0.5171f,  0.5494f,0.7198f,-0.4244f,  0.5500f,0.7295f,-0.4066f,  
	0.6328f,0.7109f,-0.3070f,  0.6333f,0.7182f,-0.2884f,  0.7050f,0.6849f,-0.1841f,  0.7053f,0.6893f,-0.1656f,  0.7648f,0.6415f,-0.0593f,  0.7650f,0.6427f,-0.0414f,  0.8113f,0.5811f,0.0644f,  0.8115f,0.5788f,0.0811f,  0.8438f,0.5043f,0.1837f,  0.8440f,0.4984f,0.1982f,  0.8618f,0.4124f,0.2953f,  0.8622f,0.4028f,0.3071f,  0.8653f,0.3071f,0.3961f,  0.8661f,0.2941f,0.4042f,  0.8545f,0.1906f,0.4833f,  
	0.8558f,0.1742f,0.4872f,  0.8296f,0.0652f,0.5545f,  0.8316f,0.0460f,0.5535f,  0.7914f,-0.0660f,0.6077f,  0.7943f,-0.0875f,0.6013f,  0.7407f,-0.1999f,0.6414f,  0.7446f,-0.2230f,0.6292f,  0.6786f,-0.3330f,0.6547f,  0.6835f,-0.3571f,0.6367f,  0.6060f,-0.4619f,0.6476f,  0.6120f,-0.4862f,0.6238f,  0.5243f,-0.5830f,0.6206f,  0.5313f,-0.6067f,0.5912f,  0.4346f,-0.6932f,0.5750f,  0.4426f,-0.7154f,0.5406f,  
	0.3383f,-0.7891f,0.5127f,  0.3471f,-0.8092f,0.4740f,  0.2367f,-0.8681f,0.4363f,  0.2460f,-0.8854f,0.3943f,  0.1310f,-0.9279f,0.3490f,  0.1405f,-0.9420f,0.3049f,  0.0225f,-0.9668f,0.2544f,  0.0318f,-0.9773f,0.2095f,  -0.0875f,-0.9838f,0.1565f,  -0.0788f,-0.9906f,0.1121f,  -0.1977f,-0.9785f,0.0593f,  -0.1900f,-0.9816f,0.0170f,  -0.3069f,-0.9512f,-0.0329f,  -0.3004f,-0.9511f,-0.0718f,  -0.4135f,-0.9031f,-0.1163f,  
	-0.4085f,-0.9003f,-0.1502f,  -0.5161f,-0.8359f,-0.1870f,  -0.5128f,-0.8312f,-0.2150f,  -0.6132f,-0.7519f,-0.2421f,  -0.6115f,-0.7462f,-0.2630f,  -0.7030f,-0.6542f,-0.2791f,  -0.7030f,-0.6484f,-0.2923f,  -0.7139f,-0.5711f,0.4051f,  -0.7130f,-0.5743f,0.4023f,  -0.7746f,-0.5634f,0.2875f,  -0.7738f,-0.5661f,0.2844f,  -0.8223f,-0.5447f,0.1648f,  -0.8215f,-0.5468f,0.1615f,  -0.8561f,-0.5153f,0.0394f,  -0.8555f,-0.5166f,0.0359f,  
	-0.8755f,-0.4753f,-0.0866f,  -0.8750f,-0.4757f,-0.0903f,  -0.8801f,-0.4253f,-0.2109f,  -0.8796f,-0.4246f,-0.2145f,  -0.8698f,-0.3658f,-0.3310f,  -0.8693f,-0.3638f,-0.3346f,  -0.8448f,-0.2977f,-0.4447f,  -0.8442f,-0.2943f,-0.4480f,  -0.8053f,-0.2220f,-0.5498f,  -0.8047f,-0.2171f,-0.5526f,  -0.7520f,-0.1399f,-0.6442f,  -0.7513f,-0.1334f,-0.6463f,  -0.6858f,-0.0528f,-0.7258f,  -0.6850f,-0.0447f,-0.7271f,  -0.6079f,0.0378f,-0.7931f,  
	-0.6070f,0.0475f,-0.7933f,  -0.5195f,0.1302f,-0.8445f,  -0.5184f,0.1414f,-0.8433f,  -0.4221f,0.2224f,-0.8788f,  -0.4209f,0.2351f,-0.8761f,  -0.3174f,0.3128f,-0.8952f,  -0.3162f,0.3266f,-0.8907f,  -0.2073f,0.3991f,-0.8932f,  -0.2060f,0.4139f,-0.8867f,  -0.0937f,0.4794f,-0.8726f,  -0.0924f,0.4948f,-0.8641f,  0.0215f,0.5518f,-0.8337f,  0.0228f,0.5674f,-0.8231f,  0.1361f,0.6142f,-0.7774f,  0.1374f,0.6296f,-0.7647f,  
	0.2482f,0.6647f,-0.7047f,  0.2494f,0.6794f,-0.6901f,  0.3559f,0.7017f,-0.6172f,  0.3569f,0.7153f,-0.6009f,  0.4571f,0.7237f,-0.5171f,  0.4579f,0.7355f,-0.4993f,  0.5500f,0.7295f,-0.4066f,  0.5507f,0.7392f,-0.3877f,  0.6333f,0.7182f,-0.2884f,  0.6338f,0.7252f,-0.2691f,  0.7053f,0.6893f,-0.1656f,  0.7057f,0.6933f,-0.1464f,  0.7650f,0.6427f,-0.0414f,  0.7653f,0.6433f,-0.0230f,  0.8115f,0.5788f,0.0811f,  
	0.8117f,0.5758f,0.0978f,  0.8440f,0.4984f,0.1982f,  0.8444f,0.4917f,0.2127f,  0.8622f,0.4028f,0.3071f,  0.8629f,0.3926f,0.3182f,  0.8661f,0.2941f,0.4042f,  0.8672f,0.2803f,0.4115f,  0.8558f,0.1742f,0.4872f,  0.8575f,0.1573f,0.4898f,  0.8316f,0.0460f,0.5535f,  0.8342f,0.0263f,0.5509f,  0.7943f,-0.0875f,0.6013f,  0.7978f,-0.1093f,0.5929f,  0.7446f,-0.2230f,0.6292f,  0.7492f,-0.2463f,0.6148f,  
	0.6835f,-0.3571f,0.6367f,  0.6893f,-0.3810f,0.6162f,  0.6120f,-0.4862f,0.6238f,  0.6189f,-0.5099f,0.5974f,  0.5313f,-0.6067f,0.5912f,  0.5393f,-0.6295f,0.5593f,  0.4426f,-0.7154f,0.5406f,  0.4516f,-0.7365f,0.5036f,  0.3471f,-0.8092f,0.4740f,  0.3567f,-0.8278f,0.4329f,  0.2460f,-0.8854f,0.3943f,  0.2560f,-0.9010f,0.3501f,  0.1405f,-0.9420f,0.3049f,  0.1505f,-0.9541f,0.2588f,  0.0318f,-0.9773f,0.2095f,  
	0.0415f,-0.9858f,0.1629f,  -0.0788f,-0.9906f,0.1121f,  -0.0698f,-0.9953f,0.0665f,  -0.1900f,-0.9816f,0.0170f,  -0.1821f,-0.9829f,-0.0261f,  -0.3004f,-0.9511f,-0.0718f,  -0.2939f,-0.9494f,-0.1109f,  -0.4085f,-0.9003f,-0.1502f,  -0.4037f,-0.8962f,-0.1841f,  -0.5128f,-0.8312f,-0.2150f,  -0.5096f,-0.8255f,-0.2424f,  -0.6115f,-0.7462f,-0.2630f,  -0.6101f,-0.7400f,-0.2832f,  -0.7030f,-0.6484f,-0.2923f,  -0.7030f,-0.6426f,-0.3046f,  
	-0.7130f,-0.5743f,0.4023f,  -0.7120f,-0.5777f,0.3991f,  -0.7738f,-0.5661f,0.2844f,  -0.7729f,-0.5690f,0.2809f,  -0.8215f,-0.5468f,0.1615f,  -0.8208f,-0.5491f,0.1577f,  -0.8555f,-0.5166f,0.0359f,  -0.8548f,-0.5180f,0.0318f,  -0.8750f,-0.4757f,-0.0903f,  -0.8743f,-0.4761f,-0.0944f,  -0.8796f,-0.4246f,-0.2145f,  -0.8790f,-0.4237f,-0.2187f,  -0.8693f,-0.3638f,-0.3346f,  -0.8687f,-0.3616f,-0.3385f,  -0.8442f,-0.2943f,-0.4480f,  
	-0.8435f,-0.2906f,-0.4517f,  -0.8047f,-0.2171f,-0.5526f,  -0.8039f,-0.2118f,-0.5558f,  -0.7513f,-0.1334f,-0.6463f,  -0.7505f,-0.1264f,-0.6487f,  -0.6850f,-0.0447f,-0.7271f,  -0.6841f,-0.0360f,-0.7285f,  -0.6070f,0.0475f,-0.7933f,  -0.6059f,0.0579f,-0.7934f,  -0.5184f,0.1414f,-0.8433f,  -0.5172f,0.1534f,-0.8420f,  -0.4209f,0.2351f,-0.8761f,  -0.4197f,0.2485f,-0.8730f,  -0.3162f,0.3266f,-0.8907f,  -0.3148f,0.3412f,-0.8857f,  
	-0.2060f,0.4139f,-0.8867f,  -0.2046f,0.4294f,-0.8796f,  -0.0924f,0.4948f,-0.8641f,  -0.0909f,0.5109f,-0.8548f,  0.0228f,0.5674f,-0.8231f,  0.0242f,0.5836f,-0.8117f,  0.1374f,0.6296f,-0.7647f,  0.1387f,0.6455f,-0.7511f,  0.2494f,0.6794f,-0.6901f,  0.2506f,0.6946f,-0.6744f,  0.3569f,0.7153f,-0.6009f,  0.3579f,0.7290f,-0.5835f,  0.4579f,0.7355f,-0.4993f,  0.4587f,0.7475f,-0.4804f,  0.5507f,0.7392f,-0.3877f,  
	0.5513f,0.7488f,-0.3680f,  0.6338f,0.7252f,-0.2691f,  0.6342f,0.7320f,-0.2490f,  0.7057f,0.6933f,-0.1464f,  0.7060f,0.6968f,-0.1266f,  0.7653f,0.6433f,-0.0230f,  0.7656f,0.6433f,-0.0043f,  0.8117f,0.5758f,0.0978f,  0.8121f,0.5721f,0.1147f,  0.8444f,0.4917f,0.2127f,  0.8450f,0.4844f,0.2267f,  0.8629f,0.3926f,0.3182f,  0.8639f,0.3816f,0.3288f,  0.8672f,0.2803f,0.4115f,  0.8688f,0.2658f,0.4178f,  
	0.8575f,0.1573f,0.4898f,  0.8598f,0.1398f,0.4911f,  0.8342f,0.0263f,0.5509f,  0.8375f,0.0064f,0.5465f,  0.7978f,-0.1093f,0.5929f,  0.8022f,-0.1312f,0.5825f,  0.7492f,-0.2463f,0.6148f,  0.7548f,-0.2693f,0.5982f,  0.6893f,-0.3810f,0.6162f,  0.6960f,-0.4044f,0.5933f,  0.6189f,-0.5099f,0.5974f,  0.6269f,-0.5329f,0.5684f,  0.5393f,-0.6295f,0.5593f,  0.5483f,-0.6512f,0.5247f,  0.4516f,-0.7365f,0.5036f,  
	0.4614f,-0.7561f,0.4641f,  0.3567f,-0.8278f,0.4329f,  0.3672f,-0.8447f,0.3894f,  0.2560f,-0.9010f,0.3501f,  0.2667f,-0.9146f,0.3038f,  0.1505f,-0.9541f,0.2588f,  0.1611f,-0.9641f,0.2110f,  0.0415f,-0.9858f,0.1629f,  0.0515f,-0.9920f,0.1150f,  -0.0698f,-0.9953f,0.0665f,  -0.0608f,-0.9979f,0.0201f,  -0.1821f,-0.9829f,-0.0261f,  -0.1743f,-0.9822f,-0.0696f,  -0.2939f,-0.9494f,-0.1109f,  -0.2877f,-0.9459f,-0.1501f,  
	-0.4037f,-0.8962f,-0.1841f,  -0.3991f,-0.8907f,-0.2176f,  -0.5096f,-0.8255f,-0.2424f,  -0.5068f,-0.8189f,-0.2693f,  -0.6101f,-0.7400f,-0.2832f,  -0.6089f,-0.7333f,-0.3026f,  -0.7030f,-0.6426f,-0.3046f,  -0.7032f,-0.6368f,-0.3161f,  -0.7120f,-0.5777f,0.3991f,  -0.7110f,-0.5814f,0.3955f,  -0.7729f,-0.5690f,0.2809f,  -0.7720f,-0.5721f,0.2769f,  -0.8208f,-0.5491f,0.1577f,  -0.8200f,-0.5514f,0.1534f,  -0.8548f,-0.5180f,0.0318f,  
	-0.8541f,-0.5194f,0.0273f,  -0.8743f,-0.4761f,-0.0944f,  -0.8736f,-0.4764f,-0.0991f,  -0.8790f,-0.4237f,-0.2187f,  -0.8783f,-0.4227f,-0.2233f,  -0.8687f,-0.3616f,-0.3385f,  -0.8680f,-0.3591f,-0.3430f,  -0.8435f,-0.2906f,-0.4517f,  -0.8428f,-0.2865f,-0.4557f,  -0.8039f,-0.2118f,-0.5558f,  -0.8031f,-0.2060f,-0.5592f,  -0.7505f,-0.1264f,-0.6487f,  -0.7495f,-0.1188f,-0.6512f,  -0.6841f,-0.0360f,-0.7285f,  -0.6830f,-0.0266f,-0.7299f,  
	-0.6059f,0.0579f,-0.7934f,  -0.6047f,0.0690f,-0.7935f,  -0.5172f,0.1534f,-0.8420f,  -0.5159f,0.1661f,-0.8404f,  -0.4197f,0.2485f,-0.8730f,  -0.4182f,0.2627f,-0.8695f,  -0.3148f,0.3412f,-0.8857f,  -0.3133f,0.3566f,-0.8801f,  -0.2046f,0.4294f,-0.8796f,  -0.2031f,0.4457f,-0.8718f,  -0.0909f,0.5109f,-0.8548f,  -0.0894f,0.5277f,-0.8447f,  0.0242f,0.5836f,-0.8117f,  0.0257f,0.6005f,-0.7992f,  0.1387f,0.6455f,-0.7511f,  
	0.1401f,0.6620f,-0.7363f,  0.2506f,0.6946f,-0.6744f,  0.2517f,0.7100f,-0.6577f,  0.3579f,0.7290f,-0.5835f,  0.3589f,0.7430f,-0.5649f,  0.4587f,0.7475f,-0.4804f,  0.4595f,0.7595f,-0.4605f,  0.5513f,0.7488f,-0.3680f,  0.5519f,0.7582f,-0.3473f,  0.6342f,0.7320f,-0.2490f,  0.6346f,0.7384f,-0.2281f,  0.7060f,0.6968f,-0.1266f,  0.7063f,0.6999f,-0.1063f,  0.7656f,0.6433f,-0.0043f,  0.7659f,0.6428f,0.0148f,  
	0.8121f,0.5721f,0.1147f,  0.8126f,0.5679f,0.1314f,  0.8450f,0.4844f,0.2267f,  0.8458f,0.4763f,0.2404f,  0.8639f,0.3816f,0.3288f,  0.8652f,0.3699f,0.3386f,  0.8688f,0.2658f,0.4178f,  0.8708f,0.2508f,0.4229f,  0.8598f,0.1398f,0.4911f,  0.8628f,0.1219f,0.4907f,  0.8375f,0.0064f,0.5465f,  0.8414f,-0.0139f,0.5402f,  0.8022f,-0.1312f,0.5825f,  0.8074f,-0.1530f,0.5699f,  0.7548f,-0.2693f,0.5982f,  
	0.7612f,-0.2920f,0.5791f,  0.6960f,-0.4044f,0.5933f,  0.7037f,-0.4272f,0.5678f,  0.6269f,-0.5329f,0.5684f,  0.6357f,-0.5548f,0.5367f,  0.5483f,-0.6512f,0.5247f,  0.5582f,-0.6715f,0.4874f,  0.4614f,-0.7561f,0.4641f,  0.4721f,-0.7740f,0.4220f,  0.3672f,-0.8447f,0.3894f,  0.3783f,-0.8596f,0.3436f,  0.2667f,-0.9146f,0.3038f,  0.2779f,-0.9260f,0.2554f,  0.1611f,-0.9641f,0.2110f,  0.1719f,-0.9718f,0.1615f,  
	0.0515f,-0.9920f,0.1150f,  0.0616f,-0.9959f,0.0658f,  -0.0608f,-0.9979f,0.0201f,  -0.0518f,-0.9983f,-0.0272f,  -0.1743f,-0.9822f,-0.0696f,  -0.1668f,-0.9794f,-0.1135f,  -0.2877f,-0.9459f,-0.1501f,  -0.2817f,-0.9407f,-0.1891f,  -0.3991f,-0.8907f,-0.2176f,  -0.3949f,-0.8839f,-0.2507f,  -0.5068f,-0.8189f,-0.2693f,  -0.5043f,-0.8114f,-0.2955f,  -0.6089f,-0.7333f,-0.3026f,  -0.6079f,-0.7262f,-0.3212f,  -0.7032f,-0.6368f,-0.3161f,  
	-0.7035f,-0.6311f,-0.3268f,  -0.7110f,-0.5814f,0.3955f,  -0.7101f,-0.5854f,0.3913f,  -0.7720f,-0.5721f,0.2769f,  -0.7712f,-0.5754f,0.2724f,  -0.8200f,-0.5514f,0.1534f,  -0.8192f,-0.5539f,0.1486f,  -0.8541f,-0.5194f,0.0273f,  -0.8533f,-0.5209f,0.0222f,  -0.8736f,-0.4764f,-0.0991f,  -0.8729f,-0.4766f,-0.1042f,  -0.8783f,-0.4227f,-0.2233f,  -0.8776f,-0.4215f,-0.2284f,  -0.8680f,-0.3591f,-0.3430f,  -0.8672f,-0.3564f,-0.3478f,  
	-0.8428f,-0.2865f,-0.4557f,  -0.8419f,-0.2820f,-0.4601f,  -0.8031f,-0.2060f,-0.5592f,  -0.8021f,-0.1997f,-0.5628f,  -0.7495f,-0.1188f,-0.6512f,  -0.7484f,-0.1106f,-0.6539f,  -0.6830f,-0.0266f,-0.7299f,  -0.6818f,-0.0165f,-0.7314f,  -0.6047f,0.0690f,-0.7935f,  -0.6034f,0.0809f,-0.7934f,  -0.5159f,0.1661f,-0.8404f,  -0.5145f,0.1797f,-0.8385f,  -0.4182f,0.2627f,-0.8695f,  -0.4167f,0.2777f,-0.8656f,  -0.3133f,0.3566f,-0.8801f,  
	-0.3117f,0.3729f,-0.8740f,  -0.2031f,0.4457f,-0.8718f,  -0.2015f,0.4628f,-0.8633f,  -0.0894f,0.5277f,-0.8447f,  -0.0879f,0.5453f,-0.8336f,  0.0257f,0.6005f,-0.7992f,  0.0271f,0.6180f,-0.7857f,  0.1401f,0.6620f,-0.7363f,  0.1413f,0.6788f,-0.7206f,  0.2517f,0.7100f,-0.6577f,  0.2528f,0.7258f,-0.6397f,  0.3589f,0.7430f,-0.5649f,  0.3597f,0.7572f,-0.5453f,  0.4595f,0.7595f,-0.4605f,  0.4601f,0.7714f,-0.4397f,  
	0.5519f,0.7582f,-0.3473f,  0.5523f,0.7674f,-0.3257f,  0.6346f,0.7384f,-0.2281f,  0.6349f,0.7445f,-0.2065f,  0.7063f,0.6999f,-0.1063f,  0.7066f,0.7025f,-0.0854f,  0.7659f,0.6428f,0.0148f,  0.7662f,0.6417f,0.0338f,  0.8126f,0.5679f,0.1314f,  0.8131f,0.5629f,0.1480f,  0.8458f,0.4763f,0.2404f,  0.8468f,0.4676f,0.2536f,  0.8652f,0.3699f,0.3386f,  0.8669f,0.3576f,0.3474f,  0.8708f,0.2508f,0.4229f,  
	0.8733f,0.2354f,0.4265f,  0.8628f,0.1219f,0.4907f,  0.8663f,0.1037f,0.4887f,  0.8414f,-0.0139f,0.5402f,  0.8461f,-0.0341f,0.5319f,  0.8074f,-0.1530f,0.5699f,  0.8133f,-0.1746f,0.5550f,  0.7612f,-0.2920f,0.5791f,  0.7685f,-0.3141f,0.5575f,  0.7037f,-0.4272f,0.5678f,  0.7123f,-0.4489f,0.5396f,  0.6357f,-0.5548f,0.5367f,  0.6455f,-0.5753f,0.5023f,  0.5582f,-0.6715f,0.4874f,  0.5689f,-0.6900f,0.4475f,  
	0.4721f,-0.7740f,0.4220f,  0.4835f,-0.7898f,0.3775f,  0.3783f,-0.8596f,0.3436f,  0.3900f,-0.8721f,0.2955f,  0.2779f,-0.9260f,0.2554f,  0.2894f,-0.9350f,0.2052f,  0.1719f,-0.9718f,0.1615f,  0.1829f,-0.9769f,0.1105f,  0.0616f,-0.9959f,0.0658f,  0.0716f,-0.9973f,0.0157f,  -0.0518f,-0.9983f,-0.0272f,  -0.0430f,-0.9963f,-0.0750f,  -0.1668f,-0.9794f,-0.1135f,  -0.1595f,-0.9746f,-0.1574f,  -0.2817f,-0.9407f,-0.1891f,  
	-0.2762f,-0.9337f,-0.2279f,  -0.3949f,-0.8839f,-0.2507f,  -0.3911f,-0.8757f,-0.2833f,  -0.5043f,-0.8114f,-0.2955f,  -0.5021f,-0.8031f,-0.3209f,  -0.6079f,-0.7262f,-0.3212f,  -0.6071f,-0.7187f,-0.3390f,  -0.7035f,-0.6311f,-0.3268f,  -0.7038f,-0.6255f,-0.3368f,  -0.7101f,-0.5854f,0.3913f,  -0.7092f,-0.5896f,0.3866f,  -0.7712f,-0.5754f,0.2724f,  -0.7704f,-0.5789f,0.2673f,  -0.8192f,-0.5539f,0.1486f,  -0.8184f,-0.5565f,0.1432f,  
	-0.8533f,-0.5209f,0.0222f,  -0.8526f,-0.5223f,0.0166f,  -0.8729f,-0.4766f,-0.1042f,  -0.8722f,-0.4767f,-0.1099f,  -0.8776f,-0.4215f,-0.2284f,  -0.8768f,-0.4201f,-0.2340f,  -0.8672f,-0.3564f,-0.3478f,  -0.8663f,-0.3532f,-0.3531f,  -0.8419f,-0.2820f,-0.4601f,  -0.8409f,-0.2771f,-0.4648f,  -0.8021f,-0.1997f,-0.5628f,  -0.8010f,-0.1928f,-0.5668f,  -0.7484f,-0.1106f,-0.6539f,  -0.7472f,-0.1018f,-0.6568f,  -0.6818f,-0.0165f,-0.7314f,  
	-0.6805f,-0.0057f,-0.7328f,  -0.6034f,0.0809f,-0.7934f,  -0.6019f,0.0936f,-0.7931f,  -0.5145f,0.1797f,-0.8385f,  -0.5129f,0.1941f,-0.8362f,  -0.4167f,0.2777f,-0.8656f,  -0.4151f,0.2936f,-0.8611f,  -0.3117f,0.3729f,-0.8740f,  -0.3101f,0.3899f,-0.8671f,  -0.2015f,0.4628f,-0.8633f,  -0.1999f,0.4807f,-0.8538f,  -0.0879f,0.5453f,-0.8336f,  -0.0863f,0.5635f,-0.8216f,  0.0271f,0.6180f,-0.7857f,  0.0284f,0.6360f,-0.7712f,  
	0.1413f,0.6788f,-0.7206f,  0.1425f,0.6962f,-0.7036f,  0.2528f,0.7258f,-0.6397f,  0.2538f,0.7419f,-0.6206f,  0.3597f,0.7572f,-0.5453f,  0.3604f,0.7713f,-0.5245f,  0.4601f,0.7714f,-0.4397f,  0.4606f,0.7832f,-0.4177f,  0.5523f,0.7674f,-0.3257f,  0.5527f,0.7763f,-0.3032f,  0.6349f,0.7445f,-0.2065f,  0.6351f,0.7501f,-0.1842f,  0.7066f,0.7025f,-0.0854f,  0.7068f,0.7045f,-0.0644f,  0.7662f,0.6417f,0.0338f,  
	0.7666f,0.6399f,0.0530f,  0.8131f,0.5629f,0.1480f,  0.8139f,0.5573f,0.1643f,  0.8468f,0.4676f,0.2536f,  0.8481f,0.4583f,0.2660f,  0.8669f,0.3576f,0.3474f,  0.8689f,0.3448f,0.3551f,  0.8733f,0.2354f,0.4265f,  0.8763f,0.2195f,0.4288f,  0.8663f,0.1037f,0.4887f,  0.8705f,0.0855f,0.4848f,  0.8461f,-0.0341f,0.5319f,  0.8516f,-0.0542f,0.5214f,  0.8133f,-0.1746f,0.5550f,  0.8201f,-0.1957f,0.5377f,  
	0.7685f,-0.3141f,0.5575f,  0.7767f,-0.3353f,0.5332f,  0.7123f,-0.4489f,0.5396f,  0.7217f,-0.4694f,0.5087f,  0.6455f,-0.5753f,0.5023f,  0.6561f,-0.5942f,0.4652f,  0.5689f,-0.6900f,0.4475f,  0.5804f,-0.7065f,0.4050f,  0.4835f,-0.7898f,0.3775f,  0.4954f,-0.8033f,0.3306f,  0.3900f,-0.8721f,0.2955f,  0.4019f,-0.8822f,0.2454f,  0.2894f,-0.9350f,0.2052f,  0.3011f,-0.9412f,0.1533f,  0.1829f,-0.9769f,0.1105f,  
	0.1938f,-0.9793f,0.0583f,  0.0716f,-0.9973f,0.0157f,  0.0814f,-0.9961f,-0.0352f,  -0.0430f,-0.9963f,-0.0750f,  -0.0346f,-0.9918f,-0.1230f,  -0.1595f,-0.9746f,-0.1574f,  -0.1528f,-0.9676f,-0.2012f,  -0.2762f,-0.9337f,-0.2279f,  -0.2712f,-0.9250f,-0.2662f,  -0.3911f,-0.8757f,-0.2833f,  -0.3878f,-0.8662f,-0.3150f,  -0.5021f,-0.8031f,-0.3209f,  -0.5004f,-0.7939f,-0.3454f,  -0.6071f,-0.7187f,-0.3390f,  -0.6066f,-0.7108f,-0.3559f,  
	-0.7038f,-0.6255f,-0.3368f,  -0.7043f,-0.6199f,-0.3460f,  -0.7092f,-0.5896f,0.3866f,  -0.7083f,-0.5940f,0.3814f,  -0.7704f,-0.5789f,0.2673f,  -0.7695f,-0.5825f,0.2616f,  -0.8184f,-0.5565f,0.1432f,  -0.8177f,-0.5591f,0.1372f,  -0.8526f,-0.5223f,0.0166f,  -0.8518f,-0.5237f,0.0104f,  -0.8722f,-0.4767f,-0.1099f,  -0.8714f,-0.4767f,-0.1162f,  -0.8768f,-0.4201f,-0.2340f,  -0.8760f,-0.4184f,-0.2401f,  -0.8663f,-0.3532f,-0.3531f,  
	-0.8654f,-0.3497f,-0.3588f,  -0.8409f,-0.2771f,-0.4648f,  -0.8399f,-0.2716f,-0.4699f,  -0.8010f,-0.1928f,-0.5668f,  -0.7998f,-0.1852f,-0.5709f,  -0.7472f,-0.1018f,-0.6568f,  -0.7459f,-0.0922f,-0.6596f,  -0.6805f,-0.0057f,-0.7328f,  -0.6790f,0.0059f,-0.7341f,  -0.6019f,0.0936f,-0.7931f,  -0.6004f,0.1071f,-0.7925f,  -0.5129f,0.1941f,-0.8362f,  -0.5113f,0.2094f,-0.8335f,  -0.4151f,0.2936f,-0.8611f,  -0.4134f,0.3104f,-0.8560f,  
	-0.3101f,0.3899f,-0.8671f,  -0.3084f,0.4078f,-0.8594f,  -0.1999f,0.4807f,-0.8538f,  -0.1983f,0.4993f,-0.8434f,  -0.0863f,0.5635f,-0.8216f,  -0.0849f,0.5823f,-0.8085f,  0.0284f,0.6360f,-0.7712f,  0.0297f,0.6546f,-0.7554f,  0.1425f,0.6962f,-0.7036f,  0.1436f,0.7139f,-0.6854f,  0.2538f,0.7419f,-0.6206f,  0.2546f,0.7581f,-0.6004f,  0.3604f,0.7713f,-0.5245f,  0.3610f,0.7856f,-0.5025f,  0.4606f,0.7832f,-0.4177f,  
	0.4609f,0.7948f,-0.3947f,  0.5527f,0.7763f,-0.3032f,  0.5528f,0.7849f,-0.2798f,  0.6351f,0.7501f,-0.1842f,  0.6352f,0.7553f,-0.1615f,  0.7068f,0.7045f,-0.0644f,  0.7069f,0.7060f,-0.0429f,  0.7666f,0.6399f,0.0530f,  0.7670f,0.6375f,0.0721f,  0.8139f,0.5573f,0.1643f,  0.8148f,0.5511f,0.1801f,  0.8481f,0.4583f,0.2660f,  0.8496f,0.4484f,0.2776f,  0.8689f,0.3448f,0.3551f,  0.8714f,0.3316f,0.3616f,  
	0.8763f,0.2195f,0.4288f,  0.8799f,0.2035f,0.4294f,  0.8705f,0.0855f,0.4848f,  0.8753f,0.0672f,0.4789f,  0.8516f,-0.0542f,0.5214f,  0.8578f,-0.0739f,0.5087f,  0.8201f,-0.1957f,0.5377f,  0.8278f,-0.2160f,0.5178f,  0.7767f,-0.3353f,0.5332f,  0.7857f,-0.3554f,0.5064f,  0.7217f,-0.4694f,0.5087f,  0.7320f,-0.4883f,0.4751f,  0.6561f,-0.5942f,0.4652f,  0.6673f,-0.6112f,0.4255f,  0.5804f,-0.7065f,0.4050f,  
	0.5923f,-0.7208f,0.3600f,  0.4954f,-0.8033f,0.3306f,  0.5076f,-0.8143f,0.2814f,  0.4019f,-0.8822f,0.2454f,  0.4141f,-0.8894f,0.1934f,  0.3011f,-0.9412f,0.1533f,  0.3127f,-0.9446f,0.1000f,  0.1938f,-0.9793f,0.0583f,  0.2045f,-0.9789f,0.0052f,  0.0814f,-0.9961f,-0.0352f,  0.0908f,-0.9921f,-0.0865f,  -0.0346f,-0.9918f,-0.1230f,  -0.0269f,-0.9849f,-0.1711f,  -0.1528f,-0.9676f,-0.2012f,  -0.1468f,-0.9585f,-0.2446f,  
	-0.2712f,-0.9250f,-0.2662f,  -0.2669f,-0.9146f,-0.3038f,  -0.3878f,-0.8662f,-0.3150f,  -0.3851f,-0.8556f,-0.3459f,  -0.5004f,-0.7939f,-0.3454f,  -0.4991f,-0.7841f,-0.3690f,  -0.6066f,-0.7108f,-0.3559f,  -0.6065f,-0.7028f,-0.3719f,  -0.7043f,-0.6199f,-0.3460f,  -0.7048f,-0.6145f,-0.3545f,  -0.7083f,-0.5940f,0.3814f,  -0.7075f,-0.5986f,0.3756f,  -0.7695f,-0.5825f,0.2616f,  -0.7688f,-0.5863f,0.2554f,  -0.8177f,-0.5591f,0.1372f,  
	-0.8169f,-0.5618f,0.1306f,  -0.8518f,-0.5237f,0.0104f,  -0.8511f,-0.5250f,0.0036f,  -0.8714f,-0.4767f,-0.1162f,  -0.8706f,-0.4764f,-0.1230f,  -0.8760f,-0.4184f,-0.2401f,  -0.8751f,-0.4164f,-0.2467f,  -0.8654f,-0.3497f,-0.3588f,  -0.8644f,-0.3458f,-0.3650f,  -0.8399f,-0.2716f,-0.4699f,  -0.8388f,-0.2656f,-0.4753f,  -0.7998f,-0.1852f,-0.5709f,  -0.7986f,-0.1771f,-0.5752f,  -0.7459f,-0.0922f,-0.6596f,  -0.7445f,-0.0819f,-0.6625f,  
	-0.6790f,0.0059f,-0.7341f,  -0.6775f,0.0183f,-0.7353f,  -0.6004f,0.1071f,-0.7925f,  -0.5987f,0.1215f,-0.7917f,  -0.5113f,0.2094f,-0.8335f,  -0.5096f,0.2255f,-0.8303f,  -0.4134f,0.3104f,-0.8560f,  -0.4117f,0.3280f,-0.8502f,  -0.3084f,0.4078f,-0.8594f,  -0.3068f,0.4266f,-0.8508f,  -0.1983f,0.4993f,-0.8434f,  -0.1968f,0.5186f,-0.8320f,  -0.0849f,0.5823f,-0.8085f,  -0.0835f,0.6018f,-0.7942f,  0.0297f,0.6546f,-0.7554f,  
	0.0309f,0.6737f,-0.7384f,  0.1436f,0.7139f,-0.6854f,  0.1444f,0.7319f,-0.6659f,  0.2546f,0.7581f,-0.6004f,  0.2552f,0.7745f,-0.5788f,  0.3610f,0.7856f,-0.5025f,  0.3613f,0.7997f,-0.4795f,  0.4609f,0.7948f,-0.3947f,  0.4610f,0.8063f,-0.3706f,  0.5528f,0.7849f,-0.2798f,  0.5527f,0.7931f,-0.2558f,  0.6352f,0.7553f,-0.1615f,  0.6351f,0.7600f,-0.1381f,  0.7069f,0.7060f,-0.0429f,  0.7071f,0.7068f,-0.0212f,  
	0.7670f,0.6375f,0.0721f,  0.7675f,0.6345f,0.0911f,  0.8148f,0.5511f,0.1801f,  0.8158f,0.5443f,0.1953f,  0.8496f,0.4484f,0.2776f,  0.8515f,0.4381f,0.2882f,  0.8714f,0.3316f,0.3616f,  0.8743f,0.3182f,0.3666f,  0.8799f,0.2035f,0.4294f,  0.8840f,0.1874f,0.4283f,  0.8753f,0.0672f,0.4789f,  0.8808f,0.0492f,0.4710f,  0.8578f,-0.0739f,0.5087f,  0.8648f,-0.0930f,0.4935f,  0.8278f,-0.2160f,0.5178f,  
	0.8362f,-0.2353f,0.4954f,  0.7857f,-0.3554f,0.5064f,  0.7954f,-0.3741f,0.4768f,  0.7320f,-0.4883f,0.4751f,  0.7430f,-0.5054f,0.4388f,  0.6673f,-0.6112f,0.4255f,  0.6792f,-0.6260f,0.3832f,  0.5923f,-0.7208f,0.3600f,  0.6047f,-0.7325f,0.3127f,  0.5076f,-0.8143f,0.2814f,  0.5201f,-0.8225f,0.2303f,  0.4141f,-0.8894f,0.1934f,  0.4263f,-0.8937f,0.1399f,  0.3127f,-0.9446f,0.1000f,  0.3241f,-0.9449f,0.0455f,  
	0.2045f,-0.9789f,0.0052f,  0.2147f,-0.9755f,-0.0485f,  0.0908f,-0.9921f,-0.0865f,  0.0996f,-0.9854f,-0.1380f,  -0.0269f,-0.9849f,-0.1711f,  -0.0198f,-0.9756f,-0.2188f,  -0.1468f,-0.9585f,-0.2446f,  -0.1414f,-0.9473f,-0.2873f,  -0.2669f,-0.9146f,-0.3038f,  -0.2633f,-0.9027f,-0.3404f,  -0.3851f,-0.8556f,-0.3459f,  -0.3831f,-0.8439f,-0.3757f,  -0.4991f,-0.7841f,-0.3690f,  -0.4983f,-0.7736f,-0.3915f,  -0.6065f,-0.7028f,-0.3719f,  
	-0.6066f,-0.6945f,-0.3870f,  -0.7048f,-0.6145f,-0.3545f,  -0.7054f,-0.6092f,-0.3623f,  -0.7075f,-0.5986f,0.3756f,  -0.7067f,-0.6035f,0.3692f,  -0.7688f,-0.5863f,0.2554f,  -0.7680f,-0.5902f,0.2485f,  -0.8169f,-0.5618f,0.1306f,  -0.8162f,-0.5645f,0.1234f,  -0.8511f,-0.5250f,0.0036f,  -0.8503f,-0.5263f,-0.0038f,  -0.8706f,-0.4764f,-0.1230f,  -0.8698f,-0.4760f,-0.1304f,  -0.8751f,-0.4164f,-0.2467f,  -0.8741f,-0.4141f,-0.2538f,  
	-0.8644f,-0.3458f,-0.3650f,  -0.8634f,-0.3414f,-0.3716f,  -0.8388f,-0.2656f,-0.4753f,  -0.8376f,-0.2590f,-0.4810f,  -0.7986f,-0.1771f,-0.5752f,  -0.7973f,-0.1683f,-0.5797f,  -0.7445f,-0.0819f,-0.6625f,  -0.7431f,-0.0708f,-0.6654f,  -0.6775f,0.0183f,-0.7353f,  -0.6760f,0.0316f,-0.7363f,  -0.5987f,0.1215f,-0.7917f,  -0.5971f,0.1368f,-0.7904f,  -0.5096f,0.2255f,-0.8303f,  -0.5079f,0.2426f,-0.8265f,  -0.4117f,0.3280f,-0.8502f,  
	-0.4101f,0.3466f,-0.8436f,  -0.3068f,0.4266f,-0.8508f,  -0.3053f,0.4461f,-0.8413f,  -0.1968f,0.5186f,-0.8320f,  -0.1954f,0.5388f,-0.8195f,  -0.0835f,0.6018f,-0.7942f,  -0.0824f,0.6219f,-0.7787f,  0.0309f,0.6737f,-0.7384f,  0.0317f,0.6931f,-0.7201f,  0.1444f,0.7319f,-0.6659f,  0.1450f,0.7502f,-0.6451f,  0.2552f,0.7745f,-0.5788f,  0.2555f,0.7909f,-0.5560f,  0.3613f,0.7997f,-0.4795f,  0.3614f,0.8138f,-0.4551f,  
	0.4610f,0.8063f,-0.3706f,  0.4608f,0.8174f,-0.3457f,  0.5527f,0.7931f,-0.2558f,  0.5525f,0.8009f,-0.2309f,  0.6351f,0.7600f,-0.1381f,  0.6350f,0.7641f,-0.1143f,  0.7071f,0.7068f,-0.0212f,  0.7071f,0.7071f,0.0005f,  0.7675f,0.6345f,0.0911f,  0.7680f,0.6309f,0.1097f,  0.8158f,0.5443f,0.1953f,  0.8171f,0.5370f,0.2099f,  0.8515f,0.4381f,0.2882f,  0.8537f,0.4273f,0.2976f,  0.8743f,0.3182f,0.3666f,  
	0.8776f,0.3044f,0.3703f,  0.8840f,0.1874f,0.4283f,  0.8887f,0.1714f,0.4252f,  0.8808f,0.0492f,0.4710f,  0.8869f,0.0317f,0.4608f,  0.8648f,-0.0930f,0.4935f,  0.8724f,-0.1113f,0.4760f,  0.8362f,-0.2353f,0.4954f,  0.8453f,-0.2534f,0.4704f,  0.7954f,-0.3741f,0.4768f,  0.8059f,-0.3910f,0.4446f,  0.7430f,-0.5054f,0.4388f,  0.7544f,-0.5204f,0.3999f,  0.6792f,-0.6260f,0.3832f,  0.6914f,-0.6383f,0.3385f,  
	0.6047f,-0.7325f,0.3127f,  0.6172f,-0.7415f,0.2632f,  0.5201f,-0.8225f,0.2303f,  0.5325f,-0.8276f,0.1774f,  0.4263f,-0.8937f,0.1399f,  0.4381f,-0.8949f,0.0850f,  0.3241f,-0.9449f,0.0455f,  0.3350f,-0.9422f,-0.0097f,  0.2147f,-0.9755f,-0.0485f,  0.2243f,-0.9691f,-0.1026f,  0.0996f,-0.9854f,-0.1380f,  0.1075f,-0.9760f,-0.1893f,  -0.0198f,-0.9756f,-0.2188f,  -0.0136f,-0.9639f,-0.2660f,  -0.1414f,-0.9473f,-0.2873f,  
	-0.1370f,-0.9343f,-0.3291f,  -0.2633f,-0.9027f,-0.3404f,  -0.2605f,-0.8893f,-0.3760f,  -0.3831f,-0.8439f,-0.3757f,  -0.3817f,-0.8312f,-0.4043f,  -0.4983f,-0.7736f,-0.3915f,  -0.4980f,-0.7625f,-0.4129f,  -0.6066f,-0.6945f,-0.3870f,  -0.6070f,-0.6860f,-0.4012f,  -0.7054f,-0.6092f,-0.3623f,  -0.7061f,-0.6041f,-0.3694f,  -0.7067f,-0.6035f,0.3692f,  -0.7060f,-0.6086f,0.3622f,  -0.7680f,-0.5902f,0.2485f,  -0.7674f,-0.5942f,0.2410f,  
	-0.8162f,-0.5645f,0.1234f,  -0.8155f,-0.5671f,0.1155f,  -0.8503f,-0.5263f,-0.0038f,  -0.8496f,-0.5274f,-0.0118f,  -0.8698f,-0.4760f,-0.1304f,  -0.8689f,-0.4752f,-0.1383f,  -0.8741f,-0.4141f,-0.2538f,  -0.8732f,-0.4113f,-0.2615f,  -0.8634f,-0.3414f,-0.3716f,  -0.8623f,-0.3364f,-0.3785f,  -0.8376f,-0.2590f,-0.4810f,  -0.8364f,-0.2518f,-0.4869f,  -0.7973f,-0.1683f,-0.5797f,  -0.7959f,-0.1587f,-0.5842f,  -0.7431f,-0.0708f,-0.6654f,  
	-0.7416f,-0.0588f,-0.6683f,  -0.6760f,0.0316f,-0.7363f,  -0.6744f,0.0458f,-0.7370f,  -0.5971f,0.1368f,-0.7904f,  -0.5955f,0.1530f,-0.7887f,  -0.5079f,0.2426f,-0.8265f,  -0.5063f,0.2606f,-0.8220f,  -0.4101f,0.3466f,-0.8436f,  -0.4086f,0.3659f,-0.8362f,  -0.3053f,0.4461f,-0.8413f,  -0.3039f,0.4664f,-0.8307f,  -0.1954f,0.5388f,-0.8195f,  -0.1942f,0.5595f,-0.8057f,  -0.0824f,0.6219f,-0.7787f,  -0.0815f,0.6425f,-0.7620f,  
	0.0317f,0.6931f,-0.7201f,  0.0324f,0.7130f,-0.7004f,  0.1450f,0.7502f,-0.6451f,  0.1453f,0.7686f,-0.6230f,  0.2555f,0.7909f,-0.5560f,  0.2555f,0.8074f,-0.5318f,  0.3614f,0.8138f,-0.4551f,  0.3610f,0.8276f,-0.4298f,  0.4608f,0.8174f,-0.3457f,  0.4603f,0.8282f,-0.3198f,  0.5525f,0.8009f,-0.2309f,  0.5520f,0.8082f,-0.2053f,  0.6350f,0.7641f,-0.1143f,  0.6346f,0.7676f,-0.0901f,  0.7071f,0.7071f,0.0005f,  
	0.7071f,0.7068f,0.0222f,  0.7680f,0.6309f,0.1097f,  0.7686f,0.6268f,0.1280f,  0.8171f,0.5370f,0.2099f,  0.8185f,0.5292f,0.2236f,  0.8537f,0.4273f,0.2976f,  0.8562f,0.4163f,0.3060f,  0.8776f,0.3044f,0.3703f,  0.8814f,0.2907f,0.3722f,  0.8887f,0.1714f,0.4252f,  0.8940f,0.1558f,0.4202f,  0.8869f,0.0317f,0.4608f,  0.8937f,0.0148f,0.4484f,  0.8724f,-0.1113f,0.4760f,  0.8807f,-0.1284f,0.4559f,  
	0.8453f,-0.2534f,0.4704f,  0.8550f,-0.2699f,0.4428f,  0.8059f,-0.3910f,0.4446f,  0.8168f,-0.4060f,0.4098f,  0.7544f,-0.5204f,0.3999f,  0.7663f,-0.5331f,0.3586f,  0.6914f,-0.6383f,0.3385f,  0.7038f,-0.6478f,0.2916f,  0.6172f,-0.7415f,0.2632f,  0.6297f,-0.7474f,0.2119f,  0.5325f,-0.8276f,0.1774f,  0.5447f,-0.8296f,0.1231f,  0.4381f,-0.8949f,0.0850f,  0.4495f,-0.8928f,0.0292f,  0.3350f,-0.9422f,-0.0097f,  
	0.3451f,-0.9363f,-0.0654f,  0.2243f,-0.9691f,-0.1026f,  0.2329f,-0.9598f,-0.1566f,  0.1075f,-0.9760f,-0.1893f,  0.1145f,-0.9640f,-0.2401f,  -0.0136f,-0.9639f,-0.2660f,  -0.0084f,-0.9500f,-0.3123f,  -0.1370f,-0.9343f,-0.3291f,  -0.1335f,-0.9195f,-0.3698f,  -0.2605f,-0.8893f,-0.3760f,  -0.2585f,-0.8746f,-0.4103f,  -0.3817f,-0.8312f,-0.4043f,  -0.3810f,-0.8176f,-0.4317f,  -0.4980f,-0.7625f,-0.4129f,  -0.4982f,-0.7511f,-0.4332f,  
	-0.6070f,-0.6860f,-0.4012f,  -0.6077f,-0.6775f,-0.4144f,  -0.7061f,-0.6041f,-0.3694f,  -0.7069f,-0.5992f,-0.3758f,  -0.7060f,-0.6086f,0.3622f,  -0.7054f,-0.6138f,0.3545f,  -0.7674f,-0.5942f,0.2410f,  -0.7668f,-0.5983f,0.2327f,  -0.8155f,-0.5671f,0.1155f,  -0.8148f,-0.5698f,0.1069f,  -0.8496f,-0.5274f,-0.0118f,  -0.8488f,-0.5283f,-0.0205f,  -0.8689f,-0.4752f,-0.1383f,  -0.8681f,-0.4742f,-0.1469f,  -0.8732f,-0.4113f,-0.2615f,  
	-0.8722f,-0.4081f,-0.2695f,  -0.8623f,-0.3364f,-0.3785f,  -0.8612f,-0.3310f,-0.3858f,  -0.8364f,-0.2518f,-0.4869f,  -0.8351f,-0.2439f,-0.4930f,  -0.7959f,-0.1587f,-0.5842f,  -0.7945f,-0.1483f,-0.5888f,  -0.7416f,-0.0588f,-0.6683f,  -0.7401f,-0.0461f,-0.6709f,  -0.6744f,0.0458f,-0.7370f,  -0.6728f,0.0609f,-0.7373f,  -0.5955f,0.1530f,-0.7887f,  -0.5939f,0.1702f,-0.7863f,  -0.5063f,0.2606f,-0.8220f,  -0.5048f,0.2794f,-0.8167f,  
	-0.4086f,0.3659f,-0.8362f,  -0.4071f,0.3861f,-0.8277f,  -0.3039f,0.4664f,-0.8307f,  -0.3027f,0.4875f,-0.8190f,  -0.1942f,0.5595f,-0.8057f,  -0.1933f,0.5808f,-0.7907f,  -0.0815f,0.6425f,-0.7620f,  -0.0808f,0.6636f,-0.7437f,  0.0324f,0.7130f,-0.7004f,  0.0326f,0.7331f,-0.6794f,  0.1453f,0.7686f,-0.6230f,  0.1452f,0.7872f,-0.5994f,  0.2555f,0.8074f,-0.5318f,  0.2550f,0.8237f,-0.5065f,  0.3610f,0.8276f,-0.4298f,  
	0.3603f,0.8412f,-0.4032f,  0.4603f,0.8282f,-0.3198f,  0.4595f,0.8385f,-0.2929f,  0.5520f,0.8082f,-0.2053f,  0.5512f,0.8149f,-0.1791f,  0.6346f,0.7676f,-0.0901f,  0.6340f,0.7705f,-0.0657f,  0.7071f,0.7068f,0.0222f,  0.7070f,0.7058f,0.0439f,  0.7686f,0.6268f,0.1280f,  0.7693f,0.6221f,0.1457f,  0.8185f,0.5292f,0.2236f,  0.8201f,0.5210f,0.2364f,  0.8562f,0.4163f,0.3060f,  0.8591f,0.4051f,0.3129f,  
	0.8814f,0.2907f,0.3722f,  0.8857f,0.2772f,0.3724f,  0.8940f,0.1558f,0.4202f,  0.8997f,0.1405f,0.4132f,  0.8937f,0.0148f,0.4484f,  0.9011f,-0.0012f,0.4337f,  0.8807f,-0.1284f,0.4559f,  0.8896f,-0.1442f,0.4334f,  0.8550f,-0.2699f,0.4428f,  0.8652f,-0.2847f,0.4127f,  0.8168f,-0.4060f,0.4098f,  0.8281f,-0.4188f,0.3726f,  0.7663f,-0.5331f,0.3586f,  0.7783f,-0.5431f,0.3151f,  0.7038f,-0.6478f,0.2916f,  
	0.7161f,-0.6544f,0.2428f,  0.6297f,-0.7474f,0.2119f,  0.6419f,-0.7501f,0.1590f,  0.5447f,-0.8296f,0.1231f,  0.5563f,-0.8282f,0.0677f,  0.4495f,-0.8928f,0.0292f,  0.4601f,-0.8874f,-0.0272f,  0.3451f,-0.9363f,-0.0654f,  0.3544f,-0.9272f,-0.1212f,  0.2329f,-0.9598f,-0.1566f,  0.2406f,-0.9476f,-0.2101f,  0.1145f,-0.9640f,-0.2401f,  0.1204f,-0.9494f,-0.2900f,  -0.0084f,-0.9500f,-0.3123f,  -0.0042f,-0.9339f,-0.3574f,  
	-0.1335f,-0.9195f,-0.3698f,  -0.1309f,-0.9030f,-0.4092f,  -0.2585f,-0.8746f,-0.4103f,  -0.2574f,-0.8587f,-0.4432f,  -0.3810f,-0.8176f,-0.4317f,  -0.3809f,-0.8033f,-0.4578f,  -0.4982f,-0.7511f,-0.4332f,  -0.4989f,-0.7393f,-0.4523f,  -0.6077f,-0.6775f,-0.4144f,  -0.6086f,-0.6690f,-0.4267f,  -0.7069f,-0.5992f,-0.3758f,  -0.7077f,-0.5945f,-0.3817f,  -0.7054f,-0.6138f,0.3545f,  -0.7049f,-0.6192f,0.3461f,  -0.7668f,-0.5983f,0.2327f,  
	-0.7662f,-0.6024f,0.2239f,  -0.8148f,-0.5698f,0.1069f,  -0.8142f,-0.5723f,0.0977f,  -0.8488f,-0.5283f,-0.0205f,  -0.8481f,-0.5290f,-0.0298f,  -0.8681f,-0.4742f,-0.1469f,  -0.8672f,-0.4728f,-0.1560f,  -0.8722f,-0.4081f,-0.2695f,  -0.8713f,-0.4045f,-0.2780f,  -0.8612f,-0.3310f,-0.3858f,  -0.8601f,-0.3248f,-0.3934f,  -0.8351f,-0.2439f,-0.4930f,  -0.8339f,-0.2352f,-0.4993f,  -0.7945f,-0.1483f,-0.5888f,  -0.7932f,-0.1371f,-0.5934f,  
	-0.7401f,-0.0461f,-0.6709f,  -0.7386f,-0.0324f,-0.6733f,  -0.6728f,0.0609f,-0.7373f,  -0.6713f,0.0769f,-0.7372f,  -0.5939f,0.1702f,-0.7863f,  -0.5925f,0.1882f,-0.7833f,  -0.5048f,0.2794f,-0.8167f,  -0.5034f,0.2992f,-0.8106f,  -0.4071f,0.3861f,-0.8277f,  -0.4060f,0.4072f,-0.8182f,  -0.3027f,0.4875f,-0.8190f,  -0.3018f,0.5093f,-0.8060f,  -0.1933f,0.5808f,-0.7907f,  -0.1927f,0.6028f,-0.7743f,  -0.0808f,0.6636f,-0.7437f,  
	-0.0806f,0.6850f,-0.7241f,  0.0326f,0.7331f,-0.6794f,  0.0325f,0.7535f,-0.6567f,  0.1452f,0.7872f,-0.5994f,  0.1446f,0.8057f,-0.5745f,  0.2550f,0.8237f,-0.5065f,  0.2540f,0.8399f,-0.4797f,  0.3603f,0.8412f,-0.4032f,  0.3592f,0.8544f,-0.3754f,  0.4595f,0.8385f,-0.2929f,  0.4583f,0.8483f,-0.2651f,  0.5512f,0.8149f,-0.1791f,  0.5501f,0.8211f,-0.1524f,  0.6340f,0.7705f,-0.0657f,  0.6333f,0.7728f,-0.0410f,  
	0.7070f,0.7058f,0.0439f,  0.7069f,0.7043f,0.0653f,  0.7693f,0.6221f,0.1457f,  0.7700f,0.6169f,0.1628f,  0.8201f,0.5210f,0.2364f,  0.8220f,0.5126f,0.2482f,  0.8591f,0.4051f,0.3129f,  0.8623f,0.3939f,0.3183f,  0.8857f,0.2772f,0.3724f,  0.8904f,0.2639f,0.3709f,  0.8997f,0.1405f,0.4132f,  0.9060f,0.1260f,0.4040f,  0.9011f,-0.0012f,0.4337f,  0.9089f,-0.0160f,0.4167f,  0.8896f,-0.1442f,0.4334f,  
	0.8989f,-0.1584f,0.4085f,  0.8652f,-0.2847f,0.4127f,  0.8758f,-0.2974f,0.3802f,  0.8281f,-0.4188f,0.3726f,  0.8396f,-0.4291f,0.3331f,  0.7783f,-0.5431f,0.3151f,  0.7903f,-0.5502f,0.2695f,  0.7161f,-0.6544f,0.2428f,  0.7282f,-0.6578f,0.1923f,  0.6419f,-0.7501f,0.1590f,  0.6537f,-0.7495f,0.1050f,  0.5563f,-0.8282f,0.0677f,  0.5672f,-0.8235f,0.0116f,  0.4601f,-0.8874f,-0.0272f,  0.4698f,-0.8788f,-0.0837f,  
	0.3544f,-0.9272f,-0.1212f,  0.3626f,-0.9151f,-0.1766f,  0.2406f,-0.9476f,-0.2101f,  0.2471f,-0.9326f,-0.2629f,  0.1204f,-0.9494f,-0.2900f,  0.1251f,-0.9325f,-0.3388f,  -0.0042f,-0.9339f,-0.3574f,  -0.0012f,-0.9160f,-0.4011f,  -0.1309f,-0.9030f,-0.4092f,  -0.1294f,-0.8851f,-0.4470f,  -0.2574f,-0.8587f,-0.4432f,  -0.2571f,-0.8418f,-0.4746f,  -0.3809f,-0.8033f,-0.4578f,  -0.3815f,-0.7885f,-0.4824f,  -0.4989f,-0.7393f,-0.4523f,  
	-0.5000f,-0.7273f,-0.4702f,  -0.6086f,-0.6690f,-0.4267f,  -0.6099f,-0.6604f,-0.4381f,  -0.7077f,-0.5945f,-0.3817f,  -0.7086f,-0.5900f,-0.3869f,  -0.7049f,-0.6192f,0.3461f,  -0.7044f,-0.6247f,0.3369f,  -0.7662f,-0.6024f,0.2239f,  -0.7657f,-0.6065f,0.2142f,  -0.8142f,-0.5723f,0.0977f,  -0.8136f,-0.5747f,0.0878f,  -0.8481f,-0.5290f,-0.0298f,  -0.8474f,-0.5294f,-0.0397f,  -0.8672f,-0.4728f,-0.1560f,  -0.8664f,-0.4711f,-0.1656f,  
	-0.8713f,-0.4045f,-0.2780f,  -0.8703f,-0.4003f,-0.2870f,  -0.8601f,-0.3248f,-0.3934f,  -0.8590f,-0.3181f,-0.4013f,  -0.8339f,-0.2352f,-0.4993f,  -0.8326f,-0.2258f,-0.5057f,  -0.7932f,-0.1371f,-0.5934f,  -0.7918f,-0.1250f,-0.5978f,  -0.7386f,-0.0324f,-0.6733f,  -0.7372f,-0.0178f,-0.6754f,  -0.6713f,0.0769f,-0.7372f,  -0.6699f,0.0937f,-0.7365f,  -0.5925f,0.1882f,-0.7833f,  -0.5911f,0.2072f,-0.7795f,  -0.5034f,0.2992f,-0.8106f,  
	-0.5023f,0.3199f,-0.8034f,  -0.4060f,0.4072f,-0.8182f,  -0.4051f,0.4289f,-0.8074f,  -0.3018f,0.5093f,-0.8060f,  -0.3012f,0.5317f,-0.7916f,  -0.1927f,0.6028f,-0.7743f,  -0.1925f,0.6252f,-0.7564f,  -0.0806f,0.6850f,-0.7241f,  -0.0808f,0.7068f,-0.7028f,  0.0325f,0.7535f,-0.6567f,  0.0317f,0.7738f,-0.6326f,  0.1446f,0.8057f,-0.5745f,  0.1435f,0.8241f,-0.5480f,  0.2540f,0.8399f,-0.4797f,  0.2527f,0.8557f,-0.4515f,  
	0.3592f,0.8544f,-0.3754f,  0.3576f,0.8672f,-0.3466f,  0.4583f,0.8483f,-0.2651f,  0.4567f,0.8576f,-0.2366f,  0.5501f,0.8211f,-0.1524f,  0.5487f,0.8266f,-0.1251f,  0.6333f,0.7728f,-0.0410f,  0.6323f,0.7745f,-0.0163f,  0.7069f,0.7043f,0.0653f,  0.7067f,0.7022f,0.0865f,  0.7700f,0.6169f,0.1628f,  0.7708f,0.6113f,0.1792f,  0.8220f,0.5126f,0.2482f,  0.8241f,0.5039f,0.2588f,  0.8623f,0.3939f,0.3183f,  
	0.8658f,0.3828f,0.3223f,  0.8904f,0.2639f,0.3709f,  0.8955f,0.2511f,0.3675f,  0.9060f,0.1260f,0.4040f,  0.9127f,0.1125f,0.3928f,  0.9089f,-0.0160f,0.4167f,  0.9172f,-0.0295f,0.3974f,  0.8989f,-0.1584f,0.4085f,  0.9085f,-0.1708f,0.3814f,  0.8758f,-0.2974f,0.3802f,  0.8865f,-0.3077f,0.3455f,  0.8396f,-0.4291f,0.3331f,  0.8511f,-0.4366f,0.2916f,  0.7903f,-0.5502f,0.2695f,  0.8021f,-0.5543f,0.2222f,  
	0.7282f,-0.6578f,0.1923f,  0.7399f,-0.6579f,0.1405f,  0.6537f,-0.7495f,0.1050f,  0.6647f,-0.7454f,0.0501f,  0.5672f,-0.8235f,0.0116f,  0.5772f,-0.8154f,-0.0448f,  0.4698f,-0.8788f,-0.0837f,  0.4784f,-0.8669f,-0.1400f,  0.3626f,-0.9151f,-0.1766f,  0.3696f,-0.9000f,-0.2312f,  0.2471f,-0.9326f,-0.2629f,  0.2523f,-0.9151f,-0.3145f,  0.1251f,-0.9325f,-0.3388f,  0.1286f,-0.9134f,-0.3862f,  -0.0012f,-0.9160f,-0.4011f,  
	0.0007f,-0.8964f,-0.4433f,  -0.1294f,-0.8851f,-0.4470f,  -0.1290f,-0.8660f,-0.4832f,  -0.2571f,-0.8418f,-0.4746f,  -0.2577f,-0.8241f,-0.5044f,  -0.3815f,-0.7885f,-0.4824f,  -0.3828f,-0.7731f,-0.5057f,  -0.5000f,-0.7273f,-0.4702f,  -0.5016f,-0.7151f,-0.4869f,  -0.6099f,-0.6604f,-0.4381f,  -0.6113f,-0.6520f,-0.4486f,  -0.7086f,-0.5900f,-0.3869f,  -0.7095f,-0.5858f,-0.3917f,  -0.7044f,-0.6247f,0.3369f,  -0.7040f,-0.6304f,0.3271f,  
	-0.7657f,-0.6065f,0.2142f,  -0.7652f,-0.6106f,0.2038f,  -0.8136f,-0.5747f,0.0878f,  -0.8131f,-0.5770f,0.0772f,  -0.8474f,-0.5294f,-0.0397f,  -0.8468f,-0.5296f,-0.0503f,  -0.8664f,-0.4711f,-0.1656f,  -0.8656f,-0.4689f,-0.1757f,  -0.8703f,-0.4003f,-0.2870f,  -0.8693f,-0.3955f,-0.2964f,  -0.8590f,-0.3181f,-0.4013f,  -0.8579f,-0.3106f,-0.4094f,  -0.8326f,-0.2258f,-0.5057f,  -0.8314f,-0.2156f,-0.5121f,  -0.7918f,-0.1250f,-0.5978f,  
	-0.7905f,-0.1121f,-0.6021f,  -0.7372f,-0.0178f,-0.6754f,  -0.7359f,-0.0023f,-0.6770f,  -0.6699f,0.0937f,-0.7365f,  -0.6687f,0.1116f,-0.7351f,  -0.5911f,0.2072f,-0.7795f,  -0.5900f,0.2271f,-0.7748f,  -0.5023f,0.3199f,-0.8034f,  -0.5014f,0.3413f,-0.7950f,  -0.4051f,0.4289f,-0.8074f,  -0.4045f,0.4515f,-0.7953f,  -0.3012f,0.5317f,-0.7916f,  -0.3010f,0.5547f,-0.7757f,  -0.1925f,0.6252f,-0.7564f,  -0.1928f,0.6481f,-0.7368f,  
	-0.0808f,0.7068f,-0.7028f,  -0.0817f,0.7287f,-0.6800f,  0.0317f,0.7738f,-0.6326f,  0.0304f,0.7942f,-0.6069f,  0.1435f,0.8241f,-0.5480f,  0.1419f,0.8424f,-0.5199f,  0.2527f,0.8557f,-0.4515f,  0.2507f,0.8712f,-0.4221f,  0.3576f,0.8672f,-0.3466f,  0.3555f,0.8794f,-0.3167f,  0.4567f,0.8576f,-0.2366f,  0.4547f,0.8662f,-0.2071f,  0.5487f,0.8266f,-0.1251f,  0.5470f,0.8315f,-0.0975f,  0.6323f,0.7745f,-0.0163f,  
	0.6313f,0.7755f,0.0086f,  0.7067f,0.7022f,0.0865f,  0.7065f,0.6996f,0.1072f,  0.7708f,0.6113f,0.1792f,  0.7718f,0.6053f,0.1948f,  0.8241f,0.5039f,0.2588f,  0.8264f,0.4952f,0.2682f,  0.8658f,0.3828f,0.3223f,  0.8696f,0.3720f,0.3247f,  0.8955f,0.2511f,0.3675f,  0.9009f,0.2390f,0.3622f,  0.9127f,0.1125f,0.3928f,  0.9198f,0.1000f,0.3795f,  0.9172f,-0.0295f,0.3974f,  0.9257f,-0.0414f,0.3760f,  
	0.9085f,-0.1708f,0.3814f,  0.9183f,-0.1811f,0.3520f,  0.8865f,-0.3077f,0.3455f,  0.8972f,-0.3156f,0.3088f,  0.8511f,-0.4366f,0.2916f,  0.8623f,-0.4413f,0.2484f,  0.8021f,-0.5543f,0.2222f,  0.8135f,-0.5551f,0.1736f,  0.7399f,-0.6579f,0.1405f,  0.7508f,-0.6547f,0.0878f,  0.6647f,-0.7454f,0.0501f,  0.6748f,-0.7380f,-0.0051f,  0.5772f,-0.8154f,-0.0448f,  0.5861f,-0.8039f,-0.1010f,  0.4784f,-0.8669f,-0.1400f,  
	0.4858f,-0.8519f,-0.1956f,  0.3696f,-0.9000f,-0.2312f,  0.3752f,-0.8821f,-0.2848f,  0.2523f,-0.9151f,-0.3145f,  0.2562f,-0.8952f,-0.3647f,  0.1286f,-0.9134f,-0.3862f,  0.1308f,-0.8924f,-0.4319f,  0.0007f,-0.8964f,-0.4433f,  0.0014f,-0.8753f,-0.4836f,  -0.1290f,-0.8660f,-0.4832f,  -0.1295f,-0.8457f,-0.5176f,  -0.2577f,-0.8241f,-0.5044f,  -0.2591f,-0.8058f,-0.5325f,  -0.3828f,-0.7731f,-0.5057f,  -0.3847f,-0.7575f,-0.5275f,  
	-0.5016f,-0.7151f,-0.4869f,  -0.5036f,-0.7028f,-0.5025f,  -0.6113f,-0.6520f,-0.4486f,  -0.6130f,-0.6437f,-0.4582f,  -0.7095f,-0.5858f,-0.3917f,  -0.7105f,-0.5818f,-0.3959f,  -0.7040f,-0.6304f,0.3271f,  -0.7038f,-0.6361f,0.3164f,  -0.7652f,-0.6106f,0.2038f,  -0.7649f,-0.6147f,0.1927f,  -0.8131f,-0.5770f,0.0772f,  -0.8126f,-0.5791f,0.0659f,  -0.8468f,-0.5296f,-0.0503f,  -0.8461f,-0.5294f,-0.0615f,  -0.8656f,-0.4689f,-0.1757f,  
	-0.8648f,-0.4661f,-0.1864f,  -0.8693f,-0.3955f,-0.2964f,  -0.8684f,-0.3901f,-0.3061f,  -0.8579f,-0.3106f,-0.4094f,  -0.8568f,-0.3024f,-0.4176f,  -0.8314f,-0.2156f,-0.5121f,  -0.8303f,-0.2045f,-0.5185f,  -0.7905f,-0.1121f,-0.6021f,  -0.7893f,-0.0983f,-0.6060f,  -0.7359f,-0.0023f,-0.6770f,  -0.7348f,0.0142f,-0.6782f,  -0.6687f,0.1116f,-0.7351f,  -0.6676f,0.1304f,-0.7330f,  -0.5900f,0.2271f,-0.7748f,  -0.5892f,0.2478f,-0.7690f,  
	-0.5014f,0.3413f,-0.7950f,  -0.5009f,0.3636f,-0.7854f,  -0.4045f,0.4515f,-0.7953f,  -0.4044f,0.4746f,-0.7818f,  -0.3010f,0.5547f,-0.7757f,  -0.3014f,0.5782f,-0.7582f,  -0.1928f,0.6481f,-0.7368f,  -0.1937f,0.6711f,-0.7156f,  -0.0817f,0.7287f,-0.6800f,  -0.0831f,0.7507f,-0.6554f,  0.0304f,0.7942f,-0.6069f,  0.0286f,0.8145f,-0.5794f,  0.1419f,0.8424f,-0.5199f,  0.1396f,0.8603f,-0.4904f,  0.2507f,0.8712f,-0.4221f,  
	0.2482f,0.8862f,-0.3913f,  0.3555f,0.8794f,-0.3167f,  0.3529f,0.8910f,-0.2856f,  0.4547f,0.8662f,-0.2071f,  0.4522f,0.8742f,-0.1770f,  0.5470f,0.8315f,-0.0975f,  0.5450f,0.8356f,-0.0693f,  0.6313f,0.7755f,0.0086f,  0.6300f,0.7759f,0.0334f,  0.7065f,0.6996f,0.1072f,  0.7062f,0.6964f,0.1275f,  0.7718f,0.6053f,0.1948f,  0.7728f,0.5991f,0.2095f,  0.8264f,0.4952f,0.2682f,  0.8288f,0.4865f,0.2763f,  
	0.8696f,0.3720f,0.3247f,  0.8737f,0.3616f,0.3255f,  0.9009f,0.2390f,0.3622f,  0.9067f,0.2278f,0.3551f,  0.9198f,0.1000f,0.3795f,  0.9271f,0.0888f,0.3642f,  0.9257f,-0.0414f,0.3760f,  0.9344f,-0.0514f,0.3525f,  0.9183f,-0.1811f,0.3520f,  0.9281f,-0.1891f,0.3207f,  0.8972f,-0.3156f,0.3088f,  0.9078f,-0.3207f,0.2703f,  0.8623f,-0.4413f,0.2484f,  0.8731f,-0.4429f,0.2037f,  0.8135f,-0.5551f,0.1736f,  
	0.8241f,-0.5527f,0.1240f,  0.7508f,-0.6547f,0.0878f,  0.7608f,-0.6481f,0.0346f,  0.6748f,-0.7380f,-0.0051f,  0.6837f,-0.7273f,-0.0604f,  0.5861f,-0.8039f,-0.1010f,  0.5936f,-0.7893f,-0.1567f,  0.4858f,-0.8519f,-0.1956f,  0.4917f,-0.8340f,-0.2503f,  0.3752f,-0.8821f,-0.2848f,  0.3794f,-0.8617f,-0.3370f,  0.2562f,-0.8952f,-0.3647f,  0.2587f,-0.8731f,-0.4132f,  0.1308f,-0.8924f,-0.4319f,  0.1317f,-0.8697f,-0.4758f,  
	0.0014f,-0.8753f,-0.4836f,  0.0009f,-0.8529f,-0.5221f,  -0.1295f,-0.8457f,-0.5176f,  -0.1310f,-0.8247f,-0.5502f,  -0.2591f,-0.8058f,-0.5325f,  -0.2613f,-0.7869f,-0.5590f,  -0.3847f,-0.7575f,-0.5275f,  -0.3872f,-0.7416f,-0.5478f,  -0.5036f,-0.7028f,-0.5025f,  -0.5059f,-0.6906f,-0.5169f,  -0.6130f,-0.6437f,-0.4582f,  -0.6148f,-0.6355f,-0.4670f,  -0.7105f,-0.5818f,-0.3959f,  -0.7115f,-0.5781f,-0.3996f,  -0.7038f,-0.6361f,0.3164f,  
	-0.7035f,-0.6419f,0.3050f,  -0.7649f,-0.6147f,0.1927f,  -0.7646f,-0.6187f,0.1809f,  -0.8126f,-0.5791f,0.0659f,  -0.8122f,-0.5809f,0.0539f,  -0.8461f,-0.5294f,-0.0615f,  -0.8456f,-0.5288f,-0.0733f,  -0.8648f,-0.4661f,-0.1864f,  -0.8641f,-0.4629f,-0.1975f,  -0.8684f,-0.3901f,-0.3061f,  -0.8675f,-0.3840f,-0.3160f,  -0.8568f,-0.3024f,-0.4176f,  -0.8558f,-0.2934f,-0.4260f,  -0.8303f,-0.2045f,-0.5185f,  -0.8292f,-0.1927f,-0.5246f,  
	-0.7893f,-0.0983f,-0.6060f,  -0.7883f,-0.0835f,-0.6096f,  -0.7348f,0.0142f,-0.6782f,  -0.7338f,0.0316f,-0.6786f,  -0.6676f,0.1304f,-0.7330f,  -0.6669f,0.1501f,-0.7299f,  -0.5892f,0.2478f,-0.7690f,  -0.5888f,0.2694f,-0.7621f,  -0.5009f,0.3636f,-0.7854f,  -0.5009f,0.3865f,-0.7744f,  -0.4044f,0.4746f,-0.7818f,  -0.4048f,0.4985f,-0.7666f,  -0.3014f,0.5782f,-0.7582f,  -0.3024f,0.6020f,-0.7391f,  -0.1937f,0.6711f,-0.7156f,  
	-0.1952f,0.6944f,-0.6926f,  -0.0831f,0.7507f,-0.6554f,  -0.0851f,0.7728f,-0.6290f,  0.0286f,0.8145f,-0.5794f,  0.0260f,0.8346f,-0.5503f,  0.1396f,0.8603f,-0.4904f,  0.1366f,0.8777f,-0.4593f,  0.2482f,0.8862f,-0.3913f,  0.2450f,0.9005f,-0.3592f,  0.3529f,0.8910f,-0.2856f,  0.3497f,0.9019f,-0.2536f,  0.4522f,0.8742f,-0.1770f,  0.4494f,0.8813f,-0.1461f,  0.5450f,0.8356f,-0.0693f,  0.5427f,0.8389f,-0.0410f,  
	0.6300f,0.7759f,0.0334f,  0.6286f,0.7756f,0.0580f,  0.7062f,0.6964f,0.1275f,  0.7060f,0.6928f,0.1473f,  0.7728f,0.5991f,0.2095f,  0.7739f,0.5926f,0.2233f,  0.8288f,0.4865f,0.2763f,  0.8315f,0.4779f,0.2832f,  0.8737f,0.3616f,0.3255f,  0.8780f,0.3517f,0.3246f,  0.9067f,0.2278f,0.3551f,  0.9126f,0.2175f,0.3462f,  0.9271f,0.0888f,0.3642f,  0.9345f,0.0792f,0.3469f,  0.9344f,-0.0514f,0.3525f,  
	0.9431f,-0.0594f,0.3271f,  0.9281f,-0.1891f,0.3207f,  0.9377f,-0.1947f,0.2876f,  0.9078f,-0.3207f,0.2703f,  0.9179f,-0.3230f,0.2304f,  0.8731f,-0.4429f,0.2037f,  0.8833f,-0.4414f,0.1580f,  0.8241f,-0.5527f,0.1240f,  0.8338f,-0.5471f,0.0737f,  0.7608f,-0.6481f,0.0346f,  0.7697f,-0.6382f,-0.0187f,  0.6837f,-0.7273f,-0.0604f,  0.6914f,-0.7133f,-0.1151f,  0.5936f,-0.7893f,-0.1567f,  0.5998f,-0.7717f,-0.2114f,  
	0.4917f,-0.8340f,-0.2503f,  0.4962f,-0.8135f,-0.3035f,  0.3794f,-0.8617f,-0.3370f,  0.3822f,-0.8390f,-0.3874f,  0.2587f,-0.8731f,-0.4132f,  0.2598f,-0.8492f,-0.4598f,  0.1317f,-0.8697f,-0.4758f,  0.1313f,-0.8455f,-0.5176f,  0.0009f,-0.8529f,-0.5221f,  -0.0007f,-0.8295f,-0.5585f,  -0.1310f,-0.8247f,-0.5502f,  -0.1335f,-0.8030f,-0.5809f,  -0.2613f,-0.7869f,-0.5590f,  -0.2643f,-0.7677f,-0.5838f,  -0.3872f,-0.7416f,-0.5478f,  
	-0.3902f,-0.7256f,-0.5667f,  -0.5059f,-0.6906f,-0.5169f,  -0.5086f,-0.6784f,-0.5302f,  -0.6148f,-0.6355f,-0.4670f,  -0.6168f,-0.6276f,-0.4751f,  -0.7115f,-0.5781f,-0.3996f,  -0.7125f,-0.5746f,-0.4028f,  -0.7035f,-0.6419f,0.3050f,  -0.7034f,-0.6476f,0.2928f,  -0.7646f,-0.6187f,0.1809f,  -0.7643f,-0.6225f,0.1682f,  -0.8122f,-0.5809f,0.0539f,  -0.8118f,-0.5825f,0.0412f,  -0.8456f,-0.5288f,-0.0733f,  -0.8450f,-0.5278f,-0.0856f,  
	-0.8641f,-0.4629f,-0.1975f,  -0.8634f,-0.4591f,-0.2090f,  -0.8675f,-0.3840f,-0.3160f,  -0.8668f,-0.3773f,-0.3262f,  -0.8558f,-0.2934f,-0.4260f,  -0.8550f,-0.2837f,-0.4343f,  -0.8292f,-0.1927f,-0.5246f,  -0.8283f,-0.1798f,-0.5306f,  -0.7883f,-0.0835f,-0.6096f,  -0.7875f,-0.0679f,-0.6126f,  -0.7338f,0.0316f,-0.6786f,  -0.7332f,0.0498f,-0.6782f,  -0.6669f,0.1501f,-0.7299f,  -0.6665f,0.1707f,-0.7257f,  -0.5888f,0.2694f,-0.7621f,  
	-0.5887f,0.2917f,-0.7539f,  -0.5009f,0.3865f,-0.7744f,  -0.5013f,0.4102f,-0.7619f,  -0.4048f,0.4985f,-0.7666f,  -0.4058f,0.5226f,-0.7498f,  -0.3024f,0.6020f,-0.7391f,  -0.3040f,0.6261f,-0.7180f,  -0.1952f,0.6944f,-0.6926f,  -0.1974f,0.7177f,-0.6678f,  -0.0851f,0.7728f,-0.6290f,  -0.0880f,0.7946f,-0.6007f,  0.0260f,0.8346f,-0.5503f,  0.0227f,0.8542f,-0.5194f,  0.1366f,0.8777f,-0.4593f,  0.1329f,0.8946f,-0.4266f,  
	0.2450f,0.9005f,-0.3592f,  0.2412f,0.9142f,-0.3258f,  0.3497f,0.9019f,-0.2536f,  0.3461f,0.9120f,-0.2204f,  0.4494f,0.8813f,-0.1461f,  0.4461f,0.8876f,-0.1147f,  0.5427f,0.8389f,-0.0410f,  0.5401f,0.8415f,-0.0124f,  0.6286f,0.7756f,0.0580f,  0.6270f,0.7746f,0.0825f,  0.7060f,0.6928f,0.1473f,  0.7057f,0.6887f,0.1664f,  0.7739f,0.5926f,0.2233f,  0.7751f,0.5861f,0.2360f,  0.8315f,0.4779f,0.2832f,  
	0.8343f,0.4696f,0.2887f,  0.8780f,0.3517f,0.3246f,  0.8825f,0.3425f,0.3223f,  0.9126f,0.2175f,0.3462f,  0.9187f,0.2085f,0.3355f,  0.9345f,0.0792f,0.3469f,  0.9420f,0.0713f,0.3279f,  0.9431f,-0.0594f,0.3271f,  0.9517f,-0.0653f,0.3000f,  0.9377f,-0.1947f,0.2876f,  0.9470f,-0.1976f,0.2531f,  0.9179f,-0.3230f,0.2304f,  0.9275f,-0.3224f,0.1894f,  0.8833f,-0.4414f,0.1580f,  0.8927f,-0.4367f,0.1116f,  
	0.8338f,-0.5471f,0.0737f,  0.8425f,-0.5381f,0.0234f,  0.7697f,-0.6382f,-0.0187f,  0.7773f,-0.6250f,-0.0716f,  0.6914f,-0.7133f,-0.1151f,  0.6976f,-0.6963f,-0.1690f,  0.5998f,-0.7717f,-0.2114f,  0.6044f,-0.7514f,-0.2649f,  0.4962f,-0.8135f,-0.3035f,  0.4991f,-0.7905f,-0.3551f,  0.3822f,-0.8390f,-0.3874f,  0.3834f,-0.8142f,-0.4360f,  0.2598f,-0.8492f,-0.4598f,  0.2595f,-0.8236f,-0.5043f,  0.1313f,-0.8455f,-0.5176f,  
	0.1297f,-0.8201f,-0.5573f,  -0.0007f,-0.8295f,-0.5585f,  -0.0033f,-0.8052f,-0.5929f,  -0.1335f,-0.8030f,-0.5809f,  -0.1368f,-0.7807f,-0.6097f,  -0.2643f,-0.7677f,-0.5838f,  -0.2679f,-0.7483f,-0.6069f,  -0.3902f,-0.7256f,-0.5667f,  -0.3938f,-0.7096f,-0.5843f,  -0.5086f,-0.6784f,-0.5302f,  -0.5117f,-0.6663f,-0.5424f,  -0.6168f,-0.6276f,-0.4751f,  -0.6190f,-0.6198f,-0.4824f,  -0.7125f,-0.5746f,-0.4028f,  -0.7134f,-0.5714f,-0.4056f,  
	-0.7034f,-0.6476f,0.2928f,  -0.7034f,-0.6534f,0.2798f,  -0.7643f,-0.6225f,0.1682f,  -0.7642f,-0.6261f,0.1548f,  -0.8118f,-0.5825f,0.0412f,  -0.8115f,-0.5837f,0.0278f,  -0.8450f,-0.5278f,-0.0856f,  -0.8446f,-0.5263f,-0.0984f,  -0.8634f,-0.4591f,-0.2090f,  -0.8628f,-0.4547f,-0.2209f,  -0.8668f,-0.3773f,-0.3262f,  -0.8660f,-0.3698f,-0.3365f,  -0.8550f,-0.2837f,-0.4343f,  -0.8542f,-0.2730f,-0.4425f,  -0.8283f,-0.1798f,-0.5306f,  
	-0.8276f,-0.1661f,-0.5361f,  -0.7875f,-0.0679f,-0.6126f,  -0.7869f,-0.0513f,-0.6149f,  -0.7332f,0.0498f,-0.6782f,  -0.7328f,0.0690f,-0.6769f,  -0.6665f,0.1707f,-0.7257f,  -0.6665f,0.1920f,-0.7204f,  -0.5887f,0.2917f,-0.7539f,  -0.5892f,0.3149f,-0.7441f,  -0.5013f,0.4102f,-0.7619f,  -0.5024f,0.4342f,-0.7477f,  -0.4058f,0.5226f,-0.7498f,  -0.4075f,0.5471f,-0.7312f,  -0.3040f,0.6261f,-0.7180f,  -0.3063f,0.6504f,-0.6951f,  
	-0.1974f,0.7177f,-0.6678f,  -0.2004f,0.7410f,-0.6409f,  -0.0880f,0.7946f,-0.6007f,  -0.0916f,0.8161f,-0.5706f,  0.0227f,0.8542f,-0.5194f,  0.0186f,0.8734f,-0.4867f,  0.1329f,0.8946f,-0.4266f,  0.1285f,0.9108f,-0.3924f,  0.2412f,0.9142f,-0.3258f,  0.2368f,0.9270f,-0.2909f,  0.3461f,0.9120f,-0.2204f,  0.3419f,0.9211f,-0.1864f,  0.4461f,0.8876f,-0.1147f,  0.4424f,0.8930f,-0.0826f,  0.5401f,0.8415f,-0.0124f,  
	0.5374f,0.8432f,0.0164f,  0.6270f,0.7746f,0.0825f,  0.6254f,0.7730f,0.1067f,  0.7057f,0.6887f,0.1664f,  0.7054f,0.6843f,0.1849f,  0.7751f,0.5861f,0.2360f,  0.7764f,0.5795f,0.2478f,  0.8343f,0.4696f,0.2887f,  0.8373f,0.4617f,0.2929f,  0.8825f,0.3425f,0.3223f,  0.8871f,0.3342f,0.3183f,  0.9187f,0.2085f,0.3355f,  0.9248f,0.2008f,0.3232f,  0.9420f,0.0713f,0.3279f,  0.9494f,0.0652f,0.3073f,  
	0.9517f,-0.0653f,0.3000f,  0.9600f,-0.0689f,0.2716f,  0.9470f,-0.1976f,0.2531f,  0.9558f,-0.1980f,0.2175f,  0.9275f,-0.3224f,0.1894f,  0.9362f,-0.3189f,0.1476f,  0.8927f,-0.4367f,0.1116f,  0.9010f,-0.4290f,0.0650f,  0.8425f,-0.5381f,0.0234f,  0.8500f,-0.5261f,-0.0268f,  0.7773f,-0.6250f,-0.0716f,  0.7835f,-0.6089f,-0.1237f,  0.6976f,-0.6963f,-0.1690f,  0.7023f,-0.6764f,-0.2217f,  0.6044f,-0.7514f,-0.2649f,  
	0.6075f,-0.7285f,-0.3166f,  0.4991f,-0.7905f,-0.3551f,  0.5005f,-0.7653f,-0.4047f,  0.3834f,-0.8142f,-0.4360f,  0.3832f,-0.7877f,-0.4824f,  0.2595f,-0.8236f,-0.5043f,  0.2578f,-0.7967f,-0.5466f,  0.1297f,-0.8201f,-0.5573f,  0.1268f,-0.7938f,-0.5948f,  -0.0033f,-0.8052f,-0.5929f,  -0.0070f,-0.7804f,-0.6253f,  -0.1368f,-0.7807f,-0.6097f,  -0.1410f,-0.7582f,-0.6366f,  -0.2679f,-0.7483f,-0.6069f,  -0.2722f,-0.7287f,-0.6284f,  
	-0.3938f,-0.7096f,-0.5843f,  -0.3978f,-0.6936f,-0.6005f,  -0.5117f,-0.6663f,-0.5424f,  -0.5150f,-0.6544f,-0.5537f,  -0.6190f,-0.6198f,-0.4824f,  -0.6213f,-0.6123f,-0.4889f,  -0.7134f,-0.5714f,-0.4056f,  -0.7144f,-0.5685f,-0.4080f,  -0.7034f,-0.6534f,0.2798f,  -0.7035f,-0.6591f,0.2659f,  -0.7642f,-0.6261f,0.1548f,  -0.7641f,-0.6296f,0.1407f,  -0.8115f,-0.5837f,0.0278f,  -0.8112f,-0.5846f,0.0138f,  -0.8446f,-0.5263f,-0.0984f,  
	-0.8442f,-0.5243f,-0.1117f,  -0.8628f,-0.4547f,-0.2209f,  -0.8623f,-0.4497f,-0.2329f,  -0.8660f,-0.3698f,-0.3365f,  -0.8655f,-0.3615f,-0.3468f,  -0.8542f,-0.2730f,-0.4425f,  -0.8536f,-0.2615f,-0.4505f,  -0.8276f,-0.1661f,-0.5361f,  -0.8271f,-0.1516f,-0.5412f,  -0.7869f,-0.0513f,-0.6149f,  -0.7866f,-0.0339f,-0.6165f,  -0.7328f,0.0690f,-0.6769f,  -0.7329f,0.0890f,-0.6745f,  -0.6665f,0.1920f,-0.7204f,  -0.6670f,0.2142f,-0.7136f,  
	-0.5892f,0.3149f,-0.7441f,  -0.5903f,0.3384f,-0.7328f,  -0.5024f,0.4342f,-0.7477f,  -0.5041f,0.4588f,-0.7317f,  -0.4075f,0.5471f,-0.7312f,  -0.4099f,0.5719f,-0.7105f,  -0.3063f,0.6504f,-0.6951f,  -0.3095f,0.6747f,-0.6701f,  -0.2004f,0.7410f,-0.6409f,  -0.2044f,0.7639f,-0.6121f,  -0.0916f,0.8161f,-0.5706f,  -0.0961f,0.8371f,-0.5385f,  0.0186f,0.8734f,-0.4867f,  0.0136f,0.8918f,-0.4523f,  0.1285f,0.9108f,-0.3924f,  
	0.1234f,0.9261f,-0.3565f,  0.2368f,0.9270f,-0.2909f,  0.2317f,0.9388f,-0.2549f,  0.3419f,0.9211f,-0.1864f,  0.3371f,0.9292f,-0.1514f,  0.4424f,0.8930f,-0.0826f,  0.4384f,0.8974f,-0.0500f,  0.5374f,0.8432f,0.0164f,  0.5343f,0.8441f,0.0452f,  0.6254f,0.7730f,0.1067f,  0.6236f,0.7708f,0.1306f,  0.7054f,0.6843f,0.1849f,  0.7051f,0.6795f,0.2027f,  0.7764f,0.5795f,0.2478f,  0.7778f,0.5729f,0.2585f,  
	0.8373f,0.4617f,0.2929f,  0.8404f,0.4542f,0.2957f,  0.8871f,0.3342f,0.3183f,  0.8917f,0.3269f,0.3129f,  0.9248f,0.2008f,0.3232f,  0.9308f,0.1946f,0.3094f,  0.9494f,0.0652f,0.3073f,  0.9565f,0.0611f,0.2854f,  0.9600f,-0.0689f,0.2716f,  0.9678f,-0.0701f,0.2419f,  0.9558f,-0.1980f,0.2175f,  0.9638f,-0.1956f,0.1811f,  0.9362f,-0.3189f,0.1476f,  0.9441f,-0.3124f,0.1055f,  0.9010f,-0.4290f,0.0650f,  
	0.9082f,-0.4182f,0.0185f,  0.8500f,-0.5261f,-0.0268f,  0.8561f,-0.5111f,-0.0763f,  0.7835f,-0.6089f,-0.1237f,  0.7883f,-0.5900f,-0.1747f,  0.7023f,-0.6764f,-0.2217f,  0.7055f,-0.6541f,-0.2728f,  0.6075f,-0.7285f,-0.3166f,  0.6090f,-0.7034f,-0.3665f,  0.5005f,-0.7653f,-0.4047f,  0.5003f,-0.7384f,-0.4522f,  0.3832f,-0.7877f,-0.4824f,  0.3815f,-0.7598f,-0.5265f,  0.2578f,-0.7967f,-0.5466f,  0.2548f,-0.7688f,-0.5866f,  
	0.1268f,-0.7938f,-0.5948f,  0.1228f,-0.7667f,-0.6301f,  -0.0070f,-0.7804f,-0.6253f,  -0.0117f,-0.7551f,-0.6555f,  -0.1410f,-0.7582f,-0.6366f,  -0.1460f,-0.7354f,-0.6617f,  -0.2722f,-0.7287f,-0.6284f,  -0.2771f,-0.7091f,-0.6483f,  -0.3978f,-0.6936f,-0.6005f,  -0.4022f,-0.6778f,-0.6155f,  -0.5150f,-0.6544f,-0.5537f,  -0.5186f,-0.6427f,-0.5639f,  -0.6213f,-0.6123f,-0.4889f,  -0.6237f,-0.6051f,-0.4949f,  -0.7144f,-0.5685f,-0.4080f,  
	-0.7153f,-0.5658f,-0.4101f,  -0.7035f,-0.6591f,0.2659f,  -0.7036f,-0.6647f,0.2512f,  -0.7641f,-0.6296f,0.1407f,  -0.7640f,-0.6328f,0.1259f,  -0.8112f,-0.5846f,0.0138f,  -0.8111f,-0.5850f,-0.0007f,  -0.8442f,-0.5243f,-0.1117f,  -0.8438f,-0.5218f,-0.1254f,  -0.8623f,-0.4497f,-0.2329f,  -0.8619f,-0.4438f,-0.2453f,  -0.8655f,-0.3615f,-0.3468f,  -0.8650f,-0.3524f,-0.3571f,  -0.8536f,-0.2615f,-0.4505f,  -0.8532f,-0.2492f,-0.4581f,  
	-0.8271f,-0.1516f,-0.5412f,  -0.8269f,-0.1361f,-0.5456f,  -0.7866f,-0.0339f,-0.6165f,  -0.7867f,-0.0156f,-0.6172f,  -0.7329f,0.0890f,-0.6745f,  -0.7334f,0.1098f,-0.6709f,  -0.6670f,0.2142f,-0.7136f,  -0.6681f,0.2368f,-0.7054f,  -0.5903f,0.3384f,-0.7328f,  -0.5920f,0.3626f,-0.7198f,  -0.5041f,0.4588f,-0.7317f,  -0.5065f,0.4837f,-0.7138f,  -0.4099f,0.5719f,-0.7105f,  -0.4132f,0.5968f,-0.6878f,  -0.3095f,0.6747f,-0.6701f,  
	-0.3136f,0.6987f,-0.6431f,  -0.2044f,0.7639f,-0.6121f,  -0.2091f,0.7865f,-0.5811f,  -0.0961f,0.8371f,-0.5385f,  -0.1015f,0.8575f,-0.5045f,  0.0136f,0.8918f,-0.4523f,  0.0080f,0.9094f,-0.4159f,  0.1234f,0.9261f,-0.3565f,  0.1176f,0.9404f,-0.3191f,  0.2317f,0.9388f,-0.2549f,  0.2260f,0.9495f,-0.2177f,  0.3371f,0.9292f,-0.1514f,  0.3320f,0.9362f,-0.1156f,  0.4384f,0.8974f,-0.0500f,  0.4341f,0.9007f,-0.0170f,  
	0.5343f,0.8441f,0.0452f,  0.5311f,0.8441f,0.0741f,  0.6236f,0.7708f,0.1306f,  0.6218f,0.7679f,0.1541f,  0.7051f,0.6795f,0.2027f,  0.7049f,0.6744f,0.2197f,  0.7778f,0.5729f,0.2585f,  0.7792f,0.5665f,0.2681f,  0.8404f,0.4542f,0.2957f,  0.8434f,0.4474f,0.2973f,  0.8917f,0.3269f,0.3129f,  0.8963f,0.3207f,0.3062f,  0.9308f,0.1946f,0.3094f,  0.9366f,0.1900f,0.2943f,  0.9565f,0.0611f,0.2854f,  
	0.9632f,0.0589f,0.2623f,  0.9678f,-0.0701f,0.2419f,  0.9749f,-0.0689f,0.2115f,  0.9638f,-0.1956f,0.1811f,  0.9710f,-0.1905f,0.1442f,  0.9441f,-0.3124f,0.1055f,  0.9508f,-0.3031f,0.0634f,  0.9082f,-0.4182f,0.0185f,  0.9141f,-0.4046f,-0.0276f,  0.8561f,-0.5111f,-0.0763f,  0.8608f,-0.4934f,-0.1248f,  0.7883f,-0.5900f,-0.1747f,  0.7915f,-0.5685f,-0.2243f,  0.7055f,-0.6541f,-0.2728f,  0.7071f,-0.6294f,-0.3221f,  
	0.6090f,-0.7034f,-0.3665f,  0.6090f,-0.6764f,-0.4144f,  0.5003f,-0.7384f,-0.4522f,  0.4987f,-0.7098f,-0.4974f,  0.3815f,-0.7598f,-0.5265f,  0.3785f,-0.7306f,-0.5683f,  0.2548f,-0.7688f,-0.5866f,  0.2506f,-0.7400f,-0.6242f,  0.1228f,-0.7667f,-0.6301f,  0.1178f,-0.7391f,-0.6632f,  -0.0117f,-0.7551f,-0.6555f,  -0.0173f,-0.7295f,-0.6838f,  -0.1460f,-0.7354f,-0.6617f,  -0.1517f,-0.7125f,-0.6851f,  -0.2771f,-0.7091f,-0.6483f,  
	-0.2825f,-0.6896f,-0.6668f,  -0.4022f,-0.6778f,-0.6155f,  -0.4070f,-0.6622f,-0.6292f,  -0.5186f,-0.6427f,-0.5639f,  -0.5224f,-0.6313f,-0.5732f,  -0.6237f,-0.6051f,-0.4949f,  -0.6262f,-0.5981f,-0.5002f,  -0.7153f,-0.5658f,-0.4101f,  -0.7162f,-0.5634f,-0.4118f,  -0.7036f,-0.6647f,0.2512f,  -0.7038f,-0.6702f,0.2357f,  -0.7640f,-0.6328f,0.1259f,  -0.7641f,-0.6356f,0.1103f,  -0.8111f,-0.5850f,-0.0007f,  -0.8109f,-0.5850f,-0.0158f,  
	-0.8438f,-0.5218f,-0.1254f,  -0.8436f,-0.5186f,-0.1395f,  -0.8619f,-0.4438f,-0.2453f,  -0.8616f,-0.4373f,-0.2577f,  -0.8650f,-0.3524f,-0.3571f,  -0.8647f,-0.3426f,-0.3672f,  -0.8532f,-0.2492f,-0.4581f,  -0.8531f,-0.2360f,-0.4653f,  -0.8269f,-0.1361f,-0.5456f,  -0.8270f,-0.1198f,-0.5492f,  -0.7867f,-0.0156f,-0.6172f,  -0.7872f,0.0036f,-0.6167f,  -0.7334f,0.1098f,-0.6709f,  -0.7344f,0.1312f,-0.6659f,  -0.6681f,0.2368f,-0.7054f,  
	-0.6698f,0.2602f,-0.6955f,  -0.5920f,0.3626f,-0.7198f,  -0.5945f,0.3871f,-0.7048f,  -0.5065f,0.4837f,-0.7138f,  -0.5098f,0.5087f,-0.6938f,  -0.4132f,0.5968f,-0.6878f,  -0.4174f,0.6215f,-0.6630f,  -0.3136f,0.6987f,-0.6431f,  -0.3185f,0.7224f,-0.6137f,  -0.2091f,0.7865f,-0.5811f,  -0.2148f,0.8084f,-0.5480f,  -0.1015f,0.8575f,-0.5045f,  -0.1076f,0.8770f,-0.4682f,  0.0080f,0.9094f,-0.4159f,  0.0015f,0.9259f,-0.3778f,  
	0.1176f,0.9404f,-0.3191f,  0.1110f,0.9535f,-0.2801f,  0.2260f,0.9495f,-0.2177f,  0.2198f,0.9590f,-0.1791f,  0.3320f,0.9362f,-0.1156f,  0.3264f,0.9419f,-0.0789f,  0.4341f,0.9007f,-0.0170f,  0.4295f,0.9029f,0.0164f,  0.5311f,0.8441f,0.0741f,  0.5277f,0.8432f,0.1029f,  0.6218f,0.7679f,0.1541f,  0.6199f,0.7644f,0.1772f,  0.7049f,0.6744f,0.2197f,  0.7047f,0.6691f,0.2359f,  0.7792f,0.5665f,0.2681f,  
	0.7807f,0.5603f,0.2767f,  0.8434f,0.4474f,0.2973f,  0.8465f,0.4413f,0.2978f,  0.8963f,0.3207f,0.3062f,  0.9008f,0.3157f,0.2982f,  0.9366f,0.1900f,0.2943f,  0.9422f,0.1870f,0.2781f,  0.9632f,0.0589f,0.2623f,  0.9694f,0.0589f,0.2383f,  0.9749f,-0.0689f,0.2115f,  0.9814f,-0.0654f,0.1805f,  0.9710f,-0.1905f,0.1442f,  0.9773f,-0.1829f,0.1072f,  0.9508f,-0.3031f,0.0634f,  0.9564f,-0.2912f,0.0216f,  
	0.9141f,-0.4046f,-0.0276f,  0.9186f,-0.3884f,-0.0727f,  0.8608f,-0.4934f,-0.1248f,  0.8640f,-0.4732f,-0.1720f,  0.7915f,-0.5685f,-0.2243f,  0.7932f,-0.5447f,-0.2722f,  0.7071f,-0.6294f,-0.3221f,  0.7072f,-0.6029f,-0.3694f,  0.6090f,-0.6764f,-0.4144f,  0.6074f,-0.6477f,-0.4599f,  0.4987f,-0.7098f,-0.4974f,  0.4956f,-0.6800f,-0.5403f,  0.3785f,-0.7306f,-0.5683f,  0.3741f,-0.7006f,-0.6077f,  0.2506f,-0.7400f,-0.6242f,  
	0.2452f,-0.7105f,-0.6596f,  0.1178f,-0.7391f,-0.6632f,  0.1117f,-0.7111f,-0.6941f,  -0.0173f,-0.7295f,-0.6838f,  -0.0237f,-0.7037f,-0.7101f,  -0.1517f,-0.7125f,-0.6851f,  -0.1580f,-0.6897f,-0.7067f,  -0.2825f,-0.6896f,-0.6668f,  -0.2884f,-0.6703f,-0.6838f,  -0.4070f,-0.6622f,-0.6292f,  -0.4121f,-0.6467f,-0.6418f,  -0.5224f,-0.6313f,-0.5732f,  -0.5264f,-0.6201f,-0.5817f,  -0.6262f,-0.5981f,-0.5002f,  -0.6288f,-0.5914f,-0.5049f,  
	-0.7162f,-0.5634f,-0.4118f,  -0.7171f,-0.5613f,-0.4132f,  -0.7038f,-0.6702f,0.2357f,  -0.7040f,-0.6754f,0.2195f,  -0.7641f,-0.6356f,0.1103f,  -0.7641f,-0.6382f,0.0942f,  -0.8109f,-0.5850f,-0.0158f,  -0.8109f,-0.5844f,-0.0314f,  -0.8436f,-0.5186f,-0.1395f,  -0.8434f,-0.5147f,-0.1538f,  -0.8616f,-0.4373f,-0.2577f,  -0.8614f,-0.4301f,-0.2701f,  -0.8647f,-0.3426f,-0.3672f,  -0.8647f,-0.3319f,-0.3771f,  -0.8531f,-0.2360f,-0.4653f,  
	-0.8532f,-0.2220f,-0.4719f,  -0.8270f,-0.1198f,-0.5492f,  -0.8276f,-0.1026f,-0.5519f,  -0.7872f,0.0036f,-0.6167f,  -0.7882f,0.0234f,-0.6149f,  -0.7344f,0.1312f,-0.6659f,  -0.7361f,0.1533f,-0.6593f,  -0.6698f,0.2602f,-0.6955f,  -0.6722f,0.2840f,-0.6838f,  -0.5945f,0.3871f,-0.7048f,  -0.5977f,0.4119f,-0.6878f,  -0.5098f,0.5087f,-0.6938f,  -0.5140f,0.5336f,-0.6716f,  -0.4174f,0.6215f,-0.6630f,  -0.4225f,0.6460f,-0.6358f,  
	-0.3185f,0.7224f,-0.6137f,  -0.3245f,0.7456f,-0.5821f,  -0.2148f,0.8084f,-0.5480f,  -0.2214f,0.8296f,-0.5126f,  -0.1076f,0.8770f,-0.4682f,  -0.1147f,0.8955f,-0.4300f,  0.0015f,0.9259f,-0.3778f,  -0.0059f,0.9412f,-0.3378f,  0.1110f,0.9535f,-0.2801f,  0.1039f,0.9653f,-0.2396f,  0.2198f,0.9590f,-0.1791f,  0.2131f,0.9670f,-0.1395f,  0.3264f,0.9419f,-0.0789f,  0.3204f,0.9464f,-0.0416f,  0.4295f,0.9029f,0.0164f,  
	0.4246f,0.9040f,0.0501f,  0.5277f,0.8432f,0.1029f,  0.5243f,0.8413f,0.1317f,  0.6199f,0.7644f,0.1772f,  0.6181f,0.7603f,0.1997f,  0.7047f,0.6691f,0.2359f,  0.7046f,0.6636f,0.2513f,  0.7807f,0.5603f,0.2767f,  0.7822f,0.5544f,0.2843f,  0.8465f,0.4413f,0.2978f,  0.8495f,0.4360f,0.2971f,  0.9008f,0.3157f,0.2982f,  0.9050f,0.3119f,0.2892f,  0.9422f,0.1870f,0.2781f,  0.9473f,0.1857f,0.2610f,  
	0.9694f,0.0589f,0.2383f,  0.9750f,0.0609f,0.2137f,  0.9814f,-0.0654f,0.1805f,  0.9870f,-0.0595f,0.1493f,  0.9773f,-0.1829f,0.1072f,  0.9824f,-0.1728f,0.0705f,  0.9564f,-0.2912f,0.0216f,  0.9607f,-0.2768f,-0.0194f,  0.9186f,-0.3884f,-0.0727f,  0.9217f,-0.3698f,-0.1168f,  0.8640f,-0.4732f,-0.1720f,  0.8657f,-0.4507f,-0.2177f,  0.7932f,-0.5447f,-0.2722f,  0.7933f,-0.5190f,-0.3182f,  0.7072f,-0.6029f,-0.3694f,  
	0.7057f,-0.5746f,-0.4145f,  0.6074f,-0.6477f,-0.4599f,  0.6043f,-0.6178f,-0.5031f,  0.4956f,-0.6800f,-0.5403f,  0.4912f,-0.6492f,-0.5808f,  0.3741f,-0.7006f,-0.6077f,  0.3685f,-0.6698f,-0.6447f,  0.2452f,-0.7105f,-0.6596f,  0.2388f,-0.6806f,-0.6926f,  0.1117f,-0.7111f,-0.6941f,  0.1047f,-0.6829f,-0.7230f,  -0.0237f,-0.7037f,-0.7101f,  -0.0308f,-0.6780f,-0.7345f,  -0.1580f,-0.6897f,-0.7067f,  -0.1650f,-0.6670f,-0.7266f,  
	-0.2884f,-0.6703f,-0.6838f,  -0.2948f,-0.6511f,-0.6994f,  -0.4121f,-0.6467f,-0.6418f,  -0.4175f,-0.6316f,-0.6533f,  -0.5264f,-0.6201f,-0.5817f,  -0.5305f,-0.6092f,-0.5894f,  -0.6288f,-0.5914f,-0.5049f,  -0.6313f,-0.5850f,-0.5091f,  -0.7171f,-0.5613f,-0.4132f,  -0.7178f,-0.5595f,-0.4143f,  -0.7040f,-0.6754f,0.2195f,  -0.7043f,-0.6805f,0.2025f,  -0.7641f,-0.6382f,0.0942f,  -0.7643f,-0.6403f,0.0773f,  -0.8109f,-0.5844f,-0.0314f,  
	-0.8109f,-0.5833f,-0.0474f,  -0.8434f,-0.5147f,-0.1538f,  -0.8434f,-0.5102f,-0.1683f,  -0.8614f,-0.4301f,-0.2701f,  -0.8615f,-0.4221f,-0.2824f,  -0.8647f,-0.3319f,-0.3771f,  -0.8649f,-0.3204f,-0.3865f,  -0.8532f,-0.2220f,-0.4719f,  -0.8538f,-0.2071f,-0.4777f,  -0.8276f,-0.1026f,-0.5519f,  -0.8285f,-0.0849f,-0.5535f,  -0.7882f,0.0234f,-0.6149f,  -0.7898f,0.0439f,-0.6118f,  -0.7361f,0.1533f,-0.6593f,  -0.7384f,0.1760f,-0.6510f,  
	-0.6722f,0.2840f,-0.6838f,  -0.6754f,0.3080f,-0.6701f,  -0.5977f,0.4119f,-0.6878f,  -0.6019f,0.4366f,-0.6686f,  -0.5140f,0.5336f,-0.6716f,  -0.5192f,0.5584f,-0.6470f,  -0.4225f,0.6460f,-0.6358f,  -0.4285f,0.6699f,-0.6063f,  -0.3245f,0.7456f,-0.5821f,  -0.3313f,0.7680f,-0.5481f,  -0.2214f,0.8296f,-0.5126f,  -0.2289f,0.8497f,-0.4749f,  -0.1147f,0.8955f,-0.4300f,  -0.1227f,0.9127f,-0.3898f,  -0.0059f,0.9412f,-0.3378f,  
	-0.0139f,0.9551f,-0.2960f,  0.1039f,0.9653f,-0.2396f,  0.0961f,0.9755f,-0.1977f,  0.2131f,0.9670f,-0.1395f,  0.2059f,0.9736f,-0.0989f,  0.3204f,0.9464f,-0.0416f,  0.3142f,0.9493f,-0.0037f,  0.4246f,0.9040f,0.0501f,  0.4197f,0.9038f,0.0840f,  0.5243f,0.8413f,0.1317f,  0.5209f,0.8384f,0.1602f,  0.6181f,0.7603f,0.1997f,  0.6164f,0.7556f,0.2217f,  0.7046f,0.6636f,0.2513f,  0.7045f,0.6580f,0.2660f,  
	0.7822f,0.5544f,0.2843f,  0.7837f,0.5488f,0.2909f,  0.8495f,0.4360f,0.2971f,  0.8524f,0.4314f,0.2954f,  0.9050f,0.3119f,0.2892f,  0.9090f,0.3094f,0.2793f,  0.9473f,0.1857f,0.2610f,  0.9519f,0.1861f,0.2432f,  0.9750f,0.0609f,0.2137f,  0.9799f,0.0649f,0.1887f,  0.9870f,-0.0595f,0.1493f,  0.9917f,-0.0514f,0.1181f,  0.9824f,-0.1728f,0.0705f,  0.9865f,-0.1604f,0.0342f,  0.9607f,-0.2768f,-0.0194f,  
	0.9637f,-0.2601f,-0.0595f,  0.9217f,-0.3698f,-0.1168f,  0.9235f,-0.3490f,-0.1594f,  0.8657f,-0.4507f,-0.2177f,  0.8659f,-0.4264f,-0.2615f,  0.7933f,-0.5190f,-0.3182f,  0.7919f,-0.4917f,-0.3621f,  0.7057f,-0.5746f,-0.4145f,  0.7027f,-0.5450f,-0.4573f,  0.6043f,-0.6178f,-0.5031f,  0.5999f,-0.5867f,-0.5439f,  0.4912f,-0.6492f,-0.5808f,  0.4855f,-0.6176f,-0.6188f,  0.3685f,-0.6698f,-0.6447f,  0.3618f,-0.6385f,-0.6793f,  
	0.2388f,-0.6806f,-0.6926f,  0.2313f,-0.6504f,-0.7235f,  0.1047f,-0.6829f,-0.7230f,  0.0969f,-0.6546f,-0.7497f,  -0.0308f,-0.6780f,-0.7345f,  -0.0387f,-0.6522f,-0.7570f,  -0.1650f,-0.6670f,-0.7266f,  -0.1725f,-0.6444f,-0.7450f,  -0.2948f,-0.6511f,-0.6994f,  -0.3016f,-0.6322f,-0.7137f,  -0.4175f,-0.6316f,-0.6533f,  -0.4232f,-0.6167f,-0.6638f,  -0.5305f,-0.6092f,-0.5894f,  -0.5348f,-0.5986f,-0.5963f,  -0.6313f,-0.5850f,-0.5091f,  
	-0.6340f,-0.5789f,-0.5128f,  -0.7178f,-0.5595f,-0.4143f,  -0.7185f,-0.5580f,-0.4152f,  -0.7043f,-0.6805f,0.2025f,  -0.7046f,-0.6852f,0.1847f,  -0.7643f,-0.6403f,0.0773f,  -0.7644f,-0.6419f,0.0599f,  -0.8109f,-0.5833f,-0.0474f,  -0.8109f,-0.5817f,-0.0637f,  -0.8434f,-0.5102f,-0.1683f,  -0.8435f,-0.5050f,-0.1829f,  -0.8615f,-0.4221f,-0.2824f,  -0.8617f,-0.4133f,-0.2945f,  -0.8649f,-0.3204f,-0.3865f,  -0.8653f,-0.3081f,-0.3953f,  
	-0.8538f,-0.2071f,-0.4777f,  -0.8546f,-0.1917f,-0.4826f,  -0.8285f,-0.0849f,-0.5535f,  -0.8300f,-0.0663f,-0.5538f,  -0.7898f,0.0439f,-0.6118f,  -0.7920f,0.0650f,-0.6071f,  -0.7384f,0.1760f,-0.6510f,  -0.7415f,0.1989f,-0.6408f,  -0.6754f,0.3080f,-0.6701f,  -0.6794f,0.3322f,-0.6542f,  -0.6019f,0.4366f,-0.6686f,  -0.6069f,0.4614f,-0.6471f,  -0.5192f,0.5584f,-0.6470f,  -0.5253f,0.5828f,-0.6201f,  -0.4285f,0.6699f,-0.6063f,  
	-0.4356f,0.6932f,-0.5742f,  -0.3313f,0.7680f,-0.5481f,  -0.3391f,0.7894f,-0.5117f,  -0.2289f,0.8497f,-0.4749f,  -0.2374f,0.8686f,-0.4350f,  -0.1227f,0.9127f,-0.3898f,  -0.1313f,0.9285f,-0.3474f,  -0.0139f,0.9551f,-0.2960f,  -0.0225f,0.9673f,-0.2525f,  0.0961f,0.9755f,-0.1977f,  0.0878f,0.9841f,-0.1544f,  0.2059f,0.9736f,-0.0989f,  0.1984f,0.9784f,-0.0573f,  0.3142f,0.9493f,-0.0037f,  0.3079f,0.9508f,0.0348f,  
	0.4197f,0.9038f,0.0840f,  0.4148f,0.9022f,0.1179f,  0.5209f,0.8384f,0.1602f,  0.5176f,0.8347f,0.1884f,  0.6164f,0.7556f,0.2217f,  0.6147f,0.7504f,0.2431f,  0.7045f,0.6580f,0.2660f,  0.7044f,0.6523f,0.2797f,  0.7837f,0.5488f,0.2909f,  0.7852f,0.5436f,0.2966f,  0.8524f,0.4314f,0.2954f,  0.8551f,0.4278f,0.2928f,  0.9090f,0.3094f,0.2793f,  0.9126f,0.3082f,0.2687f,  0.9519f,0.1861f,0.2432f,  
	0.9560f,0.1882f,0.2250f,  0.9799f,0.0649f,0.1887f,  0.9840f,0.0708f,0.1637f,  0.9917f,-0.0514f,0.1181f,  0.9953f,-0.0413f,0.0873f,  0.9865f,-0.1604f,0.0342f,  0.9893f,-0.1459f,-0.0012f,  0.9637f,-0.2601f,-0.0595f,  0.9654f,-0.2414f,-0.0984f,  0.9235f,-0.3490f,-0.1594f,  0.9238f,-0.3264f,-0.2004f,  0.8659f,-0.4264f,-0.2615f,  0.8647f,-0.4003f,-0.3034f,  0.7919f,-0.4917f,-0.3621f,  0.7891f,-0.4629f,-0.4037f,  
	0.7027f,-0.5450f,-0.4573f,  0.6984f,-0.5143f,-0.4978f,  0.5999f,-0.5867f,-0.5439f,  0.5942f,-0.5549f,-0.5823f,  0.4855f,-0.6176f,-0.6188f,  0.4786f,-0.5854f,-0.6544f,  0.3618f,-0.6385f,-0.6793f,  0.3539f,-0.6068f,-0.7117f,  0.2313f,-0.6504f,-0.7235f,  0.2229f,-0.6201f,-0.7522f,  0.0969f,-0.6546f,-0.7497f,  0.0882f,-0.6264f,-0.7745f,  -0.0387f,-0.6522f,-0.7570f,  -0.0472f,-0.6267f,-0.7779f,  -0.1725f,-0.6444f,-0.7450f,  
	-0.1805f,-0.6220f,-0.7619f,  -0.3016f,-0.6322f,-0.7137f,  -0.3087f,-0.6135f,-0.7268f,  -0.4232f,-0.6167f,-0.6638f,  -0.4292f,-0.6021f,-0.6733f,  -0.5348f,-0.5986f,-0.5963f,  -0.5393f,-0.5884f,-0.6025f,  -0.6340f,-0.5789f,-0.5128f,  -0.6366f,-0.5731f,-0.5160f,  -0.7185f,-0.5580f,-0.4152f,  -0.7191f,-0.5568f,-0.4158f,  -0.7046f,-0.6852f,0.1847f,  -0.7049f,-0.6895f,0.1662f,  -0.7644f,-0.6419f,0.0599f,  -0.7646f,-0.6431f,0.0420f,  
	-0.8109f,-0.5817f,-0.0637f,  -0.8111f,-0.5793f,-0.0803f,  -0.8435f,-0.5050f,-0.1829f,  -0.8437f,-0.4991f,-0.1974f,  -0.8617f,-0.4133f,-0.2945f,  -0.8621f,-0.4037f,-0.3062f,  -0.8653f,-0.3081f,-0.3953f,  -0.8661f,-0.2951f,-0.4034f,  -0.8546f,-0.1917f,-0.4826f,  -0.8560f,-0.1754f,-0.4864f,  -0.8300f,-0.0663f,-0.5538f,  -0.8320f,-0.0471f,-0.5528f,  -0.7920f,0.0650f,-0.6071f,  -0.7948f,0.0864f,-0.6006f,  -0.7415f,0.1989f,-0.6408f,  
	-0.7453f,0.2221f,-0.6286f,  -0.6794f,0.3322f,-0.6542f,  -0.6843f,0.3564f,-0.6361f,  -0.6069f,0.4614f,-0.6471f,  -0.6130f,0.4856f,-0.6232f,  -0.5253f,0.5828f,-0.6201f,  -0.5323f,0.6065f,-0.5905f,  -0.4356f,0.6932f,-0.5742f,  -0.4436f,0.7155f,-0.5397f,  -0.3391f,0.7894f,-0.5117f,  -0.3480f,0.8095f,-0.4729f,  -0.2374f,0.8686f,-0.4350f,  -0.2466f,0.8859f,-0.3928f,  -0.1313f,0.9285f,-0.3474f,  -0.1408f,0.9425f,-0.3031f,  
	-0.0225f,0.9673f,-0.2525f,  -0.0318f,0.9777f,-0.2074f,  0.0878f,0.9841f,-0.1544f,  0.0792f,0.9908f,-0.1100f,  0.1984f,0.9784f,-0.0573f,  0.1908f,0.9815f,-0.0149f,  0.3079f,0.9508f,0.0348f,  0.3015f,0.9506f,0.0737f,  0.4148f,0.9022f,0.1179f,  0.4099f,0.8994f,0.1518f,  0.5176f,0.8347f,0.1884f,  0.5143f,0.8299f,0.2162f,  0.6147f,0.7504f,0.2431f,  0.6131f,0.7446f,0.2638f,  0.7044f,0.6523f,0.2797f,  
	0.7045f,0.6465f,0.2927f,  0.7852f,0.5436f,0.2966f,  0.7866f,0.5389f,0.3014f,  0.8551f,0.4278f,0.2928f,  0.8576f,0.4250f,0.2895f,  0.9126f,0.3082f,0.2687f,  0.9158f,0.3083f,0.2575f,  0.9560f,0.1882f,0.2250f,  0.9594f,0.1919f,0.2066f,  0.9840f,0.0708f,0.1637f,  0.9872f,0.0785f,0.1389f,  0.9953f,-0.0413f,0.0873f,  0.9979f,-0.0293f,0.0571f,  0.9893f,-0.1459f,-0.0012f,  0.9909f,-0.1295f,-0.0357f,  
	0.9654f,-0.2414f,-0.0984f,  0.9658f,-0.2208f,-0.1358f,  0.9238f,-0.3264f,-0.2004f,  0.9227f,-0.3021f,-0.2396f,  0.8647f,-0.4003f,-0.3034f,  0.8620f,-0.3729f,-0.3433f,  0.7891f,-0.4629f,-0.4037f,  0.7849f,-0.4330f,-0.4432f,  0.6984f,-0.5143f,-0.4978f,  0.6927f,-0.4827f,-0.5359f,  0.5942f,-0.5549f,-0.5823f,  0.5872f,-0.5224f,-0.6183f,  0.4786f,-0.5854f,-0.6544f,  0.4705f,-0.5529f,-0.6877f,  0.3539f,-0.6068f,-0.7117f,  
	0.3451f,-0.5750f,-0.7418f,  0.2229f,-0.6201f,-0.7522f,  0.2136f,-0.5898f,-0.7788f,  0.0882f,-0.6264f,-0.7745f,  0.0789f,-0.5982f,-0.7975f,  -0.0472f,-0.6267f,-0.7779f,  -0.0563f,-0.6013f,-0.7971f,  -0.1805f,-0.6220f,-0.7619f,  -0.1890f,-0.6000f,-0.7774f,  -0.3087f,-0.6135f,-0.7268f,  -0.3162f,-0.5952f,-0.7388f,  -0.4292f,-0.6021f,-0.6733f,  -0.4354f,-0.5878f,-0.6819f,  -0.5393f,-0.5884f,-0.6025f,  -0.5438f,-0.5784f,-0.6081f,  
	-0.6366f,-0.5731f,-0.5160f,  -0.6392f,-0.5676f,-0.5188f,  -0.7191f,-0.5568f,-0.4158f,  -0.7196f,-0.5558f,-0.4163f,  -0.7049f,-0.6895f,0.1662f,  -0.7052f,-0.6936f,0.1470f,  -0.7646f,-0.6431f,0.0420f,  -0.7649f,-0.6437f,0.0237f,  -0.8111f,-0.5793f,-0.0803f,  -0.8114f,-0.5764f,-0.0970f,  -0.8437f,-0.4991f,-0.1974f,  -0.8442f,-0.4925f,-0.2118f,  -0.8621f,-0.4037f,-0.3062f,  -0.8628f,-0.3936f,-0.3173f,  -0.8661f,-0.2951f,-0.4034f,  
	-0.8673f,-0.2814f,-0.4106f,  -0.8560f,-0.1754f,-0.4864f,  -0.8578f,-0.1584f,-0.4890f,  -0.8320f,-0.0471f,-0.5528f,  -0.8346f,-0.0275f,-0.5501f,  -0.7948f,0.0864f,-0.6006f,  -0.7984f,0.1081f,-0.5923f,  -0.7453f,0.2221f,-0.6286f,  -0.7500f,0.2453f,-0.6143f,  -0.6843f,0.3564f,-0.6361f,  -0.6902f,0.3802f,-0.6157f,  -0.6130f,0.4856f,-0.6232f,  -0.6200f,0.5095f,-0.5967f,  -0.5323f,0.6065f,-0.5905f,  -0.5404f,0.6294f,-0.5584f,  
	-0.4436f,0.7155f,-0.5397f,  -0.4526f,0.7366f,-0.5026f,  -0.3480f,0.8095f,-0.4729f,  -0.3576f,0.8282f,-0.4315f,  -0.2466f,0.8859f,-0.3928f,  -0.2566f,0.9015f,-0.3484f,  -0.1408f,0.9425f,-0.3031f,  -0.1508f,0.9546f,-0.2568f,  -0.0318f,0.9777f,-0.2074f,  -0.0414f,0.9861f,-0.1607f,  0.0792f,0.9908f,-0.1100f,  0.0704f,0.9954f,-0.0643f,  0.1908f,0.9815f,-0.0149f,  0.1830f,0.9827f,0.0282f,  0.3015f,0.9506f,0.0737f,  
	0.2951f,0.9488f,0.1127f,  0.4099f,0.8994f,0.1518f,  0.4051f,0.8952f,0.1856f,  0.5143f,0.8299f,0.2162f,  0.5113f,0.8242f,0.2435f,  0.6131f,0.7446f,0.2638f,  0.6117f,0.7384f,0.2839f,  0.7045f,0.6465f,0.2927f,  0.7046f,0.6408f,0.3049f,  0.7866f,0.5389f,0.3014f,  0.7880f,0.5346f,0.3055f,  0.8576f,0.4250f,0.2895f,  0.8599f,0.4232f,0.2856f,  0.9158f,0.3083f,0.2575f,  0.9185f,0.3097f,0.2459f,  
	0.9594f,0.1919f,0.2066f,  0.9622f,0.1970f,0.1881f,  0.9872f,0.0785f,0.1389f,  0.9895f,0.0879f,0.1144f,  0.9979f,-0.0293f,0.0571f,  0.9995f,-0.0155f,0.0276f,  0.9909f,-0.1295f,-0.0357f,  0.9914f,-0.1115f,-0.0689f,  0.9658f,-0.2208f,-0.1358f,  0.9649f,-0.1988f,-0.1717f,  0.9227f,-0.3021f,-0.2396f,  0.9202f,-0.2765f,-0.2769f,  0.8620f,-0.3729f,-0.3433f,  0.8581f,-0.3444f,-0.3810f,  0.7849f,-0.4330f,-0.4432f,  
	0.7794f,-0.4022f,-0.4803f,  0.6927f,-0.4827f,-0.5359f,  0.6858f,-0.4504f,-0.5717f,  0.5872f,-0.5224f,-0.6183f,  0.5791f,-0.4895f,-0.6520f,  0.4705f,-0.5529f,-0.6877f,  0.4615f,-0.5201f,-0.7187f,  0.3451f,-0.5750f,-0.7418f,  0.3354f,-0.5432f,-0.7697f,  0.2136f,-0.5898f,-0.7788f,  0.2036f,-0.5595f,-0.8034f,  0.0789f,-0.5982f,-0.7975f,  0.0688f,-0.5702f,-0.8186f,  -0.0563f,-0.6013f,-0.7971f,  -0.0660f,-0.5761f,-0.8147f,  
	-0.1890f,-0.6000f,-0.7774f,  -0.1979f,-0.5781f,-0.7916f,  -0.3162f,-0.5952f,-0.7388f,  -0.3241f,-0.5771f,-0.7496f,  -0.4354f,-0.5878f,-0.6819f,  -0.4418f,-0.5738f,-0.6897f,  -0.5438f,-0.5784f,-0.6081f,  -0.5485f,-0.5687f,-0.6130f,  -0.6392f,-0.5676f,-0.5188f,  -0.6419f,-0.5624f,-0.5213f,  -0.7196f,-0.5558f,-0.4163f,  -0.7199f,-0.5552f,-0.4165f,  -0.7052f,-0.6936f,0.1470f,  -0.7056f,-0.6971f,0.1273f,  -0.7649f,-0.6437f,0.0237f,  
	-0.7651f,-0.6438f,0.0051f,  -0.8114f,-0.5764f,-0.0970f,  -0.8118f,-0.5728f,-0.1138f,  -0.8442f,-0.4925f,-0.2118f,  -0.8447f,-0.4853f,-0.2258f,  -0.8628f,-0.3936f,-0.3173f,  -0.8638f,-0.3826f,-0.3279f,  -0.8673f,-0.2814f,-0.4106f,  -0.8689f,-0.2670f,-0.4169f,  -0.8578f,-0.1584f,-0.4890f,  -0.8601f,-0.1410f,-0.4902f,  -0.8346f,-0.0275f,-0.5501f,  -0.8379f,-0.0076f,-0.5458f,  -0.7984f,0.1081f,-0.5923f,  -0.8028f,0.1300f,-0.5818f,  
	-0.7500f,0.2453f,-0.6143f,  -0.7556f,0.2683f,-0.5976f,  -0.6902f,0.3802f,-0.6157f,  -0.6970f,0.4037f,-0.5927f,  -0.6200f,0.5095f,-0.5967f,  -0.6279f,0.5325f,-0.5676f,  -0.5404f,0.6294f,-0.5584f,  -0.5494f,0.6510f,-0.5237f,  -0.4526f,0.7366f,-0.5026f,  -0.4624f,0.7563f,-0.4628f,  -0.3576f,0.8282f,-0.4315f,  -0.3680f,0.8451f,-0.3878f,  -0.2566f,0.9015f,-0.3484f,  -0.2673f,0.9151f,-0.3019f,  -0.1508f,0.9546f,-0.2568f,  
	-0.1613f,0.9646f,-0.2088f,  -0.0414f,0.9861f,-0.1607f,  -0.0513f,0.9923f,-0.1127f,  0.0704f,0.9954f,-0.0643f,  0.0614f,0.9980f,-0.0178f,  0.1830f,0.9827f,0.0282f,  0.1753f,0.9819f,0.0717f,  0.2951f,0.9488f,0.1127f,  0.2890f,0.9452f,0.1518f,  0.4051f,0.8952f,0.1856f,  0.4006f,0.8897f,0.2190f,  0.5113f,0.8242f,0.2435f,  0.5085f,0.8176f,0.2702f,  0.6117f,0.7384f,0.2839f,  0.6105f,0.7317f,0.3031f,  
	0.7046f,0.6408f,0.3049f,  0.7048f,0.6350f,0.3162f,  0.7880f,0.5346f,0.3055f,  0.7892f,0.5308f,0.3088f,  0.8599f,0.4232f,0.2856f,  0.8618f,0.4222f,0.2811f,  0.9185f,0.3097f,0.2459f,  0.9207f,0.3122f,0.2342f,  0.9622f,0.1970f,0.1881f,  0.9642f,0.2036f,0.1698f,  0.9895f,0.0879f,0.1144f,  0.9910f,0.0988f,0.0905f,  0.9995f,-0.0155f,0.0276f,  1.0000f,-0.0002f,-0.0008f,  0.9914f,-0.1115f,-0.0689f,  
	0.9906f,-0.0920f,-0.1008f,  0.9649f,-0.1988f,-0.1717f,  0.9627f,-0.1754f,-0.2058f,  0.9202f,-0.2765f,-0.2769f,  0.9165f,-0.2498f,-0.3123f,  0.8581f,-0.3444f,-0.3810f,  0.8528f,-0.3149f,-0.4166f,  0.7794f,-0.4022f,-0.4803f,  0.7727f,-0.3708f,-0.5153f,  0.6858f,-0.4504f,-0.5717f,  0.6777f,-0.4177f,-0.6051f,  0.5791f,-0.4895f,-0.6520f,  0.5699f,-0.4563f,-0.6834f,  0.4615f,-0.5201f,-0.7187f,  0.4514f,-0.4872f,-0.7475f,  
	0.3354f,-0.5432f,-0.7697f,  0.3248f,-0.5113f,-0.7957f,  0.2036f,-0.5595f,-0.8034f,  0.1928f,-0.5294f,-0.8262f,  0.0688f,-0.5702f,-0.8186f,  0.0582f,-0.5424f,-0.8381f,  -0.0660f,-0.5761f,-0.8147f,  -0.0761f,-0.5512f,-0.8309f,  -0.1979f,-0.5781f,-0.7916f,  -0.2072f,-0.5566f,-0.8045f,  -0.3241f,-0.5771f,-0.7496f,  -0.3322f,-0.5593f,-0.7595f,  -0.4418f,-0.5738f,-0.6897f,  -0.4483f,-0.5601f,-0.6966f,  -0.5485f,-0.5687f,-0.6130f,  
	-0.5532f,-0.5593f,-0.6174f,  -0.6419f,-0.5624f,-0.5213f,  -0.6444f,-0.5575f,-0.5234f,  -0.7199f,-0.5552f,-0.4165f,  -0.7201f,-0.5548f,-0.4167f,  -0.7056f,-0.6971f,0.1273f,  -0.7058f,-0.7002f,0.1070f,  -0.7651f,-0.6438f,0.0051f,  -0.7655f,-0.6433f,-0.0139f,  -0.8118f,-0.5728f,-0.1138f,  -0.8122f,-0.5686f,-0.1305f,  -0.8447f,-0.4853f,-0.2258f,  -0.8455f,-0.4773f,-0.2395f,  -0.8638f,-0.3826f,-0.3279f,  -0.8651f,-0.3710f,-0.3376f,  
	-0.8689f,-0.2670f,-0.4169f,  -0.8709f,-0.2520f,-0.4219f,  -0.8601f,-0.1410f,-0.4902f,  -0.8630f,-0.1232f,-0.4899f,  -0.8379f,-0.0076f,-0.5458f,  -0.8419f,0.0126f,-0.5395f,  -0.8028f,0.1300f,-0.5818f,  -0.8080f,0.1518f,-0.5693f,  -0.7556f,0.2683f,-0.5976f,  -0.7620f,0.2910f,-0.5785f,  -0.6970f,0.4037f,-0.5927f,  -0.7047f,0.4264f,-0.5671f,  -0.6279f,0.5325f,-0.5676f,  -0.6368f,0.5543f,-0.5359f,  -0.5494f,0.6510f,-0.5237f,  
	-0.5593f,0.6714f,-0.4863f,  -0.4624f,0.7563f,-0.4628f,  -0.4731f,0.7741f,-0.4206f,  -0.3680f,0.8451f,-0.3878f,  -0.3791f,0.8599f,-0.3418f,  -0.2673f,0.9151f,-0.3019f,  -0.2784f,0.9264f,-0.2533f,  -0.1613f,0.9646f,-0.2088f,  -0.1721f,0.9722f,-0.1591f,  -0.0513f,0.9923f,-0.1127f,  -0.0613f,0.9961f,-0.0634f,  0.0614f,0.9980f,-0.0178f,  0.0525f,0.9982f,0.0295f,  0.1753f,0.9819f,0.0717f,  0.1678f,0.9790f,0.1155f,  
	0.2890f,0.9452f,0.1518f,  0.2831f,0.9399f,0.1908f,  0.4006f,0.8897f,0.2190f,  0.3965f,0.8828f,0.2520f,  0.5085f,0.8176f,0.2702f,  0.5060f,0.8101f,0.2962f,  0.6105f,0.7317f,0.3031f,  0.6096f,0.7246f,0.3216f,  0.7048f,0.6350f,0.3162f,  0.7051f,0.6293f,0.3268f,  0.7892f,0.5308f,0.3088f,  0.7904f,0.5275f,0.3115f,  0.8618f,0.4222f,0.2811f,  0.8635f,0.4220f,0.2762f,  0.9207f,0.3122f,0.2342f,  
	0.9224f,0.3158f,0.2223f,  0.9642f,0.2036f,0.1698f,  0.9655f,0.2114f,0.1518f,  0.9910f,0.0988f,0.0905f,  0.9915f,0.1110f,0.0673f,  1.0000f,-0.0002f,-0.0008f,  0.9995f,0.0163f,-0.0282f,  0.9906f,-0.0920f,-0.1008f,  0.9888f,-0.0713f,-0.1312f,  0.9627f,-0.1754f,-0.2058f,  0.9594f,-0.1510f,-0.2383f,  0.9165f,-0.2498f,-0.3123f,  0.9116f,-0.2222f,-0.3457f,  0.8528f,-0.3149f,-0.4166f,  0.8464f,-0.2848f,-0.4500f,  
	0.7727f,-0.3708f,-0.5153f,  0.7648f,-0.3388f,-0.5480f,  0.6777f,-0.4177f,-0.6051f,  0.6686f,-0.3847f,-0.6364f,  0.5699f,-0.4563f,-0.6834f,  0.5597f,-0.4230f,-0.7126f,  0.4514f,-0.4872f,-0.7475f,  0.4405f,-0.4544f,-0.7743f,  0.3248f,-0.5113f,-0.7957f,  0.3134f,-0.4796f,-0.8196f,  0.1928f,-0.5294f,-0.8262f,  0.1813f,-0.4995f,-0.8472f,  0.0582f,-0.5424f,-0.8381f,  0.0469f,-0.5149f,-0.8560f,  -0.0761f,-0.5512f,-0.8309f,  
	-0.0868f,-0.5266f,-0.8457f,  -0.2072f,-0.5566f,-0.8045f,  -0.2169f,-0.5354f,-0.8163f,  -0.3322f,-0.5593f,-0.7595f,  -0.3405f,-0.5419f,-0.7684f,  -0.4483f,-0.5601f,-0.6966f,  -0.4551f,-0.5467f,-0.7029f,  -0.5532f,-0.5593f,-0.6174f,  -0.5580f,-0.5502f,-0.6212f,  -0.6444f,-0.5575f,-0.5234f,  -0.6470f,-0.5529f,-0.5251f,  -0.7201f,-0.5548f,-0.4167f,  -0.7201f,-0.5548f,-0.4167f,  -0.7058f,-0.7002f,0.1070f,  -0.7061f,-0.7028f,0.0863f,  
	-0.7655f,-0.6433f,-0.0139f,  -0.7658f,-0.6423f,-0.0329f,  -0.8122f,-0.5686f,-0.1305f,  -0.8128f,-0.5637f,-0.1470f,  -0.8455f,-0.4773f,-0.2395f,  -0.8465f,-0.4686f,-0.2526f,  -0.8651f,-0.3710f,-0.3376f,  -0.8668f,-0.3588f,-0.3464f,  -0.8709f,-0.2520f,-0.4219f,  -0.8734f,-0.2367f,-0.4256f,  -0.8630f,-0.1232f,-0.4899f,  -0.8666f,-0.1051f,-0.4878f,  -0.8419f,0.0126f,-0.5395f,  -0.8466f,0.0327f,-0.5312f,  -0.8080f,0.1518f,-0.5693f,  
	-0.8140f,0.1734f,-0.5543f,  -0.7620f,0.2910f,-0.5785f,  -0.7694f,0.3131f,-0.5568f,  -0.7047f,0.4264f,-0.5671f,  -0.7133f,0.4481f,-0.5388f,  -0.6368f,0.5543f,-0.5359f,  -0.6466f,0.5749f,-0.5013f,  -0.5593f,0.6714f,-0.4863f,  -0.5701f,0.6899f,-0.4462f,  -0.4731f,0.7741f,-0.4206f,  -0.4845f,0.7899f,-0.3758f,  -0.3791f,0.8599f,-0.3418f,  -0.3908f,0.8724f,-0.2935f,  -0.2784f,0.9264f,-0.2533f,  -0.2899f,0.9353f,-0.2028f,  
	-0.1721f,0.9722f,-0.1591f,  -0.1830f,0.9772f,-0.1080f,  -0.0613f,0.9961f,-0.0634f,  -0.0713f,0.9974f,-0.0132f,  0.0525f,0.9982f,0.0295f,  0.0438f,0.9960f,0.0773f,  0.1678f,0.9790f,0.1155f,  0.1607f,0.9740f,0.1594f,  0.2831f,0.9399f,0.1908f,  0.2777f,0.9329f,0.2295f,  0.3965f,0.8828f,0.2520f,  0.3928f,0.8746f,0.2843f,  0.5060f,0.8101f,0.2962f,  0.5039f,0.8017f,0.3215f,  0.6096f,0.7246f,0.3216f,  
	0.6089f,0.7171f,0.3392f,  0.7051f,0.6293f,0.3268f,  0.7055f,0.6237f,0.3366f,  0.7904f,0.5275f,0.3115f,  0.7915f,0.5246f,0.3136f,  0.8635f,0.4220f,0.2762f,  0.8648f,0.4226f,0.2711f,  0.9224f,0.3158f,0.2223f,  0.9236f,0.3205f,0.2105f,  0.9655f,0.2114f,0.1518f,  0.9661f,0.2204f,0.1342f,  0.9915f,0.1110f,0.0673f,  0.9912f,0.1244f,0.0449f,  0.9995f,0.0163f,-0.0282f,  0.9979f,0.0340f,-0.0544f,  
	0.9888f,-0.0713f,-0.1312f,  0.9858f,-0.0496f,-0.1602f,  0.9594f,-0.1510f,-0.2383f,  0.9549f,-0.1257f,-0.2689f,  0.9116f,-0.2222f,-0.3457f,  0.9056f,-0.1939f,-0.3772f,  0.8464f,-0.2848f,-0.4500f,  0.8388f,-0.2541f,-0.4814f,  0.7648f,-0.3388f,-0.5480f,  0.7559f,-0.3066f,-0.5785f,  0.6686f,-0.3847f,-0.6364f,  0.6585f,-0.3515f,-0.6655f,  0.5597f,-0.4230f,-0.7126f,  0.5486f,-0.3896f,-0.7397f,  0.4405f,-0.4544f,-0.7743f,  
	0.4287f,-0.4215f,-0.7991f,  0.3134f,-0.4796f,-0.8196f,  0.3013f,-0.4480f,-0.8418f,  0.1813f,-0.4995f,-0.8472f,  0.1691f,-0.4697f,-0.8665f,  0.0469f,-0.5149f,-0.8560f,  0.0351f,-0.4876f,-0.8724f,  -0.0868f,-0.5266f,-0.8457f,  -0.0979f,-0.5022f,-0.8592f,  -0.2169f,-0.5354f,-0.8163f,  -0.2269f,-0.5144f,-0.8270f,  -0.3405f,-0.5419f,-0.7684f,  -0.3492f,-0.5247f,-0.7764f,  -0.4551f,-0.5467f,-0.7029f,  -0.4620f,-0.5336f,-0.7084f,  
	-0.5580f,-0.5502f,-0.6212f,  -0.5628f,-0.5414f,-0.6246f,  -0.6470f,-0.5529f,-0.5251f,  -0.6495f,-0.5485f,-0.5266f,  -0.7201f,-0.5548f,-0.4167f,  -0.7200f,-0.5550f,-0.4167f,  -0.7061f,-0.7028f,0.0863f,  -0.7063f,-0.7049f,0.0653f,  -0.7658f,-0.6423f,-0.0329f,  -0.7661f,-0.6406f,-0.0520f,  -0.8128f,-0.5637f,-0.1470f,  -0.8135f,-0.5582f,-0.1632f,  -0.8465f,-0.4686f,-0.2526f,  -0.8478f,-0.4593f,-0.2650f,  -0.8668f,-0.3588f,-0.3464f,  
	-0.8688f,-0.3461f,-0.3541f,  -0.8734f,-0.2367f,-0.4256f,  -0.8764f,-0.2209f,-0.4279f,  -0.8666f,-0.1051f,-0.4878f,  -0.8708f,-0.0869f,-0.4840f,  -0.8466f,0.0327f,-0.5312f,  -0.8521f,0.0528f,-0.5207f,  -0.8140f,0.1734f,-0.5543f,  -0.8209f,0.1945f,-0.5369f,  -0.7694f,0.3131f,-0.5568f,  -0.7776f,0.3343f,-0.5325f,  -0.7133f,0.4481f,-0.5388f,  -0.7228f,0.4687f,-0.5078f,  -0.6466f,0.5749f,-0.5013f,  -0.6572f,0.5938f,-0.4641f,  
	-0.5701f,0.6899f,-0.4462f,  -0.5815f,0.7064f,-0.4035f,  -0.4845f,0.7899f,-0.3758f,  -0.4964f,0.8034f,-0.3287f,  -0.3908f,0.8724f,-0.2935f,  -0.4027f,0.8824f,-0.2432f,  -0.2899f,0.9353f,-0.2028f,  -0.3015f,0.9415f,-0.1508f,  -0.1830f,0.9772f,-0.1080f,  -0.1938f,0.9795f,-0.0558f,  -0.0713f,0.9974f,-0.0132f,  -0.0810f,0.9960f,0.0377f,  0.0438f,0.9960f,0.0773f,  0.0355f,0.9915f,0.1253f,  0.1607f,0.9740f,0.1594f,  
	0.1541f,0.9670f,0.2031f,  0.2777f,0.9329f,0.2295f,  0.2728f,0.9241f,0.2676f,  0.3928f,0.8746f,0.2843f,  0.3896f,0.8651f,0.3159f,  0.5039f,0.8017f,0.3215f,  0.5022f,0.7926f,0.3459f,  0.6089f,0.7171f,0.3392f,  0.6084f,0.7093f,0.3559f,  0.7055f,0.6237f,0.3366f,  0.7059f,0.6182f,0.3456f,  0.7915f,0.5246f,0.3136f,  0.7924f,0.5222f,0.3152f,  0.8648f,0.4226f,0.2711f,  0.8658f,0.4240f,0.2657f,  
	0.9236f,0.3205f,0.2105f,  0.9242f,0.3260f,0.1989f,  0.9661f,0.2204f,0.1342f,  0.9660f,0.2304f,0.1172f,  0.9912f,0.1244f,0.0449f,  0.9900f,0.1388f,0.0234f,  0.9979f,0.0340f,-0.0544f,  0.9955f,0.0527f,-0.0793f,  0.9858f,-0.0496f,-0.1602f,  0.9819f,-0.0271f,-0.1875f,  0.9549f,-0.1257f,-0.2689f,  0.9494f,-0.0997f,-0.2978f,  0.9056f,-0.1939f,-0.3772f,  0.8985f,-0.1651f,-0.4067f,  0.8388f,-0.2541f,-0.4814f,  
	0.8303f,-0.2231f,-0.5108f,  0.7559f,-0.3066f,-0.5785f,  0.7460f,-0.2741f,-0.6070f,  0.6585f,-0.3515f,-0.6655f,  0.6474f,-0.3183f,-0.6925f,  0.5486f,-0.3896f,-0.7397f,  0.5367f,-0.3563f,-0.7649f,  0.4287f,-0.4215f,-0.7991f,  0.4161f,-0.3888f,-0.8220f,  0.3013f,-0.4480f,-0.8418f,  0.2884f,-0.4166f,-0.8621f,  0.1691f,-0.4697f,-0.8665f,  0.1564f,-0.4402f,-0.8842f,  0.0351f,-0.4876f,-0.8724f,  0.0227f,-0.4605f,-0.8873f,  
	-0.0979f,-0.5022f,-0.8592f,  -0.1095f,-0.4782f,-0.8714f,  -0.2269f,-0.5144f,-0.8270f,  -0.2373f,-0.4938f,-0.8366f,  -0.3492f,-0.5247f,-0.7764f,  -0.3581f,-0.5078f,-0.7835f,  -0.4620f,-0.5336f,-0.7084f,  -0.4690f,-0.5208f,-0.7133f,  -0.5628f,-0.5414f,-0.6246f,  -0.5677f,-0.5329f,-0.6275f,  -0.6495f,-0.5485f,-0.5266f,  -0.6519f,-0.5445f,-0.5278f,  -0.7200f,-0.5550f,-0.4167f,  -0.7197f,-0.5555f,-0.4165f,  -0.7063f,-0.7049f,0.0653f,  
	-0.7064f,-0.7064f,0.0439f,  -0.7661f,-0.6406f,-0.0520f,  -0.7666f,-0.6382f,-0.0711f,  -0.8135f,-0.5582f,-0.1632f,  -0.8144f,-0.5520f,-0.1790f,  -0.8478f,-0.4593f,-0.2650f,  -0.8494f,-0.4496f,-0.2765f,  -0.8688f,-0.3461f,-0.3541f,  -0.8713f,-0.3330f,-0.3605f,  -0.8764f,-0.2209f,-0.4279f,  -0.8800f,-0.2050f,-0.4285f,  -0.8708f,-0.0869f,-0.4840f,  -0.8756f,-0.0687f,-0.4781f,  -0.8521f,0.0528f,-0.5207f,  -0.8584f,0.0725f,-0.5079f,  
	-0.8209f,0.1945f,-0.5369f,  -0.8286f,0.2147f,-0.5171f,  -0.7776f,0.3343f,-0.5325f,  -0.7867f,0.3544f,-0.5055f,  -0.7228f,0.4687f,-0.5078f,  -0.7331f,0.4876f,-0.4741f,  -0.6572f,0.5938f,-0.4641f,  -0.6685f,0.6108f,-0.4242f,  -0.5815f,0.7064f,-0.4035f,  -0.5935f,0.7207f,-0.3584f,  -0.4964f,0.8034f,-0.3287f,  -0.5086f,0.8144f,-0.2794f,  -0.4027f,0.8824f,-0.2432f,  -0.4148f,0.8896f,-0.1910f,  -0.3015f,0.9415f,-0.1508f,  
	-0.3131f,0.9447f,-0.0974f,  -0.1938f,0.9795f,-0.0558f,  -0.2044f,0.9789f,-0.0026f,  -0.0810f,0.9960f,0.0377f,  -0.0903f,0.9919f,0.0890f,  0.0355f,0.9915f,0.1253f,  0.0278f,0.9845f,0.1732f,  0.1541f,0.9670f,0.2031f,  0.1481f,0.9578f,0.2463f,  0.2728f,0.9241f,0.2676f,  0.2685f,0.9137f,0.3050f,  0.3896f,0.8651f,0.3159f,  0.3869f,0.8545f,0.3466f,  0.5022f,0.7926f,0.3459f,  0.5009f,0.7828f,0.3693f,  
	0.6084f,0.7093f,0.3559f,  0.6083f,0.7013f,0.3718f,  0.7059f,0.6182f,0.3456f,  0.7064f,0.6129f,0.3540f,  0.7924f,0.5222f,0.3152f,  0.7932f,0.5203f,0.3164f,  0.8658f,0.4240f,0.2657f,  0.8665f,0.4261f,0.2602f,  0.9242f,0.3260f,0.1989f,  0.9243f,0.3324f,0.1876f,  0.9660f,0.2304f,0.1172f,  0.9652f,0.2412f,0.1008f,  0.9900f,0.1388f,0.0234f,  0.9881f,0.1540f,0.0029f,  0.9955f,0.0527f,-0.0793f,  
	0.9921f,0.0721f,-0.1029f,  0.9819f,-0.0271f,-0.1875f,  0.9770f,-0.0039f,-0.2134f,  0.9494f,-0.0997f,-0.2978f,  0.9429f,-0.0733f,-0.3250f,  0.8985f,-0.1651f,-0.4067f,  0.8905f,-0.1359f,-0.4343f,  0.8303f,-0.2231f,-0.5108f,  0.8207f,-0.1919f,-0.5381f,  0.7460f,-0.2741f,-0.6070f,  0.7351f,-0.2414f,-0.6335f,  0.6474f,-0.3183f,-0.6925f,  0.6354f,-0.2850f,-0.7176f,  0.5367f,-0.3563f,-0.7649f,  0.5239f,-0.3231f,-0.7881f,  
	0.4161f,-0.3888f,-0.8220f,  0.4028f,-0.3563f,-0.8431f,  0.2884f,-0.4166f,-0.8621f,  0.2749f,-0.3854f,-0.8809f,  0.1564f,-0.4402f,-0.8842f,  0.1430f,-0.4110f,-0.9004f,  0.0227f,-0.4605f,-0.8873f,  0.0099f,-0.4338f,-0.9010f,  -0.1095f,-0.4782f,-0.8714f,  -0.1214f,-0.4544f,-0.8825f,  -0.2373f,-0.4938f,-0.8366f,  -0.2481f,-0.4734f,-0.8452f,  -0.3581f,-0.5078f,-0.7835f,  -0.3672f,-0.4912f,-0.7899f,  -0.4690f,-0.5208f,-0.7133f,  
	-0.4762f,-0.5082f,-0.7176f,  -0.5677f,-0.5329f,-0.6275f,  -0.5726f,-0.5247f,-0.6299f,  -0.6519f,-0.5445f,-0.5278f,  -0.6542f,-0.5407f,-0.5288f,  -0.7197f,-0.5555f,-0.4165f,  -0.7192f,-0.5563f,-0.4164f,  -0.7064f,-0.7064f,0.0439f,  -0.7065f,-0.7073f,0.0223f,  -0.7666f,-0.6382f,-0.0711f,  -0.7670f,-0.6353f,-0.0899f,  -0.8144f,-0.5520f,-0.1790f,  -0.8154f,-0.5453f,-0.1942f,  -0.8494f,-0.4496f,-0.2765f,  -0.8512f,-0.4393f,-0.2871f,  
	-0.8713f,-0.3330f,-0.3605f,  -0.8742f,-0.3196f,-0.3656f,  -0.8800f,-0.2050f,-0.4285f,  -0.8841f,-0.1889f,-0.4273f,  -0.8756f,-0.0687f,-0.4781f,  -0.8811f,-0.0507f,-0.4701f,  -0.8584f,0.0725f,-0.5079f,  -0.8653f,0.0915f,-0.4928f,  -0.8286f,0.2147f,-0.5171f,  -0.8370f,0.2341f,-0.4946f,  -0.7867f,0.3544f,-0.5055f,  -0.7965f,0.3730f,-0.4759f,  -0.7331f,0.4876f,-0.4741f,  -0.7441f,0.5047f,-0.4377f,  -0.6685f,0.6108f,-0.4242f,  
	-0.6804f,0.6255f,-0.3818f,  -0.5935f,0.7207f,-0.3584f,  -0.6058f,0.7324f,-0.3108f,  -0.5086f,0.8144f,-0.2794f,  -0.5210f,0.8225f,-0.2281f,  -0.4148f,0.8896f,-0.1910f,  -0.4269f,0.8938f,-0.1374f,  -0.3131f,0.9447f,-0.0974f,  -0.3243f,0.9450f,-0.0428f,  -0.2044f,0.9789f,-0.0026f,  -0.2145f,0.9754f,0.0512f,  -0.0903f,0.9919f,0.0890f,  -0.0989f,0.9851f,0.1404f,  0.0278f,0.9845f,0.1732f,  0.0208f,0.9751f,0.2209f,  
	0.1481f,0.9578f,0.2463f,  0.1428f,0.9466f,0.2889f,  0.2685f,0.9137f,0.3050f,  0.2649f,0.9018f,0.3415f,  0.3869f,0.8545f,0.3466f,  0.3849f,0.8428f,0.3763f,  0.5009f,0.7828f,0.3693f,  0.5002f,0.7723f,0.3916f,  0.6083f,0.7013f,0.3718f,  0.6084f,0.6931f,0.3867f,  0.7064f,0.6129f,0.3540f,  0.7070f,0.6077f,0.3616f,  0.7932f,0.5203f,0.3164f,  0.7939f,0.5188f,0.3171f,  0.8665f,0.4261f,0.2602f,  
	0.8668f,0.4287f,0.2547f,  0.9243f,0.3324f,0.1876f,  0.9239f,0.3395f,0.1765f,  0.9652f,0.2412f,0.1008f,  0.9638f,0.2528f,0.0851f,  0.9881f,0.1540f,0.0029f,  0.9853f,0.1700f,-0.0166f,  0.9921f,0.0721f,-0.1029f,  0.9878f,0.0921f,-0.1253f,  0.9770f,-0.0039f,-0.2134f,  0.9711f,0.0199f,-0.2377f,  0.9429f,-0.0733f,-0.3250f,  0.9355f,-0.0464f,-0.3504f,  0.8905f,-0.1359f,-0.4343f,  0.8815f,-0.1064f,-0.4601f,  
	0.8207f,-0.1919f,-0.5381f,  0.8103f,-0.1605f,-0.5637f,  0.7351f,-0.2414f,-0.6335f,  0.7234f,-0.2088f,-0.6581f,  0.6354f,-0.2850f,-0.7176f,  0.6227f,-0.2518f,-0.7409f,  0.5239f,-0.3231f,-0.7881f,  0.5103f,-0.2899f,-0.8096f,  0.4028f,-0.3563f,-0.8431f,  0.3888f,-0.3239f,-0.8625f,  0.2749f,-0.3854f,-0.8809f,  0.2608f,-0.3544f,-0.8980f,  0.1430f,-0.4110f,-0.9004f,  0.1291f,-0.3819f,-0.9151f,  0.0099f,-0.4338f,-0.9010f,  
	-0.0035f,-0.4072f,-0.9133f,  -0.1214f,-0.4544f,-0.8825f,  -0.1338f,-0.4308f,-0.8925f,  -0.2481f,-0.4734f,-0.8452f,  -0.2591f,-0.4532f,-0.8529f,  -0.3672f,-0.4912f,-0.7899f,  -0.3766f,-0.4749f,-0.7954f,  -0.4762f,-0.5082f,-0.7176f,  -0.4835f,-0.4960f,-0.7213f,  -0.5726f,-0.5247f,-0.6299f,  -0.5775f,-0.5168f,-0.6320f,  -0.6542f,-0.5407f,-0.5288f,  -0.6564f,-0.5373f,-0.5296f,  -0.7192f,-0.5563f,-0.4164f,  -0.7184f,-0.5573f,-0.4162f,  
	-0.7065f,-0.7073f,0.0223f,  -0.7066f,-0.7076f,0.0006f,  -0.7670f,-0.6353f,-0.0899f,  -0.7675f,-0.6318f,-0.1085f,  -0.8154f,-0.5453f,-0.1942f,  -0.8166f,-0.5381f,-0.2087f,  -0.8512f,-0.4393f,-0.2871f,  -0.8534f,-0.4287f,-0.2965f,  -0.8742f,-0.3196f,-0.3656f,  -0.8775f,-0.3060f,-0.3692f,  -0.8841f,-0.1889f,-0.4273f,  -0.8888f,-0.1730f,-0.4243f,  -0.8811f,-0.0507f,-0.4701f,  -0.8873f,-0.0333f,-0.4600f,  -0.8653f,0.0915f,-0.4928f,  
	-0.8730f,0.1097f,-0.4752f,  -0.8370f,0.2341f,-0.4946f,  -0.8461f,0.2521f,-0.4696f,  -0.7965f,0.3730f,-0.4759f,  -0.8069f,0.3900f,-0.4436f,  -0.7441f,0.5047f,-0.4377f,  -0.7556f,0.5197f,-0.3988f,  -0.6804f,0.6255f,-0.3818f,  -0.6926f,0.6378f,-0.3370f,  -0.6058f,0.7324f,-0.3108f,  -0.6183f,0.7413f,-0.2612f,  -0.5210f,0.8225f,-0.2281f,  -0.5334f,0.8275f,-0.1751f,  -0.4269f,0.8938f,-0.1374f,  -0.4387f,0.8949f,-0.0824f,  
	-0.3243f,0.9450f,-0.0428f,  -0.3351f,0.9421f,0.0125f,  -0.2145f,0.9754f,0.0512f,  -0.2240f,0.9689f,0.1052f,  -0.0989f,0.9851f,0.1404f,  -0.1068f,0.9756f,0.1916f,  0.0208f,0.9751f,0.2209f,  0.0147f,0.9633f,0.2679f,  0.1428f,0.9466f,0.2889f,  0.1384f,0.9336f,0.3306f,  0.2649f,0.9018f,0.3415f,  0.2622f,0.8884f,0.3769f,  0.3849f,0.8428f,0.3763f,  0.3835f,0.8301f,0.4048f,  0.5002f,0.7723f,0.3916f,  
	0.4999f,0.7613f,0.4129f,  0.6084f,0.6931f,0.3867f,  0.6088f,0.6847f,0.4008f,  0.7070f,0.6077f,0.3616f,  0.7077f,0.6027f,0.3686f,  0.7939f,0.5188f,0.3171f,  0.7944f,0.5177f,0.3176f,  0.8668f,0.4287f,0.2547f,  0.8668f,0.4320f,0.2492f,  0.9239f,0.3395f,0.1765f,  0.9230f,0.3472f,0.1658f,  0.9638f,0.2528f,0.0851f,  0.9617f,0.2650f,0.0701f,  0.9853f,0.1700f,-0.0166f,  0.9818f,0.1865f,-0.0350f,  
	0.9878f,0.0921f,-0.1253f,  0.9828f,0.1127f,-0.1464f,  0.9711f,0.0199f,-0.2377f,  0.9645f,0.0440f,-0.2605f,  0.9355f,-0.0464f,-0.3504f,  0.9272f,-0.0192f,-0.3742f,  0.8815f,-0.1064f,-0.4601f,  0.8716f,-0.0768f,-0.4842f,  0.8103f,-0.1605f,-0.5637f,  0.7990f,-0.1290f,-0.5874f,  0.7234f,-0.2088f,-0.6581f,  0.7108f,-0.1761f,-0.6809f,  0.6227f,-0.2518f,-0.7409f,  0.6091f,-0.2186f,-0.7623f,  0.5103f,-0.2899f,-0.8096f,  
	0.4961f,-0.2569f,-0.8294f,  0.3888f,-0.3239f,-0.8625f,  0.3741f,-0.2917f,-0.8803f,  0.2608f,-0.3544f,-0.8980f,  0.2460f,-0.3236f,-0.9137f,  0.1291f,-0.3819f,-0.9151f,  0.1146f,-0.3531f,-0.9285f,  -0.0035f,-0.4072f,-0.9133f,  -0.0173f,-0.3809f,-0.9244f,  -0.1338f,-0.4308f,-0.8925f,  -0.1466f,-0.4075f,-0.9013f,  -0.2591f,-0.4532f,-0.8529f,  -0.2705f,-0.4334f,-0.8597f,  -0.3766f,-0.4749f,-0.7954f,  -0.3862f,-0.4587f,-0.8003f,  
	-0.4835f,-0.4960f,-0.7213f,  -0.4910f,-0.4839f,-0.7244f,  -0.5775f,-0.5168f,-0.6320f,  -0.5825f,-0.5091f,-0.6337f,  -0.6564f,-0.5373f,-0.5296f,  -0.6586f,-0.5340f,-0.5302f,  -0.7184f,-0.5573f,-0.4162f,  -0.7174f,-0.5587f,-0.4161f,  -0.7066f,-0.7076f,0.0006f,  -0.7065f,-0.7074f,-0.0210f,  -0.7675f,-0.6318f,-0.1085f,  -0.7681f,-0.6277f,-0.1267f,  -0.8166f,-0.5381f,-0.2087f,  -0.8180f,-0.5304f,-0.2224f,  -0.8534f,-0.4287f,-0.2965f,  
	-0.8559f,-0.4177f,-0.3048f,  -0.8775f,-0.3060f,-0.3692f,  -0.8813f,-0.2923f,-0.3712f,  -0.8888f,-0.1730f,-0.4243f,  -0.8941f,-0.1574f,-0.4193f,  -0.8873f,-0.0333f,-0.4600f,  -0.8941f,-0.0164f,-0.4476f,  -0.8730f,0.1097f,-0.4752f,  -0.8813f,0.1269f,-0.4551f,  -0.8461f,0.2521f,-0.4696f,  -0.8559f,0.2686f,-0.4420f,  -0.8069f,0.3900f,-0.4436f,  -0.8179f,0.4049f,-0.4088f,  -0.7556f,0.5197f,-0.3988f,  -0.7674f,0.5323f,-0.3573f,  
	-0.6926f,0.6378f,-0.3370f,  -0.7049f,0.6473f,-0.2899f,  -0.6183f,0.7413f,-0.2612f,  -0.6308f,0.7471f,-0.2098f,  -0.5334f,0.8275f,-0.1751f,  -0.5455f,0.8294f,-0.1206f,  -0.4387f,0.8949f,-0.0824f,  -0.4500f,0.8926f,-0.0266f,  -0.3351f,0.9421f,0.0125f,  -0.3452f,0.9360f,0.0681f,  -0.2240f,0.9689f,0.1052f,  -0.2326f,0.9595f,0.1591f,  -0.1068f,0.9756f,0.1916f,  -0.1137f,0.9635f,0.2423f,  0.0147f,0.9633f,0.2679f,  
	0.0095f,0.9494f,0.3140f,  0.1384f,0.9336f,0.3306f,  0.1350f,0.9187f,0.3711f,  0.2622f,0.8884f,0.3769f,  0.2602f,0.8737f,0.4110f,  0.3835f,0.8301f,0.4048f,  0.3828f,0.8166f,0.4320f,  0.4999f,0.7613f,0.4129f,  0.5000f,0.7500f,0.4330f,  0.6088f,0.6847f,0.4008f,  0.6094f,0.6763f,0.4139f,  0.7077f,0.6027f,0.3686f,  0.7084f,0.5979f,0.3750f,  0.7944f,0.5177f,0.3176f,  0.7948f,0.5170f,0.3177f,  
	0.8668f,0.4320f,0.2492f,  0.8665f,0.4357f,0.2438f,  0.9230f,0.3472f,0.1658f,  0.9217f,0.3555f,0.1556f,  0.9617f,0.2650f,0.0701f,  0.9590f,0.2777f,0.0558f,  0.9818f,0.1865f,-0.0350f,  0.9777f,0.2035f,-0.0525f,  0.9828f,0.1127f,-0.1464f,  0.9770f,0.1336f,-0.1662f,  0.9645f,0.0440f,-0.2605f,  0.9570f,0.0684f,-0.2819f,  0.9272f,-0.0192f,-0.3742f,  0.9180f,0.0082f,-0.3964f,  0.8716f,-0.0768f,-0.4842f,  
	0.8609f,-0.0471f,-0.5066f,  0.7990f,-0.1290f,-0.5874f,  0.7869f,-0.0975f,-0.6094f,  0.7108f,-0.1761f,-0.6809f,  0.6975f,-0.1436f,-0.7020f,  0.6091f,-0.2186f,-0.7623f,  0.5948f,-0.1856f,-0.7821f,  0.4961f,-0.2569f,-0.8294f,  0.4811f,-0.2241f,-0.8476f,  0.3741f,-0.2917f,-0.8803f,  0.3588f,-0.2597f,-0.8966f,  0.2460f,-0.3236f,-0.9137f,  0.2306f,-0.2930f,-0.9279f,  0.1146f,-0.3531f,-0.9285f,  0.0995f,-0.3245f,-0.9406f,  
	-0.0173f,-0.3809f,-0.9244f,  -0.0316f,-0.3549f,-0.9344f,  -0.1466f,-0.4075f,-0.9013f,  -0.1599f,-0.3844f,-0.9092f,  -0.2705f,-0.4334f,-0.8597f,  -0.2823f,-0.4137f,-0.8656f,  -0.3862f,-0.4587f,-0.8003f,  -0.3960f,-0.4429f,-0.8044f,  -0.4910f,-0.4839f,-0.7244f,  -0.4986f,-0.4722f,-0.7270f,  -0.5825f,-0.5091f,-0.6337f,  -0.5875f,-0.5016f,-0.6350f,  -0.6586f,-0.5340f,-0.5302f,  -0.6606f,-0.5311f,-0.5306f,  -0.7174f,-0.5587f,-0.4161f,  
	-0.7162f,-0.5604f,-0.4160f,  -0.7065f,-0.7074f,-0.0210f,  -0.7064f,-0.7065f,-0.0426f,  -0.7681f,-0.6277f,-0.1267f,  -0.7687f,-0.6231f,-0.1444f,  -0.8180f,-0.5304f,-0.2224f,  -0.8197f,-0.5223f,-0.2352f,  -0.8559f,-0.4177f,-0.3048f,  -0.8588f,-0.4066f,-0.3117f,  -0.8813f,-0.2923f,-0.3712f,  -0.8856f,-0.2789f,-0.3715f,  -0.8941f,-0.1574f,-0.4193f,  -0.8999f,-0.1423f,-0.4123f,  -0.8941f,-0.0164f,-0.4476f,  -0.9014f,-0.0005f,-0.4329f,  
	-0.8813f,0.1269f,-0.4551f,  -0.8902f,0.1426f,-0.4327f,  -0.8559f,0.2686f,-0.4420f,  -0.8661f,0.2833f,-0.4119f,  -0.8179f,0.4049f,-0.4088f,  -0.8291f,0.4177f,-0.3715f,  -0.7674f,0.5323f,-0.3573f,  -0.7795f,0.5423f,-0.3137f,  -0.7049f,0.6473f,-0.2899f,  -0.7173f,0.6538f,-0.2410f,  -0.6308f,0.7471f,-0.2098f,  -0.6429f,0.7497f,-0.1568f,  -0.5455f,0.8294f,-0.1206f,  -0.5571f,0.8279f,-0.0652f,  -0.4500f,0.8926f,-0.0266f,  
	-0.4605f,0.8871f,0.0298f,  -0.3452f,0.9360f,0.0681f,  -0.3544f,0.9269f,0.1238f,  -0.2326f,0.9595f,0.1591f,  -0.2401f,0.9472f,0.2125f,  -0.1137f,0.9635f,0.2423f,  -0.1195f,0.9489f,0.2921f,  0.0095f,0.9494f,0.3140f,  0.0054f,0.9333f,0.3590f,  0.1350f,0.9187f,0.3711f,  0.1325f,0.9023f,0.4103f,  0.2602f,0.8737f,0.4110f,  0.2591f,0.8579f,0.4438f,  0.3828f,0.8166f,0.4320f,  0.3827f,0.8024f,0.4579f,  
	0.5000f,0.7500f,0.4330f,  0.5007f,0.7382f,0.4520f,  0.6094f,0.6763f,0.4139f,  0.6103f,0.6678f,0.4260f,  0.7084f,0.5979f,0.3750f,  0.7092f,0.5933f,0.3808f,  0.7948f,0.5170f,0.3177f,  0.7951f,0.5167f,0.3176f,  0.8665f,0.4357f,0.2438f,  0.8658f,0.4399f,0.2384f,  0.9217f,0.3555f,0.1556f,  0.9198f,0.3642f,0.1457f,  0.9590f,0.2777f,0.0558f,  0.9558f,0.2909f,0.0423f,  0.9777f,0.2035f,-0.0525f,  
	0.9728f,0.2209f,-0.0690f,  0.9770f,0.1336f,-0.1662f,  0.9705f,0.1549f,-0.1848f,  0.9570f,0.0684f,-0.2819f,  0.9488f,0.0931f,-0.3019f,  0.9180f,0.0082f,-0.3964f,  0.9081f,0.0357f,-0.4171f,  0.8609f,-0.0471f,-0.5066f,  0.8494f,-0.0172f,-0.5274f,  0.7869f,-0.0975f,-0.6094f,  0.7740f,-0.0660f,-0.6297f,  0.6975f,-0.1436f,-0.7020f,  0.6834f,-0.1110f,-0.7215f,  0.5948f,-0.1856f,-0.7821f,  0.5798f,-0.1527f,-0.8003f,  
	0.4811f,-0.2241f,-0.8476f,  0.4654f,-0.1914f,-0.8642f,  0.3588f,-0.2597f,-0.8966f,  0.3427f,-0.2279f,-0.9114f,  0.2306f,-0.2930f,-0.9279f,  0.2146f,-0.2626f,-0.9407f,  0.0995f,-0.3245f,-0.9406f,  0.0839f,-0.2961f,-0.9515f,  -0.0316f,-0.3549f,-0.9344f,  -0.0465f,-0.3290f,-0.9432f,  -0.1599f,-0.3844f,-0.9092f,  -0.1735f,-0.3616f,-0.9161f,  -0.2823f,-0.4137f,-0.8656f,  -0.2943f,-0.3942f,-0.8706f,  -0.3960f,-0.4429f,-0.8044f,  
	-0.4061f,-0.4272f,-0.8078f,  -0.4986f,-0.4722f,-0.7270f,  -0.5063f,-0.4606f,-0.7290f,  -0.5875f,-0.5016f,-0.6350f,  -0.5924f,-0.4944f,-0.6360f,  -0.6606f,-0.5311f,-0.5306f,  -0.6625f,-0.5284f,-0.5309f,  -0.7162f,-0.5604f,-0.4160f,  -0.7147f,-0.5623f,-0.4160f,  -0.7064f,-0.7065f,-0.0426f,  -0.7062f,-0.7051f,-0.0640f,  -0.7687f,-0.6231f,-0.1444f,  -0.7694f,-0.6180f,-0.1615f,  -0.8197f,-0.5223f,-0.2352f,  -0.8215f,-0.5140f,-0.2469f,  
	-0.8588f,-0.4066f,-0.3117f,  -0.8619f,-0.3956f,-0.3172f,  -0.8856f,-0.2789f,-0.3715f,  -0.8902f,-0.2657f,-0.3699f,  -0.8999f,-0.1423f,-0.4123f,  -0.9061f,-0.1279f,-0.4032f,  -0.9014f,-0.0005f,-0.4329f,  -0.9093f,0.0142f,-0.4159f,  -0.8902f,0.1426f,-0.4327f,  -0.8995f,0.1568f,-0.4078f,  -0.8661f,0.2833f,-0.4119f,  -0.8767f,0.2959f,-0.3793f,  -0.8291f,0.4177f,-0.3715f,  -0.8406f,0.4279f,-0.3320f,  -0.7795f,0.5423f,-0.3137f,  
	-0.7915f,0.5493f,-0.2680f,  -0.7173f,0.6538f,-0.2410f,  -0.7293f,0.6571f,-0.1904f,  -0.6429f,0.7497f,-0.1568f,  -0.6546f,0.7489f,-0.1027f,  -0.5571f,0.8279f,-0.0652f,  -0.5679f,0.8230f,-0.0091f,  -0.4605f,0.8871f,0.0298f,  -0.4701f,0.8784f,0.0863f,  -0.3544f,0.9269f,0.1238f,  -0.3625f,0.9146f,0.1790f,  -0.2401f,0.9472f,0.2125f,  -0.2466f,0.9322f,0.2651f,  -0.1195f,0.9489f,0.2921f,  -0.1242f,0.9319f,0.3407f,  
	0.0054f,0.9333f,0.3590f,  0.0024f,0.9154f,0.4025f,  0.1325f,0.9023f,0.4103f,  0.1310f,0.8844f,0.4480f,  0.2591f,0.8579f,0.4438f,  0.2589f,0.8410f,0.4750f,  0.3827f,0.8024f,0.4579f,  0.3833f,0.7876f,0.4825f,  0.5007f,0.7382f,0.4520f,  0.5018f,0.7263f,0.4698f,  0.6103f,0.6678f,0.4260f,  0.6115f,0.6594f,0.4373f,  0.7092f,0.5933f,0.3808f,  0.7101f,0.5889f,0.3860f,  0.7951f,0.5167f,0.3176f,  
	0.7952f,0.5167f,0.3172f,  0.8658f,0.4399f,0.2384f,  0.8649f,0.4444f,0.2332f,  0.9198f,0.3642f,0.1457f,  0.9176f,0.3733f,0.1363f,  0.9558f,0.2909f,0.0423f,  0.9521f,0.3045f,0.0294f,  0.9728f,0.2209f,-0.0690f,  0.9674f,0.2387f,-0.0845f,  0.9705f,0.1549f,-0.1848f,  0.9633f,0.1764f,-0.2022f,  0.9488f,0.0931f,-0.3019f,  0.9399f,0.1179f,-0.3206f,  0.9081f,0.0357f,-0.4171f,  0.8975f,0.0633f,-0.4364f,  
	0.8494f,-0.0172f,-0.5274f,  0.8372f,0.0126f,-0.5467f,  0.7740f,-0.0660f,-0.6297f,  0.7604f,-0.0346f,-0.6486f,  0.6834f,-0.1110f,-0.7215f,  0.6686f,-0.0786f,-0.7394f,  0.5798f,-0.1527f,-0.8003f,  0.5640f,-0.1198f,-0.8170f,  0.4654f,-0.1914f,-0.8642f,  0.4490f,-0.1589f,-0.8793f,  0.3427f,-0.2279f,-0.9114f,  0.3260f,-0.1962f,-0.9248f,  0.2146f,-0.2626f,-0.9407f,  0.1980f,-0.2324f,-0.9523f,  0.0839f,-0.2961f,-0.9515f,  
	0.0677f,-0.2679f,-0.9611f,  -0.0465f,-0.3290f,-0.9432f,  -0.0618f,-0.3033f,-0.9509f,  -0.1735f,-0.3616f,-0.9161f,  -0.1876f,-0.3388f,-0.9219f,  -0.2943f,-0.3942f,-0.8706f,  -0.3068f,-0.3750f,-0.8748f,  -0.4061f,-0.4272f,-0.8078f,  -0.4165f,-0.4118f,-0.8106f,  -0.5063f,-0.4606f,-0.7290f,  -0.5142f,-0.4493f,-0.7306f,  -0.5924f,-0.4944f,-0.6360f,  -0.5974f,-0.4875f,-0.6367f,  -0.6625f,-0.5284f,-0.5309f,  -0.6643f,-0.5260f,-0.5311f,  
	-0.7147f,-0.5623f,-0.4160f,  -0.7129f,-0.5645f,-0.4160f,  -0.7062f,-0.7051f,-0.0640f,  -0.7060f,-0.7031f,-0.0851f,  -0.7694f,-0.6180f,-0.1615f,  -0.7702f,-0.6125f,-0.1779f,  -0.8215f,-0.5140f,-0.2469f,  -0.8235f,-0.5055f,-0.2576f,  -0.8619f,-0.3956f,-0.3172f,  -0.8654f,-0.3845f,-0.3212f,  -0.8902f,-0.2657f,-0.3699f,  -0.8953f,-0.2530f,-0.3666f,  -0.9061f,-0.1279f,-0.4032f,  -0.9128f,-0.1144f,-0.3920f,  -0.9093f,0.0142f,-0.4159f,  
	-0.9175f,0.0276f,-0.3967f,  -0.8995f,0.1568f,-0.4078f,  -0.9091f,0.1691f,-0.3806f,  -0.8767f,0.2959f,-0.3793f,  -0.8874f,0.3063f,-0.3446f,  -0.8406f,0.4279f,-0.3320f,  -0.8521f,0.4354f,-0.2905f,  -0.7915f,0.5493f,-0.2680f,  -0.8032f,0.5533f,-0.2207f,  -0.7293f,0.6571f,-0.1904f,  -0.7409f,0.6571f,-0.1386f,  -0.6546f,0.7489f,-0.1027f,  -0.6656f,0.7448f,-0.0478f,  -0.5679f,0.8230f,-0.0091f,  -0.5778f,0.8148f,0.0473f,  
	-0.4701f,0.8784f,0.0863f,  -0.4786f,0.8664f,0.1425f,  -0.3625f,0.9146f,0.1790f,  -0.3694f,0.8994f,0.2336f,  -0.2466f,0.9322f,0.2651f,  -0.2517f,0.9146f,0.3166f,  -0.1242f,0.9319f,0.3407f,  -0.1276f,0.9128f,0.3879f,  0.0024f,0.9154f,0.4025f,  0.0006f,0.8958f,0.4445f,  0.1310f,0.8844f,0.4480f,  0.1305f,0.8653f,0.4840f,  0.2589f,0.8410f,0.4750f,  0.2594f,0.8234f,0.5047f,  0.3833f,0.7876f,0.4825f,  
	0.3846f,0.7723f,0.5056f,  0.5018f,0.7263f,0.4698f,  0.5033f,0.7142f,0.4864f,  0.6115f,0.6594f,0.4373f,  0.6129f,0.6510f,0.4478f,  0.7101f,0.5889f,0.3860f,  0.7110f,0.5848f,0.3906f,  0.7952f,0.5167f,0.3172f,  0.7952f,0.5171f,0.3167f,  0.8649f,0.4444f,0.2332f,  0.8637f,0.4494f,0.2281f,  0.9176f,0.3733f,0.1363f,  0.9150f,0.3828f,0.1273f,  0.9521f,0.3045f,0.0294f,  0.9478f,0.3184f,0.0173f,  
	0.9674f,0.2387f,-0.0845f,  0.9614f,0.2566f,-0.0990f,  0.9633f,0.1764f,-0.2022f,  0.9555f,0.1981f,-0.2185f,  0.9399f,0.1179f,-0.3206f,  0.9303f,0.1429f,-0.3380f,  0.8975f,0.0633f,-0.4364f,  0.8862f,0.0910f,-0.4543f,  0.8372f,0.0126f,-0.5467f,  0.8243f,0.0424f,-0.5646f,  0.7604f,-0.0346f,-0.6486f,  0.7460f,-0.0032f,-0.6659f,  0.6686f,-0.0786f,-0.7394f,  0.6531f,-0.0462f,-0.7559f,  0.5640f,-0.1198f,-0.8170f,  
	0.5476f,-0.0871f,-0.8322f,  0.4490f,-0.1589f,-0.8793f,  0.4319f,-0.1264f,-0.8930f,  0.3260f,-0.1962f,-0.9248f,  0.3087f,-0.1646f,-0.9368f,  0.1980f,-0.2324f,-0.9523f,  0.1807f,-0.2022f,-0.9625f,  0.0677f,-0.2679f,-0.9611f,  0.0509f,-0.2398f,-0.9695f,  -0.0618f,-0.3033f,-0.9509f,  -0.0777f,-0.2777f,-0.9575f,  -0.1876f,-0.3388f,-0.9219f,  -0.2022f,-0.3163f,-0.9269f,  -0.3068f,-0.3750f,-0.8748f,  -0.3196f,-0.3559f,-0.8782f,  
	-0.4165f,-0.4118f,-0.8106f,  -0.4271f,-0.3965f,-0.8126f,  -0.5142f,-0.4493f,-0.7306f,  -0.5222f,-0.4382f,-0.7316f,  -0.5974f,-0.4875f,-0.6367f,  -0.6025f,-0.4808f,-0.6371f,  -0.6643f,-0.5260f,-0.5311f,  -0.6659f,-0.5239f,-0.5311f,  -0.7129f,-0.5645f,-0.4160f,  -0.7108f,-0.5670f,-0.4162f,  -0.7060f,-0.7031f,-0.0851f,  -0.7057f,-0.7005f,-0.1058f,  -0.7702f,-0.6125f,-0.1779f,  -0.7710f,-0.6067f,-0.1935f,  -0.8235f,-0.5055f,-0.2576f,  
	-0.8258f,-0.4968f,-0.2670f,  -0.8654f,-0.3845f,-0.3212f,  -0.8692f,-0.3738f,-0.3237f,  -0.8953f,-0.2530f,-0.3666f,  -0.9007f,-0.2410f,-0.3614f,  -0.9128f,-0.1144f,-0.3920f,  -0.9198f,-0.1020f,-0.3788f,  -0.9175f,0.0276f,-0.3967f,  -0.9261f,0.0395f,-0.3753f,  -0.9091f,0.1691f,-0.3806f,  -0.9189f,0.1794f,-0.3513f,  -0.8874f,0.3063f,-0.3446f,  -0.8981f,0.3141f,-0.3079f,  -0.8521f,0.4354f,-0.2905f,  -0.8633f,0.4400f,-0.2472f,  
	-0.8032f,0.5533f,-0.2207f,  -0.8145f,0.5541f,-0.1720f,  -0.7409f,0.6571f,-0.1386f,  -0.7518f,0.6538f,-0.0858f,  -0.6656f,0.7448f,-0.0478f,  -0.6755f,0.7373f,0.0074f,  -0.5778f,0.8148f,0.0473f,  -0.5865f,0.8033f,0.1034f,  -0.4786f,0.8664f,0.1425f,  -0.4859f,0.8513f,0.1980f,  -0.3694f,0.8994f,0.2336f,  -0.3749f,0.8815f,0.2870f,  -0.2517f,0.9146f,0.3166f,  -0.2555f,0.8946f,0.3666f,  -0.1276f,0.9128f,0.3879f,  
	-0.1298f,0.8918f,0.4334f,  0.0006f,0.8958f,0.4445f,  -0.0001f,0.8747f,0.4847f,  0.1305f,0.8653f,0.4840f,  0.1311f,0.8451f,0.5183f,  0.2594f,0.8234f,0.5047f,  0.2608f,0.8051f,0.5327f,  0.3846f,0.7723f,0.5056f,  0.3865f,0.7567f,0.5272f,  0.5033f,0.7142f,0.4864f,  0.5053f,0.7020f,0.5019f,  0.6129f,0.6510f,0.4478f,  0.6145f,0.6428f,0.4573f,  0.7110f,0.5848f,0.3906f,  0.7119f,0.5809f,0.3947f,  
	0.7952f,0.5171f,0.3167f,  0.7951f,0.5177f,0.3159f,  0.8637f,0.4494f,0.2281f,  0.8623f,0.4547f,0.2231f,  0.9150f,0.3828f,0.1273f,  0.9120f,0.3927f,0.1188f,  0.9478f,0.3184f,0.0173f,  0.9431f,0.3325f,0.0059f,  0.9614f,0.2566f,-0.0990f,  0.9549f,0.2749f,-0.1127f,  0.9555f,0.1981f,-0.2185f,  0.9471f,0.2199f,-0.2337f,  0.9303f,0.1429f,-0.3380f,  0.9200f,0.1679f,-0.3541f,  0.8862f,0.0910f,-0.4543f,  
	0.8742f,0.1188f,-0.4709f,  0.8243f,0.0424f,-0.5646f,  0.8107f,0.0723f,-0.5810f,  0.7460f,-0.0032f,-0.6659f,  0.7309f,0.0282f,-0.6819f,  0.6531f,-0.0462f,-0.7559f,  0.6368f,-0.0139f,-0.7709f,  0.5476f,-0.0871f,-0.8322f,  0.5304f,-0.0545f,-0.8460f,  0.4319f,-0.1264f,-0.8930f,  0.4141f,-0.0940f,-0.9054f,  0.3087f,-0.1646f,-0.9368f,  0.2906f,-0.1331f,-0.9475f,  0.1807f,-0.2022f,-0.9625f,  0.1628f,-0.1722f,-0.9715f,  
	0.0509f,-0.2398f,-0.9695f,  0.0335f,-0.2118f,-0.9767f,  -0.0777f,-0.2777f,-0.9575f,  -0.0941f,-0.2522f,-0.9631f,  -0.2022f,-0.3163f,-0.9269f,  -0.2172f,-0.2939f,-0.9308f,  -0.3196f,-0.3559f,-0.8782f,  -0.3328f,-0.3369f,-0.8808f,  -0.4271f,-0.3965f,-0.8126f,  -0.4380f,-0.3814f,-0.8140f,  -0.5222f,-0.4382f,-0.7316f,  -0.5304f,-0.4273f,-0.7322f,  -0.6025f,-0.4808f,-0.6371f,  -0.6075f,-0.4743f,-0.6372f,  -0.6659f,-0.5239f,-0.5311f,  
	-0.6674f,-0.5220f,-0.5311f,  -0.7108f,-0.5670f,-0.4162f,  -0.7084f,-0.5698f,-0.4166f,  -0.7057f,-0.7005f,-0.1058f,  -0.7054f,-0.6975f,-0.1261f,  -0.7710f,-0.6067f,-0.1935f,  -0.7720f,-0.6005f,-0.2082f,  -0.8258f,-0.4968f,-0.2670f,  -0.8282f,-0.4882f,-0.2752f,  -0.8692f,-0.3738f,-0.3237f,  -0.8732f,-0.3635f,-0.3245f,  -0.9007f,-0.2410f,-0.3614f,  -0.9064f,-0.2298f,-0.3543f,  -0.9198f,-0.1020f,-0.3788f,  -0.9271f,-0.0909f,-0.3635f,  
	-0.9261f,0.0395f,-0.3753f,  -0.9348f,0.0495f,-0.3518f,  -0.9189f,0.1794f,-0.3513f,  -0.9287f,0.1873f,-0.3200f,  -0.8981f,0.3141f,-0.3079f,  -0.9086f,0.3192f,-0.2694f,  -0.8633f,0.4400f,-0.2472f,  -0.8741f,0.4415f,-0.2025f,  -0.8145f,0.5541f,-0.1720f,  -0.8251f,0.5516f,-0.1224f,  -0.7518f,0.6538f,-0.0858f,  -0.7617f,0.6471f,-0.0327f,  -0.6755f,0.7373f,0.0074f,  -0.6844f,0.7264f,0.0625f,  -0.5865f,0.8033f,0.1034f,  
	-0.5940f,0.7886f,0.1589f,  -0.4859f,0.8513f,0.1980f,  -0.4917f,0.8334f,0.2524f,  -0.3749f,0.8815f,0.2870f,  -0.3791f,0.8611f,0.3390f,  -0.2555f,0.8946f,0.3666f,  -0.2580f,0.8725f,0.4149f,  -0.1298f,0.8918f,0.4334f,  -0.1306f,0.8691f,0.4771f,  -0.0001f,0.8747f,0.4847f,  0.0004f,0.8523f,0.5230f,  0.1311f,0.8451f,0.5183f,  0.1326f,0.8241f,0.5507f,  0.2608f,0.8051f,0.5327f,  0.2630f,0.7863f,0.5590f,  
	0.3865f,0.7567f,0.5272f,  0.3889f,0.7409f,0.5475f,  0.5053f,0.7020f,0.5019f,  0.5076f,0.6898f,0.5162f,  0.6145f,0.6428f,0.4573f,  0.6163f,0.6347f,0.4661f,  0.7119f,0.5809f,0.3947f,  0.7128f,0.5772f,0.3984f,  0.7951f,0.5177f,0.3159f,  0.7948f,0.5187f,0.3150f,  0.8623f,0.4547f,0.2231f,  0.8606f,0.4602f,0.2182f,  0.9120f,0.3927f,0.1188f,  0.9086f,0.4028f,0.1106f,  0.9431f,0.3325f,0.0059f,  
	0.9379f,0.3469f,-0.0049f,  0.9549f,0.2749f,-0.1127f,  0.9478f,0.2932f,-0.1255f,  0.9471f,0.2199f,-0.2337f,  0.9381f,0.2419f,-0.2479f,  0.9200f,0.1679f,-0.3541f,  0.9091f,0.1930f,-0.3692f,  0.8742f,0.1188f,-0.4709f,  0.8615f,0.1465f,-0.4862f,  0.8107f,0.0723f,-0.5810f,  0.7963f,0.1022f,-0.5962f,  0.7309f,0.0282f,-0.6819f,  0.7151f,0.0596f,-0.6964f,  0.6368f,-0.0139f,-0.7709f,  0.6198f,0.0184f,-0.7846f,  
	0.5304f,-0.0545f,-0.8460f,  0.5124f,-0.0219f,-0.8584f,  0.4141f,-0.0940f,-0.9054f,  0.3956f,-0.0617f,-0.9164f,  0.2906f,-0.1331f,-0.9475f,  0.2719f,-0.1017f,-0.9569f,  0.1628f,-0.1722f,-0.9715f,  0.1442f,-0.1423f,-0.9793f,  0.0335f,-0.2118f,-0.9767f,  0.0155f,-0.1839f,-0.9828f,  -0.0941f,-0.2522f,-0.9631f,  -0.1111f,-0.2269f,-0.9676f,  -0.2172f,-0.2939f,-0.9308f,  -0.2327f,-0.2716f,-0.9339f,  -0.3328f,-0.3369f,-0.8808f,  
	-0.3463f,-0.3181f,-0.8825f,  -0.4380f,-0.3814f,-0.8140f,  -0.4492f,-0.3666f,-0.8148f,  -0.5304f,-0.4273f,-0.7322f,  -0.5387f,-0.4167f,-0.7323f,  -0.6075f,-0.4743f,-0.6372f,  -0.6125f,-0.4681f,-0.6369f,  -0.6674f,-0.5220f,-0.5311f,  -0.6687f,-0.5204f,-0.5310f,  -0.7084f,-0.5698f,-0.4166f,  -0.7057f,-0.5728f,-0.4171f,  -0.7054f,-0.6975f,-0.1261f,  -0.7051f,-0.6940f,-0.1459f,  -0.7720f,-0.6005f,-0.2082f,  -0.7731f,-0.5942f,-0.2220f,  
	-0.8282f,-0.4882f,-0.2752f,  -0.8308f,-0.4797f,-0.2821f,  -0.8732f,-0.3635f,-0.3245f,  -0.8775f,-0.3537f,-0.3238f,  -0.9064f,-0.2298f,-0.3543f,  -0.9124f,-0.2196f,-0.3455f,  -0.9271f,-0.0909f,-0.3635f,  -0.9346f,-0.0813f,-0.3464f,  -0.9348f,0.0495f,-0.3518f,  -0.9434f,0.0574f,-0.3265f,  -0.9287f,0.1873f,-0.3200f,  -0.9383f,0.1929f,-0.2870f,  -0.9086f,0.3192f,-0.2694f,  -0.9187f,0.3214f,-0.2295f,  -0.8741f,0.4415f,-0.2025f,  
	-0.8842f,0.4399f,-0.1568f,  -0.8251f,0.5516f,-0.1224f,  -0.8348f,0.5459f,-0.0722f,  -0.7617f,0.6471f,-0.0327f,  -0.7705f,0.6371f,0.0205f,  -0.6844f,0.7264f,0.0625f,  -0.6920f,0.7124f,0.1171f,  -0.5940f,0.7886f,0.1589f,  -0.6001f,0.7709f,0.2135f,  -0.4917f,0.8334f,0.2524f,  -0.4961f,0.8127f,0.3055f,  -0.3791f,0.8611f,0.3390f,  -0.3817f,0.8383f,0.3893f,  -0.2580f,0.8725f,0.4149f,  -0.2590f,0.8486f,0.4613f,  
	-0.1306f,0.8691f,0.4771f,  -0.1302f,0.8449f,0.5188f,  0.0004f,0.8523f,0.5230f,  0.0020f,0.8290f,0.5593f,  0.1326f,0.8241f,0.5507f,  0.1350f,0.8024f,0.5813f,  0.2630f,0.7863f,0.5590f,  0.2659f,0.7672f,0.5837f,  0.3889f,0.7409f,0.5475f,  0.3919f,0.7250f,0.5663f,  0.5076f,0.6898f,0.5162f,  0.5102f,0.6777f,0.5295f,  0.6163f,0.6347f,0.4661f,  0.6183f,0.6269f,0.4741f,  0.7128f,0.5772f,0.3984f,  
	0.7137f,0.5739f,0.4016f,  0.7948f,0.5187f,0.3150f,  0.7944f,0.5199f,0.3140f,  0.8606f,0.4602f,0.2182f,  0.8586f,0.4661f,0.2135f,  0.9086f,0.4028f,0.1106f,  0.9048f,0.4131f,0.1029f,  0.9379f,0.3469f,-0.0049f,  0.9322f,0.3616f,-0.0151f,  0.9478f,0.2932f,-0.1255f,  0.9402f,0.3118f,-0.1374f,  0.9381f,0.2419f,-0.2479f,  0.9285f,0.2640f,-0.2611f,  0.9091f,0.1930f,-0.3692f,  0.8976f,0.2183f,-0.3831f,  
	0.8615f,0.1465f,-0.4862f,  0.8481f,0.1744f,-0.5003f,  0.7963f,0.1022f,-0.5962f,  0.7813f,0.1320f,-0.6101f,  0.7151f,0.0596f,-0.6964f,  0.6986f,0.0910f,-0.7097f,  0.6198f,0.0184f,-0.7846f,  0.6020f,0.0507f,-0.7969f,  0.5124f,-0.0219f,-0.8584f,  0.4938f,0.0107f,-0.8695f,  0.3956f,-0.0617f,-0.9164f,  0.3763f,-0.0294f,-0.9260f,  0.2719f,-0.1017f,-0.9569f,  0.2524f,-0.0703f,-0.9651f,  0.1442f,-0.1423f,-0.9793f,  
	0.1249f,-0.1124f,-0.9858f,  0.0155f,-0.1839f,-0.9828f,  -0.0031f,-0.1560f,-0.9877f,  -0.1111f,-0.2269f,-0.9676f,  -0.1287f,-0.2016f,-0.9710f,  -0.2327f,-0.2716f,-0.9339f,  -0.2488f,-0.2494f,-0.9359f,  -0.3463f,-0.3181f,-0.8825f,  -0.3604f,-0.2995f,-0.8834f,  -0.4492f,-0.3666f,-0.8148f,  -0.4607f,-0.3519f,-0.8148f,  -0.5387f,-0.4167f,-0.7323f,  -0.5472f,-0.4062f,-0.7318f,  -0.6125f,-0.4681f,-0.6369f,  -0.6176f,-0.4622f,-0.6364f,  
	-0.6687f,-0.5204f,-0.5310f,  -0.6699f,-0.5191f,-0.5309f,  -0.7057f,-0.5728f,-0.4171f,  -0.7025f,-0.5761f,-0.4178f,  -0.7051f,-0.6940f,-0.1459f,  -0.7047f,-0.6900f,-0.1651f,  -0.7731f,-0.5942f,-0.2220f,  -0.7742f,-0.5877f,-0.2349f,  -0.8308f,-0.4797f,-0.2821f,  -0.8336f,-0.4715f,-0.2877f,  -0.8775f,-0.3537f,-0.3238f,  -0.8820f,-0.3446f,-0.3215f,  -0.9124f,-0.2196f,-0.3455f,  -0.9184f,-0.2107f,-0.3349f,  -0.9346f,-0.0813f,-0.3464f,  
	-0.9420f,-0.0734f,-0.3274f,  -0.9434f,0.0574f,-0.3265f,  -0.9520f,0.0633f,-0.2995f,  -0.9383f,0.1929f,-0.2870f,  -0.9476f,0.1958f,-0.2525f,  -0.9187f,0.3214f,-0.2295f,  -0.9282f,0.3207f,-0.1885f,  -0.8842f,0.4399f,-0.1568f,  -0.8935f,0.4352f,-0.1105f,  -0.8348f,0.5459f,-0.0722f,  -0.8434f,0.5369f,-0.0219f,  -0.7705f,0.6371f,0.0205f,  -0.7781f,0.6239f,0.0733f,  -0.6920f,0.7124f,0.1171f,  -0.6981f,0.6953f,0.1709f,  
	-0.6001f,0.7709f,0.2135f,  -0.6046f,0.7505f,0.2668f,  -0.4961f,0.8127f,0.3055f,  -0.4990f,0.7897f,0.3569f,  -0.3817f,0.8383f,0.3893f,  -0.3829f,0.8136f,0.4376f,  -0.2590f,0.8486f,0.4613f,  -0.2586f,0.8231f,0.5056f,  -0.1302f,0.8449f,0.5188f,  -0.1286f,0.8196f,0.5583f,  0.0020f,0.8290f,0.5593f,  0.0047f,0.8048f,0.5936f,  0.1350f,0.8024f,0.5813f,  0.1384f,0.7803f,0.6100f,  0.2659f,0.7672f,0.5837f,  
	0.2695f,0.7478f,0.6068f,  0.3919f,0.7250f,0.5663f,  0.3954f,0.7091f,0.5838f,  0.5102f,0.6777f,0.5295f,  0.5132f,0.6657f,0.5417f,  0.6183f,0.6269f,0.4741f,  0.6204f,0.6192f,0.4813f,  0.7137f,0.5739f,0.4016f,  0.7147f,0.5707f,0.4044f,  0.7944f,0.5199f,0.3140f,  0.7939f,0.5214f,0.3128f,  0.8586f,0.4661f,0.2135f,  0.8563f,0.4722f,0.2090f,  0.9048f,0.4131f,0.1029f,  0.9007f,0.4237f,0.0956f,  
	0.9322f,0.3616f,-0.0151f,  0.9261f,0.3764f,-0.0246f,  0.9402f,0.3118f,-0.1374f,  0.9320f,0.3305f,-0.1486f,  0.9285f,0.2640f,-0.2611f,  0.9183f,0.2863f,-0.2734f,  0.8976f,0.2183f,-0.3831f,  0.8854f,0.2436f,-0.3959f,  0.8481f,0.1744f,-0.5003f,  0.8341f,0.2023f,-0.5133f,  0.7813f,0.1320f,-0.6101f,  0.7655f,0.1620f,-0.6227f,  0.6986f,0.0910f,-0.7097f,  0.6813f,0.1224f,-0.7217f,  0.6020f,0.0507f,-0.7969f,  
	0.5835f,0.0830f,-0.8078f,  0.4938f,0.0107f,-0.8695f,  0.4743f,0.0433f,-0.8793f,  0.3763f,-0.0294f,-0.9260f,  0.3563f,0.0028f,-0.9344f,  0.2524f,-0.0703f,-0.9651f,  0.2322f,-0.0389f,-0.9719f,  0.1249f,-0.1124f,-0.9858f,  0.1049f,-0.0825f,-0.9911f,  -0.0031f,-0.1560f,-0.9877f,  -0.0225f,-0.1282f,-0.9915f,  -0.1287f,-0.2016f,-0.9710f,  -0.1469f,-0.1765f,-0.9733f,  -0.2488f,-0.2494f,-0.9359f,  -0.2654f,-0.2274f,-0.9369f,  
	-0.3604f,-0.2995f,-0.8834f,  -0.3748f,-0.2811f,-0.8835f,  -0.4607f,-0.3519f,-0.8148f,  -0.4725f,-0.3374f,-0.8142f,  -0.5472f,-0.4062f,-0.7318f,  -0.5559f,-0.3960f,-0.7309f,  -0.6176f,-0.4622f,-0.6364f,  -0.6226f,-0.4565f,-0.6356f,  -0.6699f,-0.5191f,-0.5309f,  -0.6708f,-0.5180f,-0.5307f,  -0.7025f,-0.5761f,-0.4178f,  -0.6990f,-0.5796f,-0.4188f,  -0.7047f,-0.6900f,-0.1651f,  -0.7044f,-0.6857f,-0.1836f,  -0.7742f,-0.5877f,-0.2349f,  
	-0.7754f,-0.5813f,-0.2467f,  -0.8336f,-0.4715f,-0.2877f,  -0.8365f,-0.4637f,-0.2920f,  -0.8820f,-0.3446f,-0.3215f,  -0.8865f,-0.3364f,-0.3177f,  -0.9184f,-0.2107f,-0.3349f,  -0.9245f,-0.2030f,-0.3227f,  -0.9420f,-0.0734f,-0.3274f,  -0.9493f,-0.0674f,-0.3069f,  -0.9520f,0.0633f,-0.2995f,  -0.9602f,0.0668f,-0.2712f,  -0.9476f,0.1958f,-0.2525f,  -0.9563f,0.1961f,-0.2170f,  -0.9282f,0.3207f,-0.1885f,  -0.9369f,0.3172f,-0.1468f,  
	-0.8935f,0.4352f,-0.1105f,  -0.9018f,0.4274f,-0.0639f,  -0.8434f,0.5369f,-0.0219f,  -0.8508f,0.5248f,0.0282f,  -0.7781f,0.6239f,0.0733f,  -0.7842f,0.6077f,0.1253f,  -0.6981f,0.6953f,0.1709f,  -0.7028f,0.6754f,0.2234f,  -0.6046f,0.7505f,0.2668f,  -0.6076f,0.7276f,0.3184f,  -0.4990f,0.7897f,0.3569f,  -0.5003f,0.7646f,0.4064f,  -0.3829f,0.8136f,0.4376f,  -0.3826f,0.7871f,0.4838f,  -0.2586f,0.8231f,0.5056f,  
	-0.2569f,0.7962f,0.5477f,  -0.1286f,0.8196f,0.5583f,  -0.1257f,0.7933f,0.5957f,  0.0047f,0.8048f,0.5936f,  0.0084f,0.7800f,0.6258f,  0.1384f,0.7803f,0.6100f,  0.1425f,0.7577f,0.6368f,  0.2695f,0.7478f,0.6068f,  0.2738f,0.7283f,0.6282f,  0.3954f,0.7091f,0.5838f,  0.3994f,0.6932f,0.6000f,  0.5132f,0.6657f,0.5417f,  0.5165f,0.6539f,0.5528f,  0.6204f,0.6192f,0.4813f,  0.6227f,0.6118f,0.4879f,  
	0.7147f,0.5707f,0.4044f,  0.7156f,0.5679f,0.4068f,  0.7939f,0.5214f,0.3128f,  0.7932f,0.5232f,0.3116f,  0.8563f,0.4722f,0.2090f,  0.8539f,0.4786f,0.2046f,  0.9007f,0.4237f,0.0956f,  0.8963f,0.4345f,0.0887f,  0.9261f,0.3764f,-0.0246f,  0.9196f,0.3914f,-0.0335f,  0.9320f,0.3305f,-0.1486f,  0.9234f,0.3494f,-0.1590f,  0.9183f,0.2863f,-0.2734f,  0.9076f,0.3087f,-0.2847f,  0.8854f,0.2436f,-0.3959f,  
	0.8726f,0.2690f,-0.4077f,  0.8341f,0.2023f,-0.5133f,  0.8193f,0.2302f,-0.5251f,  0.7655f,0.1620f,-0.6227f,  0.7490f,0.1920f,-0.6341f,  0.6813f,0.1224f,-0.7217f,  0.6633f,0.1538f,-0.7324f,  0.5835f,0.0830f,-0.8078f,  0.5642f,0.1153f,-0.8175f,  0.4743f,0.0433f,-0.8793f,  0.4541f,0.0759f,-0.8877f,  0.3563f,0.0028f,-0.9344f,  0.3354f,0.0351f,-0.9414f,  0.2322f,-0.0389f,-0.9719f,  0.2111f,-0.0076f,-0.9774f,  
	0.1049f,-0.0825f,-0.9911f,  0.0841f,-0.0526f,-0.9951f,  -0.0225f,-0.1282f,-0.9915f,  -0.0426f,-0.1005f,-0.9940f,  -0.1469f,-0.1765f,-0.9733f,  -0.1658f,-0.1514f,-0.9745f,  -0.2654f,-0.2274f,-0.9369f,  -0.2826f,-0.2055f,-0.9370f,  -0.3748f,-0.2811f,-0.8835f,  -0.3898f,-0.2628f,-0.8826f,  -0.4725f,-0.3374f,-0.8142f,  -0.4847f,-0.3231f,-0.8128f,  -0.5559f,-0.3960f,-0.7309f,  -0.5647f,-0.3861f,-0.7294f,  -0.6226f,-0.4565f,-0.6356f,  
	-0.6277f,-0.4511f,-0.6344f,  -0.6708f,-0.5180f,-0.5307f,  -0.6716f,-0.5173f,-0.5304f,  -0.6990f,-0.5796f,-0.4188f,  -0.6951f,-0.5835f,-0.4200f,  -0.7044f,-0.6857f,-0.1836f,  -0.7040f,-0.6810f,-0.2014f,  -0.7754f,-0.5813f,-0.2467f,  -0.7767f,-0.5748f,-0.2575f,  -0.8365f,-0.4637f,-0.2920f,  -0.8395f,-0.4563f,-0.2950f,  -0.8865f,-0.3364f,-0.3177f,  -0.8911f,-0.3291f,-0.3124f,  -0.9245f,-0.2030f,-0.3227f,  -0.9305f,-0.1969f,-0.3090f,  
	-0.9493f,-0.0674f,-0.3069f,  -0.9564f,-0.0633f,-0.2851f,  -0.9602f,0.0668f,-0.2712f,  -0.9680f,0.0680f,-0.2416f,  -0.9563f,0.1961f,-0.2170f,  -0.9643f,0.1937f,-0.1806f,  -0.9369f,0.3172f,-0.1468f,  -0.9447f,0.3107f,-0.1048f,  -0.9018f,0.4274f,-0.0639f,  -0.9089f,0.4167f,-0.0175f,  -0.8508f,0.5248f,0.0282f,  -0.8568f,0.5098f,0.0775f,  -0.7842f,0.6077f,0.1253f,  -0.7889f,0.5888f,0.1762f,  -0.7028f,0.6754f,0.2234f,  
	-0.7059f,0.6530f,0.2744f,  -0.6076f,0.7276f,0.3184f,  -0.6091f,0.7025f,0.3681f,  -0.5003f,0.7646f,0.4064f,  -0.5001f,0.7376f,0.4537f,  -0.3826f,0.7871f,0.4838f,  -0.3809f,0.7592f,0.5278f,  -0.2569f,0.7962f,0.5477f,  -0.2540f,0.7683f,0.5876f,  -0.1257f,0.7933f,0.5957f,  -0.1217f,0.7663f,0.6309f,  0.0084f,0.7800f,0.6258f,  0.0131f,0.7547f,0.6559f,  0.1425f,0.7577f,0.6368f,  0.1475f,0.7350f,0.6618f,  
	0.2738f,0.7283f,0.6282f,  0.2786f,0.7088f,0.6481f,  0.3994f,0.6932f,0.6000f,  0.4037f,0.6774f,0.6149f,  0.5165f,0.6539f,0.5528f,  0.5200f,0.6423f,0.5630f,  0.6227f,0.6118f,0.4879f,  0.6250f,0.6046f,0.4938f,  0.7156f,0.5679f,0.4068f,  0.7165f,0.5653f,0.4088f,  0.7932f,0.5232f,0.3116f,  0.7924f,0.5253f,0.3102f,  0.8539f,0.4786f,0.2046f,  0.8511f,0.4852f,0.2004f,  0.8963f,0.4345f,0.0887f,  
	0.8915f,0.4455f,0.0821f,  0.9196f,0.3914f,-0.0335f,  0.9127f,0.4066f,-0.0419f,  0.9234f,0.3494f,-0.1590f,  0.9142f,0.3685f,-0.1686f,  0.9076f,0.3087f,-0.2847f,  0.8962f,0.3312f,-0.2951f,  0.8726f,0.2690f,-0.4077f,  0.8591f,0.2945f,-0.4185f,  0.8193f,0.2302f,-0.5251f,  0.8039f,0.2583f,-0.5358f,  0.7490f,0.1920f,-0.6341f,  0.7318f,0.2221f,-0.6444f,  0.6633f,0.1538f,-0.7324f,  0.6445f,0.1854f,-0.7418f,  
	0.5642f,0.1153f,-0.8175f,  0.5441f,0.1477f,-0.8259f,  0.4541f,0.0759f,-0.8877f,  0.4330f,0.1086f,-0.8948f,  0.3354f,0.0351f,-0.9414f,  0.3137f,0.0675f,-0.9471f,  0.2111f,-0.0076f,-0.9774f,  0.1892f,0.0238f,-0.9816f,  0.0841f,-0.0526f,-0.9951f,  0.0624f,-0.0228f,-0.9978f,  -0.0426f,-0.1005f,-0.9940f,  -0.0635f,-0.0728f,-0.9953f,  -0.1658f,-0.1514f,-0.9745f,  -0.1855f,-0.1264f,-0.9745f,  -0.2826f,-0.2055f,-0.9370f,  
	-0.3004f,-0.1837f,-0.9360f,  -0.3898f,-0.2628f,-0.8826f,  -0.4052f,-0.2447f,-0.8809f,  -0.4847f,-0.3231f,-0.8128f,  -0.4972f,-0.3092f,-0.8107f,  -0.5647f,-0.3861f,-0.7294f,  -0.5738f,-0.3765f,-0.7273f,  -0.6277f,-0.4511f,-0.6344f,  -0.6327f,-0.4461f,-0.6330f,  -0.6716f,-0.5173f,-0.5304f,  -0.6722f,-0.5168f,-0.5302f,  -0.6951f,-0.5835f,-0.4200f,  -0.6908f,-0.5875f,-0.4214f,  -0.7040f,-0.6810f,-0.2014f,  -0.7037f,-0.6760f,-0.2185f,  
	-0.7767f,-0.5748f,-0.2575f,  -0.7781f,-0.5684f,-0.2672f,  -0.8395f,-0.4563f,-0.2950f,  -0.8425f,-0.4496f,-0.2967f,  -0.8911f,-0.3291f,-0.3124f,  -0.8956f,-0.3229f,-0.3058f,  -0.9305f,-0.1969f,-0.3090f,  -0.9362f,-0.1923f,-0.2941f,  -0.9564f,-0.0633f,-0.2851f,  -0.9631f,-0.0612f,-0.2621f,  -0.9680f,0.0680f,-0.2416f,  -0.9751f,0.0668f,-0.2112f,  -0.9643f,0.1937f,-0.1806f,  -0.9715f,0.1886f,-0.1438f,  -0.9447f,0.3107f,-0.1048f,  
	-0.9514f,0.3014f,-0.0628f,  -0.9089f,0.4167f,-0.0175f,  -0.9147f,0.4031f,0.0285f,  -0.8568f,0.5098f,0.0775f,  -0.8614f,0.4920f,0.1259f,  -0.7889f,0.5888f,0.1762f,  -0.7920f,0.5673f,0.2256f,  -0.7059f,0.6530f,0.2744f,  -0.7074f,0.6284f,0.3235f,  -0.6091f,0.7025f,0.3681f,  -0.6090f,0.6755f,0.4157f,  -0.5001f,0.7376f,0.4537f,  -0.4984f,0.7091f,0.4987f,  -0.3809f,0.7592f,0.5278f,  -0.3779f,0.7301f,0.5694f,  
	-0.2540f,0.7683f,0.5876f,  -0.2497f,0.7395f,0.6251f,  -0.1217f,0.7663f,0.6309f,  -0.1166f,0.7387f,0.6638f,  0.0131f,0.7547f,0.6559f,  0.0186f,0.7292f,0.6841f,  0.1475f,0.7350f,0.6618f,  0.1531f,0.7122f,0.6850f,  0.2786f,0.7088f,0.6481f,  0.2840f,0.6894f,0.6664f,  0.4037f,0.6774f,0.6149f,  0.4085f,0.6619f,0.6286f,  0.5200f,0.6423f,0.5630f,  0.5238f,0.6309f,0.5723f,  0.6250f,0.6046f,0.4938f,  
	0.6275f,0.5977f,0.4990f,  0.7165f,0.5653f,0.4088f,  0.7173f,0.5630f,0.4105f,  0.7924f,0.5253f,0.3102f,  0.7914f,0.5276f,0.3088f,  0.8511f,0.4852f,0.2004f,  0.8481f,0.4921f,0.1964f,  0.8915f,0.4455f,0.0821f,  0.8863f,0.4568f,0.0759f,  0.9127f,0.4066f,-0.0419f,  0.9053f,0.4220f,-0.0497f,  0.9142f,0.3685f,-0.1686f,  0.9045f,0.3877f,-0.1775f,  0.8962f,0.3312f,-0.2951f,  0.8843f,0.3538f,-0.3047f,  
	0.8591f,0.2945f,-0.4185f,  0.8450f,0.3202f,-0.4282f,  0.8039f,0.2583f,-0.5358f,  0.7877f,0.2865f,-0.5454f,  0.7318f,0.2221f,-0.6444f,  0.7137f,0.2523f,-0.6534f,  0.6445f,0.1854f,-0.7418f,  0.6248f,0.2170f,-0.7500f,  0.5441f,0.1477f,-0.8259f,  0.5231f,0.1803f,-0.8330f,  0.4330f,0.1086f,-0.8948f,  0.4110f,0.1414f,-0.9006f,  0.3137f,0.0675f,-0.9471f,  0.2911f,0.0999f,-0.9515f,  0.1892f,0.0238f,-0.9816f,  
	0.1664f,0.0553f,-0.9845f,  0.0624f,-0.0228f,-0.9978f,  0.0399f,0.0070f,-0.9992f,  -0.0635f,-0.0728f,-0.9953f,  -0.0852f,-0.0452f,-0.9953f,  -0.1855f,-0.1264f,-0.9745f,  -0.2058f,-0.1015f,-0.9733f,  -0.3004f,-0.1837f,-0.9360f,  -0.3188f,-0.1622f,-0.9338f,  -0.4052f,-0.2447f,-0.8809f,  -0.4212f,-0.2269f,-0.8781f,  -0.4972f,-0.3092f,-0.8107f,  -0.5101f,-0.2955f,-0.8078f,  -0.5738f,-0.3765f,-0.7273f,  -0.5830f,-0.3672f,-0.7248f,  
	-0.6327f,-0.4461f,-0.6330f,  -0.6377f,-0.4414f,-0.6313f,  -0.6722f,-0.5168f,-0.5302f,  -0.6725f,-0.5167f,-0.5299f,  -0.6908f,-0.5875f,-0.4214f,  -0.6860f,-0.5918f,-0.4232f,  -0.7037f,-0.6760f,-0.2185f,  -0.7035f,-0.6708f,-0.2349f,  -0.7781f,-0.5684f,-0.2672f,  -0.7795f,-0.5623f,-0.2760f,  -0.8425f,-0.4496f,-0.2967f,  -0.8455f,-0.4435f,-0.2973f,  -0.8956f,-0.3229f,-0.3058f,  -0.9001f,-0.3180f,-0.2980f,  -0.9362f,-0.1923f,-0.2941f,  
	-0.9417f,-0.1893f,-0.2780f,  -0.9631f,-0.0612f,-0.2621f,  -0.9693f,-0.0611f,-0.2382f,  -0.9751f,0.0668f,-0.2112f,  -0.9816f,0.0633f,-0.1803f,  -0.9715f,0.1886f,-0.1438f,  -0.9777f,0.1810f,-0.1069f,  -0.9514f,0.3014f,-0.0628f,  -0.9570f,0.2894f,-0.0211f,  -0.9147f,0.4031f,0.0285f,  -0.9192f,0.3868f,0.0735f,  -0.8614f,0.4920f,0.1259f,  -0.8646f,0.4718f,0.1730f,  -0.7920f,0.5673f,0.2256f,  -0.7936f,0.5435f,0.2734f,  
	-0.7074f,0.6284f,0.3235f,  -0.7074f,0.6018f,0.3707f,  -0.6090f,0.6755f,0.4157f,  -0.6073f,0.6469f,0.4611f,  -0.4984f,0.7091f,0.4987f,  -0.4953f,0.6793f,0.5415f,  -0.3779f,0.7301f,0.5694f,  -0.3735f,0.7000f,0.6087f,  -0.2497f,0.7395f,0.6251f,  -0.2443f,0.7101f,0.6604f,  -0.1166f,0.7387f,0.6638f,  -0.1106f,0.7108f,0.6947f,  0.0186f,0.7292f,0.6841f,  0.0250f,0.7035f,0.7103f,  0.1531f,0.7122f,0.6850f,  
	0.1594f,0.6895f,0.7065f,  0.2840f,0.6894f,0.6664f,  0.2899f,0.6701f,0.6833f,  0.4085f,0.6619f,0.6286f,  0.4136f,0.6465f,0.6411f,  0.5238f,0.6309f,0.5723f,  0.5277f,0.6198f,0.5808f,  0.6275f,0.5977f,0.4990f,  0.6300f,0.5911f,0.5037f,  0.7173f,0.5630f,0.4105f,  0.7181f,0.5610f,0.4119f,  0.7914f,0.5276f,0.3088f,  0.7902f,0.5302f,0.3073f,  0.8481f,0.4921f,0.1964f,  0.8448f,0.4992f,0.1925f,  
	0.8863f,0.4568f,0.0759f,  0.8808f,0.4683f,0.0702f,  0.9053f,0.4220f,-0.0497f,  0.8974f,0.4376f,-0.0569f,  0.9045f,0.3877f,-0.1775f,  0.8943f,0.4071f,-0.1857f,  0.8843f,0.3538f,-0.3047f,  0.8717f,0.3767f,-0.3134f,  0.8450f,0.3202f,-0.4282f,  0.8302f,0.3460f,-0.4370f,  0.7877f,0.2865f,-0.5454f,  0.7708f,0.3148f,-0.5538f,  0.7137f,0.2523f,-0.6534f,  0.6949f,0.2826f,-0.6613f,  0.6248f,0.2170f,-0.7500f,  
	0.6043f,0.2488f,-0.7569f,  0.5231f,0.1803f,-0.8330f,  0.5012f,0.2129f,-0.8387f,  0.4110f,0.1414f,-0.9006f,  0.3880f,0.1743f,-0.9050f,  0.2911f,0.0999f,-0.9515f,  0.2675f,0.1324f,-0.9544f,  0.1664f,0.0553f,-0.9845f,  0.1427f,0.0867f,-0.9860f,  0.0399f,0.0070f,-0.9992f,  0.0165f,0.0368f,-0.9992f,  -0.0852f,-0.0452f,-0.9953f,  -0.1078f,-0.0176f,-0.9940f,  -0.2058f,-0.1015f,-0.9733f,  -0.2270f,-0.0768f,-0.9709f,  
	-0.3188f,-0.1622f,-0.9338f,  -0.3379f,-0.1408f,-0.9306f,  -0.4212f,-0.2269f,-0.8781f,  -0.4376f,-0.2094f,-0.8744f,  -0.5101f,-0.2955f,-0.8078f,  -0.5233f,-0.2821f,-0.8041f,  -0.5830f,-0.3672f,-0.7248f,  -0.5924f,-0.3584f,-0.7216f,  -0.6377f,-0.4414f,-0.6313f,  -0.6427f,-0.4370f,-0.6293f,  -0.6725f,-0.5167f,-0.5299f,  -0.6726f,-0.5169f,-0.5296f,  -0.6860f,-0.5918f,-0.4232f,  -0.6808f,-0.5963f,-0.4253f,  -0.7035f,-0.6708f,-0.2349f,  
	-0.7032f,-0.6654f,-0.2504f,  -0.7795f,-0.5623f,-0.2760f,  -0.7810f,-0.5564f,-0.2837f,  -0.8455f,-0.4435f,-0.2973f,  -0.8485f,-0.4382f,-0.2968f,  -0.9001f,-0.3180f,-0.2980f,  -0.9043f,-0.3142f,-0.2891f,  -0.9417f,-0.1893f,-0.2780f,  -0.9468f,-0.1880f,-0.2610f,  -0.9693f,-0.0611f,-0.2382f,  -0.9749f,-0.0631f,-0.2137f,  -0.9816f,0.0633f,-0.1803f,  -0.9871f,0.0574f,-0.1492f,  -0.9777f,0.1810f,-0.1069f,  -0.9828f,0.1709f,-0.0703f,  
	-0.9570f,0.2894f,-0.0211f,  -0.9612f,0.2750f,0.0198f,  -0.9192f,0.3868f,0.0735f,  -0.9223f,0.3682f,0.1174f,  -0.8646f,0.4718f,0.1730f,  -0.8662f,0.4494f,0.2185f,  -0.7936f,0.5435f,0.2734f,  -0.7937f,0.5179f,0.3192f,  -0.7074f,0.6018f,0.3707f,  -0.7059f,0.5736f,0.4156f,  -0.6073f,0.6469f,0.4611f,  -0.6043f,0.6170f,0.5042f,  -0.4953f,0.6793f,0.5415f,  -0.4908f,0.6486f,0.5818f,  -0.3735f,0.7000f,0.6087f,  
	-0.3679f,0.6693f,0.6455f,  -0.2443f,0.7101f,0.6604f,  -0.2379f,0.6802f,0.6933f,  -0.1106f,0.7108f,0.6947f,  -0.1036f,0.6827f,0.7234f,  0.0250f,0.7035f,0.7103f,  0.0321f,0.6778f,0.7346f,  0.1594f,0.6895f,0.7065f,  0.1664f,0.6668f,0.7264f,  0.2899f,0.6701f,0.6833f,  0.2962f,0.6510f,0.6989f,  0.4136f,0.6465f,0.6411f,  0.4189f,0.6314f,0.6526f,  0.5277f,0.6198f,0.5808f,  0.5319f,0.6090f,0.5884f,  
	0.6300f,0.5911f,0.5037f,  0.6325f,0.5847f,0.5079f,  0.7181f,0.5610f,0.4119f,  0.7188f,0.5592f,0.4130f,  0.7902f,0.5302f,0.3073f,  0.7889f,0.5330f,0.3058f,  0.8448f,0.4992f,0.1925f,  0.8413f,0.5065f,0.1888f,  0.8808f,0.4683f,0.0702f,  0.8749f,0.4800f,0.0647f,  0.8974f,0.4376f,-0.0569f,  0.8891f,0.4534f,-0.0636f,  0.8943f,0.4071f,-0.1857f,  0.8835f,0.4267f,-0.1932f,  0.8717f,0.3767f,-0.3134f,  
	0.8585f,0.3997f,-0.3212f,  0.8302f,0.3460f,-0.4370f,  0.8147f,0.3720f,-0.4448f,  0.7708f,0.3148f,-0.5538f,  0.7531f,0.3434f,-0.5612f,  0.6949f,0.2826f,-0.6613f,  0.6752f,0.3131f,-0.6679f,  0.6043f,0.2488f,-0.7569f,  0.5829f,0.2808f,-0.7625f,  0.5012f,0.2129f,-0.8387f,  0.4783f,0.2457f,-0.8431f,  0.3880f,0.1743f,-0.9050f,  0.3641f,0.2073f,-0.9080f,  0.2675f,0.1324f,-0.9544f,  0.2430f,0.1650f,-0.9559f,  
	0.1427f,0.0867f,-0.9860f,  0.1179f,0.1182f,-0.9860f,  0.0165f,0.0368f,-0.9992f,  -0.0079f,0.0666f,-0.9977f,  -0.1078f,-0.0176f,-0.9940f,  -0.1313f,0.0098f,-0.9913f,  -0.2270f,-0.0768f,-0.9709f,  -0.2489f,-0.0523f,-0.9671f,  -0.3379f,-0.1408f,-0.9306f,  -0.3577f,-0.1198f,-0.9261f,  -0.4376f,-0.2094f,-0.8744f,  -0.4546f,-0.1922f,-0.8697f,  -0.5233f,-0.2821f,-0.8041f,  -0.5369f,-0.2692f,-0.7996f,  -0.5924f,-0.3584f,-0.7216f,  
	-0.6019f,-0.3499f,-0.7178f,  -0.6427f,-0.4370f,-0.6293f,  -0.6476f,-0.4331f,-0.6269f,  -0.6726f,-0.5169f,-0.5296f,  -0.6724f,-0.5174f,-0.5293f,  -0.6808f,-0.5963f,-0.4253f,  -0.6751f,-0.6011f,-0.4277f,  -0.7032f,-0.6654f,-0.2504f,  -0.7031f,-0.6598f,-0.2652f,  -0.7810f,-0.5564f,-0.2837f,  -0.7824f,-0.5509f,-0.2905f,  -0.8485f,-0.4382f,-0.2968f,  -0.8513f,-0.4337f,-0.2953f,  -0.9043f,-0.3142f,-0.2891f,  -0.9082f,-0.3117f,-0.2794f,  
	-0.9468f,-0.1880f,-0.2610f,  -0.9514f,-0.1884f,-0.2434f,  -0.9749f,-0.0631f,-0.2137f,  -0.9797f,-0.0671f,-0.1889f,  -0.9871f,0.0574f,-0.1492f,  -0.9918f,0.0494f,-0.1182f,  -0.9828f,0.1709f,-0.0703f,  -0.9868f,0.1585f,-0.0341f,  -0.9612f,0.2750f,0.0198f,  -0.9642f,0.2584f,0.0598f,  -0.9223f,0.3682f,0.1174f,  -0.9240f,0.3475f,0.1599f,  -0.8662f,0.4494f,0.2185f,  -0.8664f,0.4250f,0.2622f,  -0.7937f,0.5179f,0.3192f,  
	-0.7923f,0.4905f,0.3629f,  -0.7059f,0.5736f,0.4156f,  -0.7028f,0.5441f,0.4583f,  -0.6043f,0.6170f,0.5042f,  -0.5998f,0.5860f,0.5449f,  -0.4908f,0.6486f,0.5818f,  -0.4851f,0.6170f,0.6197f,  -0.3679f,0.6693f,0.6455f,  -0.3611f,0.6380f,0.6801f,  -0.2379f,0.6802f,0.6933f,  -0.2304f,0.6501f,0.7240f,  -0.1036f,0.6827f,0.7234f,  -0.0958f,0.6544f,0.7500f,  0.0321f,0.6778f,0.7346f,  0.0400f,0.6521f,0.7571f,  
	0.1664f,0.6668f,0.7264f,  0.1738f,0.6443f,0.7448f,  0.2962f,0.6510f,0.6989f,  0.3030f,0.6321f,0.7132f,  0.4189f,0.6314f,0.6526f,  0.4246f,0.6165f,0.6630f,  0.5319f,0.6090f,0.5884f,  0.5361f,0.5985f,0.5953f,  0.6325f,0.5847f,0.5079f,  0.6351f,0.5787f,0.5116f,  0.7188f,0.5592f,0.4130f,  0.7195f,0.5578f,0.4138f,  0.7889f,0.5330f,0.3058f,  0.7874f,0.5362f,0.3042f,  0.8413f,0.5065f,0.1888f,  
	0.8375f,0.5141f,0.1853f,  0.8749f,0.4800f,0.0647f,  0.8686f,0.4919f,0.0597f,  0.8891f,0.4534f,-0.0636f,  0.8802f,0.4694f,-0.0697f,  0.8835f,0.4267f,-0.1932f,  0.8722f,0.4465f,-0.1999f,  0.8585f,0.3997f,-0.3212f,  0.8447f,0.4229f,-0.3282f,  0.8147f,0.3720f,-0.4448f,  0.7984f,0.3983f,-0.4516f,  0.7531f,0.3434f,-0.5612f,  0.7345f,0.3721f,-0.5675f,  0.6752f,0.3131f,-0.6679f,  0.6546f,0.3438f,-0.6733f,  
	0.5829f,0.2808f,-0.7625f,  0.5604f,0.3129f,-0.7668f,  0.4783f,0.2457f,-0.8431f,  0.4544f,0.2786f,-0.8461f,  0.3641f,0.2073f,-0.9080f,  0.3391f,0.2404f,-0.9095f,  0.2430f,0.1650f,-0.9559f,  0.2173f,0.1976f,-0.9559f,  0.1179f,0.1182f,-0.9860f,  0.0921f,0.1497f,-0.9844f,  -0.0079f,0.0666f,-0.9977f,  -0.0333f,0.0963f,-0.9948f,  -0.1313f,0.0098f,-0.9913f,  -0.1557f,0.0371f,-0.9871f,  -0.2489f,-0.0523f,-0.9671f,  
	-0.2717f,-0.0281f,-0.9620f,  -0.3577f,-0.1198f,-0.9261f,  -0.3782f,-0.0991f,-0.9204f,  -0.4546f,-0.1922f,-0.8697f,  -0.4722f,-0.1755f,-0.8638f,  -0.5369f,-0.2692f,-0.7996f,  -0.5508f,-0.2568f,-0.7942f,  -0.6019f,-0.3499f,-0.7178f,  -0.6115f,-0.3420f,-0.7135f,  -0.6476f,-0.4331f,-0.6269f,  -0.6524f,-0.4297f,-0.6243f,  -0.6724f,-0.5174f,-0.5293f,  -0.6718f,-0.5184f,-0.5291f,  -0.6751f,-0.6011f,-0.4277f,  -0.6689f,-0.6060f,-0.4305f,  
	-0.7031f,-0.6598f,-0.2652f,  -0.7030f,-0.6542f,-0.2791f,  -0.7824f,-0.5509f,-0.2905f,  -0.7838f,-0.5457f,-0.2963f,  -0.8513f,-0.4337f,-0.2953f,  -0.8540f,-0.4300f,-0.2929f,  -0.9082f,-0.3117f,-0.2794f,  -0.9118f,-0.3105f,-0.2689f,  -0.9514f,-0.1884f,-0.2434f,  -0.9555f,-0.1904f,-0.2253f,  -0.9797f,-0.0671f,-0.1889f,  -0.9838f,-0.0730f,-0.1640f,  -0.9918f,0.0494f,-0.1182f,  -0.9954f,0.0393f,-0.0875f,  -0.9868f,0.1585f,-0.0341f,  
	-0.9896f,0.1441f,0.0012f,  -0.9642f,0.2584f,0.0598f,  -0.9658f,0.2397f,0.0986f,  -0.9240f,0.3475f,0.1599f,  -0.9242f,0.3249f,0.2008f,  -0.8664f,0.4250f,0.2622f,  -0.8651f,0.3990f,0.3040f,  -0.7923f,0.4905f,0.3629f,  -0.7894f,0.4618f,0.4045f,  -0.7028f,0.5441f,0.4583f,  -0.6984f,0.5134f,0.4986f,  -0.5998f,0.5860f,0.5449f,  -0.5940f,0.5542f,0.5831f,  -0.4851f,0.6170f,0.6197f,  -0.4782f,0.5849f,0.6552f,  
	-0.3611f,0.6380f,0.6801f,  -0.3533f,0.6065f,0.7123f,  -0.2304f,0.6501f,0.7240f,  -0.2220f,0.6199f,0.7526f,  -0.0958f,0.6544f,0.7500f,  -0.0871f,0.6262f,0.7748f,  0.0400f,0.6521f,0.7571f,  0.0484f,0.6266f,0.7779f,  0.1738f,0.6443f,0.7448f,  0.1818f,0.6220f,0.7616f,  0.3030f,0.6321f,0.7132f,  0.3101f,0.6135f,0.7263f,  0.4246f,0.6165f,0.6630f,  0.4305f,0.6020f,0.6725f,  0.5361f,0.5985f,0.5953f,  
	0.5405f,0.5883f,0.6015f,  0.6351f,0.5787f,0.5116f,  0.6377f,0.5730f,0.5148f,  0.7195f,0.5578f,0.4138f,  0.7200f,0.5566f,0.4145f,  0.7874f,0.5362f,0.3042f,  0.7857f,0.5395f,0.3027f,  0.8375f,0.5141f,0.1853f,  0.8333f,0.5220f,0.1820f,  0.8686f,0.4919f,0.0597f,  0.8619f,0.5041f,0.0551f,  0.8802f,0.4694f,-0.0697f,  0.8709f,0.4856f,-0.0752f,  0.8722f,0.4465f,-0.1999f,  0.8602f,0.4665f,-0.2059f,  
	0.8447f,0.4229f,-0.3282f,  0.8301f,0.4463f,-0.3342f,  0.7984f,0.3983f,-0.4516f,  0.7813f,0.4247f,-0.4573f,  0.7345f,0.3721f,-0.5675f,  0.7151f,0.4010f,-0.5726f,  0.6546f,0.3438f,-0.6733f,  0.6330f,0.3747f,-0.6774f,  0.5604f,0.3129f,-0.7668f,  0.5370f,0.3452f,-0.7697f,  0.4544f,0.2786f,-0.8461f,  0.4294f,0.3117f,-0.8476f,  0.3391f,0.2404f,-0.9095f,  0.3130f,0.2736f,-0.9095f,  0.2173f,0.1976f,-0.9559f,  
	0.1905f,0.2302f,-0.9543f,  0.0921f,0.1497f,-0.9844f,  0.0652f,0.1811f,-0.9813f,  -0.0333f,0.0963f,-0.9948f,  -0.0598f,0.1258f,-0.9903f,  -0.1557f,0.0371f,-0.9871f,  -0.1811f,0.0641f,-0.9814f,  -0.2717f,-0.0281f,-0.9620f,  -0.2954f,-0.0042f,-0.9554f,  -0.3782f,-0.0991f,-0.9204f,  -0.3995f,-0.0789f,-0.9134f,  -0.4722f,-0.1755f,-0.8638f,  -0.4903f,-0.1594f,-0.8569f,  -0.5508f,-0.2568f,-0.7942f,  -0.5650f,-0.2450f,-0.7879f,  
	-0.6115f,-0.3420f,-0.7135f,  -0.6213f,-0.3347f,-0.7085f,  -0.6524f,-0.4297f,-0.6243f,  -0.6571f,-0.4269f,-0.6213f,  -0.6718f,-0.5184f,-0.5291f,  -0.6710f,-0.5197f,-0.5289f,  -0.6689f,-0.6060f,-0.4305f,  -0.6621f,-0.6111f,-0.4337f,  -0.7030f,-0.6542f,-0.2791f,  -0.7030f,-0.6484f,-0.2923f,  -0.7838f,-0.5457f,-0.2963f,  -0.7852f,-0.5409f,-0.3013f,  -0.8540f,-0.4300f,-0.2929f,  -0.8565f,-0.4272f,-0.2897f,  -0.9118f,-0.3105f,-0.2689f,  
	-0.9149f,-0.3105f,-0.2578f,  -0.9555f,-0.1904f,-0.2253f,  -0.9589f,-0.1941f,-0.2070f,  -0.9838f,-0.0730f,-0.1640f,  -0.9870f,-0.0806f,-0.1393f,  -0.9954f,0.0393f,-0.0875f,  -0.9980f,0.0273f,-0.0573f,  -0.9896f,0.1441f,0.0012f,  -0.9912f,0.1277f,0.0356f,  -0.9658f,0.2397f,0.0986f,  -0.9662f,0.2192f,0.1359f,  -0.9242f,0.3249f,0.2008f,  -0.9231f,0.3007f,0.2399f,  -0.8651f,0.3990f,0.3040f,  -0.8624f,0.3717f,0.3437f,  
	-0.7894f,0.4618f,0.4045f,  -0.7851f,0.4320f,0.4438f,  -0.6984f,0.5134f,0.4986f,  -0.6927f,0.4818f,0.5366f,  -0.5940f,0.5542f,0.5831f,  -0.5870f,0.5217f,0.6190f,  -0.4782f,0.5849f,0.6552f,  -0.4701f,0.5524f,0.6884f,  -0.3533f,0.6065f,0.7123f,  -0.3445f,0.5747f,0.7423f,  -0.2220f,0.6199f,0.7526f,  -0.2128f,0.5896f,0.7792f,  -0.0871f,0.6262f,0.7748f,  -0.0778f,0.5981f,0.7976f,  0.0484f,0.6266f,0.7779f,  
	0.0575f,0.6012f,0.7970f,  0.1818f,0.6220f,0.7616f,  0.1903f,0.5999f,0.7771f,  0.3101f,0.6135f,0.7263f,  0.3176f,0.5952f,0.7382f,  0.4305f,0.6020f,0.6725f,  0.4367f,0.5877f,0.6811f,  0.5405f,0.5883f,0.6015f,  0.5451f,0.5783f,0.6070f,  0.6377f,0.5730f,0.5148f,  0.6403f,0.5675f,0.5176f,  0.7200f,0.5566f,0.4145f,  0.7205f,0.5557f,0.4149f,  0.7857f,0.5395f,0.3027f,  0.7837f,0.5432f,0.3011f,  
	0.8333f,0.5220f,0.1820f,  0.8288f,0.5301f,0.1789f,  0.8619f,0.5041f,0.0551f,  0.8548f,0.5165f,0.0509f,  0.8709f,0.4856f,-0.0752f,  0.8611f,0.5022f,-0.0801f,  0.8602f,0.4665f,-0.2059f,  0.8476f,0.4868f,-0.2112f,  0.8301f,0.4463f,-0.3342f,  0.8148f,0.4701f,-0.3394f,  0.7813f,0.4247f,-0.4573f,  0.7634f,0.4514f,-0.4620f,  0.7151f,0.4010f,-0.5726f,  0.6947f,0.4302f,-0.5764f,  0.6330f,0.3747f,-0.6774f,  
	0.6104f,0.4059f,-0.6802f,  0.5370f,0.3452f,-0.7697f,  0.5125f,0.3777f,-0.7712f,  0.4294f,0.3117f,-0.8476f,  0.4033f,0.3449f,-0.8476f,  0.3130f,0.2736f,-0.9095f,  0.2857f,0.3068f,-0.9079f,  0.1905f,0.2302f,-0.9543f,  0.1626f,0.2628f,-0.9510f,  0.0652f,0.1811f,-0.9813f,  0.0371f,0.2124f,-0.9765f,  -0.0598f,0.1258f,-0.9903f,  -0.0873f,0.1551f,-0.9840f,  -0.1811f,0.0641f,-0.9814f,  -0.2075f,0.0907f,-0.9740f,  
	-0.2954f,-0.0042f,-0.9554f,  -0.3199f,0.0192f,-0.9473f,  -0.3995f,-0.0789f,-0.9134f,  -0.4214f,-0.0592f,-0.9050f,  -0.4903f,-0.1594f,-0.8569f,  -0.5088f,-0.1439f,-0.8488f,  -0.5650f,-0.2450f,-0.7879f,  -0.5795f,-0.2339f,-0.7807f,  -0.6213f,-0.3347f,-0.7085f,  -0.6311f,-0.3280f,-0.7030f,  -0.6571f,-0.4269f,-0.6213f,  -0.6616f,-0.4246f,-0.6181f,  -0.6710f,-0.5197f,-0.5289f,  -0.6697f,-0.5215f,-0.5287f,  -0.6621f,-0.6111f,-0.4337f,  
	-0.6548f,-0.6164f,-0.4373f,  -0.7030f,-0.6484f,-0.2923f,  -0.7030f,-0.6426f,-0.3046f,  -0.7852f,-0.5409f,-0.3013f,  -0.7866f,-0.5366f,-0.3055f,  -0.8565f,-0.4272f,-0.2897f,  -0.8587f,-0.4253f,-0.2859f,  -0.9149f,-0.3105f,-0.2578f,  -0.9176f,-0.3118f,-0.2464f,  -0.9589f,-0.1941f,-0.2070f,  -0.9616f,-0.1992f,-0.1887f,  -0.9870f,-0.0806f,-0.1393f,  -0.9893f,-0.0899f,-0.1149f,  -0.9980f,0.0273f,-0.0573f,  -0.9995f,0.0136f,-0.0280f,  
	-0.9912f,0.1277f,0.0356f,  -0.9916f,0.1097f,0.0687f,  -0.9662f,0.2192f,0.1359f,  -0.9652f,0.1972f,0.1716f,  -0.9231f,0.3007f,0.2399f,  -0.9206f,0.2751f,0.2771f,  -0.8624f,0.3717f,0.3437f,  -0.8584f,0.3432f,0.3814f,  -0.7851f,0.4320f,0.4438f,  -0.7796f,0.4012f,0.4809f,  -0.6927f,0.4818f,0.5366f,  -0.6858f,0.4496f,0.5723f,  -0.5870f,0.5217f,0.6190f,  -0.5789f,0.4889f,0.6526f,  -0.4701f,0.5524f,0.6884f,  
	-0.4610f,0.5197f,0.7193f,  -0.3445f,0.5747f,0.7423f,  -0.3347f,0.5429f,0.7702f,  -0.2128f,0.5896f,0.7792f,  -0.2027f,0.5594f,0.8038f,  -0.0778f,0.5981f,0.7976f,  -0.0678f,0.5701f,0.8188f,  0.0575f,0.6012f,0.7970f,  0.0672f,0.5761f,0.8146f,  0.1903f,0.5999f,0.7771f,  0.1992f,0.5782f,0.7912f,  0.3176f,0.5952f,0.7382f,  0.3254f,0.5772f,0.7490f,  0.4367f,0.5877f,0.6811f,  0.4430f,0.5738f,0.6888f,  
	0.5451f,0.5783f,0.6070f,  0.5497f,0.5687f,0.6119f,  0.6403f,0.5675f,0.5176f,  0.6429f,0.5624f,0.5200f,  0.7205f,0.5557f,0.4149f,  0.7208f,0.5551f,0.4152f,  0.7837f,0.5432f,0.3011f,  0.7816f,0.5472f,0.2996f,  0.8288f,0.5301f,0.1789f,  0.8240f,0.5386f,0.1760f,  0.8548f,0.5165f,0.0509f,  0.8472f,0.5292f,0.0470f,  0.8611f,0.5022f,-0.0801f,  0.8506f,0.5190f,-0.0845f,  0.8476f,0.4868f,-0.2112f,  
	0.8343f,0.5074f,-0.2156f,  0.8148f,0.4701f,-0.3394f,  0.7987f,0.4940f,-0.3436f,  0.7634f,0.4514f,-0.4620f,  0.7446f,0.4783f,-0.4656f,  0.6947f,0.4302f,-0.5764f,  0.6734f,0.4596f,-0.5791f,  0.6104f,0.4059f,-0.6802f,  0.5867f,0.4372f,-0.6816f,  0.5125f,0.3777f,-0.7712f,  0.4868f,0.4103f,-0.7712f,  0.4033f,0.3449f,-0.8476f,  0.3760f,0.3782f,-0.8459f,  0.2857f,0.3068f,-0.9079f,  0.2572f,0.3401f,-0.9045f,  
	0.1626f,0.2628f,-0.9510f,  0.1334f,0.2954f,-0.9460f,  0.0371f,0.2124f,-0.9765f,  0.0079f,0.2435f,-0.9699f,  -0.0873f,0.1551f,-0.9840f,  -0.1160f,0.1841f,-0.9760f,  -0.2075f,0.0907f,-0.9740f,  -0.2349f,0.1169f,-0.9650f,  -0.3199f,0.0192f,-0.9473f,  -0.3453f,0.0420f,-0.9376f,  -0.4214f,-0.0592f,-0.9050f,  -0.4440f,-0.0403f,-0.8951f,  -0.5088f,-0.1439f,-0.8488f,  -0.5279f,-0.1292f,-0.8394f,  -0.5795f,-0.2339f,-0.7807f,  
	-0.5942f,-0.2236f,-0.7726f,  -0.6311f,-0.3280f,-0.7030f,  -0.6408f,-0.3222f,-0.6968f,  -0.6616f,-0.4246f,-0.6181f,  -0.6658f,-0.4230f,-0.6147f,  -0.6697f,-0.5215f,-0.5287f,  -0.6680f,-0.5237f,-0.5287f,  -0.6548f,-0.6164f,-0.4373f,  -0.6470f,-0.6218f,-0.4414f,  -0.7030f,-0.6426f,-0.3046f,  -0.7032f,-0.6368f,-0.3161f,  -0.7866f,-0.5366f,-0.3055f,  -0.7878f,-0.5328f,-0.3090f,  -0.8587f,-0.4253f,-0.2859f,  -0.8607f,-0.4242f,-0.2816f,  
	-0.9176f,-0.3118f,-0.2464f,  -0.9198f,-0.3143f,-0.2348f,  -0.9616f,-0.1992f,-0.1887f,  -0.9637f,-0.2056f,-0.1705f,  -0.9893f,-0.0899f,-0.1149f,  -0.9907f,-0.1008f,-0.0911f,  -0.9995f,0.0136f,-0.0280f,  -1.0000f,-0.0016f,0.0003f,  -0.9916f,0.1097f,0.0687f,  -0.9908f,0.0903f,0.1005f,  -0.9652f,0.1972f,0.1716f,  -0.9630f,0.1739f,0.2057f,  -0.9206f,0.2751f,0.2771f,  -0.9169f,0.2485f,0.3124f,  -0.8584f,0.3432f,0.3814f,  
	-0.8531f,0.3138f,0.4169f,  -0.7796f,0.4012f,0.4809f,  -0.7728f,0.3699f,0.5157f,  -0.6858f,0.4496f,0.5723f,  -0.6777f,0.4170f,0.6057f,  -0.5789f,0.4889f,0.6526f,  -0.5697f,0.4558f,0.6839f,  -0.4610f,0.5197f,0.7193f,  -0.4510f,0.4868f,0.7481f,  -0.3347f,0.5429f,0.7702f,  -0.3241f,0.5111f,0.7961f,  -0.2027f,0.5594f,0.8038f,  -0.1919f,0.5293f,0.8265f,  -0.0678f,0.5701f,0.8188f,  -0.0571f,0.5424f,0.8382f,  
	0.0672f,0.5761f,0.8146f,  0.0773f,0.5512f,0.8308f,  0.1992f,0.5782f,0.7912f,  0.2084f,0.5567f,0.8042f,  0.3254f,0.5772f,0.7490f,  0.3334f,0.5594f,0.7589f,  0.4430f,0.5738f,0.6888f,  0.4496f,0.5602f,0.6958f,  0.5497f,0.5687f,0.6119f,  0.5543f,0.5594f,0.6163f,  0.6429f,0.5624f,0.5200f,  0.6455f,0.5575f,0.5221f,  0.7208f,0.5551f,0.4152f,  0.7209f,0.5548f,0.4153f,  0.7816f,0.5472f,0.2996f,  
	0.7791f,0.5514f,0.2982f,  0.8240f,0.5386f,0.1760f,  0.8188f,0.5472f,0.1734f,  0.8472f,0.5292f,0.0470f,  0.8391f,0.5422f,0.0437f,  0.8506f,0.5190f,-0.0845f,  0.8396f,0.5360f,-0.0882f,  0.8343f,0.5074f,-0.2156f,  0.8203f,0.5282f,-0.2193f,  0.7987f,0.4940f,-0.3436f,  0.7817f,0.5183f,-0.3468f,  0.7446f,0.4783f,-0.4656f,  0.7248f,0.5055f,-0.4681f,  0.6734f,0.4596f,-0.5791f,  0.6509f,0.4893f,-0.5804f,  
	0.5867f,0.4372f,-0.6816f,  0.5619f,0.4687f,-0.6816f,  0.4868f,0.4103f,-0.7712f,  0.4599f,0.4431f,-0.7696f,  0.3760f,0.3782f,-0.8459f,  0.3474f,0.4115f,-0.8426f,  0.2572f,0.3401f,-0.9045f,  0.2274f,0.3733f,-0.8994f,  0.1334f,0.2954f,-0.9460f,  0.1029f,0.3277f,-0.9391f,  0.0079f,0.2435f,-0.9699f,  -0.0226f,0.2743f,-0.9614f,  -0.1160f,0.1841f,-0.9760f,  -0.1459f,0.2125f,-0.9662f,  -0.2349f,0.1169f,-0.9650f,  
	-0.2633f,0.1424f,-0.9541f,  -0.3453f,0.0420f,-0.9376f,  -0.3715f,0.0640f,-0.9262f,  -0.4440f,-0.0403f,-0.8951f,  -0.4672f,-0.0223f,-0.8839f,  -0.5279f,-0.1292f,-0.8394f,  -0.5473f,-0.1154f,-0.8289f,  -0.5942f,-0.2236f,-0.7726f,  -0.6091f,-0.2143f,-0.7636f,  -0.6408f,-0.3222f,-0.6968f,  -0.6505f,-0.3172f,-0.6901f,  -0.6658f,-0.4230f,-0.6147f,  -0.6697f,-0.4220f,-0.6110f,  -0.6680f,-0.5237f,-0.5287f,  -0.6658f,-0.5264f,-0.5288f,  
	-0.6470f,-0.6218f,-0.4414f,  -0.6385f,-0.6274f,-0.4458f,  -0.7032f,-0.6368f,-0.3161f,  -0.7035f,-0.6311f,-0.3268f,  -0.7878f,-0.5328f,-0.3090f,  -0.7890f,-0.5294f,-0.3119f,  -0.8607f,-0.4242f,-0.2816f,  -0.8623f,-0.4240f,-0.2768f,  -0.9198f,-0.3143f,-0.2348f,  -0.9215f,-0.3178f,-0.2231f,  -0.9637f,-0.2056f,-0.1705f,  -0.9650f,-0.2134f,-0.1526f,  -0.9907f,-0.1008f,-0.0911f,  -0.9913f,-0.1129f,-0.0680f,  -1.0000f,-0.0016f,0.0003f,  
	-0.9995f,-0.0181f,0.0276f,  -0.9908f,0.0903f,0.1005f,  -0.9889f,0.0697f,0.1308f,  -0.9630f,0.1739f,0.2057f,  -0.9597f,0.1496f,0.2381f,  -0.9169f,0.2485f,0.3124f,  -0.9119f,0.2210f,0.3457f,  -0.8531f,0.3138f,0.4169f,  -0.8466f,0.2837f,0.4503f,  -0.7728f,0.3699f,0.5157f,  -0.7649f,0.3380f,0.5483f,  -0.6777f,0.4170f,0.6057f,  -0.6686f,0.3840f,0.6368f,  -0.5697f,0.4558f,0.6839f,  -0.5595f,0.4225f,0.7131f,  
	-0.4510f,0.4868f,0.7481f,  -0.4400f,0.4540f,0.7748f,  -0.3241f,0.5111f,0.7961f,  -0.3127f,0.4794f,0.8200f,  -0.1919f,0.5293f,0.8265f,  -0.1804f,0.4994f,0.8474f,  -0.0571f,0.5424f,0.8382f,  -0.0459f,0.5149f,0.8560f,  0.0773f,0.5512f,0.8308f,  0.0879f,0.5267f,0.8455f,  0.2084f,0.5567f,0.8042f,  0.2181f,0.5355f,0.8159f,  0.3334f,0.5594f,0.7589f,  0.3418f,0.5420f,0.7677f,  0.4496f,0.5602f,0.6958f,  
	0.4563f,0.5468f,0.7020f,  0.5543f,0.5594f,0.6163f,  0.5591f,0.5503f,0.6201f,  0.6455f,0.5575f,0.5221f,  0.6480f,0.5529f,0.5238f,  0.7209f,0.5548f,0.4153f,  0.7209f,0.5548f,0.4153f,  0.7791f,0.5514f,0.2982f,  0.7764f,0.5559f,0.2969f,  0.8188f,0.5472f,0.1734f,  0.8132f,0.5562f,0.1711f,  0.8391f,0.5422f,0.0437f,  0.8305f,0.5555f,0.0408f,  0.8396f,0.5360f,-0.0882f,  0.8279f,0.5534f,-0.0912f,  
	0.8203f,0.5282f,-0.2193f,  0.8055f,0.5494f,-0.2221f,  0.7817f,0.5183f,-0.3468f,  0.7639f,0.5428f,-0.3490f,  0.7248f,0.5055f,-0.4681f,  0.7041f,0.5330f,-0.4693f,  0.6509f,0.4893f,-0.5804f,  0.6274f,0.5191f,-0.5804f,  0.5619f,0.4687f,-0.6816f,  0.5359f,0.5004f,-0.6800f,  0.4599f,0.4431f,-0.7696f,  0.4317f,0.4759f,-0.7663f,  0.3474f,0.4115f,-0.8426f,  0.3175f,0.4448f,-0.8374f,  0.2274f,0.3733f,-0.8994f,  
	0.1963f,0.4064f,-0.8924f,  0.1029f,0.3277f,-0.9391f,  0.0712f,0.3598f,-0.9303f,  -0.0226f,0.2743f,-0.9614f,  -0.0544f,0.3046f,-0.9509f,  -0.1459f,0.2125f,-0.9662f,  -0.1768f,0.2404f,-0.9544f,  -0.2633f,0.1424f,-0.9541f,  -0.2927f,0.1671f,-0.9415f,  -0.3715f,0.0640f,-0.9262f,  -0.3985f,0.0850f,-0.9132f,  -0.4672f,-0.0223f,-0.8839f,  -0.4910f,-0.0053f,-0.8712f,  -0.5473f,-0.1154f,-0.8289f,  -0.5670f,-0.1028f,-0.8173f,  
	-0.6091f,-0.2143f,-0.7636f,  -0.6241f,-0.2061f,-0.7537f,  -0.6505f,-0.3172f,-0.6901f,  -0.6600f,-0.3132f,-0.6829f,  -0.6697f,-0.4220f,-0.6110f,  -0.6733f,-0.4219f,-0.6072f,  -0.6658f,-0.5264f,-0.5288f,  -0.6631f,-0.5295f,-0.5291f,  -0.6385f,-0.6274f,-0.4458f,  -0.6294f,-0.6330f,-0.4507f,  -0.7035f,-0.6311f,-0.3268f,  -0.7038f,-0.6255f,-0.3368f,  -0.7890f,-0.5294f,-0.3119f,  -0.7900f,-0.5265f,-0.3141f,  -0.8623f,-0.4240f,-0.2768f,  
	-0.8637f,-0.4245f,-0.2718f,  -0.9215f,-0.3178f,-0.2231f,  -0.9227f,-0.3224f,-0.2114f,  -0.9650f,-0.2134f,-0.1526f,  -0.9656f,-0.2223f,-0.1351f,  -0.9913f,-0.1129f,-0.0680f,  -0.9910f,-0.1262f,-0.0457f,  -0.9995f,-0.0181f,0.0276f,  -0.9979f,-0.0357f,0.0537f,  -0.9889f,0.0697f,0.1308f,  -0.9860f,0.0481f,0.1597f,  -0.9597f,0.1496f,0.2381f,  -0.9552f,0.1244f,0.2687f,  -0.9119f,0.2210f,0.3457f,  -0.9059f,0.1927f,0.3771f,  
	-0.8466f,0.2837f,0.4503f,  -0.8391f,0.2531f,0.4816f,  -0.7649f,0.3380f,0.5483f,  -0.7560f,0.3057f,0.5788f,  -0.6686f,0.3840f,0.6368f,  -0.6584f,0.3509f,0.6659f,  -0.5595f,0.4225f,0.7131f,  -0.5484f,0.3892f,0.7402f,  -0.4400f,0.4540f,0.7748f,  -0.4282f,0.4212f,0.7995f,  -0.3127f,0.4794f,0.8200f,  -0.3006f,0.4478f,0.8421f,  -0.1804f,0.4994f,0.8474f,  -0.1683f,0.4697f,0.8667f,  -0.0459f,0.5149f,0.8560f,  
	-0.0341f,0.4876f,0.8724f,  0.0879f,0.5267f,0.8455f,  0.0990f,0.5024f,0.8590f,  0.2181f,0.5355f,0.8159f,  0.2281f,0.5146f,0.8265f,  0.3418f,0.5420f,0.7677f,  0.3504f,0.5249f,0.7757f,  0.4563f,0.5468f,0.7020f,  0.4632f,0.5338f,0.7075f,  0.5591f,0.5503f,0.6201f,  0.5639f,0.5416f,0.6234f,  0.6480f,0.5529f,0.5238f,  0.6504f,0.5487f,0.5253f,  0.7209f,0.5548f,0.4153f,  0.7208f,0.5550f,0.4152f,  
	0.7764f,0.5559f,0.2969f,  0.7734f,0.5607f,0.2956f,  0.8132f,0.5562f,0.1711f,  0.8072f,0.5655f,0.1691f,  0.8305f,0.5555f,0.0408f,  0.8214f,0.5691f,0.0384f,  0.8279f,0.5534f,-0.0912f,  0.8156f,0.5711f,-0.0936f,  0.8055f,0.5494f,-0.2221f,  0.7900f,0.5708f,-0.2240f,  0.7639f,0.5428f,-0.3490f,  0.7452f,0.5676f,-0.3501f,  0.7041f,0.5330f,-0.4693f,  0.6822f,0.5607f,-0.4692f,  0.6274f,0.5191f,-0.5804f,  
	0.6027f,0.5492f,-0.5789f,  0.5359f,0.5004f,-0.6800f,  0.5086f,0.5322f,-0.6768f,  0.4317f,0.4759f,-0.7663f,  0.4022f,0.5088f,-0.7612f,  0.3175f,0.4448f,-0.8374f,  0.2862f,0.4781f,-0.8304f,  0.1963f,0.4064f,-0.8924f,  0.1637f,0.4392f,-0.8833f,  0.0712f,0.3598f,-0.9303f,  0.0380f,0.3914f,-0.9194f,  -0.0544f,0.3046f,-0.9509f,  -0.0874f,0.3343f,-0.9384f,  -0.1768f,0.2404f,-0.9544f,  -0.2089f,0.2674f,-0.9407f,  
	-0.2927f,0.1671f,-0.9415f,  -0.3231f,0.1909f,-0.9269f,  -0.3985f,0.0850f,-0.9132f,  -0.4263f,0.1050f,-0.8985f,  -0.4910f,-0.0053f,-0.8712f,  -0.5152f,0.0105f,-0.8570f,  -0.5670f,-0.1028f,-0.8173f,  -0.5870f,-0.0914f,-0.8044f,  -0.6241f,-0.2061f,-0.7537f,  -0.6389f,-0.1991f,-0.7431f,  -0.6600f,-0.3132f,-0.6829f,  -0.6692f,-0.3103f,-0.6752f,  -0.6733f,-0.4219f,-0.6072f,  -0.6763f,-0.4226f,-0.6033f,  -0.6631f,-0.5295f,-0.5291f,  
	-0.6598f,-0.5331f,-0.5296f,  -0.6294f,-0.6330f,-0.4507f,  -0.6197f,-0.6387f,-0.4561f,  -0.7038f,-0.6255f,-0.3368f,  -0.7043f,-0.6199f,-0.3460f,  -0.7900f,-0.5265f,-0.3141f,  -0.7910f,-0.5240f,-0.3159f,  -0.8637f,-0.4245f,-0.2718f,  -0.8647f,-0.4258f,-0.2666f,  -0.9227f,-0.3224f,-0.2114f,  -0.9234f,-0.3278f,-0.1999f,  -0.9656f,-0.2223f,-0.1351f,  -0.9655f,-0.2321f,-0.1182f,  -0.9910f,-0.1262f,-0.0457f,  -0.9898f,-0.1405f,-0.0243f,  
	-0.9979f,-0.0357f,0.0537f,  -0.9954f,-0.0542f,0.0786f,  -0.9860f,0.0481f,0.1597f,  -0.9820f,0.0256f,0.1870f,  -0.9552f,0.1244f,0.2687f,  -0.9496f,0.0985f,0.2975f,  -0.9059f,0.1927f,0.3771f,  -0.8988f,0.1640f,0.4066f,  -0.8391f,0.2531f,0.4816f,  -0.8305f,0.2222f,0.5108f,  -0.7560f,0.3057f,0.5788f,  -0.7460f,0.2733f,0.6072f,  -0.6584f,0.3509f,0.6659f,  -0.6473f,0.3177f,0.6929f,  -0.5484f,0.3892f,0.7402f,  
	-0.5364f,0.3559f,0.7653f,  -0.4282f,0.4212f,0.7995f,  -0.4157f,0.3886f,0.8223f,  -0.3006f,0.4478f,0.8421f,  -0.2878f,0.4164f,0.8624f,  -0.1683f,0.4697f,0.8667f,  -0.1555f,0.4402f,0.8843f,  -0.0341f,0.4876f,0.8724f,  -0.0217f,0.4606f,0.8873f,  0.0990f,0.5024f,0.8590f,  0.1106f,0.4783f,0.8712f,  0.2281f,0.5146f,0.8265f,  0.2385f,0.4940f,0.8361f,  0.3504f,0.5249f,0.7757f,  0.3593f,0.5080f,0.7828f,  
	0.4632f,0.5338f,0.7075f,  0.4702f,0.5210f,0.7124f,  0.5639f,0.5416f,0.6234f,  0.5687f,0.5332f,0.6263f,  0.6504f,0.5487f,0.5253f,  0.6528f,0.5447f,0.5265f,  0.7208f,0.5550f,0.4152f,  0.7204f,0.5556f,0.4151f,  0.7734f,0.5607f,0.2956f,  0.7701f,0.5658f,0.2945f,  0.8072f,0.5655f,0.1691f,  0.8008f,0.5751f,0.1674f,  0.8214f,0.5691f,0.0384f,  0.8117f,0.5830f,0.0366f,  0.8156f,0.5711f,-0.0936f,  
	0.8025f,0.5890f,-0.0951f,  0.7900f,0.5708f,-0.2240f,  0.7735f,0.5925f,-0.2250f,  0.7452f,0.5676f,-0.3501f,  0.7254f,0.5926f,-0.3501f,  0.6822f,0.5607f,-0.4692f,  0.6593f,0.5886f,-0.4678f,  0.6027f,0.5492f,-0.5789f,  0.5768f,0.5794f,-0.5758f,  0.5086f,0.5322f,-0.6768f,  0.4800f,0.5641f,-0.6719f,  0.4022f,0.5088f,-0.7612f,  0.3713f,0.5416f,-0.7542f,  0.2862f,0.4781f,-0.8304f,  0.2535f,0.5111f,-0.8213f,  
	0.1637f,0.4392f,-0.8833f,  0.1298f,0.4716f,-0.8722f,  0.0380f,0.3914f,-0.9194f,  0.0036f,0.4225f,-0.9064f,  -0.0874f,0.3343f,-0.9384f,  -0.1216f,0.3632f,-0.9238f,  -0.2089f,0.2674f,-0.9407f,  -0.2421f,0.2934f,-0.9248f,  -0.3231f,0.1909f,-0.9269f,  -0.3543f,0.2134f,-0.9105f,  -0.4263f,0.1050f,-0.8985f,  -0.4546f,0.1235f,-0.8821f,  -0.5152f,0.0105f,-0.8570f,  -0.5398f,0.0248f,-0.8414f,  -0.5870f,-0.0914f,-0.8044f,  
	-0.6070f,-0.0815f,-0.7905f,  -0.6389f,-0.1991f,-0.7431f,  -0.6537f,-0.1934f,-0.7317f,  -0.6692f,-0.3103f,-0.6752f,  -0.6780f,-0.3085f,-0.6672f,  -0.6763f,-0.4226f,-0.6033f,  -0.6789f,-0.4242f,-0.5994f,  -0.6598f,-0.5331f,-0.5296f,  -0.6559f,-0.5372f,-0.5303f,  -0.6197f,-0.6387f,-0.4561f,  -0.6094f,-0.6444f,-0.4619f,  -0.7043f,-0.6199f,-0.3460f,  -0.7048f,-0.6145f,-0.3545f,  -0.7910f,-0.5240f,-0.3159f,  -0.7918f,-0.5220f,-0.3171f,  
	-0.8647f,-0.4258f,-0.2666f,  -0.8653f,-0.4278f,-0.2612f,  -0.9234f,-0.3278f,-0.1999f,  -0.9235f,-0.3341f,-0.1886f,  -0.9655f,-0.2321f,-0.1182f,  -0.9647f,-0.2428f,-0.1018f,  -0.9898f,-0.1405f,-0.0243f,  -0.9878f,-0.1556f,-0.0038f,  -0.9954f,-0.0542f,0.0786f,  -0.9920f,-0.0735f,0.1022f,  -0.9820f,0.0256f,0.1870f,  -0.9771f,0.0025f,0.2128f,  -0.9496f,0.0985f,0.2975f,  -0.9431f,0.0720f,0.3246f,  -0.8988f,0.1640f,0.4066f,  
	-0.8907f,0.1349f,0.4341f,  -0.8305f,0.2222f,0.5108f,  -0.8209f,0.1910f,0.5382f,  -0.7460f,0.2733f,0.6072f,  -0.7352f,0.2407f,0.6337f,  -0.6473f,0.3177f,0.6929f,  -0.6354f,0.2844f,0.7179f,  -0.5364f,0.3559f,0.7653f,  -0.5236f,0.3227f,0.7885f,  -0.4157f,0.3886f,0.8223f,  -0.4024f,0.3561f,0.8434f,  -0.2878f,0.4164f,0.8624f,  -0.2743f,0.3853f,0.8811f,  -0.1555f,0.4402f,0.8843f,  -0.1421f,0.4110f,0.9005f,  
	-0.0217f,0.4606f,0.8873f,  -0.0089f,0.4339f,0.9009f,  0.1106f,0.4783f,0.8712f,  0.1225f,0.4546f,0.8822f,  0.2385f,0.4940f,0.8361f,  0.2492f,0.4736f,0.8447f,  0.3593f,0.5080f,0.7828f,  0.3684f,0.4915f,0.7891f,  0.4702f,0.5210f,0.7124f,  0.4773f,0.5085f,0.7166f,  0.5687f,0.5332f,0.6263f,  0.5736f,0.5250f,0.6288f,  0.6528f,0.5447f,0.5265f,  0.6551f,0.5410f,0.5275f,  0.7204f,0.5556f,0.4151f,  
	0.7199f,0.5564f,0.4149f,  0.7701f,0.5658f,0.2945f,  0.7665f,0.5712f,0.2936f,  0.8008f,0.5751f,0.1674f,  0.7939f,0.5850f,0.1661f,  0.8117f,0.5830f,0.0366f,  0.8013f,0.5972f,0.0353f,  0.8025f,0.5890f,-0.0951f,  0.7887f,0.6073f,-0.0959f,  0.7735f,0.5925f,-0.2250f,  0.7562f,0.6145f,-0.2249f,  0.7254f,0.5926f,-0.3501f,  0.7046f,0.6180f,-0.3488f,  0.6593f,0.5886f,-0.4678f,  0.6352f,0.6167f,-0.4650f,  
	0.5768f,0.5794f,-0.5758f,  0.5496f,0.6097f,-0.5711f,  0.4800f,0.5641f,-0.6719f,  0.4500f,0.5959f,-0.6651f,  0.3713f,0.5416f,-0.7542f,  0.3390f,0.5742f,-0.7452f,  0.2535f,0.5111f,-0.8213f,  0.2194f,0.5437f,-0.8101f,  0.1298f,0.4716f,-0.8722f,  0.0945f,0.5034f,-0.8589f,  0.0036f,0.4225f,-0.9064f,  -0.0322f,0.4527f,-0.8911f,  -0.1216f,0.3632f,-0.9238f,  -0.1570f,0.3910f,-0.9069f,  -0.2421f,0.2934f,-0.9248f,  
	-0.2763f,0.3182f,-0.9069f,  -0.3543f,0.2134f,-0.9105f,  -0.3863f,0.2345f,-0.8921f,  -0.4546f,0.1235f,-0.8821f,  -0.4835f,0.1405f,-0.8640f,  -0.5398f,0.0248f,-0.8414f,  -0.5647f,0.0374f,-0.8245f,  -0.6070f,-0.0815f,-0.7905f,  -0.6270f,-0.0732f,-0.7756f,  -0.6537f,-0.1934f,-0.7317f,  -0.6681f,-0.1892f,-0.7197f,  -0.6780f,-0.3085f,-0.6672f,  -0.6863f,-0.3080f,-0.6589f,  -0.6789f,-0.4242f,-0.5994f,  -0.6807f,-0.4266f,-0.5955f,  
	-0.6559f,-0.5372f,-0.5303f,  -0.6512f,-0.5418f,-0.5314f,  -0.6094f,-0.6444f,-0.4619f,  -0.5984f,-0.6502f,-0.4682f,  -0.7048f,-0.6145f,-0.3545f,  -0.7054f,-0.6092f,-0.3623f,  -0.7918f,-0.5220f,-0.3171f,  -0.7925f,-0.5204f,-0.3180f,  -0.8653f,-0.4278f,-0.2612f,  -0.8657f,-0.4303f,-0.2558f,  -0.9235f,-0.3341f,-0.1886f,  -0.9231f,-0.3411f,-0.1776f,  -0.9647f,-0.2428f,-0.1018f,  -0.9633f,-0.2543f,-0.0862f,  -0.9878f,-0.1556f,-0.0038f,  
	-0.9851f,-0.1715f,0.0156f,  -0.9920f,-0.0735f,0.1022f,  -0.9878f,-0.0935f,0.1245f,  -0.9771f,0.0025f,0.2128f,  -0.9713f,-0.0211f,0.2371f,  -0.9431f,0.0720f,0.3246f,  -0.9357f,0.0452f,0.3500f,  -0.8907f,0.1349f,0.4341f,  -0.8817f,0.1055f,0.4599f,  -0.8209f,0.1910f,0.5382f,  -0.8104f,0.1597f,0.5636f,  -0.7352f,0.2407f,0.6337f,  -0.7234f,0.2081f,0.6583f,  -0.6354f,0.2844f,0.7179f,  -0.6226f,0.2512f,0.7411f,  
	-0.5236f,0.3227f,0.7885f,  -0.5101f,0.2896f,0.8099f,  -0.4024f,0.3561f,0.8434f,  -0.3883f,0.3237f,0.8628f,  -0.2743f,0.3853f,0.8811f,  -0.2601f,0.3543f,0.8982f,  -0.1421f,0.4110f,0.9005f,  -0.1282f,0.3820f,0.9152f,  -0.0089f,0.4339f,0.9009f,  0.0045f,0.4074f,0.9132f,  0.1225f,0.4546f,0.8822f,  0.1349f,0.4311f,0.8922f,  0.2492f,0.4736f,0.8447f,  0.2603f,0.4535f,0.8524f,  0.3684f,0.4915f,0.7891f,  
	0.3777f,0.4752f,0.7947f,  0.4773f,0.5085f,0.7166f,  0.4846f,0.4963f,0.7203f,  0.5736f,0.5250f,0.6288f,  0.5785f,0.5171f,0.6308f,  0.6551f,0.5410f,0.5275f,  0.6573f,0.5375f,0.5282f,  0.7199f,0.5564f,0.4149f,  0.7191f,0.5576f,0.4147f,  0.7665f,0.5712f,0.2936f,  0.7625f,0.5769f,0.2928f,  0.7939f,0.5850f,0.1661f,  0.7864f,0.5951f,0.1652f,  0.8013f,0.5972f,0.0353f,  0.7903f,0.6117f,0.0347f,  
	0.7887f,0.6073f,-0.0959f,  0.7740f,0.6258f,-0.0959f,  0.7562f,0.6145f,-0.2249f,  0.7379f,0.6368f,-0.2238f,  0.7046f,0.6180f,-0.3488f,  0.6827f,0.6435f,-0.3462f,  0.6352f,0.6167f,-0.4650f,  0.6098f,0.6449f,-0.4606f,  0.5496f,0.6097f,-0.5711f,  0.5211f,0.6400f,-0.5647f,  0.4500f,0.5959f,-0.6651f,  0.4186f,0.6276f,-0.6564f,  0.3390f,0.5742f,-0.7452f,  0.3052f,0.6065f,-0.7342f,  0.2194f,0.5437f,-0.8101f,  
	0.1838f,0.5758f,-0.7967f,  0.0945f,0.5034f,-0.8589f,  0.0578f,0.5345f,-0.8432f,  -0.0322f,0.4527f,-0.8911f,  -0.0692f,0.4818f,-0.8735f,  -0.1570f,0.3910f,-0.9069f,  -0.1935f,0.4175f,-0.8878f,  -0.2763f,0.3182f,-0.9069f,  -0.3113f,0.3414f,-0.8869f,  -0.3863f,0.2345f,-0.8921f,  -0.4189f,0.2539f,-0.8718f,  -0.4835f,0.1405f,-0.8640f,  -0.5127f,0.1557f,-0.8443f,  -0.5647f,0.0374f,-0.8245f,  -0.5895f,0.0483f,-0.8063f,  
	-0.6270f,-0.0732f,-0.7756f,  -0.6467f,-0.0666f,-0.7598f,  -0.6681f,-0.1892f,-0.7197f,  -0.6820f,-0.1865f,-0.7072f,  -0.6863f,-0.3080f,-0.6589f,  -0.6940f,-0.3087f,-0.6505f,  -0.6807f,-0.4266f,-0.5955f,  -0.6819f,-0.4299f,-0.5918f,  -0.6512f,-0.5418f,-0.5314f,  -0.6458f,-0.5468f,-0.5328f,  -0.5984f,-0.6502f,-0.4682f,  -0.5867f,-0.6559f,-0.4750f,  -0.7054f,-0.6092f,-0.3623f,  -0.7061f,-0.6041f,-0.3694f,  -0.7925f,-0.5204f,-0.3180f,  
	-0.7931f,-0.5192f,-0.3185f,  -0.8657f,-0.4303f,-0.2558f,  -0.8657f,-0.4335f,-0.2504f,  -0.9231f,-0.3411f,-0.1776f,  -0.9222f,-0.3487f,-0.1670f,  -0.9633f,-0.2543f,-0.0862f,  -0.9612f,-0.2664f,-0.0712f,  -0.9851f,-0.1715f,0.0156f,  -0.9816f,-0.1879f,0.0340f,  -0.9878f,-0.0935f,0.1245f,  -0.9828f,-0.1140f,0.1455f,  -0.9713f,-0.0211f,0.2371f,  -0.9646f,-0.0452f,0.2598f,  -0.9357f,0.0452f,0.3500f,  -0.9274f,0.0181f,0.3737f,  
	-0.8817f,0.1055f,0.4599f,  -0.8718f,0.0759f,0.4839f,  -0.8104f,0.1597f,0.5636f,  -0.7991f,0.1282f,0.5873f,  -0.7234f,0.2081f,0.6583f,  -0.7109f,0.1755f,0.6810f,  -0.6226f,0.2512f,0.7411f,  -0.6090f,0.2181f,0.7626f,  -0.5101f,0.2896f,0.8099f,  -0.4958f,0.2566f,0.8297f,  -0.3883f,0.3237f,0.8628f,  -0.3736f,0.2916f,0.8806f,  -0.2601f,0.3543f,0.8982f,  -0.2453f,0.3235f,0.9139f,  -0.1282f,0.3820f,0.9152f,  
	-0.1137f,0.3532f,0.9286f,  0.0045f,0.4074f,0.9132f,  0.0183f,0.3811f,0.9243f,  0.1349f,0.4311f,0.8922f,  0.1477f,0.4078f,0.9010f,  0.2603f,0.4535f,0.8524f,  0.2717f,0.4337f,0.8591f,  0.3777f,0.4752f,0.7947f,  0.3873f,0.4591f,0.7995f,  0.4846f,0.4963f,0.7203f,  0.4921f,0.4843f,0.7234f,  0.5785f,0.5171f,0.6308f,  0.5835f,0.5094f,0.6325f,  0.6573f,0.5375f,0.5282f,  0.6594f,0.5344f,0.5288f,  
	0.7191f,0.5576f,0.4147f,  0.7181f,0.5590f,0.4146f,  0.7625f,0.5769f,0.2928f,  0.7581f,0.5829f,0.2923f,  0.7864f,0.5951f,0.1652f,  0.7785f,0.6056f,0.1648f,  0.7903f,0.6117f,0.0347f,  0.7787f,0.6265f,0.0348f,  0.7740f,0.6258f,-0.0959f,  0.7585f,0.6447f,-0.0949f,  0.7379f,0.6368f,-0.2238f,  0.7186f,0.6592f,-0.2215f,  0.6827f,0.6435f,-0.3462f,  0.6596f,0.6691f,-0.3422f,  0.6098f,0.6449f,-0.4606f,  
	0.5832f,0.6732f,-0.4546f,  0.5211f,0.6400f,-0.5647f,  0.4911f,0.6702f,-0.5564f,  0.4186f,0.6276f,-0.6564f,  0.3858f,0.6590f,-0.6457f,  0.3052f,0.6065f,-0.7342f,  0.2699f,0.6383f,-0.7209f,  0.1838f,0.5758f,-0.7967f,  0.1467f,0.6071f,-0.7809f,  0.0578f,0.5345f,-0.8432f,  0.0197f,0.5645f,-0.8252f,  -0.0692f,0.4818f,-0.8735f,  -0.1075f,0.5097f,-0.8536f,  -0.1935f,0.4175f,-0.8878f,  -0.2311f,0.4425f,-0.8665f,  
	-0.3113f,0.3414f,-0.8869f,  -0.3471f,0.3629f,-0.8648f,  -0.4189f,0.2539f,-0.8718f,  -0.4519f,0.2713f,-0.8498f,  -0.5127f,0.1557f,-0.8443f,  -0.5420f,0.1689f,-0.8232f,  -0.5895f,0.0483f,-0.8063f,  -0.6143f,0.0571f,-0.7870f,  -0.6467f,-0.0666f,-0.7598f,  -0.6660f,-0.0619f,-0.7434f,  -0.6820f,-0.1865f,-0.7072f,  -0.6953f,-0.1855f,-0.6944f,  -0.6940f,-0.3087f,-0.6505f,  -0.7009f,-0.3107f,-0.6421f,  -0.6819f,-0.4299f,-0.5918f,  
	-0.6822f,-0.4341f,-0.5883f,  -0.6458f,-0.5468f,-0.5328f,  -0.6396f,-0.5522f,-0.5347f,  -0.5867f,-0.6559f,-0.4750f,  -0.5743f,-0.6615f,-0.4822f,  -0.7061f,-0.6041f,-0.3694f,  -0.7069f,-0.5992f,-0.3758f,  -0.7931f,-0.5192f,-0.3185f,  -0.7935f,-0.5184f,-0.3188f,  -0.8657f,-0.4335f,-0.2504f,  -0.8654f,-0.4371f,-0.2450f,  -0.9222f,-0.3487f,-0.1670f,  -0.9209f,-0.3568f,-0.1568f,  -0.9612f,-0.2664f,-0.0712f,  -0.9586f,-0.2791f,-0.0570f,  
	-0.9816f,-0.1879f,0.0340f,  -0.9774f,-0.2048f,0.0514f,  -0.9828f,-0.1140f,0.1455f,  -0.9770f,-0.1348f,0.1653f,  -0.9646f,-0.0452f,0.2598f,  -0.9571f,-0.0695f,0.2812f,  -0.9274f,0.0181f,0.3737f,  -0.9182f,-0.0092f,0.3959f,  -0.8718f,0.0759f,0.4839f,  -0.8611f,0.0462f,0.5063f,  -0.7991f,0.1282f,0.5873f,  -0.7870f,0.0968f,0.6093f,  -0.7109f,0.1755f,0.6810f,  -0.6976f,0.1430f,0.7021f,  -0.6090f,0.2181f,0.7626f,  
	-0.5947f,0.1851f,0.7823f,  -0.4958f,0.2566f,0.8297f,  -0.4808f,0.2238f,0.8478f,  -0.3736f,0.2916f,0.8806f,  -0.3583f,0.2596f,0.8968f,  -0.2453f,0.3235f,0.9139f,  -0.2299f,0.2930f,0.9281f,  -0.1137f,0.3532f,0.9286f,  -0.0987f,0.3246f,0.9407f,  0.0183f,0.3811f,0.9243f,  0.0326f,0.3551f,0.9343f,  0.1477f,0.4078f,0.9010f,  0.1610f,0.3847f,0.9089f,  0.2717f,0.4337f,0.8591f,  0.2834f,0.4140f,0.8650f,  
	0.3873f,0.4591f,0.7995f,  0.3972f,0.4433f,0.8036f,  0.4921f,0.4843f,0.7234f,  0.4996f,0.4726f,0.7259f,  0.5835f,0.5094f,0.6325f,  0.5884f,0.5021f,0.6338f,  0.6594f,0.5344f,0.5288f,  0.6614f,0.5315f,0.5292f,  0.7181f,0.5590f,0.4146f,  0.7168f,0.5607f,0.4144f,  0.7581f,0.5829f,0.2923f,  0.7533f,0.5892f,0.2921f,  0.7785f,0.6056f,0.1648f,  0.7700f,0.6164f,0.1648f,  0.7787f,0.6265f,0.0348f,  
	0.7662f,0.6416f,0.0355f,  0.7585f,0.6447f,-0.0949f,  0.7422f,0.6637f,-0.0930f,  0.7186f,0.6592f,-0.2215f,  0.6982f,0.6819f,-0.2180f,  0.6596f,0.6691f,-0.3422f,  0.6354f,0.6949f,-0.3368f,  0.5832f,0.6732f,-0.4546f,  0.5552f,0.7014f,-0.4469f,  0.4911f,0.6702f,-0.5564f,  0.4598f,0.7002f,-0.5462f,  0.3858f,0.6590f,-0.6457f,  0.3514f,0.6900f,-0.6328f,  0.2699f,0.6383f,-0.7209f,  0.2332f,0.6694f,-0.7053f,  
	0.1467f,0.6071f,-0.7809f,  0.1083f,0.6375f,-0.7628f,  0.0197f,0.5645f,-0.8252f,  -0.0197f,0.5932f,-0.8048f,  -0.1075f,0.5097f,-0.8536f,  -0.1469f,0.5360f,-0.8313f,  -0.2311f,0.4425f,-0.8665f,  -0.2695f,0.4656f,-0.8430f,  -0.3471f,0.3629f,-0.8648f,  -0.3835f,0.3823f,-0.8407f,  -0.4519f,0.2713f,-0.8498f,  -0.4853f,0.2866f,-0.8261f,  -0.5420f,0.1689f,-0.8232f,  -0.5714f,0.1798f,-0.8008f,  -0.6143f,0.0571f,-0.7870f,  
	-0.6387f,0.0638f,-0.7668f,  -0.6660f,-0.0619f,-0.7434f,  -0.6848f,-0.0592f,-0.7264f,  -0.6953f,-0.1855f,-0.6944f,  -0.7079f,-0.1862f,-0.6814f,  -0.7009f,-0.3107f,-0.6421f,  -0.7069f,-0.3140f,-0.6338f,  -0.6822f,-0.4341f,-0.5883f,  -0.6816f,-0.4392f,-0.5853f,  -0.6396f,-0.5522f,-0.5347f,  -0.6326f,-0.5580f,-0.5370f,  -0.5743f,-0.6615f,-0.4822f,  -0.5613f,-0.6670f,-0.4899f,  -0.7069f,-0.5992f,-0.3758f,  -0.7077f,-0.5945f,-0.3817f,  
	-0.7935f,-0.5184f,-0.3188f,  -0.7938f,-0.5180f,-0.3187f,  -0.8654f,-0.4371f,-0.2450f,  -0.8648f,-0.4412f,-0.2397f,  -0.9209f,-0.3568f,-0.1568f,  -0.9191f,-0.3655f,-0.1470f,  -0.9586f,-0.2791f,-0.0570f,  -0.9554f,-0.2922f,-0.0435f,  -0.9774f,-0.2048f,0.0514f,  -0.9726f,-0.2221f,0.0679f,  -0.9770f,-0.1348f,0.1653f,  -0.9705f,-0.1560f,0.1838f,  -0.9571f,-0.0695f,0.2812f,  -0.9489f,-0.0941f,0.3012f,  -0.9182f,-0.0092f,0.3959f,  
	-0.9083f,-0.0367f,0.4166f,  -0.8611f,0.0462f,0.5063f,  -0.8496f,0.0164f,0.5271f,  -0.7870f,0.0968f,0.6093f,  -0.7741f,0.0654f,0.6297f,  -0.6976f,0.1430f,0.7021f,  -0.6835f,0.1105f,0.7216f,  -0.5947f,0.1851f,0.7823f,  -0.5797f,0.1522f,0.8005f,  -0.4808f,0.2238f,0.8478f,  -0.4651f,0.1911f,0.8644f,  -0.3583f,0.2596f,0.8968f,  -0.3422f,0.2277f,0.9116f,  -0.2299f,0.2930f,0.9281f,  -0.2139f,0.2626f,0.9409f,  
	-0.0987f,0.3246f,0.9407f,  -0.0830f,0.2963f,0.9515f,  0.0326f,0.3551f,0.9343f,  0.0475f,0.3292f,0.9431f,  0.1610f,0.3847f,0.9089f,  0.1746f,0.3619f,0.9157f,  0.2834f,0.4140f,0.8650f,  0.2955f,0.3946f,0.8700f,  0.3972f,0.4433f,0.8036f,  0.4073f,0.4277f,0.8070f,  0.4996f,0.4726f,0.7259f,  0.5074f,0.4611f,0.7280f,  0.5884f,0.5021f,0.6338f,  0.5934f,0.4949f,0.6348f,  0.6614f,0.5315f,0.5292f,  
	0.6633f,0.5289f,0.5295f,  0.7168f,0.5607f,0.4144f,  0.7153f,0.5627f,0.4144f,  0.7533f,0.5892f,0.2921f,  0.7481f,0.5958f,0.2921f,  0.7700f,0.6164f,0.1648f,  0.7608f,0.6275f,0.1654f,  0.7662f,0.6416f,0.0355f,  0.7531f,0.6569f,0.0371f,  0.7422f,0.6637f,-0.0930f,  0.7248f,0.6830f,-0.0900f,  0.6982f,0.6819f,-0.2180f,  0.6767f,0.7047f,-0.2132f,  0.6354f,0.6949f,-0.3368f,  0.6099f,0.7206f,-0.3297f,  
	0.5552f,0.7014f,-0.4469f,  0.5259f,0.7295f,-0.4373f,  0.4598f,0.7002f,-0.5462f,  0.4270f,0.7298f,-0.5339f,  0.3514f,0.6900f,-0.6328f,  0.3157f,0.7203f,-0.6177f,  0.2332f,0.6694f,-0.7053f,  0.1951f,0.6996f,-0.6874f,  0.1083f,0.6375f,-0.7628f,  0.0685f,0.6666f,-0.7423f,  -0.0197f,0.5932f,-0.8048f,  -0.0602f,0.6204f,-0.7820f,  -0.1469f,0.5360f,-0.8313f,  -0.1872f,0.5605f,-0.8067f,  -0.2695f,0.4656f,-0.8430f,  
	-0.3085f,0.4867f,-0.8173f,  -0.3835f,0.3823f,-0.8407f,  -0.4203f,0.3993f,-0.8148f,  -0.4853f,0.2866f,-0.8261f,  -0.5187f,0.2994f,-0.8008f,  -0.5714f,0.1798f,-0.8008f,  -0.6004f,0.1883f,-0.7772f,  -0.6387f,0.0638f,-0.7668f,  -0.6625f,0.0681f,-0.7459f,  -0.6848f,-0.0592f,-0.7264f,  -0.7027f,-0.0585f,-0.7090f,  -0.7079f,-0.1862f,-0.6814f,  -0.7195f,-0.1885f,-0.6684f,  -0.7069f,-0.3140f,-0.6338f,  -0.7119f,-0.3186f,-0.6258f,  
	-0.6816f,-0.4392f,-0.5853f,  -0.6801f,-0.4450f,-0.5826f,  -0.6326f,-0.5580f,-0.5370f,  -0.6248f,-0.5641f,-0.5399f,  -0.5613f,-0.6670f,-0.4899f,  -0.5476f,-0.6724f,-0.4981f,  -0.7077f,-0.5945f,-0.3817f,  -0.7086f,-0.5900f,-0.3869f,  -0.7938f,-0.5180f,-0.3187f,  -0.7940f,-0.5179f,-0.3184f,  -0.8648f,-0.4412f,-0.2397f,  -0.8640f,-0.4456f,-0.2345f,  -0.9191f,-0.3655f,-0.1470f,  -0.9169f,-0.3745f,-0.1376f,  -0.9554f,-0.2922f,-0.0435f,  
	-0.9516f,-0.3057f,-0.0307f,  -0.9726f,-0.2221f,0.0679f,  -0.9672f,-0.2398f,0.0834f,  -0.9705f,-0.1560f,0.1838f,  -0.9633f,-0.1774f,0.2013f,  -0.9489f,-0.0941f,0.3012f,  -0.9400f,-0.1189f,0.3198f,  -0.9083f,-0.0367f,0.4166f,  -0.8977f,-0.0642f,0.4359f,  -0.8496f,0.0164f,0.5271f,  -0.8374f,-0.0134f,0.5464f,  -0.7741f,0.0654f,0.6297f,  -0.7605f,0.0339f,0.6485f,  -0.6835f,0.1105f,0.7216f,  -0.6686f,0.0781f,0.7395f,  
	-0.5797f,0.1522f,0.8005f,  -0.5639f,0.1194f,0.8172f,  -0.4651f,0.1911f,0.8644f,  -0.4487f,0.1586f,0.8795f,  -0.3422f,0.2277f,0.9116f,  -0.3255f,0.1961f,0.9250f,  -0.2139f,0.2626f,0.9409f,  -0.1973f,0.2324f,0.9524f,  -0.0830f,0.2963f,0.9515f,  -0.0668f,0.2680f,0.9611f,  0.0475f,0.3292f,0.9431f,  0.0628f,0.3035f,0.9508f,  0.1746f,0.3619f,0.9157f,  0.1887f,0.3392f,0.9216f,  0.2955f,0.3946f,0.8700f,  
	0.3079f,0.3754f,0.8742f,  0.4073f,0.4277f,0.8070f,  0.4176f,0.4123f,0.8097f,  0.5074f,0.4611f,0.7280f,  0.5152f,0.4499f,0.7295f,  0.5934f,0.4949f,0.6348f,  0.5984f,0.4880f,0.6354f,  0.6633f,0.5289f,0.5295f,  0.6650f,0.5266f,0.5296f,  0.7153f,0.5627f,0.4144f,  0.7135f,0.5650f,0.4144f,  0.7481f,0.5958f,0.2921f,  0.7424f,0.6027f,0.2925f,  0.7608f,0.6275f,0.1654f,  0.7511f,0.6388f,0.1666f,  
	0.7531f,0.6569f,0.0371f,  0.7391f,0.6725f,0.0395f,  0.7248f,0.6830f,-0.0900f,  0.7065f,0.7025f,-0.0859f,  0.6767f,0.7047f,-0.2132f,  0.6541f,0.7276f,-0.2069f,  0.6099f,0.7206f,-0.3297f,  0.5831f,0.7463f,-0.3210f,  0.5259f,0.7295f,-0.4373f,  0.4952f,0.7572f,-0.4259f,  0.4270f,0.7298f,-0.5339f,  0.3928f,0.7589f,-0.5195f,  0.3157f,0.7203f,-0.6177f,  0.2785f,0.7498f,-0.6002f,  0.1951f,0.6996f,-0.6874f,  
	0.1555f,0.7286f,-0.6670f,  0.0685f,0.6666f,-0.7423f,  0.0275f,0.6942f,-0.7193f,  -0.0602f,0.6204f,-0.7820f,  -0.1017f,0.6457f,-0.7567f,  -0.1872f,0.5605f,-0.8067f,  -0.2283f,0.5828f,-0.7799f,  -0.3085f,0.4867f,-0.8173f,  -0.3481f,0.5053f,-0.7896f,  -0.4203f,0.3993f,-0.8148f,  -0.4572f,0.4139f,-0.7872f,  -0.5187f,0.2994f,-0.8008f,  -0.5519f,0.3096f,-0.7743f,  -0.6004f,0.1883f,-0.7772f,  -0.6290f,0.1943f,-0.7528f,  
	-0.6625f,0.0681f,-0.7459f,  -0.6856f,0.0702f,-0.7246f,  -0.7027f,-0.0585f,-0.7090f,  -0.7198f,-0.0599f,-0.6916f,  -0.7195f,-0.1885f,-0.6684f,  -0.7300f,-0.1926f,-0.6557f,  -0.7119f,-0.3186f,-0.6258f,  -0.7159f,-0.3244f,-0.6183f,  -0.6801f,-0.4450f,-0.5826f,  -0.6775f,-0.4515f,-0.5806f,  -0.6248f,-0.5641f,-0.5399f,  -0.6160f,-0.5704f,-0.5432f,  -0.5476f,-0.6724f,-0.4981f,  -0.5332f,-0.6775f,-0.5067f,  -0.7086f,-0.5900f,-0.3869f,  
	-0.7095f,-0.5858f,-0.3917f,  -0.7940f,-0.5179f,-0.3184f,  -0.7940f,-0.5182f,-0.3179f,  -0.8640f,-0.4456f,-0.2345f,  -0.8628f,-0.4505f,-0.2294f,  -0.9169f,-0.3745f,-0.1376f,  -0.9143f,-0.3839f,-0.1287f,  -0.9516f,-0.3057f,-0.0307f,  -0.9474f,-0.3195f,-0.0186f,  -0.9672f,-0.2398f,0.0834f,  -0.9613f,-0.2577f,0.0979f,  -0.9633f,-0.1774f,0.2013f,  -0.9555f,-0.1991f,0.2175f,  -0.9400f,-0.1189f,0.3198f,  -0.9304f,-0.1438f,0.3372f,  
	-0.8977f,-0.0642f,0.4359f,  -0.8864f,-0.0919f,0.4538f,  -0.8374f,-0.0134f,0.5464f,  -0.8245f,-0.0432f,0.5642f,  -0.7605f,0.0339f,0.6485f,  -0.7461f,0.0025f,0.6658f,  -0.6686f,0.0781f,0.7395f,  -0.6531f,0.0457f,0.7559f,  -0.5639f,0.1194f,0.8172f,  -0.5474f,0.0867f,0.8324f,  -0.4487f,0.1586f,0.8795f,  -0.4316f,0.1262f,0.8932f,  -0.3255f,0.1961f,0.9250f,  -0.3082f,0.1645f,0.9370f,  -0.1973f,0.2324f,0.9524f,  
	-0.1800f,0.2023f,0.9626f,  -0.0668f,0.2680f,0.9611f,  -0.0501f,0.2400f,0.9695f,  0.0628f,0.3035f,0.9508f,  0.0787f,0.2780f,0.9574f,  0.1887f,0.3392f,0.9216f,  0.2032f,0.3167f,0.9265f,  0.3079f,0.3754f,0.8742f,  0.3207f,0.3564f,0.8776f,  0.4176f,0.4123f,0.8097f,  0.4282f,0.3971f,0.8118f,  0.5152f,0.4499f,0.7295f,  0.5232f,0.4388f,0.7305f,  0.5984f,0.4880f,0.6354f,  0.6034f,0.4814f,0.6358f,  
	0.6650f,0.5266f,0.5296f,  0.6666f,0.5245f,0.5296f,  0.7135f,0.5650f,0.4144f,  0.7113f,0.5675f,0.4146f,  0.7424f,0.6027f,0.2925f,  0.7363f,0.6098f,0.2933f,  0.7511f,0.6388f,0.1666f,  0.7407f,0.6504f,0.1684f,  0.7391f,0.6725f,0.0395f,  0.7242f,0.6883f,0.0429f,  0.7065f,0.7025f,-0.0859f,  0.6872f,0.7220f,-0.0806f,  0.6541f,0.7276f,-0.2069f,  0.6303f,0.7504f,-0.1992f,  0.5831f,0.7463f,-0.3210f,  
	0.5550f,0.7717f,-0.3105f,  0.4952f,0.7572f,-0.4259f,  0.4631f,0.7845f,-0.4124f,  0.3928f,0.7589f,-0.5195f,  0.3571f,0.7872f,-0.5028f,  0.2785f,0.7498f,-0.6002f,  0.2399f,0.7782f,-0.5804f,  0.1555f,0.7286f,-0.6670f,  0.1148f,0.7561f,-0.6443f,  0.0275f,0.6942f,-0.7193f,  -0.0146f,0.7200f,-0.6938f,  -0.1017f,0.6457f,-0.7567f,  -0.1442f,0.6689f,-0.7292f,  -0.2283f,0.5828f,-0.7799f,  -0.2699f,0.6026f,-0.7510f,  
	-0.3481f,0.5053f,-0.7896f,  -0.3878f,0.5213f,-0.7602f,  -0.4572f,0.4139f,-0.7872f,  -0.4939f,0.4256f,-0.7582f,  -0.5519f,0.3096f,-0.7743f,  -0.5846f,0.3171f,-0.7468f,  -0.6290f,0.1943f,-0.7528f,  -0.6568f,0.1976f,-0.7277f,  -0.6856f,0.0702f,-0.7246f,  -0.7078f,0.0698f,-0.7030f,  -0.7198f,-0.0599f,-0.6916f,  -0.7357f,-0.0633f,-0.6744f,  -0.7300f,-0.1926f,-0.6557f,  -0.7394f,-0.1982f,-0.6434f,  -0.7159f,-0.3244f,-0.6183f,  
	-0.7187f,-0.3313f,-0.6114f,  -0.6775f,-0.4515f,-0.5806f,  -0.6739f,-0.4588f,-0.5791f,  -0.6160f,-0.5704f,-0.5432f,  -0.6064f,-0.5769f,-0.5472f,  -0.5332f,-0.6775f,-0.5067f,  -0.5183f,-0.6823f,-0.5156f,  -0.7095f,-0.5858f,-0.3917f,  -0.7105f,-0.5818f,-0.3959f,  -0.7940f,-0.5182f,-0.3179f,  -0.7939f,-0.5187f,-0.3172f,  -0.8628f,-0.4505f,-0.2294f,  -0.8614f,-0.4557f,-0.2244f,  -0.9143f,-0.3839f,-0.1287f,  -0.9114f,-0.3937f,-0.1201f,  
	-0.9474f,-0.3195f,-0.0186f,  -0.9427f,-0.3335f,-0.0071f,  -0.9613f,-0.2577f,0.0979f,  -0.9547f,-0.2758f,0.1115f,  -0.9555f,-0.1991f,0.2175f,  -0.9471f,-0.2209f,0.2328f,  -0.9304f,-0.1438f,0.3372f,  -0.9201f,-0.1688f,0.3534f,  -0.8864f,-0.0919f,0.4538f,  -0.8743f,-0.1196f,0.4703f,  -0.8245f,-0.0432f,0.5642f,  -0.8108f,-0.0730f,0.5807f,  -0.7461f,0.0025f,0.6658f,  -0.7310f,-0.0288f,0.6817f,  -0.6531f,0.0457f,0.7559f,  
	-0.6368f,0.0134f,0.7709f,  -0.5474f,0.0867f,0.8324f,  -0.5302f,0.0541f,0.8461f,  -0.4316f,0.1262f,0.8932f,  -0.4138f,0.0938f,0.9055f,  -0.3082f,0.1645f,0.9370f,  -0.2901f,0.1330f,0.9477f,  -0.1800f,0.2023f,0.9626f,  -0.1621f,0.1723f,0.9716f,  -0.0501f,0.2400f,0.9695f,  -0.0327f,0.2120f,0.9767f,  0.0787f,0.2780f,0.9574f,  0.0951f,0.2526f,0.9629f,  0.2032f,0.3167f,0.9265f,  0.2183f,0.2943f,0.9304f,  
	0.3207f,0.3564f,0.8776f,  0.3339f,0.3375f,0.8801f,  0.4282f,0.3971f,0.8118f,  0.4391f,0.3821f,0.8132f,  0.5232f,0.4388f,0.7305f,  0.5314f,0.4280f,0.7311f,  0.6034f,0.4814f,0.6358f,  0.6084f,0.4750f,0.6358f,  0.6666f,0.5245f,0.5296f,  0.6681f,0.5227f,0.5296f,  0.7113f,0.5675f,0.4146f,  0.7089f,0.5704f,0.4149f,  0.7363f,0.6098f,0.2933f,  0.7296f,0.6172f,0.2944f,  0.7407f,0.6504f,0.1684f,  
	0.7295f,0.6622f,0.1709f,  0.7242f,0.6883f,0.0429f,  0.7085f,0.7042f,0.0472f,  0.6872f,0.7220f,-0.0806f,  0.6668f,0.7416f,-0.0740f,  0.6303f,0.7504f,-0.1992f,  0.6053f,0.7730f,-0.1899f,  0.5550f,0.7717f,-0.3105f,  0.5256f,0.7967f,-0.2982f,  0.4631f,0.7845f,-0.4124f,  0.4297f,0.8111f,-0.3968f,  0.3571f,0.7872f,-0.5028f,  0.3202f,0.8144f,-0.4839f,  0.2399f,0.7782f,-0.5804f,  0.2001f,0.8052f,-0.5583f,  
	0.1148f,0.7561f,-0.6443f,  0.0729f,0.7819f,-0.6191f,  -0.0146f,0.7200f,-0.6938f,  -0.0576f,0.7437f,-0.6661f,  -0.1442f,0.6689f,-0.7292f,  -0.1872f,0.6897f,-0.6995f,  -0.2699f,0.6026f,-0.7510f,  -0.3119f,0.6198f,-0.7201f,  -0.3878f,0.5213f,-0.7602f,  -0.4276f,0.5344f,-0.7291f,  -0.4939f,0.4256f,-0.7582f,  -0.5304f,0.4345f,-0.7280f,  -0.5846f,0.3171f,-0.7468f,  -0.6167f,0.3217f,-0.7185f,  -0.6568f,0.1976f,-0.7277f,  
	-0.6836f,0.1983f,-0.7024f,  -0.7078f,0.0698f,-0.7030f,  -0.7287f,0.0670f,-0.6815f,  -0.7357f,-0.0633f,-0.6744f,  -0.7503f,-0.0687f,-0.6575f,  -0.7394f,-0.1982f,-0.6434f,  -0.7475f,-0.2054f,-0.6317f,  -0.7187f,-0.3313f,-0.6114f,  -0.7202f,-0.3392f,-0.6051f,  -0.6739f,-0.4588f,-0.5791f,  -0.6692f,-0.4665f,-0.5784f,  -0.6064f,-0.5769f,-0.5472f,  -0.5960f,-0.5834f,-0.5517f,  -0.5183f,-0.6823f,-0.5156f,  -0.5026f,-0.6869f,-0.5250f,  
	-0.7105f,-0.5818f,-0.3959f,  -0.7115f,-0.5781f,-0.3996f,  -0.7939f,-0.5187f,-0.3172f,  -0.7937f,-0.5196f,-0.3163f,  -0.8614f,-0.4557f,-0.2244f,  -0.8597f,-0.4611f,-0.2196f,  -0.9114f,-0.3937f,-0.1201f,  -0.9080f,-0.4037f,-0.1120f,  -0.9427f,-0.3335f,-0.0071f,  -0.9375f,-0.3479f,0.0037f,  -0.9547f,-0.2758f,0.1115f,  -0.9476f,-0.2942f,0.1243f,  -0.9471f,-0.2209f,0.2328f,  -0.9381f,-0.2428f,0.2469f,  -0.9201f,-0.1688f,0.3534f,  
	-0.9092f,-0.1939f,0.3684f,  -0.8743f,-0.1196f,0.4703f,  -0.8616f,-0.1473f,0.4857f,  -0.8108f,-0.0730f,0.5807f,  -0.7965f,-0.1028f,0.5959f,  -0.7310f,-0.0288f,0.6817f,  -0.7152f,-0.0602f,0.6963f,  -0.6368f,0.0134f,0.7709f,  -0.6198f,-0.0189f,0.7846f,  -0.5302f,0.0541f,0.8461f,  -0.5123f,0.0215f,0.8586f,  -0.4138f,0.0938f,0.9055f,  -0.3952f,0.0615f,0.9165f,  -0.2901f,0.1330f,0.9477f,  -0.2713f,0.1016f,0.9571f,  
	-0.1621f,0.1723f,0.9716f,  -0.1435f,0.1423f,0.9794f,  -0.0327f,0.2120f,0.9767f,  -0.0147f,0.1841f,0.9828f,  0.0951f,0.2526f,0.9629f,  0.1121f,0.2273f,0.9674f,  0.2183f,0.2943f,0.9304f,  0.2338f,0.2721f,0.9334f,  0.3339f,0.3375f,0.8801f,  0.3475f,0.3188f,0.8819f,  0.4391f,0.3821f,0.8132f,  0.4503f,0.3672f,0.8139f,  0.5314f,0.4280f,0.7311f,  0.5397f,0.4174f,0.7311f,  0.6084f,0.4750f,0.6358f,  
	0.6134f,0.4689f,0.6355f,  0.6681f,0.5227f,0.5296f,  0.6694f,0.5212f,0.5295f,  0.7089f,0.5704f,0.4149f,  0.7061f,0.5735f,0.4154f,  0.7296f,0.6172f,0.2944f,  0.7224f,0.6249f,0.2960f,  0.7295f,0.6622f,0.1709f,  0.7177f,0.6743f,0.1741f,  0.7085f,0.7042f,0.0472f,  0.6918f,0.7202f,0.0525f,  0.6668f,0.7416f,-0.0740f,  0.6453f,0.7611f,-0.0660f,  0.6053f,0.7730f,-0.1899f,  0.5791f,0.7953f,-0.1790f,  
	0.5256f,0.7967f,-0.2982f,  0.4949f,0.8212f,-0.2840f,  0.4297f,0.8111f,-0.3968f,  0.3950f,0.8368f,-0.3791f,  0.3202f,0.8144f,-0.4839f,  0.2819f,0.8405f,-0.4627f,  0.2001f,0.8052f,-0.5583f,  0.1591f,0.8306f,-0.5337f,  0.0729f,0.7819f,-0.6191f,  0.0300f,0.8057f,-0.5915f,  -0.0576f,0.7437f,-0.6661f,  -0.1013f,0.7649f,-0.6361f,  -0.1872f,0.6897f,-0.6995f,  -0.2307f,0.7077f,-0.6678f,  -0.3119f,0.6198f,-0.7201f,  
	-0.3539f,0.6340f,-0.6876f,  -0.4276f,0.5344f,-0.7291f,  -0.4670f,0.5444f,-0.6968f,  -0.5304f,0.4345f,-0.7280f,  -0.5661f,0.4402f,-0.6969f,  -0.6167f,0.3217f,-0.7185f,  -0.6478f,0.3233f,-0.6897f,  -0.6836f,0.1983f,-0.7024f,  -0.7093f,0.1962f,-0.6770f,  -0.7287f,0.0670f,-0.6815f,  -0.7484f,0.0620f,-0.6604f,  -0.7503f,-0.0687f,-0.6575f,  -0.7636f,-0.0760f,-0.6413f,  -0.7475f,-0.2054f,-0.6317f,  -0.7542f,-0.2140f,-0.6208f,  
	-0.7202f,-0.3392f,-0.6051f,  -0.7205f,-0.3481f,-0.5997f,  -0.6692f,-0.4665f,-0.5784f,  -0.6634f,-0.4746f,-0.5784f,  -0.5960f,-0.5834f,-0.5517f,  -0.5846f,-0.5900f,-0.5569f,  -0.5026f,-0.6869f,-0.5250f,  -0.4864f,-0.6910f,-0.5347f,  -0.7115f,-0.5781f,-0.3996f,  -0.7125f,-0.5746f,-0.4028f,  -0.7937f,-0.5196f,-0.3163f,  -0.7934f,-0.5207f,-0.3153f,  -0.8597f,-0.4611f,-0.2196f,  -0.8578f,-0.4669f,-0.2149f,  -0.9080f,-0.4037f,-0.1120f,  
	-0.9043f,-0.4140f,-0.1043f,  -0.9375f,-0.3479f,0.0037f,  -0.9319f,-0.3624f,0.0138f,  -0.9476f,-0.2942f,0.1243f,  -0.9400f,-0.3127f,0.1363f,  -0.9381f,-0.2428f,0.2469f,  -0.9285f,-0.2649f,0.2601f,  -0.9092f,-0.1939f,0.3684f,  -0.8977f,-0.2191f,0.3823f,  -0.8616f,-0.1473f,0.4857f,  -0.8483f,-0.1751f,0.4998f,  -0.7965f,-0.1028f,0.5959f,  -0.7814f,-0.1327f,0.6097f,  -0.7152f,-0.0602f,0.6963f,  -0.6987f,-0.0915f,0.7095f,  
	-0.6198f,-0.0189f,0.7846f,  -0.6020f,-0.0511f,0.7969f,  -0.5123f,0.0215f,0.8586f,  -0.4936f,-0.0110f,0.8696f,  -0.3952f,0.0615f,0.9165f,  -0.3759f,0.0293f,0.9262f,  -0.2713f,0.1016f,0.9571f,  -0.2518f,0.0703f,0.9652f,  -0.1435f,0.1423f,0.9794f,  -0.1242f,0.1125f,0.9859f,  -0.0147f,0.1841f,0.9828f,  0.0040f,0.1563f,0.9877f,  0.1121f,0.2273f,0.9674f,  0.1297f,0.2020f,0.9707f,  0.2338f,0.2721f,0.9334f,  
	0.2499f,0.2500f,0.9355f,  0.3475f,0.3188f,0.8819f,  0.3615f,0.3002f,0.8827f,  0.4503f,0.3672f,0.8139f,  0.4618f,0.3526f,0.8139f,  0.5397f,0.4174f,0.7311f,  0.5482f,0.4070f,0.7306f,  0.6134f,0.4689f,0.6355f,  0.6184f,0.4630f,0.6350f,  0.6694f,0.5212f,0.5295f,  0.6705f,0.5199f,0.5293f,  0.7061f,0.5735f,0.4154f,  0.7029f,0.5769f,0.4161f,  0.7224f,0.6249f,0.2960f,  0.7146f,0.6328f,0.2981f,  
	0.7177f,0.6743f,0.1741f,  0.7051f,0.6864f,0.1781f,  0.6918f,0.7202f,0.0525f,  0.6742f,0.7362f,0.0590f,  0.6453f,0.7611f,-0.0660f,  0.6227f,0.7804f,-0.0567f,  0.5791f,0.7953f,-0.1790f,  0.5517f,0.8173f,-0.1664f,  0.4949f,0.8212f,-0.2840f,  0.4630f,0.8449f,-0.2679f,  0.3950f,0.8368f,-0.3791f,  0.3590f,0.8614f,-0.3593f,  0.2819f,0.8405f,-0.4627f,  0.2425f,0.8650f,-0.4393f,  0.1591f,0.8306f,-0.5337f,  
	0.1171f,0.8540f,-0.5069f,  0.0300f,0.8057f,-0.5915f,  -0.0136f,0.8272f,-0.5618f,  -0.1013f,0.7649f,-0.6361f,  -0.1455f,0.7835f,-0.6041f,  -0.2307f,0.7077f,-0.6678f,  -0.2742f,0.7228f,-0.6343f,  -0.3539f,0.6340f,-0.6876f,  -0.3957f,0.6451f,-0.6537f,  -0.4670f,0.5444f,-0.6968f,  -0.5059f,0.5512f,-0.6635f,  -0.5661f,0.4402f,-0.6969f,  -0.6010f,0.4429f,-0.6653f,  -0.6478f,0.3233f,-0.6897f,  -0.6778f,0.3221f,-0.6610f,  
	-0.7093f,0.1962f,-0.6770f,  -0.7336f,0.1916f,-0.6520f,  -0.7484f,0.0620f,-0.6604f,  -0.7666f,0.0547f,-0.6398f,  -0.7636f,-0.0760f,-0.6413f,  -0.7753f,-0.0850f,-0.6258f,  -0.7542f,-0.2140f,-0.6208f,  -0.7595f,-0.2238f,-0.6108f,  -0.7205f,-0.3481f,-0.5997f,  -0.7196f,-0.3577f,-0.5952f,  -0.6634f,-0.4746f,-0.5784f,  -0.6565f,-0.4831f,-0.5793f,  -0.5846f,-0.5900f,-0.5569f,  -0.5724f,-0.5965f,-0.5626f,  -0.4864f,-0.6910f,-0.5347f,  
	-0.4696f,-0.6948f,-0.5447f,  -0.7125f,-0.5746f,-0.4028f,  -0.7134f,-0.5714f,-0.4056f,  -0.7934f,-0.5207f,-0.3153f,  -0.7929f,-0.5222f,-0.3142f,  -0.8578f,-0.4669f,-0.2149f,  -0.8556f,-0.4730f,-0.2104f,  -0.9043f,-0.4140f,-0.1043f,  -0.9002f,-0.4245f,-0.0970f,  -0.9319f,-0.3624f,0.0138f,  -0.9259f,-0.3772f,0.0233f,  -0.9400f,-0.3127f,0.1363f,  -0.9319f,-0.3313f,0.1474f,  -0.9285f,-0.2649f,0.2601f,  -0.9184f,-0.2871f,0.2724f,  
	-0.8977f,-0.2191f,0.3823f,  -0.8855f,-0.2443f,0.3951f,  -0.8483f,-0.1751f,0.4998f,  -0.8342f,-0.2029f,0.5127f,  -0.7814f,-0.1327f,0.6097f,  -0.7657f,-0.1626f,0.6224f,  -0.6987f,-0.0915f,0.7095f,  -0.6814f,-0.1229f,0.7215f,  -0.6020f,-0.0511f,0.7969f,  -0.5835f,-0.0834f,0.8078f,  -0.4936f,-0.0110f,0.8696f,  -0.4741f,-0.0436f,0.8794f,  -0.3759f,0.0293f,0.9262f,  -0.3559f,-0.0030f,0.9345f,  -0.2518f,0.0703f,0.9652f,  
	-0.2316f,0.0389f,0.9720f,  -0.1242f,0.1125f,0.9859f,  -0.1041f,0.0826f,0.9911f,  0.0040f,0.1563f,0.9877f,  0.0234f,0.1285f,0.9914f,  0.1297f,0.2020f,0.9707f,  0.1480f,0.1769f,0.9730f,  0.2499f,0.2500f,0.9355f,  0.2665f,0.2280f,0.9365f,  0.3615f,0.3002f,0.8827f,  0.3759f,0.2818f,0.8828f,  0.4618f,0.3526f,0.8139f,  0.4736f,0.3382f,0.8132f,  0.5482f,0.4070f,0.7306f,  0.5568f,0.3969f,0.7297f,  
	0.6184f,0.4630f,0.6350f,  0.6234f,0.4574f,0.6341f,  0.6705f,0.5199f,0.5293f,  0.6714f,0.5190f,0.5291f,  0.7029f,0.5769f,0.4161f,  0.6993f,0.5805f,0.4170f,  0.7146f,0.6328f,0.2981f,  0.7063f,0.6409f,0.3007f,  0.7051f,0.6864f,0.1781f,  0.6917f,0.6986f,0.1830f,  0.6742f,0.7362f,0.0590f,  0.6557f,0.7521f,0.0666f,  0.6227f,0.7804f,-0.0567f,  0.5991f,0.7994f,-0.0459f,  0.5517f,0.8173f,-0.1664f,  
	0.5232f,0.8386f,-0.1520f,  0.4630f,0.8449f,-0.2679f,  0.4299f,0.8677f,-0.2497f,  0.3590f,0.8614f,-0.3593f,  0.3218f,0.8847f,-0.3374f,  0.2425f,0.8650f,-0.4393f,  0.2021f,0.8877f,-0.4136f,  0.1171f,0.8540f,-0.5069f,  0.0742f,0.8753f,-0.4779f,  -0.0136f,0.8272f,-0.5618f,  -0.0578f,0.8460f,-0.5300f,  -0.1455f,0.7835f,-0.6041f,  -0.1899f,0.7992f,-0.5702f,  -0.2742f,0.7228f,-0.6343f,  -0.3177f,0.7347f,-0.5994f,  
	-0.3957f,0.6451f,-0.6537f,  -0.4370f,0.6529f,-0.6187f,  -0.5059f,0.5512f,-0.6635f,  -0.5439f,0.5548f,-0.6296f,  -0.6010f,0.4429f,-0.6653f,  -0.6347f,0.4424f,-0.6336f,  -0.6778f,0.3221f,-0.6610f,  -0.7064f,0.3179f,-0.6324f,  -0.7336f,0.1916f,-0.6520f,  -0.7564f,0.1845f,-0.6275f,  -0.7666f,0.0547f,-0.6398f,  -0.7831f,0.0454f,-0.6202f,  -0.7753f,-0.0850f,-0.6258f,  -0.7855f,-0.0956f,-0.6114f,  -0.7595f,-0.2238f,-0.6108f,  
	-0.7634f,-0.2346f,-0.6019f,  -0.7196f,-0.3577f,-0.5952f,  -0.7173f,-0.3680f,-0.5917f,  -0.6565f,-0.4831f,-0.5793f,  -0.6486f,-0.4918f,-0.5809f,  -0.5724f,-0.5965f,-0.5626f,  -0.5595f,-0.6028f,-0.5689f,  -0.4696f,-0.6948f,-0.5447f,  -0.4524f,-0.6981f,-0.5550f,  -0.7134f,-0.5714f,-0.4056f,  -0.7144f,-0.5685f,-0.4080f,  -0.7929f,-0.5222f,-0.3142f,  -0.7922f,-0.5239f,-0.3129f,  -0.8556f,-0.4730f,-0.2104f,  -0.8531f,-0.4793f,-0.2061f,  
	-0.9002f,-0.4245f,-0.0970f,  -0.8958f,-0.4352f,-0.0900f,  -0.9259f,-0.3772f,0.0233f,  -0.9194f,-0.3921f,0.0322f,  -0.9319f,-0.3313f,0.1474f,  -0.9233f,-0.3502f,0.1578f,  -0.9184f,-0.2871f,0.2724f,  -0.9076f,-0.3094f,0.2837f,  -0.8855f,-0.2443f,0.3951f,  -0.8727f,-0.2697f,0.4069f,  -0.8342f,-0.2029f,0.5127f,  -0.8195f,-0.2309f,0.5245f,  -0.7657f,-0.1626f,0.6224f,  -0.7492f,-0.1926f,0.6338f,  -0.6814f,-0.1229f,0.7215f,  
	-0.6634f,-0.1543f,0.7322f,  -0.5835f,-0.0834f,0.8078f,  -0.5642f,-0.1157f,0.8175f,  -0.4741f,-0.0436f,0.8794f,  -0.4538f,-0.0762f,0.8878f,  -0.3559f,-0.0030f,0.9345f,  -0.3350f,-0.0353f,0.9415f,  -0.2316f,0.0389f,0.9720f,  -0.2105f,0.0076f,0.9776f,  -0.1041f,0.0826f,0.9911f,  -0.0833f,0.0528f,0.9951f,  0.0234f,0.1285f,0.9914f,  0.0435f,0.1008f,0.9940f,  0.1480f,0.1769f,0.9730f,  0.1669f,0.1519f,0.9742f,  
	0.2665f,0.2280f,0.9365f,  0.2837f,0.2061f,0.9365f,  0.3759f,0.2818f,0.8828f,  0.3909f,0.2636f,0.8819f,  0.4736f,0.3382f,0.8132f,  0.4857f,0.3241f,0.8118f,  0.5568f,0.3969f,0.7297f,  0.5656f,0.3871f,0.7281f,  0.6234f,0.4574f,0.6341f,  0.6284f,0.4522f,0.6330f,  0.6714f,0.5190f,0.5291f,  0.6721f,0.5183f,0.5288f,  0.6993f,0.5805f,0.4170f,  0.6954f,0.5844f,0.4182f,  0.7063f,0.6409f,0.3007f,  
	0.6973f,0.6492f,0.3039f,  0.6917f,0.6986f,0.1830f,  0.6775f,0.7109f,0.1888f,  0.6557f,0.7521f,0.0666f,  0.6362f,0.7678f,0.0754f,  0.5991f,0.7994f,-0.0459f,  0.5744f,0.8179f,-0.0335f,  0.5232f,0.8386f,-0.1520f,  0.4935f,0.8591f,-0.1359f,  0.4299f,0.8677f,-0.2497f,  0.3957f,0.8892f,-0.2296f,  0.3218f,0.8847f,-0.3374f,  0.2837f,0.9063f,-0.3133f,  0.2021f,0.8877f,-0.4136f,  0.1608f,0.9084f,-0.3859f,  
	0.0742f,0.8753f,-0.4779f,  0.0308f,0.8940f,-0.4469f,  -0.0578f,0.8460f,-0.5300f,  -0.1023f,0.8620f,-0.4964f,  -0.1899f,0.7992f,-0.5702f,  -0.2342f,0.8118f,-0.5349f,  -0.3177f,0.7347f,-0.5994f,  -0.3607f,0.7433f,-0.5634f,  -0.4370f,0.6529f,-0.6187f,  -0.4775f,0.6573f,-0.5830f,  -0.5439f,0.5548f,-0.6296f,  -0.5808f,0.5551f,-0.5954f,  -0.6347f,0.4424f,-0.6336f,  -0.6671f,0.4388f,-0.6021f,  -0.7064f,0.3179f,-0.6324f,  
	-0.7334f,0.3110f,-0.6045f,  -0.7564f,0.1845f,-0.6275f,  -0.7776f,0.1750f,-0.6040f,  -0.7831f,0.0454f,-0.6202f,  -0.7980f,0.0342f,-0.6016f,  -0.7855f,-0.0956f,-0.6114f,  -0.7941f,-0.1076f,-0.5982f,  -0.7634f,-0.2346f,-0.6019f,  -0.7657f,-0.2465f,-0.5941f,  -0.7173f,-0.3680f,-0.5917f,  -0.7137f,-0.3787f,-0.5892f,  -0.6486f,-0.4918f,-0.5809f,  -0.6397f,-0.5005f,-0.5833f,  -0.5595f,-0.6028f,-0.5689f,  -0.5458f,-0.6088f,-0.5757f,  
	-0.4524f,-0.6981f,-0.5550f,  -0.4346f,-0.7009f,-0.5655f,  -0.7144f,-0.5685f,-0.4080f,  -0.7153f,-0.5658f,-0.4101f,  -0.7922f,-0.5239f,-0.3129f,  -0.7915f,-0.5258f,-0.3116f,  -0.8531f,-0.4793f,-0.2061f,  -0.8504f,-0.4858f,-0.2018f,  -0.8958f,-0.4352f,-0.0900f,  -0.8910f,-0.4462f,-0.0835f,  -0.9194f,-0.3921f,0.0322f,  -0.9124f,-0.4073f,0.0406f,  -0.9233f,-0.3502f,0.1578f,  -0.9142f,-0.3691f,0.1675f,  -0.9076f,-0.3094f,0.2837f,  
	-0.8963f,-0.3319f,0.2942f,  -0.8727f,-0.2697f,0.4069f,  -0.8593f,-0.2952f,0.4177f,  -0.8195f,-0.2309f,0.5245f,  -0.8041f,-0.2589f,0.5352f,  -0.7492f,-0.1926f,0.6338f,  -0.7319f,-0.2226f,0.6440f,  -0.6634f,-0.1543f,0.7322f,  -0.6445f,-0.1859f,0.7416f,  -0.5642f,-0.1157f,0.8175f,  -0.5440f,-0.1481f,0.8259f,  -0.4538f,-0.0762f,0.8878f,  -0.4327f,-0.1089f,0.8949f,  -0.3350f,-0.0353f,0.9415f,  -0.3133f,-0.0676f,0.9472f,  
	-0.2105f,0.0076f,0.9776f,  -0.1886f,-0.0238f,0.9818f,  -0.0833f,0.0528f,0.9951f,  -0.0616f,0.0230f,0.9978f,  0.0435f,0.1008f,0.9940f,  0.0644f,0.0731f,0.9952f,  0.1669f,0.1519f,0.9742f,  0.1865f,0.1269f,0.9742f,  0.2837f,0.2061f,0.9365f,  0.3015f,0.1844f,0.9355f,  0.3909f,0.2636f,0.8819f,  0.4063f,0.2456f,0.8801f,  0.4857f,0.3241f,0.8118f,  0.4982f,0.3101f,0.8097f,  0.5656f,0.3871f,0.7281f,  
	0.5746f,0.3776f,0.7261f,  0.6284f,0.4522f,0.6330f,  0.6334f,0.4472f,0.6315f,  0.6721f,0.5183f,0.5288f,  0.6726f,0.5179f,0.5285f,  0.6954f,0.5844f,0.4182f,  0.6910f,0.5886f,0.4197f,  0.6973f,0.6492f,0.3039f,  0.6877f,0.6576f,0.3077f,  0.6775f,0.7109f,0.1888f,  0.6625f,0.7231f,0.1955f,  0.6362f,0.7678f,0.0754f,  0.6158f,0.7832f,0.0855f,  0.5744f,0.8179f,-0.0335f,  0.5487f,0.8358f,-0.0197f,  
	0.4935f,0.8591f,-0.1359f,  0.4628f,0.8786f,-0.1180f,  0.3957f,0.8892f,-0.2296f,  0.3605f,0.9094f,-0.2076f,  0.2837f,0.9063f,-0.3133f,  0.2447f,0.9261f,-0.2873f,  0.1608f,0.9084f,-0.3859f,  0.1189f,0.9268f,-0.3562f,  0.0308f,0.8940f,-0.4469f,  -0.0129f,0.9101f,-0.4141f,  -0.1023f,0.8620f,-0.4964f,  -0.1468f,0.8750f,-0.4613f,  -0.2342f,0.8118f,-0.5349f,  -0.2782f,0.8211f,-0.4984f,  -0.3607f,0.7433f,-0.5634f,  
	-0.4030f,0.7485f,-0.5266f,  -0.4775f,0.6573f,-0.5830f,  -0.5170f,0.6584f,-0.5471f,  -0.5808f,0.5551f,-0.5954f,  -0.6164f,0.5522f,-0.5614f,  -0.6671f,0.4388f,-0.6021f,  -0.6979f,0.4323f,-0.5711f,  -0.7334f,0.3110f,-0.6045f,  -0.7587f,0.3015f,-0.5774f,  -0.7776f,0.1750f,-0.6040f,  -0.7970f,0.1633f,-0.5815f,  -0.7980f,0.0342f,-0.6016f,  -0.8112f,0.0213f,-0.5843f,  -0.7941f,-0.1076f,-0.5982f,  -0.8010f,-0.1209f,-0.5863f,  
	-0.7657f,-0.2465f,-0.5941f,  -0.7666f,-0.2591f,-0.5875f,  -0.7137f,-0.3787f,-0.5892f,  -0.7090f,-0.3896f,-0.5878f,  -0.6397f,-0.5005f,-0.5833f,  -0.6299f,-0.5092f,-0.5865f,  -0.5458f,-0.6088f,-0.5757f,  -0.5313f,-0.6146f,-0.5830f,  -0.4346f,-0.7009f,-0.5655f,  -0.4167f,-0.7032f,-0.5762f,  -0.7153f,-0.5658f,-0.4101f,  -0.7162f,-0.5634f,-0.4118f,  -0.7915f,-0.5258f,-0.3116f,  -0.7905f,-0.5281f,-0.3102f,  -0.8504f,-0.4858f,-0.2018f,  
	-0.8475f,-0.4926f,-0.1978f,  -0.8910f,-0.4462f,-0.0835f,  -0.8859f,-0.4574f,-0.0773f,  -0.9124f,-0.4073f,0.0406f,  -0.9050f,-0.4226f,0.0484f,  -0.9142f,-0.3691f,0.1675f,  -0.9045f,-0.3883f,0.1764f,  -0.8963f,-0.3319f,0.2942f,  -0.8844f,-0.3545f,0.3037f,  -0.8593f,-0.2952f,0.4177f,  -0.8452f,-0.3208f,0.4275f,  -0.8041f,-0.2589f,0.5352f,  -0.7879f,-0.2871f,0.5448f,  -0.7319f,-0.2226f,0.6440f,  -0.7139f,-0.2528f,0.6530f,  
	-0.6445f,-0.1859f,0.7416f,  -0.6249f,-0.2175f,0.7498f,  -0.5440f,-0.1481f,0.8259f,  -0.5230f,-0.1807f,0.8330f,  -0.4327f,-0.1089f,0.8949f,  -0.4107f,-0.1417f,0.9007f,  -0.3133f,-0.0676f,0.9472f,  -0.2907f,-0.1001f,0.9516f,  -0.1886f,-0.0238f,0.9818f,  -0.1658f,-0.0552f,0.9846f,  -0.0616f,0.0230f,0.9978f,  -0.0391f,-0.0068f,0.9992f,  0.0644f,0.0731f,0.9952f,  0.0862f,0.0455f,0.9952f,  0.1865f,0.1269f,0.9742f,  
	0.2069f,0.1021f,0.9730f,  0.3015f,0.1844f,0.9355f,  0.3199f,0.1629f,0.9333f,  0.4063f,0.2456f,0.8801f,  0.4222f,0.2279f,0.8774f,  0.4982f,0.3101f,0.8097f,  0.5110f,0.2966f,0.8068f,  0.5746f,0.3776f,0.7261f,  0.5838f,0.3684f,0.7235f,  0.6334f,0.4472f,0.6315f,  0.6383f,0.4426f,0.6298f,  0.6726f,0.5179f,0.5285f,  0.6729f,0.5179f,0.5282f,  0.6910f,0.5886f,0.4197f,  0.6861f,0.5930f,0.4214f,  
	0.6877f,0.6576f,0.3077f,  0.6775f,0.6660f,0.3121f,  0.6625f,0.7231f,0.1955f,  0.6467f,0.7352f,0.2031f,  0.6158f,0.7832f,0.0855f,  0.5945f,0.7983f,0.0968f,  0.5487f,0.8358f,-0.0197f,  0.5219f,0.8530f,-0.0043f,  0.4628f,0.8786f,-0.1180f,  0.4311f,0.8969f,-0.0983f,  0.3605f,0.9094f,-0.2076f,  0.3244f,0.9279f,-0.1837f,  0.2447f,0.9261f,-0.2873f,  0.2050f,0.9438f,-0.2594f,  0.1189f,0.9268f,-0.3562f,  
	0.0766f,0.9427f,-0.3247f,  -0.0129f,0.9101f,-0.4141f,  -0.0568f,0.9233f,-0.3798f,  -0.1468f,0.8750f,-0.4613f,  -0.1910f,0.8848f,-0.4250f,  -0.2782f,0.8211f,-0.4984f,  -0.3216f,0.8270f,-0.4611f,  -0.4030f,0.7485f,-0.5266f,  -0.4443f,0.7504f,-0.4894f,  -0.5170f,0.6584f,-0.5471f,  -0.5551f,0.6561f,-0.5112f,  -0.6164f,0.5522f,-0.5614f,  -0.6504f,0.5461f,-0.5279f,  -0.6979f,0.4323f,-0.5711f,  -0.7269f,0.4230f,-0.5410f,  
	-0.7587f,0.3015f,-0.5774f,  -0.7822f,0.2897f,-0.5515f,  -0.7970f,0.1633f,-0.5815f,  -0.8145f,0.1498f,-0.5605f,  -0.8112f,0.0213f,-0.5843f,  -0.8226f,0.0068f,-0.5685f,  -0.8010f,-0.1209f,-0.5863f,  -0.8064f,-0.1350f,-0.5758f,  -0.7666f,-0.2591f,-0.5875f,  -0.7661f,-0.2721f,-0.5822f,  -0.7090f,-0.3896f,-0.5878f,  -0.7031f,-0.4008f,-0.5874f,  -0.6299f,-0.5092f,-0.5865f,  -0.6190f,-0.5177f,-0.5906f,  -0.5313f,-0.6146f,-0.5830f,  
	-0.5165f,-0.6199f,-0.5908f,  -0.4167f,-0.7032f,-0.5762f,  -0.3982f,-0.7049f,-0.5870f,  -0.7162f,-0.5634f,-0.4118f,  -0.7171f,-0.5613f,-0.4132f,  -0.7905f,-0.5281f,-0.3102f,  -0.7894f,-0.5306f,-0.3087f,  -0.8475f,-0.4926f,-0.1978f,  -0.8442f,-0.4997f,-0.1939f,  -0.8859f,-0.4574f,-0.0773f,  -0.8804f,-0.4688f,-0.0716f,  -0.9050f,-0.4226f,0.0484f,  -0.8972f,-0.4381f,0.0556f,  -0.9045f,-0.3883f,0.1764f,  -0.8943f,-0.4077f,0.1845f,  
	-0.8844f,-0.3545f,0.3037f,  -0.8718f,-0.3773f,0.3124f,  -0.8452f,-0.3208f,0.4275f,  -0.8304f,-0.3466f,0.4362f,  -0.7879f,-0.2871f,0.5448f,  -0.7710f,-0.3154f,0.5533f,  -0.7139f,-0.2528f,0.6530f,  -0.6950f,-0.2832f,0.6609f,  -0.6249f,-0.2175f,0.7498f,  -0.6043f,-0.2493f,0.7567f,  -0.5230f,-0.1807f,0.8330f,  -0.5011f,-0.2133f,0.8387f,  -0.4107f,-0.1417f,0.9007f,  -0.3877f,-0.1746f,0.9051f,  -0.2907f,-0.1001f,0.9516f,  
	-0.2671f,-0.1325f,0.9545f,  -0.1658f,-0.0552f,0.9846f,  -0.1420f,-0.0867f,0.9861f,  -0.0391f,-0.0068f,0.9992f,  -0.0156f,-0.0366f,0.9992f,  0.0862f,0.0455f,0.9952f,  0.1088f,0.0180f,0.9939f,  0.2069f,0.1021f,0.9730f,  0.2280f,0.0774f,0.9706f,  0.3199f,0.1629f,0.9333f,  0.3390f,0.1416f,0.9300f,  0.4222f,0.2279f,0.8774f,  0.4387f,0.2104f,0.8737f,  0.5110f,0.2966f,0.8068f,  0.5242f,0.2833f,0.8031f,  
	0.5838f,0.3684f,0.7235f,  0.5931f,0.3596f,0.7203f,  0.6383f,0.4426f,0.6298f,  0.6432f,0.4384f,0.6278f,  0.6729f,0.5179f,0.5282f,  0.6729f,0.5182f,0.5279f,  0.6861f,0.5930f,0.4214f,  0.6808f,0.5976f,0.4235f,  0.6775f,0.6660f,0.3121f,  0.6666f,0.6746f,0.3172f,  0.6467f,0.7352f,0.2031f,  0.6301f,0.7471f,0.2117f,  0.5945f,0.7983f,0.0968f,  0.5722f,0.8128f,0.1095f,  0.5219f,0.8530f,-0.0043f,  
	0.4943f,0.8692f,0.0126f,  0.4311f,0.8969f,-0.0983f,  0.3985f,0.9139f,-0.0770f,  0.3244f,0.9279f,-0.1837f,  0.2876f,0.9446f,-0.1581f,  0.2050f,0.9438f,-0.2594f,  0.1649f,0.9592f,-0.2298f,  0.0766f,0.9427f,-0.3247f,  0.0341f,0.9559f,-0.2918f,  -0.0568f,0.9233f,-0.3798f,  -0.1005f,0.9335f,-0.3442f,  -0.1910f,0.8848f,-0.4250f,  -0.2347f,0.8914f,-0.3878f,  -0.3216f,0.8270f,-0.4611f,  -0.3641f,0.8296f,-0.4234f,  
	-0.4443f,0.7504f,-0.4894f,  -0.4844f,0.7489f,-0.4523f,  -0.5551f,0.6561f,-0.5112f,  -0.5918f,0.6507f,-0.4758f,  -0.6504f,0.5461f,-0.5279f,  -0.6827f,0.5371f,-0.4953f,  -0.7269f,0.4230f,-0.5410f,  -0.7542f,0.4111f,-0.5121f,  -0.7822f,0.2897f,-0.5515f,  -0.8039f,0.2757f,-0.5271f,  -0.8145f,0.1498f,-0.5605f,  -0.8302f,0.1344f,-0.5410f,  -0.8226f,0.0068f,-0.5685f,  -0.8323f,-0.0087f,-0.5542f,  -0.8064f,-0.1350f,-0.5758f,  
	-0.8101f,-0.1500f,-0.5667f,  -0.7661f,-0.2721f,-0.5822f,  -0.7643f,-0.2856f,-0.5782f,  -0.7031f,-0.4008f,-0.5874f,  -0.6960f,-0.4120f,-0.5881f,  -0.6190f,-0.5177f,-0.5906f,  -0.6074f,-0.5260f,-0.5953f,  -0.5165f,-0.6199f,-0.5908f,  -0.5010f,-0.6247f,-0.5990f,  -0.3982f,-0.7049f,-0.5870f,  -0.3796f,-0.7060f,-0.5978f,  -0.7171f,-0.5613f,-0.4132f,  -0.7178f,-0.5595f,-0.4143f,  -0.7894f,-0.5306f,-0.3087f,  -0.7881f,-0.5334f,-0.3072f,  
	-0.8442f,-0.4997f,-0.1939f,  -0.8407f,-0.5070f,-0.1902f,  -0.8804f,-0.4688f,-0.0716f,  -0.8745f,-0.4804f,-0.0662f,  -0.8972f,-0.4381f,0.0556f,  -0.8889f,-0.4539f,0.0622f,  -0.8943f,-0.4077f,0.1845f,  -0.8835f,-0.4272f,0.1920f,  -0.8718f,-0.3773f,0.3124f,  -0.8586f,-0.4003f,0.3202f,  -0.8304f,-0.3466f,0.4362f,  -0.8149f,-0.3726f,0.4440f,  -0.7710f,-0.3154f,0.5533f,  -0.7533f,-0.3439f,0.5606f,  -0.6950f,-0.2832f,0.6609f,  
	-0.6753f,-0.3137f,0.6675f,  -0.6043f,-0.2493f,0.7567f,  -0.5829f,-0.2812f,0.7623f,  -0.5011f,-0.2133f,0.8387f,  -0.4782f,-0.2461f,0.8431f,  -0.3877f,-0.1746f,0.9051f,  -0.3638f,-0.2075f,0.9081f,  -0.2671f,-0.1325f,0.9545f,  -0.2425f,-0.1651f,0.9560f,  -0.1420f,-0.0867f,0.9861f,  -0.1172f,-0.1182f,0.9861f,  -0.0156f,-0.0366f,0.9992f,  0.0088f,-0.0664f,0.9978f,  0.1088f,0.0180f,0.9939f,  0.1323f,-0.0094f,0.9912f,  
	0.2280f,0.0774f,0.9706f,  0.2500f,0.0530f,0.9668f,  0.3390f,0.1416f,0.9300f,  0.3588f,0.1207f,0.9256f,  0.4387f,0.2104f,0.8737f,  0.4557f,0.1933f,0.8689f,  0.5242f,0.2833f,0.8031f,  0.5378f,0.2705f,0.7985f,  0.5931f,0.3596f,0.7203f,  0.6026f,0.3513f,0.7166f,  0.6432f,0.4384f,0.6278f,  0.6481f,0.4346f,0.6254f,  0.6729f,0.5182f,0.5279f,  0.6726f,0.5189f,0.5277f,  0.6808f,0.5976f,0.4235f,  
	0.6750f,0.6024f,0.4260f,  0.6666f,0.6746f,0.3172f,  0.6550f,0.6831f,0.3230f,  0.6301f,0.7471f,0.2117f,  0.6127f,0.7587f,0.2214f,  0.5722f,0.8128f,0.1095f,  0.5492f,0.8266f,0.1234f,  0.4943f,0.8692f,0.0126f,  0.4659f,0.8843f,0.0310f,  0.3985f,0.9139f,-0.0770f,  0.3653f,0.9293f,-0.0541f,  0.2876f,0.9446f,-0.1581f,  0.2503f,0.9593f,-0.1309f,  0.1649f,0.9592f,-0.2298f,  0.1245f,0.9721f,-0.1988f,  
	0.0341f,0.9559f,-0.2918f,  -0.0083f,0.9662f,-0.2576f,  -0.1005f,0.9335f,-0.3442f,  -0.1438f,0.9405f,-0.3078f,  -0.2347f,0.8914f,-0.3878f,  -0.2777f,0.8946f,-0.3501f,  -0.3641f,0.8296f,-0.4234f,  -0.4055f,0.8288f,-0.3856f,  -0.4844f,0.7489f,-0.4523f,  -0.5232f,0.7441f,-0.4155f,  -0.5918f,0.6507f,-0.4758f,  -0.6269f,0.6422f,-0.4412f,  -0.6827f,0.5371f,-0.4953f,  -0.7132f,0.5255f,-0.4639f,  -0.7542f,0.4111f,-0.5121f,  
	-0.7795f,0.3969f,-0.4846f,  -0.8039f,0.2757f,-0.5271f,  -0.8236f,0.2597f,-0.5042f,  -0.8302f,0.1344f,-0.5410f,  -0.8441f,0.1177f,-0.5232f,  -0.8323f,-0.0087f,-0.5542f,  -0.8403f,-0.0253f,-0.5415f,  -0.8101f,-0.1500f,-0.5667f,  -0.8124f,-0.1655f,-0.5591f,  -0.7643f,-0.2856f,-0.5782f,  -0.7611f,-0.2993f,-0.5755f,  -0.6960f,-0.4120f,-0.5881f,  -0.6880f,-0.4231f,-0.5896f,  -0.6074f,-0.5260f,-0.5953f,  -0.5951f,-0.5339f,-0.6007f,  
	-0.5010f,-0.6247f,-0.5990f,  -0.4850f,-0.6291f,-0.6075f,  -0.3796f,-0.7060f,-0.5978f,  -0.3608f,-0.7066f,-0.6087f,  -0.7178f,-0.5595f,-0.4143f,  -0.7185f,-0.5580f,-0.4152f,  -0.7881f,-0.5334f,-0.3072f,  -0.7866f,-0.5365f,-0.3056f,  -0.8407f,-0.5070f,-0.1902f,  -0.8369f,-0.5145f,-0.1867f,  -0.8745f,-0.4804f,-0.0662f,  -0.8683f,-0.4923f,-0.0611f,  -0.8889f,-0.4539f,0.0622f,  -0.8801f,-0.4699f,0.0683f,  -0.8835f,-0.4272f,0.1920f,  
	-0.8722f,-0.4470f,0.1987f,  -0.8586f,-0.4003f,0.3202f,  -0.8448f,-0.4234f,0.3272f,  -0.8149f,-0.3726f,0.4440f,  -0.7986f,-0.3988f,0.4508f,  -0.7533f,-0.3439f,0.5606f,  -0.7347f,-0.3727f,0.5669f,  -0.6753f,-0.3137f,0.6675f,  -0.6547f,-0.3444f,0.6729f,  -0.5829f,-0.2812f,0.7623f,  -0.5604f,-0.3134f,0.7666f,  -0.4782f,-0.2461f,0.8431f,  -0.4543f,-0.2790f,0.8461f,  -0.3638f,-0.2075f,0.9081f,  -0.3388f,-0.2406f,0.9096f,  
	-0.2425f,-0.1651f,0.9560f,  -0.2168f,-0.1977f,0.9560f,  -0.1172f,-0.1182f,0.9861f,  -0.0914f,-0.1496f,0.9845f,  0.0088f,-0.0664f,0.9978f,  0.0342f,-0.0960f,0.9948f,  0.1323f,-0.0094f,0.9912f,  0.1567f,-0.0366f,0.9870f,  0.2500f,0.0530f,0.9668f,  0.2728f,0.0288f,0.9616f,  0.3588f,0.1207f,0.9256f,  0.3793f,0.1000f,0.9198f,  0.4557f,0.1933f,0.8689f,  0.4732f,0.1767f,0.8630f,  0.5378f,0.2705f,0.7985f,  
	0.5517f,0.2582f,0.7931f,  0.6026f,0.3513f,0.7166f,  0.6122f,0.3435f,0.7122f,  0.6481f,0.4346f,0.6254f,  0.6528f,0.4313f,0.6228f,  0.6726f,0.5189f,0.5277f,  0.6719f,0.5199f,0.5274f,  0.6750f,0.6024f,0.4260f,  0.6687f,0.6074f,0.4288f,  0.6550f,0.6831f,0.3230f,  0.6428f,0.6916f,0.3295f,  0.6127f,0.7587f,0.2214f,  0.5946f,0.7698f,0.2320f,  0.5492f,0.8266f,0.1234f,  0.5253f,0.8396f,0.1385f,  
	0.4659f,0.8843f,0.0310f,  0.4368f,0.8981f,0.0508f,  0.3653f,0.9293f,-0.0541f,  0.3315f,0.9430f,-0.0298f,  0.2503f,0.9593f,-0.1309f,  0.2127f,0.9717f,-0.1024f,  0.1245f,0.9721f,-0.1988f,  0.0840f,0.9824f,-0.1666f,  -0.0083f,0.9662f,-0.2576f,  -0.0505f,0.9736f,-0.2226f,  -0.1438f,0.9405f,-0.3078f,  -0.1865f,0.9444f,-0.2708f,  -0.2777f,0.8946f,-0.3501f,  -0.3196f,0.8946f,-0.3123f,  -0.4055f,0.8288f,-0.3856f,  
	-0.4456f,0.8248f,-0.3481f,  -0.5232f,0.7441f,-0.4155f,  -0.5603f,0.7362f,-0.3796f,  -0.6269f,0.6422f,-0.4412f,  -0.6601f,0.6308f,-0.4078f,  -0.7132f,0.5255f,-0.4639f,  -0.7418f,0.5113f,-0.4339f,  -0.7795f,0.3969f,-0.4846f,  -0.8029f,0.3805f,-0.4589f,  -0.8236f,0.2597f,-0.5042f,  -0.8414f,0.2421f,-0.4832f,  -0.8441f,0.1177f,-0.5232f,  -0.8561f,0.0998f,-0.5071f,  -0.8403f,-0.0253f,-0.5415f,  -0.8466f,-0.0426f,-0.5305f,  
	-0.8124f,-0.1655f,-0.5591f,  -0.8131f,-0.1815f,-0.5531f,  -0.7611f,-0.2993f,-0.5755f,  -0.7568f,-0.3129f,-0.5739f,  -0.6880f,-0.4231f,-0.5896f,  -0.6790f,-0.4339f,-0.5922f,  -0.5951f,-0.5339f,-0.6007f,  -0.5821f,-0.5414f,-0.6067f,  -0.4850f,-0.6291f,-0.6075f,  -0.4687f,-0.6329f,-0.6162f,  -0.3608f,-0.7066f,-0.6087f,  -0.3420f,-0.7065f,-0.6195f,  -0.7185f,-0.5580f,-0.4152f,  -0.7191f,-0.5568f,-0.4158f,  -0.7866f,-0.5365f,-0.3056f,  
	-0.7849f,-0.5398f,-0.3041f,  -0.8369f,-0.5145f,-0.1867f,  -0.8328f,-0.5223f,-0.1834f,  -0.8683f,-0.4923f,-0.0611f,  -0.8616f,-0.5045f,-0.0565f,  -0.8801f,-0.4699f,0.0683f,  -0.8708f,-0.4861f,0.0738f,  -0.8722f,-0.4470f,0.1987f,  -0.8602f,-0.4670f,0.2047f,  -0.8448f,-0.4234f,0.3272f,  -0.8302f,-0.4469f,0.3332f,  -0.7986f,-0.3988f,0.4508f,  -0.7815f,-0.4252f,0.4565f,  -0.7347f,-0.3727f,0.5669f,  -0.7153f,-0.4016f,0.5720f,  
	-0.6547f,-0.3444f,0.6729f,  -0.6331f,-0.3753f,0.6770f,  -0.5604f,-0.3134f,0.7666f,  -0.5370f,-0.3457f,0.7695f,  -0.4543f,-0.2790f,0.8461f,  -0.4293f,-0.3121f,0.8476f,  -0.3388f,-0.2406f,0.9096f,  -0.3126f,-0.2738f,0.9096f,  -0.2168f,-0.1977f,0.9560f,  -0.1900f,-0.2303f,0.9544f,  -0.0914f,-0.1496f,0.9845f,  -0.0644f,-0.1810f,0.9814f,  0.0342f,-0.0960f,0.9948f,  0.0607f,-0.1255f,0.9902f,  0.1567f,-0.0366f,0.9870f,  
	0.1821f,-0.0635f,0.9812f,  0.2728f,0.0288f,0.9616f,  0.2965f,0.0050f,0.9550f,  0.3793f,0.1000f,0.9198f,  0.4006f,0.0799f,0.9128f,  0.4732f,0.1767f,0.8630f,  0.4913f,0.1607f,0.8561f,  0.5517f,0.2582f,0.7931f,  0.5658f,0.2465f,0.7868f,  0.6122f,0.3435f,0.7122f,  0.6219f,0.3363f,0.7072f,  0.6528f,0.4313f,0.6228f,  0.6574f,0.4285f,0.6199f,  0.6719f,0.5199f,0.5274f,  0.6710f,0.5214f,0.5273f,  
	0.6687f,0.6074f,0.4288f,  0.6618f,0.6126f,0.4320f,  0.6428f,0.6916f,0.3295f,  0.6299f,0.6999f,0.3367f,  0.5946f,0.7698f,0.2320f,  0.5757f,0.7805f,0.2437f,  0.5253f,0.8396f,0.1385f,  0.5008f,0.8516f,0.1548f,  0.4368f,0.8981f,0.0508f,  0.4070f,0.9106f,0.0718f,  0.3315f,0.9430f,-0.0298f,  0.2973f,0.9548f,-0.0043f,  0.2127f,0.9717f,-0.1024f,  0.1749f,0.9819f,-0.0727f,  0.0840f,0.9824f,-0.1666f,  
	0.0437f,0.9901f,-0.1335f,  -0.0505f,0.9736f,-0.2226f,  -0.0921f,0.9780f,-0.1870f,  -0.1865f,0.9444f,-0.2708f,  -0.2283f,0.9451f,-0.2337f,  -0.3196f,0.8946f,-0.3123f,  -0.3604f,0.8914f,-0.2748f,  -0.4456f,0.8248f,-0.3481f,  -0.4842f,0.8177f,-0.3114f,  -0.5603f,0.7362f,-0.3796f,  -0.5957f,0.7255f,-0.3447f,  -0.6601f,0.6308f,-0.4078f,  -0.6915f,0.6169f,-0.3758f,  -0.7418f,0.5113f,-0.4339f,  -0.7685f,0.4948f,-0.4057f,  
	-0.8029f,0.3805f,-0.4589f,  -0.8243f,0.3624f,-0.4349f,  -0.8414f,0.2421f,-0.4832f,  -0.8573f,0.2233f,-0.4640f,  -0.8561f,0.0998f,-0.5071f,  -0.8664f,0.0811f,-0.4928f,  -0.8466f,-0.0426f,-0.5305f,  -0.8513f,-0.0605f,-0.5212f,  -0.8131f,-0.1815f,-0.5531f,  -0.8126f,-0.1974f,-0.5484f,  -0.7568f,-0.3129f,-0.5739f,  -0.7513f,-0.3265f,-0.5736f,  -0.6790f,-0.4339f,-0.5922f,  -0.6691f,-0.4444f,-0.5956f,  -0.5821f,-0.5414f,-0.6067f,  
	-0.5685f,-0.5484f,-0.6133f,  -0.4687f,-0.6329f,-0.6162f,  -0.4522f,-0.6361f,-0.6252f,  -0.3420f,-0.7065f,-0.6195f,  -0.3232f,-0.7059f,-0.6303f,  -0.7191f,-0.5568f,-0.4158f,  -0.7196f,-0.5558f,-0.4163f,  -0.7849f,-0.5398f,-0.3041f,  -0.7830f,-0.5434f,-0.3026f,  -0.8328f,-0.5223f,-0.1834f,  -0.8283f,-0.5304f,-0.1804f,  -0.8616f,-0.5045f,-0.0565f,  -0.8545f,-0.5169f,-0.0523f,  -0.8708f,-0.4861f,0.0738f,  -0.8609f,-0.5026f,0.0788f,  
	-0.8602f,-0.4670f,0.2047f,  -0.8476f,-0.4873f,0.2100f,  -0.8302f,-0.4469f,0.3332f,  -0.8149f,-0.4706f,0.3384f,  -0.7815f,-0.4252f,0.4565f,  -0.7636f,-0.4519f,0.4612f,  -0.7153f,-0.4016f,0.5720f,  -0.6949f,-0.4307f,0.5758f,  -0.6331f,-0.3753f,0.6770f,  -0.6105f,-0.4064f,0.6798f,  -0.5370f,-0.3457f,0.7695f,  -0.5124f,-0.3781f,0.7710f,  -0.4293f,-0.3121f,0.8476f,  -0.4031f,-0.3453f,0.8475f,  -0.3126f,-0.2738f,0.9096f,  
	-0.2853f,-0.3071f,0.9079f,  -0.1900f,-0.2303f,0.9544f,  -0.1620f,-0.2630f,0.9511f,  -0.0644f,-0.1810f,0.9814f,  -0.0363f,-0.2123f,0.9765f,  0.0607f,-0.1255f,0.9902f,  0.0883f,-0.1548f,0.9840f,  0.1821f,-0.0635f,0.9812f,  0.2086f,-0.0901f,0.9739f,  0.2965f,0.0050f,0.9550f,  0.3210f,-0.0183f,0.9469f,  0.4006f,0.0799f,0.9128f,  0.4225f,0.0603f,0.9044f,  0.4913f,0.1607f,0.8561f,  0.5098f,0.1453f,0.8479f,  
	0.5658f,0.2465f,0.7868f,  0.5803f,0.2355f,0.7796f,  0.6219f,0.3363f,0.7072f,  0.6316f,0.3297f,0.7017f,  0.6574f,0.4285f,0.6199f,  0.6618f,0.4263f,0.6167f,  0.6710f,0.5214f,0.5273f,  0.6696f,0.5232f,0.5272f,  0.6618f,0.6126f,0.4320f,  0.6544f,0.6180f,0.4357f,  0.6299f,0.6999f,0.3367f,  0.6164f,0.7080f,0.3446f,  0.5757f,0.7805f,0.2437f,  0.5562f,0.7906f,0.2562f,  0.5008f,0.8516f,0.1548f,  
	0.4756f,0.8626f,0.1722f,  0.4070f,0.9106f,0.0718f,  0.3768f,0.9215f,0.0940f,  0.2973f,0.9548f,-0.0043f,  0.2628f,0.9646f,0.0224f,  0.1749f,0.9819f,-0.0727f,  0.1372f,0.9896f,-0.0422f,  0.0437f,0.9901f,-0.1335f,  0.0038f,0.9950f,-0.0999f,  -0.0921f,0.9780f,-0.1870f,  -0.1331f,0.9795f,-0.1511f,  -0.2283f,0.9451f,-0.2337f,  -0.2691f,0.9428f,-0.1968f,  -0.3604f,0.8914f,-0.2748f,  -0.3998f,0.8852f,-0.2380f,  
	-0.4842f,0.8177f,-0.3114f,  -0.5212f,0.8077f,-0.2758f,  -0.5957f,0.7255f,-0.3447f,  -0.6293f,0.7121f,-0.3113f,  -0.6915f,0.6169f,-0.3758f,  -0.7210f,0.6007f,-0.3455f,  -0.7685f,0.4948f,-0.4057f,  -0.7932f,0.4765f,-0.3793f,  -0.8243f,0.3624f,-0.4349f,  -0.8438f,0.3428f,-0.4129f,  -0.8573f,0.2233f,-0.4640f,  -0.8713f,0.2033f,-0.4467f,  -0.8664f,0.0811f,-0.4928f,  -0.8749f,0.0615f,-0.4805f,  -0.8513f,-0.0605f,-0.5212f,  
	-0.8545f,-0.0786f,-0.5135f,  -0.8126f,-0.1974f,-0.5484f,  -0.8107f,-0.2135f,-0.5451f,  -0.7513f,-0.3265f,-0.5736f,  -0.7448f,-0.3398f,-0.5743f,  -0.6691f,-0.4444f,-0.5956f,  -0.6586f,-0.4545f,-0.5998f,  -0.5685f,-0.5484f,-0.6133f,  -0.5545f,-0.5548f,-0.6202f,  -0.4522f,-0.6361f,-0.6252f,  -0.4354f,-0.6387f,-0.6344f,  -0.3232f,-0.7059f,-0.6303f,  -0.3045f,-0.7046f,-0.6409f,  -0.7196f,-0.5558f,-0.4163f,  -0.7199f,-0.5552f,-0.4165f,  
	-0.7830f,-0.5434f,-0.3026f,  -0.7809f,-0.5473f,-0.3011f,  -0.8283f,-0.5304f,-0.1804f,  -0.8235f,-0.5388f,-0.1775f,  -0.8545f,-0.5169f,-0.0523f,  -0.8469f,-0.5295f,-0.0485f,  -0.8609f,-0.5026f,0.0788f,  -0.8505f,-0.5193f,0.0831f,  -0.8476f,-0.4873f,0.2100f,  -0.8344f,-0.5078f,0.2144f,  -0.8149f,-0.4706f,0.3384f,  -0.7988f,-0.4945f,0.3426f,  -0.7636f,-0.4519f,0.4612f,  -0.7448f,-0.4788f,0.4648f,  -0.6949f,-0.4307f,0.5758f,  
	-0.6735f,-0.4601f,0.5785f,  -0.6105f,-0.4064f,0.6798f,  -0.5868f,-0.4377f,0.6812f,  -0.5124f,-0.3781f,0.7710f,  -0.4867f,-0.4108f,0.7710f,  -0.4031f,-0.3453f,0.8475f,  -0.3758f,-0.3786f,0.8459f,  -0.2853f,-0.3071f,0.9079f,  -0.2568f,-0.3404f,0.9046f,  -0.1620f,-0.2630f,0.9511f,  -0.1328f,-0.2955f,0.9461f,  -0.0363f,-0.2123f,0.9765f,  -0.0070f,-0.2434f,0.9699f,  0.0883f,-0.1548f,0.9840f,  0.1170f,-0.1837f,0.9760f,  
	0.2086f,-0.0901f,0.9739f,  0.2360f,-0.1162f,0.9648f,  0.3210f,-0.0183f,0.9469f,  0.3464f,-0.0410f,0.9372f,  0.4225f,0.0603f,0.9044f,  0.4450f,0.0415f,0.8945f,  0.5098f,0.1453f,0.8479f,  0.5288f,0.1307f,0.8386f,  0.5803f,0.2355f,0.7796f,  0.5950f,0.2253f,0.7715f,  0.6316f,0.3297f,0.7017f,  0.6413f,0.3240f,0.6956f,  0.6618f,0.4263f,0.6167f,  0.6659f,0.4248f,0.6133f,  0.6696f,0.5232f,0.5272f,  
	0.6678f,0.5255f,0.5272f,  0.6544f,0.6180f,0.4357f,  0.6464f,0.6235f,0.4398f,  0.6164f,0.7080f,0.3446f,  0.6022f,0.7160f,0.3533f,  0.5562f,0.7906f,0.2562f,  0.5361f,0.7999f,0.2697f,  0.4756f,0.8626f,0.1722f,  0.4499f,0.8725f,0.1906f,  0.3768f,0.9215f,0.0940f,  0.3463f,0.9308f,0.1171f,  0.2628f,0.9646f,0.0224f,  0.2283f,0.9723f,0.0498f,  0.1372f,0.9896f,-0.0422f,  0.0996f,0.9950f,-0.0111f,  
	0.0038f,0.9950f,-0.0999f,  -0.0356f,0.9972f,-0.0659f,  -0.1331f,0.9795f,-0.1511f,  -0.1731f,0.9781f,-0.1154f,  -0.2691f,0.9428f,-0.1968f,  -0.3086f,0.9376f,-0.1605f,  -0.3998f,0.8852f,-0.2380f,  -0.4377f,0.8761f,-0.2021f,  -0.5212f,0.8077f,-0.2758f,  -0.5565f,0.7950f,-0.2414f,  -0.6293f,0.7121f,-0.3113f,  -0.6611f,0.6963f,-0.2795f,  -0.7210f,0.6007f,-0.3455f,  -0.7485f,0.5824f,-0.3171f,  -0.7932f,0.4765f,-0.3793f,  
	-0.8159f,0.4566f,-0.3549f,  -0.8438f,0.3428f,-0.4129f,  -0.8613f,0.3220f,-0.3930f,  -0.8713f,0.2033f,-0.4467f,  -0.8835f,0.1824f,-0.4315f,  -0.8749f,0.0615f,-0.4805f,  -0.8818f,0.0416f,-0.4698f,  -0.8545f,-0.0786f,-0.5135f,  -0.8563f,-0.0969f,-0.5074f,  -0.8107f,-0.2135f,-0.5451f,  -0.8076f,-0.2295f,-0.5432f,  -0.7448f,-0.3398f,-0.5743f,  -0.7374f,-0.3528f,-0.5761f,  -0.6586f,-0.4545f,-0.5998f,  -0.6474f,-0.4640f,-0.6046f,  
	-0.5545f,-0.5548f,-0.6202f,  -0.5402f,-0.5606f,-0.6276f,  -0.4354f,-0.6387f,-0.6344f,  -0.4187f,-0.6407f,-0.6436f,  -0.3045f,-0.7046f,-0.6409f,  -0.2860f,-0.7028f,-0.6514f,  -0.7199f,-0.5552f,-0.4165f,  -0.7201f,-0.5548f,-0.4167f,  -0.7809f,-0.5473f,-0.3011f,  -0.7785f,-0.5515f,-0.2997f,  -0.8235f,-0.5388f,-0.1775f,  -0.8184f,-0.5474f,-0.1749f,  -0.8469f,-0.5295f,-0.0485f,  -0.8389f,-0.5425f,-0.0452f,  -0.8505f,-0.5193f,0.0831f,  
	-0.8395f,-0.5364f,0.0868f,  -0.8344f,-0.5078f,0.2144f,  -0.8204f,-0.5286f,0.2181f,  -0.7988f,-0.4945f,0.3426f,  -0.7819f,-0.5187f,0.3458f,  -0.7448f,-0.4788f,0.4648f,  -0.7250f,-0.5060f,0.4672f,  -0.6735f,-0.4601f,0.5785f,  -0.6511f,-0.4898f,0.5798f,  -0.5868f,-0.4377f,0.6812f,  -0.5620f,-0.4692f,0.6812f,  -0.4867f,-0.4108f,0.7710f,  -0.4598f,-0.4436f,0.7693f,  -0.3758f,-0.3786f,0.8459f,  -0.3471f,-0.4119f,0.8425f,  
	-0.2568f,-0.3404f,0.9046f,  -0.2269f,-0.3736f,0.8994f,  -0.1328f,-0.2955f,0.9461f,  -0.1023f,-0.3278f,0.9392f,  -0.0070f,-0.2434f,0.9699f,  0.0235f,-0.2741f,0.9614f,  0.1170f,-0.1837f,0.9760f,  0.1469f,-0.2121f,0.9661f,  0.2360f,-0.1162f,0.9648f,  0.2644f,-0.1417f,0.9539f,  0.3464f,-0.0410f,0.9372f,  0.3726f,-0.0630f,0.9258f,  0.4450f,0.0415f,0.8945f,  0.4682f,0.0236f,0.8833f,  0.5288f,0.1307f,0.8386f,  
	0.5482f,0.1170f,0.8281f,  0.5950f,0.2253f,0.7715f,  0.6098f,0.2161f,0.7626f,  0.6413f,0.3240f,0.6956f,  0.6508f,0.3191f,0.6889f,  0.6659f,0.4248f,0.6133f,  0.6697f,0.4240f,0.6097f,  0.6678f,0.5255f,0.5272f,  0.6655f,0.5283f,0.5273f,  0.6464f,0.6235f,0.4398f,  0.6379f,0.6291f,0.4443f,  0.6022f,0.7160f,0.3533f,  0.5874f,0.7236f,0.3626f,  0.5361f,0.7999f,0.2697f,  0.5153f,0.8086f,0.2840f,  
	0.4499f,0.8725f,0.1906f,  0.4238f,0.8811f,0.2099f,  0.3463f,0.9308f,0.1171f,  0.3155f,0.9384f,0.1411f,  0.2283f,0.9723f,0.0498f,  0.1938f,0.9779f,0.0779f,  0.0996f,0.9950f,-0.0111f,  0.0625f,0.9978f,0.0204f,  -0.0356f,0.9972f,-0.0659f,  -0.0742f,0.9967f,-0.0321f,  -0.1731f,0.9781f,-0.1154f,  -0.2121f,0.9739f,-0.0802f,  -0.3086f,0.9376f,-0.1605f,  -0.3468f,0.9296f,-0.1250f,  -0.4377f,0.8761f,-0.2021f,  
	-0.4741f,0.8644f,-0.1674f,  -0.5565f,0.7950f,-0.2414f,  -0.5900f,0.7800f,-0.2087f,  -0.6611f,0.6963f,-0.2795f,  -0.6910f,0.6784f,-0.2496f,  -0.7485f,0.5824f,-0.3171f,  -0.7740f,0.5625f,-0.2907f,  -0.8159f,0.4566f,-0.3549f,  -0.8367f,0.4353f,-0.3325f,  -0.8613f,0.3220f,-0.3930f,  -0.8770f,0.3002f,-0.3751f,  -0.8835f,0.1824f,-0.4315f,  -0.8940f,0.1610f,-0.4181f,  -0.8818f,0.0416f,-0.4698f,  -0.8872f,0.0214f,-0.4610f,  
	-0.8563f,-0.0969f,-0.5074f,  -0.8567f,-0.1152f,-0.5028f,  -0.8076f,-0.2295f,-0.5432f,  -0.8035f,-0.2451f,-0.5425f,  -0.7374f,-0.3528f,-0.5761f,  -0.7292f,-0.3652f,-0.5787f,  -0.6474f,-0.4640f,-0.6046f,  -0.6357f,-0.4729f,-0.6102f,  -0.5402f,-0.5606f,-0.6276f,  -0.5256f,-0.5658f,-0.6353f,  -0.4187f,-0.6407f,-0.6436f,  -0.4019f,-0.6420f,-0.6529f,  -0.2860f,-0.7028f,-0.6514f,  -0.2678f,-0.7004f,-0.6616f,  -0.7201f,-0.5548f,-0.4167f,  
	-0.7201f,-0.5548f,-0.4167f,  -0.7785f,-0.5515f,-0.2997f,  -0.7758f,-0.5560f,-0.2984f,  -0.8184f,-0.5474f,-0.1749f,  -0.8128f,-0.5564f,-0.1726f,  -0.8389f,-0.5425f,-0.0452f,  -0.8303f,-0.5557f,-0.0423f,  -0.8395f,-0.5364f,0.0868f,  -0.8279f,-0.5537f,0.0898f,  -0.8204f,-0.5286f,0.2181f,  -0.8056f,-0.5497f,0.2208f,  -0.7819f,-0.5187f,0.3458f,  -0.7641f,-0.5432f,0.3480f,  -0.7250f,-0.5060f,0.4672f,  -0.7043f,-0.5335f,0.4684f,  
	-0.6511f,-0.4898f,0.5798f,  -0.6276f,-0.5197f,0.5797f,  -0.5620f,-0.4692f,0.6812f,  -0.5359f,-0.5009f,0.6796f,  -0.4598f,-0.4436f,0.7693f,  -0.4316f,-0.4764f,0.7660f,  -0.3471f,-0.4119f,0.8425f,  -0.3172f,-0.4453f,0.8373f,  -0.2269f,-0.3736f,0.8994f,  -0.1957f,-0.4066f,0.8924f,  -0.1023f,-0.3278f,0.9392f,  -0.0704f,-0.3599f,0.9303f,  0.0235f,-0.2741f,0.9614f,  0.0553f,-0.3044f,0.9509f,  0.1469f,-0.2121f,0.9661f,  
	0.1779f,-0.2400f,0.9543f,  0.2644f,-0.1417f,0.9539f,  0.2939f,-0.1664f,0.9413f,  0.3726f,-0.0630f,0.9258f,  0.3997f,-0.0840f,0.9128f,  0.4682f,0.0236f,0.8833f,  0.4920f,0.0067f,0.8706f,  0.5482f,0.1170f,0.8281f,  0.5679f,0.1044f,0.8165f,  0.6098f,0.2161f,0.7626f,  0.6246f,0.2079f,0.7527f,  0.6508f,0.3191f,0.6889f,  0.6602f,0.3152f,0.6817f,  0.6697f,0.4240f,0.6097f,  0.6731f,0.4240f,0.6059f,  
	0.6655f,0.5283f,0.5273f,  0.6626f,0.5315f,0.5277f,  0.6379f,0.6291f,0.4443f,  0.6287f,0.6348f,0.4493f,  0.5874f,0.7236f,0.3626f,  0.5720f,0.7308f,0.3726f,  0.5153f,0.8086f,0.2840f,  0.4941f,0.8163f,0.2991f,  0.4238f,0.8811f,0.2099f,  0.3974f,0.8884f,0.2299f,  0.3155f,0.9384f,0.1411f,  0.2847f,0.9442f,0.1656f,  0.1938f,0.9779f,0.0779f,  0.1596f,0.9814f,0.1064f,  0.0625f,0.9978f,0.0204f,  
	0.0259f,0.9983f,0.0519f,  -0.0742f,0.9967f,-0.0321f,  -0.1121f,0.9937f,0.0015f,  -0.2121f,0.9739f,-0.0802f,  -0.2500f,0.9672f,-0.0457f,  -0.3468f,0.9296f,-0.1250f,  -0.3836f,0.9190f,-0.0906f,  -0.4741f,0.8644f,-0.1674f,  -0.5088f,0.8503f,-0.1343f,  -0.5900f,0.7800f,-0.2087f,  -0.6217f,0.7628f,-0.1777f,  -0.6910f,0.6784f,-0.2496f,  -0.7190f,0.6588f,-0.2216f,  -0.7740f,0.5625f,-0.2907f,  -0.7977f,0.5410f,-0.2664f,  
	-0.8367f,0.4353f,-0.3325f,  -0.8556f,0.4128f,-0.3124f,  -0.8770f,0.3002f,-0.3751f,  -0.8910f,0.2777f,-0.3592f,  -0.8940f,0.1610f,-0.4181f,  -0.9029f,0.1393f,-0.4067f,  -0.8872f,0.0214f,-0.4610f,  -0.8911f,0.0011f,-0.4538f,  -0.8567f,-0.1152f,-0.5028f,  -0.8559f,-0.1332f,-0.4997f,  -0.8035f,-0.2451f,-0.5425f,  -0.7985f,-0.2602f,-0.5429f,  -0.7292f,-0.3652f,-0.5787f,  -0.7203f,-0.3771f,-0.5822f,  -0.6357f,-0.4729f,-0.6102f,  
	-0.6236f,-0.4812f,-0.6161f,  -0.5256f,-0.5658f,-0.6353f,  -0.5109f,-0.5703f,-0.6432f,  -0.4019f,-0.6420f,-0.6529f,  -0.3853f,-0.6428f,-0.6621f,  -0.2678f,-0.7004f,-0.6616f,  -0.2499f,-0.6975f,-0.6716f,  -0.7201f,-0.5548f,-0.4167f,  -0.7200f,-0.5550f,-0.4167f,  -0.7758f,-0.5560f,-0.2984f,  -0.7729f,-0.5607f,-0.2971f,  -0.8128f,-0.5564f,-0.1726f,  -0.8068f,-0.5656f,-0.1706f,  -0.8303f,-0.5557f,-0.0423f,  -0.8212f,-0.5693f,-0.0399f,  
	-0.8279f,-0.5537f,0.0898f,  -0.8155f,-0.5713f,0.0921f,  -0.8056f,-0.5497f,0.2208f,  -0.7901f,-0.5711f,0.2227f,  -0.7641f,-0.5432f,0.3480f,  -0.7453f,-0.5680f,0.3490f,  -0.7043f,-0.5335f,0.4684f,  -0.6824f,-0.5612f,0.4684f,  -0.6276f,-0.5197f,0.5797f,  -0.6029f,-0.5497f,0.5782f,  -0.5359f,-0.5009f,0.6796f,  -0.5086f,-0.5327f,0.6764f,  -0.4316f,-0.4764f,0.7660f,  -0.4020f,-0.5093f,0.7609f,  -0.3172f,-0.4453f,0.8373f,  
	-0.2859f,-0.4785f,0.8303f,  -0.1957f,-0.4066f,0.8924f,  -0.1632f,-0.4395f,0.8833f,  -0.0704f,-0.3599f,0.9303f,  -0.0373f,-0.3915f,0.9194f,  0.0553f,-0.3044f,0.9509f,  0.0883f,-0.3341f,0.9384f,  0.1779f,-0.2400f,0.9543f,  0.2100f,-0.2669f,0.9405f,  0.2939f,-0.1664f,0.9413f,  0.3242f,-0.1901f,0.9267f,  0.3997f,-0.0840f,0.9128f,  0.4274f,-0.1038f,0.8981f,  0.4920f,0.0067f,0.8706f,  0.5162f,-0.0090f,0.8564f,  
	0.5679f,0.1044f,0.8165f,  0.5877f,0.0932f,0.8037f,  0.6246f,0.2079f,0.7527f,  0.6394f,0.2010f,0.7421f,  0.6602f,0.3152f,0.6817f,  0.6693f,0.3124f,0.6741f,  0.6731f,0.4240f,0.6059f,  0.6761f,0.4247f,0.6021f,  0.6626f,0.5315f,0.5277f,  0.6592f,0.5351f,0.5283f,  0.6287f,0.6348f,0.4493f,  0.6188f,0.6405f,0.4547f,  0.5720f,0.7308f,0.3726f,  0.5560f,0.7376f,0.3831f,  0.4941f,0.8163f,0.2991f,  
	0.4724f,0.8232f,0.3148f,  0.3974f,0.8884f,0.2299f,  0.3707f,0.8944f,0.2505f,  0.2847f,0.9442f,0.1656f,  0.2539f,0.9483f,0.1905f,  0.1596f,0.9814f,0.1064f,  0.1256f,0.9829f,0.1349f,  0.0259f,0.9983f,0.0519f,  -0.0101f,0.9965f,0.0831f,  -0.1121f,0.9937f,0.0015f,  -0.1489f,0.9883f,0.0344f,  -0.2500f,0.9672f,-0.0457f,  -0.2866f,0.9580f,-0.0122f,  -0.3836f,0.9190f,-0.0906f,  -0.4190f,0.9062f,-0.0576f,  
	-0.5088f,0.8503f,-0.1343f,  -0.5419f,0.8341f,-0.1029f,  -0.6217f,0.7628f,-0.1777f,  -0.6517f,0.7438f,-0.1487f,  -0.7190f,0.6588f,-0.2216f,  -0.7451f,0.6376f,-0.1957f,  -0.7977f,0.5410f,-0.2664f,  -0.8195f,0.5184f,-0.2443f,  -0.8556f,0.4128f,-0.3124f,  -0.8727f,0.3897f,-0.2942f,  -0.8910f,0.2777f,-0.3592f,  -0.9032f,0.2548f,-0.3454f,  -0.9029f,0.1393f,-0.4067f,  -0.9102f,0.1173f,-0.3971f,  -0.8911f,0.0011f,-0.4538f,  
	-0.8937f,-0.0190f,-0.4483f,  -0.8559f,-0.1332f,-0.4997f,  -0.8541f,-0.1508f,-0.4978f,  -0.7985f,-0.2602f,-0.5429f,  -0.7925f,-0.2749f,-0.5444f,  -0.7203f,-0.3771f,-0.5822f,  -0.7108f,-0.3884f,-0.5864f,  -0.6236f,-0.4812f,-0.6161f,  -0.6112f,-0.4888f,-0.6226f,  -0.5109f,-0.5703f,-0.6432f,  -0.4962f,-0.5742f,-0.6513f,  -0.3853f,-0.6428f,-0.6621f,  -0.3688f,-0.6430f,-0.6713f,  -0.2499f,-0.6975f,-0.6716f,  -0.2325f,-0.6941f,-0.6813f,  
	-0.7200f,-0.5550f,-0.4167f,  -0.7197f,-0.5555f,-0.4165f,  -0.7729f,-0.5607f,-0.2971f,  -0.7696f,-0.5658f,-0.2961f,  -0.8068f,-0.5656f,-0.1706f,  -0.8004f,-0.5751f,-0.1690f,  -0.8212f,-0.5693f,-0.0399f,  -0.8115f,-0.5831f,-0.0381f,  -0.8155f,-0.5713f,0.0921f,  -0.8025f,-0.5892f,0.0937f,  -0.7901f,-0.5711f,0.2227f,  -0.7736f,-0.5928f,0.2237f,  -0.7453f,-0.5680f,0.3490f,  -0.7256f,-0.5931f,0.3490f,  -0.6824f,-0.5612f,0.4684f,  
	-0.6595f,-0.5891f,0.4669f,  -0.6029f,-0.5497f,0.5782f,  -0.5770f,-0.5799f,0.5752f,  -0.5086f,-0.5327f,0.6764f,  -0.4800f,-0.5646f,0.6714f,  -0.4020f,-0.5093f,0.7609f,  -0.3711f,-0.5421f,0.7539f,  -0.2859f,-0.4785f,0.8303f,  -0.2531f,-0.5115f,0.8212f,  -0.1632f,-0.4395f,0.8833f,  -0.1292f,-0.4719f,0.8722f,  -0.0373f,-0.3915f,0.9194f,  -0.0028f,-0.4225f,0.9063f,  0.0883f,-0.3341f,0.9384f,  0.1226f,-0.3629f,0.9237f,  
	0.2100f,-0.2669f,0.9405f,  0.2432f,-0.2929f,0.9247f,  0.3242f,-0.1901f,0.9267f,  0.3555f,-0.2125f,0.9102f,  0.4274f,-0.1038f,0.8981f,  0.4557f,-0.1223f,0.8817f,  0.5162f,-0.0090f,0.8564f,  0.5408f,-0.0232f,0.8408f,  0.5877f,0.0932f,0.8037f,  0.6077f,0.0833f,0.7898f,  0.6394f,0.2010f,0.7421f,  0.6540f,0.1954f,0.7308f,  0.6693f,0.3124f,0.6741f,  0.6780f,0.3107f,0.6662f,  0.6761f,0.4247f,0.6021f,  
	0.6785f,0.4263f,0.5983f,  0.6592f,0.5351f,0.5283f,  0.6551f,0.5393f,0.5291f,  0.6188f,0.6405f,0.4547f,  0.6084f,0.6463f,0.4607f,  0.5560f,0.7376f,0.3831f,  0.5394f,0.7440f,0.3943f,  0.4724f,0.8232f,0.3148f,  0.4504f,0.8292f,0.3310f,  0.3707f,0.8944f,0.2505f,  0.3439f,0.8989f,0.2715f,  0.2539f,0.9483f,0.1905f,  0.2232f,0.9506f,0.2157f,  0.1256f,0.9829f,0.1349f,  0.0921f,0.9823f,0.1633f,  
	-0.0101f,0.9965f,0.0831f,  -0.0453f,0.9925f,0.1139f,  -0.1489f,0.9883f,0.0344f,  -0.1847f,0.9805f,0.0664f,  -0.2866f,0.9580f,-0.0122f,  -0.3219f,0.9465f,0.0200f,  -0.4190f,0.9062f,-0.0576f,  -0.4527f,0.8913f,-0.0262f,  -0.5419f,0.8341f,-0.1029f,  -0.5732f,0.8161f,-0.0732f,  -0.6517f,0.7438f,-0.1487f,  -0.6798f,0.7232f,-0.1217f,  -0.7451f,0.6376f,-0.1957f,  -0.7694f,0.6151f,-0.1720f,  -0.8195f,0.5184f,-0.2443f,  
	-0.8395f,0.4949f,-0.2242f,  -0.8727f,0.3897f,-0.2942f,  -0.8881f,0.3659f,-0.2783f,  -0.9032f,0.2548f,-0.3454f,  -0.9138f,0.2316f,-0.3336f,  -0.9102f,0.1173f,-0.3971f,  -0.9162f,0.0955f,-0.3892f,  -0.8937f,-0.0190f,-0.4483f,  -0.8951f,-0.0388f,-0.4442f,  -0.8541f,-0.1508f,-0.4978f,  -0.8512f,-0.1680f,-0.4972f,  -0.7925f,-0.2749f,-0.5444f,  -0.7859f,-0.2889f,-0.5467f,  -0.7108f,-0.3884f,-0.5864f,  -0.7009f,-0.3989f,-0.5912f,  
	-0.6112f,-0.4888f,-0.6226f,  -0.5985f,-0.4957f,-0.6293f,  -0.4962f,-0.5742f,-0.6513f,  -0.4814f,-0.5774f,-0.6594f,  -0.3688f,-0.6430f,-0.6713f,  -0.3527f,-0.6425f,-0.6803f,  -0.2325f,-0.6941f,-0.6813f,  -0.2155f,-0.6903f,-0.6907f,  -0.7197f,-0.5555f,-0.4165f,  -0.7192f,-0.5563f,-0.4164f,  -0.7696f,-0.5658f,-0.2961f,  -0.7660f,-0.5711f,-0.2951f,  -0.8004f,-0.5751f,-0.1690f,  -0.7935f,-0.5849f,-0.1677f,  -0.8115f,-0.5831f,-0.0381f,  
	-0.8012f,-0.5973f,-0.0369f,  -0.8025f,-0.5892f,0.0937f,  -0.7887f,-0.6075f,0.0945f,  -0.7736f,-0.5928f,0.2237f,  -0.7563f,-0.6148f,0.2236f,  -0.7256f,-0.5931f,0.3490f,  -0.7048f,-0.6184f,0.3477f,  -0.6595f,-0.5891f,0.4669f,  -0.6354f,-0.6172f,0.4641f,  -0.5770f,-0.5799f,0.5752f,  -0.5497f,-0.6102f,0.5704f,  -0.4800f,-0.5646f,0.6714f,  -0.4500f,-0.5965f,0.6646f,  -0.3711f,-0.5421f,0.7539f,  -0.3388f,-0.5747f,0.7449f,  
	-0.2531f,-0.5115f,0.8212f,  -0.2189f,-0.5441f,0.8099f,  -0.1292f,-0.4719f,0.8722f,  -0.0938f,-0.5037f,0.8588f,  -0.0028f,-0.4225f,0.9063f,  0.0331f,-0.4527f,0.8910f,  0.1226f,-0.3629f,0.9237f,  0.1581f,-0.3908f,0.9068f,  0.2432f,-0.2929f,0.9247f,  0.2774f,-0.3176f,0.9067f,  0.3555f,-0.2125f,0.9102f,  0.3875f,-0.2336f,0.8918f,  0.4557f,-0.1223f,0.8817f,  0.4846f,-0.1392f,0.8636f,  0.5408f,-0.0232f,0.8408f,  
	0.5656f,-0.0358f,0.8239f,  0.6077f,0.0833f,0.7898f,  0.6276f,0.0751f,0.7749f,  0.6540f,0.1954f,0.7308f,  0.6683f,0.1913f,0.7189f,  0.6780f,0.3107f,0.6662f,  0.6862f,0.3102f,0.6580f,  0.6785f,0.4263f,0.5983f,  0.6802f,0.4288f,0.5945f,  0.6551f,0.5393f,0.5291f,  0.6504f,0.5439f,0.5303f,  0.6084f,0.6463f,0.4607f,  0.5973f,0.6520f,0.4671f,  0.5394f,0.7440f,0.3943f,  0.5224f,0.7499f,0.4060f,  
	0.4504f,0.8292f,0.3310f,  0.4280f,0.8342f,0.3478f,  0.3439f,0.8989f,0.2715f,  0.3170f,0.9021f,0.2927f,  0.2232f,0.9506f,0.2157f,  0.1927f,0.9513f,0.2408f,  0.0921f,0.9823f,0.1633f,  0.0592f,0.9797f,0.1914f,  -0.0453f,0.9925f,0.1139f,  -0.0796f,0.9864f,0.1439f,  -0.1847f,0.9805f,0.0664f,  -0.2194f,0.9708f,0.0974f,  -0.3219f,0.9465f,0.0200f,  -0.3559f,0.9331f,0.0509f,  -0.4527f,0.8913f,-0.0262f,  
	-0.4850f,0.8745f,0.0035f,  -0.5732f,0.8161f,-0.0732f,  -0.6029f,0.7965f,-0.0456f,  -0.6798f,0.7232f,-0.1217f,  -0.7062f,0.7013f,-0.0968f,  -0.7694f,0.6151f,-0.1720f,  -0.7920f,0.5918f,-0.1504f,  -0.8395f,0.4949f,-0.2242f,  -0.8578f,0.4708f,-0.2064f,  -0.8881f,0.3659f,-0.2783f,  -0.9018f,0.3418f,-0.2644f,  -0.9138f,0.2316f,-0.3336f,  -0.9230f,0.2083f,-0.3236f,  -0.9162f,0.0955f,-0.3892f,  -0.9208f,0.0739f,-0.3830f,  
	-0.8951f,-0.0388f,-0.4442f,  -0.8954f,-0.0582f,-0.4415f,  -0.8512f,-0.1680f,-0.4972f,  -0.8475f,-0.1845f,-0.4976f,  -0.7859f,-0.2889f,-0.5467f,  -0.7787f,-0.3022f,-0.5498f,  -0.7009f,-0.3989f,-0.5912f,  -0.6906f,-0.4088f,-0.5966f,  -0.5985f,-0.4957f,-0.6293f,  -0.5858f,-0.5019f,-0.6364f,  -0.4814f,-0.5774f,-0.6594f,  -0.4669f,-0.5799f,-0.6676f,  -0.3527f,-0.6425f,-0.6803f,  -0.3369f,-0.6416f,-0.6891f,  -0.2155f,-0.6903f,-0.6907f,  
	-0.1990f,-0.6861f,-0.6998f,  -0.7192f,-0.5563f,-0.4164f,  -0.7184f,-0.5573f,-0.4162f,  -0.7660f,-0.5711f,-0.2951f,  -0.7620f,-0.5768f,-0.2944f,  -0.7935f,-0.5849f,-0.1677f,  -0.7862f,-0.5951f,-0.1668f,  -0.8012f,-0.5973f,-0.0369f,  -0.7902f,-0.6117f,-0.0363f,  -0.7887f,-0.6075f,0.0945f,  -0.7741f,-0.6260f,0.0944f,  -0.7563f,-0.6148f,0.2236f,  -0.7380f,-0.6370f,0.2225f,  -0.7048f,-0.6184f,0.3477f,  -0.6829f,-0.6438f,0.3451f,  
	-0.6354f,-0.6172f,0.4641f,  -0.6100f,-0.6454f,0.4597f,  -0.5497f,-0.6102f,0.5704f,  -0.5212f,-0.6406f,0.5640f,  -0.4500f,-0.5965f,0.6646f,  -0.4186f,-0.6281f,0.6559f,  -0.3388f,-0.5747f,0.7449f,  -0.3049f,-0.6071f,0.7338f,  -0.2189f,-0.5441f,0.8099f,  -0.1833f,-0.5762f,0.7965f,  -0.0938f,-0.5037f,0.8588f,  -0.0571f,-0.5347f,0.8431f,  0.0331f,-0.4527f,0.8910f,  0.0702f,-0.4819f,0.8734f,  0.1581f,-0.3908f,0.9068f,  
	0.1947f,-0.4173f,0.8877f,  0.2774f,-0.3176f,0.9067f,  0.3125f,-0.3408f,0.8867f,  0.3875f,-0.2336f,0.8918f,  0.4201f,-0.2529f,0.8715f,  0.4846f,-0.1392f,0.8636f,  0.5137f,-0.1543f,0.8439f,  0.5656f,-0.0358f,0.8239f,  0.5904f,-0.0466f,0.8058f,  0.6276f,0.0751f,0.7749f,  0.6472f,0.0686f,0.7592f,  0.6683f,0.1913f,0.7189f,  0.6822f,0.1887f,0.7064f,  0.6862f,0.3102f,0.6580f,  0.6937f,0.3109f,0.6497f,  
	0.6802f,0.4288f,0.5945f,  0.6813f,0.4321f,0.5909f,  0.6504f,0.5439f,0.5303f,  0.6449f,0.5489f,0.5319f,  0.5973f,0.6520f,0.4671f,  0.5855f,0.6577f,0.4740f,  0.5224f,0.7499f,0.4060f,  0.5048f,0.7552f,0.4181f,  0.4280f,0.8342f,0.3478f,  0.4053f,0.8382f,0.3648f,  0.3170f,0.9021f,0.2927f,  0.2901f,0.9040f,0.3141f,  0.1927f,0.9513f,0.2408f,  0.1626f,0.9502f,0.2657f,  0.0592f,0.9797f,0.1914f,  
	0.0268f,0.9754f,0.2190f,  -0.0796f,0.9864f,0.1439f,  -0.1132f,0.9784f,0.1730f,  -0.2194f,0.9708f,0.0974f,  -0.2530f,0.9591f,0.1271f,  -0.3559f,0.9331f,0.0509f,  -0.3885f,0.9180f,0.0801f,  -0.4850f,0.8745f,0.0035f,  -0.5157f,0.8562f,0.0313f,  -0.6029f,0.7965f,-0.0456f,  -0.6310f,0.7755f,-0.0200f,  -0.7062f,0.7013f,-0.0968f,  -0.7309f,0.6785f,-0.0740f,  -0.7920f,0.5918f,-0.1504f,  -0.8128f,0.5676f,-0.1310f,  
	-0.8578f,0.4708f,-0.2064f,  -0.8744f,0.4462f,-0.1906f,  -0.9018f,0.3418f,-0.2644f,  -0.9140f,0.3176f,-0.2524f,  -0.9230f,0.2083f,-0.3236f,  -0.9307f,0.1853f,-0.3153f,  -0.9208f,0.0739f,-0.3830f,  -0.9242f,0.0526f,-0.3783f,  -0.8954f,-0.0582f,-0.4415f,  -0.8947f,-0.0770f,-0.4401f,  -0.8475f,-0.1845f,-0.4976f,  -0.8431f,-0.2004f,-0.4990f,  -0.7787f,-0.3022f,-0.5498f,  -0.7709f,-0.3148f,-0.5537f,  -0.6906f,-0.4088f,-0.5966f,  
	-0.6801f,-0.4179f,-0.6024f,  -0.5858f,-0.5019f,-0.6364f,  -0.5731f,-0.5074f,-0.6435f,  -0.4669f,-0.5799f,-0.6676f,  -0.4526f,-0.5819f,-0.6757f,  -0.3369f,-0.6416f,-0.6891f,  -0.3215f,-0.6401f,-0.6978f,  -0.1990f,-0.6861f,-0.6998f,  -0.1832f,-0.6815f,-0.7085f,  -0.7184f,-0.5573f,-0.4162f,  -0.7174f,-0.5587f,-0.4161f,  -0.7620f,-0.5768f,-0.2944f,  -0.7577f,-0.5827f,-0.2939f,  -0.7862f,-0.5951f,-0.1668f,  -0.7782f,-0.6055f,-0.1664f,  
	-0.7902f,-0.6117f,-0.0363f,  -0.7786f,-0.6265f,-0.0364f,  -0.7741f,-0.6260f,0.0944f,  -0.7586f,-0.6448f,0.0934f,  -0.7380f,-0.6370f,0.2225f,  -0.7188f,-0.6595f,0.2202f,  -0.6829f,-0.6438f,0.3451f,  -0.6599f,-0.6695f,0.3411f,  -0.6100f,-0.6454f,0.4597f,  -0.5834f,-0.6737f,0.4537f,  -0.5212f,-0.6406f,0.5640f,  -0.4912f,-0.6708f,0.5557f,  -0.4186f,-0.6281f,0.6559f,  -0.3857f,-0.6596f,0.6451f,  -0.3049f,-0.6071f,0.7338f,  
	-0.2696f,-0.6389f,0.7205f,  -0.1833f,-0.5762f,0.7965f,  -0.1462f,-0.6076f,0.7807f,  -0.0571f,-0.5347f,0.8431f,  -0.0189f,-0.5648f,0.8250f,  0.0702f,-0.4819f,0.8734f,  0.1085f,-0.5098f,0.8534f,  0.1947f,-0.4173f,0.8877f,  0.2322f,-0.4422f,0.8663f,  0.3125f,-0.3408f,0.8867f,  0.3484f,-0.3622f,0.8646f,  0.4201f,-0.2529f,0.8715f,  0.4531f,-0.2703f,0.8495f,  0.5137f,-0.1543f,0.8439f,  0.5431f,-0.1674f,0.8228f,  
	0.5904f,-0.0466f,0.8058f,  0.6150f,-0.0553f,0.7865f,  0.6472f,0.0686f,0.7592f,  0.6665f,0.0639f,0.7428f,  0.6822f,0.1887f,0.7064f,  0.6954f,0.1877f,0.6937f,  0.6937f,0.3109f,0.6497f,  0.7005f,0.3130f,0.6414f,  0.6813f,0.4321f,0.5909f,  0.6815f,0.4363f,0.5876f,  0.6449f,0.5489f,0.5319f,  0.6386f,0.5543f,0.5339f,  0.5855f,0.6577f,0.4740f,  0.5730f,0.6633f,0.4813f,  0.5048f,0.7552f,0.4181f,  
	0.4868f,0.7600f,0.4307f,  0.4053f,0.8382f,0.3648f,  0.3824f,0.8413f,0.3821f,  0.2901f,0.9040f,0.3141f,  0.2632f,0.9045f,0.3354f,  0.1626f,0.9502f,0.2657f,  0.1328f,0.9477f,0.2903f,  0.0268f,0.9754f,0.2190f,  -0.0050f,0.9693f,0.2458f,  -0.1132f,0.9784f,0.1730f,  -0.1457f,0.9687f,0.2011f,  -0.2530f,0.9591f,0.1271f,  -0.2853f,0.9458f,0.1554f,  -0.3885f,0.9180f,0.0801f,  -0.4197f,0.9013f,0.1077f,  
	-0.5157f,0.8562f,0.0313f,  -0.5449f,0.8365f,0.0572f,  -0.6310f,0.7755f,-0.0200f,  -0.6574f,0.7536f,0.0037f,  -0.7309f,0.6785f,-0.0740f,  -0.7540f,0.6548f,-0.0533f,  -0.8128f,0.5676f,-0.1310f,  -0.8320f,0.5430f,-0.1136f,  -0.8744f,0.4462f,-0.1906f,  -0.8894f,0.4214f,-0.1768f,  -0.9140f,0.3176f,-0.2524f,  -0.9248f,0.2935f,-0.2422f,  -0.9307f,0.1853f,-0.3153f,  -0.9372f,0.1625f,-0.3087f,  -0.9242f,0.0526f,-0.3783f,  
	-0.9265f,0.0319f,-0.3750f,  -0.8947f,-0.0770f,-0.4401f,  -0.8931f,-0.0951f,-0.4397f,  -0.8431f,-0.2004f,-0.4990f,  -0.8380f,-0.2155f,-0.5013f,  -0.7709f,-0.3148f,-0.5537f,  -0.7628f,-0.3267f,-0.5581f,  -0.6801f,-0.4179f,-0.6024f,  -0.6695f,-0.4262f,-0.6084f,  -0.5731f,-0.5074f,-0.6435f,  -0.5605f,-0.5122f,-0.6508f,  -0.4526f,-0.5819f,-0.6757f,  -0.4385f,-0.5832f,-0.6838f,  -0.3215f,-0.6401f,-0.6978f,  -0.3067f,-0.6382f,-0.7061f,  
	-0.1832f,-0.6815f,-0.7085f,  -0.1680f,-0.6766f,-0.7169f,  -0.7174f,-0.5587f,-0.4161f,  -0.7162f,-0.5604f,-0.4160f,  -0.7577f,-0.5827f,-0.2939f,  -0.7529f,-0.5889f,-0.2937f,  -0.7782f,-0.6055f,-0.1664f,  -0.7698f,-0.6162f,-0.1665f,  -0.7786f,-0.6265f,-0.0364f,  -0.7662f,-0.6415f,-0.0372f,  -0.7586f,-0.6448f,0.0934f,  -0.7423f,-0.6638f,0.0915f,  -0.7188f,-0.6595f,0.2202f,  -0.6984f,-0.6821f,0.2166f,  -0.6599f,-0.6695f,0.3411f,  
	-0.6356f,-0.6952f,0.3356f,  -0.5834f,-0.6737f,0.4537f,  -0.5554f,-0.7019f,0.4459f,  -0.4912f,-0.6708f,0.5557f,  -0.4598f,-0.7008f,0.5454f,  -0.3857f,-0.6596f,0.6451f,  -0.3513f,-0.6906f,0.6322f,  -0.2696f,-0.6389f,0.7205f,  -0.2329f,-0.6700f,0.7049f,  -0.1462f,-0.6076f,0.7807f,  -0.1077f,-0.6379f,0.7625f,  -0.0189f,-0.5648f,0.8250f,  0.0205f,-0.5935f,0.8046f,  0.1085f,-0.5098f,0.8534f,  0.1480f,-0.5361f,0.8311f,  
	0.2322f,-0.4422f,0.8663f,  0.2707f,-0.4653f,0.8427f,  0.3484f,-0.3622f,0.8646f,  0.3848f,-0.3816f,0.8405f,  0.4531f,-0.2703f,0.8495f,  0.4865f,-0.2854f,0.8258f,  0.5431f,-0.1674f,0.8228f,  0.5723f,-0.1783f,0.8004f,  0.6150f,-0.0553f,0.7865f,  0.6394f,-0.0619f,0.7664f,  0.6665f,0.0639f,0.7428f,  0.6851f,0.0613f,0.7259f,  0.6954f,0.1877f,0.6937f,  0.7078f,0.1884f,0.6808f,  0.7005f,0.3130f,0.6414f,  
	0.7064f,0.3163f,0.6332f,  0.6815f,0.4363f,0.5876f,  0.6808f,0.4413f,0.5846f,  0.6386f,0.5543f,0.5339f,  0.6315f,0.5600f,0.5363f,  0.5730f,0.6633f,0.4813f,  0.5599f,0.6687f,0.4891f,  0.4868f,0.7600f,0.4307f,  0.4683f,0.7642f,0.4435f,  0.3824f,0.8413f,0.3821f,  0.3594f,0.8434f,0.3995f,  0.2632f,0.9045f,0.3354f,  0.2365f,0.9038f,0.3566f,  0.1328f,0.9477f,0.2903f,  0.1033f,0.9437f,0.3143f,  
	-0.0050f,0.9693f,0.2458f,  -0.0360f,0.9617f,0.2718f,  -0.1457f,0.9687f,0.2011f,  -0.1773f,0.9574f,0.2280f,  -0.2853f,0.9458f,0.1554f,  -0.3165f,0.9309f,0.1822f,  -0.4197f,0.9013f,0.1077f,  -0.4496f,0.8832f,0.1334f,  -0.5449f,0.8365f,0.0572f,  -0.5726f,0.8158f,0.0813f,  -0.6574f,0.7536f,0.0037f,  -0.6822f,0.7307f,0.0252f,  -0.7540f,0.6548f,-0.0533f,  -0.7754f,0.6305f,-0.0347f,  -0.8320f,0.5430f,-0.1136f,  
	-0.8497f,0.5181f,-0.0983f,  -0.8894f,0.4214f,-0.1768f,  -0.9030f,0.3967f,-0.1649f,  -0.9248f,0.2935f,-0.2422f,  -0.9342f,0.2696f,-0.2337f,  -0.9372f,0.1625f,-0.3087f,  -0.9425f,0.1402f,-0.3035f,  -0.9265f,0.0319f,-0.3750f,  -0.9278f,0.0118f,-0.3729f,  -0.8931f,-0.0951f,-0.4397f,  -0.8908f,-0.1125f,-0.4403f,  -0.8380f,-0.2155f,-0.5013f,  -0.8324f,-0.2299f,-0.5043f,  -0.7628f,-0.3267f,-0.5581f,  -0.7544f,-0.3377f,-0.5629f,  
	-0.6695f,-0.4262f,-0.6084f,  -0.6587f,-0.4338f,-0.6147f,  -0.5605f,-0.5122f,-0.6508f,  -0.5480f,-0.5163f,-0.6581f,  -0.4385f,-0.5832f,-0.6838f,  -0.4249f,-0.5840f,-0.6917f,  -0.3067f,-0.6382f,-0.7061f,  -0.2923f,-0.6359f,-0.7143f,  -0.1680f,-0.6766f,-0.7169f,  -0.1535f,-0.6715f,-0.7250f,  -0.7162f,-0.5604f,-0.4160f,  -0.7147f,-0.5623f,-0.4160f,  -0.7529f,-0.5889f,-0.2937f,  -0.7477f,-0.5955f,-0.2938f,  -0.7698f,-0.6162f,-0.1665f,  
	-0.7607f,-0.6273f,-0.1671f,  -0.7662f,-0.6415f,-0.0372f,  -0.7531f,-0.6568f,-0.0388f,  -0.7423f,-0.6638f,0.0915f,  -0.7250f,-0.6831f,0.0885f,  -0.6984f,-0.6821f,0.2166f,  -0.6770f,-0.7049f,0.2118f,  -0.6356f,-0.6952f,0.3356f,  -0.6101f,-0.7210f,0.3285f,  -0.5554f,-0.7019f,0.4459f,  -0.5261f,-0.7300f,0.4363f,  -0.4598f,-0.7008f,0.5454f,  -0.4270f,-0.7304f,0.5331f,  -0.3513f,-0.6906f,0.6322f,  -0.3155f,-0.7209f,0.6170f,  
	-0.2329f,-0.6700f,0.7049f,  -0.1947f,-0.7002f,0.6869f,  -0.1077f,-0.6379f,0.7625f,  -0.0679f,-0.6671f,0.7419f,  0.0205f,-0.5935f,0.8046f,  0.0611f,-0.6207f,0.7817f,  0.1480f,-0.5361f,0.8311f,  0.1883f,-0.5605f,0.8065f,  0.2707f,-0.4653f,0.8427f,  0.3098f,-0.4863f,0.8170f,  0.3848f,-0.3816f,0.8405f,  0.4215f,-0.3986f,0.8145f,  0.4865f,-0.2854f,0.8258f,  0.5198f,-0.2982f,0.8005f,  0.5723f,-0.1783f,0.8004f,  
	0.6013f,-0.1867f,0.7769f,  0.6394f,-0.0619f,0.7664f,  0.6631f,-0.0663f,0.7456f,  0.6851f,0.0613f,0.7259f,  0.7030f,0.0606f,0.7086f,  0.7078f,0.1884f,0.6808f,  0.7193f,0.1908f,0.6680f,  0.7064f,0.3163f,0.6332f,  0.7113f,0.3208f,0.6253f,  0.6808f,0.4413f,0.5846f,  0.6791f,0.4471f,0.5821f,  0.6315f,0.5600f,0.5363f,  0.6235f,0.5660f,0.5393f,  0.5599f,0.6687f,0.4891f,  0.5462f,0.6740f,0.4974f,  
	0.4683f,0.7642f,0.4435f,  0.4494f,0.7678f,0.4567f,  0.3594f,0.8434f,0.3995f,  0.3362f,0.8445f,0.4169f,  0.2365f,0.9038f,0.3566f,  0.2099f,0.9019f,0.3775f,  0.1033f,0.9437f,0.3143f,  0.0744f,0.9383f,0.3378f,  -0.0360f,0.9617f,0.2718f,  -0.0663f,0.9526f,0.2968f,  -0.1773f,0.9574f,0.2280f,  -0.2079f,0.9447f,0.2537f,  -0.3165f,0.9309f,0.1822f,  -0.3466f,0.9148f,0.2074f,  -0.4496f,0.8832f,0.1334f,  
	-0.4781f,0.8641f,0.1575f,  -0.5726f,0.8158f,0.0813f,  -0.5989f,0.7941f,0.1034f,  -0.6822f,0.7307f,0.0252f,  -0.7056f,0.7072f,0.0448f,  -0.7754f,0.6305f,-0.0347f,  -0.7954f,0.6059f,-0.0181f,  -0.8497f,0.5181f,-0.0983f,  -0.8658f,0.4931f,-0.0848f,  -0.9030f,0.3967f,-0.1649f,  -0.9152f,0.3721f,-0.1548f,  -0.9342f,0.2696f,-0.2337f,  -0.9423f,0.2461f,-0.2268f,  -0.9425f,0.1402f,-0.3035f,  -0.9467f,0.1185f,-0.2997f,  
	-0.9278f,0.0118f,-0.3729f,  -0.9283f,-0.0076f,-0.3719f,  -0.8908f,-0.1125f,-0.4403f,  -0.8877f,-0.1292f,-0.4419f,  -0.8324f,-0.2299f,-0.5043f,  -0.8264f,-0.2434f,-0.5078f,  -0.7544f,-0.3377f,-0.5629f,  -0.7458f,-0.3479f,-0.5682f,  -0.6587f,-0.4338f,-0.6147f,  -0.6481f,-0.4406f,-0.6212f,  -0.5480f,-0.5163f,-0.6581f,  -0.5358f,-0.5198f,-0.6654f,  -0.4249f,-0.5840f,-0.6917f,  -0.4116f,-0.5843f,-0.6994f,  -0.2923f,-0.6359f,-0.7143f,  
	-0.2786f,-0.6332f,-0.7222f,  -0.1535f,-0.6715f,-0.7250f,  -0.1397f,-0.6661f,-0.7327f,  -0.7147f,-0.5623f,-0.4160f,  -0.7129f,-0.5645f,-0.4160f,  -0.7477f,-0.5955f,-0.2938f,  -0.7421f,-0.6023f,-0.2942f,  -0.7607f,-0.6273f,-0.1671f,  -0.7510f,-0.6385f,-0.1683f,  -0.7531f,-0.6568f,-0.0388f,  -0.7391f,-0.6723f,-0.0412f,  -0.7250f,-0.6831f,0.0885f,  -0.7067f,-0.7025f,0.0843f,  -0.6770f,-0.7049f,0.2118f,  -0.6544f,-0.7277f,0.2055f,  
	-0.6101f,-0.7210f,0.3285f,  -0.5834f,-0.7466f,0.3198f,  -0.5261f,-0.7300f,0.4363f,  -0.4954f,-0.7577f,0.4248f,  -0.4270f,-0.7304f,0.5331f,  -0.3928f,-0.7594f,0.5186f,  -0.3155f,-0.7209f,0.6170f,  -0.2783f,-0.7504f,0.5996f,  -0.1947f,-0.7002f,0.6869f,  -0.1551f,-0.7292f,0.6665f,  -0.0679f,-0.6671f,0.7419f,  -0.0268f,-0.6947f,0.7188f,  0.0611f,-0.6207f,0.7817f,  0.1027f,-0.6460f,0.7564f,  0.1883f,-0.5605f,0.8065f,  
	0.2295f,-0.5828f,0.7796f,  0.3098f,-0.4863f,0.8170f,  0.3494f,-0.5049f,0.7893f,  0.4215f,-0.3986f,0.8145f,  0.4584f,-0.4131f,0.7869f,  0.5198f,-0.2982f,0.8005f,  0.5530f,-0.3084f,0.7740f,  0.6013f,-0.1867f,0.7769f,  0.6298f,-0.1927f,0.7525f,  0.6631f,-0.0663f,0.7456f,  0.6862f,-0.0682f,0.7243f,  0.7030f,0.0606f,0.7086f,  0.7199f,0.0620f,0.6913f,  0.7193f,0.1908f,0.6680f,  0.7298f,0.1948f,0.6554f,  
	0.7113f,0.3208f,0.6253f,  0.7152f,0.3266f,0.6179f,  0.6791f,0.4471f,0.5821f,  0.6765f,0.4536f,0.5802f,  0.6235f,0.5660f,0.5393f,  0.6147f,0.5723f,0.5428f,  0.5462f,0.6740f,0.4974f,  0.5318f,0.6790f,0.5061f,  0.4494f,0.7678f,0.4567f,  0.4301f,0.7707f,0.4701f,  0.3362f,0.8445f,0.4169f,  0.3128f,0.8447f,0.4342f,  0.2099f,0.9019f,0.3775f,  0.1835f,0.8988f,0.3980f,  0.0744f,0.9383f,0.3378f,  
	0.0459f,0.9316f,0.3605f,  -0.0663f,0.9526f,0.2968f,  -0.0958f,0.9423f,0.3208f,  -0.2079f,0.9447f,0.2537f,  -0.2376f,0.9308f,0.2778f,  -0.3466f,0.9148f,0.2074f,  -0.3754f,0.8976f,0.2311f,  -0.4781f,0.8641f,0.1575f,  -0.5053f,0.8440f,0.1798f,  -0.5989f,0.7941f,0.1034f,  -0.6237f,0.7718f,0.1236f,  -0.7056f,0.7072f,0.0448f,  -0.7275f,0.6833f,0.0624f,  -0.7954f,0.6059f,-0.0181f,  -0.8138f,0.5811f,-0.0033f,  
	-0.8658f,0.4931f,-0.0848f,  -0.8806f,0.4682f,-0.0732f,  -0.9152f,0.3721f,-0.1548f,  -0.9261f,0.3478f,-0.1462f,  -0.9423f,0.2461f,-0.2268f,  -0.9493f,0.2231f,-0.2213f,  -0.9467f,0.1185f,-0.2997f,  -0.9499f,0.0974f,-0.2970f,  -0.9283f,-0.0076f,-0.3719f,  -0.9279f,-0.0263f,-0.3719f,  -0.8877f,-0.1292f,-0.4419f,  -0.8842f,-0.1450f,-0.4440f,  -0.8264f,-0.2434f,-0.5078f,  -0.8200f,-0.2561f,-0.5119f,  -0.7458f,-0.3479f,-0.5682f,  
	-0.7370f,-0.3573f,-0.5737f,  -0.6481f,-0.4406f,-0.6212f,  -0.6375f,-0.4467f,-0.6277f,  -0.5358f,-0.5198f,-0.6654f,  -0.5239f,-0.5226f,-0.6726f,  -0.4116f,-0.5843f,-0.6994f,  -0.3989f,-0.5840f,-0.7070f,  -0.2786f,-0.6332f,-0.7222f,  -0.2655f,-0.6301f,-0.7297f,  -0.1397f,-0.6661f,-0.7327f,  -0.1266f,-0.6606f,-0.7400f,  -0.7129f,-0.5645f,-0.4160f,  -0.7108f,-0.5670f,-0.4162f,  -0.7421f,-0.6023f,-0.2942f,  -0.7360f,-0.6094f,-0.2950f,  
	-0.7510f,-0.6385f,-0.1683f,  -0.7406f,-0.6501f,-0.1702f,  -0.7391f,-0.6723f,-0.0412f,  -0.7243f,-0.6880f,-0.0446f,  -0.7067f,-0.7025f,0.0843f,  -0.6874f,-0.7220f,0.0790f,  -0.6544f,-0.7277f,0.2055f,  -0.6306f,-0.7505f,0.1977f,  -0.5834f,-0.7466f,0.3198f,  -0.5553f,-0.7720f,0.3093f,  -0.4954f,-0.7577f,0.4248f,  -0.4633f,-0.7850f,0.4113f,  -0.3928f,-0.7594f,0.5186f,  -0.3571f,-0.7877f,0.5019f,  -0.2783f,-0.7504f,0.5996f,  
	-0.2397f,-0.7788f,0.5797f,  -0.1551f,-0.7292f,0.6665f,  -0.1142f,-0.7567f,0.6437f,  -0.0268f,-0.6947f,0.7188f,  0.0154f,-0.7205f,0.6933f,  0.1027f,-0.6460f,0.7564f,  0.1452f,-0.6692f,0.7288f,  0.2295f,-0.5828f,0.7796f,  0.2712f,-0.6026f,0.7506f,  0.3494f,-0.5049f,0.7893f,  0.3892f,-0.5209f,0.7598f,  0.4584f,-0.4131f,0.7869f,  0.4952f,-0.4248f,0.7578f,  0.5530f,-0.3084f,0.7740f,  0.5857f,-0.3158f,0.7464f,  
	0.6298f,-0.1927f,0.7525f,  0.6576f,-0.1960f,0.7275f,  0.6862f,-0.0682f,0.7243f,  0.7082f,-0.0678f,0.7028f,  0.7199f,0.0620f,0.6913f,  0.7357f,0.0655f,0.6741f,  0.7298f,0.1948f,0.6554f,  0.7390f,0.2004f,0.6432f,  0.7152f,0.3266f,0.6179f,  0.7179f,0.3335f,0.6111f,  0.6765f,0.4536f,0.5802f,  0.6728f,0.4608f,0.5788f,  0.6147f,0.5723f,0.5428f,  0.6051f,0.5787f,0.5468f,  0.5318f,0.6790f,0.5061f,  
	0.5168f,0.6838f,0.5152f,  0.4301f,0.7707f,0.4701f,  0.4105f,0.7731f,0.4835f,  0.3128f,0.8447f,0.4342f,  0.2895f,0.8440f,0.4514f,  0.1835f,0.8988f,0.3980f,  0.1573f,0.8947f,0.4180f,  0.0459f,0.9316f,0.3605f,  0.0180f,0.9238f,0.3824f,  -0.0958f,0.9423f,0.3208f,  -0.1246f,0.9309f,0.3435f,  -0.2376f,0.9308f,0.2778f,  -0.2662f,0.9158f,0.3007f,  -0.3754f,0.8976f,0.2311f,  -0.4030f,0.8795f,0.2531f,  
	-0.5053f,0.8440f,0.1798f,  -0.5312f,0.8233f,0.2002f,  -0.6237f,0.7718f,0.1236f,  -0.6472f,0.7490f,0.1419f,  -0.7275f,0.6833f,0.0624f,  -0.7480f,0.6591f,0.0782f,  -0.8138f,0.5811f,-0.0033f,  -0.8309f,0.5563f,0.0096f,  -0.8806f,0.4682f,-0.0732f,  -0.8940f,0.4436f,-0.0632f,  -0.9261f,0.3478f,-0.1462f,  -0.9358f,0.3240f,-0.1392f,  -0.9493f,0.2231f,-0.2213f,  -0.9553f,0.2008f,-0.2171f,  -0.9499f,0.0974f,-0.2970f,  
	-0.9522f,0.0771f,-0.2956f,  -0.9279f,-0.0263f,-0.3719f,  -0.9269f,-0.0441f,-0.3727f,  -0.8842f,-0.1450f,-0.4440f,  -0.8802f,-0.1599f,-0.4468f,  -0.8200f,-0.2561f,-0.5119f,  -0.8134f,-0.2679f,-0.5163f,  -0.7370f,-0.3573f,-0.5737f,  -0.7283f,-0.3659f,-0.5793f,  -0.6375f,-0.4467f,-0.6277f,  -0.6271f,-0.4521f,-0.6343f,  -0.5239f,-0.5226f,-0.6726f,  -0.5123f,-0.5249f,-0.6797f,  -0.3989f,-0.5840f,-0.7070f,  -0.3867f,-0.5834f,-0.7143f,  
	-0.2655f,-0.6301f,-0.7297f,  -0.2530f,-0.6268f,-0.7370f,  -0.1266f,-0.6606f,-0.7400f,  -0.1143f,-0.6549f,-0.7470f,  -0.7108f,-0.5670f,-0.4162f,  -0.7084f,-0.5698f,-0.4166f,  -0.7360f,-0.6094f,-0.2950f,  -0.7294f,-0.6167f,-0.2962f,  -0.7406f,-0.6501f,-0.1702f,  -0.7295f,-0.6618f,-0.1727f,  -0.7243f,-0.6880f,-0.0446f,  -0.7086f,-0.7039f,-0.0489f,  -0.6874f,-0.7220f,0.0790f,  -0.6670f,-0.7415f,0.0723f,  -0.6306f,-0.7505f,0.1977f,  
	-0.6056f,-0.7731f,0.1884f,  -0.5553f,-0.7720f,0.3093f,  -0.5259f,-0.7970f,0.2969f,  -0.4633f,-0.7850f,0.4113f,  -0.4299f,-0.8116f,0.3957f,  -0.3571f,-0.7877f,0.5019f,  -0.3201f,-0.8150f,0.4830f,  -0.2397f,-0.7788f,0.5797f,  -0.1998f,-0.8058f,0.5575f,  -0.1142f,-0.7567f,0.6437f,  -0.0723f,-0.7825f,0.6184f,  0.0154f,-0.7205f,0.6933f,  0.0585f,-0.7441f,0.6655f,  0.1452f,-0.6692f,0.7288f,  0.1883f,-0.6899f,0.6990f,  
	0.2712f,-0.6026f,0.7506f,  0.3132f,-0.6197f,0.7196f,  0.3892f,-0.5209f,0.7598f,  0.4289f,-0.5339f,0.7287f,  0.4952f,-0.4248f,0.7578f,  0.5316f,-0.4336f,0.7276f,  0.5857f,-0.3158f,0.7464f,  0.6178f,-0.3203f,0.7182f,  0.6576f,-0.1960f,0.7275f,  0.6844f,-0.1966f,0.7021f,  0.7082f,-0.0678f,0.7028f,  0.7291f,-0.0651f,0.6813f,  0.7357f,0.0655f,0.6741f,  0.7502f,0.0708f,0.6574f,  0.7390f,0.2004f,0.6432f,  
	0.7470f,0.2076f,0.6316f,  0.7179f,0.3335f,0.6111f,  0.7194f,0.3414f,0.6050f,  0.6728f,0.4608f,0.5788f,  0.6680f,0.4684f,0.5782f,  0.6051f,0.5787f,0.5468f,  0.5945f,0.5851f,0.5515f,  0.5168f,0.6838f,0.5152f,  0.5011f,0.6883f,0.5246f,  0.4105f,0.7731f,0.4835f,  0.3905f,0.7748f,0.4972f,  0.2895f,0.8440f,0.4514f,  0.2661f,0.8425f,0.4684f,  0.1573f,0.8947f,0.4180f,  0.1315f,0.8896f,0.4375f,  
	0.0180f,0.9238f,0.3824f,  -0.0095f,0.9150f,0.4033f,  -0.1246f,0.9309f,0.3435f,  -0.1526f,0.9184f,0.3652f,  -0.2662f,0.9158f,0.3007f,  -0.2939f,0.8999f,0.3221f,  -0.4030f,0.8795f,0.2531f,  -0.4296f,0.8606f,0.2734f,  -0.5312f,0.8233f,0.2002f,  -0.5558f,0.8020f,0.2189f,  -0.6472f,0.7490f,0.1419f,  -0.6693f,0.7259f,0.1585f,  -0.7480f,0.6591f,0.0782f,  -0.7671f,0.6349f,0.0923f,  -0.8309f,0.5563f,0.0096f,  
	-0.8467f,0.5317f,0.0209f,  -0.8940f,0.4436f,-0.0632f,  -0.9062f,0.4193f,-0.0547f,  -0.9358f,0.3240f,-0.1392f,  -0.9443f,0.3007f,-0.1335f,  -0.9553f,0.2008f,-0.2171f,  -0.9602f,0.1791f,-0.2142f,  -0.9522f,0.0771f,-0.2956f,  -0.9538f,0.0576f,-0.2949f,  -0.9269f,-0.0441f,-0.3727f,  -0.9254f,-0.0610f,-0.3742f,  -0.8802f,-0.1599f,-0.4468f,  -0.8758f,-0.1740f,-0.4502f,  -0.8134f,-0.2679f,-0.5163f,  -0.8067f,-0.2789f,-0.5210f,  
	-0.7283f,-0.3659f,-0.5793f,  -0.7197f,-0.3737f,-0.5851f,  -0.6271f,-0.4521f,-0.6343f,  -0.6170f,-0.4568f,-0.6408f,  -0.5123f,-0.5249f,-0.6797f,  -0.5013f,-0.5266f,-0.6866f,  -0.3867f,-0.5834f,-0.7143f,  -0.3750f,-0.5823f,-0.7213f,  -0.2530f,-0.6268f,-0.7370f,  -0.2412f,-0.6232f,-0.7439f,  -0.1143f,-0.6549f,-0.7470f,  -0.1027f,-0.6492f,-0.7537f,  -0.7084f,-0.5698f,-0.4166f,  -0.7057f,-0.5728f,-0.4171f,  -0.7294f,-0.6167f,-0.2962f,  
	-0.7222f,-0.6243f,-0.2978f,  -0.7295f,-0.6618f,-0.1727f,  -0.7177f,-0.6738f,-0.1759f,  -0.7086f,-0.7039f,-0.0489f,  -0.6920f,-0.7198f,-0.0543f,  -0.6670f,-0.7415f,0.0723f,  -0.6456f,-0.7610f,0.0644f,  -0.6056f,-0.7731f,0.1884f,  -0.5795f,-0.7954f,0.1775f,  -0.5259f,-0.7970f,0.2969f,  -0.4952f,-0.8215f,0.2827f,  -0.4299f,-0.8116f,0.3957f,  -0.3951f,-0.8373f,0.3780f,  -0.3201f,-0.8150f,0.4830f,  -0.2818f,-0.8411f,0.4617f,  
	-0.1998f,-0.8058f,0.5575f,  -0.1587f,-0.8312f,0.5329f,  -0.0723f,-0.7825f,0.6184f,  -0.0293f,-0.8063f,0.5908f,  0.0585f,-0.7441f,0.6655f,  0.1022f,-0.7654f,0.6354f,  0.1883f,-0.6899f,0.6990f,  0.2319f,-0.7079f,0.6672f,  0.3132f,-0.6197f,0.7196f,  0.3553f,-0.6339f,0.6870f,  0.4289f,-0.5339f,0.7287f,  0.4684f,-0.5439f,0.6963f,  0.5316f,-0.4336f,0.7276f,  0.5674f,-0.4392f,0.6965f,  0.6178f,-0.3203f,0.7182f,  
	0.6488f,-0.3219f,0.6895f,  0.6844f,-0.1966f,0.7021f,  0.7100f,-0.1945f,0.6768f,  0.7291f,-0.0651f,0.6813f,  0.7486f,-0.0600f,0.6603f,  0.7502f,0.0708f,0.6574f,  0.7634f,0.0781f,0.6412f,  0.7470f,0.2076f,0.6316f,  0.7536f,0.2161f,0.6208f,  0.7194f,0.3414f,0.6050f,  0.7196f,0.3501f,0.5997f,  0.6680f,0.4684f,0.5782f,  0.6622f,0.4765f,0.5783f,  0.5945f,0.5851f,0.5515f,  0.5832f,0.5916f,0.5567f,  
	0.5011f,0.6883f,0.5246f,  0.4849f,0.6923f,0.5344f,  0.3905f,0.7748f,0.4972f,  0.3703f,0.7759f,0.5108f,  0.2661f,0.8425f,0.4684f,  0.2427f,0.8401f,0.4851f,  0.1315f,0.8896f,0.4375f,  0.1058f,0.8835f,0.4562f,  -0.0095f,0.9150f,0.4033f,  -0.0363f,0.9052f,0.4234f,  -0.1526f,0.9184f,0.3652f,  -0.1798f,0.9050f,0.3856f,  -0.2939f,0.8999f,0.3221f,  -0.3206f,0.8833f,0.3420f,  -0.4296f,0.8606f,0.2734f,  
	-0.4550f,0.8412f,0.2922f,  -0.5558f,0.8020f,0.2189f,  -0.5792f,0.7803f,0.2360f,  -0.6693f,0.7259f,0.1585f,  -0.6901f,0.7026f,0.1733f,  -0.7671f,0.6349f,0.0923f,  -0.7849f,0.6108f,0.1047f,  -0.8467f,0.5317f,0.0209f,  -0.8612f,0.5074f,0.0307f,  -0.9062f,0.4193f,-0.0547f,  -0.9172f,0.3955f,-0.0477f,  -0.9443f,0.3007f,-0.1335f,  -0.9518f,0.2781f,-0.1291f,  -0.9602f,0.1791f,-0.2142f,  -0.9643f,0.1583f,-0.2121f,  
	-0.9538f,0.0576f,-0.2949f,  -0.9547f,0.0390f,-0.2951f,  -0.9254f,-0.0610f,-0.3742f,  -0.9233f,-0.0770f,-0.3763f,  -0.8758f,-0.1740f,-0.4502f,  -0.8712f,-0.1871f,-0.4538f,  -0.8067f,-0.2789f,-0.5210f,  -0.7999f,-0.2890f,-0.5259f,  -0.7197f,-0.3737f,-0.5851f,  -0.7112f,-0.3808f,-0.5910f,  -0.6170f,-0.4568f,-0.6408f,  -0.6073f,-0.4608f,-0.6472f,  -0.5013f,-0.5266f,-0.6866f,  -0.4906f,-0.5278f,-0.6933f,  -0.3750f,-0.5823f,-0.7213f,  
	-0.3640f,-0.5809f,-0.7281f,  -0.2412f,-0.6232f,-0.7439f,  -0.2301f,-0.6195f,-0.7505f,  -0.1027f,-0.6492f,-0.7537f,  -0.0919f,-0.6434f,-0.7600f,  -0.7057f,-0.5728f,-0.4171f,  -0.7025f,-0.5761f,-0.4178f,  -0.7222f,-0.6243f,-0.2978f,  -0.7145f,-0.6321f,-0.2999f,  -0.7177f,-0.6738f,-0.1759f,  -0.7051f,-0.6858f,-0.1800f,  -0.6920f,-0.7198f,-0.0543f,  -0.6745f,-0.7358f,-0.0608f,  -0.6456f,-0.7610f,0.0644f,  -0.6231f,-0.7802f,0.0550f,  
	-0.5795f,-0.7954f,0.1775f,  -0.5521f,-0.8173f,0.1648f,  -0.4952f,-0.8215f,0.2827f,  -0.4633f,-0.8452f,0.2665f,  -0.3951f,-0.8373f,0.3780f,  -0.3591f,-0.8619f,0.3581f,  -0.2818f,-0.8411f,0.4617f,  -0.2424f,-0.8656f,0.4382f,  -0.1587f,-0.8312f,0.5329f,  -0.1166f,-0.8546f,0.5060f,  -0.0293f,-0.8063f,0.5908f,  0.0144f,-0.8277f,0.5609f,  0.1022f,-0.7654f,0.6354f,  0.1465f,-0.7840f,0.6033f,  0.2319f,-0.7079f,0.6672f,  
	0.2755f,-0.7229f,0.6336f,  0.3553f,-0.6339f,0.6870f,  0.3971f,-0.6449f,0.6530f,  0.4684f,-0.5439f,0.6963f,  0.5072f,-0.5506f,0.6630f,  0.5674f,-0.4392f,0.6965f,  0.6022f,-0.4418f,0.6650f,  0.6488f,-0.3219f,0.6895f,  0.6787f,-0.3206f,0.6607f,  0.7100f,-0.1945f,0.6768f,  0.7342f,-0.1898f,0.6519f,  0.7486f,-0.0600f,0.6603f,  0.7667f,-0.0527f,0.6399f,  0.7634f,0.0781f,0.6412f,  0.7750f,0.0871f,0.6259f,  
	0.7536f,0.2161f,0.6208f,  0.7588f,0.2258f,0.6109f,  0.7196f,0.3501f,0.5997f,  0.7185f,0.3596f,0.5953f,  0.6622f,0.4765f,0.5783f,  0.6553f,0.4849f,0.5792f,  0.5832f,0.5916f,0.5567f,  0.5710f,0.5980f,0.5625f,  0.4849f,0.6923f,0.5344f,  0.4682f,0.6960f,0.5445f,  0.3703f,0.7759f,0.5108f,  0.3498f,0.7763f,0.5244f,  0.2427f,0.8401f,0.4851f,  0.2193f,0.8370f,0.5014f,  0.1058f,0.8835f,0.4562f,  
	0.0806f,0.8766f,0.4744f,  -0.0363f,0.9052f,0.4234f,  -0.0626f,0.8946f,0.4425f,  -0.1798f,0.9050f,0.3856f,  -0.2062f,0.8909f,0.4047f,  -0.3206f,0.8833f,0.3420f,  -0.3463f,0.8661f,0.3606f,  -0.4550f,0.8412f,0.2922f,  -0.4792f,0.8213f,0.3095f,  -0.5792f,0.7803f,0.2360f,  -0.6013f,0.7584f,0.2515f,  -0.6901f,0.7026f,0.1733f,  -0.7096f,0.6794f,0.1867f,  -0.7849f,0.6108f,0.1047f,  -0.8014f,0.5869f,0.1156f,  
	-0.8612f,0.5074f,0.0307f,  -0.8745f,0.4835f,0.0390f,  -0.9172f,0.3955f,-0.0477f,  -0.9272f,0.3723f,-0.0419f,  -0.9518f,0.2781f,-0.1291f,  -0.9584f,0.2563f,-0.1257f,  -0.9643f,0.1583f,-0.2121f,  -0.9677f,0.1383f,-0.2110f,  -0.9547f,0.0390f,-0.2951f,  -0.9550f,0.0212f,-0.2960f,  -0.9233f,-0.0770f,-0.3763f,  -0.9209f,-0.0921f,-0.3788f,  -0.8712f,-0.1871f,-0.4538f,  -0.8664f,-0.1994f,-0.4578f,  -0.7999f,-0.2890f,-0.5259f,  
	-0.7931f,-0.2984f,-0.5310f,  -0.7112f,-0.3808f,-0.5910f,  -0.7029f,-0.3872f,-0.5967f,  -0.6073f,-0.4608f,-0.6472f,  -0.5978f,-0.4643f,-0.6535f,  -0.4906f,-0.5278f,-0.6933f,  -0.4805f,-0.5286f,-0.6998f,  -0.3640f,-0.5809f,-0.7281f,  -0.3536f,-0.5792f,-0.7346f,  -0.2301f,-0.6195f,-0.7505f,  -0.2197f,-0.6156f,-0.7568f,  -0.0919f,-0.6434f,-0.7600f,  -0.0818f,-0.6377f,-0.7659f,  -0.7025f,-0.5761f,-0.4178f,  -0.6990f,-0.5796f,-0.4188f,  
	-0.7145f,-0.6321f,-0.2999f,  -0.7062f,-0.6401f,-0.3026f,  -0.7051f,-0.6858f,-0.1800f,  -0.6918f,-0.6980f,-0.1849f,  -0.6745f,-0.7358f,-0.0608f,  -0.6560f,-0.7516f,-0.0684f,  -0.6231f,-0.7802f,0.0550f,  -0.5995f,-0.7992f,0.0442f,  -0.5521f,-0.8173f,0.1648f,  -0.5236f,-0.8386f,0.1504f,  -0.4633f,-0.8452f,0.2665f,  -0.4302f,-0.8679f,0.2483f,  -0.3591f,-0.8619f,0.3581f,  -0.3220f,-0.8851f,0.3361f,  -0.2424f,-0.8656f,0.4382f,  
	-0.2020f,-0.8883f,0.4125f,  -0.1166f,-0.8546f,0.5060f,  -0.0738f,-0.8759f,0.4769f,  0.0144f,-0.8277f,0.5609f,  0.0586f,-0.8466f,0.5290f,  0.1465f,-0.7840f,0.6033f,  0.1910f,-0.7996f,0.5693f,  0.2755f,-0.7229f,0.6336f,  0.3190f,-0.7348f,0.5986f,  0.3971f,-0.6449f,0.6530f,  0.4384f,-0.6526f,0.6180f,  0.5072f,-0.5506f,0.6630f,  0.5452f,-0.5541f,0.6290f,  0.6022f,-0.4418f,0.6650f,  0.6358f,-0.4413f,0.6332f,  
	0.6787f,-0.3206f,0.6607f,  0.7072f,-0.3164f,0.6322f,  0.7342f,-0.1898f,0.6519f,  0.7569f,-0.1827f,0.6275f,  0.7667f,-0.0527f,0.6399f,  0.7832f,-0.0434f,0.6203f,  0.7750f,0.0871f,0.6259f,  0.7851f,0.0977f,0.6116f,  0.7588f,0.2258f,0.6109f,  0.7626f,0.2366f,0.6020f,  0.7185f,0.3596f,0.5953f,  0.7162f,0.3698f,0.5918f,  0.6553f,0.4849f,0.5792f,  0.6473f,0.4934f,0.5809f,  0.5710f,0.5980f,0.5625f,  
	0.5580f,0.6042f,0.5688f,  0.4682f,0.6960f,0.5445f,  0.4509f,0.6992f,0.5548f,  0.3498f,0.7763f,0.5244f,  0.3291f,0.7761f,0.5379f,  0.2193f,0.8370f,0.5014f,  0.1961f,0.8330f,0.5173f,  0.0806f,0.8766f,0.4744f,  0.0557f,0.8689f,0.4918f,  -0.0626f,0.8946f,0.4425f,  -0.0883f,0.8832f,0.4606f,  -0.2062f,0.8909f,0.4047f,  -0.2318f,0.8761f,0.4227f,  -0.3463f,0.8661f,0.3606f,  -0.3710f,0.8483f,0.3777f,  
	-0.4792f,0.8213f,0.3095f,  -0.5023f,0.8012f,0.3253f,  -0.6013f,0.7584f,0.2515f,  -0.6223f,0.7364f,0.2655f,  -0.7096f,0.6794f,0.1867f,  -0.7279f,0.6563f,0.1985f,  -0.8014f,0.5869f,0.1156f,  -0.8167f,0.5633f,0.1250f,  -0.8745f,0.4835f,0.0390f,  -0.8866f,0.4602f,0.0460f,  -0.9272f,0.3723f,-0.0419f,  -0.9361f,0.3498f,-0.0372f,  -0.9584f,0.2563f,-0.1257f,  -0.9641f,0.2352f,-0.1233f,  -0.9677f,0.1383f,-0.2110f,  
	-0.9703f,0.1191f,-0.2107f,  -0.9550f,0.0212f,-0.2960f,  -0.9547f,0.0044f,-0.2974f,  -0.9209f,-0.0921f,-0.3788f,  -0.9181f,-0.1063f,-0.3818f,  -0.8664f,-0.1994f,-0.4578f,  -0.8615f,-0.2108f,-0.4620f,  -0.7931f,-0.2984f,-0.5310f,  -0.7864f,-0.3069f,-0.5360f,  -0.7029f,-0.3872f,-0.5967f,  -0.6948f,-0.3928f,-0.6024f,  -0.5978f,-0.4643f,-0.6535f,  -0.5888f,-0.4672f,-0.6595f,  -0.4805f,-0.5286f,-0.6998f,  -0.4709f,-0.5289f,-0.7060f,  
	-0.3536f,-0.5792f,-0.7346f,  -0.3437f,-0.5772f,-0.7407f,  -0.2197f,-0.6156f,-0.7568f,  -0.2099f,-0.6116f,-0.7628f,  -0.0818f,-0.6377f,-0.7659f,  -0.0724f,-0.6320f,-0.7716f,  -0.6990f,-0.5796f,-0.4188f,  -0.6951f,-0.5835f,-0.4200f,  -0.7062f,-0.6401f,-0.3026f,  -0.6973f,-0.6483f,-0.3058f,  -0.6918f,-0.6980f,-0.1849f,  -0.6777f,-0.7102f,-0.1906f,  -0.6560f,-0.7516f,-0.0684f,  -0.6366f,-0.7673f,-0.0772f,  -0.5995f,-0.7992f,0.0442f,  
	-0.5748f,-0.8176f,0.0318f,  -0.5236f,-0.8386f,0.1504f,  -0.4939f,-0.8591f,0.1343f,  -0.4302f,-0.8679f,0.2483f,  -0.3960f,-0.8895f,0.2282f,  -0.3220f,-0.8851f,0.3361f,  -0.2838f,-0.9067f,0.3120f,  -0.2020f,-0.8883f,0.4125f,  -0.1607f,-0.9090f,0.3846f,  -0.0738f,-0.8759f,0.4769f,  -0.0303f,-0.8946f,0.4458f,  0.0586f,-0.8466f,0.5290f,  0.1031f,-0.8626f,0.4953f,  0.1910f,-0.7996f,0.5693f,  0.2353f,-0.8121f,0.5339f,  
	0.3190f,-0.7348f,0.5986f,  0.3620f,-0.7434f,0.5625f,  0.4384f,-0.6526f,0.6180f,  0.4789f,-0.6570f,0.5823f,  0.5452f,-0.5541f,0.6290f,  0.5821f,-0.5543f,0.5949f,  0.6358f,-0.4413f,0.6332f,  0.6681f,-0.4376f,0.6017f,  0.7072f,-0.3164f,0.6322f,  0.7341f,-0.3095f,0.6043f,  0.7569f,-0.1827f,0.6275f,  0.7779f,-0.1732f,0.6040f,  0.7832f,-0.0434f,0.6203f,  0.7980f,-0.0322f,0.6018f,  0.7851f,0.0977f,0.6116f,  
	0.7936f,0.1096f,0.5985f,  0.7626f,0.2366f,0.6020f,  0.7649f,0.2484f,0.5943f,  0.7162f,0.3698f,0.5918f,  0.7126f,0.3804f,0.5894f,  0.6473f,0.4934f,0.5809f,  0.6384f,0.5020f,0.5834f,  0.5580f,0.6042f,0.5688f,  0.5444f,0.6101f,0.5757f,  0.4509f,0.6992f,0.5548f,  0.4332f,0.7019f,0.5653f,  0.3291f,0.7761f,0.5379f,  0.3083f,0.7752f,0.5514f,  0.1961f,0.8330f,0.5173f,  0.1731f,0.8284f,0.5328f,  
	0.0557f,0.8689f,0.4918f,  0.0312f,0.8605f,0.5085f,  -0.0883f,0.8832f,0.4606f,  -0.1133f,0.8712f,0.4777f,  -0.2318f,0.8761f,0.4227f,  -0.2565f,0.8608f,0.4395f,  -0.3710f,0.8483f,0.3777f,  -0.3947f,0.8303f,0.3936f,  -0.5023f,0.8012f,0.3253f,  -0.5243f,0.7808f,0.3397f,  -0.6223f,0.7364f,0.2655f,  -0.6420f,0.7145f,0.2780f,  -0.7279f,0.6563f,0.1985f,  -0.7451f,0.6334f,0.2089f,  -0.8167f,0.5633f,0.1250f,  
	-0.8310f,0.5402f,0.1332f,  -0.8866f,0.4602f,0.0460f,  -0.8977f,0.4374f,0.0520f,  -0.9361f,0.3498f,-0.0372f,  -0.9440f,0.3281f,-0.0335f,  -0.9641f,0.2352f,-0.1233f,  -0.9690f,0.2150f,-0.1217f,  -0.9703f,0.1191f,-0.2107f,  -0.9723f,0.1009f,-0.2109f,  -0.9547f,0.0044f,-0.2974f,  -0.9541f,-0.0115f,-0.2993f,  -0.9181f,-0.1063f,-0.3818f,  -0.9151f,-0.1196f,-0.3850f,  -0.8615f,-0.2108f,-0.4620f,  -0.8565f,-0.2213f,-0.4662f,  
	-0.7864f,-0.3069f,-0.5360f,  -0.7799f,-0.3147f,-0.5411f,  -0.6948f,-0.3928f,-0.6024f,  -0.6871f,-0.3979f,-0.6080f,  -0.5888f,-0.4672f,-0.6595f,  -0.5802f,-0.4696f,-0.6654f,  -0.4709f,-0.5289f,-0.7060f,  -0.4619f,-0.5289f,-0.7120f,  -0.3437f,-0.5772f,-0.7407f,  -0.3345f,-0.5750f,-0.7466f,  -0.2099f,-0.6116f,-0.7628f,  -0.2009f,-0.6075f,-0.7685f,  -0.0724f,-0.6320f,-0.7716f,  -0.0638f,-0.6263f,-0.7770f,  -0.6951f,-0.5835f,-0.4200f,  
	-0.6908f,-0.5875f,-0.4214f,  -0.6973f,-0.6483f,-0.3058f,  -0.6878f,-0.6566f,-0.3096f,  -0.6777f,-0.7102f,-0.1906f,  -0.6628f,-0.7223f,-0.1973f,  -0.6366f,-0.7673f,-0.0772f,  -0.6162f,-0.7827f,-0.0873f,  -0.5748f,-0.8176f,0.0318f,  -0.5492f,-0.8355f,0.0180f,  -0.4939f,-0.8591f,0.1343f,  -0.4632f,-0.8786f,0.1164f,  -0.3960f,-0.8895f,0.2282f,  -0.3608f,-0.9096f,0.2061f,  -0.2838f,-0.9067f,0.3120f,  -0.2448f,-0.9265f,0.2858f,  
	-0.1607f,-0.9090f,0.3846f,  -0.1187f,-0.9274f,0.3549f,  -0.0303f,-0.8946f,0.4458f,  0.0135f,-0.9107f,0.4128f,  0.1031f,-0.8626f,0.4953f,  0.1476f,-0.8755f,0.4601f,  0.2353f,-0.8121f,0.5339f,  0.2794f,-0.8214f,0.4973f,  0.3620f,-0.7434f,0.5625f,  0.4043f,-0.7485f,0.5256f,  0.4789f,-0.6570f,0.5823f,  0.5183f,-0.6579f,0.5463f,  0.5821f,-0.5543f,0.5949f,  0.6176f,-0.5513f,0.5609f,  0.6681f,-0.4376f,0.6017f,  
	0.6988f,-0.4311f,0.5708f,  0.7341f,-0.3095f,0.6043f,  0.7594f,-0.3000f,0.5774f,  0.7779f,-0.1732f,0.6040f,  0.7972f,-0.1615f,0.5817f,  0.7980f,-0.0322f,0.6018f,  0.8111f,-0.0194f,0.5846f,  0.7936f,0.1096f,0.5985f,  0.8005f,0.1228f,0.5866f,  0.7649f,0.2484f,0.5943f,  0.7658f,0.2609f,0.5878f,  0.7126f,0.3804f,0.5894f,  0.7079f,0.3913f,0.5880f,  0.6384f,0.5020f,0.5834f,  0.6285f,0.5106f,0.5867f,  
	0.5444f,0.6101f,0.5757f,  0.5300f,0.6158f,0.5830f,  0.4332f,0.7019f,0.5653f,  0.4153f,0.7041f,0.5760f,  0.3083f,0.7752f,0.5514f,  0.2874f,0.7737f,0.5646f,  0.1731f,0.8284f,0.5328f,  0.1502f,0.8230f,0.5478f,  0.0312f,0.8605f,0.5085f,  0.0072f,0.8514f,0.5244f,  -0.1133f,0.8712f,0.4777f,  -0.1376f,0.8586f,0.4938f,  -0.2565f,0.8608f,0.4395f,  -0.2803f,0.8451f,0.4551f,  -0.3947f,0.8303f,0.3936f,  
	-0.4173f,0.8119f,0.4082f,  -0.5243f,0.7808f,0.3397f,  -0.5452f,0.7605f,0.3528f,  -0.6420f,0.7145f,0.2780f,  -0.6607f,0.6927f,0.2893f,  -0.7451f,0.6334f,0.2089f,  -0.7611f,0.6109f,0.2181f,  -0.8310f,0.5402f,0.1332f,  -0.8441f,0.5176f,0.1403f,  -0.8977f,0.4374f,0.0520f,  -0.9079f,0.4154f,0.0569f,  -0.9440f,0.3281f,-0.0335f,  -0.9512f,0.3072f,-0.0307f,  -0.9690f,0.2150f,-0.1217f,  -0.9732f,0.1957f,-0.1208f,  
	-0.9723f,0.1009f,-0.2109f,  -0.9738f,0.0836f,-0.2117f,  -0.9541f,-0.0115f,-0.2993f,  -0.9531f,-0.0264f,-0.3015f,  -0.9151f,-0.1196f,-0.3850f,  -0.9120f,-0.1320f,-0.3884f,  -0.8565f,-0.2213f,-0.4662f,  -0.8516f,-0.2311f,-0.4705f,  -0.7799f,-0.3147f,-0.5411f,  -0.7735f,-0.3217f,-0.5461f,  -0.6871f,-0.3979f,-0.6080f,  -0.6796f,-0.4023f,-0.6134f,  -0.5802f,-0.4696f,-0.6654f,  -0.5721f,-0.4716f,-0.6710f,  -0.4619f,-0.5289f,-0.7120f,  
	-0.4533f,-0.5286f,-0.7177f,  -0.3345f,-0.5750f,-0.7466f,  -0.3259f,-0.5727f,-0.7522f,  -0.2009f,-0.6075f,-0.7685f,  -0.1925f,-0.6034f,-0.7738f,  -0.0638f,-0.6263f,-0.7770f,  -0.0558f,-0.6208f,-0.7820f,  -0.6908f,-0.5875f,-0.4214f,  -0.6860f,-0.5918f,-0.4232f,  -0.6878f,-0.6566f,-0.3096f,  -0.6776f,-0.6650f,-0.3140f,  -0.6628f,-0.7223f,-0.1973f,  -0.6471f,-0.7344f,-0.2050f,  -0.6162f,-0.7827f,-0.0873f,  -0.5950f,-0.7977f,-0.0987f,  
	-0.5492f,-0.8355f,0.0180f,  -0.5225f,-0.8526f,0.0026f,  -0.4632f,-0.8786f,0.1164f,  -0.4316f,-0.8969f,0.0967f,  -0.3608f,-0.9096f,0.2061f,  -0.3247f,-0.9281f,0.1822f,  -0.2448f,-0.9265f,0.2858f,  -0.2051f,-0.9442f,0.2579f,  -0.1187f,-0.9274f,0.3549f,  -0.0764f,-0.9432f,0.3233f,  0.0135f,-0.9107f,0.4128f,  0.0574f,-0.9239f,0.3784f,  0.1476f,-0.8755f,0.4601f,  0.1919f,-0.8853f,0.4237f,  0.2794f,-0.8214f,0.4973f,  
	0.3228f,-0.8272f,0.4599f,  0.4043f,-0.7485f,0.5256f,  0.4456f,-0.7502f,0.4884f,  0.5183f,-0.6579f,0.5463f,  0.5564f,-0.6556f,0.5105f,  0.6176f,-0.5513f,0.5609f,  0.6515f,-0.5452f,0.5275f,  0.6988f,-0.4311f,0.5708f,  0.7278f,-0.4217f,0.5408f,  0.7594f,-0.3000f,0.5774f,  0.7828f,-0.2881f,0.5515f,  0.7972f,-0.1615f,0.5817f,  0.8147f,-0.1480f,0.5607f,  0.8111f,-0.0194f,0.5846f,  0.8225f,-0.0050f,0.5688f,  
	0.8005f,0.1228f,0.5866f,  0.8058f,0.1369f,0.5761f,  0.7658f,0.2609f,0.5878f,  0.7652f,0.2739f,0.5826f,  0.7079f,0.3913f,0.5880f,  0.7019f,0.4023f,0.5877f,  0.6285f,0.5106f,0.5867f,  0.6177f,0.5191f,0.5907f,  0.5300f,0.6158f,0.5830f,  0.5151f,0.6209f,0.5908f,  0.4153f,0.7041f,0.5760f,  0.3969f,0.7057f,0.5868f,  0.2874f,0.7737f,0.5646f,  0.2665f,0.7715f,0.5777f,  0.1502f,0.8230f,0.5478f,  
	0.1275f,0.8171f,0.5622f,  0.0072f,0.8514f,0.5244f,  -0.0163f,0.8418f,0.5396f,  -0.1376f,0.8586f,0.4938f,  -0.1612f,0.8456f,0.5089f,  -0.2803f,0.8451f,0.4551f,  -0.3033f,0.8291f,0.4696f,  -0.4173f,0.8119f,0.4082f,  -0.4390f,0.7935f,0.4215f,  -0.5452f,0.7605f,0.3528f,  -0.5650f,0.7402f,0.3646f,  -0.6607f,0.6927f,0.2893f,  -0.6782f,0.6712f,0.2992f,  -0.7611f,0.6109f,0.2181f,  -0.7760f,0.5888f,0.2262f,  
	-0.8441f,0.5176f,0.1403f,  -0.8562f,0.4956f,0.1463f,  -0.9079f,0.4154f,0.0569f,  -0.9171f,0.3941f,0.0608f,  -0.9512f,0.3072f,-0.0307f,  -0.9575f,0.2871f,-0.0286f,  -0.9732f,0.1957f,-0.1208f,  -0.9768f,0.1773f,-0.1204f,  -0.9738f,0.0836f,-0.2117f,  -0.9748f,0.0672f,-0.2128f,  -0.9531f,-0.0264f,-0.3015f,  -0.9518f,-0.0405f,-0.3039f,  -0.9120f,-0.1320f,-0.3884f,  -0.9087f,-0.1436f,-0.3919f,  -0.8516f,-0.2311f,-0.4705f,  
	-0.8467f,-0.2400f,-0.4748f,  -0.7735f,-0.3217f,-0.5461f,  -0.7674f,-0.3281f,-0.5509f,  -0.6796f,-0.4023f,-0.6134f,  -0.6726f,-0.4062f,-0.6186f,  -0.5721f,-0.4716f,-0.6710f,  -0.5644f,-0.4732f,-0.6764f,  -0.4533f,-0.5286f,-0.7177f,  -0.4454f,-0.5280f,-0.7230f,  -0.3259f,-0.5727f,-0.7522f,  -0.3180f,-0.5702f,-0.7575f,  -0.1925f,-0.6034f,-0.7738f,  -0.1848f,-0.5993f,-0.7789f,  -0.0558f,-0.6208f,-0.7820f,  -0.0485f,-0.6153f,-0.7868f,  
	-0.6860f,-0.5918f,-0.4232f,  -0.6808f,-0.5963f,-0.4253f,  -0.6776f,-0.6650f,-0.3140f,  -0.6668f,-0.6735f,-0.3191f,  -0.6471f,-0.7344f,-0.2050f,  -0.6306f,-0.7462f,-0.2136f,  -0.5950f,-0.7977f,-0.0987f,  -0.5728f,-0.8121f,-0.1113f,  -0.5225f,-0.8526f,0.0026f,  -0.4950f,-0.8688f,-0.0144f,  -0.4316f,-0.8969f,0.0967f,  -0.3991f,-0.9138f,0.0753f,  -0.3247f,-0.9281f,0.1822f,  -0.2880f,-0.9448f,0.1565f,  -0.2051f,-0.9442f,0.2579f,  
	-0.1650f,-0.9595f,0.2283f,  -0.0764f,-0.9432f,0.3233f,  -0.0339f,-0.9563f,0.2903f,  0.0574f,-0.9239f,0.3784f,  0.1012f,-0.9340f,0.3428f,  0.1919f,-0.8853f,0.4237f,  0.2357f,-0.8917f,0.3864f,  0.3228f,-0.8272f,0.4599f,  0.3653f,-0.8297f,0.4221f,  0.4456f,-0.7502f,0.4884f,  0.4857f,-0.7486f,0.4513f,  0.5564f,-0.6556f,0.5105f,  0.5931f,-0.6501f,0.4751f,  0.6515f,-0.5452f,0.5275f,  0.6838f,-0.5362f,0.4949f,  
	0.7278f,-0.4217f,0.5408f,  0.7550f,-0.4098f,0.5119f,  0.7828f,-0.2881f,0.5515f,  0.8043f,-0.2741f,0.5272f,  0.8147f,-0.1480f,0.5607f,  0.8303f,-0.1327f,0.5413f,  0.8225f,-0.0050f,0.5688f,  0.8321f,0.0105f,0.5546f,  0.8058f,0.1369f,0.5761f,  0.8095f,0.1517f,0.5671f,  0.7652f,0.2739f,0.5826f,  0.7634f,0.2872f,0.5786f,  0.7019f,0.4023f,0.5877f,  0.6949f,0.4135f,0.5883f,  0.6177f,0.5191f,0.5907f,  
	0.6062f,0.5272f,0.5955f,  0.5151f,0.6209f,0.5908f,  0.4997f,0.6257f,0.5990f,  0.3969f,0.7057f,0.5868f,  0.3784f,0.7068f,0.5977f,  0.2665f,0.7715f,0.5777f,  0.2457f,0.7688f,0.5905f,  0.1275f,0.8171f,0.5622f,  0.1052f,0.8105f,0.5762f,  -0.0163f,0.8418f,0.5396f,  -0.0393f,0.8316f,0.5540f,  -0.1612f,0.8456f,0.5089f,  -0.1840f,0.8322f,0.5231f,  -0.3033f,0.8291f,0.4696f,  -0.3253f,0.8129f,0.4831f,  
	-0.4390f,0.7935f,0.4215f,  -0.4596f,0.7750f,0.4338f,  -0.5650f,0.7402f,0.3646f,  -0.5837f,0.7200f,0.3753f,  -0.6782f,0.6712f,0.2992f,  -0.6946f,0.6500f,0.3082f,  -0.7760f,0.5888f,0.2262f,  -0.7898f,0.5673f,0.2332f,  -0.8562f,0.4956f,0.1463f,  -0.8673f,0.4743f,0.1513f,  -0.9171f,0.3941f,0.0608f,  -0.9254f,0.3736f,0.0641f,  -0.9575f,0.2871f,-0.0286f,  -0.9631f,0.2679f,-0.0271f,  -0.9768f,0.1773f,-0.1204f,  
	-0.9798f,0.1598f,-0.1205f,  -0.9748f,0.0672f,-0.2128f,  -0.9754f,0.0518f,-0.2143f,  -0.9518f,-0.0405f,-0.3039f,  -0.9503f,-0.0536f,-0.3066f,  -0.9087f,-0.1436f,-0.3919f,  -0.9055f,-0.1543f,-0.3954f,  -0.8467f,-0.2400f,-0.4748f,  -0.8420f,-0.2483f,-0.4790f,  -0.7674f,-0.3281f,-0.5509f,  -0.7615f,-0.3339f,-0.5556f,  -0.6726f,-0.4062f,-0.6186f,  -0.6658f,-0.4096f,-0.6236f,  -0.5644f,-0.4732f,-0.6764f,  -0.5572f,-0.4743f,-0.6815f,  
	-0.4454f,-0.5280f,-0.7230f,  -0.4380f,-0.5272f,-0.7281f,  -0.3180f,-0.5702f,-0.7575f,  -0.3107f,-0.5676f,-0.7624f,  -0.1848f,-0.5993f,-0.7789f,  -0.1778f,-0.5953f,-0.7836f,  -0.0485f,-0.6153f,-0.7868f,  -0.0419f,-0.6101f,-0.7912f,  -0.6808f,-0.5963f,-0.4253f,  -0.6751f,-0.6011f,-0.4277f,  -0.6668f,-0.6735f,-0.3191f,  -0.6553f,-0.6819f,-0.3249f,  -0.6306f,-0.7462f,-0.2136f,  -0.6133f,-0.7577f,-0.2232f,  -0.5728f,-0.8121f,-0.1113f,  
	-0.5498f,-0.8259f,-0.1252f,  -0.4950f,-0.8688f,-0.0144f,  -0.4666f,-0.8839f,-0.0327f,  -0.3991f,-0.9138f,0.0753f,  -0.3659f,-0.9292f,0.0525f,  -0.2880f,-0.9448f,0.1565f,  -0.2507f,-0.9594f,0.1293f,  -0.1650f,-0.9595f,0.2283f,  -0.1245f,-0.9724f,0.1972f,  -0.0339f,-0.9563f,0.2903f,  0.0086f,-0.9666f,0.2560f,  0.1012f,-0.9340f,0.3428f,  0.1445f,-0.9409f,0.3063f,  0.2357f,-0.8917f,0.3864f,  0.2786f,-0.8949f,0.3487f,  
	0.3653f,-0.8297f,0.4221f,  0.4067f,-0.8288f,0.3844f,  0.4857f,-0.7486f,0.4513f,  0.5244f,-0.7437f,0.4146f,  0.5931f,-0.6501f,0.4751f,  0.6281f,-0.6414f,0.4406f,  0.6838f,-0.5362f,0.4949f,  0.7142f,-0.5244f,0.4635f,  0.7550f,-0.4098f,0.5119f,  0.7802f,-0.3955f,0.4846f,  0.8043f,-0.2741f,0.5272f,  0.8240f,-0.2581f,0.5044f,  0.8303f,-0.1327f,0.5413f,  0.8441f,-0.1160f,0.5235f,  0.8321f,0.0105f,0.5546f,  
	0.8400f,0.0270f,0.5420f,  0.8095f,0.1517f,0.5671f,  0.8117f,0.1671f,0.5596f,  0.7634f,0.2872f,0.5786f,  0.7602f,0.3009f,0.5759f,  0.6949f,0.4135f,0.5883f,  0.6869f,0.4244f,0.5900f,  0.6062f,0.5272f,0.5955f,  0.5938f,0.5350f,0.6009f,  0.4997f,0.6257f,0.5990f,  0.4838f,0.6300f,0.6075f,  0.3784f,0.7068f,0.5977f,  0.3597f,0.7073f,0.6086f,  0.2457f,0.7688f,0.5905f,  0.2250f,0.7654f,0.6030f,  
	0.1052f,0.8105f,0.5762f,  0.0833f,0.8034f,0.5896f,  -0.0393f,0.8316f,0.5540f,  -0.0616f,0.8210f,0.5676f,  -0.1840f,0.8322f,0.5231f,  -0.2060f,0.8184f,0.5364f,  -0.3253f,0.8129f,0.4831f,  -0.3464f,0.7965f,0.4955f,  -0.4596f,0.7750f,0.4338f,  -0.4793f,0.7565f,0.4449f,  -0.5837f,0.7200f,0.3753f,  -0.6014f,0.7001f,0.3849f,  -0.6946f,0.6500f,0.3082f,  -0.7100f,0.6293f,0.3161f,  -0.7898f,0.5673f,0.2332f,  
	-0.8027f,0.5463f,0.2392f,  -0.8673f,0.4743f,0.1513f,  -0.8775f,0.4537f,0.1556f,  -0.9254f,0.3736f,0.0641f,  -0.9329f,0.3539f,0.0666f,  -0.9631f,0.2679f,-0.0271f,  -0.9680f,0.2495f,-0.0262f,  -0.9798f,0.1598f,-0.1205f,  -0.9823f,0.1432f,-0.1210f,  -0.9754f,0.0518f,-0.2143f,  -0.9757f,0.0373f,-0.2160f,  -0.9503f,-0.0536f,-0.3066f,  -0.9487f,-0.0659f,-0.3093f,  -0.9055f,-0.1543f,-0.3954f,  -0.9022f,-0.1642f,-0.3989f,  
	-0.8420f,-0.2483f,-0.4790f,  -0.8374f,-0.2558f,-0.4831f,  -0.7615f,-0.3339f,-0.5556f,  -0.7558f,-0.3390f,-0.5601f,  -0.6658f,-0.4096f,-0.6236f,  -0.6595f,-0.4125f,-0.6283f,  -0.5572f,-0.4743f,-0.6815f,  -0.5506f,-0.4752f,-0.6863f,  -0.4380f,-0.5272f,-0.7281f,  -0.4312f,-0.5262f,-0.7329f,  -0.3107f,-0.5676f,-0.7624f,  -0.3040f,-0.5650f,-0.7671f,  -0.1778f,-0.5953f,-0.7836f,  -0.1714f,-0.5913f,-0.7881f,  -0.0419f,-0.6101f,-0.7912f,  
	-0.0359f,-0.6050f,-0.7954f,  -0.6751f,-0.6011f,-0.4277f,  -0.6689f,-0.6060f,-0.4305f,  -0.6553f,-0.6819f,-0.3249f,  -0.6432f,-0.6903f,-0.3313f,  -0.6133f,-0.7577f,-0.2232f,  -0.5952f,-0.7688f,-0.2338f,  -0.5498f,-0.8259f,-0.1252f,  -0.5261f,-0.8388f,-0.1403f,  -0.4666f,-0.8839f,-0.0327f,  -0.4375f,-0.8977f,-0.0525f,  -0.3659f,-0.9292f,0.0525f,  -0.3321f,-0.9428f,0.0281f,  -0.2507f,-0.9594f,0.1293f,  -0.2131f,-0.9718f,0.1007f,  
	-0.1245f,-0.9724f,0.1972f,  -0.0841f,-0.9827f,0.1650f,  0.0086f,-0.9666f,0.2560f,  0.0508f,-0.9740f,0.2209f,  0.1445f,-0.9409f,0.3063f,  0.1872f,-0.9447f,0.2693f,  0.2786f,-0.8949f,0.3487f,  0.3206f,-0.8948f,0.3109f,  0.4067f,-0.8288f,0.3844f,  0.4467f,-0.8247f,0.3470f,  0.5244f,-0.7437f,0.4146f,  0.5615f,-0.7358f,0.3787f,  0.6281f,-0.6414f,0.4406f,  0.6612f,-0.6301f,0.4072f,  0.7142f,-0.5244f,0.4635f,  
	0.7427f,-0.5102f,0.4337f,  0.7802f,-0.3955f,0.4846f,  0.8035f,-0.3791f,0.4589f,  0.8240f,-0.2581f,0.5044f,  0.8417f,-0.2406f,0.4834f,  0.8441f,-0.1160f,0.5235f,  0.8561f,-0.0982f,0.5075f,  0.8400f,0.0270f,0.5420f,  0.8462f,0.0442f,0.5310f,  0.8117f,0.1671f,0.5596f,  0.8125f,0.1831f,0.5535f,  0.7602f,0.3009f,0.5759f,  0.7558f,0.3144f,0.5744f,  0.6869f,0.4244f,0.5900f,  0.6779f,0.4352f,0.5925f,  
	0.5938f,0.5350f,0.6009f,  0.5809f,0.5424f,0.6069f,  0.4838f,0.6300f,0.6075f,  0.4675f,0.6338f,0.6163f,  0.3597f,0.7073f,0.6086f,  0.3409f,0.7072f,0.6194f,  0.2250f,0.7654f,0.6030f,  0.2045f,0.7615f,0.6151f,  0.0833f,0.8034f,0.5896f,  0.0619f,0.7958f,0.6024f,  -0.0616f,0.8210f,0.5676f,  -0.0833f,0.8100f,0.5805f,  -0.2060f,0.8184f,0.5364f,  -0.2273f,0.8045f,0.5488f,  -0.3464f,0.7965f,0.4955f,  
	-0.3666f,0.7801f,0.5069f,  -0.4793f,0.7565f,0.4449f,  -0.4979f,0.7383f,0.4551f,  -0.6014f,0.7001f,0.3849f,  -0.6180f,0.6806f,0.3936f,  -0.7100f,0.6293f,0.3161f,  -0.7244f,0.6091f,0.3231f,  -0.8027f,0.5463f,0.2392f,  -0.8146f,0.5260f,0.2446f,  -0.8775f,0.4537f,0.1556f,  -0.8868f,0.4338f,0.1592f,  -0.9329f,0.3539f,0.0666f,  -0.9397f,0.3350f,0.0686f,  -0.9680f,0.2495f,-0.0262f,  -0.9723f,0.2321f,-0.0256f,  
	-0.9823f,0.1432f,-0.1210f,  -0.9843f,0.1275f,-0.1217f,  -0.9757f,0.0373f,-0.2160f,  -0.9757f,0.0237f,-0.2178f,  -0.9487f,-0.0659f,-0.3093f,  -0.9469f,-0.0773f,-0.3120f,  -0.9022f,-0.1642f,-0.3989f,  -0.8989f,-0.1733f,-0.4023f,  -0.8374f,-0.2558f,-0.4831f,  -0.8329f,-0.2627f,-0.4871f,  -0.7558f,-0.3390f,-0.5601f,  -0.7506f,-0.3437f,-0.5644f,  -0.6595f,-0.4125f,-0.6283f,  -0.6537f,-0.4151f,-0.6328f,  -0.5506f,-0.4752f,-0.6863f,  
	-0.5444f,-0.4758f,-0.6908f,  -0.4312f,-0.5262f,-0.7329f,  -0.4250f,-0.5251f,-0.7374f,  -0.3040f,-0.5650f,-0.7671f,  -0.2979f,-0.5623f,-0.7714f,  -0.1714f,-0.5913f,-0.7881f,  -0.1655f,-0.5874f,-0.7922f,  -0.0359f,-0.6050f,-0.7954f,  -0.0306f,-0.6001f,-0.7994f,  -0.6689f,-0.6060f,-0.4305f,  -0.6621f,-0.6111f,-0.4337f,  -0.6432f,-0.6903f,-0.3313f,  -0.6304f,-0.6985f,-0.3385f,  -0.5952f,-0.7688f,-0.2338f,  -0.5765f,-0.7794f,-0.2454f,  
	-0.5261f,-0.8388f,-0.1403f,  -0.5016f,-0.8508f,-0.1566f,  -0.4375f,-0.8977f,-0.0525f,  -0.4078f,-0.9101f,-0.0735f,  -0.3321f,-0.9428f,0.0281f,  -0.2979f,-0.9546f,0.0025f,  -0.2131f,-0.9718f,0.1007f,  -0.1753f,-0.9820f,0.0710f,  -0.0841f,-0.9827f,0.1650f,  -0.0438f,-0.9903f,0.1318f,  0.0508f,-0.9740f,0.2209f,  0.0925f,-0.9783f,0.1853f,  0.1872f,-0.9447f,0.2693f,  0.2290f,-0.9454f,0.2321f,  0.3206f,-0.8948f,0.3109f,  
	0.3613f,-0.8915f,0.2734f,  0.4467f,-0.8247f,0.3470f,  0.4853f,-0.8174f,0.3103f,  0.5615f,-0.7358f,0.3787f,  0.5969f,-0.7249f,0.3439f,  0.6612f,-0.6301f,0.4072f,  0.6925f,-0.6161f,0.3753f,  0.7427f,-0.5102f,0.4337f,  0.7693f,-0.4937f,0.4055f,  0.8035f,-0.3791f,0.4589f,  0.8249f,-0.3610f,0.4350f,  0.8417f,-0.2406f,0.4834f,  0.8575f,-0.2217f,0.4643f,  0.8561f,-0.0982f,0.5075f,  0.8662f,-0.0795f,0.4933f,  
	0.8462f,0.0442f,0.5310f,  0.8509f,0.0621f,0.5217f,  0.8125f,0.1831f,0.5535f,  0.8119f,0.1989f,0.5489f,  0.7558f,0.3144f,0.5744f,  0.7503f,0.3279f,0.5740f,  0.6779f,0.4352f,0.5925f,  0.6681f,0.4456f,0.5959f,  0.5809f,0.5424f,0.6069f,  0.5674f,0.5494f,0.6134f,  0.4675f,0.6338f,0.6163f,  0.4510f,0.6369f,0.6253f,  0.3409f,0.7072f,0.6194f,  0.3222f,0.7065f,0.6301f,  0.2045f,0.7615f,0.6151f,  
	0.1843f,0.7569f,0.6270f,  0.0619f,0.7958f,0.6024f,  0.0409f,0.7877f,0.6147f,  -0.0833f,0.8100f,0.5805f,  -0.1043f,0.7987f,0.5926f,  -0.2273f,0.8045f,0.5488f,  -0.2477f,0.7904f,0.5603f,  -0.3666f,0.7801f,0.5069f,  -0.3858f,0.7638f,0.5175f,  -0.4979f,0.7383f,0.4551f,  -0.5155f,0.7202f,0.4644f,  -0.6180f,0.6806f,0.3936f,  -0.6336f,0.6614f,0.4013f,  -0.7244f,0.6091f,0.3231f,  -0.7377f,0.5894f,0.3293f,  
	-0.8146f,0.5260f,0.2446f,  -0.8255f,0.5064f,0.2491f,  -0.8868f,0.4338f,0.1592f,  -0.8953f,0.4148f,0.1622f,  -0.9397f,0.3350f,0.0686f,  -0.9458f,0.3171f,0.0702f,  -0.9723f,0.2321f,-0.0256f,  -0.9761f,0.2156f,-0.0254f,  -0.9843f,0.1275f,-0.1217f,  -0.9860f,0.1128f,-0.1227f,  -0.9757f,0.0237f,-0.2178f,  -0.9755f,0.0109f,-0.2197f,  -0.9469f,-0.0773f,-0.3120f,  -0.9451f,-0.0879f,-0.3147f,  -0.8989f,-0.1733f,-0.4023f,  
	-0.8957f,-0.1818f,-0.4057f,  -0.8329f,-0.2627f,-0.4871f,  -0.8287f,-0.2689f,-0.4908f,  -0.7506f,-0.3437f,-0.5644f,  -0.7456f,-0.3478f,-0.5684f,  -0.6537f,-0.4151f,-0.6328f,  -0.6482f,-0.4172f,-0.6370f,  -0.5444f,-0.4758f,-0.6908f,  -0.5387f,-0.4761f,-0.6951f,  -0.4250f,-0.5251f,-0.7374f,  -0.4193f,-0.5238f,-0.7415f,  -0.2979f,-0.5623f,-0.7714f,  -0.2923f,-0.5597f,-0.7754f,  -0.1655f,-0.5874f,-0.7922f,  -0.1603f,-0.5836f,-0.7961f,  
	-0.0306f,-0.6001f,-0.7994f,  -0.0257f,-0.5953f,-0.8031f,  -0.6621f,-0.6111f,-0.4337f,  -0.6548f,-0.6164f,-0.4373f,  -0.6304f,-0.6985f,-0.3385f,  -0.6170f,-0.7066f,-0.3464f,  -0.5765f,-0.7794f,-0.2454f,  -0.5570f,-0.7894f,-0.2579f,  -0.5016f,-0.8508f,-0.1566f,  -0.4765f,-0.8618f,-0.1739f,  -0.4078f,-0.9101f,-0.0735f,  -0.3777f,-0.9210f,-0.0957f,  -0.2979f,-0.9546f,0.0025f,  -0.2635f,-0.9644f,-0.0241f,  -0.1753f,-0.9820f,0.0710f,  
	-0.1376f,-0.9897f,0.0405f,  -0.0438f,-0.9903f,0.1318f,  -0.0038f,-0.9952f,0.0982f,  0.0925f,-0.9783f,0.1853f,  0.1334f,-0.9797f,0.1495f,  0.2290f,-0.9454f,0.2321f,  0.2697f,-0.9429f,0.1953f,  0.3613f,-0.8915f,0.2734f,  0.4007f,-0.8851f,0.2366f,  0.4853f,-0.8174f,0.3103f,  0.5222f,-0.8074f,0.2747f,  0.5969f,-0.7249f,0.3439f,  0.6304f,-0.7115f,0.3105f,  0.6925f,-0.6161f,0.3753f,  0.7219f,-0.5997f,0.3451f,  
	0.7693f,-0.4937f,0.4055f,  0.7939f,-0.4753f,0.3792f,  0.8249f,-0.3610f,0.4350f,  0.8442f,-0.3415f,0.4131f,  0.8575f,-0.2217f,0.4643f,  0.8714f,-0.2018f,0.4471f,  0.8662f,-0.0795f,0.4933f,  0.8747f,-0.0599f,0.4809f,  0.8509f,0.0621f,0.5217f,  0.8540f,0.0801f,0.5140f,  0.8119f,0.1989f,0.5489f,  0.8100f,0.2149f,0.5456f,  0.7503f,0.3279f,0.5740f,  0.7438f,0.3411f,0.5747f,  0.6681f,0.4456f,0.5959f,  
	0.6575f,0.4556f,0.6001f,  0.5674f,0.5494f,0.6134f,  0.5534f,0.5557f,0.6204f,  0.4510f,0.6369f,0.6253f,  0.4344f,0.6394f,0.6344f,  0.3222f,0.7065f,0.6301f,  0.3036f,0.7052f,0.6408f,  0.1843f,0.7569f,0.6270f,  0.1644f,0.7519f,0.6384f,  0.0409f,0.7877f,0.6147f,  0.0205f,0.7792f,0.6264f,  -0.1043f,0.7987f,0.5926f,  -0.1246f,0.7871f,0.6041f,  -0.2477f,0.7904f,0.5603f,  -0.2672f,0.7762f,0.5710f,  
	-0.3858f,0.7638f,0.5175f,  -0.4041f,0.7475f,0.5272f,  -0.5155f,0.7202f,0.4644f,  -0.5321f,0.7024f,0.4728f,  -0.6336f,0.6614f,0.4013f,  -0.6483f,0.6427f,0.4084f,  -0.7377f,0.5894f,0.3293f,  -0.7501f,0.5703f,0.3347f,  -0.8255f,0.5064f,0.2491f,  -0.8356f,0.4875f,0.2531f,  -0.8953f,0.4148f,0.1622f,  -0.9031f,0.3966f,0.1648f,  -0.9458f,0.3171f,0.0702f,  -0.9513f,0.3000f,0.0713f,  -0.9761f,0.2156f,-0.0254f,  
	-0.9795f,0.2000f,-0.0254f,  -0.9860f,0.1128f,-0.1227f,  -0.9874f,0.0989f,-0.1237f,  -0.9755f,0.0109f,-0.2197f,  -0.9751f,-0.0010f,-0.2217f,  -0.9451f,-0.0879f,-0.3147f,  -0.9432f,-0.0978f,-0.3174f,  -0.8957f,-0.1818f,-0.4057f,  -0.8927f,-0.1896f,-0.4089f,  -0.8287f,-0.2689f,-0.4908f,  -0.8247f,-0.2746f,-0.4944f,  -0.7456f,-0.3478f,-0.5684f,  -0.7410f,-0.3515f,-0.5722f,  -0.6482f,-0.4172f,-0.6370f,  -0.6432f,-0.4190f,-0.6409f,  
	-0.5387f,-0.4761f,-0.6951f,  -0.5335f,-0.4763f,-0.6990f,  -0.4193f,-0.5238f,-0.7415f,  -0.4141f,-0.5224f,-0.7454f,  -0.2923f,-0.5597f,-0.7754f,  -0.2873f,-0.5571f,-0.7792f,  -0.1603f,-0.5836f,-0.7961f,  -0.1556f,-0.5799f,-0.7997f,  -0.0257f,-0.5953f,-0.8031f,  -0.0214f,-0.5908f,-0.8065f,  -0.6548f,-0.6164f,-0.4373f,  -0.6470f,-0.6218f,-0.4414f,  -0.6170f,-0.7066f,-0.3464f,  -0.6029f,-0.7145f,-0.3549f,  -0.5570f,-0.7894f,-0.2579f,  
	-0.5370f,-0.7988f,-0.2714f,  -0.4765f,-0.8618f,-0.1739f,  -0.4509f,-0.8716f,-0.1923f,  -0.3777f,-0.9210f,-0.0957f,  -0.3472f,-0.9303f,-0.1188f,  -0.2635f,-0.9644f,-0.0241f,  -0.2290f,-0.9721f,-0.0515f,  -0.1376f,-0.9897f,0.0405f,  -0.1001f,-0.9949f,0.0094f,  -0.0038f,-0.9952f,0.0982f,  0.0355f,-0.9973f,0.0642f,  0.1334f,-0.9797f,0.1495f,  0.1734f,-0.9783f,0.1138f,  0.2697f,-0.9429f,0.1953f,  0.3093f,-0.9376f,0.1589f,  
	0.4007f,-0.8851f,0.2366f,  0.4386f,-0.8760f,0.2008f,  0.5222f,-0.8074f,0.2747f,  0.5575f,-0.7946f,0.2404f,  0.6304f,-0.7115f,0.3105f,  0.6622f,-0.6956f,0.2788f,  0.7219f,-0.5997f,0.3451f,  0.7494f,-0.5815f,0.3168f,  0.7939f,-0.4753f,0.3792f,  0.8165f,-0.4554f,0.3548f,  0.8442f,-0.3415f,0.4131f,  0.8617f,-0.3207f,0.3932f,  0.8714f,-0.2018f,0.4471f,  0.8836f,-0.1809f,0.4318f,  0.8747f,-0.0599f,0.4809f,  
	0.8816f,-0.0401f,0.4703f,  0.8540f,0.0801f,0.5140f,  0.8558f,0.0983f,0.5079f,  0.8100f,0.2149f,0.5456f,  0.8069f,0.2308f,0.5437f,  0.7438f,0.3411f,0.5747f,  0.7365f,0.3540f,0.5765f,  0.6575f,0.4556f,0.6001f,  0.6464f,0.4650f,0.6050f,  0.5534f,0.5557f,0.6204f,  0.5391f,0.5615f,0.6278f,  0.4344f,0.6394f,0.6344f,  0.4177f,0.6414f,0.6436f,  0.3036f,0.7052f,0.6408f,  0.2851f,0.7033f,0.6512f,  
	0.1644f,0.7519f,0.6384f,  0.1450f,0.7464f,0.6495f,  0.0205f,0.7792f,0.6264f,  0.0007f,0.7704f,0.6375f,  -0.1246f,0.7871f,0.6041f,  -0.1440f,0.7754f,0.6148f,  -0.2672f,0.7762f,0.5710f,  -0.2859f,0.7620f,0.5810f,  -0.4041f,0.7475f,0.5272f,  -0.4215f,0.7314f,0.5361f,  -0.5321f,0.7024f,0.4728f,  -0.5477f,0.6849f,0.4805f,  -0.6483f,0.6427f,0.4084f,  -0.6619f,0.6244f,0.4146f,  -0.7501f,0.5703f,0.3347f,  
	-0.7617f,0.5519f,0.3396f,  -0.8356f,0.4875f,0.2531f,  -0.8449f,0.4694f,0.2566f,  -0.9031f,0.3966f,0.1648f,  -0.9101f,0.3792f,0.1669f,  -0.9513f,0.3000f,0.0713f,  -0.9562f,0.2838f,0.0722f,  -0.9795f,0.2000f,-0.0254f,  -0.9824f,0.1852f,-0.0256f,  -0.9874f,0.0989f,-0.1237f,  -0.9884f,0.0859f,-0.1249f,  -0.9751f,-0.0010f,-0.2217f,  -0.9746f,-0.0122f,-0.2236f,  -0.9432f,-0.0978f,-0.3174f,  -0.9414f,-0.1070f,-0.3199f,  
	-0.8927f,-0.1896f,-0.4089f,  -0.8898f,-0.1967f,-0.4118f,  -0.8247f,-0.2746f,-0.4944f,  -0.8210f,-0.2798f,-0.4977f,  -0.7410f,-0.3515f,-0.5722f,  -0.7366f,-0.3548f,-0.5757f,  -0.6432f,-0.4190f,-0.6409f,  -0.6385f,-0.4206f,-0.6445f,  -0.5335f,-0.4763f,-0.6990f,  -0.5287f,-0.4762f,-0.7026f,  -0.4141f,-0.5224f,-0.7454f,  -0.4094f,-0.5210f,-0.7490f,  -0.2873f,-0.5571f,-0.7792f,  -0.2828f,-0.5545f,-0.7827f,  -0.1556f,-0.5799f,-0.7997f,  
	-0.1514f,-0.5764f,-0.8030f,  -0.0214f,-0.5908f,-0.8065f,  -0.0177f,-0.5866f,-0.8097f,  -0.6470f,-0.6218f,-0.4414f,  -0.6385f,-0.6274f,-0.4458f,  -0.6029f,-0.7145f,-0.3549f,  -0.5883f,-0.7220f,-0.3642f,  -0.5370f,-0.7988f,-0.2714f,  -0.5164f,-0.8073f,-0.2856f,  -0.4509f,-0.8716f,-0.1923f,  -0.4248f,-0.8802f,-0.2115f,  -0.3472f,-0.9303f,-0.1188f,  -0.3165f,-0.9378f,-0.1427f,  -0.2290f,-0.9721f,-0.0515f,  -0.1945f,-0.9777f,-0.0796f,  
	-0.1001f,-0.9949f,0.0094f,  -0.0629f,-0.9978f,-0.0221f,  0.0355f,-0.9973f,0.0642f,  0.0742f,-0.9968f,0.0304f,  0.1734f,-0.9783f,0.1138f,  0.2125f,-0.9740f,0.0786f,  0.3093f,-0.9376f,0.1589f,  0.3475f,-0.9295f,0.1235f,  0.4386f,-0.8760f,0.2008f,  0.4749f,-0.8642f,0.1662f,  0.5575f,-0.7946f,0.2404f,  0.5910f,-0.7795f,0.2078f,  0.6622f,-0.6956f,0.2788f,  0.6920f,-0.6776f,0.2490f,  0.7494f,-0.5815f,0.3168f,  
	0.7749f,-0.5615f,0.2905f,  0.8165f,-0.4554f,0.3548f,  0.8372f,-0.4341f,0.3326f,  0.8617f,-0.3207f,0.3932f,  0.8774f,-0.2988f,0.3754f,  0.8836f,-0.1809f,0.4318f,  0.8941f,-0.1596f,0.4186f,  0.8816f,-0.0401f,0.4703f,  0.8869f,-0.0199f,0.4615f,  0.8558f,0.0983f,0.5079f,  0.8562f,0.1165f,0.5034f,  0.8069f,0.2308f,0.5437f,  0.8028f,0.2463f,0.5430f,  0.7365f,0.3540f,0.5765f,  0.7283f,0.3663f,0.5791f,  
	0.6464f,0.4650f,0.6050f,  0.6347f,0.4739f,0.6104f,  0.5391f,0.5615f,0.6278f,  0.5246f,0.5666f,0.6354f,  0.4177f,0.6414f,0.6436f,  0.4010f,0.6427f,0.6528f,  0.2851f,0.7033f,0.6512f,  0.2670f,0.7009f,0.6614f,  0.1450f,0.7464f,0.6495f,  0.1260f,0.7406f,0.6601f,  0.0007f,0.7704f,0.6375f,  -0.0184f,0.7613f,0.6481f,  -0.1440f,0.7754f,0.6148f,  -0.1627f,0.7635f,0.6249f,  -0.2859f,0.7620f,0.5810f,  
	-0.3036f,0.7479f,0.5903f,  -0.4215f,0.7314f,0.5361f,  -0.4378f,0.7155f,0.5444f,  -0.5477f,0.6849f,0.4805f,  -0.5624f,0.6679f,0.4874f,  -0.6619f,0.6244f,0.4146f,  -0.6747f,0.6068f,0.4203f,  -0.7617f,0.5519f,0.3396f,  -0.7723f,0.5341f,0.3439f,  -0.8449f,0.4694f,0.2566f,  -0.8534f,0.4520f,0.2596f,  -0.9101f,0.3792f,0.1669f,  -0.9165f,0.3627f,0.1687f,  -0.9562f,0.2838f,0.0722f,  -0.9605f,0.2684f,0.0729f,  
	-0.9824f,0.1852f,-0.0256f,  -0.9849f,0.1713f,-0.0259f,  -0.9884f,0.0859f,-0.1249f,  -0.9893f,0.0737f,-0.1261f,  -0.9746f,-0.0122f,-0.2236f,  -0.9740f,-0.0226f,-0.2255f,  -0.9414f,-0.1070f,-0.3199f,  -0.9396f,-0.1155f,-0.3223f,  -0.8898f,-0.1967f,-0.4118f,  -0.8870f,-0.2033f,-0.4146f,  -0.8210f,-0.2798f,-0.4977f,  -0.8175f,-0.2845f,-0.5007f,  -0.7366f,-0.3548f,-0.5757f,  -0.7327f,-0.3577f,-0.5789f,  -0.6385f,-0.4206f,-0.6445f,  
	-0.6343f,-0.4219f,-0.6478f,  -0.5287f,-0.4762f,-0.7026f,  -0.5244f,-0.4761f,-0.7059f,  -0.4094f,-0.5210f,-0.7490f,  -0.4052f,-0.5196f,-0.7522f,  -0.2828f,-0.5545f,-0.7827f,  -0.2788f,-0.5520f,-0.7859f,  -0.1514f,-0.5764f,-0.8030f,  -0.1477f,-0.5730f,-0.8061f,  -0.0177f,-0.5866f,-0.8097f,  -0.0144f,-0.5826f,-0.8127f,  -0.6385f,-0.6274f,-0.4458f,  -0.6294f,-0.6330f,-0.4507f,  -0.5883f,-0.7220f,-0.3642f,  -0.5730f,-0.7292f,-0.3740f,  
	-0.5164f,-0.8073f,-0.2856f,  -0.4952f,-0.8151f,-0.3006f,  -0.4248f,-0.8802f,-0.2115f,  -0.3985f,-0.8875f,-0.2314f,  -0.3165f,-0.9378f,-0.1427f,  -0.2857f,-0.9436f,-0.1672f,  -0.1945f,-0.9777f,-0.0796f,  -0.1603f,-0.9811f,-0.1080f,  -0.0629f,-0.9978f,-0.0221f,  -0.0264f,-0.9982f,-0.0535f,  0.0742f,-0.9968f,0.0304f,  0.1120f,-0.9937f,-0.0031f,  0.2125f,-0.9740f,0.0786f,  0.2503f,-0.9672f,0.0441f,  0.3475f,-0.9295f,0.1235f,  
	0.3842f,-0.9189f,0.0892f,  0.4749f,-0.8642f,0.1662f,  0.5097f,-0.8500f,0.1332f,  0.5910f,-0.7795f,0.2078f,  0.6227f,-0.7622f,0.1769f,  0.6920f,-0.6776f,0.2490f,  0.7199f,-0.6580f,0.2211f,  0.7749f,-0.5615f,0.2905f,  0.7984f,-0.5401f,0.2662f,  0.8372f,-0.4341f,0.3326f,  0.8561f,-0.4116f,0.3125f,  0.8774f,-0.2988f,0.3754f,  0.8912f,-0.2764f,0.3596f,  0.8941f,-0.1596f,0.4186f,  0.9029f,-0.1379f,0.4071f,  
	0.8869f,-0.0199f,0.4615f,  0.8908f,0.0003f,0.4544f,  0.8562f,0.1165f,0.5034f,  0.8554f,0.1345f,0.5002f,  0.8028f,0.2463f,0.5430f,  0.7977f,0.2614f,0.5434f,  0.7283f,0.3663f,0.5791f,  0.7194f,0.3782f,0.5826f,  0.6347f,0.4739f,0.6104f,  0.6226f,0.4821f,0.6164f,  0.5246f,0.5666f,0.6354f,  0.5100f,0.5710f,0.6433f,  0.4010f,0.6427f,0.6528f,  0.3844f,0.6434f,0.6620f,  0.2670f,0.7009f,0.6614f,  
	0.2491f,0.6980f,0.6714f,  0.1260f,0.7406f,0.6601f,  0.1076f,0.7342f,0.6703f,  -0.0184f,0.7613f,0.6481f,  -0.0368f,0.7520f,0.6582f,  -0.1627f,0.7635f,0.6249f,  -0.1806f,0.7516f,0.6344f,  -0.3036f,0.7479f,0.5903f,  -0.3204f,0.7339f,0.5989f,  -0.4378f,0.7155f,0.5444f,  -0.4533f,0.7000f,0.5519f,  -0.5624f,0.6679f,0.4874f,  -0.5761f,0.6513f,0.4938f,  -0.6747f,0.6068f,0.4203f,  -0.6865f,0.5897f,0.4254f,  
	-0.7723f,0.5341f,0.3439f,  -0.7821f,0.5170f,0.3478f,  -0.8534f,0.4520f,0.2596f,  -0.8612f,0.4354f,0.2622f,  -0.9165f,0.3627f,0.1687f,  -0.9223f,0.3470f,0.1702f,  -0.9605f,0.2684f,0.0729f,  -0.9644f,0.2539f,0.0734f,  -0.9849f,0.1713f,-0.0259f,  -0.9870f,0.1583f,-0.0263f,  -0.9893f,0.0737f,-0.1261f,  -0.9899f,0.0622f,-0.1272f,  -0.9740f,-0.0226f,-0.2255f,  -0.9733f,-0.0322f,-0.2272f,  -0.9396f,-0.1155f,-0.3223f,  
	-0.9378f,-0.1233f,-0.3245f,  -0.8870f,-0.2033f,-0.4146f,  -0.8844f,-0.2093f,-0.4172f,  -0.8175f,-0.2845f,-0.5007f,  -0.8143f,-0.2887f,-0.5035f,  -0.7327f,-0.3577f,-0.5789f,  -0.7291f,-0.3603f,-0.5819f,  -0.6343f,-0.4219f,-0.6478f,  -0.6305f,-0.4230f,-0.6508f,  -0.5244f,-0.4761f,-0.7059f,  -0.5206f,-0.4758f,-0.7089f,  -0.4052f,-0.5196f,-0.7522f,  -0.4014f,-0.5182f,-0.7552f,  -0.2788f,-0.5520f,-0.7859f,  -0.2753f,-0.5496f,-0.7888f,  
	-0.1477f,-0.5730f,-0.8061f,  -0.1445f,-0.5698f,-0.8089f,  -0.0144f,-0.5826f,-0.8127f,  -0.0115f,-0.5788f,-0.8154f,  -0.6294f,-0.6330f,-0.4507f,  -0.6197f,-0.6387f,-0.4561f,  -0.5730f,-0.7292f,-0.3740f,  -0.5571f,-0.7361f,-0.3845f,  -0.4952f,-0.8151f,-0.3006f,  -0.4736f,-0.8220f,-0.3162f,  -0.3985f,-0.8875f,-0.2314f,  -0.3719f,-0.8935f,-0.2519f,  -0.2857f,-0.9436f,-0.1672f,  -0.2549f,-0.9477f,-0.1921f,  -0.1603f,-0.9811f,-0.1080f,  
	-0.1264f,-0.9825f,-0.1365f,  -0.0264f,-0.9982f,-0.0535f,  0.0096f,-0.9964f,-0.0847f,  0.1120f,-0.9937f,-0.0031f,  0.1488f,-0.9882f,-0.0359f,  0.2503f,-0.9672f,0.0441f,  0.2869f,-0.9579f,0.0107f,  0.3842f,-0.9189f,0.0892f,  0.4196f,-0.9060f,0.0564f,  0.5097f,-0.8500f,0.1332f,  0.5427f,-0.8338f,0.1019f,  0.6227f,-0.7622f,0.1769f,  0.6525f,-0.7432f,0.1480f,  0.7199f,-0.6580f,0.2211f,  0.7459f,-0.6368f,0.1953f,  
	0.7984f,-0.5401f,0.2662f,  0.8202f,-0.5173f,0.2442f,  0.8561f,-0.4116f,0.3125f,  0.8731f,-0.3885f,0.2945f,  0.8912f,-0.2764f,0.3596f,  0.9034f,-0.2535f,0.3458f,  0.9029f,-0.1379f,0.4071f,  0.9102f,-0.1160f,0.3976f,  0.8908f,0.0003f,0.4544f,  0.8934f,0.0203f,0.4488f,  0.8554f,0.1345f,0.5002f,  0.8535f,0.1520f,0.4984f,  0.7977f,0.2614f,0.5434f,  0.7918f,0.2760f,0.5448f,  0.7194f,0.3782f,0.5826f,  
	0.7100f,0.3894f,0.5868f,  0.6226f,0.4821f,0.6164f,  0.6102f,0.4896f,0.6228f,  0.5100f,0.5710f,0.6433f,  0.4953f,0.5749f,0.6513f,  0.3844f,0.6434f,0.6620f,  0.3680f,0.6435f,0.6711f,  0.2491f,0.6980f,0.6714f,  0.2317f,0.6946f,0.6811f,  0.1076f,0.7342f,0.6703f,  0.0898f,0.7276f,0.6801f,  -0.0368f,0.7520f,0.6582f,  -0.0545f,0.7425f,0.6676f,  -0.1806f,0.7516f,0.6344f,  -0.1976f,0.7397f,0.6433f,  
	-0.3204f,0.7339f,0.5989f,  -0.3364f,0.7200f,0.6070f,  -0.4533f,0.7000f,0.5519f,  -0.4678f,0.6847f,0.5589f,  -0.5761f,0.6513f,0.4938f,  -0.5889f,0.6352f,0.4997f,  -0.6865f,0.5897f,0.4254f,  -0.6974f,0.5732f,0.4301f,  -0.7821f,0.5170f,0.3478f,  -0.7911f,0.5007f,0.3513f,  -0.8612f,0.4354f,0.2622f,  -0.8683f,0.4196f,0.2646f,  -0.9223f,0.3470f,0.1702f,  -0.9275f,0.3321f,0.1716f,  -0.9644f,0.2539f,0.0734f,  
	-0.9679f,0.2401f,0.0738f,  -0.9870f,0.1583f,-0.0263f,  -0.9889f,0.1460f,-0.0267f,  -0.9899f,0.0622f,-0.1272f,  -0.9904f,0.0516f,-0.1282f,  -0.9733f,-0.0322f,-0.2272f,  -0.9726f,-0.0412f,-0.2288f,  -0.9378f,-0.1233f,-0.3245f,  -0.9362f,-0.1305f,-0.3265f,  -0.8844f,-0.2093f,-0.4172f,  -0.8820f,-0.2148f,-0.4195f,  -0.8143f,-0.2887f,-0.5035f,  -0.8114f,-0.2926f,-0.5060f,  -0.7291f,-0.3603f,-0.5819f,  -0.7258f,-0.3627f,-0.5845f,  
	-0.6305f,-0.4230f,-0.6508f,  -0.6271f,-0.4239f,-0.6535f,  -0.5206f,-0.4758f,-0.7089f,  -0.5171f,-0.4755f,-0.7117f,  -0.4014f,-0.5182f,-0.7552f,  -0.3981f,-0.5168f,-0.7579f,  -0.2753f,-0.5496f,-0.7888f,  -0.2722f,-0.5474f,-0.7914f,  -0.1445f,-0.5698f,-0.8089f,  -0.1417f,-0.5669f,-0.8115f,  -0.0115f,-0.5788f,-0.8154f,  -0.0091f,-0.5753f,-0.8179f,  -0.6197f,-0.6387f,-0.4561f,  -0.6094f,-0.6444f,-0.4619f,  -0.5571f,-0.7361f,-0.3845f,  
	-0.5406f,-0.7424f,-0.3956f,  -0.4736f,-0.8220f,-0.3162f,  -0.4516f,-0.8280f,-0.3324f,  -0.3719f,-0.8935f,-0.2519f,  -0.3451f,-0.8980f,-0.2729f,  -0.2549f,-0.9477f,-0.1921f,  -0.2243f,-0.9500f,-0.2171f,  -0.1264f,-0.9825f,-0.1365f,  -0.0929f,-0.9819f,-0.1648f,  0.0096f,-0.9964f,-0.0847f,  0.0448f,-0.9923f,-0.1154f,  0.1488f,-0.9882f,-0.0359f,  0.1846f,-0.9805f,-0.0679f,  0.2869f,-0.9579f,0.0107f,  0.3222f,-0.9464f,-0.0214f,  
	0.4196f,-0.9060f,0.0564f,  0.4533f,-0.8910f,0.0251f,  0.5427f,-0.8338f,0.1019f,  0.5740f,-0.8157f,0.0724f,  0.6525f,-0.7432f,0.1480f,  0.6806f,-0.7226f,0.1211f,  0.7459f,-0.6368f,0.1953f,  0.7702f,-0.6142f,0.1717f,  0.8202f,-0.5173f,0.2442f,  0.8401f,-0.4939f,0.2243f,  0.8731f,-0.3885f,0.2945f,  0.8885f,-0.3647f,0.2785f,  0.9034f,-0.2535f,0.3458f,  0.9140f,-0.2303f,0.3340f,  0.9102f,-0.1160f,0.3976f,  
	0.9161f,-0.0942f,0.3897f,  0.8934f,0.0203f,0.4488f,  0.8948f,0.0400f,0.4448f,  0.8535f,0.1520f,0.4984f,  0.8507f,0.1692f,0.4977f,  0.7918f,0.2760f,0.5448f,  0.7852f,0.2900f,0.5471f,  0.7100f,0.3894f,0.5868f,  0.7001f,0.3999f,0.5916f,  0.6102f,0.4896f,0.6228f,  0.5977f,0.4965f,0.6295f,  0.4953f,0.5749f,0.6513f,  0.4806f,0.5781f,0.6595f,  0.3680f,0.6435f,0.6711f,  0.3519f,0.6431f,0.6802f,  
	0.2317f,0.6946f,0.6811f,  0.2148f,0.6907f,0.6905f,  0.0898f,0.7276f,0.6801f,  0.0726f,0.7207f,0.6895f,  -0.0545f,0.7425f,0.6676f,  -0.0713f,0.7328f,0.6767f,  -0.1976f,0.7397f,0.6433f,  -0.2137f,0.7278f,0.6516f,  -0.3364f,0.7200f,0.6070f,  -0.3514f,0.7064f,0.6144f,  -0.4678f,0.6847f,0.5589f,  -0.4813f,0.6699f,0.5654f,  -0.5889f,0.6352f,0.4997f,  -0.6009f,0.6196f,0.5050f,  -0.6974f,0.5732f,0.4301f,  
	-0.7076f,0.5574f,0.4343f,  -0.7911f,0.5007f,0.3513f,  -0.7994f,0.4851f,0.3544f,  -0.8683f,0.4196f,0.2646f,  -0.8748f,0.4046f,0.2667f,  -0.9275f,0.3321f,0.1716f,  -0.9323f,0.3179f,0.1727f,  -0.9679f,0.2401f,0.0738f,  -0.9710f,0.2272f,0.0742f,  -0.9889f,0.1460f,-0.0267f,  -0.9905f,0.1345f,-0.0271f,  -0.9904f,0.0516f,-0.1282f,  -0.9908f,0.0416f,-0.1292f,  -0.9726f,-0.0412f,-0.2288f,  -0.9719f,-0.0495f,-0.2302f,  
	-0.9362f,-0.1305f,-0.3265f,  -0.9346f,-0.1372f,-0.3283f,  -0.8820f,-0.2148f,-0.4195f,  -0.8798f,-0.2199f,-0.4215f,  -0.8114f,-0.2926f,-0.5060f,  -0.8087f,-0.2961f,-0.5083f,  -0.7258f,-0.3627f,-0.5845f,  -0.7229f,-0.3648f,-0.5869f,  -0.6271f,-0.4239f,-0.6535f,  -0.6240f,-0.4247f,-0.6559f,  -0.5171f,-0.4755f,-0.7117f,  -0.5141f,-0.4752f,-0.7141f,  -0.3981f,-0.5168f,-0.7579f,  -0.3952f,-0.5155f,-0.7603f,  -0.2722f,-0.5474f,-0.7914f,  
	-0.2695f,-0.5452f,-0.7938f,  -0.1417f,-0.5669f,-0.8115f,  -0.1393f,-0.5641f,-0.8139f,  -0.0091f,-0.5753f,-0.8179f,  -0.0070f,-0.5720f,-0.8202f,  -0.6094f,-0.6444f,-0.4619f,  -0.5984f,-0.6502f,-0.4682f,  -0.5406f,-0.7424f,-0.3956f,  -0.5237f,-0.7483f,-0.4072f,  -0.4516f,-0.8280f,-0.3324f,  -0.4293f,-0.8330f,-0.3490f,  -0.3451f,-0.8980f,-0.2729f,  -0.3182f,-0.9013f,-0.2940f,  -0.2243f,-0.9500f,-0.2171f,  -0.1938f,-0.9507f,-0.2421f,  
	-0.0929f,-0.9819f,-0.1648f,  -0.0599f,-0.9794f,-0.1928f,  0.0448f,-0.9923f,-0.1154f,  0.0792f,-0.9862f,-0.1453f,  0.1846f,-0.9805f,-0.0679f,  0.2194f,-0.9706f,-0.0987f,  0.3222f,-0.9464f,-0.0214f,  0.3562f,-0.9330f,-0.0521f,  0.4533f,-0.8910f,0.0251f,  0.4855f,-0.8742f,-0.0045f,  0.5740f,-0.8157f,0.0724f,  0.6036f,-0.7961f,0.0448f,  0.6806f,-0.7226f,0.1211f,  0.7070f,-0.7006f,0.0963f,  0.7702f,-0.6142f,0.1717f,  
	0.7926f,-0.5909f,0.1502f,  0.8401f,-0.4939f,0.2243f,  0.8583f,-0.4698f,0.2065f,  0.8885f,-0.3647f,0.2785f,  0.9022f,-0.3407f,0.2646f,  0.9140f,-0.2303f,0.3340f,  0.9231f,-0.2071f,0.3240f,  0.9161f,-0.0942f,0.3897f,  0.9207f,-0.0726f,0.3835f,  0.8948f,0.0400f,0.4448f,  0.8950f,0.0594f,0.4420f,  0.8507f,0.1692f,0.4977f,  0.8470f,0.1856f,0.4981f,  0.7852f,0.2900f,0.5471f,  0.7780f,0.3033f,0.5503f,  
	0.7001f,0.3999f,0.5916f,  0.6899f,0.4097f,0.5969f,  0.5977f,0.4965f,0.6295f,  0.5850f,0.5027f,0.6365f,  0.4806f,0.5781f,0.6595f,  0.4661f,0.5806f,0.6676f,  0.3519f,0.6431f,0.6802f,  0.3362f,0.6421f,0.6890f,  0.2148f,0.6907f,0.6905f,  0.1984f,0.6865f,0.6995f,  0.0726f,0.7207f,0.6895f,  0.0561f,0.7135f,0.6984f,  -0.0713f,0.7328f,0.6767f,  -0.0874f,0.7231f,0.6851f,  -0.2137f,0.7278f,0.6516f,  
	-0.2290f,0.7160f,0.6594f,  -0.3514f,0.7064f,0.6144f,  -0.3655f,0.6931f,0.6214f,  -0.4813f,0.6699f,0.5654f,  -0.4940f,0.6554f,0.5713f,  -0.6009f,0.6196f,0.5050f,  -0.6119f,0.6046f,0.5100f,  -0.7076f,0.5574f,0.4343f,  -0.7169f,0.5422f,0.4382f,  -0.7994f,0.4851f,0.3544f,  -0.8070f,0.4701f,0.3573f,  -0.8748f,0.4046f,0.2667f,  -0.8806f,0.3903f,0.2687f,  -0.9323f,0.3179f,0.1727f,  -0.9365f,0.3046f,0.1738f,  
	-0.9710f,0.2272f,0.0742f,  -0.9737f,0.2151f,0.0745f,  -0.9905f,0.1345f,-0.0271f,  -0.9919f,0.1237f,-0.0274f,  -0.9908f,0.0416f,-0.1292f,  -0.9910f,0.0323f,-0.1300f,  -0.9719f,-0.0495f,-0.2302f,  -0.9712f,-0.0573f,-0.2314f,  -0.9346f,-0.1372f,-0.3283f,  -0.9331f,-0.1434f,-0.3298f,  -0.8798f,-0.2199f,-0.4215f,  -0.8777f,-0.2245f,-0.4233f,  -0.8087f,-0.2961f,-0.5083f,  -0.8063f,-0.2994f,-0.5102f,  -0.7229f,-0.3648f,-0.5869f,  
	-0.7202f,-0.3667f,-0.5890f,  -0.6240f,-0.4247f,-0.6559f,  -0.6213f,-0.4254f,-0.6580f,  -0.5141f,-0.4752f,-0.7141f,  -0.5114f,-0.4748f,-0.7162f,  -0.3952f,-0.5155f,-0.7603f,  -0.3927f,-0.5143f,-0.7625f,  -0.2695f,-0.5452f,-0.7938f,  -0.2672f,-0.5433f,-0.7959f,  -0.1393f,-0.5641f,-0.8139f,  -0.1373f,-0.5615f,-0.8160f,  -0.0070f,-0.5720f,-0.8202f,  -0.0053f,-0.5691f,-0.8223f,  -0.5984f,-0.6502f,-0.4682f,  -0.5867f,-0.6559f,-0.4750f,  
	-0.5237f,-0.7483f,-0.4072f,  -0.5062f,-0.7537f,-0.4192f,  -0.4293f,-0.8330f,-0.3490f,  -0.4067f,-0.8371f,-0.3660f,  -0.3182f,-0.9013f,-0.2940f,  -0.2913f,-0.9032f,-0.3153f,  -0.1938f,-0.9507f,-0.2421f,  -0.1636f,-0.9497f,-0.2670f,  -0.0599f,-0.9794f,-0.1928f,  -0.0276f,-0.9750f,-0.2203f,  0.0792f,-0.9862f,-0.1453f,  0.1128f,-0.9782f,-0.1744f,  0.2194f,-0.9706f,-0.0987f,  0.2529f,-0.9589f,-0.1283f,  0.3562f,-0.9330f,-0.0521f,  
	0.3888f,-0.9178f,-0.0812f,  0.4855f,-0.8742f,-0.0045f,  0.5162f,-0.8559f,-0.0322f,  0.6036f,-0.7961f,0.0448f,  0.6317f,-0.7750f,0.0193f,  0.7070f,-0.7006f,0.0963f,  0.7316f,-0.6778f,0.0736f,  0.7926f,-0.5909f,0.1502f,  0.8134f,-0.5667f,0.1309f,  0.8583f,-0.4698f,0.2065f,  0.8749f,-0.4452f,0.1907f,  0.9022f,-0.3407f,0.2646f,  0.9143f,-0.3165f,0.2527f,  0.9231f,-0.2071f,0.3240f,  0.9308f,-0.1841f,0.3157f,  
	0.9207f,-0.0726f,0.3835f,  0.9240f,-0.0514f,0.3788f,  0.8950f,0.0594f,0.4420f,  0.8943f,0.0781f,0.4406f,  0.8470f,0.1856f,0.4981f,  0.8426f,0.2014f,0.4995f,  0.7780f,0.3033f,0.5503f,  0.7703f,0.3158f,0.5541f,  0.6899f,0.4097f,0.5969f,  0.6793f,0.4187f,0.6026f,  0.5850f,0.5027f,0.6365f,  0.5723f,0.5081f,0.6436f,  0.4661f,0.5806f,0.6676f,  0.4518f,0.5825f,0.6757f,  0.3362f,0.6421f,0.6890f,  
	0.3209f,0.6406f,0.6976f,  0.1984f,0.6865f,0.6995f,  0.1826f,0.6819f,0.7083f,  0.0561f,0.7135f,0.6984f,  0.0404f,0.7061f,0.7069f,  -0.0874f,0.7231f,0.6851f,  -0.1027f,0.7134f,0.6932f,  -0.2290f,0.7160f,0.6594f,  -0.2434f,0.7044f,0.6668f,  -0.3655f,0.6931f,0.6214f,  -0.3787f,0.6800f,0.6278f,  -0.4940f,0.6554f,0.5713f,  -0.5058f,0.6414f,0.5769f,  -0.6119f,0.6046f,0.5100f,  -0.6221f,0.5901f,0.5145f,  
	-0.7169f,0.5422f,0.4382f,  -0.7255f,0.5277f,0.4418f,  -0.8070f,0.4701f,0.3573f,  -0.8140f,0.4559f,0.3600f,  -0.8806f,0.3903f,0.2687f,  -0.8860f,0.3767f,0.2705f,  -0.9365f,0.3046f,0.1738f,  -0.9403f,0.2920f,0.1749f,  -0.9737f,0.2151f,0.0745f,  -0.9762f,0.2037f,0.0749f,  -0.9919f,0.1237f,-0.0274f,  -0.9931f,0.1136f,-0.0276f,  -0.9910f,0.0323f,-0.1300f,  -0.9911f,0.0237f,-0.1306f,  -0.9712f,-0.0573f,-0.2314f,  
	-0.9705f,-0.0644f,-0.2325f,  -0.9331f,-0.1434f,-0.3298f,  -0.9317f,-0.1491f,-0.3311f,  -0.8777f,-0.2245f,-0.4233f,  -0.8759f,-0.2288f,-0.4248f,  -0.8063f,-0.2994f,-0.5102f,  -0.8041f,-0.3023f,-0.5119f,  -0.7202f,-0.3667f,-0.5890f,  -0.7179f,-0.3684f,-0.5907f,  -0.6213f,-0.4254f,-0.6580f,  -0.6189f,-0.4260f,-0.6599f,  -0.5114f,-0.4748f,-0.7162f,  -0.5091f,-0.4745f,-0.7181f,  -0.3927f,-0.5143f,-0.7625f,  -0.3905f,-0.5131f,-0.7643f,  
	-0.2672f,-0.5433f,-0.7959f,  -0.2653f,-0.5414f,-0.7978f,  -0.1373f,-0.5615f,-0.8160f,  -0.1356f,-0.5592f,-0.8178f,  -0.0053f,-0.5691f,-0.8223f,  -0.0039f,-0.5664f,-0.8242f,  -0.5867f,-0.6559f,-0.4750f,  -0.5743f,-0.6615f,-0.4822f,  -0.5062f,-0.7537f,-0.4192f,  -0.4882f,-0.7585f,-0.4316f,  -0.4067f,-0.8371f,-0.3660f,  -0.3838f,-0.8402f,-0.3831f,  -0.2913f,-0.9032f,-0.3153f,  -0.2645f,-0.9037f,-0.3366f,  -0.1636f,-0.9497f,-0.2670f,  
	-0.1339f,-0.9472f,-0.2915f,  -0.0276f,-0.9750f,-0.2203f,  0.0042f,-0.9690f,-0.2470f,  0.1128f,-0.9782f,-0.1744f,  0.1453f,-0.9685f,-0.2024f,  0.2529f,-0.9589f,-0.1283f,  0.2853f,-0.9456f,-0.1566f,  0.3888f,-0.9178f,-0.0812f,  0.4199f,-0.9010f,-0.1087f,  0.5162f,-0.8559f,-0.0322f,  0.5455f,-0.8361f,-0.0581f,  0.6317f,-0.7750f,0.0193f,  0.6580f,-0.7530f,-0.0042f,  0.7316f,-0.6778f,0.0736f,  0.7546f,-0.6540f,0.0530f,  
	0.8134f,-0.5667f,0.1309f,  0.8326f,-0.5421f,0.1136f,  0.8749f,-0.4452f,0.1907f,  0.8899f,-0.4204f,0.1770f,  0.9143f,-0.3165f,0.2527f,  0.9250f,-0.2924f,0.2425f,  0.9308f,-0.1841f,0.3157f,  0.9373f,-0.1613f,0.3091f,  0.9240f,-0.0514f,0.3788f,  0.9263f,-0.0308f,0.3755f,  0.8943f,0.0781f,0.4406f,  0.8927f,0.0962f,0.4402f,  0.8426f,0.2014f,0.4995f,  0.8375f,0.2165f,0.5017f,  0.7703f,0.3158f,0.5541f,  
	0.7621f,0.3276f,0.5584f,  0.6793f,0.4187f,0.6026f,  0.6687f,0.4270f,0.6087f,  0.5723f,0.5081f,0.6436f,  0.5597f,0.5129f,0.6509f,  0.4518f,0.5825f,0.6757f,  0.4378f,0.5838f,0.6837f,  0.3209f,0.6406f,0.6976f,  0.3060f,0.6387f,0.7060f,  0.1826f,0.6819f,0.7083f,  0.1675f,0.6770f,0.7167f,  0.0404f,0.7061f,0.7069f,  0.0254f,0.6986f,0.7151f,  -0.1027f,0.7134f,0.6932f,  -0.1171f,0.7037f,0.7008f,  
	-0.2434f,0.7044f,0.6668f,  -0.2569f,0.6930f,0.6736f,  -0.3787f,0.6800f,0.6278f,  -0.3910f,0.6673f,0.6339f,  -0.5058f,0.6414f,0.5769f,  -0.5167f,0.6279f,0.5820f,  -0.6221f,0.5901f,0.5145f,  -0.6315f,0.5762f,0.5188f,  -0.7255f,0.5277f,0.4418f,  -0.7334f,0.5138f,0.4452f,  -0.8140f,0.4559f,0.3600f,  -0.8203f,0.4424f,0.3625f,  -0.8860f,0.3767f,0.2705f,  -0.8908f,0.3639f,0.2722f,  -0.9403f,0.2920f,0.1749f,  
	-0.9437f,0.2802f,0.1759f,  -0.9762f,0.2037f,0.0749f,  -0.9783f,0.1930f,0.0753f,  -0.9931f,0.1136f,-0.0276f,  -0.9942f,0.1042f,-0.0276f,  -0.9911f,0.0237f,-0.1306f,  -0.9912f,0.0156f,-0.1311f,  -0.9705f,-0.0644f,-0.2325f,  -0.9698f,-0.0711f,-0.2333f,  -0.9317f,-0.1491f,-0.3311f,  -0.9305f,-0.1544f,-0.3322f,  -0.8759f,-0.2288f,-0.4248f,  -0.8742f,-0.2328f,-0.4261f,  -0.8041f,-0.3023f,-0.5119f,  -0.8022f,-0.3050f,-0.5133f,  
	-0.7179f,-0.3684f,-0.5907f,  -0.7158f,-0.3699f,-0.5922f,  -0.6189f,-0.4260f,-0.6599f,  -0.6169f,-0.4266f,-0.6614f,  -0.5091f,-0.4745f,-0.7181f,  -0.5072f,-0.4742f,-0.7197f,  -0.3905f,-0.5131f,-0.7643f,  -0.3887f,-0.5121f,-0.7659f,  -0.2653f,-0.5414f,-0.7978f,  -0.2637f,-0.5398f,-0.7994f,  -0.1356f,-0.5592f,-0.8178f,  -0.1343f,-0.5571f,-0.8195f,  -0.0039f,-0.5664f,-0.8242f,  -0.0027f,-0.5639f,-0.8258f,  -0.5743f,-0.6615f,-0.4822f,  
	-0.5613f,-0.6670f,-0.4899f,  -0.4882f,-0.7585f,-0.4316f,  -0.4698f,-0.7628f,-0.4444f,  -0.3838f,-0.8402f,-0.3831f,  -0.3608f,-0.8423f,-0.4004f,  -0.2645f,-0.9037f,-0.3366f,  -0.2378f,-0.9031f,-0.3577f,  -0.1339f,-0.9472f,-0.2915f,  -0.1044f,-0.9432f,-0.3154f,  0.0042f,-0.9690f,-0.2470f,  0.0353f,-0.9614f,-0.2729f,  0.1453f,-0.9685f,-0.2024f,  0.1770f,-0.9572f,-0.2292f,  0.2853f,-0.9456f,-0.1566f,  0.3165f,-0.9307f,-0.1833f,  
	0.4199f,-0.9010f,-0.1087f,  0.4500f,-0.8829f,-0.1344f,  0.5455f,-0.8361f,-0.0581f,  0.5731f,-0.8154f,-0.0820f,  0.6580f,-0.7530f,-0.0042f,  0.6829f,-0.7301f,-0.0257f,  0.7546f,-0.6540f,0.0530f,  0.7761f,-0.6297f,0.0345f,  0.8326f,-0.5421f,0.1136f,  0.8502f,-0.5172f,0.0983f,  0.8899f,-0.4204f,0.1770f,  0.9034f,-0.3957f,0.1651f,  0.9250f,-0.2924f,0.2425f,  0.9344f,-0.2685f,0.2340f,  0.9373f,-0.1613f,0.3091f,  
	0.9425f,-0.1391f,0.3039f,  0.9263f,-0.0308f,0.3755f,  0.9276f,-0.0107f,0.3733f,  0.8927f,0.0962f,0.4402f,  0.8904f,0.1136f,0.4408f,  0.8375f,0.2165f,0.5017f,  0.8319f,0.2309f,0.5047f,  0.7621f,0.3276f,0.5584f,  0.7537f,0.3386f,0.5633f,  0.6687f,0.4270f,0.6087f,  0.6581f,0.4345f,0.6149f,  0.5597f,0.5129f,0.6509f,  0.5473f,0.5170f,0.6582f,  0.4378f,0.5838f,0.6837f,  0.4242f,0.5845f,0.6916f,  
	0.3060f,0.6387f,0.7060f,  0.2917f,0.6363f,0.7141f,  0.1675f,0.6770f,0.7167f,  0.1530f,0.6719f,0.7247f,  0.0254f,0.6986f,0.7151f,  0.0112f,0.6910f,0.7228f,  -0.1171f,0.7037f,0.7008f,  -0.1307f,0.6941f,0.7079f,  -0.2569f,0.6930f,0.6736f,  -0.2695f,0.6818f,0.6801f,  -0.3910f,0.6673f,0.6339f,  -0.4024f,0.6550f,0.6396f,  -0.5167f,0.6279f,0.5820f,  -0.5268f,0.6149f,0.5869f,  -0.6315f,0.5762f,0.5188f,  
	-0.6402f,0.5629f,0.5227f,  -0.7334f,0.5138f,0.4452f,  -0.7406f,0.5006f,0.4483f,  -0.8203f,0.4424f,0.3625f,  -0.8260f,0.4297f,0.3649f,  -0.8908f,0.3639f,0.2722f,  -0.8951f,0.3519f,0.2739f,  -0.9437f,0.2802f,0.1759f,  -0.9467f,0.2690f,0.1770f,  -0.9783f,0.1930f,0.0753f,  -0.9802f,0.1829f,0.0759f,  -0.9942f,0.1042f,-0.0276f,  -0.9951f,0.0954f,-0.0276f,  -0.9912f,0.0156f,-0.1311f,  -0.9913f,0.0081f,-0.1314f,  
	-0.9698f,-0.0711f,-0.2333f,  -0.9692f,-0.0773f,-0.2339f,  -0.9305f,-0.1544f,-0.3322f,  -0.9294f,-0.1593f,-0.3331f,  -0.8742f,-0.2328f,-0.4261f,  -0.8727f,-0.2364f,-0.4271f,  -0.8022f,-0.3050f,-0.5133f,  -0.8005f,-0.3075f,-0.5144f,  -0.7158f,-0.3699f,-0.5922f,  -0.7141f,-0.3714f,-0.5934f,  -0.6169f,-0.4266f,-0.6614f,  -0.6151f,-0.4271f,-0.6627f,  -0.5072f,-0.4742f,-0.7197f,  -0.5055f,-0.4739f,-0.7210f,  -0.3887f,-0.5121f,-0.7659f,  
	-0.3872f,-0.5112f,-0.7673f,  -0.2637f,-0.5398f,-0.7994f,  -0.2624f,-0.5384f,-0.8008f,  -0.1343f,-0.5571f,-0.8195f,  -0.1332f,-0.5553f,-0.8209f,  -0.0027f,-0.5639f,-0.8258f,  -0.0019f,-0.5618f,-0.8273f,  -0.5613f,-0.6670f,-0.4899f,  -0.5476f,-0.6724f,-0.4981f,  -0.4698f,-0.7628f,-0.4444f,  -0.4509f,-0.7664f,-0.4575f,  -0.3608f,-0.8423f,-0.4004f,  -0.3376f,-0.8435f,-0.4178f,  -0.2378f,-0.9031f,-0.3577f,  -0.2111f,-0.9012f,-0.3785f,  
	-0.1044f,-0.9432f,-0.3154f,  -0.0754f,-0.9378f,-0.3388f,  0.0353f,-0.9614f,-0.2729f,  0.0656f,-0.9524f,-0.2979f,  0.1770f,-0.9572f,-0.2292f,  0.2076f,-0.9445f,-0.2547f,  0.3165f,-0.9307f,-0.1833f,  0.3467f,-0.9145f,-0.2084f,  0.4500f,-0.8829f,-0.1344f,  0.4783f,-0.8638f,-0.1583f,  0.5731f,-0.8154f,-0.0820f,  0.5994f,-0.7937f,-0.1040f,  0.6829f,-0.7301f,-0.0257f,  0.7062f,-0.7066f,-0.0452f,  0.7761f,-0.6297f,0.0345f,  
	0.7960f,-0.6051f,0.0179f,  0.8502f,-0.5172f,0.0983f,  0.8663f,-0.4922f,0.0849f,  0.9034f,-0.3957f,0.1651f,  0.9156f,-0.3711f,0.1550f,  0.9344f,-0.2685f,0.2340f,  0.9425f,-0.2451f,0.2271f,  0.9425f,-0.1391f,0.3039f,  0.9467f,-0.1174f,0.3001f,  0.9276f,-0.0107f,0.3733f,  0.9281f,0.0087f,0.3723f,  0.8904f,0.1136f,0.4408f,  0.8874f,0.1302f,0.4423f,  0.8319f,0.2309f,0.5047f,  0.8258f,0.2443f,0.5082f,  
	0.7537f,0.3386f,0.5633f,  0.7452f,0.3487f,0.5685f,  0.6581f,0.4345f,0.6149f,  0.6474f,0.4413f,0.6214f,  0.5473f,0.5170f,0.6582f,  0.5351f,0.5204f,0.6655f,  0.4242f,0.5845f,0.6916f,  0.4110f,0.5848f,0.6994f,  0.2917f,0.6363f,0.7141f,  0.2780f,0.6336f,0.7220f,  0.1530f,0.6719f,0.7247f,  0.1392f,0.6665f,0.7324f,  0.0112f,0.6910f,0.7228f,  -0.0023f,0.6834f,0.7301f,  -0.1307f,0.6941f,0.7079f,  
	-0.1434f,0.6846f,0.7147f,  -0.2695f,0.6818f,0.6801f,  -0.2813f,0.6708f,0.6862f,  -0.4024f,0.6550f,0.6396f,  -0.4130f,0.6430f,0.6449f,  -0.5268f,0.6149f,0.5869f,  -0.5361f,0.6023f,0.5914f,  -0.6402f,0.5629f,0.5227f,  -0.6482f,0.5502f,0.5265f,  -0.7406f,0.5006f,0.4483f,  -0.7471f,0.4880f,0.4513f,  -0.8260f,0.4297f,0.3649f,  -0.8312f,0.4175f,0.3672f,  -0.8951f,0.3519f,0.2739f,  -0.8990f,0.3405f,0.2755f,  
	-0.9467f,0.2690f,0.1770f,  -0.9494f,0.2586f,0.1780f,  -0.9802f,0.1829f,0.0759f,  -0.9818f,0.1736f,0.0765f,  -0.9951f,0.0954f,-0.0276f,  -0.9958f,0.0872f,-0.0274f,  -0.9913f,0.0081f,-0.1314f,  -0.9913f,0.0011f,-0.1316f,  -0.9692f,-0.0773f,-0.2339f,  -0.9686f,-0.0830f,-0.2343f,  -0.9294f,-0.1593f,-0.3331f,  -0.9284f,-0.1638f,-0.3336f,  -0.8727f,-0.2364f,-0.4271f,  -0.8714f,-0.2398f,-0.4279f,  -0.8005f,-0.3075f,-0.5144f,  
	-0.7990f,-0.3099f,-0.5153f,  -0.7141f,-0.3714f,-0.5934f,  -0.7125f,-0.3728f,-0.5944f,  -0.6151f,-0.4271f,-0.6627f,  -0.6136f,-0.4277f,-0.6637f,  -0.5055f,-0.4739f,-0.7210f,  -0.5041f,-0.4738f,-0.7221f,  -0.3872f,-0.5112f,-0.7673f,  -0.3860f,-0.5104f,-0.7684f,  -0.2624f,-0.5384f,-0.8008f,  -0.2613f,-0.5372f,-0.8020f,  -0.1332f,-0.5553f,-0.8209f,  -0.1323f,-0.5537f,-0.8221f,  -0.0019f,-0.5618f,-0.8273f,  -0.0012f,-0.5599f,-0.8285f,  
	-0.5476f,-0.6724f,-0.4981f,  -0.5332f,-0.6775f,-0.5067f,  -0.4509f,-0.7664f,-0.4575f,  -0.4317f,-0.7695f,-0.4707f,  -0.3376f,-0.8435f,-0.4178f,  -0.3143f,-0.8438f,-0.4350f,  -0.2111f,-0.9012f,-0.3785f,  -0.1847f,-0.8982f,-0.3989f,  -0.0754f,-0.9378f,-0.3388f,  -0.0469f,-0.9312f,-0.3614f,  0.0656f,-0.9524f,-0.2979f,  0.0951f,-0.9420f,-0.3217f,  0.2076f,-0.9445f,-0.2547f,  0.2374f,-0.9305f,-0.2788f,  0.3467f,-0.9145f,-0.2084f,  
	0.3754f,-0.8974f,-0.2319f,  0.4783f,-0.8638f,-0.1583f,  0.5056f,-0.8437f,-0.1805f,  0.5994f,-0.7937f,-0.1040f,  0.6242f,-0.7713f,-0.1241f,  0.7062f,-0.7066f,-0.0452f,  0.7281f,-0.6826f,-0.0628f,  0.7960f,-0.6051f,0.0179f,  0.8144f,-0.5803f,0.0032f,  0.8663f,-0.4922f,0.0849f,  0.8811f,-0.4673f,0.0733f,  0.9156f,-0.3711f,0.1550f,  0.9264f,-0.3468f,0.1465f,  0.9425f,-0.2451f,0.2271f,  0.9495f,-0.2221f,0.2217f,  
	0.9467f,-0.1174f,0.3001f,  0.9498f,-0.0964f,0.2975f,  0.9281f,0.0087f,0.3723f,  0.9277f,0.0273f,0.3723f,  0.8874f,0.1302f,0.4423f,  0.8838f,0.1459f,0.4445f,  0.8258f,0.2443f,0.5082f,  0.8195f,0.2570f,0.5122f,  0.7452f,0.3487f,0.5685f,  0.7365f,0.3581f,0.5739f,  0.6474f,0.4413f,0.6214f,  0.6369f,0.4473f,0.6279f,  0.5351f,0.5204f,0.6655f,  0.5233f,0.5232f,0.6726f,  0.4110f,0.5848f,0.6994f,  
	0.3983f,0.5845f,0.7069f,  0.2780f,0.6336f,0.7220f,  0.2650f,0.6305f,0.7295f,  0.1392f,0.6665f,0.7324f,  0.1262f,0.6609f,0.7398f,  -0.0023f,0.6834f,0.7301f,  -0.0149f,0.6757f,0.7370f,  -0.1434f,0.6846f,0.7147f,  -0.1553f,0.6752f,0.7211f,  -0.2813f,0.6708f,0.6862f,  -0.2922f,0.6602f,0.6919f,  -0.4130f,0.6430f,0.6449f,  -0.4229f,0.6315f,0.6499f,  -0.5361f,0.6023f,0.5914f,  -0.5447f,0.5903f,0.5956f,  
	-0.6482f,0.5502f,0.5265f,  -0.6554f,0.5380f,0.5300f,  -0.7471f,0.4880f,0.4513f,  -0.7530f,0.4761f,0.4542f,  -0.8312f,0.4175f,0.3672f,  -0.8358f,0.4061f,0.3694f,  -0.8990f,0.3405f,0.2755f,  -0.9025f,0.3298f,0.2772f,  -0.9494f,0.2586f,0.1780f,  -0.9518f,0.2487f,0.1792f,  -0.9818f,0.1736f,0.0765f,  -0.9833f,0.1648f,0.0772f,  -0.9958f,0.0872f,-0.0274f,  -0.9965f,0.0795f,-0.0270f,  -0.9913f,0.0011f,-0.1316f,  
	-0.9913f,-0.0054f,-0.1315f,  -0.9686f,-0.0830f,-0.2343f,  -0.9681f,-0.0884f,-0.2344f,  -0.9284f,-0.1638f,-0.3336f,  -0.9275f,-0.1681f,-0.3340f,  -0.8714f,-0.2398f,-0.4279f,  -0.8703f,-0.2430f,-0.4284f,  -0.7990f,-0.3099f,-0.5153f,  -0.7978f,-0.3121f,-0.5159f,  -0.7125f,-0.3728f,-0.5944f,  -0.7113f,-0.3741f,-0.5951f,  -0.6136f,-0.4277f,-0.6637f,  -0.6124f,-0.4283f,-0.6645f,  -0.5041f,-0.4738f,-0.7221f,  -0.5030f,-0.4737f,-0.7229f,  
	-0.3860f,-0.5104f,-0.7684f,  -0.3850f,-0.5098f,-0.7693f,  -0.2613f,-0.5372f,-0.8020f,  -0.2605f,-0.5361f,-0.8029f,  -0.1323f,-0.5537f,-0.8221f,  -0.1317f,-0.5524f,-0.8231f,  -0.0012f,-0.5599f,-0.8285f,  -0.0007f,-0.5583f,-0.8296f,  -0.5332f,-0.6775f,-0.5067f,  -0.5183f,-0.6823f,-0.5156f,  -0.4317f,-0.7695f,-0.4707f,  -0.4120f,-0.7719f,-0.4841f,  -0.3143f,-0.8438f,-0.4350f,  -0.2909f,-0.8432f,-0.4521f,  -0.1847f,-0.8982f,-0.3989f,  
	-0.1585f,-0.8941f,-0.4188f,  -0.0469f,-0.9312f,-0.3614f,  -0.0189f,-0.9235f,-0.3832f,  0.0951f,-0.9420f,-0.3217f,  0.1241f,-0.9306f,-0.3444f,  0.2374f,-0.9305f,-0.2788f,  0.2660f,-0.9156f,-0.3016f,  0.3754f,-0.8974f,-0.2319f,  0.4031f,-0.8792f,-0.2539f,  0.5056f,-0.8437f,-0.1805f,  0.5315f,-0.8229f,-0.2009f,  0.6242f,-0.7713f,-0.1241f,  0.6477f,-0.7485f,-0.1424f,  0.7281f,-0.6826f,-0.0628f,  0.7485f,-0.6585f,-0.0785f,  
	0.8144f,-0.5803f,0.0032f,  0.8315f,-0.5555f,-0.0097f,  0.8811f,-0.4673f,0.0733f,  0.8944f,-0.4427f,0.0633f,  0.9264f,-0.3468f,0.1465f,  0.9361f,-0.3230f,0.1395f,  0.9495f,-0.2221f,0.2217f,  0.9554f,-0.1998f,0.2175f,  0.9498f,-0.0964f,0.2975f,  0.9522f,-0.0761f,0.2959f,  0.9277f,0.0273f,0.3723f,  0.9267f,0.0450f,0.3731f,  0.8838f,0.1459f,0.4445f,  0.8798f,0.1608f,0.4472f,  0.8195f,0.2570f,0.5122f,  
	0.8129f,0.2688f,0.5166f,  0.7365f,0.3581f,0.5739f,  0.7278f,0.3666f,0.5796f,  0.6369f,0.4473f,0.6279f,  0.6266f,0.4527f,0.6344f,  0.5233f,0.5232f,0.6726f,  0.5118f,0.5255f,0.6797f,  0.3983f,0.5845f,0.7069f,  0.3861f,0.5838f,0.7142f,  0.2650f,0.6305f,0.7295f,  0.2525f,0.6272f,0.7368f,  0.1262f,0.6609f,0.7398f,  0.1139f,0.6553f,0.7468f,  -0.0149f,0.6757f,0.7370f,  -0.0267f,0.6681f,0.7436f,  
	-0.1553f,0.6752f,0.7211f,  -0.1664f,0.6661f,0.7271f,  -0.2922f,0.6602f,0.6919f,  -0.3024f,0.6499f,0.6973f,  -0.4229f,0.6315f,0.6499f,  -0.4319f,0.6204f,0.6546f,  -0.5447f,0.5903f,0.5956f,  -0.5525f,0.5788f,0.5997f,  -0.6554f,0.5380f,0.5300f,  -0.6620f,0.5265f,0.5334f,  -0.7530f,0.4761f,0.4542f,  -0.7584f,0.4648f,0.4569f,  -0.8358f,0.4061f,0.3694f,  -0.8400f,0.3953f,0.3716f,  -0.9025f,0.3298f,0.2772f,  
	-0.9056f,0.3197f,0.2788f,  -0.9518f,0.2487f,0.1792f,  -0.9540f,0.2395f,0.1804f,  -0.9833f,0.1648f,0.0772f,  -0.9846f,0.1565f,0.0780f,  -0.9965f,0.0795f,-0.0270f,  -0.9970f,0.0724f,-0.0265f,  -0.9913f,-0.0054f,-0.1315f,  -0.9913f,-0.0115f,-0.1312f,  -0.9681f,-0.0884f,-0.2344f,  -0.9676f,-0.0934f,-0.2344f,  -0.9275f,-0.1681f,-0.3340f,  -0.9267f,-0.1720f,-0.3341f,  -0.8703f,-0.2430f,-0.4284f,  -0.8693f,-0.2460f,-0.4286f,  
	-0.7978f,-0.3121f,-0.5159f,  -0.7967f,-0.3142f,-0.5163f,  -0.7113f,-0.3741f,-0.5951f,  -0.7102f,-0.3755f,-0.5955f,  -0.6124f,-0.4283f,-0.6645f,  -0.6114f,-0.4289f,-0.6650f,  -0.5030f,-0.4737f,-0.7229f,  -0.5021f,-0.4738f,-0.7235f,  -0.3850f,-0.5098f,-0.7693f,  -0.3843f,-0.5094f,-0.7700f,  -0.2605f,-0.5361f,-0.8029f,  -0.2599f,-0.5354f,-0.8036f,  -0.1317f,-0.5524f,-0.8231f,  -0.1312f,-0.5513f,-0.8239f,  -0.0007f,-0.5583f,-0.8296f,  
	-0.0004f,-0.5570f,-0.8305f,  -0.5183f,-0.6823f,-0.5156f,  -0.5026f,-0.6869f,-0.5250f,  -0.4120f,-0.7719f,-0.4841f,  -0.3920f,-0.7737f,-0.4977f,  -0.2909f,-0.8432f,-0.4521f,  -0.2674f,-0.8417f,-0.4690f,  -0.1585f,-0.8941f,-0.4188f,  -0.1327f,-0.8890f,-0.4382f,  -0.0189f,-0.9235f,-0.3832f,  0.0087f,-0.9146f,-0.4042f,  0.1241f,-0.9306f,-0.3444f,  0.1520f,-0.9181f,-0.3660f,  0.2660f,-0.9156f,-0.3016f,  0.2937f,-0.8997f,-0.3229f,  
	0.4031f,-0.8792f,-0.2539f,  0.4297f,-0.8603f,-0.2742f,  0.5315f,-0.8229f,-0.2009f,  0.5562f,-0.8016f,-0.2196f,  0.6477f,-0.7485f,-0.1424f,  0.6697f,-0.7254f,-0.1589f,  0.7485f,-0.6585f,-0.0785f,  0.7676f,-0.6342f,-0.0925f,  0.8315f,-0.5555f,-0.0097f,  0.8472f,-0.5309f,-0.0210f,  0.8944f,-0.4427f,0.0633f,  0.9066f,-0.4184f,0.0549f,  0.9361f,-0.3230f,0.1395f,  0.9446f,-0.2998f,0.1338f,  0.9554f,-0.1998f,0.2175f,  
	0.9603f,-0.1781f,0.2145f,  0.9522f,-0.0761f,0.2959f,  0.9537f,-0.0567f,0.2953f,  0.9267f,0.0450f,0.3731f,  0.9251f,0.0619f,0.3746f,  0.8798f,0.1608f,0.4472f,  0.8755f,0.1748f,0.4505f,  0.8129f,0.2688f,0.5166f,  0.8062f,0.2797f,0.5213f,  0.7278f,0.3666f,0.5796f,  0.7191f,0.3745f,0.5853f,  0.6266f,0.4527f,0.6344f,  0.6165f,0.4574f,0.6409f,  0.5118f,0.5255f,0.6797f,  0.5007f,0.5271f,0.6866f,  
	0.3861f,0.5838f,0.7142f,  0.3745f,0.5827f,0.7212f,  0.2525f,0.6272f,0.7368f,  0.2408f,0.6236f,0.7437f,  0.1139f,0.6553f,0.7468f,  0.1024f,0.6495f,0.7534f,  -0.0267f,0.6681f,0.7436f,  -0.0378f,0.6606f,0.7498f,  -0.1664f,0.6661f,0.7271f,  -0.1768f,0.6571f,0.7328f,  -0.3024f,0.6499f,0.6973f,  -0.3118f,0.6399f,0.7024f,  -0.4319f,0.6204f,0.6546f,  -0.4402f,0.6098f,0.6591f,  -0.5525f,0.5788f,0.5997f,  
	-0.5597f,0.5679f,0.6035f,  -0.6620f,0.5265f,0.5334f,  -0.6680f,0.5155f,0.5367f,  -0.7584f,0.4648f,0.4569f,  -0.7633f,0.4541f,0.4596f,  -0.8400f,0.3953f,0.3716f,  -0.8438f,0.3851f,0.3737f,  -0.9056f,0.3197f,0.2788f,  -0.9083f,0.3102f,0.2806f,  -0.9540f,0.2395f,0.1804f,  -0.9559f,0.2309f,0.1817f,  -0.9846f,0.1565f,0.0780f,  -0.9857f,0.1489f,0.0790f,  -0.9970f,0.0724f,-0.0265f,  -0.9975f,0.0657f,-0.0258f,  
	-0.9913f,-0.0115f,-0.1312f,  -0.9913f,-0.0171f,-0.1308f,  -0.9676f,-0.0934f,-0.2344f,  -0.9672f,-0.0981f,-0.2341f,  -0.9267f,-0.1720f,-0.3341f,  -0.9261f,-0.1757f,-0.3340f,  -0.8693f,-0.2460f,-0.4286f,  -0.8685f,-0.2488f,-0.4286f,  -0.7967f,-0.3142f,-0.5163f,  -0.7958f,-0.3162f,-0.5164f,  -0.7102f,-0.3755f,-0.5955f,  -0.7093f,-0.3768f,-0.5957f,  -0.6114f,-0.4289f,-0.6650f,  -0.6106f,-0.4296f,-0.6653f,  -0.5021f,-0.4738f,-0.7235f,  
	-0.5014f,-0.4740f,-0.7238f,  -0.3843f,-0.5094f,-0.7700f,  -0.3837f,-0.5092f,-0.7704f,  -0.2599f,-0.5354f,-0.8036f,  -0.2595f,-0.5348f,-0.8041f,  -0.1312f,-0.5513f,-0.8239f,  -0.1309f,-0.5505f,-0.8245f,  -0.0004f,-0.5570f,-0.8305f,  -0.0002f,-0.5560f,-0.8312f,  -0.5026f,-0.6869f,-0.5250f,  -0.4864f,-0.6910f,-0.5347f,  -0.3920f,-0.7737f,-0.4977f,  -0.3717f,-0.7749f,-0.5113f,  -0.2674f,-0.8417f,-0.4690f,  -0.2441f,-0.8394f,-0.4857f,  
	-0.1327f,-0.8890f,-0.4382f,  -0.1068f,-0.8830f,-0.4570f,  0.0087f,-0.9146f,-0.4042f,  0.0355f,-0.9049f,-0.4241f,  0.1520f,-0.9181f,-0.3660f,  0.1793f,-0.9047f,-0.3864f,  0.2937f,-0.8997f,-0.3229f,  0.3205f,-0.8831f,-0.3428f,  0.4297f,-0.8603f,-0.2742f,  0.4551f,-0.8409f,-0.2929f,  0.5562f,-0.8016f,-0.2196f,  0.5795f,-0.7799f,-0.2366f,  0.6697f,-0.7254f,-0.1589f,  0.6906f,-0.7021f,-0.1737f,  0.7676f,-0.6342f,-0.0925f,  
	0.7853f,-0.6101f,-0.1049f,  0.8472f,-0.5309f,-0.0210f,  0.8616f,-0.5067f,-0.0307f,  0.9066f,-0.4184f,0.0549f,  0.9176f,-0.3947f,0.0478f,  0.9446f,-0.2998f,0.1338f,  0.9521f,-0.2772f,0.1294f,  0.9603f,-0.1781f,0.2145f,  0.9644f,-0.1574f,0.2125f,  0.9537f,-0.0567f,0.2953f,  0.9546f,-0.0381f,0.2955f,  0.9251f,0.0619f,0.3746f,  0.9231f,0.0779f,0.3767f,  0.8755f,0.1748f,0.4505f,  0.8709f,0.1879f,0.4542f,  
	0.8062f,0.2797f,0.5213f,  0.7994f,0.2898f,0.5262f,  0.7191f,0.3745f,0.5853f,  0.7107f,0.3815f,0.5911f,  0.6165f,0.4574f,0.6409f,  0.6067f,0.4614f,0.6473f,  0.5007f,0.5271f,0.6866f,  0.4901f,0.5283f,0.6933f,  0.3745f,0.5827f,0.7212f,  0.3635f,0.5813f,0.7280f,  0.2408f,0.6236f,0.7437f,  0.2297f,0.6198f,0.7504f,  0.1024f,0.6495f,0.7534f,  0.0915f,0.6437f,0.7597f,  -0.0378f,0.6606f,0.7498f,  
	-0.0481f,0.6531f,0.7557f,  -0.1768f,0.6571f,0.7328f,  -0.1863f,0.6484f,0.7381f,  -0.3118f,0.6399f,0.7024f,  -0.3204f,0.6303f,0.7072f,  -0.4402f,0.6098f,0.6591f,  -0.4478f,0.5996f,0.6633f,  -0.5597f,0.5679f,0.6035f,  -0.5662f,0.5574f,0.6072f,  -0.6680f,0.5155f,0.5367f,  -0.6734f,0.5051f,0.5398f,  -0.7633f,0.4541f,0.4596f,  -0.7676f,0.4440f,0.4622f,  -0.8438f,0.3851f,0.3737f,  -0.8472f,0.3756f,0.3759f,  
	-0.9083f,0.3102f,0.2806f,  -0.9108f,0.3013f,0.2823f,  -0.9559f,0.2309f,0.1817f,  -0.9575f,0.2228f,0.1831f,  -0.9857f,0.1489f,0.0790f,  -0.9867f,0.1417f,0.0801f,  -0.9975f,0.0657f,-0.0258f,  -0.9979f,0.0594f,-0.0249f,  -0.9913f,-0.0171f,-0.1308f,  -0.9912f,-0.0224f,-0.1301f,  -0.9672f,-0.0981f,-0.2341f,  -0.9669f,-0.1025f,-0.2336f,  -0.9261f,-0.1757f,-0.3340f,  -0.9255f,-0.1792f,-0.3337f,  -0.8685f,-0.2488f,-0.4286f,  
	-0.8679f,-0.2515f,-0.4284f,  -0.7958f,-0.3162f,-0.5164f,  -0.7951f,-0.3182f,-0.5163f,  -0.7093f,-0.3768f,-0.5957f,  -0.7087f,-0.3781f,-0.5957f,  -0.6106f,-0.4296f,-0.6653f,  -0.6100f,-0.4304f,-0.6653f,  -0.5014f,-0.4740f,-0.7238f,  -0.5009f,-0.4743f,-0.7240f,  -0.3837f,-0.5092f,-0.7704f,  -0.3833f,-0.5092f,-0.7706f,  -0.2595f,-0.5348f,-0.8041f,  -0.2592f,-0.5345f,-0.8044f,  -0.1309f,-0.5505f,-0.8245f,  -0.1307f,-0.5499f,-0.8249f,  
	-0.0002f,-0.5560f,-0.8312f,  -0.0001f,-0.5553f,-0.8316f,  -0.4864f,-0.6910f,-0.5347f,  -0.4696f,-0.6948f,-0.5447f,  -0.3717f,-0.7749f,-0.5113f,  -0.3513f,-0.7754f,-0.5248f,  -0.2441f,-0.8394f,-0.4857f,  -0.2205f,-0.8363f,-0.5019f,  -0.1068f,-0.8830f,-0.4570f,  -0.0816f,-0.8762f,-0.4750f,  0.0355f,-0.9049f,-0.4241f,  0.0619f,-0.8943f,-0.4432f,  0.1793f,-0.9047f,-0.3864f,  0.2058f,-0.8906f,-0.4055f,  0.3205f,-0.8831f,-0.3428f,  
	0.3463f,-0.8658f,-0.3613f,  0.4551f,-0.8409f,-0.2929f,  0.4793f,-0.8210f,-0.3101f,  0.5795f,-0.7799f,-0.2366f,  0.6017f,-0.7579f,-0.2520f,  0.6906f,-0.7021f,-0.1737f,  0.7101f,-0.6789f,-0.1870f,  0.7853f,-0.6101f,-0.1049f,  0.8018f,-0.5862f,-0.1157f,  0.8616f,-0.5067f,-0.0307f,  0.8749f,-0.4828f,-0.0390f,  0.9176f,-0.3947f,0.0478f,  0.9275f,-0.3715f,0.0420f,  0.9521f,-0.2772f,0.1294f,  0.9586f,-0.2554f,0.1260f,  
	0.9644f,-0.1574f,0.2125f,  0.9677f,-0.1374f,0.2113f,  0.9546f,-0.0381f,0.2955f,  0.9549f,-0.0203f,0.2963f,  0.9231f,0.0779f,0.3767f,  0.9206f,0.0930f,0.3792f,  0.8709f,0.1879f,0.4542f,  0.8661f,0.2002f,0.4581f,  0.7994f,0.2898f,0.5262f,  0.7927f,0.2991f,0.5312f,  0.7107f,0.3815f,0.5911f,  0.7024f,0.3878f,0.5969f,  0.6067f,0.4614f,0.6473f,  0.5973f,0.4649f,0.6535f,  0.4901f,0.5283f,0.6933f,  
	0.4800f,0.5290f,0.6998f,  0.3635f,0.5813f,0.7280f,  0.3531f,0.5796f,0.7345f,  0.2297f,0.6198f,0.7504f,  0.2193f,0.6159f,0.7567f,  0.0915f,0.6437f,0.7597f,  0.0814f,0.6380f,0.7657f,  -0.0481f,0.6531f,0.7557f,  -0.0576f,0.6459f,0.7613f,  -0.1863f,0.6484f,0.7381f,  -0.1950f,0.6400f,0.7432f,  -0.3204f,0.6303f,0.7072f,  -0.3282f,0.6210f,0.7118f,  -0.4478f,0.5996f,0.6633f,  -0.4547f,0.5898f,0.6674f,  
	-0.5662f,0.5574f,0.6072f,  -0.5721f,0.5475f,0.6107f,  -0.6734f,0.5051f,0.5398f,  -0.6783f,0.4953f,0.5428f,  -0.7676f,0.4440f,0.4622f,  -0.7715f,0.4345f,0.4648f,  -0.8472f,0.3756f,0.3759f,  -0.8501f,0.3666f,0.3781f,  -0.9108f,0.3013f,0.2823f,  -0.9129f,0.2930f,0.2841f,  -0.9575f,0.2228f,0.1831f,  -0.9590f,0.2153f,0.1846f,  -0.9867f,0.1417f,0.0801f,  -0.9875f,0.1349f,0.0813f,  -0.9979f,0.0594f,-0.0249f,  
	-0.9983f,0.0536f,-0.0239f,  -0.9912f,-0.0224f,-0.1301f,  -0.9912f,-0.0274f,-0.1293f,  -0.9669f,-0.1025f,-0.2336f,  -0.9666f,-0.1066f,-0.2330f,  -0.9255f,-0.1792f,-0.3337f,  -0.9250f,-0.1826f,-0.3331f,  -0.8679f,-0.2515f,-0.4284f,  -0.8673f,-0.2541f,-0.4280f,  -0.7951f,-0.3182f,-0.5163f,  -0.7946f,-0.3201f,-0.5159f,  -0.7087f,-0.3781f,-0.5957f,  -0.7081f,-0.3795f,-0.5954f,  -0.6100f,-0.4304f,-0.6653f,  -0.6095f,-0.4313f,-0.6652f,  
	-0.5009f,-0.4743f,-0.7240f,  -0.5005f,-0.4748f,-0.7239f,  -0.3833f,-0.5092f,-0.7706f,  -0.3830f,-0.5094f,-0.7706f,  -0.2592f,-0.5345f,-0.8044f,  -0.2590f,-0.5344f,-0.8045f,  -0.1307f,-0.5499f,-0.8249f,  -0.1306f,-0.5497f,-0.8251f,  -0.0001f,-0.5553f,-0.8316f,  -0.0001f,-0.5549f,-0.8319f,  -0.4696f,-0.6948f,-0.5447f,  -0.4524f,-0.6981f,-0.5550f,  -0.3513f,-0.7754f,-0.5248f,  -0.3304f,-0.7753f,-0.5383f,  -0.2205f,-0.8363f,-0.5019f,  
	-0.1974f,-0.8324f,-0.5178f,  -0.0816f,-0.8762f,-0.4750f,  -0.0566f,-0.8685f,-0.4924f,  0.0619f,-0.8943f,-0.4432f,  0.0876f,-0.8829f,-0.4612f,  0.2058f,-0.8906f,-0.4055f,  0.2315f,-0.8759f,-0.4234f,  0.3463f,-0.8658f,-0.3613f,  0.3709f,-0.8481f,-0.3784f,  0.4793f,-0.8210f,-0.3101f,  0.5025f,-0.8008f,-0.3259f,  0.6017f,-0.7579f,-0.2520f,  0.6226f,-0.7360f,-0.2659f,  0.7101f,-0.6789f,-0.1870f,  0.7284f,-0.6557f,-0.1988f,  
	0.8018f,-0.5862f,-0.1157f,  0.8172f,-0.5626f,-0.1252f,  0.8749f,-0.4828f,-0.0390f,  0.8870f,-0.4594f,-0.0460f,  0.9275f,-0.3715f,0.0420f,  0.9364f,-0.3490f,0.0374f,  0.9586f,-0.2554f,0.1260f,  0.9643f,-0.2344f,0.1236f,  0.9677f,-0.1374f,0.2113f,  0.9703f,-0.1183f,0.2110f,  0.9549f,-0.0203f,0.2963f,  0.9546f,-0.0036f,0.2977f,  0.9206f,0.0930f,0.3792f,  0.9179f,0.1071f,0.3821f,  0.8661f,0.2002f,0.4581f,  
	0.8612f,0.2116f,0.4622f,  0.7927f,0.2991f,0.5312f,  0.7860f,0.3076f,0.5363f,  0.7024f,0.3878f,0.5969f,  0.6943f,0.3934f,0.6026f,  0.5973f,0.4649f,0.6535f,  0.5883f,0.4677f,0.6596f,  0.4800f,0.5290f,0.6998f,  0.4705f,0.5294f,0.7060f,  0.3531f,0.5796f,0.7345f,  0.3433f,0.5776f,0.7406f,  0.2193f,0.6159f,0.7567f,  0.2096f,0.6119f,0.7627f,  0.0814f,0.6380f,0.7657f,  0.0721f,0.6323f,0.7714f,  
	-0.0576f,0.6459f,0.7613f,  -0.0664f,0.6387f,0.7666f,  -0.1950f,0.6400f,0.7432f,  -0.2030f,0.6318f,0.7481f,  -0.3282f,0.6210f,0.7118f,  -0.3354f,0.6121f,0.7161f,  -0.4547f,0.5898f,0.6674f,  -0.4609f,0.5805f,0.6712f,  -0.5721f,0.5475f,0.6107f,  -0.5773f,0.5381f,0.6141f,  -0.6783f,0.4953f,0.5428f,  -0.6826f,0.4860f,0.5457f,  -0.7715f,0.4345f,0.4648f,  -0.7749f,0.4255f,0.4673f,  -0.8501f,0.3666f,0.3781f,  
	-0.8528f,0.3581f,0.3802f,  -0.9129f,0.2930f,0.2841f,  -0.9148f,0.2852f,0.2860f,  -0.9590f,0.2153f,0.1846f,  -0.9602f,0.2082f,0.1862f,  -0.9875f,0.1349f,0.0813f,  -0.9882f,0.1287f,0.0827f,  -0.9983f,0.0536f,-0.0239f,  -0.9986f,0.0481f,-0.0228f,  -0.9912f,-0.0274f,-0.1293f,  -0.9912f,-0.0321f,-0.1283f,  -0.9666f,-0.1066f,-0.2330f,  -0.9664f,-0.1105f,-0.2321f,  -0.9250f,-0.1826f,-0.3331f,  -0.9247f,-0.1858f,-0.3323f,  
	-0.8673f,-0.2541f,-0.4280f,  -0.8669f,-0.2567f,-0.4273f,  -0.7946f,-0.3201f,-0.5159f,  -0.7942f,-0.3221f,-0.5153f,  -0.7081f,-0.3795f,-0.5954f,  -0.7077f,-0.3809f,-0.5950f,  -0.6095f,-0.4313f,-0.6652f,  -0.6092f,-0.4324f,-0.6648f,  -0.5005f,-0.4748f,-0.7239f,  -0.5003f,-0.4755f,-0.7236f,  -0.3830f,-0.5094f,-0.7706f,  -0.3829f,-0.5098f,-0.7704f,  -0.2590f,-0.5344f,-0.8045f,  -0.2589f,-0.5346f,-0.8044f,  -0.1306f,-0.5497f,-0.8251f,  
	-0.1306f,-0.5497f,-0.8251f,  -0.0001f,-0.5549f,-0.8319f,  -0.0001f,-0.5547f,-0.8320f,  -0.4524f,-0.6981f,-0.5550f,  -0.4346f,-0.7009f,-0.5655f,  -0.3304f,-0.7753f,-0.5383f,  -0.3097f,-0.7744f,-0.5517f,  -0.1974f,-0.8324f,-0.5178f,  -0.1742f,-0.8278f,-0.5333f,  -0.0566f,-0.8685f,-0.4924f,  -0.0321f,-0.8601f,-0.5091f,  0.0876f,-0.8829f,-0.4612f,  0.1127f,-0.8709f,-0.4783f,  0.2315f,-0.8759f,-0.4234f,  0.2562f,-0.8606f,-0.4402f,  
	0.3709f,-0.8481f,-0.3784f,  0.3947f,-0.8299f,-0.3942f,  0.5025f,-0.8008f,-0.3259f,  0.5245f,-0.7805f,-0.3402f,  0.6226f,-0.7360f,-0.2659f,  0.6424f,-0.7140f,-0.2784f,  0.7284f,-0.6557f,-0.1988f,  0.7455f,-0.6329f,-0.2092f,  0.8172f,-0.5626f,-0.1252f,  0.8314f,-0.5395f,-0.1333f,  0.8870f,-0.4594f,-0.0460f,  0.8981f,-0.4367f,-0.0519f,  0.9364f,-0.3490f,0.0374f,  0.9443f,-0.3273f,0.0337f,  0.9643f,-0.2344f,0.1236f,  
	0.9692f,-0.2141f,0.1220f,  0.9703f,-0.1183f,0.2110f,  0.9723f,-0.1001f,0.2112f,  0.9546f,-0.0036f,0.2977f,  0.9540f,0.0123f,0.2996f,  0.9179f,0.1071f,0.3821f,  0.9149f,0.1204f,0.3853f,  0.8612f,0.2116f,0.4622f,  0.8562f,0.2221f,0.4665f,  0.7860f,0.3076f,0.5363f,  0.7795f,0.3153f,0.5413f,  0.6943f,0.3934f,0.6026f,  0.6866f,0.3984f,0.6081f,  0.5883f,0.4677f,0.6596f,  0.5798f,0.4701f,0.6655f,  
	0.4705f,0.5294f,0.7060f,  0.4614f,0.5293f,0.7120f,  0.3433f,0.5776f,0.7406f,  0.3341f,0.5754f,0.7465f,  0.2096f,0.6119f,0.7627f,  0.2005f,0.6078f,0.7683f,  0.0721f,0.6323f,0.7714f,  0.0635f,0.6266f,0.7768f,  -0.0664f,0.6387f,0.7666f,  -0.0744f,0.6318f,0.7715f,  -0.2030f,0.6318f,0.7481f,  -0.2104f,0.6240f,0.7526f,  -0.3354f,0.6121f,0.7161f,  -0.3419f,0.6036f,0.7202f,  -0.4609f,0.5805f,0.6712f,  
	-0.4665f,0.5717f,0.6749f,  -0.5773f,0.5381f,0.6141f,  -0.5821f,0.5292f,0.6174f,  -0.6826f,0.4860f,0.5457f,  -0.6865f,0.4772f,0.5486f,  -0.7749f,0.4255f,0.4673f,  -0.7780f,0.4171f,0.4698f,  -0.8528f,0.3581f,0.3802f,  -0.8551f,0.3502f,0.3824f,  -0.9148f,0.2852f,0.2860f,  -0.9165f,0.2778f,0.2879f,  -0.9602f,0.2082f,0.1862f,  -0.9613f,0.2015f,0.1879f,  -0.9882f,0.1287f,0.0827f,  -0.9889f,0.1228f,0.0842f,  
	-0.9986f,0.0481f,-0.0228f,  -0.9988f,0.0429f,-0.0214f,  -0.9912f,-0.0321f,-0.1283f,  -0.9912f,-0.0365f,-0.1271f,  -0.9664f,-0.1105f,-0.2321f,  -0.9662f,-0.1142f,-0.2310f,  -0.9247f,-0.1858f,-0.3323f,  -0.9244f,-0.1888f,-0.3314f,  -0.8669f,-0.2567f,-0.4273f,  -0.8666f,-0.2592f,-0.4265f,  -0.7942f,-0.3221f,-0.5153f,  -0.7938f,-0.3241f,-0.5146f,  -0.7077f,-0.3809f,-0.5950f,  -0.7075f,-0.3825f,-0.5943f,  -0.6092f,-0.4324f,-0.6648f,  
	-0.6090f,-0.4335f,-0.6642f,  -0.5003f,-0.4755f,-0.7236f,  -0.5002f,-0.4764f,-0.7231f,  -0.3829f,-0.5098f,-0.7704f,  -0.3828f,-0.5104f,-0.7700f,  -0.2589f,-0.5346f,-0.8044f,  -0.2589f,-0.5351f,-0.8041f,  -0.1306f,-0.5497f,-0.8251f,  -0.1306f,-0.5500f,-0.8249f,  -0.0001f,-0.5547f,-0.8320f,  -0.0001f,-0.5549f,-0.8319f,  -0.4346f,-0.7009f,-0.5655f,  -0.4167f,-0.7032f,-0.5762f,  -0.3097f,-0.7744f,-0.5517f,  -0.2887f,-0.7730f,-0.5649f,  
	-0.1742f,-0.8278f,-0.5333f,  -0.1512f,-0.8226f,-0.5482f,  -0.0321f,-0.8601f,-0.5091f,  -0.0080f,-0.8511f,-0.5250f,  0.1127f,-0.8709f,-0.4783f,  0.1371f,-0.8584f,-0.4944f,  0.2562f,-0.8606f,-0.4402f,  0.2801f,-0.8449f,-0.4558f,  0.3947f,-0.8299f,-0.3942f,  0.4174f,-0.8116f,-0.4087f,  0.5245f,-0.7805f,-0.3402f,  0.5454f,-0.7601f,-0.3533f,  0.6424f,-0.7140f,-0.2784f,  0.6610f,-0.6923f,-0.2896f,  0.7455f,-0.6329f,-0.2092f,  
	0.7615f,-0.6103f,-0.2183f,  0.8314f,-0.5395f,-0.1333f,  0.8445f,-0.5169f,-0.1403f,  0.8981f,-0.4367f,-0.0519f,  0.9082f,-0.4147f,-0.0568f,  0.9443f,-0.3273f,0.0337f,  0.9514f,-0.3064f,0.0308f,  0.9692f,-0.2141f,0.1220f,  0.9733f,-0.1949f,0.1210f,  0.9723f,-0.1001f,0.2112f,  0.9738f,-0.0828f,0.2119f,  0.9540f,0.0123f,0.2996f,  0.9530f,0.0272f,0.3018f,  0.9149f,0.1204f,0.3853f,  0.9118f,0.1328f,0.3887f,  
	0.8562f,0.2221f,0.4665f,  0.8513f,0.2318f,0.4708f,  0.7795f,0.3153f,0.5413f,  0.7731f,0.3223f,0.5463f,  0.6866f,0.3984f,0.6081f,  0.6792f,0.4028f,0.6135f,  0.5798f,0.4701f,0.6655f,  0.5717f,0.4721f,0.6711f,  0.4614f,0.5293f,0.7120f,  0.4529f,0.5290f,0.7176f,  0.3341f,0.5754f,0.7465f,  0.3255f,0.5730f,0.7521f,  0.2005f,0.6078f,0.7683f,  0.1922f,0.6037f,0.7737f,  0.0635f,0.6266f,0.7768f,  
	0.0555f,0.6210f,0.7818f,  -0.0744f,0.6318f,0.7715f,  -0.0817f,0.6251f,0.7762f,  -0.2104f,0.6240f,0.7526f,  -0.2170f,0.6164f,0.7569f,  -0.3419f,0.6036f,0.7202f,  -0.3478f,0.5955f,0.7242f,  -0.4665f,0.5717f,0.6749f,  -0.4716f,0.5633f,0.6784f,  -0.5821f,0.5292f,0.6174f,  -0.5863f,0.5208f,0.6205f,  -0.6865f,0.4772f,0.5486f,  -0.6899f,0.4690f,0.5514f,  -0.7780f,0.4171f,0.4698f,  -0.7807f,0.4092f,0.4723f,  
	-0.8551f,0.3502f,0.3824f,  -0.8571f,0.3428f,0.3846f,  -0.9165f,0.2778f,0.2879f,  -0.9179f,0.2710f,0.2899f,  -0.9613f,0.2015f,0.1879f,  -0.9622f,0.1953f,0.1897f,  -0.9889f,0.1228f,0.0842f,  -0.9894f,0.1172f,0.0858f,  -0.9988f,0.0429f,-0.0214f,  -0.9991f,0.0381f,-0.0200f,  -0.9912f,-0.0365f,-0.1271f,  -0.9912f,-0.0407f,-0.1258f,  -0.9662f,-0.1142f,-0.2310f,  -0.9661f,-0.1177f,-0.2298f,  -0.9244f,-0.1888f,-0.3314f,  
	-0.9242f,-0.1918f,-0.3303f,  -0.8666f,-0.2592f,-0.4265f,  -0.8664f,-0.2616f,-0.4254f,  -0.7938f,-0.3241f,-0.5146f,  -0.7936f,-0.3261f,-0.5136f,  -0.7075f,-0.3825f,-0.5943f,  -0.7073f,-0.3841f,-0.5934f,  -0.6090f,-0.4335f,-0.6642f,  -0.6089f,-0.4349f,-0.6634f,  -0.5002f,-0.4764f,-0.7231f,  -0.5001f,-0.4775f,-0.7224f,  -0.3828f,-0.5104f,-0.7700f,  -0.3828f,-0.5113f,-0.7694f,  -0.2589f,-0.5351f,-0.8041f,  -0.2589f,-0.5358f,-0.8037f,  
	-0.1306f,-0.5500f,-0.8249f,  -0.1306f,-0.5506f,-0.8245f,  -0.0001f,-0.5549f,-0.8319f,  -0.0000f,-0.5553f,-0.8316f,  -0.4167f,-0.7032f,-0.5762f,  -0.3982f,-0.7049f,-0.5870f,  -0.2887f,-0.7730f,-0.5649f,  -0.2677f,-0.7709f,-0.5780f,  -0.1512f,-0.8226f,-0.5482f,  -0.1285f,-0.8166f,-0.5627f,  -0.0080f,-0.8511f,-0.5250f,  0.0156f,-0.8414f,-0.5401f,  0.1371f,-0.8584f,-0.4944f,  0.1608f,-0.8453f,-0.5095f,  0.2801f,-0.8449f,-0.4558f,  
	0.3031f,-0.8289f,-0.4702f,  0.4174f,-0.8116f,-0.4087f,  0.4390f,-0.7932f,-0.4221f,  0.5454f,-0.7601f,-0.3533f,  0.5652f,-0.7398f,-0.3651f,  0.6610f,-0.6923f,-0.2896f,  0.6785f,-0.6707f,-0.2996f,  0.7615f,-0.6103f,-0.2183f,  0.7763f,-0.5883f,-0.2264f,  0.8445f,-0.5169f,-0.1403f,  0.8565f,-0.4949f,-0.1463f,  0.9082f,-0.4147f,-0.0568f,  0.9174f,-0.3933f,-0.0608f,  0.9514f,-0.3064f,0.0308f,  0.9577f,-0.2863f,0.0287f,  
	0.9733f,-0.1949f,0.1210f,  0.9769f,-0.1765f,0.1207f,  0.9738f,-0.0828f,0.2119f,  0.9748f,-0.0665f,0.2131f,  0.9530f,0.0272f,0.3018f,  0.9517f,0.0412f,0.3042f,  0.9118f,0.1328f,0.3887f,  0.9085f,0.1443f,0.3921f,  0.8513f,0.2318f,0.4708f,  0.8464f,0.2407f,0.4750f,  0.7731f,0.3223f,0.5463f,  0.7670f,0.3287f,0.5511f,  0.6792f,0.4028f,0.6135f,  0.6721f,0.4067f,0.6187f,  0.5717f,0.4721f,0.6711f,  
	0.5640f,0.4736f,0.6765f,  0.4529f,0.5290f,0.7176f,  0.4450f,0.5284f,0.7230f,  0.3255f,0.5730f,0.7521f,  0.3176f,0.5705f,0.7574f,  0.1922f,0.6037f,0.7737f,  0.1845f,0.5996f,0.7787f,  0.0555f,0.6210f,0.7818f,  0.0483f,0.6156f,0.7866f,  -0.0817f,0.6251f,0.7762f,  -0.0884f,0.6186f,0.7807f,  -0.2170f,0.6164f,0.7569f,  -0.2230f,0.6092f,0.7610f,  -0.3478f,0.5955f,0.7242f,  -0.3530f,0.5878f,0.7279f,  
	-0.4716f,0.5633f,0.6784f,  -0.4761f,0.5554f,0.6818f,  -0.5863f,0.5208f,0.6205f,  -0.5900f,0.5129f,0.6236f,  -0.6899f,0.4690f,0.5514f,  -0.6929f,0.4612f,0.5542f,  -0.7807f,0.4092f,0.4723f,  -0.7830f,0.4018f,0.4748f,  -0.8571f,0.3428f,0.3846f,  -0.8588f,0.3358f,0.3869f,  -0.9179f,0.2710f,0.2899f,  -0.9191f,0.2646f,0.2919f,  -0.9622f,0.1953f,0.1897f,  -0.9630f,0.1895f,0.1915f,  -0.9894f,0.1172f,0.0858f,  
	-0.9898f,0.1120f,0.0875f,  -0.9991f,0.0381f,-0.0200f,  -0.9993f,0.0335f,-0.0184f,  -0.9912f,-0.0407f,-0.1258f,  -0.9912f,-0.0447f,-0.1243f,  -0.9661f,-0.1177f,-0.2298f,  -0.9660f,-0.1212f,-0.2284f,  -0.9242f,-0.1918f,-0.3303f,  -0.9241f,-0.1947f,-0.3290f,  -0.8664f,-0.2616f,-0.4254f,  -0.8662f,-0.2641f,-0.4242f,  -0.7936f,-0.3261f,-0.5136f,  -0.7935f,-0.3282f,-0.5125f,  -0.7073f,-0.3841f,-0.5934f,  -0.7072f,-0.3859f,-0.5924f,  
	-0.6089f,-0.4349f,-0.6634f,  -0.6088f,-0.4364f,-0.6625f,  -0.5001f,-0.4775f,-0.7224f,  -0.5001f,-0.4788f,-0.7216f,  -0.3828f,-0.5113f,-0.7694f,  -0.3828f,-0.5125f,-0.7687f,  -0.2589f,-0.5358f,-0.8037f,  -0.2589f,-0.5368f,-0.8030f,  -0.1306f,-0.5506f,-0.8245f,  -0.1306f,-0.5515f,-0.8239f,  -0.0000f,-0.5553f,-0.8316f,  0.0000f,-0.5561f,-0.8311f,  -0.3982f,-0.7049f,-0.5870f,  -0.3796f,-0.7060f,-0.5978f,  -0.2677f,-0.7709f,-0.5780f,  
	-0.2468f,-0.7682f,-0.5907f,  -0.1285f,-0.8166f,-0.5627f,  -0.1061f,-0.8101f,-0.5766f,  0.0156f,-0.8414f,-0.5401f,  0.0386f,-0.8313f,-0.5545f,  0.1608f,-0.8453f,-0.5095f,  0.1836f,-0.8319f,-0.5237f,  0.3031f,-0.8289f,-0.4702f,  0.3252f,-0.8126f,-0.4837f,  0.4390f,-0.7932f,-0.4221f,  0.4597f,-0.7747f,-0.4343f,  0.5652f,-0.7398f,-0.3651f,  0.5840f,-0.7196f,-0.3757f,  0.6785f,-0.6707f,-0.2996f,  0.6949f,-0.6496f,-0.3085f,  
	0.7763f,-0.5883f,-0.2264f,  0.7902f,-0.5667f,-0.2333f,  0.8565f,-0.4949f,-0.1463f,  0.8676f,-0.4736f,-0.1514f,  0.9174f,-0.3933f,-0.0608f,  0.9257f,-0.3729f,-0.0640f,  0.9577f,-0.2863f,0.0287f,  0.9633f,-0.2671f,0.0273f,  0.9769f,-0.1765f,0.1207f,  0.9799f,-0.1590f,0.1208f,  0.9748f,-0.0665f,0.2131f,  0.9754f,-0.0511f,0.2146f,  0.9517f,0.0412f,0.3042f,  0.9502f,0.0543f,0.3068f,  0.9085f,0.1443f,0.3921f,  
	0.9052f,0.1549f,0.3957f,  0.8464f,0.2407f,0.4750f,  0.8417f,0.2489f,0.4792f,  0.7670f,0.3287f,0.5511f,  0.7611f,0.3344f,0.5558f,  0.6721f,0.4067f,0.6187f,  0.6654f,0.4101f,0.6237f,  0.5640f,0.4736f,0.6765f,  0.5568f,0.4748f,0.6816f,  0.4450f,0.5284f,0.7230f,  0.4377f,0.5276f,0.7281f,  0.3176f,0.5705f,0.7574f,  0.3103f,0.5679f,0.7623f,  0.1845f,0.5996f,0.7787f,  0.1775f,0.5955f,0.7835f,  
	0.0483f,0.6156f,0.7866f,  0.0417f,0.6103f,0.7911f,  -0.0884f,0.6186f,0.7807f,  -0.0945f,0.6124f,0.7849f,  -0.2230f,0.6092f,0.7610f,  -0.2284f,0.6023f,0.7649f,  -0.3530f,0.5878f,0.7279f,  -0.3577f,0.5806f,0.7314f,  -0.4761f,0.5554f,0.6818f,  -0.4801f,0.5479f,0.6851f,  -0.5900f,0.5129f,0.6236f,  -0.5933f,0.5054f,0.6265f,  -0.6929f,0.4612f,0.5542f,  -0.6955f,0.4540f,0.5569f,  -0.7830f,0.4018f,0.4748f,  
	-0.7850f,0.3949f,0.4773f,  -0.8588f,0.3358f,0.3869f,  -0.8603f,0.3293f,0.3892f,  -0.9191f,0.2646f,0.2919f,  -0.9202f,0.2586f,0.2940f,  -0.9630f,0.1895f,0.1915f,  -0.9637f,0.1841f,0.1935f,  -0.9898f,0.1120f,0.0875f,  -0.9902f,0.1072f,0.0893f,  -0.9993f,0.0335f,-0.0184f,  -0.9994f,0.0292f,-0.0167f,  -0.9912f,-0.0447f,-0.1243f,  -0.9913f,-0.0484f,-0.1227f,  -0.9660f,-0.1212f,-0.2284f,  -0.9659f,-0.1245f,-0.2269f,  
	-0.9241f,-0.1947f,-0.3290f,  -0.9240f,-0.1976f,-0.3275f,  -0.8662f,-0.2641f,-0.4242f,  -0.8661f,-0.2666f,-0.4228f,  -0.7935f,-0.3282f,-0.5125f,  -0.7934f,-0.3304f,-0.5112f,  -0.7072f,-0.3859f,-0.5924f,  -0.7072f,-0.3879f,-0.5912f,  -0.6088f,-0.4364f,-0.6625f,  -0.6088f,-0.4381f,-0.6613f,  -0.5001f,-0.4788f,-0.7216f,  -0.5001f,-0.4804f,-0.7205f,  -0.3828f,-0.5125f,-0.7687f,  -0.3827f,-0.5139f,-0.7677f,  -0.2589f,-0.5368f,-0.8030f,  
	-0.2589f,-0.5382f,-0.8021f,  -0.1306f,-0.5515f,-0.8239f,  -0.1305f,-0.5526f,-0.8232f,  0.0000f,-0.5561f,-0.8311f,  0.0002f,-0.5571f,-0.8305f,  -0.3796f,-0.7060f,-0.5978f,  -0.3608f,-0.7066f,-0.6087f,  -0.2468f,-0.7682f,-0.5907f,  -0.2260f,-0.7649f,-0.6032f,  -0.1061f,-0.8101f,-0.5766f,  -0.0841f,-0.8030f,-0.5900f,  0.0386f,-0.8313f,-0.5545f,  0.0610f,-0.8207f,-0.5681f,  0.1836f,-0.8319f,-0.5237f,  0.2057f,-0.8182f,-0.5369f,  
	0.3252f,-0.8126f,-0.4837f,  0.3463f,-0.7963f,-0.4960f,  0.4597f,-0.7747f,-0.4343f,  0.4794f,-0.7562f,-0.4454f,  0.5840f,-0.7196f,-0.3757f,  0.6016f,-0.6997f,-0.3853f,  0.6949f,-0.6496f,-0.3085f,  0.7103f,-0.6288f,-0.3164f,  0.7902f,-0.5667f,-0.2333f,  0.8030f,-0.5457f,-0.2394f,  0.8676f,-0.4736f,-0.1514f,  0.8778f,-0.4531f,-0.1557f,  0.9257f,-0.3729f,-0.0640f,  0.9332f,-0.3532f,-0.0665f,  0.9633f,-0.2671f,0.0273f,  
	0.9682f,-0.2488f,0.0263f,  0.9799f,-0.1590f,0.1208f,  0.9823f,-0.1425f,0.1212f,  0.9754f,-0.0511f,0.2146f,  0.9757f,-0.0366f,0.2162f,  0.9502f,0.0543f,0.3068f,  0.9486f,0.0665f,0.3095f,  0.9052f,0.1549f,0.3957f,  0.9020f,0.1648f,0.3991f,  0.8417f,0.2489f,0.4792f,  0.8371f,0.2564f,0.4833f,  0.7611f,0.3344f,0.5558f,  0.7555f,0.3396f,0.5603f,  0.6654f,0.4101f,0.6237f,  0.6592f,0.4130f,0.6284f,  
	0.5568f,0.4748f,0.6816f,  0.5502f,0.4756f,0.6864f,  0.4377f,0.5276f,0.7281f,  0.4309f,0.5265f,0.7329f,  0.3103f,0.5679f,0.7623f,  0.3036f,0.5653f,0.7670f,  0.1775f,0.5955f,0.7835f,  0.1711f,0.5915f,0.7879f,  0.0417f,0.6103f,0.7911f,  0.0357f,0.6052f,0.7953f,  -0.0945f,0.6124f,0.7849f,  -0.0999f,0.6065f,0.7888f,  -0.2284f,0.6023f,0.7649f,  -0.2332f,0.5958f,0.7686f,  -0.3577f,0.5806f,0.7314f,  
	-0.3619f,0.5737f,0.7348f,  -0.4801f,0.5479f,0.6851f,  -0.4836f,0.5409f,0.6882f,  -0.5933f,0.5054f,0.6265f,  -0.5962f,0.4984f,0.6294f,  -0.6955f,0.4540f,0.5569f,  -0.6978f,0.4472f,0.5595f,  -0.7850f,0.3949f,0.4773f,  -0.7868f,0.3884f,0.4798f,  -0.8603f,0.3293f,0.3892f,  -0.8616f,0.3232f,0.3915f,  -0.9202f,0.2586f,0.2940f,  -0.9210f,0.2530f,0.2962f,  -0.9637f,0.1841f,0.1935f,  -0.9642f,0.1790f,0.1955f,  
	-0.9902f,0.1072f,0.0893f,  -0.9905f,0.1026f,0.0913f,  -0.9994f,0.0292f,-0.0167f,  -0.9996f,0.0251f,-0.0148f,  -0.9913f,-0.0484f,-0.1227f,  -0.9913f,-0.0521f,-0.1209f,  -0.9659f,-0.1245f,-0.2269f,  -0.9659f,-0.1277f,-0.2252f,  -0.9240f,-0.1976f,-0.3275f,  -0.9239f,-0.2004f,-0.3259f,  -0.8661f,-0.2666f,-0.4228f,  -0.8661f,-0.2691f,-0.4213f,  -0.7934f,-0.3304f,-0.5112f,  -0.7934f,-0.3327f,-0.5098f,  -0.7072f,-0.3879f,-0.5912f,  
	-0.7071f,-0.3900f,-0.5898f,  -0.6088f,-0.4381f,-0.6613f,  -0.6088f,-0.4401f,-0.6601f,  -0.5001f,-0.4804f,-0.7205f,  -0.5001f,-0.4822f,-0.7193f,  -0.3827f,-0.5139f,-0.7677f,  -0.3827f,-0.5156f,-0.7666f,  -0.2589f,-0.5382f,-0.8021f,  -0.2587f,-0.5397f,-0.8011f,  -0.1305f,-0.5526f,-0.8232f,  -0.1303f,-0.5541f,-0.8222f,  0.0002f,-0.5571f,-0.8305f,  0.0005f,-0.5584f,-0.8296f,  -0.3608f,-0.7066f,-0.6087f,  -0.3420f,-0.7065f,-0.6195f,  
	-0.2260f,-0.7649f,-0.6032f,  -0.2054f,-0.7610f,-0.6154f,  -0.0841f,-0.8030f,-0.5900f,  -0.0626f,-0.7954f,-0.6028f,  0.0610f,-0.8207f,-0.5681f,  0.0828f,-0.8097f,-0.5810f,  0.2057f,-0.8182f,-0.5369f,  0.2270f,-0.8042f,-0.5493f,  0.3463f,-0.7963f,-0.4960f,  0.3666f,-0.7798f,-0.5075f,  0.4794f,-0.7562f,-0.4454f,  0.4980f,-0.7379f,-0.4556f,  0.6016f,-0.6997f,-0.3853f,  0.6182f,-0.6801f,-0.3940f,  0.7103f,-0.6288f,-0.3164f,  
	0.7247f,-0.6085f,-0.3233f,  0.8030f,-0.5457f,-0.2394f,  0.8149f,-0.5255f,-0.2447f,  0.8778f,-0.4531f,-0.1557f,  0.8871f,-0.4332f,-0.1593f,  0.9332f,-0.3532f,-0.0665f,  0.9399f,-0.3344f,-0.0685f,  0.9682f,-0.2488f,0.0263f,  0.9725f,-0.2315f,0.0258f,  0.9823f,-0.1425f,0.1212f,  0.9844f,-0.1269f,0.1219f,  0.9757f,-0.0366f,0.2162f,  0.9757f,-0.0230f,0.2181f,  0.9486f,0.0665f,0.3095f,  0.9468f,0.0779f,0.3122f,  
	0.9020f,0.1648f,0.3991f,  0.8987f,0.1739f,0.4026f,  0.8371f,0.2564f,0.4833f,  0.8326f,0.2632f,0.4873f,  0.7555f,0.3396f,0.5603f,  0.7502f,0.3442f,0.5646f,  0.6592f,0.4130f,0.6284f,  0.6533f,0.4155f,0.6329f,  0.5502f,0.4756f,0.6864f,  0.5440f,0.4762f,0.6909f,  0.4309f,0.5265f,0.7329f,  0.4246f,0.5254f,0.7373f,  0.3036f,0.5653f,0.7670f,  0.2975f,0.5626f,0.7713f,  0.1711f,0.5915f,0.7879f,  
	0.1653f,0.5876f,0.7921f,  0.0357f,0.6052f,0.7953f,  0.0303f,0.6002f,0.7992f,  -0.0999f,0.6065f,0.7888f,  -0.1048f,0.6008f,0.7925f,  -0.2332f,0.5958f,0.7686f,  -0.2375f,0.5896f,0.7720f,  -0.3619f,0.5737f,0.7348f,  -0.3656f,0.5671f,0.7380f,  -0.4836f,0.5409f,0.6882f,  -0.4866f,0.5343f,0.6912f,  -0.5962f,0.4984f,0.6294f,  -0.5987f,0.4918f,0.6322f,  -0.6978f,0.4472f,0.5595f,  -0.6998f,0.4408f,0.5622f,  
	-0.7868f,0.3884f,0.4798f,  -0.7882f,0.3823f,0.4822f,  -0.8616f,0.3232f,0.3915f,  -0.8626f,0.3175f,0.3938f,  -0.9210f,0.2530f,0.2962f,  -0.9217f,0.2477f,0.2984f,  -0.9642f,0.1790f,0.1955f,  -0.9647f,0.1742f,0.1976f,  -0.9905f,0.1026f,0.0913f,  -0.9908f,0.0982f,0.0933f,  -0.9996f,0.0251f,-0.0148f,  -0.9997f,0.0212f,-0.0129f,  -0.9913f,-0.0521f,-0.1209f,  -0.9913f,-0.0556f,-0.1191f,  -0.9659f,-0.1277f,-0.2252f,  
	-0.9659f,-0.1309f,-0.2234f,  -0.9239f,-0.2004f,-0.3259f,  -0.9239f,-0.2033f,-0.3242f,  -0.8661f,-0.2691f,-0.4213f,  -0.8660f,-0.2718f,-0.4197f,  -0.7934f,-0.3327f,-0.5098f,  -0.7934f,-0.3351f,-0.5082f,  -0.7071f,-0.3900f,-0.5898f,  -0.7071f,-0.3922f,-0.5883f,  -0.6088f,-0.4401f,-0.6601f,  -0.6088f,-0.4423f,-0.6586f,  -0.5001f,-0.4822f,-0.7193f,  -0.5000f,-0.4843f,-0.7179f,  -0.3827f,-0.5156f,-0.7666f,  -0.3826f,-0.5176f,-0.7653f,  
	-0.2587f,-0.5397f,-0.8011f,  -0.2586f,-0.5416f,-0.7999f,  -0.1303f,-0.5541f,-0.8222f,  -0.1300f,-0.5559f,-0.8211f,  0.0005f,-0.5584f,-0.8296f,  0.0010f,-0.5600f,-0.8285f,  -0.3420f,-0.7065f,-0.6195f,  -0.3232f,-0.7059f,-0.6303f,  -0.2054f,-0.7610f,-0.6154f,  -0.1852f,-0.7565f,-0.6272f,  -0.0626f,-0.7954f,-0.6028f,  -0.0416f,-0.7873f,-0.6151f,  0.0828f,-0.8097f,-0.5810f,  0.1038f,-0.7984f,-0.5931f,  0.2270f,-0.8042f,-0.5493f,  
	0.2475f,-0.7901f,-0.5608f,  0.3666f,-0.7798f,-0.5075f,  0.3858f,-0.7634f,-0.5180f,  0.4980f,-0.7379f,-0.4556f,  0.5156f,-0.7198f,-0.4648f,  0.6182f,-0.6801f,-0.3940f,  0.6339f,-0.6609f,-0.4017f,  0.7247f,-0.6085f,-0.3233f,  0.7380f,-0.5889f,-0.3295f,  0.8149f,-0.5255f,-0.2447f,  0.8258f,-0.5058f,-0.2492f,  0.8871f,-0.4332f,-0.1593f,  0.8956f,-0.4142f,-0.1623f,  0.9399f,-0.3344f,-0.0685f,  0.9460f,-0.3165f,-0.0701f,  
	0.9725f,-0.2315f,0.0258f,  0.9763f,-0.2150f,0.0256f,  0.9844f,-0.1269f,0.1219f,  0.9861f,-0.1121f,0.1229f,  0.9757f,-0.0230f,0.2181f,  0.9755f,-0.0103f,0.2200f,  0.9468f,0.0779f,0.3122f,  0.9450f,0.0885f,0.3149f,  0.8987f,0.1739f,0.4026f,  0.8955f,0.1824f,0.4059f,  0.8326f,0.2632f,0.4873f,  0.8284f,0.2694f,0.4910f,  0.7502f,0.3442f,0.5646f,  0.7453f,0.3483f,0.5686f,  0.6533f,0.4155f,0.6329f,  
	0.6479f,0.4176f,0.6371f,  0.5440f,0.4762f,0.6909f,  0.5383f,0.4765f,0.6951f,  0.4246f,0.5254f,0.7373f,  0.4189f,0.5241f,0.7415f,  0.2975f,0.5626f,0.7713f,  0.2920f,0.5599f,0.7754f,  0.1653f,0.5876f,0.7921f,  0.1600f,0.5838f,0.7960f,  0.0303f,0.6002f,0.7992f,  0.0255f,0.5955f,0.8029f,  -0.1048f,0.6008f,0.7925f,  -0.1091f,0.5954f,0.7960f,  -0.2375f,0.5896f,0.7720f,  -0.2413f,0.5837f,0.7753f,  
	-0.3656f,0.5671f,0.7380f,  -0.3688f,0.5610f,0.7411f,  -0.4866f,0.5343f,0.6912f,  -0.4893f,0.5281f,0.6941f,  -0.5987f,0.4918f,0.6322f,  -0.6008f,0.4857f,0.6349f,  -0.6998f,0.4408f,0.5622f,  -0.7014f,0.4349f,0.5647f,  -0.7882f,0.3823f,0.4822f,  -0.7895f,0.3767f,0.4846f,  -0.8626f,0.3175f,0.3938f,  -0.8635f,0.3122f,0.3961f,  -0.9217f,0.2477f,0.2984f,  -0.9223f,0.2428f,0.3006f,  -0.9647f,0.1742f,0.1976f,  
	-0.9650f,0.1696f,0.1998f,  -0.9908f,0.0982f,0.0933f,  -0.9910f,0.0941f,0.0953f,  -0.9997f,0.0212f,-0.0129f,  -0.9998f,0.0174f,-0.0109f,  -0.9913f,-0.0556f,-0.1191f,  -0.9914f,-0.0591f,-0.1171f,  -0.9659f,-0.1309f,-0.2234f,  -0.9659f,-0.1340f,-0.2215f,  -0.9239f,-0.2033f,-0.3242f,  -0.9239f,-0.2062f,-0.3223f,  -0.8660f,-0.2718f,-0.4197f,  -0.8660f,-0.2745f,-0.4179f,  -0.7934f,-0.3351f,-0.5082f,  -0.7934f,-0.3377f,-0.5065f,  
	-0.7071f,-0.3922f,-0.5883f,  -0.7071f,-0.3947f,-0.5866f,  -0.6088f,-0.4423f,-0.6586f,  -0.6088f,-0.4447f,-0.6570f,  -0.5000f,-0.4843f,-0.7179f,  -0.4999f,-0.4866f,-0.7164f,  -0.3826f,-0.5176f,-0.7653f,  -0.3824f,-0.5199f,-0.7638f,  -0.2586f,-0.5416f,-0.7999f,  -0.2582f,-0.5438f,-0.7985f,  -0.1300f,-0.5559f,-0.8211f,  -0.1295f,-0.5579f,-0.8197f,  0.0010f,-0.5600f,-0.8285f,  0.0016f,-0.5618f,-0.8272f,  -0.3232f,-0.7059f,-0.6303f,  
	-0.3045f,-0.7046f,-0.6409f,  -0.1852f,-0.7565f,-0.6272f,  -0.1652f,-0.7515f,-0.6387f,  -0.0416f,-0.7873f,-0.6151f,  -0.0211f,-0.7789f,-0.6268f,  0.1038f,-0.7984f,-0.5931f,  0.1242f,-0.7868f,-0.6046f,  0.2475f,-0.7901f,-0.5608f,  0.2670f,-0.7759f,-0.5715f,  0.3858f,-0.7634f,-0.5180f,  0.4041f,-0.7472f,-0.5277f,  0.5156f,-0.7198f,-0.4648f,  0.5322f,-0.7020f,-0.4732f,  0.6339f,-0.6609f,-0.4017f,  0.6485f,-0.6422f,-0.4087f,  
	0.7380f,-0.5889f,-0.3295f,  0.7504f,-0.5698f,-0.3349f,  0.8258f,-0.5058f,-0.2492f,  0.8359f,-0.4870f,-0.2532f,  0.8956f,-0.4142f,-0.1623f,  0.9033f,-0.3960f,-0.1648f,  0.9460f,-0.3165f,-0.0701f,  0.9515f,-0.2994f,-0.0713f,  0.9763f,-0.2150f,0.0256f,  0.9796f,-0.1994f,0.0256f,  0.9861f,-0.1121f,0.1229f,  0.9874f,-0.0983f,0.1239f,  0.9755f,-0.0103f,0.2200f,  0.9751f,0.0016f,0.2219f,  0.9450f,0.0885f,0.3149f,  
	0.9431f,0.0984f,0.3176f,  0.8955f,0.1824f,0.4059f,  0.8925f,0.1901f,0.4091f,  0.8284f,0.2694f,0.4910f,  0.8245f,0.2751f,0.4945f,  0.7453f,0.3483f,0.5686f,  0.7406f,0.3519f,0.5724f,  0.6479f,0.4176f,0.6371f,  0.6428f,0.4194f,0.6410f,  0.5383f,0.4765f,0.6951f,  0.5331f,0.4766f,0.6990f,  0.4189f,0.5241f,0.7415f,  0.4137f,0.5227f,0.7454f,  0.2920f,0.5599f,0.7754f,  0.2870f,0.5573f,0.7791f,  
	0.1600f,0.5838f,0.7960f,  0.1553f,0.5801f,0.7996f,  0.0255f,0.5955f,0.8029f,  0.0212f,0.5910f,0.8064f,  -0.1091f,0.5954f,0.7960f,  -0.1129f,0.5903f,0.7993f,  -0.2413f,0.5837f,0.7753f,  -0.2446f,0.5782f,0.7784f,  -0.3688f,0.5610f,0.7411f,  -0.3715f,0.5553f,0.7440f,  -0.4893f,0.5281f,0.6941f,  -0.4916f,0.5223f,0.6968f,  -0.6008f,0.4857f,0.6349f,  -0.6026f,0.4800f,0.6375f,  -0.7014f,0.4349f,0.5647f,  
	-0.7028f,0.4293f,0.5672f,  -0.7895f,0.3767f,0.4846f,  -0.7905f,0.3714f,0.4871f,  -0.8635f,0.3122f,0.3961f,  -0.8642f,0.3073f,0.3984f,  -0.9223f,0.2428f,0.3006f,  -0.9228f,0.2382f,0.3028f,  -0.9650f,0.1696f,0.1998f,  -0.9653f,0.1654f,0.2019f,  -0.9910f,0.0941f,0.0953f,  -0.9911f,0.0901f,0.0975f,  -0.9998f,0.0174f,-0.0109f,  -0.9999f,0.0138f,-0.0088f,  -0.9914f,-0.0591f,-0.1171f,  -0.9914f,-0.0624f,-0.1151f,  
	-0.9659f,-0.1340f,-0.2215f,  -0.9659f,-0.1372f,-0.2195f,  -0.9239f,-0.2062f,-0.3223f,  -0.9239f,-0.2092f,-0.3204f,  -0.8660f,-0.2745f,-0.4179f,  -0.8660f,-0.2774f,-0.4160f,  -0.7934f,-0.3377f,-0.5065f,  -0.7934f,-0.3405f,-0.5046f,  -0.7071f,-0.3947f,-0.5866f,  -0.7071f,-0.3974f,-0.5849f,  -0.6088f,-0.4447f,-0.6570f,  -0.6087f,-0.4473f,-0.6553f,  -0.4999f,-0.4866f,-0.7164f,  -0.4997f,-0.4893f,-0.7148f,  -0.3824f,-0.5199f,-0.7638f,  
	-0.3821f,-0.5225f,-0.7622f,  -0.2582f,-0.5438f,-0.7985f,  -0.2578f,-0.5463f,-0.7969f,  -0.1295f,-0.5579f,-0.8197f,  -0.1288f,-0.5603f,-0.8182f,  0.0016f,-0.5618f,-0.8272f,  0.0025f,-0.5640f,-0.8258f,  -0.3045f,-0.7046f,-0.6409f,  -0.2860f,-0.7028f,-0.6514f,  -0.1652f,-0.7515f,-0.6387f,  -0.1457f,-0.7460f,-0.6498f,  -0.0211f,-0.7789f,-0.6268f,  -0.0012f,-0.7701f,-0.6380f,  0.1242f,-0.7868f,-0.6046f,  0.1437f,-0.7751f,-0.6153f,  
	0.2670f,-0.7759f,-0.5715f,  0.2857f,-0.7617f,-0.5815f,  0.4041f,-0.7472f,-0.5277f,  0.4215f,-0.7310f,-0.5366f,  0.5322f,-0.7020f,-0.4732f,  0.5478f,-0.6846f,-0.4808f,  0.6485f,-0.6422f,-0.4087f,  0.6622f,-0.6240f,-0.4149f,  0.7504f,-0.5698f,-0.3349f,  0.7619f,-0.5514f,-0.3398f,  0.8359f,-0.4870f,-0.2532f,  0.8452f,-0.4689f,-0.2566f,  0.9033f,-0.3960f,-0.1648f,  0.9104f,-0.3787f,-0.1669f,  0.9515f,-0.2994f,-0.0713f,  
	0.9563f,-0.2832f,-0.0722f,  0.9796f,-0.1994f,0.0256f,  0.9825f,-0.1847f,0.0258f,  0.9874f,-0.0983f,0.1239f,  0.9885f,-0.0853f,0.1251f,  0.9751f,0.0016f,0.2219f,  0.9745f,0.0128f,0.2238f,  0.9431f,0.0984f,0.3176f,  0.9413f,0.1075f,0.3201f,  0.8925f,0.1901f,0.4091f,  0.8896f,0.1972f,0.4120f,  0.8245f,0.2751f,0.4945f,  0.8207f,0.2802f,0.4978f,  0.7406f,0.3519f,0.5724f,  0.7364f,0.3552f,0.5759f,  
	0.6428f,0.4194f,0.6410f,  0.6382f,0.4210f,0.6446f,  0.5331f,0.4766f,0.6990f,  0.5284f,0.4765f,0.7026f,  0.4137f,0.5227f,0.7454f,  0.4091f,0.5213f,0.7490f,  0.2870f,0.5573f,0.7791f,  0.2825f,0.5547f,0.7826f,  0.1553f,0.5801f,0.7996f,  0.1512f,0.5765f,0.8030f,  0.0212f,0.5910f,0.8064f,  0.0175f,0.5867f,0.8096f,  -0.1129f,0.5903f,0.7993f,  -0.1162f,0.5855f,0.8023f,  -0.2446f,0.5782f,0.7784f,  
	-0.2474f,0.5730f,0.7813f,  -0.3715f,0.5553f,0.7440f,  -0.3739f,0.5500f,0.7468f,  -0.4916f,0.5223f,0.6968f,  -0.4935f,0.5169f,0.6995f,  -0.6026f,0.4800f,0.6375f,  -0.6041f,0.4747f,0.6401f,  -0.7028f,0.4293f,0.5672f,  -0.7039f,0.4242f,0.5697f,  -0.7905f,0.3714f,0.4871f,  -0.7913f,0.3665f,0.4894f,  -0.8642f,0.3073f,0.3984f,  -0.8648f,0.3026f,0.4007f,  -0.9228f,0.2382f,0.3028f,  -0.9232f,0.2338f,0.3051f,  
	-0.9653f,0.1654f,0.2019f,  -0.9656f,0.1613f,0.2042f,  -0.9911f,0.0901f,0.0975f,  -0.9913f,0.0864f,0.0997f,  -0.9999f,0.0138f,-0.0088f,  -0.9999f,0.0102f,-0.0067f,  -0.9914f,-0.0624f,-0.1151f,  -0.9914f,-0.0658f,-0.1130f,  -0.9659f,-0.1372f,-0.2195f,  -0.9659f,-0.1403f,-0.2175f,  -0.9239f,-0.2092f,-0.3204f,  -0.9239f,-0.2123f,-0.3184f,  -0.8660f,-0.2774f,-0.4160f,  -0.8660f,-0.2803f,-0.4140f,  -0.7934f,-0.3405f,-0.5046f,  
	-0.7933f,-0.3434f,-0.5027f,  -0.7071f,-0.3974f,-0.5849f,  -0.7070f,-0.4004f,-0.5830f,  -0.6087f,-0.4473f,-0.6553f,  -0.6085f,-0.4503f,-0.6535f,  -0.4997f,-0.4893f,-0.7148f,  -0.4994f,-0.4922f,-0.7130f,  -0.3821f,-0.5225f,-0.7622f,  -0.3816f,-0.5254f,-0.7605f,  -0.2578f,-0.5463f,-0.7969f,  -0.2571f,-0.5492f,-0.7952f,  -0.1288f,-0.5603f,-0.8182f,  -0.1280f,-0.5630f,-0.8165f,  0.0025f,-0.5640f,-0.8258f,  0.0036f,-0.5664f,-0.8241f,  
	-0.2860f,-0.7028f,-0.6514f,  -0.2678f,-0.7004f,-0.6616f,  -0.1457f,-0.7460f,-0.6498f,  -0.1266f,-0.7401f,-0.6604f,  -0.0012f,-0.7701f,-0.6380f,  0.0179f,-0.7610f,-0.6485f,  0.1437f,-0.7751f,-0.6153f,  0.1624f,-0.7632f,-0.6254f,  0.2857f,-0.7617f,-0.5815f,  0.3035f,-0.7476f,-0.5908f,  0.4215f,-0.7310f,-0.5366f,  0.4378f,-0.7152f,-0.5448f,  0.5478f,-0.6846f,-0.4808f,  0.5625f,-0.6675f,-0.4878f,  0.6622f,-0.6240f,-0.4149f,  
	0.6749f,-0.6063f,-0.4206f,  0.7619f,-0.5514f,-0.3398f,  0.7725f,-0.5336f,-0.3441f,  0.8452f,-0.4689f,-0.2566f,  0.8537f,-0.4515f,-0.2596f,  0.9104f,-0.3787f,-0.1669f,  0.9167f,-0.3621f,-0.1687f,  0.9563f,-0.2832f,-0.0722f,  0.9607f,-0.2678f,-0.0728f,  0.9825f,-0.1847f,0.0258f,  0.9850f,-0.1708f,0.0261f,  0.9885f,-0.0853f,0.1251f,  0.9893f,-0.0731f,0.1262f,  0.9745f,0.0128f,0.2238f,  0.9739f,0.0231f,0.2257f,  
	0.9413f,0.1075f,0.3201f,  0.9394f,0.1160f,0.3225f,  0.8896f,0.1972f,0.4120f,  0.8868f,0.2037f,0.4148f,  0.8207f,0.2802f,0.4978f,  0.8173f,0.2849f,0.5009f,  0.7364f,0.3552f,0.5759f,  0.7324f,0.3581f,0.5791f,  0.6382f,0.4210f,0.6446f,  0.6340f,0.4222f,0.6479f,  0.5284f,0.4765f,0.7026f,  0.5241f,0.4764f,0.7059f,  0.4091f,0.5213f,0.7490f,  0.4049f,0.5199f,0.7522f,  0.2825f,0.5547f,0.7826f,  
	0.2785f,0.5522f,0.7858f,  0.1512f,0.5765f,0.8030f,  0.1475f,0.5732f,0.8061f,  0.0175f,0.5867f,0.8096f,  0.0142f,0.5827f,0.8126f,  -0.1162f,0.5855f,0.8023f,  -0.1191f,0.5809f,0.8052f,  -0.2474f,0.5730f,0.7813f,  -0.2499f,0.5682f,0.7840f,  -0.3739f,0.5500f,0.7468f,  -0.3759f,0.5450f,0.7495f,  -0.4935f,0.5169f,0.6995f,  -0.4951f,0.5119f,0.7020f,  -0.6041f,0.4747f,0.6401f,  -0.6053f,0.4697f,0.6426f,  
	-0.7039f,0.4242f,0.5697f,  -0.7048f,0.4194f,0.5721f,  -0.7913f,0.3665f,0.4894f,  -0.7919f,0.3619f,0.4918f,  -0.8648f,0.3026f,0.4007f,  -0.8652f,0.2983f,0.4030f,  -0.9232f,0.2338f,0.3051f,  -0.9234f,0.2297f,0.3074f,  -0.9656f,0.1613f,0.2042f,  -0.9657f,0.1575f,0.2064f,  -0.9913f,0.0864f,0.0997f,  -0.9914f,0.0827f,0.1019f,  -0.9999f,0.0102f,-0.0067f,  -1.0000f,0.0068f,-0.0045f,  -0.9914f,-0.0658f,-0.1130f,  
	-0.9914f,-0.0691f,-0.1108f,  -0.9659f,-0.1403f,-0.2175f,  -0.9659f,-0.1436f,-0.2153f,  -0.9239f,-0.2123f,-0.3184f,  -0.9239f,-0.2154f,-0.3163f,  -0.8660f,-0.2803f,-0.4140f,  -0.8660f,-0.2835f,-0.4119f,  -0.7933f,-0.3434f,-0.5027f,  -0.7932f,-0.3466f,-0.5007f,  -0.7070f,-0.4004f,-0.5830f,  -0.7068f,-0.4036f,-0.5810f,  -0.6085f,-0.4503f,-0.6535f,  -0.6082f,-0.4535f,-0.6515f,  -0.4994f,-0.4922f,-0.7130f,  -0.4989f,-0.4955f,-0.7110f,  
	-0.3816f,-0.5254f,-0.7605f,  -0.3810f,-0.5286f,-0.7586f,  -0.2571f,-0.5492f,-0.7952f,  -0.2562f,-0.5523f,-0.7933f,  -0.1280f,-0.5630f,-0.8165f,  -0.1268f,-0.5659f,-0.8146f,  0.0036f,-0.5664f,-0.8241f,  0.0050f,-0.5691f,-0.8222f,  -0.2678f,-0.7004f,-0.6616f,  -0.2499f,-0.6975f,-0.6716f,  -0.1266f,-0.7401f,-0.6604f,  -0.1082f,-0.7338f,-0.6706f,  0.0179f,-0.7610f,-0.6485f,  0.0363f,-0.7517f,-0.6586f,  0.1624f,-0.7632f,-0.6254f,  
	0.1803f,-0.7513f,-0.6349f,  0.3035f,-0.7476f,-0.5908f,  0.3203f,-0.7336f,-0.5994f,  0.4378f,-0.7152f,-0.5448f,  0.4533f,-0.6996f,-0.5523f,  0.5625f,-0.6675f,-0.4878f,  0.5763f,-0.6509f,-0.4942f,  0.6749f,-0.6063f,-0.4206f,  0.6867f,-0.5893f,-0.4257f,  0.7725f,-0.5336f,-0.3441f,  0.7824f,-0.5166f,-0.3480f,  0.8537f,-0.4515f,-0.2596f,  0.8614f,-0.4349f,-0.2623f,  0.9167f,-0.3621f,-0.1687f,  0.9225f,-0.3465f,-0.1702f,  
	0.9607f,-0.2678f,-0.0728f,  0.9646f,-0.2533f,-0.0733f,  0.9850f,-0.1708f,0.0261f,  0.9871f,-0.1577f,0.0265f,  0.9893f,-0.0731f,0.1262f,  0.9899f,-0.0617f,0.1274f,  0.9739f,0.0231f,0.2257f,  0.9733f,0.0327f,0.2274f,  0.9394f,0.1160f,0.3225f,  0.9377f,0.1238f,0.3247f,  0.8868f,0.2037f,0.4148f,  0.8842f,0.2097f,0.4173f,  0.8173f,0.2849f,0.5009f,  0.8141f,0.2891f,0.5037f,  0.7324f,0.3581f,0.5791f,  
	0.7288f,0.3607f,0.5820f,  0.6340f,0.4222f,0.6479f,  0.6302f,0.4233f,0.6509f,  0.5241f,0.4764f,0.7059f,  0.5203f,0.4761f,0.7090f,  0.4049f,0.5199f,0.7522f,  0.4011f,0.5184f,0.7552f,  0.2785f,0.5522f,0.7858f,  0.2750f,0.5498f,0.7887f,  0.1475f,0.5732f,0.8061f,  0.1443f,0.5700f,0.8089f,  0.0142f,0.5827f,0.8126f,  0.0113f,0.5789f,0.8153f,  -0.1191f,0.5809f,0.8052f,  -0.1215f,0.5767f,0.8078f,  
	-0.2499f,0.5682f,0.7840f,  -0.2519f,0.5637f,0.7866f,  -0.3759f,0.5450f,0.7495f,  -0.3776f,0.5404f,0.7519f,  -0.4951f,0.5119f,0.7020f,  -0.4964f,0.5073f,0.7045f,  -0.6053f,0.4697f,0.6426f,  -0.6063f,0.4652f,0.6449f,  -0.7048f,0.4194f,0.5721f,  -0.7056f,0.4150f,0.5744f,  -0.7919f,0.3619f,0.4918f,  -0.7924f,0.3576f,0.4941f,  -0.8652f,0.2983f,0.4030f,  -0.8655f,0.2942f,0.4053f,  -0.9234f,0.2297f,0.3074f,  
	-0.9236f,0.2259f,0.3096f,  -0.9657f,0.1575f,0.2064f,  -0.9658f,0.1538f,0.2087f,  -0.9914f,0.0827f,0.1019f,  -0.9914f,0.0792f,0.1041f,  -1.0000f,0.0068f,-0.0045f,  -1.0000f,0.0034f,-0.0022f,  -0.9914f,-0.0691f,-0.1108f,  -0.9914f,-0.0724f,-0.1086f,  -0.9659f,-0.1436f,-0.2153f,  -0.9659f,-0.1469f,-0.2131f,  -0.9239f,-0.2154f,-0.3163f,  -0.9238f,-0.2188f,-0.3141f,  -0.8660f,-0.2835f,-0.4119f,  -0.8659f,-0.2869f,-0.4098f,  
	-0.7932f,-0.3466f,-0.5007f,  -0.7931f,-0.3500f,-0.4985f,  -0.7068f,-0.4036f,-0.5810f,  -0.7065f,-0.4071f,-0.5789f,  -0.6082f,-0.4535f,-0.6515f,  -0.6077f,-0.4571f,-0.6494f,  -0.4989f,-0.4955f,-0.7110f,  -0.4983f,-0.4991f,-0.7089f,  -0.3810f,-0.5286f,-0.7586f,  -0.3801f,-0.5322f,-0.7565f,  -0.2562f,-0.5523f,-0.7933f,  -0.2551f,-0.5558f,-0.7912f,  -0.1268f,-0.5659f,-0.8146f,  -0.1254f,-0.5692f,-0.8126f,  0.0050f,-0.5691f,-0.8222f,  
	0.0067f,-0.5721f,-0.8201f,  -0.2499f,-0.6975f,-0.6716f,  -0.2325f,-0.6941f,-0.6813f,  -0.1082f,-0.7338f,-0.6706f,  -0.0903f,-0.7272f,-0.6804f,  0.0363f,-0.7517f,-0.6586f,  0.0541f,-0.7421f,-0.6681f,  0.1803f,-0.7513f,-0.6349f,  0.1973f,-0.7394f,-0.6437f,  0.3203f,-0.7336f,-0.5994f,  0.3363f,-0.7197f,-0.6074f,  0.4533f,-0.6996f,-0.5523f,  0.4678f,-0.6844f,-0.5593f,  0.5763f,-0.6509f,-0.4942f,  0.5891f,-0.6348f,-0.5000f,  
	0.6867f,-0.5893f,-0.4257f,  0.6976f,-0.5728f,-0.4303f,  0.7824f,-0.5166f,-0.3480f,  0.7914f,-0.5002f,-0.3514f,  0.8614f,-0.4349f,-0.2623f,  0.8685f,-0.4191f,-0.2646f,  0.9225f,-0.3465f,-0.1702f,  0.9277f,-0.3316f,-0.1715f,  0.9646f,-0.2533f,-0.0733f,  0.9681f,-0.2396f,-0.0737f,  0.9871f,-0.1577f,0.0265f,  0.9890f,-0.1455f,0.0269f,  0.9899f,-0.0617f,0.1274f,  0.9904f,-0.0511f,0.1284f,  0.9733f,0.0327f,0.2274f,  
	0.9725f,0.0417f,0.2290f,  0.9377f,0.1238f,0.3247f,  0.9360f,0.1310f,0.3267f,  0.8842f,0.2097f,0.4173f,  0.8818f,0.2152f,0.4196f,  0.8141f,0.2891f,0.5037f,  0.8111f,0.2930f,0.5062f,  0.7288f,0.3607f,0.5820f,  0.7256f,0.3630f,0.5846f,  0.6302f,0.4233f,0.6509f,  0.6268f,0.4242f,0.6536f,  0.5203f,0.4761f,0.7090f,  0.5169f,0.4758f,0.7117f,  0.4011f,0.5184f,0.7552f,  0.3978f,0.5170f,0.7579f,  
	0.2750f,0.5498f,0.7887f,  0.2719f,0.5475f,0.7914f,  0.1443f,0.5700f,0.8089f,  0.1415f,0.5670f,0.8115f,  0.0113f,0.5789f,0.8153f,  0.0089f,0.5754f,0.8178f,  -0.1215f,0.5767f,0.8078f,  -0.1236f,0.5728f,0.8103f,  -0.2519f,0.5637f,0.7866f,  -0.2536f,0.5596f,0.7890f,  -0.3776f,0.5404f,0.7519f,  -0.3790f,0.5361f,0.7543f,  -0.4964f,0.5073f,0.7045f,  -0.4975f,0.5030f,0.7068f,  -0.6063f,0.4652f,0.6449f,  
	-0.6071f,0.4610f,0.6472f,  -0.7056f,0.4150f,0.5744f,  -0.7061f,0.4109f,0.5767f,  -0.7924f,0.3576f,0.4941f,  -0.7928f,0.3537f,0.4963f,  -0.8655f,0.2942f,0.4053f,  -0.8658f,0.2904f,0.4076f,  -0.9236f,0.2259f,0.3096f,  -0.9238f,0.2222f,0.3119f,  -0.9658f,0.1538f,0.2087f,  -0.9659f,0.1503f,0.2109f,  -0.9914f,0.0792f,0.1041f,  -0.9914f,0.0758f,0.1064f,  -1.0000f,0.0034f,-0.0022f,  -1.0000f,0.0000f,-0.0000f,  
	-0.9914f,-0.0724f,-0.1086f,  -0.9914f,-0.0758f,-0.1064f,  -0.9659f,-0.1469f,-0.2131f,  -0.9659f,-0.1503f,-0.2109f,  -0.9238f,-0.2188f,-0.3141f,  -0.9238f,-0.2222f,-0.3119f,  -0.8659f,-0.2869f,-0.4098f,  -0.8658f,-0.2904f,-0.4076f,  -0.7931f,-0.3500f,-0.4985f,  -0.7928f,-0.3537f,-0.4963f,  -0.7065f,-0.4071f,-0.5789f,  -0.7061f,-0.4109f,-0.5767f,  -0.6077f,-0.4571f,-0.6494f,  -0.6072f,-0.4610f,-0.6472f,  -0.4983f,-0.4991f,-0.7089f,  
	-0.4975f,-0.5030f,-0.7068f,  -0.3801f,-0.5322f,-0.7565f,  -0.3790f,-0.5361f,-0.7543f,  -0.2551f,-0.5558f,-0.7912f,  -0.2537f,-0.5596f,-0.7890f,  -0.1254f,-0.5692f,-0.8126f,  -0.1237f,-0.5728f,-0.8103f,  0.0067f,-0.5721f,-0.8201f,  0.0088f,-0.5754f,-0.8178f,  -0.2325f,-0.6941f,-0.6813f,  -0.2155f,-0.6903f,-0.6907f,  -0.0903f,-0.7272f,-0.6804f,  -0.0731f,-0.7203f,-0.6898f,  0.0541f,-0.7421f,-0.6681f,  0.0710f,-0.7325f,-0.6770f,  
	0.1973f,-0.7394f,-0.6437f,  0.2135f,-0.7275f,-0.6520f,  0.3363f,-0.7197f,-0.6074f,  0.3513f,-0.7061f,-0.6148f,  0.4678f,-0.6844f,-0.5593f,  0.4814f,-0.6695f,-0.5657f,  0.5891f,-0.6348f,-0.5000f,  0.6010f,-0.6193f,-0.5053f,  0.6976f,-0.5728f,-0.4303f,  0.7078f,-0.5570f,-0.4345f,  0.7914f,-0.5002f,-0.3514f,  0.7996f,-0.4846f,-0.3546f,  0.8685f,-0.4191f,-0.2646f,  0.8749f,-0.4041f,-0.2668f,  0.9277f,-0.3316f,-0.1715f,  
	0.9324f,-0.3174f,-0.1727f,  0.9681f,-0.2396f,-0.0737f,  0.9711f,-0.2267f,-0.0741f,  0.9890f,-0.1455f,0.0269f,  0.9906f,-0.1340f,0.0272f,  0.9904f,-0.0511f,0.1284f,  0.9907f,-0.0412f,0.1293f,  0.9725f,0.0417f,0.2290f,  0.9718f,0.0500f,0.2304f,  0.9360f,0.1310f,0.3267f,  0.9344f,0.1376f,0.3284f,  0.8818f,0.2152f,0.4196f,  0.8796f,0.2203f,0.4217f,  0.8111f,0.2930f,0.5062f,  0.8085f,0.2965f,0.5084f,  
	0.7256f,0.3630f,0.5846f,  0.7226f,0.3651f,0.5870f,  0.6268f,0.4242f,0.6536f,  0.6237f,0.4250f,0.6560f,  0.5169f,0.4758f,0.7117f,  0.5138f,0.4754f,0.7141f,  0.3978f,0.5170f,0.7579f,  0.3949f,0.5157f,0.7603f,  0.2719f,0.5475f,0.7914f,  0.2693f,0.5454f,0.7938f,  0.1415f,0.5670f,0.8115f,  0.1391f,0.5642f,0.8138f,  0.0089f,0.5754f,0.8178f,  0.0068f,0.5721f,0.8201f,  -0.1236f,0.5728f,0.8103f,  
	-0.1254f,0.5692f,0.8126f,  -0.2536f,0.5596f,0.7890f,  -0.2551f,0.5558f,0.7912f,  -0.3790f,0.5361f,0.7543f,  -0.3801f,0.5322f,0.7565f,  -0.4975f,0.5030f,0.7068f,  -0.4983f,0.4991f,0.7090f,  -0.6071f,0.4610f,0.6472f,  -0.6077f,0.4571f,0.6494f,  -0.7061f,0.4109f,0.5767f,  -0.7065f,0.4071f,0.5789f,  -0.7928f,0.3537f,0.4963f,  -0.7931f,0.3500f,0.4985f,  -0.8658f,0.2904f,0.4076f,  -0.8659f,0.2869f,0.4098f,  
	-0.9238f,0.2222f,0.3119f,  -0.9238f,0.2188f,0.3141f,  -0.9659f,0.1503f,0.2109f,  -0.9659f,0.1469f,0.2131f,  -0.9914f,0.0758f,0.1064f,  -0.9914f,0.0724f,0.1086f,  -1.0000f,0.0000f,-0.0000f,  -1.0000f,-0.0034f,0.0022f,  -0.9914f,-0.0758f,-0.1064f,  -0.9914f,-0.0792f,-0.1041f,  -0.9659f,-0.1503f,-0.2109f,  -0.9658f,-0.1538f,-0.2087f,  -0.9238f,-0.2222f,-0.3119f,  -0.9236f,-0.2259f,-0.3096f,  -0.8658f,-0.2904f,-0.4076f,  
	-0.8655f,-0.2942f,-0.4053f,  -0.7928f,-0.3537f,-0.4963f,  -0.7925f,-0.3576f,-0.4941f,  -0.7061f,-0.4109f,-0.5767f,  -0.7056f,-0.4150f,-0.5744f,  -0.6072f,-0.4610f,-0.6472f,  -0.6064f,-0.4652f,-0.6449f,  -0.4975f,-0.5030f,-0.7068f,  -0.4965f,-0.5072f,-0.7044f,  -0.3790f,-0.5361f,-0.7543f,  -0.3777f,-0.5403f,-0.7519f,  -0.2537f,-0.5596f,-0.7890f,  -0.2520f,-0.5637f,-0.7866f,  -0.1237f,-0.5728f,-0.8103f,  -0.1216f,-0.5767f,-0.8078f,  
	0.0088f,-0.5754f,-0.8178f,  0.0112f,-0.5789f,-0.8153f,  -0.2155f,-0.6903f,-0.6907f,  -0.1990f,-0.6861f,-0.6998f,  -0.0731f,-0.7203f,-0.6898f,  -0.0566f,-0.7131f,-0.6988f,  0.0710f,-0.7325f,-0.6770f,  0.0871f,-0.7228f,-0.6855f,  0.2135f,-0.7275f,-0.6520f,  0.2288f,-0.7157f,-0.6598f,  0.3513f,-0.7061f,-0.6148f,  0.3654f,-0.6928f,-0.6217f,  0.4814f,-0.6695f,-0.5657f,  0.4940f,-0.6551f,-0.5717f,  0.6010f,-0.6193f,-0.5053f,  
	0.6120f,-0.6042f,-0.5102f,  0.7078f,-0.5570f,-0.4345f,  0.7171f,-0.5418f,-0.4384f,  0.7996f,-0.4846f,-0.3546f,  0.8072f,-0.4697f,-0.3575f,  0.8749f,-0.4041f,-0.2668f,  0.8808f,-0.3898f,-0.2687f,  0.9324f,-0.3174f,-0.1727f,  0.9366f,-0.3041f,-0.1738f,  0.9711f,-0.2267f,-0.0741f,  0.9739f,-0.2146f,-0.0744f,  0.9906f,-0.1340f,0.0272f,  0.9920f,-0.1233f,0.0275f,  0.9907f,-0.0412f,0.1293f,  0.9910f,-0.0319f,0.1301f,  
	0.9718f,0.0500f,0.2304f,  0.9711f,0.0577f,0.2316f,  0.9344f,0.1376f,0.3284f,  0.9330f,0.1438f,0.3300f,  0.8796f,0.2203f,0.4217f,  0.8775f,0.2249f,0.4235f,  0.8085f,0.2965f,0.5084f,  0.8061f,0.2997f,0.5104f,  0.7226f,0.3651f,0.5870f,  0.7200f,0.3669f,0.5891f,  0.6237f,0.4250f,0.6560f,  0.6211f,0.4257f,0.6581f,  0.5138f,0.4754f,0.7141f,  0.5112f,0.4750f,0.7163f,  0.3949f,0.5157f,0.7603f,  
	0.3924f,0.5144f,0.7625f,  0.2693f,0.5454f,0.7938f,  0.2670f,0.5434f,0.7959f,  0.1391f,0.5642f,0.8138f,  0.1371f,0.5617f,0.8159f,  0.0068f,0.5721f,0.8201f,  0.0051f,0.5692f,0.8222f,  -0.1254f,0.5692f,0.8126f,  -0.1268f,0.5660f,0.8146f,  -0.2551f,0.5558f,0.7912f,  -0.2562f,0.5523f,0.7933f,  -0.3801f,0.5322f,0.7565f,  -0.3809f,0.5286f,0.7586f,  -0.4983f,0.4991f,0.7090f,  -0.4989f,0.4955f,0.7110f,  
	-0.6077f,0.4571f,0.6494f,  -0.6082f,0.4536f,0.6515f,  -0.7065f,0.4071f,0.5789f,  -0.7068f,0.4036f,0.5810f,  -0.7931f,0.3500f,0.4985f,  -0.7932f,0.3466f,0.5007f,  -0.8659f,0.2869f,0.4098f,  -0.8660f,0.2835f,0.4119f,  -0.9238f,0.2188f,0.3141f,  -0.9239f,0.2155f,0.3163f,  -0.9659f,0.1469f,0.2131f,  -0.9659f,0.1436f,0.2153f,  -0.9914f,0.0724f,0.1086f,  -0.9914f,0.0691f,0.1108f,  -1.0000f,-0.0034f,0.0022f,  
	-1.0000f,-0.0068f,0.0045f,  -0.9914f,-0.0792f,-0.1041f,  -0.9914f,-0.0827f,-0.1019f,  -0.9658f,-0.1538f,-0.2087f,  -0.9657f,-0.1574f,-0.2064f,  -0.9236f,-0.2259f,-0.3096f,  -0.9234f,-0.2297f,-0.3074f,  -0.8655f,-0.2942f,-0.4053f,  -0.8652f,-0.2983f,-0.4030f,  -0.7925f,-0.3576f,-0.4941f,  -0.7920f,-0.3619f,-0.4918f,  -0.7056f,-0.4150f,-0.5744f,  -0.7049f,-0.4194f,-0.5721f,  -0.6064f,-0.4652f,-0.6449f,  -0.6054f,-0.4697f,-0.6425f,  
	-0.4965f,-0.5072f,-0.7044f,  -0.4951f,-0.5119f,-0.7020f,  -0.3777f,-0.5403f,-0.7519f,  -0.3760f,-0.5449f,-0.7494f,  -0.2520f,-0.5637f,-0.7866f,  -0.2500f,-0.5682f,-0.7840f,  -0.1216f,-0.5767f,-0.8078f,  -0.1192f,-0.5809f,-0.8052f,  0.0112f,-0.5789f,-0.8153f,  0.0140f,-0.5827f,-0.8126f,  -0.1990f,-0.6861f,-0.6998f,  -0.1832f,-0.6815f,-0.7085f,  -0.0566f,-0.7131f,-0.6988f,  -0.0408f,-0.7058f,-0.7073f,  0.0871f,-0.7228f,-0.6855f,  
	0.1024f,-0.7131f,-0.6935f,  0.2288f,-0.7157f,-0.6598f,  0.2432f,-0.7041f,-0.6671f,  0.3654f,-0.6928f,-0.6217f,  0.3786f,-0.6797f,-0.6282f,  0.4940f,-0.6551f,-0.5717f,  0.5058f,-0.6411f,-0.5772f,  0.6120f,-0.6042f,-0.5102f,  0.6222f,-0.5898f,-0.5148f,  0.7171f,-0.5418f,-0.4384f,  0.7256f,-0.5273f,-0.4420f,  0.8072f,-0.4697f,-0.3575f,  0.8141f,-0.4555f,-0.3601f,  0.8808f,-0.3898f,-0.2687f,  0.8861f,-0.3763f,-0.2705f,  
	0.9366f,-0.3041f,-0.1738f,  0.9404f,-0.2916f,-0.1748f,  0.9739f,-0.2146f,-0.0744f,  0.9763f,-0.2033f,-0.0748f,  0.9920f,-0.1233f,0.0275f,  0.9932f,-0.1132f,0.0277f,  0.9910f,-0.0319f,0.1301f,  0.9911f,-0.0233f,0.1308f,  0.9711f,0.0577f,0.2316f,  0.9704f,0.0648f,0.2326f,  0.9330f,0.1438f,0.3300f,  0.9316f,0.1495f,0.3313f,  0.8775f,0.2249f,0.4235f,  0.8757f,0.2292f,0.4250f,  0.8061f,0.2997f,0.5104f,  
	0.8039f,0.3026f,0.5120f,  0.7200f,0.3669f,0.5891f,  0.7177f,0.3686f,0.5908f,  0.6211f,0.4257f,0.6581f,  0.6187f,0.4263f,0.6599f,  0.5112f,0.4750f,0.7163f,  0.5089f,0.4747f,0.7181f,  0.3924f,0.5144f,0.7625f,  0.3903f,0.5133f,0.7643f,  0.2670f,0.5434f,0.7959f,  0.2651f,0.5416f,0.7978f,  0.1371f,0.5617f,0.8159f,  0.1354f,0.5593f,0.8178f,  0.0051f,0.5692f,0.8222f,  0.0037f,0.5664f,0.8241f,  
	-0.1268f,0.5660f,0.8146f,  -0.1279f,0.5630f,0.8165f,  -0.2562f,0.5523f,0.7933f,  -0.2571f,0.5492f,0.7952f,  -0.3809f,0.5286f,0.7586f,  -0.3816f,0.5254f,0.7605f,  -0.4989f,0.4955f,0.7110f,  -0.4994f,0.4922f,0.7130f,  -0.6082f,0.4536f,0.6515f,  -0.6085f,0.4503f,0.6535f,  -0.7068f,0.4036f,0.5810f,  -0.7070f,0.4004f,0.5830f,  -0.7932f,0.3466f,0.5007f,  -0.7933f,0.3434f,0.5027f,  -0.8660f,0.2835f,0.4119f,  
	-0.8660f,0.2804f,0.4140f,  -0.9239f,0.2155f,0.3163f,  -0.9239f,0.2123f,0.3184f,  -0.9659f,0.1436f,0.2153f,  -0.9659f,0.1403f,0.2175f,  -0.9914f,0.0691f,0.1108f,  -0.9914f,0.0658f,0.1130f,  -1.0000f,-0.0068f,0.0045f,  -0.9999f,-0.0102f,0.0067f,  -0.9914f,-0.0827f,-0.1019f,  -0.9913f,-0.0863f,-0.0997f,  -0.9657f,-0.1574f,-0.2064f,  -0.9656f,-0.1613f,-0.2042f,  -0.9234f,-0.2297f,-0.3074f,  -0.9232f,-0.2338f,-0.3051f,  
	-0.8652f,-0.2983f,-0.4030f,  -0.8648f,-0.3026f,-0.4007f,  -0.7920f,-0.3619f,-0.4918f,  -0.7913f,-0.3664f,-0.4894f,  -0.7049f,-0.4194f,-0.5721f,  -0.7040f,-0.4242f,-0.5697f,  -0.6054f,-0.4697f,-0.6425f,  -0.6042f,-0.4746f,-0.6401f,  -0.4951f,-0.5119f,-0.7020f,  -0.4936f,-0.5169f,-0.6995f,  -0.3760f,-0.5449f,-0.7494f,  -0.3740f,-0.5499f,-0.7468f,  -0.2500f,-0.5682f,-0.7840f,  -0.2475f,-0.5730f,-0.7813f,  -0.1192f,-0.5809f,-0.8052f,  
	-0.1163f,-0.5854f,-0.8023f,  0.0140f,-0.5827f,-0.8126f,  0.0173f,-0.5867f,-0.8096f,  -0.1832f,-0.6815f,-0.7085f,  -0.1680f,-0.6766f,-0.7169f,  -0.0408f,-0.7058f,-0.7073f,  -0.0258f,-0.6983f,-0.7154f,  0.1024f,-0.7131f,-0.6935f,  0.1168f,-0.7034f,-0.7011f,  0.2432f,-0.7041f,-0.6671f,  0.2567f,-0.6927f,-0.6740f,  0.3786f,-0.6797f,-0.6282f,  0.3909f,-0.6670f,-0.6342f,  0.5058f,-0.6411f,-0.5772f,  0.5167f,-0.6276f,-0.5823f,  
	0.6222f,-0.5898f,-0.5148f,  0.6316f,-0.5759f,-0.5190f,  0.7256f,-0.5273f,-0.4420f,  0.7335f,-0.5134f,-0.4453f,  0.8141f,-0.4555f,-0.3601f,  0.8204f,-0.4421f,-0.3626f,  0.8861f,-0.3763f,-0.2705f,  0.8909f,-0.3636f,-0.2722f,  0.9404f,-0.2916f,-0.1748f,  0.9438f,-0.2798f,-0.1759f,  0.9763f,-0.2033f,-0.0748f,  0.9784f,-0.1926f,-0.0753f,  0.9932f,-0.1132f,0.0277f,  0.9942f,-0.1038f,0.0278f,  0.9911f,-0.0233f,0.1308f,  
	0.9912f,-0.0152f,0.1313f,  0.9704f,0.0648f,0.2326f,  0.9697f,0.0715f,0.2334f,  0.9316f,0.1495f,0.3313f,  0.9304f,0.1547f,0.3324f,  0.8757f,0.2292f,0.4250f,  0.8740f,0.2331f,0.4263f,  0.8039f,0.3026f,0.5120f,  0.8020f,0.3053f,0.5134f,  0.7177f,0.3686f,0.5908f,  0.7156f,0.3702f,0.5923f,  0.6187f,0.4263f,0.6599f,  0.6167f,0.4268f,0.6615f,  0.5089f,0.4747f,0.7181f,  0.5069f,0.4744f,0.7197f,  
	0.3903f,0.5133f,0.7643f,  0.3885f,0.5122f,0.7660f,  0.2651f,0.5416f,0.7978f,  0.2635f,0.5399f,0.7994f,  0.1354f,0.5593f,0.8178f,  0.1341f,0.5572f,0.8195f,  0.0037f,0.5664f,0.8241f,  0.0026f,0.5640f,0.8258f,  -0.1279f,0.5630f,0.8165f,  -0.1288f,0.5603f,0.8182f,  -0.2571f,0.5492f,0.7952f,  -0.2577f,0.5463f,0.7969f,  -0.3816f,0.5254f,0.7605f,  -0.3821f,0.5225f,0.7622f,  -0.4994f,0.4922f,0.7130f,  
	-0.4997f,0.4893f,0.7148f,  -0.6085f,0.4503f,0.6535f,  -0.6086f,0.4474f,0.6553f,  -0.7070f,0.4004f,0.5830f,  -0.7071f,0.3974f,0.5849f,  -0.7933f,0.3434f,0.5027f,  -0.7934f,0.3405f,0.5046f,  -0.8660f,0.2804f,0.4140f,  -0.8660f,0.2774f,0.4160f,  -0.9239f,0.2123f,0.3184f,  -0.9239f,0.2092f,0.3204f,  -0.9659f,0.1403f,0.2175f,  -0.9659f,0.1372f,0.2195f,  -0.9914f,0.0658f,0.1130f,  -0.9914f,0.0624f,0.1151f,  
	-0.9999f,-0.0102f,0.0067f,  -0.9999f,-0.0138f,0.0088f,  -0.9913f,-0.0863f,-0.0997f,  -0.9912f,-0.0901f,-0.0975f,  -0.9656f,-0.1613f,-0.2042f,  -0.9653f,-0.1653f,-0.2019f,  -0.9232f,-0.2338f,-0.3051f,  -0.9228f,-0.2381f,-0.3028f,  -0.8648f,-0.3026f,-0.4007f,  -0.8642f,-0.3072f,-0.3984f,  -0.7913f,-0.3664f,-0.4894f,  -0.7905f,-0.3713f,-0.4870f,  -0.7040f,-0.4242f,-0.5697f,  -0.7028f,-0.4293f,-0.5672f,  -0.6042f,-0.4746f,-0.6401f,  
	-0.6027f,-0.4799f,-0.6375f,  -0.4936f,-0.5169f,-0.6995f,  -0.4917f,-0.5223f,-0.6968f,  -0.3740f,-0.5499f,-0.7468f,  -0.3716f,-0.5553f,-0.7440f,  -0.2475f,-0.5730f,-0.7813f,  -0.2447f,-0.5782f,-0.7784f,  -0.1163f,-0.5854f,-0.8023f,  -0.1130f,-0.5902f,-0.7993f,  0.0173f,-0.5867f,-0.8096f,  0.0210f,-0.5910f,-0.8064f,  -0.1680f,-0.6766f,-0.7169f,  -0.1535f,-0.6715f,-0.7250f,  -0.0258f,-0.6983f,-0.7154f,  -0.0115f,-0.6907f,-0.7231f,  
	0.1168f,-0.7034f,-0.7011f,  0.1304f,-0.6938f,-0.7083f,  0.2567f,-0.6927f,-0.6740f,  0.2694f,-0.6815f,-0.6804f,  0.3909f,-0.6670f,-0.6342f,  0.4024f,-0.6547f,-0.6399f,  0.5167f,-0.6276f,-0.5823f,  0.5268f,-0.6146f,-0.5871f,  0.6316f,-0.5759f,-0.5190f,  0.6403f,-0.5626f,-0.5230f,  0.7335f,-0.5134f,-0.4453f,  0.7407f,-0.5002f,-0.4485f,  0.8204f,-0.4421f,-0.3626f,  0.8261f,-0.4293f,-0.3650f,  0.8909f,-0.3636f,-0.2722f,  
	0.8952f,-0.3515f,-0.2739f,  0.9438f,-0.2798f,-0.1759f,  0.9469f,-0.2687f,-0.1769f,  0.9784f,-0.1926f,-0.0753f,  0.9803f,-0.1826f,-0.0758f,  0.9942f,-0.1038f,0.0278f,  0.9951f,-0.0950f,0.0277f,  0.9912f,-0.0152f,0.1313f,  0.9913f,-0.0078f,0.1316f,  0.9697f,0.0715f,0.2334f,  0.9691f,0.0776f,0.2340f,  0.9304f,0.1547f,0.3324f,  0.9292f,0.1596f,0.3332f,  0.8740f,0.2331f,0.4263f,  0.8726f,0.2367f,0.4273f,  
	0.8020f,0.3053f,0.5134f,  0.8003f,0.3078f,0.5146f,  0.7156f,0.3702f,0.5923f,  0.7139f,0.3716f,0.5935f,  0.6167f,0.4268f,0.6615f,  0.6149f,0.4273f,0.6628f,  0.5069f,0.4744f,0.7197f,  0.5053f,0.4741f,0.7210f,  0.3885f,0.5122f,0.7660f,  0.3870f,0.5113f,0.7673f,  0.2635f,0.5399f,0.7994f,  0.2622f,0.5385f,0.8008f,  0.1341f,0.5572f,0.8195f,  0.1330f,0.5554f,0.8209f,  0.0026f,0.5640f,0.8258f,  
	0.0017f,0.5618f,0.8272f,  -0.1288f,0.5603f,0.8182f,  -0.1295f,0.5579f,0.8197f,  -0.2577f,0.5463f,0.7969f,  -0.2582f,0.5438f,0.7985f,  -0.3821f,0.5225f,0.7622f,  -0.3824f,0.5199f,0.7638f,  -0.4997f,0.4893f,0.7148f,  -0.4999f,0.4867f,0.7164f,  -0.6086f,0.4474f,0.6553f,  -0.6088f,0.4447f,0.6570f,  -0.7071f,0.3974f,0.5849f,  -0.7071f,0.3947f,0.5866f,  -0.7934f,0.3405f,0.5046f,  -0.7934f,0.3377f,0.5065f,  
	-0.8660f,0.2774f,0.4160f,  -0.8660f,0.2745f,0.4179f,  -0.9239f,0.2092f,0.3204f,  -0.9239f,0.2062f,0.3223f,  -0.9659f,0.1372f,0.2195f,  -0.9659f,0.1340f,0.2215f,  -0.9914f,0.0624f,0.1151f,  -0.9914f,0.0591f,0.1171f,  -0.9999f,-0.0138f,0.0088f,  -0.9998f,-0.0174f,0.0109f,  -0.9912f,-0.0901f,-0.0975f,  -0.9910f,-0.0940f,-0.0953f,  -0.9653f,-0.1653f,-0.2019f,  -0.9651f,-0.1696f,-0.1997f,  -0.9228f,-0.2381f,-0.3028f,  
	-0.9224f,-0.2427f,-0.3006f,  -0.8642f,-0.3072f,-0.3984f,  -0.8635f,-0.3122f,-0.3961f,  -0.7905f,-0.3713f,-0.4870f,  -0.7895f,-0.3766f,-0.4846f,  -0.7028f,-0.4293f,-0.5672f,  -0.7015f,-0.4348f,-0.5647f,  -0.6027f,-0.4799f,-0.6375f,  -0.6009f,-0.4857f,-0.6349f,  -0.4917f,-0.5223f,-0.6968f,  -0.4894f,-0.5280f,-0.6940f,  -0.3716f,-0.5553f,-0.7440f,  -0.3689f,-0.5610f,-0.7411f,  -0.2447f,-0.5782f,-0.7784f,  -0.2414f,-0.5837f,-0.7753f,  
	-0.1130f,-0.5902f,-0.7993f,  -0.1093f,-0.5954f,-0.7960f,  0.0210f,-0.5910f,-0.8064f,  0.0253f,-0.5955f,-0.8029f,  -0.1535f,-0.6715f,-0.7250f,  -0.1397f,-0.6661f,-0.7327f,  -0.0115f,-0.6907f,-0.7231f,  0.0019f,-0.6830f,-0.7304f,  0.1304f,-0.6938f,-0.7083f,  0.1432f,-0.6843f,-0.7150f,  0.2694f,-0.6815f,-0.6804f,  0.2812f,-0.6706f,-0.6865f,  0.4024f,-0.6547f,-0.6399f,  0.4130f,-0.6428f,-0.6452f,  0.5268f,-0.6146f,-0.5871f,  
	0.5362f,-0.6021f,-0.5916f,  0.6403f,-0.5626f,-0.5230f,  0.6483f,-0.5499f,-0.5267f,  0.7407f,-0.5002f,-0.4485f,  0.7472f,-0.4877f,-0.4515f,  0.8261f,-0.4293f,-0.3650f,  0.8313f,-0.4172f,-0.3672f,  0.8952f,-0.3515f,-0.2739f,  0.8991f,-0.3401f,-0.2755f,  0.9469f,-0.2687f,-0.1769f,  0.9495f,-0.2582f,-0.1780f,  0.9803f,-0.1826f,-0.0758f,  0.9819f,-0.1732f,-0.0764f,  0.9951f,-0.0950f,0.0277f,  0.9958f,-0.0868f,0.0275f,  
	0.9913f,-0.0078f,0.1316f,  0.9913f,-0.0008f,0.1317f,  0.9691f,0.0776f,0.2340f,  0.9686f,0.0834f,0.2344f,  0.9292f,0.1596f,0.3332f,  0.9283f,0.1641f,0.3338f,  0.8726f,0.2367f,0.4273f,  0.8713f,0.2401f,0.4280f,  0.8003f,0.3078f,0.5146f,  0.7989f,0.3101f,0.5154f,  0.7139f,0.3716f,0.5935f,  0.7124f,0.3730f,0.5945f,  0.6149f,0.4273f,0.6628f,  0.6134f,0.4279f,0.6638f,  0.5053f,0.4741f,0.7210f,  
	0.5039f,0.4739f,0.7221f,  0.3870f,0.5113f,0.7673f,  0.3858f,0.5105f,0.7685f,  0.2622f,0.5385f,0.8008f,  0.2611f,0.5373f,0.8020f,  0.1330f,0.5554f,0.8209f,  0.1321f,0.5538f,0.8221f,  0.0017f,0.5618f,0.8272f,  0.0010f,0.5600f,0.8285f,  -0.1295f,0.5579f,0.8197f,  -0.1299f,0.5559f,0.8211f,  -0.2582f,0.5438f,0.7985f,  -0.2585f,0.5416f,0.7999f,  -0.3824f,0.5199f,0.7638f,  -0.3826f,0.5176f,0.7653f,  
	-0.4999f,0.4867f,0.7164f,  -0.5000f,0.4843f,0.7179f,  -0.6088f,0.4447f,0.6570f,  -0.6088f,0.4423f,0.6586f,  -0.7071f,0.3947f,0.5866f,  -0.7071f,0.3922f,0.5883f,  -0.7934f,0.3377f,0.5065f,  -0.7934f,0.3351f,0.5082f,  -0.8660f,0.2745f,0.4179f,  -0.8660f,0.2718f,0.4197f,  -0.9239f,0.2062f,0.3223f,  -0.9239f,0.2033f,0.3242f,  -0.9659f,0.1340f,0.2215f,  -0.9659f,0.1309f,0.2234f,  -0.9914f,0.0591f,0.1171f,  
	-0.9913f,0.0556f,0.1191f,  -0.9998f,-0.0174f,0.0109f,  -0.9997f,-0.0211f,0.0129f,  -0.9910f,-0.0940f,-0.0953f,  -0.9908f,-0.0982f,-0.0932f,  -0.9651f,-0.1696f,-0.1997f,  -0.9647f,-0.1741f,-0.1976f,  -0.9224f,-0.2427f,-0.3006f,  -0.9218f,-0.2477f,-0.2983f,  -0.8635f,-0.3122f,-0.3961f,  -0.8627f,-0.3175f,-0.3937f,  -0.7895f,-0.3766f,-0.4846f,  -0.7883f,-0.3823f,-0.4822f,  -0.7015f,-0.4348f,-0.5647f,  -0.6998f,-0.4408f,-0.5621f,  
	-0.6009f,-0.4857f,-0.6349f,  -0.5988f,-0.4918f,-0.6322f,  -0.4894f,-0.5280f,-0.6940f,  -0.4868f,-0.5342f,-0.6911f,  -0.3689f,-0.5610f,-0.7411f,  -0.3657f,-0.5671f,-0.7380f,  -0.2414f,-0.5837f,-0.7753f,  -0.2377f,-0.5895f,-0.7720f,  -0.1093f,-0.5954f,-0.7960f,  -0.1050f,-0.6008f,-0.7925f,  0.0253f,-0.5955f,-0.8029f,  0.0301f,-0.6002f,-0.7993f,  -0.1397f,-0.6661f,-0.7327f,  -0.1266f,-0.6606f,-0.7400f,  0.0019f,-0.6830f,-0.7304f,  
	0.0146f,-0.6754f,-0.7373f,  0.1432f,-0.6843f,-0.7150f,  0.1551f,-0.6750f,-0.7214f,  0.2812f,-0.6706f,-0.6865f,  0.2921f,-0.6599f,-0.6922f,  0.4130f,-0.6428f,-0.6452f,  0.4229f,-0.6312f,-0.6502f,  0.5362f,-0.6021f,-0.5916f,  0.5448f,-0.5901f,-0.5959f,  0.6483f,-0.5499f,-0.5267f,  0.6555f,-0.5377f,-0.5302f,  0.7472f,-0.4877f,-0.4515f,  0.7531f,-0.4758f,-0.4543f,  0.8313f,-0.4172f,-0.3672f,  0.8360f,-0.4058f,-0.3695f,  
	0.8991f,-0.3401f,-0.2755f,  0.9026f,-0.3294f,-0.2772f,  0.9495f,-0.2582f,-0.1780f,  0.9519f,-0.2484f,-0.1791f,  0.9819f,-0.1732f,-0.0764f,  0.9834f,-0.1644f,-0.0771f,  0.9958f,-0.0868f,0.0275f,  0.9965f,-0.0792f,0.0271f,  0.9913f,-0.0008f,0.1317f,  0.9913f,0.0057f,0.1316f,  0.9686f,0.0834f,0.2344f,  0.9680f,0.0887f,0.2346f,  0.9283f,0.1641f,0.3338f,  0.9274f,0.1683f,0.3341f,  0.8713f,0.2401f,0.4280f,  
	0.8702f,0.2433f,0.4285f,  0.7989f,0.3101f,0.5154f,  0.7976f,0.3123f,0.5160f,  0.7124f,0.3730f,0.5945f,  0.7111f,0.3743f,0.5952f,  0.6134f,0.4279f,0.6638f,  0.6122f,0.4284f,0.6646f,  0.5039f,0.4739f,0.7221f,  0.5028f,0.4739f,0.7229f,  0.3858f,0.5105f,0.7685f,  0.3848f,0.5099f,0.7693f,  0.2611f,0.5373f,0.8020f,  0.2603f,0.5362f,0.8029f,  0.1321f,0.5538f,0.8221f,  0.1315f,0.5524f,0.8231f,  
	0.0010f,0.5600f,0.8285f,  0.0006f,0.5584f,0.8296f,  -0.1299f,0.5559f,0.8211f,  -0.1303f,0.5541f,0.8222f,  -0.2585f,0.5416f,0.7999f,  -0.2587f,0.5397f,0.8011f,  -0.3826f,0.5176f,0.7653f,  -0.3827f,0.5156f,0.7666f,  -0.5000f,0.4843f,0.7179f,  -0.5001f,0.4822f,0.7193f,  -0.6088f,0.4423f,0.6586f,  -0.6088f,0.4401f,0.6600f,  -0.7071f,0.3922f,0.5883f,  -0.7071f,0.3900f,0.5898f,  -0.7934f,0.3351f,0.5082f,  
	-0.7934f,0.3327f,0.5098f,  -0.8660f,0.2718f,0.4197f,  -0.8661f,0.2692f,0.4213f,  -0.9239f,0.2033f,0.3242f,  -0.9239f,0.2005f,0.3259f,  -0.9659f,0.1309f,0.2234f,  -0.9659f,0.1277f,0.2252f,  -0.9913f,0.0556f,0.1191f,  -0.9913f,0.0521f,0.1209f,  -0.9997f,-0.0211f,0.0129f,  -0.9996f,-0.0251f,0.0149f,  -0.9908f,-0.0982f,-0.0932f,  -0.9905f,-0.1025f,-0.0912f,  -0.9647f,-0.1741f,-0.1976f,  -0.9642f,-0.1789f,-0.1955f,  
	-0.9218f,-0.2477f,-0.2983f,  -0.9211f,-0.2529f,-0.2961f,  -0.8627f,-0.3175f,-0.3937f,  -0.8616f,-0.3231f,-0.3914f,  -0.7883f,-0.3823f,-0.4822f,  -0.7868f,-0.3883f,-0.4797f,  -0.6998f,-0.4408f,-0.5621f,  -0.6979f,-0.4471f,-0.5595f,  -0.5988f,-0.4918f,-0.6322f,  -0.5963f,-0.4983f,-0.6294f,  -0.4868f,-0.5342f,-0.6911f,  -0.4837f,-0.5408f,-0.6881f,  -0.3657f,-0.5671f,-0.7380f,  -0.3621f,-0.5736f,-0.7348f,  -0.2377f,-0.5895f,-0.7720f,  
	-0.2334f,-0.5957f,-0.7685f,  -0.1050f,-0.6008f,-0.7925f,  -0.1001f,-0.6064f,-0.7888f,  0.0301f,-0.6002f,-0.7993f,  0.0354f,-0.6052f,-0.7953f,  -0.1266f,-0.6606f,-0.7400f,  -0.1143f,-0.6549f,-0.7470f,  0.0146f,-0.6754f,-0.7373f,  0.0264f,-0.6678f,-0.7439f,  0.1551f,-0.6750f,-0.7214f,  0.1662f,-0.6658f,-0.7274f,  0.2921f,-0.6599f,-0.6922f,  0.3023f,-0.6496f,-0.6976f,  0.4229f,-0.6312f,-0.6502f,  0.4319f,-0.6202f,-0.6549f,  
	0.5448f,-0.5901f,-0.5959f,  0.5526f,-0.5786f,-0.5999f,  0.6555f,-0.5377f,-0.5302f,  0.6621f,-0.5262f,-0.5336f,  0.7531f,-0.4758f,-0.4543f,  0.7585f,-0.4645f,-0.4570f,  0.8360f,-0.4058f,-0.3695f,  0.8402f,-0.3950f,-0.3716f,  0.9026f,-0.3294f,-0.2772f,  0.9057f,-0.3194f,-0.2789f,  0.9519f,-0.2484f,-0.1791f,  0.9541f,-0.2392f,-0.1804f,  0.9834f,-0.1644f,-0.0771f,  0.9846f,-0.1563f,-0.0780f,  0.9965f,-0.0792f,0.0271f,  
	0.9970f,-0.0720f,0.0266f,  0.9913f,0.0057f,0.1316f,  0.9913f,0.0117f,0.1314f,  0.9680f,0.0887f,0.2346f,  0.9676f,0.0937f,0.2345f,  0.9274f,0.1683f,0.3341f,  0.9266f,0.1723f,0.3342f,  0.8702f,0.2433f,0.4285f,  0.8692f,0.2462f,0.4287f,  0.7976f,0.3123f,0.5160f,  0.7966f,0.3144f,0.5164f,  0.7111f,0.3743f,0.5952f,  0.7100f,0.3756f,0.5956f,  0.6122f,0.4284f,0.6646f,  0.6112f,0.4291f,0.6651f,  
	0.5028f,0.4739f,0.7229f,  0.5019f,0.4739f,0.7235f,  0.3848f,0.5099f,0.7693f,  0.3841f,0.5095f,0.7700f,  0.2603f,0.5362f,0.8029f,  0.2597f,0.5354f,0.8037f,  0.1315f,0.5524f,0.8231f,  0.1310f,0.5513f,0.8239f,  0.0006f,0.5584f,0.8296f,  0.0002f,0.5571f,0.8305f,  -0.1303f,0.5541f,0.8222f,  -0.1305f,0.5526f,0.8232f,  -0.2587f,0.5397f,0.8011f,  -0.2588f,0.5382f,0.8021f,  -0.3827f,0.5156f,0.7666f,  
	-0.3827f,0.5139f,0.7677f,  -0.5001f,0.4822f,0.7193f,  -0.5001f,0.4804f,0.7205f,  -0.6088f,0.4401f,0.6600f,  -0.6088f,0.4382f,0.6613f,  -0.7071f,0.3900f,0.5898f,  -0.7072f,0.3879f,0.5912f,  -0.7934f,0.3327f,0.5098f,  -0.7934f,0.3304f,0.5112f,  -0.8661f,0.2692f,0.4213f,  -0.8661f,0.2666f,0.4228f,  -0.9239f,0.2005f,0.3259f,  -0.9240f,0.1976f,0.3275f,  -0.9659f,0.1277f,0.2252f,  -0.9659f,0.1245f,0.2269f,  
	-0.9913f,0.0521f,0.1209f,  -0.9913f,0.0485f,0.1227f,  -0.9996f,-0.0251f,0.0149f,  -0.9994f,-0.0292f,0.0167f,  -0.9905f,-0.1025f,-0.0912f,  -0.9902f,-0.1071f,-0.0893f,  -0.9642f,-0.1789f,-0.1955f,  -0.9637f,-0.1840f,-0.1934f,  -0.9211f,-0.2529f,-0.2961f,  -0.9202f,-0.2585f,-0.2940f,  -0.8616f,-0.3231f,-0.3914f,  -0.8604f,-0.3292f,-0.3891f,  -0.7868f,-0.3883f,-0.4797f,  -0.7851f,-0.3948f,-0.4772f,  -0.6979f,-0.4471f,-0.5595f,  
	-0.6957f,-0.4539f,-0.5568f,  -0.5963f,-0.4983f,-0.6294f,  -0.5934f,-0.5053f,-0.6265f,  -0.4837f,-0.5408f,-0.6881f,  -0.4802f,-0.5479f,-0.6850f,  -0.3621f,-0.5736f,-0.7348f,  -0.3579f,-0.5805f,-0.7314f,  -0.2334f,-0.5957f,-0.7685f,  -0.2286f,-0.6023f,-0.7649f,  -0.1001f,-0.6064f,-0.7888f,  -0.0947f,-0.6124f,-0.7849f,  0.0354f,-0.6052f,-0.7953f,  0.0414f,-0.6103f,-0.7911f,  -0.1143f,-0.6549f,-0.7470f,  -0.1027f,-0.6492f,-0.7537f,  
	0.0264f,-0.6678f,-0.7439f,  0.0375f,-0.6603f,-0.7501f,  0.1662f,-0.6658f,-0.7274f,  0.1766f,-0.6569f,-0.7331f,  0.3023f,-0.6496f,-0.6976f,  0.3117f,-0.6396f,-0.7027f,  0.4319f,-0.6202f,-0.6549f,  0.4402f,-0.6095f,-0.6593f,  0.5526f,-0.5786f,-0.5999f,  0.5597f,-0.5676f,-0.6037f,  0.6621f,-0.5262f,-0.5336f,  0.6681f,-0.5153f,-0.5368f,  0.7585f,-0.4645f,-0.4570f,  0.7633f,-0.4538f,-0.4597f,  0.8402f,-0.3950f,-0.3716f,  
	0.8439f,-0.3849f,-0.3738f,  0.9057f,-0.3194f,-0.2789f,  0.9084f,-0.3099f,-0.2806f,  0.9541f,-0.2392f,-0.1804f,  0.9559f,-0.2306f,-0.1817f,  0.9846f,-0.1563f,-0.0780f,  0.9857f,-0.1486f,-0.0789f,  0.9970f,-0.0720f,0.0266f,  0.9975f,-0.0654f,0.0259f,  0.9913f,0.0117f,0.1314f,  0.9912f,0.0174f,0.1309f,  0.9676f,0.0937f,0.2345f,  0.9672f,0.0983f,0.2342f,  0.9266f,0.1723f,0.3342f,  0.9260f,0.1760f,0.3341f,  
	0.8692f,0.2462f,0.4287f,  0.8684f,0.2491f,0.4287f,  0.7966f,0.3144f,0.5164f,  0.7957f,0.3164f,0.5165f,  0.7100f,0.3756f,0.5956f,  0.7092f,0.3769f,0.5958f,  0.6112f,0.4291f,0.6651f,  0.6104f,0.4298f,0.6653f,  0.5019f,0.4739f,0.7235f,  0.5012f,0.4741f,0.7239f,  0.3841f,0.5095f,0.7700f,  0.3835f,0.5093f,0.7704f,  0.2597f,0.5354f,0.8037f,  0.2593f,0.5349f,0.8042f,  0.1310f,0.5513f,0.8239f,  
	0.1307f,0.5505f,0.8245f,  0.0002f,0.5571f,0.8305f,  0.0001f,0.5561f,0.8311f,  -0.1305f,0.5526f,0.8232f,  -0.1306f,0.5515f,0.8239f,  -0.2588f,0.5382f,0.8021f,  -0.2589f,0.5369f,0.8030f,  -0.3827f,0.5139f,0.7677f,  -0.3828f,0.5125f,0.7687f,  -0.5001f,0.4804f,0.7205f,  -0.5001f,0.4788f,0.7216f,  -0.6088f,0.4382f,0.6613f,  -0.6088f,0.4364f,0.6625f,  -0.7072f,0.3879f,0.5912f,  -0.7072f,0.3859f,0.5924f,  
	-0.7934f,0.3304f,0.5112f,  -0.7935f,0.3282f,0.5125f,  -0.8661f,0.2666f,0.4228f,  -0.8662f,0.2641f,0.4242f,  -0.9240f,0.1976f,0.3275f,  -0.9240f,0.1947f,0.3290f,  -0.9659f,0.1245f,0.2269f,  -0.9660f,0.1212f,0.2284f,  -0.9913f,0.0485f,0.1227f,  -0.9912f,0.0447f,0.1243f,  -0.9994f,-0.0292f,0.0167f,  -0.9993f,-0.0335f,0.0184f,  -0.9902f,-0.1071f,-0.0893f,  -0.9899f,-0.1120f,-0.0875f,  -0.9637f,-0.1840f,-0.1934f,  
	-0.9630f,-0.1895f,-0.1915f,  -0.9202f,-0.2585f,-0.2940f,  -0.9192f,-0.2645f,-0.2919f,  -0.8604f,-0.3292f,-0.3891f,  -0.8589f,-0.3357f,-0.3868f,  -0.7851f,-0.3948f,-0.4772f,  -0.7831f,-0.4017f,-0.4747f,  -0.6957f,-0.4539f,-0.5568f,  -0.6930f,-0.4612f,-0.5541f,  -0.5934f,-0.5053f,-0.6265f,  -0.5902f,-0.5128f,-0.6235f,  -0.4802f,-0.5479f,-0.6850f,  -0.4763f,-0.5553f,-0.6817f,  -0.3579f,-0.5805f,-0.7314f,  -0.3532f,-0.5878f,-0.7278f,  
	-0.2286f,-0.6023f,-0.7649f,  -0.2233f,-0.6091f,-0.7610f,  -0.0947f,-0.6124f,-0.7849f,  -0.0887f,-0.6186f,-0.7807f,  0.0414f,-0.6103f,-0.7911f,  0.0480f,-0.6156f,-0.7866f,  -0.1027f,-0.6492f,-0.7537f,  -0.0919f,-0.6434f,-0.7600f,  0.0375f,-0.6603f,-0.7501f,  0.0478f,-0.6529f,-0.7560f,  0.1766f,-0.6569f,-0.7331f,  0.1861f,-0.6482f,-0.7384f,  0.3117f,-0.6396f,-0.7027f,  0.3203f,-0.6300f,-0.7075f,  0.4402f,-0.6095f,-0.6593f,  
	0.4477f,-0.5993f,-0.6636f,  0.5597f,-0.5676f,-0.6037f,  0.5662f,-0.5572f,-0.6074f,  0.6681f,-0.5153f,-0.5368f,  0.6735f,-0.5049f,-0.5399f,  0.7633f,-0.4538f,-0.4597f,  0.7677f,-0.4438f,-0.4623f,  0.8439f,-0.3849f,-0.3738f,  0.8472f,-0.3753f,-0.3759f,  0.9084f,-0.3099f,-0.2806f,  0.9109f,-0.3011f,-0.2823f,  0.9559f,-0.2306f,-0.1817f,  0.9576f,-0.2226f,-0.1831f,  0.9857f,-0.1486f,-0.0789f,  0.9867f,-0.1414f,-0.0800f,  
	0.9975f,-0.0654f,0.0259f,  0.9979f,-0.0592f,0.0250f,  0.9912f,0.0174f,0.1309f,  0.9912f,0.0227f,0.1302f,  0.9672f,0.0983f,0.2342f,  0.9669f,0.1027f,0.2338f,  0.9260f,0.1760f,0.3341f,  0.9254f,0.1795f,0.3338f,  0.8684f,0.2491f,0.4287f,  0.8678f,0.2517f,0.4285f,  0.7957f,0.3164f,0.5165f,  0.7950f,0.3183f,0.5164f,  0.7092f,0.3769f,0.5958f,  0.7085f,0.3783f,0.5958f,  0.6104f,0.4298f,0.6653f,  
	0.6098f,0.4305f,0.6654f,  0.5012f,0.4741f,0.7239f,  0.5007f,0.4744f,0.7240f,  0.3835f,0.5093f,0.7704f,  0.3831f,0.5093f,0.7706f,  0.2593f,0.5349f,0.8042f,  0.2590f,0.5346f,0.8045f,  0.1307f,0.5505f,0.8245f,  0.1306f,0.5500f,0.8249f,  0.0001f,0.5561f,0.8311f,  -0.0000f,0.5553f,0.8316f,  -0.1306f,0.5515f,0.8239f,  -0.1306f,0.5506f,0.8245f,  -0.2589f,0.5369f,0.8030f,  -0.2589f,0.5358f,0.8036f,  
	-0.3828f,0.5125f,0.7687f,  -0.3828f,0.5113f,0.7694f,  -0.5001f,0.4788f,0.7216f,  -0.5001f,0.4775f,0.7224f,  -0.6088f,0.4364f,0.6625f,  -0.6089f,0.4349f,0.6634f,  -0.7072f,0.3859f,0.5924f,  -0.7073f,0.3841f,0.5934f,  -0.7935f,0.3282f,0.5125f,  -0.7936f,0.3261f,0.5136f,  -0.8662f,0.2641f,0.4242f,  -0.8663f,0.2616f,0.4254f,  -0.9240f,0.1947f,0.3290f,  -0.9242f,0.1918f,0.3303f,  -0.9660f,0.1212f,0.2284f,  
	-0.9661f,0.1178f,0.2298f,  -0.9912f,0.0447f,0.1243f,  -0.9912f,0.0407f,0.1258f,  -0.9993f,-0.0335f,0.0184f,  -0.9991f,-0.0380f,0.0200f,  -0.9899f,-0.1120f,-0.0875f,  -0.9894f,-0.1172f,-0.0857f,  -0.9630f,-0.1895f,-0.1915f,  -0.9622f,-0.1953f,-0.1896f,  -0.9192f,-0.2645f,-0.2919f,  -0.9179f,-0.2709f,-0.2898f,  -0.8589f,-0.3357f,-0.3868f,  -0.8571f,-0.3427f,-0.3846f,  -0.7831f,-0.4017f,-0.4747f,  -0.7808f,-0.4091f,-0.4723f,  
	-0.6930f,-0.4612f,-0.5541f,  -0.6901f,-0.4689f,-0.5513f,  -0.5902f,-0.5128f,-0.6235f,  -0.5865f,-0.5207f,-0.6204f,  -0.4763f,-0.5553f,-0.6817f,  -0.4718f,-0.5632f,-0.6784f,  -0.3532f,-0.5878f,-0.7278f,  -0.3480f,-0.5955f,-0.7241f,  -0.2233f,-0.6091f,-0.7610f,  -0.2173f,-0.6164f,-0.7569f,  -0.0887f,-0.6186f,-0.7807f,  -0.0820f,-0.6251f,-0.7762f,  0.0480f,-0.6156f,-0.7866f,  0.0552f,-0.6210f,-0.7818f,  -0.0919f,-0.6434f,-0.7600f,  
	-0.0818f,-0.6377f,-0.7659f,  0.0478f,-0.6529f,-0.7560f,  0.0573f,-0.6456f,-0.7615f,  0.1861f,-0.6482f,-0.7384f,  0.1949f,-0.6397f,-0.7435f,  0.3203f,-0.6300f,-0.7075f,  0.3281f,-0.6208f,-0.7120f,  0.4477f,-0.5993f,-0.6636f,  0.4546f,-0.5896f,-0.6676f,  0.5662f,-0.5572f,-0.6074f,  0.5721f,-0.5473f,-0.6109f,  0.6735f,-0.5049f,-0.5399f,  0.6783f,-0.4951f,-0.5429f,  0.7677f,-0.4438f,-0.4623f,  0.7716f,-0.4343f,-0.4649f,  
	0.8472f,-0.3753f,-0.3759f,  0.8502f,-0.3663f,-0.3781f,  0.9109f,-0.3011f,-0.2823f,  0.9130f,-0.2927f,-0.2841f,  0.9576f,-0.2226f,-0.1831f,  0.9590f,-0.2150f,-0.1846f,  0.9867f,-0.1414f,-0.0800f,  0.9875f,-0.1347f,-0.0813f,  0.9979f,-0.0592f,0.0250f,  0.9983f,-0.0533f,0.0240f,  0.9912f,0.0227f,0.1302f,  0.9912f,0.0276f,0.1294f,  0.9669f,0.1027f,0.2338f,  0.9666f,0.1068f,0.2331f,  0.9254f,0.1795f,0.3338f,  
	0.9250f,0.1828f,0.3332f,  0.8678f,0.2517f,0.4285f,  0.8672f,0.2543f,0.4281f,  0.7950f,0.3183f,0.5164f,  0.7945f,0.3203f,0.5160f,  0.7085f,0.3783f,0.5958f,  0.7080f,0.3796f,0.5955f,  0.6098f,0.4305f,0.6654f,  0.6094f,0.4314f,0.6652f,  0.5007f,0.4744f,0.7240f,  0.5004f,0.4749f,0.7239f,  0.3831f,0.5093f,0.7706f,  0.3829f,0.5094f,0.7706f,  0.2590f,0.5346f,0.8045f,  0.2588f,0.5345f,0.8046f,  
	0.1306f,0.5500f,0.8249f,  0.1305f,0.5497f,0.8251f,  -0.0000f,0.5553f,0.8316f,  -0.0001f,0.5549f,0.8319f,  -0.1306f,0.5506f,0.8245f,  -0.1306f,0.5500f,0.8249f,  -0.2589f,0.5358f,0.8036f,  -0.2589f,0.5351f,0.8041f,  -0.3828f,0.5113f,0.7694f,  -0.3828f,0.5104f,0.7700f,  -0.5001f,0.4775f,0.7224f,  -0.5002f,0.4764f,0.7231f,  -0.6089f,0.4349f,0.6634f,  -0.6090f,0.4335f,0.6642f,  -0.7073f,0.3841f,0.5934f,  
	-0.7075f,0.3825f,0.5943f,  -0.7936f,0.3261f,0.5136f,  -0.7938f,0.3241f,0.5146f,  -0.8663f,0.2616f,0.4254f,  -0.8666f,0.2592f,0.4265f,  -0.9242f,0.1918f,0.3303f,  -0.9244f,0.1889f,0.3314f,  -0.9661f,0.1178f,0.2298f,  -0.9662f,0.1142f,0.2311f,  -0.9912f,0.0407f,0.1258f,  -0.9912f,0.0366f,0.1272f,  -0.9991f,-0.0380f,0.0200f,  -0.9988f,-0.0429f,0.0215f,  -0.9894f,-0.1172f,-0.0857f,  -0.9889f,-0.1227f,-0.0841f,  
	-0.9622f,-0.1953f,-0.1896f,  -0.9613f,-0.2015f,-0.1878f,  -0.9179f,-0.2709f,-0.2898f,  -0.9165f,-0.2777f,-0.2878f,  -0.8571f,-0.3427f,-0.3846f,  -0.8551f,-0.3501f,-0.3823f,  -0.7808f,-0.4091f,-0.4723f,  -0.7781f,-0.4170f,-0.4697f,  -0.6901f,-0.4689f,-0.5513f,  -0.6866f,-0.4771f,-0.5485f,  -0.5865f,-0.5207f,-0.6204f,  -0.5823f,-0.5291f,-0.6173f,  -0.4718f,-0.5632f,-0.6784f,  -0.4668f,-0.5716f,-0.6748f,  -0.3480f,-0.5955f,-0.7241f,  
	-0.3422f,-0.6036f,-0.7202f,  -0.2173f,-0.6164f,-0.7569f,  -0.2107f,-0.6239f,-0.7526f,  -0.0820f,-0.6251f,-0.7762f,  -0.0747f,-0.6318f,-0.7715f,  0.0552f,-0.6210f,-0.7818f,  0.0631f,-0.6266f,-0.7768f,  -0.0818f,-0.6377f,-0.7659f,  -0.0724f,-0.6320f,-0.7716f,  0.0573f,-0.6456f,-0.7615f,  0.0661f,-0.6385f,-0.7668f,  0.1949f,-0.6397f,-0.7435f,  0.2029f,-0.6316f,-0.7483f,  0.3281f,-0.6208f,-0.7120f,  0.3353f,-0.6119f,-0.7163f,  
	0.4546f,-0.5896f,-0.6676f,  0.4609f,-0.5803f,-0.6714f,  0.5721f,-0.5473f,-0.6109f,  0.5773f,-0.5379f,-0.6143f,  0.6783f,-0.4951f,-0.5429f,  0.6827f,-0.4858f,-0.5459f,  0.7716f,-0.4343f,-0.4649f,  0.7750f,-0.4253f,-0.4674f,  0.8502f,-0.3663f,-0.3781f,  0.8528f,-0.3579f,-0.3803f,  0.9130f,-0.2927f,-0.2841f,  0.9149f,-0.2849f,-0.2860f,  0.9590f,-0.2150f,-0.1846f,  0.9603f,-0.2080f,-0.1862f,  0.9875f,-0.1347f,-0.0813f,  
	0.9883f,-0.1284f,-0.0826f,  0.9983f,-0.0533f,0.0240f,  0.9986f,-0.0479f,0.0228f,  0.9912f,0.0276f,0.1294f,  0.9912f,0.0323f,0.1284f,  0.9666f,0.1068f,0.2331f,  0.9664f,0.1107f,0.2322f,  0.9250f,0.1828f,0.3332f,  0.9246f,0.1859f,0.3324f,  0.8672f,0.2543f,0.4281f,  0.8668f,0.2568f,0.4274f,  0.7945f,0.3203f,0.5160f,  0.7940f,0.3222f,0.5154f,  0.7080f,0.3796f,0.5955f,  0.7076f,0.3811f,0.5950f,  
	0.6094f,0.4314f,0.6652f,  0.6091f,0.4325f,0.6648f,  0.5004f,0.4749f,0.7239f,  0.5001f,0.4756f,0.7236f,  0.3829f,0.5094f,0.7706f,  0.3827f,0.5098f,0.7705f,  0.2588f,0.5345f,0.8046f,  0.2588f,0.5347f,0.8045f,  0.1305f,0.5497f,0.8251f,  0.1304f,0.5497f,0.8251f,  -0.0001f,0.5549f,0.8319f,  -0.0001f,0.5547f,0.8320f,  -0.1306f,0.5500f,0.8249f,  -0.1306f,0.5497f,0.8251f,  -0.2589f,0.5351f,0.8041f,  
	-0.2589f,0.5346f,0.8044f,  -0.3828f,0.5104f,0.7700f,  -0.3829f,0.5098f,0.7704f,  -0.5002f,0.4764f,0.7231f,  -0.5003f,0.4755f,0.7236f,  -0.6090f,0.4335f,0.6642f,  -0.6092f,0.4324f,0.6648f,  -0.7075f,0.3825f,0.5943f,  -0.7077f,0.3810f,0.5950f,  -0.7938f,0.3241f,0.5146f,  -0.7941f,0.3221f,0.5154f,  -0.8666f,0.2592f,0.4265f,  -0.8669f,0.2567f,0.4273f,  -0.9244f,0.1889f,0.3314f,  -0.9247f,0.1858f,0.3324f,  
	-0.9662f,0.1142f,0.2311f,  -0.9664f,0.1105f,0.2321f,  -0.9912f,0.0366f,0.1272f,  -0.9912f,0.0322f,0.1284f,  -0.9988f,-0.0429f,0.0215f,  -0.9986f,-0.0480f,0.0228f,  -0.9889f,-0.1227f,-0.0841f,  -0.9883f,-0.1286f,-0.0826f,  -0.9613f,-0.2015f,-0.1878f,  -0.9602f,-0.2081f,-0.1861f,  -0.9165f,-0.2777f,-0.2878f,  -0.9149f,-0.2850f,-0.2859f,  -0.8551f,-0.3501f,-0.3823f,  -0.8529f,-0.3580f,-0.3801f,  -0.7781f,-0.4170f,-0.4697f,  
	-0.7751f,-0.4254f,-0.4672f,  -0.6866f,-0.4771f,-0.5485f,  -0.6828f,-0.4859f,-0.5456f,  -0.5823f,-0.5291f,-0.6173f,  -0.5775f,-0.5380f,-0.6140f,  -0.4668f,-0.5716f,-0.6748f,  -0.4612f,-0.5804f,-0.6711f,  -0.3422f,-0.6036f,-0.7202f,  -0.3357f,-0.6120f,-0.7161f,  -0.2107f,-0.6239f,-0.7526f,  -0.2034f,-0.6318f,-0.7480f,  -0.0747f,-0.6318f,-0.7715f,  -0.0667f,-0.6387f,-0.7665f,  0.0631f,-0.6266f,-0.7768f,  0.0717f,-0.6323f,-0.7714f,  
	-0.0724f,-0.6320f,-0.7716f,  -0.0638f,-0.6263f,-0.7770f,  0.0661f,-0.6385f,-0.7668f,  0.0742f,-0.6316f,-0.7717f,  0.2029f,-0.6316f,-0.7483f,  0.2102f,-0.6237f,-0.7528f,  0.3353f,-0.6119f,-0.7163f,  0.3418f,-0.6034f,-0.7204f,  0.4609f,-0.5803f,-0.6714f,  0.4665f,-0.5715f,-0.6751f,  0.5773f,-0.5379f,-0.6143f,  0.5821f,-0.5290f,-0.6175f,  0.6827f,-0.4858f,-0.5459f,  0.6865f,-0.4771f,-0.5487f,  0.7750f,-0.4253f,-0.4674f,  
	0.7780f,-0.4169f,-0.4699f,  0.8528f,-0.3579f,-0.3803f,  0.8551f,-0.3500f,-0.3825f,  0.9149f,-0.2849f,-0.2860f,  0.9165f,-0.2776f,-0.2879f,  0.9603f,-0.2080f,-0.1862f,  0.9613f,-0.2013f,-0.1879f,  0.9883f,-0.1284f,-0.0826f,  0.9889f,-0.1226f,-0.0841f,  0.9986f,-0.0479f,0.0228f,  0.9989f,-0.0427f,0.0215f,  0.9912f,0.0323f,0.1284f,  0.9912f,0.0367f,0.1272f,  0.9664f,0.1107f,0.2322f,  0.9662f,0.1144f,0.2311f,  
	0.9246f,0.1859f,0.3324f,  0.9243f,0.1890f,0.3315f,  0.8668f,0.2568f,0.4274f,  0.8665f,0.2593f,0.4265f,  0.7940f,0.3222f,0.5154f,  0.7937f,0.3242f,0.5147f,  0.7076f,0.3811f,0.5950f,  0.7074f,0.3826f,0.5944f,  0.6091f,0.4325f,0.6648f,  0.6089f,0.4336f,0.6643f,  0.5001f,0.4756f,0.7236f,  0.5000f,0.4765f,0.7232f,  0.3827f,0.5098f,0.7705f,  0.3826f,0.5105f,0.7701f,  0.2588f,0.5347f,0.8045f,  
	0.2587f,0.5351f,0.8042f,  0.1304f,0.5497f,0.8251f,  0.1304f,0.5500f,0.8249f,  -0.0001f,0.5547f,0.8320f,  -0.0001f,0.5549f,0.8319f,  -0.1306f,0.5497f,0.8251f,  -0.1306f,0.5497f,0.8251f,  -0.2589f,0.5346f,0.8044f,  -0.2590f,0.5344f,0.8045f,  -0.3829f,0.5098f,0.7704f,  -0.3830f,0.5094f,0.7706f,  -0.5003f,0.4755f,0.7236f,  -0.5005f,0.4748f,0.7239f,  -0.6092f,0.4324f,0.6648f,  -0.6095f,0.4313f,0.6652f,  
	-0.7077f,0.3810f,0.5950f,  -0.7081f,0.3795f,0.5955f,  -0.7941f,0.3221f,0.5154f,  -0.7946f,0.3202f,0.5159f,  -0.8669f,0.2567f,0.4273f,  -0.8673f,0.2542f,0.4280f,  -0.9247f,0.1858f,0.3324f,  -0.9250f,0.1826f,0.3331f,  -0.9664f,0.1105f,0.2321f,  -0.9666f,0.1066f,0.2330f,  -0.9912f,0.0322f,0.1284f,  -0.9912f,0.0275f,0.1294f,  -0.9986f,-0.0480f,0.0228f,  -0.9983f,-0.0535f,0.0240f,  -0.9883f,-0.1286f,-0.0826f,  
	-0.9875f,-0.1348f,-0.0812f,  -0.9602f,-0.2081f,-0.1861f,  -0.9590f,-0.2151f,-0.1845f,  -0.9149f,-0.2850f,-0.2859f,  -0.9130f,-0.2928f,-0.2840f,  -0.8529f,-0.3580f,-0.3801f,  -0.8502f,-0.3664f,-0.3779f,  -0.7751f,-0.4254f,-0.4672f,  -0.7716f,-0.4344f,-0.4647f,  -0.6828f,-0.4859f,-0.5456f,  -0.6785f,-0.4952f,-0.5427f,  -0.5775f,-0.5380f,-0.6140f,  -0.5723f,-0.5474f,-0.6106f,  -0.4612f,-0.5804f,-0.6711f,  -0.4549f,-0.5897f,-0.6673f,  
	-0.3357f,-0.6120f,-0.7161f,  -0.3285f,-0.6209f,-0.7117f,  -0.2034f,-0.6318f,-0.7480f,  -0.1954f,-0.6399f,-0.7432f,  -0.0667f,-0.6387f,-0.7665f,  -0.0580f,-0.6458f,-0.7613f,  0.0717f,-0.6323f,-0.7714f,  0.0810f,-0.6380f,-0.7658f,  -0.0638f,-0.6263f,-0.7770f,  -0.0558f,-0.6208f,-0.7820f,  0.0742f,-0.6316f,-0.7717f,  0.0815f,-0.6249f,-0.7764f,  0.2102f,-0.6237f,-0.7528f,  0.2169f,-0.6162f,-0.7571f,  0.3418f,-0.6034f,-0.7204f,  
	0.3477f,-0.5954f,-0.7243f,  0.4665f,-0.5715f,-0.6751f,  0.4715f,-0.5632f,-0.6786f,  0.5821f,-0.5290f,-0.6175f,  0.5863f,-0.5206f,-0.6207f,  0.6865f,-0.4771f,-0.5487f,  0.6899f,-0.4688f,-0.5515f,  0.7780f,-0.4169f,-0.4699f,  0.7807f,-0.4090f,-0.4724f,  0.8551f,-0.3500f,-0.3825f,  0.8571f,-0.3426f,-0.3847f,  0.9165f,-0.2776f,-0.2879f,  0.9180f,-0.2708f,-0.2899f,  0.9613f,-0.2013f,-0.1879f,  0.9623f,-0.1952f,-0.1896f,  
	0.9889f,-0.1226f,-0.0841f,  0.9894f,-0.1171f,-0.0857f,  0.9989f,-0.0427f,0.0215f,  0.9991f,-0.0379f,0.0201f,  0.9912f,0.0367f,0.1272f,  0.9912f,0.0408f,0.1259f,  0.9662f,0.1144f,0.2311f,  0.9660f,0.1179f,0.2299f,  0.9243f,0.1890f,0.3315f,  0.9241f,0.1919f,0.3304f,  0.8665f,0.2593f,0.4265f,  0.8663f,0.2617f,0.4255f,  0.7937f,0.3242f,0.5147f,  0.7935f,0.3262f,0.5137f,  0.7074f,0.3826f,0.5944f,  
	0.7072f,0.3842f,0.5935f,  0.6089f,0.4336f,0.6643f,  0.6088f,0.4350f,0.6635f,  0.5000f,0.4765f,0.7232f,  0.4999f,0.4776f,0.7225f,  0.3826f,0.5105f,0.7701f,  0.3826f,0.5114f,0.7695f,  0.2587f,0.5351f,0.8042f,  0.2587f,0.5359f,0.8037f,  0.1304f,0.5500f,0.8249f,  0.1304f,0.5506f,0.8245f,  -0.0001f,0.5549f,0.8319f,  -0.0001f,0.5553f,0.8316f,  -0.1306f,0.5497f,0.8251f,  -0.1307f,0.5499f,0.8249f,  
	-0.2590f,0.5344f,0.8045f,  -0.2592f,0.5345f,0.8044f,  -0.3830f,0.5094f,0.7706f,  -0.3833f,0.5092f,0.7706f,  -0.5005f,0.4748f,0.7239f,  -0.5009f,0.4743f,0.7240f,  -0.6095f,0.4313f,0.6652f,  -0.6100f,0.4304f,0.6653f,  -0.7081f,0.3795f,0.5955f,  -0.7086f,0.3781f,0.5957f,  -0.7946f,0.3202f,0.5159f,  -0.7951f,0.3182f,0.5163f,  -0.8673f,0.2542f,0.4280f,  -0.8678f,0.2516f,0.4284f,  -0.9250f,0.1826f,0.3331f,  
	-0.9255f,0.1793f,0.3337f,  -0.9666f,0.1066f,0.2330f,  -0.9669f,0.1025f,0.2337f,  -0.9912f,0.0275f,0.1294f,  -0.9912f,0.0225f,0.1302f,  -0.9983f,-0.0535f,0.0240f,  -0.9979f,-0.0593f,0.0250f,  -0.9875f,-0.1348f,-0.0812f,  -0.9867f,-0.1416f,-0.0800f,  -0.9590f,-0.2151f,-0.1845f,  -0.9576f,-0.2227f,-0.1830f,  -0.9130f,-0.2928f,-0.2840f,  -0.9109f,-0.3012f,-0.2822f,  -0.8502f,-0.3664f,-0.3779f,  -0.8473f,-0.3754f,-0.3757f,  
	-0.7716f,-0.4344f,-0.4647f,  -0.7678f,-0.4439f,-0.4621f,  -0.6785f,-0.4952f,-0.5427f,  -0.6736f,-0.5050f,-0.5396f,  -0.5723f,-0.5474f,-0.6106f,  -0.5664f,-0.5573f,-0.6071f,  -0.4549f,-0.5897f,-0.6673f,  -0.4481f,-0.5994f,-0.6632f,  -0.3285f,-0.6209f,-0.7117f,  -0.3207f,-0.6302f,-0.7071f,  -0.1954f,-0.6399f,-0.7432f,  -0.1867f,-0.6484f,-0.7381f,  -0.0580f,-0.6458f,-0.7613f,  -0.0485f,-0.6531f,-0.7557f,  0.0810f,-0.6380f,-0.7658f,  
	0.0910f,-0.6438f,-0.7598f,  -0.0558f,-0.6208f,-0.7820f,  -0.0485f,-0.6153f,-0.7868f,  0.0815f,-0.6249f,-0.7764f,  0.0882f,-0.6184f,-0.7809f,  0.2169f,-0.6162f,-0.7571f,  0.2229f,-0.6090f,-0.7612f,  0.3477f,-0.5954f,-0.7243f,  0.3529f,-0.5877f,-0.7281f,  0.4715f,-0.5632f,-0.6786f,  0.4760f,-0.5553f,-0.6820f,  0.5863f,-0.5206f,-0.6207f,  0.5900f,-0.5127f,-0.6237f,  0.6899f,-0.4688f,-0.5515f,  0.6929f,-0.4611f,-0.5543f,  
	0.7807f,-0.4090f,-0.4724f,  0.7830f,-0.4016f,-0.4749f,  0.8571f,-0.3426f,-0.3847f,  0.8589f,-0.3356f,-0.3869f,  0.9180f,-0.2708f,-0.2899f,  0.9192f,-0.2644f,-0.2919f,  0.9623f,-0.1952f,-0.1896f,  0.9630f,-0.1894f,-0.1915f,  0.9894f,-0.1171f,-0.0857f,  0.9899f,-0.1119f,-0.0875f,  0.9991f,-0.0379f,0.0201f,  0.9993f,-0.0334f,0.0184f,  0.9912f,0.0408f,0.1259f,  0.9912f,0.0448f,0.1244f,  0.9660f,0.1179f,0.2299f,  
	0.9660f,0.1213f,0.2285f,  0.9241f,0.1919f,0.3304f,  0.9240f,0.1948f,0.3291f,  0.8663f,0.2617f,0.4255f,  0.8661f,0.2642f,0.4243f,  0.7935f,0.3262f,0.5137f,  0.7934f,0.3283f,0.5126f,  0.7072f,0.3842f,0.5935f,  0.7071f,0.3860f,0.5925f,  0.6088f,0.4350f,0.6635f,  0.6087f,0.4365f,0.6625f,  0.4999f,0.4776f,0.7225f,  0.4999f,0.4789f,0.7216f,  0.3826f,0.5114f,0.7695f,  0.3826f,0.5125f,0.7687f,  
	0.2587f,0.5359f,0.8037f,  0.2587f,0.5369f,0.8030f,  0.1304f,0.5506f,0.8245f,  0.1304f,0.5515f,0.8239f,  -0.0001f,0.5553f,0.8316f,  -0.0002f,0.5560f,0.8312f,  -0.1307f,0.5499f,0.8249f,  -0.1309f,0.5505f,0.8245f,  -0.2592f,0.5345f,0.8044f,  -0.2595f,0.5348f,0.8042f,  -0.3833f,0.5092f,0.7706f,  -0.3837f,0.5092f,0.7704f,  -0.5009f,0.4743f,0.7240f,  -0.5014f,0.4740f,0.7239f,  -0.6100f,0.4304f,0.6653f,  
	-0.6106f,0.4296f,0.6653f,  -0.7086f,0.3781f,0.5957f,  -0.7093f,0.3768f,0.5958f,  -0.7951f,0.3182f,0.5163f,  -0.7958f,0.3162f,0.5164f,  -0.8678f,0.2516f,0.4284f,  -0.8685f,0.2489f,0.4287f,  -0.9255f,0.1793f,0.3337f,  -0.9260f,0.1758f,0.3341f,  -0.9669f,0.1025f,0.2337f,  -0.9672f,0.0981f,0.2342f,  -0.9912f,0.0225f,0.1302f,  -0.9912f,0.0172f,0.1309f,  -0.9979f,-0.0593f,0.0250f,  -0.9975f,-0.0656f,0.0259f,  
	-0.9867f,-0.1416f,-0.0800f,  -0.9857f,-0.1487f,-0.0789f,  -0.9576f,-0.2227f,-0.1830f,  -0.9559f,-0.2308f,-0.1816f,  -0.9109f,-0.3012f,-0.2822f,  -0.9084f,-0.3100f,-0.2804f,  -0.8473f,-0.3754f,-0.3757f,  -0.8439f,-0.3850f,-0.3736f,  -0.7678f,-0.4439f,-0.4621f,  -0.7634f,-0.4539f,-0.4595f,  -0.6736f,-0.5050f,-0.5396f,  -0.6682f,-0.5154f,-0.5365f,  -0.5664f,-0.5573f,-0.6071f,  -0.5600f,-0.5677f,-0.6034f,  -0.4481f,-0.5994f,-0.6632f,  
	-0.4406f,-0.6096f,-0.6590f,  -0.3207f,-0.6302f,-0.7071f,  -0.3121f,-0.6398f,-0.7023f,  -0.1867f,-0.6484f,-0.7381f,  -0.1772f,-0.6571f,-0.7327f,  -0.0485f,-0.6531f,-0.7557f,  -0.0383f,-0.6606f,-0.7498f,  0.0910f,-0.6438f,-0.7598f,  0.1018f,-0.6496f,-0.7534f,  -0.0485f,-0.6153f,-0.7868f,  -0.0419f,-0.6101f,-0.7912f,  0.0882f,-0.6184f,-0.7809f,  0.0943f,-0.6122f,-0.7850f,  0.2229f,-0.6090f,-0.7612f,  0.2283f,-0.6021f,-0.7651f,  
	0.3529f,-0.5877f,-0.7281f,  0.3576f,-0.5804f,-0.7316f,  0.4760f,-0.5553f,-0.6820f,  0.4800f,-0.5478f,-0.6852f,  0.5900f,-0.5127f,-0.6237f,  0.5933f,-0.5053f,-0.6267f,  0.6929f,-0.4611f,-0.5543f,  0.6956f,-0.4538f,-0.5570f,  0.7830f,-0.4016f,-0.4749f,  0.7851f,-0.3947f,-0.4774f,  0.8589f,-0.3356f,-0.3869f,  0.8603f,-0.3292f,-0.3892f,  0.9192f,-0.2644f,-0.2919f,  0.9202f,-0.2584f,-0.2940f,  0.9630f,-0.1894f,-0.1915f,  
	0.9637f,-0.1839f,-0.1935f,  0.9899f,-0.1119f,-0.0875f,  0.9902f,-0.1070f,-0.0893f,  0.9993f,-0.0334f,0.0184f,  0.9994f,-0.0291f,0.0167f,  0.9912f,0.0448f,0.1244f,  0.9912f,0.0486f,0.1227f,  0.9660f,0.1213f,0.2285f,  0.9659f,0.1246f,0.2270f,  0.9240f,0.1948f,0.3291f,  0.9239f,0.1977f,0.3276f,  0.8661f,0.2642f,0.4243f,  0.8660f,0.2667f,0.4229f,  0.7934f,0.3283f,0.5126f,  0.7933f,0.3304f,0.5113f,  
	0.7071f,0.3860f,0.5925f,  0.7071f,0.3879f,0.5912f,  0.6087f,0.4365f,0.6625f,  0.6087f,0.4382f,0.6614f,  0.4999f,0.4789f,0.7216f,  0.4999f,0.4804f,0.7206f,  0.3826f,0.5125f,0.7687f,  0.3826f,0.5140f,0.7678f,  0.2587f,0.5369f,0.8030f,  0.2587f,0.5382f,0.8022f,  0.1304f,0.5515f,0.8239f,  0.1303f,0.5526f,0.8232f,  -0.0002f,0.5560f,0.8312f,  -0.0004f,0.5570f,0.8305f,  -0.1309f,0.5505f,0.8245f,  
	-0.1312f,0.5513f,0.8239f,  -0.2595f,0.5348f,0.8042f,  -0.2599f,0.5354f,0.8036f,  -0.3837f,0.5092f,0.7704f,  -0.3842f,0.5094f,0.7700f,  -0.5014f,0.4740f,0.7239f,  -0.5021f,0.4738f,0.7235f,  -0.6106f,0.4296f,0.6653f,  -0.6114f,0.4289f,0.6650f,  -0.7093f,0.3768f,0.5958f,  -0.7102f,0.3755f,0.5956f,  -0.7958f,0.3162f,0.5164f,  -0.7967f,0.3142f,0.5163f,  -0.8685f,0.2489f,0.4287f,  -0.8693f,0.2461f,0.4287f,  
	-0.9260f,0.1758f,0.3341f,  -0.9267f,0.1721f,0.3342f,  -0.9672f,0.0981f,0.2342f,  -0.9676f,0.0935f,0.2345f,  -0.9912f,0.0172f,0.1309f,  -0.9913f,0.0116f,0.1314f,  -0.9975f,-0.0656f,0.0259f,  -0.9970f,-0.0722f,0.0266f,  -0.9857f,-0.1487f,-0.0789f,  -0.9846f,-0.1564f,-0.0779f,  -0.9559f,-0.2308f,-0.1816f,  -0.9541f,-0.2394f,-0.1802f,  -0.9084f,-0.3100f,-0.2804f,  -0.9057f,-0.3195f,-0.2787f,  -0.8439f,-0.3850f,-0.3736f,  
	-0.8402f,-0.3951f,-0.3714f,  -0.7634f,-0.4539f,-0.4595f,  -0.7586f,-0.4646f,-0.4568f,  -0.6682f,-0.5154f,-0.5365f,  -0.6623f,-0.5263f,-0.5333f,  -0.5600f,-0.5677f,-0.6034f,  -0.5529f,-0.5787f,-0.5996f,  -0.4406f,-0.6096f,-0.6590f,  -0.4323f,-0.6203f,-0.6545f,  -0.3121f,-0.6398f,-0.7023f,  -0.3028f,-0.6498f,-0.6972f,  -0.1772f,-0.6571f,-0.7327f,  -0.1669f,-0.6660f,-0.7270f,  -0.0383f,-0.6606f,-0.7498f,  -0.0272f,-0.6681f,-0.7436f,  
	0.1018f,-0.6496f,-0.7534f,  0.1134f,-0.6553f,-0.7468f,  -0.0419f,-0.6101f,-0.7912f,  -0.0359f,-0.6050f,-0.7954f,  0.0943f,-0.6122f,-0.7850f,  0.0997f,-0.6063f,-0.7890f,  0.2283f,-0.6021f,-0.7651f,  0.2331f,-0.5956f,-0.7687f,  0.3576f,-0.5804f,-0.7316f,  0.3618f,-0.5735f,-0.7350f,  0.4800f,-0.5478f,-0.6852f,  0.4835f,-0.5408f,-0.6883f,  0.5933f,-0.5053f,-0.6267f,  0.5962f,-0.4983f,-0.6295f,  0.6956f,-0.4538f,-0.5570f,  
	0.6978f,-0.4470f,-0.5596f,  0.7851f,-0.3947f,-0.4774f,  0.7868f,-0.3882f,-0.4798f,  0.8603f,-0.3292f,-0.3892f,  0.8616f,-0.3231f,-0.3915f,  0.9202f,-0.2584f,-0.2940f,  0.9211f,-0.2528f,-0.2962f,  0.9637f,-0.1839f,-0.1935f,  0.9643f,-0.1788f,-0.1955f,  0.9902f,-0.1070f,-0.0893f,  0.9905f,-0.1024f,-0.0912f,  0.9994f,-0.0291f,0.0167f,  0.9996f,-0.0250f,0.0149f,  0.9912f,0.0486f,0.1227f,  0.9913f,0.0522f,0.1210f,  
	0.9659f,0.1246f,0.2270f,  0.9659f,0.1278f,0.2253f,  0.9239f,0.1977f,0.3276f,  0.9239f,0.2005f,0.3260f,  0.8660f,0.2667f,0.4229f,  0.8660f,0.2692f,0.4214f,  0.7933f,0.3304f,0.5113f,  0.7933f,0.3327f,0.5098f,  0.7071f,0.3879f,0.5912f,  0.7071f,0.3900f,0.5899f,  0.6087f,0.4382f,0.6614f,  0.6087f,0.4401f,0.6601f,  0.4999f,0.4804f,0.7206f,  0.4999f,0.4823f,0.7194f,  0.3826f,0.5140f,0.7678f,  
	0.3826f,0.5157f,0.7666f,  0.2587f,0.5382f,0.8022f,  0.2586f,0.5397f,0.8011f,  0.1303f,0.5526f,0.8232f,  0.1301f,0.5541f,0.8222f,  -0.0004f,0.5570f,0.8305f,  -0.0007f,0.5583f,0.8296f,  -0.1312f,0.5513f,0.8239f,  -0.1317f,0.5524f,0.8231f,  -0.2599f,0.5354f,0.8036f,  -0.2605f,0.5362f,0.8029f,  -0.3842f,0.5094f,0.7700f,  -0.3850f,0.5098f,0.7693f,  -0.5021f,0.4738f,0.7235f,  -0.5029f,0.4737f,0.7229f,  
	-0.6114f,0.4289f,0.6650f,  -0.6124f,0.4283f,0.6645f,  -0.7102f,0.3755f,0.5956f,  -0.7112f,0.3742f,0.5951f,  -0.7967f,0.3142f,0.5163f,  -0.7977f,0.3121f,0.5160f,  -0.8693f,0.2461f,0.4287f,  -0.8703f,0.2431f,0.4285f,  -0.9267f,0.1721f,0.3342f,  -0.9274f,0.1681f,0.3341f,  -0.9676f,0.0935f,0.2345f,  -0.9681f,0.0885f,0.2346f,  -0.9913f,0.0116f,0.1314f,  -0.9913f,0.0055f,0.1316f,  -0.9970f,-0.0722f,0.0266f,  
	-0.9965f,-0.0794f,0.0272f,  -0.9846f,-0.1564f,-0.0779f,  -0.9833f,-0.1646f,-0.0770f,  -0.9541f,-0.2394f,-0.1802f,  -0.9519f,-0.2486f,-0.1790f,  -0.9057f,-0.3195f,-0.2787f,  -0.9026f,-0.3296f,-0.2770f,  -0.8402f,-0.3951f,-0.3714f,  -0.8360f,-0.4059f,-0.3692f,  -0.7586f,-0.4646f,-0.4568f,  -0.7533f,-0.4759f,-0.4540f,  -0.6623f,-0.5263f,-0.5333f,  -0.6557f,-0.5378f,-0.5299f,  -0.5529f,-0.5787f,-0.5996f,  -0.5450f,-0.5902f,-0.5955f,  
	-0.4323f,-0.6203f,-0.6545f,  -0.4233f,-0.6314f,-0.6498f,  -0.3028f,-0.6498f,-0.6972f,  -0.2927f,-0.6601f,-0.6918f,  -0.1669f,-0.6660f,-0.7270f,  -0.1559f,-0.6752f,-0.7210f,  -0.0272f,-0.6681f,-0.7436f,  -0.0154f,-0.6757f,-0.7370f,  0.1134f,-0.6553f,-0.7468f,  0.1256f,-0.6610f,-0.7398f,  -0.0359f,-0.6050f,-0.7954f,  -0.0306f,-0.6001f,-0.7994f,  0.0997f,-0.6063f,-0.7890f,  0.1046f,-0.6006f,-0.7927f,  0.2331f,-0.5956f,-0.7687f,  
	0.2374f,-0.5894f,-0.7722f,  0.3618f,-0.5735f,-0.7350f,  0.3655f,-0.5670f,-0.7382f,  0.4835f,-0.5408f,-0.6883f,  0.4866f,-0.5342f,-0.6913f,  0.5962f,-0.4983f,-0.6295f,  0.5987f,-0.4917f,-0.6323f,  0.6978f,-0.4470f,-0.5596f,  0.6998f,-0.4407f,-0.5622f,  0.7868f,-0.3882f,-0.4798f,  0.7883f,-0.3822f,-0.4823f,  0.8616f,-0.3231f,-0.3915f,  0.8626f,-0.3174f,-0.3938f,  0.9211f,-0.2528f,-0.2962f,  0.9218f,-0.2476f,-0.2984f,  
	0.9643f,-0.1788f,-0.1955f,  0.9647f,-0.1741f,-0.1976f,  0.9905f,-0.1024f,-0.0912f,  0.9908f,-0.0981f,-0.0932f,  0.9996f,-0.0250f,0.0149f,  0.9997f,-0.0211f,0.0129f,  0.9913f,0.0522f,0.1210f,  0.9913f,0.0557f,0.1191f,  0.9659f,0.1278f,0.2253f,  0.9659f,0.1310f,0.2235f,  0.9239f,0.2005f,0.3260f,  0.9238f,0.2034f,0.3243f,  0.8660f,0.2692f,0.4214f,  0.8660f,0.2718f,0.4197f,  0.7933f,0.3327f,0.5098f,  
	0.7933f,0.3352f,0.5083f,  0.7071f,0.3900f,0.5899f,  0.7071f,0.3923f,0.5884f,  0.6087f,0.4401f,0.6601f,  0.6087f,0.4423f,0.6587f,  0.4999f,0.4823f,0.7194f,  0.4999f,0.4843f,0.7180f,  0.3826f,0.5157f,0.7666f,  0.3825f,0.5176f,0.7654f,  0.2586f,0.5397f,0.8011f,  0.2584f,0.5416f,0.7999f,  0.1301f,0.5541f,0.8222f,  0.1298f,0.5558f,0.8211f,  -0.0007f,0.5583f,0.8296f,  -0.0012f,0.5599f,0.8285f,  
	-0.1317f,0.5524f,0.8231f,  -0.1323f,0.5537f,0.8221f,  -0.2605f,0.5362f,0.8029f,  -0.2613f,0.5372f,0.8020f,  -0.3850f,0.5098f,0.7693f,  -0.3859f,0.5104f,0.7684f,  -0.5029f,0.4737f,0.7229f,  -0.5041f,0.4738f,0.7221f,  -0.6124f,0.4283f,0.6645f,  -0.6136f,0.4277f,0.6638f,  -0.7112f,0.3742f,0.5951f,  -0.7125f,0.3728f,0.5944f,  -0.7977f,0.3121f,0.5160f,  -0.7989f,0.3099f,0.5154f,  -0.8703f,0.2431f,0.4285f,  
	-0.8714f,0.2399f,0.4280f,  -0.9274f,0.1681f,0.3341f,  -0.9283f,0.1639f,0.3337f,  -0.9681f,0.0885f,0.2346f,  -0.9686f,0.0832f,0.2344f,  -0.9913f,0.0055f,0.1316f,  -0.9913f,-0.0010f,0.1317f,  -0.9965f,-0.0794f,0.0272f,  -0.9958f,-0.0870f,0.0275f,  -0.9833f,-0.1646f,-0.0770f,  -0.9819f,-0.1734f,-0.0763f,  -0.9519f,-0.2486f,-0.1790f,  -0.9495f,-0.2584f,-0.1778f,  -0.9026f,-0.3296f,-0.2770f,  -0.8991f,-0.3403f,-0.2753f,  
	-0.8360f,-0.4059f,-0.3692f,  -0.8314f,-0.4173f,-0.3670f,  -0.7533f,-0.4759f,-0.4540f,  -0.7473f,-0.4878f,-0.4511f,  -0.6557f,-0.5378f,-0.5299f,  -0.6485f,-0.5500f,-0.5263f,  -0.5450f,-0.5902f,-0.5955f,  -0.5365f,-0.6022f,-0.5912f,  -0.4233f,-0.6314f,-0.6498f,  -0.4135f,-0.6429f,-0.6448f,  -0.2927f,-0.6601f,-0.6918f,  -0.2818f,-0.6707f,-0.6861f,  -0.1559f,-0.6752f,-0.7210f,  -0.1439f,-0.6845f,-0.7146f,  -0.0154f,-0.6757f,-0.7370f,  
	-0.0028f,-0.6834f,-0.7301f,  0.1256f,-0.6610f,-0.7398f,  0.1386f,-0.6666f,-0.7324f,  -0.0306f,-0.6001f,-0.7994f,  -0.0257f,-0.5953f,-0.8031f,  0.1046f,-0.6006f,-0.7927f,  0.1089f,-0.5952f,-0.7961f,  0.2374f,-0.5894f,-0.7722f,  0.2411f,-0.5836f,-0.7754f,  0.3655f,-0.5670f,-0.7382f,  0.3687f,-0.5609f,-0.7413f,  0.4866f,-0.5342f,-0.6913f,  0.4892f,-0.5280f,-0.6942f,  0.5987f,-0.4917f,-0.6323f,  0.6008f,-0.4856f,-0.6350f,  
	0.6998f,-0.4407f,-0.5622f,  0.7014f,-0.4348f,-0.5648f,  0.7883f,-0.3822f,-0.4823f,  0.7895f,-0.3766f,-0.4847f,  0.8626f,-0.3174f,-0.3938f,  0.8635f,-0.3121f,-0.3961f,  0.9218f,-0.2476f,-0.2984f,  0.9224f,-0.2427f,-0.3006f,  0.9647f,-0.1741f,-0.1976f,  0.9651f,-0.1695f,-0.1998f,  0.9908f,-0.0981f,-0.0932f,  0.9910f,-0.0940f,-0.0953f,  0.9997f,-0.0211f,0.0129f,  0.9998f,-0.0173f,0.0109f,  0.9913f,0.0557f,0.1191f,  
	0.9914f,0.0591f,0.1172f,  0.9659f,0.1310f,0.2235f,  0.9659f,0.1341f,0.2216f,  0.9238f,0.2034f,0.3243f,  0.9238f,0.2063f,0.3224f,  0.8660f,0.2718f,0.4197f,  0.8660f,0.2746f,0.4179f,  0.7933f,0.3352f,0.5083f,  0.7933f,0.3377f,0.5065f,  0.7071f,0.3923f,0.5884f,  0.7071f,0.3948f,0.5867f,  0.6087f,0.4423f,0.6587f,  0.6087f,0.4447f,0.6571f,  0.4999f,0.4843f,0.7180f,  0.4998f,0.4867f,0.7165f,  
	0.3825f,0.5176f,0.7654f,  0.3823f,0.5199f,0.7639f,  0.2584f,0.5416f,0.7999f,  0.2581f,0.5438f,0.7985f,  0.1298f,0.5558f,0.8211f,  0.1293f,0.5579f,0.8198f,  -0.0012f,0.5599f,0.8285f,  -0.0018f,0.5618f,0.8273f,  -0.1323f,0.5537f,0.8221f,  -0.1331f,0.5553f,0.8209f,  -0.2613f,0.5372f,0.8020f,  -0.2623f,0.5384f,0.8008f,  -0.3859f,0.5104f,0.7684f,  -0.3872f,0.5112f,0.7673f,  -0.5041f,0.4738f,0.7221f,  
	-0.5054f,0.4740f,0.7210f,  -0.6136f,0.4277f,0.6638f,  -0.6150f,0.4272f,0.6627f,  -0.7125f,0.3728f,0.5944f,  -0.7140f,0.3715f,0.5935f,  -0.7989f,0.3099f,0.5154f,  -0.8004f,0.3076f,0.5145f,  -0.8714f,0.2399f,0.4280f,  -0.8727f,0.2365f,0.4272f,  -0.9283f,0.1639f,0.3337f,  -0.9293f,0.1594f,0.3332f,  -0.9686f,0.0832f,0.2344f,  -0.9691f,0.0774f,0.2340f,  -0.9913f,-0.0010f,0.1317f,  -0.9913f,-0.0080f,0.1316f,  
	-0.9958f,-0.0870f,0.0275f,  -0.9951f,-0.0952f,0.0277f,  -0.9819f,-0.1734f,-0.0763f,  -0.9802f,-0.1828f,-0.0756f,  -0.9495f,-0.2584f,-0.1778f,  -0.9468f,-0.2688f,-0.1768f,  -0.8991f,-0.3403f,-0.2753f,  -0.8952f,-0.3516f,-0.2737f,  -0.8314f,-0.4173f,-0.3670f,  -0.8262f,-0.4294f,-0.3647f,  -0.7473f,-0.4878f,-0.4511f,  -0.7408f,-0.5004f,-0.4481f,  -0.6485f,-0.5500f,-0.5263f,  -0.6406f,-0.5627f,-0.5226f,  -0.5365f,-0.6022f,-0.5912f,  
	-0.5272f,-0.6147f,-0.5867f,  -0.4135f,-0.6429f,-0.6448f,  -0.4029f,-0.6548f,-0.6394f,  -0.2818f,-0.6707f,-0.6861f,  -0.2700f,-0.6817f,-0.6800f,  -0.1439f,-0.6845f,-0.7146f,  -0.1312f,-0.6940f,-0.7079f,  -0.0028f,-0.6834f,-0.7301f,  0.0105f,-0.6910f,-0.7227f,  0.1386f,-0.6666f,-0.7324f,  0.1524f,-0.6719f,-0.7248f,  -0.0257f,-0.5953f,-0.8031f,  -0.0214f,-0.5908f,-0.8065f,  0.1089f,-0.5952f,-0.7961f,  0.1127f,-0.5901f,-0.7994f,  
	0.2411f,-0.5836f,-0.7754f,  0.2444f,-0.5781f,-0.7785f,  0.3687f,-0.5609f,-0.7413f,  0.3714f,-0.5552f,-0.7442f,  0.4892f,-0.5280f,-0.6942f,  0.4915f,-0.5222f,-0.6969f,  0.6008f,-0.4856f,-0.6350f,  0.6026f,-0.4799f,-0.6377f,  0.7014f,-0.4348f,-0.5648f,  0.7028f,-0.4293f,-0.5673f,  0.7895f,-0.3766f,-0.4847f,  0.7905f,-0.3713f,-0.4871f,  0.8635f,-0.3121f,-0.3961f,  0.8642f,-0.3072f,-0.3985f,  0.9224f,-0.2427f,-0.3006f,  
	0.9228f,-0.2381f,-0.3029f,  0.9651f,-0.1695f,-0.1998f,  0.9653f,-0.1653f,-0.2020f,  0.9910f,-0.0940f,-0.0953f,  0.9912f,-0.0901f,-0.0975f,  0.9998f,-0.0173f,0.0109f,  0.9999f,-0.0137f,0.0088f,  0.9914f,0.0591f,0.1172f,  0.9914f,0.0625f,0.1151f,  0.9659f,0.1341f,0.2216f,  0.9659f,0.1372f,0.2196f,  0.9238f,0.2063f,0.3224f,  0.9239f,0.2093f,0.3205f,  0.8660f,0.2746f,0.4179f,  0.8660f,0.2774f,0.4160f,  
	0.7933f,0.3377f,0.5065f,  0.7933f,0.3405f,0.5047f,  0.7071f,0.3948f,0.5867f,  0.7070f,0.3975f,0.5849f,  0.6087f,0.4447f,0.6571f,  0.6086f,0.4474f,0.6554f,  0.4998f,0.4867f,0.7165f,  0.4996f,0.4893f,0.7148f,  0.3823f,0.5199f,0.7639f,  0.3820f,0.5225f,0.7623f,  0.2581f,0.5438f,0.7985f,  0.2576f,0.5463f,0.7970f,  0.1293f,0.5579f,0.8198f,  0.1287f,0.5603f,0.8183f,  -0.0018f,0.5618f,0.8273f,  
	-0.0027f,0.5639f,0.8258f,  -0.1331f,0.5553f,0.8209f,  -0.1342f,0.5571f,0.8195f,  -0.2623f,0.5384f,0.8008f,  -0.2636f,0.5398f,0.7994f,  -0.3872f,0.5112f,0.7673f,  -0.3886f,0.5121f,0.7660f,  -0.5054f,0.4740f,0.7210f,  -0.5071f,0.4742f,0.7197f,  -0.6150f,0.4272f,0.6627f,  -0.6168f,0.4266f,0.6615f,  -0.7140f,0.3715f,0.5935f,  -0.7157f,0.3700f,0.5923f,  -0.8004f,0.3076f,0.5145f,  -0.8021f,0.3051f,0.5134f,  
	-0.8727f,0.2365f,0.4272f,  -0.8741f,0.2329f,0.4263f,  -0.9293f,0.1594f,0.3332f,  -0.9304f,0.1545f,0.3324f,  -0.9691f,0.0774f,0.2340f,  -0.9698f,0.0712f,0.2335f,  -0.9913f,-0.0080f,0.1316f,  -0.9912f,-0.0154f,0.1313f,  -0.9951f,-0.0952f,0.0277f,  -0.9942f,-0.1040f,0.0278f,  -0.9802f,-0.1828f,-0.0756f,  -0.9784f,-0.1928f,-0.0751f,  -0.9468f,-0.2688f,-0.1768f,  -0.9438f,-0.2799f,-0.1757f,  -0.8952f,-0.3516f,-0.2737f,  
	-0.8909f,-0.3637f,-0.2720f,  -0.8262f,-0.4294f,-0.3647f,  -0.8205f,-0.4422f,-0.3623f,  -0.7408f,-0.5004f,-0.4481f,  -0.7337f,-0.5136f,-0.4450f,  -0.6406f,-0.5627f,-0.5226f,  -0.6319f,-0.5760f,-0.5186f,  -0.5272f,-0.6147f,-0.5867f,  -0.5171f,-0.6277f,-0.5819f,  -0.4029f,-0.6548f,-0.6394f,  -0.3915f,-0.6671f,-0.6338f,  -0.2700f,-0.6817f,-0.6800f,  -0.2574f,-0.6929f,-0.6735f,  -0.1312f,-0.6940f,-0.7079f,  -0.1177f,-0.7037f,-0.7007f,  
	0.0105f,-0.6910f,-0.7227f,  0.0248f,-0.6986f,-0.7151f,  0.1524f,-0.6719f,-0.7248f,  0.1668f,-0.6771f,-0.7167f,  -0.0214f,-0.5908f,-0.8065f,  -0.0177f,-0.5866f,-0.8097f,  0.1127f,-0.5901f,-0.7994f,  0.1160f,-0.5853f,-0.8025f,  0.2444f,-0.5781f,-0.7785f,  0.2473f,-0.5729f,-0.7814f,  0.3714f,-0.5552f,-0.7442f,  0.3738f,-0.5499f,-0.7469f,  0.4915f,-0.5222f,-0.6969f,  0.4934f,-0.5168f,-0.6996f,  0.6026f,-0.4799f,-0.6377f,  
	0.6041f,-0.4746f,-0.6402f,  0.7028f,-0.4293f,-0.5673f,  0.7039f,-0.4241f,-0.5698f,  0.7905f,-0.3713f,-0.4871f,  0.7913f,-0.3664f,-0.4895f,  0.8642f,-0.3072f,-0.3985f,  0.8648f,-0.3026f,-0.4008f,  0.9228f,-0.2381f,-0.3029f,  0.9232f,-0.2338f,-0.3051f,  0.9653f,-0.1653f,-0.2020f,  0.9656f,-0.1613f,-0.2042f,  0.9912f,-0.0901f,-0.0975f,  0.9913f,-0.0863f,-0.0997f,  0.9999f,-0.0137f,0.0088f,  0.9999f,-0.0102f,0.0067f,  
	0.9914f,0.0625f,0.1151f,  0.9914f,0.0658f,0.1130f,  0.9659f,0.1372f,0.2196f,  0.9659f,0.1404f,0.2175f,  0.9239f,0.2093f,0.3205f,  0.9239f,0.2123f,0.3184f,  0.8660f,0.2774f,0.4160f,  0.8660f,0.2804f,0.4140f,  0.7933f,0.3405f,0.5047f,  0.7933f,0.3434f,0.5027f,  0.7070f,0.3975f,0.5849f,  0.7069f,0.4004f,0.5830f,  0.6086f,0.4474f,0.6554f,  0.6084f,0.4503f,0.6535f,  0.4996f,0.4893f,0.7148f,  
	0.4993f,0.4922f,0.7130f,  0.3820f,0.5225f,0.7623f,  0.3815f,0.5254f,0.7605f,  0.2576f,0.5463f,0.7970f,  0.2570f,0.5491f,0.7953f,  0.1287f,0.5603f,0.8183f,  0.1278f,0.5629f,0.8166f,  -0.0027f,0.5639f,0.8258f,  -0.0038f,0.5664f,0.8241f,  -0.1342f,0.5571f,0.8195f,  -0.1356f,0.5592f,0.8178f,  -0.2636f,0.5398f,0.7994f,  -0.2652f,0.5415f,0.7978f,  -0.3886f,0.5121f,0.7660f,  -0.3905f,0.5131f,0.7644f,  
	-0.5071f,0.4742f,0.7197f,  -0.5090f,0.4745f,0.7181f,  -0.6168f,0.4266f,0.6615f,  -0.6188f,0.4261f,0.6599f,  -0.7157f,0.3700f,0.5923f,  -0.7178f,0.3684f,0.5908f,  -0.8021f,0.3051f,0.5134f,  -0.8040f,0.3024f,0.5120f,  -0.8741f,0.2329f,0.4263f,  -0.8758f,0.2289f,0.4250f,  -0.9304f,0.1545f,0.3324f,  -0.9316f,0.1492f,0.3313f,  -0.9698f,0.0712f,0.2335f,  -0.9704f,0.0646f,0.2326f,  -0.9912f,-0.0154f,0.1313f,  
	-0.9911f,-0.0235f,0.1309f,  -0.9942f,-0.1040f,0.0278f,  -0.9932f,-0.1134f,0.0278f,  -0.9784f,-0.1928f,-0.0751f,  -0.9762f,-0.2034f,-0.0747f,  -0.9438f,-0.2799f,-0.1757f,  -0.9404f,-0.2918f,-0.1746f,  -0.8909f,-0.3637f,-0.2720f,  -0.8861f,-0.3765f,-0.2702f,  -0.8205f,-0.4422f,-0.3623f,  -0.8142f,-0.4557f,-0.3598f,  -0.7337f,-0.5136f,-0.4450f,  -0.7258f,-0.5274f,-0.4416f,  -0.6319f,-0.5760f,-0.5186f,  -0.6225f,-0.5899f,-0.5143f,  
	-0.5171f,-0.6277f,-0.5819f,  -0.5062f,-0.6412f,-0.5767f,  -0.3915f,-0.6671f,-0.6338f,  -0.3792f,-0.6798f,-0.6277f,  -0.2574f,-0.6929f,-0.6735f,  -0.2440f,-0.7043f,-0.6667f,  -0.1177f,-0.7037f,-0.7007f,  -0.1033f,-0.7134f,-0.6931f,  0.0248f,-0.6986f,-0.7151f,  0.0397f,-0.7062f,-0.7069f,  0.1668f,-0.6771f,-0.7167f,  0.1820f,-0.6820f,-0.7083f,  -0.0177f,-0.5866f,-0.8097f,  -0.0144f,-0.5826f,-0.8127f,  0.1160f,-0.5853f,-0.8025f,  
	0.1189f,-0.5808f,-0.8053f,  0.2473f,-0.5729f,-0.7814f,  0.2497f,-0.5681f,-0.7841f,  0.3738f,-0.5499f,-0.7469f,  0.3758f,-0.5449f,-0.7496f,  0.4934f,-0.5168f,-0.6996f,  0.4950f,-0.5118f,-0.7021f,  0.6041f,-0.4746f,-0.6402f,  0.6053f,-0.4697f,-0.6427f,  0.7039f,-0.4241f,-0.5698f,  0.7048f,-0.4194f,-0.5722f,  0.7913f,-0.3664f,-0.4895f,  0.7919f,-0.3618f,-0.4918f,  0.8648f,-0.3026f,-0.4008f,  0.8652f,-0.2982f,-0.4031f,  
	0.9232f,-0.2338f,-0.3051f,  0.9234f,-0.2297f,-0.3074f,  0.9656f,-0.1613f,-0.2042f,  0.9657f,-0.1574f,-0.2064f,  0.9913f,-0.0863f,-0.0997f,  0.9914f,-0.0827f,-0.1019f,  0.9999f,-0.0102f,0.0067f,  1.0000f,-0.0067f,0.0045f,  0.9914f,0.0658f,0.1130f,  0.9914f,0.0691f,0.1108f,  0.9659f,0.1404f,0.2175f,  0.9659f,0.1436f,0.2154f,  0.9239f,0.2123f,0.3184f,  0.9239f,0.2155f,0.3163f,  0.8660f,0.2804f,0.4140f,  
	0.8660f,0.2835f,0.4120f,  0.7933f,0.3434f,0.5027f,  0.7932f,0.3466f,0.5007f,  0.7069f,0.4004f,0.5830f,  0.7068f,0.4036f,0.5810f,  0.6084f,0.4503f,0.6535f,  0.6081f,0.4535f,0.6515f,  0.4993f,0.4922f,0.7130f,  0.4989f,0.4955f,0.7111f,  0.3815f,0.5254f,0.7605f,  0.3809f,0.5286f,0.7586f,  0.2570f,0.5491f,0.7953f,  0.2561f,0.5523f,0.7934f,  0.1278f,0.5629f,0.8166f,  0.1267f,0.5659f,0.8147f,  
	-0.0038f,0.5664f,0.8241f,  -0.0052f,0.5691f,0.8223f,  -0.1356f,0.5592f,0.8178f,  -0.1372f,0.5616f,0.8160f,  -0.2652f,0.5415f,0.7978f,  -0.2671f,0.5433f,0.7959f,  -0.3905f,0.5131f,0.7644f,  -0.3926f,0.5143f,0.7625f,  -0.5090f,0.4745f,0.7181f,  -0.5113f,0.4749f,0.7163f,  -0.6188f,0.4261f,0.6599f,  -0.6212f,0.4255f,0.6581f,  -0.7178f,0.3684f,0.5908f,  -0.7201f,0.3667f,0.5891f,  -0.8040f,0.3024f,0.5120f,  
	-0.8061f,0.2995f,0.5104f,  -0.8758f,0.2289f,0.4250f,  -0.8776f,0.2247f,0.4235f,  -0.9316f,0.1492f,0.3313f,  -0.9330f,0.1435f,0.3300f,  -0.9704f,0.0646f,0.2326f,  -0.9711f,0.0574f,0.2316f,  -0.9911f,-0.0235f,0.1309f,  -0.9910f,-0.0321f,0.1302f,  -0.9932f,-0.1134f,0.0278f,  -0.9920f,-0.1235f,0.0276f,  -0.9762f,-0.2034f,-0.0747f,  -0.9738f,-0.2148f,-0.0743f,  -0.9404f,-0.2918f,-0.1746f,  -0.9366f,-0.3043f,-0.1736f,  
	-0.8861f,-0.3765f,-0.2702f,  -0.8808f,-0.3900f,-0.2684f,  -0.8142f,-0.4557f,-0.3598f,  -0.8073f,-0.4698f,-0.3571f,  -0.7258f,-0.5274f,-0.4416f,  -0.7173f,-0.5419f,-0.4380f,  -0.6225f,-0.5899f,-0.5143f,  -0.6123f,-0.6043f,-0.5098f,  -0.5062f,-0.6412f,-0.5767f,  -0.4945f,-0.6552f,-0.5711f,  -0.3792f,-0.6798f,-0.6277f,  -0.3660f,-0.6929f,-0.6212f,  -0.2440f,-0.7043f,-0.6667f,  -0.2296f,-0.7159f,-0.6593f,  -0.1033f,-0.7134f,-0.6931f,  
	-0.0881f,-0.7231f,-0.6851f,  0.0397f,-0.7062f,-0.7069f,  0.0555f,-0.7135f,-0.6984f,  0.1820f,-0.6820f,-0.7083f,  0.1977f,-0.6867f,-0.6996f,  -0.0144f,-0.5826f,-0.8127f,  -0.0115f,-0.5788f,-0.8154f,  0.1189f,-0.5808f,-0.8053f,  0.1214f,-0.5766f,-0.8079f,  0.2497f,-0.5681f,-0.7841f,  0.2518f,-0.5636f,-0.7867f,  0.3758f,-0.5449f,-0.7496f,  0.3775f,-0.5403f,-0.7520f,  0.4950f,-0.5118f,-0.7021f,  0.4963f,-0.5072f,-0.7046f,  
	0.6053f,-0.4697f,-0.6427f,  0.6063f,-0.4651f,-0.6450f,  0.7048f,-0.4194f,-0.5722f,  0.7055f,-0.4149f,-0.5745f,  0.7919f,-0.3618f,-0.4918f,  0.7924f,-0.3576f,-0.4942f,  0.8652f,-0.2982f,-0.4031f,  0.8655f,-0.2942f,-0.4054f,  0.9234f,-0.2297f,-0.3074f,  0.9236f,-0.2258f,-0.3097f,  0.9657f,-0.1574f,-0.2064f,  0.9658f,-0.1538f,-0.2087f,  0.9914f,-0.0827f,-0.1019f,  0.9914f,-0.0792f,-0.1041f,  1.0000f,-0.0067f,0.0045f,  
	1.0000f,-0.0034f,0.0022f,  0.9914f,0.0691f,0.1108f,  0.9914f,0.0724f,0.1086f,  0.9659f,0.1436f,0.2154f,  0.9659f,0.1469f,0.2132f,  0.9239f,0.2155f,0.3163f,  0.9238f,0.2188f,0.3141f,  0.8660f,0.2835f,0.4120f,  0.8659f,0.2869f,0.4098f,  0.7932f,0.3466f,0.5007f,  0.7930f,0.3500f,0.4986f,  0.7068f,0.4036f,0.5810f,  0.7065f,0.4071f,0.5789f,  0.6081f,0.4535f,0.6515f,  0.6077f,0.4571f,0.6495f,  
	0.4989f,0.4955f,0.7111f,  0.4982f,0.4990f,0.7090f,  0.3809f,0.5286f,0.7586f,  0.3800f,0.5322f,0.7566f,  0.2561f,0.5523f,0.7934f,  0.2550f,0.5557f,0.7913f,  0.1267f,0.5659f,0.8147f,  0.1253f,0.5692f,0.8126f,  -0.0052f,0.5691f,0.8223f,  -0.0069f,0.5721f,0.8202f,  -0.1372f,0.5616f,0.8160f,  -0.1392f,0.5641f,0.8139f,  -0.2671f,0.5433f,0.7959f,  -0.2694f,0.5453f,0.7938f,  -0.3926f,0.5143f,0.7625f,  
	-0.3951f,0.5155f,0.7604f,  -0.5113f,0.4749f,0.7163f,  -0.5140f,0.4752f,0.7141f,  -0.6212f,0.4255f,0.6581f,  -0.6239f,0.4248f,0.6560f,  -0.7201f,0.3667f,0.5891f,  -0.7227f,0.3649f,0.5870f,  -0.8061f,0.2995f,0.5104f,  -0.8085f,0.2963f,0.5084f,  -0.8776f,0.2247f,0.4235f,  -0.8796f,0.2200f,0.4217f,  -0.9330f,0.1435f,0.3300f,  -0.9345f,0.1374f,0.3285f,  -0.9711f,0.0574f,0.2316f,  -0.9718f,0.0497f,0.2304f,  
	-0.9910f,-0.0321f,0.1302f,  -0.9907f,-0.0414f,0.1294f,  -0.9920f,-0.1235f,0.0276f,  -0.9906f,-0.1342f,0.0273f,  -0.9738f,-0.2148f,-0.0743f,  -0.9711f,-0.2270f,-0.0739f,  -0.9366f,-0.3043f,-0.1736f,  -0.9324f,-0.3176f,-0.1725f,  -0.8808f,-0.3900f,-0.2684f,  -0.8750f,-0.4042f,-0.2664f,  -0.8073f,-0.4698f,-0.3571f,  -0.7997f,-0.4847f,-0.3542f,  -0.7173f,-0.5419f,-0.4380f,  -0.7079f,-0.5571f,-0.4341f,  -0.6123f,-0.6043f,-0.5098f,  
	-0.6013f,-0.6193f,-0.5048f,  -0.4945f,-0.6552f,-0.5711f,  -0.4819f,-0.6696f,-0.5652f,  -0.3660f,-0.6929f,-0.6212f,  -0.3520f,-0.7062f,-0.6143f,  -0.2296f,-0.7159f,-0.6593f,  -0.2144f,-0.7277f,-0.6515f,  -0.0881f,-0.7231f,-0.6851f,  -0.0720f,-0.7328f,-0.6766f,  0.0555f,-0.7135f,-0.6984f,  0.0719f,-0.7207f,-0.6895f,  0.1977f,-0.6867f,-0.6996f,  0.2141f,-0.6909f,-0.6905f,  -0.0115f,-0.5788f,-0.8154f,  -0.0091f,-0.5753f,-0.8179f,  
	0.1214f,-0.5766f,-0.8079f,  0.1235f,-0.5727f,-0.8104f,  0.2518f,-0.5636f,-0.7867f,  0.2535f,-0.5595f,-0.7891f,  0.3775f,-0.5403f,-0.7520f,  0.3789f,-0.5361f,-0.7544f,  0.4963f,-0.5072f,-0.7046f,  0.4974f,-0.5030f,-0.7069f,  0.6063f,-0.4651f,-0.6450f,  0.6071f,-0.4609f,-0.6473f,  0.7055f,-0.4149f,-0.5745f,  0.7061f,-0.4109f,-0.5768f,  0.7924f,-0.3576f,-0.4942f,  0.7928f,-0.3537f,-0.4964f,  0.8655f,-0.2942f,-0.4054f,  
	0.8657f,-0.2904f,-0.4076f,  0.9236f,-0.2258f,-0.3097f,  0.9238f,-0.2222f,-0.3119f,  0.9658f,-0.1538f,-0.2087f,  0.9659f,-0.1503f,-0.2109f,  0.9914f,-0.0792f,-0.1041f,  0.9914f,-0.0758f,-0.1064f,  1.0000f,-0.0034f,0.0022f,  1.0000f,0.0000f,-0.0000f,  0.9914f,0.0724f,0.1086f,  0.9914f,0.0758f,0.1064f,  0.9659f,0.1469f,0.2132f,  0.9659f,0.1503f,0.2109f,  0.9238f,0.2188f,0.3141f,  0.9238f,0.2222f,0.3119f,  
	0.8659f,0.2869f,0.4098f,  0.8657f,0.2904f,0.4076f,  0.7930f,0.3500f,0.4986f,  0.7928f,0.3537f,0.4964f,  0.7065f,0.4071f,0.5789f,  0.7061f,0.4109f,0.5767f,  0.6077f,0.4571f,0.6495f,  0.6071f,0.4609f,0.6473f,  0.4982f,0.4990f,0.7090f,  0.4974f,0.5030f,0.7068f,  0.3800f,0.5322f,0.7566f,  0.3789f,0.5361f,0.7544f,  0.2550f,0.5557f,0.7913f,  0.2536f,0.5595f,0.7891f,  0.1253f,0.5692f,0.8126f,  
	0.1235f,0.5728f,0.8104f,  -0.0069f,0.5721f,0.8202f,  -0.0090f,0.5753f,0.8179f,  -0.1392f,0.5641f,0.8139f,  -0.1416f,0.5669f,0.8115f,  -0.2694f,0.5453f,0.7938f,  -0.2720f,0.5474f,0.7914f,  -0.3951f,0.5155f,0.7604f,  -0.3980f,0.5169f,0.7579f,  -0.5140f,0.4752f,0.7141f,  -0.5170f,0.4756f,0.7117f,  -0.6239f,0.4248f,0.6560f,  -0.6269f,0.4240f,0.6536f,  -0.7227f,0.3649f,0.5870f,  -0.7256f,0.3628f,0.5847f,  
	-0.8085f,0.2963f,0.5084f,  -0.8112f,0.2928f,0.5062f,  -0.8796f,0.2200f,0.4217f,  -0.8819f,0.2150f,0.4197f,  -0.9345f,0.1374f,0.3285f,  -0.9361f,0.1307f,0.3267f,  -0.9718f,0.0497f,0.2304f,  -0.9725f,0.0414f,0.2290f,  -0.9907f,-0.0414f,0.1294f,  -0.9904f,-0.0513f,0.1285f,  -0.9906f,-0.1342f,0.0273f,  -0.9890f,-0.1457f,0.0270f,  -0.9711f,-0.2270f,-0.0739f,  -0.9680f,-0.2398f,-0.0735f,  -0.9324f,-0.3176f,-0.1725f,  
	-0.9277f,-0.3317f,-0.1712f,  -0.8750f,-0.4042f,-0.2664f,  -0.8685f,-0.4193f,-0.2643f,  -0.7997f,-0.4847f,-0.3542f,  -0.7915f,-0.5004f,-0.3510f,  -0.7079f,-0.5571f,-0.4341f,  -0.6979f,-0.5729f,-0.4298f,  -0.6013f,-0.6193f,-0.5048f,  -0.5894f,-0.6349f,-0.4994f,  -0.4819f,-0.6696f,-0.5652f,  -0.4683f,-0.6845f,-0.5587f,  -0.3520f,-0.7062f,-0.6143f,  -0.3370f,-0.7199f,-0.6068f,  -0.2144f,-0.7277f,-0.6515f,  -0.1983f,-0.7396f,-0.6432f,  
	-0.0720f,-0.7328f,-0.6766f,  -0.0552f,-0.7425f,-0.6676f,  0.0719f,-0.7207f,-0.6895f,  0.0890f,-0.7277f,-0.6801f,  0.2141f,-0.6909f,-0.6905f,  0.2310f,-0.6948f,-0.6811f,  -0.0091f,-0.5753f,-0.8179f,  -0.0070f,-0.5720f,-0.8202f,  0.1235f,-0.5727f,-0.8104f,  0.1252f,-0.5692f,-0.8126f,  0.2535f,-0.5595f,-0.7891f,  0.2549f,-0.5557f,-0.7913f,  0.3789f,-0.5361f,-0.7544f,  0.3800f,-0.5322f,-0.7566f,  0.4974f,-0.5030f,-0.7069f,  
	0.4982f,-0.4990f,-0.7090f,  0.6071f,-0.4609f,-0.6473f,  0.6077f,-0.4571f,-0.6495f,  0.7061f,-0.4109f,-0.5768f,  0.7065f,-0.4071f,-0.5789f,  0.7928f,-0.3537f,-0.4964f,  0.7930f,-0.3500f,-0.4986f,  0.8657f,-0.2904f,-0.4076f,  0.8659f,-0.2869f,-0.4098f,  0.9238f,-0.2222f,-0.3119f,  0.9238f,-0.2188f,-0.3141f,  0.9659f,-0.1503f,-0.2109f,  0.9659f,-0.1469f,-0.2132f,  0.9914f,-0.0758f,-0.1064f,  0.9914f,-0.0724f,-0.1086f,  
	1.0000f,0.0000f,-0.0000f,  1.0000f,0.0034f,-0.0022f,  0.9914f,0.0758f,0.1064f,  0.9914f,0.0792f,0.1041f,  0.9659f,0.1503f,0.2109f,  0.9658f,0.1538f,0.2087f,  0.9238f,0.2222f,0.3119f,  0.9236f,0.2259f,0.3097f,  0.8657f,0.2904f,0.4076f,  0.8655f,0.2942f,0.4054f,  0.7928f,0.3537f,0.4964f,  0.7924f,0.3576f,0.4941f,  0.7061f,0.4109f,0.5767f,  0.7056f,0.4149f,0.5745f,  0.6071f,0.4609f,0.6473f,  
	0.6063f,0.4651f,0.6450f,  0.4974f,0.5030f,0.7068f,  0.4964f,0.5072f,0.7045f,  0.3789f,0.5361f,0.7544f,  0.3776f,0.5403f,0.7520f,  0.2536f,0.5595f,0.7891f,  0.2519f,0.5636f,0.7867f,  0.1235f,0.5728f,0.8104f,  0.1215f,0.5766f,0.8079f,  -0.0090f,0.5753f,0.8179f,  -0.0114f,0.5788f,0.8154f,  -0.1416f,0.5669f,0.8115f,  -0.1444f,0.5699f,0.8090f,  -0.2720f,0.5474f,0.7914f,  -0.2751f,0.5497f,0.7888f,  
	-0.3980f,0.5169f,0.7579f,  -0.4012f,0.5183f,0.7553f,  -0.5170f,0.4756f,0.7117f,  -0.5204f,0.4759f,0.7090f,  -0.6269f,0.4240f,0.6536f,  -0.6303f,0.4231f,0.6509f,  -0.7256f,0.3628f,0.5847f,  -0.7289f,0.3605f,0.5820f,  -0.8112f,0.2928f,0.5062f,  -0.8141f,0.2889f,0.5037f,  -0.8819f,0.2150f,0.4197f,  -0.8843f,0.2095f,0.4174f,  -0.9361f,0.1307f,0.3267f,  -0.9377f,0.1235f,0.3247f,  -0.9725f,0.0414f,0.2290f,  
	-0.9733f,0.0325f,0.2274f,  -0.9904f,-0.0513f,0.1285f,  -0.9899f,-0.0620f,0.1275f,  -0.9890f,-0.1457f,0.0270f,  -0.9871f,-0.1579f,0.0266f,  -0.9680f,-0.2398f,-0.0735f,  -0.9646f,-0.2535f,-0.0731f,  -0.9277f,-0.3317f,-0.1712f,  -0.9225f,-0.3466f,-0.1699f,  -0.8685f,-0.4193f,-0.2643f,  -0.8615f,-0.4351f,-0.2619f,  -0.7915f,-0.5004f,-0.3510f,  -0.7825f,-0.5167f,-0.3475f,  -0.6979f,-0.5729f,-0.4298f,  -0.6869f,-0.5894f,-0.4252f,  
	-0.5894f,-0.6349f,-0.4994f,  -0.5767f,-0.6510f,-0.4936f,  -0.4683f,-0.6845f,-0.5587f,  -0.4539f,-0.6997f,-0.5517f,  -0.3370f,-0.7199f,-0.6068f,  -0.3211f,-0.7337f,-0.5988f,  -0.1983f,-0.7396f,-0.6432f,  -0.1813f,-0.7515f,-0.6343f,  -0.0552f,-0.7425f,-0.6676f,  -0.0376f,-0.7520f,-0.6581f,  0.0890f,-0.7277f,-0.6801f,  0.1068f,-0.7344f,-0.6703f,  0.2310f,-0.6948f,-0.6811f,  0.2484f,-0.6982f,-0.6714f,  -0.0070f,-0.5720f,-0.8202f,  
	-0.0053f,-0.5691f,-0.8223f,  0.1252f,-0.5692f,-0.8126f,  0.1266f,-0.5659f,-0.8147f,  0.2549f,-0.5557f,-0.7913f,  0.2560f,-0.5523f,-0.7934f,  0.3800f,-0.5322f,-0.7566f,  0.3808f,-0.5286f,-0.7586f,  0.4982f,-0.4990f,-0.7090f,  0.4988f,-0.4955f,-0.7111f,  0.6077f,-0.4571f,-0.6495f,  0.6081f,-0.4535f,-0.6516f,  0.7065f,-0.4071f,-0.5789f,  0.7068f,-0.4036f,-0.5810f,  0.7930f,-0.3500f,-0.4986f,  0.7932f,-0.3466f,-0.5007f,  
	0.8659f,-0.2869f,-0.4098f,  0.8660f,-0.2835f,-0.4120f,  0.9238f,-0.2188f,-0.3141f,  0.9239f,-0.2155f,-0.3163f,  0.9659f,-0.1469f,-0.2132f,  0.9659f,-0.1436f,-0.2154f,  0.9914f,-0.0724f,-0.1086f,  0.9914f,-0.0691f,-0.1108f,  1.0000f,0.0034f,-0.0022f,  1.0000f,0.0068f,-0.0045f,  0.9914f,0.0792f,0.1041f,  0.9914f,0.0827f,0.1019f,  0.9658f,0.1538f,0.2087f,  0.9657f,0.1574f,0.2064f,  0.9236f,0.2259f,0.3097f,  
	0.9234f,0.2297f,0.3074f,  0.8655f,0.2942f,0.4054f,  0.8652f,0.2982f,0.4031f,  0.7924f,0.3576f,0.4941f,  0.7920f,0.3618f,0.4918f,  0.7056f,0.4149f,0.5745f,  0.7049f,0.4194f,0.5721f,  0.6063f,0.4651f,0.6450f,  0.6053f,0.4697f,0.6426f,  0.4964f,0.5072f,0.7045f,  0.4951f,0.5118f,0.7021f,  0.3776f,0.5403f,0.7520f,  0.3759f,0.5449f,0.7495f,  0.2519f,0.5636f,0.7867f,  0.2498f,0.5681f,0.7841f,  
	0.1215f,0.5766f,0.8079f,  0.1190f,0.5808f,0.8053f,  -0.0114f,0.5788f,0.8154f,  -0.0142f,0.5826f,0.8126f,  -0.1444f,0.5699f,0.8090f,  -0.1476f,0.5730f,0.8061f,  -0.2751f,0.5497f,0.7888f,  -0.2786f,0.5521f,0.7859f,  -0.4012f,0.5183f,0.7553f,  -0.4050f,0.5197f,0.7523f,  -0.5204f,0.4759f,0.7090f,  -0.5242f,0.4762f,0.7060f,  -0.6303f,0.4231f,0.6509f,  -0.6341f,0.4220f,0.6479f,  -0.7289f,0.3605f,0.5820f,  
	-0.7325f,0.3579f,0.5791f,  -0.8141f,0.2889f,0.5037f,  -0.8174f,0.2846f,0.5009f,  -0.8843f,0.2095f,0.4174f,  -0.8869f,0.2035f,0.4148f,  -0.9377f,0.1235f,0.3247f,  -0.9395f,0.1157f,0.3225f,  -0.9733f,0.0325f,0.2274f,  -0.9739f,0.0228f,0.2257f,  -0.9899f,-0.0620f,0.1275f,  -0.9893f,-0.0734f,0.1263f,  -0.9871f,-0.1579f,0.0266f,  -0.9849f,-0.1710f,0.0263f,  -0.9646f,-0.2535f,-0.0731f,  -0.9607f,-0.2680f,-0.0726f,  
	-0.9225f,-0.3466f,-0.1699f,  -0.9167f,-0.3623f,-0.1684f,  -0.8615f,-0.4351f,-0.2619f,  -0.8537f,-0.4516f,-0.2593f,  -0.7825f,-0.5167f,-0.3475f,  -0.7727f,-0.5337f,-0.3436f,  -0.6869f,-0.5894f,-0.4252f,  -0.6751f,-0.6064f,-0.4201f,  -0.5767f,-0.6510f,-0.4936f,  -0.5630f,-0.6676f,-0.4872f,  -0.4539f,-0.6997f,-0.5517f,  -0.4385f,-0.7153f,-0.5442f,  -0.3211f,-0.7337f,-0.5988f,  -0.3043f,-0.7477f,-0.5902f,  -0.1813f,-0.7515f,-0.6343f,  
	-0.1635f,-0.7634f,-0.6248f,  -0.0376f,-0.7520f,-0.6581f,  -0.0192f,-0.7613f,-0.6481f,  0.1068f,-0.7344f,-0.6703f,  0.1252f,-0.7407f,-0.6601f,  0.2484f,-0.6982f,-0.6714f,  0.2662f,-0.7012f,-0.6614f,  -0.0053f,-0.5691f,-0.8223f,  -0.0039f,-0.5664f,-0.8242f,  0.1266f,-0.5659f,-0.8147f,  0.1277f,-0.5629f,-0.8166f,  0.2560f,-0.5523f,-0.7934f,  0.2569f,-0.5491f,-0.7953f,  0.3808f,-0.5286f,-0.7586f,  0.3815f,-0.5254f,-0.7606f,  
	0.4988f,-0.4955f,-0.7111f,  0.4993f,-0.4922f,-0.7130f,  0.6081f,-0.4535f,-0.6516f,  0.6084f,-0.4503f,-0.6535f,  0.7068f,-0.4036f,-0.5810f,  0.7069f,-0.4004f,-0.5830f,  0.7932f,-0.3466f,-0.5007f,  0.7933f,-0.3434f,-0.5028f,  0.8660f,-0.2835f,-0.4120f,  0.8660f,-0.2804f,-0.4141f,  0.9239f,-0.2155f,-0.3163f,  0.9239f,-0.2123f,-0.3184f,  0.9659f,-0.1436f,-0.2154f,  0.9659f,-0.1404f,-0.2175f,  0.9914f,-0.0691f,-0.1108f,  
	0.9914f,-0.0658f,-0.1130f,  1.0000f,0.0068f,-0.0045f,  0.9999f,0.0102f,-0.0067f,  0.9914f,0.0827f,0.1019f,  0.9913f,0.0863f,0.0996f,  0.9657f,0.1574f,0.2064f,  0.9656f,0.1613f,0.2042f,  0.9234f,0.2297f,0.3074f,  0.9232f,0.2338f,0.3051f,  0.8652f,0.2982f,0.4031f,  0.8648f,0.3025f,0.4007f,  0.7920f,0.3618f,0.4918f,  0.7913f,0.3664f,0.4895f,  0.7049f,0.4194f,0.5721f,  0.7040f,0.4241f,0.5697f,  
	0.6053f,0.4697f,0.6426f,  0.6041f,0.4746f,0.6401f,  0.4951f,0.5118f,0.7021f,  0.4935f,0.5168f,0.6995f,  0.3759f,0.5449f,0.7495f,  0.3739f,0.5499f,0.7469f,  0.2498f,0.5681f,0.7841f,  0.2474f,0.5729f,0.7814f,  0.1190f,0.5808f,0.8053f,  0.1162f,0.5853f,0.8024f,  -0.0142f,0.5826f,0.8126f,  -0.0175f,0.5866f,0.8097f,  -0.1476f,0.5730f,0.8061f,  -0.1512f,0.5764f,0.8030f,  -0.2786f,0.5521f,0.7859f,  
	-0.2826f,0.5546f,0.7827f,  -0.4050f,0.5197f,0.7523f,  -0.4091f,0.5211f,0.7490f,  -0.5242f,0.4762f,0.7060f,  -0.5285f,0.4763f,0.7027f,  -0.6341f,0.4220f,0.6479f,  -0.6383f,0.4207f,0.6446f,  -0.7325f,0.3579f,0.5791f,  -0.7365f,0.3550f,0.5759f,  -0.8174f,0.2846f,0.5009f,  -0.8208f,0.2800f,0.4979f,  -0.8869f,0.2035f,0.4148f,  -0.8896f,0.1969f,0.4121f,  -0.9395f,0.1157f,0.3225f,  -0.9413f,0.1072f,0.3202f,  
	-0.9739f,0.0228f,0.2257f,  -0.9745f,0.0125f,0.2239f,  -0.9893f,-0.0734f,0.1263f,  -0.9884f,-0.0855f,0.1252f,  -0.9849f,-0.1710f,0.0263f,  -0.9824f,-0.1849f,0.0260f,  -0.9607f,-0.2680f,-0.0726f,  -0.9563f,-0.2834f,-0.0719f,  -0.9167f,-0.3623f,-0.1684f,  -0.9104f,-0.3788f,-0.1666f,  -0.8537f,-0.4516f,-0.2593f,  -0.8452f,-0.4690f,-0.2562f,  -0.7727f,-0.5337f,-0.3436f,  -0.7621f,-0.5515f,-0.3393f,  -0.6751f,-0.6064f,-0.4201f,  
	-0.6624f,-0.6241f,-0.4144f,  -0.5630f,-0.6676f,-0.4872f,  -0.5483f,-0.6846f,-0.4803f,  -0.4385f,-0.7153f,-0.5442f,  -0.4221f,-0.7311f,-0.5360f,  -0.3043f,-0.7477f,-0.5902f,  -0.2866f,-0.7618f,-0.5809f,  -0.1635f,-0.7634f,-0.6248f,  -0.1449f,-0.7753f,-0.6147f,  -0.0192f,-0.7613f,-0.6481f,  -0.0001f,-0.7704f,-0.6375f,  0.1252f,-0.7407f,-0.6601f,  0.1442f,-0.7466f,-0.6495f,  0.2662f,-0.7012f,-0.6614f,  0.2844f,-0.7036f,-0.6512f,  
	-0.0039f,-0.5664f,-0.8242f,  -0.0027f,-0.5639f,-0.8258f,  0.1277f,-0.5629f,-0.8166f,  0.1286f,-0.5602f,-0.8183f,  0.2569f,-0.5491f,-0.7953f,  0.2576f,-0.5463f,-0.7970f,  0.3815f,-0.5254f,-0.7606f,  0.3819f,-0.5225f,-0.7623f,  0.4993f,-0.4922f,-0.7130f,  0.4996f,-0.4893f,-0.7148f,  0.6084f,-0.4503f,-0.6535f,  0.6086f,-0.4474f,-0.6554f,  0.7069f,-0.4004f,-0.5830f,  0.7070f,-0.3975f,-0.5849f,  0.7933f,-0.3434f,-0.5028f,  
	0.7933f,-0.3405f,-0.5047f,  0.8660f,-0.2804f,-0.4141f,  0.8660f,-0.2774f,-0.4160f,  0.9239f,-0.2123f,-0.3184f,  0.9239f,-0.2093f,-0.3205f,  0.9659f,-0.1404f,-0.2175f,  0.9659f,-0.1372f,-0.2196f,  0.9914f,-0.0658f,-0.1130f,  0.9914f,-0.0625f,-0.1151f,  0.9999f,0.0102f,-0.0067f,  0.9999f,0.0137f,-0.0088f,  0.9913f,0.0863f,0.0996f,  0.9912f,0.0901f,0.0974f,  0.9656f,0.1613f,0.2042f,  0.9654f,0.1653f,0.2019f,  
	0.9232f,0.2338f,0.3051f,  0.9228f,0.2381f,0.3028f,  0.8648f,0.3025f,0.4007f,  0.8642f,0.3072f,0.3984f,  0.7913f,0.3664f,0.4895f,  0.7905f,0.3713f,0.4871f,  0.7040f,0.4241f,0.5697f,  0.7028f,0.4292f,0.5673f,  0.6041f,0.4746f,0.6401f,  0.6026f,0.4799f,0.6376f,  0.4935f,0.5168f,0.6995f,  0.4916f,0.5222f,0.6969f,  0.3739f,0.5499f,0.7469f,  0.3715f,0.5552f,0.7441f,  0.2474f,0.5729f,0.7814f,  
	0.2446f,0.5781f,0.7785f,  0.1162f,0.5853f,0.8024f,  0.1129f,0.5902f,0.7994f,  -0.0175f,0.5866f,0.8097f,  -0.0213f,0.5909f,0.8065f,  -0.1512f,0.5764f,0.8030f,  -0.1554f,0.5799f,0.7997f,  -0.2826f,0.5546f,0.7827f,  -0.2871f,0.5571f,0.7792f,  -0.4091f,0.5211f,0.7490f,  -0.4138f,0.5225f,0.7455f,  -0.5285f,0.4763f,0.7027f,  -0.5332f,0.4764f,0.6991f,  -0.6383f,0.4207f,0.6446f,  -0.6429f,0.4192f,0.6410f,  
	-0.7365f,0.3550f,0.5759f,  -0.7407f,0.3517f,0.5724f,  -0.8208f,0.2800f,0.4979f,  -0.8245f,0.2748f,0.4946f,  -0.8896f,0.1969f,0.4121f,  -0.8925f,0.1898f,0.4091f,  -0.9413f,0.1072f,0.3202f,  -0.9431f,0.0981f,0.3176f,  -0.9745f,0.0125f,0.2239f,  -0.9750f,0.0014f,0.2220f,  -0.9884f,-0.0855f,0.1252f,  -0.9874f,-0.0985f,0.1241f,  -0.9824f,-0.1849f,0.0260f,  -0.9795f,-0.1996f,0.0258f,  -0.9563f,-0.2834f,-0.0719f,  
	-0.9514f,-0.2996f,-0.0710f,  -0.9104f,-0.3788f,-0.1666f,  -0.9033f,-0.3962f,-0.1644f,  -0.8452f,-0.4690f,-0.2562f,  -0.8360f,-0.4871f,-0.2528f,  -0.7621f,-0.5515f,-0.3393f,  -0.7506f,-0.5699f,-0.3344f,  -0.6624f,-0.6241f,-0.4144f,  -0.6488f,-0.6423f,-0.4081f,  -0.5483f,-0.6846f,-0.4803f,  -0.5327f,-0.7021f,-0.4726f,  -0.4221f,-0.7311f,-0.5360f,  -0.4049f,-0.7472f,-0.5270f,  -0.2866f,-0.7618f,-0.5809f,  -0.2680f,-0.7760f,-0.5709f,  
	-0.1449f,-0.7753f,-0.6147f,  -0.1254f,-0.7871f,-0.6040f,  -0.0001f,-0.7704f,-0.6375f,  0.0197f,-0.7793f,-0.6264f,  0.1442f,-0.7466f,-0.6495f,  0.1636f,-0.7521f,-0.6384f,  0.2844f,-0.7036f,-0.6512f,  0.3028f,-0.7055f,-0.6408f,  -0.0027f,-0.5639f,-0.8258f,  -0.0019f,-0.5618f,-0.8273f,  0.1286f,-0.5602f,-0.8183f,  0.1293f,-0.5579f,-0.8198f,  0.2576f,-0.5463f,-0.7970f,  0.2581f,-0.5438f,-0.7985f,  0.3819f,-0.5225f,-0.7623f,  
	0.3823f,-0.5199f,-0.7639f,  0.4996f,-0.4893f,-0.7148f,  0.4998f,-0.4867f,-0.7165f,  0.6086f,-0.4474f,-0.6554f,  0.6087f,-0.4447f,-0.6571f,  0.7070f,-0.3975f,-0.5849f,  0.7071f,-0.3948f,-0.5867f,  0.7933f,-0.3405f,-0.5047f,  0.7933f,-0.3377f,-0.5065f,  0.8660f,-0.2774f,-0.4160f,  0.8660f,-0.2746f,-0.4179f,  0.9239f,-0.2093f,-0.3205f,  0.9238f,-0.2063f,-0.3224f,  0.9659f,-0.1372f,-0.2196f,  0.9659f,-0.1341f,-0.2216f,  
	0.9914f,-0.0625f,-0.1151f,  0.9913f,-0.0591f,-0.1172f,  0.9999f,0.0137f,-0.0088f,  0.9998f,0.0173f,-0.0109f,  0.9912f,0.0901f,0.0974f,  0.9910f,0.0940f,0.0953f,  0.9654f,0.1653f,0.2019f,  0.9651f,0.1695f,0.1997f,  0.9228f,0.2381f,0.3028f,  0.9224f,0.2427f,0.3006f,  0.8642f,0.3072f,0.3984f,  0.8635f,0.3121f,0.3961f,  0.7905f,0.3713f,0.4871f,  0.7895f,0.3765f,0.4846f,  0.7028f,0.4292f,0.5673f,  
	0.7015f,0.4348f,0.5647f,  0.6026f,0.4799f,0.6376f,  0.6009f,0.4856f,0.6350f,  0.4916f,0.5222f,0.6969f,  0.4893f,0.5280f,0.6941f,  0.3715f,0.5552f,0.7441f,  0.3688f,0.5609f,0.7412f,  0.2446f,0.5781f,0.7785f,  0.2413f,0.5836f,0.7754f,  0.1129f,0.5902f,0.7994f,  0.1091f,0.5952f,0.7961f,  -0.0213f,0.5909f,0.8065f,  -0.0255f,0.5954f,0.8030f,  -0.1554f,0.5799f,0.7997f,  -0.1601f,0.5836f,0.7961f,  
	-0.2871f,0.5571f,0.7792f,  -0.2921f,0.5598f,0.7755f,  -0.4138f,0.5225f,0.7455f,  -0.4190f,0.5239f,0.7416f,  -0.5332f,0.4764f,0.6991f,  -0.5384f,0.4763f,0.6952f,  -0.6429f,0.4192f,0.6410f,  -0.6480f,0.4174f,0.6371f,  -0.7407f,0.3517f,0.5724f,  -0.7453f,0.3480f,0.5686f,  -0.8245f,0.2748f,0.4946f,  -0.8285f,0.2692f,0.4911f,  -0.8925f,0.1898f,0.4091f,  -0.8956f,0.1821f,0.4059f,  -0.9431f,0.0981f,0.3176f,  
	-0.9450f,0.0883f,0.3150f,  -0.9750f,0.0014f,0.2220f,  -0.9754f,-0.0105f,0.2201f,  -0.9874f,-0.0985f,0.1241f,  -0.9860f,-0.1124f,0.1230f,  -0.9795f,-0.1996f,0.0258f,  -0.9762f,-0.2152f,0.0258f,  -0.9514f,-0.2996f,-0.0710f,  -0.9460f,-0.3166f,-0.0698f,  -0.9033f,-0.3962f,-0.1644f,  -0.8956f,-0.4144f,-0.1619f,  -0.8360f,-0.4871f,-0.2528f,  -0.8259f,-0.5059f,-0.2488f,  -0.7506f,-0.5699f,-0.3344f,  -0.7382f,-0.5889f,-0.3290f,  
	-0.6488f,-0.6423f,-0.4081f,  -0.6342f,-0.6610f,-0.4011f,  -0.5327f,-0.7021f,-0.4726f,  -0.5162f,-0.7198f,-0.4641f,  -0.4049f,-0.7472f,-0.5270f,  -0.3866f,-0.7635f,-0.5173f,  -0.2680f,-0.7760f,-0.5709f,  -0.2485f,-0.7902f,-0.5602f,  -0.1254f,-0.7871f,-0.6040f,  -0.1051f,-0.7986f,-0.5926f,  0.0197f,-0.7793f,-0.6264f,  0.0401f,-0.7878f,-0.6147f,  0.1636f,-0.7521f,-0.6384f,  0.1835f,-0.7571f,-0.6270f,  0.3028f,-0.7055f,-0.6408f,  
	0.3214f,-0.7068f,-0.6302f,  -0.0019f,-0.5618f,-0.8273f,  -0.0012f,-0.5599f,-0.8285f,  0.1293f,-0.5579f,-0.8198f,  0.1298f,-0.5558f,-0.8211f,  0.2581f,-0.5438f,-0.7985f,  0.2584f,-0.5416f,-0.7999f,  0.3823f,-0.5199f,-0.7639f,  0.3825f,-0.5176f,-0.7654f,  0.4998f,-0.4867f,-0.7165f,  0.4999f,-0.4843f,-0.7180f,  0.6087f,-0.4447f,-0.6571f,  0.6087f,-0.4423f,-0.6587f,  0.7071f,-0.3948f,-0.5867f,  0.7071f,-0.3923f,-0.5884f,  
	0.7933f,-0.3377f,-0.5065f,  0.7933f,-0.3352f,-0.5083f,  0.8660f,-0.2746f,-0.4179f,  0.8660f,-0.2718f,-0.4197f,  0.9238f,-0.2063f,-0.3224f,  0.9238f,-0.2034f,-0.3243f,  0.9659f,-0.1341f,-0.2216f,  0.9659f,-0.1310f,-0.2235f,  0.9913f,-0.0591f,-0.1172f,  0.9913f,-0.0557f,-0.1191f,  0.9998f,0.0173f,-0.0109f,  0.9997f,0.0211f,-0.0130f,  0.9910f,0.0940f,0.0953f,  0.9908f,0.0981f,0.0932f,  0.9651f,0.1695f,0.1997f,  
	0.9647f,0.1740f,0.1976f,  0.9224f,0.2427f,0.3006f,  0.9218f,0.2476f,0.2983f,  0.8635f,0.3121f,0.3961f,  0.8627f,0.3174f,0.3937f,  0.7895f,0.3765f,0.4846f,  0.7883f,0.3822f,0.4822f,  0.7015f,0.4348f,0.5647f,  0.6998f,0.4407f,0.5622f,  0.6009f,0.4856f,0.6350f,  0.5987f,0.4917f,0.6322f,  0.4893f,0.5280f,0.6941f,  0.4867f,0.5342f,0.6912f,  0.3688f,0.5609f,0.7412f,  0.3656f,0.5670f,0.7381f,  
	0.2413f,0.5836f,0.7754f,  0.2376f,0.5894f,0.7721f,  0.1091f,0.5952f,0.7961f,  0.1048f,0.6006f,0.7926f,  -0.0255f,0.5954f,0.8030f,  -0.0303f,0.6001f,0.7994f,  -0.1601f,0.5836f,0.7961f,  -0.1653f,0.5874f,0.7922f,  -0.2921f,0.5598f,0.7755f,  -0.2976f,0.5624f,0.7714f,  -0.4190f,0.5239f,0.7416f,  -0.4247f,0.5252f,0.7374f,  -0.5384f,0.4763f,0.6952f,  -0.5441f,0.4759f,0.6910f,  -0.6480f,0.4174f,0.6371f,  
	-0.6534f,0.4152f,0.6330f,  -0.7453f,0.3480f,0.5686f,  -0.7503f,0.3439f,0.5646f,  -0.8285f,0.2692f,0.4911f,  -0.8327f,0.2629f,0.4873f,  -0.8956f,0.1821f,0.4059f,  -0.8987f,0.1737f,0.4027f,  -0.9450f,0.0883f,0.3150f,  -0.9468f,0.0777f,0.3123f,  -0.9754f,-0.0105f,0.2201f,  -0.9756f,-0.0233f,0.2182f,  -0.9860f,-0.1124f,0.1230f,  -0.9843f,-0.1271f,0.1221f,  -0.9762f,-0.2152f,0.0258f,  -0.9724f,-0.2317f,0.0260f,  
	-0.9460f,-0.3166f,-0.0698f,  -0.9399f,-0.3346f,-0.0683f,  -0.8956f,-0.4144f,-0.1619f,  -0.8871f,-0.4334f,-0.1589f,  -0.8259f,-0.5059f,-0.2488f,  -0.8149f,-0.5256f,-0.2442f,  -0.7382f,-0.5889f,-0.3290f,  -0.7248f,-0.6086f,-0.3228f,  -0.6342f,-0.6610f,-0.4011f,  -0.6187f,-0.6801f,-0.3933f,  -0.5162f,-0.7198f,-0.4641f,  -0.4986f,-0.7379f,-0.4549f,  -0.3866f,-0.7635f,-0.5173f,  -0.3674f,-0.7799f,-0.5068f,  -0.2485f,-0.7902f,-0.5602f,  
	-0.2281f,-0.8043f,-0.5487f,  -0.1051f,-0.7986f,-0.5926f,  -0.0841f,-0.8099f,-0.5804f,  0.0401f,-0.7878f,-0.6147f,  0.0610f,-0.7958f,-0.6024f,  0.1835f,-0.7571f,-0.6270f,  0.2037f,-0.7617f,-0.6151f,  0.3214f,-0.7068f,-0.6302f,  0.3402f,-0.7075f,-0.6194f,  -0.0012f,-0.5599f,-0.8285f,  -0.0007f,-0.5583f,-0.8296f,  0.1298f,-0.5558f,-0.8211f,  0.1301f,-0.5541f,-0.8222f,  0.2584f,-0.5416f,-0.7999f,  0.2586f,-0.5397f,-0.8011f,  
	0.3825f,-0.5176f,-0.7654f,  0.3826f,-0.5156f,-0.7666f,  0.4999f,-0.4843f,-0.7180f,  0.4999f,-0.4822f,-0.7194f,  0.6087f,-0.4423f,-0.6587f,  0.6087f,-0.4401f,-0.6601f,  0.7071f,-0.3923f,-0.5884f,  0.7071f,-0.3900f,-0.5899f,  0.7933f,-0.3352f,-0.5083f,  0.7933f,-0.3327f,-0.5099f,  0.8660f,-0.2718f,-0.4197f,  0.8660f,-0.2692f,-0.4214f,  0.9238f,-0.2034f,-0.3243f,  0.9239f,-0.2005f,-0.3260f,  0.9659f,-0.1310f,-0.2235f,  
	0.9659f,-0.1278f,-0.2253f,  0.9913f,-0.0557f,-0.1191f,  0.9913f,-0.0522f,-0.1210f,  0.9997f,0.0211f,-0.0130f,  0.9996f,0.0250f,-0.0149f,  0.9908f,0.0981f,0.0932f,  0.9906f,0.1024f,0.0912f,  0.9647f,0.1740f,0.1976f,  0.9643f,0.1788f,0.1954f,  0.9218f,0.2476f,0.2983f,  0.9211f,0.2528f,0.2961f,  0.8627f,0.3174f,0.3937f,  0.8616f,0.3231f,0.3914f,  0.7883f,0.3822f,0.4822f,  0.7868f,0.3882f,0.4797f,  
	0.6998f,0.4407f,0.5622f,  0.6979f,0.4470f,0.5595f,  0.5987f,0.4917f,0.6322f,  0.5963f,0.4982f,0.6294f,  0.4867f,0.5342f,0.6912f,  0.4837f,0.5408f,0.6882f,  0.3656f,0.5670f,0.7381f,  0.3620f,0.5735f,0.7349f,  0.2376f,0.5894f,0.7721f,  0.2333f,0.5956f,0.7686f,  0.1048f,0.6006f,0.7926f,  0.0999f,0.6063f,0.7889f,  -0.0303f,0.6001f,0.7994f,  -0.0357f,0.6050f,0.7954f,  -0.1653f,0.5874f,0.7922f,  
	-0.1711f,0.5913f,0.7881f,  -0.2976f,0.5624f,0.7714f,  -0.3037f,0.5651f,0.7671f,  -0.4247f,0.5252f,0.7374f,  -0.4309f,0.5263f,0.7330f,  -0.5441f,0.4759f,0.6910f,  -0.5503f,0.4754f,0.6865f,  -0.6534f,0.4152f,0.6330f,  -0.6593f,0.4127f,0.6285f,  -0.7503f,0.3439f,0.5646f,  -0.7556f,0.3393f,0.5603f,  -0.8327f,0.2629f,0.4873f,  -0.8371f,0.2561f,0.4834f,  -0.8987f,0.1737f,0.4027f,  -0.9020f,0.1645f,0.3992f,  
	-0.9468f,0.0777f,0.3123f,  -0.9486f,0.0662f,0.3096f,  -0.9756f,-0.0233f,0.2182f,  -0.9756f,-0.0369f,0.2163f,  -0.9843f,-0.1271f,0.1221f,  -0.9823f,-0.1427f,0.1214f,  -0.9724f,-0.2317f,0.0260f,  -0.9681f,-0.2491f,0.0265f,  -0.9399f,-0.3346f,-0.0683f,  -0.9331f,-0.3534f,-0.0663f,  -0.8871f,-0.4334f,-0.1589f,  -0.8778f,-0.4532f,-0.1553f,  -0.8149f,-0.5256f,-0.2442f,  -0.8031f,-0.5458f,-0.2389f,  -0.7248f,-0.6086f,-0.3228f,  
	-0.7106f,-0.6288f,-0.3158f,  -0.6187f,-0.6801f,-0.3933f,  -0.6020f,-0.6997f,-0.3847f,  -0.4986f,-0.7379f,-0.4549f,  -0.4800f,-0.7562f,-0.4447f,  -0.3674f,-0.7799f,-0.5068f,  -0.3472f,-0.7963f,-0.4954f,  -0.2281f,-0.8043f,-0.5487f,  -0.2069f,-0.8183f,-0.5363f,  -0.0841f,-0.8099f,-0.5804f,  -0.0624f,-0.8210f,-0.5676f,  0.0610f,-0.7958f,-0.6024f,  0.0825f,-0.8035f,-0.5896f,  0.2037f,-0.7617f,-0.6151f,  0.2242f,-0.7656f,-0.6030f,  
	0.3402f,-0.7075f,-0.6194f,  0.3590f,-0.7076f,-0.6086f,  -0.0007f,-0.5583f,-0.8296f,  -0.0004f,-0.5570f,-0.8305f,  0.1301f,-0.5541f,-0.8222f,  0.1303f,-0.5526f,-0.8232f,  0.2586f,-0.5397f,-0.8011f,  0.2587f,-0.5382f,-0.8022f,  0.3826f,-0.5156f,-0.7666f,  0.3826f,-0.5140f,-0.7678f,  0.4999f,-0.4822f,-0.7194f,  0.4999f,-0.4804f,-0.7206f,  0.6087f,-0.4401f,-0.6601f,  0.6087f,-0.4382f,-0.6614f,  0.7071f,-0.3900f,-0.5899f,  
	0.7071f,-0.3879f,-0.5913f,  0.7933f,-0.3327f,-0.5099f,  0.7933f,-0.3304f,-0.5113f,  0.8660f,-0.2692f,-0.4214f,  0.8660f,-0.2667f,-0.4229f,  0.9239f,-0.2005f,-0.3260f,  0.9239f,-0.1977f,-0.3276f,  0.9659f,-0.1278f,-0.2253f,  0.9659f,-0.1246f,-0.2270f,  0.9913f,-0.0522f,-0.1210f,  0.9912f,-0.0486f,-0.1228f,  0.9996f,0.0250f,-0.0149f,  0.9994f,0.0291f,-0.0168f,  0.9906f,0.1024f,0.0912f,  0.9902f,0.1070f,0.0892f,  
	0.9643f,0.1788f,0.1954f,  0.9637f,0.1839f,0.1934f,  0.9211f,0.2528f,0.2961f,  0.9202f,0.2584f,0.2939f,  0.8616f,0.3231f,0.3914f,  0.8604f,0.3291f,0.3891f,  0.7868f,0.3882f,0.4797f,  0.7851f,0.3947f,0.4773f,  0.6979f,0.4470f,0.5595f,  0.6957f,0.4538f,0.5569f,  0.5963f,0.4982f,0.6294f,  0.5934f,0.5052f,0.6266f,  0.4837f,0.5408f,0.6882f,  0.4802f,0.5478f,0.6851f,  0.3620f,0.5735f,0.7349f,  
	0.3578f,0.5804f,0.7315f,  0.2333f,0.5956f,0.7686f,  0.2285f,0.6022f,0.7650f,  0.0999f,0.6063f,0.7889f,  0.0945f,0.6123f,0.7850f,  -0.0357f,0.6050f,0.7954f,  -0.0416f,0.6101f,0.7912f,  -0.1711f,0.5913f,0.7881f,  -0.1775f,0.5954f,0.7836f,  -0.3037f,0.5651f,0.7671f,  -0.3104f,0.5677f,0.7625f,  -0.4309f,0.5263f,0.7330f,  -0.4377f,0.5274f,0.7282f,  -0.5503f,0.4754f,0.6865f,  -0.5569f,0.4745f,0.6817f,  
	-0.6593f,0.4127f,0.6285f,  -0.6655f,0.4098f,0.6238f,  -0.7556f,0.3393f,0.5603f,  -0.7612f,0.3341f,0.5559f,  -0.8371f,0.2561f,0.4834f,  -0.8417f,0.2486f,0.4793f,  -0.9020f,0.1645f,0.3992f,  -0.9053f,0.1546f,0.3957f,  -0.9486f,0.0662f,0.3096f,  -0.9502f,0.0540f,0.3069f,  -0.9756f,-0.0369f,0.2163f,  -0.9753f,-0.0514f,0.2147f,  -0.9823f,-0.1427f,0.1214f,  -0.9798f,-0.1593f,0.1209f,  -0.9681f,-0.2491f,0.0265f,  
	-0.9632f,-0.2673f,0.0275f,  -0.9331f,-0.3534f,-0.0663f,  -0.9256f,-0.3730f,-0.0637f,  -0.8778f,-0.4532f,-0.1553f,  -0.8676f,-0.4737f,-0.1510f,  -0.8031f,-0.5458f,-0.2389f,  -0.7903f,-0.5667f,-0.2328f,  -0.7106f,-0.6288f,-0.3158f,  -0.6952f,-0.6495f,-0.3079f,  -0.6020f,-0.6997f,-0.3847f,  -0.5844f,-0.7196f,-0.3751f,  -0.4800f,-0.7562f,-0.4447f,  -0.4604f,-0.7746f,-0.4336f,  -0.3472f,-0.7963f,-0.4954f,  -0.3261f,-0.8126f,-0.4830f,  
	-0.2069f,-0.8183f,-0.5363f,  -0.1848f,-0.8320f,-0.5230f,  -0.0624f,-0.8210f,-0.5676f,  -0.0401f,-0.8316f,-0.5539f,  0.0825f,-0.8035f,-0.5896f,  0.1045f,-0.8106f,-0.5762f,  0.2242f,-0.7656f,-0.6030f,  0.2450f,-0.7690f,-0.5905f,  0.3590f,-0.7076f,-0.6086f,  0.3777f,-0.7072f,-0.5977f,  -0.0004f,-0.5570f,-0.8305f,  -0.0002f,-0.5560f,-0.8312f,  0.1303f,-0.5526f,-0.8232f,  0.1304f,-0.5515f,-0.8239f,  0.2587f,-0.5382f,-0.8022f,  
	0.2587f,-0.5369f,-0.8030f,  0.3826f,-0.5140f,-0.7678f,  0.3826f,-0.5125f,-0.7687f,  0.4999f,-0.4804f,-0.7206f,  0.4999f,-0.4789f,-0.7216f,  0.6087f,-0.4382f,-0.6614f,  0.6087f,-0.4365f,-0.6625f,  0.7071f,-0.3879f,-0.5913f,  0.7071f,-0.3860f,-0.5925f,  0.7933f,-0.3304f,-0.5113f,  0.7934f,-0.3283f,-0.5126f,  0.8660f,-0.2667f,-0.4229f,  0.8661f,-0.2642f,-0.4243f,  0.9239f,-0.1977f,-0.3276f,  0.9240f,-0.1948f,-0.3291f,  
	0.9659f,-0.1246f,-0.2270f,  0.9659f,-0.1213f,-0.2285f,  0.9912f,-0.0486f,-0.1228f,  0.9912f,-0.0448f,-0.1244f,  0.9994f,0.0291f,-0.0168f,  0.9993f,0.0334f,-0.0185f,  0.9902f,0.1070f,0.0892f,  0.9899f,0.1119f,0.0874f,  0.9637f,0.1839f,0.1934f,  0.9631f,0.1893f,0.1914f,  0.9202f,0.2584f,0.2939f,  0.9192f,0.2644f,0.2918f,  0.8604f,0.3291f,0.3891f,  0.8589f,0.3356f,0.3868f,  0.7851f,0.3947f,0.4773f,  
	0.7831f,0.4016f,0.4748f,  0.6957f,0.4538f,0.5569f,  0.6931f,0.4611f,0.5542f,  0.5934f,0.5052f,0.6266f,  0.5902f,0.5127f,0.6236f,  0.4802f,0.5478f,0.6851f,  0.4762f,0.5552f,0.6819f,  0.3578f,0.5804f,0.7315f,  0.3531f,0.5877f,0.7280f,  0.2285f,0.6022f,0.7650f,  0.2231f,0.6090f,0.7611f,  0.0945f,0.6123f,0.7850f,  0.0885f,0.6185f,0.7808f,  -0.0416f,0.6101f,0.7912f,  -0.0483f,0.6154f,0.7867f,  
	-0.1775f,0.5954f,0.7836f,  -0.1845f,0.5994f,0.7789f,  -0.3104f,0.5677f,0.7625f,  -0.3177f,0.5703f,0.7575f,  -0.4377f,0.5274f,0.7282f,  -0.4451f,0.5282f,0.7231f,  -0.5569f,0.4745f,0.6817f,  -0.5641f,0.4734f,0.6766f,  -0.6655f,0.4098f,0.6238f,  -0.6722f,0.4065f,0.6188f,  -0.7612f,0.3341f,0.5559f,  -0.7670f,0.3284f,0.5512f,  -0.8417f,0.2486f,0.4793f,  -0.8465f,0.2404f,0.4751f,  -0.9053f,0.1546f,0.3957f,  
	-0.9085f,0.1440f,0.3922f,  -0.9502f,0.0540f,0.3069f,  -0.9517f,0.0409f,0.3043f,  -0.9753f,-0.0514f,0.2147f,  -0.9747f,-0.0668f,0.2132f,  -0.9798f,-0.1593f,0.1209f,  -0.9768f,-0.1767f,0.1208f,  -0.9632f,-0.2673f,0.0275f,  -0.9576f,-0.2865f,0.0290f,  -0.9256f,-0.3730f,-0.0637f,  -0.9173f,-0.3935f,-0.0605f,  -0.8676f,-0.4737f,-0.1510f,  -0.8566f,-0.4950f,-0.1459f,  -0.7903f,-0.5667f,-0.2328f,  -0.7765f,-0.5883f,-0.2258f,  
	-0.6952f,-0.6495f,-0.3079f,  -0.6788f,-0.6707f,-0.2990f,  -0.5844f,-0.7196f,-0.3751f,  -0.5657f,-0.7397f,-0.3644f,  -0.4604f,-0.7746f,-0.4336f,  -0.4398f,-0.7931f,-0.4214f,  -0.3261f,-0.8126f,-0.4830f,  -0.3041f,-0.8289f,-0.4695f,  -0.1848f,-0.8320f,-0.5230f,  -0.1620f,-0.8455f,-0.5088f,  -0.0401f,-0.8316f,-0.5539f,  -0.0171f,-0.8418f,-0.5396f,  0.1045f,-0.8106f,-0.5762f,  0.1268f,-0.8172f,-0.5623f,  0.2450f,-0.7690f,-0.5905f,  
	0.2658f,-0.7718f,-0.5777f,  0.3777f,-0.7072f,-0.5977f,  0.3962f,-0.7061f,-0.5868f,  -0.0002f,-0.5560f,-0.8312f,  -0.0001f,-0.5553f,-0.8316f,  0.1304f,-0.5515f,-0.8239f,  0.1304f,-0.5506f,-0.8245f,  0.2587f,-0.5369f,-0.8030f,  0.2587f,-0.5359f,-0.8037f,  0.3826f,-0.5125f,-0.7687f,  0.3826f,-0.5114f,-0.7695f,  0.4999f,-0.4789f,-0.7216f,  0.4999f,-0.4776f,-0.7225f,  0.6087f,-0.4365f,-0.6625f,  0.6088f,-0.4350f,-0.6635f,  
	0.7071f,-0.3860f,-0.5925f,  0.7072f,-0.3842f,-0.5935f,  0.7934f,-0.3283f,-0.5126f,  0.7935f,-0.3262f,-0.5137f,  0.8661f,-0.2642f,-0.4243f,  0.8663f,-0.2618f,-0.4255f,  0.9240f,-0.1948f,-0.3291f,  0.9241f,-0.1920f,-0.3304f,  0.9659f,-0.1213f,-0.2285f,  0.9660f,-0.1179f,-0.2299f,  0.9912f,-0.0448f,-0.1244f,  0.9912f,-0.0409f,-0.1259f,  0.9993f,0.0334f,-0.0185f,  0.9991f,0.0379f,-0.0201f,  0.9899f,0.1119f,0.0874f,  
	0.9894f,0.1170f,0.0856f,  0.9631f,0.1893f,0.1914f,  0.9623f,0.1951f,0.1895f,  0.9192f,0.2644f,0.2918f,  0.9180f,0.2708f,0.2898f,  0.8589f,0.3356f,0.3868f,  0.8572f,0.3425f,0.3846f,  0.7831f,0.4016f,0.4748f,  0.7808f,0.4090f,0.4723f,  0.6931f,0.4611f,0.5542f,  0.6901f,0.4688f,0.5514f,  0.5902f,0.5127f,0.6236f,  0.5865f,0.5206f,0.6205f,  0.4762f,0.5552f,0.6819f,  0.4717f,0.5632f,0.6785f,  
	0.3531f,0.5877f,0.7280f,  0.3479f,0.5954f,0.7242f,  0.2231f,0.6090f,0.7611f,  0.2171f,0.6162f,0.7570f,  0.0885f,0.6185f,0.7808f,  0.0818f,0.6249f,0.7764f,  -0.0483f,0.6154f,0.7867f,  -0.0555f,0.6208f,0.7820f,  -0.1845f,0.5994f,0.7789f,  -0.1922f,0.6035f,0.7738f,  -0.3177f,0.5703f,0.7575f,  -0.3256f,0.5728f,0.7522f,  -0.4451f,0.5282f,0.7231f,  -0.4530f,0.5288f,0.7178f,  -0.5641f,0.4734f,0.6766f,  
	-0.5717f,0.4718f,0.6712f,  -0.6722f,0.4065f,0.6188f,  -0.6792f,0.4026f,0.6136f,  -0.7670f,0.3284f,0.5512f,  -0.7732f,0.3220f,0.5463f,  -0.8465f,0.2404f,0.4751f,  -0.8513f,0.2315f,0.4708f,  -0.9085f,0.1440f,0.3922f,  -0.9118f,0.1325f,0.3888f,  -0.9517f,0.0409f,0.3043f,  -0.9530f,0.0269f,0.3019f,  -0.9747f,-0.0668f,0.2132f,  -0.9737f,-0.0831f,0.2121f,  -0.9768f,-0.1767f,0.1208f,  -0.9733f,-0.1951f,0.1211f,  
	-0.9576f,-0.2865f,0.0290f,  -0.9513f,-0.3066f,0.0311f,  -0.9173f,-0.3935f,-0.0605f,  -0.9082f,-0.4148f,-0.0565f,  -0.8566f,-0.4950f,-0.1459f,  -0.8445f,-0.5170f,-0.1399f,  -0.7765f,-0.5883f,-0.2258f,  -0.7616f,-0.6103f,-0.2178f,  -0.6788f,-0.6707f,-0.2990f,  -0.6613f,-0.6922f,-0.2890f,  -0.5657f,-0.7397f,-0.3644f,  -0.5459f,-0.7600f,-0.3526f,  -0.4398f,-0.7931f,-0.4214f,  -0.4181f,-0.8116f,-0.4080f,  -0.3041f,-0.8289f,-0.4695f,  
	-0.2812f,-0.8449f,-0.4550f,  -0.1620f,-0.8455f,-0.5088f,  -0.1384f,-0.8585f,-0.4937f,  -0.0171f,-0.8418f,-0.5396f,  0.0064f,-0.8514f,-0.5244f,  0.1268f,-0.8172f,-0.5623f,  0.1493f,-0.8232f,-0.5478f,  0.2658f,-0.7718f,-0.5777f,  0.2867f,-0.7740f,-0.5646f,  0.3962f,-0.7061f,-0.5868f,  0.4146f,-0.7045f,-0.5760f,  -0.0001f,-0.5553f,-0.8316f,  -0.0001f,-0.5549f,-0.8319f,  0.1304f,-0.5506f,-0.8245f,  0.1304f,-0.5500f,-0.8249f,  
	0.2587f,-0.5359f,-0.8037f,  0.2587f,-0.5351f,-0.8042f,  0.3826f,-0.5114f,-0.7695f,  0.3826f,-0.5105f,-0.7701f,  0.4999f,-0.4776f,-0.7225f,  0.5000f,-0.4765f,-0.7232f,  0.6088f,-0.4350f,-0.6635f,  0.6089f,-0.4336f,-0.6643f,  0.7072f,-0.3842f,-0.5935f,  0.7073f,-0.3826f,-0.5944f,  0.7935f,-0.3262f,-0.5137f,  0.7937f,-0.3242f,-0.5147f,  0.8663f,-0.2618f,-0.4255f,  0.8665f,-0.2593f,-0.4266f,  0.9241f,-0.1920f,-0.3304f,  
	0.9243f,-0.1890f,-0.3315f,  0.9660f,-0.1179f,-0.2299f,  0.9662f,-0.1144f,-0.2312f,  0.9912f,-0.0409f,-0.1259f,  0.9912f,-0.0367f,-0.1273f,  0.9991f,0.0379f,-0.0201f,  0.9989f,0.0427f,-0.0216f,  0.9894f,0.1170f,0.0856f,  0.9889f,0.1225f,0.0840f,  0.9623f,0.1951f,0.1895f,  0.9614f,0.2013f,0.1877f,  0.9180f,0.2708f,0.2898f,  0.9166f,0.2776f,0.2878f,  0.8572f,0.3425f,0.3846f,  0.8552f,0.3499f,0.3823f,  
	0.7808f,0.4090f,0.4723f,  0.7782f,0.4169f,0.4698f,  0.6901f,0.4688f,0.5514f,  0.6867f,0.4770f,0.5486f,  0.5865f,0.5206f,0.6205f,  0.5823f,0.5290f,0.6174f,  0.4717f,0.5632f,0.6785f,  0.4667f,0.5715f,0.6749f,  0.3479f,0.5954f,0.7242f,  0.3421f,0.6034f,0.7203f,  0.2171f,0.6162f,0.7570f,  0.2105f,0.6238f,0.7527f,  0.0818f,0.6249f,0.7764f,  0.0745f,0.6316f,0.7717f,  -0.0555f,0.6208f,0.7820f,  
	-0.0634f,0.6264f,0.7769f,  -0.1922f,0.6035f,0.7738f,  -0.2005f,0.6076f,0.7685f,  -0.3256f,0.5728f,0.7522f,  -0.3341f,0.5752f,0.7467f,  -0.4530f,0.5288f,0.7178f,  -0.4614f,0.5291f,0.7121f,  -0.5717f,0.4718f,0.6712f,  -0.5798f,0.4699f,0.6656f,  -0.6792f,0.4026f,0.6136f,  -0.6867f,0.3982f,0.6082f,  -0.7732f,0.3220f,0.5463f,  -0.7795f,0.3150f,0.5414f,  -0.8513f,0.2315f,0.4708f,  -0.8562f,0.2218f,0.4665f,  
	-0.9118f,0.1325f,0.3888f,  -0.9149f,0.1201f,0.3854f,  -0.9530f,0.0269f,0.3019f,  -0.9540f,0.0120f,0.2997f,  -0.9737f,-0.0831f,0.2121f,  -0.9723f,-0.1004f,0.2113f,  -0.9733f,-0.1951f,0.1211f,  -0.9691f,-0.2144f,0.1221f,  -0.9513f,-0.3066f,0.0311f,  -0.9443f,-0.3275f,0.0339f,  -0.9082f,-0.4148f,-0.0565f,  -0.8981f,-0.4368f,-0.0516f,  -0.8445f,-0.5170f,-0.1399f,  -0.8314f,-0.5395f,-0.1329f,  -0.7616f,-0.6103f,-0.2178f,  
	-0.7456f,-0.6328f,-0.2087f,  -0.6613f,-0.6922f,-0.2890f,  -0.6427f,-0.7140f,-0.2778f,  -0.5459f,-0.7600f,-0.3526f,  -0.5250f,-0.7804f,-0.3396f,  -0.4181f,-0.8116f,-0.4080f,  -0.3955f,-0.8299f,-0.3935f,  -0.2812f,-0.8449f,-0.4550f,  -0.2573f,-0.8606f,-0.4395f,  -0.1384f,-0.8585f,-0.4937f,  -0.1141f,-0.8711f,-0.4777f,  0.0064f,-0.8514f,-0.5244f,  0.0304f,-0.8605f,-0.5085f,  0.1493f,-0.8232f,-0.5478f,  0.1722f,-0.8285f,-0.5328f,  
	0.2867f,-0.7740f,-0.5646f,  0.3076f,-0.7755f,-0.5514f,  0.4146f,-0.7045f,-0.5760f,  0.4327f,-0.7023f,-0.5653f,  -0.0001f,-0.5549f,-0.8319f,  -0.0001f,-0.5547f,-0.8320f,  0.1304f,-0.5500f,-0.8249f,  0.1304f,-0.5497f,-0.8251f,  0.2587f,-0.5351f,-0.8042f,  0.2588f,-0.5347f,-0.8045f,  0.3826f,-0.5105f,-0.7701f,  0.3827f,-0.5098f,-0.7705f,  0.5000f,-0.4765f,-0.7232f,  0.5001f,-0.4756f,-0.7237f,  0.6089f,-0.4336f,-0.6643f,  
	0.6091f,-0.4325f,-0.6649f,  0.7073f,-0.3826f,-0.5944f,  0.7076f,-0.3811f,-0.5951f,  0.7937f,-0.3242f,-0.5147f,  0.7940f,-0.3222f,-0.5155f,  0.8665f,-0.2593f,-0.4266f,  0.8668f,-0.2568f,-0.4274f,  0.9243f,-0.1890f,-0.3315f,  0.9246f,-0.1860f,-0.3325f,  0.9662f,-0.1144f,-0.2312f,  0.9663f,-0.1107f,-0.2323f,  0.9912f,-0.0367f,-0.1273f,  0.9912f,-0.0323f,-0.1285f,  0.9989f,0.0427f,-0.0216f,  0.9986f,0.0478f,-0.0229f,  
	0.9889f,0.1225f,0.0840f,  0.9883f,0.1284f,0.0825f,  0.9614f,0.2013f,0.1877f,  0.9603f,0.2079f,0.1860f,  0.9166f,0.2776f,0.2878f,  0.9150f,0.2849f,0.2858f,  0.8552f,0.3499f,0.3823f,  0.8529f,0.3578f,0.3801f,  0.7782f,0.4169f,0.4698f,  0.7751f,0.4253f,0.4672f,  0.6867f,0.4770f,0.5486f,  0.6828f,0.4858f,0.5457f,  0.5823f,0.5290f,0.6174f,  0.5775f,0.5379f,0.6141f,  0.4667f,0.5715f,0.6749f,  
	0.4611f,0.5803f,0.6713f,  0.3421f,0.6034f,0.7203f,  0.3356f,0.6119f,0.7162f,  0.2105f,0.6238f,0.7527f,  0.2032f,0.6316f,0.7482f,  0.0745f,0.6316f,0.7717f,  0.0665f,0.6386f,0.7667f,  -0.0634f,0.6264f,0.7769f,  -0.0720f,0.6321f,0.7716f,  -0.2005f,0.6076f,0.7685f,  -0.2095f,0.6117f,0.7628f,  -0.3341f,0.5752f,0.7467f,  -0.3433f,0.5774f,0.7408f,  -0.4614f,0.5291f,0.7121f,  -0.4705f,0.5292f,0.7061f,  
	-0.5798f,0.4699f,0.6656f,  -0.5884f,0.4675f,0.6597f,  -0.6867f,0.3982f,0.6082f,  -0.6944f,0.3932f,0.6027f,  -0.7795f,0.3150f,0.5414f,  -0.7861f,0.3073f,0.5364f,  -0.8562f,0.2218f,0.4665f,  -0.8612f,0.2112f,0.4623f,  -0.9149f,0.1201f,0.3854f,  -0.9179f,0.1068f,0.3822f,  -0.9540f,0.0120f,0.2997f,  -0.9546f,-0.0039f,0.2978f,  -0.9723f,-0.1004f,0.2113f,  -0.9703f,-0.1186f,0.2110f,  -0.9691f,-0.2144f,0.1221f,  
	-0.9642f,-0.2346f,0.1238f,  -0.9443f,-0.3275f,0.0339f,  -0.9363f,-0.3492f,0.0376f,  -0.8981f,-0.4368f,-0.0516f,  -0.8870f,-0.4595f,-0.0457f,  -0.8314f,-0.5395f,-0.1329f,  -0.8172f,-0.5627f,-0.1247f,  -0.7456f,-0.6328f,-0.2087f,  -0.7285f,-0.6557f,-0.1982f,  -0.6427f,-0.7140f,-0.2778f,  -0.6230f,-0.7359f,-0.2653f,  -0.5250f,-0.7804f,-0.3396f,  -0.5031f,-0.8007f,-0.3252f,  -0.3955f,-0.8299f,-0.3935f,  -0.3718f,-0.8480f,-0.3777f,  
	-0.2573f,-0.8606f,-0.4395f,  -0.2326f,-0.8759f,-0.4227f,  -0.1141f,-0.8711f,-0.4777f,  -0.0891f,-0.8831f,-0.4606f,  0.0304f,-0.8605f,-0.5085f,  0.0548f,-0.8690f,-0.4918f,  0.1722f,-0.8285f,-0.5328f,  0.1954f,-0.8332f,-0.5173f,  0.3076f,-0.7755f,-0.5514f,  0.3285f,-0.7763f,-0.5380f,  0.4327f,-0.7023f,-0.5653f,  0.4504f,-0.6996f,-0.5548f,  -0.0001f,-0.5547f,-0.8320f,  -0.0001f,-0.5549f,-0.8319f,  0.1304f,-0.5497f,-0.8251f,  
	0.1305f,-0.5497f,-0.8251f,  0.2588f,-0.5347f,-0.8045f,  0.2588f,-0.5345f,-0.8046f,  0.3827f,-0.5098f,-0.7705f,  0.3828f,-0.5094f,-0.7707f,  0.5001f,-0.4756f,-0.7237f,  0.5004f,-0.4749f,-0.7239f,  0.6091f,-0.4325f,-0.6649f,  0.6094f,-0.4314f,-0.6652f,  0.7076f,-0.3811f,-0.5951f,  0.7080f,-0.3796f,-0.5955f,  0.7940f,-0.3222f,-0.5155f,  0.7944f,-0.3203f,-0.5160f,  0.8668f,-0.2568f,-0.4274f,  0.8672f,-0.2543f,-0.4281f,  
	0.9246f,-0.1860f,-0.3325f,  0.9249f,-0.1828f,-0.3333f,  0.9663f,-0.1107f,-0.2323f,  0.9666f,-0.1068f,-0.2332f,  0.9912f,-0.0323f,-0.1285f,  0.9912f,-0.0277f,-0.1295f,  0.9986f,0.0478f,-0.0229f,  0.9983f,0.0533f,-0.0241f,  0.9883f,0.1284f,0.0825f,  0.9876f,0.1346f,0.0811f,  0.9603f,0.2079f,0.1860f,  0.9591f,0.2150f,0.1844f,  0.9150f,0.2849f,0.2858f,  0.9131f,0.2927f,0.2839f,  0.8529f,0.3578f,0.3801f,  
	0.8503f,0.3663f,0.3779f,  0.7751f,0.4253f,0.4672f,  0.7717f,0.4342f,0.4647f,  0.6828f,0.4858f,0.5457f,  0.6785f,0.4950f,0.5428f,  0.5775f,0.5379f,0.6141f,  0.5723f,0.5473f,0.6107f,  0.4611f,0.5803f,0.6713f,  0.4549f,0.5896f,0.6674f,  0.3356f,0.6119f,0.7162f,  0.3284f,0.6208f,0.7119f,  0.2032f,0.6316f,0.7482f,  0.1952f,0.6398f,0.7434f,  0.0665f,0.6386f,0.7667f,  0.0577f,0.6457f,0.7614f,  
	-0.0720f,0.6321f,0.7716f,  -0.0814f,0.6378f,0.7659f,  -0.2095f,0.6117f,0.7628f,  -0.2192f,0.6157f,0.7568f,  -0.3433f,0.5774f,0.7408f,  -0.3531f,0.5794f,0.7346f,  -0.4705f,0.5292f,0.7061f,  -0.4801f,0.5288f,0.6999f,  -0.5884f,0.4675f,0.6597f,  -0.5974f,0.4646f,0.6536f,  -0.6944f,0.3932f,0.6027f,  -0.7024f,0.3875f,0.5970f,  -0.7861f,0.3073f,0.5364f,  -0.7928f,0.2988f,0.5313f,  -0.8612f,0.2112f,0.4623f,  
	-0.8661f,0.1999f,0.4581f,  -0.9179f,0.1068f,0.3822f,  -0.9207f,0.0926f,0.3792f,  -0.9546f,-0.0039f,0.2978f,  -0.9548f,-0.0207f,0.2964f,  -0.9703f,-0.1186f,0.2110f,  -0.9676f,-0.1376f,0.2115f,  -0.9642f,-0.2346f,0.1238f,  -0.9585f,-0.2556f,0.1262f,  -0.9363f,-0.3492f,0.0376f,  -0.9274f,-0.3717f,0.0423f,  -0.8870f,-0.4595f,-0.0457f,  -0.8748f,-0.4829f,-0.0387f,  -0.8172f,-0.5627f,-0.1247f,  -0.8019f,-0.5862f,-0.1153f,  
	-0.7285f,-0.6557f,-0.1982f,  -0.7102f,-0.6788f,-0.1865f,  -0.6230f,-0.7359f,-0.2653f,  -0.6021f,-0.7579f,-0.2513f,  -0.5031f,-0.8007f,-0.3252f,  -0.4800f,-0.8209f,-0.3094f,  -0.3718f,-0.8480f,-0.3777f,  -0.3471f,-0.8657f,-0.3605f,  -0.2326f,-0.8759f,-0.4227f,  -0.2071f,-0.8907f,-0.4047f,  -0.0891f,-0.8831f,-0.4606f,  -0.0635f,-0.8945f,-0.4425f,  0.0548f,-0.8690f,-0.4918f,  0.0798f,-0.8767f,-0.4744f,  0.1954f,-0.8332f,-0.5173f,  
	0.2186f,-0.8371f,-0.5014f,  0.3285f,-0.7763f,-0.5380f,  0.3492f,-0.7766f,-0.5244f,  0.4504f,-0.6996f,-0.5548f,  0.4677f,-0.6963f,-0.5444f,  -0.0001f,-0.5549f,-0.8319f,  -0.0000f,-0.5553f,-0.8316f,  0.1305f,-0.5497f,-0.8251f,  0.1306f,-0.5500f,-0.8249f,  0.2588f,-0.5345f,-0.8046f,  0.2590f,-0.5346f,-0.8045f,  0.3828f,-0.5094f,-0.7707f,  0.3831f,-0.5093f,-0.7706f,  0.5004f,-0.4749f,-0.7239f,  0.5007f,-0.4744f,-0.7240f,  
	0.6094f,-0.4314f,-0.6652f,  0.6098f,-0.4306f,-0.6654f,  0.7080f,-0.3796f,-0.5955f,  0.7085f,-0.3783f,-0.5958f,  0.7944f,-0.3203f,-0.5160f,  0.7950f,-0.3184f,-0.5164f,  0.8672f,-0.2543f,-0.4281f,  0.8677f,-0.2518f,-0.4286f,  0.9249f,-0.1828f,-0.3333f,  0.9254f,-0.1795f,-0.3338f,  0.9666f,-0.1068f,-0.2332f,  0.9668f,-0.1027f,-0.2338f,  0.9912f,-0.0277f,-0.1295f,  0.9912f,-0.0227f,-0.1304f,  0.9983f,0.0533f,-0.0241f,  
	0.9979f,0.0591f,-0.0252f,  0.9876f,0.1346f,0.0811f,  0.9867f,0.1413f,0.0799f,  0.9591f,0.2150f,0.1844f,  0.9576f,0.2225f,0.1829f,  0.9131f,0.2927f,0.2839f,  0.9109f,0.3010f,0.2821f,  0.8503f,0.3663f,0.3779f,  0.8474f,0.3752f,0.3757f,  0.7717f,0.4342f,0.4647f,  0.7678f,0.4437f,0.4621f,  0.6785f,0.4950f,0.5428f,  0.6737f,0.5048f,0.5397f,  0.5723f,0.5473f,0.6107f,  0.5664f,0.5572f,0.6072f,  
	0.4549f,0.5896f,0.6674f,  0.4480f,0.5993f,0.6634f,  0.3284f,0.6208f,0.7119f,  0.3206f,0.6300f,0.7073f,  0.1952f,0.6398f,0.7434f,  0.1865f,0.6482f,0.7383f,  0.0577f,0.6457f,0.7614f,  0.0482f,0.6530f,0.7559f,  -0.0814f,0.6378f,0.7659f,  -0.0914f,0.6436f,0.7599f,  -0.2192f,0.6157f,0.7568f,  -0.2296f,0.6197f,0.7505f,  -0.3531f,0.5794f,0.7346f,  -0.3635f,0.5811f,0.7281f,  -0.4801f,0.5288f,0.6999f,  
	-0.4902f,0.5281f,0.6934f,  -0.5974f,0.4646f,0.6536f,  -0.6068f,0.4611f,0.6474f,  -0.7024f,0.3875f,0.5970f,  -0.7108f,0.3812f,0.5912f,  -0.7928f,0.2988f,0.5313f,  -0.7995f,0.2895f,0.5263f,  -0.8661f,0.1999f,0.4581f,  -0.8709f,0.1876f,0.4542f,  -0.9207f,0.0926f,0.3792f,  -0.9231f,0.0776f,0.3767f,  -0.9548f,-0.0207f,0.2964f,  -0.9546f,-0.0383f,0.2956f,  -0.9676f,-0.1376f,0.2115f,  -0.9643f,-0.1576f,0.2126f,  
	-0.9585f,-0.2556f,0.1262f,  -0.9520f,-0.2774f,0.1295f,  -0.9274f,-0.3717f,0.0423f,  -0.9175f,-0.3948f,0.0480f,  -0.8748f,-0.4829f,-0.0387f,  -0.8616f,-0.5067f,-0.0304f,  -0.8019f,-0.5862f,-0.1153f,  -0.7854f,-0.6101f,-0.1045f,  -0.7102f,-0.6788f,-0.1865f,  -0.6907f,-0.7021f,-0.1732f,  -0.6021f,-0.7579f,-0.2513f,  -0.5799f,-0.7798f,-0.2359f,  -0.4800f,-0.8209f,-0.3094f,  -0.4557f,-0.8408f,-0.2922f,  -0.3471f,-0.8657f,-0.3605f,  
	-0.3215f,-0.8830f,-0.3420f,  -0.2071f,-0.8907f,-0.4047f,  -0.1807f,-0.9048f,-0.3856f,  -0.0635f,-0.8945f,-0.4425f,  -0.0371f,-0.9052f,-0.4234f,  0.0798f,-0.8767f,-0.4744f,  0.1051f,-0.8836f,-0.4563f,  0.2186f,-0.8371f,-0.5014f,  0.2420f,-0.8403f,-0.4851f,  0.3492f,-0.7766f,-0.5244f,  0.3697f,-0.7761f,-0.5108f,  0.4677f,-0.6963f,-0.5444f,  0.4844f,-0.6927f,-0.5343f,  -0.0000f,-0.5553f,-0.8316f,  0.0000f,-0.5561f,-0.8311f,  
	0.1306f,-0.5500f,-0.8249f,  0.1307f,-0.5505f,-0.8245f,  0.2590f,-0.5346f,-0.8045f,  0.2593f,-0.5349f,-0.8042f,  0.3831f,-0.5093f,-0.7706f,  0.3835f,-0.5093f,-0.7704f,  0.5007f,-0.4744f,-0.7240f,  0.5012f,-0.4741f,-0.7239f,  0.6098f,-0.4306f,-0.6654f,  0.6104f,-0.4298f,-0.6654f,  0.7085f,-0.3783f,-0.5958f,  0.7091f,-0.3770f,-0.5958f,  0.7950f,-0.3184f,-0.5164f,  0.7957f,-0.3164f,-0.5165f,  0.8677f,-0.2518f,-0.4286f,  
	0.8684f,-0.2491f,-0.4288f,  0.9254f,-0.1795f,-0.3338f,  0.9259f,-0.1760f,-0.3342f,  0.9668f,-0.1027f,-0.2338f,  0.9672f,-0.0984f,-0.2343f,  0.9912f,-0.0227f,-0.1304f,  0.9912f,-0.0175f,-0.1310f,  0.9979f,0.0591f,-0.0252f,  0.9975f,0.0653f,-0.0260f,  0.9867f,0.1413f,0.0799f,  0.9858f,0.1485f,0.0787f,  0.9576f,0.2225f,0.1829f,  0.9560f,0.2305f,0.1815f,  0.9109f,0.3010f,0.2821f,  0.9085f,0.3098f,0.2803f,  
	0.8474f,0.3752f,0.3757f,  0.8440f,0.3848f,0.3736f,  0.7678f,0.4437f,0.4621f,  0.7635f,0.4538f,0.4595f,  0.6737f,0.5048f,0.5397f,  0.6683f,0.5152f,0.5366f,  0.5664f,0.5572f,0.6072f,  0.5600f,0.5676f,0.6035f,  0.4480f,0.5993f,0.6634f,  0.4405f,0.6095f,0.6591f,  0.3206f,0.6300f,0.7073f,  0.3120f,0.6397f,0.7025f,  0.1865f,0.6482f,0.7383f,  0.1770f,0.6569f,0.7329f,  0.0482f,0.6530f,0.7559f,  
	0.0380f,0.6604f,0.7500f,  -0.0914f,0.6436f,0.7599f,  -0.1022f,0.6493f,0.7536f,  -0.2296f,0.6197f,0.7505f,  -0.2407f,0.6234f,0.7439f,  -0.3635f,0.5811f,0.7281f,  -0.3745f,0.5825f,0.7214f,  -0.4902f,0.5281f,0.6934f,  -0.5007f,0.5269f,0.6868f,  -0.6068f,0.4611f,0.6474f,  -0.6166f,0.4571f,0.6410f,  -0.7108f,0.3812f,0.5912f,  -0.7192f,0.3741f,0.5854f,  -0.7995f,0.2895f,0.5263f,  -0.8063f,0.2794f,0.5213f,  
	-0.8709f,0.1876f,0.4542f,  -0.8756f,0.1745f,0.4505f,  -0.9231f,0.0776f,0.3767f,  -0.9251f,0.0616f,0.3746f,  -0.9546f,-0.0383f,0.2956f,  -0.9537f,-0.0569f,0.2954f,  -0.9643f,-0.1576f,0.2126f,  -0.9603f,-0.1785f,0.2146f,  -0.9520f,-0.2774f,0.1295f,  -0.9445f,-0.3000f,0.1339f,  -0.9175f,-0.3948f,0.0480f,  -0.9065f,-0.4186f,0.0551f,  -0.8616f,-0.5067f,-0.0304f,  -0.8471f,-0.5310f,-0.0207f,  -0.7854f,-0.6101f,-0.1045f,  
	-0.7676f,-0.6343f,-0.0921f,  -0.6907f,-0.7021f,-0.1732f,  -0.6699f,-0.7253f,-0.1584f,  -0.5799f,-0.7798f,-0.2359f,  -0.5566f,-0.8015f,-0.2189f,  -0.4557f,-0.8408f,-0.2922f,  -0.4304f,-0.8602f,-0.2734f,  -0.3215f,-0.8830f,-0.3420f,  -0.2948f,-0.8996f,-0.3221f,  -0.1807f,-0.9048f,-0.3856f,  -0.1534f,-0.9182f,-0.3652f,  -0.0371f,-0.9052f,-0.4234f,  -0.0103f,-0.9150f,-0.4034f,  0.1051f,-0.8836f,-0.4563f,  0.1307f,-0.8897f,-0.4375f,  
	0.2420f,-0.8403f,-0.4851f,  0.2654f,-0.8427f,-0.4684f,  0.3697f,-0.7761f,-0.5108f,  0.3900f,-0.7751f,-0.4972f,  0.4844f,-0.6927f,-0.5343f,  0.5007f,-0.6886f,-0.5245f,  0.0000f,-0.5561f,-0.8311f,  0.0002f,-0.5571f,-0.8305f,  0.1307f,-0.5505f,-0.8245f,  0.1310f,-0.5513f,-0.8239f,  0.2593f,-0.5349f,-0.8042f,  0.2597f,-0.5354f,-0.8037f,  0.3835f,-0.5093f,-0.7704f,  0.3840f,-0.5095f,-0.7700f,  0.5012f,-0.4741f,-0.7239f,  
	0.5019f,-0.4739f,-0.7236f,  0.6104f,-0.4298f,-0.6654f,  0.6112f,-0.4291f,-0.6651f,  0.7091f,-0.3770f,-0.5958f,  0.7100f,-0.3757f,-0.5957f,  0.7957f,-0.3164f,-0.5165f,  0.7965f,-0.3144f,-0.5164f,  0.8684f,-0.2491f,-0.4288f,  0.8692f,-0.2463f,-0.4288f,  0.9259f,-0.1760f,-0.3342f,  0.9266f,-0.1723f,-0.3343f,  0.9672f,-0.0984f,-0.2343f,  0.9676f,-0.0937f,-0.2346f,  0.9912f,-0.0175f,-0.1310f,  0.9912f,-0.0118f,-0.1315f,  
	0.9975f,0.0653f,-0.0260f,  0.9970f,0.0720f,-0.0268f,  0.9858f,0.1485f,0.0787f,  0.9847f,0.1561f,0.0778f,  0.9560f,0.2305f,0.1815f,  0.9541f,0.2391f,0.1802f,  0.9085f,0.3098f,0.2803f,  0.9058f,0.3193f,0.2786f,  0.8440f,0.3848f,0.3736f,  0.8403f,0.3949f,0.3714f,  0.7635f,0.4538f,0.4595f,  0.7587f,0.4644f,0.4568f,  0.6683f,0.5152f,0.5366f,  0.6623f,0.5262f,0.5333f,  0.5600f,0.5676f,0.6035f,  
	0.5529f,0.5786f,0.5997f,  0.4405f,0.6095f,0.6591f,  0.4322f,0.6202f,0.6546f,  0.3120f,0.6397f,0.7025f,  0.3027f,0.6497f,0.6974f,  0.1770f,0.6569f,0.7329f,  0.1667f,0.6659f,0.7272f,  0.0380f,0.6604f,0.7500f,  0.0269f,0.6679f,0.7438f,  -0.1022f,0.6493f,0.7536f,  -0.1138f,0.6551f,0.7469f,  -0.2407f,0.6234f,0.7439f,  -0.2525f,0.6270f,0.7370f,  -0.3745f,0.5825f,0.7214f,  -0.3861f,0.5836f,0.7144f,  
	-0.5007f,0.5269f,0.6868f,  -0.5118f,0.5252f,0.6798f,  -0.6166f,0.4571f,0.6410f,  -0.6267f,0.4524f,0.6345f,  -0.7192f,0.3741f,0.5854f,  -0.7279f,0.3663f,0.5796f,  -0.8063f,0.2794f,0.5213f,  -0.8130f,0.2684f,0.5166f,  -0.8756f,0.1745f,0.4505f,  -0.8799f,0.1605f,0.4472f,  -0.9251f,0.0616f,0.3746f,  -0.9267f,0.0447f,0.3731f,  -0.9537f,-0.0569f,0.2954f,  -0.9521f,-0.0764f,0.2959f,  -0.9603f,-0.1785f,0.2146f,  
	-0.9553f,-0.2001f,0.2176f,  -0.9445f,-0.3000f,0.1339f,  -0.9360f,-0.3233f,0.1396f,  -0.9065f,-0.4186f,0.0551f,  -0.8944f,-0.4428f,0.0635f,  -0.8471f,-0.5310f,-0.0207f,  -0.8314f,-0.5556f,-0.0094f,  -0.7676f,-0.6343f,-0.0921f,  -0.7485f,-0.6585f,-0.0781f,  -0.6699f,-0.7253f,-0.1584f,  -0.6479f,-0.7484f,-0.1418f,  -0.5566f,-0.8015f,-0.2189f,  -0.5320f,-0.8228f,-0.2002f,  -0.4304f,-0.8602f,-0.2734f,  -0.4039f,-0.8791f,-0.2531f,  
	-0.2948f,-0.8996f,-0.3221f,  -0.2671f,-0.9155f,-0.3007f,  -0.1534f,-0.9182f,-0.3652f,  -0.1254f,-0.9307f,-0.3436f,  -0.0103f,-0.9150f,-0.4034f,  0.0172f,-0.9238f,-0.3824f,  0.1307f,-0.8897f,-0.4375f,  0.1566f,-0.8948f,-0.4181f,  0.2654f,-0.8427f,-0.4684f,  0.2889f,-0.8442f,-0.4515f,  0.3900f,-0.7751f,-0.4972f,  0.4100f,-0.7734f,-0.4835f,  0.5007f,-0.6886f,-0.5245f,  0.5164f,-0.6842f,-0.5151f,  0.0002f,-0.5571f,-0.8305f,  
	0.0005f,-0.5584f,-0.8296f,  0.1310f,-0.5513f,-0.8239f,  0.1315f,-0.5524f,-0.8231f,  0.2597f,-0.5354f,-0.8037f,  0.2603f,-0.5362f,-0.8029f,  0.3840f,-0.5095f,-0.7700f,  0.3848f,-0.5099f,-0.7694f,  0.5019f,-0.4739f,-0.7236f,  0.5027f,-0.4739f,-0.7230f,  0.6112f,-0.4291f,-0.6651f,  0.6122f,-0.4285f,-0.6646f,  0.7100f,-0.3757f,-0.5957f,  0.7110f,-0.3744f,-0.5952f,  0.7965f,-0.3144f,-0.5164f,  0.7976f,-0.3123f,-0.5161f,  
	0.8692f,-0.2463f,-0.4288f,  0.8701f,-0.2433f,-0.4286f,  0.9266f,-0.1723f,-0.3343f,  0.9273f,-0.1684f,-0.3342f,  0.9676f,-0.0937f,-0.2346f,  0.9680f,-0.0888f,-0.2347f,  0.9912f,-0.0118f,-0.1315f,  0.9913f,-0.0058f,-0.1318f,  0.9970f,0.0720f,-0.0268f,  0.9965f,0.0791f,-0.0273f,  0.9847f,0.1561f,0.0778f,  0.9834f,0.1643f,0.0769f,  0.9541f,0.2391f,0.1802f,  0.9520f,0.2483f,0.1789f,  0.9058f,0.3193f,0.2786f,  
	0.9027f,0.3293f,0.2769f,  0.8403f,0.3949f,0.3714f,  0.8361f,0.4057f,0.3692f,  0.7587f,0.4644f,0.4568f,  0.7534f,0.4757f,0.4540f,  0.6623f,0.5262f,0.5333f,  0.6558f,0.5377f,0.5299f,  0.5529f,0.5786f,0.5997f,  0.5451f,0.5900f,0.5956f,  0.4322f,0.6202f,0.6546f,  0.4232f,0.6313f,0.6499f,  0.3027f,0.6497f,0.6974f,  0.2926f,0.6600f,0.6920f,  0.1667f,0.6659f,0.7272f,  0.1556f,0.6750f,0.7212f,  
	0.0269f,0.6679f,0.7438f,  0.0151f,0.6755f,0.7372f,  -0.1138f,0.6551f,0.7469f,  -0.1261f,0.6608f,0.7399f,  -0.2525f,0.6270f,0.7370f,  -0.2649f,0.6304f,0.7297f,  -0.3861f,0.5836f,0.7144f,  -0.3983f,0.5843f,0.7070f,  -0.5118f,0.5252f,0.6798f,  -0.5233f,0.5230f,0.6728f,  -0.6267f,0.4524f,0.6345f,  -0.6370f,0.4471f,0.6279f,  -0.7279f,0.3663f,0.5796f,  -0.7366f,0.3578f,0.5739f,  -0.8130f,0.2684f,0.5166f,  
	-0.8196f,0.2567f,0.5122f,  -0.8799f,0.1605f,0.4472f,  -0.8839f,0.1456f,0.4444f,  -0.9267f,0.0447f,0.3731f,  -0.9277f,0.0269f,0.3723f,  -0.9521f,-0.0764f,0.2959f,  -0.9498f,-0.0967f,0.2974f,  -0.9553f,-0.2001f,0.2176f,  -0.9494f,-0.2224f,0.2217f,  -0.9360f,-0.3233f,0.1396f,  -0.9263f,-0.3471f,0.1466f,  -0.8944f,-0.4428f,0.0635f,  -0.8810f,-0.4675f,0.0735f,  -0.8314f,-0.5556f,-0.0094f,  -0.8143f,-0.5804f,0.0035f,  
	-0.7485f,-0.6585f,-0.0781f,  -0.7281f,-0.6826f,-0.0624f,  -0.6479f,-0.7484f,-0.1418f,  -0.6245f,-0.7712f,-0.1235f,  -0.5320f,-0.8228f,-0.2002f,  -0.5061f,-0.8435f,-0.1797f,  -0.4039f,-0.8791f,-0.2531f,  -0.3762f,-0.8972f,-0.2311f,  -0.2671f,-0.9155f,-0.3007f,  -0.2384f,-0.9305f,-0.2780f,  -0.1254f,-0.9307f,-0.3436f,  -0.0966f,-0.9422f,-0.3208f,  0.0172f,-0.9238f,-0.3824f,  0.0452f,-0.9316f,-0.3606f,  0.1566f,-0.8948f,-0.4181f,  
	0.1828f,-0.8989f,-0.3981f,  0.2889f,-0.8442f,-0.4515f,  0.3123f,-0.8449f,-0.4343f,  0.4100f,-0.7734f,-0.4835f,  0.4297f,-0.7710f,-0.4700f,  0.5164f,-0.6842f,-0.5151f,  0.5314f,-0.6793f,-0.5060f,  0.0005f,-0.5584f,-0.8296f,  0.0010f,-0.5600f,-0.8285f,  0.1315f,-0.5524f,-0.8231f,  0.1321f,-0.5538f,-0.8221f,  0.2603f,-0.5362f,-0.8029f,  0.2611f,-0.5373f,-0.8020f,  0.3848f,-0.5099f,-0.7694f,  0.3857f,-0.5106f,-0.7685f,  
	0.5027f,-0.4739f,-0.7230f,  0.5039f,-0.4740f,-0.7221f,  0.6122f,-0.4285f,-0.6646f,  0.6134f,-0.4279f,-0.6638f,  0.7110f,-0.3744f,-0.5952f,  0.7123f,-0.3731f,-0.5945f,  0.7976f,-0.3123f,-0.5161f,  0.7988f,-0.3102f,-0.5155f,  0.8701f,-0.2433f,-0.4286f,  0.8712f,-0.2402f,-0.4281f,  0.9273f,-0.1684f,-0.3342f,  0.9282f,-0.1642f,-0.3339f,  0.9680f,-0.0888f,-0.2347f,  0.9685f,-0.0835f,-0.2346f,  0.9913f,-0.0058f,-0.1318f,  
	0.9913f,0.0007f,-0.1319f,  0.9965f,0.0791f,-0.0273f,  0.9958f,0.0867f,-0.0277f,  0.9834f,0.1643f,0.0769f,  0.9820f,0.1731f,0.0762f,  0.9520f,0.2483f,0.1789f,  0.9496f,0.2581f,0.1777f,  0.9027f,0.3293f,0.2769f,  0.8992f,0.3400f,0.2753f,  0.8361f,0.4057f,0.3692f,  0.8315f,0.4171f,0.3670f,  0.7534f,0.4757f,0.4540f,  0.7474f,0.4876f,0.4512f,  0.6558f,0.5377f,0.5299f,  0.6485f,0.5498f,0.5264f,  
	0.5451f,0.5900f,0.5956f,  0.5365f,0.6020f,0.5913f,  0.4232f,0.6313f,0.6499f,  0.4134f,0.6428f,0.6449f,  0.2926f,0.6600f,0.6920f,  0.2816f,0.6706f,0.6863f,  0.1556f,0.6750f,0.7212f,  0.1437f,0.6844f,0.7148f,  0.0151f,0.6755f,0.7372f,  0.0024f,0.6832f,0.7302f,  -0.1261f,0.6608f,0.7399f,  -0.1391f,0.6663f,0.7326f,  -0.2649f,0.6304f,0.7297f,  -0.2780f,0.6334f,0.7221f,  -0.3983f,0.5843f,0.7070f,  
	-0.4110f,0.5846f,0.6995f,  -0.5233f,0.5230f,0.6728f,  -0.5353f,0.5201f,0.6655f,  -0.6370f,0.4471f,0.6279f,  -0.6476f,0.4410f,0.6214f,  -0.7366f,0.3578f,0.5739f,  -0.7453f,0.3484f,0.5685f,  -0.8196f,0.2567f,0.5122f,  -0.8260f,0.2440f,0.5082f,  -0.8839f,0.1456f,0.4444f,  -0.8875f,0.1298f,0.4422f,  -0.9277f,0.0269f,0.3723f,  -0.9281f,0.0083f,0.3723f,  -0.9498f,-0.0967f,0.2974f,  -0.9466f,-0.1178f,0.3000f,  
	-0.9494f,-0.2224f,0.2217f,  -0.9424f,-0.2454f,0.2271f,  -0.9263f,-0.3471f,0.1466f,  -0.9155f,-0.3713f,0.1551f,  -0.8810f,-0.4675f,0.0735f,  -0.8662f,-0.4924f,0.0850f,  -0.8143f,-0.5804f,0.0035f,  -0.7959f,-0.6052f,0.0182f,  -0.7281f,-0.6826f,-0.0624f,  -0.7063f,-0.7065f,-0.0447f,  -0.6245f,-0.7712f,-0.1235f,  -0.5996f,-0.7936f,-0.1033f,  -0.5061f,-0.8435f,-0.1797f,  -0.4789f,-0.8636f,-0.1576f,  -0.3762f,-0.8972f,-0.2311f,  
	-0.3474f,-0.9145f,-0.2075f,  -0.2384f,-0.9305f,-0.2780f,  -0.2088f,-0.9445f,-0.2538f,  -0.0966f,-0.9422f,-0.3208f,  -0.0670f,-0.9525f,-0.2969f,  0.0452f,-0.9316f,-0.3606f,  0.0737f,-0.9383f,-0.3379f,  0.1828f,-0.8989f,-0.3981f,  0.2092f,-0.9020f,-0.3776f,  0.3123f,-0.8449f,-0.4343f,  0.3356f,-0.8447f,-0.4169f,  0.4297f,-0.7710f,-0.4700f,  0.4490f,-0.7680f,-0.4567f,  0.5314f,-0.6793f,-0.5060f,  0.5459f,-0.6743f,-0.4973f,  
	0.0010f,-0.5600f,-0.8285f,  0.0016f,-0.5618f,-0.8272f,  0.1321f,-0.5538f,-0.8221f,  0.1329f,-0.5554f,-0.8209f,  0.2611f,-0.5373f,-0.8020f,  0.2621f,-0.5385f,-0.8008f,  0.3857f,-0.5106f,-0.7685f,  0.3869f,-0.5113f,-0.7673f,  0.5039f,-0.4740f,-0.7221f,  0.5052f,-0.4741f,-0.7211f,  0.6134f,-0.4279f,-0.6638f,  0.6148f,-0.4274f,-0.6628f,  0.7123f,-0.3731f,-0.5945f,  0.7138f,-0.3717f,-0.5936f,  0.7988f,-0.3102f,-0.5155f,  
	0.8002f,-0.3079f,-0.5147f,  0.8712f,-0.2402f,-0.4281f,  0.8725f,-0.2368f,-0.4274f,  0.9282f,-0.1642f,-0.3339f,  0.9292f,-0.1597f,-0.3333f,  0.9685f,-0.0835f,-0.2346f,  0.9691f,-0.0777f,-0.2342f,  0.9913f,0.0007f,-0.1319f,  0.9912f,0.0076f,-0.1318f,  0.9958f,0.0867f,-0.0277f,  0.9951f,0.0949f,-0.0279f,  0.9820f,0.1731f,0.0762f,  0.9803f,0.1824f,0.0755f,  0.9496f,0.2581f,0.1777f,  0.9469f,0.2685f,0.1766f,  
	0.8992f,0.3400f,0.2753f,  0.8954f,0.3514f,0.2736f,  0.8315f,0.4171f,0.3670f,  0.8263f,0.4292f,0.3647f,  0.7474f,0.4876f,0.4512f,  0.7409f,0.5002f,0.4482f,  0.6485f,0.5498f,0.5264f,  0.6406f,0.5625f,0.5226f,  0.5365f,0.6020f,0.5913f,  0.5272f,0.6146f,0.5868f,  0.4134f,0.6428f,0.6449f,  0.4028f,0.6547f,0.6396f,  0.2816f,0.6706f,0.6863f,  0.2698f,0.6816f,0.6802f,  0.1437f,0.6844f,0.7148f,  
	0.1310f,0.6939f,0.7080f,  0.0024f,0.6832f,0.7302f,  -0.0110f,0.6908f,0.7229f,  -0.1391f,0.6663f,0.7326f,  -0.1529f,0.6717f,0.7249f,  -0.2780f,0.6334f,0.7221f,  -0.2917f,0.6362f,0.7143f,  -0.4110f,0.5846f,0.6995f,  -0.4243f,0.5843f,0.6918f,  -0.5353f,0.5201f,0.6655f,  -0.5474f,0.5167f,0.6583f,  -0.6476f,0.4410f,0.6214f,  -0.6582f,0.4343f,0.6150f,  -0.7453f,0.3484f,0.5685f,  -0.7539f,0.3383f,0.5633f,  
	-0.8260f,0.2440f,0.5082f,  -0.8320f,0.2305f,0.5046f,  -0.8875f,0.1298f,0.4422f,  -0.8905f,0.1132f,0.4407f,  -0.9281f,0.0083f,0.3723f,  -0.9277f,-0.0111f,0.3732f,  -0.9466f,-0.1178f,0.3000f,  -0.9425f,-0.1395f,0.3038f,  -0.9424f,-0.2454f,0.2271f,  -0.9343f,-0.2688f,0.2340f,  -0.9155f,-0.3713f,0.1551f,  -0.9033f,-0.3959f,0.1651f,  -0.8662f,-0.4924f,0.0850f,  -0.8501f,-0.5174f,0.0984f,  -0.7959f,-0.6052f,0.0182f,  
	-0.7760f,-0.6298f,0.0348f,  -0.7063f,-0.7065f,-0.0447f,  -0.6829f,-0.7300f,-0.0252f,  -0.5996f,-0.7936f,-0.1033f,  -0.5734f,-0.8152f,-0.0814f,  -0.4789f,-0.8636f,-0.1576f,  -0.4504f,-0.8828f,-0.1336f,  -0.3474f,-0.9145f,-0.2075f,  -0.3174f,-0.9306f,-0.1824f,  -0.2088f,-0.9445f,-0.2538f,  -0.1781f,-0.9572f,-0.2282f,  -0.0670f,-0.9525f,-0.2969f,  -0.0367f,-0.9616f,-0.2720f,  0.0737f,-0.9383f,-0.3379f,  0.1026f,-0.9437f,-0.3145f,  
	0.2092f,-0.9020f,-0.3776f,  0.2358f,-0.9040f,-0.3567f,  0.3356f,-0.8447f,-0.4169f,  0.3589f,-0.8436f,-0.3995f,  0.4490f,-0.7680f,-0.4567f,  0.4679f,-0.7644f,-0.4435f,  0.5459f,-0.6743f,-0.4973f,  0.5597f,-0.6690f,-0.4891f,  0.0016f,-0.5618f,-0.8272f,  0.0025f,-0.5640f,-0.8258f,  0.1329f,-0.5554f,-0.8209f,  0.1340f,-0.5572f,-0.8195f,  0.2621f,-0.5385f,-0.8008f,  0.2634f,-0.5399f,-0.7994f,  0.3869f,-0.5113f,-0.7673f,  
	0.3884f,-0.5122f,-0.7660f,  0.5052f,-0.4741f,-0.7211f,  0.5069f,-0.4744f,-0.7198f,  0.6148f,-0.4274f,-0.6628f,  0.6166f,-0.4269f,-0.6615f,  0.7138f,-0.3717f,-0.5936f,  0.7155f,-0.3703f,-0.5924f,  0.8002f,-0.3079f,-0.5147f,  0.8019f,-0.3054f,-0.5135f,  0.8725f,-0.2368f,-0.4274f,  0.8740f,-0.2332f,-0.4264f,  0.9292f,-0.1597f,-0.3333f,  0.9303f,-0.1548f,-0.3325f,  0.9691f,-0.0777f,-0.2342f,  0.9697f,-0.0716f,-0.2336f,  
	0.9912f,0.0076f,-0.1318f,  0.9912f,0.0151f,-0.1315f,  0.9951f,0.0949f,-0.0279f,  0.9942f,0.1037f,-0.0280f,  0.9803f,0.1824f,0.0755f,  0.9784f,0.1924f,0.0750f,  0.9469f,0.2685f,0.1766f,  0.9439f,0.2796f,0.1756f,  0.8954f,0.3514f,0.2736f,  0.8911f,0.3634f,0.2719f,  0.8263f,0.4292f,0.3647f,  0.8206f,0.4420f,0.3623f,  0.7409f,0.5002f,0.4482f,  0.7338f,0.5134f,0.4450f,  0.6406f,0.5625f,0.5226f,  
	0.6320f,0.5758f,0.5186f,  0.5272f,0.6146f,0.5868f,  0.5171f,0.6276f,0.5820f,  0.4028f,0.6547f,0.6396f,  0.3914f,0.6671f,0.6339f,  0.2698f,0.6816f,0.6802f,  0.2572f,0.6928f,0.6737f,  0.1310f,0.6939f,0.7080f,  0.1174f,0.7035f,0.7009f,  -0.0110f,0.6908f,0.7229f,  -0.0252f,0.6985f,0.7152f,  -0.1529f,0.6717f,0.7249f,  -0.1674f,0.6769f,0.7168f,  -0.2917f,0.6362f,0.7143f,  -0.3061f,0.6385f,0.7061f,  
	-0.4243f,0.5843f,0.6918f,  -0.4379f,0.5836f,0.6838f,  -0.5474f,0.5167f,0.6583f,  -0.5598f,0.5127f,0.6510f,  -0.6582f,0.4343f,0.6150f,  -0.6689f,0.4267f,0.6087f,  -0.7539f,0.3383f,0.5633f,  -0.7623f,0.3272f,0.5584f,  -0.8320f,0.2305f,0.5046f,  -0.8376f,0.2162f,0.5016f,  -0.8905f,0.1132f,0.4407f,  -0.8928f,0.0958f,0.4401f,  -0.9277f,-0.0111f,0.3732f,  -0.9264f,-0.0312f,0.3753f,  -0.9425f,-0.1395f,0.3038f,  
	-0.9372f,-0.1618f,0.3090f,  -0.9343f,-0.2688f,0.2340f,  -0.9250f,-0.2927f,0.2424f,  -0.9033f,-0.3959f,0.1651f,  -0.8898f,-0.4207f,0.1770f,  -0.8501f,-0.5174f,0.0984f,  -0.8325f,-0.5422f,0.1138f,  -0.7760f,-0.6298f,0.0348f,  -0.7546f,-0.6540f,0.0534f,  -0.6829f,-0.7300f,-0.0252f,  -0.6581f,-0.7529f,-0.0037f,  -0.5734f,-0.8152f,-0.0814f,  -0.5457f,-0.8360f,-0.0574f,  -0.4504f,-0.8828f,-0.1336f,  -0.4205f,-0.9009f,-0.1079f,  
	-0.3174f,-0.9306f,-0.1824f,  -0.2862f,-0.9455f,-0.1556f,  -0.1781f,-0.9572f,-0.2282f,  -0.1465f,-0.9685f,-0.2014f,  -0.0367f,-0.9616f,-0.2720f,  -0.0057f,-0.9693f,-0.2460f,  0.1026f,-0.9437f,-0.3145f,  0.1321f,-0.9477f,-0.2904f,  0.2358f,-0.9040f,-0.3567f,  0.2626f,-0.9047f,-0.3356f,  0.3589f,-0.8436f,-0.3995f,  0.3820f,-0.8415f,-0.3821f,  0.4679f,-0.7644f,-0.4435f,  0.4864f,-0.7602f,-0.4306f,  0.5597f,-0.6690f,-0.4891f,  
	0.5728f,-0.6635f,-0.4812f,  0.0025f,-0.5640f,-0.8258f,  0.0036f,-0.5664f,-0.8241f,  0.1340f,-0.5572f,-0.8195f,  0.1354f,-0.5593f,-0.8178f,  0.2634f,-0.5399f,-0.7994f,  0.2650f,-0.5416f,-0.7978f,  0.3884f,-0.5122f,-0.7660f,  0.3902f,-0.5133f,-0.7644f,  0.5069f,-0.4744f,-0.7198f,  0.5088f,-0.4747f,-0.7182f,  0.6166f,-0.4269f,-0.6615f,  0.6186f,-0.4263f,-0.6600f,  0.7155f,-0.3703f,-0.5924f,  0.7176f,-0.3687f,-0.5909f,  
	0.8019f,-0.3054f,-0.5135f,  0.8038f,-0.3027f,-0.5122f,  0.8740f,-0.2332f,-0.4264f,  0.8756f,-0.2293f,-0.4252f,  0.9303f,-0.1548f,-0.3325f,  0.9315f,-0.1496f,-0.3315f,  0.9697f,-0.0716f,-0.2336f,  0.9703f,-0.0650f,-0.2328f,  0.9912f,0.0151f,-0.1315f,  0.9911f,0.0231f,-0.1310f,  0.9942f,0.1037f,-0.0280f,  0.9932f,0.1131f,-0.0279f,  0.9784f,0.1924f,0.0750f,  0.9763f,0.2031f,0.0745f,  0.9439f,0.2796f,0.1756f,  
	0.9405f,0.2914f,0.1745f,  0.8911f,0.3634f,0.2719f,  0.8863f,0.3762f,0.2702f,  0.8206f,0.4420f,0.3623f,  0.8144f,0.4554f,0.3597f,  0.7338f,0.5134f,0.4450f,  0.7260f,0.5272f,0.4416f,  0.6320f,0.5758f,0.5186f,  0.6226f,0.5897f,0.5144f,  0.5171f,0.6276f,0.5820f,  0.5062f,0.6411f,0.5768f,  0.3914f,0.6671f,0.6339f,  0.3791f,0.6798f,0.6279f,  0.2572f,0.6928f,0.6737f,  0.2438f,0.7042f,0.6668f,  
	0.1174f,0.7035f,0.7009f,  0.1030f,0.7133f,0.6933f,  -0.0252f,0.6985f,0.7152f,  -0.0402f,0.7060f,0.7071f,  -0.1674f,0.6769f,0.7168f,  -0.1826f,0.6818f,0.7084f,  -0.3061f,0.6385f,0.7061f,  -0.3209f,0.6405f,0.6977f,  -0.4379f,0.5836f,0.6838f,  -0.4519f,0.5823f,0.6758f,  -0.5598f,0.5127f,0.6510f,  -0.5725f,0.5079f,0.6437f,  -0.6689f,0.4267f,0.6087f,  -0.6796f,0.4184f,0.6026f,  -0.7623f,0.3272f,0.5584f,  
	-0.7705f,0.3154f,0.5540f,  -0.8376f,0.2162f,0.5016f,  -0.8427f,0.2010f,0.4994f,  -0.8928f,0.0958f,0.4401f,  -0.8944f,0.0777f,0.4404f,  -0.9264f,-0.0312f,0.3753f,  -0.9241f,-0.0519f,0.3786f,  -0.9372f,-0.1618f,0.3090f,  -0.9308f,-0.1845f,0.3155f,  -0.9250f,-0.2927f,0.2424f,  -0.9142f,-0.3169f,0.2525f,  -0.8898f,-0.4207f,0.1770f,  -0.8747f,-0.4454f,0.1908f,  -0.8325f,-0.5422f,0.1138f,  -0.8133f,-0.5669f,0.1311f,  
	-0.7546f,-0.6540f,0.0534f,  -0.7315f,-0.6778f,0.0740f,  -0.6581f,-0.7529f,-0.0037f,  -0.6317f,-0.7750f,0.0198f,  -0.5457f,-0.8360f,-0.0574f,  -0.5165f,-0.8557f,-0.0315f,  -0.4205f,-0.9009f,-0.1079f,  -0.3893f,-0.9176f,-0.0803f,  -0.2862f,-0.9455f,-0.1556f,  -0.2538f,-0.9588f,-0.1273f,  -0.1465f,-0.9685f,-0.2014f,  -0.1140f,-0.9783f,-0.1733f,  -0.0057f,-0.9693f,-0.2460f,  0.0260f,-0.9753f,-0.2192f,  0.1321f,-0.9477f,-0.2904f,  
	0.1619f,-0.9503f,-0.2659f,  0.2626f,-0.9047f,-0.3356f,  0.2895f,-0.9041f,-0.3142f,  0.3820f,-0.8415f,-0.3821f,  0.4049f,-0.8384f,-0.3648f,  0.4864f,-0.7602f,-0.4306f,  0.5045f,-0.7554f,-0.4181f,  0.5728f,-0.6635f,-0.4812f,  0.5853f,-0.6579f,-0.4739f,  0.0036f,-0.5664f,-0.8241f,  0.0050f,-0.5691f,-0.8222f,  0.1354f,-0.5593f,-0.8178f,  0.1370f,-0.5617f,-0.8159f,  0.2650f,-0.5416f,-0.7978f,  0.2669f,-0.5434f,-0.7959f,  
	0.3902f,-0.5133f,-0.7644f,  0.3923f,-0.5145f,-0.7625f,  0.5088f,-0.4747f,-0.7182f,  0.5111f,-0.4751f,-0.7163f,  0.6186f,-0.4263f,-0.6600f,  0.6209f,-0.4257f,-0.6582f,  0.7176f,-0.3687f,-0.5909f,  0.7199f,-0.3670f,-0.5892f,  0.8038f,-0.3027f,-0.5122f,  0.8059f,-0.2998f,-0.5105f,  0.8756f,-0.2293f,-0.4252f,  0.8774f,-0.2250f,-0.4236f,  0.9315f,-0.1496f,-0.3315f,  0.9329f,-0.1439f,-0.3302f,  0.9703f,-0.0650f,-0.2328f,  
	0.9710f,-0.0578f,-0.2318f,  0.9911f,0.0231f,-0.1310f,  0.9910f,0.0317f,-0.1304f,  0.9932f,0.1131f,-0.0279f,  0.9920f,0.1231f,-0.0278f,  0.9763f,0.2031f,0.0745f,  0.9739f,0.2145f,0.0741f,  0.9405f,0.2914f,0.1745f,  0.9368f,0.3040f,0.1734f,  0.8863f,0.3762f,0.2702f,  0.8810f,0.3897f,0.2683f,  0.8144f,0.4554f,0.3597f,  0.8075f,0.4696f,0.3570f,  0.7260f,0.5272f,0.4416f,  0.7174f,0.5417f,0.4380f,  
	0.6226f,0.5897f,0.5144f,  0.6124f,0.6042f,0.5098f,  0.5062f,0.6411f,0.5768f,  0.4945f,0.6551f,0.5713f,  0.3791f,0.6798f,0.6279f,  0.3659f,0.6928f,0.6214f,  0.2438f,0.7042f,0.6668f,  0.2293f,0.7159f,0.6595f,  0.1030f,0.7133f,0.6933f,  0.0877f,0.7230f,0.6853f,  -0.0402f,0.7060f,0.7071f,  -0.0560f,0.7134f,0.6986f,  -0.1826f,0.6818f,0.7084f,  -0.1984f,0.6864f,0.6997f,  -0.3209f,0.6405f,0.6977f,  
	-0.3362f,0.6420f,0.6891f,  -0.4519f,0.5823f,0.6758f,  -0.4662f,0.5804f,0.6677f,  -0.5725f,0.5079f,0.6437f,  -0.5852f,0.5024f,0.6365f,  -0.6796f,0.4184f,0.6026f,  -0.6901f,0.4093f,0.5968f,  -0.7705f,0.3154f,0.5540f,  -0.7782f,0.3029f,0.5501f,  -0.8427f,0.2010f,0.4994f,  -0.8472f,0.1852f,0.4979f,  -0.8944f,0.0777f,0.4404f,  -0.8952f,0.0589f,0.4418f,  -0.9241f,-0.0519f,0.3786f,  -0.9207f,-0.0731f,0.3833f,  
	-0.9308f,-0.1845f,0.3155f,  -0.9231f,-0.2076f,0.3238f,  -0.9142f,-0.3169f,0.2525f,  -0.9021f,-0.3410f,0.2645f,  -0.8747f,-0.4454f,0.1908f,  -0.8582f,-0.4700f,0.2065f,  -0.8133f,-0.5669f,0.1311f,  -0.7925f,-0.5910f,0.1504f,  -0.7315f,-0.6778f,0.0740f,  -0.7069f,-0.7007f,0.0967f,  -0.6317f,-0.7750f,0.0198f,  -0.6036f,-0.7960f,0.0454f,  -0.5165f,-0.8557f,-0.0315f,  -0.4858f,-0.8741f,-0.0037f,  -0.3893f,-0.9176f,-0.0803f,  
	-0.3567f,-0.9328f,-0.0512f,  -0.2538f,-0.9588f,-0.1273f,  -0.2202f,-0.9705f,-0.0977f,  -0.1140f,-0.9783f,-0.1733f,  -0.0805f,-0.9863f,-0.1442f,  0.0260f,-0.9753f,-0.2192f,  0.0584f,-0.9797f,-0.1917f,  0.1619f,-0.9503f,-0.2659f,  0.1921f,-0.9513f,-0.2409f,  0.2895f,-0.9041f,-0.3142f,  0.3164f,-0.9023f,-0.2928f,  0.4049f,-0.8384f,-0.3648f,  0.4276f,-0.8344f,-0.3478f,  0.5045f,-0.7554f,-0.4181f,  0.5221f,-0.7501f,-0.4059f,  
	0.5853f,-0.6579f,-0.4739f,  0.5971f,-0.6522f,-0.4670f,  0.0050f,-0.5691f,-0.8222f,  0.0067f,-0.5721f,-0.8201f,  0.1370f,-0.5617f,-0.8159f,  0.1390f,-0.5642f,-0.8138f,  0.2669f,-0.5434f,-0.7959f,  0.2692f,-0.5454f,-0.7938f,  0.3923f,-0.5145f,-0.7625f,  0.3948f,-0.5157f,-0.7604f,  0.5111f,-0.4751f,-0.7163f,  0.5137f,-0.4755f,-0.7142f,  0.6209f,-0.4257f,-0.6582f,  0.6236f,-0.4251f,-0.6561f,  0.7199f,-0.3670f,-0.5892f,  
	0.7225f,-0.3652f,-0.5871f,  0.8059f,-0.2998f,-0.5105f,  0.8083f,-0.2966f,-0.5086f,  0.8774f,-0.2250f,-0.4236f,  0.8795f,-0.2204f,-0.4219f,  0.9329f,-0.1439f,-0.3302f,  0.9343f,-0.1378f,-0.3286f,  0.9710f,-0.0578f,-0.2318f,  0.9718f,-0.0501f,-0.2306f,  0.9910f,0.0317f,-0.1304f,  0.9907f,0.0410f,-0.1296f,  0.9920f,0.1231f,-0.0278f,  0.9906f,0.1338f,-0.0275f,  0.9739f,0.2145f,0.0741f,  0.9712f,0.2266f,0.0737f,  
	0.9368f,0.3040f,0.1734f,  0.9325f,0.3173f,0.1723f,  0.8810f,0.3897f,0.2683f,  0.8752f,0.4039f,0.2663f,  0.8075f,0.4696f,0.3570f,  0.7999f,0.4845f,0.3541f,  0.7174f,0.5417f,0.4380f,  0.7081f,0.5569f,0.4341f,  0.6124f,0.6042f,0.5098f,  0.6014f,0.6192f,0.5049f,  0.4945f,0.6551f,0.5713f,  0.4818f,0.6695f,0.5653f,  0.3659f,0.6928f,0.6214f,  0.3518f,0.7062f,0.6144f,  0.2293f,0.7159f,0.6595f,  
	0.2141f,0.7276f,0.6517f,  0.0877f,0.7230f,0.6853f,  0.0716f,0.7327f,0.6768f,  -0.0560f,0.7134f,0.6986f,  -0.0725f,0.7205f,0.6896f,  -0.1984f,0.6864f,0.6997f,  -0.2148f,0.6906f,0.6906f,  -0.3362f,0.6420f,0.6891f,  -0.3520f,0.6429f,0.6802f,  -0.4662f,0.5804f,0.6677f,  -0.4808f,0.5778f,0.6595f,  -0.5852f,0.5024f,0.6365f,  -0.5979f,0.4962f,0.6295f,  -0.6901f,0.4093f,0.5968f,  -0.7004f,0.3995f,0.5915f,  
	-0.7782f,0.3029f,0.5501f,  -0.7855f,0.2895f,0.5470f,  -0.8472f,0.1852f,0.4979f,  -0.8509f,0.1686f,0.4975f,  -0.8952f,0.0589f,0.4418f,  -0.8949f,0.0395f,0.4445f,  -0.9207f,-0.0731f,0.3833f,  -0.9162f,-0.0948f,0.3895f,  -0.9231f,-0.2076f,0.3238f,  -0.9140f,-0.2308f,0.3338f,  -0.9021f,-0.3410f,0.2645f,  -0.8883f,-0.3651f,0.2784f,  -0.8582f,-0.4700f,0.2065f,  -0.8399f,-0.4942f,0.2243f,  -0.7925f,-0.5910f,0.1504f,  
	-0.7700f,-0.6145f,0.1719f,  -0.7069f,-0.7007f,0.0967f,  -0.6805f,-0.7226f,0.1215f,  -0.6036f,-0.7960f,0.0454f,  -0.5740f,-0.8156f,0.0730f,  -0.4858f,-0.8741f,-0.0037f,  -0.4535f,-0.8909f,0.0259f,  -0.3567f,-0.9328f,-0.0512f,  -0.3228f,-0.9463f,-0.0204f,  -0.2202f,-0.9705f,-0.0977f,  -0.1856f,-0.9804f,-0.0667f,  -0.0805f,-0.9863f,-0.1442f,  -0.0461f,-0.9924f,-0.1142f,  0.0584f,-0.9797f,-0.1917f,  0.0914f,-0.9823f,-0.1636f,  
	0.1921f,-0.9513f,-0.2409f,  0.2226f,-0.9507f,-0.2158f,  0.3164f,-0.9023f,-0.2928f,  0.3434f,-0.8991f,-0.2716f,  0.4276f,-0.8344f,-0.3478f,  0.4500f,-0.8294f,-0.3311f,  0.5221f,-0.7501f,-0.4059f,  0.5392f,-0.7442f,-0.3943f,  0.5971f,-0.6522f,-0.4670f,  0.6082f,-0.6465f,-0.4606f,  0.0067f,-0.5721f,-0.8201f,  0.0088f,-0.5754f,-0.8178f,  0.1390f,-0.5642f,-0.8138f,  0.1414f,-0.5670f,-0.8115f,  0.2692f,-0.5454f,-0.7938f,  
	0.2718f,-0.5476f,-0.7914f,  0.3948f,-0.5157f,-0.7604f,  0.3977f,-0.5171f,-0.7579f,  0.5137f,-0.4755f,-0.7142f,  0.5167f,-0.4758f,-0.7118f,  0.6236f,-0.4251f,-0.6561f,  0.6266f,-0.4243f,-0.6537f,  0.7225f,-0.3652f,-0.5871f,  0.7254f,-0.3631f,-0.5848f,  0.8083f,-0.2966f,-0.5086f,  0.8110f,-0.2931f,-0.5063f,  0.8795f,-0.2204f,-0.4219f,  0.8817f,-0.2154f,-0.4198f,  0.9343f,-0.1378f,-0.3286f,  0.9359f,-0.1311f,-0.3269f,  
	0.9718f,-0.0501f,-0.2306f,  0.9725f,-0.0419f,-0.2292f,  0.9907f,0.0410f,-0.1296f,  0.9904f,0.0509f,-0.1287f,  0.9906f,0.1338f,-0.0275f,  0.9890f,0.1453f,-0.0272f,  0.9712f,0.2266f,0.0737f,  0.9681f,0.2394f,0.0734f,  0.9325f,0.3173f,0.1723f,  0.9279f,0.3314f,0.1711f,  0.8752f,0.4039f,0.2663f,  0.8687f,0.4190f,0.2642f,  0.7999f,0.4845f,0.3541f,  0.7916f,0.5001f,0.3510f,  0.7081f,0.5569f,0.4341f,  
	0.6980f,0.5728f,0.4298f,  0.6014f,0.6192f,0.5049f,  0.5895f,0.6348f,0.4995f,  0.4818f,0.6695f,0.5653f,  0.4683f,0.6844f,0.5588f,  0.3518f,0.7062f,0.6144f,  0.3368f,0.7198f,0.6070f,  0.2141f,0.7276f,0.6517f,  0.1979f,0.7395f,0.6434f,  0.0716f,0.7327f,0.6768f,  0.0547f,0.7424f,0.6677f,  -0.0725f,0.7205f,0.6896f,  -0.0896f,0.7275f,0.6802f,  -0.2148f,0.6906f,0.6906f,  -0.2317f,0.6945f,0.6812f,  
	-0.3520f,0.6429f,0.6802f,  -0.3682f,0.6434f,0.6712f,  -0.4808f,0.5778f,0.6595f,  -0.4955f,0.5746f,0.6513f,  -0.5979f,0.4962f,0.6295f,  -0.6106f,0.4893f,0.6227f,  -0.7004f,0.3995f,0.5915f,  -0.7103f,0.3890f,0.5866f,  -0.7855f,0.2895f,0.5470f,  -0.7921f,0.2755f,0.5446f,  -0.8509f,0.1686f,0.4975f,  -0.8538f,0.1515f,0.4981f,  -0.8949f,0.0395f,0.4445f,  -0.8936f,0.0197f,0.4485f,  -0.9162f,-0.0948f,0.3895f,  
	-0.9103f,-0.1166f,0.3973f,  -0.9140f,-0.2308f,0.3338f,  -0.9034f,-0.2540f,0.3456f,  -0.8883f,-0.3651f,0.2784f,  -0.8730f,-0.3889f,0.2943f,  -0.8399f,-0.4942f,0.2243f,  -0.8200f,-0.5177f,0.2442f,  -0.7700f,-0.6145f,0.1719f,  -0.7457f,-0.6369f,0.1956f,  -0.6805f,-0.7226f,0.1215f,  -0.6524f,-0.7432f,0.1485f,  -0.5740f,-0.8156f,0.0730f,  -0.5426f,-0.8337f,0.1025f,  -0.4535f,-0.8909f,0.0259f,  -0.4198f,-0.9058f,0.0573f,  
	-0.3228f,-0.9463f,-0.0204f,  -0.2875f,-0.9577f,0.0118f,  -0.1856f,-0.9804f,-0.0667f,  -0.1497f,-0.9881f,-0.0348f,  -0.0461f,-0.9924f,-0.1142f,  -0.0109f,-0.9965f,-0.0834f,  0.0914f,-0.9823f,-0.1636f,  0.1249f,-0.9829f,-0.1352f,  0.2226f,-0.9507f,-0.2158f,  0.2533f,-0.9484f,-0.1907f,  0.3434f,-0.8991f,-0.2716f,  0.3702f,-0.8945f,-0.2506f,  0.4500f,-0.8294f,-0.3311f,  0.4721f,-0.8234f,-0.3148f,  0.5392f,-0.7442f,-0.3943f,  
	0.5558f,-0.7378f,-0.3831f,  0.6082f,-0.6465f,-0.4606f,  0.6187f,-0.6407f,-0.4547f,  0.0088f,-0.5754f,-0.8178f,  0.0112f,-0.5789f,-0.8153f,  0.1414f,-0.5670f,-0.8115f,  0.1441f,-0.5700f,-0.8089f,  0.2718f,-0.5476f,-0.7914f,  0.2749f,-0.5498f,-0.7888f,  0.3977f,-0.5171f,-0.7579f,  0.4009f,-0.5185f,-0.7553f,  0.5167f,-0.4758f,-0.7118f,  0.5201f,-0.4762f,-0.7090f,  0.6266f,-0.4243f,-0.6537f,  0.6300f,-0.4234f,-0.6510f,  
	0.7254f,-0.3631f,-0.5848f,  0.7286f,-0.3608f,-0.5821f,  0.8110f,-0.2931f,-0.5063f,  0.8139f,-0.2893f,-0.5038f,  0.8817f,-0.2154f,-0.4198f,  0.8841f,-0.2099f,-0.4175f,  0.9359f,-0.1311f,-0.3269f,  0.9376f,-0.1240f,-0.3249f,  0.9725f,-0.0419f,-0.2292f,  0.9732f,-0.0329f,-0.2277f,  0.9904f,0.0509f,-0.1287f,  0.9899f,0.0615f,-0.1277f,  0.9890f,0.1453f,-0.0272f,  0.9872f,0.1575f,-0.0269f,  0.9681f,0.2394f,0.0734f,  
	0.9647f,0.2531f,0.0729f,  0.9279f,0.3314f,0.1711f,  0.9227f,0.3462f,0.1697f,  0.8687f,0.4190f,0.2642f,  0.8616f,0.4348f,0.2618f,  0.7916f,0.5001f,0.3510f,  0.7826f,0.5165f,0.3475f,  0.6980f,0.5728f,0.4298f,  0.6871f,0.5892f,0.4252f,  0.5895f,0.6348f,0.4995f,  0.5767f,0.6509f,0.4937f,  0.4683f,0.6844f,0.5588f,  0.4538f,0.6997f,0.5518f,  0.3368f,0.7198f,0.6070f,  0.3209f,0.7337f,0.5989f,  
	0.1979f,0.7395f,0.6434f,  0.1809f,0.7515f,0.6345f,  0.0547f,0.7424f,0.6677f,  0.0371f,0.7519f,0.6582f,  -0.0896f,0.7275f,0.6802f,  -0.1075f,0.7342f,0.6704f,  -0.2317f,0.6945f,0.6812f,  -0.2492f,0.6979f,0.6714f,  -0.3682f,0.6434f,0.6712f,  -0.3846f,0.6432f,0.6621f,  -0.4955f,0.5746f,0.6513f,  -0.5103f,0.5708f,0.6433f,  -0.6106f,0.4893f,0.6227f,  -0.6230f,0.4817f,0.6163f,  -0.7103f,0.3890f,0.5866f,  
	-0.7198f,0.3777f,0.5824f,  -0.7921f,0.2755f,0.5446f,  -0.7981f,0.2609f,0.5431f,  -0.8538f,0.1515f,0.4981f,  -0.8557f,0.1338f,0.4999f,  -0.8936f,0.0197f,0.4485f,  -0.8910f,-0.0004f,0.4540f,  -0.9103f,-0.1166f,0.3973f,  -0.9029f,-0.1385f,0.4069f,  -0.9034f,-0.2540f,0.3456f,  -0.8912f,-0.2770f,0.3593f,  -0.8730f,-0.3889f,0.2943f,  -0.8559f,-0.4121f,0.3123f,  -0.8200f,-0.5177f,0.2442f,  -0.7982f,-0.5404f,0.2663f,  
	-0.7457f,-0.6369f,0.1956f,  -0.7196f,-0.6582f,0.2214f,  -0.6524f,-0.7432f,0.1485f,  -0.6224f,-0.7623f,0.1774f,  -0.5426f,-0.8337f,0.1025f,  -0.5096f,-0.8499f,0.1339f,  -0.4198f,-0.9058f,0.0573f,  -0.3845f,-0.9187f,0.0902f,  -0.2875f,-0.9577f,0.0118f,  -0.2509f,-0.9670f,0.0453f,  -0.1497f,-0.9881f,-0.0348f,  -0.1129f,-0.9936f,-0.0018f,  -0.0109f,-0.9965f,-0.0834f,  0.0251f,-0.9983f,-0.0522f,  0.1249f,-0.9829f,-0.1352f,  
	0.1589f,-0.9815f,-0.1066f,  0.2533f,-0.9484f,-0.1907f,  0.2841f,-0.9443f,-0.1658f,  0.3702f,-0.8945f,-0.2506f,  0.3969f,-0.8886f,-0.2300f,  0.4721f,-0.8234f,-0.3148f,  0.4938f,-0.8165f,-0.2991f,  0.5558f,-0.7378f,-0.3831f,  0.5718f,-0.7310f,-0.3725f,  0.6187f,-0.6407f,-0.4547f,  0.6286f,-0.6349f,-0.4492f,  0.0112f,-0.5789f,-0.8153f,  0.0140f,-0.5827f,-0.8126f,  0.1441f,-0.5700f,-0.8089f,  0.1473f,-0.5732f,-0.8061f,  
	0.2749f,-0.5498f,-0.7888f,  0.2783f,-0.5522f,-0.7858f,  0.4009f,-0.5185f,-0.7553f,  0.4047f,-0.5199f,-0.7523f,  0.5201f,-0.4762f,-0.7090f,  0.5239f,-0.4765f,-0.7060f,  0.6300f,-0.4234f,-0.6510f,  0.6338f,-0.4223f,-0.6480f,  0.7286f,-0.3608f,-0.5821f,  0.7322f,-0.3583f,-0.5792f,  0.8139f,-0.2893f,-0.5038f,  0.8171f,-0.2851f,-0.5011f,  0.8841f,-0.2099f,-0.4175f,  0.8867f,-0.2039f,-0.4150f,  0.9376f,-0.1240f,-0.3249f,  
	0.9393f,-0.1162f,-0.3227f,  0.9732f,-0.0329f,-0.2277f,  0.9739f,-0.0233f,-0.2260f,  0.9899f,0.0615f,-0.1277f,  0.9893f,0.0729f,-0.1266f,  0.9872f,0.1575f,-0.0269f,  0.9850f,0.1705f,-0.0265f,  0.9647f,0.2531f,0.0729f,  0.9608f,0.2676f,0.0724f,  0.9227f,0.3462f,0.1697f,  0.9169f,0.3620f,0.1682f,  0.8616f,0.4348f,0.2618f,  0.8539f,0.4513f,0.2591f,  0.7826f,0.5165f,0.3475f,  0.7729f,0.5335f,0.3436f,  
	0.6871f,0.5892f,0.4252f,  0.6753f,0.6063f,0.4200f,  0.5767f,0.6509f,0.4937f,  0.5630f,0.6675f,0.4873f,  0.4538f,0.6997f,0.5518f,  0.4384f,0.7153f,0.5443f,  0.3209f,0.7337f,0.5989f,  0.3041f,0.7477f,0.5903f,  0.1809f,0.7515f,0.6345f,  0.1631f,0.7634f,0.6250f,  0.0371f,0.7519f,0.6582f,  0.0186f,0.7613f,0.6482f,  -0.1075f,0.7342f,0.6704f,  -0.1260f,0.7405f,0.6602f,  -0.2492f,0.6979f,0.6714f,  
	-0.2671f,0.7008f,0.6614f,  -0.3846f,0.6432f,0.6621f,  -0.4013f,0.6425f,0.6528f,  -0.5103f,0.5708f,0.6433f,  -0.5250f,0.5663f,0.6354f,  -0.6230f,0.4817f,0.6163f,  -0.6351f,0.4735f,0.6103f,  -0.7198f,0.3777f,0.5824f,  -0.7287f,0.3658f,0.5789f,  -0.7981f,0.2609f,0.5431f,  -0.8032f,0.2457f,0.5427f,  -0.8557f,0.1338f,0.4999f,  -0.8565f,0.1159f,0.5030f,  -0.8910f,-0.0004f,0.4540f,  -0.8871f,-0.0206f,0.4612f,  
	-0.9029f,-0.1385f,0.4069f,  -0.8941f,-0.1603f,0.4182f,  -0.8912f,-0.2770f,0.3593f,  -0.8773f,-0.2995f,0.3751f,  -0.8559f,-0.4121f,0.3123f,  -0.8370f,-0.4345f,0.3325f,  -0.7982f,-0.5404f,0.2663f,  -0.7746f,-0.5618f,0.2905f,  -0.7196f,-0.6582f,0.2214f,  -0.6916f,-0.6779f,0.2493f,  -0.6224f,-0.7623f,0.1774f,  -0.5908f,-0.7795f,0.2083f,  -0.5096f,-0.8499f,0.1339f,  -0.4749f,-0.8640f,0.1671f,  -0.3845f,-0.9187f,0.0902f,  
	-0.3477f,-0.9293f,0.1245f,  -0.2509f,-0.9670f,0.0453f,  -0.2130f,-0.9738f,0.0798f,  -0.1129f,-0.9936f,-0.0018f,  -0.0751f,-0.9967f,0.0317f,  0.0251f,-0.9983f,-0.0522f,  0.0618f,-0.9979f,-0.0207f,  0.1589f,-0.9815f,-0.1066f,  0.1932f,-0.9780f,-0.0782f,  0.2841f,-0.9443f,-0.1658f,  0.3150f,-0.9385f,-0.1413f,  0.3969f,-0.8886f,-0.2300f,  0.4234f,-0.8813f,-0.2100f,  0.4938f,-0.8165f,-0.2991f,  0.5151f,-0.8087f,-0.2840f,  
	0.5718f,-0.7310f,-0.3725f,  0.5872f,-0.7237f,-0.3626f,  0.6286f,-0.6349f,-0.4492f,  0.6378f,-0.6292f,-0.4442f,  0.0140f,-0.5827f,-0.8126f,  0.0173f,-0.5867f,-0.8096f,  0.1473f,-0.5732f,-0.8061f,  0.1510f,-0.5766f,-0.8030f,  0.2783f,-0.5522f,-0.7858f,  0.2823f,-0.5547f,-0.7827f,  0.4047f,-0.5199f,-0.7523f,  0.4088f,-0.5214f,-0.7490f,  0.5239f,-0.4765f,-0.7060f,  0.5282f,-0.4766f,-0.7027f,  0.6338f,-0.4223f,-0.6480f,  
	0.6380f,-0.4211f,-0.6447f,  0.7322f,-0.3583f,-0.5792f,  0.7362f,-0.3554f,-0.5760f,  0.8171f,-0.2851f,-0.5011f,  0.8206f,-0.2804f,-0.4980f,  0.8867f,-0.2039f,-0.4150f,  0.8894f,-0.1974f,-0.4123f,  0.9393f,-0.1162f,-0.3227f,  0.9411f,-0.1078f,-0.3204f,  0.9739f,-0.0233f,-0.2260f,  0.9745f,-0.0130f,-0.2242f,  0.9893f,0.0729f,-0.1266f,  0.9884f,0.0850f,-0.1255f,  0.9850f,0.1705f,-0.0265f,  0.9825f,0.1844f,-0.0262f,  
	0.9608f,0.2676f,0.0724f,  0.9565f,0.2829f,0.0717f,  0.9169f,0.3620f,0.1682f,  0.9105f,0.3785f,0.1664f,  0.8539f,0.4513f,0.2591f,  0.8454f,0.4687f,0.2561f,  0.7729f,0.5335f,0.3436f,  0.7622f,0.5513f,0.3392f,  0.6753f,0.6063f,0.4200f,  0.6626f,0.6239f,0.4144f,  0.5630f,0.6675f,0.4873f,  0.5483f,0.6846f,0.4803f,  0.4384f,0.7153f,0.5443f,  0.4220f,0.7312f,0.5360f,  0.3041f,0.7477f,0.5903f,  
	0.2864f,0.7619f,0.5810f,  0.1631f,0.7634f,0.6250f,  0.1444f,0.7753f,0.6148f,  0.0186f,0.7613f,0.6482f,  -0.0006f,0.7704f,0.6376f,  -0.1260f,0.7405f,0.6602f,  -0.1450f,0.7464f,0.6495f,  -0.2671f,0.7008f,0.6614f,  -0.2854f,0.7032f,0.6512f,  -0.4013f,0.6425f,0.6528f,  -0.4180f,0.6412f,0.6436f,  -0.5250f,0.5663f,0.6354f,  -0.5396f,0.5611f,0.6277f,  -0.6351f,0.4735f,0.6103f,  -0.6469f,0.4645f,0.6048f,  
	-0.7287f,0.3658f,0.5789f,  -0.7370f,0.3534f,0.5762f,  -0.8032f,0.2457f,0.5427f,  -0.8073f,0.2301f,0.5434f,  -0.8565f,0.1159f,0.5030f,  -0.8561f,0.0976f,0.5076f,  -0.8871f,-0.0206f,0.4612f,  -0.8818f,-0.0408f,0.4699f,  -0.8941f,-0.1603f,0.4182f,  -0.8836f,-0.1817f,0.4315f,  -0.8773f,-0.2995f,0.3751f,  -0.8616f,-0.3213f,0.3929f,  -0.8370f,-0.4345f,0.3325f,  -0.8163f,-0.4559f,0.3547f,  -0.7746f,-0.5618f,0.2905f,  
	-0.7490f,-0.5819f,0.3169f,  -0.6916f,-0.6779f,0.2493f,  -0.6618f,-0.6958f,0.2792f,  -0.5908f,-0.7795f,0.2083f,  -0.5573f,-0.7946f,0.2410f,  -0.4749f,-0.8640f,0.1671f,  -0.4386f,-0.8758f,0.2016f,  -0.3477f,-0.9293f,0.1245f,  -0.3095f,-0.9373f,0.1600f,  -0.2130f,-0.9738f,0.0798f,  -0.1740f,-0.9780f,0.1150f,  -0.0751f,-0.9967f,0.0317f,  -0.0364f,-0.9972f,0.0655f,  0.0618f,-0.9979f,-0.0207f,  0.0989f,-0.9950f,0.0107f,  
	0.1932f,-0.9780f,-0.0782f,  0.2277f,-0.9724f,-0.0501f,  0.3150f,-0.9385f,-0.1413f,  0.3458f,-0.9310f,-0.1173f,  0.4234f,-0.8813f,-0.2100f,  0.4495f,-0.8727f,-0.1907f,  0.5151f,-0.8087f,-0.2840f,  0.5358f,-0.8001f,-0.2697f,  0.5872f,-0.7237f,-0.3626f,  0.6020f,-0.7161f,-0.3532f,  0.6378f,-0.6292f,-0.4442f,  0.6464f,-0.6236f,-0.4397f,  0.0173f,-0.5867f,-0.8096f,  0.0210f,-0.5910f,-0.8064f,  0.1510f,-0.5766f,-0.8030f,  
	0.1551f,-0.5801f,-0.7996f,  0.2823f,-0.5547f,-0.7827f,  0.2868f,-0.5573f,-0.7792f,  0.4088f,-0.5214f,-0.7490f,  0.4135f,-0.5228f,-0.7455f,  0.5282f,-0.4766f,-0.7027f,  0.5329f,-0.4767f,-0.6991f,  0.6380f,-0.4211f,-0.6447f,  0.6426f,-0.4196f,-0.6411f,  0.7362f,-0.3554f,-0.5760f,  0.7404f,-0.3521f,-0.5725f,  0.8206f,-0.2804f,-0.4980f,  0.8243f,-0.2753f,-0.4948f,  0.8894f,-0.1974f,-0.4123f,  0.8923f,-0.1903f,-0.4093f,  
	0.9411f,-0.1078f,-0.3204f,  0.9430f,-0.0987f,-0.3179f,  0.9745f,-0.0130f,-0.2242f,  0.9750f,-0.0019f,-0.2223f,  0.9884f,0.0850f,-0.1255f,  0.9874f,0.0980f,-0.1244f,  0.9825f,0.1844f,-0.0262f,  0.9796f,0.1991f,-0.0260f,  0.9565f,0.2829f,0.0717f,  0.9516f,0.2991f,0.0708f,  0.9105f,0.3785f,0.1664f,  0.9035f,0.3958f,0.1642f,  0.8454f,0.4687f,0.2561f,  0.8362f,0.4868f,0.2526f,  0.7622f,0.5513f,0.3392f,  
	0.7508f,0.5697f,0.3343f,  0.6626f,0.6239f,0.4144f,  0.6489f,0.6422f,0.4081f,  0.5483f,0.6846f,0.4803f,  0.5327f,0.7021f,0.4726f,  0.4220f,0.7312f,0.5360f,  0.4047f,0.7473f,0.5271f,  0.2864f,0.7619f,0.5810f,  0.2677f,0.7761f,0.5710f,  0.1444f,0.7753f,0.6148f,  0.1248f,0.7871f,0.6041f,  -0.0006f,0.7704f,0.6376f,  -0.0204f,0.7792f,0.6264f,  -0.1450f,0.7464f,0.6495f,  -0.1646f,0.7519f,0.6384f,  
	-0.2854f,0.7032f,0.6512f,  -0.3039f,0.7051f,0.6407f,  -0.4180f,0.6412f,0.6436f,  -0.4348f,0.6392f,0.6343f,  -0.5396f,0.5611f,0.6277f,  -0.5540f,0.5553f,0.6203f,  -0.6469f,0.4645f,0.6048f,  -0.6581f,0.4550f,0.5999f,  -0.7370f,0.3534f,0.5762f,  -0.7444f,0.3405f,0.5745f,  -0.8073f,0.2301f,0.5434f,  -0.8104f,0.2142f,0.5453f,  -0.8561f,0.0976f,0.5076f,  -0.8544f,0.0793f,0.5136f,  -0.8818f,-0.0408f,0.4699f,  
	-0.8749f,-0.0608f,0.4805f,  -0.8836f,-0.1817f,0.4315f,  -0.8714f,-0.2026f,0.4467f,  -0.8616f,-0.3213f,0.3929f,  -0.8441f,-0.3422f,0.4128f,  -0.8163f,-0.4559f,0.3547f,  -0.7936f,-0.4759f,0.3791f,  -0.7490f,-0.5819f,0.3169f,  -0.7216f,-0.6001f,0.3452f,  -0.6618f,-0.6958f,0.2792f,  -0.6301f,-0.7116f,0.3109f,  -0.5573f,-0.7946f,0.2410f,  -0.5220f,-0.8073f,0.2753f,  -0.4386f,-0.8758f,0.2016f,  -0.4007f,-0.8849f,0.2375f,  
	-0.3095f,-0.9373f,0.1600f,  -0.2699f,-0.9427f,0.1963f,  -0.1740f,-0.9780f,0.1150f,  -0.1339f,-0.9795f,0.1506f,  -0.0364f,-0.9972f,0.0655f,  0.0030f,-0.9950f,0.0994f,  0.0989f,-0.9950f,0.0107f,  0.1365f,-0.9898f,0.0418f,  0.2277f,-0.9724f,-0.0501f,  0.2622f,-0.9647f,-0.0227f,  0.3458f,-0.9310f,-0.1173f,  0.3763f,-0.9217f,-0.0942f,  0.4495f,-0.8727f,-0.1907f,  0.4753f,-0.8628f,-0.1723f,  0.5358f,-0.8001f,-0.2697f,  
	0.5560f,-0.7907f,-0.2562f,  0.6020f,-0.7161f,-0.3532f,  0.6162f,-0.7082f,-0.3446f,  0.6464f,-0.6236f,-0.4397f,  0.6544f,-0.6181f,-0.4356f,  0.0210f,-0.5910f,-0.8064f,  0.0253f,-0.5955f,-0.8029f,  0.1551f,-0.5801f,-0.7996f,  0.1598f,-0.5838f,-0.7960f,  0.2868f,-0.5573f,-0.7792f,  0.2917f,-0.5600f,-0.7754f,  0.4135f,-0.5228f,-0.7455f,  0.4186f,-0.5242f,-0.7416f,  0.5329f,-0.4767f,-0.6991f,  0.5381f,-0.4766f,-0.6952f,  
	0.6426f,-0.4196f,-0.6411f,  0.6476f,-0.4178f,-0.6372f,  0.7404f,-0.3521f,-0.5725f,  0.7450f,-0.3485f,-0.5688f,  0.8243f,-0.2753f,-0.4948f,  0.8282f,-0.2697f,-0.4912f,  0.8923f,-0.1903f,-0.4093f,  0.8954f,-0.1826f,-0.4062f,  0.9430f,-0.0987f,-0.3179f,  0.9448f,-0.0889f,-0.3153f,  0.9750f,-0.0019f,-0.2223f,  0.9754f,0.0099f,-0.2204f,  0.9874f,0.0980f,-0.1244f,  0.9860f,0.1118f,-0.1233f,  0.9796f,0.1991f,-0.0260f,  
	0.9763f,0.2147f,-0.0260f,  0.9516f,0.2991f,0.0708f,  0.9462f,0.3162f,0.0695f,  0.9035f,0.3958f,0.1642f,  0.8958f,0.4140f,0.1617f,  0.8362f,0.4868f,0.2526f,  0.8261f,0.5057f,0.2486f,  0.7508f,0.5697f,0.3343f,  0.7383f,0.5888f,0.3289f,  0.6489f,0.6422f,0.4081f,  0.6343f,0.6609f,0.4010f,  0.5327f,0.7021f,0.4726f,  0.5161f,0.7199f,0.4641f,  0.4047f,0.7473f,0.5271f,  0.3864f,0.7636f,0.5173f,  
	0.2677f,0.7761f,0.5710f,  0.2481f,0.7903f,0.5602f,  0.1248f,0.7871f,0.6041f,  0.1045f,0.7987f,0.5926f,  -0.0204f,0.7792f,0.6264f,  -0.0409f,0.7877f,0.6147f,  -0.1646f,0.7519f,0.6384f,  -0.1845f,0.7569f,0.6269f,  -0.3039f,0.7051f,0.6407f,  -0.3226f,0.7063f,0.6301f,  -0.4348f,0.6392f,0.6343f,  -0.4516f,0.6366f,0.6252f,  -0.5540f,0.5553f,0.6203f,  -0.5680f,0.5489f,0.6133f,  -0.6581f,0.4550f,0.5999f,  
	-0.6687f,0.4450f,0.5957f,  -0.7444f,0.3405f,0.5745f,  -0.7509f,0.3272f,0.5737f,  -0.8104f,0.2142f,0.5453f,  -0.8123f,0.1981f,0.5485f,  -0.8544f,0.0793f,0.5136f,  -0.8512f,0.0611f,0.5213f,  -0.8749f,-0.0608f,0.4805f,  -0.8664f,-0.0804f,0.4929f,  -0.8714f,-0.2026f,0.4467f,  -0.8575f,-0.2226f,0.4639f,  -0.8441f,-0.3422f,0.4128f,  -0.8247f,-0.3618f,0.4348f,  -0.7936f,-0.4759f,0.3791f,  -0.7690f,-0.4943f,0.4054f,  
	-0.7216f,-0.6001f,0.3452f,  -0.6921f,-0.6164f,0.3755f,  -0.6301f,-0.7116f,0.3109f,  -0.5965f,-0.7250f,0.3443f,  -0.5220f,-0.8073f,0.2753f,  -0.4850f,-0.8174f,0.3109f,  -0.4007f,-0.8849f,0.2375f,  -0.3613f,-0.8912f,0.2743f,  -0.2699f,-0.9427f,0.1963f,  -0.2292f,-0.9450f,0.2332f,  -0.1339f,-0.9795f,0.1506f,  -0.0930f,-0.9781f,0.1864f,  0.0030f,-0.9950f,0.0994f,  0.0429f,-0.9902f,0.1331f,  0.1365f,-0.9898f,0.0418f,  
	0.1742f,-0.9820f,0.0724f,  0.2622f,-0.9647f,-0.0227f,  0.2967f,-0.9550f,0.0040f,  0.3763f,-0.9217f,-0.0942f,  0.4066f,-0.9108f,-0.0720f,  0.4753f,-0.8628f,-0.1723f,  0.5004f,-0.8518f,-0.1549f,  0.5560f,-0.7907f,-0.2562f,  0.5755f,-0.7806f,-0.2437f,  0.6162f,-0.7082f,-0.3446f,  0.6298f,-0.7000f,-0.3367f,  0.6544f,-0.6181f,-0.4356f,  0.6618f,-0.6127f,-0.4320f,  0.0253f,-0.5955f,-0.8029f,  0.0301f,-0.6002f,-0.7993f,  
	0.1598f,-0.5838f,-0.7960f,  0.1650f,-0.5876f,-0.7921f,  0.2917f,-0.5600f,-0.7754f,  0.2973f,-0.5627f,-0.7714f,  0.4186f,-0.5242f,-0.7416f,  0.4243f,-0.5255f,-0.7374f,  0.5381f,-0.4766f,-0.6952f,  0.5437f,-0.4763f,-0.6910f,  0.6476f,-0.4178f,-0.6372f,  0.6530f,-0.4157f,-0.6331f,  0.7450f,-0.3485f,-0.5688f,  0.7500f,-0.3444f,-0.5648f,  0.8282f,-0.2697f,-0.4912f,  0.8324f,-0.2635f,-0.4875f,  0.8954f,-0.1826f,-0.4062f,  
	0.8985f,-0.1743f,-0.4029f,  0.9448f,-0.0889f,-0.3153f,  0.9467f,-0.0783f,-0.3126f,  0.9754f,0.0099f,-0.2204f,  0.9756f,0.0227f,-0.2185f,  0.9860f,0.1118f,-0.1233f,  0.9844f,0.1265f,-0.1224f,  0.9763f,0.2147f,-0.0260f,  0.9726f,0.2311f,-0.0263f,  0.9462f,0.3162f,0.0695f,  0.9401f,0.3341f,0.0680f,  0.8958f,0.4140f,0.1617f,  0.8873f,0.4330f,0.1586f,  0.8261f,0.5057f,0.2486f,  0.8152f,0.5253f,0.2440f,  
	0.7383f,0.5888f,0.3289f,  0.7250f,0.6085f,0.3226f,  0.6343f,0.6609f,0.4010f,  0.6187f,0.6801f,0.3932f,  0.5161f,0.7199f,0.4641f,  0.4985f,0.7380f,0.4548f,  0.3864f,0.7636f,0.5173f,  0.3671f,0.7800f,0.5068f,  0.2481f,0.7903f,0.5602f,  0.2276f,0.8045f,0.5487f,  0.1045f,0.7987f,0.5926f,  0.0834f,0.8100f,0.5804f,  -0.0409f,0.7877f,0.6147f,  -0.0620f,0.7958f,0.6024f,  -0.1845f,0.7569f,0.6269f,  
	-0.2048f,0.7614f,0.6151f,  -0.3226f,0.7063f,0.6301f,  -0.3414f,0.7070f,0.6193f,  -0.4516f,0.6366f,0.6252f,  -0.4681f,0.6334f,0.6162f,  -0.5680f,0.5489f,0.6133f,  -0.5815f,0.5419f,0.6067f,  -0.6687f,0.4450f,0.5957f,  -0.6785f,0.4345f,0.5923f,  -0.7509f,0.3272f,0.5737f,  -0.7564f,0.3136f,0.5740f,  -0.8123f,0.1981f,0.5485f,  -0.8129f,0.1821f,0.5531f,  -0.8512f,0.0611f,0.5213f,  -0.8465f,0.0433f,0.5306f,  
	-0.8664f,-0.0804f,0.4929f,  -0.8562f,-0.0992f,0.5071f,  -0.8575f,-0.2226f,0.4639f,  -0.8416f,-0.2416f,0.4830f,  -0.8247f,-0.3618f,0.4348f,  -0.8033f,-0.3800f,0.4587f,  -0.7690f,-0.4943f,0.4054f,  -0.7424f,-0.5107f,0.4336f,  -0.6921f,-0.6164f,0.3755f,  -0.6608f,-0.6304f,0.4074f,  -0.5965f,-0.7250f,0.3443f,  -0.5611f,-0.7358f,0.3791f,  -0.4850f,-0.8174f,0.3109f,  -0.4464f,-0.8245f,0.3476f,  -0.3613f,-0.8912f,0.2743f,  
	-0.3205f,-0.8945f,0.3118f,  -0.2292f,-0.9450f,0.2332f,  -0.1874f,-0.9444f,0.2703f,  -0.0930f,-0.9781f,0.1864f,  -0.0513f,-0.9737f,0.2221f,  0.0429f,-0.9902f,0.1331f,  0.0833f,-0.9826f,0.1662f,  0.1742f,-0.9820f,0.0724f,  0.2120f,-0.9719f,0.1021f,  0.2967f,-0.9550f,0.0040f,  0.3310f,-0.9432f,0.0296f,  0.4066f,-0.9108f,-0.0720f,  0.4363f,-0.8983f,-0.0509f,  0.5004f,-0.8518f,-0.1549f,  0.5250f,-0.8397f,-0.1386f,  
	0.5755f,-0.7806f,-0.2437f,  0.5944f,-0.7699f,-0.2321f,  0.6298f,-0.7000f,-0.3367f,  0.6427f,-0.6916f,-0.3295f,  0.6618f,-0.6127f,-0.4320f,  0.6687f,-0.6075f,-0.4288f,  0.0301f,-0.6002f,-0.7993f,  0.0354f,-0.6052f,-0.7953f,  0.1650f,-0.5876f,-0.7921f,  0.1708f,-0.5915f,-0.7880f,  0.2973f,-0.5627f,-0.7714f,  0.3034f,-0.5653f,-0.7670f,  0.4243f,-0.5255f,-0.7374f,  0.4306f,-0.5267f,-0.7330f,  0.5437f,-0.4763f,-0.6910f,  
	0.5499f,-0.4758f,-0.6865f,  0.6530f,-0.4157f,-0.6331f,  0.6589f,-0.4132f,-0.6286f,  0.7500f,-0.3444f,-0.5648f,  0.7552f,-0.3398f,-0.5605f,  0.8324f,-0.2635f,-0.4875f,  0.8368f,-0.2567f,-0.4836f,  0.8985f,-0.1743f,-0.4029f,  0.9017f,-0.1652f,-0.3995f,  0.9467f,-0.0783f,-0.3126f,  0.9484f,-0.0669f,-0.3099f,  0.9756f,0.0227f,-0.2185f,  0.9756f,0.0362f,-0.2167f,  0.9844f,0.1265f,-0.1224f,  0.9823f,0.1421f,-0.1217f,  
	0.9726f,0.2311f,-0.0263f,  0.9683f,0.2485f,-0.0269f,  0.9401f,0.3341f,0.0680f,  0.9333f,0.3529f,0.0660f,  0.8873f,0.4330f,0.1586f,  0.8780f,0.4528f,0.1550f,  0.8152f,0.5253f,0.2440f,  0.8033f,0.5456f,0.2387f,  0.7250f,0.6085f,0.3226f,  0.7107f,0.6287f,0.3156f,  0.6187f,0.6801f,0.3932f,  0.6021f,0.6997f,0.3845f,  0.4985f,0.7380f,0.4548f,  0.4799f,0.7563f,0.4446f,  0.3671f,0.7800f,0.5068f,  
	0.3469f,0.7965f,0.4953f,  0.2276f,0.8045f,0.5487f,  0.2063f,0.8185f,0.5363f,  0.0834f,0.8100f,0.5804f,  0.0616f,0.8210f,0.5675f,  -0.0620f,0.7958f,0.6024f,  -0.0835f,0.8034f,0.5896f,  -0.2048f,0.7614f,0.6151f,  -0.2254f,0.7653f,0.6029f,  -0.3414f,0.7070f,0.6193f,  -0.3603f,0.7071f,0.6085f,  -0.4681f,0.6334f,0.6162f,  -0.4844f,0.6296f,0.6074f,  -0.5815f,0.5419f,0.6067f,  -0.5945f,0.5345f,0.6007f,  
	-0.6785f,0.4345f,0.5923f,  -0.6876f,0.4237f,0.5897f,  -0.7564f,0.3136f,0.5740f,  -0.7608f,0.2999f,0.5755f,  -0.8129f,0.1821f,0.5531f,  -0.8122f,0.1661f,0.5592f,  -0.8465f,0.0433f,0.5306f,  -0.8403f,0.0259f,0.5415f,  -0.8562f,-0.0992f,0.5071f,  -0.8442f,-0.1171f,0.5231f,  -0.8416f,-0.2416f,0.4830f,  -0.8239f,-0.2591f,0.5041f,  -0.8033f,-0.3800f,0.4587f,  -0.7799f,-0.3963f,0.4844f,  -0.7424f,-0.5107f,0.4336f,  
	-0.7138f,-0.5250f,0.4635f,  -0.6608f,-0.6304f,0.4074f,  -0.6276f,-0.6418f,0.4408f,  -0.5611f,-0.7358f,0.3791f,  -0.5240f,-0.7438f,0.4150f,  -0.4464f,-0.8245f,0.3476f,  -0.4063f,-0.8286f,0.3850f,  -0.3205f,-0.8945f,0.3118f,  -0.2785f,-0.8946f,0.3495f,  -0.1874f,-0.9444f,0.2703f,  -0.1447f,-0.9406f,0.3073f,  -0.0513f,-0.9737f,0.2221f,  -0.0091f,-0.9663f,0.2571f,  0.0833f,-0.9826f,0.1662f,  0.1237f,-0.9723f,0.1984f,  
	0.2120f,-0.9719f,0.1021f,  0.2497f,-0.9595f,0.1306f,  0.3310f,-0.9432f,0.0296f,  0.3648f,-0.9295f,0.0539f,  0.4363f,-0.8983f,-0.0509f,  0.4655f,-0.8845f,-0.0311f,  0.5250f,-0.8397f,-0.1386f,  0.5489f,-0.8267f,-0.1235f,  0.5944f,-0.7699f,-0.2321f,  0.6126f,-0.7588f,-0.2214f,  0.6427f,-0.6916f,-0.3295f,  0.6549f,-0.6832f,-0.3230f,  0.6687f,-0.6075f,-0.4288f,  0.6750f,-0.6024f,-0.4259f,  0.0354f,-0.6052f,-0.7953f,  
	0.0414f,-0.6103f,-0.7911f,  0.1708f,-0.5915f,-0.7880f,  0.1772f,-0.5956f,-0.7835f,  0.3034f,-0.5653f,-0.7670f,  0.3100f,-0.5680f,-0.7624f,  0.4306f,-0.5267f,-0.7330f,  0.4373f,-0.5277f,-0.7282f,  0.5499f,-0.4758f,-0.6865f,  0.5565f,-0.4749f,-0.6817f,  0.6589f,-0.4132f,-0.6286f,  0.6651f,-0.4103f,-0.6239f,  0.7552f,-0.3398f,-0.5605f,  0.7608f,-0.3347f,-0.5560f,  0.8368f,-0.2567f,-0.4836f,  0.8414f,-0.2492f,-0.4795f,  
	0.9017f,-0.1652f,-0.3995f,  0.9050f,-0.1553f,-0.3960f,  0.9484f,-0.0669f,-0.3099f,  0.9501f,-0.0547f,-0.3072f,  0.9756f,0.0362f,-0.2167f,  0.9753f,0.0507f,-0.2150f,  0.9823f,0.1421f,-0.1217f,  0.9799f,0.1587f,-0.1213f,  0.9683f,0.2485f,-0.0269f,  0.9634f,0.2668f,-0.0279f,  0.9333f,0.3529f,0.0660f,  0.9258f,0.3726f,0.0634f,  0.8780f,0.4528f,0.1550f,  0.8679f,0.4734f,0.1507f,  0.8033f,0.5456f,0.2387f,  
	0.7905f,0.5665f,0.2326f,  0.7107f,0.6287f,0.3156f,  0.6954f,0.6495f,0.3077f,  0.6021f,0.6997f,0.3845f,  0.5844f,0.7197f,0.3749f,  0.4799f,0.7563f,0.4446f,  0.4602f,0.7748f,0.4335f,  0.3469f,0.7965f,0.4953f,  0.3257f,0.8129f,0.4829f,  0.2063f,0.8185f,0.5363f,  0.1842f,0.8322f,0.5230f,  0.0616f,0.8210f,0.5675f,  0.0392f,0.8317f,0.5539f,  -0.0835f,0.8034f,0.5896f,  -0.1056f,0.8105f,0.5761f,  
	-0.2254f,0.7653f,0.6029f,  -0.2462f,0.7686f,0.5904f,  -0.3603f,0.7071f,0.6085f,  -0.3790f,0.7065f,0.5976f,  -0.4844f,0.6296f,0.6074f,  -0.5004f,0.6253f,0.5989f,  -0.5945f,0.5345f,0.6007f,  -0.6070f,0.5265f,0.5953f,  -0.6876f,0.4237f,0.5897f,  -0.6957f,0.4126f,0.5881f,  -0.7608f,0.2999f,0.5755f,  -0.7640f,0.2862f,0.5783f,  -0.8122f,0.1661f,0.5592f,  -0.8100f,0.1506f,0.5667f,  -0.8403f,0.0259f,0.5415f,  
	-0.8324f,0.0093f,0.5542f,  -0.8442f,-0.1171f,0.5231f,  -0.8304f,-0.1339f,0.5409f,  -0.8239f,-0.2591f,0.5041f,  -0.8042f,-0.2751f,0.5269f,  -0.7799f,-0.3963f,0.4844f,  -0.7547f,-0.4106f,0.5118f,  -0.7138f,-0.5250f,0.4635f,  -0.6833f,-0.5367f,0.4949f,  -0.6276f,-0.6418f,0.4408f,  -0.5926f,-0.6504f,0.4753f,  -0.5240f,-0.7438f,0.4150f,  -0.4853f,-0.7487f,0.4517f,  -0.4063f,-0.8286f,0.3850f,  -0.3649f,-0.8295f,0.4228f,  
	-0.2785f,-0.8946f,0.3495f,  -0.2356f,-0.8914f,0.3872f,  -0.1447f,-0.9406f,0.3073f,  -0.1014f,-0.9336f,0.3437f,  -0.0091f,-0.9663f,0.2571f,  0.0333f,-0.9560f,0.2913f,  0.1237f,-0.9723f,0.1984f,  0.1642f,-0.9594f,0.2294f,  0.2497f,-0.9595f,0.1306f,  0.2870f,-0.9448f,0.1578f,  0.3648f,-0.9295f,0.0539f,  0.3981f,-0.9141f,0.0768f,  0.4655f,-0.8845f,-0.0311f,  0.4940f,-0.8694f,-0.0128f,  0.5489f,-0.8267f,-0.1235f,  
	0.5720f,-0.8129f,-0.1095f,  0.6126f,-0.7588f,-0.2214f,  0.6300f,-0.7472f,-0.2118f,  0.6549f,-0.6832f,-0.3230f,  0.6665f,-0.6746f,-0.3172f,  0.6750f,-0.6024f,-0.4259f,  0.6808f,-0.5976f,-0.4235f,  0.0414f,-0.6103f,-0.7911f,  0.0480f,-0.6156f,-0.7866f,  0.1772f,-0.5956f,-0.7835f,  0.1842f,-0.5996f,-0.7788f,  0.3100f,-0.5680f,-0.7624f,  0.3173f,-0.5706f,-0.7574f,  0.4373f,-0.5277f,-0.7282f,  0.4447f,-0.5285f,-0.7231f,  
	0.5565f,-0.4749f,-0.6817f,  0.5637f,-0.4738f,-0.6766f,  0.6651f,-0.4103f,-0.6239f,  0.6718f,-0.4070f,-0.6189f,  0.7608f,-0.3347f,-0.5560f,  0.7667f,-0.3290f,-0.5514f,  0.8414f,-0.2492f,-0.4795f,  0.8462f,-0.2410f,-0.4753f,  0.9050f,-0.1553f,-0.3960f,  0.9083f,-0.1446f,-0.3925f,  0.9501f,-0.0547f,-0.3072f,  0.9516f,-0.0416f,-0.3046f,  0.9753f,0.0507f,-0.2150f,  0.9747f,0.0661f,-0.2136f,  0.9799f,0.1587f,-0.1213f,  
	0.9769f,0.1761f,-0.1212f,  0.9634f,0.2668f,-0.0279f,  0.9578f,0.2860f,-0.0294f,  0.9258f,0.3726f,0.0634f,  0.9175f,0.3931f,0.0601f,  0.8679f,0.4734f,0.1507f,  0.8568f,0.4947f,0.1456f,  0.7905f,0.5665f,0.2326f,  0.7767f,0.5881f,0.2255f,  0.6954f,0.6495f,0.3077f,  0.6789f,0.6707f,0.2987f,  0.5844f,0.7197f,0.3749f,  0.5656f,0.7399f,0.3642f,  0.4602f,0.7748f,0.4335f,  0.4395f,0.7933f,0.4212f,  
	0.3257f,0.8129f,0.4829f,  0.3036f,0.8291f,0.4694f,  0.1842f,0.8322f,0.5230f,  0.1613f,0.8457f,0.5087f,  0.0392f,0.8317f,0.5539f,  0.0161f,0.8418f,0.5395f,  -0.1056f,0.8105f,0.5761f,  -0.1280f,0.8171f,0.5622f,  -0.2462f,0.7686f,0.5904f,  -0.2671f,0.7714f,0.5776f,  -0.3790f,0.7065f,0.5976f,  -0.3977f,0.7054f,0.5867f,  -0.5004f,0.6253f,0.5989f,  -0.5160f,0.6204f,0.5907f,  -0.6070f,0.5265f,0.5953f,  
	-0.6186f,0.5182f,0.5906f,  -0.6957f,0.4126f,0.5881f,  -0.7027f,0.4014f,0.5874f,  -0.7640f,0.2862f,0.5783f,  -0.7659f,0.2727f,0.5823f,  -0.8100f,0.1506f,0.5667f,  -0.8063f,0.1356f,0.5758f,  -0.8324f,0.0093f,0.5542f,  -0.8227f,-0.0063f,0.5684f,  -0.8304f,-0.1339f,0.5409f,  -0.8147f,-0.1492f,0.5604f,  -0.8042f,-0.2751f,0.5269f,  -0.7826f,-0.2892f,0.5513f,  -0.7547f,-0.4106f,0.5118f,  -0.7274f,-0.4225f,0.5406f,  
	-0.6833f,-0.5367f,0.4949f,  -0.6510f,-0.5458f,0.5275f,  -0.5926f,-0.6504f,0.4753f,  -0.5559f,-0.6559f,0.5107f,  -0.4853f,-0.7487f,0.4517f,  -0.4451f,-0.7503f,0.4888f,  -0.3649f,-0.8295f,0.4228f,  -0.3224f,-0.8270f,0.4605f,  -0.2356f,-0.8914f,0.3872f,  -0.1918f,-0.8849f,0.4244f,  -0.1014f,-0.9336f,0.3437f,  -0.0576f,-0.9235f,0.3793f,  0.0333f,-0.9560f,0.2913f,  0.0759f,-0.9429f,0.3243f,  0.1642f,-0.9594f,0.2294f,  
	0.2044f,-0.9440f,0.2590f,  0.2870f,-0.9448f,0.1578f,  0.3238f,-0.9282f,0.1834f,  0.3981f,-0.9141f,0.0768f,  0.4306f,-0.8972f,0.0981f,  0.4940f,-0.8694f,-0.0128f,  0.5216f,-0.8532f,0.0042f,  0.5720f,-0.8129f,-0.1095f,  0.5943f,-0.7984f,-0.0969f,  0.6300f,-0.7472f,-0.2118f,  0.6466f,-0.7353f,-0.2031f,  0.6665f,-0.6746f,-0.3172f,  0.6774f,-0.6661f,-0.3121f,  0.6808f,-0.5976f,-0.4235f,  0.6861f,-0.5930f,-0.4214f,  
	0.0480f,-0.6156f,-0.7866f,  0.0552f,-0.6210f,-0.7818f,  0.1842f,-0.5996f,-0.7788f,  0.1918f,-0.6038f,-0.7737f,  0.3173f,-0.5706f,-0.7574f,  0.3252f,-0.5731f,-0.7522f,  0.4447f,-0.5285f,-0.7231f,  0.4525f,-0.5292f,-0.7178f,  0.5637f,-0.4738f,-0.6766f,  0.5713f,-0.4723f,-0.6713f,  0.6718f,-0.4070f,-0.6189f,  0.6788f,-0.4031f,-0.6138f,  0.7667f,-0.3290f,-0.5514f,  0.7728f,-0.3226f,-0.5465f,  0.8462f,-0.2410f,-0.4753f,  
	0.8510f,-0.2321f,-0.4711f,  0.9083f,-0.1446f,-0.3925f,  0.9115f,-0.1332f,-0.3891f,  0.9516f,-0.0416f,-0.3046f,  0.9528f,-0.0276f,-0.3022f,  0.9747f,0.0661f,-0.2136f,  0.9737f,0.0824f,-0.2124f,  0.9769f,0.1761f,-0.1212f,  0.9733f,0.1945f,-0.1216f,  0.9578f,0.2860f,-0.0294f,  0.9515f,0.3061f,-0.0315f,  0.9175f,0.3931f,0.0601f,  0.9084f,0.4143f,0.0561f,  0.8568f,0.4947f,0.1456f,  0.8447f,0.5167f,0.1395f,  
	0.7767f,0.5881f,0.2255f,  0.7618f,0.6102f,0.2175f,  0.6789f,0.6707f,0.2987f,  0.6614f,0.6923f,0.2887f,  0.5656f,0.7399f,0.3642f,  0.5458f,0.7602f,0.3523f,  0.4395f,0.7933f,0.4212f,  0.4178f,0.8119f,0.4078f,  0.3036f,0.8291f,0.4694f,  0.2806f,0.8452f,0.4549f,  0.1613f,0.8457f,0.5087f,  0.1375f,0.8588f,0.4936f,  0.0161f,0.8418f,0.5395f,  -0.0075f,0.8515f,0.5243f,  -0.1280f,0.8171f,0.5622f,  
	-0.1507f,0.8230f,0.5477f,  -0.2671f,0.7714f,0.5776f,  -0.2881f,0.7735f,0.5646f,  -0.3977f,0.7054f,0.5867f,  -0.4161f,0.7037f,0.5759f,  -0.5160f,0.6204f,0.5907f,  -0.5310f,0.6151f,0.5829f,  -0.6186f,0.5182f,0.5906f,  -0.6294f,0.5097f,0.5865f,  -0.7027f,0.4014f,0.5874f,  -0.7087f,0.3902f,0.5878f,  -0.7659f,0.2727f,0.5823f,  -0.7664f,0.2596f,0.5875f,  -0.8063f,0.1356f,0.5758f,  -0.8010f,0.1214f,0.5863f,  
	-0.8227f,-0.0063f,0.5684f,  -0.8113f,-0.0207f,0.5843f,  -0.8147f,-0.1492f,0.5604f,  -0.7972f,-0.1628f,0.5813f,  -0.7826f,-0.2892f,0.5513f,  -0.7591f,-0.3011f,0.5771f,  -0.7274f,-0.4225f,0.5406f,  -0.6984f,-0.4319f,0.5707f,  -0.6510f,-0.5458f,0.5275f,  -0.6170f,-0.5519f,0.5610f,  -0.5559f,-0.6559f,0.5107f,  -0.5177f,-0.6582f,0.5465f,  -0.4451f,-0.7503f,0.4888f,  -0.4038f,-0.7485f,0.5260f,  -0.3224f,-0.8270f,0.4605f,  
	-0.2791f,-0.8211f,0.4979f,  -0.1918f,-0.8849f,0.4244f,  -0.1476f,-0.8752f,0.4608f,  -0.0576f,-0.9235f,0.3793f,  -0.0137f,-0.9103f,0.4136f,  0.0759f,-0.9429f,0.3243f,  0.1182f,-0.9271f,0.3558f,  0.2044f,-0.9440f,0.2590f,  0.2441f,-0.9263f,0.2869f,  0.3238f,-0.9282f,0.1834f,  0.3600f,-0.9096f,0.2073f,  0.4306f,-0.8972f,0.0981f,  0.4624f,-0.8788f,0.1178f,  0.5216f,-0.8532f,0.0042f,  0.5484f,-0.8360f,0.0196f,  
	0.5943f,-0.7984f,-0.0969f,  0.6156f,-0.7834f,-0.0856f,  0.6466f,-0.7353f,-0.2031f,  0.6624f,-0.7232f,-0.1955f,  0.6774f,-0.6661f,-0.3121f,  0.6877f,-0.6576f,-0.3077f,  0.6861f,-0.5930f,-0.4214f,  0.6910f,-0.5886f,-0.4196f,  0.0552f,-0.6210f,-0.7818f,  0.0631f,-0.6266f,-0.7768f,  0.1918f,-0.6038f,-0.7737f,  0.2001f,-0.6079f,-0.7684f,  0.3252f,-0.5731f,-0.7522f,  0.3337f,-0.5755f,-0.7466f,  0.4525f,-0.5292f,-0.7178f,  
	0.4610f,-0.5295f,-0.7121f,  0.5713f,-0.4723f,-0.6713f,  0.5794f,-0.4704f,-0.6656f,  0.6788f,-0.4031f,-0.6138f,  0.6862f,-0.3987f,-0.6084f,  0.7728f,-0.3226f,-0.5465f,  0.7791f,-0.3156f,-0.5416f,  0.8510f,-0.2321f,-0.4711f,  0.8559f,-0.2225f,-0.4668f,  0.9115f,-0.1332f,-0.3891f,  0.9147f,-0.1208f,-0.3857f,  0.9528f,-0.0276f,-0.3022f,  0.9538f,-0.0127f,-0.3001f,  0.9737f,0.0824f,-0.2124f,  0.9722f,0.0997f,-0.2117f,  
	0.9733f,0.1945f,-0.1216f,  0.9692f,0.2138f,-0.1226f,  0.9515f,0.3061f,-0.0315f,  0.9444f,0.3269f,-0.0344f,  0.9084f,0.4143f,0.0561f,  0.8983f,0.4364f,0.0511f,  0.8447f,0.5167f,0.1395f,  0.8316f,0.5393f,0.1325f,  0.7618f,0.6102f,0.2175f,  0.7458f,0.6328f,0.2083f,  0.6614f,0.6923f,0.2887f,  0.6427f,0.7141f,0.2774f,  0.5458f,0.7602f,0.3523f,  0.5249f,0.7807f,0.3392f,  0.4178f,0.8119f,0.4078f,  
	0.3951f,0.8302f,0.3932f,  0.2806f,0.8452f,0.4549f,  0.2566f,0.8609f,0.4392f,  0.1375f,0.8588f,0.4936f,  0.1131f,0.8713f,0.4775f,  -0.0075f,0.8515f,0.5243f,  -0.0316f,0.8606f,0.5084f,  -0.1507f,0.8230f,0.5477f,  -0.1736f,0.8283f,0.5327f,  -0.2881f,0.7735f,0.5646f,  -0.3092f,0.7749f,0.5513f,  -0.4161f,0.7037f,0.5759f,  -0.4342f,0.7014f,0.5653f,  -0.5310f,0.6151f,0.5829f,  -0.5454f,0.6093f,0.5756f,  
	-0.6294f,0.5097f,0.5865f,  -0.6394f,0.5010f,0.5833f,  -0.7087f,0.3902f,0.5878f,  -0.7135f,0.3791f,0.5892f,  -0.7664f,0.2596f,0.5875f,  -0.7656f,0.2470f,0.5941f,  -0.8010f,0.1214f,0.5863f,  -0.7940f,0.1082f,0.5982f,  -0.8113f,-0.0207f,0.5843f,  -0.7982f,-0.0337f,0.6015f,  -0.7972f,-0.1628f,0.5813f,  -0.7779f,-0.1745f,0.6037f,  -0.7591f,-0.3011f,0.5771f,  -0.7338f,-0.3107f,0.6041f,  -0.6984f,-0.4319f,0.5707f,  
	-0.6676f,-0.4385f,0.6016f,  -0.6170f,-0.5519f,0.5610f,  -0.5815f,-0.5549f,0.5949f,  -0.5177f,-0.6582f,0.5465f,  -0.4783f,-0.6572f,0.5825f,  -0.4038f,-0.7485f,0.5260f,  -0.3615f,-0.7433f,0.5628f,  -0.2791f,-0.8211f,0.4979f,  -0.2350f,-0.8119f,0.5344f,  -0.1476f,-0.8752f,0.4608f,  -0.1031f,-0.8622f,0.4959f,  -0.0137f,-0.9103f,0.4136f,  0.0301f,-0.8943f,0.4465f,  0.1182f,-0.9271f,0.3558f,  0.1601f,-0.9087f,0.3855f,  
	0.2441f,-0.9263f,0.2869f,  0.2831f,-0.9066f,0.3130f,  0.3600f,-0.9096f,0.2073f,  0.3952f,-0.8895f,0.2294f,  0.4624f,-0.8788f,0.1178f,  0.4931f,-0.8593f,0.1357f,  0.5484f,-0.8360f,0.0196f,  0.5741f,-0.8181f,0.0335f,  0.6156f,-0.7834f,-0.0856f,  0.6361f,-0.7679f,-0.0755f,  0.6624f,-0.7232f,-0.1955f,  0.6774f,-0.7110f,-0.1888f,  0.6877f,-0.6576f,-0.3077f,  0.6973f,-0.6492f,-0.3039f,  0.6910f,-0.5886f,-0.4196f,  
	0.6954f,-0.5844f,-0.4182f,  0.0631f,-0.6266f,-0.7768f,  0.0717f,-0.6323f,-0.7714f,  0.2001f,-0.6079f,-0.7684f,  0.2091f,-0.6120f,-0.7627f,  0.3337f,-0.5755f,-0.7466f,  0.3428f,-0.5777f,-0.7407f,  0.4610f,-0.5295f,-0.7121f,  0.4700f,-0.5296f,-0.7061f,  0.5794f,-0.4704f,-0.6656f,  0.5879f,-0.4680f,-0.6598f,  0.6862f,-0.3987f,-0.6084f,  0.6940f,-0.3938f,-0.6028f,  0.7791f,-0.3156f,-0.5416f,  0.7857f,-0.3080f,-0.5366f,  
	0.8559f,-0.2225f,-0.4668f,  0.8609f,-0.2120f,-0.4626f,  0.9147f,-0.1208f,-0.3857f,  0.9177f,-0.1076f,-0.3825f,  0.9538f,-0.0127f,-0.3001f,  0.9545f,0.0031f,-0.2982f,  0.9722f,0.0997f,-0.2117f,  0.9702f,0.1178f,-0.2115f,  0.9692f,0.2138f,-0.1226f,  0.9643f,0.2339f,-0.1242f,  0.9444f,0.3269f,-0.0344f,  0.9365f,0.3486f,-0.0381f,  0.8983f,0.4364f,0.0511f,  0.8872f,0.4592f,0.0452f,  0.8316f,0.5393f,0.1325f,  
	0.8174f,0.5625f,0.1243f,  0.7458f,0.6328f,0.2083f,  0.7287f,0.6557f,0.1978f,  0.6427f,0.7141f,0.2774f,  0.6229f,0.7361f,0.2649f,  0.5249f,0.7807f,0.3392f,  0.5029f,0.8010f,0.3248f,  0.3951f,0.8302f,0.3932f,  0.3713f,0.8484f,0.3774f,  0.2566f,0.8609f,0.4392f,  0.2318f,0.8763f,0.4224f,  0.1131f,0.8713f,0.4775f,  0.0880f,0.8834f,0.4603f,  -0.0316f,0.8606f,0.5084f,  -0.0562f,0.8690f,0.4917f,  
	-0.1736f,0.8283f,0.5327f,  -0.1969f,0.8329f,0.5172f,  -0.3092f,0.7749f,0.5513f,  -0.3301f,0.7757f,0.5379f,  -0.4342f,0.7014f,0.5653f,  -0.4520f,0.6985f,0.5547f,  -0.5454f,0.6093f,0.5756f,  -0.5591f,0.6033f,0.5687f,  -0.6394f,0.5010f,0.5833f,  -0.6483f,0.4922f,0.5808f,  -0.7135f,0.3791f,0.5892f,  -0.7171f,0.3684f,0.5917f,  -0.7656f,0.2470f,0.5941f,  -0.7632f,0.2352f,0.6018f,  -0.7940f,0.1082f,0.5982f,  
	-0.7855f,0.0961f,0.6113f,  -0.7982f,-0.0337f,0.6015f,  -0.7833f,-0.0449f,0.6200f,  -0.7779f,-0.1745f,0.6037f,  -0.7568f,-0.1841f,0.6273f,  -0.7338f,-0.3107f,0.6041f,  -0.7068f,-0.3176f,0.6321f,  -0.6676f,-0.4385f,0.6016f,  -0.6353f,-0.4422f,0.6331f,  -0.5815f,-0.5549f,0.5949f,  -0.5446f,-0.5547f,0.6291f,  -0.4783f,-0.6572f,0.5825f,  -0.4378f,-0.6529f,0.6182f,  -0.3615f,-0.7433f,0.5628f,  -0.3185f,-0.7348f,0.5989f,  
	-0.2350f,-0.8119f,0.5344f,  -0.1907f,-0.7994f,0.5697f,  -0.1031f,-0.8622f,0.4959f,  -0.0586f,-0.8463f,0.5295f,  0.0301f,-0.8943f,0.4465f,  0.0735f,-0.8755f,0.4775f,  0.1601f,-0.9087f,0.3855f,  0.2015f,-0.8880f,0.4133f,  0.2831f,-0.9066f,0.3130f,  0.3213f,-0.8850f,0.3371f,  0.3952f,-0.8895f,0.2294f,  0.4295f,-0.8679f,0.2495f,  0.4931f,-0.8593f,0.1357f,  0.5228f,-0.8388f,0.1519f,  0.5741f,-0.8181f,0.0335f,  
	0.5989f,-0.7995f,0.0458f,  0.6361f,-0.7679f,-0.0755f,  0.6556f,-0.7522f,-0.0666f,  0.6774f,-0.7110f,-0.1888f,  0.6916f,-0.6987f,-0.1830f,  0.6973f,-0.6492f,-0.3039f,  0.7062f,-0.6409f,-0.3007f,  0.6954f,-0.5844f,-0.4182f,  0.6993f,-0.5805f,-0.4170f,  0.0717f,-0.6323f,-0.7714f,  0.0810f,-0.6380f,-0.7658f,  0.2091f,-0.6120f,-0.7627f,  0.2188f,-0.6160f,-0.7567f,  0.3428f,-0.5777f,-0.7407f,  0.3526f,-0.5797f,-0.7346f,  
	0.4700f,-0.5296f,-0.7061f,  0.4796f,-0.5293f,-0.6999f,  0.5879f,-0.4680f,-0.6598f,  0.5969f,-0.4651f,-0.6537f,  0.6940f,-0.3938f,-0.6028f,  0.7020f,-0.3882f,-0.5971f,  0.7857f,-0.3080f,-0.5366f,  0.7923f,-0.2995f,-0.5315f,  0.8609f,-0.2120f,-0.4626f,  0.8658f,-0.2006f,-0.4585f,  0.9177f,-0.1076f,-0.3825f,  0.9204f,-0.0934f,-0.3796f,  0.9545f,0.0031f,-0.2982f,  0.9547f,0.0199f,-0.2968f,  0.9702f,0.1178f,-0.2115f,  
	0.9676f,0.1369f,-0.2120f,  0.9643f,0.2339f,-0.1242f,  0.9586f,0.2549f,-0.1267f,  0.9365f,0.3486f,-0.0381f,  0.9276f,0.3711f,-0.0428f,  0.8872f,0.4592f,0.0452f,  0.8751f,0.4825f,0.0381f,  0.8174f,0.5625f,0.1243f,  0.8021f,0.5861f,0.1148f,  0.7287f,0.6557f,0.1978f,  0.7103f,0.6789f,0.1860f,  0.6229f,0.7361f,0.2649f,  0.6020f,0.7581f,0.2509f,  0.5029f,0.8010f,0.3248f,  0.4796f,0.8213f,0.3090f,  
	0.3713f,0.8484f,0.3774f,  0.3465f,0.8662f,0.3602f,  0.2318f,0.8763f,0.4224f,  0.2062f,0.8910f,0.4044f,  0.0880f,0.8834f,0.4603f,  0.0623f,0.8947f,0.4422f,  -0.0562f,0.8690f,0.4917f,  -0.0813f,0.8766f,0.4743f,  -0.1969f,0.8329f,0.5172f,  -0.2202f,0.8368f,0.5013f,  -0.3301f,0.7757f,0.5379f,  -0.3509f,0.7758f,0.5244f,  -0.4520f,0.6985f,0.5547f,  -0.4693f,0.6952f,0.5445f,  -0.5591f,0.6033f,0.5687f,  
	-0.5721f,0.5969f,0.5625f,  -0.6483f,0.4922f,0.5808f,  -0.6563f,0.4836f,0.5792f,  -0.7171f,0.3684f,0.5917f,  -0.7193f,0.3582f,0.5952f,  -0.7632f,0.2352f,0.6018f,  -0.7594f,0.2242f,0.6107f,  -0.7855f,0.0961f,0.6113f,  -0.7754f,0.0854f,0.6257f,  -0.7833f,-0.0449f,0.6200f,  -0.7667f,-0.0543f,0.6396f,  -0.7568f,-0.1841f,0.6273f,  -0.7340f,-0.1913f,0.6517f,  -0.7068f,-0.3176f,0.6321f,  -0.6783f,-0.3218f,0.6606f,  
	-0.6353f,-0.4422f,0.6331f,  -0.6016f,-0.4427f,0.6649f,  -0.5446f,-0.5547f,0.6291f,  -0.5066f,-0.5512f,0.6630f,  -0.4378f,-0.6529f,0.6182f,  -0.3965f,-0.6451f,0.6532f,  -0.3185f,-0.7348f,0.5989f,  -0.2750f,-0.7229f,0.6338f,  -0.1907f,-0.7994f,0.5697f,  -0.1463f,-0.7838f,0.6036f,  -0.0586f,-0.8463f,0.5295f,  -0.0144f,-0.8275f,0.5614f,  0.0735f,-0.8755f,0.4775f,  0.1164f,-0.8543f,0.5065f,  0.2015f,-0.8880f,0.4133f,  
	0.2419f,-0.8653f,0.4390f,  0.3213f,-0.8850f,0.3371f,  0.3585f,-0.8617f,0.3591f,  0.4295f,-0.8679f,0.2495f,  0.4626f,-0.8452f,0.2677f,  0.5228f,-0.8388f,0.1519f,  0.5514f,-0.8175f,0.1663f,  0.5989f,-0.7995f,0.0458f,  0.6225f,-0.7805f,0.0566f,  0.6556f,-0.7522f,-0.0666f,  0.6741f,-0.7363f,-0.0590f,  0.6916f,-0.6987f,-0.1830f,  0.7050f,-0.6865f,-0.1782f,  0.7062f,-0.6409f,-0.3007f,  0.7146f,-0.6328f,-0.2981f,  
	0.6993f,-0.5805f,-0.4170f,  0.7029f,-0.5769f,-0.4161f,  0.0810f,-0.6380f,-0.7658f,  0.0910f,-0.6438f,-0.7598f,  0.2188f,-0.6160f,-0.7567f,  0.2292f,-0.6200f,-0.7504f,  0.3526f,-0.5797f,-0.7346f,  0.3630f,-0.5815f,-0.7281f,  0.4796f,-0.5293f,-0.6999f,  0.4896f,-0.5285f,-0.6935f,  0.5969f,-0.4651f,-0.6537f,  0.6063f,-0.4617f,-0.6475f,  0.7020f,-0.3882f,-0.5971f,  0.7103f,-0.3819f,-0.5914f,  0.7923f,-0.2995f,-0.5315f,  
	0.7991f,-0.2902f,-0.5265f,  0.8658f,-0.2006f,-0.4585f,  0.8706f,-0.1884f,-0.4545f,  0.9204f,-0.0934f,-0.3796f,  0.9228f,-0.0784f,-0.3771f,  0.9547f,0.0199f,-0.2968f,  0.9544f,0.0375f,-0.2960f,  0.9676f,0.1369f,-0.2120f,  0.9644f,0.1568f,-0.2131f,  0.9586f,0.2549f,-0.1267f,  0.9521f,0.2768f,-0.1301f,  0.9276f,0.3711f,-0.0428f,  0.9177f,0.3943f,-0.0486f,  0.8751f,0.4825f,0.0381f,  0.8618f,0.5064f,0.0298f,  
	0.8021f,0.5861f,0.1148f,  0.7855f,0.6100f,0.1039f,  0.7103f,0.6789f,0.1860f,  0.6908f,0.7021f,0.1726f,  0.6020f,0.7581f,0.2509f,  0.5797f,0.7801f,0.2354f,  0.4796f,0.8213f,0.3090f,  0.4553f,0.8412f,0.2917f,  0.3465f,0.8662f,0.3602f,  0.3208f,0.8834f,0.3416f,  0.2062f,0.8910f,0.4044f,  0.1797f,0.9052f,0.3852f,  0.0623f,0.8947f,0.4422f,  0.0358f,0.9053f,0.4232f,  -0.0813f,0.8766f,0.4743f,  
	-0.1066f,0.8835f,0.4561f,  -0.2202f,0.8368f,0.5013f,  -0.2437f,0.8398f,0.4850f,  -0.3509f,0.7758f,0.5244f,  -0.3714f,0.7753f,0.5108f,  -0.4693f,0.6952f,0.5445f,  -0.4861f,0.6914f,0.5344f,  -0.5721f,0.5969f,0.5625f,  -0.5843f,0.5905f,0.5567f,  -0.6563f,0.4836f,0.5792f,  -0.6632f,0.4751f,0.5783f,  -0.7193f,0.3582f,0.5952f,  -0.7204f,0.3485f,0.5997f,  -0.7594f,0.2242f,0.6107f,  -0.7542f,0.2144f,0.6207f,  
	-0.7754f,0.0854f,0.6257f,  -0.7636f,0.0764f,0.6411f,  -0.7667f,-0.0543f,0.6396f,  -0.7486f,-0.0617f,0.6601f,  -0.7340f,-0.1913f,0.6517f,  -0.7097f,-0.1960f,0.6767f,  -0.6783f,-0.3218f,0.6606f,  -0.6483f,-0.3232f,0.6894f,  -0.6016f,-0.4427f,0.6649f,  -0.5667f,-0.4402f,0.6965f,  -0.5066f,-0.5512f,0.6630f,  -0.4677f,-0.5444f,0.6963f,  -0.3965f,-0.6451f,0.6532f,  -0.3547f,-0.6341f,0.6871f,  -0.2750f,-0.7229f,0.6338f,  
	-0.2314f,-0.7079f,0.6673f,  -0.1463f,-0.7838f,0.6036f,  -0.1020f,-0.7652f,0.6356f,  -0.0144f,-0.8275f,0.5614f,  0.0293f,-0.8060f,0.5911f,  0.1164f,-0.8543f,0.5065f,  0.1584f,-0.8309f,0.5334f,  0.2419f,-0.8653f,0.4390f,  0.2814f,-0.8408f,0.4625f,  0.3585f,-0.8617f,0.3591f,  0.3945f,-0.8371f,0.3789f,  0.4626f,-0.8452f,0.2677f,  0.4946f,-0.8215f,0.2839f,  0.5514f,-0.8175f,0.1663f,  0.5788f,-0.7956f,0.1789f,  
	0.6225f,-0.7805f,0.0566f,  0.6451f,-0.7612f,0.0660f,  0.6741f,-0.7363f,-0.0590f,  0.6917f,-0.7203f,-0.0526f,  0.7050f,-0.6865f,-0.1782f,  0.7176f,-0.6743f,-0.1741f,  0.7146f,-0.6328f,-0.2981f,  0.7224f,-0.6249f,-0.2960f,  0.7029f,-0.5769f,-0.4161f,  0.7061f,-0.5735f,-0.4154f,  0.0910f,-0.6438f,-0.7598f,  0.1018f,-0.6496f,-0.7534f,  0.2292f,-0.6200f,-0.7504f,  0.2403f,-0.6237f,-0.7438f,  0.3630f,-0.5815f,-0.7281f,  
	0.3740f,-0.5829f,-0.7213f,  0.4896f,-0.5285f,-0.6935f,  0.5002f,-0.5274f,-0.6868f,  0.6063f,-0.4617f,-0.6475f,  0.6160f,-0.4577f,-0.6411f,  0.7103f,-0.3819f,-0.5914f,  0.7187f,-0.3749f,-0.5856f,  0.7991f,-0.2902f,-0.5265f,  0.8059f,-0.2801f,-0.5216f,  0.8706f,-0.1884f,-0.4545f,  0.8752f,-0.1753f,-0.4509f,  0.9228f,-0.0784f,-0.3771f,  0.9249f,-0.0625f,-0.3751f,  0.9544f,0.0375f,-0.2960f,  0.9536f,0.0561f,-0.2959f,  
	0.9644f,0.1568f,-0.2131f,  0.9603f,0.1777f,-0.2151f,  0.9521f,0.2768f,-0.1301f,  0.9446f,0.2994f,-0.1346f,  0.9177f,0.3943f,-0.0486f,  0.9067f,0.4181f,-0.0557f,  0.8618f,0.5064f,0.0298f,  0.8473f,0.5308f,0.0200f,  0.7855f,0.6100f,0.1039f,  0.7678f,0.6342f,0.0914f,  0.6908f,0.7021f,0.1726f,  0.6699f,0.7255f,0.1577f,  0.5797f,0.7801f,0.2354f,  0.5563f,0.8018f,0.2183f,  0.4553f,0.8412f,0.2917f,  
	0.4299f,0.8607f,0.2729f,  0.3208f,0.8834f,0.3416f,  0.2940f,0.9001f,0.3216f,  0.1797f,0.9052f,0.3852f,  0.1522f,0.9186f,0.3648f,  0.0358f,0.9053f,0.4232f,  0.0088f,0.9151f,0.4031f,  -0.1066f,0.8835f,0.4561f,  -0.1324f,0.8895f,0.4374f,  -0.2437f,0.8398f,0.4850f,  -0.2672f,0.8421f,0.4684f,  -0.3714f,0.7753f,0.5108f,  -0.3918f,0.7741f,0.4973f,  -0.4861f,0.6914f,0.5344f,  -0.5023f,0.6873f,0.5247f,  
	-0.5843f,0.5905f,0.5567f,  -0.5957f,0.5839f,0.5516f,  -0.6632f,0.4751f,0.5783f,  -0.6690f,0.4669f,0.5783f,  -0.7204f,0.3485f,0.5997f,  -0.7201f,0.3397f,0.6051f,  -0.7542f,0.2144f,0.6207f,  -0.7475f,0.2058f,0.6316f,  -0.7636f,0.0764f,0.6411f,  -0.7504f,0.0690f,0.6573f,  -0.7486f,-0.0617f,0.6601f,  -0.7290f,-0.0668f,0.6813f,  -0.7097f,-0.1960f,0.6767f,  -0.6840f,-0.1981f,0.7021f,  -0.6483f,-0.3232f,0.6894f,  
	-0.6172f,-0.3216f,0.7181f,  -0.5667f,-0.4402f,0.6965f,  -0.5310f,-0.4345f,0.7275f,  -0.4677f,-0.5444f,0.6963f,  -0.4283f,-0.5345f,0.7286f,  -0.3547f,-0.6341f,0.6871f,  -0.3126f,-0.6200f,0.7197f,  -0.2314f,-0.7079f,0.6673f,  -0.1879f,-0.6899f,0.6990f,  -0.1020f,-0.7652f,0.6356f,  -0.0583f,-0.7440f,0.6656f,  0.0293f,-0.8060f,0.5911f,  0.0722f,-0.7823f,0.6187f,  0.1584f,-0.8309f,0.5334f,  0.1995f,-0.8055f,0.5580f,  
	0.2814f,-0.8408f,0.4625f,  0.3197f,-0.8148f,0.4837f,  0.3945f,-0.8371f,0.3789f,  0.4293f,-0.8114f,0.3966f,  0.4946f,-0.8215f,0.2839f,  0.5253f,-0.7970f,0.2981f,  0.5788f,-0.7956f,0.1789f,  0.6051f,-0.7732f,0.1898f,  0.6451f,-0.7612f,0.0660f,  0.6666f,-0.7418f,0.0739f,  0.6917f,-0.7203f,-0.0526f,  0.7084f,-0.7043f,-0.0472f,  0.7176f,-0.6743f,-0.1741f,  0.7295f,-0.6623f,-0.1709f,  0.7224f,-0.6249f,-0.2960f,  
	0.7296f,-0.6173f,-0.2944f,  0.7061f,-0.5735f,-0.4154f,  0.7089f,-0.5704f,-0.4149f,  0.1018f,-0.6496f,-0.7534f,  0.1134f,-0.6553f,-0.7468f,  0.2403f,-0.6237f,-0.7438f,  0.2520f,-0.6273f,-0.7369f,  0.3740f,-0.5829f,-0.7213f,  0.3856f,-0.5840f,-0.7143f,  0.5002f,-0.5274f,-0.6868f,  0.5112f,-0.5257f,-0.6799f,  0.6160f,-0.4577f,-0.6411f,  0.6261f,-0.4531f,-0.6346f,  0.7187f,-0.3749f,-0.5856f,  0.7274f,-0.3671f,-0.5798f,  
	0.8059f,-0.2801f,-0.5216f,  0.8126f,-0.2692f,-0.5170f,  0.8752f,-0.1753f,-0.4509f,  0.8795f,-0.1614f,-0.4477f,  0.9249f,-0.0625f,-0.3751f,  0.9265f,-0.0456f,-0.3736f,  0.9536f,0.0561f,-0.2959f,  0.9520f,0.0756f,-0.2965f,  0.9603f,0.1777f,-0.2151f,  0.9553f,0.1993f,-0.2182f,  0.9446f,0.2994f,-0.1346f,  0.9361f,0.3226f,-0.1402f,  0.9067f,0.4181f,-0.0557f,  0.8945f,0.4424f,-0.0642f,  0.8473f,0.5308f,0.0200f,  
	0.8316f,0.5554f,0.0087f,  0.7678f,0.6342f,0.0914f,  0.7486f,0.6585f,0.0774f,  0.6699f,0.7255f,0.1577f,  0.6477f,0.7487f,0.1411f,  0.5563f,0.8018f,0.2183f,  0.5316f,0.8232f,0.1995f,  0.4299f,0.8607f,0.2729f,  0.4033f,0.8796f,0.2525f,  0.2940f,0.9001f,0.3216f,  0.2661f,0.9160f,0.3002f,  0.1522f,0.9186f,0.3648f,  0.1241f,0.9310f,0.3432f,  0.0088f,0.9151f,0.4031f,  -0.0188f,0.9239f,0.3822f,  
	-0.1324f,0.8895f,0.4374f,  -0.1584f,0.8945f,0.4180f,  -0.2672f,0.8421f,0.4684f,  -0.2907f,0.8436f,0.4515f,  -0.3918f,0.7741f,0.4973f,  -0.4118f,0.7723f,0.4837f,  -0.5023f,0.6873f,0.5247f,  -0.5180f,0.6827f,0.5154f,  -0.5957f,0.5839f,0.5516f,  -0.6062f,0.5773f,0.5470f,  -0.6690f,0.4669f,0.5783f,  -0.6737f,0.4591f,0.5790f,  -0.7201f,0.3397f,0.6051f,  -0.7186f,0.3317f,0.6113f,  -0.7475f,0.2058f,0.6316f,  
	-0.7394f,0.1986f,0.6433f,  -0.7504f,0.0690f,0.6573f,  -0.7358f,0.0636f,0.6742f,  -0.7290f,-0.0668f,0.6813f,  -0.7080f,-0.0696f,0.7028f,  -0.6840f,-0.1981f,0.7021f,  -0.6572f,-0.1975f,0.7274f,  -0.6172f,-0.3216f,0.7181f,  -0.5852f,-0.3170f,0.7464f,  -0.5310f,-0.4345f,0.7275f,  -0.4945f,-0.4257f,0.7578f,  -0.4283f,-0.5345f,0.7286f,  -0.3885f,-0.5214f,0.7597f,  -0.3126f,-0.6200f,0.7197f,  -0.2706f,-0.6029f,0.7505f,  
	-0.1879f,-0.6899f,0.6990f,  -0.1448f,-0.6692f,0.7288f,  -0.0583f,-0.7440f,0.6656f,  -0.0152f,-0.7203f,0.6935f,  0.0722f,-0.7823f,0.6187f,  0.1142f,-0.7565f,0.6439f,  0.1995f,-0.8055f,0.5580f,  0.2394f,-0.7785f,0.5802f,  0.3197f,-0.8148f,0.4837f,  0.3567f,-0.7875f,0.5026f,  0.4293f,-0.8114f,0.3966f,  0.4628f,-0.7848f,0.4122f,  0.5253f,-0.7970f,0.2981f,  0.5547f,-0.7720f,0.3104f,  0.6051f,-0.7732f,0.1898f,  
	0.6301f,-0.7506f,0.1991f,  0.6666f,-0.7418f,0.0739f,  0.6870f,-0.7222f,0.0805f,  0.7084f,-0.7043f,-0.0472f,  0.7241f,-0.6883f,-0.0429f,  0.7295f,-0.6623f,-0.1709f,  0.7406f,-0.6505f,-0.1684f,  0.7296f,-0.6173f,-0.2944f,  0.7363f,-0.6098f,-0.2933f,  0.7089f,-0.5704f,-0.4149f,  0.7113f,-0.5675f,-0.4146f,  0.1134f,-0.6553f,-0.7468f,  0.1256f,-0.6610f,-0.7398f,  0.2520f,-0.6273f,-0.7369f,  0.2644f,-0.6307f,-0.7296f,  
	0.3856f,-0.5840f,-0.7143f,  0.3977f,-0.5847f,-0.7070f,  0.5112f,-0.5257f,-0.6799f,  0.5227f,-0.5235f,-0.6728f,  0.6261f,-0.4531f,-0.6346f,  0.6364f,-0.4477f,-0.6281f,  0.7274f,-0.3671f,-0.5798f,  0.7360f,-0.3586f,-0.5742f,  0.8126f,-0.2692f,-0.5170f,  0.8191f,-0.2575f,-0.5126f,  0.8795f,-0.1614f,-0.4477f,  0.8835f,-0.1465f,-0.4449f,  0.9265f,-0.0456f,-0.3736f,  0.9275f,-0.0278f,-0.3728f,  0.9520f,0.0756f,-0.2965f,  
	0.9497f,0.0958f,-0.2981f,  0.9553f,0.1993f,-0.2182f,  0.9494f,0.2216f,-0.2224f,  0.9361f,0.3226f,-0.1402f,  0.9264f,0.3465f,-0.1473f,  0.8945f,0.4424f,-0.0642f,  0.8811f,0.4670f,-0.0742f,  0.8316f,0.5554f,0.0087f,  0.8144f,0.5802f,-0.0043f,  0.7486f,0.6585f,0.0774f,  0.7281f,0.6827f,0.0615f,  0.6477f,0.7487f,0.1411f,  0.6243f,0.7715f,0.1227f,  0.5316f,0.8232f,0.1995f,  0.5056f,0.8440f,0.1790f,  
	0.4033f,0.8796f,0.2525f,  0.3754f,0.8978f,0.2304f,  0.2661f,0.9160f,0.3002f,  0.2373f,0.9310f,0.2774f,  0.1241f,0.9310f,0.3432f,  0.0951f,0.9425f,0.3204f,  -0.0188f,0.9239f,0.3822f,  -0.0469f,0.9316f,0.3603f,  -0.1584f,0.8945f,0.4180f,  -0.1847f,0.8986f,0.3980f,  -0.2907f,0.8436f,0.4515f,  -0.3142f,0.8442f,0.4344f,  -0.4118f,0.7723f,0.4837f,  -0.4314f,0.7699f,0.4703f,  -0.5180f,0.6827f,0.5154f,  
	-0.5331f,0.6778f,0.5064f,  -0.6062f,0.5773f,0.5470f,  -0.6158f,0.5708f,0.5431f,  -0.6737f,0.4591f,0.5790f,  -0.6774f,0.4519f,0.5805f,  -0.7186f,0.3317f,0.6113f,  -0.7158f,0.3247f,0.6182f,  -0.7394f,0.1986f,0.6433f,  -0.7301f,0.1928f,0.6556f,  -0.7358f,0.0636f,0.6742f,  -0.7199f,0.0601f,0.6914f,  -0.7080f,-0.0696f,0.7028f,  -0.6859f,-0.0700f,0.7243f,  -0.6572f,-0.1975f,0.7274f,  -0.6294f,-0.1942f,0.7525f,  
	-0.5852f,-0.3170f,0.7464f,  -0.5524f,-0.3096f,0.7739f,  -0.4945f,-0.4257f,0.7578f,  -0.4577f,-0.4140f,0.7868f,  -0.3885f,-0.5214f,0.7597f,  -0.3487f,-0.5055f,0.7892f,  -0.2706f,-0.6029f,0.7505f,  -0.2289f,-0.5831f,0.7795f,  -0.1448f,-0.6692f,0.7288f,  -0.1024f,-0.6461f,0.7564f,  -0.0152f,-0.7203f,0.6935f,  0.0269f,-0.6946f,0.7189f,  0.1142f,-0.7565f,0.6439f,  0.1550f,-0.7290f,0.6668f,  0.2394f,-0.7785f,0.5802f,  
	0.2780f,-0.7502f,0.6000f,  0.3567f,-0.7875f,0.5026f,  0.3923f,-0.7592f,0.5193f,  0.4628f,-0.7848f,0.4122f,  0.4949f,-0.7575f,0.4257f,  0.5547f,-0.7720f,0.3104f,  0.5828f,-0.7466f,0.3209f,  0.6301f,-0.7506f,0.1991f,  0.6539f,-0.7277f,0.2069f,  0.6870f,-0.7222f,0.0805f,  0.7064f,-0.7026f,0.0859f,  0.7241f,-0.6883f,-0.0429f,  0.7390f,-0.6726f,-0.0395f,  0.7406f,-0.6505f,-0.1684f,  0.7511f,-0.6389f,-0.1666f,  
	0.7363f,-0.6098f,-0.2933f,  0.7424f,-0.6027f,-0.2925f,  0.7113f,-0.5675f,-0.4146f,  0.7135f,-0.5650f,-0.4144f,  0.1256f,-0.6610f,-0.7398f,  0.1386f,-0.6666f,-0.7324f,  0.2644f,-0.6307f,-0.7296f,  0.2774f,-0.6338f,-0.7221f,  0.3977f,-0.5847f,-0.7070f,  0.4104f,-0.5850f,-0.6995f,  0.5227f,-0.5235f,-0.6728f,  0.5346f,-0.5207f,-0.6656f,  0.6364f,-0.4477f,-0.6281f,  0.6469f,-0.4417f,-0.6216f,  0.7360f,-0.3586f,-0.5742f,  
	0.7447f,-0.3493f,-0.5687f,  0.8191f,-0.2575f,-0.5126f,  0.8254f,-0.2449f,-0.5086f,  0.8835f,-0.1465f,-0.4449f,  0.8871f,-0.1308f,-0.4427f,  0.9275f,-0.0278f,-0.3728f,  0.9278f,-0.0093f,-0.3728f,  0.9497f,0.0958f,-0.2981f,  0.9465f,0.1169f,-0.3007f,  0.9494f,0.2216f,-0.2224f,  0.9425f,0.2446f,-0.2279f,  0.9264f,0.3465f,-0.1473f,  0.9156f,0.3707f,-0.1558f,  0.8811f,0.4670f,-0.0742f,  0.8663f,0.4920f,-0.0859f,  
	0.8144f,0.5802f,-0.0043f,  0.7960f,0.6051f,-0.0191f,  0.7281f,0.6827f,0.0615f,  0.7062f,0.7067f,0.0439f,  0.6243f,0.7715f,0.1227f,  0.5994f,0.7939f,0.1025f,  0.5056f,0.8440f,0.1790f,  0.4782f,0.8641f,0.1567f,  0.3754f,0.8978f,0.2304f,  0.3464f,0.9150f,0.2068f,  0.2373f,0.9310f,0.2774f,  0.2074f,0.9449f,0.2532f,  0.0951f,0.9425f,0.3204f,  0.0654f,0.9528f,0.2965f,  -0.0469f,0.9316f,0.3603f,  
	-0.0755f,0.9382f,0.3377f,  -0.1847f,0.8986f,0.3980f,  -0.2112f,0.9016f,0.3776f,  -0.3142f,0.8442f,0.4344f,  -0.3375f,0.8439f,0.4171f,  -0.4314f,0.7699f,0.4703f,  -0.4508f,0.7668f,0.4570f,  -0.5331f,0.6778f,0.5064f,  -0.5474f,0.6727f,0.4978f,  -0.6158f,0.5708f,0.5431f,  -0.6246f,0.5644f,0.5397f,  -0.6774f,0.4519f,0.5805f,  -0.6800f,0.4453f,0.5825f,  -0.7158f,0.3247f,0.6182f,  -0.7119f,0.3188f,0.6257f,  
	-0.7301f,0.1928f,0.6556f,  -0.7196f,0.1888f,0.6683f,  -0.7199f,0.0601f,0.6914f,  -0.7029f,0.0587f,0.7088f,  -0.6859f,-0.0700f,0.7243f,  -0.6628f,-0.0680f,0.7457f,  -0.6294f,-0.1942f,0.7525f,  -0.6008f,-0.1883f,0.7769f,  -0.5524f,-0.3096f,0.7739f,  -0.5192f,-0.2994f,0.8005f,  -0.4577f,-0.4140f,0.7868f,  -0.4208f,-0.3995f,0.8144f,  -0.3487f,-0.5055f,0.7892f,  -0.3091f,-0.4869f,0.8169f,  -0.2289f,-0.5831f,0.7795f,  
	-0.1878f,-0.5608f,0.8064f,  -0.1024f,-0.6461f,0.7564f,  -0.0608f,-0.6208f,0.7816f,  0.0269f,-0.6946f,0.7189f,  0.0680f,-0.6670f,0.7420f,  0.1550f,-0.7290f,0.6668f,  0.1945f,-0.7000f,0.6872f,  0.2780f,-0.7502f,0.6000f,  0.3152f,-0.7207f,0.6175f,  0.3923f,-0.7592f,0.5193f,  0.4266f,-0.7302f,0.5337f,  0.4949f,-0.7575f,0.4257f,  0.5256f,-0.7298f,0.4372f,  0.5828f,-0.7466f,0.3209f,  0.6096f,-0.7209f,0.3297f,  
	0.6539f,-0.7277f,0.2069f,  0.6765f,-0.7049f,0.2131f,  0.7064f,-0.7026f,0.0859f,  0.7247f,-0.6832f,0.0900f,  0.7390f,-0.6726f,-0.0395f,  0.7530f,-0.6570f,-0.0371f,  0.7511f,-0.6389f,-0.1666f,  0.7608f,-0.6275f,-0.1654f,  0.7424f,-0.6027f,-0.2925f,  0.7481f,-0.5958f,-0.2921f,  0.7135f,-0.5650f,-0.4144f,  0.7153f,-0.5627f,-0.4144f,  0.1386f,-0.6666f,-0.7324f,  0.1524f,-0.6719f,-0.7248f,  0.2774f,-0.6338f,-0.7221f,  
	0.2911f,-0.6365f,-0.7142f,  0.4104f,-0.5850f,-0.6995f,  0.4236f,-0.5848f,-0.6918f,  0.5346f,-0.5207f,-0.6656f,  0.5467f,-0.5173f,-0.6584f,  0.6469f,-0.4417f,-0.6216f,  0.6575f,-0.4350f,-0.6152f,  0.7447f,-0.3493f,-0.5687f,  0.7533f,-0.3391f,-0.5636f,  0.8254f,-0.2449f,-0.5086f,  0.8315f,-0.2314f,-0.5050f,  0.8871f,-0.1308f,-0.4427f,  0.8901f,-0.1142f,-0.4412f,  0.9278f,-0.0093f,-0.3728f,  0.9274f,0.0101f,-0.3739f,  
	0.9465f,0.1169f,-0.3007f,  0.9424f,0.1386f,-0.3045f,  0.9425f,0.2446f,-0.2279f,  0.9344f,0.2680f,-0.2348f,  0.9156f,0.3707f,-0.1558f,  0.9034f,0.3954f,-0.1660f,  0.8663f,0.4920f,-0.0859f,  0.8502f,0.5170f,-0.0994f,  0.7960f,0.6051f,-0.0191f,  0.7760f,0.6297f,-0.0358f,  0.7062f,0.7067f,0.0439f,  0.6828f,0.7302f,0.0242f,  0.5994f,0.7939f,0.1025f,  0.5729f,0.8157f,0.0804f,  0.4782f,0.8641f,0.1567f,  
	0.4496f,0.8833f,0.1327f,  0.3464f,0.9150f,0.2068f,  0.3162f,0.9311f,0.1816f,  0.2074f,0.9449f,0.2532f,  0.1767f,0.9576f,0.2276f,  0.0654f,0.9528f,0.2965f,  0.0350f,0.9618f,0.2715f,  -0.0755f,0.9382f,0.3377f,  -0.1046f,0.9436f,0.3143f,  -0.2112f,0.9016f,0.3776f,  -0.2378f,0.9034f,0.3568f,  -0.3375f,0.8439f,0.4171f,  -0.3608f,0.8426f,0.3998f,  -0.4508f,0.7668f,0.4570f,  -0.4697f,0.7631f,0.4440f,  
	-0.5474f,0.6727f,0.4978f,  -0.5612f,0.6673f,0.4897f,  -0.6246f,0.5644f,0.5397f,  -0.6325f,0.5583f,0.5369f,  -0.6800f,0.4453f,0.5825f,  -0.6815f,0.4394f,0.5852f,  -0.7119f,0.3188f,0.6257f,  -0.7069f,0.3142f,0.6337f,  -0.7196f,0.1888f,0.6683f,  -0.7079f,0.1864f,0.6812f,  -0.7029f,0.0587f,0.7088f,  -0.6850f,0.0593f,0.7262f,  -0.6628f,-0.0680f,0.7457f,  -0.6390f,-0.0637f,0.7666f,  -0.6008f,-0.1883f,0.7769f,  
	-0.5718f,-0.1798f,0.8005f,  -0.5192f,-0.2994f,0.8005f,  -0.4858f,-0.2867f,0.8257f,  -0.4208f,-0.3995f,0.8144f,  -0.3840f,-0.3825f,0.8404f,  -0.3091f,-0.4869f,0.8169f,  -0.2700f,-0.4659f,0.8426f,  -0.1878f,-0.5608f,0.8064f,  -0.1475f,-0.5364f,0.8310f,  -0.0608f,-0.6208f,0.7816f,  -0.0202f,-0.5936f,0.8045f,  0.0680f,-0.6670f,0.7420f,  0.1078f,-0.6379f,0.7626f,  0.1945f,-0.7000f,0.6872f,  0.2327f,-0.6698f,0.7051f,  
	0.3152f,-0.7207f,0.6175f,  0.3510f,-0.6903f,0.6326f,  0.4266f,-0.7302f,0.5337f,  0.4594f,-0.7006f,0.5461f,  0.5256f,-0.7298f,0.4372f,  0.5549f,-0.7017f,0.4468f,  0.6096f,-0.7209f,0.3297f,  0.6352f,-0.6951f,0.3367f,  0.6765f,-0.7049f,0.2131f,  0.6980f,-0.6821f,0.2180f,  0.7247f,-0.6832f,0.0900f,  0.7420f,-0.6639f,0.0930f,  0.7530f,-0.6570f,-0.0371f,  0.7662f,-0.6416f,-0.0356f,  0.7608f,-0.6275f,-0.1654f,  
	0.7699f,-0.6165f,-0.1648f,  0.7481f,-0.5958f,-0.2921f,  0.7533f,-0.5892f,-0.2921f,  0.7153f,-0.5627f,-0.4144f,  0.7168f,-0.5607f,-0.4144f,  0.1524f,-0.6719f,-0.7248f,  0.1668f,-0.6771f,-0.7167f,  0.2911f,-0.6365f,-0.7142f,  0.3054f,-0.6389f,-0.7061f,  0.4236f,-0.5848f,-0.6918f,  0.4372f,-0.5841f,-0.6839f,  0.5467f,-0.5173f,-0.6584f,  0.5591f,-0.5133f,-0.6511f,  0.6575f,-0.4350f,-0.6152f,  0.6682f,-0.4275f,-0.6089f,  
	0.7533f,-0.3391f,-0.5636f,  0.7617f,-0.3282f,-0.5587f,  0.8315f,-0.2314f,-0.5050f,  0.8371f,-0.2172f,-0.5021f,  0.8901f,-0.1142f,-0.4412f,  0.8924f,-0.0968f,-0.4406f,  0.9274f,0.0101f,-0.3739f,  0.9261f,0.0302f,-0.3760f,  0.9424f,0.1386f,-0.3045f,  0.9371f,0.1608f,-0.3097f,  0.9344f,0.2680f,-0.2348f,  0.9250f,0.2920f,-0.2433f,  0.9034f,0.3954f,-0.1660f,  0.8898f,0.4202f,-0.1779f,  0.8502f,0.5170f,-0.0994f,  
	0.8326f,0.5419f,-0.1147f,  0.7760f,0.6297f,-0.0358f,  0.7545f,0.6540f,-0.0544f,  0.6828f,0.7302f,0.0242f,  0.6578f,0.7532f,0.0027f,  0.5729f,0.8157f,0.0804f,  0.5451f,0.8364f,0.0563f,  0.4496f,0.8833f,0.1327f,  0.4196f,0.9014f,0.1069f,  0.3162f,0.9311f,0.1816f,  0.2849f,0.9460f,0.1548f,  0.1767f,0.9576f,0.2276f,  0.1449f,0.9689f,0.2007f,  0.0350f,0.9618f,0.2715f,  0.0038f,0.9694f,0.2456f,  
	-0.1046f,0.9436f,0.3143f,  -0.1341f,0.9475f,0.2903f,  -0.2378f,0.9034f,0.3568f,  -0.2647f,0.9040f,0.3357f,  -0.3608f,0.8426f,0.3998f,  -0.3839f,0.8404f,0.3825f,  -0.4697f,0.7631f,0.4440f,  -0.4882f,0.7588f,0.4312f,  -0.5612f,0.6673f,0.4897f,  -0.5742f,0.6618f,0.4820f,  -0.6325f,0.5583f,0.5369f,  -0.6396f,0.5525f,0.5346f,  -0.6815f,0.4394f,0.5852f,  -0.6821f,0.4343f,0.5882f,  -0.7069f,0.3142f,0.6337f,  
	-0.7009f,0.3109f,0.6420f,  -0.7079f,0.1864f,0.6812f,  -0.6954f,0.1857f,0.6942f,  -0.6850f,0.0593f,0.7262f,  -0.6662f,0.0620f,0.7432f,  -0.6390f,-0.0637f,0.7666f,  -0.6146f,-0.0571f,0.7868f,  -0.5718f,-0.1798f,0.8005f,  -0.5424f,-0.1689f,0.8229f,  -0.4858f,-0.2867f,0.8257f,  -0.4524f,-0.2715f,0.8495f,  -0.3840f,-0.3825f,0.8404f,  -0.3476f,-0.3631f,0.8645f,  -0.2700f,-0.4659f,0.8426f,  -0.2316f,-0.4428f,0.8662f,  
	-0.1475f,-0.5364f,0.8310f,  -0.1081f,-0.5101f,0.8533f,  -0.0202f,-0.5936f,0.8045f,  0.0192f,-0.5649f,0.8250f,  0.1078f,-0.6379f,0.7626f,  0.1462f,-0.6075f,0.7807f,  0.2327f,-0.6698f,0.7051f,  0.2695f,-0.6387f,0.7207f,  0.3510f,-0.6903f,0.6326f,  0.3854f,-0.6594f,0.6456f,  0.4594f,-0.7006f,0.5461f,  0.4908f,-0.6706f,0.5563f,  0.5549f,-0.7017f,0.4468f,  0.5829f,-0.6735f,0.4546f,  0.6352f,-0.6951f,0.3367f,  
	0.6594f,-0.6694f,0.3422f,  0.6980f,-0.6821f,0.2180f,  0.7184f,-0.6594f,0.2215f,  0.7420f,-0.6639f,0.0930f,  0.7584f,-0.6448f,0.0949f,  0.7662f,-0.6416f,-0.0356f,  0.7786f,-0.6266f,-0.0348f,  0.7699f,-0.6165f,-0.1648f,  0.7785f,-0.6057f,-0.1648f,  0.7533f,-0.5892f,-0.2921f,  0.7581f,-0.5829f,-0.2923f,  0.7168f,-0.5607f,-0.4144f,  0.7181f,-0.5590f,-0.4146f,  0.1668f,-0.6771f,-0.7167f,  0.1820f,-0.6820f,-0.7083f,  
	0.3054f,-0.6389f,-0.7061f,  0.3202f,-0.6409f,-0.6977f,  0.4372f,-0.5841f,-0.6839f,  0.4511f,-0.5828f,-0.6759f,  0.5591f,-0.5133f,-0.6511f,  0.5717f,-0.5086f,-0.6438f,  0.6682f,-0.4275f,-0.6089f,  0.6788f,-0.4192f,-0.6029f,  0.7617f,-0.3282f,-0.5587f,  0.7698f,-0.3164f,-0.5544f,  0.8371f,-0.2172f,-0.5021f,  0.8422f,-0.2021f,-0.4999f,  0.8924f,-0.0968f,-0.4406f,  0.8940f,-0.0787f,-0.4410f,  0.9261f,0.0302f,-0.3760f,  
	0.9239f,0.0508f,-0.3793f,  0.9371f,0.1608f,-0.3097f,  0.9307f,0.1836f,-0.3164f,  0.9250f,0.2920f,-0.2433f,  0.9142f,0.3161f,-0.2535f,  0.8898f,0.4202f,-0.1779f,  0.8748f,0.4449f,-0.1918f,  0.8326f,0.5419f,-0.1147f,  0.8133f,0.5666f,-0.1321f,  0.7545f,0.6540f,-0.0544f,  0.7314f,0.6778f,-0.0751f,  0.6578f,0.7532f,0.0027f,  0.6313f,0.7753f,-0.0210f,  0.5451f,0.8364f,0.0563f,  0.5158f,0.8562f,0.0304f,  
	0.4196f,0.9014f,0.1069f,  0.3882f,0.9182f,0.0793f,  0.2849f,0.9460f,0.1548f,  0.2523f,0.9594f,0.1265f,  0.1449f,0.9689f,0.2007f,  0.1122f,0.9786f,0.1726f,  0.0038f,0.9694f,0.2456f,  -0.0280f,0.9754f,0.2188f,  -0.1341f,0.9475f,0.2903f,  -0.1640f,0.9500f,0.2658f,  -0.2647f,0.9040f,0.3357f,  -0.2915f,0.9034f,0.3144f,  -0.3839f,0.8404f,0.3825f,  -0.4068f,0.8373f,0.3654f,  -0.4882f,0.7588f,0.4312f,  
	-0.5062f,0.7539f,0.4188f,  -0.5742f,0.6618f,0.4820f,  -0.5866f,0.6561f,0.4748f,  -0.6396f,0.5525f,0.5346f,  -0.6457f,0.5470f,0.5327f,  -0.6821f,0.4343f,0.5882f,  -0.6818f,0.4301f,0.5917f,  -0.7009f,0.3109f,0.6420f,  -0.6940f,0.3088f,0.6504f,  -0.6954f,0.1857f,0.6942f,  -0.6821f,0.1866f,0.7070f,  -0.6662f,0.0620f,0.7432f,  -0.6469f,0.0667f,0.7597f,  -0.6146f,-0.0571f,0.7868f,  -0.5898f,-0.0483f,0.8061f,  
	-0.5424f,-0.1689f,0.8229f,  -0.5131f,-0.1558f,0.8441f,  -0.4524f,-0.2715f,0.8495f,  -0.4193f,-0.2541f,0.8716f,  -0.3476f,-0.3631f,0.8645f,  -0.3118f,-0.3417f,0.8866f,  -0.2316f,-0.4428f,0.8662f,  -0.1940f,-0.4178f,0.8876f,  -0.1081f,-0.5101f,0.8533f,  -0.0698f,-0.4822f,0.8733f,  0.0192f,-0.5649f,0.8250f,  0.0573f,-0.5349f,0.8430f,  0.1462f,-0.6075f,0.7807f,  0.1833f,-0.5762f,0.7965f,  0.2695f,-0.6387f,0.7207f,  
	0.3047f,-0.6069f,0.7340f,  0.3854f,-0.6594f,0.6456f,  0.4183f,-0.6279f,0.6563f,  0.4908f,-0.6706f,0.5563f,  0.5208f,-0.6403f,0.5646f,  0.5829f,-0.6735f,0.4546f,  0.6096f,-0.6452f,0.4606f,  0.6594f,-0.6694f,0.3422f,  0.6825f,-0.6437f,0.3462f,  0.7184f,-0.6594f,0.2215f,  0.7378f,-0.6369f,0.2238f,  0.7584f,-0.6448f,0.0949f,  0.7739f,-0.6259f,0.0959f,  0.7786f,-0.6266f,-0.0348f,  0.7903f,-0.6118f,-0.0347f,  
	0.7785f,-0.6057f,-0.1648f,  0.7864f,-0.5952f,-0.1652f,  0.7581f,-0.5829f,-0.2923f,  0.7625f,-0.5769f,-0.2928f,  0.7181f,-0.5590f,-0.4146f,  0.7191f,-0.5576f,-0.4147f,  0.1820f,-0.6820f,-0.7083f,  0.1977f,-0.6867f,-0.6996f,  0.3202f,-0.6409f,-0.6977f,  0.3355f,-0.6424f,-0.6891f,  0.4511f,-0.5828f,-0.6759f,  0.4654f,-0.5810f,-0.6677f,  0.5717f,-0.5086f,-0.6438f,  0.5844f,-0.5031f,-0.6367f,  0.6788f,-0.4192f,-0.6029f,  
	0.6893f,-0.4102f,-0.5971f,  0.7698f,-0.3164f,-0.5544f,  0.7775f,-0.3039f,-0.5505f,  0.8422f,-0.2021f,-0.4999f,  0.8467f,-0.1863f,-0.4985f,  0.8940f,-0.0787f,-0.4410f,  0.8948f,-0.0600f,-0.4425f,  0.9239f,0.0508f,-0.3793f,  0.9205f,0.0721f,-0.3841f,  0.9307f,0.1836f,-0.3164f,  0.9230f,0.2066f,-0.3247f,  0.9142f,0.3161f,-0.2535f,  0.9021f,0.3403f,-0.2655f,  0.8748f,0.4449f,-0.1918f,  0.8582f,0.4695f,-0.2076f,  
	0.8133f,0.5666f,-0.1321f,  0.7924f,0.5908f,-0.1516f,  0.7314f,0.6778f,-0.0751f,  0.7066f,0.7008f,-0.0979f,  0.6313f,0.7753f,-0.0210f,  0.6031f,0.7963f,-0.0466f,  0.5158f,0.8562f,0.0304f,  0.4849f,0.8746f,0.0025f,  0.3882f,0.9182f,0.0793f,  0.3554f,0.9334f,0.0500f,  0.2523f,0.9594f,0.1265f,  0.2186f,0.9710f,0.0967f,  0.1122f,0.9786f,0.1726f,  0.0785f,0.9865f,0.1435f,  -0.0280f,0.9754f,0.2188f,  
	-0.0606f,0.9797f,0.1913f,  -0.1640f,0.9500f,0.2658f,  -0.1942f,0.9509f,0.2409f,  -0.2915f,0.9034f,0.3144f,  -0.3185f,0.9015f,0.2932f,  -0.4068f,0.8373f,0.3654f,  -0.4294f,0.8332f,0.3484f,  -0.5062f,0.7539f,0.4188f,  -0.5237f,0.7485f,0.4068f,  -0.5866f,0.6561f,0.4748f,  -0.5983f,0.6504f,0.4680f,  -0.6457f,0.5470f,0.5327f,  -0.6512f,0.5420f,0.5313f,  -0.6818f,0.4301f,0.5917f,  -0.6807f,0.4268f,0.5954f,  
	-0.6940f,0.3088f,0.6504f,  -0.6863f,0.3081f,0.6588f,  -0.6821f,0.1866f,0.7070f,  -0.6682f,0.1893f,0.7195f,  -0.6469f,0.0667f,0.7597f,  -0.6272f,0.0732f,0.7754f,  -0.5898f,-0.0483f,0.8061f,  -0.5649f,-0.0375f,0.8243f,  -0.5131f,-0.1558f,0.8441f,  -0.4838f,-0.1407f,0.8638f,  -0.4193f,-0.2541f,0.8716f,  -0.3867f,-0.2347f,0.8919f,  -0.3118f,-0.3417f,0.8866f,  -0.2767f,-0.3185f,0.9066f,  -0.1940f,-0.4178f,0.8876f,  
	-0.1575f,-0.3913f,0.9067f,  -0.0698f,-0.4822f,0.8733f,  -0.0327f,-0.4530f,0.8909f,  0.0573f,-0.5349f,0.8430f,  0.0940f,-0.5038f,0.8587f,  0.1833f,-0.5762f,0.7965f,  0.2189f,-0.5441f,0.8100f,  0.3047f,-0.6069f,0.7340f,  0.3386f,-0.5746f,0.7451f,  0.4183f,-0.6279f,0.6563f,  0.4497f,-0.5962f,0.6650f,  0.5208f,-0.6403f,0.5646f,  0.5493f,-0.6100f,0.5711f,  0.6096f,-0.6452f,0.4606f,  0.6350f,-0.6169f,0.4650f,  
	0.6825f,-0.6437f,0.3462f,  0.7044f,-0.6182f,0.3488f,  0.7378f,-0.6369f,0.2238f,  0.7561f,-0.6146f,0.2249f,  0.7739f,-0.6259f,0.0959f,  0.7886f,-0.6074f,0.0960f,  0.7903f,-0.6118f,-0.0347f,  0.8013f,-0.5972f,-0.0353f,  0.7864f,-0.5952f,-0.1652f,  0.7938f,-0.5850f,-0.1661f,  0.7625f,-0.5769f,-0.2928f,  0.7665f,-0.5712f,-0.2936f,  0.7191f,-0.5576f,-0.4147f,  0.7199f,-0.5564f,-0.4149f,  0.1977f,-0.6867f,-0.6996f,  
	0.2141f,-0.6909f,-0.6905f,  0.3355f,-0.6424f,-0.6891f,  0.3512f,-0.6434f,-0.6802f,  0.4654f,-0.5810f,-0.6677f,  0.4799f,-0.5784f,-0.6596f,  0.5844f,-0.5031f,-0.6367f,  0.5971f,-0.4970f,-0.6297f,  0.6893f,-0.4102f,-0.5971f,  0.6996f,-0.4005f,-0.5918f,  0.7775f,-0.3039f,-0.5505f,  0.7848f,-0.2906f,-0.5474f,  0.8467f,-0.1863f,-0.4985f,  0.8504f,-0.1698f,-0.4980f,  0.8948f,-0.0600f,-0.4425f,  0.8945f,-0.0406f,-0.4452f,  
	0.9205f,0.0721f,-0.3841f,  0.9159f,0.0937f,-0.3903f,  0.9230f,0.2066f,-0.3247f,  0.9139f,0.2298f,-0.3347f,  0.9021f,0.3403f,-0.2655f,  0.8883f,0.3643f,-0.2795f,  0.8582f,0.4695f,-0.2076f,  0.8399f,0.4937f,-0.2254f,  0.7924f,0.5908f,-0.1516f,  0.7699f,0.6143f,-0.1732f,  0.7066f,0.7008f,-0.0979f,  0.6801f,0.7227f,-0.1228f,  0.6031f,0.7963f,-0.0466f,  0.5733f,0.8160f,-0.0743f,  0.4849f,0.8746f,0.0025f,  
	0.4525f,0.8914f,-0.0272f,  0.3554f,0.9334f,0.0500f,  0.3213f,0.9468f,0.0192f,  0.2186f,0.9710f,0.0967f,  0.1838f,0.9808f,0.0658f,  0.0785f,0.9865f,0.1435f,  0.0440f,0.9926f,0.1135f,  -0.0606f,0.9797f,0.1913f,  -0.0936f,0.9821f,0.1633f,  -0.1942f,0.9509f,0.2409f,  -0.2247f,0.9502f,0.2159f,  -0.3185f,0.9015f,0.2932f,  -0.3454f,0.8982f,0.2720f,  -0.4294f,0.8332f,0.3484f,  -0.4518f,0.8281f,0.3319f,  
	-0.5237f,0.7485f,0.4068f,  -0.5407f,0.7426f,0.3953f,  -0.5983f,0.6504f,0.4680f,  -0.6093f,0.6446f,0.4617f,  -0.6512f,0.5420f,0.5313f,  -0.6558f,0.5374f,0.5302f,  -0.6807f,0.4268f,0.5954f,  -0.6789f,0.4243f,0.5993f,  -0.6863f,0.3081f,0.6588f,  -0.6780f,0.3086f,0.6671f,  -0.6682f,0.1893f,0.7195f,  -0.6538f,0.1934f,0.7315f,  -0.6272f,0.0732f,0.7754f,  -0.6072f,0.0815f,0.7904f,  -0.5649f,-0.0375f,0.8243f,  
	-0.5401f,-0.0249f,0.8412f,  -0.4838f,-0.1407f,0.8638f,  -0.4550f,-0.1237f,0.8819f,  -0.3867f,-0.2347f,0.8919f,  -0.3547f,-0.2136f,0.9103f,  -0.2767f,-0.3185f,0.9066f,  -0.2425f,-0.2937f,0.9246f,  -0.1575f,-0.3913f,0.9067f,  -0.1220f,-0.3635f,0.9236f,  -0.0327f,-0.4530f,0.8909f,  0.0032f,-0.4228f,0.9062f,  0.0940f,-0.5038f,0.8587f,  0.1294f,-0.4720f,0.8721f,  0.2189f,-0.5441f,0.8100f,  0.2531f,-0.5114f,0.8212f,  
	0.3386f,-0.5746f,0.7451f,  0.3709f,-0.5420f,0.7541f,  0.4497f,-0.5962f,0.6650f,  0.4797f,-0.5644f,0.6718f,  0.5493f,-0.6100f,0.5711f,  0.5766f,-0.5797f,0.5758f,  0.6350f,-0.6169f,0.4650f,  0.6591f,-0.5888f,0.4678f,  0.7044f,-0.6182f,0.3488f,  0.7253f,-0.5928f,0.3501f,  0.7561f,-0.6146f,0.2249f,  0.7734f,-0.5926f,0.2250f,  0.7886f,-0.6074f,0.0960f,  0.8024f,-0.5891f,0.0952f,  0.8013f,-0.5972f,-0.0353f,  
	0.8116f,-0.5830f,-0.0366f,  0.7938f,-0.5850f,-0.1661f,  0.8008f,-0.5751f,-0.1674f,  0.7665f,-0.5712f,-0.2936f,  0.7701f,-0.5658f,-0.2945f,  0.7199f,-0.5564f,-0.4149f,  0.7204f,-0.5556f,-0.4151f,  0.2141f,-0.6909f,-0.6905f,  0.2310f,-0.6948f,-0.6811f,  0.3512f,-0.6434f,-0.6802f,  0.3673f,-0.6438f,-0.6712f,  0.4799f,-0.5784f,-0.6596f,  0.4946f,-0.5753f,-0.6515f,  0.5971f,-0.4970f,-0.6297f,  0.6097f,-0.4902f,-0.6229f,  
	0.6996f,-0.4005f,-0.5918f,  0.7095f,-0.3900f,-0.5870f,  0.7848f,-0.2906f,-0.5474f,  0.7914f,-0.2767f,-0.5451f,  0.8504f,-0.1698f,-0.4980f,  0.8532f,-0.1527f,-0.4987f,  0.8945f,-0.0406f,-0.4452f,  0.8932f,-0.0209f,-0.4493f,  0.9159f,0.0937f,-0.3903f,  0.9100f,0.1154f,-0.3982f,  0.9139f,0.2298f,-0.3347f,  0.9033f,0.2530f,-0.3466f,  0.8883f,0.3643f,-0.2795f,  0.8729f,0.3882f,-0.2954f,  0.8399f,0.4937f,-0.2254f,  
	0.8199f,0.5172f,-0.2454f,  0.7699f,0.6143f,-0.1732f,  0.7455f,0.6368f,-0.1969f,  0.6801f,0.7227f,-0.1228f,  0.6519f,0.7434f,-0.1499f,  0.5733f,0.8160f,-0.0743f,  0.5418f,0.8341f,-0.1040f,  0.4525f,0.8914f,-0.0272f,  0.4185f,0.9063f,-0.0587f,  0.3213f,0.9468f,0.0192f,  0.2859f,0.9582f,-0.0131f,  0.1838f,0.9808f,0.0658f,  0.1477f,0.9885f,0.0338f,  0.0440f,0.9926f,0.1135f,  0.0087f,0.9965f,0.0828f,  
	-0.0936f,0.9821f,0.1633f,  -0.1272f,0.9827f,0.1350f,  -0.2247f,0.9502f,0.2159f,  -0.2554f,0.9478f,0.1909f,  -0.3454f,0.8982f,0.2720f,  -0.3722f,0.8935f,0.2512f,  -0.4518f,0.8281f,0.3319f,  -0.4738f,0.8221f,0.3157f,  -0.5407f,0.7426f,0.3953f,  -0.5571f,0.7362f,0.3842f,  -0.6093f,0.6446f,0.4617f,  -0.6197f,0.6388f,0.4559f,  -0.6558f,0.5374f,0.5302f,  -0.6598f,0.5333f,0.5295f,  -0.6789f,0.4243f,0.5993f,  
	-0.6763f,0.4227f,0.6032f,  -0.6780f,0.3086f,0.6671f,  -0.6692f,0.3104f,0.6751f,  -0.6538f,0.1934f,0.7315f,  -0.6391f,0.1991f,0.7429f,  -0.6072f,0.0815f,0.7904f,  -0.5872f,0.0914f,0.8043f,  -0.5401f,-0.0249f,0.8412f,  -0.5155f,-0.0106f,0.8568f,  -0.4550f,-0.1237f,0.8819f,  -0.4266f,-0.1051f,0.8983f,  -0.3547f,-0.2136f,0.9103f,  -0.3234f,-0.1911f,0.9267f,  -0.2425f,-0.2937f,0.9246f,  -0.2093f,-0.2677f,0.9405f,  
	-0.1220f,-0.3635f,0.9236f,  -0.0878f,-0.3346f,0.9383f,  0.0032f,-0.4228f,0.9062f,  0.0376f,-0.3918f,0.9193f,  0.1294f,-0.4720f,0.8721f,  0.1634f,-0.4396f,0.8832f,  0.2531f,-0.5114f,0.8212f,  0.2859f,-0.4784f,0.8303f,  0.3709f,-0.5420f,0.7541f,  0.4019f,-0.5091f,0.7611f,  0.4797f,-0.5644f,0.6718f,  0.5083f,-0.5325f,0.6768f,  0.5766f,-0.5797f,0.5758f,  0.6025f,-0.5495f,0.5789f,  0.6591f,-0.5888f,0.4678f,  
	0.6821f,-0.5609f,0.4692f,  0.7253f,-0.5928f,0.3501f,  0.7450f,-0.5678f,0.3501f,  0.7734f,-0.5926f,0.2250f,  0.7899f,-0.5709f,0.2240f,  0.8024f,-0.5891f,0.0952f,  0.8155f,-0.5711f,0.0936f,  0.8116f,-0.5830f,-0.0366f,  0.8213f,-0.5692f,-0.0384f,  0.8008f,-0.5751f,-0.1674f,  0.8072f,-0.5655f,-0.1691f,  0.7701f,-0.5658f,-0.2945f,  0.7734f,-0.5607f,-0.2956f,  0.7204f,-0.5556f,-0.4151f,  0.7208f,-0.5550f,-0.4152f,  
	0.2310f,-0.6948f,-0.6811f,  0.2484f,-0.6982f,-0.6714f,  0.3673f,-0.6438f,-0.6712f,  0.3837f,-0.6437f,-0.6621f,  0.4946f,-0.5753f,-0.6515f,  0.5093f,-0.5715f,-0.6434f,  0.6097f,-0.4902f,-0.6229f,  0.6220f,-0.4826f,-0.6165f,  0.7095f,-0.3900f,-0.5870f,  0.7189f,-0.3788f,-0.5828f,  0.7914f,-0.2767f,-0.5451f,  0.7974f,-0.2620f,-0.5437f,  0.8532f,-0.1527f,-0.4987f,  0.8551f,-0.1351f,-0.5005f,  0.8932f,-0.0209f,-0.4493f,  
	0.8906f,-0.0009f,-0.4548f,  0.9100f,0.1154f,-0.3982f,  0.9027f,0.1373f,-0.4077f,  0.9033f,0.2530f,-0.3466f,  0.8910f,0.2760f,-0.3604f,  0.8729f,0.3882f,-0.2954f,  0.8559f,0.4114f,-0.3135f,  0.8199f,0.5172f,-0.2454f,  0.7980f,0.5399f,-0.2676f,  0.7455f,0.6368f,-0.1969f,  0.7193f,0.6580f,-0.2228f,  0.6519f,0.7434f,-0.1499f,  0.6218f,0.7624f,-0.1789f,  0.5418f,0.8341f,-0.1040f,  0.5086f,0.8503f,-0.1355f,  
	0.4185f,0.9063f,-0.0587f,  0.3831f,0.9192f,-0.0917f,  0.2859f,0.9582f,-0.0131f,  0.2490f,0.9674f,-0.0465f,  0.1477f,0.9885f,0.0338f,  0.1108f,0.9938f,0.0009f,  0.0087f,0.9965f,0.0828f,  -0.0274f,0.9983f,0.0516f,  -0.1272f,0.9827f,0.1350f,  -0.1611f,0.9812f,0.1065f,  -0.2554f,0.9478f,0.1909f,  -0.2863f,0.9436f,0.1662f,  -0.3722f,0.8935f,0.2512f,  -0.3988f,0.8875f,0.2307f,  -0.4738f,0.8221f,0.3157f,  
	-0.4954f,0.8152f,0.3001f,  -0.5571f,0.7362f,0.3842f,  -0.5730f,0.7293f,0.3738f,  -0.6197f,0.6388f,0.4559f,  -0.6294f,0.6331f,0.4506f,  -0.6598f,0.5333f,0.5295f,  -0.6631f,0.5296f,0.5290f,  -0.6763f,0.4227f,0.6032f,  -0.6733f,0.4220f,0.6071f,  -0.6692f,0.3104f,0.6751f,  -0.6600f,0.3133f,0.6828f,  -0.6391f,0.1991f,0.7429f,  -0.6242f,0.2061f,0.7536f,  -0.5872f,0.0914f,0.8043f,  -0.5672f,0.1027f,0.8171f,  
	-0.5155f,-0.0106f,0.8568f,  -0.4912f,0.0051f,0.8710f,  -0.4266f,-0.1051f,0.8983f,  -0.3988f,-0.0852f,0.9131f,  -0.3234f,-0.1911f,0.9267f,  -0.2931f,-0.1674f,0.9413f,  -0.2093f,-0.2677f,0.9405f,  -0.1772f,-0.2407f,0.9543f,  -0.0878f,-0.3346f,0.9383f,  -0.0547f,-0.3049f,0.9508f,  0.0376f,-0.3918f,0.9193f,  0.0708f,-0.3602f,0.9302f,  0.1634f,-0.4396f,0.8832f,  0.1959f,-0.4067f,0.8923f,  0.2859f,-0.4784f,0.8303f,  
	0.3172f,-0.4452f,0.8374f,  0.4019f,-0.5091f,0.7611f,  0.4314f,-0.4763f,0.7662f,  0.5083f,-0.5325f,0.6768f,  0.5356f,-0.5007f,0.6800f,  0.6025f,-0.5495f,0.5789f,  0.6272f,-0.5194f,0.5804f,  0.6821f,-0.5609f,0.4692f,  0.7039f,-0.5332f,0.4693f,  0.7450f,-0.5678f,0.3501f,  0.7638f,-0.5430f,0.3490f,  0.7899f,-0.5709f,0.2240f,  0.8054f,-0.5495f,0.2221f,  0.8155f,-0.5711f,0.0936f,  0.8279f,-0.5535f,0.0912f,  
	0.8213f,-0.5692f,-0.0384f,  0.8305f,-0.5556f,-0.0408f,  0.8072f,-0.5655f,-0.1691f,  0.8132f,-0.5562f,-0.1711f,  0.7734f,-0.5607f,-0.2956f,  0.7764f,-0.5559f,-0.2969f,  0.7208f,-0.5550f,-0.4152f,  0.7209f,-0.5548f,-0.4153f,  0.2484f,-0.6982f,-0.6714f,  0.2662f,-0.7012f,-0.6614f,  0.3837f,-0.6437f,-0.6621f,  0.4003f,-0.6430f,-0.6529f,  0.5093f,-0.5715f,-0.6434f,  0.5240f,-0.5670f,-0.6355f,  0.6220f,-0.4826f,-0.6165f,  
	0.6341f,-0.4744f,-0.6106f,  0.7189f,-0.3788f,-0.5828f,  0.7278f,-0.3669f,-0.5793f,  0.7974f,-0.2620f,-0.5437f,  0.8024f,-0.2469f,-0.5432f,  0.8551f,-0.1351f,-0.5005f,  0.8559f,-0.1172f,-0.5037f,  0.8906f,-0.0009f,-0.4548f,  0.8867f,0.0193f,-0.4619f,  0.9027f,0.1373f,-0.4077f,  0.8939f,0.1591f,-0.4192f,  0.8910f,0.2760f,-0.3604f,  0.8771f,0.2985f,-0.3762f,  0.8559f,0.4114f,-0.3135f,  0.8369f,0.4338f,-0.3337f,  
	0.7980f,0.5399f,-0.2676f,  0.7743f,0.5614f,-0.2919f,  0.7193f,0.6580f,-0.2228f,  0.6912f,0.6778f,-0.2508f,  0.6218f,0.7624f,-0.1789f,  0.5900f,0.7796f,-0.2100f,  0.5086f,0.8503f,-0.1355f,  0.4737f,0.8644f,-0.1687f,  0.3831f,0.9192f,-0.0917f,  0.3460f,0.9297f,-0.1260f,  0.2490f,0.9674f,-0.0465f,  0.2110f,0.9741f,-0.0810f,  0.1108f,0.9938f,0.0009f,  0.0728f,0.9968f,-0.0326f,  -0.0274f,0.9983f,0.0516f,  
	-0.0641f,0.9977f,0.0203f,  -0.1611f,0.9812f,0.1065f,  -0.1954f,0.9776f,0.0782f,  -0.2863f,0.9436f,0.1662f,  -0.3171f,0.9378f,0.1417f,  -0.3988f,0.8875f,0.2307f,  -0.4252f,0.8802f,0.2108f,  -0.4954f,0.8152f,0.3001f,  -0.5165f,0.8074f,0.2852f,  -0.5730f,0.7293f,0.3738f,  -0.5883f,0.7221f,0.3639f,  -0.6294f,0.6331f,0.4506f,  -0.6385f,0.6274f,0.4457f,  -0.6631f,0.5296f,0.5290f,  -0.6658f,0.5264f,0.5287f,  
	-0.6733f,0.4220f,0.6071f,  -0.6697f,0.4221f,0.6110f,  -0.6600f,0.3133f,0.6828f,  -0.6505f,0.3172f,0.6900f,  -0.6242f,0.2061f,0.7536f,  -0.6092f,0.2143f,0.7635f,  -0.5672f,0.1027f,0.8171f,  -0.5475f,0.1154f,0.8288f,  -0.4912f,0.0051f,0.8710f,  -0.4674f,0.0221f,0.8838f,  -0.3988f,-0.0852f,0.9131f,  -0.3718f,-0.0642f,0.9261f,  -0.2931f,-0.1674f,0.9413f,  -0.2636f,-0.1427f,0.9540f,  -0.1772f,-0.2407f,0.9543f,  
	-0.1462f,-0.2128f,0.9661f,  -0.0547f,-0.3049f,0.9508f,  -0.0230f,-0.2746f,0.9613f,  0.0708f,-0.3602f,0.9302f,  0.1026f,-0.3281f,0.9391f,  0.1959f,-0.4067f,0.8923f,  0.2271f,-0.3737f,0.8993f,  0.3172f,-0.4452f,0.8374f,  0.3471f,-0.4119f,0.8425f,  0.4314f,-0.4763f,0.7662f,  0.4596f,-0.4434f,0.7695f,  0.5356f,-0.5007f,0.6800f,  0.5617f,-0.4690f,0.6816f,  0.6272f,-0.5194f,0.5804f,  0.6508f,-0.4895f,0.5804f,  
	0.7039f,-0.5332f,0.4693f,  0.7247f,-0.5057f,0.4681f,  0.7638f,-0.5430f,0.3490f,  0.7816f,-0.5184f,0.3468f,  0.8054f,-0.5495f,0.2221f,  0.8202f,-0.5283f,0.2193f,  0.8279f,-0.5535f,0.0912f,  0.8395f,-0.5361f,0.0882f,  0.8305f,-0.5556f,-0.0408f,  0.8391f,-0.5423f,-0.0437f,  0.8132f,-0.5562f,-0.1711f,  0.8188f,-0.5473f,-0.1734f,  0.7764f,-0.5559f,-0.2969f,  0.7791f,-0.5514f,-0.2982f,  0.7209f,-0.5548f,-0.4153f,  
	0.7209f,-0.5548f,-0.4153f,  0.2662f,-0.7012f,-0.6614f,  0.2844f,-0.7036f,-0.6512f,  0.4003f,-0.6430f,-0.6529f,  0.4170f,-0.6417f,-0.6437f,  0.5240f,-0.5670f,-0.6355f,  0.5385f,-0.5620f,-0.6279f,  0.6341f,-0.4744f,-0.6106f,  0.6459f,-0.4656f,-0.6051f,  0.7278f,-0.3669f,-0.5793f,  0.7360f,-0.3546f,-0.5767f,  0.8024f,-0.2469f,-0.5432f,  0.8066f,-0.2315f,-0.5439f,  0.8559f,-0.1172f,-0.5037f,  0.8555f,-0.0990f,-0.5083f,  
	0.8867f,0.0193f,-0.4619f,  0.8814f,0.0395f,-0.4708f,  0.8939f,0.1591f,-0.4192f,  0.8834f,0.1805f,-0.4325f,  0.8771f,0.2985f,-0.3762f,  0.8614f,0.3203f,-0.3941f,  0.8369f,0.4338f,-0.3337f,  0.8161f,0.4552f,-0.3560f,  0.7743f,0.5614f,-0.2919f,  0.7487f,0.5815f,-0.3184f,  0.6912f,0.6778f,-0.2508f,  0.6612f,0.6957f,-0.2808f,  0.5900f,0.7796f,-0.2100f,  0.5563f,0.7947f,-0.2427f,  0.4737f,0.8644f,-0.1687f,  
	0.4372f,0.8761f,-0.2033f,  0.3460f,0.9297f,-0.1260f,  0.3077f,0.9377f,-0.1615f,  0.2110f,0.9741f,-0.0810f,  0.1719f,0.9782f,-0.1162f,  0.0728f,0.9968f,-0.0326f,  0.0341f,0.9972f,-0.0664f,  -0.0641f,0.9977f,0.0203f,  -0.1013f,0.9948f,-0.0111f,  -0.1954f,0.9776f,0.0782f,  -0.2299f,0.9719f,0.0503f,  -0.3171f,0.9378f,0.1417f,  -0.3478f,0.9301f,0.1179f,  -0.4252f,0.8802f,0.2108f,  -0.4513f,0.8716f,0.1917f,  
	-0.5165f,0.8074f,0.2852f,  -0.5372f,0.7987f,0.2710f,  -0.5883f,0.7221f,0.3639f,  -0.6030f,0.7145f,0.3547f,  -0.6385f,0.6274f,0.4457f,  -0.6470f,0.6219f,0.4412f,  -0.6658f,0.5264f,0.5287f,  -0.6680f,0.5237f,0.5286f,  -0.6697f,0.4221f,0.6110f,  -0.6658f,0.4230f,0.6146f,  -0.6505f,0.3172f,0.6900f,  -0.6409f,0.3222f,0.6968f,  -0.6092f,0.2143f,0.7635f,  -0.5944f,0.2236f,0.7725f,  -0.5475f,0.1154f,0.8288f,  
	-0.5280f,0.1291f,0.8394f,  -0.4674f,0.0221f,0.8838f,  -0.4442f,0.0401f,0.8950f,  -0.3718f,-0.0642f,0.9261f,  -0.3455f,-0.0422f,0.9375f,  -0.2636f,-0.1427f,0.9540f,  -0.2352f,-0.1171f,0.9649f,  -0.1462f,-0.2128f,0.9661f,  -0.1163f,-0.1844f,0.9759f,  -0.0230f,-0.2746f,0.9613f,  0.0076f,-0.2438f,0.9698f,  0.1026f,-0.3281f,0.9391f,  0.1331f,-0.2957f,0.9460f,  0.2271f,-0.3737f,0.8993f,  0.2569f,-0.3404f,0.9045f,  
	0.3471f,-0.4119f,0.8425f,  0.3757f,-0.3785f,0.8459f,  0.4596f,-0.4434f,0.7695f,  0.4865f,-0.4106f,0.7712f,  0.5617f,-0.4690f,0.6816f,  0.5865f,-0.4375f,0.6816f,  0.6508f,-0.4895f,0.5804f,  0.6732f,-0.4599f,0.5791f,  0.7247f,-0.5057f,0.4681f,  0.7445f,-0.4785f,0.4656f,  0.7816f,-0.5184f,0.3468f,  0.7986f,-0.4942f,0.3436f,  0.8202f,-0.5283f,0.2193f,  0.8342f,-0.5075f,0.2156f,  0.8395f,-0.5361f,0.0882f,  
	0.8506f,-0.5190f,0.0845f,  0.8391f,-0.5423f,-0.0437f,  0.8471f,-0.5293f,-0.0470f,  0.8188f,-0.5473f,-0.1734f,  0.8240f,-0.5386f,-0.1760f,  0.7791f,-0.5514f,-0.2982f,  0.7816f,-0.5472f,-0.2996f,  0.7209f,-0.5548f,-0.4153f,  0.7208f,-0.5551f,-0.4152f,  0.2844f,-0.7036f,-0.6512f,  0.3028f,-0.7055f,-0.6408f,  0.4170f,-0.6417f,-0.6437f,  0.4337f,-0.6399f,-0.6344f,  0.5385f,-0.5620f,-0.6279f,  0.5528f,-0.5562f,-0.6205f,  
	0.6459f,-0.4656f,-0.6051f,  0.6570f,-0.4561f,-0.6002f,  0.7360f,-0.3546f,-0.5767f,  0.7434f,-0.3418f,-0.5749f,  0.8066f,-0.2315f,-0.5439f,  0.8096f,-0.2156f,-0.5459f,  0.8555f,-0.0990f,-0.5083f,  0.8538f,-0.0807f,-0.5143f,  0.8814f,0.0395f,-0.4708f,  0.8745f,0.0594f,-0.4814f,  0.8834f,0.1805f,-0.4325f,  0.8712f,0.2013f,-0.4478f,  0.8614f,0.3203f,-0.3941f,  0.8439f,0.3412f,-0.4141f,  0.8161f,0.4552f,-0.3560f,  
	0.7934f,0.4752f,-0.3805f,  0.7487f,0.5815f,-0.3184f,  0.7211f,0.5997f,-0.3468f,  0.6612f,0.6957f,-0.2808f,  0.6294f,0.7114f,-0.3126f,  0.5563f,0.7947f,-0.2427f,  0.5208f,0.8074f,-0.2771f,  0.4372f,0.8761f,-0.2033f,  0.3991f,0.8851f,-0.2393f,  0.3077f,0.9377f,-0.1615f,  0.2680f,0.9429f,-0.1979f,  0.1719f,0.9782f,-0.1162f,  0.1317f,0.9796f,-0.1519f,  0.0341f,0.9972f,-0.0664f,  -0.0053f,0.9950f,-0.1002f,  
	-0.1013f,0.9948f,-0.0111f,  -0.1388f,0.9894f,-0.0421f,  -0.2299f,0.9719f,0.0503f,  -0.2644f,0.9641f,0.0229f,  -0.3478f,0.9301f,0.1179f,  -0.3783f,0.9208f,0.0949f,  -0.4513f,0.8716f,0.1917f,  -0.4769f,0.8617f,0.1734f,  -0.5372f,0.7987f,0.2710f,  -0.5572f,0.7894f,0.2576f,  -0.6030f,0.7145f,0.3547f,  -0.6171f,0.7067f,0.3462f,  -0.6470f,0.6219f,0.4412f,  -0.6549f,0.6164f,0.4373f,  -0.6680f,0.5237f,0.5286f,  
	-0.6697f,0.5215f,0.5287f,  -0.6658f,0.4230f,0.6146f,  -0.6616f,0.4246f,0.6181f,  -0.6409f,0.3222f,0.6968f,  -0.6311f,0.3280f,0.7029f,  -0.5944f,0.2236f,0.7725f,  -0.5796f,0.2339f,0.7806f,  -0.5280f,0.1291f,0.8394f,  -0.5090f,0.1438f,0.8487f,  -0.4442f,0.0401f,0.8950f,  -0.4216f,0.0591f,0.9049f,  -0.3455f,-0.0422f,0.9375f,  -0.3201f,-0.0194f,0.9472f,  -0.2352f,-0.1171f,0.9649f,  -0.2077f,-0.0910f,0.9739f,  
	-0.1163f,-0.1844f,0.9759f,  -0.0876f,-0.1554f,0.9840f,  0.0076f,-0.2438f,0.9698f,  0.0368f,-0.2127f,0.9764f,  0.1331f,-0.2957f,0.9460f,  0.1623f,-0.2632f,0.9510f,  0.2569f,-0.3404f,0.9045f,  0.2854f,-0.3071f,0.9078f,  0.3757f,-0.3785f,0.8459f,  0.4031f,-0.3452f,0.8476f,  0.4865f,-0.4106f,0.7712f,  0.5122f,-0.3780f,0.7712f,  0.5865f,-0.4375f,0.6816f,  0.6102f,-0.4061f,0.6802f,  0.6732f,-0.4599f,0.5791f,  
	0.6946f,-0.4304f,0.5765f,  0.7445f,-0.4785f,0.4656f,  0.7633f,-0.4516f,0.4621f,  0.7986f,-0.4942f,0.3436f,  0.8147f,-0.4702f,0.3394f,  0.8342f,-0.5075f,0.2156f,  0.8475f,-0.4869f,0.2112f,  0.8506f,-0.5190f,0.0845f,  0.8610f,-0.5022f,0.0801f,  0.8471f,-0.5293f,-0.0470f,  0.8547f,-0.5166f,-0.0508f,  0.8240f,-0.5386f,-0.1760f,  0.8288f,-0.5302f,-0.1789f,  0.7816f,-0.5472f,-0.2996f,  0.7837f,-0.5432f,-0.3011f,  
	0.7208f,-0.5551f,-0.4152f,  0.7205f,-0.5557f,-0.4149f,  0.3028f,-0.7055f,-0.6408f,  0.3214f,-0.7068f,-0.6302f,  0.4337f,-0.6399f,-0.6344f,  0.4504f,-0.6373f,-0.6253f,  0.5528f,-0.5562f,-0.6205f,  0.5668f,-0.5498f,-0.6135f,  0.6570f,-0.4561f,-0.6002f,  0.6676f,-0.4462f,-0.5960f,  0.7434f,-0.3418f,-0.5749f,  0.7499f,-0.3285f,-0.5741f,  0.8096f,-0.2156f,-0.5459f,  0.8116f,-0.1996f,-0.5491f,  0.8538f,-0.0807f,-0.5143f,  
	0.8506f,-0.0626f,-0.5220f,  0.8745f,0.0594f,-0.4814f,  0.8660f,0.0789f,-0.4938f,  0.8712f,0.2013f,-0.4478f,  0.8572f,0.2213f,-0.4650f,  0.8439f,0.3412f,-0.4141f,  0.8244f,0.3608f,-0.4360f,  0.7934f,0.4752f,-0.3805f,  0.7687f,0.4935f,-0.4069f,  0.7211f,0.5997f,-0.3468f,  0.6916f,0.6160f,-0.3771f,  0.6294f,0.7114f,-0.3126f,  0.5956f,0.7249f,-0.3461f,  0.5208f,0.8074f,-0.2771f,  0.4837f,0.8174f,-0.3128f,  
	0.3991f,0.8851f,-0.2393f,  0.3595f,0.8914f,-0.2761f,  0.2680f,0.9429f,-0.1979f,  0.2271f,0.9452f,-0.2347f,  0.1317f,0.9796f,-0.1519f,  0.0907f,0.9781f,-0.1876f,  -0.0053f,0.9950f,-0.1002f,  -0.0453f,0.9900f,-0.1338f,  -0.1388f,0.9894f,-0.0421f,  -0.1765f,0.9816f,-0.0725f,  -0.2644f,0.9641f,0.0229f,  -0.2988f,0.9543f,-0.0036f,  -0.3783f,0.9208f,0.0949f,  -0.4084f,0.9099f,0.0728f,  -0.4769f,0.8617f,0.1734f,  
	-0.5019f,0.8507f,0.1561f,  -0.5572f,0.7894f,0.2576f,  -0.5767f,0.7793f,0.2451f,  -0.6171f,0.7067f,0.3462f,  -0.6305f,0.6985f,0.3383f,  -0.6549f,0.6164f,0.4373f,  -0.6622f,0.6111f,0.4337f,  -0.6697f,0.5215f,0.5287f,  -0.6710f,0.5198f,0.5288f,  -0.6616f,0.4246f,0.6181f,  -0.6571f,0.4269f,0.6213f,  -0.6311f,0.3280f,0.7029f,  -0.6213f,0.3347f,0.7085f,  -0.5796f,0.2339f,0.7806f,  -0.5651f,0.2450f,0.7878f,  
	-0.5090f,0.1438f,0.8487f,  -0.4904f,0.1593f,0.8568f,  -0.4216f,0.0591f,0.9049f,  -0.3996f,0.0787f,0.9133f,  -0.3201f,-0.0194f,0.9472f,  -0.2956f,0.0040f,0.9553f,  -0.2077f,-0.0910f,0.9739f,  -0.1813f,-0.0643f,0.9813f,  -0.0876f,-0.1554f,0.9840f,  -0.0600f,-0.1261f,0.9902f,  0.0368f,-0.2127f,0.9764f,  0.0649f,-0.1814f,0.9813f,  0.1623f,-0.2632f,0.9510f,  0.1903f,-0.2305f,0.9543f,  0.2854f,-0.3071f,0.9078f,  
	0.3127f,-0.2739f,0.9095f,  0.4031f,-0.3452f,0.8476f,  0.4292f,-0.3120f,0.8476f,  0.5122f,-0.3780f,0.7712f,  0.5368f,-0.3455f,0.7698f,  0.6102f,-0.4061f,0.6802f,  0.6328f,-0.3750f,0.6774f,  0.6946f,-0.4304f,0.5765f,  0.7149f,-0.4013f,0.5726f,  0.7633f,-0.4516f,0.4621f,  0.7812f,-0.4249f,0.4574f,  0.8147f,-0.4702f,0.3394f,  0.8300f,-0.4465f,0.3343f,  0.8475f,-0.4869f,0.2112f,  0.8601f,-0.4666f,0.2059f,  
	0.8610f,-0.5022f,0.0801f,  0.8709f,-0.4857f,0.0752f,  0.8547f,-0.5166f,-0.0508f,  0.8619f,-0.5041f,-0.0551f,  0.8288f,-0.5302f,-0.1789f,  0.8333f,-0.5220f,-0.1820f,  0.7837f,-0.5432f,-0.3011f,  0.7857f,-0.5395f,-0.3027f,  0.7205f,-0.5557f,-0.4149f,  0.7200f,-0.5566f,-0.4145f,  0.3214f,-0.7068f,-0.6302f,  0.3402f,-0.7075f,-0.6194f,  0.4504f,-0.6373f,-0.6253f,  0.4669f,-0.6342f,-0.6163f,  0.5668f,-0.5498f,-0.6135f,  
	0.5803f,-0.5430f,-0.6070f,  0.6676f,-0.4462f,-0.5960f,  0.6774f,-0.4358f,-0.5926f,  0.7499f,-0.3285f,-0.5741f,  0.7555f,-0.3150f,-0.5745f,  0.8116f,-0.1996f,-0.5491f,  0.8122f,-0.1836f,-0.5537f,  0.8506f,-0.0626f,-0.5220f,  0.8460f,-0.0448f,-0.5313f,  0.8660f,0.0789f,-0.4938f,  0.8558f,0.0977f,-0.5080f,  0.8572f,0.2213f,-0.4650f,  0.8413f,0.2403f,-0.4842f,  0.8244f,0.3608f,-0.4360f,  0.8030f,0.3789f,-0.4600f,  
	0.7687f,0.4935f,-0.4069f,  0.7420f,0.5100f,-0.4352f,  0.6916f,0.6160f,-0.3771f,  0.6601f,0.6300f,-0.4091f,  0.5956f,0.7249f,-0.3461f,  0.5601f,0.7357f,-0.3810f,  0.4837f,0.8174f,-0.3128f,  0.4450f,0.8245f,-0.3495f,  0.3595f,0.8914f,-0.2761f,  0.3187f,0.8945f,-0.3135f,  0.2271f,0.9452f,-0.2347f,  0.1852f,0.9444f,-0.2718f,  0.0907f,0.9781f,-0.1876f,  0.0490f,0.9736f,-0.2231f,  -0.0453f,0.9900f,-0.1338f,  
	-0.0856f,0.9823f,-0.1667f,  -0.1765f,0.9816f,-0.0725f,  -0.2142f,0.9714f,-0.1021f,  -0.2988f,0.9543f,-0.0036f,  -0.3330f,0.9425f,-0.0291f,  -0.4084f,0.9099f,0.0728f,  -0.4381f,0.8975f,0.0519f,  -0.5019f,0.8507f,0.1561f,  -0.5264f,0.8387f,0.1399f,  -0.5767f,0.7793f,0.2451f,  -0.5954f,0.7687f,0.2336f,  -0.6305f,0.6985f,0.3383f,  -0.6433f,0.6903f,0.3312f,  -0.6622f,0.6111f,0.4337f,  -0.6689f,0.6060f,0.4305f,  
	-0.6710f,0.5198f,0.5288f,  -0.6718f,0.5184f,0.5290f,  -0.6571f,0.4269f,0.6213f,  -0.6524f,0.4297f,0.6242f,  -0.6213f,0.3347f,0.7085f,  -0.6116f,0.3420f,0.7135f,  -0.5651f,0.2450f,0.7878f,  -0.5509f,0.2568f,0.7941f,  -0.4904f,0.1593f,0.8568f,  -0.4723f,0.1754f,0.8638f,  -0.3996f,0.0787f,0.9133f,  -0.3784f,0.0989f,0.9203f,  -0.2956f,0.0040f,0.9553f,  -0.2719f,0.0279f,0.9619f,  -0.1813f,-0.0643f,0.9813f,  
	-0.1559f,-0.0373f,0.9871f,  -0.0600f,-0.1261f,0.9902f,  -0.0336f,-0.0966f,0.9948f,  0.0649f,-0.1814f,0.9813f,  0.0919f,-0.1500f,0.9844f,  0.1903f,-0.2305f,0.9543f,  0.2171f,-0.1979f,0.9559f,  0.3127f,-0.2739f,0.9095f,  0.3389f,-0.2407f,0.9095f,  0.4292f,-0.3120f,0.8476f,  0.4542f,-0.2789f,0.8461f,  0.5368f,-0.3455f,0.7698f,  0.5602f,-0.3132f,0.7669f,  0.6328f,-0.3750f,0.6774f,  0.6544f,-0.3441f,0.6733f,  
	0.7149f,-0.4013f,0.5726f,  0.7344f,-0.3723f,0.5675f,  0.7812f,-0.4249f,0.4574f,  0.7983f,-0.3984f,0.4516f,  0.8300f,-0.4465f,0.3343f,  0.8446f,-0.4230f,0.3282f,  0.8601f,-0.4666f,0.2059f,  0.8721f,-0.4466f,0.1999f,  0.8709f,-0.4857f,0.0752f,  0.8802f,-0.4695f,0.0697f,  0.8619f,-0.5041f,-0.0551f,  0.8686f,-0.4919f,-0.0597f,  0.8333f,-0.5220f,-0.1820f,  0.8374f,-0.5142f,-0.1853f,  0.7857f,-0.5395f,-0.3027f,  
	0.7874f,-0.5362f,-0.3042f,  0.7200f,-0.5566f,-0.4145f,  0.7195f,-0.5578f,-0.4138f,  0.3402f,-0.7075f,-0.6194f,  0.3590f,-0.7076f,-0.6086f,  0.4669f,-0.6342f,-0.6163f,  0.4831f,-0.6305f,-0.6075f,  0.5803f,-0.5430f,-0.6070f,  0.5933f,-0.5356f,-0.6009f,  0.6774f,-0.4358f,-0.5926f,  0.6864f,-0.4250f,-0.5901f,  0.7555f,-0.3150f,-0.5745f,  0.7599f,-0.3014f,-0.5760f,  0.8122f,-0.1836f,-0.5537f,  0.8115f,-0.1678f,-0.5598f,  
	0.8460f,-0.0448f,-0.5313f,  0.8397f,-0.0275f,-0.5423f,  0.8558f,0.0977f,-0.5080f,  0.8438f,0.1156f,-0.5240f,  0.8413f,0.2403f,-0.4842f,  0.8236f,0.2578f,-0.5052f,  0.8030f,0.3789f,-0.4600f,  0.7796f,0.3952f,-0.4857f,  0.7420f,0.5100f,-0.4352f,  0.7133f,0.5242f,-0.4651f,  0.6601f,0.6300f,-0.4091f,  0.6268f,0.6413f,-0.4426f,  0.5601f,0.7357f,-0.3810f,  0.5228f,0.7435f,-0.4169f,  0.4450f,0.8245f,-0.3495f,  
	0.4048f,0.8285f,-0.3869f,  0.3187f,0.8945f,-0.3135f,  0.2766f,0.8945f,-0.3512f,  0.1852f,0.9444f,-0.2718f,  0.1425f,0.9404f,-0.3087f,  0.0490f,0.9736f,-0.2231f,  0.0068f,0.9661f,-0.2581f,  -0.0856f,0.9823f,-0.1667f,  -0.1260f,0.9719f,-0.1988f,  -0.2142f,0.9714f,-0.1021f,  -0.2518f,0.9589f,-0.1305f,  -0.3330f,0.9425f,-0.0291f,  -0.3667f,0.9288f,-0.0533f,  -0.4381f,0.8975f,0.0519f,  -0.4671f,0.8836f,0.0322f,  
	-0.5264f,0.8387f,0.1399f,  -0.5501f,0.8257f,0.1249f,  -0.5954f,0.7687f,0.2336f,  -0.6134f,0.7576f,0.2230f,  -0.6433f,0.6903f,0.3312f,  -0.6554f,0.6819f,0.3248f,  -0.6689f,0.6060f,0.4305f,  -0.6751f,0.6011f,0.4277f,  -0.6718f,0.5184f,0.5290f,  -0.6724f,0.5175f,0.5293f,  -0.6524f,0.4297f,0.6242f,  -0.6476f,0.4331f,0.6269f,  -0.6116f,0.3420f,0.7135f,  -0.6019f,0.3499f,0.7178f,  -0.5509f,0.2568f,0.7941f,  
	-0.5369f,0.2692f,0.7995f,  -0.4723f,0.1754f,0.8638f,  -0.4548f,0.1921f,0.8696f,  -0.3784f,0.0989f,0.9203f,  -0.3579f,0.1196f,0.9261f,  -0.2719f,0.0279f,0.9619f,  -0.2491f,0.0521f,0.9671f,  -0.1559f,-0.0373f,0.9871f,  -0.1315f,-0.0101f,0.9913f,  -0.0336f,-0.0966f,0.9948f,  -0.0081f,-0.0669f,0.9977f,  0.0919f,-0.1500f,0.9844f,  0.1177f,-0.1185f,0.9860f,  0.2171f,-0.1979f,0.9559f,  0.2427f,-0.1653f,0.9559f,  
	0.3389f,-0.2407f,0.9095f,  0.3639f,-0.2076f,0.9080f,  0.4542f,-0.2789f,0.8461f,  0.4781f,-0.2460f,0.8432f,  0.5602f,-0.3132f,0.7669f,  0.5827f,-0.2810f,0.7626f,  0.6544f,-0.3441f,0.6733f,  0.6750f,-0.3134f,0.6679f,  0.7344f,-0.3723f,0.5675f,  0.7529f,-0.3436f,0.5613f,  0.7983f,-0.3984f,0.4516f,  0.8146f,-0.3722f,0.4449f,  0.8446f,-0.4230f,0.3282f,  0.8585f,-0.3998f,0.3213f,  0.8721f,-0.4466f,0.1999f,  
	0.8835f,-0.4268f,0.1932f,  0.8802f,-0.4695f,0.0697f,  0.8890f,-0.4534f,0.0636f,  0.8686f,-0.4919f,-0.0597f,  0.8749f,-0.4800f,-0.0647f,  0.8374f,-0.5142f,-0.1853f,  0.8413f,-0.5065f,-0.1888f,  0.7874f,-0.5362f,-0.3042f,  0.7889f,-0.5330f,-0.3058f,  0.7195f,-0.5578f,-0.4138f,  0.7188f,-0.5592f,-0.4130f,  0.3590f,-0.7076f,-0.6086f,  0.3777f,-0.7072f,-0.5977f,  0.4831f,-0.6305f,-0.6075f,  0.4990f,-0.6262f,-0.5990f,  
	0.5933f,-0.5356f,-0.6009f,  0.6057f,-0.5277f,-0.5955f,  0.6864f,-0.4250f,-0.5901f,  0.6945f,-0.4140f,-0.5884f,  0.7599f,-0.3014f,-0.5760f,  0.7631f,-0.2878f,-0.5787f,  0.8115f,-0.1678f,-0.5598f,  0.8093f,-0.1523f,-0.5673f,  0.8397f,-0.0275f,-0.5423f,  0.8318f,-0.0110f,-0.5549f,  0.8438f,0.1156f,-0.5240f,  0.8300f,0.1323f,-0.5418f,  0.8236f,0.2578f,-0.5052f,  0.8039f,0.2737f,-0.5280f,  0.7796f,0.3952f,-0.4857f,  
	0.7543f,0.4095f,-0.5132f,  0.7133f,0.5242f,-0.4651f,  0.6828f,0.5359f,-0.4966f,  0.6268f,0.6413f,-0.4426f,  0.5917f,0.6498f,-0.4771f,  0.5228f,0.7435f,-0.4169f,  0.4840f,0.7483f,-0.4536f,  0.4048f,0.8285f,-0.3869f,  0.3632f,0.8293f,-0.4247f,  0.2766f,0.8945f,-0.3512f,  0.2336f,0.8912f,-0.3889f,  0.1425f,0.9404f,-0.3087f,  0.0991f,0.9334f,-0.3450f,  0.0068f,0.9661f,-0.2581f,  -0.0356f,0.9557f,-0.2921f,  
	-0.1260f,0.9719f,-0.1988f,  -0.1664f,0.9589f,-0.2297f,  -0.2518f,0.9589f,-0.1305f,  -0.2891f,0.9443f,-0.1576f,  -0.3667f,0.9288f,-0.0533f,  -0.3999f,0.9134f,-0.0761f,  -0.4671f,0.8836f,0.0322f,  -0.4954f,0.8685f,0.0139f,  -0.5501f,0.8257f,0.1249f,  -0.5731f,0.8119f,0.1110f,  -0.6134f,0.7576f,0.2230f,  -0.6307f,0.7461f,0.2135f,  -0.6554f,0.6819f,0.3248f,  -0.6669f,0.6734f,0.3190f,  -0.6751f,0.6011f,0.4277f,  
	-0.6808f,0.5963f,0.4253f,  -0.6724f,0.5175f,0.5293f,  -0.6726f,0.5169f,0.5296f,  -0.6476f,0.4331f,0.6269f,  -0.6427f,0.4370f,0.6292f,  -0.6019f,0.3499f,0.7178f,  -0.5924f,0.3583f,0.7216f,  -0.5369f,0.2692f,0.7995f,  -0.5234f,0.2821f,0.8041f,  -0.4548f,0.1921f,0.8696f,  -0.4377f,0.2093f,0.8744f,  -0.3579f,0.1196f,0.9261f,  -0.3381f,0.1407f,0.9306f,  -0.2491f,0.0521f,0.9671f,  -0.2271f,0.0766f,0.9708f,  
	-0.1315f,-0.0101f,0.9913f,  -0.1080f,0.0174f,0.9940f,  -0.0081f,-0.0669f,0.9977f,  0.0163f,-0.0371f,0.9992f,  0.1177f,-0.1185f,0.9860f,  0.1424f,-0.0870f,0.9860f,  0.2427f,-0.1653f,0.9559f,  0.2673f,-0.1327f,0.9544f,  0.3639f,-0.2076f,0.9080f,  0.3878f,-0.1746f,0.9051f,  0.4781f,-0.2460f,0.8432f,  0.5010f,-0.2132f,0.8388f,  0.5827f,-0.2810f,0.7626f,  0.6041f,-0.2491f,0.7570f,  0.6750f,-0.3134f,0.6679f,  
	0.6947f,-0.2828f,0.6613f,  0.7529f,-0.3436f,0.5613f,  0.7707f,-0.3150f,0.5539f,  0.8146f,-0.3722f,0.4449f,  0.8301f,-0.3462f,0.4371f,  0.8585f,-0.3998f,0.3213f,  0.8717f,-0.3768f,0.3134f,  0.8835f,-0.4268f,0.1932f,  0.8943f,-0.4072f,0.1858f,  0.8890f,-0.4534f,0.0636f,  0.8974f,-0.4376f,0.0569f,  0.8749f,-0.4800f,-0.0647f,  0.8808f,-0.4683f,-0.0701f,  0.8413f,-0.5065f,-0.1888f,  0.8448f,-0.4992f,-0.1925f,  
	0.7889f,-0.5330f,-0.3058f,  0.7902f,-0.5302f,-0.3073f,  0.7188f,-0.5592f,-0.4130f,  0.7181f,-0.5610f,-0.4119f,  0.3777f,-0.7072f,-0.5977f,  0.3962f,-0.7061f,-0.5868f,  0.4990f,-0.6262f,-0.5990f,  0.5145f,-0.6214f,-0.5908f,  0.6057f,-0.5277f,-0.5955f,  0.6173f,-0.5196f,-0.5907f,  0.6945f,-0.4140f,-0.5884f,  0.7016f,-0.4029f,-0.5877f,  0.7631f,-0.2878f,-0.5787f,  0.7650f,-0.2744f,-0.5827f,  0.8093f,-0.1523f,-0.5673f,  
	0.8056f,-0.1374f,-0.5763f,  0.8318f,-0.0110f,-0.5549f,  0.8222f,0.0046f,-0.5692f,  0.8300f,0.1323f,-0.5418f,  0.8144f,0.1475f,-0.5613f,  0.8039f,0.2737f,-0.5280f,  0.7823f,0.2878f,-0.5525f,  0.7543f,0.4095f,-0.5132f,  0.7270f,0.4214f,-0.5421f,  0.6828f,0.5359f,-0.4966f,  0.6504f,0.5449f,-0.5292f,  0.5917f,0.6498f,-0.4771f,  0.5549f,0.6553f,-0.5125f,  0.4840f,0.7483f,-0.4536f,  0.4437f,0.7498f,-0.4907f,  
	0.3632f,0.8293f,-0.4247f,  0.3207f,0.8267f,-0.4624f,  0.2336f,0.8912f,-0.3889f,  0.1898f,0.8846f,-0.4260f,  0.0991f,0.9334f,-0.3450f,  0.0555f,0.9231f,-0.3804f,  -0.0356f,0.9557f,-0.2921f,  -0.0781f,0.9425f,-0.3250f,  -0.1664f,0.9589f,-0.2297f,  -0.2065f,0.9435f,-0.2592f,  -0.2891f,0.9443f,-0.1576f,  -0.3258f,0.9276f,-0.1831f,  -0.3999f,0.9134f,-0.0761f,  -0.4323f,0.8965f,-0.0973f,  -0.4954f,0.8685f,0.0139f,  
	-0.5229f,0.8524f,-0.0029f,  -0.5731f,0.8119f,0.1110f,  -0.5952f,0.7975f,0.0984f,  -0.6307f,0.7461f,0.2135f,  -0.6472f,0.7343f,0.2048f,  -0.6669f,0.6734f,0.3190f,  -0.6777f,0.6650f,0.3139f,  -0.6808f,0.5963f,0.4253f,  -0.6861f,0.5918f,0.4232f,  -0.6726f,0.5169f,0.5296f,  -0.6725f,0.5167f,0.5299f,  -0.6427f,0.4370f,0.6292f,  -0.6377f,0.4413f,0.6313f,  -0.5924f,0.3583f,0.7216f,  -0.5830f,0.3672f,0.7247f,  
	-0.5234f,0.2821f,0.8041f,  -0.5101f,0.2954f,0.8078f,  -0.4377f,0.2093f,0.8744f,  -0.4213f,0.2268f,0.8781f,  -0.3381f,0.1407f,0.9306f,  -0.3189f,0.1620f,0.9338f,  -0.2271f,0.0766f,0.9708f,  -0.2060f,0.1013f,0.9733f,  -0.1080f,0.0174f,0.9940f,  -0.0854f,0.0449f,0.9953f,  0.0163f,-0.0371f,0.9992f,  0.0397f,-0.0073f,0.9992f,  0.1424f,-0.0870f,0.9860f,  0.1662f,-0.0555f,0.9845f,  0.2673f,-0.1327f,0.9544f,  
	0.2909f,-0.1002f,0.9515f,  0.3878f,-0.1746f,0.9051f,  0.4108f,-0.1417f,0.9007f,  0.5010f,-0.2132f,0.8388f,  0.5229f,-0.1805f,0.8330f,  0.6041f,-0.2491f,0.7570f,  0.6247f,-0.2173f,0.7501f,  0.6947f,-0.2828f,0.6613f,  0.7136f,-0.2525f,0.6535f,  0.7707f,-0.3150f,0.5539f,  0.7876f,-0.2867f,0.5454f,  0.8301f,-0.3462f,0.4371f,  0.8449f,-0.3204f,0.4283f,  0.8717f,-0.3768f,0.3134f,  0.8842f,-0.3540f,0.3048f,  
	0.8943f,-0.4072f,0.1858f,  0.9045f,-0.3878f,0.1776f,  0.8974f,-0.4376f,0.0569f,  0.9052f,-0.4220f,0.0497f,  0.8808f,-0.4683f,-0.0701f,  0.8863f,-0.4568f,-0.0759f,  0.8448f,-0.4992f,-0.1925f,  0.8481f,-0.4921f,-0.1964f,  0.7902f,-0.5302f,-0.3073f,  0.7914f,-0.5276f,-0.3088f,  0.7181f,-0.5610f,-0.4119f,  0.7173f,-0.5630f,-0.4105f,  0.3962f,-0.7061f,-0.5868f,  0.4146f,-0.7045f,-0.5760f,  0.5145f,-0.6214f,-0.5908f,  
	0.5295f,-0.6162f,-0.5830f,  0.6173f,-0.5196f,-0.5907f,  0.6281f,-0.5111f,-0.5867f,  0.7016f,-0.4029f,-0.5877f,  0.7076f,-0.3918f,-0.5880f,  0.7650f,-0.2744f,-0.5827f,  0.7655f,-0.2614f,-0.5879f,  0.8056f,-0.1374f,-0.5763f,  0.8003f,-0.1232f,-0.5868f,  0.8222f,0.0046f,-0.5692f,  0.8109f,0.0189f,-0.5849f,  0.8144f,0.1475f,-0.5613f,  0.7968f,0.1612f,-0.5823f,  0.7823f,0.2878f,-0.5525f,  0.7588f,0.2996f,-0.5783f,  
	0.7270f,0.4214f,-0.5421f,  0.6979f,0.4307f,-0.5721f,  0.6504f,0.5449f,-0.5292f,  0.6163f,0.5510f,-0.5626f,  0.5549f,0.6553f,-0.5125f,  0.5166f,0.6575f,-0.5484f,  0.4437f,0.7498f,-0.4907f,  0.4024f,0.7480f,-0.5279f,  0.3207f,0.8267f,-0.4624f,  0.2773f,0.8207f,-0.4996f,  0.1898f,0.8846f,-0.4260f,  0.1456f,0.8747f,-0.4622f,  0.0555f,0.9231f,-0.3804f,  0.0116f,0.9099f,-0.4147f,  -0.0781f,0.9425f,-0.3250f,  
	-0.1203f,0.9266f,-0.3563f,  -0.2065f,0.9435f,-0.2592f,  -0.2461f,0.9258f,-0.2870f,  -0.3258f,0.9276f,-0.1831f,  -0.3617f,0.9090f,-0.2068f,  -0.4323f,0.8965f,-0.0973f,  -0.4639f,0.8782f,-0.1168f,  -0.5229f,0.8524f,-0.0029f,  -0.5496f,0.8352f,-0.0183f,  -0.5952f,0.7975f,0.0984f,  -0.6165f,0.7825f,0.0871f,  -0.6472f,0.7343f,0.2048f,  -0.6629f,0.7223f,0.1972f,  -0.6777f,0.6650f,0.3139f,  -0.6878f,0.6566f,0.3095f,  
	-0.6861f,0.5918f,0.4232f,  -0.6908f,0.5875f,0.4214f,  -0.6725f,0.5167f,0.5299f,  -0.6722f,0.5168f,0.5302f,  -0.6377f,0.4413f,0.6313f,  -0.6327f,0.4461f,0.6330f,  -0.5830f,0.3672f,0.7247f,  -0.5738f,0.3765f,0.7273f,  -0.5101f,0.2954f,0.8078f,  -0.4973f,0.3091f,0.8107f,  -0.4213f,0.2268f,0.8781f,  -0.4053f,0.2446f,0.8808f,  -0.3189f,0.1620f,0.9338f,  -0.3005f,0.1836f,0.9360f,  -0.2060f,0.1013f,0.9733f,  
	-0.1856f,0.1262f,0.9745f,  -0.0854f,0.0449f,0.9953f,  -0.0637f,0.0726f,0.9953f,  0.0397f,-0.0073f,0.9992f,  0.0623f,0.0226f,0.9978f,  0.1662f,-0.0555f,0.9845f,  0.1890f,-0.0241f,0.9817f,  0.2909f,-0.1002f,0.9515f,  0.3135f,-0.0678f,0.9472f,  0.4108f,-0.1417f,0.9007f,  0.4328f,-0.1089f,0.8949f,  0.5229f,-0.1805f,0.8330f,  0.5439f,-0.1480f,0.8260f,  0.6247f,-0.2173f,0.7501f,  0.6443f,-0.1856f,0.7419f,  
	0.7136f,-0.2525f,0.6535f,  0.7316f,-0.2223f,0.6444f,  0.7876f,-0.2867f,0.5454f,  0.8038f,-0.2585f,0.5358f,  0.8449f,-0.3204f,0.4283f,  0.8591f,-0.2947f,0.4185f,  0.8842f,-0.3540f,0.3048f,  0.8962f,-0.3313f,0.2952f,  0.9045f,-0.3878f,0.1776f,  0.9142f,-0.3686f,0.1687f,  0.9052f,-0.4220f,0.0497f,  0.9126f,-0.4066f,0.0419f,  0.8863f,-0.4568f,-0.0759f,  0.8915f,-0.4456f,-0.0821f,  0.8481f,-0.4921f,-0.1964f,  
	0.8511f,-0.4852f,-0.2004f,  0.7914f,-0.5276f,-0.3088f,  0.7924f,-0.5253f,-0.3102f,  0.7173f,-0.5630f,-0.4105f,  0.7165f,-0.5653f,-0.4088f,  0.4146f,-0.7045f,-0.5760f,  0.4327f,-0.7023f,-0.5653f,  0.5295f,-0.6162f,-0.5830f,  0.5439f,-0.6106f,-0.5756f,  0.6281f,-0.5111f,-0.5867f,  0.6381f,-0.5025f,-0.5834f,  0.7076f,-0.3918f,-0.5880f,  0.7124f,-0.3809f,-0.5894f,  0.7655f,-0.2614f,-0.5879f,  0.7647f,-0.2489f,-0.5944f,  
	0.8003f,-0.1232f,-0.5868f,  0.7934f,-0.1101f,-0.5986f,  0.8109f,0.0189f,-0.5849f,  0.7977f,0.0318f,-0.6022f,  0.7968f,0.1612f,-0.5823f,  0.7775f,0.1728f,-0.6046f,  0.7588f,0.2996f,-0.5783f,  0.7335f,0.3092f,-0.6053f,  0.6979f,0.4307f,-0.5721f,  0.6671f,0.4373f,-0.6031f,  0.6163f,0.5510f,-0.5626f,  0.5807f,0.5540f,-0.5966f,  0.5166f,0.6575f,-0.5484f,  0.4772f,0.6564f,-0.5843f,  0.4024f,0.7480f,-0.5279f,  
	0.3600f,0.7427f,-0.5646f,  0.2773f,0.8207f,-0.4996f,  0.2333f,0.8113f,-0.5360f,  0.1456f,0.8747f,-0.4622f,  0.1011f,0.8617f,-0.4972f,  0.0116f,0.9099f,-0.4147f,  -0.0321f,0.8938f,-0.4474f,  -0.1203f,0.9266f,-0.3563f,  -0.1622f,0.9082f,-0.3859f,  -0.2461f,0.9258f,-0.2870f,  -0.2850f,0.9060f,-0.3129f,  -0.3617f,0.9090f,-0.2068f,  -0.3968f,0.8889f,-0.2288f,  -0.4639f,0.8782f,-0.1168f,  -0.4945f,0.8587f,-0.1347f,  
	-0.5496f,0.8352f,-0.0183f,  -0.5752f,0.8174f,-0.0321f,  -0.6165f,0.7825f,0.0871f,  -0.6368f,0.7672f,0.0771f,  -0.6629f,0.7223f,0.1972f,  -0.6778f,0.7101f,0.1906f,  -0.6878f,0.6566f,0.3095f,  -0.6973f,0.6483f,0.3057f,  -0.6908f,0.5875f,0.4214f,  -0.6951f,0.5835f,0.4200f,  -0.6722f,0.5168f,0.5302f,  -0.6716f,0.5173f,0.5304f,  -0.6327f,0.4461f,0.6330f,  -0.6277f,0.4511f,0.6344f,  -0.5738f,0.3765f,0.7273f,  
	-0.5648f,0.3861f,0.7294f,  -0.4973f,0.3091f,0.8107f,  -0.4847f,0.3231f,0.8128f,  -0.4053f,0.2446f,0.8808f,  -0.3899f,0.2627f,0.8826f,  -0.3005f,0.1836f,0.9360f,  -0.2827f,0.2053f,0.9370f,  -0.1856f,0.1262f,0.9745f,  -0.1660f,0.1512f,0.9745f,  -0.0637f,0.0726f,0.9953f,  -0.0428f,0.1003f,0.9940f,  0.0623f,0.0226f,0.9978f,  0.0839f,0.0524f,0.9951f,  0.1890f,-0.0241f,0.9817f,  0.2109f,0.0073f,0.9775f,  
	0.3135f,-0.0678f,0.9472f,  0.3352f,-0.0354f,0.9415f,  0.4328f,-0.1089f,0.8949f,  0.4539f,-0.0762f,0.8878f,  0.5439f,-0.1480f,0.8260f,  0.5641f,-0.1156f,0.8176f,  0.6443f,-0.1856f,0.7419f,  0.6632f,-0.1541f,0.7324f,  0.7316f,-0.2223f,0.6444f,  0.7489f,-0.1922f,0.6342f,  0.8038f,-0.2585f,0.5358f,  0.8192f,-0.2304f,0.5252f,  0.8591f,-0.2947f,0.4185f,  0.8725f,-0.2692f,0.4078f,  0.8962f,-0.3313f,0.2952f,  
	0.9075f,-0.3088f,0.2848f,  0.9142f,-0.3686f,0.1687f,  0.9233f,-0.3495f,0.1590f,  0.9126f,-0.4066f,0.0419f,  0.9196f,-0.3914f,0.0336f,  0.8915f,-0.4456f,-0.0821f,  0.8963f,-0.4345f,-0.0886f,  0.8511f,-0.4852f,-0.2004f,  0.8539f,-0.4786f,-0.2046f,  0.7924f,-0.5253f,-0.3102f,  0.7932f,-0.5232f,-0.3116f,  0.7165f,-0.5653f,-0.4088f,  0.7156f,-0.5679f,-0.4068f,  0.4327f,-0.7023f,-0.5653f,  0.4504f,-0.6996f,-0.5548f,  
	0.5439f,-0.6106f,-0.5756f,  0.5576f,-0.6046f,-0.5688f,  0.6381f,-0.5025f,-0.5834f,  0.6470f,-0.4939f,-0.5809f,  0.7124f,-0.3809f,-0.5894f,  0.7160f,-0.3703f,-0.5918f,  0.7647f,-0.2489f,-0.5944f,  0.7624f,-0.2372f,-0.6021f,  0.7934f,-0.1101f,-0.5986f,  0.7849f,-0.0981f,-0.6118f,  0.7977f,0.0318f,-0.6022f,  0.7829f,0.0430f,-0.6207f,  0.7775f,0.1728f,-0.6046f,  0.7564f,0.1823f,-0.6281f,  0.7335f,0.3092f,-0.6053f,  
	0.7065f,0.3161f,-0.6332f,  0.6671f,0.4373f,-0.6031f,  0.6348f,0.4409f,-0.6346f,  0.5807f,0.5540f,-0.5966f,  0.5438f,0.5536f,-0.6307f,  0.4772f,0.6564f,-0.5843f,  0.4366f,0.6520f,-0.6199f,  0.3600f,0.7427f,-0.5646f,  0.3170f,0.7340f,-0.6006f,  0.2333f,0.8113f,-0.5360f,  0.1890f,0.7988f,-0.5712f,  0.1011f,0.8617f,-0.4972f,  0.0567f,0.8456f,-0.5307f,  -0.0321f,0.8938f,-0.4474f,  -0.0754f,0.8750f,-0.4783f,  
	-0.1622f,0.9082f,-0.3859f,  -0.2033f,0.8875f,-0.4136f,  -0.2850f,0.9060f,-0.3129f,  -0.3230f,0.8844f,-0.3368f,  -0.3968f,0.8889f,-0.2288f,  -0.4310f,0.8674f,-0.2488f,  -0.4945f,0.8587f,-0.1347f,  -0.5241f,0.8382f,-0.1508f,  -0.5752f,0.8174f,-0.0321f,  -0.5998f,0.7989f,-0.0444f,  -0.6368f,0.7672f,0.0771f,  -0.6562f,0.7515f,0.0683f,  -0.6778f,0.7101f,0.1906f,  -0.6919f,0.6979f,0.1848f,  -0.6973f,0.6483f,0.3057f,  
	-0.7062f,0.6401f,0.3025f,  -0.6951f,0.5835f,0.4200f,  -0.6990f,0.5796f,0.4188f,  -0.6716f,0.5173f,0.5304f,  -0.6708f,0.5180f,0.5307f,  -0.6277f,0.4511f,0.6344f,  -0.6226f,0.4565f,0.6356f,  -0.5648f,0.3861f,0.7294f,  -0.5559f,0.3960f,0.7308f,  -0.4847f,0.3231f,0.8128f,  -0.4726f,0.3373f,0.8142f,  -0.3899f,0.2627f,0.8826f,  -0.3749f,0.2810f,0.8835f,  -0.2827f,0.2053f,0.9370f,  -0.2655f,0.2272f,0.9370f,  
	-0.1660f,0.1512f,0.9745f,  -0.1471f,0.1763f,0.9733f,  -0.0428f,0.1003f,0.9940f,  -0.0227f,0.1280f,0.9915f,  0.0839f,0.0524f,0.9951f,  0.1047f,0.0823f,0.9911f,  0.2109f,0.0073f,0.9775f,  0.2320f,0.0387f,0.9720f,  0.3352f,-0.0354f,0.9415f,  0.3561f,-0.0031f,0.9344f,  0.4539f,-0.0762f,0.8878f,  0.4742f,-0.0436f,0.8794f,  0.5641f,-0.1156f,0.8176f,  0.5834f,-0.0833f,0.8079f,  0.6632f,-0.1541f,0.7324f,  
	0.6812f,-0.1226f,0.7218f,  0.7489f,-0.1922f,0.6342f,  0.7654f,-0.1622f,0.6228f,  0.8192f,-0.2304f,0.5252f,  0.8340f,-0.2025f,0.5134f,  0.8725f,-0.2692f,0.4078f,  0.8853f,-0.2438f,0.3960f,  0.9075f,-0.3088f,0.2848f,  0.9183f,-0.2864f,0.2734f,  0.9233f,-0.3495f,0.1590f,  0.9320f,-0.3306f,0.1487f,  0.9196f,-0.3914f,0.0336f,  0.9261f,-0.3764f,0.0246f,  0.8963f,-0.4345f,-0.0886f,  0.9007f,-0.4237f,-0.0956f,  
	0.8539f,-0.4786f,-0.2046f,  0.8563f,-0.4722f,-0.2090f,  0.7932f,-0.5232f,-0.3116f,  0.7939f,-0.5214f,-0.3128f,  0.7156f,-0.5679f,-0.4068f,  0.7147f,-0.5707f,-0.4044f,  0.4504f,-0.6996f,-0.5548f,  0.4677f,-0.6963f,-0.5444f,  0.5576f,-0.6046f,-0.5688f,  0.5706f,-0.5984f,-0.5624f,  0.6470f,-0.4939f,-0.5809f,  0.6550f,-0.4853f,-0.5792f,  0.7160f,-0.3703f,-0.5918f,  0.7183f,-0.3601f,-0.5952f,  0.7624f,-0.2372f,-0.6021f,  
	0.7587f,-0.2263f,-0.6109f,  0.7849f,-0.0981f,-0.6118f,  0.7748f,-0.0875f,-0.6261f,  0.7829f,0.0430f,-0.6207f,  0.7664f,0.0523f,-0.6402f,  0.7564f,0.1823f,-0.6281f,  0.7337f,0.1895f,-0.6525f,  0.7065f,0.3161f,-0.6332f,  0.6779f,0.3203f,-0.6617f,  0.6348f,0.4409f,-0.6346f,  0.6010f,0.4414f,-0.6663f,  0.5438f,0.5536f,-0.6307f,  0.5057f,0.5501f,-0.6646f,  0.4366f,0.6520f,-0.6199f,  0.3953f,0.6442f,-0.6548f,  
	0.3170f,0.7340f,-0.6006f,  0.2736f,0.7221f,-0.6354f,  0.1890f,0.7988f,-0.5712f,  0.1446f,0.7830f,-0.6049f,  0.0567f,0.8456f,-0.5307f,  0.0126f,0.8268f,-0.5624f,  -0.0754f,0.8750f,-0.4783f,  -0.1182f,0.8537f,-0.5071f,  -0.2033f,0.8875f,-0.4136f,  -0.2436f,0.8648f,-0.4391f,  -0.3230f,0.8844f,-0.3368f,  -0.3601f,0.8612f,-0.3587f,  -0.4310f,0.8674f,-0.2488f,  -0.4640f,0.8447f,-0.2669f,  -0.5241f,0.8382f,-0.1508f,  
	-0.5526f,0.8170f,-0.1651f,  -0.5998f,0.7989f,-0.0444f,  -0.6234f,0.7800f,-0.0552f,  -0.6562f,0.7515f,0.0683f,  -0.6747f,0.7356f,0.0607f,  -0.6919f,0.6979f,0.1848f,  -0.7052f,0.6858f,0.1799f,  -0.7062f,0.6401f,0.3025f,  -0.7145f,0.6321f,0.2999f,  -0.6990f,0.5796f,0.4188f,  -0.7025f,0.5761f,0.4178f,  -0.6708f,0.5180f,0.5307f,  -0.6699f,0.5191f,0.5309f,  -0.6226f,0.4565f,0.6356f,  -0.6176f,0.4622f,0.6364f,  
	-0.5559f,0.3960f,0.7308f,  -0.5472f,0.4062f,0.7318f,  -0.4726f,0.3373f,0.8142f,  -0.4608f,0.3518f,0.8148f,  -0.3749f,0.2810f,0.8835f,  -0.3604f,0.2994f,0.8834f,  -0.2655f,0.2272f,0.9370f,  -0.2489f,0.2493f,0.9359f,  -0.1471f,0.1763f,0.9733f,  -0.1288f,0.2015f,0.9710f,  -0.0227f,0.1280f,0.9915f,  -0.0033f,0.1558f,0.9878f,  0.1047f,0.0823f,0.9911f,  0.1248f,0.1121f,0.9858f,  0.2320f,0.0387f,0.9720f,  
	0.2522f,0.0701f,0.9651f,  0.3561f,-0.0031f,0.9344f,  0.3762f,0.0292f,0.9261f,  0.4742f,-0.0436f,0.8794f,  0.4936f,-0.0110f,0.8696f,  0.5834f,-0.0833f,0.8079f,  0.6019f,-0.0510f,0.7970f,  0.6812f,-0.1226f,0.7218f,  0.6985f,-0.0912f,0.7098f,  0.7654f,-0.1622f,0.6228f,  0.7811f,-0.1323f,0.6102f,  0.8340f,-0.2025f,0.5134f,  0.8480f,-0.1746f,0.5004f,  0.8853f,-0.2438f,0.3960f,  0.8975f,-0.2185f,0.3832f,  
	0.9183f,-0.2864f,0.2734f,  0.9284f,-0.2642f,0.2612f,  0.9320f,-0.3306f,0.1487f,  0.9401f,-0.3119f,0.1375f,  0.9261f,-0.3764f,0.0246f,  0.9322f,-0.3616f,0.0151f,  0.9007f,-0.4237f,-0.0956f,  0.9048f,-0.4131f,-0.1029f,  0.8563f,-0.4722f,-0.2090f,  0.8586f,-0.4661f,-0.2135f,  0.7939f,-0.5214f,-0.3128f,  0.7944f,-0.5199f,-0.3140f,  0.7147f,-0.5707f,-0.4044f,  0.7137f,-0.5739f,-0.4016f,  0.4677f,-0.6963f,-0.5444f,  
	0.4844f,-0.6927f,-0.5343f,  0.5706f,-0.5984f,-0.5624f,  0.5828f,-0.5920f,-0.5566f,  0.6550f,-0.4853f,-0.5792f,  0.6619f,-0.4769f,-0.5782f,  0.7183f,-0.3601f,-0.5952f,  0.7194f,-0.3505f,-0.5996f,  0.7587f,-0.2263f,-0.6109f,  0.7535f,-0.2165f,-0.6208f,  0.7748f,-0.0875f,-0.6261f,  0.7632f,-0.0785f,-0.6414f,  0.7664f,0.0523f,-0.6402f,  0.7483f,0.0596f,-0.6607f,  0.7337f,0.1895f,-0.6525f,  0.7094f,0.1942f,-0.6775f,  
	0.6779f,0.3203f,-0.6617f,  0.6480f,0.3215f,-0.6905f,  0.6010f,0.4414f,-0.6663f,  0.5662f,0.4387f,-0.6978f,  0.5057f,0.5501f,-0.6646f,  0.4669f,0.5432f,-0.6978f,  0.3953f,0.6442f,-0.6548f,  0.3536f,0.6331f,-0.6886f,  0.2736f,0.7221f,-0.6354f,  0.2301f,0.7070f,-0.6688f,  0.1446f,0.7830f,-0.6049f,  0.1005f,0.7644f,-0.6369f,  0.0126f,0.8268f,-0.5624f,  -0.0310f,0.8053f,-0.5920f,  -0.1182f,0.8537f,-0.5071f,  
	-0.1601f,0.8303f,-0.5338f,  -0.2436f,0.8648f,-0.4391f,  -0.2830f,0.8403f,-0.4625f,  -0.3601f,0.8612f,-0.3587f,  -0.3960f,0.8366f,-0.3785f,  -0.4640f,0.8447f,-0.2669f,  -0.4958f,0.8210f,-0.2830f,  -0.5526f,0.8170f,-0.1651f,  -0.5799f,0.7951f,-0.1777f,  -0.6234f,0.7800f,-0.0552f,  -0.6459f,0.7607f,-0.0645f,  -0.6747f,0.7356f,0.0607f,  -0.6922f,0.7197f,0.0542f,  -0.7052f,0.6858f,0.1799f,  -0.7178f,0.6737f,0.1759f,  
	-0.7145f,0.6321f,0.2999f,  -0.7222f,0.6243f,0.2978f,  -0.7025f,0.5761f,0.4178f,  -0.7057f,0.5728f,0.4171f,  -0.6699f,0.5191f,0.5309f,  -0.6687f,0.5204f,0.5310f,  -0.6176f,0.4622f,0.6364f,  -0.6125f,0.4681f,0.6369f,  -0.5472f,0.4062f,0.7318f,  -0.5387f,0.4166f,0.7323f,  -0.4608f,0.3518f,0.8148f,  -0.4493f,0.3665f,0.8148f,  -0.3604f,0.2994f,0.8834f,  -0.3464f,0.3180f,0.8825f,  -0.2489f,0.2493f,0.9359f,  
	-0.2328f,0.2715f,0.9339f,  -0.1288f,0.2015f,0.9710f,  -0.1112f,0.2267f,0.9676f,  -0.0033f,0.1558f,0.9878f,  0.0154f,0.1837f,0.9829f,  0.1248f,0.1121f,0.9858f,  0.1441f,0.1420f,0.9793f,  0.2522f,0.0701f,0.9651f,  0.2717f,0.1014f,0.9570f,  0.3762f,0.0292f,0.9261f,  0.3954f,0.0615f,0.9164f,  0.4936f,-0.0110f,0.8696f,  0.5123f,0.0216f,0.8585f,  0.6019f,-0.0510f,0.7970f,  0.6196f,-0.0187f,0.7847f,  
	0.6985f,-0.0912f,0.7098f,  0.7150f,-0.0598f,0.6966f,  0.7811f,-0.1323f,0.6102f,  0.7962f,-0.1024f,0.5963f,  0.8480f,-0.1746f,0.5004f,  0.8614f,-0.1467f,0.4863f,  0.8975f,-0.2185f,0.3832f,  0.9090f,-0.1932f,0.3693f,  0.9284f,-0.2642f,0.2612f,  0.9380f,-0.2421f,0.2480f,  0.9401f,-0.3119f,0.1375f,  0.9477f,-0.2934f,0.1255f,  0.9322f,-0.3616f,0.0151f,  0.9378f,-0.3470f,0.0050f,  0.9048f,-0.4131f,-0.1029f,  
	0.9086f,-0.4028f,-0.1106f,  0.8586f,-0.4661f,-0.2135f,  0.8606f,-0.4602f,-0.2182f,  0.7944f,-0.5199f,-0.3140f,  0.7948f,-0.5187f,-0.3150f,  0.7137f,-0.5739f,-0.4016f,  0.7128f,-0.5772f,-0.3984f,  0.4844f,-0.6927f,-0.5343f,  0.5007f,-0.6886f,-0.5245f,  0.5828f,-0.5920f,-0.5566f,  0.5942f,-0.5856f,-0.5514f,  0.6619f,-0.4769f,-0.5782f,  0.6678f,-0.4688f,-0.5781f,  0.7194f,-0.3505f,-0.5996f,  0.7192f,-0.3418f,-0.6049f,  
	0.7535f,-0.2165f,-0.6208f,  0.7469f,-0.2080f,-0.6316f,  0.7632f,-0.0785f,-0.6414f,  0.7500f,-0.0712f,-0.6576f,  0.7483f,0.0596f,-0.6607f,  0.7287f,0.0648f,-0.6818f,  0.7094f,0.1942f,-0.6775f,  0.6838f,0.1962f,-0.7028f,  0.6480f,0.3215f,-0.6905f,  0.6169f,0.3199f,-0.7191f,  0.5662f,0.4387f,-0.6978f,  0.5304f,0.4330f,-0.7288f,  0.4669f,0.5432f,-0.6978f,  0.4275f,0.5332f,-0.7300f,  0.3536f,0.6331f,-0.6886f,  
	0.3116f,0.6188f,-0.7211f,  0.2301f,0.7070f,-0.6688f,  0.1866f,0.6890f,-0.7004f,  0.1005f,0.7644f,-0.6369f,  0.0568f,0.7432f,-0.6667f,  -0.0310f,0.8053f,-0.5920f,  -0.0737f,0.7816f,-0.6194f,  -0.1601f,0.8303f,-0.5338f,  -0.2010f,0.8049f,-0.5583f,  -0.2830f,0.8403f,-0.4625f,  -0.3211f,0.8143f,-0.4836f,  -0.3960f,0.8366f,-0.3785f,  -0.4306f,0.8110f,-0.3961f,  -0.4958f,0.8210f,-0.2830f,  -0.5264f,0.7966f,-0.2972f,  
	-0.5799f,0.7951f,-0.1777f,  -0.6060f,0.7728f,-0.1886f,  -0.6459f,0.7607f,-0.0645f,  -0.6673f,0.7413f,-0.0724f,  -0.6922f,0.7197f,0.0542f,  -0.7087f,0.7038f,0.0489f,  -0.7178f,0.6737f,0.1759f,  -0.7296f,0.6618f,0.1727f,  -0.7222f,0.6243f,0.2978f,  -0.7294f,0.6167f,0.2962f,  -0.7057f,0.5728f,0.4171f,  -0.7084f,0.5697f,0.4166f,  -0.6687f,0.5204f,0.5310f,  -0.6674f,0.5220f,0.5311f,  -0.6125f,0.4681f,0.6369f,  
	-0.6075f,0.4743f,0.6372f,  -0.5387f,0.4166f,0.7323f,  -0.5304f,0.4273f,0.7322f,  -0.4493f,0.3665f,0.8148f,  -0.4381f,0.3814f,0.8140f,  -0.3464f,0.3180f,0.8825f,  -0.3328f,0.3368f,0.8808f,  -0.2328f,0.2715f,0.9339f,  -0.2173f,0.2937f,0.9309f,  -0.1112f,0.2267f,0.9676f,  -0.0942f,0.2521f,0.9631f,  0.0154f,0.1837f,0.9829f,  0.0334f,0.2116f,0.9768f,  0.1441f,0.1420f,0.9793f,  0.1627f,0.1720f,0.9716f,  
	0.2717f,0.1014f,0.9570f,  0.2905f,0.1329f,0.9476f,  0.3954f,0.0615f,0.9164f,  0.4139f,0.0938f,0.9055f,  0.5123f,0.0216f,0.8585f,  0.5302f,0.0542f,0.8461f,  0.6196f,-0.0187f,0.7847f,  0.6366f,0.0136f,0.7710f,  0.7150f,-0.0598f,0.6966f,  0.7308f,-0.0285f,0.6820f,  0.7962f,-0.1024f,0.5963f,  0.8106f,-0.0725f,0.5812f,  0.8614f,-0.1467f,0.4863f,  0.8741f,-0.1190f,0.4710f,  0.9090f,-0.1932f,0.3693f,  
	0.9199f,-0.1681f,0.3542f,  0.9380f,-0.2421f,0.2480f,  0.9470f,-0.2201f,0.2338f,  0.9477f,-0.2934f,0.1255f,  0.9548f,-0.2750f,0.1128f,  0.9378f,-0.3470f,0.0050f,  0.9430f,-0.3326f,-0.0058f,  0.9086f,-0.4028f,-0.1106f,  0.9120f,-0.3927f,-0.1187f,  0.8606f,-0.4602f,-0.2182f,  0.8623f,-0.4547f,-0.2231f,  0.7948f,-0.5187f,-0.3150f,  0.7951f,-0.5177f,-0.3159f,  0.7128f,-0.5772f,-0.3984f,  0.7119f,-0.5809f,-0.3947f,  
	0.5007f,-0.6886f,-0.5245f,  0.5164f,-0.6842f,-0.5151f,  0.5942f,-0.5856f,-0.5514f,  0.6048f,-0.5790f,-0.5467f,  0.6678f,-0.4688f,-0.5781f,  0.6726f,-0.4612f,-0.5787f,  0.7192f,-0.3418f,-0.6049f,  0.7178f,-0.3338f,-0.6110f,  0.7469f,-0.2080f,-0.6316f,  0.7389f,-0.2008f,-0.6432f,  0.7500f,-0.0712f,-0.6576f,  0.7355f,-0.0658f,-0.6743f,  0.7287f,0.0648f,-0.6818f,  0.7078f,0.0675f,-0.7032f,  0.6838f,0.1962f,-0.7028f,  
	0.6570f,0.1956f,-0.7281f,  0.6169f,0.3199f,-0.7191f,  0.5849f,0.3153f,-0.7473f,  0.5304f,0.4330f,-0.7288f,  0.4940f,0.4242f,-0.7589f,  0.4275f,0.5332f,-0.7300f,  0.3878f,0.5201f,-0.7610f,  0.3116f,0.6188f,-0.7211f,  0.2696f,0.6017f,-0.7518f,  0.1866f,0.6890f,-0.7004f,  0.1436f,0.6682f,-0.7299f,  0.0568f,0.7432f,-0.6667f,  0.0139f,0.7195f,-0.6944f,  -0.0737f,0.7816f,-0.6194f,  -0.1156f,0.7558f,-0.6445f,  
	-0.2010f,0.8049f,-0.5583f,  -0.2408f,0.7780f,-0.5804f,  -0.3211f,0.8143f,-0.4836f,  -0.3580f,0.7870f,-0.5024f,  -0.4306f,0.8110f,-0.3961f,  -0.4640f,0.7844f,-0.4116f,  -0.5264f,0.7966f,-0.2972f,  -0.5558f,0.7716f,-0.3095f,  -0.6060f,0.7728f,-0.1886f,  -0.6309f,0.7502f,-0.1979f,  -0.6673f,0.7413f,-0.0724f,  -0.6876f,0.7218f,-0.0790f,  -0.7087f,0.7038f,0.0489f,  -0.7244f,0.6879f,0.0445f,  -0.7296f,0.6618f,0.1727f,  
	-0.7406f,0.6500f,0.1701f,  -0.7294f,0.6167f,0.2962f,  -0.7360f,0.6093f,0.2950f,  -0.7084f,0.5697f,0.4166f,  -0.7108f,0.5670f,0.4162f,  -0.6674f,0.5220f,0.5311f,  -0.6659f,0.5239f,0.5311f,  -0.6075f,0.4743f,0.6372f,  -0.6025f,0.4808f,0.6371f,  -0.5304f,0.4273f,0.7322f,  -0.5222f,0.4382f,0.7316f,  -0.4381f,0.3814f,0.8140f,  -0.4272f,0.3964f,0.8126f,  -0.3328f,0.3368f,0.8808f,  -0.3196f,0.3557f,0.8782f,  
	-0.2173f,0.2937f,0.9309f,  -0.2023f,0.3161f,0.9269f,  -0.0942f,0.2521f,0.9631f,  -0.0778f,0.2775f,0.9576f,  0.0334f,0.2116f,0.9768f,  0.0508f,0.2396f,0.9695f,  0.1627f,0.1720f,0.9716f,  0.1806f,0.2020f,0.9626f,  0.2905f,0.1329f,0.9476f,  0.3085f,0.1643f,0.9369f,  0.4139f,0.0938f,0.9055f,  0.4317f,0.1261f,0.8931f,  0.5302f,0.0542f,0.8461f,  0.5474f,0.0868f,0.8323f,  0.6366f,0.0136f,0.7710f,  
	0.6529f,0.0459f,0.7560f,  0.7308f,-0.0285f,0.6820f,  0.7459f,0.0029f,0.6661f,  0.8106f,-0.0725f,0.5812f,  0.8242f,-0.0427f,0.5647f,  0.8741f,-0.1190f,0.4710f,  0.8861f,-0.0912f,0.4544f,  0.9199f,-0.1681f,0.3542f,  0.9302f,-0.1430f,0.3381f,  0.9470f,-0.2201f,0.2338f,  0.9555f,-0.1982f,0.2186f,  0.9548f,-0.2750f,0.1128f,  0.9614f,-0.2568f,0.0991f,  0.9430f,-0.3326f,-0.0058f,  0.9478f,-0.3185f,-0.0172f,  
	0.9120f,-0.3927f,-0.1187f,  0.9150f,-0.3829f,-0.1273f,  0.8623f,-0.4547f,-0.2231f,  0.8637f,-0.4494f,-0.2280f,  0.7951f,-0.5177f,-0.3159f,  0.7952f,-0.5171f,-0.3166f,  0.7119f,-0.5809f,-0.3947f,  0.7110f,-0.5848f,-0.3906f,  0.5164f,-0.6842f,-0.5151f,  0.5314f,-0.6793f,-0.5060f,  0.6048f,-0.5790f,-0.5467f,  0.6145f,-0.5726f,-0.5426f,  0.6726f,-0.4612f,-0.5787f,  0.6763f,-0.4540f,-0.5800f,  0.7178f,-0.3338f,-0.6110f,  
	0.7151f,-0.3269f,-0.6179f,  0.7389f,-0.2008f,-0.6432f,  0.7296f,-0.1951f,-0.6554f,  0.7355f,-0.0658f,-0.6743f,  0.7197f,-0.0623f,-0.6915f,  0.7078f,0.0675f,-0.7032f,  0.6858f,0.0679f,-0.7247f,  0.6570f,0.1956f,-0.7281f,  0.6292f,0.1923f,-0.7531f,  0.5849f,0.3153f,-0.7473f,  0.5521f,0.3079f,-0.7748f,  0.4940f,0.4242f,-0.7589f,  0.4573f,0.4124f,-0.7879f,  0.3878f,0.5201f,-0.7610f,  0.3480f,0.5041f,-0.7904f,  
	0.2696f,0.6017f,-0.7518f,  0.2280f,0.5819f,-0.7807f,  0.1436f,0.6682f,-0.7299f,  0.1013f,0.6451f,-0.7574f,  0.0139f,0.7195f,-0.6944f,  -0.0281f,0.6937f,-0.7197f,  -0.1156f,0.7558f,-0.6445f,  -0.1563f,0.7283f,-0.6672f,  -0.2408f,0.7780f,-0.5804f,  -0.2793f,0.7496f,-0.6001f,  -0.3580f,0.7870f,-0.5024f,  -0.3936f,0.7588f,-0.5190f,  -0.4640f,0.7844f,-0.4116f,  -0.4960f,0.7572f,-0.4251f,  -0.5558f,0.7716f,-0.3095f,  
	-0.5838f,0.7462f,-0.3199f,  -0.6309f,0.7502f,-0.1979f,  -0.6546f,0.7274f,-0.2056f,  -0.6876f,0.7218f,-0.0790f,  -0.7069f,0.7023f,-0.0844f,  -0.7244f,0.6879f,0.0445f,  -0.7392f,0.6722f,0.0412f,  -0.7406f,0.6500f,0.1701f,  -0.7510f,0.6385f,0.1683f,  -0.7360f,0.6093f,0.2950f,  -0.7421f,0.6023f,0.2942f,  -0.7108f,0.5670f,0.4162f,  -0.7129f,0.5645f,0.4160f,  -0.6659f,0.5239f,0.5311f,  -0.6643f,0.5260f,0.5311f,  
	-0.6025f,0.4808f,0.6371f,  -0.5975f,0.4875f,0.6367f,  -0.5222f,0.4382f,0.7316f,  -0.5142f,0.4493f,0.7306f,  -0.4272f,0.3964f,0.8126f,  -0.4165f,0.4117f,0.8106f,  -0.3196f,0.3557f,0.8782f,  -0.3068f,0.3748f,0.8748f,  -0.2023f,0.3161f,0.9269f,  -0.1877f,0.3387f,0.9220f,  -0.0778f,0.2775f,0.9576f,  -0.0619f,0.3031f,0.9510f,  0.0508f,0.2396f,0.9695f,  0.0676f,0.2677f,0.9611f,  0.1806f,0.2020f,0.9626f,  
	0.1978f,0.2321f,0.9524f,  0.3085f,0.1643f,0.9369f,  0.3259f,0.1959f,0.9249f,  0.4317f,0.1261f,0.8931f,  0.4488f,0.1586f,0.8794f,  0.5474f,0.0868f,0.8323f,  0.5639f,0.1196f,0.8171f,  0.6529f,0.0459f,0.7560f,  0.6685f,0.0783f,0.7396f,  0.7459f,0.0029f,0.6661f,  0.7602f,0.0343f,0.6487f,  0.8242f,-0.0427f,0.5647f,  0.8371f,-0.0128f,0.5468f,  0.8861f,-0.0912f,0.4544f,  0.8974f,-0.0636f,0.4366f,  
	0.9302f,-0.1430f,0.3381f,  0.9398f,-0.1181f,0.3207f,  0.9555f,-0.1982f,0.2186f,  0.9633f,-0.1765f,0.2023f,  0.9614f,-0.2568f,0.0991f,  0.9674f,-0.2388f,0.0846f,  0.9478f,-0.3185f,-0.0172f,  0.9520f,-0.3046f,-0.0294f,  0.9150f,-0.3829f,-0.1273f,  0.9176f,-0.3734f,-0.1363f,  0.8637f,-0.4494f,-0.2280f,  0.8649f,-0.4444f,-0.2332f,  0.7952f,-0.5171f,-0.3166f,  0.7952f,-0.5167f,-0.3172f,  0.7110f,-0.5848f,-0.3906f,  
	0.7101f,-0.5889f,-0.3859f,  0.5314f,-0.6793f,-0.5060f,  0.5459f,-0.6743f,-0.4973f,  0.6145f,-0.5726f,-0.5426f,  0.6233f,-0.5664f,-0.5391f,  0.6763f,-0.4540f,-0.5800f,  0.6790f,-0.4475f,-0.5820f,  0.7151f,-0.3269f,-0.6179f,  0.7113f,-0.3211f,-0.6253f,  0.7296f,-0.1951f,-0.6554f,  0.7192f,-0.1911f,-0.6680f,  0.7197f,-0.0623f,-0.6915f,  0.7028f,-0.0609f,-0.7088f,  0.6858f,0.0679f,-0.7247f,  0.6627f,0.0659f,-0.7459f,  
	0.6292f,0.1923f,-0.7531f,  0.6007f,0.1863f,-0.7774f,  0.5521f,0.3079f,-0.7748f,  0.5190f,0.2977f,-0.8013f,  0.4573f,0.4124f,-0.7879f,  0.4204f,0.3979f,-0.8154f,  0.3480f,0.5041f,-0.7904f,  0.3085f,0.4855f,-0.8180f,  0.2280f,0.5819f,-0.7807f,  0.1870f,0.5596f,-0.8074f,  0.1013f,0.6451f,-0.7574f,  0.0598f,0.6198f,-0.7825f,  -0.0281f,0.6937f,-0.7197f,  -0.0691f,0.6662f,-0.7426f,  -0.1563f,0.7283f,-0.6672f,  
	-0.1957f,0.6993f,-0.6875f,  -0.2793f,0.7496f,-0.6001f,  -0.3164f,0.7202f,-0.6175f,  -0.3936f,0.7588f,-0.5190f,  -0.4277f,0.7298f,-0.5334f,  -0.4960f,0.7572f,-0.4251f,  -0.5266f,0.7294f,-0.4365f,  -0.5838f,0.7462f,-0.3199f,  -0.6105f,0.7206f,-0.3286f,  -0.6546f,0.7274f,-0.2056f,  -0.6772f,0.7047f,-0.2118f,  -0.7069f,0.7023f,-0.0844f,  -0.7251f,0.6829f,-0.0885f,  -0.7392f,0.6722f,0.0412f,  -0.7531f,0.6567f,0.0387f,  
	-0.7510f,0.6385f,0.1683f,  -0.7607f,0.6272f,0.1671f,  -0.7421f,0.6023f,0.2942f,  -0.7477f,0.5955f,0.2938f,  -0.7129f,0.5645f,0.4160f,  -0.7147f,0.5623f,0.4160f,  -0.6643f,0.5260f,0.5311f,  -0.6625f,0.5284f,0.5309f,  -0.5975f,0.4875f,0.6367f,  -0.5925f,0.4944f,0.6360f,  -0.5142f,0.4493f,0.7306f,  -0.5063f,0.4606f,0.7290f,  -0.4165f,0.4117f,0.8106f,  -0.4062f,0.4271f,0.8078f,  -0.3068f,0.3748f,0.8748f,  
	-0.2944f,0.3941f,0.8706f,  -0.1877f,0.3387f,0.9220f,  -0.1736f,0.3614f,0.9161f,  -0.0619f,0.3031f,0.9510f,  -0.0466f,0.3288f,0.9433f,  0.0676f,0.2677f,0.9611f,  0.0838f,0.2959f,0.9515f,  0.1978f,0.2321f,0.9524f,  0.2145f,0.2624f,0.9408f,  0.3259f,0.1959f,0.9249f,  0.3426f,0.2276f,0.9115f,  0.4488f,0.1586f,0.8794f,  0.4652f,0.1911f,0.8643f,  0.5639f,0.1196f,0.8171f,  0.5797f,0.1524f,0.8005f,  
	0.6685f,0.0783f,0.7396f,  0.6833f,0.1107f,0.7217f,  0.7602f,0.0343f,0.6487f,  0.7739f,0.0658f,0.6299f,  0.8371f,-0.0128f,0.5468f,  0.8493f,0.0170f,0.5276f,  0.8974f,-0.0636f,0.4366f,  0.9081f,-0.0360f,0.4173f,  0.9398f,-0.1181f,0.3207f,  0.9487f,-0.0933f,0.3020f,  0.9633f,-0.1765f,0.2023f,  0.9705f,-0.1550f,0.1849f,  0.9674f,-0.2388f,0.0846f,  0.9728f,-0.2210f,0.0691f,  0.9520f,-0.3046f,-0.0294f,  
	0.9558f,-0.2910f,-0.0422f,  0.9176f,-0.3734f,-0.1363f,  0.9198f,-0.3642f,-0.1457f,  0.8649f,-0.4444f,-0.2332f,  0.8658f,-0.4399f,-0.2384f,  0.7952f,-0.5167f,-0.3172f,  0.7951f,-0.5167f,-0.3176f,  0.7101f,-0.5889f,-0.3859f,  0.7093f,-0.5933f,-0.3807f,  0.5459f,-0.6743f,-0.4973f,  0.5597f,-0.6690f,-0.4891f,  0.6233f,-0.5664f,-0.5391f,  0.6313f,-0.5603f,-0.5362f,  0.6790f,-0.4475f,-0.5820f,  0.6807f,-0.4416f,-0.5845f,  
	0.7113f,-0.3211f,-0.6253f,  0.7064f,-0.3165f,-0.6331f,  0.7192f,-0.1911f,-0.6680f,  0.7077f,-0.1887f,-0.6809f,  0.7028f,-0.0609f,-0.7088f,  0.6849f,-0.0615f,-0.7261f,  0.6627f,0.0659f,-0.7459f,  0.6390f,0.0616f,-0.7668f,  0.6007f,0.1863f,-0.7774f,  0.5717f,0.1779f,-0.8009f,  0.5190f,0.2977f,-0.8013f,  0.4856f,0.2849f,-0.8264f,  0.4204f,0.3979f,-0.8154f,  0.3837f,0.3809f,-0.8412f,  0.3085f,0.4855f,-0.8180f,  
	0.2695f,0.4646f,-0.8435f,  0.1870f,0.5596f,-0.8074f,  0.1467f,0.5352f,-0.8319f,  0.0598f,0.6198f,-0.7825f,  0.0194f,0.5926f,-0.8052f,  -0.0691f,0.6662f,-0.7426f,  -0.1088f,0.6371f,-0.7631f,  -0.1957f,0.6993f,-0.6875f,  -0.2338f,0.6692f,-0.7054f,  -0.3164f,0.7202f,-0.6175f,  -0.3521f,0.6898f,-0.6325f,  -0.4277f,0.7298f,-0.5334f,  -0.4605f,0.7002f,-0.5456f,  -0.5266f,0.7294f,-0.4365f,  -0.5559f,0.7014f,-0.4461f,  
	-0.6105f,0.7206f,-0.3286f,  -0.6359f,0.6949f,-0.3357f,  -0.6772f,0.7047f,-0.2118f,  -0.6986f,0.6819f,-0.2167f,  -0.7251f,0.6829f,-0.0885f,  -0.7424f,0.6637f,-0.0915f,  -0.7531f,0.6567f,0.0387f,  -0.7663f,0.6414f,0.0372f,  -0.7607f,0.6272f,0.1671f,  -0.7698f,0.6162f,0.1665f,  -0.7477f,0.5955f,0.2938f,  -0.7529f,0.5889f,0.2937f,  -0.7147f,0.5623f,0.4160f,  -0.7162f,0.5604f,0.4160f,  -0.6625f,0.5284f,0.5309f,  
	-0.6606f,0.5311f,0.5306f,  -0.5925f,0.4944f,0.6360f,  -0.5875f,0.5016f,0.6350f,  -0.5063f,0.4606f,0.7290f,  -0.4986f,0.4721f,0.7270f,  -0.4062f,0.4271f,0.8078f,  -0.3961f,0.4428f,0.8044f,  -0.2944f,0.3941f,0.8706f,  -0.2823f,0.4136f,0.8656f,  -0.1736f,0.3614f,0.9161f,  -0.1600f,0.3843f,0.9092f,  -0.0466f,0.3288f,0.9433f,  -0.0317f,0.3547f,0.9344f,  0.0838f,0.2959f,0.9515f,  0.0994f,0.3243f,0.9407f,  
	0.2145f,0.2624f,0.9408f,  0.2305f,0.2928f,0.9280f,  0.3426f,0.2276f,0.9115f,  0.3586f,0.2595f,0.8967f,  0.4652f,0.1911f,0.8643f,  0.4809f,0.2238f,0.8477f,  0.5797f,0.1524f,0.8005f,  0.5947f,0.1853f,0.7823f,  0.6833f,0.1107f,0.7217f,  0.6974f,0.1433f,0.7022f,  0.7739f,0.0658f,0.6299f,  0.7868f,0.0972f,0.6096f,  0.8493f,0.0170f,0.5276f,  0.8608f,0.0468f,0.5068f,  0.9081f,-0.0360f,0.4173f,  
	0.9180f,-0.0084f,0.3966f,  0.9487f,-0.0933f,0.3020f,  0.9569f,-0.0686f,0.2821f,  0.9705f,-0.1550f,0.1849f,  0.9770f,-0.1338f,0.1663f,  0.9728f,-0.2210f,0.0691f,  0.9776f,-0.2036f,0.0526f,  0.9558f,-0.2910f,-0.0422f,  0.9590f,-0.2778f,-0.0557f,  0.9198f,-0.3642f,-0.1457f,  0.9217f,-0.3555f,-0.1555f,  0.8658f,-0.4399f,-0.2384f,  0.8665f,-0.4357f,-0.2438f,  0.7951f,-0.5167f,-0.3176f,  0.7948f,-0.5170f,-0.3177f,  
	0.7093f,-0.5933f,-0.3807f,  0.7085f,-0.5979f,-0.3750f,  0.5597f,-0.6690f,-0.4891f,  0.5728f,-0.6635f,-0.4812f,  0.6313f,-0.5603f,-0.5362f,  0.6385f,-0.5545f,-0.5338f,  0.6807f,-0.4416f,-0.5845f,  0.6814f,-0.4366f,-0.5875f,  0.7064f,-0.3165f,-0.6331f,  0.7005f,-0.3132f,-0.6413f,  0.7077f,-0.1887f,-0.6809f,  0.6952f,-0.1879f,-0.6938f,  0.6849f,-0.0615f,-0.7261f,  0.6662f,-0.0642f,-0.7430f,  0.6390f,0.0616f,-0.7668f,  
	0.6146f,0.0550f,-0.7869f,  0.5717f,0.1779f,-0.8009f,  0.5425f,0.1670f,-0.8233f,  0.4856f,0.2849f,-0.8264f,  0.4523f,0.2697f,-0.8501f,  0.3837f,0.3809f,-0.8412f,  0.3474f,0.3615f,-0.8652f,  0.2695f,0.4646f,-0.8435f,  0.2312f,0.4415f,-0.8670f,  0.1467f,0.5352f,-0.8319f,  0.1074f,0.5089f,-0.8541f,  0.0194f,0.5926f,-0.8052f,  -0.0199f,0.5639f,-0.8256f,  -0.1088f,0.6371f,-0.7631f,  -0.1472f,0.6068f,-0.7811f,  
	-0.2338f,0.6692f,-0.7054f,  -0.2705f,0.6381f,-0.7209f,  -0.3521f,0.6898f,-0.6325f,  -0.3864f,0.6589f,-0.6454f,  -0.4605f,0.7002f,-0.5456f,  -0.4918f,0.6702f,-0.5558f,  -0.5559f,0.7014f,-0.4461f,  -0.5838f,0.6733f,-0.4538f,  -0.6359f,0.6949f,-0.3357f,  -0.6602f,0.6692f,-0.3411f,  -0.6986f,0.6819f,-0.2167f,  -0.7189f,0.6593f,-0.2202f,  -0.7424f,0.6637f,-0.0915f,  -0.7587f,0.6447f,-0.0935f,  -0.7663f,0.6414f,0.0372f,  
	-0.7786f,0.6264f,0.0364f,  -0.7698f,0.6162f,0.1665f,  -0.7783f,0.6055f,0.1664f,  -0.7529f,0.5889f,0.2937f,  -0.7577f,0.5827f,0.2939f,  -0.7162f,0.5604f,0.4160f,  -0.7174f,0.5587f,0.4161f,  -0.6606f,0.5311f,0.5306f,  -0.6586f,0.5340f,0.5302f,  -0.5875f,0.5016f,0.6350f,  -0.5825f,0.5090f,0.6337f,  -0.4986f,0.4721f,0.7270f,  -0.4910f,0.4839f,0.7244f,  -0.3961f,0.4428f,0.8044f,  -0.3862f,0.4587f,0.8003f,  
	-0.2823f,0.4136f,0.8656f,  -0.2706f,0.4332f,0.8597f,  -0.1600f,0.3843f,0.9092f,  -0.1467f,0.4074f,0.9014f,  -0.0317f,0.3547f,0.9344f,  -0.0174f,0.3808f,0.9245f,  0.0994f,0.3243f,0.9407f,  0.1144f,0.3529f,0.9286f,  0.2305f,0.2928f,0.9280f,  0.2459f,0.3233f,0.9138f,  0.3586f,0.2595f,0.8967f,  0.3740f,0.2915f,0.8804f,  0.4809f,0.2238f,0.8477f,  0.4959f,0.2566f,0.8296f,  0.5947f,0.1853f,0.7823f,  
	0.6090f,0.2183f,0.7625f,  0.6974f,0.1433f,0.7022f,  0.7107f,0.1758f,0.6811f,  0.7868f,0.0972f,0.6096f,  0.7989f,0.1287f,0.5876f,  0.8608f,0.0468f,0.5068f,  0.8715f,0.0765f,0.4844f,  0.9180f,-0.0084f,0.3966f,  0.9271f,0.0189f,0.3744f,  0.9569f,-0.0686f,0.2821f,  0.9644f,-0.0442f,0.2607f,  0.9770f,-0.1338f,0.1663f,  0.9828f,-0.1128f,0.1465f,  0.9776f,-0.2036f,0.0526f,  0.9818f,-0.1866f,0.0352f,  
	0.9590f,-0.2778f,-0.0557f,  0.9617f,-0.2650f,-0.0700f,  0.9217f,-0.3555f,-0.1555f,  0.9230f,-0.3472f,-0.1658f,  0.8665f,-0.4357f,-0.2438f,  0.8668f,-0.4319f,-0.2492f,  0.7948f,-0.5170f,-0.3177f,  0.7944f,-0.5177f,-0.3175f,  0.7085f,-0.5979f,-0.3750f,  0.7078f,-0.6027f,-0.3686f,  0.5728f,-0.6635f,-0.4812f,  0.5853f,-0.6579f,-0.4739f,  0.6385f,-0.5545f,-0.5338f,  0.6448f,-0.5491f,-0.5318f,  0.6814f,-0.4366f,-0.5875f,  
	0.6812f,-0.4323f,-0.5908f,  0.7005f,-0.3132f,-0.6413f,  0.6937f,-0.3111f,-0.6496f,  0.6952f,-0.1879f,-0.6938f,  0.6820f,-0.1889f,-0.7065f,  0.6662f,-0.0642f,-0.7430f,  0.6470f,-0.0688f,-0.7594f,  0.6146f,0.0550f,-0.7869f,  0.5900f,0.0462f,-0.8061f,  0.5425f,0.1670f,-0.8233f,  0.5132f,0.1539f,-0.8444f,  0.4523f,0.2697f,-0.8501f,  0.4193f,0.2523f,-0.8721f,  0.3474f,0.3615f,-0.8652f,  0.3116f,0.3402f,-0.8872f,  
	0.2312f,0.4415f,-0.8670f,  0.1937f,0.4165f,-0.8882f,  0.1074f,0.5089f,-0.8541f,  0.0692f,0.4811f,-0.8739f,  -0.0199f,0.5639f,-0.8256f,  -0.0580f,0.5339f,-0.8435f,  -0.1472f,0.6068f,-0.7811f,  -0.1842f,0.5755f,-0.7968f,  -0.2705f,0.6381f,-0.7209f,  -0.3057f,0.6064f,-0.7341f,  -0.3864f,0.6589f,-0.6454f,  -0.4192f,0.6275f,-0.6561f,  -0.4918f,0.6702f,-0.5558f,  -0.5216f,0.6401f,-0.5641f,  -0.5838f,0.6733f,-0.4538f,  
	-0.6104f,0.6450f,-0.4598f,  -0.6602f,0.6692f,-0.3411f,  -0.6832f,0.6436f,-0.3451f,  -0.7189f,0.6593f,-0.2202f,  -0.7382f,0.6368f,-0.2225f,  -0.7587f,0.6447f,-0.0935f,  -0.7742f,0.6259f,-0.0944f,  -0.7786f,0.6264f,0.0364f,  -0.7903f,0.6117f,0.0363f,  -0.7783f,0.6055f,0.1664f,  -0.7862f,0.5950f,0.1668f,  -0.7577f,0.5827f,0.2939f,  -0.7620f,0.5767f,0.2944f,  -0.7174f,0.5587f,0.4161f,  -0.7184f,0.5573f,0.4162f,  
	-0.6586f,0.5340f,0.5302f,  -0.6564f,0.5373f,0.5296f,  -0.5825f,0.5090f,0.6337f,  -0.5775f,0.5167f,0.6320f,  -0.4910f,0.4839f,0.7244f,  -0.4835f,0.4959f,0.7213f,  -0.3862f,0.4587f,0.8003f,  -0.3766f,0.4748f,0.7955f,  -0.2706f,0.4332f,0.8597f,  -0.2592f,0.4531f,0.8529f,  -0.1467f,0.4074f,0.9014f,  -0.1339f,0.4307f,0.8925f,  -0.0174f,0.3808f,0.9245f,  -0.0036f,0.4070f,0.9134f,  0.1144f,0.3529f,0.9286f,  
	0.1289f,0.3817f,0.9152f,  0.2459f,0.3233f,0.9138f,  0.2607f,0.3541f,0.8981f,  0.3740f,0.2915f,0.8804f,  0.3887f,0.3236f,0.8627f,  0.4959f,0.2566f,0.8296f,  0.5102f,0.2896f,0.8098f,  0.6090f,0.2183f,0.7625f,  0.6226f,0.2514f,0.7411f,  0.7107f,0.1758f,0.6811f,  0.7233f,0.2085f,0.6583f,  0.7989f,0.1287f,0.5876f,  0.8102f,0.1602f,0.5639f,  0.8715f,0.0765f,0.4844f,  0.8814f,0.1062f,0.4603f,  
	0.9271f,0.0189f,0.3744f,  0.9354f,0.0461f,0.3506f,  0.9644f,-0.0442f,0.2607f,  0.9711f,-0.0201f,0.2379f,  0.9828f,-0.1128f,0.1465f,  0.9878f,-0.0923f,0.1255f,  0.9818f,-0.1866f,0.0352f,  0.9853f,-0.1701f,0.0167f,  0.9617f,-0.2650f,-0.0700f,  0.9638f,-0.2528f,-0.0850f,  0.9230f,-0.3472f,-0.1658f,  0.9239f,-0.3395f,-0.1765f,  0.8668f,-0.4319f,-0.2492f,  0.8668f,-0.4287f,-0.2547f,  0.7944f,-0.5177f,-0.3175f,  
	0.7939f,-0.5188f,-0.3171f,  0.7078f,-0.6027f,-0.3686f,  0.7071f,-0.6077f,-0.3616f,  0.5853f,-0.6579f,-0.4739f,  0.5971f,-0.6522f,-0.4670f,  0.6448f,-0.5491f,-0.5318f,  0.6503f,-0.5441f,-0.5302f,  0.6812f,-0.4323f,-0.5908f,  0.6802f,-0.4290f,-0.5944f,  0.6937f,-0.3111f,-0.6496f,  0.6861f,-0.3103f,-0.6580f,  0.6820f,-0.1889f,-0.7065f,  0.6682f,-0.1915f,-0.7189f,  0.6470f,-0.0688f,-0.7594f,  0.6273f,-0.0753f,-0.7751f,  
	0.5900f,0.0462f,-0.8061f,  0.5652f,0.0355f,-0.8242f,  0.5132f,0.1539f,-0.8444f,  0.4840f,0.1388f,-0.8640f,  0.4193f,0.2523f,-0.8721f,  0.3868f,0.2330f,-0.8923f,  0.3116f,0.3402f,-0.8872f,  0.2766f,0.3170f,-0.9072f,  0.1937f,0.4165f,-0.8882f,  0.1572f,0.3900f,-0.9073f,  0.0692f,0.4811f,-0.8739f,  0.0322f,0.4520f,-0.8914f,  -0.0580f,0.5339f,-0.8435f,  -0.0947f,0.5029f,-0.8591f,  -0.1842f,0.5755f,-0.7968f,  
	-0.2197f,0.5434f,-0.8102f,  -0.3057f,0.6064f,-0.7341f,  -0.3394f,0.5741f,-0.7451f,  -0.4192f,0.6275f,-0.6561f,  -0.4505f,0.5959f,-0.6648f,  -0.5216f,0.6401f,-0.5641f,  -0.5501f,0.6098f,-0.5705f,  -0.6104f,0.6450f,-0.4598f,  -0.6357f,0.6168f,-0.4641f,  -0.6832f,0.6436f,-0.3451f,  -0.7050f,0.6181f,-0.3477f,  -0.7382f,0.6368f,-0.2225f,  -0.7565f,0.6146f,-0.2236f,  -0.7742f,0.6259f,-0.0944f,  -0.7888f,0.6074f,-0.0945f,  
	-0.7903f,0.6117f,0.0363f,  -0.8012f,0.5972f,0.0369f,  -0.7862f,0.5950f,0.1668f,  -0.7936f,0.5849f,0.1677f,  -0.7620f,0.5767f,0.2944f,  -0.7660f,0.5711f,0.2951f,  -0.7184f,0.5573f,0.4162f,  -0.7192f,0.5563f,0.4164f,  -0.6564f,0.5373f,0.5296f,  -0.6542f,0.5407f,0.5288f,  -0.5775f,0.5167f,0.6320f,  -0.5726f,0.5247f,0.6299f,  -0.4835f,0.4959f,0.7213f,  -0.4762f,0.5082f,0.7176f,  -0.3766f,0.4748f,0.7955f,  
	-0.3673f,0.4911f,0.7899f,  -0.2592f,0.4531f,0.8529f,  -0.2481f,0.4733f,0.8453f,  -0.1339f,0.4307f,0.8925f,  -0.1215f,0.4542f,0.8826f,  -0.0036f,0.4070f,0.9134f,  0.0098f,0.4336f,0.9011f,  0.1289f,0.3817f,0.9152f,  0.1429f,0.4107f,0.9005f,  0.2607f,0.3541f,0.8981f,  0.2748f,0.3851f,0.8810f,  0.3887f,0.3236f,0.8627f,  0.4027f,0.3560f,0.8433f,  0.5102f,0.2896f,0.8098f,  0.5238f,0.3227f,0.7884f,  
	0.6226f,0.2514f,0.7411f,  0.6353f,0.2846f,0.7179f,  0.7233f,0.2085f,0.6583f,  0.7350f,0.2411f,0.6338f,  0.8102f,0.1602f,0.5639f,  0.8206f,0.1916f,0.5384f,  0.8814f,0.1062f,0.4603f,  0.8904f,0.1356f,0.4346f,  0.9354f,0.0461f,0.3506f,  0.9428f,0.0730f,0.3252f,  0.9711f,-0.0201f,0.2379f,  0.9769f,0.0036f,0.2136f,  0.9878f,-0.0923f,0.1255f,  0.9920f,-0.0722f,0.1031f,  0.9853f,-0.1701f,0.0167f,  
	0.9880f,-0.1541f,-0.0028f,  0.9638f,-0.2528f,-0.0850f,  0.9652f,-0.2412f,-0.1007f,  0.9239f,-0.3395f,-0.1765f,  0.9243f,-0.3324f,-0.1875f,  0.8668f,-0.4287f,-0.2547f,  0.8665f,-0.4260f,-0.2602f,  0.7939f,-0.5188f,-0.3171f,  0.7933f,-0.5202f,-0.3163f,  0.7071f,-0.6077f,-0.3616f,  0.7065f,-0.6128f,-0.3539f,  0.5971f,-0.6522f,-0.4670f,  0.6082f,-0.6465f,-0.4606f,  0.6503f,-0.5441f,-0.5302f,  0.6551f,-0.5394f,-0.5290f,  
	0.6802f,-0.4290f,-0.5944f,  0.6785f,-0.4265f,-0.5982f,  0.6861f,-0.3103f,-0.6580f,  0.6780f,-0.3108f,-0.6661f,  0.6682f,-0.1915f,-0.7189f,  0.6539f,-0.1956f,-0.7308f,  0.6273f,-0.0753f,-0.7751f,  0.6075f,-0.0836f,-0.7899f,  0.5652f,0.0355f,-0.8242f,  0.5404f,0.0229f,-0.8411f,  0.4840f,0.1388f,-0.8640f,  0.4552f,0.1219f,-0.8820f,  0.3868f,0.2330f,-0.8923f,  0.3548f,0.2120f,-0.9106f,  0.2766f,0.3170f,-0.9072f,  
	0.2425f,0.2923f,-0.9251f,  0.1572f,0.3900f,-0.9073f,  0.1218f,0.3623f,-0.9241f,  0.0322f,0.4520f,-0.8914f,  -0.0036f,0.4218f,-0.9067f,  -0.0947f,0.5029f,-0.8591f,  -0.1300f,0.4712f,-0.8724f,  -0.2197f,0.5434f,-0.8102f,  -0.2538f,0.5108f,-0.8214f,  -0.3394f,0.5741f,-0.7451f,  -0.3717f,0.5415f,-0.7541f,  -0.4505f,0.5959f,-0.6648f,  -0.4805f,0.5641f,-0.6715f,  -0.5501f,0.6098f,-0.5705f,  -0.5773f,0.5795f,-0.5752f,  
	-0.6357f,0.6168f,-0.4641f,  -0.6598f,0.5888f,-0.4670f,  -0.7050f,0.6181f,-0.3477f,  -0.7258f,0.5928f,-0.3490f,  -0.7565f,0.6146f,-0.2236f,  -0.7738f,0.5926f,-0.2237f,  -0.7888f,0.6074f,-0.0945f,  -0.8026f,0.5891f,-0.0937f,  -0.8012f,0.5972f,0.0369f,  -0.8115f,0.5831f,0.0381f,  -0.7936f,0.5849f,0.1677f,  -0.8004f,0.5751f,0.1690f,  -0.7660f,0.5711f,0.2951f,  -0.7696f,0.5658f,0.2961f,  -0.7192f,0.5563f,0.4164f,  
	-0.7197f,0.5555f,0.4165f,  -0.6542f,0.5407f,0.5288f,  -0.6519f,0.5445f,0.5278f,  -0.5726f,0.5247f,0.6299f,  -0.5677f,0.5329f,0.6275f,  -0.4762f,0.5082f,0.7176f,  -0.4690f,0.5207f,0.7133f,  -0.3673f,0.4911f,0.7899f,  -0.3581f,0.5077f,0.7836f,  -0.2481f,0.4733f,0.8453f,  -0.2374f,0.4936f,0.8366f,  -0.1215f,0.4542f,0.8826f,  -0.1095f,0.4780f,0.8715f,  0.0098f,0.4336f,0.9011f,  0.0227f,0.4603f,0.8875f,  
	0.1429f,0.4107f,0.9005f,  0.1562f,0.4400f,0.8843f,  0.2748f,0.3851f,0.8810f,  0.2883f,0.4163f,0.8623f,  0.4027f,0.3560f,0.8433f,  0.4160f,0.3885f,0.8222f,  0.5238f,0.3227f,0.7884f,  0.5365f,0.3560f,0.7651f,  0.6353f,0.2846f,0.7179f,  0.6473f,0.3179f,0.6928f,  0.7350f,0.2411f,0.6338f,  0.7459f,0.2737f,0.6073f,  0.8206f,0.1916f,0.5384f,  0.8302f,0.2228f,0.5111f,  0.8904f,0.1356f,0.4346f,  
	0.8985f,0.1648f,0.4070f,  0.9428f,0.0730f,0.3252f,  0.9493f,0.0995f,0.2981f,  0.9769f,0.0036f,0.2136f,  0.9818f,0.0268f,0.1878f,  0.9920f,-0.0722f,0.1031f,  0.9954f,-0.0528f,0.0795f,  0.9880f,-0.1541f,-0.0028f,  0.9900f,-0.1389f,-0.0232f,  0.9652f,-0.2412f,-0.1007f,  0.9660f,-0.2304f,-0.1171f,  0.9243f,-0.3324f,-0.1875f,  0.9242f,-0.3260f,-0.1988f,  0.8665f,-0.4260f,-0.2602f,  0.8658f,-0.4239f,-0.2657f,  
	0.7933f,-0.5202f,-0.3163f,  0.7925f,-0.5222f,-0.3152f,  0.7065f,-0.6128f,-0.3539f,  0.7060f,-0.6182f,-0.3456f,  0.6082f,-0.6465f,-0.4606f,  0.6187f,-0.6407f,-0.4547f,  0.6551f,-0.5394f,-0.5290f,  0.6592f,-0.5353f,-0.5282f,  0.6785f,-0.4265f,-0.5982f,  0.6761f,-0.4248f,-0.6020f,  0.6780f,-0.3108f,-0.6661f,  0.6693f,-0.3125f,-0.6741f,  0.6539f,-0.1956f,-0.7308f,  0.6393f,-0.2012f,-0.7422f,  0.6075f,-0.0836f,-0.7899f,  
	0.5875f,-0.0934f,-0.8038f,  0.5404f,0.0229f,-0.8411f,  0.5159f,0.0087f,-0.8566f,  0.4552f,0.1219f,-0.8820f,  0.4269f,0.1034f,-0.8984f,  0.3548f,0.2120f,-0.9106f,  0.3237f,0.1896f,-0.9270f,  0.2425f,0.2923f,-0.9251f,  0.2094f,0.2664f,-0.9409f,  0.1218f,0.3623f,-0.9241f,  0.0876f,0.3335f,-0.9387f,  -0.0036f,0.4218f,-0.9067f,  -0.0380f,0.3908f,-0.9197f,  -0.1300f,0.4712f,-0.8724f,  -0.1638f,0.4388f,-0.8835f,  
	-0.2538f,0.5108f,-0.8214f,  -0.2865f,0.4779f,-0.8304f,  -0.3717f,0.5415f,-0.7541f,  -0.4026f,0.5087f,-0.7610f,  -0.4805f,0.5641f,-0.6715f,  -0.5091f,0.5323f,-0.6764f,  -0.5773f,0.5795f,-0.5752f,  -0.6032f,0.5493f,-0.5783f,  -0.6598f,0.5888f,-0.4670f,  -0.6827f,0.5609f,-0.4684f,  -0.7258f,0.5928f,-0.3490f,  -0.7455f,0.5678f,-0.3490f,  -0.7738f,0.5926f,-0.2237f,  -0.7902f,0.5710f,-0.2227f,  -0.8026f,0.5891f,-0.0937f,  
	-0.8156f,0.5712f,-0.0921f,  -0.8115f,0.5831f,0.0381f,  -0.8212f,0.5692f,0.0399f,  -0.8004f,0.5751f,0.1690f,  -0.8069f,0.5656f,0.1706f,  -0.7696f,0.5658f,0.2961f,  -0.7729f,0.5607f,0.2971f,  -0.7197f,0.5555f,0.4165f,  -0.7200f,0.5550f,0.4167f,  -0.6519f,0.5445f,0.5278f,  -0.6495f,0.5485f,0.5266f,  -0.5677f,0.5329f,0.6275f,  -0.5628f,0.5414f,0.6246f,  -0.4690f,0.5207f,0.7133f,  -0.4620f,0.5335f,0.7084f,  
	-0.3581f,0.5077f,0.7836f,  -0.3492f,0.5246f,0.7764f,  -0.2374f,0.4936f,0.8366f,  -0.2270f,0.5143f,0.8270f,  -0.1095f,0.4780f,0.8715f,  -0.0980f,0.5021f,0.8593f,  0.0227f,0.4603f,0.8875f,  0.0350f,0.4874f,0.8725f,  0.1562f,0.4400f,0.8843f,  0.1690f,0.4695f,0.8666f,  0.2883f,0.4163f,0.8623f,  0.3012f,0.4477f,0.8419f,  0.4160f,0.3885f,0.8222f,  0.4286f,0.4212f,0.7993f,  0.5365f,0.3560f,0.7651f,  
	0.5485f,0.3893f,0.7400f,  0.6473f,0.3179f,0.6928f,  0.6583f,0.3512f,0.6658f,  0.7459f,0.2737f,0.6073f,  0.7558f,0.3062f,0.5788f,  0.8302f,0.2228f,0.5111f,  0.8387f,0.2538f,0.4817f,  0.8985f,0.1648f,0.4070f,  0.9055f,0.1936f,0.3775f,  0.9493f,0.0995f,0.2981f,  0.9549f,0.1255f,0.2692f,  0.9818f,0.0268f,0.1878f,  0.9858f,0.0494f,0.1605f,  0.9954f,-0.0528f,0.0795f,  0.9979f,-0.0342f,0.0546f,  
	0.9900f,-0.1389f,-0.0232f,  0.9912f,-0.1245f,-0.0447f,  0.9660f,-0.2304f,-0.1171f,  0.9661f,-0.2204f,-0.1341f,  0.9242f,-0.3260f,-0.1988f,  0.9236f,-0.3204f,-0.2105f,  0.8658f,-0.4239f,-0.2657f,  0.8649f,-0.4226f,-0.2710f,  0.7925f,-0.5222f,-0.3152f,  0.7916f,-0.5245f,-0.3136f,  0.7060f,-0.6182f,-0.3456f,  0.7056f,-0.6236f,-0.3365f,  0.6187f,-0.6407f,-0.4547f,  0.6286f,-0.6349f,-0.4492f,  0.6592f,-0.5353f,-0.5282f,  
	0.6626f,-0.5316f,-0.5276f,  0.6761f,-0.4248f,-0.6020f,  0.6731f,-0.4241f,-0.6059f,  0.6693f,-0.3125f,-0.6741f,  0.6602f,-0.3153f,-0.6817f,  0.6393f,-0.2012f,-0.7422f,  0.6245f,-0.2081f,-0.7528f,  0.5875f,-0.0934f,-0.8038f,  0.5676f,-0.1047f,-0.8166f,  0.5159f,0.0087f,-0.8566f,  0.4917f,-0.0069f,-0.8708f,  0.4269f,0.1034f,-0.8984f,  0.3992f,0.0836f,-0.9130f,  0.3237f,0.1896f,-0.9270f,  0.2933f,0.1659f,-0.9415f,  
	0.2094f,0.2664f,-0.9409f,  0.1773f,0.2394f,-0.9546f,  0.0876f,0.3335f,-0.9387f,  0.0547f,0.3038f,-0.9512f,  -0.0380f,0.3908f,-0.9197f,  -0.0711f,0.3593f,-0.9305f,  -0.1638f,0.4388f,-0.8835f,  -0.1963f,0.4060f,-0.8925f,  -0.2865f,0.4779f,-0.8304f,  -0.3177f,0.4447f,-0.8374f,  -0.4026f,0.5087f,-0.7610f,  -0.4320f,0.4759f,-0.7661f,  -0.5091f,0.5323f,-0.6764f,  -0.5363f,0.5005f,-0.6796f,  -0.6032f,0.5493f,-0.5783f,  
	-0.6279f,0.5193f,-0.5797f,  -0.6827f,0.5609f,-0.4684f,  -0.7045f,0.5332f,-0.4684f,  -0.7455f,0.5678f,-0.3490f,  -0.7642f,0.5430f,-0.3479f,  -0.7902f,0.5710f,-0.2227f,  -0.8057f,0.5496f,-0.2208f,  -0.8156f,0.5712f,-0.0921f,  -0.8279f,0.5536f,-0.0898f,  -0.8212f,0.5692f,0.0399f,  -0.8303f,0.5557f,0.0423f,  -0.8069f,0.5656f,0.1706f,  -0.8128f,0.5563f,0.1726f,  -0.7729f,0.5607f,0.2971f,  -0.7758f,0.5559f,0.2984f,  
	-0.7200f,0.5550f,0.4167f,  -0.7201f,0.5548f,0.4167f,  -0.6495f,0.5485f,0.5266f,  -0.6470f,0.5529f,0.5251f,  -0.5628f,0.5414f,0.6246f,  -0.5580f,0.5502f,0.6212f,  -0.4620f,0.5335f,0.7084f,  -0.4551f,0.5466f,0.7029f,  -0.3492f,0.5246f,0.7764f,  -0.3406f,0.5418f,0.7684f,  -0.2270f,0.5143f,0.8270f,  -0.2169f,0.5352f,0.8164f,  -0.0980f,0.5021f,0.8593f,  -0.0869f,0.5264f,0.8458f,  0.0350f,0.4874f,0.8725f,  
	0.0468f,0.5146f,0.8561f,  0.1690f,0.4695f,0.8666f,  0.1812f,0.4992f,0.8473f,  0.3012f,0.4477f,0.8419f,  0.3133f,0.4793f,0.8198f,  0.4286f,0.4212f,0.7993f,  0.4404f,0.4540f,0.7746f,  0.5485f,0.3893f,0.7400f,  0.5596f,0.4226f,0.7129f,  0.6583f,0.3512f,0.6658f,  0.6685f,0.3844f,0.6367f,  0.7558f,0.3062f,0.5788f,  0.7647f,0.3385f,0.5483f,  0.8387f,0.2538f,0.4817f,  0.8463f,0.2845f,0.4504f,  
	0.9055f,0.1936f,0.3775f,  0.9116f,0.2219f,0.3461f,  0.9549f,0.1255f,0.2692f,  0.9593f,0.1507f,0.2386f,  0.9858f,0.0494f,0.1605f,  0.9888f,0.0711f,0.1316f,  0.9979f,-0.0342f,0.0546f,  0.9995f,-0.0165f,0.0285f,  0.9912f,-0.1245f,-0.0447f,  0.9915f,-0.1111f,-0.0670f,  0.9661f,-0.2204f,-0.1341f,  0.9656f,-0.2114f,-0.1516f,  0.9236f,-0.3204f,-0.2105f,  0.9224f,-0.3158f,-0.2222f,  0.8649f,-0.4226f,-0.2710f,  
	0.8635f,-0.4219f,-0.2762f,  0.7916f,-0.5245f,-0.3136f,  0.7905f,-0.5273f,-0.3114f,  0.7056f,-0.6236f,-0.3365f,  0.7052f,-0.6292f,-0.3267f,  0.6286f,-0.6349f,-0.4492f,  0.6378f,-0.6292f,-0.4442f,  0.6626f,-0.5316f,-0.5276f,  0.6655f,-0.5283f,-0.5273f,  0.6731f,-0.4241f,-0.6059f,  0.6697f,-0.4241f,-0.6096f,  0.6602f,-0.3153f,-0.6817f,  0.6508f,-0.3192f,-0.6889f,  0.6245f,-0.2081f,-0.7528f,  0.6097f,-0.2162f,-0.7626f,  
	0.5676f,-0.1047f,-0.8166f,  0.5480f,-0.1172f,-0.8282f,  0.4917f,-0.0069f,-0.8708f,  0.4679f,-0.0238f,-0.8834f,  0.3992f,0.0836f,-0.9130f,  0.3722f,0.0626f,-0.9260f,  0.2933f,0.1659f,-0.9415f,  0.2639f,0.1412f,-0.9541f,  0.1773f,0.2394f,-0.9546f,  0.1464f,0.2116f,-0.9663f,  0.0547f,0.3038f,-0.9512f,  0.0229f,0.2736f,-0.9616f,  -0.0711f,0.3593f,-0.9305f,  -0.1028f,0.3273f,-0.9393f,  -0.1963f,0.4060f,-0.8925f,  
	-0.2274f,0.3730f,-0.8995f,  -0.3177f,0.4447f,-0.8374f,  -0.3476f,0.4114f,-0.8426f,  -0.4320f,0.4759f,-0.7661f,  -0.4602f,0.4431f,-0.7694f,  -0.5363f,0.5005f,-0.6796f,  -0.5623f,0.4688f,-0.6812f,  -0.6279f,0.5193f,-0.5797f,  -0.6514f,0.4894f,-0.5798f,  -0.7045f,0.5332f,-0.4684f,  -0.7252f,0.5058f,-0.4672f,  -0.7642f,0.5430f,-0.3479f,  -0.7820f,0.5185f,-0.3457f,  -0.8057f,0.5496f,-0.2208f,  -0.8205f,0.5285f,-0.2180f,  
	-0.8279f,0.5536f,-0.0898f,  -0.8396f,0.5363f,-0.0868f,  -0.8303f,0.5557f,0.0423f,  -0.8389f,0.5424f,0.0452f,  -0.8128f,0.5563f,0.1726f,  -0.8184f,0.5474f,0.1749f,  -0.7758f,0.5559f,0.2984f,  -0.7785f,0.5515f,0.2997f,  -0.7201f,0.5548f,0.4167f,  -0.7201f,0.5548f,0.4167f,  -0.6470f,0.5529f,0.5251f,  -0.6445f,0.5575f,0.5234f,  -0.5580f,0.5502f,0.6212f,  -0.5532f,0.5593f,0.6174f,  -0.4551f,0.5466f,0.7029f,  
	-0.4484f,0.5600f,0.6967f,  -0.3406f,0.5418f,0.7684f,  -0.3322f,0.5592f,0.7595f,  -0.2169f,0.5352f,0.8164f,  -0.2072f,0.5565f,0.8046f,  -0.0869f,0.5264f,0.8458f,  -0.0762f,0.5510f,0.8310f,  0.0468f,0.5146f,0.8561f,  0.0581f,0.5422f,0.8383f,  0.1812f,0.4992f,0.8473f,  0.1927f,0.5291f,0.8264f,  0.3133f,0.4793f,0.8198f,  0.3247f,0.5110f,0.7959f,  0.4404f,0.4540f,0.7746f,  0.4513f,0.4869f,0.7478f,  
	0.5596f,0.4226f,0.7129f,  0.5698f,0.4559f,0.6837f,  0.6685f,0.3844f,0.6367f,  0.6776f,0.4174f,0.6055f,  0.7647f,0.3385f,0.5483f,  0.7726f,0.3704f,0.5156f,  0.8463f,0.2845f,0.4504f,  0.8527f,0.3146f,0.4170f,  0.9116f,0.2219f,0.3461f,  0.9165f,0.2495f,0.3127f,  0.9593f,0.1507f,0.2386f,  0.9627f,0.1752f,0.2062f,  0.9888f,0.0711f,0.1316f,  0.9906f,0.0918f,0.1012f,  0.9995f,-0.0165f,0.0285f,  
	1.0000f,0.0001f,0.0012f,  0.9915f,-0.1111f,-0.0670f,  0.9910f,-0.0988f,-0.0902f,  0.9656f,-0.2114f,-0.1516f,  0.9642f,-0.2036f,-0.1696f,  0.9224f,-0.3158f,-0.2222f,  0.9208f,-0.3121f,-0.2341f,  0.8635f,-0.4219f,-0.2762f,  0.8619f,-0.4220f,-0.2811f,  0.7905f,-0.5273f,-0.3114f,  0.7894f,-0.5306f,-0.3088f,  0.7052f,-0.6292f,-0.3267f,  0.7050f,-0.6349f,-0.3161f,  0.6378f,-0.6292f,-0.4442f,  0.6464f,-0.6236f,-0.4397f,  
	0.6655f,-0.5283f,-0.5273f,  0.6678f,-0.5256f,-0.5271f,  0.6697f,-0.4241f,-0.6096f,  0.6659f,-0.4249f,-0.6132f,  0.6508f,-0.3192f,-0.6889f,  0.6412f,-0.3241f,-0.6956f,  0.6097f,-0.2162f,-0.7626f,  0.5949f,-0.2254f,-0.7716f,  0.5480f,-0.1172f,-0.8282f,  0.5286f,-0.1308f,-0.8387f,  0.4679f,-0.0238f,-0.8834f,  0.4448f,-0.0418f,-0.8947f,  0.3722f,0.0626f,-0.9260f,  0.3460f,0.0407f,-0.9373f,  0.2639f,0.1412f,-0.9541f,  
	0.2355f,0.1158f,-0.9649f,  0.1464f,0.2116f,-0.9663f,  0.1165f,0.1832f,-0.9761f,  0.0229f,0.2736f,-0.9616f,  -0.0075f,0.2429f,-0.9700f,  -0.1028f,0.3273f,-0.9393f,  -0.1333f,0.2949f,-0.9462f,  -0.2274f,0.3730f,-0.8995f,  -0.2572f,0.3398f,-0.9046f,  -0.3476f,0.4114f,-0.8426f,  -0.3762f,0.3781f,-0.8459f,  -0.4602f,0.4431f,-0.7694f,  -0.4871f,0.4103f,-0.7710f,  -0.5623f,0.4688f,-0.6812f,  -0.5871f,0.4373f,-0.6812f,  
	-0.6514f,0.4894f,-0.5798f,  -0.6738f,0.4598f,-0.5784f,  -0.7252f,0.5058f,-0.4672f,  -0.7450f,0.4786f,-0.4648f,  -0.7820f,0.5185f,-0.3457f,  -0.7990f,0.4943f,-0.3425f,  -0.8205f,0.5285f,-0.2180f,  -0.8345f,0.5077f,-0.2144f,  -0.8396f,0.5363f,-0.0868f,  -0.8506f,0.5192f,-0.0831f,  -0.8389f,0.5424f,0.0452f,  -0.8469f,0.5295f,0.0485f,  -0.8184f,0.5474f,0.1749f,  -0.8235f,0.5388f,0.1775f,  -0.7785f,0.5515f,0.2997f,  
	-0.7809f,0.5473f,0.3011f,  -0.7201f,0.5548f,0.4167f,  -0.7199f,0.5552f,0.4165f,  -0.6445f,0.5575f,0.5234f,  -0.6419f,0.5624f,0.5213f,  -0.5532f,0.5593f,0.6174f,  -0.5485f,0.5687f,0.6130f,  -0.4484f,0.5600f,0.6967f,  -0.4418f,0.5737f,0.6897f,  -0.3322f,0.5592f,0.7595f,  -0.3241f,0.5770f,0.7497f,  -0.2072f,0.5565f,0.8046f,  -0.1979f,0.5780f,0.7917f,  -0.0762f,0.5510f,0.8310f,  -0.0660f,0.5759f,0.8148f,  
	0.0581f,0.5422f,0.8383f,  0.0687f,0.5700f,0.8188f,  0.1927f,0.5291f,0.8264f,  0.2035f,0.5592f,0.8036f,  0.3247f,0.5110f,0.7959f,  0.3353f,0.5428f,0.7700f,  0.4513f,0.4869f,0.7478f,  0.4613f,0.5198f,0.7190f,  0.5698f,0.4559f,0.6837f,  0.5790f,0.4891f,0.6524f,  0.6776f,0.4174f,0.6055f,  0.6857f,0.4501f,0.5721f,  0.7726f,0.3704f,0.5156f,  0.7793f,0.4019f,0.4808f,  0.8527f,0.3146f,0.4170f,  
	0.8580f,0.3440f,0.3815f,  0.9165f,0.2495f,0.3127f,  0.9202f,0.2762f,0.2774f,  0.9627f,0.1752f,0.2062f,  0.9649f,0.1985f,0.1721f,  0.9906f,0.0918f,0.1012f,  0.9914f,0.1113f,0.0693f,  1.0000f,0.0001f,0.0012f,  0.9995f,0.0154f,-0.0273f,  0.9910f,-0.0988f,-0.0902f,  0.9896f,-0.0879f,-0.1141f,  0.9642f,-0.2036f,-0.1696f,  0.9622f,-0.1970f,-0.1879f,  0.9208f,-0.3121f,-0.2341f,  0.9186f,-0.3095f,-0.2458f,  
	0.8619f,-0.4220f,-0.2811f,  0.8600f,-0.4230f,-0.2855f,  0.7894f,-0.5306f,-0.3088f,  0.7881f,-0.5344f,-0.3054f,  0.7050f,-0.6349f,-0.3161f,  0.7048f,-0.6406f,-0.3048f,  0.6464f,-0.6236f,-0.4397f,  0.6544f,-0.6181f,-0.4356f,  0.6678f,-0.5256f,-0.5271f,  0.6696f,-0.5233f,-0.5271f,  0.6659f,-0.4249f,-0.6132f,  0.6618f,-0.4264f,-0.6167f,  0.6412f,-0.3241f,-0.6956f,  0.6315f,-0.3298f,-0.7017f,  0.5949f,-0.2254f,-0.7716f,  
	0.5802f,-0.2356f,-0.7797f,  0.5286f,-0.1308f,-0.8387f,  0.5096f,-0.1454f,-0.8480f,  0.4448f,-0.0418f,-0.8947f,  0.4222f,-0.0606f,-0.9045f,  0.3460f,0.0407f,-0.9373f,  0.3207f,0.0180f,-0.9470f,  0.2355f,0.1158f,-0.9649f,  0.2082f,0.0897f,-0.9740f,  0.1165f,0.1832f,-0.9761f,  0.0879f,0.1543f,-0.9841f,  -0.0075f,0.2429f,-0.9700f,  -0.0368f,0.2118f,-0.9766f,  -0.1333f,0.2949f,-0.9462f,  -0.1624f,0.2624f,-0.9512f,  
	-0.2572f,0.3398f,-0.9046f,  -0.2857f,0.3066f,-0.9079f,  -0.3762f,0.3781f,-0.8459f,  -0.4035f,0.3448f,-0.8475f,  -0.4871f,0.4103f,-0.7710f,  -0.5128f,0.3777f,-0.7710f,  -0.5871f,0.4373f,-0.6812f,  -0.6108f,0.4060f,-0.6798f,  -0.6738f,0.4598f,-0.5784f,  -0.6951f,0.4304f,-0.5758f,  -0.7450f,0.4786f,-0.4648f,  -0.7638f,0.4517f,-0.4612f,  -0.7990f,0.4943f,-0.3425f,  -0.8150f,0.4704f,-0.3383f,  -0.8345f,0.5077f,-0.2144f,  
	-0.8477f,0.4871f,-0.2099f,  -0.8506f,0.5192f,-0.0831f,  -0.8610f,0.5025f,-0.0788f,  -0.8469f,0.5295f,0.0485f,  -0.8545f,0.5168f,0.0523f,  -0.8235f,0.5388f,0.1775f,  -0.8283f,0.5304f,0.1804f,  -0.7809f,0.5473f,0.3011f,  -0.7830f,0.5434f,0.3026f,  -0.7199f,0.5552f,0.4165f,  -0.7196f,0.5558f,0.4163f,  -0.6419f,0.5624f,0.5213f,  -0.6393f,0.5676f,0.5189f,  -0.5485f,0.5687f,0.6130f,  -0.5439f,0.5783f,0.6081f,  
	-0.4418f,0.5737f,0.6897f,  -0.4354f,0.5877f,0.6819f,  -0.3241f,0.5770f,0.7497f,  -0.3163f,0.5951f,0.7388f,  -0.1979f,0.5780f,0.7917f,  -0.1890f,0.5998f,0.7775f,  -0.0660f,0.5759f,0.8148f,  -0.0564f,0.6011f,0.7972f,  0.0687f,0.5700f,0.8188f,  0.0788f,0.5979f,0.7977f,  0.2035f,0.5592f,0.8036f,  0.2135f,0.5895f,0.7790f,  0.3353f,0.5428f,0.7700f,  0.3450f,0.5747f,0.7421f,  0.4613f,0.5198f,0.7190f,  
	0.4704f,0.5525f,0.6881f,  0.5790f,0.4891f,0.6524f,  0.5871f,0.5220f,0.6187f,  0.6857f,0.4501f,0.5721f,  0.6926f,0.4823f,0.5364f,  0.7793f,0.4019f,0.4808f,  0.7848f,0.4326f,0.4437f,  0.8580f,0.3440f,0.3815f,  0.8620f,0.3726f,0.3438f,  0.9202f,0.2762f,0.2774f,  0.9226f,0.3018f,0.2401f,  0.9649f,0.1985f,0.1721f,  0.9658f,0.2206f,0.1363f,  0.9914f,0.1113f,0.0693f,  0.9909f,0.1294f,0.0361f,  
	0.9995f,0.0154f,-0.0273f,  0.9980f,0.0292f,-0.0567f,  0.9896f,-0.0879f,-0.1141f,  0.9872f,-0.0785f,-0.1386f,  0.9622f,-0.1970f,-0.1879f,  0.9595f,-0.1918f,-0.2064f,  0.9186f,-0.3095f,-0.2458f,  0.9158f,-0.3082f,-0.2574f,  0.8600f,-0.4230f,-0.2855f,  0.8577f,-0.4249f,-0.2895f,  0.7881f,-0.5344f,-0.3054f,  0.7867f,-0.5387f,-0.3014f,  0.7048f,-0.6406f,-0.3048f,  0.7047f,-0.6463f,-0.2926f,  0.6544f,-0.6181f,-0.4356f,  
	0.6618f,-0.6127f,-0.4320f,  0.6696f,-0.5233f,-0.5271f,  0.6710f,-0.5214f,-0.5272f,  0.6618f,-0.4264f,-0.6167f,  0.6574f,-0.4285f,-0.6198f,  0.6315f,-0.3298f,-0.7017f,  0.6218f,-0.3363f,-0.7073f,  0.5802f,-0.2356f,-0.7797f,  0.5657f,-0.2466f,-0.7868f,  0.5096f,-0.1454f,-0.8480f,  0.4911f,-0.1608f,-0.8561f,  0.4222f,-0.0606f,-0.9045f,  0.4003f,-0.0801f,-0.9129f,  0.3207f,0.0180f,-0.9470f,  0.2962f,-0.0053f,-0.9551f,  
	0.2082f,0.0897f,-0.9740f,  0.1818f,0.0631f,-0.9813f,  0.0879f,0.1543f,-0.9841f,  0.0603f,0.1251f,-0.9903f,  -0.0368f,0.2118f,-0.9766f,  -0.0649f,0.1806f,-0.9814f,  -0.1624f,0.2624f,-0.9512f,  -0.1904f,0.2298f,-0.9544f,  -0.2857f,0.3066f,-0.9079f,  -0.3130f,0.2734f,-0.9096f,  -0.4035f,0.3448f,-0.8475f,  -0.4296f,0.3116f,-0.8475f,  -0.5128f,0.3777f,-0.7710f,  -0.5373f,0.3453f,-0.7695f,  -0.6108f,0.4060f,-0.6798f,  
	-0.6333f,0.3749f,-0.6770f,  -0.6951f,0.4304f,-0.5758f,  -0.7155f,0.4013f,-0.5719f,  -0.7638f,0.4517f,-0.4612f,  -0.7817f,0.4250f,-0.4565f,  -0.8150f,0.4704f,-0.3383f,  -0.8303f,0.4467f,-0.3332f,  -0.8477f,0.4871f,-0.2099f,  -0.8603f,0.4669f,-0.2047f,  -0.8610f,0.5025f,-0.0788f,  -0.8708f,0.4860f,-0.0738f,  -0.8545f,0.5168f,0.0523f,  -0.8616f,0.5044f,0.0565f,  -0.8283f,0.5304f,0.1804f,  -0.8328f,0.5223f,0.1834f,  
	-0.7830f,0.5434f,0.3026f,  -0.7849f,0.5398f,0.3041f,  -0.7196f,0.5558f,0.4163f,  -0.7191f,0.5568f,0.4158f,  -0.6393f,0.5676f,0.5189f,  -0.6366f,0.5731f,0.5160f,  -0.5439f,0.5783f,0.6081f,  -0.5393f,0.5883f,0.6025f,  -0.4354f,0.5877f,0.6819f,  -0.4292f,0.6020f,0.6733f,  -0.3163f,0.5951f,0.7388f,  -0.3088f,0.6134f,0.7269f,  -0.1890f,0.5998f,0.7775f,  -0.1805f,0.6219f,0.7620f,  -0.0564f,0.6011f,0.7972f,  
	-0.0473f,0.6264f,0.7780f,  0.0788f,0.5979f,0.7977f,  0.0882f,0.6261f,0.7747f,  0.2135f,0.5895f,0.7790f,  0.2228f,0.6198f,0.7525f,  0.3450f,0.5747f,0.7421f,  0.3538f,0.6065f,0.7120f,  0.4704f,0.5525f,0.6881f,  0.4785f,0.5850f,0.6548f,  0.5871f,0.5220f,0.6187f,  0.5941f,0.5545f,0.5828f,  0.6926f,0.4823f,0.5364f,  0.6983f,0.5139f,0.4983f,  0.7848f,0.4326f,0.4437f,  0.7891f,0.4625f,0.4043f,  
	0.8620f,0.3726f,0.3438f,  0.8646f,0.4000f,0.3040f,  0.9226f,0.3018f,0.2401f,  0.9237f,0.3261f,0.2009f,  0.9658f,0.2206f,0.1363f,  0.9654f,0.2411f,0.0989f,  0.9909f,0.1294f,0.0361f,  0.9893f,0.1458f,0.0017f,  0.9980f,0.0292f,-0.0567f,  0.9954f,0.0413f,-0.0869f,  0.9872f,-0.0785f,-0.1386f,  0.9840f,-0.0708f,-0.1635f,  0.9595f,-0.1918f,-0.2064f,  0.9561f,-0.1881f,-0.2249f,  0.9158f,-0.3082f,-0.2574f,  
	0.9127f,-0.3080f,-0.2686f,  0.8577f,-0.4249f,-0.2895f,  0.8552f,-0.4276f,-0.2928f,  0.7867f,-0.5387f,-0.3014f,  0.7853f,-0.5434f,-0.2966f,  0.7047f,-0.6463f,-0.2926f,  0.7047f,-0.6521f,-0.2797f,  0.6618f,-0.6127f,-0.4320f,  0.6687f,-0.6075f,-0.4288f,  0.6710f,-0.5214f,-0.5272f,  0.6719f,-0.5199f,-0.5274f,  0.6574f,-0.4285f,-0.6198f,  0.6528f,-0.4313f,-0.6228f,  0.6218f,-0.3363f,-0.7073f,  0.6122f,-0.3435f,-0.7122f,  
	0.5657f,-0.2466f,-0.7868f,  0.5516f,-0.2583f,-0.7931f,  0.4911f,-0.1608f,-0.8561f,  0.4731f,-0.1769f,-0.8631f,  0.4003f,-0.0801f,-0.9129f,  0.3791f,-0.1003f,-0.9199f,  0.2962f,-0.0053f,-0.9551f,  0.2725f,-0.0291f,-0.9617f,  0.1818f,0.0631f,-0.9813f,  0.1564f,0.0362f,-0.9870f,  0.0603f,0.1251f,-0.9903f,  0.0338f,0.0956f,-0.9948f,  -0.0649f,0.1806f,-0.9814f,  -0.0918f,0.1492f,-0.9845f,  -0.1904f,0.2298f,-0.9544f,  
	-0.2171f,0.1972f,-0.9560f,  -0.3130f,0.2734f,-0.9096f,  -0.3391f,0.2402f,-0.9096f,  -0.4296f,0.3116f,-0.8475f,  -0.4546f,0.2786f,-0.8460f,  -0.5373f,0.3453f,-0.7695f,  -0.5607f,0.3130f,-0.7666f,  -0.6333f,0.3749f,-0.6770f,  -0.6549f,0.3440f,-0.6729f,  -0.7155f,0.4013f,-0.5719f,  -0.7349f,0.3724f,-0.5668f,  -0.7817f,0.4250f,-0.4565f,  -0.7987f,0.3986f,-0.4507f,  -0.8303f,0.4467f,-0.3332f,  -0.8449f,0.4233f,-0.3271f,  
	-0.8603f,0.4669f,-0.2047f,  -0.8722f,0.4469f,-0.1987f,  -0.8708f,0.4860f,-0.0738f,  -0.8801f,0.4698f,-0.0683f,  -0.8616f,0.5044f,0.0565f,  -0.8683f,0.4923f,0.0611f,  -0.8328f,0.5223f,0.1834f,  -0.8369f,0.5145f,0.1867f,  -0.7849f,0.5398f,0.3041f,  -0.7866f,0.5365f,0.3056f,  -0.7191f,0.5568f,0.4158f,  -0.7185f,0.5580f,0.4152f,  -0.6366f,0.5731f,0.5160f,  -0.6340f,0.5789f,0.5128f,  -0.5393f,0.5883f,0.6025f,  
	-0.5349f,0.5986f,0.5963f,  -0.4292f,0.6020f,0.6733f,  -0.4233f,0.6166f,0.6638f,  -0.3088f,0.6134f,0.7269f,  -0.3016f,0.6321f,0.7138f,  -0.1805f,0.6219f,0.7620f,  -0.1725f,0.6442f,0.7451f,  -0.0473f,0.6264f,0.7780f,  -0.0388f,0.6520f,0.7572f,  0.0882f,0.6261f,0.7747f,  0.0968f,0.6543f,0.7500f,  0.2228f,0.6198f,0.7525f,  0.2312f,0.6501f,0.7238f,  0.3538f,0.6065f,0.7120f,  0.3617f,0.6381f,0.6797f,  
	0.4785f,0.5850f,0.6548f,  0.4854f,0.6172f,0.6192f,  0.5941f,0.5545f,0.5828f,  0.5998f,0.5863f,0.5444f,  0.6983f,0.5139f,0.4983f,  0.7027f,0.5446f,0.4579f,  0.7891f,0.4625f,0.4043f,  0.7919f,0.4913f,0.3627f,  0.8646f,0.4000f,0.3040f,  0.8659f,0.4260f,0.2621f,  0.9237f,0.3261f,0.2009f,  0.9235f,0.3488f,0.1600f,  0.9654f,0.2411f,0.0989f,  0.9638f,0.2599f,0.0601f,  0.9893f,0.1458f,0.0017f,  
	0.9865f,0.1603f,-0.0337f,  0.9954f,0.0413f,-0.0869f,  0.9917f,0.0515f,-0.1177f,  0.9840f,-0.0708f,-0.1635f,  0.9799f,-0.0648f,-0.1885f,  0.9561f,-0.1881f,-0.2249f,  0.9520f,-0.1860f,-0.2431f,  0.9127f,-0.3080f,-0.2686f,  0.9091f,-0.3092f,-0.2792f,  0.8552f,-0.4276f,-0.2928f,  0.8525f,-0.4312f,-0.2954f,  0.7853f,-0.5434f,-0.2966f,  0.7839f,-0.5486f,-0.2909f,  0.7047f,-0.6521f,-0.2797f,  0.7047f,-0.6578f,-0.2659f,  
	0.6687f,-0.6075f,-0.4288f,  0.6750f,-0.6024f,-0.4259f,  0.6719f,-0.5199f,-0.5274f,  0.6726f,-0.5189f,-0.5276f,  0.6528f,-0.4313f,-0.6228f,  0.6481f,-0.4346f,-0.6254f,  0.6122f,-0.3435f,-0.7122f,  0.6026f,-0.3514f,-0.7166f,  0.5516f,-0.2583f,-0.7931f,  0.5377f,-0.2706f,-0.7985f,  0.4731f,-0.1769f,-0.8631f,  0.4555f,-0.1935f,-0.8689f,  0.3791f,-0.1003f,-0.9199f,  0.3586f,-0.1209f,-0.9256f,  0.2725f,-0.0291f,-0.9617f,  
	0.2498f,-0.0533f,-0.9668f,  0.1564f,0.0362f,-0.9870f,  0.1320f,0.0090f,-0.9912f,  0.0338f,0.0956f,-0.9948f,  0.0084f,0.0660f,-0.9978f,  -0.0918f,0.1492f,-0.9845f,  -0.1176f,0.1178f,-0.9861f,  -0.2171f,0.1972f,-0.9560f,  -0.2428f,0.1647f,-0.9560f,  -0.3391f,0.2402f,-0.9096f,  -0.3641f,0.2071f,-0.9080f,  -0.4546f,0.2786f,-0.8460f,  -0.4785f,0.2457f,-0.8430f,  -0.5607f,0.3130f,-0.7666f,  -0.5831f,0.2809f,-0.7623f,  
	-0.6549f,0.3440f,-0.6729f,  -0.6755f,0.3133f,-0.6675f,  -0.7349f,0.3724f,-0.5668f,  -0.7534f,0.3437f,-0.5606f,  -0.7987f,0.3986f,-0.4507f,  -0.8150f,0.3724f,-0.4440f,  -0.8449f,0.4233f,-0.3271f,  -0.8587f,0.4001f,-0.3202f,  -0.8722f,0.4469f,-0.1987f,  -0.8836f,0.4271f,-0.1920f,  -0.8801f,0.4698f,-0.0683f,  -0.8889f,0.4538f,-0.0622f,  -0.8683f,0.4923f,0.0611f,  -0.8746f,0.4804f,0.0662f,  -0.8369f,0.5145f,0.1867f,  
	-0.8407f,0.5070f,0.1902f,  -0.7866f,0.5365f,0.3056f,  -0.7881f,0.5334f,0.3072f,  -0.7185f,0.5580f,0.4152f,  -0.7178f,0.5595f,0.4143f,  -0.6340f,0.5789f,0.5128f,  -0.6314f,0.5850f,0.5091f,  -0.5349f,0.5986f,0.5963f,  -0.5305f,0.6092f,0.5894f,  -0.4233f,0.6166f,0.6638f,  -0.4176f,0.6315f,0.6534f,  -0.3016f,0.6321f,0.7138f,  -0.2948f,0.6510f,0.6995f,  -0.1725f,0.6442f,0.7451f,  -0.1650f,0.6668f,0.7268f,  
	-0.0388f,0.6520f,0.7572f,  -0.0309f,0.6777f,0.7347f,  0.0968f,0.6543f,0.7500f,  0.1046f,0.6826f,0.7232f,  0.2312f,0.6501f,0.7238f,  0.2387f,0.6803f,0.6930f,  0.3617f,0.6381f,0.6797f,  0.3684f,0.6694f,0.6451f,  0.4854f,0.6172f,0.6192f,  0.4911f,0.6488f,0.5813f,  0.5998f,0.5863f,0.5444f,  0.6043f,0.6174f,0.5037f,  0.7027f,0.5446f,0.4579f,  0.7056f,0.5742f,0.4152f,  0.7919f,0.4913f,0.3627f,  
	0.7933f,0.5187f,0.3189f,  0.8659f,0.4260f,0.2621f,  0.8657f,0.4504f,0.2183f,  0.9235f,0.3488f,0.1600f,  0.9218f,0.3696f,0.1174f,  0.9638f,0.2599f,0.0601f,  0.9608f,0.2766f,0.0200f,  0.9865f,0.1603f,-0.0337f,  0.9825f,0.1728f,-0.0700f,  0.9917f,0.0515f,-0.1177f,  0.9871f,0.0596f,-0.1489f,  0.9799f,-0.0648f,-0.1885f,  0.9751f,-0.0607f,-0.2134f,  0.9520f,-0.1860f,-0.2431f,  0.9474f,-0.1855f,-0.2608f,  
	0.9091f,-0.3092f,-0.2792f,  0.9051f,-0.3116f,-0.2892f,  0.8525f,-0.4312f,-0.2954f,  0.8497f,-0.4357f,-0.2971f,  0.7839f,-0.5486f,-0.2909f,  0.7824f,-0.5541f,-0.2843f,  0.7047f,-0.6578f,-0.2659f,  0.7049f,-0.6633f,-0.2513f,  0.6750f,-0.6024f,-0.4259f,  0.6808f,-0.5976f,-0.4235f,  0.6726f,-0.5189f,-0.5276f,  0.6729f,-0.5182f,-0.5279f,  0.6481f,-0.4346f,-0.6254f,  0.6432f,-0.4384f,-0.6277f,  0.6026f,-0.3514f,-0.7166f,  
	0.5931f,-0.3597f,-0.7203f,  0.5377f,-0.2706f,-0.7985f,  0.5242f,-0.2834f,-0.8031f,  0.4555f,-0.1935f,-0.8689f,  0.4386f,-0.2106f,-0.8737f,  0.3586f,-0.1209f,-0.9256f,  0.3388f,-0.1419f,-0.9301f,  0.2498f,-0.0533f,-0.9668f,  0.2278f,-0.0777f,-0.9706f,  0.1320f,0.0090f,-0.9912f,  0.1085f,-0.0184f,-0.9939f,  0.0084f,0.0660f,-0.9978f,  -0.0160f,0.0363f,-0.9992f,  -0.1176f,0.1178f,-0.9861f,  -0.1423f,0.0863f,-0.9861f,  
	-0.2428f,0.1647f,-0.9560f,  -0.2674f,0.1321f,-0.9545f,  -0.3641f,0.2071f,-0.9080f,  -0.3880f,0.1742f,-0.9050f,  -0.4785f,0.2457f,-0.8430f,  -0.5013f,0.2129f,-0.8386f,  -0.5831f,0.2809f,-0.7623f,  -0.6046f,0.2489f,-0.7567f,  -0.6755f,0.3133f,-0.6675f,  -0.6952f,0.2828f,-0.6608f,  -0.7534f,0.3437f,-0.5606f,  -0.7711f,0.3152f,-0.5532f,  -0.8150f,0.3724f,-0.4440f,  -0.8305f,0.3464f,-0.4362f,  -0.8587f,0.4001f,-0.3202f,  
	-0.8719f,0.3771f,-0.3123f,  -0.8836f,0.4271f,-0.1920f,  -0.8943f,0.4076f,-0.1845f,  -0.8889f,0.4538f,-0.0622f,  -0.8972f,0.4381f,-0.0556f,  -0.8746f,0.4804f,0.0662f,  -0.8804f,0.4688f,0.0716f,  -0.8407f,0.5070f,0.1902f,  -0.8442f,0.4997f,0.1939f,  -0.7881f,0.5334f,0.3072f,  -0.7894f,0.5306f,0.3087f,  -0.7178f,0.5595f,0.4143f,  -0.7171f,0.5613f,0.4132f,  -0.6314f,0.5850f,0.5091f,  -0.6288f,0.5914f,0.5049f,  
	-0.5305f,0.6092f,0.5894f,  -0.5264f,0.6201f,0.5818f,  -0.4176f,0.6315f,0.6534f,  -0.4121f,0.6467f,0.6419f,  -0.2948f,0.6510f,0.6995f,  -0.2885f,0.6702f,0.6839f,  -0.1650f,0.6668f,0.7268f,  -0.1581f,0.6895f,0.7068f,  -0.0309f,0.6777f,0.7347f,  -0.0237f,0.7035f,0.7103f,  0.1046f,0.6826f,0.7232f,  0.1116f,0.7108f,0.6945f,  0.2387f,0.6803f,0.6930f,  0.2452f,0.7101f,0.6600f,  0.3684f,0.6694f,0.6451f,  
	0.3740f,0.7002f,0.6082f,  0.4911f,0.6488f,0.5813f,  0.4956f,0.6796f,0.5409f,  0.6043f,0.6174f,0.5037f,  0.6073f,0.6473f,0.4606f,  0.7056f,0.5742f,0.4152f,  0.7071f,0.6025f,0.3701f,  0.7933f,0.5187f,0.3189f,  0.7932f,0.5444f,0.2729f,  0.8657f,0.4504f,0.2183f,  0.8640f,0.4729f,0.1727f,  0.9218f,0.3696f,0.1174f,  0.9186f,0.3882f,0.0734f,  0.9608f,0.2766f,0.0200f,  0.9565f,0.2911f,-0.0210f,  
	0.9825f,0.1728f,-0.0700f,  0.9773f,0.1829f,-0.1067f,  0.9871f,0.0596f,-0.1489f,  0.9815f,0.0655f,-0.1801f,  0.9751f,-0.0607f,-0.2134f,  0.9695f,-0.0587f,-0.2380f,  0.9474f,-0.1855f,-0.2608f,  0.9423f,-0.1867f,-0.2780f,  0.9051f,-0.3116f,-0.2892f,  0.9009f,-0.3154f,-0.2982f,  0.8497f,-0.4357f,-0.2971f,  0.8467f,-0.4410f,-0.2978f,  0.7824f,-0.5541f,-0.2843f,  0.7809f,-0.5600f,-0.2767f,  0.7049f,-0.6633f,-0.2513f,  
	0.7050f,-0.6688f,-0.2359f,  0.6808f,-0.5976f,-0.4235f,  0.6861f,-0.5930f,-0.4214f,  0.6729f,-0.5182f,-0.5279f,  0.6729f,-0.5179f,-0.5282f,  0.6432f,-0.4384f,-0.6277f,  0.6383f,-0.4426f,-0.6298f,  0.5931f,-0.3597f,-0.7203f,  0.5838f,-0.3685f,-0.7235f,  0.5242f,-0.2834f,-0.8031f,  0.5110f,-0.2966f,-0.8068f,  0.4386f,-0.2106f,-0.8737f,  0.4221f,-0.2280f,-0.8774f,  0.3388f,-0.1419f,-0.9301f,  0.3197f,-0.1631f,-0.9334f,  
	0.2278f,-0.0777f,-0.9706f,  0.2067f,-0.1024f,-0.9730f,  0.1085f,-0.0184f,-0.9939f,  0.0859f,-0.0459f,-0.9952f,  -0.0160f,0.0363f,-0.9992f,  -0.0394f,0.0065f,-0.9992f,  -0.1423f,0.0863f,-0.9861f,  -0.1660f,0.0549f,-0.9846f,  -0.2674f,0.1321f,-0.9545f,  -0.2909f,0.0997f,-0.9515f,  -0.3880f,0.1742f,-0.9050f,  -0.4110f,0.1413f,-0.9006f,  -0.5013f,0.2129f,-0.8386f,  -0.5233f,0.1803f,-0.8329f,  -0.6046f,0.2489f,-0.7567f,  
	-0.6251f,0.2172f,-0.7497f,  -0.6952f,0.2828f,-0.6608f,  -0.7141f,0.2525f,-0.6530f,  -0.7711f,0.3152f,-0.5532f,  -0.7881f,0.2868f,-0.5447f,  -0.8305f,0.3464f,-0.4362f,  -0.8453f,0.3206f,-0.4274f,  -0.8719f,0.3771f,-0.3123f,  -0.8845f,0.3543f,-0.3037f,  -0.8943f,0.4076f,-0.1845f,  -0.9045f,0.3882f,-0.1763f,  -0.8972f,0.4381f,-0.0556f,  -0.9051f,0.4225f,-0.0483f,  -0.8804f,0.4688f,0.0716f,  -0.8859f,0.4573f,0.0774f,  
	-0.8442f,0.4997f,0.1939f,  -0.8475f,0.4926f,0.1978f,  -0.7894f,0.5306f,0.3087f,  -0.7905f,0.5281f,0.3102f,  -0.7171f,0.5613f,0.4132f,  -0.7162f,0.5634f,0.4118f,  -0.6288f,0.5914f,0.5049f,  -0.6262f,0.5981f,0.5002f,  -0.5264f,0.6201f,0.5818f,  -0.5224f,0.6312f,0.5733f,  -0.4121f,0.6467f,0.6419f,  -0.4070f,0.6621f,0.6293f,  -0.2885f,0.6702f,0.6839f,  -0.2825f,0.6895f,0.6669f,  -0.1581f,0.6895f,0.7068f,  
	-0.1517f,0.7123f,0.6853f,  -0.0237f,0.7035f,0.7103f,  -0.0173f,0.7292f,0.6841f,  0.1116f,0.7108f,0.6945f,  0.1177f,0.7388f,0.6636f,  0.2452f,0.7101f,0.6600f,  0.2505f,0.7396f,0.6247f,  0.3740f,0.7002f,0.6082f,  0.3784f,0.7302f,0.5688f,  0.4956f,0.6796f,0.5409f,  0.4986f,0.7094f,0.4981f,  0.6073f,0.6473f,0.4606f,  0.6089f,0.6760f,0.4151f,  0.7071f,0.6025f,0.3701f,  0.7071f,0.6291f,0.3229f,  
	0.7932f,0.5444f,0.2729f,  0.7915f,0.5682f,0.2251f,  0.8640f,0.4729f,0.1727f,  0.8608f,0.4932f,0.1256f,  0.9186f,0.3882f,0.0734f,  0.9141f,0.4045f,0.0283f,  0.9565f,0.2911f,-0.0210f,  0.9509f,0.3031f,-0.0627f,  0.9773f,0.1829f,-0.1067f,  0.9711f,0.1906f,-0.1437f,  0.9815f,0.0655f,-0.1801f,  0.9750f,0.0691f,-0.2111f,  0.9695f,-0.0587f,-0.2380f,  0.9633f,-0.0587f,-0.2620f,  0.9423f,-0.1867f,-0.2780f,  
	0.9367f,-0.1897f,-0.2942f,  0.9009f,-0.3154f,-0.2982f,  0.8965f,-0.3204f,-0.3062f,  0.8467f,-0.4410f,-0.2978f,  0.8436f,-0.4471f,-0.2974f,  0.7809f,-0.5600f,-0.2767f,  0.7795f,-0.5662f,-0.2682f,  0.7050f,-0.6688f,-0.2359f,  0.7052f,-0.6741f,-0.2197f,  0.6861f,-0.5930f,-0.4214f,  0.6910f,-0.5886f,-0.4196f,  0.6729f,-0.5179f,-0.5282f,  0.6726f,-0.5180f,-0.5285f,  0.6383f,-0.4426f,-0.6298f,  0.6334f,-0.4472f,-0.6315f,  
	0.5838f,-0.3685f,-0.7235f,  0.5746f,-0.3776f,-0.7261f,  0.5110f,-0.2966f,-0.8068f,  0.4981f,-0.3102f,-0.8097f,  0.4221f,-0.2280f,-0.8774f,  0.4062f,-0.2457f,-0.8801f,  0.3197f,-0.1631f,-0.9334f,  0.3013f,-0.1846f,-0.9355f,  0.2067f,-0.1024f,-0.9730f,  0.1863f,-0.1272f,-0.9742f,  0.0859f,-0.0459f,-0.9952f,  0.0642f,-0.0735f,-0.9952f,  -0.0394f,0.0065f,-0.9992f,  -0.0619f,-0.0233f,-0.9978f,  -0.1660f,0.0549f,-0.9846f,  
	-0.1888f,0.0235f,-0.9817f,  -0.2909f,0.0997f,-0.9515f,  -0.3135f,0.0673f,-0.9472f,  -0.4110f,0.1413f,-0.9006f,  -0.4330f,0.1085f,-0.8948f,  -0.5233f,0.1803f,-0.8329f,  -0.5443f,0.1478f,-0.8258f,  -0.6251f,0.2172f,-0.7497f,  -0.6448f,0.1855f,-0.7415f,  -0.7141f,0.2525f,-0.6530f,  -0.7321f,0.2223f,-0.6439f,  -0.7881f,0.2868f,-0.5447f,  -0.8042f,0.2587f,-0.5351f,  -0.8453f,0.3206f,-0.4274f,  -0.8594f,0.2950f,-0.4176f,  
	-0.8845f,0.3543f,-0.3037f,  -0.8964f,0.3317f,-0.2941f,  -0.9045f,0.3882f,-0.1763f,  -0.9142f,0.3690f,-0.1674f,  -0.9051f,0.4225f,-0.0483f,  -0.9124f,0.4072f,-0.0406f,  -0.8859f,0.4573f,0.0774f,  -0.8911f,0.4461f,0.0835f,  -0.8475f,0.4926f,0.1978f,  -0.8504f,0.4858f,0.2019f,  -0.7905f,0.5281f,0.3102f,  -0.7915f,0.5258f,0.3116f,  -0.7162f,0.5634f,0.4118f,  -0.7153f,0.5658f,0.4101f,  -0.6262f,0.5981f,0.5002f,  
	-0.6237f,0.6050f,0.4949f,  -0.5224f,0.6312f,0.5733f,  -0.5186f,0.6427f,0.5640f,  -0.4070f,0.6621f,0.6293f,  -0.4022f,0.6777f,0.6156f,  -0.2825f,0.6895f,0.6669f,  -0.2771f,0.7090f,0.6485f,  -0.1517f,0.7123f,0.6853f,  -0.1460f,0.7352f,0.6620f,  -0.0173f,0.7292f,0.6841f,  -0.0118f,0.7548f,0.6559f,  0.1177f,0.7388f,0.6636f,  0.1228f,0.7664f,0.6305f,  0.2505f,0.7396f,0.6247f,  0.2548f,0.7684f,0.5871f,  
	0.3784f,0.7302f,0.5688f,  0.3814f,0.7594f,0.5271f,  0.4986f,0.7094f,0.4981f,  0.5003f,0.7380f,0.4529f,  0.6089f,0.6760f,0.4151f,  0.6090f,0.7030f,0.3673f,  0.7071f,0.6291f,0.3229f,  0.7055f,0.6537f,0.2736f,  0.7915f,0.5682f,0.2251f,  0.7883f,0.5897f,0.1756f,  0.8608f,0.4932f,0.1256f,  0.8561f,0.5110f,0.0771f,  0.9141f,0.4045f,0.0283f,  0.9082f,0.4181f,-0.0177f,  0.9509f,0.3031f,-0.0627f,  
	0.9441f,0.3124f,-0.1048f,  0.9711f,0.1906f,-0.1437f,  0.9639f,0.1957f,-0.1806f,  0.9750f,0.0691f,-0.2111f,  0.9678f,0.0703f,-0.2416f,  0.9633f,-0.0587f,-0.2620f,  0.9566f,-0.0608f,-0.2851f,  0.9367f,-0.1897f,-0.2942f,  0.9309f,-0.1943f,-0.3093f,  0.8965f,-0.3204f,-0.3062f,  0.8919f,-0.3265f,-0.3129f,  0.8436f,-0.4471f,-0.2974f,  0.8405f,-0.4539f,-0.2958f,  0.7795f,-0.5662f,-0.2682f,  0.7780f,-0.5726f,-0.2586f,  
	0.7052f,-0.6741f,-0.2197f,  0.7055f,-0.6791f,-0.2027f,  0.6910f,-0.5886f,-0.4196f,  0.6954f,-0.5844f,-0.4182f,  0.6726f,-0.5180f,-0.5285f,  0.6721f,-0.5183f,-0.5288f,  0.6334f,-0.4472f,-0.6315f,  0.6284f,-0.4522f,-0.6330f,  0.5746f,-0.3776f,-0.7261f,  0.5656f,-0.3872f,-0.7281f,  0.4981f,-0.3102f,-0.8097f,  0.4856f,-0.3241f,-0.8118f,  0.4062f,-0.2457f,-0.8801f,  0.3908f,-0.2637f,-0.8819f,  0.3013f,-0.1846f,-0.9355f,  
	0.2835f,-0.2063f,-0.9365f,  0.1863f,-0.1272f,-0.9742f,  0.1667f,-0.1521f,-0.9742f,  0.0642f,-0.0735f,-0.9952f,  0.0433f,-0.1011f,-0.9939f,  -0.0619f,-0.0233f,-0.9978f,  -0.0835f,-0.0531f,-0.9951f,  -0.1888f,0.0235f,-0.9817f,  -0.2108f,-0.0079f,-0.9775f,  -0.3135f,0.0673f,-0.9472f,  -0.3353f,0.0349f,-0.9415f,  -0.4330f,0.1085f,-0.8948f,  -0.4541f,0.0759f,-0.8877f,  -0.5443f,0.1478f,-0.8258f,  -0.5644f,0.1154f,-0.8174f,  
	-0.6448f,0.1855f,-0.7415f,  -0.6636f,0.1540f,-0.7321f,  -0.7321f,0.2223f,-0.6439f,  -0.7493f,0.1923f,-0.6337f,  -0.8042f,0.2587f,-0.5351f,  -0.8196f,0.2306f,-0.5244f,  -0.8594f,0.2950f,-0.4176f,  -0.8728f,0.2695f,-0.4068f,  -0.8964f,0.3317f,-0.2941f,  -0.9077f,0.3092f,-0.2836f,  -0.9142f,0.3690f,-0.1674f,  -0.9234f,0.3500f,-0.1578f,  -0.9124f,0.4072f,-0.0406f,  -0.9194f,0.3920f,-0.0322f,  -0.8911f,0.4461f,0.0835f,  
	-0.8958f,0.4352f,0.0901f,  -0.8504f,0.4858f,0.2019f,  -0.8531f,0.4792f,0.2061f,  -0.7915f,0.5258f,0.3116f,  -0.7922f,0.5239f,0.3129f,  -0.7153f,0.5658f,0.4101f,  -0.7144f,0.5685f,0.4080f,  -0.6237f,0.6050f,0.4949f,  -0.6213f,0.6123f,0.4890f,  -0.5186f,0.6427f,0.5640f,  -0.5150f,0.6543f,0.5537f,  -0.4022f,0.6777f,0.6156f,  -0.3978f,0.6935f,0.6006f,  -0.2771f,0.7090f,0.6485f,  -0.2722f,0.7286f,0.6286f,  
	-0.1460f,0.7352f,0.6620f,  -0.1411f,0.7579f,0.6369f,  -0.0118f,0.7548f,0.6559f,  -0.0071f,0.7801f,0.6256f,  0.1228f,0.7664f,0.6305f,  0.1268f,0.7935f,0.5953f,  0.2548f,0.7684f,0.5871f,  0.2577f,0.7964f,0.5471f,  0.3814f,0.7594f,0.5271f,  0.3831f,0.7873f,0.4830f,  0.5003f,0.7380f,0.4529f,  0.5004f,0.7649f,0.4055f,  0.6090f,0.7030f,0.3673f,  0.6075f,0.7281f,0.3175f,  0.7055f,0.6537f,0.2736f,  
	0.7024f,0.6761f,0.2226f,  0.7883f,0.5897f,0.1756f,  0.7836f,0.6087f,0.1246f,  0.8561f,0.5110f,0.0771f,  0.8500f,0.5260f,0.0276f,  0.9082f,0.4181f,-0.0177f,  0.9011f,0.4289f,-0.0642f,  0.9441f,0.3124f,-0.1048f,  0.9363f,0.3190f,-0.1469f,  0.9639f,0.1957f,-0.1806f,  0.9558f,0.1982f,-0.2170f,  0.9678f,0.0703f,-0.2416f,  0.9600f,0.0692f,-0.2712f,  0.9566f,-0.0608f,-0.2851f,  0.9495f,-0.0649f,-0.3071f,  
	0.9309f,-0.1943f,-0.3093f,  0.9249f,-0.2004f,-0.3231f,  0.8919f,-0.3265f,-0.3129f,  0.8873f,-0.3338f,-0.3184f,  0.8405f,-0.4539f,-0.2958f,  0.8375f,-0.4613f,-0.2930f,  0.7780f,-0.5726f,-0.2586f,  0.7767f,-0.5791f,-0.2479f,  0.7055f,-0.6791f,-0.2027f,  0.7058f,-0.6839f,-0.1850f,  0.6954f,-0.5844f,-0.4182f,  0.6993f,-0.5805f,-0.4170f,  0.6721f,-0.5183f,-0.5288f,  0.6714f,-0.5190f,-0.5291f,  0.6284f,-0.4522f,-0.6330f,  
	0.6234f,-0.4575f,-0.6341f,  0.5656f,-0.3872f,-0.7281f,  0.5568f,-0.3970f,-0.7297f,  0.4856f,-0.3241f,-0.8118f,  0.4735f,-0.3383f,-0.8132f,  0.3908f,-0.2637f,-0.8819f,  0.3758f,-0.2819f,-0.8828f,  0.2835f,-0.2063f,-0.9365f,  0.2663f,-0.2282f,-0.9365f,  0.1667f,-0.1521f,-0.9742f,  0.1478f,-0.1771f,-0.9730f,  0.0433f,-0.1011f,-0.9939f,  0.0232f,-0.1288f,-0.9914f,  -0.0835f,-0.0531f,-0.9951f,  -0.1043f,-0.0829f,-0.9911f,  
	-0.2108f,-0.0079f,-0.9775f,  -0.2318f,-0.0393f,-0.9720f,  -0.3353f,0.0349f,-0.9415f,  -0.3561f,0.0026f,-0.9344f,  -0.4541f,0.0759f,-0.8877f,  -0.4743f,0.0432f,-0.8793f,  -0.5644f,0.1154f,-0.8174f,  -0.5837f,0.0830f,-0.8077f,  -0.6636f,0.1540f,-0.7321f,  -0.6816f,0.1226f,-0.7214f,  -0.7493f,0.1923f,-0.6337f,  -0.7658f,0.1623f,-0.6222f,  -0.8196f,0.2306f,-0.5244f,  -0.8344f,0.2027f,-0.5126f,  -0.8728f,0.2695f,-0.4068f,  
	-0.8856f,0.2441f,-0.3950f,  -0.9077f,0.3092f,-0.2836f,  -0.9184f,0.2869f,-0.2723f,  -0.9234f,0.3500f,-0.1578f,  -0.9320f,0.3312f,-0.1474f,  -0.9194f,0.3920f,-0.0322f,  -0.9259f,0.3771f,-0.0233f,  -0.8958f,0.4352f,0.0901f,  -0.9002f,0.4244f,0.0970f,  -0.8531f,0.4792f,0.2061f,  -0.8556f,0.4730f,0.2104f,  -0.7922f,0.5239f,0.3129f,  -0.7929f,0.5222f,0.3142f,  -0.7144f,0.5685f,0.4080f,  -0.7134f,0.5714f,0.4056f,  
	-0.6213f,0.6123f,0.4890f,  -0.6190f,0.6198f,0.4824f,  -0.5150f,0.6543f,0.5537f,  -0.5117f,0.6662f,0.5425f,  -0.3978f,0.6935f,0.6006f,  -0.3938f,0.7095f,0.5844f,  -0.2722f,0.7286f,0.6286f,  -0.2680f,0.7481f,0.6071f,  -0.1411f,0.7579f,0.6369f,  -0.1369f,0.7805f,0.6100f,  -0.0071f,0.7801f,0.6256f,  -0.0034f,0.8050f,0.5933f,  0.1268f,0.7935f,0.5953f,  0.1296f,0.8198f,0.5578f,  0.2577f,0.7964f,0.5471f,  
	0.2594f,0.8233f,0.5049f,  0.3831f,0.7873f,0.4830f,  0.3834f,0.8138f,0.4367f,  0.5004f,0.7649f,0.4055f,  0.4991f,0.7901f,0.3559f,  0.6075f,0.7281f,0.3175f,  0.6044f,0.7510f,0.2658f,  0.7024f,0.6761f,0.2226f,  0.6976f,0.6960f,0.1700f,  0.7836f,0.6087f,0.1246f,  0.7774f,0.6249f,0.0725f,  0.8500f,0.5260f,0.0276f,  0.8426f,0.5381f,-0.0225f,  0.9011f,0.4289f,-0.0642f,  0.8927f,0.4367f,-0.1108f,  
	0.9363f,0.3190f,-0.1469f,  0.9275f,0.3226f,-0.1887f,  0.9558f,0.1982f,-0.2170f,  0.9471f,0.1979f,-0.2526f,  0.9600f,0.0692f,-0.2712f,  0.9518f,0.0656f,-0.2997f,  0.9495f,-0.0649f,-0.3071f,  0.9421f,-0.0709f,-0.3278f,  0.9249f,-0.2004f,-0.3231f,  0.9188f,-0.2081f,-0.3355f,  0.8873f,-0.3338f,-0.3184f,  0.8827f,-0.3421f,-0.3223f,  0.8375f,-0.4613f,-0.2930f,  0.8346f,-0.4691f,-0.2888f,  0.7767f,-0.5791f,-0.2479f,  
	0.7754f,-0.5856f,-0.2362f,  0.7058f,-0.6839f,-0.1850f,  0.7061f,-0.6883f,-0.1665f,  0.6993f,-0.5805f,-0.4170f,  0.7029f,-0.5769f,-0.4161f,  0.6714f,-0.5190f,-0.5291f,  0.6705f,-0.5199f,-0.5293f,  0.6234f,-0.4575f,-0.6341f,  0.6184f,-0.4630f,-0.6350f,  0.5568f,-0.3970f,-0.7297f,  0.5481f,-0.4071f,-0.7306f,  0.4735f,-0.3383f,-0.8132f,  0.4617f,-0.3527f,-0.8139f,  0.3758f,-0.2819f,-0.8828f,  0.3614f,-0.3003f,-0.8827f,  
	0.2663f,-0.2282f,-0.9365f,  0.2497f,-0.2502f,-0.9354f,  0.1478f,-0.1771f,-0.9730f,  0.1296f,-0.2023f,-0.9707f,  0.0232f,-0.1288f,-0.9914f,  0.0038f,-0.1566f,-0.9877f,  -0.1043f,-0.0829f,-0.9911f,  -0.1244f,-0.1128f,-0.9858f,  -0.2318f,-0.0393f,-0.9720f,  -0.2521f,-0.0706f,-0.9651f,  -0.3561f,0.0026f,-0.9344f,  -0.3762f,-0.0296f,-0.9261f,  -0.4743f,0.0432f,-0.8793f,  -0.4938f,0.0107f,-0.8695f,  -0.5837f,0.0830f,-0.8077f,  
	-0.6022f,0.0508f,-0.7967f,  -0.6816f,0.1226f,-0.7214f,  -0.6989f,0.0912f,-0.7094f,  -0.7658f,0.1623f,-0.6222f,  -0.7816f,0.1324f,-0.6096f,  -0.8344f,0.2027f,-0.5126f,  -0.8484f,0.1748f,-0.4996f,  -0.8856f,0.2441f,-0.3950f,  -0.8978f,0.2188f,-0.3822f,  -0.9184f,0.2869f,-0.2723f,  -0.9286f,0.2647f,-0.2600f,  -0.9320f,0.3312f,-0.1474f,  -0.9401f,0.3125f,-0.1362f,  -0.9259f,0.3771f,-0.0233f,  -0.9319f,0.3623f,-0.0137f,  
	-0.9002f,0.4244f,0.0970f,  -0.9043f,0.4139f,0.1043f,  -0.8556f,0.4730f,0.2104f,  -0.8578f,0.4669f,0.2149f,  -0.7929f,0.5222f,0.3142f,  -0.7934f,0.5207f,0.3153f,  -0.7134f,0.5714f,0.4056f,  -0.7125f,0.5746f,0.4028f,  -0.6190f,0.6198f,0.4824f,  -0.6168f,0.6275f,0.4751f,  -0.5117f,0.6662f,0.5425f,  -0.5087f,0.6783f,0.5303f,  -0.3938f,0.7095f,0.5844f,  -0.3903f,0.7255f,0.5669f,  -0.2680f,0.7481f,0.6071f,  
	-0.2643f,0.7675f,0.5840f,  -0.1369f,0.7805f,0.6100f,  -0.1335f,0.8027f,0.5812f,  -0.0034f,0.8050f,0.5933f,  -0.0007f,0.8292f,0.5590f,  0.1296f,0.8198f,0.5578f,  0.1313f,0.8451f,0.5182f,  0.2594f,0.8233f,0.5049f,  0.2597f,0.8488f,0.4605f,  0.3834f,0.8138f,0.4367f,  0.3821f,0.8386f,0.3882f,  0.4991f,0.7901f,0.3559f,  0.4961f,0.8131f,0.3044f,  0.6044f,0.7510f,0.2658f,  0.5998f,0.7715f,0.2124f,  
	0.6976f,0.6960f,0.1700f,  0.6914f,0.7131f,0.1161f,  0.7774f,0.6249f,0.0725f,  0.7697f,0.6381f,0.0196f,  0.8426f,0.5381f,-0.0225f,  0.8339f,0.5471f,-0.0728f,  0.8927f,0.4367f,-0.1108f,  0.8834f,0.4415f,-0.1572f,  0.9275f,0.3226f,-0.1887f,  0.9180f,0.3233f,-0.2297f,  0.9471f,0.1979f,-0.2526f,  0.9378f,0.1950f,-0.2871f,  0.9518f,0.0656f,-0.2997f,  0.9432f,0.0598f,-0.3268f,  0.9421f,-0.0709f,-0.3278f,  
	0.9346f,-0.0788f,-0.3468f,  0.9188f,-0.2081f,-0.3355f,  0.9127f,-0.2171f,-0.3462f,  0.8827f,-0.3421f,-0.3223f,  0.8782f,-0.3512f,-0.3248f,  0.8346f,-0.4691f,-0.2888f,  0.8317f,-0.4774f,-0.2834f,  0.7754f,-0.5856f,-0.2362f,  0.7742f,-0.5922f,-0.2235f,  0.7061f,-0.6883f,-0.1665f,  0.7064f,-0.6923f,-0.1474f,  0.7029f,-0.5769f,-0.4161f,  0.7061f,-0.5735f,-0.4154f,  0.6705f,-0.5199f,-0.5293f,  0.6694f,-0.5212f,-0.5295f,  
	0.6184f,-0.4630f,-0.6350f,  0.6134f,-0.4689f,-0.6355f,  0.5481f,-0.4071f,-0.7306f,  0.5396f,-0.4174f,-0.7311f,  0.4617f,-0.3527f,-0.8139f,  0.4502f,-0.3673f,-0.8139f,  0.3614f,-0.3003f,-0.8827f,  0.3474f,-0.3189f,-0.8818f,  0.2497f,-0.2502f,-0.9354f,  0.2337f,-0.2723f,-0.9334f,  0.1296f,-0.2023f,-0.9707f,  0.1120f,-0.2275f,-0.9673f,  0.0038f,-0.1566f,-0.9877f,  -0.0148f,-0.1844f,-0.9827f,  -0.1244f,-0.1128f,-0.9858f,  
	-0.1437f,-0.1427f,-0.9793f,  -0.2521f,-0.0706f,-0.9651f,  -0.2715f,-0.1020f,-0.9570f,  -0.3762f,-0.0296f,-0.9261f,  -0.3954f,-0.0619f,-0.9164f,  -0.4938f,0.0107f,-0.8695f,  -0.5125f,-0.0219f,-0.8584f,  -0.6022f,0.0508f,-0.7967f,  -0.6200f,0.0185f,-0.7844f,  -0.6989f,0.0912f,-0.7094f,  -0.7154f,0.0598f,-0.6961f,  -0.7816f,0.1324f,-0.6096f,  -0.7966f,0.1025f,-0.5957f,  -0.8484f,0.1748f,-0.4996f,  -0.8618f,0.1470f,-0.4855f,  
	-0.8978f,0.2188f,-0.3822f,  -0.9093f,0.1937f,-0.3683f,  -0.9286f,0.2647f,-0.2600f,  -0.9382f,0.2426f,-0.2468f,  -0.9401f,0.3125f,-0.1362f,  -0.9477f,0.2940f,-0.1242f,  -0.9319f,0.3623f,-0.0137f,  -0.9376f,0.3478f,-0.0036f,  -0.9043f,0.4139f,0.1043f,  -0.9080f,0.4037f,0.1120f,  -0.8578f,0.4669f,0.2149f,  -0.8597f,0.4611f,0.2196f,  -0.7934f,0.5207f,0.3153f,  -0.7937f,0.5196f,0.3163f,  -0.7125f,0.5746f,0.4028f,  
	-0.7115f,0.5781f,0.3996f,  -0.6168f,0.6275f,0.4751f,  -0.6148f,0.6355f,0.4671f,  -0.5087f,0.6783f,0.5303f,  -0.5059f,0.6905f,0.5170f,  -0.3903f,0.7255f,0.5669f,  -0.3873f,0.7415f,0.5480f,  -0.2643f,0.7675f,0.5840f,  -0.2614f,0.7867f,0.5592f,  -0.1335f,0.8027f,0.5812f,  -0.1311f,0.8244f,0.5506f,  -0.0007f,0.8292f,0.5590f,  0.0009f,0.8526f,0.5226f,  0.1313f,0.8451f,0.5182f,  0.1317f,0.8693f,0.4764f,  
	0.2597f,0.8488f,0.4605f,  0.2587f,0.8728f,0.4140f,  0.3821f,0.8386f,0.3882f,  0.3794f,0.8613f,0.3378f,  0.4961f,0.8131f,0.3044f,  0.4917f,0.8337f,0.2512f,  0.5998f,0.7715f,0.2124f,  0.5937f,0.7891f,0.1577f,  0.6914f,0.7131f,0.1161f,  0.6838f,0.7271f,0.0614f,  0.7697f,0.6381f,0.0196f,  0.7609f,0.6480f,-0.0336f,  0.8339f,0.5471f,-0.0728f,  0.8242f,0.5528f,-0.1230f,  0.8834f,0.4415f,-0.1572f,  
	0.8732f,0.4431f,-0.2029f,  0.9180f,0.3233f,-0.2297f,  0.9079f,0.3210f,-0.2697f,  0.9378f,0.1950f,-0.2871f,  0.9282f,0.1895f,-0.3202f,  0.9432f,0.0598f,-0.3268f,  0.9345f,0.0519f,-0.3522f,  0.9346f,-0.0788f,-0.3468f,  0.9272f,-0.0884f,-0.3641f,  0.9127f,-0.2171f,-0.3462f,  0.9068f,-0.2273f,-0.3552f,  0.8782f,-0.3512f,-0.3248f,  0.8738f,-0.3610f,-0.3256f,  0.8317f,-0.4774f,-0.2834f,  0.8291f,-0.4860f,-0.2766f,  
	0.7742f,-0.5922f,-0.2235f,  0.7731f,-0.5986f,-0.2097f,  0.7064f,-0.6923f,-0.1474f,  0.7067f,-0.6959f,-0.1277f,  0.7061f,-0.5735f,-0.4154f,  0.7089f,-0.5704f,-0.4149f,  0.6694f,-0.5212f,-0.5295f,  0.6681f,-0.5227f,-0.5296f,  0.6134f,-0.4689f,-0.6355f,  0.6083f,-0.4750f,-0.6358f,  0.5396f,-0.4174f,-0.7311f,  0.5313f,-0.4280f,-0.7311f,  0.4502f,-0.3673f,-0.8139f,  0.4390f,-0.3821f,-0.8132f,  0.3474f,-0.3189f,-0.8818f,  
	0.3338f,-0.3376f,-0.8801f,  0.2337f,-0.2723f,-0.9334f,  0.2182f,-0.2945f,-0.9304f,  0.1120f,-0.2275f,-0.9673f,  0.0950f,-0.2528f,-0.9628f,  -0.0148f,-0.1844f,-0.9827f,  -0.0328f,-0.2123f,-0.9767f,  -0.1437f,-0.1427f,-0.9793f,  -0.1623f,-0.1726f,-0.9715f,  -0.2715f,-0.1020f,-0.9570f,  -0.2903f,-0.1334f,-0.9476f,  -0.3954f,-0.0619f,-0.9164f,  -0.4140f,-0.0942f,-0.9054f,  -0.5125f,-0.0219f,-0.8584f,  -0.5304f,-0.0545f,-0.8460f,  
	-0.6200f,0.0185f,-0.7844f,  -0.6370f,-0.0138f,-0.7708f,  -0.7154f,0.0598f,-0.6961f,  -0.7312f,0.0285f,-0.6816f,  -0.7966f,0.1025f,-0.5957f,  -0.8110f,0.0727f,-0.5805f,  -0.8618f,0.1470f,-0.4855f,  -0.8745f,0.1193f,-0.4702f,  -0.9093f,0.1937f,-0.3683f,  -0.9202f,0.1686f,-0.3532f,  -0.9382f,0.2426f,-0.2468f,  -0.9472f,0.2207f,-0.2326f,  -0.9477f,0.2940f,-0.1242f,  -0.9548f,0.2757f,-0.1115f,  -0.9376f,0.3478f,-0.0036f,  
	-0.9427f,0.3335f,0.0072f,  -0.9080f,0.4037f,0.1120f,  -0.9114f,0.3936f,0.1201f,  -0.8597f,0.4611f,0.2196f,  -0.8614f,0.4556f,0.2244f,  -0.7937f,0.5196f,0.3163f,  -0.7939f,0.5187f,0.3172f,  -0.7115f,0.5781f,0.3996f,  -0.7105f,0.5818f,0.3959f,  -0.6148f,0.6355f,0.4671f,  -0.6130f,0.6436f,0.4582f,  -0.5059f,0.6905f,0.5170f,  -0.5036f,0.7027f,0.5026f,  -0.3873f,0.7415f,0.5480f,  -0.3848f,0.7573f,0.5276f,  
	-0.2614f,0.7867f,0.5592f,  -0.2592f,0.8056f,0.5328f,  -0.1311f,0.8244f,0.5506f,  -0.1295f,0.8455f,0.5180f,  0.0009f,0.8526f,0.5226f,  0.0013f,0.8750f,0.4842f,  0.1317f,0.8693f,0.4764f,  0.1308f,0.8921f,0.4326f,  0.2587f,0.8728f,0.4140f,  0.2562f,0.8949f,0.3655f,  0.3794f,0.8613f,0.3378f,  0.3752f,0.8818f,0.2857f,  0.4917f,0.8337f,0.2512f,  0.4858f,0.8517f,0.1966f,  0.5937f,0.7891f,0.1577f,  
	0.5861f,0.8038f,0.1021f,  0.6838f,0.7271f,0.0614f,  0.6748f,0.7379f,0.0062f,  0.7609f,0.6480f,-0.0336f,  0.7509f,0.6547f,-0.0868f,  0.8242f,0.5528f,-0.1230f,  0.8135f,0.5553f,-0.1727f,  0.8732f,0.4431f,-0.2029f,  0.8624f,0.4415f,-0.2476f,  0.9079f,0.3210f,-0.2697f,  0.8973f,0.3160f,-0.3082f,  0.9282f,0.1895f,-0.3202f,  0.9184f,0.1816f,-0.3516f,  0.9345f,0.0519f,-0.3522f,  0.9258f,0.0418f,-0.3757f,  
	0.9272f,-0.0884f,-0.3641f,  0.9198f,-0.0994f,-0.3795f,  0.9068f,-0.2273f,-0.3552f,  0.9010f,-0.2385f,-0.3623f,  0.8738f,-0.3610f,-0.3256f,  0.8697f,-0.3715f,-0.3249f,  0.8291f,-0.4860f,-0.2766f,  0.8266f,-0.4947f,-0.2684f,  0.7731f,-0.5986f,-0.2097f,  0.7721f,-0.6048f,-0.1951f,  0.7067f,-0.6959f,-0.1277f,  0.7069f,-0.6991f,-0.1075f,  0.7089f,-0.5704f,-0.4149f,  0.7113f,-0.5675f,-0.4146f,  0.6681f,-0.5227f,-0.5296f,  
	0.6666f,-0.5245f,-0.5296f,  0.6083f,-0.4750f,-0.6358f,  0.6033f,-0.4814f,-0.6358f,  0.5313f,-0.4280f,-0.7311f,  0.5232f,-0.4388f,-0.7305f,  0.4390f,-0.3821f,-0.8132f,  0.4282f,-0.3971f,-0.8118f,  0.3338f,-0.3376f,-0.8801f,  0.3206f,-0.3565f,-0.8776f,  0.2182f,-0.2945f,-0.9304f,  0.2031f,-0.3169f,-0.9265f,  0.0950f,-0.2528f,-0.9628f,  0.0786f,-0.2782f,-0.9573f,  -0.0328f,-0.2123f,-0.9767f,  -0.0502f,-0.2402f,-0.9694f,  
	-0.1623f,-0.1726f,-0.9715f,  -0.1802f,-0.2026f,-0.9625f,  -0.2903f,-0.1334f,-0.9476f,  -0.3084f,-0.1649f,-0.9369f,  -0.4140f,-0.0942f,-0.9054f,  -0.4318f,-0.1265f,-0.8931f,  -0.5304f,-0.0545f,-0.8460f,  -0.5476f,-0.0871f,-0.8322f,  -0.6370f,-0.0138f,-0.7708f,  -0.6533f,-0.0461f,-0.7557f,  -0.7312f,0.0285f,-0.6816f,  -0.7463f,-0.0029f,-0.6656f,  -0.8110f,0.0727f,-0.5805f,  -0.8246f,0.0429f,-0.5640f,  -0.8745f,0.1193f,-0.4702f,  
	-0.8865f,0.0916f,-0.4536f,  -0.9202f,0.1686f,-0.3532f,  -0.9305f,0.1435f,-0.3370f,  -0.9472f,0.2207f,-0.2326f,  -0.9556f,0.1989f,-0.2174f,  -0.9548f,0.2757f,-0.1115f,  -0.9613f,0.2576f,-0.0978f,  -0.9427f,0.3335f,0.0072f,  -0.9474f,0.3194f,0.0186f,  -0.9114f,0.3936f,0.1201f,  -0.9144f,0.3839f,0.1287f,  -0.8614f,0.4556f,0.2244f,  -0.8628f,0.4505f,0.2294f,  -0.7939f,0.5187f,0.3172f,  -0.7940f,0.5181f,0.3179f,  
	-0.7105f,0.5818f,0.3959f,  -0.7095f,0.5858f,0.3917f,  -0.6130f,0.6436f,0.4582f,  -0.6113f,0.6519f,0.4486f,  -0.5036f,0.7027f,0.5026f,  -0.5016f,0.7150f,0.4870f,  -0.3848f,0.7573f,0.5276f,  -0.3829f,0.7730f,0.5059f,  -0.2592f,0.8056f,0.5328f,  -0.2578f,0.8239f,0.5047f,  -0.1295f,0.8455f,0.5180f,  -0.1290f,0.8657f,0.4837f,  0.0013f,0.8750f,0.4842f,  0.0007f,0.8961f,0.4439f,  0.1308f,0.8921f,0.4326f,  
	0.1286f,0.9131f,0.3869f,  0.2562f,0.8949f,0.3655f,  0.2523f,0.9148f,0.3154f,  0.3752f,0.8818f,0.2857f,  0.3696f,0.8997f,0.2322f,  0.4858f,0.8517f,0.1966f,  0.4784f,0.8667f,0.1410f,  0.5861f,0.8038f,0.1021f,  0.5773f,0.8153f,0.0459f,  0.6748f,0.7379f,0.0062f,  0.6647f,0.7455f,-0.0490f,  0.7509f,0.6547f,-0.0868f,  0.7400f,0.6580f,-0.1395f,  0.8135f,0.5553f,-0.1727f,  0.8022f,0.5545f,-0.2213f,  
	0.8624f,0.4415f,-0.2476f,  0.8511f,0.4370f,-0.2909f,  0.8973f,0.3160f,-0.3082f,  0.8866f,0.3082f,-0.3450f,  0.9184f,0.1816f,-0.3516f,  0.9086f,0.1713f,-0.3810f,  0.9258f,0.0418f,-0.3757f,  0.9172f,0.0300f,-0.3973f,  0.9198f,-0.0994f,-0.3795f,  0.9128f,-0.1119f,-0.3928f,  0.9010f,-0.2385f,-0.3623f,  0.8956f,-0.2506f,-0.3676f,  0.8697f,-0.3715f,-0.3249f,  0.8659f,-0.3823f,-0.3225f,  0.8266f,-0.4947f,-0.2684f,  
	0.8243f,-0.5034f,-0.2591f,  0.7721f,-0.6048f,-0.1951f,  0.7712f,-0.6108f,-0.1795f,  0.7069f,-0.6991f,-0.1075f,  0.7072f,-0.7017f,-0.0868f,  0.7113f,-0.5675f,-0.4146f,  0.7135f,-0.5650f,-0.4144f,  0.6666f,-0.5245f,-0.5296f,  0.6650f,-0.5266f,-0.5296f,  0.6033f,-0.4814f,-0.6358f,  0.5984f,-0.4881f,-0.6354f,  0.5232f,-0.4388f,-0.7305f,  0.5152f,-0.4499f,-0.7295f,  0.4282f,-0.3971f,-0.8118f,  0.4175f,-0.4123f,-0.8097f,  
	0.3206f,-0.3565f,-0.8776f,  0.3078f,-0.3755f,-0.8742f,  0.2031f,-0.3169f,-0.9265f,  0.1886f,-0.3394f,-0.9215f,  0.0786f,-0.2782f,-0.9573f,  0.0627f,-0.3037f,-0.9507f,  -0.0502f,-0.2402f,-0.9694f,  -0.0670f,-0.2683f,-0.9610f,  -0.1802f,-0.2026f,-0.9625f,  -0.1975f,-0.2327f,-0.9523f,  -0.3084f,-0.1649f,-0.9369f,  -0.3257f,-0.1964f,-0.9248f,  -0.4318f,-0.1265f,-0.8931f,  -0.4489f,-0.1590f,-0.8793f,  -0.5476f,-0.0871f,-0.8322f,  
	-0.5641f,-0.1198f,-0.8170f,  -0.6533f,-0.0461f,-0.7557f,  -0.6688f,-0.0784f,-0.7393f,  -0.7463f,-0.0029f,-0.6656f,  -0.7607f,-0.0343f,-0.6482f,  -0.8246f,0.0429f,-0.5640f,  -0.8376f,0.0130f,-0.5462f,  -0.8865f,0.0916f,-0.4536f,  -0.8978f,0.0639f,-0.4357f,  -0.9305f,0.1435f,-0.3370f,  -0.9401f,0.1186f,-0.3196f,  -0.9556f,0.1989f,-0.2174f,  -0.9634f,0.1772f,-0.2011f,  -0.9613f,0.2576f,-0.0978f,  -0.9673f,0.2397f,-0.0832f,  
	-0.9474f,0.3194f,0.0186f,  -0.9517f,0.3056f,0.0307f,  -0.9144f,0.3839f,0.1287f,  -0.9170f,0.3745f,0.1376f,  -0.8628f,0.4505f,0.2294f,  -0.8640f,0.4456f,0.2345f,  -0.7940f,0.5181f,0.3179f,  -0.7940f,0.5179f,0.3184f,  -0.7095f,0.5858f,0.3917f,  -0.7086f,0.5900f,0.3870f,  -0.6113f,0.6519f,0.4486f,  -0.6099f,0.6604f,0.4381f,  -0.5016f,0.7150f,0.4870f,  -0.5000f,0.7272f,0.4703f,  -0.3829f,0.7730f,0.5059f,  
	-0.3816f,0.7883f,0.4827f,  -0.2578f,0.8239f,0.5047f,  -0.2572f,0.8416f,0.4749f,  -0.1290f,0.8657f,0.4837f,  -0.1295f,0.8848f,0.4475f,  0.0007f,0.8961f,0.4439f,  -0.0012f,0.9157f,0.4018f,  0.1286f,0.9131f,0.3869f,  0.1251f,0.9322f,0.3396f,  0.2523f,0.9148f,0.3154f,  0.2471f,0.9324f,0.2638f,  0.3696f,0.8997f,0.2322f,  0.3626f,0.9149f,0.1776f,  0.4784f,0.8667f,0.1410f,  0.4699f,0.8787f,0.0848f,  
	0.5773f,0.8153f,0.0459f,  0.5673f,0.8235f,-0.0105f,  0.6647f,0.7455f,-0.0490f,  0.6537f,0.7495f,-0.1039f,  0.7400f,0.6580f,-0.1395f,  0.7283f,0.6580f,-0.1913f,  0.8022f,0.5545f,-0.2213f,  0.7904f,0.5505f,-0.2687f,  0.8511f,0.4370f,-0.2909f,  0.8396f,0.4295f,-0.3325f,  0.8866f,0.3082f,-0.3450f,  0.8758f,0.2979f,-0.3797f,  0.9086f,0.1713f,-0.3810f,  0.8989f,0.1590f,-0.4083f,  0.9172f,0.0300f,-0.3973f,  
	0.9090f,0.0166f,-0.4166f,  0.9128f,-0.1119f,-0.3928f,  0.9061f,-0.1255f,-0.4041f,  0.8956f,-0.2506f,-0.3676f,  0.8905f,-0.2634f,-0.3711f,  0.8659f,-0.3823f,-0.3225f,  0.8624f,-0.3933f,-0.3186f,  0.8243f,-0.5034f,-0.2591f,  0.8222f,-0.5120f,-0.2485f,  0.7712f,-0.6108f,-0.1795f,  0.7704f,-0.6164f,-0.1632f,  0.7072f,-0.7017f,-0.0868f,  0.7073f,-0.7038f,-0.0656f,  0.7135f,-0.5650f,-0.4144f,  0.7153f,-0.5627f,-0.4144f,  
	0.6650f,-0.5266f,-0.5296f,  0.6633f,-0.5289f,-0.5295f,  0.5984f,-0.4881f,-0.6354f,  0.5934f,-0.4949f,-0.6348f,  0.5152f,-0.4499f,-0.7295f,  0.5073f,-0.4612f,-0.7280f,  0.4175f,-0.4123f,-0.8097f,  0.4072f,-0.4278f,-0.8070f,  0.3078f,-0.3755f,-0.8742f,  0.2954f,-0.3948f,-0.8700f,  0.1886f,-0.3394f,-0.9215f,  0.1745f,-0.3621f,-0.9157f,  0.0627f,-0.3037f,-0.9507f,  0.0473f,-0.3294f,-0.9430f,  -0.0670f,-0.2683f,-0.9610f,  
	-0.0832f,-0.2965f,-0.9514f,  -0.1975f,-0.2327f,-0.9523f,  -0.2141f,-0.2629f,-0.9408f,  -0.3257f,-0.1964f,-0.9248f,  -0.3424f,-0.2281f,-0.9114f,  -0.4489f,-0.1590f,-0.8793f,  -0.4653f,-0.1915f,-0.8642f,  -0.5641f,-0.1198f,-0.8170f,  -0.5799f,-0.1526f,-0.8003f,  -0.6688f,-0.0784f,-0.7393f,  -0.6836f,-0.1109f,-0.7213f,  -0.7607f,-0.0343f,-0.6482f,  -0.7743f,-0.0657f,-0.6294f,  -0.8376f,0.0130f,-0.5462f,  -0.8498f,-0.0168f,-0.5269f,  
	-0.8978f,0.0639f,-0.4357f,  -0.9085f,0.0364f,-0.4164f,  -0.9401f,0.1186f,-0.3196f,  -0.9490f,0.0939f,-0.3010f,  -0.9634f,0.1772f,-0.2011f,  -0.9706f,0.1558f,-0.1837f,  -0.9673f,0.2397f,-0.0832f,  -0.9727f,0.2220f,-0.0678f,  -0.9517f,0.3056f,0.0307f,  -0.9554f,0.2921f,0.0435f,  -0.9170f,0.3745f,0.1376f,  -0.9192f,0.3654f,0.1470f,  -0.8640f,0.4456f,0.2345f,  -0.8648f,0.4411f,0.2397f,  -0.7940f,0.5179f,0.3184f,  
	-0.7938f,0.5180f,0.3187f,  -0.7086f,0.5900f,0.3870f,  -0.7077f,0.5945f,0.3817f,  -0.6099f,0.6604f,0.4381f,  -0.6087f,0.6689f,0.4267f,  -0.5000f,0.7272f,0.4703f,  -0.4989f,0.7392f,0.4524f,  -0.3816f,0.7883f,0.4827f,  -0.3810f,0.8032f,0.4580f,  -0.2572f,0.8416f,0.4749f,  -0.2575f,0.8585f,0.4436f,  -0.1295f,0.8848f,0.4475f,  -0.1310f,0.9028f,0.4097f,  -0.0012f,0.9157f,0.4018f,  -0.0043f,0.9337f,0.3581f,  
	0.1251f,0.9322f,0.3396f,  0.1204f,0.9492f,0.2909f,  0.2471f,0.9324f,0.2638f,  0.2406f,0.9474f,0.2111f,  0.3626f,0.9149f,0.1776f,  0.3544f,0.9271f,0.1222f,  0.4699f,0.8787f,0.0848f,  0.4601f,0.8874f,0.0283f,  0.5673f,0.8235f,-0.0105f,  0.5564f,0.8283f,-0.0666f,  0.6537f,0.7495f,-0.1039f,  0.6420f,0.7502f,-0.1580f,  0.7283f,0.6580f,-0.1913f,  0.7162f,0.6547f,-0.2418f,  0.7904f,0.5505f,-0.2687f,  
	0.7784f,0.5435f,-0.3143f,  0.8396f,0.4295f,-0.3325f,  0.8281f,0.4193f,-0.3720f,  0.8758f,0.2979f,-0.3797f,  0.8652f,0.2853f,-0.4123f,  0.8989f,0.1590f,-0.4083f,  0.8896f,0.1448f,-0.4332f,  0.9090f,0.0166f,-0.4166f,  0.9011f,0.0018f,-0.4336f,  0.9061f,-0.1255f,-0.4041f,  0.8998f,-0.1400f,-0.4133f,  0.8905f,-0.2634f,-0.3711f,  0.8858f,-0.2766f,-0.3727f,  0.8624f,-0.3933f,-0.3186f,  0.8592f,-0.4045f,-0.3132f,  
	0.8222f,-0.5120f,-0.2485f,  0.8204f,-0.5205f,-0.2368f,  0.7704f,-0.6164f,-0.1632f,  0.7696f,-0.6216f,-0.1461f,  0.7073f,-0.7038f,-0.0656f,  0.7075f,-0.7054f,-0.0442f,  0.7153f,-0.5627f,-0.4144f,  0.7168f,-0.5607f,-0.4144f,  0.6633f,-0.5289f,-0.5295f,  0.6614f,-0.5315f,-0.5292f,  0.5934f,-0.4949f,-0.6348f,  0.5884f,-0.5021f,-0.6338f,  0.5073f,-0.4612f,-0.7280f,  0.4996f,-0.4726f,-0.7259f,  0.4072f,-0.4278f,-0.8070f,  
	0.3971f,-0.4434f,-0.8036f,  0.2954f,-0.3948f,-0.8700f,  0.2833f,-0.4142f,-0.8650f,  0.1745f,-0.3621f,-0.9157f,  0.1609f,-0.3849f,-0.9088f,  0.0473f,-0.3294f,-0.9430f,  0.0325f,-0.3553f,-0.9342f,  -0.0832f,-0.2965f,-0.9514f,  -0.0988f,-0.3249f,-0.9406f,  -0.2141f,-0.2629f,-0.9408f,  -0.2301f,-0.2933f,-0.9279f,  -0.3424f,-0.2281f,-0.9114f,  -0.3585f,-0.2599f,-0.8966f,  -0.4653f,-0.1915f,-0.8642f,  -0.4810f,-0.2242f,-0.8476f,  
	-0.5799f,-0.1526f,-0.8003f,  -0.5949f,-0.1856f,-0.7821f,  -0.6836f,-0.1109f,-0.7213f,  -0.6977f,-0.1434f,-0.7019f,  -0.7743f,-0.0657f,-0.6294f,  -0.7872f,-0.0972f,-0.6090f,  -0.8498f,-0.0168f,-0.5269f,  -0.8613f,-0.0466f,-0.5060f,  -0.9085f,0.0364f,-0.4164f,  -0.9183f,0.0089f,-0.3957f,  -0.9490f,0.0939f,-0.3010f,  -0.9572f,0.0693f,-0.2810f,  -0.9706f,0.1558f,-0.1837f,  -0.9771f,0.1346f,-0.1651f,  -0.9727f,0.2220f,-0.0678f,  
	-0.9775f,0.2047f,-0.0513f,  -0.9554f,0.2921f,0.0435f,  -0.9586f,0.2790f,0.0571f,  -0.9192f,0.3654f,0.1470f,  -0.9209f,0.3568f,0.1568f,  -0.8648f,0.4411f,0.2397f,  -0.8654f,0.4371f,0.2450f,  -0.7938f,0.5180f,0.3187f,  -0.7935f,0.5184f,0.3187f,  -0.7077f,0.5945f,0.3817f,  -0.7069f,0.5992f,0.3758f,  -0.6087f,0.6689f,0.4267f,  -0.6077f,0.6774f,0.4144f,  -0.4989f,0.7392f,0.4524f,  -0.4983f,0.7510f,0.4333f,  
	-0.3810f,0.8032f,0.4580f,  -0.3810f,0.8174f,0.4320f,  -0.2575f,0.8585f,0.4436f,  -0.2586f,0.8743f,0.4107f,  -0.1310f,0.9028f,0.4097f,  -0.1335f,0.9192f,0.3704f,  -0.0043f,0.9337f,0.3581f,  -0.0084f,0.9497f,0.3130f,  0.1204f,0.9492f,0.2909f,  0.1145f,0.9638f,0.2410f,  0.2406f,0.9474f,0.2111f,  0.2329f,0.9596f,0.1576f,  0.3544f,0.9271f,0.1222f,  0.3451f,0.9362f,0.0665f,  0.4601f,0.8874f,0.0283f,  
	0.4495f,0.8928f,-0.0281f,  0.5564f,0.8283f,-0.0666f,  0.5447f,0.8297f,-0.1220f,  0.6420f,0.7502f,-0.1580f,  0.6297f,0.7476f,-0.2109f,  0.7162f,0.6547f,-0.2418f,  0.7038f,0.6482f,-0.2907f,  0.7784f,0.5435f,-0.3143f,  0.7663f,0.5336f,-0.3579f,  0.8281f,0.4193f,-0.3720f,  0.8168f,0.4066f,-0.4093f,  0.8652f,0.2853f,-0.4123f,  0.8550f,0.2705f,-0.4424f,  0.8896f,0.1448f,-0.4332f,  0.8807f,0.1290f,-0.4557f,  
	0.9011f,0.0018f,-0.4336f,  0.8937f,-0.0142f,-0.4484f,  0.8998f,-0.1400f,-0.4133f,  0.8940f,-0.1551f,-0.4204f,  0.8858f,-0.2766f,-0.3727f,  0.8815f,-0.2901f,-0.3725f,  0.8592f,-0.4045f,-0.3132f,  0.8564f,-0.4157f,-0.3063f,  0.8204f,-0.5205f,-0.2368f,  0.8187f,-0.5287f,-0.2240f,  0.7696f,-0.6216f,-0.1461f,  0.7690f,-0.6263f,-0.1284f,  0.7075f,-0.7054f,-0.0442f,  0.7076f,-0.7063f,-0.0226f,  0.7168f,-0.5607f,-0.4144f,  
	0.7181f,-0.5590f,-0.4146f,  0.6614f,-0.5315f,-0.5292f,  0.6594f,-0.5344f,-0.5288f,  0.5884f,-0.5021f,-0.6338f,  0.5835f,-0.5095f,-0.6325f,  0.4996f,-0.4726f,-0.7259f,  0.4920f,-0.4844f,-0.7234f,  0.3971f,-0.4434f,-0.8036f,  0.3873f,-0.4592f,-0.7995f,  0.2833f,-0.4142f,-0.8650f,  0.2716f,-0.4338f,-0.8591f,  0.1609f,-0.3849f,-0.9088f,  0.1476f,-0.4080f,-0.9010f,  0.0325f,-0.3553f,-0.9342f,  0.0182f,-0.3814f,-0.9243f,  
	-0.0988f,-0.3249f,-0.9406f,  -0.1139f,-0.3535f,-0.9285f,  -0.2301f,-0.2933f,-0.9279f,  -0.2455f,-0.3239f,-0.9137f,  -0.3585f,-0.2599f,-0.8966f,  -0.3738f,-0.2919f,-0.8804f,  -0.4810f,-0.2242f,-0.8476f,  -0.4960f,-0.2570f,-0.8294f,  -0.5949f,-0.1856f,-0.7821f,  -0.6092f,-0.2186f,-0.7623f,  -0.6977f,-0.1434f,-0.7019f,  -0.7111f,-0.1760f,-0.6807f,  -0.7872f,-0.0972f,-0.6090f,  -0.7993f,-0.1287f,-0.5870f,  -0.8613f,-0.0466f,-0.5060f,  
	-0.8719f,-0.0763f,-0.4836f,  -0.9183f,0.0089f,-0.3957f,  -0.9275f,-0.0185f,-0.3735f,  -0.9572f,0.0693f,-0.2810f,  -0.9647f,0.0449f,-0.2596f,  -0.9771f,0.1346f,-0.1651f,  -0.9828f,0.1137f,-0.1453f,  -0.9775f,0.2047f,-0.0513f,  -0.9816f,0.1877f,-0.0339f,  -0.9586f,0.2790f,0.0571f,  -0.9612f,0.2663f,0.0713f,  -0.9209f,0.3568f,0.1568f,  -0.9223f,0.3486f,0.1670f,  -0.8654f,0.4371f,0.2450f,  -0.8657f,0.4334f,0.2503f,  
	-0.7935f,0.5184f,0.3187f,  -0.7931f,0.5192f,0.3185f,  -0.7069f,0.5992f,0.3758f,  -0.7062f,0.6041f,0.3694f,  -0.6077f,0.6774f,0.4144f,  -0.6071f,0.6859f,0.4012f,  -0.4983f,0.7510f,0.4333f,  -0.4981f,0.7624f,0.4131f,  -0.3810f,0.8174f,0.4320f,  -0.3817f,0.8310f,0.4046f,  -0.2586f,0.8743f,0.4107f,  -0.2606f,0.8890f,0.3764f,  -0.1335f,0.9192f,0.3704f,  -0.1371f,0.9341f,0.3298f,  -0.0084f,0.9497f,0.3130f,  
	-0.0136f,0.9637f,0.2668f,  0.1145f,0.9638f,0.2410f,  0.1075f,0.9758f,0.1902f,  0.2329f,0.9596f,0.1576f,  0.2242f,0.9690f,0.1036f,  0.3451f,0.9362f,0.0665f,  0.3350f,0.9422f,0.0108f,  0.4495f,0.8928f,-0.0281f,  0.4382f,0.8950f,-0.0839f,  0.5447f,0.8297f,-0.1220f,  0.5325f,0.8278f,-0.1763f,  0.6297f,0.7476f,-0.2109f,  0.6172f,0.7418f,-0.2622f,  0.7038f,0.6482f,-0.2907f,  0.6914f,0.6387f,-0.3377f,  
	0.7663f,0.5336f,-0.3579f,  0.7544f,0.5210f,-0.3993f,  0.8168f,0.4066f,-0.4093f,  0.8059f,0.3916f,-0.4441f,  0.8550f,0.2705f,-0.4424f,  0.8453f,0.2540f,-0.4701f,  0.8807f,0.1290f,-0.4557f,  0.8724f,0.1119f,-0.4758f,  0.8937f,-0.0142f,-0.4484f,  0.8869f,-0.0310f,-0.4609f,  0.8940f,-0.1551f,-0.4204f,  0.8887f,-0.1708f,-0.4254f,  0.8815f,-0.2901f,-0.3725f,  0.8777f,-0.3038f,-0.3706f,  0.8564f,-0.4157f,-0.3063f,  
	0.8538f,-0.4267f,-0.2981f,  0.8187f,-0.5287f,-0.2240f,  0.8173f,-0.5365f,-0.2103f,  0.7690f,-0.6263f,-0.1284f,  0.7684f,-0.6304f,-0.1102f,  0.7076f,-0.7063f,-0.0226f,  0.7076f,-0.7066f,-0.0009f,  0.7181f,-0.5590f,-0.4146f,  0.7191f,-0.5576f,-0.4147f,  0.6594f,-0.5344f,-0.5288f,  0.6573f,-0.5375f,-0.5282f,  0.5835f,-0.5095f,-0.6325f,  0.5785f,-0.5171f,-0.6308f,  0.4920f,-0.4844f,-0.7234f,  0.4846f,-0.4963f,-0.7203f,  
	0.3873f,-0.4592f,-0.7995f,  0.3777f,-0.4753f,-0.7947f,  0.2716f,-0.4338f,-0.8591f,  0.2602f,-0.4537f,-0.8523f,  0.1476f,-0.4080f,-0.9010f,  0.1348f,-0.4312f,-0.8921f,  0.0182f,-0.3814f,-0.9243f,  0.0043f,-0.4076f,-0.9131f,  -0.1139f,-0.3535f,-0.9285f,  -0.1284f,-0.3823f,-0.9151f,  -0.2455f,-0.3239f,-0.9137f,  -0.2603f,-0.3547f,-0.8980f,  -0.3738f,-0.2919f,-0.8804f,  -0.3885f,-0.3241f,-0.8626f,  -0.4960f,-0.2570f,-0.8294f,  
	-0.5103f,-0.2900f,-0.8096f,  -0.6092f,-0.2186f,-0.7623f,  -0.6228f,-0.2517f,-0.7408f,  -0.7111f,-0.1760f,-0.6807f,  -0.7236f,-0.2086f,-0.6579f,  -0.7993f,-0.1287f,-0.5870f,  -0.8106f,-0.1601f,-0.5633f,  -0.8719f,-0.0763f,-0.4836f,  -0.8818f,-0.1059f,-0.4595f,  -0.9275f,-0.0185f,-0.3735f,  -0.9358f,-0.0456f,-0.3496f,  -0.9647f,0.0449f,-0.2596f,  -0.9713f,0.0208f,-0.2368f,  -0.9828f,0.1137f,-0.1453f,  -0.9879f,0.0933f,-0.1242f,  
	-0.9816f,0.1877f,-0.0339f,  -0.9851f,0.1713f,-0.0154f,  -0.9612f,0.2663f,0.0713f,  -0.9633f,0.2542f,0.0863f,  -0.9223f,0.3486f,0.1670f,  -0.9231f,0.3410f,0.1777f,  -0.8657f,0.4334f,0.2503f,  -0.8657f,0.4303f,0.2558f,  -0.7931f,0.5192f,0.3185f,  -0.7925f,0.5203f,0.3180f,  -0.7062f,0.6041f,0.3694f,  -0.7055f,0.6091f,0.3623f,  -0.6071f,0.6859f,0.4012f,  -0.6067f,0.6944f,0.3871f,  -0.4981f,0.7624f,0.4131f,  
	-0.4984f,0.7734f,0.3917f,  -0.3817f,0.8310f,0.4046f,  -0.3832f,0.8437f,0.3760f,  -0.2606f,0.8890f,0.3764f,  -0.2634f,0.9024f,0.3409f,  -0.1371f,0.9341f,0.3298f,  -0.1415f,0.9471f,0.2880f,  -0.0136f,0.9637f,0.2668f,  -0.0198f,0.9754f,0.2196f,  0.1075f,0.9758f,0.1902f,  0.0995f,0.9853f,0.1390f,  0.2242f,0.9690f,0.1036f,  0.2147f,0.9754f,0.0496f,  0.3350f,0.9422f,0.0108f,  0.3241f,0.9450f,-0.0444f,  
	0.4382f,0.8950f,-0.0839f,  0.4263f,0.8939f,-0.1388f,  0.5325f,0.8278f,-0.1763f,  0.5201f,0.8228f,-0.2293f,  0.6172f,0.7418f,-0.2622f,  0.6047f,0.7329f,-0.3117f,  0.6914f,0.6387f,-0.3377f,  0.6792f,0.6265f,-0.3824f,  0.7544f,0.5210f,-0.3993f,  0.7429f,0.5060f,-0.4382f,  0.8059f,0.3916f,-0.4441f,  0.7954f,0.3747f,-0.4764f,  0.8453f,0.2540f,-0.4701f,  0.8362f,0.2360f,-0.4951f,  0.8724f,0.1119f,-0.4758f,  
	0.8647f,0.0937f,-0.4935f,  0.8869f,-0.0310f,-0.4609f,  0.8807f,-0.0485f,-0.4711f,  0.8887f,-0.1708f,-0.4254f,  0.8840f,-0.1867f,-0.4285f,  0.8777f,-0.3038f,-0.3706f,  0.8743f,-0.3175f,-0.3670f,  0.8538f,-0.4267f,-0.2981f,  0.8517f,-0.4375f,-0.2886f,  0.8173f,-0.5365f,-0.2103f,  0.8161f,-0.5438f,-0.1958f,  0.7684f,-0.6304f,-0.1102f,  0.7679f,-0.6340f,-0.0915f,  0.7076f,-0.7066f,-0.0009f,  0.7191f,-0.5576f,-0.4147f,  
	0.7199f,-0.5564f,-0.4149f,  0.6573f,-0.5375f,-0.5282f,  0.6551f,-0.5410f,-0.5275f,  0.5785f,-0.5171f,-0.6308f,  0.5736f,-0.5250f,-0.6288f,  0.4846f,-0.4963f,-0.7203f,  0.4773f,-0.5086f,-0.7166f,  0.3777f,-0.4753f,-0.7947f,  0.3683f,-0.4916f,-0.7891f,  0.2602f,-0.4537f,-0.8523f,  0.2492f,-0.4737f,-0.8447f,  0.1348f,-0.4312f,-0.8921f,  0.1224f,-0.4548f,-0.8822f,  0.0043f,-0.4076f,-0.9131f,  -0.0090f,-0.4341f,-0.9008f,  
	-0.1284f,-0.3823f,-0.9151f,  -0.1423f,-0.4113f,-0.9003f,  -0.2603f,-0.3547f,-0.8980f,  -0.2744f,-0.3856f,-0.8809f,  -0.3885f,-0.3241f,-0.8626f,  -0.4025f,-0.3565f,-0.8431f,  -0.5103f,-0.2900f,-0.8096f,  -0.5238f,-0.3231f,-0.7882f,  -0.6228f,-0.2517f,-0.7408f,  -0.6356f,-0.2849f,-0.7176f,  -0.7236f,-0.2086f,-0.6579f,  -0.7354f,-0.2412f,-0.6333f,  -0.8106f,-0.1601f,-0.5633f,  -0.8211f,-0.1915f,-0.5378f,  -0.8818f,-0.1059f,-0.4595f,  
	-0.8908f,-0.1353f,-0.4337f,  -0.9358f,-0.0456f,-0.3496f,  -0.9432f,-0.0724f,-0.3242f,  -0.9713f,0.0208f,-0.2368f,  -0.9772f,-0.0028f,-0.2125f,  -0.9879f,0.0933f,-0.1242f,  -0.9921f,0.0733f,-0.1019f,  -0.9851f,0.1713f,-0.0154f,  -0.9878f,0.1554f,0.0040f,  -0.9633f,0.2542f,0.0863f,  -0.9647f,0.2427f,0.1019f,  -0.9231f,0.3410f,0.1777f,  -0.9235f,0.3340f,0.1886f,  -0.8657f,0.4303f,0.2558f,  -0.8654f,0.4277f,0.2612f,  
	-0.7925f,0.5203f,0.3180f,  -0.7919f,0.5219f,0.3171f,  -0.7055f,0.6091f,0.3623f,  -0.7049f,0.6144f,0.3545f,  -0.6067f,0.6944f,0.3871f,  -0.6066f,0.7026f,0.3720f,  -0.4984f,0.7734f,0.3917f,  -0.4992f,0.7839f,0.3692f,  -0.3832f,0.8437f,0.3760f,  -0.3852f,0.8554f,0.3462f,  -0.2634f,0.9024f,0.3409f,  -0.2670f,0.9144f,0.3043f,  -0.1415f,0.9471f,0.2880f,  -0.1468f,0.9583f,0.2453f,  -0.0198f,0.9754f,0.2196f,  
	-0.0269f,0.9847f,0.1719f,  0.0995f,0.9853f,0.1390f,  0.0908f,0.9920f,0.0875f,  0.2147f,0.9754f,0.0496f,  0.2045f,0.9789f,-0.0041f,  0.3241f,0.9450f,-0.0444f,  0.3127f,0.9447f,-0.0989f,  0.4263f,0.8939f,-0.1388f,  0.4141f,0.8897f,-0.1924f,  0.5201f,0.8228f,-0.2293f,  0.5076f,0.8146f,-0.2804f,  0.6047f,0.7329f,-0.3117f,  0.5923f,0.7213f,-0.3591f,  0.6792f,0.6265f,-0.3824f,  0.6673f,0.6118f,-0.4248f,  
	0.7429f,0.5060f,-0.4382f,  0.7320f,0.4889f,-0.4745f,  0.7954f,0.3747f,-0.4764f,  0.7856f,0.3560f,-0.5060f,  0.8362f,0.2360f,-0.4951f,  0.8277f,0.2167f,-0.5176f,  0.8647f,0.0937f,-0.4935f,  0.8577f,0.0746f,-0.5087f,  0.8807f,-0.0485f,-0.4711f,  0.8752f,-0.0665f,-0.4791f,  0.8840f,-0.1867f,-0.4285f,  0.8799f,-0.2029f,-0.4297f,  0.8743f,-0.3175f,-0.3670f,  0.8714f,-0.3310f,-0.3620f,  0.8517f,-0.4375f,-0.2886f,  
	0.8498f,-0.4478f,-0.2781f,  0.8161f,-0.5438f,-0.1958f,  0.8150f,-0.5506f,-0.1806f,  0.7679f,-0.6340f,-0.0915f,  0.7199f,-0.5564f,-0.4149f,  0.7204f,-0.5556f,-0.4151f,  0.6551f,-0.5410f,-0.5275f,  0.6528f,-0.5447f,-0.5265f,  0.5736f,-0.5250f,-0.6288f,  0.5687f,-0.5332f,-0.6263f,  0.4773f,-0.5086f,-0.7166f,  0.4701f,-0.5211f,-0.7124f,  0.3683f,-0.4916f,-0.7891f,  0.3592f,-0.5081f,-0.7828f,  0.2492f,-0.4737f,-0.8447f,  
	0.2384f,-0.4941f,-0.8361f,  0.1224f,-0.4548f,-0.8822f,  0.1105f,-0.4785f,-0.8711f,  -0.0090f,-0.4341f,-0.9008f,  -0.0219f,-0.4609f,-0.8872f,  -0.1423f,-0.4113f,-0.9003f,  -0.1556f,-0.4405f,-0.8841f,  -0.2744f,-0.3856f,-0.8809f,  -0.2879f,-0.4168f,-0.8622f,  -0.4025f,-0.3565f,-0.8431f,  -0.4159f,-0.3890f,-0.8220f,  -0.5238f,-0.3231f,-0.7882f,  -0.5366f,-0.3564f,-0.7649f,  -0.6356f,-0.2849f,-0.7176f,  -0.6475f,-0.3182f,-0.6925f,  
	-0.7354f,-0.2412f,-0.6333f,  -0.7462f,-0.2738f,-0.6068f,  -0.8211f,-0.1915f,-0.5378f,  -0.8306f,-0.2227f,-0.5104f,  -0.8908f,-0.1353f,-0.4337f,  -0.8989f,-0.1645f,-0.4061f,  -0.9432f,-0.0724f,-0.3242f,  -0.9497f,-0.0989f,-0.2971f,  -0.9772f,-0.0028f,-0.2125f,  -0.9821f,-0.0260f,-0.1866f,  -0.9921f,0.0733f,-0.1019f,  -0.9955f,0.0540f,-0.0783f,  -0.9878f,0.1554f,0.0040f,  -0.9898f,0.1403f,0.0245f,  -0.9647f,0.2427f,0.1019f,  
	-0.9655f,0.2320f,0.1183f,  -0.9235f,0.3340f,0.1886f,  -0.9234f,0.3277f,0.1999f,  -0.8654f,0.4277f,0.2612f,  -0.8647f,0.4257f,0.2665f,  -0.7919f,0.5219f,0.3171f,  -0.7911f,0.5239f,0.3158f,  -0.7049f,0.6144f,0.3545f,  -0.7044f,0.6198f,0.3460f,  -0.6066f,0.7026f,0.3720f,  -0.6068f,0.7107f,0.3560f,  -0.4992f,0.7839f,0.3692f,  -0.5005f,0.7938f,0.3456f,  -0.3852f,0.8554f,0.3462f,  -0.3879f,0.8660f,0.3154f,  
	-0.2670f,0.9144f,0.3043f,  -0.2713f,0.9248f,0.2668f,  -0.1468f,0.9583f,0.2453f,  -0.1529f,0.9674f,0.2019f,  -0.0269f,0.9847f,0.1719f,  -0.0347f,0.9917f,0.1239f,  0.0908f,0.9920f,0.0875f,  0.0814f,0.9960f,0.0362f,  0.2045f,0.9789f,-0.0041f,  0.1938f,0.9794f,-0.0573f,  0.3127f,0.9447f,-0.0989f,  0.3010f,0.9414f,-0.1522f,  0.4141f,0.8897f,-0.1924f,  0.4019f,0.8825f,-0.2444f,  0.5076f,0.8146f,-0.2804f,  
	0.4953f,0.8038f,-0.3296f,  0.5923f,0.7213f,-0.3591f,  0.5803f,0.7071f,-0.4041f,  0.6673f,0.6118f,-0.4248f,  0.6560f,0.5948f,-0.4646f,  0.7320f,0.4889f,-0.4745f,  0.7217f,0.4700f,-0.5082f,  0.7856f,0.3560f,-0.5060f,  0.7766f,0.3360f,-0.5330f,  0.8277f,0.2167f,-0.5176f,  0.8200f,0.1964f,-0.5376f,  0.8577f,0.0746f,-0.5087f,  0.8515f,0.0549f,-0.5215f,  0.8752f,-0.0665f,-0.4791f,  0.8704f,-0.0847f,-0.4850f,  
	0.8799f,-0.2029f,-0.4297f,  0.8763f,-0.2189f,-0.4291f,  0.8714f,-0.3310f,-0.3620f,  0.8690f,-0.3442f,-0.3555f,  0.8498f,-0.4478f,-0.2781f,  0.8482f,-0.4577f,-0.2665f,  0.8150f,-0.5506f,-0.1806f,  0.7204f,-0.5556f,-0.4151f,  0.7208f,-0.5550f,-0.4152f,  0.6528f,-0.5447f,-0.5265f,  0.6504f,-0.5487f,-0.5253f,  0.5687f,-0.5332f,-0.6263f,  0.5639f,-0.5416f,-0.6234f,  0.4701f,-0.5211f,-0.7124f,  0.4631f,-0.5338f,-0.7075f,  
	0.3592f,-0.5081f,-0.7828f,  0.3504f,-0.5250f,-0.7757f,  0.2384f,-0.4941f,-0.8361f,  0.2281f,-0.5147f,-0.8265f,  0.1105f,-0.4785f,-0.8711f,  0.0989f,-0.5026f,-0.8589f,  -0.0219f,-0.4609f,-0.8872f,  -0.0342f,-0.4879f,-0.8722f,  -0.1556f,-0.4405f,-0.8841f,  -0.1684f,-0.4700f,-0.8664f,  -0.2879f,-0.4168f,-0.8622f,  -0.3008f,-0.4482f,-0.8418f,  -0.4159f,-0.3890f,-0.8220f,  -0.4284f,-0.4217f,-0.7991f,  -0.5366f,-0.3564f,-0.7649f,  
	-0.5486f,-0.3897f,-0.7397f,  -0.6475f,-0.3182f,-0.6925f,  -0.6586f,-0.3514f,-0.6654f,  -0.7462f,-0.2738f,-0.6068f,  -0.7562f,-0.3063f,-0.5783f,  -0.8306f,-0.2227f,-0.5104f,  -0.8392f,-0.2537f,-0.4810f,  -0.8989f,-0.1645f,-0.4061f,  -0.9060f,-0.1932f,-0.3766f,  -0.9497f,-0.0989f,-0.2971f,  -0.9553f,-0.1248f,-0.2682f,  -0.9821f,-0.0260f,-0.1866f,  -0.9860f,-0.0484f,-0.1593f,  -0.9955f,0.0540f,-0.0783f,  -0.9979f,0.0354f,-0.0534f,  
	-0.9898f,0.1403f,0.0245f,  -0.9910f,0.1260f,0.0459f,  -0.9655f,0.2320f,0.1183f,  -0.9656f,0.2221f,0.1352f,  -0.9234f,0.3277f,0.1999f,  -0.9227f,0.3223f,0.2114f,  -0.8647f,0.4257f,0.2665f,  -0.8637f,0.4245f,0.2718f,  -0.7911f,0.5239f,0.3158f,  -0.7901f,0.5264f,0.3141f,  -0.7044f,0.6198f,0.3460f,  -0.7039f,0.6253f,0.3368f,  -0.6068f,0.7107f,0.3560f,  -0.6072f,0.7185f,0.3391f,  -0.5005f,0.7938f,0.3456f,  
	-0.5022f,0.8029f,0.3211f,  -0.3879f,0.8660f,0.3154f,  -0.3912f,0.8755f,0.2837f,  -0.2713f,0.9248f,0.2668f,  -0.2763f,0.9335f,0.2285f,  -0.1529f,0.9674f,0.2019f,  -0.1596f,0.9744f,0.1582f,  -0.0347f,0.9917f,0.1239f,  -0.0431f,0.9962f,0.0759f,  0.0814f,0.9960f,0.0362f,  0.0716f,0.9973f,-0.0147f,  0.1938f,0.9794f,-0.0573f,  0.1828f,0.9770f,-0.1095f,  0.3010f,0.9414f,-0.1522f,  0.2894f,0.9352f,-0.2042f,  
	0.4019f,0.8825f,-0.2444f,  0.3899f,0.8725f,-0.2945f,  0.4953f,0.8038f,-0.3296f,  0.4834f,0.7903f,-0.3766f,  0.5803f,0.7071f,-0.4041f,  0.5689f,0.6905f,-0.4467f,  0.6560f,0.5948f,-0.4646f,  0.6454f,0.5760f,-0.5017f,  0.7217f,0.4700f,-0.5082f,  0.7122f,0.4496f,-0.5392f,  0.7766f,0.3360f,-0.5330f,  0.7683f,0.3148f,-0.5573f,  0.8200f,0.1964f,-0.5376f,  0.8132f,0.1754f,-0.5550f,  0.8515f,0.0549f,-0.5215f,  
	0.8460f,0.0348f,-0.5320f,  0.8704f,-0.0847f,-0.4850f,  0.8662f,-0.1030f,-0.4889f,  0.8763f,-0.2189f,-0.4291f,  0.8733f,-0.2347f,-0.4269f,  0.8690f,-0.3442f,-0.3555f,  0.8669f,-0.3570f,-0.3478f,  0.8482f,-0.4577f,-0.2665f,  0.7208f,-0.5550f,-0.4152f,  0.7209f,-0.5548f,-0.4153f,  0.6504f,-0.5487f,-0.5253f,  0.6480f,-0.5529f,-0.5238f,  0.5639f,-0.5416f,-0.6234f,  0.5591f,-0.5504f,-0.6201f,  0.4631f,-0.5338f,-0.7075f,  
	0.4563f,-0.5469f,-0.7020f,  0.3504f,-0.5250f,-0.7757f,  0.3417f,-0.5421f,-0.7677f,  0.2281f,-0.5147f,-0.8265f,  0.2180f,-0.5356f,-0.8158f,  0.0989f,-0.5026f,-0.8589f,  0.0878f,-0.5269f,-0.8454f,  -0.0342f,-0.4879f,-0.8722f,  -0.0460f,-0.5152f,-0.8559f,  -0.1684f,-0.4700f,-0.8664f,  -0.1806f,-0.4997f,-0.8471f,  -0.3008f,-0.4482f,-0.8418f,  -0.3129f,-0.4798f,-0.8197f,  -0.4284f,-0.4217f,-0.7991f,  -0.4402f,-0.4545f,-0.7744f,  
	-0.5486f,-0.3897f,-0.7397f,  -0.5597f,-0.4231f,-0.7126f,  -0.6586f,-0.3514f,-0.6654f,  -0.6687f,-0.3846f,-0.6363f,  -0.7562f,-0.3063f,-0.5783f,  -0.7651f,-0.3386f,-0.5477f,  -0.8392f,-0.2537f,-0.4810f,  -0.8468f,-0.2843f,-0.4496f,  -0.9060f,-0.1932f,-0.3766f,  -0.9120f,-0.2215f,-0.3451f,  -0.9553f,-0.1248f,-0.2682f,  -0.9597f,-0.1500f,-0.2375f,  -0.9860f,-0.0484f,-0.1593f,  -0.9890f,-0.0701f,-0.1304f,  -0.9979f,0.0354f,-0.0534f,  
	-0.9995f,0.0178f,-0.0273f,  -0.9910f,0.1260f,0.0459f,  -0.9913f,0.1127f,0.0682f,  -0.9656f,0.2221f,0.1352f,  -0.9650f,0.2132f,0.1527f,  -0.9227f,0.3223f,0.2114f,  -0.9216f,0.3177f,0.2231f,  -0.8637f,0.4245f,0.2718f,  -0.8624f,0.4239f,0.2768f,  -0.7901f,0.5264f,0.3141f,  -0.7891f,0.5293f,0.3118f,  -0.7039f,0.6253f,0.3368f,  -0.7036f,0.6310f,0.3269f,  -0.6072f,0.7185f,0.3391f,  -0.6080f,0.7260f,0.3214f,  
	-0.5022f,0.8029f,0.3211f,  -0.5044f,0.8112f,0.2958f,  -0.3912f,0.8755f,0.2837f,  -0.3950f,0.8837f,0.2512f,  -0.2763f,0.9335f,0.2285f,  -0.2818f,0.9405f,0.1898f,  -0.1596f,0.9744f,0.1582f,  -0.1669f,0.9793f,0.1143f,  -0.0431f,0.9962f,0.0759f,  -0.0519f,0.9983f,0.0281f,  0.0716f,0.9973f,-0.0147f,  0.0615f,0.9960f,-0.0649f,  0.1828f,0.9770f,-0.1095f,  0.1718f,0.9720f,-0.1605f,  0.2894f,0.9352f,-0.2042f,  
	0.2778f,0.9263f,-0.2544f,  0.3899f,0.8725f,-0.2945f,  0.3782f,0.8600f,-0.3426f,  0.4834f,0.7903f,-0.3766f,  0.4720f,0.7745f,-0.4212f,  0.5689f,0.6905f,-0.4467f,  0.5581f,0.6720f,-0.4867f,  0.6454f,0.5760f,-0.5017f,  0.6356f,0.5555f,-0.5362f,  0.7122f,0.4496f,-0.5392f,  0.7035f,0.4279f,-0.5674f,  0.7683f,0.3148f,-0.5573f,  0.7610f,0.2928f,-0.5789f,  0.8132f,0.1754f,-0.5550f,  0.8072f,0.1538f,-0.5699f,  
	0.8460f,0.0348f,-0.5320f,  0.8413f,0.0146f,-0.5404f,  0.8662f,-0.1030f,-0.4889f,  0.8627f,-0.1212f,-0.4910f,  0.8733f,-0.2347f,-0.4269f,  0.8708f,-0.2502f,-0.4232f,  0.8669f,-0.3570f,-0.3478f,  0.7209f,-0.5548f,-0.4153f,  0.7209f,-0.5548f,-0.4153f,  0.6480f,-0.5529f,-0.5238f,  0.6455f,-0.5575f,-0.5221f,  0.5591f,-0.5504f,-0.6201f,  0.5543f,-0.5594f,-0.6163f,  0.4563f,-0.5469f,-0.7020f,  0.4495f,-0.5602f,-0.6957f,  
	0.3417f,-0.5421f,-0.7677f,  0.3334f,-0.5595f,-0.7588f,  0.2180f,-0.5356f,-0.8158f,  0.2084f,-0.5568f,-0.8041f,  0.0878f,-0.5269f,-0.8454f,  0.0772f,-0.5515f,-0.8306f,  -0.0460f,-0.5152f,-0.8559f,  -0.0572f,-0.5427f,-0.8380f,  -0.1806f,-0.4997f,-0.8471f,  -0.1920f,-0.5297f,-0.8262f,  -0.3129f,-0.4798f,-0.8197f,  -0.3243f,-0.5115f,-0.7957f,  -0.4402f,-0.4545f,-0.7744f,  -0.4511f,-0.4874f,-0.7476f,  -0.5597f,-0.4231f,-0.7126f,  
	-0.5698f,-0.4564f,-0.6834f,  -0.6687f,-0.3846f,-0.6363f,  -0.6779f,-0.4176f,-0.6050f,  -0.7651f,-0.3386f,-0.5477f,  -0.7730f,-0.3705f,-0.5150f,  -0.8468f,-0.2843f,-0.4496f,  -0.8532f,-0.3144f,-0.4162f,  -0.9120f,-0.2215f,-0.3451f,  -0.9169f,-0.2491f,-0.3117f,  -0.9597f,-0.1500f,-0.2375f,  -0.9631f,-0.1744f,-0.2051f,  -0.9890f,-0.0701f,-0.1304f,  -0.9908f,-0.0907f,-0.1000f,  -0.9995f,0.0178f,-0.0273f,  -1.0000f,0.0013f,0.0000f,  
	-0.9913f,0.1127f,0.0682f,  -0.9907f,0.1005f,0.0913f,  -0.9650f,0.2132f,0.1527f,  -0.9637f,0.2055f,0.1706f,  -0.9216f,0.3177f,0.2231f,  -0.9199f,0.3141f,0.2348f,  -0.8624f,0.4239f,0.2768f,  -0.8607f,0.4241f,0.2815f,  -0.7891f,0.5293f,0.3118f,  -0.7879f,0.5326f,0.3090f,  -0.7036f,0.6310f,0.3269f,  -0.7033f,0.6367f,0.3161f,  -0.6080f,0.7260f,0.3214f,  -0.6090f,0.7331f,0.3028f,  -0.5044f,0.8112f,0.2958f,  
	-0.5070f,0.8187f,0.2696f,  -0.3950f,0.8837f,0.2512f,  -0.3992f,0.8905f,0.2181f,  -0.2818f,0.9405f,0.1898f,  -0.2878f,0.9458f,0.1507f,  -0.1669f,0.9793f,0.1143f,  -0.1745f,0.9821f,0.0704f,  -0.0519f,0.9983f,0.0281f,  -0.0609f,0.9980f,-0.0192f,  0.0615f,0.9960f,-0.0649f,  0.0514f,0.9921f,-0.1140f,  0.1718f,0.9720f,-0.1605f,  0.1609f,0.9644f,-0.2100f,  0.2778f,0.9263f,-0.2544f,  0.2666f,0.9150f,-0.3028f,  
	0.3782f,0.8600f,-0.3426f,  0.3671f,0.8452f,-0.3885f,  0.4720f,0.7745f,-0.4212f,  0.4613f,0.7567f,-0.4633f,  0.5581f,0.6720f,-0.4867f,  0.5482f,0.6519f,-0.5240f,  0.6356f,0.5555f,-0.5362f,  0.6267f,0.5336f,-0.5679f,  0.7035f,0.4279f,-0.5674f,  0.6958f,0.4052f,-0.5930f,  0.7610f,0.2928f,-0.5789f,  0.7546f,0.2701f,-0.5981f,  0.8072f,0.1538f,-0.5699f,  0.8020f,0.1319f,-0.5826f,  0.8413f,0.0146f,-0.5404f,  
	0.8373f,-0.0056f,-0.5467f,  0.8627f,-0.1212f,-0.4910f,  0.8598f,-0.1391f,-0.4914f,  0.8708f,-0.2502f,-0.4232f,  0.7209f,-0.5548f,-0.4153f,  0.7208f,-0.5551f,-0.4152f,  0.6455f,-0.5575f,-0.5221f,  0.6429f,-0.5624f,-0.5200f,  0.5543f,-0.5594f,-0.6163f,  0.5496f,-0.5687f,-0.6119f,  0.4495f,-0.5602f,-0.6957f,  0.4430f,-0.5739f,-0.6888f,  0.3334f,-0.5595f,-0.7588f,  0.3253f,-0.5773f,-0.7490f,  0.2084f,-0.5568f,-0.8041f,  
	0.1991f,-0.5783f,-0.7911f,  0.0772f,-0.5515f,-0.8306f,  0.0671f,-0.5763f,-0.8144f,  -0.0572f,-0.5427f,-0.8380f,  -0.0679f,-0.5705f,-0.8185f,  -0.1920f,-0.5297f,-0.8262f,  -0.2028f,-0.5598f,-0.8034f,  -0.3243f,-0.5115f,-0.7957f,  -0.3349f,-0.5434f,-0.7698f,  -0.4511f,-0.4874f,-0.7476f,  -0.4612f,-0.5203f,-0.7188f,  -0.5698f,-0.4564f,-0.6834f,  -0.5790f,-0.4895f,-0.6520f,  -0.6779f,-0.4176f,-0.6050f,  -0.6860f,-0.4503f,-0.5716f,  
	-0.7730f,-0.3705f,-0.5150f,  -0.7797f,-0.4019f,-0.4801f,  -0.8532f,-0.3144f,-0.4162f,  -0.8585f,-0.3438f,-0.3806f,  -0.9169f,-0.2491f,-0.3117f,  -0.9207f,-0.2757f,-0.2763f,  -0.9631f,-0.1744f,-0.2051f,  -0.9652f,-0.1977f,-0.1710f,  -0.9908f,-0.0907f,-0.1000f,  -0.9916f,-0.1102f,-0.0681f,  -1.0000f,0.0013f,0.0000f,  -0.9995f,-0.0139f,0.0284f,  -0.9907f,0.1005f,0.0913f,  -0.9893f,0.0897f,0.1152f,  -0.9637f,0.2055f,0.1706f,  
	-0.9616f,0.1990f,0.1888f,  -0.9199f,0.3141f,0.2348f,  -0.9177f,0.3117f,0.2464f,  -0.8607f,0.4241f,0.2815f,  -0.8588f,0.4251f,0.2859f,  -0.7879f,0.5326f,0.3090f,  -0.7867f,0.5364f,0.3055f,  -0.7033f,0.6367f,0.3161f,  -0.7032f,0.6424f,0.3046f,  -0.6090f,0.7331f,0.3028f,  -0.6103f,0.7398f,0.2834f,  -0.5070f,0.8187f,0.2696f,  -0.5098f,0.8253f,0.2428f,  -0.3992f,0.8905f,0.2181f,  -0.4038f,0.8960f,0.1846f,  
	-0.2878f,0.9458f,0.1507f,  -0.2941f,0.9492f,0.1116f,  -0.1745f,0.9821f,0.0704f,  -0.1823f,0.9829f,0.0269f,  -0.0609f,0.9980f,-0.0192f,  -0.0700f,0.9954f,-0.0656f,  0.0514f,0.9921f,-0.1140f,  0.0413f,0.9859f,-0.1620f,  0.1609f,0.9644f,-0.2100f,  0.1504f,0.9544f,-0.2579f,  0.2666f,0.9150f,-0.3028f,  0.2559f,0.9014f,-0.3492f,  0.3671f,0.8452f,-0.3885f,  0.3566f,0.8283f,-0.4321f,  0.4613f,0.7567f,-0.4633f,  
	0.4514f,0.7371f,-0.5029f,  0.5482f,0.6519f,-0.5240f,  0.5391f,0.6302f,-0.5587f,  0.6267f,0.5336f,-0.5679f,  0.6187f,0.5107f,-0.5970f,  0.6958f,0.4052f,-0.5930f,  0.6891f,0.3818f,-0.6160f,  0.7546f,0.2701f,-0.5981f,  0.7490f,0.2470f,-0.6148f,  0.8020f,0.1319f,-0.5826f,  0.7977f,0.1100f,-0.5930f,  0.8373f,-0.0056f,-0.5467f,  0.8341f,-0.0256f,-0.5511f,  0.8598f,-0.1391f,-0.4914f,  0.7208f,-0.5551f,-0.4152f,  
	0.7205f,-0.5557f,-0.4149f,  0.6429f,-0.5624f,-0.5200f,  0.6403f,-0.5675f,-0.5176f,  0.5496f,-0.5687f,-0.6119f,  0.5450f,-0.5784f,-0.6070f,  0.4430f,-0.5739f,-0.6888f,  0.4367f,-0.5878f,-0.6810f,  0.3253f,-0.5773f,-0.7490f,  0.3175f,-0.5953f,-0.7381f,  0.1991f,-0.5783f,-0.7911f,  0.1902f,-0.6001f,-0.7770f,  0.0671f,-0.5763f,-0.8144f,  0.0574f,-0.6015f,-0.7968f,  -0.0679f,-0.5705f,-0.8185f,  -0.0779f,-0.5984f,-0.7974f,  
	-0.2028f,-0.5598f,-0.8034f,  -0.2129f,-0.5900f,-0.7788f,  -0.3349f,-0.5434f,-0.7698f,  -0.3446f,-0.5753f,-0.7418f,  -0.4612f,-0.5203f,-0.7188f,  -0.4703f,-0.5530f,-0.6877f,  -0.5790f,-0.4895f,-0.6520f,  -0.5872f,-0.5224f,-0.6183f,  -0.6860f,-0.4503f,-0.5716f,  -0.6929f,-0.4826f,-0.5358f,  -0.7797f,-0.4019f,-0.4801f,  -0.7852f,-0.4327f,-0.4429f,  -0.8585f,-0.3438f,-0.3806f,  -0.8624f,-0.3724f,-0.3428f,  -0.9207f,-0.2757f,-0.2763f,  
	-0.9231f,-0.3013f,-0.2390f,  -0.9652f,-0.1977f,-0.1710f,  -0.9662f,-0.2197f,-0.1351f,  -0.9916f,-0.1102f,-0.0681f,  -0.9911f,-0.1282f,-0.0349f,  -0.9995f,-0.0139f,0.0284f,  -0.9979f,-0.0276f,0.0578f,  -0.9893f,0.0897f,0.1152f,  -0.9869f,0.0804f,0.1395f,  -0.9616f,0.1990f,0.1888f,  -0.9589f,0.1938f,0.2071f,  -0.9177f,0.3117f,0.2464f,  -0.9150f,0.3103f,0.2579f,  -0.8588f,0.4251f,0.2859f,  -0.8566f,0.4270f,0.2897f,  
	-0.7867f,0.5364f,0.3055f,  -0.7854f,0.5407f,0.3013f,  -0.7032f,0.6424f,0.3046f,  -0.7031f,0.6482f,0.2923f,  -0.6103f,0.7398f,0.2834f,  -0.6117f,0.7460f,0.2632f,  -0.5098f,0.8253f,0.2428f,  -0.5130f,0.8310f,0.2153f,  -0.4038f,0.8960f,0.1846f,  -0.4087f,0.9001f,0.1508f,  -0.2941f,0.9492f,0.1116f,  -0.3006f,0.9510f,0.0725f,  -0.1823f,0.9829f,0.0269f,  -0.1901f,0.9816f,-0.0161f,  -0.0700f,0.9954f,-0.0656f,  
	-0.0790f,0.9907f,-0.1112f,  0.0413f,0.9859f,-0.1620f,  0.0316f,0.9775f,-0.2085f,  0.1504f,0.9544f,-0.2579f,  0.1403f,0.9423f,-0.3040f,  0.2559f,0.9014f,-0.3492f,  0.2458f,0.8859f,-0.3935f,  0.3566f,0.8283f,-0.4321f,  0.3469f,0.8098f,-0.4732f,  0.4514f,0.7371f,-0.5029f,  0.4424f,0.7161f,-0.5399f,  0.5391f,0.6302f,-0.5587f,  0.5311f,0.6074f,-0.5907f,  0.6187f,0.5107f,-0.5970f,  0.6118f,0.4869f,-0.6234f,  
	0.6891f,0.3818f,-0.6160f,  0.6832f,0.3579f,-0.6365f,  0.7490f,0.2470f,-0.6148f,  0.7443f,0.2238f,-0.6292f,  0.7977f,0.1100f,-0.5930f,  0.7941f,0.0882f,-0.6014f,  0.8341f,-0.0256f,-0.5511f,  0.7205f,-0.5557f,-0.4149f,  0.7200f,-0.5566f,-0.4145f,  0.6403f,-0.5675f,-0.5176f,  0.6377f,-0.5730f,-0.5148f,  0.5450f,-0.5784f,-0.6070f,  0.5405f,-0.5883f,-0.6015f,  0.4367f,-0.5878f,-0.6810f,  0.4305f,-0.6021f,-0.6724f,  
	0.3175f,-0.5953f,-0.7381f,  0.3100f,-0.6136f,-0.7262f,  0.1902f,-0.6001f,-0.7770f,  0.1817f,-0.6222f,-0.7615f,  0.0574f,-0.6015f,-0.7968f,  0.0483f,-0.6268f,-0.7776f,  -0.0779f,-0.5984f,-0.7974f,  -0.0873f,-0.6266f,-0.7744f,  -0.2129f,-0.5900f,-0.7788f,  -0.2222f,-0.6204f,-0.7522f,  -0.3446f,-0.5753f,-0.7418f,  -0.3534f,-0.6071f,-0.7117f,  -0.4703f,-0.5530f,-0.6877f,  -0.4783f,-0.5856f,-0.6545f,  -0.5872f,-0.5224f,-0.6183f,  
	-0.5941f,-0.5549f,-0.5823f,  -0.6929f,-0.4826f,-0.5358f,  -0.6986f,-0.5142f,-0.4977f,  -0.7852f,-0.4327f,-0.4429f,  -0.7895f,-0.4626f,-0.4035f,  -0.8624f,-0.3724f,-0.3428f,  -0.8651f,-0.3998f,-0.3030f,  -0.9231f,-0.3013f,-0.2390f,  -0.9242f,-0.3256f,-0.1998f,  -0.9662f,-0.2197f,-0.1351f,  -0.9658f,-0.2402f,-0.0977f,  -0.9911f,-0.1282f,-0.0349f,  -0.9895f,-0.1445f,-0.0006f,  -0.9979f,-0.0276f,0.0578f,  -0.9953f,-0.0396f,0.0879f,  
	-0.9869f,0.0804f,0.1395f,  -0.9837f,0.0727f,0.1643f,  -0.9589f,0.1938f,0.2071f,  -0.9555f,0.1902f,0.2254f,  -0.9150f,0.3103f,0.2579f,  -0.9118f,0.3103f,0.2689f,  -0.8566f,0.4270f,0.2897f,  -0.8541f,0.4298f,0.2928f,  -0.7854f,0.5407f,0.3013f,  -0.7840f,0.5455f,0.2963f,  -0.7031f,0.6482f,0.2923f,  -0.7032f,0.6539f,0.2792f,  -0.6117f,0.7460f,0.2632f,  -0.6133f,0.7517f,0.2424f,  -0.5130f,0.8310f,0.2153f,  
	-0.5163f,0.8356f,0.1875f,  -0.4087f,0.9001f,0.1508f,  -0.4137f,0.9029f,0.1168f,  -0.3006f,0.9510f,0.0725f,  -0.3071f,0.9511f,0.0336f,  -0.1901f,0.9816f,-0.0161f,  -0.1980f,0.9785f,-0.0585f,  -0.0790f,0.9907f,-0.1112f,  -0.0877f,0.9839f,-0.1556f,  0.0316f,0.9775f,-0.2085f,  0.0223f,0.9671f,-0.2535f,  0.1403f,0.9423f,-0.3040f,  0.1308f,0.9283f,-0.3481f,  0.2458f,0.8859f,-0.3935f,  0.2365f,0.8686f,-0.4355f,  
	0.3469f,0.8098f,-0.4732f,  0.3381f,0.7897f,-0.5120f,  0.4424f,0.7161f,-0.5399f,  0.4343f,0.6938f,-0.5744f,  0.5311f,0.6074f,-0.5907f,  0.5240f,0.5838f,-0.6202f,  0.6118f,0.4869f,-0.6234f,  0.6057f,0.4626f,-0.6473f,  0.6832f,0.3579f,-0.6365f,  0.6783f,0.3337f,-0.6546f,  0.7443f,0.2238f,-0.6292f,  0.7405f,0.2006f,-0.6414f,  0.7941f,0.0882f,-0.6014f,  0.7200f,-0.5566f,-0.4145f,  0.7195f,-0.5578f,-0.4138f,  
	0.6377f,-0.5730f,-0.5148f,  0.6351f,-0.5787f,-0.5116f,  0.5405f,-0.5883f,-0.6015f,  0.5361f,-0.5985f,-0.5953f,  0.4305f,-0.6021f,-0.6724f,  0.4246f,-0.6166f,-0.6630f,  0.3100f,-0.6136f,-0.7262f,  0.3029f,-0.6322f,-0.7131f,  0.1817f,-0.6222f,-0.7615f,  0.1738f,-0.6445f,-0.7446f,  0.0483f,-0.6268f,-0.7776f,  0.0399f,-0.6524f,-0.7568f,  -0.0873f,-0.6266f,-0.7744f,  -0.0959f,-0.6548f,-0.7497f,  -0.2222f,-0.6204f,-0.7522f,  
	-0.2305f,-0.6507f,-0.7235f,  -0.3534f,-0.6071f,-0.7117f,  -0.3612f,-0.6387f,-0.6794f,  -0.4783f,-0.5856f,-0.6545f,  -0.4852f,-0.6177f,-0.6188f,  -0.5941f,-0.5549f,-0.5823f,  -0.5999f,-0.5868f,-0.5439f,  -0.6986f,-0.5142f,-0.4977f,  -0.7029f,-0.5449f,-0.4572f,  -0.7895f,-0.4626f,-0.4035f,  -0.7923f,-0.4914f,-0.3617f,  -0.8651f,-0.3998f,-0.3030f,  -0.8663f,-0.4258f,-0.2610f,  -0.9242f,-0.3256f,-0.1998f,  -0.9239f,-0.3482f,-0.1588f,  
	-0.9658f,-0.2402f,-0.0977f,  -0.9641f,-0.2589f,-0.0589f,  -0.9895f,-0.1445f,-0.0006f,  -0.9867f,-0.1590f,0.0349f,  -0.9953f,-0.0396f,0.0879f,  -0.9917f,-0.0497f,0.1187f,  -0.9837f,0.0727f,0.1643f,  -0.9797f,0.0668f,0.1892f,  -0.9555f,0.1902f,0.2254f,  -0.9515f,0.1882f,0.2435f,  -0.9118f,0.3103f,0.2689f,  -0.9083f,0.3114f,0.2794f,  -0.8541f,0.4298f,0.2928f,  -0.8515f,0.4334f,0.2952f,  -0.7840f,0.5455f,0.2963f,  
	-0.7826f,0.5506f,0.2905f,  -0.7032f,0.6539f,0.2792f,  -0.7033f,0.6596f,0.2653f,  -0.6133f,0.7517f,0.2424f,  -0.6151f,0.7568f,0.2209f,  -0.5163f,0.8356f,0.1875f,  -0.5198f,0.8393f,0.1592f,  -0.4137f,0.9029f,0.1168f,  -0.4187f,0.9043f,0.0828f,  -0.3071f,0.9511f,0.0336f,  -0.3136f,0.9496f,-0.0048f,  -0.1980f,0.9785f,-0.0585f,  -0.2055f,0.9735f,-0.1000f,  -0.0877f,0.9839f,-0.1556f,  -0.0960f,0.9753f,-0.1987f,  
	0.0223f,0.9671f,-0.2535f,  0.0136f,0.9548f,-0.2969f,  0.1308f,0.9283f,-0.3481f,  0.1220f,0.9126f,-0.3903f,  0.2365f,0.8686f,-0.4355f,  0.2280f,0.8498f,-0.4753f,  0.3381f,0.7897f,-0.5120f,  0.3302f,0.7683f,-0.5483f,  0.4343f,0.6938f,-0.5744f,  0.4273f,0.6706f,-0.6064f,  0.5240f,0.5838f,-0.6202f,  0.5179f,0.5594f,-0.6471f,  0.6057f,0.4626f,-0.6473f,  0.6007f,0.4380f,-0.6688f,  0.6783f,0.3337f,-0.6546f,  
	0.6743f,0.3095f,-0.6704f,  0.7405f,0.2006f,-0.6414f,  0.7195f,-0.5578f,-0.4138f,  0.7188f,-0.5592f,-0.4130f,  0.6351f,-0.5787f,-0.5116f,  0.6325f,-0.5847f,-0.5079f,  0.5361f,-0.5985f,-0.5953f,  0.5318f,-0.6091f,-0.5884f,  0.4246f,-0.6166f,-0.6630f,  0.4189f,-0.6315f,-0.6525f,  0.3029f,-0.6322f,-0.7131f,  0.2962f,-0.6511f,-0.6988f,  0.1738f,-0.6445f,-0.7446f,  0.1663f,-0.6670f,-0.7262f,  0.0399f,-0.6524f,-0.7568f,  
	0.0320f,-0.6781f,-0.7343f,  -0.0959f,-0.6548f,-0.7497f,  -0.1037f,-0.6831f,-0.7229f,  -0.2305f,-0.6507f,-0.7235f,  -0.2380f,-0.6808f,-0.6927f,  -0.3612f,-0.6387f,-0.6794f,  -0.3680f,-0.6700f,-0.6448f,  -0.4852f,-0.6177f,-0.6188f,  -0.4909f,-0.6494f,-0.5808f,  -0.5999f,-0.5868f,-0.5439f,  -0.6043f,-0.6178f,-0.5031f,  -0.7029f,-0.5449f,-0.4572f,  -0.7059f,-0.5745f,-0.4143f,  -0.7923f,-0.4914f,-0.3617f,  -0.7937f,-0.5187f,-0.3178f,  
	-0.8663f,-0.4258f,-0.2610f,  -0.8661f,-0.4502f,-0.2172f,  -0.9239f,-0.3482f,-0.1588f,  -0.9222f,-0.3689f,-0.1162f,  -0.9641f,-0.2589f,-0.0589f,  -0.9611f,-0.2756f,-0.0188f,  -0.9867f,-0.1590f,0.0349f,  -0.9826f,-0.1713f,0.0710f,  -0.9917f,-0.0497f,0.1187f,  -0.9870f,-0.0578f,0.1497f,  -0.9797f,0.0668f,0.1892f,  -0.9748f,0.0628f,0.2140f,  -0.9515f,0.1882f,0.2435f,  -0.9469f,0.1878f,0.2612f,  -0.9083f,0.3114f,0.2794f,  
	-0.9044f,0.3139f,0.2891f,  -0.8515f,0.4334f,0.2952f,  -0.8486f,0.4379f,0.2968f,  -0.7826f,0.5506f,0.2905f,  -0.7811f,0.5561f,0.2838f,  -0.7033f,0.6596f,0.2653f,  -0.7035f,0.6651f,0.2506f,  -0.6151f,0.7568f,0.2209f,  -0.6170f,0.7614f,0.1988f,  -0.5198f,0.8393f,0.1592f,  -0.5233f,0.8421f,0.1306f,  -0.4187f,0.9043f,0.0828f,  -0.4238f,0.9044f,0.0490f,  -0.3136f,0.9496f,-0.0048f,  -0.3198f,0.9465f,-0.0427f,  
	-0.2055f,0.9735f,-0.1000f,  -0.2128f,0.9669f,-0.1406f,  -0.0960f,0.9753f,-0.1987f,  -0.1039f,0.9651f,-0.2405f,  0.0136f,0.9548f,-0.2969f,  0.0054f,0.9410f,-0.3385f,  0.1220f,0.9126f,-0.3903f,  0.1140f,0.8954f,-0.4304f,  0.2280f,0.8498f,-0.4753f,  0.2204f,0.8297f,-0.5128f,  0.3302f,0.7683f,-0.5483f,  0.3233f,0.7460f,-0.5822f,  0.4273f,0.6706f,-0.6064f,  0.4213f,0.6467f,-0.6359f,  0.5179f,0.5594f,-0.6471f,  
	0.5128f,0.5347f,-0.6716f,  0.6007f,0.4380f,-0.6688f,  0.5966f,0.4132f,-0.6880f,  0.6743f,0.3095f,-0.6704f,  0.7188f,-0.5592f,-0.4130f,  0.7181f,-0.5610f,-0.4119f,  0.6325f,-0.5847f,-0.5079f,  0.6300f,-0.5911f,-0.5037f,  0.5318f,-0.6091f,-0.5884f,  0.5277f,-0.6199f,-0.5807f,  0.4189f,-0.6315f,-0.6525f,  0.4135f,-0.6466f,-0.6410f,  0.2962f,-0.6511f,-0.6988f,  0.2898f,-0.6702f,-0.6832f,  0.1663f,-0.6670f,-0.7262f,  
	0.1594f,-0.6897f,-0.7063f,  0.0320f,-0.6781f,-0.7343f,  0.0249f,-0.7038f,-0.7099f,  -0.1037f,-0.6831f,-0.7229f,  -0.1107f,-0.7113f,-0.6941f,  -0.2380f,-0.6808f,-0.6927f,  -0.2444f,-0.7107f,-0.6596f,  -0.3680f,-0.6700f,-0.6448f,  -0.3736f,-0.7008f,-0.6078f,  -0.4909f,-0.6494f,-0.5808f,  -0.4954f,-0.6802f,-0.5403f,  -0.6043f,-0.6178f,-0.5031f,  -0.6074f,-0.6478f,-0.4598f,  -0.7059f,-0.5745f,-0.4143f,  -0.7074f,-0.6028f,-0.3692f,  
	-0.7937f,-0.5187f,-0.3178f,  -0.7935f,-0.5445f,-0.2718f,  -0.8661f,-0.4502f,-0.2172f,  -0.8644f,-0.4726f,-0.1715f,  -0.9222f,-0.3689f,-0.1162f,  -0.9190f,-0.3875f,-0.0722f,  -0.9611f,-0.2756f,-0.0188f,  -0.9568f,-0.2900f,0.0222f,  -0.9826f,-0.1713f,0.0710f,  -0.9775f,-0.1814f,0.1077f,  -0.9870f,-0.0578f,0.1497f,  -0.9814f,-0.0636f,0.1809f,  -0.9748f,0.0628f,0.2140f,  -0.9692f,0.0608f,0.2385f,  -0.9469f,0.1878f,0.2612f,  
	-0.9418f,0.1890f,0.2781f,  -0.9044f,0.3139f,0.2891f,  -0.9002f,0.3176f,0.2980f,  -0.8486f,0.4379f,0.2968f,  -0.8457f,0.4432f,0.2973f,  -0.7811f,0.5561f,0.2838f,  -0.7797f,0.5620f,0.2761f,  -0.7035f,0.6651f,0.2506f,  -0.7037f,0.6705f,0.2351f,  -0.6170f,0.7614f,0.1988f,  -0.6189f,0.7654f,0.1762f,  -0.5233f,0.8421f,0.1306f,  -0.5268f,0.8438f,0.1019f,  -0.4238f,0.9044f,0.0490f,  -0.4287f,0.9033f,0.0153f,  
	-0.3198f,0.9465f,-0.0427f,  -0.3259f,0.9420f,-0.0800f,  -0.2128f,0.9669f,-0.1406f,  -0.2196f,0.9588f,-0.1801f,  -0.1039f,0.9651f,-0.2405f,  -0.1112f,0.9533f,-0.2808f,  0.0054f,0.9410f,-0.3385f,  -0.0019f,0.9257f,-0.3783f,  0.1140f,0.8954f,-0.4304f,  0.1069f,0.8770f,-0.4685f,  0.2204f,0.8297f,-0.5128f,  0.2138f,0.8086f,-0.5481f,  0.3233f,0.7460f,-0.5822f,  0.3174f,0.7229f,-0.6137f,  0.4213f,0.6467f,-0.6359f,  
	0.4162f,0.6223f,-0.6630f,  0.5128f,0.5347f,-0.6716f,  0.5087f,0.5098f,-0.6938f,  0.5966f,0.4132f,-0.6880f,  0.7181f,-0.5610f,-0.4119f,  0.7173f,-0.5630f,-0.4105f,  0.6300f,-0.5911f,-0.5037f,  0.6275f,-0.5977f,-0.4990f,  0.5277f,-0.6199f,-0.5807f,  0.5238f,-0.6310f,-0.5723f,  0.4135f,-0.6466f,-0.6410f,  0.4084f,-0.6620f,-0.6285f,  0.2898f,-0.6702f,-0.6832f,  0.2840f,-0.6896f,-0.6662f,  0.1594f,-0.6897f,-0.7063f,  
	0.1531f,-0.7125f,-0.6848f,  0.0249f,-0.7038f,-0.7099f,  0.0185f,-0.7296f,-0.6837f,  -0.1107f,-0.7113f,-0.6941f,  -0.1167f,-0.7393f,-0.6632f,  -0.2444f,-0.7107f,-0.6596f,  -0.2498f,-0.7402f,-0.6243f,  -0.3736f,-0.7008f,-0.6078f,  -0.3779f,-0.7309f,-0.5683f,  -0.4954f,-0.6802f,-0.5403f,  -0.4984f,-0.7100f,-0.4974f,  -0.6074f,-0.6478f,-0.4598f,  -0.6089f,-0.6765f,-0.4142f,  -0.7074f,-0.6028f,-0.3692f,  -0.7073f,-0.6294f,-0.3219f,  
	-0.7935f,-0.5445f,-0.2718f,  -0.7918f,-0.5682f,-0.2239f,  -0.8644f,-0.4726f,-0.1715f,  -0.8612f,-0.4928f,-0.1243f,  -0.9190f,-0.3875f,-0.0722f,  -0.9145f,-0.4038f,-0.0270f,  -0.9568f,-0.2900f,0.0222f,  -0.9512f,-0.3019f,0.0639f,  -0.9775f,-0.1814f,0.1077f,  -0.9713f,-0.1890f,0.1447f,  -0.9814f,-0.0636f,0.1809f,  -0.9750f,-0.0672f,0.2118f,  -0.9692f,0.0608f,0.2385f,  -0.9631f,0.0608f,0.2624f,  -0.9418f,0.1890f,0.2781f,  
	-0.9363f,0.1919f,0.2942f,  -0.9002f,0.3176f,0.2980f,  -0.8958f,0.3226f,0.3059f,  -0.8457f,0.4432f,0.2973f,  -0.8427f,0.4492f,0.2968f,  -0.7797f,0.5620f,0.2761f,  -0.7783f,0.5681f,0.2674f,  -0.7037f,0.6705f,0.2351f,  -0.7040f,0.6757f,0.2188f,  -0.6189f,0.7654f,0.1762f,  -0.6209f,0.7688f,0.1531f,  -0.5268f,0.8438f,0.1019f,  -0.5303f,0.8447f,0.0731f,  -0.4287f,0.9033f,0.0153f,  -0.4334f,0.9010f,-0.0181f,  
	-0.3259f,0.9420f,-0.0800f,  -0.3316f,0.9362f,-0.1165f,  -0.2196f,0.9588f,-0.1801f,  -0.2260f,0.9493f,-0.2184f,  -0.1112f,0.9533f,-0.2808f,  -0.1178f,0.9402f,-0.3197f,  -0.0019f,0.9257f,-0.3783f,  -0.0085f,0.9092f,-0.4163f,  0.1069f,0.8770f,-0.4685f,  0.1007f,0.8575f,-0.5046f,  0.2138f,0.8086f,-0.5481f,  0.2081f,0.7868f,-0.5811f,  0.3174f,0.7229f,-0.6137f,  0.3125f,0.6992f,-0.6430f,  0.4162f,0.6223f,-0.6630f,  
	0.4121f,0.5976f,-0.6878f,  0.5087f,0.5098f,-0.6938f,  0.7173f,-0.5630f,-0.4105f,  0.7165f,-0.5653f,-0.4088f,  0.6275f,-0.5977f,-0.4990f,  0.6250f,-0.6046f,-0.4937f,  0.5238f,-0.6310f,-0.5723f,  0.5200f,-0.6424f,-0.5630f,  0.4084f,-0.6620f,-0.6285f,  0.4037f,-0.6776f,-0.6148f,  0.2840f,-0.6896f,-0.6662f,  0.2786f,-0.7090f,-0.6479f,  0.1531f,-0.7125f,-0.6848f,  0.1474f,-0.7353f,-0.6615f,  0.0185f,-0.7296f,-0.6837f,  
	0.0130f,-0.7551f,-0.6554f,  -0.1167f,-0.7393f,-0.6632f,  -0.1218f,-0.7669f,-0.6301f,  -0.2498f,-0.7402f,-0.6243f,  -0.2540f,-0.7690f,-0.5866f,  -0.3779f,-0.7309f,-0.5683f,  -0.3810f,-0.7600f,-0.5265f,  -0.4984f,-0.7100f,-0.4974f,  -0.5000f,-0.7386f,-0.4522f,  -0.6089f,-0.6765f,-0.4142f,  -0.6090f,-0.7035f,-0.3664f,  -0.7073f,-0.6294f,-0.3219f,  -0.7057f,-0.6540f,-0.2725f,  -0.7918f,-0.5682f,-0.2239f,  -0.7886f,-0.5897f,-0.1743f,  
	-0.8612f,-0.4928f,-0.1243f,  -0.8565f,-0.5106f,-0.0758f,  -0.9145f,-0.4038f,-0.0270f,  -0.9086f,-0.4173f,0.0190f,  -0.9512f,-0.3019f,0.0639f,  -0.9444f,-0.3112f,0.1060f,  -0.9713f,-0.1890f,0.1447f,  -0.9641f,-0.1941f,0.1815f,  -0.9750f,-0.0672f,0.2118f,  -0.9678f,-0.0684f,0.2421f,  -0.9631f,0.0608f,0.2624f,  -0.9564f,0.0629f,0.2854f,  -0.9363f,0.1919f,0.2942f,  -0.9305f,0.1965f,0.3092f,  -0.8958f,0.3226f,0.3059f,  
	-0.8912f,0.3287f,0.3125f,  -0.8427f,0.4492f,0.2968f,  -0.8397f,0.4559f,0.2951f,  -0.7783f,0.5681f,0.2674f,  -0.7770f,0.5744f,0.2577f,  -0.7040f,0.6757f,0.2188f,  -0.7043f,0.6806f,0.2017f,  -0.6209f,0.7688f,0.1531f,  -0.6228f,0.7716f,0.1296f,  -0.5303f,0.8447f,0.0731f,  -0.5336f,0.8446f,0.0442f,  -0.4334f,0.9010f,-0.0181f,  -0.4379f,0.8976f,-0.0510f,  -0.3316f,0.9362f,-0.1165f,  -0.3369f,0.9292f,-0.1523f,  
	-0.2260f,0.9493f,-0.2184f,  -0.2317f,0.9386f,-0.2556f,  -0.1178f,0.9402f,-0.3197f,  -0.1237f,0.9259f,-0.3569f,  -0.0085f,0.9092f,-0.4163f,  -0.0142f,0.8917f,-0.4525f,  0.1007f,0.8575f,-0.5046f,  0.0953f,0.8372f,-0.5386f,  0.2081f,0.7868f,-0.5811f,  0.2034f,0.7643f,-0.6120f,  0.3125f,0.6992f,-0.6430f,  0.3084f,0.6752f,-0.6700f,  0.4121f,0.5976f,-0.6878f,  0.7165f,-0.5653f,-0.4088f,  0.7156f,-0.5679f,-0.4068f,  
	0.6250f,-0.6046f,-0.4937f,  0.6227f,-0.6118f,-0.4878f,  0.5200f,-0.6424f,-0.5630f,  0.5165f,-0.6540f,-0.5528f,  0.4037f,-0.6776f,-0.6148f,  0.3993f,-0.6933f,-0.5999f,  0.2786f,-0.7090f,-0.6479f,  0.2738f,-0.7285f,-0.6280f,  0.1474f,-0.7353f,-0.6615f,  0.1425f,-0.7581f,-0.6364f,  0.0130f,-0.7551f,-0.6554f,  0.0084f,-0.7804f,-0.6252f,  -0.1218f,-0.7669f,-0.6301f,  -0.1257f,-0.7939f,-0.5949f,  -0.2540f,-0.7690f,-0.5866f,  
	-0.2570f,-0.7970f,-0.5466f,  -0.3810f,-0.7600f,-0.5265f,  -0.3826f,-0.7880f,-0.4824f,  -0.5000f,-0.7386f,-0.4522f,  -0.5002f,-0.7656f,-0.4046f,  -0.6090f,-0.7035f,-0.3664f,  -0.6074f,-0.7286f,-0.3164f,  -0.7057f,-0.6540f,-0.2725f,  -0.7025f,-0.6764f,-0.2213f,  -0.7886f,-0.5897f,-0.1743f,  -0.7838f,-0.6086f,-0.1232f,  -0.8565f,-0.5106f,-0.0758f,  -0.8504f,-0.5256f,-0.0262f,  -0.9086f,-0.4173f,0.0190f,  -0.9014f,-0.4281f,0.0655f,  
	-0.9444f,-0.3112f,0.1060f,  -0.9366f,-0.3177f,0.1481f,  -0.9641f,-0.1941f,0.1815f,  -0.9560f,-0.1965f,0.2178f,  -0.9678f,-0.0684f,0.2421f,  -0.9600f,-0.0672f,0.2717f,  -0.9564f,0.0629f,0.2854f,  -0.9493f,0.0670f,0.3072f,  -0.9305f,0.1965f,0.3092f,  -0.9245f,0.2026f,0.3228f,  -0.8912f,0.3287f,0.3125f,  -0.8867f,0.3360f,0.3178f,  -0.8397f,0.4559f,0.2951f,  -0.8367f,0.4633f,0.2921f,  -0.7770f,0.5744f,0.2577f,  
	-0.7757f,0.5808f,0.2469f,  -0.7043f,0.6806f,0.2017f,  -0.7046f,0.6853f,0.1839f,  -0.6228f,0.7716f,0.1296f,  -0.6246f,0.7737f,0.1057f,  -0.5336f,0.8446f,0.0442f,  -0.5367f,0.8436f,0.0154f,  -0.4379f,0.8976f,-0.0510f,  -0.4420f,0.8931f,-0.0835f,  -0.3369f,0.9292f,-0.1523f,  -0.3417f,0.9210f,-0.1871f,  -0.2317f,0.9386f,-0.2556f,  -0.2369f,0.9268f,-0.2915f,  -0.1237f,0.9259f,-0.3569f,  -0.1289f,0.9106f,-0.3927f,  
	-0.0142f,0.8917f,-0.4525f,  -0.0192f,0.8733f,-0.4869f,  0.0953f,0.8372f,-0.5386f,  0.0908f,0.8162f,-0.5706f,  0.2034f,0.7643f,-0.6120f,  0.1995f,0.7413f,-0.6408f,  0.3084f,0.6752f,-0.6700f,  0.7156f,-0.5679f,-0.4068f,  0.7147f,-0.5707f,-0.4044f,  0.6227f,-0.6118f,-0.4878f,  0.6204f,-0.6192f,-0.4813f,  0.5165f,-0.6540f,-0.5528f,  0.5132f,-0.6658f,-0.5416f,  0.3993f,-0.6933f,-0.5999f,  0.3954f,-0.7092f,-0.5837f,  
	0.2738f,-0.7285f,-0.6280f,  0.2695f,-0.7480f,-0.6065f,  0.1425f,-0.7581f,-0.6364f,  0.1383f,-0.7806f,-0.6095f,  0.0084f,-0.7804f,-0.6252f,  0.0047f,-0.8053f,-0.5929f,  -0.1257f,-0.7939f,-0.5949f,  -0.1285f,-0.8203f,-0.5573f,  -0.2570f,-0.7970f,-0.5466f,  -0.2586f,-0.8239f,-0.5043f,  -0.3826f,-0.7880f,-0.4824f,  -0.3828f,-0.8145f,-0.4359f,  -0.5002f,-0.7656f,-0.4046f,  -0.4987f,-0.7907f,-0.3549f,  -0.6074f,-0.7286f,-0.3164f,  
	-0.6043f,-0.7515f,-0.2646f,  -0.7025f,-0.6764f,-0.2213f,  -0.6977f,-0.6963f,-0.1686f,  -0.7838f,-0.6086f,-0.1232f,  -0.7776f,-0.6248f,-0.0710f,  -0.8504f,-0.5256f,-0.0262f,  -0.8429f,-0.5376f,0.0239f,  -0.9014f,-0.4281f,0.0655f,  -0.8930f,-0.4358f,0.1122f,  -0.9366f,-0.3177f,0.1481f,  -0.9278f,-0.3212f,0.1898f,  -0.9560f,-0.1965f,0.2178f,  -0.9473f,-0.1962f,0.2534f,  -0.9600f,-0.0672f,0.2717f,  -0.9518f,-0.0637f,0.3001f,  
	-0.9493f,0.0670f,0.3072f,  -0.9420f,0.0730f,0.3277f,  -0.9245f,0.2026f,0.3228f,  -0.9185f,0.2102f,0.3350f,  -0.8867f,0.3360f,0.3178f,  -0.8821f,0.3442f,0.3216f,  -0.8367f,0.4633f,0.2921f,  -0.8338f,0.4710f,0.2879f,  -0.7757f,0.5808f,0.2469f,  -0.7745f,0.5873f,0.2351f,  -0.7046f,0.6853f,0.1839f,  -0.7050f,0.6896f,0.1654f,  -0.6246f,0.7737f,0.1057f,  -0.6264f,0.7752f,0.0815f,  -0.5367f,0.8436f,0.0154f,  
	-0.5397f,0.8418f,-0.0133f,  -0.4420f,0.8931f,-0.0835f,  -0.4458f,0.8877f,-0.1155f,  -0.3417f,0.9210f,-0.1871f,  -0.3460f,0.9118f,-0.2211f,  -0.2369f,0.9268f,-0.2915f,  -0.2413f,0.9140f,-0.3261f,  -0.1289f,0.9106f,-0.3927f,  -0.1333f,0.8945f,-0.4268f,  -0.0192f,0.8733f,-0.4869f,  -0.0233f,0.8542f,-0.5194f,  0.0908f,0.8162f,-0.5706f,  0.0872f,0.7948f,-0.6006f,  0.1995f,0.7413f,-0.6408f,  0.7147f,-0.5707f,-0.4044f,  
	0.7137f,-0.5739f,-0.4016f,  0.6204f,-0.6192f,-0.4813f,  0.6183f,-0.6269f,-0.4740f,  0.5132f,-0.6658f,-0.5416f,  0.5102f,-0.6778f,-0.5294f,  0.3954f,-0.7092f,-0.5837f,  0.3919f,-0.7252f,-0.5662f,  0.2695f,-0.7480f,-0.6065f,  0.2659f,-0.7674f,-0.5834f,  0.1383f,-0.7806f,-0.6095f,  0.1350f,-0.8028f,-0.5808f,  0.0047f,-0.8053f,-0.5929f,  0.0020f,-0.8295f,-0.5585f,  -0.1285f,-0.8203f,-0.5573f,  -0.1302f,-0.8456f,-0.5176f,  
	-0.2586f,-0.8239f,-0.5043f,  -0.2589f,-0.8494f,-0.4598f,  -0.3828f,-0.8145f,-0.4359f,  -0.3815f,-0.8393f,-0.3874f,  -0.4987f,-0.7907f,-0.3549f,  -0.4958f,-0.8138f,-0.3033f,  -0.6043f,-0.7515f,-0.2646f,  -0.5996f,-0.7720f,-0.2111f,  -0.6977f,-0.6963f,-0.1686f,  -0.6914f,-0.7133f,-0.1146f,  -0.7776f,-0.6248f,-0.0710f,  -0.7699f,-0.6379f,-0.0181f,  -0.8429f,-0.5376f,0.0239f,  -0.8341f,-0.5465f,0.0743f,  -0.8930f,-0.4358f,0.1122f,  
	-0.8837f,-0.4405f,0.1585f,  -0.9278f,-0.3212f,0.1898f,  -0.9182f,-0.3219f,0.2307f,  -0.9473f,-0.1962f,0.2534f,  -0.9380f,-0.1932f,0.2878f,  -0.9518f,-0.0637f,0.3001f,  -0.9433f,-0.0579f,0.3270f,  -0.9420f,0.0730f,0.3277f,  -0.9345f,0.0809f,0.3466f,  -0.9185f,0.2102f,0.3350f,  -0.9124f,0.2192f,0.3456f,  -0.8821f,0.3442f,0.3216f,  -0.8777f,0.3532f,0.3239f,  -0.8338f,0.4710f,0.2879f,  -0.8310f,0.4792f,0.2823f,  
	-0.7745f,0.5873f,0.2351f,  -0.7733f,0.5938f,0.2223f,  -0.7050f,0.6896f,0.1654f,  -0.7054f,0.6936f,0.1463f,  -0.6264f,0.7752f,0.0815f,  -0.6281f,0.7761f,0.0571f,  -0.5397f,0.8418f,-0.0133f,  -0.5423f,0.8391f,-0.0418f,  -0.4458f,0.8877f,-0.1155f,  -0.4492f,0.8813f,-0.1469f,  -0.3460f,0.9118f,-0.2211f,  -0.3497f,0.9018f,-0.2540f,  -0.2413f,0.9140f,-0.3261f,  -0.2452f,0.9004f,-0.3595f,  -0.1333f,0.8945f,-0.4268f,  
	-0.1370f,0.8776f,-0.4594f,  -0.0233f,0.8542f,-0.5194f,  -0.0266f,0.8346f,-0.5502f,  0.0872f,0.7948f,-0.6006f,  0.7137f,-0.5739f,-0.4016f,  0.7128f,-0.5772f,-0.3984f,  0.6183f,-0.6269f,-0.4740f,  0.6163f,-0.6348f,-0.4660f,  0.5102f,-0.6778f,-0.5294f,  0.5076f,-0.6899f,-0.5161f,  0.3919f,-0.7252f,-0.5662f,  0.3889f,-0.7411f,-0.5473f,  0.2659f,-0.7674f,-0.5834f,  0.2630f,-0.7866f,-0.5587f,  0.1350f,-0.8028f,-0.5808f,  
	0.1326f,-0.8245f,-0.5501f,  0.0020f,-0.8295f,-0.5585f,  0.0005f,-0.8529f,-0.5221f,  -0.1302f,-0.8456f,-0.5176f,  -0.1305f,-0.8698f,-0.4758f,  -0.2589f,-0.8494f,-0.4598f,  -0.2578f,-0.8734f,-0.4132f,  -0.3815f,-0.8393f,-0.3874f,  -0.3787f,-0.8620f,-0.3369f,  -0.4958f,-0.8138f,-0.3033f,  -0.4913f,-0.8344f,-0.2500f,  -0.5996f,-0.7720f,-0.2111f,  -0.5934f,-0.7896f,-0.1563f,  -0.6914f,-0.7133f,-0.1146f,  -0.6837f,-0.7273f,-0.0598f,  
	-0.7699f,-0.6379f,-0.0181f,  -0.7610f,-0.6478f,0.0352f,  -0.8341f,-0.5465f,0.0743f,  -0.8244f,-0.5522f,0.1245f,  -0.8837f,-0.4405f,0.1585f,  -0.8735f,-0.4420f,0.2042f,  -0.9182f,-0.3219f,0.2307f,  -0.9081f,-0.3196f,0.2706f,  -0.9380f,-0.1932f,0.2878f,  -0.9284f,-0.1878f,0.3207f,  -0.9433f,-0.0579f,0.3270f,  -0.9346f,-0.0499f,0.3523f,  -0.9345f,0.0809f,0.3466f,  -0.9271f,0.0904f,0.3638f,  -0.9124f,0.2192f,0.3456f,  
	-0.9065f,0.2293f,0.3545f,  -0.8777f,0.3532f,0.3239f,  -0.8734f,0.3630f,0.3247f,  -0.8310f,0.4792f,0.2823f,  -0.8284f,0.4877f,0.2755f,  -0.7733f,0.5938f,0.2223f,  -0.7723f,0.6001f,0.2086f,  -0.7054f,0.6936f,0.1463f,  -0.7057f,0.6971f,0.1266f,  -0.6281f,0.7761f,0.0571f,  -0.6296f,0.7763f,0.0325f,  -0.5423f,0.8391f,-0.0418f,  -0.5447f,0.8357f,-0.0701f,  -0.4492f,0.8813f,-0.1469f,  -0.4521f,0.8741f,-0.1776f,  
	-0.3497f,0.9018f,-0.2540f,  -0.3529f,0.8909f,-0.2860f,  -0.2452f,0.9004f,-0.3595f,  -0.2484f,0.8860f,-0.3915f,  -0.1370f,0.8776f,-0.4594f,  -0.1400f,0.8602f,-0.4904f,  -0.0266f,0.8346f,-0.5502f,  0.7128f,-0.5772f,-0.3984f,  0.7119f,-0.5809f,-0.3947f,  0.6163f,-0.6348f,-0.4660f,  0.6145f,-0.6428f,-0.4573f,  0.5076f,-0.6899f,-0.5161f,  0.5053f,-0.7021f,-0.5018f,  0.3889f,-0.7411f,-0.5473f,  0.3865f,-0.7569f,-0.5270f,  
	0.2630f,-0.7866f,-0.5587f,  0.2609f,-0.8054f,-0.5323f,  0.1326f,-0.8245f,-0.5501f,  0.1311f,-0.8455f,-0.5176f,  0.0005f,-0.8529f,-0.5221f,  0.0001f,-0.8753f,-0.4837f,  -0.1305f,-0.8698f,-0.4758f,  -0.1296f,-0.8925f,-0.4320f,  -0.2578f,-0.8734f,-0.4132f,  -0.2552f,-0.8955f,-0.3647f,  -0.3787f,-0.8620f,-0.3369f,  -0.3745f,-0.8825f,-0.2847f,  -0.4913f,-0.8344f,-0.2500f,  -0.4853f,-0.8523f,-0.1953f,  -0.5934f,-0.7896f,-0.1563f,  
	-0.5858f,-0.8042f,-0.1005f,  -0.6837f,-0.7273f,-0.0598f,  -0.6747f,-0.7381f,-0.0045f,  -0.7610f,-0.6478f,0.0352f,  -0.7509f,-0.6545f,0.0884f,  -0.8244f,-0.5522f,0.1245f,  -0.8137f,-0.5546f,0.1742f,  -0.8735f,-0.4420f,0.2042f,  -0.8626f,-0.4404f,0.2488f,  -0.9081f,-0.3196f,0.2706f,  -0.8976f,-0.3145f,0.3090f,  -0.9284f,-0.1878f,0.3207f,  -0.9186f,-0.1798f,0.3520f,  -0.9346f,-0.0499f,0.3523f,  -0.9259f,-0.0399f,0.3757f,  
	-0.9271f,0.0904f,0.3638f,  -0.9198f,0.1015f,0.3790f,  -0.9065f,0.2293f,0.3545f,  -0.9008f,0.2405f,0.3616f,  -0.8734f,0.3630f,0.3247f,  -0.8693f,0.3733f,0.3239f,  -0.8284f,0.4877f,0.2755f,  -0.8260f,0.4963f,0.2673f,  -0.7723f,0.6001f,0.2086f,  -0.7713f,0.6062f,0.1939f,  -0.7057f,0.6971f,0.1266f,  -0.7060f,0.7001f,0.1063f,  -0.6296f,0.7763f,0.0325f,  -0.6309f,0.7758f,0.0078f,  -0.5447f,0.8357f,-0.0701f,  
	-0.5467f,0.8316f,-0.0981f,  -0.4521f,0.8741f,-0.1776f,  -0.4546f,0.8662f,-0.2076f,  -0.3529f,0.8909f,-0.2860f,  -0.3555f,0.8793f,-0.3169f,  -0.2484f,0.8860f,-0.3915f,  -0.2510f,0.8711f,-0.4222f,  -0.1400f,0.8602f,-0.4904f,  0.7119f,-0.5809f,-0.3947f,  0.7110f,-0.5848f,-0.3906f,  0.6145f,-0.6428f,-0.4573f,  0.6129f,-0.6511f,-0.4477f,  0.5053f,-0.7021f,-0.5018f,  0.5033f,-0.7143f,-0.4863f,  0.3865f,-0.7569f,-0.5270f,  
	0.3846f,-0.7725f,-0.5053f,  0.2609f,-0.8054f,-0.5323f,  0.2595f,-0.8237f,-0.5042f,  0.1311f,-0.8455f,-0.5176f,  0.1306f,-0.8657f,-0.4832f,  0.0001f,-0.8753f,-0.4837f,  0.0008f,-0.8964f,-0.4433f,  -0.1296f,-0.8925f,-0.4320f,  -0.1273f,-0.9136f,-0.3863f,  -0.2552f,-0.8955f,-0.3647f,  -0.2513f,-0.9154f,-0.3145f,  -0.3745f,-0.8825f,-0.2847f,  -0.3688f,-0.9003f,-0.2310f,  -0.4853f,-0.8523f,-0.1953f,  -0.4779f,-0.8673f,-0.1396f,  
	-0.5858f,-0.8042f,-0.1005f,  -0.5769f,-0.8156f,-0.0442f,  -0.6747f,-0.7381f,-0.0045f,  -0.6646f,-0.7455f,0.0507f,  -0.7509f,-0.6545f,0.0884f,  -0.7400f,-0.6577f,0.1411f,  -0.8137f,-0.5546f,0.1742f,  -0.8023f,-0.5537f,0.2228f,  -0.8626f,-0.4404f,0.2488f,  -0.8514f,-0.4358f,0.2920f,  -0.8976f,-0.3145f,0.3090f,  -0.8869f,-0.3066f,0.3456f,  -0.9186f,-0.1798f,0.3520f,  -0.9088f,-0.1695f,0.3813f,  -0.9259f,-0.0399f,0.3757f,  
	-0.9174f,-0.0281f,0.3971f,  -0.9198f,0.1015f,0.3790f,  -0.9128f,0.1139f,0.3923f,  -0.9008f,0.2405f,0.3616f,  -0.8954f,0.2525f,0.3668f,  -0.8693f,0.3733f,0.3239f,  -0.8655f,0.3840f,0.3215f,  -0.8260f,0.4963f,0.2673f,  -0.8238f,0.5049f,0.2579f,  -0.7713f,0.6062f,0.1939f,  -0.7705f,0.6120f,0.1783f,  -0.7060f,0.7001f,0.1063f,  -0.7063f,0.7027f,0.0856f,  -0.6309f,0.7758f,0.0078f,  -0.6320f,0.7748f,-0.0169f,  
	-0.5467f,0.8316f,-0.0981f,  -0.5485f,0.8267f,-0.1257f,  -0.4546f,0.8662f,-0.2076f,  -0.4566f,0.8575f,-0.2369f,  -0.3555f,0.8793f,-0.3169f,  -0.3577f,0.8671f,-0.3467f,  -0.2510f,0.8711f,-0.4222f,  0.7110f,-0.5848f,-0.3906f,  0.7101f,-0.5889f,-0.3859f,  0.6129f,-0.6511f,-0.4477f,  0.6115f,-0.6594f,-0.4373f,  0.5033f,-0.7143f,-0.4863f,  0.5018f,-0.7264f,-0.4696f,  0.3846f,-0.7725f,-0.5053f,  0.3834f,-0.7878f,-0.4821f,  
	0.2595f,-0.8237f,-0.5042f,  0.2590f,-0.8413f,-0.4744f,  0.1306f,-0.8657f,-0.4832f,  0.1312f,-0.8848f,-0.4471f,  0.0008f,-0.8964f,-0.4433f,  0.0027f,-0.9160f,-0.4012f,  -0.1273f,-0.9136f,-0.3863f,  -0.1238f,-0.9326f,-0.3389f,  -0.2513f,-0.9154f,-0.3145f,  -0.2460f,-0.9330f,-0.2628f,  -0.3688f,-0.9003f,-0.2310f,  -0.3618f,-0.9155f,-0.1763f,  -0.4779f,-0.8673f,-0.1396f,  -0.4692f,-0.8792f,-0.0832f,  -0.5769f,-0.8156f,-0.0442f,  
	-0.5669f,-0.8237f,0.0122f,  -0.6646f,-0.7455f,0.0507f,  -0.6535f,-0.7495f,0.1057f,  -0.7400f,-0.6577f,0.1411f,  -0.7283f,-0.6575f,0.1929f,  -0.8023f,-0.5537f,0.2228f,  -0.7905f,-0.5497f,0.2700f,  -0.8514f,-0.4358f,0.2920f,  -0.8399f,-0.4283f,0.3334f,  -0.8869f,-0.3066f,0.3456f,  -0.8761f,-0.2963f,0.3803f,  -0.9088f,-0.1695f,0.3813f,  -0.8992f,-0.1572f,0.4084f,  -0.9174f,-0.0281f,0.3971f,  -0.9091f,-0.0147f,0.4163f,  
	-0.9128f,0.1139f,0.3923f,  -0.9061f,0.1274f,0.4035f,  -0.8954f,0.2525f,0.3668f,  -0.8903f,0.2652f,0.3702f,  -0.8655f,0.3840f,0.3215f,  -0.8621f,0.3950f,0.3175f,  -0.8238f,0.5049f,0.2579f,  -0.8217f,0.5134f,0.2473f,  -0.7705f,0.6120f,0.1783f,  -0.7697f,0.6175f,0.1620f,  -0.7063f,0.7027f,0.0856f,  -0.7065f,0.7047f,0.0645f,  -0.6320f,0.7748f,-0.0169f,  -0.6330f,0.7730f,-0.0416f,  -0.5485f,0.8267f,-0.1257f,  
	-0.5499f,0.8211f,-0.1529f,  -0.4566f,0.8575f,-0.2369f,  -0.4583f,0.8483f,-0.2654f,  -0.3577f,0.8671f,-0.3467f,  0.7101f,-0.5889f,-0.3859f,  0.7093f,-0.5933f,-0.3807f,  0.6115f,-0.6594f,-0.4373f,  0.6104f,-0.6678f,-0.4260f,  0.5018f,-0.7264f,-0.4696f,  0.5007f,-0.7383f,-0.4518f,  0.3834f,-0.7878f,-0.4821f,  0.3828f,-0.8026f,-0.4576f,  0.2590f,-0.8413f,-0.4744f,  0.2593f,-0.8581f,-0.4431f,  0.1312f,-0.8848f,-0.4471f,  
	0.1327f,-0.9027f,-0.4093f,  0.0027f,-0.9160f,-0.4012f,  0.0058f,-0.9339f,-0.3575f,  -0.1238f,-0.9326f,-0.3389f,  -0.1190f,-0.9496f,-0.2901f,  -0.2460f,-0.9330f,-0.2628f,  -0.2394f,-0.9479f,-0.2100f,  -0.3618f,-0.9155f,-0.1763f,  -0.3535f,-0.9276f,-0.1208f,  -0.4692f,-0.8792f,-0.0832f,  -0.4594f,-0.8878f,-0.0267f,  -0.5669f,-0.8237f,0.0122f,  -0.5559f,-0.8284f,0.0683f,  -0.6535f,-0.7495f,0.1057f,  -0.6418f,-0.7501f,0.1597f,  
	-0.7283f,-0.6575f,0.1929f,  -0.7162f,-0.6541f,0.2434f,  -0.7905f,-0.5497f,0.2700f,  -0.7785f,-0.5425f,0.3155f,  -0.8399f,-0.4283f,0.3334f,  -0.8284f,-0.4180f,0.3729f,  -0.8761f,-0.2963f,0.3803f,  -0.8656f,-0.2837f,0.4127f,  -0.8992f,-0.1572f,0.4084f,  -0.8899f,-0.1431f,0.4332f,  -0.9091f,-0.0147f,0.4163f,  -0.9013f,0.0000f,0.4333f,  -0.9061f,0.1274f,0.4035f,  -0.8998f,0.1418f,0.4126f,  -0.8903f,0.2652f,0.3702f,  
	-0.8856f,0.2783f,0.3717f,  -0.8621f,0.3950f,0.3175f,  -0.8589f,0.4061f,0.3121f,  -0.8217f,0.5134f,0.2473f,  -0.8199f,0.5218f,0.2356f,  -0.7697f,0.6175f,0.1620f,  -0.7690f,0.6226f,0.1449f,  -0.7065f,0.7047f,0.0645f,  -0.7067f,0.7062f,0.0432f,  -0.6330f,0.7730f,-0.0416f,  -0.6338f,0.7707f,-0.0662f,  -0.5499f,0.8211f,-0.1529f,  -0.5510f,0.8150f,-0.1795f,  -0.4583f,0.8483f,-0.2654f,  0.7093f,-0.5933f,-0.3807f,  
	0.7085f,-0.5979f,-0.3750f,  0.6104f,-0.6678f,-0.4260f,  0.6095f,-0.6763f,-0.4137f,  0.5007f,-0.7383f,-0.4518f,  0.5001f,-0.7500f,-0.4328f,  0.3828f,-0.8026f,-0.4576f,  0.3829f,-0.8168f,-0.4316f,  0.2593f,-0.8581f,-0.4431f,  0.2604f,-0.8740f,-0.4103f,  0.1327f,-0.9027f,-0.4093f,  0.1353f,-0.9191f,-0.3700f,  0.0058f,-0.9339f,-0.3575f,  0.0100f,-0.9499f,-0.3124f,  -0.1190f,-0.9496f,-0.2901f,  -0.1130f,-0.9641f,-0.2401f,  
	-0.2394f,-0.9479f,-0.2100f,  -0.2317f,-0.9601f,-0.1564f,  -0.3535f,-0.9276f,-0.1208f,  -0.3441f,-0.9367f,-0.0651f,  -0.4594f,-0.8878f,-0.0267f,  -0.4487f,-0.8932f,0.0298f,  -0.5559f,-0.8284f,0.0683f,  -0.5442f,-0.8298f,0.1238f,  -0.6418f,-0.7501f,0.1597f,  -0.6295f,-0.7474f,0.2125f,  -0.7162f,-0.6541f,0.2434f,  -0.7038f,-0.6475f,0.2922f,  -0.7785f,-0.5425f,0.3155f,  -0.7665f,-0.5325f,0.3590f,  -0.8284f,-0.4180f,0.3729f,  
	-0.8171f,-0.4053f,0.4100f,  -0.8656f,-0.2837f,0.4127f,  -0.8554f,-0.2690f,0.4428f,  -0.8899f,-0.1431f,0.4332f,  -0.8810f,-0.1273f,0.4557f,  -0.9013f,0.0000f,0.4333f,  -0.8939f,0.0159f,0.4480f,  -0.8998f,0.1418f,0.4126f,  -0.8941f,0.1568f,0.4196f,  -0.8856f,0.2783f,0.3717f,  -0.8814f,0.2917f,0.3715f,  -0.8589f,0.4061f,0.3121f,  -0.8561f,0.4171f,0.3052f,  -0.8199f,0.5218f,0.2356f,  -0.8183f,0.5299f,0.2228f,  
	-0.7690f,0.6226f,0.1449f,  -0.7684f,0.6272f,0.1272f,  -0.7067f,0.7062f,0.0432f,  -0.7068f,0.7070f,0.0216f,  -0.6338f,0.7707f,-0.0662f,  -0.6344f,0.7677f,-0.0905f,  -0.5510f,0.8150f,-0.1795f,  0.7085f,-0.5979f,-0.3750f,  0.7078f,-0.6027f,-0.3686f,  0.6095f,-0.6763f,-0.4137f,  0.6088f,-0.6847f,-0.4006f,  0.5001f,-0.7500f,-0.4328f,  0.5000f,-0.7614f,-0.4126f,  0.3829f,-0.8168f,-0.4316f,  0.3837f,-0.8303f,-0.4043f,  
	0.2604f,-0.8740f,-0.4103f,  0.2624f,-0.8886f,-0.3761f,  0.1353f,-0.9191f,-0.3700f,  0.1388f,-0.9340f,-0.3293f,  0.0100f,-0.9499f,-0.3124f,  0.0153f,-0.9638f,-0.2661f,  -0.1130f,-0.9641f,-0.2401f,  -0.1060f,-0.9762f,-0.1893f,  -0.2317f,-0.9601f,-0.1564f,  -0.2230f,-0.9694f,-0.1024f,  -0.3441f,-0.9367f,-0.0651f,  -0.3339f,-0.9426f,-0.0094f,  -0.4487f,-0.8932f,0.0298f,  -0.4374f,-0.8952f,0.0856f,  -0.5442f,-0.8298f,0.1238f,  
	-0.5320f,-0.8278f,0.1780f,  -0.6295f,-0.7474f,0.2125f,  -0.6170f,-0.7414f,0.2638f,  -0.7038f,-0.6475f,0.2922f,  -0.6914f,-0.6380f,0.3391f,  -0.7665f,-0.5325f,0.3590f,  -0.7546f,-0.5199f,0.4003f,  -0.8171f,-0.4053f,0.4100f,  -0.8062f,-0.3903f,0.4447f,  -0.8554f,-0.2690f,0.4428f,  -0.8456f,-0.2525f,0.4703f,  -0.8810f,-0.1273f,0.4557f,  -0.8727f,-0.1102f,0.4757f,  -0.8939f,0.0159f,0.4480f,  -0.8871f,0.0327f,0.4603f,  
	-0.8941f,0.1568f,0.4196f,  -0.8888f,0.1724f,0.4246f,  -0.8814f,0.2917f,0.3715f,  -0.8776f,0.3053f,0.3696f,  -0.8561f,0.4171f,0.3052f,  -0.8536f,0.4281f,0.2969f,  -0.8183f,0.5299f,0.2228f,  -0.8169f,0.5376f,0.2092f,  -0.7684f,0.6272f,0.1272f,  -0.7678f,0.6313f,0.1091f,  -0.7068f,0.7070f,0.0216f,  -0.7069f,0.7073f,-0.0000f,  -0.6344f,0.7677f,-0.0905f,  0.7078f,-0.6027f,-0.3686f,  0.7071f,-0.6077f,-0.3616f,  
	0.6088f,-0.6847f,-0.4006f,  0.6085f,-0.6931f,-0.3866f,  0.5000f,-0.7614f,-0.4126f,  0.5003f,-0.7724f,-0.3913f,  0.3837f,-0.8303f,-0.4043f,  0.3851f,-0.8429f,-0.3757f,  0.2624f,-0.8886f,-0.3761f,  0.2653f,-0.9020f,-0.3406f,  0.1388f,-0.9340f,-0.3293f,  0.1433f,-0.9470f,-0.2876f,  0.0153f,-0.9638f,-0.2661f,  0.0215f,-0.9755f,-0.2190f,  -0.1060f,-0.9762f,-0.1893f,  -0.0980f,-0.9856f,-0.1380f,  -0.2230f,-0.9694f,-0.1024f,  
	-0.2134f,-0.9758f,-0.0484f,  -0.3339f,-0.9426f,-0.0094f,  -0.3230f,-0.9453f,0.0459f,  -0.4374f,-0.8952f,0.0856f,  -0.4255f,-0.8940f,0.1404f,  -0.5320f,-0.8278f,0.1780f,  -0.5196f,-0.8226f,0.2309f,  -0.6170f,-0.7414f,0.2638f,  -0.6045f,-0.7324f,0.3132f,  -0.6914f,-0.6380f,0.3391f,  -0.6792f,-0.6257f,0.3837f,  -0.7546f,-0.5199f,0.4003f,  -0.7432f,-0.5049f,0.4391f,  -0.8062f,-0.3903f,0.4447f,  -0.7958f,-0.3733f,0.4769f,  
	-0.8456f,-0.2525f,0.4703f,  -0.8365f,-0.2344f,0.4953f,  -0.8727f,-0.1102f,0.4757f,  -0.8650f,-0.0921f,0.4932f,  -0.8871f,0.0327f,0.4603f,  -0.8810f,0.0501f,0.4705f,  -0.8888f,0.1724f,0.4246f,  -0.8841f,0.1883f,0.4277f,  -0.8776f,0.3053f,0.3696f,  -0.8743f,0.3189f,0.3660f,  -0.8536f,0.4281f,0.2969f,  -0.8514f,0.4387f,0.2875f,  -0.8169f,0.5376f,0.2092f,  -0.8156f,0.5448f,0.1947f,  -0.7678f,0.6313f,0.1091f,  
	-0.7673f,0.6349f,0.0905f,  -0.7069f,0.7073f,-0.0000f,  0.7071f,-0.6077f,-0.3616f,  0.7065f,-0.6128f,-0.3539f,  0.6085f,-0.6931f,-0.3866f,  0.6084f,-0.7013f,-0.3716f,  0.5003f,-0.7724f,-0.3913f,  0.5011f,-0.7828f,-0.3689f,  0.3851f,-0.8429f,-0.3757f,  0.3872f,-0.8546f,-0.3460f,  0.2653f,-0.9020f,-0.3406f,  0.2689f,-0.9139f,-0.3041f,  0.1433f,-0.9470f,-0.2876f,  0.1487f,-0.9581f,-0.2449f,  0.0215f,-0.9755f,-0.2190f,  
	0.0286f,-0.9848f,-0.1713f,  -0.0980f,-0.9856f,-0.1380f,  -0.0892f,-0.9922f,-0.0866f,  -0.2134f,-0.9758f,-0.0484f,  -0.2032f,-0.9791f,0.0054f,  -0.3230f,-0.9453f,0.0459f,  -0.3116f,-0.9449f,0.1003f,  -0.4255f,-0.8940f,0.1404f,  -0.4133f,-0.8897f,0.1939f,  -0.5196f,-0.8226f,0.2309f,  -0.5072f,-0.8144f,0.2820f,  -0.6045f,-0.7324f,0.3132f,  -0.5921f,-0.7207f,0.3605f,  -0.6792f,-0.6257f,0.3837f,  -0.6674f,-0.6109f,0.4259f,  
	-0.7432f,-0.5049f,0.4391f,  -0.7322f,-0.4878f,0.4753f,  -0.7958f,-0.3733f,0.4769f,  -0.7860f,-0.3546f,0.5064f,  -0.8365f,-0.2344f,0.4953f,  -0.8281f,-0.2152f,0.5177f,  -0.8650f,-0.0921f,0.4932f,  -0.8581f,-0.0730f,0.5083f,  -0.8810f,0.0501f,0.4705f,  -0.8755f,0.0680f,0.4785f,  -0.8841f,0.1883f,0.4277f,  -0.8800f,0.2043f,0.4288f,  -0.8743f,0.3189f,0.3660f,  -0.8714f,0.3323f,0.3610f,  -0.8514f,0.4387f,0.2875f,  
	-0.8495f,0.4490f,0.2770f,  -0.8156f,0.5448f,0.1947f,  -0.8146f,0.5515f,0.1795f,  -0.7673f,0.6349f,0.0905f,  0.7065f,-0.6128f,-0.3539f,  0.7060f,-0.6182f,-0.3456f,  0.6084f,-0.7013f,-0.3716f,  0.6086f,-0.7093f,-0.3558f,  0.5011f,-0.7828f,-0.3689f,  0.5024f,-0.7926f,-0.3455f,  0.3872f,-0.8546f,-0.3460f,  0.3899f,-0.8652f,-0.3153f,  0.2689f,-0.9139f,-0.3041f,  0.2732f,-0.9243f,-0.2666f,  0.1487f,-0.9581f,-0.2449f,  
	0.1548f,-0.9672f,-0.2016f,  0.0286f,-0.9848f,-0.1713f,  0.0364f,-0.9917f,-0.1233f,  -0.0892f,-0.9922f,-0.0866f,  -0.0799f,-0.9962f,-0.0352f,  -0.2032f,-0.9791f,0.0054f,  -0.1925f,-0.9796f,0.0585f,  -0.3116f,-0.9449f,0.1003f,  -0.3000f,-0.9415f,0.1536f,  -0.4133f,-0.8897f,0.1939f,  -0.4012f,-0.8824f,0.2459f,  -0.5072f,-0.8144f,0.2820f,  -0.4949f,-0.8034f,0.3311f,  -0.5921f,-0.7207f,0.3605f,  -0.5802f,-0.7064f,0.4054f,  
	-0.6674f,-0.6109f,0.4259f,  -0.6561f,-0.5939f,0.4656f,  -0.7322f,-0.4878f,0.4753f,  -0.7220f,-0.4688f,0.5089f,  -0.7860f,-0.3546f,0.5064f,  -0.7769f,-0.3347f,0.5332f,  -0.8281f,-0.2152f,0.5177f,  -0.8204f,-0.1949f,0.5375f,  -0.8581f,-0.0730f,0.5083f,  -0.8518f,-0.0534f,0.5211f,  -0.8755f,0.0680f,0.4785f,  -0.8706f,0.0862f,0.4843f,  -0.8800f,0.2043f,0.4288f,  -0.8764f,0.2203f,0.4282f,  -0.8714f,0.3323f,0.3610f,  
	-0.8689f,0.3455f,0.3545f,  -0.8495f,0.4490f,0.2770f,  -0.8480f,0.4588f,0.2655f,  -0.8146f,0.5515f,0.1795f,  0.7060f,-0.6182f,-0.3456f,  0.7056f,-0.6236f,-0.3365f,  0.6086f,-0.7093f,-0.3558f,  0.6090f,-0.7170f,-0.3390f,  0.5024f,-0.7926f,-0.3455f,  0.5042f,-0.8017f,-0.3211f,  0.3899f,-0.8652f,-0.3153f,  0.3931f,-0.8746f,-0.2837f,  0.2732f,-0.9243f,-0.2666f,  0.2782f,-0.9330f,-0.2284f,  0.1548f,-0.9672f,-0.2016f,  
	0.1615f,-0.9742f,-0.1579f,  0.0364f,-0.9917f,-0.1233f,  0.0448f,-0.9962f,-0.0753f,  -0.0799f,-0.9962f,-0.0352f,  -0.0700f,-0.9974f,0.0156f,  -0.1925f,-0.9796f,0.0585f,  -0.1815f,-0.9771f,0.1106f,  -0.3000f,-0.9415f,0.1536f,  -0.2884f,-0.9352f,0.2055f,  -0.4012f,-0.8824f,0.2459f,  -0.3892f,-0.8723f,0.2960f,  -0.4949f,-0.8034f,0.3311f,  -0.4830f,-0.7898f,0.3780f,  -0.5802f,-0.7064f,0.4054f,  -0.5688f,-0.6898f,0.4479f,  
	-0.6561f,-0.5939f,0.4656f,  -0.6456f,-0.5749f,0.5027f,  -0.7220f,-0.4688f,0.5089f,  -0.7125f,-0.4484f,0.5397f,  -0.7769f,-0.3347f,0.5332f,  -0.7687f,-0.3135f,0.5575f,  -0.8204f,-0.1949f,0.5375f,  -0.8136f,-0.1739f,0.5548f,  -0.8518f,-0.0534f,0.5211f,  -0.8464f,-0.0333f,0.5316f,  -0.8706f,0.0862f,0.4843f,  -0.8664f,0.1044f,0.4882f,  -0.8764f,0.2203f,0.4282f,  -0.8734f,0.2360f,0.4260f,  -0.8689f,0.3455f,0.3545f,  
	-0.8668f,0.3582f,0.3469f,  -0.8480f,0.4588f,0.2655f,  0.7056f,-0.6236f,-0.3365f,  0.7052f,-0.6292f,-0.3267f,  0.6090f,-0.7170f,-0.3390f,  0.6098f,-0.7245f,-0.3214f,  0.5042f,-0.8017f,-0.3211f,  0.5063f,-0.8100f,-0.2958f,  0.3931f,-0.8746f,-0.2837f,  0.3969f,-0.8828f,-0.2513f,  0.2782f,-0.9330f,-0.2284f,  0.2837f,-0.9399f,-0.1897f,  0.1615f,-0.9742f,-0.1579f,  0.1687f,-0.9791f,-0.1140f,  0.0448f,-0.9962f,-0.0753f,  
	0.0536f,-0.9982f,-0.0275f,  -0.0700f,-0.9974f,0.0156f,  -0.0600f,-0.9960f,0.0657f,  -0.1815f,-0.9771f,0.1106f,  -0.1706f,-0.9720f,0.1616f,  -0.2884f,-0.9352f,0.2055f,  -0.2768f,-0.9263f,0.2557f,  -0.3892f,-0.8723f,0.2960f,  -0.3776f,-0.8597f,0.3440f,  -0.4830f,-0.7898f,0.3780f,  -0.4717f,-0.7740f,0.4225f,  -0.5688f,-0.6898f,0.4479f,  -0.5581f,-0.6712f,0.4878f,  -0.6456f,-0.5749f,0.5027f,  -0.6358f,-0.5545f,0.5370f,  
	-0.7125f,-0.4484f,0.5397f,  -0.7039f,-0.4267f,0.5679f,  -0.7687f,-0.3135f,0.5575f,  -0.7614f,-0.2915f,0.5790f,  -0.8136f,-0.1739f,0.5548f,  -0.8076f,-0.1524f,0.5697f,  -0.8464f,-0.0333f,0.5316f,  -0.8416f,-0.0132f,0.5399f,  -0.8664f,0.1044f,0.4882f,  -0.8629f,0.1225f,0.4903f,  -0.8734f,0.2360f,0.4260f,  -0.8709f,0.2514f,0.4224f,  -0.8668f,0.3582f,0.3469f,  0.7052f,-0.6292f,-0.3267f,  0.7050f,-0.6349f,-0.3161f,  
	0.6098f,-0.7245f,-0.3214f,  0.6108f,-0.7316f,-0.3029f,  0.5063f,-0.8100f,-0.2958f,  0.5088f,-0.8175f,-0.2698f,  0.3969f,-0.8828f,-0.2513f,  0.4011f,-0.8896f,-0.2183f,  0.2837f,-0.9399f,-0.1897f,  0.2897f,-0.9452f,-0.1507f,  0.1687f,-0.9791f,-0.1140f,  0.1763f,-0.9818f,-0.0702f,  0.0536f,-0.9982f,-0.0275f,  0.0626f,-0.9978f,0.0196f,  -0.0600f,-0.9960f,0.0657f,  -0.0499f,-0.9921f,0.1148f,  -0.1706f,-0.9720f,0.1616f,  
	-0.1598f,-0.9643f,0.2110f,  -0.2768f,-0.9263f,0.2557f,  -0.2657f,-0.9148f,0.3041f,  -0.3776f,-0.8597f,0.3440f,  -0.3665f,-0.8448f,0.3898f,  -0.4717f,-0.7740f,0.4225f,  -0.4611f,-0.7560f,0.4645f,  -0.5581f,-0.6712f,0.4878f,  -0.5482f,-0.6511f,0.5250f,  -0.6358f,-0.5545f,0.5370f,  -0.6269f,-0.5326f,0.5686f,  -0.7039f,-0.4267f,0.5679f,  -0.6962f,-0.4040f,0.5934f,  -0.7614f,-0.2915f,0.5790f,  -0.7550f,-0.2688f,0.5981f,  
	-0.8076f,-0.1524f,0.5697f,  -0.8024f,-0.1306f,0.5823f,  -0.8416f,-0.0132f,0.5399f,  -0.8377f,0.0070f,0.5462f,  -0.8629f,0.1225f,0.4903f,  -0.8600f,0.1404f,0.4906f,  -0.8709f,0.2514f,0.4224f,  0.7050f,-0.6349f,-0.3161f,  0.7048f,-0.6406f,-0.3048f,  0.6108f,-0.7316f,-0.3029f,  0.6120f,-0.7382f,-0.2837f,  0.5088f,-0.8175f,-0.2698f,  0.5117f,-0.8241f,-0.2431f,  0.4011f,-0.8896f,-0.2183f,  0.4057f,-0.8951f,-0.1848f,  
	0.2897f,-0.9452f,-0.1507f,  0.2959f,-0.9487f,-0.1116f,  0.1763f,-0.9818f,-0.0702f,  0.1840f,-0.9826f,-0.0267f,  0.0626f,-0.9978f,0.0196f,  0.0716f,-0.9952f,0.0661f,  -0.0499f,-0.9921f,0.1148f,  -0.0400f,-0.9859f,0.1627f,  -0.1598f,-0.9643f,0.2110f,  -0.1493f,-0.9543f,0.2589f,  -0.2657f,-0.9148f,0.3041f,  -0.2551f,-0.9012f,0.3504f,  -0.3665f,-0.8448f,0.3898f,  -0.3561f,-0.8279f,0.4333f,  -0.4611f,-0.7560f,0.4645f,  
	-0.4512f,-0.7365f,0.5040f,  -0.5482f,-0.6511f,0.5250f,  -0.5392f,-0.6294f,0.5596f,  -0.6269f,-0.5326f,0.5686f,  -0.6190f,-0.5097f,0.5976f,  -0.6962f,-0.4040f,0.5934f,  -0.6894f,-0.3806f,0.6163f,  -0.7550f,-0.2688f,0.5981f,  -0.7494f,-0.2457f,0.6148f,  -0.8024f,-0.1306f,0.5823f,  -0.7981f,-0.1087f,0.5927f,  -0.8377f,0.0070f,0.5462f,  -0.8344f,0.0269f,0.5506f,  -0.8600f,0.1404f,0.4906f,  0.7048f,-0.6406f,-0.3048f,  
	0.7047f,-0.6463f,-0.2926f,  0.6120f,-0.7382f,-0.2837f,  0.6134f,-0.7444f,-0.2636f,  0.5117f,-0.8241f,-0.2431f,  0.5148f,-0.8297f,-0.2157f,  0.4057f,-0.8951f,-0.1848f,  0.4105f,-0.8992f,-0.1511f,  0.2959f,-0.9487f,-0.1116f,  0.3023f,-0.9504f,-0.0726f,  0.1840f,-0.9826f,-0.0267f,  0.1918f,-0.9813f,0.0163f,  0.0716f,-0.9952f,0.0661f,  0.0805f,-0.9905f,0.1116f,  -0.0400f,-0.9859f,0.1627f,  -0.0303f,-0.9774f,0.2092f,  
	-0.1493f,-0.9543f,0.2589f,  -0.1393f,-0.9421f,0.3049f,  -0.2551f,-0.9012f,0.3504f,  -0.2451f,-0.8856f,0.3946f,  -0.3561f,-0.8279f,0.4333f,  -0.3465f,-0.8093f,0.4743f,  -0.4512f,-0.7365f,0.5040f,  -0.4423f,-0.7154f,0.5409f,  -0.5392f,-0.6294f,0.5596f,  -0.5312f,-0.6066f,0.5915f,  -0.6190f,-0.5097f,0.5976f,  -0.6121f,-0.4859f,0.6239f,  -0.6894f,-0.3806f,0.6163f,  -0.6836f,-0.3567f,0.6367f,  -0.7494f,-0.2457f,0.6148f,  
	-0.7448f,-0.2226f,0.6291f,  -0.7981f,-0.1087f,0.5927f,  -0.7945f,-0.0870f,0.6011f,  -0.8344f,0.0269f,0.5506f,  0.7047f,-0.6463f,-0.2926f,  0.7047f,-0.6521f,-0.2797f,  0.6134f,-0.7444f,-0.2636f,  0.6150f,-0.7502f,-0.2429f,  0.5148f,-0.8297f,-0.2157f,  0.5181f,-0.8344f,-0.1880f,  0.4105f,-0.8992f,-0.1511f,  0.4155f,-0.9020f,-0.1172f,  0.3023f,-0.9504f,-0.0726f,  0.3088f,-0.9505f,-0.0338f,  0.1918f,-0.9813f,0.0163f,  
	0.1996f,-0.9781f,0.0585f,  0.0805f,-0.9905f,0.1116f,  0.0891f,-0.9837f,0.1560f,  -0.0303f,-0.9774f,0.2092f,  -0.0211f,-0.9669f,0.2542f,  -0.1393f,-0.9421f,0.3049f,  -0.1299f,-0.9281f,0.3490f,  -0.2451f,-0.8856f,0.3946f,  -0.2358f,-0.8683f,0.4365f,  -0.3465f,-0.8093f,0.4743f,  -0.3378f,-0.7892f,0.5130f,  -0.4423f,-0.7154f,0.5409f,  -0.4343f,-0.6931f,0.5753f,  -0.5312f,-0.6066f,0.5915f,  -0.5242f,-0.5829f,0.6209f,  
	-0.6121f,-0.4859f,0.6239f,  -0.6061f,-0.4616f,0.6478f,  -0.6836f,-0.3567f,0.6367f,  -0.6787f,-0.3326f,0.6547f,  -0.7448f,-0.2226f,0.6291f,  -0.7409f,-0.1994f,0.6413f,  -0.7945f,-0.0870f,0.6011f,  0.7047f,-0.6521f,-0.2797f,  0.7047f,-0.6578f,-0.2659f,  0.6150f,-0.7502f,-0.2429f,  0.6167f,-0.7553f,-0.2216f,  0.5181f,-0.8344f,-0.1880f,  0.5215f,-0.8382f,-0.1598f,  0.4155f,-0.9020f,-0.1172f,  0.4204f,-0.9035f,-0.0833f,  
	0.3088f,-0.9505f,-0.0338f,  0.3152f,-0.9490f,0.0045f,  0.1996f,-0.9781f,0.0585f,  0.2070f,-0.9732f,0.1000f,  0.0891f,-0.9837f,0.1560f,  0.0974f,-0.9751f,0.1991f,  -0.0211f,-0.9669f,0.2542f,  -0.0125f,-0.9546f,0.2975f,  -0.1299f,-0.9281f,0.3490f,  -0.1212f,-0.9123f,0.3911f,  -0.2358f,-0.8683f,0.4365f,  -0.2275f,-0.8494f,0.4762f,  -0.3378f,-0.7892f,0.5130f,  -0.3300f,-0.7678f,0.5492f,  -0.4343f,-0.6931f,0.5753f,  
	-0.4274f,-0.6699f,0.6072f,  -0.5242f,-0.5829f,0.6209f,  -0.5182f,-0.5585f,0.6477f,  -0.6061f,-0.4616f,0.6478f,  -0.6011f,-0.4369f,0.6692f,  -0.6787f,-0.3326f,0.6547f,  -0.6747f,-0.3084f,0.6705f,  -0.7409f,-0.1994f,0.6413f,  0.7047f,-0.6578f,-0.2659f,  0.7049f,-0.6633f,-0.2513f,  0.6167f,-0.7553f,-0.2216f,  0.6185f,-0.7600f,-0.1996f,  0.5215f,-0.8382f,-0.1598f,  0.5249f,-0.8410f,-0.1313f,  0.4204f,-0.9035f,-0.0833f,  
	0.4254f,-0.9036f,-0.0495f,  0.3152f,-0.9490f,0.0045f,  0.3214f,-0.9460f,0.0424f,  0.2070f,-0.9732f,0.1000f,  0.2142f,-0.9666f,0.1406f,  0.0974f,-0.9751f,0.1991f,  0.1051f,-0.9649f,0.2408f,  -0.0125f,-0.9546f,0.2975f,  -0.0044f,-0.9408f,0.3390f,  -0.1212f,-0.9123f,0.3911f,  -0.1133f,-0.8951f,0.4312f,  -0.2275f,-0.8494f,0.4762f,  -0.2200f,-0.8293f,0.5136f,  -0.3300f,-0.7678f,0.5492f,  -0.3232f,-0.7454f,0.5830f,  
	-0.4274f,-0.6699f,0.6072f,  -0.4214f,-0.6460f,0.6366f,  -0.5182f,-0.5585f,0.6477f,  -0.5131f,-0.5338f,0.6722f,  -0.6011f,-0.4369f,0.6692f,  -0.5970f,-0.4121f,0.6883f,  -0.6747f,-0.3084f,0.6705f,  0.7049f,-0.6633f,-0.2513f,  0.7050f,-0.6688f,-0.2359f,  0.6185f,-0.7600f,-0.1996f,  0.6204f,-0.7641f,-0.1770f,  0.5249f,-0.8410f,-0.1313f,  0.5284f,-0.8428f,-0.1027f,  0.4254f,-0.9036f,-0.0495f,  0.4302f,-0.9026f,-0.0159f,  
	0.3214f,-0.9460f,0.0424f,  0.3273f,-0.9415f,0.0797f,  0.2142f,-0.9666f,0.1406f,  0.2209f,-0.9585f,0.1801f,  0.1051f,-0.9649f,0.2408f,  0.1124f,-0.9531f,0.2811f,  -0.0044f,-0.9408f,0.3390f,  0.0028f,-0.9255f,0.3788f,  -0.1133f,-0.8951f,0.4312f,  -0.1063f,-0.8767f,0.4692f,  -0.2200f,-0.8293f,0.5136f,  -0.2135f,-0.8082f,0.5489f,  -0.3232f,-0.7454f,0.5830f,  -0.3173f,-0.7223f,0.6145f,  -0.4214f,-0.6460f,0.6366f,  
	-0.4163f,-0.6215f,0.6636f,  -0.5131f,-0.5338f,0.6722f,  -0.5090f,-0.5088f,0.6943f,  -0.5970f,-0.4121f,0.6883f,  0.7050f,-0.6688f,-0.2359f,  0.7052f,-0.6741f,-0.2197f,  0.6204f,-0.7641f,-0.1770f,  0.6223f,-0.7675f,-0.1541f,  0.5284f,-0.8428f,-0.1027f,  0.5317f,-0.8437f,-0.0739f,  0.4302f,-0.9026f,-0.0159f,  0.4349f,-0.9003f,0.0174f,  0.3273f,-0.9415f,0.0797f,  0.3329f,-0.9358f,0.1162f,  0.2209f,-0.9585f,0.1801f,  
	0.2272f,-0.9490f,0.2184f,  0.1124f,-0.9531f,0.2811f,  0.1189f,-0.9400f,0.3199f,  0.0028f,-0.9255f,0.3788f,  0.0093f,-0.9090f,0.4168f,  -0.1063f,-0.8767f,0.4692f,  -0.1001f,-0.8571f,0.5053f,  -0.2135f,-0.8082f,0.5489f,  -0.2079f,-0.7863f,0.5818f,  -0.3173f,-0.7223f,0.6145f,  -0.3124f,-0.6986f,0.6437f,  -0.4163f,-0.6215f,0.6636f,  -0.4122f,-0.5968f,0.6884f,  -0.5090f,-0.5088f,0.6943f,  0.7052f,-0.6741f,-0.2197f,  
	0.7055f,-0.6791f,-0.2027f,  0.6223f,-0.7675f,-0.1541f,  0.6241f,-0.7704f,-0.1306f,  0.5317f,-0.8437f,-0.0739f,  0.5350f,-0.8437f,-0.0450f,  0.4349f,-0.9003f,0.0174f,  0.4392f,-0.8970f,0.0504f,  0.3329f,-0.9358f,0.1162f,  0.3382f,-0.9287f,0.1519f,  0.2272f,-0.9490f,0.2184f,  0.2329f,-0.9383f,0.2556f,  0.1189f,-0.9400f,0.3199f,  0.1247f,-0.9257f,0.3572f,  0.0093f,-0.9090f,0.4168f,  0.0149f,-0.8914f,0.4530f,  
	-0.1001f,-0.8571f,0.5053f,  -0.0948f,-0.8368f,0.5392f,  -0.2079f,-0.7863f,0.5818f,  -0.2032f,-0.7638f,0.6127f,  -0.3124f,-0.6986f,0.6437f,  -0.3084f,-0.6746f,0.6707f,  -0.4122f,-0.5968f,0.6884f,  0.7055f,-0.6791f,-0.2027f,  0.7058f,-0.6839f,-0.1850f,  0.6241f,-0.7704f,-0.1306f,  0.6259f,-0.7726f,-0.1067f,  0.5350f,-0.8437f,-0.0450f,  0.5380f,-0.8428f,-0.0162f,  0.4392f,-0.8970f,0.0504f,  0.4433f,-0.8925f,0.0829f,  
	0.3382f,-0.9287f,0.1519f,  0.3429f,-0.9206f,0.1868f,  0.2329f,-0.9383f,0.2556f,  0.2379f,-0.9265f,0.2915f,  0.1247f,-0.9257f,0.3572f,  0.1297f,-0.9104f,0.3929f,  0.0149f,-0.8914f,0.4530f,  0.0198f,-0.8730f,0.4873f,  -0.0948f,-0.8368f,0.5392f,  -0.0904f,-0.8158f,0.5712f,  -0.2032f,-0.7638f,0.6127f,  -0.1993f,-0.7408f,0.6415f,  -0.3084f,-0.6746f,0.6707f,  0.7058f,-0.6839f,-0.1850f,  0.7061f,-0.6883f,-0.1665f,  
	0.6259f,-0.7726f,-0.1067f,  0.6276f,-0.7742f,-0.0825f,  0.5380f,-0.8428f,-0.0162f,  0.5409f,-0.8410f,0.0124f,  0.4433f,-0.8925f,0.0829f,  0.4470f,-0.8871f,0.1149f,  0.3429f,-0.9206f,0.1868f,  0.3471f,-0.9115f,0.2207f,  0.2379f,-0.9265f,0.2915f,  0.2423f,-0.9137f,0.3262f,  0.1297f,-0.9104f,0.3929f,  0.1341f,-0.8942f,0.4271f,  0.0198f,-0.8730f,0.4873f,  0.0239f,-0.8539f,0.5199f,  -0.0904f,-0.8158f,0.5712f,  
	-0.0868f,-0.7943f,0.6012f,  -0.1993f,-0.7408f,0.6415f,  0.7061f,-0.6883f,-0.1665f,  0.7064f,-0.6923f,-0.1474f,  0.6276f,-0.7742f,-0.0825f,  0.6292f,-0.7751f,-0.0582f,  0.5409f,-0.8410f,0.0124f,  0.5434f,-0.8384f,0.0410f,  0.4470f,-0.8871f,0.1149f,  0.4503f,-0.8808f,0.1463f,  0.3471f,-0.9115f,0.2207f,  0.3507f,-0.9014f,0.2538f,  0.2423f,-0.9137f,0.3262f,  0.2461f,-0.9001f,0.3595f,  0.1341f,-0.8942f,0.4271f,  
	0.1377f,-0.8774f,0.4596f,  0.0239f,-0.8539f,0.5199f,  0.0271f,-0.8343f,0.5507f,  -0.0868f,-0.7943f,0.6012f,  0.7064f,-0.6923f,-0.1474f,  0.7067f,-0.6959f,-0.1277f,  0.6292f,-0.7751f,-0.0582f,  0.6306f,-0.7754f,-0.0335f,  0.5434f,-0.8384f,0.0410f,  0.5457f,-0.8351f,0.0693f,  0.4503f,-0.8808f,0.1463f,  0.4531f,-0.8737f,0.1771f,  0.3507f,-0.9014f,0.2538f,  0.3538f,-0.8906f,0.2857f,  0.2461f,-0.9001f,0.3595f,  
	0.2492f,-0.8858f,0.3915f,  0.1377f,-0.8774f,0.4596f,  0.1406f,-0.8599f,0.4907f,  0.0271f,-0.8343f,0.5507f,  0.7067f,-0.6959f,-0.1277f,  0.7069f,-0.6991f,-0.1075f,  0.6306f,-0.7754f,-0.0335f,  0.6319f,-0.7750f,-0.0088f,  0.5457f,-0.8351f,0.0693f,  0.5477f,-0.8310f,0.0974f,  0.4531f,-0.8737f,0.1771f,  0.4555f,-0.8658f,0.2071f,  0.3538f,-0.8906f,0.2857f,  0.3564f,-0.8790f,0.3167f,  0.2492f,-0.8858f,0.3915f,  
	0.2517f,-0.8708f,0.4223f,  0.1406f,-0.8599f,0.4907f,  0.7069f,-0.6991f,-0.1075f,  0.7072f,-0.7017f,-0.0868f,  0.6319f,-0.7750f,-0.0088f,  0.6329f,-0.7740f,0.0160f,  0.5477f,-0.8310f,0.0974f,  0.5494f,-0.8262f,0.1250f,  0.4555f,-0.8658f,0.2071f,  0.4575f,-0.8572f,0.2365f,  0.3564f,-0.8790f,0.3167f,  0.3585f,-0.8668f,0.3466f,  0.2517f,-0.8708f,0.4223f,  0.7072f,-0.7017f,-0.0868f,  0.7073f,-0.7038f,-0.0656f,  
	0.6329f,-0.7740f,0.0160f,  0.6339f,-0.7724f,0.0407f,  0.5494f,-0.8262f,0.1250f,  0.5508f,-0.8207f,0.1522f,  0.4575f,-0.8572f,0.2365f,  0.4591f,-0.8479f,0.2650f,  0.3585f,-0.8668f,0.3466f,  0.7073f,-0.7038f,-0.0656f,  0.7075f,-0.7054f,-0.0442f,  0.6339f,-0.7724f,0.0407f,  0.6346f,-0.7701f,0.0653f,  0.5508f,-0.8207f,0.1522f,  0.5519f,-0.8145f,0.1789f,  0.4591f,-0.8479f,0.2650f,  0.7075f,-0.7054f,-0.0442f,  
	0.7076f,-0.7063f,-0.0226f,  0.6346f,-0.7701f,0.0653f,  0.6352f,-0.7671f,0.0898f,  0.5519f,-0.8145f,0.1789f,  0.7076f,-0.7066f,-0.0009f,  0.6352f,-0.7671f,0.0898f,  0.7076f,-0.7063f,-0.0226f,  
_NULL_ }; /* End of torus_knot01_Normals */

unsigned short torus_knot01_StripLength[] = {
	96,95,96, 93,96,91, 96,89,96, 87,96,85, 96,83,96, 81,96,79, 96,77,96, 75,96,73, 96,71,96, 69,96,67, 96,65,96, 63,96,61, 96,59,96, 57,
	96,55, 96,53,96, 51,96,49, 96,47,96, 45,96,43, 96,41,96, 39,96,37, 96,35,96, 33,96,31, 96,29,96, 27,96,25, 96,23,96, 21,96,19, 96,17,
	96, 15,96,13, 96,11,96, 9,96,7, 96,5,96, 3,96,1, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 
	96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,
	96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,
	96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 
	96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,
	96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,
	96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 
	96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,
	96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,
	96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 
	96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,96, 96,96,95, 93,91,89, 87,85,83, 81,
	79,77, 75,73,71, 69,67,65, 63,61,59, 57,55,53, 51,49,47, 45,43,41, 39,37,35, 33,31,29, 27,25,23, 21,19,17, 15,13,11, 9,7,5, 3,1,
	
_NULL_ }; /* End of torus_knot01_pStripLength */



/**********************************************************
 * MESHES 
 **********************************************************/

Struct_Mesh 
   Mesh[NUM_MESHES] = {

/* MESH 0 */
	/* NumVertex      */ torus_knot01_NumVertex,
	/* NumFaces       */ torus_knot01_NumFaces,
	/* NumStrips      */ torus_knot01_NumStrips,
	/* Flags          */ torus_knot01_Flags,
	/* Material       */ torus_knot01_Material,
	/* Center         */  0.548569f, 0.000000f, 1.684380f,
	/* Vertices       */ (float *)torus_knot01_Vertices,
	/* UV             */ (float *)torus_knot01_UV,
	/* Normals        */ (float *)torus_knot01_Normals,
	/* Packed Vertices*/ (float *)_NULL_,
	/* VertexColor    */ (unsigned int *)_NULL_,
	/* VertexMaterial */ (unsigned int *)_NULL_,
	/* Faces          */ (unsigned short *)_NULL_,
	/* Strips         */ (unsigned short *)_NULL_,
	/* StripLength    */ (unsigned short *)torus_knot01_StripLength, 
	/* Patch type     */ 0,
	/* NumPatches     */ 0,
	/* NumPVertices   */ 0,
	/* NumSubdivisions*/ 0,
	/* ControlPoints  */ (float *)_NULL_,
	/* PatchUVs       */ (float *)_NULL_

	}; /* End of Meshes */
