/******************************************************************************

 @File         Vase.h

 @Title        Vase

 @Copyright    Copyright (C) 2005 - 2006 by Imagination Technologies Limited.

 @Platform     Independant

 @Description  Shows textured transparency and reflections. Requires the PVRShell.

******************************************************************************/
#define STRUCT_MESH_DEFINED 

typedef struct { unsigned int      nNumVertex;
                 unsigned int      nNumFaces;
                 unsigned int      nNumStrips;
                 unsigned int      nFlags;
                 int               nMaterial;
                 float             fCenter[3];
                 const float             *pVertex;
                 const float             *pUV;
                 const float             *pNormals;
                 const float             *pPackedVertex;
                 const unsigned int      *pVertexColor;
                 const unsigned int      *pVertexMaterial;
                 const unsigned short    *pFaces;
                 const unsigned short    *pStrips;
                 const unsigned short    *pStripLength;
                 struct
                 {
                     unsigned int  nType;
                     unsigned int  nNumPatches;
                     unsigned int  nNumVertices;
                     unsigned int  nNumSubdivisions;
                     const float         *pControlPoints;
                     const float         *pUVs;
                 } Patch;
               }   Struct_Mesh;


/**********************************************************
 * GENERAL DEFINITIONS
 **********************************************************/

#ifndef _NULL_
#define _NULL_ 0
#endif

#define NUM_MESHES     2
#define NUM_BONES	   0
#define NUM_MATERIALS  2
#define NUM_LIGHTS     0
#define NUM_CAMERAS    0
#define NUM_PATHS      0
#define NUM_FRAMES     0

/* MESH FLAGS */

#ifndef MF_MATERIAL
#define MF_MATERIAL        1
#define MF_UV              2
#define MF_NORMALS         4
#define MF_VERTEXCOLOR     8
#define MF_VERTEXMATERIAL  16
#define MF_STRIPS          32
#define MF_NOFACELIST      64
#define MF_UV2             128
#define MF_VERTEXWEIGHTS   256

#define MF_NOPATCH         0
#define MF_RECTPATCH       1
#define MF_TRIPATCH        2
#define MF_MIXPATCH        3

#endif


/* MESHES */
#define M_GLASS  0
#define M_SILVER  1


/**********************************************************
 * MESHES DATA 
 **********************************************************/

/* MESH 0 */

#define glass_NumVertex  287
#define glass_NumFaces   480
#define glass_NumStrips  0
#define glass_Flags      0x00020006
#define glass_Material   0

const float glass_Vertices[] = {
	24.7641f,-13.3051f,-0.9635f, 24.4682f,-13.3051f,-4.7226f, 24.7641f,-25.0968f,-0.9635f, 24.4682f,-25.0968f,-4.7226f, 23.5880f,-13.3051f,-8.3891f, 
	23.5880f,-25.0968f,-8.3891f, 22.1450f,-13.3051f,-11.8728f, 22.1450f,-25.0968f,-11.8728f, 20.1748f,-13.3051f,-15.0878f, 20.1748f,-25.0968f,-15.0878f, 
	17.7259f,-13.3051f,-17.9551f, 17.7259f,-25.0968f,-17.9551f, 14.8587f,-13.3051f,-20.4040f, 14.8587f,-25.0968f,-20.4040f, 11.6436f,-13.3051f,-22.3742f, 
	11.6436f,-25.0968f,-22.3742f, 8.1599f,-13.3051f,-23.8171f, 8.1599f,-25.0968f,-23.8171f, 4.4934f,-13.3051f,-24.6974f, 4.4934f,-25.0968f,-24.6974f, 
	0.7343f,-13.3051f,-24.9932f, 0.7343f,-25.0968f,-24.9932f, 0.7343f,-13.3051f,-24.9932f, -3.0248f,-13.3051f,-24.6974f, 0.7343f,-25.0968f,-24.9932f, 
	-3.0248f,-25.0968f,-24.6974f, -6.6913f,-13.3051f,-23.8171f, -6.6913f,-25.0968f,-23.8171f, -10.1749f,-13.3051f,-22.3742f, -10.1749f,-25.0968f,-22.3742f, 
	-13.3900f,-13.3051f,-20.4040f, -13.3900f,-25.0968f,-20.4040f, -16.2573f,-13.3051f,-17.9551f, -16.2573f,-25.0968f,-17.9551f, -18.7061f,-13.3051f,-15.0878f, 
	-18.7061f,-25.0968f,-15.0878f, -20.6763f,-13.3051f,-11.8728f, -20.6763f,-25.0968f,-11.8728f, -22.1193f,-13.3051f,-8.3891f, -22.1193f,-25.0968f,-8.3891f, 
	-22.9996f,-13.3051f,-4.7226f, -22.9996f,-25.0968f,-4.7226f, -23.2954f,-13.3051f,-0.9635f, -23.2954f,-25.0968f,-0.9635f, -22.9996f,-13.3051f,2.7956f, 
	-22.9996f,-25.0968f,2.7956f, -22.1193f,-13.3051f,6.4621f, -22.1193f,-25.0968f,6.4621f, -20.6763f,-13.3051f,9.9458f, -20.6763f,-25.0968f,9.9458f, 
	-18.7061f,-13.3051f,13.1608f, -18.7061f,-25.0968f,13.1608f, -16.2573f,-13.3051f,16.0281f, -16.2573f,-25.0968f,16.0281f, -13.3900f,-13.3051f,18.4770f, 
	-13.3900f,-25.0968f,18.4770f, -10.1749f,-13.3051f,20.4471f, -10.1749f,-25.0968f,20.4471f, -6.6913f,-13.3051f,21.8901f, -6.6913f,-25.0968f,21.8901f, 
	-3.0247f,-13.3051f,22.7704f, -3.0247f,-25.0968f,22.7704f, 0.7343f,-13.3051f,23.0662f, 0.7343f,-25.0968f,23.0662f, 4.4934f,-13.3051f,22.7704f, 
	4.4934f,-25.0968f,22.7704f, 8.1599f,-13.3051f,21.8901f, 8.1599f,-25.0968f,21.8901f, 11.6436f,-13.3051f,20.4471f, 11.6436f,-25.0968f,20.4471f, 
	14.8587f,-13.3051f,18.4770f, 14.8587f,-25.0968f,18.4770f, 17.7259f,-13.3051f,16.0281f, 17.7259f,-25.0968f,16.0281f, 20.1748f,-13.3051f,13.1608f, 
	20.1748f,-25.0968f,13.1608f, 22.1450f,-13.3051f,9.9458f, 22.1450f,-25.0968f,9.9458f, 23.5880f,-13.3051f,6.4621f, 23.5880f,-25.0968f,6.4621f, 
	24.4682f,-13.3051f,2.7956f, 24.4682f,-25.0968f,2.7956f, 24.7641f,-1.5135f,-0.9635f, 24.4682f,-1.5135f,-4.7226f, 23.5880f,-1.5135f,-8.3891f, 
	22.1450f,-1.5135f,-11.8728f, 20.1748f,-1.5135f,-15.0878f, 17.7259f,-1.5135f,-17.9551f, 14.8587f,-1.5135f,-20.4040f, 11.6436f,-1.5135f,-22.3742f, 
	8.1599f,-1.5135f,-23.8171f, 4.4934f,-1.5135f,-24.6974f, 0.7343f,-1.5135f,-24.9932f, 0.7343f,-1.5135f,-24.9932f, -3.0248f,-1.5135f,-24.6974f, 
	-6.6913f,-1.5135f,-23.8171f, -10.1749f,-1.5135f,-22.3742f, -13.3900f,-1.5135f,-20.4040f, -16.2573f,-1.5135f,-17.9551f, -18.7061f,-1.5135f,-15.0878f, 
	-20.6763f,-1.5135f,-11.8728f, -22.1193f,-1.5135f,-8.3891f, -22.9996f,-1.5135f,-4.7226f, -23.2954f,-1.5135f,-0.9635f, -22.9996f,-1.5135f,2.7956f, 
	-22.1193f,-1.5135f,6.4621f, -20.6763f,-1.5135f,9.9458f, -18.7061f,-1.5135f,13.1608f, -16.2573f,-1.5135f,16.0281f, -13.3900f,-1.5135f,18.4770f, 
	-10.1749f,-1.5135f,20.4471f, -6.6913f,-1.5135f,21.8901f, -3.0247f,-1.5135f,22.7704f, 0.7343f,-1.5135f,23.0662f, 4.4934f,-1.5135f,22.7704f, 
	8.1599f,-1.5135f,21.8901f, 11.6436f,-1.5135f,20.4471f, 14.8587f,-1.5135f,18.4770f, 17.7259f,-1.5135f,16.0281f, 20.1748f,-1.5135f,13.1608f, 
	22.1450f,-1.5135f,9.9458f, 23.5880f,-1.5135f,6.4621f, 24.4682f,-1.5135f,2.7956f, 24.7641f,10.2782f,-0.9635f, 24.4682f,10.2782f,-4.7226f, 
	23.5880f,10.2782f,-8.3891f, 22.1450f,10.2782f,-11.8728f, 20.1748f,10.2782f,-15.0878f, 17.7259f,10.2782f,-17.9551f, 14.8587f,10.2782f,-20.4040f, 
	11.6436f,10.2782f,-22.3742f, 8.1599f,10.2782f,-23.8171f, 4.4934f,10.2782f,-24.6974f, 0.7343f,10.2782f,-24.9932f, 0.7343f,10.2782f,-24.9932f, 
	-3.0248f,10.2782f,-24.6974f, -6.6913f,10.2782f,-23.8171f, -10.1749f,10.2782f,-22.3742f, -13.3900f,10.2782f,-20.4040f, -16.2573f,10.2782f,-17.9551f, 
	-18.7061f,10.2782f,-15.0878f, -20.6763f,10.2782f,-11.8728f, -22.1193f,10.2782f,-8.3891f, -22.9996f,10.2782f,-4.7226f, -23.2954f,10.2782f,-0.9635f, 
	-22.9996f,10.2782f,2.7956f, -22.1193f,10.2782f,6.4621f, -20.6763f,10.2782f,9.9458f, -18.7061f,10.2782f,13.1608f, -16.2573f,10.2782f,16.0281f, 
	-13.3900f,10.2782f,18.4770f, -10.1749f,10.2782f,20.4471f, -6.6913f,10.2782f,21.8901f, -3.0247f,10.2782f,22.7704f, 0.7343f,10.2782f,23.0662f, 
	4.4934f,10.2782f,22.7704f, 8.1599f,10.2782f,21.8901f, 11.6436f,10.2782f,20.4471f, 14.8587f,10.2782f,18.4770f, 17.7259f,10.2782f,16.0281f, 
	20.1748f,10.2782f,13.1608f, 22.1450f,10.2782f,9.9458f, 23.5880f,10.2782f,6.4621f, 24.4682f,10.2782f,2.7956f, 24.7641f,22.0699f,-0.9635f, 
	24.4682f,22.0699f,-4.7226f, 23.5880f,22.0699f,-8.3891f, 22.1450f,22.0699f,-11.8728f, 20.1748f,22.0699f,-15.0878f, 17.7259f,22.0699f,-17.9551f, 
	14.8587f,22.0699f,-20.4040f, 11.6436f,22.0699f,-22.3742f, 8.1599f,22.0699f,-23.8171f, 4.4934f,22.0699f,-24.6974f, 0.7343f,22.0699f,-24.9932f, 
	0.7343f,22.0699f,-24.9932f, -3.0248f,22.0699f,-24.6974f, -6.6913f,22.0699f,-23.8171f, -10.1749f,22.0699f,-22.3742f, -13.3900f,22.0699f,-20.4040f, 
	-16.2573f,22.0699f,-17.9551f, -18.7061f,22.0699f,-15.0878f, -20.6763f,22.0699f,-11.8728f, -22.1193f,22.0699f,-8.3891f, -22.9996f,22.0699f,-4.7226f, 
	-23.2954f,22.0699f,-0.9635f, -22.9996f,22.0699f,2.7956f, -22.1193f,22.0699f,6.4621f, -20.6763f,22.0699f,9.9458f, -18.7061f,22.0699f,13.1608f, 
	-16.2573f,22.0699f,16.0281f, -13.3900f,22.0699f,18.4770f, -10.1749f,22.0699f,20.4471f, -6.6913f,22.0699f,21.8901f, -3.0247f,22.0699f,22.7704f, 
	0.7343f,22.0699f,23.0662f, 4.4934f,22.0699f,22.7704f, 8.1599f,22.0699f,21.8901f, 11.6436f,22.0699f,20.4471f, 14.8587f,22.0699f,18.4770f, 
	17.7259f,22.0699f,16.0281f, 20.1748f,22.0699f,13.1608f, 22.1450f,22.0699f,9.9458f, 23.5880f,22.0699f,6.4621f, 24.4682f,22.0699f,2.7956f, 
	24.7641f,33.8616f,-0.9635f, 24.4682f,33.8616f,-4.7226f, 23.5880f,33.8616f,-8.3891f, 22.1450f,33.8616f,-11.8728f, 20.1748f,33.8616f,-15.0878f, 
	17.7259f,33.8616f,-17.9551f, 14.8587f,33.8616f,-20.4040f, 11.6436f,33.8616f,-22.3742f, 8.1599f,33.8616f,-23.8171f, 4.4934f,33.8616f,-24.6974f, 
	0.7343f,33.8616f,-24.9932f, 0.7343f,33.8616f,-24.9932f, -3.0248f,33.8616f,-24.6974f, -6.6913f,33.8616f,-23.8171f, -10.1749f,33.8616f,-22.3742f, 
	-13.3900f,33.8616f,-20.4040f, -16.2573f,33.8616f,-17.9551f, -18.7061f,33.8616f,-15.0878f, -20.6763f,33.8616f,-11.8728f, -22.1193f,33.8616f,-8.3891f, 
	-22.9996f,33.8616f,-4.7226f, -23.2954f,33.8616f,-0.9635f, -22.9996f,33.8616f,2.7956f, -22.1193f,33.8616f,6.4621f, -20.6763f,33.8616f,9.9458f, 
	-18.7061f,33.8616f,13.1608f, -16.2573f,33.8616f,16.0281f, -13.3900f,33.8616f,18.4770f, -10.1749f,33.8616f,20.4471f, -6.6913f,33.8616f,21.8901f, 
	-3.0247f,33.8616f,22.7704f, 0.7343f,33.8616f,23.0662f, 4.4934f,33.8616f,22.7704f, 8.1599f,33.8616f,21.8901f, 11.6436f,33.8616f,20.4471f, 
	14.8587f,33.8616f,18.4770f, 17.7259f,33.8616f,16.0281f, 20.1748f,33.8616f,13.1608f, 22.1450f,33.8616f,9.9458f, 23.5880f,33.8616f,6.4621f, 
	24.4682f,33.8616f,2.7956f, 24.7641f,45.6533f,-0.9635f, 24.4682f,45.6533f,-4.7226f, 23.5880f,45.6533f,-8.3891f, 22.1450f,45.6533f,-11.8728f, 
	20.1748f,45.6533f,-15.0878f, 17.7259f,45.6533f,-17.9551f, 14.8587f,45.6533f,-20.4040f, 11.6436f,45.6533f,-22.3742f, 8.1599f,45.6533f,-23.8171f, 
	4.4934f,45.6533f,-24.6974f, 0.7343f,45.6533f,-24.9932f, 0.7343f,45.6533f,-24.9932f, -3.0248f,45.6533f,-24.6974f, -6.6913f,45.6533f,-23.8171f, 
	-10.1749f,45.6533f,-22.3742f, -13.3900f,45.6533f,-20.4040f, -16.2573f,45.6533f,-17.9551f, -18.7061f,45.6533f,-15.0878f, -20.6763f,45.6533f,-11.8728f, 
	-22.1193f,45.6533f,-8.3891f, -22.9996f,45.6533f,-4.7226f, -23.2954f,45.6533f,-0.9635f, -22.9996f,45.6533f,2.7956f, -22.1193f,45.6533f,6.4621f, 
	-20.6763f,45.6533f,9.9458f, -18.7061f,45.6533f,13.1608f, -16.2573f,45.6533f,16.0281f, -13.3900f,45.6533f,18.4770f, -10.1749f,45.6533f,20.4471f, 
	-6.6913f,45.6533f,21.8901f, -3.0247f,45.6533f,22.7704f, 0.7343f,45.6533f,23.0662f, 4.4934f,45.6533f,22.7704f, 8.1599f,45.6533f,21.8901f, 
	11.6436f,45.6533f,20.4471f, 14.8587f,45.6533f,18.4770f, 17.7259f,45.6533f,16.0281f, 20.1748f,45.6533f,13.1608f, 22.1450f,45.6533f,9.9458f, 
	23.5880f,45.6533f,6.4621f, 24.4682f,45.6533f,2.7956f, 
_NULL_ }; /* End of glass_Vertices */

const float glass_UV[] = {
	1.5000f,0.1670f,1.5500f,0.1670f,1.5000f,0.0005f,1.5500f,0.0005f,1.6000f,0.1670f,1.6000f,0.0005f,1.6500f,0.1670f,1.6500f,0.0005f,1.7000f,0.1670f,1.7000f,0.0005f,1.7500f,0.1670f,1.7500f,0.0005f,1.8000f,0.1670f,1.8000f,0.0005f,1.8500f,0.1670f,
	1.8500f,0.0005f,1.9000f,0.1670f,1.9000f,0.0005f,1.9500f,0.1670f,1.9500f,0.0005f,2.0000f,0.1670f,2.0000f,0.0005f,-0.0000f,0.1670f,0.0500f,0.1670f,-0.0000f,0.0005f,0.0500f,0.0005f,0.1000f,0.1670f,0.1000f,0.0005f,0.1500f,0.1670f,0.1500f,0.0005f,
	0.2000f,0.1670f,0.2000f,0.0005f,0.2500f,0.1670f,0.2500f,0.0005f,0.3000f,0.1670f,0.3000f,0.0005f,0.3500f,0.1670f,0.3500f,0.0005f,0.4000f,0.1670f,0.4000f,0.0005f,0.4500f,0.1670f,0.4500f,0.0005f,0.5000f,0.1670f,0.5000f,0.0005f,0.5500f,0.1670f,
	0.5500f,0.0005f,0.6000f,0.1670f,0.6000f,0.0005f,0.6500f,0.1670f,0.6500f,0.0005f,0.7000f,0.1670f,0.7000f,0.0005f,0.7500f,0.1670f,0.7500f,0.0005f,0.8000f,0.1670f,0.8000f,0.0005f,0.8500f,0.1670f,0.8500f,0.0005f,0.9000f,0.1670f,0.9000f,0.0005f,
	0.9500f,0.1670f,0.9500f,0.0005f,1.0000f,0.1670f,1.0000f,0.0005f,1.0500f,0.1670f,1.0500f,0.0005f,1.1000f,0.1670f,1.1000f,0.0005f,1.1500f,0.1670f,1.1500f,0.0005f,1.2000f,0.1670f,1.2000f,0.0005f,1.2500f,0.1670f,1.2500f,0.0005f,1.3000f,0.1670f,
	1.3000f,0.0005f,1.3500f,0.1670f,1.3500f,0.0005f,1.4000f,0.1670f,1.4000f,0.0005f,1.4500f,0.1670f,1.4500f,0.0005f,1.5000f,0.3335f,1.5500f,0.3335f,1.6000f,0.3335f,1.6500f,0.3335f,1.7000f,0.3335f,1.7500f,0.3335f,1.8000f,0.3335f,1.8500f,0.3335f,
	1.9000f,0.3335f,1.9500f,0.3335f,2.0000f,0.3335f,-0.0000f,0.3335f,0.0500f,0.3335f,0.1000f,0.3335f,0.1500f,0.3335f,0.2000f,0.3335f,0.2500f,0.3335f,0.3000f,0.3335f,0.3500f,0.3335f,0.4000f,0.3335f,0.4500f,0.3335f,0.5000f,0.3335f,0.5500f,0.3335f,
	0.6000f,0.3335f,0.6500f,0.3335f,0.7000f,0.3335f,0.7500f,0.3335f,0.8000f,0.3335f,0.8500f,0.3335f,0.9000f,0.3335f,0.9500f,0.3335f,1.0000f,0.3335f,1.0500f,0.3335f,1.1000f,0.3335f,1.1500f,0.3335f,1.2000f,0.3335f,1.2500f,0.3335f,1.3000f,0.3335f,
	1.3500f,0.3335f,1.4000f,0.3335f,1.4500f,0.3335f,1.5000f,0.5000f,1.5500f,0.5000f,1.6000f,0.5000f,1.6500f,0.5000f,1.7000f,0.5000f,1.7500f,0.5000f,1.8000f,0.5000f,1.8500f,0.5000f,1.9000f,0.5000f,1.9500f,0.5000f,2.0000f,0.5000f,-0.0000f,0.5000f,
	0.0500f,0.5000f,0.1000f,0.5000f,0.1500f,0.5000f,0.2000f,0.5000f,0.2500f,0.5000f,0.3000f,0.5000f,0.3500f,0.5000f,0.4000f,0.5000f,0.4500f,0.5000f,0.5000f,0.5000f,0.5500f,0.5000f,0.6000f,0.5000f,0.6500f,0.5000f,0.7000f,0.5000f,0.7500f,0.5000f,
	0.8000f,0.5000f,0.8500f,0.5000f,0.9000f,0.5000f,0.9500f,0.5000f,1.0000f,0.5000f,1.0500f,0.5000f,1.1000f,0.5000f,1.1500f,0.5000f,1.2000f,0.5000f,1.2500f,0.5000f,1.3000f,0.5000f,1.3500f,0.5000f,1.4000f,0.5000f,1.4500f,0.5000f,1.5000f,0.6665f,
	1.5500f,0.6665f,1.6000f,0.6665f,1.6500f,0.6665f,1.7000f,0.6665f,1.7500f,0.6665f,1.8000f,0.6665f,1.8500f,0.6665f,1.9000f,0.6665f,1.9500f,0.6665f,2.0000f,0.6665f,-0.0000f,0.6665f,0.0500f,0.6665f,0.1000f,0.6665f,0.1500f,0.6665f,0.2000f,0.6665f,
	0.2500f,0.6665f,0.3000f,0.6665f,0.3500f,0.6665f,0.4000f,0.6665f,0.4500f,0.6665f,0.5000f,0.6665f,0.5500f,0.6665f,0.6000f,0.6665f,0.6500f,0.6665f,0.7000f,0.6665f,0.7500f,0.6665f,0.8000f,0.6665f,0.8500f,0.6665f,0.9000f,0.6665f,0.9500f,0.6665f,
	1.0000f,0.6665f,1.0500f,0.6665f,1.1000f,0.6665f,1.1500f,0.6665f,1.2000f,0.6665f,1.2500f,0.6665f,1.3000f,0.6665f,1.3500f,0.6665f,1.4000f,0.6665f,1.4500f,0.6665f,1.5000f,0.8330f,1.5500f,0.8330f,1.6000f,0.8330f,1.6500f,0.8330f,1.7000f,0.8330f,
	1.7500f,0.8330f,1.8000f,0.8330f,1.8500f,0.8330f,1.9000f,0.8330f,1.9500f,0.8330f,2.0000f,0.8330f,-0.0000f,0.8330f,0.0500f,0.8330f,0.1000f,0.8330f,0.1500f,0.8330f,0.2000f,0.8330f,0.2500f,0.8330f,0.3000f,0.8330f,0.3500f,0.8330f,0.4000f,0.8330f,
	0.4500f,0.8330f,0.5000f,0.8330f,0.5500f,0.8330f,0.6000f,0.8330f,0.6500f,0.8330f,0.7000f,0.8330f,0.7500f,0.8330f,0.8000f,0.8330f,0.8500f,0.8330f,0.9000f,0.8330f,0.9500f,0.8330f,1.0000f,0.8330f,1.0500f,0.8330f,1.1000f,0.8330f,1.1500f,0.8330f,
	1.2000f,0.8330f,1.2500f,0.8330f,1.3000f,0.8330f,1.3500f,0.8330f,1.4000f,0.8330f,1.4500f,0.8330f,1.5000f,0.9995f,1.5500f,0.9995f,1.6000f,0.9995f,1.6500f,0.9995f,1.7000f,0.9995f,1.7500f,0.9995f,1.8000f,0.9995f,1.8500f,0.9995f,1.9000f,0.9995f,
	1.9500f,0.9995f,2.0000f,0.9995f,-0.0000f,0.9995f,0.0500f,0.9995f,0.1000f,0.9995f,0.1500f,0.9995f,0.2000f,0.9995f,0.2500f,0.9995f,0.3000f,0.9995f,0.3500f,0.9995f,0.4000f,0.9995f,0.4500f,0.9995f,0.5000f,0.9995f,0.5500f,0.9995f,0.6000f,0.9995f,
	0.6500f,0.9995f,0.7000f,0.9995f,0.7500f,0.9995f,0.8000f,0.9995f,0.8500f,0.9995f,0.9000f,0.9995f,0.9500f,0.9995f,1.0000f,0.9995f,1.0500f,0.9995f,1.1000f,0.9995f,1.1500f,0.9995f,1.2000f,0.9995f,1.2500f,0.9995f,1.3000f,0.9995f,1.3500f,0.9995f,
	1.4000f,0.9995f,1.4500f,0.9995f,
_NULL_ }; /* End of glass_UV */

const float glass_Normals[] = {
	1.0000f,0.0000f,0.0000f, 0.9877f,0.0000f,-0.1564f, 1.0000f,0.0000f,0.0000f, 0.9877f,0.0000f,-0.1564f, 0.9511f,0.0000f,-0.3090f, 0.9511f,0.0000f,-0.3090f, 0.8910f,0.0000f,-0.4540f, 0.8910f,0.0000f,-0.4540f, 0.8090f,0.0000f,-0.5878f, 0.8090f,0.0000f,-0.5878f, 0.7071f,0.0000f,-0.7071f, 0.7071f,0.0000f,-0.7071f, 0.5878f,0.0000f,-0.8090f, 0.5878f,0.0000f,-0.8090f, 0.4540f,0.0000f,-0.8910f, 
	0.4540f,0.0000f,-0.8910f, 0.3090f,0.0000f,-0.9511f, 0.3090f,0.0000f,-0.9511f, 0.1564f,0.0000f,-0.9877f, 0.1564f,0.0000f,-0.9877f, 0.0000f,0.0000f,-1.0000f, 0.0000f,0.0000f,-1.0000f, 0.0000f,0.0000f,-1.0000f, -0.1564f,0.0000f,-0.9877f, 0.0000f,0.0000f,-1.0000f, -0.1564f,0.0000f,-0.9877f, -0.3090f,0.0000f,-0.9511f, -0.3090f,0.0000f,-0.9511f, -0.4540f,0.0000f,-0.8910f, -0.4540f,0.0000f,-0.8910f, 
	-0.5878f,0.0000f,-0.8090f, -0.5878f,0.0000f,-0.8090f, -0.7071f,0.0000f,-0.7071f, -0.7071f,0.0000f,-0.7071f, -0.8090f,0.0000f,-0.5878f, -0.8090f,0.0000f,-0.5878f, -0.8910f,0.0000f,-0.4540f, -0.8910f,0.0000f,-0.4540f, -0.9511f,0.0000f,-0.3090f, -0.9511f,0.0000f,-0.3090f, -0.9877f,0.0000f,-0.1564f, -0.9877f,0.0000f,-0.1564f, -1.0000f,0.0000f,0.0000f, -1.0000f,0.0000f,0.0000f, -0.9877f,0.0000f,0.1564f, 
	-0.9877f,0.0000f,0.1564f, -0.9511f,0.0000f,0.3090f, -0.9511f,0.0000f,0.3090f, -0.8910f,0.0000f,0.4540f, -0.8910f,0.0000f,0.4540f, -0.8090f,0.0000f,0.5878f, -0.8090f,0.0000f,0.5878f, -0.7071f,0.0000f,0.7071f, -0.7071f,0.0000f,0.7071f, -0.5878f,0.0000f,0.8090f, -0.5878f,0.0000f,0.8090f, -0.4540f,0.0000f,0.8910f, -0.4540f,0.0000f,0.8910f, -0.3090f,0.0000f,0.9511f, -0.3090f,0.0000f,0.9511f, 
	-0.1564f,0.0000f,0.9877f, -0.1564f,0.0000f,0.9877f, -0.0000f,0.0000f,1.0000f, -0.0000f,0.0000f,1.0000f, 0.1564f,0.0000f,0.9877f, 0.1564f,0.0000f,0.9877f, 0.3090f,0.0000f,0.9511f, 0.3090f,0.0000f,0.9511f, 0.4540f,0.0000f,0.8910f, 0.4540f,0.0000f,0.8910f, 0.5878f,0.0000f,0.8090f, 0.5878f,0.0000f,0.8090f, 0.7071f,0.0000f,0.7071f, 0.7071f,0.0000f,0.7071f, 0.8090f,0.0000f,0.5878f, 
	0.8090f,0.0000f,0.5878f, 0.8910f,0.0000f,0.4540f, 0.8910f,0.0000f,0.4540f, 0.9511f,0.0000f,0.3090f, 0.9511f,0.0000f,0.3090f, 0.9877f,0.0000f,0.1564f, 0.9877f,0.0000f,0.1564f, 1.0000f,0.0000f,0.0000f, 0.9877f,0.0000f,-0.1564f, 0.9511f,0.0000f,-0.3090f, 0.8910f,0.0000f,-0.4540f, 0.8090f,0.0000f,-0.5878f, 0.7071f,0.0000f,-0.7071f, 0.5878f,0.0000f,-0.8090f, 0.4540f,0.0000f,-0.8910f, 
	0.3090f,0.0000f,-0.9511f, 0.1564f,0.0000f,-0.9877f, 0.0000f,0.0000f,-1.0000f, 0.0000f,0.0000f,-1.0000f, -0.1564f,0.0000f,-0.9877f, -0.3090f,0.0000f,-0.9511f, -0.4540f,0.0000f,-0.8910f, -0.5878f,0.0000f,-0.8090f, -0.7071f,0.0000f,-0.7071f, -0.8090f,0.0000f,-0.5878f, -0.8910f,0.0000f,-0.4540f, -0.9511f,0.0000f,-0.3090f, -0.9877f,0.0000f,-0.1564f, -1.0000f,0.0000f,0.0000f, -0.9877f,0.0000f,0.1564f, 
	-0.9511f,0.0000f,0.3090f, -0.8910f,0.0000f,0.4540f, -0.8090f,0.0000f,0.5878f, -0.7071f,0.0000f,0.7071f, -0.5878f,0.0000f,0.8090f, -0.4540f,0.0000f,0.8910f, -0.3090f,0.0000f,0.9511f, -0.1564f,0.0000f,0.9877f, -0.0000f,0.0000f,1.0000f, 0.1564f,0.0000f,0.9877f, 0.3090f,0.0000f,0.9511f, 0.4540f,0.0000f,0.8910f, 0.5878f,0.0000f,0.8090f, 0.7071f,0.0000f,0.7071f, 0.8090f,0.0000f,0.5878f, 
	0.8910f,0.0000f,0.4540f, 0.9511f,0.0000f,0.3090f, 0.9877f,0.0000f,0.1564f, 1.0000f,0.0000f,0.0000f, 0.9877f,0.0000f,-0.1564f, 0.9511f,0.0000f,-0.3090f, 0.8910f,0.0000f,-0.4540f, 0.8090f,0.0000f,-0.5878f, 0.7071f,0.0000f,-0.7071f, 0.5878f,0.0000f,-0.8090f, 0.4540f,0.0000f,-0.8910f, 0.3090f,0.0000f,-0.9511f, 0.1564f,0.0000f,-0.9877f, 0.0000f,0.0000f,-1.0000f, 0.0000f,0.0000f,-1.0000f, 
	-0.1564f,0.0000f,-0.9877f, -0.3090f,0.0000f,-0.9511f, -0.4540f,0.0000f,-0.8910f, -0.5878f,0.0000f,-0.8090f, -0.7071f,0.0000f,-0.7071f, -0.8090f,0.0000f,-0.5878f, -0.8910f,0.0000f,-0.4540f, -0.9511f,0.0000f,-0.3090f, -0.9877f,0.0000f,-0.1564f, -1.0000f,0.0000f,0.0000f, -0.9877f,0.0000f,0.1564f, -0.9511f,0.0000f,0.3090f, -0.8910f,0.0000f,0.4540f, -0.8090f,0.0000f,0.5878f, -0.7071f,0.0000f,0.7071f, 
	-0.5878f,0.0000f,0.8090f, -0.4540f,0.0000f,0.8910f, -0.3090f,0.0000f,0.9511f, -0.1564f,0.0000f,0.9877f, -0.0000f,0.0000f,1.0000f, 0.1564f,0.0000f,0.9877f, 0.3090f,0.0000f,0.9511f, 0.4540f,0.0000f,0.8910f, 0.5878f,0.0000f,0.8090f, 0.7071f,0.0000f,0.7071f, 0.8090f,0.0000f,0.5878f, 0.8910f,0.0000f,0.4540f, 0.9511f,0.0000f,0.3090f, 0.9877f,0.0000f,0.1564f, 1.0000f,0.0000f,0.0000f, 
	0.9877f,0.0000f,-0.1564f, 0.9511f,0.0000f,-0.3090f, 0.8910f,0.0000f,-0.4540f, 0.8090f,0.0000f,-0.5878f, 0.7071f,0.0000f,-0.7071f, 0.5878f,0.0000f,-0.8090f, 0.4540f,0.0000f,-0.8910f, 0.3090f,0.0000f,-0.9511f, 0.1564f,0.0000f,-0.9877f, 0.0000f,0.0000f,-1.0000f, 0.0000f,0.0000f,-1.0000f, -0.1564f,0.0000f,-0.9877f, -0.3090f,0.0000f,-0.9511f, -0.4540f,0.0000f,-0.8910f, -0.5878f,0.0000f,-0.8090f, 
	-0.7071f,0.0000f,-0.7071f, -0.8090f,0.0000f,-0.5878f, -0.8910f,0.0000f,-0.4540f, -0.9511f,0.0000f,-0.3090f, -0.9877f,0.0000f,-0.1564f, -1.0000f,0.0000f,0.0000f, -0.9877f,0.0000f,0.1564f, -0.9511f,0.0000f,0.3090f, -0.8910f,0.0000f,0.4540f, -0.8090f,0.0000f,0.5878f, -0.7071f,0.0000f,0.7071f, -0.5878f,0.0000f,0.8090f, -0.4540f,0.0000f,0.8910f, -0.3090f,0.0000f,0.9511f, -0.1564f,0.0000f,0.9877f, 
	-0.0000f,0.0000f,1.0000f, 0.1564f,0.0000f,0.9877f, 0.3090f,0.0000f,0.9511f, 0.4540f,0.0000f,0.8910f, 0.5878f,0.0000f,0.8090f, 0.7071f,0.0000f,0.7071f, 0.8090f,0.0000f,0.5878f, 0.8910f,0.0000f,0.4540f, 0.9511f,0.0000f,0.3090f, 0.9877f,0.0000f,0.1564f, 1.0000f,0.0000f,0.0000f, 0.9877f,0.0000f,-0.1564f, 0.9511f,0.0000f,-0.3090f, 0.8910f,0.0000f,-0.4540f, 0.8090f,0.0000f,-0.5878f, 
	0.7071f,0.0000f,-0.7071f, 0.5878f,0.0000f,-0.8090f, 0.4540f,0.0000f,-0.8910f, 0.3090f,0.0000f,-0.9511f, 0.1564f,0.0000f,-0.9877f, 0.0000f,0.0000f,-1.0000f, 0.0000f,0.0000f,-1.0000f, -0.1564f,0.0000f,-0.9877f, -0.3090f,0.0000f,-0.9511f, -0.4540f,0.0000f,-0.8910f, -0.5878f,0.0000f,-0.8090f, -0.7071f,0.0000f,-0.7071f, -0.8090f,0.0000f,-0.5878f, -0.8910f,0.0000f,-0.4540f, -0.9511f,0.0000f,-0.3090f, 
	-0.9877f,0.0000f,-0.1564f, -1.0000f,0.0000f,0.0000f, -0.9877f,0.0000f,0.1564f, -0.9511f,0.0000f,0.3090f, -0.8910f,0.0000f,0.4540f, -0.8090f,0.0000f,0.5878f, -0.7071f,0.0000f,0.7071f, -0.5878f,0.0000f,0.8090f, -0.4540f,0.0000f,0.8910f, -0.3090f,0.0000f,0.9511f, -0.1564f,0.0000f,0.9877f, -0.0000f,0.0000f,1.0000f, 0.1564f,0.0000f,0.9877f, 0.3090f,0.0000f,0.9511f, 0.4540f,0.0000f,0.8910f, 
	0.5878f,0.0000f,0.8090f, 0.7071f,0.0000f,0.7071f, 0.8090f,0.0000f,0.5878f, 0.8910f,0.0000f,0.4540f, 0.9511f,0.0000f,0.3090f, 0.9877f,0.0000f,0.1564f, 1.0000f,0.0000f,0.0000f, 0.9877f,0.0000f,-0.1564f, 0.9511f,0.0000f,-0.3090f, 0.8910f,0.0000f,-0.4540f, 0.8090f,0.0000f,-0.5878f, 0.7071f,0.0000f,-0.7071f, 0.5878f,0.0000f,-0.8090f, 0.4540f,0.0000f,-0.8910f, 0.3090f,0.0000f,-0.9511f, 
	0.1564f,0.0000f,-0.9877f, 0.0000f,0.0000f,-1.0000f, 0.0000f,0.0000f,-1.0000f, -0.1564f,0.0000f,-0.9877f, -0.3090f,0.0000f,-0.9511f, -0.4540f,0.0000f,-0.8910f, -0.5878f,0.0000f,-0.8090f, -0.7071f,0.0000f,-0.7071f, -0.8090f,0.0000f,-0.5878f, -0.8910f,0.0000f,-0.4540f, -0.9511f,0.0000f,-0.3090f, -0.9877f,0.0000f,-0.1564f, -1.0000f,0.0000f,0.0000f, -0.9877f,0.0000f,0.1564f, -0.9511f,0.0000f,0.3090f, 
	-0.8910f,0.0000f,0.4540f, -0.8090f,0.0000f,0.5878f, -0.7071f,0.0000f,0.7071f, -0.5878f,0.0000f,0.8090f, -0.4540f,0.0000f,0.8910f, -0.3090f,0.0000f,0.9511f, -0.1564f,0.0000f,0.9877f, -0.0000f,0.0000f,1.0000f, 0.1564f,0.0000f,0.9877f, 0.3090f,0.0000f,0.9511f, 0.4540f,0.0000f,0.8910f, 0.5878f,0.0000f,0.8090f, 0.7071f,0.0000f,0.7071f, 0.8090f,0.0000f,0.5878f, 0.8910f,0.0000f,0.4540f, 
	0.9511f,0.0000f,0.3090f, 0.9877f,0.0000f,0.1564f, 
_NULL_ }; /* End of glass_Normals */

const unsigned short glass_Faces[] = {
	0, 2, 1, 1, 2, 3, 1, 3, 4, 4, 3, 5, 4, 5, 6, 6, 5, 7, 6, 7, 8, 8, 7, 9, 8, 9, 10, 10, 9, 11, 
	10, 11, 12, 12, 11, 13, 12, 13, 14, 14, 13, 15, 14, 15, 16, 16, 15, 17, 16, 17, 18, 18, 17, 19, 18, 19, 20, 20, 19, 21, 
	22, 24, 23, 23, 24, 25, 23, 25, 26, 26, 25, 27, 26, 27, 28, 28, 27, 29, 28, 29, 30, 30, 29, 31, 30, 31, 32, 32, 31, 33, 
	32, 33, 34, 34, 33, 35, 34, 35, 36, 36, 35, 37, 36, 37, 38, 38, 37, 39, 38, 39, 40, 40, 39, 41, 40, 41, 42, 42, 41, 43, 
	42, 43, 44, 44, 43, 45, 44, 45, 46, 46, 45, 47, 46, 47, 48, 48, 47, 49, 48, 49, 50, 50, 49, 51, 50, 51, 52, 52, 51, 53, 
	52, 53, 54, 54, 53, 55, 54, 55, 56, 56, 55, 57, 56, 57, 58, 58, 57, 59, 58, 59, 60, 60, 59, 61, 60, 61, 62, 62, 61, 63, 
	62, 63, 64, 64, 63, 65, 64, 65, 66, 66, 65, 67, 66, 67, 68, 68, 67, 69, 68, 69, 70, 70, 69, 71, 70, 71, 72, 72, 71, 73, 
	72, 73, 74, 74, 73, 75, 74, 75, 76, 76, 75, 77, 76, 77, 78, 78, 77, 79, 78, 79, 80, 80, 79, 81, 80, 81, 0, 0, 81, 2, 
	82, 0, 83, 83, 0, 1, 83, 1, 84, 84, 1, 4, 84, 4, 85, 85, 4, 6, 85, 6, 86, 86, 6, 8, 86, 8, 87, 87, 8, 10, 
	87, 10, 88, 88, 10, 12, 88, 12, 89, 89, 12, 14, 89, 14, 90, 90, 14, 16, 90, 16, 91, 91, 16, 18, 91, 18, 92, 92, 18, 20, 
	93, 22, 94, 94, 22, 23, 94, 23, 95, 95, 23, 26, 95, 26, 96, 96, 26, 28, 96, 28, 97, 97, 28, 30, 97, 30, 98, 98, 30, 32, 
	98, 32, 99, 99, 32, 34, 99, 34, 100, 100, 34, 36, 100, 36, 101, 101, 36, 38, 101, 38, 102, 102, 38, 40, 102, 40, 103, 103, 40, 42, 
	103, 42, 104, 104, 42, 44, 104, 44, 105, 105, 44, 46, 105, 46, 106, 106, 46, 48, 106, 48, 107, 107, 48, 50, 107, 50, 108, 108, 50, 52, 
	108, 52, 109, 109, 52, 54, 109, 54, 110, 110, 54, 56, 110, 56, 111, 111, 56, 58, 111, 58, 112, 112, 58, 60, 112, 60, 113, 113, 60, 62, 
	113, 62, 114, 114, 62, 64, 114, 64, 115, 115, 64, 66, 115, 66, 116, 116, 66, 68, 116, 68, 117, 117, 68, 70, 117, 70, 118, 118, 70, 72, 
	118, 72, 119, 119, 72, 74, 119, 74, 120, 120, 74, 76, 120, 76, 121, 121, 76, 78, 121, 78, 122, 122, 78, 80, 122, 80, 82, 82, 80, 0, 
	123, 82, 124, 124, 82, 83, 124, 83, 125, 125, 83, 84, 125, 84, 126, 126, 84, 85, 126, 85, 127, 127, 85, 86, 127, 86, 128, 128, 86, 87, 
	128, 87, 129, 129, 87, 88, 129, 88, 130, 130, 88, 89, 130, 89, 131, 131, 89, 90, 131, 90, 132, 132, 90, 91, 132, 91, 133, 133, 91, 92, 
	134, 93, 135, 135, 93, 94, 135, 94, 136, 136, 94, 95, 136, 95, 137, 137, 95, 96, 137, 96, 138, 138, 96, 97, 138, 97, 139, 139, 97, 98, 
	139, 98, 140, 140, 98, 99, 140, 99, 141, 141, 99, 100, 141, 100, 142, 142, 100, 101, 142, 101, 143, 143, 101, 102, 143, 102, 144, 144, 102, 103, 
	144, 103, 145, 145, 103, 104, 145, 104, 146, 146, 104, 105, 146, 105, 147, 147, 105, 106, 147, 106, 148, 148, 106, 107, 148, 107, 149, 149, 107, 108, 
	149, 108, 150, 150, 108, 109, 150, 109, 151, 151, 109, 110, 151, 110, 152, 152, 110, 111, 152, 111, 153, 153, 111, 112, 153, 112, 154, 154, 112, 113, 
	154, 113, 155, 155, 113, 114, 155, 114, 156, 156, 114, 115, 156, 115, 157, 157, 115, 116, 157, 116, 158, 158, 116, 117, 158, 117, 159, 159, 117, 118, 
	159, 118, 160, 160, 118, 119, 160, 119, 161, 161, 119, 120, 161, 120, 162, 162, 120, 121, 162, 121, 163, 163, 121, 122, 163, 122, 123, 123, 122, 82, 
	164, 123, 165, 165, 123, 124, 165, 124, 166, 166, 124, 125, 166, 125, 167, 167, 125, 126, 167, 126, 168, 168, 126, 127, 168, 127, 169, 169, 127, 128, 
	169, 128, 170, 170, 128, 129, 170, 129, 171, 171, 129, 130, 171, 130, 172, 172, 130, 131, 172, 131, 173, 173, 131, 132, 173, 132, 174, 174, 132, 133, 
	175, 134, 176, 176, 134, 135, 176, 135, 177, 177, 135, 136, 177, 136, 178, 178, 136, 137, 178, 137, 179, 179, 137, 138, 179, 138, 180, 180, 138, 139, 
	180, 139, 181, 181, 139, 140, 181, 140, 182, 182, 140, 141, 182, 141, 183, 183, 141, 142, 183, 142, 184, 184, 142, 143, 184, 143, 185, 185, 143, 144, 
	185, 144, 186, 186, 144, 145, 186, 145, 187, 187, 145, 146, 187, 146, 188, 188, 146, 147, 188, 147, 189, 189, 147, 148, 189, 148, 190, 190, 148, 149, 
	190, 149, 191, 191, 149, 150, 191, 150, 192, 192, 150, 151, 192, 151, 193, 193, 151, 152, 193, 152, 194, 194, 152, 153, 194, 153, 195, 195, 153, 154, 
	195, 154, 196, 196, 154, 155, 196, 155, 197, 197, 155, 156, 197, 156, 198, 198, 156, 157, 198, 157, 199, 199, 157, 158, 199, 158, 200, 200, 158, 159, 
	200, 159, 201, 201, 159, 160, 201, 160, 202, 202, 160, 161, 202, 161, 203, 203, 161, 162, 203, 162, 204, 204, 162, 163, 204, 163, 164, 164, 163, 123, 
	205, 164, 206, 206, 164, 165, 206, 165, 207, 207, 165, 166, 207, 166, 208, 208, 166, 167, 208, 167, 209, 209, 167, 168, 209, 168, 210, 210, 168, 169, 
	210, 169, 211, 211, 169, 170, 211, 170, 212, 212, 170, 171, 212, 171, 213, 213, 171, 172, 213, 172, 214, 214, 172, 173, 214, 173, 215, 215, 173, 174, 
	216, 175, 217, 217, 175, 176, 217, 176, 218, 218, 176, 177, 218, 177, 219, 219, 177, 178, 219, 178, 220, 220, 178, 179, 220, 179, 221, 221, 179, 180, 
	221, 180, 222, 222, 180, 181, 222, 181, 223, 223, 181, 182, 223, 182, 224, 224, 182, 183, 224, 183, 225, 225, 183, 184, 225, 184, 226, 226, 184, 185, 
	226, 185, 227, 227, 185, 186, 227, 186, 228, 228, 186, 187, 228, 187, 229, 229, 187, 188, 229, 188, 230, 230, 188, 189, 230, 189, 231, 231, 189, 190, 
	231, 190, 232, 232, 190, 191, 232, 191, 233, 233, 191, 192, 233, 192, 234, 234, 192, 193, 234, 193, 235, 235, 193, 194, 235, 194, 236, 236, 194, 195, 
	236, 195, 237, 237, 195, 196, 237, 196, 238, 238, 196, 197, 238, 197, 239, 239, 197, 198, 239, 198, 240, 240, 198, 199, 240, 199, 241, 241, 199, 200, 
	241, 200, 242, 242, 200, 201, 242, 201, 243, 243, 201, 202, 243, 202, 244, 244, 202, 203, 244, 203, 245, 245, 203, 204, 245, 204, 205, 205, 204, 164, 
	246, 205, 247, 247, 205, 206, 247, 206, 248, 248, 206, 207, 248, 207, 249, 249, 207, 208, 249, 208, 250, 250, 208, 209, 250, 209, 251, 251, 209, 210, 
	251, 210, 252, 252, 210, 211, 252, 211, 253, 253, 211, 212, 253, 212, 254, 254, 212, 213, 254, 213, 255, 255, 213, 214, 255, 214, 256, 256, 214, 215, 
	257, 216, 258, 258, 216, 217, 258, 217, 259, 259, 217, 218, 259, 218, 260, 260, 218, 219, 260, 219, 261, 261, 219, 220, 261, 220, 262, 262, 220, 221, 
	262, 221, 263, 263, 221, 222, 263, 222, 264, 264, 222, 223, 264, 223, 265, 265, 223, 224, 265, 224, 266, 266, 224, 225, 266, 225, 267, 267, 225, 226, 
	267, 226, 268, 268, 226, 227, 268, 227, 269, 269, 227, 228, 269, 228, 270, 270, 228, 229, 270, 229, 271, 271, 229, 230, 271, 230, 272, 272, 230, 231, 
	272, 231, 273, 273, 231, 232, 273, 232, 274, 274, 232, 233, 274, 233, 275, 275, 233, 234, 275, 234, 276, 276, 234, 235, 276, 235, 277, 277, 235, 236, 
	277, 236, 278, 278, 236, 237, 278, 237, 279, 279, 237, 238, 279, 238, 280, 280, 238, 239, 280, 239, 281, 281, 239, 240, 281, 240, 282, 282, 240, 241, 
	282, 241, 283, 283, 241, 242, 283, 242, 284, 284, 242, 243, 284, 243, 285, 285, 243, 244, 285, 244, 286, 286, 244, 245, 286, 245, 246, 246, 245, 205, 
	
_NULL_ }; /* End of glass_Faces */


/* MESH 1 */

#define silver_NumVertex  1335
#define silver_NumFaces   2352
#define silver_NumStrips  0
#define silver_Flags      0x00020006
#define silver_Material   1

const float silver_Vertices[] = {
	8.7126f,-36.8364f,-37.7996f, 8.5380f,-33.7031f,-36.9784f, 0.7625f,-36.8364f,-38.6352f, 0.7625f,-33.7031f,-37.7956f, 8.0611f,-31.4094f,-34.7348f, 
	0.7625f,-31.4094f,-35.5019f, 7.4097f,-30.5698f,-31.6700f, 0.7625f,-30.5698f,-32.3686f, 6.7582f,-31.4094f,-28.6052f, 0.7625f,-31.4094f,-29.2353f, 
	6.2813f,-33.7031f,-26.3616f, 0.7625f,-33.7031f,-26.9416f, 6.1068f,-36.8364f,-25.5403f, 0.7625f,-36.8364f,-26.1020f, 6.2813f,-39.9697f,-26.3616f, 
	0.7625f,-39.9697f,-26.9416f, 6.7582f,-42.2634f,-28.6052f, 0.7625f,-42.2634f,-29.2353f, 7.4097f,-43.1030f,-31.6700f, 0.7625f,-43.1030f,-32.3686f, 
	8.0611f,-42.2634f,-34.7348f, 0.7625f,-42.2634f,-35.5019f, 8.5380f,-39.9697f,-36.9784f, 0.7625f,-39.9697f,-37.7956f, 8.7126f,-36.8364f,-37.7996f, 
	0.7625f,-36.8364f,-38.6352f, 16.3151f,-36.8364f,-35.3294f, 15.9736f,-33.7031f,-34.5624f, 15.0407f,-31.4094f,-32.4670f, 13.7663f,-30.5698f,-29.6046f, 
	12.4919f,-31.4094f,-26.7422f, 11.5589f,-33.7031f,-24.6468f, 11.2174f,-36.8364f,-23.8798f, 11.5589f,-39.9697f,-24.6468f, 12.4919f,-42.2634f,-26.7422f, 
	13.7663f,-43.1030f,-29.6046f, 15.0407f,-42.2634f,-32.4670f, 15.9736f,-39.9697f,-34.5624f, 16.3151f,-36.8364f,-35.3294f, 23.2380f,-36.8364f,-31.3325f, 
	22.7445f,-33.7031f,-30.6533f, 21.3963f,-31.4094f,-28.7976f, 19.5546f,-30.5698f,-26.2627f, 17.7129f,-31.4094f,-23.7278f, 16.3647f,-33.7031f,-21.8722f, 
	15.8712f,-36.8364f,-21.1929f, 16.3647f,-39.9697f,-21.8722f, 17.7129f,-42.2634f,-23.7278f, 19.5546f,-43.1030f,-26.2627f, 21.3963f,-42.2634f,-28.7976f, 
	22.7445f,-39.9697f,-30.6533f, 23.2380f,-36.8364f,-31.3325f, 29.1786f,-36.8364f,-25.9836f, 28.5546f,-33.7031f,-25.4218f, 26.8501f,-31.4094f,-23.8870f, 
	24.5216f,-30.5698f,-21.7904f, 22.1931f,-31.4094f,-19.6938f, 20.4885f,-33.7031f,-18.1590f, 19.8646f,-36.8364f,-17.5973f, 20.4885f,-39.9697f,-18.1590f, 
	22.1931f,-42.2634f,-19.6938f, 24.5216f,-43.1030f,-21.7904f, 26.8501f,-42.2634f,-23.8870f, 28.5546f,-39.9697f,-25.4218f, 29.1786f,-36.8364f,-25.9836f, 
	33.8772f,-36.8364f,-19.5164f, 33.1501f,-33.7031f,-19.0966f, 31.1637f,-31.4094f,-17.9498f, 28.4502f,-30.5698f,-16.3831f, 25.7367f,-31.4094f,-14.8165f, 
	23.7503f,-33.7031f,-13.6696f, 23.0232f,-36.8364f,-13.2499f, 23.7503f,-39.9697f,-13.6696f, 25.7367f,-42.2634f,-14.8165f, 28.4502f,-43.1030f,-16.3831f, 
	31.1637f,-42.2634f,-17.9498f, 33.1501f,-39.9697f,-19.0966f, 33.8772f,-36.8364f,-19.5164f, 37.1286f,-36.8364f,-12.2137f, 36.3301f,-33.7031f,-11.9543f, 
	34.1486f,-31.4094f,-11.2455f, 31.1687f,-30.5698f,-10.2772f, 28.1888f,-31.4094f,-9.3090f, 26.0073f,-33.7031f,-8.6002f, 25.2088f,-36.8364f,-8.3408f, 
	26.0073f,-39.9697f,-8.6002f, 28.1888f,-42.2634f,-9.3090f, 31.1687f,-43.1030f,-10.2772f, 34.1487f,-42.2634f,-11.2455f, 36.3301f,-39.9697f,-11.9543f, 
	37.1286f,-36.8364f,-12.2137f, 38.7906f,-36.8364f,-4.3946f, 37.9556f,-33.7031f,-4.3068f, 35.6745f,-31.4094f,-4.0671f, 32.5583f,-30.5698f,-3.7395f, 
	29.4422f,-31.4094f,-3.4120f, 27.1611f,-33.7031f,-3.1723f, 26.3261f,-36.8364f,-3.0845f, 27.1611f,-39.9697f,-3.1723f, 29.4422f,-42.2634f,-3.4120f, 
	32.5583f,-43.1030f,-3.7395f, 35.6745f,-42.2634f,-4.0671f, 37.9556f,-39.9697f,-4.3068f, 38.7906f,-36.8364f,-4.3946f, 38.7906f,-36.8364f,3.5993f, 
	37.9556f,-33.7031f,3.5115f, 35.6745f,-31.4094f,3.2717f, 32.5583f,-30.5698f,2.9442f, 29.4422f,-31.4094f,2.6167f, 27.1611f,-33.7031f,2.3769f, 
	26.3261f,-36.8364f,2.2892f, 27.1611f,-39.9697f,2.3769f, 29.4422f,-42.2634f,2.6167f, 32.5583f,-43.1030f,2.9442f, 35.6745f,-42.2634f,3.2717f, 
	37.9556f,-39.9697f,3.5115f, 38.7906f,-36.8364f,3.5993f, 37.1286f,-36.8364f,11.4184f, 36.3301f,-33.7031f,11.1590f, 34.1486f,-31.4094f,10.4502f, 
	31.1687f,-30.5698f,9.4819f, 28.1888f,-31.4094f,8.5137f, 26.0073f,-33.7031f,7.8049f, 25.2088f,-36.8364f,7.5454f, 26.0073f,-39.9697f,7.8049f, 
	28.1888f,-42.2634f,8.5137f, 31.1687f,-43.1030f,9.4819f, 34.1487f,-42.2634f,10.4502f, 36.3301f,-39.9697f,11.1590f, 37.1286f,-36.8364f,11.4184f, 
	33.8772f,-36.8364f,18.7211f, 33.1501f,-33.7031f,18.3013f, 31.1637f,-31.4094f,17.1545f, 28.4502f,-30.5698f,15.5878f, 25.7367f,-31.4094f,14.0212f, 
	23.7503f,-33.7031f,12.8743f, 23.0232f,-36.8364f,12.4545f, 23.7503f,-39.9697f,12.8743f, 25.7367f,-42.2634f,14.0212f, 28.4502f,-43.1030f,15.5878f, 
	31.1637f,-42.2634f,17.1545f, 33.1501f,-39.9697f,18.3013f, 33.8772f,-36.8364f,18.7211f, 29.1786f,-36.8364f,25.1883f, 28.5546f,-33.7031f,24.6265f, 
	26.8501f,-31.4094f,23.0917f, 24.5216f,-30.5698f,20.9951f, 22.1931f,-31.4094f,18.8985f, 20.4885f,-33.7031f,17.3637f, 19.8646f,-36.8364f,16.8019f, 
	20.4885f,-39.9697f,17.3637f, 22.1931f,-42.2634f,18.8985f, 24.5216f,-43.1030f,20.9951f, 26.8501f,-42.2634f,23.0917f, 28.5546f,-39.9697f,24.6265f, 
	29.1786f,-36.8364f,25.1883f, 23.2380f,-36.8364f,30.5372f, 22.7445f,-33.7031f,29.8579f, 21.3963f,-31.4094f,28.0023f, 19.5546f,-30.5698f,25.4674f, 
	17.7129f,-31.4094f,22.9325f, 16.3647f,-33.7031f,21.0768f, 15.8712f,-36.8364f,20.3976f, 16.3647f,-39.9697f,21.0768f, 17.7129f,-42.2634f,22.9325f, 
	19.5546f,-43.1030f,25.4674f, 21.3963f,-42.2634f,28.0023f, 22.7445f,-39.9697f,29.8579f, 23.2380f,-36.8364f,30.5372f, 16.3151f,-36.8364f,34.5341f, 
	15.9736f,-33.7031f,33.7671f, 15.0407f,-31.4094f,31.6717f, 13.7663f,-30.5698f,28.8093f, 12.4919f,-31.4094f,25.9469f, 11.5589f,-33.7031f,23.8514f, 
	11.2174f,-36.8364f,23.0845f, 11.5589f,-39.9697f,23.8514f, 12.4919f,-42.2634f,25.9469f, 13.7663f,-43.1030f,28.8093f, 15.0407f,-42.2634f,31.6717f, 
	15.9736f,-39.9697f,33.7671f, 16.3151f,-36.8364f,34.5341f, 8.7126f,-36.8364f,37.0043f, 8.5380f,-33.7031f,36.1831f, 8.0611f,-31.4094f,33.9395f, 
	7.4097f,-30.5698f,30.8747f, 6.7582f,-31.4094f,27.8098f, 6.2813f,-33.7031f,25.5662f, 6.1068f,-36.8364f,24.7450f, 6.2813f,-39.9697f,25.5662f, 
	6.7582f,-42.2634f,27.8098f, 7.4097f,-43.1030f,30.8747f, 8.0611f,-42.2634f,33.9395f, 8.5380f,-39.9697f,36.1831f, 8.7126f,-36.8364f,37.0043f, 
	0.7625f,-36.8364f,37.8399f, 0.7625f,-33.7031f,37.0003f, 0.7625f,-31.4094f,34.7066f, 0.7625f,-30.5698f,31.5733f, 0.7625f,-31.4094f,28.4400f, 
	0.7625f,-33.7031f,26.1463f, 0.7625f,-36.8364f,25.3067f, 0.7625f,-39.9697f,26.1463f, 0.7625f,-42.2634f,28.4400f, 0.7625f,-43.1030f,31.5733f, 
	0.7625f,-42.2634f,34.7066f, 0.7625f,-39.9697f,37.0003f, 0.7625f,-36.8364f,37.8399f, -7.1875f,-36.8364f,37.0043f, -7.0130f,-33.7031f,36.1831f, 
	-6.5361f,-31.4094f,33.9395f, -5.8846f,-30.5698f,30.8747f, -5.2332f,-31.4094f,27.8098f, -4.7563f,-33.7031f,25.5662f, -4.5817f,-36.8364f,24.7450f, 
	-4.7563f,-39.9697f,25.5662f, -5.2332f,-42.2634f,27.8098f, -5.8846f,-43.1030f,30.8747f, -6.5361f,-42.2634f,33.9395f, -7.0130f,-39.9697f,36.1831f, 
	-7.1875f,-36.8364f,37.0043f, -14.7901f,-36.8364f,34.5341f, -14.4486f,-33.7031f,33.7671f, -13.5157f,-31.4094f,31.6717f, -12.2412f,-30.5698f,28.8093f, 
	-10.9668f,-31.4094f,25.9469f, -10.0339f,-33.7031f,23.8514f, -9.6924f,-36.8364f,23.0845f, -10.0339f,-39.9697f,23.8514f, -10.9668f,-42.2634f,25.9469f, 
	-12.2412f,-43.1030f,28.8093f, -13.5157f,-42.2634f,31.6717f, -14.4486f,-39.9697f,33.7671f, -14.7901f,-36.8364f,34.5341f, -21.7129f,-36.8364f,30.5372f, 
	-21.2195f,-33.7031f,29.8579f, -19.8712f,-31.4094f,28.0023f, -18.0295f,-30.5698f,25.4674f, -16.1878f,-31.4094f,22.9325f, -14.8396f,-33.7031f,21.0768f, 
	-14.3461f,-36.8364f,20.3976f, -14.8396f,-39.9697f,21.0768f, -16.1878f,-42.2634f,22.9325f, -18.0295f,-43.1030f,25.4674f, -19.8712f,-42.2634f,28.0023f, 
	-21.2195f,-39.9697f,29.8579f, -21.7129f,-36.8364f,30.5372f, -27.6535f,-36.8364f,25.1883f, -27.0296f,-33.7031f,24.6265f, -25.3250f,-31.4094f,23.0917f, 
	-22.9965f,-30.5698f,20.9951f, -20.6680f,-31.4094f,18.8985f, -18.9635f,-33.7031f,17.3637f, -18.3396f,-36.8364f,16.8019f, -18.9635f,-39.9697f,17.3637f, 
	-20.6681f,-42.2634f,18.8985f, -22.9965f,-43.1030f,20.9951f, -25.3250f,-42.2634f,23.0917f, -27.0296f,-39.9697f,24.6265f, -27.6535f,-36.8364f,25.1883f, 
	-32.3522f,-36.8364f,18.7211f, -31.6251f,-33.7031f,18.3013f, -29.6387f,-31.4094f,17.1545f, -26.9251f,-30.5698f,15.5878f, -24.2116f,-31.4094f,14.0212f, 
	-22.2252f,-33.7031f,12.8743f, -21.4981f,-36.8364f,12.4545f, -22.2252f,-39.9697f,12.8743f, -24.2116f,-42.2634f,14.0212f, -26.9251f,-43.1030f,15.5878f, 
	-29.6387f,-42.2634f,17.1545f, -31.6251f,-39.9697f,18.3013f, -32.3522f,-36.8364f,18.7211f, -35.6035f,-36.8364f,11.4184f, -34.8051f,-33.7031f,11.1590f, 
	-32.6236f,-31.4094f,10.4501f, -29.6437f,-30.5698f,9.4819f, -26.6637f,-31.4094f,8.5137f, -24.4823f,-33.7031f,7.8049f, -23.6838f,-36.8364f,7.5454f, 
	-24.4823f,-39.9697f,7.8049f, -26.6637f,-42.2634f,8.5137f, -29.6437f,-43.1030f,9.4819f, -32.6236f,-42.2634f,10.4501f, -34.8051f,-39.9697f,11.1590f, 
	-35.6035f,-36.8364f,11.4184f, -37.2655f,-36.8364f,3.5992f, -36.4306f,-33.7031f,3.5115f, -34.1494f,-31.4094f,3.2717f, -31.0333f,-30.5698f,2.9442f, 
	-27.9172f,-31.4094f,2.6167f, -25.6360f,-33.7031f,2.3769f, -24.8011f,-36.8364f,2.2892f, -25.6360f,-39.9697f,2.3769f, -27.9172f,-42.2634f,2.6167f, 
	-31.0333f,-43.1030f,2.9442f, -34.1494f,-42.2634f,3.2717f, -36.4306f,-39.9697f,3.5115f, -37.2655f,-36.8364f,3.5992f, -37.2655f,-36.8364f,-4.3946f, 
	-36.4306f,-33.7031f,-4.3068f, -34.1494f,-31.4094f,-4.0671f, -31.0333f,-30.5698f,-3.7395f, -27.9172f,-31.4094f,-3.4120f, -25.6360f,-33.7031f,-3.1723f, 
	-24.8011f,-36.8364f,-3.0845f, -25.6360f,-39.9697f,-3.1723f, -27.9172f,-42.2634f,-3.4120f, -31.0333f,-43.1030f,-3.7395f, -34.1494f,-42.2634f,-4.0671f, 
	-36.4306f,-39.9697f,-4.3068f, -37.2655f,-36.8364f,-4.3946f, -35.6035f,-36.8364f,-12.2137f, -34.8051f,-33.7031f,-11.9543f, -32.6236f,-31.4094f,-11.2455f, 
	-29.6437f,-30.5698f,-10.2772f, -26.6637f,-31.4094f,-9.3090f, -24.4823f,-33.7031f,-8.6002f, -23.6838f,-36.8364f,-8.3408f, -24.4823f,-39.9697f,-8.6002f, 
	-26.6637f,-42.2634f,-9.3090f, -29.6437f,-43.1030f,-10.2772f, -32.6236f,-42.2634f,-11.2455f, -34.8051f,-39.9697f,-11.9543f, -35.6035f,-36.8364f,-12.2137f, 
	-32.3522f,-36.8364f,-19.5164f, -31.6251f,-33.7031f,-19.0966f, -29.6387f,-31.4094f,-17.9498f, -26.9251f,-30.5698f,-16.3831f, -24.2116f,-31.4094f,-14.8165f, 
	-22.2252f,-33.7031f,-13.6696f, -21.4981f,-36.8364f,-13.2499f, -22.2252f,-39.9697f,-13.6696f, -24.2116f,-42.2634f,-14.8165f, -26.9251f,-43.1030f,-16.3831f, 
	-29.6387f,-42.2634f,-17.9498f, -31.6251f,-39.9697f,-19.0967f, -32.3522f,-36.8364f,-19.5164f, -27.6535f,-36.8364f,-25.9836f, -27.0296f,-33.7031f,-25.4218f, 
	-25.3250f,-31.4094f,-23.8870f, -22.9965f,-30.5698f,-21.7904f, -20.6680f,-31.4094f,-19.6938f, -18.9635f,-33.7031f,-18.1590f, -18.3396f,-36.8364f,-17.5973f, 
	-18.9635f,-39.9697f,-18.1590f, -20.6680f,-42.2634f,-19.6938f, -22.9965f,-43.1030f,-21.7904f, -25.3250f,-42.2634f,-23.8870f, -27.0296f,-39.9697f,-25.4218f, 
	-27.6535f,-36.8364f,-25.9836f, -21.7129f,-36.8364f,-31.3325f, -21.2195f,-33.7031f,-30.6533f, -19.8712f,-31.4094f,-28.7976f, -18.0295f,-30.5698f,-26.2627f, 
	-16.1878f,-31.4094f,-23.7278f, -14.8396f,-33.7031f,-21.8722f, -14.3461f,-36.8364f,-21.1929f, -14.8396f,-39.9697f,-21.8722f, -16.1878f,-42.2634f,-23.7278f, 
	-18.0295f,-43.1030f,-26.2627f, -19.8712f,-42.2634f,-28.7976f, -21.2195f,-39.9697f,-30.6533f, -21.7129f,-36.8364f,-31.3325f, -14.7901f,-36.8364f,-35.3294f, 
	-14.4486f,-33.7031f,-34.5624f, -13.5157f,-31.4094f,-32.4670f, -12.2412f,-30.5698f,-29.6046f, -10.9668f,-31.4094f,-26.7422f, -10.0339f,-33.7031f,-24.6468f, 
	-9.6924f,-36.8364f,-23.8798f, -10.0339f,-39.9697f,-24.6468f, -10.9668f,-42.2634f,-26.7422f, -12.2412f,-43.1030f,-29.6046f, -13.5157f,-42.2634f,-32.4670f, 
	-14.4486f,-39.9697f,-34.5624f, -14.7901f,-36.8364f,-35.3294f, -7.1875f,-36.8364f,-37.7996f, -7.0130f,-33.7031f,-36.9784f, -6.5361f,-31.4094f,-34.7348f, 
	-5.8846f,-30.5698f,-31.6700f, -5.2332f,-31.4094f,-28.6052f, -4.7563f,-33.7031f,-26.3616f, -4.5817f,-36.8364f,-25.5403f, -4.7563f,-39.9697f,-26.3616f, 
	-5.2332f,-42.2634f,-28.6052f, -5.8846f,-43.1030f,-31.6700f, -6.5361f,-42.2634f,-34.7348f, -7.0130f,-39.9697f,-36.9784f, -7.1875f,-36.8364f,-37.7996f, 
	0.7625f,-36.8364f,-38.6352f, 0.7625f,-33.7031f,-37.7956f, 0.7625f,-31.4094f,-35.5019f, 0.7625f,-30.5698f,-32.3686f, 0.7625f,-31.4094f,-29.2353f, 
	0.7625f,-33.7031f,-26.9416f, 0.7625f,-36.8364f,-26.1020f, 0.7625f,-39.9697f,-26.9416f, 0.7625f,-42.2634f,-29.2353f, 0.7625f,-43.1030f,-32.3686f, 
	0.7625f,-42.2634f,-35.5019f, 0.7625f,-39.9697f,-37.7956f, 0.7625f,-36.8364f,-38.6352f, 7.4725f,-29.8482f,-32.3960f, 7.0529f,-27.1291f,-30.4219f, 
	0.6710f,-29.8482f,-33.1109f, 0.6710f,-27.1291f,-31.0927f, 6.8508f,-24.6780f,-29.2586f, 0.7253f,-24.6780f,-29.9024f, 5.0186f,-23.8256f,-20.6387f, 
	0.7253f,-23.8256f,-21.0900f, 3.1864f,-24.6780f,-12.0189f, 0.7253f,-24.6780f,-12.2776f, 1.6282f,-27.1291f,-4.9006f, 0.6710f,-27.1291f,-5.0012f, 
	1.2086f,-29.8482f,-2.9266f, 0.6710f,-29.8482f,-2.9831f, 1.6282f,-32.5673f,-4.9006f, 0.6710f,-32.5673f,-5.0012f, 2.7746f,-34.5578f,-10.2939f, 
	0.6710f,-34.5578f,-10.5150f, 4.3405f,-35.2864f,-17.6613f, 0.6710f,-35.2864f,-18.0470f, 5.9065f,-34.5578f,-25.0286f, 0.6710f,-34.5578f,-25.5789f, 
	7.0529f,-32.5673f,-30.4219f, 0.6710f,-32.5673f,-31.0927f, 7.4725f,-29.8482f,-32.3960f, 0.6710f,-29.8482f,-33.1109f, 13.9767f,-29.8482f,-30.2827f, 
	13.1558f,-27.1291f,-28.4390f, 12.7086f,-24.6780f,-27.3552f, 9.1243f,-23.8256f,-19.3047f, 5.5399f,-24.6780f,-11.2542f, 2.5435f,-27.1291f,-4.6032f, 
	1.7226f,-29.8482f,-2.7595f, 2.5435f,-32.5673f,-4.6032f, 4.7861f,-34.5578f,-9.6403f, 7.8497f,-35.2864f,-16.5211f, 10.9132f,-34.5578f,-23.4019f, 
	13.1558f,-32.5673f,-28.4390f, 13.9767f,-29.8482f,-30.2827f, 19.8994f,-29.8482f,-26.8632f, 18.7131f,-27.1291f,-25.2305f, 18.0426f,-24.6780f,-24.2756f, 
	12.8629f,-23.8256f,-17.1462f, 7.6831f,-24.6780f,-10.0169f, 3.3770f,-27.1291f,-4.1220f, 2.1907f,-29.8482f,-2.4893f, 3.3770f,-32.5673f,-4.1220f, 
	6.6179f,-34.5578f,-8.5828f, 11.0450f,-35.2864f,-14.6762f, 15.4722f,-34.5578f,-20.7697f, 18.7131f,-32.5673f,-25.2305f, 19.8994f,-29.8482f,-26.8632f, 
	24.9817f,-29.8482f,-22.2871f, 23.4819f,-27.1291f,-20.9366f, 22.6199f,-24.6780f,-20.1543f, 16.0710f,-23.8256f,-14.2576f, 9.5221f,-24.6780f,-8.3610f, 
	4.0922f,-27.1291f,-3.4781f, 2.5924f,-29.8482f,-2.1276f, 4.0922f,-32.5673f,-3.4781f, 8.1897f,-34.5578f,-7.1675f, 13.7870f,-35.2864f,-12.2073f, 
	19.3844f,-34.5578f,-17.2472f, 23.4819f,-32.5673f,-20.9366f, 24.9817f,-29.8482f,-22.2871f, 29.0015f,-29.8482f,-16.7543f, 27.2537f,-27.1291f,-15.7452f, 
	26.2402f,-24.6780f,-15.1713f, 18.6084f,-23.8256f,-10.7651f, 10.9767f,-24.6780f,-6.3590f, 4.6579f,-27.1291f,-2.6994f, 2.9101f,-29.8482f,-1.6904f, 
	4.6579f,-32.5673f,-2.6994f, 9.4329f,-34.5578f,-5.4563f, 15.9558f,-35.2864f,-9.2223f, 22.4786f,-34.5578f,-12.9883f, 27.2537f,-32.5673f,-15.7452f, 
	29.0015f,-29.8482f,-16.7543f, 31.7831f,-29.8482f,-10.5066f, 29.8637f,-27.1291f,-9.8829f, 28.7454f,-24.6780f,-9.5446f, 20.3643f,-23.8256f,-6.8214f, 
	11.9832f,-24.6780f,-4.0982f, 5.0493f,-27.1291f,-1.8202f, 3.1299f,-29.8482f,-1.1966f, 5.0493f,-32.5673f,-1.8202f, 10.2932f,-34.5578f,-3.5241f, 
	17.4565f,-35.2864f,-5.8516f, 24.6198f,-34.5578f,-8.1791f, 29.8637f,-32.5673f,-9.8829f, 31.7831f,-29.8482f,-10.5066f, 33.2050f,-29.8482f,-3.8171f, 
	31.1979f,-27.1291f,-3.6062f, 30.0259f,-24.6780f,-3.5199f, 21.2618f,-23.8256f,-2.5988f, 12.4977f,-24.6780f,-1.6776f, 5.2494f,-27.1291f,-0.8789f, 
	3.2423f,-29.8482f,-0.6679f, 5.2494f,-32.5673f,-0.8789f, 10.7330f,-34.5578f,-1.4552f, 18.2237f,-35.2864f,-2.2425f, 25.7144f,-34.5578f,-3.0298f, 
	31.1979f,-32.5673f,-3.6062f, 33.2050f,-29.8482f,-3.8171f, 33.2050f,-29.8482f,3.0218f, 31.1979f,-27.1291f,2.8108f, 30.0259f,-24.6780f,2.6393f, 
	21.2618f,-23.8256f,1.7182f, 12.4977f,-24.6780f,0.7970f, 5.2494f,-27.1291f,0.0835f, 3.2423f,-29.8482f,-0.1274f, 5.2494f,-32.5673f,0.0835f, 
	10.7330f,-34.5578f,0.6599f, 18.2237f,-35.2864f,1.4472f, 25.7144f,-34.5578f,2.2345f, 31.1979f,-32.5673f,2.8108f, 33.2050f,-29.8482f,3.0218f, 
	31.7831f,-29.8482f,9.7113f, 29.8637f,-27.1291f,9.0876f, 28.7454f,-24.6780f,8.6640f, 20.3643f,-23.8256f,5.9408f, 11.9832f,-24.6780f,3.2176f, 
	5.0493f,-27.1291f,1.0249f, 3.1299f,-29.8482f,0.4013f, 5.0493f,-32.5673f,1.0249f, 10.2932f,-34.5578f,2.7288f, 17.4565f,-35.2864f,5.0563f, 
	24.6198f,-34.5578f,7.3838f, 29.8637f,-32.5673f,9.0876f, 31.7831f,-29.8482f,9.7113f, 29.0015f,-29.8482f,15.9589f, 27.2537f,-27.1291f,14.9498f, 
	26.2402f,-24.6780f,14.2907f, 18.6084f,-23.8256f,9.8845f, 10.9767f,-24.6780f,5.4783f, 4.6579f,-27.1291f,1.9041f, 2.9101f,-29.8482f,0.8950f, 
	4.6579f,-32.5673f,1.9041f, 9.4329f,-34.5578f,4.6610f, 15.9558f,-35.2864f,8.4270f, 22.4786f,-34.5578f,12.1930f, 27.2537f,-32.5673f,14.9499f, 
	29.0015f,-29.8482f,15.9589f, 24.9817f,-29.8482f,21.4917f, 23.4819f,-27.1291f,20.1413f, 22.6199f,-24.6780f,19.2737f, 16.0710f,-23.8256f,13.3770f, 
	9.5221f,-24.6780f,7.4804f, 4.0922f,-27.1291f,2.6827f, 2.5924f,-29.8482f,1.3323f, 4.0922f,-32.5673f,2.6827f, 8.1897f,-34.5578f,6.3722f, 
	13.7870f,-35.2864f,11.4120f, 19.3844f,-34.5578f,16.4519f, 23.4819f,-32.5673f,20.1413f, 24.9817f,-29.8482f,21.4917f, 19.8994f,-29.8482f,26.0679f, 
	18.7131f,-27.1291f,24.4351f, 18.0426f,-24.6780f,23.3950f, 12.8629f,-23.8256f,16.2656f, 7.6831f,-24.6780f,9.1363f, 3.3770f,-27.1291f,3.3267f, 
	2.1907f,-29.8482f,1.6940f, 3.3770f,-32.5673f,3.3267f, 6.6179f,-34.5578f,7.7874f, 11.0450f,-35.2864f,13.8809f, 15.4722f,-34.5578f,19.9744f, 
	18.7131f,-32.5673f,24.4351f, 19.8994f,-29.8482f,26.0679f, 13.9767f,-29.8482f,29.4873f, 13.1558f,-27.1291f,27.6436f, 12.7086f,-24.6780f,26.4746f, 
	9.1243f,-23.8256f,18.4241f, 5.5399f,-24.6780f,10.3736f, 2.5435f,-27.1291f,3.8079f, 1.7226f,-29.8482f,1.9642f, 2.5435f,-32.5673f,3.8079f, 
	4.7861f,-34.5578f,8.8450f, 7.8497f,-35.2864f,15.7258f, 10.9132f,-34.5578f,22.6066f, 13.1558f,-32.5673f,27.6436f, 13.9767f,-29.8482f,29.4873f, 
	7.4725f,-29.8482f,31.6007f, 7.0529f,-27.1291f,29.6266f, 6.8508f,-24.6780f,28.3779f, 5.0186f,-23.8256f,19.7581f, 3.1864f,-24.6780f,11.1383f, 
	1.6282f,-27.1291f,4.1053f, 1.2086f,-29.8482f,2.1312f, 1.6282f,-32.5673f,4.1053f, 2.7746f,-34.5578f,9.4986f, 4.3405f,-35.2864f,16.8660f, 
	5.9065f,-34.5578f,24.2333f, 7.0529f,-32.5673f,29.6266f, 7.4725f,-29.8482f,31.6007f, 0.6710f,-29.8482f,32.3155f, 0.6710f,-27.1291f,30.2974f, 
	0.7253f,-24.6780f,29.0218f, 0.7253f,-23.8256f,20.2094f, 0.7253f,-24.6780f,11.3970f, 0.6710f,-27.1291f,4.2059f, 0.6710f,-29.8482f,2.1877f, 
	0.6710f,-32.5673f,4.2059f, 0.6710f,-34.5578f,9.7197f, 0.6710f,-35.2864f,17.2516f, 0.6710f,-34.5578f,24.7836f, 0.6710f,-32.5673f,30.2974f, 
	0.6710f,-29.8482f,32.3155f, -6.1304f,-29.8482f,31.6007f, -5.7108f,-27.1291f,29.6266f, -5.4002f,-24.6780f,28.3779f, -3.5680f,-23.8256f,19.7581f, 
	-1.7358f,-24.6780f,11.1383f, -0.2861f,-27.1291f,4.1053f, 0.1335f,-29.8482f,2.1312f, -0.2861f,-32.5673f,4.1053f, -1.4325f,-34.5578f,9.4986f, 
	-2.9985f,-35.2864f,16.8660f, -4.5644f,-34.5578f,24.2333f, -5.7108f,-32.5673f,29.6266f, -6.1304f,-29.8482f,31.6007f, -12.6346f,-29.8482f,29.4873f, 
	-11.8137f,-27.1291f,27.6436f, -11.2580f,-24.6780f,26.4746f, -7.6737f,-23.8256f,18.4241f, -4.0894f,-24.6780f,10.3736f, -1.2014f,-27.1291f,3.8079f, 
	-0.3805f,-29.8482f,1.9642f, -1.2014f,-32.5673f,3.8079f, -3.4441f,-34.5578f,8.8450f, -6.5076f,-35.2864f,15.7258f, -9.5711f,-34.5578f,22.6066f, 
	-11.8137f,-32.5673f,27.6436f, -12.6346f,-29.8482f,29.4873f, -18.5573f,-29.8482f,26.0679f, -17.3710f,-27.1291f,24.4351f, -16.5921f,-24.6780f,23.3950f, 
	-11.4123f,-23.8256f,16.2656f, -6.2325f,-24.6780f,9.1363f, -2.0349f,-27.1291f,3.3267f, -0.8486f,-29.8482f,1.6940f, -2.0349f,-32.5673f,3.3267f, 
	-5.2758f,-34.5578f,7.7874f, -9.7030f,-35.2864f,13.8809f, -14.1301f,-34.5578f,19.9744f, -17.3710f,-32.5673f,24.4351f, -18.5573f,-29.8482f,26.0679f, 
	-23.6396f,-29.8482f,21.4917f, -22.1398f,-27.1291f,20.1413f, -21.1693f,-24.6780f,19.2737f, -14.6204f,-23.8256f,13.3770f, -8.0715f,-24.6780f,7.4804f, 
	-2.7501f,-27.1291f,2.6827f, -1.2503f,-29.8482f,1.3323f, -2.7501f,-32.5673f,2.6827f, -6.8476f,-34.5578f,6.3722f, -12.4449f,-35.2864f,11.4120f, 
	-18.0423f,-34.5578f,16.4519f, -22.1398f,-32.5673f,20.1413f, -23.6396f,-29.8482f,21.4917f, -27.6594f,-29.8482f,15.9589f, -25.9116f,-27.1291f,14.9498f, 
	-24.7896f,-24.6780f,14.2907f, -17.1579f,-23.8256f,9.8845f, -9.5261f,-24.6780f,5.4783f, -3.3158f,-27.1291f,1.9041f, -1.5680f,-29.8482f,0.8950f, 
	-3.3158f,-32.5673f,1.9041f, -8.0908f,-34.5578f,4.6610f, -14.6137f,-35.2864f,8.4270f, -21.1366f,-34.5578f,12.1930f, -25.9116f,-32.5673f,14.9499f, 
	-27.6594f,-29.8482f,15.9589f, -30.4411f,-29.8482f,9.7113f, -28.5217f,-27.1291f,9.0876f, -27.2948f,-24.6780f,8.6640f, -18.9137f,-23.8256f,5.9408f, 
	-10.5327f,-24.6780f,3.2176f, -3.7072f,-27.1291f,1.0249f, -1.7878f,-29.8482f,0.4013f, -3.7072f,-32.5673f,1.0249f, -8.9511f,-34.5578f,2.7288f, 
	-16.1144f,-35.2864f,5.0563f, -23.2777f,-34.5578f,7.3838f, -28.5217f,-32.5673f,9.0876f, -30.4411f,-29.8482f,9.7113f, -31.8629f,-29.8482f,3.0218f, 
	-29.8558f,-27.1291f,2.8108f, -28.5754f,-24.6780f,2.6393f, -19.8113f,-23.8256f,1.7182f, -11.0472f,-24.6780f,0.7970f, -3.9073f,-27.1291f,0.0835f, 
	-1.9002f,-29.8482f,-0.1274f, -3.9073f,-32.5673f,0.0835f, -9.3909f,-34.5578f,0.6599f, -16.8816f,-35.2864f,1.4472f, -24.3723f,-34.5578f,2.2345f, 
	-29.8558f,-32.5673f,2.8108f, -31.8629f,-29.8482f,3.0218f, -31.8629f,-29.8482f,-3.8171f, -29.8558f,-27.1291f,-3.6062f, -28.5754f,-24.6780f,-3.5199f, 
	-19.8113f,-23.8256f,-2.5988f, -11.0472f,-24.6780f,-1.6776f, -3.9073f,-27.1291f,-0.8789f, -1.9002f,-29.8482f,-0.6679f, -3.9073f,-32.5673f,-0.8789f, 
	-9.3909f,-34.5578f,-1.4552f, -16.8816f,-35.2864f,-2.2425f, -24.3723f,-34.5578f,-3.0298f, -29.8558f,-32.5673f,-3.6062f, -31.8629f,-29.8482f,-3.8171f, 
	-30.4411f,-29.8482f,-10.5066f, -28.5217f,-27.1291f,-9.8829f, -27.2948f,-24.6780f,-9.5446f, -18.9137f,-23.8256f,-6.8214f, -10.5327f,-24.6780f,-4.0982f, 
	-3.7072f,-27.1291f,-1.8202f, -1.7878f,-29.8482f,-1.1966f, -3.7072f,-32.5673f,-1.8202f, -8.9511f,-34.5578f,-3.5241f, -16.1144f,-35.2864f,-5.8516f, 
	-23.2777f,-34.5578f,-8.1791f, -28.5217f,-32.5673f,-9.8829f, -30.4411f,-29.8482f,-10.5066f, -27.6594f,-29.8482f,-16.7543f, -25.9116f,-27.1291f,-15.7452f, 
	-24.7896f,-24.6780f,-15.1713f, -17.1579f,-23.8256f,-10.7651f, -9.5261f,-24.6780f,-6.3590f, -3.3158f,-27.1291f,-2.6994f, -1.5680f,-29.8482f,-1.6904f, 
	-3.3158f,-32.5673f,-2.6994f, -8.0908f,-34.5578f,-5.4563f, -14.6137f,-35.2864f,-9.2223f, -21.1366f,-34.5578f,-12.9883f, -25.9116f,-32.5673f,-15.7452f, 
	-27.6594f,-29.8482f,-16.7543f, -23.6396f,-29.8482f,-22.2871f, -22.1398f,-27.1291f,-20.9366f, -21.1693f,-24.6780f,-20.1543f, -14.6204f,-23.8256f,-14.2576f, 
	-8.0715f,-24.6780f,-8.3610f, -2.7501f,-27.1291f,-3.4781f, -1.2503f,-29.8482f,-2.1276f, -2.7501f,-32.5673f,-3.4781f, -6.8476f,-34.5578f,-7.1675f, 
	-12.4449f,-35.2864f,-12.2073f, -18.0423f,-34.5578f,-17.2472f, -22.1398f,-32.5673f,-20.9366f, -23.6396f,-29.8482f,-22.2871f, -18.5573f,-29.8482f,-26.8632f, 
	-17.3710f,-27.1291f,-25.2305f, -16.5921f,-24.6780f,-24.2756f, -11.4123f,-23.8256f,-17.1462f, -6.2325f,-24.6780f,-10.0169f, -2.0349f,-27.1291f,-4.1220f, 
	-0.8486f,-29.8482f,-2.4893f, -2.0349f,-32.5673f,-4.1220f, -5.2758f,-34.5578f,-8.5828f, -9.7030f,-35.2864f,-14.6762f, -14.1301f,-34.5578f,-20.7697f, 
	-17.3710f,-32.5673f,-25.2305f, -18.5573f,-29.8482f,-26.8632f, -12.6346f,-29.8482f,-30.2827f, -11.8137f,-27.1291f,-28.4390f, -11.2580f,-24.6780f,-27.3552f, 
	-7.6737f,-23.8256f,-19.3047f, -4.0894f,-24.6780f,-11.2542f, -1.2014f,-27.1291f,-4.6032f, -0.3805f,-29.8482f,-2.7595f, -1.2014f,-32.5673f,-4.6032f, 
	-3.4440f,-34.5578f,-9.6403f, -6.5076f,-35.2864f,-16.5211f, -9.5711f,-34.5578f,-23.4019f, -11.8137f,-32.5673f,-28.4390f, -12.6346f,-29.8482f,-30.2827f, 
	-6.1304f,-29.8482f,-32.3960f, -5.7108f,-27.1291f,-30.4219f, -5.4002f,-24.6780f,-29.2586f, -3.5680f,-23.8256f,-20.6387f, -1.7358f,-24.6780f,-12.0189f, 
	-0.2861f,-27.1291f,-4.9006f, 0.1335f,-29.8482f,-2.9266f, -0.2861f,-32.5673f,-4.9006f, -1.4325f,-34.5578f,-10.2939f, -2.9984f,-35.2864f,-17.6613f, 
	-4.5644f,-34.5578f,-25.0286f, -5.7108f,-32.5673f,-30.4219f, -6.1304f,-29.8482f,-32.3960f, 0.6710f,-29.8482f,-33.1109f, 0.6710f,-27.1291f,-31.0927f, 
	0.7253f,-24.6780f,-29.9024f, 0.7253f,-23.8256f,-21.0900f, 0.7253f,-24.6780f,-12.2776f, 0.6710f,-27.1291f,-5.0012f, 0.6710f,-29.8482f,-2.9831f, 
	0.6710f,-32.5673f,-5.0012f, 0.6710f,-34.5578f,-10.5150f, 0.6710f,-35.2864f,-18.0470f, 0.6710f,-34.5578f,-25.5789f, 0.6710f,-32.5673f,-31.0927f, 
	0.6710f,-29.8482f,-33.1109f, 5.8789f,48.2289f,-26.1966f, 5.8028f,49.5958f,-25.8384f, 0.4607f,48.2289f,-26.7661f, 0.4607f,49.5958f,-26.3999f, 
	5.5948f,50.5964f,-24.8596f, 0.4607f,50.5964f,-25.3992f, 5.3106f,50.9627f,-23.5226f, 0.4607f,50.9627f,-24.0324f, 5.0264f,50.5964f,-22.1856f, 
	0.4607f,50.5964f,-22.6655f, 4.8183f,49.5958f,-21.2069f, 0.4607f,49.5958f,-21.6649f, 4.7422f,48.2289f,-20.8486f, 0.4607f,48.2289f,-21.2986f, 
	4.8183f,46.8621f,-21.2069f, 0.4607f,46.8621f,-21.6649f, 5.0264f,45.8615f,-22.1856f, 0.4607f,45.8615f,-22.6655f, 5.3106f,45.4952f,-23.5226f, 
	0.4607f,45.4952f,-24.0324f, 5.5948f,45.8615f,-24.8596f, 0.4607f,45.8615f,-25.3992f, 5.8028f,46.8621f,-25.8384f, 0.4607f,46.8621f,-26.3999f, 
	5.8789f,48.2289f,-26.1966f, 0.4607f,48.2289f,-26.7661f, 11.0604f,48.2289f,-24.5131f, 10.9115f,49.5958f,-24.1785f, 10.5045f,50.5964f,-23.2644f, 
	9.9485f,50.9627f,-22.0157f, 9.3926f,50.5964f,-20.7670f, 8.9856f,49.5958f,-19.8529f, 8.8366f,48.2289f,-19.5183f, 8.9856f,46.8621f,-19.8529f, 
	9.3926f,45.8615f,-20.7670f, 9.9485f,45.4952f,-22.0157f, 10.5045f,45.8615f,-23.2644f, 10.9115f,46.8621f,-24.1785f, 11.0604f,48.2289f,-24.5131f, 
	15.7786f,48.2289f,-21.7890f, 15.5634f,49.5958f,-21.4927f, 14.9752f,50.5964f,-20.6832f, 14.1718f,50.9627f,-19.5774f, 13.3684f,50.5964f,-18.4715f, 
	12.7802f,49.5958f,-17.6620f, 12.5649f,48.2289f,-17.3657f, 12.7802f,46.8621f,-17.6620f, 13.3684f,45.8615f,-18.4715f, 14.1718f,45.4952f,-19.5774f, 
	14.9752f,45.8615f,-20.6832f, 15.5634f,46.8621f,-21.4927f, 15.7786f,48.2289f,-21.7890f, 19.8274f,48.2289f,-18.1435f, 19.5552f,49.5958f,-17.8984f, 
	18.8116f,50.5964f,-17.2289f, 17.7958f,50.9627f,-16.3143f, 16.7800f,50.5964f,-15.3996f, 16.0364f,49.5958f,-14.7301f, 15.7643f,48.2289f,-14.4850f, 
	16.0364f,46.8621f,-14.7301f, 16.7800f,45.8615f,-15.3996f, 17.7958f,45.4952f,-16.3143f, 18.8116f,45.8615f,-17.2289f, 19.5552f,46.8621f,-17.8984f, 
	19.8274f,48.2289f,-18.1435f, 23.0297f,48.2289f,-13.7359f, 22.7125f,49.5958f,-13.5527f, 21.8460f,50.5964f,-13.0524f, 20.6622f,50.9627f,-12.3690f, 
	19.4785f,50.5964f,-11.6856f, 18.6119f,49.5958f,-11.1852f, 18.2947f,48.2289f,-11.0021f, 18.6119f,46.8621f,-11.1852f, 19.4785f,45.8615f,-11.6856f, 
	20.6622f,45.4952f,-12.3690f, 21.8460f,45.8615f,-13.0524f, 22.7125f,46.8621f,-13.5527f, 23.0297f,48.2289f,-13.7359f, 25.2457f,48.2289f,-8.7587f, 
	24.8973f,49.5958f,-8.6456f, 23.9457f,50.5964f,-8.3364f, 22.6457f,50.9627f,-7.9140f, 21.3458f,50.5964f,-7.4916f, 20.3941f,49.5958f,-7.1824f, 
	20.0458f,48.2289f,-7.0692f, 20.3941f,46.8621f,-7.1824f, 21.3458f,45.8615f,-7.4916f, 22.6457f,45.4952f,-7.9140f, 23.9457f,45.8615f,-8.3364f, 
	24.8973f,46.8621f,-8.6456f, 25.2457f,48.2289f,-8.7587f, 26.3784f,48.2289f,-3.4297f, 26.0142f,49.5958f,-3.3914f, 25.0190f,50.5964f,-3.2868f, 
	23.6596f,50.9627f,-3.1439f, 22.3003f,50.5964f,-3.0010f, 21.3051f,49.5958f,-2.8964f, 20.9409f,48.2289f,-2.8582f, 21.3051f,46.8621f,-2.8964f, 
	22.3003f,45.8615f,-3.0010f, 23.6596f,45.4952f,-3.1439f, 25.0190f,45.8615f,-3.2868f, 26.0142f,46.8621f,-3.3914f, 26.3784f,48.2289f,-3.4297f, 
	26.3784f,48.2289f,2.0185f, 26.0142f,49.5958f,1.9802f, 25.0190f,50.5964f,1.8756f, 23.6596f,50.9627f,1.7327f, 22.3003f,50.5964f,1.5898f, 
	21.3051f,49.5958f,1.4852f, 20.9409f,48.2289f,1.4470f, 21.3051f,46.8621f,1.4852f, 22.3003f,45.8615f,1.5898f, 23.6596f,45.4952f,1.7327f, 
	25.0190f,45.8615f,1.8756f, 26.0142f,46.8621f,1.9802f, 26.3784f,48.2289f,2.0185f, 25.2457f,48.2289f,7.3475f, 24.8973f,49.5958f,7.2344f, 
	23.9457f,50.5964f,6.9251f, 22.6457f,50.9627f,6.5028f, 21.3458f,50.5964f,6.0804f, 20.3941f,49.5958f,5.7712f, 20.0458f,48.2289f,5.6580f, 
	20.3941f,46.8621f,5.7712f, 21.3458f,45.8615f,6.0804f, 22.6457f,45.4952f,6.5028f, 23.9457f,45.8615f,6.9251f, 24.8973f,46.8621f,7.2344f, 
	25.2457f,48.2289f,7.3475f, 23.0297f,48.2289f,12.3246f, 22.7125f,49.5958f,12.1415f, 21.8460f,50.5964f,11.6412f, 20.6622f,50.9627f,10.9578f, 
	19.4785f,50.5964f,10.2743f, 18.6119f,49.5958f,9.7740f, 18.2947f,48.2289f,9.5909f, 18.6119f,46.8621f,9.7740f, 19.4785f,45.8615f,10.2743f, 
	20.6622f,45.4952f,10.9578f, 21.8460f,45.8615f,11.6412f, 22.7125f,46.8621f,12.1415f, 23.0297f,48.2289f,12.3246f, 19.8274f,48.2289f,16.7323f, 
	19.5552f,49.5958f,16.4872f, 18.8116f,50.5964f,15.8177f, 17.7958f,50.9627f,14.9030f, 16.7800f,50.5964f,13.9884f, 16.0364f,49.5958f,13.3189f, 
	15.7643f,48.2289f,13.0738f, 16.0364f,46.8621f,13.3189f, 16.7800f,45.8615f,13.9884f, 17.7958f,45.4952f,14.9030f, 18.8116f,45.8615f,15.8177f, 
	19.5552f,46.8621f,16.4872f, 19.8274f,48.2289f,16.7323f, 15.7786f,48.2289f,20.3778f, 15.5634f,49.5958f,20.0815f, 14.9752f,50.5964f,19.2720f, 
	14.1718f,50.9627f,18.1661f, 13.3684f,50.5964f,17.0603f, 12.7802f,49.5958f,16.2508f, 12.5649f,48.2289f,15.9545f, 12.7802f,46.8621f,16.2508f, 
	13.3684f,45.8615f,17.0603f, 14.1718f,45.4952f,18.1661f, 14.9752f,45.8615f,19.2720f, 15.5634f,46.8621f,20.0815f, 15.7786f,48.2289f,20.3778f, 
	11.0604f,48.2289f,23.1018f, 10.9115f,49.5958f,22.7673f, 10.5045f,50.5964f,21.8531f, 9.9485f,50.9627f,20.6045f, 9.3926f,50.5964f,19.3558f, 
	8.9856f,49.5958f,18.4416f, 8.8366f,48.2289f,18.1071f, 8.9856f,46.8621f,18.4416f, 9.3926f,45.8615f,19.3558f, 9.9485f,45.4952f,20.6045f, 
	10.5045f,45.8615f,21.8531f, 10.9115f,46.8621f,22.7673f, 11.0604f,48.2289f,23.1018f, 5.8789f,48.2289f,24.7854f, 5.8028f,49.5958f,24.4272f, 
	5.5948f,50.5964f,23.4484f, 5.3106f,50.9627f,22.1114f, 5.0264f,50.5964f,20.7744f, 4.8183f,49.5958f,19.7957f, 4.7422f,48.2289f,19.4374f, 
	4.8183f,46.8621f,19.7957f, 5.0264f,45.8615f,20.7744f, 5.3106f,45.4952f,22.1114f, 5.5948f,45.8615f,23.4484f, 5.8028f,46.8621f,24.4272f, 
	5.8789f,48.2289f,24.7854f, 0.4607f,48.2289f,25.3549f, 0.4607f,49.5958f,24.9886f, 0.4607f,50.5964f,23.9880f, 0.4607f,50.9627f,22.6212f, 
	0.4607f,50.5964f,21.2543f, 0.4607f,49.5958f,20.2537f, 0.4607f,48.2289f,19.8874f, 0.4607f,46.8621f,20.2537f, 0.4607f,45.8615f,21.2543f, 
	0.4607f,45.4952f,22.6212f, 0.4607f,45.8615f,23.9880f, 0.4607f,46.8621f,24.9886f, 0.4607f,48.2289f,25.3549f, -4.9576f,48.2289f,24.7854f, 
	-4.8815f,49.5958f,24.4272f, -4.6734f,50.5964f,23.4484f, -4.3892f,50.9627f,22.1114f, -4.1051f,50.5964f,20.7744f, -3.8970f,49.5958f,19.7957f, 
	-3.8209f,48.2289f,19.4374f, -3.8970f,46.8621f,19.7957f, -4.1051f,45.8615f,20.7744f, -4.3892f,45.4952f,22.1114f, -4.6734f,45.8615f,23.4484f, 
	-4.8815f,46.8621f,24.4272f, -4.9576f,48.2289f,24.7854f, -10.1391f,48.2289f,23.1018f, -9.9901f,49.5958f,22.7673f, -9.5831f,50.5964f,21.8531f, 
	-9.0272f,50.9627f,20.6044f, -8.4712f,50.5964f,19.3558f, -8.0642f,49.5958f,18.4416f, -7.9153f,48.2289f,18.1071f, -8.0642f,46.8621f,18.4416f, 
	-8.4712f,45.8615f,19.3558f, -9.0272f,45.4952f,20.6044f, -9.5831f,45.8615f,21.8531f, -9.9901f,46.8621f,22.7673f, -10.1391f,48.2289f,23.1018f, 
	-14.8573f,48.2289f,20.3778f, -14.6420f,49.5958f,20.0815f, -14.0539f,50.5964f,19.2720f, -13.2505f,50.9627f,18.1661f, -12.4470f,50.5964f,17.0603f, 
	-11.8589f,49.5958f,16.2508f, -11.6436f,48.2289f,15.9545f, -11.8589f,46.8621f,16.2508f, -12.4470f,45.8615f,17.0603f, -13.2505f,45.4952f,18.1661f, 
	-14.0539f,45.8615f,19.2720f, -14.6420f,46.8621f,20.0815f, -14.8573f,48.2289f,20.3778f, -18.9061f,48.2289f,16.7323f, -18.6339f,49.5958f,16.4872f, 
	-17.8903f,50.5964f,15.8177f, -16.8745f,50.9627f,14.9030f, -15.8587f,50.5964f,13.9884f, -15.1151f,49.5958f,13.3189f, -14.8429f,48.2289f,13.0738f, 
	-15.1151f,46.8621f,13.3189f, -15.8587f,45.8615f,13.9884f, -16.8745f,45.4952f,14.9030f, -17.8903f,45.8615f,15.8177f, -18.6339f,46.8621f,16.4872f, 
	-18.9061f,48.2289f,16.7323f, -22.1084f,48.2289f,12.3246f, -21.7912f,49.5958f,12.1415f, -20.9246f,50.5964f,11.6412f, -19.7409f,50.9627f,10.9578f, 
	-18.5572f,50.5964f,10.2743f, -17.6906f,49.5958f,9.7740f, -17.3734f,48.2289f,9.5909f, -17.6906f,46.8621f,9.7740f, -18.5572f,45.8615f,10.2743f, 
	-19.7409f,45.4952f,10.9578f, -20.9246f,45.8615f,11.6412f, -21.7912f,46.8621f,12.1415f, -22.1084f,48.2289f,12.3246f, -24.3243f,48.2289f,7.3475f, 
	-23.9760f,49.5958f,7.2344f, -23.0244f,50.5964f,6.9251f, -21.7244f,50.9627f,6.5028f, -20.4244f,50.5964f,6.0804f, -19.4728f,49.5958f,5.7712f, 
	-19.1245f,48.2289f,5.6580f, -19.4728f,46.8621f,5.7712f, -20.4244f,45.8615f,6.0804f, -21.7244f,45.4952f,6.5028f, -23.0244f,45.8615f,6.9251f, 
	-23.9760f,46.8621f,7.2344f, -24.3243f,48.2289f,7.3475f, -25.4571f,48.2289f,2.0185f, -25.0928f,49.5958f,1.9802f, -24.0977f,50.5964f,1.8756f, 
	-22.7383f,50.9627f,1.7327f, -21.3789f,50.5964f,1.5898f, -20.3838f,49.5958f,1.4852f, -20.0195f,48.2289f,1.4469f, -20.3838f,46.8621f,1.4852f, 
	-21.3789f,45.8615f,1.5898f, -22.7383f,45.4952f,1.7327f, -24.0977f,45.8615f,1.8756f, -25.0928f,46.8621f,1.9802f, -25.4571f,48.2289f,2.0185f, 
	-25.4571f,48.2289f,-3.4297f, -25.0928f,49.5958f,-3.3914f, -24.0977f,50.5964f,-3.2868f, -22.7383f,50.9627f,-3.1439f, -21.3789f,50.5964f,-3.0010f, 
	-20.3838f,49.5958f,-2.8964f, -20.0195f,48.2289f,-2.8582f, -20.3838f,46.8621f,-2.8964f, -21.3789f,45.8615f,-3.0010f, -22.7383f,45.4952f,-3.1439f, 
	-24.0977f,45.8615f,-3.2868f, -25.0928f,46.8621f,-3.3914f, -25.4571f,48.2289f,-3.4297f, -24.3243f,48.2289f,-8.7587f, -23.9760f,49.5958f,-8.6456f, 
	-23.0244f,50.5964f,-8.3364f, -21.7244f,50.9627f,-7.9140f, -20.4244f,50.5964f,-7.4916f, -19.4728f,49.5958f,-7.1824f, -19.1245f,48.2289f,-7.0692f, 
	-19.4728f,46.8621f,-7.1824f, -20.4244f,45.8615f,-7.4916f, -21.7244f,45.4952f,-7.9140f, -23.0244f,45.8615f,-8.3364f, -23.9760f,46.8621f,-8.6456f, 
	-24.3243f,48.2289f,-8.7587f, -22.1084f,48.2289f,-13.7359f, -21.7912f,49.5958f,-13.5527f, -20.9246f,50.5964f,-13.0524f, -19.7409f,50.9627f,-12.3690f, 
	-18.5572f,50.5964f,-11.6856f, -17.6906f,49.5958f,-11.1852f, -17.3734f,48.2289f,-11.0021f, -17.6906f,46.8621f,-11.1852f, -18.5572f,45.8615f,-11.6856f, 
	-19.7409f,45.4952f,-12.3690f, -20.9246f,45.8615f,-13.0524f, -21.7912f,46.8621f,-13.5527f, -22.1084f,48.2289f,-13.7359f, -18.9061f,48.2289f,-18.1435f, 
	-18.6339f,49.5958f,-17.8984f, -17.8903f,50.5964f,-17.2289f, -16.8745f,50.9627f,-16.3143f, -15.8587f,50.5964f,-15.3996f, -15.1151f,49.5958f,-14.7301f, 
	-14.8429f,48.2289f,-14.4850f, -15.1151f,46.8621f,-14.7301f, -15.8587f,45.8615f,-15.3996f, -16.8745f,45.4952f,-16.3143f, -17.8903f,45.8615f,-17.2289f, 
	-18.6339f,46.8621f,-17.8984f, -18.9061f,48.2289f,-18.1435f, -14.8573f,48.2289f,-21.7890f, -14.6420f,49.5958f,-21.4927f, -14.0539f,50.5964f,-20.6832f, 
	-13.2505f,50.9627f,-19.5774f, -12.4470f,50.5964f,-18.4715f, -11.8589f,49.5958f,-17.6620f, -11.6436f,48.2289f,-17.3657f, -11.8589f,46.8621f,-17.6620f, 
	-12.4470f,45.8615f,-18.4715f, -13.2505f,45.4952f,-19.5774f, -14.0539f,45.8615f,-20.6832f, -14.6420f,46.8621f,-21.4927f, -14.8573f,48.2289f,-21.7890f, 
	-10.1391f,48.2289f,-24.5131f, -9.9901f,49.5958f,-24.1785f, -9.5831f,50.5964f,-23.2644f, -9.0272f,50.9627f,-22.0157f, -8.4712f,50.5964f,-20.7670f, 
	-8.0642f,49.5958f,-19.8529f, -7.9153f,48.2289f,-19.5183f, -8.0642f,46.8621f,-19.8529f, -8.4712f,45.8615f,-20.7670f, -9.0272f,45.4952f,-22.0157f, 
	-9.5831f,45.8615f,-23.2644f, -9.9901f,46.8621f,-24.1785f, -10.1391f,48.2289f,-24.5131f, -4.9576f,48.2289f,-26.1966f, -4.8815f,49.5958f,-25.8384f, 
	-4.6734f,50.5964f,-24.8596f, -4.3892f,50.9627f,-23.5226f, -4.1051f,50.5964f,-22.1856f, -3.8970f,49.5958f,-21.2069f, -3.8209f,48.2289f,-20.8486f, 
	-3.8970f,46.8621f,-21.2069f, -4.1051f,45.8615f,-22.1856f, -4.3892f,45.4952f,-23.5226f, -4.6734f,45.8615f,-24.8596f, -4.8815f,46.8621f,-25.8384f, 
	-4.9576f,48.2289f,-26.1966f, 0.4607f,48.2289f,-26.7661f, 0.4607f,49.5958f,-26.3999f, 0.4607f,50.5964f,-25.3992f, 0.4607f,50.9627f,-24.0324f, 
	0.4607f,50.5964f,-22.6655f, 0.4607f,49.5958f,-21.6649f, 0.4607f,48.2289f,-21.2986f, 0.4607f,46.8621f,-21.6649f, 0.4607f,45.8615f,-22.6655f, 
	0.4607f,45.4952f,-24.0324f, 0.4607f,45.8615f,-25.3992f, 0.4607f,46.8621f,-26.3999f, 0.4607f,48.2289f,-26.7661f, 2.4905f,-14.2461f,-23.9520f, 
	1.8544f,-14.2461f,-25.4876f, 2.4905f,-26.8184f,-23.9520f, 1.8544f,-26.8184f,-25.4876f, 0.3188f,-14.2461f,-26.1236f, 0.3188f,-26.8184f,-26.1236f, 
	-1.2167f,-14.2461f,-25.4876f, -1.2167f,-26.8184f,-25.4876f, -1.8528f,-14.2461f,-23.9520f, -1.8528f,-26.8184f,-23.9520f, -1.2167f,-14.2461f,-22.4164f, 
	-1.2167f,-26.8184f,-22.4164f, 0.3188f,-14.2461f,-21.7804f, 0.3188f,-26.8184f,-21.7804f, 0.3188f,-14.2461f,-21.7804f, 1.8544f,-14.2461f,-22.4164f, 
	0.3188f,-26.8184f,-21.7804f, 1.8544f,-26.8184f,-22.4164f, 2.4905f,-1.6738f,-23.9520f, 1.8544f,-1.6738f,-25.4876f, 0.3188f,-1.6738f,-26.1236f, 
	-1.2167f,-1.6738f,-25.4876f, -1.8528f,-1.6738f,-23.9520f, -1.2167f,-1.6738f,-22.4164f, 0.3188f,-1.6738f,-21.7804f, 0.3188f,-1.6738f,-21.7804f, 
	1.8544f,-1.6738f,-22.4164f, 2.4905f,10.8985f,-23.9520f, 1.8544f,10.8985f,-25.4876f, 0.3188f,10.8985f,-26.1236f, -1.2167f,10.8985f,-25.4876f, 
	-1.8528f,10.8985f,-23.9520f, -1.2167f,10.8985f,-22.4164f, 0.3188f,10.8985f,-21.7804f, 0.3188f,10.8985f,-21.7804f, 1.8544f,10.8985f,-22.4164f, 
	2.4905f,23.4708f,-23.9520f, 1.8544f,23.4708f,-25.4876f, 0.3188f,23.4708f,-26.1236f, -1.2167f,23.4708f,-25.4876f, -1.8528f,23.4708f,-23.9520f, 
	-1.2167f,23.4708f,-22.4164f, 0.3188f,23.4708f,-21.7804f, 0.3188f,23.4708f,-21.7804f, 1.8544f,23.4708f,-22.4164f, 2.4905f,36.0431f,-23.9520f, 
	1.8544f,36.0431f,-25.4876f, 0.3188f,36.0431f,-26.1236f, -1.2167f,36.0431f,-25.4876f, -1.8528f,36.0431f,-23.9520f, -1.2167f,36.0431f,-22.4164f, 
	0.3188f,36.0431f,-21.7804f, 0.3188f,36.0431f,-21.7804f, 1.8544f,36.0431f,-22.4164f, 2.4905f,48.6154f,-23.9520f, 1.8544f,48.6154f,-25.4876f, 
	0.3188f,48.6154f,-26.1236f, -1.2167f,48.6154f,-25.4876f, -1.8528f,48.6154f,-23.9520f, -1.2167f,48.6154f,-22.4164f, 0.3188f,48.6154f,-21.7804f, 
	0.3188f,48.6154f,-21.7804f, 1.8544f,48.6154f,-22.4164f, 2.4905f,-14.2461f,22.4326f, 1.8544f,-14.2461f,20.8970f, 2.4905f,-26.8184f,22.4326f, 
	1.8544f,-26.8184f,20.8970f, 0.3188f,-14.2461f,20.2610f, 0.3188f,-26.8184f,20.2610f, -1.2167f,-14.2461f,20.8970f, -1.2167f,-26.8184f,20.8970f, 
	-1.8528f,-14.2461f,22.4326f, -1.8528f,-26.8184f,22.4326f, -1.2167f,-14.2461f,23.9682f, -1.2167f,-26.8184f,23.9682f, 0.3188f,-14.2461f,24.6042f, 
	0.3188f,-26.8184f,24.6042f, 0.3188f,-14.2461f,24.6042f, 1.8544f,-14.2461f,23.9682f, 0.3188f,-26.8184f,24.6042f, 1.8544f,-26.8184f,23.9682f, 
	2.4905f,-1.6738f,22.4326f, 1.8544f,-1.6738f,20.8970f, 0.3188f,-1.6738f,20.2610f, -1.2167f,-1.6738f,20.8970f, -1.8528f,-1.6738f,22.4326f, 
	-1.2167f,-1.6738f,23.9682f, 0.3188f,-1.6738f,24.6042f, 0.3188f,-1.6738f,24.6042f, 1.8544f,-1.6738f,23.9682f, 2.4905f,10.8985f,22.4326f, 
	1.8544f,10.8985f,20.8970f, 0.3188f,10.8985f,20.2610f, -1.2167f,10.8985f,20.8970f, -1.8528f,10.8985f,22.4326f, -1.2167f,10.8985f,23.9682f, 
	0.3188f,10.8985f,24.6042f, 0.3188f,10.8985f,24.6042f, 1.8544f,10.8985f,23.9682f, 2.4905f,23.4708f,22.4326f, 1.8544f,23.4708f,20.8970f, 
	0.3188f,23.4708f,20.2610f, -1.2167f,23.4708f,20.8970f, -1.8528f,23.4708f,22.4326f, -1.2167f,23.4708f,23.9682f, 0.3188f,23.4708f,24.6042f, 
	0.3188f,23.4708f,24.6042f, 1.8544f,23.4708f,23.9682f, 2.4905f,36.0431f,22.4326f, 1.8544f,36.0431f,20.8970f, 0.3188f,36.0431f,20.2610f, 
	-1.2167f,36.0431f,20.8970f, -1.8528f,36.0431f,22.4326f, -1.2167f,36.0431f,23.9682f, 0.3188f,36.0431f,24.6042f, 0.3188f,36.0431f,24.6042f, 
	1.8544f,36.0431f,23.9682f, 2.4905f,48.6154f,22.4326f, 1.8544f,48.6154f,20.8970f, 0.3188f,48.6154f,20.2610f, -1.2167f,48.6154f,20.8970f, 
	-1.8528f,48.6154f,22.4326f, -1.2167f,48.6154f,23.9682f, 0.3188f,48.6154f,24.6042f, 0.3188f,48.6154f,24.6042f, 1.8544f,48.6154f,23.9682f, 
	
_NULL_ }; /* End of silver_Vertices */

const float silver_UV[] = {
	0.0000f,0.0333f,0.0833f,0.0333f,0.0000f,0.0000f,0.0833f,0.0000f,0.1667f,0.0333f,0.1667f,0.0000f,0.2500f,0.0333f,0.2500f,0.0000f,0.3333f,0.0333f,0.3333f,0.0000f,0.4167f,0.0333f,0.4167f,0.0000f,0.5000f,0.0333f,0.5000f,0.0000f,0.5833f,0.0333f,
	0.5833f,0.0000f,0.6667f,0.0333f,0.6667f,0.0000f,0.7500f,0.0333f,0.7500f,0.0000f,0.8333f,0.0333f,0.8333f,0.0000f,0.9167f,0.0333f,0.9167f,0.0000f,1.0000f,0.0333f,1.0000f,0.0000f,0.0000f,0.0667f,0.0833f,0.0667f,0.1667f,0.0667f,0.2500f,0.0667f,
	0.3333f,0.0667f,0.4167f,0.0667f,0.5000f,0.0667f,0.5833f,0.0667f,0.6667f,0.0667f,0.7500f,0.0667f,0.8333f,0.0667f,0.9167f,0.0667f,1.0000f,0.0667f,0.0000f,0.1000f,0.0833f,0.1000f,0.1667f,0.1000f,0.2500f,0.1000f,0.3333f,0.1000f,0.4167f,0.1000f,
	0.5000f,0.1000f,0.5833f,0.1000f,0.6667f,0.1000f,0.7500f,0.1000f,0.8333f,0.1000f,0.9167f,0.1000f,1.0000f,0.1000f,0.0000f,0.1333f,0.0833f,0.1333f,0.1667f,0.1333f,0.2500f,0.1333f,0.3333f,0.1333f,0.4167f,0.1333f,0.5000f,0.1333f,0.5833f,0.1333f,
	0.6667f,0.1333f,0.7500f,0.1333f,0.8333f,0.1333f,0.9167f,0.1333f,1.0000f,0.1333f,0.0000f,0.1667f,0.0833f,0.1667f,0.1667f,0.1667f,0.2500f,0.1667f,0.3333f,0.1667f,0.4167f,0.1667f,0.5000f,0.1667f,0.5833f,0.1667f,0.6667f,0.1667f,0.7500f,0.1667f,
	0.8333f,0.1667f,0.9167f,0.1667f,1.0000f,0.1667f,0.0000f,0.2000f,0.0833f,0.2000f,0.1667f,0.2000f,0.2500f,0.2000f,0.3333f,0.2000f,0.4167f,0.2000f,0.5000f,0.2000f,0.5833f,0.2000f,0.6667f,0.2000f,0.7500f,0.2000f,0.8333f,0.2000f,0.9167f,0.2000f,
	1.0000f,0.2000f,0.0000f,0.2333f,0.0833f,0.2333f,0.1667f,0.2333f,0.2500f,0.2333f,0.3333f,0.2333f,0.4167f,0.2333f,0.5000f,0.2333f,0.5833f,0.2333f,0.6667f,0.2333f,0.7500f,0.2333f,0.8333f,0.2333f,0.9167f,0.2333f,1.0000f,0.2333f,0.0000f,0.2667f,
	0.0833f,0.2667f,0.1667f,0.2667f,0.2500f,0.2667f,0.3333f,0.2667f,0.4167f,0.2667f,0.5000f,0.2667f,0.5833f,0.2667f,0.6667f,0.2667f,0.7500f,0.2667f,0.8333f,0.2667f,0.9167f,0.2667f,1.0000f,0.2667f,0.0000f,0.3000f,0.0833f,0.3000f,0.1667f,0.3000f,
	0.2500f,0.3000f,0.3333f,0.3000f,0.4167f,0.3000f,0.5000f,0.3000f,0.5833f,0.3000f,0.6667f,0.3000f,0.7500f,0.3000f,0.8333f,0.3000f,0.9167f,0.3000f,1.0000f,0.3000f,0.0000f,0.3333f,0.0833f,0.3333f,0.1667f,0.3333f,0.2500f,0.3333f,0.3333f,0.3333f,
	0.4167f,0.3333f,0.5000f,0.3333f,0.5833f,0.3333f,0.6667f,0.3333f,0.7500f,0.3333f,0.8333f,0.3333f,0.9167f,0.3333f,1.0000f,0.3333f,0.0000f,0.3667f,0.0833f,0.3667f,0.1667f,0.3667f,0.2500f,0.3667f,0.3333f,0.3667f,0.4167f,0.3667f,0.5000f,0.3667f,
	0.5833f,0.3667f,0.6667f,0.3667f,0.7500f,0.3667f,0.8333f,0.3667f,0.9167f,0.3667f,1.0000f,0.3667f,0.0000f,0.4000f,0.0833f,0.4000f,0.1667f,0.4000f,0.2500f,0.4000f,0.3333f,0.4000f,0.4167f,0.4000f,0.5000f,0.4000f,0.5833f,0.4000f,0.6667f,0.4000f,
	0.7500f,0.4000f,0.8333f,0.4000f,0.9167f,0.4000f,1.0000f,0.4000f,0.0000f,0.4333f,0.0833f,0.4333f,0.1667f,0.4333f,0.2500f,0.4333f,0.3333f,0.4333f,0.4167f,0.4333f,0.5000f,0.4333f,0.5833f,0.4333f,0.6667f,0.4333f,0.7500f,0.4333f,0.8333f,0.4333f,
	0.9167f,0.4333f,1.0000f,0.4333f,0.0000f,0.4667f,0.0833f,0.4667f,0.1667f,0.4667f,0.2500f,0.4667f,0.3333f,0.4667f,0.4167f,0.4667f,0.5000f,0.4667f,0.5833f,0.4667f,0.6667f,0.4667f,0.7500f,0.4667f,0.8333f,0.4667f,0.9167f,0.4667f,1.0000f,0.4667f,
	0.0000f,0.5000f,0.0833f,0.5000f,0.1667f,0.5000f,0.2500f,0.5000f,0.3333f,0.5000f,0.4167f,0.5000f,0.5000f,0.5000f,0.5833f,0.5000f,0.6667f,0.5000f,0.7500f,0.5000f,0.8333f,0.5000f,0.9167f,0.5000f,1.0000f,0.5000f,0.0000f,0.5333f,0.0833f,0.5333f,
	0.1667f,0.5333f,0.2500f,0.5333f,0.3333f,0.5333f,0.4167f,0.5333f,0.5000f,0.5333f,0.5833f,0.5333f,0.6667f,0.5333f,0.7500f,0.5333f,0.8333f,0.5333f,0.9167f,0.5333f,1.0000f,0.5333f,0.0000f,0.5667f,0.0833f,0.5667f,0.1667f,0.5667f,0.2500f,0.5667f,
	0.3333f,0.5667f,0.4167f,0.5667f,0.5000f,0.5667f,0.5833f,0.5667f,0.6667f,0.5667f,0.7500f,0.5667f,0.8333f,0.5667f,0.9167f,0.5667f,1.0000f,0.5667f,0.0000f,0.6000f,0.0833f,0.6000f,0.1667f,0.6000f,0.2500f,0.6000f,0.3333f,0.6000f,0.4167f,0.6000f,
	0.5000f,0.6000f,0.5833f,0.6000f,0.6667f,0.6000f,0.7500f,0.6000f,0.8333f,0.6000f,0.9167f,0.6000f,1.0000f,0.6000f,0.0000f,0.6333f,0.0833f,0.6333f,0.1667f,0.6333f,0.2500f,0.6333f,0.3333f,0.6333f,0.4167f,0.6333f,0.5000f,0.6333f,0.5833f,0.6333f,
	0.6667f,0.6333f,0.7500f,0.6333f,0.8333f,0.6333f,0.9167f,0.6333f,1.0000f,0.6333f,0.0000f,0.6667f,0.0833f,0.6667f,0.1667f,0.6667f,0.2500f,0.6667f,0.3333f,0.6667f,0.4167f,0.6667f,0.5000f,0.6667f,0.5833f,0.6667f,0.6667f,0.6667f,0.7500f,0.6667f,
	0.8333f,0.6667f,0.9167f,0.6667f,1.0000f,0.6667f,0.0000f,0.7000f,0.0833f,0.7000f,0.1667f,0.7000f,0.2500f,0.7000f,0.3333f,0.7000f,0.4167f,0.7000f,0.5000f,0.7000f,0.5833f,0.7000f,0.6667f,0.7000f,0.7500f,0.7000f,0.8333f,0.7000f,0.9167f,0.7000f,
	1.0000f,0.7000f,0.0000f,0.7333f,0.0833f,0.7333f,0.1667f,0.7333f,0.2500f,0.7333f,0.3333f,0.7333f,0.4167f,0.7333f,0.5000f,0.7333f,0.5833f,0.7333f,0.6667f,0.7333f,0.7500f,0.7333f,0.8333f,0.7333f,0.9167f,0.7333f,1.0000f,0.7333f,0.0000f,0.7667f,
	0.0833f,0.7667f,0.1667f,0.7667f,0.2500f,0.7667f,0.3333f,0.7667f,0.4167f,0.7667f,0.5000f,0.7667f,0.5833f,0.7667f,0.6667f,0.7667f,0.7500f,0.7667f,0.8333f,0.7667f,0.9167f,0.7667f,1.0000f,0.7667f,0.0000f,0.8000f,0.0833f,0.8000f,0.1667f,0.8000f,
	0.2500f,0.8000f,0.3333f,0.8000f,0.4167f,0.8000f,0.5000f,0.8000f,0.5833f,0.8000f,0.6667f,0.8000f,0.7500f,0.8000f,0.8333f,0.8000f,0.9167f,0.8000f,1.0000f,0.8000f,0.0000f,0.8333f,0.0833f,0.8333f,0.1667f,0.8333f,0.2500f,0.8333f,0.3333f,0.8333f,
	0.4167f,0.8333f,0.5000f,0.8333f,0.5833f,0.8333f,0.6667f,0.8333f,0.7500f,0.8333f,0.8333f,0.8333f,0.9167f,0.8333f,1.0000f,0.8333f,0.0000f,0.8667f,0.0833f,0.8667f,0.1667f,0.8667f,0.2500f,0.8667f,0.3333f,0.8667f,0.4167f,0.8667f,0.5000f,0.8667f,
	0.5833f,0.8667f,0.6667f,0.8667f,0.7500f,0.8667f,0.8333f,0.8667f,0.9167f,0.8667f,1.0000f,0.8667f,0.0000f,0.9000f,0.0833f,0.9000f,0.1667f,0.9000f,0.2500f,0.9000f,0.3333f,0.9000f,0.4167f,0.9000f,0.5000f,0.9000f,0.5833f,0.9000f,0.6667f,0.9000f,
	0.7500f,0.9000f,0.8333f,0.9000f,0.9167f,0.9000f,1.0000f,0.9000f,0.0000f,0.9333f,0.0833f,0.9333f,0.1667f,0.9333f,0.2500f,0.9333f,0.3333f,0.9333f,0.4167f,0.9333f,0.5000f,0.9333f,0.5833f,0.9333f,0.6667f,0.9333f,0.7500f,0.9333f,0.8333f,0.9333f,
	0.9167f,0.9333f,1.0000f,0.9333f,0.0000f,0.9667f,0.0833f,0.9667f,0.1667f,0.9667f,0.2500f,0.9667f,0.3333f,0.9667f,0.4167f,0.9667f,0.5000f,0.9667f,0.5833f,0.9667f,0.6667f,0.9667f,0.7500f,0.9667f,0.8333f,0.9667f,0.9167f,0.9667f,1.0000f,0.9667f,
	0.0000f,1.0000f,0.0833f,1.0000f,0.1667f,1.0000f,0.2500f,1.0000f,0.3333f,1.0000f,0.4167f,1.0000f,0.5000f,1.0000f,0.5833f,1.0000f,0.6667f,1.0000f,0.7500f,1.0000f,0.8333f,1.0000f,0.9167f,1.0000f,1.0000f,1.0000f,0.0000f,0.0333f,0.0833f,0.0333f,
	0.0000f,0.0000f,0.0833f,0.0000f,0.1667f,0.0333f,0.1667f,0.0000f,0.2500f,0.0333f,0.2500f,0.0000f,0.3333f,0.0333f,0.3333f,0.0000f,0.4167f,0.0333f,0.4167f,0.0000f,0.5000f,0.0333f,0.5000f,0.0000f,0.5833f,0.0333f,0.5833f,0.0000f,0.6667f,0.0333f,
	0.6667f,0.0000f,0.7500f,0.0333f,0.7500f,0.0000f,0.8333f,0.0333f,0.8333f,0.0000f,0.9167f,0.0333f,0.9167f,0.0000f,1.0000f,0.0333f,1.0000f,0.0000f,0.0000f,0.0667f,0.0833f,0.0667f,0.1667f,0.0667f,0.2500f,0.0667f,0.3333f,0.0667f,0.4167f,0.0667f,
	0.5000f,0.0667f,0.5833f,0.0667f,0.6667f,0.0667f,0.7500f,0.0667f,0.8333f,0.0667f,0.9167f,0.0667f,1.0000f,0.0667f,0.0000f,0.1000f,0.0833f,0.1000f,0.1667f,0.1000f,0.2500f,0.1000f,0.3333f,0.1000f,0.4167f,0.1000f,0.5000f,0.1000f,0.5833f,0.1000f,
	0.6667f,0.1000f,0.7500f,0.1000f,0.8333f,0.1000f,0.9167f,0.1000f,1.0000f,0.1000f,0.0000f,0.1333f,0.0833f,0.1333f,0.1667f,0.1333f,0.2500f,0.1333f,0.3333f,0.1333f,0.4167f,0.1333f,0.5000f,0.1333f,0.5833f,0.1333f,0.6667f,0.1333f,0.7500f,0.1333f,
	0.8333f,0.1333f,0.9167f,0.1333f,1.0000f,0.1333f,0.0000f,0.1667f,0.0833f,0.1667f,0.1667f,0.1667f,0.2500f,0.1667f,0.3333f,0.1667f,0.4167f,0.1667f,0.5000f,0.1667f,0.5833f,0.1667f,0.6667f,0.1667f,0.7500f,0.1667f,0.8333f,0.1667f,0.9167f,0.1667f,
	1.0000f,0.1667f,0.0000f,0.2000f,0.0833f,0.2000f,0.1667f,0.2000f,0.2500f,0.2000f,0.3333f,0.2000f,0.4167f,0.2000f,0.5000f,0.2000f,0.5833f,0.2000f,0.6667f,0.2000f,0.7500f,0.2000f,0.8333f,0.2000f,0.9167f,0.2000f,1.0000f,0.2000f,0.0000f,0.2333f,
	0.0833f,0.2333f,0.1667f,0.2333f,0.2500f,0.2333f,0.3333f,0.2333f,0.4167f,0.2333f,0.5000f,0.2333f,0.5833f,0.2333f,0.6667f,0.2333f,0.7500f,0.2333f,0.8333f,0.2333f,0.9167f,0.2333f,1.0000f,0.2333f,0.0000f,0.2667f,0.0833f,0.2667f,0.1667f,0.2667f,
	0.2500f,0.2667f,0.3333f,0.2667f,0.4167f,0.2667f,0.5000f,0.2667f,0.5833f,0.2667f,0.6667f,0.2667f,0.7500f,0.2667f,0.8333f,0.2667f,0.9167f,0.2667f,1.0000f,0.2667f,0.0000f,0.3000f,0.0833f,0.3000f,0.1667f,0.3000f,0.2500f,0.3000f,0.3333f,0.3000f,
	0.4167f,0.3000f,0.5000f,0.3000f,0.5833f,0.3000f,0.6667f,0.3000f,0.7500f,0.3000f,0.8333f,0.3000f,0.9167f,0.3000f,1.0000f,0.3000f,0.0000f,0.3333f,0.0833f,0.3333f,0.1667f,0.3333f,0.2500f,0.3333f,0.3333f,0.3333f,0.4167f,0.3333f,0.5000f,0.3333f,
	0.5833f,0.3333f,0.6667f,0.3333f,0.7500f,0.3333f,0.8333f,0.3333f,0.9167f,0.3333f,1.0000f,0.3333f,0.0000f,0.3667f,0.0833f,0.3667f,0.1667f,0.3667f,0.2500f,0.3667f,0.3333f,0.3667f,0.4167f,0.3667f,0.5000f,0.3667f,0.5833f,0.3667f,0.6667f,0.3667f,
	0.7500f,0.3667f,0.8333f,0.3667f,0.9167f,0.3667f,1.0000f,0.3667f,0.0000f,0.4000f,0.0833f,0.4000f,0.1667f,0.4000f,0.2500f,0.4000f,0.3333f,0.4000f,0.4167f,0.4000f,0.5000f,0.4000f,0.5833f,0.4000f,0.6667f,0.4000f,0.7500f,0.4000f,0.8333f,0.4000f,
	0.9167f,0.4000f,1.0000f,0.4000f,0.0000f,0.4333f,0.0833f,0.4333f,0.1667f,0.4333f,0.2500f,0.4333f,0.3333f,0.4333f,0.4167f,0.4333f,0.5000f,0.4333f,0.5833f,0.4333f,0.6667f,0.4333f,0.7500f,0.4333f,0.8333f,0.4333f,0.9167f,0.4333f,1.0000f,0.4333f,
	0.0000f,0.4667f,0.0833f,0.4667f,0.1667f,0.4667f,0.2500f,0.4667f,0.3333f,0.4667f,0.4167f,0.4667f,0.5000f,0.4667f,0.5833f,0.4667f,0.6667f,0.4667f,0.7500f,0.4667f,0.8333f,0.4667f,0.9167f,0.4667f,1.0000f,0.4667f,0.0000f,0.5000f,0.0833f,0.5000f,
	0.1667f,0.5000f,0.2500f,0.5000f,0.3333f,0.5000f,0.4167f,0.5000f,0.5000f,0.5000f,0.5833f,0.5000f,0.6667f,0.5000f,0.7500f,0.5000f,0.8333f,0.5000f,0.9167f,0.5000f,1.0000f,0.5000f,0.0000f,0.5333f,0.0833f,0.5333f,0.1667f,0.5333f,0.2500f,0.5333f,
	0.3333f,0.5333f,0.4167f,0.5333f,0.5000f,0.5333f,0.5833f,0.5333f,0.6667f,0.5333f,0.7500f,0.5333f,0.8333f,0.5333f,0.9167f,0.5333f,1.0000f,0.5333f,0.0000f,0.5667f,0.0833f,0.5667f,0.1667f,0.5667f,0.2500f,0.5667f,0.3333f,0.5667f,0.4167f,0.5667f,
	0.5000f,0.5667f,0.5833f,0.5667f,0.6667f,0.5667f,0.7500f,0.5667f,0.8333f,0.5667f,0.9167f,0.5667f,1.0000f,0.5667f,0.0000f,0.6000f,0.0833f,0.6000f,0.1667f,0.6000f,0.2500f,0.6000f,0.3333f,0.6000f,0.4167f,0.6000f,0.5000f,0.6000f,0.5833f,0.6000f,
	0.6667f,0.6000f,0.7500f,0.6000f,0.8333f,0.6000f,0.9167f,0.6000f,1.0000f,0.6000f,0.0000f,0.6333f,0.0833f,0.6333f,0.1667f,0.6333f,0.2500f,0.6333f,0.3333f,0.6333f,0.4167f,0.6333f,0.5000f,0.6333f,0.5833f,0.6333f,0.6667f,0.6333f,0.7500f,0.6333f,
	0.8333f,0.6333f,0.9167f,0.6333f,1.0000f,0.6333f,0.0000f,0.6667f,0.0833f,0.6667f,0.1667f,0.6667f,0.2500f,0.6667f,0.3333f,0.6667f,0.4167f,0.6667f,0.5000f,0.6667f,0.5833f,0.6667f,0.6667f,0.6667f,0.7500f,0.6667f,0.8333f,0.6667f,0.9167f,0.6667f,
	1.0000f,0.6667f,0.0000f,0.7000f,0.0833f,0.7000f,0.1667f,0.7000f,0.2500f,0.7000f,0.3333f,0.7000f,0.4167f,0.7000f,0.5000f,0.7000f,0.5833f,0.7000f,0.6667f,0.7000f,0.7500f,0.7000f,0.8333f,0.7000f,0.9167f,0.7000f,1.0000f,0.7000f,0.0000f,0.7333f,
	0.0833f,0.7333f,0.1667f,0.7333f,0.2500f,0.7333f,0.3333f,0.7333f,0.4167f,0.7333f,0.5000f,0.7333f,0.5833f,0.7333f,0.6667f,0.7333f,0.7500f,0.7333f,0.8333f,0.7333f,0.9167f,0.7333f,1.0000f,0.7333f,0.0000f,0.7667f,0.0833f,0.7667f,0.1667f,0.7667f,
	0.2500f,0.7667f,0.3333f,0.7667f,0.4167f,0.7667f,0.5000f,0.7667f,0.5833f,0.7667f,0.6667f,0.7667f,0.7500f,0.7667f,0.8333f,0.7667f,0.9167f,0.7667f,1.0000f,0.7667f,0.0000f,0.8000f,0.0833f,0.8000f,0.1667f,0.8000f,0.2500f,0.8000f,0.3333f,0.8000f,
	0.4167f,0.8000f,0.5000f,0.8000f,0.5833f,0.8000f,0.6667f,0.8000f,0.7500f,0.8000f,0.8333f,0.8000f,0.9167f,0.8000f,1.0000f,0.8000f,0.0000f,0.8333f,0.0833f,0.8333f,0.1667f,0.8333f,0.2500f,0.8333f,0.3333f,0.8333f,0.4167f,0.8333f,0.5000f,0.8333f,
	0.5833f,0.8333f,0.6667f,0.8333f,0.7500f,0.8333f,0.8333f,0.8333f,0.9167f,0.8333f,1.0000f,0.8333f,0.0000f,0.8667f,0.0833f,0.8667f,0.1667f,0.8667f,0.2500f,0.8667f,0.3333f,0.8667f,0.4167f,0.8667f,0.5000f,0.8667f,0.5833f,0.8667f,0.6667f,0.8667f,
	0.7500f,0.8667f,0.8333f,0.8667f,0.9167f,0.8667f,1.0000f,0.8667f,0.0000f,0.9000f,0.0833f,0.9000f,0.1667f,0.9000f,0.2500f,0.9000f,0.3333f,0.9000f,0.4167f,0.9000f,0.5000f,0.9000f,0.5833f,0.9000f,0.6667f,0.9000f,0.7500f,0.9000f,0.8333f,0.9000f,
	0.9167f,0.9000f,1.0000f,0.9000f,0.0000f,0.9333f,0.0833f,0.9333f,0.1667f,0.9333f,0.2500f,0.9333f,0.3333f,0.9333f,0.4167f,0.9333f,0.5000f,0.9333f,0.5833f,0.9333f,0.6667f,0.9333f,0.7500f,0.9333f,0.8333f,0.9333f,0.9167f,0.9333f,1.0000f,0.9333f,
	0.0000f,0.9667f,0.0833f,0.9667f,0.1667f,0.9667f,0.2500f,0.9667f,0.3333f,0.9667f,0.4167f,0.9667f,0.5000f,0.9667f,0.5833f,0.9667f,0.6667f,0.9667f,0.7500f,0.9667f,0.8333f,0.9667f,0.9167f,0.9667f,1.0000f,0.9667f,0.0000f,1.0000f,0.0833f,1.0000f,
	0.1667f,1.0000f,0.2500f,1.0000f,0.3333f,1.0000f,0.4167f,1.0000f,0.5000f,1.0000f,0.5833f,1.0000f,0.6667f,1.0000f,0.7500f,1.0000f,0.8333f,1.0000f,0.9167f,1.0000f,1.0000f,1.0000f,0.0000f,0.0333f,0.0833f,0.0333f,0.0000f,0.0000f,0.0833f,0.0000f,
	0.1667f,0.0333f,0.1667f,0.0000f,0.2500f,0.0333f,0.2500f,0.0000f,0.3333f,0.0333f,0.3333f,0.0000f,0.4167f,0.0333f,0.4167f,0.0000f,0.5000f,0.0333f,0.5000f,0.0000f,0.5833f,0.0333f,0.5833f,0.0000f,0.6667f,0.0333f,0.6667f,0.0000f,0.7500f,0.0333f,
	0.7500f,0.0000f,0.8333f,0.0333f,0.8333f,0.0000f,0.9167f,0.0333f,0.9167f,0.0000f,1.0000f,0.0333f,1.0000f,0.0000f,0.0000f,0.0667f,0.0833f,0.0667f,0.1667f,0.0667f,0.2500f,0.0667f,0.3333f,0.0667f,0.4167f,0.0667f,0.5000f,0.0667f,0.5833f,0.0667f,
	0.6667f,0.0667f,0.7500f,0.0667f,0.8333f,0.0667f,0.9167f,0.0667f,1.0000f,0.0667f,0.0000f,0.1000f,0.0833f,0.1000f,0.1667f,0.1000f,0.2500f,0.1000f,0.3333f,0.1000f,0.4167f,0.1000f,0.5000f,0.1000f,0.5833f,0.1000f,0.6667f,0.1000f,0.7500f,0.1000f,
	0.8333f,0.1000f,0.9167f,0.1000f,1.0000f,0.1000f,0.0000f,0.1333f,0.0833f,0.1333f,0.1667f,0.1333f,0.2500f,0.1333f,0.3333f,0.1333f,0.4167f,0.1333f,0.5000f,0.1333f,0.5833f,0.1333f,0.6667f,0.1333f,0.7500f,0.1333f,0.8333f,0.1333f,0.9167f,0.1333f,
	1.0000f,0.1333f,0.0000f,0.1667f,0.0833f,0.1667f,0.1667f,0.1667f,0.2500f,0.1667f,0.3333f,0.1667f,0.4167f,0.1667f,0.5000f,0.1667f,0.5833f,0.1667f,0.6667f,0.1667f,0.7500f,0.1667f,0.8333f,0.1667f,0.9167f,0.1667f,1.0000f,0.1667f,0.0000f,0.2000f,
	0.0833f,0.2000f,0.1667f,0.2000f,0.2500f,0.2000f,0.3333f,0.2000f,0.4167f,0.2000f,0.5000f,0.2000f,0.5833f,0.2000f,0.6667f,0.2000f,0.7500f,0.2000f,0.8333f,0.2000f,0.9167f,0.2000f,1.0000f,0.2000f,0.0000f,0.2333f,0.0833f,0.2333f,0.1667f,0.2333f,
	0.2500f,0.2333f,0.3333f,0.2333f,0.4167f,0.2333f,0.5000f,0.2333f,0.5833f,0.2333f,0.6667f,0.2333f,0.7500f,0.2333f,0.8333f,0.2333f,0.9167f,0.2333f,1.0000f,0.2333f,0.0000f,0.2667f,0.0833f,0.2667f,0.1667f,0.2667f,0.2500f,0.2667f,0.3333f,0.2667f,
	0.4167f,0.2667f,0.5000f,0.2667f,0.5833f,0.2667f,0.6667f,0.2667f,0.7500f,0.2667f,0.8333f,0.2667f,0.9167f,0.2667f,1.0000f,0.2667f,0.0000f,0.3000f,0.0833f,0.3000f,0.1667f,0.3000f,0.2500f,0.3000f,0.3333f,0.3000f,0.4167f,0.3000f,0.5000f,0.3000f,
	0.5833f,0.3000f,0.6667f,0.3000f,0.7500f,0.3000f,0.8333f,0.3000f,0.9167f,0.3000f,1.0000f,0.3000f,0.0000f,0.3333f,0.0833f,0.3333f,0.1667f,0.3333f,0.2500f,0.3333f,0.3333f,0.3333f,0.4167f,0.3333f,0.5000f,0.3333f,0.5833f,0.3333f,0.6667f,0.3333f,
	0.7500f,0.3333f,0.8333f,0.3333f,0.9167f,0.3333f,1.0000f,0.3333f,0.0000f,0.3667f,0.0833f,0.3667f,0.1667f,0.3667f,0.2500f,0.3667f,0.3333f,0.3667f,0.4167f,0.3667f,0.5000f,0.3667f,0.5833f,0.3667f,0.6667f,0.3667f,0.7500f,0.3667f,0.8333f,0.3667f,
	0.9167f,0.3667f,1.0000f,0.3667f,0.0000f,0.4000f,0.0833f,0.4000f,0.1667f,0.4000f,0.2500f,0.4000f,0.3333f,0.4000f,0.4167f,0.4000f,0.5000f,0.4000f,0.5833f,0.4000f,0.6667f,0.4000f,0.7500f,0.4000f,0.8333f,0.4000f,0.9167f,0.4000f,1.0000f,0.4000f,
	0.0000f,0.4333f,0.0833f,0.4333f,0.1667f,0.4333f,0.2500f,0.4333f,0.3333f,0.4333f,0.4167f,0.4333f,0.5000f,0.4333f,0.5833f,0.4333f,0.6667f,0.4333f,0.7500f,0.4333f,0.8333f,0.4333f,0.9167f,0.4333f,1.0000f,0.4333f,0.0000f,0.4667f,0.0833f,0.4667f,
	0.1667f,0.4667f,0.2500f,0.4667f,0.3333f,0.4667f,0.4167f,0.4667f,0.5000f,0.4667f,0.5833f,0.4667f,0.6667f,0.4667f,0.7500f,0.4667f,0.8333f,0.4667f,0.9167f,0.4667f,1.0000f,0.4667f,0.0000f,0.5000f,0.0833f,0.5000f,0.1667f,0.5000f,0.2500f,0.5000f,
	0.3333f,0.5000f,0.4167f,0.5000f,0.5000f,0.5000f,0.5833f,0.5000f,0.6667f,0.5000f,0.7500f,0.5000f,0.8333f,0.5000f,0.9167f,0.5000f,1.0000f,0.5000f,0.0000f,0.5333f,0.0833f,0.5333f,0.1667f,0.5333f,0.2500f,0.5333f,0.3333f,0.5333f,0.4167f,0.5333f,
	0.5000f,0.5333f,0.5833f,0.5333f,0.6667f,0.5333f,0.7500f,0.5333f,0.8333f,0.5333f,0.9167f,0.5333f,1.0000f,0.5333f,0.0000f,0.5667f,0.0833f,0.5667f,0.1667f,0.5667f,0.2500f,0.5667f,0.3333f,0.5667f,0.4167f,0.5667f,0.5000f,0.5667f,0.5833f,0.5667f,
	0.6667f,0.5667f,0.7500f,0.5667f,0.8333f,0.5667f,0.9167f,0.5667f,1.0000f,0.5667f,0.0000f,0.6000f,0.0833f,0.6000f,0.1667f,0.6000f,0.2500f,0.6000f,0.3333f,0.6000f,0.4167f,0.6000f,0.5000f,0.6000f,0.5833f,0.6000f,0.6667f,0.6000f,0.7500f,0.6000f,
	0.8333f,0.6000f,0.9167f,0.6000f,1.0000f,0.6000f,0.0000f,0.6333f,0.0833f,0.6333f,0.1667f,0.6333f,0.2500f,0.6333f,0.3333f,0.6333f,0.4167f,0.6333f,0.5000f,0.6333f,0.5833f,0.6333f,0.6667f,0.6333f,0.7500f,0.6333f,0.8333f,0.6333f,0.9167f,0.6333f,
	1.0000f,0.6333f,0.0000f,0.6667f,0.0833f,0.6667f,0.1667f,0.6667f,0.2500f,0.6667f,0.3333f,0.6667f,0.4167f,0.6667f,0.5000f,0.6667f,0.5833f,0.6667f,0.6667f,0.6667f,0.7500f,0.6667f,0.8333f,0.6667f,0.9167f,0.6667f,1.0000f,0.6667f,0.0000f,0.7000f,
	0.0833f,0.7000f,0.1667f,0.7000f,0.2500f,0.7000f,0.3333f,0.7000f,0.4167f,0.7000f,0.5000f,0.7000f,0.5833f,0.7000f,0.6667f,0.7000f,0.7500f,0.7000f,0.8333f,0.7000f,0.9167f,0.7000f,1.0000f,0.7000f,0.0000f,0.7333f,0.0833f,0.7333f,0.1667f,0.7333f,
	0.2500f,0.7333f,0.3333f,0.7333f,0.4167f,0.7333f,0.5000f,0.7333f,0.5833f,0.7333f,0.6667f,0.7333f,0.7500f,0.7333f,0.8333f,0.7333f,0.9167f,0.7333f,1.0000f,0.7333f,0.0000f,0.7667f,0.0833f,0.7667f,0.1667f,0.7667f,0.2500f,0.7667f,0.3333f,0.7667f,
	0.4167f,0.7667f,0.5000f,0.7667f,0.5833f,0.7667f,0.6667f,0.7667f,0.7500f,0.7667f,0.8333f,0.7667f,0.9167f,0.7667f,1.0000f,0.7667f,0.0000f,0.8000f,0.0833f,0.8000f,0.1667f,0.8000f,0.2500f,0.8000f,0.3333f,0.8000f,0.4167f,0.8000f,0.5000f,0.8000f,
	0.5833f,0.8000f,0.6667f,0.8000f,0.7500f,0.8000f,0.8333f,0.8000f,0.9167f,0.8000f,1.0000f,0.8000f,0.0000f,0.8333f,0.0833f,0.8333f,0.1667f,0.8333f,0.2500f,0.8333f,0.3333f,0.8333f,0.4167f,0.8333f,0.5000f,0.8333f,0.5833f,0.8333f,0.6667f,0.8333f,
	0.7500f,0.8333f,0.8333f,0.8333f,0.9167f,0.8333f,1.0000f,0.8333f,0.0000f,0.8667f,0.0833f,0.8667f,0.1667f,0.8667f,0.2500f,0.8667f,0.3333f,0.8667f,0.4167f,0.8667f,0.5000f,0.8667f,0.5833f,0.8667f,0.6667f,0.8667f,0.7500f,0.8667f,0.8333f,0.8667f,
	0.9167f,0.8667f,1.0000f,0.8667f,0.0000f,0.9000f,0.0833f,0.9000f,0.1667f,0.9000f,0.2500f,0.9000f,0.3333f,0.9000f,0.4167f,0.9000f,0.5000f,0.9000f,0.5833f,0.9000f,0.6667f,0.9000f,0.7500f,0.9000f,0.8333f,0.9000f,0.9167f,0.9000f,1.0000f,0.9000f,
	0.0000f,0.9333f,0.0833f,0.9333f,0.1667f,0.9333f,0.2500f,0.9333f,0.3333f,0.9333f,0.4167f,0.9333f,0.5000f,0.9333f,0.5833f,0.9333f,0.6667f,0.9333f,0.7500f,0.9333f,0.8333f,0.9333f,0.9167f,0.9333f,1.0000f,0.9333f,0.0000f,0.9667f,0.0833f,0.9667f,
	0.1667f,0.9667f,0.2500f,0.9667f,0.3333f,0.9667f,0.4167f,0.9667f,0.5000f,0.9667f,0.5833f,0.9667f,0.6667f,0.9667f,0.7500f,0.9667f,0.8333f,0.9667f,0.9167f,0.9667f,1.0000f,0.9667f,0.0000f,1.0000f,0.0833f,1.0000f,0.1667f,1.0000f,0.2500f,1.0000f,
	0.3333f,1.0000f,0.4167f,1.0000f,0.5000f,1.0000f,0.5833f,1.0000f,0.6667f,1.0000f,0.7500f,1.0000f,0.8333f,1.0000f,0.9167f,1.0000f,1.0000f,1.0000f,0.7500f,0.1667f,0.6250f,0.1667f,0.7500f,0.0000f,0.6250f,0.0000f,0.5000f,0.1667f,0.5000f,0.0000f,
	0.3750f,0.1667f,0.3750f,0.0000f,0.2500f,0.1667f,0.2500f,0.0000f,0.1250f,0.1667f,0.1250f,0.0000f,-0.0000f,0.1667f,-0.0000f,0.0000f,1.0000f,0.1667f,0.8750f,0.1667f,1.0000f,0.0000f,0.8750f,0.0000f,0.7500f,0.3333f,0.6250f,0.3333f,0.5000f,0.3333f,
	0.3750f,0.3333f,0.2500f,0.3333f,0.1250f,0.3333f,-0.0000f,0.3333f,1.0000f,0.3333f,0.8750f,0.3333f,0.7500f,0.5000f,0.6250f,0.5000f,0.5000f,0.5000f,0.3750f,0.5000f,0.2500f,0.5000f,0.1250f,0.5000f,-0.0000f,0.5000f,1.0000f,0.5000f,0.8750f,0.5000f,
	0.7500f,0.6667f,0.6250f,0.6667f,0.5000f,0.6667f,0.3750f,0.6667f,0.2500f,0.6667f,0.1250f,0.6667f,-0.0000f,0.6667f,1.0000f,0.6667f,0.8750f,0.6667f,0.7500f,0.8333f,0.6250f,0.8333f,0.5000f,0.8333f,0.3750f,0.8333f,0.2500f,0.8333f,0.1250f,0.8333f,
	-0.0000f,0.8333f,1.0000f,0.8333f,0.8750f,0.8333f,0.7500f,1.0000f,0.6250f,1.0000f,0.5000f,1.0000f,0.3750f,1.0000f,0.2500f,1.0000f,0.1250f,1.0000f,-0.0000f,1.0000f,1.0000f,1.0000f,0.8750f,1.0000f,0.7500f,0.1667f,0.6250f,0.1667f,0.7500f,0.0000f,
	0.6250f,0.0000f,0.5000f,0.1667f,0.5000f,0.0000f,0.3750f,0.1667f,0.3750f,0.0000f,0.2500f,0.1667f,0.2500f,0.0000f,0.1250f,0.1667f,0.1250f,0.0000f,-0.0000f,0.1667f,-0.0000f,0.0000f,1.0000f,0.1667f,0.8750f,0.1667f,1.0000f,0.0000f,0.8750f,0.0000f,
	0.7500f,0.3333f,0.6250f,0.3333f,0.5000f,0.3333f,0.3750f,0.3333f,0.2500f,0.3333f,0.1250f,0.3333f,-0.0000f,0.3333f,1.0000f,0.3333f,0.8750f,0.3333f,0.7500f,0.5000f,0.6250f,0.5000f,0.5000f,0.5000f,0.3750f,0.5000f,0.2500f,0.5000f,0.1250f,0.5000f,
	-0.0000f,0.5000f,1.0000f,0.5000f,0.8750f,0.5000f,0.7500f,0.6667f,0.6250f,0.6667f,0.5000f,0.6667f,0.3750f,0.6667f,0.2500f,0.6667f,0.1250f,0.6667f,-0.0000f,0.6667f,1.0000f,0.6667f,0.8750f,0.6667f,0.7500f,0.8333f,0.6250f,0.8333f,0.5000f,0.8333f,
	0.3750f,0.8333f,0.2500f,0.8333f,0.1250f,0.8333f,-0.0000f,0.8333f,1.0000f,0.8333f,0.8750f,0.8333f,0.7500f,1.0000f,0.6250f,1.0000f,0.5000f,1.0000f,0.3750f,1.0000f,0.2500f,1.0000f,0.1250f,1.0000f,-0.0000f,1.0000f,1.0000f,1.0000f,0.8750f,1.0000f,
	
_NULL_ }; /* End of silver_UV */

const float silver_Normals[] = {
	0.2079f,-0.0000f,-0.9781f, 0.1809f,0.4927f,-0.8512f, -0.0000f,0.0000f,-1.0000f, 0.0000f,0.4927f,-0.8702f, 0.1066f,0.8586f,-0.5015f, -0.0000f,0.8586f,-0.5127f, 0.0036f,0.9999f,-0.0169f, 0.0000f,0.9999f,-0.0172f, -0.1012f,0.8735f,0.4762f, 0.0000f,0.8735f,0.4868f, -0.1791f,0.5076f,0.8428f, 0.0000f,0.5076f,0.8616f, -0.2079f,-0.0000f,0.9781f, -0.0000f,-0.0000f,1.0000f, -0.1791f,-0.5076f,0.8428f, 
	0.0000f,-0.5076f,0.8616f, -0.1012f,-0.8735f,0.4762f, 0.0000f,-0.8735f,0.4868f, 0.0036f,-0.9999f,-0.0169f, 0.0000f,-0.9999f,-0.0173f, 0.1066f,-0.8586f,-0.5015f, 0.0000f,-0.8586f,-0.5127f, 0.1809f,-0.4927f,-0.8512f, -0.0000f,-0.4927f,-0.8702f, 0.2079f,-0.0000f,-0.9781f, -0.0000f,0.0000f,-1.0000f, 0.4067f,-0.0000f,-0.9135f, 0.3539f,0.4927f,-0.7950f, 0.2085f,0.8586f,-0.4684f, 0.0070f,0.9999f,-0.0158f, 
	-0.1980f,0.8735f,0.4448f, -0.3504f,0.5076f,0.7871f, -0.4067f,-0.0000f,0.9135f, -0.3504f,-0.5076f,0.7871f, -0.1980f,-0.8735f,0.4448f, 0.0070f,-0.9999f,-0.0158f, 0.2085f,-0.8586f,-0.4684f, 0.3539f,-0.4927f,-0.7950f, 0.4067f,-0.0000f,-0.9135f, 0.5878f,0.0000f,-0.8090f, 0.5115f,0.4927f,-0.7040f, 0.3014f,0.8586f,-0.4148f, 0.0101f,0.9999f,-0.0140f, -0.2862f,0.8735f,0.3939f, -0.5064f,0.5076f,0.6970f, 
	-0.5878f,-0.0000f,0.8090f, -0.5064f,-0.5076f,0.6970f, -0.2862f,-0.8735f,0.3939f, 0.0101f,-0.9999f,-0.0140f, 0.3014f,-0.8586f,-0.4148f, 0.5115f,-0.4927f,-0.7040f, 0.5878f,0.0000f,-0.8090f, 0.7431f,-0.0000f,-0.6691f, 0.6467f,0.4927f,-0.5823f, 0.3810f,0.8586f,-0.3431f, 0.0128f,0.9999f,-0.0115f, -0.3618f,0.8735f,0.3258f, -0.6403f,0.5076f,0.5765f, -0.7431f,-0.0000f,0.6691f, -0.6403f,-0.5076f,0.5765f, 
	-0.3618f,-0.8735f,0.3258f, 0.0128f,-0.9999f,-0.0115f, 0.3810f,-0.8586f,-0.3431f, 0.6467f,-0.4927f,-0.5823f, 0.7431f,-0.0000f,-0.6691f, 0.8660f,-0.0000f,-0.5000f, 0.7536f,0.4927f,-0.4351f, 0.4440f,0.8586f,-0.2564f, 0.0149f,0.9999f,-0.0086f, -0.4216f,0.8735f,0.2434f, -0.7462f,0.5076f,0.4308f, -0.8660f,-0.0000f,0.5000f, -0.7462f,-0.5076f,0.4308f, -0.4216f,-0.8735f,0.2434f, 0.0149f,-0.9999f,-0.0086f, 
	0.4440f,-0.8586f,-0.2564f, 0.7536f,-0.4927f,-0.4351f, 0.8660f,-0.0000f,-0.5000f, 0.9511f,0.0000f,-0.3090f, 0.8276f,0.4927f,-0.2689f, 0.4876f,0.8586f,-0.1584f, 0.0164f,0.9999f,-0.0053f, -0.4630f,0.8735f,0.1504f, -0.8194f,0.5076f,0.2662f, -0.9511f,-0.0000f,0.3090f, -0.8194f,-0.5076f,0.2662f, -0.4630f,-0.8735f,0.1504f, 0.0164f,-0.9999f,-0.0053f, 0.4876f,-0.8586f,-0.1584f, 0.8276f,-0.4927f,-0.2689f, 
	0.9511f,0.0000f,-0.3090f, 0.9945f,0.0000f,-0.1045f, 0.8654f,0.4927f,-0.0910f, 0.5099f,0.8586f,-0.0536f, 0.0172f,0.9999f,-0.0018f, -0.4842f,0.8735f,0.0509f, -0.8569f,0.5076f,0.0901f, -0.9945f,-0.0000f,0.1045f, -0.8569f,-0.5076f,0.0901f, -0.4842f,-0.8735f,0.0509f, 0.0172f,-0.9999f,-0.0018f, 0.5099f,-0.8586f,-0.0536f, 0.8654f,-0.4927f,-0.0910f, 0.9945f,0.0000f,-0.1045f, 0.9945f,-0.0000f,0.1045f, 
	0.8654f,0.4927f,0.0910f, 0.5099f,0.8586f,0.0536f, 0.0172f,0.9999f,0.0018f, -0.4842f,0.8735f,-0.0509f, -0.8569f,0.5076f,-0.0901f, -0.9945f,0.0000f,-0.1045f, -0.8569f,-0.5076f,-0.0901f, -0.4842f,-0.8735f,-0.0509f, 0.0172f,-0.9999f,0.0018f, 0.5099f,-0.8586f,0.0536f, 0.8654f,-0.4927f,0.0910f, 0.9945f,-0.0000f,0.1045f, 0.9511f,-0.0000f,0.3090f, 0.8276f,0.4927f,0.2689f, 0.4876f,0.8586f,0.1584f, 
	0.0164f,0.9999f,0.0053f, -0.4630f,0.8735f,-0.1504f, -0.8194f,0.5076f,-0.2662f, -0.9511f,0.0000f,-0.3090f, -0.8194f,-0.5076f,-0.2662f, -0.4630f,-0.8735f,-0.1504f, 0.0164f,-0.9999f,0.0053f, 0.4876f,-0.8586f,0.1584f, 0.8276f,-0.4927f,0.2689f, 0.9511f,-0.0000f,0.3090f, 0.8660f,-0.0000f,0.5000f, 0.7536f,0.4927f,0.4351f, 0.4440f,0.8586f,0.2564f, 0.0149f,0.9999f,0.0086f, -0.4216f,0.8735f,-0.2434f, 
	-0.7462f,0.5076f,-0.4308f, -0.8660f,-0.0000f,-0.5000f, -0.7462f,-0.5076f,-0.4308f, -0.4216f,-0.8735f,-0.2434f, 0.0149f,-0.9999f,0.0086f, 0.4440f,-0.8586f,0.2564f, 0.7536f,-0.4927f,0.4351f, 0.8660f,-0.0000f,0.5000f, 0.7431f,0.0000f,0.6691f, 0.6467f,0.4927f,0.5823f, 0.3810f,0.8586f,0.3431f, 0.0128f,0.9999f,0.0115f, -0.3618f,0.8735f,-0.3258f, -0.6403f,0.5076f,-0.5765f, -0.7431f,-0.0000f,-0.6691f, 
	-0.6403f,-0.5076f,-0.5765f, -0.3618f,-0.8735f,-0.3258f, 0.0128f,-0.9999f,0.0115f, 0.3810f,-0.8586f,0.3431f, 0.6467f,-0.4927f,0.5823f, 0.7431f,0.0000f,0.6691f, 0.5878f,0.0000f,0.8090f, 0.5115f,0.4927f,0.7040f, 0.3014f,0.8586f,0.4148f, 0.0101f,0.9999f,0.0140f, -0.2862f,0.8735f,-0.3939f, -0.5064f,0.5076f,-0.6970f, -0.5878f,-0.0000f,-0.8090f, -0.5064f,-0.5076f,-0.6970f, -0.2862f,-0.8735f,-0.3939f, 
	0.0101f,-0.9999f,0.0140f, 0.3014f,-0.8586f,0.4148f, 0.5115f,-0.4927f,0.7040f, 0.5878f,0.0000f,0.8090f, 0.4067f,0.0000f,0.9135f, 0.3539f,0.4927f,0.7950f, 0.2085f,0.8586f,0.4684f, 0.0070f,0.9999f,0.0158f, -0.1980f,0.8735f,-0.4448f, -0.3504f,0.5076f,-0.7871f, -0.4067f,-0.0000f,-0.9135f, -0.3504f,-0.5076f,-0.7871f, -0.1980f,-0.8735f,-0.4448f, 0.0070f,-0.9999f,0.0158f, 0.2085f,-0.8586f,0.4684f, 
	0.3539f,-0.4927f,0.7950f, 0.4067f,0.0000f,0.9135f, 0.2079f,-0.0000f,0.9781f, 0.1809f,0.4927f,0.8512f, 0.1066f,0.8586f,0.5015f, 0.0036f,0.9999f,0.0169f, -0.1012f,0.8735f,-0.4762f, -0.1791f,0.5076f,-0.8428f, -0.2079f,-0.0000f,-0.9781f, -0.1791f,-0.5076f,-0.8428f, -0.1012f,-0.8735f,-0.4762f, 0.0036f,-0.9999f,0.0169f, 0.1066f,-0.8586f,0.5015f, 0.1809f,-0.4927f,0.8512f, 0.2079f,-0.0000f,0.9781f, 
	-0.0000f,-0.0000f,1.0000f, -0.0000f,0.4927f,0.8702f, -0.0000f,0.8586f,0.5127f, -0.0000f,0.9999f,0.0172f, 0.0000f,0.8735f,-0.4868f, 0.0000f,0.5076f,-0.8616f, 0.0000f,-0.0000f,-1.0000f, 0.0000f,-0.5076f,-0.8616f, 0.0000f,-0.8735f,-0.4868f, -0.0000f,-0.9999f,0.0173f, -0.0000f,-0.8586f,0.5127f, 0.0000f,-0.4927f,0.8702f, -0.0000f,-0.0000f,1.0000f, -0.2079f,-0.0000f,0.9781f, -0.1809f,0.4927f,0.8512f, 
	-0.1066f,0.8586f,0.5015f, -0.0036f,0.9999f,0.0169f, 0.1012f,0.8735f,-0.4762f, 0.1791f,0.5076f,-0.8428f, 0.2079f,-0.0000f,-0.9781f, 0.1791f,-0.5076f,-0.8428f, 0.1012f,-0.8735f,-0.4762f, -0.0036f,-0.9999f,0.0169f, -0.1066f,-0.8586f,0.5015f, -0.1809f,-0.4927f,0.8512f, -0.2079f,-0.0000f,0.9781f, -0.4067f,0.0000f,0.9135f, -0.3539f,0.4927f,0.7950f, -0.2085f,0.8586f,0.4684f, -0.0070f,0.9999f,0.0158f, 
	0.1980f,0.8735f,-0.4448f, 0.3504f,0.5076f,-0.7871f, 0.4067f,-0.0000f,-0.9135f, 0.3504f,-0.5076f,-0.7871f, 0.1980f,-0.8735f,-0.4448f, -0.0070f,-0.9999f,0.0158f, -0.2085f,-0.8586f,0.4684f, -0.3539f,-0.4927f,0.7950f, -0.4067f,0.0000f,0.9135f, -0.5878f,0.0000f,0.8090f, -0.5115f,0.4927f,0.7040f, -0.3014f,0.8586f,0.4148f, -0.0101f,0.9999f,0.0140f, 0.2862f,0.8735f,-0.3939f, 0.5064f,0.5076f,-0.6970f, 
	0.5878f,-0.0000f,-0.8090f, 0.5064f,-0.5076f,-0.6970f, 0.2862f,-0.8735f,-0.3939f, -0.0101f,-0.9999f,0.0140f, -0.3014f,-0.8586f,0.4148f, -0.5115f,-0.4927f,0.7040f, -0.5878f,0.0000f,0.8090f, -0.7431f,0.0000f,0.6691f, -0.6467f,0.4927f,0.5823f, -0.3810f,0.8586f,0.3431f, -0.0128f,0.9999f,0.0115f, 0.3618f,0.8735f,-0.3258f, 0.6403f,0.5076f,-0.5765f, 0.7431f,-0.0000f,-0.6691f, 0.6403f,-0.5076f,-0.5765f, 
	0.3618f,-0.8735f,-0.3258f, -0.0128f,-0.9999f,0.0115f, -0.3810f,-0.8586f,0.3431f, -0.6467f,-0.4927f,0.5823f, -0.7431f,0.0000f,0.6691f, -0.8660f,-0.0000f,0.5000f, -0.7536f,0.4927f,0.4351f, -0.4440f,0.8586f,0.2564f, -0.0149f,0.9999f,0.0086f, 0.4216f,0.8735f,-0.2434f, 0.7462f,0.5076f,-0.4308f, 0.8660f,-0.0000f,-0.5000f, 0.7462f,-0.5076f,-0.4308f, 0.4216f,-0.8735f,-0.2434f, -0.0149f,-0.9999f,0.0086f, 
	-0.4440f,-0.8586f,0.2564f, -0.7536f,-0.4927f,0.4351f, -0.8660f,-0.0000f,0.5000f, -0.9511f,-0.0000f,0.3090f, -0.8276f,0.4927f,0.2689f, -0.4876f,0.8586f,0.1584f, -0.0164f,0.9999f,0.0053f, 0.4630f,0.8735f,-0.1504f, 0.8194f,0.5076f,-0.2662f, 0.9511f,-0.0000f,-0.3090f, 0.8194f,-0.5076f,-0.2662f, 0.4630f,-0.8735f,-0.1504f, -0.0164f,-0.9999f,0.0053f, -0.4876f,-0.8586f,0.1584f, -0.8276f,-0.4927f,0.2689f, 
	-0.9511f,-0.0000f,0.3090f, -0.9945f,0.0000f,0.1045f, -0.8654f,0.4927f,0.0910f, -0.5099f,0.8586f,0.0536f, -0.0172f,0.9999f,0.0018f, 0.4842f,0.8735f,-0.0509f, 0.8569f,0.5076f,-0.0901f, 0.9945f,0.0000f,-0.1045f, 0.8569f,-0.5076f,-0.0901f, 0.4842f,-0.8735f,-0.0509f, -0.0172f,-0.9999f,0.0018f, -0.5099f,-0.8586f,0.0536f, -0.8654f,-0.4927f,0.0910f, -0.9945f,0.0000f,0.1045f, -0.9945f,0.0000f,-0.1045f, 
	-0.8654f,0.4927f,-0.0910f, -0.5099f,0.8586f,-0.0536f, -0.0172f,0.9999f,-0.0018f, 0.4842f,0.8735f,0.0509f, 0.8569f,0.5076f,0.0901f, 0.9945f,-0.0000f,0.1045f, 0.8569f,-0.5076f,0.0901f, 0.4842f,-0.8735f,0.0509f, -0.0172f,-0.9999f,-0.0018f, -0.5099f,-0.8586f,-0.0536f, -0.8654f,-0.4927f,-0.0910f, -0.9945f,0.0000f,-0.1045f, -0.9511f,0.0000f,-0.3090f, -0.8276f,0.4927f,-0.2689f, -0.4876f,0.8586f,-0.1584f, 
	-0.0164f,0.9999f,-0.0053f, 0.4630f,0.8735f,0.1504f, 0.8194f,0.5076f,0.2662f, 0.9511f,-0.0000f,0.3090f, 0.8194f,-0.5076f,0.2662f, 0.4630f,-0.8735f,0.1504f, -0.0164f,-0.9999f,-0.0053f, -0.4876f,-0.8586f,-0.1584f, -0.8276f,-0.4927f,-0.2689f, -0.9511f,0.0000f,-0.3090f, -0.8660f,-0.0000f,-0.5000f, -0.7536f,0.4927f,-0.4351f, -0.4440f,0.8586f,-0.2564f, -0.0149f,0.9999f,-0.0086f, 0.4216f,0.8735f,0.2434f, 
	0.7462f,0.5076f,0.4308f, 0.8660f,-0.0000f,0.5000f, 0.7462f,-0.5076f,0.4308f, 0.4216f,-0.8735f,0.2434f, -0.0149f,-0.9999f,-0.0086f, -0.4440f,-0.8586f,-0.2564f, -0.7536f,-0.4927f,-0.4351f, -0.8660f,-0.0000f,-0.5000f, -0.7431f,-0.0000f,-0.6691f, -0.6467f,0.4927f,-0.5823f, -0.3810f,0.8586f,-0.3431f, -0.0128f,0.9999f,-0.0115f, 0.3618f,0.8735f,0.3258f, 0.6403f,0.5076f,0.5765f, 0.7431f,-0.0000f,0.6691f, 
	0.6403f,-0.5076f,0.5765f, 0.3618f,-0.8735f,0.3258f, -0.0128f,-0.9999f,-0.0115f, -0.3810f,-0.8586f,-0.3431f, -0.6467f,-0.4927f,-0.5823f, -0.7431f,-0.0000f,-0.6691f, -0.5878f,-0.0000f,-0.8090f, -0.5115f,0.4927f,-0.7040f, -0.3014f,0.8586f,-0.4148f, -0.0101f,0.9999f,-0.0140f, 0.2862f,0.8735f,0.3939f, 0.5064f,0.5076f,0.6970f, 0.5878f,-0.0000f,0.8090f, 0.5064f,-0.5076f,0.6970f, 0.2862f,-0.8735f,0.3939f, 
	-0.0101f,-0.9999f,-0.0140f, -0.3014f,-0.8586f,-0.4148f, -0.5115f,-0.4927f,-0.7040f, -0.5878f,-0.0000f,-0.8090f, -0.4067f,0.0000f,-0.9135f, -0.3539f,0.4927f,-0.7950f, -0.2085f,0.8586f,-0.4684f, -0.0070f,0.9999f,-0.0158f, 0.1980f,0.8735f,0.4448f, 0.3504f,0.5076f,0.7871f, 0.4067f,-0.0000f,0.9135f, 0.3504f,-0.5076f,0.7871f, 0.1980f,-0.8735f,0.4448f, -0.0070f,-0.9999f,-0.0158f, -0.2085f,-0.8586f,-0.4684f, 
	-0.3539f,-0.4927f,-0.7950f, -0.4067f,0.0000f,-0.9135f, -0.2079f,-0.0000f,-0.9781f, -0.1809f,0.4927f,-0.8512f, -0.1066f,0.8586f,-0.5015f, -0.0036f,0.9999f,-0.0169f, 0.1012f,0.8735f,0.4762f, 0.1791f,0.5076f,0.8428f, 0.2079f,-0.0000f,0.9781f, 0.1791f,-0.5076f,0.8428f, 0.1012f,-0.8735f,0.4762f, -0.0036f,-0.9999f,-0.0169f, -0.1066f,-0.8586f,-0.5015f, -0.1809f,-0.4927f,-0.8512f, -0.2079f,-0.0000f,-0.9781f, 
	-0.0000f,0.0000f,-1.0000f, 0.0000f,0.4927f,-0.8702f, -0.0000f,0.8586f,-0.5127f, 0.0000f,0.9999f,-0.0172f, 0.0000f,0.8735f,0.4868f, 0.0000f,0.5076f,0.8616f, -0.0000f,-0.0000f,1.0000f, 0.0000f,-0.5076f,0.8616f, 0.0000f,-0.8735f,0.4868f, 0.0000f,-0.9999f,-0.0173f, 0.0000f,-0.8586f,-0.5127f, -0.0000f,-0.4927f,-0.8702f, -0.0000f,0.0000f,-1.0000f, 0.2079f,0.0000f,-0.9781f, 0.1771f,0.5200f,-0.8356f, 
	-0.0000f,0.0000f,-1.0000f, -0.0006f,0.5214f,-0.8533f, 0.1240f,0.8049f,-0.5804f, 0.0007f,0.8059f,-0.5921f, 0.0013f,1.0000f,-0.0063f, 0.0000f,1.0000f,-0.0064f, -0.0420f,0.9795f,0.1972f, -0.0001f,0.9794f,0.2018f, -0.1194f,0.8185f,0.5619f, 0.0001f,0.8184f,0.5747f, -0.2079f,-0.0000f,0.9781f, 0.0000f,-0.0000f,1.0000f, -0.1214f,-0.8117f,0.5713f, 0.0000f,-0.8117f,0.5840f, -0.0440f,-0.9773f,0.2071f, 
	-0.0000f,-0.9773f,0.2117f, 0.0013f,-1.0000f,-0.0063f, -0.0000f,-1.0000f,-0.0064f, 0.0473f,-0.9738f,-0.2225f, -0.0000f,-0.9738f,-0.2275f, 0.1266f,-0.7933f,-0.5955f, 0.0000f,-0.7933f,-0.6088f, 0.2079f,0.0000f,-0.9781f, -0.0000f,0.0000f,-1.0000f, 0.4067f,0.0000f,-0.9135f, 0.3474f,0.5189f,-0.7811f, 0.2422f,0.8040f,-0.5430f, 0.0026f,1.0000f,-0.0059f, -0.0820f,0.9795f,0.1840f, -0.2336f,0.8186f,0.5247f, 
	-0.4067f,-0.0000f,0.9135f, -0.2375f,-0.8117f,0.5335f, -0.0861f,-0.9773f,0.1934f, 0.0026f,-1.0000f,-0.0059f, 0.0925f,-0.9738f,-0.2078f, 0.2476f,-0.7933f,-0.5562f, 0.4067f,0.0000f,-0.9135f, 0.5878f,0.0000f,-0.8090f, 0.5026f,0.5182f,-0.6920f, 0.3501f,0.8035f,-0.4815f, 0.0038f,1.0000f,-0.0052f, -0.1184f,0.9795f,0.1629f, -0.3375f,0.8187f,0.4646f, -0.5878f,-0.0000f,0.8090f, -0.3433f,-0.8117f,0.4725f, 
	-0.1244f,-0.9773f,0.1713f, 0.0038f,-1.0000f,-0.0052f, 0.1337f,-0.9738f,-0.1841f, 0.3578f,-0.7933f,-0.4925f, 0.5878f,0.0000f,-0.8090f, 0.7431f,0.0000f,-0.6691f, 0.6357f,0.5178f,-0.5725f, 0.4427f,0.8033f,-0.3985f, 0.0048f,1.0000f,-0.0043f, -0.1496f,0.9795f,0.1347f, -0.4267f,0.8187f,0.3842f, -0.7431f,-0.0000f,0.6691f, -0.4340f,-0.8117f,0.3908f, -0.1573f,-0.9773f,0.1417f, 0.0048f,-1.0000f,-0.0043f, 
	0.1691f,-0.9738f,-0.1522f, 0.4524f,-0.7933f,-0.4074f, 0.7431f,0.0000f,-0.6691f, 0.8660f,0.0000f,-0.5000f, 0.7409f,0.5179f,-0.4276f, 0.5157f,0.8033f,-0.2979f, 0.0056f,1.0000f,-0.0032f, -0.1744f,0.9795f,0.1007f, -0.4973f,0.8187f,0.2871f, -0.8660f,-0.0000f,0.5000f, -0.5058f,-0.8117f,0.2920f, -0.1833f,-0.9773f,0.1058f, 0.0056f,-1.0000f,-0.0032f, 0.1970f,-0.9738f,-0.1138f, 0.5272f,-0.7933f,-0.3044f, 
	0.8660f,0.0000f,-0.5000f, 0.9511f,0.0000f,-0.3090f, 0.8134f,0.5184f,-0.2640f, 0.5658f,0.8037f,-0.1842f, 0.0061f,1.0000f,-0.0020f, -0.1915f,0.9795f,0.0623f, -0.5462f,0.8187f,0.1774f, -0.9511f,-0.0000f,0.3090f, -0.5554f,-0.8117f,0.1805f, -0.2013f,-0.9773f,0.0654f, 0.0061f,-1.0000f,-0.0020f, 0.2164f,-0.9738f,-0.0703f, 0.5790f,-0.7933f,-0.1881f, 0.9511f,0.0000f,-0.3090f, 0.9945f,0.0000f,-0.1045f, 
	0.8500f,0.5193f,-0.0890f, 0.5909f,0.8043f,-0.0626f, 0.0064f,1.0000f,-0.0007f, -0.2004f,0.9795f,0.0211f, -0.5712f,0.8186f,0.0600f, -0.9945f,-0.0000f,0.1045f, -0.5808f,-0.8117f,0.0610f, -0.2105f,-0.9773f,0.0221f, 0.0064f,-1.0000f,-0.0007f, 0.2263f,-0.9738f,-0.0238f, 0.6055f,-0.7933f,-0.0636f, 0.9945f,0.0000f,-0.1045f, 0.9945f,-0.0000f,0.1045f, 0.8491f,0.5205f,0.0897f, 0.5898f,0.8052f,0.0613f, 
	0.0064f,1.0000f,0.0007f, -0.2006f,0.9794f,-0.0210f, -0.5714f,0.8185f,-0.0601f, -0.9945f,-0.0000f,-0.1045f, -0.5808f,-0.8117f,-0.0610f, -0.2105f,-0.9773f,-0.0221f, 0.0064f,-1.0000f,0.0007f, 0.2263f,-0.9738f,0.0238f, 0.6055f,-0.7933f,0.0636f, 0.9945f,-0.0000f,0.1045f, 0.9511f,0.0000f,0.3090f, 0.8110f,0.5220f,0.2641f, 0.5628f,0.8063f,0.1820f, 0.0061f,1.0000f,0.0020f, -0.1920f,0.9794f,-0.0623f, 
	-0.5466f,0.8183f,-0.1777f, -0.9511f,-0.0000f,-0.3090f, -0.5554f,-0.8117f,-0.1805f, -0.2013f,-0.9773f,-0.0654f, 0.0061f,-1.0000f,0.0020f, 0.2164f,-0.9738f,0.0703f, 0.5790f,-0.7933f,0.1881f, 0.9511f,0.0000f,0.3090f, 0.8660f,0.0000f,0.5000f, 0.7374f,0.5238f,0.4265f, 0.5112f,0.8076f,0.2941f, 0.0056f,1.0000f,0.0032f, -0.1751f,0.9794f,-0.1010f, -0.4979f,0.8182f,-0.2876f, -0.8660f,-0.0000f,-0.5000f, 
	-0.5058f,-0.8117f,-0.2920f, -0.1833f,-0.9773f,-0.1058f, 0.0056f,-1.0000f,0.0032f, 0.1970f,-0.9738f,0.1138f, 0.5272f,-0.7933f,0.3044f, 0.8660f,0.0000f,0.5000f, 0.7431f,0.0000f,0.6691f, 0.6317f,0.5256f,0.5697f, 0.4375f,0.8090f,0.3927f, 0.0048f,1.0000f,0.0043f, -0.1505f,0.9793f,-0.1354f, -0.4274f,0.8180f,-0.3850f, -0.7431f,-0.0000f,-0.6691f, -0.4340f,-0.8117f,-0.3908f, -0.1573f,-0.9773f,-0.1417f, 
	0.0048f,-1.0000f,0.0043f, 0.1691f,-0.9738f,0.1522f, 0.4524f,-0.7933f,0.4074f, 0.7431f,0.0000f,0.6691f, 0.5878f,0.0000f,0.8090f, 0.4988f,0.5276f,0.6877f, 0.3451f,0.8104f,0.4735f, 0.0038f,1.0000f,0.0052f, -0.1192f,0.9792f,-0.1639f, -0.3382f,0.8178f,-0.4657f, -0.5878f,-0.0000f,-0.8090f, -0.3433f,-0.8117f,-0.4725f, -0.1244f,-0.9773f,-0.1713f, 0.0038f,-1.0000f,0.0052f, 0.1337f,-0.9738f,0.1841f, 
	0.3578f,-0.7933f,0.4925f, 0.5878f,0.0000f,0.8090f, 0.4067f,-0.0000f,0.9135f, 0.3444f,0.5295f,0.7752f, 0.2383f,0.8118f,0.5331f, 0.0026f,1.0000f,0.0059f, -0.0826f,0.9792f,-0.1854f, -0.2341f,0.8176f,-0.5260f, -0.4067f,-0.0000f,-0.9135f, -0.2375f,-0.8117f,-0.5335f, -0.0861f,-0.9773f,-0.1934f, 0.0026f,-1.0000f,0.0059f, 0.0925f,-0.9738f,0.2078f, 0.2476f,-0.7933f,0.5562f, 0.4067f,-0.0000f,0.9135f, 
	0.2079f,-0.0000f,0.9781f, 0.1755f,0.5313f,0.8288f, 0.1218f,0.8131f,0.5693f, 0.0013f,1.0000f,0.0063f, -0.0423f,0.9791f,-0.1988f, -0.1197f,0.8174f,-0.5634f, -0.2079f,-0.0000f,-0.9781f, -0.1214f,-0.8117f,-0.5713f, -0.0440f,-0.9773f,-0.2071f, 0.0013f,-1.0000f,0.0063f, 0.0473f,-0.9738f,0.2225f, 0.1266f,-0.7933f,0.5955f, 0.2079f,-0.0000f,0.9781f, 0.0000f,0.0000f,1.0000f, -0.0005f,0.5329f,0.8462f, 
	0.0007f,0.8143f,0.5805f, -0.0000f,1.0000f,0.0064f, -0.0001f,0.9791f,-0.2035f, 0.0001f,0.8173f,-0.5762f, -0.0000f,-0.0000f,-1.0000f, 0.0000f,-0.8117f,-0.5840f, 0.0000f,-0.9773f,-0.2117f, 0.0000f,-1.0000f,0.0064f, -0.0000f,-0.9738f,0.2275f, 0.0000f,-0.7933f,0.6088f, 0.0000f,0.0000f,1.0000f, -0.2079f,-0.0000f,0.9781f, -0.1762f,0.5342f,0.8268f, -0.1198f,0.8152f,0.5666f, -0.0013f,1.0000f,0.0063f, 
	0.0423f,0.9790f,-0.1992f, 0.1199f,0.8171f,-0.5638f, 0.2079f,-0.0000f,-0.9781f, 0.1214f,-0.8117f,-0.5713f, 0.0440f,-0.9773f,-0.2071f, -0.0013f,-1.0000f,0.0063f, -0.0473f,-0.9738f,0.2225f, -0.1266f,-0.7933f,0.5955f, -0.2079f,-0.0000f,0.9781f, -0.4067f,-0.0000f,0.9135f, -0.3438f,0.5353f,0.7715f, -0.2347f,0.8160f,0.5282f, -0.0026f,1.0000f,0.0059f, 0.0829f,0.9790f,-0.1862f, 0.2346f,0.8170f,-0.5267f, 
	0.4067f,-0.0000f,-0.9135f, 0.2375f,-0.8117f,-0.5335f, 0.0861f,-0.9773f,-0.1934f, -0.0026f,-1.0000f,0.0059f, -0.0925f,-0.9738f,0.2078f, -0.2476f,-0.7933f,0.5562f, -0.4067f,-0.0000f,0.9135f, -0.5878f,-0.0000f,0.8090f, -0.4964f,0.5360f,0.6829f, -0.3391f,0.8165f,0.4672f, -0.0038f,1.0000f,0.0052f, 0.1199f,0.9790f,-0.1650f, 0.3390f,0.8170f,-0.4665f, 0.5878f,-0.0000f,-0.8090f, 0.3433f,-0.8117f,-0.4725f, 
	0.1244f,-0.9773f,-0.1713f, -0.0038f,-1.0000f,0.0052f, -0.1337f,-0.9738f,0.1841f, -0.3578f,-0.7933f,0.4925f, -0.5878f,-0.0000f,0.8090f, -0.7431f,0.0000f,0.6691f, -0.6273f,0.5363f,0.5647f, -0.4287f,0.8168f,0.3861f, -0.0048f,1.0000f,0.0043f, 0.1516f,0.9790f,-0.1365f, 0.4286f,0.8169f,-0.3859f, 0.7431f,-0.0000f,-0.6691f, 0.4340f,-0.8117f,-0.3908f, 0.1573f,-0.9773f,-0.1417f, -0.0048f,-1.0000f,0.0043f, 
	-0.1691f,-0.9738f,0.1522f, -0.4524f,-0.7933f,0.4074f, -0.7431f,0.0000f,0.6691f, -0.8660f,0.0000f,0.5000f, -0.7309f,0.5362f,0.4221f, -0.4998f,0.8167f,0.2884f, -0.0056f,1.0000f,0.0032f, 0.1767f,0.9790f,-0.1020f, 0.4994f,0.8169f,-0.2884f, 0.8660f,-0.0000f,-0.5000f, 0.5058f,-0.8117f,-0.2920f, 0.1833f,-0.9773f,-0.1058f, -0.0056f,-1.0000f,0.0032f, -0.1970f,-0.9738f,0.1138f, -0.5272f,-0.7933f,0.3044f, 
	-0.8660f,0.0000f,0.5000f, -0.9511f,-0.0000f,0.3090f, -0.8030f,0.5358f,0.2611f, -0.5494f,0.8164f,0.1782f, -0.0061f,1.0000f,0.0020f, 0.1939f,0.9790f,-0.0630f, 0.5484f,0.8170f,-0.1782f, 0.9511f,-0.0000f,-0.3090f, 0.5554f,-0.8117f,-0.1805f, 0.2013f,-0.9773f,-0.0654f, -0.0061f,-1.0000f,0.0020f, -0.2164f,-0.9738f,0.0703f, -0.5790f,-0.7933f,0.1881f, -0.9511f,-0.0000f,0.3090f, -0.9945f,0.0000f,0.1045f, 
	-0.8402f,0.5350f,0.0887f, -0.5753f,0.8158f,0.0600f, -0.0064f,1.0000f,0.0007f, 0.2027f,0.9790f,-0.0213f, 0.5734f,0.8171f,-0.0603f, 0.9945f,-0.0000f,-0.1045f, 0.5808f,-0.8117f,-0.0610f, 0.2105f,-0.9773f,-0.0221f, -0.0064f,-1.0000f,0.0007f, -0.2263f,-0.9738f,0.0238f, -0.6055f,-0.7933f,0.0636f, -0.9945f,0.0000f,0.1045f, -0.9945f,0.0000f,-0.1045f, -0.8410f,0.5338f,-0.0879f, -0.5763f,0.8149f,-0.0612f, 
	-0.0064f,1.0000f,-0.0007f, 0.2025f,0.9791f,0.0213f, 0.5732f,0.8172f,0.0602f, 0.9945f,-0.0000f,0.1045f, 0.5808f,-0.8117f,0.0610f, 0.2105f,-0.9773f,0.0221f, -0.0064f,-1.0000f,-0.0007f, -0.2263f,-0.9738f,-0.0238f, -0.6055f,-0.7933f,-0.0636f, -0.9945f,0.0000f,-0.1045f, -0.9511f,-0.0000f,-0.3090f, -0.8053f,0.5323f,-0.2611f, -0.5524f,0.8138f,-0.1803f, -0.0061f,1.0000f,-0.0020f, 0.1934f,0.9791f,0.0629f, 
	0.5480f,0.8173f,0.1779f, 0.9511f,-0.0000f,0.3090f, 0.5554f,-0.8117f,0.1805f, 0.2013f,-0.9773f,0.0654f, -0.0061f,-1.0000f,-0.0020f, -0.2164f,-0.9738f,-0.0703f, -0.5790f,-0.7933f,-0.1881f, -0.9511f,-0.0000f,-0.3090f, -0.8660f,-0.0000f,-0.5000f, -0.7344f,0.5306f,-0.4232f, -0.5043f,0.8126f,-0.2921f, -0.0056f,1.0000f,-0.0032f, 0.1759f,0.9791f,0.1016f, 0.4988f,0.8175f,0.2879f, 0.8660f,-0.0000f,0.5000f, 
	0.5058f,-0.8117f,0.2920f, 0.1833f,-0.9773f,0.1058f, -0.0056f,-1.0000f,-0.0032f, -0.1970f,-0.9738f,-0.1138f, -0.5272f,-0.7933f,-0.3044f, -0.8660f,-0.0000f,-0.5000f, -0.7431f,0.0000f,-0.6691f, -0.6312f,0.5288f,-0.5674f, -0.4339f,0.8113f,-0.3919f, -0.0048f,1.0000f,-0.0043f, 0.1507f,0.9792f,0.1358f, 0.4279f,0.8177f,0.3851f, 0.7431f,-0.0000f,0.6691f, 0.4340f,-0.8117f,0.3908f, 0.1573f,-0.9773f,0.1417f, 
	-0.0048f,-1.0000f,-0.0043f, -0.1691f,-0.9738f,-0.1522f, -0.4524f,-0.7933f,-0.4074f, -0.7431f,0.0000f,-0.6691f, -0.5878f,0.0000f,-0.8090f, -0.5001f,0.5269f,-0.6872f, -0.3441f,0.8099f,-0.4751f, -0.0038f,1.0000f,-0.0052f, 0.1190f,0.9793f,0.1639f, 0.3383f,0.8179f,0.4654f, 0.5878f,-0.0000f,0.8090f, 0.3433f,-0.8117f,0.4725f, 0.1244f,-0.9773f,0.1713f, -0.0038f,-1.0000f,-0.0052f, -0.1337f,-0.9738f,-0.1841f, 
	-0.3578f,-0.7933f,-0.4925f, -0.5878f,0.0000f,-0.8090f, -0.4067f,0.0000f,-0.9135f, -0.3468f,0.5250f,-0.7773f, -0.2386f,0.8085f,-0.5380f, -0.0026f,1.0000f,-0.0059f, 0.0822f,0.9793f,0.1849f, 0.2340f,0.8181f,0.5254f, 0.4067f,-0.0000f,0.9135f, 0.2375f,-0.8117f,0.5335f, 0.0861f,-0.9773f,0.1934f, -0.0026f,-1.0000f,-0.0059f, -0.0925f,-0.9738f,-0.2078f, -0.2476f,-0.7933f,-0.5562f, -0.4067f,0.0000f,-0.9135f, 
	-0.2079f,0.0000f,-0.9781f, -0.1778f,0.5231f,-0.8335f, -0.1219f,0.8071f,-0.5777f, -0.0013f,1.0000f,-0.0063f, 0.0419f,0.9794f,0.1977f, 0.1196f,0.8182f,0.5623f, 0.2079f,-0.0000f,0.9781f, 0.1214f,-0.8117f,0.5713f, 0.0440f,-0.9773f,0.2071f, -0.0013f,-1.0000f,-0.0063f, -0.0473f,-0.9738f,-0.2225f, -0.1266f,-0.7933f,-0.5955f, -0.2079f,0.0000f,-0.9781f, -0.0000f,0.0000f,-1.0000f, -0.0006f,0.5214f,-0.8533f, 
	0.0007f,0.8059f,-0.5921f, 0.0000f,1.0000f,-0.0064f, -0.0001f,0.9794f,0.2018f, 0.0001f,0.8184f,0.5747f, 0.0000f,-0.0000f,1.0000f, 0.0000f,-0.8117f,0.5840f, -0.0000f,-0.9773f,0.2117f, -0.0000f,-1.0000f,-0.0064f, -0.0000f,-0.9738f,-0.2275f, 0.0000f,-0.7933f,-0.6088f, -0.0000f,0.0000f,-1.0000f, 0.2079f,-0.0000f,-0.9781f, 0.1809f,0.4927f,-0.8512f, -0.0000f,-0.0000f,-1.0000f, -0.0000f,0.4927f,-0.8702f, 
	0.1066f,0.8586f,-0.5015f, -0.0000f,0.8586f,-0.5127f, 0.0036f,0.9999f,-0.0169f, 0.0000f,0.9999f,-0.0172f, -0.1012f,0.8735f,0.4762f, 0.0000f,0.8735f,0.4868f, -0.1791f,0.5076f,0.8428f, 0.0000f,0.5076f,0.8616f, -0.2079f,-0.0000f,0.9781f, 0.0000f,-0.0000f,1.0000f, -0.1791f,-0.5076f,0.8428f, 0.0000f,-0.5076f,0.8616f, -0.1012f,-0.8735f,0.4762f, 0.0000f,-0.8735f,0.4868f, 0.0036f,-0.9999f,-0.0169f, 
	0.0000f,-0.9999f,-0.0172f, 0.1066f,-0.8586f,-0.5015f, -0.0000f,-0.8586f,-0.5127f, 0.1809f,-0.4927f,-0.8512f, -0.0000f,-0.4927f,-0.8702f, 0.2079f,-0.0000f,-0.9781f, -0.0000f,-0.0000f,-1.0000f, 0.4067f,-0.0000f,-0.9135f, 0.3539f,0.4927f,-0.7950f, 0.2085f,0.8586f,-0.4684f, 0.0070f,0.9999f,-0.0158f, -0.1980f,0.8735f,0.4448f, -0.3504f,0.5076f,0.7871f, -0.4067f,-0.0000f,0.9135f, -0.3504f,-0.5076f,0.7871f, 
	-0.1980f,-0.8735f,0.4448f, 0.0070f,-0.9999f,-0.0158f, 0.2085f,-0.8586f,-0.4684f, 0.3539f,-0.4927f,-0.7950f, 0.4067f,-0.0000f,-0.9135f, 0.5878f,-0.0000f,-0.8090f, 0.5115f,0.4927f,-0.7040f, 0.3014f,0.8586f,-0.4148f, 0.0101f,0.9999f,-0.0140f, -0.2862f,0.8735f,0.3939f, -0.5064f,0.5076f,0.6970f, -0.5878f,-0.0000f,0.8090f, -0.5064f,-0.5076f,0.6970f, -0.2862f,-0.8735f,0.3939f, 0.0101f,-0.9999f,-0.0140f, 
	0.3014f,-0.8586f,-0.4148f, 0.5115f,-0.4927f,-0.7040f, 0.5878f,-0.0000f,-0.8090f, 0.7431f,-0.0000f,-0.6691f, 0.6467f,0.4927f,-0.5823f, 0.3810f,0.8586f,-0.3431f, 0.0128f,0.9999f,-0.0115f, -0.3618f,0.8735f,0.3258f, -0.6403f,0.5076f,0.5765f, -0.7431f,-0.0000f,0.6691f, -0.6403f,-0.5076f,0.5765f, -0.3618f,-0.8735f,0.3258f, 0.0128f,-0.9999f,-0.0115f, 0.3810f,-0.8586f,-0.3431f, 0.6467f,-0.4927f,-0.5823f, 
	0.7431f,-0.0000f,-0.6691f, 0.8660f,-0.0000f,-0.5000f, 0.7536f,0.4927f,-0.4351f, 0.4440f,0.8586f,-0.2564f, 0.0149f,0.9999f,-0.0086f, -0.4216f,0.8735f,0.2434f, -0.7462f,0.5076f,0.4308f, -0.8660f,-0.0000f,0.5000f, -0.7462f,-0.5076f,0.4308f, -0.4216f,-0.8735f,0.2434f, 0.0149f,-0.9999f,-0.0086f, 0.4440f,-0.8586f,-0.2564f, 0.7536f,-0.4927f,-0.4351f, 0.8660f,-0.0000f,-0.5000f, 0.9511f,-0.0000f,-0.3090f, 
	0.8276f,0.4927f,-0.2689f, 0.4876f,0.8586f,-0.1584f, 0.0164f,0.9999f,-0.0053f, -0.4630f,0.8735f,0.1504f, -0.8194f,0.5076f,0.2662f, -0.9511f,-0.0000f,0.3090f, -0.8194f,-0.5076f,0.2662f, -0.4630f,-0.8735f,0.1504f, 0.0164f,-0.9999f,-0.0053f, 0.4876f,-0.8586f,-0.1584f, 0.8276f,-0.4927f,-0.2689f, 0.9511f,-0.0000f,-0.3090f, 0.9945f,-0.0000f,-0.1045f, 0.8654f,0.4927f,-0.0910f, 0.5099f,0.8586f,-0.0536f, 
	0.0172f,0.9999f,-0.0018f, -0.4842f,0.8735f,0.0509f, -0.8569f,0.5076f,0.0901f, -0.9945f,-0.0000f,0.1045f, -0.8569f,-0.5076f,0.0901f, -0.4842f,-0.8735f,0.0509f, 0.0172f,-0.9999f,-0.0018f, 0.5099f,-0.8586f,-0.0536f, 0.8654f,-0.4927f,-0.0910f, 0.9945f,-0.0000f,-0.1045f, 0.9945f,-0.0000f,0.1045f, 0.8654f,0.4927f,0.0910f, 0.5099f,0.8586f,0.0536f, 0.0172f,0.9999f,0.0018f, -0.4842f,0.8735f,-0.0509f, 
	-0.8569f,0.5076f,-0.0901f, -0.9945f,-0.0000f,-0.1045f, -0.8569f,-0.5076f,-0.0901f, -0.4842f,-0.8735f,-0.0509f, 0.0172f,-0.9999f,0.0018f, 0.5099f,-0.8586f,0.0536f, 0.8654f,-0.4927f,0.0910f, 0.9945f,-0.0000f,0.1045f, 0.9511f,-0.0000f,0.3090f, 0.8276f,0.4927f,0.2689f, 0.4876f,0.8586f,0.1584f, 0.0164f,0.9999f,0.0053f, -0.4630f,0.8735f,-0.1504f, -0.8194f,0.5076f,-0.2662f, -0.9511f,-0.0000f,-0.3090f, 
	-0.8194f,-0.5076f,-0.2662f, -0.4630f,-0.8735f,-0.1504f, 0.0164f,-0.9999f,0.0053f, 0.4876f,-0.8586f,0.1584f, 0.8276f,-0.4927f,0.2689f, 0.9511f,-0.0000f,0.3090f, 0.8660f,-0.0000f,0.5000f, 0.7536f,0.4927f,0.4351f, 0.4440f,0.8586f,0.2564f, 0.0149f,0.9999f,0.0086f, -0.4216f,0.8735f,-0.2434f, -0.7462f,0.5076f,-0.4308f, -0.8660f,-0.0000f,-0.5000f, -0.7462f,-0.5076f,-0.4308f, -0.4216f,-0.8735f,-0.2434f, 
	0.0149f,-0.9999f,0.0086f, 0.4440f,-0.8586f,0.2564f, 0.7536f,-0.4927f,0.4351f, 0.8660f,-0.0000f,0.5000f, 0.7431f,-0.0000f,0.6691f, 0.6467f,0.4927f,0.5823f, 0.3810f,0.8586f,0.3431f, 0.0128f,0.9999f,0.0115f, -0.3618f,0.8735f,-0.3258f, -0.6403f,0.5076f,-0.5765f, -0.7431f,-0.0000f,-0.6691f, -0.6403f,-0.5076f,-0.5765f, -0.3618f,-0.8735f,-0.3258f, 0.0128f,-0.9999f,0.0115f, 0.3810f,-0.8586f,0.3431f, 
	0.6467f,-0.4927f,0.5823f, 0.7431f,-0.0000f,0.6691f, 0.5878f,-0.0000f,0.8090f, 0.5115f,0.4927f,0.7040f, 0.3014f,0.8586f,0.4148f, 0.0101f,0.9999f,0.0140f, -0.2862f,0.8735f,-0.3939f, -0.5064f,0.5076f,-0.6970f, -0.5878f,-0.0000f,-0.8090f, -0.5064f,-0.5076f,-0.6970f, -0.2862f,-0.8735f,-0.3939f, 0.0101f,-0.9999f,0.0140f, 0.3014f,-0.8586f,0.4148f, 0.5115f,-0.4927f,0.7040f, 0.5878f,-0.0000f,0.8090f, 
	0.4067f,-0.0000f,0.9135f, 0.3539f,0.4927f,0.7950f, 0.2085f,0.8586f,0.4684f, 0.0070f,0.9999f,0.0158f, -0.1980f,0.8735f,-0.4448f, -0.3504f,0.5076f,-0.7871f, -0.4067f,-0.0000f,-0.9135f, -0.3504f,-0.5076f,-0.7871f, -0.1980f,-0.8735f,-0.4448f, 0.0070f,-0.9999f,0.0158f, 0.2085f,-0.8586f,0.4684f, 0.3539f,-0.4927f,0.7950f, 0.4067f,-0.0000f,0.9135f, 0.2079f,-0.0000f,0.9781f, 0.1809f,0.4927f,0.8512f, 
	0.1066f,0.8586f,0.5015f, 0.0036f,0.9999f,0.0169f, -0.1012f,0.8735f,-0.4762f, -0.1791f,0.5076f,-0.8428f, -0.2079f,-0.0000f,-0.9781f, -0.1791f,-0.5076f,-0.8428f, -0.1012f,-0.8735f,-0.4762f, 0.0036f,-0.9999f,0.0169f, 0.1066f,-0.8586f,0.5015f, 0.1809f,-0.4927f,0.8512f, 0.2079f,-0.0000f,0.9781f, -0.0000f,-0.0000f,1.0000f, -0.0000f,0.4927f,0.8702f, 0.0000f,0.8586f,0.5127f, 0.0000f,0.9999f,0.0172f, 
	0.0000f,0.8735f,-0.4868f, 0.0000f,0.5076f,-0.8616f, 0.0000f,-0.0000f,-1.0000f, 0.0000f,-0.5076f,-0.8616f, 0.0000f,-0.8735f,-0.4868f, -0.0000f,-0.9999f,0.0172f, -0.0000f,-0.8586f,0.5127f, -0.0000f,-0.4927f,0.8702f, -0.0000f,-0.0000f,1.0000f, -0.2079f,-0.0000f,0.9781f, -0.1809f,0.4927f,0.8512f, -0.1066f,0.8586f,0.5015f, -0.0036f,0.9999f,0.0169f, 0.1012f,0.8735f,-0.4762f, 0.1791f,0.5076f,-0.8428f, 
	0.2079f,-0.0000f,-0.9781f, 0.1791f,-0.5076f,-0.8428f, 0.1012f,-0.8735f,-0.4762f, -0.0036f,-0.9999f,0.0169f, -0.1066f,-0.8586f,0.5015f, -0.1809f,-0.4927f,0.8512f, -0.2079f,-0.0000f,0.9781f, -0.4067f,-0.0000f,0.9135f, -0.3539f,0.4927f,0.7950f, -0.2085f,0.8586f,0.4684f, -0.0070f,0.9999f,0.0158f, 0.1980f,0.8735f,-0.4448f, 0.3504f,0.5076f,-0.7871f, 0.4067f,-0.0000f,-0.9135f, 0.3504f,-0.5076f,-0.7871f, 
	0.1980f,-0.8735f,-0.4448f, -0.0070f,-0.9999f,0.0158f, -0.2085f,-0.8586f,0.4684f, -0.3539f,-0.4927f,0.7950f, -0.4067f,-0.0000f,0.9135f, -0.5878f,-0.0000f,0.8090f, -0.5115f,0.4927f,0.7040f, -0.3014f,0.8586f,0.4148f, -0.0101f,0.9999f,0.0140f, 0.2862f,0.8735f,-0.3939f, 0.5064f,0.5076f,-0.6970f, 0.5878f,-0.0000f,-0.8090f, 0.5064f,-0.5076f,-0.6970f, 0.2862f,-0.8735f,-0.3939f, -0.0101f,-0.9999f,0.0140f, 
	-0.3014f,-0.8586f,0.4148f, -0.5115f,-0.4927f,0.7040f, -0.5878f,-0.0000f,0.8090f, -0.7431f,-0.0000f,0.6691f, -0.6467f,0.4927f,0.5823f, -0.3810f,0.8586f,0.3431f, -0.0128f,0.9999f,0.0115f, 0.3618f,0.8735f,-0.3258f, 0.6403f,0.5076f,-0.5765f, 0.7431f,-0.0000f,-0.6691f, 0.6403f,-0.5076f,-0.5765f, 0.3618f,-0.8735f,-0.3258f, -0.0128f,-0.9999f,0.0115f, -0.3810f,-0.8586f,0.3431f, -0.6467f,-0.4927f,0.5823f, 
	-0.7431f,-0.0000f,0.6691f, -0.8660f,-0.0000f,0.5000f, -0.7536f,0.4927f,0.4351f, -0.4440f,0.8586f,0.2564f, -0.0149f,0.9999f,0.0086f, 0.4216f,0.8735f,-0.2434f, 0.7462f,0.5076f,-0.4308f, 0.8660f,-0.0000f,-0.5000f, 0.7462f,-0.5076f,-0.4308f, 0.4216f,-0.8735f,-0.2434f, -0.0149f,-0.9999f,0.0086f, -0.4440f,-0.8586f,0.2564f, -0.7536f,-0.4927f,0.4351f, -0.8660f,-0.0000f,0.5000f, -0.9511f,-0.0000f,0.3090f, 
	-0.8276f,0.4927f,0.2689f, -0.4876f,0.8586f,0.1584f, -0.0164f,0.9999f,0.0053f, 0.4630f,0.8735f,-0.1504f, 0.8194f,0.5076f,-0.2662f, 0.9511f,-0.0000f,-0.3090f, 0.8194f,-0.5076f,-0.2662f, 0.4630f,-0.8735f,-0.1504f, -0.0164f,-0.9999f,0.0053f, -0.4876f,-0.8586f,0.1584f, -0.8276f,-0.4927f,0.2689f, -0.9511f,-0.0000f,0.3090f, -0.9945f,-0.0000f,0.1045f, -0.8654f,0.4927f,0.0910f, -0.5099f,0.8586f,0.0536f, 
	-0.0172f,0.9999f,0.0018f, 0.4842f,0.8735f,-0.0509f, 0.8569f,0.5076f,-0.0901f, 0.9945f,-0.0000f,-0.1045f, 0.8569f,-0.5076f,-0.0901f, 0.4842f,-0.8735f,-0.0509f, -0.0172f,-0.9999f,0.0018f, -0.5099f,-0.8586f,0.0536f, -0.8654f,-0.4927f,0.0910f, -0.9945f,-0.0000f,0.1045f, -0.9945f,-0.0000f,-0.1045f, -0.8654f,0.4927f,-0.0910f, -0.5099f,0.8586f,-0.0536f, -0.0172f,0.9999f,-0.0018f, 0.4842f,0.8735f,0.0509f, 
	0.8569f,0.5076f,0.0901f, 0.9945f,-0.0000f,0.1045f, 0.8569f,-0.5076f,0.0901f, 0.4842f,-0.8735f,0.0509f, -0.0172f,-0.9999f,-0.0018f, -0.5099f,-0.8586f,-0.0536f, -0.8654f,-0.4927f,-0.0910f, -0.9945f,-0.0000f,-0.1045f, -0.9511f,-0.0000f,-0.3090f, -0.8276f,0.4927f,-0.2689f, -0.4876f,0.8586f,-0.1584f, -0.0164f,0.9999f,-0.0053f, 0.4630f,0.8735f,0.1504f, 0.8194f,0.5076f,0.2662f, 0.9511f,-0.0000f,0.3090f, 
	0.8194f,-0.5076f,0.2662f, 0.4630f,-0.8735f,0.1504f, -0.0164f,-0.9999f,-0.0053f, -0.4876f,-0.8586f,-0.1584f, -0.8276f,-0.4927f,-0.2689f, -0.9511f,-0.0000f,-0.3090f, -0.8660f,-0.0000f,-0.5000f, -0.7536f,0.4927f,-0.4351f, -0.4440f,0.8586f,-0.2564f, -0.0149f,0.9999f,-0.0086f, 0.4216f,0.8735f,0.2434f, 0.7462f,0.5076f,0.4308f, 0.8660f,-0.0000f,0.5000f, 0.7462f,-0.5076f,0.4308f, 0.4216f,-0.8735f,0.2434f, 
	-0.0149f,-0.9999f,-0.0086f, -0.4440f,-0.8586f,-0.2564f, -0.7536f,-0.4927f,-0.4351f, -0.8660f,-0.0000f,-0.5000f, -0.7431f,-0.0000f,-0.6691f, -0.6467f,0.4927f,-0.5823f, -0.3810f,0.8586f,-0.3431f, -0.0128f,0.9999f,-0.0115f, 0.3618f,0.8735f,0.3258f, 0.6403f,0.5076f,0.5765f, 0.7431f,-0.0000f,0.6691f, 0.6403f,-0.5076f,0.5765f, 0.3618f,-0.8735f,0.3258f, -0.0128f,-0.9999f,-0.0115f, -0.3810f,-0.8586f,-0.3431f, 
	-0.6467f,-0.4927f,-0.5823f, -0.7431f,-0.0000f,-0.6691f, -0.5878f,-0.0000f,-0.8090f, -0.5115f,0.4927f,-0.7040f, -0.3014f,0.8586f,-0.4148f, -0.0101f,0.9999f,-0.0140f, 0.2862f,0.8735f,0.3939f, 0.5064f,0.5076f,0.6970f, 0.5878f,-0.0000f,0.8090f, 0.5064f,-0.5076f,0.6970f, 0.2862f,-0.8735f,0.3939f, -0.0101f,-0.9999f,-0.0140f, -0.3014f,-0.8586f,-0.4148f, -0.5115f,-0.4927f,-0.7040f, -0.5878f,-0.0000f,-0.8090f, 
	-0.4067f,-0.0000f,-0.9135f, -0.3539f,0.4927f,-0.7950f, -0.2085f,0.8586f,-0.4684f, -0.0070f,0.9999f,-0.0158f, 0.1980f,0.8735f,0.4448f, 0.3504f,0.5076f,0.7871f, 0.4067f,-0.0000f,0.9135f, 0.3504f,-0.5076f,0.7871f, 0.1980f,-0.8735f,0.4448f, -0.0070f,-0.9999f,-0.0158f, -0.2085f,-0.8586f,-0.4684f, -0.3539f,-0.4927f,-0.7950f, -0.4067f,-0.0000f,-0.9135f, -0.2079f,-0.0000f,-0.9781f, -0.1809f,0.4927f,-0.8512f, 
	-0.1066f,0.8586f,-0.5015f, -0.0036f,0.9999f,-0.0169f, 0.1012f,0.8735f,0.4762f, 0.1791f,0.5076f,0.8428f, 0.2079f,-0.0000f,0.9781f, 0.1791f,-0.5076f,0.8428f, 0.1012f,-0.8735f,0.4762f, -0.0036f,-0.9999f,-0.0169f, -0.1066f,-0.8586f,-0.5015f, -0.1809f,-0.4927f,-0.8512f, -0.2079f,-0.0000f,-0.9781f, -0.0000f,-0.0000f,-1.0000f, -0.0000f,0.4927f,-0.8702f, -0.0000f,0.8586f,-0.5127f, 0.0000f,0.9999f,-0.0172f, 
	0.0000f,0.8735f,0.4868f, 0.0000f,0.5076f,0.8616f, 0.0000f,-0.0000f,1.0000f, 0.0000f,-0.5076f,0.8616f, 0.0000f,-0.8735f,0.4868f, 0.0000f,-0.9999f,-0.0172f, -0.0000f,-0.8586f,-0.5127f, -0.0000f,-0.4927f,-0.8702f, -0.0000f,-0.0000f,-1.0000f, 1.0000f,0.0000f,-0.0000f, 0.7071f,0.0000f,-0.7071f, 1.0000f,-0.0000f,-0.0000f, 0.7071f,0.0000f,-0.7071f, -0.0000f,0.0000f,-1.0000f, -0.0000f,0.0000f,-1.0000f, 
	-0.7071f,0.0000f,-0.7071f, -0.7071f,0.0000f,-0.7071f, -1.0000f,-0.0000f,0.0000f, -1.0000f,-0.0000f,0.0000f, -0.7071f,-0.0000f,0.7071f, -0.7071f,-0.0000f,0.7071f, 0.0000f,-0.0000f,1.0000f, 0.0000f,-0.0000f,1.0000f, 0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 1.0000f,0.0000f,-0.0000f, 0.7071f,0.0000f,-0.7071f, -0.0000f,0.0000f,-1.0000f, 
	-0.7071f,0.0000f,-0.7071f, -1.0000f,0.0000f,0.0000f, -0.7071f,-0.0000f,0.7071f, 0.0000f,-0.0000f,1.0000f, 0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 1.0000f,-0.0000f,-0.0000f, 0.7071f,0.0000f,-0.7071f, -0.0000f,0.0000f,-1.0000f, -0.7071f,0.0000f,-0.7071f, -1.0000f,-0.0000f,0.0000f, -0.7071f,-0.0000f,0.7071f, 0.0000f,-0.0000f,1.0000f, 0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 
	1.0000f,0.0000f,-0.0000f, 0.7071f,0.0000f,-0.7071f, -0.0000f,0.0000f,-1.0000f, -0.7071f,0.0000f,-0.7071f, -1.0000f,-0.0000f,0.0000f, -0.7071f,-0.0000f,0.7071f, 0.0000f,-0.0000f,1.0000f, 0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 1.0000f,0.0000f,-0.0000f, 0.7071f,0.0000f,-0.7071f, -0.0000f,0.0000f,-1.0000f, -0.7071f,0.0000f,-0.7071f, -1.0000f,0.0000f,0.0000f, -0.7071f,-0.0000f,0.7071f, 
	0.0000f,-0.0000f,1.0000f, 0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 1.0000f,-0.0000f,-0.0000f, 0.7071f,0.0000f,-0.7071f, 0.0000f,0.0000f,-1.0000f, -0.7071f,0.0000f,-0.7071f, -1.0000f,-0.0000f,0.0000f, -0.7071f,-0.0000f,0.7071f, 0.0000f,-0.0000f,1.0000f, 0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 1.0000f,0.0000f,0.0000f, 0.7071f,0.0000f,-0.7071f, 1.0000f,0.0000f,-0.0000f, 
	0.7071f,0.0000f,-0.7071f, 0.0000f,0.0000f,-1.0000f, -0.0000f,0.0000f,-1.0000f, -0.7071f,0.0000f,-0.7071f, -0.7071f,0.0000f,-0.7071f, -1.0000f,-0.0000f,0.0000f, -1.0000f,0.0000f,0.0000f, -0.7071f,-0.0000f,0.7071f, -0.7071f,0.0000f,0.7071f, -0.0000f,-0.0000f,1.0000f, 0.0000f,0.0000f,1.0000f, -0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 0.0000f,0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 
	1.0000f,-0.0000f,-0.0000f, 0.7071f,0.0000f,-0.7071f, -0.0000f,0.0000f,-1.0000f, -0.7071f,0.0000f,-0.7071f, -1.0000f,-0.0000f,0.0000f, -0.7071f,-0.0000f,0.7071f, 0.0000f,-0.0000f,1.0000f, 0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 1.0000f,0.0000f,0.0000f, 0.7071f,0.0000f,-0.7071f, -0.0000f,0.0000f,-1.0000f, -0.7071f,0.0000f,-0.7071f, -1.0000f,-0.0000f,0.0000f, -0.7071f,-0.0000f,0.7071f, 
	0.0000f,-0.0000f,1.0000f, 0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 1.0000f,-0.0000f,-0.0000f, 0.7071f,0.0000f,-0.7071f, -0.0000f,0.0000f,-1.0000f, -0.7071f,0.0000f,-0.7071f, -1.0000f,0.0000f,-0.0000f, -0.7071f,-0.0000f,0.7071f, 0.0000f,-0.0000f,1.0000f, 0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 1.0000f,0.0000f,-0.0000f, 0.7071f,0.0000f,-0.7071f, -0.0000f,0.0000f,-1.0000f, 
	-0.7071f,0.0000f,-0.7071f, -1.0000f,-0.0000f,0.0000f, -0.7071f,-0.0000f,0.7071f, 0.0000f,-0.0000f,1.0000f, 0.0000f,-0.0000f,1.0000f, 0.7071f,-0.0000f,0.7071f, 1.0000f,0.0000f,-0.0000f, 0.7071f,0.0000f,-0.7071f, -0.0000f,0.0000f,-1.0000f, -0.7071f,0.0000f,-0.7071f, -1.0000f,-0.0000f,0.0000f, -0.7071f,-0.0000f,0.7071f, 0.0000f,0.0000f,1.0000f, 0.0000f,0.0000f,1.0000f, 0.7071f,0.0000f,0.7071f, 
	
_NULL_ }; /* End of silver_Normals */

const unsigned short silver_Faces[] = {
	0, 2, 1, 1, 2, 3, 1, 3, 4, 4, 3, 5, 4, 5, 6, 6, 5, 7, 6, 7, 8, 8, 7, 9, 8, 9, 10, 10, 9, 11, 
	10, 11, 12, 12, 11, 13, 12, 13, 14, 14, 13, 15, 14, 15, 16, 16, 15, 17, 16, 17, 18, 18, 17, 19, 18, 19, 20, 20, 19, 21, 
	20, 21, 22, 22, 21, 23, 22, 23, 24, 24, 23, 25, 26, 0, 27, 27, 0, 1, 27, 1, 28, 28, 1, 4, 28, 4, 29, 29, 4, 6, 
	29, 6, 30, 30, 6, 8, 30, 8, 31, 31, 8, 10, 31, 10, 32, 32, 10, 12, 32, 12, 33, 33, 12, 14, 33, 14, 34, 34, 14, 16, 
	34, 16, 35, 35, 16, 18, 35, 18, 36, 36, 18, 20, 36, 20, 37, 37, 20, 22, 37, 22, 38, 38, 22, 24, 39, 26, 40, 40, 26, 27, 
	40, 27, 41, 41, 27, 28, 41, 28, 42, 42, 28, 29, 42, 29, 43, 43, 29, 30, 43, 30, 44, 44, 30, 31, 44, 31, 45, 45, 31, 32, 
	45, 32, 46, 46, 32, 33, 46, 33, 47, 47, 33, 34, 47, 34, 48, 48, 34, 35, 48, 35, 49, 49, 35, 36, 49, 36, 50, 50, 36, 37, 
	50, 37, 51, 51, 37, 38, 52, 39, 53, 53, 39, 40, 53, 40, 54, 54, 40, 41, 54, 41, 55, 55, 41, 42, 55, 42, 56, 56, 42, 43, 
	56, 43, 57, 57, 43, 44, 57, 44, 58, 58, 44, 45, 58, 45, 59, 59, 45, 46, 59, 46, 60, 60, 46, 47, 60, 47, 61, 61, 47, 48, 
	61, 48, 62, 62, 48, 49, 62, 49, 63, 63, 49, 50, 63, 50, 64, 64, 50, 51, 65, 52, 66, 66, 52, 53, 66, 53, 67, 67, 53, 54, 
	67, 54, 68, 68, 54, 55, 68, 55, 69, 69, 55, 56, 69, 56, 70, 70, 56, 57, 70, 57, 71, 71, 57, 58, 71, 58, 72, 72, 58, 59, 
	72, 59, 73, 73, 59, 60, 73, 60, 74, 74, 60, 61, 74, 61, 75, 75, 61, 62, 75, 62, 76, 76, 62, 63, 76, 63, 77, 77, 63, 64, 
	78, 65, 79, 79, 65, 66, 79, 66, 80, 80, 66, 67, 80, 67, 81, 81, 67, 68, 81, 68, 82, 82, 68, 69, 82, 69, 83, 83, 69, 70, 
	83, 70, 84, 84, 70, 71, 84, 71, 85, 85, 71, 72, 85, 72, 86, 86, 72, 73, 86, 73, 87, 87, 73, 74, 87, 74, 88, 88, 74, 75, 
	88, 75, 89, 89, 75, 76, 89, 76, 90, 90, 76, 77, 91, 78, 92, 92, 78, 79, 92, 79, 93, 93, 79, 80, 93, 80, 94, 94, 80, 81, 
	94, 81, 95, 95, 81, 82, 95, 82, 96, 96, 82, 83, 96, 83, 97, 97, 83, 84, 97, 84, 98, 98, 84, 85, 98, 85, 99, 99, 85, 86, 
	99, 86, 100, 100, 86, 87, 100, 87, 101, 101, 87, 88, 101, 88, 102, 102, 88, 89, 102, 89, 103, 103, 89, 90, 104, 91, 105, 105, 91, 92, 
	105, 92, 106, 106, 92, 93, 106, 93, 107, 107, 93, 94, 107, 94, 108, 108, 94, 95, 108, 95, 109, 109, 95, 96, 109, 96, 110, 110, 96, 97, 
	110, 97, 111, 111, 97, 98, 111, 98, 112, 112, 98, 99, 112, 99, 113, 113, 99, 100, 113, 100, 114, 114, 100, 101, 114, 101, 115, 115, 101, 102, 
	115, 102, 116, 116, 102, 103, 117, 104, 118, 118, 104, 105, 118, 105, 119, 119, 105, 106, 119, 106, 120, 120, 106, 107, 120, 107, 121, 121, 107, 108, 
	121, 108, 122, 122, 108, 109, 122, 109, 123, 123, 109, 110, 123, 110, 124, 124, 110, 111, 124, 111, 125, 125, 111, 112, 125, 112, 126, 126, 112, 113, 
	126, 113, 127, 127, 113, 114, 127, 114, 128, 128, 114, 115, 128, 115, 129, 129, 115, 116, 130, 117, 131, 131, 117, 118, 131, 118, 132, 132, 118, 119, 
	132, 119, 133, 133, 119, 120, 133, 120, 134, 134, 120, 121, 134, 121, 135, 135, 121, 122, 135, 122, 136, 136, 122, 123, 136, 123, 137, 137, 123, 124, 
	137, 124, 138, 138, 124, 125, 138, 125, 139, 139, 125, 126, 139, 126, 140, 140, 126, 127, 140, 127, 141, 141, 127, 128, 141, 128, 142, 142, 128, 129, 
	143, 130, 144, 144, 130, 131, 144, 131, 145, 145, 131, 132, 145, 132, 146, 146, 132, 133, 146, 133, 147, 147, 133, 134, 147, 134, 148, 148, 134, 135, 
	148, 135, 149, 149, 135, 136, 149, 136, 150, 150, 136, 137, 150, 137, 151, 151, 137, 138, 151, 138, 152, 152, 138, 139, 152, 139, 153, 153, 139, 140, 
	153, 140, 154, 154, 140, 141, 154, 141, 155, 155, 141, 142, 156, 143, 157, 157, 143, 144, 157, 144, 158, 158, 144, 145, 158, 145, 159, 159, 145, 146, 
	159, 146, 160, 160, 146, 147, 160, 147, 161, 161, 147, 148, 161, 148, 162, 162, 148, 149, 162, 149, 163, 163, 149, 150, 163, 150, 164, 164, 150, 151, 
	164, 151, 165, 165, 151, 152, 165, 152, 166, 166, 152, 153, 166, 153, 167, 167, 153, 154, 167, 154, 168, 168, 154, 155, 169, 156, 170, 170, 156, 157, 
	170, 157, 171, 171, 157, 158, 171, 158, 172, 172, 158, 159, 172, 159, 173, 173, 159, 160, 173, 160, 174, 174, 160, 161, 174, 161, 175, 175, 161, 162, 
	175, 162, 176, 176, 162, 163, 176, 163, 177, 177, 163, 164, 177, 164, 178, 178, 164, 165, 178, 165, 179, 179, 165, 166, 179, 166, 180, 180, 166, 167, 
	180, 167, 181, 181, 167, 168, 182, 169, 183, 183, 169, 170, 183, 170, 184, 184, 170, 171, 184, 171, 185, 185, 171, 172, 185, 172, 186, 186, 172, 173, 
	186, 173, 187, 187, 173, 174, 187, 174, 188, 188, 174, 175, 188, 175, 189, 189, 175, 176, 189, 176, 190, 190, 176, 177, 190, 177, 191, 191, 177, 178, 
	191, 178, 192, 192, 178, 179, 192, 179, 193, 193, 179, 180, 193, 180, 194, 194, 180, 181, 195, 182, 196, 196, 182, 183, 196, 183, 197, 197, 183, 184, 
	197, 184, 198, 198, 184, 185, 198, 185, 199, 199, 185, 186, 199, 186, 200, 200, 186, 187, 200, 187, 201, 201, 187, 188, 201, 188, 202, 202, 188, 189, 
	202, 189, 203, 203, 189, 190, 203, 190, 204, 204, 190, 191, 204, 191, 205, 205, 191, 192, 205, 192, 206, 206, 192, 193, 206, 193, 207, 207, 193, 194, 
	208, 195, 209, 209, 195, 196, 209, 196, 210, 210, 196, 197, 210, 197, 211, 211, 197, 198, 211, 198, 212, 212, 198, 199, 212, 199, 213, 213, 199, 200, 
	213, 200, 214, 214, 200, 201, 214, 201, 215, 215, 201, 202, 215, 202, 216, 216, 202, 203, 216, 203, 217, 217, 203, 204, 217, 204, 218, 218, 204, 205, 
	218, 205, 219, 219, 205, 206, 219, 206, 220, 220, 206, 207, 221, 208, 222, 222, 208, 209, 222, 209, 223, 223, 209, 210, 223, 210, 224, 224, 210, 211, 
	224, 211, 225, 225, 211, 212, 225, 212, 226, 226, 212, 213, 226, 213, 227, 227, 213, 214, 227, 214, 228, 228, 214, 215, 228, 215, 229, 229, 215, 216, 
	229, 216, 230, 230, 216, 217, 230, 217, 231, 231, 217, 218, 231, 218, 232, 232, 218, 219, 232, 219, 233, 233, 219, 220, 234, 221, 235, 235, 221, 222, 
	235, 222, 236, 236, 222, 223, 236, 223, 237, 237, 223, 224, 237, 224, 238, 238, 224, 225, 238, 225, 239, 239, 225, 226, 239, 226, 240, 240, 226, 227, 
	240, 227, 241, 241, 227, 228, 241, 228, 242, 242, 228, 229, 242, 229, 243, 243, 229, 230, 243, 230, 244, 244, 230, 231, 244, 231, 245, 245, 231, 232, 
	245, 232, 246, 246, 232, 233, 247, 234, 248, 248, 234, 235, 248, 235, 249, 249, 235, 236, 249, 236, 250, 250, 236, 237, 250, 237, 251, 251, 237, 238, 
	251, 238, 252, 252, 238, 239, 252, 239, 253, 253, 239, 240, 253, 240, 254, 254, 240, 241, 254, 241, 255, 255, 241, 242, 255, 242, 256, 256, 242, 243, 
	256, 243, 257, 257, 243, 244, 257, 244, 258, 258, 244, 245, 258, 245, 259, 259, 245, 246, 260, 247, 261, 261, 247, 248, 261, 248, 262, 262, 248, 249, 
	262, 249, 263, 263, 249, 250, 263, 250, 264, 264, 250, 251, 264, 251, 265, 265, 251, 252, 265, 252, 266, 266, 252, 253, 266, 253, 267, 267, 253, 254, 
	267, 254, 268, 268, 254, 255, 268, 255, 269, 269, 255, 256, 269, 256, 270, 270, 256, 257, 270, 257, 271, 271, 257, 258, 271, 258, 272, 272, 258, 259, 
	273, 260, 274, 274, 260, 261, 274, 261, 275, 275, 261, 262, 275, 262, 276, 276, 262, 263, 276, 263, 277, 277, 263, 264, 277, 264, 278, 278, 264, 265, 
	278, 265, 279, 279, 265, 266, 279, 266, 280, 280, 266, 267, 280, 267, 281, 281, 267, 268, 281, 268, 282, 282, 268, 269, 282, 269, 283, 283, 269, 270, 
	283, 270, 284, 284, 270, 271, 284, 271, 285, 285, 271, 272, 286, 273, 287, 287, 273, 274, 287, 274, 288, 288, 274, 275, 288, 275, 289, 289, 275, 276, 
	289, 276, 290, 290, 276, 277, 290, 277, 291, 291, 277, 278, 291, 278, 292, 292, 278, 279, 292, 279, 293, 293, 279, 280, 293, 280, 294, 294, 280, 281, 
	294, 281, 295, 295, 281, 282, 295, 282, 296, 296, 282, 283, 296, 283, 297, 297, 283, 284, 297, 284, 298, 298, 284, 285, 299, 286, 300, 300, 286, 287, 
	300, 287, 301, 301, 287, 288, 301, 288, 302, 302, 288, 289, 302, 289, 303, 303, 289, 290, 303, 290, 304, 304, 290, 291, 304, 291, 305, 305, 291, 292, 
	305, 292, 306, 306, 292, 293, 306, 293, 307, 307, 293, 294, 307, 294, 308, 308, 294, 295, 308, 295, 309, 309, 295, 296, 309, 296, 310, 310, 296, 297, 
	310, 297, 311, 311, 297, 298, 312, 299, 313, 313, 299, 300, 313, 300, 314, 314, 300, 301, 314, 301, 315, 315, 301, 302, 315, 302, 316, 316, 302, 303, 
	316, 303, 317, 317, 303, 304, 317, 304, 318, 318, 304, 305, 318, 305, 319, 319, 305, 306, 319, 306, 320, 320, 306, 307, 320, 307, 321, 321, 307, 308, 
	321, 308, 322, 322, 308, 309, 322, 309, 323, 323, 309, 310, 323, 310, 324, 324, 310, 311, 325, 312, 326, 326, 312, 313, 326, 313, 327, 327, 313, 314, 
	327, 314, 328, 328, 314, 315, 328, 315, 329, 329, 315, 316, 329, 316, 330, 330, 316, 317, 330, 317, 331, 331, 317, 318, 331, 318, 332, 332, 318, 319, 
	332, 319, 333, 333, 319, 320, 333, 320, 334, 334, 320, 321, 334, 321, 335, 335, 321, 322, 335, 322, 336, 336, 322, 323, 336, 323, 337, 337, 323, 324, 
	338, 325, 339, 339, 325, 326, 339, 326, 340, 340, 326, 327, 340, 327, 341, 341, 327, 328, 341, 328, 342, 342, 328, 329, 342, 329, 343, 343, 329, 330, 
	343, 330, 344, 344, 330, 331, 344, 331, 345, 345, 331, 332, 345, 332, 346, 346, 332, 333, 346, 333, 347, 347, 333, 334, 347, 334, 348, 348, 334, 335, 
	348, 335, 349, 349, 335, 336, 349, 336, 350, 350, 336, 337, 351, 338, 352, 352, 338, 339, 352, 339, 353, 353, 339, 340, 353, 340, 354, 354, 340, 341, 
	354, 341, 355, 355, 341, 342, 355, 342, 356, 356, 342, 343, 356, 343, 357, 357, 343, 344, 357, 344, 358, 358, 344, 345, 358, 345, 359, 359, 345, 346, 
	359, 346, 360, 360, 346, 347, 360, 347, 361, 361, 347, 348, 361, 348, 362, 362, 348, 349, 362, 349, 363, 363, 349, 350, 364, 351, 365, 365, 351, 352, 
	365, 352, 366, 366, 352, 353, 366, 353, 367, 367, 353, 354, 367, 354, 368, 368, 354, 355, 368, 355, 369, 369, 355, 356, 369, 356, 370, 370, 356, 357, 
	370, 357, 371, 371, 357, 358, 371, 358, 372, 372, 358, 359, 372, 359, 373, 373, 359, 360, 373, 360, 374, 374, 360, 361, 374, 361, 375, 375, 361, 362, 
	375, 362, 376, 376, 362, 363, 377, 364, 378, 378, 364, 365, 378, 365, 379, 379, 365, 366, 379, 366, 380, 380, 366, 367, 380, 367, 381, 381, 367, 368, 
	381, 368, 382, 382, 368, 369, 382, 369, 383, 383, 369, 370, 383, 370, 384, 384, 370, 371, 384, 371, 385, 385, 371, 372, 385, 372, 386, 386, 372, 373, 
	386, 373, 387, 387, 373, 374, 387, 374, 388, 388, 374, 375, 388, 375, 389, 389, 375, 376, 390, 377, 391, 391, 377, 378, 391, 378, 392, 392, 378, 379, 
	392, 379, 393, 393, 379, 380, 393, 380, 394, 394, 380, 381, 394, 381, 395, 395, 381, 382, 395, 382, 396, 396, 382, 383, 396, 383, 397, 397, 383, 384, 
	397, 384, 398, 398, 384, 385, 398, 385, 399, 399, 385, 386, 399, 386, 400, 400, 386, 387, 400, 387, 401, 401, 387, 388, 401, 388, 402, 402, 388, 389, 
	403, 405, 404, 404, 405, 406, 404, 406, 407, 407, 406, 408, 407, 408, 409, 409, 408, 410, 409, 410, 411, 411, 410, 412, 411, 412, 413, 413, 412, 414, 
	413, 414, 415, 415, 414, 416, 415, 416, 417, 417, 416, 418, 417, 418, 419, 419, 418, 420, 419, 420, 421, 421, 420, 422, 421, 422, 423, 423, 422, 424, 
	423, 424, 425, 425, 424, 426, 425, 426, 427, 427, 426, 428, 429, 403, 430, 430, 403, 404, 430, 404, 431, 431, 404, 407, 431, 407, 432, 432, 407, 409, 
	432, 409, 433, 433, 409, 411, 433, 411, 434, 434, 411, 413, 434, 413, 435, 435, 413, 415, 435, 415, 436, 436, 415, 417, 436, 417, 437, 437, 417, 419, 
	437, 419, 438, 438, 419, 421, 438, 421, 439, 439, 421, 423, 439, 423, 440, 440, 423, 425, 440, 425, 441, 441, 425, 427, 442, 429, 443, 443, 429, 430, 
	443, 430, 444, 444, 430, 431, 444, 431, 445, 445, 431, 432, 445, 432, 446, 446, 432, 433, 446, 433, 447, 447, 433, 434, 447, 434, 448, 448, 434, 435, 
	448, 435, 449, 449, 435, 436, 449, 436, 450, 450, 436, 437, 450, 437, 451, 451, 437, 438, 451, 438, 452, 452, 438, 439, 452, 439, 453, 453, 439, 440, 
	453, 440, 454, 454, 440, 441, 455, 442, 456, 456, 442, 443, 456, 443, 457, 457, 443, 444, 457, 444, 458, 458, 444, 445, 458, 445, 459, 459, 445, 446, 
	459, 446, 460, 460, 446, 447, 460, 447, 461, 461, 447, 448, 461, 448, 462, 462, 448, 449, 462, 449, 463, 463, 449, 450, 463, 450, 464, 464, 450, 451, 
	464, 451, 465, 465, 451, 452, 465, 452, 466, 466, 452, 453, 466, 453, 467, 467, 453, 454, 468, 455, 469, 469, 455, 456, 469, 456, 470, 470, 456, 457, 
	470, 457, 471, 471, 457, 458, 471, 458, 472, 472, 458, 459, 472, 459, 473, 473, 459, 460, 473, 460, 474, 474, 460, 461, 474, 461, 475, 475, 461, 462, 
	475, 462, 476, 476, 462, 463, 476, 463, 477, 477, 463, 464, 477, 464, 478, 478, 464, 465, 478, 465, 479, 479, 465, 466, 479, 466, 480, 480, 466, 467, 
	481, 468, 482, 482, 468, 469, 482, 469, 483, 483, 469, 470, 483, 470, 484, 484, 470, 471, 484, 471, 485, 485, 471, 472, 485, 472, 486, 486, 472, 473, 
	486, 473, 487, 487, 473, 474, 487, 474, 488, 488, 474, 475, 488, 475, 489, 489, 475, 476, 489, 476, 490, 490, 476, 477, 490, 477, 491, 491, 477, 478, 
	491, 478, 492, 492, 478, 479, 492, 479, 493, 493, 479, 480, 494, 481, 495, 495, 481, 482, 495, 482, 496, 496, 482, 483, 496, 483, 497, 497, 483, 484, 
	497, 484, 498, 498, 484, 485, 498, 485, 499, 499, 485, 486, 499, 486, 500, 500, 486, 487, 500, 487, 501, 501, 487, 488, 501, 488, 502, 502, 488, 489, 
	502, 489, 503, 503, 489, 490, 503, 490, 504, 504, 490, 491, 504, 491, 505, 505, 491, 492, 505, 492, 506, 506, 492, 493, 507, 494, 508, 508, 494, 495, 
	508, 495, 509, 509, 495, 496, 509, 496, 510, 510, 496, 497, 510, 497, 511, 511, 497, 498, 511, 498, 512, 512, 498, 499, 512, 499, 513, 513, 499, 500, 
	513, 500, 514, 514, 500, 501, 514, 501, 515, 515, 501, 502, 515, 502, 516, 516, 502, 503, 516, 503, 517, 517, 503, 504, 517, 504, 518, 518, 504, 505, 
	518, 505, 519, 519, 505, 506, 520, 507, 521, 521, 507, 508, 521, 508, 522, 522, 508, 509, 522, 509, 523, 523, 509, 510, 523, 510, 524, 524, 510, 511, 
	524, 511, 525, 525, 511, 512, 525, 512, 526, 526, 512, 513, 526, 513, 527, 527, 513, 514, 527, 514, 528, 528, 514, 515, 528, 515, 529, 529, 515, 516, 
	529, 516, 530, 530, 516, 517, 530, 517, 531, 531, 517, 518, 531, 518, 532, 532, 518, 519, 533, 520, 534, 534, 520, 521, 534, 521, 535, 535, 521, 522, 
	535, 522, 536, 536, 522, 523, 536, 523, 537, 537, 523, 524, 537, 524, 538, 538, 524, 525, 538, 525, 539, 539, 525, 526, 539, 526, 540, 540, 526, 527, 
	540, 527, 541, 541, 527, 528, 541, 528, 542, 542, 528, 529, 542, 529, 543, 543, 529, 530, 543, 530, 544, 544, 530, 531, 544, 531, 545, 545, 531, 532, 
	546, 533, 547, 547, 533, 534, 547, 534, 548, 548, 534, 535, 548, 535, 549, 549, 535, 536, 549, 536, 550, 550, 536, 537, 550, 537, 551, 551, 537, 538, 
	551, 538, 552, 552, 538, 539, 552, 539, 553, 553, 539, 540, 553, 540, 554, 554, 540, 541, 554, 541, 555, 555, 541, 542, 555, 542, 556, 556, 542, 543, 
	556, 543, 557, 557, 543, 544, 557, 544, 558, 558, 544, 545, 559, 546, 560, 560, 546, 547, 560, 547, 561, 561, 547, 548, 561, 548, 562, 562, 548, 549, 
	562, 549, 563, 563, 549, 550, 563, 550, 564, 564, 550, 551, 564, 551, 565, 565, 551, 552, 565, 552, 566, 566, 552, 553, 566, 553, 567, 567, 553, 554, 
	567, 554, 568, 568, 554, 555, 568, 555, 569, 569, 555, 556, 569, 556, 570, 570, 556, 557, 570, 557, 571, 571, 557, 558, 572, 559, 573, 573, 559, 560, 
	573, 560, 574, 574, 560, 561, 574, 561, 575, 575, 561, 562, 575, 562, 576, 576, 562, 563, 576, 563, 577, 577, 563, 564, 577, 564, 578, 578, 564, 565, 
	578, 565, 579, 579, 565, 566, 579, 566, 580, 580, 566, 567, 580, 567, 581, 581, 567, 568, 581, 568, 582, 582, 568, 569, 582, 569, 583, 583, 569, 570, 
	583, 570, 584, 584, 570, 571, 585, 572, 586, 586, 572, 573, 586, 573, 587, 587, 573, 574, 587, 574, 588, 588, 574, 575, 588, 575, 589, 589, 575, 576, 
	589, 576, 590, 590, 576, 577, 590, 577, 591, 591, 577, 578, 591, 578, 592, 592, 578, 579, 592, 579, 593, 593, 579, 580, 593, 580, 594, 594, 580, 581, 
	594, 581, 595, 595, 581, 582, 595, 582, 596, 596, 582, 583, 596, 583, 597, 597, 583, 584, 598, 585, 599, 599, 585, 586, 599, 586, 600, 600, 586, 587, 
	600, 587, 601, 601, 587, 588, 601, 588, 602, 602, 588, 589, 602, 589, 603, 603, 589, 590, 603, 590, 604, 604, 590, 591, 604, 591, 605, 605, 591, 592, 
	605, 592, 606, 606, 592, 593, 606, 593, 607, 607, 593, 594, 607, 594, 608, 608, 594, 595, 608, 595, 609, 609, 595, 596, 609, 596, 610, 610, 596, 597, 
	611, 598, 612, 612, 598, 599, 612, 599, 613, 613, 599, 600, 613, 600, 614, 614, 600, 601, 614, 601, 615, 615, 601, 602, 615, 602, 616, 616, 602, 603, 
	616, 603, 617, 617, 603, 604, 617, 604, 618, 618, 604, 605, 618, 605, 619, 619, 605, 606, 619, 606, 620, 620, 606, 607, 620, 607, 621, 621, 607, 608, 
	621, 608, 622, 622, 608, 609, 622, 609, 623, 623, 609, 610, 624, 611, 625, 625, 611, 612, 625, 612, 626, 626, 612, 613, 626, 613, 627, 627, 613, 614, 
	627, 614, 628, 628, 614, 615, 628, 615, 629, 629, 615, 616, 629, 616, 630, 630, 616, 617, 630, 617, 631, 631, 617, 618, 631, 618, 632, 632, 618, 619, 
	632, 619, 633, 633, 619, 620, 633, 620, 634, 634, 620, 621, 634, 621, 635, 635, 621, 622, 635, 622, 636, 636, 622, 623, 637, 624, 638, 638, 624, 625, 
	638, 625, 639, 639, 625, 626, 639, 626, 640, 640, 626, 627, 640, 627, 641, 641, 627, 628, 641, 628, 642, 642, 628, 629, 642, 629, 643, 643, 629, 630, 
	643, 630, 644, 644, 630, 631, 644, 631, 645, 645, 631, 632, 645, 632, 646, 646, 632, 633, 646, 633, 647, 647, 633, 634, 647, 634, 648, 648, 634, 635, 
	648, 635, 649, 649, 635, 636, 650, 637, 651, 651, 637, 638, 651, 638, 652, 652, 638, 639, 652, 639, 653, 653, 639, 640, 653, 640, 654, 654, 640, 641, 
	654, 641, 655, 655, 641, 642, 655, 642, 656, 656, 642, 643, 656, 643, 657, 657, 643, 644, 657, 644, 658, 658, 644, 645, 658, 645, 659, 659, 645, 646, 
	659, 646, 660, 660, 646, 647, 660, 647, 661, 661, 647, 648, 661, 648, 662, 662, 648, 649, 663, 650, 664, 664, 650, 651, 664, 651, 665, 665, 651, 652, 
	665, 652, 666, 666, 652, 653, 666, 653, 667, 667, 653, 654, 667, 654, 668, 668, 654, 655, 668, 655, 669, 669, 655, 656, 669, 656, 670, 670, 656, 657, 
	670, 657, 671, 671, 657, 658, 671, 658, 672, 672, 658, 659, 672, 659, 673, 673, 659, 660, 673, 660, 674, 674, 660, 661, 674, 661, 675, 675, 661, 662, 
	676, 663, 677, 677, 663, 664, 677, 664, 678, 678, 664, 665, 678, 665, 679, 679, 665, 666, 679, 666, 680, 680, 666, 667, 680, 667, 681, 681, 667, 668, 
	681, 668, 682, 682, 668, 669, 682, 669, 683, 683, 669, 670, 683, 670, 684, 684, 670, 671, 684, 671, 685, 685, 671, 672, 685, 672, 686, 686, 672, 673, 
	686, 673, 687, 687, 673, 674, 687, 674, 688, 688, 674, 675, 689, 676, 690, 690, 676, 677, 690, 677, 691, 691, 677, 678, 691, 678, 692, 692, 678, 679, 
	692, 679, 693, 693, 679, 680, 693, 680, 694, 694, 680, 681, 694, 681, 695, 695, 681, 682, 695, 682, 696, 696, 682, 683, 696, 683, 697, 697, 683, 684, 
	697, 684, 698, 698, 684, 685, 698, 685, 699, 699, 685, 686, 699, 686, 700, 700, 686, 687, 700, 687, 701, 701, 687, 688, 702, 689, 703, 703, 689, 690, 
	703, 690, 704, 704, 690, 691, 704, 691, 705, 705, 691, 692, 705, 692, 706, 706, 692, 693, 706, 693, 707, 707, 693, 694, 707, 694, 708, 708, 694, 695, 
	708, 695, 709, 709, 695, 696, 709, 696, 710, 710, 696, 697, 710, 697, 711, 711, 697, 698, 711, 698, 712, 712, 698, 699, 712, 699, 713, 713, 699, 700, 
	713, 700, 714, 714, 700, 701, 715, 702, 716, 716, 702, 703, 716, 703, 717, 717, 703, 704, 717, 704, 718, 718, 704, 705, 718, 705, 719, 719, 705, 706, 
	719, 706, 720, 720, 706, 707, 720, 707, 721, 721, 707, 708, 721, 708, 722, 722, 708, 709, 722, 709, 723, 723, 709, 710, 723, 710, 724, 724, 710, 711, 
	724, 711, 725, 725, 711, 712, 725, 712, 726, 726, 712, 713, 726, 713, 727, 727, 713, 714, 728, 715, 729, 729, 715, 716, 729, 716, 730, 730, 716, 717, 
	730, 717, 731, 731, 717, 718, 731, 718, 732, 732, 718, 719, 732, 719, 733, 733, 719, 720, 733, 720, 734, 734, 720, 721, 734, 721, 735, 735, 721, 722, 
	735, 722, 736, 736, 722, 723, 736, 723, 737, 737, 723, 724, 737, 724, 738, 738, 724, 725, 738, 725, 739, 739, 725, 726, 739, 726, 740, 740, 726, 727, 
	741, 728, 742, 742, 728, 729, 742, 729, 743, 743, 729, 730, 743, 730, 744, 744, 730, 731, 744, 731, 745, 745, 731, 732, 745, 732, 746, 746, 732, 733, 
	746, 733, 747, 747, 733, 734, 747, 734, 748, 748, 734, 735, 748, 735, 749, 749, 735, 736, 749, 736, 750, 750, 736, 737, 750, 737, 751, 751, 737, 738, 
	751, 738, 752, 752, 738, 739, 752, 739, 753, 753, 739, 740, 754, 741, 755, 755, 741, 742, 755, 742, 756, 756, 742, 743, 756, 743, 757, 757, 743, 744, 
	757, 744, 758, 758, 744, 745, 758, 745, 759, 759, 745, 746, 759, 746, 760, 760, 746, 747, 760, 747, 761, 761, 747, 748, 761, 748, 762, 762, 748, 749, 
	762, 749, 763, 763, 749, 750, 763, 750, 764, 764, 750, 751, 764, 751, 765, 765, 751, 752, 765, 752, 766, 766, 752, 753, 767, 754, 768, 768, 754, 755, 
	768, 755, 769, 769, 755, 756, 769, 756, 770, 770, 756, 757, 770, 757, 771, 771, 757, 758, 771, 758, 772, 772, 758, 759, 772, 759, 773, 773, 759, 760, 
	773, 760, 774, 774, 760, 761, 774, 761, 775, 775, 761, 762, 775, 762, 776, 776, 762, 763, 776, 763, 777, 777, 763, 764, 777, 764, 778, 778, 764, 765, 
	778, 765, 779, 779, 765, 766, 780, 767, 781, 781, 767, 768, 781, 768, 782, 782, 768, 769, 782, 769, 783, 783, 769, 770, 783, 770, 784, 784, 770, 771, 
	784, 771, 785, 785, 771, 772, 785, 772, 786, 786, 772, 773, 786, 773, 787, 787, 773, 774, 787, 774, 788, 788, 774, 775, 788, 775, 789, 789, 775, 776, 
	789, 776, 790, 790, 776, 777, 790, 777, 791, 791, 777, 778, 791, 778, 792, 792, 778, 779, 793, 780, 794, 794, 780, 781, 794, 781, 795, 795, 781, 782, 
	795, 782, 796, 796, 782, 783, 796, 783, 797, 797, 783, 784, 797, 784, 798, 798, 784, 785, 798, 785, 799, 799, 785, 786, 799, 786, 800, 800, 786, 787, 
	800, 787, 801, 801, 787, 788, 801, 788, 802, 802, 788, 789, 802, 789, 803, 803, 789, 790, 803, 790, 804, 804, 790, 791, 804, 791, 805, 805, 791, 792, 
	806, 808, 807, 807, 808, 809, 807, 809, 810, 810, 809, 811, 810, 811, 812, 812, 811, 813, 812, 813, 814, 814, 813, 815, 814, 815, 816, 816, 815, 817, 
	816, 817, 818, 818, 817, 819, 818, 819, 820, 820, 819, 821, 820, 821, 822, 822, 821, 823, 822, 823, 824, 824, 823, 825, 824, 825, 826, 826, 825, 827, 
	826, 827, 828, 828, 827, 829, 828, 829, 830, 830, 829, 831, 832, 806, 833, 833, 806, 807, 833, 807, 834, 834, 807, 810, 834, 810, 835, 835, 810, 812, 
	835, 812, 836, 836, 812, 814, 836, 814, 837, 837, 814, 816, 837, 816, 838, 838, 816, 818, 838, 818, 839, 839, 818, 820, 839, 820, 840, 840, 820, 822, 
	840, 822, 841, 841, 822, 824, 841, 824, 842, 842, 824, 826, 842, 826, 843, 843, 826, 828, 843, 828, 844, 844, 828, 830, 845, 832, 846, 846, 832, 833, 
	846, 833, 847, 847, 833, 834, 847, 834, 848, 848, 834, 835, 848, 835, 849, 849, 835, 836, 849, 836, 850, 850, 836, 837, 850, 837, 851, 851, 837, 838, 
	851, 838, 852, 852, 838, 839, 852, 839, 853, 853, 839, 840, 853, 840, 854, 854, 840, 841, 854, 841, 855, 855, 841, 842, 855, 842, 856, 856, 842, 843, 
	856, 843, 857, 857, 843, 844, 858, 845, 859, 859, 845, 846, 859, 846, 860, 860, 846, 847, 860, 847, 861, 861, 847, 848, 861, 848, 862, 862, 848, 849, 
	862, 849, 863, 863, 849, 850, 863, 850, 864, 864, 850, 851, 864, 851, 865, 865, 851, 852, 865, 852, 866, 866, 852, 853, 866, 853, 867, 867, 853, 854, 
	867, 854, 868, 868, 854, 855, 868, 855, 869, 869, 855, 856, 869, 856, 870, 870, 856, 857, 871, 858, 872, 872, 858, 859, 872, 859, 873, 873, 859, 860, 
	873, 860, 874, 874, 860, 861, 874, 861, 875, 875, 861, 862, 875, 862, 876, 876, 862, 863, 876, 863, 877, 877, 863, 864, 877, 864, 878, 878, 864, 865, 
	878, 865, 879, 879, 865, 866, 879, 866, 880, 880, 866, 867, 880, 867, 881, 881, 867, 868, 881, 868, 882, 882, 868, 869, 882, 869, 883, 883, 869, 870, 
	884, 871, 885, 885, 871, 872, 885, 872, 886, 886, 872, 873, 886, 873, 887, 887, 873, 874, 887, 874, 888, 888, 874, 875, 888, 875, 889, 889, 875, 876, 
	889, 876, 890, 890, 876, 877, 890, 877, 891, 891, 877, 878, 891, 878, 892, 892, 878, 879, 892, 879, 893, 893, 879, 880, 893, 880, 894, 894, 880, 881, 
	894, 881, 895, 895, 881, 882, 895, 882, 896, 896, 882, 883, 897, 884, 898, 898, 884, 885, 898, 885, 899, 899, 885, 886, 899, 886, 900, 900, 886, 887, 
	900, 887, 901, 901, 887, 888, 901, 888, 902, 902, 888, 889, 902, 889, 903, 903, 889, 890, 903, 890, 904, 904, 890, 891, 904, 891, 905, 905, 891, 892, 
	905, 892, 906, 906, 892, 893, 906, 893, 907, 907, 893, 894, 907, 894, 908, 908, 894, 895, 908, 895, 909, 909, 895, 896, 910, 897, 911, 911, 897, 898, 
	911, 898, 912, 912, 898, 899, 912, 899, 913, 913, 899, 900, 913, 900, 914, 914, 900, 901, 914, 901, 915, 915, 901, 902, 915, 902, 916, 916, 902, 903, 
	916, 903, 917, 917, 903, 904, 917, 904, 918, 918, 904, 905, 918, 905, 919, 919, 905, 906, 919, 906, 920, 920, 906, 907, 920, 907, 921, 921, 907, 908, 
	921, 908, 922, 922, 908, 909, 923, 910, 924, 924, 910, 911, 924, 911, 925, 925, 911, 912, 925, 912, 926, 926, 912, 913, 926, 913, 927, 927, 913, 914, 
	927, 914, 928, 928, 914, 915, 928, 915, 929, 929, 915, 916, 929, 916, 930, 930, 916, 917, 930, 917, 931, 931, 917, 918, 931, 918, 932, 932, 918, 919, 
	932, 919, 933, 933, 919, 920, 933, 920, 934, 934, 920, 921, 934, 921, 935, 935, 921, 922, 936, 923, 937, 937, 923, 924, 937, 924, 938, 938, 924, 925, 
	938, 925, 939, 939, 925, 926, 939, 926, 940, 940, 926, 927, 940, 927, 941, 941, 927, 928, 941, 928, 942, 942, 928, 929, 942, 929, 943, 943, 929, 930, 
	943, 930, 944, 944, 930, 931, 944, 931, 945, 945, 931, 932, 945, 932, 946, 946, 932, 933, 946, 933, 947, 947, 933, 934, 947, 934, 948, 948, 934, 935, 
	949, 936, 950, 950, 936, 937, 950, 937, 951, 951, 937, 938, 951, 938, 952, 952, 938, 939, 952, 939, 953, 953, 939, 940, 953, 940, 954, 954, 940, 941, 
	954, 941, 955, 955, 941, 942, 955, 942, 956, 956, 942, 943, 956, 943, 957, 957, 943, 944, 957, 944, 958, 958, 944, 945, 958, 945, 959, 959, 945, 946, 
	959, 946, 960, 960, 946, 947, 960, 947, 961, 961, 947, 948, 962, 949, 963, 963, 949, 950, 963, 950, 964, 964, 950, 951, 964, 951, 965, 965, 951, 952, 
	965, 952, 966, 966, 952, 953, 966, 953, 967, 967, 953, 954, 967, 954, 968, 968, 954, 955, 968, 955, 969, 969, 955, 956, 969, 956, 970, 970, 956, 957, 
	970, 957, 971, 971, 957, 958, 971, 958, 972, 972, 958, 959, 972, 959, 973, 973, 959, 960, 973, 960, 974, 974, 960, 961, 975, 962, 976, 976, 962, 963, 
	976, 963, 977, 977, 963, 964, 977, 964, 978, 978, 964, 965, 978, 965, 979, 979, 965, 966, 979, 966, 980, 980, 966, 967, 980, 967, 981, 981, 967, 968, 
	981, 968, 982, 982, 968, 969, 982, 969, 983, 983, 969, 970, 983, 970, 984, 984, 970, 971, 984, 971, 985, 985, 971, 972, 985, 972, 986, 986, 972, 973, 
	986, 973, 987, 987, 973, 974, 988, 975, 989, 989, 975, 976, 989, 976, 990, 990, 976, 977, 990, 977, 991, 991, 977, 978, 991, 978, 992, 992, 978, 979, 
	992, 979, 993, 993, 979, 980, 993, 980, 994, 994, 980, 981, 994, 981, 995, 995, 981, 982, 995, 982, 996, 996, 982, 983, 996, 983, 997, 997, 983, 984, 
	997, 984, 998, 998, 984, 985, 998, 985, 999, 999, 985, 986, 999, 986, 1000, 1000, 986, 987, 1001, 988, 1002, 1002, 988, 989, 1002, 989, 1003, 1003, 989, 990, 
	1003, 990, 1004, 1004, 990, 991, 1004, 991, 1005, 1005, 991, 992, 1005, 992, 1006, 1006, 992, 993, 1006, 993, 1007, 1007, 993, 994, 1007, 994, 1008, 1008, 994, 995, 
	1008, 995, 1009, 1009, 995, 996, 1009, 996, 1010, 1010, 996, 997, 1010, 997, 1011, 1011, 997, 998, 1011, 998, 1012, 1012, 998, 999, 1012, 999, 1013, 1013, 999, 1000, 
	1014, 1001, 1015, 1015, 1001, 1002, 1015, 1002, 1016, 1016, 1002, 1003, 1016, 1003, 1017, 1017, 1003, 1004, 1017, 1004, 1018, 1018, 1004, 1005, 1018, 1005, 1019, 1019, 1005, 1006, 
	1019, 1006, 1020, 1020, 1006, 1007, 1020, 1007, 1021, 1021, 1007, 1008, 1021, 1008, 1022, 1022, 1008, 1009, 1022, 1009, 1023, 1023, 1009, 1010, 1023, 1010, 1024, 1024, 1010, 1011, 
	1024, 1011, 1025, 1025, 1011, 1012, 1025, 1012, 1026, 1026, 1012, 1013, 1027, 1014, 1028, 1028, 1014, 1015, 1028, 1015, 1029, 1029, 1015, 1016, 1029, 1016, 1030, 1030, 1016, 1017, 
	1030, 1017, 1031, 1031, 1017, 1018, 1031, 1018, 1032, 1032, 1018, 1019, 1032, 1019, 1033, 1033, 1019, 1020, 1033, 1020, 1034, 1034, 1020, 1021, 1034, 1021, 1035, 1035, 1021, 1022, 
	1035, 1022, 1036, 1036, 1022, 1023, 1036, 1023, 1037, 1037, 1023, 1024, 1037, 1024, 1038, 1038, 1024, 1025, 1038, 1025, 1039, 1039, 1025, 1026, 1040, 1027, 1041, 1041, 1027, 1028, 
	1041, 1028, 1042, 1042, 1028, 1029, 1042, 1029, 1043, 1043, 1029, 1030, 1043, 1030, 1044, 1044, 1030, 1031, 1044, 1031, 1045, 1045, 1031, 1032, 1045, 1032, 1046, 1046, 1032, 1033, 
	1046, 1033, 1047, 1047, 1033, 1034, 1047, 1034, 1048, 1048, 1034, 1035, 1048, 1035, 1049, 1049, 1035, 1036, 1049, 1036, 1050, 1050, 1036, 1037, 1050, 1037, 1051, 1051, 1037, 1038, 
	1051, 1038, 1052, 1052, 1038, 1039, 1053, 1040, 1054, 1054, 1040, 1041, 1054, 1041, 1055, 1055, 1041, 1042, 1055, 1042, 1056, 1056, 1042, 1043, 1056, 1043, 1057, 1057, 1043, 1044, 
	1057, 1044, 1058, 1058, 1044, 1045, 1058, 1045, 1059, 1059, 1045, 1046, 1059, 1046, 1060, 1060, 1046, 1047, 1060, 1047, 1061, 1061, 1047, 1048, 1061, 1048, 1062, 1062, 1048, 1049, 
	1062, 1049, 1063, 1063, 1049, 1050, 1063, 1050, 1064, 1064, 1050, 1051, 1064, 1051, 1065, 1065, 1051, 1052, 1066, 1053, 1067, 1067, 1053, 1054, 1067, 1054, 1068, 1068, 1054, 1055, 
	1068, 1055, 1069, 1069, 1055, 1056, 1069, 1056, 1070, 1070, 1056, 1057, 1070, 1057, 1071, 1071, 1057, 1058, 1071, 1058, 1072, 1072, 1058, 1059, 1072, 1059, 1073, 1073, 1059, 1060, 
	1073, 1060, 1074, 1074, 1060, 1061, 1074, 1061, 1075, 1075, 1061, 1062, 1075, 1062, 1076, 1076, 1062, 1063, 1076, 1063, 1077, 1077, 1063, 1064, 1077, 1064, 1078, 1078, 1064, 1065, 
	1079, 1066, 1080, 1080, 1066, 1067, 1080, 1067, 1081, 1081, 1067, 1068, 1081, 1068, 1082, 1082, 1068, 1069, 1082, 1069, 1083, 1083, 1069, 1070, 1083, 1070, 1084, 1084, 1070, 1071, 
	1084, 1071, 1085, 1085, 1071, 1072, 1085, 1072, 1086, 1086, 1072, 1073, 1086, 1073, 1087, 1087, 1073, 1074, 1087, 1074, 1088, 1088, 1074, 1075, 1088, 1075, 1089, 1089, 1075, 1076, 
	1089, 1076, 1090, 1090, 1076, 1077, 1090, 1077, 1091, 1091, 1077, 1078, 1092, 1079, 1093, 1093, 1079, 1080, 1093, 1080, 1094, 1094, 1080, 1081, 1094, 1081, 1095, 1095, 1081, 1082, 
	1095, 1082, 1096, 1096, 1082, 1083, 1096, 1083, 1097, 1097, 1083, 1084, 1097, 1084, 1098, 1098, 1084, 1085, 1098, 1085, 1099, 1099, 1085, 1086, 1099, 1086, 1100, 1100, 1086, 1087, 
	1100, 1087, 1101, 1101, 1087, 1088, 1101, 1088, 1102, 1102, 1088, 1089, 1102, 1089, 1103, 1103, 1089, 1090, 1103, 1090, 1104, 1104, 1090, 1091, 1105, 1092, 1106, 1106, 1092, 1093, 
	1106, 1093, 1107, 1107, 1093, 1094, 1107, 1094, 1108, 1108, 1094, 1095, 1108, 1095, 1109, 1109, 1095, 1096, 1109, 1096, 1110, 1110, 1096, 1097, 1110, 1097, 1111, 1111, 1097, 1098, 
	1111, 1098, 1112, 1112, 1098, 1099, 1112, 1099, 1113, 1113, 1099, 1100, 1113, 1100, 1114, 1114, 1100, 1101, 1114, 1101, 1115, 1115, 1101, 1102, 1115, 1102, 1116, 1116, 1102, 1103, 
	1116, 1103, 1117, 1117, 1103, 1104, 1118, 1105, 1119, 1119, 1105, 1106, 1119, 1106, 1120, 1120, 1106, 1107, 1120, 1107, 1121, 1121, 1107, 1108, 1121, 1108, 1122, 1122, 1108, 1109, 
	1122, 1109, 1123, 1123, 1109, 1110, 1123, 1110, 1124, 1124, 1110, 1111, 1124, 1111, 1125, 1125, 1111, 1112, 1125, 1112, 1126, 1126, 1112, 1113, 1126, 1113, 1127, 1127, 1113, 1114, 
	1127, 1114, 1128, 1128, 1114, 1115, 1128, 1115, 1129, 1129, 1115, 1116, 1129, 1116, 1130, 1130, 1116, 1117, 1131, 1118, 1132, 1132, 1118, 1119, 1132, 1119, 1133, 1133, 1119, 1120, 
	1133, 1120, 1134, 1134, 1120, 1121, 1134, 1121, 1135, 1135, 1121, 1122, 1135, 1122, 1136, 1136, 1122, 1123, 1136, 1123, 1137, 1137, 1123, 1124, 1137, 1124, 1138, 1138, 1124, 1125, 
	1138, 1125, 1139, 1139, 1125, 1126, 1139, 1126, 1140, 1140, 1126, 1127, 1140, 1127, 1141, 1141, 1127, 1128, 1141, 1128, 1142, 1142, 1128, 1129, 1142, 1129, 1143, 1143, 1129, 1130, 
	1144, 1131, 1145, 1145, 1131, 1132, 1145, 1132, 1146, 1146, 1132, 1133, 1146, 1133, 1147, 1147, 1133, 1134, 1147, 1134, 1148, 1148, 1134, 1135, 1148, 1135, 1149, 1149, 1135, 1136, 
	1149, 1136, 1150, 1150, 1136, 1137, 1150, 1137, 1151, 1151, 1137, 1138, 1151, 1138, 1152, 1152, 1138, 1139, 1152, 1139, 1153, 1153, 1139, 1140, 1153, 1140, 1154, 1154, 1140, 1141, 
	1154, 1141, 1155, 1155, 1141, 1142, 1155, 1142, 1156, 1156, 1142, 1143, 1157, 1144, 1158, 1158, 1144, 1145, 1158, 1145, 1159, 1159, 1145, 1146, 1159, 1146, 1160, 1160, 1146, 1147, 
	1160, 1147, 1161, 1161, 1147, 1148, 1161, 1148, 1162, 1162, 1148, 1149, 1162, 1149, 1163, 1163, 1149, 1150, 1163, 1150, 1164, 1164, 1150, 1151, 1164, 1151, 1165, 1165, 1151, 1152, 
	1165, 1152, 1166, 1166, 1152, 1153, 1166, 1153, 1167, 1167, 1153, 1154, 1167, 1154, 1168, 1168, 1154, 1155, 1168, 1155, 1169, 1169, 1155, 1156, 1170, 1157, 1171, 1171, 1157, 1158, 
	1171, 1158, 1172, 1172, 1158, 1159, 1172, 1159, 1173, 1173, 1159, 1160, 1173, 1160, 1174, 1174, 1160, 1161, 1174, 1161, 1175, 1175, 1161, 1162, 1175, 1162, 1176, 1176, 1162, 1163, 
	1176, 1163, 1177, 1177, 1163, 1164, 1177, 1164, 1178, 1178, 1164, 1165, 1178, 1165, 1179, 1179, 1165, 1166, 1179, 1166, 1180, 1180, 1166, 1167, 1180, 1167, 1181, 1181, 1167, 1168, 
	1181, 1168, 1182, 1182, 1168, 1169, 1183, 1170, 1184, 1184, 1170, 1171, 1184, 1171, 1185, 1185, 1171, 1172, 1185, 1172, 1186, 1186, 1172, 1173, 1186, 1173, 1187, 1187, 1173, 1174, 
	1187, 1174, 1188, 1188, 1174, 1175, 1188, 1175, 1189, 1189, 1175, 1176, 1189, 1176, 1190, 1190, 1176, 1177, 1190, 1177, 1191, 1191, 1177, 1178, 1191, 1178, 1192, 1192, 1178, 1179, 
	1192, 1179, 1193, 1193, 1179, 1180, 1193, 1180, 1194, 1194, 1180, 1181, 1194, 1181, 1195, 1195, 1181, 1182, 1196, 1183, 1197, 1197, 1183, 1184, 1197, 1184, 1198, 1198, 1184, 1185, 
	1198, 1185, 1199, 1199, 1185, 1186, 1199, 1186, 1200, 1200, 1186, 1187, 1200, 1187, 1201, 1201, 1187, 1188, 1201, 1188, 1202, 1202, 1188, 1189, 1202, 1189, 1203, 1203, 1189, 1190, 
	1203, 1190, 1204, 1204, 1190, 1191, 1204, 1191, 1205, 1205, 1191, 1192, 1205, 1192, 1206, 1206, 1192, 1193, 1206, 1193, 1207, 1207, 1193, 1194, 1207, 1194, 1208, 1208, 1194, 1195, 
	1209, 1211, 1210, 1210, 1211, 1212, 1210, 1212, 1213, 1213, 1212, 1214, 1213, 1214, 1215, 1215, 1214, 1216, 1215, 1216, 1217, 1217, 1216, 1218, 1217, 1218, 1219, 1219, 1218, 1220, 
	1219, 1220, 1221, 1221, 1220, 1222, 1223, 1225, 1224, 1224, 1225, 1226, 1224, 1226, 1209, 1209, 1226, 1211, 1227, 1209, 1228, 1228, 1209, 1210, 1228, 1210, 1229, 1229, 1210, 1213, 
	1229, 1213, 1230, 1230, 1213, 1215, 1230, 1215, 1231, 1231, 1215, 1217, 1231, 1217, 1232, 1232, 1217, 1219, 1232, 1219, 1233, 1233, 1219, 1221, 1234, 1223, 1235, 1235, 1223, 1224, 
	1235, 1224, 1227, 1227, 1224, 1209, 1236, 1227, 1237, 1237, 1227, 1228, 1237, 1228, 1238, 1238, 1228, 1229, 1238, 1229, 1239, 1239, 1229, 1230, 1239, 1230, 1240, 1240, 1230, 1231, 
	1240, 1231, 1241, 1241, 1231, 1232, 1241, 1232, 1242, 1242, 1232, 1233, 1243, 1234, 1244, 1244, 1234, 1235, 1244, 1235, 1236, 1236, 1235, 1227, 1245, 1236, 1246, 1246, 1236, 1237, 
	1246, 1237, 1247, 1247, 1237, 1238, 1247, 1238, 1248, 1248, 1238, 1239, 1248, 1239, 1249, 1249, 1239, 1240, 1249, 1240, 1250, 1250, 1240, 1241, 1250, 1241, 1251, 1251, 1241, 1242, 
	1252, 1243, 1253, 1253, 1243, 1244, 1253, 1244, 1245, 1245, 1244, 1236, 1254, 1245, 1255, 1255, 1245, 1246, 1255, 1246, 1256, 1256, 1246, 1247, 1256, 1247, 1257, 1257, 1247, 1248, 
	1257, 1248, 1258, 1258, 1248, 1249, 1258, 1249, 1259, 1259, 1249, 1250, 1259, 1250, 1260, 1260, 1250, 1251, 1261, 1252, 1262, 1262, 1252, 1253, 1262, 1253, 1254, 1254, 1253, 1245, 
	1263, 1254, 1264, 1264, 1254, 1255, 1264, 1255, 1265, 1265, 1255, 1256, 1265, 1256, 1266, 1266, 1256, 1257, 1266, 1257, 1267, 1267, 1257, 1258, 1267, 1258, 1268, 1268, 1258, 1259, 
	1268, 1259, 1269, 1269, 1259, 1260, 1270, 1261, 1271, 1271, 1261, 1262, 1271, 1262, 1263, 1263, 1262, 1254, 1272, 1274, 1273, 1273, 1274, 1275, 1273, 1275, 1276, 1276, 1275, 1277, 
	1276, 1277, 1278, 1278, 1277, 1279, 1278, 1279, 1280, 1280, 1279, 1281, 1280, 1281, 1282, 1282, 1281, 1283, 1282, 1283, 1284, 1284, 1283, 1285, 1286, 1288, 1287, 1287, 1288, 1289, 
	1287, 1289, 1272, 1272, 1289, 1274, 1290, 1272, 1291, 1291, 1272, 1273, 1291, 1273, 1292, 1292, 1273, 1276, 1292, 1276, 1293, 1293, 1276, 1278, 1293, 1278, 1294, 1294, 1278, 1280, 
	1294, 1280, 1295, 1295, 1280, 1282, 1295, 1282, 1296, 1296, 1282, 1284, 1297, 1286, 1298, 1298, 1286, 1287, 1298, 1287, 1290, 1290, 1287, 1272, 1299, 1290, 1300, 1300, 1290, 1291, 
	1300, 1291, 1301, 1301, 1291, 1292, 1301, 1292, 1302, 1302, 1292, 1293, 1302, 1293, 1303, 1303, 1293, 1294, 1303, 1294, 1304, 1304, 1294, 1295, 1304, 1295, 1305, 1305, 1295, 1296, 
	1306, 1297, 1307, 1307, 1297, 1298, 1307, 1298, 1299, 1299, 1298, 1290, 1308, 1299, 1309, 1309, 1299, 1300, 1309, 1300, 1310, 1310, 1300, 1301, 1310, 1301, 1311, 1311, 1301, 1302, 
	1311, 1302, 1312, 1312, 1302, 1303, 1312, 1303, 1313, 1313, 1303, 1304, 1313, 1304, 1314, 1314, 1304, 1305, 1315, 1306, 1316, 1316, 1306, 1307, 1316, 1307, 1308, 1308, 1307, 1299, 
	1317, 1308, 1318, 1318, 1308, 1309, 1318, 1309, 1319, 1319, 1309, 1310, 1319, 1310, 1320, 1320, 1310, 1311, 1320, 1311, 1321, 1321, 1311, 1312, 1321, 1312, 1322, 1322, 1312, 1313, 
	1322, 1313, 1323, 1323, 1313, 1314, 1324, 1315, 1325, 1325, 1315, 1316, 1325, 1316, 1317, 1317, 1316, 1308, 1326, 1317, 1327, 1327, 1317, 1318, 1327, 1318, 1328, 1328, 1318, 1319, 
	1328, 1319, 1329, 1329, 1319, 1320, 1329, 1320, 1330, 1330, 1320, 1321, 1330, 1321, 1331, 1331, 1321, 1322, 1331, 1322, 1332, 1332, 1322, 1323, 1333, 1324, 1334, 1334, 1324, 1325, 
	1334, 1325, 1326, 1326, 1325, 1317, 
_NULL_ }; /* End of silver_Faces */



/**********************************************************
 * MESHES 
 **********************************************************/

const Struct_Mesh 
   Mesh[NUM_MESHES] = {

/* MESH 0 */
	/* NumVertex      */ glass_NumVertex,
	/* NumFaces       */ glass_NumFaces,
	/* NumStrips      */ glass_NumStrips,
	/* Flags          */ glass_Flags,
	/* Material       */ glass_Material,
	/* Center         */  0.734328f, -25.096840f, 0.963506f,
	/* Vertices       */ glass_Vertices,
	/* UV             */ glass_UV,
	/* Normals        */ glass_Normals,
	/* Packed Vertices*/ (const float *) _NULL_,
	/* VertexColor    */ (const unsigned int *)_NULL_,
	/* VertexMaterial */ (const unsigned int *)_NULL_,
	/* Faces          */ (const unsigned short *)glass_Faces,
	/* Strips         */ (const unsigned short *)_NULL_,
	/* StripLength    */ (const unsigned short *)_NULL_,
	/* Patch type     */ 0,
	/* NumPatches     */ 0,
	/* NumPVertices   */ 0,
	/* NumSubdivisions*/ 0,
	/* ControlPoints  */ _NULL_,
	/* PatchUVs       */ _NULL_,

/* MESH 1 */
	/* NumVertex      */ silver_NumVertex,
	/* NumFaces       */ silver_NumFaces,
	/* NumStrips      */ silver_NumStrips,
	/* Flags          */ silver_Flags,
	/* Material       */ silver_Material,
	/* Center         */  0.324127f, -25.425522f, 23.954473f,
	/* Vertices       */ silver_Vertices,
	/* UV             */ silver_UV,
	/* Normals        */ silver_Normals,
	/* Packed Vertices*/ (const float *) _NULL_,
	/* VertexColor    */ (const unsigned int *)_NULL_,
	/* VertexMaterial */ (const unsigned int *)_NULL_,
	/* Faces          */ (const unsigned short *)silver_Faces,
	/* Strips         */ (const unsigned short *)_NULL_,
	/* StripLength    */ (const unsigned short *)_NULL_,
	/* Patch type     */ 0,
	/* NumPatches     */ 0,
	/* NumPVertices   */ 0,
	/* NumSubdivisions*/ 0,
	/* ControlPoints  */ _NULL_,
	/* PatchUVs       */ _NULL_

	}; /* End of Meshes */
